package ModelicaServices "ModelicaServices (OpenModelica implementation) - Models and functions used in the Modelica Standard Library requiring a tool specific implementation"
  extends Modelica.Icons.Package;
  constant String target = "OpenModelica" "Target of this ModelicaServices implementation";

  package UsersGuide "User's Guide"
    extends Modelica.Icons.Information;

    class ModelicaLicense2 "Modelica License 2"
      extends Modelica.Icons.Information;
      annotation(
        Documentation(info = "<html>
<head>
  <title>The Modelica License 2</title>
<style type=\"text/css\">
*       { font-size: 10pt; font-family: Arial,sans-serif; }
code    { font-size:  9pt; font-family: Courier,monospace;}
h6      { font-size: 10pt; font-weight: bold; color: green; }
h5      { font-size: 11pt; font-weight: bold; color: green; }
h4      { font-size: 13pt; font-weight: bold; color: green; }
address {                  font-weight: normal}
td      { solid #000; vertical-align:top; }
th      { solid #000; vertical-align:top; font-weight: bold; }
table   { solid #000; border-collapse: collapse;}
</style>
</head>

<p>All files in this directory (Modelica) and in all
subdirectories are licensed by the <b><u>Modelica Association</u></b> under the
<b><u>Modelica License 2</u></b>.</p>

<p style=\"margin-left: 40px;\"><b>Licensor:</b><br>
Modelica Association<br>
(Ideella F&ouml;reningar 822003-8858 in Link&ouml;ping) <br>
c/o PELAB, IDA, Link&ouml;pings Universitet <br>
S-58183 Link&ouml;ping <br>
Sweden<br>
email: Board@Modelica.org<br>
web: <a href=\"https://www.Modelica.org\">https://www.Modelica.org</a></p>

<p style=\"margin-left: 40px;\"><b>Copyright notices of the files:</b><br/>
Copyright &copy; 1998-2015, DLR, Dassault Syst&egrave;mes AB.
</p>

<p>
<a href=\"#The_Modelica_License_2-outline\">The Modelica License 2</a><br>
<a href=\"#Frequently_Asked_Questions-outline\">Frequently Asked Questions</a><br>
</p>

<hr>

<h4><a name=\"The_Modelica_License_2-outline\"></a>The Modelica License 2</h4>

<p>
<b>Preamble.</b> The goal of this license is that Modelica related
model libraries, software, images, documents, data files etc. can be
used freely in the original or a modified form, in open source and in
commercial environments (as long as the license conditions below are
fulfilled, in particular sections 2c) and 2d). The Original Work is
provided free of charge and the use is completely at your own risk.
Developers of free Modelica packages are encouraged to utilize this
license for their work.</p>

<p>
The Modelica License applies to any Original Work that contains the
following licensing notice adjacent to the copyright notice(s) for
this Original Work:</p>

<p><b>Licensed by the Modelica Association under the Modelica License 2</b></p>

<p><b>1. Definitions.</b></p>
<ol type=\"a\">
  <li>&ldquo;License&rdquo; is this Modelica License.</li>

  <li>&ldquo;Original Work&rdquo; is any work of authorship, including
  software, images, documents, data files, that contains the above
  licensing notice or that is packed together with a licensing notice
  referencing it.</li>

  <li>&ldquo;Licensor&rdquo; is the provider of the Original Work who has
  placed this licensing notice adjacent to the copyright notice(s) for
  the Original Work. The Original Work is either directly provided by
  the owner of the Original Work, or by a licensee of the owner.</li>

  <li>&ldquo;Derivative Work&rdquo; is any modification of the Original
  Work which represents, as a whole, an original work of authorship.
  For the matter of clarity and as examples:

  <ol  type=\"A\">
    <li>Derivative Work shall not include work that remains separable from
    the Original Work, as well as merely extracting a part of the
    Original Work without modifying it.</li>

    <li>Derivative Work shall not include (a) fixing of errors and/or (b)
    adding vendor specific Modelica annotations and/or (c) using a
    subset of the classes of a Modelica package, and/or (d) using a
    different representation, e.g., a binary representation.</li>

    <li>Derivative Work shall include classes that are copied from the
    Original Work where declarations, equations or the documentation
    are modified.</li>

    <li>Derivative Work shall include executables to simulate the models
    that are generated by a Modelica translator based on the Original
    Work (of a Modelica package).</li>
  </ol>

  <li>&ldquo;Modified Work&rdquo; is any modification of the Original Work
  with the following exceptions: (a) fixing of errors and/or (b)
  adding vendor specific Modelica annotations and/or (c) using a
  subset of the classes of a Modelica package, and/or (d) using a
  different representation, e.g., a binary representation.</li>

  <li>&quot;Source Code&quot; means the preferred form of the Original
  Work for making modifications to it and all available documentation
  describing how to modify the Original Work.</li>

  <li>&ldquo;You&rdquo; means an individual or a legal entity exercising
  rights under, and complying with all of the terms of, this License.</li>

  <li>&ldquo;Modelica package&rdquo; means any Modelica library that is
  defined with the &ldquo;<code><b>package</b>&nbsp;&lt;Name&gt;&nbsp;...&nbsp;<b>end</b>&nbsp;&lt;Name&gt;;</code>&rdquo; Modelica language element.</li>

</ol>

<p>
<b>2. Grant of Copyright License.</b> Licensor grants You a
worldwide, royalty-free, non-exclusive, sublicensable license, for
the duration of the copyright, to do the following:</p>

<ol type=\"a\">
  <li><p>
  To reproduce the Original Work in copies, either alone or as part of
  a collection.</p></li>
  <li><p>
  To create Derivative Works according to Section 1d) of this License.</p></li>
  <li><p>
  To distribute or communicate to the public copies of the <u>Original
  Work</u> or a <u>Derivative Work</u> under <u>this License</u>. No
  fee, neither as a copyright-license fee, nor as a selling fee for
  the copy as such may be charged under this License. Furthermore, a
  verbatim copy of this License must be included in any copy of the
  Original Work or a Derivative Work under this License.<br>
  For the matter of clarity, it is permitted A) to distribute or
  communicate such copies as part of a (possible commercial)
  collection where other parts are provided under different licenses
  and a license fee is charged for the other parts only and B) to
  charge for mere printing and shipping costs.</p></li>
  <li><p>
  To distribute or communicate to the public copies of a <u>Derivative
  Work</u>, alternatively to Section 2c), under <u>any other license</u>
  of your choice, especially also under a license for
  commercial/proprietary software, as long as You comply with Sections
  3, 4 and 8 below. <br>      For the matter of clarity, no
  restrictions regarding fees, either as to a copyright-license fee or
  as to a selling fee for the copy as such apply.</p></li>
  <li><p>
  To perform the Original Work publicly.</p></li>
  <li><p>
  To display the Original Work publicly.</p></li>
</ol>

<p>
<b>3. Acceptance.</b> Any use of the Original Work or a
Derivative Work, or any action according to either Section 2a) to 2f)
above constitutes Your acceptance of this License.</p>

<p>
<b>4. Designation of Derivative Works and of Modified Works.
</b>The identifying designation of Derivative Work and of Modified
Work must be different to the corresponding identifying designation
of the Original Work. This means especially that the (root-level)
name of a Modelica package under this license must be changed if the
package is modified (besides fixing of errors, adding vendor specific
Modelica annotations, using a subset of the classes of a Modelica
package, or using another representation, e.g. a binary
representation).</p>

<p>
<b>5. Grant of Patent License.</b>
Licensor grants You a worldwide, royalty-free, non-exclusive, sublicensable license,
under patent claims owned by the Licensor or licensed to the Licensor by
the owners of the Original Work that are embodied in the Original Work
as furnished by the Licensor, for the duration of the patents,
to make, use, sell, offer for sale, have made, and import the Original Work
and Derivative Works under the conditions as given in Section 2.
For the matter of clarity, the license regarding Derivative Works covers
patent claims to the extent as they are embodied in the Original Work only.</p>

<p>
<b>6. Provision of Source Code.</b> Licensor agrees to provide
You with a copy of the Source Code of the Original Work but reserves
the right to decide freely on the manner of how the Original Work is
provided.<br>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;For the matter of clarity, Licensor might provide only a binary
representation of the Original Work. In that case, You may (a) either
reproduce the Source Code from the binary representation if this is
possible (e.g., by performing a copy of an encrypted Modelica
package, if encryption allows the copy operation) or (b) request the
Source Code from the Licensor who will provide it to You.</p>

<p>
<b>7. Exclusions from License Grant.</b> Neither the names of
Licensor, nor the names of any contributors to the Original Work, nor
any of their trademarks or service marks, may be used to endorse or
promote products derived from this Original Work without express
prior permission of the Licensor. Except as otherwise expressly
stated in this License and in particular in Sections 2 and 5, nothing
in this License grants any license to Licensor&rsquo;s trademarks,
copyrights, patents, trade secrets or any other intellectual
property, and no patent license is granted to make, use, sell, offer
for sale, have made, or import embodiments of any patent claims.<br>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;No license is granted to the trademarks of
Licensor even if such trademarks are included in the Original Work,
except as expressly stated in this License. Nothing in this License
shall be interpreted to prohibit Licensor from licensing under terms
different from this License any Original Work that Licensor otherwise
would have a right to license.</p>

<p>
<b>8. Attribution Rights.</b> You must retain in the Source
Code of the Original Work and of any Derivative Works that You
create, all author, copyright, patent, or trademark notices, as well
as any descriptive text identified therein as an &quot;Attribution
Notice&quot;. The same applies to the licensing notice of this
License in the Original Work. For the matter of clarity, &ldquo;author
notice&rdquo; means the notice that identifies the original
author(s). <br>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;You must cause the Source Code for any Derivative
Works that You create to carry a prominent Attribution Notice
reasonably calculated to inform recipients that You have modified the
Original Work. <br>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;In case the Original Work or Derivative Work is not provided in
Source Code, the Attribution Notices shall be appropriately
displayed, e.g., in the documentation of the Derivative Work.</p>

<p><b>9. Disclaimer
of Warranty. <br></b><u><b>The Original Work is provided under this
License on an &quot;as is&quot; basis and without warranty, either
express or implied, including, without limitation, the warranties of
non-infringement, merchantability or fitness for a particular
purpose. The entire risk as to the quality of the Original Work is
with You.</b></u> This disclaimer of warranty constitutes an
essential part of this License. No license to the Original Work is
granted by this License except under this disclaimer.</p>

<p>
<b>10. Limitation of Liability.</b> Under no circumstances and
under no legal theory, whether in tort (including negligence),
contract, or otherwise, shall the Licensor, the owner or a licensee
of the Original Work be liable to anyone for any direct, indirect,
general, special, incidental, or consequential damages of any
character arising as a result of this License or the use of the
Original Work including, without limitation, damages for loss of
goodwill, work stoppage, computer failure or malfunction, or any and
all other commercial damages or losses. This limitation of liability
shall not apply to the extent applicable law prohibits such
limitation.</p>

<p>
<b>11. Termination.</b> This License conditions your rights to
undertake the activities listed in Section 2 and 5, including your
right to create Derivative Works based upon the Original Work, and
doing so without observing these terms and conditions is prohibited
by copyright law and international treaty. Nothing in this License is
intended to affect copyright exceptions and limitations. This License
shall terminate immediately and You may no longer exercise any of the
rights granted to You by this License upon your failure to observe
the conditions of this license.</p>

<p>
<b>12. Termination for Patent Action.</b> This License shall
terminate automatically and You may no longer exercise any of the
rights granted to You by this License as of the date You commence an
action, including a cross-claim or counterclaim, against Licensor,
any owners of the Original Work or any licensee alleging that the
Original Work infringes a patent. This termination provision shall
not apply for an action alleging patent infringement through
combinations of the Original Work under combination with other
software or hardware.</p>

<p>
<b>13. Jurisdiction.</b> Any action or suit relating to this
License may be brought only in the courts of a jurisdiction wherein
the Licensor resides and under the laws of that jurisdiction
excluding its conflict-of-law provisions. The application of the
United Nations Convention on Contracts for the International Sale of
Goods is expressly excluded. Any use of the Original Work outside the
scope of this License or after its termination shall be subject to
the requirements and penalties of copyright or patent law in the
appropriate jurisdiction. This section shall survive the termination
of this License.</p>

<p>
<b>14. Attorneys&rsquo; Fees.</b> In any action to enforce the
terms of this License or seeking damages relating thereto, the
prevailing party shall be entitled to recover its costs and expenses,
including, without limitation, reasonable attorneys' fees and costs
incurred in connection with such action, including any appeal of such
action. This section shall survive the termination of this License.</p>

<p>
<b>15. Miscellaneous.</b>
</p>
<ol type=\"a\">
  <li>If any
  provision of this License is held to be unenforceable, such
  provision shall be reformed only to the extent necessary to make it
  enforceable.</li>

  <li>No verbal
  ancillary agreements have been made. Changes and additions to this
  License must appear in writing to be valid. This also applies to
  changing the clause pertaining to written form.</li>

  <li>You may use the
  Original Work in all ways not otherwise restricted or conditioned by
  this License or by law, and Licensor promises not to interfere with
  or be responsible for such uses by You.</li>
</ol>

<hr>

<h5><a name=\"Frequently_Asked_Questions-outline\"></a>
Frequently Asked Questions</h5>
<p>This
section contains questions/answer to users and/or distributors of
Modelica packages and/or documents under Modelica License 2. Note,
the answers to the questions below are not a legal interpretation of
the Modelica License 2. In case of a conflict, the language of the
license shall prevail.</p>

<h6>Using or Distributing a Modelica <u>Package</u> under the Modelica License 2</h6>

<p><b>What are the main
differences to the previous version of the Modelica License?</b></p>
<ol>
  <li><p>
  Modelica License 1 is unclear whether the licensed Modelica package
  can be distributed under a different license. Version 2 explicitly
  allows that &ldquo;Derivative Work&rdquo; can be distributed under
  any license of Your choice, see examples in Section 1d) as to what
  qualifies as Derivative Work (so, version 2 is clearer).</p>
  <li><p>
  If You modify a Modelica package under Modelica License 2 (besides
  fixing of errors, adding vendor specific Modelica annotations, using
  a subset of the classes of a Modelica package, or using another
  representation, e.g., a binary representation), you must rename the
  root-level name of the package for your distribution. In version 1
  you could keep the name (so, version 2 is more restrictive). The
  reason of this restriction is to reduce the risk that Modelica
  packages are available that have identical names, but different
  functionality.</p>
  <li><p>
  Modelica License 1 states that &ldquo;It is not allowed to charge a
  fee for the original version or a modified version of the software,
  besides a reasonable fee for distribution and support&rdquo;.
  Version 2 has a similar intention for all Original Work under
  <u>Modelica License 2</u> (to remain free of charge and open source)
  but states this more clearly as &ldquo;No fee, neither as a
  copyright-license fee, nor as a selling fee for the copy as such may
  be charged&rdquo;. Contrary to version 1, Modelica License 2 has no
  restrictions on fees for Derivative Work that is provided under a
  different license (so, version 2 is clearer and has fewer
  restrictions).</p>
  <li><p>
  Modelica License 2 introduces several useful provisions for the
  licensee (articles 5, 6, 12), and for the licensor (articles 7, 12,
  13, 14) that have no counter part in version 1.</p>
  <li><p>
  Modelica License 2 can be applied to all type of work, including
  documents, images and data files, contrary to version 1 that was
  dedicated for software only (so, version 2 is more general).</p>
</ol>

<p><b>Can I distribute a
Modelica package (under Modelica License 2) as part of my commercial
Modelica modeling and simulation environment?</b></p>
<p>Yes,
according to Section 2c). However, you are not allowed to charge a
fee for this part of your environment. Of course, you can charge for
your part of the environment.
</p>

<p><b>Can I distribute a
Modelica package (under Modelica License 2) under a different
license?</b></p>
<p>No.
The license of an unmodified Modelica package cannot be changed
according to Sections 2c) and 2d). This means that you cannot <u>sell</u>
copies of it, any distribution has to be free of charge.</p>

<p><b>Can I distribute a
Modelica package (under Modelica License 2) under a different license
when I first encrypt the package?</b></p>
<p>No.
Merely encrypting a package does not qualify for Derivative Work and
therefore the encrypted package has to stay under Modelica License 2.</p>

<p><b>Can I distribute a
Modelica package (under Modelica License 2) under a different license
when I first add classes to the package?</b></p>
<p>No.
The package itself remains unmodified, i.e., it is Original Work, and
therefore the license for this part must remain under Modelica
License 2. The newly added classes can be, however, under a different
license.
</p>

<p><b>Can
I copy a class out of a Modelica package (under Modelica License 2)
and include it </b><u><b>unmodified</b></u><b> in a Modelica package
under a </b><u><b>commercial/proprietary license</b></u><b>?</b></p>
<p>No,
according to article 2c). However, you can include model, block,
function, package, record and connector classes in your Modelica
package under <u>Modelica License 2</u>. This means that your
Modelica package could be under a commercial/proprietary license, but
one or more classes of it are under Modelica License 2.<br>Note, a
&ldquo;type&rdquo; class (e.g., type Angle = Real(unit=&rdquo;rad&rdquo;))
can be copied and included unmodified under a commercial/proprietary
license (for details, see the next question).</p>

<p><b>Can
I copy a type class or </b><u><b>part</b></u><b> of a model, block,
function, record, connector class, out of a Modelica package (under
Modelica License 2) and include it modified or unmodified in a
Modelica package under a </b><u><b>commercial/proprietary</b></u><b>
license</b></p>
<p>Yes,
according to article 2d), since this will in the end usually qualify
as Derivative Work. The reasoning is the following: A type class or
part of another class (e.g., an equation, a declaration, part of a
class description) cannot be utilized &ldquo;by its own&rdquo;. In
order to make this &ldquo;usable&rdquo;, you have to add additional
code in order that the class can be utilized. This is therefore
usually Derivative Work and Derivative Work can be provided under a
different license. Note, this only holds, if the additional code
introduced is sufficient to qualify for Derivative Work. Merely, just
copying a class and changing, say, one character in the documentation
of this class would be no Derivative Work and therefore the copied
code would have to stay under Modelica License 2.</p>

<p><b>Can
I copy a class out of a Modelica package (under Modelica License 2)
and include it in </b><u><b>modified </b></u><b>form in a
</b><u><b>commercial/proprietary</b></u><b> Modelica package?</b></p>
<p>Yes.
If the modification can be seen as a &ldquo;Derivative Work&rdquo;,
you can place it under your commercial/proprietary license. If the
modification does not qualify as &ldquo;Derivative Work&rdquo; (e.g.,
bug fixes, vendor specific annotations), it must remain under
Modelica License 2. This means that your Modelica package could be
under a commercial/proprietary license, but one or more parts of it
are under Modelica License 2.</p>

<p><b>Can I distribute a
&ldquo;save total model&rdquo; under my commercial/proprietary
license, even if classes under Modelica License 2 are included?</b></p>
<p>Your
classes of the &ldquo;save total model&rdquo; can be distributed
under your commercial/proprietary license, but the classes under
Modelica License 2 must remain under Modelica License 2. This means
you can distribute a &ldquo;save total model&rdquo;, but some parts
might be under Modelica License 2.</p>

<p><b>Can I distribute a
Modelica package (under Modelica License 2) in encrypted form?</b></p>
<p>Yes.
Note, if the encryption does not allow &ldquo;copying&rdquo; of
classes (in to unencrypted Modelica source code), you have to send
the Modelica source code of this package to your customer, if he/she
wishes it, according to article&nbsp;6.</p>

<p><b>Can I distribute an
executable under my commercial/proprietary license, if the model from
which the executable is generated uses models from a Modelica package
under Modelica License 2?</b></p>
<p>Yes,
according to article 2d), since this is seen as Derivative Work. The
reasoning is the following: An executable allows the simulation of a
concrete model, whereas models from a Modelica package (without
pre-processing, translation, tool run-time library) are not able to
be simulated without tool support. By the processing of the tool and
by its run-time libraries, significant new functionality is added (a
model can be simulated whereas previously it could not be simulated)
and functionality available in the package is removed (e.g., to build
up a new model by dragging components of the package is no longer
possible with the executable).</p>

<p><b>Is my modification to
a Modelica package (under Modelica License 2) a Derivative Work?</b></p>
<p>It
is not possible to give a general answer to it. To be regarded as &quot;an
original work of authorship&quot;, a derivative work must be
different enough from the original or must contain a substantial
amount of new material. Making minor changes or additions of little
substance to a preexisting work will not qualify the work as a new
version for such purposes.
</p>

<h6>Using or Distributing a Modelica <u>Document</u> under the Modelica License 2</h6>

<p>This
section is devoted especially for the following applications:</p>
<ol type=\"a\">
  <li><p>
  A Modelica tool extracts information out of a Modelica package and
  presents the result in form of a &ldquo;manual&rdquo; for this
  package in, e.g., html, doc, or pdf format.</p>
  <li><p>
  The Modelica language specification is a document defining the
  Modelica language. It will be licensed under Modelica License 2.</p>
  <li><p>
  Someone writes a book about the Modelica language and/or Modelica
  packages and uses information which is available in the Modelica
  language specification and/or the corresponding Modelica package.</p>
</ol>

<p><b>Can I sell a manual
that was basically derived by extracting information automatically
from a Modelica package under Modelica License 2 (e.g., a &ldquo;reference
guide&rdquo; of the Modelica Standard Library):</b></p>
<p>Yes.
Extracting information from a Modelica package, and providing it in a
human readable, suitable format, like html, doc or pdf format, where
the content is significantly modified (e.g. tables with interface
information are constructed from the declarations of the public
variables) qualifies as Derivative Work and there are no restrictions
to charge a fee for Derivative Work under alternative 2d).</p>

<p><b>Can
I copy a text passage out of a Modelica document (under Modelica
License 2) and use it </b><u><b>unmodified</b></u><b> in my document
(e.g. the Modelica syntax description in the Modelica Specification)?</b></p>
<p>Yes.
In case you distribute your document, the copied parts are still
under Modelica License 2 and you are not allowed to charge a license
fee for this part. You can, of course, charge a fee for the rest of
your document.</p>

<p><b>Can
I copy a text passage out of a Modelica document (under Modelica
License 2) and use it in </b><u><b>modified</b></u><b> form in my
document?</b></p>
<p>Yes,
the creation of Derivative Works is allowed. In case the content is
significantly modified this qualifies as Derivative Work and there
are no restrictions to charge a fee for Derivative Work under
alternative 2d).</p>

<p><b>Can I sell a printed
version of a Modelica document (under Modelica License 2), e.g., the
Modelica Language Specification?</b></p>
<p>No,
if you are not the copyright-holder, since article 2c) does not allow
a selling fee for a (in this case physical) copy. However, mere
printing and shipping costs may be recovered.</p>
</html>"));
    end ModelicaLicense2;

    class ReleaseNotes "Release notes"
      extends Modelica.Icons.ReleaseNotes;
      annotation(
        Documentation(info = "<html>
<h4>Version 3.2.1, 2012-12-05</h4>

<ul>
<li> Version numbering adapted to the corresponding version number of
     package Modelica (= Modelica Standard Library).</li>
<li> New function
     <a href=\"modelica://ModelicaServices.ExternalReferences.loadResource\">loadResource</a>
     to determine the absolute, local file name from an URI path name.
     </li>
<li> New String type with tool dependent choices
     <a href=\"modelica://ModelicaServices.Types.SolverMethod\">SolverMethod</a>
     to define the integration method to solve differential equations in a
     clocked discretized continuous-time partition.
     </li>
<li> New package
     <a href=\"modelica://ModelicaServices.Machine\">Machine</a>
     to define the processor dependent constants as used in
     <a href=\"modelica://Modelica.Constants\">Modelica.Constants</a>.
     </li>
</ul>


<h4>Version 1.1, 2010-07-30</h4>

<ul>
<li> New model <a href=\"modelica://ModelicaServices.Animation.Surface\">Surface</a>
     to describe movable, parameterized surfaces.</li>
<li> New string constant ModelicaServices.target to define the
     target of the corresponding ModelicaServices implementation</li>
<li> Package icons adapted to the icons of package Modelica, version 3.2.</li>
<li> ModelicaServices library on the Modelica subversion server provided in three versions:
     <ol>
     <li> <b>Default/ModelicaServices</b><br>
          (for tools that do not support 3-dim. visualization).</li>

     <li> <b>Dymola/ModelicaServices</b><br>
          (a Dymola-specific implementation).</li>

     <li> <b>DymolaAndDLRVisualization/ModelicaServices</b><br>
          (an implementation that uses the DLR Visualization library
           in combination with Dymola).</li>
     </ol>
     </li>
</ul>

<h4>Version 1.0, 2009-06-21</h4>

<p>
First version of the ModelicaServices library.
</p>
</html>"));
    end ReleaseNotes;

    class Contact "Contact"
      extends Modelica.Icons.Contact;
      annotation(
        Documentation(info = "<html>
<h5>Main Author:</h5>

<table border=0 cellspacing=0 cellpadding=2>
<tr>
<td>
<a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a><br>
    Deutsches Zentrum f&uuml;r Luft und Raumfahrt e.V. (DLR)<br>
    Robotik und Mechatronik Zentrum (RMC)<br>
    Institut f&uuml;r Systemdynamik und Regelungstechnik (SR)<br>
    Postfach 1116<br>
    D-82230 Wessling<br>
    Germany<br>
    email: <A HREF=\"mailto:Martin.Otter@dlr.de\">Martin.Otter@dlr.de</A></td>
</tr>
</table>

<p><b>Acknowledgements:</b></p>

<p>
The design of the Animation.Shape component is from Hilding Elmqvist, Dassault Syst&egrave;mes AB.
</p>
</html>"));
    end Contact;
    annotation(
      DocumentationClass = true);
  end UsersGuide;

  package Animation "Models and functions for 3-dim. animation"
    extends Modelica.Icons.Package;

    model Shape "Different visual shapes with variable size; all data have to be set as modifiers (see info layer)"
      extends Modelica.Utilities.Internal.PartialModelicaServices.Animation.PartialShape;
      annotation(
        Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-150, -110}, {150, -140}}, lineColor = {0, 0, 0}, textString = "default")}),
        Documentation(info = "<html>
<p>
The interface of this model is documented at
<a href=\"modelica://Modelica.Mechanics.MultiBody.Visualizers.Advanced.Shape\">Modelica.Mechanics.MultiBody.Visualizers.Advanced.Shape</a>.
</p>

</html>"));
    end Shape;

    model Surface "Animation of a moveable, parameterized surface; the surface characteristic is provided by a function"
      extends Modelica.Utilities.Internal.PartialModelicaServices.Animation.PartialSurface;
      annotation(
        Documentation(info = "<html>
<p>
The interface of this model is documented at
<a href=\"modelica://Modelica.Mechanics.MultiBody.Visualizers.Advanced.Surface\">Modelica.Mechanics.MultiBody.Visualizers.Advanced.Surface</a>.<br>
The interface of this model is defined at
<a href=\"modelica://Modelica.Utilities.Internal.PartialModelicaServices.Animation.PartialSurface\">Modelica.Utilities.Internal.PartialModelicaServices.Animation.PartialSurface</a>.
</p>

</html>"));
    end Surface;
  end Animation;

  package ExternalReferences "Library of functions to access external resources"
    extends Modelica.Icons.Package;

    function loadResource "Return the absolute path name of a URI or local file name (in this default implementation URIs are not supported, but only local file names)"
      extends Modelica.Utilities.Internal.PartialModelicaServices.ExternalReferences.PartialLoadResource;
    algorithm
      fileReference := OpenModelica.Scripting.uriToFilename(uri);
      annotation(
        Documentation(info = "<html>
<p>
The interface of this model is documented at
<a href=\"modelica://Modelica.Utilities.Files.loadResource\">Modelica.Utilities.Files.loadResource</a>.
</p>
</html>"));
    end loadResource;
  end ExternalReferences;

  package Machine
    // Machine dependent constants
    extends Modelica.Icons.Package;
    final constant Real eps = 1.e-15 "Biggest number such that 1.0 + eps = 1.0";
    final constant Real small = 1.e-60 "Smallest number such that small and -small are representable on the machine";
    final constant Real inf = 1.e+60 "Biggest Real number such that inf and -inf are representable on the machine";
    final constant Integer Integer_inf = OpenModelica.Internal.Architecture.integerMax() "Biggest Integer number such that Integer_inf and -Integer_inf are representable on the machine";
    annotation(
      Documentation(info = "<html>
<p>
Package in which processor specific constants are defined that are needed
by numerical algorithms. Typically these constants are not directly used,
but indirectly via the alias definition in
<a href=\"modelica://Modelica.Constants\">Modelica.Constants</a>.
</p>
</html>"));
  end Machine;

  package Types "Library of types with vendor specific choices"
    extends Modelica.Icons.Package;
    type SolverMethod = String "String defining the integration method to solve differential equations in a clocked discretized continuous-time partition" annotation(
      choices(choice = "External" "Solver specified externally", choice = "ExplicitEuler" "Explicit Euler method (order 1)", choice = "ExplicitMidPoint2" "Explicit mid point rule (order 2)", choice = "ExplicitRungeKutta4" "Explicit Runge-Kutta method (order 4)", choice = "ImplicitEuler" "Implicit Euler method (order 1)", choice = "ImplicitTrapezoid" "Implicit trapezoid rule (order 2)"),
      Documentation(info = "<html>
<p>
Type <b>SolverMethod</b> is a String type with menu choices to select the
integration method to solve differential equations in a clocked discretized
continuous-time partition. The choices are tool dependent.
For details, see chapter 16.8.2 \"Solver Method\" in the Modelica Language
Specification (version &ge; 3.3).
</p>
</html>"));
  end Types;
  annotation(
    Protection(access = Access.hide),
    preferredView = "info",
    version = "3.2.1",
    versionBuild = 2,
    versionDate = "2013-08-14",
    dateModified = "2013-08-14 08:44:41Z",
    revisionId = "$Id::                                       $",
    uses(Modelica(version = "3.2.1")),
    conversion(noneFromVersion = "1.0", noneFromVersion = "1.1", noneFromVersion = "1.2"),
    Documentation(info = "<html>
<p>
This package contains a set of functions and models to be used in the
Modelica Standard Library that requires a tool specific implementation.
These are:
</p>

<ul>
<li> <a href=\"modelica://ModelicaServices.Animation.Shape\">Shape</a>
     provides a 3-dim. visualization of elementary
     mechanical objects. It is used in
<a href=\"modelica://Modelica.Mechanics.MultiBody.Visualizers.Advanced.Shape\">Modelica.Mechanics.MultiBody.Visualizers.Advanced.Shape</a>
     via inheritance.</li>

<li> <a href=\"modelica://ModelicaServices.Animation.Surface\">Surface</a>
     provides a 3-dim. visualization of
     moveable parameterized surface. It is used in
<a href=\"modelica://Modelica.Mechanics.MultiBody.Visualizers.Advanced.Surface\">Modelica.Mechanics.MultiBody.Visualizers.Advanced.Surface</a>
     via inheritance.</li>

<li> <a href=\"modelica://ModelicaServices.ExternalReferences.loadResource\">loadResource</a>
     provides a function to return the absolute path name of an URI or a local file name. It is used in
<a href=\"modelica://Modelica.Utilities.Files.loadResource\">Modelica.Utilities.Files.loadResource</a>
     via inheritance.</li>

<li> <a href=\"modelica://ModelicaServices.Machine\">ModelicaServices.Machine</a>
     provides a package of machine constants. It is used in
<a href=\"modelica://Modelica.Constants\">Modelica.Constants</a>.</li>

<li> <a href=\"modelica://ModelicaServices.Types.SolverMethod\">Types.SolverMethod</a>
     provides a string defining the integration method to solve differential equations in
     a clocked discretized continuous-time partition (see Modelica 3.3 language specification).
     It is not yet used in the Modelica Standard Library, but in the Modelica_Synchronous library
     that provides convenience blocks for the clock operators of Modelica version &ge; 3.3.</li>
</ul>

<p>
This is the default implementation, if no tool-specific implementation is available.
This ModelicaServices package provides only \"dummy\" models that do nothing.
</p>

<p>
<b>Licensed by DLR and Dassault Syst&egrave;mes AB under the Modelica License 2</b><br>
Copyright &copy; 2009-2015, DLR and Dassault Syst&egrave;mes AB.
</p>

<p>
<i>This Modelica package is <u>free</u> software and the use is completely at <u>your own risk</u>; it can be redistributed and/or modified under the terms of the Modelica License 2. For license conditions (including the disclaimer of warranty) see <a href=\"modelica://Modelica.UsersGuide.ModelicaLicense2\">Modelica.UsersGuide.ModelicaLicense2</a> or visit <a href=\"https://www.modelica.org/licenses/ModelicaLicense2\"> https://www.modelica.org/licenses/ModelicaLicense2</a>.</i>
</p>

</html>"));
end ModelicaServices;

operator record Complex "Complex number with overloaded operators"
  //record Complex "Complex number with overloaded operators"
  replaceable Real re "Real part of complex number" annotation(
    Dialog);
  replaceable Real im "Imaginary part of complex number" annotation(
    Dialog);

  encapsulated operator 'constructor' "Constructor"
    function fromReal "Construct Complex from Real"
      import Complex;
      input Real re "Real part of complex number";
      input Real im = 0 "Imaginary part of complex number";
      output Complex result(re = re, im = im) "Complex number";
    algorithm

      annotation(
        Inline = true,
        Documentation(info = "<html>
<p>This function returns a Complex number defined by real part <i>re</i> and optional imaginary part <i>im</i> (default=0).</p>
</html>"));
    end fromReal;
    annotation(
      Documentation(info = "<html>
<p>Here the constructor operator(s) is/are defined.</p>
</html>"),
      Icon(graphics = {Rectangle(lineColor = {200, 200, 200}, fillColor = {248, 248, 248}, fillPattern = FillPattern.HorizontalCylinder, extent = {{-100, -100}, {100, 100}}, radius = 25.0), Rectangle(lineColor = {128, 128, 128}, fillPattern = FillPattern.None, extent = {{-100, -100}, {100, 100}}, radius = 25.0)}));
  end 'constructor';

  encapsulated operator function '0' "Zero-element of addition (= Complex(0))"
    import Complex;
    output Complex result "Complex(0)";
  algorithm
    result := Complex(0);
    annotation(
      Inline = true,
      Documentation(info = "<html>
<p>This function returns the zero-element of Complex, that is, Complex(0) = 0 + j*0.</p>
</html>"));
  end '0';

  encapsulated operator '-' "Unary and binary minus"
    function negate "Unary minus (multiply complex number by -1)"
      import Complex;
      input Complex c1 "Complex number";
      output Complex c2 "= -c1";
    algorithm
      c2 := Complex(-c1.re, -c1.im);
      annotation(
        Inline = true,
        Documentation(info = "<html>
<p>This function returns the binary minus of the given Complex number.</p>
</html>"));
    end negate;

    function subtract "Subtract two complex numbers"
      import Complex;
      input Complex c1 "Complex number 1";
      input Complex c2 "Complex number 2";
      output Complex c3 "= c1 - c2";
    algorithm
      c3 := Complex(c1.re - c2.re, c1.im - c2.im);
      annotation(
        Inline = true,
        Documentation(info = "<html>
<p>This function returns the difference of two given Complex numbers.</p>
</html>"));
    end subtract;
    annotation(
      Documentation(info = "<html>
<p>Here the unary and binary minus operator(s) is/are defined.</p>
</html>"),
      Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(lineColor = {200, 200, 200}, fillColor = {248, 248, 248}, fillPattern = FillPattern.HorizontalCylinder, extent = {{-100, -100}, {100, 100}}, radius = 25.0), Rectangle(lineColor = {128, 128, 128}, fillPattern = FillPattern.None, extent = {{-100, -100}, {100, 100}}, radius = 25.0), Line(points = {{-50, 0}, {50, 0}}, color = {0, 0, 0}, smooth = Smooth.None)}));
  end '-';

  encapsulated operator '*' "Multiplication"
    function multiply "Multiply two complex numbers"
      import Complex;
      input Complex c1 "Complex number 1";
      input Complex c2 "Complex number 2";
      output Complex c3 "= c1*c2";
    algorithm
      c3 := Complex(c1.re * c2.re - c1.im * c2.im, c1.re * c2.im + c1.im * c2.re);
      annotation(
        Inline = true,
        Documentation(info = "<html>
<p>This function returns the product of two given Complex numbers.</p>
</html>"));
    end multiply;

    function scalarProduct "Scalar product c1*c2 of two complex vectors"
      import Complex;
      input Complex c1[:] "Vector of Complex numbers 1";
      input Complex c2[size(c1, 1)] "Vector of Complex numbers 2";
      output Complex c3 "= c1*c2";
    algorithm
      c3 := Complex(0);
      for i in 1:size(c1, 1) loop
        c3 := c3 + c1[i] * c2[i];
      end for;
/*
       c3 :=Complex(c3.re + c1[i].re*c2[i].re - c1[i].im*c2[i].im,
                    c3.im + c1[i].re*c2[i].im + c1[i].im*c2[i].re);
       */
      annotation(
        Inline = true,
        Documentation(info = "<html>
<p>This function returns the scalar product of two given arrays of Complex numbers.</p>
</html>"));
    end scalarProduct;
    annotation(
      Documentation(info = "<html>
<p>Here the multiplication operator(s) is/are defined.</p>
</html>"),
      Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(lineColor = {200, 200, 200}, fillColor = {248, 248, 248}, fillPattern = FillPattern.HorizontalCylinder, extent = {{-100, -100}, {100, 100}}, radius = 25.0), Rectangle(lineColor = {128, 128, 128}, fillPattern = FillPattern.None, extent = {{-100, -100}, {100, 100}}, radius = 25.0), Line(points = {{-42, 36}, {39, -34}}, color = {0, 0, 0}, smooth = Smooth.None), Line(points = {{-42, -35}, {39, 37}}, color = {0, 0, 0}, smooth = Smooth.None), Line(points = {{-55, 1}, {52, 1}}, color = {0, 0, 0}, smooth = Smooth.None), Line(points = {{-1.5, 55}, {-2, -53}}, color = {0, 0, 0}, smooth = Smooth.None)}));
  end '*';

  encapsulated operator function '+' "Add two complex numbers"
    import Complex;
    input Complex c1 "Complex number 1";
    input Complex c2 "Complex number 2";
    output Complex c3 "= c1 + c2";
  algorithm
    c3 := Complex(c1.re + c2.re, c1.im + c2.im);
    annotation(
      Inline = true,
      Documentation(info = "<html>
<p>This function returns the sum of two given Complex numbers.</p>
</html>"));
  end '+';

  encapsulated operator function '/' "Divide two complex numbers"
    import Complex;
    input Complex c1 "Complex number 1";
    input Complex c2 "Complex number 2";
    output Complex c3 "= c1/c2";
  algorithm
    c3 := Complex(((+c1.re * c2.re) + c1.im * c2.im) / (c2.re * c2.re + c2.im * c2.im), ((-c1.re * c2.im) + c1.im * c2.re) / (c2.re * c2.re + c2.im * c2.im));
    annotation(
      Inline = true,
      Documentation(info = "<html>
<p>This function returns the quotient of two given Complex numbers.</p>
</html>"));
  end '/';

  encapsulated operator function '^' "Complex power of complex number"
    import Complex;
    input Complex c1 "Complex number";
    input Complex c2 "Complex exponent";
    output Complex c3 "= c1^c2";
  protected
    Real lnz = 0.5 * log(c1.re * c1.re + c1.im * c1.im);
    Real phi = atan2(c1.im, c1.re);
    Real re = lnz * c2.re - phi * c2.im;
    Real im = lnz * c2.im + phi * c2.re;
  algorithm
    c3 := Complex(exp(re) * cos(im), exp(re) * sin(im));
    annotation(
      Inline = true,
      Documentation(info = "<html>
<p>This function returns the given Complex numbers c1 to the power of the Complex number c2.</p>
</html>"));
  end '^';

  encapsulated operator function '==' "Test whether two complex numbers are identical"
    import Complex;
    input Complex c1 "Complex number 1";
    input Complex c2 "Complex number 2";
    output Boolean result "c1 == c2";
  algorithm
    result := c1.re == c2.re and c1.im == c2.im;
    annotation(
      Inline = true,
      Documentation(info = "<html>
<p>This function tests whether two given Complex numbers are equal.</p>
</html>"));
  end '==';

  encapsulated operator function 'String' "Transform Complex number into a String representation"
    import Complex;
    input Complex c "Complex number to be transformed in a String representation";
    input String name = "j" "Name of variable representing sqrt(-1) in the string";
    input Integer significantDigits = 6 "Number of significant digits that are shown";
    output String s = "";
  algorithm
    s := String(c.re, significantDigits = significantDigits);
    if c.im <> 0 then
      if c.im > 0 then
        s := s + " + ";
      else
        s := s + " - ";
      end if;
      s := s + String(abs(c.im), significantDigits = significantDigits) + "*" + name;
    end if;
    annotation(
      Inline = true,
      Documentation(info = "<html>
<p>This function converts a given Complex number to String representation.</p>
</html>"));
  end 'String';
  annotation(
    Protection(access = Access.hide),
    version = "3.2.1",
    versionBuild = 2,
    versionDate = "2013-08-14",
    dateModified = "2013-08-14 08:44:41Z",
    revisionId = "$Id::                                       $",
    conversion(noneFromVersion = "1.0", noneFromVersion = "1.1"),
    Documentation(info = "<html>
<p>Complex number defined as a record containing real and imaginary part, utilizing operator overloading.</p>
</html>"),
    Icon(graphics = {Rectangle(lineColor = {160, 160, 164}, fillColor = {160, 160, 164}, fillPattern = FillPattern.Solid, extent = {{-100, -100}, {100, 100}}, radius = 25.0), Text(lineColor = {255, 255, 255}, extent = {{-90, -50}, {90, 50}}, textString = "C")}));
end Complex;

package Modelica "Modelica Standard Library - Version 3.2.1 (Build 4)"
  extends Modelica.Icons.Package;

  package UsersGuide "User's Guide"
    extends Modelica.Icons.Information;

    class Overview "Overview of Modelica Library"
      extends Modelica.Icons.Information;
      annotation(
        Documentation(info = "<html>
<p>
The Modelica Standard Library consists of the following
main sub-libraries:
</p>

<table border=1 cellspacing=0 cellpadding=2>
<tr><th>Library Components</th> <th>Description</th></tr>

<tr><td valign=\"top\">
 <img src=\"modelica://Modelica/Resources/Images/UsersGuide/Lib-Electrical.png\">
 </td>
 <td valign=\"top\">
 <a href=\"modelica://Modelica.Electrical.Analog\">Analog</a><br>
 Analog electric and electronic components, such as
 resistor, capacitor, transformers, diodes, transistors,
 transmission lines, switches, sources, sensors.
 </td>
</tr>

<tr><td valign=\"top\">
 <img src=\"modelica://Modelica/Resources/Images/UsersGuide/Lib-Digital.png\">
 </td>
 <td valign=\"top\">
 <a href=\"modelica://Modelica.Electrical.Digital\">Digital</a><br>
 Digital electrical components based on the VHDL standard,
 like basic logic blocks with 9-value logic, delays, gates,
 sources, converters between 2-, 3-, 4-, and 9-valued logic.
 </td>
</tr>

<tr><td valign=\"top\">
 <img src=\"modelica://Modelica/Resources/Images/UsersGuide/Lib-Machines.png\">
 </td>
 <td valign=\"top\">
 <a href=\"modelica://Modelica.Electrical.Machines\">Machines</a><br>
            Electrical asynchronous-, synchronous-, and DC-machines
 (motors and generators) as well as 3-phase transformers.
 </td>
</tr>

<tr><td valign=\"top\">
 <img src=\"modelica://Modelica/Resources/Images/UsersGuide/Lib-FluxTubes.png\">
 </td>
 <td valign=\"top\">
 <a href=\"modelica://Modelica.Magnetic.FluxTubes\">FluxTubes</a><br>
Based on magnetic flux tubes concepts. Especially to model electro-magnetic actuators. Nonlinear shape, force, leakage, and material models. Material data for steel, electric sheet, pure iron, Cobalt iron, Nickel iron, NdFeB, Sm2Co17, and more.
 </td>
</tr>

<tr><td valign=\"top\">
 <img src=\"modelica://Modelica/Resources/Images/UsersGuide/Lib-Translational.png\">
 </td>
 <td valign=\"top\">
 <a href=\"modelica://Modelica.Mechanics.Translational\">Translational</a><br>
 1-dim. mechanical, translational systems, e.g.,
 sliding mass, mass with stops, spring, damper.
 </td>
</tr>

<tr><td valign=\"top\">
 <img src=\"modelica://Modelica/Resources/Images/UsersGuide/Lib-Rotational.png\">
 </td>
 <td valign=\"top\">
 <a href=\"modelica://Modelica.Mechanics.Rotational\">Rotational</a><br>
 1-dim. mechanical, rotational systems, e.g., inertias, gears,
 planetary gears, convenient definition of speed/torque dependent friction
 (clutches, brakes, bearings, ..)
 </td>
</tr>

<tr><td valign=\"top\" width=100>
 <img src=\"modelica://Modelica/Resources/Images/UsersGuide/Lib-MultiBody1.png\"><br>
 <img src=\"modelica://Modelica/Resources/Images/UsersGuide/Lib-MultiBody2.png\">
 </td>
 <td valign=\"top\">
 <a href=\"modelica://Modelica.Mechanics.MultiBody\">MultiBody</a>
 3-dim. mechanical systems consisting of joints, bodies, force and
 sensor elements. Joints can be driven by drive trains defined by
 1-dim. mechanical system library (Rotational).
 Every component has a default animation.
 Components can be arbitrarily connected together.
 </td>
</tr>

<tr><td valign=\"top\">
 <img src=\"modelica://Modelica/Resources/Images/UsersGuide/Lib-Fluid.png\">
 </td>
 <td valign=\"top\">
 <a href=\"modelica://Modelica.Fluid\">Fluid</a><br>
        1-dim. thermo-fluid flow in networks of vessels, pipes,
        fluid machines, valves and fittings. All media from the
        Modelica.Media library can be used (so incompressible or compressible,
        single or multiple substance, one or two phase medium).
 </td>
</tr>

<tr><td valign=\"top\">
 <img src=\"modelica://Modelica/Resources/Images/UsersGuide/Lib-Media.png\">
 </td>
 <td valign=\"top\">
 <a href=\"modelica://Modelica.Media\">Media</a><br>
 Large media library providing models and functions
 to compute media properties, such as h = h(p,T), d = d(p,T),
 for the following media:
 <ul>
 <li> 1240 gases and mixtures between these gases.</li>
 <li> incompressible, table based liquids (h = h(T), etc.).</li>
 <li> compressible liquids</li>
 <li> dry and moist air</li>
 <li> high precision model for water (IF97).</li>
 </ul>
 </td>
</tr>

<tr><td valign=\"top\">
 <img src=\"modelica://Modelica/Resources/Images/UsersGuide/Lib-Thermal.png\">
 </td>
 <td valign=\"top\">
 <a href=\"modelica://Modelica.Thermal.FluidHeatFlow\">FluidHeatFlow</a>,
 <a href=\"modelica://Modelica.Thermal.HeatTransfer\">HeatTransfer</a>
 Simple thermo-fluid pipe flow, especially to model cooling of machines
 with air or water (pipes, pumps, valves, ambient, sensors, sources) and
 lumped heat transfer with heat capacitors, thermal conductors, convection,
 body radiation, sources and sensors.
 </td>
</tr>

<tr><td valign=\"top\">
 <img src=\"modelica://Modelica/Resources/Images/UsersGuide/Lib-Blocks1.png\"><br>
 <img src=\"modelica://Modelica/Resources/Images/UsersGuide/Lib-Blocks2.png\">
 </td>
 <td valign=\"top\">
 <a href=\"modelica://Modelica.Blocks\">Blocks</a><br>
 Input/output blocks to model block diagrams and logical networks, e.g.,
 integrator, PI, PID, transfer function, linear state space system,
 sampler, unit delay, discrete transfer function, and/or blocks,
 timer, hysteresis, nonlinear and routing blocks, sources, tables.
 </td>
</tr>

<tr><td valign=\"top\">
 <img src=\"modelica://Modelica/Resources/Images/UsersGuide/Lib-StateGraph.png\">
 </td>
 <td valign=\"top\">
 <a href=\"modelica://Modelica.StateGraph\">StateGraph</a><br>
 Hierarchical state machines with a similar modeling power as Statecharts.
 Modelica is used as synchronous action language, i.e., deterministic
 behavior is guaranteed
 </td>
</tr>

<tr><td valign=\"top\">
 <pre>
 A = [1,2,3;
   3,4,5;
   2,1,4];
 b = {10,22,12};
 x = Matrices.solve(A,b);
 Matrices.eigenValues(A);
 </pre>
 </td>
 <td valign=\"top\">
 <a href=\"modelica://Modelica.Math\">Math</a>,
 <a href=\"modelica://Modelica.Utilities\">Utilities</a><br>
 Functions operating on vectors and matrices, such as for solving
 linear systems, eigen and singular values etc.,  and
 functions operating on strings, streams, files, e.g.,
 to copy and remove a file or sort a vector of strings.
 </td>
</tr>

</table>

</html>"));
    end Overview;

    class Connectors "Connectors"
      extends Modelica.Icons.Information;
      annotation(
        Documentation(info = "<html>

<p>
The Modelica standard library defines the most important
<b>elementary connectors</b> in various domains. If any possible,
a user should utilize these connectors in order that components
from the Modelica Standard Library and from other libraries
can be combined without problems.
The following elementary connectors are defined
(the meaning of potential, flow, and stream
variables is explained in section \"Connector Equations\" below):
</p>

<table border=1 cellspacing=0 cellpadding=1>
<tr><td valign=\"top\"><b>domain</b></td>
   <td valign=\"top\"><b>potential<br>variables</b></td>
   <td valign=\"top\"><b>flow<br>variables</b></td>
   <td valign=\"top\"><b>stream<br>variables</b></td>
   <td valign=\"top\"><b>connector definition</b></td>
   <td valign=\"top\"><b>icons</b></td></tr>

<tr><td valign=\"top\"><b>electrical<br>analog</b></td>
   <td valign=\"top\">electrical potential</td>
   <td valign=\"top\">electrical current</td>
   <td valign=\"top\"></td>
   <td valign=\"top\"><a href=\"modelica://Modelica.Electrical.Analog.Interfaces\">Modelica.Electrical.Analog.Interfaces</a>
     <br>Pin, PositivePin, NegativePin</td>
   <td valign=\"top\"><img src=\"modelica://Modelica/Resources/Images/UsersGuide/ElectricalPins.png\"></td></tr>

<tr><td valign=\"top\"><b>electrical<br>multi-phase</b></td>
   <td colspan=\"3\">vector of electrical pins</td>
   <td valign=\"top\"><a href=\"modelica://Modelica.Electrical.MultiPhase.Interfaces\">Modelica.Electrical.MultiPhase.Interfaces</a>
     <br>Plug, PositivePlug, NegativePlug</td>
   <td valign=\"top\"><img src=\"modelica://Modelica/Resources/Images/UsersGuide/ElectricalPlugs.png\"></td></tr>

<tr><td valign=\"top\"><b>electrical <br>space phasor</b></td>
   <td valign=\"top\">2 electrical potentials</td>
   <td valign=\"top\">2 electrical currents</td>
   <td valign=\"top\"></td>
   <td valign=\"top\"><a href=\"modelica://Modelica.Electrical.Machines.Interfaces\">Modelica.Electrical.Machines.Interfaces</a>
     <br>SpacePhasor</td>
   <td valign=\"top\"><img src=\"modelica://Modelica/Resources/Images/UsersGuide/SpacePhasor.png\"></td></tr>

<tr><td valign=\"top\"><b>quasi<br>stationaray<br>single phase</b></td>
   <td valign=\"top\">complex electrical potential</td>
   <td valign=\"top\">complex electrical current</td>
   <td valign=\"top\"></td>
   <td valign=\"top\"><a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Interfaces\">
                                       Modelica.Electrical.QuasiStationary.SinglePhase.Interfaces</a>
     <br>Pin, PositivePin, NegativePin</td>
   <td valign=\"top\"><img src=\"modelica://Modelica/Resources/Images/UsersGuide/QuasiStationarySinglePhasePins.png\"></td></tr>

<tr><td valign=\"top\"><b>quasi<br>stationary<br>multi-phase</b></td>
   <td colspan=\"3\">vector of quasi stationary single phase pins</td>
   <td valign=\"top\"><a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Interfaces\">Modelica.Electrical.QuasiStationary.MultiPhase.Interfaces</a>
     <br>Plug, PositivePlug, NegativePlug</td>
   <td valign=\"top\"><img src=\"modelica://Modelica/Resources/Images/UsersGuide/QuasiStationaryMultiPhasePlugs.png\"></td></tr>

<tr><td valign=\"top\"><b>electrical <br>digital</b></td>
   <td valign=\"top\">Integer (1..9)</td>
   <td valign=\"top\"></td>
   <td valign=\"top\"></td>
   <td valign=\"top\"><a href=\"modelica://Modelica.Electrical.Digital.Interfaces\">Modelica.Electrical.Digital.Interfaces</a>
     <br>DigitalSignal, DigitalInput, DigitalOutput</td>
   <td valign=\"top\"><img src=\"modelica://Modelica/Resources/Images/UsersGuide/Digital.png\"></td></tr>

<tr><td valign=\"top\"><b>magnetic<br>flux tubes</b></td>
   <td valign=\"top\">magnetic potential</td>
   <td valign=\"top\">magnetic flux</td>
   <td valign=\"top\"></td>
   <td valign=\"top\">
<a href=\"modelica://Modelica.Magnetic.FluxTubes.Interfaces\">Modelica.Magnetic.FluxTubes.Interfaces</a>
     <br>MagneticPort, PositiveMagneticPort, <br>NegativeMagneticPort</td>
   <td valign=\"top\"><img src=\"modelica://Modelica/Resources/Images/UsersGuide/MagneticPorts.png\"></td></tr>

<tr><td valign=\"top\"><b>magnetic<br>fundamental<br>wave</b></td>
   <td valign=\"top\">complex magnetic potential</td>
   <td valign=\"top\">complex magnetic flux</td>
   <td valign=\"top\"></td>
   <td valign=\"top\">
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.Interfaces\">Modelica.Magnetic.FundamentalWave.Interfaces</a>
     <br>MagneticPort, PositiveMagneticPort, <br>NegativeMagneticPort</td>
   <td valign=\"top\"><img src=\"modelica://Modelica/Resources/Images/UsersGuide/FundamentalWavePorts.png\"></td></tr>

<tr><td valign=\"top\"><b>translational</b></td>
   <td valign=\"top\">distance</td>
   <td valign=\"top\">cut-force</td>
   <td valign=\"top\"></td>
   <td valign=\"top\"><a href=\"modelica://Modelica.Mechanics.Translational.Interfaces\">Modelica.Mechanics.Translational.Interfaces</a>
     <br>Flange_a, Flange_b</td>
   <td valign=\"top\"><img src=\"modelica://Modelica/Resources/Images/UsersGuide/TranslationalFlanges.png\"></td></tr>

<tr><td valign=\"top\"><b>rotational</b></td>
   <td valign=\"top\">angle</td>
   <td valign=\"top\">cut-torque</td>
   <td valign=\"top\"></td>
   <td valign=\"top\"><a href=\"modelica://Modelica.Mechanics.Rotational.Interfaces\">Modelica.Mechanics.Rotational.Interfaces</a>
     <br>Flange_a, Flange_b</td>
   <td valign=\"top\"><img src=\"modelica://Modelica/Resources/Images/UsersGuide/RotationalFlanges.png\"></td></tr>

<tr><td valign=\"top\"><b>3-dim.<br>mechanics</b></td>
   <td valign=\"top\">position vector<br>
    orientation object</td>
   <td valign=\"top\">cut-force vector<br>
    cut-torque vector</td>
   <td valign=\"top\"></td>
   <td valign=\"top\"><a href=\"modelica://Modelica.Mechanics.MultiBody.Interfaces\">Modelica.Mechanics.MultiBody.Interfaces</a>
     <br>Frame, Frame_a, Frame_b, Frame_resolve</td>
   <td valign=\"top\"><img src=\"modelica://Modelica/Resources/Images/UsersGuide/MultiBodyFrames.png\"></td></tr>

<tr><td valign=\"top\"><b>simple<br>fluid flow</b></td>
   <td valign=\"top\">pressure<br>
    specific enthalpy</td>
   <td valign=\"top\">mass flow rate<br>
    enthalpy flow rate</td>
   <td valign=\"top\"></td>
   <td valign=\"top\"><a href=\"modelica://Modelica.Thermal.FluidHeatFlow.Interfaces\">Modelica.Thermal.FluidHeatFlow.Interfaces</a>
     <br>FlowPort, FlowPort_a, FlowPort_b</td>
   <td valign=\"top\"><img src=\"modelica://Modelica/Resources/Images/UsersGuide/FluidHeatFlowPorts.png\"></td></tr>

<tr><td valign=\"top\"><b>thermo<br>fluid flow</b></td>
   <td valign=\"top\">pressure</td>
   <td valign=\"top\">mass flow rate</td>
   <td valign=\"top\">specific enthalpy<br>mass fractions</td>
   <td valign=\"top\">
<a href=\"modelica://Modelica.Fluid.Interfaces\">Modelica.Fluid.Interfaces</a>
     <br>FluidPort, FluidPort_a, FluidPort_b</td>
   <td valign=\"top\"><img src=\"modelica://Modelica/Resources/Images/UsersGuide/FluidPorts.png\"></td></tr>

<tr><td valign=\"top\"><b>heat<br>transfer</b></td>
   <td valign=\"top\">temperature</td>
   <td valign=\"top\">heat flow rate</td>
   <td valign=\"top\"></td>
   <td valign=\"top\"><a href=\"modelica://Modelica.Thermal.HeatTransfer.Interfaces\">Modelica.Thermal.HeatTransfer.Interfaces</a>
     <br>HeatPort, HeatPort_a, HeatPort_b</td>
   <td valign=\"top\"><img src=\"modelica://Modelica/Resources/Images/UsersGuide/ThermalHeatPorts.png\"></td></tr>

<tr><td valign=\"top\"><b>blocks</b></td>
   <td valign=\"top\">
    Real variable<br>
    Integer variable<br>
    Boolean variable</td>
   <td valign=\"top\"></td>
   <td valign=\"top\"></td>
   <td valign=\"top\"><a href=\"modelica://Modelica.Blocks.Interfaces\">Modelica.Blocks.Interfaces</a>
     <br>
      RealSignal, RealInput, RealOutput<br>
      IntegerSignal, IntegerInput, IntegerOutput<br>
      BooleanSignal, BooleanInput, BooleanOutput</td>
   <td valign=\"top\"><img src=\"modelica://Modelica/Resources/Images/UsersGuide/Signals.png\"></tr>

<tr><td valign=\"top\"><b>complex<br>blocks</b></td>
   <td valign=\"top\">
    Complex variable</td>
   <td valign=\"top\"></td>
   <td valign=\"top\"></td>
   <td valign=\"top\"><a href=\"modelica://Modelica.ComplexBlocks.Interfaces\">Modelica.ComplexBlocks.Interfaces</a>
     <br>ComplexSignal, ComplexInput, ComplexOutput</td>
   <td valign=\"top\"><img src=\"modelica://Modelica/Resources/Images/UsersGuide/ComplexSignals.png\"></tr>

<tr><td valign=\"top\"><b>state<br>machine</b></td>
   <td valign=\"top\">Boolean variables<br>
    (occupied, set, <br>
     available, reset)</td>
   <td valign=\"top\"></td>
   <td valign=\"top\"></td>
   <td valign=\"top\"><a href=\"modelica://Modelica.StateGraph.Interfaces\">Modelica.StateGraph.Interfaces</a>
     <br>Step_in, Step_out, Transition_in, Transition_out</td>
   <td valign=\"top\"><img src=\"modelica://Modelica/Resources/Images/UsersGuide/StateGraphPorts.png\"></td></tr>

<tr><td colspan=\"5\">&nbsp;<br><b>Connectors from other libraries</b></td></tr>

<tr><td valign=\"top\"><b>hydraulic</b></td>
   <td valign=\"top\">pressure</td>
   <td valign=\"top\">volume flow rate</td>
   <td valign=\"top\"></td>
   <td valign=\"top\">HyLibLight.Interfaces
     <br>Port_A, Port_b</td>
   <td valign=\"top\"><img src=\"modelica://Modelica/Resources/Images/UsersGuide/HydraulicPorts.png\"></td></tr>

<tr><td valign=\"top\"><b>pneumatic</b></td>
   <td valign=\"top\">pressure</td>
   <td valign=\"top\">mass flow rate</td>
   <td valign=\"top\"></td>
   <td valign=\"top\">PneuLibLight.Interfaces
     <br>Port_1, Port_2</td>
   <td valign=\"top\"><img src=\"modelica://Modelica/Resources/Images/UsersGuide/PneumaticPorts.png\"></td></tr>
</table>

<p>
In all domains, usually 2 connectors are defined. The variable declarations
are <b>identical</b>, only the icons are different in order that it is easy
to distinguish connectors of the same domain that are attached at the same
component.
</p>

<h4>Hierarchical Connectors </h4>
<p>
Modelica supports also hierarchical connectors, in a similar way as hierarchical models.
As a result, it is, e.g., possible, to collect elementary connectors together.
For example, an electrical plug consisting of two electrical pins can be defined as:
</p>

<blockquote>
<pre>
<b>connector</b> Plug
   <b>import</b> Modelica.Electrical.Analog.Interfaces;
   Interfaces.PositivePin phase;
   Interfaces.NegativePin ground;
<b>end</b> Plug;
</pre>
</blockquote>

<p>
With one connect(..) equation, either two plugs can be connected
(and therefore implicitly also the phase and ground pins) or a
Pin connector can be directly connected to the phase or ground of
a Plug connector, such as \"connect(resistor.p, plug.phase)\".
</p>

<h4 id=\"ConnectorEquations\">Connector Equations</h4>

<p>
The connector variables listed above have been basically determined
with the following strategy:
</p>

<ol>
<li> State the relevant balance equations and boundary
     conditions of a volume for the particular physical domain.</li>
<li> Simplify the balance equations and boundary conditions
     of (1) by taking the
     limit of an infinitesimal small volume
     (e.g., thermal domain:
      temperatures are identical and heat flow rates
      sum up to zero).
</li>
<li> Use the variables needed for the balance equations
     and boundary conditions of (2)
     in the connector and select appropriate Modelica
     <b>prefixes</b>, so that these equations
     are generated by the Modelica connection semantics.
</ol>

<p>
The Modelica connection semantics is sketched at hand
of an example: Three connectors c1, c2, c3 with the definition
</p>

<pre>
<b>connector</b> Demo
  Real        p;  // potential variable
  <b>flow</b>   Real f;  // flow variable
  <b>stream</b> Real s;  // stream variable
<b>end</b> Demo;
</pre>

<p>
are connected together with
</p>

<pre>
   <b>connect</b>(c1,c2);
   <b>connect</b>(c1,c3);
</pre>

<p>
then this leads to the following equations:
</p>

<pre>
  // Potential variables are identical
  c1.p = c2.p;
  c1.p = c3.p;

  // The sum of the flow variables is zero
  0 = c1.f + c2.f + c3.f;

  /* The sum of the product of flow variables and upstream stream variables is zero
     (this implicit set of equations is explicitly solved when generating code;
     the \"&lt;undefined&gt;\" parts are defined in such a way that
     inStream(..) is continuous).
  */
  0 = c1.f*(<b>if</b> c1.f > 0 <b>then</b> s_mix <b>else</b> c1.s) +
      c2.f*(<b>if</b> c2.f > 0 <b>then</b> s_mix <b>else</b> c2.s) +
      c3.f*(<b>if</b> c3.f > 0 <b>then</b> s_mix <b>else</b> c3.s);

  <b>inStream</b>(c1.s) = <b>if</b> c1.f > 0 <b>then</b> s_mix <b>else</b> &lt;undefined&gt;;
  <b>inStream</b>(c2.s) = <b>if</b> c2.f > 0 <b>then</b> s_mix <b>else</b> &lt;undefined&gt;;
  <b>inStream</b>(c3.s) = <b>if</b> c3.f > 0 <b>then</b> s_mix <b>else</b> &lt;undefined&gt;;
</pre>

</html>"));
    end Connectors;

    package Conventions "Conventions"
      extends Modelica.Icons.Information;

      package Documentation "HTML documentation"
        extends Modelica.Icons.Information;

        package Format "Format"
          extends Modelica.Icons.Information;

          class Cases "Cases"
            extends Modelica.Icons.Information;
            annotation(
              Documentation(info = "<html>

<p>In the Modelica documentation sometimes different cases have to be distinguished. If the case distinction refers to Modelica parameters or variables (Boolean expressions) the comparisons should be written in the style of Modelica code within <code>&lt;code&gt;</code> and <code>&lt;/code&gt;</code>
</p>

<h5>Example 1</h5>

<code>&lt;p&gt;If &lt;code&gt;useCage == true&lt;/code&gt;, a damper cage is considered in the model...&lt;/p&gt;</code>

<p>appears as</p>

<p>If <code>useCage == true</code>, a damper cage is considered in the model...</p>

<p>
For more complex case scenarios a unordered list should be used. In this case only Modelica specific code segments and Boolean expressions.
</p>

<h5>Example 2</h5>

<pre>
&lt;ul&gt;
  &lt;li&gt; If &lt;code&gt;useCage == true&lt;/code&gt;, a damper cage is considered in the model.
       Cage parameters must be specified in this case.&lt;/li&gt;
  &lt;li&gt; If &lt;code&gt;useCage == false&lt;/code&gt;, the damper cage is omitted.&lt;/li&gt;
&lt;/ul&gt;</pre>

<p>appears as</p>

<ul>
  <li> If <code>useCage == true</code>, a damper cage is considered in the model.
       Cage parameters must be specified in this case.</li>
  <li> If <code>useCage == false</code>, the damper cage is omitted.</li>
</ul>

<p>
In a more equation oriented case additional equations or code segments can be added.
</p>

<h5>Example 3</h5>

<pre>
&lt;ul&gt;
  &lt;li&gt;if &lt;code&gt;usePolar == true&lt;/code&gt;, assign magnitude and angle to output &lt;br&gt;
  &lt;!-- insert graphical representation of equations --&gt;
  y[i,1] = sqrt( a[i]^2 + b[i]^2 ) &lt;br&gt;
  y[i,2] = atan2( b[i], a[i] )
  &lt;/li&gt;
  &lt;li&gt;if &lt;code&gt;usePolar == false&lt;/code&gt;, assign cosine and sine to output &lt;br&gt;
  &lt;!-- insert graphical representation of equations --&gt;
  y[i,1] = a[i] &lt;br&gt;
  y[i,2] = b[i]
  &lt;/li&gt;
&lt;/ul&gt;</pre>

<p>appears as</p>

<ul>
  <li>if <code>usePolar == true</code>, assign magnitude and angle to output <br>

  <img src=\"modelica://Modelica/Resources/Images/UsersGuide/Conventions/Documentation/Format/Cases/y_i1_polar.png\"
       alt=\"y[i,1] = sqrt( a[i]^2 + b[i]^2 )\"> <br>
  <img src=\"modelica://Modelica/Resources/Images/UsersGuide/Conventions/Documentation/Format/Cases/y_i2_polar.png\"
       alt=\"y[i,2] = atan2( b[i], a[i] )\">
  </li>
  <li>if <code>usePolar == false</code>, assign cosine and sine to output <br>
  <img src=\"modelica://Modelica/Resources/Images/UsersGuide/Conventions/Documentation/Format/Cases/y_i1_rect.png\"
       alt=\"y[i,1] = a[i]\"> <br>
  <img src=\"modelica://Modelica/Resources/Images/UsersGuide/Conventions/Documentation/Format/Cases/y_i2_rect.png\"
       alt=\" y[i,2] = b[i]\">
  </li>
</ul>

</html>"));
          end Cases;

          class Code "Code"
            extends Modelica.Icons.Information;
            annotation(
              Documentation(info = "<html>
<p>
<a href=\"modelica://Modelica.UsersGuide.Conventions.ModelicaCode\">Modelica code</a> conventions of class and instance names,
parameters and variables are specified separately. In this section it is summarized how to refer to
Modelica code in the HTML documentation.
</p>

<ol>
<li> For constants, parameters and variables in code segments <code>&lt;code&gt;</code> and <code>&lt;/code&gt;</code>
     should to be used, e.g., <br>
     <code><b>parameter</b> Modelica.SIunits.Time tStart \"Start time\"</code></li>
<li> Write multi or single line code segments using <code>&lt;pre&gt;</code> and <code>&lt;/pre&gt;</code>.</li>
<li> Multi line or single line code shall not be indented.</li>
<li> Inline code segments may be typeset with <code>&lt;code&gt;</code> and <code>&lt;/code&gt;</code>.</li>
<li> In code segments use bold to emphasize Modelica keywords.</li>
</ol>

<h5>Example 1</h5>

<pre>
&lt;pre&gt;
&lt;b&gt;connector&lt;/b&gt; Frame
   ...
   &lt;b&gt;flow&lt;/b&gt; SI.Force f[Woehrnschimmel1998] &lt;b&gt;annotation&lt;/b&gt;(unassignedMessage=\"...\");
&lt;b&gt;end&lt;/b&gt; Frame;
&lt;/pre&gt;</pre>

<p>appears as</p>

<pre>
<b>connector</b> Frame
   ...
   <b>flow</b> SI.Force f[Woehrnschimmel1998] <b>annotation</b>(unassignedMessage=\"...\");
<b>end</b> Frame;
</pre>

<h5>Example 2</h5>

<pre>
&lt;pre&gt;
&lt;b&gt;parameter&lt;/b&gt; Modelica.SIunits.Conductance G=1 &quot;Conductance&quot;;
&lt;/pre&gt;
</pre>

<p>appears as</p>

<pre>
<b>parameter</b> Modelica.SIunits.Conductance G=1 &quot;Conductance&quot;;
</pre>
</html>"));
          end Code;

          class Equations "Equations"
            extends Modelica.Icons.Information;
            annotation(
              Documentation(info = "<html>

<p>
In the context of <a href=\"http://www.w3c.org/\">HTML</a> documentation
equations should have a graphical representation in PNG format. For that purpose tool
specific math typing capabilities can be used. Alternatively the LaTeX to HTML translator
<a href=\"http://www.latex2html.org\">LaTeX2HTML</a>, or the
<a href=\"http://www.homeschoolmath.net/worksheets/equation_editor.php\">Online Equation Editor</a>
or <a href=\"http://www.codecogs.com/components/eqneditor/editor.php\">codecogs</a> can be used.
</p>

<p>
A typical equation, e.g., of a Fourier synthesis, could look like<br>
<img
 src=\"modelica://Modelica/Resources/Images/UsersGuide/Conventions/Documentation/Format/Equations/fourier.png\"> <br>
or <br>
<img
 src=\"modelica://Modelica/Resources/Images/UsersGuide/Conventions/Documentation/Format/Equations/sample.png\"
 alt=\"y=a_1+a_2\"><br>
In an <code>alt</code> tag the original equation should be stored, e.g.,</p>
<pre>
&lt;img
&nbsp;src=&quot;modelica://Modelica/Resources/Images/UsersGuide/Conventions/Documentation/Format/Equations/sample.png&quot;
&nbsp;alt=&quot;y=a_1+a_2&quot;&gt;
</pre>

<p>
If one wants to refer to particular variables and parameters in the documentation text, either a
graphical representation (PNG file) or italic fonts for regular physical symbols and lower case
<a href=\"http://www.w3.org/TR/html4/sgml/entities.html\">Greek letters</a>
should be used. Full word variables and full word indices should be spelled within &lt;code&gt; and &lt;/code&gt;.
Vector and array indices should be typeset as subscripts using the &lt;sub&gt; and &lt;/sub&gt; tags.
</p>

<p> Examples for such variables and parameters are:
<i>&phi;</i>, <i>&phi;</i><sub>ref</sub>, <i>v<sub>2</sub></i>, <code>useDamperCage</code>.
</p>

<h4>Numbered equations</h4>

<p>For numbering equations a one row table with two columns should be used. The equation number should be placed in the right column:</p>

<pre>
&lt;table border=\"0\" cellspacing=\"10\" cellpadding=\"2\"&gt;
  &lt;tr&gt;
      &lt;td&gt;&lt;img
      src=&quot;modelica://Modelica/Resources/Images/UsersGuide/Conventions/Documentation/Format/Equations/sample.png&quot;
      alt=&quot;y=a_1+a_2&quot;&gt; &lt;/td&gt;
      &lt;td&gt;(1)&lt;/td&gt;
  &lt;/tr&gt;
&lt;/table&gt;
</pre>

<p>appears as:</p>

<table border=\"0\" cellspacing=\"10\" cellpadding=\"2\">
  <tr>
    <td><img
         src=\"modelica://Modelica/Resources/Images/UsersGuide/Conventions/Documentation/Format/Equations/sample.png\"
         alt=\"y=a_1+a_2\"></td>
    <td>(1)</td>
  </tr>
</table>

</html>"));
          end Equations;

          class Figures "Figures"
            extends Modelica.Icons.Information;
            annotation(
              Documentation(info = "<html>
<p>
Figures should in particular be included to examples to discuss the problems and results of the respective model. The library developers are yet encouraged to add figures to the documentation of other components to support the understanding of the users of the library.
</p>

<ol>
<li> Figures have to be placed <strong>outside</strong> of paragraphs to be HTML compliant.</li>
<li> Figures need to have <strong>at least</strong> a <code>src</code> and an <code>alt</code> attribute defined to be HTML compliant.</li>
<li> Technical figures should be placed within a table environment. Each technical figure should then also have a caption. The figure caption starts with a capital letter.</li>
<li> Illustration can be embedded without table environment.</li>
</ol>

<h4>Location of files</h4>

The <code>PNG</code> files should be placed in a folder which exactly represents the package structure.

<h5>Example 1</h5>

<p>This example shows how an illustration should be embedded in the Example
<a href=\"modelica://Modelica.Blocks.Examples.PID_Controller\">PID_Controller</a> of the
<a href=\"modelica://Modelica.Blocks\">Blocks</a> package.</p>

<pre>
&lt;img src=\"modelica://Modelica/Resources/Images/UsersGuide/Conventions/Images/Blocks/Examples/PID_controller.png\"
     alt=\"PID_controller.png\"&gt;
</pre>

<h5>Example 2</h5>

<p>This is a simple example of a technical figure with caption.</p>

<pre>
&lt;table border=\"0\" cellspacing=\"0\" cellpadding=\"2\"&gt;
  &lt;caption align=\"bottom\"&gt;Caption starts with a capital letter&lt;/caption&gt;
  &lt;tr&gt;
    &lt;td&gt;
      &lt;img src=\"modelica://Modelica/Resources/Images/UsersGuide/Conventions/Images/Blocks/Examples/PID_controller.png\"
           alt=\"PID_controller.png\"&gt;
    &lt;/td&gt;
  &lt;/tr&gt;
&lt;/table&gt;
</pre>

<h5>Example 3</h5>

<p>To refer to a certain figure, a figure number may be added. In such case the figure name (Fig.) including the figure enumeration (1,2,...) have to be displayed bold using <code>&lt;strong&gt;</code> and <code>&lt;/strong&gt;</code>.</p>
<p>The figure name and enumeration should look like this: <strong>Fig. 1:</strong></p>
<p>Figures have to be enumerated manually. </p>

<pre>
&lt;table border=\"0\" cellspacing=\"0\" cellpadding=\"2\"&gt;
  &lt;caption align=\"bottom\"&gt;&lt;strong&gt;Fig. 2: &lt;/strong&gt;Caption starts with a capital letter&lt;/caption&gt;
  &lt;tr&gt;
    &lt;td&gt;
      &lt;img src=\"modelica://Modelica/Resources/Images/UsersGuide/Conventions/Images/Blocks/Examples/PID_controller.png\"
           alt=\"PID_controller.png\"&gt;
    &lt;/td&gt;
  &lt;/tr&gt;
&lt;/table&gt;
</pre>

</html>"));
          end Figures;

          class Hyperlinks "Hyperlinks"
            extends Modelica.Icons.Information;
            annotation(
              Documentation(info = "<html>
<ol>
<li> Hyperlinks should always be made when referring to a component or package.</li>
<li> The hyperlink text in between <code>&lt;a href=&quot;...&quot;&gt;</code> and <code>&lt;/a&gt;</code> should include the full main package name.</li>
<li> A link to an external component should include the full name of the package that it is referred to.</li>
<li> Modelica hyperlinks have to use the scheme <code>&quot;modelica://...&quot;</code></li>
<li> For hyperlinks referring to a Modelica component, see Example 1 and 2.</li>
<li> No links to commercial web sites are allowed.</li>
</ol>
<h5>Example 1</h5>
<pre>
&lt;a href=\"modelica://Modelica.Mechanics.MultiBody.UsersGuide.Tutorial.LoopStructures.PlanarLoops\"&gt;
         Modelica.Mechanics.MultiBody.UsersGuide.Tutorial.LoopStructures.PlanarLoops&lt;/a&gt;</pre>
<p>appears as</p>
<a href=\"modelica://Modelica.Mechanics.MultiBody.UsersGuide.Tutorial.LoopStructures.PlanarLoops\">
         Modelica.Mechanics.MultiBody.UsersGuide.Tutorial.LoopStructures.PlanarLoops</a>
<h5>Example 2</h5>
<pre>
&lt;p&gt;
  The feeder cables are connected to an
  &lt;a href=\"modelica://Modelica.Electrical.Machines.BasicMachines.AsynchronousInductionMachines.AIM_SquirrelCage\"&gt;
  induction machine&lt;/a&gt;.
&lt;/p&gt;</pre>
<p>appears as</p>
<p>
  The feeder cables are connected to an
  <a href=\"modelica://Modelica.Electrical.Machines.BasicMachines.AsynchronousInductionMachines.AIM_SquirrelCage\">
  induction machine</a>.
</p>
</html>"));
          end Hyperlinks;

          class Lists "Lists"
            extends Modelica.Icons.Information;
            annotation(
              Documentation(info = "<html>
<p>
Lists have to be placed <strong>outside</strong> of paragraphs to be HTML compliant.
</p>

<ol>
<li> Items of a list shall start with
<ul>
    <li> a capital letter if each item is a full sentence</li>
    <li> a small letter, if only text fragments are used or the list is fragment of a sentence</li>
</ul></li>
</ol>

<h5>Example 1</h5>

<p>This is a simple example of an enumerated (ordered) list</p>

<pre>
&lt;ol&gt;
  &lt;li&gt;item 1&lt;/li&gt;
  &lt;li&gt;item 2&lt;/li&gt;
&lt;/ol&gt;
</pre>
<p>appears as</p>
<ol>
  <li>item 1</li>
  <li>item 2</li>
</ol>

<h5>Example 2</h5>

<p>This is a simple example of an unnumbered list.</p>

<pre>
&lt;ul&gt;
  &lt;li&gt;item 1&lt;/li&gt;
  &lt;li&gt;item 2&lt;/li&gt;
&lt;/ul&gt;
</pre>
<p>appears as</p>
<ul>
  <li>item 1</li>
  <li>item 2</li>
</ul>
</html>"));
          end Lists;

          class References "References"
            extends Modelica.Icons.Information;
            annotation(
              Documentation(info = "<html>

<h4>General</h4>
<ol>
<li> Refer to references by [1], [Andronov1973], etc. by hyperlink and summarize literature in the references subsection of
     <a href=\"modelica://Modelica.UsersGuide.Conventions.UsersGuide.References\">Conventions.UsersGuide.References</a>.
<li> There has to be made at least one citation to each reference.</li>
</ol>

<h5>Example</h5>

<pre>
&lt;p&gt;
  More details about sensorless rotor temperature estimation
  can be found in &lt;a href=\"modelica://Modelica.UsersGuide.Conventions.UsersGuide.References\"&gt;[Gao2008]&lt;/a.&gt;
&lt;/p&gt;</pre>
<p>appears as</p>
<p>
  More details about sensorless rotor temperature estimation can be found in <a href=\"modelica://Modelica.UsersGuide.Conventions.UsersGuide.References\">[Gao2008]</a>.
</p>

</html>"));
          end References;

          class Tables "Tables"
            extends Modelica.Icons.Information;
            annotation(
              Documentation(info = "<html>
<ol>
<li> Tables should always be typeset with <code>&lt;table&gt;</code> and <code>&lt;/table&gt;</code>,
     not with <code>&lt;pre&gt;</code> and <code>&lt;/pre&gt;</code>.</li>
<li> Tables have to be placed <strong>outside</strong> of paragraphs to be HTML compliant.</li>
<li> Each table must have a table caption. </li>
<li> Table headers and entries start with capital letters.</li>
</ol>

<h5>Example 1</h5>

<p>This is a simple example of a table.</p>

<pre>
&lt;table border=\"1\" cellspacing=\"0\" cellpadding=\"2\"&gt;
  &lt;caption align=\"bottom\"&gt;Caption starts with a capital letter&lt;/caption&gt;
  &lt;tr&gt;
    &lt;th&gt;Head 1&lt;/th&gt;
    &lt;th&gt;Head 2&lt;/th&gt;
  &lt;/tr&gt;
  &lt;tr&gt;
    &lt;td&gt;Entry 1&lt;/td&gt;
    &lt;td&gt;Entry 2&lt;/td&gt;
  &lt;/tr&gt;
  &lt;tr&gt;
    &lt;td&gt;Entry 3&lt;/td&gt;
    &lt;td&gt;Entry 4&lt;/td&gt;
  &lt;/tr&gt;
&lt;/table&gt;
</pre>
<p>appears as</p>
<table border=\"1\" cellspacing=\"0\" cellpadding=\"2\">
  <caption align=\"bottom\">Caption starts with a capital letter</caption>
  <tr>
    <th><b>Head 1</b></th>
    <th><b>Head 2</b></th>
  </tr>
  <tr>
    <td>Entry 1</td>
    <td>Entry 2</td>
  </tr>
  <tr>
    <td>Entry 3</td>
    <td>Entry 4</td>
  </tr>
</table>

<h5>Example 2</h5>

<p>In this case of table captions, the table name (Tab.) including the table enumeration (1,2,...)
has to be displayed bold using <code>&lt;b&gt;</code> and <code>&lt;/b&gt;</code>. The table name
and enumeration should look like this: <b>Tab. 1:</b> Tables have to be enumerated manually.</p>

<pre>
&lt;table border=\"1\" cellspacing=\"0\" cellpadding=\"2\"&gt;
  &lt;caption align=\"bottom\"&gt;&lt;b&gt;Tab 2: &lt;/b&gt;Caption starts with a capital letter&lt;/caption&gt;
  &lt;tr&gt;
    &lt;th&gt;Head 1&lt;/th&gt;
    &lt;th&gt;Head 2&lt;/th&gt;
  &lt;/tr&gt;
  &lt;tr&gt;
    &lt;td&gt;Entry 1&lt;/td&gt;
    &lt;td&gt;Entry 2&lt;/td&gt;
  &lt;/tr&gt;
  &lt;tr&gt;
    &lt;td&gt;Entry 3&lt;/td&gt;
    &lt;td&gt;Entry 4&lt;/td&gt;
  &lt;/tr&gt;
&lt;/table&gt;
</pre>
<p>appears as</p>
<table border=\"1\" cellspacing=\"0\" cellpadding=\"2\">
  <caption align=\"bottom\"><b>Tab. 2: </b>Caption starts with a capital letter</caption>
  <tr>
    <th>Head 1</th>
    <th>Head 2</th>
  </tr>
  <tr>
    <td>Entry 1</td>
    <td>Entry 2</td>
  </tr>
  <tr>
    <td>Entry 3</td>
    <td>Entry 4</td>
  </tr>
</table>

</html>"));
          end Tables;
          annotation(
            Documentation(info = "<html>

<p>
In this section the format UsersGuide of the HTML documentation are specified.
The <a href=\"modelica://Modelica.UsersGuide.Conventions.Documentation.Structure\">structure</a> of the documentation is specified separately.
</p>

<h4>Paragraphs</h4>

<ol>
<li> In each section the paragraphs should start with <code>&lt;p&gt;</code>
     and terminate with <code>&lt;/p&gt;</code>.</li>
<li> Do not write plain text without putting it in a paragraph.</li>
<li> No artificial line breaks <code>&lt;br&gt;</code> should be added within text paragraphs if possible.
     Use separate paragraphs instead.</li>
<li> After a colon (:) continue with capital letter if new sentence starts;
     for text fragments continue with lower case letter</li>
</ol>

<h4>Emphasis</h4>

<ol>
<li> For setting text in <strong>strong font</strong> (normally interpreted as <strong>boldface</strong>) the tags <code>&lt;strong&gt;</code> and <code>&lt;/strong&gt;</code> have to be used.</li>
<li> For <em>emphasizing</em> text fragments <code>&lt;em&gt;</code> and <code>&lt;/em&gt;</code> has to be used.</li>
<li> Modelica terms such as expandable bus, array, etc. should not be emphasized anyhow.</li>
</ol>

<h4>Capitalization of Text</h4>

<ol>
<li> Table headers and entries should start with capital letters</li>
<li> Table entries should start with lower case letter if only text fragments are used.</li>
<li> Table and figure captions start with a capital letter</li>
</ol>

</html>"));
        end Format;

        class Structure "Structure"
          extends Modelica.Icons.Information;
          annotation(
            Documentation(info = "<html>

<h4>General</h4>

<ol>
<li> In the HTML documentation of any Modelica library, the headings <code>&lt;h1&gt;</code>,
     <code>&lt;h2&gt;</code> and <code>&lt;h3&gt;</code> should not be used, because they are utilized by
     the automatically generated documentation.
<li> The utilized heading format starts with <code>&lt;h4&gt;</code> and terminates with <code>&lt;/h4&gt;</code>, e.g.,
     <code>&lt;h4&gt;Description&lt;/h4&gt;</code> </li>
<li> The  <code>&lt;h4&gt;</code> and  <code>&lt;h5&gt;</code> headings must not be terminated by a colon (:).</li>
<li> For additional structuring <code>&lt;h5&gt;</code> and <code>&lt;/h5&gt;</code> may be used as demonstrated below.
</ol>

<h4>Structure</h4>
<p>
The following parts should be added to the documentation of each component:
</p>

<ol>
<li> General information without additional subsection explains how the class works</li>
<li> <b>Syntax</b> (for functions only): shows syntax of function call with minimum and full input parameters</li>
<li> <b>Implementation</b> (optional): explains how the implementation is made </li>
<li> <b>Limitations</b> (optional): explains the limitations of the component</li>
<li> <b>Notes</b> (optional): if required/useful </li>
<li> <b>Examples</b> (optional): if required/useful </li>
<li> <b>Acknowledgments</b> (optional): if required </li>
<li> <b>See also</b>: shows hyperlinks to related models </li>
<li> <b>Revision history</b> (optional): if required/intended for a package/model, the revision history
        should be placed in <code>annotation(Documentation(revisions=&quot;...&quot;));</code></li>
</ol>

<p>
These sections should appear in the listed order. The only exceptions are hierarchically structured notes and examples as explained in the following.
</p>

<h4>Additional notes and examples</h4>

<p>Some additional notes or examples may require additional <code>&lt;h5&gt;</code> headings. For either notes or examples the following cases may be applied:</p>

<h5>Example 1</h5>
<p>
This is an example of a single note.
</p>

<pre>
&lt;h5&gt;Note&lt;/h5&gt;
&lt;p&gt;This is the note.&lt;/p&gt;
</pre>

<h5>Example 2</h5>
<p>
This is an example of a very simple structure.
</p>

<pre>
&lt;h5&gt;Notes&lt;/h5&gt;
&lt;p&gt;This is the first note.&lt;/p&gt;
&lt;p&gt;This is the second note.&lt;/p&gt;
</pre>

<h5>Example 3</h5>

<p>
This example shows a more complex structure with enumeration.
</p>

<pre>
&lt;h5&gt;Note 1&lt;/h5&gt;
...
&lt;h5&gt;Note 2&lt;/h5&gt;
...
</pre>
<h4>Automatically created documentation</h4>
<p>
For parameters, connectors, as well as inputs and outputs of function automatic documentation is generated by the tool from the quoted comments.
</p>
</html>"));
        end Structure;
        annotation(
          Documentation(info = "<html>
<a href=\"http://www.w3c.org/\">HTML</a> documentation of Modelica classes.
</html>"));
      end Documentation;

      package ModelicaCode "Modelica code"
        extends Modelica.Icons.Information;

        class Format "Format"
          extends Modelica.Icons.Information;
          annotation(
            Documentation(info = "<html>

<ol>
<li> In the <b>icon</b> of a component the instance name is displayed
     (text string <code>%name</code>) in <b>blue color</b>.
     Parameter values, e.g., resistance, mass, gear ratio, are displayed
     in the icon in <b>black color</b> in a smaller font size as the instance name.</li>
<li> Comments and annotations always start with a capital letter, e.g.,
     <code><b>parameter</b> Real a = 1 \"Arbitrary factor\";</code></li>
</ol>

</html>"));
        end Format;

        class Naming "Naming convention"
          extends Modelica.Icons.Information;
          annotation(
            Documentation(info = "<html>

<ol>
<li> <b>Class and instance names</b> are usually written in upper and lower case
     letters, e.g., \"ElectricCurrent\". An underscore may be used in names.
     However, it has to be taken into account that the last underscore in a
     name might indicate that the following characters are rendered as a subscript.
     Example: \"pin_a\" may be rendered as \"pin<sub>a</sub>\".</li>

<li> <b>Class names</b> start always with an upper case letter,
     with the exception of functions, that start with a lower case letter.</li>

<li> <b>Instance names</b>, i.e., names of component instances and
     of variables (with the exception of constants),
     start usually with a lower case letter with only
     a few exceptions if this is common sense
     (such as <code>T</code> for a temperature variable).</li>

<li> <b>Constant names</b>, i.e., names of variables declared with the
     \"constant\" prefix, follow the usual naming conventions
     (= upper and lower case letters) and start usually with an
     upper case letter, e.g., UniformGravity, SteadyState.</li>

<li> The two <b>connectors</b> of a domain that have identical declarations
     and different icons are usually distinguished by <code>_a</code>, <code>_b</code>
     or <code>_p</code>, <code>_n</code>, e.g., <code>Flange_a</code>, <code>Flange_b</code>,
     <code>HeatPort_a</code>, <code>HeatPort_b</code>.</li>

<li> A <b>connector class</b> has the instance
     name definition in the diagram layer and not in the icon layer.</li>
</ol>

<h4>Variable names</h4>
<p>In the following table typical variable names are listed. This list should be completed.</p>

<table border=\"1\" cellpadding=\"2\" cellspacing=\"0\" >
   <caption align=\"bottom\">Variables and names</caption>
   <tr>
      <th>Variable</th>
      <th>Quantity</th>
    </tr>
     <tr>
      <td>a</td>
      <td>acceleration</td>
    </tr>
    <tr>
      <td>A</td>
      <td>area</td>
    </tr>
    <tr>
      <td>C</td>
      <td>capacitance</td>
    </tr>
    <tr>
      <td>d</td>
      <td>damping, density, diameter</td>
    </tr>
    <tr>
      <td>dp</td>
      <td>pressureDrop</td>
    </tr>
    <tr>
      <td>e</td>
      <td>specificEntropy</td>
    </tr>
    <tr>
      <td>E</td>
      <td>energy, entropy</td>
    </tr>
    <tr>
      <td>eta</td>
      <td>efficiency</td>
    </tr>
    <tr>
      <td>f</td>
      <td>force, frequency</td>
    </tr>
    <tr>
      <td>G</td>
      <td>conductance</td>
    </tr>
    <tr>
      <td>H</td>
      <td>enthalpy</td>
    </tr>
    <tr>
      <td>h</td>
      <td>height, specificEnthalpy</td>
    </tr>
    <tr>
      <td>HFlow</td>
      <td>enthalpyFlow</td>
    </tr>
    <tr>
      <td>i</td>
      <td>current</td>
    </tr>
    <tr>
      <td>J</td>
      <td>inertia</td>
    </tr>
    <tr>
      <td>l</td>
      <td>length</td>
    </tr>
    <tr>
      <td>L</td>
      <td>Inductance</td>
    </tr>
    <tr>
      <td>m</td>
      <td>mass</td>
    </tr>
    <tr>
      <td>M</td>
      <td>mutualInductance</td>
    </tr>
    <tr>
      <td>mFlow</td>
      <td>massFlow</td>
    </tr>
    <tr>
      <td>P</td>
      <td>power</td>
    </tr>
    <tr>
      <td>p</td>
      <td>pressure</td>
    </tr>
    <tr>
      <td>Q</td>
      <td>heat</td>
    </tr>
    <tr>
      <td>Qflow</td>
      <td>heatFlow</td>
    </tr>
    <tr>
      <td>r</td>
      <td>radius</td>
    </tr>
    <tr>
      <td>R</td>
      <td>radius, resistance</td>
    </tr>
    <tr>
      <td>t</td>
      <td>time</td>
    </tr>
    <tr>
      <td>T</td>
      <td>temperature</td>
    </tr>
    <tr>
      <td>tau</td>
      <td>torque</td>
    </tr>
    <tr>
      <td>U</td>
      <td>internalEnergy</td>
    </tr>
    <tr>
      <td>v</td>
      <td>electricPotential, specificVolume, velocity, voltage</td>
    </tr>
    <tr>
      <td>V</td>
      <td>volume</td>
    </tr>
    <tr>
      <td>w</td>
      <td>angularVelocity</td>
    </tr>
    <tr>
      <td>X</td>
      <td>reactance</td>
    </tr>
    <tr>
      <td>Z</td>
      <td>impedance</td>
    </tr>
</table>
</html>"));
        end Naming;
        annotation(
          Documentation(info = "<html>

<p>In this section the
<a href=\"modelica://Modelica.UsersGuide.Conventions.ModelicaCode.Naming\">naming conventions</a> of class and instance names, parameters and variables are specified. Additionally some
<a href=\"modelica://Modelica.UsersGuide.Conventions.ModelicaCode.Format\">format UsersGuide</a> are stated.</p>

</html>"));
      end ModelicaCode;

      package UsersGuide "User's Guide"
        extends Modelica.Icons.Information;

        class Implementation "Implementation notes"
          extends Modelica.Icons.Information;
          annotation(
            Documentation(info = "<html>
<p>
This class summarizes general information about the implementation which is not stated elsewhere.
</p>
<ol>
<li>The <code>&lt;caption&gt;</code> tag is currently not supported in some tools.</li>
<li>The <code>&amp;sim;</code> symbol (i.e., '&sim;' ) is currently not supported in some tools.</li>
<li>The <code>&amp;prop;</code> symbol (i.e., '&prop;' ) is currently not supported in some tools.</li>
</ol>
</html>"));
        end Implementation;

        class References "References"
          extends Modelica.Icons.References;
          annotation(
            Documentation(info = "<html>

<ol>
<li> Citation formats should be unified according to IEEE Transactions style.</li>
<li> Reference should be formatted as tables with two columns.</li>
</ol>

<p>In the following the reference formats will be explained based on five examples:</p>

<ul>
<li> Journal (or conference) [Gao2008] </li>
<li> Book [Andronov1973]</li>
<li> Master's thesis [Woehrnschimmel1998]</li>
<li> PhD thesis [Farnleitner1999]</li>
<li> Technical report [Marlino2005]</li>
</ul>

<p>The <a href=\"modelica://Modelica.UsersGuide.Conventions.Documentation.Format.References\">citation</a> is also explained.</p>

<h4>Example</h4>

<pre>
&lt;table border=\"0\" cellspacing=\"0\" cellpadding=\"2\"&gt;
    &lt;tr&gt;
      &lt;td valign=\"top\"&gt;[Gao2008]&lt;/td&gt;
      &lt;td valign=\"top\"&gt;Z. Gao, T. G. Habetler, R. G. Harley, and R. S. Colby,
        &quot;A sensorless  rotor temperature estimator for induction
                 machines based on a current harmonic spectral
                 estimation scheme,&quot;
        &lt;i&gt;IEEE Transactions on Industrial Electronics&lt;/i&gt;,
        vol. 55, no. 1, pp. 407-416, Jan. 2008.&lt;/td&gt;
    &lt;/tr&gt;
    &lt;tr&gt;
      &lt;td valign=\"top\"&gt;[Andronov1973]&lt;/td&gt;
      &lt;td valign=\"top\"&gt;A. Andronov, E. Leontovich, I. Gordon, and A. Maier,
        &lt;i&gt;Theory of  Bifurcations of Dynamic Systems on a plane&lt;/i&gt;,
        1st ed. New York: J. Wiley &amp; Sons, 1973.&lt;/td&gt;
    &lt;/tr&gt;
    &lt;tr&gt;
      &lt;td valign=\"top\"&gt;[Woehrnschimmel1998]&lt;/td&gt;
      &lt;td valign=\"top\"&gt;R. W&ouml;hrnschimmel,
        &quot;Simulation, modeling and fault detection for vector
              controlled induction machines,&quot;
        Master&#39;;s thesis, Vienna University of Technology,
        Vienna, Austria, 1998.&lt;/td&gt;
    &lt;/tr&gt;
    &lt;tr&gt;
      &lt;td valign=\"top\"&gt;[Farnleitner1999]&lt;/td&gt;
      &lt;td valign=\"top\"&gt;E. Farnleitner,
        &quot;Computational ?uid dynamics analysis for rotating
              electrical machinery,&quot;
        Ph.D. dissertation, University of Leoben,
        Department  of Applied Mathematics, Leoben, Austria, 1999.&lt;/td&gt;
    &lt;/tr&gt;
    &lt;tr&gt;
      &lt;td valign=\"top\"&gt;[Marlino2005]&lt;/td&gt;
      &lt;td valign=\"top\"&gt;L. D. Marlino,
        &quot;Oak ridge national laboratory annual progress report for the
              power electronics and electric machinery program,&quot;
      Oak Ridge National Laboratory, prepared for the U.S. Department of Energy,
      Tennessee, USA, Tech. Rep. FY2004 Progress Report, January 2005.&lt;/td&gt;
    &lt;/tr&gt;
&lt;/table&gt;</pre>

<p>appears as</p>

<table border=\"0\" cellspacing=\"0\" cellpadding=\"2\">
    <tr>
      <td valign=\"top\">[Gao08]</td>
      <td valign=\"top\">Z. Gao, T. G. Habetler, R. G. Harley, and R. S. Colby,
        &quot;A sensorless  rotor temperature estimator for induction
                 machines based on a current harmonic spectral
                 estimation scheme,&quot;
        <i>IEEE Transactions on Industrial Electronics</i>,
        vol. 55, no. 1, pp. 407-416, Jan. 2008.</td>
    </tr>
    <tr>
      <td valign=\"top\">[Andronov1973]</td>
      <td valign=\"top\">A. Andronov, E. Leontovich, I. Gordon, and A. Maier,
        <i>Theory of  Bifurcations of Dynamic Systems on a plane</i>,
        1st ed. New York: J. Wiley &amp; Sons, 1973.</td>
    </tr>
    <tr>
      <td valign=\"top\">[Woehrnschimmel1998]</td>
      <td valign=\"top\">R. W&ouml;hrnschimmel,
        &quot;Simulation, modeling and fault detection for vector
              controlled induction machines,&quot;
        Master&#39;;s thesis, Vienna University of Technology,
        Vienna, Austria, 1998.</td>
    </tr>
    <tr>
      <td valign=\"top\">[Farnleitner1999]</td>
      <td valign=\"top\">E. Farnleitner,
        &quot;Computational Fluid dynamics analysis for rotating
              electrical machinery,&quot;
        Ph.D. dissertation, University of Leoben,
        Department  of Applied Mathematics, Leoben, Austria, 1999.</td>
    </tr>
    <tr>
      <td valign=\"top\">[Marlino2005]</td>
      <td valign=\"top\">L. D. Marlino,
        &quot;Oak ridge national laboratory annual progress report for the
              power electronics and electric machinery program,&quot;
      Oak Ridge National Laboratory, prepared for the U.S. Department of Energy,
      Tennessee, USA, Tech. Rep. FY2004 Progress Report, January 2005.</td>
    </tr>
</table>

</html>"));
        end References;

        class Contact "Contact"
          extends Modelica.Icons.Contact;
          annotation(
            Documentation(info = "<html>

<p>
This class summarizes contact information of the contributing persons.
</p>

<h5>Example</h5>

<pre>
&lt;p&gt;This package is developed an maintained by the following contributors&lt;/p&gt;
  &lt;table border=\"1\" cellspacing=\"0\" cellpadding=\"2\">
    &lt;tr&gt;
      &lt;th&gt;&lt;/th&gt;
      &lt;th&gt;Name&lt;/th&gt;
      &lt;th&gt;Affiliation&lt;/th&gt;
    &lt;/tr&gt;
    &lt;tr&gt;
      &lt;td valign=\"top\"&gt;Library officer&lt;/td&gt;
      &lt;td valign=\"top\"&gt;
      &lt;a href=\"mailto:a.haumer@haumer.at\"&gt;A. Haumer&lt;/a&gt;
      &lt;/td&gt;
      &lt;td valign=\"top\"&gt;
        &lt;a href=\"http://www.haumer.at\"&gt;Technical Consulting &amp;amp; Electrical Engineering&lt;/a&gt;&lt;br&gt;
        3423 St.Andrae-Woerdern&lt;br&gt;
        Austria
      &lt;/td&gt;
    &lt;/tr&gt;
    &lt;tr&gt;
      &lt;td valign=\"top\"&gt;Contributor&lt;/td&gt;
      &lt;td valign=\"top\"&gt;
        &lt;a href=\"mailto:dr.christian.kral@gmail.com\"&gt;C. Kral&lt;/a&gt;
      &lt;/td&gt;
      &lt;td valign=\"top\"&gt;
        &lt;a href=\"http://christiankral.net\"&gt;Electric Machines, Drives and Systems&lt;/a&gt;&lt;br&gt;
        1060 Vienna&lt;br&gt;
        Austria
      &lt;/td&gt;
    &lt;/tr&gt;
  &lt;/table&gt;</pre>


<p>appears as</p>

<p>This package is developed an maintained by the following contributors</p>
  <table border=\"1\" cellspacing=\"0\" cellpadding=\"2\">
    <tr>
      <th></th>
      <th>Name</th>
      <th>Affiliation</th>
    </tr>
    <tr>
      <td valign=\"top\">Library officer</td>
      <td valign=\"top\">
      <a href=\"mailto:a.haumer@haumer.at\">A. Haumer</a>
      </td>
      <td valign=\"top\">
        <a href=\"http://www.haumer.at\">Technical Consulting &amp; Electrical Engineering</a><br>
        3423 St.Andrae-Woerdern<br>
        Austria
      </td>
    </tr>
    <tr>
      <td valign=\"top\">Contributor</td>
      <td valign=\"top\">
        <a href=\"mailto:dr.christian.kral@gmail.com\">C. Kral</a>
      </td>
      <td valign=\"top\">
        <a href=\"http://christiankral.net\">Electric Machines, Drives and Systems</a><br>
        1060 Vienna<br>
        Austria
      </td>
    </tr>
    <tr>
      <td valign=\"top\">Contributor</td>
      <td valign=\"top\">
        <a href=\"http://www.linkedin.com/in/dietmarw\">D. Winkler</a>
      </td>
      <td valign=\"top\">
        <a href=\"http://www.hit.no\">Telemark University College</a><br>
        PO box 203<br>
        3901 Porsgrunn<br>
        Norway
      </td>
    </tr>
  </table>

</html>"));
        end Contact;

        class RevisionHistory "Revision History"
          extends Modelica.Icons.ReleaseNotes;
          annotation(
            Documentation(info = "<html>

<ol>
<li> The revision history needs to answer the question:
     What has changed and what are the improvements over the previous versions and revision.</li>
<li> The revision history includes the documentation of the development history of each class and/or package.</li>
<li> Version number, revision number, date, author and comments shall be included.</li>
</ol>

<h5>Example</h5>

<pre>
&lt;table border=\"1\" cellspacing=\"0\" cellpadding=\"2\"&gt;
    &lt;tr&gt;
      &lt;th&gt;Version&lt;/th&gt;
      &lt;th&gt;Revision&lt;/th&gt;
      &lt;th&gt;Date&lt;/th&gt;
      &lt;th&gt;Author&lt;/th&gt;
      &lt;th&gt;Comment&lt;/th&gt;
    &lt;/tr&gt;
    ...
    &lt;tr&gt;
      &lt;td valign=\"top\"&gt;1.0.1&lt;/td&gt;
      &lt;td valign=\"top\"&gt;828&lt;/td&gt;
      &lt;td valign=\"top\"&gt;2008-05-26&lt;/td&gt;
      &lt;td valign=\"top\"&gt;A. Haumer&lt;br&gt;C. Kral&lt;/td&gt;
      &lt;td valign=\"top\"&gt;Fixed bug in documentation&lt;/td&gt;
    &lt;/tr&gt;
    &lt;tr&gt;
      &lt;td valign=\"top\"&gt;1.0.0&lt;/td&gt;
      &lt;td valign=\"top\"&gt;821&lt;/td&gt;
      &lt;td valign=\"top\"&gt;2008-05-21&lt;/td&gt;
      &lt;td valign=\"top\"&gt;A. Haumer&lt;/td&gt;
      &lt;td valign=\"top\"&gt;&lt;/td&gt;
    &lt;/tr&gt;
&lt;/table&gt;</pre>

<p>This code appears then as in the \"Revisions\" section below.</p>

</html>", revisions = "<html>
<table border=\"1\" cellspacing=\"0\" cellpadding=\"2\">
    <tr>
      <th>Version</th>
      <th>Revision</th>
      <th>Date</th>
      <th>Author</th>
      <th>Comment</th>
    </tr>
    <tr>
      <td valign=\"top\">1.1.0</td>
      <td valign=\"top\"></td>
      <td valign=\"top\">2010-04-22</td>
      <td valign=\"top\">C. Kral</td>
      <td valign=\"top\">Migrated Conventions to UsersGuide of MSL</td>
    </tr>
    <tr>
      <td valign=\"top\">1.0.5</td>
      <td valign=\"top\">3540</td>
      <td valign=\"top\">2010-03-11</td>
      <td valign=\"top\">D. Winkler</td>
      <td valign=\"top\">Updated image links guide to new 'modelica://' URIs, added contact details</td>
    </tr>
    <tr>
      <td valign=\"top\">1.0.4</td>
      <td valign=\"top\">2960</td>
      <td valign=\"top\">2009-09-28</td>
      <td valign=\"top\">C. Kral</td>
      <td valign=\"top\">Applied new rules for equations as discussed on the 63rd Modelica Design Meeting</td>
    </tr>
    <tr>
      <td valign=\"top\">1.0.3</td>
      <td valign=\"top\">2579</td>
      <td valign=\"top\">2008-05-26</td>
      <td valign=\"top\">D. Winkler</td>
      <td valign=\"top\">Layout fixes and enhancements</td>
    </tr>
    <tr>
      <td valign=\"top\">1.0.1</td>
      <td valign=\"top\">828</td>
      <td valign=\"top\">2008-05-26</td>
      <td valign=\"top\">A. Haumer<br>C. Kral</td>
      <td valign=\"top\">Fixed bug in documentation</td>
    </tr>
    <tr>
      <td valign=\"top\">1.0.0</td>
      <td valign=\"top\">821</td>
      <td valign=\"top\">2008-05-21</td>
      <td valign=\"top\">A. Haumer</td>
      <td valign=\"top\"></td>
    </tr>
</table>
</html>"));
        end RevisionHistory;
        annotation(
          Documentation(info = "<html>
<p>The UsersGuide of each package should consist of the following classes</p>
<ol>
<li> <a href=\"modelica://Modelica.UsersGuide.Conventions.UsersGuide.Contact\">Contact</a> information of
     the library officer and the co-authors </li>
<li> Optional <a href=\"modelica://Modelica.UsersGuide.Conventions.UsersGuide.Implementation\">Implementation Notes</a> to give general information about the implementation
<li> <a href=\"modelica://Modelica.UsersGuide.Conventions.UsersGuide.References\">References</a> for summarizing the literature of the package</li>
<li> <a href=\"modelica://Modelica.UsersGuide.Conventions.UsersGuide.RevisionHistory\">Revision history </a> to summarize the most important changes and improvements of the package</li>
</ol>
</html>"));
      end UsersGuide;
      annotation(
        DocumentationClass = true,
        Documentation(info = "<html>
<p>A Modelica main package should be compliant with the UsersGuide stated in this documentation:</p>
<ol>
<li> Conventions of the <a href=\"modelica://Modelica.UsersGuide.Conventions.ModelicaCode\">Modelica code</a> </li>
<li> Consistent HTML documentation <a href=\"modelica://Modelica.UsersGuide.Conventions.Documentation\">UsersGuide</a> </li>
<li> Structure to be provided by a main package
<ul>
     <li> <a href=\"modelica://Modelica.UsersGuide.Conventions.UsersGuide\">User's Guide</a></li>
     <li> <b>Examples</b> containing models demonstrating the usage of the library.</li>
     <li> <b>Components</b> -- in case of a complex library a more detailed structure can be established.</li>
     <li> <b>Sensors</b></li>
     <li> <b>Sources</b></li>
     <li> <b>Interfaces</b> containing connectors and partial models.</li>
     <li> <b>Types</b> containing type, enumeration and choice definitions.</li>
</ul></li>
<li> These packages should appear in the listed order.</li>
</ol>
</html>"));
    end Conventions;

    class ParameterDefaults "Parameter defaults"
      extends Modelica.Icons.Information;
      annotation(
        Documentation(info = "<html>

<p>
In this section the convention is summarized how default parameters are
handled in the Modelica Standard Library (since version 3.0).
</p>

<p>
Many models in this library have parameter declarations to define
constants of a model that might be changed before simulation starts.
Example:
</p>

<blockquote>
<pre>
<b>model</b> SpringDamper
<b>parameter</b> Real c(final unit=\"N.m/rad\")    = 1e5 \"Spring constant\";
<b>parameter</b> Real d(final unit=\"N.m.s/rad\")  = 0   \"Damping constant\";
<b>parameter</b> Modelica.SIunits.Angle phi_rel0 = 0   \"Unstretched spring angle\";
...
<b>end</b> SpringDamper;
</pre>
</blockquote>

<p>
In Modelica it is possible to define a default value of a parameter in
the parameter declaration. In the example above, this is performed for
all parameters. Providing default values for all parameters can lead to
errors that are difficult to detect, since a modeler may have forgotten
to provide a meaningful value (the model simulates but gives wrong
results due to wrong parameter values). In general the following basic
situations are present:
</p>

<ol>
<li> The parameter value could be anything (e.g., a spring constant or
  a resistance value) and therefore the user should provide a value in
  all cases. A Modelica translator should warn, if no value is provided.
</li>

<li> The parameter value is not changed in &gt; 95 % of the cases
  (e.g., initialization or visualization parameters, or parameter phi_rel0
  in the example above). In this case a default parameter value should be
  provided, in order that the model or function can be conveniently
  used by a modeler.
</li>

<li> A modeler would like to quickly utilize a model, e.g.,
  <ul>
  <li> to automatically check that the model still translates and/or simulates
    (after some changes in the library),</li>
  <li> to make a quick demo of a library by drag-and-drop of components,</li>
  <li> to implement a simple test model in order to get a better understanding
    of the desired component.</li>
  </ul>
  In all these cases, it would be not practical, if the modeler would
  have to provide explicit values for all parameters first.
  </li>
</ol>

<p>
To handle the conflicting goals of (1) and (3), the Modelica Standard Library
uses two approaches to define default parameters, as demonstrated with the
following example:
</p>

<blockquote>
<pre>
<b>model</b> SpringDamper
<b>parameter</b> Real c(final unit=\"N.m/rad\"  , start=1e5) \"Spring constant\";
<b>parameter</b> Real d(final unit=\"N.m.s/rad\", start=  0) \"Damping constant\";
<b>parameter</b> Modelica.SIunits.Angle phi_rel0 = 0       \"Unstretched spring angle\";
...
<b>end</b> SpringDamper;

SpringDamper sp1;              // warning for \"c\" and \"d\"
SpringDamper sp2(c=1e4, d=0);  // fine, no warning
</pre>
</blockquote>

<p>
Both definition forms, using a \"start\" value (for \"c\" and \"d\") and providing
a declaration equation (for \"phi_rel0\"), are valid Modelica and define the value
of the parameter. By convention, it is expected that Modelica translators will
trigger a warning message for parameters that are <b>not</b> defined by a declaration
equation, by a modifier equation or in an initial equation/algorithm section.
A Modelica translator might have options to change this behavior, especially,
that no messages are printed in such cases and/or that an error is triggered
instead of a warning.
</p>

</html>"));
    end ParameterDefaults;

    class ModelicaLicense2 "Modelica License 2"
      extends Modelica.Icons.Information;
      annotation(
        Documentation(info = "<html>
<head>
<title>The Modelica License 2</title>
<style type=\"text/css\">
*       { font-size: 10pt; font-family: Arial,sans-serif; }
code    { font-size:  9pt; font-family: Courier,monospace;}
h6      { font-size: 10pt; font-weight: bold; color: green; }
h5      { font-size: 11pt; font-weight: bold; color: green; }
h4      { font-size: 13pt; font-weight: bold; color: green; }
address {                  font-weight: normal}
td      { solid #000; vertical-align:top; }
th      { solid #000; vertical-align:top; font-weight: bold; }
table   { solid #000; border-collapse: collapse;}
</style>
</head>

<p>All files in this directory (Modelica) and in all
subdirectories, especially all files that build package \"Modelica\" and all
files in \"Modelica/Resources/*\" and \"Modelica/help\" are licensed by the <b><u>Modelica Association</u></b> under the <b><u>Modelica License 2</u></b> (with exception of files
\"Modelica/Resources/C-Sources/win32_dirent.*, ModelicaMatIO.*, ModelicaStandardTables.*\").</p>

<p style=\"margin-left: 40px;\"><b>Licensor:</b><br>
Modelica Association<br>
(Ideella F&ouml;reningar 822003-8858 in Link&ouml;ping) <br>
c/o PELAB, IDA, Link&ouml;pings Universitet <br>
S-58183 Link&ouml;ping <br>
Sweden<br>
email: Board@Modelica.org<br>
web: <a href=\"https://www.Modelica.org\">https://www.Modelica.org</a></p>

<p style=\"margin-left: 40px;\"><b>Copyright notices of the files:</b><br/>
Copyright &copy; 1998-2015,
ABB, Austrian Institute of Technology, T.&nbsp;B&ouml;drich, DLR, Dassault Syst&egrave;mes AB, Fraunhofer, A.&nbsp;Haumer, ITI, Modelon,
TU Hamburg-Harburg, Politecnico di Milano, XRG Simulation.
</p>

<p>
<a href=\"#The_Modelica_License_2-outline\">The Modelica License 2</a><br>
<a href=\"#Frequently_Asked_Questions-outline\">Frequently Asked Questions</a><br>
</p>

<hr>

<h4><a name=\"The_Modelica_License_2-outline\"></a>The Modelica License 2</h4>

<p>
<b>Preamble.</b> The goal of this license is that Modelica related
model libraries, software, images, documents, data files etc. can be
used freely in the original or a modified form, in open source and in
commercial environments (as long as the license conditions below are
fulfilled, in particular sections 2c) and 2d). The Original Work is
provided free of charge and the use is completely at your own risk.
Developers of free Modelica packages are encouraged to utilize this
license for their work.</p>

<p>
The Modelica License applies to any Original Work that contains the
following licensing notice adjacent to the copyright notice(s) for
this Original Work:</p>

<p><b>Licensed by the Modelica Association under the Modelica License 2</b></p>

<p><b>1. Definitions.</b></p>
<ol type=\"a\">
  <li>&ldquo;License&rdquo; is this Modelica License.</li>

  <li>&ldquo;Original Work&rdquo; is any work of authorship, including
  software, images, documents, data files, that contains the above
  licensing notice or that is packed together with a licensing notice
  referencing it.</li>

  <li>&ldquo;Licensor&rdquo; is the provider of the Original Work who has
  placed this licensing notice adjacent to the copyright notice(s) for
  the Original Work. The Original Work is either directly provided by
  the owner of the Original Work, or by a licensee of the owner.</li>

  <li>&ldquo;Derivative Work&rdquo; is any modification of the Original
  Work which represents, as a whole, an original work of authorship.
  For the matter of clarity and as examples:

  <ol  type=\"A\">
    <li>Derivative Work shall not include work that remains separable from
    the Original Work, as well as merely extracting a part of the
    Original Work without modifying it.</li>

    <li>Derivative Work shall not include (a) fixing of errors and/or (b)
    adding vendor specific Modelica annotations and/or (c) using a
    subset of the classes of a Modelica package, and/or (d) using a
    different representation, e.g., a binary representation.</li>

    <li>Derivative Work shall include classes that are copied from the
    Original Work where declarations, equations or the documentation
    are modified.</li>

    <li>Derivative Work shall include executables to simulate the models
    that are generated by a Modelica translator based on the Original
    Work (of a Modelica package).</li>
  </ol>

  <li>&ldquo;Modified Work&rdquo; is any modification of the Original Work
  with the following exceptions: (a) fixing of errors and/or (b)
  adding vendor specific Modelica annotations and/or (c) using a
  subset of the classes of a Modelica package, and/or (d) using a
  different representation, e.g., a binary representation.</li>

  <li>&quot;Source Code&quot; means the preferred form of the Original
  Work for making modifications to it and all available documentation
  describing how to modify the Original Work.</li>

  <li>&ldquo;You&rdquo; means an individual or a legal entity exercising
  rights under, and complying with all of the terms of, this License.</li>

  <li>&ldquo;Modelica package&rdquo; means any Modelica library that is
  defined with the &ldquo;<code><b>package</b>&nbsp;&lt;Name&gt;&nbsp;...&nbsp;<b>end</b>&nbsp;&lt;Name&gt;;</code>&rdquo; Modelica language element.</li>

</ol>

<p>
<b>2. Grant of Copyright License.</b> Licensor grants You a
worldwide, royalty-free, non-exclusive, sublicensable license, for
the duration of the copyright, to do the following:</p>

<ol type=\"a\">
  <li><p>
  To reproduce the Original Work in copies, either alone or as part of
  a collection.</p></li>
  <li><p>
  To create Derivative Works according to Section 1d) of this License.</p></li>
  <li><p>
  To distribute or communicate to the public copies of the <u>Original
  Work</u> or a <u>Derivative Work</u> under <u>this License</u>. No
  fee, neither as a copyright-license fee, nor as a selling fee for
  the copy as such may be charged under this License. Furthermore, a
  verbatim copy of this License must be included in any copy of the
  Original Work or a Derivative Work under this License.<br>
  For the matter of clarity, it is permitted A) to distribute or
  communicate such copies as part of a (possible commercial)
  collection where other parts are provided under different licenses
  and a license fee is charged for the other parts only and B) to
  charge for mere printing and shipping costs.</p></li>
  <li><p>
  To distribute or communicate to the public copies of a <u>Derivative
  Work</u>, alternatively to Section 2c), under <u>any other license</u>
  of your choice, especially also under a license for
  commercial/proprietary software, as long as You comply with Sections
  3, 4 and 8 below. <br>      For the matter of clarity, no
  restrictions regarding fees, either as to a copyright-license fee or
  as to a selling fee for the copy as such apply.</p></li>
  <li><p>
  To perform the Original Work publicly.</p></li>
  <li><p>
  To display the Original Work publicly.</p></li>
</ol>

<p>
<b>3. Acceptance.</b> Any use of the Original Work or a
Derivative Work, or any action according to either Section 2a) to 2f)
above constitutes Your acceptance of this License.</p>

<p>
<b>4. Designation of Derivative Works and of Modified Works.
</b>The identifying designation of Derivative Work and of Modified
Work must be different to the corresponding identifying designation
of the Original Work. This means especially that the (root-level)
name of a Modelica package under this license must be changed if the
package is modified (besides fixing of errors, adding vendor specific
Modelica annotations, using a subset of the classes of a Modelica
package, or using another representation, e.g. a binary
representation).</p>

<p>
<b>5. Grant of Patent License.</b>
Licensor grants You a worldwide, royalty-free, non-exclusive, sublicensable license,
under patent claims owned by the Licensor or licensed to the Licensor by
the owners of the Original Work that are embodied in the Original Work
as furnished by the Licensor, for the duration of the patents,
to make, use, sell, offer for sale, have made, and import the Original Work
and Derivative Works under the conditions as given in Section 2.
For the matter of clarity, the license regarding Derivative Works covers
patent claims to the extent as they are embodied in the Original Work only.</p>

<p>
<b>6. Provision of Source Code.</b> Licensor agrees to provide
You with a copy of the Source Code of the Original Work but reserves
the right to decide freely on the manner of how the Original Work is
provided.<br>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;For the matter of clarity, Licensor might provide only a binary
representation of the Original Work. In that case, You may (a) either
reproduce the Source Code from the binary representation if this is
possible (e.g., by performing a copy of an encrypted Modelica
package, if encryption allows the copy operation) or (b) request the
Source Code from the Licensor who will provide it to You.</p>

<p>
<b>7. Exclusions from License Grant.</b> Neither the names of
Licensor, nor the names of any contributors to the Original Work, nor
any of their trademarks or service marks, may be used to endorse or
promote products derived from this Original Work without express
prior permission of the Licensor. Except as otherwise expressly
stated in this License and in particular in Sections 2 and 5, nothing
in this License grants any license to Licensor&rsquo;s trademarks,
copyrights, patents, trade secrets or any other intellectual
property, and no patent license is granted to make, use, sell, offer
for sale, have made, or import embodiments of any patent claims.<br>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;No license is granted to the trademarks of
Licensor even if such trademarks are included in the Original Work,
except as expressly stated in this License. Nothing in this License
shall be interpreted to prohibit Licensor from licensing under terms
different from this License any Original Work that Licensor otherwise
would have a right to license.</p>

<p>
<b>8. Attribution Rights.</b> You must retain in the Source
Code of the Original Work and of any Derivative Works that You
create, all author, copyright, patent, or trademark notices, as well
as any descriptive text identified therein as an &quot;Attribution
Notice&quot;. The same applies to the licensing notice of this
License in the Original Work. For the matter of clarity, &ldquo;author
notice&rdquo; means the notice that identifies the original
author(s). <br>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;You must cause the Source Code for any Derivative
Works that You create to carry a prominent Attribution Notice
reasonably calculated to inform recipients that You have modified the
Original Work. <br>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;In case the Original Work or Derivative Work is not provided in
Source Code, the Attribution Notices shall be appropriately
displayed, e.g., in the documentation of the Derivative Work.</p>

<p><b>9. Disclaimer
of Warranty. <br></b><u><b>The Original Work is provided under this
License on an &quot;as is&quot; basis and without warranty, either
express or implied, including, without limitation, the warranties of
non-infringement, merchantability or fitness for a particular
purpose. The entire risk as to the quality of the Original Work is
with You.</b></u> This disclaimer of warranty constitutes an
essential part of this License. No license to the Original Work is
granted by this License except under this disclaimer.</p>

<p>
<b>10. Limitation of Liability.</b> Under no circumstances and
under no legal theory, whether in tort (including negligence),
contract, or otherwise, shall the Licensor, the owner or a licensee
of the Original Work be liable to anyone for any direct, indirect,
general, special, incidental, or consequential damages of any
character arising as a result of this License or the use of the
Original Work including, without limitation, damages for loss of
goodwill, work stoppage, computer failure or malfunction, or any and
all other commercial damages or losses. This limitation of liability
shall not apply to the extent applicable law prohibits such
limitation.</p>

<p>
<b>11. Termination.</b> This License conditions your rights to
undertake the activities listed in Section 2 and 5, including your
right to create Derivative Works based upon the Original Work, and
doing so without observing these terms and conditions is prohibited
by copyright law and international treaty. Nothing in this License is
intended to affect copyright exceptions and limitations. This License
shall terminate immediately and You may no longer exercise any of the
rights granted to You by this License upon your failure to observe
the conditions of this license.</p>

<p>
<b>12. Termination for Patent Action.</b> This License shall
terminate automatically and You may no longer exercise any of the
rights granted to You by this License as of the date You commence an
action, including a cross-claim or counterclaim, against Licensor,
any owners of the Original Work or any licensee alleging that the
Original Work infringes a patent. This termination provision shall
not apply for an action alleging patent infringement through
combinations of the Original Work under combination with other
software or hardware.</p>

<p>
<b>13. Jurisdiction.</b> Any action or suit relating to this
License may be brought only in the courts of a jurisdiction wherein
the Licensor resides and under the laws of that jurisdiction
excluding its conflict-of-law provisions. The application of the
United Nations Convention on Contracts for the International Sale of
Goods is expressly excluded. Any use of the Original Work outside the
scope of this License or after its termination shall be subject to
the requirements and penalties of copyright or patent law in the
appropriate jurisdiction. This section shall survive the termination
of this License.</p>

<p>
<b>14. Attorneys&rsquo; Fees.</b> In any action to enforce the
terms of this License or seeking damages relating thereto, the
prevailing party shall be entitled to recover its costs and expenses,
including, without limitation, reasonable attorneys' fees and costs
incurred in connection with such action, including any appeal of such
action. This section shall survive the termination of this License.</p>

<p>
<b>15. Miscellaneous.</b>
</p>
<ol type=\"a\">
  <li>If any
  provision of this License is held to be unenforceable, such
  provision shall be reformed only to the extent necessary to make it
  enforceable.</li>

  <li>No verbal
  ancillary agreements have been made. Changes and additions to this
  License must appear in writing to be valid. This also applies to
  changing the clause pertaining to written form.</li>

  <li>You may use the
  Original Work in all ways not otherwise restricted or conditioned by
  this License or by law, and Licensor promises not to interfere with
  or be responsible for such uses by You.</li>
</ol>

<hr>

<h5><a name=\"Frequently_Asked_Questions-outline\"></a>
Frequently Asked Questions</h5>
<p>This
section contains questions/answer to users and/or distributors of
Modelica packages and/or documents under Modelica License 2. Note,
the answers to the questions below are not a legal interpretation of
the Modelica License 2. In case of a conflict, the language of the
license shall prevail.</p>

<h6>Using or Distributing a Modelica <u>Package</u> under the Modelica License 2</h6>

<p><b>What are the main
differences to the previous version of the Modelica License?</b></p>
<ol>
  <li><p>
  Modelica License 1 is unclear whether the licensed Modelica package
  can be distributed under a different license. Version 2 explicitly
  allows that &ldquo;Derivative Work&rdquo; can be distributed under
  any license of Your choice, see examples in Section 1d) as to what
  qualifies as Derivative Work (so, version 2 is clearer).</p>
  <li><p>
  If You modify a Modelica package under Modelica License 2 (besides
  fixing of errors, adding vendor specific Modelica annotations, using
  a subset of the classes of a Modelica package, or using another
  representation, e.g., a binary representation), you must rename the
  root-level name of the package for your distribution. In version 1
  you could keep the name (so, version 2 is more restrictive). The
  reason of this restriction is to reduce the risk that Modelica
  packages are available that have identical names, but different
  functionality.</p>
  <li><p>
  Modelica License 1 states that &ldquo;It is not allowed to charge a
  fee for the original version or a modified version of the software,
  besides a reasonable fee for distribution and support&rdquo;.
  Version 2 has a similar intention for all Original Work under
  <u>Modelica License 2</u> (to remain free of charge and open source)
  but states this more clearly as &ldquo;No fee, neither as a
  copyright-license fee, nor as a selling fee for the copy as such may
  be charged&rdquo;. Contrary to version 1, Modelica License 2 has no
  restrictions on fees for Derivative Work that is provided under a
  different license (so, version 2 is clearer and has fewer
  restrictions).</p>
  <li><p>
  Modelica License 2 introduces several useful provisions for the
  licensee (articles 5, 6, 12), and for the licensor (articles 7, 12,
  13, 14) that have no counter part in version 1.</p>
  <li><p>
  Modelica License 2 can be applied to all type of work, including
  documents, images and data files, contrary to version 1 that was
  dedicated for software only (so, version 2 is more general).</p>
</ol>

<p><b>Can I distribute a
Modelica package (under Modelica License 2) as part of my commercial
Modelica modeling and simulation environment?</b></p>
<p>Yes,
according to Section 2c). However, you are not allowed to charge a
fee for this part of your environment. Of course, you can charge for
your part of the environment.
</p>

<p><b>Can I distribute a
Modelica package (under Modelica License 2) under a different
license?</b></p>
<p>No.
The license of an unmodified Modelica package cannot be changed
according to Sections 2c) and 2d). This means that you cannot <u>sell</u>
copies of it, any distribution has to be free of charge.</p>

<p><b>Can I distribute a
Modelica package (under Modelica License 2) under a different license
when I first encrypt the package?</b></p>
<p>No.
Merely encrypting a package does not qualify for Derivative Work and
therefore the encrypted package has to stay under Modelica License 2.</p>

<p><b>Can I distribute a
Modelica package (under Modelica License 2) under a different license
when I first add classes to the package?</b></p>
<p>No.
The package itself remains unmodified, i.e., it is Original Work, and
therefore the license for this part must remain under Modelica
License 2. The newly added classes can be, however, under a different
license.
</p>

<p><b>Can
I copy a class out of a Modelica package (under Modelica License 2)
and include it </b><u><b>unmodified</b></u><b> in a Modelica package
under a </b><u><b>commercial/proprietary license</b></u><b>?</b></p>
<p>No,
according to article 2c). However, you can include model, block,
function, package, record and connector classes in your Modelica
package under <u>Modelica License 2</u>. This means that your
Modelica package could be under a commercial/proprietary license, but
one or more classes of it are under Modelica License 2.<br>Note, a
&ldquo;type&rdquo; class (e.g., type Angle = Real(unit=&rdquo;rad&rdquo;))
can be copied and included unmodified under a commercial/proprietary
license (for details, see the next question).</p>

<p><b>Can
I copy a type class or </b><u><b>part</b></u><b> of a model, block,
function, record, connector class, out of a Modelica package (under
Modelica License 2) and include it modified or unmodified in a
Modelica package under a </b><u><b>commercial/proprietary</b></u><b>
license</b></p>
<p>Yes,
according to article 2d), since this will in the end usually qualify
as Derivative Work. The reasoning is the following: A type class or
part of another class (e.g., an equation, a declaration, part of a
class description) cannot be utilized &ldquo;by its own&rdquo;. In
order to make this &ldquo;usable&rdquo;, you have to add additional
code in order that the class can be utilized. This is therefore
usually Derivative Work and Derivative Work can be provided under a
different license. Note, this only holds, if the additional code
introduced is sufficient to qualify for Derivative Work. Merely, just
copying a class and changing, say, one character in the documentation
of this class would be no Derivative Work and therefore the copied
code would have to stay under Modelica License 2.</p>

<p><b>Can
I copy a class out of a Modelica package (under Modelica License 2)
and include it in </b><u><b>modified </b></u><b>form in a
</b><u><b>commercial/proprietary</b></u><b> Modelica package?</b></p>
<p>Yes.
If the modification can be seen as a &ldquo;Derivative Work&rdquo;,
you can place it under your commercial/proprietary license. If the
modification does not qualify as &ldquo;Derivative Work&rdquo; (e.g.,
bug fixes, vendor specific annotations), it must remain under
Modelica License 2. This means that your Modelica package could be
under a commercial/proprietary license, but one or more parts of it
are under Modelica License 2.</p>

<p><b>Can I distribute a
&ldquo;save total model&rdquo; under my commercial/proprietary
license, even if classes under Modelica License 2 are included?</b></p>
<p>Your
classes of the &ldquo;save total model&rdquo; can be distributed
under your commercial/proprietary license, but the classes under
Modelica License 2 must remain under Modelica License 2. This means
you can distribute a &ldquo;save total model&rdquo;, but some parts
might be under Modelica License 2.</p>

<p><b>Can I distribute a
Modelica package (under Modelica License 2) in encrypted form?</b></p>
<p>Yes.
Note, if the encryption does not allow &ldquo;copying&rdquo; of
classes (in to unencrypted Modelica source code), you have to send
the Modelica source code of this package to your customer, if he/she
wishes it, according to article&nbsp;6.</p>

<p><b>Can I distribute an
executable under my commercial/proprietary license, if the model from
which the executable is generated uses models from a Modelica package
under Modelica License 2?</b></p>
<p>Yes,
according to article 2d), since this is seen as Derivative Work. The
reasoning is the following: An executable allows the simulation of a
concrete model, whereas models from a Modelica package (without
pre-processing, translation, tool run-time library) are not able to
be simulated without tool support. By the processing of the tool and
by its run-time libraries, significant new functionality is added (a
model can be simulated whereas previously it could not be simulated)
and functionality available in the package is removed (e.g., to build
up a new model by dragging components of the package is no longer
possible with the executable).</p>

<p><b>Is my modification to
a Modelica package (under Modelica License 2) a Derivative Work?</b></p>
<p>It
is not possible to give a general answer to it. To be regarded as &quot;an
original work of authorship&quot;, a derivative work must be
different enough from the original or must contain a substantial
amount of new material. Making minor changes or additions of little
substance to a preexisting work will not qualify the work as a new
version for such purposes.
</p>

<h6>Using or Distributing a Modelica <u>Document</u> under the Modelica License 2</h6>

<p>This
section is devoted especially for the following applications:</p>
<ol type=\"a\">
  <li><p>
  A Modelica tool extracts information out of a Modelica package and
  presents the result in form of a &ldquo;manual&rdquo; for this
  package in, e.g., html, doc, or pdf format.</p>
  <li><p>
  The Modelica language specification is a document defining the
  Modelica language. It will be licensed under Modelica License 2.</p>
  <li><p>
  Someone writes a book about the Modelica language and/or Modelica
  packages and uses information which is available in the Modelica
  language specification and/or the corresponding Modelica package.</p>
</ol>

<p><b>Can I sell a manual
that was basically derived by extracting information automatically
from a Modelica package under Modelica License 2 (e.g., a &ldquo;reference
guide&rdquo; of the Modelica Standard Library):</b></p>
<p>Yes.
Extracting information from a Modelica package, and providing it in a
human readable, suitable format, like html, doc or pdf format, where
the content is significantly modified (e.g. tables with interface
information are constructed from the declarations of the public
variables) qualifies as Derivative Work and there are no restrictions
to charge a fee for Derivative Work under alternative 2d).</p>

<p><b>Can
I copy a text passage out of a Modelica document (under Modelica
License 2) and use it </b><u><b>unmodified</b></u><b> in my document
(e.g. the Modelica syntax description in the Modelica Specification)?</b></p>
<p>Yes.
In case you distribute your document, the copied parts are still
under Modelica License 2 and you are not allowed to charge a license
fee for this part. You can, of course, charge a fee for the rest of
your document.</p>

<p><b>Can
I copy a text passage out of a Modelica document (under Modelica
License 2) and use it in </b><u><b>modified</b></u><b> form in my
document?</b></p>
<p>Yes,
the creation of Derivative Works is allowed. In case the content is
significantly modified this qualifies as Derivative Work and there
are no restrictions to charge a fee for Derivative Work under
alternative 2d).</p>

<p><b>Can I sell a printed
version of a Modelica document (under Modelica License 2), e.g., the
Modelica Language Specification?</b></p>
<p>No,
if you are not the copyright-holder, since article 2c) does not allow
a selling fee for a (in this case physical) copy. However, mere
printing and shipping costs may be recovered.</p>
</html>"));
    end ModelicaLicense2;

    package ReleaseNotes "Release notes"
      extends Modelica.Icons.ReleaseNotes;

      class VersionManagement "Version Management"
        extends Modelica.Icons.ReleaseNotes;
        annotation(
          Documentation(info = "<html>
<p>
Maintenance of the Modelica Standard Library is performed with
three branches on the subversion server of the Modelica Association
(<a href=\"https://svn.modelica.org/projects/Modelica\">https://svn.modelica.org/projects/Modelica</a>):
</p>

<h4>Released branch</h4>
<p>
Example: \"/tags/v3.0.1/Modelica\"
</p>

<p>
This branch contains the released Modelica versions (e.g., version 3.0.1),
where all available test cases and compatibility checks with other Modelica
libraries have been performed on the respective release. This version is
usually shipped with a Modelica modeling and simulation environment and
utilized by a Modelica user.
</p>

<h4>Development branch</h4>
<p>
Example: \"/trunk/Modelica\"
</p>

<p>
This branch contains the actual development version, i.e., all bug fixes
and new features based on the last Modelica release.
New features should have been tested before including them.
However, the exhaustive tests for a new version are (usually) not performed.
This version is usually only be used by the developers of the
Modelica Standard Library and is not utilized by Modelica users.
</p>

<h4>Maintenance branch</h4>
<p>
Example: \"/branches/maintenance/3.0.1/Modelica\"
</p>

<p>
This branch contains the released Modelica version (e.g., version 3.0.1)
where all bug fixes since this release date are included (up to a  new release,
when becoming available; i.e., after a new release, the previous maintenance
versions are no longer changed).
These bug fixes might be not yet tested with all test cases or with
other Modelica libraries. The goal is that a vendor may take this version at
any time for a new release of its software, in order to incorporate the latest
bug fixes, without changing the version number of the Modelica Standard Library.
</p>

<p>
Incorporation of bug fixes (subversion \"commit\") shall be performed in the following way:
</p>

<ul>
<li> One person is fixing the bug and another person is checking whether the
         fix is fine.</li>
<li> It is up to the library developer, whether he opens a new branch for
         testing and then merges it with the \"head\" maintenance branch or not.</li>
<li> Every change to the maintenance branch has to be done at the development
         branch (see above) as well. One exception are pure changes to the
         \"versionBuild\" annotation as these have no meaning in the development trunk.</li>
<li> Every change to the maintenance branch requires introducing a
         description of the bug fix under
         Modelica.UsersGuide.ReleaseNotes.Version_&lt;release-number&gt;_BugFixes.</li>
<li> Annotations \"version\" and \"versionDate\" must <u>not</u> be changed in a maintenance release.</li>
<li> Every change to the maintenance branch requires changing the \"versionBuild\" number (incrementing it by one),
     as well as the \"dateModified\" field.<br>
     Example:
         <pre>  annotation(version      = \"3.1\",
             versionDate  = \"2009-06-22\",
             versionBuild = 3,
             dateModified = \"2009-08-28 07:40:19Z\",
             revisionId   = \"$I&#8203;d::                                       $\")</pre>
     The \"revisionId\" field is a bit special though. If written like in the example above it will be automatically
     expanded to:
        <pre>             revisionId   = \"$I&#8203;d:: package.mo 2879 2009-08-28 07:40:19Z #$\"</pre>
     by the subversion checkout procedure.</li>
<li> If time does not permit, a vendor makes the bug fix in its local version
         and then has to include it in the maintenance version. It would be best to make these
         changes at a new branch in order to get a unique release number.</li>
</ul>

<p>
A valid \"commit\" to the maintenance branch may contain one or
more of the following changes.
</p>

<ul>
<li> Correcting an equation.</li>
<li> Correcting attributes quantity/unit/defaultUnit in a declaration.</li>
<li> Improving/fixing the documentation.</li>
<li> Introducing a new name in the public section of a class
         (model, package, ...) or in any section of a partial class is <b>not</b> allowed.
         Since otherwise, a user might use this new name and when storing its model
         and loading it with an older build-version, an error would occur.</li>
<li> Introducing a new name in the protected section of a non-partial
         class should only be done if absolutely necessary to fix a bug.
         The problem is that this might be non-backward compatible,
         because a user might already extend from this class and already using the same name.</li>
</ul>
</html>"));
      end VersionManagement;

      class Version_3_2_1_build_4 "Version 3.2.1 build 4 (Sept. 30, 2015)"
        extends Modelica.Icons.ReleaseNotes;
        annotation(
          Documentation(info = "<html>
<p>
Version 3.2.1 build 4 is backward compatible to version 3.2.1 build 3,
that is models developed with versions 3.0, 3.0.1, 3.1, 3.2 or 3.2.1 will
work without any changes also with version 3.2.1 build 4
(see also issue (4) below, ticket <a href=\"https://trac.modelica.org/Modelica/ticket/1757\">1757</a>).
This version is a bug-fix release. Short Overview:
</p>

<ol>
<li> This version of the Modelica package is <b>fully compatible</b> to
     Modelica Specification <b>3.2 revision 2</b>
     (with exception of issue (3) reported below and in
      ticket <a href=\"https://trac.modelica.org/Modelica/ticket/1758\">1758</a>).
     </li>

<li> About <a href=\"modelica://Modelica/Resources/Documentation/Version-3.2.1/ResolvedTracTickets-build-4.html\">10 tickets</a>
     have been fixed for this release. In particular the following <b>critical bug</b> has been fixed:<br>
     Ticket <a href=\"https://trac.modelica.org/Modelica/ticket/1768\">1768</a>
     fixes an issue with block <a href=\"modelica://Modelica.Blocks.Sources.CombiTimeTable\">CombiTimeTable</a>
     (wrong output when using fixed time step integrator with time step greater than table resolution).
      </li>

<li> Ticket <a href=\"https://trac.modelica.org/Modelica/ticket/1758\">1758</a>
     states that simulation of
     <a href=\"modelica://Modelica.Fluid.Examples.HeatingSystem\">Modelica.Fluid.Examples.HeatingSystem</a>
     fails in Dymola 2016 if option \"pedantic mode for checking Modelica semantics\" is set.
     This issue was not fixed in the library due to the following reasons:<br>
     The Modelica.Fluid library uses a particular pattern to define some parameters resulting
     in a cyclic dependency of parameters if only incident information is taken into account.
     According to Modelica Specification 3.2 revision 2 this is not allowed
     (and therefore Dymola 2016 correctly reports errors if the pedantic flag is set).
     In ticket <a href=\"https://trac.modelica.org/Modelica/ticket/1320\">1320</a>
     this issue was resolved for Modelica Specification 3.3 revision 1 by allowing
     cyclic parameter definitions if the cycles disappear when evaluating parameters
     that have annotation Evaluate=true. Modelica.Fluid is correct with respect
     to Modelica Specification 3.3 revision 1.
     Changing the Modelica.Fluid library for build 4 so that no cyclic parameter dependencies
     would be present anymore would (a) result in a non-backwards compatible
     change and (b) make the usage of Modelica.Fluid less convenient. For this
     reason Modelica.Fluid is not changed. (Practically, this means for example that
     the pedantic flag in Dymola 2016 needs to be switched off, when using the
     Modelica.Fluid library in version 3.2.1 build 4 and any previous version).
     </li>

<li> In ticket <a href=\"https://trac.modelica.org/Modelica/ticket/1757\">1757</a> it is (correctly) stated
     that the example model <a href=\"modelica://Modelica.Media.Examples.PsychrometricData\">PsychrometricData</a>
     was moved to another location and that this is a non-backwards compatible change.
     This non-backwards compatible change is accepted, because it fixes a circular depedency (a model references
     a package in which it resides), for details see ticket
     <a href=\"https://trac.modelica.org/Modelica/ticket/1679\">1679</a>.
     Fixing this ticket is seen as of much higher priority, as the small drawback that
     an example model is moved (and the probability is very high that this moved model is not
     used in any user model).
     </li>

<li> In the release notes of build 3 it is stated that no models, blocks, or functions have been
     newly introduced. This is not correct.<br>
     In package Modelica the following models have been newly introduced:<br>
     &nbsp;&nbsp;&nbsp;Modelica.Mechanics.Rotational.Examples.Utilities.SpringDamperNoRelativeStates
     (to fix ticket <a href=\"https://trac.modelica.org/Modelica/ticket/1375\">1375</a>).<br>
     &nbsp;&nbsp;&nbsp;Modelica.Fluid.Examples.NonCircularPipes
     (to check the fix of ticket <a href=\"https://trac.modelica.org/Modelica/ticket/1601\">1681</a>)<br>
     &nbsp;&nbsp;&nbsp;Modelica.Media.Examples.PsychrometricData
     (to fix ticket <a href=\"https://trac.modelica.org/Modelica/ticket/1679\">1679</a>).<br>
     In package ModelicaTest the following models have been newly introduced:<br>
     &nbsp;&nbsp;&nbsp;Modelica.Tables.CombiTable2D.Test19<br>
     &nbsp;&nbsp;&nbsp;Tables.CombiTable2D.Test20<br>
     &nbsp;&nbsp;&nbsp;Tables.CombiTable2D.Test21<br>
     &nbsp;&nbsp;&nbsp;Tables.CombiTable2D.Test22<br>
     &nbsp;&nbsp;&nbsp;Tables.CombiTable2D.Test23<br>
     &nbsp;&nbsp;&nbsp;Tables.CombiTimeTable.Test68<br>
     &nbsp;&nbsp;&nbsp;Tables.CombiTimeTable.Test69<br>
     &nbsp;&nbsp;&nbsp;Tables.CombiTimeTable.Test70<br>
     &nbsp;&nbsp;&nbsp;Tables.CombiTimeTable.Test71<br>
     &nbsp;&nbsp;&nbsp;Tables.CombiTimeTable.Test72<br>
     &nbsp;&nbsp;&nbsp;MultiBody.PlanarLoopWithMove
     </li>
</ol>

<p>
This release of package Modelica, and the accompanying ModelicaTest,
has been tested with the following tools (the tools are listed alphabetically.
At the time of the test, some of the tools might not yet support the complete Modelica package):
</p>

<ul>
<li> Dymola 2016<br>
     Regression test results with regards to Modelica 3.2.1 build 3 are available
     <a href=\"https://trac.modelica.org/Modelica/ticket/1681\">here</a>.</li>

<li> Maplesim Parser</li>
<li> OpenModelica(Windows, Linux, Mac)<br>
     Test reports for the daily builds are available
     <a href=\"https://trac.openmodelica.org/OpenModelica/wiki\">here</a>.</li>
</ul>

<p>
The exact difference between package Modelica version 3.2.1 build 3 and version 3.2.1 build 4 is summarized in
a <a href=\"modelica://Modelica/Resources/Documentation/Version-3.2.1/DifferencesTo3.2.1+build.3.html\">comparison table</a>.
</p>
</html>"));
      end Version_3_2_1_build_4;

      class Version_3_2_1_build_3 "Version 3.2.1 build 3 (July 30, 2015)"
        extends Modelica.Icons.ReleaseNotes;
        annotation(
          Documentation(info = "<html>
<p>Version 3.2.1 build 3 is backward compatible to version 3.2.1 build 2, that is models developed with versions 3.0, 3.0.1, 3.1, 3.2 or 3.2.1 will work without any changes also with version 3.2.1 build 3. This version is a bug-fix release. Short Overview: </p>
<ul>
<li>This version of the Modelica package is <b>fully compatible</b> to Modelica Specification <b>3.2 revision 2</b>. </li>
<li>About <a href=\"modelica://Modelica/Resources/Documentation/Version-3.2.1/ResolvedTracTickets-build-3.html\">103 tickets</a> have been fixed for this release. </li>
<li><b>No</b> models and blocks and functions are newly included. </li>
</ul>
<p>This release of package Modelica, and the accompanying ModelicaTest, has been tested with the following tools (the tools are listed alphabetically. At the time of the test, some of the tools might not yet support the complete Modelica package): </p>
<ul>
<li>Dymola 2016<br>Regression test results with regards to Modelica 3.2.1 build 2 are available <a href=\"https://trac.modelica.org/Modelica/ticket/1681\">here</a>.</li>
<li>Maplesim Parser</li>
<li>OpenModelica(Windows, Linux, Mac)<br>Test reports for the daily builds are available <a href=\"https://trac.openmodelica.org/OpenModelica/wiki\">here</a>.</li>
<li>LMS Imagine.Lab Amesim 14.1</li>
</ul>
<p>The exact difference between package Modelica version 3.2.1 build 2 and version 3.2.1 build 3 is summarized in a <a href=\"modelica://Modelica/Resources/Documentation/Version-3.2.1/DifferencesTo3.2.1+build.2.html\">comparison table</a>. </p>
</html>"));
      end Version_3_2_1_build_3;

      class Version_3_2_1 "Version 3.2.1 (August 14, 2013)"
        extends Modelica.Icons.ReleaseNotes;
        annotation(
          Documentation(info = "<html>
<p>
Version 3.2.1 is backward compatible to version 3.2, that is models developed with
versions 3.0, 3.0.1, 3.1, or 3.2 will work without any changes also with version 3.2.1.
This version is a \"clean-up\" with major emphasis on quality improvement and
tool compatibility. The goal is that all
<a href=\"https://www.modelica.org/tools\">Modelica tools</a> will support this package
and will interpret it in the same way. Short Overview:
</p>

<ul>
<li> This version of the Modelica package is <b>fully compatible</b> to
     Modelica Specification <b>3.2 revision 2</b>.<br>
     (Especially, some operators used in package Modelica,
     such as \"rooted\", have been standardized in 3.2 rev. 2,
     as well as vendor specific annotations. Furthermore,
     ambiguous/unclear descriptions in the specification have
     been corrected/improved. One important improvement in packages
     Modelica and ModelicaTest is that the initialization has been fully defined
     in all example models, in order that all tools can produce the same result
     without relying on tool heuristics).
     </li>

<li> About <a href=\"modelica://Modelica/Resources/Documentation/Version-3.2.1/ResolvedTracTickets.html\">400 tickets</a>
     have been fixed for this release, and
     especially all compliance issues and all relevant defect issues.
     </li>

<li> An open source implementation of the <b>table blocks</b> has been provided
     by <a href=\"http://www.itisim.com\">ITI GmbH</a>. This work has been
     <a href=\"https://www.modelica.org/news_items/call-texts-to-improve-modelica-2012/2012-12-20-Call-for-quotation-for-MSL-tables.pdf/at_download/file\">paid by Modelica Association</a>.
     As a result, all parts of package Modelica are now available
     in a free implementation. Additionally new features have been added to the table blocks
     by this implementation:
     <ul>
     <li>The table outputs can be differentiated once.</li>
     <li>Support of binary MATLAB MAT-file formats v6 and v7</li>
     <li>New option ConstantSegments for parameter Smoothness</li>
     <li>New option NoExtrapolation for parameter Extrapolation</li>
     <li>Support of tables provided in the C-Code (usertab.c, for realtime systems without file system)</li>
     </ul>

<li> <b>Icons</b> have been re-designed by Wolfram Research to provide a more modern view .</li>

<li> The <b>Modelica.Media.Air.MoistAir</b> media model has been improved so that it
     can be used in a temperature range of 190 ... 647 K (previously: 240 ... 400 K).</li>

<li> New media models for air (<b>ReferenceAir</b> with a large operating range: 30 ... 2000 K,
     0 ... 2000 MPa), for moist air (<b>ReferenceMoistAir</b> with a large operating range:
     143.15 ... 2000 K, 0 .. 10 MPa;  but 1-2 orders of magnitude slower as
     Modelica.Media.Air.MoistAir),
     and the refrigerant <b>R134a</b> are included in the Modelica.Media library in order to
     improve the modeling of air conditioning systems especially in aircrafts.
     These models have been developed by
     <a href=\"http://www.xrg-simulation.de/\">XRG Simulation GmbH</a>
     as part of the <a href=\"http://www.cleansky.eu/\">Clean Sky</a> JTI project
     (Project number: 296369; Theme: JTI-CS-2011-1-SGO-02-026).
     The partial financial support by the European Union for this development
     is highly appreciated.</li>

<li> <b>60</b> models and blocks and <b>90</b> functions are newly included, for details see below.</li>

</ul>

<p>
This release of package Modelica, and the accompanying ModelicaTest, has been tested with the
following tools (the tools are listed alphabetically. At the time of the test, some of the
tools might not yet supported the complete Modelica package):
</p>

<ul>
<li> CyModelica </li>
<li> Dymola 2014 (Windows 64 bit)<br>
     Regression test results with regards to Modelica 3.2 are available
     in ticket <a href=\"https://trac.modelica.org/Modelica/ticket/1114\">#1114</a>.</li>
<li> Dymola 2014 FD01 development with pedantic flag (Windows 64 bit)<br>
     (\"pedantic flag\" means that strict Modelica compliance is checked.
     Dymola 2014 fails with pedantic flag, e.g., because the annotation DocumentationClass
     was not standardized when this version of Dymola was released).</li>
<li> Maplesim Parser</li>
<li> MWorks 3.2</li>
<li> OpenModelica 1.9.0 Beta4+dev (Windows, Linux, Mac)<br>
     Test reports for the daily builds are available
     <a href=\"https://trac.openmodelica.org/OpenModelica/wiki\">here</a>.
     Test reports of comparisons with Dymola result files are available
     <a href=\"https://test.openmodelica.org/hudson/job/OpenModelica_TEST_CLANG/lastCompletedBuild/testReport/(root)/simulation_libraries_msl32/\">here</a>.
     </li>
<li> SimulationX 3.6</li>
</ul>

<p>
The following Modelica packages have been tested that they work together with this release of package Modelica
(alphabetical list):
</p>

<ul>
<li> Buildings 1.4 (LBNL) </li>
<li> FlexibleBodies 2.0.1 (DLR)</li>
<li> Modelica_Synchronous 0.91 (DLR)</li>
<li> Optimization 2.2 (DLR)</li>
<li> PowerTrain 2.2.0 (DLR) </li>
</ul>

<p>
The new open source tables have been tested by T. Beutlich (ITI):
</p>

<ul>
<li> 193 Modelica test models for compatibility check with previous table implementation
     (available in ModelicaTest.Tables).
     Performed tests with SimulationX 3.5.707 (32 bit) and
     Dymola 2013 FD01 (32 bit). Furthermore a basic check was performed in OpenModelica
     to make sure it works in general.
     </li>
<li> The two C source files (Modelica/Resources/C-Sources/ModelicaStandardTables.c; ModelicaMatIO.c)
     have been tested to successfully compile for the following platforms<br>
     &nbsp;&nbsp;&nbsp;Windows 32 and 64 bit<br>
     &nbsp;&nbsp;&nbsp;Linux 32 and 64 bit<br>
     &nbsp;&nbsp;&nbsp;dSPACE SCALEXIO<br>
     &nbsp;&nbsp;&nbsp;dSPACE DS1005 (no file system)<br>
     &nbsp;&nbsp;&nbsp;dSPACE DS1006 (no file system)<br>
     &nbsp;&nbsp;&nbsp;dSPACE DS1401 (no file system)
     </li>
<li> The following compilers/environments have been used for the platform evaluation<br>
     &nbsp;&nbsp;&nbsp;Microsoft compilers (VC6 and &ge; VS2005 (Win32 and x64)) <br>
     &nbsp;&nbsp;&nbsp;MinGW (GCC 4.4.0 and GCC 4.7.2)<br>
     &nbsp;&nbsp;&nbsp;Cygwin (GCC 4.3.0)<br>
     &nbsp;&nbsp;&nbsp;Open WATCOM 1.3<br>
     &nbsp;&nbsp;&nbsp;LCC 2.4.1<br>
     &nbsp;&nbsp;&nbsp;Borland C/C++ (free command line tools) 5.5<br>
     &nbsp;&nbsp;&nbsp;GCC 4.x on Linux<br>
     &nbsp;&nbsp;&nbsp;GCC 3.3.5 (for DS1006)<br>
     &nbsp;&nbsp;&nbsp;Microtec PowerPC Compiler 3.7 (for DS1005)
     </li>
</ul>

<p>
The exact difference between package Modelica version 3.2 and version 3.2.1 is
summarized in a
<a href=\"modelica://Modelica/Resources/Documentation/Version-3.2.1/DifferencesTo32.html\">comparison table</a>.
</p>

<p>
About <b>400</b> trac tickets have been fixed for this release. An overview is given
<a href=\"modelica://Modelica/Resources/Documentation/Version-3.2.1/ResolvedTracTickets.html\">here</a>.
Clicking on a ticket gives all information about it.
</p>


<p><br>
The following <b style=\"color:blue\">new components</b> have been added
to <b style=\"color:blue\">existing</b> libraries:
</p>

<table border=\"1\" cellspacing=0 cellpadding=2 style=\"border-collapse:collapse;\">
<tr><td colspan=\"2\"><b>Modelica.Blocks.Logical.</b></td></tr>
<tr><td valign=\"top\" width=\"150\"> RSFlipFlop</td>
    <td valign=\"top\"> Basic RS flip flop</td></tr>
<tr><td colspan=\"2\"><b>Modelica.Blocks.Math.</b></td></tr>
<tr><td valign=\"top\" width=\"150\"> MinMax</td>
    <td valign=\"top\">Output the minimum and the maximum element of the input vector </td></tr>
<tr><td valign=\"top\" width=\"150\"> LinearDependency </td>
    <td valign=\"top\">Output a linear combination of the two inputs </td></tr>
<tr><td colspan=\"2\"><b>Modelica.Blocks.Nonlinear.</b></td></tr>
<tr><td valign=\"top\" width=\"150\"> SlewRateLimiter</td>
    <td valign=\"top\"> Limit the slew rate of a signal </td></tr>

<tr><td colspan=\"2\"><b>Modelica.Electrical.Digital.Memories</b></td></tr>
<tr><td valign=\"top\" width=\"150\"> DLATRAM</td>
    <td valign=\"top\"> Level sensitive Random Access Memory </td></tr>
<tr><td valign=\"top\" width=\"150\"> DLATROM</td>
    <td valign=\"top\"> Level sensitive Read Only Memory </td></tr>
<tr><td colspan=\"2\"><b>Modelica.Electrical.Digital.Multiplexers</b></td></tr>
<tr><td valign=\"top\" width=\"150\"> MUX2x1</td>
    <td valign=\"top\"> A two inputs MULTIPLEXER for multiple value logic (2 data inputs, 1 select input, 1 output) </td></tr>

<tr><td colspan=\"2\"><b>Modelica.Electrical.Machines.Examples.AsynchronousInductionMachines.</b></td></tr>
<tr><td valign=\"top\" width=\"150\"> AIMC_Initialize </td>
    <td valign=\"top\"> Steady-State Initialization examnple of AsynchronousInductionMachineSquirrelCage </td></tr>
<tr><td colspan=\"2\"><b>Modelica.Electrical.Machines.Examples.SynchronousInductionMachines.</b></td></tr>
<tr><td valign=\"top\" width=\"150\"> SMPM_VoltageSource </td>
    <td valign=\"top\"> PermanentMagnetSynchronousInductionMachine example fed by FOC </td></tr>

<tr><td colspan=\"2\"><b>Modelica.Electrical.MultiPhase.Examples.</b></td></tr>
<tr><td valign=\"top\" width=\"150\"> TestSensors </td>
    <td valign=\"top\"> Example for multiphase quasiRMS sensors: A sinusoidal source feeds a load consisting of resistor and inductor </td></tr>
<tr><td colspan=\"2\"><b>Modelica.Electrical.MultiPhase.Sensors.</b></td></tr>
<tr><td valign=\"top\" width=\"150\"> VoltageQuasiRMSSensor </td>
    <td valign=\"top\"> Continuous quasi voltage RMS sensor for multi phase system </td></tr>
<tr><td valign=\"top\" width=\"150\"> CurrentQuasiRMSSensor </td>
    <td valign=\"top\"> Continuous quasi current RMS sensor for multi phase system </td></tr>
<tr><td colspan=\"2\"><b>Modelica.Electrical.MultiPhase.Blocks.</b></td></tr>
<tr><td valign=\"top\" width=\"150\"> QuasiRMS </td>
    <td valign=\"top\"> Determine quasi RMS value of a multi-phase system </td></tr>
<tr><td colspan=\"2\"><b>Modelica.Electrical.MultiPhase.Functions.</b></td></tr>
<tr><td valign=\"top\" width=\"150\"> quasiRMS </td>
    <td valign=\"top\"> Calculate continuous quasi RMS value of input </td></tr>
<tr><td valign=\"top\" width=\"150\"> activePower </td>
    <td valign=\"top\"> Calculate active power of voltage and current input </td></tr>
<tr><td valign=\"top\" width=\"150\"> symmetricOrientation </td>
    <td valign=\"top\"> Orientations of the resulting fundamental wave field phasors </td></tr>

<tr><td colspan=\"2\"><b>Modelica.Electrical.Spice3.Examples.</b></td></tr>
<tr><td valign=\"top\" width=\"150\"> CoupledInductors<br>
                                      CascodeCircuit<br>
                                      Spice3BenchmarkDifferentialPair<br>
                                      Spice3BenchmarkMosfetCharacterization<br>
                                      Spice3BenchmarkRtlInverter<br>
                                      Spice3BenchmarkFourBitBinaryAdder</td>
    <td valign=\"top\"> Spice3 examples and benchmarks from the SPICE3 Version e3 Users's Manual </td></tr>
<tr><td colspan=\"2\"><b>Modelica.Electrical.Spice3.Basic.</b></td></tr>
<tr><td valign=\"top\" width=\"150\"> K_CoupledInductors</td>
    <td valign=\"top\"> Inductive coupling via coupling factor K </td></tr>
<tr><td colspan=\"2\"><b>Modelica.Electrical.Spice3.Semiconductors.</b></td></tr>
<tr><td valign=\"top\" width=\"150\"> M_NMOS2 <br>
                                      M_PMOS2 <br>
                                      ModelcardMOS2</td>
    <td valign=\"top\">  N/P channel MOSFET transistor with fixed level 2 </td></tr>
<tr><td valign=\"top\" width=\"150\"> J_NJFJFE <br>
                                      J_PJFJFE <br>
                                      ModelcardJFET</td>
    <td valign=\"top\">  N/P-channel junction field-effect transistor </td></tr>
<tr><td valign=\"top\" width=\"150\"> C_Capacitor <br>
                                      ModelcardCAPACITOR</td>
    <td valign=\"top\">  Semiconductor capacitor model </td></tr>

<tr><td colspan=\"2\"><b>Modelica.Magnetic.FundamentalWave.Examples.BasicMachines.</b></td></tr>
<tr><td valign=\"top\" width=\"150\"> AIMC_DOL_MultiPhase<br>
                                      AIMS_Start_MultiPhase<br>
                                      SMPM_Inverter_MultiPhase<br>
                                      SMEE_Generator_MultiPhase<br>
                                      SMR_Inverter_MultiPhase</td>
    <td valign=\"top\"> Multi-phase machine examples </td></tr>

<tr><td colspan=\"2\"><b>Modelica.Fluid.Sensors.</b></td></tr>
<tr><td valign=\"top\" width=\"150\"> MassFractions<br>
                                      MassFractionsTwoPort</td>
    <td valign=\"top\"> Ideal mass fraction sensors </td></tr>

<tr><td colspan=\"2\"><b>Modelica.Media.</b></td></tr>
<tr><td valign=\"top\" width=\"150\">R134a</td>
    <td valign=\"top\"> R134a (Tetrafluoroethane) medium model in the range (0.0039 bar .. 700 bar,
    169.85 K .. 455 K)</td></tr>

<tr><td colspan=\"2\"><b>Modelica.Media.Air.</b></td></tr>
<tr><td valign=\"top\" width=\"150\"> ReferenceAir</td>
    <td valign=\"top\"> Detailed dry air model with a large operating range (130 ... 2000 K, 0 ... 2000 MPa)
                        based on Helmholtz equations of state</td></tr>
<tr><td valign=\"top\" width=\"150\"> ReferenceMoistAir</td>
    <td valign=\"top\"> Detailed moist air model (143.15 ... 2000 K)</td></tr>
<tr><td valign=\"top\" width=\"150\"> MoistAir</td>
    <td valign=\"top\"> Temperature range of functions of MoistAir medium enlarged from
                        240 - 400 K to  190 - 647 K.</td></tr>
<tr><td colspan=\"2\"><b>Modelica.Media.Air.MoistAir.</b></td></tr>
<tr><td valign=\"top\" width=\"150\"> velocityOfSound<br>
                                      isobaricExpansionCoefficient<br>
                                      isothermalCompressibility<br>
                                      density_derp_h<br>
                                      density_derh_p<br>
                                      density_derp_T<br>
                                      density_derT_p<br>
                                      density_derX<br>
                                      molarMass<br>
                                      T_psX<br>
                                      setState_psX<br>
                                      s_pTX<br>
                                      s_pTX_der<br>
                                      isentropicEnthalpy</td>
    <td valign=\"top\"> Functions returning additional properties of the moist air medium model</td></tr>

<tr><td colspan=\"2\"><b>Modelica.Thermal.HeatTransfer.Components.</b></td></tr>
<tr><td valign=\"top\" width=\"150\"> ThermalResistor</td>
    <td valign=\"top\"> Lumped thermal element transporting heat without storing it (dT = R*Q_flow) </td></tr>
<tr><td valign=\"top\" width=\"150\"> ConvectiveResistor</td>
    <td valign=\"top\"> Lumped thermal element for heat convection (dT = Rc*Q_flow) </td></tr>

<tr><td colspan=\"2\"><b>Modelica.MultiBody.Examples.Constraints.</b></td></tr>
<tr><td valign=\"top\" width=\"150\"> PrismaticConstraint <br>
                        RevoluteConstraint<br>
                        SphericalConstraint<br>
                        UniversalConstraint</td>
    <td valign=\"top\"> Demonstrates the use of the new Joints.Constraints joints by comparing
                        them with the standard joints.</td></tr>
<tr><td colspan=\"2\"><b>Modelica.MultiBody.Joints.Constraints.</b></td></tr>
<tr><td valign=\"top\" width=\"150\"> Prismatic <br>
                        Revolute<br>
                        Spherical<br>
                        Universal</td>
    <td valign=\"top\"> Joint elements formulated as kinematic constraints. These elements are
                        designed to break kinematic loops and result usually in numerically more
                        efficient and reliable loop handling as the (standard) automatic handling.</td></tr>

<tr><td colspan=\"2\"><b>Modelica.Mechanics.Rotational.</b></td></tr>
<tr><td valign=\"top\" width=\"150\"> MultiSensor</td>
    <td valign=\"top\"> Ideal sensor to measure the torque and power between two flanges and the absolute angular velocity </td></tr>

<tr><td colspan=\"2\"><b>Modelica.Mechanics.Translational.</b></td></tr>
<tr><td valign=\"top\" width=\"150\"> MultiSensor</td>
    <td valign=\"top\"> Ideal sensor to measure the absolute velocity, force and power between two flanges </td></tr>

<tr><td colspan=\"2\"><b>Modelica.Math.</b></td></tr>
<tr><td valign=\"top\" width=\"150\"> isPowerOf2</td>
    <td valign=\"top\"> Determine if the integer input is a power of 2 </td></tr>
<tr><td colspan=\"2\"><b>Modelica.Math.Vectors.</b></td></tr>
<tr><td valign=\"top\" width=\"150\"> normalizedWithAssert</td>
    <td valign=\"top\"> Return normalized vector such that length = 1 (trigger an assert for zero vector) </td></tr>
<tr><td colspan=\"2\"><b>Modelica.Math.BooleanVectors.</b></td></tr>
<tr><td valign=\"top\" width=\"150\"> countTrue</td>
    <td valign=\"top\"> Returns the number of true entries in a Boolean vector  </td></tr>
<tr><td valign=\"top\" width=\"150\"> enumerate</td>
    <td valign=\"top\"> Enumerates the true entries in a Boolean vector (0 for false entries) </td></tr>
<tr><td valign=\"top\" width=\"150\"> index</td>
    <td valign=\"top\"> Returns the indices of the true entries of a Boolean vector</td></tr>

<tr><td colspan=\"2\"><b>Modelica.Utilities.Files.</b></td></tr>
<tr><td valign=\"top\" width=\"150\"> loadResource</td>
    <td valign=\"top\"> Return the absolute path name of a URI or local file name  </td></tr>

<tr><td colspan=\"2\"><b>Modelica.SIunits.</b></td></tr>
<tr><td valign=\"top\" width=\"150\"> PressureDifference<br>
                        MolarDensity<br>
                        MolarEnergy<br>
                        MolarEnthalpy<br>
                        TimeAging<br>
                        ChargeAging<br>
                        PerUnit<br>
                        DerPressureByDensity<br>
                        DerPressureByTemperature</td>
    <td valign=\"top\"> New SI unit types </td></tr>
</table>
</html>"));
      end Version_3_2_1;

      class Version_3_2 "Version 3.2 (Oct. 25, 2010)"
        extends Modelica.Icons.ReleaseNotes;
        annotation(
          Documentation(info = "<html>

<p>
Version 3.2 is backward compatible to version 3.1, i.e., models developed with
versions 3.0, 3.0.1, or 3.1 will work without any changes also with version 3.2.
This version is a major improvement:
</p>

<ul>
<li> <b>357</b> models and blocks and <b>295</b> functions are newly included.</li>

<li><b>7</b> new libraries are included.</li>

<li> The icons of the library are newly designed to provide a modern, unified view,
     see <a href=\"modelica://Modelica.Icons\">Modelica.Icons</a>.</li>

<li> All non-Modelica files, such as images, pdf-files, C-source files,
     scripts are moved to the new directory \"Modelica\\Resources\".
     Furthermore, all file references are changed to URIs as introduced in
     Modelica 3.1 (e.g., a file with the file name
     \"Modelica/Resources/Images/xxx\" is referenced as
     \"modelica://Modelica/Resources/Images/xxx\").</li>

<li> All physical models that dissipate heat (such as electrical elements,
     electrical machines, bearings, dampers, etc.), have now an optional heat port
     to which the dissipated energy is flowing, if activated.
     This will significantly improve design studies about the thermal efficiency
     of technical systems.</li>

<li> All electrical machines in the
     <a href=\"modelica://Modelica.Electrical.Machines\">Machines</a>
     library have now a \"Losses\" tab in the parameter menu to optionally
     model machines losses such as frictional losses, stator core losses
     or stray load losses, respectively.</li>

<li> All electrical machines in the
     <a href=\"modelica://Modelica.Electrical.Machines\">Machines</a>
     library have now a \"powerBalance\" result record,
     summarizing converted power and losses.</li>
</ul>

<p>
Version 3.2 is slightly based on the Modelica Specification 3.2. It uses
the following new language elements (compared to Modelica Specification 3.1):
</p>

<ul>
<li> Operator records and overloaded operators. </li>
<li> Functions as input arguments to functions.</li>
<li> Improved expandable connectors (variables declared in expandable
     connectors are ignored if not referenced).</li>
</ul>

<p>
A large part of the new classes have been developed with
partial financial support by
<a href=\"http://www.bmbf.de/en/index.php\">BMBF</a>
(BMBF F&ouml;rderkennzeichen: 01IS07022F)
within the <a href=\"http://www.itea2.org\">ITEA2</a> project
<a href=\"http://www.eurosyslib.com/\">EUROSYSLIB</a>.
We highly appreciate this funding.
</p>

<p>
The following <b style=\"color:blue\">new libraries</b> have been added:
</p>

<table border=\"1\" cellspacing=0 cellpadding=2>
<tr><td valign=\"top\"><a href=\"modelica://Complex\">Complex</a></td>
    <td valign=\"top\">
    This is a top-level record outside of the Modelica Standard Library.
    It is used for complex numbers and contains overloaded operators.
    From a users point of view, Complex is used in a similar way as the
    built-in type Real. Example:<br>
    <code>&nbsp;  Real     a    = 2;</code><br>
    <code>&nbsp;  Complex  j    = Modelica.ComplexMath.j;</code><br>
    <code>&nbsp;  Complex  b    = 2 + 3*j;</code><br>
    <code>&nbsp;  Complex  c    = (2*b + a)/b;</code><br>
    <code>&nbsp;  Complex  d    = Modelica.ComplexMath.sin(c);</code><br>
    <code>&nbsp;  Complex  v[3] = {b/2, c, 2*d};</code><br>
    (This library was developed by Marcus Baur, DLR).
    </td></tr>

<tr><td valign=\"top\"><a href=\"modelica://Modelica.ComplexBlocks\">Modelica.ComplexBlocks</a></td>
    <td valign=\"top\">
    Library of basic input/output control blocks with Complex signals.<br>
    This library is especially useful in combination with the new
    <a href=\"modelica://Modelica.Electrical.QuasiStationary\">Modelica.Electrical.QuasiStationary</a>
    library in order to build up very fast simulations of electrical circuits with periodic
    currents and voltages.<br>
    (This library was developed by Anton Haumer).
    </td></tr>

<tr><td valign=\"top\"><a href=\"modelica://Modelica.Electrical.QuasiStationary\">Modelica.Electrical.QuasiStationary</a></td>
    <td valign=\"top\">
    Library for quasi-stationary electrical singlephase and multiphase AC simulation.<br>
    This library allows very fast simulations of electrical circuits with sinusoidal
    currents and voltages by only taking into account the quasi-stationary, periodic part
    and neglecting non-periodic transients.<br>
    (This library was developed by Anton Haumer and Christian Kral).
    </td></tr>

<tr><td valign=\"top\"><a href=\"modelica://Modelica.Electrical.Spice3\">Modelica.Electrical.Spice3</a></td>
    <td valign=\"top\">
    Library with components of the Berkeley
    <a href=\"http://bwrc.eecs.berkeley.edu/Classes/IcBook/SPICE/\">SPICE3</a>
    simulator:<br>
    R, C, L, controlled and independent sources, semiconductor device models
    (MOSFET Level 1, Bipolar junction transistor, Diode, Semiconductor resistor).
    The components have been intensively tested with more than 1000 test models
    and compared with results from the SPICE3 simulator. All test models give identical
    results in Dymola 7.4 with respect to the Berkeley SPICE3 simulator up to the relative
    tolerance of the integrators.<br>
    This library allows detailed simulations of electronic circuits.
    Work on Level 2 SPICE3 models, i.e., even more detailed models, is under way.
    Furthermore, a pre-processor is under development to transform automatically
    a SPICE netlist into a Modelica model, in order that the many available
    SPICE3 models can be directly used in a Modelica model.<br>
    (This library was developed by Fraunhofer Gesellschaft, Dresden).
    </td></tr>

<tr><td valign=\"top\"><a href=\"modelica://Modelica.Magnetic.FundamentalWave\">Modelica.Magnetic.FundamentalWave</a></td>
    <td valign=\"top\">
     Library for magnetic fundamental wave effects in electric machines for the
     application in three phase electric machines.
     The library is an alternative approach to the Modelica.Electrical.Machines library.
     A great advantage of this library is the strict object orientation of the
     electrical and magnetic components that the electric machines models are composed of.
     This allows an easier incorporation of more detailed physical effects of
     electrical machines.
     From a didactic point of view this library is very beneficial for students in the field
     of electrical engineering.<br>
     (This library was developed by Christian Kral, and Anton Haumer, using
     ideas and source code of a library from Michael Beuschel from 2000).
    </td></tr>

<tr><td valign=\"top\"><a href=\"modelica://Modelica.Fluid.Dissipation\">Modelica.Fluid.Dissipation</a></td>
    <td valign=\"top\">
     Library with functions to compute convective heat transfer and pressure loss characteristics.<br>
     (This library was developed by Thorben Vahlenkamp and Stefan Wischhusen from
     XRG Simulation GmbH).
    </td></tr>

<tr><td valign=\"top\"><a href=\"modelica://Modelica.ComplexMath\">Modelica.ComplexMath</a></td>
    <td valign=\"top\">
    Library of complex mathematical functions (e.g., sin, cos) and of functions operating
    on complex vectors.<br>
    (This library was developed by Marcus Baur from DLR-RM, Anton Haumer, and
     HansJ&uuml;rg Wiesmann).
    </td></tr>
</table>

<p><br>
The following <b style=\"color:blue\">new components</b> have been added
to <b style=\"color:blue\">existing</b> libraries:
</p>

<table border=\"1\" cellspacing=0 cellpadding=2 style=\"border-collapse:collapse;\">
<tr><td colspan=\"2\"><b>Modelica.UsersGuide</b></td></tr>
<tr><td valign=\"top\"> Conventions
                      </td>
    <td valign=\"top\"> Considerably improved 'Conventions' for the Modelica Standard Library. </td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Blocks.Examples</b></td></tr>
<tr><td valign=\"top\"> Filter<br>
                      FilterWithDifferentation<br>
                      FilterWithRiseTime<br>
                      RealNetwork1<br>
                      IntegerNetwork1<br>
                      BooleanNetwork1<br>
                      Interaction1
                      </td>
    <td valign=\"top\"> Examples for the newly introduced block components. </td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Blocks.Continuous</b></td></tr>
<tr><td valign=\"top\"> Filter </td>
    <td valign=\"top\"> Continuous low pass, high pass, band pass and band stop
                      IIR-filter of type CriticalDamping, Bessel, Butterworth and Chebyshev I. </td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Blocks.Interaction.Show</b></td></tr>
<tr><td valign=\"top\"> RealValue<br>
                      IntegerValue<br>
                      BooleanValue</td>
    <td valign=\"top\"> Blocks to show the values of variables in a diagram animation. </td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Blocks.Interfaces</b></td></tr>
<tr><td valign=\"top\"> RealVectorInput<br>
                      IntegerVectorInput<br>
                      BooleanVectorInput<br>
                      PartialRealMISO<br>
                      PartialIntegerSISO<br>
                      PartialIntegerMISO<br>
                      PartialBooleanSISO_small<br>
                      PartialBooleanMISO
                      </td>
    <td valign=\"top\"> Interfaces and partial blocks for the new block components. </td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Blocks.Math</b></td></tr>
<tr><td valign=\"top\"> MultiSum<br>
                      MultiProduct<br>
                      MultiSwitch </td>
    <td valign=\"top\"> Sum, product and switch blocks with 1,2,...,N inputs
                      (based on connectorSizing annotation to handle vectors of
                       connectors in a convenient way). </td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Blocks.MathInteger</b></td></tr>
<tr><td valign=\"top\"> MultiSwitch<br>
                      Sum<br>
                      Product<br>
                      TriggeredAdd</td>
    <td valign=\"top\"> Mathematical blocks for Integer signals. </td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Blocks.Boolean</b></td></tr>
<tr><td valign=\"top\"> MultiSwitch<br>
                      And<br>
                      Or<br>
                      Xor<br>
                      Nand<br>
                      Nor<br>
                      Not<br>
                      RisingEdge<br>
                      FallingEdge<br>
                      ChangingEdge<br>
                      OnDelay</td>
    <td valign=\"top\"> Mathematical blocks for Boolean signals.
                      Some of these blocks are available also in library
                      <a href=\"modelica://Modelica.Blocks.Logical\">Logical</a>.
                      The new design is based on the connectorSizing annotation that allows
                      the convenient handling of an arbitrary number of input signals
                      (e.g., the \"And\" block has 1,2,...,N inputs, instead of only 2 inputs
                      in the <a href=\"modelica://Modelica.Blocks.Logical\">Logical</a> library).
                      Additionally, the icons are smaller so that the diagram area is
                      better utilized</td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Blocks.Sources</b></td></tr>
<tr><td valign=\"top\"> RadioButtonSource</td>
    <td valign=\"top\"> Boolean signal source that mimics a radio button. </td> </tr>
<tr><td valign=\"top\"> IntegerTable</td>
    <td valign=\"top\"> Generate an Integer output signal based on a table matrix
                      with [time, yi] values. </td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Electrical.Analog.Examples</b></td></tr>
<tr><td valign=\"top\"> SimpleTriacCircuit,<br>
                      IdealTriacCircuit,<br>
                      AD_DA_conversion </td>
    <td valign=\"top\"> Examples for the newly introduced Analog components. </td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Electrical.Analog.Ideal</b></td></tr>
<tr><td valign=\"top\"> IdealTriac,<br>
                      AD_Converter,<br>
                      DA_Converter </td>
    <td valign=\"top\"> AD and DA converter, ideal triac (based on ideal thyristor). </td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Electrical.Analog.Semiconductors</b></td></tr>
<tr><td valign=\"top\"> SimpleTriac </td>
    <td valign=\"top\"> Simple triac based on semiconductor thyristor model. </td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Electrical.Digital.Examples</b></td></tr>
<tr><td valign=\"top\">  Delay_example,<br>
                       DFFREG_example,<br>
                       DFFREGL_example,<br>
                       DFFREGSRH_example,<br>
                       DFFREGSRL_example,<br>
                       DLATREG_example,<br>
                       DLATREGL_example,<br>
                       DLATREGSRH_example,<br>
                       DLATREGSRL_example,<br>
                       NXFER_example,<br>
                       NRXFER_example,<br>
                       BUF3S_example,<br>
                       INV3S_example,<br>
                       WiredX_example </td>
    <td valign=\"top\"> Examples for the newly introduced Digital components. </td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Electrical.Digital.Interfaces</b></td></tr>
<tr><td valign=\"top\"> UX01,<br>
                      Strength,<br>
                      MIMO </td>
    <td valign=\"top\"> Interfaces for the newly introduced Digital components. </td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Electrical.Digital.Tables</b></td></tr>
<tr><td valign=\"top\"> ResolutionTable,<br>
                      StrengthMap,<br>
                      NXferTable,<br>
                      NRXferTable,<br>
                      PXferTable,<br>
                      PRXferTable,<br>
                      Buf3sTable,<br>
                      Buf3slTable </td>
    <td valign=\"top\"> New Digital table components. </td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Electrical.Digital.Delay</b></td></tr>
<tr><td valign=\"top\"> InertialDelaySensitiveVector </td>
    <td valign=\"top\"> New Digital delay component. </td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Electrical.Digital.Registers</b></td></tr>
<tr><td valign=\"top\"> DFFR,<br>
                      DFFREG,<br>
                      DFFREGL,<br>
                      DFFSR,<br>
                      DFFREGSRH,<br>
                      DFFREGSRL,<br>
                      DLATR,<br>
                      DLATREG,<br>
                      DLATREGL,<br>
                      DLATSR,<br>
                      DLATREGSRH,<br>
                      DLATREGSRL </td>
    <td valign=\"top\"> Various register components (collection of flipflops and latches)
                      according to the VHDL standard. </td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Electrical.Digital.Tristates</b></td></tr>
<tr><td valign=\"top\"> NXFERGATE,<br>
                      NRXFERGATE,<br>
                      PXFERGATE,<br>
                      PRXFERGATE,<br>
                      BUF3S,<br>
                      BUF3SL,<br>
                      INV3S,<br>
                      INV3SL,<br>
                      WiredX </td>
    <td valign=\"top\"> Transfer gates, buffers, inverters and wired node.</td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Electrical.MultiPhase.Basic</b></td></tr>
<tr><td valign=\"top\"> MutualInductor </td>
    <td valign=\"top\"> Multi phase inductor providing a mutual inductance matrix model. </td> </tr>
<tr><td valign=\"top\"> ZeroInductor </td>
    <td valign=\"top\"> Multi phase zero sequence inductor. </td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Electrical.Machines</b></td></tr>
<tr><td valign=\"top\"> Examples </td>
    <td valign=\"top\"> Structured according to machine types:<br>
                      AsynchronousInductionMachines<br>
                      SynchronousInductionMachines<br>
                      DCMachines<br>
                      Transformers </td> </tr>
<tr><td valign=\"top\"> Losses.* </td>
    <td valign=\"top\"> Parameter records and models for losses in electrical machines and transformers (where applicable): <br>
                      Friction losses <br>
                      Brush losses <br>
                      Stray Load losses <br>
                      Core losses (only eddy current losses but no hysteresis losses; not for transformers) </td> </tr>
<tr><td valign=\"top\"> Thermal.* </td>
    <td valign=\"top\"> Simple thermal ambients, to be connected to the thermal ports of machines, <br>
                      as well as material constants and utility functions. </td> </tr>
<tr><td valign=\"top\"> Icons.* </td>
    <td valign=\"top\"> Icons for transient and quasistationary electrical machines and transformers. </td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Electrical.Machines.Examples.AsynchronousInductionMachines.</b></td></tr>
<tr><td valign=\"top\"> AIMC_withLosses </td>
    <td valign=\"top\"> Asynchronous induction machine with squirrel cage with losses </td> </tr>
<tr><td valign=\"top\"> AIMC_Transformer </td>
    <td valign=\"top\"> Asynchronous induction machine with squirrel cage - transformer starting </td> </tr>
<tr><td valign=\"top\"> AIMC_withLosses </td>
    <td valign=\"top\"> Test example of an asynchronous induction machine with squirrel cage with losses </td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Electrical.Machines.Examples.SynchronousInductionMachines.</b></td></tr>
<tr><td valign=\"top\"> SMPM_CurrentSource </td>
    <td valign=\"top\"> Permanent magnet synchronous induction machine fed by a current source </td> </tr>
<tr><td valign=\"top\"> SMEE_LoadDump </td>
    <td valign=\"top\"> Electrical excited synchronous induction machine with voltage controller </td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Electrical.Machines.Examples.DCMachines.</b></td></tr>
<tr><td valign=\"top\"> DCSE_SinglePhase </td>
    <td valign=\"top\"> Series excited DC machine, fed by sinusoidal voltage </td> </tr>
<tr><td valign=\"top\"> DCPM_Temperature </td>
    <td valign=\"top\"> Permanent magnet DC machine, demonstration of varying temperature </td> </tr>
<tr><td valign=\"top\"> DCPM_Cooling </td>
    <td valign=\"top\"> Permanent magnet DC machine, coupled with a simple thermal model </td> </tr>
<tr><td valign=\"top\"> DCPM_QuasiStationary </td>
    <td valign=\"top\"> Permanent magnet DC machine, comparison between transient and quasistationary model </td> </tr>
<tr><td valign=\"top\"> DCPM_Losses </td>
    <td valign=\"top\"> Permanent magnet DC machine, comparison between model with and without losses </td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Electrical.Machines.BasicMachines.QuasiStationaryDCMachines.</b></td></tr>
<tr><td valign=\"top\"> DC_PermanentMagnet <br>
                      DC_ElectricalExcited <br>
                      DC_SeriesExcited </td>
    <td valign=\"top\"> QuasiStationary DC machines, i.e., neglecting electrical transients </td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Electrical.Machines.BasicMachines.Components.</b></td></tr>
<tr><td valign=\"top\"> InductorDC </td>
    <td valign=\"top\"> Inductor model which neglects der(i) if Boolean parameter quasiStationary = true </td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Electrical.Machines.Interfaces.</b></td></tr>
<tr><td valign=\"top\">  ThermalPortTransformer <br>
                       PowerBalanceTransformer </td>
    <td valign=\"top\"> Thermal ports and power balances for electrical machines and transformers. </td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Electrical.Machines.Utilities</b></td></tr>
<tr><td valign=\"top\"> SwitchedRheostat </td>
    <td valign=\"top\"> Switched rheostat, used for starting asynchronous induction motors with slipring rotor. </td> </tr>
<tr><td valign=\"top\"> RampedRheostat </td>
    <td valign=\"top\"> Ramped rheostat, used for starting asynchronous induction motors with slipring rotor. </td> </tr>
<tr><td valign=\"top\"> SynchronousMachineData </td>
    <td valign=\"top\"> The parameters of the synchronous machine model with electrical excitation (and damper) are calculated
                      from parameters normally given in a technical description,
                      according to the standard EN 60034-4:2008 Appendix C. </td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Mechanics.MultiBody.Examples.Elementary.</b></td></tr>
<tr><td valign=\"top\"> HeatLosses </td>
    <td valign=\"top\"> Demonstrate the modeling of heat losses. </td> </tr>
<tr><td valign=\"top\"> UserDefinedGravityField </td>
    <td valign=\"top\"> Demonstrate the modeling of a user-defined gravity field. </td> </tr>
<tr><td valign=\"top\"> Surfaces </td>
    <td valign=\"top\"> Demonstrate the visualization of a sine surface,<br>
                      as well as a torus and a wheel constructed from a surface. </td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Mechanics.MultiBody.Joints.</b></td></tr>
<tr><td valign=\"top\"> FreeMotionScalarInit </td>
    <td valign=\"top\"> Free motion joint that allows initialization and state selection<br>
                      of single elements of the relevant vectors<br>
                      (e.g., initialize r_rel_a[2] but not the other elements of r_rel_a;<br>
                      this new component fixes ticket
                      <a href=\"https://trac.modelica.org/Modelica/ticket/274\">#274</a>) </td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Mechanics.MultiBody.Visualizers.</b></td></tr>
<tr><td valign=\"top\"> Torus </td>
    <td valign=\"top\"> Visualizing a torus. </td> </tr>
<tr><td valign=\"top\"> VoluminousWheel </td>
    <td valign=\"top\"> Visualizing a voluminous wheel. </td> </tr>
<tr><td valign=\"top\"> PipeWithScalarField </td>
    <td valign=\"top\"> Visualizing a pipe with scalar field quantities along the pipe axis. </td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Mechanics.MultiBody.Visualizers.ColorMaps.</b></td></tr>
<tr><td valign=\"top\"> jet<br>
                      hot<br>
                      gray<br>
                      spring<br>
                      summer<br>
                      autumn<br>
                      winter </td>
    <td valign=\"top\"> Functions returning different color maps. </td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Mechanics.MultiBody.Visualizers.Colors.</b></td></tr>
<tr><td valign=\"top\"> colorMapToSvg </td>
    <td valign=\"top\"> Save a color map on file in svg (scalable vector graphics) format. </td> </tr>
<tr><td valign=\"top\"> scalarToColor </td>
    <td valign=\"top\"> Map a scalar to a color using a color map. </td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Mechanics.MultiBody.Visualizers.Advanced.</b></td></tr>
<tr><td valign=\"top\"> Surface </td>
    <td valign=\"top\"> Visualizing a moveable, parameterized surface;<br>
                      the surface characteristic is provided by a function<br>
                      (this new component fixes ticket
                       <a href=\"https://trac.modelica.org/Modelica/ticket/181\">#181</a>)</td> </tr>
<tr><td valign=\"top\"> PipeWithScalarField </td>
    <td valign=\"top\"> Visualizing a pipe with a scalar field. </td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Mechanics.MultiBody.Visualizers.Advanced.SurfaceCharacteristics.</b></td></tr>
<tr><td valign=\"top\"> torus </td>
    <td valign=\"top\"> Function defining the surface characteristic of a torus. </td> </tr>
<tr><td valign=\"top\"> pipeWithScalarField </td>
    <td valign=\"top\"> Function defining the surface characteristic of a pipe<br>
                      where a scalar field value is displayed with color along the pipe axis. </td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Mechanics.Rotational.Examples.</b></td></tr>
<tr><td valign=\"top\"> HeatLosses </td>
    <td valign=\"top\"> Demonstrate the modeling of heat losses. </td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Mechanics.Translational.Examples.</b></td></tr>
<tr><td valign=\"top\"> HeatLosses </td>
    <td valign=\"top\"> Demonstrate the modeling of heat losses. </td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Fluid.Fittings.Bends</b></td></tr>
<tr><td valign=\"top\"> CurvedBend<br>
                      EdgedBend</td>
    <td valign=\"top\"> New fitting (pressure loss) components. </td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Fluid.Fittings.Orifices.</b></td></tr>
<tr><td valign=\"top\"> ThickEdgedOrifice</td>
    <td valign=\"top\"> New fitting (pressure loss) component. </td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Fluid.Fittings.GenericResistances.</b></td></tr>
<tr><td valign=\"top\"> VolumeFlowRate</td>
    <td valign=\"top\"> New fitting (pressure loss) component. </td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Math</b></td></tr>
<tr><td valign=\"top\"> isEqual </td>
    <td valign=\"top\"> Determine if two Real scalars are numerically identical. </td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Math.Vectors</b></td></tr>
<tr><td valign=\"top\"> find </td>
    <td valign=\"top\"> Find element in vector. </td> </tr>
<tr><td valign=\"top\"> toString </td>
    <td valign=\"top\"> Convert a real vector to a string. </td> </tr>
<tr><td valign=\"top\"> interpolate </td>
    <td valign=\"top\"> Interpolate in a vector. </td> </tr>
<tr><td valign=\"top\"> relNodePositions </td>
    <td valign=\"top\"> Return vector of relative node positions (0..1). </td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Math.Vectors.Utilities</b></td></tr>
<tr><td valign=\"top\"> householderVector<br>
                      householderReflection<br>
                      roots </td>
    <td valign=\"top\"> Utility functions for vectors that are used by the newly introduced functions,
                      but are only of interested for a specialist. </td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Math.Matrices</b></td></tr>
<tr><td valign=\"top\"> continuousRiccati<br>
                      discreteRiccati </td>
    <td valign=\"top\"> Return solution of continuous-time and discrete-time
                      algebraic Riccati equation respectively. </td> </tr>
<tr><td valign=\"top\"> continuousSylvester<br>
                      discreteSylvester </td>
    <td valign=\"top\"> Return solution of continuous-time and discrete-time
                      Sylvester equation respectively. </td> </tr>
<tr><td valign=\"top\"> continuousLyapunov<br>
                      discreteLyapunov </td>
    <td valign=\"top\"> Return solution of continuous-time and discrete-time
                      Lyapunov equation respectively. </td> </tr>
<tr><td valign=\"top\"> trace </td>
    <td valign=\"top\"> Return the trace of a matrix. </td> </tr>
<tr><td valign=\"top\"> conditionNumber </td>
    <td valign=\"top\"> Compute the condition number of a matrix. </td> </tr>
<tr><td valign=\"top\"> rcond </td>
    <td valign=\"top\"> Estimate the reciprocal condition number of a matrix. </td> </tr>
<tr><td valign=\"top\"> nullSpace </td>
    <td valign=\"top\"> Return a orthonormal basis for the null space of a matrix. </td> </tr>
<tr><td valign=\"top\"> toString </td>
    <td valign=\"top\"> Convert a matrix into its string representation. </td> </tr>
<tr><td valign=\"top\"> flipLeftRight </td>
    <td valign=\"top\"> Flip the columns of a matrix in left/right direction. </td> </tr>
<tr><td valign=\"top\"> flipUpDown </td>
    <td valign=\"top\"> Flip the rows of a matrix in up/down direction. </td> </tr>
<tr><td valign=\"top\"> cholesky </td>
    <td valign=\"top\"> Perform Cholesky factorization of a real symmetric positive definite matrix. </td> </tr>
<tr><td valign=\"top\"> hessenberg </td>
    <td valign=\"top\"> Transform a matrix to upper Hessenberg form. </td> </tr>
<tr><td valign=\"top\"> realSchur </td>
    <td valign=\"top\"> Computes the real Schur form of a matrix. </td> </tr>
<tr><td valign=\"top\"> frobeniusNorm </td>
    <td valign=\"top\"> Return the Frobenius norm of a matrix. </td> </tr>
<tr><td colspan=\"2\"><b>Modelica.Math.Matrices.LAPACK.</b></td></tr>
<tr><td valign=\"top\"> dtrevc<br>
                      dpotrf<br>
                      dtrsm<br>
                      dgees<br>
                      dtrsen<br>
                      dgesvx<br>
                      dhseqr<br>
                      dlange<br>
                      dgecon<br>
                      dgehrd<br>
                      dgeqrf<br>
                      dggevx<br>
                      dgesdd<br>
                      dggev<br>
                      dggevx<br>
                      dhgeqz<br>
                      dormhr<br>
                      dormqr<br>
                      dorghr</td>
    <td valign=\"top\"> New interface functions for LAPACK
                      (should usually not directly be used but only indirectly via
                      Modelica.Math.Matrices). </td> </tr>
<tr><td colspan=\"2\"><b>Modelica.Math.Matrices.Utilities.</b></td></tr>
<tr><td valign=\"top\"> reorderRSF<br>
                      continuousRiccatiIterative<br>
                      discreteRiccatiIterative<br>
                      eigenvaluesHessenberg<br>
                      toUpperHessenberg<br>
                      householderReflection<br>
                      householderSimilarityTransformation<br>
                      findLokal_tk</td>
    <td valign=\"top\"> Utility functions for matrices that are used by the newly introduced functions,
                      but are only of interested for a specialist. </td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Math.Nonlinear</b></td></tr>
<tr><td valign=\"top\"> quadratureLobatto </td>
    <td valign=\"top\"> Return the integral of an integrand function using an adaptive Lobatto rule.</td> </tr>
<tr><td valign=\"top\"> solveOneNonlinearEquation </td>
    <td valign=\"top\"> Solve f(u) = 0 in a very reliable and efficient way
                      (f(u_min) and f(u_max) must have different signs). </td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Math.Nonlinear.Examples.</b></td></tr>
<tr><td valign=\"top\"> quadratureLobatto1<br>
                      quadratureLobatto2<br>
                      solveNonlinearEquations1<br>
                      solveNonlinearEquations2 </td>
    <td valign=\"top\"> Examples that demonstrate the usage of the Modelica.Math.Nonlinear functions
                      to integrate over functions and to solve scalar nonlinear equations. </td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Math.BooleanVectors.</b></td></tr>
<tr><td valign=\"top\"> allTrue </td>
    <td valign=\"top\"> Returns true, if all elements of the Boolean input vector are true. </td> </tr>
<tr><td valign=\"top\"> anyTrue </td>
    <td valign=\"top\"> Returns true, if at least on element of the Boolean input vector is true. </td> </tr>
<tr><td valign=\"top\"> oneTrue </td>
    <td valign=\"top\"> Returns true, if exactly one element of the Boolean input vector is true. </td> </tr>
<tr><td valign=\"top\"> firstTrueIndex </td>
    <td valign=\"top\"> Returns the index of the first element of the Boolean vector that
                      is true and returns 0, if no element is true </td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Icons.</b></td></tr>
<tr><td valign=\"top\"> Information<br>
                      Contact<br>
                      ReleaseNotes<br>
                      References<br>
                      ExamplesPackage<br>
                      Example<br>
                      Package<br>
                      BasesPackage<br>
                      VariantsPackage<br>
                      InterfacesPackage<br>
                      SourcesPackage<br>
                      SensorsPackage<br>
                      MaterialPropertiesPackage<br>
                      MaterialProperty </td>
    <td valign=\"top\"> New icons to get a unified view on different categories
                      of packages. </td> </tr>

<tr><td colspan=\"2\"><b>Modelica.SIunits.</b></td></tr>
<tr><td valign=\"top\"> ComplexCurrent<br>
                      ComplexCurrentSlope<br>
                      ComplexCurrentDensity<br>
                      ComplexElectricPotential<br>
                      ComplexPotentialDifference<br>
                      ComplexVoltage<br>
                      ComplexVoltageSlope<br>
                      ComplexElectricFieldStrength<br>
                      ComplexElectricFluxDensity<br>
                      ComplexElectricFlux<br>
                      ComplexMagneticFieldStrength<br>
                      ComplexMagneticPotential<br>
                      ComplexMagneticPotentialDifference<br>
                      ComplexMagnetomotiveForce<br>
                      ComplexMagneticFluxDensity<br>
                      ComplexMagneticFlux<br>
                      ComplexReluctance<br>
                      ComplexImpedance<br>
                      ComplexAdmittance<br>
                      ComplexPower</td>
    <td valign=\"top\"> SIunits to be used in physical models using complex variables, e.g.,<br>
                      <a href=\"modelica://Modelica.Electrical.QuasiStationary\">Modelica.Electrical.QuasiStationary</a>,
                      <a href=\"modelica://Modelica.Magnetic.FundamentalWave\">Modelica.Magnetic.FundamentalWave</a> </td> </tr>
<tr><td valign=\"top\"> ImpulseFlowRate<br>
                      AngularImpulseFlowRate</td>
    <td valign=\"top\"> New SIunits for mechanics.</td> </tr>
</table>

<p><br>
The following <b style=\"color:blue\">existing components</b>
have been <b style=\"color:blue\">improved</b> in a
<b style=\"color:blue\">backward compatible</b> way:
</p>

<table border=\"1\" cellspacing=0 cellpadding=2 style=\"border-collapse:collapse;\">
<tr><td colspan=\"2\"><b>Modelica.Blocks.Sources.</b></td></tr>
<tr><td valign=\"top\"> Pulse<br>
                      SawTooth </td>
    <td valign=\"top\"> New parameter \"nperiod\" introduced to define the number of periods
                      for the signal type. Default is \"infinite number of periods
                      (nperiods=-1).</td></tr>
<tr><td colspan=\"2\"><b>Modelica.Electrical.</b></td></tr>
<tr><td valign=\"top\"> MultiPhase.*</td>
    <td valign=\"top\"> All dissipative components have now an optional heatPort connector
                      to which the dissipated losses are transported in form of heat.
                       </td> </tr>
<tr><td valign=\"top\"> Machines.*</td>
    <td valign=\"top\"> To all electric machines (asynchronous and synchronous induction machines, DC machines)
                      and transformers loss models have been added (where applicable): <br>
                      Temperature dependent resistances (ohmic losses) <br>
                      Friction losses <br>
                      Brush losses <br>
                      Stray Load losses <br>
                      Core losses (only eddy current losses but no hysteresis losses; not for transformers) <br>
                      As default, temperature dependency and losses are set to zero. <br><br>
                      To all electric machines (asynchronous and synchronous induction machines, DC machines)
                      and transformers conditional thermal ports have been added,
                      to which the dissipated losses are flowing, if activated.
                      The thermal port contains a <a href=\"modelica://Modelica.Thermal.HeatTransfer.Interfaces.HeatPort\">HeatPort</a>
                      for each loss source of the specific machine type. <br><br>
                      To all electric machines (asynchronous and synchronous induction machines, DC machines)
                      a \"powerBalance\" result record has been added, summarizing converted power and losses.
                       </td> </tr>
<tr><td colspan=\"2\"><b>Modelica.Mechanics.</b></td></tr>
<tr><td valign=\"top\"> MultiBody.*<br>
                      Rotational.*<br>
                      Translational.*</td>
    <td valign=\"top\"> All dissipative components in Modelica.Mechanics have now an
                      optional heatPort connector to which the dissipated energy is
                      transported in form of heat.<br>
                      All icons in Modelica.Mechanics are unified according to the
                      Modelica.Blocks library:<br>
                      \"%name\": width: -150 .. 150, height: 40, color: blue<br>
                      other text: height: 30, color: black
                       </td> </tr>
<tr><td colspan=\"2\"><b>Modelica.Mechanics.MultiBody.</b></td></tr>
<tr><td valign=\"top\"> World </td>
    <td valign=\"top\"> Function gravityAcceleration is made replaceable, so that redeclaration
                      yields user-defined gravity fields.
                       </td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Fluid.Valves.</b></td></tr>
<tr><td valign=\"top\"> ValveIncompressible<br>
                      ValveVaporizing<br>
                      ValveCompressible</td>
    <td valign=\"top\"> (a) Optional filtering of opening signal introduced to model
                      the delay time of the opening/closing drive. In this case, an optional
                      leakageOpening can be defined to model leakage flow and/or to
                      improve the numerics in certain situations.
                      (b) Improved regularization of the valve characteristics in some cases
                      so that it is twice differentiable (smooth=2),
                      instead of continuous (smooth=0).</td>
                      </tr>

<tr><td colspan=\"2\"><b>Modelica.Fluid.Sources.</b></td></tr>
<tr><td valign=\"top\"> FixedBoundary<br>
                      Boundary_pT<br>
                      Boundary_ph</td>
    <td valign=\"top\"> Changed the implementation so that no non-linear algebraic
                      equation system occurs, if the given variables (e.g. p,T,X) do
                      not correspond to the medium states (e.g. p,h,X). This is
                      achieved by using appropriate \"setState_xxx\" calls to compute the
                      medium state from the given variables. If a nonlinear equation
                      system occurs, it is solved by a specialized handler inside the
                      setState_xxx(..) function, but in the model this equation system is
                      not visible.</td>
                      </tr>

<tr><td colspan=\"2\"><b>Modelica.Media.Interfaces.</b></td></tr>
<tr><td valign=\"top\"> PartialMedium </td>
    <td valign=\"top\"> The min/max values of types SpecificEnthalpy, SpecificEntropy,
                      SpecificHeatCapacity increased, due to reported user problems.<br>
                      New constant C_nominal introduced to provide nominal values for
                      trace substances (utilized in Modelica.Fluid to avoid numerical problems;
                      this fixes ticket
                      <a href=\"https://trac.modelica.org/Modelica/ticket/393\">#393</a>).</td>
                      </tr>

<tr><td colspan=\"2\"><b>Modelica.Thermal.</b></td></tr>
<tr><td valign=\"top\"> HeatTransfer.*</td>
    <td valign=\"top\"> All icons are unified according to the
                      Modelica.Blocks library:<br>
                      \"%name\": width: -150 .. 150, height: 40, color: blue<br>
                      other text: height: 30, color: black
                       </td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Math.Matrices</b></td></tr>
<tr><td valign=\"top\"> QR </td>
    <td valign=\"top\"> A Boolean input \"pivoting\" has been added (now QR(A, pivoting)) to provide QR-decomposition without pivoting (QR(A, false)). Default is pivoting=true.</td> </tr>
</table>

<p><br>
The following <b style=\"color:red\">critical errors</b> have been fixed (i.e., errors
that can lead to wrong simulation results):
</p>

<table border=\"1\" cellspacing=0 cellpadding=2 style=\"border-collapse:collapse;\">
<tr><td colspan=\"2\"><b>Modelica.Electrical.Digital.Delay.</b></td></tr>
<tr><td valign=\"top\"> InertialDelaySensitive </td>
    <td valign=\"top\"> In order to decide whether the rising delay (tLH) or
                      the falling delay (tHL) is used, the \"previous\" value of the
                      output y has to be used and not the \"previous\" value of the
                      input x (delayType = delayTable[y_old, x] and not
                      delayType = delayTable[x_old, x]). This has been corrected.</td></tr>
<tr><td colspan=\"2\"><b>Modelica.Mechanics.MultiBody.Parts.</b></td></tr>
<tr><td valign=\"top\"> BodyBox<br>
                      BodyCylinder </td>
    <td valign=\"top\"> Fixes ticket
                      <a href=\"https://trac.modelica.org/Modelica/ticket/373\">#373</a>:
                      The \"Center of Mass\" was calculated as normalize(r)*length/2. This is
                      only correct if the box/cylinder is attached between frame_a and frame_b.
                      If this is not the case, the calculation is wrong.
                      The has been fixed by using the correct formula:<br>
                      r_shape + normalize(lengthDirection)*length/2</td></tr>
<tr><td valign=\"top\"> BodyShape<br>
                      BodyBox<br>
                      BodyCylinder </td>
    <td valign=\"top\"> Fixes ticket
                      <a href=\"https://trac.modelica.org/Modelica/ticket/300\">#300</a>:
                      If parameter enforceStates=true, an error occurred.
                      This has been fixed.</td></tr>
<tr><td colspan=\"2\"><b>Modelica.Mechanics.Rotational.Components.</b></td></tr>
<tr><td valign=\"top\"> LossyGear</td>
    <td valign=\"top\"> In cases where the driving flange is not obvious, the component could
                      lead to a non-convergent event iteration. This has been fixed
                      (a detailed description is provided in ticket
                      <a href=\"https://trac.modelica.org/Modelica/ticket/108\">#108</a>
                      and in the
                      <a href=\"modelica://Modelica/Resources/Documentation/Mechanics/Lossy-Gear-Bug_Solution.pdf\">attachment</a>
                      of this ticket).</td> </tr>

<tr><td valign=\"top\"> Gearbox</td>
    <td valign=\"top\"> If useSupport=false, the support flange of the internal LossyGear
                      model was connected to the (disabled) support connector. As a result, the
                      LossyGear was \"free floating\". This has been corrected.</td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Fluid.Pipes.</b></td></tr>
<tr><td valign=\"top\"> DynamicPipe</td>
    <td valign=\"top\"> Bug fix for dynamic mass, energy and momentum balances
                      for pipes with nParallel&gt;1.</td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Fluid.Pipes.BaseClasses.HeatTransfer.</b></td></tr>
<tr><td valign=\"top\"> PartialPipeFlowHeatTransfer</td>
    <td valign=\"top\"> Calculation of Reynolds numbers for the heat transfer through
                      walls corrected, if nParallel&gt;1.
                      This partial model is used by LocalPipeFlowHeatTransfer
                      for laminar and turbulent forced convection in pipes.</td> </tr>
<tr><td colspan=\"2\"><b>Modelica.Media.Interfaces.PartialLinearFluid</b></td></tr>
<tr><td valign=\"top\"> setState_psX</td>
    <td valign=\"top\"> Sign error fixed.</td> </tr>
<tr><td colspan=\"2\"><b>Modelica.Media.CompressibleLiquids.</b></td></tr>
<tr><td valign=\"top\"> LinearColdWater</td>
    <td valign=\"top\"> Fixed wrong values for thermal conductivity and viscosity.</td> </tr>

</table>

<p><br>
The following <b style=\"color:red\">uncritical errors</b> have been fixed (i.e., errors
that do <b style=\"color:red\">not</b> lead to wrong simulation results, but, e.g.,
units are wrong or errors in documentation):
</p>
<table border=\"1\" cellspacing=0 cellpadding=2 style=\"border-collapse:collapse;\">
<tr><td colspan=\"2\"><b>Modelica.Math.Matrices.LAPACK</b></td></tr>
<tr><td valign=\"top\"> dgesv_vec<br>
                        dgesv<br>
                        dgetrs<br>
                        dgetrf<br>
                        dgetrs_vec<br>
                        dgetri<br>
                        dgeqpf<br>
                        dorgqr<br>
                        dgesvx<br>
                        dtrsyl</td>
    <td valign=\"top\"> Integer inputs to specify leading dimensions of matrices have got a lower bound 1 (e.g., lda=max(1,n))
                      to avoid incorrect values (e.g., lda=0) in the case of empty matrices.<br>
                      The Integer variable \"info\" to indicate the successful call of a LAPACK routine has been converted to an output where it had been a protected variable. </td> </tr>
</table>

<p><br>
The following
<a href=\"http://trac.modelica.org/Modelica\">trac tickets</a>
have been fixed:
</p>

<table border=\"1\" cellspacing=0 cellpadding=2 style=\"border-collapse:collapse;\">
<tr><td colspan=\"2\"><b>Modelica</b></td></tr>
<tr><td valign=\"top\">
    <a href=\"https://trac.modelica.org/Modelica/ticket/155\">#155</a></td>
    <td valign=\"top\">Wrong usage of \"fillColor\" and \"fillPattern\" annotations for lines</td>
</tr>

<tr><td valign=\"top\">
    <a href=\"https://trac.modelica.org/Modelica/ticket/211\">#211</a></td>
    <td valign=\"top\">Undefined function realString used in MSL</td>
</tr>

<tr><td valign=\"top\">
    <a href=\"https://trac.modelica.org/Modelica/ticket/216\">#216</a></td>
    <td valign=\"top\">Make MSL version 3.2 more Modelica 3.1 conform</td>
</tr>

<tr><td valign=\"top\">
    <a href=\"https://trac.modelica.org/Modelica/ticket/218\">#218</a></td>
    <td valign=\"top\">Replace `Modelica://`-URIs by `modelica://`-URIs</td>
</tr>

<tr><td valign=\"top\">
    <a href=\"https://trac.modelica.org/Modelica/ticket/271\">#271</a></td>
    <td valign=\"top\">Documentation URI errors in MSL 3.1</td>
</tr>

<tr><td valign=\"top\">
    <a href=\"https://trac.modelica.org/Modelica/ticket/292\">#292</a></td>
    <td valign=\"top\">Remove empty \"\" annotations\"</td>
</tr>
<tr><td valign=\"top\">
    <a href=\"https://trac.modelica.org/Modelica/ticket/294\">#294</a></td>
    <td valign=\"top\">Typo 'w.r.t' --> 'w.r.t.'</td>
</tr>

<tr><td valign=\"top\">
    <a href=\"https://trac.modelica.org/Modelica/ticket/296\">#296</a></td>
    <td valign=\"top\">Unify disclaimer message and improve bad style \"here\" links</td>
</tr>

<tr><td valign=\"top\">
    <a href=\"https://trac.modelica.org/Modelica/ticket/333\">#333</a></td>
    <td valign=\"top\">Fix real number formats of the form `.[0-9]+`</td>
</tr>

<tr><td valign=\"top\">
    <a href=\"https://trac.modelica.org/Modelica/ticket/347\">#347</a></td>
    <td valign=\"top\">invalid URI in MSL 3.2</td>
</tr>

<tr><td valign=\"top\">
    <a href=\"https://trac.modelica.org/Modelica/ticket/355\">#355</a></td>
    <td valign=\"top\">Non-standard annotations</td>
</tr>

<tr><td colspan=\"2\"><br><b>Modelica.Blocks</b></td></tr>

<tr><td valign=\"top\">
    <a href=\"https://trac.modelica.org/Modelica/ticket/227\">#227</a></td>
    <td valign=\"top\">Enhance unit deduction functionality by adding 'unit=\"1\"' to some blocks\"</td>
</tr>

<tr><td valign=\"top\">
    <a href=\"https://trac.modelica.org/Modelica/ticket/349\">#349</a></td>
    <td valign=\"top\">Incorrect annotation in Blocks/Continuous.mo</td>
</tr>

<tr><td valign=\"top\">
    <a href=\"https://trac.modelica.org/Modelica/ticket/374\">#374</a></td>
    <td valign=\"top\">Parameter with no value at all in Modelica.Blocks.Continuous.TransferFunction</td>
</tr>

<tr><td colspan=\"2\"><br><b>Modelica.Constants</b></td></tr>
<tr><td valign=\"top\">
    <a href=\"https://trac.modelica.org/Modelica/ticket/356\">#356</a></td>
    <td valign=\"top\">Add Euler-Mascheroni constant to Modelica.Constants</td>
</tr>

<tr><td colspan=\"2\"><br><b>Modelica.Electrical.Analog</b></td></tr>

<tr><td valign=\"top\">
    <a href=\"https://trac.modelica.org/Modelica/ticket/346\">#346</a></td>
    <td valign=\"top\">Multiple text in Modelica.Electrical.Analog.Basic.Conductor</td>
</tr>

<tr><td valign=\"top\">
    <a href=\"https://trac.modelica.org/Modelica/ticket/363\">#363</a></td>
    <td valign=\"top\">Mixture of Real and Integer in index expressions in Modelica.Electrical.Analog.Lines</td>
</tr>

<tr><td valign=\"top\">
    <a href=\"https://trac.modelica.org/Modelica/ticket/384\">#384</a></td>
    <td valign=\"top\">Incomplete annotations in some examples</td>
</tr>

<tr><td valign=\"top\">
    <a href=\"https://trac.modelica.org/Modelica/ticket/396\">#396</a></td>
    <td valign=\"top\">Bug in Modelica.Electrical.Analog.Ideal.ControlledIdealIntermediateSwitch</td>
</tr>

<tr><td colspan=\"2\"><br><b>Modelica.Machines</b></td></tr>

<tr><td valign=\"top\">
    <a href=\"https://trac.modelica.org/Modelica/ticket/276\">#276</a></td>
    <td valign=\"top\">Improve/fix documentation of Modelica.Electrical.Machines</td>
</tr>

<tr><td valign=\"top\">
    <a href=\"https://trac.modelica.org/Modelica/ticket/288\">#288</a></td>
    <td valign=\"top\">Describe thermal concept of machines</td>
</tr>

<tr><td valign=\"top\">
    <a href=\"https://trac.modelica.org/Modelica/ticket/301\">#301</a></td>
    <td valign=\"top\">Documentation of Electrical.Machines.Examples needs update</td>
</tr>

<tr><td valign=\"top\">
    <a href=\"https://trac.modelica.org/Modelica/ticket/306\">#306</a></td>
    <td valign=\"top\">Merge content of `Modelica.Electrical.Machines.Icons` into `Modelica.Icons`</td>
</tr>

<tr><td valign=\"top\">
    <a href=\"https://trac.modelica.org/Modelica/ticket/362\">#362</a></td>
    <td valign=\"top\">Incomplete example model for DC machines</td>
</tr>

<tr><td valign=\"top\">
    <a href=\"https://trac.modelica.org/Modelica/ticket/375\">#375</a></td>
    <td valign=\"top\">Strangeness with final parameters with no value but a start value</td>
</tr>

<tr><td colspan=\"2\"><br><b>Modelica.Electrical.MultiPhase</b></td></tr>
<tr><td valign=\"top\">
    <a href=\"https://trac.modelica.org/Modelica/ticket/173\">#173</a></td>
    <td valign=\"top\">m-phase mutual inductor</td>
</tr>

<tr><td valign=\"top\">
    <a href=\"https://trac.modelica.org/Modelica/ticket/200\">#200</a></td>
    <td valign=\"top\">adjust Multiphase to Analog</td>
</tr>

<tr><td valign=\"top\">
    <a href=\"https://trac.modelica.org/Modelica/ticket/277\">#277</a></td>
    <td valign=\"top\">Improve/fix documentation of Modelica.Electrical.Multiphase</td>
</tr>

<tr><td valign=\"top\">
    <a href=\"https://trac.modelica.org/Modelica/ticket/352\">#352</a></td>
    <td valign=\"top\">Odd annotation in Modelica.Electrical.MultiPhase.Sources.SignalVoltage</td>
</tr>

<tr><td colspan=\"2\"><br><b>Modelica.Fluid</b></td></tr>

<tr><td valign=\"top\">
    <a href=\"https://trac.modelica.org/Modelica/ticket/215\">#215</a></td>
    <td valign=\"top\">Bug in Modelica.Fluid.Pipes.DynamicPipe</td>
</tr>

<tr><td valign=\"top\">
    <a href=\"https://trac.modelica.org/Modelica/ticket/219\">#219</a></td>
    <td valign=\"top\">Fluid.Examples.HeatExchanger: Heat transfer is switched off and cannot be enabled</td>
</tr>

<tr><td colspan=\"2\"><br><b>Modelica.Math</b></td></tr>

<tr><td valign=\"top\">
    <a href=\"https://trac.modelica.org/Modelica/ticket/348\">#348</a></td>
    <td valign=\"top\">Small error in documentation</td>
</tr>

<tr><td valign=\"top\">
    <a href=\"https://trac.modelica.org/Modelica/ticket/371\">#371</a></td>
    <td valign=\"top\">Modelica.Math functions declared as \"C\" not \"builtin\"\"</td>
</tr>

<tr><td colspan=\"2\"><br><b>Modelica.Mechanics.MultiBody</b></td></tr>

<tr><td valign=\"top\">
    <a href=\"https://trac.modelica.org/Modelica/ticket/50\">#50</a></td>
    <td valign=\"top\">Error in LineForce handling of potential root</td>
</tr>

<tr><td valign=\"top\">
    <a href=\"https://trac.modelica.org/Modelica/ticket/71\">#71</a></td>
    <td valign=\"top\">Make MultiBody.World replaceable</td>
</tr>

<tr><td valign=\"top\">
    <a href=\"https://trac.modelica.org/Modelica/ticket/181\">#181</a></td>
    <td valign=\"top\">3d surface visualisation</td>
</tr>

<tr><td valign=\"top\">
    <a href=\"https://trac.modelica.org/Modelica/ticket/210\">#210</a></td>
    <td valign=\"top\">Description of internal gear wheel missing</td>
</tr>

<tr><td valign=\"top\">
    <a href=\"https://trac.modelica.org/Modelica/ticket/242\">#242</a></td>
    <td valign=\"top\">Missing each qualifier for modifiers in MultiBody.</td>
</tr>

<tr><td valign=\"top\">
    <a href=\"https://trac.modelica.org/Modelica/ticket/251\">#251</a></td>
    <td valign=\"top\">Using enforceStates=true for BodyShape causes errors</td>
</tr>

<tr><td valign=\"top\">
    <a href=\"https://trac.modelica.org/Modelica/ticket/255\">#255</a></td>
    <td valign=\"top\">Error in Revolute's handling of non-normalized axis of rotations</td>
</tr>

<tr><td valign=\"top\">
    <a href=\"https://trac.modelica.org/Modelica/ticket/268\">#268</a></td>
    <td valign=\"top\">Non-standard annotation in MultiBody,Examples.Systems.RobotR3</td>
</tr>

<tr><td valign=\"top\">
    <a href=\"https://trac.modelica.org/Modelica/ticket/269\">#269</a></td>
    <td valign=\"top\">What is the purpose of MultiBody.Examples.Systems.RobotR3.Components.InternalConnectors?</td>
</tr>

<tr><td valign=\"top\">
    <a href=\"https://trac.modelica.org/Modelica/ticket/272\">#272</a></td>
    <td valign=\"top\">Function World.gravityAcceleration should not be protected</td>
</tr>

<tr><td valign=\"top\">
    <a href=\"https://trac.modelica.org/Modelica/ticket/274\">#274</a></td>
    <td valign=\"top\">Convenient and mighty  initialization of frame kinematics</td>
</tr>

<tr><td valign=\"top\">
    <a href=\"https://trac.modelica.org/Modelica/ticket/286\">#286</a></td>
    <td valign=\"top\">Typo in Multibody/Frames.mo</td>
</tr>

<tr><td valign=\"top\">
    <a href=\"https://trac.modelica.org/Modelica/ticket/300\">#300</a></td>
    <td valign=\"top\">enforceStates parameter managed incorrectly in BodyShape, BodyBox, BodyCylinder</td>
</tr>

<tr><td valign=\"top\">
    <a href=\"https://trac.modelica.org/Modelica/ticket/320\">#320</a></td>
    <td valign=\"top\">Replace non-standard annotation by `showStartAttribute`</td>
</tr>

<tr><td valign=\"top\">
    <a href=\"https://trac.modelica.org/Modelica/ticket/373\">#373</a></td>
    <td valign=\"top\">Error in Modelica Mechanics</td>
</tr>

<tr><td valign=\"top\">
    <a href=\"https://trac.modelica.org/Modelica/ticket/389\">#389</a></td>
    <td valign=\"top\">Shape.rxvisobj wrongly referenced in Arrow/DoubleArrow</td>
</tr>

<tr><td colspan=\"2\"><br><b>Modelica.Mechanics.Rotational</b></td></tr>

<tr><td valign=\"top\">
    <a href=\"https://trac.modelica.org/Modelica/ticket/108\">#108</a></td>
    <td valign=\"top\">Problem with model \"Lossy Gear\" and approach to a solution</td>
</tr>

<tr><td valign=\"top\">
    <a href=\"https://trac.modelica.org/Modelica/ticket/278\">#278</a></td>
    <td valign=\"top\">Improve/fix documentation of Modelica.Mechanics.Rotational</td>
</tr>

<tr><td valign=\"top\">
    <a href=\"https://trac.modelica.org/Modelica/ticket/381\">#381</a></td>
    <td valign=\"top\">Bug in Modelica.Mechanics.Rotational.Gearbox</td>
</tr>

<tr><td colspan=\"2\"><br><b>Modelica.Mechanics.Translational</b></td></tr>

<tr><td valign=\"top\">
    <a href=\"https://trac.modelica.org/Modelica/ticket/279\">#279</a></td>
    <td valign=\"top\">Improve/fix documentation of Modelica.Mechanics.Translational</td>
</tr>

<tr><td valign=\"top\">
    <a href=\"https://trac.modelica.org/Modelica/ticket/310\">#310</a></td>
    <td valign=\"top\">Erroneous image links in `Modelica.Mechanics.Translational`</td>
</tr>

<tr><td colspan=\"2\"><br><b>Modelica.Media</b></td></tr>
<tr><td valign=\"top\">
    <a href=\"https://trac.modelica.org/Modelica/ticket/72\">#72</a></td>
    <td valign=\"top\">PartialMedium functions not provided for all media in  Modelica.Media</td>
</tr>

<tr><td valign=\"top\">
    <a href=\"https://trac.modelica.org/Modelica/ticket/217\">#217</a></td>
    <td valign=\"top\">Missing image file Air.png</td>
</tr>

<tr><td valign=\"top\">
    <a href=\"https://trac.modelica.org/Modelica/ticket/224\">#224</a></td>
    <td valign=\"top\">dpT calculation in waterBaseProp_dT</td>
</tr>

<tr><td valign=\"top\">
    <a href=\"https://trac.modelica.org/Modelica/ticket/393\">#393</a></td>
    <td valign=\"top\">Provide C_nominal in Modelica.Media to allow propagating
                     value and avoid wrong numerical results</td>
</tr>

<tr><td colspan=\"2\"><br><b>Modelica.StateGraph</b></td></tr>

<tr><td valign=\"top\">
    <a href=\"https://trac.modelica.org/Modelica/ticket/206\">#206</a></td>
    <td valign=\"top\">Syntax error in StateGraph.mo</td>
</tr>

<tr><td valign=\"top\">
    <a href=\"https://trac.modelica.org/Modelica/ticket/261\">#261</a></td>
    <td valign=\"top\">Modelica.StateGraph should mention the availability of Modelica_StateGraph2</td>
</tr>

<tr><td valign=\"top\">
    <a href=\"https://trac.modelica.org/Modelica/ticket/354\">#354</a></td>
    <td valign=\"top\">Bad annotation in Modelica.StateGraph.Temporary.NumericValue</td>
</tr>

<tr><td colspan=\"2\"><br><b>Modelica.Thermal.FluidHeatFlow</b></td></tr>

<tr><td valign=\"top\">
    <a href=\"https://trac.modelica.org/Modelica/ticket/280\">#280</a></td>
    <td valign=\"top\">Improve/fix documentation of Modelica.Thermal.FluidHeatFlow</td>
</tr>

<tr><td colspan=\"2\"><br><b>Modelica.Thermal.HeatTransfer</b></td></tr>

<tr><td valign=\"top\">
    <a href=\"https://trac.modelica.org/Modelica/ticket/281\">#281</a></td>
    <td valign=\"top\">Improve/fix documentation of Modelica.Thermal.HeatTransfer</td>
</tr>

<tr><td colspan=\"2\"><br><b>Modelica.UsersGuide</b></td></tr>

<tr><td valign=\"top\">
    <a href=\"https://trac.modelica.org/Modelica/ticket/198\">#198</a></td>
    <td valign=\"top\">Name of components in MSL not according to naming conventions</td>
</tr>

<tr><td valign=\"top\">
    <a href=\"https://trac.modelica.org/Modelica/ticket/204\">#204</a></td>
    <td valign=\"top\">Minor correction to User's Guide's section on version management</td>
</tr>

<tr><td valign=\"top\">
    <a href=\"https://trac.modelica.org/Modelica/ticket/244\">#244</a></td>
    <td valign=\"top\">Update the contacts section of the User's Guide</td>
</tr>

<tr><td valign=\"top\">
    <a href=\"https://trac.modelica.org/Modelica/ticket/267\">#267</a></td>
    <td valign=\"top\">MSL-Documentation: Shouldn't equations be numbered on the right hand side?</td>
</tr>

<tr><td valign=\"top\">
    <a href=\"https://trac.modelica.org/Modelica/ticket/299\">#299</a></td>
    <td valign=\"top\">SVN keyword expansion messed up the User's guide section on version management</td>
</tr>

<tr><td colspan=\"2\"><br><b>Modelica.Utilities</b></td></tr>

<tr><td valign=\"top\">
    <a href=\"https://trac.modelica.org/Modelica/ticket/249\">#249</a></td>
    <td valign=\"top\">Documentation error in ModelicaUtilities.h</td>
</tr>

<tr><td colspan=\"2\"><br><b>ModelicaServices</b></td></tr>

<tr><td valign=\"top\">
    <a href=\"https://trac.modelica.org/Modelica/ticket/248\">#248</a></td>
    <td valign=\"top\">No uses statement on ModelicaServices in MSL 3.1</td>
</tr>

</table>

<p>
Note:
</p>
<ul>
<li> Libraries
     <a href=\"https://www.modelica.org/libraries/Modelica_FundamentalWave\">Modelica_FundamentalWave</a>
     and
     <a href=\"https://www.modelica.org/libraries/Modelica_QuasiStationary\">Modelica_QuasiStationary</a>
     are included in this version in an improved form.</li>
<li> From library
     <a href=\"https://www.modelica.org/libraries/Modelica_LinearSystems2\">Modelica_LinearSystems2</a>,
     the sublibraries
     Math.Complex, Math.Vectors and Math.Matrices are included in this version
     in an improved form.</li>
<li> From library
     <a href=\"https://www.modelica.org/libraries/Modelica_StateGraph2\">Modelica_StateGraph2</a>,
     the sublibrary Blocks is included in this version in an improved form.</li>
</ul>
</html>"));
      end Version_3_2;

      class Version_3_1 "Version 3.1 (August 14, 2009)"
        extends Modelica.Icons.ReleaseNotes;
        annotation(
          Documentation(info = "<html>

<p>
Version 3.1 is backward compatible to version 3.0 and 3.0.1,
i.e., models developed with version 3.0 or 3.0.1 will work without any
changes also with version 3.1.
</p>

<p>
Version 3.1 is slightly based on the Modelica Specification 3.1. It uses
the following new language elements (compared to Modelica Specification 3.0):
</p>

<ul>
<li> Prefix <u>stream</u> and built-in operators <u>inStream(..)</u>
     and <u>actualStream(..)</u> in Modelica.Fluid.</li>
<li> Annotation <u>connectorSizing</u> in Modelica.Fluid.</li>
<li> Annotation <u>inverse</u> in Modelica.Media.</li>
<li> Annotations <u>versionBuild</u>, <u>dateModified</u>,
     <u>revisionId</u> at the root level annotation of package Modelica,
     to improve the version handling.</li>
<li> Modifiers can be used in connectors instances (so balanced models
     are less restrictive). This allowed to make the implementation
     of conditional connectors (support and heatPort) in the Rotational,
     Translational and Electrical libraries simpler.</li>
</ul>

<p>
The following <b style=\"color:blue\">new libraries</b> have been added:
</p>
<table border=\"1\" cellspacing=0 cellpadding=2>
<tr><td valign=\"top\"><a href=\"modelica://Modelica.Fluid\">Modelica.Fluid</a></td>
    <td valign=\"top\">
     Components to model 1-dim. thermo-fluid flow in networks of vessels, pipes,
     fluid machines, valves and fittings. All media from the
     Modelica.Media library can be used (so incompressible or compressible,
     single or multiple substance, one or two phase medium).
    The library is using the stream-concept from Modelica Specification 3.1.
    </td>

<tr><td valign=\"top\"><a href=\"modelica://Modelica.Magnetic.FluxTubes\">Modelica.Magnetic.FluxTubes</a></td>
    <td valign=\"top\">
     Components to model magnetic devices based on the magnetic flux tubes concepts.
     Especially to model
     electro-magnetic actuators. Nonlinear shape, force, leakage, and
     Material models. Material data for steel, electric sheet, pure iron,
     Cobalt iron, Nickel iron, NdFeB, Sm2Co17, and more.
    </td>

<tr><td valign=\"top\"><a href=\"modelica://ModelicaServices\">ModelicaServices</a></td>
    <td valign=\"top\">
     New top level package that shall contain functions and models to be used in the
     Modelica Standard Library that requires a tool specific implementation.
     ModelicaServices is then used in the Modelica package.
     In this first version, the 3-dim. animation with model Modelica.Mechanics.MultiBody.Visualizers.Advanced.Shape
     was moved to ModelicaServices. Tool vendors can now provide their own implementation
     of the animation.
    </td>
</table>

<p><br>
The following <b style=\"color:blue\">new components</b> have been added
to <b style=\"color:blue\">existing</b> libraries:
</p>

<table border=\"1\" cellspacing=0 cellpadding=2 style=\"border-collapse:collapse;\">
<tr><td colspan=\"2\"><b>Modelica.</b></td></tr>
<tr><td valign=\"top\"> versionBuild<br>versionDate<br>dateModified<br>revisionId </td>
    <td valign=\"top\"> New annotations from Modelica 3.1 for version handling added.</td> </tr>

<tr><td colspan=\"2\"><b>Modelica.UsersGuide.ReleaseNotes.</b></td></tr>
<tr><td valign=\"top\"> VersionManagement </td>
    <td valign=\"top\"> Copied from info layer of previous ReleaseNotes (to make it more
                      visible) and adapted it to the new possibilities in
                      Modelica Specification 3.1.</td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Blocks.Math.</b></td></tr>
<tr><td valign=\"top\"> RectangularToPolar<br>
                      PolarToRectangular </td>
    <td valign=\"top\"> New blocks to convert between rectangular and polar form
                      of space phasors. </td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Blocks.Routing.</b></td></tr>
<tr><td valign=\"top\"> Replicator </td>
    <td valign=\"top\"> New block to replicate an input signal to many output signals. </td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Electrical.Analog.Examples.</b></td></tr>
<tr><td valign=\"top\"> AmplifierWithOpAmpDetailed<br>
                      HeatingResistor<br>
                      CompareTransformers<br>
                      OvervoltageProtection<br>
                      ControlledSwitchWithArc<br>
                      SwitchWithArc<br>
                      ThyristorBehaviourTest</td>
    <td valign=\"top\"> New examples to demonstrate the usage of new components. </td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Electrical.Analog.Basic.</b></td></tr>
<tr><td valign=\"top\"> OpAmpDetailed<br>
                      TranslationalEMF<br>
                      M_Transformer</td>
    <td valign=\"top\"> New detailed model of an operational amplifier. <br>
                      New electromotoric force from electrical energy into mechanical translational energy.<br>
                      Generic transformer with choosable number of inductors</td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Electrical.Analog.Ideal.</b></td></tr>
<tr><td valign=\"top\"> OpenerWithArc<br>
                      CloserWithArc<br>
                      ControlledOpenerWithArc<br>
                      ControlledCloserWithArc</td>
    <td valign=\"top\"> New switches with simple arc model. </td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Electrical.Analog.Interfaces.</b></td></tr>
<tr><td valign=\"top\"> ConditionalHeatPort</td>
    <td valign=\"top\"> New partial model to add a conditional HeatPort to
                      an electrical component. </td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Electrical.Analog.Lines.</b></td></tr>
<tr><td valign=\"top\"> M_Oline</td>
    <td valign=\"top\"> New multiple line model, both the number of lines and the number of segments choosable. </td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Electrical.Analog.Semiconductors.</b></td></tr>
<tr><td valign=\"top\"> ZDiode<br>Thyristor</td>
    <td valign=\"top\"> Zener Diode with 3 working areas and simple thyristor model. </td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Electrical.MultiPhase.Ideal.</b></td></tr>
<tr><td valign=\"top\"> OpenerWithArc<br>CloserWithArc</td>
    <td valign=\"top\"> New switches with simple arc model (as in Modelica.Electrical.Analog.Ideal. </td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Mechanics.MultiBody.Examples.Elementary.</b></td></tr>
<tr><td valign=\"top\"> RollingWheel<br>
                      RollingWheelSetDriving<br>
                      RollingWheelSetPulling</td>
    <td valign=\"top\"> New examples to demonstrate the usage of new components.</td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Mechanics.MultiBody.Joints.</b></td></tr>
<tr><td valign=\"top\"> RollingWheel<br>
                      RollingWheelSet</td>
    <td valign=\"top\"> New joints (no mass, no inertia) that describe an
                      ideal rolling wheel and a ideal rolling wheel set consisting
                      of two wheels rolling on the plane z=0.</td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Mechanics.MultiBody.Parts.</b></td></tr>
<tr><td valign=\"top\"> RollingWheel<br>
                      RollingWheelSet</td>
    <td valign=\"top\"> New ideal rolling wheel and ideal rolling wheel set consisting
                      of two wheels rolling on the plane z=0.</td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Mechanics.MultiBody.Visualizers.</b></td></tr>
<tr><td valign=\"top\"> Ground</td>
    <td valign=\"top\"> New model to visualize the ground (box at z=0).</td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Mechanics.Rotational.Interfaces.</b></td></tr>
<tr><td valign=\"top\"> PartialElementaryOneFlangeAndSupport2<br>
                      PartialElementaryTwoFlangesAndSupport2</td>
    <td valign=\"top\"> New partial model with one and two flanges and the support flange
                      with a much simpler implementation as previously.</td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Mechanics.Translational.Interfaces.</b></td></tr>
<tr><td valign=\"top\"> PartialElementaryOneFlangeAndSupport2<br>
                      PartialElementaryTwoFlangesAndSupport2</td>
    <td valign=\"top\"> New partial model with one and two flanges and the support flange
                      with a much simpler implementation as previously.</td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Media.IdealGases.Common.MixtureGasNasa.</b></td></tr>
<tr><td valign=\"top\"> setSmoothState</td>
    <td valign=\"top\"> Return thermodynamic state so that it smoothly approximates:
                      if x &gt; 0 then state_a else state_b.</td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Utilities.Internal.</b></td></tr>
<tr><td valign=\"top\"> PartialModelicaServices</td>
    <td valign=\"top\"> New package containing the interface description of
                      models and functions that require a tool dependent
                      implementation (currently only \"Shape\" for 3-dim. animation,
                      but will be extended in the future)</td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Thermal.HeatTransfer.Components.</b></td></tr>
<tr><td valign=\"top\"> ThermalCollector</td>
    <td valign=\"top\"> New auxiliary model to collect the heat flows
                      from m heatports to a single heatport;
                      useful for multiphase resistors (with heatports)
                      as a junction of the m heatports.</td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Icons.</b></td></tr>
<tr><td valign=\"top\"> VariantLibrary<br>
                      BaseClassLibrary<br>
                      ObsoleteModel</td>
    <td valign=\"top\"> New icons (VariantLibrary and BaseClassLibrary have been moved
                      from Modelica_Fluid.Icons to this place).</td> </tr>

<tr><td colspan=\"2\"><b>Modelica.SIunits.</b></td></tr>
<tr><td valign=\"top\"> ElectricalForceConstant </td>
    <td valign=\"top\"> New type added (#190).</td> </tr>

<tr><td colspan=\"2\"><b>Modelica.SIunits.Conversions.</b></td></tr>
<tr><td valign=\"top\"> from_Hz<br>
                      to_Hz</td>
    <td valign=\"top\"> New functions to convert between frequency [Hz] and
                      angular velocity [1/s]. (#156) </td> </tr>

</table>

<p><br>
The following <b style=\"color:blue\">existing components</b>
have been <b style=\"color:blue\">improved</b> in a
<b style=\"color:blue\">backward compatible</b> way:
</p>

<table border=\"1\" cellspacing=0 cellpadding=2 style=\"border-collapse:collapse;\">
<tr><td colspan=\"2\"><b>Modelica.</b></td></tr>
<tr><td valign=\"top\"> Blocks<br>Mechanics<br>StateGraph </td>
    <td valign=\"top\"> Provided missing parameter values for examples
                      (these parameters had only start values)</td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Electrical.Analog.Basic</b></td></tr>
<tr><td valign=\"top\"> Resistor, Conductor, VariableResistor, VariableConductor</td>
    <td valign=\"top\"> Conditional heatport added for coupling to thermal network. </td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Electrical.Analog.Ideal</b></td></tr>
<tr><td valign=\"top\"> Thyristors, Switches, IdealDiode</td>
    <td valign=\"top\"> Conditional heatport added for coupling to thermal network. </td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Electrical.Analog.Semiconductors</b></td></tr>
<tr><td valign=\"top\"> Diode, ZDiode, PMOS, NMOS, NPN, PNP</td>
    <td valign=\"top\"> Conditional heatport added for coupling to thermal network. </td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Electrical.MultiPhase.Basic</b></td></tr>
<tr><td valign=\"top\"> Resistor, Conductor, VariableResistor, VariableConductor</td>
    <td valign=\"top\"> Conditional heatport added for coupling to thermal network (as in Modelica.Electrical.Analog). </td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Electrical.MultiPhase.Ideal</b></td></tr>
<tr><td valign=\"top\"> Thyristors, Switches, IdealDiode</td>
    <td valign=\"top\"> Conditional heatport added for coupling to thermal network (as in Modelica.Electrical.Analog). </td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Mechanics.MultiBody.Visualizers.Advanced.</b></td></tr>
<tr><td valign=\"top\"> Shape </td>
    <td valign=\"top\"> New implementation by inheriting from ModelicaServices. This allows a
                      tool vendor to provide its own implementation of Shape. </td> </tr>
<tr><td colspan=\"2\"><b>Modelica.StateGraph.</b></td></tr>
<tr><td valign=\"top\"> Examples </td>
    <td valign=\"top\"> Introduced \"StateGraphRoot\" on the top level of all example models. </td> </tr>

<tr><td colspan=\"2\"><b>Modelica.StateGraph.Interfaces.</b></td></tr>
<tr><td valign=\"top\"> StateGraphRoot<br>PartialCompositeStep<br>CompositeStepState </td>
    <td valign=\"top\"> Replaced the wrong Modelica code \"flow output Real xxx\"
                      by \"Real dummy; flow Real xxx;\".
                      As a side effect, several \"blocks\" had to be changed to \"models\". </td> </tr>
<tr><td valign=\"top\"> PartialStep </td>
    <td valign=\"top\"> Changed model by packing the protected outer connector in to a model.
                      Otherwise, there might be differences in the sign of the flow variable
                      in Modelica 3.0 and 3.1. </td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Utilities.Examples.</b></td></tr>
<tr><td valign=\"top\"> expression </td>
    <td valign=\"top\"> Changed local variable \"operator\" to \"opString\" since \"operator\"
                      is a reserved keyword in Modelica 3.1 </td> </tr>
</table>

<p><br>
The following <b style=\"color:red\">uncritical errors</b> have been fixed (i.e., errors
that do <b style=\"color:red\">not</b> lead to wrong simulation results, but, e.g.,
units are wrong or errors in documentation):
</p>
<table border=\"1\" cellspacing=0 cellpadding=2 style=\"border-collapse:collapse;\">
<tr><td colspan=\"2\"><b>Modelica.</b></td></tr>
<tr><td valign=\"top\"> Many models</td>
    <td valign=\"top\"> Removed wrong usages of annotations fillColor and fillPattern
                      in text annotations (#155, #185).</td> </tr>
<tr><td colspan=\"2\"><b>Modelica.Electrical.Machines</b></td></tr>
<tr><td valign=\"top\"> All machine models</td>
    <td valign=\"top\"> The conditional heatports of the instantiated resistors
                        (which are new in Modelica.Electrical.Analog and Modelica.Electrical.MultiPhase)
                        are finally switched off until a thermal connector design for machines is implemented. </td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Media.Air.MoistAir</b></td></tr>
<tr><td valign=\"top\"> saturationPressureLiquid<br>
                      sublimationPressureIce<br>
                      saturationPressure</td>
          <td valign=\"top\"> For these three functions, an error in the <code>derivative</code> annotation was corrected. However, the effect of
                            this bug was minor, as a Modelica tool was allowed to compute derivatives automatically via
                            the <code>smoothOrder</code> annotation.</td>
</tr>
<tr><td colspan=\"2\"><b>Modelica.Math.Matrices.</b></td></tr>
<tr><td valign=\"top\"> eigenValues</td>
    <td valign=\"top\"> Wrong documentation corrected (#162)</td> </tr>
</table>

</html>"));
      end Version_3_1;

      class Version_3_0_1 "Version 3.0.1 (Jan. 27, 2009)"
        extends Modelica.Icons.ReleaseNotes;
        annotation(
          Documentation(info = "<html>

<p>
This Modelica package is provided under the
<a href=\"modelica://Modelica.UsersGuide.ModelicaLicense2\">Modelica License 2</a>
and no longer under Modelica License 1.1. There are the following reasons
why the Modelica Association changes from Modelica License 1.1 to this
new license text (note, the text below is not a legal interpretation of the
Modelica License 2. In case of a conflict, the language of the license shall prevail):
</p>

<ol>
<li> The rights of licensor and licensee are much more clearly defined. For example:
         <ul>
         <li> The licensed work (Original Work) can be used in unmodified form in
                  open source and commercial software (the licensee cannot change the
                  license and the work must be provided without fees)</li>
         <li> If a model component is copied out of a Modelica package under
                  Modelica License 2 and is modified in order to adapt it to the needs
                  of the modeler, then the result can be licensed under any license
                  (including a commercial license).</li>
         <li> It is practically not possible to change the license of a
                  Modelica package under Modelica License 2 to another license, i.e., a
                  licensee cannot change the license by adding material or changing classes,
                  so the work must remain under Modelica License 2 (to be more precise,
                  if the licensee makes modifications to the Original Work \"which represents,
                  as a whole, an original work of authorship\", he/she can change the license
                  to another license. However, for a Modelica package this would
                  require a lot of changes which is usually unrealistic).</li>
         <li> If an executable is constructed using a Modelica package under
                  Modelica License 2, then this executable can be licensed under any
                  license (including a commercial license).</li>
         </ul>
         We hope that this compromise between open source contributors, commercial
         Modelica environments and Modelica users will motivate even more people to
         provide their Modelica packages freely under the Modelica License 2.<br><br></li>
<li> There are several new provisions that shall make law suites against licensors and licensees more unlikely (so the small risk is further reduced).</li>
</ol>

<p><br>
The following <b style=\"color:blue\">new components</b> have been added
to <b style=\"color:blue\">existing</b> libraries:
</p>

<table border=\"1\" cellspacing=0 cellpadding=2 style=\"border-collapse:collapse;\">
<tr><td colspan=\"2\"><b>Electrical.Analog.Basic.</b></td></tr>
<tr><td valign=\"top\">M_Transformer</td>
          <td valign=\"top\"> Transformer, with the possibility to
        choose the number of inductors. The inductances and the coupled inductances
        can be chosen arbitrarily.</td> </tr>

<tr><td colspan=\"2\"><b>Electrical.Analog.Lines.</b></td></tr>
<tr><td valign=\"top\">M_OLine</td>
          <td valign=\"top\"> Segmented line model that enables the use of
        multiple lines, that means, the number of segments and the number of
        single lines can be chosen by the user. The model allows to investigate
        phenomena at multiple lines like mutual magnetic or capacitive influence.</td> </tr>
<tr><td colspan=\"2\"><b>Mechanics.Translational.Components.Examples.</b></td></tr>
<tr><td valign=\"top\">Brake</td>
          <td valign=\"top\"> Demonstrates the usage of the translational brake component.</td> </tr>
<tr><td colspan=\"2\"><b>Media.Interfaces.PartialMedium.</b></td></tr>
<tr><td valign=\"top\">ThermoStates</td>
          <td valign=\"top\"> Enumeration type for independent variables to identify the independent
                                                variables of the medium (pT, ph, phX, pTX, dTX).<br>
                                                An implementation of this enumeration is provided for every medium.
                                                (This is useful for fluid libraries that do not use the
                                                PartialMedium.BaseProperties model). </td> </tr>
<tr><td valign=\"top\">setSmoothState</td>
          <td valign=\"top\"> Function that returns the thermodynamic state which smoothly approximates:
                                                if x > 0 then state_a else state_b.<br>
                                                (This is useful for pressure drop components in fluid libraries
                                                 where the upstream density and/or viscosity has to be computed
                                                 and these properties should be smooth a zero mass flow rate)<br>
                                                An implementation of this function is provided for every medium.</td> </tr>
<tr><td colspan=\"2\"><b>Media.Common.</b></td></tr>
<tr><td valign=\"top\">smoothStep</td>
          <td valign=\"top\"> Approximation of a general step, such that the characteristic
                                                is continuous and differentiable.</td> </tr>
<tr><td colspan=\"2\"><b>Media.UsersGuide.</b></td></tr>
<tr><td valign=\"top\">Future</td>
          <td valign=\"top\"> Short description of goals and changes of upcoming release of Modelica.Media. </td> </tr>
<tr><td colspan=\"2\"><b>Media.Media.Air.MoistAir.</b></td></tr>
<tr><td valign=\"top\">isentropicExponent</td>
          <td valign=\"top\"> Implemented Missing Function from interface. </td> </tr>
<tr><td valign=\"top\">isentropicEnthalpyApproximation</td>
<td valign=\"top\"> Implemented function that approximates the isentropic enthalpy change.
This is only correct as long as there is no liquid in the stream.</td> </tr>
</table>

<p><br>
The following <b style=\"color:blue\">existing components</b>
have been <b style=\"color:blue\">changed</b> (in a
<b style=\"color:blue\">backward compatible</b> way):
</p>

<table border=\"1\" cellspacing=0 cellpadding=2 style=\"border-collapse:collapse;\">
<tr><td colspan=\"2\"><b>Mechanics.Rotational.Interfaces.</b></td></tr>
<tr><td valign=\"top\"> PartialFriction </td>
          <td valign=\"top\"> Improvement of friction model so that in certain situations
                                                the number of iterations is much smaller.</td>
</tr>

<tr><td colspan=\"2\"><b>Mechanics.Translational.Components.Examples.</b></td></tr>
<tr><td valign=\"top\"> Friction </td>
          <td valign=\"top\"> Added a third variant, where friction is modelled with
                                                the SupportFriction component.</td> </tr>

<tr><td colspan=\"2\"><b>Mechanics.Translational.Components.</b></td></tr>
<tr><td valign=\"top\"> MassWithStopAndFriction </td>
          <td valign=\"top\"> Improvement of friction model so that in certain situations
                                                the number of iterations is much smaller.</td>
</tr>

<tr><td colspan=\"2\"><b>Mechanics.Translational.Interfaces.</b></td></tr>
<tr><td valign=\"top\"> PartialFriction </td>
          <td valign=\"top\"> Improvement of friction model so that in certain situations
                                                the number of iterations is much smaller.</td>
</tr>

<tr><td colspan=\"2\"><b>Media.Examples.</b></td></tr>
<tr><td valign=\"top\"> SimpleLiquidWater <br>
                                                IdealGasH20 <br>
                                                WaterIF97 <br>
                                                MixtureGases <br>
                                                MoistAir </td>
          <td valign=\"top\"> Added equations to test the new setSmoothState(..) functions
                                                including the analytic derivatives of these functions.</td> </tr>

<tr><td colspan=\"2\"><b>Media.Interfaces.PartialLinearFluid.</b></td></tr>
<tr><td valign=\"top\"> setState_pTX <br>
                                                setState_phX <br>
                                                setState_psX <br>
                                                setState_dTX </td>
          <td valign=\"top\"> Rewritten function in one statement so that it is usually inlined.</td> </tr>

<tr><td colspan=\"2\"><b>Media.Interfaces.PartialLinearFluid.</b></td></tr>
<tr><td valign=\"top\"> consistent use of reference_d instead of density(state </td>
          <td valign=\"top\"> Change was done to achieve consistency with analytic inverse functions.</td> </tr>

<tr><td colspan=\"2\"><b>Media.Air.MoistAir.</b></td></tr>
<tr><td valign=\"top\"> T_phX </td>
          <td valign=\"top\"> Interval of nonlinear solver to compute T from p,h,X changed
                                                from 200..6000 to 240 ..400 K.</td> </tr>

</table>

<p><br>
The following <b style=\"color:red\">critical errors</b> have been fixed (i.e., errors
that can lead to wrong simulation results):
</p>

<table border=\"1\" cellspacing=0 cellpadding=2 style=\"border-collapse:collapse;\">
<tr><td colspan=\"2\"><b>Mechanics.MultiBody.Forces</b></td></tr>
<tr><td valign=\"top\"> WorldTorque </td>
          <td valign=\"top\"> Parameter \"ResolveInFrame\" was not propagated and therefore
                                                always the default (resolved in world frame) was used, independently
                                                of the setting of this parameter. </td>
</tr>
<tr><td valign=\"top\"> WorldForceAndTorque </td>
          <td valign=\"top\"> Parameter \"ResolveInFrame\" was not propagated and therefore
                                                always the default (resolved in world frame) was used, independently
                                                of the setting of this parameter.<br>
                                                Furthermore, internally WorldTorque was used instead of
                                                Internal.BasicWorldTorque and therefore the visualization of
                                                worldTorque was performed twice. </td>
</tr>
<tr><td colspan=\"2\"><b>Mechanics.MultiBody.Sensors</b></td></tr>
<tr><td valign=\"top\"> AbsoluteSensor </td>
          <td valign=\"top\"> Velocity, acceleration and angular acceleration were computed
                                                  by differentiating in the resolveInFrame frame. This has been corrected, by
                                                  first transforming the vectors in to the world frame, differentiating here
                                                  and then transforming into resolveInFrame. The parameter in the Advanced menu
                                                  resolveInFrameAfterDifferentiation is then superfluous and was removed . </td>
</tr>
<tr><td valign=\"top\"> AbsoluteVelocity </td>
          <td valign=\"top\"> The velocity was computed
                                                  by differentiating in the resolveInFrame frame. This has been corrected, by
                                                  first transforming the velocity in to the world frame, differentiating here
                                                  and then transforming into resolveInFrame </td>
</tr>
<tr><td valign=\"top\"> RelativeSensor </td>
          <td valign=\"top\"> If resolveInFrame &lt;&gt; frame_resolve and
                                                   resolveInFrameAfterDifferentiation = frame_resolve, a translation
                                                error occurred, since frame_resolve was not enabled in this situation.
                                                This has been corrected.</td>
</tr>
<tr><td valign=\"top\"> RelativeVelocity </td>
          <td valign=\"top\"> The velocity has have been computed
                                                  by differentiating in the resolveInFrame frame. This has been corrected, by
                                                  first transforming the relative position in to frame_a, differentiating here
                                                  and then transforming into resolveInFrame </td>
</tr>
<tr><td valign=\"top\"> TransformRelativeVector </td>
          <td valign=\"top\"> The transformation was wrong, since the parameters frame_r_in and frame_r_out
                                                have not been propagated to the submodel that performs the transformation.
                                                This has been corrected. </td>
</tr>
<tr><td colspan=\"2\"><b>Mechanics.Translational.Components.</b></td></tr>
<tr><td valign=\"top\"> SupportFriction<br>
                                                Brake </td>
          <td valign=\"top\"> The sign of the friction force was wrong and therefore friction accelerated
                                                instead of decelerated. This was fixed. </td>
<tr><td valign=\"top\"> SupportFriction</td>
          <td valign=\"top\"> The component was only correct for fixed support.
                                                This was corrected. </td>
</tr>

<tr><td colspan=\"2\"><b>Media.Interfaces.</b></td></tr>
<tr><td valign=\"top\"> PartialSimpleMedium<br>
                                                PartialSimpleIdealGasMedium </td>
          <td valign=\"top\"> BaseProperties.p was not defined as preferred state and BaseProperties.T was
                                                always defined as preferred state. This has been fixed by
                                                Defining p,T as preferred state if parameter preferredMediumState = true.
                                                This error had the effect that mass m is selected as state instead of p
                                                and if default initialization is used then m=0 could give not the expected
                                                behavior. This means, simulation is not wrong but the numerics is not as good
                                                and if a model relies on default initial values, the result could be not
                                                as expected. </td>
</tr>

</table>

<p><br>
The following <b style=\"color:red\">uncritical errors</b> have been fixed (i.e., errors
that do <b style=\"color:red\">not</b> lead to wrong simulation results, but, e.g.,
units are wrong or errors in documentation):
</p>

<table border=\"1\" cellspacing=0 cellpadding=2 style=\"border-collapse:collapse;\">
<tr><td colspan=\"2\"><b>Blocks.Math.</b></td></tr>
<tr><td valign=\"top\"> InverseBlockConstraint </td>
          <td valign=\"top\"> Changed annotation preserveAspectRatio from true to false.</td>
</tr>

<tr><td colspan=\"2\"><b>Blocks.Sources.</b></td></tr>
<tr><td valign=\"top\"> RealExpression<br>
                                                IntegerExpression<br>
                                                BooleanExpression </td>
          <td valign=\"top\"> Changed annotation preserveAspectRatio from true to false.</td>
</tr>

<tr><td colspan=\"2\"><b>Electrical.Analog.Basic.</b></td></tr>
<tr><td valign=\"top\"> SaturatingInductor</td>
          <td valign=\"top\"> Replaced non-standard \"arctan\" by \"atan\" function.</td>
</tr>

<tr><td colspan=\"2\"><b>Modelica.Electrical.Digital.</b></td></tr>
<tr><td valign=\"top\"> UsersGuide</td>
          <td valign=\"top\"> Removed empty documentation placeholders and added the missing
                                                  release comment for version 1.0.7</td>
</tr>

<tr><td colspan=\"2\"><b>Modelica.Mechanics.Translational.Components.</b></td></tr>
<tr><td valign=\"top\"> MassWithStopAndFriction </td>
          <td valign=\"top\"> Changed usage of reinit(..), in order that it appears
                                                only once for one variable according to the language specification
                                                (if a tool could simulate the model, there is no difference).</td>
</tr>

<tr><td colspan=\"2\"><b>Media.Interfaces.PartialSimpleMedium</b></td></tr>
<tr><td valign=\"top\"> pressure<br>
                                                temperature<br>
                                                density<br>
                                                specificEnthalpy </td>
          <td valign=\"top\"> Missing functions added.</td>
</tr>

</table>

</html>"));
      end Version_3_0_1;

      class Version_3_0 "Version 3.0 (March 1, 2008)"
        extends Modelica.Icons.ReleaseNotes;
        annotation(
          Documentation(info = "<html>
<p>
Version 3.0 is <b>not</b> backward compatible to previous versions.
A conversion script is provided to transform models and libraries
of previous versions to the new version. Therefore, conversion
should be automatic.
</p>

<p>
The following changes are present for the whole library:
</p>

<ul>
<li> In the Modelica language version 3.0, several restrictions have been
         introduced to allow better checking, e.g., models on all levels must be balanced
         (number of equations = number of unknown variables - unknown variables that have
         to be defined when using the component). A few models of the Modelica
         Standard Library did not fulfill these new restrictions and had
         either to be moved to library ObsoleteModelica3 (e.g., Blocks.Math.TwoInputs)
         or had to be differently implemented
         (e.g., Media.Interfaces.PartialMedium.BaseProperties).
         The Modelica Standard Library version 3.0 fulfills all the restrictions of
         the Modelica Language version 3.0.<br>&nbsp;
         </li>

<li> The graphical annotations describing the layout of icon and diagram layer
         are changed from Modelica language version 1 to Modelica language version 3.
         This gives several significant improvements:<br>Especially, the coordinate systems
         of icon and diagram layers are no longer coupled and therefore the size of the
         icon layer can be changed independently of the size of the diagram layer.
         Also it can be defined that the aspect ratio of a component icon is kept when changing
         its size in a model. This flag is set so that all icons of the Modelica
         Standard Library keep its aspect ratios. This is slightly non-backward compatible:
         If the aspect ratio was not kept when using a component from the Modelica
         Standard Library, it is now resized so that the aspect ratio is maintained.<br>&nbsp; </li>

<li> All non-standard annotations removed by:<br>
         (1) Removing the annotation since without effect
                 (e.g., \"__Dymola_experimentSetupOutput\", \"Window\", \"Terminal\" removed).<br>
         (2) Renaming the annotation to a standard name (e.g., \"Hide\" renamed to \"HideResult\").<br>
         (3) Renaming the annotation to a vendor specific name
                 (e.g., \"checkBox\" renamed to \"__Dymola_checkBox\").<br>&nbsp; </li>

<li> All emulated enumerations (defined via packages and constants) have been
         replaced by \"real\" enumerations. User models are automatically correctly
         converted, provided the user models used the package constants previously.
         <b>Existing models that use directly literal values for enumerations, might give in
         some cases wrong results</b> (if the first constant of the emulated enumeration
         had value zero, whereas the first value of an enumeration is one).<br>&nbsp; </li>

<li> The operator \"cardinality\" will be removed in one of the next versions of the
         Modelica language, since it is a reflective operator and its usage significantly
         reduces the possibilities of advanced model checks (e.g., to guarantee that a model
         is \"balanced\", i.e., the number of equations and unknowns is identical,
         for all valid usages of the component). As a preparation for this change, all
         models that contain the \"cardinality(..)\" operator are rewritten: If possible
         the operator is removed. If this is not possible, it is only used in asserts to
         check that, e.g., a connector is connected at least once or is connected exactly
         once. In the next Modelica language version new language elements will be introduced
         to specify such a property check without the cardinality operator. Once these
         language elements are available, the cardinality operator will be removed completely
         from the Modelica Standard Library.<br>
         The changes with respect to the cardinality(..) operator are usually not backward
         compatible. This is the reason for the changes of the
         Rotational and Translational library (see below).<br>&nbsp;</li>

<li> The design of the <b>Rotational</b> and <b>Translational</b> libraries have been changed
         (especially to remove the cardinality(..) operator, see above):
         <ul>
         <li> Components have a <b>useSupport</b> flag to enable or disable a support flange.
                  If the support flange is enabled, it must be connected. If it is disabled, it must
                  not be connected and the component is then internally grounded. The grounding
                  is visualized in the icon.</li>
         <li> The relative angle/distance and the relative speed of all force/torque elements
                  (that need the relative speed) are by default defined with \"StateSelect.prefer\", i.e.,
                  to use these variables as preferred states. This improves the numerics if the
                  absolute angle or the absolute distance are continuously increasing during
                  operation (e.g., driving shaft of the wheels of a car). The effect is that relative
                  angles/distances and speeds are used as states and the size of these variables is
                  limited. Previously, the default was to use the absolute angle/distance
                  and absolute speed of every inertia/mass which has the disadvantage that the absolute
                  angle and or distance are state variables that grow in size continuously.<br>
                  A significant advantage is also, that default initialization is usually better,
                  because a default value of zero for a relative angle/distance is usually what the
                  user would like to have. Previously, say, the load was initialized to a non-zero
                  angle and then the elastically coupled motor inertia had to be explicitly
                  also initialized with this value. This is now, no longer needed. Since the default
                  nominal value of 1 is usually too large for a relative quantity, the nominal
                  values of the relative angle/distance was changed to 1e-4.</li>
         <li> The two libraries have been restructured in sublibraries to cope
                  with the growing number of components.</li>
         <li> Finally, the Translational library has been
                  made as similar as possible to the Rotational library by, e.g., adding missing
                  components.<br>&nbsp;</li>
         </ul></li>

<li> The initialization of the MultiBody, Rotational and Translational libraries have
         been significantly simplified by removing the \"initType\" parameters and only
         using start/fixed values. This design assumes that a tool has special support
         for start/fixed values in the parameter menu.<br>&nbsp;</li>

<li> Nearly all parameters defined in the Modelica Standard Library had been
         defined with a default equation, e.g.,
         <pre>   <b>parameter</b> Modelica.SIunits.Resistance R=1; </pre>
         Physical parameters, such as a resistance, mass, gear ratio, do not have a meaningful
         default and in nearly all cases, the user of the corresponding component has to
         provide values for such parameters. If the user forgets this, a tool
         cannot provide diagnostics, since a default value is present in the library
         (such as 1 Ohm for the resistance). In most cases the model will simulate but will
         give wrong results due to wrong parameter values. To improve this situation, all physical
         parameter declarations in the Modelica Standard Library have been changed, so
         that the previous default becomes a start value. For example, the above
         declaration is changed to:
         <pre>   <b>parameter</b> Modelica.SIunits.Resistance R(start=1);  </pre>
         This is a backward compatible change and completely equivalent from the perspective
         of the Modelica language. It is, however, advised that tools will print a warning
         or optionally an error message, if the start value of a parameter is defined, but
         no value for the parameter is given via a modification. Furthermore, it is expected,
         that the input field of a parameter menu is empty, if no default equation is defined,
         but only a start value. This shows clearly to the modeler that a value has to
         be provided.</li>
</ul>

<p><br>
The following <b style=\"color:blue\">new components</b> have been added
to <b style=\"color:blue\">existing</b> libraries (note, the names in parentheses
are the new sublibrary names that are introduced in version 3.0):
</p>

<table border=\"1\" cellspacing=0 cellpadding=2 style=\"border-collapse:collapse;\">
<tr><td colspan=\"2\"><b>Blocks.Examples.</b></td></tr>
<tr><td valign=\"top\">InverseModel</td>
          <td valign=\"top\"> Demonstrates the construction of an inverse model.</td> </tr>

<tr><td colspan=\"2\"><b>Blocks.Math.</b></td></tr>
<tr><td valign=\"top\">InverseBlockConstraints</td>
          <td valign=\"top\"> Construct inverse model by requiring that two inputs
                                                and two outputs are identical (replaces the previously,
                                                unbalanced, TwoInputs and TwoOutputs blocks).</td> </tr>

<tr><td colspan=\"2\"><b>Electrical.Machines.Utilities</b></td></tr>
<tr><td valign=\"top\">TransformerData</td>
          <td valign=\"top\"> A record that calculates required impedances (parameters) from nominal data of transformers.</td> </tr>

<tr><td colspan=\"2\"><b>Mechanics.MultiBody.Examples.Rotational3DEffects</b></td></tr>
<tr><td valign=\"top\"> GyroscopicEffects<br>
                                                ActuatedDrive<br>
                                                MovingActuatedDrive<br>
                                                GearConstraint </td>
          <td valign=\"top\"> New examples to demonstrate the usage of the Rotational library
                                                in combination with multi-body components. </td>
</tr>

<tr><td colspan=\"2\"><b>Mechanics.MultiBody.Sensors</b></td></tr>
<tr><td valign=\"top\"> AbsolutePosition<br>
                                                AbsoluteVelocity<br>
                                                AbsoluteAngles<br>
                                                AbsoluteAngularVelocity<br>
                                                RelativePosition<br>
                                                RelativeVelocity<br>
                                                RelativeAngles<br>
                                                RelativeAngularVelocity</td>
          <td valign=\"top\"> New sensors to measure one vector. </td>
</tr>
<tr><td valign=\"top\"> TransformAbsoluteVector<br>
                                                TransformRelativeVector</td>
          <td valign=\"top\"> Transform absolute and/or relative vector into another frame. </td>
</tr>

<tr><td colspan=\"2\"><b>Mechanics.Rotational.(Components)</b></td></tr>
<tr><td valign=\"top\"> Disc </td>
          <td valign=\"top\"> Right flange is rotated by a fixed angle with respect to left flange</td> </tr>
<tr><td valign=\"top\"> IdealRollingWheel </td>
          <td valign=\"top\"> Simple 1-dim. model of an ideal rolling wheel without inertia</td> </tr>

<tr><td colspan=\"2\"><b>Mechanics.Translational.Sensors</b></td></tr>
<tr><td valign=\"top\">RelPositionSensor<br>RelSpeedSensor<br>RelAccSensor<br>PowerSensor</td>
          <td valign=\"top\"> Relative position sensor, i.e., distance between two flanges<br>
                                                Relative speed sensor<br>
                                                Relative acceleration sensor<br>
                                                Ideal power sensor</td> </tr>
<tr><td colspan=\"2\"><b>Mechanics.Translational(.Components)</b></td></tr>
<tr><td valign=\"top\">SupportFriction<br>Brake<br>InitializeFlange</td>
          <td valign=\"top\"> Model of friction due to support<br>
                                                Model of a brake, base on Coulomb friction<br>
                                                Initializes a flange with pre-defined position, speed and acceleration .</td> </tr>
<tr><td colspan=\"2\"><b>Mechanics.Translational(.Sources)</b></td></tr>
<tr><td valign=\"top\">Force2<br>LinearSpeedDependentForce<br>QuadraticSpeedDependentForce<br>
                                           ConstantForce<br>ConstantSpeed<br>ForceStep</td>
          <td valign=\"top\"> Force acting on 2 flanges<br>
                                                Force linearly dependent on flange speed<br>
                                                Force quadratic dependent on flange speed<br>
                                                Constant force source<br>
                                                Constant speed source<br>
                                                Force step</td> </tr>
</table>

<p><br>
The following <b style=\"color:blue\">existing components</b>
have been <b style=\"color:blue\">changed</b> in a
<b style=\"color:blue\">non-backward compatible</b> way
(the conversion script transforms models and libraries
of previous versions to the new version. Therefore, conversion
should be automatic):
</p>

<table border=\"1\" cellspacing=0 cellpadding=2 style=\"border-collapse:collapse;\">
<tr><td colspan=\"2\"><b>Blocks.Continuous.</b></td></tr>
<tr><td valign=\"top\"> CriticalDamping </td>
          <td valign=\"top\"> New parameter \"normalized\" to define whether filter is provided
                                                in normalized or non-normalized form. Default is \"normalized = true\".
                                                The previous implementation was a non-normalized filter.
                                                The conversion script automatically introduces the modifier
                                                \"normalized=false\" for existing models.</td> </tr>

<tr><td colspan=\"2\"><b>Blocks.Interfaces.</b></td></tr>
<tr><td valign=\"top\"> RealInput<br>
                                                RealOutput</td>
          <td valign=\"top\"> Removed \"SignalType\", since extending from a replaceable class
                                                and this is not allowed in Modelica 3.<br>The conversion script
                                                removes modifiers to SignalType.</td> </tr>

<tr><td valign=\"top\"> RealSignal<br>
                                                IntegerSignal<br>
                                                BooleanSignal</td>
          <td valign=\"top\"> Moved to library ObsoleteModelica3, since these connectors
                                                are no longer allowed in Modelica 3<br>
                                                (prefixes input and/or output are required). </td> </tr>

<tr><td colspan=\"2\"><b>Blocks.Interfaces.Adaptors.</b></td></tr>
<tr><td valign=\"top\"> AdaptorReal<br>
                                                AdaptorBoolean<br>
                                                AdaptorInteger</td>
          <td valign=\"top\"> Moved to library ObsoleteModelica3, since the models are not \"balanced\".
                                                These are completely obsolete adaptors<br>between the Real, Boolean, Integer
                                                signal connectors of version 1.6 and version &ge; 2.1 of the Modelica
                                                Standard Library. </td> </tr>

<tr><td colspan=\"2\"><b>Blocks.Math.</b></td></tr>
<tr><td valign=\"top\"> ConvertAllUnits</td>
          <td valign=\"top\"> Moved to library ObsoleteModelica3, since extending from a replaceable class
                                                and this is not allowed in Modelica 3.<br> It would be possible to rewrite this
                                                model to use a replaceable component. However, the information about the
                                                conversion<br> cannot be visualized in the icon in this case.</td> </tr>

<tr><td colspan=\"2\"><b>Blocks.Math.UnitConversions.</b></td></tr>
<tr><td valign=\"top\"> TwoInputs<br>
                                                TwoOutputs</td>
          <td valign=\"top\"> Moved to library ObsoleteModelica3, since the models are not \"balanced\".
                                                A new component<br>\"InverseBlockConstraints\"
                                                is provided instead that has the same feature, but is \"balanced\".</td> </tr>

<tr><td colspan=\"2\"><b>Electrical.Analog.Baisc.</b></td></tr>
<tr><td valign=\"top\"> HeatingResistor</td>
          <td valign=\"top\"> The heatPort has to be connected; otherwise the component Resistor (without heatPort) has to be used.<br>
                                                cardinality() is only used to check whether the heatPort is connected.</td> </tr>

<tr><td colspan=\"2\"><b>Electrical.MultiPhase.Examples.</b></td></tr>
<tr><td valign=\"top\"> </td>
          <td valign=\"top\"> Changed the instance names of components used in the examples to more up-to-date style.</td> </tr>

<tr><td colspan=\"2\"><b>Electrical.Machines.</b></td></tr>
<tr><td valign=\"top\"> </td>
          <td valign=\"top\"> Moved package <code>Machines.Examples.Utilities</code> to <code>Machines.Utilities</code></td> </tr>
<tr><td valign=\"top\"> </td>
          <td valign=\"top\"> Removed all nonSIunits; especially in DCMachines<br>
                                                parameter NonSIunits.AngularVelocity_rpm rpmNominal was replaced by<br>
                                                parameter SIunits.AngularVelocity wNominal</td> </tr>
<tr><td valign=\"top\"> </td>
          <td valign=\"top\"> Changed the following component variable and parameter names to be more concise:<br>
                                                Removed suffix \"DamperCage\" from all synchronous induction machines
                                                since the user can choose whether the damper cage is present or not.<br><code>
                                                RotorAngle ... RotorDisplacementAngle<br>
                                                J_Rotor ... Jr<br>
                                                Rr ........ Rrd (damper of synchronous induction machines)<br>
                                                Lrsigma ... Lrsigmad (damper of synchronous induction machines)<br>
                                                phi_mechanical ... phiMechanical<br>
                                                w_mechanical ..... wMechanical<br>
                                                rpm_mechanical ... rpmMechanical<br>
                                                tau_electrical ... tauElectrical<br>
                                                tau_shaft ........ tauShaft<br>
                                                TurnsRatio ....... turnsRatio    (AIMS)<br>
                                                VsNom ............ VsNominal     (AIMS)<br>
                                                Vr_Lr ............ VrLockedRotor (AIMS)<br>
                                                DamperCage ....... useDamperCage (synchronous induction machines)<br>
                                                V0 ............... VsOpenCicuit  (SMPM)<br>
                                                Ie0 .............. IeOpenCicuit  (SMEE)
                                                </code></td> </tr>
<tr><td valign=\"top\">Interfaces.</td>
          <td valign=\"top\"> Moved as much code as possible from specific machine models to partials to reduce redundant code.</td> </tr>
<tr><td valign=\"top\">Interfaces.Adapter</td>
          <td valign=\"top\"> Removed to avoid cardinality; instead, the following solution has been implemented:</td> </tr>
<tr><td valign=\"top\">Sensors.RotorDisplacementAngle<br>Interfaces.PartialBasicMachine</td>
          <td valign=\"top\"> Introduced <code>parameter Boolean useSupport=false \"enable / disable (=fixed stator) support\"</code><br>
                                                The rotational support connector is only present with <code>useSupport = true;</code><br>
                                                otherwise the stator is fixed internally.</td> </tr>

<tr><td colspan=\"2\"><b>Electrical.Machines.Examples.</b></td></tr>
<tr><td valign=\"top\"> </td>
          <td valign=\"top\"> Changed the names of the examples to more meaningful names.<br>
                                                Changed the instance names of components used in the examples to more up-to-date style.</td> </tr>
<tr><td valign=\"top\">SMEE_Generator</td>
          <td valign=\"top\"> Initialization of <code>smee.phiMechanical</code> with <code>fixed=true</code></td> </tr>

<tr><td colspan=\"2\"><b>Mechanics.MultiBody.</b></td></tr>
<tr><td valign=\"top\"> World</td>
          <td valign=\"top\"> Changed default value of parameter driveTrainMechanics3D from false to true.<br>
                                                3-dim. effects in Rotor1D, Mounting1D and BevelGear1D are therefore taken<br>
                                                into account by default (previously this was only the case, if
                                                world.driveTrainMechanics3D was explicitly set).</td> </tr>

<tr><td colspan=\"2\"><b>Mechanics.MultiBody.Forces.</b></td></tr>
<tr><td valign=\"top\"> FrameForce<br>
                                                FrameTorque<br>
                                                FrameForceAndTorque</td>
          <td valign=\"top\"> Models removed, since functionality now available via Force, Torque, ForceAndTorque</td> </tr>
<tr><td valign=\"top\"> WorldForce<br>
                                                WorldTorque<br>
                                                WorldForceAndTorque<br>
                                                Force<br>
                                                Torque<br>
                                                ForceAndTorque</td>
          <td valign=\"top\"> Connector frame_resolve is optionally enabled via parameter resolveInFrame<br>.
                                                Forces and torques and be resolved in all meaningful frames defined
                                                by enumeration resolveInFrame.</td> </tr>

<tr><td colspan=\"2\"><b>Mechanics.MultiBody.Frames.</b></td></tr>
<tr><td valign=\"top\"> length<br>
                                                normalize</td>
          <td valign=\"top\"> Removed functions, since available also in Modelica.Math.Vectors
                                                <br>The conversion script changes the references correspondingly.</td> </tr>

<tr><td colspan=\"2\"><b>Mechanics.MultiBody.Joints.</b></td></tr>
<tr><td valign=\"top\"> Prismatic<br>
                                                ActuatedPrismatic<br>
                                                Revolute<br>
                                                ActuatedRevolute<br>
                                                Cylindrical<br>
                                                Universal<br>
                                                Planar<br>
                                                Spherical<br>
                                                FreeMotion</td>
          <td valign=\"top\"> Changed initialization, by replacing initial value parameters with
                                                start/fixed attributes.<br>
                                                When start/fixed attributes are properly supported
                                                in the parameter menu by a Modelica tool,<br>
                                                the initialization is considerably simplified for the
                                                user and the implementation is much simpler.<br>
                                                Replaced parameter \"enforceStates\" by the more general
                                                built-in enumeration stateSelect=StateSelection.xxx.<br>
                                                The conversion script automatically
                                                transforms from the \"old\" to the \"new\" forms.</td> </tr>
<tr><td valign=\"top\"> Revolute<br>
                                                ActuatedRevolute</td>
          <td valign=\"top\"> Parameter \"planarCutJoint\" in the \"Advanced\" menu of \"Revolute\" and of
                                                \"ActuatedRevolute\" removed.<br>
                                                A new joint \"RevolutePlanarLoopConstraint\" introduced that defines the constraints
                                                of a revolute joint<br> as cut-joint in a planar loop.
                                                This change was needed in order that the revolute joint can be
                                                properly used<br>in advanced model checking.<br>
                                                ActuatedRevolute joint removed. Flange connectors of Revolute joint<br>
                                                can be enabled with parameter useAxisFlange.</td> </tr>
<tr><td valign=\"top\"> Prismatic<br>
                                                ActuatedPrismatic</td>
          <td valign=\"top\"> ActuatedPrismatic joint removed. Flange connectors of Prismatic joint<br>
                                                can be enabled with parameter useAxisFlange.</td> </tr>
<tr><td valign=\"top\"> Assemblies</td>
          <td valign=\"top\"> Assembly joint implementation slightly changed, so that
                                                annotation \"structurallyIncomplete\" <br>could be removed
                                                (all Assembly joint models are now \"balanced\").</td> </tr>

<tr><td colspan=\"2\"><b>Mechanics.MultiBody.Joints.Internal</b></td></tr>
<tr><td valign=\"top\"> RevoluteWithLengthConstraint<br>
                                                PrismaticWithLengthConstraint</td>
          <td valign=\"top\"> These joints should not be used by a user of the MultiBody library.
                                                They are only provided to built-up the
                                                MultiBody.Joints.Assemblies.JointXYZ joints.
                                                These two joints have been changed in a slightly not backward compatible
                                                way, in order that the usage in the Assemblies.JointXYZ joints results in
                                                balanced models (<b>no conversion is provided for this change since the
                                                user should not have used these joints and the conversion would be too
                                                complicated</b>):
                                                In releases before version 3.0 of the Modelica Standard Library,
                                                it was possible to activate the torque/force projection equation
                                                (= cut-torque/-force projected to the rotation/translation
                                                axis must be identical to
                                                the drive torque/force of flange axis) via parameter <b>axisTorqueBalance</b>.
                                                This is no longer possible, since otherwise this model would not be
                                                \"balanced\" (= same number of unknowns as equations). Instead, when
                                                using this model in version 3.0 and later versions, the torque/force
                                                projection equation must be provided in the Advanced menu of joints
                                                Joints.SphericalSpherical and Joints.UniversalSpherical
                                                via the new parameter \"constraintResidue\".</td> </tr>

<tr><td colspan=\"2\"><b>Mechanics.MultiBody.Parts.</b></td></tr>
<tr><td valign=\"top\"> BodyBox<br>
                                                BodyCylinder</td>
          <td valign=\"top\"> Changed unit of parameter density from g/cm3 to the SI unit kg/m3
                                                in order to allow stricter unit checking.<br>The conversion script multiplies
                                                previous density values with 1000.</td> </tr>
<tr><td valign=\"top\"> Body<br>
                                                BodyShape<br>
                                                BodyBox<br>
                                                BodyCylinder<br>
                                                PointMass
                                                Rotor1D</td>
          <td valign=\"top\"> Changed initialization, by replacing initial value parameters with
                                                start/fixed attributes.<br>
                                                When start/fixed attributes are properly supported
                                                in the parameter menu by a Modelica tool,<br>
                                                the initialization is considerably simplified for the
                                                user and the implementation is much simpler.<br>The conversion script automatically
                                                transforms from the \"old\" to the \"new\" form of initialization.</td> </tr>

<tr><td colspan=\"2\"><b>Mechanics.MultiBody.Sensors.</b></td></tr>
<tr><td valign=\"top\"> AbsoluteSensor<br>
                                                RelativeSensor<br>
                                                CutForceAndTorque</td>
          <td valign=\"top\"> New design of sensor components: Via Boolean parameters<br>
                                                signal connectors for the respective vectors are enabled/disabled.<br>
                                                It is not possible to automatically convert models to this new design.<br>
                                                Instead, references in existing models are changed to ObsoleteModelice3.<br>
                                                This means that these models must be manually adapted.</td> </tr>
<tr><td valign=\"top\"> CutForce<br>
                                                CutTorque</td>
          <td valign=\"top\"> Slightly new design. The force and/or torque component can be
                                                resolved in world, frame_a, or frame_resolved.<br>
                                                Existing models are automatically converted.</td> </tr>

<tr><td colspan=\"2\"><b>Mechanics.Rotational.</b></td></tr>
<tr><td valign=\"top\"> </td>
          <td valign=\"top\"> Moved components to structured sub-packages (Sources, Components)</td> </tr>
<tr><td valign=\"top\"> Inertia<br>
                                                SpringDamper<br>
                                                RelativeStates</td>
          <td valign=\"top\"> Changed initialization, by replacing initial value parameters with
                                                start/fixed attributes.<br>
                                                When start/fixed attributes are properly supported
                                                in the parameter menu by a Modelica tool,<br>
                                                the initialization is considerably simplified for the
                                                user and the implementation is much simpler.<br>
                                                Parameter \"stateSelection\" in \"Inertia\" and \"SpringDamper\" replaced
                                                by the built-in enumeration<br>stateSelect=StateSelection.xxx.
                                                Introduced the \"stateSelect\" enumeration in \"RelativeStates\".<br>
                                                The conversion script automatically
                                                transforms from the \"old\" to the \"new\" forms.</td> </tr>
<tr><td valign=\"top\"> LossyGear<br>
                                                GearBox</td>
          <td valign=\"top\"> Renamed gear ratio parameter \"i\" to \"ratio\", in order to have a
                                                consistent naming convention.<br>
                                                Existing models are automatically converted. </td> </tr>
<tr><td valign=\"top\"> SpringDamper<br>
                                                ElastoBacklash<br>
                                                Clutch<br>
                                                OneWayClutch</td>
          <td valign=\"top\"> Relative quantities (phi_rel, w_rel) are used as states, if possible
                                                (due to StateSelect.prefer). <br>
                                                In most cases, relative states in drive trains are better suited as
                                                absolute states. <br> This change might give changes in the selected states
                                                of existing models.<br>
                                                This might give rise to problems if, e.g., the initialization was not
                                                completely defined in a user model,<br> since the default
                                                initialization heuristic may give different initial values.</td> </tr>

<tr><td colspan=\"2\"><b>Mechanics.Translational.</b></td></tr>
<tr><td valign=\"top\"> </td>
          <td valign=\"top\"> Moved components to structured sub-packages (Sources, Components)</td> </tr>
<tr><td valign=\"top\"> </td>
          <td valign=\"top\"> Adaptions corresponding to Rotational</td> </tr>
<tr><td valign=\"top\"> Stop</td>
          <td valign=\"top\"> Renamed to Components.MassWithStopAndFriction to be more concise.<br>
                                                MassWithStopAndFriction is not available with a support connector, <br>
                                                since the reaction force can't be modeled in a meaningful way due to reinit of velocity v.<br>
                                                Until a sound implementation of a hard stop is available, the old model may be used.</td> </tr>
<tr><td colspan=\"2\"><b>Media.</b></td></tr>
<tr><td valign=\"top\"> constant nX <br>
                                                constant nXi <br>
                                                constant reference_X<br>
                                                BaseProperties</td>
          <td valign=\"top\"> The package constant nX = nS, now always, even for single species media. This also allows to define mixtures with only 1 element. The package constant nXi=if fixedX then 0 else if reducedX or nS==1 then nS - 1 else nS. This required that all BaseProperties for single species media get an additional equation to define the composition X as {1.0} (or reference_X, which is {1.0} for single species). This will also mean that all user defined single species media need to be updated by that equation.</td> </tr>

<tr><td colspan=\"2\"><b>SIunits.</b></td></tr>
<tr><td valign=\"top\"> CelsiusTemperature </td>
          <td valign=\"top\"> Removed, since no SI unit. The conversion script changes references to
                                                SIunits.Conversions.NonSIunits.Temperature_degC </td> </tr>
<tr><td valign=\"top\"> ThermodynamicTemperature <br>
                                                TemperatureDifference</td>
          <td valign=\"top\"> Added annotation \"absoluteValue=true/false\"
                                                in order that unit checking is possible<br>
                                                (the unit checker needs to know for a unit that has an offset,
                                                whether it is used as absolute or as a relative number)</td> </tr>

<tr><td colspan=\"2\"><b>SIunits.Conversions.NonSIunits.</b></td></tr>
<tr><td valign=\"top\"> Temperature_degC<br>
                                                Temperature_degF<br>
                                                Temperature_degRk </td>
          <td valign=\"top\"> Added annotation \"absoluteValue=true\"
                                                in order that unit checking is possible<br>
                                                (the unit checker needs to know for a unit that has an offset,
                                                whether it is used as absolute or as a relative number)</td> </tr>

<tr><td colspan=\"2\"><b>StateGraph.Examples.</b></td></tr>
<tr><td valign=\"top\"> ControlledTanks </td>
          <td valign=\"top\"> The connectors of the ControlledTanks did not fulfill the new
                                                restrictions of Modelica 3. This has been fixed.</td> </tr>
<tr><td valign=\"top\"> Utilities </td>
          <td valign=\"top\"> Replacing inflow, outflow by connectors inflow1, inflow2,
                                                outflow1, outflow2 with appropriate input/output prefixes in
                                                order to fulfill the restrictions of Modelica 3 to arrive
                                                at balanced models. No conversion is provided, since
                                                too difficult and since the non-backward compatible change is in
                                                an example. </td> </tr>

<tr><td colspan=\"2\"><b>Thermal.FluidHeatFlow.Sensors.</b></td></tr>
<tr><td valign=\"top\"> <br>
                                                pSensor<br>TSensor<br>dpSensor<br>dTSensor<br>m_flowSensor<br>V_flowSensor<br>H_flowSensor</td>
          <td valign=\"top\"> renamed to:<br>
                                                PressureSensor<br>TemperatureSensor<br>RelPressureSensor<br>RelTemperatureSensor<br>MassFlowSensor<br>VolumeFlowSensor<br>EnthalpyFlowSensor
                                                </td> </tr>

<tr><td colspan=\"2\"><b>Thermal.FluidHeatFlow.Sources.</b></td></tr>
<tr><td valign=\"top\"> Ambient<br>PrescribedAmbient</td>
          <td valign=\"top\"> available as one combined component Ambient<br>
                                                Boolean parameters usePressureInput and useTemperatureInput decide
                                                whether pressure and/or temperature are constant or prescribed</td> </tr>
<tr><td valign=\"top\"> ConstantVolumeFlow<br>PrescribedVolumeFlow</td>
          <td valign=\"top\"> available as one combined component VolumeFlow<br>
                                                Boolean parameter useVolumeFlowInput decides
                                                whether volume flow is constant or prescribed</td> </tr>
<tr><td valign=\"top\"> ConstantPressureIncrease<br>PrescribedPressureIncrease</td>
          <td valign=\"top\"> available as one combined component PressureIncrease<br>
                                                Boolean parameter usePressureIncreaseInput decides
                                                whether pressure increase is constant or prescribed</td> </tr>

<tr><td colspan=\"2\"><b>Thermal.FluidHeatFlow.Examples.</b></td></tr>
<tr><td valign=\"top\"> </td>
          <td valign=\"top\"> Changed the instance names of components used in the examples to more up-to-date style.</td> </tr>

<tr><td colspan=\"2\"><b>Thermal.HeatTransfer.(Components)</b></td></tr>
<tr><td valign=\"top\"> HeatCapacitor</td>
          <td valign=\"top\"> Initialization changed: SteadyStateStart removed. Instead
                                                start/fixed values for T and der_T<br>(initial temperature and its derivative).</td> </tr>

<tr><td valign=\"top\"> <br><br>HeatCapacitor<br>ThermalConductor<br>ThermalConvection<br>BodyRadiation<br><br>
                                                TemperatureSensor<br>RelTemperatureSensor<br>HeatFlowSensor<br><br>
                                                FixedTemperature<br>PrescribedTemperature<br>FixedHeatFlow<br>PrescribedHeatFlow</td>
          <td valign=\"top\"> Moved components to sub-packages:<br><br>
                                                Components.HeatCapacitor<br>Components.ThermalConductor<br>Components.ThermalConvection<br>Components.BodyRadiation<br><br>
                                                Sensors.TemperatureSensor<br>Sensors.RelTemperatureSensor<br>Sensors.HeatFlowSensor<br><br>
                                                Sources.FixedTemperature<br>Sources.PrescribedTemperature<br>Sources.FixedHeatFlow<br>Sources.PrescribedHeatFlow
                                                </td> </tr>

<tr><td colspan=\"2\"><b>Thermal.FluidHeatFlow.Examples.</b></td></tr>
<tr><td valign=\"top\"> </td>
          <td valign=\"top\"> Changed the instance names of components used in the examples to more up-to-date style.</td> </tr>
</table>

<p><br>
The following <b style=\"color:blue\">existing components</b>
have been <b style=\"color:blue\">improved</b> in a
<b style=\"color:blue\">backward compatible</b> way:
</p>

<table border=\"1\" cellspacing=0 cellpadding=2 style=\"border-collapse:collapse;\">
<tr><td valign=\"top\"> <b>Modelica.*</b> </td>
          <td valign=\"top\"> Parameter declarations, input and output function arguments without description
                                                strings improved<br> by providing meaningful description texts.
                                                </td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Blocks.Continuous.</b></td></tr>
<tr><td valign=\"top\"> TransferFunction </td>
          <td valign=\"top\"> Internal scaling of the controller canonical states introduced
                                                in order to enlarge the range of transfer functions where the default
                                                relative tolerance of the simulator is sufficient.</td>
</tr>

<tr><td valign=\"top\"> Butterworth<br>CriticalDamping </td>
          <td valign=\"top\"> Documentation improved and plots of the filter characteristics added.</td> </tr>

<tr><td colspan=\"2\"><b>Electrical.Analog.Basic.</b></td></tr>
<tr><td valign=\"top\"> EMF </td>
          <td valign=\"top\"> New parameter \"useSupport\" to optionally enable a support connector.</td> </tr>

<tr><td colspan=\"2\"><b>Icons.</b></td></tr>
<tr><td valign=\"top\"> TranslationalSensor<br>
                                                RotationalSensor</td>
          <td valign=\"top\"> Removed drawing from the diagram layer (kept drawing only in
                                                icon layer),<br> in order that this icon can be used in situations
                                                where components are dragged in the diagram layer.</td> </tr>

<tr><td colspan=\"2\"><b>Math.Vectors.</b></td></tr>
<tr><td valign=\"top\"> normalize</td>
          <td valign=\"top\"> Implementation changed, so that the result is awalys continuous<br>
                                                (previously, this was not the case for small vectors: normalize(eps,eps)).
                                                </td> </tr>

<tr><td colspan=\"2\"><b>Mechanics.MultiBody.</b></td></tr>
<tr><td valign=\"top\"> </td>
          <td valign=\"top\"> Renamed non-standard keywords defineBranch, defineRoot, definePotentialRoot,
                                                isRooted to the standard names:<br>
                                                Connections.branch/.root/.potentialRoot/.isRooted.</td> </tr>
<tr><td valign=\"top\"> Frames </td>
          <td valign=\"top\"> Added annotation \"Inline=true\" to all one-line functions
                                                (which should be all inlined).</td> </tr>
<tr><td colspan=\"2\"><b>Mechanics.MultiBody.Parts.</b></td></tr>
<tr><td valign=\"top\"> Mounting1D<br>
                                                Rotor1D<br>
                                                BevelGear1D</td>
          <td valign=\"top\"> Changed implementation so that no longer modifiers for connector
                                                variables are used,<br>because this violates the restrictions on
                                                \"balanced models\" of Modelica 3. </td> </tr>

<tr><td colspan=\"2\"><b>Mechanics.Rotational.</b></td></tr>
<tr><td valign=\"top\"> InitializeFlange</td>
          <td valign=\"top\"> Changed implementation so that counting unknowns and
                                                equations is possible without actual values of parameters.</td> </tr>

<tr><td colspan=\"2\"><b>Thermal.FluidHeatFlow.Interfaces.Partials.</b></td></tr>
<tr><td valign=\"top\">TwoPort</td>
          <td valign=\"top\"> Introduced <code>parameter Real tapT(final min=0, final max=1)=1</code> <br> that defines the temperature of the heatPort
                                                between inlet and outlet.</td> </tr>

<tr><td colspan=\"2\"><b>StateGraph.</b></td></tr>
<tr><td valign=\"top\"> InitialStep<br>
                                                InitialStepWithSignal<br>
                                                Step<br>
                                                StepWithSignal</td>
          <td valign=\"top\"> Changed implementation so that no longer modifiers for output
                                                variables are used,<br>because this violates the restrictions on
                                                \"balanced models\" of Modelica 3. </td> </tr>

</table>

<p><br>
The following <b style=\"color:red\">critical errors</b> have been fixed (i.e., errors
that can lead to wrong simulation results):
</p>

<table border=\"1\" cellspacing=0 cellpadding=2 style=\"border-collapse:collapse;\">
<tr><td colspan=\"2\"><b>Electrical.Analog.Examples.</b></td></tr>
<tr><td valign=\"top\"> CauerLowPassSC </td>
          <td valign=\"top\"> Wrong calculation of Capacitor1 both in Rn and Rp corrected
                                                (C=clock/R instead of C=clock*R) </td>
</tr>

<tr><td colspan=\"2\"><b>Mechanics.MultiBody.Parts.</b></td></tr>
<tr><td valign=\"top\"> Rotor1D </td>
          <td valign=\"top\"> The 3D reaction torque was not completely correct and gave in
                                                some situations a wrong result. This bug should not influence the
                                                movement of a multi-body system, but only the constraint torques
                                                are sometimes not correct. </td>
</tr>

<tr><td colspan=\"2\"><b>Mechanics.Rotational.</b></td></tr>
<tr><td valign=\"top\"> ElastoBacklash </td>
          <td valign=\"top\"> If the damping torque was too large, the reaction torque
                                                could \"pull\" which is unphysical. The component was
                                                newly written by limiting the damping torque in such a case
                                                so that \"pulling\" torques can no longer occur. Furthermore,
                                                during initialization the characteristics is made continuous
                                                to reduce numerical errors. The relative angle and relative
                                                angular velocities are used as states, if possible
                                                (StateSelect.prefer), since relative quantities lead usually
                                                to better behavior.  </td>
</tr>
<tr><td valign=\"top\"> Position<br>Speed<br>Accelerate<br>Move</td>
          <td valign=\"top\"> The movement of the flange was wrongly defined as absolute;
                                                this is corrected as relative to connector support.<br>
                                                For Accelerate, it was necessary to rename
                                                RealInput a to a_ref, as well as the start values
                                                phi_start to phi.start and w_start to w.start.
                                                The conversion script performs the necessary conversion of
                                                existing models automatically.</td>
</tr>
<tr><td colspan=\"2\"><b>Media.Interfaces.</b></td></tr>
<tr><td valign=\"top\"> PartialSimpleIdealGasMedium </td>
          <td valign=\"top\"> Inconsistency in reference temperature corrected. This may give
                                                different results for functions:<br>
                                                specificEnthalpy, specificInternalEnergy, specificGibbsEnergy,
                                                specificHelmholtzEnergy. </td>
</tr>
<tr><td colspan=\"2\"><b>Media.Air.</b></td></tr>
<tr><td valign=\"top\"> specificEntropy </td>
          <td valign=\"top\"> Small bug in entropy computation of ideal gas mixtures corrected.</td>
</tr>
<tr><td colspan=\"2\"><b>Media.IdealGases.Common.MixtureGasNasa</b></td></tr>
<tr><td valign=\"top\"> specificEntropy </td>
          <td valign=\"top\"> Small bug in entropy computation of ideal gas mixtures corrected.</td>
</tr>
</table>

<p><br>
The following <b style=\"color:red\">uncritical errors</b> have been fixed (i.e., errors
that do <b style=\"color:red\">not</b> lead to wrong simulation results, but, e.g.,
units are wrong or errors in documentation):
</p>

<table border=\"1\" cellspacing=0 cellpadding=2 style=\"border-collapse:collapse;\">
<tr><td colspan=\"2\"><b>Blocks.Tables.</b></td></tr>
<tr><td valign=\"top\"> CombiTable2D</td>
          <td valign=\"top\"> Documentation improved.</td>
</tr>

<tr><td colspan=\"2\"><b>Electrica.Digital.Gates</b></td></tr>
<tr><td valign=\"top\"> AndGate<br>
                                                NandGate<br>
                                                OrGate<br>
                                                NorGate<br>
                                                XorGate<br>
                                                XnorGate</td>
          <td valign=\"top\"> The number of inputs was not correctly propagated
                                                to the included base model.<br>
                                                This gave a translation error, if the number
                                                of inputs was changed (and not the default used).</td>
</tr>

<tr><td colspan=\"2\"><b>Electrica.Digital.Sources</b></td></tr>
<tr><td valign=\"top\"> Pulse </td>
          <td valign=\"top\"> Model differently implemented, so that
                                                warning message about \"cannot properly initialize\" is gone.</td>
</tr>

<tr><td colspan=\"2\"><b>Mechanics.Rotational.</b></td></tr>
<tr><td valign=\"top\"> BearingFriction<br>
                                                Clutch<br>
                                                OneWayClutch<br>
                                                Brake <br>
                                                Gear </td>
          <td valign=\"top\"> Declaration of table parameter changed from
                                                table[:,:] to table[:,2].</td>
</tr>

<tr><td colspan=\"2\"><b>Modelica.Mechanics.MultiBody.Examples.Loops.Utilities.</b></td></tr>
<tr><td valign=\"top\"> GasForce </td>
          <td valign=\"top\"> Unit of variable \"press\" corrected (from Pa to bar)</td>
</tr>

<tr><td colspan=\"2\"><b>StateGraph.Examples.</b></td></tr>
<tr><td valign=\"top\">SimpleFriction</td>
          <td valign=\"top\"> The internal parameter k is defined and calculated with the appropriate unit.</td> </tr>

<tr><td colspan=\"2\"><b>Thermal.FluidHeatFlow.Interfaces.Partials.</b></td></tr>
<tr><td valign=\"top\">SimpleFriction</td>
          <td valign=\"top\"> The internal parameter k is defined and calculated with the appropriate unit.</td> </tr>

</table>

</html>"));
      end Version_3_0;

      class Version_2_2_2 "Version 2.2.2 (Aug. 31, 2007)"
        extends Modelica.Icons.ReleaseNotes;
        annotation(
          Documentation(info = "<html>
<p>
Version 2.2.2 is backward compatible to version 2.2.1 and 2.2 with
the following exceptions:
</p>
<ul>
<li> Removal of package Modelica.Media.Interfaces.PartialTwoPhaseMediumWithCache
         (this was not yet utilized).</li>
<li> Removal of the media packages in
         Modelica.Media.IdealGases.SingleGases that are not type compatible
         to Modelica.Media.Interfaces.PartialMedium, because a FluidConstants
         record definition is missing,
         for details, see
          <a href=\"modelica://Modelica.Media.IdealGases\">Modelica.Media.IdealGases</a>
         (this is seen as a bug fix).
</ul>

<p>
An overview of the differences between version 2.2.2 and the previous
version 2.2.1 is given below. The exact differences (but without
differences in the documentation) are available in
<a href=\"modelica://Modelica/Resources/Documentation/Differences-Modelica-221-222.html\">Differences-Modelica-221-222.html</a>.
This comparison file was generated automatically with Dymola's
ModelManagement.compare function.
</p>

<p>
In this version, <b>no</b> new libraries have been added. The <b>documentation</b>
of the whole library was improved.
</p>

<p><br>
The following <b style=\"color:blue\">new components</b> have been added
to <b style=\"color:blue\">existing</b> libraries:
</p>

<table border=\"1\" cellspacing=0 cellpadding=2 style=\"border-collapse:collapse;\">
<tr><td colspan=\"2\"><b>Blocks.Logical.</b></td></tr>
<tr><td valign=\"top\"> TerminateSimulation</td>
          <td valign=\"top\"> Terminate a simulation by a given condition.</td>
</tr>

<tr><td colspan=\"2\"><b>Blocks.Routing.</b></td></tr>
<tr><td valign=\"top\"> RealPassThrough<br>
                   IntegerPassThrough<br>
                   BooleanPassThrough</td>
          <td valign=\"top\"> Pass a signal from input to output
                   (useful in combination with a bus due to restrictions
                   of expandable connectors).</td>
</tr>

<tr><td colspan=\"2\"><b>Blocks.Sources.</b></td></tr>
<tr><td valign=\"top\"> KinematicPTP2 </td>
          <td valign=\"top\"> Directly gives q,qd,qdd as output (and not just qdd as KinematicPTP).
          </td>
</tr>

<tr><td colspan=\"2\"><b>Electrical.Machines.Examples.</b></td></tr>
<tr><td valign=\"top\"> TransformerTestbench </td>
          <td valign=\"top\"> Transformer Testbench
          </td>
<tr><td valign=\"top\"> Rectifier6pulse </td>
          <td valign=\"top\"> 6-pulse rectifier with 1 transformer
          </td>
</tr>
<tr><td valign=\"top\"> Rectifier12pulse </td>
          <td valign=\"top\"> 12-pulse rectifier with 2 transformers
          </td>
</tr>
<tr><td valign=\"top\"> AIMC_Steinmetz </td>
          <td valign=\"top\"> Asynchronous induction machine squirrel cage with Steinmetz connection
          </td>
</tr>

<tr><td colspan=\"2\"><b>Electrical.Machines.BasicMachines.Components.</b></td></tr>
<tr><td valign=\"top\"> BasicAIM </td>
          <td valign=\"top\"> Partial model for asynchronous induction machine
          </td>
<tr><td valign=\"top\"> BasicSM </td>
          <td valign=\"top\"> Partial model for synchronous induction machine
          </td>
<tr><td valign=\"top\"> PartialAirGap </td>
          <td valign=\"top\"> Partial airgap model
          </td>
<tr><td valign=\"top\"> BasicDCMachine </td>
          <td valign=\"top\"> Partial model for DC machine
          </td>
<tr><td valign=\"top\"> PartialAirGapDC </td>
          <td valign=\"top\"> Partial airgap model of a DC machine
          </td>
<tr><td valign=\"top\"> BasicTransformer </td>
          <td valign=\"top\"> Partial model of threephase transformer
          </td>
<tr><td valign=\"top\"> PartialCore </td>
          <td valign=\"top\"> Partial model of transformer core with 3 windings
          </td>
<tr><td valign=\"top\"> IdealCore </td>
          <td valign=\"top\"> Ideal transformer with 3 windings
          </td>
</tr>

<tr><td colspan=\"2\"><b>Electrical.Machines.BasicMachines.</b></td></tr>
<tr><td valign=\"top\"> Transformers </td>
          <td valign=\"top\"> Sub-Library for technical 3phase transformers
          </td>
</tr>

<tr><td colspan=\"2\"><b>Electrical.Machines.Interfaces.</b></td></tr>
<tr><td valign=\"top\"> Adapter </td>
          <td valign=\"top\"> Adapter to model housing of electrical machine
          </td>
</tr>

<tr><td colspan=\"2\"><b>Math.</b></td></tr>
<tr><td valign=\"top\"> Vectors </td>
          <td valign=\"top\"> New library of functions operating on vectors
          </td>
</tr>
<tr><td valign=\"top\"> atan3 </td>
          <td valign=\"top\"> Four quadrant inverse tangent (select solution that is closest to given angle y0)
          </td>
</tr>
<tr><td valign=\"top\"> asinh </td>
          <td valign=\"top\"> Inverse of sinh (area hyperbolic sine)
          </td>
</tr>
<tr><td valign=\"top\"> acosh </td>
          <td valign=\"top\"> Inverse of cosh (area hyperbolic cosine)
          </td>
</tr>

<tr><td colspan=\"2\"><b>Math.Vectors</b></td></tr>
<tr><td valign=\"top\"> isEqual </td>
          <td valign=\"top\"> Determine if two Real vectors are numerically identical
          </td>
</tr>
<tr><td valign=\"top\"> norm </td>
          <td valign=\"top\"> Return the p-norm of a vector
          </td></tr>
<tr><td valign=\"top\"> length </td>
          <td valign=\"top\"> Return length of a vector (better as norm(), if further symbolic processing is performed)
          </td></tr>
<tr><td valign=\"top\"> normalize </td>
          <td valign=\"top\"> Return normalized vector such that length = 1 and prevent zero-division for zero vector
          </td></tr>
<tr><td valign=\"top\"> reverse </td>
          <td valign=\"top\"> Reverse vector elements (e.g., v[1] becomes last element)
          </td></tr>
<tr><td valign=\"top\"> sort </td>
          <td valign=\"top\"> Sort elements of vector in ascending or descending order
          </td></tr>

<tr><td colspan=\"2\"><b>Math.Matrices</b></td></tr>
<tr><td valign=\"top\"> solve2 </td>
          <td valign=\"top\"> Solve real system of linear equations A*X=B with a B matrix
                   (Gaussian elimination with partial pivoting)
          </td>
</tr>
<tr><td valign=\"top\"> LU_solve2 </td>
          <td valign=\"top\"> Solve real system of linear equations P*L*U*X=B with a B matrix
                   and an LU decomposition (from LU(..))
          </td></tr>

<tr><td colspan=\"2\"><b>Mechanics.Rotational.</b></td></tr>
<tr><td valign=\"top\"> InitializeFlange </td>
          <td valign=\"top\"> Initialize a flange according to given signals
                   (useful if initialization signals are provided by a signal bus).
          </td>
</tr>

<tr><td colspan=\"2\"><b>Media.Interfaces.PartialMedium.</b></td></tr>
<tr><td valign=\"top\"> density_pTX </td>
          <td valign=\"top\"> Return density from p, T, and X or Xi
          </td>
</tr>

<tr><td colspan=\"2\"><b>Media.Interfaces.PartialTwoPhaseMedium.</b></td></tr>
<tr><td valign=\"top\"> BaseProperties </td>
          <td valign=\"top\"> Base properties (p, d, T, h, u, R, MM, x) of a two phase medium
          </td>
</tr>
<tr><td valign=\"top\"> molarMass </td>
          <td valign=\"top\"> Return the molar mass of the medium
          </td>
</tr>
<tr><td valign=\"top\"> saturationPressure_sat </td>
          <td valign=\"top\"> Return saturation pressure
          </td>
</tr>
<tr><td valign=\"top\"> saturationTemperature_sat </td>
          <td valign=\"top\"> Return saturation temperature
          </td>
</tr>
<tr><td valign=\"top\"> saturationTemperature_derp_sat </td>
          <td valign=\"top\"> Return derivative of saturation temperature w.r.t. pressure
          </td>
</tr>  <tr><td valign=\"top\"> setState_px </td>
          <td valign=\"top\"> Return thermodynamic state from pressure and vapour quality
          </td>
</tr>  <tr><td valign=\"top\"> setState_Tx </td>
          <td valign=\"top\"> Return thermodynamic state from temperature and vapour quality
          </td>
</tr>  <tr><td valign=\"top\"> vapourQuality </td>
          <td valign=\"top\"> Return vapour quality
          </td>
</tr>

<tr><td colspan=\"2\"><b>Media.Interfaces.</b></td></tr>
<tr><td valign=\"top\"> PartialLinearFluid </td>
          <td valign=\"top\"> Generic pure liquid model with constant cp,
                   compressibility and thermal expansion coefficients
          </td>
</tr>

<tr><td colspan=\"2\"><b>Media.Air.MoistAir.</b></td></tr>
<tr><td valign=\"top\"> massFraction_pTphi </td>
          <td valign=\"top\"> Return the steam mass fraction from relative humidity and T
          </td>
</tr>
<tr><td valign=\"top\"> saturationTemperature </td>
          <td valign=\"top\"> Return saturation temperature from (partial) pressure
                   via numerical inversion of function saturationPressure
          </td>
</tr>
<tr><td valign=\"top\"> enthalpyOfWater </td>
          <td valign=\"top\"> Return specific enthalpy of water (solid/liquid) near
                   atmospheric pressure from temperature
          </td>
</tr>
<tr><td valign=\"top\"> enthalpyOfWater_der </td>
          <td valign=\"top\"> Return derivative of enthalpyOfWater()\" function
          </td>
</tr>
<tr><td valign=\"top\"> PsychrometricData </td>
          <td valign=\"top\"> Model to generate plot data for psychrometric chart
          </td>
</tr>

<tr><td colspan=\"2\"><b>Media.CompressibleLiquids.</b><br>
          New sub-library for simple compressible liquid models</td></tr>
<tr><td valign=\"top\"> LinearColdWater </td>
          <td valign=\"top\"> Cold water model with linear compressibility
          </td>
<tr><td valign=\"top\"> LinearWater_pT_Ambient </td>
          <td valign=\"top\"> Liquid, linear compressibility water model at 1.01325 bar
                   and 25 degree Celsius
          </td>

<tr><td colspan=\"2\"><b>SIunits.</b></td></tr>
<tr><td valign=\"top\"> TemperatureDifference </td>
          <td valign=\"top\"> Type for temperature difference
          </td>
</table>

<p><br>
The following <b style=\"color:blue\">existing components</b>
have been <b style=\"color:blue\">improved</b>:
</p>

<table border=\"1\" cellspacing=0 cellpadding=2 style=\"border-collapse:collapse;\">
<tr><td colspan=\"2\"><b>Blocks.Examples.</b></td></tr>
<tr><td valign=\"top\"> BusUsage</td>
          <td valign=\"top\"> Example changed from the \"old\" to the \"new\" bus concept with
                   expandable connectors.</td> </tr>

<tr><td colspan=\"2\"><b>Blocks.Discrete.</b></td></tr>
<tr><td valign=\"top\"> ZeroOrderHold</td>
          <td valign=\"top\"> Sample output ySample moved from \"protected\" to \"public\"
                   section with new attributes (start=0, fixed=true).
          </td>
</tr>
<tr><td valign=\"top\"> TransferFunction</td>
          <td valign=\"top\"> Discrete state x with new attributes (each start=0, each fixed=0).
          </td>
</tr>

<tr><td colspan=\"2\"><b>Electrical.</b></td></tr>
<tr><td valign=\"top\"> Analog<br>MultiPhase</td>
          <td valign=\"top\"> Improved some icons.
          </td>
</tr>

<tr><td colspan=\"2\"><b>Electrical.Digital.Interfaces.</b></td></tr>
<tr><td valign=\"top\"> MISO</td>
          <td valign=\"top\"> Removed \"algorithm\" from this partial block.
          </td>
</tr>

<tr><td colspan=\"2\"><b>Electrical.Digital.Delay.</b></td></tr>
<tr><td valign=\"top\"> DelayParams</td>
          <td valign=\"top\"> Removed \"algorithm\" from this partial block.
          </td>
</tr>

<tr><td colspan=\"2\"><b>Electrical.Digital.Delay.</b></td></tr>
<tr><td valign=\"top\"> DelayParams</td>
          <td valign=\"top\"> Removed \"algorithm\" from this partial block.
          </td>
</tr>
<tr><td valign=\"top\"> TransportDelay</td>
          <td valign=\"top\">  If delay time is zero, an infinitely small delay is
                        introduced via pre(x) (previously \"x\" was used).
          </td>
</tr>

<tr><td colspan=\"2\"><b>Electrical.Digital.Sources.</b></td></tr>
<tr><td valign=\"top\"> Clock<br>Step</td>
          <td valign=\"top\"> Changed if-conditions from \"xxx < time\" to \"time >= xxx\"
                   (according to the Modelica specification, in the second case
                   a time event should be triggered, i.e., this change leads
                   potentially to a faster simulation).
          </td>
</tr>

<tr><td colspan=\"2\"><b>Electrical.Digital.Converters.</b></td></tr>
<tr><td valign=\"top\"> BooleanToLogic<br>
                   LogicToBoolean<br>
                   RealToLogic<br>
                   LogicToReal</td>
          <td valign=\"top\"> Changed from \"algorithm\" to \"equation\" section
                   to allow better symbolic preprocessing
          </td>
</tr>

<tr><td colspan=\"2\"><b>Electrical.</b></td></tr>
<tr><td valign=\"top\"> Machines</td>
          <td valign=\"top\"> Slightly improved documentation, typos in
                   documentation corrected
          </td>
</tr>

<tr><td colspan=\"2\"><b>Electrical.Machines.Examples.</b></td></tr>
<tr><td valign=\"top\"> AIMS_start</td>
          <td valign=\"top\"> Changed QuadraticLoadTorque1(TorqueDirection=true) to
                   QuadraticLoadTorque1(TorqueDirection=false) since more realistic
          </td>
</tr>

<tr><td colspan=\"2\"><b>Electrical.Machines.Interfaces.</b></td></tr>
<tr><td valign=\"top\"> PartialBasicMachine</td>
          <td valign=\"top\"> Introduced support flange to model the
                   reaction torque to the housing
          </td>
</tr>

<tr><td colspan=\"2\"><b>Electrical.Machines.Sensors.</b></td></tr>
<tr><td valign=\"top\"> Rotorangle</td>
          <td valign=\"top\"> Introduced support flange to model the
                   reaction torque to the housing
          </td>
</tr>

<tr><td colspan=\"2\"><b>Mechanics.MultiBody.Examples.Elementary.</b></td></tr>
<tr><td valign=\"top\"> PointMassesWithGravity</td>
          <td valign=\"top\"> Added two point masses connected by a line force to demonstrate
                   additionally how this works. Connections of point masses
                   with 3D-elements are demonstrated in the new example
                   PointMassesWithGravity (there is the difficulty that the orientation
                   is not defined in a PointMass object and therefore some
                   special handling is needed in case of a connection with
                   3D-elements, where the orientation of the point mass is not
                   determined by these elements.</td>
</tr>

<tr><td colspan=\"2\"><b>Mechanics.MultiBody.Examples.Systems.</b></td></tr>
<tr><td valign=\"top\"> RobotR3</td>
          <td valign=\"top\"> Changed from the \"old\" to the \"new\" bus concept with expandable connectors.
                   Replaced the non-standard Modelica function \"constrain()\" by
                   standard Modelica components. As a result, the non-standard function
                   constrain() is no longer used in the Modelica Standard Library.</td>
</tr>

<tr><td colspan=\"2\"><b>Mechanics.MultiBody.Frames.Orientation.</b></td></tr>
<tr><td valign=\"top\"> equalityConstraint</td>
          <td valign=\"top\"> Use a better residual for the equalityConstraint function.
                   As a result, the non-linear equation system of a kinematic
                   loop is formulated in a better way (the range where the
                   desired result is a unique solution of the non-linear
                   system of equations becomes much larger).</td>
</tr>

<tr><td colspan=\"2\"><b>Mechanics.MultiBody.</b></td></tr>
<tr><td valign=\"top\"> Visualizers.</td>
          <td valign=\"top\"> Removed (misleading) annotation \"structurallyIncomplete\"
                   in the models of this sub-library
          </td>
</tr>

<tr><td colspan=\"2\"><b>Mechanics.Rotational.</b></td></tr>
<tr><td valign=\"top\"> Examples</td>
          <td valign=\"top\"> For all models in this sub-library:
                   <ul>
                   <li> Included a housing object in all examples to compute
                                all support torques.</li>
                   <li> Replaced initialization by modifiers via the
                                initialization menu parameters of Inertia components.</li>
                   <li> Removed \"encapsulated\" and unnecessary \"import\". </li>
                   <li> Included \"StopTime\" in the annotations.</li>
                   </ul>
          </td>
</tr>

<tr><td colspan=\"2\"><b>Mechanics.Rotational.Interfaces.</b></td></tr>
<tr><td valign=\"top\"> FrictionBase</td>
          <td valign=\"top\"> Introduced \"fixed=true\" for Boolean variables startForward,
                   startBackward, mode.
          </td>
</tr>

<tr><td colspan=\"2\"><b>Mechanics.Translational.Interfaces.</b></td></tr>
<tr><td valign=\"top\"> FrictionBase</td>
          <td valign=\"top\"> Introduced \"fixed=true\" for Boolean variables startForward,
                   startBackward, mode.
          </td>
</tr>

<tr><td colspan=\"2\"><b>Media.UsersGuide.MediumUsage.</b></td></tr>
<tr><td valign=\"top\"> TwoPhase</td>
          <td valign=\"top\"> Improved documentation and demonstrating the newly introduced functions
          </td>
</tr>

<tr><td colspan=\"2\"><b>Media.Examples.</b></td></tr>
<tr><td valign=\"top\"> WaterIF97</td>
          <td valign=\"top\"> Provided (missing) units for variables V, dV, H_flow_ext, m, U.
          </td>
</tr>

<tr><td colspan=\"2\"><b> Media.Interfaces.</b></td></tr>
<tr><td valign=\"top\"> PartialMedium</td>
          <td valign=\"top\"> Final modifiers are removed from nX and nXi, to allow
                   customized medium models such as mixtures of refrigerants with oil, etc.
          </td>
</tr>
<tr><td valign=\"top\"> PartialCondensingGases</td>
          <td valign=\"top\"> Included attributes \"min=1, max=2\" for input argument FixedPhase
                   for functions setDewState and setBubbleState (in order to guarantee
                   that input arguments are correct).
          </td>
</tr>

<tr><td colspan=\"2\"><b> Media.Interfaces.PartialMedium.</b></td></tr>
<tr><td valign=\"top\"> BaseProperties</td>
          <td valign=\"top\"> New Boolean parameter \"standardOrderComponents\".
                   If true, last element vector X is computed from 1-sum(Xi) (= default)
                   otherwise, no equation is provided for it in PartialMedium.
          </td>
</tr>
<tr><td valign=\"top\"> IsentropicExponent</td>
          <td valign=\"top\"> \"max\" value changed from 1.7 to 500000
          </td>
</tr>
<tr><td valign=\"top\"> setState_pTX<br>
                   setState_phX<br>
                   setState_psX<br>
                   setState_dTX<br>
                   specificEnthalpy_pTX<br>
                   temperature_phX<br>
                   density_phX<br>
                   temperature_psX<br>
                   density_psX<br>
                   specificEnthalpy_psX</td>
          <td valign=\"top\"> Introduced default value \"reference_X\" for input argument \"X\".
          </td>
</tr>

<tr><td colspan=\"2\"><b> Media.Interfaces.PartialSimpleMedium.</b></td></tr>
<tr><td valign=\"top\"> setState_pTX<br>
                   setState_phX<br>
                   setState_psX<br>
                   setState_dTX</td>
          <td valign=\"top\"> Introduced default value \"reference_X\" for input argument \"X\".
          </td>
</tr>

<tr><td colspan=\"2\"><b> Media.Interfaces.PartialSimpleIdealGasMedium.</b></td></tr>
<tr><td valign=\"top\"> setState_pTX<br>
                   setState_phX<br>
                   setState_psX<br>
                   setState_dTX</td>
          <td valign=\"top\"> Introduced default value \"reference_X\" for input argument \"X\".
          </td>
</tr>

<tr><td colspan=\"2\"><b> Media.Air.MoistAir.</b></td></tr>
<tr><td valign=\"top\"> setState_pTX<br>
                   setState_phX<br>
                   setState_dTX</td>
          <td valign=\"top\"> Introduced default value \"reference_X\" for input argument \"X\".
          </td>
</tr>

<tr><td colspan=\"2\"><b> Media.IdealGases.Common.SingleGasNasa.</b></td></tr>
<tr><td valign=\"top\"> setState_pTX<br>
                   setState_phX<br>
                   setState_psX<br>
                   setState_dTX</td>
          <td valign=\"top\"> Introduced default value \"reference_X\" for input argument \"X\".
          </td>
</tr>

<tr><td colspan=\"2\"><b> Media.IdealGases.Common.MixtureGasNasa.</b></td></tr>
<tr><td valign=\"top\"> setState_pTX<br>
                   setState_phX<br>
                   setState_psX<br>
                   setState_dTX<br>
                   h_TX</td>
          <td valign=\"top\"> Introduced default value \"reference_X\" for input argument \"X\".
          </td>
</tr>

<tr><td colspan=\"2\"><b> Media.Common.</b></td></tr>
<tr><td valign=\"top\"> IF97PhaseBoundaryProperties<br>
                   gibbsToBridgmansTables </td>
          <td valign=\"top\"> Introduced unit for variables vt, vp.
          </td>
</tr>
<tr><td valign=\"top\"> SaturationProperties</td>
          <td valign=\"top\"> Introduced unit for variable dpT.
          </td>
</tr>
<tr><td valign=\"top\"> BridgmansTables</td>
          <td valign=\"top\"> Introduced unit for dfs, dgs.
          </td>
</tr>

<tr><td colspan=\"2\"><b> Media.Common.ThermoFluidSpecial.</b></td></tr>
<tr><td valign=\"top\"> gibbsToProps_ph<br>
                   gibbsToProps_ph  <br>
                   gibbsToBoundaryProps<br>
                   gibbsToProps_dT<br>
                   gibbsToProps_pT</td>
          <td valign=\"top\"> Introduced unit for variables vt, vp.
          </td>
<tr><td valign=\"top\"> TwoPhaseToProps_ph</td>
          <td valign=\"top\"> Introduced unit for variables dht, dhd, detph.
          </td>
</tr>

<tr><td colspan=\"2\"><b> Media.</b></td></tr>
<tr><td valign=\"top\"> MoistAir</td>
          <td valign=\"top\"> Documentation of moist air model significantly improved.
          </td>
</tr>

<tr><td colspan=\"2\"><b> Media.MoistAir.</b></td></tr>
<tr><td valign=\"top\"> enthalpyOfVaporization</td>
          <td valign=\"top\"> Replaced by linear correlation since simpler and more
                   accurate in the entire region.
          </td>
</tr>

<tr><td colspan=\"2\"><b> Media.Water.IF97_Utilities.BaseIF97.Regions.</b></td></tr>
<tr><td valign=\"top\"> drhovl_dp</td>
          <td valign=\"top\"> Introduced unit for variable dd_dp.
          </td>
</tr>

<tr><td colspan=\"2\"><b> Thermal.</b></td></tr>
<tr><td valign=\"top\"> FluidHeatFlow</td>
          <td valign=\"top\"> Introduced new parameter tapT (0..1) to define the
                   temperature of the HeatPort as linear combination of the
                   flowPort_a (tapT=0) and flowPort_b (tapT=1) temperatures.
          </td>
</tr>
</table>

<p><br>
The following <b style=\"color:red\">critical errors</b> have been fixed (i.e., errors
that can lead to wrong simulation results):
</p>

<table border=\"1\" cellspacing=0 cellpadding=2 style=\"border-collapse:collapse;\">
<tr><td colspan=\"2\"><b>Electrical.Machines.BasicMachines.Components.</b></td></tr>
<tr><td valign=\"top\"> ElectricalExcitation</td>
          <td valign=\"top\"> Excitation voltage ve is calculated as
                   \"spacePhasor_r.v_[1]*TurnsRatio*3/2\" instead of
                   \"spacePhasor_r.v_[1]*TurnsRatio
           </td>
</tr>

<tr><td colspan=\"2\"><b>Mechanics.MultiBody.Parts.</b></td></tr>
<tr><td valign=\"top\"> FixedRotation</td>
          <td valign=\"top\"> Bug corrected that the torque balance was wrong in the
                   following cases (since vector r was not transformed
                   from frame_a to frame_b; note this special case occurs very seldom in practice):
                   <ul><li> frame_b is in the spanning tree closer to the root
                                        (usually this is frame_a).</li>
                           <li> vector r from frame_a to frame_b is not zero.</li>
                   </ul>
           </td>
</tr>

<tr><td valign=\"top\"> PointMass</td>
         <td valign=\"top\"> If a PointMass model is connected so that no equations are present
                  to compute its orientation object, the orientation was arbitrarily
                  set to a unit rotation. In some cases this can lead to a wrong overall
                  model, depending on how the PointMass model is used. For this reason,
                  such cases lead now to an error (via an assert(..)) with an explanation
                  how to fix this.
           </td>
</tr>

<tr><td colspan=\"2\"><b>Media.Interfaces.PartialPureSubstance.</b></td></tr>
<tr><td valign=\"top\"> pressure_dT<br>
                   specificEnthalpy_dT
          </td>
          <td valign=\"top\"> Changed wrong call from \"setState_pTX\" to \"setState_dTX\"
           </td>
</tr>

<tr><td colspan=\"2\"><b>Media.Interfaces.PartialTwoPhaseMedium.</b></td></tr>
<tr><td valign=\"top\"> pressure_dT<br>
                   specificEnthalpy_dT
          </td>
          <td valign=\"top\"> Changed wrong call from \"setState_pTX\" to \"setState_dTX\"
           </td>
</tr>

<tr><td colspan=\"2\"><b>Media.Common.ThermoFluidSpecial.</b></td></tr>
<tr><td valign=\"top\"> gibbsToProps_dT<br>
                   helmholtzToProps_ph<br>
                   helmholtzToProps_pT<br>
                   helmholtzToProps_dT</td>
          <td valign=\"top\"> Bugs in equations corrected </td>
</tr>

<tr><td colspan=\"2\"><b>Media.Common.</b></td></tr>
<tr><td valign=\"top\"> helmholtzToBridgmansTables<br>
                   helmholtzToExtraDerivs</td>
          <td valign=\"top\"> Bugs in equations corrected </td>
</tr>

<tr><td colspan=\"2\"><b>Media.IdealGases.Common.SingleGasNasa.</b></td></tr>
<tr><td valign=\"top\"> density_derp_T</td>
          <td valign=\"top\"> Bug in equation of partial derivative corrected </td>
</tr>

<tr><td colspan=\"2\"><b>Media.Water.IF97_Utilities.</b></td></tr>
<tr><td valign=\"top\"> BaseIF97.Inverses.dtofps3<br>
                   isentropicExponent_props_ph<br>
                   isentropicExponent_props_pT<br>
                   isentropicExponent_props_dT</td>
          <td valign=\"top\"> Bugs in equations corrected </td>
</tr>

<tr><td colspan=\"2\"><b>Media.Air.MoistAir.</b></td></tr>
<tr><td valign=\"top\"> h_pTX</td>
          <td valign=\"top\"> Bug in setState_phX due to wrong vector size in h_pTX corrected.
                   Furthermore, syntactical errors corrected:
                   <ul><li> In function massFractionpTphi an equation
                                        sign is used in an algorithm.</li>
                           <li> Two consecutive semicolons removed</li>
                   </ul>
          </td>
</tr>

<tr><td colspan=\"2\"><b>Media.Water.</b></td></tr>
<tr><td valign=\"top\"> waterConstants</td>
          <td valign=\"top\"> Bug in equation of criticalMolarVolume corrected.
          </td>
</tr>

<tr><td colspan=\"2\"><b>Media.Water.IF97_Utilities.BaseIF97.Regions.</b></td></tr>
<tr><td valign=\"top\"> region_ph<br>
                   region_ps</td>
          <td valign=\"top\"> Bug in region determination corrected.
          </td>
</tr>

<tr><td valign=\"top\"> boilingcurve_p<br>
                   dewcurve_p</td>
          <td valign=\"top\"> Bug in equation of plim corrected.
          </td>
</tr>
</table>

<p><br>
The following <b style=\"color:red\">uncritical errors</b> have been fixed (i.e., errors
that do <b style=\"color:red\">not</b> lead to wrong simulation results, but, e.g.,
units are wrong or errors in documentation):
</p>

<table border=\"1\" cellspacing=0 cellpadding=2 style=\"border-collapse:collapse;\">
<tr><td colspan=\"2\"><b>Blocks.</b></td></tr>
<tr><td valign=\"top\"> Examples</td>
          <td valign=\"top\"> Corrected typos in description texts of bus example models.
           </td>
</tr>

<tr><td colspan=\"2\"><b>Blocks.Continuous.</b></td></tr>
<tr><td valign=\"top\"> LimIntegrator</td>
          <td valign=\"top\"> removed incorrect smooth(0,..) because expression might be discontinuous.
           </td>
</tr>

<tr><td colspan=\"2\"><b>Blocks.Math.UnitConversions.</b></td></tr>
<tr><td valign=\"top\"> block_To_kWh<br>block_From_kWh</td>
          <td valign=\"top\"> Corrected unit from \"kWh\" to (syntactically correct) \"kW.h\".
           </td>
</tr>

<tr><td colspan=\"2\"><b>Electrical.Analog.Examples.</b></td></tr>
<tr><td valign=\"top\"> HeatingNPN_OrGate</td>
          <td valign=\"top\"> Included start values, so that initialization is
                                                successful </td>
</tr>

<tr><td colspan=\"2\"><b>Electrical.Analog.Lines.</b></td></tr>
<tr><td valign=\"top\"> OLine</td>
          <td valign=\"top\"> Corrected unit from \"Siemens/m\" to \"S/m\".
           </td>
<tr><td valign=\"top\"> TLine2</td>
          <td valign=\"top\"> Changed wrong type of parameter NL (normalized length) from
                   SIunits.Length to Real.
           </td>
</tr>

<tr><td colspan=\"2\"><b>Electrical.Digital.Delay.</b></td></tr>
<tr><td valign=\"top\"> TransportDelay</td>
          <td valign=\"top\"> Syntax error corrected
                   (\":=\" in equation section is converted by Dymola silently to \"=\").
           </td>
</tr>

<tr><td colspan=\"2\"><b>Electrical.Digital</b></td></tr>
<tr><td valign=\"top\"> Converters</td>
          <td valign=\"top\"> Syntax error corrected
                   (\":=\" in equation section is converted by Dymola silently to \"=\").
           </td>
</tr>

<tr><td colspan=\"2\"><b>Electrical.MultiPhase.Basic.</b></td></tr>
<tr><td valign=\"top\"> Conductor</td>
          <td valign=\"top\"> Changed wrong type of parameter G from SIunits.Resistance to
                   SIunits.Conductance.
           </td>
</tr>

<tr><td colspan=\"2\"><b>Electrical.MultiPhase.Interfaces.</b></td></tr>
<tr><td valign=\"top\"> Plug<br></td>
          <td valign=\"top\"> Made used \"pin\" connectors non-graphical (otherwise,
                   there are difficulties to connect to Plug).
           </td>
</tr>

<tr><td colspan=\"2\"><b>Electrical.MultiPhase.Sources.</b></td></tr>
<tr><td valign=\"top\"> SineCurrent</td>
          <td valign=\"top\"> Changed wrong type of parameter offset from SIunits.Voltage to
                   SIunits.Current.
           </td>
</tr>

<tr><td colspan=\"2\"><b>Mechanics.MultiBody.Examples.Loops.</b></td></tr>
<tr><td valign=\"top\"> EngineV6</td>
          <td valign=\"top\"> Corrected wrong crankAngleOffset of some cylinders
                   and improved the example.
           </td>
</tr>

<tr><td colspan=\"2\"><b>Mechanics.MultiBody.Examples.Loops.Utilities.</b></td></tr>
<tr><td valign=\"top\"> GasForce</td>
          <td valign=\"top\"> Wrong units corrected:
                   \"SIunitsPosition x,y\" to \"Real x,y\";
           \"SIunits.Pressure press\" to \"SIunits.Conversions.NonSIunits.Pressure_bar\"
           </td>
</tr>
<tr><td valign=\"top\"> GasForce2</td>
          <td valign=\"top\"> Wrong unit corrected: \"SIunits.Position x\" to \"Real x\".
           </td>
</tr>
<tr><td valign=\"top\"> EngineV6_analytic</td>
          <td valign=\"top\"> Corrected wrong crankAngleOffset of some cylinders.
           </td>
</tr>

<tr><td colspan=\"2\"><b>Mechanics.MultiBody.Interfaces.</b></td></tr>
<tr><td valign=\"top\"> PartialLineForce</td>
          <td valign=\"top\"> Corrected wrong unit: \"SIunits.Position eRod_a\" to \"Real eRod_a\";
           </td>
</tr>
<tr><td valign=\"top\"> FlangeWithBearingAdaptor </td>
          <td valign=\"top\"> If includeBearingConnector = false, connector \"fr\"
                           + \"ame\" was not
                   removed. As long as the connecting element to \"frame\" determines
                   the non-flow variables, this is fine. In the corrected version, \"frame\"
                   is conditionally removed in this case.</td>
</tr>

<tr><td colspan=\"2\"><b>Mechanics.MultiBody.Forces.</b></td></tr>
<tr><td valign=\"top\"> ForceAndTorque</td>
          <td valign=\"top\"> Corrected wrong unit: \"SIunits.Force t_b_0\" to \"SIunits.Torque t_b_0\".
           </td>
</tr>
<tr><td valign=\"top\"> LineForceWithTwoMasses</td>
          <td valign=\"top\"> Corrected wrong unit: \"SIunits.Position e_rel_0\" to \"Real e_rel_0\".
           </td>
</tr>

<tr><td colspan=\"2\"><b>Mechanics.MultiBody.Frames.</b></td></tr>
<tr><td valign=\"top\"> axisRotation</td>
          <td valign=\"top\"> Corrected wrong unit: \"SIunits.Angle der_angle\" to
                        \"SIunits.AngularVelocity der_angle\".
           </td>
</tr>

<tr><td colspan=\"2\"><b>Mechanics.MultiBody.Joints.Assemblies.</b></td></tr>
<tr><td valign=\"top\"> JointUSP<br>JointSSP</td>
          <td valign=\"top\"> Corrected wrong unit: \"SIunits.Position aux\"  to \"Real\"
           </td>
</tr>

<tr><td colspan=\"2\"><b>Mechanics.MultiBody.Sensors.</b></td></tr>
<tr><td valign=\"top\"> AbsoluteSensor</td>
          <td valign=\"top\"> Corrected wrong units: \"SIunits.Acceleration angles\" to
                   \"SIunits.Angle angles\" and
                   \"SIunits.Velocity w_abs_0\" to \"SIunits.AngularVelocity w_abs_0\"
           </td>
</tr>
<tr><td valign=\"top\"> RelativeSensor</td>
          <td valign=\"top\"> Corrected wrong units: \"SIunits.Acceleration angles\" to
                   \"SIunits.Angle angles\"
           </td>
</tr>
<tr><td valign=\"top\"> Distance</td>
          <td valign=\"top\"> Corrected wrong units: \"SIunits.Length L2\" to \"SIunits.Area L2\" and
                   SIunits.Length s_small2\" to \"SIunits.Area s_small2\"
           </td>
</tr>

<tr><td colspan=\"2\"><b>Mechanics.MultiBody.Visualizers.Advanced.</b></td></tr>
<tr><td valign=\"top\"> Shape</td>
          <td valign=\"top\"> Changed \"MultiBody.Types.Color color\" to \"Real color[3]\", since
                   \"Types.Color\" is \"Integer color[3]\" and there have been backward
                   compatibility problems with models using \"color\" before it was changed
                   to \"Types.Color\".
           </td>
</tr>

<tr><td colspan=\"2\"><b>Mechanics.Rotational.Interfaces.</b></td></tr>
<tr><td valign=\"top\"> FrictionBase</td>
          <td valign=\"top\"> Rewrote equations with new variables \"unitAngularAcceleration\" and
                   \"unitTorque\" in order that the equations are correct with respect
                   to units (previously, variable \"s\" can be both a torque and an
                   angular acceleration and this lead to unit incompatibilities)
           </td>
</tr>

<tr><td colspan=\"2\"><b>Mechanics.Rotational.</b></td></tr>
<tr><td valign=\"top\"> OneWayClutch<br>LossyGear</td>
          <td valign=\"top\"> Rewrote equations with new variables \"unitAngularAcceleration\" and
                   \"unitTorque\" in order that the equations are correct with respect
                   to units (previously, variable \"s\" can be both a torque and an
                   angular acceleration and this lead to unit incompatibilities)
           </td>
</tr>

<tr><td colspan=\"2\"><b>Mechanics.Translational.Interfaces.</b></td></tr>
<tr><td valign=\"top\"> FrictionBase</td>
          <td valign=\"top\"> Rewrote equations with new variables \"unitAngularAcceleration\" and
                   \"unitTorque\" in order that the equations are correct with respect
                   to units (previously, variable \"s\" can be both a torque and an
                   angular acceleration and this lead to unit incompatibilities)
           </td>
</tr>

<tr><td colspan=\"2\"><b>Mechanics.Translational.</b></td></tr>
<tr><td valign=\"top\"> Speed</td>
          <td valign=\"top\"> Corrected unit of v_ref from SIunits.Position to SIunits.Velocity
           </td>
</tr>

<tr><td colspan=\"2\"><b>Media.Examples.Tests.Components.</b></td></tr>
<tr><td valign=\"top\"> PartialTestModel<br>PartialTestModel2</td>
          <td valign=\"top\"> Corrected unit of h_start from \"SIunits.Density\" to \"SIunits.SpecificEnthalpy\"
           </td>
</tr>

<tr><td colspan=\"2\"><b>Media.Examples.SolveOneNonlinearEquation.</b></td></tr>
<tr><td valign=\"top\"> Inverse_sh_T
                   InverseIncompressible_sh_T<br>
                   Inverse_sh_TX</td>
          <td valign=\"top\"> Rewrote equations so that dimensional (unit) analysis is correct\"
           </td>
</tr>

<tr><td colspan=\"2\"><b>Media.Incompressible.Examples.</b></td></tr>
<tr><td valign=\"top\"> TestGlycol</td>
          <td valign=\"top\"> Rewrote equations so that dimensional (unit) analysis is correct\"
           </td>
</tr>

<tr><td colspan=\"2\"><b>Media.Interfaces.PartialTwoPhaseMedium.</b></td></tr>
<tr><td valign=\"top\"> dBubbleDensity_dPressure<br>dDewDensity_dPressure</td>
          <td valign=\"top\"> Changed wrong type of ddldp from \"DerDensityByEnthalpy\"
                   to \"DerDensityByPressure\".
           </td>
</tr>

<tr><td colspan=\"2\"><b>Media.Common.ThermoFluidSpecial.</b></td></tr>
<tr><td valign=\"top\"> ThermoProperties</td>
          <td valign=\"top\"> Changed wrong units:
                   \"SIunits.DerEnergyByPressure dupT\" to \"Real dupT\" and
                   \"SIunits.DerEnergyByDensity dudT\" to \"Real dudT\"
           </td>
</tr>
<tr><td valign=\"top\"> ThermoProperties_ph</td>
          <td valign=\"top\"> Changed wrong unit from \"SIunits.DerEnergyByPressure duph\" to \"Real duph\"
           </td>
</tr>
<tr><td valign=\"top\"> ThermoProperties_pT</td>
          <td valign=\"top\"> Changed wrong unit from \"SIunits.DerEnergyByPressure dupT\" to \"Real dupT\"
           </td>
</tr>
<tr><td valign=\"top\"> ThermoProperties_dT</td>
          <td valign=\"top\">  Changed wrong unit from \"SIunits.DerEnergyByDensity dudT\" to \"Real dudT\"
           </td>
</tr>

<tr><td colspan=\"2\"><b>Media.IdealGases.Common.SingleGasNasa.</b></td></tr>
<tr><td valign=\"top\"> cp_Tlow_der</td>
          <td valign=\"top\"> Changed wrong unit from \"SIunits.Temperature dT\" to \"Real dT\".
           </td>
</tr>

<tr><td colspan=\"2\"><b>Media.Water.IF97_Utilities.BaseIF97.Basic.</b></td></tr>
<tr><td valign=\"top\"> p1_hs<br>
                   h2ab_s<br>
                   p2a_hs<br>
                   p2b_hs<br>
                   p2c_hs<br>
                   h3ab_p<br>
                   T3a_ph<br>
                   T3b_ph<br>
                   v3a_ph<br>
                   v3b_ph<br>
                   T3a_ps<br>
                   T3b_ps<br>
                   v3a_ps<br>
                   v3b_ps</td>
          <td valign=\"top\"> Changed wrong unit of variables h/hstar, s, sstar from
                   \"SIunits.Enthalpy\" to \"SIunits.SpecificEnthalpy\",
                   \"SIunits.SpecificEntropy\", \"SIunits.SpecificEntropy
           </td>
</tr>

<tr><td colspan=\"2\"><b>Media.Water.IF97_Utilities.BaseIF97.Transport.</b></td></tr>
<tr><td valign=\"top\"> cond_dTp</td>
          <td valign=\"top\"> Changed wrong unit of TREL, rhoREL, lambdaREL from
                   \"SIunits.Temperature\", \"SIunit.Density\", \"SIunits.ThermalConductivity\"
                   to \"Real\".
           </td>
</tr>

<tr><td colspan=\"2\"><b>Media.Water.IF97_Utilities.BaseIF97.Inverses.</b></td></tr>
<tr><td valign=\"top\"> tofps5<br>tofpst5</td>
          <td valign=\"top\"> Changed wrong unit of pros from \"SIunits.SpecificEnthalpy\" to
                   \"SIunits.SpecificEntropy\".
           </td>
</tr>

<tr><td colspan=\"2\"><b>Media.Water.IF97_Utilities.</b></td></tr>
<tr><td valign=\"top\"> waterBaseProp_ph</td>
          <td valign=\"top\"> Improved calculation at the limits of the validity.
           </td>
</tr>

        <tr><td colspan=\"2\"><b>Thermal.</b></td></tr>
<tr><td valign=\"top\"> FluidHeatFlow<br>HeatTransfer</td>
          <td valign=\"top\"> Corrected wrong unit \"SIunits.Temperature\" of difference temperature
                        variables with \"SIunits.TemperatureDifference\".
           </td>
</tr>

</table>

</html>"));
      end Version_2_2_2;

      class Version_2_2_1 "Version 2.2.1 (March 24, 2006)"
        extends Modelica.Icons.ReleaseNotes;
        annotation(
          Documentation(info = "<html>

<p>
Version 2.2.1 is backward compatible to version 2.2.
</p>

<p>
In this version, <b>no</b> new libraries have been added.
The following major improvements have been made:
</p>

<ul>
<li> The <b>Documentation</b> of the Modelica standard library was
         considerably improved:<br>
         In Dymola 6, the new feature was introduced to automatically add tables
         for class content and component interface definitions (parameters and
         connectors) to the info layer. For this reason, the corresponding (partial)
         tables previously present in the Modelica Standard Library have been
         removed. The new feature of Dymola 6 has the significant advantage that
         all tables are now guaranteed to be up-to-date.<br>
         Additionally, the documentation has been improved by adding appropriate
         description texts to parameters, connector instances, function input
         and output arguments etc., in order that the automatically generated
         tables do not have empty entries. Also new User's Guides for sublibraries
         Rotational and SIunits have been added and the User's Guide on top
         level (Modelica.UsersGuide) has been improved.<br>&nbsp;</li>

<li> Initialization options have been added to the Modelica.Blocks.<b>Continuous</b>
         blocks (NoInit, SteadyState, InitialState, InitialOutput). If InitialOutput
         is selected, the block output is provided as initial condition. The states
         of the block are then initialized as close as possible to steady state.
         Furthermore, the Continuous.LimPID block has been significantly
         improved and much better documented.<br>&nbsp;</li>

<li> The Modelica.<b>Media</b> library has been significantly improved:<br>
         New functions setState_pTX, setState_phX, setState_psX, setState_dTX
         have been added to PartialMedium to compute the independent medium variables
         (= state of medium) from p,T,X, or from p,h,X or from p,s,X or from
         d,T,X. Then functions are provided for all interesting medium variables
         to compute them from its medium state. All these functions are
         implemented in a robust way for all media (with a few exceptions, if the
         generic function does not make sense for a particular medium).</li>
</ul>

<p>
The following <b>new components</b> have been added to <b>existing</b> libraries:
</p>

<table border=\"1\" cellspacing=0 cellpadding=2>
<tr><td colspan=\"2\"><b>Modelica.Blocks.Examples.</b></td></tr>
<tr><td valign=\"top\"> PID_Controller</td>
          <td valign=\"top\"> Example to demonstrate the usage of the
                   Blocks.Continuous.LimPID block.</td> </tr>
<tr><td colspan=\"2\"><b>Modelica.Blocks.Math.</b></td></tr>
<tr><td valign=\"top\"> UnitConversions.*</td>
          <td valign=\"top\"> New package that provides blocks for unit conversions.
                   UnitConversions.ConvertAllBlocks allows to select all
                   available conversions from a menu.</td> </tr>
<tr><td colspan=\"2\"><b>Modelica.Electrical.Machines.BasicMachines.SynchronousInductionMachines.</b></td></tr>
<tr><td valign=\"top\"> SM_ElectricalExcitedDamperCage</td>
          <td valign=\"top\"> Electrical excited synchronous induction machine with damper cage</td> </tr>
<tr><td colspan=\"2\"><b>Modelica.Electrical.Machines.BasicMachines.Components.</b></td></tr>
<tr><td valign=\"top\"> ElectricalExcitation </td>
          <td valign=\"top\"> Electrical excitation for electrical excited synchronous
                   induction machines</td> </tr>
<tr><td valign=\"top\"> DamperCage</td>
          <td valign=\"top\"> Unsymmetrical damper cage for electrical excited synchronous
                   induction machines. At least the user has to specify the dampers
                   resistance and stray inductance in d-axis; if he omits the
                   parameters of the q-axis, the same values as for the d.axis
                   are used, assuming a symmetrical damper.</td> </tr>
<tr><td colspan=\"2\"><b>Modelica.Electrical.Machines.Examples.</b></td></tr>
<tr><td valign=\"top\"> SMEE_Gen </td>
          <td valign=\"top\"> Test example 7: ElectricalExcitedSynchronousInductionMachine
                   as Generator</td> </tr>
<tr><td valign=\"top\"> Utilities.TerminalBox</td>
          <td valign=\"top\"> Terminal box for three-phase induction machines to choose
                   either star (wye) ? or delta ? connection</td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Math.Matrices.</b></td></tr>
<tr><td valign=\"top\"> equalityLeastSquares</td>
          <td valign=\"top\"> Solve a linear equality constrained least squares problem:<br>
                  min|A*x-a|^2 subject to B*x=b</td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Mechanics.MultiBody.</b></td></tr>
<tr><td valign=\"top\"> Parts.PointMass</td>
          <td valign=\"top\"> Point mass, i.e., body where inertia tensor is neglected.</td> </tr>
<tr><td valign=\"top\"> Interfaces.FlangeWithBearing</td>
          <td valign=\"top\"> Connector consisting of 1-dim. rotational flange and its
                   3-dim. bearing frame.</td> </tr>
<tr><td valign=\"top\"> Interfaces.FlangeWithBearingAdaptor</td>
          <td valign=\"top\"> Adaptor to allow direct connections to the sub-connectors
                   of FlangeWithBearing.</td> </tr>
<tr><td valign=\"top\"> Types.SpecularCoefficient</td>
          <td valign=\"top\"> New type to define a specular coefficient.</td> </tr>
<tr><td valign=\"top\"> Types.ShapeExtra</td>
          <td valign=\"top\"> New type to define the extra data for visual shape objects and to
                   have a central place for the documentation of this data.</td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Mechanics.MultiBody.Examples.Elementary</b></td></tr>
<tr><td valign=\"top\"> PointGravityWithPointMasses</td>
          <td valign=\"top\"> Example of two point masses in a central gravity field.</td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Mechanics.Rotational.</b></td></tr>
<tr><td valign=\"top\">UsersGuide</td>
          <td valign=\"top\"> A User's Guide has been added by using the documentation previously
                   present in the package documentation of Rotational.</td> </tr>
<tr><td valign=\"top\">Sensors.PowerSensor</td>
          <td valign=\"top\"> New component to measure the energy flow between two connectors
                   of the Rotational library.</td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Mechanics.Translational.</b></td></tr>
<tr><td valign=\"top\">Speed</td>
          <td valign=\"top\"> New component to move a translational flange
                   according to a reference speed</td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Media.Interfaces.PartialMedium.</b></td></tr>
<tr><td valign=\"top\">specificEnthalpy_pTX</td>
          <td valign=\"top\"> New function to compute specific enthalpy from pressure, temperature
                   and mass fractions.</td> </tr>
<tr><td valign=\"top\">temperature_phX</td>
          <td valign=\"top\"> New function to compute temperature from pressure, specific enthalpy,
                   and mass fractions.</td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Icons.</b></td></tr>
<tr><td valign=\"top\"> SignalBus</td>
          <td valign=\"top\"> Icon for signal bus</td> </tr>
<tr><td valign=\"top\"> SignalSubBus</td>
          <td valign=\"top\"> Icon for signal sub-bus</td> </tr>

<tr><td colspan=\"2\"><b>Modelica.SIunits.</b></td></tr>
<tr><td valign=\"top\">UsersGuide</td>
          <td valign=\"top\"> A User's Guide has been added that describes unit handling.</td> </tr>
<tr><td valign=\"top\"> Resistance<br>
                   Conductance</td>
          <td valign=\"top\"> Attribute 'min=0' removed from these types.</td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Thermal.FluidHeatFlow.</b></td></tr>
<tr><td valign=\"top\"> Components.Valve</td>
          <td valign=\"top\"> Simple controlled valve with either linear or
                   exponential characteristic.</td> </tr>
<tr><td valign=\"top\"> Sources. IdealPump </td>
          <td valign=\"top\"> Simple ideal pump (resp. fan)  dependent on the shaft?s speed;
                   pressure increase versus volume flow is defined as a linear
                   function. Torque * Speed = Pressure increase * Volume flow
                   (without losses).</td> </tr>
<tr><td valign=\"top\"> Examples.PumpAndValve </td>
          <td valign=\"top\"> Test example for valves.</td> </tr>
<tr><td valign=\"top\"> Examples.PumpDropOut </td>
          <td valign=\"top\"> Drop out of 1 pump to test behavior of semiLinear.</td> </tr>
<tr><td valign=\"top\"> Examples.ParallelPumpDropOut </td>
          <td valign=\"top\"> Drop out of 2 parallel pumps to test behavior of semiLinear.</td> </tr>
<tr><td valign=\"top\"> Examples.OneMass </td>
          <td valign=\"top\"> Cooling of 1 hot mass to test behavior of semiLinear.</td> </tr>
<tr><td valign=\"top\"> Examples.TwoMass </td>
          <td valign=\"top\"> Cooling of 2 hot masses to test behavior of semiLinear.</td> </tr>
</table>

<p>
The following <b>components</b> have been improved:
</p>

<table border=\"1\" cellspacing=0 cellpadding=2>
<tr><td colspan=\"2\"><b>Modelica.</b></td></tr>
<tr><td valign=\"top\"> UsersGuide</td>
          <td valign=\"top\"> User's Guide and package description of Modelica Standard Library improved.</td> </tr>
<tr><td colspan=\"2\"><b>Modelica.Blocks.Interfaces.</b></td></tr>
<tr><td valign=\"top\"> RealInput<br>
                   BooleanInput<br>
                   IntegerInput</td>
          <td valign=\"top\"> When dragging one of these connectors the width and height
                   is a factor of 2 larger as a standard icon. Previously,
                   these connectors have been dragged and then manually enlarged
                   by a factor of 2 in the Modelica standard library.</td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Blocks.</b></td></tr>
<tr><td valign=\"top\"> Continuous.*</td>
          <td valign=\"top\"> Initialization options added to all blocks
                   (NoInit, SteadyState, InitialState, InitialOutput).
                   New parameter limitsAtInit to switch off the limits
                   of LimIntegrator or LimPID during initialization</td> </tr>
<tr><td valign=\"top\"> Continuous.LimPID</td>
          <td valign=\"top\"> Option to select P, PI, PD, PID controller.
                   Documentation significantly improved.</td> </tr>
<tr><td valign=\"top\"> Nonlinear.Limiter<br>
                   Nonlinear.VariableLimiter<br>
                   Nonlinear.Deadzone</td>
          <td valign=\"top\"> New parameter limitsAtInit/deadZoneAtInit to switch off the limits
                   or the dead zone during initialization</td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Electrical.Analog. </b></td></tr>
<tr><td valign=\"top\"> Sources</td>
          <td valign=\"top\"> Icon improved (+/- added to voltage sources, arrow added to
                   current sources).</td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Electrical.Analog.Semiconductors. </b></td></tr>
<tr><td valign=\"top\"> Diode</td>
          <td valign=\"top\"> smooth() operator included to improve numerics.</td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Electrical.Machines.BasicMachines.SynchronousInductionMachines. </b></td></tr>
<tr><td valign=\"top\"> SM_PermanentMagnetDamperCage<br>
                   SM_ElectricalExcitedDamperCage<br>
                   SM_ReluctanceRotorDamperCage</td>
          <td valign=\"top\"> The user can choose \"DamperCage = false\" (default: true)
                   to remove all equations for the damper cage from the model.</td> </tr>
<tr><td colspan=\"2\"><b>Modelica.Electrical.Machines.BasicMachines.AsynchronousInductionMachines. </b></td></tr>
<tr><td valign=\"top\"> AIM_SlipRing</td>
          <td valign=\"top\"> Easier parameterization: if the user selects \"useTurnsRatio = false\"
                   (default: true, this is the same behavior as before),
                        parameter TurnsRatio is calculated internally from
                        Nominal stator voltage and Locked-rotor voltage.</td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Math.Matrices.</b></td></tr>
<tr><td valign=\"top\">leastSquares</td>
          <td valign=\"top\">The A matrix in the least squares problem might be rank deficient.
                  Previously, it was required that A has full rank.</td>

<tr><td colspan=\"2\"><b>Modelica.Mechanics.MultiBody.</b></td></tr>
<tr><td valign=\"top\">all models</td>
          <td valign=\"top\"> <ul>
                   <li> All components with animation information have a new variable
                                <b>specularCoefficient</b> to define the reflection of ambient light.
                                The default value is world.defaultSpecularCoefficient which has
                                a default of 0.7. By changing world.defaultSpecularCoefficient, the
                                specularCoefficient of all components is changed that are not
                                explicitly set differently. Since specularCoefficient is a variable
                                (and no parameter), it can be changed during simulation. Since
                                annotation(Dialog) is set, this variable still appears in the
                                parameter menus.<br>
                                Previously, a constant specularCoefficient of 0.7 was used
                                for all components.</li>
                   <li> Variable <b>color</b> of all components is no longer a parameter
                                but an input variable. Also all parameters in package <b>Visualizers</b>,
                                with the exception of <b>shapeType</b> are no longer parameters but
                                defined as input variables with annotation(Dialog). As a result,
                                all these variables appear still in parameter menus, but they can
                                be changed during simulation (e.g., color might be used to
                                display the temperature of a part).</li>
                   <li> All menus have been changed to follow the Modelica 2.2 annotations
                                \"Dialog, group, tab, enable\" (previously, a non-standard Dymola
                                definition for menus was used). Also, the \"enable\" annotation
                                is used in all menus
                                to disable input fields if the input would be ignored.</li>
                   <li> All visual shapes are now defined with conditional declarations
                                (to remove them, if animation is switched off). Previously,
                                these (protected) objects have been defined by arrays with
                                dimension 0 or 1.</li>
                   </ul></td></tr>

<tr><td valign=\"top\">Frames.resolveRelative</td>
          <td valign=\"top\"> The derivative of this function added as function and defined via
                   an annotation. In certain situations, tools had previously
                   difficulties to differentiate the inlined function automatically.</td>

<tr><td valign=\"top\">Forces.*</td>
          <td valign=\"top\"> The scaling factors N_to_m and Nm_to_m have no longer a default
                   value of 1000 but a default value of world.defaultN_to_m (=1000)
                   and world.defaultNm_to_m (=1000). This allows to change the
                   scaling factors for all forces and torques in the world
                   object.</td>
<tr><td valign=\"top\">Interfaces.Frame.a<br>
                  Interfaces.Frame.b<br>
                  Interfaces.Frame_resolve</td>
          <td valign=\"top\"> The Frame connectors are now centered around the origin to ease
                   the usage. The shape was changed, such that the icon is a factor
                   of 1.6 larger as a standard icon (previously, the icon had a
                   standard size when dragged and then the icon was manually enlarged
                   by a factor of 1.5 in the y-direction in the MultiBody library;
                   the height of 16 allows easy positioning on the standard grid size of 2).
                   The double line width of the border in icon and diagram layer was changed
                   to a single line width and when making a connection the connection
                   line is dark grey and no longer black which looks better.</td>
<tr><td valign=\"top\">Joints.Assemblies.*</td>
          <td valign=\"top\"> When dragging an assembly joint, the icon is a factor of 2
                   larger as a standard icon. Icon texts and connectors have a
                   standard size in this enlarged icon (and are not a factor of 2
                   larger as previously). </td>
<tr><td valign=\"top\">Types.*</td>
          <td valign=\"top\"> All types have a corresponding icon now to visualize the content
                   in the package browser (previously, the types did not have an icon).</td>

<tr><td colspan=\"2\"><b>Modelica.Mechanics.Rotational.</b></td></tr>
<tr><td valign=\"top\"> Inertia</td>
          <td valign=\"top\"> Initialization and state selection added.</td> </tr>
<tr><td valign=\"top\"> SpringDamper</td>
          <td valign=\"top\"> Initialization and state selection added.</td> </tr>
<tr><td valign=\"top\"> Move</td>
          <td valign=\"top\"> New implementation based solely on Modelica 2.2 language
                   (previously, the Dymola specific constrain(..) function was used).</td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Mechanics.Translational.</b></td></tr>
<tr><td valign=\"top\"> Move</td>
          <td valign=\"top\"> New implementation based solely on Modelica 2.2 language
                   (previously, the Dymola specific constrain(..) function was used).</td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Thermal.FluidHeatFlow.Interfaces.Partials.</b></td></tr>
<tr><td valign=\"top\"> SimpleFriction</td>
          <td valign=\"top\"> Calculates friction losses from pressure drop and volume flow.</td> </tr>
<tr><td colspan=\"2\"><b>Modelica.Thermal.FluidHeatFlow.Components.</b></td></tr>
<tr><td valign=\"top\"> IsolatedPipe <br>
                   HeatedPipe</td>
          <td valign=\"top\"> Added geodetic height as a source of pressure change;
                   feeds friction losses as calculated by simple friction to
                   the energy balance of the medium.</td> </tr>
<tr><td colspan=\"2\"><b>Modelica.Media.Interfaces.PartialMedium.FluidConstants.</b></td></tr>
<tr><td valign=\"top\">HCRIT0</td><td valign=\"top\">Critical specific enthalpy of the fundamental
                  equation (base formulation of the fluid medium model). </td></tr>
<tr><td valign=\"top\">SCRIT0</td><td valign=\"top\">Critical specific entropy of the fundamental
                  equation (base formulation of the fluid medium model). </td></tr>
<tr><td valign=\"top\">deltah</td><td valign=\"top\">Enthalpy offset (default: 0) between the
                  specific enthalpy of the fluid model and the user-visible
                  specific enthalpy in the model: deltah = h_model - h_fundamentalEquation.
</td></tr>
<tr><td valign=\"top\">deltas</td><td valign=\"top\">Entropy offset (default: 0) between the
                  specific entropy of the fluid model and the user-visible
                  specific entropy in the model: deltas = s_model - s_fundamentalEquation.</td></tr>
<tr><td valign=\"top\">T_default</td><td valign=\"top\">Default value for temperature of medium (for initialization)</td></tr>
<tr><td valign=\"top\">h_default</td><td valign=\"top\">Default value for specific enthalpy of medium (for initialization)</td></tr>
<tr><td valign=\"top\">p_default</td><td valign=\"top\">Default value for pressure of medium (for initialization)</td></tr>
<tr><td valign=\"top\">X_default</td><td valign=\"top\">Default value for mass fractions of medium (for initialization)</td></tr>
</table>
<p>
The following <b>errors</b> have been fixed:
</p>

<table border=\"1\" cellspacing=0 cellpadding=2>
<tr><td colspan=\"2\"><b>Modelica.Blocks.Tables.</b></td></tr>
<tr><td valign=\"top\">CombiTable1D<br>
                  CombiTable1Ds<br>
                  CombiTable2D</td>
          <td valign=\"top\"> Parameter \"tableOnFile\" determines now whether a table is read from
                   file or used from parameter \"table\". Previously, if \"fileName\" was not
                   \"NoName\", a table was always read from file \"fileName\", independently
                   of the setting of \"tableOnFile\". This has been corrected.<br>
                   Furthermore, the initialization of a table is now performed in a
                   when-clause and no longer in a parameter declaration, because some
                   tools evaluate the parameter declaration in some situation more than
                   once and then the table is unnecessarily read several times
                   (and occupies also more memory).</td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Blocks.Sources.</b></td></tr>
<tr><td valign=\"top\">CombiTimeTable</td>
          <td valign=\"top\"> Same bug fix/improvement as for the tables from Modelica.Blocks.Tables
                   as outlined above.</td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Electrical.Analog.Semiconductors. </b></td></tr>
<tr><td valign=\"top\"> PMOS<br>
                   NMOS<br>
                   HeatingPMOS<br>
                   HeatingNMOS</td>
          <td valign=\"top\"> The Drain-Source-Resistance RDS had actually a resistance of
                   RDS/v, with v=Beta*(W+dW)/(L+dL). The correct formula is without
                   the division by \"v\". This has now been corrected.<br>
                   This bug fix should not have an essential effect in most applications.
                   In the default case (Beta=1e-5), the Drain-Source-Resistance was
                   a factor of 1e5 too large and had in the default case the
                   wrong value 1e12, although it should have the value 1e7. The effect
                   was that this resistance had practically no effect.</td> </tr>

<tr><td colspan=\"2\"> <b>Modelica.Media.IdealGases.Common.SingleGasNasa.</b></td></tr>
<tr><td valign=\"top\"> dynamicViscosityLowPressure</td>
          <td valign=\"top\"> Viscosity and thermal conductivity (which needs viscosity as input)
                   were computed wrong for polar gases and gas mixtures
                   (i.e., if dipole moment not 0.0). This has been fixed in version 2.2.1.</td> </tr>
<tr><td colspan=\"2\"><b>Modelica.Utilities.Streams.</b></td></tr>
<tr><td valign=\"top\">readLine</td>
          <td valign=\"top\"> Depending on the C-implementation, the stream was not correctly closed.
                   This has been corrected by adding a \"Streams.close(..)\"
                   after reading the file content.</td> </tr>

</table>
</html>"));
      end Version_2_2_1;

      class Version_2_2 "Version 2.2 (April 6, 2005)"
        extends Modelica.Icons.ReleaseNotes;
        annotation(
          Documentation(info = "<html>

<p>
Version 2.2 is backward compatible to version 2.1.
</p>

<p>
The following <b>new libraries</b> have been added:
</p>

<table border=\"1\" cellspacing=0 cellpadding=2>
<tr><td valign=\"top\"><a href=\"modelica://Modelica.Media\">Modelica.Media</a></td>
          <td valign=\"top\"> Property models of liquids and gases, especially
                   <ul>
                   <li>1241 detailed gas models,</li>
                   <li> moist air,</li>
                   <li> high precision water model (according to IAPWS/IF97 standard), </li>
                   <li> incompressible media defined by tables (cp(T), rho(t), eta(T), etc. are defined by tables).</li>
                   </ul>
                   The user can conveniently define mixtures of gases between the
                   1241 gas models. The models are
                   designed to work well in dynamic simulations. They
                   are based on a new standard interface for media with
                   single and multiple substances and one or multiple phases
                   with the following features:
                   <ul>
                   <li> The independent variables of a medium model do not influence the
                                definition of a fluid connector port or how the
                                balance equations have to be implemented.<br>
                                Used independent variables: \"p,T\", \"p,T,X\", \"p,h\", \"d,T\".</li>
                   <li> Optional variables, e.g., dynamic viscosity, are only computed
                                if needed.</li>
                   <li> The medium models are implemented with regards to efficient
                                dynamic simulation.</li>
                   </ul>
          </td></tr>
<tr><td valign=\"top\"><a href=\"modelica://Modelica.Thermal.FluidHeatFlow\">Modelica.Thermal.FluidHeatFlow</a></td>
          <td valign=\"top\"> Simple components for 1-dim., incompressible thermo-fluid flow
                   to model coolant flows, e.g., of electrical machines.
                   Components can be connected arbitrarily together (= ideal mixing
                   at connection points) and fluid may reverse direction of flow.
</td></tr>
</table>
<p>
The following <b>changes</b> have been performed in the
<b>Modelica.Mechanics.MultiBody</b> library:
</p>
<ul>
<li> Component MultiBody.World has a new parameter
         <b>driveTrainMechanics3D</b>. If set to <b>true</b>, 3D mechanical effects
         of MultiBody.Parts.Mounting1D/Rotor1D/BevelGear1D are taken into account. If set to
         <b>false</b> (= default), 3D mechanical effects in these elements
         are not taken into account and the
         frame connectors to connect to 3D parts are disabled (all
         connections to such a disabled connector are also disabled, due to the
         new feature of conditional declarations in Modelica language 2.2)</li>
<li> All references to \"MultiBody.xxx\" have
         been changed to \"Modelica.Mechanics.MultiBodys.xxx\" in order that after
         copying of a component outside of the Modelica library, the references
         still remain valid.</li>
</ul>
</html>"));
      end Version_2_2;

      class Version_2_1 "Version 2.1 (Nov. 11, 2004)"
        extends Modelica.Icons.ReleaseNotes;
        annotation(
          Documentation(info = "<html>

<p> This is a major change with respect to previous versions of the
        Modelica Standard Library, because <b>many new libraries</b> and components
        are included and because the input/output blocks (Modelica.Blocks)
        have been considerably simplified:
</p>
<ul>
<li> An input/output connector is defined <b>without</b> a hierarchy (this is possible
         due to new features of the Modelica language). For example, the input
         signal of a block \"FirstOrder\" was previously accessed as \"FirstOrder.inPort.signal[1]\".
         Now it is accessed as \"FirstOrder.u\". This simplifies the understanding and usage
         especially for beginners.</li>
<li> De-vectorized the <b>Modelica.Blocks</b> library. All blocks in the
         Modelica.Blocks library are now scalar blocks. As a result,
         the parameters of the Blocks are scalars and no vectors any
         more. For example, a parameter \"amplitude\" that might had
         a value of \"{1}\" previously, has now a value of \"1\". This simplifies
         the understanding and usage especially for beginners.<br>
         If a vector of blocks is needed, this can be easily
         accomplished by adding a dimension to the instance. For example
         \"Constant const[3](k={1,2,3}\" defines three Constant blocks.
         An additional advantage of the new approach is that
         the implementation of Modelica.Blocks is much simpler and is easier to
         understand.
</li>
</ul>

<p>
The discussed changes of Modelica.Blocks are not backward compatible.
A script to <b>automatically</b> convert models to this new version is
provided. There might be rare cases, where this script does not convert.
In this case models have to be manually converted.
In any case you should make a back-up copy of your model
before automatic conversion is performed.
</p>
<p>
The following <b>new libraries</b> have been added:
</p>
<table border=\"1\" cellspacing=0 cellpadding=2>
<tr><td valign=\"top\"><a href=\"modelica://Modelica.Electrical.Digital\">Modelica.Electrical.Digital</a></td>
          <td valign=\"top\">Digital electrical components based on 2-,3-,4-, and 9-valued logic<br>
                  according to the VHDL standard</td></tr>
<tr><td valign=\"top\"><a href=\"modelica://Modelica.Electrical.Machines\">Modelica.Electrical.Machines</a></td>
          <td valign=\"top\">Asynchronous, synchronous and DC motor and generator models</td></tr>
<tr><td valign=\"top\"><a href=\"modelica://Modelica.Math.Matrices\">Modelica.Math.Matrices</a></td>
          <td valign=\"top\">Functions operating on matrices such as solve() (A*x=b), leastSquares(),<br>
                  norm(), LU(), QR(),  eigenValues(), singularValues(), exp(), ...</td></tr>
<tr><td valign=\"top\"><a href=\"modelica://Modelica.StateGraph\">Modelica.StateGraph</a></td>
          <td valign=\"top\"> Modeling of <b>discrete event</b> and <b>reactive</b> systems in a convenient way using<br>
                   <b>hierarchical state machines</b> and <b>Modelica</b> as <b>action language</b>. <br>
                   It is based on JGrafchart and Grafcet and  has a similar modeling power as <br>
                   StateCharts. It avoids deficiencies of usually used action languages. <br>
                   This library makes the ModelicaAdditions.PetriNets library obsolet.</td></tr>
<tr><td valign=\"top\"><a href=\"modelica://Modelica.Utilities.Files\">Modelica.Utilities.Files</a></td>
          <td valign=\"top\">Functions to operate on files and directories (copy, move, remove files etc.)</td></tr>
<tr><td valign=\"top\"><a href=\"modelica://Modelica.Utilities.Streams\">Modelica.Utilities.Streams</a></td>
          <td valign=\"top\">Read from files and write to files (print, readLine, readFile, error, ...)</td></tr>
<tr><td valign=\"top\"><a href=\"modelica://Modelica.Utilities.Strings\">Modelica.Utilities.Strings</a></td>
          <td valign=\"top\">Operations on strings (substring, find, replace, sort, scanToken, ...)</td></tr>
<tr><td valign=\"top\"><a href=\"modelica://Modelica.Utilities.System\">Modelica.Utilities.System</a></td>
          <td valign=\"top\">Get/set current directory, get/set environment variable, execute shell command, etc.</td></tr>
</table>
<p>
The following existing libraries outside of the Modelica standard library
have been improved and added as <b>new libraries</b>
(models using the previous libraries are automatically converted
to the new sublibraries inside package Modelica):
</p>
<table border=\"1\" cellspacing=0 cellpadding=2>
<tr><td valign=\"top\"><a href=\"modelica://Modelica.Blocks.Discrete\">Modelica.Blocks.Discrete</a></td>
          <td valign=\"top\"> Discrete input/output blocks with fixed sample period<br>
                   (from ModelicaAdditions.Blocks.Discrete)</td></tr>
<tr><td valign=\"top\"><a href=\"modelica://Modelica.Blocks.Logical\">Modelica.Blocks.Logical</a></td>
          <td valign=\"top\"> Logical components with Boolean input and output signals<br>
                   (from ModelicaAdditions.Blocks.Logical)</td></tr>
<tr><td valign=\"top\"><a href=\"modelica://Modelica.Blocks.Nonlinear\">Modelica.Blocks.Nonlinear</a></td>
          <td valign=\"top\"> Discontinuous or non-differentiable algebraic control blocks such as variable limiter,<br>
                   fixed, variable and Pade delay etc. (from ModelicaAdditions.Blocks.Nonlinear)</td></tr>
<tr><td valign=\"top\"><a href=\"modelica://Modelica.Blocks.Routing\">Modelica.Blocks.Routing</a></td>
          <td valign=\"top\"> Blocks to combine and extract signals, such as multiplexer<br>
                   (from ModelicaAdditions.Blocks.Multiplexer)</td></tr>
<tr><td valign=\"top\"><a href=\"modelica://Modelica.Blocks.Tables\">Modelica.Blocks.Tables</a></td>
          <td valign=\"top\"> One and two-dimensional interpolation in tables. CombiTimeTable is available<br>
                   in Modelica.Blocks.Sources (from ModelicaAdditions.Tables)</td></tr>
<tr><td valign=\"top\"><a href=\"modelica://Modelica.Mechanics.MultiBody\">Modelica.Mechanics.MultiBody</a></td>
          <td valign=\"top\"> Components to model the movement of 3-dimensional mechanical systems. Contains <br>
                   body, joint, force and sensor components, analytic handling of kinematic loops,<br>
                   force elements with mass, series/parallel connection of 3D force elements etc.<br>
                   (from MultiBody 1.0 where the new signal connectors are used;<br>
                   makes the ModelicaAdditions.MultiBody library obsolete)</td></tr>
</table>
<p>
As a result, the ModelicaAdditions library is obsolete, because all components
are either included in the Modelica library or are replaced by much more
powerful libraries (MultiBody, StateGraph).
</p>
<p>
The following <b>new components</b> have been added to <b>existing</b> libraries.
</p>
<table border=\"1\" cellspacing=0 cellpadding=2>
<tr><td colspan=\"2\"><b>Modelica.Blocks.Logical.</b></td></tr>
<tr><td valign=\"top\">Pre</td>
          <td valign=\"top\">y = pre(u): Breaks algebraic loops by an infinitesimal small <br>
                  time delay (event iteration continues until u = pre(u))</td></tr>
<tr><td valign=\"top\">Edge</td>
          <td valign=\"top\">y = edge(u): Output y is true, if the input u has a rising edge </td></tr>
<tr><td valign=\"top\">FallingEdge</td>
          <td valign=\"top\">y = edge(not u): Output y is true, if the input u has a falling edge </td></tr>
<tr><td valign=\"top\">Change</td>
          <td valign=\"top\">y = change(u): Output y is true, if the input u has a rising or falling edge </td></tr>
<tr><td valign=\"top\">GreaterEqual</td>
          <td valign=\"top\">Output y is true, if input u1 is greater or equal than input u2 </td></tr>
<tr><td valign=\"top\">Less</td>
          <td valign=\"top\">Output y is true, if input u1 is less than input u2 </td></tr>
<tr><td valign=\"top\">LessEqual</td>
          <td valign=\"top\">Output y is true, if input u1 is less or equal than input u2 </td></tr>
<tr><td valign=\"top\">Timer</td>
          <td valign=\"top\">Timer measuring the time from the time instant where the <br>
                  Boolean input became true </td></tr>
<tr><td colspan=\"2\"><b>Modelica.Blocks.Math.</b></td></tr>
<tr><td valign=\"top\">BooleanToReal</td>
          <td valign=\"top\">Convert Boolean to Real signal</td></tr>
<tr><td valign=\"top\">BooleanToInteger</td>
          <td valign=\"top\">Convert Boolean to Integer signal</td></tr>
<tr><td valign=\"top\">RealToBoolean</td>
          <td valign=\"top\">Convert Real to Boolean signal</td></tr>
<tr><td valign=\"top\">IntegerToBoolean</td>
          <td valign=\"top\">Convert Integer to Boolean signal</td></tr>
<tr><td colspan=\"2\"><b>Modelica.Blocks.Sources.</b></td></tr>
<tr><td valign=\"top\">RealExpression</td>
          <td valign=\"top\">Set output signal to a time varying Real expression</td></tr>
<tr><td valign=\"top\">IntegerExpression</td>
          <td valign=\"top\">Set output signal to a time varying Integer expression</td></tr>
<tr><td valign=\"top\">BooleanExpression</td>
          <td valign=\"top\">Set output signal to a time varying Boolean expression</td></tr>
<tr><td valign=\"top\">BooleanTable</td>
          <td valign=\"top\">Generate a Boolean output signal based on a vector of time instants</td></tr>
<tr><td colspan=\"2\"><b>Modelica.Mechanics.MultiBody.</b></td></tr>
<tr><td valign=\"top\">Frames.from_T2</td>
          <td valign=\"top\">Return orientation object R from transformation matrix T and its derivative der(T)</td></tr>
<tr><td colspan=\"2\"><b>Modelica.Mechanics.Rotational.</b></td></tr>
<tr><td valign=\"top\">LinearSpeedDependentTorque</td>
          <td valign=\"top\">Linear dependency of torque versus speed (acts as load torque)</td></tr>
<tr><td valign=\"top\">QuadraticSpeedDependentTorque</td>
          <td valign=\"top\">Quadratic dependency of torque versus speed (acts as load torque)</td></tr>
<tr><td valign=\"top\">ConstantTorque</td>
          <td valign=\"top\">Constant torque, not dependent on speed (acts as load torque)</td></tr>
<tr><td valign=\"top\">ConstantSpeed</td>
          <td valign=\"top\">Constant speed, not dependent on torque (acts as load torque)</td></tr>
<tr><td valign=\"top\">TorqueStep</td>
          <td valign=\"top\">Constant torque, not dependent on speed (acts as load torque)</td></tr>
</table>
<p>
The following <b>bugs</b> have been <b>corrected</b>:
</p>
<table border=\"1\" cellspacing=0 cellpadding=2>
<tr><td colspan=\"2\"><b>Modelica.Mechanics.MultiBody.Forces.</b></td></tr>
<tr><td valign=\"top\">LineForceWithMass<br>
                  Spring</td>
          <td valign=\"top\">If mass of the line force or spring component is not zero, the<br>
                  mass was (implicitly) treated as \"mass*mass\" instead of as \"mass\"</td></tr>
<tr><td colspan=\"2\"><b>Modelica.Mechanics.Rotational.</b></td></tr>
<tr><td valign=\"top\">Speed</td>
          <td valign=\"top\">If parameter exact=<b>false</b>, the filter was wrong<br>
                  (position was filtered and not the speed input signal)</td></tr>
</table>
<p>
Other changes:
</p>
<ul>
<li> All connectors are now smaller in the diagram layer. This gives
         a nicer layout when connectors and components are used together
         in a diagram</li>
<li> Default instance names are defined for all connectors, according
         to a new annotation introduced in Modelica 2.1. For example,
         when dragging connector \"Flange_a\" from the Rotational library to
         the diagram layer, the default connector instance name is
         \"flange_a\" and not \"Flange_a1\".</li>
<li> The Modelica.Mechanics.Rotational connectors are changed from
         a square to a circle</li>
<li> The Modelica.Mechanics.Translational connectors are changed from a
         green to a dark green color in order that connection lines
         can be better seen, especially when printed.</li>
<li> The Modelica.Blocks connectors for Real signals are changed from
         blue to dark blue in order to distinguish them from electrical signals.</li>
</ul>
</html>"));
      end Version_2_1;

      class Version_1_6 "Version 1.6 (June 21, 2004)"
        extends Modelica.Icons.ReleaseNotes;
        annotation(
          Documentation(info = "<html>

<p> Added 1 new library (Electrical.MultiPhase), 17 new components,
        improved 3 existing components
        in the Modelica.Electrical library and improved 3 types
        in the Modelica.SIunits library. Furthermore,
        this User's Guide has been started. The improvements
        in more detail:
</p>
<p>
<b>New components</b>
</p>
<table border=\"1\" cellspacing=0 cellpadding=2>
<tr><td colspan=\"2\"><b>Modelica.Electrical.Analog.Basic.</b></td></tr>
<tr><td valign=\"top\">SaturatingInductor</td>
          <td valign=\"top\">Simple model of an inductor with saturation</td></tr>
<tr><td valign=\"top\">VariableResistor</td>
          <td valign=\"top\">Ideal linear electrical resistor with variable resistance</td></tr>
<tr><td valign=\"top\">VariableConductor</td>
          <td valign=\"top\">Ideal linear electrical conductor with variable conductance</td></tr>
<tr><td valign=\"top\">VariableCapacitor</td>
          <td valign=\"top\">Ideal linear electrical capacitor with variable capacitance</td></tr>
<tr><td valign=\"top\">VariableInductor</td>
          <td valign=\"top\">Ideal linear electrical inductor with variable inductance</td></tr>
<tr><td colspan=\"2\"><b>Modelica.Electrical.Analog.Semiconductors.</b></td></tr>
<tr><td valign=\"top\">HeatingDiode</td>
          <td valign=\"top\">Simple diode with heating port</td></tr>
<tr><td valign=\"top\">HeatingNMOS</td>
          <td valign=\"top\">Simple MOS Transistor with heating port</td></tr>
<tr><td valign=\"top\">HeatingPMOS</td>
          <td valign=\"top\">Simple PMOS Transistor with heating port</td></tr>
<tr><td valign=\"top\">HeatingNPN</td>
          <td valign=\"top\">Simple NPN BJT according to Ebers-Moll with heating port</td></tr>
<tr><td valign=\"top\">HeatingPNP</td>
          <td valign=\"top\">Simple PNP BJT according to Ebers-Moll with heating port</td></tr>
<tr><td colspan=\"2\"><b>Modelica.Electrical.MultiPhase</b><br>
          A new library for multi-phase electrical circuits</td></tr>
</table>
<p>
<b>New examples</b>
</p>
<p>
The following new examples have been added to
Modelica.Electrical.Analog.Examples:
</p>
<p>
CharacteristicThyristors,
CharacteristicIdealDiodes,
HeatingNPN_OrGate,
HeatingMOSInverter,
HeatingRectifier,
Rectifier,
ShowSaturatingInductor
ShowVariableResistor
</p>
<p>
<b>Improved existing components</b>
</p>
<p>In the library Modelica.Electrical.Analog.Ideal,
a knee voltage has been introduced for the components
IdealThyristor, IdealGTOThyristor, IdealDiode in order
that the approximation of these ideal elements is improved
with not much computational effort.</p>
<p> In the Modelica.SIunits library, the following changes
        have been made:</p>
<table border=\"1\" cellspacing=0 cellpadding=2>
<tr><td valign=\"top\">Inductance</td>
          <td valign=\"top\">min=0 removed</td></tr>
<tr><td valign=\"top\">SelfInductance</td>
          <td valign=\"top\">min=0 added</td></tr>
<tr><td valign=\"top\">ThermodynamicTemperature</td>
          <td valign=\"top\">min=0 added</td></tr>
</table>
</html>"));
      end Version_1_6;

      class Version_1_5 "Version 1.5 (Dec. 16, 2002)"
        extends Modelica.Icons.ReleaseNotes;
        annotation(
          Documentation(info = "<html>

<p> Added 55 new components. In particular, added new package
        <b>Thermal.HeatTransfer</b> for modeling of lumped
        heat transfer, added model <b>LossyGear</b> in Mechanics.Rotational
        to model gear efficiency and bearing friction according to a new
        theory in a robust way, added 10 new models in Electrical.Analog and
        added several other new models and improved existing models.
</p>
<p>
<b>New components</b>
</p>
<table border=\"1\" cellspacing=0 cellpadding=2>
<tr><td colspan=\"2\"><b>Modelica.Blocks.</b></td></tr>
<tr><td valign=\"top\">Continuous.Der</td><td valign=\"top\">Derivative of input (= analytic differentiations)</td></tr>
<tr><td valign=\"top\"><b><i>Examples</i></b></td><td valign=\"top\">Demonstration examples of the components of this package</td></tr>
<tr><td valign=\"top\">Nonlinear.VariableLimiter</td><td valign=\"top\">Limit the range of a signal with variable limits</td></tr>
<tr><td colspan=\"2\"><b>Modelica.Blocks.Interfaces.</b></td></tr>
<tr><td valign=\"top\">RealPort</td><td valign=\"top\">Real port (both input/output possible)</td></tr>
<tr><td valign=\"top\">IntegerPort</td><td valign=\"top\">Integer port (both input/output possible)</td></tr>
<tr><td valign=\"top\">BooleanPort</td><td valign=\"top\">Boolean port (both input/output possible)</td></tr>
<tr><td valign=\"top\">SIMO</td><td valign=\"top\">Single Input Multiple Output continuous control block</td></tr>
<tr><td valign=\"top\">IntegerBlockIcon</td><td valign=\"top\">Basic graphical layout of Integer block</td></tr>
<tr><td valign=\"top\">IntegerMO</td><td valign=\"top\">Multiple Integer Output continuous control block</td></tr>
<tr><td valign=\"top\">IntegerSignalSource</td><td valign=\"top\">Base class for continuous Integer signal source</td></tr>
<tr><td valign=\"top\">IntegerMIBooleanMOs</td><td valign=\"top\">Multiple Integer Input Multiple Boolean Output continuous control block with same number of inputs and outputs</td></tr>
<tr><td valign=\"top\">BooleanMIMOs</td><td valign=\"top\">Multiple Input Multiple Output continuous control block with same number of inputs and outputs of Boolean type</td></tr>
<tr><td valign=\"top\"><b><i>BusAdaptors</i></b></td><td valign=\"top\">Components to send signals to the bus or receive signals from the bus</td></tr>
<tr><td colspan=\"2\"><b>Modelica.Blocks.Math.</b></td></tr>
<tr><td valign=\"top\">RealToInteger</td><td valign=\"top\">Convert real to integer signals</td></tr>
<tr><td valign=\"top\">IntegerToReal</td><td valign=\"top\">Convert integer to real signals</td></tr>
<tr><td valign=\"top\">Max</td><td valign=\"top\">Pass through the largest signal</td></tr>
<tr><td valign=\"top\">Min</td><td valign=\"top\">Pass through the smallest signal</td></tr>
<tr><td valign=\"top\">Edge</td><td valign=\"top\">Indicates rising edge of Boolean signal</td></tr>
<tr><td valign=\"top\">BooleanChange</td><td valign=\"top\">Indicates Boolean signal changing</td></tr>
<tr><td valign=\"top\">IntegerChange</td><td valign=\"top\">Indicates integer signal changing</td></tr>
<tr><td colspan=\"2\"><b>Modelica.Blocks.Sources.</b></td></tr>
<tr><td valign=\"top\">IntegerConstant</td><td valign=\"top\">Generate constant signals of type Integer</td></tr>
<tr><td valign=\"top\">IntegerStep</td><td valign=\"top\">Generate step signals of type Integer</td></tr>
<tr><td colspan=\"2\"><b>Modelica.Electrical.Analog.Basic.</b></td></tr>
<tr><td valign=\"top\">HeatingResistor</td><td valign=\"top\">Temperature dependent electrical resistor</td></tr>
<tr><td valign=\"top\">OpAmp</td><td valign=\"top\">Simple nonideal model of an OpAmp with limitation</td></tr>
<tr><td colspan=\"2\"><b>Modelica.Electrical.Analog.Ideal.</b></td></tr>
<tr><td valign=\"top\">IdealCommutingSwitch</td><td valign=\"top\">Ideal commuting switch</td></tr>
<tr><td valign=\"top\">IdealIntermediateSwitch</td><td valign=\"top\">Ideal intermediate switch</td></tr>
<tr><td valign=\"top\">ControlledIdealCommutingSwitch</td><td valign=\"top\">Controlled ideal commuting switch</td></tr>
<tr><td valign=\"top\">ControlledIdealIntermediateSwitch</td><td valign=\"top\">Controlled ideal intermediate switch</td></tr>
<tr><td valign=\"top\">IdealOpAmpLimited</td><td valign=\"top\">Ideal operational amplifier with limitation</td></tr>
<tr><td valign=\"top\">IdealOpeningSwitch</td><td valign=\"top\">Ideal opener</td></tr>
<tr><td valign=\"top\">IdealClosingSwitch</td><td valign=\"top\">Ideal closer</td></tr>
<tr><td valign=\"top\">ControlledIdealOpeningSwitch</td><td valign=\"top\">Controlled ideal opener</td></tr>
<tr><td valign=\"top\">ControlledIdealClosingSwitch</td><td valign=\"top\">Controlled ideal closer</td></tr>

<tr><td colspan=\"2\"><b>Modelica.Electrical.Analog.Lines.</b></td></tr>
<tr><td valign=\"top\">TLine1</td><td valign=\"top\">Lossless transmission line (Z0, TD)</td></tr>
<tr><td valign=\"top\">TLine2</td><td valign=\"top\">Lossless transmission line (Z0, F, NL)</td></tr>
<tr><td valign=\"top\">TLine2</td><td valign=\"top\">Lossless transmission line (Z0, F)</td></tr>
<tr><td colspan=\"2\"><b>Modelica.Icons.</b></td></tr>
<tr><td valign=\"top\">Function</td><td valign=\"top\">Icon for a function</td></tr>
<tr><td valign=\"top\">Record</td><td valign=\"top\">Icon for a record</td></tr>
<tr><td valign=\"top\">Enumeration</td><td valign=\"top\">Icon for an enumeration</td></tr>
<tr><td colspan=\"2\"><b>Modelica.Math.</b></td></tr>
<tr><td valign=\"top\">tempInterpol2</td><td valign=\"top\">temporary routine for vectorized linear interpolation (will be removed)</td></tr>
<tr><td colspan=\"2\"><b>Modelica.Mechanics.Rotational.</b></td></tr>
<tr><td valign=\"top\">Examples.LossyGearDemo1</td><td valign=\"top\">Example to show that gear efficiency may lead to stuck motion</td></tr>
<tr><td valign=\"top\">Examples.LossyGearDemo2</td><td valign=\"top\">Example to show combination of LossyGear and BearingFriction</td></tr>
<tr><td valign=\"top\">LossyGear</td><td valign=\"top\">Gear with mesh efficiency and bearing friction (stuck/rolling possible)</td></tr>
<tr><td valign=\"top\">Gear2</td><td valign=\"top\">Realistic model of a gearbox (based on LossyGear)</td></tr>
<tr><td colspan=\"2\"><b>Modelica.SIunits.</b></td></tr>
<tr><td valign=\"top\"><b><i>Conversions</i></b></td><td valign=\"top\">Conversion functions to/from non SI units and type definitions of non SI units</td></tr>
<tr><td valign=\"top\">EnergyFlowRate</td><td valign=\"top\">Same definition as <i>Power</i></td></tr>
<tr><td valign=\"top\">EnthalpyFlowRate</td><td valign=\"top\"><code>Real (final quantity=\"EnthalpyFlowRate\", final unit=\"W\")</code></td></tr>
<tr><td colspan=\"2\"><b>Modelica.</b></td></tr>
<tr><td valign=\"top\"><b><i>Thermal.HeatTransfer</i></b></td><td valign=\"top\">1-dimensional heat transfer with lumped elements</td></tr>
<tr><td colspan=\"2\"><b>ModelicaAdditions.Blocks.Discrete.</b></td></tr>
<tr><td valign=\"top\">TriggeredSampler</td><td valign=\"top\">Triggered sampling of continuous signals</td></tr>
<tr><td valign=\"top\">TriggeredMax</td><td valign=\"top\">Compute maximum, absolute value of continuous signal at trigger instants</td></tr>
<tr><td colspan=\"2\"><b>ModelicaAdditions.Blocks.Logical.Interfaces.</b></td></tr>
<tr><td valign=\"top\">BooleanMIRealMOs</td><td valign=\"top\">Multiple Boolean Input Multiple Real Output continuous control block with same number of inputs and outputs</td></tr>
<tr><td valign=\"top\">RealMIBooleanMOs</td><td valign=\"top\">Multiple Real Input Multiple Boolean Output continuous control block with same number of inputs and outputs</td></tr>
<tr><td colspan=\"2\"><b>ModelicaAdditions.Blocks.Logical.</b></td></tr>
<tr><td valign=\"top\">TriggeredTrapezoid</td><td valign=\"top\">Triggered trapezoid generator</td></tr>
<tr><td valign=\"top\">Hysteresis</td><td valign=\"top\">Transform Real to Boolean with Hysteresis</td></tr>
<tr><td valign=\"top\">OnOffController</td><td valign=\"top\">On-off controller</td></tr>
<tr><td valign=\"top\">Compare</td><td valign=\"top\">True, if signal of inPort1 is larger than signal of inPort2</td></tr>
<tr><td valign=\"top\">ZeroCrossing</td><td valign=\"top\">Trigger zero crossing of input signal</td></tr>
<tr><td colspan=\"2\"><b>ModelicaAdditions.</b></td></tr>
<tr><td valign=\"top\">Blocks.Multiplexer.Extractor</td><td valign=\"top\">Extract scalar signal out of signal vector dependent on IntegerRealInput index</td></tr>
<tr><td valign=\"top\">Tables.CombiTable1Ds</td><td valign=\"top\">Table look-up in one dimension (matrix/file) with only single input</td></tr>
</table>
<p>
<b>Package-specific Changes</b>
</p>
<ul>
<li>All example models made <b>encapsulated</b></li>
<li>Upper case constants changed to lower case (cf. Modelica.Constants)</li>
<li>Introduced Modelica.SIunits.Wavelength due to typo \"Wavelenght\"</li>
<li>Introduced ModelicaAdditions.Blocks.Logical.Interfaces.Comparison due to typo \"Comparision\"</li>
<li>Changed these components of *.Blocks to <code>block</code> class, which have not been already of block type</li>
<li>Changed *.Interfaces.RelativeSensor to <code>partial</code> models</li>
</ul>
<p>
<b>Class-specific Changes</b>
</p>
<p>
<i>Modelica.SIunits</i>
</p>
<p>Removed <code>final</code> from quantity attribute for <i>Mass</i> and <i>MassFlowRate</i>.</p>
<p>
<i>Modelica.Blocks.Math.Sum</i>
</p>
<p>Implemented avoiding algorithm section, which would lead to expensive function calls.</p>
<p><i>Modelica.Blocks.Sources.Step</i></p>
<pre>
block Step \"Generate step signals of type Real\"
        parameter Real height[:]={1} \"Heights of steps\";
<b> // parameter Real offset[:]={0} \"Offsets of output signals\";
// parameter SIunits.Time startTime[:]={0} \"Output = offset for time < startTime\";
// extends Interfaces.MO          (final nout=max([size(height, 1); size(offset, 1); size(startTime, 1)]));
        extends Interfaces.SignalSource(final nout=max([size(height, 1); size(offset, 1); size(startTime, 1)]));</b>
</pre>
<p><i>Modelica.Blocks.Sources.Exponentials</i></p>
<p>Replaced usage of built-in function <code>exp</code> by Modelica.Math.exp.</p>
<p><i>Modelica.Blocks.Sources.TimeTable</i></p>
<p>Interface definition changed from</p>
<pre>    parameter Real table[:, :]=[0, 0; 1, 1; 2, 4] \"Table matrix (time = first column)\";
</pre>
<p>to</p>
<pre>    parameter Real table[:, <b>2</b>]=[0, 0; 1, 1; 2, 4] \"Table matrix (time = first column)\";
</pre>
<p>Did the same for subfunction <i>getInterpolationCoefficients</i>.</p>
<p>Bug in <i>getInterpolationCoefficients</i> for startTime <> 0 fixed:</p>
<pre>        ...
                end if;
          end if;
          <b>// Take into account startTime \"a*(time - startTime) + b\"
          b := b - a*startTime;</b>
        end getInterpolationCoefficients;
</pre>
<p><i>Modelica.Blocks.Sources.BooleanStep</i></p>
<pre>
block BooleanStep \"Generate step signals of type Boolean\"
        parameter SIunits.Time startTime[:]={0} \"Time instants of steps\";
        <b>parameter Boolean startValue[size(startTime, 1)]=fill(false, size(startTime, 1)) \"Output before startTime\";</b>
        extends Interfaces.BooleanSignalSource(final nout=size(startTime, 1));
equation
        for i in 1:nout loop
<b>//   outPort.signal[i] = time >= startTime[i];
          outPort.signal[i] = if time >= startTime[i] then not startValue[i] else startValue[i];</b>
        end for;
end BooleanStep;
</pre>
<p>
<i>Modelica.Electrical.Analog</i></p>
<p>Corrected table of values and default for Beta by dividing them by 1000
(consistent with the values used in the NAND-example model):
</p>
<ul>
<li>Semiconductors.PMOS</li>
<li>Semiconductors.NMOS</li>
</ul>
<p>Corrected parameter defaults, unit and description for TrapezoidCurrent.
This makes the parameters consistent with their use in the model.
Models specifying parameter values are not changed.
Models not specifying parameter values did not generate trapezoids previously.
</p>
<p>Icon layer background changed from transparent to white:</p>
<ul>
<li>Basic.Gyrator</li>
<li>Basic.EMF</li>
<li>Ideal.Idle</li>
<li>Ideal.Short</li>
</ul>
<p>Basic.Transformer: Replaced invalid escape characters '\\ ' and '\\[newline]' in documentation by '|'.</p>
<p><i>Modelica.Mechanics.Rotational</i></p>
<p>Removed arrows and names documentation from flanges in diagram layer</p>
<p><i>Modelica.Mechanics.Rotational.Interfaces.FrictionBase</i></p>
<p><i>Modelica.Mechanics.Rotational.Position</i></p>
<p>Replaced <code>reinit</code> by <code>initial equation</code></p>
<p><i>Modelica.Mechanics.Rotational.RelativeStates</i></p>
<p>Bug corrected by using modifier <code>stateSelect = StateSelect.prefer</code> as implementation</p>
<p><i>Modelica.Mechanics.Translational.Interfaces.flange_b</i></p>
<p>Attribute <b>fillColor=7</b> added to Rectangle on Icon layer, i.e., it is now
filled with white and not transparent any more.</p>
<p><i>Modelica.Mechanics.Translational.Position</i></p>
<p>Replaced <code>reinit</code> by <code>initial equation</code></p>
<p><i>Modelica.Mechanics.Translational.RelativeStates</i></p>
<p>Bug corrected by using modifier <code>stateSelect = StateSelect.prefer</code> as implementation</p>
<p><i>Modelica.Mechanics.Translational.Stop</i></p>
<p>Use <code>stateSelect = StateSelect.prefer</code>.</p>
<p><i>Modelica.Mechanics.Translational.Examples.PreLoad</i></p>
<p>Improved documentation and coordinate system used for example.</p>
<p><i>ModelicaAdditions.Blocks.Nonlinear.PadeDelay</i></p>
<p>Replaced <code>reinit</code> by <code>initial equation</code></p>
<p><i>ModelicaAdditions.HeatFlow1D.Interfaces</i></p>
<p>Definition of connectors <i>Surface_a</i> and <i>Surface_b</i>:<br>
<code>flow SIunits.HeatFlux q;</code> changed to <code>flow SIunits.HeatFlowRate q;</code></p>
<p><i>MultiBody.Parts.InertialSystem</i></p>
<p>Icon corrected.</p>
</html>"));
      end Version_1_5;

      class Version_1_4 "Version 1.4 (June 28, 2001)"
        extends Modelica.Icons.ReleaseNotes;
        annotation(
          Documentation(info = "<html>

<ul>
<li>Several minor bugs fixed. </li>
<li>New models:<br>
        Modelica.Blocks.Interfaces.IntegerRealInput/IntegerRealOutput,<br>
        Modelica.Blocks.Math.TwoInputs/TwoOutputs<br>
        Modelica.Electrical.Analog.Ideal.IdealOpAmp3Pin,<br>
        Modelica.Mechanics.Rotational.Move,<br>
        Modelica.Mechanics.Translational.Move.<br>
        </li>
</ul>
<hr>
<h4>Version 1.4.1beta1 (February 12, 2001)</h4>
<p> Adapted to Modelica 1.4</p>
<hr>
<h4>Version 1.3.2beta2 (June 20, 2000)</h4>
<ul>
        <li>New subpackage Modelica.Mechanics.<b>Translational</b></li>
        <li>Changes to Modelica.Mechanics.<b>Rotational</b>:<br>
           New elements:
<pre>
IdealGearR2T    Ideal gear transforming rotational in translational motion.
Position        Forced movement of a flange with a reference angle
                                   given as input signal
RelativeStates  Definition of relative state variables
</pre>
</li>
        <li>Changes to Modelica.<b>SIunits</b>:<br>
          Introduced new types:<br>
          type Temperature = ThermodynamicTemperature;<br>
          types DerDensityByEnthalpy, DerDensityByPressure,
          DerDensityByTemperature, DerEnthalpyByPressure,
          DerEnergyByDensity, DerEnergyByPressure<br>
          Attribute \"final\" removed from min and max values
          in order that these values can still be changed to narrow
          the allowed range of values.<br>
          Quantity=\"Stress\" removed from type \"Stress\", in order
          that a type \"Stress\" can be connected to a type \"Pressure\".</li>
        <li>Changes to Modelica.<b>Icons</b>:<br>
           New icons for motors and gearboxes.</li>
        <li>Changes to Modelica.<b>Blocks.Interfaces</b>:<br>
           Introduced a replaceable signal type into
           Blocks.Interfaces.RealInput/RealOutput:
<pre>
replaceable type SignalType = Real
</pre>
           in order that the type of the signal of an input/output block
           can be changed to a physical type, for example:
<pre>
Sine sin1(outPort(redeclare type SignalType=Modelica.SIunits.Torque))
</pre>
</li>
</ul>
<hr>
<h4>Version 1.3.1 (Dec. 13, 1999)</h4>
<p>
First official release of the library.
</p>
</html>"));
      end Version_1_4;
      annotation(
        Documentation(info = "<html>

<p>
This section summarizes the changes that have been performed
on the Modelica standard library. Furthermore, it is explained in
<a href=\"modelica://Modelica.UsersGuide.ReleaseNotes.VersionManagement\">Modelica.UsersGuide.ReleaseNotes.VersionManagement</a>
how the versions are managed with the subversion management systems.
This is especially important for maintenance (bug-fix) releases where the
main version number is not changed.
</p>

</html>"));
    end ReleaseNotes;

    class Contact "Contact"
      extends Modelica.Icons.Contact;
      annotation(
        Documentation(info = "<html>
<dl><dt>The Modelica Standard Library (this Modelica package) is developed by many people from different organizations (see list below). It is licensed under the <a href=\"modelica://Modelica.UsersGuide.ModelicaLicense2\">Modelica License 2</a> by:</dt>
<dt>&nbsp;</dt>
<dd>Modelica Association</dd>
<dd>(Ideella F&ouml;reningar 822003-8858 in Link&ouml;ping)</dd>
<dd>c/o PELAB, IDA, Link&ouml;pings Universitet</dd>
<dd>S-58183 Link&ouml;ping</dd>
<dd>Sweden</dd>
<dd>email: <a href=\"mailto:Board@Modelica.org\">Board@Modelica.org</a></dd>
<dd>web: <a href=\"https://www.Modelica.org\">https://www.Modelica.org</a></dd>
<dd>&nbsp;&nbsp;</dd>

<dt>The development of this Modelica package, starting with version 3.2.1, is organized by:</dt>
<dd><a href=\"http://www.haumer.at/eindex.htm\">Anton Haumer</a></dd>
<dd>Technical Consulting &amp; Electrical Engineering</dd>
<dd>A-3423 St.Andrae-Woerdern, Hadersfelderweg 21</dd>
<dd>Austria</dd>
<dd>email: <a href=\"mailto:A.Haumer@Haumer.at\">A.Haumer@Haumer.at</a></dd>
<dd>&nbsp;&nbsp;</dd>

<dt>The development of this Modelica package up to and including version 3.2.1 was organized by:</dt>
<dd><a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a></dd>
<dd>German Aerospace Center (DLR)</dd>
<dd>Robotics and Mechatronics Center (RMC)</dd>
<dd>Institute of System Dynamics and Control (SR)</dd>
<dd>Postfach 1116</dd>
<dd>D-82230 Wessling</dd>
<dd>Germany</dd>
<dd>email: <a href=\"mailto:Martin.Otter@dlr.de\">Martin.Otter@dlr.de</a></dd>
</dl>
<p>Since end of 2007, the development of the sublibraries of package Modelica is organized by personal and/or organizational <b>library officers</b> assigned by the Modelica Association. They are responsible for the maintenance and for the further organization of the development. Other persons may also contribute, but the final decision for library improvements and/or changes is performed by the responsible library officer(s). In order that a new sublibrary or a new version of a sublibrary is ready to be released, the responsible library officers report the changes to the members of the Modelica Association and the library is made available for beta testing to interested parties before a final decision. A new release of a sublibrary is formally decided by voting of the Modelica Association members.</p>
<p>The following library officers are currently assigned:</p>

<table border=1 cellspacing=0 cellpadding=2>
<tr><td valign=\"top\"><b>Sublibraries</b> </td>
   <td valign=\"top\"><b>Library officers</b></td>
</tr>

<tr><td valign=\"top\"> Complex </td>
   <td valign=\"top\"> DLR Institute of System Dynamics and Control, Oberpfaffenhofen, Germany<br>
        (Martin Otter)<br>
        Anton Haumer, Consultant, St.Andrae-Woerdern, Austria, <br>
        Christian Kral, Electric Machines, Drives and Systems, Vienna, Austria
</td>

</tr>

<tr><td valign=\"top\"> Modelica.Blocks <br>
                      Modelica.Constants </td>
   <td valign=\"top\"> DLR Institute of System Dynamics and Control, Oberpfaffenhofen, Germany<br>
        (Martin Otter)</td>
</tr>

<tr><td valign=\"top\"> Modelica.Electrical.Analog<br>
                      Modelica.Electrical.Digital<br>
                      Modelica.Electrical.Spice3 </td>
   <td valign=\"top\"> Fraunhofer Institute for Integrated Circuits, Dresden, Germany<br>
      (Christoph Clauss)</td>
</tr>

<tr><td valign=\"top\"> Modelica.ComplexBlocks<br>
                      Modelica.Electrical.Machines<br>
                      Modelica.Electrical.MultiPhase<br>
                      Modelica.Electrical.QuasiStationary </td>
   <td valign=\"top\"> Anton Haumer, Consultant, St.Andrae-Woerdern, Austria, and<br>
      Christian Kral, Electric Machines, Drives and Systems, Vienna, Austria</td>
</tr>

<tr><td valign=\"top\"> Modelica.Magnetic.FluxTubes </td>
   <td valign=\"top\"> Thomas B&ouml;drich, Dresden, Germany<br>
                               (Dresden University of Technology,<br>
                               Institute of Electromechanical and Electronic Design)
</td>
</tr>

<tr><td valign=\"top\"> Modelica.Magnetic.FundamentalWave </td>
   <td valign=\"top\"> Electric Machines, Drives and Systems, Vienna, Austria<br>
                     Anton Haumer, Consultant, St.Andrae-Woerdern, Austria
</td>
</tr>

<tr><td valign=\"top\"> Modelica.Fluid </td>
   <td valign=\"top\"> Politecnico di Milano (Francesco Casella), and<br>
                            R&uuml;diger Franke (ABB)</td>
</tr>

<tr><td valign=\"top\"> Modelica.Fluid.Dissipation </td>
   <td valign=\"top\"> XRG Simulation, Hamburg, Germany (Stefan Wischhusen)</td>
</tr>

<tr><td valign=\"top\"> Modelica.Icons </td>
   <td valign=\"top\"> Modelon AB, Lund, Sweden (Johan Andreasson) </td>
</tr>

<tr><td valign=\"top\"> Modelica.Math </td>
   <td valign=\"top\"> DLR Institute of System Dynamics and Control, Oberpfaffenhofen, Germany<br>
      (Martin Otter)</td>
</tr>

<tr><td valign=\"top\"> Modelica.ComplexMath </td>
   <td valign=\"top\"> DLR Institute of System Dynamics and Control, Oberpfaffenhofen, Germany<br>
      (Martin Otter)<br>
      Anton Haumer, Consultant, St.Andrae-Woerdern, Austria,<br>
      Christian Kral, Electric Machines, Drives and Systems, Vienna, Austria
   </td>
</tr>

<tr><td valign=\"top\"> Modelica.Mechanics.MultiBody </td>
   <td valign=\"top\"> DLR Institute of System Dynamics and Control, Oberpfaffenhofen, Germany<br>
      (Martin Otter),<br>
       Modelon AB, Lund, Sweden (Johan Andreasson) </td>
</tr>

<tr><td valign=\"top\"> Modelica.Mechanics.Rotational </td>
   <td valign=\"top\"> DLR Institute of System Dynamics and Control, Oberpfaffenhofen, Germany<br>
      (Martin Otter)<br>
      Anton Haumer, Consultant, St.Andrae-Woerdern, Austria,<br>
      Christian Kral, Electric Machines, Drives and Systems, Vienna, Austria,<br>
      Modelon AB, Lund, Sweden (Johan Andreasson)</td>
</tr>

<tr><td valign=\"top\"> Modelica.Mechanics.Translational </td>
   <td valign=\"top\"> Anton Haumer, Consultant, St.Andrae-Woerdern, Austria,<br>
      Christian Kral, Electric Machines, Drives and Systems, Vienna, Austria, <br>
      DLR Institute of System Dynamics and Control, Oberpfaffenhofen, Germany<br>
      (Martin Otter)<br>
       Modelon AB, Lund, Sweden (Johan Andreasson)</td>
</tr>

<tr><td valign=\"top\"> Modelica.Media </td>
   <td valign=\"top\"> Modelon AB, Lund, Sweden (Hubertus Tummescheit) </td>
</tr>

<tr><td valign=\"top\"> Modelica.SIunits <br>
      Modelica.StateGraph </td>
   <td valign=\"top\"> DLR Institute of System Dynamics and Control, Oberpfaffenhofen, Germany<br>
      (Martin Otter)</td>
</tr>

<tr><td valign=\"top\"> Modelica.Thermal.FluidHeatFlow <br>
      Modelica.Thermal.HeatTransfer </td>
   <td valign=\"top\"> Anton Haumer, Consultant, St.Andrae-Woerdern, Austria, and<br>
      Christian Kral, Electric Machines, Drives and Systems, Vienna, Austria</td>

<tr><td valign=\"top\"> Modelica.Utilities </td>
   <td valign=\"top\"> DLR Institute of System Dynamics and Control, Oberpfaffenhofen, Germany<br>
      (Martin Otter)<br>
      Dassault Syst&egrave;mes AB, Lund, Sweden (Hans Olsson)</td>
</tr>
</table>

<p>
The following people have directly contributed to the implementation
of the Modelica package (many more people have contributed to the design):
</p>

<table border=1 cellspacing=0 cellpadding=2>
<tr><td valign=\"top\"><b>Marcus Baur</b> </td>
   <td valign=\"top\"> Institute of System Dynamics and Control<br>
     DLR, German Aerospace Center, <br>
     Oberpfaffenhofen, Germany</td>
   <td valign=\"top\"> Complex<br>
                     Modelica.Math.Vectors<br>
                     Modelica.Math.Matrices</td>
</tr>

<tr><td valign=\"top\"><b>Peter Beater</b> </td>
   <td valign=\"top\"> University of Paderborn, Germany</td>
   <td valign=\"top\"> Modelica.Mechanics.Translational </td>
</tr>



<tr><td valign=\"top\"><b>Thomas Beutlich</b> </td>
   <td valign=\"top\"> ITI GmbH, Germany</td>
   <td valign=\"top\"> Modelica.Blocks.Sources.CombiTimeTable<br>
                       Modelica.Blocks.Tables </td>
</tr>

<tr><td valign=\"top\"><b>Thomas B&ouml;drich</b> </td>
   <td valign=\"top\"> Dresden University of Technology, Germany</td>
   <td valign=\"top\"> Modelica.Magnetic.FluxTubes </td>

<tr><td valign=\"top\"><b>Dag Br&uuml;ck</b> </td>
   <td valign=\"top\"> Dassault Syst&egrave;mes AB, Lund, Sweden</td>
   <td valign=\"top\"> Modelica.Utilities</td>
</tr>

<tr><td valign=\"top\"><b>Francesco Casella</b> </td>
   <td valign=\"top\"> Politecnico di Milano, Milano, Italy</td>
   <td valign=\"top\"> Modelica.Fluid<br>
                            Modelica.Media</td>
</tr>

<tr><td valign=\"top\"><b>Christoph Clauss</b> </td>
   <td valign=\"top\"> Fraunhofer Institute for Integrated Circuits,<br> Dresden, Germany</td>
   <td valign=\"top\"> Modelica.Electrical.Analog<br>
     Modelica.Electrical.Digital<br>
     Modelica.Electrical.Spice3</td>
</tr>

<tr><td valign=\"top\"><b>Jonas Eborn</b> </td>
   <td valign=\"top\"> Modelon AB, Lund, Sweden</td>
   <td valign=\"top\"> Modelica.Media</td>
</tr>

<tr><td valign=\"top\"><b>Hilding Elmqvist</b> </td>
   <td valign=\"top\"> Dassault Syst&egrave;mes AB, Lund, Sweden</td>
   <td valign=\"top\"> Modelica.Mechanics.MultiBody<br>
                   Modelica.Fluid<br>
     Modelica.Media<br>
     Modelica.StateGraph<br>
     Modelica.Utilities<br>
     Conversion from 1.6 to 2.0</td>
</tr>

<tr><td valign=\"top\"><b>R&uuml;diger Franke</b> </td>
   <td valign=\"top\"> ABB Corporate Research,<br>Ladenburg, German</td>
   <td valign=\"top\"> Modelica.Fluid<br>
                            Modelica.Media</td>
</tr>

<tr><td valign=\"top\"><b>Manuel Gr&auml;ber</b> </td>
   <td valign=\"top\"> Institut f&uuml;r Thermodynamik, <br>
     Technische Universit&auml;t Braunschweig, <br>
     Germany</td>
   <td valign=\"top\"> Modelica.Fluid</td>
</tr>

<tr><td valign=\"top\"><b>Anton Haumer</b> </td>
   <td valign=\"top\"> Consultant, St.Andrae-Woerdern,<br>Austria</td>
   <td valign=\"top\"> Modelica.ComplexBlocks<br>
     Modelica.Electrical.Machines<br>
     Modelica.Electrical.Multiphase<br>
     Modelica.Electrical.QuasiStationary<br>
     Modelica.Magnetics.FundamentalWave<br>
     Modelica.Mechanics.Rotational<br>
     Modelica.Mechanics.Translational<br>
     Modelica.Thermal.FluidHeatFlow<br>
     Modelica.Thermal.HeatTransfer<br>
     Modelica.ComplexMath<br>
     Conversion from 1.6 to 2.0<br>
     Conversion from 2.2 to 3.0</td>
</tr>

<tr><td valign=\"top\"><b>Hans-Dieter Joos</b> </td>
   <td valign=\"top\"> Institute of System Dynamics and Control<br>
     DLR, German Aerospace Center, <br>
     Oberpfaffenhofen, Germany</td>
   <td valign=\"top\"> Modelica.Math.Matrices</td>
</tr>

<tr><td valign=\"top\"><b>Christian Kral</b> </td>
   <td valign=\"top\"> Electric Machines, Drives and Systems, Vienna, Austria</td>
   <td valign=\"top\"> Modelica.ComplexBlocks<br>
     Modelica.Electrical.Machines<br>
     Modelica.Electrical.MultiPhase<br>
     Modelica.Electrical.QuasiStationary<br>
     Modelica.Magnetics.FundamentalWave<br>
     Modelica.Mechanics.Rotational<br>
     Modelica.Mechanics.Translational<br>
     Modelica.Thermal.FluidHeatFlow<br>
     Modelica.Thermal.HeatTransfer<br>
     Modelica.ComplexMath
     </td>
</tr>

<tr><td valign=\"top\"><b>Sven Erik Mattsson</b> </td>
   <td valign=\"top\"> Dassault Syst&egrave;mes AB, Lund, Sweden</td>
   <td valign=\"top\"> Modelica.Mechanics.MultiBody</td>
</tr>
<tr><td valign=\"top\"><b>Hans Olsson</b> </td>
   <td valign=\"top\"> Dassault Syst&egrave;mes AB, Lund, Sweden</td>
   <td valign=\"top\"> Modelica.Blocks<br>
     Modelica.Math.Matrices<br>
     Modelica.Utilities<br>
     Conversion from 1.6 to 2.0<br>
     Conversion from 2.2 to 3.0</td>
</tr>

<tr><td valign=\"top\"><b>Martin Otter</b> </td>
   <td valign=\"top\"> Institute of System Dynamics and Control<br>
     DLR, German Aerospace Center, <br>
     Oberpfaffenhofen, Germany</td>
   <td valign=\"top\"> Complex<br>
     Modelica.Blocks<br>
     Modelica.Fluid<br>
     Modelica.Mechanics.MultiBody<br>
     Modelica.Mechanics.Rotational<br>
     Modelica.Mechanics.Translational<br>
     Modelica.Math<br>
     Modelica.ComplexMath<br>
     Modelica.Media<br>
     Modelica.SIunits<br>
     Modelica.StateGraph<br>
     Modelica.Thermal.HeatTransfer<br>
     Modelica.Utilities<br>
     ModelicaReference<br>
     Conversion from 1.6 to 2.0<br>
     Conversion from 2.2 to 3.0</td>
</tr>

<tr><td valign=\"top\"><b>Katrin Pr&ouml;l&szlig;</b> </td>
   <td valign=\"top\"> Modelon AB, Lund, Sweden<br>
                            until 2008:<br>
                            Department of Technical Thermodynamics,<br>
     Technical University Hamburg-Harburg,<br>Germany</td>
   <td valign=\"top\"> Modelica.Fluid<br>
                            Modelica.Media</td>
</tr>

<tr><td valign=\"top\"><b>Christoph C. Richter</b> </td>
   <td valign=\"top\"> until 2009:<br>
     Institut f&uuml;r Thermodynamik, <br>
     Technische Universit&auml;t Braunschweig, <br>
     Germany</td>
   <td valign=\"top\"> Modelica.Fluid<br>
                            Modelica.Media</td>
</tr>

<tr><td valign=\"top\"><b>Andr&eacute; Schneider</b> </td>
   <td valign=\"top\"> Fraunhofer Institute for Integrated Circuits,<br> Dresden, Germany</td>
   <td valign=\"top\"> Modelica.Electrical.Analog<br>
     Modelica.Electrical.Digital</td>
</tr>
<tr><td valign=\"top\"><b>Christian Schweiger</b> </td>
   <td valign=\"top\"> Until 2006:<br>
     Institute of System Dynamics and Control,<br>
     DLR, German Aerospace Center,<br>
     Oberpfaffenhofen, Germany</td>
   <td valign=\"top\"> Modelica.Mechanics.Rotational<br>
     ModelicaReference<br>
     Conversion from 1.6 to 2.0</td>
</tr>

<tr><td valign=\"top\"><b>Michael Sielemann</b> </td>
   <td valign=\"top\"> Institute of System Dynamics and Control<br>
     DLR, German Aerospace Center, <br>
     Oberpfaffenhofen, Germany</td>
   <td valign=\"top\"> Modelica.Fluid<br>
                       Modelica.Media</td>
</tr>

<tr><td valign=\"top\"><b>Michael Tiller</b> </td>
   <td valign=\"top\"> Emmeskay, Inc., Dearborn, MI, U.S.A, <br>
     (previously Ford Motor Company, Dearborn) </td>
   <td valign=\"top\"> Modelica.Media<br>
     Modelica.Thermal.HeatTransfer</td>
</tr>
<tr><td valign=\"top\"><b>Hubertus Tummescheit</b> </td>
   <td valign=\"top\"> Modelon AB, Lund, Sweden </td>
   <td valign=\"top\"> Modelica.Media<br>
     Modelica.Thermal.HeatTransfer</td>
</tr>

<tr><td valign=\"top\"><b>Thorsten Vahlenkamp</b> </td>
   <td valign=\"top\"> until 2010:<br>
                     XRG Simulation GmbH, Hamburg, Germany</td>
   <td valign=\"top\"> Modelica.Fluid.Dissipation</td>
</tr>

<tr><td valign=\"top\"><b>Nico Walter</b> </td>
   <td valign=\"top\"> Master thesis at HTWK Leipzig<br>
     (Prof. R. M&uuml;ller) and<br>
     DLR Oberpfaffenhofen, Germany</td>
   <td valign=\"top\"> Modelica.Math.Matrices</td>
</tr>

<tr><td valign=\"top\"><b>Michael Wetter</b> </td>
   <td valign=\"top\"> Lawrence Berkeley National Laboratory; U.S.A.</td>
   <td valign=\"top\"> Modelica.Fluid</td>
</tr>

<tr><td valign=\"top\"><b>Hans-J&uuml;rg Wiesmann</b> </td>
   <td valign=\"top\"> Switzerland</td>
   <td valign=\"top\"> Modelica.ComplexMath</td>
</tr>

<tr><td valign=\"top\"><b>Stefan Wischhusen</b> </td>
   <td valign=\"top\"> XRG Simulation GmbH, Hamburg, Germany</td>
   <td valign=\"top\"> Modelica.Fluid.Dissipation<br>
                       Modelica.Media</td>
</tr>
</table>

</html>"));
    end Contact;
    annotation(
      DocumentationClass = true,
      Documentation(info = "<html>
<p>
Package <b>Modelica</b> is a <b>standardized</b> and <b>pre-defined</b> package
that is developed together with the Modelica language from the
Modelica Association, see
<a href=\"https://www.Modelica.org\">https://www.Modelica.org</a>.
It is also called <b>Modelica Standard Library</b>.
It provides constants, types, connectors, partial models and model
components in various disciplines.
</p>
<p>
This is a short <b>User's Guide</b> for
the overall library. Some of the main sublibraries have their own
User's Guides that can be accessed by the following links:
</p>

<table border=1 cellspacing=0 cellpadding=2>
<tr><td valign=\"top\"><a href=\"modelica://Modelica.Electrical.Digital.UsersGuide\">Digital</a>
   </td>
   <td valign=\"top\">Library for digital electrical components based on the VHDL standard
   (2-,3-,4-,9-valued logic)</td>
</tr>

<tr><td valign=\"top\"><a href=\"modelica://Modelica.Magnetic.FluxTubes.UsersGuide\">FluxTubes</a>
    </td>
   <td valign=\"top\">Library for modelling of electromagnetic devices with lumped magnetic networks</td>
</tr>

<tr><td valign=\"top\"><a href=\"modelica://Modelica.Mechanics.MultiBody.UsersGuide\">MultiBody</a>
    </td>
   <td valign=\"top\">Library to model 3-dimensional mechanical systems</td>
</tr>

<tr><td valign=\"top\"><a href=\"modelica://Modelica.Mechanics.Rotational.UsersGuide\">Rotational</a>
    </td>
   <td valign=\"top\">Library to model 1-dimensional mechanical systems</td>
</tr>

<tr><td valign=\"top\"><a href=\"modelica://Modelica.Fluid.UsersGuide\">Fluid</a></td>
    <td valign=\"top\">Library of 1-dim. thermo-fluid flow models using the Modelica.Media media description</td>

<tr><td valign=\"top\"><a href=\"modelica://Modelica.Media.UsersGuide\">Media</a>
    </td>
   <td valign=\"top\">Library of media property models</td>
</tr>
<tr><td valign=\"top\"><a href=\"modelica://Modelica.SIunits.UsersGuide\">SIunits</a> </td>
   <td valign=\"top\">Library of type definitions based on SI units according to ISO 31-1992</td>
</tr>

<tr><td valign=\"top\"><a href=\"modelica://Modelica.StateGraph.UsersGuide\">StateGraph</a>
    </td>
   <td valign=\"top\">Library to model discrete event and reactive systems by hierarchical state machines</td>
</tr>

<tr><td valign=\"top\"><a href=\"modelica://Modelica.Utilities.UsersGuide\">Utilities</a>
    </td>
   <td valign=\"top\">Library of utility functions especially for scripting (Files, Streams, Strings, System)</td>
</tr>
</table>

</html>"));
  end UsersGuide;

  package Blocks "Library of basic input/output control blocks (continuous, discrete, logical, table blocks)"
    import SI = Modelica.SIunits;
    extends Modelica.Icons.Package;

    package Examples "Library of examples to demonstrate the usage of package Blocks"
      extends Modelica.Icons.ExamplesPackage;

      model PID_Controller "Demonstrates the usage of a Continuous.LimPID controller"
        extends Modelica.Icons.Example;
        parameter Modelica.SIunits.Angle driveAngle = 1.57 "Reference distance to move";
        Modelica.Blocks.Continuous.LimPID PI(k = 100, Ti = 0.1, yMax = 12, Ni = 0.1, initType = Modelica.Blocks.Types.InitPID.SteadyState, limitsAtInit = false, controllerType = Modelica.Blocks.Types.SimpleController.PI, Td = 0.1) annotation(
          Placement(transformation(extent = {{-56, -20}, {-36, 0}}, rotation = 0)));
        Modelica.Mechanics.Rotational.Components.Inertia inertia1(phi(fixed = true, start = 0), J = 1, a(fixed = true, start = 0)) annotation(
          Placement(transformation(extent = {{2, -20}, {22, 0}}, rotation = 0)));
        Modelica.Mechanics.Rotational.Sources.Torque torque annotation(
          Placement(transformation(extent = {{-25, -20}, {-5, 0}}, rotation = 0)));
        Modelica.Mechanics.Rotational.Components.SpringDamper spring(c = 1e4, d = 100, stateSelect = StateSelect.prefer, w_rel(fixed = true)) annotation(
          Placement(transformation(extent = {{32, -20}, {52, 0}}, rotation = 0)));
        Modelica.Mechanics.Rotational.Components.Inertia inertia2(J = 2) annotation(
          Placement(transformation(extent = {{60, -20}, {80, 0}}, rotation = 0)));
        Modelica.Blocks.Sources.KinematicPTP kinematicPTP(startTime = 0.5, deltaq = {driveAngle}, qd_max = {1}, qdd_max = {1}) annotation(
          Placement(transformation(extent = {{-92, 20}, {-72, 40}}, rotation = 0)));
        Modelica.Blocks.Continuous.Integrator integrator(initType = Modelica.Blocks.Types.Init.InitialState) annotation(
          Placement(transformation(extent = {{-63, 20}, {-43, 40}}, rotation = 0)));
        Modelica.Mechanics.Rotational.Sensors.SpeedSensor speedSensor annotation(
          Placement(transformation(extent = {{22, -50}, {2, -30}}, rotation = 0)));
        Modelica.Mechanics.Rotational.Sources.ConstantTorque loadTorque(tau_constant = 10, useSupport = false) annotation(
          Placement(transformation(extent = {{98, -15}, {88, -5}}, rotation = 0)));
      initial equation
        der(spring.w_rel) = 0;
      equation
        connect(spring.flange_b, inertia2.flange_a) annotation(
          Line(points = {{52, -10}, {60, -10}}, color = {0, 0, 0}));
        connect(inertia1.flange_b, spring.flange_a) annotation(
          Line(points = {{22, -10}, {32, -10}}, color = {0, 0, 0}));
        connect(torque.flange, inertia1.flange_a) annotation(
          Line(points = {{-5, -10}, {2, -10}}, color = {0, 0, 0}));
        connect(kinematicPTP.y[1], integrator.u) annotation(
          Line(points = {{-71, 30}, {-65, 30}}, color = {0, 0, 127}));
        connect(speedSensor.flange, inertia1.flange_b) annotation(
          Line(points = {{22, -40}, {22, -10}}, color = {0, 0, 0}));
        connect(loadTorque.flange, inertia2.flange_b) annotation(
          Line(points = {{88, -10}, {80, -10}}, color = {0, 0, 0}));
        connect(PI.y, torque.tau) annotation(
          Line(points = {{-35, -10}, {-27, -10}}, color = {0, 0, 127}));
        connect(speedSensor.w, PI.u_m) annotation(
          Line(points = {{1, -40}, {-46, -40}, {-46, -22}}, color = {0, 0, 127}));
        connect(integrator.y, PI.u_s) annotation(
          Line(points = {{-42, 30}, {-37, 30}, {-37, 11}, {-67, 11}, {-67, -10}, {-58, -10}}, color = {0, 0, 127}));
        annotation(
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-99, 48}, {-32, 8}}, lineColor = {255, 0, 0}), Text(extent = {{-98, 59}, {-31, 51}}, lineColor = {255, 0, 0}, textString = "reference speed generation"), Text(extent = {{-98, -46}, {-60, -52}}, lineColor = {255, 0, 0}, textString = "PI controller"), Line(points = {{-76, -44}, {-57, -23}}, color = {255, 0, 0}, arrow = {Arrow.None, Arrow.Filled}), Rectangle(extent = {{-25, 6}, {99, -50}}, lineColor = {255, 0, 0}), Text(extent = {{4, 14}, {71, 7}}, lineColor = {255, 0, 0}, textString = "plant (simple drive train)")}),
          experiment(StopTime = 4),
          Documentation(info = "<html>

<p>
This is a simple drive train controlled by a PID controller:
</p>

<ul>
<li> The two blocks \"kinematic_PTP\" and \"integrator\" are used to generate
     the reference speed (= constant acceleration phase, constant speed phase,
     constant deceleration phase until inertia is at rest). To check
     whether the system starts in steady state, the reference speed is
     zero until time = 0.5 s and then follows the sketched trajectory.</li>

<li> The block \"PI\" is an instance of \"Blocks.Continuous.LimPID\" which is
     a PID controller where several practical important aspects, such as
     anti-windup-compensation has been added. In this case, the control block
     is used as PI controller.</li>

<li> The output of the controller is a torque that drives a motor inertia
     \"inertia1\". Via a compliant spring/damper component, the load
     inertia \"inertia2\" is attached. A constant external torque of 10 Nm
     is acting on the load inertia.</li>
</ul>

<p>
The PI controller settings included \"limitAtInit=false\", in order that
the controller output limits of 12 Nm are removed from the initialization
problem.
</p>

<p>
The PI controller is initialized in steady state (initType=SteadyState)
and the drive shall also be initialized in steady state.
However, it is not possible to initialize \"inertia1\" in SteadyState, because
\"der(inertia1.phi)=inertia1.w=0\" is an input to the PI controller that
defines that the derivative of the integrator state is zero (= the same
condition that was already defined by option SteadyState of the PI controller).
Furthermore, one initial condition is missing, because the absolute position
of inertia1 or inertia2 is not defined. The solution shown in this examples is
to initialize the angle and the angular acceleration of \"inertia1\".
</p>

<p>
In the following figure, results of a typical simulation are shown:
</p>

<img src=\"modelica://Modelica/Resources/Images/Blocks/PID_controller.png\"
     alt=\"PID_controller.png\"><br>

<img src=\"modelica://Modelica/Resources/Images/Blocks/PID_controller2.png\"
     alt=\"PID_controller2.png\">

<p>
In the upper figure the reference speed (= integrator.y) and
the actual speed (= inertia1.w) are shown. As can be seen,
the system initializes in steady state, since no transients
are present. The inertia follows the reference speed quite good
until the end of the constant speed phase. Then there is a deviation.
In the lower figure the reason can be seen: The output of the
controller (PI.y) is in its limits. The anti-windup compensation
works reasonably, since the input to the limiter (PI.limiter.u)
is forced back to its limit after a transient phase.
</p>

</html>"));
      end PID_Controller;

      model Filter "Demonstrates the Continuous.Filter block with various options"
        extends Modelica.Icons.Example;
        parameter Integer order = 3;
        parameter Modelica.SIunits.Frequency f_cut = 2;
        parameter Modelica.Blocks.Types.FilterType filterType = Modelica.Blocks.Types.FilterType.LowPass "Type of filter (LowPass/HighPass)";
        parameter Modelica.Blocks.Types.Init init = Modelica.Blocks.Types.Init.SteadyState "Type of initialization (no init/steady state/initial state/initial output)";
        parameter Boolean normalized = true;
        Modelica.Blocks.Sources.Step step(startTime = 0.1, offset = 0.1) annotation(
          Placement(transformation(extent = {{-60, 40}, {-40, 60}})));
        Modelica.Blocks.Continuous.Filter CriticalDamping(analogFilter = Modelica.Blocks.Types.AnalogFilter.CriticalDamping, normalized = normalized, init = init, filterType = filterType, order = order, f_cut = f_cut, f_min = 0.8 * f_cut) annotation(
          Placement(transformation(extent = {{-20, 40}, {0, 60}})));
        Modelica.Blocks.Continuous.Filter Bessel(normalized = normalized, analogFilter = Modelica.Blocks.Types.AnalogFilter.Bessel, init = init, filterType = filterType, order = order, f_cut = f_cut, f_min = 0.8 * f_cut) annotation(
          Placement(transformation(extent = {{-20, 0}, {0, 20}})));
        Modelica.Blocks.Continuous.Filter Butterworth(normalized = normalized, analogFilter = Modelica.Blocks.Types.AnalogFilter.Butterworth, init = init, filterType = filterType, order = order, f_cut = f_cut, f_min = 0.8 * f_cut) annotation(
          Placement(transformation(extent = {{-20, -40}, {0, -20}})));
        Modelica.Blocks.Continuous.Filter ChebyshevI(normalized = normalized, analogFilter = Modelica.Blocks.Types.AnalogFilter.ChebyshevI, init = init, filterType = filterType, order = order, f_cut = f_cut, f_min = 0.8 * f_cut) annotation(
          Placement(transformation(extent = {{-20, -80}, {0, -60}})));
      equation
        connect(step.y, CriticalDamping.u) annotation(
          Line(points = {{-39, 50}, {-22, 50}}, color = {0, 0, 127}, smooth = Smooth.None));
        connect(step.y, Bessel.u) annotation(
          Line(points = {{-39, 50}, {-32, 50}, {-32, 10}, {-22, 10}}, color = {0, 0, 127}, smooth = Smooth.None));
        connect(Butterworth.u, step.y) annotation(
          Line(points = {{-22, -30}, {-32, -30}, {-32, 50}, {-39, 50}}, color = {0, 0, 127}, smooth = Smooth.None));
        connect(ChebyshevI.u, step.y) annotation(
          Line(points = {{-22, -70}, {-32, -70}, {-32, 50}, {-39, 50}}, color = {0, 0, 127}, smooth = Smooth.None));
        annotation(
          experiment(StopTime = 0.9),
          Documentation(info = "<html>

<p>
This example demonstrates various options of the
<a href=\"modelica://Modelica.Blocks.Continuous.Filter\">Filter</a> block.
A step input starts at 0.1 s with an offset of 0.1, in order to demonstrate
the initialization options. This step input drives 4 filter blocks that
have identical parameters, with the only exception of the used analog filter type
(CriticalDamping, Bessel, Butterworth, Chebyshev of type I). All the main options
can be set via parameters and are then applied to all the 4 filters.
The default setting uses low pass filters of order 3 with a cut-off frequency of
2 Hz resulting in the following outputs:
</p>

<img src=\"modelica://Modelica/Resources/Images/Blocks/Filter1.png\"
     alt=\"Filter1.png\">
</html>"));
      end Filter;

      model FilterWithDifferentiation "Demonstrates the use of low pass filters to determine derivatives of filters"
        extends Modelica.Icons.Example;
        parameter Modelica.SIunits.Frequency f_cut = 2;
        Modelica.Blocks.Sources.Step step(startTime = 0.1, offset = 0.1) annotation(
          Placement(transformation(extent = {{-80, 40}, {-60, 60}})));
        Modelica.Blocks.Continuous.Filter Bessel(f_cut = f_cut, filterType = Modelica.Blocks.Types.FilterType.LowPass, order = 3, analogFilter = Modelica.Blocks.Types.AnalogFilter.Bessel) annotation(
          Placement(transformation(extent = {{-40, 40}, {-20, 60}})));
        Continuous.Der der1 annotation(
          Placement(transformation(extent = {{-6, 40}, {14, 60}})));
        Continuous.Der der2 annotation(
          Placement(transformation(extent = {{30, 40}, {50, 60}})));
        Continuous.Der der3 annotation(
          Placement(transformation(extent = {{62, 40}, {82, 60}})));
      equation
        connect(step.y, Bessel.u) annotation(
          Line(points = {{-59, 50}, {-42, 50}}, color = {0, 0, 127}, smooth = Smooth.None));
        connect(Bessel.y, der1.u) annotation(
          Line(points = {{-19, 50}, {-8, 50}}, color = {0, 0, 127}, smooth = Smooth.None));
        connect(der1.y, der2.u) annotation(
          Line(points = {{15, 50}, {28, 50}}, color = {0, 0, 127}, smooth = Smooth.None));
        connect(der2.y, der3.u) annotation(
          Line(points = {{51, 50}, {60, 50}}, color = {0, 0, 127}, smooth = Smooth.None));
        annotation(
          experiment(StopTime = 0.9),
          Documentation(info = "<html>

<p>
This example demonstrates that the output of the
<a href=\"modelica://Modelica.Blocks.Continuous.Filter\">Filter</a> block
can be differentiated up to the order of the filter. This feature can be
used in order to make an inverse model realizable or to \"smooth\" a potential
discontinuous control signal.
</p>

</html>"));
      end FilterWithDifferentiation;

      model FilterWithRiseTime "Demonstrates to use the rise time instead of the cut-off frequency to define a filter"
        extends Modelica.Icons.Example;
        parameter Integer order = 2 "Filter order";
        parameter Modelica.SIunits.Time riseTime = 2 "Time to reach the step input";
        constant Real pi = Modelica.Constants.pi;
        Continuous.Filter filter_fac5(f_cut = 5 / (2 * pi * riseTime), order = order) annotation(
          Placement(transformation(extent = {{-20, -20}, {0, 0}})));
        Sources.Step step(startTime = 1) annotation(
          Placement(transformation(extent = {{-60, 20}, {-40, 40}})));
        Continuous.Filter filter_fac4(f_cut = 4 / (2 * pi * riseTime), order = order) annotation(
          Placement(transformation(extent = {{-20, 20}, {0, 40}})));
        Continuous.Filter filter_fac3(f_cut = 3 / (2 * pi * riseTime), order = order) annotation(
          Placement(transformation(extent = {{-20, 62}, {0, 82}})));
      equation
        connect(step.y, filter_fac5.u) annotation(
          Line(points = {{-39, 30}, {-30, 30}, {-30, -10}, {-22, -10}}, color = {0, 0, 127}, smooth = Smooth.None));
        connect(step.y, filter_fac4.u) annotation(
          Line(points = {{-39, 30}, {-22, 30}}, color = {0, 0, 127}, smooth = Smooth.None));
        connect(step.y, filter_fac3.u) annotation(
          Line(points = {{-39, 30}, {-30, 30}, {-30, 72}, {-22, 72}}, color = {0, 0, 127}, smooth = Smooth.None));
        annotation(
          experiment(StopTime = 4),
          Documentation(info = "<html>
<p>
Filters are usually parameterized with the cut-off frequency.
Sometimes, it is more meaningful to parameterize a filter with its
rise time, i.e., the time it needs until the output reaches the end value
of a step input. This is performed with the formula:
</p>

<blockquote><pre>
f_cut = fac/(2*pi*riseTime);
</pre></blockquote>

<p>
where \"fac\" is typically 3, 4, or 5. The following image shows
the results of a simulation of this example model
(riseTime = 2 s, fac=3, 4, and 5):
</p>

<img src=\"modelica://Modelica/Resources/Images/Blocks/FilterWithRiseTime.png\"
     alt=\"FilterWithRiseTime.png\">

<p>
Since the step starts at 1 s, and the rise time is 2 s, the filter output y
shall reach the value of 1 after 1+2=3 s. Depending on the factor \"fac\" this is
reached with different precisions. This is summarized in the following table:
</p>

<blockquote><table border=1 cellspacing=0 cellpadding=2>
<tr>
   <td>Filter order</td>
   <td>Factor fac</td>
   <td>Percentage of step value reached after rise time</td>
</tr>
<tr>
   <td align=\"center\">1</td>
   <td align=\"center\">3</td>
   <td align=\"center\">95.1 %</td>
</tr>
<tr>
   <td align=\"center\">1</td>
   <td align=\"center\">4</td>
   <td align=\"center\">98.2 %</td>
</tr>
<tr>
   <td align=\"center\">1</td>
   <td align=\"center\">5</td>
   <td align=\"center\">99.3 %</td>
</tr>

<tr>
   <td align=\"center\">2</td>
   <td align=\"center\">3</td>
   <td align=\"center\">94.7 %</td>
</tr>
<tr>
   <td align=\"center\">2</td>
   <td align=\"center\">4</td>
   <td align=\"center\">98.6 %</td>
</tr>
<tr>
   <td align=\"center\">2</td>
   <td align=\"center\">5</td>
   <td align=\"center\">99.6 %</td>
</tr>
</table></blockquote>

</html>"));
      end FilterWithRiseTime;

      model InverseModel "Demonstrates the construction of an inverse model"
        extends Modelica.Icons.Example;
        Continuous.FirstOrder firstOrder1(k = 1, T = 0.3, initType = Modelica.Blocks.Types.Init.SteadyState) annotation(
          Placement(transformation(extent = {{20, 20}, {0, 40}})));
        Sources.Sine sine(freqHz = 2, offset = 1, startTime = 0.2) annotation(
          Placement(transformation(extent = {{-80, 20}, {-60, 40}})));
        Math.InverseBlockConstraints inverseBlockConstraints annotation(
          Placement(transformation(extent = {{-10, 20}, {30, 40}})));
        Continuous.FirstOrder firstOrder2(k = 1, T = 0.3, initType = Modelica.Blocks.Types.Init.SteadyState) annotation(
          Placement(transformation(extent = {{20, -20}, {0, 0}})));
        Math.Feedback feedback annotation(
          Placement(transformation(extent = {{-40, 0}, {-60, -20}})));
        Continuous.CriticalDamping criticalDamping(n = 1, f = 50 * sine.freqHz) annotation(
          Placement(transformation(extent = {{-40, 20}, {-20, 40}})));
      equation
        connect(firstOrder1.y, inverseBlockConstraints.u2) annotation(
          Line(points = {{-1, 30}, {-6, 30}}, color = {0, 0, 127}, smooth = Smooth.None));
        connect(inverseBlockConstraints.y2, firstOrder1.u) annotation(
          Line(points = {{27, 30}, {22, 30}}, color = {0, 0, 127}, smooth = Smooth.None));
        connect(firstOrder2.y, feedback.u1) annotation(
          Line(points = {{-1, -10}, {-42, -10}}, color = {0, 0, 127}, smooth = Smooth.None));
        connect(sine.y, criticalDamping.u) annotation(
          Line(points = {{-59, 30}, {-42, 30}}, color = {0, 0, 127}, smooth = Smooth.None));
        connect(criticalDamping.y, inverseBlockConstraints.u1) annotation(
          Line(points = {{-19, 30}, {-12, 30}}, color = {0, 0, 127}, smooth = Smooth.None));
        connect(sine.y, feedback.u2) annotation(
          Line(points = {{-59, 30}, {-50, 30}, {-50, -2}}, color = {0, 0, 127}, smooth = Smooth.None));
        connect(inverseBlockConstraints.y1, firstOrder2.u) annotation(
          Line(points = {{31, 30}, {40, 30}, {40, -10}, {22, -10}}, color = {0, 0, 127}, smooth = Smooth.None));
        annotation(
          Documentation(info = "<html>
<p>
This example demonstrates how to construct an inverse model in Modelica
(for more details see <a href=\"https://www.modelica.org/events/Conference2005/online_proceedings/Session3/Session3c3.pdf\">Looye, Th&uuml;mmel, Kurze, Otter, Bals: Nonlinear Inverse Models for Control</a>).
</p>

<p>
For a linear, single input, single output system
</p>

<pre>
   y = n(s)/d(s) * u   // plant model
</pre>

<p>
the inverse model is derived by simply exchanging the numerator and
the denominator polynomial:
</p>

<pre>
   u = d(s)/n(s) * y   // inverse plant model
</pre>

<p>
If the denominator polynomial d(s) has a higher degree as the
numerator polynomial n(s) (which is usually the case for plant models),
then the inverse model is no longer proper, i.e., it is not causal.
To avoid this, an approximate inverse model is constructed by adding
a sufficient number of poles to the denominator of the inverse model.
This can be interpreted as filtering the desired output signal y:
</p>

<pre>
   u = d(s)/(n(s)*f(s)) * y  // inverse plant model with filtered y
</pre>

<p>
With Modelica it is in principal possible to construct inverse models not only
for linear but also for non-linear models and in particular for every
Modelica model. The basic construction mechanism is explained at hand
of this example:
</p>

<img src=\"modelica://Modelica/Resources/Images/Blocks/InverseModelSchematic.png\"
     alt=\"InverseModelSchematic.png\">

<p>
Here the first order block \"firstOrder1\" shall be inverted. This is performed
by connecting its inputs and outputs with an instance of block
Modelica.Blocks.Math.<b>InverseBlockConstraints</b>. By this connection,
the inputs and outputs are exchanged. The goal is to compute the input of the
\"firstOrder1\" block so that its output follows a given sine signal.
For this, the sine signal \"sin\" is first filtered with a \"CriticalDamping\"
filter of order 1 and then the output of this filter is connected to the output
of the \"firstOrder1\" block (via the InverseBlockConstraints block, since
2 outputs cannot be connected directly together in a block diagram).
</p>

<p>
In order to check the inversion, the computed input of \"firstOrder1\" is used
as input in an identical block \"firstOrder2\". The output of \"firstOrder2\" should
be the given \"sine\" function. The difference is constructed with the \"feedback\"
block. Since the \"sine\" function is filtered, one cannot expect that this difference
is zero. The higher the cut-off frequency of the filter, the closer is the
agreement. A typical simulation result is shown in the next figure:
</p>

<img src=\"modelica://Modelica/Resources/Images/Blocks/InverseModel.png\"
     alt=\"InverseModel.png\">
</html>"),
          experiment(StopTime = 1.0));
      end InverseModel;

      model ShowLogicalSources "Demonstrates the usage of logical sources together with their diagram animation"
        extends Modelica.Icons.Example;
        Sources.BooleanTable table(table = {2, 4, 6, 8}) annotation(
          Placement(transformation(extent = {{-60, -100}, {-40, -80}}, rotation = 0)));
        Sources.BooleanConstant const annotation(
          Placement(transformation(extent = {{-60, 60}, {-40, 80}}, rotation = 0)));
        Sources.BooleanStep step(startTime = 4) annotation(
          Placement(transformation(extent = {{-60, 20}, {-40, 40}}, rotation = 0)));
        Sources.BooleanPulse pulse(period = 1.5) annotation(
          Placement(transformation(extent = {{-60, -20}, {-40, 0}}, rotation = 0)));
        Sources.SampleTrigger sample(period = 0.5) annotation(
          Placement(transformation(extent = {{-60, -60}, {-40, -40}}, rotation = 0)));
        Sources.BooleanExpression booleanExpression(y = pulse.y and step.y) annotation(
          Placement(transformation(extent = {{20, 20}, {80, 40}}, rotation = 0)));
        annotation(
          experiment(StopTime = 10),
          Documentation(info = "<html>
<p>
This simple example demonstrates the logical sources in
<a href=\"modelica://Modelica.Blocks.Sources\">Modelica.Blocks.Sources</a> and demonstrate
their diagram animation (see \"small circle\" close to the output connector).
The \"booleanExpression\" source shows how a logical expression can be defined
in its parameter menu referring to variables available on this level of the
model.
</p>

</html>"));
      end ShowLogicalSources;

      model LogicalNetwork1 "Demonstrates the usage of logical blocks"
        extends Modelica.Icons.Example;
        Sources.BooleanTable table2(table = {1, 3, 5, 7}) annotation(
          Placement(transformation(extent = {{-80, -20}, {-60, 0}}, rotation = 0)));
        Sources.BooleanTable table1(table = {2, 4, 6, 8}) annotation(
          Placement(transformation(extent = {{-80, 20}, {-60, 40}}, rotation = 0)));
        Logical.Not Not1 annotation(
          Placement(transformation(extent = {{-40, -20}, {-20, 0}}, rotation = 0)));
        Logical.And And1 annotation(
          Placement(transformation(extent = {{0, -20}, {20, 0}}, rotation = 0)));
        Logical.Or Or1 annotation(
          Placement(transformation(extent = {{40, 20}, {60, 40}}, rotation = 0)));
        Logical.Pre Pre1 annotation(
          Placement(transformation(extent = {{-40, -60}, {-20, -40}}, rotation = 0)));
      equation
        connect(table2.y, Not1.u) annotation(
          Line(points = {{-59, -10}, {-42, -10}}, color = {255, 0, 255}));
        connect(And1.y, Or1.u2) annotation(
          Line(points = {{21, -10}, {28, -10}, {28, 22}, {38, 22}}, color = {255, 0, 255}));
        connect(table1.y, Or1.u1) annotation(
          Line(points = {{-59, 30}, {38, 30}}, color = {255, 0, 255}));
        connect(Not1.y, And1.u1) annotation(
          Line(points = {{-19, -10}, {-2, -10}}, color = {255, 0, 255}));
        connect(Pre1.y, And1.u2) annotation(
          Line(points = {{-19, -50}, {-10, -50}, {-10, -18}, {-2, -18}}, color = {255, 0, 255}));
        connect(Or1.y, Pre1.u) annotation(
          Line(points = {{61, 30}, {68, 30}, {68, -70}, {-60, -70}, {-60, -50}, {-42, -50}}, color = {255, 0, 255}));
        annotation(
          experiment(StopTime = 10),
          Documentation(info = "<html>
<p>
This example demonstrates a network of logical blocks. Note, that
the Boolean values of the input and output signals are visualized
in the diagram animation, by the small \"circles\" close to the connectors.
If a \"circle\" is \"white\", the signal is <b>false</b>. It a
\"circle\" is \"green\", the signal is <b>true</b>.
</p>
</html>"));
      end LogicalNetwork1;

      model RealNetwork1 "Demonstrates the usage of blocks from Modelica.Blocks.Math"
        extends Modelica.Icons.Example;
        Modelica.Blocks.Math.MultiSum add(nu = 2) annotation(
          Placement(transformation(extent = {{-14, 64}, {-2, 76}})));
        Sources.Sine sine(amplitude = 3, freqHz = 0.1) annotation(
          Placement(transformation(extent = {{-96, 60}, {-76, 80}})));
        Sources.Step integerStep(height = 3, startTime = 2) annotation(
          Placement(transformation(extent = {{-60, 30}, {-40, 50}})));
        Sources.Constant integerConstant(k = 1) annotation(
          Placement(transformation(extent = {{-60, -10}, {-40, 10}})));
        Modelica.Blocks.Interaction.Show.RealValue showValue annotation(
          Placement(transformation(extent = {{66, 60}, {86, 80}})));
        Math.MultiProduct product(nu = 2) annotation(
          Placement(transformation(extent = {{6, 24}, {18, 36}})));
        Modelica.Blocks.Interaction.Show.RealValue showValue1(significantDigits = 2) annotation(
          Placement(transformation(extent = {{64, 20}, {84, 40}})));
        Sources.BooleanPulse booleanPulse1(period = 1) annotation(
          Placement(transformation(extent = {{-12, -30}, {8, -10}})));
        Math.MultiSwitch multiSwitch(nu = 2, expr = {4, 6}, y_default = 2) annotation(
          Placement(transformation(extent = {{28, -60}, {68, -40}})));
        Sources.BooleanPulse booleanPulse2(period = 2, width = 80) annotation(
          Placement(transformation(extent = {{-12, -70}, {8, -50}})));
        Modelica.Blocks.Interaction.Show.RealValue showValue3(use_numberPort = false, number = multiSwitch.y, significantDigits = 1) annotation(
          Placement(transformation(extent = {{40, -84}, {60, -64}})));
        Math.LinearDependency linearDependency1(y0 = 1, k1 = 2, k2 = 3) annotation(
          Placement(transformation(extent = {{40, 80}, {60, 100}})));
        Math.MinMax minMax(nu = 2) annotation(
          Placement(transformation(extent = {{58, -16}, {78, 4}})));
      equation
        connect(booleanPulse1.y, multiSwitch.u[1]) annotation(
          Line(points = {{9, -20}, {18, -20}, {18, -48}, {28, -48}, {28, -48.5}}, color = {255, 0, 255}, smooth = Smooth.None));
        connect(booleanPulse2.y, multiSwitch.u[2]) annotation(
          Line(points = {{9, -60}, {18, -60}, {18, -52}, {28, -52}, {28, -51.5}}, color = {255, 0, 255}, smooth = Smooth.None));
        connect(sine.y, add.u[1]) annotation(
          Line(points = {{-75, 70}, {-46.5, 70}, {-46.5, 72.1}, {-14, 72.1}}, color = {0, 0, 127}, smooth = Smooth.None));
        connect(integerStep.y, add.u[2]) annotation(
          Line(points = {{-39, 40}, {-28, 40}, {-28, 67.9}, {-14, 67.9}}, color = {0, 0, 127}, smooth = Smooth.None));
        connect(add.y, showValue.numberPort) annotation(
          Line(points = {{-0.98, 70}, {64.5, 70}}, color = {0, 0, 127}, smooth = Smooth.None));
        connect(integerStep.y, product.u[1]) annotation(
          Line(points = {{-39, 40}, {-20, 40}, {-20, 32.1}, {6, 32.1}}, color = {0, 0, 127}, smooth = Smooth.None));
        connect(integerConstant.y, product.u[2]) annotation(
          Line(points = {{-39, 0}, {-20, 0}, {-20, 27.9}, {6, 27.9}}, color = {0, 0, 127}, smooth = Smooth.None));
        connect(product.y, showValue1.numberPort) annotation(
          Line(points = {{19.02, 30}, {62.5, 30}}, color = {0, 0, 127}, smooth = Smooth.None));
        connect(add.y, linearDependency1.u1) annotation(
          Line(points = {{-0.98, 70}, {20, 70}, {20, 96}, {38, 96}}, color = {0, 0, 127}, smooth = Smooth.None));
        connect(product.y, linearDependency1.u2) annotation(
          Line(points = {{19.02, 30}, {30, 30}, {30, 84}, {38, 84}}, color = {0, 0, 127}, smooth = Smooth.None));
        connect(add.y, minMax.u[1]) annotation(
          Line(points = {{-0.98, 70}, {48, 70}, {48, -2.5}, {58, -2.5}}, color = {0, 0, 127}, smooth = Smooth.None));
        connect(product.y, minMax.u[2]) annotation(
          Line(points = {{19.02, 30}, {40, 30}, {40, -9.5}, {58, -9.5}}, color = {0, 0, 127}, smooth = Smooth.None));
        annotation(
          experiment(StopTime = 10),
          Documentation(info = "<html>
<p>
This example demonstrates a network of mathematical Real blocks.
from package <a href=\"modelica://Modelica.Blocks.Math\">Modelica.Blocks.Math</a>.
Note, that
</p>

<ul>
<li> at the right side of the model, several Math.ShowValue blocks
     are present, that visualize the actual value of the respective Real
     signal in a diagram animation.</li>

<li> the Boolean values of the input and output signals are visualized
     in the diagram animation, by the small \"circles\" close to the connectors.
     If a \"circle\" is \"white\", the signal is <b>false</b>. If a
     \"circle\" is \"green\", the signal is <b>true</b>.</li>
</ul>

</html>"));
      end RealNetwork1;

      model IntegerNetwork1 "Demonstrates the usage of blocks from Modelica.Blocks.MathInteger"
        extends Modelica.Icons.Example;
        MathInteger.Sum sum(nu = 3) annotation(
          Placement(transformation(extent = {{-14, 64}, {-2, 76}})));
        Sources.Sine sine(amplitude = 3, freqHz = 0.1) annotation(
          Placement(transformation(extent = {{-100, 60}, {-80, 80}})));
        Math.RealToInteger realToInteger annotation(
          Placement(transformation(extent = {{-60, 60}, {-40, 80}})));
        Sources.IntegerStep integerStep(height = 3, startTime = 2) annotation(
          Placement(transformation(extent = {{-60, 30}, {-40, 50}})));
        Sources.IntegerConstant integerConstant(k = 1) annotation(
          Placement(transformation(extent = {{-60, -10}, {-40, 10}})));
        Modelica.Blocks.Interaction.Show.IntegerValue showValue annotation(
          Placement(transformation(extent = {{40, 60}, {60, 80}})));
        MathInteger.Product product(nu = 2) annotation(
          Placement(transformation(extent = {{16, 24}, {28, 36}})));
        Modelica.Blocks.Interaction.Show.IntegerValue showValue1 annotation(
          Placement(transformation(extent = {{40, 20}, {60, 40}})));
        MathInteger.TriggeredAdd triggeredAdd(use_reset = false, use_set = false) annotation(
          Placement(transformation(extent = {{16, -6}, {28, 6}})));
        Sources.BooleanPulse booleanPulse1(period = 1) annotation(
          Placement(transformation(extent = {{-12, -30}, {8, -10}})));
        Modelica.Blocks.Interaction.Show.IntegerValue showValue2 annotation(
          Placement(transformation(extent = {{40, -10}, {60, 10}})));
        MathInteger.MultiSwitch multiSwitch1(nu = 2, expr = {4, 6}, y_default = 2, use_pre_as_default = false) annotation(
          Placement(transformation(extent = {{28, -60}, {68, -40}})));
        Sources.BooleanPulse booleanPulse2(period = 2, width = 80) annotation(
          Placement(transformation(extent = {{-12, -70}, {8, -50}})));
        Modelica.Blocks.Interaction.Show.IntegerValue showValue3(use_numberPort = false, number = multiSwitch1.y) annotation(
          Placement(transformation(extent = {{40, -84}, {60, -64}})));
      equation
        connect(sine.y, realToInteger.u) annotation(
          Line(points = {{-79, 70}, {-62, 70}}, color = {0, 0, 127}, smooth = Smooth.None));
        connect(realToInteger.y, sum.u[1]) annotation(
          Line(points = {{-39, 70}, {-32, 70}, {-32, 72}, {-14, 72}, {-14, 72.8}}, color = {255, 127, 0}, smooth = Smooth.None));
        connect(integerStep.y, sum.u[2]) annotation(
          Line(points = {{-39, 40}, {-28, 40}, {-28, 70}, {-14, 70}}, color = {255, 127, 0}, smooth = Smooth.None));
        connect(integerConstant.y, sum.u[3]) annotation(
          Line(points = {{-39, 0}, {-22, 0}, {-22, 67.2}, {-14, 67.2}}, color = {255, 127, 0}, smooth = Smooth.None));
        connect(sum.y, showValue.numberPort) annotation(
          Line(points = {{-1.1, 70}, {38.5, 70}}, color = {255, 127, 0}, smooth = Smooth.None));
        connect(sum.y, product.u[1]) annotation(
          Line(points = {{-1.1, 70}, {4, 70}, {4, 32.1}, {16, 32.1}}, color = {255, 127, 0}, smooth = Smooth.None));
        connect(integerStep.y, product.u[2]) annotation(
          Line(points = {{-39, 40}, {-8, 40}, {-8, 27.9}, {16, 27.9}}, color = {255, 127, 0}, smooth = Smooth.None));
        connect(product.y, showValue1.numberPort) annotation(
          Line(points = {{28.9, 30}, {38.5, 30}}, color = {255, 127, 0}, smooth = Smooth.None));
        connect(integerConstant.y, triggeredAdd.u) annotation(
          Line(points = {{-39, 0}, {13.6, 0}}, color = {255, 127, 0}, smooth = Smooth.None));
        connect(booleanPulse1.y, triggeredAdd.trigger) annotation(
          Line(points = {{9, -20}, {18.4, -20}, {18.4, -7.2}}, color = {255, 0, 255}, smooth = Smooth.None));
        connect(triggeredAdd.y, showValue2.numberPort) annotation(
          Line(points = {{29.2, 0}, {38.5, 0}}, color = {255, 127, 0}, smooth = Smooth.None));
        connect(booleanPulse1.y, multiSwitch1.u[1]) annotation(
          Line(points = {{9, -20}, {18, -20}, {18, -48}, {28, -48}, {28, -48.5}}, color = {255, 0, 255}, smooth = Smooth.None));
        connect(booleanPulse2.y, multiSwitch1.u[2]) annotation(
          Line(points = {{9, -60}, {18, -60}, {18, -52}, {28, -52}, {28, -51.5}}, color = {255, 0, 255}, smooth = Smooth.None));
        annotation(
          experiment(StopTime = 10),
          Documentation(info = "<html>
<p>
This example demonstrates a network of Integer blocks.
from package <a href=\"modelica://Modelica.Blocks.MathInteger\">Modelica.Blocks.MathInteger</a>.
Note, that
</p>

<ul>
<li> at the right side of the model, several MathInteger.ShowValue blocks
     are present, that visualize the actual value of the respective Integer
     signal in a diagram animation.</li>

<li> the Boolean values of the input and output signals are visualized
     in the diagram animation, by the small \"circles\" close to the connectors.
     If a \"circle\" is \"white\", the signal is <b>false</b>. If a
     \"circle\" is \"green\", the signal is <b>true</b>.</li>

</ul>

</html>"));
      end IntegerNetwork1;

      model BooleanNetwork1 "Demonstrates the usage of blocks from Modelica.Blocks.MathBoolean"
        extends Modelica.Icons.Example;
        Modelica.Blocks.Interaction.Show.BooleanValue showValue annotation(
          Placement(transformation(extent = {{-36, 74}, {-16, 94}})));
        MathBoolean.And and1(nu = 3) annotation(
          Placement(transformation(extent = {{-58, 64}, {-46, 76}})));
        Sources.BooleanPulse booleanPulse1(width = 20, period = 1) annotation(
          Placement(transformation(extent = {{-100, 60}, {-80, 80}})));
        Sources.BooleanPulse booleanPulse2(period = 1, width = 80) annotation(
          Placement(transformation(extent = {{-100, -4}, {-80, 16}})));
        Sources.BooleanStep booleanStep(startTime = 1.5) annotation(
          Placement(transformation(extent = {{-100, 28}, {-80, 48}})));
        MathBoolean.Or or1(nu = 2) annotation(
          Placement(transformation(extent = {{-28, 62}, {-16, 74}})));
        MathBoolean.Xor xor1(nu = 2) annotation(
          Placement(transformation(extent = {{-2, 60}, {10, 72}})));
        Modelica.Blocks.Interaction.Show.BooleanValue showValue2 annotation(
          Placement(transformation(extent = {{-2, 74}, {18, 94}})));
        Modelica.Blocks.Interaction.Show.BooleanValue showValue3 annotation(
          Placement(transformation(extent = {{24, 56}, {44, 76}})));
        MathBoolean.Nand nand1(nu = 2) annotation(
          Placement(transformation(extent = {{22, 40}, {34, 52}})));
        MathBoolean.Nor or2(nu = 2) annotation(
          Placement(transformation(extent = {{46, 38}, {58, 50}})));
        Modelica.Blocks.Interaction.Show.BooleanValue showValue4 annotation(
          Placement(transformation(extent = {{90, 34}, {110, 54}})));
        MathBoolean.Not nor1 annotation(
          Placement(transformation(extent = {{68, 40}, {76, 48}})));
        MathBoolean.OnDelay onDelay(delayTime = 1) annotation(
          Placement(transformation(extent = {{-56, -94}, {-48, -86}})));
        MathBoolean.RisingEdge rising annotation(
          Placement(transformation(extent = {{-56, -15}, {-48, -7}})));
        MathBoolean.MultiSwitch set1(nu = 2, expr = {false, true}) annotation(
          Placement(transformation(extent = {{-30, -23}, {10, -3}})));
        MathBoolean.FallingEdge falling annotation(
          Placement(transformation(extent = {{-56, -32}, {-48, -24}})));
        Sources.BooleanTable booleanTable(table = {2, 4, 6, 6.5, 7, 9, 11}) annotation(
          Placement(transformation(extent = {{-100, -100}, {-80, -80}})));
        MathBoolean.ChangingEdge changing annotation(
          Placement(transformation(extent = {{-56, -59}, {-48, -51}})));
        MathInteger.TriggeredAdd triggeredAdd annotation(
          Placement(transformation(extent = {{14, -56}, {26, -44}})));
        Sources.IntegerConstant integerConstant(k = 2) annotation(
          Placement(transformation(extent = {{-20, -60}, {0, -40}})));
        Modelica.Blocks.Interaction.Show.IntegerValue showValue1 annotation(
          Placement(transformation(extent = {{40, -60}, {60, -40}})));
        Modelica.Blocks.Interaction.Show.BooleanValue showValue5 annotation(
          Placement(transformation(extent = {{24, -23}, {44, -3}})));
        Modelica.Blocks.Interaction.Show.BooleanValue showValue6 annotation(
          Placement(transformation(extent = {{-32, -100}, {-12, -80}})));
        Logical.RSFlipFlop rSFlipFlop annotation(
          Placement(transformation(extent = {{70, -90}, {90, -70}})));
        Sources.SampleTrigger sampleTriggerSet(period = 0.5, startTime = 0) annotation(
          Placement(transformation(extent = {{40, -76}, {54, -62}})));
        Sources.SampleTrigger sampleTriggerReset(period = 0.5, startTime = 0.3) annotation(
          Placement(transformation(extent = {{40, -98}, {54, -84}})));
      equation
        connect(booleanPulse1.y, and1.u[1]) annotation(
          Line(points = {{-79, 70}, {-68, 70}, {-68, 72.8}, {-58, 72.8}}, color = {255, 0, 255}, smooth = Smooth.None));
        connect(booleanStep.y, and1.u[2]) annotation(
          Line(points = {{-79, 38}, {-64, 38}, {-64, 70}, {-58, 70}}, color = {255, 0, 255}, smooth = Smooth.None));
        connect(booleanPulse2.y, and1.u[3]) annotation(
          Line(points = {{-79, 6}, {-62, 6}, {-62, 67.2}, {-58, 67.2}}, color = {255, 0, 255}, smooth = Smooth.None));
        connect(and1.y, or1.u[1]) annotation(
          Line(points = {{-45.1, 70}, {-36.4, 70}, {-36.4, 70.1}, {-28, 70.1}}, color = {255, 0, 255}, smooth = Smooth.None));
        connect(booleanPulse2.y, or1.u[2]) annotation(
          Line(points = {{-79, 6}, {-40, 6}, {-40, 65.9}, {-28, 65.9}}, color = {255, 0, 255}, smooth = Smooth.None));
        connect(or1.y, xor1.u[1]) annotation(
          Line(points = {{-15.1, 68}, {-8, 68}, {-8, 68.1}, {-2, 68.1}}, color = {255, 0, 255}, smooth = Smooth.None));
        connect(booleanPulse2.y, xor1.u[2]) annotation(
          Line(points = {{-79, 6}, {-12, 6}, {-12, 63.9}, {-2, 63.9}}, color = {255, 0, 255}, smooth = Smooth.None));
        connect(and1.y, showValue.activePort) annotation(
          Line(points = {{-45.1, 70}, {-42, 70}, {-42, 84}, {-37.5, 84}}, color = {255, 0, 255}, smooth = Smooth.None));
        connect(or1.y, showValue2.activePort) annotation(
          Line(points = {{-15.1, 68}, {-12, 68}, {-12, 84}, {-3.5, 84}}, color = {255, 0, 255}, smooth = Smooth.None));
        connect(xor1.y, showValue3.activePort) annotation(
          Line(points = {{10.9, 66}, {22.5, 66}}, color = {255, 0, 255}, smooth = Smooth.None));
        connect(xor1.y, nand1.u[1]) annotation(
          Line(points = {{10.9, 66}, {16, 66}, {16, 48.1}, {22, 48.1}}, color = {255, 0, 255}, smooth = Smooth.None));
        connect(booleanPulse2.y, nand1.u[2]) annotation(
          Line(points = {{-79, 6}, {16, 6}, {16, 44}, {22, 44}, {22, 43.9}}, color = {255, 0, 255}, smooth = Smooth.None));
        connect(nand1.y, or2.u[1]) annotation(
          Line(points = {{34.9, 46}, {46, 46}, {46, 46.1}}, color = {255, 0, 255}, smooth = Smooth.None));
        connect(booleanPulse2.y, or2.u[2]) annotation(
          Line(points = {{-79, 6}, {42, 6}, {42, 41.9}, {46, 41.9}}, color = {255, 0, 255}, smooth = Smooth.None));
        connect(or2.y, nor1.u) annotation(
          Line(points = {{58.9, 44}, {66.4, 44}}, color = {255, 0, 255}, smooth = Smooth.None));
        connect(nor1.y, showValue4.activePort) annotation(
          Line(points = {{76.8, 44}, {88.5, 44}}, color = {255, 0, 255}, smooth = Smooth.None));
        connect(booleanPulse2.y, rising.u) annotation(
          Line(points = {{-79, 6}, {-62, 6}, {-62, -11}, {-57.6, -11}}, color = {255, 0, 255}, smooth = Smooth.None));
        connect(rising.y, set1.u[1]) annotation(
          Line(points = {{-47.2, -11}, {-38.6, -11}, {-38.6, -11.5}, {-30, -11.5}}, color = {255, 0, 255}, smooth = Smooth.None));
        connect(falling.y, set1.u[2]) annotation(
          Line(points = {{-47.2, -28}, {-40, -28}, {-40, -14.5}, {-30, -14.5}}, color = {255, 0, 255}, smooth = Smooth.None));
        connect(booleanPulse2.y, falling.u) annotation(
          Line(points = {{-79, 6}, {-62, 6}, {-62, -28}, {-57.6, -28}}, color = {255, 0, 255}, smooth = Smooth.None));
        connect(booleanTable.y, onDelay.u) annotation(
          Line(points = {{-79, -90}, {-57.6, -90}}, color = {255, 0, 255}, smooth = Smooth.None));
        connect(booleanPulse2.y, changing.u) annotation(
          Line(points = {{-79, 6}, {-62, 6}, {-62, -55}, {-57.6, -55}}, color = {255, 0, 255}, smooth = Smooth.None));
        connect(integerConstant.y, triggeredAdd.u) annotation(
          Line(points = {{1, -50}, {11.6, -50}}, color = {255, 127, 0}, smooth = Smooth.None));
        connect(changing.y, triggeredAdd.trigger) annotation(
          Line(points = {{-47.2, -55}, {-30, -55}, {-30, -74}, {16.4, -74}, {16.4, -57.2}}, color = {255, 0, 255}, smooth = Smooth.None));
        connect(triggeredAdd.y, showValue1.numberPort) annotation(
          Line(points = {{27.2, -50}, {38.5, -50}}, color = {255, 127, 0}, smooth = Smooth.None));
        connect(set1.y, showValue5.activePort) annotation(
          Line(points = {{11, -13}, {22.5, -13}}, color = {255, 0, 255}, smooth = Smooth.None));
        connect(onDelay.y, showValue6.activePort) annotation(
          Line(points = {{-47.2, -90}, {-33.5, -90}}, color = {255, 0, 255}, smooth = Smooth.None));
        connect(sampleTriggerSet.y, rSFlipFlop.S) annotation(
          Line(points = {{54.7, -69}, {60, -69}, {60, -74}, {68, -74}}, color = {255, 0, 255}, smooth = Smooth.None));
        connect(sampleTriggerReset.y, rSFlipFlop.R) annotation(
          Line(points = {{54.7, -91}, {60, -91}, {60, -86}, {68, -86}}, color = {255, 0, 255}, smooth = Smooth.None));
        annotation(
          experiment(StopTime = 10),
          Documentation(info = "<html>
<p>
This example demonstrates a network of Boolean blocks
from package <a href=\"modelica://Modelica.Blocks.MathBoolean\">Modelica.Blocks.MathBoolean</a>.
Note, that
</p>

<ul>
<li> at the right side of the model, several MathBoolean.ShowValue blocks
     are present, that visualize the actual value of the respective Boolean
     signal in a diagram animation (\"green\" means \"true\").</li>

<li> the Boolean values of the input and output signals are visualized
     in the diagram animation, by the small \"circles\" close to the connectors.
     If a \"circle\" is \"white\", the signal is <b>false</b>. If a
     \"circle\" is \"green\", the signal is <b>true</b>.</li>

</ul>

</html>"));
      end BooleanNetwork1;

      model Interaction1 "Demonstrates the usage of blocks from Modelica.Blocks.Interaction.Show"
        extends Modelica.Icons.Example;
        Interaction.Show.IntegerValue integerValue annotation(
          Placement(transformation(extent = {{-40, 20}, {-20, 40}})));
        Sources.IntegerTable integerTable(table = [0, 0; 1, 2; 2, 4; 3, 6; 4, 4; 6, 2]) annotation(
          Placement(transformation(extent = {{-80, 20}, {-60, 40}})));
        Sources.TimeTable timeTable(table = [0, 0; 1, 2.1; 2, 4.2; 3, 6.3; 4, 4.2; 6, 2.1; 6, 2.1]) annotation(
          Placement(transformation(extent = {{-80, 60}, {-60, 80}})));
        Interaction.Show.RealValue realValue annotation(
          Placement(transformation(extent = {{-40, 60}, {-20, 80}})));
        Sources.BooleanTable booleanTable(table = {1, 2, 3, 4, 5, 6, 7, 8, 9}) annotation(
          Placement(transformation(extent = {{-80, -20}, {-60, 0}})));
        Interaction.Show.BooleanValue booleanValue annotation(
          Placement(transformation(extent = {{-40, -20}, {-20, 0}})));
        Sources.RadioButtonSource start(buttonTimeTable = {1, 3}, reset = {stop.on}) annotation(
          Placement(transformation(extent = {{24, 64}, {36, 76}})));
        Sources.RadioButtonSource stop(buttonTimeTable = {2, 4}, reset = {start.on}) annotation(
          Placement(transformation(extent = {{48, 64}, {60, 76}})));
      equation
        connect(integerTable.y, integerValue.numberPort) annotation(
          Line(points = {{-59, 30}, {-41.5, 30}}, color = {255, 127, 0}, smooth = Smooth.None));
        connect(timeTable.y, realValue.numberPort) annotation(
          Line(points = {{-59, 70}, {-41.5, 70}}, color = {0, 0, 127}, smooth = Smooth.None));
        connect(booleanTable.y, booleanValue.activePort) annotation(
          Line(points = {{-59, -10}, {-41.5, -10}}, color = {255, 0, 255}, smooth = Smooth.None));
        annotation(
          experiment(StopTime = 10),
          Documentation(info = "<html>
<p>
This example demonstrates a network of blocks
from package <a href=\"modelica://Modelica.Blocks.Interaction\">Modelica.Blocks.Interaction</a>
to show how diagram animations can be constructed.
</p>

</html>"));
      end Interaction1;

      model BusUsage "Demonstrates the usage of a signal bus"
        extends Modelica.Icons.Example;
      public
        Modelica.Blocks.Sources.IntegerStep integerStep(height = 1, offset = 2, startTime = 0.5) annotation(
          Placement(transformation(extent = {{-60, -40}, {-40, -20}}, rotation = 0)));
        Modelica.Blocks.Sources.BooleanStep booleanStep(startTime = 0.5) annotation(
          Placement(transformation(extent = {{-58, 0}, {-38, 20}}, rotation = 0)));
        Modelica.Blocks.Sources.Sine sine(freqHz = 1) annotation(
          Placement(transformation(extent = {{-60, 40}, {-40, 60}}, rotation = 0)));
        Modelica.Blocks.Examples.BusUsage_Utilities.Part part annotation(
          Placement(transformation(extent = {{-60, -80}, {-40, -60}}, rotation = 0)));
        Modelica.Blocks.Math.Gain gain(k = 1) annotation(
          Placement(transformation(extent = {{-40, 70}, {-60, 90}}, rotation = 0)));
      protected
        BusUsage_Utilities.Interfaces.ControlBus controlBus annotation(
          Placement(transformation(origin = {30, 10}, extent = {{-20, 20}, {20, -20}}, rotation = 90)));
      equation
        connect(sine.y, controlBus.realSignal1) annotation(
          Line(points = {{-39, 50}, {12, 50}, {12, 14}, {30, 14}, {30, 10}}, color = {0, 0, 127}, smooth = Smooth.None));
        connect(booleanStep.y, controlBus.booleanSignal) annotation(
          Line(points = {{-37, 10}, {30, 10}}, color = {255, 0, 255}, smooth = Smooth.None));
        connect(integerStep.y, controlBus.integerSignal) annotation(
          Line(points = {{-39, -30}, {0, -30}, {0, 6}, {32, 6}, {32, 10}, {30, 10}}, color = {255, 127, 0}, smooth = Smooth.None));
        connect(part.subControlBus, controlBus.subControlBus) annotation(
          Line(points = {{-40, -70}, {30, -70}, {30, 10}}, color = {255, 204, 51}, thickness = 0.5, smooth = Smooth.None));
        connect(gain.u, controlBus.realSignal1) annotation(
          Line(points = {{-38, 80}, {20, 80}, {20, 18}, {32, 18}, {32, 10}, {30, 10}}, color = {0, 0, 127}, smooth = Smooth.None));
        annotation(
          Documentation(info = "<html>
<p><b>Signal bus concept</b></p>
<p>
In technical systems, such as vehicles, robots or satellites, many signals
are exchanged between components. In a simulation system, these signals
are usually modelled by signal connections of input/output blocks.
Unfortunately, the signal connection structure may become very complicated,
especially for hierarchical models.
</p>

<p>
The same is also true for real technical systems. To reduce complexity
and get higher flexibility, many technical systems use data buses to
exchange data between components. For the same reasons, it is often better
to use a \"signal bus\" concept also in a Modelica model. This is demonstrated
at hand of this model (Modelica.Blocks.Examples.BusUsage):
</p>

<img src=\"modelica://Modelica/Resources/Images/Blocks/BusUsage.png\"
     alt=\"BusUsage.png\">

<ul>
<li> Connector instance \"controlBus\" is a hierarchical connector that is
     used to exchange signals between different components. It is
     defined as \"expandable connector\" in order that <b>no</b> central definition
     of the connector is needed but is automatically constructed by the
     signals connected to it (see also Modelica specification 2.2.1).</li>
<li> Input/output signals can be directly connected to the \"controlBus\".</li>
<li> A component, such as \"part\", can be directly connected to the \"controlBus\",
     provided it has also a bus connector, or the \"part\" connector is a
     sub-connector contained in the \"controlBus\". </li>
</ul>

<p>
The control and sub-control bus icons are provided within Modelica.Icons.
In <a href=\"modelica://Modelica.Blocks.Examples.BusUsage_Utilities.Interfaces\">Modelica.Blocks.Examples.BusUsage_Utilities.Interfaces</a>
the buses for this example are defined. Both the \"ControlBus\" and the \"SubControlBus\" are
<b>expandable</b> connectors that do not define any variable. For example,
<a href=\"modelica://Modelica.Blocks.Examples.BusUsage_Utilities.Interfaces.ControlBus#text\">Interfaces.ControlBus</a>
is defined as:
</p>
<pre>  <b>expandable connector</b> ControlBus
      <b>extends</b> Modelica.Icons.ControlBus;
      <b>annotation</b> ();
  <b>end</b> ControlBus;
</pre>
<p>
Note, the \"annotation\" in the connector is important since the color
and thickness of a connector line are taken from the first
line element in the icon annotation of a connector class. Above, a small rectangle in the
color of the bus is defined (and therefore this rectangle is not
visible). As a result, when connecting from an instance of this
connector to another connector instance, the connecting line has
the color of the \"ControlBus\" with double width (due to \"thickness=0.5\").
</p>

<p>
An <b>expandable</b> connector is a connector where the content of the connector
is constructed by the variables connected to instances of this connector.
For example, if \"sine.y\" is connected to the \"controlBus\", the following
menu pops-up in Dymola:
</p>

<img src=\"modelica://Modelica/Resources/Images/Blocks/BusUsage2.png\"
     alt=\"BusUsage2.png\">

<p>
The \"Add variable/New name\" field allows the user to define the name of the signal on
the \"controlBus\". When typing \"realSignal1\" as \"New name\", a connection of the form:
</p>

<pre>     <b>connect</b>(sine.y, controlBus.realSignal1)
</pre>

<p>
is generated and the \"controlBus\" contains the new signal \"realSignal1\". Modelica tools
may give more support in order to list potential signals for a connection.
For example, in Dymola all variables are listed in the menu that are contained in
connectors which are derived by inheritance from \"controlBus\". Therefore, in
<a href=\"modelica://Modelica.Blocks.Examples.BusUsage_Utilities.Interfaces\">BusUsage_Utilities.Interfaces</a>
the expected implementation of the \"ControlBus\" and of the \"SubControlBus\" are given.
For example \"Internal.ControlBus\" is defined as:
</p>

<pre>  <b>expandable connector</b> StandardControlBus
    <b>extends</b> BusUsage_Utilities.Interfaces.ControlBus;

    <b>import</b> SI = Modelica.SIunits;
    SI.AngularVelocity    realSignal1   \"First Real signal\";
    SI.Velocity           realSignal2   \"Second Real signal\";
    Integer               integerSignal \"Integer signal\";
    Boolean               booleanSignal \"Boolean signal\";
    StandardSubControlBus subControlBus \"Combined signal\";
  <b>end</b> StandardControlBus;
</pre>

<p>
Consequently, when connecting now from \"sine.y\" to \"controlBus\", the menu
looks differently:
</p>

<img src=\"modelica://Modelica/Resources/Images/Blocks/BusUsage3.png\"
     alt=\"BusUsage3.png\">

<p>
Note, even if the signals from \"Internal.StandardControlBus\" are listed, these are
just potential signals. The user might still add different signal names.
</p>

</html>"),
          experiment(StopTime = 2));
      end BusUsage;

      package BusUsage_Utilities "Utility models and connectors for example Modelica.Blocks.Examples.BusUsage"
        extends Modelica.Icons.UtilitiesPackage;

        package Interfaces "Interfaces specialised for this example"
          extends Modelica.Icons.InterfacesPackage;

          expandable connector ControlBus "Control bus that is adapted to the signals connected to it"
            extends Modelica.Icons.SignalBus;
            import SI = Modelica.SIunits;
            SI.AngularVelocity realSignal1 "First Real signal (angular velocity)" annotation(
              HideResult = false);
            SI.Velocity realSignal2 "Second Real signal" annotation(
              HideResult = false);
            Integer integerSignal "Integer signal" annotation(
              HideResult = false);
            Boolean booleanSignal "Boolean signal" annotation(
              HideResult = false);
            SubControlBus subControlBus "Combined signal" annotation(
              HideResult = false);
            annotation(
              Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-20, 2}, {22, -2}}, lineColor = {255, 204, 51}, lineThickness = 0.5)}),
              Documentation(info = "<html>
<p>
This connector defines the \"expandable connector\" ControlBus that
is used as bus in the
<a href=\"modelica://Modelica.Blocks.Examples.BusUsage\">BusUsage</a> example.
Note, this connector contains \"default\" signals that might be utilized
in a connection (the input/output causalities of the signals
are determined from the connections to this bus).
</p>
</html>"));
          end ControlBus;

          expandable connector SubControlBus "Sub-control bus that is adapted to the signals connected to it"
            extends Modelica.Icons.SignalSubBus;
            Real myRealSignal annotation(
              HideResult = false);
            Boolean myBooleanSignal annotation(
              HideResult = false);
            annotation(
              defaultComponentPrefixes = "protected",
              Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-20, 2}, {22, -2}}, lineColor = {255, 204, 51}, lineThickness = 0.5)}),
              Documentation(info = "<html>
<p>
This connector defines the \"expandable connector\" SubControlBus that
is used as sub-bus in the
<a href=\"modelica://Modelica.Blocks.Examples.BusUsage\">BusUsage</a> example.
Note, this is an expandable connector which has a \"default\" set of
signals (the input/output causalities of the signals are
determined from the connections to this bus).
</p>
</html>"));
          end SubControlBus;
          annotation(
            Documentation(info = "<html>
<p>
This package contains the bus definitions needed for the
<a href=\"modelica://Modelica.Blocks.Examples.BusUsage\">BusUsage</a> example.
</p>
</html>"));
        end Interfaces;

        model Part "Component with sub-control bus"
          Interfaces.SubControlBus subControlBus annotation(
            Placement(transformation(origin = {100, 0}, extent = {{-20, -20}, {20, 20}}, rotation = 270)));
          Sources.RealExpression realExpression(y = time) annotation(
            Placement(transformation(extent = {{-6, 0}, {20, 20}}, rotation = 0)));
          Sources.BooleanExpression booleanExpression(y = time > 0.5) annotation(
            Placement(transformation(extent = {{-6, -30}, {20, -10}}, rotation = 0)));
        equation
          connect(realExpression.y, subControlBus.myRealSignal) annotation(
            Line(points = {{21.3, 10}, {88, 10}, {88, 6}, {98, 6}, {98, 0}, {100, 0}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(booleanExpression.y, subControlBus.myBooleanSignal) annotation(
            Line(points = {{21.3, -20}, {60, -20}, {60, 0}, {100, 0}}, color = {255, 0, 255}, smooth = Smooth.None));
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, 60}, {100, -60}}, fillColor = {159, 159, 223}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 127}), Text(extent = {{-106, 124}, {114, 68}}, textString = "%name", lineColor = {0, 0, 255})}),
            Documentation(info = "<html>
<p>
This model is used to demonstrate the bus usage in example
<a href=\"modelica://Modelica.Blocks.Examples.BusUsage\">BusUsage</a>.
</p>
</html>"));
        end Part;
        annotation(
          Documentation(info = "<html>
<p>
This package contains utility models and bus definitions needed for the
<a href=\"modelica://Modelica.Blocks.Examples.BusUsage\">BusUsage</a> example.
</p>
</html>"));
      end BusUsage_Utilities;
      annotation(
        Documentation(info = "<html>
<p>
This package contains example models to demonstrate the
usage of package blocks.
</p>
</html>"));
    end Examples;

    package Continuous "Library of continuous control blocks with internal states"
      import Modelica.Blocks.Interfaces;
      import Modelica.SIunits;
      extends Modelica.Icons.Package;

      block Integrator "Output the integral of the input signal"
        import Modelica.Blocks.Types.Init;
        parameter Real k(unit = "1") = 1 "Integrator gain";
        /* InitialState is the default, because it was the default in Modelica 2.2
             and therefore this setting is backward compatible
          */
        parameter Modelica.Blocks.Types.Init initType = Modelica.Blocks.Types.Init.InitialState "Type of initialization (1: no init, 2: steady state, 3,4: initial output)" annotation(
          Evaluate = true,
          Dialog(group = "Initialization"));
        parameter Real y_start = 0 "Initial or guess value of output (= state)" annotation(
          Dialog(group = "Initialization"));
        extends Interfaces.SISO(y(start = y_start));
      initial equation
        if initType == Init.SteadyState then
          der(y) = 0;
        elseif initType == Init.InitialState or initType == Init.InitialOutput then
          y = y_start;
        end if;
      equation
        der(y) = k * u;
        annotation(
          Documentation(info = "<html>
<p>
This blocks computes output <b>y</b> (element-wise) as
<i>integral</i> of the input <b>u</b> multiplied with
the gain <i>k</i>:
</p>
<pre>
         k
     y = - u
         s
</pre>

<p>
It might be difficult to initialize the integrator in steady state.
This is discussed in the description of package
<a href=\"modelica://Modelica.Blocks.Continuous#info\">Continuous</a>.
</p>

</html>"),
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100.0, -100.0}, {100.0, 100.0}}, initialScale = 0.1), graphics = {Line(visible = true, points = {{-80.0, 78.0}, {-80.0, -90.0}}, color = {192, 192, 192}), Polygon(visible = true, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid, points = {{-80.0, 90.0}, {-88.0, 68.0}, {-72.0, 68.0}, {-80.0, 90.0}}), Line(visible = true, points = {{-90.0, -80.0}, {82.0, -80.0}}, color = {192, 192, 192}), Polygon(visible = true, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid, points = {{90.0, -80.0}, {68.0, -72.0}, {68.0, -88.0}, {90.0, -80.0}}), Text(visible = true, lineColor = {192, 192, 192}, extent = {{0.0, -70.0}, {60.0, -10.0}}, textString = "I"), Text(visible = true, extent = {{-150.0, -150.0}, {150.0, -110.0}}, textString = "k=%k"), Line(visible = true, points = {{-80.0, -80.0}, {80.0, 80.0}}, color = {0, 0, 127})}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-60, 60}, {60, -60}}, lineColor = {0, 0, 255}), Line(points = {{-100, 0}, {-60, 0}}, color = {0, 0, 255}), Line(points = {{60, 0}, {100, 0}}, color = {0, 0, 255}), Text(extent = {{-36, 60}, {32, 2}}, lineColor = {0, 0, 0}, textString = "k"), Text(extent = {{-32, 0}, {36, -58}}, lineColor = {0, 0, 0}, textString = "s"), Line(points = {{-46, 0}, {46, 0}}, color = {0, 0, 0})}));
      end Integrator;

      block LimIntegrator "Integrator with limited value of the output"
        import Modelica.Blocks.Types.Init;
        parameter Real k(unit = "1") = 1 "Integrator gain";
        parameter Real outMax(start = 1) "Upper limit of output";
        parameter Real outMin = -outMax "Lower limit of output";
        parameter Modelica.Blocks.Types.Init initType = Modelica.Blocks.Types.Init.InitialState "Type of initialization (1: no init, 2: steady state, 3/4: initial output)" annotation(
          Evaluate = true,
          Dialog(group = "Initialization"));
        parameter Boolean limitsAtInit = true "= false, if limits are ignored during initialization (i.e., der(y)=k*u)" annotation(
          Evaluate = true,
          Dialog(group = "Initialization"));
        parameter Real y_start = 0 "Initial or guess value of output (must be in the limits outMin .. outMax)" annotation(
          Dialog(group = "Initialization"));
        parameter Boolean strict = false "= true, if strict limits with noEvent(..)" annotation(
          Evaluate = true,
          choices(checkBox = true),
          Dialog(tab = "Advanced"));
        extends Interfaces.SISO(y(start = y_start));
      initial equation
        if initType == Init.SteadyState then
          der(y) = 0;
        elseif initType == Init.InitialState or initType == Init.InitialOutput then
          y = y_start;
        end if;
      equation
        if initial() and not limitsAtInit then
          der(y) = k * u;
          assert(y >= outMin - 0.001 * abs(outMax - outMin) and y <= outMax + 0.001 * abs(outMax - outMin), "LimIntegrator: During initialization the limits have been ignored.\n" + "However, the result is that the output y is not within the required limits:\n" + "  y = " + String(y) + ", outMin = " + String(outMin) + ", outMax = " + String(outMax));
        elseif strict then
          der(y) = noEvent(if y < outMin and k * u < 0 or y > outMax and k * u > 0 then 0 else k * u);
        else
          der(y) = if y < outMin and k * u < 0 or y > outMax and k * u > 0 then 0 else k * u;
        end if;
        annotation(
          Documentation(info = "<html>
<p>
This blocks computes <b>y</b> (element-wise) as <i>integral</i>
of the input <b>u</b> multiplied with the gain <i>k</i>. If the
integral reaches a given upper or lower <i>limit</i> and the
input will drive the integral outside of this bound, the
integration is halted and only restarted if the input drives
the integral away from the bounds.
</p>

<p>
It might be difficult to initialize the integrator in steady state.
This is discussed in the description of package
<a href=\"modelica://Modelica.Blocks.Continuous#info\">Continuous</a>.
</p>

<p>
If parameter <b>limitAtInit</b> = <b>false</b>, the limits of the
integrator are removed from the initialization problem which
leads to a much simpler equation system. After initialization has been
performed, it is checked via an assert whether the output is in the
defined limits. For backward compatibility reasons
<b>limitAtInit</b> = <b>true</b>. In most cases it is best
to use <b>limitAtInit</b> = <b>false</b>.
</p>
</html>"),
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-80, 78}, {-80, -90}}, color = {192, 192, 192}), Polygon(points = {{-80, 90}, {-88, 68}, {-72, 68}, {-80, 90}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-90, -80}, {82, -80}}, color = {192, 192, 192}), Polygon(points = {{90, -80}, {68, -72}, {68, -88}, {90, -80}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-80, -80}, {20, 20}, {80, 20}}, color = {0, 0, 127}), Text(extent = {{0, -10}, {60, -70}}, lineColor = {192, 192, 192}, textString = "I"), Text(extent = {{-150, -150}, {150, -110}}, lineColor = {0, 0, 0}, textString = "k=%k"), Line(visible = strict, points = {{20, 20}, {80, 20}}, color = {255, 0, 0}, smooth = Smooth.None)}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-60, 60}, {60, -60}}, lineColor = {0, 0, 255}), Text(extent = {{-54, 46}, {-4, -48}}, lineColor = {0, 0, 0}, textString = "lim"), Line(points = {{-100, 0}, {-60, 0}}, color = {0, 0, 255}), Line(points = {{60, 0}, {100, 0}}, color = {0, 0, 255}), Text(extent = {{-8, 60}, {60, 2}}, lineColor = {0, 0, 0}, textString = "k"), Text(extent = {{-8, -2}, {60, -60}}, lineColor = {0, 0, 0}, textString = "s"), Line(points = {{4, 0}, {46, 0}}, color = {0, 0, 0})}));
      end LimIntegrator;

      block Derivative "Approximated derivative block"
        import Modelica.Blocks.Types.Init;
        parameter Real k(unit = "1") = 1 "Gains";
        parameter SIunits.Time T(min = Modelica.Constants.small) = 0.01 "Time constants (T>0 required; T=0 is ideal derivative block)";
        parameter Modelica.Blocks.Types.Init initType = Modelica.Blocks.Types.Init.NoInit "Type of initialization (1: no init, 2: steady state, 3: initial state, 4: initial output)" annotation(
          Evaluate = true,
          Dialog(group = "Initialization"));
        parameter Real x_start = 0 "Initial or guess value of state" annotation(
          Dialog(group = "Initialization"));
        parameter Real y_start = 0 "Initial value of output (= state)" annotation(
          Dialog(enable = initType == Init.InitialOutput, group = "Initialization"));
        extends Interfaces.SISO;
        output Real x(start = x_start) "State of block";
      protected
        parameter Boolean zeroGain = abs(k) < Modelica.Constants.eps;
      initial equation
        if initType == Init.SteadyState then
          der(x) = 0;
        elseif initType == Init.InitialState then
          x = x_start;
        elseif initType == Init.InitialOutput then
          if zeroGain then
            x = u;
          else
            y = y_start;
          end if;
        end if;
      equation
        der(x) = if zeroGain then 0 else (u - x) / T;
        y = if zeroGain then 0 else k / T * (u - x);
        annotation(
          Documentation(info = "<html>
<p>
This blocks defines the transfer function between the
input u and the output y
(element-wise) as <i>approximated derivative</i>:
</p>
<pre>
             k * s
     y = ------------ * u
            T * s + 1
</pre>
<p>
If you would like to be able to change easily between different
transfer functions (FirstOrder, SecondOrder, ... ) by changing
parameters, use the general block <b>TransferFunction</b> instead
and model a derivative block with parameters<br>
b = {k,0}, a = {T, 1}.
</p>

<p>
If k=0, the block reduces to y=0.
</p>
</html>"),
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100.0, -100.0}, {100.0, 100.0}}, initialScale = 0.1), graphics = {Line(visible = true, points = {{-80.0, 78.0}, {-80.0, -90.0}}, color = {192, 192, 192}), Polygon(visible = true, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid, points = {{-80.0, 90.0}, {-88.0, 68.0}, {-72.0, 68.0}, {-80.0, 90.0}}), Line(visible = true, points = {{-90.0, -80.0}, {82.0, -80.0}}, color = {192, 192, 192}), Polygon(visible = true, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid, points = {{90.0, -80.0}, {68.0, -72.0}, {68.0, -88.0}, {90.0, -80.0}}), Line(visible = true, origin = {-24.667, -27.333}, points = {{-55.333, 87.333}, {-19.333, -40.667}, {86.667, -52.667}}, color = {0, 0, 127}, smooth = Smooth.Bezier), Text(visible = true, lineColor = {192, 192, 192}, extent = {{-30.0, 14.0}, {86.0, 60.0}}, textString = "DT1"), Text(visible = true, extent = {{-150.0, -150.0}, {150.0, -110.0}}, textString = "k=%k")}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-54, 52}, {50, 10}}, lineColor = {0, 0, 0}, textString = "k s"), Text(extent = {{-54, -6}, {52, -52}}, lineColor = {0, 0, 0}, textString = "T s + 1"), Line(points = {{-50, 0}, {50, 0}}, color = {0, 0, 0}), Rectangle(extent = {{-60, 60}, {60, -60}}, lineColor = {0, 0, 255}), Line(points = {{-100, 0}, {-60, 0}}, color = {0, 0, 255}), Line(points = {{60, 0}, {100, 0}}, color = {0, 0, 255})}));
      end Derivative;

      block FirstOrder "First order transfer function block (= 1 pole)"
        import Modelica.Blocks.Types.Init;
        parameter Real k(unit = "1") = 1 "Gain";
        parameter SIunits.Time T(start = 1) "Time Constant";
        parameter Modelica.Blocks.Types.Init initType = Modelica.Blocks.Types.Init.NoInit "Type of initialization (1: no init, 2: steady state, 3/4: initial output)" annotation(
          Evaluate = true,
          Dialog(group = "Initialization"));
        parameter Real y_start = 0 "Initial or guess value of output (= state)" annotation(
          Dialog(group = "Initialization"));
        extends Interfaces.SISO(y(start = y_start));
      initial equation
        if initType == Init.SteadyState then
          der(y) = 0;
        elseif initType == Init.InitialState or initType == Init.InitialOutput then
          y = y_start;
        end if;
      equation
        der(y) = (k * u - y) / T;
        annotation(
          Documentation(info = "<HTML>
<p>
This blocks defines the transfer function between the input u
and the output y (element-wise) as <i>first order</i> system:
</p>
<pre>
               k
     y = ------------ * u
            T * s + 1
</pre>
<p>
If you would like to be able to change easily between different
transfer functions (FirstOrder, SecondOrder, ... ) by changing
parameters, use the general block <b>TransferFunction</b> instead
and model a first order SISO system with parameters<br>
b = {k}, a = {T, 1}.
</p>
<pre>
Example:
   parameter: k = 0.3, T = 0.4
   results in:
             0.3
      y = ----------- * u
          0.4 s + 1.0
</pre>

</html>"),
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100.0, -100.0}, {100.0, 100.0}}, initialScale = 0.1), graphics = {Line(visible = true, points = {{-80.0, 78.0}, {-80.0, -90.0}}, color = {192, 192, 192}), Polygon(visible = true, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid, points = {{-80.0, 90.0}, {-88.0, 68.0}, {-72.0, 68.0}, {-80.0, 90.0}}), Line(visible = true, points = {{-90.0, -80.0}, {82.0, -80.0}}, color = {192, 192, 192}), Polygon(visible = true, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid, points = {{90.0, -80.0}, {68.0, -72.0}, {68.0, -88.0}, {90.0, -80.0}}), Line(visible = true, origin = {-26.667, 6.667}, points = {{106.667, 43.333}, {-13.333, 29.333}, {-53.333, -86.667}}, color = {0, 0, 127}, smooth = Smooth.Bezier), Text(visible = true, lineColor = {192, 192, 192}, extent = {{0.0, -60.0}, {60.0, 0.0}}, textString = "PT1"), Text(visible = true, extent = {{-150.0, -150.0}, {150.0, -110.0}}, textString = "T=%T")}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-48, 52}, {50, 8}}, lineColor = {0, 0, 0}, textString = "k"), Text(extent = {{-54, -6}, {56, -56}}, lineColor = {0, 0, 0}, textString = "T s + 1"), Line(points = {{-50, 0}, {50, 0}}, color = {0, 0, 0}), Rectangle(extent = {{-60, 60}, {60, -60}}, lineColor = {0, 0, 255}), Line(points = {{-100, 0}, {-60, 0}}, color = {0, 0, 255}), Line(points = {{60, 0}, {100, 0}}, color = {0, 0, 255})}));
      end FirstOrder;

      block SecondOrder "Second order transfer function block (= 2 poles)"
        import Modelica.Blocks.Types.Init;
        parameter Real k(unit = "1") = 1 "Gain";
        parameter Real w(start = 1) "Angular frequency";
        parameter Real D(start = 1) "Damping";
        parameter Modelica.Blocks.Types.Init initType = Modelica.Blocks.Types.Init.NoInit "Type of initialization (1: no init, 2: steady state, 3/4: initial output)" annotation(
          Evaluate = true,
          Dialog(group = "Initialization"));
        parameter Real y_start = 0 "Initial or guess value of output (= state)" annotation(
          Dialog(group = "Initialization"));
        parameter Real yd_start = 0 "Initial or guess value of derivative of output (= state)" annotation(
          Dialog(group = "Initialization"));
        extends Interfaces.SISO(y(start = y_start));
        output Real yd(start = yd_start) "Derivative of y";
      initial equation
        if initType == Init.SteadyState then
          der(y) = 0;
          der(yd) = 0;
        elseif initType == Init.InitialState or initType == Init.InitialOutput then
          y = y_start;
          yd = yd_start;
        end if;
      equation
        der(y) = yd;
        der(yd) = w * (w * (k * u - y) - 2 * D * yd);
        annotation(
          Documentation(info = "<HTML>
<p>
This blocks defines the transfer function between the input u and
the output y (element-wise) as <i>second order</i> system:
</p>
<pre>
                             k
     y = ---------------------------------------- * u
            ( s / w )^2 + 2*D*( s / w ) + 1
</pre>
<p>
If you would like to be able to change easily between different
transfer functions (FirstOrder, SecondOrder, ... ) by changing
parameters, use the general model class <b>TransferFunction</b>
instead and model a second order SISO system with parameters<br>
b = {k}, a = {1/w^2, 2*D/w, 1}.
</p>
<pre>
Example:

   parameter: k =  0.3,  w = 0.5,  D = 0.4
   results in:
                  0.3
      y = ------------------- * u
          4.0 s^2 + 1.6 s + 1
</pre>

</html>"),
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100.0, -100.0}, {100.0, 100.0}}, initialScale = 0.1), graphics = {Line(visible = true, points = {{-80.0, 78.0}, {-80.0, -90.0}}, color = {192, 192, 192}), Polygon(visible = true, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid, points = {{-80.0, 90.0}, {-88.0, 68.0}, {-72.0, 68.0}, {-80.0, 90.0}}), Line(visible = true, points = {{-90.0, -80.0}, {82.0, -80.0}}, color = {192, 192, 192}), Polygon(visible = true, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid, points = {{90.0, -80.0}, {68.0, -72.0}, {68.0, -88.0}, {90.0, -80.0}}), Line(visible = true, origin = {-1.939, -1.816}, points = {{81.939, 36.056}, {65.362, 36.056}, {14.39, -26.199}, {-29.966, 113.485}, {-65.374, -61.217}, {-78.061, -78.184}}, color = {0, 0, 127}, smooth = Smooth.Bezier), Text(visible = true, lineColor = {192, 192, 192}, extent = {{0.0, -70.0}, {60.0, -10.0}}, textString = "PT2"), Text(visible = true, extent = {{-150.0, -150.0}, {150.0, -110.0}}, textString = "w=%w")}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-60, 60}, {60, -60}}, lineColor = {0, 0, 255}), Text(extent = {{-60, 60}, {60, 14}}, lineColor = {0, 0, 0}, textString = "k"), Text(extent = {{-60, 8}, {-32, -20}}, lineColor = {0, 0, 0}, textString = "s"), Line(points = {{-100, 0}, {-60, 0}}, color = {0, 0, 255}), Line(points = {{60, 0}, {100, 0}}, color = {0, 0, 255}), Line(points = {{-50, 14}, {50, 14}}, color = {0, 0, 0}), Line(points = {{-54, -20}, {-38, -20}}, color = {0, 0, 0}), Text(extent = {{-52, -26}, {-36, -48}}, lineColor = {0, 0, 0}, textString = "w"), Line(points = {{-50, 2}, {-56, -8}, {-56, -28}, {-52, -46}}, color = {0, 0, 0}), Line(points = {{-40, 2}, {-34, -10}, {-34, -30}, {-38, -46}}, color = {0, 0, 0}), Text(extent = {{-34, 8}, {-22, -10}}, lineColor = {0, 0, 0}, textString = "2"), Text(extent = {{-34, -6}, {6, -36}}, lineColor = {0, 0, 0}, textString = "+2D"), Text(extent = {{2, 8}, {30, -20}}, lineColor = {0, 0, 0}, textString = "s"), Line(points = {{8, -20}, {24, -20}}, color = {0, 0, 0}), Text(extent = {{10, -26}, {26, -48}}, lineColor = {0, 0, 0}, textString = "w"), Line(points = {{12, 2}, {6, -8}, {6, -28}, {10, -46}}, color = {0, 0, 0}), Line(points = {{22, 2}, {28, -10}, {28, -30}, {24, -46}}, color = {0, 0, 0}), Text(extent = {{30, 2}, {58, -42}}, lineColor = {0, 0, 0}, textString = "+1")}));
      end SecondOrder;

      block PI "Proportional-Integral controller"
        import Modelica.Blocks.Types.Init;
        parameter Real k(unit = "1") = 1 "Gain";
        parameter SIunits.Time T(start = 1, min = Modelica.Constants.small) "Time Constant (T>0 required)";
        parameter Modelica.Blocks.Types.Init initType = Modelica.Blocks.Types.Init.NoInit "Type of initialization (1: no init, 2: steady state, 3: initial state, 4: initial output)" annotation(
          Evaluate = true,
          Dialog(group = "Initialization"));
        parameter Real x_start = 0 "Initial or guess value of state" annotation(
          Dialog(group = "Initialization"));
        parameter Real y_start = 0 "Initial value of output" annotation(
          Dialog(enable = initType == Init.SteadyState or initType == Init.InitialOutput, group = "Initialization"));
        extends Interfaces.SISO;
        output Real x(start = x_start) "State of block";
      initial equation
        if initType == Init.SteadyState then
          der(x) = 0;
        elseif initType == Init.InitialState then
          x = x_start;
        elseif initType == Init.InitialOutput then
          y = y_start;
        end if;
      equation
        der(x) = u / T;
        y = k * (x + u);
        annotation(
          defaultComponentName = "PI",
          Documentation(info = "<html>
<p>
This blocks defines the transfer function between the input u and
the output y (element-wise) as <i>PI</i> system:
</p>
<pre>
                 1
   y = k * (1 + ---) * u
                T*s
           T*s + 1
     = k * ------- * u
             T*s
</pre>
<p>
If you would like to be able to change easily between different
transfer functions (FirstOrder, SecondOrder, ... ) by changing
parameters, use the general model class <b>TransferFunction</b>
instead and model a PI SISO system with parameters<br>
b = {k*T, k}, a = {T, 0}.
</p>
<pre>
Example:

   parameter: k = 0.3,  T = 0.4

   results in:
               0.4 s + 1
      y = 0.3 ----------- * u
                 0.4 s
</pre>

<p>
It might be difficult to initialize the PI component in steady state
due to the integrator part.
This is discussed in the description of package
<a href=\"modelica://Modelica.Blocks.Continuous#info\">Continuous</a>.
</p>

</html>"),
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-80, 78}, {-80, -90}}, color = {192, 192, 192}), Polygon(points = {{-80, 90}, {-88, 68}, {-72, 68}, {-80, 90}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-90, -80}, {82, -80}}, color = {192, 192, 192}), Polygon(points = {{90, -80}, {68, -72}, {68, -88}, {90, -80}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-80.0, -80.0}, {-80.0, -20.0}, {60.0, 80.0}}, color = {0, 0, 127}), Text(extent = {{0, 6}, {60, -56}}, lineColor = {192, 192, 192}, textString = "PI"), Text(extent = {{-150, -150}, {150, -110}}, lineColor = {0, 0, 0}, textString = "T=%T")}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-60, 60}, {60, -60}}, lineColor = {0, 0, 255}), Text(extent = {{-68, 24}, {-24, -18}}, lineColor = {0, 0, 0}, textString = "k"), Text(extent = {{-32, 48}, {60, 0}}, lineColor = {0, 0, 0}, textString = "T s + 1"), Text(extent = {{-30, -8}, {52, -40}}, lineColor = {0, 0, 0}, textString = "T s"), Line(points = {{-24, 0}, {54, 0}}, color = {0, 0, 0}), Line(points = {{-100, 0}, {-60, 0}}, color = {0, 0, 255}), Line(points = {{62, 0}, {100, 0}}, color = {0, 0, 255})}));
      end PI;

      block PID "PID-controller in additive description form"
        import Modelica.Blocks.Types.InitPID;
        import Modelica.Blocks.Types.Init;
        extends Interfaces.SISO;
        parameter Real k(unit = "1") = 1 "Gain";
        parameter SIunits.Time Ti(min = Modelica.Constants.small, start = 0.5) "Time Constant of Integrator";
        parameter SIunits.Time Td(min = 0, start = 0.1) "Time Constant of Derivative block";
        parameter Real Nd(min = Modelica.Constants.small) = 10 "The higher Nd, the more ideal the derivative block";
        parameter Modelica.Blocks.Types.InitPID initType = Modelica.Blocks.Types.InitPID.DoNotUse_InitialIntegratorState "Type of initialization (1: no init, 2: steady state, 3: initial state, 4: initial output)" annotation(
          Evaluate = true,
          Dialog(group = "Initialization"));
        parameter Real xi_start = 0 "Initial or guess value value for integrator output (= integrator state)" annotation(
          Dialog(group = "Initialization"));
        parameter Real xd_start = 0 "Initial or guess value for state of derivative block" annotation(
          Dialog(group = "Initialization"));
        parameter Real y_start = 0 "Initial value of output" annotation(
          Dialog(enable = initType == InitPID.InitialOutput, group = "Initialization"));
        constant SI.Time unitTime = 1 annotation(
          HideResult = true);
        Blocks.Math.Gain P(k = 1) "Proportional part of PID controller" annotation(
          Placement(transformation(extent = {{-60, 60}, {-20, 100}}, rotation = 0)));
        Blocks.Continuous.Integrator I(k = unitTime / Ti, y_start = xi_start, initType = if initType == InitPID.SteadyState then Init.SteadyState else if initType == InitPID.InitialState or initType == InitPID.DoNotUse_InitialIntegratorState then Init.InitialState else Init.NoInit) "Integral part of PID controller" annotation(
          Placement(transformation(extent = {{-60, -20}, {-20, 20}}, rotation = 0)));
        Blocks.Continuous.Derivative D(k = Td / unitTime, T = max([Td / Nd, 100 * Modelica.Constants.eps]), x_start = xd_start, initType = if initType == InitPID.SteadyState or initType == InitPID.InitialOutput then Init.SteadyState else if initType == InitPID.InitialState then Init.InitialState else Init.NoInit) "Derivative part of PID controller" annotation(
          Placement(transformation(extent = {{-60, -100}, {-20, -60}}, rotation = 0)));
        Blocks.Math.Gain Gain(k = k) "Gain of PID controller" annotation(
          Placement(transformation(extent = {{60, -10}, {80, 10}}, rotation = 0)));
        Blocks.Math.Add3 Add annotation(
          Placement(transformation(extent = {{20, -10}, {40, 10}}, rotation = 0)));
      initial equation
        if initType == InitPID.InitialOutput then
          y = y_start;
        end if;
      equation
        connect(u, P.u) annotation(
          Line(points = {{-120, 0}, {-80, 0}, {-80, 80}, {-64, 80}}, color = {0, 0, 127}));
        connect(u, I.u) annotation(
          Line(points = {{-120, 0}, {-64, 0}}, color = {0, 0, 127}));
        connect(u, D.u) annotation(
          Line(points = {{-120, 0}, {-80, 0}, {-80, -80}, {-64, -80}}, color = {0, 0, 127}));
        connect(P.y, Add.u1) annotation(
          Line(points = {{-18, 80}, {0, 80}, {0, 8}, {18, 8}}, color = {0, 0, 127}));
        connect(I.y, Add.u2) annotation(
          Line(points = {{-18, 0}, {18, 0}}, color = {0, 0, 127}));
        connect(D.y, Add.u3) annotation(
          Line(points = {{-18, -80}, {0, -80}, {0, -8}, {18, -8}}, color = {0, 0, 127}));
        connect(Add.y, Gain.u) annotation(
          Line(points = {{41, 0}, {58, 0}}, color = {0, 0, 127}));
        connect(Gain.y, y) annotation(
          Line(points = {{81, 0}, {110, 0}}, color = {0, 0, 127}));
        annotation(
          defaultComponentName = "PID",
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100.0, -100.0}, {100.0, 100.0}}, initialScale = 0.1), graphics = {Line(visible = true, points = {{-80.0, 78.0}, {-80.0, -90.0}}, color = {192, 192, 192}), Polygon(visible = true, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid, points = {{-80.0, 90.0}, {-88.0, 68.0}, {-72.0, 68.0}, {-80.0, 90.0}}), Line(visible = true, points = {{-90.0, -80.0}, {82.0, -80.0}}, color = {192, 192, 192}), Polygon(visible = true, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid, points = {{90.0, -80.0}, {68.0, -72.0}, {68.0, -88.0}, {90.0, -80.0}}), Line(visible = true, points = {{-80, -80}, {-80, -20}, {60, 80}}, color = {0, 0, 127}), Text(visible = true, lineColor = {192, 192, 192}, extent = {{-20.0, -60.0}, {80.0, -20.0}}, textString = "PID"), Text(visible = true, extent = {{-150.0, -150.0}, {150.0, -110.0}}, textString = "Ti=%Ti")}),
          Documentation(info = "<HTML>
<p>
This is the text-book version of a PID-controller.
For a more practically useful PID-controller, use
block LimPID.
</p>

<p>
The PID block can be initialized in different
ways controlled by parameter <b>initType</b>. The possible
values of initType are defined in
<a href=\"modelica://Modelica.Blocks.Types.InitPID\">Modelica.Blocks.Types.InitPID</a>.
This type is identical to
<a href=\"modelica://Modelica.Blocks.Types.Init\">Types.Init</a>,
with the only exception that the additional option
<b>DoNotUse_InitialIntegratorState</b> is added for
backward compatibility reasons (= integrator is initialized with
InitialState whereas differential part is initialized with
NoInit which was the initialization in version 2.2 of the Modelica
standard library).
</p>

<p>
Based on the setting of initType, the integrator (I) and derivative (D)
blocks inside the PID controller are initialized according to the following table:
</p>

<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"><b>initType</b></td>
      <td valign=\"top\"><b>I.initType</b></td>
      <td valign=\"top\"><b>D.initType</b></td></tr>

  <tr><td valign=\"top\"><b>NoInit</b></td>
      <td valign=\"top\">NoInit</td>
      <td valign=\"top\">NoInit</td></tr>

  <tr><td valign=\"top\"><b>SteadyState</b></td>
      <td valign=\"top\">SteadyState</td>
      <td valign=\"top\">SteadyState</td></tr>

  <tr><td valign=\"top\"><b>InitialState</b></td>
      <td valign=\"top\">InitialState</td>
      <td valign=\"top\">InitialState</td></tr>

  <tr><td valign=\"top\"><b>InitialOutput</b><br>
          and initial equation: y = y_start</td>
      <td valign=\"top\">NoInit</td>
      <td valign=\"top\">SteadyState</td></tr>

  <tr><td valign=\"top\"><b>DoNotUse_InitialIntegratorState</b></td>
      <td valign=\"top\">InitialState</td>
      <td valign=\"top\">NoInit</td></tr>
</table>

<p>
In many cases, the most useful initial condition is
<b>SteadyState</b> because initial transients are then no longer
present. If initType = InitPID.SteadyState, then in some
cases difficulties might occur. The reason is the
equation of the integrator:
</p>

<pre>
   <b>der</b>(y) = k*u;
</pre>

<p>
The steady state equation \"der(x)=0\" leads to the condition that the input u to the
integrator is zero. If the input u is already (directly or indirectly) defined
by another initial condition, then the initialization problem is <b>singular</b>
(has none or infinitely many solutions). This situation occurs often
for mechanical systems, where, e.g., u = desiredSpeed - measuredSpeed and
since speed is both a state and a derivative, it is natural to
initialize it with zero. As sketched this is, however, not possible.
The solution is to not initialize u or the variable that is used
to compute u by an algebraic equation.
</p>

</html>"));
      end PID;

      block LimPID "P, PI, PD, and PID controller with limited output, anti-windup compensation and setpoint weighting"
        import Modelica.Blocks.Types.InitPID;
        import Modelica.Blocks.Types.Init;
        import Modelica.Blocks.Types.SimpleController;
        extends Interfaces.SVcontrol;
        output Real controlError = u_s - u_m "Control error (set point - measurement)";
        parameter .Modelica.Blocks.Types.SimpleController controllerType = .Modelica.Blocks.Types.SimpleController.PID "Type of controller";
        parameter Real k(min = 0, unit = "1") = 1 "Gain of controller";
        parameter SIunits.Time Ti(min = Modelica.Constants.small) = 0.5 "Time constant of Integrator block" annotation(
          Dialog(enable = controllerType == .Modelica.Blocks.Types.SimpleController.PI or controllerType == .Modelica.Blocks.Types.SimpleController.PID));
        parameter SIunits.Time Td(min = 0) = 0.1 "Time constant of Derivative block" annotation(
          Dialog(enable = controllerType == .Modelica.Blocks.Types.SimpleController.PD or controllerType == .Modelica.Blocks.Types.SimpleController.PID));
        parameter Real yMax(start = 1) "Upper limit of output";
        parameter Real yMin = -yMax "Lower limit of output";
        parameter Real wp(min = 0) = 1 "Set-point weight for Proportional block (0..1)";
        parameter Real wd(min = 0) = 0 "Set-point weight for Derivative block (0..1)" annotation(
          Dialog(enable = controllerType == .Modelica.Blocks.Types.SimpleController.PD or controllerType == .Modelica.Blocks.Types.SimpleController.PID));
        parameter Real Ni(min = 100 * Modelica.Constants.eps) = 0.9 "Ni*Ti is time constant of anti-windup compensation" annotation(
          Dialog(enable = controllerType == .Modelica.Blocks.Types.SimpleController.PI or controllerType == .Modelica.Blocks.Types.SimpleController.PID));
        parameter Real Nd(min = 100 * Modelica.Constants.eps) = 10 "The higher Nd, the more ideal the derivative block" annotation(
          Dialog(enable = controllerType == .Modelica.Blocks.Types.SimpleController.PD or controllerType == .Modelica.Blocks.Types.SimpleController.PID));
        parameter .Modelica.Blocks.Types.InitPID initType = .Modelica.Blocks.Types.InitPID.DoNotUse_InitialIntegratorState "Type of initialization (1: no init, 2: steady state, 3: initial state, 4: initial output)" annotation(
          Evaluate = true,
          Dialog(group = "Initialization"));
        parameter Boolean limitsAtInit = true "= false, if limits are ignored during initialization" annotation(
          Evaluate = true,
          Dialog(group = "Initialization"));
        parameter Real xi_start = 0 "Initial or guess value value for integrator output (= integrator state)" annotation(
          Dialog(group = "Initialization", enable = controllerType == .Modelica.Blocks.Types.SimpleController.PI or controllerType == .Modelica.Blocks.Types.SimpleController.PID));
        parameter Real xd_start = 0 "Initial or guess value for state of derivative block" annotation(
          Dialog(group = "Initialization", enable = controllerType == .Modelica.Blocks.Types.SimpleController.PD or controllerType == .Modelica.Blocks.Types.SimpleController.PID));
        parameter Real y_start = 0 "Initial value of output" annotation(
          Dialog(enable = initType == .Modelica.Blocks.Types.InitPID.InitialOutput, group = "Initialization"));
        parameter Boolean strict = false "= true, if strict limits with noEvent(..)" annotation(
          Evaluate = true,
          choices(checkBox = true),
          Dialog(tab = "Advanced"));
        constant SI.Time unitTime = 1 annotation(
          HideResult = true);
        Blocks.Math.Add addP(k1 = wp, k2 = -1) annotation(
          Placement(transformation(extent = {{-80, 40}, {-60, 60}}, rotation = 0)));
        Blocks.Math.Add addD(k1 = wd, k2 = -1) if with_D annotation(
          Placement(transformation(extent = {{-80, -10}, {-60, 10}}, rotation = 0)));
        Blocks.Math.Gain P(k = 1) annotation(
          Placement(transformation(extent = {{-40, 40}, {-20, 60}}, rotation = 0)));
        Blocks.Continuous.Integrator I(k = unitTime / Ti, y_start = xi_start, initType = if initType == InitPID.SteadyState then Init.SteadyState else if initType == InitPID.InitialState or initType == InitPID.DoNotUse_InitialIntegratorState then Init.InitialState else Init.NoInit) if with_I annotation(
          Placement(transformation(extent = {{-40, -60}, {-20, -40}}, rotation = 0)));
        Blocks.Continuous.Derivative D(k = Td / unitTime, T = max([Td / Nd, 1.e-14]), x_start = xd_start, initType = if initType == InitPID.SteadyState or initType == InitPID.InitialOutput then Init.SteadyState else if initType == InitPID.InitialState then Init.InitialState else Init.NoInit) if with_D annotation(
          Placement(transformation(extent = {{-40, -10}, {-20, 10}}, rotation = 0)));
        Blocks.Math.Gain gainPID(k = k) annotation(
          Placement(transformation(extent = {{30, -10}, {50, 10}}, rotation = 0)));
        Blocks.Math.Add3 addPID annotation(
          Placement(transformation(extent = {{0, -10}, {20, 10}}, rotation = 0)));
        Blocks.Math.Add3 addI(k2 = -1) if with_I annotation(
          Placement(transformation(extent = {{-80, -60}, {-60, -40}}, rotation = 0)));
        Blocks.Math.Add addSat(k1 = +1, k2 = -1) if with_I annotation(
          Placement(transformation(origin = {80, -50}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
        Blocks.Math.Gain gainTrack(k = 1 / (k * Ni)) if with_I annotation(
          Placement(transformation(extent = {{40, -80}, {20, -60}}, rotation = 0)));
        Blocks.Nonlinear.Limiter limiter(uMax = yMax, uMin = yMin, strict = strict, limitsAtInit = limitsAtInit) annotation(
          Placement(transformation(extent = {{70, -10}, {90, 10}}, rotation = 0)));
      protected
        parameter Boolean with_I = controllerType == SimpleController.PI or controllerType == SimpleController.PID annotation(
          Evaluate = true,
          HideResult = true);
        parameter Boolean with_D = controllerType == SimpleController.PD or controllerType == SimpleController.PID annotation(
          Evaluate = true,
          HideResult = true);
      public
        Sources.Constant Dzero(k = 0) if not with_D annotation(
          Placement(transformation(extent = {{-30, 20}, {-20, 30}}, rotation = 0)));
        Sources.Constant Izero(k = 0) if not with_I annotation(
          Placement(transformation(extent = {{10, -55}, {0, -45}}, rotation = 0)));
      initial equation
        if initType == InitPID.InitialOutput then
          gainPID.y = y_start;
        end if;
      equation
        assert(yMax >= yMin, "LimPID: Limits must be consistent. However, yMax (=" + String(yMax) + ") < yMin (=" + String(yMin) + ")");
        if initType == InitPID.InitialOutput and (y_start < yMin or y_start > yMax) then
          Modelica.Utilities.Streams.error("LimPID: Start value y_start (=" + String(y_start) + ") is outside of the limits of yMin (=" + String(yMin) + ") and yMax (=" + String(yMax) + ")");
        end if;
        assert(limitsAtInit or not limitsAtInit and y >= yMin and y <= yMax, "LimPID: During initialization the limits have been switched off.\n" + "After initialization, the output y (=" + String(y) + ") is outside of the limits of yMin (=" + String(yMin) + ") and yMax (=" + String(yMax) + ")");
        connect(u_s, addP.u1) annotation(
          Line(points = {{-120, 0}, {-96, 0}, {-96, 56}, {-82, 56}}, color = {0, 0, 127}));
        connect(u_s, addD.u1) annotation(
          Line(points = {{-120, 0}, {-96, 0}, {-96, 6}, {-82, 6}}, color = {0, 0, 127}));
        connect(u_s, addI.u1) annotation(
          Line(points = {{-120, 0}, {-96, 0}, {-96, -42}, {-82, -42}}, color = {0, 0, 127}));
        connect(addP.y, P.u) annotation(
          Line(points = {{-59, 50}, {-42, 50}}, color = {0, 0, 127}));
        connect(addD.y, D.u) annotation(
          Line(points = {{-59, 0}, {-42, 0}}, color = {0, 0, 127}));
        connect(addI.y, I.u) annotation(
          Line(points = {{-59, -50}, {-42, -50}}, color = {0, 0, 127}));
        connect(P.y, addPID.u1) annotation(
          Line(points = {{-19, 50}, {-10, 50}, {-10, 8}, {-2, 8}}, color = {0, 0, 127}));
        connect(D.y, addPID.u2) annotation(
          Line(points = {{-19, 0}, {-2, 0}}, color = {0, 0, 127}));
        connect(I.y, addPID.u3) annotation(
          Line(points = {{-19, -50}, {-10, -50}, {-10, -8}, {-2, -8}}, color = {0, 0, 127}));
        connect(addPID.y, gainPID.u) annotation(
          Line(points = {{21, 0}, {28, 0}}, color = {0, 0, 127}));
        connect(gainPID.y, addSat.u2) annotation(
          Line(points = {{51, 0}, {60, 0}, {60, -20}, {74, -20}, {74, -38}}, color = {0, 0, 127}));
        connect(gainPID.y, limiter.u) annotation(
          Line(points = {{51, 0}, {68, 0}}, color = {0, 0, 127}));
        connect(limiter.y, addSat.u1) annotation(
          Line(points = {{91, 0}, {94, 0}, {94, -20}, {86, -20}, {86, -38}}, color = {0, 0, 127}));
        connect(limiter.y, y) annotation(
          Line(points = {{91, 0}, {110, 0}}, color = {0, 0, 127}));
        connect(addSat.y, gainTrack.u) annotation(
          Line(points = {{80, -61}, {80, -70}, {42, -70}}, color = {0, 0, 127}));
        connect(gainTrack.y, addI.u3) annotation(
          Line(points = {{19, -70}, {-88, -70}, {-88, -58}, {-82, -58}}, color = {0, 0, 127}));
        connect(u_m, addP.u2) annotation(
          Line(points = {{0, -120}, {0, -92}, {-92, -92}, {-92, 44}, {-82, 44}}, color = {0, 0, 127}, thickness = 0.5));
        connect(u_m, addD.u2) annotation(
          Line(points = {{0, -120}, {0, -92}, {-92, -92}, {-92, -6}, {-82, -6}}, color = {0, 0, 127}, thickness = 0.5));
        connect(u_m, addI.u2) annotation(
          Line(points = {{0, -120}, {0, -92}, {-92, -92}, {-92, -50}, {-82, -50}}, color = {0, 0, 127}, thickness = 0.5));
        connect(Dzero.y, addPID.u2) annotation(
          Line(points = {{-19.5, 25}, {-14, 25}, {-14, 0}, {-2, 0}}, color = {0, 0, 127}));
        connect(Izero.y, addPID.u3) annotation(
          Line(points = {{-0.5, -50}, {-10, -50}, {-10, -8}, {-2, -8}}, color = {0, 0, 127}));
        annotation(
          defaultComponentName = "PID",
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-80, 78}, {-80, -90}}, color = {192, 192, 192}), Polygon(points = {{-80, 90}, {-88, 68}, {-72, 68}, {-80, 90}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-90, -80}, {82, -80}}, color = {192, 192, 192}), Polygon(points = {{90, -80}, {68, -72}, {68, -88}, {90, -80}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-80, -80}, {-80, -20}, {30, 60}, {80, 60}}, color = {0, 0, 127}), Text(extent = {{-20, -20}, {80, -60}}, lineColor = {192, 192, 192}, textString = "%controllerType"), Line(visible = strict, points = {{30, 60}, {81, 60}}, color = {255, 0, 0}, smooth = Smooth.None)}),
          Documentation(info = "<HTML>
<p>
Via parameter <b>controllerType</b> either <b>P</b>, <b>PI</b>, <b>PD</b>,
or <b>PID</b> can be selected. If, e.g., PI is selected, all components belonging to the
D-part are removed from the block (via conditional declarations).
The example model
<a href=\"modelica://Modelica.Blocks.Examples.PID_Controller\">Modelica.Blocks.Examples.PID_Controller</a>
demonstrates the usage of this controller.
Several practical aspects of PID controller design are incorporated
according to chapter 3 of the book:
</p>

<dl>
<dt>&Aring;str&ouml;m K.J., and H&auml;gglund T.:</dt>
<dd> <b>PID Controllers: Theory, Design, and Tuning</b>.
     Instrument Society of America, 2nd edition, 1995.
</dd>
</dl>

<p>
Besides the additive <b>proportional, integral</b> and <b>derivative</b>
part of this controller, the following features are present:
</p>
<ul>
<li> The output of this controller is limited. If the controller is
     in its limits, anti-windup compensation is activated to drive
     the integrator state to zero. </li>
<li> The high-frequency gain of the derivative part is limited
     to avoid excessive amplification of measurement noise.</li>
<li> Setpoint weighting is present, which allows to weight
     the setpoint in the proportional and the derivative part
     independently from the measurement. The controller will respond
     to load disturbances and measurement noise independently of this setting
     (parameters wp, wd). However, setpoint changes will depend on this
     setting. For example, it is useful to set the setpoint weight wd
     for the derivative part to zero, if steps may occur in the
     setpoint signal.</li>
</ul>

<p>
The parameters of the controller can be manually adjusted by performing
simulations of the closed loop system (= controller + plant connected
together) and using the following strategy:
</p>

<ol>
<li> Set very large limits, e.g., yMax = Modelica.Constants.inf</li>
<li> Select a <b>P</b>-controller and manually enlarge parameter <b>k</b>
     (the total gain of the controller) until the closed-loop response
     cannot be improved any more.</li>
<li> Select a <b>PI</b>-controller and manually adjust parameters
     <b>k</b> and <b>Ti</b> (the time constant of the integrator).
     The first value of Ti can be selected, such that it is in the
     order of the time constant of the oscillations occurring with
     the P-controller. If, e.g., vibrations in the order of T=10 ms
     occur in the previous step, start with Ti=0.01 s.</li>
<li> If you want to make the reaction of the control loop faster
     (but probably less robust against disturbances and measurement noise)
     select a <b>PID</b>-Controller and manually adjust parameters
     <b>k</b>, <b>Ti</b>, <b>Td</b> (time constant of derivative block).</li>
<li> Set the limits yMax and yMin according to your specification.</li>
<li> Perform simulations such that the output of the PID controller
     goes in its limits. Tune <b>Ni</b> (Ni*Ti is the time constant of
     the anti-windup compensation) such that the input to the limiter
     block (= limiter.u) goes quickly enough back to its limits.
     If Ni is decreased, this happens faster. If Ni=infinity, the
     anti-windup compensation is switched off and the controller works bad.</li>
</ol>

<p>
<b>Initialization</b>
</p>

<p>
This block can be initialized in different
ways controlled by parameter <b>initType</b>. The possible
values of initType are defined in
<a href=\"modelica://Modelica.Blocks.Types.InitPID\">Modelica.Blocks.Types.InitPID</a>.
This type is identical to
<a href=\"modelica://Modelica.Blocks.Types.Init\">Types.Init</a>,
with the only exception that the additional option
<b>DoNotUse_InitialIntegratorState</b> is added for
backward compatibility reasons (= integrator is initialized with
InitialState whereas differential part is initialized with
NoInit which was the initialization in version 2.2 of the Modelica
standard library).
</p>

<p>
Based on the setting of initType, the integrator (I) and derivative (D)
blocks inside the PID controller are initialized according to the following table:
</p>

<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"><b>initType</b></td>
      <td valign=\"top\"><b>I.initType</b></td>
      <td valign=\"top\"><b>D.initType</b></td></tr>

  <tr><td valign=\"top\"><b>NoInit</b></td>
      <td valign=\"top\">NoInit</td>
      <td valign=\"top\">NoInit</td></tr>

  <tr><td valign=\"top\"><b>SteadyState</b></td>
      <td valign=\"top\">SteadyState</td>
      <td valign=\"top\">SteadyState</td></tr>

  <tr><td valign=\"top\"><b>InitialState</b></td>
      <td valign=\"top\">InitialState</td>
      <td valign=\"top\">InitialState</td></tr>

  <tr><td valign=\"top\"><b>InitialOutput</b><br>
          and initial equation: y = y_start</td>
      <td valign=\"top\">NoInit</td>
      <td valign=\"top\">SteadyState</td></tr>

  <tr><td valign=\"top\"><b>DoNotUse_InitialIntegratorState</b></td>
      <td valign=\"top\">InitialState</td>
      <td valign=\"top\">NoInit</td></tr>
</table>

<p>
In many cases, the most useful initial condition is
<b>SteadyState</b> because initial transients are then no longer
present. If initType = InitPID.SteadyState, then in some
cases difficulties might occur. The reason is the
equation of the integrator:
</p>

<pre>
   <b>der</b>(y) = k*u;
</pre>

<p>
The steady state equation \"der(x)=0\" leads to the condition that the input u to the
integrator is zero. If the input u is already (directly or indirectly) defined
by another initial condition, then the initialization problem is <b>singular</b>
(has none or infinitely many solutions). This situation occurs often
for mechanical systems, where, e.g., u = desiredSpeed - measuredSpeed and
since speed is both a state and a derivative, it is natural to
initialize it with zero. As sketched this is, however, not possible.
The solution is to not initialize u_m or the variable that is used
to compute u_m by an algebraic equation.
</p>

<p>
If parameter <b>limitAtInit</b> = <b>false</b>, the limits at the
output of this controller block are removed from the initialization problem which
leads to a much simpler equation system. After initialization has been
performed, it is checked via an assert whether the output is in the
defined limits. For backward compatibility reasons
<b>limitAtInit</b> = <b>true</b>. In most cases it is best
to use <b>limitAtInit</b> = <b>false</b>.
</p>
</html>"));
      end LimPID;

      block TransferFunction "Linear transfer function"
        import Modelica.Blocks.Types.Init;
        extends Interfaces.SISO;
        parameter Real b[:] = {1} "Numerator coefficients of transfer function (e.g., 2*s+3 is specified as {2,3})";
        parameter Real a[:] = {1} "Denominator coefficients of transfer function (e.g., 5*s+6 is specified as {5,6})";
        parameter Modelica.Blocks.Types.Init initType = Modelica.Blocks.Types.Init.NoInit "Type of initialization (1: no init, 2: steady state, 3: initial state, 4: initial output)" annotation(
          Evaluate = true,
          Dialog(group = "Initialization"));
        parameter Real x_start[size(a, 1) - 1] = zeros(nx) "Initial or guess values of states" annotation(
          Dialog(group = "Initialization"));
        parameter Real y_start = 0 "Initial value of output (derivatives of y are zero upto nx-1-th derivative)" annotation(
          Dialog(enable = initType == Init.InitialOutput, group = "Initialization"));
        output Real x[size(a, 1) - 1](start = x_start) "State of transfer function from controller canonical form";
      protected
        parameter Integer na = size(a, 1) "Size of Denominator of transfer function.";
        parameter Integer nb = size(b, 1) "Size of Numerator of transfer function.";
        parameter Integer nx = size(a, 1) - 1;
        parameter Real bb[:] = vector([zeros(max(0, na - nb), 1); b]);
        parameter Real d = bb[1] / a[1];
        parameter Real a_end = if a[end] > 100 * Modelica.Constants.eps * sqrt(a * a) then a[end] else 1.0;
        Real x_scaled[size(x, 1)] "Scaled vector x";
      initial equation
        if initType == Init.SteadyState then
          der(x_scaled) = zeros(nx);
        elseif initType == Init.InitialState then
          x_scaled = x_start * a_end;
        elseif initType == Init.InitialOutput then
          y = y_start;
          der(x_scaled[2:nx]) = zeros(nx - 1);
        end if;
      equation
        assert(size(b, 1) <= size(a, 1), "Transfer function is not proper");
        if nx == 0 then
          y = d * u;
        else
          der(x_scaled[1]) = ((-a[2:na] * x_scaled) + a_end * u) / a[1];
          der(x_scaled[2:nx]) = x_scaled[1:nx - 1];
          y = (bb[2:na] - d * a[2:na]) * x_scaled / a_end + d * u;
          x = x_scaled / a_end;
        end if;
        annotation(
          Documentation(info = "<html>
<p>
This block defines the transfer function between the input
u and the output y
as (nb = dimension of b, na = dimension of a):
</p>
<pre>
           b[1]*s^[nb-1] + b[2]*s^[nb-2] + ... + b[nb]
   y(s) = --------------------------------------------- * u(s)
           a[1]*s^[na-1] + a[2]*s^[na-2] + ... + a[na]
</pre>
<p>
State variables <b>x</b> are defined according to <b>controller canonical</b>
form. Internally, vector <b>x</b> is scaled to improve the numerics (the states in versions before version 3.0 of the Modelica Standard Library have been not scaled). This scaling is
not visible from the outside of this block because the non-scaled vector <b>x</b>
is provided as output signal and the start value is with respect to the non-scaled
vector <b>x</b>.
Initial values of the states <b>x</b> can be set via parameter <b>x_start</b>.
</p>

<p>
Example:
</p>
<pre>
     TransferFunction g(b = {2,4}, a = {1,3});
</pre>
<p>
results in the following transfer function:
</p>
<pre>
        2*s + 4
   y = --------- * u
         s + 3
</pre>
</html>"),
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100.0, -100.0}, {100.0, 100.0}}, initialScale = 0.1), graphics = {Line(visible = true, points = {{-80.0, 0.0}, {80.0, 0.0}}, color = {0, 0, 127}), Text(visible = true, lineColor = {0, 0, 127}, extent = {{-90.0, 10.0}, {90.0, 90.0}}, textString = "b(s)"), Text(visible = true, lineColor = {0, 0, 127}, extent = {{-90.0, -90.0}, {90.0, -10.0}}, textString = "a(s)")}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{40, 0}, {-40, 0}}, color = {0, 0, 0}), Text(extent = {{-55, 55}, {55, 5}}, lineColor = {0, 0, 0}, textString = "b(s)"), Text(extent = {{-55, -5}, {55, -55}}, lineColor = {0, 0, 0}, textString = "a(s)"), Rectangle(extent = {{-60, 60}, {60, -60}}, lineColor = {0, 0, 255}), Line(points = {{-100, 0}, {-60, 0}}, color = {0, 0, 255}), Line(points = {{60, 0}, {100, 0}}, color = {0, 0, 255})}));
      end TransferFunction;

      block StateSpace "Linear state space system"
        import Modelica.Blocks.Types.Init;
        parameter Real A[:, size(A, 1)] = [1, 0; 0, 1] "Matrix A of state space model (e.g., A=[1, 0; 0, 1])";
        parameter Real B[size(A, 1), :] = [1; 1] "Matrix B of state space model (e.g., B=[1; 1])";
        parameter Real C[:, size(A, 1)] = [1, 1] "Matrix C of state space model (e.g., C=[1, 1])";
        parameter Real D[size(C, 1), size(B, 2)] = zeros(size(C, 1), size(B, 2)) "Matrix D of state space model";
        parameter Modelica.Blocks.Types.Init initType = Modelica.Blocks.Types.Init.NoInit "Type of initialization (1: no init, 2: steady state, 3: initial state, 4: initial output)" annotation(
          Evaluate = true,
          Dialog(group = "Initialization"));
        parameter Real x_start[nx] = zeros(nx) "Initial or guess values of states" annotation(
          Dialog(group = "Initialization"));
        parameter Real y_start[ny] = zeros(ny) "Initial values of outputs (remaining states are in steady state if possible)" annotation(
          Dialog(enable = initType == Init.InitialOutput, group = "Initialization"));
        extends Interfaces.MIMO(final nin = size(B, 2), final nout = size(C, 1));
        output Real x[size(A, 1)](start = x_start) "State vector";
      protected
        parameter Integer nx = size(A, 1) "number of states";
        parameter Integer ny = size(C, 1) "number of outputs";
      initial equation
        if initType == Init.SteadyState then
          der(x) = zeros(nx);
        elseif initType == Init.InitialState then
          x = x_start;
        elseif initType == Init.InitialOutput then
          x = Modelica.Math.Matrices.equalityLeastSquares(A, -B * u, C, y_start - D * u);
        end if;
      equation
        der(x) = A * x + B * u;
        y = C * x + D * u;
        annotation(
          Documentation(info = "<HTML>
<p>
The State Space block defines the relation
between the input u and the output
y in state space form:
</p>
<pre>

    der(x) = A * x + B * u
        y  = C * x + D * u
</pre>
<p>
The input is a vector of length nu, the output is a vector
of length ny and nx is the number of states. Accordingly
</p>
<pre>
        A has the dimension: A(nx,nx),
        B has the dimension: B(nx,nu),
        C has the dimension: C(ny,nx),
        D has the dimension: D(ny,nu)
</pre>
<p>
Example:
</p>
<pre>
     parameter: A = [0.12, 2;3, 1.5]
     parameter: B = [2, 7;3, 1]
     parameter: C = [0.1, 2]
     parameter: D = zeros(ny,nu)
results in the following equations:
  [der(x[1])]   [0.12  2.00] [x[1]]   [2.0  7.0] [u[1]]
  [         ] = [          ]*[    ] + [        ]*[    ]
  [der(x[2])]   [3.00  1.50] [x[2]]   [0.1  2.0] [u[2]]
                             [x[1]]            [u[1]]
       y[1]   = [0.1  2.0] * [    ] + [0  0] * [    ]
                             [x[2]]            [u[2]]
</pre>
</html>"),
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-90, 10}, {-10, 90}}, textString = "A", lineColor = {0, 0, 127}), Text(extent = {{10, 10}, {90, 90}}, textString = "B", lineColor = {0, 0, 127}), Text(extent = {{-90, -10}, {-10, -90}}, textString = "C", lineColor = {0, 0, 127}), Text(extent = {{10, -10}, {90, -90}}, textString = "D", lineColor = {0, 0, 127}), Line(points = {{0, -90}, {0, 90}}, color = {192, 192, 192}), Line(points = {{-90, 0}, {90, 0}}, color = {192, 192, 192})}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-60, 60}, {60, -60}}, lineColor = {0, 0, 255}), Text(extent = {{-60, 40}, {60, 0}}, lineColor = {0, 0, 0}, textString = "sx=Ax+Bu"), Text(extent = {{-60, 0}, {60, -40}}, lineColor = {0, 0, 0}, textString = " y=Cx+Du"), Line(points = {{-100, 0}, {-60, 0}}, color = {0, 0, 255}), Line(points = {{60, 0}, {100, 0}}, color = {0, 0, 255})}));
      end StateSpace;

      block Der "Derivative of input (= analytic differentiations)"
        extends Interfaces.SISO;
      equation
        y = der(u);
        annotation(
          defaultComponentName = "der1",
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-96, 28}, {94, -24}}, textString = "der()", lineColor = {0, 0, 127})}),
          Documentation(info = "<HTML>
<p>
Defines that the output y is the <i>derivative</i>
of the input u. Note, that Modelica.Blocks.Continuous.Derivative
computes the derivative in an approximate sense, where as this block computes
the derivative exactly. This requires that the input u is differentiated
by the Modelica translator, if this derivative is not yet present in
the model.
</p>
</HTML>"));
      end Der;

      block LowpassButterworth "Output the input signal filtered with a low pass Butterworth filter of any order"
        import Modelica.Blocks.Types.Init;
        extends Modelica.Blocks.Interfaces.SISO;
        parameter Integer n(min = 1) = 2 "Order of filter";
        parameter SI.Frequency f(start = 1) "Cut-off frequency";
        parameter Modelica.Blocks.Types.Init initType = Modelica.Blocks.Types.Init.NoInit "Type of initialization (1: no init, 2: steady state, 3: initial state, 4: initial output)" annotation(
          Evaluate = true,
          Dialog(group = "Initialization"));
        parameter Real x1_start[m] = zeros(m) "Initial or guess values of states 1 (der(x1)=x2)" annotation(
          Dialog(group = "Initialization"));
        parameter Real x2_start[m] = zeros(m) "Initial or guess values of states 2" annotation(
          Dialog(group = "Initialization"));
        parameter Real xr_start = 0.0 "Initial or guess value of real pole for uneven order otherwise dummy" annotation(
          Dialog(group = "Initialization"));
        parameter Real y_start = 0.0 "Initial value of output (states are initialized in steady state if possible)" annotation(
          Dialog(enable = initType == Init.InitialOutput, group = "Initialization"));
        output Real x1[m](start = x1_start) "states 1 of second order filters (der(x1) = x2)";
        output Real x2[m](start = x2_start) "states 2 of second order filters";
        output Real xr(start = xr_start) "state of real pole for uneven order otherwise dummy";
      protected
        constant Real pi = Modelica.Constants.pi;
        parameter Integer m = integer(n / 2);
        parameter Boolean evenOrder = 2 * m == n;
        parameter Real w = 2 * pi * f;
        Real z[m + 1];
        Real polereal[m];
        Real poleimag[m];
        Real realpol;
        Real k2[m];
        Real D[m];
        Real w0[m];
        Real k1;
        Real T;
      initial equation
        if initType == Init.SteadyState then
          der(x1) = zeros(m);
          der(x2) = zeros(m);
          if not evenOrder then
            der(xr) = 0.0;
          end if;
        elseif initType == Init.InitialState then
          x1 = x1_start;
          x2 = x2_start;
          if not evenOrder then
            xr = xr_start;
          end if;
        elseif initType == Init.InitialOutput then
          y = y_start;
          der(x1) = zeros(m);
          if evenOrder then
            if m > 1 then
              der(x2[1:m - 1]) = zeros(m - 1);
            end if;
          else
            der(x1) = zeros(m);
          end if;
        end if;
      equation
        k2 = ones(m);
        k1 = 1;
        z[1] = u;
// calculate filter parameters
        for i in 1:m loop
// poles of prototype lowpass
          polereal[i] = cos(pi / 2 + pi / n * (i - 0.5));
          poleimag[i] = sin(pi / 2 + pi / n * (i - 0.5));
// scaling and calculation of second order filter coefficients
          w0[i] = (polereal[i] ^ 2 + poleimag[i] ^ 2) * w;
          D[i] = -polereal[i] / w0[i] * w;
        end for;
        realpol = 1 * w;
        T = 1 / realpol;
// calculate second order filters
        for i in 1:m loop
          der(x1[i]) = x2[i];
          der(x2[i]) = k2[i] * w0[i] ^ 2 * z[i] - 2 * D[i] * w0[i] * x2[i] - w0[i] ^ 2 * x1[i];
          z[i + 1] = x1[i];
        end for;
// calculate first order filter if necessary
        if evenOrder then
// even order
          xr = 0;
          y = z[m + 1];
        else
// uneven order
          der(xr) = (k1 * z[m + 1] - xr) / T;
          y = xr;
        end if;
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100.0, -100.0}, {100.0, 100.0}}, initialScale = 0.1), graphics = {Line(visible = true, points = {{-80.0, 78.0}, {-80.0, -90.0}}, color = {192, 192, 192}), Polygon(visible = true, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid, points = {{-79.5584, 91.817}, {-87.5584, 69.817}, {-71.5584, 69.817}, {-79.5584, 91.817}}), Line(visible = true, origin = {-1.939, -1.816}, points = {{81.939, 36.056}, {65.362, 36.056}, {14.39, -26.199}, {-29.966, 113.485}, {-65.374, -61.217}, {-78.061, -78.184}}, color = {0, 0, 127}, smooth = Smooth.Bezier), Line(visible = true, points = {{-90.9779, -80.7697}, {81.0221, -80.7697}}, color = {192, 192, 192}), Polygon(visible = true, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid, points = {{91.3375, -79.8233}, {69.3375, -71.8233}, {69.3375, -87.8233}, {91.3375, -79.8233}}), Text(visible = true, lineColor = {192, 192, 192}, extent = {{-45.1735, -68.0}, {92.0, -11.47}}, textString = "LowpassButterworthFilter"), Text(visible = true, extent = {{8.0, -146.0}, {8.0, -106.0}}, textString = "f=%f"), Text(visible = true, lineColor = {192, 192, 192}, extent = {{-2.0, 48.0}, {94.0, 94.0}}, textString = "%n")}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{40, 0}, {-40, 0}}, color = {0, 0, 0}), Text(extent = {{-55, 55}, {55, 5}}, lineColor = {0, 0, 0}, textString = "1"), Text(extent = {{-55, -5}, {55, -55}}, lineColor = {0, 0, 0}, textString = "a(s)"), Rectangle(extent = {{-60, 60}, {60, -60}}, lineColor = {0, 0, 255}), Line(points = {{-100, 0}, {-60, 0}}, color = {0, 0, 255}), Line(points = {{60, 0}, {100, 0}}, color = {0, 0, 255})}),
          Documentation(info = "<html>
<p>
This block defines the transfer function between the input u
and the output y as an n-th order low pass filter with <i>Butterworth</i>
characteristics and cut-off frequency f. It is implemented as
a series of second order filters and a first order filter.
Butterworth filters have the feature that the amplitude at the
cut-off frequency f is 1/sqrt(2) (= 3 dB), i.e., they are
always \"normalized\". Step responses of the Butterworth filter of
different orders are shown in the next figure:
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Blocks/Butterworth.png\"
     alt=\"Butterworth.png\">
</p>

<p>
If transients at the simulation start shall be avoided, the filter
should be initialized in steady state (e.g., using option
initType=Modelica.Blocks.Types.Init.SteadyState).
</p>

</html>"));
      end LowpassButterworth;

      block CriticalDamping "Output the input signal filtered with an n-th order filter with critical damping"
        import Modelica.Blocks.Types.Init;
        extends Modelica.Blocks.Interfaces.SISO;
        parameter Integer n = 2 "Order of filter";
        parameter Modelica.SIunits.Frequency f(start = 1) "Cut-off frequency";
        parameter Boolean normalized = true "= true, if amplitude at f_cut is 3 dB, otherwise unmodified filter";
        parameter Modelica.Blocks.Types.Init initType = Modelica.Blocks.Types.Init.NoInit "Type of initialization (1: no init, 2: steady state, 3: initial state, 4: initial output)" annotation(
          Evaluate = true,
          Dialog(group = "Initialization"));
        parameter Real x_start[n] = zeros(n) "Initial or guess values of states" annotation(
          Dialog(group = "Initialization"));
        parameter Real y_start = 0.0 "Initial value of output (remaining states are in steady state)" annotation(
          Dialog(enable = initType == Init.InitialOutput, group = "Initialization"));
        output Real x[n](start = x_start) "Filter states";
      protected
        parameter Real alpha = if normalized then sqrt(2 ^ (1 / n) - 1) else 1.0 "Frequency correction factor for normalized filter";
        parameter Real w = 2 * Modelica.Constants.pi * f / alpha;
      initial equation
        if initType == Init.SteadyState then
          der(x) = zeros(n);
        elseif initType == Init.InitialState then
          x = x_start;
        elseif initType == Init.InitialOutput then
          y = y_start;
          der(x[1:n - 1]) = zeros(n - 1);
        end if;
      equation
        der(x[1]) = (u - x[1]) * w;
        for i in 2:n loop
          der(x[i]) = (x[i - 1] - x[i]) * w;
        end for;
        y = x[n];
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100.0, -100.0}, {100.0, 100.0}}, initialScale = 0.1), graphics = {Line(visible = true, points = {{-80.6897, 77.6256}, {-80.6897, -90.3744}}, color = {192, 192, 192}), Polygon(visible = true, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid, points = {{-79.7044, 90.6305}, {-87.7044, 68.6305}, {-71.7044, 68.6305}, {-79.7044, 90.6305}}), Line(visible = true, points = {{-90.0, -80.0}, {82.0, -80.0}}, color = {192, 192, 192}), Polygon(visible = true, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid, points = {{90.0, -80.0}, {68.0, -72.0}, {68.0, -88.0}, {90.0, -80.0}}), Text(visible = true, lineColor = {192, 192, 192}, extent = {{0.0, -60.0}, {60.0, 0.0}}, textString = "PTn"), Line(visible = true, origin = {-17.976, -6.521}, points = {{96.962, 55.158}, {16.42, 50.489}, {-18.988, 18.583}, {-32.024, -53.479}, {-62.024, -73.479}}, color = {0, 0, 127}, smooth = Smooth.Bezier), Text(visible = true, lineColor = {192, 192, 192}, extent = {{-70.0, 48.0}, {26.0, 94.0}}, textString = "%n"), Text(visible = true, extent = {{8.0, -146.0}, {8.0, -106.0}}, textString = "f=%f")}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{40, 0}, {-40, 0}}, color = {0, 0, 0}), Text(extent = {{-55, 55}, {55, 5}}, lineColor = {0, 0, 0}, textString = "1"), Rectangle(extent = {{-60, 60}, {60, -60}}, lineColor = {0, 0, 255}), Line(points = {{-100, 0}, {-60, 0}}, color = {0, 0, 255}), Line(points = {{60, 0}, {100, 0}}, color = {0, 0, 255}), Text(extent = {{-54, -6}, {44, -56}}, lineColor = {0, 0, 0}, textString = "(s/w + 1)"), Text(extent = {{38, -10}, {58, -30}}, lineColor = {0, 0, 0}, textString = "n")}),
          Documentation(info = "<html>
<p>This block defines the transfer function between the
input u and the output y
as an n-th order filter with <i>critical damping</i>
characteristics and cut-off frequency f. It is
implemented as a series of first order filters.
This filter type is especially useful to filter the input of an
inverse model, since the filter does not introduce any transients.
</p>

<p>
If parameter <b>normalized</b> = <b>true</b> (default), the filter
is normalized such that the amplitude of the filter transfer function
at the cut-off frequency f is 1/sqrt(2) (= 3 dB). Otherwise, the filter
is not normalized, i.e., it is unmodified. A normalized filter is usually
much better for applications, since filters of different orders are
\"comparable\", whereas non-normalized filters usually require to adapt the
cut-off frequency, when the order of the filter is changed.
Figures of the filter step responses are shown below.
Note, in versions before version 3.0 of the Modelica Standard library,
the CriticalDamping filter was provided only in non-normalized form.
</p>

<p>If transients at the simulation start shall be avoided, the filter
should be initialized in steady state (e.g., using option
initType=Modelica.Blocks.Types.Init.SteadyState).
</p>

<p>
The critical damping filter is defined as
</p>

<pre>
    &alpha; = <b>if</b> normalized <b>then</b> <b>sqrt</b>(2^(1/n) - 1) <b>else</b> 1 // frequency correction factor
    &omega; = 2*&pi;*f/&alpha;
              1
    y = ------------- * u
         (s/w + 1)^n

</pre>

<p>
<img src=\"modelica://Modelica/Resources/Images/Blocks/CriticalDampingNormalized.png\"
     alt=\"CriticalDampingNormalized.png\">
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Blocks/CriticalDampingNonNormalized.png\"
     alt=\"CriticalDampingNonNormalized.png\">
</p>

</html>"));
      end CriticalDamping;

      block Filter "Continuous low pass, high pass, band pass or band stop IIR-filter of type CriticalDamping, Bessel, Butterworth or ChebyshevI"
        import Modelica.Blocks.Continuous.Internal;
        extends Modelica.Blocks.Interfaces.SISO;
        parameter Modelica.Blocks.Types.AnalogFilter analogFilter = Modelica.Blocks.Types.AnalogFilter.CriticalDamping "Analog filter characteristics (CriticalDamping/Bessel/Butterworth/ChebyshevI)";
        parameter Modelica.Blocks.Types.FilterType filterType = Modelica.Blocks.Types.FilterType.LowPass "Type of filter (LowPass/HighPass/BandPass/BandStop)";
        parameter Integer order(min = 1) = 2 "Order of filter";
        parameter Modelica.SIunits.Frequency f_cut "Cut-off frequency";
        parameter Real gain = 1.0 "Gain (= amplitude of frequency response at zero frequency)";
        parameter Real A_ripple(unit = "dB") = 0.5 "Pass band ripple for Chebyshev filter (otherwise not used); > 0 required" annotation(
          Dialog(enable = analogFilter == Modelica.Blocks.Types.AnalogFilter.ChebyshevI));
        parameter Modelica.SIunits.Frequency f_min = 0 "Band of band pass/stop filter is f_min (A=-3db*gain) .. f_cut (A=-3db*gain)" annotation(
          Dialog(enable = filterType == Modelica.Blocks.Types.FilterType.BandPass or filterType == Modelica.Blocks.Types.FilterType.BandStop));
        parameter Boolean normalized = true "= true, if amplitude at f_cut = -3db, otherwise unmodified filter";
        parameter Modelica.Blocks.Types.Init init = Modelica.Blocks.Types.Init.SteadyState "Type of initialization (no init/steady state/initial state/initial output)" annotation(
          Evaluate = true,
          Dialog(tab = "Advanced"));
        final parameter Integer nx = if filterType == Modelica.Blocks.Types.FilterType.LowPass or filterType == Modelica.Blocks.Types.FilterType.HighPass then order else 2 * order;
        parameter Real x_start[nx] = zeros(nx) "Initial or guess values of states" annotation(
          Dialog(tab = "Advanced"));
        parameter Real y_start = 0 "Initial value of output" annotation(
          Dialog(tab = "Advanced"));
        parameter Real u_nominal = 1.0 "Nominal value of input (used for scaling the states)" annotation(
          Dialog(tab = "Advanced"));
        Modelica.Blocks.Interfaces.RealOutput x[nx] "Filter states";
      protected
        parameter Integer ncr = if analogFilter == Modelica.Blocks.Types.AnalogFilter.CriticalDamping then order else mod(order, 2);
        parameter Integer nc0 = if analogFilter == Modelica.Blocks.Types.AnalogFilter.CriticalDamping then 0 else integer(order / 2);
        parameter Integer na = if filterType == Modelica.Blocks.Types.FilterType.BandPass or filterType == Modelica.Blocks.Types.FilterType.BandStop then order else if analogFilter == Modelica.Blocks.Types.AnalogFilter.CriticalDamping then 0 else integer(order / 2);
        parameter Integer nr = if filterType == Modelica.Blocks.Types.FilterType.BandPass or filterType == Modelica.Blocks.Types.FilterType.BandStop then 0 else if analogFilter == Modelica.Blocks.Types.AnalogFilter.CriticalDamping then order else mod(order, 2);
        // Coefficients of prototype base filter (low pass filter with w_cut = 1 rad/s)
        parameter Real cr[ncr](each fixed = false);
        parameter Real c0[nc0](each fixed = false);
        parameter Real c1[nc0](each fixed = false);
        // Coefficients for differential equations.
        parameter Real r[nr](each fixed = false);
        parameter Real a[na](each fixed = false);
        parameter Real b[na](each fixed = false);
        parameter Real ku[na](each fixed = false);
        parameter Real k1[if filterType == Modelica.Blocks.Types.FilterType.LowPass then 0 else na](each fixed = false);
        parameter Real k2[if filterType == Modelica.Blocks.Types.FilterType.LowPass then 0 else na](each fixed = false);
        // Auxiliary variables
        Real uu[na + nr + 1];
      initial equation
        if analogFilter == Modelica.Blocks.Types.AnalogFilter.CriticalDamping then
          cr = Internal.Filter.base.CriticalDamping(order, normalized);
        elseif analogFilter == Modelica.Blocks.Types.AnalogFilter.Bessel then
          (cr, c0, c1) = Internal.Filter.base.Bessel(order, normalized);
        elseif analogFilter == Modelica.Blocks.Types.AnalogFilter.Butterworth then
          (cr, c0, c1) = Internal.Filter.base.Butterworth(order, normalized);
        elseif analogFilter == Modelica.Blocks.Types.AnalogFilter.ChebyshevI then
          (cr, c0, c1) = Internal.Filter.base.ChebyshevI(order, A_ripple, normalized);
        end if;
        if filterType == Modelica.Blocks.Types.FilterType.LowPass then
          (r, a, b, ku) = Internal.Filter.roots.lowPass(cr, c0, c1, f_cut);
        elseif filterType == Modelica.Blocks.Types.FilterType.HighPass then
          (r, a, b, ku, k1, k2) = Internal.Filter.roots.highPass(cr, c0, c1, f_cut);
        elseif filterType == Modelica.Blocks.Types.FilterType.BandPass then
          (a, b, ku, k1, k2) = Internal.Filter.roots.bandPass(cr, c0, c1, f_min, f_cut);
        elseif filterType == Modelica.Blocks.Types.FilterType.BandStop then
          (a, b, ku, k1, k2) = Internal.Filter.roots.bandStop(cr, c0, c1, f_min, f_cut);
        end if;
        if init == Modelica.Blocks.Types.Init.InitialState then
          x = x_start;
        elseif init == Modelica.Blocks.Types.Init.SteadyState then
          der(x) = zeros(nx);
        elseif init == Modelica.Blocks.Types.Init.InitialOutput then
          y = y_start;
          if nx > 1 then
            der(x[1:nx - 1]) = zeros(nx - 1);
          end if;
        end if;
      equation
        assert(u_nominal > 0, "u_nominal > 0 required");
        assert(filterType == Modelica.Blocks.Types.FilterType.LowPass or filterType == Modelica.Blocks.Types.FilterType.HighPass or f_min > 0, "f_min > 0 required for band pass and band stop filter");
        assert(A_ripple > 0, "A_ripple > 0 required");
        assert(f_cut > 0, "f_cut > 0 required");
/* All filters have the same basic differential equations:
        Real poles:
           der(x) = r*x - r*u
        Complex conjugate poles:
           der(x1) = a*x1 - b*x2 + ku*u;
           der(x2) = b*x1 + a*x2;
   */
        uu[1] = u / u_nominal;
        for i in 1:nr loop
          der(x[i]) = r[i] * (x[i] - uu[i]);
        end for;
        for i in 1:na loop
          der(x[nr + 2 * i - 1]) = a[i] * x[nr + 2 * i - 1] - b[i] * x[nr + 2 * i] + ku[i] * uu[nr + i];
          der(x[nr + 2 * i]) = b[i] * x[nr + 2 * i - 1] + a[i] * x[nr + 2 * i];
        end for;
// The output equation is different for the different filter types
        if filterType == Modelica.Blocks.Types.FilterType.LowPass then
/* Low pass filter
           Real poles             :  y = x
           Complex conjugate poles:  y = x2
      */
          for i in 1:nr loop
            uu[i + 1] = x[i];
          end for;
          for i in 1:na loop
            uu[nr + i + 1] = x[nr + 2 * i];
          end for;
        elseif filterType == Modelica.Blocks.Types.FilterType.HighPass then
/* High pass filter
           Real poles             :  y = -x + u;
           Complex conjugate poles:  y = k1*x1 + k2*x2 + u;
      */
          for i in 1:nr loop
            uu[i + 1] = (-x[i]) + uu[i];
          end for;
          for i in 1:na loop
            uu[nr + i + 1] = k1[i] * x[nr + 2 * i - 1] + k2[i] * x[nr + 2 * i] + uu[nr + i];
          end for;
        elseif filterType == Modelica.Blocks.Types.FilterType.BandPass then
/* Band pass filter
           Complex conjugate poles:  y = k1*x1 + k2*x2;
      */
          for i in 1:na loop
            uu[nr + i + 1] = k1[i] * x[nr + 2 * i - 1] + k2[i] * x[nr + 2 * i];
          end for;
        elseif filterType == Modelica.Blocks.Types.FilterType.BandStop then
/* Band pass filter
           Complex conjugate poles:  y = k1*x1 + k2*x2 + u;
      */
          for i in 1:na loop
            uu[nr + i + 1] = k1[i] * x[nr + 2 * i - 1] + k2[i] * x[nr + 2 * i] + uu[nr + i];
          end for;
        else
          assert(false, "filterType (= " + String(filterType) + ") is unknown");
          uu = zeros(na + nr + 1);
        end if;
        y = gain * u_nominal * uu[nr + na + 1];
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100.0, -100.0}, {100.0, 100.0}}, initialScale = 0.1), graphics = {Line(visible = true, points = {{-80.0, 80.0}, {-80.0, -88.0}}, color = {192, 192, 192}), Polygon(visible = true, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid, points = {{-80.0, 92.0}, {-88.0, 70.0}, {-72.0, 70.0}, {-80.0, 92.0}}), Line(visible = true, points = {{-90.0, -78.0}, {82.0, -78.0}}, color = {192, 192, 192}), Polygon(visible = true, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid, points = {{90.0, -78.0}, {68.0, -70.0}, {68.0, -86.0}, {90.0, -78.0}}), Text(visible = true, lineColor = {192, 192, 192}, extent = {{-66.0, 52.0}, {88.0, 90.0}}, textString = "%order"), Text(visible = true, fillPattern = FillPattern.Solid, extent = {{-138.0, -140.0}, {162.0, -110.0}}, textString = "f_cut=%f_cut"), Rectangle(visible = true, lineColor = {160, 160, 164}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Backward, extent = {{-80.0, -78.0}, {22.0, 10.0}}), Line(visible = true, origin = {3.333, -6.667}, points = {{-83.333, 34.667}, {24.667, 34.667}, {42.667, -71.333}}, color = {0, 0, 127}, smooth = Smooth.Bezier)}),
          Documentation(info = "<html>

<p>
This blocks models various types of filters:
</p>

<blockquote>
<b>low pass, high pass, band pass, and band stop filters</b>
</blockquote>

<p>
using various filter characteristics:
</p>

<blockquote>
<b>CriticalDamping, Bessel, Butterworth, Chebyshev Type I filters</b>
</blockquote>

<p>
By default, a filter block is initialized in <b>steady-state</b>, in order to
avoid unwanted oscillations at the beginning. In special cases, it might be
useful to select one of the other initialization options under tab
\"Advanced\".
</p>

<p>
Typical frequency responses for the 4 supported low pass filter types
are shown in the next figure:
</p>

<blockquote>
<img src=\"modelica://Modelica/Resources/Images/Blocks/LowPassOrder4Filters.png\"
     alt=\"LowPassOrder4Filters.png\">
</blockquote>

<p>
The step responses of the same low pass filters are shown in the next figure,
starting from a steady state initial filter with initial input = 0.2:
</p>

<blockquote>
<img src=\"modelica://Modelica/Resources/Images/Blocks/LowPassOrder4FiltersStepResponse.png\"
     alt=\"LowPassOrder4FiltersStepResponse.png\">
</blockquote>

<p>
Obviously, the frequency responses give a somewhat wrong impression
of the filter characteristics: Although Butterworth and Chebyshev
filters have a significantly steeper magnitude as the
CriticalDamping and Bessel filters, the step responses of
the latter ones are much better. This means for example, that
a CriticalDamping or a Bessel filter should be selected,
if a filter is mainly used to make a non-linear inverse model
realizable.
</p>

<p>
Typical frequency responses for the 4 supported high pass filter types
are shown in the next figure:
</p>

<blockquote>
<img src=\"modelica://Modelica/Resources/Images/Blocks/HighPassOrder4Filters.png\"
     alt=\"HighPassOrder4Filters.png\">
</blockquote>

<p>
The corresponding step responses of these high pass filters are
shown in the next figure:
</p>
<blockquote>
<img src=\"modelica://Modelica/Resources/Images/Blocks/HighPassOrder4FiltersStepResponse.png\"
     alt=\"HighPassOrder4FiltersStepResponse.png\">
</blockquote>

<p>
All filters are available in <b>normalized</b> (default) and non-normalized form.
In the normalized form, the amplitude of the filter transfer function
at the cut-off frequency f_cut is -3 dB (= 10^(-3/20) = 0.70794..).
Note, when comparing the filters of this function with other software systems,
the setting of \"normalized\" has to be selected appropriately. For example, the signal processing
toolbox of Matlab provides the filters in non-normalized form and
therefore a comparison makes only sense, if normalized = <b>false</b>
is set. A normalized filter is usually better suited for applications,
since filters of different orders are \"comparable\",
whereas non-normalized filters usually require to adapt the
cut-off frequency, when the order of the filter is changed.
See a comparison of \"normalized\" and \"non-normalized\" filters at hand of
CriticalDamping filters of order 1,2,3:
</p>

<blockquote>
<img src=\"modelica://Modelica/Resources/Images/Blocks/CriticalDampingNormalized.png\"
     alt=\"CriticalDampingNormalized.png\">
</blockquote>

<blockquote>
<img src=\"modelica://Modelica/Resources/Images/Blocks/CriticalDampingNonNormalized.png\"
     alt=\"CriticalDampingNonNormalized.png\">
</blockquote>

<h4>Implementation</h4>

<p>
The filters are implemented in the following, reliable way:
</p>

<ol>
<li> A prototype low pass filter with a cut-off angular frequency of 1 rad/s is constructed
     from the desired analogFilter and the desired normalization.</li>

<li> This prototype low pass filter is transformed to the desired filterType and the
     desired cut-off frequency f_cut using a transformation on the Laplace variable \"s\".</li>

<li> The resulting first and second order transfer functions are implemented in
     state space form, using the \"eigen value\" representation of a transfer function:
     <pre>

  // second order block with eigen values: a +/- jb
  <b>der</b>(x1) = a*x1 - b*x2 + (a^2 + b^2)/b*u;
  <b>der</b>(x2) = b*x1 + a*x2;
       y  = x2;
     </pre>
     The dc-gain from the input to the output of this block is one and the selected
     states are in the order of the input (if \"u\" is in the order of \"one\", then the
     states are also in the order of \"one\"). In the \"Advanced\" tab, a \"nominal\" value for
     the input \"u\" can be given. If appropriately selected, the states are in the order of \"one\" and
     then step-size control is always appropriate.</li>
</ol>

<h4>References</h4>

<dl>
<dt>Tietze U., and Schenk C. (2002):</dt>
<dd> <b>Halbleiter-Schaltungstechnik</b>.
     Springer Verlag, 12. Auflage, pp. 815-852.</dd>
</dl>

</html>", revisions = "<html>
<dl>
  <dt><b>Main Author:</b></dt>
  <dd><a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a>,
      DLR Oberpfaffenhofen.</dd>
</dl>

<h4>Acknowledgement</h4>

<p>
The development of this block was partially funded by BMBF within the
     <a href=\"http://www.eurosyslib.com/\">ITEA2 EUROSYSLIB</a>
      project.
</p>

</html>"));
      end Filter;

      package Internal "Internal utility functions and blocks that should not be directly utilized by the user"
        extends Modelica.Icons.InternalPackage;

        package Filter "Internal utility functions for filters that should not be directly used"
          extends Modelica.Icons.InternalPackage;

          package base "Prototype low pass filters with cut-off frequency of 1 rad/s (other filters are derived by transformation from these base filters)"
            extends Modelica.Icons.InternalPackage;

            function CriticalDamping "Return base filter coefficients of CriticalDamping filter (= low pass filter with w_cut = 1 rad/s)"
              extends Modelica.Icons.Function;
              input Integer order(min = 1) "Order of filter";
              input Boolean normalized = true "= true, if amplitude at f_cut = -3db, otherwise unmodified filter";
              output Real cr[order] "Coefficients of real poles";
            protected
              Real alpha = 1.0 "Frequency correction factor";
              Real alpha2 "= alpha*alpha";
              Real den1[order] "[p] coefficients of denominator first order polynomials (a*p + 1)";
              Real den2[0, 2] "[p^2, p] coefficients of denominator second order polynomials (b*p^2 + a*p + 1)";
              Real c0[0] "Coefficients of s^0 term if conjugate complex pole";
              Real c1[0] "Coefficients of s^1 term if conjugate complex pole";
            algorithm
              if normalized then
                alpha := sqrt(10 ^ (3 / 10 / order) - 1);
              else
                alpha := 1.0;
              end if;
// alpha := sqrt(2^(1/order) - 1);
              for i in 1:order loop
                den1[i] := alpha;
              end for;
// Determine polynomials with highest power of s equal to one
              (cr, c0, c1) := Modelica.Blocks.Continuous.Internal.Filter.Utilities.toHighestPowerOne(den1, den2);
            end CriticalDamping;

            function Bessel "Return base filter coefficients of Bessel filter (= low pass filter with w_cut = 1 rad/s)"
              extends Modelica.Icons.Function;
              input Integer order(min = 1) "Order of filter";
              input Boolean normalized = true "= true, if amplitude at f_cut = -3db, otherwise unmodified filter";
              output Real cr[mod(order, 2)] "Coefficient of real pole";
              output Real c0[integer(order / 2)] "Coefficients of s^0 term if conjugate complex pole";
              output Real c1[integer(order / 2)] "Coefficients of s^1 term if conjugate complex pole";
            protected
              Real alpha = 1.0 "Frequency correction factor";
              Real alpha2 "= alpha*alpha";
              Real den1[size(cr, 1)] "[p] coefficients of denominator first order polynomials (a*p + 1)";
              Real den2[size(c0, 1), 2] "[p^2, p] coefficients of denominator second order polynomials (b*p^2 + a*p + 1)";
            algorithm
              (den1, den2, alpha) := Modelica.Blocks.Continuous.Internal.Filter.Utilities.BesselBaseCoefficients(order);
              if not normalized then
                alpha2 := alpha * alpha;
                for i in 1:size(c0, 1) loop
                  den2[i, 1] := den2[i, 1] * alpha2;
                  den2[i, 2] := den2[i, 2] * alpha;
                end for;
                if size(cr, 1) == 1 then
                  den1[1] := den1[1] * alpha;
                end if;
              end if;
// Determine polynomials with highest power of s equal to one
              (cr, c0, c1) := Modelica.Blocks.Continuous.Internal.Filter.Utilities.toHighestPowerOne(den1, den2);
            end Bessel;

            function Butterworth "Return base filter coefficients of Butterworth filter (= low pass filter with w_cut = 1 rad/s)"
              extends Modelica.Icons.Function;
              input Integer order(min = 1) "Order of filter";
              input Boolean normalized = true "= true, if amplitude at f_cut = -3db, otherwise unmodified filter";
              output Real cr[mod(order, 2)] "Coefficient of real pole";
              output Real c0[integer(order / 2)] "Coefficients of s^0 term if conjugate complex pole";
              output Real c1[integer(order / 2)] "Coefficients of s^1 term if conjugate complex pole";
            protected
              Real alpha = 1.0 "Frequency correction factor";
              Real alpha2 "= alpha*alpha";
              Real den1[size(cr, 1)] "[p] coefficients of denominator first order polynomials (a*p + 1)";
              Real den2[size(c0, 1), 2] "[p^2, p] coefficients of denominator second order polynomials (b*p^2 + a*p + 1)";
              constant Real pi = Modelica.Constants.pi;
            algorithm
              for i in 1:size(c0, 1) loop
                den2[i, 1] := 1.0;
                den2[i, 2] := -2 * Modelica.Math.cos(pi * (0.5 + (i - 0.5) / order));
              end for;
              if size(cr, 1) == 1 then
                den1[1] := 1.0;
              end if;
/* Transformation of filter transfer function with "new(p) = alpha*p"
     in order that the filter transfer function has an amplitude of
     -3 db at the cutoff frequency
  */
/*
    if normalized then
      alpha := Internal.normalizationFactor(den1, den2);
      alpha2 := alpha*alpha;
      for i in 1:size(c0, 1) loop
        den2[i, 1] := den2[i, 1]*alpha2;
        den2[i, 2] := den2[i, 2]*alpha;
      end for;
      if size(cr,1) == 1 then
        den1[1] := den1[1]*alpha;
      end if;
    end if;
  */
// Determine polynomials with highest power of s equal to one
              (cr, c0, c1) := Modelica.Blocks.Continuous.Internal.Filter.Utilities.toHighestPowerOne(den1, den2);
            end Butterworth;

            function ChebyshevI "Return base filter coefficients of Chebyshev I filter (= low pass filter with w_cut = 1 rad/s)"
              import Modelica.Math.asinh;
              extends Modelica.Icons.Function;
              input Integer order(min = 1) "Order of filter";
              input Real A_ripple = 0.5 "Pass band ripple in [dB]";
              input Boolean normalized = true "= true, if amplitude at f_cut = -3db, otherwise unmodified filter";
              output Real cr[mod(order, 2)] "Coefficient of real pole";
              output Real c0[integer(order / 2)] "Coefficients of s^0 term if conjugate complex pole";
              output Real c1[integer(order / 2)] "Coefficients of s^1 term if conjugate complex pole";
            protected
              Real epsilon;
              Real fac;
              Real alpha = 1.0 "Frequency correction factor";
              Real alpha2 "= alpha*alpha";
              Real den1[size(cr, 1)] "[p] coefficients of denominator first order polynomials (a*p + 1)";
              Real den2[size(c0, 1), 2] "[p^2, p] coefficients of denominator second order polynomials (b*p^2 + a*p + 1)";
              constant Real pi = Modelica.Constants.pi;
            algorithm
              epsilon := sqrt(10 ^ (A_ripple / 10) - 1);
              fac := asinh(1 / epsilon) / order;
              den1 := fill(1 / sinh(fac), size(den1, 1));
              if size(cr, 1) == 0 then
                for i in 1:size(c0, 1) loop
                  den2[i, 1] := 1 / (cosh(fac) ^ 2 - cos((2 * i - 1) * pi / (2 * order)) ^ 2);
                  den2[i, 2] := 2 * den2[i, 1] * sinh(fac) * cos((2 * i - 1) * pi / (2 * order));
                end for;
              else
                for i in 1:size(c0, 1) loop
                  den2[i, 1] := 1 / (cosh(fac) ^ 2 - cos(i * pi / order) ^ 2);
                  den2[i, 2] := 2 * den2[i, 1] * sinh(fac) * cos(i * pi / order);
                end for;
              end if;
/* Transformation of filter transfer function with "new(p) = alpha*p"
       in order that the filter transfer function has an amplitude of
       -3 db at the cutoff frequency
    */
              if normalized then
                alpha := Modelica.Blocks.Continuous.Internal.Filter.Utilities.normalizationFactor(den1, den2);
                alpha2 := alpha * alpha;
                for i in 1:size(c0, 1) loop
                  den2[i, 1] := den2[i, 1] * alpha2;
                  den2[i, 2] := den2[i, 2] * alpha;
                end for;
                den1 := den1 * alpha;
              end if;
// Determine polynomials with highest power of s equal to one
              (cr, c0, c1) := Modelica.Blocks.Continuous.Internal.Filter.Utilities.toHighestPowerOne(den1, den2);
            end ChebyshevI;
          end base;

          package coefficients "Filter coefficients"
            extends Modelica.Icons.InternalPackage;

            function lowPass "Return low pass filter coefficients at given cut-off frequency"
              extends Modelica.Icons.Function;
              input Real cr_in[:] "Coefficients of real poles";
              input Real c0_in[:] "Coefficients of s^0 term if conjugate complex pole";
              input Real c1_in[size(c0_in, 1)] "Coefficients of s^1 term if conjugate complex pole";
              input Modelica.SIunits.Frequency f_cut "Cut-off frequency";
              output Real cr[size(cr_in, 1)] "Coefficient of real pole";
              output Real c0[size(c0_in, 1)] "Coefficients of s^0 term if conjugate complex pole";
              output Real c1[size(c0_in, 1)] "Coefficients of s^1 term if conjugate complex pole";
            protected
              constant Real pi = Modelica.Constants.pi;
              Modelica.SIunits.AngularVelocity w_cut = 2 * pi * f_cut "Cut-off angular frequency";
              Real w_cut2 = w_cut * w_cut;
            algorithm
              assert(f_cut > 0, "Cut-off frequency f_cut must be positive");
/* Change filter coefficients according to transformation new(s) = s/w_cut
     s + cr           -> (s/w) + cr              = (s + w*cr)/w
     s^2 + c1*s + c0  -> (s/w)^2 + c1*(s/w) + c0 = (s^2 + (c1*w)*s + (c0*w^2))/w^2
  */
              cr := w_cut * cr_in;
              c1 := w_cut * c1_in;
              c0 := w_cut2 * c0_in;
            end lowPass;

            function highPass "Return high pass filter coefficients at given cut-off frequency"
              extends Modelica.Icons.Function;
              input Real cr_in[:] "Coefficients of real poles";
              input Real c0_in[:] "Coefficients of s^0 term if conjugate complex pole";
              input Real c1_in[size(c0_in, 1)] "Coefficients of s^1 term if conjugate complex pole";
              input Modelica.SIunits.Frequency f_cut "Cut-off frequency";
              output Real cr[size(cr_in, 1)] "Coefficient of real pole";
              output Real c0[size(c0_in, 1)] "Coefficients of s^0 term if conjugate complex pole";
              output Real c1[size(c0_in, 1)] "Coefficients of s^1 term if conjugate complex pole";
            protected
              constant Real pi = Modelica.Constants.pi;
              Modelica.SIunits.AngularVelocity w_cut = 2 * pi * f_cut "Cut-off angular frequency";
              Real w_cut2 = w_cut * w_cut;
            algorithm
              assert(f_cut > 0, "Cut-off frequency f_cut must be positive");
/* Change filter coefficients according to transformation: new(s) = 1/s
        1/(s + cr)          -> 1/(1/s + cr)                = (1/cr)*s / (s + (1/cr))
        1/(s^2 + c1*s + c0) -> 1/((1/s)^2 + c1*(1/s) + c0) = (1/c0)*s^2 / (s^2 + (c1/c0)*s + 1/c0)

     Check whether transformed roots are also conjugate complex:
        c0 - c1^2/4 > 0  -> (1/c0) - (c1/c0)^2 / 4
                            = (c0 - c1^2/4) / c0^2 > 0
        It is therefore guaranteed that the roots remain conjugate complex

     Change filter coefficients according to transformation new(s) = s/w_cut
        s + 1/cr                -> (s/w) + 1/cr                   = (s + w/cr)/w
        s^2 + (c1/c0)*s + 1/c0  -> (s/w)^2 + (c1/c0)*(s/w) + 1/c0 = (s^2 + (w*c1/c0)*s + (w^2/c0))/w^2
  */
              for i in 1:size(cr_in, 1) loop
                cr[i] := w_cut / cr_in[i];
              end for;
              for i in 1:size(c0_in, 1) loop
                c0[i] := w_cut2 / c0_in[i];
                c1[i] := w_cut * c1_in[i] / c0_in[i];
              end for;
            end highPass;

            function bandPass "Return band pass filter coefficients at given cut-off frequency"
              extends Modelica.Icons.Function;
              input Real cr_in[:] "Coefficients of real poles";
              input Real c0_in[:] "Coefficients of s^0 term if conjugate complex pole";
              input Real c1_in[size(c0_in, 1)] "Coefficients of s^1 term if conjugate complex pole";
              input Modelica.SIunits.Frequency f_min "Band of band pass filter is f_min (A=-3db) .. f_max (A=-3db)";
              input Modelica.SIunits.Frequency f_max "Upper band frequency";
              output Real cr[0] "Coefficient of real pole";
              output Real c0[size(cr_in, 1) + 2 * size(c0_in, 1)] "Coefficients of s^0 term if conjugate complex pole";
              output Real c1[size(cr_in, 1) + 2 * size(c0_in, 1)] "Coefficients of s^1 term if conjugate complex pole";
              output Real cn "Numerator coefficient of the PT2 terms";
            protected
              constant Real pi = Modelica.Constants.pi;
              Modelica.SIunits.Frequency f0 = sqrt(f_min * f_max);
              Modelica.SIunits.AngularVelocity w_cut = 2 * pi * f0 "Cut-off angular frequency";
              Real w_band = (f_max - f_min) / f0;
              Real w_cut2 = w_cut * w_cut;
              Real c;
              Real alpha;
              Integer j;
            algorithm
              assert(f_min > 0 and f_min < f_max, "Band frequencies f_min and f_max are wrong");
/* The band pass filter is derived from the low pass filter by
       the transformation new(s) = (s + 1/s)/w   (w = w_band = (f_max - f_min)/sqrt(f_max*f_min) )

       1/(s + cr)         -> 1/((s/w + 1/s/w) + cr)
                             = w*s / (s^2 + cr*w*s + 1)

       1/(s^2 + c1*s + c0) -> 1/( (s+1/s)^2/w^2 + c1*(s + 1/s)/w + c0 )
                              = 1 /( ( s^2 + 1/s^2 + 2)/w^2 + (s + 1/s)*c1/w + c0 )
                              = w^2*s^2 / (s^4 + 2*s^2 + 1 + (s^3 + s)*c1*w + c0*w^2*s^2)
                              = w^2*s^2 / (s^4 + c1*w*s^3 + (2+c0*w^2)*s^2 + c1*w*s + 1)

                              Assume the following description with PT2:
                              = w^2*s^2 /( (s^2 + s*(c/alpha) + 1/alpha^2)*
                                           (s^2 + s*(c*alpha) + alpha^2) )
                              = w^2*s^2 / ( s^4 + c*(alpha + 1/alpha)*s^3
                                                + (alpha^2 + 1/alpha^2 + c^2)*s^2
                                                + c*(alpha + 1/alpha)*s + 1 )

                              and therefore:
                                c*(alpha + 1/alpha) = c1*w       -> c = c1*w / (alpha + 1/alpha)
                                                                      = c1*w*alpha/(1+alpha^2)
                                alpha^2 + 1/alpha^2 + c^2 = 2+c0*w^2 -> equation to determine alpha
                                alpha^4 + 1 + c1^2*w^2*alpha^4/(1+alpha^2)^2 = (2+c0*w^2)*alpha^2
                                or z = alpha^2
                                z^2 + c^1^2*w^2*z^2/(1+z)^2 - (2+c0*w^2)*z + 1 = 0

     Check whether roots remain conjugate complex
        c0 - (c1/2)^2 > 0:    1/alpha^2 - (c/alpha)^2/4
                              = 1/alpha^2*(1 - c^2/4)    -> not possible to figure this out

     Afterwards, change filter coefficients according to transformation new(s) = s/w_cut
        w_band*s/(s^2 + c1*s + c0)  -> w_band*(s/w)/((s/w)^2 + c1*(s/w) + c0 =
                                       (w_band/w)*s/(s^2 + (c1*w)*s + (c0*w^2))/w^2) =
                                       (w_band*w)*s/(s^2 + (c1*w)*s + (c0*w^2))
    */
              for i in 1:size(cr_in, 1) loop
                c1[i] := w_cut * cr_in[i] * w_band;
                c0[i] := w_cut2;
              end for;
              for i in 1:size(c1_in, 1) loop
                alpha := Modelica.Blocks.Continuous.Internal.Filter.Utilities.bandPassAlpha(c1_in[i], c0_in[i], w_band);
                c := c1_in[i] * w_band / (alpha + 1 / alpha);
                j := size(cr_in, 1) + 2 * i - 1;
                c1[j] := w_cut * c / alpha;
                c1[j + 1] := w_cut * c * alpha;
                c0[j] := w_cut2 / alpha ^ 2;
                c0[j + 1] := w_cut2 * alpha ^ 2;
              end for;
              cn := w_band * w_cut;
            end bandPass;

            function bandStop "Return band stop filter coefficients at given cut-off frequency"
              extends Modelica.Icons.Function;
              input Real cr_in[:] "Coefficients of real poles";
              input Real c0_in[:] "Coefficients of s^0 term if conjugate complex pole";
              input Real c1_in[size(c0_in, 1)] "Coefficients of s^1 term if conjugate complex pole";
              input Modelica.SIunits.Frequency f_min "Band of band stop filter is f_min (A=-3db) .. f_max (A=-3db)";
              input Modelica.SIunits.Frequency f_max "Upper band frequency";
              output Real cr[0] "Coefficient of real pole";
              output Real c0[size(cr_in, 1) + 2 * size(c0_in, 1)] "Coefficients of s^0 term if conjugate complex pole";
              output Real c1[size(cr_in, 1) + 2 * size(c0_in, 1)] "Coefficients of s^1 term if conjugate complex pole";
            protected
              constant Real pi = Modelica.Constants.pi;
              Modelica.SIunits.Frequency f0 = sqrt(f_min * f_max);
              Modelica.SIunits.AngularVelocity w_cut = 2 * pi * f0 "Cut-off angular frequency";
              Real w_band = (f_max - f_min) / f0;
              Real w_cut2 = w_cut * w_cut;
              Real c;
              Real ww;
              Real alpha;
              Integer j;
            algorithm
              assert(f_min > 0 and f_min < f_max, "Band frequencies f_min and f_max are wrong");
/* The band pass filter is derived from the low pass filter by
       the transformation new(s) = (s + 1/s)/w   (w = w_band = (f_max - f_min)/sqrt(f_max*f_min) )

       1/(s + cr)         -> 1/((s/w + 1/s/w) + cr)
                             = w*s / (s^2 + cr*w*s + 1)

       1/(s^2 + c1*s + c0) -> 1/( (s+1/s)^2/w^2 + c1*(s + 1/s)/w + c0 )
                              = 1 /( ( s^2 + 1/s^2 + 2)/w^2 + (s + 1/s)*c1/w + c0 )
                              = w^2*s^2 / (s^4 + 2*s^2 + 1 + (s^3 + s)*c1*w + c0*w^2*s^2)
                              = w^2*s^2 / (s^4 + c1*w*s^3 + (2+c0*w^2)*s^2 + c1*w*s + 1)

                              Assume the following description with PT2:
                              = w^2*s^2 /( (s^2 + s*(c/alpha) + 1/alpha^2)*
                                           (s^2 + s*(c*alpha) + alpha^2) )
                              = w^2*s^2 / ( s^4 + c*(alpha + 1/alpha)*s^3
                                                + (alpha^2 + 1/alpha^2 + c^2)*s^2
                                                + c*(alpha + 1/alpha)*s + 1 )

                              and therefore:
                                c*(alpha + 1/alpha) = c1*w       -> c = c1*w / (alpha + 1/alpha)
                                                                      = c1*w*alpha/(1+alpha^2)
                                alpha^2 + 1/alpha^2 + c^2 = 2+c0*w^2 -> equation to determine alpha
                                alpha^4 + 1 + c1^2*w^2*alpha^4/(1+alpha^2)^2 = (2+c0*w^2)*alpha^2
                                or z = alpha^2
                                z^2 + c^1^2*w^2*z^2/(1+z)^2 - (2+c0*w^2)*z + 1 = 0

       The band stop filter is derived from the low pass filter by
       the transformation new(s) = w/( (s + 1/s) )   (w = w_band = (f_max - f_min)/sqrt(f_max*f_min) )

       cr/(s + cr)         -> 1/(( w/(s + 1/s) ) + cr)
                              = (s^2 + 1) / (s^2 + (w/cr)*s + 1)

       c0/(s^2 + c1*s + c0) -> c0/( w^2/(s + 1/s)^2 + c1*w/(s + 1/s) + c0 )
                               = c0*(s^2 + 1)^2 / (s^4 + c1*w*s^3/c0 + (2+w^2/b)*s^2 + c1*w*s/c0 + 1)

                               Assume the following description with PT2:
                               = c0*(s^2 + 1)^2 / ( (s^2 + s*(c/alpha) + 1/alpha^2)*
                                                    (s^2 + s*(c*alpha) + alpha^2) )
                               = c0*(s^2 + 1)^2 / (  s^4 + c*(alpha + 1/alpha)*s^3
                                                         + (alpha^2 + 1/alpha^2 + c^2)*s^2
                                                         + c*(alpha + 1/alpha)*p + 1 )

                            and therefore:
                              c*(alpha + 1/alpha) = c1*w/b         -> c = c1*w/(c0*(alpha + 1/alpha))
                              alpha^2 + 1/alpha^2 + c^2 = 2+w^2/c0 -> equation to determine alpha
                              alpha^4 + 1 + (c1*w/c0*alpha^2)^2/(1+alpha^2)^2 = (2+w^2/c0)*alpha^2
                              or z = alpha^2
                              z^2 + (c1*w/c0*z)^2/(1+z)^2 - (2+w^2/c0)*z + 1 = 0

                            same as:  ww = w/c0
                              z^2 + (c1*ww*z)^2/(1+z)^2 - (2+c0*ww)*z + 1 = 0  -> same equation as for BandPass

     Afterwards, change filter coefficients according to transformation new(s) = s/w_cut
        c0*(s^2+1)(s^2 + c1*s + c0)  -> c0*((s/w)^2 + 1) / ((s/w)^2 + c1*(s/w) + c0 =
                                        c0/w^2*(s^2 + w^2) / (s^2 + (c1*w)*s + (c0*w^2))/w^2) =
                                        (s^2 + c0*w^2) / (s^2 + (c1*w)*s + (c0*w^2))
    */
              for i in 1:size(cr_in, 1) loop
                c1[i] := w_cut * w_band / cr_in[i];
                c0[i] := w_cut2;
              end for;
              for i in 1:size(c1_in, 1) loop
                ww := w_band / c0_in[i];
                alpha := Modelica.Blocks.Continuous.Internal.Filter.Utilities.bandPassAlpha(c1_in[i], c0_in[i], ww);
                c := c1_in[i] * ww / (alpha + 1 / alpha);
                j := size(cr_in, 1) + 2 * i - 1;
                c1[j] := w_cut * c / alpha;
                c1[j + 1] := w_cut * c * alpha;
                c0[j] := w_cut2 / alpha ^ 2;
                c0[j + 1] := w_cut2 * alpha ^ 2;
              end for;
            end bandStop;
          end coefficients;

          package roots "Filter roots and gain as needed for block implementations"
            extends Modelica.Icons.InternalPackage;

            function lowPass "Return low pass filter roots as needed for block for given cut-off frequency"
              extends Modelica.Icons.Function;
              input Real cr_in[:] "Coefficients of real poles of base filter";
              input Real c0_in[:] "Coefficients of s^0 term of base filter if conjugate complex pole";
              input Real c1_in[size(c0_in, 1)] "Coefficients of s^1 term of base filter if conjugate complex pole";
              input Modelica.SIunits.Frequency f_cut "Cut-off frequency";
              output Real r[size(cr_in, 1)] "Real eigenvalues";
              output Real a[size(c0_in, 1)] "Real parts of complex conjugate eigenvalues";
              output Real b[size(c0_in, 1)] "Imaginary parts of complex conjugate eigenvalues";
              output Real ku[size(c0_in, 1)] "Input gain";
            protected
              Real c0[size(c0_in, 1)];
              Real c1[size(c0_in, 1)];
              Real cr[size(cr_in, 1)];
            algorithm
// Get coefficients of low pass filter at f_cut
              (cr, c0, c1) := coefficients.lowPass(cr_in, c0_in, c1_in, f_cut);
// Transform coefficients in to root
              for i in 1:size(cr_in, 1) loop
                r[i] := -cr[i];
              end for;
              for i in 1:size(c0_in, 1) loop
                a[i] := -c1[i] / 2;
                b[i] := sqrt(c0[i] - a[i] * a[i]);
                ku[i] := c0[i] / b[i];
              end for;
              annotation(
                Documentation(info = "<html>

<p>
The goal is to implement the filter in the following form:
</p>

<pre>
  // real pole:
   der(x) = r*x - r*u
       y  = x

  // complex conjugate poles:
  der(x1) = a*x1 - b*x2 + ku*u;
  der(x2) = b*x1 + a*x2;
       y  = x2;

            ku = (a^2 + b^2)/b
</pre>
<p>
This representation has the following transfer function:
</p>
<pre>
// real pole:
    s*y = r*y - r*u
  or
    (s-r)*y = -r*u
  or
    y = -r/(s-r)*u

  comparing coefficients with
    y = cr/(s + cr)*u  ->  r = -cr      // r is the real eigenvalue

// complex conjugate poles
    s*x2 =  a*x2 + b*x1
    s*x1 = -b*x2 + a*x1 + ku*u
  or
    (s-a)*x2               = b*x1  ->  x2 = b/(s-a)*x1
    (s + b^2/(s-a) - a)*x1 = ku*u  ->  (s(s-a) + b^2 - a*(s-a))*x1  = ku*(s-a)*u
                                   ->  (s^2 - 2*a*s + a^2 + b^2)*x1 = ku*(s-a)*u
  or
    x1 = ku*(s-a)/(s^2 - 2*a*s + a^2 + b^2)*u
    x2 = b/(s-a)*ku*(s-a)/(s^2 - 2*a*s + a^2 + b^2)*u
       = b*ku/(s^2 - 2*a*s + a^2 + b^2)*u
    y  = x2

  comparing coefficients with
    y = c0/(s^2 + c1*s + c0)*u  ->  a  = -c1/2
                                    b  = sqrt(c0 - a^2)
                                    ku = c0/b
                                       = (a^2 + b^2)/b

  comparing with eigenvalue representation:
    (s - (a+jb))*(s - (a-jb)) = s^2 -2*a*s + a^2 + b^2
  shows that:
    a: real part of eigenvalue
    b: imaginary part of eigenvalue

  time -> infinity:
    y(s=0) = x2(s=0) = 1
             x1(s=0) = -ku*a/(a^2 + b^2)*u
                     = -(a/b)*u
</pre>

</html>"));
            end lowPass;

            function highPass "Return high pass filter roots as needed for block for given cut-off frequency"
              extends Modelica.Icons.Function;
              input Real cr_in[:] "Coefficients of real poles of base filter";
              input Real c0_in[:] "Coefficients of s^0 term of base filter if conjugate complex pole";
              input Real c1_in[size(c0_in, 1)] "Coefficients of s^1 term of base filter if conjugate complex pole";
              input Modelica.SIunits.Frequency f_cut "Cut-off frequency";
              output Real r[size(cr_in, 1)] "Real eigenvalues";
              output Real a[size(c0_in, 1)] "Real parts of complex conjugate eigenvalues";
              output Real b[size(c0_in, 1)] "Imaginary parts of complex conjugate eigenvalues";
              output Real ku[size(c0_in, 1)] "Gains of input terms";
              output Real k1[size(c0_in, 1)] "Gains of y = k1*x1 + k2*x + u";
              output Real k2[size(c0_in, 1)] "Gains of y = k1*x1 + k2*x + u";
            protected
              Real c0[size(c0_in, 1)];
              Real c1[size(c0_in, 1)];
              Real cr[size(cr_in, 1)];
              Real ba2;
            algorithm
// Get coefficients of high pass filter at f_cut
              (cr, c0, c1) := coefficients.highPass(cr_in, c0_in, c1_in, f_cut);
// Transform coefficients in to roots
              for i in 1:size(cr_in, 1) loop
                r[i] := -cr[i];
              end for;
              for i in 1:size(c0_in, 1) loop
                a[i] := -c1[i] / 2;
                b[i] := sqrt(c0[i] - a[i] * a[i]);
                ku[i] := c0[i] / b[i];
                k1[i] := 2 * a[i] / ku[i];
                ba2 := (b[i] / a[i]) ^ 2;
                k2[i] := (1 - ba2) / (1 + ba2);
              end for;
              annotation(
                Documentation(info = "<html>

<p>
The goal is to implement the filter in the following form:
</p>

<pre>
  // real pole:
   der(x) = r*x - r*u
       y  = -x + u

  // complex conjugate poles:
  der(x1) = a*x1 - b*x2 + ku*u;
  der(x2) = b*x1 + a*x2;
       y  = k1*x1 + k2*x2 + u;

            ku = (a^2 + b^2)/b
            k1 = 2*a/ku
            k2 = (a^2 - b^2) / (b*ku)
               = (a^2 - b^2) / (a^2 + b^2)
               = (1 - (b/a)^2) / (1 + (b/a)^2)

</pre>
<p>
This representation has the following transfer function:
</p>
<pre>
// real pole:
    s*x = r*x - r*u
  or
    (s-r)*x = -r*u   -> x = -r/(s-r)*u
  or
    y = r/(s-r)*u + (s-r)/(s-r)*u
      = (r+s-r)/(s-r)*u
      = s/(s-r)*u

  comparing coefficients with
    y = s/(s + cr)*u  ->  r = -cr      // r is the real eigenvalue

// complex conjugate poles
    s*x2 =  a*x2 + b*x1
    s*x1 = -b*x2 + a*x1 + ku*u
  or
    (s-a)*x2               = b*x1  ->  x2 = b/(s-a)*x1
    (s + b^2/(s-a) - a)*x1 = ku*u  ->  (s(s-a) + b^2 - a*(s-a))*x1  = ku*(s-a)*u
                                   ->  (s^2 - 2*a*s + a^2 + b^2)*x1 = ku*(s-a)*u
  or
    x1 = ku*(s-a)/(s^2 - 2*a*s + a^2 + b^2)*u
    x2 = b/(s-a)*ku*(s-a)/(s^2 - 2*a*s + a^2 + b^2)*u
       = b*ku/(s^2 - 2*a*s + a^2 + b^2)*u
    y  = k1*x1 + k2*x2 + u
       = (k1*ku*(s-a) + k2*b*ku +  s^2 - 2*a*s + a^2 + b^2) /
         (s^2 - 2*a*s + a^2 + b^2)*u
       = (s^2 + (k1*ku - 2*a)*s + k2*b*ku - k1*ku*a + a^2 + b^2) /
         (s^2 - 2*a*s + a^2 + b^2)*u
       = (s^2 + (2*a-2*a)*s + a^2 - b^2 - 2*a^2 + a^2 + b^2) /
         (s^2 - 2*a*s + a^2 + b^2)*u
       = s^2 / (s^2 - 2*a*s + a^2 + b^2)*u

  comparing coefficients with
    y = s^2/(s^2 + c1*s + c0)*u  ->  a = -c1/2
                                     b = sqrt(c0 - a^2)

  comparing with eigenvalue representation:
    (s - (a+jb))*(s - (a-jb)) = s^2 -2*a*s + a^2 + b^2
  shows that:
    a: real part of eigenvalue
    b: imaginary part of eigenvalue
</pre>

</html>"));
            end highPass;

            function bandPass "Return band pass filter roots as needed for block for given cut-off frequency"
              extends Modelica.Icons.Function;
              input Real cr_in[:] "Coefficients of real poles of base filter";
              input Real c0_in[:] "Coefficients of s^0 term of base filter if conjugate complex pole";
              input Real c1_in[size(c0_in, 1)] "Coefficients of s^1 term of base filter if conjugate complex pole";
              input Modelica.SIunits.Frequency f_min "Band of band pass filter is f_min (A=-3db) .. f_max (A=-3db)";
              input Modelica.SIunits.Frequency f_max "Upper band frequency";
              output Real a[size(cr_in, 1) + 2 * size(c0_in, 1)] "Real parts of complex conjugate eigenvalues";
              output Real b[size(cr_in, 1) + 2 * size(c0_in, 1)] "Imaginary parts of complex conjugate eigenvalues";
              output Real ku[size(cr_in, 1) + 2 * size(c0_in, 1)] "Gains of input terms";
              output Real k1[size(cr_in, 1) + 2 * size(c0_in, 1)] "Gains of y = k1*x1 + k2*x";
              output Real k2[size(cr_in, 1) + 2 * size(c0_in, 1)] "Gains of y = k1*x1 + k2*x";
            protected
              Real cr[0];
              Real c0[size(a, 1)];
              Real c1[size(a, 1)];
              Real cn;
              Real bb;
            algorithm
// Get coefficients of band pass filter at f_cut
              (cr, c0, c1, cn) := coefficients.bandPass(cr_in, c0_in, c1_in, f_min, f_max);
// Transform coefficients in to roots
              for i in 1:size(a, 1) loop
                a[i] := -c1[i] / 2;
                bb := c0[i] - a[i] * a[i];
                assert(bb >= 0, "\nNot possible to use band pass filter, since transformation results in\n" + "system that does not have conjugate complex poles.\n" + "Try to use another analog filter for the band pass.\n");
                b[i] := sqrt(bb);
                ku[i] := c0[i] / b[i];
                k1[i] := cn / ku[i];
                k2[i] := cn * a[i] / (b[i] * ku[i]);
              end for;
              annotation(
                Documentation(info = "<html>

<p>
The goal is to implement the filter in the following form:
</p>

<pre>
  // complex conjugate poles:
  der(x1) = a*x1 - b*x2 + ku*u;
  der(x2) = b*x1 + a*x2;
       y  = k1*x1 + k2*x2;

            ku = (a^2 + b^2)/b
            k1 = cn/ku
            k2 = cn*a/(b*ku)
</pre>
<p>
This representation has the following transfer function:
</p>
<pre>
// complex conjugate poles
    s*x2 =  a*x2 + b*x1
    s*x1 = -b*x2 + a*x1 + ku*u
  or
    (s-a)*x2               = b*x1  ->  x2 = b/(s-a)*x1
    (s + b^2/(s-a) - a)*x1 = ku*u  ->  (s(s-a) + b^2 - a*(s-a))*x1  = ku*(s-a)*u
                                   ->  (s^2 - 2*a*s + a^2 + b^2)*x1 = ku*(s-a)*u
  or
    x1 = ku*(s-a)/(s^2 - 2*a*s + a^2 + b^2)*u
    x2 = b/(s-a)*ku*(s-a)/(s^2 - 2*a*s + a^2 + b^2)*u
       = b*ku/(s^2 - 2*a*s + a^2 + b^2)*u
    y  = k1*x1 + k2*x2
       = (k1*ku*(s-a) + k2*b*ku) / (s^2 - 2*a*s + a^2 + b^2)*u
       = (k1*ku*s + k2*b*ku - k1*ku*a) / (s^2 - 2*a*s + a^2 + b^2)*u
       = (cn*s + cn*a - cn*a) / (s^2 - 2*a*s + a^2 + b^2)*u
       = cn*s / (s^2 - 2*a*s + a^2 + b^2)*u

  comparing coefficients with
    y = cn*s / (s^2 + c1*s + c0)*u  ->  a = -c1/2
                                        b = sqrt(c0 - a^2)

  comparing with eigenvalue representation:
    (s - (a+jb))*(s - (a-jb)) = s^2 -2*a*s + a^2 + b^2
  shows that:
    a: real part of eigenvalue
    b: imaginary part of eigenvalue
</pre>

</html>"));
            end bandPass;

            function bandStop "Return band stop filter roots as needed for block for given cut-off frequency"
              extends Modelica.Icons.Function;
              input Real cr_in[:] "Coefficients of real poles of base filter";
              input Real c0_in[:] "Coefficients of s^0 term of base filter if conjugate complex pole";
              input Real c1_in[size(c0_in, 1)] "Coefficients of s^1 term of base filter if conjugate complex pole";
              input Modelica.SIunits.Frequency f_min "Band of band stop filter is f_min (A=-3db) .. f_max (A=-3db)";
              input Modelica.SIunits.Frequency f_max "Upper band frequency";
              output Real a[size(cr_in, 1) + 2 * size(c0_in, 1)] "Real parts of complex conjugate eigenvalues";
              output Real b[size(cr_in, 1) + 2 * size(c0_in, 1)] "Imaginary parts of complex conjugate eigenvalues";
              output Real ku[size(cr_in, 1) + 2 * size(c0_in, 1)] "Gains of input terms";
              output Real k1[size(cr_in, 1) + 2 * size(c0_in, 1)] "Gains of y = k1*x1 + k2*x";
              output Real k2[size(cr_in, 1) + 2 * size(c0_in, 1)] "Gains of y = k1*x1 + k2*x";
            protected
              Real cr[0];
              Real c0[size(a, 1)];
              Real c1[size(a, 1)];
              Real cn;
              Real bb;
            algorithm
// Get coefficients of band stop filter at f_cut
              (cr, c0, c1) := coefficients.bandStop(cr_in, c0_in, c1_in, f_min, f_max);
// Transform coefficients in to roots
              for i in 1:size(a, 1) loop
                a[i] := -c1[i] / 2;
                bb := c0[i] - a[i] * a[i];
                assert(bb >= 0, "\nNot possible to use band stop filter, since transformation results in\n" + "system that does not have conjugate complex poles.\n" + "Try to use another analog filter for the band stop filter.\n");
                b[i] := sqrt(bb);
                ku[i] := c0[i] / b[i];
                k1[i] := 2 * a[i] / ku[i];
                k2[i] := (c0[i] + a[i] ^ 2 - b[i] ^ 2) / (b[i] * ku[i]);
              end for;
              annotation(
                Documentation(info = "<html>

<p>
The goal is to implement the filter in the following form:
</p>

<pre>
  // complex conjugate poles:
  der(x1) = a*x1 - b*x2 + ku*u;
  der(x2) = b*x1 + a*x2;
       y  = k1*x1 + k2*x2 + u;

            ku = (a^2 + b^2)/b
            k1 = 2*a/ku
            k2 = (c0 + a^2 - b^2)/(b*ku)
</pre>
<p>
This representation has the following transfer function:
</p>
<pre>
// complex conjugate poles
    s*x2 =  a*x2 + b*x1
    s*x1 = -b*x2 + a*x1 + ku*u
  or
    (s-a)*x2               = b*x1  ->  x2 = b/(s-a)*x1
    (s + b^2/(s-a) - a)*x1 = ku*u  ->  (s(s-a) + b^2 - a*(s-a))*x1  = ku*(s-a)*u
                                   ->  (s^2 - 2*a*s + a^2 + b^2)*x1 = ku*(s-a)*u
  or
    x1 = ku*(s-a)/(s^2 - 2*a*s + a^2 + b^2)*u
    x2 = b/(s-a)*ku*(s-a)/(s^2 - 2*a*s + a^2 + b^2)*u
       = b*ku/(s^2 - 2*a*s + a^2 + b^2)*u
    y  = k1*x1 + k2*x2 + u
       = (k1*ku*(s-a) + k2*b*ku + s^2 - 2*a*s + a^2 + b^2) / (s^2 - 2*a*s + a^2 + b^2)*u
       = (s^2 + (k1*ku-2*a)*s + k2*b*ku - k1*ku*a + a^2 + b^2) / (s^2 - 2*a*s + a^2 + b^2)*u
       = (s^2 + c0 + a^2 - b^2 - 2*a^2 + a^2 + b^2) / (s^2 - 2*a*s + a^2 + b^2)*u
       = (s^2 + c0) / (s^2 - 2*a*s + a^2 + b^2)*u

  comparing coefficients with
    y = (s^2 + c0) / (s^2 + c1*s + c0)*u  ->  a = -c1/2
                                              b = sqrt(c0 - a^2)

  comparing with eigenvalue representation:
    (s - (a+jb))*(s - (a-jb)) = s^2 -2*a*s + a^2 + b^2
  shows that:
    a: real part of eigenvalue
    b: imaginary part of eigenvalue
</pre>

</html>"));
            end bandStop;
          end roots;

          package Utilities "Utility functions for filter computations"
            extends Modelica.Icons.InternalPackage;

            function BesselBaseCoefficients "Return coefficients of normalized low pass Bessel filter (= gain at cut-off frequency 1 rad/s is decreased 3dB)"
              extends Modelica.Icons.Function;
              import Modelica.Utilities.Streams;
              input Integer order "Order of filter in the range 1..41";
              output Real c1[mod(order, 2)] "[p] coefficients of Bessel denominator polynomials (a*p + 1)";
              output Real c2[integer(order / 2), 2] "[p^2, p] coefficients of Bessel denominator polynomials (b2*p^2 + b1*p + 1)";
              output Real alpha "Normalization factor";
            algorithm
              if order == 1 then
                alpha := 1.002377293007601;
                c1[1] := 0.9976283451109835;
              elseif order == 2 then
                alpha := 0.7356641785819585;
                c2[1, 1] := 0.6159132201783791;
                c2[1, 2] := 1.359315879600889;
              elseif order == 3 then
                alpha := 0.5704770156982642;
                c1[1] := 0.7548574865985343;
                c2[1, 1] := 0.4756958028827457;
                c2[1, 2] := 0.9980615136104388;
              elseif order == 4 then
                alpha := 0.4737978580281427;
                c2[1, 1] := 0.4873729247240677;
                c2[1, 2] := 1.337564170455762;
                c2[2, 1] := 0.3877724315741958;
                c2[2, 2] := 0.7730405590839861;
              elseif order == 5 then
                alpha := 0.4126226974763408;
                c1[1] := 0.6645723262620757;
                c2[1, 1] := 0.4115231900614016;
                c2[1, 2] := 1.138349926728708;
                c2[2, 1] := 0.3234938702877912;
                c2[2, 2] := 0.6205992985771313;
              elseif order == 6 then
                alpha := 0.3705098000736233;
                c2[1, 1] := 0.3874508649098960;
                c2[1, 2] := 1.219740879520741;
                c2[2, 1] := 0.3493298843155746;
                c2[2, 2] := 0.9670265529381365;
                c2[3, 1] := 0.2747419229514599;
                c2[3, 2] := 0.5122165075105700;
              elseif order == 7 then
                alpha := 0.3393452623586350;
                c1[1] := 0.5927147125821412;
                c2[1, 1] := 0.3383379423919174;
                c2[1, 2] := 1.092630816438030;
                c2[2, 1] := 0.3001025788696046;
                c2[2, 2] := 0.8289928256598656;
                c2[3, 1] := 0.2372867471539579;
                c2[3, 2] := 0.4325128641920154;
              elseif order == 8 then
                alpha := 0.3150267393795002;
                c2[1, 1] := 0.3151115975207653;
                c2[1, 2] := 1.109403015460190;
                c2[2, 1] := 0.2969344839572762;
                c2[2, 2] := 0.9737455812222699;
                c2[3, 1] := 0.2612545921889538;
                c2[3, 2] := 0.7190394712068573;
                c2[4, 1] := 0.2080523342974281;
                c2[4, 2] := 0.3721456473047434;
              elseif order == 9 then
                alpha := 0.2953310177184124;
                c1[1] := 0.5377196679501422;
                c2[1, 1] := 0.2824689124281034;
                c2[1, 2] := 1.022646191567475;
                c2[2, 1] := 0.2626824161383468;
                c2[2, 2] := 0.8695626454762596;
                c2[3, 1] := 0.2302781917677917;
                c2[3, 2] := 0.6309047553448520;
                c2[4, 1] := 0.1847991729757028;
                c2[4, 2] := 0.3251978031287202;
              elseif order == 10 then
                alpha := 0.2789426890619463;
                c2[1, 1] := 0.2640769908255582;
                c2[1, 2] := 1.019788132875305;
                c2[2, 1] := 0.2540802639216947;
                c2[2, 2] := 0.9377020417760623;
                c2[3, 1] := 0.2343577229427963;
                c2[3, 2] := 0.7802229808216112;
                c2[4, 1] := 0.2052193139338624;
                c2[4, 2] := 0.5594176813008133;
                c2[5, 1] := 0.1659546953748916;
                c2[5, 2] := 0.2878349616233292;
              elseif order == 11 then
                alpha := 0.2650227766037203;
                c1[1] := 0.4950265498954191;
                c2[1, 1] := 0.2411858478546218;
                c2[1, 2] := 0.9567800996387417;
                c2[2, 1] := 0.2296849355380925;
                c2[2, 2] := 0.8592523717113126;
                c2[3, 1] := 0.2107851705677406;
                c2[3, 2] := 0.7040216048898129;
                c2[4, 1] := 0.1846461385164021;
                c2[4, 2] := 0.5006729207276717;
                c2[5, 1] := 0.1504217970817433;
                c2[5, 2] := 0.2575070491320295;
              elseif order == 12 then
                alpha := 0.2530051198547209;
                c2[1, 1] := 0.2268294941204543;
                c2[1, 2] := 0.9473116570034053;
                c2[2, 1] := 0.2207657387793729;
                c2[2, 2] := 0.8933728946287606;
                c2[3, 1] := 0.2087600700376653;
                c2[3, 2] := 0.7886236252756229;
                c2[4, 1] := 0.1909959101492760;
                c2[4, 2] := 0.6389263649257017;
                c2[5, 1] := 0.1675208146048472;
                c2[5, 2] := 0.4517847275162215;
                c2[6, 1] := 0.1374257286372761;
                c2[6, 2] := 0.2324699157474680;
              elseif order == 13 then
                alpha := 0.2424910397561007;
                c1[1] := 0.4608848369928040;
                c2[1, 1] := 0.2099813050274780;
                c2[1, 2] := 0.8992478823790660;
                c2[2, 1] := 0.2027250423101359;
                c2[2, 2] := 0.8328117484224146;
                c2[3, 1] := 0.1907635894058731;
                c2[3, 2] := 0.7257379204691213;
                c2[4, 1] := 0.1742280397887686;
                c2[4, 2] := 0.5830640944868014;
                c2[5, 1] := 0.1530858190490478;
                c2[5, 2] := 0.4106192089751885;
                c2[6, 1] := 0.1264090712880446;
                c2[6, 2] := 0.2114980230156001;
              elseif order == 14 then
                alpha := 0.2331902368695848;
                c2[1, 1] := 0.1986162311411235;
                c2[1, 2] := 0.8876961808055535;
                c2[2, 1] := 0.1946683341271615;
                c2[2, 2] := 0.8500754229171967;
                c2[3, 1] := 0.1868331332895056;
                c2[3, 2] := 0.7764629313723603;
                c2[4, 1] := 0.1752118757862992;
                c2[4, 2] := 0.6699720402924552;
                c2[5, 1] := 0.1598906457908402;
                c2[5, 2] := 0.5348446712848934;
                c2[6, 1] := 0.1407810153019944;
                c2[6, 2] := 0.3755841316563539;
                c2[7, 1] := 0.1169627966707339;
                c2[7, 2] := 0.1937088226304455;
              elseif order == 15 then
                alpha := 0.2248854870552422;
                c1[1] := 0.4328492272335646;
                c2[1, 1] := 0.1857292591004588;
                c2[1, 2] := 0.8496337061962563;
                c2[2, 1] := 0.1808644178280136;
                c2[2, 2] := 0.8020517898136011;
                c2[3, 1] := 0.1728264404199081;
                c2[3, 2] := 0.7247449729331105;
                c2[4, 1] := 0.1616970125901954;
                c2[4, 2] := 0.6205369315943097;
                c2[5, 1] := 0.1475257264578426;
                c2[5, 2] := 0.4929612162355906;
                c2[6, 1] := 0.1301861023357119;
                c2[6, 2] := 0.3454770708040735;
                c2[7, 1] := 0.1087810777120188;
                c2[7, 2] := 0.1784526655428406;
              elseif order == 16 then
                alpha := 0.2174105053474761;
                c2[1, 1] := 0.1765637967473151;
                c2[1, 2] := 0.8377453068635511;
                c2[2, 1] := 0.1738525357503125;
                c2[2, 2] := 0.8102988957433199;
                c2[3, 1] := 0.1684627004613343;
                c2[3, 2] := 0.7563265923413258;
                c2[4, 1] := 0.1604519074815815;
                c2[4, 2] := 0.6776082294687619;
                c2[5, 1] := 0.1498828607802206;
                c2[5, 2] := 0.5766417034027680;
                c2[6, 1] := 0.1367764717792823;
                c2[6, 2] := 0.4563528264410489;
                c2[7, 1] := 0.1209810465419295;
                c2[7, 2] := 0.3193782657322374;
                c2[8, 1] := 0.1016312648007554;
                c2[8, 2] := 0.1652419227369036;
              elseif order == 17 then
                alpha := 0.2106355148193306;
                c1[1] := 0.4093223608497299;
                c2[1, 1] := 0.1664014345826274;
                c2[1, 2] := 0.8067173752345952;
                c2[2, 1] := 0.1629839591538256;
                c2[2, 2] := 0.7712924931447541;
                c2[3, 1] := 0.1573277802512491;
                c2[3, 2] := 0.7134213666303411;
                c2[4, 1] := 0.1494828185148637;
                c2[4, 2] := 0.6347841731714884;
                c2[5, 1] := 0.1394948812681826;
                c2[5, 2] := 0.5375594414619047;
                c2[6, 1] := 0.1273627583380806;
                c2[6, 2] := 0.4241608926375478;
                c2[7, 1] := 0.1129187258461290;
                c2[7, 2] := 0.2965752009703245;
                c2[8, 1] := 0.9533357359908857e-1;
                c2[8, 2] := 0.1537041700889585;
              elseif order == 18 then
                alpha := 0.2044575288651841;
                c2[1, 1] := 0.1588768571976356;
                c2[1, 2] := 0.7951914263212913;
                c2[2, 1] := 0.1569357024981854;
                c2[2, 2] := 0.7744529690772538;
                c2[3, 1] := 0.1530722206358810;
                c2[3, 2] := 0.7335304425992080;
                c2[4, 1] := 0.1473206710524167;
                c2[4, 2] := 0.6735038935387268;
                c2[5, 1] := 0.1397225420331520;
                c2[5, 2] := 0.5959151542621590;
                c2[6, 1] := 0.1303092459809849;
                c2[6, 2] := 0.5026483447894845;
                c2[7, 1] := 0.1190627367060072;
                c2[7, 2] := 0.3956893824587150;
                c2[8, 1] := 0.1058058030798994;
                c2[8, 2] := 0.2765091830730650;
                c2[9, 1] := 0.8974708108800873e-1;
                c2[9, 2] := 0.1435505288284833;
              elseif order == 19 then
                alpha := 0.1987936248083529;
                c1[1] := 0.3892259966869526;
                c2[1, 1] := 0.1506640012172225;
                c2[1, 2] := 0.7693121733774260;
                c2[2, 1] := 0.1481728062796673;
                c2[2, 2] := 0.7421133586741549;
                c2[3, 1] := 0.1440444668388838;
                c2[3, 2] := 0.6975075386214800;
                c2[4, 1] := 0.1383101628540374;
                c2[4, 2] := 0.6365464378910025;
                c2[5, 1] := 0.1310032283190998;
                c2[5, 2] := 0.5606211948462122;
                c2[6, 1] := 0.1221431166405330;
                c2[6, 2] := 0.4713530424221445;
                c2[7, 1] := 0.1116991161103884;
                c2[7, 2] := 0.3703717538617073;
                c2[8, 1] := 0.9948917351196349e-1;
                c2[8, 2] := 0.2587371155559744;
                c2[9, 1] := 0.8475989238107367e-1;
                c2[9, 2] := 0.1345537894555993;
              elseif order == 20 then
                alpha := 0.1935761760416219;
                c2[1, 1] := 0.1443871348337404;
                c2[1, 2] := 0.7584165598446141;
                c2[2, 1] := 0.1429501891353184;
                c2[2, 2] := 0.7423000962318863;
                c2[3, 1] := 0.1400877384920004;
                c2[3, 2] := 0.7104185332215555;
                c2[4, 1] := 0.1358210369491446;
                c2[4, 2] := 0.6634599783272630;
                c2[5, 1] := 0.1301773703034290;
                c2[5, 2] := 0.6024175491895959;
                c2[6, 1] := 0.1231826501439148;
                c2[6, 2] := 0.5285332736326852;
                c2[7, 1] := 0.1148465498575254;
                c2[7, 2] := 0.4431977385498628;
                c2[8, 1] := 0.1051289462376788;
                c2[8, 2] := 0.3477444062821162;
                c2[9, 1] := 0.9384622797485121e-1;
                c2[9, 2] := 0.2429038300327729;
                c2[10, 1] := 0.8028211612831444e-1;
                c2[10, 2] := 0.1265329974009533;
              elseif order == 21 then
                alpha := 0.1887494014766075;
                c1[1] := 0.3718070668941645;
                c2[1, 1] := 0.1376151928386445;
                c2[1, 2] := 0.7364290859445481;
                c2[2, 1] := 0.1357438914390695;
                c2[2, 2] := 0.7150167318935022;
                c2[3, 1] := 0.1326398453462415;
                c2[3, 2] := 0.6798001808470175;
                c2[4, 1] := 0.1283231214897678;
                c2[4, 2] := 0.6314663440439816;
                c2[5, 1] := 0.1228169159777534;
                c2[5, 2] := 0.5709353626166905;
                c2[6, 1] := 0.1161406100773184;
                c2[6, 2] := 0.4993087153571335;
                c2[7, 1] := 0.1082959649233524;
                c2[7, 2] := 0.4177766148584385;
                c2[8, 1] := 0.9923596957485723e-1;
                c2[8, 2] := 0.3274257287232124;
                c2[9, 1] := 0.8877776108724853e-1;
                c2[9, 2] := 0.2287218166767916;
                c2[10, 1] := 0.7624076527736326e-1;
                c2[10, 2] := 0.1193423971506988;
              elseif order == 22 then
                alpha := 0.1842668221199706;
                c2[1, 1] := 0.1323053462701543;
                c2[1, 2] := 0.7262446126765204;
                c2[2, 1] := 0.1312121721769772;
                c2[2, 2] := 0.7134286088450949;
                c2[3, 1] := 0.1290330911166814;
                c2[3, 2] := 0.6880287870435514;
                c2[4, 1] := 0.1257817990372067;
                c2[4, 2] := 0.6505015800059301;
                c2[5, 1] := 0.1214765261983008;
                c2[5, 2] := 0.6015107185211451;
                c2[6, 1] := 0.1161365140967959;
                c2[6, 2] := 0.5418983553698413;
                c2[7, 1] := 0.1097755171533100;
                c2[7, 2] := 0.4726370779831614;
                c2[8, 1] := 0.1023889478519956;
                c2[8, 2] := 0.3947439506537486;
                c2[9, 1] := 0.9392485861253800e-1;
                c2[9, 2] := 0.3090996703083202;
                c2[10, 1] := 0.8420273775456455e-1;
                c2[10, 2] := 0.2159561978556017;
                c2[11, 1] := 0.7257600023938262e-1;
                c2[11, 2] := 0.1128633732721116;
              elseif order == 23 then
                alpha := 0.1800893554453722;
                c1[1] := 0.3565232673929280;
                c2[1, 1] := 0.1266275171652706;
                c2[1, 2] := 0.7072778066734162;
                c2[2, 1] := 0.1251865227648538;
                c2[2, 2] := 0.6900676345785905;
                c2[3, 1] := 0.1227944815236645;
                c2[3, 2] := 0.6617011100576023;
                c2[4, 1] := 0.1194647013077667;
                c2[4, 2] := 0.6226432315773119;
                c2[5, 1] := 0.1152132989252356;
                c2[5, 2] := 0.5735222810625359;
                c2[6, 1] := 0.1100558598478487;
                c2[6, 2] := 0.5151027978024605;
                c2[7, 1] := 0.1040013558214886;
                c2[7, 2] := 0.4482410942032739;
                c2[8, 1] := 0.9704014176512626e-1;
                c2[8, 2] := 0.3738049984631116;
                c2[9, 1] := 0.8911683905758054e-1;
                c2[9, 2] := 0.2925028692588410;
                c2[10, 1] := 0.8005438265072295e-1;
                c2[10, 2] := 0.2044134600278901;
                c2[11, 1] := 0.6923832296800832e-1;
                c2[11, 2] := 0.1069984887283394;
              elseif order == 24 then
                alpha := 0.1761838665838427;
                c2[1, 1] := 0.1220804912720132;
                c2[1, 2] := 0.6978026874156063;
                c2[2, 1] := 0.1212296762358897;
                c2[2, 2] := 0.6874139794926736;
                c2[3, 1] := 0.1195328372961027;
                c2[3, 2] := 0.6667954259551859;
                c2[4, 1] := 0.1169990987333593;
                c2[4, 2] := 0.6362602049901176;
                c2[5, 1] := 0.1136409040480130;
                c2[5, 2] := 0.5962662188435553;
                c2[6, 1] := 0.1094722001757955;
                c2[6, 2] := 0.5474001634109253;
                c2[7, 1] := 0.1045052832229087;
                c2[7, 2] := 0.4903523180249535;
                c2[8, 1] := 0.9874509806025907e-1;
                c2[8, 2] := 0.4258751523524645;
                c2[9, 1] := 0.9217799943472177e-1;
                c2[9, 2] := 0.3547079765396403;
                c2[10, 1] := 0.8474633796250476e-1;
                c2[10, 2] := 0.2774145482392767;
                c2[11, 1] := 0.7627722381240495e-1;
                c2[11, 2] := 0.1939329108084139;
                c2[12, 1] := 0.6618645465422745e-1;
                c2[12, 2] := 0.1016670147947242;
              elseif order == 25 then
                alpha := 0.1725220521949266;
                c1[1] := 0.3429735385896000;
                c2[1, 1] := 0.1172525033170618;
                c2[1, 2] := 0.6812327932576614;
                c2[2, 1] := 0.1161194585333535;
                c2[2, 2] := 0.6671566071153211;
                c2[3, 1] := 0.1142375145794466;
                c2[3, 2] := 0.6439167855053158;
                c2[4, 1] := 0.1116157454252308;
                c2[4, 2] := 0.6118378416180135;
                c2[5, 1] := 0.1082654809459177;
                c2[5, 2] := 0.5713609763370088;
                c2[6, 1] := 0.1041985674230918;
                c2[6, 2] := 0.5230289949762722;
                c2[7, 1] := 0.9942439308123559e-1;
                c2[7, 2] := 0.4674627926041906;
                c2[8, 1] := 0.9394453593830893e-1;
                c2[8, 2] := 0.4053226688298811;
                c2[9, 1] := 0.8774221237222533e-1;
                c2[9, 2] := 0.3372372276379071;
                c2[10, 1] := 0.8075839512216483e-1;
                c2[10, 2] := 0.2636485508005428;
                c2[11, 1] := 0.7282483286646764e-1;
                c2[11, 2] := 0.1843801345273085;
                c2[12, 1] := 0.6338571166846652e-1;
                c2[12, 2] := 0.9680153764737715e-1;
              elseif order == 26 then
                alpha := 0.1690795702796737;
                c2[1, 1] := 0.1133168695796030;
                c2[1, 2] := 0.6724297955493932;
                c2[2, 1] := 0.1126417845769961;
                c2[2, 2] := 0.6638709519790540;
                c2[3, 1] := 0.1112948749545606;
                c2[3, 2] := 0.6468652038763624;
                c2[4, 1] := 0.1092823986944244;
                c2[4, 2] := 0.6216337070799265;
                c2[5, 1] := 0.1066130386697976;
                c2[5, 2] := 0.5885011413992190;
                c2[6, 1] := 0.1032969057045413;
                c2[6, 2] := 0.5478864278297548;
                c2[7, 1] := 0.9934388184210715e-1;
                c2[7, 2] := 0.5002885306054287;
                c2[8, 1] := 0.9476081523436283e-1;
                c2[8, 2] := 0.4462644847551711;
                c2[9, 1] := 0.8954648464575577e-1;
                c2[9, 2] := 0.3863930785049522;
                c2[10, 1] := 0.8368166847159917e-1;
                c2[10, 2] := 0.3212074592527143;
                c2[11, 1] := 0.7710664731701103e-1;
                c2[11, 2] := 0.2510470347119383;
                c2[12, 1] := 0.6965807988411425e-1;
                c2[12, 2] := 0.1756419294111342;
                c2[13, 1] := 0.6080674930548766e-1;
                c2[13, 2] := 0.9234535279274277e-1;
              elseif order == 27 then
                alpha := 0.1658353543067995;
                c1[1] := 0.3308543720638957;
                c2[1, 1] := 0.1091618578712746;
                c2[1, 2] := 0.6577977071169651;
                c2[2, 1] := 0.1082549561495043;
                c2[2, 2] := 0.6461121666520275;
                c2[3, 1] := 0.1067479247890451;
                c2[3, 2] := 0.6267937760991321;
                c2[4, 1] := 0.1046471079537577;
                c2[4, 2] := 0.6000750116745808;
                c2[5, 1] := 0.1019605976654259;
                c2[5, 2] := 0.5662734183049320;
                c2[6, 1] := 0.9869726954433709e-1;
                c2[6, 2] := 0.5257827234948534;
                c2[7, 1] := 0.9486520934132483e-1;
                c2[7, 2] := 0.4790595019077763;
                c2[8, 1] := 0.9046906518775348e-1;
                c2[8, 2] := 0.4266025862147336;
                c2[9, 1] := 0.8550529998276152e-1;
                c2[9, 2] := 0.3689188223512328;
                c2[10, 1] := 0.7995282239306020e-1;
                c2[10, 2] := 0.3064589322702932;
                c2[11, 1] := 0.7375174596252882e-1;
                c2[11, 2] := 0.2394754504667310;
                c2[12, 1] := 0.6674377263329041e-1;
                c2[12, 2] := 0.1676223546666024;
                c2[13, 1] := 0.5842458027529246e-1;
                c2[13, 2] := 0.8825044329219431e-1;
              elseif order == 28 then
                alpha := 0.1627710671942929;
                c2[1, 1] := 0.1057232656113488;
                c2[1, 2] := 0.6496161226860832;
                c2[2, 1] := 0.1051786825724864;
                c2[2, 2] := 0.6424661279909941;
                c2[3, 1] := 0.1040917964935006;
                c2[3, 2] := 0.6282470268918791;
                c2[4, 1] := 0.1024670101953951;
                c2[4, 2] := 0.6071189030701136;
                c2[5, 1] := 0.1003105109519892;
                c2[5, 2] := 0.5793175191747016;
                c2[6, 1] := 0.9762969425430802e-1;
                c2[6, 2] := 0.5451486608855443;
                c2[7, 1] := 0.9443223803058400e-1;
                c2[7, 2] := 0.5049796971628137;
                c2[8, 1] := 0.9072460982036488e-1;
                c2[8, 2] := 0.4592270546572523;
                c2[9, 1] := 0.8650956423253280e-1;
                c2[9, 2] := 0.4083368605952977;
                c2[10, 1] := 0.8178165740374893e-1;
                c2[10, 2] := 0.3527525188880655;
                c2[11, 1] := 0.7651838885868020e-1;
                c2[11, 2] := 0.2928534570013572;
                c2[12, 1] := 0.7066010532447490e-1;
                c2[12, 2] := 0.2288185204390681;
                c2[13, 1] := 0.6405358596145789e-1;
                c2[13, 2] := 0.1602396172588190;
                c2[14, 1] := 0.5621780070227172e-1;
                c2[14, 2] := 0.8447589564915071e-1;
              elseif order == 29 then
                alpha := 0.1598706626277596;
                c1[1] := 0.3199314513011623;
                c2[1, 1] := 0.1021101032532951;
                c2[1, 2] := 0.6365758882240111;
                c2[2, 1] := 0.1013729819392774;
                c2[2, 2] := 0.6267495975736321;
                c2[3, 1] := 0.1001476175660628;
                c2[3, 2] := 0.6104876178266819;
                c2[4, 1] := 0.9843854640428316e-1;
                c2[4, 2] := 0.5879603139195113;
                c2[5, 1] := 0.9625164534591696e-1;
                c2[5, 2] := 0.5594012291050210;
                c2[6, 1] := 0.9359356960417668e-1;
                c2[6, 2] := 0.5251016150410664;
                c2[7, 1] := 0.9047086748649986e-1;
                c2[7, 2] := 0.4854024475590397;
                c2[8, 1] := 0.8688856407189167e-1;
                c2[8, 2] := 0.4406826457109709;
                c2[9, 1] := 0.8284779224069856e-1;
                c2[9, 2] := 0.3913408089298914;
                c2[10, 1] := 0.7834154620997181e-1;
                c2[10, 2] := 0.3377643999400627;
                c2[11, 1] := 0.7334628941928766e-1;
                c2[11, 2] := 0.2802710651919946;
                c2[12, 1] := 0.6780290487362146e-1;
                c2[12, 2] := 0.2189770008083379;
                c2[13, 1] := 0.6156321231528423e-1;
                c2[13, 2] := 0.1534235999306070;
                c2[14, 1] := 0.5416797446761512e-1;
                c2[14, 2] := 0.8098664736760292e-1;
              elseif order == 30 then
                alpha := 0.1571200296252450;
                c2[1, 1] := 0.9908074847842124e-1;
                c2[1, 2] := 0.6289618807831557;
                c2[2, 1] := 0.9863509708328196e-1;
                c2[2, 2] := 0.6229164525571278;
                c2[3, 1] := 0.9774542692037148e-1;
                c2[3, 2] := 0.6108853364240036;
                c2[4, 1] := 0.9641490581986484e-1;
                c2[4, 2] := 0.5929869253412513;
                c2[5, 1] := 0.9464802912225441e-1;
                c2[5, 2] := 0.5693960175547550;
                c2[6, 1] := 0.9245027206218041e-1;
                c2[6, 2] := 0.5403402396359503;
                c2[7, 1] := 0.8982754584112941e-1;
                c2[7, 2] := 0.5060948065875106;
                c2[8, 1] := 0.8678535291732599e-1;
                c2[8, 2] := 0.4669749797983789;
                c2[9, 1] := 0.8332744242052199e-1;
                c2[9, 2] := 0.4233249626334694;
                c2[10, 1] := 0.7945356393775309e-1;
                c2[10, 2] := 0.3755006094498054;
                c2[11, 1] := 0.7515543969833788e-1;
                c2[11, 2] := 0.3238400339292700;
                c2[12, 1] := 0.7040879901685638e-1;
                c2[12, 2] := 0.2686072427439079;
                c2[13, 1] := 0.6515528854010540e-1;
                c2[13, 2] := 0.2098650589782619;
                c2[14, 1] := 0.5925168237177876e-1;
                c2[14, 2] := 0.1471138832654873;
                c2[15, 1] := 0.5225913954211672e-1;
                c2[15, 2] := 0.7775248839507864e-1;
              elseif order == 31 then
                alpha := 0.1545067022920929;
                c1[1] := 0.3100206996451866;
                c2[1, 1] := 0.9591020358831668e-1;
                c2[1, 2] := 0.6172474793293396;
                c2[2, 1] := 0.9530301275601203e-1;
                c2[2, 2] := 0.6088916323460413;
                c2[3, 1] := 0.9429332655402368e-1;
                c2[3, 2] := 0.5950511595503025;
                c2[4, 1] := 0.9288445429894548e-1;
                c2[4, 2] := 0.5758534119053522;
                c2[5, 1] := 0.9108073420087422e-1;
                c2[5, 2] := 0.5514734636081183;
                c2[6, 1] := 0.8888719137536870e-1;
                c2[6, 2] := 0.5221306199481831;
                c2[7, 1] := 0.8630901440239650e-1;
                c2[7, 2] := 0.4880834248148061;
                c2[8, 1] := 0.8335074993373294e-1;
                c2[8, 2] := 0.4496225358496770;
                c2[9, 1] := 0.8001502494376102e-1;
                c2[9, 2] := 0.4070602306679052;
                c2[10, 1] := 0.7630041338037624e-1;
                c2[10, 2] := 0.3607139804818122;
                c2[11, 1] := 0.7219760885744920e-1;
                c2[11, 2] := 0.3108783301229550;
                c2[12, 1] := 0.6768185077153345e-1;
                c2[12, 2] := 0.2577706252514497;
                c2[13, 1] := 0.6269571766328638e-1;
                c2[13, 2] := 0.2014081375889921;
                c2[14, 1] := 0.5710081766945065e-1;
                c2[14, 2] := 0.1412581515841926;
                c2[15, 1] := 0.5047740914807019e-1;
                c2[15, 2] := 0.7474725873250158e-1;
              elseif order == 32 then
                alpha := 0.1520196210848210;
                c2[1, 1] := 0.9322163554339406e-1;
                c2[1, 2] := 0.6101488690506050;
                c2[2, 1] := 0.9285233997694042e-1;
                c2[2, 2] := 0.6049832320721264;
                c2[3, 1] := 0.9211494244473163e-1;
                c2[3, 2] := 0.5946969295569034;
                c2[4, 1] := 0.9101176786042449e-1;
                c2[4, 2] := 0.5793791854364477;
                c2[5, 1] := 0.8954614071360517e-1;
                c2[5, 2] := 0.5591619969234026;
                c2[6, 1] := 0.8772216763680164e-1;
                c2[6, 2] := 0.5342177994699602;
                c2[7, 1] := 0.8554440426912734e-1;
                c2[7, 2] := 0.5047560942986598;
                c2[8, 1] := 0.8301735302045588e-1;
                c2[8, 2] := 0.4710187048140929;
                c2[9, 1] := 0.8014469519188161e-1;
                c2[9, 2] := 0.4332730387207936;
                c2[10, 1] := 0.7692807528893225e-1;
                c2[10, 2] := 0.3918021436411035;
                c2[11, 1] := 0.7336507157284898e-1;
                c2[11, 2] := 0.3468890521471250;
                c2[12, 1] := 0.6944555312763458e-1;
                c2[12, 2] := 0.2987898029050460;
                c2[13, 1] := 0.6514446669420571e-1;
                c2[13, 2] := 0.2476810747407199;
                c2[14, 1] := 0.6040544477732702e-1;
                c2[14, 2] := 0.1935412053397663;
                c2[15, 1] := 0.5509478650672775e-1;
                c2[15, 2] := 0.1358108994174911;
                c2[16, 1] := 0.4881064725720192e-1;
                c2[16, 2] := 0.7194819894416505e-1;
              elseif order == 33 then
                alpha := 0.1496489351138032;
                c1[1] := 0.3009752799176432;
                c2[1, 1] := 0.9041725460994505e-1;
                c2[1, 2] := 0.5995521047364046;
                c2[2, 1] := 0.8991117804113002e-1;
                c2[2, 2] := 0.5923764112099496;
                c2[3, 1] := 0.8906941547422532e-1;
                c2[3, 2] := 0.5804822013853129;
                c2[4, 1] := 0.8789442491445575e-1;
                c2[4, 2] := 0.5639663528946501;
                c2[5, 1] := 0.8638945831033775e-1;
                c2[5, 2] := 0.5429623519607796;
                c2[6, 1] := 0.8455834602616358e-1;
                c2[6, 2] := 0.5176379938389326;
                c2[7, 1] := 0.8240517431382334e-1;
                c2[7, 2] := 0.4881921474066189;
                c2[8, 1] := 0.7993380417355076e-1;
                c2[8, 2] := 0.4548502528082586;
                c2[9, 1] := 0.7714713890732801e-1;
                c2[9, 2] := 0.4178579388038483;
                c2[10, 1] := 0.7404596598181127e-1;
                c2[10, 2] := 0.3774715722484659;
                c2[11, 1] := 0.7062702339160462e-1;
                c2[11, 2] := 0.3339432938810453;
                c2[12, 1] := 0.6687952672391507e-1;
                c2[12, 2] := 0.2874950693388235;
                c2[13, 1] := 0.6277828912909767e-1;
                c2[13, 2] := 0.2382680702894708;
                c2[14, 1] := 0.5826808305383988e-1;
                c2[14, 2] := 0.1862073169968455;
                c2[15, 1] := 0.5321974125363517e-1;
                c2[15, 2] := 0.1307323751236313;
                c2[16, 1] := 0.4724820282032780e-1;
                c2[16, 2] := 0.6933542082177094e-1;
              elseif order == 34 then
                alpha := 0.1473858373968463;
                c2[1, 1] := 0.8801537152275983e-1;
                c2[1, 2] := 0.5929204288972172;
                c2[2, 1] := 0.8770594341007476e-1;
                c2[2, 2] := 0.5884653382247518;
                c2[3, 1] := 0.8708797598072095e-1;
                c2[3, 2] := 0.5795895850253119;
                c2[4, 1] := 0.8616320590689187e-1;
                c2[4, 2] := 0.5663615383647170;
                c2[5, 1] := 0.8493413175570858e-1;
                c2[5, 2] := 0.5488825092350877;
                c2[6, 1] := 0.8340387368687513e-1;
                c2[6, 2] := 0.5272851839324592;
                c2[7, 1] := 0.8157596213131521e-1;
                c2[7, 2] := 0.5017313864372913;
                c2[8, 1] := 0.7945402670834270e-1;
                c2[8, 2] := 0.4724089864574216;
                c2[9, 1] := 0.7704133559556429e-1;
                c2[9, 2] := 0.4395276256463053;
                c2[10, 1] := 0.7434009635219704e-1;
                c2[10, 2] := 0.4033126590648964;
                c2[11, 1] := 0.7135035113853376e-1;
                c2[11, 2] := 0.3639961488919042;
                c2[12, 1] := 0.6806813160738834e-1;
                c2[12, 2] := 0.3218025212900124;
                c2[13, 1] := 0.6448214312000864e-1;
                c2[13, 2] := 0.2769235521088158;
                c2[14, 1] := 0.6056719318430530e-1;
                c2[14, 2] := 0.2294693573271038;
                c2[15, 1] := 0.5626925196925040e-1;
                c2[15, 2] := 0.1793564218840015;
                c2[16, 1] := 0.5146352031547277e-1;
                c2[16, 2] := 0.1259877129326412;
                c2[17, 1] := 0.4578069074410591e-1;
                c2[17, 2] := 0.6689147319568768e-1;
              elseif order == 35 then
                alpha := 0.1452224267615486;
                c1[1] := 0.2926764667564367;
                c2[1, 1] := 0.8551731299267280e-1;
                c2[1, 2] := 0.5832758214629523;
                c2[2, 1] := 0.8509109732853060e-1;
                c2[2, 2] := 0.5770596582643844;
                c2[3, 1] := 0.8438201446671953e-1;
                c2[3, 2] := 0.5667497616665494;
                c2[4, 1] := 0.8339191981579831e-1;
                c2[4, 2] := 0.5524209816238369;
                c2[5, 1] := 0.8212328610083385e-1;
                c2[5, 2] := 0.5341766459916322;
                c2[6, 1] := 0.8057906332198853e-1;
                c2[6, 2] := 0.5121470053512750;
                c2[7, 1] := 0.7876247299954955e-1;
                c2[7, 2] := 0.4864870722254752;
                c2[8, 1] := 0.7667670879950268e-1;
                c2[8, 2] := 0.4573736721705665;
                c2[9, 1] := 0.7432449556218945e-1;
                c2[9, 2] := 0.4250013835198991;
                c2[10, 1] := 0.7170742126011575e-1;
                c2[10, 2] := 0.3895767735915445;
                c2[11, 1] := 0.6882488171701314e-1;
                c2[11, 2] := 0.3513097926737368;
                c2[12, 1] := 0.6567231746957568e-1;
                c2[12, 2] := 0.3103999917596611;
                c2[13, 1] := 0.6223804362223595e-1;
                c2[13, 2] := 0.2670123611280899;
                c2[14, 1] := 0.5849696460782910e-1;
                c2[14, 2] := 0.2212298104867592;
                c2[15, 1] := 0.5439628409499822e-1;
                c2[15, 2] := 0.1729443731341637;
                c2[16, 1] := 0.4981540179136920e-1;
                c2[16, 2] := 0.1215462157134930;
                c2[17, 1] := 0.4439981033536435e-1;
                c2[17, 2] := 0.6460098363520967e-1;
              elseif order == 36 then
                alpha := 0.1431515914458580;
                c2[1, 1] := 0.8335881847130301e-1;
                c2[1, 2] := 0.5770670512160201;
                c2[2, 1] := 0.8309698922852212e-1;
                c2[2, 2] := 0.5731929100172432;
                c2[3, 1] := 0.8257400347039723e-1;
                c2[3, 2] := 0.5654713811993058;
                c2[4, 1] := 0.8179117911600136e-1;
                c2[4, 2] := 0.5539556343603020;
                c2[5, 1] := 0.8075042173126963e-1;
                c2[5, 2] := 0.5387245649546684;
                c2[6, 1] := 0.7945413151258206e-1;
                c2[6, 2] := 0.5198817177723069;
                c2[7, 1] := 0.7790506514288866e-1;
                c2[7, 2] := 0.4975537629595409;
                c2[8, 1] := 0.7610613635339480e-1;
                c2[8, 2] := 0.4718884193866789;
                c2[9, 1] := 0.7406012816626425e-1;
                c2[9, 2] := 0.4430516443136726;
                c2[10, 1] := 0.7176927060205631e-1;
                c2[10, 2] := 0.4112237708115829;
                c2[11, 1] := 0.6923460172504251e-1;
                c2[11, 2] := 0.3765940116389730;
                c2[12, 1] := 0.6645495833489556e-1;
                c2[12, 2] := 0.3393522147815403;
                c2[13, 1] := 0.6342528888937094e-1;
                c2[13, 2] := 0.2996755899575573;
                c2[14, 1] := 0.6013361864949449e-1;
                c2[14, 2] := 0.2577053294053830;
                c2[15, 1] := 0.5655503081322404e-1;
                c2[15, 2] := 0.2135004731531631;
                c2[16, 1] := 0.5263798119559069e-1;
                c2[16, 2] := 0.1669320999865636;
                c2[17, 1] := 0.4826589873626196e-1;
                c2[17, 2] := 0.1173807590715484;
                c2[18, 1] := 0.4309819397289806e-1;
                c2[18, 2] := 0.6245036108880222e-1;
              elseif order == 37 then
                alpha := 0.1411669104782917;
                c1[1] := 0.2850271036215707;
                c2[1, 1] := 0.8111958235023328e-1;
                c2[1, 2] := 0.5682412610563970;
                c2[2, 1] := 0.8075727567979578e-1;
                c2[2, 2] := 0.5628142923227016;
                c2[3, 1] := 0.8015440554413301e-1;
                c2[3, 2] := 0.5538087696879930;
                c2[4, 1] := 0.7931239302677386e-1;
                c2[4, 2] := 0.5412833323304460;
                c2[5, 1] := 0.7823314328639347e-1;
                c2[5, 2] := 0.5253190555393968;
                c2[6, 1] := 0.7691895211595101e-1;
                c2[6, 2] := 0.5060183741977191;
                c2[7, 1] := 0.7537237072011853e-1;
                c2[7, 2] := 0.4835036020049034;
                c2[8, 1] := 0.7359601294804538e-1;
                c2[8, 2] := 0.4579149413954837;
                c2[9, 1] := 0.7159227884849299e-1;
                c2[9, 2] := 0.4294078049978829;
                c2[10, 1] := 0.6936295002846032e-1;
                c2[10, 2] := 0.3981491350382047;
                c2[11, 1] := 0.6690857785828917e-1;
                c2[11, 2] := 0.3643121502867948;
                c2[12, 1] := 0.6422751692085542e-1;
                c2[12, 2] := 0.3280684291406284;
                c2[13, 1] := 0.6131430866206096e-1;
                c2[13, 2] := 0.2895750997170303;
                c2[14, 1] := 0.5815677249570920e-1;
                c2[14, 2] := 0.2489521814805720;
                c2[15, 1] := 0.5473023527947980e-1;
                c2[15, 2] := 0.2062377435955363;
                c2[16, 1] := 0.5098441033167034e-1;
                c2[16, 2] := 0.1612849131645336;
                c2[17, 1] := 0.4680658811093562e-1;
                c2[17, 2] := 0.1134672937045305;
                c2[18, 1] := 0.4186928031694695e-1;
                c2[18, 2] := 0.6042754777339966e-1;
              elseif order == 38 then
                alpha := 0.1392625697140030;
                c2[1, 1] := 0.7916943373658329e-1;
                c2[1, 2] := 0.5624158631591745;
                c2[2, 1] := 0.7894592250257840e-1;
                c2[2, 2] := 0.5590219398777304;
                c2[3, 1] := 0.7849941672384930e-1;
                c2[3, 2] := 0.5522551628416841;
                c2[4, 1] := 0.7783093084875645e-1;
                c2[4, 2] := 0.5421574325808380;
                c2[5, 1] := 0.7694193770482690e-1;
                c2[5, 2] := 0.5287909941093643;
                c2[6, 1] := 0.7583430534712885e-1;
                c2[6, 2] := 0.5122376814029880;
                c2[7, 1] := 0.7451020436122948e-1;
                c2[7, 2] := 0.4925978555548549;
                c2[8, 1] := 0.7297197617673508e-1;
                c2[8, 2] := 0.4699889739625235;
                c2[9, 1] := 0.7122194706992953e-1;
                c2[9, 2] := 0.4445436860615774;
                c2[10, 1] := 0.6926216260386816e-1;
                c2[10, 2] := 0.4164072786327193;
                c2[11, 1] := 0.6709399961255503e-1;
                c2[11, 2] := 0.3857341621868851;
                c2[12, 1] := 0.6471757977022456e-1;
                c2[12, 2] := 0.3526828388476838;
                c2[13, 1] := 0.6213084287116965e-1;
                c2[13, 2] := 0.3174082831364342;
                c2[14, 1] := 0.5932799638550641e-1;
                c2[14, 2] := 0.2800495563550299;
                c2[15, 1] := 0.5629672408524944e-1;
                c2[15, 2] := 0.2407078154782509;
                c2[16, 1] := 0.5301264751544952e-1;
                c2[16, 2] := 0.1994026830553859;
                c2[17, 1] := 0.4942673259817896e-1;
                c2[17, 2] := 0.1559719194038917;
                c2[18, 1] := 0.4542996716979947e-1;
                c2[18, 2] := 0.1097844277878470;
                c2[19, 1] := 0.4070720755433961e-1;
                c2[19, 2] := 0.5852181110523043e-1;
              elseif order == 39 then
                alpha := 0.1374332900196804;
                c1[1] := 0.2779468246419593;
                c2[1, 1] := 0.7715084161825772e-1;
                c2[1, 2] := 0.5543001331300056;
                c2[2, 1] := 0.7684028301163326e-1;
                c2[2, 2] := 0.5495289890712267;
                c2[3, 1] := 0.7632343924866024e-1;
                c2[3, 2] := 0.5416083298429741;
                c2[4, 1] := 0.7560141319808483e-1;
                c2[4, 2] := 0.5305846713929198;
                c2[5, 1] := 0.7467569064745969e-1;
                c2[5, 2] := 0.5165224112570647;
                c2[6, 1] := 0.7354807648551346e-1;
                c2[6, 2] := 0.4995030679271456;
                c2[7, 1] := 0.7222060351121389e-1;
                c2[7, 2] := 0.4796242430956156;
                c2[8, 1] := 0.7069540462458585e-1;
                c2[8, 2] := 0.4569982440368368;
                c2[9, 1] := 0.6897453353492381e-1;
                c2[9, 2] := 0.4317502624832354;
                c2[10, 1] := 0.6705970959388781e-1;
                c2[10, 2] := 0.4040159353969854;
                c2[11, 1] := 0.6495194541066725e-1;
                c2[11, 2] := 0.3739379843169939;
                c2[12, 1] := 0.6265098412417610e-1;
                c2[12, 2] := 0.3416613843816217;
                c2[13, 1] := 0.6015440984955930e-1;
                c2[13, 2] := 0.3073260166338746;
                c2[14, 1] := 0.5745615876877304e-1;
                c2[14, 2] := 0.2710546723961181;
                c2[15, 1] := 0.5454383762391338e-1;
                c2[15, 2] := 0.2329316824061170;
                c2[16, 1] := 0.5139340231935751e-1;
                c2[16, 2] := 0.1929604256043231;
                c2[17, 1] := 0.4795705862458131e-1;
                c2[17, 2] := 0.1509655259246037;
                c2[18, 1] := 0.4412933231935506e-1;
                c2[18, 2] := 0.1063130748962878;
                c2[19, 1] := 0.3960672309405603e-1;
                c2[19, 2] := 0.5672356837211527e-1;
              elseif order == 40 then
                alpha := 0.1356742655825434;
                c2[1, 1] := 0.7538038374294594e-1;
                c2[1, 2] := 0.5488228264329617;
                c2[2, 1] := 0.7518806529402738e-1;
                c2[2, 2] := 0.5458297722483311;
                c2[3, 1] := 0.7480383050347119e-1;
                c2[3, 2] := 0.5398604576730540;
                c2[4, 1] := 0.7422847031965465e-1;
                c2[4, 2] := 0.5309482987446206;
                c2[5, 1] := 0.7346313704205006e-1;
                c2[5, 2] := 0.5191429845322307;
                c2[6, 1] := 0.7250930053201402e-1;
                c2[6, 2] := 0.5045099368431007;
                c2[7, 1] := 0.7136868456879621e-1;
                c2[7, 2] := 0.4871295553902607;
                c2[8, 1] := 0.7004317764946634e-1;
                c2[8, 2] := 0.4670962098860498;
                c2[9, 1] := 0.6853470921527828e-1;
                c2[9, 2] := 0.4445169164956202;
                c2[10, 1] := 0.6684507689945471e-1;
                c2[10, 2] := 0.4195095960479698;
                c2[11, 1] := 0.6497570123412630e-1;
                c2[11, 2] := 0.3922007419030645;
                c2[12, 1] := 0.6292726794917847e-1;
                c2[12, 2] := 0.3627221993494397;
                c2[13, 1] := 0.6069918741663154e-1;
                c2[13, 2] := 0.3312065181294388;
                c2[14, 1] := 0.5828873983769410e-1;
                c2[14, 2] := 0.2977798532686911;
                c2[15, 1] := 0.5568964389813015e-1;
                c2[15, 2] := 0.2625503293999835;
                c2[16, 1] := 0.5288947816690705e-1;
                c2[16, 2] := 0.2255872486520188;
                c2[17, 1] := 0.4986456327645859e-1;
                c2[17, 2] := 0.1868796731919594;
                c2[18, 1] := 0.4656832613054458e-1;
                c2[18, 2] := 0.1462410193532463;
                c2[19, 1] := 0.4289867647614935e-1;
                c2[19, 2] := 0.1030361558710747;
                c2[20, 1] := 0.3856310684054106e-1;
                c2[20, 2] := 0.5502423832293889e-1;
              elseif order == 41 then
                alpha := 0.1339811106984253;
                c1[1] := 0.2713685065531391;
                c2[1, 1] := 0.7355140275160984e-1;
                c2[1, 2] := 0.5413274778282860;
                c2[2, 1] := 0.7328319082267173e-1;
                c2[2, 2] := 0.5371064088294270;
                c2[3, 1] := 0.7283676160772547e-1;
                c2[3, 2] := 0.5300963437270770;
                c2[4, 1] := 0.7221298133014343e-1;
                c2[4, 2] := 0.5203345998371490;
                c2[5, 1] := 0.7141302173623395e-1;
                c2[5, 2] := 0.5078728971879841;
                c2[6, 1] := 0.7043831559982149e-1;
                c2[6, 2] := 0.4927768111819803;
                c2[7, 1] := 0.6929049381827268e-1;
                c2[7, 2] := 0.4751250308594139;
                c2[8, 1] := 0.6797129849758392e-1;
                c2[8, 2] := 0.4550083840638406;
                c2[9, 1] := 0.6648246325101609e-1;
                c2[9, 2] := 0.4325285673076087;
                c2[10, 1] := 0.6482554675958526e-1;
                c2[10, 2] := 0.4077964789091151;
                c2[11, 1] := 0.6300169683004558e-1;
                c2[11, 2] := 0.3809299858742483;
                c2[12, 1] := 0.6101130648543355e-1;
                c2[12, 2] := 0.3520508315700898;
                c2[13, 1] := 0.5885349417435808e-1;
                c2[13, 2] := 0.3212801560701271;
                c2[14, 1] := 0.5652528148656809e-1;
                c2[14, 2] := 0.2887316252774887;
                c2[15, 1] := 0.5402021575818373e-1;
                c2[15, 2] := 0.2545001287790888;
                c2[16, 1] := 0.5132588802608274e-1;
                c2[16, 2] := 0.2186415296842951;
                c2[17, 1] := 0.4841900639702602e-1;
                c2[17, 2] := 0.1811322622296060;
                c2[18, 1] := 0.4525419574485134e-1;
                c2[18, 2] := 0.1417762065404688;
                c2[19, 1] := 0.4173260173087802e-1;
                c2[19, 2] := 0.9993834530966510e-1;
                c2[20, 1] := 0.3757210572966463e-1;
                c2[20, 2] := 0.5341611499960143e-1;
              else
                Streams.error("Input argument order (= " + String(order) + ") of Bessel filter is not in the range 1..41");
              end if;
              annotation(
                Documentation(info = "<html><p>The transfer function H(p) of a <i>n</i> 'th order Bessel filter is given by</p>
<blockquote><pre>
        Bn(0)
H(p) = -------
        Bn(p)
 </pre>
</blockquote>
<p>with the denominator polynomial</p>
<blockquote><pre>
         n             n  (2n - k)!       p^k
Bn(p) = sum c_k*p^k = sum ----------- * -------   (1)
        k=0           k=0 (n - k)!k!    2^(n-k)
</pre></blockquote>
<p>and the numerator</p>
<blockquote><pre>
               (2n)!     1
Bn(0) = c_0 = ------- * ---- .                    (2)
                n!      2^n
 </pre></blockquote>
<p>Although the coefficients c_k are integer numbers, it is not advisable to use the
polynomials in an unfactorized form because the coefficients are fast growing with order
n (c_0 is approximately 0.3e24 and 0.8e59 for order n=20 and order n=40
respectively).</p>

<p>Therefore, the polynomial Bn(p) is factorized to first and second order polynomials with
real coefficients corresponding to zeros and poles representation that is used in this library.</p>

<p>The function returns the coefficients which resulted from factorization of the normalized transfer function</p>
<blockquote><pre>
H'(p') = H(p),  p' = p/w0
</pre></blockquote>
<p>as well as</p>
<blockquote><pre>
alpha = 1/w0
</pre></blockquote>
<p>the reciprocal of the cut of frequency w0 where the gain of the transfer function is
decreased 3dB.</p>

<p>Both, coefficients and cut off frequency were calculated symbolically and were eventually evaluated
with high precision calculation. The results were stored in this function as real
numbers.</p>

<h4>Calculation of normalized Bessel filter coefficients</h4>
<p>Equation</p>
<blockquote><pre>
abs(H(j*w0)) = abs(Bn(0)/Bn(j*w0)) = 10^(-3/20)
</pre></blockquote>
<p>which must be fulfilled for cut off frequency w = w0 leads to</p>
<blockquote><pre>
[Re(Bn(j*w0))]^2 + [Im(Bn(j*w0))]^2 - (Bn(0)^2)*10^(3/10) = 0
</pre></blockquote>
<p>which has exactly one real solution w0 for each order n. This solutions of w0 are
calculated symbolically first and evaluated by using high precise values of the
coefficients c_k calculated by following (1) and (2).</p>

<p>With w0, the coefficients of the factorized polynomial can be computed by calculating the
zeros of the denominator polynomial</p>
<blockquote><pre>
        n
Bn(p) = sum w0^k*c_k*(p/w0)^k
        k=0
</pre></blockquote>
<p>of the normalized transfer function H'(p'). There exist n/2 of conjugate complex
pairs of zeros (beta +-j*gamma) if n is even and one additional real zero (alpha) if n is
odd. Finally, the coefficients a, b1_k, b2_k of the polynomials</p>
<blockquote><pre> a*p + 1,  n is odd </pre></blockquote>
<p>and</p>
<blockquote><pre>
b2_k*p^2 + b1_k*p + 1,   k = 1,... div(n,2)
</pre></blockquote>
<p>results from</p>
<blockquote><pre>
a = -1/alpha
</pre></blockquote>
<p>and</p>
<blockquote><pre>
b2_k = 1/(beta_k^2 + gamma_k^2) b1_k = -2*beta_k/(beta_k^2 + gamma_k^2)
</pre></blockquote>
</html>"));
            end BesselBaseCoefficients;

            function toHighestPowerOne "Transform filter to form with highest power of s equal 1"
              extends Modelica.Icons.Function;
              input Real den1[:] "[s] coefficients of polynomials (den1[i]*s + 1)";
              input Real den2[:, 2] "[s^2, s] coefficients of polynomials (den2[i,1]*s^2 + den2[i,2]*s + 1)";
              output Real cr[size(den1, 1)] "[s^0] coefficients of polynomials cr[i]*(s+1/cr[i])";
              output Real c0[size(den2, 1)] "[s^0] coefficients of polynomials (s^2 + (den2[i,2]/den2[i,1])*s + (1/den2[i,1]))";
              output Real c1[size(den2, 1)] "[s^1] coefficients of polynomials (s^2 + (den2[i,2]/den2[i,1])*s + (1/den2[i,1]))";
            algorithm
              for i in 1:size(den1, 1) loop
                cr[i] := 1 / den1[i];
              end for;
              for i in 1:size(den2, 1) loop
                c1[i] := den2[i, 2] / den2[i, 1];
                c0[i] := 1 / den2[i, 1];
              end for;
            end toHighestPowerOne;

            function normalizationFactor "Compute correction factor of low pass filter such that amplitude at cut-off frequency is -3db (=10^(-3/20) = 0.70794...)"
              extends Modelica.Icons.Function;
              import Modelica;
              import Modelica.Utilities.Streams;
              input Real c1[:] "[p] coefficients of denominator polynomials (c1[i}*p + 1)";
              input Real c2[:, 2] "[p^2, p] coefficients of denominator polynomials (c2[i,1]*p^2 + c2[i,2]*p + 1)";
              output Real alpha "Correction factor (replace p by alpha*p)";
            protected
              Real alpha_min;
              Real alpha_max;

              function normalizationResidue "Residue of correction factor computation"
                extends Modelica.Icons.Function;
                input Real c1[:] "[p] coefficients of denominator polynomials (c1[i]*p + 1)";
                input Real c2[:, 2] "[p^2, p] coefficients of denominator polynomials (c2[i,1]*p^2 + c2[i,2]*p + 1)";
                input Real alpha;
                output Real residue;
              protected
                constant Real beta = 10 ^ (-3 / 20) "Amplitude of -3db required, i.e., -3db = 20*log(beta)";
                Real cc1;
                Real cc2;
                Real p;
                Real alpha2 = alpha * alpha;
                Real alpha4 = alpha2 * alpha2;
                Real A2 = 1.0;
              algorithm
                assert(size(c1, 1) <= 1, "Internal error 2 (should not occur)");
                if size(c1, 1) == 1 then
                  cc1 := c1[1] * c1[1];
                  p := 1 + cc1 * alpha2;
                  A2 := A2 * p;
                end if;
                for i in 1:size(c2, 1) loop
                  cc1 := c2[i, 2] * c2[i, 2] - 2 * c2[i, 1];
                  cc2 := c2[i, 1] * c2[i, 1];
                  p := 1 + cc1 * alpha2 + cc2 * alpha4;
                  A2 := A2 * p;
                end for;
                residue := 1 / sqrt(A2) - beta;
              end normalizationResidue;

              function findInterval "Find interval for the root"
                extends Modelica.Icons.Function;
                input Real c1[:] "[p] coefficients of denominator polynomials (a*p + 1)";
                input Real c2[:, 2] "[p^2, p] coefficients of denominator polynomials (b*p^2 + a*p + 1)";
                output Real alpha_min;
                output Real alpha_max;
              protected
                Real alpha = 1.0;
                Real residue;
              algorithm
                alpha_min := 0;
                residue := normalizationResidue(c1, c2, alpha);
                if residue < 0 then
                  alpha_max := alpha;
                else
                  while residue >= 0 loop
                    alpha := 1.1 * alpha;
                    residue := normalizationResidue(c1, c2, alpha);
                  end while;
                  alpha_max := alpha;
                end if;
              end findInterval;

              function solveOneNonlinearEquation "Solve f(u) = 0; f(u_min) and f(u_max) must have different signs"
                extends Modelica.Icons.Function;
                import Modelica.Utilities.Streams.error;
                input Real c1[:] "[p] coefficients of denominator polynomials (c1[i]*p + 1)";
                input Real c2[:, 2] "[p^2, p] coefficients of denominator polynomials (c2[i,1]*p^2 + c2[i,2]*p + 1)";
                input Real u_min "Lower bound of search interval";
                input Real u_max "Upper bound of search interval";
                input Real tolerance = 100 * Modelica.Constants.eps "Relative tolerance of solution u";
                output Real u "Value of independent variable so that f(u) = 0";
              protected
                constant Real eps = Modelica.Constants.eps "machine epsilon";
                Real a = u_min "Current best minimum interval value";
                Real b = u_max "Current best maximum interval value";
                Real c "Intermediate point a <= c <= b";
                Real d;
                Real e "b - a";
                Real m;
                Real s;
                Real p;
                Real q;
                Real r;
                Real tol;
                Real fa "= f(a)";
                Real fb "= f(b)";
                Real fc;
                Boolean found = false;
              algorithm
// Check that f(u_min) and f(u_max) have different sign
                fa := normalizationResidue(c1, c2, u_min);
                fb := normalizationResidue(c1, c2, u_max);
                fc := fb;
                if fa > 0.0 and fb > 0.0 or fa < 0.0 and fb < 0.0 then
                  error("The arguments u_min and u_max to solveOneNonlinearEquation(..)\n" + "do not bracket the root of the single non-linear equation:\n" + "  u_min  = " + String(u_min) + "\n" + "  u_max  = " + String(u_max) + "\n" + "  fa = f(u_min) = " + String(fa) + "\n" + "  fb = f(u_max) = " + String(fb) + "\n" + "fa and fb must have opposite sign which is not the case");
                end if;
// Initialize variables
                c := a;
                fc := fa;
                e := b - a;
                d := e;
// Search loop
                while not found loop
                  if abs(fc) < abs(fb) then
                    a := b;
                    b := c;
                    c := a;
                    fa := fb;
                    fb := fc;
                    fc := fa;
                  end if;
                  tol := 2 * eps * abs(b) + tolerance;
                  m := (c - b) / 2;
                  if abs(m) <= tol or fb == 0.0 then
                    found := true;
                    u := b;
                  else
                    if abs(e) < tol or abs(fa) <= abs(fb) then
                      e := m;
                      d := e;
                    else
                      s := fb / fa;
                      if a == c then
                        p := 2 * m * s;
                        q := 1 - s;
                      else
                        q := fa / fc;
                        r := fb / fc;
                        p := s * (2 * m * q * (q - r) - (b - a) * (r - 1));
                        q := (q - 1) * (r - 1) * (s - 1);
                      end if;
                      if p > 0 then
                        q := -q;
                      else
                        p := -p;
                      end if;
                      s := e;
                      e := d;
                      if 2 * p < 3 * m * q - abs(tol * q) and p < abs(0.5 * s * q) then
                        d := p / q;
                      else
                        e := m;
                        d := e;
                      end if;
                    end if;
                    a := b;
                    fa := fb;
                    b := b + (if abs(d) > tol then d else if m > 0 then tol else -tol);
                    fb := normalizationResidue(c1, c2, b);
                    if fb > 0 and fc > 0 or fb < 0 and fc < 0 then
                      c := a;
                      fc := fa;
                      e := b - a;
                      d := e;
                    end if;
                  end if;
                end while;
// root found (interval is small enough)
// Determine if a bisection is needed
// linear interpolation
// inverse quadratic interpolation
// interpolation successful
// use bi-section
// Best guess value is defined as "a"
// initialize variables
                annotation(
                  Documentation(info = "<html>

<p>
This function determines the solution of <b>one non-linear algebraic equation</b> \"y=f(u)\"
in <b>one unknown</b> \"u\" in a reliable way. It is one of the best numerical
algorithms for this purpose. As input, the nonlinear function f(u)
has to be given, as well as an interval u_min, u_max that
contains the solution, i.e., \"f(u_min)\" and \"f(u_max)\" must
have a different sign. If possible, a smaller interval is computed by
inverse quadratic interpolation (interpolating with a quadratic polynomial
through the last 3 points and computing the zero). If this fails,
bisection is used, which always reduces the interval by a factor of 2.
The inverse quadratic interpolation method has superlinear convergence.
This is roughly the same convergence rate as a globally convergent Newton
method, but without the need to compute derivatives of the non-linear
function. The solver function is a direct mapping of the Algol 60 procedure
\"zero\" to Modelica, from:
</p>

<dl>
<dt> Brent R.P.:</dt>
<dd> <b>Algorithms for Minimization without derivatives</b>.
     Prentice Hall, 1973, pp. 58-59.</dd>
</dl>

</html>"));
              end solveOneNonlinearEquation;
            algorithm
// Find interval for alpha
              (alpha_min, alpha_max) := findInterval(c1, c2);
// Compute alpha, so that abs(G(p)) = -3db
              alpha := solveOneNonlinearEquation(c1, c2, alpha_min, alpha_max);
            end normalizationFactor;

            encapsulated function bandPassAlpha "Return alpha for band pass"
              extends Modelica.Icons.Function;
              import Modelica;
              input Real a "Coefficient of s^1";
              input Real b "Coefficient of s^0";
              input Modelica.SIunits.AngularVelocity w "Bandwidth angular frequency";
              output Real alpha "Alpha factor to build up band pass";
            protected
              Real alpha_min;
              Real alpha_max;
              Real z_min;
              Real z_max;
              Real z;

              function residue "Residue of non-linear equation"
                extends Modelica.Icons.Function;
                input Real a;
                input Real b;
                input Real w;
                input Real z;
                output Real res;
              algorithm
                res := z ^ 2 + (a * w * z / (1 + z)) ^ 2 - (2 + b * w ^ 2) * z + 1;
              end residue;

              function solveOneNonlinearEquation "Solve f(u) = 0; f(u_min) and f(u_max) must have different signs"
                extends Modelica.Icons.Function;
                import Modelica.Utilities.Streams.error;
                input Real aa;
                input Real bb;
                input Real ww;
                input Real u_min "Lower bound of search interval";
                input Real u_max "Upper bound of search interval";
                input Real tolerance = 100 * Modelica.Constants.eps "Relative tolerance of solution u";
                output Real u "Value of independent variable so that f(u) = 0";
              protected
                constant Real eps = Modelica.Constants.eps "machine epsilon";
                Real a = u_min "Current best minimum interval value";
                Real b = u_max "Current best maximum interval value";
                Real c "Intermediate point a <= c <= b";
                Real d;
                Real e "b - a";
                Real m;
                Real s;
                Real p;
                Real q;
                Real r;
                Real tol;
                Real fa "= f(a)";
                Real fb "= f(b)";
                Real fc;
                Boolean found = false;
              algorithm
// Check that f(u_min) and f(u_max) have different sign
                fa := residue(aa, bb, ww, u_min);
                fb := residue(aa, bb, ww, u_max);
                fc := fb;
                if fa > 0.0 and fb > 0.0 or fa < 0.0 and fb < 0.0 then
                  error("The arguments u_min and u_max to solveOneNonlinearEquation(..)\n" + "do not bracket the root of the single non-linear equation:\n" + "  u_min  = " + String(u_min) + "\n" + "  u_max  = " + String(u_max) + "\n" + "  fa = f(u_min) = " + String(fa) + "\n" + "  fb = f(u_max) = " + String(fb) + "\n" + "fa and fb must have opposite sign which is not the case");
                end if;
// Initialize variables
                c := a;
                fc := fa;
                e := b - a;
                d := e;
// Search loop
                while not found loop
                  if abs(fc) < abs(fb) then
                    a := b;
                    b := c;
                    c := a;
                    fa := fb;
                    fb := fc;
                    fc := fa;
                  end if;
                  tol := 2 * eps * abs(b) + tolerance;
                  m := (c - b) / 2;
                  if abs(m) <= tol or fb == 0.0 then
                    found := true;
                    u := b;
                  else
                    if abs(e) < tol or abs(fa) <= abs(fb) then
                      e := m;
                      d := e;
                    else
                      s := fb / fa;
                      if a == c then
                        p := 2 * m * s;
                        q := 1 - s;
                      else
                        q := fa / fc;
                        r := fb / fc;
                        p := s * (2 * m * q * (q - r) - (b - a) * (r - 1));
                        q := (q - 1) * (r - 1) * (s - 1);
                      end if;
                      if p > 0 then
                        q := -q;
                      else
                        p := -p;
                      end if;
                      s := e;
                      e := d;
                      if 2 * p < 3 * m * q - abs(tol * q) and p < abs(0.5 * s * q) then
                        d := p / q;
                      else
                        e := m;
                        d := e;
                      end if;
                    end if;
                    a := b;
                    fa := fb;
                    b := b + (if abs(d) > tol then d else if m > 0 then tol else -tol);
                    fb := residue(aa, bb, ww, b);
                    if fb > 0 and fc > 0 or fb < 0 and fc < 0 then
                      c := a;
                      fc := fa;
                      e := b - a;
                      d := e;
                    end if;
                  end if;
                end while;
// root found (interval is small enough)
// Determine if a bisection is needed
// linear interpolation
// inverse quadratic interpolation
// interpolation successful
// use bi-section
// Best guess value is defined as "a"
// initialize variables
                annotation(
                  Documentation(info = "<html>

<p>
This function determines the solution of <b>one non-linear algebraic equation</b> \"y=f(u)\"
in <b>one unknown</b> \"u\" in a reliable way. It is one of the best numerical
algorithms for this purpose. As input, the nonlinear function f(u)
has to be given, as well as an interval u_min, u_max that
contains the solution, i.e., \"f(u_min)\" and \"f(u_max)\" must
have a different sign. If possible, a smaller interval is computed by
inverse quadratic interpolation (interpolating with a quadratic polynomial
through the last 3 points and computing the zero). If this fails,
bisection is used, which always reduces the interval by a factor of 2.
The inverse quadratic interpolation method has superlinear convergence.
This is roughly the same convergence rate as a globally convergent Newton
method, but without the need to compute derivatives of the non-linear
function. The solver function is a direct mapping of the Algol 60 procedure
\"zero\" to Modelica, from:
</p>

<dl>
<dt> Brent R.P.:</dt>
<dd> <b>Algorithms for Minimization without derivatives</b>.
     Prentice Hall, 1973, pp. 58-59.</dd>
</dl>

</html>"));
              end solveOneNonlinearEquation;
            algorithm
              assert(a ^ 2 / 4 - b <= 0, "Band pass transformation cannot be computed");
              z := solveOneNonlinearEquation(a, b, w, 0, 1);
              alpha := sqrt(z);
              annotation(
                Documentation(info = "<html>
<p>
A band pass with bandwidth \"w\" is determined from a low pass
</p>

<pre>
  1/(p^2 + a*p + b)
</pre>

<p>
with the transformation
</p>

<pre>
  new(p) = (p + 1/p)/w
</pre>

<p>
This results in the following derivation:
</p>

<pre>
  1/(p^2 + a*p + b) -> 1/( (p+1/p)^2/w^2 + a*(p + 1/p)/w + b )
                     = 1 /( ( p^2 + 1/p^2 + 2)/w^2 + (p + 1/p)*a/w + b )
                     = w^2*p^2 / (p^4 + 2*p^2 + 1 + (p^3 + p)a*w + b*w^2*p^2)
                     = w^2*p^2 / (p^4 + a*w*p^3 + (2+b*w^2)*p^2 + a*w*p + 1)
</pre>

<p>
This 4th order transfer function shall be split in to two transfer functions of order 2 each
for numerical reasons. With the following formulation, the fourth order
polynomial can be represented (with the unknowns \"c\" and \"alpha\"):
</p>

<pre>
  g(p) = w^2*p^2 / ( (p*alpha)^2 + c*(p*alpha) + 1) * ( (p/alpha)^2 + c*(p/alpha) + 1)
       = w^2*p^2 / ( p^4 + c*(alpha + 1/alpha)*p^3 + (alpha^2 + 1/alpha^2 + c^2)*p^2
                                                   + c*(alpha + 1/alpha)*p + 1 )
</pre>

<p>
Comparison of coefficients:
</p>

<pre>
  c*(alpha + 1/alpha) = a*w           -> c = a*w / (alpha + 1/alpha)
  alpha^2 + 1/alpha^2 + c^2 = 2+b*w^2 -> equation to determine alpha

  alpha^4 + 1 + a^2*w^2*alpha^4/(1+alpha^2)^2 = (2+b*w^2)*alpha^2
    or z = alpha^2
  z^2 + a^2*w^2*z^2/(1+z)^2 - (2+b*w^2)*z + 1 = 0
</pre>

<p>
Therefore the last equation has to be solved for \"z\" (basically, this means to compute
a real zero of a fourth order polynomial):
</p>

<pre>
   solve: 0 = f(z)  = z^2 + a^2*w^2*z^2/(1+z)^2 - (2+b*w^2)*z + 1  for \"z\"
              f(0)  = 1  &gt; 0
              f(1)  = 1 + a^2*w^2/4 - (2+b*w^2) + 1
                    = (a^2/4 - b)*w^2  &lt; 0
                    // since b - a^2/4 > 0 requirement for complex conjugate poles
   -> 0 &lt; z &lt; 1
</pre>

<p>
This function computes the solution of this equation and returns \"alpha = sqrt(z)\";
</p>

</html>"));
            end bandPassAlpha;
          end Utilities;
        end Filter;
      end Internal;
      annotation(
        Documentation(info = "<html>
<p>
This package contains basic <b>continuous</b> input/output blocks
described by differential equations.
</p>

<p>
All blocks of this package can be initialized in different
ways controlled by parameter <b>initType</b>. The possible
values of initType are defined in
<a href=\"modelica://Modelica.Blocks.Types.Init\">Modelica.Blocks.Types.Init</a>:
</p>

<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"><b>Name</b></td>
      <td valign=\"top\"><b>Description</b></td></tr>

  <tr><td valign=\"top\"><b>Init.NoInit</b></td>
      <td valign=\"top\">no initialization (start values are used as guess values with fixed=false)</td></tr>

  <tr><td valign=\"top\"><b>Init.SteadyState</b></td>
      <td valign=\"top\">steady state initialization (derivatives of states are zero)</td></tr>

  <tr><td valign=\"top\"><b>Init.InitialState</b></td>
      <td valign=\"top\">Initialization with initial states</td></tr>

  <tr><td valign=\"top\"><b>Init.InitialOutput</b></td>
      <td valign=\"top\">Initialization with initial outputs (and steady state of the states if possible)</td></tr>
</table>

<p>
For backward compatibility reasons the default of all blocks is
<b>Init.NoInit</b>, with the exception of Integrator and LimIntegrator
where the default is <b>Init.InitialState</b> (this was the initialization
defined in version 2.2 of the Modelica standard library).
</p>

<p>
In many cases, the most useful initial condition is
<b>Init.SteadyState</b> because initial transients are then no longer
present. The drawback is that in combination with a non-linear
plant, non-linear algebraic equations occur that might be
difficult to solve if appropriate guess values for the
iteration variables are not provided (i.e., start values with fixed=false).
However, it is often already useful to just initialize
the linear blocks from the Continuous blocks library in SteadyState.
This is uncritical, because only linear algebraic equations occur.
If Init.NoInit is set, then the start values for the states are
interpreted as <b>guess</b> values and are propagated to the
states with fixed=<b>false</b>.
</p>

<p>
Note, initialization with Init.SteadyState is usually difficult
for a block that contains an integrator
(Integrator, LimIntegrator, PI, PID, LimPID).
This is due to the basic equation of an integrator:
</p>

<pre>
  <b>initial equation</b>
     <b>der</b>(y) = 0;   // Init.SteadyState
  <b>equation</b>
     <b>der</b>(y) = k*u;
</pre>

<p>
The steady state equation leads to the condition that the input to the
integrator is zero. If the input u is already (directly or indirectly) defined
by another initial condition, then the initialization problem is <b>singular</b>
(has none or infinitely many solutions). This situation occurs often
for mechanical systems, where, e.g., u = desiredSpeed - measuredSpeed and
since speed is both a state and a derivative, it is always defined by
Init.InitialState or Init.SteadyState initialization.
</p>

<p>
In such a case, <b>Init.NoInit</b> has to be selected for the integrator
and an additional initial equation has to be added to the system
to which the integrator is connected. E.g., useful initial conditions
for a 1-dim. rotational inertia controlled by a PI controller are that
<b>angle</b>, <b>speed</b>, and <b>acceleration</b> of the inertia are zero.
</p>

</html>"),
        Icon(graphics = {Line(origin = {0.061, 4.184}, points = {{81.939, 36.056}, {65.362, 36.056}, {14.39, -26.199}, {-29.966, 113.485}, {-65.374, -61.217}, {-78.061, -78.184}}, color = {95, 95, 95}, smooth = Smooth.Bezier)}));
    end Continuous;

    package Discrete "Library of discrete input/output blocks with fixed sample period"
      extends Modelica.Icons.Package;

      block Sampler "Ideal sampling of continuous signals"
        extends Interfaces.DiscreteSISO;
      equation
        when {sampleTrigger, initial()} then
          y = u;
        end when;
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100.0, -100.0}, {100.0, 100.0}}, initialScale = 0.1), graphics = {Ellipse(visible = true, lineColor = {0, 0, 127}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, extent = {{25.0, -10.0}, {45.0, 10.0}}), Line(visible = true, points = {{-100.0, 0.0}, {-45.0, 0.0}}, color = {0, 0, 127}), Line(visible = true, points = {{45.0, 0.0}, {100.0, 0.0}}, color = {0, 0, 127}), Line(visible = true, points = {{-35.0, 0.0}, {30.0, 35.0}}, color = {0, 0, 127}), Ellipse(visible = true, lineColor = {0, 0, 127}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, extent = {{-45.0, -10.0}, {-25.0, 10.0}})}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-25, -10}, {-45, 10}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Ellipse(extent = {{45, -10}, {25, 10}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{-100, 0}, {-45, 0}}, color = {0, 0, 255}), Line(points = {{45, 0}, {100, 0}}, color = {0, 0, 255}), Line(points = {{-35, 0}, {30, 35}}, color = {0, 0, 255})}),
          Documentation(info = "<HTML>
<p>
Samples the continues input signal with a sampling rate defined
via parameter <b>samplePeriod</b>.
</p>
</html>"));
      end Sampler;

      block ZeroOrderHold "Zero order hold of a sampled-data system"
        extends Interfaces.DiscreteSISO;
        output Real ySample(start = 0, fixed = true);
      equation
        when {sampleTrigger, initial()} then
          ySample = u;
        end when;
/* Define y=ySample with an infinitesimal delay to break potential
       algebraic loops if both the continuous and the discrete part have
       direct feedthrough
    */
        y = pre(ySample);
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100.0, -100.0}, {100.0, 100.0}}, initialScale = 0.1), graphics = {Line(visible = true, points = {{-78.0, -42.0}, {-52.0, -42.0}, {-52.0, 0.0}, {-26.0, 0.0}, {-26.0, 24.0}, {-6.0, 24.0}, {-6.0, 64.0}, {18.0, 64.0}, {18.0, 20.0}, {38.0, 20.0}, {38.0, 0.0}, {44.0, 0.0}, {44.0, 0.0}, {62.0, 0.0}}, color = {0, 0, 127})}),
          Documentation(info = "<HTML>
<p>
The output is identical to the sampled input signal at sample
time instants and holds the output at the value of the last
sample instant during the sample points.
</p>
</html>"));
      end ZeroOrderHold;

      block FirstOrderHold "First order hold of a sampled-data system"
        extends Modelica.Blocks.Interfaces.DiscreteSISO;
      protected
        Modelica.SIunits.Time tSample;
        Real uSample;
        Real pre_uSample;
        Real c;
      initial equation
        pre(tSample) = time;
        pre(uSample) = u;
        pre(pre_uSample) = u;
        pre(c) = 0.0;
      equation
        when sampleTrigger then
          tSample = time;
          uSample = u;
          pre_uSample = pre(uSample);
          c = if firstTrigger then 0 else (uSample - pre_uSample) / samplePeriod;
        end when;
/* Use pre_uSample and pre(c) to break potential algebraic loops by an
       infinitesimal delay if both the continuous and the discrete part
       have direct feedthrough.
    */
        y = pre_uSample + pre(c) * (time - tSample);
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100.0, -100.0}, {100.0, 100.0}}, initialScale = 0.1), graphics = {Line(visible = true, points = {{-79.0, -41.0}, {-59.0, -33.0}, {-40.0, 1.0}, {-20.0, 9.0}, {0.0, 63.0}, {21.0, 20.0}, {41.0, 10.0}, {60.0, 20.0}}, color = {0, 0, 127}), Line(visible = true, points = {{60.0, 20.0}, {81.0, 10.0}}, color = {0, 0, 127})}),
          Documentation(info = "<HTML>
<p>
The output signal is the extrapolation through the
values of the last two sampled input signals.
</p>
</html>"));
      end FirstOrderHold;

      block UnitDelay "Unit Delay Block"
        parameter Real y_start = 0 "Initial value of output signal";
        extends Interfaces.DiscreteSISO;
      equation
        when sampleTrigger then
          y = pre(u);
        end when;
      initial equation
        y = y_start;
        annotation(
          Documentation(info = "<html>
<p>
This block describes a unit delay:
</p>
<pre>
          1
     y = --- * u
          z
</pre>
<p>
that is, the output signal y is the input signal u of the
previous sample instant. Before the second sample instant,
the output y is identical to parameter yStart.
</p>

</html>"),
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100.0, -100.0}, {100.0, 100.0}}, initialScale = 0.1), graphics = {Line(visible = true, points = {{-30.0, 0.0}, {30.0, 0.0}}, color = {0, 0, 127}), Text(visible = true, lineColor = {0, 0, 127}, extent = {{-90.0, 10.0}, {90.0, 90.0}}, textString = "1"), Text(visible = true, lineColor = {0, 0, 127}, extent = {{-90.0, -90.0}, {90.0, -10.0}}, textString = "z")}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-60, 60}, {60, -60}}, lineColor = {0, 0, 255}), Text(extent = {{-160, 10}, {-140, -10}}, textString = "u", lineColor = {0, 0, 255}), Text(extent = {{115, 10}, {135, -10}}, textString = "y", lineColor = {0, 0, 255}), Line(points = {{-100, 0}, {-60, 0}}, color = {0, 0, 255}), Line(points = {{60, 0}, {100, 0}}, color = {0, 0, 255}), Line(points = {{40, 0}, {-40, 0}}, color = {0, 0, 0}), Text(extent = {{-55, 55}, {55, 5}}, lineColor = {0, 0, 0}, textString = "1"), Text(extent = {{-55, -5}, {55, -55}}, lineColor = {0, 0, 0}, textString = "z")}));
      end UnitDelay;

      block TransferFunction "Discrete Transfer Function block"
        parameter Real b[:] = {1} "Numerator coefficients of transfer function.";
        parameter Real a[:] = {1} "Denominator coefficients of transfer function.";
        extends Interfaces.DiscreteSISO;
        output Real x[size(a, 1) - 1](each start = 0, each fixed = true) "State of transfer function from controller canonical form";
      protected
        parameter Integer nb = size(b, 1) "Size of Numerator of transfer function";
        parameter Integer na = size(a, 1) "Size of Denominator of transfer function";
        Real x1;
        Real xext[size(a, 1)];
      equation
        when sampleTrigger then
/* State variables x are defined according to
       controller canonical form. */
          x1 = (u - a[2:size(a, 1)] * pre(x)) / a[1];
          xext = vector([x1; pre(x)]);
          x = xext[1:size(x, 1)];
          y = vector([zeros(na - nb, 1); b]) * xext;
        end when;
/* This is a non-sampled equation and above there are two separate
       when-clauses. This breaks feedback loops without direct terms,
       since in that case y will be independent of x1 (and only dependent
       on pre(x)).
    */
/* Corresponding (simpler) version using when-semantics of Modelica 1.3:
   equation
     when sampleTrigger then
      [x; xn] = [x1; pre(x)];
      [u] = transpose([a])*[x1; pre(x)];
      [y] = transpose([zeros(na - nb, 1); b])*[x1; pre(x)];
     end when;
*/
        annotation(
          Documentation(info = "<html>
<p>The <b>discrete transfer function</b> block defines the
transfer function between the input signal u and the output
signal y. The numerator has the order nb-1, the denominator
has the order na-1.</p>
<pre>
          b(1)*z^(nb-1) + b(2)*z^(nb-2) + ... + b(nb)
   y(z) = -------------------------------------------- * u(z)
          a(1)*z^(na-1) + a(2)*z^(na-2) + ... + a(na)
</pre>
<p>State variables <b>x</b> are defined according to
<b>controller canonical</b> form. Initial values of the
states can be set as start values of <b>x</b>.</p>
<p>Example:</p>
<pre>     Blocks.Discrete.TransferFunction g(b = {2,4}, a = {1,3});
</pre>
<p>results in the following transfer function:</p>
<pre>        2*z + 4
   y = --------- * u
         z + 3
</pre>

</html>", revisions = "<html>
<p><b>Release Notes:</b></p>
<ul>
<li><i>November 15, 2000</i>
    by <a href=\"http://www.dynasim.se\">Hans Olsson</a>:<br>
    Converted to when-semantics of Modelica 1.4 with special
    care to avoid unnecessary algebraic loops.</li>
<li><i>June 18, 2000</i>
    by <a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a>:<br>
    Realized based on a corresponding model of Dieter Moormann
    and Hilding Elmqvist.</li>
</ul>
</html>"),
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100.0, -100.0}, {100.0, 100.0}}, initialScale = 0.1), graphics = {Line(visible = true, points = {{82.0, 0.0}, {-84.0, 0.0}}, color = {0, 0, 127}), Text(visible = true, lineColor = {0, 0, 127}, extent = {{-92.0, 12.0}, {86.0, 92.0}}, textString = "b(z)"), Text(visible = true, lineColor = {0, 0, 127}, extent = {{-90.0, -90.0}, {90.0, -12.0}}, textString = "a(z)")}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-60, 60}, {60, -60}}, lineColor = {0, 0, 255}), Line(points = {{40, 0}, {-44, 0}}, color = {0, 0, 0}, thickness = 0.5), Text(extent = {{-54, 54}, {54, 4}}, lineColor = {0, 0, 0}, textString = "b(z)"), Text(extent = {{-54, -6}, {56, -56}}, lineColor = {0, 0, 0}, textString = "a(z)"), Line(points = {{-100, 0}, {-60, 0}}, color = {0, 0, 255}), Line(points = {{60, 0}, {100, 0}}, color = {0, 0, 255})}));
      end TransferFunction;

      block StateSpace "Discrete State Space block"
        parameter Real A[:, size(A, 1)] = [1, 0; 0, 1] "Matrix A of state space model";
        parameter Real B[size(A, 1), :] = [1; 1] "Matrix B of state space model";
        parameter Real C[:, size(A, 1)] = [1, 1] "Matrix C of state space model";
        parameter Real D[size(C, 1), size(B, 2)] = zeros(size(C, 1), size(B, 2)) "Matrix D of state space model";
        extends Interfaces.DiscreteMIMO(final nin = size(B, 2), final nout = size(C, 1));
        output Real x[size(A, 1)] "State vector";
      equation
        when sampleTrigger then
          x = A * pre(x) + B * u;
          y = C * pre(x) + D * u;
        end when;
        annotation(
          Documentation(info = "<html>
<p>
The <b>discrete state space</b> block defines the relation
between the input u=inPort.signal and the output
y=outPort.signal in state space form:
</p>
<pre>
    x = A * pre(x) + B * u
    y = C * pre(x) + D * u
</pre>
<p>
where pre(x) is the value of the discrete state x at
the previous sample time instant.
The input is a vector of length nu, the output is a vector
of length ny and nx is the number of states. Accordingly
</p>
<pre>
        A has the dimension: A(nx,nx),
        B has the dimension: B(nx,nu),
        C has the dimension: C(ny,nx),
        D has the dimension: D(ny,nu)
</pre>
<p>
Example:
</p>
<pre>
     parameter: A = [0.12, 2;3, 1.5]
     parameter: B = [2, 7;3, 1]
     parameter: C = [0.1, 2]
     parameter: D = zeros(ny,nu)

results in the following equations:
  [x[1]]   [0.12  2.00] [pre(x[1])]   [2.0  7.0] [u[1]]
  [    ] = [          ]*[         ] + [        ]*[    ]
  [x[2]]   [3.00  1.50] [pre(x[2])]   [0.1  2.0] [u[2]]
                             [pre(x[1])]            [u[1]]
       y[1]   = [0.1  2.0] * [         ] + [0  0] * [    ]
                             [pre(x[2])]            [u[2]]
</pre>
</html>"),
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-90, 15}, {-15, 90}}, textString = "A", lineColor = {0, 0, 127}), Text(extent = {{15, 15}, {90, 90}}, textString = "B", lineColor = {0, 0, 127}), Text(extent = {{-52, 28}, {54, -20}}, textString = "z", lineColor = {0, 0, 127}), Text(extent = {{-90, -15}, {-15, -90}}, textString = "C", lineColor = {0, 0, 127}), Text(extent = {{15, -15}, {90, -90}}, textString = "D", lineColor = {0, 0, 127})}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-60, 60}, {60, -60}}, lineColor = {0, 0, 255}), Text(extent = {{-54, 50}, {52, -10}}, lineColor = {0, 0, 0}, textString = "zx=Ax+Bu"), Text(extent = {{-56, 14}, {54, -50}}, lineColor = {0, 0, 0}, textString = "  y=Cx+Du"), Line(points = {{-102, 0}, {-60, 0}}, color = {0, 0, 255}), Line(points = {{60, 0}, {100, 0}}, color = {0, 0, 255})}));
      end StateSpace;

      block TriggeredSampler "Triggered sampling of continuous signals"
        extends Modelica.Blocks.Icons.DiscreteBlock;
        parameter Real y_start = 0 "initial value of output signal";
        Modelica.Blocks.Interfaces.RealInput u "Connector with a Real input signal" annotation(
          Placement(transformation(extent = {{-140, -20}, {-100, 20}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealOutput y "Connector with a Real output signal" annotation(
          Placement(transformation(extent = {{100, -10}, {120, 10}}, rotation = 0)));
        Modelica.Blocks.Interfaces.BooleanInput trigger annotation(
          Placement(transformation(origin = {0, -118}, extent = {{-20, -20}, {20, 20}}, rotation = 90)));
      equation
        when trigger then
          y = u;
        end when;
      initial equation
        y = y_start;
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100.0, -100.0}, {100.0, 100.0}}, initialScale = 0.1), graphics = {Ellipse(visible = true, lineColor = {0, 0, 127}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, extent = {{25.0, -10.0}, {45.0, 10.0}}), Line(visible = true, points = {{-100.0, 0.0}, {-45.0, 0.0}}, color = {0, 0, 127}), Line(visible = true, points = {{45.0, 0.0}, {100.0, 0.0}}, color = {0, 0, 127}), Line(visible = true, points = {{0.0, -100.0}, {0.0, -26.0}}, color = {255, 0, 255}), Line(visible = true, points = {{-35.0, 0.0}, {28.0, -48.0}}, color = {0, 0, 127}), Ellipse(visible = true, lineColor = {0, 0, 127}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, extent = {{-45.0, -10.0}, {-25.0, 10.0}})}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-25, -10}, {-45, 10}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Ellipse(extent = {{45, -10}, {25, 10}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{-100, 0}, {-45, 0}}, color = {0, 0, 255}), Line(points = {{45, 0}, {100, 0}}, color = {0, 0, 255}), Line(points = {{-35, 0}, {28, -48}}, color = {0, 0, 255}), Line(points = {{0, -100}, {0, -26}}, color = {255, 0, 255})}),
          Documentation(info = "<HTML>
<p>
Samples the continuous input signal whenever the trigger input
signal is rising (i.e., trigger changes from <b>false</b> to
<b>true</b>) and provides the sampled input signal as output.
Before the first sampling, the output signal is equal to
the initial value defined via parameter <b>y0</b>.
</p>
</html>"));
      end TriggeredSampler;

      block TriggeredMax "Compute maximum, absolute value of continuous signal at trigger instants"
        extends Modelica.Blocks.Icons.DiscreteBlock;
        Modelica.Blocks.Interfaces.RealInput u "Connector with a Real input signal" annotation(
          Placement(transformation(extent = {{-140, -20}, {-100, 20}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealOutput y "Connector with a Real output signal" annotation(
          Placement(transformation(extent = {{100, -10}, {120, 10}}, rotation = 0)));
        Modelica.Blocks.Interfaces.BooleanInput trigger annotation(
          Placement(transformation(origin = {0, -118}, extent = {{-20, -20}, {20, 20}}, rotation = 90)));
      equation
        when trigger then
          y = max(pre(y), abs(u));
        end when;
      initial equation
        y = 0;
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100.0, -100.0}, {100.0, 100.0}}, initialScale = 0.1), graphics = {Ellipse(visible = true, lineColor = {0, 0, 127}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, extent = {{25.0, -10.0}, {45.0, 10.0}}), Line(visible = true, points = {{-100.0, 0.0}, {-45.0, 0.0}}, color = {0, 0, 127}), Line(visible = true, points = {{45.0, 0.0}, {100.0, 0.0}}, color = {0, 0, 127}), Line(visible = true, points = {{0.0, -100.0}, {0.0, -26.0}}, color = {255, 0, 255}), Line(visible = true, points = {{-35.0, 0.0}, {28.0, -48.0}}, color = {0, 0, 127}), Text(visible = true, extent = {{-86.0, 24.0}, {82.0, 82.0}}, color = {0, 0, 127}, textString = "max"), Ellipse(visible = true, lineColor = {0, 0, 127}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, extent = {{-45.0, -10.0}, {-25.0, 10.0}})}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-25, -10}, {-45, 10}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Ellipse(extent = {{45, -10}, {25, 10}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{-100, 0}, {-45, 0}}, color = {0, 0, 255}), Line(points = {{45, 0}, {100, 0}}, color = {0, 0, 255}), Line(points = {{-35, 0}, {28, -48}}, color = {0, 0, 255}), Line(points = {{0, -100}, {0, -26}}, color = {255, 0, 255})}),
          Documentation(info = "<HTML>
<p>
Samples the continuous input signal whenever the trigger input
signal is rising (i.e., trigger changes from <b>false</b> to
<b>true</b>). The maximum, absolute value of the input signal
at the sampling point is provided as output signal.
</p>
</html>"));
      end TriggeredMax;
      annotation(
        Documentation(info = "<html>
<p>
This package contains <b>discrete control blocks</b>
with <b>fixed sample period</b>.
Every component of this package is structured in the following way:
</p>
<ol>
<li> A component has <b>continuous real</b> input and output signals.</li>
<li> The <b>input</b> signals are <b>sampled</b> by the given sample period
     defined via parameter <b>samplePeriod</b>.
     The first sample instant is defined by parameter <b>startTime</b>.
<li> The <b>output</b> signals are computed from the sampled input signals.
</ol>
<p>
A <b>sampled data system</b> may consist of components of package <b>Discrete</b>
and of every other purely <b>algebraic</b> input/output block, such
as the components of packages <b>Modelica.Blocks.Math</b>,
<b>Modelica.Blocks.Nonlinear</b> or <b>Modelica.Blocks.Sources</b>.
</p>

</html>", revisions = "<html>
<ul>
<li><i>October 21, 2002</i>
       by <a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a>:<br>
       New components TriggeredSampler and TriggeredMax added.</li>
<li><i>June 18, 2000</i>
       by <a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a>:<br>
       Realized based on a corresponding library of Dieter Moormann and
       Hilding Elmqvist.</li>
</ul>
</html>"),
        Icon(graphics = {Line(points = {{-88, 0}, {-45, 0}}, color = {95, 95, 95}), Ellipse(lineColor = {95, 95, 95}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, extent = {{-45, -10}, {-25, 10}}), Line(points = {{-35, 0}, {24, 52}}, color = {95, 95, 95}), Ellipse(lineColor = {95, 95, 95}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, extent = {{25, -10}, {45, 10}}), Line(points = {{45, 0}, {82, 0}}, color = {95, 95, 95})}));
    end Discrete;

    package Interaction "Library of user interaction blocks to input and to show variables in a diagram animation"
      extends Modelica.Icons.Package;

      package Show "Library of blocks to show variables in a diagram animation"
        extends Modelica.Icons.Package;

        block RealValue "Show Real value from numberPort or from number input field in diagram layer dynamically"
          parameter Boolean use_numberPort = true "= true, if numberPort enabled" annotation(
            Evaluate = true,
            HideResult = true,
            choices(checkBox = true));
          input Real number = 0.0 "Number to visualize if use_numberPort=false (time varying)" annotation(
            Dialog(enable = not use_numberPort),
            HideResult = true);
          parameter Integer significantDigits(min = 1) = 2 "Number of significant digits to be shown";
          Modelica.Blocks.Interfaces.RealInput numberPort if use_numberPort "Number to be shown in diagram layer if use_numberPort = true" annotation(
            HideResult = true,
            Placement(transformation(extent = {{-130, -15}, {-100, 15}})));
          Modelica.Blocks.Interfaces.RealOutput showNumber;
        equation
          if use_numberPort then
            connect(numberPort, showNumber);
          else
            showNumber = number;
          end if;
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100.0, -100.0}, {100.0, 100.0}}, initialScale = 0.1), graphics = {Rectangle(visible = true, lineColor = {0, 0, 127}, fillColor = {236, 233, 216}, fillPattern = FillPattern.Solid, lineThickness = 5.0, borderPattern = BorderPattern.Raised, extent = {{-100.0, -40.0}, {100.0, 40.0}}), Text(visible = true, extent = {{-94.0, -34.0}, {96.0, 34.0}}, textString = DynamicSelect("0.0", String(showNumber, significantDigits = significantDigits))), Text(visible = not use_numberPort, extent = {{-150.0, -70.0}, {150.0, -50.0}}, textString = "%number")}),
            Documentation(info = "<html>
<p>
This block visualizes a Real number in a diagram animation.
The number to be visualized can be defined in the following ways:
</p>

<ul>
<li> If useNumberPort = <b>true</b> (which is the default), a Real
     input is present and this input variable is shown.</li>

<li> If useNumberPort = <b>false</b> no input connector is present.
     Instead, a Real input field is activated in the parameter menu
     and the Real expression from this input menu is shown.</li>
</ul>

<p>
The two versions of the block are shown in the following image (in the right variant, the
name of the variable value that is displayed is also shown below the icon):
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Blocks/Interaction/RealValue.png\"
     alt=\"RealValue.png\">
</p>

<p>
The usage is demonstrated, e.g., in example
<a href=\"modelica://Modelica.Blocks.Examples.RealNetwork1\">Modelica.Blocks.Examples.RealNetwork1</a>.
</p>
</html>"));
        end RealValue;

        block IntegerValue "Show Integer value from numberPort or from number input field in diagram layer dynamically"
          parameter Boolean use_numberPort = true "= true, if numberPort enabled" annotation(
            Evaluate = true,
            HideResult = true,
            choices(checkBox = true));
          input Integer number = 0 "Number to visualize if use_numberPort=false (time varying)" annotation(
            Dialog(enable = not use_numberPort),
            HideResult = true);
          Modelica.Blocks.Interfaces.IntegerInput numberPort if use_numberPort "Number to be shown in diagram layer if use_numberPort = true" annotation(
            HideResult = true,
            Placement(transformation(extent = {{-130, -15}, {-100, 15}})));
          Modelica.Blocks.Interfaces.IntegerOutput showNumber;
        equation
          if use_numberPort then
            connect(numberPort, showNumber);
          else
            showNumber = number;
          end if;
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100.0, -100.0}, {100.0, 100.0}}, initialScale = 0.1), graphics = {Rectangle(visible = true, lineColor = {0, 0, 127}, fillColor = {236, 233, 216}, fillPattern = FillPattern.Solid, lineThickness = 5.0, borderPattern = BorderPattern.Raised, extent = {{-100.0, -40.0}, {100.0, 40.0}}), Text(visible = true, extent = {{-94.0, -34.0}, {96.0, 34.0}}, textString = DynamicSelect("0", String(showNumber))), Text(visible = not use_numberPort, extent = {{-150.0, -70.0}, {150.0, -50.0}}, textString = "%number")}),
            Documentation(info = "<html>
<p>
This block visualizes an Integer number in a diagram animation.
The number to be visualized can be defined in the following ways:
</p>

<ul>
<li> If useNumberPort = <b>true</b> (which is the default), an Integer
     input is present and this input variable is shown.</li>

<li> If useNumberPort = <b>false</b> no input connector is present.
     Instead, an Integer input field is activated in the parameter menu
     and the Integer expression from this input menu is shown.</li>
</ul>

<p>
The two versions of the block are shown in the following image (in the right variant, the
name of the variable value that is displayed is also shown below the icon):
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Blocks/Interaction/IntegerValue.png\"
     alt=\"IntegerValue.png\">
</p>

<p>
The usage is demonstrated, e.g., in example
<a href=\"modelica://Modelica.Blocks.Examples.IntegerNetwork1\">Modelica.Blocks.Examples.IntegerNetwork1</a>.
</p>
</html>"));
        end IntegerValue;

        block BooleanValue "Show Boolean value from numberPort or from number input field in diagram layer dynamically"
          parameter Boolean use_activePort = true "= true, if activePort enabled" annotation(
            Evaluate = true,
            HideResult = true,
            choices(checkBox = true));
          input Boolean active = false "Boolean variable to visualize if use_activePort=false (time varying)" annotation(
            Dialog(enable = not use_activePort),
            HideResult = true);
          Modelica.Blocks.Interfaces.BooleanInput activePort if use_activePort "Boolean variable to be shown in diagram layer if use_activePort = true" annotation(
            HideResult = true,
            Placement(transformation(extent = {{-130, -15}, {-100, 15}})));
          Modelica.Blocks.Interfaces.BooleanOutput showActive;
        equation
          if use_activePort then
            connect(activePort, showActive);
          else
            showActive = active;
          end if;
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100.0, -100.0}, {100.0, 100.0}}, initialScale = 0.1), graphics = {Text(visible = not use_activePort, extent = {{-188.0, -80.0}, {62.0, -60.0}}, textString = "%active"), Ellipse(visible = true, lineColor = {64, 64, 64}, fillColor = DynamicSelect({192, 192, 192}, if showActive > 0.5 then {0, 255, 0} else {235, 235, 235}), pattern = LinePattern.None, fillPattern = FillPattern.Sphere, extent = {{-100.0, -40.0}, {-20.0, 40.0}})}),
            Documentation(info = "<html>
<p>
This block visualizes a Boolean variable in a diagram animation.
The Boolean variable to be visualized can be defined in the following ways:
</p>

<ul>
<li> If useActivePort = <b>true</b> (which is the default), a Boolean
     input is present and this input variable is shown.</li>

<li> If useActivePort = <b>false</b> no input connector is present.
     Instead, a Boolean input field is activated in the parameter menu
     and the Boolean expression from this input menu is shown.</li>
</ul>

<p>
If the Boolean variable is <b>false</b> the block is \"grey\", otherwise, it is \"green\".
The two versions of the block are shown in the following image (in the right variant, the
name of the variable value that is displayed is also shown below the icon):
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Blocks/Interaction/BooleanValue.png\"
     alt=\"BooleanValue.png\">
</p>

<p>
The usage is demonstrated, e.g., in example
<a href=\"modelica://Modelica.Blocks.Examples.BooleanNetwork1\">Modelica.Blocks.Examples.BooleanNetwork1</a>.
</p>
</html>"));
        end BooleanValue;
      end Show;
      annotation(
        Icon(graphics = {Text(extent = {{-98, -30}, {96, 34}}, textString = "0", lineColor = {0, 0, 0})}));
    end Interaction;

    package Interfaces "Library of connectors and partial models for input/output blocks"
      import Modelica.SIunits;
      extends Modelica.Icons.InterfacesPackage;
      connector RealInput = input Real "'input Real' as connector" annotation(
        defaultComponentName = "u",
        Icon(graphics = {Polygon(lineColor = {0, 0, 127}, fillColor = {0, 0, 127}, fillPattern = FillPattern.Solid, points = {{-100.0, 100.0}, {100.0, 0.0}, {-100.0, -100.0}})}, coordinateSystem(extent = {{-100.0, -100.0}, {100.0, 100.0}}, preserveAspectRatio = true, initialScale = 0.2)),
        Diagram(coordinateSystem(preserveAspectRatio = true, initialScale = 0.2, extent = {{-100.0, -100.0}, {100.0, 100.0}}), graphics = {Polygon(lineColor = {0, 0, 127}, fillColor = {0, 0, 127}, fillPattern = FillPattern.Solid, points = {{0.0, 50.0}, {100.0, 0.0}, {0.0, -50.0}, {0.0, 50.0}}), Text(lineColor = {0, 0, 127}, extent = {{-10.0, 60.0}, {-10.0, 85.0}}, textString = "%name")}),
        Documentation(info = "<html>
<p>
Connector with one input signal of type Real.
</p>
</html>"));
      connector RealOutput = output Real "'output Real' as connector" annotation(
        defaultComponentName = "y",
        Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100.0, -100.0}, {100.0, 100.0}}, initialScale = 0.1), graphics = {Polygon(lineColor = {0, 0, 127}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, points = {{-100.0, 100.0}, {100.0, 0.0}, {-100.0, -100.0}})}),
        Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100.0, -100.0}, {100.0, 100.0}}, initialScale = 0.1), graphics = {Polygon(lineColor = {0, 0, 127}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, points = {{-100.0, 50.0}, {0.0, 0.0}, {-100.0, -50.0}}), Text(lineColor = {0, 0, 127}, extent = {{30.0, 60.0}, {30.0, 110.0}}, textString = "%name")}),
        Documentation(info = "<html>
<p>
Connector with one output signal of type Real.
</p>
</html>"));
      connector BooleanInput = input Boolean "'input Boolean' as connector" annotation(
        defaultComponentName = "u",
        Icon(graphics = {Polygon(points = {{-100, 100}, {100, 0}, {-100, -100}, {-100, 100}}, lineColor = {255, 0, 255}, fillColor = {255, 0, 255}, fillPattern = FillPattern.Solid)}, coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = true, initialScale = 0.2)),
        Diagram(coordinateSystem(preserveAspectRatio = true, initialScale = 0.2, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{0, 50}, {100, 0}, {0, -50}, {0, 50}}, lineColor = {255, 0, 255}, fillColor = {255, 0, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-10, 85}, {-10, 60}}, lineColor = {255, 0, 255}, textString = "%name")}),
        Documentation(info = "<html>
<p>
Connector with one input signal of type Boolean.
</p>
</html>"));
      connector BooleanOutput = output Boolean "'output Boolean' as connector" annotation(
        defaultComponentName = "y",
        Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{-100, 100}, {100, 0}, {-100, -100}, {-100, 100}}, lineColor = {255, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid)}),
        Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{-100, 50}, {0, 0}, {-100, -50}, {-100, 50}}, lineColor = {255, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Text(extent = {{30, 110}, {30, 60}}, lineColor = {255, 0, 255}, textString = "%name")}),
        Documentation(info = "<html>
<p>
Connector with one output signal of type Boolean.
</p>
</html>"));
      connector IntegerInput = input Integer "'input Integer' as connector" annotation(
        defaultComponentName = "u",
        Icon(graphics = {Polygon(points = {{-100, 100}, {100, 0}, {-100, -100}, {-100, 100}}, lineColor = {255, 127, 0}, fillColor = {255, 127, 0}, fillPattern = FillPattern.Solid)}, coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = true, initialScale = 0.2)),
        Diagram(coordinateSystem(preserveAspectRatio = true, initialScale = 0.2, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{0, 50}, {100, 0}, {0, -50}, {0, 50}}, lineColor = {255, 127, 0}, fillColor = {255, 127, 0}, fillPattern = FillPattern.Solid), Text(extent = {{-10, 85}, {-10, 60}}, lineColor = {255, 127, 0}, textString = "%name")}),
        Documentation(info = "<html>
<p>
Connector with one input signal of type Integer.
</p>
</html>"));
      connector IntegerOutput = output Integer "'output Integer' as connector" annotation(
        defaultComponentName = "y",
        Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{-100, 100}, {100, 0}, {-100, -100}, {-100, 100}}, lineColor = {255, 127, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid)}),
        Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{-100, 50}, {0, 0}, {-100, -50}, {-100, 50}}, lineColor = {255, 127, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Text(extent = {{30, 110}, {30, 60}}, lineColor = {255, 127, 0}, textString = "%name")}),
        Documentation(info = "<html>
<p>
Connector with one output signal of type Integer.
</p>
</html>"));
      connector RealVectorInput = input Real "Real input connector used for vector of connectors" annotation(
        defaultComponentName = "u",
        Icon(graphics = {Ellipse(extent = {{-100, 100}, {100, -100}}, lineColor = {0, 0, 127}, fillColor = {0, 0, 127}, fillPattern = FillPattern.Solid)}, coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = true, initialScale = 0.2)),
        Diagram(coordinateSystem(preserveAspectRatio = false, initialScale = 0.2, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-10, 85}, {-10, 60}}, lineColor = {0, 0, 127}, textString = "%name"), Ellipse(extent = {{-50, 50}, {50, -50}}, lineColor = {0, 0, 127}, fillColor = {0, 0, 127}, fillPattern = FillPattern.Solid)}),
        Documentation(info = "<html>
<p>
Real input connector that is used for a vector of connectors,
for example <a href=\"modelica://Modelica.Blocks.Interfaces.PartialRealMISO\">PartialRealMISO</a>,
and has therefore a different icon as RealInput connector.
</p>
</html>"));
      connector IntegerVectorInput = input Integer "Integer input connector used for vector of connectors" annotation(
        defaultComponentName = "u",
        Icon(graphics = {Ellipse(extent = {{-100, 100}, {100, -100}}, lineColor = {255, 128, 0}, fillColor = {255, 128, 0}, fillPattern = FillPattern.Solid)}, coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = true, initialScale = 0.2)),
        Diagram(coordinateSystem(preserveAspectRatio = false, initialScale = 0.2, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-10, 85}, {-10, 60}}, lineColor = {255, 128, 0}, textString = "%name"), Ellipse(extent = {{-50, 50}, {50, -50}}, lineColor = {255, 128, 0}, fillColor = {255, 128, 0}, fillPattern = FillPattern.Solid)}),
        Documentation(info = "<html>

<p>
Integer input connector that is used for a vector of connectors,
for example <a href=\"modelica://Modelica.Blocks.Interfaces.PartialIntegerMISO\">PartialIntegerMISO</a>,
and has therefore a different icon as IntegerInput connector.
</p>

</html>"));
      connector BooleanVectorInput = input Boolean "Boolean input connector used for vector of connectors" annotation(
        defaultComponentName = "u",
        Icon(graphics = {Ellipse(extent = {{-100, -100}, {100, 100}}, lineColor = {255, 0, 255}, fillColor = {255, 0, 255}, fillPattern = FillPattern.Solid)}, coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = false, initialScale = 0.2)),
        Diagram(coordinateSystem(preserveAspectRatio = false, initialScale = 0.2, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-10, 85}, {-10, 60}}, lineColor = {255, 0, 255}, textString = "%name"), Ellipse(extent = {{-50, 50}, {50, -50}}, lineColor = {255, 0, 255}, fillColor = {255, 0, 255}, fillPattern = FillPattern.Solid)}),
        Documentation(info = "<html>
<p>
Boolean input connector that is used for a vector of connectors,
for example <a href=\"modelica://Modelica.Blocks.Interfaces.PartialBooleanMISO\">PartialBooleanMISO</a>,
and has therefore a different icon as BooleanInput connector.
</p>
</html>"));

      partial block SO "Single Output continuous control block"
        extends Modelica.Blocks.Icons.Block;
        RealOutput y "Connector of Real output signal" annotation(
          Placement(transformation(extent = {{100, -10}, {120, 10}}, rotation = 0)));
        annotation(
          Documentation(info = "<html>
<p>
Block has one continuous Real output signal.
</p>
</html>"));
      end SO;

      partial block MO "Multiple Output continuous control block"
        extends Modelica.Blocks.Icons.Block;
        parameter Integer nout(min = 1) = 1 "Number of outputs";
        RealOutput y[nout] "Connector of Real output signals" annotation(
          Placement(transformation(extent = {{100, -10}, {120, 10}}, rotation = 0)));
        annotation(
          Documentation(info = "<html>
<p>
Block has one continuous Real output signal vector.
</p>
</html>"));
      end MO;

      partial block SISO "Single Input Single Output continuous control block"
        extends Modelica.Blocks.Icons.Block;
        RealInput u "Connector of Real input signal" annotation(
          Placement(transformation(extent = {{-140, -20}, {-100, 20}}, rotation = 0)));
        RealOutput y "Connector of Real output signal" annotation(
          Placement(transformation(extent = {{100, -10}, {120, 10}}, rotation = 0)));
        annotation(
          Documentation(info = "<html>
<p>
Block has one continuous Real input and one continuous Real output signal.
</p>
</html>"));
      end SISO;

      partial block SI2SO "2 Single Input / 1 Single Output continuous control block"
        extends Modelica.Blocks.Icons.Block;
        RealInput u1 "Connector of Real input signal 1" annotation(
          Placement(transformation(extent = {{-140, 40}, {-100, 80}}, rotation = 0)));
        RealInput u2 "Connector of Real input signal 2" annotation(
          Placement(transformation(extent = {{-140, -80}, {-100, -40}}, rotation = 0)));
        RealOutput y "Connector of Real output signal" annotation(
          Placement(transformation(extent = {{100, -10}, {120, 10}}, rotation = 0)));
        annotation(
          Documentation(info = "<html>
<p>
Block has two continuous Real input signals u1 and u2 and one
continuous Real output signal y.
</p>
</html>"));
      end SI2SO;

      partial block SIMO "Single Input Multiple Output continuous control block"
        extends Modelica.Blocks.Icons.Block;
        parameter Integer nout = 1 "Number of outputs";
        RealInput u "Connector of Real input signal" annotation(
          Placement(transformation(extent = {{-140, -20}, {-100, 20}}, rotation = 0)));
        RealOutput y[nout] "Connector of Real output signals" annotation(
          Placement(transformation(extent = {{100, -10}, {120, 10}}, rotation = 0)));
        annotation(
          Documentation(info = "<HTML>
<p> Block has one continuous Real input signal and a
    vector of continuous Real output signals.</p>

</html>"));
      end SIMO;

      partial block MISO "Multiple Input Single Output continuous control block"
        extends Modelica.Blocks.Icons.Block;
        parameter Integer nin = 1 "Number of inputs";
        RealInput u[nin] "Connector of Real input signals" annotation(
          Placement(transformation(extent = {{-140, -20}, {-100, 20}}, rotation = 0)));
        RealOutput y "Connector of Real output signal" annotation(
          Placement(transformation(extent = {{100, -10}, {120, 10}}, rotation = 0)));
        annotation(
          Documentation(info = "<HTML>
<p>
Block has a vector of continuous Real input signals and
one continuous Real output signal.
</p>
</html>"));
      end MISO;

      partial block PartialRealMISO "Partial block with a RealVectorInput and a RealOutput signal"
        parameter Integer significantDigits(min = 1) = 3 "Number of significant digits to be shown in dynamic diagram layer for y" annotation(
          Dialog(tab = "Advanced"));
        parameter Integer nu(min = 0) = 0 "Number of input connections" annotation(
          Dialog(connectorSizing = true),
          HideResult = true);
        Modelica.Blocks.Interfaces.RealVectorInput u[nu] annotation(
          Placement(transformation(extent = {{-120, 70}, {-80, -70}})));
        Modelica.Blocks.Interfaces.RealOutput y annotation(
          Placement(transformation(extent = {{100, -17}, {134, 17}})));
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}, initialScale = 0.06), graphics = {Text(extent = {{110, -50}, {300, -70}}, lineColor = {0, 0, 0}, textString = DynamicSelect(" ", String(y, significantDigits = significantDigits))), Text(extent = {{-250, 170}, {250, 110}}, textString = "%name", lineColor = {0, 0, 255}), Rectangle(extent = {{-100, 100}, {100, -100}}, lineColor = {255, 137, 0}, lineThickness = 5.0, fillColor = {255, 255, 255}, borderPattern = BorderPattern.Raised, fillPattern = FillPattern.Solid)}));
      end PartialRealMISO;

      partial block MIMO "Multiple Input Multiple Output continuous control block"
        extends Modelica.Blocks.Icons.Block;
        parameter Integer nin = 1 "Number of inputs";
        parameter Integer nout = 1 "Number of outputs";
        RealInput u[nin] "Connector of Real input signals" annotation(
          Placement(transformation(extent = {{-140, -20}, {-100, 20}}, rotation = 0)));
        RealOutput y[nout] "Connector of Real output signals" annotation(
          Placement(transformation(extent = {{100, -10}, {120, 10}}, rotation = 0)));
        annotation(
          Documentation(info = "<HTML>
<p>
Block has a continuous Real input and a continuous Real output signal vector.
The signal sizes of the input and output vector may be different.
</p>
</html>"));
      end MIMO;

      partial block MIMOs "Multiple Input Multiple Output continuous control block with same number of inputs and outputs"
        extends Modelica.Blocks.Icons.Block;
        parameter Integer n = 1 "Number of inputs (= number of outputs)";
        RealInput u[n] "Connector of Real input signals" annotation(
          Placement(transformation(extent = {{-140, -20}, {-100, 20}}, rotation = 0)));
        RealOutput y[n] "Connector of Real output signals" annotation(
          Placement(transformation(extent = {{100, -10}, {120, 10}}, rotation = 0)));
        annotation(
          Documentation(info = "<HTML>
<p>
Block has a continuous Real input and a continuous Real output signal vector
where the signal sizes of the input and output vector are identical.
</p>
</html>"));
      end MIMOs;

      partial block MI2MO "2 Multiple Input / Multiple Output continuous control block"
        extends Modelica.Blocks.Icons.Block;
        parameter Integer n = 1 "Dimension of input and output vectors.";
        RealInput u1[n] "Connector 1 of Real input signals" annotation(
          Placement(transformation(extent = {{-140, 40}, {-100, 80}}, rotation = 0)));
        RealInput u2[n] "Connector 2 of Real input signals" annotation(
          Placement(transformation(extent = {{-140, -80}, {-100, -40}}, rotation = 0)));
        RealOutput y[n] "Connector of Real output signals" annotation(
          Placement(transformation(extent = {{100, -10}, {120, 10}}, rotation = 0)));
        annotation(
          Documentation(info = "<html>
<p>
Block has two continuous Real input vectors u1 and u2 and one
continuous Real output vector y.
All vectors have the same number of elements.
</p>
</html>"));
      end MI2MO;

      partial block SignalSource "Base class for continuous signal source"
        extends SO;
        parameter Real offset = 0 "Offset of output signal y";
        parameter SIunits.Time startTime = 0 "Output y = offset for time < startTime";
        annotation(
          Documentation(info = "<html>
<p>
Basic block for Real sources of package Blocks.Sources.
This component has one continuous Real output signal y
and two parameters (offset, startTime) to shift the
generated signal.
</p>
</html>"));
      end SignalSource;

      partial block SVcontrol "Single-Variable continuous controller"
        extends Modelica.Blocks.Icons.Block;
        RealInput u_s "Connector of setpoint input signal" annotation(
          Placement(transformation(extent = {{-140, -20}, {-100, 20}}, rotation = 0)));
        RealInput u_m "Connector of measurement input signal" annotation(
          Placement(transformation(origin = {0, -120}, extent = {{20, -20}, {-20, 20}}, rotation = 270)));
        RealOutput y "Connector of actuator output signal" annotation(
          Placement(transformation(extent = {{100, -10}, {120, 10}}, rotation = 0)));
        annotation(
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-102, 34}, {-142, 24}}, textString = "(setpoint)", lineColor = {0, 0, 255}), Text(extent = {{100, 24}, {140, 14}}, textString = "(actuator)", lineColor = {0, 0, 255}), Text(extent = {{-83, -112}, {-33, -102}}, textString = " (measurement)", lineColor = {0, 0, 255})}),
          Documentation(info = "<html>
<p>
Block has two continuous Real input signals and one
continuous Real output signal. The block is designed
to be used as base class for a corresponding controller.
</p>
</html>"));
      end SVcontrol;

      partial block MVcontrol "Multi-Variable continuous controller"
        extends Modelica.Blocks.Icons.Block;
        parameter Integer nu_s = 1 "Number of setpoint inputs";
        parameter Integer nu_m = 1 "Number of measurement inputs";
        parameter Integer ny = 1 "Number of actuator outputs";
        RealInput u_s[nu_s] "Connector of setpoint input signals" annotation(
          Placement(transformation(extent = {{-140, -20}, {-100, 20}}, rotation = 0)));
        RealInput u_m[nu_m] "Connector of measurement input signals" annotation(
          Placement(transformation(origin = {0, -120}, extent = {{20, -20}, {-20, 20}}, rotation = 270)));
        RealOutput y[ny] "Connector of actuator output signals" annotation(
          Placement(transformation(extent = {{100, -10}, {120, 10}}, rotation = 0)));
        annotation(
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-100, 36}, {-140, 26}}, textString = "(setpoint)", lineColor = {0, 0, 255}), Text(extent = {{102, 24}, {142, 14}}, textString = "(actuator)", lineColor = {0, 0, 255}), Text(extent = {{-75, -108}, {-25, -98}}, textString = " (measurement)", lineColor = {0, 0, 255})}),
          Documentation(info = "<html>
<p>
Block has two continuous Real input signal vectors and one
continuous Real output signal vector. The block is designed
to be used as base class for a corresponding controller.
</p>
</html>"));
      end MVcontrol;

      partial block DiscreteBlock "Base class of discrete control blocks"
        extends Modelica.Blocks.Icons.DiscreteBlock;
        parameter SI.Time samplePeriod(min = 100 * Modelica.Constants.eps, start = 0.1) "Sample period of component";
        parameter SI.Time startTime = 0 "First sample time instant";
      protected
        output Boolean sampleTrigger "True, if sample time instant";
        output Boolean firstTrigger "Rising edge signals first sample instant";
      equation
        sampleTrigger = sample(startTime, samplePeriod);
        when sampleTrigger then
          firstTrigger = time <= startTime + samplePeriod / 2;
        end when;
        annotation(
          Documentation(info = "<html>
<p>
Basic definitions of a discrete block of library
Blocks.Discrete.
</p>
</html>"));
      end DiscreteBlock;

      partial block DiscreteSISO "Single Input Single Output discrete control block"
        extends DiscreteBlock;
        Modelica.Blocks.Interfaces.RealInput u "Continuous input signal" annotation(
          Placement(transformation(extent = {{-140, -20}, {-100, 20}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealOutput y "Continuous output signal" annotation(
          Placement(transformation(extent = {{100, -10}, {120, 10}}, rotation = 0)));
        annotation(
          Documentation(info = "<html>
<p>
Block has one continuous input and one continuous output signal
which are sampled due to the defined <b>samplePeriod</b> parameter.
</p>
</html>"));
      end DiscreteSISO;

      partial block DiscreteMIMO "Multiple Input Multiple Output discrete control block"
        extends DiscreteBlock;
        parameter Integer nin = 1 "Number of inputs";
        parameter Integer nout = 1 "Number of outputs";
        Modelica.Blocks.Interfaces.RealInput u[nin] "Continuous input signals" annotation(
          Placement(transformation(extent = {{-140, -20}, {-100, 20}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealOutput y[nout] "Continuous output signals" annotation(
          Placement(transformation(extent = {{100, -10}, {120, 10}}, rotation = 0)));
        annotation(
          Documentation(info = "<html>
<p>
Block has a continuous input and a continuous output signal vector
which are sampled due to the defined <b>samplePeriod</b> parameter.
</p>
</html>"));
      end DiscreteMIMO;

      partial block DiscreteMIMOs "Multiple Input Multiple Output discrete control block"
        parameter Integer n = 1 "Number of inputs (= number of outputs)";
        extends DiscreteBlock;
        Modelica.Blocks.Interfaces.RealInput u[n] "Continuous input signals" annotation(
          Placement(transformation(extent = {{-140, -20}, {-100, 20}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealOutput y[n] "Continuous output signals" annotation(
          Placement(transformation(extent = {{100, -10}, {120, 10}}, rotation = 0)));
        annotation(
          Documentation(info = "<html>
<p>
Block has a continuous input and a continuous output signal vector
where the signal sizes of the input and output vector are identical.
These signals are sampled due to the defined <b>samplePeriod</b> parameter.
</p>
</html>"));
      end DiscreteMIMOs;

      partial block SVdiscrete "Discrete Single-Variable controller"
        extends DiscreteBlock;
        Discrete.Sampler sampler_s(final samplePeriod = samplePeriod, final startTime = startTime) annotation(
          Placement(transformation(extent = {{-100, -10}, {-80, 10}}, rotation = 0)));
        Discrete.Sampler sampler_m(final samplePeriod = samplePeriod, final startTime = startTime) annotation(
          Placement(transformation(origin = {0, -90}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
        Modelica.Blocks.Interfaces.RealInput u_s "Continuous scalar setpoint input signal" annotation(
          Placement(transformation(extent = {{-140, -20}, {-100, 20}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealInput u_m "Continuous scalar measurement input signal" annotation(
          Placement(transformation(origin = {0, -120}, extent = {{20, -20}, {-20, 20}}, rotation = 270)));
        Modelica.Blocks.Interfaces.RealOutput y "Continuous scalar actuator output signal" annotation(
          Placement(transformation(extent = {{100, -10}, {120, 10}}, rotation = 0)));
      equation
        connect(u_s, sampler_s.u) annotation(
          Line(points = {{-120, 0}, {-102, 0}}));
        connect(u_m, sampler_m.u) annotation(
          Line(points = {{0, -120}, {0, -111}, {0, -102}}));
        annotation(
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-100, 34}, {-140, 24}}, lineColor = {0, 0, 0}, textString = "(setpoint)"), Text(extent = {{100, 22}, {130, 14}}, lineColor = {0, 0, 0}, textString = "(actuator)"), Text(extent = {{-70, -112}, {-20, -102}}, lineColor = {0, 0, 0}, textString = " (measurement)")}),
          Documentation(info = "<html>
<p>
Block has two continuous Real input signals and one
continuous Real output signal
that are sampled due to the defined <b>samplePeriod</b> parameter.
The block is designed
to be used as base class for a corresponding controller.
</p>
</html>"));
      end SVdiscrete;

      partial block MVdiscrete "Discrete Multi-Variable controller"
        extends DiscreteBlock;
        parameter Integer nu_s = 1 "Number of setpoint inputs";
        parameter Integer nu_m = 1 "Number of measurement inputs";
        parameter Integer ny = 1 "Number of actuator outputs";
        Discrete.Sampler sampler_s[nu_s](each final samplePeriod = samplePeriod, each final startTime = startTime) annotation(
          Placement(transformation(extent = {{-90, -10}, {-70, 10}}, rotation = 0)));
        Discrete.Sampler sampler_m[nu_m](each final samplePeriod = samplePeriod, each final startTime = startTime) annotation(
          Placement(transformation(origin = {0, -80}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
        Modelica.Blocks.Interfaces.RealInput u_s[nu_s] "Continuous setpoint input signals" annotation(
          Placement(transformation(extent = {{-140, -20}, {-100, 20}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealInput u_m[nu_m] "Continuous measurement input signals" annotation(
          Placement(transformation(origin = {0, -120}, extent = {{20, -20}, {-20, 20}}, rotation = 270)));
        Modelica.Blocks.Interfaces.RealOutput y[ny] "Continuous actuator output signals" annotation(
          Placement(transformation(extent = {{100, -10}, {120, 10}}, rotation = 0)));
      equation
        connect(u_s, sampler_s.u) annotation(
          Line(points = {{-120, 0}, {-92, 0}}));
        connect(u_m, sampler_m.u) annotation(
          Line(points = {{0, -120}, {0, -106}, {0, -92}}));
        annotation(
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-100, -10}, {-80, -30}}, textString = "u_s", lineColor = {0, 0, 255}), Text(extent = {{-98, 34}, {-138, 24}}, lineColor = {0, 0, 0}, textString = "(setpoint)"), Text(extent = {{98, 24}, {138, 14}}, lineColor = {0, 0, 0}, textString = "(actuator)"), Text(extent = {{-62, -110}, {-12, -100}}, lineColor = {0, 0, 0}, textString = " (measurement)")}),
          Documentation(info = "<html>
<p>
Block has two continuous Real input signal vectors and one
continuous Real output signal vector. The vector signals
are sampled due to the defined <b>samplePeriod</b> parameter.
The block is designed
to be used as base class for a corresponding controller.
</p>
</html>"));
      end MVdiscrete;

      partial block BooleanSISO "Single Input Single Output control block with signals of type Boolean"
        extends Modelica.Blocks.Icons.BooleanBlock;
      public
        BooleanInput u "Connector of Boolean input signal" annotation(
          Placement(transformation(extent = {{-140, -20}, {-100, 20}}, rotation = 0)));
        BooleanOutput y "Connector of Boolean output signal" annotation(
          Placement(transformation(extent = {{100, -10}, {120, 10}}, rotation = 0)));
        annotation(
          Documentation(info = "<html>
<p>
Block has one continuous Boolean input and one continuous Boolean output signal.
</p>
</html>"));
      end BooleanSISO;

      partial block BooleanMIMOs "Multiple Input Multiple Output continuous control block with same number of inputs and outputs of Boolean type"
        extends Modelica.Blocks.Icons.BooleanBlock;
        parameter Integer n = 1 "Number of inputs (= number of outputs)";
        BooleanInput u[n] "Connector of Boolean input signals" annotation(
          Placement(transformation(extent = {{-140, -20}, {-100, 20}}, rotation = 0)));
        BooleanOutput y[n] "Connector of Boolean output signals" annotation(
          Placement(transformation(extent = {{100, -10}, {120, 10}}, rotation = 0)));
        annotation(
          Documentation(info = "<HTML>
<p>
Block has a continuous Boolean input and a continuous Boolean output signal vector
where the signal sizes of the input and output vector are identical.
</p>
</html>"));
      end BooleanMIMOs;

      partial block MI2BooleanMOs "2 Multiple Input / Boolean Multiple Output block with same signal lengths"
        extends Modelica.Blocks.Icons.BooleanBlock;
        parameter Integer n = 1 "Dimension of input and output vectors.";
        RealInput u1[n] "Connector 1 of Boolean input signals" annotation(
          Placement(transformation(extent = {{-140, 40}, {-100, 80}}, rotation = 0)));
        RealInput u2[n] "Connector 2 of Boolean input signals" annotation(
          Placement(transformation(extent = {{-140, -80}, {-100, -40}}, rotation = 0)));
        BooleanOutput y[n] "Connector of Boolean output signals" annotation(
          Placement(transformation(extent = {{100, -10}, {120, 10}}, rotation = 0)));
        annotation(
          Documentation(info = "<html>
<p>Block has two Boolean input vectors u1 and u2 and one Boolean output
vector y. All vectors have the same number of elements.</p>
</html>"));
      end MI2BooleanMOs;

      partial block SI2BooleanSO "2 Single Input / Boolean Single Output block"
        extends Modelica.Blocks.Icons.BooleanBlock;
        BooleanInput u1 "Connector 1 of Boolean input signals" annotation(
          Placement(transformation(extent = {{-140, 40}, {-100, 80}}, rotation = 0)));
        BooleanInput u2 "Connector 2 of Boolean input signals" annotation(
          Placement(transformation(extent = {{-140, -80}, {-100, -40}}, rotation = 0)));
        BooleanOutput y "Connector of Boolean output signals" annotation(
          Placement(transformation(extent = {{100, -10}, {120, 10}}, rotation = 0)));
        annotation(
          Documentation(info = "<html>
<p>
Block has two Boolean input signals u1 and u2 and one Boolean output signal y.
</p>
</html>"));
      end SI2BooleanSO;

      partial block BooleanSignalSource "Base class for Boolean signal sources"
        extends Modelica.Blocks.Icons.BooleanBlock;
        BooleanOutput y "Connector of Boolean output signal" annotation(
          Placement(transformation(extent = {{100, -10}, {120, 10}}, rotation = 0)));
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-80, 68}, {-80, -80}}, color = {192, 192, 192}), Polygon(points = {{-80, 90}, {-88, 68}, {-72, 68}, {-80, 90}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-90, -70}, {68, -70}}, color = {192, 192, 192}), Polygon(points = {{90, -70}, {68, -62}, {68, -78}, {90, -70}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid)}),
          Documentation(info = "<html>
<p>
Basic block for Boolean sources of package Blocks.Sources.
This component has one continuous Boolean output signal y.
</p>
</html>"));
      end BooleanSignalSource;

      partial block IntegerSO "Single Integer Output continuous control block"
        extends Modelica.Blocks.Icons.IntegerBlock;
        IntegerOutput y "Connector of Integer output signal" annotation(
          Placement(transformation(extent = {{100, -10}, {120, 10}}, rotation = 0)));
        annotation(
          Documentation(info = "<html>
<p>
Block has one continuous Integer output signal.
</p>
</html>"));
      end IntegerSO;

      partial block IntegerMO "Multiple Integer Output continuous control block"
        extends Modelica.Blocks.Icons.IntegerBlock;
        parameter Integer nout(min = 1) = 1 "Number of outputs";
        IntegerOutput y[nout] "Connector of Integer output signals" annotation(
          Placement(transformation(extent = {{100, -10}, {120, 10}}, rotation = 0)));
        annotation(
          Documentation(info = "<html>
<p>
Block has one continuous Integer output signal vector.
</p>
</html>"));
      end IntegerMO;

      partial block IntegerSignalSource "Base class for continuous Integer signal source"
        extends IntegerSO;
        parameter Integer offset = 0 "Offset of output signal y";
        parameter SI.Time startTime = 0 "Output y = offset for time < startTime";
        annotation(
          Documentation(info = "<html>
<p>
Basic block for Integer sources of package Blocks.Sources.
This component has one continuous Integer output signal y
and two parameters (offset, startTime) to shift the
generated signal.
</p>
</html>"));
      end IntegerSignalSource;

      partial block IntegerSIBooleanSO "Integer Input Boolean Output continuous control block"
        extends Modelica.Blocks.Icons.BooleanBlock;
        IntegerInput u "Connector of Integer input signal" annotation(
          Placement(transformation(extent = {{-140, -20}, {-100, 20}}, rotation = 0)));
        BooleanOutput y "Connector of Boolean output signal" annotation(
          Placement(transformation(extent = {{100, -10}, {120, 10}}, rotation = 0)));
        annotation(
          Documentation(info = "<HTML>
<p>
Block has a continuous Integer input and a continuous Boolean output signal.
</p>
</html>"));
      end IntegerSIBooleanSO;

      partial block IntegerMIBooleanMOs "Multiple Integer Input Multiple Boolean Output continuous control block with same number of inputs and outputs"
        extends Modelica.Blocks.Icons.BooleanBlock;
        parameter Integer n = 1 "Number of inputs (= number of outputs)";
        IntegerInput u[n] "Connector of Integer input signals" annotation(
          Placement(transformation(extent = {{-140, -20}, {-100, 20}}, rotation = 0)));
        BooleanOutput y[n] "Connector of Boolean output signals" annotation(
          Placement(transformation(extent = {{100, -10}, {120, 10}}, rotation = 0)));
        annotation(
          Documentation(info = "<HTML>
<p>
Block has a continuous Integer input and a continuous Boolean output signal vector
where the signal sizes of the input and output vector are identical.
</p>
</html>"));
      end IntegerMIBooleanMOs;

      partial block PartialIntegerSISO "Partial block with a IntegerInput and an IntegerOutput signal"
        Modelica.Blocks.Interfaces.IntegerInput u "Integer input signal" annotation(
          Placement(transformation(extent = {{-180, -40}, {-100, 40}})));
        Modelica.Blocks.Interfaces.IntegerOutput y "Integer output signal" annotation(
          Placement(transformation(extent = {{100, -20}, {140, 20}})));
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}, initialScale = 0.06), graphics = {Text(extent = {{110, -50}, {250, -70}}, lineColor = {0, 0, 0}, textString = DynamicSelect(" ", String(y, minimumLength = 1, significantDigits = 0))), Text(extent = {{-250, 170}, {250, 110}}, textString = "%name", lineColor = {0, 0, 255}), Rectangle(extent = {{-100, 100}, {100, -100}}, lineColor = {0, 0, 0}, lineThickness = 5.0, fillColor = {255, 213, 170}, fillPattern = FillPattern.Solid, borderPattern = BorderPattern.Raised)}));
      end PartialIntegerSISO;

      partial block PartialIntegerMISO "Partial block with an IntegerVectorInput and an IntegerOutput signal"
        parameter Integer nu(min = 0) = 0 "Number of input connections" annotation(
          Dialog(connectorSizing = true),
          HideResult = true);
        Modelica.Blocks.Interfaces.IntegerVectorInput u[nu] "Vector of Integer input signals" annotation(
          Placement(transformation(extent = {{-120, 70}, {-80, -70}})));
        Modelica.Blocks.Interfaces.IntegerOutput y "Integer output signal" annotation(
          Placement(transformation(extent = {{100, -15}, {130, 15}})));
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}, initialScale = 0.06), graphics = {Text(extent = {{110, -50}, {250, -70}}, lineColor = {0, 0, 0}, textString = DynamicSelect(" ", String(y, minimumLength = 1, significantDigits = 0))), Text(extent = {{-250, 170}, {250, 110}}, textString = "%name", lineColor = {0, 0, 255}), Rectangle(extent = {{-100, 100}, {100, -100}}, lineColor = {255, 137, 0}, lineThickness = 5.0, fillColor = {255, 213, 170}, borderPattern = BorderPattern.Raised, fillPattern = FillPattern.Solid)}));
      end PartialIntegerMISO;

      partial block partialBooleanSISO "Partial block with 1 input and 1 output Boolean signal"
        extends Modelica.Blocks.Icons.PartialBooleanBlock;
        Blocks.Interfaces.BooleanInput u "Connector of Boolean input signal" annotation(
          Placement(transformation(extent = {{-140, -20}, {-100, 20}}, rotation = 0)));
        Blocks.Interfaces.BooleanOutput y "Connector of Boolean output signal" annotation(
          Placement(transformation(extent = {{100, -10}, {120, 10}}, rotation = 0)));
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-71, 7}, {-85, -7}}, lineColor = DynamicSelect({235, 235, 235}, if u > 0.5 then {0, 255, 0} else {235, 235, 235}), fillColor = DynamicSelect({235, 235, 235}, if u > 0.5 then {0, 255, 0} else {235, 235, 235}), fillPattern = FillPattern.Solid), Ellipse(extent = {{71, 7}, {85, -7}}, lineColor = DynamicSelect({235, 235, 235}, if y > 0.5 then {0, 255, 0} else {235, 235, 235}), fillColor = DynamicSelect({235, 235, 235}, if y > 0.5 then {0, 255, 0} else {235, 235, 235}), fillPattern = FillPattern.Solid)}),
          Documentation(info = "<html>
<p>
Block has one continuous Boolean input and one continuous Boolean output signal
with a 3D icon (e.g., used in Blocks.Logical library).
</p>
</html>"));
      end partialBooleanSISO;

      partial block partialBooleanSI2SO "Partial block with 2 input and 1 output Boolean signal"
        extends Modelica.Blocks.Icons.PartialBooleanBlock;
        Blocks.Interfaces.BooleanInput u1 "Connector of first Boolean input signal" annotation(
          Placement(transformation(extent = {{-140, -20}, {-100, 20}}, rotation = 0)));
        Blocks.Interfaces.BooleanInput u2 "Connector of second Boolean input signal" annotation(
          Placement(transformation(extent = {{-140, -100}, {-100, -60}}, rotation = 0)));
        Blocks.Interfaces.BooleanOutput y "Connector of Boolean output signal" annotation(
          Placement(transformation(extent = {{100, -10}, {120, 10}}, rotation = 0)));
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-71, 7}, {-85, -7}}, lineColor = DynamicSelect({235, 235, 235}, if u1 > 0.5 then {0, 255, 0} else {235, 235, 235}), fillColor = DynamicSelect({235, 235, 235}, if u1 > 0.5 then {0, 255, 0} else {235, 235, 235}), fillPattern = FillPattern.Solid), Ellipse(extent = {{-71, -74}, {-85, -88}}, lineColor = DynamicSelect({235, 235, 235}, if u2 > 0.5 then {0, 255, 0} else {235, 235, 235}), fillColor = DynamicSelect({235, 235, 235}, if u2 > 0.5 then {0, 255, 0} else {235, 235, 235}), fillPattern = FillPattern.Solid), Ellipse(extent = {{71, 7}, {85, -7}}, lineColor = DynamicSelect({235, 235, 235}, if y > 0.5 then {0, 255, 0} else {235, 235, 235}), fillColor = DynamicSelect({235, 235, 235}, if y > 0.5 then {0, 255, 0} else {235, 235, 235}), fillPattern = FillPattern.Solid)}),
          Documentation(info = "<html>
<p>
Block has two continuous Boolean input and one continuous Boolean output signal
with a 3D icon (e.g., used in Blocks.Logical library).
</p>
</html>"));
      end partialBooleanSI2SO;

      partial block partialBooleanSI3SO "Partial block with 3 input and 1 output Boolean signal"
        extends Modelica.Blocks.Icons.PartialBooleanBlock;
        Blocks.Interfaces.BooleanInput u1 "Connector of first Boolean input signal" annotation(
          Placement(transformation(extent = {{-140, 60}, {-100, 100}}, rotation = 0)));
        Blocks.Interfaces.BooleanInput u2 "Connector of second Boolean input signal" annotation(
          Placement(transformation(extent = {{-140, -20}, {-100, 20}}, rotation = 0)));
        Blocks.Interfaces.BooleanInput u3 "Connector of third Boolean input signal" annotation(
          Placement(transformation(extent = {{-140, -100}, {-100, -60}}, rotation = 0)));
        Blocks.Interfaces.BooleanOutput y "Connector of Boolean output signal" annotation(
          Placement(transformation(extent = {{100, -10}, {120, 10}}, rotation = 0)));
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-71, 74}, {-85, 88}}, lineColor = DynamicSelect({235, 235, 235}, if u1 > 0.5 then {0, 255, 0} else {235, 235, 235}), fillColor = DynamicSelect({235, 235, 235}, if u1 > 0.5 then {0, 255, 0} else {235, 235, 235}), fillPattern = FillPattern.Solid), Ellipse(extent = {{-71, 7}, {-85, -7}}, lineColor = DynamicSelect({235, 235, 235}, if u2 > 0.5 then {0, 255, 0} else {235, 235, 235}), fillColor = DynamicSelect({235, 235, 235}, if u2 > 0.5 then {0, 255, 0} else {235, 235, 235}), fillPattern = FillPattern.Solid), Ellipse(extent = {{-71, -74}, {-85, -88}}, lineColor = DynamicSelect({235, 235, 235}, if u3 > 0.5 then {0, 255, 0} else {235, 235, 235}), fillColor = DynamicSelect({235, 235, 235}, if u3 > 0.5 then {0, 255, 0} else {235, 235, 235}), fillPattern = FillPattern.Solid), Ellipse(extent = {{71, 7}, {85, -7}}, lineColor = DynamicSelect({235, 235, 235}, if y > 0.5 then {0, 255, 0} else {235, 235, 235}), fillColor = DynamicSelect({235, 235, 235}, if y > 0.5 then {0, 255, 0} else {235, 235, 235}), fillPattern = FillPattern.Solid)}),
          Documentation(info = "<html><p>
Block has three continuous Boolean input and one continuous Boolean output signal
with a 3D icon (e.g., used in Blocks.Logical library).
</p>
</html>"));
      end partialBooleanSI3SO;

      partial block partialBooleanSI "Partial block with 1 input Boolean signal"
        extends Modelica.Blocks.Icons.PartialBooleanBlock;
        Blocks.Interfaces.BooleanInput u "Connector of Boolean input signal" annotation(
          Placement(transformation(extent = {{-140, -20}, {-100, 20}}, rotation = 0)));
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-71, 7}, {-85, -7}}, lineColor = DynamicSelect({235, 235, 235}, if u > 0.5 then {0, 255, 0} else {235, 235, 235}), fillColor = DynamicSelect({235, 235, 235}, if u > 0.5 then {0, 255, 0} else {235, 235, 235}), fillPattern = FillPattern.Solid)}),
          Documentation(info = "<html>
<p>
Block has one continuous Boolean input signal
with a 3D icon (e.g., used in Blocks.Logical library).
</p>
</html>"));
      end partialBooleanSI;

      partial block partialBooleanSO "Partial block with 1 output Boolean signal"
        Blocks.Interfaces.BooleanOutput y "Connector of Boolean output signal" annotation(
          Placement(transformation(extent = {{100, -10}, {120, 10}}, rotation = 0)));
        extends Modelica.Blocks.Icons.PartialBooleanBlock;
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{71, 7}, {85, -7}}, lineColor = DynamicSelect({235, 235, 235}, if y > 0.5 then {0, 255, 0} else {235, 235, 235}), fillColor = DynamicSelect({235, 235, 235}, if y > 0.5 then {0, 255, 0} else {235, 235, 235}), fillPattern = FillPattern.Solid)}),
          Documentation(info = "<html>
<p>
Block has one continuous Boolean output signal
with a 3D icon (e.g., used in Blocks.Logical library).
</p>
</html>"));
      end partialBooleanSO;

      partial block partialBooleanSource "Partial source block (has 1 output Boolean signal and an appropriate default icon)"
        extends Modelica.Blocks.Icons.PartialBooleanBlock;
        Blocks.Interfaces.BooleanOutput y "Connector of Boolean output signal" annotation(
          Placement(transformation(extent = {{100, -10}, {120, 10}}, rotation = 0)));
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{-80, 88}, {-88, 66}, {-72, 66}, {-80, 88}}, lineColor = {255, 0, 255}, fillColor = {255, 0, 255}, fillPattern = FillPattern.Solid), Line(points = {{-80, 66}, {-80, -82}}, color = {255, 0, 255}), Line(points = {{-90, -70}, {72, -70}}, color = {255, 0, 255}), Polygon(points = {{90, -70}, {68, -62}, {68, -78}, {90, -70}}, lineColor = {255, 0, 255}, fillColor = {255, 0, 255}, fillPattern = FillPattern.Solid), Ellipse(extent = {{71, 7}, {85, -7}}, lineColor = DynamicSelect({235, 235, 235}, if y > 0.5 then {0, 255, 0} else {235, 235, 235}), fillColor = DynamicSelect({235, 235, 235}, if y > 0.5 then {0, 255, 0} else {235, 235, 235}), fillPattern = FillPattern.Solid)}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{-70, 92}, {-76, 70}, {-64, 70}, {-70, 92}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Line(points = {{-70, 70}, {-70, -88}}, color = {95, 95, 95}), Line(points = {{-90, -70}, {68, -70}}, color = {95, 95, 95}), Polygon(points = {{90, -70}, {68, -64}, {68, -76}, {90, -70}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Text(extent = {{54, -80}, {106, -92}}, lineColor = {0, 0, 0}, textString = "time"), Text(extent = {{-64, 92}, {-46, 74}}, lineColor = {0, 0, 0}, textString = "y")}),
          Documentation(info = "<html>
<p>
Basic block for Boolean sources of package Blocks.Sources.
This component has one continuous Boolean output signal y
and a 3D icon (e.g., used in Blocks.Logical library).
</p>
</html>"));
      end partialBooleanSource;

      partial block partialBooleanThresholdComparison "Partial block to compare the Real input u with a threshold and provide the result as 1 Boolean output signal"
        parameter Real threshold = 0 "Comparison with respect to threshold";
        Blocks.Interfaces.RealInput u "Connector of Boolean input signal" annotation(
          Placement(transformation(extent = {{-140, -20}, {-100, 20}}, rotation = 0)));
        Blocks.Interfaces.BooleanOutput y "Connector of Boolean output signal" annotation(
          Placement(transformation(extent = {{100, -10}, {120, 10}}, rotation = 0)));
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, 100}, {100, -100}}, lineColor = {0, 0, 0}, lineThickness = 5.0, fillColor = {210, 210, 210}, fillPattern = FillPattern.Solid, borderPattern = BorderPattern.Raised), Text(extent = {{-150, -140}, {150, -110}}, lineColor = {0, 0, 0}, textString = "%threshold"), Ellipse(extent = {{71, 7}, {85, -7}}, lineColor = DynamicSelect({235, 235, 235}, if y > 0.5 then {0, 255, 0} else {235, 235, 235}), fillColor = DynamicSelect({235, 235, 235}, if y > 0.5 then {0, 255, 0} else {235, 235, 235}), fillPattern = FillPattern.Solid)}),
          Documentation(info = "<html>
<p>
Block has one continuous Real input and one continuous Boolean output signal
as well as a 3D icon (e.g., used in Blocks.Logical library).
</p>
</html>"));
      end partialBooleanThresholdComparison;

      partial block partialBooleanComparison "Partial block with 2 Real input and 1 Boolean output signal (the result of a comparison of the two Real inputs)"
        Blocks.Interfaces.RealInput u1 "Connector of first Boolean input signal" annotation(
          Placement(transformation(extent = {{-140, -20}, {-100, 20}}, rotation = 0)));
        Blocks.Interfaces.RealInput u2 "Connector of second Boolean input signal" annotation(
          Placement(transformation(extent = {{-140, -100}, {-100, -60}}, rotation = 0)));
        Blocks.Interfaces.BooleanOutput y "Connector of Boolean output signal" annotation(
          Placement(transformation(extent = {{100, -10}, {120, 10}}, rotation = 0)));
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, 100}, {100, -100}}, lineColor = {0, 0, 0}, lineThickness = 5.0, fillColor = {210, 210, 210}, fillPattern = FillPattern.Solid, borderPattern = BorderPattern.Raised), Ellipse(extent = {{73, 7}, {87, -7}}, lineColor = DynamicSelect({235, 235, 235}, if y > 0.5 then {0, 255, 0} else {235, 235, 235}), fillColor = DynamicSelect({235, 235, 235}, if y > 0.5 then {0, 255, 0} else {235, 235, 235}), fillPattern = FillPattern.Solid), Ellipse(extent = {{32, 10}, {52, -10}}, lineColor = {0, 0, 127}), Line(points = {{-100, -80}, {42, -80}, {42, 0}}, color = {0, 0, 127})}),
          Documentation(info = "<html>
<p>
Block has two continuous Real input and one continuous Boolean output signal
as a result of the comparison of the two input signals. The block
has a 3D icon (e.g., used in Blocks.Logical library).
</p>
</html>"));
      end partialBooleanComparison;

      partial block PartialBooleanSISO_small "Partial block with a BooleanInput and a BooleanOutput signal and a small block icon"
        Modelica.Blocks.Interfaces.BooleanInput u "Boolean input signal" annotation(
          Placement(transformation(extent = {{-180, -40}, {-100, 40}})));
        Modelica.Blocks.Interfaces.BooleanOutput y "Boolean output signal" annotation(
          Placement(transformation(extent = {{100, -20}, {140, 20}})));
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}, initialScale = 0.04), graphics = {Text(extent = {{-250, 170}, {250, 110}}, textString = "%name", lineColor = {0, 0, 255}), Rectangle(extent = {{-100, 100}, {100, -100}}, lineColor = {0, 0, 0}, lineThickness = 5.0, fillColor = {215, 215, 215}, fillPattern = FillPattern.Solid, borderPattern = BorderPattern.Raised), Ellipse(extent = {{60, 10}, {80, -10}}, lineColor = DynamicSelect({235, 235, 235}, if y > 0.5 then {0, 255, 0} else {235, 235, 235}), fillColor = DynamicSelect({235, 235, 235}, if y > 0.5 then {0, 255, 0} else {235, 235, 235}), fillPattern = FillPattern.Solid)}));
      end PartialBooleanSISO_small;

      partial block PartialBooleanMISO "Partial block with a BooleanVectorInput and a BooleanOutput signal"
        parameter Integer nu(min = 0) = 0 "Number of input connections" annotation(
          Dialog(connectorSizing = true),
          HideResult = true);
        Modelica.Blocks.Interfaces.BooleanVectorInput u[nu] "Vector of Boolean input signals" annotation(
          Placement(transformation(extent = {{-120, 70}, {-80, -70}})));
        Modelica.Blocks.Interfaces.BooleanOutput y "Boolean output signal" annotation(
          Placement(transformation(extent = {{100, -15}, {130, 15}})));
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}, initialScale = 0.06), graphics = {Text(extent = {{-250, 170}, {250, 110}}, textString = "%name", lineColor = {0, 0, 255}), Rectangle(extent = {{-100, 100}, {100, -100}}, lineColor = {0, 0, 0}, lineThickness = 5.0, fillColor = {215, 215, 215}, fillPattern = FillPattern.Solid, borderPattern = BorderPattern.Raised), Ellipse(extent = {{60, 10}, {80, -10}}, lineColor = DynamicSelect({235, 235, 235}, if y > 0.5 then {0, 255, 0} else {235, 235, 235}), fillColor = DynamicSelect({235, 235, 235}, if y > 0.5 then {0, 255, 0} else {235, 235, 235}), fillPattern = FillPattern.Solid)}));
      end PartialBooleanMISO;

      package Adaptors "Obsolete package with components to send signals to a bus or receive signals from a bus (only for backward compatibility)"
        extends Modelica.Icons.Package;
        // extends Modelica.Icons.ObsoleteModel;

        block SendReal "Obsolete block to send Real signal to bus"
          // extends Modelica.Icons.ObsoleteModel;
          RealOutput toBus "Output signal to be connected to bus" annotation(
            Placement(transformation(extent = {{100, -10}, {120, 10}}, rotation = 0)));
          RealInput u "Input signal to be send to bus" annotation(
            Placement(transformation(extent = {{-140, -20}, {-100, 20}}, rotation = 0)));
        equation
          toBus = u;
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, 40}, {100, -40}}, lineColor = {0, 0, 127}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-144, 96}, {144, 46}}, lineColor = {0, 0, 0}, textString = "%name"), Text(extent = {{-100, 30}, {100, -30}}, lineColor = {0, 0, 127}, textString = "send")}),
            Documentation(info = "<html>
<p>
Obsolete block that was previously used to connect a Real signal
to a signal in a connector. This block is only provided for
backward compatibility.
</p>

<p>
It is much more convenient and more powerful to use \"expandable connectors\"
for signal buses, see example
<a href=\"modelica://Modelica.Blocks.Examples.BusUsage\">BusUsage</a>.
</p>
</html>"));
        end SendReal;

        block SendBoolean "Obsolete block to send Boolean signal to bus"
          // extends Modelica.Icons.ObsoleteModel;
          BooleanOutput toBus "Output signal to be connected to bus" annotation(
            Placement(transformation(extent = {{100, -10}, {120, 10}}, rotation = 0)));
          BooleanInput u "Input signal to be send to bus" annotation(
            Placement(transformation(extent = {{-140, -20}, {-100, 20}}, rotation = 0)));
        equation
          toBus = u;
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, 40}, {100, -40}}, lineColor = {255, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-144, 96}, {144, 46}}, lineColor = {0, 0, 0}, textString = "%name"), Text(extent = {{-100, 30}, {100, -30}}, lineColor = {255, 0, 255}, textString = "send")}),
            Documentation(info = "<html>
<p>
Obsolete block that was previously used to connect a Boolean signal
to a signal in a connector. This block is only provided for
backward compatibility.
</p>

<p>
It is much more convenient and more powerful to use \"expandable connectors\"
for signal buses, see example
<a href=\"modelica://Modelica.Blocks.Examples.BusUsage\">BusUsage</a>.
</p>
</html>"));
        end SendBoolean;

        block SendInteger "Obsolete block to send Integer signal to bus"
          // extends Modelica.Icons.ObsoleteModel;
          IntegerOutput toBus "Output signal to be connected to bus" annotation(
            Placement(transformation(extent = {{100, -10}, {120, 10}}, rotation = 0)));
          IntegerInput u "Input signal to be send to bus" annotation(
            Placement(transformation(extent = {{-140, -20}, {-100, 20}}, rotation = 0)));
        equation
          toBus = u;
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, 40}, {100, -40}}, lineColor = {255, 127, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-144, 96}, {144, 46}}, lineColor = {0, 0, 0}, textString = "%name"), Text(extent = {{-100, 30}, {100, -30}}, lineColor = {255, 127, 0}, textString = "send")}),
            Documentation(info = "<html>
<p>
Obsolete block that was previously used to connect an Integer signal
to a signal in a connector. This block is only provided for
backward compatibility.
</p>

<p>
It is much more convenient and more powerful to use \"expandable connectors\"
for signal buses, see example
<a href=\"modelica://Modelica.Blocks.Examples.BusUsage\">BusUsage</a>.
</p>
</html>"));
        end SendInteger;

        block ReceiveReal "Obsolete block to receive Real signal from bus"
          // extends Modelica.Icons.ObsoleteModel;
          RealInput fromBus "To be connected with signal on bus" annotation(
            Placement(transformation(extent = {{-120, -10}, {-100, 10}}, rotation = 0)));
          RealOutput y "Output signal to be received from bus" annotation(
            Placement(transformation(extent = {{100, -10}, {120, 10}}, rotation = 0)));
        equation
          y = fromBus;
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, 40}, {100, -40}}, lineColor = {0, 0, 127}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-100, 30}, {100, -30}}, lineColor = {0, 0, 127}, textString = "receive"), Text(extent = {{-144, 96}, {144, 46}}, lineColor = {0, 0, 0}, textString = "%name")}),
            Documentation(info = "<html>
<p>
Obsolete block that was previously used to connect a Real signal
in a connector to an input of a block. This block is only provided for
backward compatibility.
</p>

<p>
It is much more convenient and more powerful to use \"expandable connectors\"
for signal buses, see example
<a href=\"modelica://Modelica.Blocks.Examples.BusUsage\">BusUsage</a>.
</p>
</html>"));
        end ReceiveReal;

        block ReceiveBoolean "Obsolete block to receive Boolean signal from bus"
          // extends Modelica.Icons.ObsoleteModel;
          BooleanInput fromBus "To be connected with signal on bus" annotation(
            Placement(transformation(extent = {{-120, -10}, {-100, 10}}, rotation = 0)));
          BooleanOutput y "Output signal to be received from bus" annotation(
            Placement(transformation(extent = {{100, -10}, {120, 10}}, rotation = 0)));
        equation
          y = fromBus;
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, 40}, {100, -40}}, lineColor = {255, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-100, 30}, {100, -30}}, lineColor = {255, 0, 255}, textString = "receive"), Text(extent = {{-144, 96}, {144, 46}}, lineColor = {0, 0, 0}, textString = "%name")}),
            Documentation(info = "<html>
<p>
Obsolete block that was previously used to connect a Boolean signal
in a connector to an input of a block. This block is only provided for
backward compatibility.
</p>

<p>
It is much more convenient and more powerful to use \"expandable connectors\"
for signal buses, see example
<a href=\"modelica://Modelica.Blocks.Examples.BusUsage\">BusUsage</a>.
</p>
</html>"));
        end ReceiveBoolean;

        block ReceiveInteger "Obsolete block to receive Integer signal from bus"
          // extends Modelica.Icons.ObsoleteModel;
          IntegerInput fromBus "To be connected with signal on bus" annotation(
            Placement(transformation(extent = {{-120, -10}, {-100, 10}}, rotation = 0)));
          IntegerOutput y "Output signal to be received from bus" annotation(
            Placement(transformation(extent = {{100, -10}, {120, 10}}, rotation = 0)));
        equation
          y = fromBus;
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, 40}, {100, -40}}, lineColor = {255, 127, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-100, 30}, {100, -30}}, lineColor = {255, 127, 0}, textString = "receive"), Text(extent = {{-144, 96}, {144, 46}}, lineColor = {0, 0, 0}, textString = "%name")}),
            Documentation(info = "<html>
<p>
Obsolete block that was previously used to connect an Integer signal
in a connector to an input of a block. This block is only provided for
backward compatibility.
</p>

<p>
It is much more convenient and more powerful to use \"expandable connectors\"
for signal buses, see example
<a href=\"modelica://Modelica.Blocks.Examples.BusUsage\">BusUsage</a>.
</p>
</html>"));
        end ReceiveInteger;
        annotation(
          Documentation(info = "<html>
<p>
The components of this package should no longer be used.
They are only provided for backward compatibility.
It is much more convenient and more powerful to use \"expandable connectors\"
for signal buses, see example
<a href=\"modelica://Modelica.Blocks.Examples.BusUsage\">BusUsage</a>.
</p>
</html>"));
      end Adaptors;

      partial block PartialConversionBlock "Partial block defining the interface for conversion blocks"
        RealInput u "Connector of Real input signal to be converted" annotation(
          Placement(transformation(extent = {{-140, -20}, {-100, 20}}, rotation = 0)));
        RealOutput y "Connector of Real output signal containing input signal u in another unit" annotation(
          Placement(transformation(extent = {{100, -10}, {120, 10}}, rotation = 0)));
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100.0, -100.0}, {100.0, 100.0}}, initialScale = 0.1), graphics = {Rectangle(lineColor = {0, 0, 127}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, extent = {{-100.0, -100.0}, {100.0, 100.0}}), Line(points = {{-90.0, 0.0}, {30.0, 0.0}}, color = {191, 0, 0}), Polygon(lineColor = {191, 0, 0}, fillColor = {191, 0, 0}, fillPattern = FillPattern.Solid, points = {{90.0, 0.0}, {30.0, 20.0}, {30.0, -20.0}, {90.0, 0.0}}), Text(lineColor = {0, 0, 255}, extent = {{-115.0, 105.0}, {115.0, 155.0}}, textString = "%name")}),
          Documentation(info = "<html>
<p>
This block defines the interface of a conversion block that
converts from one unit into another one.
</p>

</html>"));
      end PartialConversionBlock;

      partial block BlockIcon "This icon will be removed in future Modelica versions, use Modelica.Blocks.Icons.Block instead."
        // extends Modelica.Icons.ObsoleteModel;
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, -100}, {100, 100}}, lineColor = {0, 0, 127}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-150, 150}, {150, 110}}, textString = "%name", lineColor = {0, 0, 255})}),
          Documentation(info = "<html>
<p>
This icon will be removed in future versions of the Modelica Standard Library.
Instead the icon <a href=\"modelica://Modelica.Blocks.Icons.Block\">Modelica.Blocks.Icons.Block</a> shall be used.
</p>
</html>"));
      end BlockIcon;

      partial block BooleanBlockIcon "This icon will be removed in future Modelica versions, use Modelica.Blocks.Icons.BooleanBlock instead."
        // extends Modelica.Icons.ObsoleteModel;
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, -100}, {100, 100}}, lineColor = {255, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-150, 150}, {150, 110}}, textString = "%name", lineColor = {0, 0, 255})}),
          Documentation(info = "<html>
<p>
This icon will be removed in future versions of the Modelica Standard Library.
Instead the icon <a href=\"modelica://Modelica.Blocks.Icons.BooleanBlock\">Modelica.Blocks.Icons.BooleanBlock</a> shall be used.
</p>
</html>"));
      end BooleanBlockIcon;

      partial block DiscreteBlockIcon "This icon will be removed in future Modelica versions, use Modelica.Blocks.Icons.DiscreteBlock instead."
        // extends Modelica.Icons.ObsoleteModel;
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, -100}, {100, 100}}, lineColor = {0, 0, 127}, lineThickness = 5.0, fillColor = {223, 211, 169}, fillPattern = FillPattern.Solid, borderPattern = BorderPattern.Raised), Text(extent = {{-150, 150}, {150, 110}}, textString = "%name", lineColor = {0, 0, 255})}),
          Documentation(info = "<html>
<p>
This icon will be removed in future versions of the Modelica Standard Library.
Instead the icon <a href=\"modelica://Modelica.Blocks.Icons.DiscreteBlock\">Modelica.Blocks.Icons.DiscreteBlock</a> shall be used.
</p>
</html>"));
      end DiscreteBlockIcon;

      partial block IntegerBlockIcon "This icon will be removed in future Modelica versions, use Modelica.Blocks.Icons.IntegerBlock instead."
        // extends Modelica.Icons.ObsoleteModel;
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, -100}, {100, 100}}, lineColor = {255, 127, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-150, 150}, {150, 110}}, textString = "%name", lineColor = {0, 0, 255})}),
          Documentation(info = "<html>
<p>
This icon will be removed in future versions of the Modelica Standard Library.
Instead the icon <a href=\"modelica://Modelica.Blocks.Icons.IntegerBlock\">Modelica.Blocks.Icons.IntegerBlock</a> shall be used.
</p>
</html>"));
      end IntegerBlockIcon;

      partial block partialBooleanBlockIcon "This icon will be removed in future Modelica versions, use Modelica.Blocks.Icons.PartialBooleanBlock instead."
        // extends Modelica.Icons.ObsoleteModel;
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, 100}, {100, -100}}, lineColor = {0, 0, 0}, lineThickness = 5.0, fillColor = {210, 210, 210}, fillPattern = FillPattern.Solid, borderPattern = BorderPattern.Raised), Text(extent = {{-150, 150}, {150, 110}}, textString = "%name", lineColor = {0, 0, 255})}),
          Documentation(info = "<html>
<p>
This icon will be removed in future versions of the Modelica Standard Library.
Instead the icon <a href=\"modelica://Modelica.Blocks.Icons.PartialBooleanBlock\">Modelica.Blocks.Icons.PartialBooleanBlock</a> shall be used.
</p>
</html>"));
      end partialBooleanBlockIcon;
      annotation(
        Documentation(info = "<HTML>
<p>
This package contains interface definitions for
<b>continuous</b> input/output blocks with Real,
Integer and Boolean signals. Furthermore, it contains
partial models for continuous and discrete blocks.
</p>

</html>", revisions = "<html>
<ul>
<li><i>Oct. 21, 2002</i>
       by <a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a>
       and <a href=\"http://www.robotic.dlr.de/Christian.Schweiger/\">Christian Schweiger</a>:<br>
       Added several new interfaces.
<li><i>Oct. 24, 1999</i>
       by <a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a>:<br>
       RealInputSignal renamed to RealInput. RealOutputSignal renamed to
       output RealOutput. GraphBlock renamed to BlockIcon. SISOreal renamed to
       SISO. SOreal renamed to SO. I2SOreal renamed to M2SO.
       SignalGenerator renamed to SignalSource. Introduced the following
       new models: MIMO, MIMOs, SVcontrol, MVcontrol, DiscreteBlockIcon,
       DiscreteBlock, DiscreteSISO, DiscreteMIMO, DiscreteMIMOs,
       BooleanBlockIcon, BooleanSISO, BooleanSignalSource, MI2BooleanMOs.</li>
<li><i>June 30, 1999</i>
       by <a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a>:<br>
       Realized a first version, based on an existing Dymola library
       of Dieter Moormann and Hilding Elmqvist.</li>
</ul>
</html>"));
    end Interfaces;

    package Logical "Library of components with Boolean input and output signals"
      extends Modelica.Icons.Package;

      block And "Logical 'and': y = u1 and u2"
        extends Blocks.Interfaces.partialBooleanSI2SO;
      equation
        y = u1 and u2;
        annotation(
          defaultComponentName = "and1",
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-90, 40}, {90, -40}}, lineColor = {0, 0, 0}, textString = "and")}),
          Documentation(info = "<html>
<p>
The output is <b>true</b> if all inputs are <b>true</b>, otherwise
the output is <b>false</b>.
</p>
</html>"));
      end And;

      block Or "Logical 'or': y = u1 or u2"
        extends Blocks.Interfaces.partialBooleanSI2SO;
      equation
        y = u1 or u2;
        annotation(
          defaultComponentName = "or1",
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-90, 40}, {90, -40}}, lineColor = {0, 0, 0}, textString = "or")}),
          Documentation(info = "<html>
<p>
The output is <b>true</b> if at least one input is <b>true</b>, otherwise
the output is <b>false</b>.
</p>
</html>"));
      end Or;

      block Xor "Logical 'xor': y = u1 xor u2"
        extends Blocks.Interfaces.partialBooleanSI2SO;
      equation
        y = not (u1 and u2 or not u1 and not u2);
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-90, 40}, {90, -40}}, lineColor = {0, 0, 0}, textString = "xor")}),
          Documentation(info = "<html>
<p>
The output is <b>true</b> if exactly one input is <b>true</b>, otherwise
the output is <b>false</b>.
</p>
</html>"));
      end Xor;

      block Nor "Logical 'nor': y = not (u1 or u2)"
        extends Blocks.Interfaces.partialBooleanSI2SO;
      equation
        y = not (u1 or u2);
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-90, 40}, {90, -40}}, lineColor = {0, 0, 0}, textString = "nor")}),
          Documentation(info = "<html>
<p>
The output is <b>true</b> if none of the inputs is <b>true</b>, otherwise
the output is <b>false</b>.
</p>
</html>"));
      end Nor;

      block Nand "Logical 'nand': y = not (u1 and u2)"
        extends Blocks.Interfaces.partialBooleanSI2SO;
      equation
        y = not (u1 and u2);
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-90, 40}, {90, -40}}, lineColor = {0, 0, 0}, textString = "nand")}),
          Documentation(info = "<html>
<p>
The output is <b>true</b> if at least one input is <b>false</b>, otherwise
the output is <b>false</b>.
</p>
</html>"));
      end Nand;

      block Not "Logical 'not': y = not u"
        extends Blocks.Interfaces.partialBooleanSISO;
      equation
        y = not u;
        annotation(
          defaultComponentName = "not1",
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-90, 40}, {90, -40}}, lineColor = {0, 0, 0}, textString = "not")}),
          Documentation(info = "<html>
<p>
The output is <b>true</b> if the input is <b>false</b>, otherwise
the output is <b>false</b>.
</p>
</html>"));
      end Not;

      block Pre "Breaks algebraic loops by an infinitesimal small time delay (y = pre(u): event iteration continues until u = pre(u))"
        parameter Boolean pre_u_start = false "Start value of pre(u) at initial time";
        extends Blocks.Interfaces.partialBooleanSISO;
      initial equation
        pre(u) = pre_u_start;
      equation
        y = pre(u);
        annotation(
          defaultComponentName = "pre1",
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-90, 40}, {90, -40}}, lineColor = {0, 0, 0}, textString = "pre")}),
          Documentation(info = "<html>
<p>
This block delays the Boolean input by an infinitesimal small time delay and
therefore breaks algebraic loops. In a network of logical blocks, in every
\"closed connection loop\" at least one logical block must have a delay,
since algebraic systems of Boolean equations are not solvable.
</p>

<p>
The \"Pre\" block returns the value of the \"input\" signal from the
last \"event iteration\". The \"event iteration\" stops, once both
values are identical (u = pre(u)).
</p>
</html>"));
      end Pre;

      block Edge "Output y is true, if the input u has a rising edge (y = edge(u))"
        parameter Boolean pre_u_start = false "Start value of pre(u) at initial time";
        extends Blocks.Interfaces.partialBooleanSISO;
      initial equation
        pre(u) = pre_u_start;
      equation
        y = edge(u);
        annotation(
          defaultComponentName = "edge1",
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-90, 40}, {90, -40}}, lineColor = {0, 0, 0}, textString = "edge")}),
          Documentation(info = "<html>
<p>
The output is <b>true</b> if the Boolean input has a rising edge
from <b>false</b> to <b>true</b>, otherwise
the output is <b>false</b>.
</p>
</html>"));
      end Edge;

      block FallingEdge "Output y is true, if the input u has a falling edge (y = edge(not u))"
        parameter Boolean pre_u_start = false "Start value of pre(u) at initial time";
        extends Blocks.Interfaces.partialBooleanSISO;
      protected
        Boolean not_u = not u;
      initial equation
        pre(not_u) = not pre_u_start;
      equation
        y = edge(not_u);
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-90, 40}, {90, -40}}, lineColor = {0, 0, 0}, textString = "falling")}),
          Documentation(info = "<html>
<p>
The output is <b>true</b> if the Boolean input has a falling edge
from <b>true</b> to <b>false</b>, otherwise
the output is <b>false</b>.
</p>
</html>"));
      end FallingEdge;

      block Change "Output y is true, if the input u has a rising or falling edge (y = change(u))"
        parameter Boolean pre_u_start = false "Start value of pre(u) at initial time";
        extends Blocks.Interfaces.partialBooleanSISO;
      initial equation
        pre(u) = pre_u_start;
      equation
        y = change(u);
        annotation(
          defaultComponentName = "change1",
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-90, 40}, {90, -40}}, lineColor = {0, 0, 0}, textString = "change")}),
          Documentation(info = "<html>
<p>
The output is <b>true</b> if the Boolean input has either a rising edge
from <b>false</b> to <b>true</b> or a falling edge from
<b>true</b> to <b>false</b>, otherwise
the output is <b>false</b>.
</p>
</html>"));
      end Change;

      block GreaterThreshold "Output y is true, if input u is greater than threshold"
        extends Blocks.Interfaces.partialBooleanThresholdComparison;
      equation
        y = u > threshold;
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-90, -40}, {60, 40}}, lineColor = {0, 0, 0}, textString = ">")}),
          Documentation(info = "<html>
<p>
The output is <b>true</b> if the Real input is greater than
parameter <b>threshold</b>, otherwise
the output is <b>false</b>.
</p>
</html>"));
      end GreaterThreshold;

      block GreaterEqualThreshold "Output y is true, if input u is greater or equal than threshold"
        extends Blocks.Interfaces.partialBooleanThresholdComparison;
      equation
        y = u >= threshold;
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-90, -40}, {60, 40}}, lineColor = {0, 0, 0}, textString = ">=")}),
          Documentation(info = "<html>
<p>
The output is <b>true</b> if the Real input is greater than or equal to
parameter <b>threshold</b>, otherwise
the output is <b>false</b>.
</p>
</html>"));
      end GreaterEqualThreshold;

      block LessThreshold "Output y is true, if input u is less than threshold"
        extends Blocks.Interfaces.partialBooleanThresholdComparison;
      equation
        y = u < threshold;
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-90, -40}, {60, 40}}, lineColor = {0, 0, 0}, textString = "<")}),
          Documentation(info = "<html>
<p>
The output is <b>true</b> if the Real input is less than
parameter <b>threshold</b>, otherwise
the output is <b>false</b>.
</p>
</html>"));
      end LessThreshold;

      block LessEqualThreshold "Output y is true, if input u is less or equal than threshold"
        extends Blocks.Interfaces.partialBooleanThresholdComparison;
      equation
        y = u <= threshold;
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-90, -40}, {60, 40}}, lineColor = {0, 0, 0}, textString = "<=")}),
          Documentation(info = "<html>
<p>
The output is <b>true</b> if the Real input is less than or equal to
parameter <b>threshold</b>, otherwise
the output is <b>false</b>.
</p>
</html>"));
      end LessEqualThreshold;

      block Greater "Output y is true, if input u1 is greater than input u2"
        extends Blocks.Interfaces.partialBooleanComparison;
      equation
        y = u1 > u2;
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{32, 10}, {52, -10}}, lineColor = {0, 0, 127}), Line(points = {{-100, -80}, {42, -80}, {42, 0}}, color = {0, 0, 127}), Line(points = {{-54, 22}, {-8, 2}, {-54, -18}}, color = {0, 0, 0}, smooth = Smooth.None, thickness = 0.5)}),
          Documentation(info = "<html>
<p>
The output is <b>true</b> if Real input u1 is greater than
Real input u2, otherwise the output is <b>false</b>.
</p>
</html>"));
      end Greater;

      block GreaterEqual "Output y is true, if input u1 is greater or equal than input u2"
        extends Blocks.Interfaces.partialBooleanComparison;
      equation
        y = u1 >= u2;
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{32, 10}, {52, -10}}, lineColor = {0, 0, 127}), Line(points = {{-100, -80}, {42, -80}, {42, 0}}, color = {0, 0, 127}), Line(points = {{-54, 22}, {-8, 2}, {-54, -18}}, color = {0, 0, 0}, smooth = Smooth.None, thickness = 0.5), Line(points = {{-52, -36}, {-6, -14}}, color = {0, 0, 0}, thickness = 0.5, smooth = Smooth.None)}),
          Documentation(info = "<html>
<p>
The output is <b>true</b> if Real input u1 is greater than or equal to
Real input u2, otherwise the output is <b>false</b>.
</p>
</html>"));
      end GreaterEqual;

      block Less "Output y is true, if input u1 is less than input u2"
        extends Blocks.Interfaces.partialBooleanComparison;
      equation
        y = u1 < u2;
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{32, 10}, {52, -10}}, lineColor = {0, 0, 127}), Line(points = {{-100, -80}, {42, -80}, {42, 0}}, color = {0, 0, 127}), Line(points = {{-6, 18}, {-50, -2}, {-6, -20}}, color = {0, 0, 0}, thickness = 0.5, smooth = Smooth.None)}),
          Documentation(info = "<html>
<p>
The output is <b>true</b> if Real input u1 is less than
Real input u2, otherwise the output is <b>false</b>.
</p>
</html>"));
      end Less;

      block LessEqual "Output y is true, if input u1 is less or equal than input u2"
        extends Blocks.Interfaces.partialBooleanComparison;
      equation
        y = u1 <= u2;
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{32, 10}, {52, -10}}, lineColor = {0, 0, 127}), Line(points = {{-100, -80}, {42, -80}, {42, 0}}, color = {0, 0, 127}), Line(points = {{-10, 20}, {-54, 0}, {-10, -18}}, color = {0, 0, 0}, thickness = 0.5, smooth = Smooth.None), Line(points = {{-54, -18}, {-14, -34}}, color = {0, 0, 0}, thickness = 0.5, smooth = Smooth.None)}),
          Documentation(info = "<html>
<p>
The output is <b>true</b> if Real input u1 is less than or equal to
Real input u2, otherwise the output is <b>false</b>.
</p>
</html>"));
      end LessEqual;

      block ZeroCrossing "Trigger zero crossing of input u"
        extends Blocks.Interfaces.partialBooleanSO;
        Blocks.Interfaces.RealInput u annotation(
          Placement(transformation(extent = {{-140, -20}, {-100, 20}}, rotation = 0)));
        Blocks.Interfaces.BooleanInput enable "Zero input crossing is triggered if the enable input signal is true" annotation(
          Placement(transformation(origin = {0, -120}, extent = {{-20, -20}, {20, 20}}, rotation = 90)));
      protected
        Boolean disable = not enable;
        Boolean u_pos;
      initial equation
        pre(u_pos) = false;
        pre(enable) = false;
        pre(disable) = not pre(enable);
      equation
        u_pos = enable and u >= 0;
        y = change(u_pos) and not edge(enable) and not edge(disable);
        annotation(
          Documentation(info = "<HTML>
<p>
The output \"y\" is <b>true</b> at the
time instant when the input \"u\" becomes
zero, provided the input \"enable\" is
<b>true</b>. At all other time instants, the output \"y\" is <b>false</b>.
If the input \"u\" is zero at a time instant when the \"enable\"
input changes its value, then the output y is <b>false</b>.
</p>
<p>
Note, that in the plot window of a Modelica simulator, the output of
this block is usually identically to <b>false</b>, because the output
may only be <b>true</b> at an event instant, but not during
continuous integration. In order to check that this component is
actually working as expected, one should connect its output to, e.g.,
component <i>ModelicaAdditions.Blocks.Discrete.TriggeredSampler</i>.
</p>
</HTML>"),
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-78, 68}, {-78, -80}}, color = {192, 192, 192}), Polygon(points = {{-78, 90}, {-86, 68}, {-70, 68}, {-78, 90}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-88, 0}, {70, 0}}, color = {192, 192, 192}), Line(points = {{-78, 0}, {-73.2, 32.3}, {-70, 50.3}, {-66.7, 64.5}, {-63.5, 74.2}, {-60.3, 79.3}, {-57.1, 79.6}, {-53.9, 75.3}, {-50.7, 67.1}, {-46.6, 52.2}, {-41, 25.8}, {-33, -13.9}, {-28.2, -33.7}, {-24.1, -45.9}, {-20.1, -53.2}, {-16.1, -55.3}, {-12.1, -52.5}, {-8.1, -45.3}, {-3.23, -32.1}, {10.44, 13.7}, {15.3, 26.4}, {20.1, 34.8}, {24.1, 38}, {28.9, 37.2}, {33.8, 31.8}, {40.2, 19.4}, {53.1, -10.5}, {59.5, -21.2}, {65.1, -25.9}, {70.7, -25.9}, {77.2, -20.5}, {82, -13.8}}, color = {192, 192, 192}, smooth = Smooth.Bezier), Polygon(points = {{92, 0}, {70, 8}, {70, -8}, {92, 0}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-36, -59}, {-36, 81}}, color = {255, 0, 255}), Line(points = {{6, -59}, {6, 81}}, color = {255, 0, 255}), Line(points = {{49, -59}, {49, 81}}, color = {255, 0, 255}), Line(points = {{-78, 0}, {70, 0}}, color = {255, 0, 255})}));
      end ZeroCrossing;

      block LogicalSwitch "Logical Switch"
        extends Blocks.Interfaces.partialBooleanSI3SO;
      equation
        y = if u2 then u1 else u3;
        annotation(
          Documentation(info = "<html>
<p>The LogicalSwitch switches, depending on the
Boolean u2 connector (the middle connector),
between the two possible input signals
u1 (upper connector) and u3 (lower connector).</p>
<p>If u2 is true, connector y is set equal to
u1, else it is set equal to u3.</p>
</html>"),
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}, initialScale = 0.1), graphics = {Line(points = {{12, 0}, {100, 0}}, color = {255, 0, 255}), Line(points = {{-100, 0}, {-40, 0}}, color = {255, 0, 255}), Line(points = {{-100, -80}, {-40, -80}, {-40, -80}}, color = {255, 0, 255}), Line(points = {{-40, 12}, {-40, -10}}, color = {255, 0, 255}), Line(points = {{-100, 80}, {-40, 80}}, color = {255, 0, 255}), Line(points = {{-40, 80}, {8, 2}}, color = {255, 0, 255}, thickness = 1), Ellipse(visible = true, lineColor = {0, 0, 127}, pattern = LinePattern.None, fillPattern = FillPattern.Solid, extent = {{2.0, -6.0}, {18.0, 8.0}})}));
      end LogicalSwitch;

      block Switch "Switch between two Real signals"
        extends Modelica.Blocks.Icons.PartialBooleanBlock;
        Blocks.Interfaces.RealInput u1 "Connector of first Real input signal" annotation(
          Placement(transformation(extent = {{-140, 60}, {-100, 100}}, rotation = 0)));
        Blocks.Interfaces.BooleanInput u2 "Connector of Boolean input signal" annotation(
          Placement(transformation(extent = {{-140, -20}, {-100, 20}}, rotation = 0)));
        Blocks.Interfaces.RealInput u3 "Connector of second Real input signal" annotation(
          Placement(transformation(extent = {{-140, -100}, {-100, -60}}, rotation = 0)));
        Blocks.Interfaces.RealOutput y "Connector of Real output signal" annotation(
          Placement(transformation(extent = {{100, -10}, {120, 10}}, rotation = 0)));
      equation
        y = if u2 then u1 else u3;
        annotation(
          defaultComponentName = "switch1",
          Documentation(info = "<html>
<p>The Logical.Switch switches, depending on the
logical connector u2 (the middle connector)
between the two possible input signals
u1 (upper connector) and u3 (lower connector).</p>
<p>If u2 is <b>true</b>, the output signal y is set equal to
u1, else it is set equal to u3.</p>
</html>"),
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}, initialScale = 0.1), graphics = {Line(points = {{12.0, 0.0}, {100.0, 0.0}}, color = {0, 0, 127}), Line(points = {{-100.0, 0.0}, {-40.0, 0.0}}, color = {255, 0, 255}), Line(points = {{-100.0, -80.0}, {-40.0, -80.0}, {-40.0, -80.0}}, color = {0, 0, 127}), Line(points = {{-40.0, 12.0}, {-40.0, -12.0}}, color = {255, 0, 255}), Line(points = {{-100.0, 80.0}, {-38.0, 80.0}}, color = {0, 0, 127}), Line(points = {{-38.0, 80.0}, {6.0, 2.0}}, color = {0, 0, 127}, thickness = 1.0), Ellipse(lineColor = {0, 0, 255}, pattern = LinePattern.None, fillPattern = FillPattern.Solid, extent = {{2.0, -8.0}, {18.0, 8.0}})}));
      end Switch;

      block Hysteresis "Transform Real to Boolean signal with Hysteresis"
        extends Modelica.Blocks.Icons.PartialBooleanBlock;
        parameter Real uLow(start = 0) "if y=true and u<=uLow, switch to y=false";
        parameter Real uHigh(start = 1) "if y=false and u>=uHigh, switch to y=true";
        parameter Boolean pre_y_start = false "Value of pre(y) at initial time";
        Blocks.Interfaces.RealInput u annotation(
          Placement(transformation(extent = {{-140, -20}, {-100, 20}}, rotation = 0)));
        Blocks.Interfaces.BooleanOutput y annotation(
          Placement(transformation(extent = {{100, -10}, {120, 10}}, rotation = 0)));
      initial equation
        pre(y) = pre_y_start;
      equation
        y = u > uHigh or pre(y) and u >= uLow;
        annotation(
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{-65, 89}, {-73, 67}, {-57, 67}, {-65, 89}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-65, 67}, {-65, -81}}, color = {192, 192, 192}), Line(points = {{-90, -70}, {82, -70}}, color = {192, 192, 192}), Polygon(points = {{90, -70}, {68, -62}, {68, -78}, {90, -70}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Text(extent = {{70, -80}, {94, -100}}, lineColor = {160, 160, 164}, textString = "u"), Text(extent = {{-65, 93}, {-12, 75}}, lineColor = {160, 160, 164}, textString = "y"), Line(points = {{-80, -70}, {30, -70}}, color = {0, 0, 0}, thickness = 0.5), Line(points = {{-50, 10}, {80, 10}}, color = {0, 0, 0}, thickness = 0.5), Line(points = {{-50, 10}, {-50, -70}}, color = {0, 0, 0}, thickness = 0.5), Line(points = {{30, 10}, {30, -70}}, color = {0, 0, 0}, thickness = 0.5), Line(points = {{-10, -65}, {0, -70}, {-10, -75}}, color = {0, 0, 0}, thickness = 0.5), Line(points = {{-10, 15}, {-20, 10}, {-10, 5}}, color = {0, 0, 0}, thickness = 0.5), Line(points = {{-55, -20}, {-50, -30}, {-44, -20}}, color = {0, 0, 0}, thickness = 0.5), Line(points = {{25, -30}, {30, -19}, {35, -30}}, color = {0, 0, 0}, thickness = 0.5), Text(extent = {{-99, 2}, {-70, 18}}, lineColor = {160, 160, 164}, textString = "true"), Text(extent = {{-98, -87}, {-66, -73}}, lineColor = {160, 160, 164}, textString = "false"), Text(extent = {{19, -87}, {44, -70}}, lineColor = {0, 0, 0}, textString = "uHigh"), Text(extent = {{-63, -88}, {-38, -71}}, lineColor = {0, 0, 0}, textString = "uLow"), Line(points = {{-69, 10}, {-60, 10}}, color = {160, 160, 164})}),
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{-80, 90}, {-88, 68}, {-72, 68}, {-80, 90}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-80, 68}, {-80, -29}}, color = {192, 192, 192}), Polygon(points = {{92, -29}, {70, -21}, {70, -37}, {92, -29}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-79, -29}, {84, -29}}, color = {192, 192, 192}), Line(points = {{-79, -29}, {41, -29}}, color = {0, 0, 0}), Line(points = {{-15, -21}, {1, -29}, {-15, -36}}, color = {0, 0, 0}), Line(points = {{41, 51}, {41, -29}}, color = {0, 0, 0}), Line(points = {{33, 3}, {41, 22}, {50, 3}}, color = {0, 0, 0}), Line(points = {{-49, 51}, {81, 51}}, color = {0, 0, 0}), Line(points = {{-4, 59}, {-19, 51}, {-4, 43}}, color = {0, 0, 0}), Line(points = {{-59, 29}, {-49, 11}, {-39, 29}}, color = {0, 0, 0}), Line(points = {{-49, 51}, {-49, -29}}, color = {0, 0, 0}), Text(extent = {{-92, -49}, {-9, -92}}, lineColor = {192, 192, 192}, textString = "%uLow"), Text(extent = {{2, -49}, {91, -92}}, lineColor = {192, 192, 192}, textString = "%uHigh"), Rectangle(extent = {{-91, -49}, {-8, -92}}, lineColor = {192, 192, 192}), Line(points = {{-49, -29}, {-49, -49}}, color = {192, 192, 192}), Rectangle(extent = {{2, -49}, {91, -92}}, lineColor = {192, 192, 192}), Line(points = {{41, -29}, {41, -49}}, color = {192, 192, 192})}),
          Documentation(info = "<HTML>
<p>
This block transforms a <b>Real</b> input signal into a <b>Boolean</b>
output signal:
</p>
<ul>
<li> When the output was <b>false</b> and the input becomes
     <b>greater</b> than parameter <b>uHigh</b>, the output
     switches to <b>true</b>.</li>
<li> When the output was <b>true</b> and the input becomes
     <b>less</b> than parameter <b>uLow</b>, the output
     switches to <b>false</b>.</li>
</ul>
<p>
The start value of the output is defined via parameter
<b>pre_y_start</b> (= value of pre(y) at initial time).
The default value of this parameter is <b>false</b>.
</p>
</html>"));
      end Hysteresis;

      block OnOffController "On-off controller"
        extends Modelica.Blocks.Icons.PartialBooleanBlock;
        Blocks.Interfaces.RealInput reference "Connector of Real input signal used as reference signal" annotation(
          Placement(transformation(extent = {{-140, 80}, {-100, 40}}, rotation = 0)));
        Blocks.Interfaces.RealInput u "Connector of Real input signal used as measurement signal" annotation(
          Placement(transformation(extent = {{-140, -40}, {-100, -80}}, rotation = 0)));
        Blocks.Interfaces.BooleanOutput y "Connector of Real output signal used as actuator signal" annotation(
          Placement(transformation(extent = {{100, -10}, {120, 10}}, rotation = 0)));
        parameter Real bandwidth(start = 0.1) "Bandwidth around reference signal";
        parameter Boolean pre_y_start = false "Value of pre(y) at initial time";
      initial equation
        pre(y) = pre_y_start;
      equation
        y = pre(y) and u < reference + bandwidth / 2 or u < reference - bandwidth / 2;
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}, initialScale = 0.1), graphics = {Text(extent = {{-92, 74}, {44, 44}}, lineThickness = 0.5, textString = "reference"), Text(extent = {{-94, -52}, {-34, -74}}, textString = "u"), Line(points = {{-76.0, -32.0}, {-68.0, -6.0}, {-50.0, 26.0}, {-24.0, 40.0}, {-2.0, 42.0}, {16.0, 36.0}, {32.0, 28.0}, {48.0, 12.0}, {58.0, -6.0}, {68.0, -28.0}}, color = {0, 0, 127}), Line(points = {{-78.0, -2.0}, {-6.0, 18.0}, {82.0, -12.0}}, color = {255, 0, 0}), Line(points = {{-78.0, 12.0}, {-6.0, 30.0}, {82.0, 0.0}}), Line(points = {{-78.0, -16.0}, {-6.0, 4.0}, {82.0, -26.0}}), Line(points = {{-82.0, -18.0}, {-56.0, -18.0}, {-56.0, -40.0}, {64.0, -40.0}, {64.0, -20.0}, {90.0, -20.0}}, color = {255, 0, 255})}),
          Documentation(info = "<html>
<p>The block OnOffController sets the output signal <b>y</b> to <b>true</b> when
the input signal <b>u</b> falls below the <b>reference</b> signal minus half of
the bandwidth and sets the output signal <b>y</b> to <b>false</b> when the input
signal <b>u</b> exceeds the <b>reference</b> signal plus half of the bandwidth.</p>
</html>"));
      end OnOffController;

      block TriggeredTrapezoid "Triggered trapezoid generator"
        extends Modelica.Blocks.Icons.PartialBooleanBlock;
        parameter Real amplitude = 1 "Amplitude of trapezoid";
        parameter Modelica.SIunits.Time rising(final min = 0) = 0 "Rising duration of trapezoid";
        parameter Modelica.SIunits.Time falling(final min = 0) = rising "Falling duration of trapezoid";
        parameter Real offset = 0 "Offset of output signal";
        Blocks.Interfaces.BooleanInput u "Connector of Boolean input signal" annotation(
          Placement(transformation(extent = {{-140, -20}, {-100, 20}}, rotation = 0)));
        Blocks.Interfaces.RealOutput y "Connector of Real output signal" annotation(
          Placement(transformation(extent = {{100, -10}, {120, 10}}, rotation = 0)));
      protected
        discrete Real endValue "Value of y at time of recent edge";
        discrete Real rate "Current rising/falling rate";
        discrete Modelica.SIunits.Time T "Predicted time of output reaching endValue";
      equation
        y = if time < T then endValue - (T - time) * rate else endValue;
        when {initial(), u, not u} then
          endValue = if u then offset + amplitude else offset;
          rate = if u and rising > 0 then amplitude / rising else if not u and falling > 0 then -amplitude / falling else 0;
          T = if u and not rising > 0 or not u and not falling > 0 or not abs(amplitude) > 0 or initial() then time else time + (endValue - pre(y)) / rate;
        end when;
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100.0, -100.0}, {100.0, 100.0}}, initialScale = 0.1), graphics = {Line(visible = true, points = {{-60.0, -70.0}, {-60.0, -70.0}, {-30.0, 40.0}, {8.0, 40.0}, {40.0, -70.0}, {40.0, -70.0}}, color = {0, 0, 127}), Line(visible = true, points = {{-90.0, -70.0}, {82.0, -70.0}}, color = {192, 192, 192}), Line(visible = true, points = {{-80.0, 68.0}, {-80.0, -80.0}}, color = {192, 192, 192}), Polygon(visible = true, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid, points = {{90.0, -70.0}, {68.0, -62.0}, {68.0, -78.0}, {90.0, -70.0}}), Polygon(visible = true, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid, points = {{-80.0, 90.0}, {-88.0, 68.0}, {-72.0, 68.0}, {-80.0, 90.0}}), Line(visible = true, points = {{-80.0, -70.0}, {-60.0, -70.0}, {-60.0, 24.0}, {8.0, 24.0}, {8.0, -70.0}, {60.0, -70.0}}, color = {255, 0, 255})}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-80, -20}, {-60, -20}, {-30, 40}, {8, 40}, {40, -20}, {60, -20}}, color = {0, 0, 255}), Line(points = {{-90, -70}, {82, -70}}, color = {0, 0, 0}), Line(points = {{-80, 68}, {-80, -80}}, color = {0, 0, 0}), Polygon(points = {{90, -70}, {68, -62}, {68, -78}, {90, -70}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Polygon(points = {{-80, 90}, {-88, 68}, {-72, 68}, {-80, 90}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{-80, -68}, {-60, -68}, {-60, -42}, {8, -42}, {8, -68}, {60, -68}}, color = {255, 0, 255}), Line(points = {{-60, 40}, {-60, -42}}, color = {0, 0, 0}, pattern = LinePattern.Dot), Line(points = {{8, -42}, {8, 40}}, color = {0, 0, 0}, pattern = LinePattern.Dot), Line(points = {{-60, 40}, {-30, 40}}, color = {0, 0, 0}), Line(points = {{8, -20}, {40, -20}}, color = {0, 0, 0}), Line(points = {{-20, 40}, {-20, -20}}, color = {0, 0, 0}), Line(points = {{-20, -20}, {-20, -70}}, color = {0, 0, 0}), Text(extent = {{-44, 50}, {-44, 40}}, lineColor = {0, 0, 0}, textString = "rising"), Text(extent = {{24, -10}, {24, -20}}, lineColor = {0, 0, 0}, textString = "falling"), Polygon(points = {{-60, 40}, {-56, 42}, {-56, 38}, {-60, 40}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Polygon(points = {{-30, 40}, {-34, 42}, {-34, 38}, {-30, 40}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Polygon(points = {{8, -20}, {12, -18}, {12, -22}, {8, -20}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Polygon(points = {{40, -20}, {36, -18}, {36, -22}, {40, -20}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Polygon(points = {{-22, -24}, {-20, -20}, {-18, -24}, {-22, -24}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Polygon(points = {{-18, -66}, {-22, -66}, {-20, -70}, {-18, -66}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Polygon(points = {{-22, 36}, {-20, 40}, {-18, 36}, {-22, 36}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Polygon(points = {{-18, -16}, {-22, -16}, {-20, -20}, {-18, -16}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-40, 6}, {0, -4}}, lineColor = {255, 255, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-20, 6}, {-20, -4}}, lineColor = {0, 0, 0}, textString = "amplitude"), Rectangle(extent = {{-40, -48}, {0, -58}}, lineColor = {255, 255, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-20, -48}, {-20, -58}}, lineColor = {0, 0, 0}, textString = "offset"), Text(extent = {{60, -82}, {94, -92}}, lineColor = {0, 0, 0}, textString = "time"), Text(extent = {{-88, -4}, {-54, -14}}, lineColor = {0, 0, 0}, textString = "y"), Text(extent = {{-88, -46}, {-54, -56}}, lineColor = {0, 0, 0}, textString = "u")}),
          Documentation(info = "<HTML>
<p>The block TriggeredTrapezoid has a Boolean input and a real
output signal and requires the parameters <i>amplitude</i>,
<i>rising</i>, <i>falling</i> and <i>offset</i>. The
output signal <b>y</b> represents a trapezoidal signal dependent on the
input signal <b>u</b>.
</p>
<p>The behaviour is as follows: Assume the initial input to be false. In this
case, the output will be <i>offset</i>. After a rising edge (i.e., the input
changes from false to true), the output is rising during <i>rising</i> to the
sum of <i>offset</i> and <i>amplitude</i>. In contrast, after a falling
edge (i.e., the input changes from true to false), the output is falling
during <i>falling</i> to a value of <i>offset</i>.
</p>
<p>Note, that the case of edges before expiration of rising or falling is
handled properly.</p>
</html>"));
      end TriggeredTrapezoid;

      block Timer "Timer measuring the time from the time instant where the Boolean input became true"
        extends Modelica.Blocks.Icons.PartialBooleanBlock;
        Blocks.Interfaces.BooleanInput u "Connector of Boolean input signal" annotation(
          Placement(transformation(extent = {{-140, -20}, {-100, 20}}, rotation = 0)));
        Blocks.Interfaces.RealOutput y "Connector of Real output signal" annotation(
          Placement(transformation(extent = {{100, -10}, {120, 10}}, rotation = 0)));
      protected
        discrete Modelica.SIunits.Time entryTime "Time instant when u became true";
      initial equation
        pre(entryTime) = 0;
      equation
        when u then
          entryTime = time;
        end when;
        y = if u then time - entryTime else 0.0;
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100.0, -100.0}, {100.0, 100.0}}, initialScale = 0.1), graphics = {Line(visible = true, points = {{-90.0, -70.0}, {82.0, -70.0}}, color = {192, 192, 192}), Line(visible = true, points = {{-80.0, 68.0}, {-80.0, -80.0}}, color = {192, 192, 192}), Polygon(visible = true, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid, points = {{90.0, -70.0}, {68.0, -62.0}, {68.0, -78.0}, {90.0, -70.0}}), Polygon(visible = true, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid, points = {{-80.0, 90.0}, {-88.0, 68.0}, {-72.0, 68.0}, {-80.0, 90.0}}), Line(visible = true, points = {{-80.0, -70.0}, {-60.0, -70.0}, {-60.0, -26.0}, {38.0, -26.0}, {38.0, -70.0}, {66.0, -70.0}}, color = {255, 0, 255}), Line(visible = true, points = {{-80.0, 0.0}, {-62.0, 0.0}, {40.0, 90.0}, {40.0, 0.0}, {68.0, 0.0}}, color = {0, 0, 127})}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-90, -70}, {82, -70}}, color = {0, 0, 0}), Line(points = {{-80, 68}, {-80, -80}}, color = {0, 0, 0}), Polygon(points = {{90, -70}, {68, -62}, {68, -78}, {90, -70}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Polygon(points = {{-80, 90}, {-88, 68}, {-72, 68}, {-80, 90}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{-80, -68}, {-60, -68}, {-60, -40}, {20, -40}, {20, -68}, {60, -68}}, color = {255, 0, 255}), Line(points = {{-80, -20}, {-60, -20}, {20, 60}, {20, -20}, {60, -20}, {60, -20}}, color = {0, 0, 255}), Text(extent = {{-88, 6}, {-54, -4}}, lineColor = {0, 0, 0}, textString = "y"), Text(extent = {{48, -80}, {84, -88}}, lineColor = {0, 0, 0}, textString = "time"), Text(extent = {{-88, -36}, {-54, -46}}, lineColor = {0, 0, 0}, textString = "u")}),
          Documentation(info = "<HTML>
<p> When the Boolean input \"u\" becomes <b>true</b>, the timer is started
and the output \"y\" is the time from the time instant where u became true.
The timer is stopped and the output is reset to zero, once the
input becomes false.
</p>
</html>"));
      end Timer;

      block RSFlipFlop "A basic RS Flip Flop"
        extends Modelica.Blocks.Icons.PartialBooleanBlock;
        parameter Boolean Qini = false "Start value of Q at initial time";
        Modelica.Blocks.Interfaces.BooleanOutput Q annotation(
          Placement(transformation(extent = {{100, 50}, {120, 70}}, rotation = 0)));
        Modelica.Blocks.Interfaces.BooleanOutput QI annotation(
          Placement(transformation(extent = {{100, -70}, {120, -50}}, rotation = 0)));
        Modelica.Blocks.Logical.Nor nor annotation(
          Placement(transformation(extent = {{-20, 20}, {0, 40}}, rotation = 0)));
        Modelica.Blocks.Logical.Nor nor1 annotation(
          Placement(transformation(extent = {{-20, -20}, {0, 0}}, rotation = 0)));
        Modelica.Blocks.Logical.Pre pre(pre_u_start = not Qini) annotation(
          Placement(transformation(extent = {{10, 20}, {30, 40}}, rotation = 0)));
        Modelica.Blocks.Interfaces.BooleanInput S annotation(
          Placement(transformation(extent = {{-140, 40}, {-100, 80}})));
        Modelica.Blocks.Interfaces.BooleanInput R annotation(
          Placement(transformation(extent = {{-140, -80}, {-100, -40}})));
      equation
        connect(nor1.y, nor.u2) annotation(
          Line(points = {{1, -10}, {40, -10}, {40, -40}, {-60, -40}, {-60, 22}, {-22, 22}}, color = {255, 0, 255}));
        connect(nor1.y, Q) annotation(
          Line(points = {{1, -10}, {60, -10}, {60, 60}, {110, 60}}, color = {255, 0, 255}));
        connect(nor.y, pre.u) annotation(
          Line(points = {{1, 30}, {8, 30}}, color = {255, 0, 255}));
        connect(pre.y, nor1.u1) annotation(
          Line(points = {{31, 30}, {40, 30}, {40, 10}, {-40, 10}, {-40, -10}, {-22, -10}}, color = {255, 0, 255}));
        connect(pre.y, QI) annotation(
          Line(points = {{31, 30}, {80, 30}, {80, -60}, {110, -60}}, color = {255, 0, 255}));
        connect(S, nor.u1) annotation(
          Line(points = {{-120, 60}, {-40, 60}, {-40, 30}, {-22, 30}}, color = {255, 0, 255}, smooth = Smooth.None));
        connect(R, nor1.u2) annotation(
          Line(points = {{-120, -60}, {-40, -60}, {-40, -18}, {-22, -18}}, color = {255, 0, 255}, smooth = Smooth.None));
        annotation(
          Icon(graphics = {Text(extent = {{-60, -30}, {-20, -90}}, lineColor = {0, 0, 0}, textString = "R"), Text(extent = {{-62, 90}, {-22, 30}}, lineColor = {0, 0, 0}, textString = "S"), Text(extent = {{20, 90}, {60, 30}}, lineColor = {0, 0, 0}, textString = "Q"), Text(extent = {{6, -30}, {66, -90}}, lineColor = {0, 0, 0}, textString = "Q!"), Ellipse(extent = {{-73, 54}, {-87, 68}}, lineColor = DynamicSelect({235, 235, 235}, if S > 0.5 then {0, 255, 0} else {235, 235, 235}), fillColor = DynamicSelect({235, 235, 235}, if S > 0.5 then {0, 255, 0} else {235, 235, 235}), fillPattern = FillPattern.Solid), Ellipse(extent = {{83, -53}, {69, -67}}, lineColor = DynamicSelect({235, 235, 235}, if QI > 0.5 then {0, 255, 0} else {235, 235, 235}), fillColor = DynamicSelect({235, 235, 235}, if QI > 0.5 then {0, 255, 0} else {235, 235, 235}), fillPattern = FillPattern.Solid), Ellipse(extent = {{-71, -52}, {-85, -66}}, lineColor = DynamicSelect({235, 235, 235}, if R > 0.5 then {0, 255, 0} else {235, 235, 235}), fillColor = DynamicSelect({235, 235, 235}, if R > 0.5 then {0, 255, 0} else {235, 235, 235}), fillPattern = FillPattern.Solid), Ellipse(extent = {{71, 67}, {85, 53}}, lineColor = DynamicSelect({235, 235, 235}, if Q > 0.5 then {0, 255, 0} else {235, 235, 235}), fillColor = DynamicSelect({235, 235, 235}, if Q > 0.5 then {0, 255, 0} else {235, 235, 235}), fillPattern = FillPattern.Solid)}),
          Documentation(info = "<html>
<p>
The output <code>Q</code> is set by the input <code>S</code> and reset by the input <code>R</code>, and keeps its value in between. <code>QI</code> is the inverse of <code>Q</code>.
</p>
</html>"));
      end RSFlipFlop;

      block TerminateSimulation "Terminate simulation if condition is fulfilled"
        Modelica.Blocks.Interfaces.BooleanOutput condition = false "Terminate simulation when condition becomes true" annotation(
          Dialog,
          Placement(transformation(extent = {{200, -10}, {220, 10}}, rotation = 0)));
        parameter String terminationText = "... End condition reached" "Text that will be displayed when simulation is terminated";
      equation
        when condition then
          terminate(terminationText);
        end when;
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-200.0, -20.0}, {200.0, 20.0}}, initialScale = 0.2), graphics = {Rectangle(visible = true, fillColor = {235, 235, 235}, fillPattern = FillPattern.Solid, lineThickness = 5.0, borderPattern = BorderPattern.Raised, extent = {{-200.0, -20.0}, {200.0, 20.0}}), Text(visible = true, extent = {{-166.0, -15.0}, {194.0, 15.0}}, textString = "%condition"), Rectangle(visible = true, fillColor = {161, 35, 41}, fillPattern = FillPattern.Solid, borderPattern = BorderPattern.Raised, extent = {{-194.0, -14.0}, {-168.0, 14.0}}), Text(visible = true, lineColor = {0, 0, 255}, extent = {{-200.0, 22.0}, {200.0, 46.0}}, textString = "%name")}),
          Documentation(info = "<html>
<p>
In the parameter menu, a <b>time varying</b> expression can be defined
via variable <b>condition</b>, for example \"condition = x &lt; 0\",
where \"x\" is a variable that is declared in the model in which the
\"TerminateSimulation\" block is present.
If this expression becomes <b>true</b>,
the simulation is (successfully) terminated. A termination message
explaining the reason for the termination can be given via
parameter \"terminationText\".
</p>

</html>"));
      end TerminateSimulation;
      annotation(
        Documentation(info = "<html>
<p>
This package provides blocks with Boolean input and output signals
to describe logical networks. A typical example for a logical
network built with package Logical is shown in the next figure:
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Blocks/LogicalNetwork1.png\"
     alt=\"LogicalNetwork1.png\">
</p>

<p>
The actual value of Boolean input and/or output signals is displayed
in the respective block icon as \"circle\", where \"white\" color means
value <b>false</b> and \"green\" color means value <b>true</b>. These
values are visualized in a diagram animation.
</p>
</html>"),
        Icon(graphics = {Line(visible = true, points = {{-86, -22}, {-50, -22}, {-50, 22}, {48, 22}, {48, -22}, {88, -24}}, color = {255, 0, 255})}));
    end Logical;

    package Math "Library of Real mathematical functions as input/output blocks"
      import Modelica.SIunits;
      import Modelica.Blocks.Interfaces;
      extends Modelica.Icons.Package;

      encapsulated package UnitConversions "Conversion blocks to convert between SI and non-SI unit signals"
        import Modelica;
        import SI = Modelica.SIunits;
        import NonSI = Modelica.SIunits.Conversions.NonSIunits;
        extends Modelica.Icons.Package;

        block To_degC "Convert from Kelvin to degCelsius"
          extends Modelica.Blocks.Interfaces.PartialConversionBlock(u(unit = "K"), y(unit = "degC"));
        equation
          y = SI.Conversions.to_degC(u);
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-20, 100}, {-100, 20}}, lineColor = {0, 0, 0}, textString = "K"), Text(extent = {{100, -20}, {20, -100}}, lineColor = {0, 0, 0}, textString = "degC")}),
            Documentation(info = "<html>
<p>
This block converts the input signal from Kelvin to degCelsius and returns
the result as output signal.
</p>
</html>"));
        end To_degC;

        block From_degC "Convert from degCelsius to Kelvin"
          extends Modelica.Blocks.Interfaces.PartialConversionBlock(u(unit = "degC"), y(unit = "K"));
        equation
          y = SI.Conversions.from_degC(u);
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-20, 100}, {-100, 20}}, lineColor = {0, 0, 0}, textString = "degC"), Text(extent = {{100, -20}, {20, -100}}, lineColor = {0, 0, 0}, textString = "K")}),
            Documentation(info = "<html>
<p>
This block converts the input signal from degCelsius to Kelvin and returns
the result as output signal.
</p>
</html>"));
        end From_degC;

        block To_degF "Convert from Kelvin to degFahrenheit"
          extends Modelica.Blocks.Interfaces.PartialConversionBlock(u(unit = "K"), y(unit = "degF"));
        equation
          y = SI.Conversions.to_degF(u);
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-20, 100}, {-100, 20}}, lineColor = {0, 0, 0}, textString = "K"), Text(extent = {{100, -20}, {20, -100}}, lineColor = {0, 0, 0}, textString = "degF")}),
            Documentation(info = "<html>
<p>
This block converts the input signal from Kelvin to degFahrenheit and returns
the result as output signal.
</p>
</html>"));
        end To_degF;

        block From_degF "Convert from degFahrenheit to Kelvin"
          extends Modelica.Blocks.Interfaces.PartialConversionBlock(u(unit = "degF"), y(unit = "K"));
        equation
          y = SI.Conversions.from_degF(u);
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-20, 100}, {-100, 20}}, lineColor = {0, 0, 0}, textString = "degF"), Text(extent = {{100, -20}, {20, -100}}, lineColor = {0, 0, 0}, textString = "K")}),
            Documentation(info = "<html>
<p>
This block converts the input signal from degFahrenheit to Kelvin and returns
the result as output signal.
</p>
</html>"));
        end From_degF;

        block To_degRk "Convert from Kelvin to degRankine"
          extends Modelica.Blocks.Interfaces.PartialConversionBlock(u(unit = "K"), y(unit = "degRk"));
        equation
          y = SI.Conversions.to_degRk(u);
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-20, 100}, {-100, 20}}, lineColor = {0, 0, 0}, textString = "K"), Text(extent = {{100, -20}, {20, -100}}, lineColor = {0, 0, 0}, textString = "degRk")}),
            Documentation(info = "<html>
<p>
This block converts the input signal from Kelvin to degRankine and returns
the result as output signal.
</p>
</html>"));
        end To_degRk;

        block From_degRk "Convert from degRankine to Kelvin"
          extends Modelica.Blocks.Interfaces.PartialConversionBlock(u(unit = "degRk"), y(unit = "K"));
        equation
          y = SI.Conversions.from_degRk(u);
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-20, 100}, {-100, 20}}, lineColor = {0, 0, 0}, textString = "degRk"), Text(extent = {{100, -20}, {20, -100}}, lineColor = {0, 0, 0}, textString = "K")}),
            Documentation(info = "<html>
<p>
This block converts the input signal from degRankine to Kelvin and returns
the result as output signal.
</p>
</html>"));
        end From_degRk;

        block To_deg "Convert from radian to degree"
          extends Modelica.Blocks.Interfaces.PartialConversionBlock(u(unit = "rad"), y(unit = "deg"));
        equation
          y = SI.Conversions.to_deg(u);
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-20, 100}, {-100, 20}}, lineColor = {0, 0, 0}, textString = "rad"), Text(extent = {{100, -20}, {20, -100}}, lineColor = {0, 0, 0}, textString = "deg")}),
            Documentation(info = "<html>
<p>
This block converts the input signal from radian to degree and returns
the result as output signal.
</p>
</html>"));
        end To_deg;

        block From_deg "Convert from degree to radian"
          extends Modelica.Blocks.Interfaces.PartialConversionBlock(u(unit = "deg"), y(unit = "rad"));
        equation
          y = SI.Conversions.from_deg(u);
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-20, 100}, {-100, 20}}, lineColor = {0, 0, 0}, textString = "deg"), Text(extent = {{100, -20}, {20, -100}}, lineColor = {0, 0, 0}, textString = "rad")}),
            Documentation(info = "<html>
<p>
This block converts the input signal from degree to radian and returns
the result as output signal.
</p>
</html>"));
        end From_deg;

        block To_rpm "Convert from radian per second to revolutions per minute"
          extends Modelica.Blocks.Interfaces.PartialConversionBlock(u(unit = "rad/s"), y(unit = "1/min"));
        equation
          y = SI.Conversions.to_rpm(u);
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{26, 82}, {-98, 50}}, lineColor = {0, 0, 0}, textString = "rad/s"), Text(extent = {{100, -42}, {-62, -74}}, lineColor = {0, 0, 0}, textString = "1/min")}),
            Documentation(info = "<html>
<p>
This block converts the input signal from radian per second to revolutions per minute and returns
the result as output signal.
</p>
</html>"));
        end To_rpm;

        block From_rpm "Convert from revolutions per minute to radian per second"
          extends Modelica.Blocks.Interfaces.PartialConversionBlock(u(unit = "1/min"), y(unit = "rad/s"));
        equation
          y = SI.Conversions.from_rpm(u);
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{50, 84}, {-94, 56}}, lineColor = {0, 0, 0}, textString = "1/min"), Text(extent = {{94, -42}, {-26, -74}}, lineColor = {0, 0, 0}, textString = "rad/s")}),
            Documentation(info = "<html>
<p>
This block converts the input signal from revolutions per minute to radian per second and returns
the result as output signal.
</p>
</html>"));
        end From_rpm;

        block To_kmh "Convert from metre per second to kilometre per hour"
          extends Modelica.Blocks.Interfaces.PartialConversionBlock(u(unit = "m/s"), y(unit = "km/h"));
        equation
          y = SI.Conversions.to_kmh(u);
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{0, 82}, {-96, 42}}, lineColor = {0, 0, 0}, textString = "m/s"), Text(extent = {{92, -40}, {-14, -84}}, lineColor = {0, 0, 0}, textString = "km/h")}),
            Documentation(info = "<html>
<p>
This block converts the input signal from metre per second to kilometre per hour and returns
the result as output signal.
</p>
</html>"));
        end To_kmh;

        block From_kmh "Convert from kilometre per hour to metre per second"
          extends Modelica.Blocks.Interfaces.PartialConversionBlock(u(unit = "km/h"), y(unit = "m/s"));
        equation
          y = SI.Conversions.from_kmh(u);
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{26, 80}, {-96, 48}}, lineColor = {0, 0, 0}, textString = "km/h"), Text(extent = {{92, -46}, {-20, -82}}, lineColor = {0, 0, 0}, textString = "m/s")}),
            Documentation(info = "<html>
<p>
This block converts the input signal from kilometre per hour to metre per second and returns
the result as output signal.
</p>
</html>"));
        end From_kmh;

        block To_day "Convert from second to day"
          extends Modelica.Blocks.Interfaces.PartialConversionBlock(u(unit = "s"), y(unit = "d"));
        equation
          y = SI.Conversions.to_day(u);
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-20, 100}, {-100, 20}}, lineColor = {0, 0, 0}, textString = "s"), Text(extent = {{100, -20}, {20, -100}}, lineColor = {0, 0, 0}, textString = "day")}),
            Documentation(info = "<html>
<p>
This block converts the input signal from second to day and returns
the result as output signal.
</p>
</html>"));
        end To_day;

        block From_day "Convert from day to second"
          extends Modelica.Blocks.Interfaces.PartialConversionBlock(u(unit = "d"), y(unit = "s"));
        equation
          y = SI.Conversions.from_day(u);
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-20, 100}, {-100, 20}}, lineColor = {0, 0, 0}, textString = "day"), Text(extent = {{100, -20}, {20, -100}}, lineColor = {0, 0, 0}, textString = "s")}),
            Documentation(info = "<html>
<p>
This block converts the input signal from day to second and returns
the result as output signal.
</p>
</html>"));
        end From_day;

        block To_hour "Convert from second to hour"
          extends Modelica.Blocks.Interfaces.PartialConversionBlock(u(unit = "s"), y(unit = "h"));
        equation
          y = SI.Conversions.to_hour(u);
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-20, 100}, {-100, 20}}, lineColor = {0, 0, 0}, textString = "s"), Text(extent = {{100, -20}, {20, -100}}, lineColor = {0, 0, 0}, textString = "hour")}),
            Documentation(info = "<html>
<p>
This block converts the input signal from second to hour and returns
the result as output signal.
</p>
</html>"));
        end To_hour;

        block From_hour "Convert from hour to second"
          extends Modelica.Blocks.Interfaces.PartialConversionBlock(u(unit = "h"), y(unit = "s"));
        equation
          y = SI.Conversions.from_hour(u);
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-20, 100}, {-100, 20}}, lineColor = {0, 0, 0}, textString = "hour"), Text(extent = {{100, -20}, {20, -100}}, lineColor = {0, 0, 0}, textString = "s")}),
            Documentation(info = "<html>
<p>
This block converts the input signal from hour to second and returns
the result as output signal.
</p>
</html>"));
        end From_hour;

        block To_minute "Convert from second to minute"
          extends Modelica.Blocks.Interfaces.PartialConversionBlock(u(unit = "s"), y(unit = "min"));
        equation
          y = SI.Conversions.to_minute(u);
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-20, 100}, {-100, 20}}, lineColor = {0, 0, 0}, textString = "s"), Text(extent = {{100, -20}, {20, -100}}, lineColor = {0, 0, 0}, textString = "minute")}),
            Documentation(info = "<html>
<p>
This block converts the input signal from second to minute and returns
the result as output signal.
</p>
</html>"));
        end To_minute;

        block From_minute "Convert from minute to second"
          extends Modelica.Blocks.Interfaces.PartialConversionBlock(u(unit = "min"), y(unit = "s"));
        equation
          y = SI.Conversions.from_minute(u);
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-20, 100}, {-100, 20}}, lineColor = {0, 0, 0}, textString = "minute"), Text(extent = {{100, -20}, {20, -100}}, lineColor = {0, 0, 0}, textString = "s")}),
            Documentation(info = "<html>
<p>
This block converts the input signal from minute to second and returns
the result as output signal.
</p>
</html>"));
        end From_minute;

        block To_litre "Convert from cubic metre to litre"
          extends Modelica.Blocks.Interfaces.PartialConversionBlock(u(unit = "m3"), y(unit = "l"));
        equation
          y = SI.Conversions.to_litre(u);
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-20, 100}, {-100, 20}}, lineColor = {0, 0, 0}, textString = "m3"), Text(extent = {{100, -20}, {20, -100}}, lineColor = {0, 0, 0}, textString = "litre")}),
            Documentation(info = "<html>
<p>
This block converts the input signal from metre to litre and returns
the result as output signal.
</p>
</html>"));
        end To_litre;

        block From_litre "Convert from litre to cubic metre"
          extends Modelica.Blocks.Interfaces.PartialConversionBlock(u(unit = "l"), y(unit = "m3"));
        equation
          y = SI.Conversions.from_litre(u);
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-20, 100}, {-100, 20}}, lineColor = {0, 0, 0}, textString = "litre"), Text(extent = {{100, -20}, {20, -100}}, lineColor = {0, 0, 0}, textString = "m3")}),
            Documentation(info = "<html>
<p>
This block converts the input signal from litre to cubic metre and returns
the result as output signal.
</p>
</html>"));
        end From_litre;

        block To_kWh "Convert from Joule to kilo Watt hour"
          extends Modelica.Blocks.Interfaces.PartialConversionBlock(u(unit = "J"), y(unit = "kW.h"));
        equation
          y = SI.Conversions.to_kWh(u);
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-20, 100}, {-100, 20}}, lineColor = {0, 0, 0}, textString = "J"), Text(extent = {{100, -20}, {20, -100}}, lineColor = {0, 0, 0}, textString = "kW.h")}),
            Documentation(info = "<html>
<p>
This block converts the input signal from Joule to kilo Watt hour and returns
the result as output signal.
</p>
</html>"));
        end To_kWh;

        block From_kWh "Convert from kilo Watt hour to Joule"
          extends Modelica.Blocks.Interfaces.PartialConversionBlock(u(unit = "kW.h"), y(unit = "J"));
        equation
          y = SI.Conversions.from_kWh(u);
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-20, 100}, {-100, 20}}, lineColor = {0, 0, 0}, textString = "kW.h"), Text(extent = {{100, -20}, {20, -100}}, lineColor = {0, 0, 0}, textString = "J")}),
            Documentation(info = "<html>
<p>
This block converts the input signal from kilo Watt hour to Joule and returns
the result as output signal.
</p>
</html>"));
        end From_kWh;

        block To_bar "Convert from Pascal to bar"
          extends Modelica.Blocks.Interfaces.PartialConversionBlock(u(unit = "Pa"), y(unit = "bar"));
        equation
          y = SI.Conversions.to_bar(u);
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-20, 100}, {-100, 20}}, lineColor = {0, 0, 0}, textString = "Pa"), Text(extent = {{100, -20}, {20, -100}}, lineColor = {0, 0, 0}, textString = "bar")}),
            Documentation(info = "<html>
<p>
This block converts the input signal from Pascal to bar and returns
the result as output signal.
</p>
</html>"));
        end To_bar;

        block From_bar "Convert from bar to Pascal"
          extends Modelica.Blocks.Interfaces.PartialConversionBlock(u(unit = "bar"), y(unit = "Pa"));
        equation
          y = SI.Conversions.from_bar(u);
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-20, 100}, {-100, 20}}, lineColor = {0, 0, 0}, textString = "bar"), Text(extent = {{100, -20}, {20, -100}}, lineColor = {0, 0, 0}, textString = "Pa")}),
            Documentation(info = "<html>
<p>
This block converts the input signal from bar to Pascal and returns
the result as output signal.
</p>
</html>"));
        end From_bar;

        block To_gps "Convert from kilogram per second to gram per second"
          extends Modelica.Blocks.Interfaces.PartialConversionBlock(u(unit = "kg/s"), y(unit = "g/s"));
        equation
          y = SI.Conversions.to_gps(u);
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-20, 100}, {-100, 20}}, lineColor = {0, 0, 0}, textString = "kg/s"), Text(extent = {{100, -20}, {20, -100}}, lineColor = {0, 0, 0}, textString = "g/s")}),
            Documentation(info = "<html>
<p>
This block converts the input signal from kilogram per second to gram per seconds and returns
the result as output signal.
</p>
</html>"));
        end To_gps;

        block From_gps "Convert from gram per second to kilogram per second"
          extends Modelica.Blocks.Interfaces.PartialConversionBlock(u(unit = "g/s"), y(unit = "kg/s"));
        equation
          y = SI.Conversions.from_gps(u);
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-20, 100}, {-100, 20}}, lineColor = {0, 0, 0}, textString = "g/s"), Text(extent = {{100, -20}, {20, -100}}, lineColor = {0, 0, 0}, textString = "kg/s")}),
            Documentation(info = "<html>
<p>
This block converts the input signal from gram per second to kilogram per second and returns
the result as output signal.
</p>
</html>"));
        end From_gps;
        annotation(
          Documentation(info = "<html>
<p>
This package consists of blocks that convert an input signal
with a specific unit to an output signal in another unit
(e.g., conversion of an angle signal from \"deg\" to \"rad\").
</p>

</html>"));
      end UnitConversions;

      block InverseBlockConstraints "Construct inverse model by requiring that two inputs and two outputs are identical (replaces the previously, unbalanced, TwoInputs and TwoOutputs blocks)"
        Modelica.Blocks.Interfaces.RealInput u1 "Input signal 1 (u1 = u2)" annotation(
          Placement(transformation(extent = {{-240, -20}, {-200, 20}}, rotation = 0), iconTransformation(extent = {{-240, -20}, {-200, 20}})));
        Modelica.Blocks.Interfaces.RealInput u2 "Input signal 2 (u1 = u2)" annotation(
          Placement(transformation(extent = {{-140, -20}, {-180, 20}}, rotation = 0), iconTransformation(extent = {{-140, -20}, {-180, 20}})));
        Modelica.Blocks.Interfaces.RealOutput y1 "Output signal 1 (y1 = y2)" annotation(
          Placement(transformation(extent = {{200, -10}, {220, 10}}, rotation = 0), iconTransformation(extent = {{200, -10}, {220, 10}})));
        Modelica.Blocks.Interfaces.RealOutput y2 "Output signal 2 (y2 = y2)" annotation(
          Placement(transformation(extent = {{10, -10}, {-10, 10}}, rotation = 0, origin = {170, 0}), iconTransformation(extent = {{180, -10}, {160, 10}})));
      equation
        u1 = u2;
        y1 = y2;
        annotation(
          __Dymola_structurallyIncomplete = true,
          defaultConnectionStructurallyInconsistent = true,
          Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-200, -120}, {200, 120}}), graphics = {Line(points = {{180, 0}, {200, 0}}, color = {0, 0, 127}, smooth = Smooth.None), Line(points = {{-200, 0}, {-180, 0}}, color = {0, 0, 127}, smooth = Smooth.None), Rectangle(extent = {{-190, 120}, {190, -120}}, lineColor = {135, 135, 135})}),
          Documentation(info = "<html>
<p>
Exchange input and output signals of a block, i.e., the previous
block inputs become block outputs and the previous block outputs become
block inputs. This block is used to construct inverse models.
Its usage is demonstrated in example:
<a href=\"modelica://Modelica.Blocks.Examples.InverseModel\">Modelica.Blocks.Examples.InverseModel</a>.
</p>

<p>
Note, if a block shall be inverted that has several input and output blocks,
then this can be easily achieved by using a vector of InverseBlockConstraints
instances:
</p>

<pre>
   InverseBlockConstraint invert[3];  // Block to be inverted has 3 input signals
</pre>
</html>"));
      end InverseBlockConstraints;

      block Gain "Output the product of a gain value with the input signal"
        parameter Real k(start = 1, unit = "1") "Gain value multiplied with input signal";
      public
        Interfaces.RealInput u "Input signal connector" annotation(
          Placement(transformation(extent = {{-140, -20}, {-100, 20}}, rotation = 0)));
        Interfaces.RealOutput y "Output signal connector" annotation(
          Placement(transformation(extent = {{100, -10}, {120, 10}}, rotation = 0)));
      equation
        y = k * u;
        annotation(
          Documentation(info = "<html>
<p>
This block computes output <i>y</i> as
<i>product</i> of gain <i>k</i> with the
input <i>u</i>:
</p>
<pre>
    y = k * u;
</pre>

</html>"),
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{-100, -100}, {-100, 100}, {100, 0}, {-100, -100}}, lineColor = {0, 0, 127}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-150, -140}, {150, -100}}, lineColor = {0, 0, 0}, textString = "k=%k"), Text(extent = {{-150, 140}, {150, 100}}, textString = "%name", lineColor = {0, 0, 255})}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{-100, -100}, {-100, 100}, {100, 0}, {-100, -100}}, lineColor = {0, 0, 127}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-76, 38}, {0, -34}}, textString = "k", lineColor = {0, 0, 255})}));
      end Gain;

      block MatrixGain "Output the product of a gain matrix with the input signal vector"
        parameter Real K[:, :] = [1, 0; 0, 1] "Gain matrix which is multiplied with the input";
        extends Interfaces.MIMO(final nin = size(K, 2), final nout = size(K, 1));
      equation
        y = K * u;
        annotation(
          Documentation(info = "<html>
<p>
This blocks computes output vector <b>y</b> as <i>product</i> of the
gain matrix <b>K</b> with the input signal vector <b>u</b>:
</p>
<pre>
    <b>y</b> = <b>K</b> * <b>u</b>;
</pre>
<p>
Example:
</p>
<pre>
   parameter: <b>K</b> = [0.12 2; 3 1.5]

   results in the following equations:

     | y[1] |     | 0.12  2.00 |   | u[1] |
     |      |  =  |            | * |      |
     | y[2] |     | 3.00  1.50 |   | u[2] |
</pre>

</html>"),
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-90, -60}, {90, 60}}, lineColor = {160, 160, 164}, textString = "*K"), Text(extent = {{-150, 150}, {150, 110}}, textString = "%name", lineColor = {0, 0, 255})}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, -100}, {100, 100}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-90, -60}, {90, 60}}, lineColor = {160, 160, 164}, textString = "*K")}));
      end MatrixGain;

      block MultiSum "Sum of Reals: y = k[1]*u[1] + k[2]*u[2] + ... + k[n]*u[n]"
        extends Modelica.Blocks.Interfaces.PartialRealMISO;
        parameter Real k[nu] = fill(1, nu) "Input gains";
      equation
        if size(u, 1) > 0 then
          y = k * u;
        else
          y = 0;
        end if;
        annotation(
          Icon(graphics = {Text(extent = {{-200, -110}, {200, -140}}, lineColor = {0, 0, 0}, fillColor = {255, 213, 170}, fillPattern = FillPattern.Solid, textString = "%k"), Text(extent = {{-72, 68}, {92, -68}}, lineColor = {0, 0, 0}, fillColor = {255, 213, 170}, fillPattern = FillPattern.Solid, textString = "+")}),
          Documentation(info = "<html>
<p>
This blocks computes the scalar Real output \"y\" as sum of the elements of the
Real input signal vector u:
</p>
<blockquote><pre>
y = k[1]*u[1] + k[2]*u[2] + ... k[N]*u[N];
</pre></blockquote>

<p>
The input connector is a vector of Real input signals.
When a connection line is drawn, the dimension of the input
vector is enlarged by one and the connection is automatically
connected to this new free index (thanks to the
connectorSizing annotation).
</p>

<p>
The usage is demonstrated, e.g., in example
<a href=\"modelica://Modelica.Blocks.Examples.RealNetwork1\">Modelica.Blocks.Examples.RealNetwork1</a>.
</p>

<p>
If no connection to the input connector \"u\" is present,
the output is set to zero: y=0.
</p>

</html>"));
      end MultiSum;

      block MultiProduct "Product of Reals: y = u[1]*u[2]* ... *u[n]"
        extends Modelica.Blocks.Interfaces.PartialRealMISO;
      equation
        if size(u, 1) > 0 then
          y = product(u);
        else
          y = 0;
        end if;
        annotation(
          Icon(graphics = {Text(extent = {{-74, 50}, {94, -94}}, lineColor = {0, 0, 0}, fillColor = {255, 213, 170}, fillPattern = FillPattern.Solid, textString = "*")}),
          Documentation(info = "<html>
<p>
This blocks computes the scalar Real output \"y\" as product of the elements of the
Real input signal vector u:
</p>
<blockquote><pre>
y = u[1]*u[2]* ... *u[N];
</pre></blockquote>

<p>
The input connector is a vector of Real input signals.
When a connection line is drawn, the dimension of the input
vector is enlarged by one and the connection is automatically
connected to this new free index (thanks to the
connectorSizing annotation).
</p>

<p>
The usage is demonstrated, e.g., in example
<a href=\"modelica://Modelica.Blocks.Examples.RealNetwork1\">Modelica.Blocks.Examples.RealNetwork1</a>.
</p>

<p>
If no connection to the input connector \"u\" is present,
the output is set to zero: y=0.
</p>
</html>"));
      end MultiProduct;

      block MultiSwitch "Set Real expression that is associated with the first active input signal"
        input Real expr[nu] = fill(0.0, nu) "y = if u[i] then expr[i] else y_default (time varying)" annotation(
          Dialog);
        parameter Real y_default = 0.0 "Default value of output y if all u[i] = false";
        parameter Integer nu(min = 0) = 0 "Number of input connections" annotation(
          Dialog(connectorSizing = true),
          HideResult = true);
        parameter Integer precision(min = 0) = 3 "Number of significant digits to be shown in dynamic diagram layer for y" annotation(
          Dialog(tab = "Advanced"));
        Modelica.Blocks.Interfaces.BooleanVectorInput u[nu] "Set y = expr[i], if u[i] = true" annotation(
          Placement(transformation(extent = {{-110, 30}, {-90, -30}})));
        Modelica.Blocks.Interfaces.RealOutput y "Output depending on expression" annotation(
          Placement(transformation(extent = {{300, -10}, {320, 10}})));
      protected
        Integer firstActiveIndex;
      initial equation
        pre(u) = fill(false, nu);
      equation
        firstActiveIndex = Modelica.Math.BooleanVectors.firstTrueIndex(u);
        y = if firstActiveIndex == 0 then y_default else expr[firstActiveIndex];
        annotation(
          defaultComponentName = "multiSwitch1",
          Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {300, 100}}), graphics = {Rectangle(extent = {{-100, -51}, {300, 50}}, lineThickness = 5.0, lineColor = {0, 0, 0}, fillColor = {170, 213, 255}, fillPattern = FillPattern.Solid, borderPattern = BorderPattern.Raised), Text(extent = {{-86, 16}, {295, -17}}, lineColor = {0, 0, 0}, fillColor = {255, 246, 238}, fillPattern = FillPattern.Solid, textString = "%expr"), Text(extent = {{310, -25}, {410, -45}}, lineColor = {0, 0, 0}, textString = DynamicSelect(" ", String(y, minimumLength = 1, significantDigits = precision))), Text(extent = {{-100, -60}, {300, -90}}, lineColor = {0, 0, 0}, textString = "else: %y_default"), Text(extent = {{-100, 100}, {300, 60}}, textString = "%name", lineColor = {0, 0, 255})}),
          Documentation(info = "<html>
<p>
This block has a vector of Boolean input signals u[nu] and a vector of
(time varying) Real expressions expr[nu]. The output signal y is
set to expr[i], if i is the first element in the input vector u that is true. If all input signals are
false, y is set to parameter \"y_default\".
</p>

<blockquote><pre>
  // Conceptual equation (not valid Modelica)
  i = 'first element of u[:] that is true';
  y = <b>if</b> i==0 <b>then</b> y_default <b>else</b> expr[i];
</pre></blockquote>

<p>
The input connector is a vector of Boolean input signals.
When a connection line is drawn, the dimension of the input
vector is enlarged by one and the connection is automatically
connected to this new free index (thanks to the
connectorSizing annotation).
</p>

<p>
The usage is demonstrated, e.g., in example
<a href=\"modelica://Modelica.Blocks.Examples.RealNetwork1\">Modelica.Blocks.Examples.RealNetwork1</a>.
</p>

</html>"));
      end MultiSwitch;

      block Sum "Output the sum of the elements of the input vector"
        extends Interfaces.MISO;
        parameter Real k[nin] = ones(nin) "Optional: sum coefficients";
      equation
        y = k * u;
        annotation(
          defaultComponentName = "sum1",
          Documentation(info = "<html>
<p>
This blocks computes output <b>y</b> as
<i>sum</i> of the elements of the input signal vector
<b>u</b>:
</p>
<pre>
    <b>y</b> = <b>u</b>[1] + <b>u</b>[2] + ...;
</pre>
<p>
Example:
</p>
<pre>
     parameter:   nin = 3;

  results in the following equations:

     y = u[1] + u[2] + u[3];
</pre>

</html>"),
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{26, 42}, {-34, 42}, {6, 2}, {-34, -38}, {26, -38}}, color = {0, 0, 0}, thickness = 0.25), Text(extent = {{-150, 150}, {150, 110}}, textString = "%name", lineColor = {0, 0, 255})}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, -100}, {100, 100}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{26, 42}, {-34, 42}, {6, 2}, {-34, -38}, {26, -38}}, color = {0, 0, 0}, thickness = 0.25)}));
      end Sum;

      block Feedback "Output difference between commanded and feedback input"
        Interfaces.RealInput u1 annotation(
          Placement(transformation(extent = {{-100, -20}, {-60, 20}}, rotation = 0)));
        Interfaces.RealInput u2 annotation(
          Placement(transformation(origin = {0, -80}, extent = {{-20, -20}, {20, 20}}, rotation = 90)));
        Interfaces.RealOutput y annotation(
          Placement(transformation(extent = {{80, -10}, {100, 10}}, rotation = 0)));
      equation
        y = u1 - u2;
        annotation(
          Documentation(info = "<html>
<p>
This blocks computes output <b>y</b> as <i>difference</i> of the
commanded input <b>u1</b> and the feedback
input <b>u2</b>:
</p>
<pre>
    <b>y</b> = <b>u1</b> - <b>u2</b>;
</pre>
<p>
Example:
</p>
<pre>
     parameter:   n = 2

  results in the following equations:

     y = u1 - u2
</pre>

</html>"),
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}, initialScale = 0.1), graphics = {Ellipse(lineColor = {0, 0, 127}, fillColor = {235, 235, 235}, fillPattern = FillPattern.Solid, extent = {{-20, -20}, {20, 20}}), Line(points = {{-60, 0}, {-20, 0}}, color = {0, 0, 127}), Line(points = {{20, 0}, {80, 0}}, color = {0, 0, 127}), Line(points = {{0, -20}, {0, -60}}, color = {0, 0, 127}), Text(extent = {{-14, -94}, {82, 0}}, textString = "-"), Text(lineColor = {0, 0, 255}, extent = {{-150, 44}, {150, 94}}, textString = "%name")}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-20, 20}, {20, -20}}, pattern = LinePattern.Solid, lineThickness = 0.25, fillColor = {235, 235, 235}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Line(points = {{-60, 0}, {-20, 0}}, color = {0, 0, 255}), Line(points = {{20, 0}, {80, 0}}, color = {0, 0, 255}), Line(points = {{0, -20}, {0, -60}}, color = {0, 0, 255}), Text(extent = {{-12, 10}, {84, -84}}, lineColor = {0, 0, 0}, textString = "-")}));
      end Feedback;

      block Add "Output the sum of the two inputs"
        extends Interfaces.SI2SO;
        parameter Real k1 = +1 "Gain of upper input";
        parameter Real k2 = +1 "Gain of lower input";
      equation
        y = k1 * u1 + k2 * u2;
        annotation(
          Documentation(info = "<html>
<p>
This blocks computes output <b>y</b> as <i>sum</i> of the
two input signals <b>u1</b> and <b>u2</b>:
</p>
<pre>
    <b>y</b> = k1*<b>u1</b> + k2*<b>u2</b>;
</pre>
<p>
Example:
</p>
<pre>
     parameter:   k1= +2, k2= -3

  results in the following equations:

     y = 2 * u1 - 3 * u2
</pre>

</html>"),
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}, initialScale = 0.1), graphics = {Text(lineColor = {0, 0, 255}, extent = {{-150, 110}, {150, 150}}, textString = "%name"), Line(points = {{-100, 60}, {-74, 24}, {-44, 24}}, color = {0, 0, 127}), Line(points = {{-100, -60}, {-74, -28}, {-42, -28}}, color = {0, 0, 127}), Ellipse(lineColor = {0, 0, 127}, extent = {{-50, -50}, {50, 50}}), Line(points = {{50, 0}, {100, 0}}, color = {0, 0, 127}), Text(extent = {{-38, -34}, {38, 34}}, textString = "+"), Text(extent = {{-100, 52}, {5, 92}}, textString = "%k1"), Text(extent = {{-100, -92}, {5, -52}}, textString = "%k2")}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, -100}, {100, 100}}, lineColor = {0, 0, 127}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{50, 0}, {100, 0}}, color = {0, 0, 255}), Line(points = {{-100, 60}, {-74, 24}, {-44, 24}}, color = {0, 0, 127}), Line(points = {{-100, -60}, {-74, -28}, {-42, -28}}, color = {0, 0, 127}), Ellipse(extent = {{-50, 50}, {50, -50}}, lineColor = {0, 0, 127}), Line(points = {{50, 0}, {100, 0}}, color = {0, 0, 127}), Text(extent = {{-36, 38}, {40, -30}}, lineColor = {0, 0, 0}, textString = "+"), Text(extent = {{-100, 52}, {5, 92}}, lineColor = {0, 0, 0}, textString = "k1"), Text(extent = {{-100, -52}, {5, -92}}, lineColor = {0, 0, 0}, textString = "k2")}));
      end Add;

      block Add3 "Output the sum of the three inputs"
        extends Modelica.Blocks.Icons.Block;
        parameter Real k1 = +1 "Gain of upper input";
        parameter Real k2 = +1 "Gain of middle input";
        parameter Real k3 = +1 "Gain of lower input";
        Interfaces.RealInput u1 "Connector 1 of Real input signals" annotation(
          Placement(transformation(extent = {{-140, 60}, {-100, 100}}, rotation = 0)));
        Interfaces.RealInput u2 "Connector 2 of Real input signals" annotation(
          Placement(transformation(extent = {{-140, -20}, {-100, 20}}, rotation = 0)));
        Interfaces.RealInput u3 "Connector 3 of Real input signals" annotation(
          Placement(transformation(extent = {{-140, -100}, {-100, -60}}, rotation = 0)));
        Interfaces.RealOutput y "Connector of Real output signals" annotation(
          Placement(transformation(extent = {{100, -10}, {120, 10}}, rotation = 0)));
      equation
        y = k1 * u1 + k2 * u2 + k3 * u3;
        annotation(
          Documentation(info = "<html>
<p>
This blocks computes output <b>y</b> as <i>sum</i> of the
three input signals <b>u1</b>, <b>u2</b> and <b>u3</b>:
</p>
<pre>
    <b>y</b> = k1*<b>u1</b> + k2*<b>u2</b> + k3*<b>u3</b>;
</pre>
<p>
Example:
</p>
<pre>
     parameter:   k1= +2, k2= -3, k3=1;

  results in the following equations:

     y = 2 * u1 - 3 * u2 + u3;
</pre>

</html>"),
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-100, 50}, {5, 90}}, lineColor = {0, 0, 0}, textString = "%k1"), Text(extent = {{-100, -20}, {5, 20}}, lineColor = {0, 0, 0}, textString = "%k2"), Text(extent = {{-100, -50}, {5, -90}}, lineColor = {0, 0, 0}, textString = "%k3"), Text(extent = {{2, 36}, {100, -44}}, lineColor = {0, 0, 0}, textString = "+")}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, -100}, {100, 100}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-100, 50}, {5, 90}}, lineColor = {0, 0, 0}, textString = "k1"), Text(extent = {{-100, -20}, {5, 20}}, lineColor = {0, 0, 0}, textString = "k2"), Text(extent = {{-100, -50}, {5, -90}}, lineColor = {0, 0, 0}, textString = "k3"), Text(extent = {{2, 46}, {100, -34}}, lineColor = {0, 0, 0}, textString = "+")}));
      end Add3;

      block Product "Output product of the two inputs"
        extends Interfaces.SI2SO;
      equation
        y = u1 * u2;
        annotation(
          Documentation(info = "<html>
<p>
This blocks computes the output <b>y</b> (element-wise)
as <i>product</i> of the corresponding elements of
the two inputs <b>u1</b> and <b>u2</b>:
</p>
<pre>
    y = u1 * u2;
</pre>

</html>"),
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}, initialScale = 0.1), graphics = {Line(points = {{-100, 60}, {-40, 60}, {-30, 40}}, color = {0, 0, 127}), Line(points = {{-100, -60}, {-40, -60}, {-30, -40}}, color = {0, 0, 127}), Line(points = {{50, 0}, {100, 0}}, color = {0, 0, 127}), Line(points = {{-30, 0}, {30, 0}}), Line(points = {{-15, 25.99}, {15, -25.99}}), Line(points = {{-15, -25.99}, {15, 25.99}}), Ellipse(lineColor = {0, 0, 127}, extent = {{-50, -50}, {50, 50}})}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, -100}, {100, 100}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{-100, 60}, {-40, 60}, {-30, 40}}, color = {0, 0, 255}), Line(points = {{-100, -60}, {-40, -60}, {-30, -40}}, color = {0, 0, 255}), Line(points = {{50, 0}, {100, 0}}, color = {0, 0, 255}), Line(points = {{-30, 0}, {30, 0}}, color = {0, 0, 0}), Line(points = {{-15, 25.99}, {15, -25.99}}, color = {0, 0, 0}), Line(points = {{-15, -25.99}, {15, 25.99}}, color = {0, 0, 0}), Ellipse(extent = {{-50, 50}, {50, -50}}, lineColor = {0, 0, 255})}));
      end Product;

      block Division "Output first input divided by second input"
        extends Interfaces.SI2SO;
      equation
        y = u1 / u2;
        annotation(
          Documentation(info = "<html>
<p>
This block computes the output <b>y</b> (element-wise)
by <i>dividing</i> the corresponding elements of
the two inputs <b>u1</b> and <b>u2</b>:
</p>
<pre>
    y = u1 / u2;
</pre>

</html>"),
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}, initialScale = 0.1), graphics = {Line(points = {{50, 0}, {100, 0}}, color = {0, 0, 127}), Line(points = {{-30, 0}, {30, 0}}), Ellipse(fillPattern = FillPattern.Solid, extent = {{-5, 20}, {5, 30}}), Ellipse(fillPattern = FillPattern.Solid, extent = {{-5, -30}, {5, -20}}), Ellipse(lineColor = {0, 0, 127}, extent = {{-50, -50}, {50, 50}}), Text(lineColor = {0, 0, 255}, extent = {{-150, 110}, {150, 150}}, textString = "%name"), Line(points = {{-100, 60}, {-66, 60}, {-40, 30}}, color = {0, 0, 127}), Line(points = {{-100, -60}, {0, -60}, {0, -50}}, color = {0, 0, 127})}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, -100}, {100, 100}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{50, 0}, {100, 0}}, color = {0, 0, 255}), Line(points = {{-30, 0}, {30, 0}}, color = {0, 0, 0}), Ellipse(extent = {{-5, 20}, {5, 30}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-5, -20}, {5, -30}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-50, 50}, {50, -50}}, lineColor = {0, 0, 255}), Line(points = {{-100, 60}, {-66, 60}, {-40, 30}}, color = {0, 0, 255}), Line(points = {{-100, -60}, {0, -60}, {0, -50}}, color = {0, 0, 255})}));
      end Division;

      block Abs "Output the absolute value of the input"
        extends Interfaces.SISO;
        parameter Boolean generateEvent = false "Choose whether events shall be generated" annotation(
          Evaluate = true);
      equation
//y = abs(u);
        y = if generateEvent then if u >= 0 then u else -u else if noEvent(u >= 0) then u else -u;
        annotation(
          defaultComponentName = "abs1",
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{92, 0}, {70, 8}, {70, -8}, {92, 0}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-80, 80}, {0, 0}, {80, 80}}, color = {0, 0, 0}), Line(points = {{0, -14}, {0, 68}}, color = {192, 192, 192}), Polygon(points = {{0, 90}, {-8, 68}, {8, 68}, {0, 90}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Text(extent = {{-34, -28}, {38, -76}}, lineColor = {192, 192, 192}, textString = "abs"), Line(points = {{-88, 0}, {76, 0}}, color = {192, 192, 192})}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-100, 0}, {76, 0}}, color = {192, 192, 192}), Polygon(points = {{92, 0}, {76, 6}, {76, -6}, {92, 0}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-80, 80}, {0, 0}, {80, 80}}, color = {0, 0, 0}), Line(points = {{0, -80}, {0, 68}}, color = {192, 192, 192}), Polygon(points = {{0, 90}, {-8, 68}, {8, 68}, {0, 90}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{0, -80}, {0, 68}}, color = {192, 192, 192}), Polygon(points = {{0, 90}, {-8, 68}, {8, 68}, {0, 90}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Text(extent = {{7, 98}, {34, 78}}, lineColor = {160, 160, 164}, textString = "u"), Text(extent = {{74, -8}, {96, -28}}, lineColor = {160, 160, 164}, textString = "y"), Text(extent = {{52, -3}, {72, -23}}, textString = "1", lineColor = {0, 0, 255}), Text(extent = {{-86, -1}, {-66, -21}}, textString = "-1", lineColor = {0, 0, 255}), Text(extent = {{-28, 79}, {-8, 59}}, textString = "1", lineColor = {0, 0, 255})}),
          Documentation(info = "<HTML>
<p>
This blocks computes the output <b>y</b>
as <i>absolute value</i> of the input <b>u</b>:
</p>
<pre>
    y = <b>abs</b>( u );
</pre>
<p>
The Boolean parameter generateEvent decides whether Events are generated at zero crossing (Modelica specification before 3) or not.
</p>
</html>"));
      end Abs;

      block Sign "Output the sign of the input"
        extends Interfaces.SISO;
        parameter Boolean generateEvent = false "Choose whether events shall be generated" annotation(
          Evaluate = true);
      equation
//y = sign(u);
        y = if generateEvent then if u > 0 then 1 elseif u < 0 then -1 else 0 else if noEvent(u > 0) then 1 elseif noEvent(u < 0) then -1 else 0;
        annotation(
          defaultComponentName = "sign1",
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-90, 0}, {68, 0}}, color = {192, 192, 192}), Polygon(points = {{90, 0}, {68, 8}, {68, -8}, {90, 0}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-80, -80}, {0, -80}}, color = {0, 0, 0}), Polygon(points = {{0, 90}, {-8, 68}, {8, 68}, {0, 90}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{0, -80}, {0, 68}}, color = {192, 192, 192}), Text(extent = {{-90, 72}, {-18, 24}}, lineColor = {192, 192, 192}, textString = "sign"), Line(points = {{0, 80}, {80, 80}}, color = {0, 0, 0}), Rectangle(extent = {{-2, 2}, {2, -4}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid)}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-90, 0}, {68, 0}}, color = {192, 192, 192}), Polygon(points = {{90, 0}, {68, 8}, {68, -8}, {90, 0}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-80, -80}, {0, -80}}, color = {0, 0, 0}), Line(points = {{-0.01, 0}, {0.01, 0}}, color = {0, 0, 0}), Line(points = {{0, 80}, {80, 80}}, color = {0, 0, 0}), Rectangle(extent = {{-2, 2}, {2, -4}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Polygon(points = {{0, 100}, {-6, 84}, {6, 84}, {0, 100}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{0, -90}, {0, 84}}, color = {192, 192, 192}), Text(extent = {{7, 102}, {32, 82}}, lineColor = {160, 160, 164}, textString = "y"), Text(extent = {{70, -6}, {94, -26}}, lineColor = {160, 160, 164}, textString = "u"), Text(extent = {{-25, 86}, {-5, 70}}, textString = "1", lineColor = {0, 0, 255}), Text(extent = {{5, -72}, {25, -88}}, textString = "-1", lineColor = {0, 0, 255})}),
          Documentation(info = "<HTML>
<p>
This blocks computes the output <b>y</b>
as <b>sign</b> of the input <b>u</b>:
</p>
<pre>
         1  <b>if</b> u &gt; 0
    y =  0  <b>if</b> u == 0
        -1  <b>if</b> u &lt; 0
</pre>
<p>
The Boolean parameter generateEvent decides whether Events are generated at zero crossing (Modelica specification before 3) or not.
</p>
</html>"));
      end Sign;

      block Sqrt "Output the square root of the input (input >= 0 required)"
        extends Interfaces.SISO;
      equation
        y = sqrt(u);
        annotation(
          defaultComponentName = "sqrt1",
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-90, -80}, {68, -80}}, color = {192, 192, 192}), Polygon(points = {{90, -80}, {68, -72}, {68, -88}, {90, -80}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-80, -80}, {-79.2, -68.7}, {-78.4, -64}, {-76.8, -57.3}, {-73.6, -47.9}, {-67.9, -36.1}, {-59.1, -22.2}, {-46.2, -6.49}, {-28.5, 10.7}, {-4.42, 30}, {27.7, 51.3}, {69.5, 74.7}, {80, 80}}, color = {0, 0, 0}, smooth = Smooth.Bezier), Polygon(points = {{-80, 90}, {-88, 68}, {-72, 68}, {-80, 90}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-80, -88}, {-80, 68}}, color = {192, 192, 192}), Text(extent = {{-8, -4}, {64, -52}}, lineColor = {192, 192, 192}, textString = "sqrt")}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-92, -80}, {84, -80}}, color = {192, 192, 192}), Polygon(points = {{100, -80}, {84, -74}, {84, -86}, {100, -80}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-80, -80}, {-79.2, -68.7}, {-78.4, -64}, {-76.8, -57.3}, {-73.6, -47.9}, {-67.9, -36.1}, {-59.1, -22.2}, {-46.2, -6.49}, {-28.5, 10.7}, {-4.42, 30}, {27.7, 51.3}, {69.5, 74.7}, {80, 80}}, color = {0, 0, 0}), Polygon(points = {{-80, 98}, {-86, 82}, {-74, 82}, {-80, 98}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-80, -90}, {-80, 84}}, color = {192, 192, 192}), Text(extent = {{-71, 98}, {-44, 78}}, lineColor = {160, 160, 164}, textString = "y"), Text(extent = {{60, -52}, {84, -72}}, lineColor = {160, 160, 164}, textString = "u")}),
          Documentation(info = "<HTML>
<p>
This blocks computes the output <b>y</b>
as <i>square root</i> of the input <b>u</b>:
</p>
<pre>
    y = <b>sqrt</b>( u );
</pre>
<p>
All elements of the input vector shall be zero or positive.
Otherwise an error occurs.
</p>

</html>"));
      end Sqrt;

      block Sin "Output the sine of the input"
        extends Interfaces.SISO;
      equation
        y = Modelica.Math.sin(u);
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{-80, 90}, {-88, 68}, {-72, 68}, {-80, 90}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-80, -80}, {-80, 68}}, color = {192, 192, 192}), Line(points = {{-90, 0}, {68, 0}}, color = {192, 192, 192}), Line(points = {{-80, 0}, {-68.7, 34.2}, {-61.5, 53.1}, {-55.1, 66.4}, {-49.4, 74.6}, {-43.8, 79.1}, {-38.2, 79.8}, {-32.6, 76.6}, {-26.9, 69.7}, {-21.3, 59.4}, {-14.9, 44.1}, {-6.83, 21.2}, {10.1, -30.8}, {17.3, -50.2}, {23.7, -64.2}, {29.3, -73.1}, {35, -78.4}, {40.6, -80}, {46.2, -77.6}, {51.9, -71.5}, {57.5, -61.9}, {63.9, -47.2}, {72, -24.8}, {80, 0}}, color = {0, 0, 0}, smooth = Smooth.Bezier), Polygon(points = {{90, 0}, {68, 8}, {68, -8}, {90, 0}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Text(extent = {{12, 84}, {84, 36}}, lineColor = {192, 192, 192}, textString = "sin")}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-80, 80}, {-88, 80}}, color = {192, 192, 192}), Line(points = {{-80, -80}, {-88, -80}}, color = {192, 192, 192}), Line(points = {{-80, -90}, {-80, 84}}, color = {192, 192, 192}), Text(extent = {{-75, 98}, {-46, 78}}, lineColor = {160, 160, 164}, textString = "y"), Polygon(points = {{-80, 96}, {-86, 80}, {-74, 80}, {-80, 96}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-100, 0}, {84, 0}}, color = {192, 192, 192}), Polygon(points = {{100, 0}, {84, 6}, {84, -6}, {100, 0}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-80, 0}, {-68.7, 34.2}, {-61.5, 53.1}, {-55.1, 66.4}, {-49.4, 74.6}, {-43.8, 79.1}, {-38.2, 79.8}, {-32.6, 76.6}, {-26.9, 69.7}, {-21.3, 59.4}, {-14.9, 44.1}, {-6.83, 21.2}, {10.1, -30.8}, {17.3, -50.2}, {23.7, -64.2}, {29.3, -73.1}, {35, -78.4}, {40.6, -80}, {46.2, -77.6}, {51.9, -71.5}, {57.5, -61.9}, {63.9, -47.2}, {72, -24.8}, {80, 0}}, color = {0, 0, 0}), Text(extent = {{-105, 72}, {-85, 88}}, textString = "1", lineColor = {0, 0, 255}), Text(extent = {{70, 25}, {90, 5}}, textString = "2*pi", lineColor = {0, 0, 255}), Text(extent = {{-105, -72}, {-85, -88}}, textString = "-1", lineColor = {0, 0, 255}), Text(extent = {{76, -10}, {98, -30}}, lineColor = {160, 160, 164}, textString = "u")}),
          Documentation(info = "<HTML>
<p>
This blocks computes the output <b>y</b>
as <b>sine</b> of the input <b>u</b>:
</p>
<pre>
    y = <b>sin</b>( u );
</pre>

<p>
<img src=\"modelica://Modelica/Resources/Images/Math/sin.png\"
     alt=\"sin.png\">
</p>

</html>"));
      end Sin;

      block Cos "Output the cosine of the input"
        extends Interfaces.SISO;
      equation
        y = Modelica.Math.cos(u);
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{-80, 90}, {-88, 68}, {-72, 68}, {-80, 90}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-80, -80}, {-80, 68}}, color = {192, 192, 192}), Line(points = {{-90, 0}, {68, 0}}, color = {192, 192, 192}), Polygon(points = {{90, 0}, {68, 8}, {68, -8}, {90, 0}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-80, 80}, {-74.4, 78.1}, {-68.7, 72.3}, {-63.1, 63}, {-56.7, 48.7}, {-48.6, 26.6}, {-29.3, -32.5}, {-22.1, -51.7}, {-15.7, -65.3}, {-10.1, -73.8}, {-4.42, -78.8}, {1.21, -79.9}, {6.83, -77.1}, {12.5, -70.6}, {18.1, -60.6}, {24.5, -45.7}, {32.6, -23}, {50.3, 31.3}, {57.5, 50.7}, {63.9, 64.6}, {69.5, 73.4}, {75.2, 78.6}, {80, 80}}, color = {0, 0, 0}, smooth = Smooth.Bezier), Text(extent = {{-36, 82}, {36, 34}}, lineColor = {192, 192, 192}, textString = "cos")}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-80, 80}, {-88, 80}}, color = {192, 192, 192}), Line(points = {{-80, -80}, {-88, -80}}, color = {192, 192, 192}), Line(points = {{-80, -90}, {-80, 84}}, color = {192, 192, 192}), Text(extent = {{-75, 102}, {-48, 82}}, lineColor = {160, 160, 164}, textString = "y"), Polygon(points = {{-80, 100}, {-86, 84}, {-74, 84}, {-80, 100}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Text(extent = {{-103, 72}, {-83, 88}}, textString = "1", lineColor = {0, 0, 255}), Text(extent = {{-79, -72}, {-59, -88}}, textString = "-1", lineColor = {0, 0, 255}), Text(extent = {{70, 25}, {90, 5}}, textString = "2*pi", lineColor = {0, 0, 255}), Line(points = {{-100, 0}, {84, 0}}, color = {192, 192, 192}), Polygon(points = {{100, 0}, {84, 6}, {84, -6}, {100, 0}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-80, 80}, {-74.4, 78.1}, {-68.7, 72.3}, {-63.1, 63}, {-56.7, 48.7}, {-48.6, 26.6}, {-29.3, -32.5}, {-22.1, -51.7}, {-15.7, -65.3}, {-10.1, -73.8}, {-4.42, -78.8}, {1.21, -79.9}, {6.83, -77.1}, {12.5, -70.6}, {18.1, -60.6}, {24.5, -45.7}, {32.6, -23}, {50.3, 31.3}, {57.5, 50.7}, {63.9, 64.6}, {69.5, 73.4}, {75.2, 78.6}, {80, 80}}, color = {0, 0, 0}), Text(extent = {{74, -4}, {98, -24}}, lineColor = {160, 160, 164}, textString = "u")}),
          Documentation(info = "<HTML>
<p>
This blocks computes the output <b>y</b>
as <b>cos</b> of the input <b>u</b>:
</p>
<pre>
    y = <b>cos</b>( u );
</pre>

<p>
<img src=\"modelica://Modelica/Resources/Images/Math/cos.png\"
     alt=\"cos.png\">
</p>

</html>"));
      end Cos;

      block Tan "Output the tangent of the input"
        extends Interfaces.SISO;
      equation
        y = Modelica.Math.tan(u);
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{0, 90}, {-8, 68}, {8, 68}, {0, 90}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{0, -80}, {0, 68}}, color = {192, 192, 192}), Line(points = {{-80, -80}, {-78.4, -68.4}, {-76.8, -59.7}, {-74.4, -50}, {-71.2, -40.9}, {-67.1, -33}, {-60.7, -24.8}, {-51.1, -17.2}, {-35.8, -9.98}, {-4.42, -1.07}, {33.4, 9.12}, {49.4, 16.2}, {59.1, 23.2}, {65.5, 30.6}, {70.4, 39.1}, {73.6, 47.4}, {76, 56.1}, {77.6, 63.8}, {80, 80}}, color = {0, 0, 0}, smooth = Smooth.Bezier), Line(points = {{-90, 0}, {68, 0}}, color = {192, 192, 192}), Polygon(points = {{90, 0}, {68, 8}, {68, -8}, {90, 0}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Text(extent = {{-90, 72}, {-18, 24}}, lineColor = {192, 192, 192}, textString = "tan")}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{0, 80}, {-8, 80}}, color = {192, 192, 192}), Line(points = {{0, -80}, {-8, -80}}, color = {192, 192, 192}), Line(points = {{0, -88}, {0, 86}}, color = {192, 192, 192}), Text(extent = {{11, 100}, {38, 80}}, lineColor = {160, 160, 164}, textString = "y"), Polygon(points = {{0, 102}, {-6, 86}, {6, 86}, {0, 102}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Text(extent = {{-37, -72}, {-17, -88}}, textString = "-5.8", lineColor = {0, 0, 255}), Text(extent = {{-33, 86}, {-13, 70}}, textString = " 5.8", lineColor = {0, 0, 255}), Text(extent = {{70, 25}, {90, 5}}, textString = "1.4", lineColor = {0, 0, 255}), Line(points = {{-100, 0}, {84, 0}}, color = {192, 192, 192}), Polygon(points = {{100, 0}, {84, 6}, {84, -6}, {100, 0}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-80, -80}, {-78.4, -68.4}, {-76.8, -59.7}, {-74.4, -50}, {-71.2, -40.9}, {-67.1, -33}, {-60.7, -24.8}, {-51.1, -17.2}, {-35.8, -9.98}, {-4.42, -1.07}, {33.4, 9.12}, {49.4, 16.2}, {59.1, 23.2}, {65.5, 30.6}, {70.4, 39.1}, {73.6, 47.4}, {76, 56.1}, {77.6, 63.8}, {80, 80}}, color = {0, 0, 0}), Text(extent = {{70, -6}, {94, -26}}, lineColor = {160, 160, 164}, textString = "u")}),
          Documentation(info = "<HTML>
<p>
This blocks computes the output <b>y</b>
as <b>tan</b> of the input <b>u</b>:
</p>
<pre>
    y = <b>tan</b>( u );
</pre>

<p>
<img src=\"modelica://Modelica/Resources/Images/Math/tan.png\"
     alt=\"tan.png\">
</p>

</html>"));
      end Tan;

      block Asin "Output the arc sine of the input"
        extends Interfaces.SISO;
      equation
        y = Modelica.Math.asin(u);
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{0, 90}, {-8, 68}, {8, 68}, {0, 90}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{0, -80}, {0, 68}}, color = {192, 192, 192}), Line(points = {{-80, -80}, {-79.2, -72.8}, {-77.6, -67.5}, {-73.6, -59.4}, {-66.3, -49.8}, {-53.5, -37.3}, {-30.2, -19.7}, {37.4, 24.8}, {57.5, 40.8}, {68.7, 52.7}, {75.2, 62.2}, {77.6, 67.5}, {80, 80}}, color = {0, 0, 0}, smooth = Smooth.Bezier), Line(points = {{-90, 0}, {68, 0}}, color = {192, 192, 192}), Polygon(points = {{90, 0}, {68, 8}, {68, -8}, {90, 0}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Text(extent = {{-88, 78}, {-16, 30}}, lineColor = {192, 192, 192}, textString = "asin")}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{0, 80}, {-8, 80}}, color = {192, 192, 192}), Line(points = {{0, -80}, {-8, -80}}, color = {192, 192, 192}), Line(points = {{0, -90}, {0, 84}}, color = {192, 192, 192}), Text(extent = {{9, 102}, {36, 82}}, lineColor = {160, 160, 164}, textString = "y"), Polygon(points = {{0, 100}, {-6, 84}, {6, 84}, {0, 100}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Text(extent = {{-40, -72}, {-15, -88}}, textString = "-pi/2", lineColor = {0, 0, 255}), Text(extent = {{-38, 88}, {-13, 72}}, textString = " pi/2", lineColor = {0, 0, 255}), Text(extent = {{70, 25}, {90, 5}}, textString = "+1", lineColor = {0, 0, 255}), Text(extent = {{-90, 21}, {-70, 1}}, textString = "-1", lineColor = {0, 0, 255}), Line(points = {{-100, 0}, {84, 0}}, color = {192, 192, 192}), Polygon(points = {{100, 0}, {84, 6}, {84, -6}, {100, 0}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-80, -80}, {-79.2, -72.8}, {-77.6, -67.5}, {-73.6, -59.4}, {-66.3, -49.8}, {-53.5, -37.3}, {-30.2, -19.7}, {37.4, 24.8}, {57.5, 40.8}, {68.7, 52.7}, {75.2, 62.2}, {77.6, 67.5}, {80, 80}}, color = {0, 0, 0}), Text(extent = {{74, -4}, {100, -26}}, lineColor = {160, 160, 164}, textString = "u")}),
          Documentation(info = "<HTML>
<p>
This blocks computes the output <b>y</b> as the
<i>sine-inverse</i> of the input <b>u</b>:
</p>
<pre>
    y = <b>asin</b>( u );
</pre>
<p>
The absolute values of the elements of the input <b>u</b> need to
be less or equal to one (<b>abs</b>( u ) <= 1).
Otherwise an error occurs.
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Math/asin.png\"
     alt=\"atan.png\">
</p>

</html>"));
      end Asin;

      block Acos "Output the arc cosine of the input"
        extends Interfaces.SISO;
      equation
        y = Modelica.Math.acos(u);
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{0, 90}, {-8, 68}, {8, 68}, {0, 90}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-80, 80}, {-79.2, 72.8}, {-77.6, 67.5}, {-73.6, 59.4}, {-66.3, 49.8}, {-53.5, 37.3}, {-30.2, 19.7}, {37.4, -24.8}, {57.5, -40.8}, {68.7, -52.7}, {75.2, -62.2}, {77.6, -67.5}, {80, -80}}, color = {0, 0, 0}, smooth = Smooth.Bezier), Line(points = {{0, -88}, {0, 68}}, color = {192, 192, 192}), Line(points = {{-90, -80}, {68, -80}}, color = {192, 192, 192}), Polygon(points = {{90, -80}, {68, -72}, {68, -88}, {90, -80}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Text(extent = {{-86, -14}, {-14, -62}}, lineColor = {192, 192, 192}, textString = "acos")}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{0, 80}, {-8, 80}}, color = {192, 192, 192}), Line(points = {{0, -80}, {-8, -80}}, color = {192, 192, 192}), Line(points = {{0, -90}, {0, 84}}, color = {192, 192, 192}), Text(extent = {{13, 100}, {42, 82}}, lineColor = {160, 160, 164}, textString = "y"), Polygon(points = {{0, 100}, {-6, 84}, {6, 84}, {0, 100}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-100, -80}, {84, -80}}, color = {192, 192, 192}), Polygon(points = {{100, -80}, {84, -74}, {84, -86}, {100, -80}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-80, 80}, {-79.2, 72.8}, {-77.6, 67.5}, {-73.6, 59.4}, {-66.3, 49.8}, {-53.5, 37.3}, {-30.2, 19.7}, {37.4, -24.8}, {57.5, -40.8}, {68.7, -52.7}, {75.2, -62.2}, {77.6, -67.5}, {80, -80}}, color = {0, 0, 0}), Text(extent = {{-30, 88}, {-5, 72}}, textString = " pi", lineColor = {0, 0, 255}), Text(extent = {{-94, -57}, {-74, -77}}, textString = "-1", lineColor = {0, 0, 255}), Text(extent = {{80, -45}, {100, -65}}, textString = "+1", lineColor = {0, 0, 255}), Text(extent = {{76, -84}, {102, -102}}, lineColor = {160, 160, 164}, textString = "u")}),
          Documentation(info = "<HTML>
<p>
This blocks computes the output <b>y</b> as the
<i>cosine-inverse</i> of the input <b>u</b>:
</p>
<pre>
    y = <b>acos</b>( u );
</pre>
<p>
The absolute values of the elements of the input <b>u</b> need to
be less or equal to one (<b>abs</b>( u ) <= 1).
Otherwise an error occurs.
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Math/acos.png\"
     alt=\"acos.png\">
</p>

</html>"));
      end Acos;

      block Atan "Output the arc tangent of the input"
        extends Interfaces.SISO;
      equation
        y = Modelica.Math.atan(u);
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{0, 90}, {-8, 68}, {8, 68}, {0, 90}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{0, -80}, {0, 68}}, color = {192, 192, 192}), Line(points = {{-80, -80}, {-52.7, -75.2}, {-37.4, -69.7}, {-26.9, -63}, {-19.7, -55.2}, {-14.1, -45.8}, {-10.1, -36.4}, {-6.03, -23.9}, {-1.21, -5.06}, {5.23, 21}, {9.25, 34.1}, {13.3, 44.2}, {18.1, 52.9}, {24.5, 60.8}, {33.4, 67.6}, {47, 73.6}, {69.5, 78.6}, {80, 80}}, color = {0, 0, 0}, smooth = Smooth.Bezier), Line(points = {{-90, 0}, {68, 0}}, color = {192, 192, 192}), Polygon(points = {{90, 0}, {68, 8}, {68, -8}, {90, 0}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Text(extent = {{-86, 68}, {-14, 20}}, lineColor = {192, 192, 192}, textString = "atan")}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{0, 80}, {-8, 80}}, color = {192, 192, 192}), Line(points = {{0, -80}, {-8, -80}}, color = {192, 192, 192}), Line(points = {{0, -90}, {0, 84}}, color = {192, 192, 192}), Text(extent = {{13, 102}, {42, 82}}, lineColor = {160, 160, 164}, textString = "y"), Polygon(points = {{0, 100}, {-6, 84}, {6, 84}, {0, 100}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-100, 0}, {84, 0}}, color = {192, 192, 192}), Polygon(points = {{100, 0}, {84, 6}, {84, -6}, {100, 0}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-80, -80}, {-52.7, -75.2}, {-37.4, -69.7}, {-26.9, -63}, {-19.7, -55.2}, {-14.1, -45.8}, {-10.1, -36.4}, {-6.03, -23.9}, {-1.21, -5.06}, {5.23, 21}, {9.25, 34.1}, {13.3, 44.2}, {18.1, 52.9}, {24.5, 60.8}, {33.4, 67.6}, {47, 73.6}, {69.5, 78.6}, {80, 80}}, color = {0, 0, 0}), Text(extent = {{-32, 91}, {-12, 71}}, textString = "1.4", lineColor = {0, 0, 255}), Text(extent = {{-32, -71}, {-12, -91}}, textString = "-1.4", lineColor = {0, 0, 255}), Text(extent = {{73, 26}, {93, 10}}, textString = " 5.8", lineColor = {0, 0, 255}), Text(extent = {{-103, 20}, {-83, 4}}, textString = "-5.8", lineColor = {0, 0, 255}), Text(extent = {{66, -8}, {94, -28}}, lineColor = {160, 160, 164}, textString = "u")}),
          Documentation(info = "<HTML>
<p>
This blocks computes the output <b>y</b> as the
<i>tangent-inverse</i> of the input <b>u</b>:
</p>
<pre>
    y= <b>atan</b>( u );
</pre>

<p>
<img src=\"modelica://Modelica/Resources/Images/Math/atan.png\"
     alt=\"atan.png\">
</p>

</html>"));
      end Atan;

      block Atan2 "Output atan(u1/u2) of the inputs u1 and u2"
        extends Interfaces.SI2SO;
      equation
        y = Modelica.Math.atan2(u1, u2);
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-150, 150}, {150, 110}}, textString = "%name", lineColor = {0, 0, 255}), Polygon(points = {{0, 90}, {-8, 68}, {8, 68}, {0, 90}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{0, -80}, {0, 68}}, color = {192, 192, 192}), Line(points = {{-90, 0}, {68, 0}}, color = {192, 192, 192}), Polygon(points = {{90, 0}, {68, 8}, {68, -8}, {90, 0}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-80, -34.9}, {-46.1, -31.4}, {-29.4, -27.1}, {-18.3, -21.5}, {-10.3, -14.5}, {-2.03, -3.17}, {7.97, 11.6}, {15.5, 19.4}, {24.3, 25}, {39, 30}, {62.1, 33.5}, {80, 34.9}}, color = {0, 0, 0}, smooth = Smooth.Bezier), Line(points = {{-80, 45.1}, {-45.9, 48.7}, {-29.1, 52.9}, {-18.1, 58.6}, {-10.2, 65.8}, {-1.82, 77.2}, {0, 80}}, color = {0, 0, 0}, smooth = Smooth.Bezier), Line(points = {{0, -80}, {8.93, -67.2}, {17.1, -59.3}, {27.3, -53.6}, {42.1, -49.4}, {69.9, -45.8}, {80, -45.1}}, color = {0, 0, 0}, smooth = Smooth.Bezier), Text(extent = {{-90, -46}, {-18, -94}}, lineColor = {192, 192, 192}, textString = "atan2")}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{0, 80}, {-8, 80}}, color = {192, 192, 192}), Line(points = {{0, -80}, {-8, -80}}, color = {192, 192, 192}), Line(points = {{0, -90}, {0, 84}}, color = {192, 192, 192}), Text(extent = {{11, 98}, {42, 78}}, lineColor = {160, 160, 164}, textString = "y"), Polygon(points = {{0, 100}, {-6, 84}, {6, 84}, {0, 100}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-100, 0}, {84, 0}}, color = {192, 192, 192}), Polygon(points = {{100, 0}, {84, 6}, {84, -6}, {100, 0}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{0, -80}, {8.93, -67.2}, {17.1, -59.3}, {27.3, -53.6}, {42.1, -49.4}, {69.9, -45.8}, {80, -45.1}}, color = {0, 0, 0}), Line(points = {{-80, -34.9}, {-46.1, -31.4}, {-29.4, -27.1}, {-18.3, -21.5}, {-10.3, -14.5}, {-2.03, -3.17}, {7.97, 11.6}, {15.5, 19.4}, {24.3, 25}, {39, 30}, {62.1, 33.5}, {80, 34.9}}, color = {0, 0, 0}), Line(points = {{-80, 45.1}, {-45.9, 48.7}, {-29.1, 52.9}, {-18.1, 58.6}, {-10.2, 65.8}, {-1.82, 77.2}, {0, 80}}, color = {0, 0, 0}), Text(extent = {{-30, 89}, {-10, 70}}, textString = "pi", lineColor = {0, 0, 255}), Text(extent = {{-30, -69}, {-10, -88}}, textString = "-pi", lineColor = {0, 0, 255}), Text(extent = {{-30, 49}, {-10, 30}}, textString = "pi/2", lineColor = {0, 0, 255}), Line(points = {{0, 40}, {-8, 40}}, color = {192, 192, 192}), Line(points = {{0, -40}, {-8, -40}}, color = {192, 192, 192}), Text(extent = {{-30, -31}, {-10, -50}}, textString = "-pi/2", lineColor = {0, 0, 255}), Text(extent = {{48, 0}, {100, -34}}, lineColor = {160, 160, 164}, textString = "u1 / u2")}),
          Documentation(info = "<HTML>
<p>
This blocks computes the output <b>y</b> as the
<i>tangent-inverse</i> of the input <b>u1</b> divided by
input <b>u2</b>:
</p>
<pre>
    y = <b>atan2</b>( u1, u2 );
</pre>
<p>
u1 and u2 shall not be zero at the same time instant.
<b>Atan2</b> uses the sign of u1 and u2 in order to construct
the solution in the range -180 deg &le; y &le; 180 deg, whereas
block <b>Atan</b> gives a solution in the range
-90 deg &le; y &le; 90 deg.
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Math/atan2.png\"
     alt=\"atan2.png\">
</p>

</html>"));
      end Atan2;

      block Sinh "Output the hyperbolic sine of the input"
        extends Interfaces.SISO;
      equation
        y = Modelica.Math.sinh(u);
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{0, 90}, {-8, 68}, {8, 68}, {0, 90}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Text(extent = {{-86, 80}, {-14, 32}}, lineColor = {192, 192, 192}, textString = "sinh"), Line(points = {{0, -80}, {0, 68}}, color = {192, 192, 192}), Line(points = {{-90, 0}, {68, 0}}, color = {192, 192, 192}), Line(points = {{-80, -80}, {-76, -65.4}, {-71.2, -51.4}, {-65.5, -38.8}, {-59.1, -28.1}, {-51.1, -18.7}, {-41.4, -11.4}, {-27.7, -5.5}, {-4.42, -0.653}, {24.5, 4.57}, {39, 10.1}, {49.4, 17.2}, {57.5, 25.9}, {63.9, 35.8}, {69.5, 47.4}, {74.4, 60.4}, {78.4, 73.8}, {80, 80}}, color = {0, 0, 0}, smooth = Smooth.Bezier), Polygon(points = {{90, 0}, {68, 8}, {68, -8}, {90, 0}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid)}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{0, 80}, {-8, 80}}, color = {192, 192, 192}), Line(points = {{0, -80}, {-8, -80}}, color = {192, 192, 192}), Line(points = {{0, -90}, {0, 84}}, color = {192, 192, 192}), Text(extent = {{9, 98}, {34, 78}}, lineColor = {160, 160, 164}, textString = "y"), Polygon(points = {{0, 100}, {-6, 84}, {6, 84}, {0, 100}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-100, 0}, {84, 0}}, color = {192, 192, 192}), Polygon(points = {{100, 0}, {84, 6}, {84, -6}, {100, 0}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-80, -80}, {-76, -65.4}, {-71.2, -51.4}, {-65.5, -38.8}, {-59.1, -28.1}, {-51.1, -18.7}, {-41.4, -11.4}, {-27.7, -5.5}, {-4.42, -0.653}, {24.5, 4.57}, {39, 10.1}, {49.4, 17.2}, {57.5, 25.9}, {63.9, 35.8}, {69.5, 47.4}, {74.4, 60.4}, {78.4, 73.8}, {80, 80}}, color = {0, 0, 0}), Text(extent = {{-31, 72}, {-11, 88}}, textString = "27", lineColor = {0, 0, 255}), Text(extent = {{-35, -88}, {-15, -72}}, textString = "-27", lineColor = {0, 0, 255}), Text(extent = {{70, 25}, {90, 5}}, textString = "4", lineColor = {0, 0, 255}), Text(extent = {{-98, 21}, {-78, 1}}, textString = "-4", lineColor = {0, 0, 255}), Text(extent = {{72, -6}, {96, -26}}, lineColor = {160, 160, 164}, textString = "u")}),
          Documentation(info = "<HTML>
<p>
This blocks computes the output <b>y</b> as the
<i>hyperbolic sine</i> of the input <b>u</b>:
</p>
<pre>
    y = <b>sinh</b>( u );
</pre>

<p>
<img src=\"modelica://Modelica/Resources/Images/Math/sinh.png\"
     alt=\"sinh.png\">
</p>

</html>"));
      end Sinh;

      block Cosh "Output the hyperbolic cosine of the input"
        extends Interfaces.SISO;
      equation
        y = Modelica.Math.cosh(u);
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{0, 90}, {-8, 68}, {8, 68}, {0, 90}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{0, -80}, {0, 68}}, color = {192, 192, 192}), Text(extent = {{4, 66}, {66, 20}}, lineColor = {192, 192, 192}, textString = "cosh"), Line(points = {{-80, 80}, {-77.6, 61.1}, {-74.4, 39.3}, {-71.2, 20.7}, {-67.1, 1.29}, {-63.1, -14.6}, {-58.3, -29.8}, {-52.7, -43.5}, {-46.2, -55.1}, {-39, -64.3}, {-30.2, -71.7}, {-18.9, -77.1}, {-4.42, -79.9}, {10.9, -79.1}, {23.7, -75.2}, {34.2, -68.7}, {42.2, -60.6}, {48.6, -51.2}, {54.3, -40}, {59.1, -27.5}, {63.1, -14.6}, {67.1, 1.29}, {71.2, 20.7}, {74.4, 39.3}, {77.6, 61.1}, {80, 80}}, color = {0, 0, 0}, smooth = Smooth.Bezier), Line(points = {{-90, -86.083}, {68, -86.083}}, color = {192, 192, 192}), Polygon(points = {{90, -86.083}, {68, -78.083}, {68, -94.083}, {90, -86.083}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid)}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{0, 80}, {-8, 80}}, color = {192, 192, 192}), Line(points = {{0, -80}, {-8, -80}}, color = {192, 192, 192}), Line(points = {{0, -90}, {0, 84}}, color = {192, 192, 192}), Text(extent = {{11, 98}, {38, 78}}, lineColor = {160, 160, 164}, textString = "y"), Polygon(points = {{0, 100}, {-6, 84}, {6, 84}, {0, 100}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-100, -86.083}, {84, -86.083}}, color = {192, 192, 192}), Polygon(points = {{100, -86.083}, {84, -80.083}, {84, -92.083}, {100, -86.083}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-80, 80}, {-77.6, 61.1}, {-74.4, 39.3}, {-71.2, 20.7}, {-67.1, 1.29}, {-63.1, -14.6}, {-58.3, -29.8}, {-52.7, -43.5}, {-46.2, -55.1}, {-39, -64.3}, {-30.2, -71.7}, {-18.9, -77.1}, {-4.42, -79.9}, {10.9, -79.1}, {23.7, -75.2}, {34.2, -68.7}, {42.2, -60.6}, {48.6, -51.2}, {54.3, -40}, {59.1, -27.5}, {63.1, -14.6}, {67.1, 1.29}, {71.2, 20.7}, {74.4, 39.3}, {77.6, 61.1}, {80, 80}}, color = {0, 0, 0}), Text(extent = {{-31, 72}, {-11, 88}}, textString = "27", lineColor = {0, 0, 255}), Text(extent = {{62, -67}, {82, -87}}, textString = "4", lineColor = {0, 0, 255}), Text(extent = {{-104, -63}, {-84, -83}}, textString = "-4", lineColor = {0, 0, 255}), Text(extent = {{78, -54}, {100, -74}}, lineColor = {160, 160, 164}, textString = "u")}),
          Documentation(info = "<HTML>
<p>
This blocks computes the output <b>y</b> as the
<i>hyperbolic cosine</i> of the input <b>u</b>:
</p>
<pre>
    y = <b>cosh</b>( u );
</pre>

<p>
<img src=\"modelica://Modelica/Resources/Images/Math/cosh.png\"
     alt=\"cosh.png\">
</p>

</html>"));
      end Cosh;

      block Tanh "Output the hyperbolic tangent of the input"
        extends Interfaces.SISO;
      equation
        y = Modelica.Math.tanh(u);
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{0, -90}, {0, 84}}, color = {192, 192, 192}), Line(points = {{-100, 0}, {84, 0}}, color = {192, 192, 192}), Line(points = {{-80, -80}, {-47.8, -78.7}, {-35.8, -75.7}, {-27.7, -70.6}, {-22.1, -64.2}, {-17.3, -55.9}, {-12.5, -44.3}, {-7.64, -29.2}, {-1.21, -4.82}, {6.83, 26.3}, {11.7, 42}, {16.5, 54.2}, {21.3, 63.1}, {26.9, 69.9}, {34.2, 75}, {45.4, 78.4}, {72, 79.9}, {80, 80}}, color = {0, 0, 0}, smooth = Smooth.Bezier), Polygon(points = {{0, 90}, {-8, 68}, {8, 68}, {0, 90}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Text(extent = {{-88, 72}, {-16, 24}}, lineColor = {192, 192, 192}, textString = "tanh"), Polygon(points = {{90, 0}, {68, 8}, {68, -8}, {90, 0}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid)}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{0, 80}, {-8, 80}}, color = {192, 192, 192}), Line(points = {{0, -80}, {-8, -80}}, color = {192, 192, 192}), Line(points = {{0, -90}, {0, 84}}, color = {192, 192, 192}), Text(extent = {{9, 98}, {42, 80}}, lineColor = {160, 160, 164}, textString = "y"), Polygon(points = {{0, 100}, {-6, 84}, {6, 84}, {0, 100}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-100, 0}, {84, 0}}, color = {192, 192, 192}), Polygon(points = {{100, 0}, {84, 6}, {84, -6}, {100, 0}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-80, -80}, {-47.8, -78.7}, {-35.8, -75.7}, {-27.7, -70.6}, {-22.1, -64.2}, {-17.3, -55.9}, {-12.5, -44.3}, {-7.64, -29.2}, {-1.21, -4.82}, {6.83, 26.3}, {11.7, 42}, {16.5, 54.2}, {21.3, 63.1}, {26.9, 69.9}, {34.2, 75}, {45.4, 78.4}, {72, 79.9}, {80, 80}}, color = {0, 0, 0}), Text(extent = {{70, 25}, {90, 5}}, textString = "4", lineColor = {0, 0, 255}), Text(extent = {{-106, 21}, {-86, 1}}, textString = "-4", lineColor = {0, 0, 255}), Text(extent = {{-29, 72}, {-9, 88}}, textString = "1", lineColor = {0, 0, 255}), Text(extent = {{3, -72}, {23, -88}}, textString = "-1", lineColor = {0, 0, 255}), Text(extent = {{72, -8}, {98, -26}}, lineColor = {160, 160, 164}, textString = "u")}),
          Documentation(info = "<HTML>
<p>
This blocks computes the output <b>y</b> as the
<i>hyperbolic tangent</i> of the input <b>u</b>:
</p>
<pre>
    y = <b>tanh</b>( u );
</pre>

<p>
<img src=\"modelica://Modelica/Resources/Images/Math/tanh.png\"
     alt=\"tanh.png\">
</p>

</html>"));
      end Tanh;

      block Exp "Output the exponential (base e) of the input"
        extends Interfaces.SISO;
      equation
        y = Modelica.Math.exp(u);
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{0, -80}, {0, 68}}, color = {192, 192, 192}), Polygon(points = {{0, 90}, {-8, 68}, {8, 68}, {0, 90}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Text(extent = {{-86, 50}, {-14, 2}}, lineColor = {192, 192, 192}, textString = "exp"), Line(points = {{-80, -80}, {-31, -77.9}, {-6.03, -74}, {10.9, -68.4}, {23.7, -61}, {34.2, -51.6}, {43, -40.3}, {50.3, -27.8}, {56.7, -13.5}, {62.3, 2.23}, {67.1, 18.6}, {72, 38.2}, {76, 57.6}, {80, 80}}, color = {0, 0, 0}), Line(points = {{-90, -80.3976}, {68, -80.3976}}, color = {192, 192, 192}, smooth = Smooth.Bezier), Polygon(points = {{90, -80.3976}, {68, -72.3976}, {68, -88.3976}, {90, -80.3976}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid)}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{0, 80}, {-8, 80}}, color = {192, 192, 192}), Line(points = {{0, -80}, {-8, -80}}, color = {192, 192, 192}), Line(points = {{0, -90}, {0, 84}}, color = {192, 192, 192}), Text(extent = {{9, 100}, {40, 80}}, lineColor = {160, 160, 164}, textString = "y"), Polygon(points = {{0, 100}, {-6, 84}, {6, 84}, {0, 100}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-100, -80.3976}, {84, -80.3976}}, color = {192, 192, 192}), Polygon(points = {{100, -80.3976}, {84, -74.3976}, {84, -86.3976}, {100, -80.3976}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-80, -80}, {-31, -77.9}, {-6.03, -74}, {10.9, -68.4}, {23.7, -61}, {34.2, -51.6}, {43, -40.3}, {50.3, -27.8}, {56.7, -13.5}, {62.3, 2.23}, {67.1, 18.6}, {72, 38.2}, {76, 57.6}, {80, 80}}, color = {0, 0, 0}), Text(extent = {{-31, 72}, {-11, 88}}, textString = "20", lineColor = {0, 0, 255}), Text(extent = {{-92, -83}, {-72, -103}}, textString = "-3", lineColor = {0, 0, 255}), Text(extent = {{70, -83}, {90, -103}}, textString = "3", lineColor = {0, 0, 255}), Text(extent = {{-18, -53}, {2, -73}}, textString = "1", lineColor = {0, 0, 255}), Text(extent = {{66, -52}, {96, -72}}, lineColor = {160, 160, 164}, textString = "u")}),
          Documentation(info = "<HTML>
<p>
This blocks computes the output <b>y</b> as the
<i>exponential</i> (of base e) of the input <b>u</b>:
</p>
<pre>
    y = <b>exp</b>( u );
</pre>

<p>
<img src=\"modelica://Modelica/Resources/Images/Math/exp.png\"
     alt=\"exp.png\">
</p>

</html>"));
      end Exp;

      block Log "Output the natural (base e) logarithm of the input (input > 0 required)"
        extends Interfaces.SISO;
      equation
        y = Modelica.Math.log(u);
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-80, -80}, {-80, 68}}, color = {192, 192, 192}), Polygon(points = {{-80, 90}, {-88, 68}, {-72, 68}, {-80, 90}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-80, -80}, {-79.2, -50.6}, {-78.4, -37}, {-77.6, -28}, {-76.8, -21.3}, {-75.2, -11.4}, {-72.8, -1.31}, {-69.5, 8.08}, {-64.7, 17.9}, {-57.5, 28}, {-47, 38.1}, {-31.8, 48.1}, {-10.1, 58}, {22.1, 68}, {68.7, 78.1}, {80, 80}}, color = {0, 0, 0}, smooth = Smooth.Bezier), Line(points = {{-90, 0}, {68, 0}}, color = {192, 192, 192}), Polygon(points = {{90, 0}, {68, 8}, {68, -8}, {90, 0}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Text(extent = {{-6, -24}, {66, -72}}, lineColor = {192, 192, 192}, textString = "log")}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-80, 80}, {-88, 80}}, color = {192, 192, 192}), Line(points = {{-80, -80}, {-88, -80}}, color = {192, 192, 192}), Line(points = {{-80, -90}, {-80, 84}}, color = {192, 192, 192}), Text(extent = {{-67, 98}, {-40, 80}}, lineColor = {160, 160, 164}, textString = "y"), Polygon(points = {{-80, 100}, {-86, 84}, {-74, 84}, {-80, 100}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-100, 0}, {84, 0}}, color = {192, 192, 192}), Polygon(points = {{100, 0}, {84, 6}, {84, -6}, {100, 0}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-80, -80}, {-79.2, -50.6}, {-78.4, -37}, {-77.6, -28}, {-76.8, -21.3}, {-75.2, -11.4}, {-72.8, -1.31}, {-69.5, 8.08}, {-64.7, 17.9}, {-57.5, 28}, {-47, 38.1}, {-31.8, 48.1}, {-10.1, 58}, {22.1, 68}, {68.7, 78.1}, {80, 80}}, color = {0, 0, 0}), Text(extent = {{-105, 72}, {-85, 88}}, textString = "3", lineColor = {0, 0, 255}), Text(extent = {{-109, -88}, {-89, -72}}, textString = "-3", lineColor = {0, 0, 255}), Text(extent = {{70, -3}, {90, -23}}, textString = "20", lineColor = {0, 0, 255}), Text(extent = {{-78, -1}, {-58, -21}}, textString = "1", lineColor = {0, 0, 255}), Text(extent = {{68, 28}, {94, 8}}, lineColor = {160, 160, 164}, textString = "u")}),
          Documentation(info = "<HTML>
<p>
This blocks computes the output <b>y</b> as the
<i>natural (base e) logarithm</i> of the input <b>u</b>:
</p>
<pre>
    y = <b>log</b>( u );
</pre>
<p>
An error occurs if the elements of the input <b>u</b> are
zero or negative.
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Math/log.png\"
     alt=\"log.png\">
</p>

</html>"));
      end Log;

      block Log10 "Output the base 10 logarithm of the input (input > 0 required)"
        extends Interfaces.SISO;
      equation
        y = Modelica.Math.log10(u);
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{90, 0}, {68, 8}, {68, -8}, {90, 0}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-90, 0}, {68, 0}}, color = {192, 192, 192}), Line(points = {{-79.8, -80}, {-79.2, -50.6}, {-78.4, -37}, {-77.6, -28}, {-76.8, -21.3}, {-75.2, -11.4}, {-72.8, -1.31}, {-69.5, 8.08}, {-64.7, 17.9}, {-57.5, 28}, {-47, 38.1}, {-31.8, 48.1}, {-10.1, 58}, {22.1, 68}, {68.7, 78.1}, {80, 80}}, color = {0, 0, 0}, smooth = Smooth.Bezier), Polygon(points = {{-80, 90}, {-88, 68}, {-72, 68}, {-80, 90}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-80, -80}, {-80, 68}}, color = {192, 192, 192}), Text(extent = {{-30, -22}, {60, -70}}, lineColor = {192, 192, 192}, textString = "log10")}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-80, 80}, {-88, 80}}, color = {192, 192, 192}), Line(points = {{-80, -80}, {-88, -80}}, color = {192, 192, 192}), Line(points = {{-80, -90}, {-80, 84}}, color = {192, 192, 192}), Text(extent = {{-65, 96}, {-38, 78}}, lineColor = {160, 160, 164}, textString = "y"), Polygon(points = {{-80, 100}, {-86, 84}, {-74, 84}, {-80, 100}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-100, 0}, {84, 0}}, color = {192, 192, 192}), Polygon(points = {{100, 0}, {84, 6}, {84, -6}, {100, 0}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-79.8, -80}, {-79.2, -50.6}, {-78.4, -37}, {-77.6, -28}, {-76.8, -21.3}, {-75.2, -11.4}, {-72.8, -1.31}, {-69.5, 8.08}, {-64.7, 17.9}, {-57.5, 28}, {-47, 38.1}, {-31.8, 48.1}, {-10.1, 58}, {22.1, 68}, {68.7, 78.1}, {80, 80}}, color = {0, 0, 0}), Text(extent = {{70, -3}, {90, -23}}, textString = "20", lineColor = {0, 0, 255}), Text(extent = {{-78, -1}, {-58, -21}}, textString = "1", lineColor = {0, 0, 255}), Text(extent = {{-109, 72}, {-89, 88}}, textString = " 1.3", lineColor = {0, 0, 255}), Text(extent = {{-109, -88}, {-89, -72}}, textString = "-1.3", lineColor = {0, 0, 255}), Text(extent = {{62, 30}, {90, 10}}, lineColor = {160, 160, 164}, textString = "u")}),
          Documentation(info = "<HTML>
<p>
This blocks computes the output <b>y</b> as the
<i>base 10 logarithm</i> of the input <b>u</b>:
</p>
<pre>
    y = <b>log10</b>( u );
</pre>
<p>
An error occurs if the elements of the input <b>u</b> are
zero or negative.
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Math/log10.png\"
     alt=\"log10.png\">
</p>

</html>"));
      end Log10;

      block RealToInteger "Convert Real to Integer signal"
        extends Modelica.Blocks.Icons.IntegerBlock;
      public
        Interfaces.RealInput u "Connector of Real input signal" annotation(
          Placement(transformation(extent = {{-140, -20}, {-100, 20}}, rotation = 0)));
        Interfaces.IntegerOutput y "Connector of Integer output signal" annotation(
          Placement(transformation(extent = {{100, -10}, {120, 10}}, rotation = 0)));
      equation
        y = if u > 0 then integer(floor(u + 0.5)) else integer(ceil(u - 0.5));
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100.0, -100.0}, {100.0, 100.0}}, initialScale = 0.1), graphics = {Text(lineColor = {0, 0, 127}, extent = {{-100.0, -40.0}, {0.0, 40.0}}, textString = "R"), Text(lineColor = {255, 127, 0}, extent = {{20.0, -40.0}, {120.0, 40.0}}, textString = "I"), Polygon(lineColor = {255, 127, 0}, fillColor = {255, 127, 0}, fillPattern = FillPattern.Solid, points = {{50.0, 0.0}, {30.0, 20.0}, {30.0, 10.0}, {0.0, 10.0}, {0.0, -10.0}, {30.0, -10.0}, {30.0, -20.0}, {50.0, 0.0}})}),
          Documentation(info = "<html>
<p>
This block computes the output <b>y</b>
as <i>nearest integer value</i> of the input <b>u</b>:
</p>
<pre>    y = <b>integer</b>( <b>floor</b>( u + 0.5 ) )  for  u &gt; 0;
    y = <b>integer</b>( <b>ceil </b>( u - 0.5 ) )  for  u &lt; 0;
</pre>
</html>"));
      end RealToInteger;

      block IntegerToReal "Convert Integer to Real signals"
        extends Modelica.Blocks.Icons.Block;
        Interfaces.IntegerInput u "Connector of Integer input signal" annotation(
          Placement(transformation(extent = {{-140, -20}, {-100, 20}}, rotation = 0)));
        Interfaces.RealOutput y "Connector of Real output signal" annotation(
          Placement(transformation(extent = {{100, -10}, {120, 10}}, rotation = 0)));
      equation
        y = u;
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100.0, -100.0}, {100.0, 100.0}}, initialScale = 0.1), graphics = {Text(lineColor = {255, 127, 0}, extent = {{-120.0, -40.0}, {-20.0, 40.0}}, textString = "I"), Text(lineColor = {0, 0, 127}, fillPattern = FillPattern.Solid, extent = {{0.0, -40.0}, {100.0, 40.0}}, textString = "R"), Polygon(lineColor = {0, 0, 127}, fillColor = {0, 0, 127}, fillPattern = FillPattern.Solid, points = {{10.0, 0.0}, {-10.0, 20.0}, {-10.0, 10.0}, {-40.0, 10.0}, {-40.0, -10.0}, {-10.0, -10.0}, {-10.0, -20.0}, {10.0, 0.0}})}),
          Documentation(info = "<html>
<p>
This block computes the output <b>y</b>
as <i>Real equivalent</i> of the Integer input <b>u</b>:
</p>
<pre>    y = u;
</pre>
<p>where <b>u</b> is of Integer and <b>y</b> of Real type.</p>
</html>"));
      end IntegerToReal;

      block BooleanToReal "Convert Boolean to Real signal"
        extends Interfaces.partialBooleanSI;
        parameter Real realTrue = 1.0 "Output signal for true Boolean input";
        parameter Real realFalse = 0.0 "Output signal for false Boolean input";
        Blocks.Interfaces.RealOutput y "Connector of Real output signal" annotation(
          Placement(transformation(extent = {{100, -10}, {120, 10}}, rotation = 0)));
      equation
        y = if u then realTrue else realFalse;
        annotation(
          Documentation(info = "<html>
<p>
This block computes the output <b>y</b>
as <i>Real equivalent</i> of the Boolean input <b>u</b>:
</p>
<pre>    y = <b>if</b> u <b>then</b> realTrue <b>else</b> realFalse;
</pre>
<p>where <b>u</b> is of Boolean and <b>y</b> of Real type,
and <b>realTrue</b> and <b>realFalse</b> are parameters.
</p>
</html>"),
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-86, 92}, {-6, 10}}, lineColor = {255, 0, 255}, textString = "B"), Polygon(points = {{-12, -46}, {-32, -26}, {-32, -36}, {-64, -36}, {-64, -56}, {-32, -56}, {-32, -66}, {-12, -46}}, fillColor = {0, 0, 127}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 127}), Text(extent = {{8, -4}, {92, -94}}, textString = "R", lineColor = {0, 0, 127})}));
      end BooleanToReal;

      block BooleanToInteger "Convert Boolean to Integer signal"
        extends Interfaces.partialBooleanSI;
        parameter Integer integerTrue = 1 "Output signal for true Boolean input";
        parameter Integer integerFalse = 0 "Output signal for false Boolean input";
        Blocks.Interfaces.IntegerOutput y "Connector of Integer output signal" annotation(
          Placement(transformation(extent = {{100, -10}, {120, 10}}, rotation = 0)));
      equation
        y = if u then integerTrue else integerFalse;
        annotation(
          Documentation(info = "<html>
<p>
This block computes the output <b>y</b>
as <i>Integer equivalent</i> of the Boolean input <b>u</b>:
</p>
<pre>    y = <b>if</b> u <b>then</b> integerTrue <b>else</b> integerFalse;
</pre>
<p>where <b>u</b> is of Boolean and <b>y</b> of Integer type,
and <b>integerTrue</b> and <b>integerFalse</b> are parameters.
</p>
</html>"),
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-86, 92}, {-6, 10}}, lineColor = {255, 0, 255}, textString = "B"), Polygon(points = {{-12, -46}, {-32, -26}, {-32, -36}, {-64, -36}, {-64, -56}, {-32, -56}, {-32, -66}, {-12, -46}}, lineColor = {255, 170, 85}, fillColor = {255, 170, 85}, fillPattern = FillPattern.Solid), Text(extent = {{8, -4}, {92, -94}}, lineColor = {255, 170, 85}, textString = "I")}));
      end BooleanToInteger;

      block RealToBoolean "Convert Real to Boolean signal"
        Blocks.Interfaces.RealInput u "Connector of Real input signal" annotation(
          Placement(transformation(extent = {{-140, -20}, {-100, 20}}, rotation = 0)));
        extends Interfaces.partialBooleanSO;
        parameter Real threshold = 0.5 "Output signal y is true, if input u >= threshold";
      equation
        y = u >= threshold;
        annotation(
          Documentation(info = "<html>
<p>
This block computes the Boolean output <b>y</b>
from the Real input <b> u</b> by the equation:
</p>

<pre>    y = u &ge; threshold;
</pre>

<p>
where <b>threshold</b> is a parameter.
</p>
</html>"),
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-86, 92}, {-6, 10}}, lineColor = {0, 0, 127}, textString = "R"), Polygon(points = {{-12, -46}, {-32, -26}, {-32, -36}, {-64, -36}, {-64, -56}, {-32, -56}, {-32, -66}, {-12, -46}}, lineColor = {255, 0, 255}, fillColor = {255, 0, 255}, fillPattern = FillPattern.Solid), Text(extent = {{8, -4}, {92, -94}}, lineColor = {255, 0, 255}, textString = "B")}));
      end RealToBoolean;

      block IntegerToBoolean "Convert Integer to Boolean signal"
        Blocks.Interfaces.IntegerInput u "Connector of Integer input signal" annotation(
          Placement(transformation(extent = {{-140, -20}, {-100, 20}}, rotation = 0)));
        extends Interfaces.partialBooleanSO;
        parameter Integer threshold = 1 "Output signal y is true, if input u >= threshold";
      equation
        y = u >= threshold;
        annotation(
          Documentation(info = "<html>
<p>
This block computes the Boolean output <b>y</b>
from the Integer input <b> u</b> by the equation:
</p>

<pre>    y = u &ge; threshold;
</pre>

<p>
where <b>threshold</b> is a parameter.
</p>
</html>"),
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-86, 92}, {-6, 10}}, lineColor = {255, 128, 0}, textString = "I"), Polygon(points = {{-12, -46}, {-32, -26}, {-32, -36}, {-64, -36}, {-64, -56}, {-32, -56}, {-32, -66}, {-12, -46}}, lineColor = {255, 0, 255}, fillColor = {255, 0, 255}, fillPattern = FillPattern.Solid), Text(extent = {{8, -4}, {92, -94}}, lineColor = {255, 0, 255}, textString = "B")}));
      end IntegerToBoolean;

      block RectangularToPolar "Convert rectangular coordinates to polar coordinates"
        extends Modelica.Blocks.Icons.Block;
        Modelica.Blocks.Interfaces.RealInput u_re "Real part of rectangular representation" annotation(
          Placement(transformation(extent = {{-140, 40}, {-100, 80}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealInput u_im "Imaginary part of rectangular representation" annotation(
          Placement(transformation(extent = {{-140, -80}, {-100, -40}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealOutput y_abs "Length of polar representation" annotation(
          Placement(transformation(extent = {{100, 50}, {120, 70}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealOutput y_arg "Angle of polar representation" annotation(
          Placement(transformation(extent = {{100, -70}, {120, -50}}, rotation = 0)));
      equation
        y_abs = sqrt(u_re * u_re + u_im * u_im);
        y_arg = Modelica.Math.atan2(u_im, u_re);
        annotation(
          Icon(graphics = {Text(extent = {{-90, 80}, {-20, 40}}, lineColor = {0, 0, 0}, textString = "re"), Text(extent = {{-90, -40}, {-20, -80}}, lineColor = {0, 0, 0}, textString = "im"), Text(extent = {{20, 80}, {90, 40}}, lineColor = {0, 0, 0}, textString = "abs"), Text(extent = {{20, -40}, {90, -80}}, lineColor = {0, 0, 0}, textString = "arg")}),
          Documentation(info = "<html>
<p>
The input values of this block are the rectangular components
<code>u_re</code> and <code>u_im</code> of a phasor in two dimensions.
This block calculates the length <code>y_abs</code> and
the angle <code>y_arg</code> of the polar representation of this phasor.
</p>

<pre>
  y_abs = abs(u_re + j*u_im) = sqrt( u_re<sup>2</sup> + u_im<sup>2</sup> )
  y_arg = arg(u_re + j*u_im) = atan2(u_im, u_re)
</pre>
</html>"));
      end RectangularToPolar;

      block PolarToRectangular "Convert polar coordinates to rectangular coordinates"
        extends Modelica.Blocks.Icons.Block;
        Modelica.Blocks.Interfaces.RealInput u_abs "Length of polar representation" annotation(
          Placement(transformation(extent = {{-140, 40}, {-100, 80}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealInput u_arg "Angle of polar representation" annotation(
          Placement(transformation(extent = {{-140, -80}, {-100, -40}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealOutput y_re "Real part of rectangular representation" annotation(
          Placement(transformation(extent = {{100, 50}, {120, 70}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealOutput y_im "Imaginary part of rectangular representation" annotation(
          Placement(transformation(extent = {{100, -70}, {120, -50}}, rotation = 0)));
      equation
        y_re = u_abs * Modelica.Math.cos(u_arg);
        y_im = u_abs * Modelica.Math.sin(u_arg);
        annotation(
          Icon(graphics = {Text(extent = {{-90, 80}, {-20, 40}}, lineColor = {0, 0, 0}, textString = "abs"), Text(extent = {{-90, -40}, {-20, -80}}, lineColor = {0, 0, 0}, textString = "arg"), Text(extent = {{20, 80}, {90, 40}}, lineColor = {0, 0, 0}, textString = "re"), Text(extent = {{20, -40}, {90, -80}}, lineColor = {0, 0, 0}, textString = "im")}),
          Documentation(info = "<html>
<p>
The input values of this block are the polar components <code>uabs</code> and <code>uarg</code> of a phasor.
This block calculates the components <code>y_re</code> and <code>y_im</code> of the rectangular representation of this phasor.
</p>
<pre>
   y_re = u_abs * cos( u_arg )
   y_im = u_abs * sin( u_arg )
</pre>
</html>"));
      end PolarToRectangular;

      block Mean "Calculate mean over period 1/f"
        extends Modelica.Blocks.Interfaces.SISO;
        parameter Modelica.SIunits.Frequency f(start = 50) "Base frequency";
        parameter Real x0 = 0 "Start value of integrator state";
        parameter Boolean yGreaterOrEqualZero = false "=true, if output y is guaranteed to be >= 0 for the exact solution" annotation(
          Evaluate = true,
          Dialog(tab = "Advanced"));
      protected
        parameter Modelica.SIunits.Time t0(fixed = false) "Start time of simulation";
        Real x "Integrator state";
      initial equation
        t0 = time;
        x = x0;
        y = 0;
      equation
        der(x) = u;
        when sample(t0 + 1 / f, 1 / f) then
          y = if not yGreaterOrEqualZero then f * pre(x) else max(0.0, f * pre(x));
          reinit(x, 0);
        end when;
        annotation(
          Documentation(info = "<html>
<p>
This block calculates the mean of the input signal u over the given period 1/f:
</p>
<pre>
1 T
- &int; u(t) dt
T 0
</pre>
<p>
Note: The output is updated after each period defined by 1/f.
</p>

<p>
If parameter <b>yGreaterOrEqualZero</b> in the Advanced tab is <b>true</b> (default = <b>false</b>),
then the modeller provides the information that the mean of the input signal is guaranteed
to be &ge; 0 for the exact solution. However, due to inaccuracies in the numerical integration scheme,
the output might be slightly negative. If this parameter is set to true, then the output is
explicitly set to 0.0, if the mean value results in a negative value.
</p>
</html>"),
          Icon(graphics = {Text(extent = {{-80, 60}, {80, 20}}, lineColor = {0, 0, 0}, textString = "mean"), Text(extent = {{-80, -20}, {80, -60}}, lineColor = {0, 0, 0}, textString = "f=%f")}));
      end Mean;

      block RectifiedMean "Calculate rectified mean over period 1/f"
        extends Modelica.Blocks.Interfaces.SISO;
        parameter Modelica.SIunits.Frequency f(start = 50) "Base frequency";
        parameter Real x0 = 0 "Start value of integrator state";
        Mean mean(final f = f, final x0 = x0) annotation(
          Placement(transformation(extent = {{0, -10}, {20, 10}})));
        Blocks.Math.Abs abs1 annotation(
          Placement(transformation(extent = {{-60, -10}, {-40, 10}})));
      equation
        connect(u, abs1.u) annotation(
          Line(points = {{-120, 0}, {-62, 0}}, color = {0, 0, 127}, smooth = Smooth.None));
        connect(abs1.y, mean.u) annotation(
          Line(points = {{-39, 0}, {-2, 0}}, color = {0, 0, 127}, smooth = Smooth.None));
        connect(mean.y, y) annotation(
          Line(points = {{21, 0}, {110, 0}}, color = {0, 0, 127}, smooth = Smooth.None));
        annotation(
          Documentation(info = "<html>
<p>
This block calculates the rectified mean of the input signal u over the given period 1/f, using the
<a href=\"modelica://Modelica.Blocks.Math.Mean\">mean block</a>.
</p>
<p>
Note: The output is updated after each period defined by 1/f.
</p>
</html>"),
          Icon(graphics = {Text(extent = {{-80, 60}, {80, 20}}, lineColor = {0, 0, 0}, textString = "RM"), Text(extent = {{-80, -20}, {80, -60}}, lineColor = {0, 0, 0}, textString = "f=%f")}));
      end RectifiedMean;

      block RootMeanSquare "Calculate root mean square over period 1/f"
        extends Modelica.Blocks.Interfaces.SISO;
        parameter Modelica.SIunits.Frequency f(start = 50) "Base frequency";
        parameter Real x0 = 0 "Start value of integrator state";
        MultiProduct product(nu = 2) annotation(
          Placement(transformation(extent = {{-40, -10}, {-20, 10}})));
        Mean mean(final f = f, final yGreaterOrEqualZero = true, final x0 = x0) annotation(
          Placement(transformation(extent = {{0, -10}, {20, 10}})));
        Blocks.Math.Sqrt sqrt1 annotation(
          Placement(transformation(extent = {{40, -10}, {60, 10}})));
      equation
        connect(product.y, mean.u) annotation(
          Line(points = {{-18.3, 0}, {-2, 0}}, color = {0, 0, 127}, smooth = Smooth.None));
        connect(mean.y, sqrt1.u) annotation(
          Line(points = {{21, 0}, {38, 0}}, color = {0, 0, 127}, smooth = Smooth.None));
        connect(sqrt1.y, y) annotation(
          Line(points = {{61, 0}, {110, 0}}, color = {0, 0, 127}, smooth = Smooth.None));
        connect(u, product.u[1]) annotation(
          Line(points = {{-120, 0}, {-60, 0}, {-60, 3.5}, {-40, 3.5}}, color = {0, 0, 127}, smooth = Smooth.None));
        connect(u, product.u[2]) annotation(
          Line(points = {{-120, 0}, {-60, 0}, {-60, -3.5}, {-40, -3.5}}, color = {0, 0, 127}, smooth = Smooth.None));
        annotation(
          Documentation(info = "<html>
<p>
This block calculates the root mean square of the input signal u over the given period 1/f, using the
<a href=\"modelica://Modelica.Blocks.Math.Mean\">mean block</a>.
</p>
<p>
Note: The output is updated after each period defined by 1/f.
</p>
</html>"),
          Icon(graphics = {Text(extent = {{-80, 60}, {80, 20}}, lineColor = {0, 0, 0}, textString = "RMS"), Text(extent = {{-80, -20}, {80, -60}}, lineColor = {0, 0, 0}, textString = "f=%f")}));
      end RootMeanSquare;

      block Harmonic "Calculate harmonic over period 1/f"
        extends Modelica.Blocks.Icons.Block;
        parameter Modelica.SIunits.Frequency f(start = 50) "Base frequency";
        parameter Integer k(start = 1) "Order of harmonic";
        parameter Real x0Cos = 0 "Start value of cos integrator state";
        parameter Real x0Sin = 0 "Start value of sin integrator state";
        Blocks.Sources.Sine sin1(final amplitude = sqrt(2), final phase = Modelica.Constants.pi / 2, final freqHz = k * f) annotation(
          Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 270, origin = {-80, 70})));
        Blocks.Sources.Sine sin2(final amplitude = sqrt(2), final phase = 0, final freqHz = k * f) annotation(
          Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 90, origin = {-80, -70})));
        MultiProduct product1(nu = 2) annotation(
          Placement(transformation(extent = {{-60, 30}, {-40, 50}})));
        MultiProduct product2(nu = 2) annotation(
          Placement(transformation(extent = {{-60, -50}, {-40, -30}})));
        Mean mean1(final f = f, final x0 = x0Cos) annotation(
          Placement(transformation(extent = {{-20, 30}, {0, 50}})));
        Mean mean2(final f = f, final x0 = x0Sin) annotation(
          Placement(transformation(extent = {{-20, -50}, {0, -30}})));
        Blocks.Interfaces.RealInput u annotation(
          Placement(transformation(extent = {{-140, -20}, {-100, 20}})));
        Blocks.Interfaces.RealOutput y_rms "Root mean square of polar representation" annotation(
          Placement(transformation(extent = {{100, 50}, {120, 70}}, rotation = 0)));
        Blocks.Interfaces.RealOutput y_arg "Angle of polar representation" annotation(
          Placement(transformation(extent = {{100, -70}, {120, -50}}, rotation = 0)));
        Blocks.Math.RectangularToPolar rectangularToPolar annotation(
          Placement(transformation(extent = {{40, -12}, {60, 8}})));
      equation
        connect(product2.y, mean2.u) annotation(
          Line(points = {{-38.3, -40}, {-22, -40}}, color = {0, 0, 127}, smooth = Smooth.None));
        connect(product1.y, mean1.u) annotation(
          Line(points = {{-38.3, 40}, {-22, 40}}, color = {0, 0, 127}, smooth = Smooth.None));
        connect(mean1.y, rectangularToPolar.u_re) annotation(
          Line(points = {{1, 40}, {20, 40}, {20, 4}, {38, 4}}, color = {0, 0, 127}, smooth = Smooth.None));
        connect(mean2.y, rectangularToPolar.u_im) annotation(
          Line(points = {{1, -40}, {20, -40}, {20, -8}, {38, -8}}, color = {0, 0, 127}, smooth = Smooth.None));
        connect(rectangularToPolar.y_abs, y_rms) annotation(
          Line(points = {{61, 4}, {80, 4}, {80, 60}, {110, 60}}, color = {0, 0, 127}, smooth = Smooth.None));
        connect(rectangularToPolar.y_arg, y_arg) annotation(
          Line(points = {{61, -8}, {80, -8}, {80, -60}, {110, -60}}, color = {0, 0, 127}, smooth = Smooth.None));
        connect(sin1.y, product1.u[1]) annotation(
          Line(points = {{-80, 59}, {-80, 59}, {-80, 43.5}, {-60, 43.5}}, color = {0, 0, 127}, smooth = Smooth.None));
        connect(u, product1.u[2]) annotation(
          Line(points = {{-120, 0}, {-80, 0}, {-80, 36.5}, {-60, 36.5}}, color = {0, 0, 127}, smooth = Smooth.None));
        connect(u, product2.u[1]) annotation(
          Line(points = {{-120, 0}, {-80, 0}, {-80, -36.5}, {-60, -36.5}}, color = {0, 0, 127}, smooth = Smooth.None));
        connect(sin2.y, product2.u[2]) annotation(
          Line(points = {{-80, -59}, {-80, -43.5}, {-60, -43.5}}, color = {0, 0, 127}, smooth = Smooth.None));
        annotation(
          Documentation(info = "<html>
<p>
This block calculates the root mean square and the phase angle of a single harmonic <i>k</i> of the input signal u over the given period 1/f, using the
<a href=\"modelica://Modelica.Blocks.Math.Mean\">mean block</a>.
</p>
<p>
Note: The output is updated after each period defined by 1/f.
</p>
<p>
Note: The harmonic is defined by <code>&radic;2 rms cos(k 2 &pi; f t - arg)</code>
</p>
</html>"),
          Icon(graphics = {Text(extent = {{-80, 60}, {80, 20}}, lineColor = {0, 0, 0}, textString = "H%k"), Text(extent = {{-80, -20}, {80, -60}}, lineColor = {0, 0, 0}, textString = "f=%f"), Text(extent = {{20, 100}, {100, 60}}, lineColor = {0, 0, 0}, textString = "rms"), Text(extent = {{20, -60}, {100, -100}}, lineColor = {0, 0, 0}, textString = "arg")}));
      end Harmonic;

      block Max "Pass through the largest signal"
        extends Interfaces.SI2SO;
      equation
        y = max(u1, u2);
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-90, 36}, {90, -36}}, lineColor = {160, 160, 164}, textString = "max()")}),
          Documentation(info = "<html>
<p>
This block computes the output <b>y</b> as <i>maximum</i>
of the two Real inputs <b>u1</b> and <b>u2</b>:
</p>
<pre>    y = <b>max</b> ( u1 , u2 );
</pre>
</html>"));
      end Max;

      block Min "Pass through the smallest signal"
        extends Interfaces.SI2SO;
      equation
        y = min(u1, u2);
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-90, 36}, {90, -36}}, lineColor = {160, 160, 164}, textString = "min()")}),
          Documentation(info = "<html>
<p>
This block computes the output <b>y</b> as <i>minimum</i> of
the two Real inputs <b>u1</b> and <b>u2</b>:
</p>
<pre>    y = <b>min</b> ( u1 , u2 );
</pre>
</html>"));
      end Min;

      block MinMax "Output the minimum and the maximum element of the input vector"
        extends Modelica.Blocks.Icons.Block;
        parameter Integer nu(min = 0) = 0 "Number of input connections" annotation(
          Dialog(connectorSizing = true),
          HideResult = true);
        Modelica.Blocks.Interfaces.RealVectorInput u[nu] annotation(
          Placement(transformation(extent = {{-120, 70}, {-80, -70}})));
        Modelica.Blocks.Interfaces.RealOutput yMax annotation(
          Placement(transformation(extent = {{100, 50}, {120, 70}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealOutput yMin annotation(
          Placement(transformation(extent = {{100, -70}, {120, -50}}, rotation = 0)));
      equation
        yMax = max(u);
        yMin = min(u);
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}, initialScale = 0.1), graphics = {Text(extent = {{-12, 80}, {100, 40}}, lineColor = {0, 0, 0}, textString = "yMax"), Text(extent = {{-10, -40}, {100, -80}}, lineColor = {0, 0, 0}, textString = "yMin")}),
          Documentation(info = "<html>
<p>
Determines the minimum and maximum element of the input vector and
provide both values as output.
</p>
</html>"));
      end MinMax;

      block LinearDependency "Output a linear combination of the two inputs"
        extends Modelica.Blocks.Interfaces.SI2SO;
        parameter Real y0 = 0 "Initial value";
        parameter Real k1 = 0 "u1 dependency";
        parameter Real k2 = 0 "u2 dependency";
      equation
        y = y0 * (1 + k1 * u1 + k2 * u2);
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-100, 60}, {100, 0}, {-100, -60}}, color = {0, 0, 127}, smooth = Smooth.None), Text(extent = {{-14, 88}, {94, 32}}, lineColor = {0, 0, 0}, textString = "%k1"), Text(extent = {{-40, -48}, {96, -96}}, lineColor = {0, 0, 0}, textString = "%k2"), Text(extent = {{-94, 26}, {8, -30}}, lineColor = {0, 0, 0}, textString = "%y0")}),
          Documentation(info = "<html>
<p>
Determine the linear combination of the two inputs:
<code>y = y0*(1 + k1*u1 + k2*u2)</code>
</p>
</html>"));
      end LinearDependency;

      block Edge "Indicates rising edge of Boolean signal"
        extends Interfaces.BooleanSISO;
      equation
        y = edge(u);
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-90, 36}, {90, -36}}, lineColor = {160, 160, 164}, textString = "edge()")}),
          Documentation(info = "<html>
<p>
This block sets the Boolean output <b>y</b> to true,
when the Boolean input <b>u</b> shows a <i>rising edge</i>:
</p>
<pre>    y = <b>edge</b>( u );
</pre>
</html>"));
      end Edge;

      block BooleanChange "Indicates Boolean signal changing"
        extends Interfaces.BooleanSISO;
      equation
        y = change(u);
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-90, 36}, {90, -36}}, lineColor = {160, 160, 164}, textString = "change()")}),
          Documentation(info = "<html>
<p>
This block sets the Boolean output <b>y</b> to true, when the
Boolean input <b>u</b> shows a <i>rising or falling edge</i>,
i.e., when the signal changes:
</p>
<pre>    y = <b>change</b>( u );
</pre>
</html>"));
      end BooleanChange;

      block IntegerChange "Indicates integer signal changing"
        extends Interfaces.IntegerSIBooleanSO;
      equation
        y = change(u);
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-90, 36}, {90, -36}}, lineColor = {160, 160, 164}, textString = "change()")}),
          Documentation(info = "<html>
<p>
This block sets the Boolean output <b>y</b> to true, when the
Integer input <b>u</b> changes:
</p>
<pre>    y = <b>change</b>( u );
</pre>
</html>"));
      end IntegerChange;
      annotation(
        Documentation(info = "<html>
<p>
This package contains basic <b>mathematical operations</b>,
such as summation and multiplication, and basic <b>mathematical
functions</b>, such as <b>sqrt</b> and <b>sin</b>, as
input/output blocks. All blocks of this library can be either
connected with continuous blocks or with sampled-data blocks.
</p>
</html>", revisions = "<html>
<ul>
<li><i>October 21, 2002</i>
       by <a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a>
       and <a href=\"http://www.robotic.dlr.de/Christian.Schweiger/\">Christian Schweiger</a>:<br>
       New blocks added: RealToInteger, IntegerToReal, Max, Min, Edge, BooleanChange, IntegerChange.</li>
<li><i>August 7, 1999</i>
       by <a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a>:<br>
       Realized (partly based on an existing Dymola library
       of Dieter Moormann and Hilding Elmqvist).
</li>
</ul>
</html>"),
        Icon(graphics = {Line(points = {{-80, -2}, {-68.7, 32.2}, {-61.5, 51.1}, {-55.1, 64.4}, {-49.4, 72.6}, {-43.8, 77.1}, {-38.2, 77.8}, {-32.6, 74.6}, {-26.9, 67.7}, {-21.3, 57.4}, {-14.9, 42.1}, {-6.83, 19.2}, {10.1, -32.8}, {17.3, -52.2}, {23.7, -66.2}, {29.3, -75.1}, {35, -80.4}, {40.6, -82}, {46.2, -79.6}, {51.9, -73.5}, {57.5, -63.9}, {63.9, -49.2}, {72, -26.8}, {80, -2}}, color = {95, 95, 95}, smooth = Smooth.Bezier)}));
    end Math;

    package MathInteger "Library of Integer mathematical functions as input/output blocks"
      extends Modelica.Icons.Package;

      block MultiSwitch "Set Integer expression that is associated with the first active input signal"
        input Integer expr[nu] = fill(0, nu) "y = if u[i] then expr[i] elseif use_pre_as_default then pre(y) else y_default" annotation(
          Dialog);
        parameter Integer y_default = 0 "Default value of output y if use_pre_as_default=false, as well as pre(y) at initial time";
        parameter Boolean use_pre_as_default = true "= true, y holds its last value if all u[i]=false, otherwise y=y_default" annotation(
          HideResult = true,
          choices(checkBox = true));
        parameter Integer nu(min = 0) = 0 "Number of input connections" annotation(
          Dialog(connectorSizing = true),
          HideResult = true);
        Modelica.Blocks.Interfaces.BooleanVectorInput u[nu] "Set y = expr[i], if u[i] = true" annotation(
          Placement(transformation(extent = {{-110, 30}, {-90, -30}})));
        Modelica.Blocks.Interfaces.IntegerOutput y "Output depending on expression" annotation(
          Placement(transformation(extent = {{300, -10}, {320, 10}})));
      protected
        Integer firstActiveIndex;
      initial equation
        pre(y) = y_default;
      equation
        firstActiveIndex = Modelica.Math.BooleanVectors.firstTrueIndex(u);
        y = if firstActiveIndex > 0 then expr[firstActiveIndex] else if use_pre_as_default then pre(y) else y_default;
        annotation(
          defaultComponentName = "multiSwitch1",
          Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {300, 100}}), graphics = {Text(extent = {{310, -25}, {410, -45}}, lineColor = {0, 0, 0}, textString = DynamicSelect(" ", String(y, minimumLength = 1, significantDigits = 0))), Text(visible = not use_pre_as_default, extent = {{-100, -60}, {300, -90}}, lineColor = {0, 0, 0}, textString = "else: %y_default"), Text(visible = use_pre_as_default, extent = {{-100, -50}, {300, -80}}, lineColor = {0, 0, 0}, textString = "else: pre(y)"), Rectangle(extent = {{-100, -40}, {300, 40}}, lineColor = {0, 0, 0}, fillColor = {255, 213, 170}, fillPattern = FillPattern.Solid, borderPattern = BorderPattern.Raised), Text(extent = {{-100, 90}, {300, 50}}, textString = "%name", lineColor = {0, 0, 255}), Text(extent = {{-80, 15}, {290, -15}}, lineColor = {0, 0, 0}, fillColor = {255, 246, 238}, fillPattern = FillPattern.Solid, textString = "%expr")}),
          Documentation(info = "<html>
<p>
This block has a vector of Boolean input signals u[nu] and a vector of
(time varying) Integer expressions expr[nu]. The output signal y is
set to expr[i], if i is the first element in the input vector u that is true. If all input signals are
false, y is set to parameter \"y_default\" or the last value is kept, if use_pre_as_default = <b>true</b>.
</p>

<blockquote><pre>
  // Conceptual equation (not valid Modelica)
  i = 'first element of u[:] that is true';
  y = <b>if</b> i==0 <b>then</b> (<b>if</b> use_pre_as_default <b>then</b> pre(y)
                                          <b>else</b> y_default)
      <b>else</b> expr[i];
</pre></blockquote>

<p>
The input connector is a vector of Boolean input signals.
When a connection line is drawn, the dimension of the input
vector is enlarged by one and the connection is automatically
connected to this new free index (thanks to the
connectorSizing annotation).
</p>

<p>
The usage is demonstrated, e.g., in example
<a href=\"modelica://Modelica.Blocks.Examples.IntegerNetwork1\">Modelica.Blocks.Examples.IntegerNetwork1</a>.
</p>

</html>"));
      end MultiSwitch;

      block Sum "Sum of Integers: y = k[1]*u[1] + k[2]*u[2] + ... + k[n]*u[n]"
        extends Modelica.Blocks.Interfaces.PartialIntegerMISO;
        parameter Integer k[nu] = fill(1, nu) "Input gains";
      equation
        if size(u, 1) > 0 then
          y = k * u;
        else
          y = 0;
        end if;
        annotation(
          Icon(graphics = {Text(extent = {{-200, -110}, {200, -140}}, lineColor = {0, 0, 0}, fillColor = {255, 213, 170}, fillPattern = FillPattern.Solid, textString = "%k"), Text(extent = {{-72, 68}, {92, -68}}, lineColor = {0, 0, 0}, fillColor = {255, 213, 170}, fillPattern = FillPattern.Solid, textString = "+")}),
          Documentation(info = "<html>
<p>
This blocks computes the scalar Integer output \"y\" as sum of the elements of the
Integer input signal vector u:
</p>
<blockquote><pre>
y = k[1]*u[1] + k[2]*u[2] + ... k[N]*u[N];
</pre></blockquote>

<p>
The input connector is a vector of Integer input signals.
When a connection line is drawn, the dimension of the input
vector is enlarged by one and the connection is automatically
connected to this new free index (thanks to the
connectorSizing annotation).
</p>

<p>
The usage is demonstrated, e.g., in example
<a href=\"modelica://Modelica.Blocks.Examples.IntegerNetwork1\">Modelica.Blocks.Examples.IntegerNetwork1</a>.
</p>

<p>
If no connection to the input connector \"u\" is present,
the output is set to zero: y=0.
</p>
</html>"));
      end Sum;

      block Product "Product of Integer: y = u[1]*u[2]* ... *u[n]"
        extends Modelica.Blocks.Interfaces.PartialIntegerMISO;
      equation
        if size(u, 1) > 0 then
          y = product(u);
        else
          y = 0;
        end if;
        annotation(
          Icon(graphics = {Text(extent = {{-74, 50}, {94, -94}}, lineColor = {0, 0, 0}, fillColor = {255, 213, 170}, fillPattern = FillPattern.Solid, textString = "*")}),
          Documentation(info = "<html>
<p>
This blocks computes the scalar Integer output \"y\" as product of the elements of the
Integer input signal vector u:
</p>
<blockquote><pre>
y = u[1]*u[2]* ... *u[N];
</pre></blockquote>

<p>
The input connector is a vector of Integer input signals.
When a connection line is drawn, the dimension of the input
vector is enlarged by one and the connection is automatically
connected to this new free index (thanks to the
connectorSizing annotation).
</p>

<p>
The usage is demonstrated, e.g., in example
<a href=\"modelica://Modelica.Blocks.Examples.IntegerNetwork1\">Modelica.Blocks.Examples.IntegerNetwork1</a>.
</p>

<p>
If no connection to the input connector \"u\" is present,
the output is set to zero: y=0.
</p>
</html>"));
      end Product;

      block TriggeredAdd "Add input to previous value of output, if rising edge of trigger port"
        extends Modelica.Blocks.Interfaces.PartialIntegerSISO;
        parameter Boolean use_reset = false "=true, if reset port enabled" annotation(
          Evaluate = true,
          HideResult = true,
          choices(checkBox = true));
        parameter Boolean use_set = false "=true, if set port enabled and used as default value when reset" annotation(
          Evaluate = true,
          HideResult = true,
          choices(checkBox = true));
        parameter Integer y_start = 0 "Initial and reset value of y if set port is not used";
        Modelica.Blocks.Interfaces.BooleanInput trigger annotation(
          Placement(transformation(extent = {{-20, -20}, {20, 20}}, rotation = 90, origin = {-60, -120})));
        Modelica.Blocks.Interfaces.BooleanInput reset if use_reset annotation(
          Placement(transformation(extent = {{-20, -20}, {20, 20}}, rotation = 90, origin = {60, -120})));
        Modelica.Blocks.Interfaces.IntegerInput set if use_set annotation(
          Placement(transformation(extent = {{-20, -20}, {20, 20}}, rotation = 270, origin = {60, 120}), iconTransformation(extent = {{-20, -20}, {20, 20}}, rotation = 270, origin = {28, 98})));
      protected
        Modelica.Blocks.Interfaces.BooleanOutput local_reset annotation(
          HideResult = true);
        Modelica.Blocks.Interfaces.IntegerOutput local_set;
      initial equation
        pre(y) = y_start;
      equation
        if use_reset then
          connect(reset, local_reset);
          if use_set then
            connect(set, local_set);
          else
            local_set = y_start;
          end if;
        else
          local_reset = false;
          local_set = 0;
        end if;
        when {trigger, local_reset} then
          y = if local_reset then local_set else pre(y) + u;
        end when;
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}, initialScale = 0.06), graphics = {Line(points = {{-100, 0}, {32, 76}}, color = {255, 128, 0}, smooth = Smooth.None, pattern = LinePattern.Dot), Line(points = {{-100, 0}, {32, -20}}, color = {255, 128, 0}, pattern = LinePattern.Dot, smooth = Smooth.None), Line(points = {{-54, -56}, {-26, -56}, {-26, -20}, {32, -20}, {32, 76}}, color = {0, 0, 0}, smooth = Smooth.None), Line(points = {{-60, -100}, {32, -20}}, color = {255, 0, 255}, pattern = LinePattern.Dot, smooth = Smooth.None), Text(visible = use_reset, extent = {{-28, -62}, {94, -86}}, lineColor = {0, 0, 0}, textString = "reset")}),
          Documentation(info = "<html>
<p>
Add input to previous value of output, if rising edge of trigger port

<p>
This block has one Integer input \"u\", one Boolean input \"trigger\",
an optional Boolean input \"reset\", an optional Integer input \"set\", and
an Integer output \"y\".
The optional inputs can be activated with the \"use_reset\" and
\"use_set\" flags, respectively.
</p>

<p>
The input \"u\" is added to the previous value of the
output \"y\" if the \"trigger\" port has a rising edge. At the start of the
simulation \"y = y_start\".
</p>

<p>
If the \"reset\" port is enabled, then the output \"y\" is reset to \"set\"
or to \"y_start\" (if the \"set\" port is not enabled), whenever the \"reset\"
port has a rising edge.
</p>

<p>
The usage is demonstrated, e.g., in example
<a href=\"modelica://Modelica.Blocks.Examples.IntegerNetwork1\">Modelica.Blocks.Examples.IntegerNetwork1</a>.
</p>

</html>"));
      end TriggeredAdd;
      annotation(
        Documentation(info = "<html>
<p>
This package contains basic <b>mathematical operations</b>
on <b>Integer</b> signals.
</p>
</html>"),
        Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-74, -66}, {-46, -66}, {-46, -30}, {12, -30}, {12, 66}}, color = {255, 128, 0}), Line(points = {{12, 66}, {70, 66}}, color = {255, 128, 0})}));
    end MathInteger;

    package MathBoolean "Library of Boolean mathematical functions as input/output blocks"
      extends Modelica.Icons.Package;

      block MultiSwitch "Set Boolean expression that is associated with the first active input signal"
        input Boolean expr[nu] = fill(false, nu) "y = if u[i] then expr[i] else y_default (time varying)" annotation(
          Dialog);
        parameter Boolean use_pre_as_default = true "set true to hold last value as default (y_default = pre(y))" annotation(
          HideResult = true,
          choices(checkBox = true));
        parameter Boolean y_default = false "Default value of output y if all u[i] = false" annotation(
          Dialog(enable = not use_pre_as_default));
        parameter Integer nu(min = 0) = 0 "Number of input connections" annotation(
          Dialog(connectorSizing = true),
          HideResult = true);
        Modelica.Blocks.Interfaces.BooleanVectorInput u[nu] "Set y = expr[i], if u[i] = true" annotation(
          Placement(transformation(extent = {{-110, 30}, {-90, -30}})));
        Modelica.Blocks.Interfaces.BooleanOutput y "Output depending on expression" annotation(
          Placement(transformation(extent = {{300, -10}, {320, 10}})));
      protected
        Integer firstActiveIndex;
      initial equation
        pre(y) = y_default;
      equation
        firstActiveIndex = Modelica.Math.BooleanVectors.firstTrueIndex(u);
        y = if firstActiveIndex == 0 then if use_pre_as_default then pre(y) else y_default else expr[firstActiveIndex];
        annotation(
          defaultComponentName = "set1",
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {300, 100}}), graphics = {Text(visible = not use_pre_as_default, extent = {{-100, -60}, {300, -90}}, lineColor = {0, 0, 0}, textString = "else: %y_default"), Text(visible = use_pre_as_default, extent = {{-100, -60}, {300, -90}}, lineColor = {0, 0, 0}, textString = "else: pre(y)"), Text(extent = {{-99, 99}, {300, 59}}, textString = "%name", lineColor = {0, 0, 255}), Rectangle(extent = {{-100, -51}, {300, 50}}, lineColor = {255, 127, 0}, lineThickness = 5.0, fillColor = {210, 210, 210}, fillPattern = FillPattern.Solid, borderPattern = BorderPattern.Raised), Text(extent = {{-84, 16}, {273, -15}}, lineColor = {0, 0, 0}, fillColor = {255, 246, 238}, fillPattern = FillPattern.Solid, textString = "%expr"), Ellipse(extent = {{275, 8}, {289, -6}}, lineColor = DynamicSelect({235, 235, 235}, if y > 0.5 then {0, 255, 0} else {235, 235, 235}), fillColor = DynamicSelect({235, 235, 235}, if y > 0.5 then {0, 255, 0} else {235, 235, 235}), fillPattern = FillPattern.Solid)}),
          Documentation(info = "<html>
<p>
The block has a vector of Boolean input signals u[nu] and a vector of
(time varying) Boolean expressions expr[:]. The output signal y is
set to expr[i], if i is the first element in the input vector u that is true.
If all input signals are false, y is set to parameter \"y_default\" or the
previous value of y is kept if parameter use_pre_as_default = <b>true</b>:
</p>

<blockquote><pre>
  // Conceptual equation (not valid Modelica)
  i = 'first element of u[:] that is true';
  y = <b>if</b> i==0 <b>then</b> (<b>if</b> use_pre_as_default <b>then</b> pre(y)
                                          <b>else</b> y_default)
      <b>else</b> expr[i];
</pre></blockquote>

<p>
The usage is demonstrated, e.g., in example
<a href=\"modelica://Modelica.Blocks.Examples.BooleanNetwork1\">Modelica.Blocks.Examples.BooleanNetwork1</a>.
</p>

</html>"));
      end MultiSwitch;

      block And "Logical 'and': y = u[1] and u[2] and ... and u[nu]"
        extends Modelica.Blocks.Interfaces.PartialBooleanMISO;
      equation
        y = Modelica.Math.BooleanVectors.allTrue(u);
        annotation(
          defaultComponentName = "and1",
          Icon(graphics = {Text(extent = {{-76, 40}, {60, -40}}, lineColor = {0, 0, 0}, textString = "and")}),
          Documentation(info = "<html>
<p>
The output is <b>true</b> if all inputs are <b>true</b>, otherwise
the output is <b>false</b>.
</p>

<p>
The input connector is a vector of Boolean input signals.
When a connection line is drawn, the dimension of the input
vector is enlarged by one and the connection is automatically
connected to this new free index (thanks to the
connectorSizing annotation).
</p>

<p>
The usage is demonstrated, e.g., in example
<a href=\"modelica://Modelica.Blocks.Examples.BooleanNetwork1\">Modelica.Blocks.Examples.BooleanNetwork1</a>.
</p>

<p>
If no connection to the input connector \"u\" is present,
the output is set to <b>false</b>: y=false.
</p>
</html>"));
      end And;

      block Or "Logical 'or': y = u[1] or u[2] or ... or u[nu]"
        extends Modelica.Blocks.Interfaces.PartialBooleanMISO;
      equation
        y = Modelica.Math.BooleanVectors.anyTrue(u);
        annotation(
          defaultComponentName = "or1",
          Icon(graphics = {Text(extent = {{-80, 40}, {60, -40}}, lineColor = {0, 0, 0}, textString = "or")}),
          Documentation(info = "<html>
<p>
The output is <b>true</b> if at least one input is <b>true</b>, otherwise
the output is <b>false</b>.
</p>

<p>
The input connector is a vector of Boolean input signals.
When a connection line is drawn, the dimension of the input
vector is enlarged by one and the connection is automatically
connected to this new free index (thanks to the
connectorSizing annotation).
</p>

<p>
The usage is demonstrated, e.g., in example
<a href=\"modelica://Modelica.Blocks.Examples.BooleanNetwork1\">Modelica.Blocks.Examples.BooleanNetwork1</a>.
</p>

<p>
If no connection to the input connector \"u\" is present,
the output is set to <b>false</b>: y=false.
</p>

</html>"));
      end Or;

      block Xor "Logical 'xor': y = oneTrue(u)  (y is true, if exactly one element of u is true, otherwise it is false)"
        extends Modelica.Blocks.Interfaces.PartialBooleanMISO;
      equation
        y = Modelica.Math.BooleanVectors.oneTrue(u);
        annotation(
          defaultComponentName = "xor1",
          Icon(graphics = {Text(extent = {{-80, 40}, {60, -40}}, lineColor = {0, 0, 0}, textString = "xor")}),
          Documentation(info = "<html>
<p>
The output is <b>true</b> if exactly one input is <b>true</b>, otherwise
the output is <b>false</b>.
</p>

<p>
The input connector is a vector of Boolean input signals.
When a connection line is drawn, the dimension of the input
vector is enlarged by one and the connection is automatically
connected to this new free index (thanks to the
connectorSizing annotation).
</p>

<p>
The usage is demonstrated, e.g., in example
<a href=\"modelica://Modelica.Blocks.Examples.BooleanNetwork1\">Modelica.Blocks.Examples.BooleanNetwork1</a>.
</p>

<p>
If no connection to the input connector \"u\" is present,
the output is set to <b>false</b>: y=false.
</p>

</html>"));
      end Xor;

      block Nand "Logical 'nand': y = not ( u[1] and u[2] and ... and u[nu] )"
        extends Modelica.Blocks.Interfaces.PartialBooleanMISO;
      equation
        y = not Modelica.Math.BooleanVectors.allTrue(u);
        annotation(
          defaultComponentName = "nand1",
          Icon(graphics = {Text(extent = {{-78, 36}, {64, -30}}, lineColor = {0, 0, 0}, textString = "nand")}),
          Documentation(info = "<html>
<p>
The output is <b>true</b> if at least one input is <b>false</b>, otherwise
the output is <b>false</b>.
</p>

<p>
The input connector is a vector of Boolean input signals.
When a connection line is drawn, the dimension of the input
vector is enlarged by one and the connection is automatically
connected to this new free index (thanks to the
connectorSizing annotation).
</p>

<p>
The usage is demonstrated, e.g., in example
<a href=\"modelica://Modelica.Blocks.Examples.BooleanNetwork1\">Modelica.Blocks.Examples.BooleanNetwork1</a>.
</p>

<p>
If no connection to the input connector \"u\" is present,
the output is set to <b>false</b>: y=false.
</p>

</html>"));
      end Nand;

      block Nor "Logical 'nor': y = not ( u[1] or u[2] or ... or u[nu] )"
        extends Modelica.Blocks.Interfaces.PartialBooleanMISO;
      equation
        y = not Modelica.Math.BooleanVectors.anyTrue(u);
        annotation(
          defaultComponentName = "nor1",
          Icon(graphics = {Text(extent = {{-80, 40}, {60, -40}}, lineColor = {0, 0, 0}, textString = "nor")}),
          Documentation(info = "<html>
<p>
The output is <b>false</b> if at least one input is <b>true</b>, otherwise
the output is <b>true</b>.
</p>

<p>
The input connector is a vector of Boolean input signals.
When a connection line is drawn, the dimension of the input
vector is enlarged by one and the connection is automatically
connected to this new free index (thanks to the
connectorSizing annotation).
</p>

<p>
The usage is demonstrated, e.g., in example
<a href=\"modelica://Modelica.Blocks.Examples.BooleanNetwork1\">Modelica.Blocks.Examples.BooleanNetwork1</a>.
</p>

<p>
If no connection to the input connector \"u\" is present,
the output is set to <b>false</b>: y=false.
</p>
</html>"));
      end Nor;

      block Not "Logical 'not': y = not u"
        extends Modelica.Blocks.Interfaces.PartialBooleanSISO_small;
      equation
        y = not u;
        annotation(
          defaultComponentName = "not1",
          Icon(graphics = {Text(extent = {{-98, 40}, {42, -40}}, lineColor = {0, 0, 0}, textString = "not")}),
          Documentation(info = "<html>
<p>
The output is <b>false</b> if at least one input is <b>true</b>, otherwise
the output is <b>true</b>.
</p>

<p>
The input connector is a vector of Boolean input signals.
When a connection line is drawn, the dimension of the input
vector is enlarged by one and the connection is automatically
connected to this new free index (thanks to the
connectorSizing annotation).
</p>

<p>
The usage is demonstrated, e.g., in example
<a href=\"modelica://Modelica.Blocks.Examples.BooleanNetwork1\">Modelica.Blocks.Examples.BooleanNetwork1</a>.
</p>
</html>"));
      end Not;

      block RisingEdge "Output y is true, if the input u has a rising edge, otherwise it is false (y = edge(u))"
        parameter Boolean pre_u_start = false "Value of pre(u) at initial time";
        extends Modelica.Blocks.Interfaces.PartialBooleanSISO_small;
      initial equation
        pre(u) = pre_u_start;
      equation
        y = edge(u);
        annotation(
          defaultComponentName = "rising1",
          Icon(graphics = {Line(points = {{-80, -68}, {-36, -68}, {-36, -24}, {22, -24}, {22, -68}, {66, -68}}, color = {0, 0, 0}), Line(points = {{-80, 32}, {-36, 32}, {-36, 76}, {-36, 76}, {-36, 32}, {66, 32}}, color = {255, 0, 255})}),
          Documentation(info = "<html>
<p>
A rising edge of the Boolean input u results in y = <b>true</b> at this
time instant. At all other time instants, y = <b>false</b>.
</p>

<p>
The usage is demonstrated, e.g., in example
<a href=\"modelica://Modelica.Blocks.Examples.BooleanNetwork1\">Modelica.Blocks.Examples.BooleanNetwork1</a>.
</p>

</html>"));
      end RisingEdge;

      block FallingEdge "Output y is true, if the input u has a falling edge, otherwise it is false (y = edge(not u))"
        parameter Boolean pre_u_start = false "Value of pre(u) at initial time";
        extends Modelica.Blocks.Interfaces.PartialBooleanSISO_small;
      protected
        Boolean not_u = not u annotation(
          HideResult = true);
      initial equation
        pre(not_u) = not pre_u_start;
      equation
        y = edge(not_u);
        annotation(
          defaultComponentName = "falling1",
          Icon(graphics = {Line(points = {{-80, -68}, {-36, -68}, {-36, -24}, {22, -24}, {22, -68}, {66, -68}}, color = {0, 0, 0}), Line(points = {{-80, 32}, {24, 32}, {24, 76}, {24, 76}, {24, 32}, {66, 32}}, color = {255, 0, 255})}),
          Documentation(info = "<html>
<p>
A falling edge of the Boolean input u results in y = <b>true</b> at this
time instant. At all other time instants, y = <b>false</b>.
</p>

<p>
The usage is demonstrated, e.g., in example
<a href=\"modelica://Modelica.Blocks.Examples.BooleanNetwork1\">Modelica.Blocks.Examples.BooleanNetwork1</a>.
</p>

</html>"));
      end FallingEdge;

      block ChangingEdge "Output y is true, if the input u has either a rising or a falling edge and otherwise it is false (y=change(u))"
        parameter Boolean pre_u_start = false "Value of pre(u) at initial time";
        extends Modelica.Blocks.Interfaces.PartialBooleanSISO_small;
      initial equation
        pre(u) = pre_u_start;
      equation
        y = change(u);
        annotation(
          defaultComponentName = "changing1",
          Icon(graphics = {Line(points = {{-80, -68}, {-36, -68}, {-36, -24}, {22, -24}, {22, -68}, {66, -68}}, color = {0, 0, 0}), Line(points = {{-80, 32}, {-36, 32}, {-36, 76}, {-36, 76}, {-36, 32}, {66, 32}}, color = {255, 0, 255}), Line(points = {{24, 32}, {24, 76}}, color = {255, 0, 255}, smooth = Smooth.None)}),
          Documentation(info = "<html>
<p>
A changing edge, i.e., either rising or falling,
of the Boolean input u results in y = <b>true</b> at this
time instant. At all other time instants, y = <b>false</b>.
</p>

<p>
The usage is demonstrated, e.g., in example
<a href=\"modelica://Modelica.Blocks.Examples.BooleanNetwork1\">Modelica.Blocks.Examples.BooleanNetwork1</a>.
</p>

</html>"));
      end ChangingEdge;

      block OnDelay "Delay a rising edge of the input, but do not delay a falling edge."
        extends Modelica.Blocks.Interfaces.PartialBooleanSISO_small;
        parameter Modelica.SIunits.Time delayTime "Delay time";
      protected
        Boolean delaySignal(start = false, fixed = true);
        discrete Modelica.SIunits.Time t_next;
      initial equation
        pre(u) = false;
        pre(t_next) = time - 1;
      algorithm
        when initial() then
          delaySignal := u;
          t_next := time - 1;
        elsewhen u then
          delaySignal := true;
          t_next := time + delayTime;
        elsewhen not u then
          delaySignal := false;
          t_next := time - 1;
        end when;
      equation
        if delaySignal then
          y = time >= t_next;
        else
          y = false;
        end if;
        annotation(
          Icon(graphics = {Text(extent = {{-250, -120}, {250, -150}}, lineColor = {0, 0, 0}, textString = "%delayTime s"), Line(points = {{-80, -66}, {-60, -66}, {-60, -22}, {38, -22}, {38, -66}, {66, -66}}, color = {0, 0, 0}), Line(points = {{-80, 32}, {-4, 32}, {-4, 76}, {38, 76}, {38, 32}, {66, 32}}, color = {255, 0, 255})}),
          Documentation(info = "<html>
<p>
A rising edge of the Boolean input u gives a delayed output.
A falling edge of the input is immediately given to the output.
</p>

<p>
Simulation results of a typical example with a delay time of 0.1 s
is shown in the next figure.
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Blocks/MathBoolean/OnDelay1.png\"
     alt=\"OnDelay1.png\">
<br>
<img src=\"modelica://Modelica/Resources/Images/Blocks/MathBoolean/OnDelay2.png\"
     alt=\"OnDelay2.png\">
</p>

<p>
The usage is demonstrated, e.g., in example
<a href=\"modelica://Modelica.Blocks.Examples.BooleanNetwork1\">Modelica.Blocks.Examples.BooleanNetwork1</a>.
</p>

</html>"));
      end OnDelay;
      annotation(
        Documentation(info = "<html>
<p>
This package contains basic <b>mathematical operations</b>
on <b>Boolean</b> signals.
</p>

<p>
Package MathBoolean is a new design that shall replace in the future the
<a href=\"modelica://Modelica.Blocks.Logical\">Logical</a> package.
The new features are:
</p>

<ul>
<li> If useful, blocks may have an arbitrary number of inputs (e.g., \"And\" block with 2,3,4,...
     Boolean inputs). This is based on the \"connectorSizing\" annotation which
     allows a tool to conveniently handle vectors of connectors. </li>

<li> The blocks are smaller in size, so that the diagram area is better
     utilized for trivial blocks such as \"And\" or \"Or\".</li>

</ul>

</html>"),
        Icon(graphics = {Line(points = {{-80, -16}, {-4, -16}, {-4, 28}, {38, 28}, {38, -16}, {66, -16}}, color = {255, 0, 255})}));
    end MathBoolean;

    package Nonlinear "Library of discontinuous or non-differentiable algebraic control blocks"
      import Modelica.Blocks.Interfaces;
      extends Modelica.Icons.Package;

      block Limiter "Limit the range of a signal"
        parameter Real uMax(start = 1) "Upper limits of input signals";
        parameter Real uMin = -uMax "Lower limits of input signals";
        parameter Boolean strict = false "= true, if strict limits with noEvent(..)" annotation(
          Evaluate = true,
          choices(checkBox = true),
          Dialog(tab = "Advanced"));
        parameter Boolean limitsAtInit = true "= false, if limits are ignored during initialization (i.e., y=u)" annotation(
          Evaluate = true,
          choices(checkBox = true),
          Dialog(tab = "Advanced"));
        extends Interfaces.SISO;
      equation
        assert(uMax >= uMin, "Limiter: Limits must be consistent. However, uMax (=" + String(uMax) + ") < uMin (=" + String(uMin) + ")");
        if initial() and not limitsAtInit then
          y = u;
          assert(u >= uMin - 0.01 * abs(uMin) and u <= uMax + 0.01 * abs(uMax), "Limiter: During initialization the limits have been ignored.\n" + "However, the result is that the input u is not within the required limits:\n" + "  u = " + String(u) + ", uMin = " + String(uMin) + ", uMax = " + String(uMax));
        elseif strict then
          y = smooth(0, noEvent(if u > uMax then uMax else if u < uMin then uMin else u));
        else
          y = smooth(0, if u > uMax then uMax else if u < uMin then uMin else u);
        end if;
        annotation(
          Documentation(info = "<html>
<p>
The Limiter block passes its input signal as output signal
as long as the input is within the specified upper and lower
limits. If this is not the case, the corresponding limits are passed
as output.
</p>
</html>"),
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{0, -90}, {0, 68}}, color = {192, 192, 192}), Polygon(points = {{0, 90}, {-8, 68}, {8, 68}, {0, 90}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-90, 0}, {68, 0}}, color = {192, 192, 192}), Polygon(points = {{90, 0}, {68, -8}, {68, 8}, {90, 0}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-80, -70}, {-50, -70}, {50, 70}, {80, 70}}, color = {0, 0, 0}), Text(extent = {{-150, -150}, {150, -110}}, lineColor = {0, 0, 0}, textString = "uMax=%uMax"), Text(extent = {{-150, 150}, {150, 110}}, textString = "%name", lineColor = {0, 0, 255}), Line(visible = strict, points = {{50, 70}, {80, 70}}, color = {255, 0, 0}, smooth = Smooth.None), Line(visible = strict, points = {{-80, -70}, {-50, -70}}, color = {255, 0, 0}, smooth = Smooth.None)}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{0, -60}, {0, 50}}, color = {192, 192, 192}), Polygon(points = {{0, 60}, {-5, 50}, {5, 50}, {0, 60}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-60, 0}, {50, 0}}, color = {192, 192, 192}), Polygon(points = {{60, 0}, {50, -5}, {50, 5}, {60, 0}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-50, -40}, {-30, -40}, {30, 40}, {50, 40}}, color = {0, 0, 0}), Text(extent = {{46, -6}, {68, -18}}, lineColor = {128, 128, 128}, textString = "u"), Text(extent = {{-30, 70}, {-5, 50}}, lineColor = {128, 128, 128}, textString = "y"), Text(extent = {{-58, -54}, {-28, -42}}, lineColor = {128, 128, 128}, textString = "uMin"), Text(extent = {{26, 40}, {66, 56}}, lineColor = {128, 128, 128}, textString = "uMax")}));
      end Limiter;

      block VariableLimiter "Limit the range of a signal with variable limits"
        extends Interfaces.SISO;
        parameter Boolean strict = false "= true, if strict limits with noEvent(..)" annotation(
          Evaluate = true,
          choices(checkBox = true));
        parameter Boolean limitsAtInit = true "= false, if limits are ignored during initialization (i.e., y=u)" annotation(
          Evaluate = true,
          choices(checkBox = true));
        Interfaces.RealInput limit1 "Connector of Real input signal used as maximum of input u" annotation(
          Placement(transformation(extent = {{-140, 60}, {-100, 100}}, rotation = 0)));
        Interfaces.RealInput limit2 "Connector of Real input signal used as minimum of input u" annotation(
          Placement(transformation(extent = {{-140, -100}, {-100, -60}}, rotation = 0)));
      protected
        Real uMax;
        Real uMin;
      equation
        if strict then
          uMax = noEvent(max(limit1, limit2));
          uMin = noEvent(min(limit1, limit2));
        else
          uMax = max(limit1, limit2);
          uMin = min(limit1, limit2);
        end if;
        if initial() and not limitsAtInit then
          y = u;
          assert(u >= uMin - 0.01 * abs(uMin) and u <= uMax + 0.01 * abs(uMax), "VariableLimiter: During initialization the limits have been ignored.\n" + "However, the result is that the input u is not within the required limits:\n" + "  u = " + String(u) + ", uMin = " + String(uMin) + ", uMax = " + String(uMax));
        elseif strict then
          y = smooth(0, noEvent(if u > uMax then uMax else if u < uMin then uMin else u));
        else
          y = smooth(0, if u > uMax then uMax else if u < uMin then uMin else u);
        end if;
        annotation(
          Documentation(info = "<html>
<p>
The Limiter block passes its input signal as output signal
as long as the input is within the upper and lower
limits specified by the two additional inputs limit1 and
limit2. If this is not the case, the corresponding limit
is passed as output.
</p>
</html>"),
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{0, -90}, {0, 68}}, color = {192, 192, 192}), Line(points = {{-90, 0}, {68, 0}}, color = {192, 192, 192}), Polygon(points = {{90, 0}, {68, -8}, {68, 8}, {90, 0}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-80, -70}, {-50, -70}, {50, 70}, {80, 70}}, color = {0, 0, 0}), Text(extent = {{-150, 150}, {150, 110}}, textString = "%name", lineColor = {0, 0, 255}), Line(points = {{-100, 80}, {66, 80}, {66, 70}}, color = {0, 0, 127}), Line(points = {{-100, -80}, {-64, -80}, {-64, -70}}, color = {0, 0, 127}), Polygon(points = {{-64, -70}, {-66, -74}, {-62, -74}, {-64, -70}}, lineColor = {0, 0, 127}), Polygon(points = {{66, 70}, {64, 74}, {68, 74}, {66, 70}}, lineColor = {0, 0, 127}), Polygon(points = {{0, 90}, {-8, 68}, {8, 68}, {0, 90}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(visible = strict, points = {{50, 70}, {80, 70}}, color = {255, 0, 0}, smooth = Smooth.None), Line(visible = strict, points = {{-80, -70}, {-50, -70}}, color = {255, 0, 0}, smooth = Smooth.None)}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{0, -60}, {0, 50}}, color = {192, 192, 192}), Polygon(points = {{0, 60}, {-5, 50}, {5, 50}, {0, 60}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-60, 0}, {50, 0}}, color = {192, 192, 192}), Polygon(points = {{60, 0}, {50, -5}, {50, 5}, {60, 0}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-50, -40}, {-30, -40}, {30, 40}, {50, 40}}, color = {0, 0, 0}), Text(extent = {{46, -6}, {68, -18}}, lineColor = {128, 128, 128}, textString = "inPort"), Text(extent = {{-30, 70}, {-5, 50}}, lineColor = {128, 128, 128}, textString = "outPort"), Text(extent = {{-66, -40}, {-26, -20}}, lineColor = {128, 128, 128}, textString = "uMin"), Text(extent = {{30, 20}, {70, 40}}, lineColor = {128, 128, 128}, textString = "uMax"), Line(points = {{-100, 80}, {40, 80}, {40, 40}}, color = {0, 0, 127}), Line(points = {{-100, -80}, {-40, -80}, {-40, -40}}, color = {0, 0, 127}), Polygon(points = {{40, 40}, {35, 50}, {45, 50}, {40, 40}}, lineColor = {0, 0, 127}), Polygon(points = {{-40, -40}, {-45, -50}, {-35, -50}, {-40, -40}}, lineColor = {0, 0, 127})}));
      end VariableLimiter;

      block SlewRateLimiter "Limits the slew rate of a signal"
        extends Modelica.Blocks.Interfaces.SISO;
        import Modelica.Constants.small;
        parameter Modelica.SIunits.DampingCoefficient Rising(min = small) = 1 "Maximum rising slew rate [+small..+inf)";
        parameter Modelica.SIunits.DampingCoefficient Falling(max = -small) = -Rising "Maximum falling slew rate (-inf..-small]";
        parameter Modelica.SIunits.Time Td(min = small) = 0.001 "Derivative time constant";
        parameter Boolean strict = false "= true, if strict limits with noEvent(..)" annotation(
          Evaluate = true,
          choices(checkBox = true),
          Dialog(tab = "Advanced"));
      initial equation
        y = u;
      equation
        if strict then
          der(y) = smooth(1, noEvent(min(max((u - y) / Td, Falling), Rising)));
        else
          der(y) = smooth(1, min(max((u - y) / Td, Falling), Rising));
        end if;
        annotation(
          Icon(graphics = {Line(points = {{-90, 0}, {68, 0}}, color = {192, 192, 192}), Line(points = {{0, -90}, {0, 68}}, color = {192, 192, 192}), Polygon(points = {{0, 90}, {-8, 68}, {8, 68}, {0, 90}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Polygon(points = {{90, 0}, {68, -8}, {68, 8}, {90, 0}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-50, -70}, {50, 70}}, color = {0, 0, 0}, smooth = Smooth.None), Line(visible = strict, points = {{50, 70}, {-50, -70}}, color = {255, 0, 0}, smooth = Smooth.None)}),
          Documentation(info = "<html>
<p>The <code>SlewRateLimiter</code> block limits the slew rate of its input signal in the range of <code>[Falling, Rising]</code>.</p>
<p>To ensure this for arbitrary inputs and in order to produce a differential output, the input is numerically differentiated
with derivative time constant <code>Td</code>. Smaller time constant <code>Td</code> means nearer ideal derivative.</p>
<p><em>Note: The user has to choose the derivative time constant according to the nature of the input signal.</em></p>
</html>", revisions = "<html>
<table cellspacing=\"0\" cellpadding=\"2\" border=\"1\"><tr>
<th>Revision</th>
<th>Date</th>
<th>Author</th>
<th>Comment</th>
</tr>
<tr>
<td valign=\"top\">4954</td>
<td valign=\"top\">2012-03-02</td>
<td valign=\"top\">A. Haumer &amp; D. Winkler</td>
<td valign=\"top\"><p>Initial version based on discussion in ticket <a href=\"https://trac.modelica.org/Modelica/ticket/529\">#529</a></p></td>
</tr>
</table>
</html>"));
      end SlewRateLimiter;

      block DeadZone "Provide a region of zero output"
        parameter Real uMax(start = 1) "Upper limits of dead zones";
        parameter Real uMin = -uMax "Lower limits of dead zones";
        parameter Boolean deadZoneAtInit = true "= false, if dead zone is ignored during initialization (i.e., y=u)";
        extends Interfaces.SISO;
      equation
        assert(uMax >= uMin, "DeadZone: Limits must be consistent. However, uMax (=" + String(uMax) + ") < uMin (=" + String(uMin) + ")");
        if initial() and not deadZoneAtInit then
          y = u;
        else
          y = smooth(0, if u > uMax then u - uMax else if u < uMin then u - uMin else 0);
        end if;
        annotation(
          Documentation(info = "<html>
<p>
The DeadZone block defines a region of zero output.
</p>
<p>
If the input is within uMin ... uMax, the output
is zero. Outside of this zone, the output is a linear
function of the input with a slope of 1.
</p>
</html>"),
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{0, -90}, {0, 68}}, color = {192, 192, 192}), Polygon(points = {{0, 90}, {-8, 68}, {8, 68}, {0, 90}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-90, 0}, {68, 0}}, color = {192, 192, 192}), Polygon(points = {{90, 0}, {68, -8}, {68, 8}, {90, 0}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-80, -60}, {-20, 0}, {20, 0}, {80, 60}}, color = {0, 0, 0}), Text(extent = {{-150, -150}, {150, -110}}, lineColor = {160, 160, 164}, textString = "uMax=%uMax"), Text(extent = {{-150, 150}, {150, 110}}, textString = "%name", lineColor = {0, 0, 255})}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{0, -60}, {0, 50}}, color = {192, 192, 192}), Polygon(points = {{0, 60}, {-5, 50}, {5, 50}, {0, 60}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-76, 0}, {74, 0}}, color = {192, 192, 192}), Polygon(points = {{84, 0}, {74, -5}, {74, 5}, {84, 0}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-81, -40}, {-38, 0}, {40, 0}, {80, 40}}, color = {0, 0, 0}), Text(extent = {{62, -7}, {88, -25}}, lineColor = {128, 128, 128}, textString = "u"), Text(extent = {{-36, 72}, {-5, 50}}, lineColor = {128, 128, 128}, textString = "y"), Text(extent = {{-51, 1}, {-28, 19}}, lineColor = {128, 128, 128}, textString = "uMin"), Text(extent = {{27, 21}, {52, 5}}, lineColor = {128, 128, 128}, textString = "uMax")}));
      end DeadZone;

      block FixedDelay "Delay block with fixed DelayTime"
        extends Modelica.Blocks.Interfaces.SISO;
        parameter SI.Time delayTime(start = 1) "Delay time of output with respect to input signal";
      equation
        y = delay(u, delayTime);
        annotation(
          Documentation(info = "<html>
<p>
The Input signal is delayed by a given time instant, or more precisely:
</p>
<pre>
   y = u(time - delayTime) for time &gt; time.start + delayTime
     = u(time.start)       for time &le; time.start + delayTime
</pre>
</html>"),
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100.0, -100.0}, {100.0, 100.0}}, initialScale = 0.1), graphics = {Text(extent = {{8.0, -142.0}, {8.0, -102.0}}, textString = "delayTime=%delayTime"), Line(points = {{-92.0, 0.0}, {-80.7, 34.2}, {-73.5, 53.1}, {-67.1, 66.4}, {-61.4, 74.6}, {-55.8, 79.1}, {-50.2, 79.8}, {-44.6, 76.6}, {-38.9, 69.7}, {-33.3, 59.4}, {-26.9, 44.1}, {-18.83, 21.2}, {-1.9, -30.8}, {5.3, -50.2}, {11.7, -64.2}, {17.3, -73.1}, {23.0, -78.4}, {28.6, -80.0}, {34.2, -77.6}, {39.9, -71.5}, {45.5, -61.9}, {51.9, -47.2}, {60.0, -24.8}, {68.0, 0.0}}, color = {0, 0, 127}, smooth = Smooth.Bezier), Line(points = {{-62.0, 0.0}, {-50.7, 34.2}, {-43.5, 53.1}, {-37.1, 66.4}, {-31.4, 74.6}, {-25.8, 79.1}, {-20.2, 79.8}, {-14.6, 76.6}, {-8.9, 69.7}, {-3.3, 59.4}, {3.1, 44.1}, {11.17, 21.2}, {28.1, -30.8}, {35.3, -50.2}, {41.7, -64.2}, {47.3, -73.1}, {53.0, -78.4}, {58.6, -80.0}, {64.2, -77.6}, {69.9, -71.5}, {75.5, -61.9}, {81.9, -47.2}, {90.0, -24.8}, {98.0, 0.0}}, color = {160, 160, 164}, smooth = Smooth.Bezier)}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-80, 80}, {-88, 80}}, color = {192, 192, 192}), Line(points = {{-80, -80}, {-88, -80}}, color = {192, 192, 192}), Line(points = {{-80, -88}, {-80, 86}}, color = {192, 192, 192}), Text(extent = {{-75, 98}, {-46, 78}}, lineColor = {0, 0, 255}, textString = "outPort"), Polygon(points = {{-80, 96}, {-86, 80}, {-74, 80}, {-80, 96}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-100, 0}, {84, 0}}, color = {192, 192, 192}), Polygon(points = {{100, 0}, {84, 6}, {84, -6}, {100, 0}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-80, 0}, {-68.7, 34.2}, {-61.5, 53.1}, {-55.1, 66.4}, {-49.4, 74.6}, {-43.8, 79.1}, {-38.2, 79.8}, {-32.6, 76.6}, {-26.9, 69.7}, {-21.3, 59.4}, {-14.9, 44.1}, {-6.83, 21.2}, {10.1, -30.8}, {17.3, -50.2}, {23.7, -64.2}, {29.3, -73.1}, {35, -78.4}, {40.6, -80}, {46.2, -77.6}, {51.9, -71.5}, {57.5, -61.9}, {63.9, -47.2}, {72, -24.8}, {80, 0}}, color = {0, 0, 127}, smooth = Smooth.Bezier), Text(extent = {{-24, 98}, {-2, 78}}, lineColor = {0, 0, 0}, textString = "inPort"), Line(points = {{-64, 0}, {-52.7, 34.2}, {-45.5, 53.1}, {-39.1, 66.4}, {-33.4, 74.6}, {-27.8, 79.1}, {-22.2, 79.8}, {-16.6, 76.6}, {-10.9, 69.7}, {-5.3, 59.4}, {1.1, 44.1}, {9.17, 21.2}, {26.1, -30.8}, {33.3, -50.2}, {39.7, -64.2}, {45.3, -73.1}, {51, -78.4}, {56.6, -80}, {62.2, -77.6}, {67.9, -71.5}, {73.5, -61.9}, {79.9, -47.2}, {88, -24.8}, {96, 0}}, color = {0, 0, 0}, smooth = Smooth.Bezier), Text(extent = {{67, 22}, {96, 6}}, lineColor = {160, 160, 164}, textString = "time"), Line(points = {{-64, -30}, {-64, 0}}, color = {192, 192, 192}), Text(extent = {{-58, -42}, {-58, -32}}, textString = "delayTime", lineColor = {0, 0, 255}), Line(points = {{-94, -26}, {-80, -26}}, color = {192, 192, 192}), Line(points = {{-64, -26}, {-50, -26}}, color = {192, 192, 192}), Polygon(points = {{-80, -26}, {-88, -24}, {-88, -28}, {-80, -26}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Polygon(points = {{-56, -24}, {-64, -26}, {-56, -28}, {-56, -24}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid)}));
      end FixedDelay;

      block PadeDelay "Pade approximation of delay block with fixed DelayTime"
        extends Modelica.Blocks.Interfaces.SISO;
        parameter SI.Time delayTime(start = 1) "Delay time of output with respect to input signal";
        parameter Integer n(min = 1) = 1 "Order of Pade approximation";
        parameter Integer m(min = 1, max = n) = n "Order of numerator";
      protected
        Real x1dot "Derivative of first state of TransferFcn";
        Real xn "Highest order state of TransferFcn";
        Real a[n + 1];
        Real b[m + 1];
      public
        final output Real x[n] "State of transfer function from controller canonical form";
      protected
        function padeCoefficients
          extends Modelica.Icons.Function;
          input Real T "delay time";
          input Integer n "order of denominator";
          input Integer m "order of numerator";
          output Real b[m + 1] "numerator coefficients of transfer function";
          output Real a[n + 1] "denominator coefficients of transfer function";
        protected
          Real nm;
        algorithm
          a[1] := 1;
          b[1] := 1;
          nm := n + m;
          for i in 1:n loop
            a[i + 1] := a[i] * (T * ((n - i + 1) / (nm - i + 1)) / i);
            if i <= m then
              b[i + 1] := -b[i] * (T * ((m - i + 1) / (nm - i + 1)) / i);
            end if;
          end for;
          b := b[m + 1:(-1):1];
          a := a[n + 1:(-1):1];
        end padeCoefficients;
      equation
        (b, a) = padeCoefficients(delayTime, n, m);
        [der(x); xn] = [x1dot; x];
        [u] = transpose([a]) * [x1dot; x];
        [y] = transpose([zeros(n - m, 1); b]) * [x1dot; x];
      initial equation
        x[n] = u;
        annotation(
          Documentation(info = "<html>
<p>
The Input signal is delayed by a given time instant, or more precisely:
</p>
<pre>
   y = u(time - delayTime) for time &gt; time.start + delayTime
     = u(time.start)       for time &le; time.start + delayTime
</pre>
<p>
The delay is approximated by a Pade approximation, i.e., by
a transfer function
</p>
<pre>
           b[1]*s^m + b[2]*s^[m-1] + ... + b[m+1]
   y(s) = --------------------------------------------- * u(s)
           a[1]*s^n + a[2]*s^[n-1] + ... + a[n+1]
</pre>
<p>
where the coefficients b[:] and a[:] are calculated such that the
coefficients of the Taylor expansion of the delay exp(-T*s) around s=0
are identical upto order n+m.
</p>
<p>
The main advantage of this approach is that the delay is
approximated by a linear differential equation system, which
is continuous and continuously differentiable. For example, it
is uncritical to linearize a system containing a Pade-approximated
delay.
</p>
<p>
The standard text book version uses order \"m=n\", which is
also the default setting of this block. The setting
\"m=n-1\" may yield a better approximation in certain cases.
</p>
<h5>Literature:</h5>
<p>Otto Foellinger: Regelungstechnik, 8. Auflage,
chapter 11.9, page 412-414, Huethig Verlag Heidelberg, 1994
</p>
</html>"),
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100.0, -100.0}, {100.0, 100.0}}, initialScale = 0.1), graphics = {Text(extent = {{8.0, -142.0}, {8.0, -102.0}}, textString = "delayTime=%delayTime"), Line(points = {{-94.0, 0.0}, {-82.7, 34.2}, {-75.5, 53.1}, {-69.1, 66.4}, {-63.4, 74.6}, {-57.8, 79.1}, {-52.2, 79.8}, {-46.6, 76.6}, {-40.9, 69.7}, {-35.3, 59.4}, {-28.9, 44.1}, {-20.83, 21.2}, {-3.9, -30.8}, {3.3, -50.2}, {9.7, -64.2}, {15.3, -73.1}, {21.0, -78.4}, {26.6, -80.0}, {32.2, -77.6}, {37.9, -71.5}, {43.5, -61.9}, {49.9, -47.2}, {58.0, -24.8}, {66.0, 0.0}}, color = {0, 0, 127}, smooth = Smooth.Bezier), Line(points = {{-72.0, 0.0}, {-60.7, 34.2}, {-53.5, 53.1}, {-47.1, 66.4}, {-41.4, 74.6}, {-35.8, 79.1}, {-30.2, 79.8}, {-24.6, 76.6}, {-18.9, 69.7}, {-13.3, 59.4}, {-6.9, 44.1}, {1.17, 21.2}, {18.1, -30.8}, {25.3, -50.2}, {31.7, -64.2}, {37.3, -73.1}, {43.0, -78.4}, {48.6, -80.0}, {54.2, -77.6}, {59.9, -71.5}, {65.5, -61.9}, {71.9, -47.2}, {80.0, -24.8}, {88.0, 0.0}}, color = {160, 160, 164}, smooth = Smooth.Bezier), Text(lineColor = {160, 160, 164}, extent = {{-10.0, 38.0}, {100.0, 100.0}}, textString = "m=%m"), Text(lineColor = {160, 160, 164}, extent = {{-98.0, -96.0}, {6.0, -34.0}}, textString = "n=%n")}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-80, 80}, {-88, 80}}, color = {192, 192, 192}), Line(points = {{-80, -80}, {-88, -80}}, color = {192, 192, 192}), Line(points = {{-80, -88}, {-80, 86}}, color = {192, 192, 192}), Text(extent = {{-75, 98}, {-46, 78}}, lineColor = {0, 0, 255}, textString = "outPort"), Polygon(points = {{-80, 96}, {-86, 80}, {-74, 80}, {-80, 96}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-100, 0}, {84, 0}}, color = {192, 192, 192}), Polygon(points = {{100, 0}, {84, 6}, {84, -6}, {100, 0}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-80, 0}, {-68.7, 34.2}, {-61.5, 53.1}, {-55.1, 66.4}, {-49.4, 74.6}, {-43.8, 79.1}, {-38.2, 79.8}, {-32.6, 76.6}, {-26.9, 69.7}, {-21.3, 59.4}, {-14.9, 44.1}, {-6.83, 21.2}, {10.1, -30.8}, {17.3, -50.2}, {23.7, -64.2}, {29.3, -73.1}, {35, -78.4}, {40.6, -80}, {46.2, -77.6}, {51.9, -71.5}, {57.5, -61.9}, {63.9, -47.2}, {72, -24.8}, {80, 0}}, color = {0, 0, 127}, smooth = Smooth.Bezier), Text(extent = {{-24, 98}, {-2, 78}}, lineColor = {0, 0, 0}, textString = "inPort"), Line(points = {{-64, 0}, {-52.7, 34.2}, {-45.5, 53.1}, {-39.1, 66.4}, {-33.4, 74.6}, {-27.8, 79.1}, {-22.2, 79.8}, {-16.6, 76.6}, {-10.9, 69.7}, {-5.3, 59.4}, {1.1, 44.1}, {9.17, 21.2}, {26.1, -30.8}, {33.3, -50.2}, {39.7, -64.2}, {45.3, -73.1}, {51, -78.4}, {56.6, -80}, {62.2, -77.6}, {67.9, -71.5}, {73.5, -61.9}, {79.9, -47.2}, {88, -24.8}, {96, 0}}, color = {0, 0, 0}, smooth = Smooth.Bezier), Text(extent = {{67, 22}, {96, 6}}, lineColor = {160, 160, 164}, textString = "time"), Line(points = {{-64, -30}, {-64, 0}}, color = {192, 192, 192}), Text(extent = {{-58, -42}, {-58, -32}}, textString = "delayTime", lineColor = {0, 0, 255}), Line(points = {{-94, -26}, {-80, -26}}, color = {192, 192, 192}), Line(points = {{-64, -26}, {-50, -26}}, color = {192, 192, 192}), Polygon(points = {{-80, -26}, {-88, -24}, {-88, -28}, {-80, -26}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Polygon(points = {{-56, -24}, {-64, -26}, {-56, -28}, {-56, -24}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid)}));
      end PadeDelay;

      block VariableDelay "Delay block with variable DelayTime"
        extends Modelica.Blocks.Interfaces.SISO;
        parameter Real delayMax(min = 0, start = 1) "maximum delay time";
        Modelica.Blocks.Interfaces.RealInput delayTime annotation(
          Placement(transformation(extent = {{-140, -80}, {-100, -40}}, rotation = 0)));
      equation
        y = delay(u, delayTime, delayMax);
        annotation(
          Documentation(info = "<html>
<p>
The Input signal is delayed by a given time instant, or more precisely:
</p>
<pre>
   y = u(time - delayTime) for time &gt; time.start + delayTime
     = u(time.start)       for time &le; time.start + delayTime
</pre>
<p>
where delayTime is an additional input signal which must follow
the following relationship:
</p>
<pre>  0 &le; delayTime &le; delayMax
</pre>
</html>"),
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100.0, -100.0}, {100.0, 100.0}}, initialScale = 0.1), graphics = {Text(extent = {{-100.0, -148.0}, {100.0, -108.0}}, textString = "delayMax=%delayMax"), Line(points = {{-92.0, 0.0}, {-80.7, 34.2}, {-73.5, 53.1}, {-67.1, 66.4}, {-61.4, 74.6}, {-55.8, 79.1}, {-50.2, 79.8}, {-44.6, 76.6}, {-38.9, 69.7}, {-33.3, 59.4}, {-26.9, 44.1}, {-18.83, 21.2}, {-1.9, -30.8}, {5.3, -50.2}, {11.7, -64.2}, {17.3, -73.1}, {23.0, -78.4}, {28.6, -80.0}, {34.2, -77.6}, {39.9, -71.5}, {45.5, -61.9}, {51.9, -47.2}, {60.0, -24.8}, {68.0, 0.0}}, color = {0, 0, 127}, smooth = Smooth.Bezier), Line(points = {{-64.0, 0.0}, {-52.7, 34.2}, {-45.5, 53.1}, {-39.1, 66.4}, {-33.4, 74.6}, {-27.8, 79.1}, {-22.2, 79.8}, {-16.6, 76.6}, {-10.9, 69.7}, {-5.3, 59.4}, {1.1, 44.1}, {9.17, 21.2}, {26.1, -30.8}, {33.3, -50.2}, {39.7, -64.2}, {45.3, -73.1}, {51.0, -78.4}, {56.6, -80.0}, {62.2, -77.6}, {67.9, -71.5}, {73.5, -61.9}, {79.9, -47.2}, {88.0, -24.8}, {96.0, 0.0}}, smooth = Smooth.Bezier), Polygon(fillPattern = FillPattern.Solid, lineColor = {0, 0, 127}, fillColor = {0, 0, 127}, points = {{6.0, 4.0}, {-14.0, -2.0}, {-6.0, -12.0}, {6.0, 4.0}}), Line(color = {0, 0, 127}, points = {{-100.0, -60.0}, {-76.0, -60.0}, {-8.0, -6.0}})}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, -100}, {100, 100}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Polygon(points = {{-80, 96}, {-86, 80}, {-74, 80}, {-80, 96}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Text(extent = {{-69, 98}, {-40, 78}}, lineColor = {0, 0, 255}, textString = "outPort"), Line(points = {{-64, 0}, {-52.7, 34.2}, {-45.5, 53.1}, {-39.1, 66.4}, {-33.4, 74.6}, {-27.8, 79.1}, {-22.2, 79.8}, {-16.6, 76.6}, {-10.9, 69.7}, {-5.3, 59.4}, {1.1, 44.1}, {9.17, 21.2}, {26.1, -30.8}, {33.3, -50.2}, {39.7, -64.2}, {45.3, -73.1}, {51, -78.4}, {56.6, -80}, {62.2, -77.6}, {67.9, -71.5}, {73.5, -61.9}, {79.9, -47.2}, {88, -24.8}, {96, 0}}, color = {0, 0, 0}, smooth = Smooth.Bezier), Line(points = {{-80, 0}, {-68.7, 34.2}, {-61.5, 53.1}, {-55.1, 66.4}, {-49.4, 74.6}, {-43.8, 79.1}, {-38.2, 79.8}, {-32.6, 76.6}, {-26.9, 69.7}, {-21.3, 59.4}, {-14.9, 44.1}, {-6.83, 21.2}, {10.1, -30.8}, {17.3, -50.2}, {23.7, -64.2}, {29.3, -73.1}, {35, -78.4}, {40.6, -80}, {46.2, -77.6}, {51.9, -71.5}, {57.5, -61.9}, {63.9, -47.2}, {72, -24.8}, {80, 0}}, color = {0, 0, 127}, smooth = Smooth.Bezier), Line(points = {{-100, 0}, {84, 0}}, color = {192, 192, 192}), Polygon(points = {{100, 0}, {84, 6}, {84, -6}, {100, 0}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Text(extent = {{67, 22}, {96, 6}}, lineColor = {160, 160, 164}, textString = "time"), Text(extent = {{-58, -42}, {-58, -32}}, textString = "delayTime", lineColor = {0, 0, 255}), Line(points = {{-80, -88}, {-80, 86}}, color = {192, 192, 192}), Text(extent = {{-24, 98}, {-2, 78}}, lineColor = {0, 0, 0}, textString = "inPort"), Polygon(points = {{-80, -26}, {-88, -24}, {-88, -28}, {-80, -26}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Polygon(points = {{-56, -24}, {-64, -26}, {-56, -28}, {-56, -24}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-64, -26}, {-50, -26}}, color = {192, 192, 192}), Line(points = {{-94, -26}, {-80, -26}}, color = {192, 192, 192}), Line(points = {{-100, -60}, {-70, -60}, {-64, -44}}, arrow = {Arrow.None, Arrow.Filled}, color = {0, 0, 127}), Line(points = {{-64, -30}, {-64, 0}}, color = {192, 192, 192})}));
      end VariableDelay;
      annotation(
        Documentation(info = "<html>
<p>
This package contains <b>discontinuous</b> and
<b>non-differentiable, algebraic</b> input/output blocks.
</p>
</html>", revisions = "<html>
<ul>
<li><i>October 21, 2002</i>
       by <a href=\"http://www.robotic.dlr.de/Christian.Schweiger/\">Christian Schweiger</a>:<br>
       New block VariableLimiter added.
<li><i>August 22, 1999</i>
       by <a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a>:<br>
       Realized, based on an existing Dymola library
       of Dieter Moormann and Hilding Elmqvist.
</li>
</ul>
</html>"),
        Icon(graphics = {Line(points = {{-80, -66}, {-26, -66}, {28, 52}, {88, 52}}, color = {95, 95, 95})}));
    end Nonlinear;

    package Routing "Library of blocks to combine and extract signals"
      extends Modelica.Icons.Package;

      block Replicator "Signal replicator"
        extends Modelica.Blocks.Interfaces.SIMO;
      equation
        y = fill(u, nout);
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-100, 0}, {-6, 0}}, color = {0, 0, 127}), Line(points = {{100, 0}, {10, 0}}, color = {0, 0, 127}), Line(points = {{0, 0}, {100, 10}}, color = {0, 0, 127}), Line(points = {{0, 0}, {100, -10}}, color = {0, 0, 127}), Ellipse(extent = {{-14, 16}, {16, -14}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 127}, fillPattern = FillPattern.Solid)}),
          Documentation(info = "<html>
<p>
This block replicates the input signal to an array of <code>nout</code> identical output signals.
</p>
</html>"));
      end Replicator;

      block ExtractSignal "Extract signals from an input signal vector"
        extends Modelica.Blocks.Interfaces.MIMO;
        parameter Integer extract[nout] = 1:nout "Extracting vector";
      equation
        for i in 1:nout loop
          y[i] = u[extract[i]];
        end for;
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-90, 51}, {-50, -49}}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 127}), Rectangle(extent = {{50, 50}, {90, -50}}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 127}), Polygon(points = {{-94.4104, 1.90792}, {-94.4104, -2.09208}, {-90.4104, -0.0920762}, {-94.4104, 1.90792}}, fillColor = {0, 0, 127}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 127}), Line(points = {{-72, 2}, {-60.1395, 12.907}, {-49.1395, 12.907}}, color = {0, 0, 127}), Line(points = {{-73, 4}, {-59, 40}, {-49, 40}}, color = {0, 0, 127}), Line(points = {{-113, 0}, {-76.0373, -0.0180176}}, color = {0, 0, 127}), Ellipse(extent = {{-81.0437, 4.59255}, {-71.0437, -4.90745}}, fillColor = {0, 0, 127}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 127}), Line(points = {{-73, -5}, {-60, -40}, {-49, -40}}, color = {0, 0, 127}), Line(points = {{-72, -2}, {-60.0698, -12.907}, {-49.0698, -12.907}}, color = {0, 0, 127}), Polygon(points = {{-48.8808, -11}, {-48.8808, -15}, {-44.8808, -13}, {-48.8808, -11}}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 127}), Line(points = {{-46, 13}, {-35, 13}, {35, -30}, {45, -30}}, color = {0, 0, 127}), Line(points = {{-45, 40}, {-35, 40}, {35, 0}, {44, 0}}, color = {0, 0, 127}), Line(points = {{-45, -40}, {-34, -40}, {35, 30}, {44, 30}}, color = {0, 0, 127}), Polygon(points = {{-49, 42}, {-49, 38}, {-45, 40}, {-49, 42}}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 127}), Polygon(points = {{-48.8728, -38.0295}, {-48.8728, -42.0295}, {-44.8728, -40.0295}, {-48.8728, -38.0295}}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 127}), Polygon(points = {{-48.9983, 14.8801}, {-48.9983, 10.8801}, {-44.9983, 12.8801}, {-48.9983, 14.8801}}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 127}), Ellipse(extent = {{69.3052, 4.12743}, {79.3052, -5.37257}}, fillColor = {0, 0, 127}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 127}), Line(points = {{80, 0}, {100, 0}}, color = {0, 0, 127}), Polygon(points = {{43.1618, 32.3085}, {43.1618, 28.3085}, {47.1618, 30.3085}, {43.1618, 32.3085}}, fillColor = {0, 0, 127}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 127}), Polygon(points = {{43.2575, 1.80443}, {43.2575, -2.19557}, {47.2575, -0.195573}, {43.2575, 1.80443}}, fillColor = {0, 0, 127}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 127}), Polygon(points = {{43.8805, -28.1745}, {43.8805, -32.1745}, {47.8805, -30.1745}, {43.8805, -28.1745}}, fillColor = {0, 0, 127}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 127}), Line(points = {{48, 0}, {70, 0}}, color = {0, 0, 127}), Line(points = {{47, 30}, {60, 30}, {73, 3}}, color = {0, 0, 127}), Line(points = {{49, -30}, {60, -30}, {74, -4}}, color = {0, 0, 127}), Text(extent = {{-150, -150}, {150, -110}}, lineColor = {0, 0, 0}, textString = "extract=%extract")}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-90, 50}, {-50, -50}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Rectangle(extent = {{50, 50}, {90, -50}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Polygon(points = {{-94.4104, 1.90792}, {-94.4104, -2.09208}, {-90.4104, -0.0920762}, {-94.4104, 1.90792}}, lineColor = {0, 0, 255}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Line(points = {{-72, 2}, {-60.1395, 12.907}, {-49.1395, 12.907}}, color = {0, 0, 255}), Line(points = {{-73, 4}, {-59, 40}, {-49, 40}}, color = {0, 0, 255}), Line(points = {{-112, 0}, {-75.0373, -0.0180176}}, color = {0, 0, 255}), Ellipse(extent = {{-80.0437, 4.59255}, {-70.0437, -4.90745}}, lineColor = {0, 0, 255}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid), Line(points = {{-73, -5}, {-60, -40}, {-49, -40}}, color = {0, 0, 255}), Line(points = {{-72, -2}, {-60.0698, -12.907}, {-49.0698, -12.907}}, color = {0, 0, 255}), Polygon(points = {{-48.8808, -11}, {-48.8808, -15}, {-44.8808, -13}, {-48.8808, -11}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{-46, 13}, {-35, 13}, {35, -30}, {45, -30}}, color = {0, 0, 255}), Line(points = {{-45, 40}, {-35, 40}, {35, 0}, {44, 0}}, color = {0, 0, 255}), Line(points = {{-45, -40}, {-34, -40}, {35, 30}, {44, 30}}, color = {0, 0, 255}), Polygon(points = {{-49, 42}, {-49, 38}, {-45, 40}, {-49, 42}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Polygon(points = {{-48.8728, -38.0295}, {-48.8728, -42.0295}, {-44.8728, -40.0295}, {-48.8728, -38.0295}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Polygon(points = {{-48.9983, 14.8801}, {-48.9983, 10.8801}, {-44.9983, 12.8801}, {-48.9983, 14.8801}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Ellipse(extent = {{70.3052, 4.12743}, {80.3052, -5.37257}}, lineColor = {0, 0, 255}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid), Line(points = {{80, 0}, {105, 0}}, color = {0, 0, 255}), Polygon(points = {{44.1618, 32.3085}, {44.1618, 28.3085}, {48.1618, 30.3085}, {44.1618, 32.3085}}, lineColor = {0, 0, 255}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid), Polygon(points = {{44.2575, 1.80443}, {44.2575, -2.19557}, {48.2575, -0.195573}, {44.2575, 1.80443}}, lineColor = {0, 0, 255}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid), Polygon(points = {{44.8805, -28.1745}, {44.8805, -32.1745}, {48.8805, -30.1745}, {44.8805, -28.1745}}, lineColor = {0, 0, 255}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid), Line(points = {{48, 0}, {70, 0}}, color = {0, 0, 255}), Line(points = {{47, 30}, {60, 30}, {73, 3}}, color = {0, 0, 255}), Line(points = {{49, -30}, {60, -30}, {74, -4}}, color = {0, 0, 255}), Rectangle(extent = {{-100, 80}, {100, -81}}, lineColor = {0, 0, 255})}),
          Documentation(info = "<HTML>
<p>Extract signals from the input connector and transfer them
to the output connector.</p>
<p>The extracting scheme is given by the integer vector 'extract'.
This vector specifies, which input signals are taken and in which
order they are transferred to the output vector. Note, that the
dimension of 'extract' has to match the number of outputs.
Additionally, the dimensions of the input connector signals and
the output connector signals have to be explicitly defined via the
parameters 'nin' and 'nout'.</p>
<p>Example:</p>
<pre>     nin = 7 \"Number of inputs\";
     nout = 4 \"Number of outputs\";
     extract[nout] = {6,3,3,2} \"Extracting vector\";
</pre>
<p>extracts four output signals (nout=4) from the seven elements of the
input vector (nin=7):</p>
<pre>   output no. 1 is set equal to input no. 6
   output no. 2 is set equal to input no. 3
   output no. 3 is set equal to input no. 3
   output no. 4 is set equal to input no. 2
</pre>
</html>"));
      end ExtractSignal;

      block Extractor "Extract scalar signal out of signal vector dependent on IntegerRealInput index"
        extends Modelica.Blocks.Interfaces.MISO;
        parameter Boolean allowOutOfRange = false "Index may be out of range";
        parameter Real outOfRangeValue = 1e10 "Output signal if index is out of range";
        Modelica.Blocks.Interfaces.IntegerInput index annotation(
          Placement(transformation(origin = {0, -120}, extent = {{-20, -20}, {20, 20}}, rotation = 90)));
      protected
        Real k[nin];
      equation
        when {initial(), change(index)} then
          for i in 1:nin loop
            k[i] = if index == i then 1 else 0;
          end for;
        end when;
        y = if not allowOutOfRange or index > 0 and index <= nin then k * u else outOfRangeValue;
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-80, 50}, {-40, -50}}, lineColor = {0, 0, 127}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Polygon(points = {{-84.4104, 1.9079}, {-84.4104, -2.09208}, {-80.4104, -0.09208}, {-84.4104, 1.9079}}, lineColor = {0, 0, 127}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Line(points = {{-62, 2}, {-50.1395, 12.907}, {-39.1395, 12.907}}, color = {0, 0, 127}), Line(points = {{-63, 4}, {-49, 40}, {-39, 40}}, color = {0, 0, 127}), Line(points = {{-102, 0}, {-65.0373, -0.01802}}, color = {0, 0, 127}), Ellipse(extent = {{-70.0437, 4.5925}, {-60.0437, -4.90745}}, lineColor = {0, 0, 127}, fillColor = {0, 0, 127}, fillPattern = FillPattern.Solid), Line(points = {{-63, -5}, {-50, -40}, {-39, -40}}, color = {0, 0, 127}), Line(points = {{-62, -2}, {-50.0698, -12.907}, {-39.0698, -12.907}}, color = {0, 0, 127}), Polygon(points = {{-38.8808, -11}, {-38.8808, -15}, {-34.8808, -13}, {-38.8808, -11}}, lineColor = {0, 0, 127}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Polygon(points = {{-39, 42}, {-39, 38}, {-35, 40}, {-39, 42}}, lineColor = {0, 0, 127}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Polygon(points = {{-38.8728, -38.0295}, {-38.8728, -42.0295}, {-34.8728, -40.0295}, {-38.8728, -38.0295}}, lineColor = {0, 0, 127}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Polygon(points = {{-38.9983, 14.8801}, {-38.9983, 10.8801}, {-34.9983, 12.8801}, {-38.9983, 14.8801}}, lineColor = {0, 0, 127}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-30, 50}, {30, -50}}, fillColor = {235, 235, 235}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 127}), Line(points = {{100, 0}, {0, 0}}, color = {0, 0, 127}), Line(points = {{0, 2}, {0, -104}}, color = {255, 128, 0}), Line(points = {{-35, 40}, {-20, 40}}, color = {0, 0, 127}), Line(points = {{-35, 13}, {-20, 13}}, color = {0, 0, 127}), Line(points = {{-35, -13}, {-20, -13}}, color = {0, 0, 127}), Line(points = {{-35, -40}, {-20, -40}}, color = {0, 0, 127}), Polygon(points = {{0, 0}, {-20, 13}, {-20, 13}, {0, 0}, {0, 0}}, lineColor = {0, 0, 127}), Ellipse(extent = {{-6, 6}, {6, -6}}, lineColor = {255, 128, 0}, fillColor = {255, 128, 0}, fillPattern = FillPattern.Solid)}),
          Documentation(info = "<html>
<p>This block extracts a scalar output signal out the
vector of input signals dependent on the Integer
value of the additional u index:</p>
<pre>    y = u [ index ] ;
</pre>
<p>where index is an additional Integer input signal.</p>
</html>"));
      end Extractor;

      block Multiplex2 "Multiplexer block for two input connectors"
        extends Modelica.Blocks.Icons.Block;
        parameter Integer n1 = 1 "dimension of input signal connector 1";
        parameter Integer n2 = 1 "dimension of input signal connector 2";
        Modelica.Blocks.Interfaces.RealInput u1[n1] "Connector of Real input signals 1" annotation(
          Placement(transformation(extent = {{-140, 40}, {-100, 80}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealInput u2[n2] "Connector of Real input signals 2" annotation(
          Placement(transformation(extent = {{-140, -80}, {-100, -40}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealOutput y[n1 + n2] "Connector of Real output signals" annotation(
          Placement(transformation(extent = {{100, -10}, {120, 10}}, rotation = 0)));
      equation
        [y] = [u1; u2];
        annotation(
          Documentation(info = "<HTML>
<p>
The output connector is the <b>concatenation</b> of the two input connectors.
Note, that the dimensions of the input connector signals have to be
explicitly defined via parameters n1 and n2.
</p>
</html>"),
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{8, 0}, {102, 0}}, color = {0, 0, 127}), Ellipse(extent = {{-14, 16}, {16, -14}}, fillColor = {0, 0, 127}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 127}), Line(points = {{-98, 60}, {-60, 60}, {-4, 6}}, color = {0, 0, 127}), Line(points = {{-98, -60}, {-60, -60}, {-4, -4}}, color = {0, 0, 127})}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-98, 60}, {-60, 60}, {-4, 6}}, color = {0, 0, 255}), Line(points = {{-98, -60}, {-60, -60}, {-4, -4}}, color = {0, 0, 255}), Line(points = {{8, 0}, {102, 0}}, color = {0, 0, 255}), Ellipse(extent = {{-14, 16}, {16, -14}}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255})}));
      end Multiplex2;

      block Multiplex3 "Multiplexer block for three input connectors"
        extends Modelica.Blocks.Icons.Block;
        parameter Integer n1 = 1 "dimension of input signal connector 1";
        parameter Integer n2 = 1 "dimension of input signal connector 2";
        parameter Integer n3 = 1 "dimension of input signal connector 3";
        Modelica.Blocks.Interfaces.RealInput u1[n1] "Connector of Real input signals 1" annotation(
          Placement(transformation(extent = {{-140, 50}, {-100, 90}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealInput u2[n2] "Connector of Real input signals 2" annotation(
          Placement(transformation(extent = {{-140, -20}, {-100, 20}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealInput u3[n3] "Connector of Real input signals 3" annotation(
          Placement(transformation(extent = {{-140, -90}, {-100, -50}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealOutput y[n1 + n2 + n3] "Connector of Real output signals" annotation(
          Placement(transformation(extent = {{100, -10}, {120, 10}}, rotation = 0)));
      equation
        [y] = [u1; u2; u3];
        annotation(
          Documentation(info = "<HTML>
<p>
The output connector is the <b>concatenation</b> of the three input connectors.
Note, that the dimensions of the input connector signals have to be
explicitly defined via parameters n1, n2 and n3.
</html>"),
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{8, 0}, {102, 0}}, color = {0, 0, 127}), Line(points = {{-100, 70}, {-60, 70}, {-4, 6}}, color = {0, 0, 127}), Line(points = {{-100, 0}, {-12, 0}}, color = {0, 0, 127}), Line(points = {{-100, -70}, {-62, -70}, {-4, -4}}, color = {0, 0, 127}), Ellipse(extent = {{-14, 16}, {16, -14}}, fillColor = {0, 0, 127}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 127})}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-100, 70}, {-60, 70}, {-4, 6}}, color = {0, 0, 255}), Line(points = {{-100, -70}, {-62, -70}, {-4, -4}}, color = {0, 0, 255}), Line(points = {{8, 0}, {102, 0}}, color = {0, 0, 255}), Ellipse(extent = {{-14, 16}, {16, -14}}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Line(points = {{-100, 0}, {-12, 0}}, color = {0, 0, 255})}));
      end Multiplex3;

      block Multiplex4 "Multiplexer block for four input connectors"
        extends Modelica.Blocks.Icons.Block;
        parameter Integer n1 = 1 "dimension of input signal connector 1";
        parameter Integer n2 = 1 "dimension of input signal connector 2";
        parameter Integer n3 = 1 "dimension of input signal connector 3";
        parameter Integer n4 = 1 "dimension of input signal connector 4";
        Modelica.Blocks.Interfaces.RealInput u1[n1] "Connector of Real input signals 1" annotation(
          Placement(transformation(extent = {{-140, 70}, {-100, 110}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealInput u2[n2] "Connector of Real input signals 2" annotation(
          Placement(transformation(extent = {{-140, 10}, {-100, 50}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealInput u3[n3] "Connector of Real input signals 3" annotation(
          Placement(transformation(extent = {{-140, -50}, {-100, -10}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealInput u4[n4] "Connector of Real input signals 4" annotation(
          Placement(transformation(extent = {{-140, -110}, {-100, -70}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealOutput y[n1 + n2 + n3 + n4] "Connector of Real output signals" annotation(
          Placement(transformation(extent = {{100, -10}, {120, 10}}, rotation = 0)));
      equation
        [y] = [u1; u2; u3; u4];
        annotation(
          Documentation(info = "<HTML>
<p>
The output connector is the <b>concatenation</b> of the four input connectors.
Note, that the dimensions of the input connector signals have to be
explicitly defined via parameters n1, n2, n3 and n4.
</p>
</html>"),
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{8, 0}, {102, 0}}, color = {0, 0, 127}), Line(points = {{-100, 90}, {-60, 90}, {-3, 4}}, color = {0, 0, 127}), Line(points = {{-100, 30}, {-60, 30}, {-9, 0}}, color = {0, 0, 127}), Line(points = {{-99, -30}, {-59, -30}, {-10, -5}}, color = {0, 0, 127}), Line(points = {{-100, -90}, {-60, -90}, {-5, -6}}, color = {0, 0, 127}), Ellipse(extent = {{-15, 15}, {15, -15}}, fillColor = {0, 0, 127}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 127})}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-100, 90}, {-60, 90}, {-3, 4}}, color = {0, 0, 255}), Line(points = {{-100, -90}, {-60, -90}, {-5, -6}}, color = {0, 0, 255}), Line(points = {{8, 0}, {102, 0}}, color = {0, 0, 255}), Ellipse(extent = {{-15, 15}, {15, -15}}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Line(points = {{-100, 30}, {-60, 30}, {-9, 0}}, color = {0, 0, 255}), Line(points = {{-99, -30}, {-59, -30}, {-10, -5}}, color = {0, 0, 255})}));
      end Multiplex4;

      block Multiplex5 "Multiplexer block for five input connectors"
        extends Modelica.Blocks.Icons.Block;
        parameter Integer n1 = 1 "dimension of input signal connector 1";
        parameter Integer n2 = 1 "dimension of input signal connector 2";
        parameter Integer n3 = 1 "dimension of input signal connector 3";
        parameter Integer n4 = 1 "dimension of input signal connector 4";
        parameter Integer n5 = 1 "dimension of input signal connector 5";
        Modelica.Blocks.Interfaces.RealInput u1[n1] "Connector of Real input signals 1" annotation(
          Placement(transformation(extent = {{-140, 80}, {-100, 120}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealInput u2[n2] "Connector of Real input signals 2" annotation(
          Placement(transformation(extent = {{-140, 30}, {-100, 70}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealInput u3[n3] "Connector of Real input signals 3" annotation(
          Placement(transformation(extent = {{-140, -20}, {-100, 20}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealInput u4[n4] "Connector of Real input signals 4" annotation(
          Placement(transformation(extent = {{-140, -70}, {-100, -30}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealInput u5[n5] "Connector of Real input signals 5" annotation(
          Placement(transformation(extent = {{-140, -120}, {-100, -80}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealOutput y[n1 + n2 + n3 + n4 + n5] "Connector of Real output signals" annotation(
          Placement(transformation(extent = {{100, -10}, {120, 10}}, rotation = 0)));
      equation
        [y] = [u1; u2; u3; u4; u5];
        annotation(
          Documentation(info = "<HTML>
<p>
The output connector is the <b>concatenation</b> of the five input connectors.
Note, that the dimensions of the input connector signals have to be
explicitly defined via parameters n1, n2, n3, n4 and n5.
</p>
</html>"),
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{8, 0}, {102, 0}}, color = {0, 0, 255}), Line(points = {{-100, 100}, {-60, 100}, {-4, 6}}, color = {0, 0, 127}), Line(points = {{-99, 50}, {-60, 50}, {-8, 5}}, color = {0, 0, 127}), Line(points = {{-100, 0}, {-7, 0}}, color = {0, 0, 127}), Line(points = {{-99, -50}, {-60, -50}, {-9, -6}}, color = {0, 0, 127}), Line(points = {{-100, -100}, {-60, -100}, {-4, -4}}, color = {0, 0, 127}), Ellipse(extent = {{-15, 15}, {15, -15}}, fillColor = {0, 0, 127}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 127})}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-100, 100}, {-60, 100}, {-4, 6}}, color = {0, 0, 255}), Line(points = {{-100, -100}, {-60, -100}, {-4, -4}}, color = {0, 0, 255}), Line(points = {{8, 0}, {102, 0}}, color = {0, 0, 255}), Ellipse(extent = {{-15, 15}, {15, -15}}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Line(points = {{-99, 50}, {-60, 50}, {-8, 5}}, color = {0, 0, 255}), Line(points = {{-100, 0}, {-7, 0}}, color = {0, 0, 255}), Line(points = {{-99, -50}, {-60, -50}, {-9, -6}}, color = {0, 0, 255})}));
      end Multiplex5;

      block Multiplex6 "Multiplexer block for six input connectors"
        extends Modelica.Blocks.Icons.Block;
        parameter Integer n1 = 1 "dimension of input signal connector 1";
        parameter Integer n2 = 1 "dimension of input signal connector 2";
        parameter Integer n3 = 1 "dimension of input signal connector 3";
        parameter Integer n4 = 1 "dimension of input signal connector 4";
        parameter Integer n5 = 1 "dimension of input signal connector 5";
        parameter Integer n6 = 1 "dimension of input signal connector 6";
        Modelica.Blocks.Interfaces.RealInput u1[n1] "Connector of Real input signals 1" annotation(
          Placement(transformation(extent = {{-124, 73}, {-100, 97}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealInput u2[n2] "Connector of Real input signals 2" annotation(
          Placement(transformation(extent = {{-124, 39}, {-100, 63}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealInput u3[n3] "Connector of Real input signals 3" annotation(
          Placement(transformation(extent = {{-124, 5}, {-100, 29}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealInput u4[n4] "Connector of Real input signals 4" annotation(
          Placement(transformation(extent = {{-124, -29}, {-100, -5}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealInput u5[n5] "Connector of Real input signals 5" annotation(
          Placement(transformation(extent = {{-124, -63}, {-100, -39}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealInput u6[n6] "Connector of Real input signals 6" annotation(
          Placement(transformation(extent = {{-124, -97}, {-100, -73}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealOutput y[n1 + n2 + n3 + n4 + n5 + n6] "Connector of Real output signals" annotation(
          Placement(transformation(extent = {{100, -10}, {120, 10}}, rotation = 0)));
      equation
        [y] = [u1; u2; u3; u4; u5; u6];
        annotation(
          Documentation(info = "<HTML>
<p>
The output connector is the <b>concatenation</b> of the six input connectors.
Note, that the dimensions of the input connector signals have to be
explicitly defined via parameters n1, n2, n3, n4, n5 and n6.
</p>
</html>"),
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{8, 0}, {102, 0}}, color = {0, 0, 127}), Ellipse(extent = {{-15, 15}, {15, -15}}, fillColor = {0, 0, 127}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 127}), Line(points = {{-99, 85}, {-61, 85}, {-3, 11}}, color = {0, 0, 127}), Line(points = {{-100, 51}, {-61, 51}, {-7, 6}}, color = {0, 0, 127}), Line(points = {{-101, 17}, {-60, 17}, {-9, 2}}, color = {0, 0, 127}), Line(points = {{-100, -18}, {-60, -18}, {-11, -4}}, color = {0, 0, 127}), Line(points = {{-99, -50}, {-60, -50}, {-9, -6}}, color = {0, 0, 127}), Line(points = {{-100, -85}, {-60, -85}, {-3, -10}}, color = {0, 0, 255})}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-99, 85}, {-61, 85}, {-3, 11}}, color = {0, 0, 255}), Line(points = {{-100, -85}, {-60, -85}, {-3, -10}}, color = {0, 0, 255}), Line(points = {{8, 0}, {102, 0}}, color = {0, 0, 255}), Ellipse(extent = {{-15, 15}, {15, -15}}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Line(points = {{-100, 51}, {-61, 51}, {-7, 6}}, color = {0, 0, 255}), Line(points = {{-99, -50}, {-60, -50}, {-9, -6}}, color = {0, 0, 255}), Line(points = {{-101, 17}, {-60, 17}, {-9, 2}}, color = {0, 0, 255}), Line(points = {{-100, -18}, {-60, -18}, {-11, -4}}, color = {0, 0, 255})}));
      end Multiplex6;

      block DeMultiplex2 "DeMultiplexer block for two output connectors"
        extends Modelica.Blocks.Icons.Block;
        parameter Integer n1 = 1 "dimension of output signal connector 1";
        parameter Integer n2 = 1 "dimension of output signal connector 2";
        Modelica.Blocks.Interfaces.RealInput u[n1 + n2] "Connector of Real input signals" annotation(
          Placement(transformation(extent = {{-140, -20}, {-100, 20}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealOutput y1[n1] "Connector of Real output signals 1" annotation(
          Placement(transformation(extent = {{100, 50}, {120, 70}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealOutput y2[n2] "Connector of Real output signals 2" annotation(
          Placement(transformation(extent = {{100, -70}, {120, -50}}, rotation = 0)));
      equation
        [u] = [y1; y2];
        annotation(
          Documentation(info = "<HTML>
<p>
The input connector is <b>split</b> up into two output connectors.
Note, that the dimensions of the output connector signals have to be
explicitly defined via parameters n1 and n2.
</p>
</html>"),
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{100, 60}, {60, 60}, {10, 8}}, color = {0, 0, 127}), Ellipse(extent = {{-14, 16}, {16, -14}}, fillColor = {0, 0, 127}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 127}), Line(points = {{100, -60}, {60, -60}, {8, -8}}, color = {0, 0, 127}), Line(points = {{-100, 0}, {-6, 0}}, color = {0, 0, 127})}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{100, 60}, {60, 60}, {10, 8}}, color = {0, 0, 255}), Line(points = {{100, -60}, {60, -60}, {8, -8}}, color = {0, 0, 255}), Line(points = {{-100, 0}, {-6, 0}}, color = {0, 0, 255}), Ellipse(extent = {{-14, 16}, {16, -14}}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255})}));
      end DeMultiplex2;

      block DeMultiplex3 "DeMultiplexer block for three output connectors"
        extends Modelica.Blocks.Icons.Block;
        parameter Integer n1 = 1 "dimension of output signal connector 1";
        parameter Integer n2 = 1 "dimension of output signal connector 2";
        parameter Integer n3 = 1 "dimension of output signal connector 3";
        Modelica.Blocks.Interfaces.RealInput u[n1 + n2 + n3] "Connector of Real input signals" annotation(
          Placement(transformation(extent = {{-140, -20}, {-100, 20}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealOutput y1[n1] "Connector of Real output signals 1" annotation(
          Placement(transformation(extent = {{100, 60}, {120, 80}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealOutput y2[n2] "Connector of Real output signals 2" annotation(
          Placement(transformation(extent = {{100, -10}, {120, 10}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealOutput y3[n3] "Connector of Real output signals 3" annotation(
          Placement(transformation(extent = {{100, -80}, {120, -60}}, rotation = 0)));
      equation
        [u] = [y1; y2; y3];
        annotation(
          Documentation(info = "<HTML>
<p>
The input connector is <b>split</b> into three output connectors.
Note, that the dimensions of the output connector signals have to be
explicitly defined via parameters n1, n2 and n3.
</p>
</html>"),
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-14, 16}, {16, -14}}, fillColor = {0, 0, 127}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Line(points = {{-100, 0}, {-6, 0}}, color = {0, 0, 127}), Line(points = {{100, 70}, {60, 70}, {4, 5}}, color = {0, 0, 127}), Line(points = {{0, 0}, {101, 0}}, color = {0, 0, 127}), Line(points = {{100, -70}, {61, -70}, {5, -5}}, color = {0, 0, 127})}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{100, 70}, {60, 70}, {4, 5}}, color = {0, 0, 255}), Line(points = {{100, -70}, {61, -70}, {5, -5}}, color = {0, 0, 255}), Line(points = {{-100, 0}, {-6, 0}}, color = {0, 0, 255}), Ellipse(extent = {{-16, 15}, {14, -15}}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Line(points = {{0, 0}, {101, 0}}, color = {0, 0, 255})}));
      end DeMultiplex3;

      block DeMultiplex4 "DeMultiplexer block for four output connectors"
        extends Modelica.Blocks.Icons.Block;
        parameter Integer n1 = 1 "dimension of output signal connector 1";
        parameter Integer n2 = 1 "dimension of output signal connector 2";
        parameter Integer n3 = 1 "dimension of output signal connector 3";
        parameter Integer n4 = 1 "dimension of output signal connector 4";
        Modelica.Blocks.Interfaces.RealInput u[n1 + n2 + n3 + n4] "Connector of Real input signals" annotation(
          Placement(transformation(extent = {{-140, -20}, {-100, 20}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealOutput y1[n1] "Connector of Real output signals 1" annotation(
          Placement(transformation(extent = {{100, 80}, {120, 100}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealOutput y2[n2] "Connector of Real output signals 2" annotation(
          Placement(transformation(extent = {{100, 20}, {120, 40}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealOutput y3[n3] "Connector of Real output signals 3" annotation(
          Placement(transformation(extent = {{100, -40}, {120, -20}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealOutput y4[n4] "Connector of Real output signals 4" annotation(
          Placement(transformation(extent = {{100, -100}, {120, -80}}, rotation = 0)));
      equation
        [u] = [y1; y2; y3; y4];
        annotation(
          Documentation(info = "<HTML>
<p>
The input connector is <b>split</b> into four output connectors.
Note, that the dimensions of the output connector signals have to be
explicitly defined via parameters n1, n2, n3 and n4.
</html>"),
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-14, 16}, {16, -14}}, fillColor = {0, 0, 127}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 127}), Line(points = {{-100, 0}, {-6, 0}}, color = {0, 0, 127}), Line(points = {{100, 90}, {60, 90}, {6, 5}}, color = {0, 0, 127}), Line(points = {{100, 30}, {60, 30}, {9, 2}}, color = {0, 0, 127}), Line(points = {{100, -30}, {60, -30}, {8, -4}}, color = {0, 0, 127}), Line(points = {{99, -90}, {60, -90}, {6, -6}}, color = {0, 0, 127})}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{100, 90}, {60, 90}, {6, 5}}, color = {0, 0, 255}), Line(points = {{99, -90}, {60, -90}, {6, -6}}, color = {0, 0, 255}), Line(points = {{-100, 0}, {-6, 0}}, color = {0, 0, 255}), Line(points = {{100, 30}, {60, 30}, {9, 2}}, color = {0, 0, 255}), Line(points = {{100, -30}, {60, -30}, {8, -4}}, color = {0, 0, 255}), Ellipse(extent = {{-16, 15}, {14, -15}}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255})}));
      end DeMultiplex4;

      block DeMultiplex5 "DeMultiplexer block for five output connectors"
        extends Modelica.Blocks.Icons.Block;
        parameter Integer n1 = 1 "dimension of output signal connector 1";
        parameter Integer n2 = 1 "dimension of output signal connector 2";
        parameter Integer n3 = 1 "dimension of output signal connector 3";
        parameter Integer n4 = 1 "dimension of output signal connector 4";
        parameter Integer n5 = 1 "dimension of output signal connector 5";
        Modelica.Blocks.Interfaces.RealInput u[n1 + n2 + n3 + n4 + n5] "Connector of Real input signals" annotation(
          Placement(transformation(extent = {{-140, -20}, {-100, 20}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealOutput y1[n1] "Connector of Real output signals 1" annotation(
          Placement(transformation(extent = {{100, 70}, {120, 90}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealOutput y2[n2] "Connector of Real output signals 2" annotation(
          Placement(transformation(extent = {{100, 30}, {120, 50}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealOutput y3[n3] "Connector of Real output signals 3" annotation(
          Placement(transformation(extent = {{100, -10}, {120, 10}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealOutput y4[n4] "Connector of Real output signals 4" annotation(
          Placement(transformation(extent = {{100, -50}, {120, -30}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealOutput y5[n5] "Connector of Real output signals 5" annotation(
          Placement(transformation(extent = {{100, -90}, {120, -70}}, rotation = 0)));
      equation
        [u] = [y1; y2; y3; y4; y5];
        annotation(
          Documentation(info = "<HTML>
<p>
The input connector is <b>split</b> into five output connectors.
Note, that the dimensions of the output connector signals have to be
explicitly defined via parameters n1, n2, n3, n4 and n5.
</html>"),
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-14, 16}, {16, -14}}, fillColor = {0, 0, 127}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 127}), Line(points = {{-100, 0}, {-6, 0}}, color = {0, 0, 127}), Line(points = {{99, 80}, {60, 80}, {6, 5}}, color = {0, 0, 127}), Line(points = {{100, 40}, {60, 40}, {10, 3}}, color = {0, 0, 127}), Line(points = {{100, 0}, {10, 0}}, color = {0, 0, 127}), Line(points = {{100, -40}, {61, -40}, {11, -7}}, color = {0, 0, 127}), Line(points = {{100, -80}, {60, -80}, {7, -5}}, color = {0, 0, 127})}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{99, 80}, {60, 80}, {6, 5}}, color = {0, 0, 255}), Line(points = {{100, -80}, {60, -80}, {7, -5}}, color = {0, 0, 255}), Line(points = {{-100, 0}, {-6, 0}}, color = {0, 0, 255}), Ellipse(extent = {{-14, 15}, {16, -15}}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Line(points = {{100, 40}, {60, 40}, {10, 3}}, color = {0, 0, 255}), Line(points = {{100, 0}, {10, 0}}, color = {0, 0, 255}), Line(points = {{100, -40}, {61, -40}, {11, -7}}, color = {0, 0, 255})}));
      end DeMultiplex5;

      block DeMultiplex6 "DeMultiplexer block for six output connectors"
        extends Modelica.Blocks.Icons.Block;
        parameter Integer n1 = 1 "dimension of output signal connector 1";
        parameter Integer n2 = 1 "dimension of output signal connector 2";
        parameter Integer n3 = 1 "dimension of output signal connector 3";
        parameter Integer n4 = 1 "dimension of output signal connector 4";
        parameter Integer n5 = 1 "dimension of output signal connector 5";
        parameter Integer n6 = 1 "dimension of output signal connector 6";
        Modelica.Blocks.Interfaces.RealInput u[n1 + n2 + n3 + n4 + n5 + n6] "Connector of Real input signals" annotation(
          Placement(transformation(extent = {{-140, -20}, {-100, 20}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealOutput y1[n1] "Connector of Real output signals 1" annotation(
          Placement(transformation(extent = {{100, 80}, {120, 100}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealOutput y2[n2] "Connector of Real output signals 2" annotation(
          Placement(transformation(extent = {{100, 44}, {120, 64}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealOutput y3[n3] "Connector of Real output signals 3" annotation(
          Placement(transformation(extent = {{100, 8}, {120, 28}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealOutput y4[n4] "Connector of Real output signals 4" annotation(
          Placement(transformation(extent = {{100, -28}, {120, -8}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealOutput y5[n5] "Connector of Real output signals 5" annotation(
          Placement(transformation(extent = {{100, -64}, {120, -44}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealOutput y6[n6] "Connector of Real output signals 6" annotation(
          Placement(transformation(extent = {{100, -100}, {120, -80}}, rotation = 0)));
      equation
        [u] = [y1; y2; y3; y4; y5; y6];
        annotation(
          Documentation(info = "<HTML>
<p>
The input connector is <b>split</b> into six output connectors.
Note, that the dimensions of the output connector signals have to be
explicitly defined via parameters n1, n2, n3, n4, n5 and n6.
</html>"),
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-14, 16}, {16, -14}}, fillColor = {0, 0, 127}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 127}), Line(points = {{-100, 0}, {-6, 0}}, color = {0, 0, 127}), Line(points = {{99, 90}, {60, 90}, {5, 10}}, color = {0, 0, 127}), Line(points = {{100, 53}, {60, 53}, {8, 6}}, color = {0, 0, 127}), Line(points = {{100, 18}, {59, 18}, {7, 2}}, color = {0, 0, 127}), Line(points = {{100, -19}, {60, -19}, {13, -2}}, color = {0, 0, 127}), Line(points = {{99, -54}, {60, -54}, {9, -1}}, color = {0, 0, 127}), Line(points = {{100, -91}, {60, -91}, {3, -7}}, color = {0, 0, 127})}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{99, 90}, {60, 90}, {5, 10}}, color = {0, 0, 255}), Line(points = {{100, -91}, {60, -91}, {3, -7}}, color = {0, 0, 255}), Line(points = {{-100, 0}, {-6, 0}}, color = {0, 0, 255}), Ellipse(extent = {{-14, 15}, {16, -15}}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Line(points = {{100, 53}, {60, 53}, {8, 6}}, color = {0, 0, 255}), Line(points = {{99, -54}, {60, -54}, {9, -1}}, color = {0, 0, 255}), Line(points = {{100, 18}, {59, 18}, {7, 2}}, color = {0, 0, 255}), Line(points = {{100, -19}, {60, -19}, {13, -2}}, color = {0, 0, 255})}));
      end DeMultiplex6;

      model RealPassThrough "Pass a Real signal through without modification"
        extends Modelica.Blocks.Icons.Block;
        Modelica.Blocks.Interfaces.RealInput u "Input signal" annotation(
          HideResult = true,
          Placement(transformation(extent = {{-140, -20}, {-100, 20}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealOutput y "Output signal" annotation(
          HideResult = true,
          Placement(transformation(extent = {{100, -10}, {120, 10}}, rotation = 0)));
      equation
        y = u;
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-100, 0}, {100, 0}}, color = {0, 0, 127})}),
          Documentation(info = "<html>
<p>
Passes a Real signal through without modification.  Enables signals to be read out of one bus, have their name changed and be sent back to a bus.
</p>
</html>"));
      end RealPassThrough;

      model IntegerPassThrough "Pass a Integer signal through without modification"
        extends Modelica.Blocks.Icons.IntegerBlock;
        Modelica.Blocks.Interfaces.IntegerInput u "Input signal" annotation(
          Placement(transformation(extent = {{-140, -20}, {-100, 20}}, rotation = 0)));
        Modelica.Blocks.Interfaces.IntegerOutput y "Output signal" annotation(
          Placement(transformation(extent = {{100, -10}, {120, 10}}, rotation = 0)));
      equation
        y = u;
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-100, 0}, {100, 0}}, color = {255, 128, 0})}),
          Documentation(info = "<html>
<p>Passes a Integer signal through without modification.  Enables signals to be read out of one bus, have their name changed and be sent back to a bus.</p>
</html>"));
      end IntegerPassThrough;

      model BooleanPassThrough "Pass a Boolean signal through without modification"
        extends Modelica.Blocks.Icons.BooleanBlock;
        Modelica.Blocks.Interfaces.BooleanInput u "Input signal" annotation(
          Placement(transformation(extent = {{-140, -20}, {-100, 20}}, rotation = 0)));
        Modelica.Blocks.Interfaces.BooleanOutput y "Output signal" annotation(
          Placement(transformation(extent = {{100, -10}, {120, 10}}, rotation = 0)));
      equation
        y = u;
        annotation(
          Documentation(info = "<html>
<p>Passes a Boolean signal through without modification.  Enables signals to be read out of one bus, have their name changed and be sent back to a bus.</p>
</html>"),
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100.0, -100.0}, {100.0, 100.0}}, initialScale = 0.1), graphics = {Line(points = {{-100.0, 0.0}, {100.0, 0.0}}, color = {255, 0, 255})}));
      end BooleanPassThrough;
      annotation(
        Documentation(info = "<html>
<p>
This package contains blocks to combine and extract signals.
</p>
</html>"),
        Icon(graphics = {Line(points = {{-90, 0}, {4, 0}}, color = {95, 95, 95}), Line(points = {{88, 65}, {48, 65}, {-8, 0}}, color = {95, 95, 95}), Line(points = {{-8, 0}, {93, 0}}, color = {95, 95, 95}), Line(points = {{87, -65}, {48, -65}, {-8, 0}}, color = {95, 95, 95})}));
    end Routing;

    package Sources "Library of signal source blocks generating Real and Boolean signals"
      import Modelica.Blocks.Interfaces;
      import Modelica.SIunits;
      extends Modelica.Icons.SourcesPackage;

      block RealExpression "Set output signal to a time varying Real expression"
        Modelica.Blocks.Interfaces.RealOutput y = 0.0 "Value of Real output" annotation(
          Dialog(group = "Time varying output signal"),
          Placement(transformation(extent = {{100, -10}, {120, 10}}, rotation = 0)));
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, 40}, {100, -40}}, lineColor = {0, 0, 0}, lineThickness = 5.0, fillColor = {235, 235, 235}, fillPattern = FillPattern.Solid, borderPattern = BorderPattern.Raised), Text(extent = {{-96, 15}, {96, -15}}, lineColor = {0, 0, 0}, textString = "%y"), Text(extent = {{-150, 90}, {140, 50}}, textString = "%name", lineColor = {0, 0, 255})}),
          Documentation(info = "<html>
<p>
The (time varying) Real output signal of this block can be defined in its
parameter menu via variable <b>y</b>. The purpose is to support the
easy definition of Real expressions in a block diagram. For example,
in the y-menu the definition \"if time &lt; 1 then 0 else 1\" can be given in order
to define that the output signal is one, if time &ge; 1 and otherwise
it is zero. Note, that \"time\" is a built-in variable that is always
accessible and represents the \"model time\" and that
Variable <b>y</b> is both a variable and a connector.
</p>
</html>"));
      end RealExpression;

      block IntegerExpression "Set output signal to a time varying Integer expression"
        Modelica.Blocks.Interfaces.IntegerOutput y = 0 "Value of Integer output" annotation(
          Dialog(group = "Time varying output signal"),
          Placement(transformation(extent = {{100, -10}, {120, 10}}, rotation = 0)));
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, 40}, {100, -40}}, lineColor = {0, 0, 0}, lineThickness = 5.0, fillColor = {235, 235, 235}, fillPattern = FillPattern.Solid, borderPattern = BorderPattern.Raised), Text(extent = {{-96, 15}, {96, -15}}, lineColor = {0, 0, 0}, textString = "%y"), Text(extent = {{-150, 90}, {140, 50}}, textString = "%name", lineColor = {0, 0, 255})}),
          Documentation(info = "<html>
<p>
The (time varying) Integer output signal of this block can be defined in its
parameter menu via variable <b>y</b>. The purpose is to support the
easy definition of Integer expressions in a block diagram. For example,
in the y-menu the definition \"if time &lt; 1 then 0 else 1\" can be given in order
to define that the output signal is one, if time &ge; 1 and otherwise
it is zero. Note, that \"time\" is a built-in variable that is always
accessible and represents the \"model time\" and that
Variable <b>y</b> is both a variable and a connector.
</p>
</html>"));
      end IntegerExpression;

      block BooleanExpression "Set output signal to a time varying Boolean expression"
        Modelica.Blocks.Interfaces.BooleanOutput y = false "Value of Boolean output" annotation(
          Dialog(group = "Time varying output signal"),
          Placement(transformation(extent = {{100, -10}, {120, 10}}, rotation = 0)));
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, 40}, {100, -40}}, lineColor = {0, 0, 0}, lineThickness = 5.0, fillColor = {235, 235, 235}, fillPattern = FillPattern.Solid, borderPattern = BorderPattern.Raised), Text(extent = {{-96, 15}, {96, -15}}, lineColor = {0, 0, 0}, textString = "%y"), Text(extent = {{-150, 90}, {140, 50}}, textString = "%name", lineColor = {0, 0, 255}), Polygon(points = {{100, 10}, {120, 0}, {100, -10}, {100, 10}}, lineColor = DynamicSelect({255, 0, 255}, if y > 0.5 then {0, 255, 0} else {255, 0, 255}), fillColor = DynamicSelect({255, 255, 255}, if y > 0.5 then {0, 255, 0} else {255, 255, 255}), fillPattern = FillPattern.Solid)}),
          Documentation(info = "<html>
<p>
The (time varying) Boolean output signal of this block can be defined in its
parameter menu via variable <b>y</b>. The purpose is to support the
easy definition of Boolean expressions in a block diagram. For example,
in the y-menu the definition \"time &gt;= 1 and time &lt;= 2\" can be given in order
to define that the output signal is <b>true</b> in the time interval
1 &le; time &le; 2 and otherwise it is <b>false</b>.
Note, that \"time\" is a built-in variable that is always
accessible and represents the \"model time\" and that
Variable <b>y</b> is both a variable and a connector.
</p>
</html>"));
      end BooleanExpression;

      block Clock "Generate actual time signal"
        parameter Modelica.SIunits.Time offset = 0 "Offset of output signal";
        parameter Modelica.SIunits.Time startTime = 0 "Output = offset for time < startTime";
        extends Interfaces.SO;
      equation
        y = offset + (if time < startTime then 0 else time - startTime);
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-80, 80}, {80, -80}}, lineColor = {160, 160, 164}), Line(points = {{0, 80}, {0, 60}}, color = {160, 160, 164}), Line(points = {{80, 0}, {60, 0}}, color = {160, 160, 164}), Line(points = {{0, -80}, {0, -60}}, color = {160, 160, 164}), Line(points = {{-80, 0}, {-60, 0}}, color = {160, 160, 164}), Line(points = {{37, 70}, {26, 50}}, color = {160, 160, 164}), Line(points = {{70, 38}, {49, 26}}, color = {160, 160, 164}), Line(points = {{71, -37}, {52, -27}}, color = {160, 160, 164}), Line(points = {{39, -70}, {29, -51}}, color = {160, 160, 164}), Line(points = {{-39, -70}, {-29, -52}}, color = {160, 160, 164}), Line(points = {{-71, -37}, {-50, -26}}, color = {160, 160, 164}), Line(points = {{-71, 37}, {-54, 28}}, color = {160, 160, 164}), Line(points = {{-38, 70}, {-28, 51}}, color = {160, 160, 164}), Line(points = {{0, 0}, {-50, 50}}, color = {0, 0, 0}, thickness = 0.5), Line(points = {{0, 0}, {40, 0}}, color = {0, 0, 0}, thickness = 0.5), Text(extent = {{-150, -150}, {150, -110}}, lineColor = {0, 0, 0}, textString = "startTime=%startTime")}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{-80, 90}, {-85, 68}, {-75, 68}, {-80, 90}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Line(points = {{-80, 68}, {-80, -80}}, color = {95, 95, 95}), Line(points = {{-80, 0}, {-10, 0}, {60, 70}}, color = {0, 0, 255}, thickness = 0.5), Line(points = {{-90, -70}, {82, -70}}, color = {95, 95, 95}), Polygon(points = {{90, -70}, {68, -64}, {68, -76}, {90, -70}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Polygon(points = {{-34, 0}, {-37, -13}, {-31, -13}, {-34, 0}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Line(points = {{-34, 0}, {-34, -70}}, color = {95, 95, 95}), Polygon(points = {{-34, -70}, {-37, -57}, {-31, -57}, {-34, -70}, {-34, -70}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Text(extent = {{-77, -28}, {-35, -40}}, lineColor = {0, 0, 0}, textString = "offset"), Text(extent = {{-30, -73}, {18, -86}}, lineColor = {0, 0, 0}, textString = "startTime"), Text(extent = {{-81, 91}, {-40, 71}}, lineColor = {0, 0, 0}, textString = "y"), Text(extent = {{63, -79}, {94, -89}}, lineColor = {0, 0, 0}, textString = "time"), Line(points = {{-10, 0}, {-10, -70}}, color = {95, 95, 95}), Line(points = {{-10, 0}, {50, 0}}, color = {95, 95, 95}), Line(points = {{50, 0}, {50, 60}}, color = {95, 95, 95}), Text(extent = {{35, 33}, {50, 23}}, lineColor = {0, 0, 0}, textString = "1"), Text(extent = {{14, 13}, {32, 1}}, lineColor = {0, 0, 0}, textString = "1")}),
          Documentation(info = "<html>
<p>
The Real output y is a clock signal:
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Blocks/Sources/Clock.png\"
     alt=\"Clock.png\">
</p>
</html>"));
      end Clock;

      block Constant "Generate constant signal of type Real"
        parameter Real k(start = 1) "Constant output value";
        extends Interfaces.SO;
      equation
        y = k;
        annotation(
          defaultComponentName = "const",
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-80, 68}, {-80, -80}}, color = {192, 192, 192}), Polygon(points = {{-80, 90}, {-88, 68}, {-72, 68}, {-80, 90}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-90, -70}, {82, -70}}, color = {192, 192, 192}), Polygon(points = {{90, -70}, {68, -62}, {68, -78}, {90, -70}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-80, 0}, {80, 0}}, color = {0, 0, 0}), Text(extent = {{-150, -150}, {150, -110}}, lineColor = {0, 0, 0}, textString = "k=%k")}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{-80, 90}, {-86, 68}, {-74, 68}, {-80, 90}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Line(points = {{-80, 68}, {-80, -80}}, color = {95, 95, 95}), Line(points = {{-80, 0}, {80, 0}}, color = {0, 0, 255}, thickness = 0.5), Line(points = {{-90, -70}, {82, -70}}, color = {95, 95, 95}), Polygon(points = {{90, -70}, {68, -64}, {68, -76}, {90, -70}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Text(extent = {{-83, 92}, {-30, 74}}, lineColor = {0, 0, 0}, textString = "y"), Text(extent = {{70, -80}, {94, -100}}, lineColor = {0, 0, 0}, textString = "time"), Text(extent = {{-101, 8}, {-81, -12}}, lineColor = {0, 0, 0}, textString = "k")}),
          Documentation(info = "<html>
<p>
The Real output y is a constant signal:
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Blocks/Sources/Constant.png\"
     alt=\"Constant.png\">
</p>
</html>"));
      end Constant;

      block Step "Generate step signal of type Real"
        parameter Real height = 1 "Height of step";
        extends Interfaces.SignalSource;
      equation
        y = offset + (if time < startTime then 0 else height);
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-80, 68}, {-80, -80}}, color = {192, 192, 192}), Polygon(points = {{-80, 90}, {-88, 68}, {-72, 68}, {-80, 90}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-90, -70}, {82, -70}}, color = {192, 192, 192}), Polygon(points = {{90, -70}, {68, -62}, {68, -78}, {90, -70}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-80, -70}, {0, -70}, {0, 50}, {80, 50}}, color = {0, 0, 0}), Text(extent = {{-150, -150}, {150, -110}}, lineColor = {0, 0, 0}, textString = "startTime=%startTime")}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{-80, 90}, {-86, 68}, {-74, 68}, {-80, 90}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Line(points = {{-80, 68}, {-80, -80}}, color = {95, 95, 95}), Line(points = {{-80, -18}, {0, -18}, {0, 50}, {80, 50}}, color = {0, 0, 255}, thickness = 0.5), Line(points = {{-90, -70}, {82, -70}}, color = {95, 95, 95}), Polygon(points = {{90, -70}, {68, -64}, {68, -76}, {90, -70}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Text(extent = {{70, -80}, {94, -100}}, lineColor = {0, 0, 0}, textString = "time"), Text(extent = {{-21, -72}, {25, -90}}, lineColor = {0, 0, 0}, textString = "startTime"), Line(points = {{0, -18}, {0, -70}}, color = {95, 95, 95}), Text(extent = {{-68, -36}, {-22, -54}}, lineColor = {0, 0, 0}, textString = "offset"), Line(points = {{-13, 50}, {-13, -17}}, color = {95, 95, 95}), Polygon(points = {{0, 50}, {-21, 50}, {0, 50}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Polygon(points = {{-13, -18}, {-16, -5}, {-10, -5}, {-13, -18}, {-13, -18}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Polygon(points = {{-13, 50}, {-16, 37}, {-10, 37}, {-13, 50}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Text(extent = {{-68, 26}, {-22, 8}}, lineColor = {0, 0, 0}, textString = "height"), Polygon(points = {{-13, -70}, {-16, -57}, {-10, -57}, {-13, -70}, {-13, -70}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Line(points = {{-13, -18}, {-13, -70}}, color = {95, 95, 95}), Polygon(points = {{-13, -18}, {-16, -31}, {-10, -31}, {-13, -18}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Text(extent = {{-72, 100}, {-31, 80}}, lineColor = {0, 0, 0}, textString = "y")}),
          Documentation(info = "<html>
<p>
The Real output y is a step signal:
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Blocks/Sources/Step.png\"
     alt=\"Step.png\">
</p>

</html>"));
      end Step;

      block Ramp "Generate ramp signal"
        parameter Real height = 1 "Height of ramps";
        parameter Modelica.SIunits.Time duration(min = 0.0, start = 2) "Duration of ramp (= 0.0 gives a Step)";
        parameter Real offset = 0 "Offset of output signal";
        parameter Modelica.SIunits.Time startTime = 0 "Output = offset for time < startTime";
        extends Interfaces.SO;
      equation
        y = offset + (if time < startTime then 0 else if time < startTime + duration then (time - startTime) * height / duration else height);
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-80, 68}, {-80, -80}}, color = {192, 192, 192}), Polygon(points = {{-80, 90}, {-88, 68}, {-72, 68}, {-80, 90}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-90, -70}, {82, -70}}, color = {192, 192, 192}), Polygon(points = {{90, -70}, {68, -62}, {68, -78}, {90, -70}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-80, -70}, {-40, -70}, {31, 38}}, color = {0, 0, 0}), Text(extent = {{-150, -150}, {150, -110}}, lineColor = {0, 0, 0}, textString = "duration=%duration"), Line(points = {{31, 38}, {86, 38}}, color = {0, 0, 0})}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{-80, 90}, {-86, 68}, {-74, 68}, {-80, 90}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Line(points = {{-80, 68}, {-80, -80}}, color = {95, 95, 95}), Line(points = {{-80, -20}, {-20, -20}, {50, 50}}, color = {0, 0, 255}, thickness = 0.5), Line(points = {{-90, -70}, {82, -70}}, color = {95, 95, 95}), Polygon(points = {{90, -70}, {68, -64}, {68, -76}, {90, -70}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Polygon(points = {{-40, -20}, {-42, -30}, {-38, -30}, {-40, -20}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Line(points = {{-40, -20}, {-40, -70}}, color = {95, 95, 95}, thickness = 0.25, arrow = {Arrow.None, Arrow.None}), Polygon(points = {{-40, -70}, {-42, -60}, {-38, -60}, {-40, -70}, {-40, -70}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Text(extent = {{-72, -39}, {-34, -50}}, lineColor = {0, 0, 0}, textString = "offset"), Text(extent = {{-38, -72}, {6, -83}}, lineColor = {0, 0, 0}, textString = "startTime"), Text(extent = {{-78, 92}, {-37, 72}}, lineColor = {0, 0, 0}, textString = "y"), Text(extent = {{70, -80}, {94, -91}}, lineColor = {0, 0, 0}, textString = "time"), Line(points = {{-20, -20}, {-20, -70}}, color = {95, 95, 95}), Line(points = {{-19, -20}, {50, -20}}, color = {95, 95, 95}, thickness = 0.25, arrow = {Arrow.None, Arrow.None}), Line(points = {{50, 50}, {101, 50}}, color = {0, 0, 255}, thickness = 0.5), Line(points = {{50, 50}, {50, -20}}, color = {95, 95, 95}, thickness = 0.25, arrow = {Arrow.None, Arrow.None}), Polygon(points = {{50, -20}, {42, -18}, {42, -22}, {50, -20}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Polygon(points = {{-20, -20}, {-11, -18}, {-11, -22}, {-20, -20}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Polygon(points = {{50, 50}, {48, 40}, {52, 40}, {50, 50}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Polygon(points = {{50, -20}, {48, -10}, {52, -10}, {50, -20}, {50, -20}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Text(extent = {{53, 23}, {82, 10}}, lineColor = {0, 0, 0}, textString = "height"), Text(extent = {{-2, -21}, {37, -33}}, lineColor = {0, 0, 0}, textString = "duration")}),
          Documentation(info = "<html>
<p>
The Real output y is a ramp signal:
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Blocks/Sources/Ramp.png\"
     alt=\"Ramp.png\">
</p>

<p>
If parameter duration is set to 0.0, the limiting case of a Step signal is achieved.
</p>
</html>"));
      end Ramp;

      block Sine "Generate sine signal"
        parameter Real amplitude = 1 "Amplitude of sine wave";
        parameter SIunits.Frequency freqHz(start = 1) "Frequency of sine wave";
        parameter SIunits.Angle phase = 0 "Phase of sine wave";
        parameter Real offset = 0 "Offset of output signal";
        parameter SIunits.Time startTime = 0 "Output = offset for time < startTime";
        extends Interfaces.SO;
      protected
        constant Real pi = Modelica.Constants.pi;
      equation
        y = offset + (if time < startTime then 0 else amplitude * Modelica.Math.sin(2 * pi * freqHz * (time - startTime) + phase));
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-80, 68}, {-80, -80}}, color = {192, 192, 192}), Polygon(points = {{-80, 90}, {-88, 68}, {-72, 68}, {-80, 90}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-90, 0}, {68, 0}}, color = {192, 192, 192}), Polygon(points = {{90, 0}, {68, 8}, {68, -8}, {90, 0}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-80, 0}, {-68.7, 34.2}, {-61.5, 53.1}, {-55.1, 66.4}, {-49.4, 74.6}, {-43.8, 79.1}, {-38.2, 79.8}, {-32.6, 76.6}, {-26.9, 69.7}, {-21.3, 59.4}, {-14.9, 44.1}, {-6.83, 21.2}, {10.1, -30.8}, {17.3, -50.2}, {23.7, -64.2}, {29.3, -73.1}, {35, -78.4}, {40.6, -80}, {46.2, -77.6}, {51.9, -71.5}, {57.5, -61.9}, {63.9, -47.2}, {72, -24.8}, {80, 0}}, color = {0, 0, 0}, smooth = Smooth.Bezier), Text(extent = {{-147, -152}, {153, -112}}, lineColor = {0, 0, 0}, textString = "freqHz=%freqHz")}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-80, -90}, {-80, 84}}, color = {95, 95, 95}), Polygon(points = {{-80, 97}, {-84, 81}, {-76, 81}, {-80, 97}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Line(points = {{-99, -40}, {85, -40}}, color = {95, 95, 95}), Polygon(points = {{97, -40}, {81, -36}, {81, -45}, {97, -40}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Line(points = {{-41, -2}, {-31.6, 34.2}, {-26.1, 53.1}, {-21.3, 66.4}, {-17.1, 74.6}, {-12.9, 79.1}, {-8.64, 79.8}, {-4.42, 76.6}, {-0.201, 69.7}, {4.02, 59.4}, {8.84, 44.1}, {14.9, 21.2}, {27.5, -30.8}, {33, -50.2}, {37.8, -64.2}, {42, -73.1}, {46.2, -78.4}, {50.5, -80}, {54.7, -77.6}, {58.9, -71.5}, {63.1, -61.9}, {67.9, -47.2}, {74, -24.8}, {80, 0}}, color = {0, 0, 255}, thickness = 0.5, smooth = Smooth.Bezier), Line(points = {{-41, -2}, {-80, -2}}, color = {0, 0, 255}, thickness = 0.5), Text(extent = {{-87, 12}, {-40, 0}}, lineColor = {0, 0, 0}, textString = "offset"), Line(points = {{-41, -2}, {-41, -40}}, color = {95, 95, 95}), Text(extent = {{-60, -43}, {-14, -54}}, lineColor = {0, 0, 0}, textString = "startTime"), Text(extent = {{75, -47}, {100, -60}}, lineColor = {0, 0, 0}, textString = "time"), Text(extent = {{-80, 99}, {-40, 82}}, lineColor = {0, 0, 0}, textString = "y"), Line(points = {{-9, 80}, {43, 80}}, color = {95, 95, 95}), Line(points = {{-41, -2}, {50, -2}}, color = {95, 95, 95}), Polygon(points = {{33, 80}, {30, 67}, {36, 67}, {33, 80}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Text(extent = {{37, 57}, {83, 39}}, lineColor = {0, 0, 0}, textString = "amplitude"), Polygon(points = {{33, -2}, {30, 11}, {36, 11}, {33, -2}, {33, -2}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Line(points = {{33, 80}, {33, -2}}, color = {95, 95, 95})}),
          Documentation(info = "<html>
<p>
The Real output y is a sine signal:
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Blocks/Sources/Sine.png\"
     alt=\"Sine.png\">
</p>
</html>"));
      end Sine;

      block Cosine "Generate cosine signal"
        parameter Real amplitude = 1 "Amplitude of cosine wave";
        parameter SIunits.Frequency freqHz(start = 1) "Frequency of cosine wave";
        parameter SIunits.Angle phase = 0 "Phase of cosine wave";
        parameter Real offset = 0 "Offset of output signal";
        parameter SIunits.Time startTime = 0 "Output = offset for time < startTime";
        extends Interfaces.SO;
      protected
        constant Real pi = Modelica.Constants.pi;
      equation
        y = offset + (if time < startTime then 0 else amplitude * Modelica.Math.cos(2 * pi * freqHz * (time - startTime) + phase));
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-80, 68}, {-80, -80}}, color = {192, 192, 192}), Polygon(points = {{-80, 90}, {-88, 68}, {-72, 68}, {-80, 90}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-90, 0}, {68, 0}}, color = {192, 192, 192}), Polygon(points = {{90, 0}, {68, 8}, {68, -8}, {90, 0}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-80, 80}, {-76.2, 79.8}, {-70.6, 76.6}, {-64.9, 69.7}, {-59.3, 59.4}, {-52.9, 44.1}, {-44.83, 21.2}, {-27.9, -30.8}, {-20.7, -50.2}, {-14.3, -64.2}, {-8.7, -73.1}, {-3, -78.4}, {2.6, -80}, {8.2, -77.6}, {13.9, -71.5}, {19.5, -61.9}, {25.9, -47.2}, {34, -24.8}, {42, 0}}, color = {0, 0, 0}, smooth = Smooth.Bezier), Text(extent = {{-147, -152}, {153, -112}}, lineColor = {0, 0, 0}, textString = "freqHz=%freqHz"), Line(points = {{42, 1}, {53.3, 35.2}, {60.5, 54.1}, {66.9, 67.4}, {72.6, 75.6}, {78.2, 80.1}, {83.8, 80.8}}, color = {0, 0, 0})}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-80, -90}, {-80, 84}}, color = {95, 95, 95}), Polygon(points = {{-80, 97}, {-84, 81}, {-76, 81}, {-80, 97}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Line(points = {{-99, -40}, {85, -40}}, color = {95, 95, 95}), Polygon(points = {{97, -40}, {81, -36}, {81, -45}, {97, -40}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Line(points = {{-41, -2}, {-80, -2}}, color = {0, 0, 255}, thickness = 0.5), Text(extent = {{-87, 12}, {-40, 0}}, lineColor = {0, 0, 0}, textString = "offset"), Line(points = {{-41, -2}, {-41, -40}}, color = {95, 95, 95}), Text(extent = {{-60, -43}, {-14, -54}}, lineColor = {0, 0, 0}, textString = "startTime"), Text(extent = {{75, -47}, {100, -60}}, lineColor = {0, 0, 0}, textString = "time"), Text(extent = {{-80, 99}, {-40, 82}}, lineColor = {0, 0, 0}, textString = "y"), Line(points = {{-41, 80}, {43, 80}}, color = {95, 95, 95}), Line(points = {{-41, -2}, {40, -2}}, color = {95, 95, 95}), Polygon(points = {{33, 80}, {30, 67}, {36, 67}, {33, 80}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Text(extent = {{37, 57}, {83, 39}}, lineColor = {0, 0, 0}, textString = "amplitude"), Polygon(points = {{33, -2}, {30, 11}, {36, 11}, {33, -2}, {33, -2}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Line(points = {{33, 80}, {33, -2}}, color = {95, 95, 95}), Line(points = {{-41, 80}, {-38, 79}, {-35.42, 75.6}, {-31.201, 68.7}, {-26.98, 58.4}, {-22.16, 43.1}, {-16.1, 20.2}, {-3.5, -31.8}, {2, -51.2}, {6.8, -65.2}, {11, -74.1}, {15.2, -79.4}, {19.5, -81}, {23.7, -78.6}, {27.9, -72.5}, {32.1, -62.9}, {36.9, -48.2}, {43, -25.8}, {49, -1}, {49, -1}, {57.4, 33.2}, {62.9, 52.1}, {67.7, 65.4}, {71.9, 73.6}, {76.1, 78.1}, {80, 79}}, color = {0, 0, 255}, thickness = 0.5, smooth = Smooth.Bezier), Line(points = {{-41, 80}, {-41, -2}}, color = {0, 0, 255}, thickness = 0.5)}),
          Documentation(info = "<html>
<p>
The Real output y is a cosine signal:
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Blocks/Sources/Cosine.png\"
     alt=\"Cosine.png\">
</p>
</html>"));
      end Cosine;

      block ExpSine "Generate exponentially damped sine signal"
        parameter Real amplitude = 1 "Amplitude of sine wave";
        parameter SIunits.Frequency freqHz(start = 2) "Frequency of sine wave";
        parameter SIunits.Angle phase = 0 "Phase of sine wave";
        parameter SIunits.Damping damping(start = 1) "Damping coefficient of sine wave";
        parameter Real offset = 0 "Offset of output signal";
        parameter SIunits.Time startTime = 0 "Output = offset for time < startTime";
        extends Interfaces.SO;
      protected
        constant Real pi = Modelica.Constants.pi;
      equation
        y = offset + (if time < startTime then 0 else amplitude * Modelica.Math.exp(-(time - startTime) * damping) * Modelica.Math.sin(2 * pi * freqHz * (time - startTime) + phase));
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-80, 68}, {-80, -80}}, color = {192, 192, 192}), Polygon(points = {{-80, 90}, {-88, 68}, {-72, 68}, {-80, 90}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-90, 0}, {68, 0}}, color = {192, 192, 192}), Polygon(points = {{90, 0}, {68, 8}, {68, -8}, {90, 0}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-80, 0}, {-75.2, 32.3}, {-72, 50.3}, {-68.7, 64.5}, {-65.5, 74.2}, {-62.3, 79.3}, {-59.1, 79.6}, {-55.9, 75.3}, {-52.7, 67.1}, {-48.6, 52.2}, {-43, 25.8}, {-35, -13.9}, {-30.2, -33.7}, {-26.1, -45.9}, {-22.1, -53.2}, {-18.1, -55.3}, {-14.1, -52.5}, {-10.1, -45.3}, {-5.23, -32.1}, {8.44, 13.7}, {13.3, 26.4}, {18.1, 34.8}, {22.1, 38}, {26.9, 37.2}, {31.8, 31.8}, {38.2, 19.4}, {51.1, -10.5}, {57.5, -21.2}, {63.1, -25.9}, {68.7, -25.9}, {75.2, -20.5}, {80, -13.8}}, color = {0, 0, 0}, smooth = Smooth.Bezier), Text(extent = {{-147, -152}, {153, -112}}, lineColor = {0, 0, 0}, textString = "freqHz=%freqHz")}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-86, -90}, {-86, 84}}, color = {95, 95, 95}), Polygon(points = {{-86, 98}, {-90, 82}, {-82, 82}, {-86, 98}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Line(points = {{-99, -40}, {85, -40}}, color = {95, 95, 95}), Polygon(points = {{97, -40}, {81, -35}, {81, -44}, {97, -40}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Line(points = {{-50, 0}, {-46.1, 28.2}, {-43.5, 44}, {-40.9, 56.4}, {-38.2, 64.9}, {-35.6, 69.4}, {-33, 69.6}, {-30.4, 65.9}, {-27.8, 58.7}, {-24.5, 45.7}, {-19.9, 22.5}, {-13.4, -12.2}, {-9.5, -29.5}, {-6.23, -40.1}, {-2.96, -46.5}, {0.302, -48.4}, {3.57, -45.9}, {6.83, -39.6}, {10.8, -28.1}, {21.9, 12}, {25.8, 23.1}, {29.7, 30.5}, {33, 33.3}, {36.9, 32.5}, {40.8, 27.8}, {46, 16.9}, {56.5, -9.2}, {61.7, -18.6}, {66.3, -22.7}, {70.9, -22.6}, {76.1, -18}, {80, -12.1}}, color = {0, 0, 255}, thickness = 0.5, smooth = Smooth.Bezier), Text(extent = {{-80, 17}, {-57, -3}}, lineColor = {0, 0, 0}, textString = "offset"), Text(extent = {{-72, -43}, {-25, -53}}, lineColor = {0, 0, 0}, textString = "startTime"), Text(extent = {{77, -53}, {101, -64}}, lineColor = {0, 0, 0}, textString = "time"), Text(extent = {{-89, 99}, {-49, 82}}, lineColor = {0, 0, 0}, textString = "y"), Line(points = {{-50, 0}, {18, 0}}, color = {95, 95, 95}), Line(points = {{-50, 0}, {-86, 0}}, color = {0, 0, 255}, thickness = 0.5), Line(points = {{-50, 77}, {-50, 0}}, color = {95, 95, 95}), Line(points = {{18, -1}, {18, 76}}, color = {95, 95, 95}), Line(points = {{18, 73}, {-50, 73}}, color = {95, 95, 95}), Text(extent = {{-42, 83}, {9, 74}}, lineColor = {0, 0, 0}, textString = "1/freqHz"), Polygon(points = {{-50, 73}, {-41, 75}, {-41, 71}, {-50, 73}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Polygon(points = {{18, 73}, {10, 75}, {10, 71}, {18, 73}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Line(points = {{-50, -61}, {-19, -61}}, color = {95, 95, 95}), Polygon(points = {{-18, -61}, {-26, -59}, {-26, -63}, {-18, -61}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Text(extent = {{-51, -63}, {-27, -75}}, lineColor = {0, 0, 0}, textString = "t"), Text(extent = {{-83, -69}, {100, -96}}, lineColor = {0, 0, 0}, textString = "amplitude*exp(-damping*t)*sin(2*pi*freqHz*t+phase)"), Line(points = {{-50, 0}, {-50, -40}}, color = {95, 95, 95}), Line(points = {{-50, -54}, {-50, -72}}, color = {95, 95, 95}), Line(points = {{-15, -77}, {-1, -48}}, color = {95, 95, 95})}),
          Documentation(info = "<html>
<p>
The Real output y is a sine signal with exponentially changing amplitude:
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Blocks/Sources/ExpSine.png\"
     alt=\"ExpSine.png\">
</p>
</html>"));
      end ExpSine;

      model Exponentials "Generate a rising and falling exponential signal"
        parameter Real outMax = 1 "Height of output for infinite riseTime";
        parameter SIunits.Time riseTime(min = 0, start = 0.5) "Rise time";
        parameter SIunits.Time riseTimeConst(min = Modelica.Constants.small) = 0.1 "Rise time constant; rising is defined as outMax*(1-exp(-riseTime/riseTimeConst))";
        parameter SIunits.Time fallTimeConst(min = Modelica.Constants.small) = riseTimeConst "Fall time constant";
        parameter Real offset = 0 "Offset of output signal";
        parameter SIunits.Time startTime = 0 "Output = offset for time < startTime";
        extends Interfaces.SO;
      protected
        Real y_riseTime;
      equation
        y_riseTime = outMax * (1 - Modelica.Math.exp(-riseTime / riseTimeConst));
        y = offset + (if time < startTime then 0 else if time < startTime + riseTime then outMax * (1 - Modelica.Math.exp(-(time - startTime) / riseTimeConst)) else y_riseTime * Modelica.Math.exp(-(time - startTime - riseTime) / fallTimeConst));
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-90, -70}, {68, -70}}, color = {192, 192, 192}), Polygon(points = {{90, -70}, {68, -62}, {68, -78}, {90, -70}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-80, -70}, {-77.2, -55.3}, {-74.3, -42.1}, {-70.8, -27.6}, {-67.3, -15}, {-63.7, -4.08}, {-59.5, 7.18}, {-55.3, 16.7}, {-50.3, 26}, {-44.6, 34.5}, {-38.3, 42.1}, {-31.2, 48.6}, {-22.7, 54.3}, {-12.1, 59.2}, {-10, 60}, {-7.88, 47.5}, {-5.05, 32.7}, {-2.22, 19.8}, {0.606, 8.45}, {4.14, -3.7}, {7.68, -14}, {11.9, -24.2}, {16.2, -32.6}, {21.1, -40.5}, {26.8, -47.4}, {33.1, -53.3}, {40.9, -58.5}, {50.8, -62.8}, {60, -65.4}}, color = {0, 0, 0}), Polygon(points = {{-80, 90}, {-88, 68}, {-72, 68}, {-80, 90}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-80, 68}, {-80, -80}}, color = {192, 192, 192}), Text(extent = {{-150, -150}, {150, -110}}, lineColor = {0, 0, 0}, textString = "riseTime=%riseTime")}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-90, -74}, {84, -74}}, color = {95, 95, 95}), Polygon(points = {{97, -74}, {81, -70}, {81, -78}, {97, -74}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Line(points = {{-40, -34}, {-37.2, -19.3}, {-34.3, -6.1}, {-30.8, 8.4}, {-27.3, 21}, {-23.7, 31.92}, {-19.5, 43.18}, {-15.3, 52.7}, {-10.3, 62}, {-4.6, 70.5}, {1.7, 78.1}, {8.8, 84.6}, {17.3, 90.3}, {27.9, 95.2}, {30, 96}, {32.12, 83.5}, {34.95, 68.7}, {37.78, 55.8}, {40.606, 44.45}, {44.14, 32.3}, {47.68, 22}, {51.9, 11.8}, {56.2, 3.4}, {61.1, -4.5}, {66.8, -11.4}, {73.1, -17.3}, {80.9, -22.5}, {90.8, -26.8}, {100, -29.4}}, color = {0, 0, 255}, thickness = 0.5), Polygon(points = {{-80, 86}, {-86, 64}, {-74, 64}, {-80, 86}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Line(points = {{-80, 64}, {-80, -84}}, color = {95, 95, 95}), Text(extent = {{-81, 87}, {-40, 67}}, lineColor = {95, 95, 95}, textString = "y"), Text(extent = {{-71, -46}, {-38, -55}}, lineColor = {0, 0, 0}, textString = "offset"), Polygon(points = {{-40, -74}, {-42, -64}, {-38, -64}, {-40, -74}, {-40, -74}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Line(points = {{-40, -34}, {-40, -74}}, color = {95, 95, 95}), Polygon(points = {{-40, -34}, {-42, -44}, {-38, -44}, {-40, -34}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Line(points = {{-39, -34}, {-80, -34}}, color = {0, 0, 255}, thickness = 0.5), Text(extent = {{-62, -76}, {-17, -85}}, lineColor = {0, 0, 0}, textString = "startTime"), Polygon(points = {{-40, -34}, {-31, -32}, {-31, -36}, {-40, -34}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Line(points = {{-40, -34}, {30, -34}}, color = {95, 95, 95}), Polygon(points = {{30, -34}, {22, -32}, {22, -36}, {30, -34}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Text(extent = {{-26, -22}, {19, -32}}, lineColor = {0, 0, 0}, textString = "riseTime"), Text(extent = {{75, -79}, {98, -90}}, lineColor = {0, 0, 0}, textString = "time"), Line(points = {{30, 96}, {30, -38}}, color = {95, 95, 95})}),
          Documentation(info = "<html>
<p>
The Real output y is a rising exponential followed
by a falling exponential signal:
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Blocks/Sources/Exponentials.png\"
     alt=\"Exponentials.png\">
</p>
</html>"));
      end Exponentials;

      block Pulse "Generate pulse signal of type Real"
        parameter Real amplitude = 1 "Amplitude of pulse";
        parameter Real width(final min = Modelica.Constants.small, final max = 100) = 50 "Width of pulse in % of period";
        parameter Modelica.SIunits.Time period(final min = Modelica.Constants.small, start = 1) "Time for one period";
        parameter Integer nperiod = -1 "Number of periods (< 0 means infinite number of periods)";
        parameter Real offset = 0 "Offset of output signals";
        parameter Modelica.SIunits.Time startTime = 0 "Output = offset for time < startTime";
        extends Modelica.Blocks.Interfaces.SO;
      protected
        Modelica.SIunits.Time T_width = period * width / 100;
        Modelica.SIunits.Time T_start "Start time of current period";
        Integer count "Period count";
      initial algorithm
        count := integer((time - startTime) / period);
        T_start := startTime + count * period;
      equation
        when integer((time - startTime) / period) > pre(count) then
          count = pre(count) + 1;
          T_start = time;
        end when;
        y = offset + (if time < startTime or nperiod == 0 or nperiod > 0 and count >= nperiod then 0 else if time < T_start + T_width then amplitude else 0);
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-80, 68}, {-80, -80}}, color = {192, 192, 192}), Polygon(points = {{-80, 90}, {-88, 68}, {-72, 68}, {-80, 90}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-90, -70}, {82, -70}}, color = {192, 192, 192}), Polygon(points = {{90, -70}, {68, -62}, {68, -78}, {90, -70}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-80, -70}, {-40, -70}, {-40, 44}, {0, 44}, {0, -70}, {40, -70}, {40, 44}, {79, 44}}, color = {0, 0, 0}), Text(extent = {{-147, -152}, {153, -112}}, lineColor = {0, 0, 0}, textString = "period=%period")}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{-80, 90}, {-85, 68}, {-75, 68}, {-80, 90}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Line(points = {{-80, 68}, {-80, -80}}, color = {95, 95, 95}), Line(points = {{-90, -70}, {82, -70}}, color = {95, 95, 95}), Polygon(points = {{90, -70}, {68, -65}, {68, -75}, {90, -70}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Polygon(points = {{-34, 0}, {-37, -13}, {-31, -13}, {-34, 0}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Line(points = {{-34, 0}, {-34, -70}}, color = {95, 95, 95}), Polygon(points = {{-34, -70}, {-37, -57}, {-31, -57}, {-34, -70}, {-34, -70}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Text(extent = {{-78, -24}, {-35, -36}}, lineColor = {0, 0, 0}, textString = "offset"), Text(extent = {{-30, -72}, {16, -81}}, lineColor = {0, 0, 0}, textString = "startTime"), Text(extent = {{-82, 96}, {-49, 79}}, lineColor = {0, 0, 0}, textString = "y"), Text(extent = {{66, -80}, {87, -90}}, lineColor = {0, 0, 0}, textString = "time"), Line(points = {{-10, 0}, {-10, -70}}, color = {95, 95, 95}), Line(points = {{-80, 0}, {-10, 0}, {-10, 50}, {30, 50}, {30, 0}, {50, 0}, {50, 50}, {90, 50}}, color = {0, 0, 255}, thickness = 0.5), Line(points = {{-10, 88}, {-10, 50}}, color = {95, 95, 95}), Line(points = {{30, 74}, {30, 50}}, color = {95, 95, 95}), Line(points = {{50, 88}, {50, 50}}, color = {95, 95, 95}), Line(points = {{-10, 83}, {50, 83}}, color = {95, 95, 95}), Line(points = {{-10, 69}, {30, 69}}, color = {95, 95, 95}), Text(extent = {{-3, 93}, {39, 84}}, lineColor = {0, 0, 0}, textString = "period"), Text(extent = {{-7, 78}, {30, 69}}, lineColor = {0, 0, 0}, textString = "width"), Line(points = {{-43, 50}, {-10, 50}}, color = {95, 95, 95}), Line(points = {{-34, 50}, {-34, 0}}, color = {95, 95, 95}), Text(extent = {{-77, 30}, {-37, 21}}, lineColor = {0, 0, 0}, textString = "amplitude"), Polygon(points = {{-34, 50}, {-37, 37}, {-31, 37}, {-34, 50}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Polygon(points = {{-34, 0}, {-37, 13}, {-31, 13}, {-34, 0}, {-34, 0}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Line(points = {{90, 50}, {90, 0}, {100, 0}}, color = {0, 0, 255}, thickness = 0.5), Polygon(points = {{-10, 69}, {-1, 71}, {-1, 67}, {-10, 69}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Polygon(points = {{30, 69}, {22, 71}, {22, 67}, {30, 69}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Polygon(points = {{-10, 83}, {-1, 85}, {-1, 81}, {-10, 83}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Polygon(points = {{50, 83}, {42, 85}, {42, 81}, {50, 83}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid)}),
          Documentation(info = "<html>
<p>
The Real output y is a pulse signal:
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Blocks/Sources/Pulse.png\"
     alt=\"Pulse.png\">
</p>
</html>"));
      end Pulse;

      block SawTooth "Generate saw tooth signal"
        parameter Real amplitude = 1 "Amplitude of saw tooth";
        parameter SIunits.Time period(final min = Modelica.Constants.small, start = 1) "Time for one period";
        parameter Integer nperiod = -1 "Number of periods (< 0 means infinite number of periods)";
        parameter Real offset = 0 "Offset of output signals";
        parameter SIunits.Time startTime = 0 "Output = offset for time < startTime";
        extends Interfaces.SO;
      protected
        SIunits.Time T_start(final start = startTime) "Start time of current period";
        Integer count "Period count";
      initial algorithm
        count := integer((time - startTime) / period);
        T_start := startTime + count * period;
      equation
        when integer((time - startTime) / period) > pre(count) then
          count = pre(count) + 1;
          T_start = time;
        end when;
        y = offset + (if time < startTime or nperiod == 0 or nperiod > 0 and count >= nperiod then 0 else amplitude * (time - T_start) / period);
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-80, 68}, {-80, -80}}, color = {192, 192, 192}), Polygon(points = {{-80, 90}, {-88, 68}, {-72, 68}, {-80, 90}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-90, -70}, {82, -70}}, color = {192, 192, 192}), Polygon(points = {{90, -70}, {68, -62}, {68, -78}, {90, -70}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-80, -70}, {-60, -70}, {0, 40}, {0, -70}, {60, 41}, {60, -70}}, color = {0, 0, 0}), Text(extent = {{-147, -152}, {153, -112}}, lineColor = {0, 0, 0}, textString = "period=%period")}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{-80, 90}, {-86, 68}, {-74, 68}, {-80, 90}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Line(points = {{-80, 68}, {-80, -80}}, color = {95, 95, 95}), Line(points = {{-90, -70}, {82, -70}}, color = {95, 95, 95}), Polygon(points = {{90, -70}, {68, -65}, {68, -75}, {90, -70}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Polygon(points = {{-34, -20}, {-37, -33}, {-31, -33}, {-34, -20}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Line(points = {{-34, -20}, {-34, -70}}, color = {95, 95, 95}), Polygon(points = {{-34, -70}, {-37, -57}, {-31, -57}, {-34, -70}, {-34, -70}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Text(extent = {{-65, -39}, {-29, -47}}, lineColor = {0, 0, 0}, textString = "offset"), Text(extent = {{-29, -72}, {13, -80}}, lineColor = {0, 0, 0}, textString = "startTime"), Text(extent = {{-82, 92}, {-43, 76}}, lineColor = {0, 0, 0}, textString = "y"), Text(extent = {{67, -78}, {88, -87}}, lineColor = {0, 0, 0}, textString = "time"), Line(points = {{-10, -20}, {-10, -70}}, color = {95, 95, 95}), Line(points = {{-10, 88}, {-10, -20}}, color = {95, 95, 95}), Line(points = {{30, 88}, {30, 59}}, color = {95, 95, 95}), Line(points = {{-10, 83}, {30, 83}}, color = {95, 95, 95}), Text(extent = {{-12, 94}, {34, 85}}, lineColor = {0, 0, 0}, textString = "period"), Line(points = {{-44, 60}, {30, 60}}, color = {95, 95, 95}), Line(points = {{-34, 47}, {-34, -20}}, color = {95, 95, 95}), Text(extent = {{-73, 25}, {-36, 16}}, lineColor = {0, 0, 0}, textString = "amplitude"), Polygon(points = {{-34, 60}, {-37, 47}, {-31, 47}, {-34, 60}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Polygon(points = {{-34, -20}, {-37, -7}, {-31, -7}, {-34, -20}, {-34, -20}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Polygon(points = {{-10, 83}, {-1, 85}, {-1, 81}, {-10, 83}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Polygon(points = {{30, 83}, {22, 85}, {22, 81}, {30, 83}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Line(points = {{-80, -20}, {-10, -20}, {30, 60}, {30, -20}, {72, 60}, {72, -20}}, color = {0, 0, 255}, thickness = 0.5)}),
          Documentation(info = "<html>
<p>
The Real output y is a saw tooth signal:
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Blocks/Sources/SawTooth.png\"
     alt=\"SawTooth.png\">
</p>
</html>"));
      end SawTooth;

      block Trapezoid "Generate trapezoidal signal of type Real"
        parameter Real amplitude = 1 "Amplitude of trapezoid";
        parameter SIunits.Time rising(final min = 0) = 0 "Rising duration of trapezoid";
        parameter SIunits.Time width(final min = 0) = 0.5 "Width duration of trapezoid";
        parameter SIunits.Time falling(final min = 0) = 0 "Falling duration of trapezoid";
        parameter SIunits.Time period(final min = Modelica.Constants.small, start = 1) "Time for one period";
        parameter Integer nperiod = -1 "Number of periods (< 0 means infinite number of periods)";
        parameter Real offset = 0 "Offset of output signal";
        parameter SIunits.Time startTime = 0 "Output = offset for time < startTime";
        extends Interfaces.SO;
      protected
        parameter SIunits.Time T_rising = rising "End time of rising phase within one period";
        parameter SIunits.Time T_width = T_rising + width "End time of width phase within one period";
        parameter SIunits.Time T_falling = T_width + falling "End time of falling phase within one period";
        SIunits.Time T_start "Start time of current period";
        Integer count "Period count";
      initial algorithm
        count := integer((time - startTime) / period);
        T_start := startTime + count * period;
      equation
        when integer((time - startTime) / period) > pre(count) then
          count = pre(count) + 1;
          T_start = time;
        end when;
        y = offset + (if time < startTime or nperiod == 0 or nperiod > 0 and count >= nperiod then 0 else if time < T_start + T_rising then amplitude * (time - T_start) / rising else if time < T_start + T_width then amplitude else if time < T_start + T_falling then amplitude * (T_start + T_falling - time) / falling else 0);
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-80, 68}, {-80, -80}}, color = {192, 192, 192}), Polygon(points = {{-80, 90}, {-88, 68}, {-72, 68}, {-80, 90}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-90, -70}, {82, -70}}, color = {192, 192, 192}), Polygon(points = {{90, -70}, {68, -62}, {68, -78}, {90, -70}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Text(extent = {{-147, -152}, {153, -112}}, lineColor = {0, 0, 0}, textString = "period=%period"), Line(points = {{-81, -70}, {-60, -70}, {-30, 40}, {9, 40}, {39, -70}, {61, -70}, {90, 40}}, color = {0, 0, 0})}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{-81, 90}, {-87, 68}, {-75, 68}, {-81, 90}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Line(points = {{-81, 68}, {-81, -80}}, color = {95, 95, 95}), Line(points = {{-91, -70}, {81, -70}}, color = {95, 95, 95}), Polygon(points = {{89, -70}, {67, -65}, {67, -76}, {89, -70}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Polygon(points = {{-46, -30}, {-48, -41}, {-44, -41}, {-46, -30}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Line(points = {{-46, -30}, {-46, -70}}, color = {95, 95, 95}, pattern = LinePattern.Solid, arrow = {Arrow.None, Arrow.None}), Polygon(points = {{-46, -70}, {-48, -60}, {-44, -60}, {-46, -70}, {-46, -70}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Text(extent = {{-80, -46}, {-42, -55}}, lineColor = {0, 0, 0}, textString = "offset"), Text(extent = {{-49, -71}, {-6, -81}}, lineColor = {0, 0, 0}, textString = "startTime"), Text(extent = {{-80, 95}, {-47, 80}}, lineColor = {0, 0, 0}, textString = "y"), Text(extent = {{66, -78}, {89, -89}}, lineColor = {0, 0, 0}, textString = "time"), Line(points = {{-31, 82}, {-31, -70}}, color = {95, 95, 95}, pattern = LinePattern.Dash), Line(points = {{-11, 59}, {-11, 40}}, color = {95, 95, 95}, pattern = LinePattern.Dash), Line(points = {{19, 59}, {19, 40}}, color = {95, 95, 95}, pattern = LinePattern.Dash), Line(points = {{39, 59}, {39, -30}}, color = {95, 95, 95}, pattern = LinePattern.Dash), Line(points = {{-31, 76}, {59, 76}}, color = {95, 95, 95}), Line(points = {{-31, 56}, {39, 56}}, color = {95, 95, 95}), Text(extent = {{-3, 86}, {24, 77}}, lineColor = {0, 0, 0}, textString = "period"), Text(extent = {{-11, 68}, {18, 59}}, lineColor = {0, 0, 0}, textString = "width"), Line(points = {{-43, 40}, {-11, 40}}, color = {95, 95, 95}, pattern = LinePattern.Dash), Line(points = {{-40, 40}, {-40, -30}}, color = {95, 95, 95}, pattern = LinePattern.Solid, arrow = {Arrow.None, Arrow.None}), Text(extent = {{-77, 11}, {-44, 1}}, lineColor = {0, 0, 0}, textString = "amplitude"), Polygon(points = {{-31, 56}, {-24, 58}, {-24, 54}, {-31, 56}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Polygon(points = {{-11, 56}, {-18, 58}, {-18, 54}, {-11, 56}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Polygon(points = {{-31, 76}, {-22, 78}, {-22, 74}, {-31, 76}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Polygon(points = {{59, 76}, {51, 78}, {51, 74}, {59, 76}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Line(points = {{-81, -30}, {-31, -30}, {-11, 40}, {19, 40}, {39, -30}, {59, -30}, {79, 40}, {99, 40}}, color = {0, 0, 255}, thickness = 0.5), Polygon(points = {{-40, 40}, {-42, 29}, {-38, 29}, {-40, 40}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Polygon(points = {{-40, -30}, {-42, -20}, {-38, -20}, {-40, -30}, {-40, -30}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Line(points = {{59, 84}, {59, -30}}, color = {95, 95, 95}, pattern = LinePattern.Dash), Polygon(points = {{39, 56}, {32, 58}, {32, 54}, {39, 56}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Polygon(points = {{19, 56}, {26, 58}, {26, 54}, {19, 56}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Polygon(points = {{19, 56}, {12, 58}, {12, 54}, {19, 56}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Polygon(points = {{-11, 56}, {-4, 58}, {-4, 54}, {-11, 56}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Text(extent = {{-35, 68}, {-6, 60}}, lineColor = {0, 0, 0}, textString = "rising"), Text(extent = {{16, 68}, {44, 60}}, lineColor = {0, 0, 0}, textString = "falling")}),
          Documentation(info = "<html>
<p>
The Real output y is a trapezoid signal:
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Blocks/Sources/Trapezoid.png\"
     alt=\"Trapezoid\">
</p>
</html>"));
      end Trapezoid;

      block KinematicPTP "Move as fast as possible along a distance within given kinematic constraints"
        parameter Real deltaq[:] = {1} "Distance to move";
        parameter Real qd_max[:](each final min = Modelica.Constants.small) = {1} "Maximum velocities der(q)";
        parameter Real qdd_max[:](each final min = Modelica.Constants.small) = {1} "Maximum accelerations der(qd)";
        parameter SIunits.Time startTime = 0 "Time instant at which movement starts";
        extends Interfaces.MO(final nout = max([size(deltaq, 1); size(qd_max, 1); size(qdd_max, 1)]));
      protected
        parameter Real p_deltaq[nout] = if size(deltaq, 1) == 1 then ones(nout) * deltaq[1] else deltaq;
        parameter Real p_qd_max[nout] = if size(qd_max, 1) == 1 then ones(nout) * qd_max[1] else qd_max;
        parameter Real p_qdd_max[nout] = if size(qdd_max, 1) == 1 then ones(nout) * qdd_max[1] else qdd_max;
        Real sd_max;
        Real sdd_max;
        Real sdd;
        Real aux1[nout];
        Real aux2[nout];
        SIunits.Time Ta1;
        SIunits.Time Ta2;
        SIunits.Time Tv;
        SIunits.Time Te;
        Boolean noWphase;
      equation
        for i in 1:nout loop
          aux1[i] = p_deltaq[i] / p_qd_max[i];
          aux2[i] = p_deltaq[i] / p_qdd_max[i];
        end for;
        sd_max = 1 / max(abs(aux1));
        sdd_max = 1 / max(abs(aux2));
        Ta1 = sqrt(1 / sdd_max);
        Ta2 = sd_max / sdd_max;
        noWphase = Ta2 >= Ta1;
        Tv = if noWphase then Ta1 else 1 / sd_max;
        Te = if noWphase then Ta1 + Ta1 else Tv + Ta2;
// path-acceleration
        sdd = if time < startTime then 0 else if noWphase then if time < Ta1 + startTime then sdd_max else if time < Te + startTime then -sdd_max else 0 else if time < Ta2 + startTime then sdd_max else if time < Tv + startTime then 0 else if time < Te + startTime then -sdd_max else 0;
// acceleration
        y = p_deltaq * sdd;
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-80, 78}, {-80, -82}}, color = {192, 192, 192}), Polygon(points = {{-80, 90}, {-88, 68}, {-72, 68}, {-80, 88}, {-80, 90}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-90, 0}, {82, 0}}, color = {192, 192, 192}), Polygon(points = {{90, 0}, {68, 8}, {68, -8}, {90, 0}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-80, 0}, {-70, 0}, {-70, 70}, {-30, 70}, {-30, 0}, {20, 0}, {20, -70}, {60, -70}, {60, 0}, {68, 0}}, color = {0, 0, 0}, thickness = 0.25), Text(extent = {{2, 80}, {80, 20}}, lineColor = {192, 192, 192}, textString = "acc"), Text(extent = {{-150, -150}, {150, -110}}, lineColor = {0, 0, 0}, textString = "deltaq=%deltaq")}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-80, 78}, {-80, -72}}, color = {95, 95, 95}), Polygon(points = {{-80, 91}, {-86, 71}, {-75, 71}, {-80, 91}, {-80, 91}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Line(points = {{-90, 0}, {82, 0}}, color = {95, 95, 95}), Polygon(points = {{89, 0}, {68, 5}, {68, -5}, {89, 0}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Line(points = {{-80, 0}, {-70, 0}, {-70, 70}, {-30, 70}, {-30, 0}, {20, 0}, {20, -70}, {60, -70}, {60, 0}, {68, 0}}, color = {0, 0, 255}, thickness = 0.5), Text(extent = {{-73, 95}, {-16, 80}}, lineColor = {0, 0, 0}, textString = "acceleration"), Text(extent = {{66, 20}, {88, 8}}, lineColor = {0, 0, 0}, textString = "time")}),
          Documentation(info = "<HTML>
<p>
The goal is to move as <b>fast</b> as possible along a distance
<b>deltaq</b>
under given <b>kinematical constraints</b>. The distance can be a positional or
angular range. In robotics such a movement is called <b>PTP</b> (Point-To-Point).
This source block generates the <b>acceleration</b> qdd of this signal
as output:
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Blocks/Sources/KinematicPTP.png\"
     alt=\"KinematicPTP.png\">
</p>

<p>
After integrating the output two times, the position q is
obtained. The signal is constructed in such a way that it is not possible
to move faster, given the <b>maximally</b> allowed <b>velocity</b> qd_max and
the <b>maximally</b> allowed <b>acceleration</b> qdd_max.
</p>
<p>
If several distances are given (vector deltaq has more than 1 element),
an acceleration output vector is constructed such that all signals
are in the same periods in the acceleration, constant velocity
and deceleration phase. This means that only one of the signals
is at its limits whereas the others are synchronized in such a way
that the end point is reached at the same time instant.
</p>

<p>
This element is useful to generate a reference signal for a controller
which controls a drive train or in combination with model
Modelica.Mechanics.Rotational.<b>Accelerate</b> to drive
a flange according to a given acceleration.
</p>

</html>", revisions = "<html>
<p><b>Release Notes:</b></p>
<ul>
<li><i>June 27, 2001</i>
       by Bernhard Bachmann.<br>
       Bug fixed that element is also correct if startTime is not zero.</li>
<li><i>Nov. 3, 1999</i>
       by <a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a>:<br>
       Vectorized and moved from Rotational to Blocks.Sources.</li>
<li><i>June 29, 1999</i>
       by <a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a>:<br>
       realized.</li>
</ul>
</html>"));
      end KinematicPTP;

      block KinematicPTP2 "Move as fast as possible from start to end position within given kinematic constraints with output signals q, qd=der(q), qdd=der(qd)"
        parameter Real q_begin[:] = {0} "Start position";
        parameter Real q_end[:] = {1} "End position";
        parameter Real qd_max[:](each final min = Modelica.Constants.small) = {1} "Maximum velocities der(q)";
        parameter Real qdd_max[:](each final min = Modelica.Constants.small) = {1} "Maximum accelerations der(qd)";
        parameter Modelica.SIunits.Time startTime = 0 "Time instant at which movement starts";
        extends Modelica.Blocks.Icons.Block;
        final parameter Integer nout = max([size(q_begin, 1); size(q_end, 1); size(qd_max, 1); size(qdd_max, 1)]) "Number of output signals (= dimension of q, qd, qdd, moving)";
        output Modelica.SIunits.Time endTime "Time instant at which movement stops";
        Modelica.Blocks.Interfaces.RealOutput q[nout] "Reference position of path planning" annotation(
          Placement(transformation(extent = {{100, 70}, {120, 90}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealOutput qd[nout] "Reference speed of path planning" annotation(
          Placement(transformation(extent = {{100, 20}, {120, 40}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealOutput qdd[nout] "Reference acceleration of path planning" annotation(
          Placement(transformation(extent = {{100, -40}, {120, -20}}, rotation = 0)));
        Modelica.Blocks.Interfaces.BooleanOutput moving[nout] "= true, if end position not yet reached; = false, if end position reached or axis is completely at rest" annotation(
          Placement(transformation(extent = {{100, -90}, {120, -70}}, rotation = 0)));
      protected
        parameter Real p_q_begin[nout] = if size(q_begin, 1) == 1 then ones(nout) * q_begin[1] else q_begin;
        parameter Real p_q_end[nout] = if size(q_end, 1) == 1 then ones(nout) * q_end[1] else q_end;
        parameter Real p_qd_max[nout] = if size(qd_max, 1) == 1 then ones(nout) * qd_max[1] else qd_max;
        parameter Real p_qdd_max[nout] = if size(qdd_max, 1) == 1 then ones(nout) * qdd_max[1] else qdd_max;
        parameter Real p_deltaq[nout] = p_q_end - p_q_begin;
        constant Real eps = 10 * Modelica.Constants.eps;
        Boolean motion_ref;
        Real sd_max_inv;
        Real sdd_max_inv;
        Real sd_max;
        Real sdd_max;
        Real sdd;
        Real aux1[nout];
        Real aux2[nout];
        SI.Time Ta1;
        SI.Time Ta2;
        SI.Time Tv;
        SI.Time Te;
        Boolean noWphase;
        SI.Time Ta1s;
        SI.Time Ta2s;
        SI.Time Tvs;
        SI.Time Tes;
        Real sd_max2;
        Real s1;
        Real s2;
        Real s3;
        Real s;
        Real sd;
      equation
        for i in 1:nout loop
          aux1[i] = p_deltaq[i] / p_qd_max[i];
          aux2[i] = p_deltaq[i] / p_qdd_max[i];
        end for;
        sd_max_inv = max(abs(aux1));
        sdd_max_inv = max(abs(aux2));
        if sd_max_inv <= eps or sdd_max_inv <= eps then
          sd_max = 0;
          sdd_max = 0;
          Ta1 = 0;
          Ta2 = 0;
          noWphase = false;
          Tv = 0;
          Te = 0;
          Ta1s = 0;
          Ta2s = 0;
          Tvs = 0;
          Tes = 0;
          sd_max2 = 0;
          s1 = 0;
          s2 = 0;
          s3 = 0;
          s = 0;
        else
          sd_max = 1 / max(abs(aux1));
          sdd_max = 1 / max(abs(aux2));
          Ta1 = sqrt(1 / sdd_max);
          Ta2 = sd_max / sdd_max;
          noWphase = Ta2 >= Ta1;
          Tv = if noWphase then Ta1 else 1 / sd_max;
          Te = if noWphase then Ta1 + Ta1 else Tv + Ta2;
          Ta1s = Ta1 + startTime;
          Ta2s = Ta2 + startTime;
          Tvs = Tv + startTime;
          Tes = Te + startTime;
          sd_max2 = sdd_max * Ta1;
          s1 = sdd_max * (if noWphase then Ta1 * Ta1 else Ta2 * Ta2) / 2;
          s2 = s1 + (if noWphase then sd_max2 * (Te - Ta1) - sdd_max / 2 * (Te - Ta1) ^ 2 else sd_max * (Tv - Ta2));
          s3 = s2 + sd_max * (Te - Tv) - sdd_max / 2 * (Te - Tv) * (Te - Tv);
          if time < startTime then
            s = 0;
          elseif noWphase then
            if time < Ta1s then
              s = sdd_max / 2 * (time - startTime) * (time - startTime);
            elseif time < Tes then
              s = s1 + sd_max2 * (time - Ta1s) - sdd_max / 2 * (time - Ta1s) * (time - Ta1s);
            else
              s = s2;
            end if;
          elseif time < Ta2s then
            s = sdd_max / 2 * (time - startTime) * (time - startTime);
          elseif time < Tvs then
            s = s1 + sd_max * (time - Ta2s);
          elseif time < Tes then
            s = s2 + sd_max * (time - Tvs) - sdd_max / 2 * (time - Tvs) * (time - Tvs);
          else
            s = s3;
          end if;
        end if;
        sd = der(s);
        sdd = der(sd);
        qdd = p_deltaq * sdd;
        qd = p_deltaq * sd;
        q = p_q_begin + p_deltaq * s;
        endTime = Tes;
// report when axis is moving
        motion_ref = time <= endTime;
        for i in 1:nout loop
          moving[i] = if abs(q_begin[i] - q_end[i]) > eps then motion_ref else false;
        end for;
        annotation(
          defaultComponentName = "kinematicPTP",
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-80, 78}, {-80, -82}}, color = {192, 192, 192}), Polygon(points = {{-80, 90}, {-88, 68}, {-72, 68}, {-80, 88}, {-80, 90}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-90, 0}, {17, 0}}, color = {192, 192, 192}), Line(points = {{-80, 0}, {-70, 0}, {-70, 70}, {-50, 70}, {-50, 0}, {-15, 0}, {-15, -70}, {5, -70}, {5, 0}, {18, 0}}, color = {0, 0, 0}, thickness = 0.25), Text(extent = {{34, 96}, {94, 66}}, lineColor = {0, 0, 0}, textString = "q"), Text(extent = {{40, 44}, {96, 14}}, lineColor = {0, 0, 0}, textString = "qd"), Text(extent = {{32, -18}, {99, -44}}, lineColor = {0, 0, 0}, textString = "qdd"), Text(extent = {{-32, -74}, {97, -96}}, lineColor = {0, 0, 0}, textString = "moving")}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-80, 78}, {-80, -70}}, color = {95, 95, 95}), Polygon(points = {{-80, 94}, {-86, 74}, {-74, 74}, {-80, 94}, {-80, 94}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Line(points = {{-90, 0}, {82, 0}}, color = {95, 95, 95}), Polygon(points = {{90, 0}, {68, 6}, {68, -6}, {90, 0}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Line(points = {{-80, 0}, {-70, 0}, {-70, 70}, {-30, 70}, {-30, 0}, {20, 0}, {20, -70}, {60, -70}, {60, 0}, {68, 0}}, color = {0, 0, 255}, thickness = 0.5), Text(extent = {{-72, 96}, {-15, 81}}, lineColor = {0, 0, 0}, textString = "acceleration"), Text(extent = {{69, 18}, {91, 6}}, lineColor = {0, 0, 0}, textString = "time")}),
          Documentation(info = "<html>
<p>
The goal is to move as <b>fast</b> as possible from start position <b>q_begin</b>
to end position <b>q_end</b>
under given <b>kinematical constraints</b>. The positions can be translational or
rotational definitions (i.e., q_begin/q_end is given). In robotics such a movement is called <b>PTP</b> (Point-To-Point).
This source block generates the <b>position</b> q(t), the
<b>speed</b> qd(t) = der(q), and the <b>acceleration</b> qdd = der(qd)
as output. The signals are constructed in such a way that it is not possible
to move faster, given the <b>maximally</b> allowed <b>velocity</b> qd_max and
the <b>maximally</b> allowed <b>acceleration</b> qdd_max:
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Blocks/Sources/KinematicPTP2.png\"
     alt=\"KinematicPTP2.png\">
</p>

<p>
If vectors q_begin/q_end have more than 1 element,
the output vectors are constructed such that all signals
are in the same periods in the acceleration, constant velocity
and deceleration phase. This means that only one of the signals
is at its limits whereas the others are synchronized in such a way
that the end point is reached at the same time instant.
</p>

<p>
This element is useful to generate a reference signal for a controller
which controls, e.g., a drive train, or to drive
a flange according to a given acceleration.
</p>

</html>", revisions = "<html>
<ul>
<li><i>March 24, 2007</i>
       by <a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a>:<br>
       Non-standard Modelica function \"constrain(..)\" replaced by standard
       Modelica implementation (via internal function position()).<br>
       New output signal \"moving\" added.</li>
<li><i>June 27, 2001</i>
       by Bernhard Bachmann.<br>
       Bug fixed that element is also correct if startTime is not zero.</li>
<li><i>Nov. 3, 1999</i>
       by <a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a>:<br>
       Vectorized and moved from Rotational to Blocks.Sources.</li>
<li><i>June 29, 1999</i>
       by <a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a>:<br>
       realized.</li>
</ul>
</html>"));
      end KinematicPTP2;

      block TimeTable "Generate a (possibly discontinuous) signal by linear interpolation in a table"
        parameter Real table[:, 2] = fill(0.0, 0, 2) "Table matrix (time = first column; e.g., table=[0, 0; 1, 1; 2, 4])";
        parameter Real offset = 0 "Offset of output signal";
        parameter SIunits.Time startTime = 0 "Output = offset for time < startTime";
        extends Interfaces.SO;
      protected
        Real a "Interpolation coefficients a of actual interval (y=a*x+b)";
        Real b "Interpolation coefficients b of actual interval (y=a*x+b)";
        Integer last(start = 1) "Last used lower grid index";
        discrete SIunits.Time nextEvent(start = 0, fixed = true) "Next event instant";

        function getInterpolationCoefficients "Determine interpolation coefficients and next time event"
          extends Modelica.Icons.Function;
          input Real table[:, 2] "Table for interpolation";
          input Real offset "y-offset";
          input Real startTime "time-offset";
          input Real t "Actual time instant";
          input Integer last "Last used lower grid index";
          input Real TimeEps "Relative epsilon to check for identical time instants";
          output Real a "Interpolation coefficients a (y=a*x + b)";
          output Real b "Interpolation coefficients b (y=a*x + b)";
          output Real nextEvent "Next event instant";
          output Integer next "New lower grid index";
        protected
          Integer columns = 2 "Column to be interpolated";
          Integer ncol = 2 "Number of columns to be interpolated";
          Integer nrow = size(table, 1) "Number of table rows";
          Integer next0;
          Real tp;
          Real dt;
        algorithm
          next := last;
          nextEvent := t - TimeEps * abs(t);
// in case there are no more time events
          tp := t + TimeEps * abs(t) - startTime;
          if tp < 0.0 then
            nextEvent := startTime;
            a := 0;
            b := offset;
          elseif nrow < 2 then
            a := 0;
            b := offset + table[1, columns];
          else
            while next < nrow and tp >= table[next, 1] loop
              next := next + 1;
            end while;
            if next < nrow then
              nextEvent := startTime + table[next, 1];
            end if;
            next0 := next - 1;
            dt := table[next, 1] - table[next0, 1];
            if dt <= TimeEps * abs(table[next, 1]) then
              a := 0;
              b := offset + table[next, columns];
            else
              a := (table[next, columns] - table[next0, columns]) / dt;
              b := offset + table[next0, columns] - a * table[next0, 1];
            end if;
          end if;
// First event not yet reached
// Special action if table has only one row
// Find next time event instant. Note, that two consecutive time instants
// in the table may be identical due to a discontinuous point.
// Define next time event, if last table entry not reached
// Determine interpolation coefficients
// Interpolation interval is not big enough, use "next" value
// Take into account startTime "a*(time - startTime) + b"
          b := b - a * startTime;
        end getInterpolationCoefficients;
      algorithm
        when {time >= pre(nextEvent), initial()} then
          (a, b, nextEvent, last) := getInterpolationCoefficients(table, offset, startTime, time, last, 100 * Modelica.Constants.eps);
        end when;
      equation
        y = a * time + b;
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-80, 68}, {-80, -80}}, color = {192, 192, 192}), Polygon(points = {{-80, 90}, {-88, 68}, {-72, 68}, {-80, 90}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-90, -70}, {82, -70}}, color = {192, 192, 192}), Polygon(points = {{90, -70}, {68, -62}, {68, -78}, {90, -70}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-48, 70}, {2, -50}}, lineColor = {255, 255, 255}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-48, -50}, {-48, 70}, {52, 70}, {52, -50}, {-48, -50}, {-48, -20}, {52, -20}, {52, 10}, {-48, 10}, {-48, 40}, {52, 40}, {52, 70}, {2, 70}, {2, -51}}, color = {0, 0, 0}), Text(extent = {{-150, -150}, {150, -110}}, lineColor = {0, 0, 0}, textString = "offset=%offset")}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{-80, 90}, {-85, 68}, {-74, 68}, {-80, 90}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Line(points = {{-80, 68}, {-80, -80}}, color = {95, 95, 95}), Line(points = {{-90, -70}, {82, -70}}, color = {95, 95, 95}), Polygon(points = {{88, -70}, {68, -65}, {68, -74}, {88, -70}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-20, 90}, {30, -30}}, lineColor = {255, 255, 255}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-20, -30}, {-20, 90}, {80, 90}, {80, -30}, {-20, -30}, {-20, 0}, {80, 0}, {80, 30}, {-20, 30}, {-20, 60}, {80, 60}, {80, 90}, {30, 90}, {30, -31}}, color = {0, 0, 0}), Text(extent = {{-70, -42}, {-32, -54}}, lineColor = {0, 0, 0}, textString = "offset"), Polygon(points = {{-31, -30}, {-33, -40}, {-28, -40}, {-31, -30}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Polygon(points = {{-31, -70}, {-34, -60}, {-29, -60}, {-31, -70}, {-31, -70}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Line(points = {{-31, -32}, {-31, -70}}, color = {95, 95, 95}), Line(points = {{-20, -30}, {-20, -70}}, color = {95, 95, 95}), Text(extent = {{-38, -73}, {8, -83}}, lineColor = {0, 0, 0}, textString = "startTime"), Line(points = {{-20, -30}, {-80, -30}}, color = {95, 95, 95}), Text(extent = {{-76, 93}, {-44, 75}}, lineColor = {0, 0, 0}, textString = "y"), Text(extent = {{66, -78}, {90, -88}}, lineColor = {0, 0, 0}, textString = "time"), Text(extent = {{-15, 83}, {24, 68}}, lineColor = {0, 0, 0}, textString = "time"), Text(extent = {{33, 83}, {76, 67}}, lineColor = {0, 0, 0}, textString = "y")}),
          Documentation(info = "<HTML>
<p>
This block generates an output signal by <b>linear interpolation</b> in
a table. The time points and function values are stored in a matrix
<strong><code>table[i,j]</code></strong>, where the first column table[:,1] contains the
time points and the second column contains the data to be interpolated.
The table interpolation has the following properties:
</p>
<ul>
<li>The time points need to be <b>monotonically increasing</b>. </li>
<li><b>Discontinuities</b> are allowed, by providing the same
    time point twice in the table. </li>
<li>Values <b>outside</b> of the table range, are computed by
    <b>extrapolation</b> through the last or first two points of the
    table.</li>
<li>If the table has only <b>one row</b>, no interpolation is performed and
    the function value is just returned independently of the
    actual time instant.</li>
<li>Via parameters <strong><code>startTime</code></strong> and <strong><code>offset</code></strong> the curve defined
    by the table can be shifted both in time and in the ordinate value.</li>
<li>The first point in time <strong>always</strong> has to be set to <strong><code>0</code></strong>, e.g.,
    <strong><code>table=[1,1;2,2]</code></strong> is <strong>illegal</strong>. If you want to
    shift the time table in time use the  <strong><code>startTime</code></strong> parameter instead.</li>
<li>The table is implemented in a numerically sound way by
    generating <b>time events</b> at interval boundaries.
    This generates continuously differentiable values for the integrator.</li>
</ul>
<p>
Example:
</p>
<pre>
   table = [0  0
            1  0
            1  1
            2  4
            3  9
            4 16]
If, e.g., time = 1.0, the output y =  0.0 (before event), 1.0 (after event)
    e.g., time = 1.5, the output y =  2.5,
    e.g., time = 2.0, the output y =  4.0,
    e.g., time = 5.0, the output y = 23.0 (i.e., extrapolation).
</pre>

<p>
<img src=\"modelica://Modelica/Resources/Images/Blocks/Sources/TimeTable.png\"
     alt=\"TimeTable.png\">
</p>

</html>", revisions = "<html>
<h4>Release Notes</h4>
<ul>
<li><i>Oct. 21, 2002</i>
       by <a href=\"http://www.robotic.dlr.de/Christian.Schweiger/\">Christian Schweiger</a>:<br>
       Corrected interface from
<pre>
    parameter Real table[:, :]=[0, 0; 1, 1; 2, 4];
</pre>
       to
<pre>
    parameter Real table[:, <b>2</b>]=[0, 0; 1, 1; 2, 4];
</pre>
       </li>
<li><i>Nov. 7, 1999</i>
       by <a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a>:<br>
       Realized.</li>
</ul>
</html>"));
      end TimeTable;

      block CombiTimeTable "Table look-up with respect to time and linear/periodic extrapolation methods (data from matrix/file)"
        extends Modelica.Blocks.Interfaces.MO(final nout = max([size(columns, 1); size(offset, 1)]));
        parameter Boolean tableOnFile = false "= true, if table is defined on file or in function usertab" annotation(
          Dialog(group = "Table data definition"));
        parameter Real table[:, :] = fill(0.0, 0, 2) "Table matrix (time = first column; e.g., table=[0,2])" annotation(
          Dialog(group = "Table data definition", enable = not tableOnFile));
        parameter String tableName = "NoName" "Table name on file or in function usertab (see docu)" annotation(
          Dialog(group = "Table data definition", enable = tableOnFile));
        parameter String fileName = "NoName" "File where matrix is stored" annotation(
          Dialog(group = "Table data definition", enable = tableOnFile, loadSelector(filter = "Text files (*.txt);;MATLAB MAT-files (*.mat)", caption = "Open file in which table is present")));
        parameter Boolean verboseRead = true "= true, if info message that file is loading is to be printed" annotation(
          Dialog(group = "Table data definition", enable = tableOnFile));
        parameter Integer columns[:] = 2:size(table, 2) "Columns of table to be interpolated" annotation(
          Dialog(group = "Table data interpretation"));
        parameter Modelica.Blocks.Types.Smoothness smoothness = Modelica.Blocks.Types.Smoothness.LinearSegments "Smoothness of table interpolation" annotation(
          Dialog(group = "Table data interpretation"));
        parameter Modelica.Blocks.Types.Extrapolation extrapolation = Modelica.Blocks.Types.Extrapolation.LastTwoPoints "Extrapolation of data outside the definition range" annotation(
          Dialog(group = "Table data interpretation"));
        parameter Real offset[:] = {0} "Offsets of output signals" annotation(
          Dialog(group = "Table data interpretation"));
        parameter Modelica.SIunits.Time startTime = 0 "Output = offset for time < startTime" annotation(
          Dialog(group = "Table data interpretation"));
        final parameter Modelica.SIunits.Time t_min(fixed = false) "Minimum abscissa value defined in table";
        final parameter Modelica.SIunits.Time t_max(fixed = false) "Maximum abscissa value defined in table";
      protected
        final parameter Real p_offset[nout] = if size(offset, 1) == 1 then ones(nout) * offset[1] else offset "Offsets of output signals";
        Modelica.Blocks.Types.ExternalCombiTimeTable tableID = Modelica.Blocks.Types.ExternalCombiTimeTable(if tableOnFile then tableName else "NoName", if tableOnFile and fileName <> "NoName" and not Modelica.Utilities.Strings.isEmpty(fileName) then fileName else "NoName", table, startTime, columns, smoothness, extrapolation) "External table object";
        discrete Modelica.SIunits.Time nextTimeEvent(start = 0, fixed = true) "Next time event instant";
        parameter Real tableOnFileRead(fixed = false) "= 1, if table was successfully read from file";

        function readTableData "Read table data from ASCII text or MATLAB MAT-file"
          extends Modelica.Icons.Function;
          input Modelica.Blocks.Types.ExternalCombiTimeTable tableID;
          input Boolean forceRead = false "= true: Force reading of table data; = false: Only read, if not yet read.";
          output Real readSuccess "Table read success";
          input Boolean verboseRead "= true: Print info message; = false: No info message";

          external "C" readSuccess = ModelicaStandardTables_CombiTimeTable_read(tableID, forceRead, verboseRead) annotation(
            Library = {"ModelicaStandardTables"});
        end readTableData;

        function getTableValue "Interpolate 1-dim. table where first column is time"
          extends Modelica.Icons.Function;
          input Modelica.Blocks.Types.ExternalCombiTimeTable tableID;
          input Integer icol;
          input Modelica.SIunits.Time timeIn;
          discrete input Modelica.SIunits.Time nextTimeEvent;
          discrete input Modelica.SIunits.Time pre_nextTimeEvent;
          input Real tableAvailable "Dummy input to ensure correct sorting of function calls" annotation(
            __OpenModelica_UnusedVariable = true);
          output Real y;

          external "C" y = ModelicaStandardTables_CombiTimeTable_getValue(tableID, icol, timeIn, nextTimeEvent, pre_nextTimeEvent) annotation(
            Library = {"ModelicaStandardTables"});
          annotation(
            derivative(noDerivative = nextTimeEvent, noDerivative = pre_nextTimeEvent, noDerivative = tableAvailable) = getDerTableValue);
        end getTableValue;

        function getTableValueNoDer "Interpolate 1-dim. table where first column is time (but do not provide a derivative function)"
          extends Modelica.Icons.Function;
          input Modelica.Blocks.Types.ExternalCombiTimeTable tableID;
          input Integer icol;
          input Modelica.SIunits.Time timeIn;
          discrete input Modelica.SIunits.Time nextTimeEvent;
          discrete input Modelica.SIunits.Time pre_nextTimeEvent;
          input Real tableAvailable "Dummy input to ensure correct sorting of function calls" annotation(
            __OpenModelica_UnusedVariable = true);
          output Real y;

          external "C" y = ModelicaStandardTables_CombiTimeTable_getValue(tableID, icol, timeIn, nextTimeEvent, pre_nextTimeEvent) annotation(
            Library = {"ModelicaStandardTables"});
        end getTableValueNoDer;

        function getDerTableValue "Derivative of interpolated 1-dim. table where first column is time"
          extends Modelica.Icons.Function;
          input Modelica.Blocks.Types.ExternalCombiTimeTable tableID;
          input Integer icol;
          input Modelica.SIunits.Time timeIn;
          discrete input Modelica.SIunits.Time nextTimeEvent;
          discrete input Modelica.SIunits.Time pre_nextTimeEvent;
          input Real tableAvailable "Dummy input to ensure correct sorting of function calls" annotation(
            __OpenModelica_UnusedVariable = true);
          input Real der_timeIn;
          output Real der_y;

          external "C" der_y = ModelicaStandardTables_CombiTimeTable_getDerValue(tableID, icol, timeIn, nextTimeEvent, pre_nextTimeEvent, der_timeIn) annotation(
            Library = {"ModelicaStandardTables"});
        end getDerTableValue;

        function getTableTimeTmin "Return minimum time value of 1-dim. table where first column is time"
          extends Modelica.Icons.Function;
          input Modelica.Blocks.Types.ExternalCombiTimeTable tableID;
          input Real tableAvailable "Dummy input to ensure correct sorting of function calls" annotation(
            __OpenModelica_UnusedVariable = true);
          output Modelica.SIunits.Time timeMin "Minimum time value in table";

          external "C" timeMin = ModelicaStandardTables_CombiTimeTable_minimumTime(tableID) annotation(
            Library = {"ModelicaStandardTables"});
        end getTableTimeTmin;

        function getTableTimeTmax "Return maximum time value of 1-dim. table where first column is time"
          extends Modelica.Icons.Function;
          input Modelica.Blocks.Types.ExternalCombiTimeTable tableID;
          input Real tableAvailable "Dummy input to ensure correct sorting of function calls" annotation(
            __OpenModelica_UnusedVariable = true);
          output Modelica.SIunits.Time timeMax "Maximum time value in table";

          external "C" timeMax = ModelicaStandardTables_CombiTimeTable_maximumTime(tableID) annotation(
            Library = {"ModelicaStandardTables"});
        end getTableTimeTmax;

        function getNextTimeEvent "Return next time event value of 1-dim. table where first column is time"
          extends Modelica.Icons.Function;
          input Modelica.Blocks.Types.ExternalCombiTimeTable tableID;
          input Modelica.SIunits.Time timeIn;
          input Real tableAvailable "Dummy input to ensure correct sorting of function calls" annotation(
            __OpenModelica_UnusedVariable = true);
          output Modelica.SIunits.Time nextTimeEvent "Next time event in table";

          external "C" nextTimeEvent = ModelicaStandardTables_CombiTimeTable_nextTimeEvent(tableID, timeIn) annotation(
            Library = {"ModelicaStandardTables"});
        end getNextTimeEvent;
      initial algorithm
        if tableOnFile then
          tableOnFileRead := readTableData(tableID, false, verboseRead);
        else
          tableOnFileRead := 1.;
        end if;
        t_min := getTableTimeTmin(tableID, tableOnFileRead);
        t_max := getTableTimeTmax(tableID, tableOnFileRead);
      equation
        if tableOnFile then
          assert(tableName <> "NoName", "tableOnFile = true and no table name given");
        else
          assert(size(table, 1) > 0 and size(table, 2) > 0, "tableOnFile = false and parameter table is an empty matrix");
        end if;
        when {time >= pre(nextTimeEvent), initial()} then
          nextTimeEvent = getNextTimeEvent(tableID, time, tableOnFileRead);
        end when;
        if smoothness == Modelica.Blocks.Types.Smoothness.ConstantSegments then
          for i in 1:nout loop
            y[i] = p_offset[i] + getTableValueNoDer(tableID, i, time, nextTimeEvent, pre(nextTimeEvent), tableOnFileRead);
          end for;
        else
          for i in 1:nout loop
            y[i] = p_offset[i] + getTableValue(tableID, i, time, nextTimeEvent, pre(nextTimeEvent), tableOnFileRead);
          end for;
        end if;
        annotation(
          Documentation(info = "<HTML>
<p>
This block generates an output signal y[:] by <b>linear interpolation</b> in
a table. The time points and function values are stored in a matrix
<b>table[i,j]</b>, where the first column table[:,1] contains the
time points and the other columns contain the data to be interpolated.
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Blocks/Sources/CombiTimeTable.png\"
     alt=\"CombiTimeTable.png\">
</p>

<p>
Via parameter <b>columns</b> it can be defined which columns of the
table are interpolated. If, e.g., columns={2,4}, it is assumed that
2 output signals are present and that the first output is computed
by interpolation of column 2 and the second output is computed
by interpolation of column 4 of the table matrix.
The table interpolation has the following properties:
</p>
<ul>
<li>The time points need to be <b>strictly increasing</b> if smoothness
    is ContinuousDerivative, otherwise <b>monotonically increasing</b>.</li>
<li><b>Discontinuities</b> are allowed, by providing the same
    time point twice in the table. </li>
<li>Values <b>outside</b> of the table range, are computed by
    extrapolation according to the setting of parameter
    <b>extrapolation</b>:
<pre>
  extrapolation = 1: hold the first or last value of the table,
                     if outside of the table scope.
                = 2: extrapolate by using the derivative at the first/last table
                     points if outside of the table scope.
                     (If smoothness is LinearSegments or ConstantSegments
                     this means to extrapolate linearly through the first/last
                     two table points.).
                = 3: periodically repeat the table data
                     (periodical function).
                = 4: no extrapolation, i.e. extrapolation triggers an error
</pre></li>
<li>Via parameter <b>smoothness</b> it is defined how the data is interpolated:
<pre>
  smoothness = 1: linear interpolation
             = 2: smooth interpolation with Akima-splines such
                  that der(y) is continuous, also if extrapolated.
             = 3: constant segments
</pre></li>
<li>If the table has only <b>one row</b>, no interpolation is performed and
    the table values of this row are just returned.</li>
<li>Via parameters <b>startTime</b> and <b>offset</b> the curve defined
    by the table can be shifted both in time and in the ordinate value.
    The time instants stored in the table are therefore <b>relative</b>
    to <b>startTime</b>.
    If time &lt; startTime, no interpolation is performed and the offset
    is used as ordinate value for all outputs.</li>
<li>The table is implemented in a numerically sound way by
    generating <b>time events</b> at interval boundaries.
    This generates continuously differentiable values for the integrator.</li>
<li>For special applications it is sometimes needed to know the minimum
    and maximum time instant defined in the table as a parameter. For this
    reason parameters <b>t_min</b> and <b>t_max</b> are provided and can be
    accessed from the outside of the table object.</li>
</ul>
<p>
Example:
</p>
<pre>
   table = [0  0
            1  0
            1  1
            2  4
            3  9
            4 16]; extrapolation = 3 (default)
If, e.g., time = 1.0, the output y =  0.0 (before event), 1.0 (after event)
    e.g., time = 1.5, the output y =  2.5,
    e.g., time = 2.0, the output y =  4.0,
    e.g., time = 5.0, the output y = 23.0 (i.e., extrapolation via last 2 points).
</pre>
<p>
The table matrix can be defined in the following ways:
</p>
<ol>
<li> Explicitly supplied as <b>parameter matrix</b> \"table\",
     and the other parameters have the following values:
<pre>
   tableName is \"NoName\" or has only blanks,
   fileName  is \"NoName\" or has only blanks.
</pre></li>
<li> <b>Read</b> from a <b>file</b> \"fileName\" where the matrix is stored as
      \"tableName\". Both ASCII and MAT-file format is possible.
      (The ASCII format is described below).
      The MAT-file format comes in four different versions: v4, v6, v7 and v7.3.
      The library supports at least v4, v6 and v7 whereas v7.3 is optional.
      It is most convenient to generate the MAT-file from FreeMat or MATLAB&reg;
      by command
<pre>
   save tables.mat tab1 tab2 tab3
</pre>
      or Scilab by command
<pre>
   savematfile tables.mat tab1 tab2 tab3
</pre>
      when the three tables tab1, tab2, tab3 should be used from the model.<br>
      Note, a fileName can be defined as URI by using the helper function
      <a href=\"modelica://Modelica.Utilities.Files.loadResource\">loadResource</a>.</li>
<li>  Statically stored in function \"usertab\" in file \"usertab.c\".
      The matrix is identified by \"tableName\". Parameter
      fileName = \"NoName\" or has only blanks. Row-wise storage is always to be
      preferred as otherwise the table is reallocated and transposed.</li>
</ol>
<p>
When the constant \"NO_FILE_SYSTEM\" is defined, all file I/O related parts of the
source code are removed by the C-preprocessor, such that no access to files takes place.
</p>
<p>
If tables are read from an ASCII-file, the file needs to have the
following structure (\"-----\" is not part of the file content):
</p>
<pre>
-----------------------------------------------------
#1
double tab1(6,2)   # comment line
  0   0
  1   0
  1   1
  2   4
  3   9
  4  16
double tab2(6,2)   # another comment line
  0   0
  2   0
  2   2
  4   8
  6  18
  8  32
-----------------------------------------------------
</pre>
<p>
Note, that the first two characters in the file need to be
\"#1\" (a line comment defining the version number of the file format).
Afterwards, the corresponding matrix has to be declared
with type (= \"double\" or \"float\"), name and actual dimensions.
Finally, in successive rows of the file, the elements of the matrix
have to be given. The elements have to be provided as a sequence of
numbers in row-wise order (therefore a matrix row can span several
lines in the file and need not start at the beginning of a line).
Numbers have to be given according to C syntax (such as 2.3, -2, +2.e4).
Number separators are spaces, tab (\t), comma (,), or semicolon (;).
Several matrices may be defined one after another. Line comments start
with the hash symbol (#) and can appear everywhere.
Other characters, like trailing non comments, are not allowed in the file.
</p>
<p>
MATLAB is a registered trademark of The MathWorks, Inc.
</p>
</html>", revisions = "<html>
<p><b>Release Notes:</b></p>
<ul>
<li><i>April 09, 2013</i>
       by Thomas Beutlich:<br>
       Implemented as external object.</li>
<li><i>March 31, 2001</i>
       by <a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a>:<br>
       Used CombiTableTime as a basis and added the
       arguments <b>extrapolation, columns, startTime</b>.
       This allows periodic function definitions.</li>
</ul>
</html>"),
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100.0, -100.0}, {100.0, 100.0}}, initialScale = 0.1), graphics = {Polygon(visible = true, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid, points = {{-80.0, 90.0}, {-88.0, 68.0}, {-72.0, 68.0}, {-80.0, 90.0}}), Line(visible = true, points = {{-80.0, 68.0}, {-80.0, -80.0}}, color = {192, 192, 192}), Line(visible = true, points = {{-90.0, -70.0}, {82.0, -70.0}}, color = {192, 192, 192}), Polygon(visible = true, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid, points = {{90.0, -70.0}, {68.0, -62.0}, {68.0, -78.0}, {90.0, -70.0}}), Rectangle(visible = true, lineColor = {255, 255, 255}, fillColor = {255, 215, 136}, fillPattern = FillPattern.Solid, extent = {{-48.0, -50.0}, {2.0, 70.0}}), Line(visible = true, points = {{-48.0, -50.0}, {-48.0, 70.0}, {52.0, 70.0}, {52.0, -50.0}, {-48.0, -50.0}, {-48.0, -20.0}, {52.0, -20.0}, {52.0, 10.0}, {-48.0, 10.0}, {-48.0, 40.0}, {52.0, 40.0}, {52.0, 70.0}, {2.0, 70.0}, {2.0, -51.0}})}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{-80, 90}, {-88, 68}, {-72, 68}, {-80, 90}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Line(points = {{-80, 68}, {-80, -80}}, color = {95, 95, 95}), Line(points = {{-90, -70}, {82, -70}}, color = {95, 95, 95}), Polygon(points = {{90, -70}, {68, -62}, {68, -78}, {90, -70}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-20, 90}, {20, -30}}, lineColor = {255, 255, 255}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-20, -30}, {-20, 90}, {80, 90}, {80, -30}, {-20, -30}, {-20, 0}, {80, 0}, {80, 30}, {-20, 30}, {-20, 60}, {80, 60}, {80, 90}, {20, 90}, {20, -30}}, color = {0, 0, 0}), Text(extent = {{-71, -42}, {-32, -54}}, lineColor = {0, 0, 0}, textString = "offset"), Polygon(points = {{-31, -30}, {-33, -40}, {-28, -40}, {-31, -30}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Polygon(points = {{-31, -70}, {-34, -60}, {-29, -60}, {-31, -70}, {-31, -70}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Line(points = {{-31, -31}, {-31, -70}}, color = {95, 95, 95}), Line(points = {{-20, -30}, {-20, -70}}, color = {95, 95, 95}), Text(extent = {{-42, -74}, {6, -84}}, lineColor = {0, 0, 0}, textString = "startTime"), Line(points = {{-20, -30}, {-80, -30}}, color = {95, 95, 95}), Text(extent = {{-73, 93}, {-44, 74}}, lineColor = {0, 0, 0}, textString = "y"), Text(extent = {{66, -81}, {92, -92}}, lineColor = {0, 0, 0}, textString = "time"), Text(extent = {{-19, 83}, {20, 68}}, lineColor = {0, 0, 0}, textString = "time"), Text(extent = {{21, 82}, {50, 68}}, lineColor = {0, 0, 0}, textString = "y[1]"), Line(points = {{50, 90}, {50, -30}}, color = {0, 0, 0}), Line(points = {{80, 0}, {100, 0}}, color = {0, 0, 255}), Text(extent = {{34, -30}, {71, -42}}, textString = "columns", lineColor = {0, 0, 255}), Text(extent = {{51, 82}, {80, 68}}, lineColor = {0, 0, 0}, textString = "y[2]")}));
      end CombiTimeTable;

      block BooleanConstant "Generate constant signal of type Boolean"
        parameter Boolean k = true "Constant output value";
        extends Interfaces.partialBooleanSource;
      equation
        y = k;
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-80, 0}, {80, 0}}, color = {0, 0, 0}), Text(extent = {{-150, -140}, {150, -110}}, lineColor = {0, 0, 0}, textString = "%k")}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-70, 0}, {80, 0}}, color = {0, 0, 255}, thickness = 0.5), Text(extent = {{-69, 20}, {-49, 0}}, lineColor = {0, 0, 0}, textString = "k"), Text(extent = {{-96, 6}, {-76, -4}}, lineColor = {0, 0, 0}, textString = "true"), Text(extent = {{-98, -58}, {-72, -68}}, lineColor = {0, 0, 0}, textString = "false")}),
          Documentation(info = "<html>
<p>
The Boolean output y is a constant signal:
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Blocks/Sources/BooleanConstant.png\"
     alt=\"BooleanConstant.png\">
</p>
</html>"));
      end BooleanConstant;

      block BooleanStep "Generate step signal of type Boolean"
        parameter Modelica.SIunits.Time startTime = 0 "Time instant of step start";
        parameter Boolean startValue = false "Output before startTime";
        extends Interfaces.partialBooleanSource;
      equation
        y = if time >= startTime then not startValue else startValue;
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(visible = not startValue, points = {{-80, -70}, {0, -70}, {0, 50}, {80, 50}}, color = {0, 0, 0}), Line(visible = startValue, points = {{-80, 50}, {0, 50}, {0, -70}, {68, -70}}, color = {0, 0, 0}), Text(extent = {{-150, -140}, {150, -110}}, lineColor = {0, 0, 0}, textString = "%startTime")}),
          Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-80, -70}, {0, -70}, {0, 50}, {80, 50}}, color = {0, 0, 255}, thickness = 0.5), Text(extent = {{-15, -74}, {20, -82}}, lineColor = {0, 0, 0}, textString = "startTime"), Polygon(points = {{2, 50}, {-80, 50}, {2, 50}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Text(extent = {{-66, 62}, {-22, 48}}, lineColor = {0, 0, 0}, textString = "not startValue"), Text(extent = {{-68, -58}, {-36, -72}}, lineColor = {0, 0, 0}, textString = "startValue")}),
          Documentation(info = "<html>
<p>
The Boolean output y is a step signal:
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Blocks/Sources/BooleanStep.png\"
     alt=\"BooleanStep.png\">
</p>
</html>"));
      end BooleanStep;

      block BooleanPulse "Generate pulse signal of type Boolean"
        parameter Real width(final min = Modelica.Constants.small, final max = 100) = 50 "Width of pulse in % of period";
        parameter Modelica.SIunits.Time period(final min = Modelica.Constants.small, start = 1) "Time for one period";
        parameter Modelica.SIunits.Time startTime = 0 "Time instant of first pulse";
        extends Modelica.Blocks.Interfaces.partialBooleanSource;
      protected
        parameter Modelica.SIunits.Time Twidth = period * width / 100 "width of one pulse" annotation(
          HideResult = true);
        discrete Modelica.SIunits.Time pulsStart "Start time of pulse" annotation(
          HideResult = true);
      initial equation
        pulsStart = startTime;
      equation
        when sample(startTime, period) then
          pulsStart = time;
        end when;
        y = time >= pulsStart and time < pulsStart + Twidth;
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-150, -140}, {150, -110}}, lineColor = {0, 0, 0}, textString = "%period"), Line(points = {{-80, -70}, {-40, -70}, {-40, 44}, {0, 44}, {0, -70}, {40, -70}, {40, 44}, {79, 44}}, color = {0, 0, 0})}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-60, -74}, {-19, -82}}, lineColor = {0, 0, 0}, textString = "startTime"), Line(points = {{-78, -70}, {-40, -70}, {-40, 20}, {20, 20}, {20, -70}, {50, -70}, {50, 20}, {100, 20}}, color = {0, 0, 255}, thickness = 0.5), Line(points = {{-40, 61}, {-40, 21}}, color = {95, 95, 95}), Line(points = {{20, 44}, {20, 20}}, color = {95, 95, 95}), Line(points = {{50, 58}, {50, 20}}, color = {95, 95, 95}), Line(points = {{-40, 53}, {50, 53}}, color = {95, 95, 95}), Line(points = {{-40, 35}, {20, 35}}, color = {95, 95, 95}), Text(extent = {{-30, 65}, {16, 55}}, lineColor = {0, 0, 0}, textString = "period"), Text(extent = {{-33, 47}, {14, 37}}, lineColor = {0, 0, 0}, textString = "width"), Line(points = {{-70, 20}, {-41, 20}}, color = {95, 95, 95}), Polygon(points = {{-40, 35}, {-31, 37}, {-31, 33}, {-40, 35}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Polygon(points = {{20, 35}, {12, 37}, {12, 33}, {20, 35}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Polygon(points = {{-40, 53}, {-31, 55}, {-31, 51}, {-40, 53}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Polygon(points = {{50, 53}, {42, 55}, {42, 51}, {50, 53}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Text(extent = {{-95, 26}, {-66, 17}}, lineColor = {0, 0, 0}, textString = "true"), Text(extent = {{-96, -60}, {-75, -69}}, lineColor = {0, 0, 0}, textString = "false")}),
          Documentation(info = "<html>
<p>
The Boolean output y is a pulse signal:
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Blocks/Sources/Pulse.png\"
     alt=\"Pulse.png\">
</p>
</html>"));
      end BooleanPulse;

      block SampleTrigger "Generate sample trigger signal"
        parameter Modelica.SIunits.Time period(final min = Modelica.Constants.small, start = 0.01) "Sample period";
        parameter Modelica.SIunits.Time startTime = 0 "Time instant of first sample trigger";
        extends Interfaces.partialBooleanSource;
      equation
        y = sample(startTime, period);
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-60, -70}, {-60, 70}}, color = {0, 0, 0}), Line(points = {{-20, -70}, {-20, 70}}, color = {0, 0, 0}), Line(points = {{20, -70}, {20, 70}}, color = {0, 0, 0}), Line(points = {{60, -70}, {60, 70}}, color = {0, 0, 0}), Text(extent = {{-150, -140}, {150, -110}}, lineColor = {0, 0, 0}, textString = "%period")}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-51, -72}, {-11, -81}}, lineColor = {0, 0, 0}, textString = "startTime"), Line(points = {{-30, 47}, {-30, 19}}, color = {95, 95, 95}), Line(points = {{0, 47}, {0, 18}}, color = {95, 95, 95}), Line(points = {{-30, 41}, {0, 41}}, color = {95, 95, 95}), Text(extent = {{-37, 61}, {9, 49}}, lineColor = {0, 0, 0}, textString = "period"), Line(points = {{-73, 19}, {-30, 19}}, color = {95, 95, 95}), Polygon(points = {{-30, 41}, {-21, 43}, {-21, 39}, {-30, 41}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Polygon(points = {{0, 41}, {-8, 43}, {-8, 39}, {0, 41}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Text(extent = {{-91, 23}, {-71, 13}}, lineColor = {0, 0, 0}, textString = "true"), Text(extent = {{-90, -59}, {-70, -68}}, lineColor = {0, 0, 0}, textString = "false"), Line(points = {{0, -70}, {0, 19}}, color = {0, 0, 255}, thickness = 0.5), Line(points = {{-30, -70}, {-30, 19}}, color = {0, 0, 255}, thickness = 0.5), Line(points = {{30, -70}, {30, 19}}, color = {0, 0, 255}, thickness = 0.5), Line(points = {{60, -70}, {60, 19}}, color = {0, 0, 255}, thickness = 0.5)}),
          Documentation(info = "<html>
<p>
The Boolean output y is a trigger signal where the output y is only <b>true</b>
at sample times (defined by parameter <b>period</b>) and is otherwise
<b>false</b>.
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Blocks/Sources/SampleTrigger.png\"
     alt=\"SampleTrigger.png\">
</p>
</html>"));
      end SampleTrigger;

      block BooleanTable "Generate a Boolean output signal based on a vector of time instants"
        parameter Boolean startValue = false "Start value of y. At time = table[1], y changes to 'not startValue'";
        parameter Modelica.SIunits.Time table[:] = {0, 1} "Vector of time points. At every time point, the output y gets its opposite value (e.g., table={0,1})";
        extends Interfaces.partialBooleanSource;
      protected
        function getFirstIndex "Get first index of table and check table"
          extends Modelica.Icons.Function;
          input Real table[:] "Vector of time instants";
          input Modelica.SIunits.Time simulationStartTime "Simulation start time";
          input Boolean startValue "Value of y for time < table[1]";
          output Integer index "First index to be used";
          output Modelica.SIunits.Time nextTime "Time instant of first event";
          output Boolean y "Value of y at simulationStartTime";
        protected
          Integer j;
          Integer n = size(table, 1) "Number of table points";
        algorithm
          if size(table, 1) == 0 then
            index := 0;
            nextTime := Modelica.Constants.inf;
            y := startValue;
          elseif size(table, 1) == 1 then
            index := 1;
            if table[1] > simulationStartTime then
              nextTime := table[1];
              y := startValue;
            else
              nextTime := Modelica.Constants.inf;
              y := not startValue;
            end if;
          else
            for i in 2:n loop
              assert(table[i] > table[i - 1], "Time values of table not strict monotonically increasing: table[" + String(i - 1) + "] = " + String(table[i - 1]) + ", table[" + String(i) + "] = " + String(table[i]));
            end for;
            j := 1;
            y := startValue;
            while j <= n and table[j] <= simulationStartTime loop
              y := not y;
              j := j + 1;
            end while;
            if j > n then
              nextTime := Modelica.Constants.inf;
            else
              nextTime := table[j];
            end if;
            index := j;
          end if;
// Check whether time values are strict monotonically increasing
// Determine first index in table
        end getFirstIndex;

        parameter Integer n = size(table, 1) "Number of table points";
        Modelica.SIunits.Time nextTime;
        Integer index "Index of actual table entry";
      initial algorithm
        (index, nextTime, y) := getFirstIndex(table, time, startValue);
      algorithm
        when time >= pre(nextTime) and n > 0 then
          if index < n then
            index := index + 1;
            nextTime := table[index];
            y := not y;
          elseif index == n then
            index := index + 1;
            y := not y;
          end if;
        end when;
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-18, 70}, {32, -50}}, lineColor = {255, 255, 255}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-18, -50}, {-18, 70}, {32, 70}, {32, -50}, {-18, -50}, {-18, -20}, {32, -20}, {32, 10}, {-18, 10}, {-18, 40}, {32, 40}, {32, 70}, {32, 70}, {32, -51}}, color = {0, 0, 0})}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-34, 66}, {16, -54}}, lineColor = {255, 255, 255}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-34, -54}, {-34, 66}, {16, 66}, {16, -54}, {-34, -54}, {-34, -24}, {16, -24}, {16, 6}, {-34, 6}, {-34, 36}, {16, 36}, {16, 66}, {16, 66}, {16, -55}}, color = {0, 0, 0}), Text(extent = {{-29, 59}, {10, 44}}, lineColor = {0, 0, 0}, textString = "time")}),
          Documentation(info = "<html>
<p>
The Boolean output y is a signal defined by parameter vector <b>table</b>.
In the vector time points are stored. At every time point, the output y
changes its value to the negated value of the previous one.
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Blocks/Sources/BooleanTable.png\"
     alt=\"BooleanTable.png\">
</p>

<p>
The precise semantics is:
</p>

<pre>
  <b>if</b> size(table,1) == 0 <b>then</b>
     y = startValue;
  <b>else</b>
     //            time &lt; table[1]: y = startValue
     // table[1] &le; time &lt; table[2]: y = not startValue
     // table[2] &le; time &lt; table[3]: y = startValue
     // table[3] &le; time &lt; table[4]: y = not startValue
     // ...
  <b>end if</b>;
</pre>
<p>
Note, the result of this block depends only on time, but not on the simulation start time
(changing the simulation start time, will result exactly in the same output y at the same
time instant ti);
</p>
</html>"));
      end BooleanTable;

      block RadioButtonSource "Boolean signal source that mimics a radio button"
        parameter Modelica.SIunits.Time buttonTimeTable[:] = {0, 1} "Time instants where button is pressed";
        input Boolean reset[:] = {false} "Reset button to false, if an element of reset becomes true" annotation(
          Dialog(group = "Time varying expressions"));
        Modelica.Blocks.Interfaces.BooleanOutput on(start = false, fixed = true) annotation(
          Placement(transformation(extent = {{100, -15}, {130, 15}}, rotation = 0)));
      protected
        Modelica.Blocks.Sources.BooleanTable table(table = buttonTimeTable);
        parameter Integer nReset = size(reset, 1);
        Boolean pre_reset[nReset];
      initial equation
        pre(pre_reset) = fill(false, nReset);
        pre(reset) = fill(false, nReset);
      algorithm
        pre_reset := pre(reset);
        when pre_reset then
          on := false;
        end when;
        when change(table.y) then
          on := true;
        end when;
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}, initialScale = 0.06), graphics = {Rectangle(extent = {{-100, -100}, {100, 100}}, borderPattern = BorderPattern.Raised, fillColor = DynamicSelect({192, 192, 192}, if on > 0.5 then {0, 255, 0} else {192, 192, 192}), fillPattern = DynamicSelect(FillPattern.Solid, if on > 0.5 then FillPattern.Solid else FillPattern.Solid), lineColor = {128, 128, 128}, lineThickness = 5.0), Text(extent = {{-300, 110}, {300, 175}}, lineColor = {0, 0, 255}, textString = "%name")}),
          Documentation(info = "<html>
<p>
Boolean signal source that mimics a radio button:
Via a table, a radio button is pressed (i.e., the output 'on' is set to true) and is reset when an element of the Boolean vector
'reset' becomes true. If both appear at the same time instant, setting
the button according to the table has a higher priority as resetting
the button. Example:
</p>

<pre>
  RadioButtonSource start(buttonTimeTable={1,3}, reset={stop.on});
  RadioButtonSource stop (buttonTimeTable={2,4}, reset={start.on});
</pre>

<p>
The \"start\" button is pressed at time=1 s and time=3 s,
whereas the \"stop\" button is pressed at time=2 s and time=4 s.
This gives the following result:
</p>

<blockquote>
<img src=\"modelica://Modelica/Resources/Images/Blocks/Sources/RadioButtonSource.png\"
     alt=\"RadioButtonSource.png\">
</blockquote>

<p>
This example is also available in
<a href=\"modelica://Modelica.Blocks.Examples.Interaction1\">Modelica.Blocks.Examples.Interaction1</a>
</p>

</html>"));
      end RadioButtonSource;

      block IntegerConstant "Generate constant signal of type Integer"
        parameter Integer k(start = 1) "Constant output value";
        extends Interfaces.IntegerSO;
      equation
        y = k;
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-80, 68}, {-80, -80}}, color = {192, 192, 192}), Polygon(points = {{-80, 90}, {-88, 68}, {-72, 68}, {-80, 90}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-90, -70}, {82, -70}}, color = {192, 192, 192}), Polygon(points = {{90, -70}, {68, -62}, {68, -78}, {90, -70}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-80, 0}, {80, 0}}, color = {0, 0, 0}), Text(extent = {{-150, -150}, {150, -110}}, lineColor = {0, 0, 0}, textString = "k=%k")}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{-80, 90}, {-86, 68}, {-74, 68}, {-80, 90}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Line(points = {{-80, 68}, {-80, -80}}, color = {95, 95, 95}), Line(points = {{-80, 0}, {80, 0}}, color = {0, 0, 255}, thickness = 0.5), Line(points = {{-90, -70}, {82, -70}}, color = {95, 95, 95}), Polygon(points = {{90, -70}, {68, -62}, {68, -78}, {90, -70}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Text(extent = {{-81, 92}, {-38, 74}}, lineColor = {0, 0, 0}, textString = "y"), Text(extent = {{66, -82}, {94, -94}}, lineColor = {0, 0, 0}, textString = "time"), Text(extent = {{-101, 8}, {-81, -12}}, lineColor = {0, 0, 0}, textString = "k")}),
          Documentation(info = "<html>
<p>
The Integer output y is a constant signal:
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Blocks/Sources/IntegerConstant.png\"
     alt=\"IntegerConstant.png\">
</p>
</html>"));
      end IntegerConstant;

      block IntegerStep "Generate step signal of type Integer"
        parameter Integer height = 1 "Height of step";
        extends Interfaces.IntegerSignalSource;
      equation
        y = offset + (if time < startTime then 0 else height);
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-80, 68}, {-80, -80}}, color = {192, 192, 192}), Polygon(points = {{-80, 90}, {-88, 68}, {-72, 68}, {-80, 90}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-90, -70}, {82, -70}}, color = {192, 192, 192}), Polygon(points = {{90, -70}, {68, -62}, {68, -78}, {90, -70}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-80, -70}, {0, -70}, {0, 50}, {80, 50}}, color = {0, 0, 0}), Text(extent = {{-150, -150}, {150, -110}}, lineColor = {0, 0, 0}, textString = "startTime=%startTime")}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{-80, 88}, {-86, 68}, {-74, 68}, {-80, 88}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Line(points = {{-80, 68}, {-80, -80}}, color = {95, 95, 95}), Line(points = {{-80, -18}, {0, -18}, {0, 50}, {80, 50}}, color = {0, 0, 255}, thickness = 0.5), Line(points = {{-90, -70}, {82, -70}}, color = {95, 95, 95}), Polygon(points = {{90, -70}, {70, -66}, {70, -74}, {90, -70}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Text(extent = {{66, -78}, {92, -88}}, lineColor = {0, 0, 0}, textString = "time"), Text(extent = {{-21, -76}, {26, -88}}, lineColor = {0, 0, 0}, textString = "startTime"), Line(points = {{0, -17}, {0, -71}}, color = {95, 95, 95}), Text(extent = {{-60, -36}, {-12, -48}}, lineColor = {0, 0, 0}, textString = "offset"), Line(points = {{-13, 50}, {-13, -17}}, color = {95, 95, 95}), Polygon(points = {{2, 50}, {-19, 50}, {2, 50}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Polygon(points = {{-13, -17}, {-16, -4}, {-10, -4}, {-13, -17}, {-13, -17}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Polygon(points = {{-13, 50}, {-16, 37}, {-9, 37}, {-13, 50}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Text(extent = {{-58, 22}, {-12, 10}}, lineColor = {0, 0, 0}, textString = "height"), Polygon(points = {{-13, -69}, {-16, -56}, {-10, -56}, {-13, -69}, {-13, -69}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Line(points = {{-13, -18}, {-13, -70}}, color = {95, 95, 95}), Polygon(points = {{-13, -18}, {-16, -31}, {-9, -31}, {-13, -18}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Text(extent = {{-78, 96}, {-44, 80}}, lineColor = {0, 0, 0}, textString = "y")}),
          Documentation(info = "<html>
<p>
The Integer output y is a step signal:
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Blocks/Sources/IntegerStep.png\"
     alt=\"IntegerStep.png\">
</p>
</html>"));
      end IntegerStep;

      block IntegerTable "Generate an Integer output signal based on a table matrix with [time, yi] values"
        parameter Real table[:, 2] = fill(0, 0, 2) "Table matrix (first column: time; second column: y)";
        extends Interfaces.IntegerSO;
      protected
        function getFirstIndex "Get first index of table and check table"
          extends Modelica.Icons.Function;
          input Real table[:, 2] "Table matrix";
          input Modelica.SIunits.Time simulationStartTime "Simulation start time";
          output Integer index "First index to be used";
          output Modelica.SIunits.Time nextTime "Time instant of first event";
          output Integer y "Value of y at simulationStartTime";
        protected
          Modelica.SIunits.Time t_last;
          Integer j;
          Integer n = size(table, 1) "Number of table points";
        algorithm
          if size(table, 1) == 0 then
            index := 0;
            nextTime := simulationStartTime - 1;
            y := 0;
          else
            t_last := table[1, 1];
            for i in 2:n loop
              assert(table[i, 1] > t_last, "Time values of table not strict monotonically increasing: table[" + String(i - 1) + ",1] = " + String(table[i - 1, 1]) + "table[" + String(i) + ",1] = " + String(table[i, 1]));
            end for;
            for i in 1:n loop
              assert(rem(table[i, 2], 1) == 0.0, "Table value is not an Integer: table[" + String(i) + ",2] = " + String(table[i, 2]));
            end for;
            j := 1;
            y := integer(table[1, 2]);
            while j < n and table[j, 1] <= simulationStartTime loop
              j := j + 1;
            end while;
            if j == 1 then
              nextTime := table[1, 1];
              y := integer(table[1, 2]);
            elseif j == n and table[n, 1] <= simulationStartTime then
              nextTime := simulationStartTime - 1;
              y := integer(table[n, 2]);
            else
              nextTime := table[j, 1];
              y := integer(table[j - 1, 2]);
            end if;
            index := j;
          end if;
// Check whether time values are strict monotonically increasing
// Check that all values in the second column are Integer values
// Determine index in table for "nextTime"
        end getFirstIndex;

        parameter Integer n = size(table, 1) "Number of table points";
        Modelica.SIunits.Time nextTime;
        Integer index "Index of actual table entry";
      initial algorithm
        (index, nextTime, y) := getFirstIndex(table, time);
      equation
        assert(size(table, 1) > 0, "No table values defined.");
        when time >= pre(nextTime) then
          y = integer(table[pre(index), 2]);
          index = pre(index) + 1;
          nextTime = if index <= n then table[index, 1] else pre(nextTime) - 1;
        end when;
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-80, 64}, {-80, -84}}, color = {192, 192, 192}), Polygon(points = {{-80, 86}, {-88, 64}, {-72, 64}, {-80, 86}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-90, -74}, {82, -74}}, color = {192, 192, 192}), Polygon(points = {{90, -74}, {68, -66}, {68, -82}, {90, -74}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-46, 68}, {4, -52}}, lineColor = {255, 255, 255}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-46, -52}, {-46, 68}, {54, 68}, {54, -52}, {-46, -52}, {-46, -22}, {54, -22}, {54, 8}, {-46, 8}, {-46, 38}, {54, 38}, {54, 68}, {4, 68}, {4, -53}}, color = {0, 0, 0})}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-40, 60}, {10, -60}}, lineColor = {255, 255, 255}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-40, -60}, {-40, 60}, {60, 60}, {60, -60}, {-40, -60}, {-40, -30}, {60, -30}, {60, 0}, {-40, 0}, {-40, 30}, {60, 30}, {60, 60}, {10, 60}, {10, -61}}, color = {0, 0, 0}), Text(extent = {{-35, 53}, {4, 38}}, lineColor = {0, 0, 0}, textString = "time"), Text(extent = {{13, 53}, {56, 37}}, lineColor = {0, 0, 0}, textString = "y"), Text(extent = {{66, -88}, {90, -98}}, lineColor = {0, 0, 0}, textString = "time"), Polygon(points = {{88, -80}, {68, -75}, {68, -84}, {88, -80}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Line(points = {{-90, -80}, {82, -80}}, color = {95, 95, 95}), Line(points = {{-80, 58}, {-80, -90}}, color = {95, 95, 95}), Polygon(points = {{-80, 80}, {-85, 58}, {-74, 58}, {-80, 80}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Text(extent = {{-76, 83}, {-44, 65}}, lineColor = {0, 0, 0}, textString = "y")}),
          Documentation(info = "<html>

<p>
This block generates an Integer output signal by using a table.
The time points and y-values are stored in a matrix
<b>table[i,j]</b>, where the first column table[:,1] contains the
Real time points and the second column contains the Integer value of the
output y at this time point.
</p>

<p>
An assert is triggered, if no table values are provided, if the
time points are not strict monotonically increasing, or if
the second column of the table matrix does not contain Integer values.
</p>

<p>
If the simulation time is less than the first table time instant,
then the output y = table[1,2].<br>
If the simulation time is greater than the last table time instant,
then the output y = table[end,2].
</p>

<p>
Example:
</p>
<pre>
   table = [  0, 1;
              1, 4;
            1.5, 5;
              2, 6];
</pre>
<p>
results in the following output:
</p>

<blockquote><p>
<img src=\"modelica://Modelica/Resources/Images/Blocks/Sources/IntegerTable.png\"
     alt=\"IntegerTable.png\">
</p></blockquote>

</html>"));
      end IntegerTable;
      annotation(
        Documentation(info = "<HTML>
<p>
This package contains <b>source</b> components, i.e., blocks which
have only output signals. These blocks are used as signal generators
for Real, Integer and Boolean signals.
</p>

<p>
All Real source signals (with the exception of the Constant source)
have at least the following two parameters:
</p>

<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"><b>offset</b></td>
      <td valign=\"top\">Value which is added to the signal</td>
  </tr>
  <tr><td valign=\"top\"><b>startTime</b></td>
      <td valign=\"top\">Start time of signal. For time &lt; startTime,
                the output y is set to offset.</td>
  </tr>
</table>

<p>
The <b>offset</b> parameter is especially useful in order to shift
the corresponding source, such that at initial time the system
is stationary. To determine the corresponding value of offset,
usually requires a trimming calculation.
</p>
</html>", revisions = "<html>
<ul>
<li><i>October 21, 2002</i>
       by <a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a>
       and <a href=\"http://www.robotic.dlr.de/Christian.Schweiger/\">Christian Schweiger</a>:<br>
       Integer sources added. Step, TimeTable and BooleanStep slightly changed.</li>
<li><i>Nov. 8, 1999</i>
       by <a href=\"mailto:clauss@eas.iis.fhg.de\">Christoph Clau&szlig;</a>,
       <a href=\"mailto:Andre.Schneider@eas.iis.fraunhofer.de\">Andre.Schneider@eas.iis.fraunhofer.de</a>,
       <a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a>:<br>
       New sources: Exponentials, TimeTable. Trapezoid slightly enhanced
       (nperiod=-1 is an infinite number of periods).</li>
<li><i>Oct. 31, 1999</i>
       by <a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a>:<br>
       <a href=\"mailto:clauss@eas.iis.fhg.de\">Christoph Clau&szlig;</a>,
       <a href=\"mailto:Andre.Schneider@eas.iis.fraunhofer.de\">Andre.Schneider@eas.iis.fraunhofer.de</a>,
       All sources vectorized. New sources: ExpSine, Trapezoid,
       BooleanConstant, BooleanStep, BooleanPulse, SampleTrigger.
       Improved documentation, especially detailed description of
       signals in diagram layer.</li>
<li><i>June 29, 1999</i>
       by <a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a>:<br>
       Realized a first version, based on an existing Dymola library
       of Dieter Moormann and Hilding Elmqvist.</li>
</ul>
</html>"));
    end Sources;

    package Tables "Library of blocks to interpolate in one and two-dimensional tables"
      extends Modelica.Icons.Package;

      block CombiTable1D "Table look-up in one dimension (matrix/file) with n inputs and n outputs"
        extends Modelica.Blocks.Interfaces.MIMOs(final n = size(columns, 1));
        parameter Boolean tableOnFile = false "= true, if table is defined on file or in function usertab" annotation(
          Dialog(group = "Table data definition"));
        parameter Real table[:, :] = fill(0.0, 0, 2) "Table matrix (grid = first column; e.g., table=[0,2])" annotation(
          Dialog(group = "Table data definition", enable = not tableOnFile));
        parameter String tableName = "NoName" "Table name on file or in function usertab (see docu)" annotation(
          Dialog(group = "Table data definition", enable = tableOnFile));
        parameter String fileName = "NoName" "File where matrix is stored" annotation(
          Dialog(group = "Table data definition", enable = tableOnFile, loadSelector(filter = "Text files (*.txt);;MATLAB MAT-files (*.mat)", caption = "Open file in which table is present")));
        parameter Boolean verboseRead = true "= true, if info message that file is loading is to be printed" annotation(
          Dialog(group = "Table data definition", enable = tableOnFile));
        parameter Integer columns[:] = 2:size(table, 2) "Columns of table to be interpolated" annotation(
          Dialog(group = "Table data interpretation"));
        parameter Modelica.Blocks.Types.Smoothness smoothness = Modelica.Blocks.Types.Smoothness.LinearSegments "Smoothness of table interpolation" annotation(
          Dialog(group = "Table data interpretation"));
      protected
        Modelica.Blocks.Types.ExternalCombiTable1D tableID = Modelica.Blocks.Types.ExternalCombiTable1D(if tableOnFile then tableName else "NoName", if tableOnFile and fileName <> "NoName" and not Modelica.Utilities.Strings.isEmpty(fileName) then fileName else "NoName", table, columns, smoothness) "External table object";
        parameter Real tableOnFileRead(fixed = false) "= 1, if table was successfully read from file";

        function readTableData "Read table data from ASCII text or MATLAB MAT-file"
          extends Modelica.Icons.Function;
          input Modelica.Blocks.Types.ExternalCombiTable1D tableID;
          input Boolean forceRead = false "= true: Force reading of table data; = false: Only read, if not yet read.";
          input Boolean verboseRead "= true: Print info message; = false: No info message";
          output Real readSuccess "Table read success";

          external "C" readSuccess = ModelicaStandardTables_CombiTable1D_read(tableID, forceRead, verboseRead) annotation(
            Library = {"ModelicaStandardTables"});
        end readTableData;

        function getTableValue "Interpolate 1-dim. table defined by matrix"
          extends Modelica.Icons.Function;
          input Modelica.Blocks.Types.ExternalCombiTable1D tableID;
          input Integer icol;
          input Real u;
          input Real tableAvailable "Dummy input to ensure correct sorting of function calls" annotation(
            __OpenModelica_UnusedVariable = true);
          output Real y;

          external "C" y = ModelicaStandardTables_CombiTable1D_getValue(tableID, icol, u) annotation(
            Library = {"ModelicaStandardTables"});
          annotation(
            derivative(noDerivative = tableAvailable) = getDerTableValue);
        end getTableValue;

        function getTableValueNoDer "Interpolate 1-dim. table defined by matrix (but do not provide a derivative function)"
          extends Modelica.Icons.Function;
          input Modelica.Blocks.Types.ExternalCombiTable1D tableID;
          input Integer icol;
          input Real u;
          input Real tableAvailable "Dummy input to ensure correct sorting of function calls" annotation(
            __OpenModelica_UnusedVariable = true);
          output Real y;

          external "C" y = ModelicaStandardTables_CombiTable1D_getValue(tableID, icol, u) annotation(
            Library = {"ModelicaStandardTables"});
        end getTableValueNoDer;

        function getDerTableValue "Derivative of interpolated 1-dim. table defined by matrix"
          extends Modelica.Icons.Function;
          input Modelica.Blocks.Types.ExternalCombiTable1D tableID;
          input Integer icol;
          input Real u;
          input Real tableAvailable "Dummy input to ensure correct sorting of function calls" annotation(
            __OpenModelica_UnusedVariable = true);
          input Real der_u;
          output Real der_y;

          external "C" der_y = ModelicaStandardTables_CombiTable1D_getDerValue(tableID, icol, u, der_u) annotation(
            Library = {"ModelicaStandardTables"});
        end getDerTableValue;
      initial algorithm
        if tableOnFile then
          tableOnFileRead := readTableData(tableID, false, verboseRead);
        else
          tableOnFileRead := 1.;
        end if;
      equation
        if tableOnFile then
          assert(tableName <> "NoName", "tableOnFile = true and no table name given");
        else
          assert(size(table, 1) > 0 and size(table, 2) > 0, "tableOnFile = false and parameter table is an empty matrix");
        end if;
        if smoothness == Modelica.Blocks.Types.Smoothness.ConstantSegments then
          for i in 1:n loop
            y[i] = getTableValueNoDer(tableID, i, u[i], tableOnFileRead);
          end for;
        else
          for i in 1:n loop
            y[i] = getTableValue(tableID, i, u[i], tableOnFileRead);
          end for;
        end if;
        annotation(
          Documentation(info = "<html>
<p>
<b>Linear interpolation</b> in <b>one</b> dimension of a <b>table</b>.
Via parameter <b>columns</b> it can be defined how many columns of the
table are interpolated. If, e.g., columns={2,4}, it is assumed that 2 input
and 2 output signals are present and that the first output interpolates
the first input via column 2 and the second output interpolates the
second input via column 4 of the table matrix.
</p>
<p>
The grid points and function values are stored in a matrix \"table[i,j]\",
where the first column \"table[:,1]\" contains the grid points and the
other columns contain the data to be interpolated. Example:
</p>
<pre>
   table = [0,  0;
            1,  1;
            2,  4;
            4, 16]
   If, e.g., the input u = 1.0, the output y =  1.0,
       e.g., the input u = 1.5, the output y =  2.5,
       e.g., the input u = 2.0, the output y =  4.0,
       e.g., the input u =-1.0, the output y = -1.0 (i.e., extrapolation).
</pre>
<ul>
<li> The interpolation is <b>efficient</b>, because a search for a new interpolation
     starts at the interval used in the last call.</li>
<li> If the table has only <b>one row</b>, the table value is returned,
     independent of the value of the input signal.</li>
<li> If the input signal <b>u[i]</b> is <b>outside</b> of the defined <b>interval</b>, i.e.,
     u[i] &gt; table[size(table,1),i+1] or u[i] &lt; table[1,1], the corresponding
     value is also determined by linear
     interpolation through the last or first two points of the table.</li>
<li> The grid values (first column) have to be strictly increasing.</li>
</ul>
<p>
The table matrix can be defined in the following ways:
</p>
<ol>
<li> Explicitly supplied as <b>parameter matrix</b> \"table\",
     and the other parameters have the following values:
<pre>
   tableName is \"NoName\" or has only blanks,
   fileName  is \"NoName\" or has only blanks.
</pre></li>
<li> <b>Read</b> from a <b>file</b> \"fileName\" where the matrix is stored as
      \"tableName\". Both ASCII and MAT-file format is possible.
      (The ASCII format is described below).
      The MAT-file format comes in four different versions: v4, v6, v7 and v7.3.
      The library supports at least v4, v6 and v7 whereas v7.3 is optional.
      It is most convenient to generate the MAT-file from FreeMat or MATLAB&reg;
      by command
<pre>
   save tables.mat tab1 tab2 tab3
</pre>
      or Scilab by command
<pre>
   savematfile tables.mat tab1 tab2 tab3
</pre>
      when the three tables tab1, tab2, tab3 should be used from the model.<br>
      Note, a fileName can be defined as URI by using the helper function
      <a href=\"modelica://Modelica.Utilities.Files.loadResource\">loadResource</a>.</li>
<li>  Statically stored in function \"usertab\" in file \"usertab.c\".
      The matrix is identified by \"tableName\". Parameter
      fileName = \"NoName\" or has only blanks. Row-wise storage is always to be
      preferred as otherwise the table is reallocated and transposed.
      See the <a href=\"modelica://Modelica.Blocks.Tables\">Tables</a> package
      documentation for more details.</li>
</ol>
<p>
When the constant \"NO_FILE_SYSTEM\" is defined, all file I/O related parts of the
source code are removed by the C-preprocessor, such that no access to files takes place.
</p>
<p>
If tables are read from an ASCII-file, the file needs to have the
following structure (\"-----\" is not part of the file content):
</p>
<pre>
-----------------------------------------------------
#1
double tab1(5,2)   # comment line
  0   0
  1   1
  2   4
  3   9
  4  16
double tab2(5,2)   # another comment line
  0   0
  2   2
  4   8
  6  18
  8  32
-----------------------------------------------------
</pre>
<p>
Note, that the first two characters in the file need to be
\"#1\" (a line comment defining the version number of the file format).
Afterwards, the corresponding matrix has to be declared
with type (= \"double\" or \"float\"), name and actual dimensions.
Finally, in successive rows of the file, the elements of the matrix
have to be given. The elements have to be provided as a sequence of
numbers in row-wise order (therefore a matrix row can span several
lines in the file and need not start at the beginning of a line).
Numbers have to be given according to C syntax (such as 2.3, -2, +2.e4).
Number separators are spaces, tab (\t), comma (,), or semicolon (;).
Several matrices may be defined one after another. Line comments start
with the hash symbol (#) and can appear everywhere.
Other characters, like trailing non comments, are not allowed in the file.
</p>
<p>
MATLAB is a registered trademark of The MathWorks, Inc.
</p>
</html>"),
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100.0, -100.0}, {100.0, 100.0}}, initialScale = 0.1), graphics = {Line(points = {{-60.0, 40.0}, {-60.0, -40.0}, {60.0, -40.0}, {60.0, 40.0}, {30.0, 40.0}, {30.0, -40.0}, {-30.0, -40.0}, {-30.0, 40.0}, {-60.0, 40.0}, {-60.0, 20.0}, {60.0, 20.0}, {60.0, 0.0}, {-60.0, 0.0}, {-60.0, -20.0}, {60.0, -20.0}, {60.0, -40.0}, {-60.0, -40.0}, {-60.0, 40.0}, {60.0, 40.0}, {60.0, -40.0}}), Line(points = {{0.0, 40.0}, {0.0, -40.0}}), Rectangle(fillColor = {255, 215, 136}, fillPattern = FillPattern.Solid, extent = {{-60.0, 20.0}, {-30.0, 40.0}}), Rectangle(fillColor = {255, 215, 136}, fillPattern = FillPattern.Solid, extent = {{-60.0, 0.0}, {-30.0, 20.0}}), Rectangle(fillColor = {255, 215, 136}, fillPattern = FillPattern.Solid, extent = {{-60.0, -20.0}, {-30.0, 0.0}}), Rectangle(fillColor = {255, 215, 136}, fillPattern = FillPattern.Solid, extent = {{-60.0, -40.0}, {-30.0, -20.0}})}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-60, 60}, {60, -60}}, fillColor = {235, 235, 235}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Line(points = {{-100, 0}, {-58, 0}}, color = {0, 0, 255}), Line(points = {{60, 0}, {100, 0}}, color = {0, 0, 255}), Text(extent = {{-100, 100}, {100, 64}}, textString = "1 dimensional linear table interpolation", lineColor = {0, 0, 255}), Line(points = {{-54, 40}, {-54, -40}, {54, -40}, {54, 40}, {28, 40}, {28, -40}, {-28, -40}, {-28, 40}, {-54, 40}, {-54, 20}, {54, 20}, {54, 0}, {-54, 0}, {-54, -20}, {54, -20}, {54, -40}, {-54, -40}, {-54, 40}, {54, 40}, {54, -40}}, color = {0, 0, 0}), Line(points = {{0, 40}, {0, -40}}, color = {0, 0, 0}), Rectangle(extent = {{-54, 40}, {-28, 20}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 0}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-54, 20}, {-28, 0}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 0}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-54, 0}, {-28, -20}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 0}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-54, -20}, {-28, -40}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 0}, fillPattern = FillPattern.Solid), Text(extent = {{-50, 54}, {-32, 42}}, textString = "u[1]/[2]", lineColor = {0, 0, 255}), Text(extent = {{-24, 54}, {0, 42}}, textString = "y[1]", lineColor = {0, 0, 255}), Text(extent = {{-2, -40}, {30, -54}}, textString = "columns", lineColor = {0, 0, 255}), Text(extent = {{2, 54}, {26, 42}}, textString = "y[2]", lineColor = {0, 0, 255})}));
      end CombiTable1D;

      block CombiTable1Ds "Table look-up in one dimension (matrix/file) with one input and n outputs"
        extends Modelica.Blocks.Interfaces.SIMO(final nout = size(columns, 1));
        parameter Boolean tableOnFile = false "= true, if table is defined on file or in function usertab" annotation(
          Dialog(group = "Table data definition"));
        parameter Real table[:, :] = fill(0.0, 0, 2) "Table matrix (grid = first column; e.g., table=[0,2])" annotation(
          Dialog(group = "Table data definition", enable = not tableOnFile));
        parameter String tableName = "NoName" "Table name on file or in function usertab (see docu)" annotation(
          Dialog(group = "Table data definition", enable = tableOnFile));
        parameter String fileName = "NoName" "File where matrix is stored" annotation(
          Dialog(group = "Table data definition", enable = tableOnFile, loadSelector(filter = "Text files (*.txt);;MATLAB MAT-files (*.mat)", caption = "Open file in which table is present")));
        parameter Boolean verboseRead = true "= true, if info message that file is loading is to be printed" annotation(
          Dialog(group = "Table data definition", enable = tableOnFile));
        parameter Integer columns[:] = 2:size(table, 2) "Columns of table to be interpolated" annotation(
          Dialog(group = "Table data interpretation"));
        parameter Modelica.Blocks.Types.Smoothness smoothness = Modelica.Blocks.Types.Smoothness.LinearSegments "Smoothness of table interpolation" annotation(
          Dialog(group = "Table data interpretation"));
      protected
        Modelica.Blocks.Types.ExternalCombiTable1D tableID = Modelica.Blocks.Types.ExternalCombiTable1D(if tableOnFile then tableName else "NoName", if tableOnFile and fileName <> "NoName" and not Modelica.Utilities.Strings.isEmpty(fileName) then fileName else "NoName", table, columns, smoothness) "External table object";
        parameter Real tableOnFileRead(fixed = false) "= 1, if table was successfully read from file";

        function readTableData "Read table data from ASCII text or MATLAB MAT-file"
          extends Modelica.Icons.Function;
          input Modelica.Blocks.Types.ExternalCombiTable1D tableID;
          input Boolean forceRead = false "= true: Force reading of table data; = false: Only read, if not yet read.";
          input Boolean verboseRead "= true: Print info message; = false: No info message";
          output Real readSuccess "Table read success";

          external "C" readSuccess = ModelicaStandardTables_CombiTable1D_read(tableID, forceRead, verboseRead) annotation(
            Library = {"ModelicaStandardTables"});
        end readTableData;

        function getTableValue "Interpolate 1-dim. table defined by matrix"
          extends Modelica.Icons.Function;
          input Modelica.Blocks.Types.ExternalCombiTable1D tableID;
          input Integer icol;
          input Real u;
          input Real tableAvailable "Dummy input to ensure correct sorting of function calls" annotation(
            __OpenModelica_UnusedVariable = true);
          output Real y;

          external "C" y = ModelicaStandardTables_CombiTable1D_getValue(tableID, icol, u) annotation(
            Library = {"ModelicaStandardTables"});
          annotation(
            derivative(noDerivative = tableAvailable) = getDerTableValue);
        end getTableValue;

        function getTableValueNoDer "Interpolate 1-dim. table defined by matrix (but do not provide a derivative function)"
          extends Modelica.Icons.Function;
          input Modelica.Blocks.Types.ExternalCombiTable1D tableID;
          input Integer icol;
          input Real u;
          input Real tableAvailable "Dummy input to ensure correct sorting of function calls" annotation(
            __OpenModelica_UnusedVariable = true);
          output Real y;

          external "C" y = ModelicaStandardTables_CombiTable1D_getValue(tableID, icol, u) annotation(
            Library = {"ModelicaStandardTables"});
        end getTableValueNoDer;

        function getDerTableValue "Derivative of interpolated 1-dim. table defined by matrix"
          extends Modelica.Icons.Function;
          input Modelica.Blocks.Types.ExternalCombiTable1D tableID;
          input Integer icol;
          input Real u;
          input Real tableAvailable "Dummy input to ensure correct sorting of function calls" annotation(
            __OpenModelica_UnusedVariable = true);
          input Real der_u;
          output Real der_y;

          external "C" der_y = ModelicaStandardTables_CombiTable1D_getDerValue(tableID, icol, u, der_u) annotation(
            Library = {"ModelicaStandardTables"});
        end getDerTableValue;
      initial algorithm
        if tableOnFile then
          tableOnFileRead := readTableData(tableID, false, verboseRead);
        else
          tableOnFileRead := 1.;
        end if;
      equation
        if tableOnFile then
          assert(tableName <> "NoName", "tableOnFile = true and no table name given");
        else
          assert(size(table, 1) > 0 and size(table, 2) > 0, "tableOnFile = false and parameter table is an empty matrix");
        end if;
        if smoothness == Modelica.Blocks.Types.Smoothness.ConstantSegments then
          for i in 1:nout loop
            y[i] = getTableValueNoDer(tableID, i, u, tableOnFileRead);
          end for;
        else
          for i in 1:nout loop
            y[i] = getTableValue(tableID, i, u, tableOnFileRead);
          end for;
        end if;
        annotation(
          Documentation(info = "<html>
<p>
<b>Linear interpolation</b> in <b>one</b> dimension of a <b>table</b>.
Via parameter <b>columns</b> it can be defined how many columns of the
table are interpolated. If, e.g., icol={2,4}, it is assumed that one input
and 2 output signals are present and that the first output interpolates
via column 2 and the second output interpolates via column 4 of the
table matrix.
</p>
<p>
The grid points and function values are stored in a matrix \"table[i,j]\",
where the first column \"table[:,1]\" contains the grid points and the
other columns contain the data to be interpolated. Example:
</p>
<pre>
   table = [0,  0;
            1,  1;
            2,  4;
            4, 16]
   If, e.g., the input u = 1.0, the output y =  1.0,
       e.g., the input u = 1.5, the output y =  2.5,
       e.g., the input u = 2.0, the output y =  4.0,
       e.g., the input u =-1.0, the output y = -1.0 (i.e., extrapolation).
</pre>
<ul>
<li> The interpolation is <b>efficient</b>, because a search for a new interpolation
     starts at the interval used in the last call.</li>
<li> If the table has only <b>one row</b>, the table value is returned,
     independent of the value of the input signal.</li>
<li> If the input signal <b>u</b> is <b>outside</b> of the defined <b>interval</b>, i.e.,
     u &gt; table[size(table,1),1] or u &lt; table[1,1], the corresponding
     value is also determined by linear
     interpolation through the last or first two points of the table.</li>
<li> The grid values (first column) have to be strictly increasing.</li>
</ul>
<p>
The table matrix can be defined in the following ways:
</p>
<ol>
<li> Explicitly supplied as <b>parameter matrix</b> \"table\",
     and the other parameters have the following values:
<pre>
   tableName is \"NoName\" or has only blanks,
   fileName  is \"NoName\" or has only blanks.
</pre></li>
<li> <b>Read</b> from a <b>file</b> \"fileName\" where the matrix is stored as
      \"tableName\". Both ASCII and MAT-file format is possible.
      (The ASCII format is described below).
      The MAT-file format comes in four different versions: v4, v6, v7 and v7.3.
      The library supports at least v4, v6 and v7 whereas v7.3 is optional.
      It is most convenient to generate the MAT-file from FreeMat or MATLAB&reg;
      by command
<pre>
   save tables.mat tab1 tab2 tab3
</pre>
      or Scilab by command
<pre>
   savematfile tables.mat tab1 tab2 tab3
</pre>
      when the three tables tab1, tab2, tab3 should be used from the model.<br>
      Note, a fileName can be defined as URI by using the helper function
      <a href=\"modelica://Modelica.Utilities.Files.loadResource\">loadResource</a>.</li>
<li>  Statically stored in function \"usertab\" in file \"usertab.c\".
      The matrix is identified by \"tableName\". Parameter
      fileName = \"NoName\" or has only blanks. Row-wise storage is always to be
      preferred as otherwise the table is reallocated and transposed.
      See the <a href=\"modelica://Modelica.Blocks.Tables\">Tables</a> package
      documentation for more details.</li>
</ol>
<p>
When the constant \"NO_FILE_SYSTEM\" is defined, all file I/O related parts of the
source code are removed by the C-preprocessor, such that no access to files takes place.
</p>
<p>
If tables are read from an ASCII-file, the file needs to have the
following structure (\"-----\" is not part of the file content):
</p>
<pre>
-----------------------------------------------------
#1
double tab1(5,2)   # comment line
  0   0
  1   1
  2   4
  3   9
  4  16
double tab2(5,2)   # another comment line
  0   0
  2   2
  4   8
  6  18
  8  32
-----------------------------------------------------
</pre>
<p>
Note, that the first two characters in the file need to be
\"#1\" (a line comment defining the version number of the file format).
Afterwards, the corresponding matrix has to be declared
with type (= \"double\" or \"float\"), name and actual dimensions.
Finally, in successive rows of the file, the elements of the matrix
have to be given. The elements have to be provided as a sequence of
numbers in row-wise order (therefore a matrix row can span several
lines in the file and need not start at the beginning of a line).
Numbers have to be given according to C syntax (such as 2.3, -2, +2.e4).
Number separators are spaces, tab (\t), comma (,), or semicolon (;).
Several matrices may be defined one after another. Line comments start
with the hash symbol (#) and can appear everywhere.
Other characters, like trailing non comments, are not allowed in the file.
</p>
<p>
MATLAB is a registered trademark of The MathWorks, Inc.
</p>
</html>"),
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100.0, -100.0}, {100.0, 100.0}}, initialScale = 0.1), graphics = {Line(points = {{-60.0, 40.0}, {-60.0, -40.0}, {60.0, -40.0}, {60.0, 40.0}, {30.0, 40.0}, {30.0, -40.0}, {-30.0, -40.0}, {-30.0, 40.0}, {-60.0, 40.0}, {-60.0, 20.0}, {60.0, 20.0}, {60.0, 0.0}, {-60.0, 0.0}, {-60.0, -20.0}, {60.0, -20.0}, {60.0, -40.0}, {-60.0, -40.0}, {-60.0, 40.0}, {60.0, 40.0}, {60.0, -40.0}}), Line(points = {{0.0, 40.0}, {0.0, -40.0}}), Rectangle(fillColor = {255, 215, 136}, fillPattern = FillPattern.Solid, extent = {{-60.0, 20.0}, {-30.0, 40.0}}), Rectangle(fillColor = {255, 215, 136}, fillPattern = FillPattern.Solid, extent = {{-60.0, 0.0}, {-30.0, 20.0}}), Rectangle(fillColor = {255, 215, 136}, fillPattern = FillPattern.Solid, extent = {{-60.0, -20.0}, {-30.0, 0.0}}), Rectangle(fillColor = {255, 215, 136}, fillPattern = FillPattern.Solid, extent = {{-60.0, -40.0}, {-30.0, -20.0}})}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-60, 60}, {60, -60}}, fillColor = {235, 235, 235}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Line(points = {{-100, 0}, {-58, 0}}, color = {0, 0, 255}), Line(points = {{60, 0}, {100, 0}}, color = {0, 0, 255}), Text(extent = {{-100, 100}, {100, 64}}, textString = "1 dimensional linear table interpolation", lineColor = {0, 0, 255}), Line(points = {{-54, 40}, {-54, -40}, {54, -40}, {54, 40}, {28, 40}, {28, -40}, {-28, -40}, {-28, 40}, {-54, 40}, {-54, 20}, {54, 20}, {54, 0}, {-54, 0}, {-54, -20}, {54, -20}, {54, -40}, {-54, -40}, {-54, 40}, {54, 40}, {54, -40}}, color = {0, 0, 0}), Line(points = {{0, 40}, {0, -40}}, color = {0, 0, 0}), Rectangle(extent = {{-54, 40}, {-28, 20}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 0}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-54, 20}, {-28, 0}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 0}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-54, 0}, {-28, -20}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 0}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-54, -20}, {-28, -40}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 0}, fillPattern = FillPattern.Solid), Text(extent = {{-52, 56}, {-34, 44}}, textString = "u", lineColor = {0, 0, 255}), Text(extent = {{-22, 54}, {2, 42}}, textString = "y[1]", lineColor = {0, 0, 255}), Text(extent = {{4, 54}, {28, 42}}, textString = "y[2]", lineColor = {0, 0, 255}), Text(extent = {{0, -40}, {32, -54}}, textString = "columns", lineColor = {0, 0, 255})}));
      end CombiTable1Ds;

      block CombiTable2D "Table look-up in two dimensions (matrix/file)"
        extends Modelica.Blocks.Interfaces.SI2SO;
        parameter Boolean tableOnFile = false "= true, if table is defined on file or in function usertab" annotation(
          Dialog(group = "Table data definition"));
        parameter Real table[:, :] = fill(0.0, 0, 2) "Table matrix (grid u1 = first column, grid u2 = first row; e.g., table=[0,0;0,1])" annotation(
          Dialog(group = "Table data definition", enable = not tableOnFile));
        parameter String tableName = "NoName" "Table name on file or in function usertab (see docu)" annotation(
          Dialog(group = "Table data definition", enable = tableOnFile));
        parameter String fileName = "NoName" "File where matrix is stored" annotation(
          Dialog(group = "Table data definition", enable = tableOnFile, loadSelector(filter = "Text files (*.txt);;MATLAB MAT-files (*.mat)", caption = "Open file in which table is present")));
        parameter Boolean verboseRead = true "= true, if info message that file is loading is to be printed" annotation(
          Dialog(group = "Table data definition", enable = tableOnFile));
        parameter Modelica.Blocks.Types.Smoothness smoothness = Modelica.Blocks.Types.Smoothness.LinearSegments "Smoothness of table interpolation" annotation(
          Dialog(group = "Table data interpretation"));
      protected
        Modelica.Blocks.Types.ExternalCombiTable2D tableID = Modelica.Blocks.Types.ExternalCombiTable2D(if tableOnFile then tableName else "NoName", if tableOnFile and fileName <> "NoName" and not Modelica.Utilities.Strings.isEmpty(fileName) then fileName else "NoName", table, smoothness) "External table object";
        parameter Real tableOnFileRead(fixed = false) "= 1, if table was successfully read from file";

        function readTableData "Read table data from ASCII text or MATLAB MAT-file"
          extends Modelica.Icons.Function;
          input Modelica.Blocks.Types.ExternalCombiTable2D tableID;
          input Boolean forceRead = false "= true: Force reading of table data; = false: Only read, if not yet read.";
          input Boolean verboseRead "= true: Print info message; = false: No info message";
          output Real readSuccess "Table read success";

          external "C" readSuccess = ModelicaStandardTables_CombiTable2D_read(tableID, forceRead, verboseRead) annotation(
            Library = {"ModelicaStandardTables"});
        end readTableData;

        function getTableValue "Interpolate 2-dim. table defined by matrix"
          extends Modelica.Icons.Function;
          input Modelica.Blocks.Types.ExternalCombiTable2D tableID;
          input Real u1;
          input Real u2;
          input Real tableAvailable "Dummy input to ensure correct sorting of function calls" annotation(
            __OpenModelica_UnusedVariable = true);
          output Real y;

          external "C" y = ModelicaStandardTables_CombiTable2D_getValue(tableID, u1, u2) annotation(
            Library = {"ModelicaStandardTables"});
          annotation(
            derivative(noDerivative = tableAvailable) = getDerTableValue);
        end getTableValue;

        function getTableValueNoDer "Interpolate 2-dim. table defined by matrix (but do not provide a derivative function)"
          extends Modelica.Icons.Function;
          input Modelica.Blocks.Types.ExternalCombiTable2D tableID;
          input Real u1;
          input Real u2;
          input Real tableAvailable "Dummy input to ensure correct sorting of function calls" annotation(
            __OpenModelica_UnusedVariable = true);
          output Real y;

          external "C" y = ModelicaStandardTables_CombiTable2D_getValue(tableID, u1, u2) annotation(
            Library = {"ModelicaStandardTables"});
        end getTableValueNoDer;

        function getDerTableValue "Derivative of interpolated 2-dim. table defined by matrix"
          extends Modelica.Icons.Function;
          input Modelica.Blocks.Types.ExternalCombiTable2D tableID;
          input Real u1;
          input Real u2;
          input Real tableAvailable "Dummy input to ensure correct sorting of function calls" annotation(
            __OpenModelica_UnusedVariable = true);
          input Real der_u1;
          input Real der_u2;
          output Real der_y;

          external "C" der_y = ModelicaStandardTables_CombiTable2D_getDerValue(tableID, u1, u2, der_u1, der_u2) annotation(
            Library = {"ModelicaStandardTables"});
        end getDerTableValue;
      initial algorithm
        if tableOnFile then
          tableOnFileRead := readTableData(tableID, false, verboseRead);
        else
          tableOnFileRead := 1.;
        end if;
      equation
        if tableOnFile then
          assert(tableName <> "NoName", "tableOnFile = true and no table name given");
        else
          assert(size(table, 1) > 0 and size(table, 2) > 0, "tableOnFile = false and parameter table is an empty matrix");
        end if;
        if smoothness == Modelica.Blocks.Types.Smoothness.ConstantSegments then
          y = getTableValueNoDer(tableID, u1, u2, tableOnFileRead);
        else
          y = getTableValue(tableID, u1, u2, tableOnFileRead);
        end if;
        annotation(
          Documentation(info = "<html>
<p>
<b>Linear interpolation</b> in <b>two</b> dimensions of a <b>table</b>.
The grid points and function values are stored in a matrix \"table[i,j]\",
where:
</p>
<ul>
<li> the first column \"table[2:,1]\" contains the u[1] grid points,</li>
<li> the first row \"table[1,2:]\" contains the u[2] grid points,</li>
<li> the other rows and columns contain the data to be interpolated.</li>
</ul>
<p>
Example:
</p>
<pre>
           |       |       |       |
           |  1.0  |  2.0  |  3.0  |  // u2
       ----*-------*-------*-------*
       1.0 |  1.0  |  3.0  |  5.0  |
       ----*-------*-------*-------*
       2.0 |  2.0  |  4.0  |  6.0  |
       ----*-------*-------*-------*
     // u1
   is defined as
      table = [0.0,   1.0,   2.0,   3.0;
               1.0,   1.0,   3.0,   5.0;
               2.0,   2.0,   4.0,   6.0]
   If, e.g., the input u is [1.0;1.0], the output y is 1.0,
       e.g., the input u is [2.0;1.5], the output y is 3.0.
</pre>
<ul>
<li> The interpolation is <b>efficient</b>, because a search for a new
     interpolation starts at the interval used in the last call.</li>
<li> If the table has only <b>one element</b>, the table value is returned,
     independent of the value of the input signal.</li>
<li> If the input signal <b>u1</b> or <b>u2</b> is <b>outside</b> of the defined
     <b>interval</b>, the corresponding value is also determined by linear
     interpolation through the last or first two points of the table.</li>
<li> The grid values (first column and first row) have to be strictly
     increasing.</li>
</ul>
<p>
The table matrix can be defined in the following ways:
</p>
<ol>
<li> Explicitly supplied as <b>parameter matrix</b> \"table\",
     and the other parameters have the following values:
<pre>
   tableName is \"NoName\" or has only blanks,
   fileName  is \"NoName\" or has only blanks.
</pre></li>
<li> <b>Read</b> from a <b>file</b> \"fileName\" where the matrix is stored as
      \"tableName\". Both ASCII and MAT-file format is possible.
      (The ASCII format is described below).
      The MAT-file format comes in four different versions: v4, v6, v7 and v7.3.
      The library supports at least v4, v6 and v7 whereas v7.3 is optional.
      It is most convenient to generate the MAT-file from FreeMat or MATLAB&reg;
      by command
<pre>
   save tables.mat tab1 tab2 tab3
</pre>
      or Scilab by command
<pre>
   savematfile tables.mat tab1 tab2 tab3
</pre>
      when the three tables tab1, tab2, tab3 should be used from the model.<br>
      Note, a fileName can be defined as URI by using the helper function
      <a href=\"modelica://Modelica.Utilities.Files.loadResource\">loadResource</a>.</li>
<li>  Statically stored in function \"usertab\" in file \"usertab.c\".
      The matrix is identified by \"tableName\". Parameter
      fileName = \"NoName\" or has only blanks. Row-wise storage is always to be
      preferred as otherwise the table is reallocated and transposed.
      See the <a href=\"modelica://Modelica.Blocks.Tables\">Tables</a> package
      documentation for more details.</li>
</ol>
<p>
When the constant \"NO_FILE_SYSTEM\" is defined, all file I/O related parts of the
source code are removed by the C-preprocessor, such that no access to files takes place.
</p>
<p>
If tables are read from an ASCII-file, the file needs to have the
following structure (\"-----\" is not part of the file content):
</p>
<pre>
-----------------------------------------------------
#1
double table2D_1(3,4)   # comment line
0.0  1.0  2.0  3.0  # u[2] grid points
1.0  1.0  3.0  5.0
2.0  2.0  4.0  6.0

double table2D_2(4,4)   # comment line
0.0  1.0  2.0  3.0  # u[2] grid points
1.0  1.0  3.0  5.0
2.0  2.0  4.0  6.0
3.0  3.0  5.0  7.0
-----------------------------------------------------
</pre>
<p>
Note, that the first two characters in the file need to be
\"#1\" (a line comment defining the version number of the file format).
Afterwards, the corresponding matrix has to be declared
with type (= \"double\" or \"float\"), name and actual dimensions.
Finally, in successive rows of the file, the elements of the matrix
have to be given. The elements have to be provided as a sequence of
numbers in row-wise order (therefore a matrix row can span several
lines in the file and need not start at the beginning of a line).
Numbers have to be given according to C syntax (such as 2.3, -2, +2.e4).
Number separators are spaces, tab (\t), comma (,), or semicolon (;).
Several matrices may be defined one after another. Line comments start
with the hash symbol (#) and can appear everywhere.
Other characters, like trailing non comments, are not allowed in the file.
The matrix elements are interpreted in exactly the same way
as if the matrix is given as a parameter. For example, the first
column \"table2D_1[2:,1]\" contains the u[1] grid points,
and the first row \"table2D_1[1,2:]\" contains the u[2] grid points.
</p>

<p>
MATLAB is a registered trademark of The MathWorks, Inc.
</p>
</html>"),
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100.0, -100.0}, {100.0, 100.0}}, initialScale = 0.1), graphics = {Line(points = {{-60.0, 40.0}, {-60.0, -40.0}, {60.0, -40.0}, {60.0, 40.0}, {30.0, 40.0}, {30.0, -40.0}, {-30.0, -40.0}, {-30.0, 40.0}, {-60.0, 40.0}, {-60.0, 20.0}, {60.0, 20.0}, {60.0, 0.0}, {-60.0, 0.0}, {-60.0, -20.0}, {60.0, -20.0}, {60.0, -40.0}, {-60.0, -40.0}, {-60.0, 40.0}, {60.0, 40.0}, {60.0, -40.0}}), Line(points = {{0.0, 40.0}, {0.0, -40.0}}), Line(points = {{-60.0, 40.0}, {-30.0, 20.0}}), Line(points = {{-30.0, 40.0}, {-60.0, 20.0}}), Rectangle(origin = {2.3077, -0.0}, fillColor = {255, 215, 136}, fillPattern = FillPattern.Solid, extent = {{-62.3077, 0.0}, {-32.3077, 20.0}}), Rectangle(origin = {2.3077, -0.0}, fillColor = {255, 215, 136}, fillPattern = FillPattern.Solid, extent = {{-62.3077, -20.0}, {-32.3077, 0.0}}), Rectangle(origin = {2.3077, -0.0}, fillColor = {255, 215, 136}, fillPattern = FillPattern.Solid, extent = {{-62.3077, -40.0}, {-32.3077, -20.0}}), Rectangle(fillColor = {255, 215, 136}, fillPattern = FillPattern.Solid, extent = {{-30.0, 20.0}, {0.0, 40.0}}), Rectangle(fillColor = {255, 215, 136}, fillPattern = FillPattern.Solid, extent = {{0.0, 20.0}, {30.0, 40.0}}), Rectangle(origin = {-2.3077, -0.0}, fillColor = {255, 215, 136}, fillPattern = FillPattern.Solid, extent = {{32.3077, 20.0}, {62.3077, 40.0}})}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-60, 60}, {60, -60}}, fillColor = {235, 235, 235}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Line(points = {{60, 0}, {100, 0}}, color = {0, 0, 255}), Text(extent = {{-100, 100}, {100, 64}}, textString = "2 dimensional linear table interpolation", lineColor = {0, 0, 255}), Line(points = {{-54, 40}, {-54, -40}, {54, -40}, {54, 40}, {28, 40}, {28, -40}, {-28, -40}, {-28, 40}, {-54, 40}, {-54, 20}, {54, 20}, {54, 0}, {-54, 0}, {-54, -20}, {54, -20}, {54, -40}, {-54, -40}, {-54, 40}, {54, 40}, {54, -40}}, color = {0, 0, 0}), Line(points = {{0, 40}, {0, -40}}, color = {0, 0, 0}), Rectangle(extent = {{-54, 20}, {-28, 0}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 0}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-54, 0}, {-28, -20}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 0}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-54, -20}, {-28, -40}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 0}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-28, 40}, {0, 20}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 0}, fillPattern = FillPattern.Solid), Rectangle(extent = {{0, 40}, {28, 20}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 0}, fillPattern = FillPattern.Solid), Rectangle(extent = {{28, 40}, {54, 20}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 0}, fillPattern = FillPattern.Solid), Line(points = {{-54, 40}, {-28, 20}}, color = {0, 0, 0}), Line(points = {{-28, 40}, {-54, 20}}, color = {0, 0, 0}), Text(extent = {{-54, -40}, {-30, -56}}, textString = "u1", lineColor = {0, 0, 255}), Text(extent = {{28, 58}, {52, 44}}, textString = "u2", lineColor = {0, 0, 255}), Text(extent = {{-2, 12}, {32, -22}}, textString = "y", lineColor = {0, 0, 255})}));
      end CombiTable2D;
      annotation(
        Documentation(info = "<html>
<p>This package contains blocks for one- and two-dimensional interpolation in tables. </p>
<h4>Special interest topic: Statically stored tables for real-time simulation targets</h4>
<p>Especially for use on real-time platform targets (e.g., HIL-simulators) with <b>no file system</b>, it is possible to statically
store tables using a function &quot;usertab&quot; in a file conventionally named &quot;usertab.c&quot;. This can be more efficient than providing the tables as Modelica parameter arrays.</p>
<p>This is achieved by providing the tables in a specific structure as C-code and compiling that C-code together with the rest of the simulation model into a binary
that can be executed on the target platform. The &quot;Resources/Data/Tables/&quot; subdirectory of the MSL installation directory contains the files
<a href=\"modelica://Modelica/Resources/Data/Tables/usertab.c\">&quot;usertab.c&quot;</a> and <a href=\"modelica://Modelica/Resources/Data/Tables/usertab.h\">&quot;usertab.h&quot;</a>
that can be used as a template for own developments. While &quot;usertab.c&quot; would be typically used unmodified, the
&quot;usertab.h&quot; needs to adapted for the own needs.</p>
<p>In order to work it is necessary that the compiler pulls in the &quot;usertab.c&quot; file. Different Modelica tools might provide different mechanisms to do so.
Please consult the respective documentation/support for your Modelica tool.</p>
<p>A possible (though a bit &quot;hackish&quot;) Modelica standard conformant approach is to pull in the required files by utilizing a &quot;dummy&quot;-function that uses the Modelica external function
interface to pull in the required &quot;usertab.c&quot;. An example how this can be done is given below.</p>
<pre>
model Test25_usertab \"Test utilizing the usertab.c interface\"
  extends Modelica.Icons.Example;
public
  Modelica.Blocks.Sources.RealExpression realExpression(y=getUsertab(t_new.y))
    annotation (Placement(transformation(extent={{-40,-34},{-10,-14}})));
  Modelica.Blocks.Tables.CombiTable1D t_new(tableOnFile=true, tableName=\"TestTable_1D_a\")
    annotation (Placement(transformation(extent={{-40,0},{-20,20}})));
  Modelica.Blocks.Sources.Clock clock
    annotation (Placement(transformation(extent={{-80,0},{-60,20}})));
protected
  encapsulated function getUsertab
    input Real dummy_u[:];
    output Real dummy_y;
    external \"C\" dummy_y=  mydummyfunc(dummy_u);
    annotation(IncludeDirectory=\"modelica://Modelica/Resources/Data/Tables\",
           Include = \"#include \"usertab.c\"
 double mydummyfunc(const double* dummy_in) {
        return 0;
}
\");
  end getUsertab;
equation
  connect(clock.y,t_new. u[1]) annotation (Line(
      points={{-59,10},{-42,10}},
      color={0,0,127},
      smooth=Smooth.None));
  annotation (experiment(StartTime=0, StopTime=5), uses(Modelica(version=\"3.2.1\")));
end Test25_usertab;
</pre>
</html>"),
        Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-76, -26}, {80, -76}}, lineColor = {95, 95, 95}, fillColor = {235, 235, 235}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-76, 24}, {80, -26}}, lineColor = {95, 95, 95}, fillColor = {235, 235, 235}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-76, 74}, {80, 24}}, lineColor = {95, 95, 95}, fillColor = {235, 235, 235}, fillPattern = FillPattern.Solid), Line(points = {{-28, 74}, {-28, -76}}, color = {95, 95, 95}), Line(points = {{24, 74}, {24, -76}}, color = {95, 95, 95})}));
    end Tables;

    package Types "Library of constants and types with choices, especially to build menus"
      extends Modelica.Icons.TypesPackage;
      type Smoothness = enumeration(LinearSegments "Table points are linearly interpolated", ContinuousDerivative "Table points are interpolated such that the first derivative is continuous", ConstantSegments "Table points are not interpolated, but the value from the previous abscissa point is returned") "Enumeration defining the smoothness of table interpolation";
      type Extrapolation = enumeration(HoldLastPoint "Hold the first/last table point outside of the table scope", LastTwoPoints "Extrapolate by using the derivative at the first/last table points outside of the table scope", Periodic "Repeat the table scope periodically", NoExtrapolation "Extrapolation triggers an error") "Enumeration defining the extrapolation of time table interpolation";
      type Init = enumeration(NoInit "No initialization (start values are used as guess values with fixed=false)", SteadyState "Steady state initialization (derivatives of states are zero)", InitialState "Initialization with initial states", InitialOutput "Initialization with initial outputs (and steady state of the states if possible)") "Enumeration defining initialization of a block" annotation(
        Evaluate = true,
        Documentation(info = "<html>
  <p>The following initialization alternatives are available:</p>
  <dl>
    <dt><code><strong>NoInit</strong></code></dt>
      <dd>No initialization (start values are used as guess values with <code>fixed=false</code>)</dd>
    <dt><code><strong>SteadyState</strong></code></dt>
      <dd>Steady state initialization (derivatives of states are zero)</dd>
    <dt><code><strong>InitialState</strong></code></dt>
      <dd>Initialization with initial states</dd>
    <dt><code><strong>InitialOutput</strong></code></dt>
      <dd>Initialization with initial outputs (and steady state of the states if possible)</dd>
  </dl>
</html>"));
      type InitPID = enumeration(NoInit "No initialization (start values are used as guess values with fixed=false)", SteadyState "Steady state initialization (derivatives of states are zero)", InitialState "Initialization with initial states", InitialOutput "Initialization with initial outputs (and steady state of the states if possible)", DoNotUse_InitialIntegratorState "Do not use, only for backward compatibility (initialize only integrator state)") "Enumeration defining initialization of PID and LimPID blocks" annotation(
        Evaluate = true,
        Documentation(info = "<html>
<p>
This initialization type is identical to <a href=\"modelica://Modelica.Blocks.Types.Init\">Types.Init</a> and has just one
additional option <strong><code>DoNotUse_InitialIntegratorState</code></strong>. This option
is introduced in order that the default initialization for the
<code>Continuous.PID</code> and <code>Continuous.LimPID</code> blocks are backward
compatible. In Modelica 2.2, the integrators have been initialized
with their given states where as the D-part has not been initialized.
The option <strong><code>DoNotUse_InitialIntegratorState</code></strong> leads to this
initialization definition.
</p>

 <p>The following initialization alternatives are available:</p>
  <dl>
    <dt><code><strong>NoInit</strong></code></dt>
      <dd>No initialization (start values are used as guess values with <code>fixed=false</code>)</dd>
    <dt><code><strong>SteadyState</strong></code></dt>
      <dd>Steady state initialization (derivatives of states are zero)</dd>
    <dt><code><strong>InitialState</strong></code></dt>
      <dd>Initialization with initial states</dd>
    <dt><code><strong>InitialOutput</strong></code></dt>
      <dd>Initialization with initial outputs (and steady state of the states if possible)</dd>
    <dt><code><strong>DoNotUse_InitialIntegratorState</strong></code></dt>
      <dd>Do not use, only for backward compatibility (initialize only integrator state)</dd>
  </dl>
</html>"));
      type SimpleController = enumeration(P "P controller", PI "PI controller", PD "PD controller", PID "PID controller") "Enumeration defining P, PI, PD, or PID simple controller type" annotation(
        Evaluate = true);
      type AnalogFilter = enumeration(CriticalDamping "Filter with critical damping", Bessel "Bessel filter", Butterworth "Butterworth filter", ChebyshevI "Chebyshev I filter") "Enumeration defining the method of filtering" annotation(
        Evaluate = true);
      type FilterType = enumeration(LowPass "Low pass filter", HighPass "High pass filter", BandPass "Band pass filter", BandStop "Band stop / notch filter") "Enumeration of analog filter types (low, high, band pass or band stop filter)" annotation(
        Evaluate = true);

      class ExternalCombiTimeTable "External object of 1-dim. table where first column is time"
        extends ExternalObject;

        function constructor "Initialize 1-dim. table where first column is time"
          extends Modelica.Icons.Function;
          input String tableName "Table name";
          input String fileName "File name";
          input Real table[:, :];
          input Modelica.SIunits.Time startTime;
          input Integer columns[:];
          input Modelica.Blocks.Types.Smoothness smoothness;
          input Modelica.Blocks.Types.Extrapolation extrapolation;
          output ExternalCombiTimeTable externalCombiTimeTable;

          external "C" externalCombiTimeTable = ModelicaStandardTables_CombiTimeTable_init(tableName, fileName, table, size(table, 1), size(table, 2), startTime, columns, size(columns, 1), smoothness, extrapolation) annotation(
            Library = {"ModelicaStandardTables"});
        end constructor;

        function destructor "Terminate 1-dim. table where first column is time"
          extends Modelica.Icons.Function;
          input ExternalCombiTimeTable externalCombiTimeTable;

          external "C" ModelicaStandardTables_CombiTimeTable_close(externalCombiTimeTable) annotation(
            Library = {"ModelicaStandardTables"});
        end destructor;
      end ExternalCombiTimeTable;

      class ExternalCombiTable1D "External object of 1-dim. table defined by matrix"
        extends ExternalObject;

        function constructor "Initialize 1-dim. table defined by matrix"
          extends Modelica.Icons.Function;
          input String tableName "Table name";
          input String fileName "File name";
          input Real table[:, :];
          input Integer columns[:];
          input Modelica.Blocks.Types.Smoothness smoothness;
          output ExternalCombiTable1D externalCombiTable1D;

          external "C" externalCombiTable1D = ModelicaStandardTables_CombiTable1D_init(tableName, fileName, table, size(table, 1), size(table, 2), columns, size(columns, 1), smoothness) annotation(
            Library = {"ModelicaStandardTables"});
        end constructor;

        function destructor "Terminate 1-dim. table defined by matrix"
          extends Modelica.Icons.Function;
          input ExternalCombiTable1D externalCombiTable1D;

          external "C" ModelicaStandardTables_CombiTable1D_close(externalCombiTable1D) annotation(
            Library = {"ModelicaStandardTables"});
        end destructor;
      end ExternalCombiTable1D;

      class ExternalCombiTable2D "External object of 2-dim. table defined by matrix"
        extends ExternalObject;

        function constructor "Initialize 2-dim. table defined by matrix"
          extends Modelica.Icons.Function;
          input String tableName "Table name";
          input String fileName "File name";
          input Real table[:, :];
          input Modelica.Blocks.Types.Smoothness smoothness;
          output ExternalCombiTable2D externalCombiTable2D;

          external "C" externalCombiTable2D = ModelicaStandardTables_CombiTable2D_init(tableName, fileName, table, size(table, 1), size(table, 2), smoothness) annotation(
            Library = {"ModelicaStandardTables"});
        end constructor;

        function destructor "Terminate 2-dim. table defined by matrix"
          extends Modelica.Icons.Function;
          input ExternalCombiTable2D externalCombiTable2D;

          external "C" ModelicaStandardTables_CombiTable2D_close(externalCombiTable2D) annotation(
            Library = {"ModelicaStandardTables"});
        end destructor;
      end ExternalCombiTable2D;
      annotation(
        Documentation(info = "<HTML>
<p>
In this package <b>types</b>, <b>constants</b> and <b>external objects</b> are defined that are used
in library Modelica.Blocks. The types have additional annotation choices
definitions that define the menus to be built up in the graphical
user interface when the type is used as parameter in a declaration.
</p>
</HTML>"));
    end Types;

    package Icons "Icons for Blocks"
      extends Modelica.Icons.IconsPackage;

      partial block Block "Basic graphical layout of input/output block"
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, -100}, {100, 100}}, lineColor = {0, 0, 127}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-150, 150}, {150, 110}}, textString = "%name", lineColor = {0, 0, 255})}),
          Documentation(info = "<html>
<p>
Block that has only the basic icon for an input/output
block (no declarations, no equations). Most blocks
of package Modelica.Blocks inherit directly or indirectly
from this block.
</p>
</html>"));
      end Block;

      partial block BooleanBlock "Basic graphical layout of Boolean block"
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, -100}, {100, 100}}, lineColor = {255, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-150, 150}, {150, 110}}, textString = "%name", lineColor = {0, 0, 255})}),
          Documentation(info = "<html>
<p>
Block that has only the basic icon for an input/output,
Boolean block (no declarations, no equations).
</p>
</html>"));
      end BooleanBlock;

      partial block DiscreteBlock "Graphical layout of discrete block component icon"
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, -100}, {100, 100}}, lineColor = {0, 0, 127}, fillColor = {223, 211, 169}, lineThickness = 5.0, borderPattern = BorderPattern.Raised, fillPattern = FillPattern.Solid), Text(extent = {{-150, 150}, {150, 110}}, textString = "%name", lineColor = {0, 0, 255})}),
          Documentation(info = "<html>
<p>
Block that has only the basic icon for an input/output,
discrete block (no declarations, no equations), e.g.,
from Blocks.Discrete.
</p>
</html>"));
      end DiscreteBlock;

      partial block IntegerBlock "Basic graphical layout of Integer block"
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, -100}, {100, 100}}, lineColor = {255, 127, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-150, 150}, {150, 110}}, textString = "%name", lineColor = {0, 0, 255})}),
          Documentation(info = "<html>
<p>
Block that has only the basic icon for an input/output,
Integer block (no declarations, no equations).
</p>
</html>"));
      end IntegerBlock;

      partial block PartialBooleanBlock "Basic graphical layout of logical block"
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, 100}, {100, -100}}, fillColor = {210, 210, 210}, lineThickness = 5.0, fillPattern = FillPattern.Solid, borderPattern = BorderPattern.Raised), Text(extent = {{-150, 150}, {150, 110}}, textString = "%name", lineColor = {0, 0, 255})}),
          Documentation(info = "<html>
<p>
Block that has only the basic icon for an input/output,
Boolean block (no declarations, no equations) used especially
in the Blocks.Logical library.
</p>
</html>"));
      end PartialBooleanBlock;
    end Icons;
    annotation(
      Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100.0, -100.0}, {100.0, 100.0}}, initialScale = 0.1), graphics = {Rectangle(origin = {0.0, 35.1488}, fillColor = {255, 255, 255}, extent = {{-30.0, -20.1488}, {30.0, 20.1488}}), Rectangle(origin = {0.0, -34.8512}, fillColor = {255, 255, 255}, extent = {{-30.0, -20.1488}, {30.0, 20.1488}}), Line(origin = {-51.25, 0.0}, points = {{21.25, -35.0}, {-13.75, -35.0}, {-13.75, 35.0}, {6.25, 35.0}}), Polygon(origin = {-40.0, 35.0}, pattern = LinePattern.None, fillPattern = FillPattern.Solid, points = {{10.0, 0.0}, {-5.0, 5.0}, {-5.0, -5.0}}), Line(origin = {51.25, 0.0}, points = {{-21.25, 35.0}, {13.75, 35.0}, {13.75, -35.0}, {-6.25, -35.0}}), Polygon(origin = {40.0, -35.0}, pattern = LinePattern.None, fillPattern = FillPattern.Solid, points = {{-10.0, 0.0}, {5.0, 5.0}, {5.0, -5.0}})}),
      Documentation(info = "<html>
<p>
This library contains input/output blocks to build up block diagrams.
</p>

<dl>
<dt><b>Main Author:</b>
<dd><a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a><br>
    Deutsches Zentrum f&uuml;r Luft und Raumfahrt e. V. (DLR)<br>
    Oberpfaffenhofen<br>
    Postfach 1116<br>
    D-82230 Wessling<br>
    email: <A HREF=\"mailto:Martin.Otter@dlr.de\">Martin.Otter@dlr.de</A><br>
</dl>
<p>
Copyright &copy; 1998-2015, Modelica Association and DLR.
</p>
<p>
<i>This Modelica package is <u>free</u> software and the use is completely at <u>your own risk</u>; it can be redistributed and/or modified under the terms of the Modelica License 2. For license conditions (including the disclaimer of warranty) see <a href=\"modelica://Modelica.UsersGuide.ModelicaLicense2\">Modelica.UsersGuide.ModelicaLicense2</a> or visit <a href=\"https://www.modelica.org/licenses/ModelicaLicense2\"> https://www.modelica.org/licenses/ModelicaLicense2</a>.</i>
</p>
</html>", revisions = "<html>
<ul>
<li><i>June 23, 2004</i>
       by <a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a>:<br>
       Introduced new block connectors and adapted all blocks to the new connectors.
       Included subpackages Continuous, Discrete, Logical, Nonlinear from
       package ModelicaAdditions.Blocks.
       Included subpackage ModelicaAdditions.Table in Modelica.Blocks.Sources
       and in the new package Modelica.Blocks.Tables.
       Added new blocks to Blocks.Sources and Blocks.Logical.
       </li>
<li><i>October 21, 2002</i>
       by <a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a>
       and <a href=\"http://www.robotic.dlr.de/Christian.Schweiger/\">Christian Schweiger</a>:<br>
       New subpackage Examples, additional components.
       </li>
<li><i>June 20, 2000</i>
       by <a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a> and
       Michael Tiller:<br>
       Introduced a replaceable signal type into
       Blocks.Interfaces.RealInput/RealOutput:
<pre>
   replaceable type SignalType = Real
</pre>
       in order that the type of the signal of an input/output block
       can be changed to a physical type, for example:
<pre>
   Sine sin1(outPort(redeclare type SignalType=Modelica.SIunits.Torque))
</pre>
      </li>
<li><i>Sept. 18, 1999</i>
       by <a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a>:<br>
       Renamed to Blocks. New subpackages Math, Nonlinear.
       Additional components in subpackages Interfaces, Continuous
       and Sources. </li>
<li><i>June 30, 1999</i>
       by <a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a>:<br>
       Realized a first version, based on an existing Dymola library
       of Dieter Moormann and Hilding Elmqvist.</li>
</ul>
</html>"));
  end Blocks;

  package ComplexBlocks "Library of basic input/output control blocks with Complex signals"
    extends Modelica.Icons.Package;

    package Examples "Library of examples to demonstrate the usage of package Blocks"
      extends Modelica.Icons.ExamplesPackage;

      model TestConversionBlock "Test the conversion blocks"
        extends Modelica.Icons.Example;
        Modelica.Blocks.Sources.Ramp len(duration = 1, offset = 1E-6) annotation(
          Placement(transformation(extent = {{-80, 10}, {-60, 30}}, rotation = 0)));
        Modelica.Blocks.Sources.Ramp phi(height = 4 * Modelica.Constants.pi, duration = 1) annotation(
          Placement(transformation(extent = {{-80, -30}, {-60, -10}}, rotation = 0)));
        Modelica.ComplexBlocks.ComplexMath.PolarToComplex polarToComplex annotation(
          Placement(transformation(extent = {{-40, -10}, {-20, 10}}, rotation = 0)));
        Modelica.ComplexBlocks.ComplexMath.ComplexToReal complexToReal annotation(
          Placement(transformation(extent = {{0, -10}, {20, 10}}, rotation = 0)));
      equation
        connect(phi.y, polarToComplex.phi) annotation(
          Line(points = {{-59, -20}, {-50, -20}, {-50, -6}, {-46, -6}, {-42, -6}}, color = {0, 0, 127}));
        connect(len.y, polarToComplex.len) annotation(
          Line(points = {{-59, 20}, {-50, 20}, {-50, 6}, {-42, 6}}, color = {0, 0, 127}, smooth = Smooth.None));
        connect(polarToComplex.y, complexToReal.u) annotation(
          Line(points = {{-19, 0}, {-2, 0}}, color = {85, 170, 255}, smooth = Smooth.None));
        annotation(
          Documentation(info = "<html>
<p>A Complex signal is defined by its length and angle, both linearly rising with time.
Plotting the imaginary part versus the real part, you will see an Archimedean spiral.</p>
</html>"),
          experiment(StopTime = 1.01, Interval = 0.001));
      end TestConversionBlock;
      annotation(
        Documentation(info = "<html>
<p>This library demonstrates the usage of Complex blocks.</p>
</html>"));
    end Examples;

    package Interfaces "Library of connectors and partial models for input/output blocks"
      extends Modelica.Icons.InterfacesPackage;
      connector ComplexInput = input Complex "'input Complex' as connector" annotation(
        defaultComponentName = "u",
        Icon(coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = true, initialScale = 0.2), graphics = {Polygon(points = {{-100, 100}, {100, 0}, {-100, -100}, {-100, 100}}, lineColor = {85, 170, 255}, fillColor = {85, 170, 255}, fillPattern = FillPattern.Solid)}),
        Diagram(coordinateSystem(preserveAspectRatio = true, initialScale = 0.2, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{0, 50}, {100, 0}, {0, -50}, {0, 50}}, lineColor = {85, 170, 255}, fillColor = {85, 170, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-10, 85}, {-10, 60}}, lineColor = {0, 0, 127}, textString = "%name")}),
        Documentation(info = "<html>
<p>
Connector with one input signal of type Complex.
</p>
</html>"));
      connector ComplexOutput = output Complex "'output Complex' as connector" annotation(
        defaultComponentName = "y",
        Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{-100, 100}, {100, 0}, {-100, -100}, {-100, 100}}, lineColor = {85, 170, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid)}),
        Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{-100, 50}, {0, 0}, {-100, -50}, {-100, 50}}, lineColor = {85, 170, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Text(extent = {{30, 110}, {30, 60}}, lineColor = {0, 0, 127}, textString = "%name")}),
        Documentation(info = "<html>
<p>
Connector with one output signal of type Complex.
</p>
</html>"));

      partial block ComplexSO "Single Output continuous control block"
        extends Modelica.Blocks.Icons.Block;
        ComplexOutput y "Connector of Complex output signal" annotation(
          Placement(transformation(extent = {{100, -10}, {120, 10}}, rotation = 0)));
        annotation(
          Documentation(info = "<html>
<p>
Block has one continuous Complex output signal.
</p>
</html>"));
      end ComplexSO;

      partial block ComplexMO "Multiple Output continuous control block"
        extends Modelica.Blocks.Icons.Block;
        parameter Integer nout(min = 1) = 1 "Number of outputs";
        ComplexOutput y[nout] "Connector of Complex output signals" annotation(
          Placement(transformation(extent = {{100, -10}, {120, 10}}, rotation = 0)));
        annotation(
          Documentation(info = "<html>
<p>
Block has one continuous Complex output signal vector.
</p>
</html>"));
      end ComplexMO;

      partial block ComplexSISO "Single Input Single Output continuous control block"
        extends Modelica.Blocks.Icons.Block;
        ComplexInput u "Connector of Complex input signal" annotation(
          Placement(transformation(extent = {{-140, -20}, {-100, 20}}, rotation = 0)));
        ComplexOutput y "Connector of Complex output signal" annotation(
          Placement(transformation(extent = {{100, -10}, {120, 10}}, rotation = 0)));
        annotation(
          Documentation(info = "<html>
<p>
Block has one continuous Complex input and one continuous Complex output signal.
</p>
</html>"));
      end ComplexSISO;

      partial block ComplexSI2SO "2 Single Input / 1 Single Output continuous control block"
        extends Modelica.Blocks.Icons.Block;
        ComplexInput u1 "Connector of Complex input signal 1" annotation(
          Placement(transformation(extent = {{-140, 40}, {-100, 80}}, rotation = 0)));
        ComplexInput u2 "Connector of Complex input signal 2" annotation(
          Placement(transformation(extent = {{-140, -80}, {-100, -40}}, rotation = 0)));
        ComplexOutput y "Connector of Complex output signal" annotation(
          Placement(transformation(extent = {{100, -10}, {120, 10}}, rotation = 0)));
        annotation(
          Documentation(info = "<html>
<p>
Block has two continuous Complex input signals u1 and u2 and one
continuous Complex output signal y.
</p>
</html>"));
      end ComplexSI2SO;

      partial block ComplexSIMO "Single Input Multiple Output continuous control block"
        extends Modelica.Blocks.Icons.Block;
        parameter Integer nout = 1 "Number of outputs";
        ComplexInput u "Connector of Complex input signal" annotation(
          Placement(transformation(extent = {{-140, -20}, {-100, 20}}, rotation = 0)));
        ComplexOutput y[nout] "Connector of Complex output signals" annotation(
          Placement(transformation(extent = {{100, -10}, {120, 10}}, rotation = 0)));
        annotation(
          Documentation(info = "<html>
<p>
 Block has one continuous Complex input signal and a
vector of continuous Complex output signals.
</p>
</html>"));
      end ComplexSIMO;

      partial block ComplexMISO "Multiple Input Single Output continuous control block"
        extends Modelica.Blocks.Icons.Block;
        parameter Integer nin = 1 "Number of inputs";
        ComplexInput u[nin] "Connector of Complex input signals" annotation(
          Placement(transformation(extent = {{-140, -20}, {-100, 20}}, rotation = 0)));
        ComplexOutput y "Connector of Complex output signal" annotation(
          Placement(transformation(extent = {{100, -10}, {120, 10}}, rotation = 0)));
        annotation(
          Documentation(info = "<html>
<p>
Block has a vector of continuous Complex input signals and
one continuous Complex output signal.
</p>
</html>"));
      end ComplexMISO;

      partial block ComplexMIMO "Multiple Input Multiple Output continuous control block"
        extends Modelica.Blocks.Icons.Block;
        parameter Integer nin = 1 "Number of inputs";
        parameter Integer nout = 1 "Number of outputs";
        ComplexInput u[nin] "Connector of Complex input signals" annotation(
          Placement(transformation(extent = {{-140, -20}, {-100, 20}}, rotation = 0)));
        ComplexOutput y[nout] "Connector of Complex output signals" annotation(
          Placement(transformation(extent = {{100, -10}, {120, 10}}, rotation = 0)));
        annotation(
          Documentation(info = "<html>
<p>
Block has a continuous Complex input vector and a continuous Complex output signal vector.
The signal sizes of the input and output vector may be different.
</p>
</html>"));
      end ComplexMIMO;

      partial block ComplexMIMOs "Multiple Input Multiple Output continuous control block with same number of inputs and outputs"
        extends Modelica.Blocks.Icons.Block;
        parameter Integer n = 1 "Number of inputs (= number of outputs)";
        ComplexInput u[n] "Connector of Complex input signals" annotation(
          Placement(transformation(extent = {{-140, -20}, {-100, 20}}, rotation = 0)));
        ComplexOutput y[n] "Connector of Complex output signals" annotation(
          Placement(transformation(extent = {{100, -10}, {120, 10}}, rotation = 0)));
        annotation(
          Documentation(info = "<html>
<p>
Block has a continuous Complex input vector and a continuous Complex output signal vector
where the signal sizes of the input and output vector are identical.
</p>
</html>"));
      end ComplexMIMOs;

      partial block ComplexMI2MO "2 Multiple Input / Multiple Output continuous control block"
        extends Modelica.Blocks.Icons.Block;
        parameter Integer n = 1 "Dimension of input and output vectors.";
        ComplexInput u1[n] "Connector 1 of Complex input signals" annotation(
          Placement(transformation(extent = {{-140, 40}, {-100, 80}}, rotation = 0)));
        ComplexInput u2[n] "Connector 2 of Complex input signals" annotation(
          Placement(transformation(extent = {{-140, -80}, {-100, -40}}, rotation = 0)));
        ComplexOutput y[n] "Connector of Complex output signals" annotation(
          Placement(transformation(extent = {{100, -10}, {120, 10}}, rotation = 0)));
        annotation(
          Documentation(info = "<html>
<p>
Block has two continuous Complex input vectors u1 and u2 and one
continuous Complex output vector y.
All vectors have the same number of elements.
</p>
</html>"));
      end ComplexMI2MO;

      partial block ComplexSignalSource "Base class for continuous signal source"
        extends ComplexBlocks.Interfaces.ComplexSO;
        parameter Complex offset = Complex(0) "Offset of output signal y";
        parameter SIunits.Time startTime = 0 "Output y = offset for time < startTime";
        annotation(
          Documentation(info = "<html>
<p>
Basic block for Complex sources.
This component has one continuous Complex output signal y
and two parameters (offset, startTime) to shift the
generated signal.
</p>
</html>"));
      end ComplexSignalSource;
      annotation(
        Documentation(info = "<html>
<p>This library defines Complex input and output signals, as well as partial blocks.</p>
</html>"));
    end Interfaces;

    package ComplexMath "Library of mathematical functions as input/output blocks"
      extends Modelica.Icons.Package;

      block Gain "Output the product of a gain value with the input signal"
        parameter Complex k(re(start = 1), im(start = 0)) "Gain value multiplied with input signal";
      public
        Interfaces.ComplexInput u "Input signal connector" annotation(
          Placement(transformation(extent = {{-140, -20}, {-100, 20}}, rotation = 0)));
        Interfaces.ComplexOutput y "Output signal connector" annotation(
          Placement(transformation(extent = {{100, -10}, {120, 10}}, rotation = 0)));
      equation
        y = k * u;
        annotation(
          Documentation(info = "<html>
<p>
This block computes output <i>y</i> as
<i>product</i> of gain <i>k</i> with the
input <i>u</i>:
</p>
<pre>
    y = k * u;
</pre>

</html>"),
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{-100, -100}, {-100, 100}, {100, 0}, {-100, -100}}, lineColor = {0, 0, 127}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-150, -140}, {150, -100}}, lineColor = {0, 0, 0}, textString = "k=%k"), Text(extent = {{-150, 140}, {150, 100}}, textString = "%name", lineColor = {0, 0, 255})}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{-100, -100}, {-100, 100}, {100, 0}, {-100, -100}}, lineColor = {0, 0, 127}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-76, 38}, {0, -34}}, textString = "k", lineColor = {0, 0, 255})}));
      end Gain;

      block Sum "Output the sum of the elements of the input vector"
        extends Interfaces.ComplexMISO;
        parameter Complex k[nin] = fill(Complex(1, 0), nin) "Optional: sum coefficients";
      equation
        y = k * u;
        annotation(
          defaultComponentName = "sum1",
          Documentation(info = "<html>
<p>
This blocks computes output <b>y</b> as
<i>sum</i> of the elements of the input signal vector
<b>u</b>:
</p>
<pre>
    <b>y</b> = <b>u</b>[1] + <b>u</b>[2] + ...;
</pre>
<p>
Example:
</p>
<pre>
     parameter:   nin = 3;

  results in the following equations:

     y = u[1] + u[2] + u[3];
</pre>

</html>"),
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{26, 42}, {-34, 42}, {6, 2}, {-34, -38}, {26, -38}}, color = {0, 0, 0}, thickness = 0.25), Text(extent = {{-150, 150}, {150, 110}}, textString = "%name", lineColor = {0, 0, 255})}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, -100}, {100, 100}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{26, 42}, {-34, 42}, {6, 2}, {-34, -38}, {26, -38}}, color = {0, 0, 0}, thickness = 0.25)}));
      end Sum;

      block Feedback "Output difference between commanded and feedback input"
        Interfaces.ComplexInput u1 annotation(
          Placement(transformation(extent = {{-100, -20}, {-60, 20}}, rotation = 0)));
        Interfaces.ComplexInput u2 annotation(
          Placement(transformation(origin = {0, -80}, extent = {{-20, -20}, {20, 20}}, rotation = 90)));
        Interfaces.ComplexOutput y annotation(
          Placement(transformation(extent = {{80, -10}, {100, 10}}, rotation = 0)));
      equation
        y = u1 - u2;
        annotation(
          Documentation(info = "<html>
<p>
This blocks computes output <b>y</b> as <i>difference</i> of the
commanded input <b>u1</b> and the feedback
input <b>u2</b>:
</p>
<pre>
    <b>y</b> = <b>u1</b> - <b>u2</b>;
</pre>
<p>
Example:
</p>
<pre>
     parameter:   n = 2

  results in the following equations:

     y = u1 - u2
</pre>

</html>"),
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-20, 20}, {20, -20}}, lineColor = {0, 0, 127}, fillColor = {235, 235, 235}, fillPattern = FillPattern.Solid), Line(points = {{-60, 0}, {-20, 0}}, color = {0, 0, 127}), Line(points = {{20, 0}, {80, 0}}, color = {0, 0, 127}), Line(points = {{0, -20}, {0, -60}}, color = {0, 0, 127}), Text(extent = {{-14, 0}, {82, -94}}, lineColor = {0, 0, 0}, textString = "-"), Text(extent = {{-150, 94}, {150, 44}}, textString = "%name", lineColor = {0, 0, 255})}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-20, 20}, {20, -20}}, pattern = LinePattern.Solid, lineThickness = 0.25, fillColor = {235, 235, 235}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Line(points = {{-60, 0}, {-20, 0}}, color = {0, 0, 255}), Line(points = {{20, 0}, {80, 0}}, color = {0, 0, 255}), Line(points = {{0, -20}, {0, -60}}, color = {0, 0, 255}), Text(extent = {{-12, 10}, {84, -84}}, lineColor = {0, 0, 0}, textString = "-")}));
      end Feedback;

      block Add "Output the sum of the two inputs"
        extends Interfaces.ComplexSI2SO;
        parameter Complex k1 = Complex(1, 0) "Gain of upper input";
        parameter Complex k2 = Complex(1, 0) "Gain of lower input";
      equation
        y = k1 * u1 + k2 * u2;
        annotation(
          Documentation(info = "<html>
<p>
This blocks computes output <b>y</b> as <i>sum</i> of the
two input signals <b>u1</b> and <b>u2</b>:
</p>
<pre>
    <b>y</b> = k1*<b>u1</b> + k2*<b>u2</b>;
</pre>
<p>
Example:
</p>
<pre>
     parameter:   k1= +2, k2= -3

  results in the following equations:

     y = 2 * u1 - 3 * u2
</pre>

</html>"),
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-98, -52}, {7, -92}}, lineColor = {0, 0, 0}, textString = "%k2"), Text(extent = {{-100, 90}, {5, 50}}, lineColor = {0, 0, 0}, textString = "%k1"), Text(extent = {{-150, 150}, {150, 110}}, textString = "%name", lineColor = {0, 0, 255}), Line(points = {{-100, 60}, {-40, 60}, {-30, 40}}, color = {0, 0, 255}), Ellipse(extent = {{-50, 50}, {50, -50}}, lineColor = {0, 0, 255}), Line(points = {{-100, -60}, {-40, -60}, {-30, -40}}, color = {0, 0, 255}), Line(points = {{-15, -25.99}, {15, 25.99}}, color = {0, 0, 0}), Rectangle(extent = {{-100, -100}, {100, 100}}, lineColor = {0, 0, 127}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{50, 0}, {100, 0}}, color = {0, 0, 255}), Line(points = {{-100, 60}, {-74, 24}, {-44, 24}}, color = {0, 0, 127}), Line(points = {{-100, -60}, {-74, -28}, {-42, -28}}, color = {0, 0, 127}), Ellipse(extent = {{-50, 50}, {50, -50}}, lineColor = {0, 0, 127}), Line(points = {{50, 0}, {100, 0}}, color = {0, 0, 127}), Text(extent = {{-38, 34}, {38, -34}}, lineColor = {0, 0, 0}, textString = "+"), Text(extent = {{-100, 52}, {5, 92}}, lineColor = {0, 0, 0}, textString = "%k1"), Text(extent = {{-100, -52}, {5, -92}}, lineColor = {0, 0, 0}, textString = "%k2")}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, -100}, {100, 100}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-98, -52}, {7, -92}}, lineColor = {0, 0, 0}, textString = "%k2"), Text(extent = {{-100, 90}, {5, 50}}, lineColor = {0, 0, 0}, textString = "%k1"), Line(points = {{-100, 60}, {-40, 60}, {-30, 40}}, color = {0, 0, 255}), Ellipse(extent = {{-50, 50}, {50, -50}}, lineColor = {0, 0, 255}), Line(points = {{-100, -60}, {-40, -60}, {-30, -40}}, color = {0, 0, 255}), Line(points = {{-15, -25.99}, {15, 25.99}}, color = {0, 0, 0}), Rectangle(extent = {{-100, -100}, {100, 100}}, lineColor = {0, 0, 127}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{50, 0}, {100, 0}}, color = {0, 0, 255}), Line(points = {{-100, 60}, {-74, 24}, {-44, 24}}, color = {0, 0, 127}), Line(points = {{-100, -60}, {-74, -28}, {-42, -28}}, color = {0, 0, 127}), Ellipse(extent = {{-50, 50}, {50, -50}}, lineColor = {0, 0, 127}), Line(points = {{50, 0}, {100, 0}}, color = {0, 0, 127}), Text(extent = {{-38, 34}, {38, -34}}, lineColor = {0, 0, 0}, textString = "+"), Text(extent = {{-100, 52}, {5, 92}}, lineColor = {0, 0, 0}, textString = "k1"), Text(extent = {{-100, -52}, {5, -92}}, lineColor = {0, 0, 0}, textString = "k2")}));
      end Add;

      block Add3 "Output the sum of the three inputs"
        extends Modelica.Blocks.Icons.Block;
        parameter Complex k1 = Complex(1, 0) "Gain of upper input";
        parameter Complex k2 = Complex(1, 0) "Gain of middle input";
        parameter Complex k3 = Complex(1, 0) "Gain of lower input";
        Interfaces.ComplexInput u1 "Connector 1 of Complex input signals" annotation(
          Placement(transformation(extent = {{-140, 60}, {-100, 100}}, rotation = 0)));
        Interfaces.ComplexInput u2 "Connector 2 of Complex input signals" annotation(
          Placement(transformation(extent = {{-140, -20}, {-100, 20}}, rotation = 0)));
        Interfaces.ComplexInput u3 "Connector 3 of Complex input signals" annotation(
          Placement(transformation(extent = {{-140, -100}, {-100, -60}}, rotation = 0)));
        Interfaces.ComplexOutput y "Connector of Complex output signals" annotation(
          Placement(transformation(extent = {{100, -10}, {120, 10}}, rotation = 0)));
      equation
        y = k1 * u1 + k2 * u2 + k3 * u3;
        annotation(
          Documentation(info = "<html>
<p>
This blocks computes output <b>y</b> as <i>sum</i> of the
three input signals <b>u1</b>, <b>u2</b> and <b>u3</b>:
</p>
<pre>
    <b>y</b> = k1*<b>u1</b> + k2*<b>u2</b> + k3*<b>u3</b>;
</pre>
<p>
Example:
</p>
<pre>
     parameter:   k1= +2, k2= -3, k3=1;

  results in the following equations:

     y = 2 * u1 - 3 * u2 + u3;
</pre>

</html>"),
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-100, 50}, {5, 90}}, lineColor = {0, 0, 0}, textString = "%k1"), Text(extent = {{-100, -20}, {5, 20}}, lineColor = {0, 0, 0}, textString = "%k2"), Text(extent = {{-100, -50}, {5, -90}}, lineColor = {0, 0, 0}, textString = "%k3"), Text(extent = {{2, 36}, {100, -44}}, lineColor = {0, 0, 0}, textString = "+")}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, -100}, {100, 100}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-100, 50}, {5, 90}}, lineColor = {0, 0, 0}, textString = "%k1"), Text(extent = {{-100, -20}, {5, 20}}, lineColor = {0, 0, 0}, textString = "%k2"), Text(extent = {{-100, -50}, {5, -90}}, lineColor = {0, 0, 0}, textString = "%k3"), Text(extent = {{2, 36}, {100, -44}}, lineColor = {0, 0, 0}, textString = "+"), Rectangle(extent = {{-100, -100}, {100, 100}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-100, 50}, {5, 90}}, lineColor = {0, 0, 0}, textString = "k1"), Text(extent = {{-100, -20}, {5, 20}}, lineColor = {0, 0, 0}, textString = "k2"), Text(extent = {{-100, -50}, {5, -90}}, lineColor = {0, 0, 0}, textString = "k3"), Text(extent = {{2, 36}, {100, -44}}, lineColor = {0, 0, 0}, textString = "+")}));
      end Add3;

      block Product "Output product of the two inputs"
        extends Interfaces.ComplexSI2SO;
      equation
        y = u1 * u2;
        annotation(
          Documentation(info = "<html>
<p>
This blocks computes the output <b>y</b> (element-wise)
as <i>product</i> of the corresponding elements of
the two inputs <b>u1</b> and <b>u2</b>:
</p>
<pre>
    y = u1 * u2;
</pre>

</html>"),
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-100, 60}, {-40, 60}, {-30, 40}}, color = {0, 0, 127}), Line(points = {{-100, -60}, {-40, -60}, {-30, -40}}, color = {0, 0, 127}), Line(points = {{50, 0}, {100, 0}}, color = {0, 0, 127}), Line(points = {{-30, 0}, {30, 0}}, color = {0, 0, 0}), Line(points = {{-15, 25.99}, {15, -25.99}}, color = {0, 0, 0}), Line(points = {{-15, -25.99}, {15, 25.99}}, color = {0, 0, 0}), Ellipse(extent = {{-50, 50}, {50, -50}}, lineColor = {0, 0, 127})}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, -100}, {100, 100}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{-100, 60}, {-40, 60}, {-30, 40}}, color = {0, 0, 255}), Line(points = {{-100, -60}, {-40, -60}, {-30, -40}}, color = {0, 0, 255}), Line(points = {{50, 0}, {100, 0}}, color = {0, 0, 255}), Line(points = {{-30, 0}, {30, 0}}, color = {0, 0, 0}), Line(points = {{-15, 25.99}, {15, -25.99}}, color = {0, 0, 0}), Line(points = {{-15, -25.99}, {15, 25.99}}, color = {0, 0, 0}), Ellipse(extent = {{-50, 50}, {50, -50}}, lineColor = {0, 0, 255})}));
      end Product;

      block Division "Output first input divided by second input"
        extends Interfaces.ComplexSI2SO;
      equation
        y = u1 / u2;
        annotation(
          Documentation(info = "<html>
<p>
This block computes the output <b>y</b> (element-wise)
by <i>dividing</i> the corresponding elements of
the two inputs <b>u1</b> and <b>u2</b>:
</p>
<pre>
    y = u1 / u2;
</pre>

</html>"),
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{50, 0}, {100, 0}}, color = {0, 0, 127}), Line(points = {{-30, 0}, {30, 0}}, color = {0, 0, 0}), Ellipse(extent = {{-5, 20}, {5, 30}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-5, -20}, {5, -30}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-50, 50}, {50, -50}}, lineColor = {0, 0, 127}), Text(extent = {{-150, 150}, {150, 110}}, textString = "%name", lineColor = {0, 0, 255}), Line(points = {{-100, 60}, {-66, 60}, {-40, 30}}, color = {0, 0, 127}), Line(points = {{-100, -60}, {0, -60}, {0, -50}}, color = {0, 0, 127})}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, -100}, {100, 100}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{50, 0}, {100, 0}}, color = {0, 0, 255}), Line(points = {{-30, 0}, {30, 0}}, color = {0, 0, 0}), Ellipse(extent = {{-5, 20}, {5, 30}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-5, -20}, {5, -30}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-50, 50}, {50, -50}}, lineColor = {0, 0, 255}), Line(points = {{-100, 60}, {-66, 60}, {-40, 30}}, color = {0, 0, 255}), Line(points = {{-100, -60}, {0, -60}, {0, -50}}, color = {0, 0, 255})}));
      end Division;

      block Sqrt "Output the square root of the input (input >= 0 required)"
        extends Interfaces.ComplexSISO;
      equation
        y = Modelica.ComplexMath.'sqrt'(u);
        annotation(
          defaultComponentName = "sqrt1",
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-90, -80}, {68, -80}}, color = {192, 192, 192}), Polygon(points = {{90, -80}, {68, -72}, {68, -88}, {90, -80}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-80, -80}, {-79.2, -68.7}, {-78.4, -64}, {-76.8, -57.3}, {-73.6, -47.9}, {-67.9, -36.1}, {-59.1, -22.2}, {-46.2, -6.49}, {-28.5, 10.7}, {-4.42, 30}, {27.7, 51.3}, {69.5, 74.7}, {80, 80}}, color = {0, 0, 0}), Polygon(points = {{-80, 90}, {-88, 68}, {-72, 68}, {-80, 90}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-80, -88}, {-80, 68}}, color = {192, 192, 192}), Text(extent = {{-8, -4}, {64, -52}}, lineColor = {192, 192, 192}, textString = "sqrt")}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-92, -80}, {84, -80}}, color = {192, 192, 192}), Polygon(points = {{100, -80}, {84, -74}, {84, -86}, {100, -80}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-80, -80}, {-79.2, -68.7}, {-78.4, -64}, {-76.8, -57.3}, {-73.6, -47.9}, {-67.9, -36.1}, {-59.1, -22.2}, {-46.2, -6.49}, {-28.5, 10.7}, {-4.42, 30}, {27.7, 51.3}, {69.5, 74.7}, {80, 80}}, color = {0, 0, 0}), Polygon(points = {{-80, 98}, {-86, 82}, {-74, 82}, {-80, 98}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-80, -90}, {-80, 84}}, color = {192, 192, 192}), Text(extent = {{-71, 98}, {-44, 78}}, lineColor = {160, 160, 164}, textString = "y"), Text(extent = {{60, -52}, {84, -72}}, lineColor = {160, 160, 164}, textString = "u")}),
          Documentation(info = "<HTML>
<p>
This blocks computes the output <b>y</b>
as <i>square root</i> of the input <b>u</b>:
</p>
<pre>
    y = <b>sqrt</b>( u );
</pre>
<p>
All elements of the input vector shall be zero or positive.
Otherwise an error occurs.
</p>

</html>"));
      end Sqrt;

      block Sin "Output the sine of the input"
        extends Interfaces.ComplexSISO;
      equation
        y = Modelica.ComplexMath.sin(u);
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{-80, 90}, {-88, 68}, {-72, 68}, {-80, 90}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-80, -80}, {-80, 68}}, color = {192, 192, 192}), Line(points = {{-90, 0}, {68, 0}}, color = {192, 192, 192}), Line(points = {{-80, 0}, {-68.7, 34.2}, {-61.5, 53.1}, {-55.1, 66.4}, {-49.4, 74.6}, {-43.8, 79.1}, {-38.2, 79.8}, {-32.6, 76.6}, {-26.9, 69.7}, {-21.3, 59.4}, {-14.9, 44.1}, {-6.83, 21.2}, {10.1, -30.8}, {17.3, -50.2}, {23.7, -64.2}, {29.3, -73.1}, {35, -78.4}, {40.6, -80}, {46.2, -77.6}, {51.9, -71.5}, {57.5, -61.9}, {63.9, -47.2}, {72, -24.8}, {80, 0}}, color = {0, 0, 0}), Polygon(points = {{90, 0}, {68, 8}, {68, -8}, {90, 0}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Text(extent = {{12, 84}, {84, 36}}, lineColor = {192, 192, 192}, textString = "sin")}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-80, 80}, {-88, 80}}, color = {192, 192, 192}), Line(points = {{-80, -80}, {-88, -80}}, color = {192, 192, 192}), Line(points = {{-80, -90}, {-80, 84}}, color = {192, 192, 192}), Text(extent = {{-75, 98}, {-46, 78}}, lineColor = {160, 160, 164}, textString = "y"), Polygon(points = {{-80, 96}, {-86, 80}, {-74, 80}, {-80, 96}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-100, 0}, {84, 0}}, color = {192, 192, 192}), Polygon(points = {{100, 0}, {84, 6}, {84, -6}, {100, 0}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-80, 0}, {-68.7, 34.2}, {-61.5, 53.1}, {-55.1, 66.4}, {-49.4, 74.6}, {-43.8, 79.1}, {-38.2, 79.8}, {-32.6, 76.6}, {-26.9, 69.7}, {-21.3, 59.4}, {-14.9, 44.1}, {-6.83, 21.2}, {10.1, -30.8}, {17.3, -50.2}, {23.7, -64.2}, {29.3, -73.1}, {35, -78.4}, {40.6, -80}, {46.2, -77.6}, {51.9, -71.5}, {57.5, -61.9}, {63.9, -47.2}, {72, -24.8}, {80, 0}}, color = {0, 0, 0}), Text(extent = {{-105, 72}, {-85, 88}}, textString = "1", lineColor = {0, 0, 255}), Text(extent = {{70, 25}, {90, 5}}, textString = "2*pi", lineColor = {0, 0, 255}), Text(extent = {{-105, -72}, {-85, -88}}, textString = "-1", lineColor = {0, 0, 255}), Text(extent = {{76, -10}, {98, -30}}, lineColor = {160, 160, 164}, textString = "u")}),
          Documentation(info = "<HTML>
<p>
This blocks computes the output <b>y</b>
as <b>sine</b> of the input <b>u</b>:
</p>
<pre>
    y = <b>sin</b>( u );
</pre>

<p>
<img src=\"modelica://Modelica/Resources/Images/Math/sin.png\"
     alt=\"sin.png\">
</p>

</html>"));
      end Sin;

      block Cos "Output the cosine of the input"
        extends Interfaces.ComplexSISO;
      equation
        y = Modelica.ComplexMath.cos(u);
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{-80, 90}, {-88, 68}, {-72, 68}, {-80, 90}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-80, -80}, {-80, 68}}, color = {192, 192, 192}), Line(points = {{-90, 0}, {68, 0}}, color = {192, 192, 192}), Polygon(points = {{90, 0}, {68, 8}, {68, -8}, {90, 0}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-80, 80}, {-74.4, 78.1}, {-68.7, 72.3}, {-63.1, 63}, {-56.7, 48.7}, {-48.6, 26.6}, {-29.3, -32.5}, {-22.1, -51.7}, {-15.7, -65.3}, {-10.1, -73.8}, {-4.42, -78.8}, {1.21, -79.9}, {6.83, -77.1}, {12.5, -70.6}, {18.1, -60.6}, {24.5, -45.7}, {32.6, -23}, {50.3, 31.3}, {57.5, 50.7}, {63.9, 64.6}, {69.5, 73.4}, {75.2, 78.6}, {80, 80}}, color = {0, 0, 0}), Text(extent = {{-36, 82}, {36, 34}}, lineColor = {192, 192, 192}, textString = "cos")}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-80, 80}, {-88, 80}}, color = {192, 192, 192}), Line(points = {{-80, -80}, {-88, -80}}, color = {192, 192, 192}), Line(points = {{-80, -90}, {-80, 84}}, color = {192, 192, 192}), Text(extent = {{-75, 102}, {-48, 82}}, lineColor = {160, 160, 164}, textString = "y"), Polygon(points = {{-80, 100}, {-86, 84}, {-74, 84}, {-80, 100}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Text(extent = {{-103, 72}, {-83, 88}}, textString = "1", lineColor = {0, 0, 255}), Text(extent = {{-79, -72}, {-59, -88}}, textString = "-1", lineColor = {0, 0, 255}), Text(extent = {{70, 25}, {90, 5}}, textString = "2*pi", lineColor = {0, 0, 255}), Line(points = {{-100, 0}, {84, 0}}, color = {192, 192, 192}), Polygon(points = {{100, 0}, {84, 6}, {84, -6}, {100, 0}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-80, 80}, {-74.4, 78.1}, {-68.7, 72.3}, {-63.1, 63}, {-56.7, 48.7}, {-48.6, 26.6}, {-29.3, -32.5}, {-22.1, -51.7}, {-15.7, -65.3}, {-10.1, -73.8}, {-4.42, -78.8}, {1.21, -79.9}, {6.83, -77.1}, {12.5, -70.6}, {18.1, -60.6}, {24.5, -45.7}, {32.6, -23}, {50.3, 31.3}, {57.5, 50.7}, {63.9, 64.6}, {69.5, 73.4}, {75.2, 78.6}, {80, 80}}, color = {0, 0, 0}), Text(extent = {{74, -4}, {98, -24}}, lineColor = {160, 160, 164}, textString = "u")}),
          Documentation(info = "<HTML>
<p>
This blocks computes the output <b>y</b>
as <b>cos</b> of the input <b>u</b>:
</p>
<pre>
    y = <b>cos</b>( u );
</pre>

<p>
<img src=\"modelica://Modelica/Resources/Images/Math/cos.png\"
     alt=\"cos.png\">
</p>

</html>"));
      end Cos;

      block Tan "Output the tangent of the input"
        extends Interfaces.ComplexSISO;
      equation
        y = Modelica.ComplexMath.tan(u);
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{0, 90}, {-8, 68}, {8, 68}, {0, 90}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{0, -80}, {0, 68}}, color = {192, 192, 192}), Line(points = {{-80, -80}, {-78.4, -68.4}, {-76.8, -59.7}, {-74.4, -50}, {-71.2, -40.9}, {-67.1, -33}, {-60.7, -24.8}, {-51.1, -17.2}, {-35.8, -9.98}, {-4.42, -1.07}, {33.4, 9.12}, {49.4, 16.2}, {59.1, 23.2}, {65.5, 30.6}, {70.4, 39.1}, {73.6, 47.4}, {76, 56.1}, {77.6, 63.8}, {80, 80}}, color = {0, 0, 0}), Line(points = {{-90, 0}, {68, 0}}, color = {192, 192, 192}), Polygon(points = {{90, 0}, {68, 8}, {68, -8}, {90, 0}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Text(extent = {{-90, 72}, {-18, 24}}, lineColor = {192, 192, 192}, textString = "tan")}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{0, 80}, {-8, 80}}, color = {192, 192, 192}), Line(points = {{0, -80}, {-8, -80}}, color = {192, 192, 192}), Line(points = {{0, -88}, {0, 86}}, color = {192, 192, 192}), Text(extent = {{11, 100}, {38, 80}}, lineColor = {160, 160, 164}, textString = "y"), Polygon(points = {{0, 102}, {-6, 86}, {6, 86}, {0, 102}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Text(extent = {{-37, -72}, {-17, -88}}, textString = "-5.8", lineColor = {0, 0, 255}), Text(extent = {{-33, 86}, {-13, 70}}, textString = " 5.8", lineColor = {0, 0, 255}), Text(extent = {{70, 25}, {90, 5}}, textString = "1.4", lineColor = {0, 0, 255}), Line(points = {{-100, 0}, {84, 0}}, color = {192, 192, 192}), Polygon(points = {{100, 0}, {84, 6}, {84, -6}, {100, 0}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-80, -80}, {-78.4, -68.4}, {-76.8, -59.7}, {-74.4, -50}, {-71.2, -40.9}, {-67.1, -33}, {-60.7, -24.8}, {-51.1, -17.2}, {-35.8, -9.98}, {-4.42, -1.07}, {33.4, 9.12}, {49.4, 16.2}, {59.1, 23.2}, {65.5, 30.6}, {70.4, 39.1}, {73.6, 47.4}, {76, 56.1}, {77.6, 63.8}, {80, 80}}, color = {0, 0, 0}), Text(extent = {{70, -6}, {94, -26}}, lineColor = {160, 160, 164}, textString = "u")}),
          Documentation(info = "<HTML>
<p>
This blocks computes the output <b>y</b>
as <b>tan</b> of the input <b>u</b>:
</p>
<pre>
    y = <b>tan</b>( u );
</pre>

<p>
<img src=\"modelica://Modelica/Resources/Images/Math/tan.png\"
     alt=\"tan.png\">
</p>

</html>"));
      end Tan;

      block Asin "Output the arc sine of the input"
        extends Interfaces.ComplexSISO;
      equation
        y = Modelica.ComplexMath.asin(u);
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{0, 90}, {-8, 68}, {8, 68}, {0, 90}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{0, -80}, {0, 68}}, color = {192, 192, 192}), Line(points = {{-80, -80}, {-79.2, -72.8}, {-77.6, -67.5}, {-73.6, -59.4}, {-66.3, -49.8}, {-53.5, -37.3}, {-30.2, -19.7}, {37.4, 24.8}, {57.5, 40.8}, {68.7, 52.7}, {75.2, 62.2}, {77.6, 67.5}, {80, 80}}, color = {0, 0, 0}), Line(points = {{-90, 0}, {68, 0}}, color = {192, 192, 192}), Polygon(points = {{90, 0}, {68, 8}, {68, -8}, {90, 0}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Text(extent = {{-88, 78}, {-16, 30}}, lineColor = {192, 192, 192}, textString = "asin")}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{0, 80}, {-8, 80}}, color = {192, 192, 192}), Line(points = {{0, -80}, {-8, -80}}, color = {192, 192, 192}), Line(points = {{0, -90}, {0, 84}}, color = {192, 192, 192}), Text(extent = {{9, 102}, {36, 82}}, lineColor = {160, 160, 164}, textString = "y"), Polygon(points = {{0, 100}, {-6, 84}, {6, 84}, {0, 100}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Text(extent = {{-40, -72}, {-15, -88}}, textString = "-pi/2", lineColor = {0, 0, 255}), Text(extent = {{-38, 88}, {-13, 72}}, textString = " pi/2", lineColor = {0, 0, 255}), Text(extent = {{70, 25}, {90, 5}}, textString = "+1", lineColor = {0, 0, 255}), Text(extent = {{-90, 21}, {-70, 1}}, textString = "-1", lineColor = {0, 0, 255}), Line(points = {{-100, 0}, {84, 0}}, color = {192, 192, 192}), Polygon(points = {{100, 0}, {84, 6}, {84, -6}, {100, 0}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-80, -80}, {-79.2, -72.8}, {-77.6, -67.5}, {-73.6, -59.4}, {-66.3, -49.8}, {-53.5, -37.3}, {-30.2, -19.7}, {37.4, 24.8}, {57.5, 40.8}, {68.7, 52.7}, {75.2, 62.2}, {77.6, 67.5}, {80, 80}}, color = {0, 0, 0}), Text(extent = {{74, -4}, {100, -26}}, lineColor = {160, 160, 164}, textString = "u")}),
          Documentation(info = "<HTML>
<p>
This blocks computes the output <b>y</b> as the
<i>sine-inverse</i> of the input <b>u</b>:
</p>
<pre>
    y = <b>asin</b>( u );
</pre>
<p>
The absolute values of the elements of the input <b>u</b> need to
be less or equal to one (<b>abs</b>( u ) <= 1).
Otherwise an error occurs.
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Math/asin.png\"
     alt=\"asin.png\">
</p>

</html>"));
      end Asin;

      block Acos "Output the arc cosine of the input"
        extends Interfaces.ComplexSISO;
      equation
        y = Modelica.ComplexMath.acos(u);
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{0, 90}, {-8, 68}, {8, 68}, {0, 90}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-80, 80}, {-79.2, 72.8}, {-77.6, 67.5}, {-73.6, 59.4}, {-66.3, 49.8}, {-53.5, 37.3}, {-30.2, 19.7}, {37.4, -24.8}, {57.5, -40.8}, {68.7, -52.7}, {75.2, -62.2}, {77.6, -67.5}, {80, -80}}, color = {0, 0, 0}), Line(points = {{0, -88}, {0, 68}}, color = {192, 192, 192}), Line(points = {{-90, -80}, {68, -80}}, color = {192, 192, 192}), Polygon(points = {{90, -80}, {68, -72}, {68, -88}, {90, -80}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Text(extent = {{-86, -14}, {-14, -62}}, lineColor = {192, 192, 192}, textString = "acos")}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{0, 80}, {-8, 80}}, color = {192, 192, 192}), Line(points = {{0, -80}, {-8, -80}}, color = {192, 192, 192}), Line(points = {{0, -90}, {0, 84}}, color = {192, 192, 192}), Text(extent = {{13, 100}, {42, 82}}, lineColor = {160, 160, 164}, textString = "y"), Polygon(points = {{0, 100}, {-6, 84}, {6, 84}, {0, 100}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-100, -80}, {84, -80}}, color = {192, 192, 192}), Polygon(points = {{100, -80}, {84, -74}, {84, -86}, {100, -80}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-80, 80}, {-79.2, 72.8}, {-77.6, 67.5}, {-73.6, 59.4}, {-66.3, 49.8}, {-53.5, 37.3}, {-30.2, 19.7}, {37.4, -24.8}, {57.5, -40.8}, {68.7, -52.7}, {75.2, -62.2}, {77.6, -67.5}, {80, -80}}, color = {0, 0, 0}), Text(extent = {{-30, 88}, {-5, 72}}, textString = " pi", lineColor = {0, 0, 255}), Text(extent = {{-94, -57}, {-74, -77}}, textString = "-1", lineColor = {0, 0, 255}), Text(extent = {{80, -45}, {100, -65}}, textString = "+1", lineColor = {0, 0, 255}), Text(extent = {{76, -84}, {102, -102}}, lineColor = {160, 160, 164}, textString = "u")}),
          Documentation(info = "<HTML>
<p>
This blocks computes the output <b>y</b> as the
<i>cosine-inverse</i> of the input <b>u</b>:
</p>
<pre>
    y = <b>acos</b>( u );
</pre>
<p>
The absolute values of the elements of the input <b>u</b> need to
be less or equal to one (<b>abs</b>( u ) <= 1).
Otherwise an error occurs.
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Math/acos.png\"
     alt=\"acos.png\">
</p>

</html>"));
      end Acos;

      block Atan "Output the arc tangent of the input"
        extends Interfaces.ComplexSISO;
      equation
        y = Modelica.ComplexMath.atan(u);
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{0, 90}, {-8, 68}, {8, 68}, {0, 90}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{0, -80}, {0, 68}}, color = {192, 192, 192}), Line(points = {{-80, -80}, {-52.7, -75.2}, {-37.4, -69.7}, {-26.9, -63}, {-19.7, -55.2}, {-14.1, -45.8}, {-10.1, -36.4}, {-6.03, -23.9}, {-1.21, -5.06}, {5.23, 21}, {9.25, 34.1}, {13.3, 44.2}, {18.1, 52.9}, {24.5, 60.8}, {33.4, 67.6}, {47, 73.6}, {69.5, 78.6}, {80, 80}}, color = {0, 0, 0}), Line(points = {{-90, 0}, {68, 0}}, color = {192, 192, 192}), Polygon(points = {{90, 0}, {68, 8}, {68, -8}, {90, 0}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Text(extent = {{-86, 68}, {-14, 20}}, lineColor = {192, 192, 192}, textString = "atan")}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{0, 80}, {-8, 80}}, color = {192, 192, 192}), Line(points = {{0, -80}, {-8, -80}}, color = {192, 192, 192}), Line(points = {{0, -90}, {0, 84}}, color = {192, 192, 192}), Text(extent = {{13, 102}, {42, 82}}, lineColor = {160, 160, 164}, textString = "y"), Polygon(points = {{0, 100}, {-6, 84}, {6, 84}, {0, 100}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-100, 0}, {84, 0}}, color = {192, 192, 192}), Polygon(points = {{100, 0}, {84, 6}, {84, -6}, {100, 0}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-80, -80}, {-52.7, -75.2}, {-37.4, -69.7}, {-26.9, -63}, {-19.7, -55.2}, {-14.1, -45.8}, {-10.1, -36.4}, {-6.03, -23.9}, {-1.21, -5.06}, {5.23, 21}, {9.25, 34.1}, {13.3, 44.2}, {18.1, 52.9}, {24.5, 60.8}, {33.4, 67.6}, {47, 73.6}, {69.5, 78.6}, {80, 80}}, color = {0, 0, 0}), Text(extent = {{-32, 91}, {-12, 71}}, textString = "1.4", lineColor = {0, 0, 255}), Text(extent = {{-32, -71}, {-12, -91}}, textString = "-1.4", lineColor = {0, 0, 255}), Text(extent = {{73, 26}, {93, 10}}, textString = " 5.8", lineColor = {0, 0, 255}), Text(extent = {{-103, 20}, {-83, 4}}, textString = "-5.8", lineColor = {0, 0, 255}), Text(extent = {{66, -8}, {94, -28}}, lineColor = {160, 160, 164}, textString = "u")}),
          Documentation(info = "<HTML>
<p>
This blocks computes the output <b>y</b> as the
<i>tangent-inverse</i> of the input <b>u</b>:
</p>
<pre>
    y= <b>atan</b>( u );
</pre>

<p>
<img src=\"modelica://Modelica/Resources/Images/Math/atan.png\"
     alt=\"atan.png\">
</p>

</html>"));
      end Atan;

      block Sinh "Output the hyperbolic sine of the input"
        extends Interfaces.ComplexSISO;
      equation
        y = Modelica.ComplexMath.sinh(u);
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{0, 90}, {-8, 68}, {8, 68}, {0, 90}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Text(extent = {{-86, 80}, {-14, 32}}, lineColor = {192, 192, 192}, textString = "sinh"), Line(points = {{0, -80}, {0, 68}}, color = {192, 192, 192}), Line(points = {{-90, 0}, {68, 0}}, color = {192, 192, 192}), Line(points = {{-80, -80}, {-76, -65.4}, {-71.2, -51.4}, {-65.5, -38.8}, {-59.1, -28.1}, {-51.1, -18.7}, {-41.4, -11.4}, {-27.7, -5.5}, {-4.42, -0.653}, {24.5, 4.57}, {39, 10.1}, {49.4, 17.2}, {57.5, 25.9}, {63.9, 35.8}, {69.5, 47.4}, {74.4, 60.4}, {78.4, 73.8}, {80, 80}}, color = {0, 0, 0}), Polygon(points = {{90, 0}, {68, 8}, {68, -8}, {90, 0}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid)}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{0, 80}, {-8, 80}}, color = {192, 192, 192}), Line(points = {{0, -80}, {-8, -80}}, color = {192, 192, 192}), Line(points = {{0, -90}, {0, 84}}, color = {192, 192, 192}), Text(extent = {{9, 98}, {34, 78}}, lineColor = {160, 160, 164}, textString = "y"), Polygon(points = {{0, 100}, {-6, 84}, {6, 84}, {0, 100}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-100, 0}, {84, 0}}, color = {192, 192, 192}), Polygon(points = {{100, 0}, {84, 6}, {84, -6}, {100, 0}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-80, -80}, {-76, -65.4}, {-71.2, -51.4}, {-65.5, -38.8}, {-59.1, -28.1}, {-51.1, -18.7}, {-41.4, -11.4}, {-27.7, -5.5}, {-4.42, -0.653}, {24.5, 4.57}, {39, 10.1}, {49.4, 17.2}, {57.5, 25.9}, {63.9, 35.8}, {69.5, 47.4}, {74.4, 60.4}, {78.4, 73.8}, {80, 80}}, color = {0, 0, 0}), Text(extent = {{-31, 72}, {-11, 88}}, textString = "27", lineColor = {0, 0, 255}), Text(extent = {{-35, -88}, {-15, -72}}, textString = "-27", lineColor = {0, 0, 255}), Text(extent = {{70, 25}, {90, 5}}, textString = "4", lineColor = {0, 0, 255}), Text(extent = {{-98, 21}, {-78, 1}}, textString = "-4", lineColor = {0, 0, 255}), Text(extent = {{72, -6}, {96, -26}}, lineColor = {160, 160, 164}, textString = "u")}),
          Documentation(info = "<HTML>
<p>
This blocks computes the output <b>y</b> as the
<i>hyperbolic sine</i> of the input <b>u</b>:
</p>
<pre>
    y = <b>sinh</b>( u );
</pre>

<p>
<img src=\"modelica://Modelica/Resources/Images/Math/sinh.png\"
     alt=\"sinh.png\">
</p>

</html>"));
      end Sinh;

      block Cosh "Output the hyperbolic cosine of the input"
        extends Interfaces.ComplexSISO;
      equation
        y = Modelica.ComplexMath.cosh(u);
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{0, 90}, {-8, 68}, {8, 68}, {0, 90}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{0, -80}, {0, 68}}, color = {192, 192, 192}), Text(extent = {{4, 66}, {66, 20}}, lineColor = {192, 192, 192}, textString = "cosh"), Line(points = {{-80, 80}, {-77.6, 61.1}, {-74.4, 39.3}, {-71.2, 20.7}, {-67.1, 1.29}, {-63.1, -14.6}, {-58.3, -29.8}, {-52.7, -43.5}, {-46.2, -55.1}, {-39, -64.3}, {-30.2, -71.7}, {-18.9, -77.1}, {-4.42, -79.9}, {10.9, -79.1}, {23.7, -75.2}, {34.2, -68.7}, {42.2, -60.6}, {48.6, -51.2}, {54.3, -40}, {59.1, -27.5}, {63.1, -14.6}, {67.1, 1.29}, {71.2, 20.7}, {74.4, 39.3}, {77.6, 61.1}, {80, 80}}, color = {0, 0, 0}), Line(points = {{-90, -86.083}, {68, -86.083}}, color = {192, 192, 192}), Polygon(points = {{90, -86.083}, {68, -78.083}, {68, -94.083}, {90, -86.083}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid)}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{0, 80}, {-8, 80}}, color = {192, 192, 192}), Line(points = {{0, -80}, {-8, -80}}, color = {192, 192, 192}), Line(points = {{0, -90}, {0, 84}}, color = {192, 192, 192}), Text(extent = {{11, 98}, {38, 78}}, lineColor = {160, 160, 164}, textString = "y"), Polygon(points = {{0, 100}, {-6, 84}, {6, 84}, {0, 100}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-100, -86.083}, {84, -86.083}}, color = {192, 192, 192}), Polygon(points = {{100, -86.083}, {84, -80.083}, {84, -92.083}, {100, -86.083}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-80, 80}, {-77.6, 61.1}, {-74.4, 39.3}, {-71.2, 20.7}, {-67.1, 1.29}, {-63.1, -14.6}, {-58.3, -29.8}, {-52.7, -43.5}, {-46.2, -55.1}, {-39, -64.3}, {-30.2, -71.7}, {-18.9, -77.1}, {-4.42, -79.9}, {10.9, -79.1}, {23.7, -75.2}, {34.2, -68.7}, {42.2, -60.6}, {48.6, -51.2}, {54.3, -40}, {59.1, -27.5}, {63.1, -14.6}, {67.1, 1.29}, {71.2, 20.7}, {74.4, 39.3}, {77.6, 61.1}, {80, 80}}, color = {0, 0, 0}), Text(extent = {{-31, 72}, {-11, 88}}, textString = "27", lineColor = {0, 0, 255}), Text(extent = {{62, -67}, {82, -87}}, textString = "4", lineColor = {0, 0, 255}), Text(extent = {{-104, -63}, {-84, -83}}, textString = "-4", lineColor = {0, 0, 255}), Text(extent = {{78, -54}, {100, -74}}, lineColor = {160, 160, 164}, textString = "u")}),
          Documentation(info = "<HTML>
<p>
This blocks computes the output <b>y</b> as the
<i>hyperbolic cosine</i> of the input <b>u</b>:
</p>
<pre>
    y = <b>cosh</b>( u );
</pre>

<p>
<img src=\"modelica://Modelica/Resources/Images/Math/cosh.png\"
     alt=\"cosh.png\">
</p>

</html>"));
      end Cosh;

      block Tanh "Output the hyperbolic tangent of the input"
        extends Interfaces.ComplexSISO;
      equation
        y = Modelica.ComplexMath.tanh(u);
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{0, -90}, {0, 84}}, color = {192, 192, 192}), Line(points = {{-100, 0}, {84, 0}}, color = {192, 192, 192}), Line(points = {{-80, -80}, {-47.8, -78.7}, {-35.8, -75.7}, {-27.7, -70.6}, {-22.1, -64.2}, {-17.3, -55.9}, {-12.5, -44.3}, {-7.64, -29.2}, {-1.21, -4.82}, {6.83, 26.3}, {11.7, 42}, {16.5, 54.2}, {21.3, 63.1}, {26.9, 69.9}, {34.2, 75}, {45.4, 78.4}, {72, 79.9}, {80, 80}}, color = {0, 0, 0}), Polygon(points = {{0, 90}, {-8, 68}, {8, 68}, {0, 90}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Text(extent = {{-88, 72}, {-16, 24}}, lineColor = {192, 192, 192}, textString = "tanh"), Polygon(points = {{90, 0}, {68, 8}, {68, -8}, {90, 0}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid)}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{0, 80}, {-8, 80}}, color = {192, 192, 192}), Line(points = {{0, -80}, {-8, -80}}, color = {192, 192, 192}), Line(points = {{0, -90}, {0, 84}}, color = {192, 192, 192}), Text(extent = {{9, 98}, {42, 80}}, lineColor = {160, 160, 164}, textString = "y"), Polygon(points = {{0, 100}, {-6, 84}, {6, 84}, {0, 100}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-100, 0}, {84, 0}}, color = {192, 192, 192}), Polygon(points = {{100, 0}, {84, 6}, {84, -6}, {100, 0}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-80, -80}, {-47.8, -78.7}, {-35.8, -75.7}, {-27.7, -70.6}, {-22.1, -64.2}, {-17.3, -55.9}, {-12.5, -44.3}, {-7.64, -29.2}, {-1.21, -4.82}, {6.83, 26.3}, {11.7, 42}, {16.5, 54.2}, {21.3, 63.1}, {26.9, 69.9}, {34.2, 75}, {45.4, 78.4}, {72, 79.9}, {80, 80}}, color = {0, 0, 0}), Text(extent = {{70, 25}, {90, 5}}, textString = "4", lineColor = {0, 0, 255}), Text(extent = {{-106, 21}, {-86, 1}}, textString = "-4", lineColor = {0, 0, 255}), Text(extent = {{-29, 72}, {-9, 88}}, textString = "1", lineColor = {0, 0, 255}), Text(extent = {{3, -72}, {23, -88}}, textString = "-1", lineColor = {0, 0, 255}), Text(extent = {{72, -8}, {98, -26}}, lineColor = {160, 160, 164}, textString = "u")}),
          Documentation(info = "<HTML>
<p>
This blocks computes the output <b>y</b> as the
<i>hyperbolic tangent</i> of the input <b>u</b>:
</p>
<pre>
    y = <b>tanh</b>( u );
</pre>

<p>
<img src=\"modelica://Modelica/Resources/Images/Math/tanh.png\"
     alt=\"tanh.png\">
</p>

</html>"));
      end Tanh;

      block Exp "Output the exponential (base e) of the input"
        extends Interfaces.ComplexSISO;
      equation
        y = Modelica.ComplexMath.exp(u);
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{0, -80}, {0, 68}}, color = {192, 192, 192}), Polygon(points = {{0, 90}, {-8, 68}, {8, 68}, {0, 90}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Text(extent = {{-86, 50}, {-14, 2}}, lineColor = {192, 192, 192}, textString = "exp"), Line(points = {{-80, -80}, {-31, -77.9}, {-6.03, -74}, {10.9, -68.4}, {23.7, -61}, {34.2, -51.6}, {43, -40.3}, {50.3, -27.8}, {56.7, -13.5}, {62.3, 2.23}, {67.1, 18.6}, {72, 38.2}, {76, 57.6}, {80, 80}}, color = {0, 0, 0}), Line(points = {{-90, -80.3976}, {68, -80.3976}}, color = {192, 192, 192}), Polygon(points = {{90, -80.3976}, {68, -72.3976}, {68, -88.3976}, {90, -80.3976}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid)}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{0, 80}, {-8, 80}}, color = {192, 192, 192}), Line(points = {{0, -80}, {-8, -80}}, color = {192, 192, 192}), Line(points = {{0, -90}, {0, 84}}, color = {192, 192, 192}), Text(extent = {{9, 100}, {40, 80}}, lineColor = {160, 160, 164}, textString = "y"), Polygon(points = {{0, 100}, {-6, 84}, {6, 84}, {0, 100}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-100, -80.3976}, {84, -80.3976}}, color = {192, 192, 192}), Polygon(points = {{100, -80.3976}, {84, -74.3976}, {84, -86.3976}, {100, -80.3976}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-80, -80}, {-31, -77.9}, {-6.03, -74}, {10.9, -68.4}, {23.7, -61}, {34.2, -51.6}, {43, -40.3}, {50.3, -27.8}, {56.7, -13.5}, {62.3, 2.23}, {67.1, 18.6}, {72, 38.2}, {76, 57.6}, {80, 80}}, color = {0, 0, 0}), Text(extent = {{-31, 72}, {-11, 88}}, textString = "20", lineColor = {0, 0, 255}), Text(extent = {{-92, -83}, {-72, -103}}, textString = "-3", lineColor = {0, 0, 255}), Text(extent = {{70, -83}, {90, -103}}, textString = "3", lineColor = {0, 0, 255}), Text(extent = {{-18, -53}, {2, -73}}, textString = "1", lineColor = {0, 0, 255}), Text(extent = {{66, -52}, {96, -72}}, lineColor = {160, 160, 164}, textString = "u")}),
          Documentation(info = "<HTML>
<p>
This blocks computes the output <b>y</b> as the
<i>exponential</i> (of base e) of the input <b>u</b>:
</p>
<pre>
    y = <b>exp</b>( u );
</pre>

<p>
<img src=\"modelica://Modelica/Resources/Images/Math/exp.png\"
     alt=\"exp.png\">
</p>

</html>"));
      end Exp;

      block Log "Output the natural (base e) logarithm of the input (input > 0 required)"
        extends Interfaces.ComplexSISO;
      equation
        y = Modelica.ComplexMath.log(u);
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-80, -80}, {-80, 68}}, color = {192, 192, 192}), Polygon(points = {{-80, 90}, {-88, 68}, {-72, 68}, {-80, 90}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-80, -80}, {-79.2, -50.6}, {-78.4, -37}, {-77.6, -28}, {-76.8, -21.3}, {-75.2, -11.4}, {-72.8, -1.31}, {-69.5, 8.08}, {-64.7, 17.9}, {-57.5, 28}, {-47, 38.1}, {-31.8, 48.1}, {-10.1, 58}, {22.1, 68}, {68.7, 78.1}, {80, 80}}, color = {0, 0, 0}), Line(points = {{-90, 0}, {68, 0}}, color = {192, 192, 192}), Polygon(points = {{90, 0}, {68, 8}, {68, -8}, {90, 0}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Text(extent = {{-6, -24}, {66, -72}}, lineColor = {192, 192, 192}, textString = "log")}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-80, 80}, {-88, 80}}, color = {192, 192, 192}), Line(points = {{-80, -80}, {-88, -80}}, color = {192, 192, 192}), Line(points = {{-80, -90}, {-80, 84}}, color = {192, 192, 192}), Text(extent = {{-67, 98}, {-40, 80}}, lineColor = {160, 160, 164}, textString = "y"), Polygon(points = {{-80, 100}, {-86, 84}, {-74, 84}, {-80, 100}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-100, 0}, {84, 0}}, color = {192, 192, 192}), Polygon(points = {{100, 0}, {84, 6}, {84, -6}, {100, 0}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-80, -80}, {-79.2, -50.6}, {-78.4, -37}, {-77.6, -28}, {-76.8, -21.3}, {-75.2, -11.4}, {-72.8, -1.31}, {-69.5, 8.08}, {-64.7, 17.9}, {-57.5, 28}, {-47, 38.1}, {-31.8, 48.1}, {-10.1, 58}, {22.1, 68}, {68.7, 78.1}, {80, 80}}, color = {0, 0, 0}), Text(extent = {{-105, 72}, {-85, 88}}, textString = "3", lineColor = {0, 0, 255}), Text(extent = {{-109, -88}, {-89, -72}}, textString = "-3", lineColor = {0, 0, 255}), Text(extent = {{70, -3}, {90, -23}}, textString = "20", lineColor = {0, 0, 255}), Text(extent = {{-78, -1}, {-58, -21}}, textString = "1", lineColor = {0, 0, 255}), Text(extent = {{68, 28}, {94, 8}}, lineColor = {160, 160, 164}, textString = "u")}),
          Documentation(info = "<HTML>
<p>
This blocks computes the output <b>y</b> as the
<i>natural (base e) logarithm</i> of the input <b>u</b>:
</p>
<pre>
    y = <b>log</b>( u );
</pre>
<p>
An error occurs if the elements of the input <b>u</b> are
zero or negative.
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Math/log.png\"
     alt=\"log.png\">
</p>

</html>"));
      end Log;

      block RealToComplex "Converts Cartesian representation to complex"
        extends Modelica.ComplexBlocks.Interfaces.ComplexSO;
        Blocks.Interfaces.RealInput re annotation(
          Placement(transformation(extent = {{-130, 40}, {-90, 80}}), iconTransformation(extent = {{-140, 40}, {-100, 80}})));
        Blocks.Interfaces.RealInput im annotation(
          Placement(transformation(extent = {{-130, -80}, {-90, -40}}), iconTransformation(extent = {{-140, -80}, {-100, -40}})));
      equation
        y = Complex(re, im);
        annotation(
          Icon(graphics = {Text(extent = {{-100, 80}, {-20, 40}}, lineColor = {0, 0, 127}, fillColor = {85, 170, 255}, fillPattern = FillPattern.Solid, textString = "re"), Text(extent = {{-100, -40}, {-20, -80}}, lineColor = {0, 0, 127}, fillColor = {85, 170, 255}, fillPattern = FillPattern.Solid, textString = "im"), Polygon(points = {{20, 0}, {0, 20}, {0, 10}, {-30, 10}, {-30, -10}, {0, -10}, {0, -20}, {20, 0}}, lineColor = {0, 128, 255}, fillColor = {85, 170, 255}, fillPattern = FillPattern.Solid), Text(extent = {{20, 60}, {100, -60}}, lineColor = {0, 128, 255}, fillColor = {85, 170, 255}, fillPattern = FillPattern.Solid, textString = "C")}),
          Documentation(info = "<html>
<p>Converts the Real inputs <i>re</i> (real part) and <i>im</i> (imaginary part) to the Complex output <i>y</i>.</p>
</html>"));
      end RealToComplex;

      block PolarToComplex "Converts polar representation to complex"
        extends Modelica.ComplexBlocks.Interfaces.ComplexSO;
        Blocks.Interfaces.RealInput len annotation(
          Placement(transformation(extent = {{-130, 40}, {-90, 80}}), iconTransformation(extent = {{-140, 40}, {-100, 80}})));
        Blocks.Interfaces.RealInput phi annotation(
          Placement(transformation(extent = {{-130, -80}, {-90, -40}}), iconTransformation(extent = {{-140, -80}, {-100, -40}})));
      equation
        y = Complex(len * cos(phi), len * sin(phi));
        annotation(
          Icon(graphics = {Text(extent = {{-100, 80}, {-20, 40}}, lineColor = {0, 0, 127}, fillColor = {85, 170, 255}, fillPattern = FillPattern.Solid, textString = "len"), Text(extent = {{-100, -40}, {-20, -80}}, lineColor = {0, 0, 127}, fillColor = {85, 170, 255}, fillPattern = FillPattern.Solid, textString = "phi"), Polygon(points = {{20, 0}, {0, 20}, {0, 10}, {-30, 10}, {-30, -10}, {0, -10}, {0, -20}, {20, 0}}, lineColor = {0, 128, 255}, fillColor = {85, 170, 255}, fillPattern = FillPattern.Solid), Text(extent = {{20, 60}, {100, -60}}, lineColor = {0, 128, 255}, fillColor = {85, 170, 255}, fillPattern = FillPattern.Solid, textString = "C")}),
          Documentation(info = "<html>
<p>Converts the Real inputs <i>len</i> (length, absolute) and <i>phi</i> (angle, argument) to the Complex output <i>y</i>.</p>
</html>"));
      end PolarToComplex;

      block ComplexToReal "Converts complex to Cartesian representation"
        extends Modelica.Blocks.Icons.Block;
        Blocks.Interfaces.RealOutput re annotation(
          Placement(transformation(extent = {{100, 40}, {140, 80}}), iconTransformation(extent = {{100, 40}, {140, 80}})));
        Blocks.Interfaces.RealOutput im annotation(
          Placement(transformation(extent = {{100, -80}, {140, -40}}), iconTransformation(extent = {{100, -80}, {140, -40}})));
        Interfaces.ComplexInput u annotation(
          Placement(transformation(extent = {{-140, -20}, {-100, 20}})));
      equation
        re = u.re;
        im = u.im;
        annotation(
          Icon(graphics = {Text(extent = {{20, 80}, {100, 40}}, lineColor = {0, 0, 127}, fillColor = {85, 170, 255}, fillPattern = FillPattern.Solid, textString = "re"), Text(extent = {{20, -40}, {100, -80}}, lineColor = {0, 0, 127}, fillColor = {85, 170, 255}, fillPattern = FillPattern.Solid, textString = "im"), Polygon(points = {{40, 0}, {20, 20}, {20, 10}, {-10, 10}, {-10, -10}, {20, -10}, {20, -20}, {40, 0}}, lineColor = {0, 128, 255}, fillColor = {85, 170, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-100, 60}, {-20, -60}}, lineColor = {0, 128, 255}, fillColor = {85, 170, 255}, fillPattern = FillPattern.Solid, textString = "C")}),
          Documentation(info = "<html>
<p>Converts the Complex input <i>u</i> to the Real outputs <i>re</i> (real part) and <i>im</i> (imaginary part).</p>
</html>"));
      end ComplexToReal;

      block ComplexToPolar "Converts complex to polar representation"
        extends Modelica.Blocks.Icons.Block;
        Blocks.Interfaces.RealOutput len annotation(
          Placement(transformation(extent = {{100, 40}, {140, 80}}), iconTransformation(extent = {{100, 40}, {140, 80}})));
        Blocks.Interfaces.RealOutput phi annotation(
          Placement(transformation(extent = {{100, -80}, {140, -40}}), iconTransformation(extent = {{100, -80}, {140, -40}})));
        Interfaces.ComplexInput u annotation(
          Placement(transformation(extent = {{-140, -20}, {-100, 20}})));
      equation
        len = (u.re ^ 2 + u.im ^ 2) ^ 0.5;
        phi = Modelica.Math.atan2(u.im, u.re);
        annotation(
          Icon(graphics = {Text(extent = {{20, 80}, {100, 40}}, lineColor = {0, 0, 127}, fillColor = {85, 170, 255}, fillPattern = FillPattern.Solid, textString = "len"), Text(extent = {{20, -40}, {100, -80}}, lineColor = {0, 0, 127}, fillColor = {85, 170, 255}, fillPattern = FillPattern.Solid, textString = "phi"), Polygon(points = {{40, 0}, {20, 20}, {20, 10}, {-10, 10}, {-10, -10}, {20, -10}, {20, -20}, {40, 0}}, lineColor = {0, 128, 255}, fillColor = {85, 170, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-100, 60}, {-20, -60}}, lineColor = {0, 128, 255}, fillColor = {85, 170, 255}, fillPattern = FillPattern.Solid, textString = "C")}),
          Documentation(info = "<html>
<p>Converts the Complex input <i>u</i> to the Real outputs <i>len</i> (length, absolute) and <i>phi</i> (angle, argument).</p>
</html>"));
      end ComplexToPolar;
      annotation(
        Documentation(info = "<html>
<p>
This package contains basic <b>mathematical operations</b>,
such as summation and multiplication, and basic <b>mathematical
functions</b>, such as <b>sqrt</b> and <b>sin</b>, as
input/output blocks. All blocks of this library can be either
connected with continuous blocks or with sampled-data blocks.
</p>
</html>", revisions = "<html>
<ul>
<li><i>January 28, 2010</i>
       by <a href=\"http://www.haumer.at/\">Anton Haumer</a>
</li>
</ul>
</html>"),
        Icon(graphics = {Line(points = {{-80, -2}, {-68.7, 32.2}, {-61.5, 51.1}, {-55.1, 64.4}, {-49.4, 72.6}, {-43.8, 77.1}, {-38.2, 77.8}, {-32.6, 74.6}, {-26.9, 67.7}, {-21.3, 57.4}, {-14.9, 42.1}, {-6.83, 19.2}, {10.1, -32.8}, {17.3, -52.2}, {23.7, -66.2}, {29.3, -75.1}, {35, -80.4}, {40.6, -82}, {46.2, -79.6}, {51.9, -73.5}, {57.5, -63.9}, {63.9, -49.2}, {72, -26.8}, {80, -2}}, color = {95, 95, 95}, smooth = Smooth.Bezier)}));
    end ComplexMath;

    package Sources "Library of signal source blocks generating Complex signals"
      extends Modelica.Icons.SourcesPackage;

      block ComplexExpression "Set output signal to a time varying Complex expression"
        Modelica.ComplexBlocks.Interfaces.ComplexOutput y = Complex(0) "Value of Complex output" annotation(
          Dialog(group = "Time varying output signal"),
          Placement(transformation(extent = {{100, -10}, {120, 10}}, rotation = 0)));
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, 40}, {100, -40}}, lineColor = {0, 0, 0}, lineThickness = 5.0, fillColor = {235, 235, 235}, fillPattern = FillPattern.Solid, borderPattern = BorderPattern.Raised), Text(extent = {{-96, 15}, {96, -15}}, lineColor = {0, 0, 0}, textString = "%y"), Text(extent = {{-150, 90}, {140, 50}}, textString = "%name", lineColor = {0, 0, 255})}),
          Documentation(info = "<html>
<p>
The (time varying) Complex output signal of this block can be defined in its
parameter menu via variable <b>y</b>. The purpose is to support the
easy definition of Complex expressions in a block diagram.
Note, that \"time\" is a built-in variable that is always
accessible and represents the \"model time\" and that
Variable <b>y</b> is both a variable and a connector.
</p>
</html>"));
      end ComplexExpression;

      block ComplexConstant "Generate constant signal of type Complex"
        parameter Complex k(re(start = 1), im(start = 0)) "Constant output value";
        extends Modelica.ComplexBlocks.Interfaces.ComplexSO;
      equation
        y = k;
        annotation(
          defaultComponentName = "const",
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-80, 68}, {-80, -80}}, color = {192, 192, 192}), Polygon(points = {{-80, 90}, {-88, 68}, {-72, 68}, {-80, 90}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-90, -70}, {82, -70}}, color = {192, 192, 192}), Polygon(points = {{90, -70}, {68, -62}, {68, -78}, {90, -70}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-80, 0}, {80, 0}}, color = {0, 0, 0}), Text(extent = {{-150, -150}, {150, -110}}, lineColor = {0, 0, 0}, textString = "k=%k")}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{-80, 90}, {-86, 68}, {-74, 68}, {-80, 90}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Line(points = {{-80, 68}, {-80, -80}}, color = {95, 95, 95}), Line(points = {{-80, 0}, {80, 0}}, color = {0, 0, 255}, thickness = 0.5), Line(points = {{-90, -70}, {82, -70}}, color = {95, 95, 95}), Polygon(points = {{90, -70}, {68, -64}, {68, -76}, {90, -70}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Text(extent = {{-83, 92}, {-30, 74}}, lineColor = {0, 0, 0}, textString = "y"), Text(extent = {{70, -80}, {94, -100}}, lineColor = {0, 0, 0}, textString = "time"), Text(extent = {{-101, 8}, {-81, -12}}, lineColor = {0, 0, 0}, textString = "k")}),
          Documentation(info = "<html>
<p>
The Complex output y is a constant signal:
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Blocks/Sources/Constant.png\"
     alt=\"Constant.png\">
</p>
</html>"));
      end ComplexConstant;

      block ComplexStep "Generate step signal of type Complex"
        parameter Complex height = Complex(1) "Height of step";
        extends ComplexBlocks.Interfaces.ComplexSignalSource;
      equation
        y = offset + (if time < startTime then Complex(0) else height);
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-80, 68}, {-80, -80}}, color = {192, 192, 192}), Polygon(points = {{-80, 90}, {-88, 68}, {-72, 68}, {-80, 90}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-90, -70}, {82, -70}}, color = {192, 192, 192}), Polygon(points = {{90, -70}, {68, -62}, {68, -78}, {90, -70}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-80, -70}, {0, -70}, {0, 50}, {80, 50}}, color = {0, 0, 0}), Text(extent = {{-150, -150}, {150, -110}}, lineColor = {0, 0, 0}, textString = "startTime=%startTime")}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{-80, 90}, {-86, 68}, {-74, 68}, {-80, 90}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Line(points = {{-80, 68}, {-80, -80}}, color = {95, 95, 95}), Line(points = {{-80, -18}, {0, -18}, {0, 50}, {80, 50}}, color = {0, 0, 255}, thickness = 0.5), Line(points = {{-90, -70}, {82, -70}}, color = {95, 95, 95}), Polygon(points = {{90, -70}, {68, -64}, {68, -76}, {90, -70}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Text(extent = {{70, -80}, {94, -100}}, lineColor = {0, 0, 0}, textString = "time"), Text(extent = {{-21, -72}, {25, -90}}, lineColor = {0, 0, 0}, textString = "startTime"), Line(points = {{0, -17}, {0, -71}}, color = {95, 95, 95}), Text(extent = {{-68, -36}, {-22, -54}}, lineColor = {0, 0, 0}, textString = "offset"), Line(points = {{-13, 50}, {-13, -17}}, color = {95, 95, 95}), Polygon(points = {{2, 50}, {-19, 50}, {2, 50}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Polygon(points = {{-13, -17}, {-16, -4}, {-10, -4}, {-13, -17}, {-13, -17}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Polygon(points = {{-13, 50}, {-16, 37}, {-9, 37}, {-13, 50}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Text(extent = {{-68, 26}, {-22, 8}}, lineColor = {0, 0, 0}, textString = "height"), Polygon(points = {{-13, -69}, {-16, -56}, {-10, -56}, {-13, -69}, {-13, -69}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Line(points = {{-13, -18}, {-13, -70}}, color = {95, 95, 95}), Polygon(points = {{-13, -18}, {-16, -31}, {-9, -31}, {-13, -18}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Text(extent = {{-72, 100}, {-31, 80}}, lineColor = {0, 0, 0}, textString = "y")}),
          Documentation(info = "<html>
<p>
The Complex output y is a step signal (of real and imaginary part):
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Blocks/Sources/Step.png\"
     alt=\"Step.png\">
</p>

</html>"));
      end ComplexStep;

      block ComplexRotatingPhasor "Generate a phasor with constant magnitude and constant angular velocity of type Complex"
        parameter Real magnitude = 1 "Magnitude of complex phasor";
        parameter Modelica.SIunits.AngularVelocity w = 1 "Constant angular velocity of complex phasor";
        parameter Modelica.SIunits.Angle phi0 = 0 "Initial angle of complex phasor at time = 0";
        extends Modelica.ComplexBlocks.Interfaces.ComplexSO;
      equation
        y = magnitude * Modelica.ComplexMath.exp(Complex(0, w * time + phi0));
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{-10, 90}, {-16, 68}, {-4, 68}, {-10, 90}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Line(points = {{-10, 68}, {-10, -90}}, color = {95, 95, 95}), Line(points = {{-90, 0}, {82, 0}}, color = {95, 95, 95}), Polygon(points = {{90, 0}, {68, 6}, {68, -6}, {90, 0}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Line(points = {{-10, 0}, {50, 50}}, color = {0, 0, 255}, smooth = Smooth.None, thickness = 0.5), Polygon(points = {{50, 50}, {29, 41}, {38, 30}, {50, 50}}, lineColor = {95, 95, 95}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid), Line(points = {{-47, 35}, {-40, 40}, {-32, 44}, {-20, 47}, {-7, 46}, {5, 42}, {18, 34}, {26, 23}, {31, 10}}, color = {0, 0, 255}), Polygon(points = {{-52, 29}, {-32, 36}, {-42, 47}, {-52, 29}}, lineColor = {0, 0, 255}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid)}),
          Documentation(info = "<html>
<p>
The output y is a complex phasor with constant magnitude, spinning with constant angular velocity.
</p>

</html>"));
      end ComplexRotatingPhasor;
    end Sources;
    annotation(
      Documentation(info = "<html>
<p>This library hosts blocks using Complex inputs and outputs.</p>
<p>It depends on the implementation of Complex.</p>
</html>"),
      Icon(coordinateSystem(extent = {{-100.0, -100.0}, {100.0, 100.0}}, preserveAspectRatio = true), graphics = {Rectangle(origin = {0.0, 35.1488}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, extent = {{-30.0, -20.1488}, {30.0, 20.1488}}), Rectangle(origin = {0.0, -34.8512}, fillColor = {128, 128, 128}, fillPattern = FillPattern.Solid, extent = {{-30.0, -20.1488}, {30.0, 20.1488}}), Line(origin = {-51.25, 0.0}, points = {{21.25, -35.0}, {-13.75, -35.0}, {-13.75, 35.0}, {6.25, 35.0}}), Polygon(origin = {-40.0, 35.0}, pattern = LinePattern.None, fillPattern = FillPattern.Solid, points = {{10.0, 0.0}, {-5.0, 5.0}, {-5.0, -5.0}}), Line(origin = {51.25, 0.0}, points = {{-21.25, 35.0}, {13.75, 35.0}, {13.75, -35.0}, {-6.25, -35.0}}), Polygon(origin = {40.0, -35.0}, pattern = LinePattern.None, fillPattern = FillPattern.Solid, points = {{-10.0, 0.0}, {5.0, 5.0}, {5.0, -5.0}})}));
  end ComplexBlocks;

  package StateGraph "Library of hierarchical state machine components to model discrete event and reactive systems"
    extends Modelica.Icons.Package;

    package UsersGuide "User's Guide of StateGraph Library"
      extends Modelica.Icons.Information;

      class OverView "Overview of library"
        extends Modelica.Icons.Information;
        annotation(
          Documentation(info = "<html>
<p>
In this section, an overview of the most important features
of this library is given.
</p>
<h4>Steps and Transitions</h4>
<p>
A <b>StateGraph</b> is an enhanced finite state machine.
It is based on the JGrafchart method and
takes advantage of Modelica features for
the \"action\" language. JGrafchart is a further development of
Grafcet to include elements of StateCharts that are not present
in Grafcet/Sequential Function Charts. Therefore, the StateGraph
library has a similar modeling power as StateCharts but avoids
some deficiencies of StateCharts.
</p>
<p>
The basic elements of StateGraphs are <b>steps</b> and <b>transitions</b>:
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/StateGraph/UsersGuide/StepAndTransition1.png\">
</p>

<p>
<b>Steps</b> represent the possible states a StateGraph can have.
If a step is active the Boolean variable <b>active</b> of
the step is <b>true</b>. If it is deactivated,
<b>active</b> = <b>false</b>. At the initial time, all \"usual\"
steps are deactivated. The <b>InitialStep</b> objects are steps
that are activated at the initial time. They are characterized
by a double box (see figure above).
</p>
<p>
<b>Transitions</b> are used to change the state of a StateGraph.
When the step connected to the input of a transition is active,
the step connected to the output of this transition is deactivated
and the transition condition becomes true, then the
transition fires. This means that the step connected to the input to the
transition is deactivated and the step connected to the output
of the transition is activated.
</p>
<p>
The transition <b>condition</b> is defined via the parameter menu
of the transition object. Clicking on object \"transition1\" in
the above figure, results in the following menu:
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/StateGraph/UsersGuide/StepAndTransition2.png\">
</p>

<p>
In the input field \"<b>condition</b>\", any type of time varying
Boolean expression can be given (in Modelica notation, this is
a modification of the time varying variable <b>condition</b>).
Whenever this condition is true, the transition can fire.
Additionally, it is possible to activate a timer, via
<b>enableTimer</b> (see menu above) and provide a
<b>waitTime</b>. In this case the firing of the transition
is delayed according to the provided waitTime. The transition
condition and the waitTime are displayed in the transition icon.
</p>
<p>
In the above example, the simulation starts at <b>initialStep</b>.
After 1 second, <b>transition1</b> fires and <b>step1</b> becomes
active. After another second <b>transition2</b> fires and
<b>initialStep</b> becomes again active. After a further
second <b>step1</b> becomes again active, and so on.
</p>
<p>
In JGrafcharts, Grafcet and Sequential Function Charts, the
network of steps and transitions is drawn from top to bottom.
In StateGraphs, no particular direction is defined, since
steps and transitions are blocks with input and output connectors
that can be arbitrarily placed and connected. Usually, it is
most practical to define the network from left to right,
as in the example above, since then it is easy to read the
labels on the icons.
</p>
<h4>Conditions and Actions</h4>
<p>
With the block <b>TransitionWithSignal</b>, the firing condition
can be provided as Boolean input signal, instead as entry in the
menu of the transition. An example is given in the next
figure:
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/StateGraph/UsersGuide/StepAndTransition3.png\">
</p>

<p>
Component \"step\" is an instance of \"StepWithSignal\" that is
a usual step where the active flag is available as Boolean
output signal. To this output, component \"Timer\" from
library \"Modelica.Blocks.Logical\" is connected. It measures the
time from the time instant where the Boolean input (i.e., the
active flag of the step) became true up to the current
time instant. The timer is connected to a comparison
component. The output is true, once the timer reaches
1 second. This signal is used as condition input of the
transition. As a result, \"transition2\" fires, once step
\"step\" has been active for 1 second.
Of course, any other
Modelica block with a Boolean output signal can be
connected to the condition input of such a transition block
as well.
</p>
<p>
Conditions of a transition can either be computed by
a network of logical blocks from the Logical library as
in the figure above, or via the \"SetBoolean\" component
any type of logical expression can be defined in textual
form, as shown in the next figure:
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/StateGraph/UsersGuide/StepAndTransition4.png\">
</p>

<p>
With the block \"<b>SetBoolean</b>\", a time varying expression
can be provided as modification to the output signal <b>y</b>
(see block with icon text \"timer.y > 1\" in the figure above).
The output signal can be in turn connected to the condition
input of a TransitionWithSignal block.
</p>
<p>
The \"<b>SetBoolean</b>\" block can also be used to
compute a Boolean signal depending on the active step.
In the figure above, the output of the block with the
icon text \"step.active\" is
true, when \"step\" is active, otherwise it is false
(note, the icon text of \"SetBoolean\" displays the modification
of the output signal \"y\").
This signal can then be used to compute desired <b>actions</b>
in the physical systems model. For example, if a <b>valve</b>
shall be open, when the StateGraph is in \"step1\" or
in \"step4\", a \"SetBoolean\" block may be connected to the
valve model using the following condition:
</p>
<pre>
    valve = step1.active <b>or</b> step2.active
</pre>
<p>
Via the Modelica operators <b>edge</b>(..) and <b>change</b>(..),
conditions depending on rising and falling edges of
Boolean expressions can be used when needed.
</p>
<p>
In JGrafcharts, Grafcet, Sequential Function Charts and StateCharts,
<b>actions</b> are formulated <b>within a step</b>. Such actions are
distinguished as <b>entry</b>, <b>normal</b>, <b>exit</b> and
<b>abort</b> actions. For example, a valve might be opened by
an entry action of a step and might be closed by an exit
action of the same step. In StateGraphs, this is (fortunately)
<b>not possible</b>
due to Modelica's \"single assignment rule\" that requires that every
variable is defined by exactly one equation. Instead, the
approach explained above is used. For example, via the
\"SetBoolean\" component, the valve variable is set to true
when the StateGraph is in particular steps.
</p>
<p>
This feature of a StateGraph is <b>very useful</b>, since it allows
a Modelica translator to <b>guarantee</b> that a given StateGraph
has always <b>deterministic</b> behaviour without conflicts.
In the other methodologies this is much more cumbersome. For example,
if two steps are executed in parallel and both step actions
modify the same variable, the result is either non-deterministic
or non-obvious rules have to be defined which action
takes priority. In a StateGraph, such a situation is detected by
the translator resulting in an error, since there are two equations
to compute one variable. Additional benefits of the StateGraph
approach are:
</p>
<ul>
<li> A JGrafchart or a StateChart need to potentially access
     variables in a step that are defined in
     higher hierarchical levels of a model. Therefore, mostly <b>global
     variables</b> are used in the whole network, even if the
     network is structured hierarchically. In StateGraphs this
     is not necessary, since the physical systems outside
     of a StateGraph might access the step or transition state
     via a hierarchical name. This means that <b>no global variables</b>
     are needed, because the local variables in the StateGraph
     are accessed from local variables outside of the StateGraph.
     </li>
<li> It is simpler for a user to understand the information that
     is provided in the non-graphical definition, since every
     variable is defined at exactly one place. In the other
     methodologies, the setting and re-setting of the same
     variable is cluttered within the whole network.
    </li>
</ul>
<p>
To emphasize this important difference between these methodologies,
consider the case that a state machine has the following
hierarchy:
</p>
<pre>
   stateMachine.superstate1.superstate2.step1
</pre>
<p>
Within \"step1\" a StateChart would, e.g., access variable
\"stateMachine.openValve\", say as \"entry action: openValve = true\".
This typical usage has the severe drawback that it is not possible
to use the hierarchical state \"superstate1\" as component in another
context, because \"step1\" references a particular name outside of this
component.
</p>
<p>
In a StateGraph, there would be typically a \"SetBoolean\" component
in the \"stateMachine\" component stating:
</p>
<pre>
    openValve = superstate1.superstate2.step1.active;
</pre>
<p>
As a result, the \"superstate1\" component can be used in
another context, because it does not depend on the environment
where it is used.
</p>
<h4>Execution Model</h4>
<p>
The execution model of a StateGraph follows from its
Modelica implementation: Given the states of all steps, i.e.,
whether a step is active or not active, the equations of all
steps, transitions, transition conditions, actions etc. are
sorted resulting in an execution sequence to compute
essentially the new values of the steps. If conflicts occur,
e.g., if there are more equations as variables, of if there
are algebraic loops between Boolean variables, an exception
is raised. Once all equations have been processed, the
<b>active</b> variable of all steps are updated to the newly
calculated values. Afterwards, the equations are again
evaluated. The iteration stops, once no step changes
its state anymore, i.e., once no transition fires anymore.
Then, simulation continuous until a new event is triggered,
(when a relation changes its value).
</p>
<p>
With the Modelica \"sampled(..)\" operator, a StateGraph might also
be executed within a discrete controller that is called
at regular time instants. In a future version of the StateGraph
library, this might be more directly supported.
</p>
<h4>Parallel and Alternative Execution</h4>
<p>
Parallel activities can be defined by
component <b>Parallel</b> and alternative activities
can be defined by component <b>Alternative</b>.
An example for both components is given in the next figure.
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/StateGraph/UsersGuide/Parallel1.png\">
</p>

<p>
Here, the branch from \"step2\" to \"step5\" is executed in parallel
to \"step1\". A transition connected to the output of a parallel
branch component can only fire if the final steps
in all parallel branches are active simultaneously.
The figure above is a screen-shot from the animation of the
StateGraph: Whenever a step is active or a transition can fire,
the corresponding component is marked in <b>green</b> color.
</p>
<p>
The three branches within \"step2\" to \"step5\" are
executed alternatively, depending which transition condition
of \"transition3\", \"transition4\", \"transition4a\" fires first.
Since all three transitions fire after 1 second, they are all
candidates for the active branch. If two or more transitions
would fire at the same time instant, a priority selection
is made: The alternative and parallel components have a
vector of connectors. Every branch has to be connected to
exactly one entry of the connector vector. The entry with
the lowest number has the highest priority.
</p>
<p>
Parallel, Alternative and Step components have vectors of
connectors. The dimensions of these vectors are set in the
corresponding parameter menu. E.g. in a \"Parallel\" component:
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/StateGraph/UsersGuide/Parallel2.png\">
</p>

<p>
Currently in Dymola the following menu pops up, when a branch
is connected to a vector of components in order to define
the vector index to which the component shall be connected:
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/StateGraph/UsersGuide/Parallel3.png\">
</p>

<h4>CompositeSteps, Suspend and Resume Port</h4>
<p>
A StateGraph can be hierarchically structured by using a <b>CompositeStep</b>.
This is a component that inherits from <b>PartialCompositeStep</b>.
An example is given in the next figure (from Examples.ControlledTanks):
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/StateGraph/UsersGuide/CompositeStep1.png\">
</p>

<p>
The CompositeStep component contains a local StateGraph that is
entered by one or more input transitions and that is left
by one or more output transitions. Also, other needed signals
may enter a CompositeStep. The CompositeStep has similar properties
as a \"usual\" step: The CompositeStep is <b>active</b> once at least
one step within the CompositeStep is active. Variable <b>active</b>
defines the state of the CompositeStep.
</p>
<p>
Additionally, a CompositeStep has a <b>suspend</b> port. Whenever the
transition connected to this port fires, the CompositeStep is left
at once. When leaving the CompositeStep via the suspend port, the internal
state of the CompositeStep is saved, i.e., the active flags of all
steps within the CompositeStep. The CompositeStep might be entered via
its <b>resume</b> port. In this case the internal state from the
suspend transition is reconstructed and the CompositeStep continues
the execution that it had before the suspend transition fired
(this corresponds to the history ports of StateCharts or JGrafcharts).
</p>
<p>
A CompositeStep may contain other CompositeSteps. At every level,
a CompositeStep and all of its content can be left via its suspend ports
(actually, there
is a vector of suspend connectors, i.e., a CompositeStep might
be aborted due to different transitions).
</p>
</html>"));
      end OverView;

      class FirstExample "A first example"
        extends Modelica.Icons.Information;
        annotation(
          Documentation(info = "<html>
<p>
A first example will be given here (not yet done).
</p>
</html>"));
      end FirstExample;

      class ApplicationExample "An application example"
        extends Modelica.Icons.Information;
        annotation(
          Documentation(info = "<html>
<p>
In this section a more realistic, still simple, application example
is given, to demonstrate various features of the StateGraph library.
This example shows the control of a two tank system from the master thesis
of Isolde Dressler
(<a href=\"modelica://Modelica.StateGraph.UsersGuide.Literature\">see literature</a>).
</p>
<p>
In the following figure the top level of the model is shown.
This model is available as StateGraph.Examples.ControlledTanks.
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/StateGraph/Examples/ControlledTanks1.png\">
</p>

<p>
In the right part of the figure, two tanks are shown. At the top part,
a large fluid source is present from which fluid can be filled in
<b>tank1</b> when <b>valve1</b> is open. Tank1 can be emptied via
<b>valve2</b> that is located in the bottom of tank2 and
fills a second <b>tank2</b> which in turn is emptied via
<b>valve3</b>. The actual levels of the tanks are measured
and are provided as signals <b>level1</b> and <b>level2</b>
to the <b>tankController</b>.
</p>
<p>
The <b>tankController</b> is controlled by three buttons,
<b>start</b>, <b>stop</b> and <b>shut</b> (for shutdown)
that are mutually exclusive. This means that whenever one button is
pressed (i.e., its state is <b>true</b>) then the other two
buttons are not pressed (i.e., their states are <b>false</b>).
When button <b>start</b> is pressed, the \"normal\" operation
to fill and to empty the two tanks is processed:
</p>
<ol>
<li> Valve 1 is opened and tank 1 is filled.</li>
<li> When tank 1 reaches its fill level limit,
     valve 1 is closed. </li>
<li> After a waiting time, valve 2 is
     opened and the fluid flows from tank 1 into tank 2.</li>
<li> When tank 1 is empty, valve 2 is closed. </li>
<li> After a waiting time, valve 3 is opened and
     the fluid flows out of tank 2</li>
<li> When tank 2 is empty, valve 3 is closed</li>
</ol>
<p>
The above \"normal\" process can be influenced by the following
buttons:
</p>
<ul>
<li> Button <b>start</b> starts the above process.
     When this button is pressed after a \"stop\" or
     \"shut\" operation, the process operation continues.
     </li>
<li> Button <b>stop</b> stops the above process by
     closing all valves. Then, the controller waits for
     further input (either \"start\" or \"shut\" operation).</li>
<li> Button <b>shut</b> is used to shutdown the process,
     by emptying at once both tanks. When this is achieved,
     the process goes back to its start configuration.
     Clicking on \"start\", restarts the process.</li>
</ul>
<p>
The implementation of the <b>tankController</b> is shown in
the next figure:
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/StateGraph/Examples/ControlledTanks2.png\">
</p>

<p>
When the \"<b>start</b>\" button is pressed, the stateGraph is
within the CompositeStep \"<b>makeProduct</b>\". During normal
operation this CompositeStep is only left, once tank2 is empty.
Afterwards, the CompositeStep is at once re-entered.
</p>
<p>
When the \"<b>stop</b>\" button is pressed, the \"makeProduct\"
CompositeStep is at once terminated via the \"<b>suspend</b>\" port
and the stateGraph waits in step \"<b>s2</b>\" for further
commands. When the \"<b>start</b>\" button is pressed, the CompositeStep
is re-entered via its <b>resume</b> port and the \"normal\"
operation continues at the state where it was aborted by the
suspend transition. If the \"<b>shut</b>\" button is pressed,
the stateGraph waits in the \"<b>emptyTanks</b>\" step, until
both tanks are empty and then waits at the initial step
\"<b>s1</b>\" for further input.
</p>
<p>
The opening and closing of valves is <b>not</b> directly
defined in the stateGraph. Instead via the \"<b>setValveX</b>\"
components, the Boolean state of the valves are computed.
For example, the output y of \"setValve2\" is computed as:
</p>
<pre>
  y = makeProduct.fillTank2.active or emptyTanks.active
</pre>
<p>
i.e., valve2 is open, when step \"makeProduct.fillTank2 or when
step \"emptyTanks\" is active. Otherwise, valve2 is closed.
</p>
</html>"));
      end ApplicationExample;

      class ComparisonWithStateGraph2 "Comparison with StateGraph2"
        extends Modelica.Icons.Information;
        annotation(
          Documentation(info = "<html>
<p>
There is a much improved library available called \"Modelica_StateGraph2\".
If this library is not yet distributed with your Modelica tool, you can download
it from <a href=\"https://www.modelica.org/libraries/Modelica_StateGraph2\">https://www.modelica.org/libraries/Modelica_StateGraph2</a>.
Find below a comparison with respect to Modelica.StateGraph.
It is not yet clear whether Modelica_StateGraph2 will be included in a
future version of the Modelica package. Another option is to provide
built-in support for state machines in a future Modelica language version
which would allow an even more powerful treatment of state machines in Modelica.
</p>

<p>
The Modelica_StateGraph2 library (called <u>StateGraph2</u> below)
is based on the experience with the current
Modelica.StateGraph library (called <u>StateGraph1</u> below) and is
a significantly further development of StateGraph1. Furthermore, it is heavily
based on the article (Malmheden et. al. 2008), see Literature below,
but uses a different implementation
technique as described in this article. The StateGraph2
library has the following improvements with respect to the StateGraph1
library:
</p>

<ul>
<li> <b>3 Basic Components (Step, Transition, Parallel)</b><br>
     All multiple versions of a component are
     combined in only one version (e.g., one step and not 4 step components).
     This is easier to understand and more convenient to use.
     The \"Parallel\" component is both used as \"composite step\" (so only one branch),
     as well as \"parallel step\" (so several execution branches).<br>&nbsp;</li>

<li> <b>Conveniently connecting components</b><br>
     Connecting components of a state machine in Modelica means to provide
     new vector dimensions and to provide a vector index to connect to.
     In StateGraph2, the new \"connectorSizing\" annotation is used and
     therefore all this is now performed automatically (from a users point
     of view, these actions are hidden; this is not the case in
     StateGraph1 and makes the usage of the StateGraph1 library clumsy).<br>&nbsp;</li>

<li> <b>Safer state machines</b><br>
     It is no longer possible to construct a wrong state machine in the sense that properties
     of the graph are violated (e.g., two initial steps, or branching wrongly out of a parallel
     component). Contrary to StateGraph2, in StateGraph1 such wrong graphs do not lead to an
     error but to unexpected simulation results. Still, other desirable properties
     of a state machine,  such as \"no deadlock\" or \"lifeliness\" or \"every step reachable\",
     are not (yet) guaranteed with the current StateGraph2. <br>&nbsp;</li>

<li> <b>Composite, autonomous, synchronized, preempted subgraphs</b><br>
     Composite steps and parallel steps are described in a much better and more powerful
     way as in StateGraph1: Either by component \"Parallel\" or
     by inheriting from \"PartialParallel\". The first alternative has the advantage that it
     is simple to use (not necessary to construct a new class and instantiating this class, and
     easy variable access since no new hierarchy is constructed), the second alternative
     has the advantage that it introduces a Modelica hierarchy (useful for large subgraphs).
     In both cases, various options are possible, such as
     <ol>
     <li> autonomous subgraphs (branches are executed in parallel autonomously),  </li>
     <li> synchronized subgraphs (branches are executed in parallel and are synchronized
           before leaving the subgraph via the outPort), </li>
     <li> subgraphs with preemption and exception (a parallel step can be interrupted via
          the suspend ports and can continue execution via the resume ports).</li>
     </ol>
     This is achieved by enabling/disabling the different ports.<br>&nbsp;</li>
<li> <b>No infinite looping</b>:<br>
     As in StateGraph1, there are two types of transitions: immediate transitions (during event
     iteration all immediate transitions fire until no transition condition is true anymore) and
     delayed transitions (a transition fires only after a delay). Contrary to StateGraph1,
     in StateGraph2 every loop must have at least one delayed transition. If this is not the case
     a translation error occurs which states that the model contains an algebraic loop
     between Booleans with the name \"checkOneDelayedTransitionPerLoop\".<br>
     &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;This property guarantees that an event
     iteration over a StateGraph2 converges after a
     finite number of iterations, provided the modeller does not introduce an unsafe construct
     in the actions associated with a StateGraph2 (e.g., \"i = pre(i) + 1\" in the equation section
     outside of a when-clause will give an event iteration that never stops).<br>
     &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;It is possible to switch off this feature, by setting parameter
     \"loopCheck = <b>false</b>\" in one transition of a loop, instead of using a \"delayed transition\"
     at this place (in cases where immediate transitions are
     important and the transition conditions are in a form that they cannot fire at the
     same time instant).</li>
</ul>

<h4> Literature </h4>

<p>
The Modelica_StateGraph2 library is described in detail in
(Otter et. al. 2009, see below) and is additionally
based on the following references:
</p>

<dl>
<dt>Andr&eacute;, C. (2003):</dt>
<dd><a href=\"http://www.i3s.unice.fr/~map/WEBSPORTS/Documents/2003a2005/SSMsemantics.pdf\">
     Semantics of S.S.M (Safe State Machine).</a>.
     I3S Laboratory, UMR 6070 University of Nice-Sophia Antipolis / CNRS.<br>&nbsp;</dd>

<dt>Arzen K.-E. (2004):</dt>
<dd> <b>JGrafchart User Manual. Version 1.5</b>.
     Department of Automatic Control, Lund Institute of Technology,
     Lund, Sweden, Feb. 13, 2004.<br>&nbsp;</dd>

<dt>Dressler I. (2004):</dt>
<dd> <a href=\"http://www.control.lth.se/documents/2004/5726.pdf\">
     Code Generation From JGrafchart to Modelica.</a>.
     Master thesis, supervisor: Karl-Erik Arzen,
     Department of Automatic Control, Lund Institute of Technology,
     Lund, Sweden, March 30, 2004.<br>&nbsp;</dd>

<dt>Elmqvist H., Mattsson S.E., Otter M. (2001):</dt>
<dd> <b>Object-Oriented and Hybrid Modeling in Modelica</b>.
     Journal Europeen des systemes automatises (JESA),
     Volume 35 - n. 1, 2001.<br>&nbsp;</dd>

<dt>Harel, D. (1987):</dt>
<dd> <a href=\"http://www.inf.ed.ac.uk/teaching/courses/seoc1/2005_2006/resources/statecharts.pdf\">
      A Visual Formalism for Complex Systems</a>.
      Science of Computer Programming 8, 231-274. Department of Applied Mathematics,
      The Weizmann Institute of Science, Rehovot, Israel.<br>&nbsp;</dd>

<dt>Malmheden M. (2007):</dt>
<dd> <a href=\"http://www.control.lth.se/documents/2007/5808.pdf\">
      ModeGraph - A Mode-Automata-Based Modelica Library for Embedded Control</a>.
      Master thesis, Department of Automatic Control, Lund University, Sweden.<br>&nbsp;
     </dd>

<dt>Malmheden M., Elmqvist H., Mattsson S.E., Henrisson D., Otter M. (2008):</dt>
<dd> <a href=\"https://www.modelica.org/events/modelica2008/Proceedings/sessions/session3a3.pdf\">
      ModeGraph - A Modelica Library for Embedded Control based on Mode-Automata</a>.
      Modelica'2008 Conference, March 3-4, 2008.<br>&nbsp;
     </dd>

<dt>Maraninchi F., R&eacute;mond, Y. (2002):</dt>
<dd> <a href=\"http://dx.doi.org/10.1016/S0167-6423(02)00093-X\">Mode-Automata:
     A New Domain-Specific Construct for the Development of Safe Critical Systems</a>.<br>&nbsp;
     </dd>

<dt>Mosterman P., Otter M., Elmqvist H. (1998):</dt>
<dd><a href=\"https://www.modelica.org/publications/papers/scsc98fp.pdf\">
     Modeling Petri Nets as Local Constraint Equations for
     Hybrid Systems using Modelica</a>.
     SCSC'98, Reno, Nevada, USA,
     Society for Computer Simulation International, pp. 314-319, 1998.<br>&nbsp;
     </dd>

<dt>Otter M., Malmheden M., Elmqvist H., Mattsson S.E., Johnsson C. (2009):</dt>
<dd> <a href=\"https://www.modelica.org/events/modelica2009\">
     A New Formalism for Modeling of Reactive and Hybrid Systems</a>.
      Modelica'2009 Conference, Como, Italy, Sept. 20-22, 2009.
     </dd>
</dl>

</html>"));
      end ComparisonWithStateGraph2;

      class ReleaseNotes "Release notes"
        extends Modelica.Icons.ReleaseNotes;
        annotation(
          Documentation(info = "<html>
<h4>Version 0.87, 2004-06-23</h4>
<ul>
<li> Included in Modelica standard library 2.0 Beta 1 with the new block connectors.
     Changed all the references to the block connectors and the Logical library
     correspondingly.</li>
</ul>
<h4>Version 0.86, 2004-06-20</h4>
<ul>
<li> New components \"Alternative\" and \"Parallel\" for alternative and
     parallel execution paths.</li>
<li> A step has now a vector of input and output connectors in order
     that multiple connections to and from a step are possible</li>
<li> Removed components \"AlternativeSplit\", \"AlternativeJoin\",
     \"ParallelSplit\" and \"ParallelJoin\" since the newly introduced
     components (\"Alternative\", \"Parallel\", vector connectors of steps)
     have the same modeling power but are safer and more convenient.</li>
<li> Removed the timer in a step (attach instead Logical.Timer to
     the \"active\" port of a \"StepWithSignal\" component). Note, that in
     most cases it is more convenient and more efficient to use the
     built-in timer of a transition.</li>
<li> Component \"StepInitial\" renamed to \"InitialStep\".</li>
<li> New component \"Timer\" within sublibrary Logical.</li>
<li> Updated and improved documentation of the library.</li>
</ul>
<h4>Version 0.85, 2004-06-17</h4>
<ul>
<li> Renamed \"MacroStep\" to \"CompositeStep\" and the ports of the MacroStep
     from \"abort\" to \"suspend\" and \"history\" to \"resume\".</li>
<li> Nested \"CompositeStep\" components are supported, based on the
     experimental feature of nested inner/outer components
     introduced by Dymola. This means that CompositeSteps can
     be suspended and resumed at every level.</li>
<li> New example \"Examples.ShowExceptions\" to demonstrate the new
     feature of nested CompositeSteps.</li>
<li> New package \"Logical\". It contains all components of
     ModelicaAdditions.Blocks.Logical, but with new block connectors
     and nicer icons. Additionally, logical blocks are also added.</li>
<li> Improved icons for several components of the StateGraph library.</li>
</ul>
<h4>Version 0.83, 2004-05-21</h4>
<ul>
<li> The \"abort\" and \"history\" connectors are no longer visible in the
     diagram layer of a CompositeStep since it is not allowed to connect
     to them in a CompositeStep.</li>
<li> Made the diagram/icon size of a CompositeStep smaller (from 200/-200 to
     150/-150).</li>
<li> Improved icons for \"SetBoolean/SetInteger/SetReal\" components.</li>
<li> Renamed \"ParameterReal\" to \"SetRealParameter\".</li>
</ul>
<h4>Version 0.82, 2004-05-18</h4>
<p>
Implemented a first version that is provided to other people.
</p>
</html>"));
      end ReleaseNotes;

      class Literature "Literature"
        extends Modelica.Icons.References;
        annotation(
          Documentation(info = "<html>
<p>
The StateGraph library is based on the following references:
</p>
<dl>
<dt>Arzen K.-E. (2004):</dt>
<dd> <b>JGrafchart User Manual. Version 1.5</b>.
     Department of Automatic Control, Lund Institute of Technology,
     Lund, Sweden, Feb. 13<br>&nbsp;</dd>
<dt>Dressler I. (2004):</dt>
<dd> <b>Code Generation From JGrafchart to Modelica</b>.
     Master thesis, supervisor: Karl-Erik Arzen,
     Department of Automatic Control, Lund Institute of Technology,
     Lund, Sweden, March 30<br>&nbsp;</dd>
<dt>Elmqvist H., Mattsson S.E., Otter M. (2001):</dt>
<dd> <b>Object-Oriented and Hybrid Modeling in Modelica</b>.
     Journal Europeen des systemes automatises (JESA),
     Volume 35 - n. 1.<br>&nbsp;</dd>
<dt>Mosterman P., Otter M., Elmqvist H. (1998):</dt>
<dd> <b>Modeling Petri Nets as Local Constraint Equations for
     Hybrid Systems using Modelica</b>.
     SCSC'98, Reno, Nevada, USA,
     Society for Computer Simulation International, pp. 314-319.
     </dd>
</dl>
</html>"));
      end Literature;

      class Contact "Contact"
        extends Modelica.Icons.Contact;
        annotation(
          Documentation(info = "<html>
<dl>
<dt><b>Main Author:</b>
<dd><a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a><br>
    Deutsches Zentrum f&uuml;r Luft und Raumfahrt e.V. (DLR)<br>
    Institut f&uuml;r Robotik und Mechatronik<br>
    Abteilung f&uuml;r Entwurfsorientierte Regelungstechnik<br>
    Postfach 1116<br>
    D-82230 Wessling<br>
    Germany<br>
    email: <A HREF=\"mailto:Martin.Otter@dlr.de\">Martin.Otter@dlr.de</A><br>
</dl>
<p><b>Acknowledgements:</b></p>
<ul>
<li> The development of this library was strongly motivated by the
     master thesis of Isolde Dressler
     (<a href=\"modelica://Modelica.StateGraph.UsersGuide.Literature\">see literature</a>),
     in which
     a compiler from JGrafchart to Modelica was designed and
     implemented. This project was supervised by Karl-Erik Arzen
     from Department of Automatic Control, Lund Institut of
     Technology, Lund, Sweden.</li>
<li> This library profits also from the experience gained
     in the focused research program (Schwerpunktprogramm)
     \"Continuous-Discrete Dynamic Systems\" (KONDISK), sponsored by the
     Deutsche Forschungsgemeinschaft under grants OT174/1-2 and EN152/22-2.
     This support is most gratefully acknowledged.
 </li>
<li> The implementation of the basic components of this library by describing
     finite state machines with equations is based on
     (Elmqvist, Mattsson and Otter, 2001),
     which in turn uses ideas from (Mosterman, Otter and Elmqvist, 1998),
     see <a href=\"modelica://Modelica.StateGraph.UsersGuide.Literature\">literature</a></li>
</ul>
</html>"));
      end Contact;
      annotation(
        DocumentationClass = true,
        Documentation(info = "<html>
<p>
Library <b>StateGraph</b> is a <b>free</b> Modelica package providing
components to model <b>discrete event</b> and <b>reactive</b>
systems in a convenient
way. This package contains the <b>User's Guide</b> for
the library and has the following content:
</p>
<ol>
<li><a href=\"modelica://Modelica.StateGraph.UsersGuide.OverView\">Overview of library</a>
     gives an overview of the library.</li>
<li> <a href=\"modelica://Modelica.StateGraph.UsersGuide.FirstExample\">A first example</a>
     demonstrates at hand of a first example how to use this library.</li>
<li> <a href=\"modelica://Modelica.StateGraph.UsersGuide.ApplicationExample\">An
     application example</a> demonstrates varies features at hand of the
     control of a two tank system.</li>
<li> <a href=\"modelica://Modelica.StateGraph.UsersGuide.ComparisonWithStateGraph2\">Comparison
     with StateGraph2</a> compares Modelica.StateGraph with the much improved version
     Modelica_StateGraph2.</li>
<li><a href=\"modelica://Modelica.StateGraph.UsersGuide.ReleaseNotes\">Release Notes</a>
    summarizes the differences between different versions of this library.</li>
<li><a href=\"modelica://Modelica.StateGraph.UsersGuide.Literature\">Literature</a>
    provides references that have been used to design and implement this
    library.</li>
<li><a href=\"modelica://Modelica.StateGraph.UsersGuide.Contact\">Contact</a>
    provides information about the authors of the library as well as
    acknowledgments.</li>
</ol>
</html>"));
    end UsersGuide;

    package Examples "Examples to demonstrate the usage of the components of the StateGraph library"
      extends Modelica.Icons.ExamplesPackage;

      model FirstExample "A first simple StateGraph example"
        extends Modelica.Icons.Example;
        InitialStep initialStep annotation(
          Placement(transformation(extent = {{-48, 0}, {-28, 20}}, rotation = 0)));
        Transition transition1(enableTimer = true, waitTime = 1) annotation(
          Placement(transformation(extent = {{-20, 0}, {0, 20}}, rotation = 0)));
        Step step annotation(
          Placement(transformation(extent = {{10, 0}, {30, 20}}, rotation = 0)));
        Transition transition2(enableTimer = true, waitTime = 1) annotation(
          Placement(transformation(extent = {{40, 0}, {60, 20}}, rotation = 0)));
        inner StateGraphRoot stateGraphRoot annotation(
          Placement(transformation(extent = {{-80, 60}, {-60, 80}})));
      equation
        connect(initialStep.outPort[1], transition1.inPort) annotation(
          Line(points = {{-27.5, 10}, {-14, 10}}, color = {0, 0, 0}));
        connect(transition1.outPort, step.inPort[1]) annotation(
          Line(points = {{-8.5, 10}, {9, 10}}, color = {0, 0, 0}));
        connect(step.outPort[1], transition2.inPort) annotation(
          Line(points = {{30.5, 10}, {46, 10}}, color = {0, 0, 0}));
        connect(transition2.outPort, initialStep.inPort[1]) annotation(
          Line(points = {{51.5, 10}, {70, 10}, {70, 32}, {-62, 32}, {-62, 10}, {-49, 10}}, color = {0, 0, 0}));
        annotation(
          experiment(StopTime = 5));
      end FirstExample;

      model FirstExample_Variant2 "A variant of the first simple StateGraph example"
        extends Modelica.Icons.Example;
        InitialStep initialStep annotation(
          Placement(transformation(extent = {{-70, 0}, {-50, 20}}, rotation = 0)));
        Transition transition1(enableTimer = true, waitTime = 1) annotation(
          Placement(transformation(extent = {{-42, 0}, {-22, 20}}, rotation = 0)));
        StepWithSignal step annotation(
          Placement(transformation(extent = {{-14, 0}, {6, 20}}, rotation = 0)));
        TransitionWithSignal transition2 annotation(
          Placement(transformation(extent = {{52, 0}, {72, 20}}, rotation = 0)));
        Modelica.Blocks.Logical.Timer timer annotation(
          Placement(transformation(extent = {{6, -40}, {26, -20}}, rotation = 0)));
        Modelica.Blocks.Logical.GreaterEqualThreshold greaterEqual(threshold = 1) annotation(
          Placement(transformation(extent = {{36, -40}, {56, -20}}, rotation = 0)));
        inner StateGraphRoot stateGraphRoot annotation(
          Placement(transformation(extent = {{-80, 60}, {-60, 80}})));
      equation
        connect(initialStep.outPort[1], transition1.inPort) annotation(
          Line(points = {{-49.5, 10}, {-36, 10}}, color = {0, 0, 0}));
        connect(transition1.outPort, step.inPort[1]) annotation(
          Line(points = {{-30.5, 10}, {-15, 10}}, color = {0, 0, 0}));
        connect(step.active, timer.u) annotation(
          Line(points = {{-4, -1}, {-4, -30}, {4, -30}}, color = {255, 0, 255}));
        connect(step.outPort[1], transition2.inPort) annotation(
          Line(points = {{6.5, 10}, {58, 10}}, color = {0, 0, 0}));
        connect(timer.y, greaterEqual.u) annotation(
          Line(points = {{27, -30}, {34, -30}}, color = {0, 0, 255}));
        connect(greaterEqual.y, transition2.condition) annotation(
          Line(points = {{57, -30}, {62, -30}, {62, -2}}, color = {255, 0, 255}));
        connect(transition2.outPort, initialStep.inPort[1]) annotation(
          Line(points = {{63.5, 10}, {82, 10}, {82, 32}, {-80, 32}, {-80, 10}, {-71, 10}}, color = {0, 0, 0}));
        annotation(
          experiment(StopTime = 5));
      end FirstExample_Variant2;

      model FirstExample_Variant3 "A variant of the first simple StateGraph example"
        extends Modelica.Icons.Example;
        InitialStep initialStep annotation(
          Placement(transformation(extent = {{-70, 0}, {-50, 20}}, rotation = 0)));
        Transition transition1(enableTimer = true, waitTime = 1) annotation(
          Placement(transformation(extent = {{-42, 0}, {-22, 20}}, rotation = 0)));
        StepWithSignal step annotation(
          Placement(transformation(extent = {{-14, 0}, {6, 20}}, rotation = 0)));
        TransitionWithSignal transition2 annotation(
          Placement(transformation(extent = {{56, 0}, {76, 20}}, rotation = 0)));
        Modelica.Blocks.Logical.Timer timer annotation(
          Placement(transformation(origin = {-4, -30}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
        Modelica.Blocks.Sources.BooleanExpression SetBoolean1(y = timer.y > 1) annotation(
          Placement(transformation(extent = {{28, -40}, {60, -20}}, rotation = 0)));
        Modelica.Blocks.Sources.BooleanExpression SetBoolean2(y = step.active) annotation(
          Placement(transformation(extent = {{-68, -40}, {-36, -20}}, rotation = 0)));
        inner StateGraphRoot stateGraphRoot annotation(
          Placement(transformation(extent = {{-80, 60}, {-60, 80}})));
      equation
        connect(initialStep.outPort[1], transition1.inPort) annotation(
          Line(points = {{-49.5, 10}, {-36, 10}}, color = {0, 0, 0}));
        connect(transition1.outPort, step.inPort[1]) annotation(
          Line(points = {{-30.5, 10}, {-15, 10}}, color = {0, 0, 0}));
        connect(step.active, timer.u) annotation(
          Line(points = {{-4, -1}, {-4, -9.5}, {-4, -18}, {-4, -18}}, color = {255, 0, 255}));
        connect(step.outPort[1], transition2.inPort) annotation(
          Line(points = {{6.5, 10}, {62, 10}}, color = {0, 0, 0}));
        connect(transition2.outPort, initialStep.inPort[1]) annotation(
          Line(points = {{67.5, 10}, {82, 10}, {82, 32}, {-80, 32}, {-80, 10}, {-71, 10}}, color = {0, 0, 0}));
        connect(SetBoolean1.y, transition2.condition) annotation(
          Line(points = {{61.6, -30}, {66, -30}, {66, -2}}, color = {255, 0, 255}));
        annotation(
          experiment(StopTime = 5));
      end FirstExample_Variant3;

      model ExecutionPaths "Example to demonstrate parallel and alternative execution paths"
        extends Modelica.Icons.Example;
        InitialStep step0 annotation(
          Placement(transformation(extent = {{-140, -100}, {-120, -80}}, rotation = 0)));
        Transition transition1(enableTimer = true, waitTime = 1) annotation(
          Placement(transformation(extent = {{-100, -100}, {-80, -80}}, rotation = 0)));
        Step step1 annotation(
          Placement(transformation(extent = {{-10, -40}, {10, -20}}, rotation = 0)));
        Transition transition2(enableTimer = true, waitTime = 1) annotation(
          Placement(transformation(extent = {{90, -100}, {110, -80}}, rotation = 0)));
        Step step6 annotation(
          Placement(transformation(extent = {{120, -100}, {140, -80}}, rotation = 0)));
        Step step2 annotation(
          Placement(transformation(extent = {{-98, 40}, {-78, 60}}, rotation = 0)));
        Transition transition3(enableTimer = true, waitTime = 1) annotation(
          Placement(transformation(extent = {{-42, 80}, {-22, 100}}, rotation = 0)));
        Transition transition4(enableTimer = true, waitTime = 1) annotation(
          Placement(transformation(extent = {{-42, 40}, {-22, 60}}, rotation = 0)));
        Step step3 annotation(
          Placement(transformation(extent = {{-8, 80}, {12, 100}}, rotation = 0)));
        Step step4 annotation(
          Placement(transformation(extent = {{-8, 40}, {12, 60}}, rotation = 0)));
        Transition transition5(enableTimer = true, waitTime = 1) annotation(
          Placement(transformation(extent = {{26, 80}, {46, 100}}, rotation = 0)));
        Transition transition6(enableTimer = true, waitTime = 1) annotation(
          Placement(transformation(extent = {{26, 40}, {46, 60}}, rotation = 0)));
        Step step5 annotation(
          Placement(transformation(extent = {{80, 40}, {100, 60}}, rotation = 0)));
        Modelica.Blocks.Sources.RealExpression setReal(y = time) annotation(
          Placement(transformation(extent = {{21, -160}, {41, -140}}, rotation = 0)));
        TransitionWithSignal transition7 annotation(
          Placement(transformation(extent = {{9, -134}, {-11, -114}}, rotation = 0)));
        Modelica.Blocks.Sources.BooleanExpression setCondition(y = time > 7) annotation(
          Placement(transformation(extent = {{-77, -160}, {-19, -140}}, rotation = 0)));
        Transition transition4a(enableTimer = true, waitTime = 1) annotation(
          Placement(transformation(extent = {{-42, 0}, {-22, 20}}, rotation = 0)));
        Step step4a annotation(
          Placement(transformation(extent = {{-8, 0}, {12, 20}}, rotation = 0)));
        Transition transition6a(enableTimer = true, waitTime = 2) annotation(
          Placement(transformation(extent = {{26, 0}, {46, 20}}, rotation = 0)));
        StateGraph.Temporary.NumericValue NumericValue1 annotation(
          Placement(transformation(extent = {{61, -160}, {81, -140}}, rotation = 0)));
        Alternative alternative(nBranches = 3) annotation(
          Placement(transformation(extent = {{-70, -10}, {72, 110}}, rotation = 0)));
        Parallel Parallel1 annotation(
          Placement(transformation(extent = {{-154, -50}, {152, 120}}, rotation = 0)));
        inner StateGraphRoot stateGraphRoot annotation(
          Placement(transformation(extent = {{-160, 120}, {-140, 140}})));
      equation
        connect(transition3.outPort, step3.inPort[1]) annotation(
          Line(points = {{-30.5, 90}, {-9, 90}}, color = {0, 0, 0}));
        connect(step3.outPort[1], transition5.inPort) annotation(
          Line(points = {{12.5, 90}, {32, 90}}, color = {0, 0, 0}));
        connect(transition4.outPort, step4.inPort[1]) annotation(
          Line(points = {{-30.5, 50}, {-9, 50}}, color = {0, 0, 0}));
        connect(step4.outPort[1], transition6.inPort) annotation(
          Line(points = {{12.5, 50}, {32, 50}}, color = {0, 0, 0}));
        connect(transition7.outPort, step0.inPort[1]) annotation(
          Line(points = {{-2.5, -124}, {-149, -124}, {-149, -90}, {-141, -90}}, color = {0, 0, 0}));
        connect(step6.outPort[1], transition7.inPort) annotation(
          Line(points = {{140.5, -90}, {150, -90}, {150, -124}, {3, -124}}, color = {0, 0, 0}));
        connect(transition4a.outPort, step4a.inPort[1]) annotation(
          Line(points = {{-30.5, 10}, {-9, 10}}, color = {0, 0, 0}));
        connect(step4a.outPort[1], transition6a.inPort) annotation(
          Line(points = {{12.5, 10}, {32, 10}}, color = {0, 0, 0}));
        connect(setCondition.y, transition7.condition) annotation(
          Line(points = {{-16.1, -150}, {-1, -150}, {-1, -136}}, color = {255, 0, 255}));
        connect(setReal.y, NumericValue1.Value) annotation(
          Line(points = {{42, -150}, {59, -150}}, color = {0, 0, 255}));
        connect(transition3.inPort, alternative.split[1]) annotation(
          Line(points = {{-36, 90}, {-55.09, 90}}, color = {0, 0, 0}));
        connect(transition4.inPort, alternative.split[2]) annotation(
          Line(points = {{-36, 50}, {-55.09, 50}}, color = {0, 0, 0}));
        connect(transition4a.inPort, alternative.split[3]) annotation(
          Line(points = {{-36, 10}, {-45.0125, 10}, {-45.0125, 10}, {-55.09, 10}}, color = {0, 0, 0}));
        connect(transition5.outPort, alternative.join[1]) annotation(
          Line(points = {{37.5, 90}, {57.09, 90}}, color = {0, 0, 0}));
        connect(transition6.outPort, alternative.join[2]) annotation(
          Line(points = {{37.5, 50}, {57.09, 50}}, color = {0, 0, 0}));
        connect(transition6a.outPort, alternative.join[3]) annotation(
          Line(points = {{37.5, 10}, {46.7625, 10}, {46.7625, 10}, {57.09, 10}}, color = {0, 0, 0}));
        connect(step2.outPort[1], alternative.inPort) annotation(
          Line(points = {{-77.5, 50}, {-72.13, 50}}, color = {0, 0, 0}));
        connect(alternative.outPort, step5.inPort[1]) annotation(
          Line(points = {{73.42, 50}, {79, 50}}, color = {0, 0, 0}));
        connect(step2.inPort[1], Parallel1.split[1]) annotation(
          Line(points = {{-99, 50}, {-118, 50}, {-118, 78}, {-119.575, 78}, {-119.575, 77.5}}, color = {0, 0, 0}));
        connect(step1.outPort[1], Parallel1.join[2]) annotation(
          Line(points = {{10.5, -30}, {118, -30}, {118, -7.5}, {117.575, -7.5}}, color = {0, 0, 0}));
        connect(step0.outPort[1], transition1.inPort) annotation(
          Line(points = {{-119.5, -90}, {-94, -90}}, color = {0, 0, 0}));
        connect(transition2.outPort, step6.inPort[1]) annotation(
          Line(points = {{101.5, -90}, {119, -90}}, color = {0, 0, 0}));
        connect(transition1.outPort, Parallel1.inPort) annotation(
          Line(points = {{-88.5, -90}, {-70, -90}, {-70, -64}, {-174, -64}, {-174, 35}, {-158.59, 35}}, color = {0, 0, 0}));
        connect(Parallel1.outPort, transition2.inPort) annotation(
          Line(points = {{155.06, 35}, {168, 35}, {168, -60}, {80, -60}, {80, -90}, {96, -90}}, color = {0, 0, 0}));
        connect(step5.outPort[1], Parallel1.join[1]) annotation(
          Line(points = {{100.5, 50}, {116, 50}, {116, 77.5}, {117.575, 77.5}}, color = {0, 0, 0}));
        connect(Parallel1.split[2], step1.inPort[1]) annotation(
          Line(points = {{-119.575, -7.5}, {-118, -8}, {-118, -30}, {-11, -30}}, color = {0, 0, 0}));
        annotation(
          Documentation(info = "<HTML>
<p>
This is an example to demonstrate in which way <b>parallel</b> activities
can be modelled by a StateGraph. When transition1 fires
(after 1 second), two branches are executed in parallel.
After 6 seconds the two branches are synchronized in order to arrive
at step6.
</p>
<p>
Before simulating the model, try to figure out whether which branch
of the alternative sequence is executed. Note, that alternatives
have priorities according to the port index (alternative.split[1]
has a higher priority to fire as alternative.split[2]).
</p>
</html>"),
          experiment(StopTime = 15),
          Diagram(coordinateSystem(extent = {{-200, -200}, {200, 200}}, initialScale = 0.1)));
      end ExecutionPaths;

      model ShowCompositeStep "Example to demonstrate parallel activities described by a StateGraph"
        extends Modelica.Icons.Example;
        Utilities.CompositeStep compositeStep annotation(
          Placement(transformation(extent = {{-10, 5}, {20, 35}}, rotation = 0)));
        InitialStep step0 annotation(
          Placement(transformation(extent = {{-89, -10}, {-69, 10}}, rotation = 0)));
        Transition transition1(enableTimer = true, waitTime = 1) annotation(
          Placement(transformation(extent = {{-59, -10}, {-39, 10}}, rotation = 0)));
        Step step1 annotation(
          Placement(transformation(extent = {{-4, -30}, {16, -10}}, rotation = 0)));
        Transition transition2(enableTimer = true, waitTime = 1) annotation(
          Placement(transformation(extent = {{45, -10}, {65, 10}}, rotation = 0)));
        Step step6 annotation(
          Placement(transformation(extent = {{71, -10}, {91, 10}}, rotation = 0)));
        TransitionWithSignal transition7 annotation(
          Placement(transformation(extent = {{10, -70}, {-10, -50}}, rotation = 0)));
        Parallel Parallel1 annotation(
          Placement(transformation(extent = {{-30, -40}, {36, 40}}, rotation = 0)));
        Modelica.Blocks.Sources.BooleanExpression setCondition(y = time > 7) annotation(
          Placement(transformation(extent = {{-40, -90}, {-10, -70}}, rotation = 0)));
        inner StateGraphRoot stateGraphRoot annotation(
          Placement(transformation(extent = {{-90, 50}, {-70, 70}})));
      equation
        connect(step0.outPort[1], transition1.inPort) annotation(
          Line(points = {{-68.5, 0}, {-53, 0}}, color = {0, 0, 0}));
        connect(transition7.outPort, step0.inPort[1]) annotation(
          Line(points = {{-1.5, -60}, {-98, -60}, {-98, 0}, {-90, 0}}, color = {0, 0, 0}));
        connect(step6.outPort[1], transition7.inPort) annotation(
          Line(points = {{91.5, 0}, {96, 0}, {96, -60}, {4, -60}}, color = {0, 0, 0}));
        connect(transition2.outPort, step6.inPort[1]) annotation(
          Line(points = {{56.5, 0}, {70, 0}}, color = {0, 0, 0}));
        connect(transition1.outPort, Parallel1.inPort) annotation(
          Line(points = {{-47.5, 0}, {-30.99, 0}}, color = {0, 0, 0}));
        connect(Parallel1.outPort, transition2.inPort) annotation(
          Line(points = {{36.66, 0}, {51, 0}}, color = {0, 0, 0}));
        connect(compositeStep.inPort, Parallel1.split[1]) annotation(
          Line(points = {{-11, 20}, {-22.575, 20}}, color = {0, 0, 0}));
        connect(compositeStep.outPort, Parallel1.join[1]) annotation(
          Line(points = {{20.5, 20}, {28.575, 20}}, color = {0, 0, 0}));
        connect(step1.inPort[1], Parallel1.split[2]) annotation(
          Line(points = {{-5, -20}, {-10, -20}, {-10, -20}, {-14, -20}, {-14, -20}, {-22.575, -20}}, color = {0, 0, 0}));
        connect(step1.outPort[1], Parallel1.join[2]) annotation(
          Line(points = {{16.5, -20}, {28.575, -20}}, color = {0, 0, 0}));
        connect(setCondition.y, transition7.condition) annotation(
          Line(points = {{-8.5, -80}, {0, -80}, {0, -72}}, color = {255, 0, 255}));
        annotation(
          Documentation(info = "<HTML>
<p>
This is the same example as \"ExecutionPaths\". The only difference
is that the alternative paths are included in a \"CompositeStep\".
</p>
</html>"),
          experiment(StopTime = 15));
      end ShowCompositeStep;

      model ShowExceptions "Example to demonstrate how a hierarchically structured StateGraph can suspend and resume actions on different levels"
        extends Modelica.Icons.Example;
        Utilities.CompositeStep1 compositeStep annotation(
          Placement(transformation(extent = {{-20, 25}, {10, 55}}, rotation = 0)));
        InitialStep initialStep annotation(
          Placement(transformation(extent = {{-80, 30}, {-60, 50}}, rotation = 0)));
        Transition transition1(enableTimer = true, waitTime = 1) annotation(
          Placement(transformation(extent = {{-50, 30}, {-30, 50}}, rotation = 0)));
        Transition transition2(enableTimer = true, waitTime = 1) annotation(
          Placement(transformation(extent = {{20, 30}, {40, 50}}, rotation = 0)));
        Transition transition3(enableTimer = true, waitTime = 2) annotation(
          Placement(transformation(extent = {{-55, -30}, {-35, -10}}, rotation = 0)));
        Step step1 annotation(
          Placement(transformation(extent = {{-24, -30}, {-4, -10}}, rotation = 0)));
        Transition transition4(enableTimer = true, waitTime = 1) annotation(
          Placement(transformation(extent = {{10, -30}, {30, -10}}, rotation = 0)));
        inner StateGraphRoot stateGraphRoot annotation(
          Placement(transformation(extent = {{-90, -80}, {-70, -60}})));
      equation
        connect(transition1.outPort, compositeStep.inPort) annotation(
          Line(points = {{-38.5, 40}, {-21, 40}}, color = {0, 0, 0}));
        connect(initialStep.outPort[1], transition1.inPort) annotation(
          Line(points = {{-59.5, 40}, {-44, 40}}, color = {0, 0, 0}));
        connect(compositeStep.outPort, transition2.inPort) annotation(
          Line(points = {{10.5, 40}, {26, 40}}, color = {0, 0, 0}));
        connect(transition2.outPort, initialStep.inPort[1]) annotation(
          Line(points = {{31.5, 40}, {46, 40}, {46, 80}, {-90, 80}, {-90, 40}, {-81, 40}}, color = {0, 0, 0}));
        connect(compositeStep.suspend[1], transition3.inPort) annotation(
          Line(points = {{-12.5, 24.5}, {-12.5, 10}, {-60, 10}, {-60, -20}, {-49, -20}}, color = {0, 0, 0}));
        connect(transition3.outPort, step1.inPort[1]) annotation(
          Line(points = {{-43.5, -20}, {-25, -20}}, color = {0, 0, 0}));
        connect(step1.outPort[1], transition4.inPort) annotation(
          Line(points = {{-3.5, -20}, {16, -20}}, color = {0, 0, 0}));
        connect(transition4.outPort, compositeStep.resume[1]) annotation(
          Line(points = {{21.5, -20}, {40, -20}, {40, 10}, {2.5, 10}, {2.5, 24}}, color = {0, 0, 0}));
        annotation(
          Documentation(info = "<HTML>
<p>
CompositeStep \"compositeStep\" is a hierarchical StateGraph consisting of
two other subgraphs. Whenever component \"compositeStep\" is suspended,
all steps with in \"compositeStep\" are deactivated. By entering \"compositeStep\"
via its \"resume\" port, all steps within \"compositeStep\" are activated
according to their setting before leaving the \"compositeStep\" via its
\"suspend\" port.
</p>
</html>"),
          experiment(StopTime = 20));
      end ShowExceptions;

      model ControlledTanks "Demonstrating the controller of a tank filling/emptying system"
        extends Modelica.Icons.Example;
        Utilities.TankController tankController annotation(
          Placement(transformation(extent = {{-50, -20}, {-10, 20}}, rotation = 0)));
        StateGraph.Temporary.RadioButton start(reset = {stop.on, shut.on}, buttonTimeTable = {1, 13, 15, 19}) annotation(
          Placement(transformation(extent = {{-90, 20}, {-70, 40}}, rotation = 0)));
        StateGraph.Temporary.RadioButton stop(reset = {start.on, shut.on}, buttonTimeTable = {13, 15, 19, 21}) annotation(
          Placement(transformation(extent = {{-90, -10}, {-70, 10}}, rotation = 0)));
        StateGraph.Temporary.RadioButton shut(reset = {start.on, stop.on}, buttonTimeTable = {21, 100}) annotation(
          Placement(transformation(extent = {{-90, -40}, {-70, -20}}, rotation = 0)));
        Utilities.Tank tank1 annotation(
          Placement(transformation(extent = {{10, 20}, {60, 70}}, rotation = 0)));
        Utilities.Tank tank2 annotation(
          Placement(transformation(extent = {{34, -60}, {84, -10}}, rotation = 0)));
        Utilities.valve valve1 annotation(
          Placement(transformation(origin = {22.5, 72}, extent = {{-5.5, -5.5}, {5.5, 5.5}}, rotation = 270)));
        Utilities.Source source annotation(
          Placement(transformation(extent = {{12.5, 80.5}, {32.5, 100.5}}, rotation = 0)));
        Utilities.valve valve2 annotation(
          Placement(transformation(origin = {46.5, 13}, extent = {{-7, -8}, {7, 8}}, rotation = 270)));
        Utilities.valve valve3 annotation(
          Placement(transformation(origin = {73.5, -77}, extent = {{-7, -8}, {7, 8}}, rotation = 270)));
        inner StateGraphRoot stateGraphRoot annotation(
          Placement(transformation(extent = {{-90, 75}, {-70, 95}})));
      equation
        connect(tank1.outflow1, valve2.outflow1) annotation(
          Line(points = {{50, 33.75}, {50, 26.875}, {46.5, 26.875}, {46.5, 16.5}}, color = {0, 0, 0}, thickness = 0.5));
        connect(tank2.inflow1, valve2.inflow1) annotation(
          Line(points = {{46.5, -18.75}, {46.5, 9.5}}, color = {0, 0, 0}, thickness = 0.5));
        connect(tank2.outflow1, valve3.outflow1) annotation(
          Line(points = {{74, -46.25}, {74, -73.5}, {73.5, -73.5}}, color = {0, 0, 0}, thickness = 0.5));
        connect(tank1.inflow1, valve1.inflow1) annotation(
          Line(points = {{22.5, 61.25}, {22.5, 69.25}}, color = {0, 0, 0}, thickness = 0.5));
        connect(shut.on, tankController.shut) annotation(
          Line(points = {{-69, -30}, {-62, -30}, {-62, -12}, {-52, -12}}, color = {255, 0, 255}));
        connect(stop.on, tankController.stop) annotation(
          Line(points = {{-69, 0}, {-52, 0}}, color = {255, 0, 255}));
        connect(start.on, tankController.start) annotation(
          Line(points = {{-69, 30}, {-60, 30}, {-60, 12}, {-52, 12}}, color = {255, 0, 255}));
        connect(tank1.levelSensor, tankController.level1) annotation(
          Line(points = {{17.25, 40}, {-30, 40}, {-30, 60}, {-97, 60}, {-97, -50}, {-42, -50}, {-42, -22}}, color = {0, 0, 255}));
        connect(tank2.levelSensor, tankController.level2) annotation(
          Line(points = {{41.25, -40}, {-18, -40}, {-18, -22}}, color = {0, 0, 255}));
        connect(tankController.valve1, valve1.valveControl) annotation(
          Line(points = {{-9, 12}, {10, 12}, {10, 72}, {18.1, 72}}, color = {255, 0, 255}));
        connect(tankController.valve2, valve2.valveControl) annotation(
          Line(points = {{-9, 0}, {30, 0}, {30, 13}, {40.1, 13}}, color = {255, 0, 255}));
        connect(tankController.valve3, valve3.valveControl) annotation(
          Line(points = {{-9, -12}, {23, -12}, {23, -77}, {67.1, -77}}, color = {255, 0, 255}));
        connect(source.outflow1, valve1.outflow1) annotation(
          Line(points = {{22.5, 85.5}, {22.5, 74.75}}, color = {0, 0, 0}, thickness = 0.5));
        annotation(
          experiment(StopTime = 100),
          Documentation(info = "<html>
<p>
With this example the controller of a tank filling/emptying system
is demonstrated. This example is from Dressler (2004),
see <a href=\"modelica://Modelica.StateGraph.UsersGuide.Literature\">Literature</a>.
The basic operation is to fill and empty the two tanks:
</p>
<ol>
<li> Valve 1 is opened and tank 1 is filled.</li>
<li> When tank 1 reaches its fill level limit,
     valve 1 is closed. </li>
<li> After a waiting time, valve 2 is
     opened and the fluid flows from tank 1 into tank 2.</li>
<li> When tank 1 is empty, valve 2 is closed. </li>
<li> After a waiting time, valve 3 is opened and
     the fluid flows out of tank 2</li>
<li> When tank 3 is empty, valve 3 is closed</li>
</ol>
<p>
The above \"normal\" process can be influenced by three
buttons:
</p>
<ul>
<li> Button <b>start</b> starts the above process.
     When this button is pressed after a \"stop\" or
     \"shut\" operation, the process operation continues.
     </li>
<li> Button <b>stop</b> stops the above process by
     closing all valves. Then, the controller waits for
     further input (either \"start\" or \"shut\" operation).</li>
<li> Button <b>shut</b> is used to shutdown the process,
     by emptying at once both tanks. When this is achieved,
     the process goes back to its start configuration.
     Clicking on \"start\", restarts the process.</li>
</ul>

</html>"));
      end ControlledTanks;

      package Utilities "Utility components for the examples"
        extends Modelica.Icons.UtilitiesPackage;

        model TankController "Controller for tank system"
          extends StateGraph.Interfaces.PartialStateGraphIcon;
          parameter Temporary.SetRealParameter limit = 0.98 "Limit level of tank 1" annotation(
            Placement(transformation(extent = {{-64, 76}, {-44, 96}}, rotation = 0)));
          parameter Temporary.SetRealParameter waitTime = 3 "Wait time" annotation(
            Placement(transformation(extent = {{-91, 76}, {-71, 96}}, rotation = 0)));
          InitialStep s1(nIn = 2) annotation(
            Placement(transformation(extent = {{-72, 30}, {-52, 50}}, rotation = 0)));
          MakeProduct makeProduct(limit = limit, waitTime = waitTime) annotation(
            Placement(transformation(extent = {{-20, 25}, {10, 55}}, rotation = 0)));
          Transition T1(condition = start) annotation(
            Placement(transformation(extent = {{-50, 50}, {-30, 30}}, rotation = 0)));
          Transition T2(condition = level2 < 0.001) annotation(
            Placement(transformation(extent = {{27, 50}, {47, 30}}, rotation = 0)));
          Transition T3(condition = stop) annotation(
            Placement(transformation(origin = {-23, -1}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          Step s2(nOut = 2) annotation(
            Placement(transformation(extent = {{-50, -60}, {-30, -40}}, rotation = 0)));
          Transition T4(condition = start) annotation(
            Placement(transformation(origin = {10, -1}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
          Transition T5(condition = shut) annotation(
            Placement(transformation(extent = {{-6, -60}, {14, -40}}, rotation = 0)));
          Step emptyTanks annotation(
            Placement(transformation(extent = {{22, -60}, {42, -40}}, rotation = 0)));
          Transition T6(condition = level1 + level2 < 0.001) annotation(
            Placement(transformation(extent = {{45, -60}, {65, -40}}, rotation = 0)));
          Modelica.Blocks.Interfaces.BooleanInput start annotation(
            Placement(transformation(extent = {{-120, 50}, {-100, 70}}, rotation = 0)));
          Modelica.Blocks.Interfaces.BooleanInput stop annotation(
            Placement(transformation(extent = {{-120, -10}, {-100, 10}}, rotation = 0)));
          Modelica.Blocks.Interfaces.BooleanInput shut annotation(
            Placement(transformation(extent = {{-120, -70}, {-100, -50}}, rotation = 0)));
          Modelica.Blocks.Interfaces.RealInput level1 annotation(
            Placement(transformation(origin = {-60, -110}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
          Modelica.Blocks.Interfaces.RealInput level2 annotation(
            Placement(transformation(origin = {60, -110}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
          Modelica.Blocks.Interfaces.BooleanOutput valve1 annotation(
            Placement(transformation(extent = {{100, 55}, {110, 65}}, rotation = 0)));
          Modelica.Blocks.Interfaces.BooleanOutput valve2 annotation(
            Placement(transformation(extent = {{100, -5}, {110, 5}}, rotation = 0)));
          Modelica.Blocks.Interfaces.BooleanOutput valve3 annotation(
            Placement(transformation(extent = {{100, -65}, {110, -55}}, rotation = 0)));
          Modelica.Blocks.Sources.BooleanExpression setValve1(y = makeProduct.fillTank1.active) annotation(
            Placement(transformation(extent = {{20, 73}, {80, 92}}, rotation = 0)));
          Modelica.Blocks.Sources.BooleanExpression setValve2(y = makeProduct.fillTank2.active or emptyTanks.active) annotation(
            Placement(transformation(extent = {{-25, -89}, {80, -68}}, rotation = 0)));
          Modelica.Blocks.Sources.BooleanExpression setValve3(y = makeProduct.emptyTank2.active or emptyTanks.active) annotation(
            Placement(transformation(extent = {{-26, -100}, {80, -80}}, rotation = 0)));
        equation
          connect(s1.outPort[1], T1.inPort) annotation(
            Line(points = {{-51.5, 40}, {-44, 40}}, color = {0, 0, 0}));
          connect(T1.outPort, makeProduct.inPort) annotation(
            Line(points = {{-38.5, 40}, {-21, 40}}, color = {0, 0, 0}));
          connect(makeProduct.outPort, T2.inPort) annotation(
            Line(points = {{10.5, 40}, {33, 40}}, color = {0, 0, 0}));
          connect(T5.outPort, emptyTanks.inPort[1]) annotation(
            Line(points = {{5.5, -50}, {21, -50}}, color = {0, 0, 0}));
          connect(emptyTanks.outPort[1], T6.inPort) annotation(
            Line(points = {{42.5, -50}, {51, -50}}, color = {0, 0, 0}));
          connect(setValve1.y, valve1) annotation(
            Line(points = {{83, 82.5}, {90, 82.5}, {90, 60}, {105, 60}}, color = {255, 0, 255}));
          connect(setValve2.y, valve2) annotation(
            Line(points = {{85.25, -78.5}, {90, -78.5}, {90, 0}, {105, 0}}, color = {255, 0, 255}));
          connect(setValve3.y, valve3) annotation(
            Line(points = {{85.3, -90}, {95, -90}, {95, -60}, {105, -60}}, color = {255, 0, 255}));
          connect(makeProduct.suspend[1], T3.inPort) annotation(
            Line(points = {{-12.5, 24.5}, {-12.5, 12}, {-23, 12}, {-23, 3}}, color = {0, 0, 0}));
          connect(T3.outPort, s2.inPort[1]) annotation(
            Line(points = {{-23, -2.5}, {-23, -20}, {-66, -20}, {-66, -50}, {-51, -50}}, color = {0, 0, 0}));
          connect(T4.outPort, makeProduct.resume[1]) annotation(
            Line(points = {{10, 0.5}, {10, 15}, {2.5, 15}, {2.5, 24}}, color = {0, 0, 0}));
          connect(level1, makeProduct.level1) annotation(
            Line(points = {{-60, -110}, {-60, -80}, {-80, -80}, {-80, 20}, {-30, 20}, {-30, 28}, {-22, 28}}, color = {0, 0, 255}));
          connect(s2.outPort[1], T5.inPort) annotation(
            Line(points = {{-29.5, -49.75}, {-30, -49.75}, {-30, -50}, {0, -50}}, color = {0, 0, 0}));
          connect(s2.outPort[2], T4.inPort) annotation(
            Line(points = {{-29.5, -50.25}, {-29, -50}, {-8, -50}, {-8, -25}, {10, -25}, {10, -5}}, color = {0, 0, 0}));
          connect(T2.outPort, s1.inPort[1]) annotation(
            Line(points = {{38.5, 40}, {70, 40}, {70, 70}, {-84, 70}, {-84, 40}, {-73, 40}, {-73, 40.5}}, color = {0, 0, 0}));
          connect(T6.outPort, s1.inPort[2]) annotation(
            Line(points = {{56.5, -50}, {70, -50}, {70, 70}, {-84, 70}, {-84, 40}, {-74, 40}, {-73, 39.5}}, color = {0, 0, 0}));
          annotation(
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, 100}, {100, -100}}, lineColor = {0, 0, 0})}),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-100, 68}, {-32, 54}}, lineColor = {0, 0, 0}, textString = "start"), Text(extent = {{-100, 6}, {-32, -8}}, lineColor = {0, 0, 0}, textString = "stop"), Text(extent = {{-100, -54}, {-32, -68}}, lineColor = {0, 0, 0}, textString = "shut"), Text(extent = {{-94, -82}, {-18, -96}}, lineColor = {0, 0, 0}, textString = "level1"), Text(extent = {{24, -84}, {96, -98}}, lineColor = {0, 0, 0}, textString = "level2"), Text(extent = {{31, 68}, {99, 54}}, lineColor = {0, 0, 0}, textString = "valve1"), Text(extent = {{33, 9}, {101, -5}}, lineColor = {0, 0, 0}, textString = "valve2"), Text(extent = {{34, -53}, {102, -67}}, lineColor = {0, 0, 0}, textString = "valve3")}));
        end TankController;

        model MakeProduct "State machine defining the time instants when to fill or empty a tank"
          extends StateGraph.PartialCompositeStep;
          parameter StateGraph.Temporary.SetRealParameter limit = 0.98 "Limit level of tank 1" annotation(
            Placement(transformation(extent = {{-60, 40}, {-20, 60}}, rotation = 0)));
          parameter StateGraph.Temporary.SetRealParameter waitTime = 3 "Wait time" annotation(
            Placement(transformation(extent = {{-120, 40}, {-80, 60}}, rotation = 0)));
          Modelica.Blocks.Interfaces.RealInput level1 annotation(
            Placement(transformation(extent = {{-190, -140}, {-150, -100}}, rotation = 0)));
          Step fillTank1 annotation(
            Placement(transformation(extent = {{-140, -10}, {-120, 10}}, rotation = 0)));
          Transition T1(condition = level1 > limit) annotation(
            Placement(transformation(extent = {{-110, -10}, {-90, 10}}, rotation = 0)));
          Step fillTank2 annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0)));
          Transition T3(condition = level1 < 0.001) annotation(
            Placement(transformation(extent = {{20, -10}, {40, 10}}, rotation = 0)));
          Step emptyTank2 annotation(
            Placement(transformation(extent = {{120, -10}, {140, 10}}, rotation = 0)));
          Step wait1 annotation(
            Placement(transformation(extent = {{-80, -10}, {-60, 10}}, rotation = 0)));
          Transition T2(enableTimer = true, waitTime = waitTime) annotation(
            Placement(transformation(extent = {{-50, -10}, {-30, 10}}, rotation = 0)));
          Step wait2 annotation(
            Placement(transformation(extent = {{54, -10}, {74, 10}}, rotation = 0)));
          Transition T4(enableTimer = true, waitTime = waitTime) annotation(
            Placement(transformation(extent = {{82, -10}, {102, 10}}, rotation = 0)));
        equation
          connect(fillTank1.inPort[1], inPort) annotation(
            Line(points = {{-141, 0}, {-160, 0}}, color = {0, 0, 0}));
          connect(fillTank1.outPort[1], T1.inPort) annotation(
            Line(points = {{-119.5, 0}, {-104, 0}}, color = {0, 0, 0}));
          connect(fillTank2.outPort[1], T3.inPort) annotation(
            Line(points = {{10.5, 0}, {26, 0}}, color = {0, 0, 0}));
          connect(emptyTank2.outPort[1], outPort) annotation(
            Line(points = {{140.5, 0}, {155, 0}}, color = {0, 0, 0}));
          connect(wait1.outPort[1], T2.inPort) annotation(
            Line(points = {{-59.5, 0}, {-44, 0}}, color = {0, 0, 0}));
          connect(T2.outPort, fillTank2.inPort[1]) annotation(
            Line(points = {{-38.5, 0}, {-11, 0}}, color = {0, 0, 0}));
          connect(T1.outPort, wait1.inPort[1]) annotation(
            Line(points = {{-98.5, 0}, {-81, 0}}, color = {0, 0, 0}));
          connect(wait2.outPort[1], T4.inPort) annotation(
            Line(points = {{74.5, 0}, {88, 0}}, color = {0, 0, 0}));
          connect(T3.outPort, wait2.inPort[1]) annotation(
            Line(points = {{31.5, 0}, {53, 0}}, color = {0, 0, 0}));
          connect(T4.outPort, emptyTank2.inPort[1]) annotation(
            Line(points = {{93.5, 0}, {119, 0}}, color = {0, 0, 0}));
        end MakeProduct;

        connector Inflow1 "Inflow connector (this is a copy from Isolde Dressler's master thesis project)"
          import Units = Modelica.SIunits;
          input Units.VolumeFlowRate Fi "inflow";
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{-100, -100}, {0, 100}, {100, -100}, {-100, -100}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid)}));
        end Inflow1;

        connector Inflow2 "Inflow connector (this is a copy from Isolde Dressler's master thesis project)"
          import Units = Modelica.SIunits;
          output Units.VolumeFlowRate Fi "inflow";
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{-100, -100}, {0, 100}, {100, -100}, {-100, -100}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid)}));
        end Inflow2;

        connector Outflow1 "Outflow connector (this is a copy from Isolde Dressler's master thesis project)"
          import Units = Modelica.SIunits;
          output Units.VolumeFlowRate Fo "outflow";
          input Boolean open "valve open";
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{-100, 100}, {0, -100}, {100, 100}, {-100, 100}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid)}));
        end Outflow1;

        connector Outflow2 "Outflow connector (this is a copy from Isolde Dressler's master thesis project)"
          import Units = Modelica.SIunits;
          input Units.VolumeFlowRate Fo "outflow";
          output Boolean open "valve open";
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{-100, 100}, {0, -100}, {100, 100}, {-100, 100}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid)}));
        end Outflow2;

        model valve "Simple valve model (this is a copy from Isolde Dressler's master thesis project)"
          Modelica.Blocks.Interfaces.BooleanInput valveControl annotation(
            Placement(transformation(origin = {0, -80}, extent = {{-20, -20}, {20, 20}}, rotation = 90)));
          Modelica.StateGraph.Examples.Utilities.Inflow2 inflow1 annotation(
            Placement(transformation(origin = {50, 0}, extent = {{-50, -50}, {50, 50}}, rotation = 90)));
          Modelica.StateGraph.Examples.Utilities.Outflow2 outflow1 annotation(
            Placement(transformation(origin = {-50, 0}, extent = {{-50, -50}, {50, 50}}, rotation = 90)));
        equation
          outflow1.Fo = inflow1.Fi;
          outflow1.open = valveControl;
          annotation(
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{0, -60}, {0, 0}}, color = {255, 0, 255})}),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{20, 20}, {20, 20}}, color = {0, 0, 0}, thickness = 0.5), Text(extent = {{-131, 125}, {136, 67}}, lineColor = {0, 0, 255}, textString = "%name"), Line(points = {{0, 0}, {0, -60}}, color = {255, 0, 255})}));
        end valve;

        model Tank "Simple tank model (this is a copy from Isolde Dressler's master thesis project)"
          Modelica.Blocks.Interfaces.RealOutput levelSensor annotation(
            Placement(transformation(extent = {{-61, -30}, {-81, -10}}, rotation = 0)));
          Modelica.StateGraph.Examples.Utilities.Inflow1 inflow1 annotation(
            Placement(transformation(extent = {{-55, 60}, {-45, 70}}, rotation = 0)));
          Modelica.StateGraph.Examples.Utilities.Outflow1 outflow1 annotation(
            Placement(transformation(extent = {{55, -50}, {65, -40}}, rotation = 0)));
          Real level(start = 0, fixed = true) "Tank level in % of max height";
          parameter Real A = 1 "Ground area of tank in m^2";
          parameter Real a = 0.2 "Area of drain hole in m^2";
          parameter Real hmax = 1 "Max height of tank in m";
          constant Real g = Modelica.Constants.g_n;
        equation
          der(level) = (inflow1.Fi - outflow1.Fo) / (hmax * A);
          if outflow1.open then
            outflow1.Fo = sqrt(max(0, 2 * g * hmax * level)) * a;
          else
            outflow1.Fo = 0;
          end if;
          levelSensor = level;
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-122, -82}, {88, -42}}, textString = "%name", lineColor = {0, 0, 255}), Rectangle(extent = {{-60, 60}, {80, -40}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Rectangle(extent = DynamicSelect({{-60, -40}, {-60, -40}}, {{-60, -40}, {80, (-40) + level * 100}}), lineColor = {0, 0, 0}, lineThickness = 0.5, fillPattern = FillPattern.HorizontalCylinder, fillColor = {191, 0, 95})}));
        end Tank;

        model Source "Simple source model (this is a copy from Isolde Dressler's master thesis project)"
          Modelica.StateGraph.Examples.Utilities.Outflow1 outflow1 annotation(
            Placement(transformation(extent = {{-10, -60}, {10, -40}}, rotation = 0)));
          parameter Real maxflow = 1 "maximal flow out of source";
        equation
          if outflow1.open then
            outflow1.Fo = maxflow;
          else
            outflow1.Fo = 0;
          end if;
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-80, 40}, {80, -40}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-144, 54}, {152, 114}}, textString = "%name", lineColor = {0, 0, 255})}));
        end Source;

        model CompositeStep "State machine demonstrating a composite step (used in StateGraph.Examples.ShowCompositeStep)"
          extends StateGraph.PartialCompositeStep;
          Transition transition3(enableTimer = true, waitTime = 1) annotation(
            Placement(transformation(extent = {{-64, 50}, {-44, 70}}, rotation = 0)));
          Transition transition4(enableTimer = true, waitTime = 1) annotation(
            Placement(transformation(extent = {{-64, -10}, {-44, 10}}, rotation = 0)));
          Step step3 annotation(
            Placement(transformation(extent = {{-10, 50}, {10, 70}}, rotation = 0)));
          Step step4 annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0)));
          Transition transition5(enableTimer = true, waitTime = 2) annotation(
            Placement(transformation(extent = {{36, 50}, {56, 70}}, rotation = 0)));
          Transition transition6(enableTimer = true, waitTime = 2) annotation(
            Placement(transformation(extent = {{36, -10}, {56, 10}}, rotation = 0)));
          Transition transition4a(enableTimer = true, waitTime = 1) annotation(
            Placement(transformation(extent = {{-64, -70}, {-44, -50}}, rotation = 0)));
          Step step4a annotation(
            Placement(transformation(extent = {{-10, -70}, {10, -50}}, rotation = 0)));
          Transition transition6a(enableTimer = true, waitTime = 2) annotation(
            Placement(transformation(extent = {{36, -70}, {56, -50}}, rotation = 0)));
          Step initStep annotation(
            Placement(transformation(extent = {{-140, -10}, {-120, 10}}, rotation = 0)));
          Step exitStep annotation(
            Placement(transformation(extent = {{120, -10}, {140, 10}}, rotation = 0)));
          Alternative Alternative1(nBranches = 3) annotation(
            Placement(transformation(extent = {{-98, -90}, {98, 90}}, rotation = 0)));
        equation
          connect(transition3.outPort, step3.inPort[1]) annotation(
            Line(points = {{-52.5, 60}, {-11, 60}}, color = {0, 0, 0}));
          connect(step3.outPort[1], transition5.inPort) annotation(
            Line(points = {{10.5, 60}, {42, 60}}, color = {0, 0, 0}));
          connect(transition4.outPort, step4.inPort[1]) annotation(
            Line(points = {{-52.5, 0}, {-11, 0}}, color = {0, 0, 0}));
          connect(step4.outPort[1], transition6.inPort) annotation(
            Line(points = {{10.5, 0}, {42, 0}}, color = {0, 0, 0}));
          connect(transition4a.outPort, step4a.inPort[1]) annotation(
            Line(points = {{-52.5, -60}, {-11, -60}}, color = {0, 0, 0}));
          connect(step4a.outPort[1], transition6a.inPort) annotation(
            Line(points = {{10.5, -60}, {42, -60}}, color = {0, 0, 0}));
          connect(initStep.inPort[1], inPort) annotation(
            Line(points = {{-141, 0}, {-160, 0}}, color = {0, 0, 0}));
          connect(exitStep.outPort[1], outPort) annotation(
            Line(points = {{140.5, 0}, {155, 0}}, color = {0, 0, 0}));
          connect(initStep.outPort[1], Alternative1.inPort) annotation(
            Line(points = {{-119.5, 0}, {-100.94, 0}}, color = {0, 0, 0}));
          connect(Alternative1.outPort, exitStep.inPort[1]) annotation(
            Line(points = {{99.96, 0}, {119, 0}}, color = {0, 0, 0}));
          connect(transition3.inPort, Alternative1.split[1]) annotation(
            Line(points = {{-58, 60}, {-77.42, 60}}, color = {0, 0, 0}));
          connect(transition4.inPort, Alternative1.split[2]) annotation(
            Line(points = {{-58, 0}, {-67.975, 0}, {-77.42, 0}}, color = {0, 0, 0}));
          connect(transition4a.inPort, Alternative1.split[3]) annotation(
            Line(points = {{-58, -60}, {-67.975, -60}, {-77.42, -60}}, color = {0, 0, 0}));
          connect(transition5.outPort, Alternative1.join[1]) annotation(
            Line(points = {{47.5, 60}, {77.42, 60}}, color = {0, 0, 0}));
          connect(transition6.outPort, Alternative1.join[2]) annotation(
            Line(points = {{47.5, 0}, {60.725, 0}, {77.42, 0}}, color = {0, 0, 0}));
          connect(transition6a.outPort, Alternative1.join[3]) annotation(
            Line(points = {{47.5, -60}, {60.725, -60}, {77.42, -60}}, color = {0, 0, 0}));
        end CompositeStep;

        model CompositeStep1 "Composite step used to demonstrate exceptions (in StateGraph.Examples.ShowExceptions)"
          extends PartialCompositeStep;
          Transition transition1(enableTimer = false, waitTime = 0, condition = time > 8) annotation(
            Placement(transformation(extent = {{-60, 20}, {-40, 40}}, rotation = 0)));
          Step initStep annotation(
            Placement(transformation(extent = {{-140, -10}, {-120, 10}}, rotation = 0)));
          Step exitStep annotation(
            Placement(transformation(extent = {{110, -10}, {130, 10}}, rotation = 0)));
          CompositeStep2 compositeStep11(waitTime = 3) annotation(
            Placement(transformation(extent = {{-20, 15}, {10, 45}}, rotation = 0)));
          CompositeStep2 compositeStep12(waitTime = 2) annotation(
            Placement(transformation(extent = {{-20, -45}, {10, -15}}, rotation = 0)));
          Transition transition2(condition = time > 4, enableTimer = false, waitTime = 0) annotation(
            Placement(transformation(extent = {{-61, -40}, {-41, -20}}, rotation = 0)));
          Transition transition3(enableTimer = false, waitTime = 0) annotation(
            Placement(transformation(extent = {{29, 20}, {49, 40}}, rotation = 0)));
          Transition transition4(enableTimer = false, waitTime = 0) annotation(
            Placement(transformation(extent = {{29, -40}, {49, -20}}, rotation = 0)));
          Alternative Alternative1 annotation(
            Placement(transformation(extent = {{-100, -60}, {89, 60}}, rotation = 0)));
        equation
          connect(exitStep.outPort[1], outPort) annotation(
            Line(points = {{130.5, 0}, {155, 0}}, color = {0, 0, 0}));
          connect(initStep.inPort[1], inPort) annotation(
            Line(points = {{-141, 0}, {-160, 0}}, color = {0, 0, 0}));
          connect(transition1.outPort, compositeStep11.inPort) annotation(
            Line(points = {{-48.5, 30}, {-21, 30}}, color = {0, 0, 0}));
          connect(transition2.outPort, compositeStep12.inPort) annotation(
            Line(points = {{-49.5, -30}, {-21, -30}}, color = {0, 0, 0}));
          connect(compositeStep11.outPort, transition3.inPort) annotation(
            Line(points = {{10.5, 30}, {35, 30}}, color = {0, 0, 0}));
          connect(compositeStep12.outPort, transition4.inPort) annotation(
            Line(points = {{10.5, -30}, {35, -30}}, color = {0, 0, 0}));
          connect(initStep.outPort[1], Alternative1.inPort) annotation(
            Line(points = {{-119.5, 0}, {-102.835, 0}}, color = {0, 0, 0}));
          connect(Alternative1.outPort, exitStep.inPort[1]) annotation(
            Line(points = {{90.89, 0}, {109, 0}}, color = {0, 0, 0}));
          connect(transition1.inPort, Alternative1.split[1]) annotation(
            Line(points = {{-54, 30}, {-80.155, 30}}, color = {0, 0, 0}));
          connect(transition2.inPort, Alternative1.split[2]) annotation(
            Line(points = {{-55, -30}, {-80.155, -30}}, color = {0, 0, 0}));
          connect(transition3.outPort, Alternative1.join[1]) annotation(
            Line(points = {{40.5, 30}, {69.155, 30}}, color = {0, 0, 0}));
          connect(transition4.outPort, Alternative1.join[2]) annotation(
            Line(points = {{40.5, -30}, {69.155, -30}}, color = {0, 0, 0}));
        end CompositeStep1;

        model CompositeStep2 "Composite step used to demonstrate exceptions (in StateGraph.Examples.ShowExceptions)"
          extends PartialCompositeStep;
          Transition transition(enableTimer = true, waitTime = waitTime) annotation(
            Placement(transformation(extent = {{-30, -10}, {-10, 10}}, rotation = 0)));
          Step initStep annotation(
            Placement(transformation(extent = {{-140, -10}, {-120, 10}}, rotation = 0)));
          Step exitStep annotation(
            Placement(transformation(extent = {{110, -10}, {130, 10}}, rotation = 0)));
          parameter Temporary.SetRealParameter waitTime = 2 "waiting time in this composite step" annotation(
            Placement(transformation(extent = {{-80, 30}, {-50, 50}}, rotation = 0)));
        equation
          connect(exitStep.outPort[1], outPort) annotation(
            Line(points = {{130.5, 0}, {155, 0}}, color = {0, 0, 0}));
          connect(initStep.inPort[1], inPort) annotation(
            Line(points = {{-141, 0}, {-160, 0}}, color = {0, 0, 0}));
          connect(initStep.outPort[1], transition.inPort) annotation(
            Line(points = {{-119.5, 0}, {-24, 0}}, color = {0, 0, 0}));
          connect(transition.outPort, exitStep.inPort[1]) annotation(
            Line(points = {{-18.5, 0}, {109, 0}}, color = {0, 0, 0}));
        end CompositeStep2;
      end Utilities;
    end Examples;

    package Interfaces "Connectors and partial models"
      extends Modelica.Icons.InterfacesPackage;

      connector Step_in "Input port of a step"
        output Boolean occupied "true, if step is active" annotation(
          HideResult = true);
        input Boolean set "true, if transition fires and step is activated" annotation(
          HideResult = true);
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{-100, 100}, {100, 0}, {-100, -100}, {-100, 100}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid)}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{0, 50}, {100, 0}, {0, -50}, {0, 50}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Text(extent = {{-141, 100}, {100, 60}}, lineColor = {0, 0, 0}, textString = "%name")}));
      end Step_in;

      connector Step_out "Output port of a step"
        output Boolean available "true, if step is active" annotation(
          HideResult = true);
        input Boolean reset "true, if transition fires and step is deactivated" annotation(
          HideResult = true);
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, 100}, {100, -100}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid)}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, 50}, {0, -50}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-100, 100}, {186, 58}}, lineColor = {0, 0, 0}, textString = "%name")}));
      end Step_out;

      connector Transition_in "Input port of a transition"
        input Boolean available "true, if step connected to the transition input is active" annotation(
          HideResult = true);
        output Boolean reset "true, if transition fires and the step connected to the transition input is deactivated" annotation(
          HideResult = true);
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{-100, 100}, {100, 0}, {-100, -100}, {-100, 100}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid)}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{0, 50}, {100, 0}, {0, -50}, {0, 50}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Text(extent = {{-141, 100}, {100, 60}}, lineColor = {0, 0, 0}, textString = "%name")}));
      end Transition_in;

      connector Transition_out "Output port of a transition"
        input Boolean occupied "true, if step connected to the transition output is active" annotation(
          HideResult = true);
        output Boolean set "true, if transition fires and step connected to the transition output becomes active" annotation(
          HideResult = true);
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, 100}, {100, -100}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid)}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-100, 100}, {146, 60}}, lineColor = {0, 0, 0}, textString = "%name"), Rectangle(extent = {{-100, 50}, {0, -50}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid)}));
      end Transition_out;

      connector CompositeStep_resume "Input port of a step (used for resume connector of a CompositeStep)"
        output Boolean occupied "true, if step is active" annotation(
          HideResult = true);
        input Boolean set "true, if transition fires and step is activated" annotation(
          HideResult = true);
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{-100, 100}, {100, 0}, {-100, -100}, {-100, 100}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid)}));
      end CompositeStep_resume;

      connector CompositeStep_suspend "Output port of a step (used for suspend connector of a CompositeStep)"
        output Boolean available "true, if step is active" annotation(
          HideResult = true);
        input Boolean reset "true, if transition fires and step is deactivated" annotation(
          HideResult = true);
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, 100}, {100, -100}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid)}));
      end CompositeStep_suspend;

      connector CompositeStepStatePort_in "Communication port between a CompositeStep and the ordinary steps within the CompositeStep (suspend/resume are inputs)"
        input Boolean suspend "= true, if suspend transition of CompositeStep fires";
        input Boolean resume "= true, if resume transition of CompositeStep fires";
        Real activeStepsDummy "Dummy variable in order that connector fulfills restriction of connector" annotation(
          HideResult = true);
        flow Real activeSteps "Number of active steps in the CompositeStep";
      end CompositeStepStatePort_in;

      connector CompositeStepStatePort_out "Communication port between a CompositeStep and the ordinary steps within the CompositeStep (suspend/resume are outputs)"
        output Boolean suspend "= true, if suspend transition of CompositeStep fires";
        output Boolean resume "= true, if resume transition of CompositeStep fires";
        Real activeStepsDummy "Dummy variable in order that connector fulfills restriction of connector" annotation(
          HideResult = true);
        flow Real activeSteps "Number of active steps in the CompositeStep";
      end CompositeStepStatePort_out;

      partial block PartialStep "Partial step with one input and one output transition port"
        parameter Integer nIn(min = 0) = 1 "Number of input connections";
        parameter Integer nOut(min = 0) = 1 "Number of output connections";
        /* localActive is introduced since component 'Step' has Boolean variable 'active'
             and component 'StepWithSignal' has connector instance 'active' defined
             and both components inherit from PartialStep
          */
        output Boolean localActive "= true if step is active, otherwise the step is not active" annotation(
          HideResult = true);
        Interfaces.Step_in inPort[nIn] "Vector of step input connectors" annotation(
          Placement(transformation(extent = {{-120, 10}, {-100, -10}}, rotation = 0)));
        Interfaces.Step_out outPort[nOut] "Vector of step output connectors" annotation(
          Placement(transformation(extent = {{100, 5}, {110, -5}}, rotation = 0)));
      protected
        outer Interfaces.CompositeStepState stateGraphRoot;

        model OuterStatePort
          CompositeStepStatePort_in subgraphStatePort;
        end OuterStatePort;

        OuterStatePort outerStatePort;
        Boolean newActive "Value of active in the next iteration" annotation(
          HideResult = true);
        Boolean oldActive "Value of active when CompositeStep was aborted";
      initial equation
        pre(newActive) = pre(localActive);
        pre(oldActive) = pre(localActive);
      equation
        connect(outerStatePort.subgraphStatePort, stateGraphRoot.subgraphStatePort);
// Check that connections to the connector are correct
        for i in 1:nIn loop
          assert(cardinality(inPort[i]) <= 1, "Connector is connected to more than one transition (this is not allowed)");
        end for;
        for i in 1:nOut loop
          assert(cardinality(outPort[i]) <= 1, "Connector is connected to more than one transition (this is not allowed)");
        end for;
// set active state
        localActive = pre(newActive);
        newActive = if outerStatePort.subgraphStatePort.resume then oldActive else (StateGraph.Temporary.anyTrue(inPort.set) or localActive and not StateGraph.Temporary.anyTrue(outPort.reset)) and not outerStatePort.subgraphStatePort.suspend;
// Remember state for suspend action
        when outerStatePort.subgraphStatePort.suspend then
          oldActive = localActive;
        end when;
// Report state to CompositeStep
        outerStatePort.subgraphStatePort.activeSteps = if localActive then 1.0 else 0.0;
// Report state to input and output transitions
        for i in 1:nIn loop
          inPort[i].occupied = if i == 1 then localActive else inPort[i - 1].occupied or inPort[i - 1].set;
        end for;
        for i in 1:nOut loop
          outPort[i].available = if i == 1 then localActive else outPort[i - 1].available and not outPort[i - 1].reset;
        end for;
// Default setting, if an inPort or an outPort is not connected
        for i in 1:nIn loop
          if cardinality(inPort[i]) == 0 then
            inPort[i].set = false;
          end if;
        end for;
        for i in 1:nOut loop
          if cardinality(outPort[i]) == 0 then
            outPort[i].reset = false;
          end if;
        end for;
      end PartialStep;

      partial block PartialTransition "Partial transition with input and output connections"
        input Boolean localCondition "= true, if transition may fire" annotation(
          HideResult = true);
        parameter Boolean enableTimer = false "= true, if timer is enabled" annotation(
          Evaluate = true,
          Dialog(group = "Timer"));
        parameter Modelica.SIunits.Time waitTime(min = 0) = 0 "Wait time before transition fires" annotation(
          Dialog(group = "Timer", enable = enableTimer));
        output Modelica.SIunits.Time t "= actual waiting time (transition will fire when t > waitTime)";
        output Boolean enableFire "= true, if all firing conditions are true";
        output Boolean fire "= true, if transition fires" annotation(
          HideResult = true);
        StateGraph.Interfaces.Transition_in inPort "Vector of transition input connectors" annotation(
          Placement(transformation(extent = {{-50, -10}, {-30, 10}}, rotation = 0)));
        StateGraph.Interfaces.Transition_out outPort "Vector of transition output connectors" annotation(
          Placement(transformation(extent = {{10, -5}, {20, 5}}, rotation = 0)));
      protected
        Modelica.SIunits.Time t_start "Time instant at which the transition would fire, if waitTime would be zero";
        Real t_dummy;
      initial equation
        if enableTimer then
          pre(t_start) = time;
        end if;
        pre(enableFire) = false;
      equation
        assert(cardinality(inPort) == 1, "Connector inPort is not connected to exactly one other connector");
        assert(cardinality(outPort) == 1, "Connector outPort is not connected to exactly one other connector");
// Handling of Timer
        if enableTimer then
          when enableFire then
            t_start = time;
          end when;
          t_dummy = time - t_start;
          t = if enableFire then t_dummy else 0;
          fire = enableFire and time >= t_start + waitTime;
        else
          t_start = 0;
          t_dummy = 0;
          t = 0;
          fire = enableFire;
        end if;
// Determine fire setting and report it to the connected step
        enableFire = localCondition and inPort.available and not outPort.occupied;
        inPort.reset = fire;
        outPort.set = fire;
      end PartialTransition;

      partial block PartialStateGraphIcon "Icon for a StateGraph object"
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, 100}, {100, -100}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Text(extent = {{160, 110}, {-160, 150}}, textString = "%name", lineColor = {0, 0, 255})}));
      end PartialStateGraphIcon;

      model CompositeStepState "Communication channel between CompositeSteps and steps in the CompositeStep"
        output Boolean suspend = false;
        output Boolean resume = false;
        CompositeStepStatePort_out subgraphStatePort;
        /*
            missingInnerMessage="No \"stateGraphRoot\" component is defined on highest level
        of the StateGraph. A stateGraphRoot component is automatically introduced.
        In order to get rid of this message, drag StateGraph.StateGraphRoot into the
        top level your model.");
        */
      equation
        suspend = subgraphStatePort.suspend;
        resume = subgraphStatePort.resume;
        subgraphStatePort.activeStepsDummy = 0;
        annotation(
          defaultComponentName = "stateGraphRoot",
          defaultComponentPrefixes = "inner",
          missingInnerMessage = "A \"stateGraphRoot\" component was automatically introduced.");
      end CompositeStepState;
    end Interfaces;

    block InitialStep "Initial step (= step that is active when simulation starts)"
      output Boolean active "= true if step is active, otherwise the step is not active";
      extends Interfaces.PartialStep;
    initial equation
      active = true;
    equation
      active = localActive;
      annotation(
        Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-200, 110}, {200, 150}}, textString = "%name", lineColor = {0, 0, 255}), Rectangle(extent = {{-100, 100}, {100, -100}}, fillColor = DynamicSelect({255, 255, 255}, if active > 0.5 then {0, 255, 0} else {255, 255, 255}), lineColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-80, 80}, {80, -80}}, lineColor = {0, 0, 0})}),
        Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, 100}, {100, -100}}, lineColor = {0, 0, 0}), Rectangle(extent = {{-80, 80}, {80, -80}}, lineColor = {0, 0, 0})}));
    end InitialStep;

    block InitialStepWithSignal "Initial step (= step that is active when simulation starts). Connector 'active' is true when the step is active"
      extends Interfaces.PartialStep;
      Modelica.Blocks.Interfaces.BooleanOutput active annotation(
        Placement(transformation(origin = {0, -110}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
    initial equation
      active = true;
    equation
      active = localActive;
      annotation(
        Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, 100}, {100, -100}}, lineColor = {0, 0, 0}), Rectangle(extent = {{-80, 80}, {80, -80}}, lineColor = {0, 0, 0})}),
        Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-200, 110}, {200, 150}}, textString = "%name", lineColor = {0, 0, 255}), Rectangle(extent = {{-100, 100}, {100, -100}}, fillColor = DynamicSelect({255, 255, 255}, if active > 0.5 then {0, 255, 0} else {255, 255, 255}), lineColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Text(extent = {{-92, -50}, {94, -68}}, lineColor = {0, 0, 0}, textString = "active"), Rectangle(extent = {{-80, 80}, {80, -80}}, lineColor = {0, 0, 0})}));
    end InitialStepWithSignal;

    block Step "Ordinary step (= step that is not active when simulation starts)"
      output Boolean active "= true if step is active, otherwise the step is not active";
      extends Interfaces.PartialStep;
    initial equation
      active = false;
    equation
      active = localActive;
      annotation(
        Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-200, 110}, {200, 150}}, textString = "%name", lineColor = {0, 0, 255}), Rectangle(extent = {{-100, 100}, {100, -100}}, fillColor = DynamicSelect({255, 255, 255}, if active > 0.5 then {0, 255, 0} else {255, 255, 255}), lineColor = {0, 0, 0}, fillPattern = FillPattern.Solid)}),
        Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, 100}, {100, -100}}, lineColor = {0, 0, 0})}));
    end Step;

    block StepWithSignal "Ordinary step (= step that is not active when simulation starts). Connector 'active' is true when the step is active"
      extends Interfaces.PartialStep;
      Modelica.Blocks.Interfaces.BooleanOutput active annotation(
        Placement(transformation(origin = {0, -110}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
    initial equation
      active = false;
    equation
      active = localActive;
      annotation(
        Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, 100}, {100, -100}}, lineColor = {0, 0, 0})}),
        Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-200, 110}, {200, 150}}, textString = "%name", lineColor = {0, 0, 255}), Rectangle(extent = {{-100, 100}, {100, -100}}, fillColor = DynamicSelect({255, 255, 255}, if active > 0.5 then {0, 255, 0} else {255, 255, 255}), lineColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Text(extent = {{-92, -74}, {94, -92}}, lineColor = {0, 0, 0}, textString = "active")}));
    end StepWithSignal;

    block Transition "Transition where the fire condition is set by a modification of variable condition"
      input Boolean condition = true "= true, if transition may fire (time varying expression)" annotation(
        Dialog(group = "Fire condition"));
      extends Interfaces.PartialTransition(final localCondition = condition);
      annotation(
        Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-10, 100}, {10, -100}}, fillColor = DynamicSelect({0, 0, 0}, if enableFire > 0.5 then {0, 255, 0} else {0, 0, 0}), lineColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Line(points = {{-30, 0}, {-10, 0}}, color = {0, 0, 0}), Text(extent = {{200, 110}, {-200, 150}}, textString = "%name", lineColor = {0, 0, 255}), Text(extent = {{20, 20}, {200, 45}}, lineColor = DynamicSelect({0, 0, 0}, if enableTimer < 0.5 then {255, 255, 255} else {0, 0, 0}), textString = "%waitTime"), Text(extent = {{-200, -120}, {200, -145}}, lineColor = DynamicSelect({0, 0, 0}, if condition > 0.5 then {0, 255, 0} else {0, 0, 0}), textString = "%condition")}),
        Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-31, 0}, {-11, 0}}, color = {0, 0, 0}), Rectangle(extent = {{-10, 100}, {10, -100}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid)}));
    end Transition;

    block TransitionWithSignal "Transition where the fire condition is set by a Boolean input signal"
      Modelica.Blocks.Interfaces.BooleanInput condition annotation(
        Placement(transformation(origin = {0, -120}, extent = {{-20, -20}, {20, 20}}, rotation = 90)));
      extends Interfaces.PartialTransition(final localCondition = condition);
      annotation(
        Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{20, 20}, {200, 45}}, lineColor = DynamicSelect({0, 0, 0}, if enableTimer < 0.5 then {255, 255, 255} else {0, 0, 0}), textString = "%waitTime"), Rectangle(extent = {{-10, 100}, {10, -100}}, fillColor = DynamicSelect({0, 0, 0}, if enableFire > 0.5 then {0, 255, 0} else {0, 0, 0}), lineColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Line(points = {{-30, 0}, {-10, 0}}, color = {0, 0, 0}), Text(extent = {{200, 110}, {-200, 150}}, textString = "%name", lineColor = {0, 0, 255}), Ellipse(extent = {{7, -81}, {-7, -95}}, lineColor = DynamicSelect({0, 0, 0}, if condition > 0.5 then {0, 255, 0} else {0, 0, 0}), fillColor = DynamicSelect({0, 0, 0}, if condition > 0.5 then {0, 255, 0} else {0, 0, 0}), fillPattern = FillPattern.Solid)}),
        Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-31, 0}, {-11, 0}}, color = {0, 0, 0}), Rectangle(extent = {{-10, 100}, {10, -100}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid)}));
    end TransitionWithSignal;

    block Alternative "Alternative splitting of execution path (use component between two steps)"
      parameter Integer nBranches(min = 1) = 2 "Number of alternative branches";
      Interfaces.Transition_in inPort annotation(
        Placement(transformation(extent = {{-106, -3}, {-100, 3}}, rotation = 0)));
      Interfaces.Transition_out outPort annotation(
        Placement(transformation(extent = {{100, -2}, {104, 2}}, rotation = 0)));
      Step_in_forAlternative join[nBranches] annotation(
        Placement(transformation(extent = {{78, 100}, {80, -100}}, rotation = 0)));
      Step_out_forAlternative split[nBranches] annotation(
        Placement(transformation(extent = {{-78, 100}, {-80, -100}}, rotation = 0)));
    protected
      connector Step_in_forAlternative "Input port of a step (has special icon for usage in component 'Alternative')"
        output Boolean occupied "true, if step is active" annotation(
          HideResult = true);
        input Boolean set "true, if transition fires and step is activated" annotation(
          HideResult = true);
        annotation(
          Icon(coordinateSystem(extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, 100}, {100, -100}}, lineColor = {0, 0, 0}, fillColor = {175, 175, 175}, fillPattern = FillPattern.Solid)}),
          Diagram(coordinateSystem(extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, 100}, {100, -100}}, lineColor = {0, 0, 0}, fillColor = {175, 175, 175}, fillPattern = FillPattern.Solid)}));
      end Step_in_forAlternative;

      connector Step_out_forAlternative "Output port of a step (has special icon for usage in component 'Alternative')"
        output Boolean available "true, if step is active" annotation(
          HideResult = true);
        input Boolean reset "true, if transition fires and step is deactivated" annotation(
          HideResult = true);
        annotation(
          Icon(coordinateSystem(extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, 100}, {100, -100}}, lineColor = {0, 0, 0}, fillColor = {175, 175, 175}, fillPattern = FillPattern.Solid)}),
          Diagram(coordinateSystem(extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, 100}, {100, -100}}, lineColor = {0, 0, 0}, fillColor = {175, 175, 175}, fillPattern = FillPattern.Solid)}));
      end Step_out_forAlternative;
    equation
// Check connections of connectors
      assert(cardinality(inPort) == 1, "Connector inPort is not connected to exactly one other connector");
      assert(cardinality(outPort) == 1, "Connector outPort is not connected to exactly one other connector");
      for i in 1:nBranches loop
        assert(cardinality(split[i]) == 1, "Connector is not connected to exactly one other connector");
        assert(cardinality(join[i]) == 1, "Connector is not connected to exactly one other connector");
      end for;
// Propagate flags between the connectors
      for i in 1:nBranches loop
        split[i].available = if i == 1 then inPort.available else split[i - 1].available and not split[i - 1].reset;
      end for;
      join.occupied = fill(outPort.occupied, nBranches);
      inPort.reset = StateGraph.Temporary.anyTrue(split.reset);
      outPort.set = StateGraph.Temporary.anyTrue(join.set);
      annotation(
        Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-80, 100}, {80, 100}}, color = {0, 0, 0}, pattern = LinePattern.Dot), Line(points = {{-80, -100}, {80, -100}}, color = {0, 0, 0}, pattern = LinePattern.Dot), Line(points = {{-100, 0}, {-80, 0}}, color = {0, 0, 0}), Line(points = {{80, 0}, {100, 0}}, color = {0, 0, 0})}),
        Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-100, 0}, {-80, 0}}, color = {0, 0, 0}), Line(points = {{80, 0}, {100, 0}}, color = {0, 0, 0})}));
    end Alternative;

    block Parallel "Parallel splitting of execution path (use component between two transitions)"
      parameter Integer nBranches(min = 1) = 2 "Number of parallel branches that are executed in parallel";
      Interfaces.Step_in inPort annotation(
        Placement(transformation(extent = {{-106, -3}, {-100, 3}}, rotation = 0)));
      Interfaces.Step_out outPort annotation(
        Placement(transformation(extent = {{100, -2}, {104, 2}}, rotation = 0)));
      Transition_in_forParallel join[nBranches] annotation(
        Placement(transformation(extent = {{75, 100}, {80, -100}}, rotation = 0)));
      Transition_out_forParallel split[nBranches] annotation(
        Placement(transformation(extent = {{-75, 100}, {-80, -100}}, rotation = 0)));
    protected
      connector Transition_in_forParallel "Input port of a transition (has special icon for usage in component 'Parallel')"
        input Boolean available "true, if step connected to the transition input is active" annotation(
          HideResult = true);
        output Boolean reset "true, if transition fires and the step connected to the transition input is deactivated" annotation(
          HideResult = true);
        annotation(
          Icon(coordinateSystem(extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, 100}, {100, -100}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-100, 100}, {100, -100}}, lineColor = {255, 255, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{-100, 100}, {-100, -100}}, color = {0, 0, 0}, thickness = 0.5), Line(points = {{100, 100}, {100, -100}}, color = {0, 0, 0}, thickness = 0.5)}),
          Diagram(coordinateSystem(extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, 100}, {100, -100}}, lineColor = {255, 255, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{-100, 100}, {-100, -100}}, color = {0, 0, 0}, thickness = 0.5), Line(points = {{100, 100}, {100, -100}}, color = {0, 0, 0}, thickness = 0.5)}));
      end Transition_in_forParallel;

      connector Transition_out_forParallel "Output port of a transition (has special icon for usage in component 'Parallel')"
        input Boolean occupied "true, if step connected to the transition output is active" annotation(
          HideResult = true);
        output Boolean set "true, if transition fires and step connected to the transition output becomes active" annotation(
          HideResult = true);
        annotation(
          Icon(coordinateSystem(extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, 100}, {100, -100}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-100, 100}, {100, -100}}, lineColor = {255, 255, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{-100, 100}, {-100, -100}}, color = {0, 0, 0}, thickness = 0.5), Line(points = {{100, 100}, {100, -100}}, color = {0, 0, 0}, thickness = 0.5)}),
          Diagram(coordinateSystem(extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, 100}, {100, -100}}, lineColor = {255, 255, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{-100, 100}, {-100, -100}}, color = {0, 0, 0}, thickness = 0.5), Line(points = {{100, 100}, {100, -100}}, color = {0, 0, 0}, thickness = 0.5)}));
      end Transition_out_forParallel;
    equation
// Check connections of connectors
      assert(cardinality(inPort) == 1, "Connector inPort is not connected to exactly one other connector");
      assert(cardinality(outPort) == 1, "Connector outPort is not connected to exactly one other connector");
      for i in 1:nBranches loop
        assert(cardinality(split[i]) == 1, "Connector is not connected to exactly one other connector");
        assert(cardinality(join[i]) == 1, "Connector is not connected to exactly one other connector");
      end for;
// Propagate flags between the connectors
      split.set = fill(inPort.set, nBranches);
      join.reset = fill(outPort.reset, nBranches);
      inPort.occupied = StateGraph.Temporary.anyTrue(split.occupied);
      outPort.available = StateGraph.Temporary.allTrue(join.available);
      annotation(
        Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-100, 0}, {-80, 0}}, color = {0, 0, 0}), Line(points = {{80, 0}, {100, 0}}, color = {0, 0, 0}), Line(points = {{-80, 100}, {80, 100}}, color = {0, 0, 0}, pattern = LinePattern.Dot), Line(points = {{-80, -100}, {80, -100}}, color = {0, 0, 0}, pattern = LinePattern.Dot)}),
        Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-100, 0}, {-80, 0}}, color = {0, 0, 0}), Line(points = {{80, 0}, {100, 0}}, color = {0, 0, 0})}));
    end Parallel;

    partial model PartialCompositeStep "Superclass of a subgraph, i.e., a composite step that has internally a StateGraph"
      parameter Integer nSuspend = 1 "Number of suspend ports" annotation(
        Dialog(group = "Exception connections"));
      parameter Integer nResume = 1 "Number of resume ports" annotation(
        Dialog(group = "Exception connections"));
      /* The modification of stateGraphRoot is with respect to the "inner"
           definition, i.e., it is reported to all components that are
           within the CompositeStep
        */
      inner outer StateGraph.Interfaces.CompositeStepState stateGraphRoot(suspend = StateGraph.Temporary.anyTrue(suspend.reset) or outerState.subgraphStatePort.suspend, resume = StateGraph.Temporary.anyTrue(resume.set) or outerState.subgraphStatePort.resume) "Communication port between the CompositeStep and the steps within the CompositeStep";
      output Boolean active "= true if step is active, otherwise the step is not active";
      StateGraph.Interfaces.Step_in inPort annotation(
        Placement(transformation(extent = {{-170, 10}, {-150, -10}}, rotation = 0)));
      StateGraph.Interfaces.Step_out outPort annotation(
        Placement(transformation(extent = {{150, 5}, {160, -5}}, rotation = 0)));
      StateGraph.Interfaces.CompositeStep_suspend suspend[nSuspend] annotation(
        Placement(transformation(origin = {-75, -155}, extent = {{-5, 5}, {5, -5}}, rotation = 270)));
      StateGraph.Interfaces.CompositeStep_resume resume[nResume] annotation(
        Placement(transformation(origin = {75, -160}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));

      model OuterState "Block containing the port that is connected to the outer stateGraphRoot"
        Interfaces.CompositeStepStatePort_in subgraphStatePort "Port connected to outer stateGraphRoot";
      end OuterState;

      OuterState outerState;
    protected
      model InnerState
        outer Interfaces.CompositeStepState stateGraphRoot;
      end InnerState;

      InnerState innerState;
      Boolean newActive "Value of active in the next iteration" annotation(
        HideResult = true);
      Integer activeSteps "Number of active steps within the CompositeStep";
    initial equation
      pre(newActive) = false;
// pre(active) = pre(newActive);
    equation
// connect to outer CompositeStep
      connect(outerState.subgraphStatePort, stateGraphRoot.subgraphStatePort);
      outerState.subgraphStatePort.activeSteps = if active then 1.0 else 0.0;
// set active flag  of CompositeStep
      activeSteps = -integer(innerState.stateGraphRoot.subgraphStatePort.activeSteps);
      active = pre(newActive);
/* The CompositeStep is active if
     - at least one step within the CompositeStep is active, and
     - the suspend transition does not fire, and
     - the suspend transition of a higher level CompositeStep does not fire or
     - no step within the CompositeStep is active, and
     - the resume transition fires or the resume transition of
       a higher level CompositeStep fires.
  */
// newActive = activeSteps > 0 and not suspend.reset or resume.set;
      newActive = activeSteps > 0 and not StateGraph.Temporary.anyTrue(suspend.reset) and not outerState.subgraphStatePort.suspend or StateGraph.Temporary.anyTrue(resume.set) or outerState.subgraphStatePort.resume;
// Report state to suspend and resume transitions
      for i in 1:nResume loop
        resume[i].occupied = if i == 1 then active else resume[i - 1].occupied or resume[i - 1].set;
      end for;
      for i in 1:nSuspend loop
        suspend[i].available = if i == 1 then active else suspend[i - 1].available and not suspend[i - 1].reset;
      end for;
/* Check that connections to the connectors are correct
     and set appropriate defaults if necessary
  */
      for i in 1:nSuspend loop
        assert(cardinality(suspend[i]) <= 1, "Connector suspend[" + String(i) + "] of the CompositeStep is connected
to more than one transition");
        if cardinality(suspend[i]) == 0 then
          suspend[i].reset = false;
        end if;
      end for;
      for i in 1:nResume loop
        assert(cardinality(resume[i]) <= 1, "Connector resume[" + String(i) + "] of the CompositeStep is connected
to more than one transition");
        if cardinality(resume[i]) == 0 then
          resume[i].set = false;
        end if;
      end for;
/* Zero sized connectors are not yet fully supported in
     Dymola. This requires to set the dimension of the connector
     to 1, if it should have a dimension of zero. This requires
     to set the connector variables to a default value in this case
  */
      if cardinality(inPort) < 2 then
        inPort.occupied = false;
        inPort.set = false;
      end if;
      if cardinality(outPort) < 2 then
        outPort.available = false;
        outPort.reset = false;
      end if;
// Check inPort/outPort connections
      assert(cardinality(inPort) <= 2, "Connector inPort of the CompositeStep has more than 2 connections.
It should have only one connection from the outside to the
inPort and one connection to a step inside the CompositeStep.");
      assert(cardinality(outPort) <= 2, "Connector outPort of the CompositeStep has more than 2 connections.
It should have only one connection from the outPort to the
outside to the CompositeStep and one connection from a step
inside the CompositeStep to the outPort connector.");
      annotation(
        Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-150, -150}, {150, 150}}), graphics = {Text(extent = {{-250, 160}, {250, 200}}, textString = "%name", lineColor = {0, 0, 255}), Rectangle(extent = {{-150, 150}, {150, -150}}, fillColor = DynamicSelect({255, 255, 255}, if active > 0.5 then {0, 255, 0} else {255, 255, 255}), lineColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Text(extent = {{4, -115}, {145, -130}}, lineColor = {0, 0, 0}, textString = "resume"), Text(extent = {{-144, -114}, {-3, -129}}, lineColor = {0, 0, 0}, textString = "suspend")}),
        Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-150, -150}, {150, 150}}), graphics = {Rectangle(extent = {{-150, 150}, {150, -150}}, lineColor = {0, 0, 0})}));
    end PartialCompositeStep;

    model StateGraphRoot "Root of a StateGraph (has to be present on the highest level of a StateGraph)"
      extends StateGraph.Interfaces.CompositeStepState;
      output Integer activeSteps "Number of active steps within the stategraph";
    equation
      activeSteps = -integer(subgraphStatePort.activeSteps);
      annotation(
        defaultComponentName = "stateGraphRoot",
        defaultComponentPrefixes = "inner",
        Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-200, 110}, {200, 150}}, textString = "%name", lineColor = {0, 0, 255}), Rectangle(extent = {{-100, 100}, {100, -100}}, lineColor = {0, 0, 0}), Text(extent = {{-92, 78}, {96, 34}}, lineColor = {0, 0, 0}, textString = "root"), Rectangle(extent = {{-82, -6}, {-44, -40}}, lineColor = {0, 0, 0}), Line(points = {{0, 10}, {0, -60}}, color = {0, 0, 0}), Rectangle(extent = {{48, -6}, {86, -40}}, lineColor = {0, 0, 0}), Polygon(points = {{-12, -16}, {0, -22}, {-12, -28}, {-12, -16}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Line(points = {{-44, -22}, {-12, -22}}, color = {0, 0, 0}), Polygon(points = {{36, -16}, {48, -22}, {36, -28}, {36, -16}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Line(points = {{0, -22}, {36, -22}}, color = {0, 0, 0})}),
        Documentation(info = "<html>
<p>
On the highest level of a StateGraph, an instance of StateGraphRoot
has to be present.
</p>
<p>
The StateGraphRoot object is needed, since all Step objects have
an \"outer\" reference to communicate with the \"nearest\" CompositeStep
(which inherits from PartialCompositeStep), especially to abort
a CompositeStep via the \"suspend\" port. Even if no \"CompositeStep\" is present,
on highest level a corresponding \"inner\" definition is needed
and is provided by the StateGraphRoot object.
</p>
</html>"));
    end StateGraphRoot;

    package Temporary "Components that will be provided by other libraries in the future"
      type SetRealParameter = Real "Define Real parameter (GUI not yet satisfactory)" annotation(
        Dialog,
        defaultComponentName = "name",
        defaultComponentPrefixes = "parameter",
        Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, 40}, {100, -40}}, lineColor = {0, 0, 0}, fillColor = {245, 245, 245}, fillPattern = FillPattern.Solid, borderPattern = BorderPattern.Raised), Text(extent = {{-150, 90}, {150, 50}}, textString = "%name ="), Text(extent = {{-96, 15}, {96, -15}}, lineColor = {0, 0, 0}, textString = "%value")}),
        Documentation(info = "<html>
<p>
This is an <b>experimental component</b> to define a
<b>Real parameter</b>
in the <b>diagram layer</b>. The idea is to drag the icon from the
package browser into the diagram layer. Then a window pops
up in which the properties of this parameter can be defined
(such as the default value). The name and default value of the
parameter are displayed in the icon of this component. Whenever
clicking on it, the dialog to change parameter settings pops-up.
</p>
<p>
In Dymola, the described property is not fully available.
Currently, when dragging this component in the diagram layer,
a dialog pops up in which the properties of the parameter
can be defined. However, afterwards, the parameter is not
visible in the diagram layer. Making it visible requires to
go into the text layer and add an annotation with the
component size, resulting for example in:
</p>
<pre>
  <b>parameter</b> StateGraph.SetRealParameter name = 2
                       <b>annotation</b>(extent=[-10,-10; 10,10]);
</pre>
<p>
This change makes the parameter icon visible in the
diagram layer. However, clicking on this icon has no
effect. Changing parameter properties, such as the default
value, still requires to go in to the text layer.
</p>
</html>"));
      extends Modelica.Icons.Package;

      function anyTrue "Returns true, if at least on element of the Boolean input vector is true"
        extends Modelica.Icons.Function;
        input Boolean b[:];
        output Boolean result;
      algorithm
        result := false;
        for i in 1:size(b, 1) loop
          result := result or b[i];
        end for;
      end anyTrue;

      function allTrue "Returns true, if all elements of the Boolean input vector are true"
        extends Modelica.Icons.Function;
        input Boolean b[:];
        output Boolean result;
      algorithm
        result := true;
        for i in 1:size(b, 1) loop
          result := result and b[i];
        end for;
      end allTrue;

      block RadioButton "Button that sets its output to true when pressed and is reset when an element of 'reset' becomes true"
        parameter Modelica.SIunits.Time buttonTimeTable[:] = {0} "Time instants where button is pressed and released";
        input Boolean reset[:] = {false} "Reset button to false, if an element of reset becomes true" annotation(
          Dialog(group = "Time varying expressions"));
        Modelica.Blocks.Interfaces.BooleanOutput on annotation(
          Placement(transformation(extent = {{100, -10}, {120, 10}}, rotation = 0)));
      protected
        Modelica.Blocks.Sources.BooleanTable table(table = buttonTimeTable);
      initial equation
        pre(reset) = fill(false, size(reset, 1));
      algorithm
        on := table.y;
        when pre(reset) then
          on := false;
        end when;
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, -100}, {100, 100}}, fillColor = DynamicSelect({192, 192, 192}, if on then {0, 255, 0} else {192, 192, 192}), fillPattern = FillPattern.Solid, lineColor = {128, 128, 128}, lineThickness = 0.5), Text(extent = {{-80, -40}, {80, 40}}, lineColor = {0, 0, 0}, textString = "%name")}, interaction = {OnMouseDownSetBoolean(on, true)}));
      end RadioButton;

      model NumericValue "Show value of Real input signal dynamically"
        parameter Integer precision(min = 0) = 3 "Number of significant digits to be shown";
        parameter Boolean hideConnector = false "= true, if connector is not shown in the dynamic object diagram";
        Modelica.Blocks.Interfaces.RealInput Value "Real value to be shown in icon" annotation(
          Dialog(enable = hideConnector),
          Placement(transformation(extent = {{-140, -20}, {-100, 20}}, rotation = 0)));
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{100, 50}, {-100, -50}}, lineColor = {0, 0, 255}, fillColor = {236, 233, 216}, fillPattern = FillPattern.Solid, borderPattern = BorderPattern.Raised), Text(extent = {{-90, -46}, {90, 34}}, textString = DynamicSelect(" ", String(Value, minimumLength = 1, significantDigits = precision)), lineColor = {0, 0, 255})}));
      end NumericValue;

      model IndicatorLamp "Dynamically show Boolean input signal (false/true = white/green color)"
        Modelica.Blocks.Interfaces.BooleanInput u annotation(
          Placement(transformation(extent = {{-140, -20}, {-100, 20}}, rotation = 0)));
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-100, -100}, {100, 100}}, fillColor = DynamicSelect({235, 235, 235}, if u > 0.5 then {0, 255, 0} else {235, 235, 235}), lineColor = {0, 0, 0}, pattern = LinePattern.None, fillPattern = FillPattern.Sphere), Text(extent = {{-150, 150}, {150, 110}}, textString = "%name", lineColor = {0, 0, 255})}));
      end IndicatorLamp;
      annotation(
        Documentation(info = "<html>
<p>
This library is just temporarily present. The components of
this library will be present in the future in the Modelica
standard library (with the new block connectors) and in the
UserInteraction library that is currently under development.
</p>
</html>"));
    end Temporary;
    annotation(
      Documentation(info = "<html>
<p>
Note, there is a much improved version of this library called
\"Modelica_StateGraph2\". If this library is not yet distributed with your
Modelica tool, you can download it from
<a href=\"https://www.modelica.org/libraries/Modelica_StateGraph2\">https://www.modelica.org/libraries/Modelica_StateGraph2</a>.
In the
<a href=\"modelica://Modelica.StateGraph.UsersGuide.ComparisonWithStateGraph2\">Users Guide</a>
a detailed comparison is given. It is highly recommended to use Modelica_StateGraph2 instead
of Modelica.StateGraph.
</p>

<p>
Library <b>StateGraph</b> is a <b>free</b> Modelica package providing
components to model <b>discrete event</b> and <b>reactive</b>
systems in a convenient
way. It is based on the JGrafchart method and
takes advantage of Modelica features for
the \"action\" language. JGrafchart is a further development of
Grafcet to include elements of StateCharts that are not present
in Grafcet/Sequential Function Charts. Therefore, the StateGraph
library has a similar modeling power as StateCharts but avoids
some deficiencies of StateCharts.
</p>
<p>
For an introduction, have especially a look at:
</p>
<ul>
<li> <a href=\"modelica://Modelica.StateGraph.UsersGuide\">StateGraph.UsersGuide</a>
     discusses the most important aspects how to use this library.</li>
<li> <a href=\"modelica://Modelica.StateGraph.Examples\">StateGraph.Examples</a>
     contains examples that demonstrate the usage of this library.</li>
</ul>
<p>
A typical model generated with this library is shown
in the next figure where on the left hand side a two-tank
system with a tank controller and on the right hand side the
top-level part of the tank controller as a StateGraph is shown:
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/StateGraph/Examples/ControlledTanks1_small.png\">

<img src=\"modelica://Modelica/Resources/Images/StateGraph/Examples/ControlledTanks2_small.png\">
</p>

<p>
The unique feature of the StateGraph library with respect to JGrafcharts,
Grafcet, Sequential Function Charts, and StateCharts, is Modelica's
\"single assignment rule\" that requires that every variable is defined
by exactly one equation. This leads to a different \"action\" definition
as in these formalisms. The advantage is that the translator can either
determine a useful evaluation sequence by equation sorting or
reports an error if this is not possible, e.g., because a model
would lead to a non-determinism or to a dead-lock. As a side effect,
this leads also to simpler and more easier to understand models and
global variables are no longer needed (whereas in JGrafcharts,
Grafcet, Sequential Function Charts and StateCharts global variables
are nearly always needed).
</p>

<p>
Copyright &copy; 1998-2015, Modelica Association and DLR
</p>
<p>
<i>This Modelica package is <u>free</u> software and the use is completely at <u>your own risk</u>; it can be redistributed and/or modified under the terms of the Modelica License 2. For license conditions (including the disclaimer of warranty) see <a href=\"modelica://Modelica.UsersGuide.ModelicaLicense2\">Modelica.UsersGuide.ModelicaLicense2</a> or visit <a href=\"https://www.modelica.org/licenses/ModelicaLicense2\"> https://www.modelica.org/licenses/ModelicaLicense2</a>.</i>
</p>
</html>"),
      Icon(coordinateSystem(extent = {{-100.0, -100.0}, {100.0, 100.0}}), graphics = {Rectangle(origin = {-70.0, -0.0}, fillColor = {255, 255, 255}, extent = {{-20.0, -20.0}, {20.0, 20.0}}), Rectangle(origin = {70.0, -0.0}, fillColor = {255, 255, 255}, extent = {{-20.0, -20.0}, {20.0, 20.0}}), Line(points = {{0.0, 50.0}, {0.0, -50.0}}), Polygon(origin = {-16.6667, 0.0}, pattern = LinePattern.None, fillPattern = FillPattern.Solid, points = {{-3.3333, 10.0}, {16.667, 0.0}, {-3.3333, -10.0}}), Line(origin = {-35.0, 0.0}, points = {{15.0, 0.0}, {-15.0, 0.0}}), Polygon(origin = {33.3333, 0.0}, pattern = LinePattern.None, fillPattern = FillPattern.Solid, points = {{-3.3333, 10.0}, {16.667, 0.0}, {-3.3333, -10.0}}), Line(origin = {15.0, -0.0}, points = {{15.0, 0.0}, {-15.0, -0.0}})}));
  end StateGraph;

  package Electrical "Library of electrical models (analog, digital, machines, multi-phase)"
    extends Modelica.Icons.Package;

    package Analog "Library for analog electrical models"
      import SI = Modelica.SIunits;
      extends Modelica.Icons.Package;

      package Examples "Examples that demonstrate the usage of the Analog electrical components"
        extends Modelica.Icons.ExamplesPackage;

        model CauerLowPassAnalog "Cauer low pass filter with analog components"
          extends Modelica.Icons.Example;
          parameter Modelica.SIunits.Inductance l1 = 1.304 "filter coefficient I1";
          parameter Modelica.SIunits.Inductance l2 = 0.8586 "filter coefficient I2";
          parameter Modelica.SIunits.Capacitance c1 = 1.072 "filter coefficient c1";
          parameter Modelica.SIunits.Capacitance c2 = 1 / (1.704992 ^ 2 * l1) "filter coefficient c2";
          parameter Modelica.SIunits.Capacitance c3 = 1.682 "filter coefficient c3";
          parameter Modelica.SIunits.Capacitance c4 = 1 / (1.179945 ^ 2 * l2) "filter coefficient c4";
          parameter Modelica.SIunits.Capacitance c5 = 0.7262 "filter coefficient c5";
          Modelica.Electrical.Analog.Basic.Ground G annotation(
            Placement(transformation(extent = {{-10, -90}, {10, -70}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Capacitor C1(C = c1, v(start = 0, fixed = true)) annotation(
            Placement(transformation(origin = {-60, -20}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          Modelica.Electrical.Analog.Basic.Capacitor C2(C = c2) annotation(
            Placement(transformation(extent = {{-40, 20}, {-20, 40}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Capacitor C3(C = c3, v(start = 0, fixed = true)) annotation(
            Placement(transformation(origin = {0, -20}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          Modelica.Electrical.Analog.Basic.Capacitor C4(C = c4) annotation(
            Placement(transformation(extent = {{20, 20}, {40, 40}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Capacitor C5(C = c5, v(start = 0, fixed = true)) annotation(
            Placement(transformation(origin = {60, -20}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          Modelica.Electrical.Analog.Basic.Inductor L1(L = l1, i(start = 0, fixed = true)) annotation(
            Placement(transformation(extent = {{-40, 60}, {-20, 80}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Inductor L2(L = l2, i(start = 0, fixed = true)) annotation(
            Placement(transformation(extent = {{20, 60}, {40, 80}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Resistor R1(R = 1) annotation(
            Placement(transformation(extent = {{-100, 20}, {-80, 40}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Resistor R2(R = 1) annotation(
            Placement(transformation(origin = {100, -20}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          Modelica.Electrical.Analog.Sources.StepVoltage V(startTime = 1, offset = 0, V = 1) annotation(
            Placement(transformation(origin = {-100, -10}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
        equation
          connect(R1.n, C1.p) annotation(
            Line(points = {{-80, 30}, {-60, 30}, {-60, -10}}, color = {0, 0, 255}));
          connect(C1.n, G.p) annotation(
            Line(points = {{-60, -30}, {-60, -50}, {0, -50}, {0, -70}}, color = {0, 0, 255}));
          connect(R1.n, C2.p) annotation(
            Line(points = {{-80, 30}, {-60, 30}, {-40, 30}}, color = {0, 0, 255}));
          connect(L1.p, C2.p) annotation(
            Line(points = {{-40, 70}, {-40, 30}}, color = {0, 0, 255}));
          connect(L1.p, C1.p) annotation(
            Line(points = {{-40, 70}, {-40, 30}, {-60, 30}, {-60, -10}}, color = {0, 0, 255}));
          connect(L1.n, C2.n) annotation(
            Line(points = {{-20, 70}, {-20, 30}}, color = {0, 0, 255}));
          connect(C2.n, C3.p) annotation(
            Line(points = {{-20, 30}, {0, 30}, {0, -10}}, color = {0, 0, 255}));
          connect(C2.n, C4.p) annotation(
            Line(points = {{-20, 30}, {20, 30}}, color = {0, 0, 255}));
          connect(L1.n, C3.p) annotation(
            Line(points = {{-20, 70}, {-20, 30}, {0, 30}, {0, -10}}, color = {0, 0, 255}));
          connect(L1.n, C4.p) annotation(
            Line(points = {{-20, 70}, {-20, 30}, {20, 30}}, color = {0, 0, 255}));
          connect(L2.p, C4.p) annotation(
            Line(points = {{20, 70}, {20, 30}}, color = {0, 0, 255}));
          connect(C2.n, L2.p) annotation(
            Line(points = {{-20, 30}, {20, 30}, {20, 70}}, color = {0, 0, 255}));
          connect(C3.p, L2.p) annotation(
            Line(points = {{0, -10}, {0, -10}, {0, 30}, {20, 30}, {20, 70}}, color = {0, 0, 255}));
          connect(L2.n, C4.n) annotation(
            Line(points = {{40, 70}, {40, 30}}, color = {0, 0, 255}));
          connect(L2.n, C5.p) annotation(
            Line(points = {{40, 70}, {40, 30}, {60, 30}, {60, -10}}, color = {0, 0, 255}));
          connect(L2.n, R2.p) annotation(
            Line(points = {{40, 70}, {40, 30}, {100, 30}, {100, -10}}, color = {0, 0, 255}));
          connect(R2.n, G.p) annotation(
            Line(points = {{100, -30}, {100, -50}, {0, -50}, {0, -70}}, color = {0, 0, 255}));
          connect(C4.n, C5.p) annotation(
            Line(points = {{40, 30}, {60, 30}, {60, -10}}, color = {0, 0, 255}));
          connect(C4.n, R2.p) annotation(
            Line(points = {{40, 30}, {100, 30}, {100, -10}}, color = {0, 0, 255}));
          connect(C3.n, G.p) annotation(
            Line(points = {{0, -30}, {0, -30}, {0, -70}}, color = {0, 0, 255}));
          connect(C5.n, G.p) annotation(
            Line(points = {{60, -30}, {60, -50}, {0, -50}, {0, -70}}, color = {0, 0, 255}));
          connect(C1.n, C3.n) annotation(
            Line(points = {{-60, -30}, {-60, -50}, {0, -50}, {0, -30}}, color = {0, 0, 255}));
          connect(C1.n, C5.n) annotation(
            Line(points = {{-60, -30}, {-60, -50}, {60, -50}, {60, -30}}, color = {0, 0, 255}));
          connect(R2.n, C5.n) annotation(
            Line(points = {{100, -30}, {100, -50}, {60, -50}, {60, -30}}, color = {0, 0, 255}));
          connect(R2.n, C3.n) annotation(
            Line(points = {{100, -30}, {100, -50}, {0, -50}, {0, -30}}, color = {0, 0, 255}));
          connect(R2.n, C1.n) annotation(
            Line(points = {{100, -30}, {100, -50}, {-60, -50}, {-60, -30}}, color = {0, 0, 255}));
          connect(C5.p, R2.p) annotation(
            Line(points = {{60, -10}, {60, 30}, {100, 30}, {100, -10}}, color = {0, 0, 255}));
          connect(R1.p, V.p) annotation(
            Line(points = {{-100, 30}, {-100, 0}}, color = {0, 0, 255}));
          connect(V.n, G.p) annotation(
            Line(points = {{-100, -20}, {-100, -70}, {0, -70}}, color = {0, 0, 255}));
          annotation(
            Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{-120, -100}, {120, 100}}), graphics = {Rectangle(extent = {{-62, 32}, {-58, 28}}, lineColor = {0, 0, 255}, fillColor = {85, 85, 255}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-2, 28}, {2, 32}}, lineColor = {0, 0, 255}, fillColor = {85, 85, 255}, fillPattern = FillPattern.Solid), Rectangle(extent = {{58, 32}, {62, 28}}, lineColor = {0, 0, 255}, fillColor = {85, 85, 255}, fillPattern = FillPattern.Solid), Rectangle(extent = {{58, -48}, {62, -52}}, lineColor = {0, 0, 255}, fillColor = {85, 85, 255}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-2, -48}, {2, -52}}, lineColor = {0, 0, 255}, fillColor = {85, 85, 255}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-62, -48}, {-58, -52}}, lineColor = {0, 0, 255}, fillColor = {85, 85, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-120, 100}, {120, 80}}, textString = "CauerLowPassAnalog", lineColor = {0, 0, 255})}),
            experiment(StopTime = 60),
            Documentation(revisions = "<html>
<ul>
<li><i>January 13, 2006</i>
       by Christoph Clauss<br>
       included into Modelica Standard Library</li>
<li><i>September 15, 2005</i>
       by Peter Trappe designed and by Teresa Schlegel<br>
       initially modelled.</li>
</ul>
</html>", info = "<html>
<p>The example Cauer Filter is a low-pass-filter of the fifth order. It is realized using an analog network. The voltage source V is the input voltage (step), and the R2.p.v is the filter output voltage. The pulse response is calculated.</p>
<p>The simulation end time should be 60. Please plot both V.p.v (input voltage) and R2.p.v (output voltage).</p>
</html>"));
        end CauerLowPassAnalog;

        model CauerLowPassOPV "Cauer low pass filter with operational amplifiers"
          extends Modelica.Icons.Example;
          parameter Modelica.SIunits.Capacitance l1 = 1.304 "filter coefficient i1";
          parameter Modelica.SIunits.Capacitance l2 = 0.8586 "filter coefficient i2";
          parameter Modelica.SIunits.Capacitance c1 = 1.072 "filter coefficient c1";
          parameter Modelica.SIunits.Capacitance c2 = 1 / (1.704992 ^ 2 * l1) "filter coefficient c2";
          parameter Modelica.SIunits.Capacitance c3 = 1.682 "filter coefficient c3";
          parameter Modelica.SIunits.Capacitance c4 = 1 / (1.179945 ^ 2 * l2) "filter coefficient c4";
          parameter Modelica.SIunits.Capacitance c5 = 0.7262 "filter coefficient c5";
          Modelica.Electrical.Analog.Basic.Capacitor C1(C = c1 + c2, v(start = 0, fixed = true)) annotation(
            Placement(transformation(extent = {{-192, 10}, {-172, 30}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Capacitor C2(C = c2, v(start = 0, fixed = true)) annotation(
            Placement(transformation(extent = {{-240, -90}, {-220, -70}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Capacitor C3(C = l1, v(start = 0, fixed = true)) annotation(
            Placement(transformation(extent = {{-90, 10}, {-70, 30}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Capacitor C4(C = c4, v(start = 0, fixed = true)) annotation(
            Placement(transformation(extent = {{-30, 30}, {-10, 50}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Capacitor C5(C = c2) annotation(
            Placement(transformation(extent = {{-30, -120}, {-10, -100}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Resistor R1(R = 1) annotation(
            Placement(transformation(extent = {{-240, -50}, {-220, -30}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Resistor R2(R = 1) annotation(
            Placement(transformation(extent = {{-240, -10}, {-220, 10}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Resistor R3(R = 1) annotation(
            Placement(transformation(extent = {{-193, 50}, {-173, 70}}, rotation = 0)));
          Modelica.Electrical.Analog.Ideal.IdealOpAmp3Pin Op1 annotation(
            Placement(transformation(extent = {{-200, -70}, {-160, -30}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Ground G annotation(
            Placement(transformation(extent = {{-188, -96}, {-174, -80}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Resistor R4(R = -1) annotation(
            Placement(transformation(extent = {{-140, -60}, {-120, -40}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Resistor R5(R = -1) annotation(
            Placement(transformation(extent = {{-140, -100}, {-120, -80}}, rotation = 0)));
          Modelica.Electrical.Analog.Ideal.IdealOpAmp3Pin Op2 annotation(
            Placement(transformation(extent = {{-100, -70}, {-60, -30}}, rotation = 0)));
          Modelica.Electrical.Analog.Ideal.IdealOpAmp3Pin Op3 annotation(
            Placement(transformation(extent = {{4, -72}, {46, -30}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Ground G1 annotation(
            Placement(transformation(extent = {{-87, -96}, {-73, -80}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Resistor R6(R = 1) annotation(
            Placement(transformation(extent = {{-31, -10}, {-11, 10}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Resistor R7(R = 1) annotation(
            Placement(transformation(extent = {{-31, -60}, {-11, -40}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Capacitor C6(C = c2 + c3 + c4) annotation(
            Placement(transformation(extent = {{9, 10}, {29, 30}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Resistor R8(R = -1) annotation(
            Placement(transformation(extent = {{60, -10}, {80, 10}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Resistor R9(R = -1) annotation(
            Placement(transformation(extent = {{60, -60}, {80, -40}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Resistor R10(R = 1) annotation(
            Placement(transformation(extent = {{160, -60}, {180, -40}}, rotation = 0)));
          Modelica.Electrical.Analog.Ideal.IdealOpAmp3Pin Op4 annotation(
            Placement(transformation(extent = {{100, -70}, {140, -30}}, rotation = 0)));
          Modelica.Electrical.Analog.Ideal.IdealOpAmp3Pin Op5 annotation(
            Placement(transformation(extent = {{200, -70}, {240, -30}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Capacitor C7(C = l2, v(start = 0, fixed = true)) annotation(
            Placement(transformation(extent = {{110, 10}, {130, 30}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Capacitor C8(C = c4) annotation(
            Placement(transformation(extent = {{160, -150}, {180, -130}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Capacitor C9(C = c4 + c5) annotation(
            Placement(transformation(extent = {{208, -10}, {228, 10}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Resistor R11(R = 1) annotation(
            Placement(transformation(extent = {{209, 30}, {229, 50}}, rotation = 0)));
        protected
          Modelica.Electrical.Analog.Interfaces.NegativePin n1 annotation(
            Placement(transformation(extent = {{-214, -42}, {-210, -38}}, rotation = 0)));
        protected
          Modelica.Electrical.Analog.Interfaces.NegativePin n2 annotation(
            Placement(transformation(extent = {{-214, -2}, {-210, 2}}, rotation = 0)));
        protected
          Modelica.Electrical.Analog.Interfaces.NegativePin n3 annotation(
            Placement(transformation(extent = {{-214, 18}, {-210, 22}}, rotation = 0)));
        protected
          Modelica.Electrical.Analog.Interfaces.NegativePin n4 annotation(
            Placement(transformation(origin = {-160, 20}, extent = {{2, 2}, {-2, -2}}, rotation = 180)));
        protected
          Modelica.Electrical.Analog.Interfaces.NegativePin n5 annotation(
            Placement(transformation(extent = {{-114, -52}, {-110, -48}}, rotation = 0)));
        protected
          Modelica.Electrical.Analog.Interfaces.PositivePin p1 annotation(
            Placement(transformation(extent = {{-154, -142}, {-148, -138}}, rotation = 0)));
        protected
          Modelica.Electrical.Analog.Interfaces.NegativePin n6 annotation(
            Placement(transformation(extent = {{-62, 18}, {-58, 22}}, rotation = 0)));
        protected
          Modelica.Electrical.Analog.Interfaces.NegativePin n7 annotation(
            Placement(transformation(extent = {{-6, -52}, {-2, -48}}, rotation = 0)));
        protected
          Modelica.Electrical.Analog.Interfaces.NegativePin n8 annotation(
            Placement(transformation(extent = {{-6, -2}, {-2, 2}}, rotation = 0)));
        protected
          Modelica.Electrical.Analog.Interfaces.PositivePin p2 annotation(
            Placement(transformation(extent = {{-6, 18}, {-2, 22}}, rotation = 0)));
        protected
          Modelica.Electrical.Analog.Interfaces.PositivePin out1 annotation(
            Placement(transformation(extent = {{44, -142}, {48, -138}}, rotation = 0)));
        protected
          Modelica.Electrical.Analog.Interfaces.PositivePin p3 annotation(
            Placement(transformation(extent = {{58, 58}, {62, 62}}, rotation = 0)));
        protected
          Modelica.Electrical.Analog.Interfaces.NegativePin n9 annotation(
            Placement(transformation(extent = {{86, -2}, {90, 2}}, rotation = 0)));
        protected
          Modelica.Electrical.Analog.Interfaces.NegativePin n10 annotation(
            Placement(transformation(extent = {{86, -52}, {90, -48}}, rotation = 0)));
        protected
          Modelica.Electrical.Analog.Interfaces.NegativePin n11 annotation(
            Placement(transformation(extent = {{138, 18}, {142, 22}}, rotation = 0)));
        public
          Modelica.Electrical.Analog.Basic.Ground G2 annotation(
            Placement(transformation(extent = {{12, -96}, {26, -80}}, rotation = 0)));
        protected
          Modelica.Electrical.Analog.Interfaces.NegativePin n12 annotation(
            Placement(transformation(extent = {{238, 38}, {242, 42}}, rotation = 0)));
        protected
          Modelica.Electrical.Analog.Interfaces.NegativePin n13 annotation(
            Placement(transformation(extent = {{238, -2}, {242, 2}}, rotation = 0)));
        protected
          Modelica.Electrical.Analog.Interfaces.PositivePin p4 annotation(
            Placement(transformation(extent = {{188, -2}, {194, 2}}, rotation = 0)));
        protected
          Modelica.Electrical.Analog.Interfaces.NegativePin n14 annotation(
            Placement(transformation(extent = {{188, -52}, {194, -48}}, rotation = 0)));
        public
          Modelica.Electrical.Analog.Basic.Ground G3 annotation(
            Placement(transformation(extent = {{113, -96}, {127, -80}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Ground G4 annotation(
            Placement(transformation(extent = {{213, -96}, {227, -80}}, rotation = 0)));
          Modelica.Electrical.Analog.Sources.StepVoltage V(startTime = 1, V = 1) annotation(
            Placement(transformation(origin = {-240, -170}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
          Modelica.Electrical.Analog.Basic.Ground Ground1 annotation(
            Placement(transformation(extent = {{-246, -202}, {-234, -190}}, rotation = 0)));
        equation
          connect(Op1.in_p, G.p) annotation(
            Line(points = {{-200, -60}, {-200, -70}, {-180, -70}, {-180, -80}, {-181, -80}}, color = {0, 0, 255}));
          connect(G1.p, Op2.in_p) annotation(
            Line(points = {{-80, -80}, {-80, -70}, {-100, -70}, {-100, -60}}, color = {0, 0, 255}));
          connect(R1.n, n1) annotation(
            Line(points = {{-220, -40}, {-212, -40}}, color = {0, 0, 255}));
          connect(n1, Op1.in_n) annotation(
            Line(points = {{-212, -40}, {-200, -40}}, color = {0, 0, 255}));
          connect(C2.n, n1) annotation(
            Line(points = {{-220, -80}, {-212, -80}, {-212, -40}}, color = {0, 0, 255}));
          connect(R2.n, n2) annotation(
            Line(points = {{-220, 0}, {-218, 0}, {-216, 0}, {-212, 0}}, color = {0, 0, 255}));
          connect(n1, n2) annotation(
            Line(points = {{-212, -40}, {-212, 0}}, color = {0, 0, 255}));
          connect(n2, n3) annotation(
            Line(points = {{-212, 0}, {-212, 20}}, color = {0, 0, 255}));
          connect(n3, C1.p) annotation(
            Line(points = {{-212, 20}, {-192, 20}}, color = {0, 0, 255}));
          connect(n3, R3.p) annotation(
            Line(points = {{-212, 20}, {-212, 60}, {-193, 60}}, color = {0, 0, 255}));
          connect(C1.n, n4) annotation(
            Line(points = {{-172, 20}, {-160, 20}}, color = {0, 0, 255}));
          connect(R3.n, n4) annotation(
            Line(points = {{-173, 60}, {-160, 60}, {-160, 20}}, color = {0, 0, 255}));
          connect(n4, Op1.out) annotation(
            Line(points = {{-160, 20}, {-160, -50}}, color = {0, 0, 255}));
          connect(R4.p, Op1.out) annotation(
            Line(points = {{-140, -50}, {-160, -50}}, color = {0, 0, 255}));
          connect(C5.p, Op1.out) annotation(
            Line(points = {{-30, -110}, {-160, -110}, {-160, -50}}, color = {0, 0, 255}));
          connect(R4.n, n5) annotation(
            Line(points = {{-120, -50}, {-112, -50}}, color = {0, 0, 255}));
          connect(n5, Op2.in_n) annotation(
            Line(points = {{-112, -50}, {-112, -40}, {-100, -40}}, color = {0, 0, 255}));
          connect(C3.p, n5) annotation(
            Line(points = {{-90, 20}, {-112, 20}, {-112, -50}}, color = {0, 0, 255}));
          connect(R5.n, n5) annotation(
            Line(points = {{-120, -90}, {-112, -90}, {-112, -50}}, color = {0, 0, 255}));
          connect(R5.p, p1) annotation(
            Line(points = {{-140, -90}, {-151, -90}, {-151, -140}}, color = {0, 0, 255}));
          connect(C2.p, p1) annotation(
            Line(points = {{-240, -80}, {-240, -140}, {-151, -140}}, color = {0, 0, 255}));
          connect(C3.n, n6) annotation(
            Line(points = {{-70, 20}, {-60, 20}}, color = {0, 0, 255}));
          connect(n6, Op2.out) annotation(
            Line(points = {{-60, 20}, {-60, -50}}, color = {0, 0, 255}));
          connect(R2.p, n6) annotation(
            Line(points = {{-240, 0}, {-240, 80}, {-60, 80}, {-60, 20}}, color = {0, 0, 255}));
          connect(Op2.out, R7.p) annotation(
            Line(points = {{-60, -50}, {-31, -50}}, color = {0, 0, 255}));
          connect(R7.n, n7) annotation(
            Line(points = {{-11, -50}, {-4, -50}}, color = {0, 0, 255}));
          connect(n7, Op3.in_n) annotation(
            Line(points = {{-4, -50}, {-4, -40.5}, {4, -40.5}}, color = {0, 0, 255}));
          connect(C5.n, n7) annotation(
            Line(points = {{-10, -110}, {-4, -110}, {-4, -50}}, color = {0, 0, 255}));
          connect(R6.n, n8) annotation(
            Line(points = {{-11, 0}, {-9.25, 0}, {-7.5, 0}, {-4, 0}}, color = {0, 0, 255}));
          connect(n7, n8) annotation(
            Line(points = {{-4, -50}, {-4, 0}}, color = {0, 0, 255}));
          connect(C6.p, p2) annotation(
            Line(points = {{9, 20}, {-4, 20}}, color = {0, 0, 255}));
          connect(n8, p2) annotation(
            Line(points = {{-4, 0}, {-4, 20}}, color = {0, 0, 255}));
          connect(C4.n, p2) annotation(
            Line(points = {{-10, 40}, {-4, 40}, {-4, 20}}, color = {0, 0, 255}));
          connect(C6.n, Op3.out) annotation(
            Line(points = {{29, 20}, {46, 20}, {46, -51}}, color = {0, 0, 255}));
          connect(R9.p, Op3.out) annotation(
            Line(points = {{60, -50}, {54, -50}, {54, -51}, {46, -51}}, color = {0, 0, 255}));
          connect(Op3.out, out1) annotation(
            Line(points = {{46, -51}, {46, -140}}, color = {0, 0, 255}));
          connect(p1, out1) annotation(
            Line(points = {{-151, -140}, {46, -140}}, color = {0, 0, 255}));
          connect(out1, C8.p) annotation(
            Line(points = {{46, -140}, {160, -140}}, color = {0, 0, 255}));
          connect(C4.p, p3) annotation(
            Line(points = {{-30, 40}, {-40, 40}, {-40, 60}, {60, 60}}, color = {0, 0, 255}));
          connect(p3, R8.p) annotation(
            Line(points = {{60, 60}, {60, 0}}, color = {0, 0, 255}));
          connect(R8.n, n9) annotation(
            Line(points = {{80, 0}, {82, 0}, {84, 0}, {88, 0}}, color = {0, 0, 255}));
          connect(n9, n10) annotation(
            Line(points = {{88, 0}, {88, -50}}, color = {0, 0, 255}));
          connect(R9.n, n10) annotation(
            Line(points = {{80, -50}, {88, -50}}, color = {0, 0, 255}));
          connect(n10, Op4.in_n) annotation(
            Line(points = {{88, -50}, {88, -40}, {100, -40}}, color = {0, 0, 255}));
          connect(n9, C7.p) annotation(
            Line(points = {{88, 0}, {88, 20}, {110, 20}}, color = {0, 0, 255}));
          connect(C7.n, n11) annotation(
            Line(points = {{130, 20}, {140, 20}}, color = {0, 0, 255}));
          connect(R6.p, n11) annotation(
            Line(points = {{-31, 0}, {-48, 0}, {-48, 80}, {140, 80}, {140, 20}}, color = {0, 0, 255}));
          connect(n11, Op4.out) annotation(
            Line(points = {{140, 20}, {140, -50}}, color = {0, 0, 255}));
          connect(Op4.out, R10.p) annotation(
            Line(points = {{140, -50}, {160, -50}}, color = {0, 0, 255}));
          connect(G2.p, Op3.in_p) annotation(
            Line(points = {{19, -80}, {4, -80}, {4, -61.5}}, color = {0, 0, 255}));
          connect(R11.n, n12) annotation(
            Line(points = {{229, 40}, {240, 40}}, color = {0, 0, 255}));
          connect(p3, n12) annotation(
            Line(points = {{60, 60}, {240, 60}, {240, 40}}, color = {0, 0, 255}));
          connect(C9.n, n13) annotation(
            Line(points = {{228, 0}, {231, 0}, {234, 0}, {240, 0}}, color = {0, 0, 255}));
          connect(n12, n13) annotation(
            Line(points = {{240, 40}, {240, 0}}, color = {0, 0, 255}));
          connect(n13, Op5.out) annotation(
            Line(points = {{240, 0}, {240, -50}}, color = {0, 0, 255}));
          connect(C9.p, p4) annotation(
            Line(points = {{208, 0}, {203.75, 0}, {199.5, 0}, {191, 0}}, color = {0, 0, 255}));
          connect(R11.p, p4) annotation(
            Line(points = {{209, 40}, {191, 40}, {191, 0}}, color = {0, 0, 255}));
          connect(R10.n, n14) annotation(
            Line(points = {{180, -50}, {191, -50}}, color = {0, 0, 255}));
          connect(p4, n14) annotation(
            Line(points = {{191, 0}, {191, -50}}, color = {0, 0, 255}));
          connect(Op5.in_n, n14) annotation(
            Line(points = {{200, -40}, {191, -40}, {191, -50}}, color = {0, 0, 255}));
          connect(C8.n, n14) annotation(
            Line(points = {{180, -140}, {191, -140}, {191, -50}}, color = {0, 0, 255}));
          connect(Op4.in_p, G3.p) annotation(
            Line(points = {{100, -60}, {100, -80}, {120, -80}}, color = {0, 0, 255}));
          connect(Op5.in_p, G4.p) annotation(
            Line(points = {{200, -60}, {200, -80}, {220, -80}}, color = {0, 0, 255}));
          connect(V.p, Ground1.p) annotation(
            Line(points = {{-240, -180}, {-240, -190}}, color = {0, 0, 255}));
          connect(V.n, R1.p) annotation(
            Line(points = {{-240, -160}, {-250, -160}, {-250, -40}, {-240, -40}}, color = {0, 0, 255}));
          annotation(
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-250, -200}, {250, 200}}), graphics = {Text(extent = {{-130, 172}, {80, 120}}, textString = "CauerLowPassOPV", lineColor = {0, 0, 255})}),
            experiment(StopTime = 60),
            Documentation(revisions = "<html>
<ul>
<li><i>January 13, 2006</i>
       by Christoph Clauss<br>
       included into Modelica Standard Library</li>
<li><i>September 15, 2005</i>
       by by Peter Trappe designed and by Teresa Schlegel<br>
       initially modelled.</li>
</ul>
</html>", info = "<html>
<p>The example Cauer Filter is a low-pass-filter of the fifth order. It is realized using an analog network with operational amplifiers. The voltage source V is the input voltage (step), and the OP5.out.v is the filter output voltage. The pulse response is calculated.</p>
<p>This model is identical to the CauerLowPassAnalog example, but inverting. To get the same response as that of the CauerLowPassAnalog example, a negative voltage step is used as input.</p>
<p>The simulation end time should be 60. Please plot both V.v (which is the inverted input voltage) and OP5.p.v (output voltage). Compare this result with the CauerLowPassAnalog result.</p>
<p>During translation some warnings are issued concerning resistor values (Value=-1 not in range[0,1.e+100]). Do not worry about it. The negative values are o.k.</p>
</html>"));
        end CauerLowPassOPV;

        model CauerLowPassSC "Cauer low-pass filter with operational amplifiers and switched capacitors"
          extends Modelica.Icons.Example;

          model Rn "Negative resistance"
            parameter Modelica.SIunits.Time clock = 1 "Clock";
            parameter Modelica.SIunits.Resistance R(min = Modelica.Constants.eps) = 1 "Resistance";
            Modelica.Blocks.Sources.BooleanPulse BooleanPulse1(period = clock) annotation(
              Placement(transformation(extent = {{-10, 50}, {10, 70}}, rotation = 0)));
            Modelica.Electrical.Analog.Basic.Capacitor Capacitor1(C = clock / R) annotation(
              Placement(transformation(extent = {{-20, -20}, {20, 20}}, rotation = 0)));
            Modelica.Electrical.Analog.Ideal.IdealCommutingSwitch IdealCommutingSwitch1 annotation(
              Placement(transformation(origin = {-50, 0}, extent = {{-10, -10}, {10, 10}}, rotation = 180)));
            Modelica.Electrical.Analog.Ideal.IdealCommutingSwitch IdealCommutingSwitch2 annotation(
              Placement(transformation(extent = {{40, -10}, {60, 10}}, rotation = 0)));
            Modelica.Electrical.Analog.Basic.Ground Ground1 annotation(
              Placement(transformation(extent = {{-66, -32}, {-54, -20}}, rotation = 0)));
            Modelica.Electrical.Analog.Basic.Ground Ground2 annotation(
              Placement(transformation(extent = {{54, -32}, {66, -20}}, rotation = 0)));
            Modelica.Electrical.Analog.Interfaces.NegativePin n1 annotation(
              Placement(transformation(extent = {{-112, -10}, {-92, 10}}, rotation = 0)));
            Modelica.Electrical.Analog.Interfaces.NegativePin n2 annotation(
              Placement(transformation(extent = {{90, -8}, {110, 12}}, rotation = 0)));
          equation
            connect(IdealCommutingSwitch1.p, Capacitor1.p) annotation(
              Line(points = {{-40, 0}, {-42, 0}, {-44, 0}, {-20, 0}}, color = {0, 0, 255}));
            connect(Capacitor1.n, IdealCommutingSwitch2.p) annotation(
              Line(points = {{20, 0}, {25, 0}, {30, 0}, {40, 0}}, color = {0, 0, 255}));
            connect(IdealCommutingSwitch2.control, BooleanPulse1.y) annotation(
              Line(points = {{50, 8}, {50, 30}, {20, 30}, {20, 60}, {11, 60}}, color = {255, 0, 255}));
            connect(IdealCommutingSwitch1.control, BooleanPulse1.y) annotation(
              Line(points = {{-50, -8}, {-50, 30}, {20, 30}, {20, 60}, {11, 60}}, color = {255, 0, 255}));
            connect(Ground2.p, IdealCommutingSwitch2.n2) annotation(
              Line(points = {{60, -20}, {60, 0}}, color = {0, 0, 255}));
            connect(IdealCommutingSwitch2.n1, n2) annotation(
              Line(points = {{60, 5}, {80, 5}, {80, 2}, {100, 2}}, color = {0, 0, 255}));
            connect(n1, IdealCommutingSwitch1.n2) annotation(
              Line(points = {{-102, 0}, {-81, 0}, {-60, 0}}, color = {0, 0, 255}));
            connect(Ground1.p, IdealCommutingSwitch1.n1) annotation(
              Line(points = {{-60, -20}, {-60, -5}}, color = {0, 0, 255}));
            annotation(
              Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-80, 32}, {80, -30}}, lineColor = {0, 0, 255}), Text(extent = {{-30, -40}, {30, -72}}, lineColor = {0, 0, 0}, textString = "R=%R"), Line(points = {{-74, 0}, {-80, 0}, {-60, 0}}, color = {85, 255, 85}), Line(points = {{-80, -20}, {-60, -20}, {-60, -16}, {-40, -8}, {-4, -8}}, color = {85, 255, 85}), Line(points = {{80, 0}, {60, 0}}, color = {85, 255, 85}), Line(points = {{-60, 2}, {-60, -4}}, color = {170, 255, 170}), Line(points = {{-4, 2}, {-4, -18}}, color = {85, 255, 85}), Line(points = {{4, 2}, {4, -18}}, color = {85, 255, 85}), Text(extent = {{-60, 80}, {60, 40}}, lineColor = {0, 0, 255}, textString = "%name"), Line(points = {{-92, 0}, {-80, 0}}, color = {85, 85, 255}), Line(points = {{90, 0}, {80, 0}, {80, 0}}, color = {85, 85, 255}), Line(points = {{4, -8}, {40, -8}, {60, -4}}, color = {85, 255, 85}), Line(points = {{60, 0}, {60, -4}}, color = {85, 255, 85}), Line(points = {{60, -16}, {60, -20}, {80, -20}}, color = {85, 255, 85})}),
              Documentation(info = "<html>
<p>This model is a<b> negative</b> resistor without thermal behavior which is described as a switched capacitor model (care for the schematic).</p>
<p>The clock source is inside the model, its frequency can be chosen by parameter. Also the resistance is a parameter, it has to be <b>positive</b>. The internal (switched) capacitor is parametrized in such a way that the total resistance is independently from the frequency equal to the negative value of the <b>negative</b> resistance parameter.</p>
</html>"));
          end Rn;

          model Rp "Positive resistance"
            parameter Modelica.SIunits.Time clock = 1 "Clock";
            parameter Modelica.SIunits.Resistance R(min = Modelica.Constants.eps) = 1 "Resistance";
            Modelica.Blocks.Sources.BooleanPulse BooleanPulse1(period = clock) annotation(
              Placement(transformation(extent = {{-10, 50}, {10, 70}}, rotation = 0)));
            Modelica.Electrical.Analog.Basic.Capacitor Capacitor1(C = clock / R) annotation(
              Placement(transformation(extent = {{-20, -20}, {20, 20}}, rotation = 0)));
            Modelica.Electrical.Analog.Ideal.IdealCommutingSwitch IdealCommutingSwitch1 annotation(
              Placement(transformation(origin = {-50, 0}, extent = {{-10, -10}, {10, 10}}, rotation = 180)));
            Modelica.Electrical.Analog.Ideal.IdealCommutingSwitch IdealCommutingSwitch2 annotation(
              Placement(transformation(extent = {{40, -10}, {60, 10}}, rotation = 0)));
            Modelica.Electrical.Analog.Basic.Ground Ground1 annotation(
              Placement(transformation(origin = {-60, 46}, extent = {{-6, -6}, {6, 6}}, rotation = 180)));
            Modelica.Electrical.Analog.Basic.Ground Ground2 annotation(
              Placement(transformation(extent = {{54, -44}, {66, -32}}, rotation = 0)));
            Modelica.Electrical.Analog.Interfaces.NegativePin n1 annotation(
              Placement(transformation(extent = {{-112, -10}, {-92, 10}}, rotation = 0)));
            Modelica.Electrical.Analog.Interfaces.NegativePin n2 annotation(
              Placement(transformation(extent = {{90, -8}, {110, 12}}, rotation = 0)));
          equation
            connect(IdealCommutingSwitch1.p, Capacitor1.p) annotation(
              Line(points = {{-40, 0}, {-42, 0}, {-44, 0}, {-20, 0}}, color = {0, 0, 255}));
            connect(Capacitor1.n, IdealCommutingSwitch2.p) annotation(
              Line(points = {{20, 0}, {25, 0}, {30, 0}, {40, 0}}, color = {0, 0, 255}));
            connect(IdealCommutingSwitch2.control, BooleanPulse1.y) annotation(
              Line(points = {{50, 8}, {50, 30}, {20, 30}, {20, 60}, {11, 60}}, color = {255, 0, 255}));
            connect(IdealCommutingSwitch1.control, BooleanPulse1.y) annotation(
              Line(points = {{-50, -8}, {-50, 30}, {20, 30}, {20, 60}, {11, 60}}, color = {255, 0, 255}));
            connect(Ground1.p, IdealCommutingSwitch1.n2) annotation(
              Line(points = {{-60, 40}, {-60, 0}}, color = {0, 0, 255}));
            connect(Ground2.p, IdealCommutingSwitch2.n2) annotation(
              Line(points = {{60, -32}, {60, 0}}, color = {0, 0, 255}));
            connect(IdealCommutingSwitch1.n1, n1) annotation(
              Line(points = {{-60, -5}, {-99, -5}, {-99, 0}, {-102, 0}}, color = {0, 0, 255}));
            connect(IdealCommutingSwitch2.n1, n2) annotation(
              Line(points = {{60, 5}, {80, 5}, {80, 2}, {100, 2}}, color = {0, 0, 255}));
            annotation(
              Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-80, 30}, {80, -32}}, lineColor = {0, 0, 255}), Line(points = {{-92, 0}, {-80, 0}}, color = {85, 85, 255}), Line(points = {{80, 0}, {92, 0}}, color = {85, 85, 255}), Text(extent = {{-40, -40}, {32, -72}}, lineColor = {0, 0, 0}, textString = "R=%R"), Line(points = {{-74, 0}, {-80, 0}, {-60, 0}}, color = {85, 255, 85}), Line(points = {{-80, -20}, {-60, -20}, {-60, -16}, {-40, -10}, {-4, -10}}, color = {85, 255, 85}), Line(points = {{4, -10}, {40, -10}, {60, -16}, {60, -20}, {80, -20}}, color = {85, 255, 85}), Line(points = {{80, 0}, {60, 0}}, color = {85, 255, 85}), Line(points = {{60, 0}, {60, -4}}, color = {85, 255, 85}), Line(points = {{-60, 0}, {-60, -6}}, color = {170, 255, 170}), Line(points = {{-4, 0}, {-4, -20}}, color = {85, 255, 85}), Line(points = {{4, 0}, {4, -20}}, color = {85, 255, 85}), Text(extent = {{-60, 80}, {60, 40}}, lineColor = {0, 0, 255}, textString = "%name")}),
              Documentation(info = "<html>
<p>This model is a <b>positive</b> resistor without thermal behavior which is described as a switched capacitor model (care for the schematic).</p>
<p>The clock source is inside the model, its frequency can be chosen by parameter. Also the resistance is a parameter, it has to be <b>positive</b>. The internal (switched) capacitor is parametrized in such a way that the total resistance is independently from the frequency equal to the resistance parameter.</p>
</html>"));
          end Rp;

          parameter Modelica.SIunits.Capacitance l1 = 1.304 "filter coefficient i1";
          parameter Modelica.SIunits.Capacitance l2 = 0.8586 "filter coefficient i2";
          parameter Modelica.SIunits.Capacitance c1 = 1.072 "filter coefficient c1";
          parameter Modelica.SIunits.Capacitance c2 = 1 / (1.704992 ^ 2 * l1) "filter coefficient c2";
          parameter Modelica.SIunits.Capacitance c3 = 1.682 "filter coefficient c3";
          parameter Modelica.SIunits.Capacitance c4 = 1 / (1.179945 ^ 2 * l2) "filter coefficient c4";
          parameter Modelica.SIunits.Capacitance c5 = 0.7262 "filter coefficient c5";
          Modelica.Electrical.Analog.Basic.Capacitor C1(C = c1 + c2, v(start = 0, fixed = true)) annotation(
            Placement(transformation(extent = {{-193, 30}, {-173, 50}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Capacitor C2(C = c2, v(start = 0, fixed = true)) annotation(
            Placement(transformation(extent = {{-241, -70}, {-221, -50}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Capacitor C3(C = l1, v(start = 0, fixed = true)) annotation(
            Placement(transformation(extent = {{-91, 30}, {-71, 50}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Capacitor C4(C = c4, v(start = 0, fixed = true)) annotation(
            Placement(transformation(extent = {{-31, 50}, {-11, 70}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Capacitor C5(C = c2) annotation(
            Placement(transformation(extent = {{-31, -100}, {-11, -80}}, rotation = 0)));
          Modelica.Electrical.Analog.Ideal.IdealOpAmp3Pin Op1 annotation(
            Placement(transformation(extent = {{-201, -50}, {-161, -10}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Ground G annotation(
            Placement(transformation(extent = {{-189, -76}, {-175, -60}}, rotation = 0)));
          Modelica.Electrical.Analog.Ideal.IdealOpAmp3Pin Op2 annotation(
            Placement(transformation(extent = {{-101, -50}, {-61, -10}}, rotation = 0)));
          Modelica.Electrical.Analog.Ideal.IdealOpAmp3Pin Op3 annotation(
            Placement(transformation(extent = {{5, -50}, {45, -10}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Ground G1 annotation(
            Placement(transformation(extent = {{-88, -76}, {-74, -60}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Capacitor C6(C = c2 + c3 + c4) annotation(
            Placement(transformation(extent = {{8, 30}, {28, 50}}, rotation = 0)));
          Modelica.Electrical.Analog.Ideal.IdealOpAmp3Pin Op4 annotation(
            Placement(transformation(extent = {{99, -50}, {139, -10}}, rotation = 0)));
          Modelica.Electrical.Analog.Ideal.IdealOpAmp3Pin Op5 annotation(
            Placement(transformation(extent = {{199, -50}, {239, -10}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Capacitor C7(C = l2, v(start = 0, fixed = true)) annotation(
            Placement(transformation(extent = {{109, 30}, {129, 50}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Capacitor C8(C = c4) annotation(
            Placement(transformation(extent = {{159, -130}, {179, -110}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Capacitor C9(C = c4 + c5) annotation(
            Placement(transformation(extent = {{207, 10}, {227, 30}}, rotation = 0)));
        protected
          Modelica.Electrical.Analog.Interfaces.NegativePin n1 annotation(
            Placement(transformation(extent = {{-215, -22}, {-211, -18}}, rotation = 0)));
        protected
          Modelica.Electrical.Analog.Interfaces.NegativePin n2 annotation(
            Placement(transformation(extent = {{-215, 18}, {-211, 22}}, rotation = 0)));
        protected
          Modelica.Electrical.Analog.Interfaces.NegativePin n3 annotation(
            Placement(transformation(extent = {{-215, 38}, {-211, 42}}, rotation = 0)));
        protected
          Modelica.Electrical.Analog.Interfaces.NegativePin n4 annotation(
            Placement(transformation(origin = {-161, 40}, extent = {{2, 2}, {-2, -2}}, rotation = 180)));
        protected
          Modelica.Electrical.Analog.Interfaces.NegativePin n5 annotation(
            Placement(transformation(extent = {{-115, -32}, {-111, -28}}, rotation = 0)));
        protected
          Modelica.Electrical.Analog.Interfaces.PositivePin p1 annotation(
            Placement(transformation(extent = {{-155, -122}, {-149, -118}}, rotation = 0)));
        protected
          Modelica.Electrical.Analog.Interfaces.NegativePin n6 annotation(
            Placement(transformation(extent = {{-63, 38}, {-59, 42}}, rotation = 0)));
        protected
          Modelica.Electrical.Analog.Interfaces.NegativePin n7 annotation(
            Placement(transformation(extent = {{-7, -32}, {-3, -28}}, rotation = 0)));
        protected
          Modelica.Electrical.Analog.Interfaces.NegativePin n8 annotation(
            Placement(transformation(extent = {{-7, 18}, {-3, 22}}, rotation = 0)));
        protected
          Modelica.Electrical.Analog.Interfaces.PositivePin p2 annotation(
            Placement(transformation(extent = {{-7, 38}, {-3, 42}}, rotation = 0)));
        protected
          Modelica.Electrical.Analog.Interfaces.PositivePin out1 annotation(
            Placement(transformation(extent = {{43, -122}, {47, -118}}, rotation = 0)));
        protected
          Modelica.Electrical.Analog.Interfaces.PositivePin p3 annotation(
            Placement(transformation(extent = {{57, 78}, {61, 82}}, rotation = 0)));
        protected
          Modelica.Electrical.Analog.Interfaces.NegativePin n9 annotation(
            Placement(transformation(extent = {{85, 18}, {89, 22}}, rotation = 0)));
        protected
          Modelica.Electrical.Analog.Interfaces.NegativePin n10 annotation(
            Placement(transformation(extent = {{85, -32}, {89, -28}}, rotation = 0)));
        protected
          Modelica.Electrical.Analog.Interfaces.NegativePin n11 annotation(
            Placement(transformation(extent = {{137, 38}, {141, 42}}, rotation = 0)));
        public
          Modelica.Electrical.Analog.Basic.Ground G2 annotation(
            Placement(transformation(extent = {{11, -76}, {25, -60}}, rotation = 0)));
        protected
          Modelica.Electrical.Analog.Interfaces.NegativePin n12 annotation(
            Placement(transformation(extent = {{237, 58}, {241, 62}}, rotation = 0)));
        protected
          Modelica.Electrical.Analog.Interfaces.NegativePin n13 annotation(
            Placement(transformation(extent = {{237, 18}, {241, 22}}, rotation = 0)));
        protected
          Modelica.Electrical.Analog.Interfaces.PositivePin p4 annotation(
            Placement(transformation(extent = {{187, 18}, {193, 22}}, rotation = 0)));
        protected
          Modelica.Electrical.Analog.Interfaces.NegativePin n14 annotation(
            Placement(transformation(extent = {{187, -32}, {193, -28}}, rotation = 0)));
        public
          Modelica.Electrical.Analog.Basic.Ground G3 annotation(
            Placement(transformation(extent = {{112, -76}, {126, -60}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Ground G4 annotation(
            Placement(transformation(extent = {{212, -76}, {226, -60}}, rotation = 0)));
          Modelica.Electrical.Analog.Sources.StepVoltage V(startTime = 1, V = 1) annotation(
            Placement(transformation(origin = {-241, -150}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
          Modelica.Electrical.Analog.Basic.Ground Ground1 annotation(
            Placement(transformation(extent = {{-247, -182}, {-235, -170}}, rotation = 0)));
          Rn R4(clock = 0.1, Capacitor1(v(start = 0, fixed = true))) annotation(
            Placement(transformation(extent = {{-140, -40}, {-120, -20}}, rotation = 0)));
          Rn R5(clock = 0.1, Capacitor1(v(start = 0, fixed = true))) annotation(
            Placement(transformation(extent = {{-140, -80}, {-120, -60}}, rotation = 0)));
          Rn R8(clock = 0.1, Capacitor1(v(start = 0, fixed = true))) annotation(
            Placement(transformation(extent = {{60, 10}, {80, 30}}, rotation = 0)));
          Rn R9(clock = 0.1, Capacitor1(v(start = 0, fixed = true))) annotation(
            Placement(transformation(extent = {{60, -40}, {80, -20}}, rotation = 0)));
          Rp R1(clock = 0.1, Capacitor1(v(start = 0, fixed = true))) annotation(
            Placement(transformation(extent = {{-240, -30}, {-220, -10}}, rotation = 0)));
          Rp R2(clock = 0.1, Capacitor1(v(start = 0, fixed = true))) annotation(
            Placement(transformation(extent = {{-240, 10}, {-220, 30}}, rotation = 0)));
          Rp R3(clock = 0.1, Capacitor1(v(start = 0, fixed = true))) annotation(
            Placement(transformation(extent = {{-200, 70}, {-180, 90}}, rotation = 0)));
          Rp Rp1(clock = 0.1, Capacitor1(v(start = 0, fixed = true))) annotation(
            Placement(transformation(extent = {{-32, 10}, {-12, 30}}, rotation = 0)));
          Rp R7(clock = 0.1, Capacitor1(v(start = 0, fixed = true))) annotation(
            Placement(transformation(extent = {{-32, -40}, {-12, -20}}, rotation = 0)));
          Rp R10(clock = 0.1, Capacitor1(v(start = 0, fixed = true))) annotation(
            Placement(transformation(extent = {{160, -40}, {180, -20}}, rotation = 0)));
          Rp R11(clock = 0.1, Capacitor1(v(start = 0, fixed = true))) annotation(
            Placement(transformation(extent = {{208, 50}, {228, 70}}, rotation = 0)));
        equation
          connect(Op1.in_p, G.p) annotation(
            Line(points = {{-201, -40}, {-201, -50}, {-181, -50}, {-181, -60}, {-182, -60}}, color = {0, 0, 255}));
          connect(G1.p, Op2.in_p) annotation(
            Line(points = {{-81, -60}, {-81, -50}, {-101, -50}, {-101, -40}}, color = {0, 0, 255}));
          connect(n1, Op1.in_n) annotation(
            Line(points = {{-213, -20}, {-201, -20}}, color = {0, 0, 255}));
          connect(C2.n, n1) annotation(
            Line(points = {{-221, -60}, {-213, -60}, {-213, -20}}, color = {0, 0, 255}));
          connect(n1, n2) annotation(
            Line(points = {{-213, -20}, {-213, 20}}, color = {0, 0, 255}));
          connect(n2, n3) annotation(
            Line(points = {{-213, 20}, {-213, 40}}, color = {0, 0, 255}));
          connect(n3, C1.p) annotation(
            Line(points = {{-213, 40}, {-193, 40}}, color = {0, 0, 255}));
          connect(C1.n, n4) annotation(
            Line(points = {{-173, 40}, {-161, 40}}, color = {0, 0, 255}));
          connect(n4, Op1.out) annotation(
            Line(points = {{-161, 40}, {-161, -30}}, color = {0, 0, 255}));
          connect(C5.p, Op1.out) annotation(
            Line(points = {{-31, -90}, {-161, -90}, {-161, -30}}, color = {0, 0, 255}));
          connect(n5, Op2.in_n) annotation(
            Line(points = {{-113, -30}, {-113, -20}, {-101, -20}}, color = {0, 0, 255}));
          connect(C3.p, n5) annotation(
            Line(points = {{-91, 40}, {-113, 40}, {-113, -30}}, color = {0, 0, 255}));
          connect(C2.p, p1) annotation(
            Line(points = {{-241, -60}, {-241, -120}, {-152, -120}}, color = {0, 0, 255}));
          connect(C3.n, n6) annotation(
            Line(points = {{-71, 40}, {-61, 40}}, color = {0, 0, 255}));
          connect(n6, Op2.out) annotation(
            Line(points = {{-61, 40}, {-61, -30}}, color = {0, 0, 255}));
          connect(n7, Op3.in_n) annotation(
            Line(points = {{-5, -30}, {-5, -20}, {5, -20}}, color = {0, 0, 255}));
          connect(C5.n, n7) annotation(
            Line(points = {{-11, -90}, {-5, -90}, {-5, -30}}, color = {0, 0, 255}));
          connect(n7, n8) annotation(
            Line(points = {{-5, -30}, {-5, 20}}, color = {0, 0, 255}));
          connect(C6.p, p2) annotation(
            Line(points = {{8, 40}, {-5, 40}}, color = {0, 0, 255}));
          connect(n8, p2) annotation(
            Line(points = {{-5, 20}, {-5, 40}}, color = {0, 0, 255}));
          connect(C4.n, p2) annotation(
            Line(points = {{-11, 60}, {-5, 60}, {-5, 40}}, color = {0, 0, 255}));
          connect(C6.n, Op3.out) annotation(
            Line(points = {{28, 40}, {45, 40}, {45, -30}}, color = {0, 0, 255}));
          connect(Op3.out, out1) annotation(
            Line(points = {{45, -30}, {45, -120}}, color = {0, 0, 255}));
          connect(p1, out1) annotation(
            Line(points = {{-152, -120}, {45, -120}}, color = {0, 0, 255}));
          connect(out1, C8.p) annotation(
            Line(points = {{45, -120}, {159, -120}}, color = {0, 0, 255}));
          connect(C4.p, p3) annotation(
            Line(points = {{-31, 60}, {-41, 60}, {-41, 80}, {59, 80}}, color = {0, 0, 255}));
          connect(n9, n10) annotation(
            Line(points = {{87, 20}, {87, -30}}, color = {0, 0, 255}));
          connect(n10, Op4.in_n) annotation(
            Line(points = {{87, -30}, {87, -20}, {99, -20}}, color = {0, 0, 255}));
          connect(n9, C7.p) annotation(
            Line(points = {{87, 20}, {87, 40}, {109, 40}}, color = {0, 0, 255}));
          connect(C7.n, n11) annotation(
            Line(points = {{129, 40}, {139, 40}}, color = {0, 0, 255}));
          connect(n11, Op4.out) annotation(
            Line(points = {{139, 40}, {139, -30}}, color = {0, 0, 255}));
          connect(G2.p, Op3.in_p) annotation(
            Line(points = {{18, -60}, {5, -60}, {5, -40}}, color = {0, 0, 255}));
          connect(p3, n12) annotation(
            Line(points = {{59, 80}, {239, 80}, {239, 60}}, color = {0, 0, 255}));
          connect(C9.n, n13) annotation(
            Line(points = {{227, 20}, {239, 20}}, color = {0, 0, 255}));
          connect(n12, n13) annotation(
            Line(points = {{239, 60}, {239, 20}}, color = {0, 0, 255}));
          connect(n13, Op5.out) annotation(
            Line(points = {{239, 20}, {239, -30}}, color = {0, 0, 255}));
          connect(C9.p, p4) annotation(
            Line(points = {{207, 20}, {190, 20}}, color = {0, 0, 255}));
          connect(p4, n14) annotation(
            Line(points = {{190, 20}, {190, -30}}, color = {0, 0, 255}));
          connect(Op5.in_n, n14) annotation(
            Line(points = {{199, -20}, {190, -20}, {190, -30}}, color = {0, 0, 255}));
          connect(C8.n, n14) annotation(
            Line(points = {{179, -120}, {190, -120}, {190, -30}}, color = {0, 0, 255}));
          connect(Op4.in_p, G3.p) annotation(
            Line(points = {{99, -40}, {99, -60}, {119, -60}}, color = {0, 0, 255}));
          connect(Op5.in_p, G4.p) annotation(
            Line(points = {{199, -40}, {199, -60}, {219, -60}}, color = {0, 0, 255}));
          connect(V.p, Ground1.p) annotation(
            Line(points = {{-241, -160}, {-241, -170}}, color = {0, 0, 255}));
          connect(R4.n2, n5) annotation(
            Line(points = {{-120, -29.8}, {-116, -29.8}, {-116, -30}, {-113, -30}}, color = {0, 0, 255}));
          connect(Op1.out, R4.n1) annotation(
            Line(points = {{-161, -30}, {-140.2, -30}}, color = {0, 0, 255}));
          connect(R5.n1, p1) annotation(
            Line(points = {{-140.2, -70}, {-152, -70}, {-152, -120}}, color = {0, 0, 255}));
          connect(R5.n2, n5) annotation(
            Line(points = {{-120, -69.8}, {-116, -69.8}, {-116, -70}, {-113, -70}, {-113, -30}}, color = {0, 0, 255}));
          connect(p3, R8.n1) annotation(
            Line(points = {{59, 80}, {59.8, 80}, {59.8, 20}}, color = {0, 0, 255}));
          connect(R8.n2, n9) annotation(
            Line(points = {{80, 20.2}, {84, 20.2}, {84, 20}, {87, 20}}, color = {0, 0, 255}));
          connect(Op3.out, R9.n1) annotation(
            Line(points = {{45, -30}, {59.8, -30}}, color = {0, 0, 255}));
          connect(R9.n2, n10) annotation(
            Line(points = {{80, -29.8}, {84, -29.8}, {84, -30}, {87, -30}}, color = {0, 0, 255}));
          connect(R1.n1, V.n) annotation(
            Line(points = {{-240.2, -20}, {-250, -20}, {-250, -130}, {-241, -130}, {-241, -140}}, color = {0, 0, 255}));
          connect(R1.n2, n1) annotation(
            Line(points = {{-220, -19.8}, {-216, -19.8}, {-216, -20}, {-213, -20}}, color = {0, 0, 255}));
          connect(R2.n2, n2) annotation(
            Line(points = {{-220, 20.2}, {-218, 20.2}, {-218, 20}, {-213, 20}}, color = {0, 0, 255}));
          connect(R2.n1, n6) annotation(
            Line(points = {{-240.2, 20}, {-240, 20}, {-240, 100}, {-61, 100}, {-61, 40}}, color = {0, 0, 255}));
          connect(R3.n1, n3) annotation(
            Line(points = {{-200.2, 80}, {-213, 80}, {-213, 40}}, color = {0, 0, 255}));
          connect(R3.n2, n4) annotation(
            Line(points = {{-180, 80.2}, {-170, 80.2}, {-170, 80}, {-161, 80}, {-161, 40}}, color = {0, 0, 255}));
          connect(Rp1.n2, n8) annotation(
            Line(points = {{-12, 20.2}, {-8, 20.2}, {-8, 20}, {-5, 20}}, color = {0, 0, 255}));
          connect(Rp1.n1, n11) annotation(
            Line(points = {{-32.2, 20}, {-52, 20}, {-52, 100}, {139, 100}, {139, 40}}, color = {0, 0, 255}));
          connect(Op2.out, R7.n1) annotation(
            Line(points = {{-61, -30}, {-32.2, -30}}, color = {0, 0, 255}));
          connect(R7.n2, n7) annotation(
            Line(points = {{-12, -29.8}, {-8, -29.8}, {-8, -30}, {-5, -30}}, color = {0, 0, 255}));
          connect(R10.n1, Op4.out) annotation(
            Line(points = {{159.8, -30}, {139, -30}}, color = {0, 0, 255}));
          connect(R10.n2, n14) annotation(
            Line(points = {{180, -29.8}, {186, -29.8}, {186, -30}, {190, -30}}, color = {0, 0, 255}));
          connect(R11.n2, n12) annotation(
            Line(points = {{228, 60.2}, {234, 60.2}, {234, 60}, {239, 60}}, color = {0, 0, 255}));
          connect(R11.n1, p4) annotation(
            Line(points = {{207.8, 60}, {190, 60}, {190, 20}}, color = {0, 0, 255}));
          annotation(
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-250, -200}, {250, 200}}), graphics = {Text(extent = {{-100, 180}, {94, 140}}, textString = "CauerLowPassSC", lineColor = {0, 0, 255})}),
            experiment(StopTime = 60, Interval = 0.04),
            Documentation(revisions = "<html>
<ul>
<li><i>January 13, 2006</i>
       by Christoph Clauss<br>
       included into Modelica Standard Library</li>
<li><i>September 15, 2005</i>
       by by Peter Trappe designed and by Teresa Schlegel<br>
       initially modelled.</li>
</ul>
</html>", info = "<html>
<p>The example CauerLowPassSC is a low-pass-filter of the fifth order. It is realized using an switched-capacitor network with operational amplifiers. The voltage source V is the input voltage (step), and the OP5.out.v is the filter output voltage. The pulse response is calculated.</p>
<p>This model is identical to the CauerLowPassAnalog example, but inverting. To get the same response as that of the CauerLowPassAnalog example, a negative voltage step is used as input.</p>
<p>This model is identical to the CauerLowPassOPV example. But the resistors are realized by switched capacitors. There are two such resistors Rp (of value +1), and Rn (of value -1). In this models the switching clock source is included. In a typical switched capacitor circuit there would be a central clock source.</p>
<p>The simulation end time should be 60. Please plot both V.v (which is the inverted input voltage) and OP5.p.v (output voltage). Compare this result with the CauerLowPassAnalog result.</p>
<p>Due to the recharging of the capacitances after switching the performance of simulation is not as good as in the CauerLowPassOPV example.</p>
</html>"));
        end CauerLowPassSC;

        model CharacteristicIdealDiodes "Characteristic of ideal diodes"
          extends Modelica.Icons.Example;
          Modelica.Electrical.Analog.Ideal.IdealDiode Ideal(Ron = 0, Goff = 0, Vknee = 0) annotation(
            Placement(transformation(extent = {{0, 40}, {20, 60}}, rotation = 0)));
          Modelica.Electrical.Analog.Ideal.IdealDiode With_Ron_Goff(Ron = 0.1, Goff = 0.1, Vknee = 0) annotation(
            Placement(transformation(extent = {{0, 0}, {20, 20}}, rotation = 0)));
          Modelica.Electrical.Analog.Ideal.IdealDiode With_Ron_Goff_Vknee(Ron = 0.2, Goff = 0.2, Vknee = 5) annotation(
            Placement(transformation(extent = {{0, -40}, {20, -20}}, rotation = 0)));
          Modelica.Electrical.Analog.Sources.SineVoltage SineVoltage1(V = 10, offset = -9, freqHz = 1) annotation(
            Placement(transformation(origin = {-40, 0}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          Modelica.Electrical.Analog.Basic.Ground Ground1 annotation(
            Placement(transformation(extent = {{-50, -80}, {-30, -60}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Resistor R1(R = 1.e-3) annotation(
            Placement(transformation(extent = {{60, 40}, {80, 60}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Resistor R2(R = 1.e-3) annotation(
            Placement(transformation(extent = {{60, 0}, {80, 20}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Resistor R3(R = 1.e-3) annotation(
            Placement(transformation(extent = {{60, -40}, {80, -20}}, rotation = 0)));
          Modelica.Electrical.Analog.Sources.SineVoltage SineVoltage2(V = 10, offset = 0, freqHz = 1) annotation(
            Placement(transformation(origin = {-60, 40}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          Modelica.Electrical.Analog.Sources.SineVoltage SineVoltage3(V = 10, offset = 0, freqHz = 1) annotation(
            Placement(transformation(origin = {-20, -40}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
        equation
          connect(Ground1.p, SineVoltage1.n) annotation(
            Line(points = {{-40, -60}, {-40, -10}}, color = {0, 0, 255}));
          connect(Ideal.n, R1.p) annotation(
            Line(points = {{20, 50}, {60, 50}}, color = {0, 0, 255}));
          connect(With_Ron_Goff.n, R2.p) annotation(
            Line(points = {{20, 10}, {60, 10}}, color = {0, 0, 255}));
          connect(With_Ron_Goff_Vknee.n, R3.p) annotation(
            Line(points = {{20, -30}, {60, -30}}, color = {0, 0, 255}));
          connect(R1.n, R2.n) annotation(
            Line(points = {{80, 50}, {80, 10}}, color = {0, 0, 255}));
          connect(R2.n, R3.n) annotation(
            Line(points = {{80, 10}, {80, -30}}, color = {0, 0, 255}));
          connect(R3.n, Ground1.p) annotation(
            Line(points = {{80, -30}, {80, -60}, {-40, -60}}, color = {0, 0, 255}));
          connect(SineVoltage2.p, Ideal.p) annotation(
            Line(points = {{-60, 50}, {0, 50}}, color = {0, 0, 255}));
          connect(SineVoltage2.n, Ground1.p) annotation(
            Line(points = {{-60, 30}, {-60, -60}, {-40, -60}}, color = {0, 0, 255}));
          connect(SineVoltage1.p, With_Ron_Goff.p) annotation(
            Line(points = {{-40, 10}, {0, 10}}, color = {0, 0, 255}));
          connect(With_Ron_Goff_Vknee.p, SineVoltage3.p) annotation(
            Line(points = {{0, -30}, {-20, -30}}, color = {0, 0, 255}));
          connect(SineVoltage3.n, Ground1.p) annotation(
            Line(points = {{-20, -50}, {-20, -60}, {-40, -60}}, color = {0, 0, 255}));
          annotation(
            Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-88, 102}, {92, 48}}, textString = "Characteristic Ideal Diodes", lineColor = {0, 0, 255})}),
            Documentation(info = "<html>
<p>Three examples of ideal diodes are shown:
<br>the <b>totally ideal diode</b> (Ideal) with all parameters to be zero,
the <b>nearly ideal diode</b> with <i>Ron=0.1</i> and <i>Goff=0.1</i>
 and the nearly ideal but <b>displaced diode</b> with <i>Vknee=5</i> and <i>Ron=0.1</i> and <i>Goff=0.1</i>.
The resistance and conductance are chosen untypically high since the slopes should be seen in the graphics.
</p><p>Simulate until T=1 s.
Plot in separate windows:
Ideal.i versus Ideal.v, With_Ron_Goff.i versus With_Ron_Goff.v, With_Ron_Goff_Vknee.i versus With_Ron_Goff_Vknee.v
</p>
</html>", revisions = "<html>
<p><b>Release Notes:</b></p>
<ul>
<li><i>Mai 7, 2004   </i>
       by Christoph Clauss<br> realized<br>
       </li>
</ul>
</html>"),
            experiment(StopTime = 1));
        end CharacteristicIdealDiodes;

        model CharacteristicThyristors "Characteristic of ideal thyristors"
          extends Modelica.Icons.Example;
          Modelica.Electrical.Analog.Ideal.IdealThyristor IdealThyristor1(off(start = true, fixed = true), Vknee = 1) annotation(
            Placement(transformation(extent = {{-20, 30}, {0, 50}}, rotation = 0)));
          Modelica.Electrical.Analog.Sources.SineVoltage SineVoltage1(V = 10, offset = 0, freqHz = 1) annotation(
            Placement(transformation(origin = {-40, -62}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          Modelica.Electrical.Analog.Basic.Ground Ground1 annotation(
            Placement(transformation(extent = {{-50, -108}, {-30, -88}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Resistor R3(R = 1.e-3) annotation(
            Placement(transformation(extent = {{40, 30}, {60, 50}}, rotation = 0)));
          Modelica.Electrical.Analog.Ideal.IdealGTOThyristor IdealGTOThyristor1(off(fixed = true, start = true), Vknee = 1) annotation(
            Placement(transformation(extent = {{-20, 0}, {0, 20}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Resistor R1(R = 1.e-3) annotation(
            Placement(transformation(extent = {{40, 0}, {60, 20}}, rotation = 0)));
          Blocks.Sources.BooleanTable booleanStep1(table = {1.25, 3.2, 4.2, 4.25, 5.7, 5.72}) annotation(
            Placement(transformation(extent = {{-68, 44}, {-48, 64}})));
          Modelica.Electrical.Analog.Ideal.IdealThyristor IdealThyristor2(off(start = true, fixed = true), Vknee = 1) annotation(
            Placement(transformation(extent = {{-20, -52}, {0, -32}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Resistor R2(R = 1.e-3) annotation(
            Placement(transformation(extent = {{40, -52}, {60, -32}}, rotation = 0)));
          Modelica.Electrical.Analog.Ideal.IdealGTOThyristor IdealGTOThyristor2(off(fixed = true, start = true), Vknee = 1) annotation(
            Placement(transformation(extent = {{-20, -82}, {0, -62}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Resistor R4(R = 1.e-3) annotation(
            Placement(transformation(extent = {{40, -82}, {60, -62}}, rotation = 0)));
          Blocks.Sources.BooleanPulse booleanPulse(width = 20, period = 1, startTime = 0.15) annotation(
            Placement(transformation(extent = {{-68, -32}, {-48, -12}})));
        initial equation
// IdealThyristor1.off = true;
        equation
          connect(IdealThyristor1.n, R3.p) annotation(
            Line(points = {{0, 40}, {40, 40}}, color = {0, 0, 255}));
          connect(Ground1.p, SineVoltage1.n) annotation(
            Line(points = {{-40, -88}, {-40, -72}}, color = {0, 0, 255}));
          connect(SineVoltage1.p, IdealThyristor1.p) annotation(
            Line(points = {{-40, -52}, {-40, 40}, {-20, 40}}, color = {0, 0, 255}));
          connect(IdealGTOThyristor1.n, R1.p) annotation(
            Line(points = {{0, 10}, {40, 10}}, color = {0, 0, 255}));
          connect(R3.n, R1.n) annotation(
            Line(points = {{60, 40}, {60, 10}}, color = {0, 0, 255}));
          connect(IdealGTOThyristor1.p, IdealThyristor1.p) annotation(
            Line(points = {{-20, 10}, {-20, 40}}, color = {0, 0, 255}));
          connect(IdealGTOThyristor1.fire, IdealThyristor1.fire) annotation(
            Line(points = {{-3, 21}, {-3, 27.5}, {-2, 27.5}, {-2, 35}, {-3, 35}, {-3, 51}}, color = {255, 0, 255}));
          connect(IdealThyristor1.fire, booleanStep1.y) annotation(
            Line(points = {{-3, 51}, {-3, 54}, {-47, 54}}, color = {255, 0, 255}, smooth = Smooth.None));
          connect(IdealThyristor2.n, R2.p) annotation(
            Line(points = {{0, -42}, {40, -42}}, color = {0, 0, 255}));
          connect(IdealGTOThyristor2.n, R4.p) annotation(
            Line(points = {{0, -72}, {40, -72}}, color = {0, 0, 255}));
          connect(R2.n, R4.n) annotation(
            Line(points = {{60, -42}, {60, -72}}, color = {0, 0, 255}));
          connect(IdealGTOThyristor2.p, IdealThyristor2.p) annotation(
            Line(points = {{-20, -72}, {-20, -42}}, color = {0, 0, 255}));
          connect(IdealGTOThyristor2.fire, IdealThyristor2.fire) annotation(
            Line(points = {{-3, -61}, {-3, -54.5}, {-2, -54.5}, {-2, -47}, {-3, -47}, {-3, -31}}, color = {255, 0, 255}));
          connect(R4.n, Ground1.p) annotation(
            Line(points = {{60, -72}, {60, -88}, {-40, -88}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(R2.n, R1.n) annotation(
            Line(points = {{60, -42}, {60, 10}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(SineVoltage1.p, IdealThyristor2.p) annotation(
            Line(points = {{-40, -52}, {-40, -42}, {-20, -42}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(booleanPulse.y, IdealThyristor2.fire) annotation(
            Line(points = {{-47, -22}, {-3, -22}, {-3, -31}}, color = {255, 0, 255}, smooth = Smooth.None));
          annotation(
            Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-96, 100}, {98, 60}}, textString = "Characteristic Thyristors", lineColor = {0, 0, 255})}),
            Documentation(info = "<html>
<p>This example compares the behavior of the <b>ideal thyristor</b> and the <b>ideal GTO thyristor</b> with <i>Vknee=1</i> both. The thyristors IdealThyristor1 and IdealGTOThyristor1 are controlled by an unregular Boolean fire signal. The aim is to show several cases for the fire signal in combination with the state (s&lt;0 or s&gt;0)of the thyristors. Please simulate until 6 seconds and compare IdealThyristor1.v with IdealGTOThyristor1.v, the same with IdealThyristor1.s and IdealGTOThyristor1.s (attention: s is a protected variable in each thyristor). Also compare IdealThyristor1.off and IdealGTOThyristor1.off and have a look at the fire signal (e.g. IdealThyristor1.fire). It can be seen that the IdealGTOThyristor1 reacts on switching off the fire signal whereas the IdealThyristor1 does not show this behavior.</p>
<p>The other thyristors IdealThyristor2 and IdealGTOThyristor2 are controlled by an periodic Boolean fire signal to show a typical use case. Please compare IdealThyristor2.v with IdealGTOThyristor2.v</p>
</html>", revisions = "<html>
<p><b>Release Notes:</b></p>
<ul>
<li><i>Jan 23, 2013   </i>
       by Kristin Majetta and Christoph Clauss<br> revised<br>
       </li>
</ul>
<ul>
<li><i>Mai 7, 2004   </i>
       by Christoph Clauss<br> realized<br>
       </li>
</ul>
</html>"),
            experiment(StopTime = 6));
        end CharacteristicThyristors;

        encapsulated model ChuaCircuit "Chua's circuit, ns, V, A"
          import Modelica.Electrical.Analog.Basic;
          import Modelica.Electrical.Analog.Examples.Utilities;
          import Modelica.Icons;
          extends Icons.Example;
          Basic.Inductor L(L = 18, i(start = 0, fixed = true)) annotation(
            Placement(transformation(origin = {-75, 38}, extent = {{-25, -25}, {25, 25}}, rotation = 270)));
          Basic.Resistor Ro(R = 12.5e-3) annotation(
            Placement(transformation(origin = {-75, -17}, extent = {{-25, -25}, {25, 25}}, rotation = 270)));
          Basic.Conductor G(G = 0.565) annotation(
            Placement(transformation(extent = {{-25, 38}, {25, 88}}, rotation = 0)));
          Basic.Capacitor C1(C = 10, v(start = 4, fixed = true)) annotation(
            Placement(transformation(origin = {25, 3}, extent = {{-25, -25}, {25, 25}}, rotation = 270)));
          Basic.Capacitor C2(C = 100, v(start = 0, fixed = true)) annotation(
            Placement(transformation(origin = {-25, 3}, extent = {{-25, -25}, {25, 25}}, rotation = 270)));
          Utilities.NonlinearResistor Nr(Ga(min = -1) = -0.757576, Gb(min = -1) = -0.409091, Ve = 1) annotation(
            Placement(transformation(origin = {75, 3}, extent = {{-25, -25}, {25, 25}}, rotation = 270)));
          Basic.Ground Gnd annotation(
            Placement(transformation(extent = {{-25, -112}, {25, -62}}, rotation = 0)));
        equation
          connect(L.n, Ro.p) annotation(
            Line(points = {{-75, 13}, {-75, 8}}));
          connect(C2.p, G.p) annotation(
            Line(points = {{-25, 28}, {-25, 45.5}, {-25, 45.5}, {-25, 63}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(L.p, G.p) annotation(
            Line(points = {{-75, 63}, {-25, 63}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(G.n, Nr.p) annotation(
            Line(points = {{25, 63}, {75, 63}, {75, 28}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(C1.p, G.n) annotation(
            Line(points = {{25, 28}, {25, 45.5}, {25, 45.5}, {25, 63}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(Ro.n, Gnd.p) annotation(
            Line(points = {{-75, -42}, {-75, -62}, {0, -62}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(C2.n, Gnd.p) annotation(
            Line(points = {{-25, -22}, {-24, -22}, {-24, -62}, {0, -62}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(Gnd.p, C1.n) annotation(
            Line(points = {{0, -62}, {25, -62}, {25, -22}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(Gnd.p, Nr.n) annotation(
            Line(points = {{0, -62}, {75, -62}, {75, -22}}, color = {0, 0, 255}, smooth = Smooth.None));
          annotation(
            Documentation(info = "<html>
<p>Chua&#39;;s circuit is the most simple nonlinear circuit which shows chaotic behaviour. The circuit consists of linear basic elements (capacitors, resistor, conductor, inductor), and one nonlinear element, which is called Chua&#39;;s diode. The chaotic behaviour is simulated.</p>
<p>The simulation end time should be set to 5e4. To get the chaotic behaviour please plot C1.v. Choose C2.v as the independent variable .</p>
<p><b>Reference:</b></p>
<p>Kennedy, M.P.: Three Steps to Chaos - Part I: Evolution. IEEE Transactions on CAS I 40 (1993)10, 640-656</p>
</html>", revisions = "<html>
<dl>
<dt>
<b>Main Authors:</b>
</dt>
<dd>
Christoph Clau&szlig;
    &lt;<a href=\"mailto:Christoph.Clauss@eas.iis.fraunhofer.de\">Christoph.Clauss@eas.iis.fraunhofer.de</a>&gt;<br>
    Andr&eacute; Schneider
    &lt;<a href=\"mailto:Andre.Schneider@eas.iis.fraunhofer.de\">Andre.Schneider@eas.iis.fraunhofer.de</a>&gt;<br>
    Fraunhofer Institute for Integrated Circuits<br>
    Design Automation Department<br>
    Zeunerstra&szlig;e 38<br>
    D-01069 Dresden<br>
</dd>
</dl>
</html>"),
            experiment(StopTime = 5e4, Interval = 1),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-98, 104}, {-32, 72}}, lineColor = {0, 0, 255}, textString = "Chua Circuit")}));
        end ChuaCircuit;

        encapsulated model DifferenceAmplifier "Simple NPN transistor amplifier circuit"
          import Modelica.Electrical.Analog.Basic;
          import Modelica.Electrical.Analog.Sources;
          import Modelica.Electrical.Analog.Examples.Utilities;
          import Modelica.Icons;
          extends Icons.Example;
          Sources.ExpSineVoltage V1(V = 0.2, freqHz = 0.2e9, damping = 0.1e8) annotation(
            Placement(transformation(origin = {-90, -50}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          Sources.RampVoltage V2(V = 15, duration = 1e-9) annotation(
            Placement(transformation(origin = {80, 50}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          Sources.RampCurrent I1(I = 0.16, duration = 1e-9) annotation(
            Placement(transformation(origin = {-10, -50}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          Basic.Resistor R1(R = 0.0001) annotation(
            Placement(transformation(extent = {{-90, -20}, {-70, 0}}, rotation = 0)));
          Basic.Resistor R2(R = 100) annotation(
            Placement(transformation(origin = {-40, 30}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          Basic.Resistor R3(R = 0.0001) annotation(
            Placement(transformation(extent = {{70, -20}, {90, 0}}, rotation = 0)));
          Basic.Resistor R4(R = 100) annotation(
            Placement(transformation(origin = {40, 30}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          Basic.Capacitor C1(C = 1e-10, v(start = 0, fixed = true)) annotation(
            Placement(transformation(origin = {-60, -50}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          Basic.Capacitor C4(C = 1e-10, v(start = 0, fixed = true)) annotation(
            Placement(transformation(extent = {{10, -14}, {30, 6}}, rotation = 0)));
          Basic.Capacitor C5(C = 1e-10, v(start = 0, fixed = true)) annotation(
            Placement(transformation(origin = {10, -50}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          Basic.Capacitor C2(C = 1e-10, v(start = 0, fixed = true)) annotation(
            Placement(transformation(extent = {{-30, -14}, {-10, 6}}, rotation = 0)));
          Basic.Capacitor C3(C = 1e-10, v(start = 0, fixed = true)) annotation(
            Placement(transformation(origin = {60, -50}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          Basic.Ground Gnd1 annotation(
            Placement(transformation(extent = {{-95, -80}, {-85, -70}}, rotation = 0)));
          Basic.Ground Gnd9 annotation(
            Placement(transformation(extent = {{75, 20}, {85, 30}}, rotation = 0)));
          Basic.Ground Gnd3 annotation(
            Placement(transformation(extent = {{-15, -20}, {-5, -10}}, rotation = 0)));
          Basic.Ground Gnd2 annotation(
            Placement(transformation(extent = {{-65, -80}, {-55, -70}}, rotation = 0)));
          Basic.Ground Gnd6 annotation(
            Placement(transformation(extent = {{5, -20}, {15, -10}}, rotation = 0)));
          Basic.Ground Gnd7 annotation(
            Placement(transformation(extent = {{-15, -80}, {-5, -70}}, rotation = 0)));
          Basic.Ground Gnd8 annotation(
            Placement(transformation(extent = {{5, -80}, {15, -70}}, rotation = 0)));
          Basic.Ground Gnd5 annotation(
            Placement(transformation(extent = {{55, -80}, {65, -70}}, rotation = 0)));
          Basic.Ground Gnd4 annotation(
            Placement(transformation(extent = {{85, -50}, {95, -40}}, rotation = 0)));
          Utilities.Transistor Transistor1(ct(v(start = 0, fixed = true))) annotation(
            Placement(transformation(extent = {{-60, -20}, {-40, 0}}, rotation = 0)));
          Utilities.Transistor Transistor2(ct(v(start = 0, fixed = true))) annotation(
            Placement(transformation(extent = {{60, -20}, {40, 0}}, rotation = 0)));
        equation
          connect(V1.n, Gnd1.p) annotation(
            Line(points = {{-90, -60}, {-90, -70}}, color = {0, 0, 255}));
          connect(C1.n, Gnd2.p) annotation(
            Line(points = {{-60, -60}, {-60, -70}}, color = {0, 0, 255}));
          connect(I1.n, Gnd7.p) annotation(
            Line(points = {{-10, -60}, {-10, -65}, {-10, -70}}, color = {0, 0, 255}));
          connect(C5.n, Gnd8.p) annotation(
            Line(points = {{10, -60}, {10, -70}}, color = {0, 0, 255}));
          connect(C3.n, Gnd5.p) annotation(
            Line(points = {{60, -60}, {60, -70}}, color = {0, 0, 255}));
          connect(R3.n, Gnd4.p) annotation(
            Line(points = {{90, -10}, {90, -40}}, color = {0, 0, 255}));
          connect(C2.n, Gnd3.p) annotation(
            Line(points = {{-10, -4}, {-10, -10}}, color = {0, 0, 255}));
          connect(C4.p, Gnd6.p) annotation(
            Line(points = {{10, -4}, {10, -10}}, color = {0, 0, 255}));
          connect(I1.p, C5.p) annotation(
            Line(points = {{-10, -40}, {10, -40}}, color = {0, 0, 255}));
          connect(R1.p, V1.p) annotation(
            Line(points = {{-90, -10}, {-90, -40}}, color = {0, 0, 255}));
          connect(R2.p, V2.p) annotation(
            Line(points = {{-40, 40}, {-40, 70}, {80, 70}, {80, 60}}, color = {0, 0, 255}));
          connect(R4.p, V2.p) annotation(
            Line(points = {{40, 40}, {40, 70}, {80, 70}, {80, 60}}, color = {0, 0, 255}));
          connect(V2.n, Gnd9.p) annotation(
            Line(points = {{80, 40}, {80, 30}}, color = {0, 0, 255}));
          connect(R1.n, Transistor1.b) annotation(
            Line(points = {{-70, -10}, {-60, -10}}, color = {0, 0, 255}));
          connect(Transistor1.b, C1.p) annotation(
            Line(points = {{-60, -10}, {-60, -40}}, color = {0, 0, 255}));
          connect(Transistor1.c, C2.p) annotation(
            Line(points = {{-40, -4}, {-30, -4}}, color = {0, 0, 255}));
          connect(R2.n, Transistor1.c) annotation(
            Line(points = {{-40, 20}, {-40, -4}}, color = {0, 0, 255}));
          connect(Transistor1.e, I1.p) annotation(
            Line(points = {{-40, -16}, {-30, -16}, {-30, -40}, {-10, -40}}, color = {0, 0, 255}));
          connect(Transistor2.b, R3.p) annotation(
            Line(points = {{60, -10}, {70, -10}}, color = {0, 0, 255}));
          connect(Transistor2.b, C3.p) annotation(
            Line(points = {{60, -10}, {60, -40}}, color = {0, 0, 255}));
          connect(C4.n, Transistor2.c) annotation(
            Line(points = {{30, -4}, {40, -4}}, color = {0, 0, 255}));
          connect(R4.n, Transistor2.c) annotation(
            Line(points = {{40, 20}, {40, -4}}, color = {0, 0, 255}));
          connect(C5.p, Transistor2.e) annotation(
            Line(points = {{10, -40}, {30, -40}, {30, -16}, {40, -16}}, color = {0, 0, 255}));
          annotation(
            Documentation(info = "<html>
<p>It is a simple NPN transistor amplifier circuit. The voltage difference between R1.p and R3.n is amplified. The output signal is the voltage between R2.n and R4.n. In this example the voltage at V1 is amplified because R3.n is grounded.</p>
<p>The simulation end time should be set to 1e- 8. Please plot the input voltage V1.v, and the output voltages R2.n.v, and R4.n.v.</p>
<p><b>Reference:</b></p>
<p>Tietze, U.; Schenk, Ch.: Halbleiter-Schaltungstechnik. Springer-Verlag Berlin Heidelberg NewYork 1980, p. 59</p>
</html>", revisions = "<html>
<dl>
<dt>
<b>Main Authors:</b>
</dt>
<dd>
Christoph Clau&szlig;
    &lt;<a href=\"mailto:Christoph.Clauss@eas.iis.fraunhofer.de\">Christoph.Clauss@eas.iis.fraunhofer.de</a>&gt;<br>
    Andr&eacute; Schneider
    &lt;<a href=\"mailto:Andre.Schneider@eas.iis.fraunhofer.de\">Andre.Schneider@eas.iis.fraunhofer.de</a>&gt;<br>
    Fraunhofer Institute for Integrated Circuits<br>
    Design Automation Department<br>
    Zeunerstra&szlig;e 38<br>
    D-01069 Dresden
</dd>
</dl>
</html>"),
            experiment(StopTime = 1),
            Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-98, 106}, {22, 60}}, lineColor = {0, 0, 255}, textString = "Difference Amplifier")}));
        end DifferenceAmplifier;

        model HeatingMOSInverter "Heating MOS Inverter"
          extends Modelica.Icons.Example;
          Modelica.Electrical.Analog.Basic.Ground G annotation(
            Placement(transformation(extent = {{-80, -40}, {-60, -20}}, rotation = 0)));
          Modelica.Electrical.Analog.Sources.SineVoltage Sin(V = 5, freqHz = 1) annotation(
            Placement(transformation(origin = {-70, 0}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          Modelica.Electrical.Analog.Basic.Capacitor Capacitor1(C = 0.00001, v(start = 0, fixed = true)) annotation(
            Placement(transformation(origin = {30, 10}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          Modelica.Thermal.HeatTransfer.Components.HeatCapacitor HeatCapacitor1(C = 0.01) annotation(
            Placement(transformation(origin = {70, -60}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          Modelica.Thermal.HeatTransfer.Components.ThermalConductor TC1(G = 0.01) annotation(
            Placement(transformation(extent = {{0, -50}, {20, -30}}, rotation = 0)));
          Semiconductors.HeatingPMOS H_PMOS(useHeatPort = true) annotation(
            Placement(transformation(extent = {{-40, 40}, {-20, 60}}, rotation = 0)));
          Semiconductors.HeatingNMOS H_NMOS(useHeatPort = true) annotation(
            Placement(transformation(extent = {{-40, 0}, {-20, 20}}, rotation = 0)));
          Modelica.Electrical.Analog.Sources.RampVoltage V(V = 5, duration = 1.e-2) annotation(
            Placement(transformation(origin = {50, 50}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          Modelica.Thermal.HeatTransfer.Components.ThermalConductor TC2(G = 0.01) annotation(
            Placement(transformation(extent = {{0, -90}, {20, -70}}, rotation = 0)));
          Modelica.Thermal.HeatTransfer.Sources.FixedTemperature FixedTemperature1(T = 300) annotation(
            Placement(transformation(origin = {90, -30}, extent = {{-10, -10}, {10, 10}}, rotation = 180)));
          Modelica.Thermal.HeatTransfer.Components.ThermalConductor TC3(G = 0.01) annotation(
            Placement(transformation(extent = {{50, -40}, {70, -20}}, rotation = 0)));
        initial equation
          HeatCapacitor1.T = 293.15;
        equation
          connect(Sin.n, G.p) annotation(
            Line(points = {{-70, -10}, {-70, -20}}, color = {0, 0, 255}));
          connect(Capacitor1.n, G.p) annotation(
            Line(points = {{30, 0}, {30, -20}, {-70, -20}}, color = {0, 0, 255}));
          connect(H_PMOS.S, H_NMOS.D) annotation(
            Line(points = {{-20, 45}, {-20, 15}}, color = {0, 0, 255}));
          connect(H_NMOS.D, Capacitor1.p) annotation(
            Line(points = {{-20, 15}, {-20, 20}, {30, 20}}, color = {0, 0, 255}));
          connect(H_NMOS.B, H_NMOS.S) annotation(
            Line(points = {{-20, 10}, {-20, 5}}, color = {0, 0, 255}));
          connect(H_NMOS.S, G.p) annotation(
            Line(points = {{-20, 5}, {-20, -20}, {-70, -20}}, color = {0, 0, 255}));
          connect(H_PMOS.B, H_PMOS.D) annotation(
            Line(points = {{-20, 50}, {-20, 55}}, color = {0, 0, 255}));
          connect(V.p, H_PMOS.D) annotation(
            Line(points = {{50, 60}, {-20, 60}, {-20, 55}}, color = {0, 0, 255}));
          connect(V.n, G.p) annotation(
            Line(points = {{50, 40}, {50, -20}, {-70, -20}}, color = {0, 0, 255}));
          connect(TC1.port_b, HeatCapacitor1.port) annotation(
            Line(points = {{20, -40}, {40, -40}, {40, -60}, {60, -60}}, color = {191, 0, 0}));
          connect(TC2.port_b, HeatCapacitor1.port) annotation(
            Line(points = {{20, -80}, {40, -80}, {40, -60}, {60, -60}}, color = {191, 0, 0}));
          connect(TC1.port_a, H_PMOS.heatPort) annotation(
            Line(points = {{0, -40}, {-10, -40}, {-10, 40}, {-30, 40}}, color = {191, 0, 0}));
          connect(TC2.port_a, H_NMOS.heatPort) annotation(
            Line(points = {{0, -80}, {-30, -80}, {-30, 0}}, color = {191, 0, 0}));
          connect(TC3.port_b, FixedTemperature1.port) annotation(
            Line(points = {{70, -30}, {80, -30}}, color = {191, 0, 0}));
          connect(TC3.port_a, HeatCapacitor1.port) annotation(
            Line(points = {{50, -30}, {40, -30}, {40, -60}, {60, -60}}, color = {191, 0, 0}));
          connect(Sin.p, H_NMOS.G) annotation(
            Line(points = {{-70, 10}, {-54, 10}, {-54, 5}, {-40, 5}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(H_PMOS.G, Sin.p) annotation(
            Line(points = {{-40, 45}, {-48, 45}, {-48, 44}, {-54, 44}, {-54, 10}, {-70, 10}}, color = {0, 0, 255}, smooth = Smooth.None));
          annotation(
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-100, 100}, {-6, 72}}, textString = "Heating MOS Inverter", lineColor = {0, 0, 255})}),
            Documentation(info = "<html>
<p>The heating MOS inverter shows a heat flow always if a transistor is leading.</p>
<p>Simulate until T=5 s. Plot in separate windows:<br> Sin.p.v and Capacitor1.p.v<br>HeatCapacitor1.port.T and H_PMOS.heatPort.T and H_NMOS.heatPort.T<br>H_PMOS.heatPort.Q_flow and H_NMOS.heatPort.Q_flow</p>
</html>", revisions = "<html>
<p><b>Release Notes:</b></p>
<ul>
<li><i>Mai 6, 2004   </i>
       by Christoph Clauss<br> realized<br>
       </li>
</ul>
</html>"),
            experiment(StopTime = 5));
        end HeatingMOSInverter;

        model HeatingNPN_OrGate "Heating NPN Or Gate"
          extends Modelica.Icons.Example;
          parameter Modelica.SIunits.Capacitance CapVal = 0 annotation(
            Evaluate = true);
          parameter Modelica.SIunits.Time tauVal = 0 annotation(
            Evaluate = true);
          Modelica.Thermal.HeatTransfer.Components.HeatCapacitor HeatCapacitor1(C = 0.1) annotation(
            Placement(transformation(origin = {76, -84}, extent = {{-10, -10}, {10, 10}}, rotation = 180)));
          Modelica.Thermal.HeatTransfer.Components.ThermalConductor TC1(G = 0.01) annotation(
            Placement(transformation(origin = {90, -50}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          Modelica.Thermal.HeatTransfer.Components.ThermalConductor TC2(G = 0.01) annotation(
            Placement(transformation(origin = {60, -50}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          Modelica.Electrical.Analog.Sources.RampVoltage V(V = 6, duration = 5) annotation(
            Placement(transformation(origin = {90, 38}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          Modelica.Electrical.Analog.Sources.TrapezoidVoltage V1(V = 6, startTime = 55, rising = 5, width = 15, falling = 5, period = 50, nperiod = 10) annotation(
            Placement(transformation(origin = {-90, 18}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          Modelica.Electrical.Analog.Sources.TrapezoidVoltage V2(V = 6, startTime = 65, rising = 5, width = 15, falling = 5, period = 50, nperiod = 10) annotation(
            Placement(transformation(origin = {-50, -42}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          Modelica.Electrical.Analog.Basic.Resistor R1(R = 1800) annotation(
            Placement(transformation(extent = {{-90, 48}, {-70, 68}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Resistor R2(R = 1800) annotation(
            Placement(transformation(extent = {{-36, -22}, {-16, -2}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Resistor RI(R = 40) annotation(
            Placement(transformation(extent = {{60, 58}, {80, 78}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Ground Gnd annotation(
            Placement(transformation(extent = {{80, 0}, {100, 20}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Ground Gnd1 annotation(
            Placement(transformation(extent = {{-100, -22}, {-80, -2}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Ground Gnd2 annotation(
            Placement(transformation(extent = {{-60, -82}, {-40, -62}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Ground Gnd3 annotation(
            Placement(transformation(extent = {{-10, 16}, {10, 36}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Ground Gnd4 annotation(
            Placement(transformation(extent = {{30, -52}, {50, -32}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Capacitor C1(C = CapVal) annotation(
            Placement(transformation(origin = {-70, 38}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          Modelica.Electrical.Analog.Basic.Capacitor C2(C = CapVal) annotation(
            Placement(transformation(origin = {60, 42}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          Modelica.Electrical.Analog.Basic.Capacitor C3(C = CapVal) annotation(
            Placement(transformation(origin = {-16, -40}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          Modelica.Electrical.Analog.Basic.Ground Gnd5 annotation(
            Placement(transformation(extent = {{-26, -76}, {-6, -56}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Ground Gnd6 annotation(
            Placement(transformation(extent = {{50, 6}, {70, 26}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Ground Gnd7 annotation(
            Placement(transformation(extent = {{-80, 2}, {-60, 22}}, rotation = 0)));
          Semiconductors.HeatingNPN T1(Bf = 100, Br = 1, Is = 1.e-14, Vak = 0, Tauf = tauVal, Taur = tauVal, Ccs = CapVal, Cje = CapVal, Cjc = CapVal, Phie = 1, Me = 0.5, Phic = 1, Mc = 0.5, Gbc = 1.e-12, Gbe = 1.e-12, EMax = 40, vt_t(start = 0.01, fixed = false), useHeatPort = true, ibe(start = 0), vbc(start = 0)) annotation(
            Placement(transformation(extent = {{-20, 48}, {0, 68}}, rotation = 0)));
          Semiconductors.HeatingNPN T2(Bf = 100, Br = 1, Is = 1.e-14, Vak = 0, Tauf = tauVal, Taur = tauVal, Ccs = CapVal, Cje = CapVal, Cjc = CapVal, Phie = 1, Me = 0.5, Phic = 1, Mc = 0.5, Gbc = 1.e-12, Gbe = 1.e-12, EMax = 40, vt_t(start = 0.01, fixed = false), useHeatPort = true, ibe(start = 0), vbc(start = 0)) annotation(
            Placement(transformation(extent = {{20, -22}, {40, -2}}, rotation = 0)));
        initial equation
          HeatCapacitor1.T = 293.15;
        equation
          connect(Gnd1.p, V1.n) annotation(
            Line(points = {{-90, -2}, {-90, 8}}, color = {0, 0, 255}));
          connect(V1.p, R1.p) annotation(
            Line(points = {{-90, 28}, {-90, 58}}, color = {0, 0, 255}));
          connect(RI.n, V.p) annotation(
            Line(points = {{80, 68}, {90, 68}, {90, 48}}, color = {0, 0, 255}));
          connect(Gnd.p, V.n) annotation(
            Line(points = {{90, 20}, {90, 28}}, color = {0, 0, 255}));
          connect(V2.p, R2.p) annotation(
            Line(points = {{-50, -32}, {-50, -12}, {-36, -12}}, color = {0, 0, 255}));
          connect(Gnd2.p, V2.n) annotation(
            Line(points = {{-50, -62}, {-50, -52}}, color = {0, 0, 255}));
          connect(Gnd7.p, C1.n) annotation(
            Line(points = {{-70, 22}, {-70, 28}}, color = {0, 0, 255}));
          connect(C2.p, RI.p) annotation(
            Line(points = {{60, 52}, {60, 68}}, color = {0, 0, 255}));
          connect(Gnd6.p, C2.n) annotation(
            Line(points = {{60, 26}, {60, 32}}, color = {0, 0, 255}));
          connect(C3.p, R2.n) annotation(
            Line(points = {{-16, -30}, {-16, -21}, {-16, -12}}, color = {0, 0, 255}));
          connect(C1.p, R1.n) annotation(
            Line(points = {{-70, 48}, {-70, 58}}, color = {0, 0, 255}));
          connect(Gnd5.p, C3.n) annotation(
            Line(points = {{-16, -56}, {-16, -50}}, color = {0, 0, 255}));
          connect(T1.B, R1.n) annotation(
            Line(points = {{-20, 58}, {-70, 58}}, color = {0, 0, 255}));
          connect(T1.E, Gnd3.p) annotation(
            Line(points = {{0, 53}, {0, 48.75}, {0, 44.5}, {0, 36}}, color = {0, 0, 255}));
          connect(RI.p, T1.C) annotation(
            Line(points = {{60, 68}, {30, 68}, {30, 63}, {0, 63}}, color = {0, 0, 255}));
          connect(T2.B, R2.n) annotation(
            Line(points = {{20, -12}, {-16, -12}}, color = {0, 0, 255}));
          connect(T2.E, Gnd4.p) annotation(
            Line(points = {{40, -17}, {40, -32}}, color = {0, 0, 255}));
          connect(T2.C, RI.p) annotation(
            Line(points = {{40, -7}, {40, 68}, {60, 68}}, color = {0, 0, 255}));
          connect(TC1.port_b, HeatCapacitor1.port) annotation(
            Line(points = {{90, -60}, {84, -60}, {84, -74}, {76, -74}}, color = {191, 0, 0}));
          connect(TC2.port_b, HeatCapacitor1.port) annotation(
            Line(points = {{60, -60}, {68, -60}, {68, -74}, {76, -74}}, color = {191, 0, 0}));
          connect(TC2.port_a, T2.heatPort) annotation(
            Line(points = {{60, -40}, {60, -22}, {30, -22}}, color = {191, 0, 0}));
          connect(TC1.port_a, T1.heatPort) annotation(
            Line(points = {{90, -40}, {90, 2}, {-10, 2}, {-10, 48}}, color = {191, 0, 0}));
          annotation(
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-100, 100}, {-6, 72}}, textString = "Heating \"NPN or\" Gate", lineColor = {0, 0, 255})}),
            Documentation(info = "<html>
<p>The heating &quot;NPN or&quot; gate shows a heat flow always if a transistor is leading.</p>
<p>Simulate until T=200 s. Plot in separate windows:
<br>V1.v and V2.v and C2.v
<br>HeatCapacitor1.port.T and T1.heatPort.T and T2.heatPort.T
<br>T1.heatPort.Q_flow and T2.heatPort.Q_flow</p>
</html>", revisions = "<html>
<p><b>Release Notes:</b></p>
<ul>
<li><i>Mai 6, 2004   </i>
       by Christoph Clauss<br> realized<br>
       </li>
</ul>
</html>"),
            experiment(StopTime = 200));
        end HeatingNPN_OrGate;

        model HeatingResistor "Heating resistor"
          extends Modelica.Icons.Example;
          Basic.HeatingResistor heatingResistor(R_ref = 100, alpha = 1e-3, T_ref = 293.15, i(start = 0)) annotation(
            Placement(transformation(extent = {{-10, 10}, {10, -10}}, rotation = -90, origin = {-32, 34})));
          Modelica.Electrical.Analog.Basic.Ground G annotation(
            Placement(transformation(extent = {{-80, 0}, {-60, 20}}, rotation = 0)));
          Modelica.Electrical.Analog.Sources.SineVoltage SineVoltage1(V = 220, freqHz = 1) annotation(
            Placement(transformation(origin = {-70, 40}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          Modelica.Thermal.HeatTransfer.Components.ThermalConductor thermalConductor(G = 50) annotation(
            Placement(transformation(origin = {4, 34}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
          Thermal.HeatTransfer.Celsius.FixedTemperature fixedTemperature(T = 20) annotation(
            Placement(transformation(extent = {{52, 24}, {32, 44}})));
        equation
          connect(SineVoltage1.n, G.p) annotation(
            Line(points = {{-70, 30}, {-70, 20}}, color = {0, 0, 255}));
          connect(heatingResistor.heatPort, thermalConductor.port_a) annotation(
            Line(points = {{-22, 34}, {-6, 34}}, color = {191, 0, 0}));
          connect(SineVoltage1.p, heatingResistor.p) annotation(
            Line(points = {{-70, 50}, {-32, 50}, {-32, 44}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(G.p, heatingResistor.n) annotation(
            Line(points = {{-70, 20}, {-32, 20}, {-32, 24}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(thermalConductor.port_b, fixedTemperature.port) annotation(
            Line(points = {{14, 34}, {32, 34}}, color = {191, 0, 0}, smooth = Smooth.None));
          annotation(
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-94, 102}, {0, 74}}, lineColor = {0, 0, 255}, textString = "HeatingResistor")}),
            Documentation(info = "<html>
<p>This is a very simple circuit consisting of a voltage source and a resistor. The loss power in the resistor is transported to the environment via its heatPort.</p>
</html>"),
            experiment(StopTime = 5));
        end HeatingResistor;

        model HeatingRectifier "Heating rectifier"
          extends Modelica.Icons.Example;
          Modelica.Electrical.Analog.Semiconductors.HeatingDiode HeatingDiode1(useHeatPort = true) annotation(
            Placement(transformation(extent = {{-20, 40}, {0, 60}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Ground G annotation(
            Placement(transformation(extent = {{-80, 0}, {-60, 20}}, rotation = 0)));
          Modelica.Electrical.Analog.Sources.SineVoltage SineVoltage1(V = 1, freqHz = 1) annotation(
            Placement(transformation(origin = {-70, 40}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          Modelica.Electrical.Analog.Basic.Capacitor Capacitor1(C = 1, v(start = 0, fixed = true)) annotation(
            Placement(transformation(extent = {{20, 40}, {40, 60}}, rotation = 0)));
          Modelica.Thermal.HeatTransfer.Components.HeatCapacitor HeatCapacitor1(C = 1) annotation(
            Placement(transformation(origin = {-20, -50}, extent = {{-10, -10}, {10, 10}}, rotation = 180)));
          Modelica.Thermal.HeatTransfer.Components.ThermalConductor ThermalConductor1(G = 10) annotation(
            Placement(transformation(origin = {-20, -10}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          Modelica.Electrical.Analog.Basic.Resistor R(R = 1) annotation(
            Placement(transformation(extent = {{20, 70}, {40, 90}}, rotation = 0)));
        initial equation
          HeatCapacitor1.T = 293.15;
        equation
          connect(SineVoltage1.p, HeatingDiode1.p) annotation(
            Line(points = {{-70, 50}, {-20, 50}}, color = {0, 0, 255}));
          connect(SineVoltage1.n, G.p) annotation(
            Line(points = {{-70, 30}, {-70, 20}}, color = {0, 0, 255}));
          connect(Capacitor1.n, G.p) annotation(
            Line(points = {{40, 50}, {40, 20}, {-70, 20}}, color = {0, 0, 255}));
          connect(HeatingDiode1.n, Capacitor1.p) annotation(
            Line(points = {{0, 50}, {20, 50}}, color = {0, 0, 255}));
          connect(HeatingDiode1.heatPort, ThermalConductor1.port_a) annotation(
            Line(points = {{-10, 40}, {-10, 0}, {-20, 0}}, color = {191, 0, 0}));
          connect(ThermalConductor1.port_b, HeatCapacitor1.port) annotation(
            Line(points = {{-20, -20}, {-20, -25.75}, {-20, -40}}, color = {191, 0, 0}));
          connect(R.p, Capacitor1.p) annotation(
            Line(points = {{20, 80}, {20, 50}}, color = {0, 0, 255}));
          connect(R.n, Capacitor1.n) annotation(
            Line(points = {{40, 80}, {40, 50}}, color = {0, 0, 255}));
          annotation(
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-94, 102}, {0, 74}}, textString = "HeatingRectifier", lineColor = {0, 0, 255})}),
            Documentation(info = "<html>
<p>The heating rectifier shows a heat flow always if the electrical capacitor is loaded.</p>
<p>Simulate until T=5 s.Plot in separate windows:
<br>SineVoltage1.v and Capacitor1.p.v
<br>HeatCapacitor1.port.T and HeatingDiode1.heatPort.T
<br>HeatingDiode1.heatPort.Q_flow
</p>
</html>", revisions = "<html>
<p><b>Release Notes:</b></p>
<ul>
<li><i>Mai 6, 2004   </i>
       by Christoph Clauss<br> realized<br>
       </li>
</ul>
</html>"),
            experiment(StopTime = 5));
        end HeatingRectifier;

        encapsulated model NandGate "CMOS NAND Gate (see Tietze/Schenk, page 157)"
          import Modelica.Electrical.Analog.Basic;
          import Modelica.Electrical.Analog.Sources;
          import Modelica.Electrical.Analog.Examples.Utilities;
          import Modelica.Icons;
          extends Icons.Example;
          Sources.TrapezoidVoltage VIN1(V = 3.5, startTime = 20e-9, rising = 1e-9, width = 19e-9, falling = 1.e-9, period = 40e-9, nperiod = -1) annotation(
            Placement(transformation(origin = {-70, 20}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          Sources.TrapezoidVoltage VIN2(V = 3.5, startTime = 10e-9, rising = 1e-9, width = 19e-9, falling = 1.e-9, period = 40e-9, nperiod = -1) annotation(
            Placement(transformation(origin = {-70, -30}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          Sources.RampVoltage VDD(V = 5, duration = 1e-9) annotation(
            Placement(transformation(origin = {50, 70}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          Basic.Ground Gnd1 annotation(
            Placement(transformation(extent = {{45, 43}, {55, 53}}, rotation = 0)));
          Basic.Ground Gnd4 annotation(
            Placement(transformation(extent = {{-75, -5}, {-65, 5}}, rotation = 0)));
          Basic.Ground Gnd5 annotation(
            Placement(transformation(extent = {{-75, -55}, {-65, -45}}, rotation = 0)));
          Utilities.Nand Nand(C4(v(start = 0, fixed = true)), C7(v(start = 0, fixed = true))) annotation(
            Placement(transformation(extent = {{-40, -45}, {40, 55}}, rotation = 0)));
        equation
          connect(VDD.n, Gnd1.p) annotation(
            Line(points = {{50, 60}, {50, 53}}, color = {0, 0, 255}));
          connect(VIN1.n, Gnd4.p) annotation(
            Line(points = {{-70, 10}, {-70, 5}}, color = {0, 0, 255}));
          connect(VIN2.n, Gnd5.p) annotation(
            Line(points = {{-70, -40}, {-70, -45}}, color = {0, 0, 255}));
          connect(Nand.Vdd, VDD.p) annotation(
            Line(points = {{0, 55}, {0, 90}, {50, 90}, {50, 80}}, color = {0, 0, 255}));
          connect(VIN1.p, Nand.x1) annotation(
            Line(points = {{-70, 30}, {-40, 30}}, color = {0, 0, 255}));
          connect(VIN2.p, Nand.x2) annotation(
            Line(points = {{-70, -20}, {-40, -20}}, color = {0, 0, 255}));
          annotation(
            Documentation(info = "<html>
<p>The nand gate is a basic CMOS building block. It consists of four CMOS transistors. The output voltage Nand.y.v is low if and only if the two input voltages at Nand.x1.v and Nand.x2.v are both high. In this way the nand functionality is realized.</p>
<p>The simulation end time should be set to 1e-7. Please plot the input voltages Nand.x1.v, d Nand.x2.v, and the output voltage Nand.y.v.</p>
<p><b>Reference:</b></p>
<p>Tietze, U.; Schenk, Ch.: Halbleiter-Schaltungstechnik. Springer-Verlag Berlin Heidelberg NewYork 1980, p. 157</p>
</html>", revisions = "<html>
<dl>
<dt>
<b>Main Authors:</b>
</dt>
<dd>
Christoph Clau&szlig;
    &lt;<a href=\"mailto:Christoph.Clauss@eas.iis.fraunhofer.de\">Christoph.Clauss@eas.iis.fraunhofer.de</a>&gt;<br>
    Andr&eacute; Schneider
    &lt;<a href=\"mailto:Andre.Schneider@eas.iis.fraunhofer.de\">Andre.Schneider@eas.iis.fraunhofer.de</a>&gt;<br>
    Fraunhofer Institute for Integrated Circuits<br>
    Design Automation Department<br>
    Zeunerstra&szlig;e 38<br>
    D-01069 Dresden
</dd>
</dl>
</html>"),
            experiment(StopTime = 1e-007),
            Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-90, 98}, {-12, 66}}, lineColor = {0, 0, 255}, textString = "NAND Gate")}));
        end NandGate;

        model OvervoltageProtection "Example for Zener diodes"
          extends Modelica.Icons.Example;
          Modelica.Electrical.Analog.Sources.SineVoltage sineVoltage(offset = 0, V = 10, freqHz = 5) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 270, origin = {-66, 6})));
          Modelica.Electrical.Analog.Basic.Resistor Rv(R = 20) annotation(
            Placement(transformation(extent = {{-56, 32}, {-36, 52}})));
          Modelica.Electrical.Analog.Basic.Resistor RL(R = 2000) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 270, origin = {26, 4})));
          Modelica.Electrical.Analog.Basic.Ground ground annotation(
            Placement(transformation(extent = {{-30, -82}, {-10, -62}})));
          Modelica.Electrical.Analog.Basic.Capacitor CL(C = 1e-7, v(start = 0, fixed = true)) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 270, origin = {62, 4})));
          Semiconductors.ZDiode zDiode annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 90, origin = {-20, -20})));
          Semiconductors.ZDiode zDiode1(v(start = 0)) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 270, origin = {-20, 22})));
        equation
          connect(sineVoltage.p, Rv.p) annotation(
            Line(points = {{-66, 16}, {-66, 42}, {-56, 42}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(RL.p, CL.p) annotation(
            Line(points = {{26, 14}, {62, 14}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(RL.n, CL.n) annotation(
            Line(points = {{26, -6}, {62, -6}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(sineVoltage.n, zDiode.p) annotation(
            Line(points = {{-66, -4}, {-66, -30}, {-20, -30}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(zDiode.p, ground.p) annotation(
            Line(points = {{-20, -30}, {-20, -62}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(zDiode.p, RL.n) annotation(
            Line(points = {{-20, -30}, {26, -30}, {26, -6}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(zDiode1.p, Rv.n) annotation(
            Line(points = {{-20, 32}, {-20, 42}, {-36, 42}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(RL.p, Rv.n) annotation(
            Line(points = {{26, 14}, {26, 42}, {-36, 42}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(zDiode1.n, zDiode.n) annotation(
            Line(points = {{-20, 12}, {-20, 1}, {-20, 1}, {-20, -10}}, color = {0, 0, 255}, smooth = Smooth.None));
          annotation(
            experiment(StopTime = 0.4),
            Documentation(info = "<html>
<p>This example is a simple circuit for overvoltage protection. If the voltage zDiode_1.n.v is too high, the Diode zDiode_2 breaks through and the voltage gets down.</p>
<p>The simulation end time should be set to 0.4. To get the typical behaviour please plot sineVoltage.p.v, RL.p.v, zDiode_2.n.v and zDiode_1.n.i.</p>
</html>", revisions = "<html>
<ul>
<li><i>February 02, 2009</i>
       by Kristin Majetta<br>
       documentation added</li>
<li><i> January 08, 2009   </i>
        by Matthias Franke <br>initially implemented
       </li>
</ul>
</html>"));
        end OvervoltageProtection;

        model Rectifier "B6 diode bridge"
          extends Modelica.Icons.Example;
          import Modelica.Electrical.Analog.Ideal;
          parameter Modelica.SIunits.Voltage VAC = 400 "RMS line-to-line";
          parameter Modelica.SIunits.Frequency f = 50 "line frequency";
          parameter Modelica.SIunits.Inductance LAC = 60E-6 "line inductor";
          parameter Modelica.SIunits.Resistance Ron = 1E-3 "diode forward resistance";
          parameter Modelica.SIunits.Conductance Goff = 1E-3 "diode backward conductance";
          parameter Modelica.SIunits.Voltage Vknee = 2 "diode threshold voltage";
          parameter Modelica.SIunits.Capacitance CDC = 15E-3 "DC capacitance";
          parameter Modelica.SIunits.Current IDC = 500 "load current";
          output Modelica.SIunits.Voltage uDC;
          output Modelica.SIunits.Current iAC[3];
          output Modelica.SIunits.Voltage uAC[3];
          output Modelica.SIunits.Power Losses;
          Modelica.Electrical.Analog.Sources.SineVoltage SineVoltage1(freqHz = f, V = VAC * sqrt(2 / 3)) annotation(
            Placement(transformation(extent = {{-70, 10}, {-90, 30}}, rotation = 0)));
          Modelica.Electrical.Analog.Sources.SineVoltage SineVoltage2(freqHz = f, phase = -2 / 3 * Modelica.Constants.pi, V = VAC * sqrt(2 / 3)) annotation(
            Placement(transformation(extent = {{-70, -10}, {-90, 10}}, rotation = 0)));
          Modelica.Electrical.Analog.Sources.SineVoltage SineVoltage3(freqHz = f, phase = -4 / 3 * Modelica.Constants.pi, V = VAC * sqrt(2 / 3)) annotation(
            Placement(transformation(extent = {{-70, -30}, {-90, -10}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Inductor Inductor1(L = LAC) annotation(
            Placement(transformation(extent = {{-60, 10}, {-40, 30}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Inductor Inductor2(L = LAC, i(start = 0, fixed = true)) annotation(
            Placement(transformation(extent = {{-60, -10}, {-40, 10}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Inductor Inductor3(L = LAC, i(start = 0, fixed = true)) annotation(
            Placement(transformation(extent = {{-60, -30}, {-40, -10}}, rotation = 0)));
          Ideal.IdealDiode IdealDiode1(Ron = Ron, Goff = Goff, Vknee = Vknee) annotation(
            Placement(transformation(origin = {-20, 40}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
          Ideal.IdealDiode IdealDiode2(Ron = Ron, Goff = Goff, Vknee = Vknee) annotation(
            Placement(transformation(origin = {0, 40}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
          Ideal.IdealDiode IdealDiode3(Ron = Ron, Goff = Goff, Vknee = Vknee) annotation(
            Placement(transformation(origin = {20, 40}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
          Ideal.IdealDiode IdealDiode4(Ron = Ron, Goff = Goff, Vknee = Vknee) annotation(
            Placement(transformation(origin = {-20, -40}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
          Ideal.IdealDiode IdealDiode5(Ron = Ron, Goff = Goff, Vknee = Vknee) annotation(
            Placement(transformation(origin = {0, -40}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
          Ideal.IdealDiode IdealDiode6(Ron = Ron, Goff = Goff, Vknee = Vknee) annotation(
            Placement(transformation(origin = {20, -40}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
          Modelica.Electrical.Analog.Basic.Capacitor Capacitor1(C = 2 * CDC) annotation(
            Placement(transformation(origin = {40, 40}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          Modelica.Electrical.Analog.Basic.Capacitor Capacitor2(C = 2 * CDC) annotation(
            Placement(transformation(origin = {40, -40}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          Modelica.Electrical.Analog.Basic.Ground Ground1 annotation(
            Placement(transformation(extent = {{40, -80}, {60, -60}}, rotation = 0)));
          Modelica.Electrical.Analog.Sources.SignalCurrent SignalCurrent1 annotation(
            Placement(transformation(origin = {60, 0}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          Modelica.Blocks.Sources.Constant Constant1(k = IDC) annotation(
            Placement(transformation(extent = {{100, -10}, {80, 10}}, rotation = 0)));
        initial equation
          Capacitor1.v = VAC * sqrt(2) / 2;
          Capacitor2.v = VAC * sqrt(2) / 2;
        equation
          uDC = Capacitor1.v + Capacitor2.v;
          iAC = {Inductor1.i, Inductor2.i, Inductor3.i};
          uAC[1] = Inductor1.n.v - Inductor2.n.v;
          uAC[2] = Inductor2.n.v - Inductor3.n.v;
          uAC[3] = Inductor3.n.v - Inductor1.n.v;
          Losses = IdealDiode1.v * IdealDiode1.i + IdealDiode2.v * IdealDiode2.i + IdealDiode3.v * IdealDiode3.i + IdealDiode4.v * IdealDiode4.i + IdealDiode5.v * IdealDiode5.i + IdealDiode6.v * IdealDiode6.i;
          connect(SineVoltage1.n, SineVoltage2.n) annotation(
            Line(points = {{-90, 20}, {-90, 0}}, color = {0, 0, 255}));
          connect(SineVoltage2.n, SineVoltage3.n) annotation(
            Line(points = {{-90, 0}, {-90, -20}}, color = {0, 0, 255}));
          connect(SineVoltage1.p, Inductor1.p) annotation(
            Line(points = {{-70, 20}, {-60, 20}}, color = {0, 0, 255}));
          connect(SineVoltage2.p, Inductor2.p) annotation(
            Line(points = {{-70, 0}, {-67.5, 0}, {-65, 0}, {-60, 0}}, color = {0, 0, 255}));
          connect(SineVoltage3.p, Inductor3.p) annotation(
            Line(points = {{-70, -20}, {-60, -20}}, color = {0, 0, 255}));
          connect(IdealDiode1.p, IdealDiode4.n) annotation(
            Line(points = {{-20, 30}, {-20, -30}}, color = {0, 0, 255}));
          connect(IdealDiode2.p, IdealDiode5.n) annotation(
            Line(points = {{0, 30}, {0, 16}, {0, 0}, {0, -30}}, color = {0, 0, 255}));
          connect(IdealDiode3.p, IdealDiode6.n) annotation(
            Line(points = {{20, 30}, {20, -30}}, color = {0, 0, 255}));
          connect(IdealDiode1.n, IdealDiode2.n) annotation(
            Line(points = {{-20, 50}, {0, 50}}, color = {0, 0, 255}));
          connect(IdealDiode2.n, IdealDiode3.n) annotation(
            Line(points = {{0, 50}, {20, 50}}, color = {0, 0, 255}));
          connect(IdealDiode4.p, IdealDiode5.p) annotation(
            Line(points = {{-20, -50}, {0, -50}}, color = {0, 0, 255}));
          connect(IdealDiode5.p, IdealDiode6.p) annotation(
            Line(points = {{0, -50}, {20, -50}}, color = {0, 0, 255}));
          connect(Capacitor2.n, IdealDiode6.p) annotation(
            Line(points = {{40, -50}, {20, -50}}, color = {0, 0, 255}));
          connect(IdealDiode3.n, Capacitor1.p) annotation(
            Line(points = {{20, 50}, {40, 50}}, color = {0, 0, 255}));
          connect(Capacitor1.n, Capacitor2.p) annotation(
            Line(points = {{40, 30}, {40, -30}}, color = {0, 0, 255}));
          connect(Capacitor2.p, Ground1.p) annotation(
            Line(points = {{40, -30}, {40, 0}, {50, 0}, {50, -60}}, color = {0, 0, 255}));
          connect(Capacitor1.p, SignalCurrent1.p) annotation(
            Line(points = {{40, 50}, {60, 50}, {60, 10}}, color = {0, 0, 255}));
          connect(SignalCurrent1.n, Capacitor2.n) annotation(
            Line(points = {{60, -10}, {60, -50}, {40, -50}}, color = {0, 0, 255}));
          connect(Constant1.y, SignalCurrent1.i) annotation(
            Line(points = {{79, 0}, {79, 0}, {67, 0}}, color = {0, 0, 255}));
          connect(Inductor1.n, IdealDiode1.p) annotation(
            Line(points = {{-40, 20}, {-20, 20}, {-20, 30}}, color = {0, 0, 255}));
          connect(Inductor2.n, IdealDiode2.p) annotation(
            Line(points = {{-40, 0}, {0, 0}, {0, 30}}, color = {0, 0, 255}));
          connect(Inductor3.n, IdealDiode3.p) annotation(
            Line(points = {{-40, -20}, {20, -20}, {20, 30}}, color = {0, 0, 255}));
          annotation(
            Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-80, 90}, {80, 70}}, lineColor = {0, 0, 0}, textString = "Rectifier"), Line(points = {{-16, 18}, {-16, 2}, {-18, 6}, {-14, 6}, {-16, 2}}, color = {0, 0, 0}), Line(points = {{-30, 22}, {-26, 20}, {-30, 18}, {-30, 22}}, color = {0, 0, 0}), Line(points = {{32, 30}, {32, -30}, {30, -26}, {34, -26}, {32, -30}}, color = {0, 0, 0}), Text(extent = {{-38, 16}, {-22, 8}}, lineColor = {0, 0, 0}, textString = "iAC"), Text(extent = {{-14, 8}, {2, 0}}, lineColor = {0, 0, 0}, textString = "uAC"), Text(extent = {{22, -16}, {38, -24}}, lineColor = {0, 0, 0}, textString = "uDC")}),
            experiment(StopTime = 0.1, Interval = 1e-005),
            Documentation(info = "<html>
<p>The rectifier example shows a B6 diode bridge fed by a three phase sinusoidal voltage, loaded by a DC current.
DC capacitors start at ideal no-load voltage, thus making easier initial transient.</p>
<p>Simulate until T=0.1 s. Plot in separate windows:
<br>uDC ... DC-voltage
<br>iAC ... AC-currents 1..3
<br>uAC ... AC-voltages 1..3 (distorted)
<br>Try different load currents iDC = 0..approximately 500 A. You may watch losses (of the whole diode bridge) trying different diode parameters.</p>
</html>", revisions = "<html>
<p><b>Release Notes:</b></p>
<ul>
<li><i>Mai 7, 2004   </i>
       by Anton Haumer<br> realized<br>
       </li>
</ul>
</html>"));
        end Rectifier;

        model ShowSaturatingInductor "Simple demo to show behaviour of SaturatingInductor component"
          extends Modelica.Icons.Example;
          parameter Modelica.SIunits.Inductance Lzer = 2 "Inductance near current=0";
          parameter Modelica.SIunits.Inductance Lnom = 1 "Nominal inductance at Nominal current";
          parameter Modelica.SIunits.Current Inom = 1 "Nominal current";
          parameter Modelica.SIunits.Inductance Linf = 0.5 "Inductance at large currents";
          parameter Modelica.SIunits.Voltage U = 1.25 "Source voltage (peak)";
          parameter Modelica.SIunits.Frequency f = 1 / (2 * Modelica.Constants.pi) "Source frequency";
          parameter Modelica.SIunits.Angle phase = Modelica.Constants.pi / 2 "Source voltage phase shift";
          //output Modelica.SIunits.Voltage v "Voltage drop over saturating inductor";
          //output Modelica.SIunits.Current i "Current across saturating inductor";
          Modelica.Electrical.Analog.Sources.SineVoltage SineVoltage1(V = U, phase = phase, freqHz = f) annotation(
            Placement(transformation(origin = {-60, -6}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          Modelica.Electrical.Analog.Basic.Ground Ground1 annotation(
            Placement(transformation(extent = {{-70, -36}, {-50, -16}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.SaturatingInductor SaturatingInductance1(Lzer = Lzer, Lnom = Lnom, Inom = Inom, Linf = Linf, i(start = 0)) annotation(
            Placement(transformation(origin = {-20, 0}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          Basic.Inductor Inductance1(L = Lnom, i(start = 0, fixed = true)) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 270, origin = {20, 0})));
        equation
//v=SaturatingInductance1.v;
//i=SaturatingInductance1.i;
          connect(SineVoltage1.n, Ground1.p) annotation(
            Line(points = {{-60, -16}, {-60, -16}}, color = {0, 0, 255}));
          connect(SineVoltage1.n, SaturatingInductance1.n) annotation(
            Line(points = {{-60, -16}, {-20, -16}, {-20, -10}}, color = {0, 0, 255}));
          connect(SaturatingInductance1.p, SineVoltage1.p) annotation(
            Line(points = {{-20, 10}, {-20, 20}, {-60, 20}, {-60, 4}}, color = {0, 0, 255}));
          connect(Inductance1.p, SineVoltage1.p) annotation(
            Line(points = {{20, 10}, {20, 20}, {-60, 20}, {-60, 4}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(Inductance1.n, SineVoltage1.n) annotation(
            Line(points = {{20, -10}, {20, -16}, {-60, -16}}, color = {0, 0, 255}, smooth = Smooth.None));
          annotation(
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-80, 84}, {70, 38}}, lineColor = {0, 0, 255}, textString = "Show Saturating Inductor")}),
            experiment(StopTime = 6.2832, Interval = 0.01),
            Documentation(info = "<html>
<p>This simple circuit uses the saturating inductor which has a changing inductivity.</p>
<p>This circuit should be simulated until 1 s. Compare <code>SaturatingInductance1.p.i</code> with <code>Inductance1.p.i</code> to see the difference between saturating and ideal inductor.</p>
</html>"));
        end ShowSaturatingInductor;

        model ShowVariableResistor "Simple demo of a VariableResistor model"
          extends Modelica.Icons.Example;
          Modelica.Electrical.Analog.Basic.VariableResistor VariableResistor annotation(
            Placement(transformation(extent = {{-20, -20}, {0, 0}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Ground Ground1 annotation(
            Placement(transformation(extent = {{-100, -80}, {-80, -60}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Ground Ground2 annotation(
            Placement(transformation(extent = {{60, -40}, {80, -20}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Resistor R1(R = 1) annotation(
            Placement(transformation(extent = {{-60, 40}, {-40, 60}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Resistor R2(R = 1) annotation(
            Placement(transformation(extent = {{-20, 40}, {0, 60}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Resistor R3(R = 1) annotation(
            Placement(transformation(extent = {{20, 40}, {40, 60}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Resistor R4(R = 1) annotation(
            Placement(transformation(extent = {{-60, -20}, {-40, 0}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Resistor R5(R = 1) annotation(
            Placement(transformation(extent = {{20, -20}, {40, 0}}, rotation = 0)));
          Modelica.Electrical.Analog.Sources.SineVoltage SineVoltage1(V = 1, freqHz = 1) annotation(
            Placement(transformation(origin = {-90, -30}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
          Modelica.Blocks.Sources.Ramp Ramp1(height = 5, offset = 2, duration = 2) annotation(
            Placement(transformation(origin = {-10, 20}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
        equation
          connect(R1.n, R2.p) annotation(
            Line(points = {{-40, 50}, {-20, 50}}, color = {0, 0, 255}));
          connect(R2.n, R3.p) annotation(
            Line(points = {{0, 50}, {20, 50}}, color = {0, 0, 255}));
          connect(R4.n, VariableResistor.p) annotation(
            Line(points = {{-40, -10}, {-20, -10}}, color = {0, 0, 255}));
          connect(VariableResistor.n, R5.p) annotation(
            Line(points = {{0, -10}, {20, -10}}, color = {0, 0, 255}));
          connect(R3.n, Ground2.p) annotation(
            Line(points = {{40, 50}, {70, 50}, {70, -20}}, color = {0, 0, 255}));
          connect(Ground2.p, R5.n) annotation(
            Line(points = {{70, -20}, {70, -10}, {40, -10}}, color = {0, 0, 255}));
          connect(SineVoltage1.p, Ground1.p) annotation(
            Line(points = {{-90, -40}, {-90, -60}}, color = {0, 0, 255}));
          connect(SineVoltage1.n, R1.p) annotation(
            Line(points = {{-90, -20}, {-90, 50}, {-60, 50}}, color = {0, 0, 255}));
          connect(SineVoltage1.n, R4.p) annotation(
            Line(points = {{-90, -20}, {-90, -10}, {-60, -10}}, color = {0, 0, 255}));
          connect(Ramp1.y, VariableResistor.R) annotation(
            Line(points = {{-10, 9}, {-10, 4.5}, {-10, 1}}, color = {0, 0, 255}));
          annotation(
            Documentation(info = "<html>
<p>It is a simple test circuit for the VariableResistor. The VariableResistor should be compared with R2.</p>
<p>Simulate until T=1 s.</p>
</html>", revisions = "<html>
<p><b>Release Notes:</b></p>
<ul>
<li><i>Mai 6, 2004   </i>
       by Teresa Schlegel<br> realized<br>
       </li>
</ul>
</html>"),
            experiment(StopTime = 1),
            Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-100, 112}, {80, 40}}, lineColor = {0, 0, 255}, textString = "Example VariableResistor")}));
        end ShowVariableResistor;

        model SwitchWithArc "Comparison of switch models both with and without arc"
          extends Modelica.Icons.Example;
          Modelica.Blocks.Sources.BooleanPulse booleanPulse(period = 1) annotation(
            Placement(transformation(extent = {{-80, -30}, {-60, -10}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Ground ground1 annotation(
            Placement(transformation(extent = {{20, -20}, {40, 0}}, rotation = 0)));
          Modelica.Electrical.Analog.Sources.ConstantVoltage constantVoltage1(V = 50) annotation(
            Placement(transformation(origin = {-20, 20}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          Modelica.Electrical.Analog.Basic.Inductor inductor1(L = 0.1, i(start = 0, fixed = true)) annotation(
            Placement(transformation(extent = {{40, 30}, {60, 50}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Resistor resistor1(R = 1) annotation(
            Placement(transformation(origin = {80, 20}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          Modelica.Electrical.Analog.Ideal.IdealClosingSwitch switch1 annotation(
            Placement(transformation(extent = {{0, 30}, {20, 50}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Ground ground2 annotation(
            Placement(transformation(extent = {{20, -100}, {40, -80}}, rotation = 0)));
          Modelica.Electrical.Analog.Sources.ConstantVoltage constantVoltage2(V = 50) annotation(
            Placement(transformation(origin = {-20, -60}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          Modelica.Electrical.Analog.Basic.Inductor inductor2(L = 0.1, i(start = 0, fixed = true)) annotation(
            Placement(transformation(extent = {{40, -50}, {60, -30}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Resistor resistor2(R = 1) annotation(
            Placement(transformation(origin = {80, -60}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          Modelica.Electrical.Analog.Ideal.CloserWithArc switch2(dVdt = 10000, V0 = 30, Vmax = 60) annotation(
            Placement(transformation(extent = {{0, -50}, {20, -30}}, rotation = 0)));
        equation
          connect(inductor1.n, resistor1.p) annotation(
            Line(points = {{60, 40}, {80, 40}, {80, 30}}, color = {0, 0, 255}));
          connect(resistor1.n, ground1.p) annotation(
            Line(points = {{80, 10}, {80, 0}, {30, 0}}, color = {0, 0, 255}));
          connect(constantVoltage1.n, ground1.p) annotation(
            Line(points = {{-20, 10}, {-20, 0}, {30, 0}}, color = {0, 0, 255}));
          connect(switch1.n, inductor1.p) annotation(
            Line(points = {{20, 40}, {40, 40}}, color = {0, 0, 255}));
          connect(constantVoltage1.p, switch1.p) annotation(
            Line(points = {{-20, 30}, {-20, 40}, {0, 40}}, color = {0, 0, 255}));
          connect(inductor2.n, resistor2.p) annotation(
            Line(points = {{60, -40}, {80, -40}, {80, -50}}, color = {0, 0, 255}));
          connect(resistor2.n, ground2.p) annotation(
            Line(points = {{80, -70}, {80, -80}, {30, -80}}, color = {0, 0, 255}));
          connect(constantVoltage2.n, ground2.p) annotation(
            Line(points = {{-20, -70}, {-20, -80}, {30, -80}}, color = {0, 0, 255}));
          connect(switch2.n, inductor2.p) annotation(
            Line(points = {{20, -40}, {40, -40}}, color = {0, 0, 255}));
          connect(constantVoltage2.p, switch2.p) annotation(
            Line(points = {{-20, -50}, {-20, -40}, {0, -40}}, color = {0, 0, 255}));
          connect(booleanPulse.y, switch1.control) annotation(
            Line(points = {{-59, -20}, {-40, -20}, {-40, 60}, {10, 60}, {10, 47}}, color = {255, 0, 255}));
          connect(booleanPulse.y, switch2.control) annotation(
            Line(points = {{-59, -20}, {10, -20}, {10, -30}}, color = {255, 0, 255}));
          annotation(
            Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-100, 80}, {100, 60}}, lineColor = {0, 0, 255}, textString = "Compare voltage and current of inductor1 and inductor2")}),
            experiment(StopTime = 2, Interval = 0.00025, Tolerance = 1e-006),
            Documentation(info = "<html>
<p align=\"justify\">This example is to compare the behaviour of switch models with and without an electric arc taking into consideration.</p>
<ul>
<li>Simulate until T=2 s. </li>
<li>Plot in one window:  <code>switch1.i</code> and <code>switch2.i</code> </li>
</ul>
<p>The difference in the closing area shows that the simple arc model avoids the suddenly switching.</p>
</html>", revisions = "<html>
<ul>
<li><i>May, 2009   </i>
       by Anton Haumer<br> initially realized<br>
       </li>
</ul>
</html>"));
        end SwitchWithArc;

        model ThyristorBehaviourTest "Thyristor demonstration example"
          extends Modelica.Icons.Example;
          Modelica.Electrical.Analog.Basic.Ground ground annotation(
            Placement(transformation(extent = {{18, -40}, {38, -20}})));
          Modelica.Electrical.Analog.Sources.SineVoltage sineVoltage(V = 30, freqHz = 10000) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 270, origin = {-72, 4})));
          Modelica.Electrical.Analog.Basic.Resistor resistor(R = 10) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 270, origin = {8, 20})));
          Modelica.Electrical.Analog.Sources.PulseCurrent pulseCurrent(width = 10, period = 0.0001, startTime = 0.00002, I = 0.005) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 180, origin = {-8, 70})));
          Modelica.Electrical.Analog.Semiconductors.Thyristor thyristor_v4_1(vControl(fixed = true), vAK(start = 0), vGK(start = 0)) annotation(
            Placement(transformation(extent = {{-44, 30}, {-24, 50}})));
          Modelica.Electrical.Analog.Basic.Inductor inductor(L = 2e-6, i(start = 0, fixed = true)) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 270, origin = {8, -4})));
        equation
          connect(sineVoltage.n, ground.p) annotation(
            Line(points = {{-72, -6}, {-72, -20}, {28, -20}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(sineVoltage.p, thyristor_v4_1.Anode) annotation(
            Line(points = {{-72, 14}, {-72, 40}, {-43, 40}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(resistor.p, thyristor_v4_1.Cathode) annotation(
            Line(points = {{8, 30}, {8, 40}, {-25, 40}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(pulseCurrent.n, thyristor_v4_1.Gate) annotation(
            Line(points = {{-18, 70}, {-26, 70}, {-26, 49}, {-27, 49}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(resistor.p, pulseCurrent.p) annotation(
            Line(points = {{8, 30}, {8, 70}, {2, 70}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(resistor.n, inductor.p) annotation(
            Line(points = {{8, 10}, {8, 8}, {8, 6}, {8, 6}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(inductor.n, ground.p) annotation(
            Line(points = {{8, -14}, {8, -20}, {28, -20}}, color = {0, 0, 255}, smooth = Smooth.None));
          annotation(
            experiment(StopTime = 0.0002),
            Documentation(info = "<html>
<p>This is a simple test circuit, to test the behavior of the thyristor model.</p><p>Interesting values to plot are Cathode.v, Gate.v and sineVoltage.p.v. and in another plot window pulseCurrent.p.i</p><p>The simulation time should be from 0 seconds to 2e-4 seconds.</p>
</html>"));
        end ThyristorBehaviourTest;

        model AmplifierWithOpAmpDetailed "Simple Amplifier circuit which uses OpAmpDetailed"
          extends Modelica.Icons.Example;
          Modelica.Electrical.Analog.Basic.OpAmpDetailed opAmp annotation(
            Placement(transformation(extent = {{-26, -4}, {2, 24}})));
          Modelica.Electrical.Analog.Basic.Resistor resistor(R = 10000) annotation(
            Placement(transformation(extent = {{-74, 8}, {-56, 26}})));
          Modelica.Electrical.Analog.Basic.Resistor resistor1(R = 20000) annotation(
            Placement(transformation(extent = {{-22, 46}, {-4, 64}})));
          Modelica.Electrical.Analog.Basic.Resistor resistor2(R = 10000) annotation(
            Placement(transformation(extent = {{-9, -9}, {9, 9}}, rotation = 270, origin = {21, -23})));
          Modelica.Electrical.Analog.Sources.SineVoltage sineVoltage(V = 12, freqHz = 1000) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 270, origin = {-80, -30})));
          Modelica.Electrical.Analog.Basic.Ground ground annotation(
            Placement(transformation(extent = {{-20, -70}, {0, -50}})));
          Modelica.Electrical.Analog.Sources.ConstantVoltage constantVoltage(V = 15) annotation(
            Placement(transformation(extent = {{48, 10}, {68, 30}})));
          Modelica.Electrical.Analog.Sources.ConstantVoltage constantVoltage1(V = -15) annotation(
            Placement(transformation(extent = {{48, -10}, {68, 10}})));
        initial equation
          resistor2.i = 0;
          opAmp.q_fp1 = 0;
          opAmp.q_fr1 = 0;
          opAmp.q_fr2 = 0;
          opAmp.q_fr3 = 0;
        equation
          connect(resistor.n, opAmp.m) annotation(
            Line(points = {{-56, 17}, {-34, 17}, {-34, 17.07}, {-26.07, 17.07}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(resistor1.n, resistor2.p) annotation(
            Line(points = {{-4, 55}, {21, 55}, {21, -14}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(resistor.p, sineVoltage.p) annotation(
            Line(points = {{-74, 17}, {-80, 17}, {-80, -20}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(resistor1.p, opAmp.m) annotation(
            Line(points = {{-22, 55}, {-26.07, 55}, {-26.07, 17.07}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(sineVoltage.n, ground.p) annotation(
            Line(points = {{-80, -40}, {-10, -40}, {-10, -50}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(opAmp.p, ground.p) annotation(
            Line(points = {{-26.07, 2.93}, {-26.07, -20}, {-10, -20}, {-10, -50}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(resistor2.n, ground.p) annotation(
            Line(points = {{21, -32}, {21, -40}, {-10, -40}, {-10, -50}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(opAmp.p_supply, constantVoltage.p) annotation(
            Line(points = {{-11.93, 20.29}, {18.035, 20.29}, {18.035, 20}, {48, 20}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(opAmp.m_supply, constantVoltage1.p) annotation(
            Line(points = {{-12, -0.36}, {18, -0.36}, {18, 0}, {48, 0}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(constantVoltage.n, constantVoltage1.n) annotation(
            Line(points = {{68, 20}, {80, 20}, {80, 0}, {68, 0}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(constantVoltage1.n, ground.p) annotation(
            Line(points = {{68, 0}, {80, 0}, {80, -40}, {-10, -40}, {-10, -50}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(opAmp.outp, resistor2.p) annotation(
            Line(points = {{2, 10}, {21, 10}, {21, -14}}, color = {0, 0, 255}, smooth = Smooth.None));
          annotation(
            experiment(StopTime = 0.003, Interval = 1.2e-6, Tolerance = 2e-007),
            Documentation(revisions = "<html>
<dl>
<dt>June 17, 2009</dt>
<dd>by Susann Wolf realized</dd>
</dl>
</html>", info = "<html>
<p>With the test circuit AmplifierWithOpAmpDetailed a time domain analysis of the example arrangement with a sinusoidal input voltage (12 V amplitude, frequency 1 kHz) using the operational amplifier model OpAmpDetailed is carried out. The working voltages are 15 V and -15 V.</p>
</html>"));
        end AmplifierWithOpAmpDetailed;

        model CompareTransformers "Transformer circuit to show the magnetization facilities"
          extends Modelica.Icons.Example;
          constant Modelica.SIunits.Angle pi = Modelica.Constants.pi;
          parameter Modelica.SIunits.Voltage Vdc = 0.1 "DC offset of voltage source";
          parameter Modelica.SIunits.Voltage Vpeak = 0.1 "Peak voltage of voltage source";
          parameter Modelica.SIunits.Frequency f = 10 "Frequency of voltage source";
          parameter Modelica.SIunits.Angle phi0 = pi / 2 "Phase of voltage source";
          parameter Real n = 2 "Turns ratio primary:secondary voltage";
          parameter Modelica.SIunits.Resistance R1 = 0.01 "Primary resistance w.r.t. primary side";
          parameter Modelica.SIunits.Inductance L1sigma = 0.05 / (2 * pi * f) "Primary leakage inductance w.r.t. primary side";
          parameter Modelica.SIunits.Inductance Lm1 = 10. / (2 * pi * f) "Magnetizing inductance w.r.t. primary side";
          parameter Modelica.SIunits.Inductance L2sigma = 0.05 / (2 * pi * f) / n ^ 2 "Secondary leakage inductance w.r.t. secondary side";
          parameter Modelica.SIunits.Resistance R2 = 0.01 / n ^ 2 "Secondary resistance w.r.t. secondary side";
          parameter Modelica.SIunits.Resistance RL = 1 / n ^ 2 "Load resistance";
          final parameter Modelica.SIunits.Inductance L1 = L1sigma + M * n "Primary no-load inductance";
          final parameter Modelica.SIunits.Inductance L2 = L2sigma + M / n "Secondary no-load inductance";
          final parameter Modelica.SIunits.Inductance M = Lm1 / n "Mutual inductance";
          output Modelica.SIunits.Voltage v1B = resistor11.n.v "Primary voltage, basic transformer";
          output Modelica.SIunits.Current i1B = resistor11.i "Primary current, basic transformer";
          output Modelica.SIunits.Voltage v2B = resistor12.p.v "Secondary voltage, basic transformer";
          output Modelica.SIunits.Current i2B = resistor12.i "Secondary current, basic transformer";
          output Modelica.SIunits.Voltage v1I = resistor21.n.v "Primary voltage, basic transformer";
          output Modelica.SIunits.Current i1I = resistor21.i "Primary current, basic transformer";
          output Modelica.SIunits.Voltage v2I = resistor22.p.v "Secondary voltage, basic transformer";
          output Modelica.SIunits.Current i2I = resistor22.i "Secondary current, basic transformer";
          Modelica.Electrical.Analog.Sources.SineVoltage sineVoltage1(V = Vpeak, phase = phi0, freqHz = f, offset = Vdc) annotation(
            Placement(transformation(origin = {-80, 40}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          Modelica.Electrical.Analog.Basic.Ground ground11 annotation(
            Placement(transformation(extent = {{-90, 0}, {-70, 20}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Resistor resistor11(R = R1) annotation(
            Placement(transformation(extent = {{-80, 50}, {-60, 70}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Resistor resistor12(R = R2) annotation(
            Placement(transformation(extent = {{60, 50}, {80, 70}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Resistor load1(R = RL) annotation(
            Placement(transformation(origin = {80, 40}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          Modelica.Electrical.Analog.Basic.Ground ground12 annotation(
            Placement(transformation(extent = {{70, 0}, {90, 20}}, rotation = 0)));
          Modelica.Electrical.Analog.Sources.SineVoltage sineVoltage2(V = Vpeak, phase = phi0, freqHz = f, offset = Vdc) annotation(
            Placement(transformation(origin = {-80, -50}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          Modelica.Electrical.Analog.Basic.Ground ground21 annotation(
            Placement(transformation(extent = {{-90, -90}, {-70, -70}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Resistor resistor21(R = R1) annotation(
            Placement(transformation(extent = {{-80, -40}, {-60, -20}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Inductor inductor21(L = L1sigma) annotation(
            Placement(transformation(extent = {{-50, -40}, {-30, -20}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Inductor inductor22(L = L2sigma, i(start = 0, fixed = true)) annotation(
            Placement(transformation(extent = {{30, -40}, {50, -20}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Resistor resistor22(R = R2) annotation(
            Placement(transformation(extent = {{60, -40}, {80, -20}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Resistor load2(R = RL) annotation(
            Placement(transformation(origin = {80, -50}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          Modelica.Electrical.Analog.Basic.Ground ground22 annotation(
            Placement(transformation(extent = {{70, -90}, {90, -70}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Transformer basicTransformer(L1 = L1, L2 = L2, M = M) annotation(
            Placement(transformation(extent = {{-10, 40}, {10, 60}}, rotation = 0)));
          Modelica.Electrical.Analog.Ideal.IdealTransformer idealTransformer(n = n, Lm1 = Lm1, considerMagnetization = false) annotation(
            Placement(transformation(extent = {{-10, -50}, {10, -30}}, rotation = 0)));
        initial equation
          basicTransformer.i1 = 0;
          basicTransformer.i2 = 0;
        equation
          connect(sineVoltage1.n, ground11.p) annotation(
            Line(points = {{-80, 30}, {-80, 20}}, color = {0, 0, 255}));
          connect(sineVoltage1.p, resistor11.p) annotation(
            Line(points = {{-80, 50}, {-80, 60}}, color = {0, 0, 255}));
          connect(load1.n, ground12.p) annotation(
            Line(points = {{80, 30}, {80, 20}}, color = {0, 0, 255}));
          connect(resistor12.n, load1.p) annotation(
            Line(points = {{80, 60}, {80, 50}}, color = {0, 0, 255}));
          connect(sineVoltage2.n, ground21.p) annotation(
            Line(points = {{-80, -60}, {-80, -70}}, color = {0, 0, 255}));
          connect(sineVoltage2.p, resistor21.p) annotation(
            Line(points = {{-80, -40}, {-80, -30}}, color = {0, 0, 255}));
          connect(resistor21.n, inductor21.p) annotation(
            Line(points = {{-60, -30}, {-50, -30}}, color = {0, 0, 255}));
          connect(inductor22.n, resistor22.p) annotation(
            Line(points = {{50, -30}, {60, -30}}, color = {0, 0, 255}));
          connect(load2.n, ground22.p) annotation(
            Line(points = {{80, -60}, {80, -70}}, color = {0, 0, 255}));
          connect(resistor22.n, load2.p) annotation(
            Line(points = {{80, -30}, {80, -40}}, color = {0, 0, 255}));
          connect(ground11.p, basicTransformer.n1) annotation(
            Line(points = {{-80, 20}, {-10, 20}, {-10, 45}}, color = {0, 0, 255}));
          connect(basicTransformer.n2, ground12.p) annotation(
            Line(points = {{10, 45}, {10, 20}, {80, 20}}, color = {0, 0, 255}));
          connect(basicTransformer.p1, resistor11.n) annotation(
            Line(points = {{-10, 55}, {-10, 60}, {-60, 60}}, color = {0, 0, 255}));
          connect(basicTransformer.p2, resistor12.p) annotation(
            Line(points = {{10, 55}, {10, 60}, {60, 60}}, color = {0, 0, 255}));
          connect(ground21.p, idealTransformer.n1) annotation(
            Line(points = {{-80, -70}, {-10, -70}, {-10, -45}}, color = {0, 0, 255}));
          connect(ground22.p, idealTransformer.n2) annotation(
            Line(points = {{80, -70}, {10, -70}, {10, -45}}, color = {0, 0, 255}));
          connect(idealTransformer.p1, inductor21.n) annotation(
            Line(points = {{-10, -35}, {-10, -30}, {-30, -30}}, color = {0, 0, 255}));
          connect(idealTransformer.p2, inductor22.p) annotation(
            Line(points = {{10, -35}, {10, -30}, {30, -30}}, color = {0, 0, 255}));
          annotation(
            Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-60, -80}, {60, -100}}, lineColor = {0, 0, 255}, textString = "try considerMagnetization=false/true"), Text(extent = {{-60, 20}, {60, 0}}, lineColor = {0, 0, 255}, textString = "Basic.Transformer (mutual inductance)")}),
            experiment(StopTime = 50, Interval = 0.001),
            Documentation(revisions = "<html>
<dl>
<dt><i>2009</i></dt>
<dd>by Anton Haumer<br> initially implemented</dd>
</dl>
</html>", info = "<html>
<p>This example is to demonstrate the behaviour of transformer models. The Basic.Transformer, which consists of mutual coupled inductors, is compared with the ideal transformer model. If the ideal model is used with considerMagnetization=true leakage inductances are taken into account, otherwise not.
The example is constructed in such a way that the ideal transformer circuit with considerMagnetization=true shows the same behaviour as the basic transformer.</p>
<p>Simulate until T=50 s with both considerMagnetization=false and considerMagnetization=true of the ideal transformer.
Plot in separate windows for comparison:
<br>basicTransformer.p1.v and idealTransformer.p1.v
<br>basicTransformer.p1.i and idealTransformer.p1.i
<br>basicTransformer.p2.v and idealTransformer.p2.v
basicTransformer.p2.i and idealTransformer.p2.i </p>
</html>"));
        end CompareTransformers;

        model ControlledSwitchWithArc "Comparison of controlled switch models both with and without arc"
          extends Modelica.Icons.Example;
          Modelica.Electrical.Analog.Basic.Ground ground1 annotation(
            Placement(transformation(extent = {{20, -20}, {40, 0}}, rotation = 0)));
          Modelica.Electrical.Analog.Sources.ConstantVoltage constantVoltage1(V = 50) annotation(
            Placement(transformation(origin = {-20, 20}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          Modelica.Electrical.Analog.Basic.Inductor inductor1(L = 0.1) annotation(
            Placement(transformation(extent = {{40, 30}, {60, 50}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Resistor resistor1(R = 1) annotation(
            Placement(transformation(origin = {80, 20}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          Modelica.Electrical.Analog.Ideal.ControlledIdealClosingSwitch switch1 annotation(
            Placement(transformation(extent = {{0, 30}, {20, 50}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Ground ground2 annotation(
            Placement(transformation(extent = {{20, -100}, {40, -80}}, rotation = 0)));
          Modelica.Electrical.Analog.Sources.ConstantVoltage constantVoltage2(V = 50) annotation(
            Placement(transformation(origin = {-20, -60}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          Modelica.Electrical.Analog.Basic.Inductor inductor2(L = 0.1) annotation(
            Placement(transformation(extent = {{40, -50}, {60, -30}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Resistor resistor2(R = 1) annotation(
            Placement(transformation(origin = {80, -60}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          Modelica.Electrical.Analog.Ideal.ControlledCloserWithArc switch2(V0 = 30, dVdt = 10000, Vmax = 60) annotation(
            Placement(transformation(extent = {{0, -50}, {20, -30}}, rotation = 0)));
          Modelica.Electrical.Analog.Sources.SineVoltage sineVoltage(V = 1, freqHz = 1) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 270, origin = {-70, -10})));
          Modelica.Electrical.Analog.Basic.Ground ground annotation(
            Placement(transformation(extent = {{-80, -60}, {-60, -40}})));
        initial equation
          inductor1.i = 0;
          inductor2.i = 0;
        equation
          connect(inductor1.n, resistor1.p) annotation(
            Line(points = {{60, 40}, {80, 40}, {80, 30}}, color = {0, 0, 255}));
          connect(resistor1.n, ground1.p) annotation(
            Line(points = {{80, 10}, {80, 0}, {30, 0}}, color = {0, 0, 255}));
          connect(constantVoltage1.n, ground1.p) annotation(
            Line(points = {{-20, 10}, {-20, 0}, {30, 0}}, color = {0, 0, 255}));
          connect(switch1.n, inductor1.p) annotation(
            Line(points = {{20, 40}, {40, 40}}, color = {0, 0, 255}));
          connect(constantVoltage1.p, switch1.p) annotation(
            Line(points = {{-20, 30}, {-20, 40}, {0, 40}}, color = {0, 0, 255}));
          connect(inductor2.n, resistor2.p) annotation(
            Line(points = {{60, -40}, {80, -40}, {80, -50}}, color = {0, 0, 255}));
          connect(resistor2.n, ground2.p) annotation(
            Line(points = {{80, -70}, {80, -80}, {30, -80}}, color = {0, 0, 255}));
          connect(constantVoltage2.n, ground2.p) annotation(
            Line(points = {{-20, -70}, {-20, -80}, {30, -80}}, color = {0, 0, 255}));
          connect(switch2.n, inductor2.p) annotation(
            Line(points = {{20, -40}, {40, -40}}, color = {0, 0, 255}));
          connect(constantVoltage2.p, switch2.p) annotation(
            Line(points = {{-20, -50}, {-20, -40}, {0, -40}}, color = {0, 0, 255}));
          connect(sineVoltage.p, switch1.control) annotation(
            Line(points = {{-70, 0}, {-40, 0}, {-40, 50}, {10, 50}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(sineVoltage.p, switch2.control) annotation(
            Line(points = {{-70, 0}, {-40, 0}, {-40, -30}, {10, -30}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(sineVoltage.n, ground.p) annotation(
            Line(points = {{-70, -20}, {-70, -40}}, color = {0, 0, 255}, smooth = Smooth.None));
          annotation(
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-100, 80}, {100, 60}}, lineColor = {0, 0, 255}, textString = "Compare voltage and current of inductor1 and inductor2")}),
            experiment(StopTime = 6, Interval = 0.00025, Tolerance = 1e-006),
            Documentation(info = "<html>
<p>This example is to compare the behaviour of switch models with and without an electric arc taking into consideration.</p>
<ul>
<li>Simulate until T=2 s. </li>
<li>Plot in one window: <code>switch1.i</code> and <code>switch2.i</code> </li>
</ul>
<p>The difference in the closing area shows that the simple arc model avoids the suddenly switching.</p>
</html>", revisions = "<html>
<ul>
<li><i>May, 2009   </i>
       by Anton Haumer<br> initially realized<br>
       </li>
</ul>
</html>"));
        end ControlledSwitchWithArc;

        model SimpleTriacCircuit "Simple triac test circuit"
          extends Modelica.Icons.Example;
          Modelica.Electrical.Analog.Basic.Ground ground annotation(
            Placement(transformation(extent = {{-10, -60}, {10, -40}})));
          Modelica.Electrical.Analog.Basic.Inductor L(L = 2e-6, i(start = 0, fixed = true), p(v(start = 0))) annotation(
            Placement(transformation(extent = {{20, 50}, {40, 70}})));
          Modelica.Electrical.Analog.Basic.Resistor R(R = 10) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 270, origin = {60, 30})));
          Modelica.Electrical.Analog.Sources.SineVoltage V(V = 30, freqHz = 10000) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 270, origin = {60, -10})));
          Modelica.Electrical.Analog.Semiconductors.SimpleTriac simpleTriac(VDRM = 400, VRRM = 400, thyristor1(vGK(start = 0))) annotation(
            Placement(transformation(extent = {{-13, -13}, {11, 11}}, rotation = 270, origin = {1, 23})));
          Modelica.Electrical.Analog.Sources.PulseCurrent pulseCurrent(I = 0.005, width = 10, startTime = 0.00002, period = 0.00005) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 90, origin = {-40, -10})));
        initial equation
          simpleTriac.thyristor.vControl = 0;
          simpleTriac.thyristor1.vControl = 0;
        equation
          connect(L.n, R.p) annotation(
            Line(points = {{40, 60}, {60, 60}, {60, 40}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(R.n, V.p) annotation(
            Line(points = {{60, 20}, {60, 0}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(V.n, ground.p) annotation(
            Line(points = {{60, -20}, {60, -40}, {0, -40}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(simpleTriac.p, ground.p) annotation(
            Line(points = {{0, 11.52}, {0, -40}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(simpleTriac.n, L.p) annotation(
            Line(points = {{0, 36}, {0, 60}, {20, 60}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(pulseCurrent.n, simpleTriac.g) annotation(
            Line(points = {{-40, 0}, {-40, 40}, {-11.52, 40}, {-11.52, 31.44}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(pulseCurrent.p, ground.p) annotation(
            Line(points = {{-40, -20}, {-40, -40}, {0, -40}}, color = {0, 0, 255}, smooth = Smooth.None));
          annotation(
            experiment(StopTime = 0.001, Interval = 5e-7, Tolerance = 1e-12),
            Documentation(revisions = "<html>
<ul>
<li><i>November 25, 2009   </i> <br>

       by Susann Wolf <br><br>
       </li>
</ul>
</html>", info = "<html>
<p>The simple TRIAC example shows how the SimpleTriac is used within an alternating current circuit.</p>
<p>The TRIAC is not conducting until the Gate input g becomes positive. Then it becomes &quot;conducting&quot;. If the TRIAC voltage changes its direction, the TRIAC becomes blocking. Due to the antiparallel connection of the internal two thyristors the same behavior is repeated in the negative half-wave.</p>
<p>Simulate until 0.001 seconds. Display V.p.v (input voltage), simpleTriac.g.i  (gate input), and both simplelTriac.n.v and simpleTriac.n.i, which demonstrate the TRIAC behavior.</p>
</html>"));
        end SimpleTriacCircuit;

        model IdealTriacCircuit "Ideal triac test circuit"
          extends Modelica.Icons.Example;
          Modelica.Electrical.Analog.Basic.Ground ground annotation(
            Placement(transformation(extent = {{-12, -40}, {8, -20}})));
          Modelica.Electrical.Analog.Basic.Resistor R(R = 20) annotation(
            Placement(transformation(extent = {{-10, -10}, {12, 12}}, rotation = 0, origin = {-44, 32})));
          Modelica.Electrical.Analog.Sources.SineVoltage V(V = 5, freqHz = 2) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0, origin = {-42, -20})));
          Modelica.Blocks.Sources.BooleanPulse booleanPulse(width = 50, period = 0.25, startTime = 0.1) annotation(
            Placement(transformation(extent = {{-22, 0}, {-6, 16}})));
          Modelica.Electrical.Analog.Ideal.IdealTriac idealTriac(capacitor(v(start = 0, fixed = true)), idealThyristor(off(start = true, fixed = true)), idealThyristor1(off(start = true, fixed = true))) annotation(
            Placement(transformation(extent = {{6, 20}, {32, 46}})));
        equation
          connect(V.n, ground.p) annotation(
            Line(points = {{-32, -20}, {-2, -20}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(R.p, V.p) annotation(
            Line(points = {{-54, 33}, {-60, 33}, {-60, -20}, {-52, -20}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(R.n, idealTriac.n) annotation(
            Line(points = {{-32, 33}, {6, 33}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(idealTriac.p, ground.p) annotation(
            Line(points = {{32.52, 33}, {42, 33}, {42, -20}, {-2, -20}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(idealTriac.fire1, booleanPulse.y) annotation(
            Line(points = {{11.2, 21.04}, {0, 21.04}, {0, 8}, {-5.2, 8}}, color = {255, 0, 255}, smooth = Smooth.None));
          annotation(
            experiment(StopTime = 2),
            Documentation(revisions = "<html>
<ul>
<li><i>November 25, 2009   </i> <br>

       by Susann Wolf <br><br>
       </li>
</ul>
</html>", info = "<html>
<p>The simple ideal TRIAC example shows how a triac is used within an alternating current circuit.</p>
<p>The TRIAC is not conducting until the Boolean input becomes true (internally coded by 1, therefore the input is called fire<b>1</b>). Then it becomes &quot;conducting&quot;, the knee voltage is reached. If the TRIAC voltage falls below the knee voltage, the TRIAC becomes blocking. Due to the antiparallel connection of the internal two thyristors the same behavior is repeated in the negative half-wave.</p>
<p>Simulate until 2 seconds. Display V.p.v (input voltage), booleanPulse.y (fire1 input), and both idealTriac.n.v and idealTriac.n.i, which demonstrate the TRIAC behavior.</p>
</html>"));
        end IdealTriacCircuit;

        model AD_DA_conversion "Conversion circuit"
          extends Modelica.Icons.Example;
          parameter Integer N = 7 "Digital signal width";
          Modelica.Electrical.Analog.Ideal.AD_Converter aD_Converter(N = N, Rin = 1000000, VRefLow = 0, VRefHigh = 10) annotation(
            Placement(transformation(extent = {{-14, -10}, {12, 16}})));
          Modelica.Electrical.Digital.Sources.Pulse pulse(pulse = Modelica.Electrical.Digital.Interfaces.Logic.'1', quiet = Modelica.Electrical.Digital.Interfaces.Logic.'0', nperiod = -1, width = 1, period = 0.001, startTime = 0) annotation(
            Placement(transformation(extent = {{-32, 32}, {-12, 52}})));
          Modelica.Electrical.Analog.Ideal.DA_Converter dA_Converter(N = N, Vref = 10) annotation(
            Placement(transformation(extent = {{18, -10}, {44, 16}})));
          Modelica.Electrical.Analog.Basic.Ground ground annotation(
            Placement(transformation(extent = {{44, -40}, {64, -20}})));
          Modelica.Electrical.Analog.Sources.SineVoltage sineVoltage(V = 5, freqHz = 10, offset = 5) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 270, origin = {-80, 4})));
          Modelica.Electrical.Analog.Basic.Resistor resistor(R = 1) annotation(
            Placement(transformation(extent = {{-28, 68}, {-8, 88}})));
        equation
          connect(pulse.y, aD_Converter.trig) annotation(
            Line(points = {{-12, 42}, {-1, 42}, {-1, 14.7}}, color = {127, 0, 127}, smooth = Smooth.None));
          connect(sineVoltage.p, aD_Converter.p) annotation(
            Line(points = {{-80, 14}, {-38.5, 14}, {-38.5, 12.1}, {-10.1, 12.1}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(sineVoltage.n, aD_Converter.n) annotation(
            Line(points = {{-80, -6}, {-38, -6}, {-38, -6.1}, {-10.1, -6.1}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(ground.p, sineVoltage.n) annotation(
            Line(points = {{54, -20}, {-80, -20}, {-80, -6}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(sineVoltage.p, resistor.p) annotation(
            Line(points = {{-80, 14}, {-80, 78}, {-28, 78}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(dA_Converter.trig, pulse.y) annotation(
            Line(points = {{31, 14.7}, {31, 42}, {-12, 42}}, color = {127, 0, 127}, smooth = Smooth.None));
          connect(aD_Converter.y, dA_Converter.x) annotation(
            Line(points = {{8.1, 3}, {21.9, 3}}, color = {127, 0, 127}, smooth = Smooth.None));
          connect(dA_Converter.p, resistor.n) annotation(
            Line(points = {{40.1, 12.1}, {40.1, 78}, {-8, 78}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(dA_Converter.n, ground.p) annotation(
            Line(points = {{40.1, -6.1}, {54, -6.1}, {54, -20}}, color = {0, 0, 255}, smooth = Smooth.None));
          annotation(
            experiment(StopTime = 0.2),
            Documentation(info = "<html>
<p>The simple converter circuit converts an analog sine signal into a N-bit (by default a 4 bit) logic signal, which is converted backward into an analog signal.</p>
<ul>
<li>Simulate for 0.2 s.</li>
</ul>
<p>Compare the input voltage (aD_Converter.p.v) with the output voltage (dA_Converter.p.v). By changing N the influence of the digital signal width can be studied. Otherwise the trigger frequency pulse.period can be changed to see its influence.</p>
</html>", revisions = "<html>
<ul>
<li><i>October 13, 2009  </i>by Matthias Franke </li>
</ul>
</html>"));
        end AD_DA_conversion;

        package Utilities "Utility components used by package Examples"
          extends Modelica.Icons.UtilitiesPackage;

          model Nand "CMOS NAND Gate (see Tietze/Schenk, page 157)"
            Semiconductors.PMOS TP1(W = 6.5e-6, L = 3.1e-6, Beta = 1.05e-5, Vt = -1, K2 = 0.41, K5 = 0.8385, dW = -2.5e-6, dL = -2.1e-6) annotation(
              Placement(transformation(extent = {{10, 60}, {30, 80}}, rotation = 0)));
            Semiconductors.PMOS TP2(W = 6.5e-6, L = 3.1e-6, Beta = 1.05e-5, Vt = -1, K2 = 0.41, K5 = 0.8385, dW = -2.5e-6, dL = -2.1e-6) annotation(
              Placement(transformation(extent = {{-40, 45}, {-20, 65}}, rotation = 0)));
            Semiconductors.NMOS TN1(W = 6.5e-6, L = 3.1e-6, Beta = 4.1e-5, Vt = 0.8, K2 = 1.144, K5 = 0.7311, dW = -2.5e-6, dL = -1.5e-6) annotation(
              Placement(transformation(extent = {{10, 15}, {30, 35}}, rotation = 0)));
            Semiconductors.NMOS TN2(W = 6.5e-6, L = 3.1e-6, Beta = 4.1e-5, Vt = 0.8, K2 = 1.144, K5 = 0.7311, dW = -2.5e-6, dL = -1.5e-6) annotation(
              Placement(transformation(extent = {{10, -25}, {30, -5}}, rotation = 0)));
            Basic.Capacitor C4(C = 0.4e-13) annotation(
              Placement(transformation(origin = {60, 30}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
            Basic.Capacitor C7(C = 0.4e-13) annotation(
              Placement(transformation(origin = {55, -50}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
            Basic.Ground Gnd annotation(
              Placement(transformation(extent = {{40, -25}, {50, -15}}, rotation = 0)));
            Basic.Ground Gnd2 annotation(
              Placement(transformation(extent = {{55, 0}, {65, 10}}, rotation = 0)));
            Basic.Ground Gnd3 annotation(
              Placement(transformation(extent = {{50, -80}, {60, -70}}, rotation = 0)));
            Basic.Ground Gnd6 annotation(
              Placement(transformation(extent = {{-15, 30}, {-5, 40}}, rotation = 0)));
            Basic.Ground Gnd7 annotation(
              Placement(transformation(extent = {{40, 15}, {50, 25}}, rotation = 0)));
            Basic.Ground Gnd8 annotation(
              Placement(transformation(extent = {{40, 60}, {50, 70}}, rotation = 0)));
            Interfaces.Pin x1 annotation(
              Placement(transformation(extent = {{-110, 40}, {-90, 60}}, rotation = 0)));
            Interfaces.Pin x2 annotation(
              Placement(transformation(extent = {{-110, -60}, {-90, -40}}, rotation = 0)));
            Interfaces.Pin Vdd annotation(
              Placement(transformation(extent = {{-10, 90}, {10, 110}}, rotation = 0)));
            Interfaces.Pin y annotation(
              Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
          equation
            connect(TN1.S, TN2.D) annotation(
              Line(points = {{30, 20}, {30, -10}}));
            connect(TN2.B, Gnd.p) annotation(
              Line(points = {{30, -15}, {38, -15}, {38, -15}, {45, -15}}));
            connect(TN2.D, C7.p) annotation(
              Line(points = {{30, -10}, {55, -10}, {55, -40}}));
            connect(TN2.S, Gnd.p) annotation(
              Line(points = {{30, -20}, {38, -20}, {38, -15}, {45, -15}}));
            connect(TP1.S, TN1.D) annotation(
              Line(points = {{30, 65}, {30, 30}}));
            connect(TP1.G, TN1.G) annotation(
              Line(points = {{10, 65}, {10, 20}}));
            connect(C4.n, Gnd2.p) annotation(
              Line(points = {{60, 20}, {60, 10}}));
            connect(C7.n, Gnd3.p) annotation(
              Line(points = {{55, -60}, {55, -70}}));
            connect(TP2.B, Gnd6.p) annotation(
              Line(points = {{-20, 55}, {-10, 55}, {-10, 40}}));
            connect(TN1.B, Gnd7.p) annotation(
              Line(points = {{30, 25}, {45, 25}}));
            connect(TP1.B, Gnd8.p) annotation(
              Line(points = {{30, 70}, {45, 70}}));
            connect(TP2.G, x1) annotation(
              Line(points = {{-40, 50}, {-100, 50}}));
            connect(TP2.G, TN2.G) annotation(
              Line(points = {{-40, 50}, {-40, -20}, {10, -20}}));
            connect(TN1.G, x2) annotation(
              Line(points = {{10, 20}, {-70, 20}, {-70, -50}, {-100, -50}}));
            connect(TP2.S, TN1.D) annotation(
              Line(points = {{-20, 50}, {30, 50}, {30, 30}}));
            connect(TN1.D, C4.p) annotation(
              Line(points = {{30, 30}, {30, 50}, {60, 50}, {60, 40}}));
            connect(Vdd, TP1.D) annotation(
              Line(points = {{0, 100}, {0, 80}, {30, 80}, {30, 75}}));
            connect(TP2.D, Vdd) annotation(
              Line(points = {{-20, 60}, {-20, 80}, {0, 80}, {0, 100}}));
            connect(C4.p, y) annotation(
              Line(points = {{60, 40}, {60, 50}, {80, 50}, {80, 0}, {100, 0}}));
            annotation(
              Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-50, 70}, {50, -70}}, lineColor = {0, 0, 0}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Text(extent = {{-16, 50}, {14, 20}}, lineColor = {0, 0, 0}, textString = "&"), Line(points = {{60, 0}, {100, 0}}, color = {0, 0, 255}), Line(points = {{0, 70}, {0, 100}}, color = {0, 0, 255}), Line(points = {{-50, 50}, {-100, 50}}, color = {0, 0, 255}), Line(points = {{-50, -50}, {-100, -50}}, color = {0, 0, 255}), Ellipse(extent = {{50, 5}, {60, -5}}, lineColor = {0, 0, 0}), Text(extent = {{-100, 34}, {-90, 24}}, textString = "x1", lineColor = {0, 0, 255}), Text(extent = {{-100, -65}, {-90, -75}}, textString = "x2", lineColor = {0, 0, 255}), Text(extent = {{90, -15}, {100, -25}}, textString = "y", lineColor = {0, 0, 255}), Text(extent = {{-151, -81}, {149, -111}}, lineColor = {0, 0, 255}, textString = "%name")}),
              Documentation(info = "<html>
<p>The nand gate is a basic CMOS building block. It consists of four CMOS transistors.</p>
<p><b>Reference:</b></p>
<p>Tietze, U.; Schenk, Ch.: Halbleiter-Schaltungstechnik. Springer-Verlag Berlin Heidelberg NewYork 1980, p. 157</p>
</html>", revisions = "<html>
<dl>
<dt>
<b>Main Authors:</b>
</dt>
<dd>
Christoph Clau&szlig;
    &lt;<a href=\"mailto:Christoph.Clauss@eas.iis.fraunhofer.de\">Christoph.Clauss@eas.iis.fraunhofer.de</a>&gt;<br>
    Andr&eacute; Schneider
    &lt;<a href=\"mailto:Andre.Schneider@eas.iis.fraunhofer.de\">Andre.Schneider@eas.iis.fraunhofer.de</a>&gt;<br>
    Fraunhofer Institute for Integrated Circuits<br>
    Design Automation Department<br>
    Zeunerstra&szlig;e 38<br>
    D-01069 Dresden
</dd>
</dl>
</html>"),
              Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-94, 93}, {-44, 70}}, lineColor = {0, 0, 255}, textString = "NAND")}));
          end Nand;

          model NonlinearResistor "Chua's resistor"
            extends Interfaces.OnePort;
            parameter SI.Conductance Ga "Conductance in inner voltage range";
            parameter SI.Conductance Gb "Conductance in outer voltage range";
            parameter SI.Voltage Ve "Inner voltage range limit";
          equation
            i = if v < (-Ve) then Gb * (v + Ve) - Ga * Ve else if v > Ve then Gb * (v - Ve) + Ga * Ve else Ga * v;
            annotation(
              Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-70, 30}, {70, -30}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{-90, 0}, {-70, 0}}, color = {0, 0, 255}), Line(points = {{70, 0}, {90, 0}}, color = {0, 0, 255}), Line(points = {{-50, -60}, {50, 60}}, color = {0, 0, 255}), Polygon(points = {{50, 60}, {38, 52}, {44, 46}, {50, 60}}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Text(extent = {{-150, 97}, {150, 67}}, lineColor = {0, 0, 255}, textString = "%name")}),
              Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-70, 30}, {70, -30}}, lineColor = {0, 0, 255}), Line(points = {{-90, 0}, {-70, 0}}, color = {0, 0, 255}), Line(points = {{70, 0}, {90, 0}}, color = {0, 0, 255}), Line(points = {{-50, -60}, {50, 60}}, color = {0, 0, 255}), Polygon(points = {{50, 60}, {38, 52}, {44, 46}, {50, 60}}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Text(extent = {{-100, 100}, {100, 70}}, textString = "%name", lineColor = {0, 0, 255})}),
              Documentation(info = "<html>
<p>This is the only nonlinear component for Chua&#39;;s circuit. It is a piecewise linear resistor with both an inner and an outer range, which includes the inner one. The slopes of both ranges are given by parameters. The resistance characteristic is continuous. For Chua&#39;;s circuit both slopes have to be chosen to be negative.</p>
</html>"));
          end NonlinearResistor;

          model RealSwitch "Ideal switch with resistance"
            Ideal.ControlledIdealCommutingSwitch S(level = 2.5) annotation(
              Placement(transformation(extent = {{11.3333, -46}, {58, 0.6667}}, rotation = 0)));
            Basic.Resistor R(R = 0.01) annotation(
              Placement(transformation(extent = {{-66, -48}, {-19.3333, -1.3333}}, rotation = 0)));
            Interfaces.Pin p annotation(
              Placement(transformation(extent = {{-110, -34}, {-90, -14}}, rotation = 0)));
            Interfaces.Pin n1 annotation(
              Placement(transformation(extent = {{90, -14}, {110, 6}}, rotation = 0)));
            Interfaces.Pin n2 annotation(
              Placement(transformation(extent = {{90, -60}, {110, -40}}, rotation = 0)));
            Interfaces.Pin control annotation(
              Placement(transformation(extent = {{24, 26}, {44, 46}}, rotation = 0)));
          equation
            connect(p, R.p) annotation(
              Line(points = {{-100, -24}, {-66, -24.6667}}));
            connect(R.n, S.p) annotation(
              Line(points = {{-19.3333, -24.6667}, {11.3333, -22.6667}}));
            connect(n1, S.n1) annotation(
              Line(points = {{100, -4}, {58, -11}}));
            connect(n2, S.n2) annotation(
              Line(points = {{100, -50}, {58, -22.6667}}));
            connect(control, S.control) annotation(
              Line(points = {{34, 36}, {34.6666, 0.6667}}));
            annotation(
              Documentation(info = "<html>
<p>This component is a special kind of a commuting switch which possesses an additional constant resistance. This resistance was necessary in an example. It is not designed for common use.</p>
</html>"),
              Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-90, 86}, {28, 70}}, lineColor = {0, 0, 255}, textString = "Real Switch")}));
          end RealSwitch;

          model Transistor "Transistor with resistance an capacitance"
            Basic.Resistor rtb(R = 0.05) annotation(
              Placement(transformation(extent = {{-80, -10}, {-60, 10}}, rotation = 0)));
            Basic.Resistor rtc(R = 0.1) annotation(
              Placement(transformation(extent = {{40, 0}, {60, 20}}, rotation = 0)));
            Basic.Capacitor ct(C = 1e-10) annotation(
              Placement(transformation(origin = {-40, -30}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
            Semiconductors.NPN Tr(Bf = 50, Br = 0.1, Is = 1e-16, Vak = 0.02, Tauf = 0.12e-9, Taur = 5e-9, Ccs = 1e-12, Cje = 0.4e-12, Cjc = 0.5e-12, Phie = 0.8, Me = 0.4, Phic = 0.8, Mc = 0.333, Gbc = 1e-15, Gbe = 1e-15, Vt = 0.02585, UIC = true) annotation(
              Placement(transformation(extent = {{-20, -20}, {20, 20}}, rotation = 0)));
            Basic.Ground Ground1 annotation(
              Placement(transformation(extent = {{-50, -80}, {-30, -60}}, rotation = 0)));
            Interfaces.Pin c annotation(
              Placement(transformation(extent = {{90, 50}, {110, 70}}, rotation = 0)));
            Interfaces.Pin b annotation(
              Placement(transformation(extent = {{-110, -10}, {-90, 10}}, rotation = 0)));
            Interfaces.Pin e annotation(
              Placement(transformation(extent = {{90, -70}, {110, -50}}, rotation = 0)));
          equation
            connect(rtb.n, Tr.B) annotation(
              Line(points = {{-60, 0}, {-20, 0}}));
            connect(rtb.n, ct.p) annotation(
              Line(points = {{-60, 0}, {-40, 0}, {-40, -20}}));
            connect(ct.n, Ground1.p) annotation(
              Line(points = {{-40, -40}, {-40, -60}}));
            connect(Tr.C, rtc.p) annotation(
              Line(points = {{20, 10}, {40, 10}}));
            connect(rtc.n, c) annotation(
              Line(points = {{60, 10}, {80, 10}, {80, 60}, {100, 60}}));
            connect(b, rtb.p) annotation(
              Line(points = {{-100, 0}, {-80, 0}}));
            connect(Tr.E, e) annotation(
              Line(points = {{20, -10}, {80, -10}, {80, -60}, {100, -60}}));
            annotation(
              Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-80, 80}, {80, -80}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{80, 60}, {100, 60}}, color = {0, 0, 255}), Line(points = {{80, -60}, {100, -60}}, color = {0, 0, 255}), Line(points = {{-100, 0}, {-80, 0}}, color = {0, 0, 255}), Line(points = {{-60, 0}, {-10, 0}}, color = {0, 0, 255}), Line(points = {{-10, 40}, {-10, -40}}, color = {0, 0, 255}), Line(points = {{60, 60}, {40, 60}, {-10, 10}}, color = {0, 0, 255}), Line(points = {{-10, -10}, {40, -60}, {60, -60}}, color = {0, 0, 255}), Text(extent = {{-150, 120}, {150, 90}}, lineColor = {0, 0, 255}, textString = "%name")}),
              Documentation(info = "<html>
<p>Since the simple bipolar transistor model does not have base or collector resistances both are added in this component. Additionally, a capacity is added to the base pin. See the schematic for more details.  In such a way the transistor model can be enhanced to become more common.</p>
</html>"),
              Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-76, 82}, {-2, 54}}, lineColor = {0, 0, 255}, textString = "Transistor")}));
          end Transistor;
          annotation(
            Documentation(info = "<html>
<p>This package contains some utility components used by package examples. These components are auxiliary components that should not be used like true MLS components since they are designed the purpose of the examples only, not for common use.</p>
</html>", revisions = "<html>
<dl>
<dt>
<b>Main Authors:</b>
</dt>
<dd>
Christoph Clau&szlig;
    &lt;<a href=\"mailto:Christoph.Clauss@eas.iis.fraunhofer.de\">Christoph.Clauss@eas.iis.fraunhofer.de</a>&gt;<br>
    Andr&eacute; Schneider
    &lt;<a href=\"mailto:Andre.Schneider@eas.iis.fraunhofer.de\">Andre.Schneider@eas.iis.fraunhofer.de</a>&gt;<br>
    Fraunhofer Institute for Integrated Circuits<br>
    Design Automation Department<br>
    Zeunerstra&szlig;e 38<br>
    D-01069 Dresden
</dd>
</dl>
</html>"));
        end Utilities;
        annotation(
          Documentation(info = "<html>
<p>This package contains examples that demonstrate the usage of the components of the Electrical.Analog library.</p>
<p>The examples are simple to understand. They will show a typical behavior of the components, and they will give hints to users.</p>
</html>", revisions = "<html>
<dl>
<dt>
<b>Main Authors:</b>
</dt>
<dd>
Christoph Clau&szlig;
    &lt;<a href=\"mailto:Christoph.Clauss@eas.iis.fraunhofer.de\">Christoph.Clauss@eas.iis.fraunhofer.de</a>&gt;<br>
    Andr&eacute; Schneider
    &lt;<a href=\"mailto:Andre.Schneider@eas.iis.fraunhofer.de\">Andre.Schneider@eas.iis.fraunhofer.de</a>&gt;<br>
    Fraunhofer Institute for Integrated Circuits<br>
    Design Automation Department<br>
    Zeunerstra&szlig;e 38<br>
    D-01069 Dresden
</dd>
</dl>
</html>"));
      end Examples;

      package Basic "Basic electrical components"
        extends Modelica.Icons.Package;

        model Ground "Ground node"
          Interfaces.Pin p annotation(
            Placement(transformation(origin = {0, 100}, extent = {{10, -10}, {-10, 10}}, rotation = 270)));
        equation
          p.v = 0;
          annotation(
            Documentation(info = "<html>
<p>Ground of an electrical circuit. The potential at the ground node is zero. Every electrical circuit has to contain at least one ground object.</p>
</html>", revisions = "<html>
<ul>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-60, 50}, {60, 50}}, color = {0, 0, 255}), Line(points = {{-40, 30}, {40, 30}}, color = {0, 0, 255}), Line(points = {{-20, 10}, {20, 10}}, color = {0, 0, 255}), Line(points = {{0, 90}, {0, 50}}, color = {0, 0, 255}), Text(extent = {{-144, -19}, {156, -59}}, textString = "%name", lineColor = {0, 0, 255})}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-60, 50}, {60, 50}}, thickness = 0.5, color = {0, 0, 255}), Line(points = {{-40, 30}, {40, 30}}, thickness = 0.5, color = {0, 0, 255}), Line(points = {{-20, 10}, {20, 10}}, thickness = 0.5, color = {0, 0, 255}), Line(points = {{0, 96}, {0, 50}}, thickness = 0.5, color = {0, 0, 255}), Text(extent = {{-24, -38}, {22, -6}}, textString = "p.v=0", lineColor = {0, 0, 255})}));
        end Ground;

        model Resistor "Ideal linear electrical resistor"
          parameter Modelica.SIunits.Resistance R(start = 1) "Resistance at temperature T_ref";
          parameter Modelica.SIunits.Temperature T_ref = 300.15 "Reference temperature";
          parameter Modelica.SIunits.LinearTemperatureCoefficient alpha = 0 "Temperature coefficient of resistance (R_actual = R*(1 + alpha*(T_heatPort - T_ref))";
          extends Modelica.Electrical.Analog.Interfaces.OnePort;
          extends Modelica.Electrical.Analog.Interfaces.ConditionalHeatPort(T = T_ref);
          Modelica.SIunits.Resistance R_actual "Actual resistance = R*(1 + alpha*(T_heatPort - T_ref))";
        equation
          assert(1 + alpha * (T_heatPort - T_ref) >= Modelica.Constants.eps, "Temperature outside scope of model!");
          R_actual = R * (1 + alpha * (T_heatPort - T_ref));
          v = R_actual * i;
          LossPower = v * i;
          annotation(
            Documentation(info = "<html>
<p>The linear resistor connects the branch voltage <i>v</i> with the branch current <i>i</i> by <i>i*R = v</i>. The Resistance <i>R</i> is allowed to be positive, zero, or negative.</p>
</html>", revisions = "<html>
<ul>
<li><i> August 07, 2009   </i>
       by Anton Haumer<br> temperature dependency of resistance added<br>
       </li>
<li><i> March 11, 2009   </i>
       by Christoph Clauss<br> conditional heat port added<br>
       </li>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-70, 30}, {70, -30}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{-90, 0}, {-70, 0}}, color = {0, 0, 255}), Line(points = {{70, 0}, {90, 0}}, color = {0, 0, 255}), Text(extent = {{-144, -40}, {142, -72}}, lineColor = {0, 0, 0}, textString = "R=%R"), Line(visible = useHeatPort, points = {{0, -100}, {0, -30}}, color = {127, 0, 0}, smooth = Smooth.None, pattern = LinePattern.Dot), Text(extent = {{-152, 87}, {148, 47}}, textString = "%name", lineColor = {0, 0, 255})}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-70, 30}, {70, -30}}, lineColor = {0, 0, 255}), Line(points = {{-96, 0}, {-70, 0}}, color = {0, 0, 255}), Line(points = {{70, 0}, {96, 0}}, color = {0, 0, 255})}));
        end Resistor;

        model HeatingResistor "Temperature dependent electrical resistor"
          parameter Modelica.SIunits.Resistance R_ref(start = 1) "Resistance at temperature T_ref";
          parameter Modelica.SIunits.Temperature T_ref = 300.15 "Reference temperature";
          parameter Modelica.SIunits.LinearTemperatureCoefficient alpha = 0 "Temperature coefficient of resistance (R = R_ref*(1 + alpha*(heatPort.T - T_ref))";
          extends Modelica.Electrical.Analog.Interfaces.OnePort;
          extends Modelica.Electrical.Analog.Interfaces.ConditionalHeatPort(T = T_ref, useHeatPort = true);
          Modelica.SIunits.Resistance R "Resistance = R_ref*(1 + alpha*(T_heatPort - T_ref))";
        equation
          assert(1 + alpha * (T_heatPort - T_ref) >= Modelica.Constants.eps, "Temperature outside scope of model!");
          R = R_ref * (1 + alpha * (T_heatPort - T_ref));
          v = R * i;
          LossPower = v * i;
          annotation(
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-110, 20}, {-85, 20}}, color = {160, 160, 164}), Polygon(points = {{-95, 23}, {-85, 20}, {-95, 17}, {-95, 23}}, lineColor = {160, 160, 164}, fillColor = {160, 160, 164}, fillPattern = FillPattern.Solid), Line(points = {{90, 20}, {115, 20}}, color = {160, 160, 164}), Line(points = {{-125, 0}, {-115, 0}}, color = {160, 160, 164}), Line(points = {{-120, -5}, {-120, 5}}, color = {160, 160, 164}), Text(extent = {{-110, 25}, {-90, 45}}, lineColor = {160, 160, 164}, textString = "i"), Polygon(points = {{105, 23}, {115, 20}, {105, 17}, {105, 23}}, lineColor = {160, 160, 164}, fillColor = {160, 160, 164}, fillPattern = FillPattern.Solid), Line(points = {{115, 0}, {125, 0}}, color = {160, 160, 164}), Text(extent = {{90, 45}, {110, 25}}, lineColor = {160, 160, 164}, textString = "i")}),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-90, 0}, {-70, 0}}, color = {0, 0, 255}), Line(points = {{70, 0}, {90, 0}}, color = {0, 0, 255}), Rectangle(extent = {{-70, 30}, {70, -30}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{-52, -50}, {48, 50}}, color = {0, 0, 255}), Polygon(points = {{40, 52}, {50, 42}, {54, 56}, {40, 52}}, lineColor = {0, 0, 255}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid), Line(visible = useHeatPort, points = {{0, -100}, {0, -30}}, color = {127, 0, 0}, smooth = Smooth.None, pattern = LinePattern.Dot), Text(extent = {{-156, 109}, {144, 69}}, textString = "%name", lineColor = {0, 0, 255})}),
            Documentation(info = "<html>
<p>This is a model for an electrical resistor where the generated heat is dissipated to the environment via connector <b>heatPort</b> and where the resistance R is temperature dependent according to the following equation:</p>
<pre>    R = R_ref*(1 + alpha*(heatPort.T - T_ref))</pre>
<p><b>alpha</b> is the <b>temperature coefficient of resistance</b>, which is often abbreviated as <b>TCR</b>. In resistor catalogues, it is usually defined as <b>X [ppm/K]</b> (parts per million, similarly to percentage) meaning <b>X*1.e-6 [1/K]</b>. Resistors are available for 1 .. 7000 ppm/K, i.e., alpha = 1e-6 .. 7e-3 1/K;</p>
<p>Via parameter <b>useHeatPort</b> the heatPort connector can be enabled and disabled (default = enabled). If it is disabled, the generated heat is transported implicitly to an internal temperature source with a fixed temperature of T_ref.</p><p>If the heatPort connector is enabled, it must be connected.</p>
</html>", revisions = "<html>
<ul>
<li><i> August 07, 2009   </i>
       by Anton Haumer<br> temperature dependency of resistance added<br>
       </li>
<li><i> March 11, 2009   </i>
       by Christoph Clauss<br> conditional heat port added<br>
       </li>
<li><i> 2002   </i>
       by Anton Haumer<br> initially implemented<br>
       </li>
</ul>
</html>"));
        end HeatingResistor;

        model Conductor "Ideal linear electrical conductor"
          parameter Modelica.SIunits.Conductance G(start = 1) "Conductance at temperature T_ref";
          parameter Modelica.SIunits.Temperature T_ref = 300.15 "Reference temperature";
          parameter Modelica.SIunits.LinearTemperatureCoefficient alpha = 0 "Temperature coefficient of conductance (G_actual = G_ref/(1 + alpha*(T_heatPort - T_ref))";
          extends Modelica.Electrical.Analog.Interfaces.OnePort;
          extends Modelica.Electrical.Analog.Interfaces.ConditionalHeatPort(T = T_ref);
          Modelica.SIunits.Conductance G_actual "Actual conductance = G_ref/(1 + alpha*(T_heatPort - T_ref))";
        equation
          assert(1 + alpha * (T_heatPort - T_ref) >= Modelica.Constants.eps, "Temperature outside scope of model!");
          G_actual = G / (1 + alpha * (T_heatPort - T_ref));
          i = G_actual * v;
          LossPower = v * i;
          annotation(
            Documentation(info = "<html>
<p>The linear conductor connects the branch voltage <i>v</i> with the branch current <i>i</i> by <i>i = v*G</i>. The Conductance <i>G</i> is allowed to be positive, zero, or negative.</p>
</html>", revisions = "<html>
<ul>
<li><i> August 07, 2009   </i>
       by Anton Haumer<br> temperature dependency of conductance added<br>
       </li>
<li><i> March 11, 2009   </i>
       by Christoph Clauss<br> conditional heat port added<br>
       </li>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-70, 30}, {70, -30}}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Rectangle(extent = {{-70, 30}, {70, -30}}, lineColor = {0, 0, 255}), Line(points = {{-90, 0}, {-70, 0}}, color = {0, 0, 255}), Line(points = {{70, 0}, {90, 0}}, color = {0, 0, 255}), Line(visible = useHeatPort, points = {{0, -100}, {0, -30}}, color = {127, 0, 0}, smooth = Smooth.None, pattern = LinePattern.Dot), Text(extent = {{-152, 87}, {148, 47}}, textString = "%name", lineColor = {0, 0, 255}), Text(extent = {{-144, -38}, {142, -70}}, lineColor = {0, 0, 0}, textString = "G=%G")}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-96, 0}, {-70, 0}}, color = {0, 0, 255}), Line(points = {{70, 0}, {96, 0}}, color = {0, 0, 255}), Rectangle(extent = {{-70, 30}, {70, -30}}, lineColor = {0, 0, 255})}));
        end Conductor;

        model Capacitor "Ideal linear electrical capacitor"
          extends Interfaces.OnePort(v(start = 0));
          parameter SI.Capacitance C(start = 1) "Capacitance";
        equation
          i = C * der(v);
          annotation(
            Documentation(info = "<html>
<p>The linear capacitor connects the branch voltage <i>v</i> with the branch current <i>i</i> by <i>i = C * dv/dt</i>. The Capacitance <i>C</i> is allowed to be positive or zero.</p>

</html>", revisions = "<html>
<ul>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-6, 28}, {-6, -28}}, color = {0, 0, 255}), Line(points = {{6, 28}, {6, -28}}, color = {0, 0, 255}), Line(points = {{-90, 0}, {-6, 0}}, color = {0, 0, 255}), Line(points = {{6, 0}, {90, 0}}, color = {0, 0, 255}), Text(extent = {{-136, -60}, {136, -92}}, lineColor = {0, 0, 0}, textString = "C=%C"), Text(extent = {{-150, 85}, {150, 45}}, textString = "%name", lineColor = {0, 0, 255})}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-20, 40}, {-20, -40}}, thickness = 0.5, color = {0, 0, 255}), Line(points = {{20, 40}, {20, -40}}, thickness = 0.5, color = {0, 0, 255}), Line(points = {{-96, 0}, {-20, 0}}, color = {0, 0, 255}), Line(points = {{20, 0}, {96, 0}}, color = {0, 0, 255})}));
        end Capacitor;

        model Inductor "Ideal linear electrical inductor"
          extends Interfaces.OnePort(i(start = 0));
          parameter SI.Inductance L(start = 1) "Inductance";
        equation
          L * der(i) = v;
          annotation(
            Documentation(info = "<html>
<p>The linear inductor connects the branch voltage <i>v</i> with the branch current <i>i</i> by <i>v = L * di/dt</i>. The Inductance <i>L</i> is allowed to be positive, or zero.</p>

</html>", revisions = "<html>
<ul>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-60, -15}, {-30, 15}}, lineColor = {0, 0, 255}), Ellipse(extent = {{-30, -15}, {0, 15}}, lineColor = {0, 0, 255}), Ellipse(extent = {{0, -15}, {30, 15}}, lineColor = {0, 0, 255}), Ellipse(extent = {{30, -15}, {60, 15}}, lineColor = {0, 0, 255}), Rectangle(extent = {{-60, -30}, {60, 0}}, lineColor = {255, 255, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{60, 0}, {90, 0}}, color = {0, 0, 255}), Line(points = {{-90, 0}, {-60, 0}}, color = {0, 0, 255}), Text(extent = {{-138, -60}, {144, -94}}, lineColor = {0, 0, 0}, textString = "L=%L"), Text(extent = {{-152, 79}, {148, 39}}, textString = "%name", lineColor = {0, 0, 255})}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-60, -15}, {-30, 15}}, lineColor = {0, 0, 255}), Ellipse(extent = {{-30, -15}, {0, 15}}, lineColor = {0, 0, 255}), Ellipse(extent = {{0, -15}, {30, 15}}, lineColor = {0, 0, 255}), Ellipse(extent = {{30, -15}, {60, 15}}, lineColor = {0, 0, 255}), Rectangle(extent = {{-60, -30}, {60, 0}}, lineColor = {255, 255, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{60, 0}, {96, 0}}, color = {0, 0, 255}), Line(points = {{-96, 0}, {-60, 0}}, color = {0, 0, 255})}));
        end Inductor;

        model SaturatingInductor "Simple model of an inductor with saturation"
          extends Modelica.Electrical.Analog.Interfaces.OnePort;
          parameter Modelica.SIunits.Current Inom(start = 1) "Nominal current";
          parameter Modelica.SIunits.Inductance Lnom(start = 1) "Nominal inductance at Nominal current";
          parameter Modelica.SIunits.Inductance Lzer(start = 2 * Lnom) "Inductance near current=0";
          parameter Modelica.SIunits.Inductance Linf(start = Lnom / 2) "Inductance at large currents";
          Modelica.SIunits.Inductance Lact(start = Lzer) "Present inductance";
          Modelica.SIunits.MagneticFlux Psi(start = 0, fixed = true) "Present flux";
        protected
          parameter Modelica.SIunits.Current Ipar(start = Inom / 10, fixed = false);
        initial equation
          Lnom - Linf = (Lzer - Linf) * Ipar / Inom * (Modelica.Constants.pi / 2 - Modelica.Math.atan(Ipar / Inom));
        equation
          assert(Lzer > Lnom + Modelica.Constants.eps, "Lzer (= " + String(Lzer) + ") has to be > Lnom (= " + String(Lnom) + ")");
          assert(Linf < Lnom - Modelica.Constants.eps, "Linf (= " + String(Linf) + ") has to be < Lnom (= " + String(Lnom) + ")");
          (Lact - Linf) * i / Ipar = (Lzer - Linf) * noEvent(Modelica.Math.atan(i / Ipar));
          Psi = Lact * i;
          v = der(Psi);
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-60, -15}, {-30, 15}}, lineColor = {0, 0, 255}), Ellipse(extent = {{-30, -15}, {0, 15}}, lineColor = {0, 0, 255}), Ellipse(extent = {{0, -15}, {30, 15}}, lineColor = {0, 0, 255}), Ellipse(extent = {{30, -15}, {60, 15}}, lineColor = {0, 0, 255}), Rectangle(extent = {{-60, -20}, {62, 0}}, lineColor = {255, 255, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{60, 0}, {90, 0}}, color = {0, 0, 255}), Line(points = {{-90, 0}, {-60, 0}}, color = {0, 0, 255}), Rectangle(extent = {{-60, -10}, {60, -20}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.Sphere, fillColor = {0, 0, 255}), Text(extent = {{-148, -50}, {152, -80}}, lineColor = {0, 0, 0}, textString = "Lnom=%Lnom"), Text(extent = {{-148, 91}, {152, 51}}, textString = "%name", lineColor = {0, 0, 255})}),
            Documentation(info = "<html>
<p>This model approximates the behaviour of an inductor with the influence of saturation, i.e., the value of the inductance depends on the current flowing through the inductor. The inductance decreases as current increases.</p><p>The parameters are:</p>
<ul>
<li>Inom...nominal current</li>
<li>Lnom...nominal inductance at nominal current</li>
<li>Lzer...inductance near current = 0; Lzer has to be greater than Lnom</li>
<li>Linf...inductance at large currents; Linf has to be less than Lnom </li>
</ul>
</html>", revisions = "<html>
<dl>
  <dt><b>Main Author:</b></dt>
  <dd>
  <a href=\"http://www.haumer.at/\">Anton Haumer</a><br>
  Technical Consulting &amp; Electrical Engineering<br>
  A-3423 St.Andrae-Woerdern<br>Austria<br>
  email: <a href=\"mailto:a.haumer@haumer.at\">a.haumer@haumer.at</a>
  </dd>
  <dt><b>Release Notes:</b></dt>
  <dd>May 27, 2004: Implemented by Anton Haumer</dd>
 </dl>
</html>"),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-60, -15}, {-30, 15}}, lineColor = {0, 0, 255}), Ellipse(extent = {{-30, -15}, {0, 15}}, lineColor = {0, 0, 255}), Ellipse(extent = {{0, -15}, {30, 15}}, lineColor = {0, 0, 255}), Ellipse(extent = {{30, -15}, {60, 15}}, lineColor = {0, 0, 255}), Rectangle(extent = {{-60, -20}, {62, 0}}, lineColor = {255, 255, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{60, 0}, {96, 0}}, color = {0, 0, 255}), Line(points = {{-96, 0}, {-60, 0}}, color = {0, 0, 255}), Rectangle(extent = {{-60, -10}, {60, -20}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.Sphere, fillColor = {0, 0, 255})}));
        end SaturatingInductor;

        model Transformer "Transformer with two ports"
          extends Interfaces.TwoPort(i1(start = 0), i2(start = 0));
          parameter SI.Inductance L1(start = 1) "Primary inductance";
          parameter SI.Inductance L2(start = 1) "Secondary inductance";
          parameter SI.Inductance M(start = 1) "Coupling inductance";
          Real dv "Difference between voltage drop over primary inductor and
              voltage drop over secondary inductor";
        equation
          v1 = L1 * der(i1) + M * der(i2);
/* Original equation:
        v2 = M*der(i1) + L2*der(i2);
     If L1 = L2 = M, then this model has one state less. However,
     it might be difficult for a tool to detect this. For this reason
     the model is defined with a relative potential:
  */
          dv = (L1 - M) * der(i1) + (M - L2) * der(i2);
          v2 = v1 - dv;
          annotation(
            Documentation(info = "<html>
<p>The transformer is a two port. The left port voltage <i>v1</i>, left port current <i>i1</i>, right port voltage <i>v2</i> and right port current <i>i2</i> are connected by the following relation:</p>
<pre>         | v1 |         | L1   M  |  | i1&#39;; |
         |    |    =    |         |  |     |
         | v2 |         | M    L2 |  | i2&#39;; |</pre>
<p><i>L1</i>, <i>L2</i>, and <i>M</i> are the primary, secondary, and coupling inductances respectively.</p>
</html>", revisions = "<html>
<ul>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-45, -50}, {-20, -25}}, lineColor = {0, 0, 255}), Ellipse(extent = {{-45, -25}, {-20, 0}}, lineColor = {0, 0, 255}), Ellipse(extent = {{-45, 0}, {-20, 25}}, lineColor = {0, 0, 255}), Ellipse(extent = {{-45, 25}, {-20, 50}}, lineColor = {0, 0, 255}), Rectangle(extent = {{-72, -60}, {-33, 60}}, lineColor = {255, 255, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{-90, 50}, {-32, 50}}, color = {0, 0, 255}), Line(points = {{-90, -50}, {-32, -50}}, color = {0, 0, 255}), Ellipse(extent = {{20, -50}, {45, -25}}, lineColor = {0, 0, 255}), Ellipse(extent = {{20, -25}, {45, 0}}, lineColor = {0, 0, 255}), Ellipse(extent = {{20, 0}, {45, 25}}, lineColor = {0, 0, 255}), Ellipse(extent = {{20, 25}, {45, 50}}, lineColor = {0, 0, 255}), Rectangle(extent = {{33, -60}, {72, 60}}, lineColor = {255, 255, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{32, 50}, {90, 50}}, color = {0, 0, 255}), Line(points = {{32, -50}, {90, -50}}, color = {0, 0, 255}), Text(extent = {{-103, 16}, {-60, -10}}, textString = "L1", lineColor = {0, 0, 0}, pattern = LinePattern.Dot), Text(extent = {{64, 15}, {105, -9}}, textString = "L2", lineColor = {0, 0, 0}, pattern = LinePattern.Dot), Text(extent = {{-27, -71}, {33, -97}}, textString = "M", lineColor = {0, 0, 0}), Text(extent = {{-153, 113}, {147, 73}}, textString = "%name", lineColor = {0, 0, 255})}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-45, -50}, {-20, -25}}, lineColor = {0, 0, 255}), Ellipse(extent = {{-45, -25}, {-20, 0}}, lineColor = {0, 0, 255}), Ellipse(extent = {{-45, 0}, {-20, 25}}, lineColor = {0, 0, 255}), Ellipse(extent = {{-45, 25}, {-20, 50}}, lineColor = {0, 0, 255}), Rectangle(extent = {{-72, -60}, {-33, 60}}, lineColor = {255, 255, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{-96, 50}, {-32, 50}}, color = {0, 0, 255}), Line(points = {{-96, -50}, {-32, -50}}, color = {0, 0, 255}), Ellipse(extent = {{20, -50}, {45, -25}}, lineColor = {0, 0, 255}), Ellipse(extent = {{20, -25}, {45, 0}}, lineColor = {0, 0, 255}), Ellipse(extent = {{20, 0}, {45, 25}}, lineColor = {0, 0, 255}), Ellipse(extent = {{20, 25}, {45, 50}}, lineColor = {0, 0, 255}), Rectangle(extent = {{33, -60}, {72, 60}}, lineColor = {255, 255, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{32, 50}, {96, 50}}, color = {0, 0, 255}), Line(points = {{32, -50}, {96, -50}}, color = {0, 0, 255})}));
        end Transformer;

        model M_Transformer "Generic transformer with free number of inductors"
          parameter Integer N(final min = 1) = 3 "Number of inductors";
        protected
          parameter Integer dimL = div(N * (N + 1), 2);
        public
          parameter Modelica.SIunits.Inductance L[dimL] = {1, 0.1, 0.2, 2, 0.3, 3} "Inductances and coupling inductances";
          Modelica.Electrical.Analog.Interfaces.PositivePin p[N] "Positive pin" annotation(
            Placement(transformation(extent = {{-80, -40}, {-62, 40}})));
          Modelica.Electrical.Analog.Interfaces.NegativePin n[N] "Negative pin" annotation(
            Placement(transformation(extent = {{62, -40}, {80, 40}})));
          Modelica.SIunits.Voltage v[N] "Voltage drop over inductors";
          Modelica.SIunits.Current i[N](each start = 0, fixed = true) "Current through inductors";
          parameter Modelica.SIunits.Inductance Lm[N, N](each final fixed = false) "Complete symmetric inductance matrix, calculated internally";
        initial equation
          for s in 1:N loop
            for z in 1:N loop
              Lm[z, s] = if z >= s then L[(s - 1) * N + z - div((s - 1) * s, 2)] else Lm[s, z];
            end for;
          end for;
        equation
          for j in 1:N loop
            v[j] = p[j].v - n[j].v;
            0 = p[j].i + n[j].i;
            i[j] = p[j].i;
          end for;
          v = Lm * der(i);
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-36, 24}, {-18, 42}}, lineColor = {0, 0, 255}), Ellipse(extent = {{18, 24}, {36, 42}}, lineColor = {0, 0, 255}), Ellipse(extent = {{0, 24}, {18, 42}}, lineColor = {0, 0, 255}), Ellipse(extent = {{-18, 24}, {0, 42}}, lineColor = {0, 0, 255}), Rectangle(extent = {{42, 23}, {-44, 34}}, lineColor = {0, 0, 255}, pattern = LinePattern.None, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{-36, 34}, {-62, 34}}, color = {0, 0, 255}), Line(points = {{62, 34}, {36, 34}}, color = {0, 0, 255}), Ellipse(extent = {{-36, 8}, {-18, 26}}, lineColor = {0, 0, 255}), Ellipse(extent = {{18, 8}, {36, 26}}, lineColor = {0, 0, 255}), Ellipse(extent = {{0, 8}, {18, 26}}, lineColor = {0, 0, 255}), Ellipse(extent = {{-18, 8}, {0, 26}}, lineColor = {0, 0, 255}), Rectangle(extent = {{42, 6}, {-44, 18}}, lineColor = {0, 0, 255}, pattern = LinePattern.None, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{-36, 18}, {-62, 18}}, color = {0, 0, 255}), Line(points = {{62, 18}, {36, 18}}, color = {0, 0, 255}), Line(points = {{-36, -29}, {-62, -29}}, color = {0, 0, 255}), Ellipse(extent = {{-36, -40}, {-18, -22}}, lineColor = {0, 0, 255}), Ellipse(extent = {{-18, -40}, {0, -22}}, lineColor = {0, 0, 255}), Ellipse(extent = {{0, -40}, {18, -22}}, lineColor = {0, 0, 255}), Ellipse(extent = {{18, -40}, {36, -22}}, lineColor = {0, 0, 255}), Line(points = {{62, -29}, {36, -29}}, color = {0, 0, 255}), Rectangle(extent = {{42, -42}, {-44, -30}}, lineColor = {0, 0, 255}, pattern = LinePattern.None, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{0, 8}, {0, -18}}, color = {0, 0, 255}, pattern = LinePattern.Dot), Text(extent = {{-150, 103}, {150, 63}}, textString = "%name", lineColor = {0, 0, 255})}),
            Documentation(info = "<html>
<p>The model <i>M_Transformer</i> is a model of a transformer with the possibility to choose the number of inductors. Inside the model, an inductance matrix is built based on the inductance of the inductors and the coupling inductances between the inductors given as a parameter vector from the user of the model.</p>

<p>An example shows that approach:<br>
The user chooses a model with <b>three</b> inductors, that means the parameter <i><b>N</b></i> has to be <b>3</b>. Then he has to specify the inductances of the three inductors and the three coupling inductances. The coupling inductances are no real existing devices, but effects that occur between two inductors. The inductivities (main diagonal of the inductance matrix) and the coupling inductivities have to be specified in the parameter vector <i>L</i>. The length <i>dimL</i> of the parameter vector is calculated as follows: <i><b>dimL=(N*(N+1))/2</b></i></p>

<p>The following example shows how the parameter vector is used to fill in the inductance matrix. To specify the inductance matrix of a three inductances transformer (<i>N=3</i>):
</p>

<p>
<img
 src=\"modelica://Modelica/Resources/Images/Electrical/Analog/Basic/M_Transformer-eq.png\"
 alt=\"L_m\">
</p>

<p>
the user has to allocate the parameter vector <i>L[6] </i>, since <i>Nv=(N*(N+1))/2=(3*(3+1))/2=6</i>. The parameter vector must be filled like this: <i>L=[1,0.1,0.2,2,0.3,3] </i>.</p>
<p>Inside the model, two loops are used to fill the inductance matrix to guarantee that it is filled in a symmetric way.</p>
</html>", revisions = "<html>
<table border=\"1\" cellspacing=\"0\" cellpadding=\"2\">
    <tr>
      <th>Version</th>
      <th>Revision</th>
      <th>Date</th>
      <th>Author</th>
      <th>Comment</th>
    </tr>
   <tr>
      <td valign=\"top\"></td>
      <td valign=\"top\">4163</td>
      <td valign=\"top\">2010-09-11</td>
      <td valign=\"top\">Dietmar Winkler</td>
      <td valign=\"top\">Documentation corrected according to documentation guidelines.</td>
    </tr>
    <tr>
      <td valign=\"top\"></td>
      <td valign=\"top\"></td>
      <td valign=\"top\">2008-11-24</td>
      <td valign=\"top\">Kristin Majetta</td>
      <td valign=\"top\">Documentation added.</td>
    </tr>
    <tr>
      <td valign=\"top\"></td>
      <td valign=\"top\"></td>
      <td valign=\"top\">2008-11-16</td>
      <td valign=\"top\">Kristin Majetta</td>
      <td valign=\"top\">Initially implemented</td>
    </tr>
</table>
</html>"));
        end M_Transformer;

        model Gyrator "Gyrator"
          extends Interfaces.TwoPort;
          parameter SI.Conductance G1(start = 1) "Gyration conductance";
          parameter SI.Conductance G2(start = 1) "Gyration conductance";
        equation
          i1 = G2 * v2;
          i2 = -G1 * v1;
          annotation(
            Documentation(info = "<html>
<p>A gyrator is a two-port element defined by the following equations:</p>
<pre>    i1 =  G2 * v2
    i2 = -G1 * v1</pre>
<p>where the constants <i>G1</i>, <i>G2</i> are called the gyration conductance.</p>
</html>", revisions = "<html>
<ul>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-60, 60}, {60, -60}}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Line(points = {{-90, 50}, {-60, 50}}, color = {0, 0, 255}), Line(points = {{-90, -50}, {-60, -50}}, color = {0, 0, 255}), Line(points = {{60, 50}, {90, 50}}, color = {0, 0, 255}), Line(points = {{60, -50}, {90, -50}}, color = {0, 0, 255}), Line(points = {{-40, 30}, {40, 30}}, color = {0, 0, 255}), Line(points = {{-20, -20}, {20, -20}}, color = {0, 0, 255}), Polygon(points = {{30, 34}, {40, 30}, {30, 26}, {30, 34}}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Polygon(points = {{-26, -19}, {-16, -15}, {-16, -23}, {-26, -19}}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Line(points = {{-5, 10}, {-10, -10}}, color = {0, 0, 0}), Line(points = {{9, 10}, {4, -9}}, color = {0, 0, 0}), Line(points = {{-14, 9}, {16, 10}}, color = {0, 0, 0}), Text(extent = {{-29, 59}, {30, 30}}, textString = "G1", lineColor = {0, 0, 0}), Text(extent = {{-29, -29}, {29, -58}}, textString = "G2", lineColor = {0, 0, 0}), Text(extent = {{-156, 117}, {144, 77}}, textString = "%name", lineColor = {0, 0, 255})}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-60, 60}, {60, -60}}, lineColor = {0, 0, 255}), Line(points = {{-96, 50}, {-60, 50}}, color = {0, 0, 255}), Line(points = {{-96, -50}, {-60, -50}}, color = {0, 0, 255}), Line(points = {{60, 50}, {96, 50}}, color = {0, 0, 255}), Line(points = {{60, -50}, {96, -50}}, color = {0, 0, 255}), Line(points = {{-40, 30}, {40, 30}}, color = {0, 0, 255}), Line(points = {{-21, -22}, {19, -22}}, color = {0, 0, 255}), Polygon(points = {{30, 34}, {40, 30}, {30, 26}, {30, 34}}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Polygon(points = {{-20, -30}, {-10, -26}, {-10, -34}, {-20, -30}}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Line(points = {{-4, 5}, {-6, -5}}, thickness = 1, color = {0, 0, 255}), Line(points = {{3, 5}, {1, -5}}, thickness = 1, color = {0, 0, 255}), Line(points = {{-8, 5}, {7, 5}}, thickness = 1, color = {0, 0, 255}), Text(extent = {{-20, 50}, {20, 35}}, textString = "G1", lineColor = {0, 0, 255}), Text(extent = {{-20, -35}, {20, -50}}, textString = "G2", lineColor = {0, 0, 255})}));
        end Gyrator;

        model EMF "Electromotoric force (electric/mechanic transformer)"
          parameter Boolean useSupport = false "= true, if support flange enabled, otherwise implicitly grounded" annotation(
            Evaluate = true,
            HideResult = true,
            choices(checkBox = true));
          parameter SI.ElectricalTorqueConstant k(start = 1) "Transformation coefficient";
          SI.Voltage v "Voltage drop between the two pins";
          SI.Current i "Current flowing from positive to negative pin";
          SI.Angle phi "Angle of shaft flange with respect to support (= flange.phi - support.phi)";
          SI.AngularVelocity w "Angular velocity of flange relative to support";
          Interfaces.PositivePin p annotation(
            Placement(transformation(origin = {0, 100}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
          Interfaces.NegativePin n annotation(
            Placement(transformation(origin = {0, -100}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
          Modelica.Mechanics.Rotational.Interfaces.Flange_b flange annotation(
            Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
          Mechanics.Rotational.Interfaces.Support support if useSupport "Support/housing of emf shaft" annotation(
            Placement(transformation(extent = {{-110, -10}, {-90, 10}})));
        protected
          Mechanics.Rotational.Components.Fixed fixed if not useSupport annotation(
            Placement(transformation(extent = {{-90, -20}, {-70, 0}})));
          Mechanics.Rotational.Interfaces.InternalSupport internalSupport(tau = -flange.tau) annotation(
            Placement(transformation(extent = {{-90, -10}, {-70, 10}})));
        equation
          v = p.v - n.v;
          0 = p.i + n.i;
          i = p.i;
          phi = flange.phi - internalSupport.phi;
          w = der(phi);
          k * w = v;
          flange.tau = -k * i;
          connect(internalSupport.flange, support) annotation(
            Line(points = {{-80, 0}, {-100, 0}}, color = {0, 0, 0}, smooth = Smooth.None));
          connect(internalSupport.flange, fixed.flange) annotation(
            Line(points = {{-80, 0}, {-80, -10}}, color = {0, 0, 0}, smooth = Smooth.None));
          annotation(
            defaultComponentName = "emf",
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-85, 10}, {-36, -10}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192, 192, 192}), Line(points = {{0, 90}, {0, 40}}, color = {0, 0, 255}), Rectangle(extent = {{35, 10}, {100, -10}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192, 192, 192}), Ellipse(extent = {{-40, 40}, {40, -40}}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Line(points = {{0, -90}, {0, -40}}, color = {0, 0, 255}), Text(extent = {{0, -50}, {199, -90}}, textString = "%name", lineColor = {0, 0, 255}), Text(extent = {{0, 80}, {189, 46}}, lineColor = {160, 160, 164}, textString = "k=%k"), Line(visible = not useSupport, points = {{-100, -30}, {-40, -30}}, color = {0, 0, 0}), Line(visible = not useSupport, points = {{-100, -50}, {-80, -30}}, color = {0, 0, 0}), Line(visible = not useSupport, points = {{-80, -50}, {-60, -30}}, color = {0, 0, 0}), Line(visible = not useSupport, points = {{-60, -50}, {-40, -30}}, color = {0, 0, 0}), Line(visible = not useSupport, points = {{-70, -30}, {-70, -10}}, color = {0, 0, 0})}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{-17, 95}, {-20, 85}, {-23, 95}, {-17, 95}}, lineColor = {160, 160, 164}, fillColor = {160, 160, 164}, fillPattern = FillPattern.Solid), Line(points = {{-20, 110}, {-20, 85}}, color = {160, 160, 164}), Text(extent = {{-40, 110}, {-30, 90}}, lineColor = {160, 160, 164}, textString = "i"), Line(points = {{9, 75}, {19, 75}}, color = {192, 192, 192}), Line(points = {{-20, -110}, {-20, -85}}, color = {160, 160, 164}), Polygon(points = {{-17, -100}, {-20, -110}, {-23, -100}, {-17, -100}}, lineColor = {160, 160, 164}, fillColor = {160, 160, 164}, fillPattern = FillPattern.Solid), Text(extent = {{-40, -110}, {-30, -90}}, lineColor = {160, 160, 164}, textString = "i"), Line(points = {{8, -79}, {18, -79}}, color = {192, 192, 192}), Line(points = {{14, 80}, {14, 70}}, color = {192, 192, 192})}),
            Documentation(info = "<html>
<p>EMF transforms electrical energy into rotational mechanical energy. It is used as basic building block of an electrical motor. The mechanical connector flange can be connected to elements of the Modelica.Mechanics.Rotational library. flange.tau is the cut-torque, flange.phi is the angle at the rotational connection.</p>
</html>", revisions = "<html>
<ul>
<li><i> 1998   </i>
       by Martin Otter<br> initially implemented<br>
       </li>
</ul>
</html>"));
        end EMF;

        model TranslationalEMF "Electromotoric force (electric/mechanic transformer)"
          parameter Boolean useSupport = false "= true, if support flange enabled, otherwise implicitly grounded" annotation(
            Evaluate = true,
            HideResult = true,
            choices(checkBox = true));
          parameter Modelica.SIunits.ElectricalForceConstant k(start = 1) "Transformation coefficient";
          Modelica.SIunits.Voltage v "Voltage drop between the two pins";
          Modelica.SIunits.Current i "Current flowing from positive to negative pin";
          Modelica.SIunits.Position s "Position of flange relative to support";
          Modelica.SIunits.Velocity vel "Velocity of flange relative to support";
          Modelica.Electrical.Analog.Interfaces.PositivePin p annotation(
            Placement(transformation(origin = {0, 100}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
          Modelica.Electrical.Analog.Interfaces.NegativePin n annotation(
            Placement(transformation(origin = {0, -100}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
          Modelica.Mechanics.Translational.Interfaces.Flange_b flange annotation(
            Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
          Modelica.Mechanics.Translational.Interfaces.Support support if useSupport "Support/housing" annotation(
            Placement(transformation(extent = {{-110, -10}, {-90, 10}})));
        protected
          Modelica.Mechanics.Translational.Components.Fixed fixed if not useSupport annotation(
            Placement(transformation(extent = {{-90, -20}, {-70, 0}})));
          Modelica.Mechanics.Translational.Interfaces.InternalSupport internalSupport(f = -flange.f) annotation(
            Placement(transformation(extent = {{-90, -10}, {-70, 10}})));
        equation
          v = p.v - n.v;
          0 = p.i + n.i;
          i = p.i;
          s = flange.s - internalSupport.s;
          vel = der(s);
          k * vel = v;
          flange.f = -k * i;
          connect(internalSupport.flange, support) annotation(
            Line(points = {{-80, 0}, {-90, 0}, {-90, 0}, {-100, 0}}, color = {0, 127, 0}, smooth = Smooth.None));
          connect(internalSupport.flange, fixed.flange) annotation(
            Line(points = {{-80, 0}, {-80, -10}}, color = {0, 127, 0}, smooth = Smooth.None));
          annotation(
            defaultComponentName = "emf",
            Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-90, 51}, {-40, -50}}, lineColor = {0, 0, 0}, fillColor = {135, 135, 135}, fillPattern = FillPattern.HorizontalCylinder), Rectangle(extent = {{-21, 20}, {90, -20}}, lineColor = {135, 135, 135}, fillColor = {135, 135, 135}, fillPattern = FillPattern.Solid), Text(extent = {{0, 80}, {148, 44}}, lineColor = {160, 160, 164}, textString = "k=%k"), Line(points = {{-30, 49}, {-30, 80}, {0, 80}, {0, 91}}, color = {0, 0, 255}), Line(points = {{20, -49}, {20, -80}, {0, -80}, {0, -89}, {0, -90}}, color = {0, 0, 255}), Ellipse(extent = {{-21, 50}, {9, -50}}, lineColor = {0, 0, 255}), Ellipse(extent = {{2, 50}, {32, -50}}, lineColor = {0, 0, 255}), Ellipse(extent = {{-43, 50}, {-13, -50}}, lineColor = {0, 0, 255}), Rectangle(extent = {{-4, 20}, {-1, -20}}, lineColor = {135, 135, 135}, fillColor = {135, 135, 135}, fillPattern = FillPattern.Solid), Rectangle(extent = {{7, 20}, {10, -20}}, lineColor = {135, 135, 135}, fillColor = {135, 135, 135}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-14, 20}, {-11, -20}}, lineColor = {135, 135, 135}, fillColor = {135, 135, 135}, fillPattern = FillPattern.Solid), Rectangle(extent = {{19, 20}, {44, -20}}, lineColor = {135, 135, 135}, fillColor = {135, 135, 135}, fillPattern = FillPattern.Solid), Line(visible = not useSupport, points = {{-100, -70}, {-40, -70}}, color = {0, 0, 0}), Line(visible = not useSupport, points = {{-100, -90}, {-80, -70}}, color = {0, 0, 0}), Line(visible = not useSupport, points = {{-80, -90}, {-60, -70}}, color = {0, 0, 0}), Line(visible = not useSupport, points = {{-60, -90}, {-40, -70}}, color = {0, 0, 0}), Line(visible = not useSupport, points = {{-70, -70}, {-70, -50}}, color = {0, 0, 0}), Text(extent = {{0, -50}, {199, -90}}, textString = "%name", lineColor = {0, 0, 255})}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{-17, 95}, {-20, 85}, {-23, 95}, {-17, 95}}, lineColor = {160, 160, 164}, fillColor = {160, 160, 164}, fillPattern = FillPattern.Solid), Line(points = {{-20, 110}, {-20, 85}}, color = {160, 160, 164}), Text(extent = {{-40, 110}, {-30, 90}}, lineColor = {160, 160, 164}, textString = "i"), Line(points = {{9, 75}, {19, 75}}, color = {192, 192, 192}), Line(points = {{-20, -110}, {-20, -85}}, color = {160, 160, 164}), Polygon(points = {{-17, -100}, {-20, -110}, {-23, -100}, {-17, -100}}, lineColor = {160, 160, 164}, fillColor = {160, 160, 164}, fillPattern = FillPattern.Solid), Text(extent = {{-40, -110}, {-30, -90}}, lineColor = {160, 160, 164}, textString = "i"), Line(points = {{8, -79}, {18, -79}}, color = {192, 192, 192}), Line(points = {{14, 80}, {14, 70}}, color = {192, 192, 192}), Polygon(points = {{140, 3}, {150, 0}, {140, -3}, {140, 3}, {140, 3}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid)}),
            Documentation(info = "<html>
<p>EMF transforms electrical energy into translational mechanical energy. It is used as basic building block of an electrical linear motor. The mechanical connector flange can be connected to elements of the Modelica.Mechanics.Translational library. flange.f is the cut-force, flange.s is the distance at the translational connection.</p>
</html>", revisions = "<html>
<dl>
<dt>2009</dt>
<dd>by Anton Haumer<br> initially implemented</dd>
</dl>
</html>"));
        end TranslationalEMF;

        model VCV "Linear voltage-controlled voltage source"
          extends Interfaces.TwoPort;
          parameter Real gain(start = 1) "Voltage gain";
        equation
          v2 = v1 * gain;
          i1 = 0;
          annotation(
            Documentation(info = "<html>
<p>The linear voltage-controlled voltage source is a TwoPort. The right port voltage v2 is controlled by the left port voltage v1 via</p>
<pre>    v2 = v1 * gain. </pre>
<p>The left port current is zero. Any voltage gain can be chosen.</p>
</html>", revisions = "<html>
<ul>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-70, 70}, {70, -70}}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Line(points = {{-90, 50}, {-30, 50}}, color = {0, 0, 255}), Line(points = {{-30, -50}, {-90, -50}}, color = {0, 0, 255}), Line(points = {{100, 50}, {30, 50}, {30, -50}, {100, -50}}, color = {0, 0, 255}), Ellipse(extent = {{10, 20}, {50, -20}}, lineColor = {0, 0, 255}), Line(points = {{-20, 60}, {20, 60}}, color = {0, 0, 255}), Polygon(points = {{20, 60}, {10, 63}, {10, 57}, {20, 60}}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Text(extent = {{-148, -81}, {152, -121}}, textString = "%name", lineColor = {0, 0, 255})}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-96, 50}, {-31, 50}}, color = {0, 0, 255}), Line(points = {{-30, -50}, {-96, -50}}, color = {0, 0, 255}), Line(points = {{96, 50}, {30, 50}, {30, -50}, {96, -50}}, color = {0, 0, 255}), Ellipse(extent = {{10, 20}, {50, -20}}, lineColor = {0, 0, 255}), Rectangle(extent = {{-70, 70}, {70, -70}}, lineColor = {0, 0, 255}), Line(points = {{-20, 60}, {20, 60}}, color = {0, 0, 255}), Polygon(points = {{20, 60}, {10, 63}, {10, 57}, {20, 60}}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255})}));
        end VCV;

        model VCC "Linear voltage-controlled current source"
          extends Interfaces.TwoPort;
          parameter SI.Conductance transConductance(start = 1) "Transconductance";
        equation
          i2 = v1 * transConductance;
          i1 = 0;
          annotation(
            Documentation(info = "<html>
<p>The linear voltage-controlled current source is a TwoPort. The right port current i2 is controlled by the left port voltage v1 via</p>
<pre>    i2 = v1 * transConductance. </pre>
<p>The left port current is zero. Any transConductance can be chosen.</p>
</html>", revisions = "<html>
<ul>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-70, 70}, {70, -70}}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Line(points = {{-90, 50}, {-30, 50}}, color = {0, 0, 255}), Line(points = {{-30, -50}, {-90, -50}}, color = {0, 0, 255}), Ellipse(extent = {{10, 20}, {50, -20}}, lineColor = {0, 0, 255}), Line(points = {{-20, 60}, {20, 60}}, color = {0, 0, 255}), Polygon(points = {{20, 60}, {10, 63}, {10, 57}, {20, 60}}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Line(points = {{90, 50}, {30, 50}, {30, 20}}, color = {0, 0, 255}), Line(points = {{91, -50}, {30, -50}, {30, -20}}, color = {0, 0, 255}), Line(points = {{10, 0}, {50, 0}}, color = {0, 0, 255}), Text(extent = {{-143, -82}, {157, -122}}, textString = "%name", lineColor = {0, 0, 255})}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-96, 50}, {-30, 50}}, color = {0, 0, 255}), Line(points = {{-30, -50}, {-96, -50}}, color = {0, 0, 255}), Ellipse(extent = {{10, 20}, {50, -20}}, lineColor = {0, 0, 255}), Rectangle(extent = {{-70, 70}, {70, -70}}, lineColor = {0, 0, 255}), Line(points = {{-20, 60}, {20, 60}}, color = {0, 0, 255}), Polygon(points = {{20, 60}, {10, 63}, {10, 57}, {20, 60}}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Line(points = {{96, 50}, {30, 50}, {30, 20}}, color = {0, 0, 255}), Line(points = {{96, -50}, {30, -50}, {30, -20}}, color = {0, 0, 255}), Line(points = {{10, 0}, {50, 0}}, color = {0, 0, 255})}));
        end VCC;

        model CCV "Linear current-controlled voltage source"
          extends Interfaces.TwoPort;
          parameter SI.Resistance transResistance(start = 1) "Transresistance";
        equation
          v2 = i1 * transResistance;
          v1 = 0;
          annotation(
            Documentation(info = "<html>
<p>The linear current-controlled voltage source is a TwoPort. The right port voltage v2 is controlled by the left port current i1 via</p>
<pre>    v2 = i1 * transResistance. </pre>
<p>The left port voltage is zero. Any transResistance can be chosen.</p>
</html>", revisions = "<html>
<ul>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-70, 70}, {70, -70}}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Line(points = {{100, 50}, {30, 50}, {30, -50}, {100, -50}}, color = {0, 0, 255}), Ellipse(extent = {{10, 20}, {50, -20}}, lineColor = {0, 0, 255}), Line(points = {{-20, 60}, {20, 60}}, color = {0, 0, 255}), Polygon(points = {{20, 60}, {10, 63}, {10, 57}, {20, 60}}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Line(points = {{-90, 50}, {-20, 50}, {-20, -50}, {-90, -50}}, color = {0, 0, 255}), Text(extent = {{-140, -82}, {160, -122}}, textString = "%name", lineColor = {0, 0, 255})}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{10, 20}, {50, -20}}, lineColor = {0, 0, 255}), Rectangle(extent = {{-70, 70}, {70, -70}}, lineColor = {0, 0, 255}), Line(points = {{-20, 60}, {20, 60}}, color = {0, 0, 255}), Polygon(points = {{20, 60}, {10, 63}, {10, 57}, {20, 60}}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Line(points = {{96, 50}, {30, 50}, {30, -50}, {96, -50}}, color = {0, 0, 255}), Line(points = {{-96, 50}, {-30, 50}, {-30, -50}, {-96, -50}}, color = {0, 0, 255})}));
        end CCV;

        model CCC "Linear current-controlled current source"
          extends Interfaces.TwoPort;
          parameter Real gain(start = 1) "Current gain";
        equation
          i2 = i1 * gain;
          v1 = 0;
          annotation(
            Documentation(info = "<html>
<p>The linear current-controlled current source is a TwoPort. The right port current i2 is controlled by the left port current i1 via</p>
<pre>    i2 = i1 * gain. </pre>
<p>The left port voltage is zero. Any current gain can be chosen.</p>
</html>", revisions = "<html>
<ul>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-70, 70}, {70, -70}}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Line(points = {{-100, 50}, {-30, 50}, {-30, -50}, {-100, -50}}, color = {0, 0, 255}), Ellipse(extent = {{10, 20}, {50, -20}}, lineColor = {0, 0, 255}), Line(points = {{-20, 60}, {20, 60}}, color = {0, 0, 255}), Polygon(points = {{20, 60}, {10, 63}, {10, 57}, {20, 60}}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Line(points = {{90, 50}, {30, 50}, {30, 20}}, color = {0, 0, 255}), Line(points = {{91, -50}, {30, -50}, {30, -20}}, color = {0, 0, 255}), Line(points = {{10, 0}, {50, 0}}, color = {0, 0, 255}), Text(extent = {{-146, -76}, {154, -116}}, textString = "%name", lineColor = {0, 0, 255})}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{10, 20}, {50, -20}}, lineColor = {0, 0, 255}), Rectangle(extent = {{-70, 70}, {70, -70}}, lineColor = {0, 0, 255}), Line(points = {{-20, 60}, {20, 60}}, color = {0, 0, 255}), Polygon(points = {{20, 60}, {10, 63}, {10, 57}, {20, 60}}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Line(points = {{96, 50}, {30, 50}, {30, 20}}, color = {0, 0, 255}), Line(points = {{96, -50}, {30, -50}, {30, -20}}, color = {0, 0, 255}), Line(points = {{10, 0}, {50, 0}}, color = {0, 0, 255}), Line(points = {{-96, 50}, {-30, 50}, {-30, -50}, {-96, -50}}, color = {0, 0, 255})}));
        end CCC;

        model OpAmp "Simple nonideal model of an OpAmp with limitation"
          parameter Real Slope(start = 10000) "Slope of the out.v/vin characteristic at vin=0";
          Modelica.Electrical.Analog.Interfaces.PositivePin in_p "Positive pin of the input port" annotation(
            Placement(transformation(extent = {{-110, -60}, {-90, -40}}, rotation = 0)));
          Modelica.Electrical.Analog.Interfaces.NegativePin in_n "Negative pin of the input port" annotation(
            Placement(transformation(extent = {{-90, 40}, {-110, 60}}, rotation = 0)));
          Modelica.Electrical.Analog.Interfaces.PositivePin out "Output pin" annotation(
            Placement(transformation(extent = {{110, -10}, {90, 10}}, rotation = 0)));
          Modelica.Electrical.Analog.Interfaces.PositivePin VMax "Positive output voltage limitation" annotation(
            Placement(transformation(extent = {{-10, 60}, {10, 80}}, rotation = 0)));
          Modelica.Electrical.Analog.Interfaces.NegativePin VMin "Negative output voltage limitation" annotation(
            Placement(transformation(extent = {{-10, -80}, {10, -60}}, rotation = 0)));
          SI.Voltage vin "input voltage";
        protected
          Real f "auxiliary variable";
          Real absSlope;
        equation
          in_p.i = 0;
          in_n.i = 0;
          VMax.i = 0;
          VMin.i = 0;
          vin = in_p.v - in_n.v;
          f = 2 / (VMax.v - VMin.v);
          absSlope = if Slope < 0 then -Slope else Slope;
          out.v = (VMax.v + VMin.v) / 2 + absSlope * vin / (1 + absSlope * smooth(0, if f * vin < 0 then -f * vin else f * vin));
          annotation(
            Documentation(info = "<html>
<p>The OpAmp is a simple nonideal model with a smooth out.v = f(vin) characteristic, where &quot;vin = in_p.v - in_n.v&quot;. The characteristic is limited by VMax.v and VMin.v. Its slope at vin=0 is the parameter Slope, which must be positive. (Therefore, the absolute value of Slope is taken into calculation.)</p>
</html>", revisions = "<html>
<ul>
<li><i> 2000   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{60, 0}, {-60, 70}, {-60, -70}, {60, 0}}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Line(points = {{-45, -10}, {-20, -10}, {-14, -9}, {-11, -7}, {-9, 7}, {-6, 9}, {0, 10}, {20, 10}}, color = {0, 0, 255}), Line(points = {{0, 35}, {0, 80}}, color = {0, 0, 255}), Line(points = {{0, -35}, {0, -80}}, color = {0, 0, 255}), Line(points = {{-90, 50}, {-60, 50}}, color = {0, 0, 255}), Line(points = {{-90, -50}, {-60, -50}}, color = {0, 0, 255}), Line(points = {{60, 0}, {90, 0}}, color = {0, 0, 255}), Line(points = {{-48, 32}, {-28, 32}}, color = {0, 0, 255}), Line(points = {{-39, -20}, {-39, -41}}, color = {0, 0, 255}), Line(points = {{-50, -31}, {-28, -31}}, color = {0, 0, 255}), Text(extent = {{-149, 138}, {151, 98}}, textString = "%name", lineColor = {0, 0, 255})}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{60, 0}, {-60, 70}, {-60, -70}, {60, 0}}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Line(points = {{-45, -10}, {-20, -10}, {-14, -9}, {-11, -7}, {-9, 7}, {-6, 9}, {0, 10}, {20, 10}}, color = {0, 0, 255}), Line(points = {{0, 35}, {0, 70}}, color = {0, 0, 255}), Line(points = {{0, -35}, {0, -70}}, color = {0, 0, 255}), Line(points = {{-96, 50}, {-60, 50}}, color = {0, 0, 255}), Line(points = {{-96, -50}, {-60, -50}}, color = {0, 0, 255}), Line(points = {{60, 0}, {96, 0}}, color = {0, 0, 255}), Line(points = {{-55, 50}, {-45, 50}}, color = {0, 0, 255}), Line(points = {{-50, -45}, {-50, -55}}, color = {0, 0, 255}), Line(points = {{-55, -50}, {-45, -50}}, color = {0, 0, 255}), Text(extent = {{-112, -93}, {-91, -73}}, lineColor = {160, 160, 164}, textString = "in_p.i=0"), Polygon(points = {{120, 3}, {110, 0}, {120, -3}, {120, 3}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {160, 160, 164}), Line(points = {{111, 0}, {136, 0}}, color = {0, 0, 0}), Text(extent = {{118, 2}, {135, 17}}, lineColor = {0, 0, 0}, textString = "i2"), Text(extent = {{-111, 60}, {-85, 80}}, lineColor = {160, 160, 164}, textString = "in_n.i=0"), Line(points = {{-100, -35}, {-100, 23}, {-100, 24}}, color = {160, 160, 164}, arrow = {Arrow.None, Arrow.Filled}), Text(extent = {{-97, -16}, {-74, 4}}, lineColor = {160, 160, 164}, textString = "vin")}));
        end OpAmp;

        model OpAmpDetailed "Detailed model of an operational amplifier"
          // literature: Conelly, J.A.; Choi, P.: Macromodelling with SPICE. Englewood Cliffs: Prentice-Hall, 1992
          parameter SI.Resistance Rdm = 2.0e6 "Input resistance (differential input mode)";
          parameter SI.Resistance Rcm = 2.0e9 "Input resistance (common mode)";
          parameter SI.Capacitance Cin = 1.4e-12 "Input capacitance";
          parameter SI.Voltage Vos = 1.0e-3 "Input offset voltage";
          parameter SI.Current Ib = 80.0e-9 "Input bias current";
          parameter SI.Current Ios = 20.0e-9 "Input offset current";
          parameter SI.Voltage vcp = 0.0 "Correction value for limiting by p_supply";
          parameter SI.Voltage vcm = 0.0 "Correction value for limiting by msupply";
          parameter Real Avd0 = 106.0 "Differential amplifier [dB]";
          parameter Real CMRR = 90.0 "Common-mode rejection [dB]";
          parameter SI.Frequency fp1 = 5.0 "Dominant pole";
          parameter SI.Frequency fp2 = 2.0e6 "Pole frequency";
          parameter SI.Frequency fp3 = 20.0e6 "Pole frequency";
          parameter SI.Frequency fp4 = 100.0e6 "Pole frequency";
          parameter SI.Frequency fz = 5.0e6 "Zero frequency";
          parameter SI.VoltageSlope sr_p = 0.5e6 "Slew rate for increase";
          parameter SI.VoltageSlope sr_m = 0.5e6 "Slew rate for decrease";
          parameter SI.Resistance Rout = 75.0 "Output resistance";
          parameter SI.Current Imaxso = 25.0e-3 "Maximal output current (source current)";
          parameter SI.Current Imaxsi = 25.0e-3 "Maximal output current (sink current)";
          // number of intervals: 2500, stop time: 0.003
          parameter SI.Time Ts = 0.0000012 "sampling time";
          // constant expressions
          constant Real Pi = 3.141592654;
          // power supply
          final parameter SI.Voltage vcp_abs = abs(vcp) "Positive correction value for limiting by p_supply";
          final parameter SI.Voltage vcm_abs = abs(vcm) "Positive correction value for limiting by msupply";
          // input stage
          //  Ib = 0.5*(I1 + I2);
          //  Ios = I1 - I2;
          final parameter SI.Current I1 = Ib + Ios / 2.0 "Current of internal source I1";
          final parameter SI.Current I2 = Ib - Ios / 2.0 "Current of internal source I2";
          // gain stage (difference and common mode)
          final parameter Real Avd0_val = 10.0 ^ (Avd0 / 20.0) "differential mode gain";
          final parameter Real Avcm_val = Avd0_val / 10.0 ^ (CMRR / 20.0) / 2.0 "common mode gain";
          // slew rate stage
          final parameter SI.VoltageSlope sr_p_val = abs(sr_p) "Value of slew rate for increase";
          final parameter SI.VoltageSlope sr_m_val = -abs(sr_m) "Negative alue of slew rate for increase";
          // output stage
          final parameter SI.Current Imaxso_val = abs(Imaxso) "Orientation out outp";
          final parameter SI.Current Imaxsi_val = abs(Imaxsi) "Orientation into outp";
          Modelica.Electrical.Analog.Interfaces.PositivePin p "Positive pin of the input port" annotation(
            Placement(transformation(extent = {{-111, -61}, {-90, -40}}, rotation = 0)));
          Modelica.Electrical.Analog.Interfaces.NegativePin m "Negative pin of the input port" annotation(
            Placement(transformation(extent = {{-90, 40}, {-111, 61}}, rotation = 0)));
          Modelica.Electrical.Analog.Interfaces.PositivePin outp "Output pin" annotation(
            Placement(transformation(extent = {{110, -10}, {90, 10}}, rotation = 0)));
          Modelica.Electrical.Analog.Interfaces.PositivePin p_supply "Positive output voltage limitation" annotation(
            Placement(transformation(extent = {{-8, 65}, {9, 82}}, rotation = 0)));
          Modelica.Electrical.Analog.Interfaces.NegativePin m_supply "Negative output voltage limitation" annotation(
            Placement(transformation(extent = {{-9, -83}, {9, -65}}, rotation = 0)));
          // power supply
          SI.Voltage v_pos;
          SI.Voltage v_neg;
          // input stage
          Modelica.SIunits.Voltage v_vos;
          Modelica.SIunits.Voltage v_3;
          Modelica.SIunits.Voltage v_in;
          Modelica.SIunits.Voltage v_4;
          Modelica.SIunits.Current i_vos;
          Modelica.SIunits.Current i_3;
          Modelica.SIunits.Current i_r2;
          Modelica.SIunits.Current i_c3;
          Modelica.SIunits.Current i_4;
          // frequency response
          Real q_fr1;
          Real q_fr2;
          Real q_fr3;
          // gain stage
          SI.Voltage q_sum;
          SI.Voltage q_sum_help;
          SI.Voltage q_fp1;
          // slew rate stage
          SI.Voltage v_source;
          SI.Voltage x "auxiliary variable for slew rate";
          // output stage
          Modelica.SIunits.Voltage v_out;
          Modelica.SIunits.Current i_out;
          // functions

          function FCNiout_limit "Internal limitation function"
            extends Modelica.Icons.Function;
            input SI.Voltage v_source;
            input SI.Voltage v_out;
            input SI.Resistance Rout;
            input SI.Current Imaxsi_val;
            input SI.Current Imaxso_val;
            output SI.Current result;
          algorithm
            if v_out > v_source + Rout * Imaxsi_val then
              result := Imaxsi_val;
            elseif v_out < v_source - Rout * Imaxso_val then
              result := -Imaxso_val;
            else
              result := (v_out - v_source) / Rout;
            end if;
            return;
            annotation(
              Documentation(info = "<html>
<p>Internal limitation function, designed for OpAmpDetailed, not for purpose of external usage.</p>
</html>"));
          end FCNiout_limit;

          function FCNq_sum_limit "Internal limitation function"
            extends Modelica.Icons.Function;
            input SI.Voltage q_sum;
            input SI.Voltage q_sum_ltf;
            input SI.Voltage v_pos;
            input SI.Voltage v_neg;
            input SI.Voltage vcp;
            input SI.Voltage vcm;
            output SI.Voltage result;
          algorithm
            if q_sum > v_pos - vcp and q_sum_ltf >= v_pos - vcp then
              result := v_pos - vcp;
            elseif q_sum < v_neg + vcm and q_sum_ltf <= v_neg + vcm then
              result := v_neg + vcm;
            else
              result := q_sum;
            end if;
            return;
            annotation(
              Documentation(info = "<html>
<p>Internal limitation function, designed for OpAmpDetailed, not for purpose of external usage.</p>
</html>"));
          end FCNq_sum_limit;
        initial equation
          v_source = q_fp1;
          x = 0;
        equation
          assert(Rout > 0.0, "Rout must be > 0.0.");
// power supply
          v_pos = p_supply.v;
          v_neg = m_supply.v;
// input stage
          p.i = i_vos;
          m.i = i_4 - i_r2 - i_c3;
          0 = i_3 + i_r2 + i_c3 - i_vos;
          p.v - m.v = v_vos + v_in;
          v_4 = m.v;
          v_3 = p.v - v_vos;
          v_vos = Vos;
          i_3 = I1 + v_3 / Rcm;
          v_in = Rdm * i_r2;
          i_c3 = Cin * der(v_in);
          i_4 = I2 + v_4 / Rcm;
// frequency response
// Laplace transformation
          der(q_fr1) = 2.0 * Pi * fp2 * (v_in - q_fr1);
          q_fr2 + 1.0 / (2.0 * Pi * fp3) * der(q_fr2) = q_fr1 + 1.0 / (2.0 * Pi * fz) * der(q_fr1);
          der(q_fr3) = 2.0 * Pi * fp4 * (q_fr2 - q_fr3);
// gain stage
// Laplace transformation
          q_sum = Avd0_val * q_fr3 + Avcm_val * (v_3 + v_4);
          q_sum_help = FCNq_sum_limit(q_sum, q_fp1, v_pos, v_neg, vcp_abs, vcm_abs);
          der(q_fp1) = 2.0 * Pi * fp1 * (q_sum_help - q_fp1);
// slew rate stage
          der(x) = (q_fp1 - v_source) / Ts;
          der(v_source) = smooth(0, noEvent(if der(x) > sr_p_val then sr_p_val else if der(x) < sr_m_val then sr_m_val else der(x)));
// output stage
          v_out = outp.v;
          i_out = outp.i;
          i_out = FCNiout_limit(v_source, v_out, Rout, Imaxsi_val, Imaxso_val);
          p_supply.i = 0;
          m_supply.i = 0;
          annotation(
            Documentation(info = "<html>
<p>The OpAmpDetailed model is a general operational amplifier model. The emphasis is on separating each important data sheet parameter into a sub-circuit independent of the other parameters. The model is broken down into five functional stages <b>input</b>, <b>frequency response</b>, <b>gain</b>, <b>slew rate</b> and an <b>output</b> stage. Each stage contains data sheet parameters to be modeled. This partitioning and the modelling of the separate submodels are based on the description in <b>[CP92]</b>.</p>
<p>Using <b>[CP92]</b> Joachim Haase (Fraunhofer Institute for Integrated Circuits, Design Automation Division) transferred 2001 operational amplifier models into VHDL-AMS. Now one of these models, the model &quot;amp(macro)&quot; was transferred into Modelica.</p>
<dl><dt><b>Reference:</b> </dt>
<dd><b>[CP92]</b> Conelly, J.A.; Choi, P.: Macromodelling with SPICE. Englewood Cliffs: Prentice-Hall, 1992 </dd>
</dl></html>", revisions = "<html>
<dl>
<dt><i>June 17, 2009</i></dt>
<dd>by Susann Wolf initially implemented</dd>
</dl>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{60, 0}, {-60, 70}, {-60, -70}, {60, 0}}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Line(points = {{0, 35}, {0, 80}}, color = {0, 0, 255}), Line(points = {{0, -35}, {0, -80}}, color = {0, 0, 255}), Line(points = {{-90, 50}, {-60, 50}}, color = {0, 0, 255}), Line(points = {{-90, -50}, {-60, -50}}, color = {0, 0, 255}), Line(points = {{60, 0}, {90, 0}}, color = {0, 0, 255}), Line(points = {{-48, 32}, {-28, 32}}, color = {0, 0, 255}), Line(points = {{-39, -20}, {-39, -41}}, color = {0, 0, 255}), Line(points = {{-50, -31}, {-28, -31}}, color = {0, 0, 255}), Text(extent = {{-151, 141}, {149, 101}}, textString = "%name", lineColor = {0, 0, 255})}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{60, 0}, {-60, 70}, {-60, -70}, {60, 0}}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Line(points = {{0, 35}, {0, 70}}, color = {0, 0, 255}), Line(points = {{0, -35}, {0, -70}}, color = {0, 0, 255}), Line(points = {{-96, 50}, {-60, 50}}, color = {0, 0, 255}), Line(points = {{-96, -50}, {-60, -50}}, color = {0, 0, 255}), Line(points = {{60, 0}, {96, 0}}, color = {0, 0, 255}), Line(points = {{-55, 50}, {-45, 50}}, color = {0, 0, 255}), Line(points = {{-50, -45}, {-50, -55}}, color = {0, 0, 255}), Line(points = {{-55, -50}, {-45, -50}}, color = {0, 0, 255}), Polygon(points = {{120, 3}, {110, 0}, {120, -3}, {120, 3}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {160, 160, 164}), Line(points = {{111, 0}, {136, 0}}, color = {0, 0, 0}), Text(extent = {{114, 2}, {131, 17}}, lineColor = {0, 0, 0}, textString = "i2"), Line(points = {{-100, -35}, {-100, 23}, {-100, 24}}, color = {160, 160, 164}, arrow = {Arrow.None, Arrow.Filled}), Text(extent = {{-97, -16}, {-74, 4}}, lineColor = {160, 160, 164}, textString = "vin")}));
        end OpAmpDetailed;

        model VariableResistor "Ideal linear electrical resistor with variable resistance"
          parameter Modelica.SIunits.Temperature T_ref = 300.15 "Reference temperature";
          parameter Modelica.SIunits.LinearTemperatureCoefficient alpha = 0 "Temperature coefficient of resistance (R_actual = R*(1 + alpha*(T_heatPort - T_ref))";
          extends Modelica.Electrical.Analog.Interfaces.OnePort;
          extends Modelica.Electrical.Analog.Interfaces.ConditionalHeatPort(T = T_ref);
          Modelica.SIunits.Resistance R_actual "Actual resistance = R*(1 + alpha*(T_heatPort - T_ref))";
          Modelica.Blocks.Interfaces.RealInput R(unit = "Ohm") annotation(
            Placement(transformation(origin = {0, 110}, extent = {{-20, -20}, {20, 20}}, rotation = 270)));
        equation
          assert(1 + alpha * (T_heatPort - T_ref) >= Modelica.Constants.eps, "Temperature outside scope of model!");
          R_actual = R * (1 + alpha * (T_heatPort - T_ref));
          v = R_actual * i;
          LossPower = v * i;
          annotation(
            Documentation(info = "<html>
<p>The linear resistor connects the branch voltage <i>v</i> with the branch current <i>i</i> by
<br><i><b>i*R = v</b></i>
<br>The Resistance <i>R</i> is given as input signal.
<br><br><b>Attention!!!</b><br>It is recommended that the R signal should not cross the zero value. Otherwise depending on the surrounding circuit the probability of singularities is high.</p>
</html>", revisions = "<html>
<ul>
<li><i> August 07, 2009   </i>
       by Anton Haumer<br> temperature dependency of resistance added<br>
       </li>
<li><i> March 11, 2009   </i>
       by Christoph Clauss<br> conditional heat port added<br>
       </li>
<li><i>June 7, 2004   </i>
       by Christoph Clauss<br>changed, docu added<br>
       </li>
<li><i>April 30, 2004</i>
       by Anton Haumer<br>implemented.
       </li>
</ul>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-90, 0}, {-70, 0}}, color = {0, 0, 255}), Rectangle(extent = {{-70, 30}, {70, -30}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{70, 0}, {90, 0}}, color = {0, 0, 255}), Line(points = {{0, 90}, {0, 30}}, color = {0, 0, 255}), Text(extent = {{-148, -41}, {152, -81}}, textString = "%name", lineColor = {0, 0, 255})}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-70, 30}, {70, -30}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{-96, 0}, {-70, 0}}, color = {0, 0, 255}), Line(points = {{0, 90}, {0, 30}}, color = {0, 0, 255}), Line(points = {{70, 0}, {96, 0}}, color = {0, 0, 255})}));
        end VariableResistor;

        model VariableConductor "Ideal linear electrical conductor with variable conductance"
          parameter Modelica.SIunits.Temperature T_ref = 300.15 "Reference temperature";
          parameter Modelica.SIunits.LinearTemperatureCoefficient alpha = 0 "Temperature coefficient of conductance (G_actual = G/(1 + alpha*(T_heatPort - T_ref))";
          extends Modelica.Electrical.Analog.Interfaces.OnePort;
          extends Modelica.Electrical.Analog.Interfaces.ConditionalHeatPort(T = T_ref);
          Modelica.SIunits.Conductance G_actual "Actual conductance = G/(1 + alpha*(T_heatPort - T_ref))";
          Modelica.Blocks.Interfaces.RealInput G(unit = "S") annotation(
            Placement(transformation(origin = {0, 110}, extent = {{-20, -20}, {20, 20}}, rotation = 270)));
        equation
          assert(1 + alpha * (T_heatPort - T_ref) >= Modelica.Constants.eps, "Temperature outside scope of model!");
          G_actual = G / (1 + alpha * (T_heatPort - T_ref));
          i = G_actual * v;
          LossPower = v * i;
          annotation(
            Documentation(info = "<html>
<p>The linear conductor connects the branch voltage <i>v</i> with the branch current <i>i</i> by
<br><i><b>i = G*v</b></i>
<br>The Conductance <i>G</i> is given as input signal.
<br><br><b>Attention!!!</b>
<br>It is recommended that the G signal should not cross the zero value. Otherwise depending on the surrounding circuit the probability of singularities is high.</p>
</html>", revisions = "<html>
<ul>
<li><i> August 07, 2009   </i>
       by Anton Haumer<br> temperature dependency of conductance added<br>
       </li>
<li><i> March 11, 2009   </i>
       by Christoph Clauss<br> conditional heat port added<br>
       </li>
<li><i>June 7, 2004   </i>
       by Christoph Clauss<br> implemented<br>
       </li>
</ul>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-90, 0}, {-70, 0}}, color = {0, 0, 255}), Rectangle(extent = {{-70, 30}, {70, -30}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{70, 0}, {90, 0}}, color = {0, 0, 255}), Line(points = {{0, 90}, {0, 30}}, color = {0, 0, 255}), Text(extent = {{-148, -41}, {152, -81}}, textString = "%name", lineColor = {0, 0, 255})}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{0, 90}, {0, 30}}, color = {0, 0, 255}), Line(points = {{-96, 0}, {-70, 0}}, color = {0, 0, 255}), Line(points = {{70, 0}, {96, 0}}, color = {0, 0, 255}), Rectangle(extent = {{-70, 30}, {70, -30}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid)}));
        end VariableConductor;

        model VariableCapacitor "Ideal linear electrical capacitor with variable capacitance"
          extends Modelica.Electrical.Analog.Interfaces.OnePort;
          Modelica.Blocks.Interfaces.RealInput C(unit = "F") annotation(
            Placement(transformation(origin = {0, 110}, extent = {{-20, -20}, {20, 20}}, rotation = 270)));
          parameter Modelica.SIunits.Capacitance Cmin = Modelica.Constants.eps "lower bound for variable capacitance";
          Modelica.SIunits.ElectricCharge Q;
          parameter Modelica.SIunits.Voltage IC = 0 "Initial Value";
          parameter Boolean UIC = false;
        initial equation
          if UIC then
            v = IC;
          end if;
        equation
          assert(C >= 0, "Capacitance C (= " + String(C) + ") has to be >= 0!");
// protect solver from index change
          Q = noEvent(max(C, Cmin)) * v;
          i = der(Q);
          annotation(
            Documentation(info = "<html>
<p>The linear capacitor connects the branch voltage <i>v</i> with the branch current <i>i</i> by
<br><i><b>i = dQ/dt</b></i> with <i><b>Q = C * v</b></i>.
<br>The capacitance <i>C</i> is given as input signal.
It is required that C &ge; 0, otherwise an assertion is raised. To avoid a variable index system,
C = Cmin, if 0 &le; C &lt; Cmin, where Cmin is a parameter with default value Modelica.Constants.eps.</p>
<p><br/>Besides the Cmin parameter the capacitor model has got the two parameters IC and UIC that belong together. With the IC parameter the user can specify an initial value of the voltage over the capacitor, which is defined from positive pin p to negative pin n (v=p.v - n.v).</p>
<p><br/>Hence the capacitor is charged at the beginning of the simulation. The other parameter UIC is of type Boolean. If UIC is true, the simulation tool uses</p>
<p><br/>the IC value at the initial calculation by adding the equation v= IC. If UIC is false, the IC value can be used (but it does not need to!) to calculate the initial values in order to simplify the numerical algorithms of initial calculation.</p>
</html>", revisions = "<html>
<ul>
<li><i>June 7, 2004   </i>
       by Christoph Clauss<br>changed, docu added<br>
       </li>
<li><i>April 30, 2004</i>
       by Anton Haumer<br>implemented.
       </li>
</ul>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-90, 0}, {-6, 0}}, color = {0, 0, 255}), Line(points = {{6, 0}, {90, 0}}, color = {0, 0, 255}), Line(points = {{0, 90}, {0, 30}}, color = {0, 0, 255}), Line(points = {{-6, 28}, {-6, -28}}, color = {0, 0, 255}), Line(points = {{6, 28}, {6, -28}}, color = {0, 0, 255}), Text(extent = {{-144, -43}, {156, -83}}, textString = "%name", lineColor = {0, 0, 255})}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-96, 0}, {-14, 0}}, color = {0, 0, 255}), Line(points = {{14, 0}, {96, 0}}, color = {0, 0, 255}), Line(points = {{0, 90}, {0, 30}}, color = {0, 0, 255}), Line(points = {{-14, 28}, {-14, -28}}, thickness = 0.5, color = {0, 0, 255}), Line(points = {{14, 28}, {14, -28}}, thickness = 0.5, color = {0, 0, 255})}));
        end VariableCapacitor;

        model VariableInductor "Ideal linear electrical inductor with variable inductance"
          extends Modelica.Electrical.Analog.Interfaces.OnePort;
          Modelica.Blocks.Interfaces.RealInput L(unit = "H") annotation(
            Placement(transformation(origin = {0, 108}, extent = {{-20, -20}, {20, 20}}, rotation = 270)));
          Modelica.SIunits.MagneticFlux Psi;
          parameter Modelica.SIunits.Inductance Lmin = Modelica.Constants.eps "lower bound for variable inductance";
          parameter Modelica.SIunits.Current IC = 0 "Initial Value";
          parameter Boolean UIC = false;
        initial equation
          if UIC then
            i = IC;
          end if;
        equation
          assert(L >= 0, "Inductance L_ (= " + String(L) + ") has to be >= 0!");
// protect solver from index change
          Psi = noEvent(max(L, Lmin)) * i;
          v = der(Psi);
          annotation(
            Documentation(info = "<html>
<p>The linear inductor connects the branch voltage <i>v</i> with the branch current <i>i</i> by
<br><i><b>v = d Psi/dt </b></i>with <i><b>Psi = L * i </b></i>.
<br>The inductance <i>L</i> is as input signal.
It is required that L &ge; 0, otherwise an assertion is raised. To avoid a variable index system, L = Lmin, if 0 &le; L &lt; Lmin, where Lmin is a parameter with default value Modelica.Constants.eps.</p>
<p>Besides the Lmin parameter the inductor model has got the two parameters IC and UIC that belong together. With the IC parameter the user can specify an initial value of the current that flows through the inductor.</p>
<p><br/>Hence the inductor has an initial current at the beginning of the simulation. The other parameter UIC is of type Boolean. If UIC is true, the simulation tool uses</p>
<p><br/>the IC value at the initial calculation by adding the equation i= IC. If UIC is false, the IC value can be used (but it does not need to!) to calculate the initial values in order to simplify the numerical algorithms of initial calculation.</p>
</html>", revisions = "<html>
<ul>
<li><i>June 7, 2004   </i>
       by Christoph Clauss<br>changed, docu added<br>
       </li>
<li><i>April 30, 2004</i>
       by Anton Haumer<br>implemented.
       </li>
</ul>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-90, 0}, {-60, 0}}, color = {0, 0, 255}), Line(points = {{60, 0}, {90, 0}}, color = {0, 0, 255}), Line(points = {{0, 90}, {0, 8}}, color = {0, 0, 255}), Ellipse(extent = {{-60, -15}, {-30, 15}}, lineColor = {0, 0, 255}), Ellipse(extent = {{-30, -15}, {0, 15}}, lineColor = {0, 0, 255}), Ellipse(extent = {{0, -15}, {30, 15}}, lineColor = {0, 0, 255}), Ellipse(extent = {{30, -15}, {60, 15}}, lineColor = {0, 0, 255}), Rectangle(extent = {{-60, -30}, {60, 0}}, lineColor = {255, 255, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-146, -47}, {154, -87}}, textString = "%name", lineColor = {0, 0, 255})}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-96, 0}, {-60, 0}}, color = {0, 0, 255}), Line(points = {{0, 90}, {0, 8}}, color = {0, 0, 255}), Ellipse(extent = {{-60, -15}, {-30, 15}}, lineColor = {0, 0, 255}), Ellipse(extent = {{-30, -15}, {0, 15}}, lineColor = {0, 0, 255}), Ellipse(extent = {{0, -15}, {30, 15}}, lineColor = {0, 0, 255}), Ellipse(extent = {{30, -15}, {60, 15}}, lineColor = {0, 0, 255}), Rectangle(extent = {{-60, -30}, {62, 0}}, lineColor = {255, 255, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{60, 0}, {96, 0}}, color = {0, 0, 255})}));
        end VariableInductor;
        annotation(
          Documentation(info = "<html>
<p>This package contains very basic analog electrical components such as resistor, conductor, condensator, inductor, and the ground (which is needed in each electrical circuit description. Furthermore, controlled sources, coupling components, and some improved (but nevertheless basic) are in this package.</p>
</html>", revisions = "<html>
<dl>
<dt>
<b>Main Authors:</b>
</dt>
<dd>
Christoph Clau&szlig;
    &lt;<a href=\"mailto:Christoph.Clauss@eas.iis.fraunhofer.de\">Christoph.Clauss@eas.iis.fraunhofer.de</a>&gt;<br>
    Andr&eacute; Schneider
    &lt;<a href=\"mailto:Andre.Schneider@eas.iis.fraunhofer.de\">Andre.Schneider@eas.iis.fraunhofer.de</a>&gt;<br>
    Fraunhofer Institute for Integrated Circuits<br>
    Design Automation Department<br>
    Zeunerstra&szlig;e 38<br>
    D-01069 Dresden<br>
</dd>
</dl>
</html>"),
          Icon(graphics = {Line(points = {{-12, 60}, {-12, -60}}, color = {0, 0, 0}), Line(points = {{-80, 0}, {-12, 0}}, color = {0, 0, 0}), Line(points = {{12, 60}, {12, -60}}, color = {0, 0, 0}), Line(points = {{12, 0}, {80, 0}}, color = {0, 0, 0})}));
      end Basic;

      package Ideal "Ideal electrical elements such as switches, diode, transformer, operational amplifier"
        extends Modelica.Icons.Package;

        model IdealDiode "Ideal diode"
          extends Modelica.Electrical.Analog.Interfaces.OnePort;
          parameter Modelica.SIunits.Resistance Ron(final min = 0) = 1.E-5 "Forward state-on differential resistance (closed diode resistance)";
          parameter Modelica.SIunits.Conductance Goff(final min = 0) = 1.E-5 "Backward state-off conductance (opened diode conductance)";
          parameter Modelica.SIunits.Voltage Vknee(final min = 0) = 0 "Forward threshold voltage";
          extends Modelica.Electrical.Analog.Interfaces.ConditionalHeatPort;
          Boolean off(start = true) "Switching state";
        protected
          Real s(start = 0, final unit = "1") "Auxiliary variable for actual position on the ideal diode characteristic";
          /* s = 0: knee point
               s < 0: below knee point, diode conducting
               s > 0: above knee point, diode locking */
          constant Modelica.SIunits.Voltage unitVoltage = 1 annotation(
            HideResult = true);
          constant Modelica.SIunits.Current unitCurrent = 1 annotation(
            HideResult = true);
        equation
          off = s < 0;
          v = s * unitCurrent * (if off then 1 else Ron) + Vknee;
          i = s * unitVoltage * (if off then Goff else 1) + Goff * Vknee;
          LossPower = v * i;
          annotation(
            Documentation(info = "<html>
<P>
This is an ideal switch which is<br><br>
<b>open </b>(off), if it is reversed biased (voltage drop less than 0)<br>
<b>closed</b> (on), if it is conducting (current > 0).<br>
<br/>
This is the behaviour if all parameters are exactly zero.<br><br>
Note, there are circuits, where this ideal description
with zero resistance and zero conductance is not possible.
In order to prevent singularities during switching, the opened
diode has a small conductance <i>Gon</i> and the closed diode has a low
resistance <i>Roff</i> which is default.
</P>
<P>
The parameter <i>Vknee</i> which is the forward threshold voltage, allows to displace
the knee point <br>
along  the <i>Gon</i>-characteristic until <i>v = Vknee</i>.
<br> <br>
<b>Please note:</b>
In case of useHeatPort=true the temperature dependence of the electrical
behavior is <b> not </b> modelled.
</p>
</html>", revisions = "<html>
<ul>
<li><i> March 11, 2009   </i>
       by Christoph Clauss<br> conditional heat port added<br>
       </li>
<li><i>Mai 7, 2004   </i>
       by Christoph Clauss and Anton Haumer<br> Vknee added<br>
       </li>
<li><i>some years ago   </i>
       by Christoph Clauss<br> realized<br>
       </li>
</ul>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{30, 0}, {-30, 40}, {-30, -40}, {30, 0}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{-90, 0}, {40, 0}}, color = {0, 0, 255}), Line(points = {{40, 0}, {90, 0}}, color = {0, 0, 255}), Line(points = {{30, 40}, {30, -40}}, color = {0, 0, 255}), Line(visible = useHeatPort, points = {{0, -100}, {0, -20}}, color = {127, 0, 0}, smooth = Smooth.None, pattern = LinePattern.Dot), Text(extent = {{-156, 101}, {144, 61}}, textString = "%name", lineColor = {0, 0, 255})}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-80, 0}, {80, 0}}, color = {128, 128, 128}), Polygon(points = {{70, 4}, {80, 0}, {70, -4}, {70, 4}}, lineColor = {128, 128, 128}, fillColor = {128, 128, 128}, fillPattern = FillPattern.Solid), Line(points = {{0, 80}, {0, -80}}, color = {128, 128, 128}), Polygon(points = {{-4, 70}, {0, 80}, {4, 70}, {-4, 70}}, lineColor = {128, 128, 128}, fillColor = {128, 128, 128}, fillPattern = FillPattern.Solid), Text(extent = {{10, 80}, {20, 70}}, lineColor = {128, 128, 128}, textString = "i"), Text(extent = {{70, -10}, {80, -20}}, lineColor = {128, 128, 128}, textString = "v"), Line(points = {{-80, -40}, {-20, -10}, {20, 10}, {40, 70}}, color = {0, 0, 0}, thickness = 0.5), Line(points = {{20, 9}, {20, 0}}, color = {128, 128, 128}, pattern = LinePattern.Dot), Text(extent = {{20, 0}, {40, -10}}, lineColor = {128, 128, 128}, pattern = LinePattern.Dot, textString = "Vknee"), Text(extent = {{20, 70}, {40, 60}}, lineColor = {128, 128, 128}, pattern = LinePattern.Dot, textString = "Ron"), Text(extent = {{-20, 10}, {0, 0}}, lineColor = {128, 128, 128}, pattern = LinePattern.Dot, textString = "Goff"), Ellipse(extent = {{18, 12}, {22, 8}}, pattern = LinePattern.Dot, lineColor = {0, 0, 255})}));
        end IdealDiode;

        model IdealThyristor "Ideal thyristor"
          extends Modelica.Electrical.Analog.Interfaces.OnePort;
          parameter Modelica.SIunits.Resistance Ron(final min = 0) = 1.E-5 "Closed thyristor resistance";
          parameter Modelica.SIunits.Conductance Goff(final min = 0) = 1.E-5 "Opened thyristor conductance";
          parameter Modelica.SIunits.Voltage Vknee(final min = 0) = 0 "Forward threshold voltage";
          extends Modelica.Electrical.Analog.Interfaces.ConditionalHeatPort(final T = 293.15);
          Boolean off(start = true) "Switching state";
          Modelica.Blocks.Interfaces.BooleanInput fire annotation(
            Placement(transformation(origin = {70, 110}, extent = {{-20, -20}, {20, 20}}, rotation = 270)));
        protected
          Real s(final unit = "1") "Auxiliary variable: if on then current, if opened then voltage";
          constant Modelica.SIunits.Voltage unitVoltage = 1 annotation(
            HideResult = true);
          constant Modelica.SIunits.Current unitCurrent = 1 annotation(
            HideResult = true);
        equation
          off = s < 0 or pre(off) and not fire;
          v = s * unitCurrent * (if off then 1 else Ron) + Vknee;
          i = s * unitVoltage * (if off then Goff else 1) + Goff * Vknee;
          LossPower = v * i;
          annotation(
            Documentation(info = "<html>
<p>This is an ideal thyristor model which is<br><br>
<b>open </b>(off), if the voltage drop is less than 0 or both the thyristor already open (off = true) and fire is false<br>
<b>closed</b> (not off), if both the voltage drop is greater or equal 0 and either the thyristor was already closed (off=false) or fire is true.
</P>
<p>
This is the behaviour if all parameters are exactly zero.<br><br>
Note, there are circuits, where this ideal description
with zero resistance and zero conductance is not possible.
In order to prevent singularities during switching, the opened
thyristor has a small conductance <i>Goff</i> and the closed thyristor has a low
resistance <i>Ron</i> which is default.
</P>
<P>
The parameter <i>Vknee</i> which is the forward threshold voltage, allows to displace
the knee point <br>
along  the <i>Goff</i>-characteristic until <i>v = Vknee</i>.
<br> <br>
<b>Please note:</b>
In case of useHeatPort=true the temperature dependence of the electrical
behavior is <b> not </b> modelled. The parameters are not temperature dependent.
</p>
</html>", revisions = "<html>
<ul>
<li><i> March 11, 2009   </i>
       by Christoph Clauss<br> conditional heat port added<br>
       </li>
<li><i>Mai 7, 2004   </i>
       by Christoph Clauss and Anton Haumer<br> Vknee added<br>
       </li>
<li><i>some years ago   </i>
       by Christoph Clauss<br> realized<br>
       </li>
</ul>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-150, -47}, {150, -87}}, textString = "%name", lineColor = {0, 0, 255}), Polygon(points = {{30, 0}, {-30, 40}, {-30, -40}, {30, 0}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{-90, 0}, {40, 0}}, color = {0, 0, 255}), Line(points = {{40, 0}, {90, 0}}, color = {0, 0, 255}), Line(points = {{30, 40}, {30, -40}}, color = {0, 0, 255}), Line(points = {{30, 20}, {70, 60}, {70, 90}}, color = {0, 0, 255}), Line(points = {{40, 50}, {60, 30}}, color = {0, 0, 255}), Line(visible = useHeatPort, points = {{0, -100}, {0, -20}}, color = {127, 0, 0}, smooth = Smooth.None, pattern = LinePattern.Dot)}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-80, 0}, {80, 0}}, color = {128, 128, 128}), Polygon(points = {{70, 4}, {80, 0}, {70, -4}, {70, 4}}, lineColor = {128, 128, 128}, fillColor = {128, 128, 128}, fillPattern = FillPattern.Solid), Line(points = {{0, 80}, {0, -80}}, color = {128, 128, 128}), Polygon(points = {{-4, 70}, {0, 80}, {4, 70}, {-4, 70}}, lineColor = {128, 128, 128}, fillColor = {128, 128, 128}, fillPattern = FillPattern.Solid), Text(extent = {{10, 80}, {20, 70}}, lineColor = {128, 128, 128}, textString = "i"), Text(extent = {{70, -10}, {80, -20}}, lineColor = {128, 128, 128}, textString = "v"), Line(points = {{-80, -40}, {-20, -10}, {20, 10}, {40, 70}}, color = {0, 0, 0}, thickness = 0.5), Line(points = {{20, 9}, {20, 0}}, color = {128, 128, 128}, pattern = LinePattern.Dot), Text(extent = {{20, 0}, {40, -10}}, lineColor = {128, 128, 128}, pattern = LinePattern.Dot, textString = "Vknee"), Text(extent = {{20, 70}, {40, 60}}, lineColor = {128, 128, 128}, pattern = LinePattern.Dot, textString = "Ron"), Text(extent = {{-20, 10}, {0, 0}}, lineColor = {128, 128, 128}, pattern = LinePattern.Dot, textString = "Goff"), Ellipse(extent = {{18, 12}, {22, 8}}, pattern = LinePattern.Dot, lineColor = {0, 0, 255}), Line(points = {{20, 10}, {70, 40}}, color = {0, 0, 0}, thickness = 0.5)}));
        end IdealThyristor;

        model IdealGTOThyristor "Ideal GTO thyristor"
          extends Modelica.Electrical.Analog.Interfaces.OnePort;
          parameter Modelica.SIunits.Resistance Ron(final min = 0) = 1.E-5 "Closed thyristor resistance";
          parameter Modelica.SIunits.Conductance Goff(final min = 0) = 1.E-5 "Opened thyristor conductance";
          parameter Modelica.SIunits.Voltage Vknee(final min = 0) = 0 "Forward threshold voltage";
          extends Modelica.Electrical.Analog.Interfaces.ConditionalHeatPort(final T = 293.15);
          Boolean off(start = true) "Switching state";
          Modelica.Blocks.Interfaces.BooleanInput fire annotation(
            Placement(transformation(origin = {70, 110}, extent = {{-20, -20}, {20, 20}}, rotation = 270)));
        protected
          Real s(final unit = "1") "Auxiliary variable: if on then current, if opened then voltage";
          constant Modelica.SIunits.Voltage unitVoltage = 1 annotation(
            HideResult = true);
          constant Modelica.SIunits.Current unitCurrent = 1 annotation(
            HideResult = true);
        equation
          off = s < 0 or not fire;
          v = s * unitCurrent * (if off then 1 else Ron) + Vknee;
          i = s * unitVoltage * (if off then Goff else 1) + Goff * Vknee;
          LossPower = v * i;
          annotation(
            Documentation(info = "<html>
<p>This is an ideal GTO thyristor model which is<br><br>
<b>open </b>(off), if the voltage drop is less than 0 or fire is false<br>
<b>closed</b> (not off), if the voltage drop is greater or equal 0  and fire is true.
</P>
<p>
This is the behaviour if all parameters are exactly zero.<br><br>
Note, there are circuits, where this ideal description
with zero resistance and zero conductance is not possible.
In order to prevent singularities during switching, the opened
thyristor has a small conductance <i>Goff</i> and the closed thyristor has a low
resistance <i>Ron</i> which is default.
</P>
<P>
The parameter <i>Vknee</i> which is the forward threshold voltage, allows to displace
the knee point <br>
along  the <i>Goff</i>-characteristic until <i>v = Vknee</i>.
<br> <br>
<b>Please note:</b>
In case of useHeatPort=true the temperature dependence of the electrical
behavior is <b> not </b> modelled. The parameters are not temperature dependent.
</p>
</html>", revisions = "<html>
<ul>
<li><i> March 11, 2009   </i>
       by Christoph Clauss<br> conditional heat port added<br>
       </li>
<li><i>Mai 7, 2004   </i>
       by Christoph Clauss and Anton Haumer<br> Vknee added<br>
       </li>
<li><i>some years ago   </i>
       by Christoph Clauss<br> realized<br>
       </li>
</ul>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{30, 0}, {-30, 40}, {-30, -40}, {30, 0}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{-90, 0}, {40, 0}}, color = {0, 0, 255}), Line(points = {{40, 0}, {90, 0}}, color = {0, 0, 255}), Line(points = {{30, 40}, {30, -40}}, color = {0, 0, 255}), Line(points = {{30, 10}, {70, 50}, {70, 90}}, color = {0, 0, 255}), Line(points = {{50, 50}, {70, 30}}, color = {0, 0, 255}), Line(visible = useHeatPort, points = {{0, -100}, {0, -20}}, color = {127, 0, 0}, smooth = Smooth.None, pattern = LinePattern.Dot), Text(extent = {{-149, -43}, {151, -83}}, textString = "%name", lineColor = {0, 0, 255}), Line(points = {{30, 22}, {70, 62}}, color = {0, 0, 255}, smooth = Smooth.None), Polygon(points = {{44, 43}, {44, 36}, {51, 36}, {44, 43}}, lineColor = {0, 0, 255}, smooth = Smooth.None, fillPattern = FillPattern.Solid, fillColor = {0, 0, 255}), Polygon(points = {{46, 33}, {53, 33}, {53, 26}, {46, 33}}, lineColor = {0, 0, 255}, smooth = Smooth.None, fillPattern = FillPattern.Solid, fillColor = {0, 0, 255}), Text(extent = {{0, 62}, {0, 34}}, lineColor = {0, 0, 255}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid, textString = "GTO")}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-80, 0}, {80, 0}}, color = {128, 128, 128}), Polygon(points = {{70, 4}, {80, 0}, {70, -4}, {70, 4}}, lineColor = {128, 128, 128}, fillColor = {128, 128, 128}, fillPattern = FillPattern.Solid), Line(points = {{0, 80}, {0, -80}}, color = {128, 128, 128}), Polygon(points = {{-4, 70}, {0, 80}, {4, 70}, {-4, 70}}, lineColor = {128, 128, 128}, fillColor = {128, 128, 128}, fillPattern = FillPattern.Solid), Text(extent = {{10, 80}, {20, 70}}, lineColor = {128, 128, 128}, textString = "i"), Text(extent = {{70, -10}, {80, -20}}, lineColor = {128, 128, 128}, textString = "v"), Line(points = {{-80, -40}, {-20, -10}, {20, 10}, {40, 70}}, color = {0, 0, 0}, thickness = 0.5), Line(points = {{20, 9}, {20, 0}}, color = {128, 128, 128}, pattern = LinePattern.Dot), Text(extent = {{20, 0}, {40, -10}}, lineColor = {128, 128, 128}, pattern = LinePattern.Dot, textString = "Vknee"), Text(extent = {{20, 70}, {40, 60}}, lineColor = {128, 128, 128}, pattern = LinePattern.Dot, textString = "Ron"), Text(extent = {{-20, 10}, {0, 0}}, lineColor = {128, 128, 128}, pattern = LinePattern.Dot, textString = "Goff"), Ellipse(extent = {{18, 12}, {22, 8}}, pattern = LinePattern.Dot, lineColor = {0, 0, 255}), Line(points = {{20, 10}, {70, 40}}, color = {0, 0, 0}, thickness = 0.5)}));
        end IdealGTOThyristor;

        model IdealCommutingSwitch "Ideal commuting switch"
          parameter SI.Resistance Ron(final min = 0) = 1.E-5 "Closed switch resistance";
          parameter SI.Conductance Goff(final min = 0) = 1.E-5 "Opened switch conductance";
          extends Modelica.Electrical.Analog.Interfaces.ConditionalHeatPort(final T = 293.15);
          Interfaces.PositivePin p annotation(
            Placement(transformation(extent = {{-110, -10}, {-90, 10}}, rotation = 0)));
          Interfaces.NegativePin n2 annotation(
            Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
          Interfaces.NegativePin n1 annotation(
            Placement(transformation(extent = {{90, 40}, {110, 60}}, rotation = 0)));
          Modelica.Blocks.Interfaces.BooleanInput control "true => p--n2 connected, false => p--n1 connected" annotation(
            Placement(transformation(origin = {0, 80}, extent = {{-20, -20}, {20, 20}}, rotation = 270)));
        protected
          Real s1(final unit = "1");
          Real s2(final unit = "1") "Auxiliary variables";
          constant Modelica.SIunits.Voltage unitVoltage = 1 annotation(
            HideResult = true);
          constant Modelica.SIunits.Current unitCurrent = 1 annotation(
            HideResult = true);
        equation
          0 = p.i + n2.i + n1.i;
          p.v - n1.v = s1 * unitCurrent * (if control then 1 else Ron);
          n1.i = -s1 * unitVoltage * (if control then Goff else 1);
          p.v - n2.v = s2 * unitCurrent * (if control then Ron else 1);
          n2.i = -s2 * unitVoltage * (if control then 1 else Goff);
          LossPower = p.i * p.v + n1.i * n1.v + n2.i * n2.v;
          annotation(
            Documentation(info = "<HTML>
<P>
The commuting switch has a positive pin p and two negative pins n1 and n2.
The switching behaviour is controlled
by the input signal control. If control is true, the pin p is connected
with the negative pin n2. Otherwise, the pin p is connected to the negative pin n1.
</P>
<P>
In order to prevent singularities during switching, the opened
switch has a (very low) conductance Goff
and the closed switch has a (very low) resistance Ron.
The limiting case is also allowed, i.e., the resistance Ron of the
closed switch could be exactly zero and the conductance Goff of the
open switch could be also exactly zero. Note, there are circuits,
where a description with zero Ron or zero Goff is not possible.
<br> <br>
<b>Please note:</b>
In case of useHeatPort=true the temperature dependence of the electrical
behavior is <b> not </b> modelled. The parameters are not temperature dependent.
</P>
</html>", revisions = "<html>
<ul>
<li><i> March 11, 2009   </i>
       by Christoph Clauss<br> conditional heat port added<br>
       </li>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-44, 4}, {-36, -4}}, lineColor = {0, 0, 255}), Text(extent = {{-80, 0}, {-60, 22}}, textString = "p", lineColor = {0, 0, 255}), Text(extent = {{60, 50}, {80, 72}}, textString = "n1", lineColor = {0, 0, 255}), Text(extent = {{60, 0}, {80, 22}}, textString = "n2", lineColor = {0, 0, 255}), Line(points = {{-90, 0}, {-44, 0}}, color = {0, 0, 255}), Line(points = {{-37, 2}, {40, 50}}, color = {0, 0, 255}), Line(points = {{40, 50}, {90, 50}}, color = {0, 0, 255}), Line(points = {{0, 90}, {0, 25}}, color = {0, 0, 255}), Line(points = {{40, 0}, {90, 0}}, color = {0, 0, 255}), Line(visible = useHeatPort, points = {{0, -100}, {0, 25}}, color = {127, 0, 0}, smooth = Smooth.None, pattern = LinePattern.Dot), Text(extent = {{-148, -22}, {152, -62}}, textString = "%name", lineColor = {0, 0, 255})}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-44, 4}, {-36, -4}}, lineColor = {0, 0, 255}), Line(points = {{-96, 0}, {-44, 0}}, color = {0, 0, 255}), Line(points = {{-37, 2}, {40, 50}}, color = {0, 0, 255}), Line(points = {{40, 50}, {96, 50}}, color = {0, 0, 255}), Line(points = {{0, 60}, {0, 25}}, color = {0, 0, 255}), Line(points = {{40, 0}, {96, 0}}, color = {0, 0, 255})}));
        end IdealCommutingSwitch;

        model IdealIntermediateSwitch "Ideal intermediate switch"
          parameter SI.Resistance Ron(final min = 0) = 1.E-5 "Closed switch resistance";
          parameter SI.Conductance Goff(final min = 0) = 1.E-5 "Opened switch conductance";
          extends Modelica.Electrical.Analog.Interfaces.ConditionalHeatPort(final T = 293.15);
          Interfaces.PositivePin p1 annotation(
            Placement(transformation(extent = {{-110, 40}, {-90, 60}}, rotation = 0)));
          Interfaces.PositivePin p2 annotation(
            Placement(transformation(extent = {{-110, -10}, {-90, 10}}, rotation = 0)));
          Interfaces.NegativePin n1 annotation(
            Placement(transformation(extent = {{90, 40}, {110, 60}}, rotation = 0)));
          Interfaces.NegativePin n2 annotation(
            Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
          Modelica.Blocks.Interfaces.BooleanInput control "true => p1--n2, p2--n1 connected, otherwise p1--n1, p2--n2  connected" annotation(
            Placement(transformation(origin = {0, 80}, extent = {{-20, -20}, {20, 20}}, rotation = 270)));
        protected
          Real s1(final unit = "1");
          Real s2(final unit = "1");
          Real s3(final unit = "1");
          Real s4(final unit = "1") "Auxiliary variables";
          constant Modelica.SIunits.Voltage unitVoltage = 1 annotation(
            HideResult = true);
          constant Modelica.SIunits.Current unitCurrent = 1 annotation(
            HideResult = true);
        equation
          p1.v - n1.v = s1 * unitCurrent * (if control then 1 else Ron);
          p2.v - n2.v = s2 * unitCurrent * (if control then 1 else Ron);
          p1.v - n2.v = s3 * unitCurrent * (if control then Ron else 1);
          p2.v - n1.v = s4 * unitCurrent * (if control then Ron else 1);
          p1.i = if control then s1 * unitVoltage * Goff + s3 * unitCurrent else s1 * unitCurrent + s3 * unitVoltage * Goff;
          p2.i = if control then s2 * unitVoltage * Goff + s4 * unitCurrent else s2 * unitCurrent + s4 * unitVoltage * Goff;
          n1.i = if control then (-s1 * unitVoltage * Goff) - s4 * unitCurrent else (-s1 * unitCurrent) - s4 * unitVoltage * Goff;
          n2.i = if control then (-s2 * unitVoltage * Goff) - s3 * unitCurrent else (-s2 * unitCurrent) - s3 * unitVoltage * Goff;
          LossPower = p1.i * p1.v + p2.i * p2.v + n1.i * n1.v + n2.i * n2.v;
          annotation(
            Documentation(info = "<html>
<p>The intermediate switch has four switching contact pins p1, p2, n1, and n2. The switching behaviour is controlled by the input signal control. If control is true, the pin p1 is connected to the pin n2, and the pin p2 is connected to the pin n1. Otherwise,if control is false, the pin p1 is connected to n1, and the pin p2 is connected to n2.</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Electrical/Analog/IdealIntermediateSwitch1.png\"
     alt=\"IdealIntermediateSwitch1.png\">
</p>

<p>In order to prevent singularities during switching, the opened switch has a (very low) conductance Goff and the closed switch has a (very low) resistance Ron.</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Electrical/Analog/IdealIntermediateSwitch2.png\"
     alt=\"IdealIntermediateSwitch2.png\">
</p>

<p>The limiting case is also allowed, i.e., the resistance Ron of the closed switch could be exactly zero and the conductance Goff of the open switch could be also exactly zero. Note, there are circuits, where a description with zero Ron or zero Goff is not possible. </p>
<p><b>Please note:</b> In case of useHeatPort=true the temperature dependence of the electrical behavior is <b>not </b>modelled. The parameters are not temperature dependent. </p>
</html>", revisions = "<html>
<ul>
<li><i> March 11, 2009   </i>
       by Christoph Clauss<br> conditional heat port added<br>
       </li>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-4, 30}, {4, 22}}, lineColor = {0, 0, 255}), Text(extent = {{-80, 50}, {-60, 72}}, textString = "p1", lineColor = {0, 0, 255}), Text(extent = {{-80, 0}, {-60, 22}}, textString = "p2", lineColor = {0, 0, 255}), Text(extent = {{60, 50}, {80, 72}}, textString = "n1", lineColor = {0, 0, 255}), Text(extent = {{60, 0}, {80, 22}}, textString = "n2", lineColor = {0, 0, 255}), Line(points = {{-90, 0}, {-44, 0}}, color = {0, 0, 255}), Line(points = {{-90, 50}, {-44, 50}}, color = {0, 0, 255}), Line(points = {{-44, 0}, {40, 50}}, color = {0, 0, 255}), Line(points = {{-44, 50}, {40, 0}}, color = {0, 0, 255}), Line(points = {{40, 50}, {90, 50}}, color = {0, 0, 255}), Line(points = {{0, 90}, {0, 25}}, color = {0, 0, 255}), Line(points = {{40, 0}, {90, 0}}, color = {0, 0, 255}), Text(extent = {{-151, -24}, {149, -64}}, textString = "%name", lineColor = {0, 0, 255})}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-4, 29}, {4, 21}}, lineColor = {0, 0, 255}), Line(points = {{-96, 0}, {-40, 0}}, color = {0, 0, 255}), Line(points = {{-96, 50}, {-40, 50}}, color = {0, 0, 255}), Line(points = {{-40, 0}, {40, 50}}, color = {0, 0, 255}), Line(points = {{-40, 50}, {40, 0}}, color = {0, 0, 255}), Line(points = {{40, 50}, {96, 50}}, color = {0, 0, 255}), Line(points = {{0, 60}, {0, 25}}, color = {0, 0, 255}), Line(points = {{40, 0}, {96, 0}}, color = {0, 0, 255})}));
        end IdealIntermediateSwitch;

        model ControlledIdealCommutingSwitch "Controlled ideal commuting switch"
          parameter SI.Voltage level = 0.5 "Switch level";
          parameter SI.Resistance Ron(final min = 0) = 1.E-5 "Closed switch resistance";
          parameter SI.Conductance Goff(final min = 0) = 1.E-5 "Opened switch conductance";
          extends Modelica.Electrical.Analog.Interfaces.ConditionalHeatPort(final T = 293.15);
          Interfaces.PositivePin p annotation(
            Placement(transformation(extent = {{-110, -10}, {-90, 10}}, rotation = 0)));
          Interfaces.NegativePin n2 annotation(
            Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
          Interfaces.NegativePin n1 annotation(
            Placement(transformation(extent = {{90, 40}, {110, 60}}, rotation = 0)));
          Interfaces.Pin control "Control pin: if control.v > level p--n2 connected, otherwise p--n1 connected" annotation(
            Placement(transformation(origin = {0, 100}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
        protected
          Real s1(final unit = "1");
          Real s2(final unit = "1") "Auxiliary variables";
          constant Modelica.SIunits.Voltage unitVoltage = 1 annotation(
            HideResult = true);
          constant Modelica.SIunits.Current unitCurrent = 1 annotation(
            HideResult = true);
        equation
          control.i = 0;
          0 = p.i + n2.i + n1.i;
          p.v - n1.v = s1 * unitCurrent * (if control.v > level then 1 else Ron);
          n1.i = -s1 * unitVoltage * (if control.v > level then Goff else 1);
          p.v - n2.v = s2 * unitCurrent * (if control.v > level then Ron else 1);
          n2.i = -s2 * unitVoltage * (if control.v > level then 1 else Goff);
          LossPower = p.i * p.v + n1.i * n1.v + n2.i * n2.v;
          annotation(
            Documentation(info = "<HTML>
<P>
The commuting switch has a positive pin p and two negative pins n1 and n2.
The switching behaviour is controlled
by the control pin. If its voltage exceeds the value of the parameter level,
the pin p is connected with the negative pin n2. Otherwise, the pin p is
connected the negative pin n1.
</P>
<P>
In order to prevent singularities during switching, the opened
switch has a (very low) conductance Goff
and the closed switch has a (very low) resistance Ron.
The limiting case is also allowed, i.e., the resistance Ron of the
closed switch could be exactly zero and the conductance Goff of the
open switch could be also exactly zero. Note, there are circuits,
where a description with zero Ron or zero Goff is not possible.
<br> <br>
<b>Please note:</b>
In case of useHeatPort=true the temperature dependence of the electrical
behavior is <b> not </b> modelled. The parameters are not temperature dependent.
</P>
</html>", revisions = "<html>
<ul>
<li><i> March 11, 2009   </i>
       by Christoph Clauss<br> conditional heat port added<br>
       </li>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-44, 4}, {-36, -4}}, lineColor = {0, 0, 255}), Text(extent = {{-80, 0}, {-60, 22}}, textString = "p", lineColor = {0, 0, 255}), Text(extent = {{60, 50}, {80, 72}}, textString = "n1", lineColor = {0, 0, 255}), Text(extent = {{60, 0}, {80, 22}}, textString = "n2", lineColor = {0, 0, 255}), Line(points = {{-90, 0}, {-44, 0}}, color = {0, 0, 255}), Line(points = {{-37, 2}, {40, 50}}, color = {0, 0, 255}), Line(points = {{40, 50}, {90, 50}}, color = {0, 0, 255}), Line(points = {{0, 90}, {0, 25}}, color = {0, 0, 255}), Line(points = {{40, 0}, {90, 0}}, color = {0, 0, 255}), Line(visible = useHeatPort, points = {{0, -99}, {0, 25}}, color = {127, 0, 0}, smooth = Smooth.None, pattern = LinePattern.Dot), Text(extent = {{-145, -21}, {155, -61}}, textString = "%name", lineColor = {0, 0, 255})}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-44, 4}, {-36, -4}}, lineColor = {0, 0, 255}), Line(points = {{-96, 0}, {-44, 0}}, color = {0, 0, 255}), Line(points = {{-37, 2}, {40, 50}}, color = {0, 0, 255}), Line(points = {{40, 50}, {96, 50}}, color = {0, 0, 255}), Line(points = {{0, 96}, {0, 25}}, color = {0, 0, 255}), Line(points = {{40, 0}, {96, 0}}, color = {0, 0, 255})}));
        end ControlledIdealCommutingSwitch;

        model ControlledIdealIntermediateSwitch "Controlled ideal intermediate switch"
          parameter SI.Voltage level = 0.5 "Switch level";
          parameter SI.Resistance Ron(final min = 0) = 1.E-5 "Closed switch resistance";
          parameter SI.Conductance Goff(final min = 0) = 1.E-5 "Opened switch conductance";
          extends Modelica.Electrical.Analog.Interfaces.ConditionalHeatPort(final T = 293.15);
          Interfaces.PositivePin p1 annotation(
            Placement(transformation(extent = {{-110, 40}, {-90, 60}}, rotation = 0)));
          Interfaces.PositivePin p2 annotation(
            Placement(transformation(extent = {{-110, -10}, {-90, 10}}, rotation = 0)));
          Interfaces.NegativePin n1 annotation(
            Placement(transformation(extent = {{90, 40}, {110, 60}}, rotation = 0)));
          Interfaces.NegativePin n2 annotation(
            Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
          Interfaces.Pin control "Control pin: if control.v > level p1--n2, p2--n1 connected,
                   otherwise p1--n1, p2--n2  connected" annotation(
            Placement(transformation(origin = {0, 100}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
        protected
          Real s1(final unit = "1");
          Real s2(final unit = "1");
          Real s3(final unit = "1");
          Real s4(final unit = "1") "Auxiliary variables";
          constant Modelica.SIunits.Voltage unitVoltage = 1 annotation(
            HideResult = true);
          constant Modelica.SIunits.Current unitCurrent = 1 annotation(
            HideResult = true);
        equation
          control.i = 0;
          p1.v - n1.v = s1 * unitCurrent * (if control.v > level then 1 else Ron);
          p2.v - n2.v = s2 * unitCurrent * (if control.v > level then 1 else Ron);
          p1.v - n2.v = s3 * unitCurrent * (if control.v > level then Ron else 1);
          p2.v - n1.v = s4 * unitCurrent * (if control.v > level then Ron else 1);
          p1.i = if control.v > level then s1 * unitVoltage * Goff + s3 * unitCurrent else s1 * unitCurrent + s3 * unitVoltage * Goff;
          p2.i = if control.v > level then s2 * unitVoltage * Goff + s4 * unitCurrent else s2 * unitCurrent + s4 * unitVoltage * Goff;
          n1.i = if control.v > level then (-s1 * unitVoltage * Goff) - s4 * unitCurrent else (-s1 * unitCurrent) - s4 * unitVoltage * Goff;
          n2.i = if control.v > level then (-s2 * unitVoltage * Goff) - s3 * unitCurrent else (-s2 * unitCurrent) - s3 * unitVoltage * Goff;
          LossPower = p1.i * p1.v + p2.i * p2.v + n1.i * n1.v + n2.i * n2.v;
          annotation(
            Documentation(info = "<html>
<p>The intermediate switch has four switching contact pins p1, p2, n1, and n2. The switching behaviour is controlled by the control pin. If its voltage exceeds the value of the parameter level, the pin p1 is connected to pin n2, and the pin p2 is connected to the pin n1. Otherwise, the pin p1 is connected to the pin n1, and the pin p2 is connected to the pin n2.
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Electrical/Analog/ControlledIdealIntermediateSwitch1.png\"
     alt=\"ControlledIdealIntermediateSwitch1.png\">
</p>

<p>
In order to prevent singularities during switching, the opened switch has a (very low) conductance Goff and the closed switch has a (very low) resistance Ron.
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Electrical/Analog/ControlledIdealIntermediateSwitch2.png\"
     alt=\"ControlledIdealIntermediateSwitch2.png\">
</p>

<p>
The limiting case is also allowed, i.e., the resistance Ron of the closed switch could be exactly zero and the conductance Goff of the open switch could be also exactly zero. Note, there are circuits, where a description with zero Ron or zero Goff is not possible. </p>
<p><br><b>Please note:</b> In case of useHeatPort=true the temperature dependence of the electrical behavior is <b>not </b>modelled. The parameters are not temperature dependent. </p>
</html>", revisions = "<html>
<ul>
<li><i> March 11, 2009   </i>
       by Christoph Clauss<br> conditional heat port added<br>
       </li>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-4, 30}, {4, 22}}, lineColor = {0, 0, 255}), Text(extent = {{-80, 50}, {-60, 72}}, textString = "p1", lineColor = {0, 0, 255}), Text(extent = {{-80, 0}, {-60, 22}}, textString = "p2", lineColor = {0, 0, 255}), Text(extent = {{60, 50}, {80, 72}}, textString = "n1", lineColor = {0, 0, 255}), Text(extent = {{60, 0}, {80, 22}}, textString = "n2", lineColor = {0, 0, 255}), Line(points = {{-90, 0}, {-40, 0}}, color = {0, 0, 255}), Line(points = {{-90, 50}, {-39, 50}}, color = {0, 0, 255}), Line(points = {{-40, 0}, {40, 50}}, color = {0, 0, 255}), Line(points = {{-40, 50}, {40, 0}}, color = {0, 0, 255}), Line(points = {{40, 50}, {90, 50}}, color = {0, 0, 255}), Line(points = {{0, 90}, {0, 25}}, color = {0, 0, 255}), Line(points = {{40, 0}, {90, 0}}, color = {0, 0, 255}), Line(visible = useHeatPort, points = {{0, -100}, {0, 22}}, color = {127, 0, 0}, smooth = Smooth.None, pattern = LinePattern.Dot), Text(extent = {{-150, -23}, {150, -63}}, textString = "%name", lineColor = {0, 0, 255})}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-4, 29}, {4, 21}}, lineColor = {0, 0, 255}), Line(points = {{-96, 0}, {-40, 0}}, color = {0, 0, 255}), Line(points = {{-96, 50}, {-40, 50}}, color = {0, 0, 255}), Line(points = {{-40, 0}, {40, 50}}, color = {0, 0, 255}), Line(points = {{-40, 50}, {40, 0}}, color = {0, 0, 255}), Line(points = {{40, 50}, {96, 50}}, color = {0, 0, 255}), Line(points = {{0, 96}, {0, 25}}, color = {0, 0, 255}), Line(points = {{40, 0}, {96, 0}}, color = {0, 0, 255})}));
        end ControlledIdealIntermediateSwitch;

        model IdealOpAmp "Ideal operational amplifier (norator-nullator pair)"
          SI.Voltage v1 "Voltage drop over the left port";
          SI.Voltage v2 "Voltage drop over the right port";
          SI.Current i1 "Current flowing from pos. to neg. pin of the left port";
          SI.Current i2 "Current flowing from pos. to neg. pin of the right port";
          Interfaces.PositivePin p1 "Positive pin of the left port" annotation(
            Placement(transformation(extent = {{-110, -60}, {-90, -40}}, rotation = 0)));
          Interfaces.NegativePin n1 "Negative pin of the left port" annotation(
            Placement(transformation(extent = {{-110, 40}, {-90, 60}}, rotation = 0)));
          Interfaces.PositivePin p2 "Positive pin of the right port" annotation(
            Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
          Interfaces.NegativePin n2 "Negative pin of the right port" annotation(
            Placement(transformation(origin = {0, -100}, extent = {{10, -10}, {-10, 10}}, rotation = 270)));
        equation
          v1 = p1.v - n1.v;
          v2 = p2.v - n2.v;
          0 = p1.i + n1.i;
          0 = p2.i + n2.i;
          i1 = p1.i;
          i2 = p2.i;
          v1 = 0;
          i1 = 0;
          annotation(
            Documentation(info = "<html>
<P>
The ideal OpAmp is a two-port. The left port is fixed to <i>v1=0</i> and <i>i1=0</i>
(nullator). At the right port both any voltage <i>v2</i> and any current <i>i2</i>
are possible (norator).
</P>
</html>", revisions = "<html>
<ul>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{60, 0}, {-60, 70}, {-60, -70}, {60, 0}}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Line(points = {{-90, 50}, {-60, 50}}, color = {0, 0, 255}), Line(points = {{-90, -50}, {-60, -50}}, color = {0, 0, 255}), Line(points = {{60, 0}, {90, 0}}, color = {0, 0, 255}), Line(points = {{0, -35}, {0, -91}}, color = {0, 0, 255}), Line(points = {{-48, 32}, {-28, 32}}, color = {0, 0, 255}), Line(points = {{-39, -20}, {-39, -41}}, color = {0, 0, 255}), Line(points = {{-50, -31}, {-28, -31}}, color = {0, 0, 255}), Text(extent = {{-150, 126}, {150, 86}}, textString = "%name", lineColor = {0, 0, 255})}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{60, 0}, {-60, 70}, {-60, -70}, {60, 0}}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Line(points = {{-96, 50}, {-60, 50}}, color = {0, 0, 255}), Line(points = {{-96, -50}, {-60, -50}}, color = {0, 0, 255}), Line(points = {{60, 0}, {96, 0}}, color = {0, 0, 255}), Line(points = {{0, -35}, {0, -96}}, color = {0, 0, 255}), Line(points = {{-55, 50}, {-45, 50}}, color = {0, 0, 255}), Line(points = {{-50, -45}, {-50, -55}}, color = {0, 0, 255}), Line(points = {{-55, -50}, {-45, -50}}, color = {0, 0, 255}), Text(extent = {{-111, -39}, {-90, -19}}, lineColor = {160, 160, 164}, textString = "p1.i=0"), Polygon(points = {{120, 3}, {110, 0}, {120, -3}, {120, 3}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {160, 160, 164}), Line(points = {{111, 0}, {136, 0}}, color = {0, 0, 0}), Text(extent = {{118, 2}, {135, 17}}, lineColor = {0, 0, 0}, textString = "i2"), Text(extent = {{-111, 60}, {-90, 80}}, lineColor = {160, 160, 164}, textString = "n1.i=0"), Line(points = {{18, -111}, {18, -86}}, color = {160, 160, 164}), Polygon(points = {{21, -101}, {18, -111}, {15, -101}, {21, -101}}, lineColor = {160, 160, 164}, fillColor = {160, 160, 164}, fillPattern = FillPattern.Solid), Text(extent = {{22, -100}, {39, -85}}, lineColor = {0, 0, 0}, textString = "i2")}));
        end IdealOpAmp;

        model IdealOpAmp3Pin "Ideal operational amplifier (norator-nullator pair), but 3 pins"
          Interfaces.PositivePin in_p "Positive pin of the input port" annotation(
            Placement(transformation(extent = {{-110, -60}, {-90, -40}}, rotation = 0)));
          Interfaces.NegativePin in_n "Negative pin of the input port" annotation(
            Placement(transformation(extent = {{-110, 40}, {-90, 60}}, rotation = 0)));
          Interfaces.PositivePin out "Output pin" annotation(
            Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
        equation
          in_p.v = in_n.v;
          in_p.i = 0;
          in_n.i = 0;
          annotation(
            Documentation(info = "<html>
<P>
The ideal OpAmp with three pins is of exactly the same behaviour as the ideal
OpAmp with four pins. Only the negative output pin is left out.
Both the input voltage and current are fixed to zero (nullator).
At the output pin both any voltage <i>v2</i> and any current <i>i2</i>
are possible.
</P>
</html>", revisions = "<html>
<ul>
<li><i> 2002   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{60, 0}, {-60, 70}, {-60, -70}, {60, 0}}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Line(points = {{-90, 50}, {-60, 50}}, color = {0, 0, 255}), Line(points = {{-90, -50}, {-60, -50}}, color = {0, 0, 255}), Line(points = {{60, 0}, {90, 0}}, color = {0, 0, 255}), Line(points = {{-48, 32}, {-28, 32}}, color = {0, 0, 255}), Line(points = {{-39, -20}, {-39, -41}}, color = {0, 0, 255}), Line(points = {{-50, -31}, {-28, -31}}, color = {0, 0, 255}), Text(extent = {{-149, 117}, {151, 77}}, textString = "%name", lineColor = {0, 0, 255})}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{60, 0}, {-60, 70}, {-60, -70}, {60, 0}}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Line(points = {{-96, 50}, {-60, 50}}, color = {0, 0, 255}), Line(points = {{-96, -50}, {-60, -50}}, color = {0, 0, 255}), Line(points = {{60, 0}, {97, 0}}, color = {0, 0, 255}), Line(points = {{-55, 50}, {-45, 50}}, color = {0, 0, 255}), Line(points = {{-50, -45}, {-50, -55}}, color = {0, 0, 255}), Line(points = {{-55, -50}, {-45, -50}}, color = {0, 0, 255}), Text(extent = {{-111, -39}, {-90, -19}}, lineColor = {160, 160, 164}, textString = "p1.i=0"), Polygon(points = {{120, 3}, {110, 0}, {120, -3}, {120, 3}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {160, 160, 164}), Line(points = {{111, 0}, {136, 0}}, color = {0, 0, 0}), Text(extent = {{118, 2}, {135, 17}}, lineColor = {0, 0, 0}, textString = "i2"), Text(extent = {{-111, 60}, {-90, 80}}, lineColor = {160, 160, 164}, textString = "n1.i=0")}));
        end IdealOpAmp3Pin;

        model IdealOpAmpLimited "Ideal operational amplifier with limitation"
          Interfaces.PositivePin in_p "Positive pin of the input port" annotation(
            Placement(transformation(extent = {{-110, -60}, {-90, -40}}, rotation = 0)));
          Interfaces.NegativePin in_n "Negative pin of the input port" annotation(
            Placement(transformation(extent = {{-110, 40}, {-90, 60}}, rotation = 0)));
          Interfaces.PositivePin out "Output pin" annotation(
            Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
          Interfaces.PositivePin VMax "Positive output voltage limitation" annotation(
            Placement(transformation(extent = {{-10, 60}, {10, 80}}, rotation = 0)));
          Interfaces.NegativePin VMin "Negative output voltage limitation" annotation(
            Placement(transformation(extent = {{-10, -80}, {10, -60}}, rotation = 0)));
          SI.Voltage vin "input voltage";
        protected
          Real s(final unit = "1") "Auxiliary variable";
          constant Modelica.SIunits.Voltage unitVoltage = 1 annotation(
            HideResult = true);
        equation
          in_p.i = 0;
          in_n.i = 0;
          VMax.i = 0;
          VMin.i = 0;
          vin = in_p.v - in_n.v;
          in_p.v - in_n.v = unitVoltage * smooth(0, if s < (-1) then s + 1 else if s > 1 then s - 1 else 0);
          out.v = smooth(0, if s < (-1) then VMin.v else if s > 1 then VMax.v else (VMax.v - VMin.v) * s / 2 + (VMax.v + VMin.v) / 2);
          annotation(
            Documentation(info = "<HTML>
<P>
The ideal OpAmp with limitation behaves like an ideal OpAmp without limitation,
if the output voltage is within the limits VMin and VMax. In this case
the input voltage vin = in_p.v - in_n.v is zero.
If the input voltage vin less than 0, the output voltage is out.v = VMin.
If the input voltage is vin larger than 0, the output voltage is out.v = VMax.
</P>
</html>", revisions = "<html>
<ul>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{60, 0}, {-60, 70}, {-60, -70}, {60, 0}}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Line(points = {{-45, -10}, {-10, -10}, {-10, 10}, {20, 10}}, color = {0, 0, 255}), Line(points = {{0, 35}, {0, 80}}, color = {0, 0, 255}), Line(points = {{0, -35}, {0, -80}}, color = {0, 0, 255}), Line(points = {{-90, 50}, {-60, 50}}, color = {0, 0, 255}), Line(points = {{-90, -50}, {-60, -50}}, color = {0, 0, 255}), Line(points = {{60, 0}, {90, 0}}, color = {0, 0, 255}), Line(points = {{-48, 32}, {-28, 32}}, color = {0, 0, 255}), Line(points = {{-39, -20}, {-39, -41}}, color = {0, 0, 255}), Line(points = {{-50, -31}, {-28, -31}}, color = {0, 0, 255}), Text(extent = {{-152, 135}, {148, 95}}, textString = "%name", lineColor = {0, 0, 255})}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{60, 0}, {-60, 70}, {-60, -70}, {60, 0}}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Line(points = {{-45, -10}, {-10, -10}, {-10, 10}, {20, 10}}, color = {0, 0, 255}), Line(points = {{0, 35}, {0, 80}}, color = {0, 0, 255}), Line(points = {{0, -35}, {0, -80}}, color = {0, 0, 255}), Line(points = {{-96, 50}, {-60, 50}}, color = {0, 0, 255}), Line(points = {{-96, -50}, {-60, -50}}, color = {0, 0, 255}), Line(points = {{60, 0}, {96, 0}}, color = {0, 0, 255}), Line(points = {{-55, 50}, {-45, 50}}, color = {0, 0, 255}), Line(points = {{-50, -45}, {-50, -55}}, color = {0, 0, 255}), Line(points = {{-55, -50}, {-45, -50}}, color = {0, 0, 255}), Text(extent = {{-111, -39}, {-90, -19}}, lineColor = {160, 160, 164}, textString = "p1.i=0"), Polygon(points = {{120, 3}, {110, 0}, {120, -3}, {120, 3}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {160, 160, 164}), Line(points = {{111, 0}, {136, 0}}, color = {0, 0, 0}), Text(extent = {{118, 2}, {135, 17}}, lineColor = {0, 0, 0}, textString = "i2"), Text(extent = {{-111, 60}, {-90, 80}}, lineColor = {160, 160, 164}, textString = "n1.i=0")}));
        end IdealOpAmpLimited;

        model IdealTransformer "Ideal transformer core with or without magnetization"
          extends Modelica.Electrical.Analog.Interfaces.TwoPort;
          parameter Real n(start = 1) "Turns ratio primary:secondary voltage";
          parameter Boolean considerMagnetization = false "Choice of considering magnetization";
          parameter Modelica.SIunits.Inductance Lm1(start = 1) "Magnetization inductance w.r.t. primary side" annotation(
            Dialog(enable = considerMagnetization));
        protected
          Modelica.SIunits.Current im1 "Magnetization current w.r.t. primary side";
          Modelica.SIunits.MagneticFlux psim1 "Magnetic flux w.r.t. primary side";
        equation
          im1 = i1 + i2 / n;
          if considerMagnetization then
            psim1 = Lm1 * im1;
            v1 = der(psim1);
          else
            psim1 = 0;
            im1 = 0;
          end if;
          v1 = n * v2;
          annotation(
            Documentation(info = "<html>
<p>
The ideal transformer is a two-port circuit element;
in case of Boolean parameter <code>considerMagnetization = false</code> it is characterized by the following equations:
</p>
<pre> i2 = -i1*n;
 v2 =  v1/n;
</pre>
<p>
where <code>n</code> is a real number called the turns ratio.
Due to this equations, also DC voltages and currents are transformed - which is not the case for technical transformers.
</p>
<p>
In case of Boolean parameter <code>considerMagnetization = true</code> it is characterized by the following equations:
</p>
<pre>
 im1  = i1 + i2/n \"Magnetizing current w.r.t. primary side\";
 psim1= Lm1*im1   \"Magnetic flux w.r.t. primary side\";
 v1 = der(psim1)  \"Primary voltage\";
 v2 = v1/n        \"Secondary voltage\";
</pre>
<p>
where <code>Lm</code> denotes the magnetizing inductance.
Due to this equations, the DC offset of secondary voltages and currents decrement according to the time constant defined by the connected circuit.
</p>
<p>
Taking primary <code>L1sigma</code> and secondary <code>L2ssigma</code> leakage inductances into account,
compared with the <a href=\"modelica://Modelica.Electrical.Analog.Basic.Transformer\">basic transformer</a>
the following parameter conversion can be applied (which leads to identical results):
</p>
<pre>
 L1 = L1sigma + M*n \"Primary inductance at secondary no-load\";
 L2 = L2sigma + M/n \"Secondary inductance at primary no-load\";
  M  = Lm1/n         \"Mutual inductance\";
</pre>
<p>
For the backward conversion, one has to decide about the partitioning of the leakage to primary and secondary side.
</p>
</html>", revisions = "<html>
<ul>
<li><i>June 3, 2009   </i>
       magnetisation current added by Anton Haumer<br>
       </li>
<li><i>1998   </i>
       initially implemented by Christoph Clauss<br>
       </li>
</ul>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-45, -50}, {-20, -25}}), Ellipse(extent = {{-45, -25}, {-20, 0}}), Ellipse(extent = {{-45, 0}, {-20, 25}}), Ellipse(extent = {{-45, 25}, {-20, 50}}), Rectangle(extent = {{-72, -60}, {-33, 60}}, lineColor = {255, 255, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{-90, 50}, {-32, 50}}), Line(points = {{-90, -50}, {-32, -50}}), Ellipse(extent = {{20, -50}, {45, -25}}), Ellipse(extent = {{20, -25}, {45, 0}}), Ellipse(extent = {{20, 0}, {45, 25}}), Ellipse(extent = {{20, 25}, {45, 50}}), Rectangle(extent = {{33, -60}, {72, 60}}, lineColor = {255, 255, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{32, 50}, {90, 50}}), Line(points = {{32, -50}, {90, -50}}), Text(extent = {{-100, -80}, {100, -100}}, textString = "n=%n"), Text(extent = {{-100, 10}, {-80, -10}}, lineColor = {0, 0, 255}, textString = "1"), Text(extent = {{80, 10}, {100, -10}}, lineColor = {0, 0, 255}, textString = "2"), Text(extent = {{-146, 115}, {154, 75}}, textString = "%name", lineColor = {0, 0, 255})}),
            Diagram(graphics = {Text(extent = {{-100, 10}, {0, -10}}, lineColor = {0, 0, 255}, textString = "1=primary"), Text(extent = {{0, 10}, {100, -10}}, lineColor = {0, 0, 255}, textString = "2=secondary")}));
        end IdealTransformer;

        model IdealGyrator "Ideal gyrator"
          extends Interfaces.TwoPort;
          parameter SI.Conductance G(start = 1) "Gyration conductance";
        equation
          i1 = G * v2;
          i2 = -G * v1;
          annotation(
            Documentation(info = "<html>
<p>
A gyrator is an ideal two-port element defined by the following equations:
<br><br>
<code>
    i1 =  G * v2<br>
    i2 = -G * v1<br>
</code>
<br>
where the constant <i>G</i> is called the gyration conductance.
</P>
</html>", revisions = "<html>
<ul>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-70, -30}, {-10, 30}}, lineColor = {0, 0, 255}), Rectangle(extent = {{-80, -40}, {-41, 40}}, lineColor = {255, 255, 255}, pattern = LinePattern.Solid, lineThickness = 0.25, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{-90, 50}, {-40, 50}, {-40, -50}, {-90, -50}}, color = {0, 0, 255}), Line(points = {{-30, 60}, {20, 60}}, color = {0, 0, 255}), Polygon(points = {{20, 63}, {30, 60}, {20, 57}, {20, 63}}, pattern = LinePattern.Solid, lineThickness = 0.25, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Ellipse(extent = {{10, -30}, {70, 30}}, lineColor = {0, 0, 255}), Rectangle(extent = {{80, -40}, {40, 40}}, lineColor = {255, 255, 255}, pattern = LinePattern.Solid, lineThickness = 0.25, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{90, 50}, {40, 50}, {40, -50}, {90, -50}}, color = {0, 0, 255}), Text(extent = {{-152, 116}, {148, 76}}, textString = "%name", lineColor = {0, 0, 255})}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-70, -30}, {-10, 30}}, lineColor = {0, 0, 255}), Rectangle(extent = {{-80, -40}, {-41, 40}}, lineColor = {255, 255, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{-96, 50}, {-40, 50}, {-40, -50}, {-96, -50}}, color = {0, 0, 255}), Line(points = {{-30, 60}, {20, 60}}, color = {0, 0, 255}), Polygon(points = {{20, 63}, {30, 60}, {20, 57}, {20, 63}}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Ellipse(extent = {{10, -30}, {70, 30}}, lineColor = {0, 0, 255}), Rectangle(extent = {{80, -40}, {40, 40}}, lineColor = {255, 255, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{96, 50}, {40, 50}, {40, -50}, {96, -50}}, color = {0, 0, 255})}));
        end IdealGyrator;

        model Idle "Idle branch"
          extends Interfaces.OnePort;
        equation
          i = 0;
          annotation(
            Documentation(info = "<html>
<p>The model Idle is a simple idle running branch. That means between both pins no current is running. This ideal device is of no influence on the circuit. Therefore, it can be neglected in each case. For purposes of completeness this component is part of the MSL, as an opposite of the short cut.</p>
</html>", revisions = "<html>
<ul>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-60, 60}, {60, -60}}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Line(points = {{-90, 0}, {-41, 0}}, color = {0, 0, 255}), Line(points = {{91, 0}, {40, 0}}, color = {0, 0, 255}), Text(extent = {{-153, 112}, {147, 72}}, textString = "%name", lineColor = {0, 0, 255})}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-60, 60}, {60, -60}}, lineColor = {0, 0, 255}), Line(points = {{-96, 0}, {-41, 0}}, color = {0, 0, 255}), Line(points = {{96, 0}, {40, 0}}, color = {0, 0, 255})}));
        end Idle;

        model Short "Short cut branch"
          extends Interfaces.OnePort;
        equation
          v = 0;
          annotation(
            Documentation(info = "<html>
<p>The model Short is a simple short cut branch. That means the voltage drop between both pins is zero. This device could be neglected if both pins are combined to one node. Besides connecting the nodes of both pins this device has no further function.</p>
</html>", revisions = "<html>
<ul>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-60, 60}, {60, -60}}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Line(points = {{91, 0}, {-90, 0}}, color = {0, 0, 255}), Text(extent = {{-151, 113}, {149, 73}}, textString = "%name", lineColor = {0, 0, 255})}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-60, 60}, {60, -60}}, lineColor = {0, 0, 255}), Line(points = {{96, 0}, {-96, 0}}, color = {0, 0, 255}), Text(extent = {{-100, 100}, {100, 70}}, textString = "Short", lineColor = {0, 0, 255})}));
        end Short;

        model IdealOpeningSwitch "Ideal electrical opener"
          extends Modelica.Electrical.Analog.Interfaces.OnePort;
          parameter SI.Resistance Ron(final min = 0) = 1.E-5 "Closed switch resistance";
          parameter SI.Conductance Goff(final min = 0) = 1.E-5 "Opened switch conductance";
          extends Modelica.Electrical.Analog.Interfaces.ConditionalHeatPort(final T = 293.15);
          Modelica.Blocks.Interfaces.BooleanInput control "true => switch open, false => p--n connected" annotation(
            Placement(transformation(origin = {0, 70}, extent = {{-20, -20}, {20, 20}}, rotation = 270)));
        protected
          Real s(final unit = "1") "Auxiliary variable";
          constant Modelica.SIunits.Voltage unitVoltage = 1 annotation(
            HideResult = true);
          constant Modelica.SIunits.Current unitCurrent = 1 annotation(
            HideResult = true);
        equation
          v = s * unitCurrent * (if control then 1 else Ron);
          i = s * unitVoltage * (if control then Goff else 1);
          LossPower = v * i;
          annotation(
            Documentation(info = "<HTML>
<P>
The ideal opening switch has a positive pin p and a negative pin n.
The switching behaviour is controlled by the input signal control.
If control is true, pin p is not connected
with negative pin n. Otherwise, pin p is connected
with negative pin n.<br><br>

In order to prevent singularities during switching, the opened
switch has a (very low) conductance Goff
and the closed switch has a (very low) resistance Ron.
The limiting case is also allowed, i.e., the resistance Ron of the
closed switch could be exactly zero and the conductance Goff of the
open switch could be also exactly zero. Note, there are circuits,
where a description with zero Ron or zero Goff is not possible.
<br> <br>
<b>Please note:</b>
In case of useHeatPort=true the temperature dependence of the electrical
behavior is <b> not </b> modelled. The parameters are not temperature dependent.
</P>
</html>", revisions = "<html>
<ul>
<li><i> March 11, 2009   </i>
       by Christoph Clauss<br> conditional heat port added<br>
       </li>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-44, 4}, {-36, -4}}, lineColor = {0, 0, 255}), Line(points = {{-90, 0}, {-44, 0}}, color = {0, 0, 255}), Line(points = {{-37, 2}, {40, 50}}, color = {0, 0, 255}), Line(points = {{40, 0}, {90, 0}}, color = {0, 0, 255}), Line(points = {{0, 51}, {0, 26}}, color = {0, 0, 255}), Line(points = {{40, 20}, {40, 0}}, color = {0, 0, 255}), Line(visible = useHeatPort, points = {{0, -100}, {0, 25}}, color = {127, 0, 0}, smooth = Smooth.None, pattern = LinePattern.Dot), Text(extent = {{-151, -21}, {149, -61}}, textString = "%name", lineColor = {0, 0, 255})}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-44, 4}, {-36, -4}}, lineColor = {0, 0, 255}), Line(points = {{-96, 0}, {-44, 0}}, color = {0, 0, 255}), Line(points = {{-37, 2}, {40, 50}}, color = {0, 0, 255}), Line(points = {{40, 0}, {96, 0}}, color = {0, 0, 255}), Text(extent = {{-100, -40}, {100, -79}}, textString = "%name", lineColor = {0, 0, 255}), Line(points = {{0, 51}, {0, 26}}, color = {0, 0, 255}), Line(points = {{40, 20}, {40, 0}}, color = {0, 0, 255})}));
        end IdealOpeningSwitch;

        model IdealClosingSwitch "Ideal electrical closer"
          extends Modelica.Electrical.Analog.Interfaces.OnePort;
          parameter SI.Resistance Ron(final min = 0) = 1.E-5 "Closed switch resistance";
          parameter SI.Conductance Goff(final min = 0) = 1.E-5 "Opened switch conductance";
          extends Modelica.Electrical.Analog.Interfaces.ConditionalHeatPort(final T = 293.15);
          Modelica.Blocks.Interfaces.BooleanInput control "true => p--n connected, false => switch open" annotation(
            Placement(transformation(origin = {0, 70}, extent = {{-20, -20}, {20, 20}}, rotation = 270)));
        protected
          Real s(final unit = "1") "Auxiliary variable";
          constant Modelica.SIunits.Voltage unitVoltage = 1 annotation(
            HideResult = true);
          constant Modelica.SIunits.Current unitCurrent = 1 annotation(
            HideResult = true);
        equation
          v = s * unitCurrent * (if control then Ron else 1);
          i = s * unitVoltage * (if control then 1 else Goff);
          LossPower = v * i;
          annotation(
            Documentation(info = "<HTML>
<P>
The ideal closing switch has a positive pin p and a negative pin n.
The switching behaviour is controlled by input signal control.
If control is true, pin p is connected
with negative pin n. Otherwise, pin p is not connected
with negative pin n.<br><br>

In order to prevent singularities during switching, the opened
switch has a (very low) conductance Goff
and the closed switch has a (very low) resistance Ron.
The limiting case is also allowed, i.e., the resistance Ron of the
closed switch could be exactly zero and the conductance Goff of the
open switch could be also exactly zero. Note, there are circuits,
where a description with zero Ron or zero Goff is not possible.
<br> <br>
<b>Please note:</b>
In case of useHeatPort=true the temperature dependence of the electrical
behavior is <b> not </b> modelled. The parameters are not temperature dependent.
</P>
</html>", revisions = "<html>
<ul>
<li><i> March 11, 2009   </i>
       by Christoph Clauss<br> conditional heat port added<br>
       </li>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-44, 4}, {-36, -4}}, lineColor = {0, 0, 255}), Line(points = {{-90, 0}, {-44, 0}}, color = {0, 0, 255}), Line(points = {{-37, 2}, {40, 50}}, color = {0, 0, 255}), Line(points = {{40, 0}, {90, 0}}, color = {0, 0, 255}), Line(points = {{0, 51}, {0, 26}}, color = {0, 0, 255}), Line(visible = useHeatPort, points = {{0, -100}, {0, 25}}, color = {127, 0, 0}, smooth = Smooth.None, pattern = LinePattern.Dot), Text(extent = {{-152, -28}, {148, -68}}, textString = "%name", lineColor = {0, 0, 255})}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-44, 4}, {-36, -4}}, lineColor = {0, 0, 255}), Line(points = {{-96, 0}, {-44, 0}}, color = {0, 0, 255}), Line(points = {{-37, 2}, {40, 50}}, color = {0, 0, 255}), Line(points = {{40, 0}, {96, 0}}, color = {0, 0, 255}), Text(extent = {{-100, -40}, {100, -79}}, textString = "%name", lineColor = {0, 0, 255}), Line(points = {{0, 51}, {0, 26}}, color = {0, 0, 255})}));
        end IdealClosingSwitch;

        model ControlledIdealOpeningSwitch "Controlled ideal electrical opener"
          parameter SI.Voltage level = 0.5 "Switch level";
          parameter SI.Resistance Ron(final min = 0) = 1.E-5 "Closed switch resistance";
          parameter SI.Conductance Goff(final min = 0) = 1.E-5 "Opened switch conductance";
          extends Modelica.Electrical.Analog.Interfaces.ConditionalHeatPort(final T = 293.15);
          Interfaces.PositivePin p annotation(
            Placement(transformation(extent = {{-110, -10}, {-90, 10}}, rotation = 0)));
          Interfaces.NegativePin n annotation(
            Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
          Interfaces.Pin control "Control pin: control.v > level switch open, otherwise p--n connected" annotation(
            Placement(transformation(origin = {0, 100}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
        protected
          Real s(final unit = "1") "Auxiliary variable";
          constant Modelica.SIunits.Voltage unitVoltage = 1 annotation(
            HideResult = true);
          constant Modelica.SIunits.Current unitCurrent = 1 annotation(
            HideResult = true);
        equation
          control.i = 0;
          0 = p.i + n.i;
          p.v - n.v = s * unitCurrent * (if control.v > level then 1 else Ron);
          p.i = s * unitVoltage * (if control.v > level then Goff else 1);
          LossPower = (p.v - n.v) * p.i;
          annotation(
            Documentation(info = "<HTML>
<P>
The ideal switch has a positive pin p and a negative pin n.
The switching behaviour is controlled by the control pin.
If its voltage exceeds the voltage of the parameter level,
pin p is not connected with negative pin n.
Otherwise, pin p is connected with negative pin n.<br><br>

In order to prevent singularities during switching, the opened
switch has a (very low) conductance Goff
and the closed switch has a (very low) resistance Ron.
The limiting case is also allowed, i.e., the resistance Ron of the
closed switch could be exactly zero and the conductance Goff of the
open switch could be also exactly zero. Note, there are circuits,
where a description with zero Ron or zero Goff is not possible.
<br> <br>
<b>Please note:</b>
In case of useHeatPort=true the temperature dependence of the electrical
behavior is <b> not </b> modelled. The parameters are not temperature dependent.
</P>
</html>", revisions = "<html>
<ul>
<li><i> March 11, 2009   </i>
       by Christoph Clauss<br> conditional heat port added<br>
       </li>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-44, 4}, {-36, -4}}, lineColor = {0, 0, 255}), Line(points = {{-90, 0}, {-44, 0}}, color = {0, 0, 255}), Line(points = {{-37, 2}, {40, 50}}, color = {0, 0, 255}), Line(points = {{40, 0}, {90, 0}}, color = {0, 0, 255}), Line(points = {{0, 90}, {0, 25}}, color = {0, 0, 255}), Line(points = {{40, 20}, {40, 0}}, color = {0, 0, 255}), Line(visible = useHeatPort, points = {{0, -100}, {0, 25}}, color = {127, 0, 0}, smooth = Smooth.None, pattern = LinePattern.Dot), Text(extent = {{-150, -34}, {150, -74}}, textString = "%name", lineColor = {0, 0, 255})}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-44, 4}, {-36, -4}}, lineColor = {0, 0, 255}), Line(points = {{-96, 0}, {-44, 0}}, color = {0, 0, 255}), Line(points = {{-37, 2}, {40, 50}}, color = {0, 0, 255}), Line(points = {{40, 0}, {96, 0}}, color = {0, 0, 255}), Line(points = {{0, 96}, {0, 25}}, color = {0, 0, 255}), Line(points = {{40, 20}, {40, 0}}, color = {0, 0, 255})}));
        end ControlledIdealOpeningSwitch;

        model ControlledIdealClosingSwitch "Controlled ideal electrical closer"
          parameter SI.Voltage level = 0.5 "Switch level";
          parameter SI.Resistance Ron(final min = 0) = 1.E-5 "Closed switch resistance";
          parameter SI.Conductance Goff(final min = 0) = 1.E-5 "Opened switch conductance";
          extends Modelica.Electrical.Analog.Interfaces.ConditionalHeatPort(final T = 293.15);
          Modelica.Electrical.Analog.Interfaces.PositivePin p annotation(
            Placement(transformation(extent = {{-110, -10}, {-90, 10}}, rotation = 0)));
          Modelica.Electrical.Analog.Interfaces.NegativePin n annotation(
            Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
          Modelica.Electrical.Analog.Interfaces.Pin control "Control pin: control.v > level switch closed, otherwise switch open" annotation(
            Placement(transformation(origin = {0, 100}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
        protected
          Real s(final unit = "1") "Auxiliary variable";
          constant Modelica.SIunits.Voltage unitVoltage = 1 annotation(
            HideResult = true);
          constant Modelica.SIunits.Current unitCurrent = 1 annotation(
            HideResult = true);
        equation
          control.i = 0;
          0 = p.i + n.i;
          p.v - n.v = s * unitCurrent * (if control.v > level then Ron else 1);
          p.i = s * unitVoltage * (if control.v > level then 1 else Goff);
          LossPower = (p.v - n.v) * p.i;
          annotation(
            Documentation(info = "<html>
<P>
The closing ideal switch has a positive pin p and a negative pin n.
The switching behaviour is controlled by the control pin.
If its voltage exceeds the voltage of the parameter level,
pin p is connected with negative pin n.
Otherwise, pin p is not connected with negative pin n.<br><br>

In order to prevent singularities during switching, the opened
switch has a (very low) conductance Goff
and the closed switch has a (very low) resistance Ron.
The limiting case is also allowed, i.e., the resistance Ron of the
closed switch could be exactly zero and the conductance Goff of the
open switch could be also exactly zero. Note, there are circuits,
where a description with zero Ron or zero Goff is not possible.
<br> <br>
<b>Please note:</b>
In case of useHeatPort=true the temperature dependence of the electrical
behavior is <b> not </b> modelled. The parameters are not temperature dependent.
</P>
</html>", revisions = "<html>
<ul>
<li><i> March 11, 2009   </i>
       by Christoph Clauss<br> conditional heat port added<br>
       </li>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-44, 4}, {-36, -4}}, lineColor = {0, 0, 255}), Line(points = {{-90, 0}, {-44, 0}}, color = {0, 0, 255}), Line(points = {{-37, 2}, {40, 50}}, color = {0, 0, 255}), Line(points = {{40, 0}, {90, 0}}, color = {0, 0, 255}), Line(points = {{0, 90}, {0, 25}}, color = {0, 0, 255}), Line(visible = useHeatPort, points = {{0, -100}, {0, 25}}, color = {127, 0, 0}, smooth = Smooth.None, pattern = LinePattern.Dot), Text(extent = {{-149, -32}, {151, -72}}, textString = "%name", lineColor = {0, 0, 255})}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-44, 4}, {-36, -4}}, lineColor = {0, 0, 255}), Line(points = {{-96, 0}, {-44, 0}}, color = {0, 0, 255}), Line(points = {{-37, 2}, {40, 50}}, color = {0, 0, 255}), Line(points = {{40, 0}, {96, 0}}, color = {0, 0, 255}), Line(points = {{0, 96}, {0, 25}}, color = {0, 0, 255})}));
        end ControlledIdealClosingSwitch;

        model OpenerWithArc "Ideal opening switch with simple arc model"
          extends Modelica.Electrical.Analog.Interfaces.OnePort;
          parameter Modelica.SIunits.Resistance Ron = 1E-5 "Closed switch resistance";
          parameter Modelica.SIunits.Conductance Goff = 1E-5 "Opened switch conductance";
          parameter Modelica.SIunits.Voltage V0(start = 30) "Initial arc voltage";
          parameter Modelica.SIunits.VoltageSlope dVdt(start = 10E3) "Arc voltage slope";
          parameter Modelica.SIunits.Voltage Vmax(start = 60) "Max. arc voltage";
          extends Modelica.Electrical.Analog.Interfaces.ConditionalHeatPort(final T = 293.15);
          Modelica.Blocks.Interfaces.BooleanInput control "false => p--n connected, true => switch open" annotation(
            Placement(transformation(origin = {0, 100}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
        protected
          Boolean off = control;
          Boolean on = not off;
          discrete Modelica.SIunits.Time tSwitch(start = -Modelica.Constants.inf, fixed = true);
          Boolean quenched(start = true, fixed = true);
        equation
          when edge(off) then
            tSwitch = time;
          end when;
          quenched = off and (abs(i) <= abs(v) * Goff or pre(quenched));
          if on then
            v = Ron * i;
          else
            if quenched then
              i = Goff * v;
            else
              v = min(Vmax, V0 + dVdt * (time - tSwitch)) * sign(i);
            end if;
          end if;
          LossPower = v * i;
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{40, 50}, {32, 32}, {48, 28}, {40, 18}}, color = {255, 0, 0}), Ellipse(extent = {{-44, 4}, {-36, -4}}), Line(points = {{-90, 0}, {-44, 0}}), Line(points = {{-37, 2}, {40, 50}}), Line(points = {{40, 0}, {90, 0}}), Line(points = {{0, 90}, {0, 26}}, color = {255, 85, 255}), Line(points = {{40, 18}, {40, 0}}), Text(extent = {{-150, -35}, {150, -75}}, textString = "%name", lineColor = {0, 0, 255})}),
            Diagram(graphics = {Line(points = {{-60, 60}, {-60, -60}, {60, -60}}, color = {0, 0, 255}), Line(points = {{-60, -60}, {-40, -60}, {-40, -40}, {-20, 40}, {40, 40}}, color = {0, 0, 0}), Text(extent = {{30, -60}, {50, -70}}, lineColor = {0, 0, 0}, textString = "time"), Text(extent = {{-60, 60}, {-20, 50}}, lineColor = {0, 0, 0}, textString = "voltage"), Text(extent = {{-60, -30}, {-40, -40}}, lineColor = {0, 0, 0}, textString = "V0"), Text(extent = {{-50, 40}, {-30, 30}}, lineColor = {0, 0, 0}, textString = "Vmax"), Text(extent = {{-40, 10}, {-20, 0}}, lineColor = {0, 0, 0}, textString = "dVdt"), Polygon(points = {{-60, 60}, {-62, 52}, {-58, 52}, {-60, 60}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Polygon(points = {{60, -60}, {54, -58}, {54, -62}, {60, -60}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid)}),
            Documentation(info = "<html>
<p>
This model is an extension to the <a href=\"modelica://Modelica.Electrical.Analog.Ideal.IdealOpeningSwitch\">IdealOpeningSwitch</a>.
</p>
<p>
The basic model interrupts the current through the switch in an infinitesimal time span.
If an inductive circuit is connected, the voltage across the switch is limited only by numerics.
In order to give a better idea for the voltage across the switch, a simple arc model is added:
</p>
<p>
When the Boolean input <code>control</code> signals to open the switch, a voltage across the opened switch is impressed.
This voltage starts with <code>V0</code> (simulating the voltage drop of the arc roots), then rising with slope <code>dVdt</code>
(simulating the rising voltage of an extending arc) until a maximum voltage <code>Vmax</code> is reached.
</p>
<pre>
     | voltage
Vmax |      +-----
     |     /
     |    /
V0   |   +
     |   |
     +---+-------- time
</pre>
<p>
This arc voltage tends to lower the current following through the switch; it depends on the connected circuit, when the arc is quenched.
Once the arc is quenched, i.e., the current flowing through the switch gets zero, the equation for the off-state is activated
<code>i=Goff*v</code>.
</p>
<p>
When the Boolean input <code>control</code> signals to close the switch again, the switch is closed immediately,
i.e., the equation for the on-state is activated <code>v=Ron*i</code>.
</p>
<p>
Please note: In an AC circuit, at least the arc quenches when the next natural zero-crossing of the current occurs.
In a DC circuit, the arc will not quench if the arc voltage is not sufficient that a zero-crossing of the current occurs.
<br> <br>
<b>Please note:</b>
In case of useHeatPort=true the temperature dependence of the electrical
behavior is <b> not </b> modelled. The parameters are not temperature dependent.
</p>
</html>", revisions = "<html>
<ul>
<li><i>June, 2009   </i>
       by Christoph Clauss<br> adapted to OpenerWithArc<br>
       </li>
<li><i>May, 2009   </i>
       by Anton Haumer<br> CloserWithArc initially implemented<br>
       </li>
</ul>
</html>"));
        end OpenerWithArc;

        model CloserWithArc "Ideal closing switch with simple arc model"
          extends Modelica.Electrical.Analog.Interfaces.OnePort;
          parameter Modelica.SIunits.Resistance Ron = 1E-5 "Closed switch resistance";
          parameter Modelica.SIunits.Conductance Goff = 1E-5 "Opened switch conductance";
          parameter Modelica.SIunits.Voltage V0(start = 30) "Initial arc voltage";
          parameter Modelica.SIunits.VoltageSlope dVdt(start = 10E3) "Arc voltage slope";
          parameter Modelica.SIunits.Voltage Vmax(start = 60) "Max. arc voltage";
          extends Modelica.Electrical.Analog.Interfaces.ConditionalHeatPort(final T = 293.15);
          Modelica.Blocks.Interfaces.BooleanInput control "true => p--n connected, false => switch open" annotation(
            Placement(transformation(origin = {0, 100}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
        protected
          Boolean on = control;
          Boolean off(start = false, fixed = true);
          discrete Modelica.SIunits.Time tSwitch(start = -Modelica.Constants.inf, fixed = true);
          Boolean quenched(start = true, fixed = true);
        equation
          off = not on;
          when edge(off) then
            tSwitch = time;
          end when;
          quenched = off and (abs(i) <= abs(v) * Goff or pre(quenched));
          if on then
            v = Ron * i;
          else
            if quenched then
              i = Goff * v;
            else
              v = min(Vmax, V0 + dVdt * (time - tSwitch)) * sign(i);
            end if;
          end if;
          LossPower = v * i;
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{40, 50}, {32, 24}, {48, 28}, {40, 0}}, color = {255, 0, 0}), Ellipse(extent = {{-44, 4}, {-36, -4}}), Line(points = {{-90, 0}, {-44, 0}}), Line(points = {{-37, 2}, {40, 50}}), Line(points = {{40, 0}, {90, 0}}), Line(points = {{0, 90}, {0, 26}}, color = {255, 85, 255}), Text(extent = {{-152, -29}, {148, -69}}, textString = "%name", lineColor = {0, 0, 255})}),
            Diagram(graphics = {Line(points = {{-60, 60}, {-60, -60}, {60, -60}}, color = {0, 0, 255}), Line(points = {{-60, -60}, {-40, -60}, {-40, -40}, {-20, 40}, {40, 40}}, color = {0, 0, 0}), Text(extent = {{30, -60}, {50, -70}}, lineColor = {0, 0, 0}, textString = "time"), Text(extent = {{-60, 60}, {-20, 50}}, lineColor = {0, 0, 0}, textString = "voltage"), Text(extent = {{-60, -30}, {-40, -40}}, lineColor = {0, 0, 0}, textString = "V0"), Text(extent = {{-50, 40}, {-30, 30}}, lineColor = {0, 0, 0}, textString = "Vmax"), Text(extent = {{-40, 10}, {-20, 0}}, lineColor = {0, 0, 0}, textString = "dVdt"), Polygon(points = {{-60, 60}, {-62, 52}, {-58, 52}, {-60, 60}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Polygon(points = {{60, -60}, {54, -58}, {54, -62}, {60, -60}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid)}),
            Documentation(info = "<html>
<p>
This model is an extension to the <a href=\"modelica://Modelica.Electrical.Analog.Ideal.IdealClosingSwitch\">IdealClosingSwitch</a>.
</p>
<p>
The basic model interrupts the current through the switch in an infinitesimal time span.
If an inductive circuit is connected, the voltage across the switch is limited only by numerics.
In order to give a better idea for the voltage across the switch, a simple arc model is added:
</p>
<p>
When the Boolean input <code>control</code> signals to open the switch, a voltage across the opened switch is impressed.
This voltage starts with <code>V0</code> (simulating the voltage drop of the arc roots), then rising with slope <code>dVdt</code>
(simulating the rising voltage of an extending arc) until a maximum voltage <code>Vmax</code> is reached.
</p>
<pre>
     | voltage
Vmax |      +-----
     |     /
     |    /
V0   |   +
     |   |
     +---+-------- time
</pre>
<p>
This arc voltage tends to lower the current following through the switch; it depends on the connected circuit, when the arc is quenched.
Once the arc is quenched, i.e., the current flowing through the switch gets zero, the equation for the off-state is activated
<code>i=Goff*v</code>.
</p>
<p>
When the Boolean input <code>control</code> signals to close the switch again, the switch is closed immediately,
i.e., the equation for the on-state is activated <code>v=Ron*i</code>.
</p>
<p>
Please note: In an AC circuit, at least the arc quenches when the next natural zero-crossing of the current occurs.
In a DC circuit, the arc will not quench if the arc voltage is not sufficient that a zero-crossing of the current occurs.
<br> <br>
<b>Please note:</b>
In case of useHeatPort=true the temperature dependence of the electrical
behavior is <b> not </b> modelled. The parameters are not temperature dependent.
</p>
</html>", revisions = "<html>
<ul>
<li><i>May, 2009   </i>
       by Anton Haumer<br> initially implemented<br>
       </li>
</ul>
</html>"));
        end CloserWithArc;

        model ControlledOpenerWithArc "Controlled ideal electrical opener with simple arc model"
          parameter Modelica.SIunits.Voltage level = 0.5 "Switch level";
          parameter Modelica.SIunits.Resistance Ron(final min = 0) = 1.E-5 "Closed switch resistance";
          parameter Modelica.SIunits.Conductance Goff(final min = 0) = 1.E-5 "Opened switch conductance";
          parameter Modelica.SIunits.Voltage V0(start = 30) "Initial arc voltage";
          parameter Modelica.SIunits.VoltageSlope dVdt(start = 10E3) "Arc voltage slope";
          parameter Modelica.SIunits.Voltage Vmax(start = 60) "Max. arc voltage";
          extends Modelica.Electrical.Analog.Interfaces.ConditionalHeatPort(final T = 293.15);
          Modelica.Electrical.Analog.Interfaces.PositivePin p annotation(
            Placement(transformation(extent = {{-110, -10}, {-90, 10}}, rotation = 0)));
          Modelica.Electrical.Analog.Interfaces.NegativePin n annotation(
            Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
          Modelica.Electrical.Analog.Interfaces.Pin control "Control pin: control.v > level switch open, otherwise p--n connected" annotation(
            Placement(transformation(origin = {0, 100}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
          Modelica.SIunits.Current i;
          Modelica.SIunits.Voltage v;
        protected
          constant Modelica.SIunits.Voltage unitVoltage = 1 annotation(
            HideResult = true);
          constant Modelica.SIunits.Current unitCurrent = 1 annotation(
            HideResult = true);
          Boolean off = control.v > level;
          Boolean on = not off;
          discrete Modelica.SIunits.Time tSwitch(start = -Modelica.Constants.inf, fixed = true);
          Boolean quenched(start = true);
        equation
          control.i = 0;
          0 = p.i + n.i;
          i = p.i;
          p.v - n.v = v;
          when edge(off) then
            tSwitch = time;
          end when;
          quenched = off and (abs(i) <= abs(v) * Goff or pre(quenched));
          if on then
            v = Ron * i;
          else
            if quenched then
              i = Goff * v;
            else
              v = min(Vmax, V0 + dVdt * (time - tSwitch)) * sign(i);
            end if;
          end if;
          LossPower = v * i;
          annotation(
            Documentation(info = "<html>

<p>
This model is an extension to the <a href=\"modelica://Modelica.Electrical.Analog.Ideal.IdealOpeningSwitch\">IdealOpeningSwitch</a>.
</p>

<p>
The basic model interrupts the current through the switch in an infinitesimal time span.
If an inductive circuit is connected, the voltage across the switch is limited only by numerics.
In order to give a better idea for the voltage across the switch, a simple arc model is added:
</p>
<p>
When the control pin voltage <code>control.v</code> signals to open the switch, a voltage across the opened switch is impressed.
This voltage starts with <code>V0</code> (simulating the voltage drop of the arc roots), then rising with slope <code>dVdt</code>
(simulating the rising voltage of an extending arc) until a maximum voltage <code>Vmax</code> is reached.
</p>
<pre>
     | voltage
Vmax |      +-----
     |     /
     |    /
V0   |   +
     |   |
     +---+-------- time
</pre>
<p>
This arc voltage tends to lower the current following through the switch; it depends on the connected circuit, when the arc is quenched.
Once the arc is quenched, i.e., the current flowing through the switch gets zero, the equation for the off-state is activated
<code>i=Goff*v</code>.
</p>
<p>
When the control pin <code>control.v</code> signals to close the switch again, the switch is closed immediately,
i.e., the equation for the on-state is activated <code>v=Ron*i</code>.
</p>
<p>
<b>Please note:</b>
In an AC circuit, at least the arc quenches when the next natural zero-crossing of the current occurs.
In a DC circuit, the arc will not quench if the arc voltage is not sufficient that a zero-crossing of the current occurs.
</p>
<p>
<b>Please note:</b>
In case of useHeatPort=true the temperature dependence of the electrical
behavior is <b> not </b> modelled. The parameters are not temperature dependent.
</p>
</html>", revisions = "<html>
<dl>
<dt><i>1998</i></dt>
<dd>by Christoph Clauss initially implemented</dd>
</dl>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-44, 4}, {-36, -4}}, lineColor = {0, 0, 255}), Line(points = {{-90, 0}, {-44, 0}}, color = {0, 0, 255}), Line(points = {{-37, 2}, {40, 50}}, color = {0, 0, 255}), Line(points = {{40, 0}, {90, 0}}, color = {0, 0, 255}), Line(points = {{0, 90}, {0, 25}}, color = {0, 0, 255}), Line(points = {{40, 20}, {40, 0}}, color = {0, 0, 255}), Line(points = {{40, 50}, {32, 34}, {48, 30}, {40, 20}}, color = {255, 0, 0}), Text(extent = {{-148, -34}, {152, -74}}, textString = "%name", lineColor = {0, 0, 255})}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-44, 4}, {-36, -4}}, lineColor = {0, 0, 255}), Line(points = {{-96, 0}, {-44, 0}}, color = {0, 0, 255}), Line(points = {{-37, 2}, {40, 50}}, color = {0, 0, 255}), Line(points = {{40, 0}, {96, 0}}, color = {0, 0, 255}), Line(points = {{0, 96}, {0, 25}}, color = {0, 0, 255}), Line(points = {{40, 20}, {40, 0}}, color = {0, 0, 255})}));
        end ControlledOpenerWithArc;

        model ControlledCloserWithArc "Controlled ideal electrical closer with simple arc model"
          parameter Modelica.SIunits.Voltage level = 0.5 "Switch level";
          parameter Modelica.SIunits.Resistance Ron(final min = 0) = 1.E-5 "Closed switch resistance";
          parameter Modelica.SIunits.Conductance Goff(final min = 0) = 1.E-5 "Opened switch conductance";
          parameter Modelica.SIunits.Voltage V0(start = 30) "Initial arc voltage";
          parameter Modelica.SIunits.VoltageSlope dVdt(start = 10E3) "Arc voltage slope";
          parameter Modelica.SIunits.Voltage Vmax(start = 60) "Max. arc voltage";
          extends Modelica.Electrical.Analog.Interfaces.ConditionalHeatPort(final T = 293.15);
          Modelica.Electrical.Analog.Interfaces.PositivePin p annotation(
            Placement(transformation(extent = {{-110, -10}, {-90, 10}}, rotation = 0)));
          Modelica.Electrical.Analog.Interfaces.NegativePin n annotation(
            Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
          Modelica.Electrical.Analog.Interfaces.Pin control "Control pin: control.v > level switch closed, otherwise switch open" annotation(
            Placement(transformation(origin = {0, 100}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
          Modelica.SIunits.Current i;
          Modelica.SIunits.Voltage v;
        protected
          constant Modelica.SIunits.Voltage unitVoltage = 1 annotation(
            HideResult = true);
          constant Modelica.SIunits.Current unitCurrent = 1 annotation(
            HideResult = true);
          Boolean off(start = false, fixed = true);
          Boolean on = not off;
          discrete Modelica.SIunits.Time tSwitch(start = -Modelica.Constants.inf, fixed = true);
          Boolean quenched(start = true, fixed = true);
        equation
          off = control.v < level;
          control.i = 0;
          0 = p.i + n.i;
          i = p.i;
          p.v - n.v = v;
          when edge(off) then
            tSwitch = time;
          end when;
          quenched = off and (abs(i) <= abs(v) * Goff or pre(quenched));
          if on then
            v = Ron * i;
          else
            if quenched then
              i = Goff * v;
            else
              v = min(Vmax, V0 + dVdt * (time - tSwitch)) * sign(i);
            end if;
          end if;
          LossPower = v * i;
          annotation(
            Documentation(info = "<html>
<p>
This model is an extension to the <a href=\"modelica://Modelica.Electrical.Analog.Ideal.IdealClosingSwitch\">IdealClosingSwitch</a>.
</p>
<p>
The basic model interrupts the current through the switch in an infinitesimal time span.
If an inductive circuit is connected, the voltage across the switch is limited only by numerics.
In order to give a better idea for the voltage across the switch, a simple arc model is added:
</p>
<p>
When the control pin voltage <code>control.v</code> signals to open the switch, a voltage across the opened switch is impressed.
This voltage starts with <code>V0</code> (simulating the voltage drop of the arc roots), then rising with slope <code>dVdt</code>
(simulating the rising voltage of an extending arc) until a maximum voltage <code>Vmax</code> is reached.
</p>
<pre>
     | voltage
Vmax |      +-----
     |     /
     |    /
V0   |   +
     |   |
     +---+-------- time
</pre>
<p>
This arc voltage tends to lower the current following through the switch; it depends on the connected circuit, when the arc is quenched.
Once the arc is quenched, i.e., the current flowing through the switch gets zero, the equation for the off-state is activated
<code>i=Goff*v</code>.
</p>
<p>
When the control pin <code>control.v</code> signals to close the switch again, the switch is closed immediately,
i.e., the equation for the on-state is activated <code>v=Ron*i</code>.
</p>
<p>
<b>Please note:</b>
In an AC circuit, at least the arc quenches when the next natural zero-crossing of the current occurs.
In a DC circuit, the arc will not quench if the arc voltage is not sufficient that a zero-crossing of the current occurs.
</p>
<p>
<b>Please note:</b>
In case of useHeatPort=true the temperature dependence of the electrical
behavior is <b>not</b> modelled. The parameters are not temperature dependent.
</p>

</html>", revisions = "<html>
<dl>
<dt><i>1998</i></dt>
<dd>by Christoph Clauss<br> initially implemented</dd>
</dl>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-44, 4}, {-36, -4}}, lineColor = {0, 0, 255}), Line(points = {{-90, 0}, {-44, 0}}, color = {0, 0, 255}), Line(points = {{-37, 2}, {40, 50}}, color = {0, 0, 255}), Line(points = {{40, 0}, {90, 0}}, color = {0, 0, 255}), Line(points = {{0, 90}, {0, 25}}, color = {0, 0, 255}), Text(extent = {{-100, -70}, {100, -100}}, textString = "%name", lineColor = {0, 0, 255}), Line(points = {{40, 50}, {32, 24}, {48, 28}, {40, 0}}, color = {255, 0, 0})}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-44, 4}, {-36, -4}}, lineColor = {0, 0, 255}), Line(points = {{-96, 0}, {-44, 0}}, color = {0, 0, 255}), Line(points = {{-37, 2}, {40, 50}}, color = {0, 0, 255}), Line(points = {{40, 0}, {96, 0}}, color = {0, 0, 255}), Line(points = {{0, 96}, {0, 25}}, color = {0, 0, 255})}));
        end ControlledCloserWithArc;

        model IdealTriac "Ideal triac, based on ideal thyristors"
          parameter Modelica.SIunits.Resistance Ron(final min = 0) = 1.e-5 "Closed triac resistance";
          parameter Modelica.SIunits.Conductance Goff(final min = 0) = 1.e-5 "Opened triac conductance";
          parameter Modelica.SIunits.Voltage Vknee(final min = 0, start = 0) = 0.8 "Threshold voltage for positive and negative phase";
          parameter Modelica.SIunits.Resistance Rdis = 100 "Resistance of disturbance elimination";
          parameter Modelica.SIunits.Capacitance Cdis = 0.005 "Capacity of disturbance elimination";
          Modelica.Electrical.Analog.Ideal.IdealThyristor idealThyristor(Ron = Ron, Goff = Goff, Vknee = Vknee) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0, origin = {-10, 32})));
          Modelica.Electrical.Analog.Ideal.IdealThyristor idealThyristor1(Ron = Ron, Goff = Goff, Vknee = Vknee) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 180, origin = {-10, -32})));
          Modelica.Electrical.Analog.Basic.Resistor resistor(R = Rdis) annotation(
            Placement(transformation(extent = {{-60, -10}, {-40, 10}})));
          Modelica.Electrical.Analog.Basic.Capacitor capacitor(C = Cdis) annotation(
            Placement(transformation(extent = {{20, -10}, {40, 10}})));
          Modelica.Blocks.Interfaces.BooleanInput fire1 "Gate" annotation(
            Placement(transformation(extent = {{-74, -106}, {-46, -78}})));
          Modelica.Electrical.Analog.Interfaces.NegativePin n "Cathode" annotation(
            Placement(transformation(extent = {{-110, -10}, {-90, 10}})));
          Modelica.Electrical.Analog.Interfaces.PositivePin p "Anode" annotation(
            Placement(transformation(extent = {{94, -10}, {114, 10}})));
        equation
          connect(idealThyristor.n, capacitor.n) annotation(
            Line(points = {{0, 32}, {40, 32}, {40, 0}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(capacitor.n, idealThyristor1.p) annotation(
            Line(points = {{40, 0}, {40, -32}, {0, -32}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(idealThyristor1.n, resistor.p) annotation(
            Line(points = {{-20, -32}, {-60, -32}, {-60, 0}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(resistor.p, idealThyristor.p) annotation(
            Line(points = {{-60, 0}, {-60, 32}, {-20, 32}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(resistor.n, capacitor.p) annotation(
            Line(points = {{-40, 0}, {20, 0}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(idealThyristor1.fire, fire1) annotation(
            Line(points = {{-17, -43}, {-17, -92}, {-60, -92}}, color = {255, 0, 255}, smooth = Smooth.None));
          connect(idealThyristor.fire, fire1) annotation(
            Line(points = {{-3, 43}, {-3, 60}, {-80, 60}, {-80, -92}, {-60, -92}}, color = {255, 0, 255}, smooth = Smooth.None));
          connect(n, idealThyristor.p) annotation(
            Line(points = {{-100, 0}, {-90, 0}, {-90, 40}, {-20, 40}, {-20, 32}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(idealThyristor1.p, p) annotation(
            Line(points = {{0, -32}, {0, -40}, {80, -40}, {80, 0}, {104, 0}}, color = {0, 0, 255}, smooth = Smooth.None));
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{-30, 0}, {-30, -100}, {70, -50}, {-30, 0}}, lineColor = {0, 0, 0}, smooth = Smooth.None), Polygon(points = {{70, 100}, {70, 0}, {-30, 50}, {70, 100}}, lineColor = {0, 0, 0}, smooth = Smooth.None), Line(points = {{70, 0}, {70, -100}}, color = {0, 0, 0}, smooth = Smooth.None), Line(points = {{-30, 0}, {-30, 100}}, color = {0, 0, 0}, smooth = Smooth.None), Line(points = {{-30, 0}, {-90, 0}}, color = {0, 0, 0}, smooth = Smooth.None), Line(points = {{70, 0}, {110, 0}}, color = {0, 0, 0}, smooth = Smooth.None), Line(points = {{-62, -86}, {-62, -56}, {-30, -44}}, color = {0, 0, 0}, smooth = Smooth.None), Text(extent = {{-102, 130}, {98, 100}}, textString = "%name", lineColor = {0, 0, 255})}),
            Documentation(info = "<html>
<p>This is an ideal triac model based on an ideal thyristor model.</p>

<p>Two ideal thyristors (Modelica.Electrical.Analog.Ideal.IdealThyristor) are contrarily connected in parallel and additionally eliminated interference with a resistor (Rdis=100) and a capacitor (Cdis=0.005), which are connected in series.</p>

<p>The electrical component triac (TRIode Alternating Current switch) is, due to whose complex structure, a multifunctional applicable construction unit. The application area of this element is the manipulation of alternating current signals in frequency, voltage and/or current and also general blocking or filtering. However, compared to a thyristor the triac is only applied for substantial lesser currents, what is justified by whose sensitive structure. Generally one is limited to maximal voltages from 800 volt and currents from 40 ampere. For comparison maximal voltages of a thyristor are 8.000 volt and currents 5.000 ampere.</p>

<p>Structure and functionality:</p>

<p>Functionality of a triac is in principle the same like functionality of a thyristor, even connecting through of current starting from a certain voltage (knee voltage), but only if the current at anode and cathode is caused by a impulse current in the gate electrode. In case of the triac this process is also possible with reverse polarity, wherefore it is possible to control both half-waves of alternating currents. By means of gate electrodes, which are connected in a triac and why only one gate electrode is necessary, the point of time can be determined, at which the triac lets the alternating current signal pass. Thereby it is possible to affect the phase, at which the alternating current signal is cut. One speaks also of phase-angle control. Also depending on doping and specific structure knee voltage and maximal current carrying are alterable.</p>

<p>Characteristics:</p>
<ul>
<li>high switching times between on-state and off state up to activation of the reverse current phase </li>
<li>gate electrode are activated with (positive) impulse (called thyristor/triac firing), after firing thyristor path holds itself in state of low resistance or conductive state up to holding voltage is fallen below, it follows change to off state and next thyristor path can fire </li>
<li>in particular by switching of inductive components triacs generate harmonic waves, whose frequency ranges into broadcast sector and could there cause transmission disturbances; therefore triacs have to eliminate interference by inductors and capacitors </li>
</ul>
<p>Applications:</p>
<ul>
<li>any stepless exposure (dimmer) </li>
<li>engine speed adjustment of electric motors </li>
<li>further applications of phase-angle control (power electronics) </li>
<li>power packs </li>
</ul>
<p>As an additional information: this model is based on the Modelica.Electrical.Analog.Ideal.IdealThyristor.</p>
</html>", revisions = "<html>
<ul>
<li><i>November 25, 2009   </i> <br>

       by Susann Wolf <br><br>
       </li>
</ul>
</html>"));
        end IdealTriac;

        model AD_Converter "Simple n-bit analog to digital converter"
          import L = Modelica.Electrical.Digital.Interfaces.Logic;
          Modelica.Electrical.Analog.Interfaces.PositivePin p "Positive electrical pin (input)" annotation(
            Placement(transformation(extent = {{-80, 60}, {-60, 80}}, rotation = 0), iconTransformation(extent = {{-80, 60}, {-60, 80}})));
          Modelica.Electrical.Analog.Interfaces.NegativePin n "Negative electrical pin (input)" annotation(
            Placement(transformation(extent = {{-80, -80}, {-60, -60}}, rotation = 0), iconTransformation(extent = {{-80, -80}, {-60, -60}})));
          Modelica.Electrical.Digital.Interfaces.DigitalOutput y[N] "Digital output" annotation(
            Placement(transformation(extent = {{60, -10}, {80, 10}}, rotation = 0), iconTransformation(extent = {{60, -10}, {80, 10}})));
          Modelica.Electrical.Digital.Interfaces.DigitalInput trig "Trigger input" annotation(
            Placement(transformation(extent = {{-10, 60}, {10, 80}}), iconTransformation(extent = {{-10, -10}, {10, 10}}, rotation = -90, origin = {0, 90})));
          parameter Integer N(final min = 1, start = 8) "Resolution in bits - output signal width";
          parameter Modelica.SIunits.Voltage VRefHigh(start = 10) "High reference voltage";
          parameter Modelica.SIunits.Voltage VRefLow(final max = VRefHigh, start = 0) "Low reference voltage";
          parameter Modelica.SIunits.Resistance Rin(start = 10 ^ 6) "Input resistance";
          Integer z(start = 0, fixed = true);
          Real u;
        initial equation
          for i in 1:N loop
            y[i] = L.'X';
          end for;
        algorithm
          when trig == L.'1' or trig == L.'H' then
            z := if u > VRefLow then integer((u - VRefLow) / (VRefHigh - VRefLow) * (2 ^ N - 1) + 0.5) else 0;
            for i in 1:N loop
              y[i] := if mod(z, 2) > 0 then L.'1' else L.'0';
              z := div(z, 2);
            end for;
          end when;
        equation
          p.v - n.v = u;
          p.i * Rin = u;
          p.i + n.i = 0;
          annotation(
            Documentation(info = "<html>
<P>
Simple analog to digital converter with a variable resolution of n bits.
It converts the input voltage <code>ppin.v-npin.v</code> to an n-vector of type Logic
(9-valued logic according to IEEE 1164 STD_ULOGIC). The input resistance between positive and negative pin is determined by <code>Rin</code>.
Further effects (like input capacities) have to be modeled outside the converter, since this should be a general model. </P>

<P>
The input signal range (VRefLo,VRefHi) is divided into 2^n-1 equally spaced stages of length Vlsb:=(VRefHi-VRefLo)/(2^n-1).
The output signal is the binary code of <code> k </code> as long as the input voltage takes values in the k-th stage, namely in the range from
<code> Vlsb*(k-0.5) </code> to <code> m*(k+0.5) </code>. This is called mid-tread operation. Additionally the output can only change
its value if the trigger signal <CODE> trig </CODE> of type Logic changes to '1' (forced or weak).
</P>

<P>
The output vector is a 'little-endian'. i.e., that the first bit y[1] is the least significant one (LSB).
</P>

<P>
This is an abstract model of an ADC. Therefore, it can not cover the dynamic behaviour of the converter.
Hence the output will change instantaneously when the trigger signal rises.
</P>

</html>", revisions = "<html>
<ul>
<li><i> October 13, 2009   </i>
       by Matthias Franke
       </li>
</ul>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-60, 80}, {60, -80}}, lineColor = {0, 0, 255}), Polygon(points = {{-60, -80}, {60, 80}, {60, -80}, {-60, -80}}, lineColor = {0, 0, 255}, smooth = Smooth.None, fillColor = {127, 0, 127}, fillPattern = FillPattern.Solid), Text(extent = {{-60, 40}, {60, 0}}, lineColor = {0, 0, 255}, textString = "%n-bit"), Text(extent = {{-60, 0}, {60, -40}}, lineColor = {0, 0, 255}, textString = "ADC")}));
        end AD_Converter;

        model DA_Converter "Simple digital to analog converter"
          import L = Modelica.Electrical.Digital.Interfaces.Logic;
          Modelica.Electrical.Digital.Interfaces.DigitalInput trig "Trigger input" annotation(
            Placement(transformation(extent = {{-10, 60}, {10, 80}}), iconTransformation(extent = {{-10, -10}, {10, 10}}, rotation = -90, origin = {0, 90})));
          Modelica.Electrical.Digital.Interfaces.DigitalInput x[N] "Digital input" annotation(
            Placement(transformation(extent = {{-80, -10}, {-60, 10}}, rotation = 0)));
          Modelica.Electrical.Analog.Interfaces.PositivePin p "Positive electrical pin (output)" annotation(
            Placement(transformation(extent = {{60, 60}, {80, 80}}, rotation = 0), iconTransformation(extent = {{60, 60}, {80, 80}})));
          Modelica.Electrical.Analog.Interfaces.NegativePin n "Negative electrical pin (output)" annotation(
            Placement(transformation(extent = {{60, -80}, {80, -60}}, rotation = 0), iconTransformation(extent = {{60, -80}, {80, -60}})));
          Modelica.SIunits.Voltage vout(start = 0, fixed = true);
          Real y(start = 0, fixed = true);
          parameter Integer N(final min = 1, start = 8) "Resolution - input signal width";
          parameter Modelica.SIunits.Voltage Vref(start = 10) "Reference voltage";
        algorithm
          when trig == L.'1' or trig == L.'H' then
            y := 0;
            for i in 1:N loop
              y := if x[i] == L.'1' or x[i] == L.'H' then y + 2 ^ (i - 1) else y;
            end for;
            vout := y * Vref / (2 ^ N - 1);
          end when;
        equation
          p.v - n.v = vout;
          p.i + n.i = 0;
          annotation(
            Documentation(info = "<html>
<p>Simple digital to analog converter with a variable input signal width of N bits. The input signal is an N-vector of type Logic (9-valued logic according to IEEE 1164 STD_ULOGIC). The output voltage of value <code>y</code> is generated by an ideal voltage source. The output can only change if the trigger signal <code>trig</code> of type Logic changes to &#39;;1&#39;; (forced or weak). In this case, the output voltage is calculated in the following way:
</p>
<pre>       N
  y = SUM ( x[i]*2^(i-1) )*Vref/(2^N-1),
      i=1
</pre>
<p>where x[i], i=1,...,N is 1 or 0. and Vref is the reference value. Therefore, the first bit in the input vector x[1] is the least significant one (LSB) and x[N] is the most significant bit (MSB).</p>
<p>This is an abstract model of a DAC. Hence, it can not cover the dynamic behaviour of the converter. Therefore the output will change instantaneously when the trigger signal rises.</p>
</html>", revisions = "<html>
<ul>
<li><i> October 13, 2009   </i>
       by Matthias Franke
       </li>
</ul>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-60, 80}, {60, -80}}, lineColor = {0, 0, 255}), Polygon(points = {{-60, -80}, {60, 80}, {-60, 80}, {-60, -80}}, lineColor = {0, 0, 255}, smooth = Smooth.None, fillColor = {127, 0, 127}, fillPattern = FillPattern.Solid), Text(extent = {{-60, 40}, {60, 0}}, lineColor = {0, 0, 255}, textString = "%n-bit"), Text(extent = {{-60, 0}, {60, -40}}, lineColor = {0, 0, 255}, textString = "DAC")}));
        end DA_Converter;
        annotation(
          Documentation(info = "<html>
<p>This package contains electrical components with idealized behaviour. To enable more realistic applications than it is possible with pure realistic behavior some components are improved by additional features. E.g. the switches have resistances for the open or close case which can be parametrized.</p>
</html>", revisions = "<html>
<dl>
<dt>
<b>Main Authors:</b>
</dt>
<dd>
Christoph Clau&szlig;
    &lt;<a href=\"mailto:Christoph.Clauss@eas.iis.fraunhofer.de\">Christoph.Clauss@eas.iis.fraunhofer.de</a>&gt;<br>
    Andr&eacute; Schneider
    &lt;<a href=\"mailto:Andre.Schneider@eas.iis.fraunhofer.de\">Andre.Schneider@eas.iis.fraunhofer.de</a>&gt;<br>
    Fraunhofer Institute for Integrated Circuits<br>
    Design Automation Department<br>
    Zeunerstra&szlig;e 38<br>
    D-01069 Dresden
</dd>
<dt>
<b>Copyright:</b>
</dt>
<dd>
Copyright &copy; 1998-2015, Modelica Association and Fraunhofer-Gesellschaft.<br>
<i>The Modelica package is <b>free</b> software; it can be redistributed and/or modified
under the terms of the <b>Modelica license</b>, see the license conditions
and the accompanying <b>disclaimer</b> in the documentation of package
Modelica in file \"Modelica/package.mo\".</i>
</dd>
</dl>
</html>"),
          Icon(graphics = {Line(points = {{-90, 0}, {-40, 0}}, color = {0, 0, 0}), Line(points = {{-40, 0}, {32, 60}}, color = {0, 0, 0}), Line(points = {{40, 0}, {90, 0}}, color = {0, 0, 0})}));
      end Ideal;

      package Interfaces "Connectors and partial models for Analog electrical components"
        extends Modelica.Icons.InterfacesPackage;

        connector Pin "Pin of an electrical component"
          Modelica.SIunits.Voltage v "Potential at the pin" annotation(
            unassignedMessage = "An electrical potential cannot be uniquely calculated.
The reason could be that
- a ground object is missing (Modelica.Electrical.Analog.Basic.Ground)
  to define the zero potential of the electrical circuit, or
- a connector of an electrical component is not connected.");
          flow Modelica.SIunits.Current i "Current flowing into the pin" annotation(
            unassignedMessage = "An electrical current cannot be uniquely calculated.
The reason could be that
- a ground object is missing (Modelica.Electrical.Analog.Basic.Ground)
  to define the zero potential of the electrical circuit, or
- a connector of an electrical component is not connected.");
          annotation(
            defaultComponentName = "pin",
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, 100}, {100, -100}}, lineColor = {0, 0, 255}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid)}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-40, 40}, {40, -40}}, lineColor = {0, 0, 255}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-160, 110}, {40, 50}}, lineColor = {0, 0, 255}, textString = "%name")}),
            Documentation(revisions = "<html>
<ul>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>", info = "<html>
<p>Pin is the basic electric connector. It includes the voltage which consists between the pin and the ground node. The ground node is the node of (any) ground device (Modelica.Electrical.Basic.Ground). Furthermore, the pin includes the current, which is considered to be <b>positive</b> if it is flowing at the pin<b> into the device</b>.</p>
</html>"));
        end Pin;

        connector PositivePin "Positive pin of an electric component"
          Modelica.SIunits.Voltage v "Potential at the pin" annotation(
            unassignedMessage = "An electrical potential cannot be uniquely calculated.
The reason could be that
- a ground object is missing (Modelica.Electrical.Analog.Basic.Ground)
  to define the zero potential of the electrical circuit, or
- a connector of an electrical component is not connected.");
          flow Modelica.SIunits.Current i "Current flowing into the pin" annotation(
            unassignedMessage = "An electrical current cannot be uniquely calculated.
The reason could be that
- a ground object is missing (Modelica.Electrical.Analog.Basic.Ground)
  to define the zero potential of the electrical circuit, or
- a connector of an electrical component is not connected.");
          annotation(
            defaultComponentName = "pin_p",
            Documentation(info = "<html>
<p>Connectors PositivePin and NegativePin are nearly identical. The only difference is that the icons are different in order to identify more easily the pins of a component. Usually, connector PositivePin is used for the positive and connector NegativePin for the negative pin of an electrical component.</p>
</html>", revisions = "<html>
<ul>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, 100}, {100, -100}}, lineColor = {0, 0, 255}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid)}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-40, 40}, {40, -40}}, lineColor = {0, 0, 255}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-160, 110}, {40, 50}}, lineColor = {0, 0, 255}, textString = "%name")}));
        end PositivePin;

        connector NegativePin "Negative pin of an electric component"
          Modelica.SIunits.Voltage v "Potential at the pin" annotation(
            unassignedMessage = "An electrical potential cannot be uniquely calculated.
The reason could be that
- a ground object is missing (Modelica.Electrical.Analog.Basic.Ground)
  to define the zero potential of the electrical circuit, or
- a connector of an electrical component is not connected.");
          flow Modelica.SIunits.Current i "Current flowing into the pin" annotation(
            unassignedMessage = "An electrical current cannot be uniquely calculated.
The reason could be that
- a ground object is missing (Modelica.Electrical.Analog.Basic.Ground)
  to define the zero potential of the electrical circuit, or
- a connector of an electrical component is not connected.");
          annotation(
            defaultComponentName = "pin_n",
            Documentation(info = "<html>
<p>Connectors PositivePin and NegativePin are nearly identical. The only difference is that the icons are different in order to identify more easily the pins of a component. Usually, connector PositivePin is used for the positive and connector NegativePin for the negative pin of an electrical component.</p>
</html>", revisions = "<html>
<dl>
<dt><i>1998</i></dt>
<dd>by Christoph Clauss initially implemented
</dd>
</dl>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, 100}, {100, -100}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid)}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-40, 40}, {40, -40}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-40, 110}, {160, 50}}, textString = "%name", lineColor = {0, 0, 255})}));
        end NegativePin;

        partial model TwoPin "Component with two electrical pins"
          SI.Voltage v "Voltage drop between the two pins (= p.v - n.v)";
          PositivePin p "Positive pin Positive pin (potential p.v > n.v for positive voltage drop v)" annotation(
            Placement(transformation(extent = {{-110, -10}, {-90, 10}}, rotation = 0)));
          NegativePin n "Negative pin" annotation(
            Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
        equation
          v = p.v - n.v;
          annotation(
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{-120, 3}, {-110, 0}, {-120, -3}, {-120, 3}}, lineColor = {160, 160, 164}, fillColor = {160, 160, 164}, fillPattern = FillPattern.Solid), Line(points = {{-136, 0}, {-111, 0}}, color = {160, 160, 164}), Text(extent = {{-134, 5}, {-117, 20}}, lineColor = {160, 160, 164}, textString = "p.i"), Line(points = {{110, 0}, {135, 0}}, color = {160, 160, 164}), Polygon(points = {{120, 3}, {110, 0}, {120, -3}, {120, 3}}, lineColor = {160, 160, 164}, fillColor = {160, 160, 164}, fillPattern = FillPattern.Solid), Text(extent = {{117, 3}, {134, 18}}, lineColor = {160, 160, 164}, textString = "n.i")}),
            Documentation(revisions = "<html>
<ul>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>", info = "<html>
<p>TwoPin is a partial model with two pins and one internal variable for the voltage over the two pins. Internal currents are not defined. It is intended to be used in cases where the model which inherits TwoPin is composed by combining other components graphically, not by equations.</p>
</html>"));
        end TwoPin;

        partial model OnePort "Component with two electrical pins p and n and current i from p to n"
          SI.Voltage v "Voltage drop between the two pins (= p.v - n.v)";
          SI.Current i "Current flowing from pin p to pin n";
          PositivePin p "Positive pin (potential p.v > n.v for positive voltage drop v)" annotation(
            Placement(transformation(extent = {{-110, -10}, {-90, 10}}, rotation = 0)));
          NegativePin n "Negative pin" annotation(
            Placement(transformation(extent = {{110, -10}, {90, 10}}, rotation = 0)));
        equation
          v = p.v - n.v;
          0 = p.i + n.i;
          i = p.i;
          annotation(
            Documentation(info = "<html>
<p>Superclass of elements which have <b>two</b> electrical pins: the positive pin connector <i>p</i>, and the negative pin connector <i>n</i>. It is assumed that the current flowing into pin p is identical to the current flowing out of pin n. This current is provided explicitly as current i.</p>
</html>", revisions = "<html>
<ul>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>"),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-110, 20}, {-85, 20}}, color = {160, 160, 164}), Polygon(points = {{-95, 23}, {-85, 20}, {-95, 17}, {-95, 23}}, lineColor = {160, 160, 164}, fillColor = {160, 160, 164}, fillPattern = FillPattern.Solid), Line(points = {{90, 20}, {115, 20}}, color = {160, 160, 164}), Line(points = {{-125, 0}, {-115, 0}}, color = {160, 160, 164}), Line(points = {{-120, -5}, {-120, 5}}, color = {160, 160, 164}), Text(extent = {{-110, 25}, {-90, 45}}, lineColor = {160, 160, 164}, textString = "i"), Polygon(points = {{105, 23}, {115, 20}, {105, 17}, {105, 23}}, lineColor = {160, 160, 164}, fillColor = {160, 160, 164}, fillPattern = FillPattern.Solid), Line(points = {{115, 0}, {125, 0}}, color = {160, 160, 164}), Text(extent = {{90, 45}, {110, 25}}, lineColor = {160, 160, 164}, textString = "i")}));
        end OnePort;

        partial model TwoPort "Component with two electrical ports, including current"
          SI.Voltage v1 "Voltage drop over the left port";
          SI.Voltage v2 "Voltage drop over the right port";
          SI.Current i1 "Current flowing from pos. to neg. pin of the left port";
          SI.Current i2 "Current flowing from pos. to neg. pin of the right port";
          PositivePin p1 "Positive pin of the left port (potential p1.v > n1.v for positive voltage drop v1)" annotation(
            Placement(transformation(extent = {{-110, 40}, {-90, 60}}, rotation = 0)));
          NegativePin n1 "Negative pin of the left port" annotation(
            Placement(transformation(extent = {{-90, -60}, {-110, -40}}, rotation = 0)));
          PositivePin p2 "Positive pin of the right port (potential p2.v > n2.v for positive voltage drop v2)" annotation(
            Placement(transformation(extent = {{110, 40}, {90, 60}}, rotation = 0)));
          NegativePin n2 "Negative pin of the right port" annotation(
            Placement(transformation(extent = {{90, -60}, {110, -40}}, rotation = 0)));
        equation
          v1 = p1.v - n1.v;
          v2 = p2.v - n2.v;
          0 = p1.i + n1.i;
          0 = p2.i + n2.i;
          i1 = p1.i;
          i2 = p2.i;
          annotation(
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{-120, 53}, {-110, 50}, {-120, 47}, {-120, 53}}, lineColor = {160, 160, 164}, fillColor = {160, 160, 164}, fillPattern = FillPattern.Solid), Line(points = {{-136, 50}, {-111, 50}}, color = {160, 160, 164}), Polygon(points = {{127, -47}, {137, -50}, {127, -53}, {127, -47}}, lineColor = {160, 160, 164}, fillColor = {160, 160, 164}, fillPattern = FillPattern.Solid), Line(points = {{111, -50}, {136, -50}}, color = {160, 160, 164}), Text(extent = {{112, -44}, {128, -29}}, lineColor = {160, 160, 164}, textString = "i2"), Text(extent = {{118, 52}, {135, 67}}, lineColor = {0, 0, 0}, textString = "i2"), Polygon(points = {{120, 53}, {110, 50}, {120, 47}, {120, 53}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {160, 160, 164}), Line(points = {{111, 50}, {136, 50}}, color = {0, 0, 0}), Line(points = {{-136, -49}, {-111, -49}}, color = {160, 160, 164}), Polygon(points = {{-126, -46}, {-136, -49}, {-126, -52}, {-126, -46}}, lineColor = {160, 160, 164}, fillColor = {160, 160, 164}, fillPattern = FillPattern.Solid), Text(extent = {{-127, -46}, {-110, -31}}, lineColor = {160, 160, 164}, textString = "i1"), Text(extent = {{-136, 53}, {-119, 68}}, lineColor = {160, 160, 164}, textString = "i1")}),
            Documentation(revisions = "<html>
<ul>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>", info = "<html>
<p>TwoPort is a partial model that consists of two ports. Like OnePort each port has two pins. It is assumed that the current flowing into the positive  pin   is identical to the current flowing out of pin n. This currents of each port are  provided explicitly as currents i1 and i2, the voltages respectively as v1 and v2.</p>
</html>"));
        end TwoPort;

        partial model ConditionalHeatPort "Partial model to include a conditional HeatPort in order to describe the power loss via a thermal network"
          parameter Boolean useHeatPort = false "=true, if HeatPort is enabled" annotation(
            Evaluate = true,
            HideResult = true,
            choices(checkBox = true));
          parameter Modelica.SIunits.Temperature T = 293.15 "Fixed device temperature if useHeatPort = false" annotation(
            Dialog(enable = not useHeatPort));
          Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a heatPort(T(start = T) = T_heatPort, Q_flow = -LossPower) if useHeatPort annotation(
            Placement(transformation(extent = {{-10, -110}, {10, -90}}), iconTransformation(extent = {{-10, -110}, {10, -90}})));
          Modelica.SIunits.Power LossPower "Loss power leaving component via HeatPort";
          Modelica.SIunits.Temperature T_heatPort "Temperature of HeatPort";
        equation
          if not useHeatPort then
            T_heatPort = T;
          end if;
          annotation(
            Documentation(revisions = "<html>
<ul>
<li><i> February 17, 2009   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>", info = "<html>
<p>
This partial model provides a conditional heating port for the connection to a thermal network.
</p>
<ul>
<li> If <b>useHeatPort</b> is set to <b>false</b> (default), no heat port is available, and the thermal
     loss power flows internally to the ground. In this case, the parameter <b>T</b> specifies
     the fixed device temperature (the default for T = 20<sup>o</sup>C).</li>
<li> If <b>useHeatPort</b> is set to <b>true</b>, a heat port is available.</li>
</ul>

<p>
If this model is used, the loss power has to be provided by an equation in the model which inherits from
ConditionalHeatingPort model (<b>lossPower = ...</b>). As device temperature
<b>T_heatPort</b> can be used to describe the influence of the device temperature
on the model behaviour.
</p>
</html>"));
        end ConditionalHeatPort;

        partial model AbsoluteSensor "Base class to measure the absolute value of a pin variable"
          extends Modelica.Icons.RotationalSensor;
          Interfaces.PositivePin p "Pin to be measured" annotation(
            Placement(transformation(extent = {{-110, -10}, {-90, 10}}, rotation = 0)));
          Modelica.Blocks.Interfaces.RealOutput y "Measured quantity as Real output signal" annotation(
            Placement(transformation(extent = {{100, -10}, {120, 10}}, rotation = 0)));
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-70, 0}, {-90, 0}}, color = {0, 0, 0}), Line(points = {{70, 0}, {100, 0}}, color = {0, 0, 255}), Text(extent = {{-150, 80}, {150, 120}}, textString = "%name", lineColor = {0, 0, 255})}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-70, 0}, {-96, 0}}, color = {0, 0, 0}), Line(points = {{70, 0}, {100, 0}}, color = {0, 0, 255})}),
            Documentation(revisions = "<html>
<ul>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>", info = "<html>
<p>The AbsoluteSensor is a partial model for converting values that can be calculated from one pin connector into a real valued signal. The special calculation has to be described in the model which inherits the AbsoluteSensor.  It is often used in sensor devices. To be a true sensor the modeller has to take care that the sensor model does not influence the electrical behavior to be measured.</p>
</html>"));
        end AbsoluteSensor;

        partial model RelativeSensor "Base class to measure a relative variable between two pins"
          extends Modelica.Icons.RotationalSensor;
          Interfaces.PositivePin p "Positive pin" annotation(
            Placement(transformation(extent = {{-110, -10}, {-90, 10}}, rotation = 0)));
          Interfaces.NegativePin n "Negative pin" annotation(
            Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
          Modelica.Blocks.Interfaces.RealOutput y "Measured quantity as Real output signal" annotation(
            Placement(transformation(origin = {0, -100}, extent = {{10, -10}, {-10, 10}}, rotation = 90)));
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-70, 0}, {-90, 0}}, color = {0, 0, 0}), Line(points = {{70, 0}, {90, 0}}, color = {0, 0, 0}), Line(points = {{0, -90}, {0, -70}}, color = {0, 0, 255}), Text(extent = {{-150, 80}, {150, 120}}, textString = "%name", lineColor = {0, 0, 255})}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-70, 0}, {-96, 0}}, color = {0, 0, 0}), Line(points = {{0, -90}, {0, -70}}, color = {0, 0, 255}), Line(points = {{70, 0}, {96, 0}}, color = {0, 0, 0})}),
            Documentation(revisions = "<html>
<ul>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>", info = "<html>
<p>The RelaticeSensor is a partial model for converting values that can be calculated from two pin connectors into a real valued signal. The special calculation has to be described in the model which inherits the RelativeSensor.  It is often used in sensor devices. To be a true sensor the modeller has to take care that the sensor model does not influence the electrical behavior to be measured.</p>
</html>"));
        end RelativeSensor;

        partial model VoltageSource "Interface for voltage sources"
          extends OnePort;
          parameter SI.Voltage offset = 0 "Voltage offset";
          parameter SI.Time startTime = 0 "Time offset";
          replaceable Modelica.Blocks.Interfaces.SignalSource signalSource(final offset = offset, final startTime = startTime) annotation(
            Placement(transformation(extent = {{70, 70}, {90, 90}}, rotation = 0)));
        equation
          v = signalSource.y;
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-50, 50}, {50, -50}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-150, 80}, {150, 120}}, textString = "%name", lineColor = {0, 0, 255}), Line(points = {{-90, 0}, {90, 0}}, color = {0, 0, 255}), Text(extent = {{-120, 50}, {-20, 0}}, lineColor = {0, 0, 255}, textString = "+"), Text(extent = {{20, 50}, {120, 0}}, lineColor = {0, 0, 255}, textString = "-")}),
            Documentation(revisions = "<html>
<ul>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>", info = "<html>
<p>The VoltageSource partial model prepares voltage sources by providing the pins, and the offset and startTime parameters, which are the same at all voltage sources. The source behavior is taken from Modelica.Blocks signal sources by inheritance and usage of the replaceable possibilities.</p>
</html>"));
        end VoltageSource;

        partial model CurrentSource "Interface for current sources"
          extends OnePort;
          parameter SI.Current offset = 0 "Current offset";
          parameter SI.Time startTime = 0 "Time offset";
          replaceable Modelica.Blocks.Interfaces.SignalSource signalSource(final offset = offset, final startTime = startTime) annotation(
            Placement(transformation(extent = {{70, 69}, {91, 89}}, rotation = 0)));
        equation
          i = signalSource.y;
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-50, 50}, {50, -50}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{-90, 0}, {-50, 0}}, color = {0, 0, 255}), Line(points = {{50, 0}, {90, 0}}, color = {0, 0, 255}), Line(points = {{0, -50}, {0, 50}}, color = {0, 0, 255}), Text(extent = {{-150, 120}, {150, 80}}, textString = "%name", lineColor = {0, 0, 255}), Polygon(points = {{90, 0}, {60, 10}, {60, -10}, {90, 0}}, lineColor = {0, 0, 255}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid)}),
            Documentation(revisions = "<html>
<ul>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>", info = "<html>
<p>The CurrentSource partial model prepares current sources by providing the pins, and the offset and startTime parameters, which are the same at all current sources. The source behavior is taken from Modelica.Blocks signal sources by inheritance and usage of the replaceable possibilities.</p>
</html>"));
        end CurrentSource;
        annotation(
          Documentation(info = "<html>
<p>This package contains connectors and interfaces (partial models) for analog electrical components. The partial models contain typical combinations of pins, and internal variables which are often used. Furthermore, the thermal heat port is in this package which can be included by inheritance.</p>
</html>", revisions = "<html>
<dl>
<dt>
<b>Main Authors:</b>
</dt>
<dd>
Christoph Clau&szlig;
    &lt;<a href=\"mailto:Christoph.Clauss@eas.iis.fraunhofer.de\">Christoph.Clauss@eas.iis.fraunhofer.de</a>&gt;<br>
    Andr&eacute; Schneider
    &lt;<a href=\"mailto:Andre.Schneider@eas.iis.fraunhofer.de\">Andre.Schneider@eas.iis.fraunhofer.de</a>&gt;<br>
    Fraunhofer Institute for Integrated Circuits<br>
    Design Automation Department<br>
    Zeunerstra&szlig;e 38<br>
    D-01069 Dresden
</dd>
<dt>
<b>Copyright:</b>
</dt>
<dd>
Copyright &copy; 1998-2015, Modelica Association and Fraunhofer-Gesellschaft.<br>
<i>The Modelica package is <b>free</b> software; it can be redistributed and/or modified
under the terms of the <b>Modelica license</b>, see the license conditions
and the accompanying <b>disclaimer</b> in the documentation of package
Modelica in file \"Modelica/package.mo\".</i>
</dd>
</dl>

<ul>
<li><i> 1998</i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>"));
      end Interfaces;

      package Lines "Lossy and lossless segmented transmission lines, and LC distributed line models"
        extends Modelica.Icons.Package;

        model OLine "Lossy Transmission Line"
          //extends Interfaces.ThreePol;
          Modelica.Electrical.Analog.Interfaces.Pin p1 annotation(
            Placement(transformation(extent = {{-110, -10}, {-90, 10}}, rotation = 0)));
          Modelica.Electrical.Analog.Interfaces.Pin p2 annotation(
            Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
          Modelica.Electrical.Analog.Interfaces.Pin p3 annotation(
            Placement(transformation(extent = {{-10, -110}, {10, -90}}, rotation = 0)));
          Modelica.SIunits.Voltage v13;
          Modelica.SIunits.Voltage v23;
          Modelica.SIunits.Current i1;
          Modelica.SIunits.Current i2;
          parameter Real r(final min = Modelica.Constants.small, unit = "Ohm/m", start = 1) "Resistance per meter";
          parameter Real l(final min = Modelica.Constants.small, unit = "H/m", start = 1) "Inductance per meter";
          parameter Real g(final min = Modelica.Constants.small, unit = "S/m", start = 1) "Conductance per meter";
          parameter Real c(final min = Modelica.Constants.small, unit = "F/m", start = 1) "Capacitance per meter";
          parameter Modelica.SIunits.Length length(final min = Modelica.Constants.small, start = 1) "Length of line";
          parameter Integer N(final min = 1, start = 1) "Number of lumped segments";
          parameter Modelica.SIunits.LinearTemperatureCoefficient alpha_R = 0 "Temperature coefficient of resistance (R_actual = R*(1 + alpha*(T_heatPort - T_ref))";
          parameter Modelica.SIunits.LinearTemperatureCoefficient alpha_G = 0 "Temperature coefficient of conductance (G_actual = G_ref/(1 + alpha*(T_heatPort - T_ref))";
          parameter Boolean useHeatPort = false "=true, if HeatPort is enabled" annotation(
            Evaluate = true,
            HideResult = true,
            choices(checkBox = true));
          parameter Modelica.SIunits.Temperature T = 293.15 "Fixed device temperature if useHeatPort = false" annotation(
            Dialog(enable = not useHeatPort));
          parameter Modelica.SIunits.Temperature T_ref = 300.15;
          Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a heatPort if useHeatPort annotation(
            Placement(transformation(extent = {{-10, -110}, {10, -90}}), iconTransformation(extent = {{-80, -80}, {-60, -60}})));
        protected
          Modelica.Electrical.Analog.Basic.Resistor R[N + 1](R = fill(r * length / (N + 1), N + 1), T_ref = fill(T_ref, N + 1), alpha = fill(alpha_R, N + 1), useHeatPort = fill(useHeatPort, N + 1), T = fill(T, N + 1));
          Modelica.Electrical.Analog.Basic.Inductor L[N + 1](L = fill(l * length / (N + 1), N + 1));
          Modelica.Electrical.Analog.Basic.Capacitor C[N](C = fill(c * length / N, N));
          Modelica.Electrical.Analog.Basic.Conductor G[N](G = fill(g * length / N, N), T_ref = fill(T_ref, N), alpha = fill(alpha_G, N), useHeatPort = fill(useHeatPort, N), T = fill(T, N));
        equation
          v13 = p1.v - p3.v;
          v23 = p2.v - p3.v;
          i1 = p1.i;
          i2 = p2.i;
          connect(p1, R[1].p);
          for i in 1:N loop
            connect(R[i].n, L[i].p);
            connect(L[i].n, C[i].p);
            connect(L[i].n, G[i].p);
            connect(C[i].n, p3);
            connect(G[i].n, p3);
            connect(L[i].n, R[i + 1].p);
          end for;
          connect(R[N + 1].n, L[N + 1].p);
          connect(L[N + 1].n, p2);
          if useHeatPort then
            for i in 1:N + 1 loop
              connect(heatPort, R[i].heatPort);
            end for;
            for i in 1:N loop
              connect(heatPort, G[i].heatPort);
            end for;
          end if;
          annotation(
            Documentation(info = "<html>
<p>Like in the picture below, the lossy transmission line OLine is a single-conductor lossy transmission line which consists of segments of lumped resistors and inductors in series and conductor and capacitors that are connected with the reference pin p3. The precision of the model depends on the number N of lumped segments.</p>
<p>To get a symmetric line model, the first resistor and inductor are cut into two parts (R1 and R_Nplus1, L1 and L_Nplus1). These two new resistors and inductors have the half of the resistance respectively inductance the original resistor respectively inductor.</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Electrical/Analog/OLine.png\"
     alt=\"OLine.png\">
</p>

<p>The capacitances are calculated with: C=c*length/N.
<br> The conductances are calculated with: G=g*length/N.
<br> The resistances are calculated with : R=r*length/(N+1).
<br> The inductances are calculated with : L=l*length/(N+1).
<br> For all capacitors, conductors, resistors and inductors the values of each segment are the same except of the first and last resistor and inductor, that only have the half of the above calculated value of the rest.
<br><p>The user has the possibility to enable a conditional heatport. If so, the OLine can be connected to a thermal network. When the parameter alpha is set to an value greater then zero, the OLine becomes temperature sensitive</p><p>due to their resistors which resistances are calculated by <code>R = R_ref*(1 + alpha*(heatPort.T - T_ref))</code> and conductors calculated by <code> (G_actual = G_ref/(1 + alpha*(T_heatPort - T_ref)).</code> </p>
<p>Note, this is different to the lumped line model of SPICE.</p>

<dl><dt><b>References:</b> </dt>
<dd>Johnson, B.; Quarles, T.; Newton, A. R.; Pederson, D. O.; Sangiovanni-Vincentelli, A.: SPICE3 Version 3e User&#39;;s Manual (April 1, 1991). Department of Electrical Engineering and Computer Sciences, University of California, Berkley p. 12, p. 106 - 107 </dd>
</dl></html>", revisions = "<html>
<ul>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-60, 60}, {60, -60}}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Line(points = {{0, -60}, {0, -90}}, color = {0, 0, 255}), Line(points = {{60, 0}, {90, 0}}, color = {0, 0, 255}), Line(points = {{-60, 0}, {-90, 0}}, color = {0, 0, 255}), Line(points = {{30, 30}, {-30, 30}}, color = {0, 0, 255}), Line(points = {{-30, 40}, {-30, 20}}, color = {0, 0, 255}), Line(points = {{30, 40}, {30, 20}}, color = {0, 0, 255}), Text(extent = {{-155, 112}, {145, 72}}, textString = "%name", lineColor = {0, 0, 255})}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-60, 60}, {60, -60}}, lineColor = {0, 0, 255}), Line(points = {{0, -60}, {0, -96}}, color = {0, 0, 255}), Line(points = {{60, 0}, {96, 0}}, color = {0, 0, 255}), Line(points = {{-60, 0}, {-96, 0}}, color = {0, 0, 255}), Line(points = {{30, 30}, {-30, 30}}, color = {0, 0, 255}), Line(points = {{-30, 40}, {-30, 20}}, color = {0, 0, 255}), Line(points = {{30, 40}, {30, 20}}, color = {0, 0, 255})}));
        end OLine;

        model M_OLine "Multiple OLine"
          parameter Modelica.SIunits.Length length(final min = Modelica.Constants.small) = 0.1 "Length of line";
          parameter Integer N(final min = 2) = 5 "Number of lumped segments";
          parameter Integer lines(final min = 2) = 4 "Number of lines";
        protected
          parameter Integer dim_vector_lgc = div(lines * (lines + 1), 2);
        public
          parameter Real r[lines](each final min = Modelica.Constants.small, each unit = "Ohm/m") = {4.76e5, 1.72e5, 1.72e5, 1.72e5} "Resistance per meter";
          parameter Real l[dim_vector_lgc](each final min = Modelica.Constants.small, each unit = "H/m") = {5.98e-7, 4.44e-7, 4.39e-7, 3.99e-7, 5.81e-7, 4.09e-7, 4.23e-7, 5.96e-7, 4.71e-7, 6.06e-7} "Inductance per meter";
          parameter Real g[dim_vector_lgc](each final min = Modelica.Constants.small, each unit = "S/m") = {8.05e-6, 3.42e-5, 2 - 91e-5, 1.76e-6, 9.16e-6, 7.12e-6, 2.43e-5, 5.93e-6, 4.19e-5, 6.64e-6} "Conductance per meter";
          parameter Real c[dim_vector_lgc](each final min = Modelica.Constants.small, each unit = "F/m") = {2.38e-11, 1.01e-10, 8.56e-11, 5.09e-12, 2.71e-11, 2.09e-11, 7.16e-11, 1.83e-11, 1.23e-10, 2.07e-11} "Capacitance per meter";
          parameter Modelica.SIunits.LinearTemperatureCoefficient alpha_R = 0 "Temperature coefficient of resistance (R_actual = R*(1 + alpha*(T_heatPort - T_ref))";
          parameter Modelica.SIunits.LinearTemperatureCoefficient alpha_G = 0 "Temperature coefficient of conductance (G_actual = G_ref/(1 + alpha*(T_heatPort - T_ref))";
          parameter Boolean useHeatPort = false "=true, if HeatPort is enabled" annotation(
            Evaluate = true,
            HideResult = true,
            choices(checkBox = true));
          parameter Modelica.SIunits.Temperature T = 293.15 "Fixed device temperature if useHeatPort = false" annotation(
            Dialog(enable = not useHeatPort));
          parameter Modelica.SIunits.Temperature T_ref = 300.15;
          Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a heatPort if useHeatPort annotation(
            Placement(transformation(extent = {{-10, -110}, {10, -90}}), iconTransformation(extent = {{-80, -80}, {-60, -60}})));

          model segment "Multiple line segment model"
            parameter Integer lines(final min = 1) = 3 "Number of lines";
            parameter Integer dim_vector_lgc = div(lines * (lines + 1), 2) "Length of the vectors for l, g, c";
            Modelica.Electrical.Analog.Interfaces.PositivePin p[lines] "Positive pin" annotation(
              Placement(transformation(extent = {{-60, -10}, {-40, 10}}, rotation = 0)));
            Modelica.Electrical.Analog.Interfaces.NegativePin n[lines] "Negative pin" annotation(
              Placement(transformation(extent = {{40, -10}, {60, 10}}, rotation = 0)));
            parameter Real Cl[dim_vector_lgc] = fill(1, dim_vector_lgc) "Capacitance matrix";
            parameter Real Rl[lines] = fill(7, lines) "Resistance matrix";
            parameter Real Ll[dim_vector_lgc] = fill(2, dim_vector_lgc) "Inductance matrix";
            parameter Real Gl[dim_vector_lgc] = fill(1, dim_vector_lgc) "Conductance matrix";
            parameter Modelica.SIunits.LinearTemperatureCoefficient alpha_R "Temperature coefficient of resistance (R_actual = R*(1 + alpha*(T_heatPort - T_ref))";
            parameter Modelica.SIunits.LinearTemperatureCoefficient alpha_G "Temperature coefficient of conductance (G_actual = G_ref/(1 + alpha*(T_heatPort - T_ref))";
            parameter Boolean useHeatPort = false "=true, if HeatPort is enabled" annotation(
              Evaluate = true,
              HideResult = true,
              choices(checkBox = true));
            parameter Modelica.SIunits.Temperature T = 293.15 "Fixed device temperature if useHeatPort = false" annotation(
              Dialog(enable = not useHeatPort));
            parameter Modelica.SIunits.Temperature T_ref;
            Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a heatPort if useHeatPort annotation(
              Placement(transformation(extent = {{-10, -110}, {10, -90}}), iconTransformation(extent = {{-80, -80}, {-60, -60}})));
            Modelica.Electrical.Analog.Basic.Capacitor C[dim_vector_lgc](C = Cl);
            Modelica.Electrical.Analog.Basic.Resistor R[lines](R = Rl, T_ref = fill(T_ref, lines), alpha = fill(alpha_R, lines), useHeatPort = fill(useHeatPort, lines), T = fill(T, lines));
            Modelica.Electrical.Analog.Basic.Conductor G[dim_vector_lgc](G = Gl, T_ref = fill(T_ref, dim_vector_lgc), alpha = fill(alpha_G, dim_vector_lgc), useHeatPort = fill(useHeatPort, dim_vector_lgc), T = fill(T, dim_vector_lgc));
            Modelica.Electrical.Analog.Basic.M_Transformer inductance(N = lines, L = Ll);
            Modelica.Electrical.Analog.Basic.Ground M;
          equation
            for j in 1:lines - 1 loop
              connect(R[j].p, p[j]);
              connect(R[j].n, inductance.p[j]);
              connect(inductance.n[j], n[j]);
              connect(inductance.n[j], C[1 + (j - 1) * lines - div((j - 2) * (j - 1), 2)].p);
              connect(C[1 + (j - 1) * lines - div((j - 2) * (j - 1), 2)].n, M.p);
              connect(inductance.n[j], G[1 + (j - 1) * lines - div((j - 2) * (j - 1), 2)].p);
              connect(G[1 + (j - 1) * lines - div((j - 2) * (j - 1), 2)].n, M.p);
              for i in j + 1:lines loop
                connect(inductance.n[j], C[1 + (j - 1) * lines - div((j - 2) * (j - 1), 2) + 1 + i - (j + 1)].p);
                connect(C[1 + (j - 1) * lines - div((j - 2) * (j - 1), 2) + 1 + i - (j + 1)].n, inductance.n[i]);
                connect(inductance.n[j], G[1 + (j - 1) * lines - div((j - 2) * (j - 1), 2) + 1 + i - (j + 1)].p);
                connect(G[1 + (j - 1) * lines - div((j - 2) * (j - 1), 2) + 1 + i - (j + 1)].n, inductance.n[i]);
              end for;
            end for;
            connect(R[lines].p, p[lines]);
            connect(R[lines].n, inductance.p[lines]);
            connect(inductance.n[lines], n[lines]);
            connect(inductance.n[lines], C[dim_vector_lgc].p);
            connect(C[dim_vector_lgc].n, M.p);
            connect(inductance.n[lines], G[dim_vector_lgc].p);
            connect(G[dim_vector_lgc].n, M.p);
            if useHeatPort then
              for j in 1:lines - 1 loop
                connect(heatPort, R[j].heatPort);
                connect(heatPort, G[1 + (j - 1) * lines - div((j - 2) * (j - 1), 2)].heatPort);
                for i in j + 1:lines loop
                  connect(heatPort, G[1 + (j - 1) * lines - div((j - 2) * (j - 1), 2) + 1 + i - (j + 1)].heatPort);
                end for;
              end for;
              connect(heatPort, R[lines].heatPort);
              connect(heatPort, G[dim_vector_lgc].heatPort);
            end if;
            annotation(
              Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{40, -40}, {-40, 40}}, lineColor = {0, 0, 255})}),
              Documentation(info = "<html>
<p>The segment model is part of the multiple line model. It describes one line segment as outlined in the M_OLine description. Using the loop possibilities of Modelica it is formulated by connecting components the number of which depends on the number of lines.</p>
</html>"));
          end segment;

          model segment_last "Multiple line last segment model"
            Modelica.Electrical.Analog.Interfaces.PositivePin p[lines] "Positive pin" annotation(
              Placement(transformation(extent = {{-40, -10}, {-20, 10}}, rotation = 0)));
            Modelica.Electrical.Analog.Interfaces.NegativePin n[lines] "Negative pin" annotation(
              Placement(transformation(extent = {{20, -10}, {40, 10}}, rotation = 0)));
            parameter Integer lines(final min = 1) = 3 "Number of lines";
            parameter Integer dim_vector_lgc = div(lines * (lines + 1), 2) "Length of the vectors for l, g, c";
            parameter Real Rl[lines] = fill(1, lines) "Resistance matrix";
            parameter Real Ll[dim_vector_lgc] = fill(1, dim_vector_lgc) "Inductance matrix";
            parameter Modelica.SIunits.LinearTemperatureCoefficient alpha_R "Temperature coefficient of resistance (R_actual = R*(1 + alpha*(T_heatPort - T_ref))";
            parameter Boolean useHeatPort = false "=true, if HeatPort is enabled" annotation(
              Evaluate = true,
              HideResult = true,
              choices(checkBox = true));
            parameter Modelica.SIunits.Temperature T = 293.15 "Fixed device temperature if useHeatPort = false" annotation(
              Dialog(enable = not useHeatPort));
            parameter Modelica.SIunits.Temperature T_ref;
            Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a heatPort if useHeatPort annotation(
              Placement(transformation(extent = {{-10, -110}, {10, -90}}), iconTransformation(extent = {{-80, -80}, {-60, -60}})));
            Modelica.Electrical.Analog.Basic.Resistor R[lines](R = Rl, T_ref = fill(T_ref, lines), useHeatPort = fill(useHeatPort, lines), T = fill(T, lines));
            Modelica.Electrical.Analog.Basic.M_Transformer inductance(N = lines, L = Ll);
            Modelica.Electrical.Analog.Basic.Ground M;
          equation
            for j in 1:lines - 1 loop
              connect(p[j], inductance.p[j]);
              connect(inductance.n[j], R[j].p);
              connect(R[j].n, n[j]);
            end for;
            connect(p[lines], inductance.p[lines]);
            connect(inductance.n[lines], R[lines].p);
            connect(R[lines].n, n[lines]);
            if useHeatPort then
              for j in 1:lines - 1 loop
                connect(heatPort, R[j].heatPort);
              end for;
              connect(heatPort, R[lines].heatPort);
            end if;
            annotation(
              Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{20, -40}, {-20, 40}}, lineColor = {0, 0, 255})}),
              Documentation(info = "<html>
<p>The segment_last model is part of the multiple line model. It describes the special  line segment which is used to get the line symmetrical as outlined in the M_OLine description. Using the loop possibilities of Modelica it is formulated by connecting components the number of which depends on the number of lines.</p>
</html>"));
          end segment_last;

          segment s[N - 1](lines = fill(lines, N - 1), dim_vector_lgc = fill(dim_vector_lgc, N - 1), Rl = fill(r * length / N, N - 1), Ll = fill(l * length / N, N - 1), Cl = fill(c * length / N, N - 1), Gl = fill(g * length / N, N - 1), alpha_R = fill(alpha_R, N - 1), alpha_G = fill(alpha_G, N - 1), T_ref = fill(T_ref, N - 1), useHeatPort = fill(useHeatPort, N - 1), T = fill(T, N - 1));
          segment s_first(lines = lines, dim_vector_lgc = dim_vector_lgc, Rl = r * length / (2 * N), Cl = c * length / N, Ll = l * length / (2 * N), Gl = g * length / N, alpha_R = alpha_R, alpha_G = alpha_G, T_ref = T_ref, useHeatPort = useHeatPort, T = T);
          segment_last s_last(lines = lines, Rl = r * length / (2 * N), Ll = l * length / (2 * N), alpha_R = alpha_R, T_ref = T_ref, useHeatPort = useHeatPort, T = T);
          Modelica.Electrical.Analog.Interfaces.PositivePin p[lines] "Positive pin" annotation(
            Placement(transformation(extent = {{-100, -80}, {-80, 80}}, rotation = 0)));
          Modelica.Electrical.Analog.Interfaces.NegativePin n[lines] "Negative pin" annotation(
            Placement(transformation(extent = {{80, -80}, {100, 80}}, rotation = 0)));
        equation
          connect(p, s_first.p);
          connect(s_first.n, s[1].p);
          for i in 1:N - 2 loop
            connect(s[i].n, s[i + 1].p);
          end for;
          connect(s[N - 1].n, s_last.p);
          connect(s_last.n, n);
          if useHeatPort then
            connect(heatPort, s_first.heatPort);
            for i in 1:N - 1 loop
              connect(heatPort, s[i].heatPort);
            end for;
            connect(heatPort, s_last.heatPort);
          end if;
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{60, 80}, {-60, -80}}, lineColor = {0, 0, 255}, fillPattern = FillPattern.Solid, fillColor = {255, 255, 255}), Line(points = {{40, 60}, {40, 40}}), Line(points = {{40, 50}, {-40, 50}}), Line(points = {{-40, 60}, {-40, 40}}), Line(points = {{40, -40}, {40, -60}}), Line(points = {{40, -50}, {-40, -50}}), Line(points = {{-40, -40}, {-40, -60}}), Line(points = {{40, 30}, {40, 10}}), Line(points = {{40, 20}, {-40, 20}}), Line(points = {{-40, 30}, {-40, 10}}), Line(points = {{0, 6}, {0, -34}}, color = {0, 0, 255}, pattern = LinePattern.Dot), Text(extent = {{-146, 135}, {154, 95}}, textString = "%name", lineColor = {0, 0, 255})}),
            Documentation(info = "<html>
<p>The M_OLine is a multi line model which consists of several segments and several single lines. Each segment consists of resistors and inductors that are connected in series in each single line, and of capacitors and conductors both between the lines and to the ground. The inductors are coupled to each other like in the M_Inductor model. The following picture shows the schematic of a segment with four single lines (lines=4):</p>

<blockquote>
<img src=\"modelica://Modelica/Resources/Images/Electrical/Analog/segment.png\"
     alt=\"segment.png\">
</blockquote>

<p>The complete multi line consists of N segments and an auxiliary segment_last:</p>
<p align=\"center\"><code>-- segment_1 -- segment_2 -- ... -- segment_N -- segment_last --</code> </p>
<p>In the picture of the segment can be seen, that a single segment is asymmetric. Connecting such asymmetric segments in a series forces also an asymmetric multi line. To get a symmetric model which is useful for coupling and which guaranties the same pin properties, in the segment_1 only half valued resistors and inductors are used. The remaining resistors and inductors are at the other end of the line within the auxiliary segment_last. For the example with 4 lines the schematic of segment_last is like this:</p>

<blockquote>
<img src=\"modelica://Modelica/Resources/Images/Electrical/Analog/segment_last.png\"
     alt=\"segment_last.png\">
</blockquote>

<p>The number of the capacitors and conductors depends on the number of single lines that are used, because each line is connected to every other line by both a capacitor and a conductor. One line consists of <b>at least two segments</b>. Inside the model M_OLine the model <i>segment</i> is used. This model represents one segment which is build as described above. For modelling the inductances and their mutual couplings the model M_Transformer is used. To fill the resistance vector, resistance values as many as lines are needed, e.g., if there are four lines, four resistances are needed. For example for a microelectronic line of 0.1m length, a sensible resistance-vector would be R=[4.76e5, 1.72e5, 1.72e5, 1.72e5].</p>
<p>Filling the matrices of the inductances, capacitances and conductances is a bit more complicated, because those components occur also between two lines and not only (like the resistor) in one line. The entries of the matrices are given by the user in form of a vector. The vector length dim_vector_lgc is calculated by <b>dim_vector_lgc = lines*(lines+1)/2</b>. Inside the model a symmetric inductance matrix, a symmetric capacitance matrix and a symmetric conductance matrix are built out of the entries of the vectors given by the user. The way of building is the same for each matrix, so the approach for filling one of the matrices will be shown at an example:</p>
<p>The number of lines is assumed to be four. To build the matrix, the model needs the values from the main diagonal and from the positions that are below the main diagonal. To get the following matrix</p>

<blockquote>
<img src=\"modelica://Modelica/Resources/Images/Electrical/Analog/Lines/M_OLine-eqMatrix.png\" alt=\"Matrix\"/>
</blockquote>

<p>the vector with dim_vector_lgc=4*5/2=10 has to appear in the following way: vector = [<b>1</b>, 0.1, 0.2, 0.4, <b>2</b>, 0.3 0.5, <b>3</b>, 0.6, <b>4</b>] </p>

<p>For the example of a microelectronic line of 0.1m length, which is used as default example for the M_OLine model, a sensible inductance-matrix would be </p>

<blockquote>
<img src=\"modelica://Modelica/Resources/Images/Electrical/Analog/Lines/M_OLine-eqL.png\" alt=\"L\"/>
</blockquote>

<p>For the example of a microelectronic line of 0.1m length, which is used as default example for the M_OLine model, a sensible capacitance-matrix would be:</p>

<blockquote>
<img src=\"modelica://Modelica/Resources/Images/Electrical/Analog/Lines/M_OLine-eqC.png\" alt=\"C\"/>
</blockquote>

<p>For the example of a microelectronic line of 0.1m length, which is used as default example for the M_OLine model, a sensible conductance-matrix would be:</p>

<blockquote>
<img src=\"modelica://Modelica/Resources/Images/Electrical/Analog/Lines/M_OLine-eqG.png\" alt=\"G\"/>
</blockquote>

<p>The user has the possibility to enable a conditional heatport. If so, the M_OLine can be connected to a thermal network. When the parameter alpha is set to an value greater then zero, the M_OLine becomes temperature sensitive due to their resistors which resistances are calculated by R = R_ref*(1 + alpha*(heatPort.T - T_ref)) and conductors calculated by (G_actual = G_ref/(1 + alpha*(T_heatPort - T_ref)).</p>
</html>", revisions = "<HTML>
<table border=\"1\" cellspacing=\"0\" cellpadding=\"2\">
    <tr>
      <th>Version</th>
      <th>Revision</th>
      <th>Date</th>
      <th>Author</th>
      <th>Comment</th>
    </tr>
   <tr>
      <td valign=\"top\"></td>
      <td valign=\"top\">4163</td>
      <td valign=\"top\">2010-09-11</td>
      <td valign=\"top\">Dietmar Winkler</td>
      <td valign=\"top\">Documentation corrected according to documentation guidelines.</td>
    </tr>
    <tr>
      <td valign=\"top\"></td>
      <td valign=\"top\"></td>
      <td valign=\"top\">2008-11-24</td>
      <td valign=\"top\">Kristin Majetta</td>
      <td valign=\"top\">Documentation added.</td>
    </tr>
    <tr>
      <td valign=\"top\"></td>
      <td valign=\"top\"></td>
      <td valign=\"top\">2007-02-26</td>
      <td valign=\"top\">Kristin Majetta</td>
      <td valign=\"top\">Initially implemented</td>
    </tr>
</table>
</HTML>"));
        end M_OLine;

        model ULine "Lossy RC Line"
          //extends Interfaces.ThreePol;
          Modelica.Electrical.Analog.Interfaces.Pin p1 annotation(
            Placement(transformation(extent = {{-110, -10}, {-90, 10}}, rotation = 0)));
          Modelica.Electrical.Analog.Interfaces.Pin p2 annotation(
            Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
          Modelica.Electrical.Analog.Interfaces.Pin p3 annotation(
            Placement(transformation(extent = {{-10, -110}, {10, -90}}, rotation = 0), iconTransformation(extent = {{-10, -110}, {10, -90}})));
          Modelica.SIunits.Voltage v13;
          Modelica.SIunits.Voltage v23;
          Modelica.SIunits.Current i1;
          Modelica.SIunits.Current i2;
          parameter Real r(final min = Modelica.Constants.small, unit = "Ohm/m", start = 1) "Resistance per meter";
          parameter Real c(final min = Modelica.Constants.small, unit = "F/m", start = 1) "Capacitance per meter";
          parameter Modelica.SIunits.Length length(final min = Modelica.Constants.small, start = 1) "Length of line";
          parameter Integer N(final min = 1, start = 1) "Number of lumped segments";
          parameter Modelica.SIunits.LinearTemperatureCoefficient alpha = 0 "Temperature coefficient of resistance (R_actual = R*(1 + alpha*(T_heatPort - T_ref))";
          parameter Boolean useHeatPort = false "=true, if HeatPort is enabled" annotation(
            Evaluate = true,
            HideResult = true,
            choices(checkBox = true));
          parameter Modelica.SIunits.Temperature T = 293.15 "Fixed device temperature if useHeatPort = false" annotation(
            Dialog(enable = not useHeatPort));
          parameter Modelica.SIunits.Temperature T_ref = 300.15;
          Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a heatPort if useHeatPort annotation(
            Placement(transformation(extent = {{-10, -110}, {10, -90}}), iconTransformation(extent = {{-80, -80}, {-60, -60}})));
        protected
          Modelica.Electrical.Analog.Basic.Resistor R[N + 1](R = fill(r * length / (N + 1), N + 1), T_ref = fill(T_ref, N + 1), alpha = fill(alpha, N + 1), useHeatPort = fill(useHeatPort, N + 1), T = fill(T, N + 1));
          Modelica.Electrical.Analog.Basic.Capacitor C[N](C = fill(c * length / N, N));
        equation
          v13 = p1.v - p3.v;
          v23 = p2.v - p3.v;
          i1 = p1.i;
          i2 = p2.i;
          connect(p1, R[1].p);
          for i in 1:N loop
            connect(R[i].n, R[i + 1].p);
          end for;
          for i in 1:N loop
            connect(R[i].n, C[i].p);
          end for;
          for i in 1:N loop
            connect(C[i].n, p3);
          end for;
          connect(R[N + 1].n, p2);
          if useHeatPort then
            for i in 1:N + 1 loop
              connect(heatPort, R[i].heatPort);
            end for;
          end if;
          annotation(
            Documentation(info = "<html>
<p>As can be seen in the picture below, the lossy RC line ULine is a single conductor lossy transmission line which consists of segments of lumped series resistors and capacitors that are connected with the reference pin p3. The precision of the model depends on the number N of lumped segments.
<br>To get a symmetric line model, the first resistor is cut into two parts (R1 and R_Nplus1). These two new resistors have the half of the resistance of the original resistor.
</p>
<blockquote>
<img src=\"modelica://Modelica/Resources/Images/Electrical/Analog/ULine.png\"
     alt=\"ULine.png\">
</blockquote>
<p>
The capacitances are calculated with: C=c*length/N.
<br>The resistances are calculated with: R=r*length/(N+1).
<br>For all capacitors and resistors the values of each segment are the same except for the first and last resistor, that only has the half of the above calculated value.<p>The user has the possibility to enable a conditional heatport. If so, the ULine can be connected to a thermal network. When the parameter alpha is set to an value greater then zero, the ULine becomes temperature sensitive</p>
<p>due to their resistors which resistances are calculated by <code>R = R_ref*(1 + alpha*(heatPort.T - T_ref)).</code> </p>
<p>Note, this is different compared with the lumped line model of SPICE.</p>
<p><b>References</b></p>
<dl><dt>Johnson, B.; Quarles, T.; Newton, A. R.; Pederson, D. O.; Sangiovanni-Vincentelli, A.</dt>
<dd>SPICE3 Version 3e User&#39;;s Manual (April 1, 1991). Department of Electrical Engineering and Computer Sciences, University of California, Berkley p. 22, p. 124 </dd>
</dl></html>", revisions = "<html>
<dl>
<dt><i>1998</i></dt>
<dd>by Christoph Clauss initially implemented</dd>
</dl>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-60, 60}, {60, -60}}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Line(points = {{0, -60}, {0, -90}}, color = {0, 0, 255}), Line(points = {{60, 0}, {90, 0}}, color = {0, 0, 255}), Line(points = {{-60, 0}, {-90, 0}}, color = {0, 0, 255}), Line(points = {{30, 30}, {-30, 30}}, color = {0, 0, 255}), Line(points = {{-30, 40}, {-30, 20}}, color = {0, 0, 255}), Line(points = {{30, 40}, {30, 20}}, color = {0, 0, 255}), Text(extent = {{-154, 117}, {146, 77}}, textString = "%name", lineColor = {0, 0, 255})}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-60, 60}, {60, -60}}, lineColor = {0, 0, 255}), Line(points = {{0, -60}, {0, -96}}, color = {0, 0, 255}), Line(points = {{60, 0}, {96, 0}}, color = {0, 0, 255}), Line(points = {{-60, 0}, {-96, 0}}, color = {0, 0, 255}), Line(points = {{30, 30}, {-30, 30}}, color = {0, 0, 255}), Line(points = {{-30, 40}, {-30, 20}}, color = {0, 0, 255}), Line(points = {{30, 40}, {30, 20}}, color = {0, 0, 255})}));
        end ULine;

        model TLine1 "Lossless transmission line with characteristic impedance Z0 and transmission delay TD"
          extends Modelica.Electrical.Analog.Interfaces.TwoPort;
          parameter Modelica.SIunits.Resistance Z0(start = 1) "Characteristic impedance";
          parameter Modelica.SIunits.Time TD(start = 1) "Transmission delay";
        protected
          Modelica.SIunits.Voltage er;
          Modelica.SIunits.Voltage es;
        equation
          assert(Z0 > 0, "Z0 has to be positive");
          assert(TD > 0, "TD has to be positive");
          i1 = (v1 - es) / Z0;
          i2 = (v2 - er) / Z0;
          es = 2 * delay(v2, TD) - delay(er, TD);
          er = 2 * delay(v1, TD) - delay(es, TD);
          annotation(
            Documentation(info = "<html>
<p>Lossless transmission line with characteristic impedance Z0 and transmission delay TD The lossless transmission line TLine1 is a two Port. Both port branches consist of a resistor with characteristic impedance Z0 and a controlled voltage source that takes into consideration the transmission delay TD. For further details see Branin&#39;;s article below. The model parameters can be derived from inductance and capacitance per length (L&#39;; resp. C&#39;;), i. e. Z0 = sqrt(L&#39;;/C&#39;;) and TD = sqrt(L&#39;;*C&#39;;)*length_of_line. Resistance R&#39;; and conductance C&#39;; per meter are assumed to be zero.</p>
<p><b>References:</b></p>
<dl><dt>Branin Jr., F. H.</dt>
<dd>Transient Analysis of Lossless Transmission Lines. Proceedings of the IEEE 55(1967), 2012 - 2013</dd>
<dt>Hoefer, E. E. E.; Nielinger, H.</dt>
<dd>SPICE : Analyseprogramm fuer elektronische Schaltungen. Springer-Verlag, Berlin, Heidelberg, New York, Tokyo, 1985. </dd>
</dl></html>", revisions = "<html>
<ul>
<li><i> 1998   </i>
       by Joachim Haase<br> initially implemented<br>
       </li>
</ul>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-60, 60}, {60, -60}}, lineColor = {0, 0, 255}, fillPattern = FillPattern.Solid, fillColor = {255, 255, 255}), Line(points = {{60, -50}, {90, -50}}, color = {0, 0, 255}), Line(points = {{60, 50}, {90, 50}}, color = {0, 0, 255}), Line(points = {{-60, 50}, {-90, 50}}, color = {0, 0, 255}), Line(points = {{-60, -50}, {-90, -50}}, color = {0, 0, 255}), Line(points = {{30, 30}, {-30, 30}}, color = {0, 0, 255}), Line(points = {{-30, 40}, {-30, 20}}, color = {0, 0, 255}), Line(points = {{30, 40}, {30, 20}}, color = {0, 0, 255}), Text(extent = {{-50, 0}, {50, -20}}, textString = "TLine1", lineColor = {0, 0, 0}), Text(extent = {{-152, 122}, {148, 82}}, textString = "%name", lineColor = {0, 0, 255})}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-60, 60}, {60, -60}}, lineColor = {0, 0, 255}), Line(points = {{60, -50}, {96, -50}}, color = {0, 0, 255}), Line(points = {{60, 50}, {96, 50}}, color = {0, 0, 255}), Line(points = {{-60, 50}, {-96, 50}}, color = {0, 0, 255}), Line(points = {{-60, -50}, {-96, -50}}, color = {0, 0, 255}), Line(points = {{30, 30}, {-30, 30}}, color = {0, 0, 255}), Line(points = {{-30, 40}, {-30, 20}}, color = {0, 0, 255}), Line(points = {{30, 40}, {30, 20}}, color = {0, 0, 255}), Text(extent = {{-100, 100}, {100, 70}}, textString = "TLine1", lineColor = {0, 0, 255}), Text(extent = {{-30, 0}, {31, -31}}, textString = "TLine1", lineColor = {0, 0, 255})}));
        end TLine1;

        model TLine2 "Lossless transmission line with characteristic impedance Z0, frequency F and normalized length NL"
          extends Modelica.Electrical.Analog.Interfaces.TwoPort;
          parameter Modelica.SIunits.Resistance Z0(start = 1) "Characteristic impedance";
          parameter Modelica.SIunits.Frequency F(start = 1) "Frequency";
          parameter Real NL(start = 1) "Normalized length";
        protected
          Modelica.SIunits.Voltage er;
          Modelica.SIunits.Voltage es;
          parameter Modelica.SIunits.Time TD = NL / F;
        equation
          assert(Z0 > 0, "Z0 has to be positive");
          assert(NL > 0, "NL has to be positive");
          assert(F > 0, "F has to be positive");
          i1 = (v1 - es) / Z0;
          i2 = (v2 - er) / Z0;
          es = 2 * delay(v2, TD) - delay(er, TD);
          er = 2 * delay(v1, TD) - delay(es, TD);
          annotation(
            Documentation(info = "<html>
<p>Lossless transmission line with characteristic impedance Z0, frequency F and normalized length NL The lossless transmission line TLine2 is a two Port. Both port branches consist of a resistor with the value of the characteristic impedance Z0 and a controlled voltage source that takes into consideration the transmission delay. For further details see Branin&#39;;s article below. Resistance R&#39;; and conductance C&#39;; per meter are assumed to be zero. The characteristic impedance Z0 can be derived from inductance and capacitance per length (L&#39;; resp. C&#39;;), i. e. Z0 = sqrt(L&#39;;/C&#39;;). The normalized length NL is equal to the length of the line divided by the wavelength corresponding to the frequency F, i. e. the transmission delay TD is the quotient of NL and F.</p>
<p><b>References:</b></p>
<dl><dt>Branin Jr., F. H.</dt>
<dd>Transient Analysis of Lossless Transmission Lines. Proceedings of the IEEE 55(1967), 2012 - 2013</dd>
<dt>Hoefer, E. E. E.; Nielinger, H.</dt>
<dd>SPICE : Analyseprogramm fuer elektronische Schaltungen. Springer-Verlag, Berlin, Heidelberg, New York, Tokyo, 1985. </dd>
</dl></html>", revisions = "<html>
<dl>
<dt><i>1998</i></dt>
<dd>by Joachim Haase initially implemented</dd>
</dl>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-60, 60}, {60, -60}}, lineColor = {0, 0, 255}, fillPattern = FillPattern.Solid, fillColor = {255, 255, 255}), Line(points = {{60, -50}, {90, -50}}, color = {0, 0, 255}), Line(points = {{60, 50}, {90, 50}}, color = {0, 0, 255}), Line(points = {{-60, 50}, {-90, 50}}, color = {0, 0, 255}), Line(points = {{-60, -50}, {-90, -50}}, color = {0, 0, 255}), Line(points = {{30, 30}, {-30, 30}}, color = {0, 0, 255}), Line(points = {{-30, 40}, {-30, 20}}, color = {0, 0, 255}), Line(points = {{30, 40}, {30, 20}}, color = {0, 0, 255}), Text(extent = {{-51, -10}, {50, -31}}, textString = "TLine2", lineColor = {0, 0, 0}), Text(extent = {{-148, 119}, {152, 79}}, textString = "%name", lineColor = {0, 0, 255})}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-60, 60}, {60, -60}}, lineColor = {0, 0, 255}), Line(points = {{60, -50}, {96, -50}}, color = {0, 0, 255}), Line(points = {{60, 50}, {96, 50}}, color = {0, 0, 255}), Line(points = {{-60, 50}, {-96, 50}}, color = {0, 0, 255}), Line(points = {{-60, -50}, {-96, -50}}, color = {0, 0, 255}), Line(points = {{30, 30}, {-30, 30}}, color = {0, 0, 255}), Line(points = {{-30, 40}, {-30, 20}}, color = {0, 0, 255}), Line(points = {{30, 40}, {30, 20}}, color = {0, 0, 255}), Text(extent = {{-100, 100}, {100, 70}}, textString = "TLine2", lineColor = {0, 0, 255})}));
        end TLine2;

        model TLine3 "Lossless transmission line with characteristic impedance Z0 and frequency F"
          extends Modelica.Electrical.Analog.Interfaces.TwoPort;
          parameter Modelica.SIunits.Resistance Z0(start = 1) "Natural impedance";
          parameter Modelica.SIunits.Frequency F(start = 1) "Frequency";
        protected
          Modelica.SIunits.Voltage er;
          Modelica.SIunits.Voltage es;
          parameter Modelica.SIunits.Time TD = 1 / F / 4;
        equation
          assert(Z0 > 0, "Z0 has to be positive");
          assert(F > 0, "F has to be positive");
          i1 = (v1 - es) / Z0;
          i2 = (v2 - er) / Z0;
          es = 2 * delay(v2, TD) - delay(er, TD);
          er = 2 * delay(v1, TD) - delay(es, TD);
          annotation(
            Documentation(info = "<html>
<p>Lossless transmission line with characteristic impedance Z0 and frequency F The lossless transmission line TLine3 is a two Port. Both port branches consist of a resistor with value of the characteristic impedance Z0 and a controlled voltage source that takes into consideration the transmission delay. For further details see Branin&#39;;s article below. Resistance R&#39;; and conductance C&#39;; per meter are assumed to be zero. The characteristic impedance Z0 can be derived from inductance and capacitance per length (L&#39;; resp. C&#39;;), i. e. Z0 = sqrt(L&#39;;/C&#39;;). The length of the line is equal to a quarter of the wavelength corresponding to the frequency F, i. e. the transmission delay is the quotient of 4 and F. In this case, the characteristic impedance is called natural impedance.</p>
<p><b>References:</b></p>
<dl><dt>Branin Jr., F. H.</dt>
<dd>Transient Analysis of Lossless Transmission Lines. Proceedings of the IEEE 55(1967), 2012 - 2013</dd>
<dt>Hoefer, E. E. E.; Nielinger, H.</dt>
<dd>SPICE : Analyseprogramm fuer elektronische Schaltungen. Springer-Verlag, Berlin, Heidelberg, New York, Tokyo, 1985. </dd>
</dl></html>", revisions = "<html>
<ul>
<li><i> 1998   </i>
       by Joachim Haase<br> initially implemented<br>
       </li>
</ul>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-60, 60}, {60, -60}}, lineColor = {0, 0, 255}, fillPattern = FillPattern.Solid, fillColor = {255, 255, 255}), Line(points = {{60, -50}, {90, -50}}, color = {0, 0, 255}), Line(points = {{60, 50}, {90, 50}}, color = {0, 0, 255}), Line(points = {{-60, 50}, {-90, 50}}, color = {0, 0, 255}), Line(points = {{-60, -50}, {-90, -50}}, color = {0, 0, 255}), Line(points = {{30, 30}, {-30, 30}}, color = {0, 0, 255}), Line(points = {{-30, 40}, {-30, 20}}, color = {0, 0, 255}), Line(points = {{30, 40}, {30, 20}}, color = {0, 0, 255}), Text(extent = {{-50, -10}, {51, -30}}, textString = "TLine3", lineColor = {0, 0, 0}), Text(extent = {{-155, 124}, {145, 84}}, textString = "%name", lineColor = {0, 0, 255})}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-60, 60}, {60, -60}}, lineColor = {0, 0, 255}), Line(points = {{60, -50}, {96, -50}}, color = {0, 0, 255}), Line(points = {{60, 50}, {96, 50}}, color = {0, 0, 255}), Line(points = {{-60, 50}, {-96, 50}}, color = {0, 0, 255}), Line(points = {{-60, -50}, {-96, -50}}, color = {0, 0, 255}), Line(points = {{30, 30}, {-30, 30}}, color = {0, 0, 255}), Line(points = {{-30, 40}, {-30, 20}}, color = {0, 0, 255}), Line(points = {{30, 40}, {30, 20}}, color = {0, 0, 255}), Text(extent = {{-100, 100}, {100, 70}}, textString = "TLine3", lineColor = {0, 0, 255})}));
        end TLine3;
        annotation(
          Documentation(info = "<html>
<p>This package contains lossy and lossless segmented transmission lines, and LC distributed line models. The line models do not yet possess a conditional heating port.</p>
</html>", revisions = "<html>
<dl>
<dt>
<b>Main Authors:</b>
</dt>
<dd>
Christoph Clau&szlig;
    &lt;<a href=\"mailto:Christoph.Clauss@eas.iis.fraunhofer.de\">Christoph.Clauss@eas.iis.fraunhofer.de</a>&gt;<br>
    <a href=\"http://people.eas.iis.fhg.de/Joachim.Haase/\">Joachim Haase;</a>
    &lt;<a href=\"mailto:haase@eas.iis.fhg.de\">haase@eas.iis.fhg.de</a>&gt;<br>
    Andr&eacute; Schneider
    &lt;<a href=\"mailto:Andre.Schneider@eas.iis.fraunhofer.de\">Andre.Schneider@eas.iis.fraunhofer.de</a>&gt;<br>
    Fraunhofer Institute for Integrated Circuits<br>
    Design Automation Department<br>
    Zeunerstra&szlig;e 38<br>
    D-01069 Dresden
</dd>
<dt>
<b>Copyright:</b>
</dt>
<dd>
Copyright &copy; 1998-2015, Modelica Association and Fraunhofer-Gesellschaft.<br>
<i>The Modelica package is <b>free</b> software; it can be redistributed and/or modified
under the terms of the <b>Modelica license</b>, see the license conditions
and the accompanying <b>disclaimer</b> in the documentation of package
Modelica in file \"Modelica/package.mo\".</i>
</dd>
</dl>
</html>"),
          Icon(graphics = {Line(points = {{-60, 50}, {-90, 50}}, color = {0, 0, 0}), Rectangle(extent = {{-60, 60}, {60, -60}}, lineColor = {0, 0, 0}), Line(points = {{-60, -50}, {-90, -50}}, color = {0, 0, 0}), Line(points = {{36, 20}, {-36, 20}}, color = {0, 0, 0}), Line(points = {{-36, 40}, {-36, 0}}, color = {0, 0, 0}), Line(points = {{36, 40}, {36, 0}}, color = {0, 0, 0}), Line(points = {{60, 50}, {90, 50}}, color = {0, 0, 0}), Line(points = {{60, -50}, {90, -50}}, color = {0, 0, 0})}));
      end Lines;

      package Semiconductors "Semiconductor devices such as diode, MOS and bipolar transistor"
        extends Modelica.Icons.Package;
        import Modelica.SIunits;

        model Diode "Simple diode"
          extends Modelica.Electrical.Analog.Interfaces.OnePort;
          parameter SIunits.Current Ids = 1.e-6 "Saturation current";
          parameter SIunits.Voltage Vt = 0.04 "Voltage equivalent of temperature (kT/qn)";
          parameter Real Maxexp(final min = Modelica.Constants.small) = 15 "Max. exponent for linear continuation";
          parameter SIunits.Resistance R = 1.e8 "Parallel ohmic resistance";
          extends Modelica.Electrical.Analog.Interfaces.ConditionalHeatPort(T = 293.15);
        equation
          i = smooth(1, if v / Vt > Maxexp then Ids * (exp(Maxexp) * (1 + v / Vt - Maxexp) - 1) + v / R else Ids * (exp(v / Vt) - 1) + v / R);
          LossPower = v * i;
          annotation(
            Documentation(info = "<html>
<p>The simple diode is a one port. It consists of the diode itself and an parallel ohmic resistance <i>R</i>. The diode formula is:</p>
<pre>                v/vt
  i  =  ids ( e      - 1).</pre>
<p>If the exponent <i>v/vt</i> reaches the limit <i>maxex</i>, the diode characteristic is linearly continued to avoid overflow.</p><p><b>Please note:</b> In case of useHeatPort=true the temperature dependence of the electrical behavior is <b>not </b>modelled yet. The parameters are not temperature dependent.</p>
</html>", revisions = "<html>
<ul>
<li><i> March 11, 2009   </i>
       by Christoph Clauss<br> conditional heat port added<br>
       </li>
<li><i> November 15, 2005   </i>
       by Christoph Clauss<br> smooth function added<br>
       </li>
<li><i> 1998   </i>
       by Christoph Clauss<br> implemented<br>
       </li>
</ul>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{30, 0}, {-30, 40}, {-30, -40}, {30, 0}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{-90, 0}, {40, 0}}, color = {0, 0, 255}), Line(points = {{40, 0}, {90, 0}}, color = {0, 0, 255}), Line(points = {{30, 40}, {30, -40}}, color = {0, 0, 255}), Text(extent = {{-150, -49}, {149, -77}}, lineColor = {0, 0, 0}, textString = "Vt=%Vt"), Text(extent = {{-154, 100}, {146, 60}}, textString = "%name", lineColor = {0, 0, 255}), Line(visible = useHeatPort, points = {{0, -100}, {0, -20}}, color = {127, 0, 0}, smooth = Smooth.None, pattern = LinePattern.Dot)}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{30, 0}, {-30, 40}, {-30, -40}, {30, 0}}, lineColor = {0, 0, 255}, fillColor = {255, 0, 0}, fillPattern = FillPattern.None), Line(points = {{-96, 0}, {96, 0}}, color = {0, 0, 255}), Line(points = {{30, 40}, {30, -40}}, color = {0, 0, 255})}));
        end Diode;

        model ZDiode "Zener diode with 3 working areas"
          extends Modelica.Electrical.Analog.Interfaces.OnePort;
          parameter Modelica.SIunits.Current Ids = 1.e-6 "Saturation current";
          parameter Modelica.SIunits.Voltage Vt = 0.04 "Voltage equivalent of temperature (kT/qn)";
          parameter Real Maxexp(final min = Modelica.Constants.small) = 30 "Max. exponent for linear continuation";
          parameter Modelica.SIunits.Resistance R = 1.e8 "Parallel ohmic resistance";
          parameter Modelica.SIunits.Voltage Bv = 5.1 "Breakthrough voltage = Zener- or Z-voltage";
          parameter Modelica.SIunits.Current Ibv = 0.7 "Breakthrough knee current";
          parameter Real Nbv = 0.74 "Breakthrough emission coefficient";
          extends Modelica.Electrical.Analog.Interfaces.ConditionalHeatPort(T = 293.15);
          Real maxexp = exp(Maxexp);
        equation
          i = smooth(1, if v > Maxexp * Vt then Ids * (exp(Maxexp) * (1 + v / Vt - Maxexp) - 1) + v / R else if v + Bv < (-Maxexp * (Nbv * Vt)) then (-Ids) - Ibv * exp(Maxexp) * (1 - (v + Bv) / (Nbv * Vt) - Maxexp) + v / R else Ids * (exp(v / Vt) - 1) - Ibv * exp(-(v + Bv) / (Nbv * Vt)) + v / R);
          LossPower = v * i;
          annotation(
            Documentation(info = "<html>
<p>The simple Zener diode is a one port. It consists of the diode itself and an parallel ohmic resistance <i>R</i>. The diode formula is:
<pre>                v/Vt                -(v+Bv)/(Nbv*Vt)
  i  =  Ids ( e      - 1) - Ibv ( e                  ).</pre>
<p>If the exponent in one of the two branches reaches the limit <i>Maxexp</i>, the diode characteristic is linearly continued to avoid overflow.</p>
<p><br>The Zener diode model permits (in contrast to the simple diode model) current in reverse direction if the breakdown voltage Bv (also known Zener knee voltage) is exceeded.</p>
<p>The thermal power is calculated by <i>i*v</i>.</p><p><b>Please note:</b> In case of useHeatPort=true the temperature dependence of the electrical behavior is <b>not </b>modelled yet. The parameters are not temperature dependent.</p>
</html>", revisions = "<html>
<ul>
<li><i> March 11, 2009   </i>
       by Christoph Clauss<br> conditional heat port added<br>
       </li>
<li><i>April 5, 2004   </i>
       by Christoph Clauss<br> implemented<br>
       </li>
</ul>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{30, 0}, {-30, 40}, {-30, -40}, {30, 0}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{-90, 0}, {40, 0}}, color = {0, 0, 255}), Line(points = {{40, 0}, {90, 0}}, color = {0, 0, 255}), Line(points = {{30, 40}, {30, -40}}, color = {0, 0, 255}), Text(extent = {{-152, 114}, {148, 74}}, textString = "%name", lineColor = {0, 0, 255}), Line(visible = useHeatPort, points = {{0, -101}, {0, -20}}, color = {127, 0, 0}, smooth = Smooth.None, pattern = LinePattern.Dot)}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{30, 0}, {-30, 40}, {-30, -40}, {30, 0}}, lineColor = {0, 0, 255}, fillColor = {255, 0, 0}, fillPattern = FillPattern.None), Line(points = {{-99, 0}, {96, 0}}, color = {0, 0, 255}), Line(points = {{30, 40}, {30, -40}}, color = {0, 0, 255})}));
        end ZDiode;

        model PMOS "Simple MOS Transistor"
          Interfaces.Pin D "Drain" annotation(
            Placement(transformation(extent = {{90, 40}, {110, 60}}, rotation = 0)));
          Interfaces.Pin G "Gate" annotation(
            Placement(transformation(extent = {{-90, -40}, {-110, -60}}, rotation = 0)));
          Interfaces.Pin S "Source" annotation(
            Placement(transformation(extent = {{90, -40}, {110, -60}}, rotation = 0)));
          Interfaces.Pin B "Bulk" annotation(
            Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
          parameter SIunits.Length W = 20.0e-6 "Width";
          parameter SIunits.Length L = 6.0e-6 "Length";
          parameter SIunits.Transconductance Beta = 0.0105e-3 "Transconductance parameter";
          parameter SIunits.Voltage Vt = -1.0 "Zero bias threshold voltage";
          parameter Real K2 = 0.41 "Bulk threshold parameter";
          parameter Real K5 = 0.839 "Reduction of pinch-off region";
          parameter SIunits.Length dW = -2.5e-6 "Narrowing of channel";
          parameter SIunits.Length dL = -2.1e-6 "Shortening of channel";
          parameter SIunits.Resistance RDS = 1.e+7 "Drain-Source-Resistance";
          extends Modelica.Electrical.Analog.Interfaces.ConditionalHeatPort(T = 293.15);
        protected
          Real v;
          Real uds;
          Real ubs;
          Real ugst;
          Real ud;
          Real us;
          Real id;
          Real gds;
        equation
          assert(L + dL > 0, "PMOS: Effective length must be positive");
          assert(W + dW > 0, "PMOS: Effective width  must be positive");
          gds = if RDS < 1.e-20 and RDS > (-1.e-20) then 1.e20 else 1 / RDS;
          v = Beta * (W + dW) / (L + dL);
          ud = smooth(0, if D.v > S.v then S.v else D.v);
          us = smooth(0, if D.v > S.v then D.v else S.v);
          uds = ud - us;
          ubs = smooth(0, if B.v < us then 0 else B.v - us);
          ugst = (G.v - us - Vt + K2 * ubs) * K5;
          id = smooth(0, if ugst >= 0 then uds * gds else if ugst < uds then (-v * uds * (ugst - uds / 2)) + uds * gds else (-v * ugst * ugst / 2) + uds * gds);
          G.i = 0;
          D.i = smooth(0, if D.v > S.v then -id else id);
          S.i = smooth(0, if D.v > S.v then id else -id);
          B.i = 0;
          LossPower = D.i * (D.v - S.v);
          annotation(
            Documentation(info = "<html>
<P>
The PMOS model is a simple model of a p-channel metal-oxide semiconductor
FET. It differs slightly from the device used in the SPICE simulator.
For more details please care for H. Spiro.
</P>
<P>
The model does not consider capacitances. A high drain-source resistance RDS
is included to avoid numerical difficulties.
<br><br>
<b>Please note: </b>
In case of useHeatPort=true the temperature dependence of the electrical
behavior is <b> not </b> modelled yet. The parameters are not temperature dependent.
</P>
<DL>
<DT><b>References:</b>
<DD>Spiro, H.: Simulation integrierter Schaltungen. R. Oldenbourg Verlag
  Muenchen Wien 1990.
</DL>
<P>
Some typical parameter sets are:
</P>
<PRE>
  W       L      Beta        Vt    K2     K5      DW       DL
  m       m      A/V^2       V     -      -       m        m
  50.e-6  8.e-6  0.0085e-3  -0.15  0.41   0.839  -3.8e-6  -4.0e-6
  20.e-6  6.e-6  0.0105e-3  -1.0   0.41   0.839  -2.5e-6  -2.1e-6
  30.e-6  5.e-6  0.0059e-3  -0.3   0.98   1.01    0       -3.9e-6
  30.e-6  5.e-6  0.0152e-3  -0.69  0.104  1.1    -0.8e-6  -0.4e-6
  30.e-6  5.e-6  0.0163e-3  -0.69  0.104  1.1    -0.8e-6  -0.4e-6
  30.e-6  5.e-6  0.0182e-3  -0.69  0.086  1.06   -0.1e-6  -0.6e-6
  20.e-6  6.e-6  0.0074e-3  -1.    0.4    0.59    0        0
</PRE>

</html>", revisions = "<html>
<ul>
<li><i> March 11, 2009   </i>
       by Christoph Clauss<br> conditional heat port added<br>
       </li>
<li><i>December 7, 2005   </i>
       by Christoph Clauss<br>
       error in RDS calculation deleted</li>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-90, -50}, {-10, -50}}, color = {0, 0, 255}), Line(points = {{-10, -50}, {-10, 50}}, color = {0, 0, 255}), Line(points = {{10, 70}, {10, 29}}, color = {0, 0, 255}), Line(points = {{10, 20}, {10, -21}}, color = {0, 0, 255}), Line(points = {{10, -30}, {10, -71}}, color = {0, 0, 255}), Line(points = {{10, 50}, {91, 50}}, color = {0, 0, 255}), Line(points = {{10, 0}, {90, 0}}, color = {0, 0, 255}), Line(points = {{10, -50}, {90, -50}}, color = {0, 0, 255}), Polygon(points = {{60, 0}, {40, 5}, {40, -5}, {60, 0}}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Text(extent = {{-149, 117}, {151, 77}}, textString = "%name", lineColor = {0, 0, 255}), Line(visible = useHeatPort, points = {{0, -90}, {0, 0}}, color = {127, 0, 0}, smooth = Smooth.None, pattern = LinePattern.Dot)}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-96, -50}, {-10, -50}}, color = {0, 0, 255}), Line(points = {{-10, -50}, {-10, 50}}, color = {0, 0, 255}), Line(points = {{10, 70}, {10, 29}}, color = {0, 0, 255}), Line(points = {{10, 20}, {10, -21}}, color = {0, 0, 255}), Line(points = {{10, -30}, {10, -71}}, color = {0, 0, 255}), Line(points = {{10, 50}, {96, 50}}, color = {0, 0, 255}), Line(points = {{10, 0}, {96, 0}}, color = {0, 0, 255}), Line(points = {{10, -50}, {96, -50}}, color = {0, 0, 255}), Polygon(points = {{60, 0}, {40, 5}, {40, -5}, {60, 0}}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255})}));
        end PMOS;

        model NMOS "Simple MOS Transistor"
          Interfaces.Pin D "Drain" annotation(
            Placement(transformation(extent = {{90, 40}, {110, 60}}, rotation = 0)));
          Interfaces.Pin G "Gate" annotation(
            Placement(transformation(extent = {{-90, -40}, {-110, -60}}, rotation = 0)));
          Interfaces.Pin S "Source" annotation(
            Placement(transformation(extent = {{90, -40}, {110, -60}}, rotation = 0)));
          Interfaces.Pin B "Bulk" annotation(
            Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
          parameter SIunits.Length W = 20.e-6 "Width";
          parameter SIunits.Length L = 6.e-6 "Length";
          parameter SIunits.Transconductance Beta = 0.041e-3 "Transconductance parameter";
          parameter SIunits.Voltage Vt = 0.8 "Zero bias threshold voltage";
          parameter Real K2 = 1.144 "Bulk threshold parameter";
          parameter Real K5 = 0.7311 "Reduction of pinch-off region";
          parameter SIunits.Length dW = -2.5e-6 "narrowing of channel";
          parameter SIunits.Length dL = -1.5e-6 "shortening of channel";
          parameter SIunits.Resistance RDS = 1.e+7 "Drain-Source-Resistance";
          extends Modelica.Electrical.Analog.Interfaces.ConditionalHeatPort(T = 293.15);
        protected
          Real v;
          Real uds;
          Real ubs;
          Real ugst;
          Real ud;
          Real us;
          Real id;
          Real gds;
        equation
          assert(L + dL > 0, "NMOS: Effective length must be positive");
          assert(W + dW > 0, "NMOS: Effective width  must be positive");
          gds = if RDS < 1.e-20 and RDS > (-1.e-20) then 1.e20 else 1 / RDS;
          v = Beta * (W + dW) / (L + dL);
          ud = smooth(0, if D.v < S.v then S.v else D.v);
          us = if D.v < S.v then D.v else S.v;
          uds = ud - us;
          ubs = smooth(0, if B.v > us then 0 else B.v - us);
          ugst = (G.v - us - Vt + K2 * ubs) * K5;
          id = smooth(0, if ugst <= 0 then uds * gds else if ugst > uds then v * uds * (ugst - uds / 2) + uds * gds else v * ugst * ugst / 2 + uds * gds);
          G.i = 0;
          D.i = smooth(0, if D.v < S.v then -id else id);
          S.i = smooth(0, if D.v < S.v then id else -id);
          B.i = 0;
          LossPower = D.i * (D.v - S.v);
          annotation(
            Documentation(info = "<html>
<P>
The NMOS model is a simple model of a n-channel metal-oxide semiconductor
FET. It differs slightly from the device used in the SPICE simulator.
For more details please care for H. Spiro.
</P>
<P>
The model does not consider capacitances. A high drain-source resistance RDS
is included to avoid numerical difficulties.
<br> <br>
<b>Please note:</b>
In case of useHeatPort=true the temperature dependence of the electrical
behavior is <b> not </b> modelled yet. The parameters are not temperature dependent.
</P>
<PRE>
  W       L      Beta         Vt      K2     K5       DW       DL
  m       m      A/V^2        V       -      -        m        m
  12.e-6  4.e-6  0.062e-3    -4.5     0.24   0.61    -1.2e-6  -0.9e-6      depletion
  60.e-6  3.e-6  0.048e-3     0.1     0.08   0.68    -1.2e-6  -0.9e-6      enhancement
  12.e-6  4.e-6  0.0625e-3   -0.8     0.21   0.78    -1.2e-6  -0.9e-6      zero
  50.e-6  8.e-6  0.0299e-3    0.24    1.144  0.7311  -5.4e-6  -4.e-6
  20.e-6  6.e-6  0.041e-3     0.8     1.144  0.7311  -2.5e-6  -1.5e-6
  30.e-6  9.e-6  0.025e-3    -4.0     0.861  0.878   -3.4e-6  -1.74e-6
  30.e-6  5.e-6  0.031e-3     0.6     1.5    0.72     0       -3.9e-6
  50.e-6  6.e-6  0.0414e-3   -3.8     0.34   0.8     -1.6e-6  -2.e-6       depletion
  50.e-6  5.e-6  0.03e-3      0.37    0.23   0.86    -1.6e-6  -2.e-6       enhancement
  50.e-6  6.e-6  0.038e-3    -0.9     0.23   0.707   -1.6e-6  -2.e-6       zero
  20.e-6  4.e-6  0.06776e-3   0.5409  0.065  0.71    -0.8e-6  -0.2e-6
  20.e-6  4.e-6  0.06505e-3   0.6209  0.065  0.71    -0.8e-6  -0.2e-6
  20.e-6  4.e-6  0.05365e-3   0.6909  0.03   0.8     -0.3e-6  -0.2e-6
  20.e-6  4.e-6  0.05365e-3   0.4909  0.03   0.8     -0.3e-6  -0.2e-6
  12.e-6  4.e-6  0.023e-3    -4.5     0.29   0.6      0        0           depletion
  60.e-6  3.e-6  0.022e-3     0.1     0.11   0.65     0        0           enhancement
  12.e-6  4.e-6  0.038e-3    -0.8     0.33   0.6      0        0           zero
  20.e-6  6.e-6  0.022e-3     0.8     1      0.66     0        0
</PRE>

<DL>
<DT><b>References:</b></DT>
<DD>Spiro, H.: Simulation integrierter Schaltungen. R. Oldenbourg Verlag
Muenchen Wien 1990.</DD>
</DL>
</html>", revisions = "<html>
<ul>
<li><i> March 11, 2009   </i>
       by Christoph Clauss<br> conditional heat port added<br>
       </li>
<li><i>December 7, 2005   </i>
       by Christoph Clauss<br>
       error in RDS calculation deleted</li>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-90, -50}, {-10, -50}}, color = {0, 0, 255}), Line(points = {{-10, -50}, {-10, 50}}, color = {0, 0, 255}), Line(points = {{10, 70}, {10, 29}}, color = {0, 0, 255}), Line(points = {{10, 20}, {10, -21}}, color = {0, 0, 255}), Line(points = {{10, -30}, {10, -71}}, color = {0, 0, 255}), Line(points = {{10, 50}, {91, 50}}, color = {0, 0, 255}), Line(points = {{10, 0}, {90, 0}}, color = {0, 0, 255}), Line(points = {{10, -50}, {90, -50}}, color = {0, 0, 255}), Polygon(points = {{40, 0}, {60, 5}, {60, -5}, {40, 0}}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Text(extent = {{-155, 119}, {145, 79}}, textString = "%name", lineColor = {0, 0, 255}), Line(visible = useHeatPort, points = {{0, -100}, {0, 0}}, color = {127, 0, 0}, smooth = Smooth.None, pattern = LinePattern.Dot)}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-96, -50}, {-10, -50}}, color = {0, 0, 255}), Line(points = {{-10, -50}, {-10, 50}}, color = {0, 0, 255}), Line(points = {{10, 70}, {10, 29}}, color = {0, 0, 255}), Line(points = {{10, 20}, {10, -21}}, color = {0, 0, 255}), Line(points = {{10, -30}, {10, -71}}, color = {0, 0, 255}), Line(points = {{10, 50}, {96, 50}}, color = {0, 0, 255}), Line(points = {{10, 0}, {96, 0}}, color = {0, 0, 255}), Line(points = {{10, -50}, {96, -50}}, color = {0, 0, 255}), Polygon(points = {{40, 0}, {60, 5}, {60, -5}, {40, 0}}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255})}));
        end NMOS;

        model NPN "Simple BJT according to Ebers-Moll"
          parameter Real Bf = 50 "Forward beta";
          parameter Real Br = 0.1 "Reverse beta";
          parameter SIunits.Current Is = 1.e-16 "Transport saturation current";
          parameter SIunits.InversePotential Vak = 0.02 "Early voltage (inverse), 1/Volt";
          parameter SIunits.Time Tauf = 0.12e-9 "Ideal forward transit time";
          parameter SIunits.Time Taur = 5e-9 "Ideal reverse transit time";
          parameter SIunits.Capacitance Ccs = 1e-12 "Collector-substrate(ground) cap.";
          parameter SIunits.Capacitance Cje = 0.4e-12 "Base-emitter zero bias depletion cap.";
          parameter SIunits.Capacitance Cjc = 0.5e-12 "Base-coll. zero bias depletion cap.";
          parameter SIunits.Voltage Phie = 0.8 "Base-emitter diffusion voltage";
          parameter Real Me = 0.4 "Base-emitter gradation exponent";
          parameter SIunits.Voltage Phic = 0.8 "Base-collector diffusion voltage";
          parameter Real Mc = 0.333 "Base-collector gradation exponent";
          parameter SIunits.Conductance Gbc = 1e-15 "Base-collector conductance";
          parameter SIunits.Conductance Gbe = 1e-15 "Base-emitter conductance";
          parameter SIunits.Voltage Vt = 0.02585 "Voltage equivalent of temperature";
          parameter Real EMin = -100 "if x < EMin, the exp(x) function is linearized";
          parameter Real EMax = 40 "if x > EMax, the exp(x) function is linearized";
          parameter Modelica.SIunits.Voltage IC = 0 "Initial Value";
          parameter Boolean UIC = false;
          extends Modelica.Electrical.Analog.Interfaces.ConditionalHeatPort(T = 293.15);
        protected
          Real vbc;
          Real vbe;
          Real qbk;
          Real ibc;
          Real ibe;
          Real cbc;
          Real cbe;
          Real ExMin;
          Real ExMax;
          Real Capcje;
          Real Capcjc;

          function pow "Just a helper function for x^y"
            extends Modelica.Icons.Function;
            input Real x;
            input Real y;
            output Real z;
          algorithm
            z := x ^ y;
          end pow;
        public
          Modelica.Electrical.Analog.Interfaces.Pin C "Collector" annotation(
            Placement(transformation(extent = {{90, 40}, {110, 60}}, rotation = 0)));
          Modelica.Electrical.Analog.Interfaces.Pin B "Base" annotation(
            Placement(transformation(extent = {{-90, -10}, {-110, 10}}, rotation = 0)));
          Modelica.Electrical.Analog.Interfaces.Pin E "Emitter" annotation(
            Placement(transformation(extent = {{90, -40}, {110, -60}}, rotation = 0)));
        initial equation
          if UIC then
            C.v = IC;
          end if;
        equation
          ExMin = exp(EMin);
          ExMax = exp(EMax);
          vbc = B.v - C.v;
          vbe = B.v - E.v;
          qbk = 1 - vbc * Vak;
          ibc = smooth(1, if vbc / Vt < EMin then Is * (ExMin * (vbc / Vt - EMin + 1) - 1) + vbc * Gbc else if vbc / Vt > EMax then Is * (ExMax * (vbc / Vt - EMax + 1) - 1) + vbc * Gbc else Is * (exp(vbc / Vt) - 1) + vbc * Gbc);
          ibe = smooth(1, if vbe / Vt < EMin then Is * (ExMin * (vbe / Vt - EMin + 1) - 1) + vbe * Gbe else if vbe / Vt > EMax then Is * (ExMax * (vbe / Vt - EMax + 1) - 1) + vbe * Gbe else Is * (exp(vbe / Vt) - 1) + vbe * Gbe);
          Capcjc = smooth(1, if vbc / Phic > 0 then Cjc * (1 + Mc * vbc / Phic) else Cjc * pow(1 - vbc / Phic, -Mc));
          Capcje = smooth(1, if vbe / Phie > 0 then Cje * (1 + Me * vbe / Phie) else Cje * pow(1 - vbe / Phie, -Me));
          cbc = smooth(1, if vbc / Vt < EMin then Taur * Is / Vt * ExMin * (vbc / Vt - EMin + 1) + Capcjc else if vbc / Vt > EMax then Taur * Is / Vt * ExMax * (vbc / Vt - EMax + 1) + Capcjc else Taur * Is / Vt * exp(vbc / Vt) + Capcjc);
          cbe = smooth(1, if vbe / Vt < EMin then Tauf * Is / Vt * ExMin * (vbe / Vt - EMin + 1) + Capcje else if vbe / Vt > EMax then Tauf * Is / Vt * ExMax * (vbe / Vt - EMax + 1) + Capcje else Tauf * Is / Vt * exp(vbe / Vt) + Capcje);
          C.i = (ibe - ibc) * qbk - ibc / Br - cbc * der(vbc) + Ccs * der(C.v);
          B.i = ibe / Bf + ibc / Br + cbc * der(vbc) + cbe * der(vbe);
          E.i = (-B.i) - C.i + Ccs * der(C.v);
          LossPower = (C.v - E.v) * (ibe - ibc) * qbk + vbc * ibc / Br + vbe * ibe / Bf;
          annotation(
            Documentation(info = "<html>
<P>
This model is a simple model of a bipolar NPN junction transistor according
to Ebers-Moll.
<br> <br>
<b>Please note:</b>
In case of useHeatPort=true the temperature dependence of the electrical
behavior is <b> not </b> modelled yet. The parameters are not temperature dependent.
</P>
<P>
A typical parameter set is:
</P>
<PRE>
  Bf  Br  Is     Vak  Tauf    Taur  Ccs   Cje     Cjc     Phie  Me   PHic   Mc     Gbc    Gbe    Vt
  -   -   A      V    s       s     F     F       F       V     -    V      -      mS     mS     V
  50  0.1 1e-16  0.02 0.12e-9 5e-9  1e-12 0.4e-12 0.5e-12 0.8   0.4  0.8    0.333  1e-15  1e-15  0.02585
</PRE>
<DL>
<DT><b>References:</b></DT>
<DD>Vlach, J.; Singal, K.: Computer methods for circuit analysis and design.
Van Nostrand Reinhold, New York 1983
on page 317 ff.</DD>
</DL>
</html>", revisions = "<html>
<ul>
<li><i> March 11, 2009   </i>
       by Christoph Clauss<br> conditional heat port added<br>
       </li>

<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-10, 40}, {-10, -40}}, color = {0, 0, 255}), Line(points = {{-10, 0}, {-90, 0}}, color = {0, 0, 255}), Line(points = {{91, 50}, {30, 50}}, color = {0, 0, 255}), Line(points = {{30, 50}, {-10, 10}}, color = {0, 0, 255}), Line(points = {{-10, -10}, {30, -50}}, color = {0, 0, 255}), Line(points = {{30, -50}, {91, -50}}, color = {0, 0, 255}), Polygon(points = {{30, -50}, {24, -36}, {16, -44}, {30, -50}}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Text(extent = {{-152, 117}, {148, 77}}, textString = "%name", lineColor = {0, 0, 255}), Line(visible = useHeatPort, points = {{0, -100}, {0, -20}}, color = {127, 0, 0}, smooth = Smooth.None, pattern = LinePattern.Dot)}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-10, 40}, {-10, -40}}, color = {0, 0, 255}), Line(points = {{-10, 0}, {-100, 0}}, color = {0, 0, 255}), Line(points = {{30, 50}, {-10, 10}}, color = {0, 0, 255}), Line(points = {{-10, -10}, {30, -50}}, color = {0, 0, 255}), Line(points = {{30, -50}, {100, -50}}, color = {0, 0, 255}), Line(points = {{100, 50}, {30, 50}}, color = {0, 0, 255}), Polygon(points = {{30, -50}, {24, -36}, {16, -44}, {30, -50}}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255})}));
        end NPN;

        model PNP "Simple BJT according to Ebers-Moll"
          parameter Real Bf = 50 "Forward beta";
          parameter Real Br = 0.1 "Reverse beta";
          parameter SIunits.Current Is = 1.e-16 "Transport saturation current";
          parameter SIunits.InversePotential Vak = 0.02 "Early voltage (inverse), 1/Volt";
          parameter SIunits.Time Tauf = 0.12e-9 "Ideal forward transit time";
          parameter SIunits.Time Taur = 5e-9 "Ideal reverse transit time";
          parameter SIunits.Capacitance Ccs = 1e-12 "Collector-substrate(ground) cap.";
          parameter SIunits.Capacitance Cje = 0.4e-12 "Base-emitter zero bias depletion cap.";
          parameter SIunits.Capacitance Cjc = 0.5e-12 "Base-coll. zero bias depletion cap.";
          parameter SIunits.Voltage Phie = 0.8 "Base-emitter diffusion voltage";
          parameter Real Me = 0.4 "Base-emitter gradation exponent";
          parameter SIunits.Voltage Phic = 0.8 "Base-collector diffusion voltage";
          parameter Real Mc = 0.333 "Base-collector gradation exponent";
          parameter SIunits.Conductance Gbc = 1e-15 "Base-collector conductance";
          parameter SIunits.Conductance Gbe = 1e-15 "Base-emitter conductance";
          parameter SIunits.Voltage Vt = 0.02585 "Voltage equivalent of temperature";
          parameter Real EMin = -100 "if x < EMin, the exp(x) function is linearized";
          parameter Real EMax = 40 "if x > EMax, the exp(x) function is linearized";
          extends Modelica.Electrical.Analog.Interfaces.ConditionalHeatPort(T = 293.15);
        protected
          Real vbc;
          Real vbe;
          Real qbk;
          Real ibc;
          Real ibe;
          Real cbc;
          Real cbe;
          Real ExMin;
          Real ExMax;
          Real Capcje;
          Real Capcjc;

          function pow "Just a helper function for x^y"
            extends Modelica.Icons.Function;
            input Real x;
            input Real y;
            output Real z;
          algorithm
            z := x ^ y;
          end pow;
        public
          Modelica.Electrical.Analog.Interfaces.Pin C "Collector" annotation(
            Placement(transformation(extent = {{90, 40}, {110, 60}}, rotation = 0)));
          Modelica.Electrical.Analog.Interfaces.Pin B "Base" annotation(
            Placement(transformation(extent = {{-90, -10}, {-110, 10}}, rotation = 0)));
          Modelica.Electrical.Analog.Interfaces.Pin E "Emitter" annotation(
            Placement(transformation(extent = {{90, -40}, {110, -60}}, rotation = 0)));
        equation
          ExMin = exp(EMin);
          ExMax = exp(EMax);
          vbc = C.v - B.v;
          vbe = E.v - B.v;
          qbk = 1 - vbc * Vak;
          ibc = smooth(1, if vbc / Vt < EMin then Is * (ExMin * (vbc / Vt - EMin + 1) - 1) + vbc * Gbc else if vbc / Vt > EMax then Is * (ExMax * (vbc / Vt - EMax + 1) - 1) + vbc * Gbc else Is * (exp(vbc / Vt) - 1) + vbc * Gbc);
          ibe = smooth(1, if vbe / Vt < EMin then Is * (ExMin * (vbe / Vt - EMin + 1) - 1) + vbe * Gbe else if vbe / Vt > EMax then Is * (ExMax * (vbe / Vt - EMax + 1) - 1) + vbe * Gbe else Is * (exp(vbe / Vt) - 1) + vbe * Gbe);
          Capcjc = smooth(1, if vbc / Phic > 0 then Cjc * (1 + Mc * vbc / Phic) else Cjc * pow(1 - vbc / Phic, -Mc));
          Capcje = smooth(1, if vbe / Phie > 0 then Cje * (1 + Me * vbe / Phie) else Cje * pow(1 - vbe / Phie, -Me));
          cbc = smooth(1, if vbc / Vt < EMin then Taur * Is / Vt * ExMin * (vbc / Vt - EMin + 1) + Capcjc else if vbc / Vt > EMax then Taur * Is / Vt * ExMax * (vbc / Vt - EMax + 1) + Capcjc else Taur * Is / Vt * exp(vbc / Vt) + Capcjc);
          cbe = smooth(1, if vbe / Vt < EMin then Tauf * Is / Vt * ExMin * (vbe / Vt - EMin + 1) + Capcje else if vbe / Vt > EMax then Tauf * Is / Vt * ExMax * (vbe / Vt - EMax + 1) + Capcje else Tauf * Is / Vt * exp(vbe / Vt) + Capcje);
          C.i = -((ibe - ibc) * qbk - ibc / Br - cbc * der(vbc) - Ccs * der(C.v));
          B.i = -(ibe / Bf + ibc / Br + cbe * der(vbe) + cbc * der(vbc));
          E.i = (-B.i) - C.i + Ccs * der(C.v);
          LossPower = (E.v - C.v) * (ibe - ibc) * qbk + vbc * ibc / Br + vbe * ibe / Bf;
          annotation(
            Documentation(info = "<html>
<P>
This model is a simple model of a bipolar PNP junction transistor according
to Ebers-Moll.
<br> <br>
<b>Please note:</b>
In case of useHeatPort=true the temperature dependence of the electrical
behavior is <b> not </b> modelled yet. The parameters are not temperature dependent.
<P>
A typical parameter set is:
</P>
<PRE>
  Bf  Br  Is     Vak  Tauf    Taur  Ccs   Cje     Cjc     Phie  Me   PHic   Mc     Gbc    Gbe    Vt
  -   -   A      V    s       s     F     F       F       V     -    V      -      mS     mS     V
  50  0.1 1e-16  0.02 0.12e-9 5e-9  1e-12 0.4e-12 0.5e-12 0.8   0.4  0.8    0.333  1e-15  1e-15  0.02585
</PRE>
<DL>
<DT><b>References:</b></DT>
<DD>Vlach, J.; Singal, K.: Computer methods for circuit analysis and design.
Van Nostrand Reinhold, New York 1983
on page 317 ff.</DD>
</DL>
</html>", revisions = "<html>
<ul>
<li><i> March 11, 2009   </i>
       by Christoph Clauss<br> conditional heat port added<br>
       </li>

<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-10, 40}, {-10, -40}}, color = {0, 0, 255}), Line(points = {{-10, 0}, {-90, 0}}, color = {0, 0, 255}), Line(points = {{91, 50}, {30, 50}}, color = {0, 0, 255}), Line(points = {{30, 50}, {-10, 10}}, color = {0, 0, 255}), Line(points = {{-10, -10}, {30, -50}}, color = {0, 0, 255}), Line(points = {{30, -50}, {91, -50}}, color = {0, 0, 255}), Polygon(points = {{-10, -10}, {5, -17}, {-3, -25}, {-10, -10}}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Text(extent = {{-158, 119}, {142, 79}}, textString = "%name", lineColor = {0, 0, 255}), Line(visible = useHeatPort, points = {{0, -100}, {0, -20}}, color = {127, 0, 0}, smooth = Smooth.None, pattern = LinePattern.Dot)}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-10, 40}, {-10, -40}}, color = {0, 0, 255}), Line(points = {{-10, 0}, {-99, 0}}, color = {0, 0, 255}), Line(points = {{100, 50}, {30, 50}}, color = {0, 0, 255}), Line(points = {{30, 50}, {-10, 10}}, color = {0, 0, 255}), Line(points = {{-10, -10}, {30, -50}}, color = {0, 0, 255}), Line(points = {{30, -50}, {99, -50}}, color = {0, 0, 255}), Polygon(points = {{-10, -10}, {5, -17}, {-3, -25}, {-10, -10}}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255})}));
        end PNP;

        model HeatingDiode "Simple diode with heating port"
          extends Modelica.Electrical.Analog.Interfaces.OnePort;
          parameter Modelica.SIunits.Current Ids = 1.e-6 "Saturation current";
          /* parameter Modelica.SIunits.Voltage Vt=0.04 "Voltage equivalent of temperature (kT/qn)"; */
          parameter Real Maxexp(final min = Modelica.Constants.small) = 15 "Max. exponent for linear continuation";
          parameter Modelica.SIunits.Resistance R = 1.e8 "Parallel ohmic resistance";
          parameter Real EG = 1.11 "activation energy";
          parameter Real N = 1 "Emission coefficient";
          parameter Modelica.SIunits.Temperature TNOM = 300.15 "Parameter measurement temperature";
          parameter Real XTI = 3 "Temperature exponent of saturation current";
          extends Modelica.Electrical.Analog.Interfaces.ConditionalHeatPort(useHeatPort = true);
          Modelica.SIunits.Temperature vt_t "Temperature voltage";
          Modelica.SIunits.Current id "diode current";
        protected
          Real k = 1.380662e-23 "Boltzmann's constant, J/K";
          Real q = 1.6021892e-19 "Electron charge, As";
          Modelica.SIunits.Temperature htemp "auxiliary temperature";
          Real aux;
          Real auxp;
          Real maxexp = exp(Maxexp);
        equation
          assert(T_heatPort > 0, "temperature must be positive");
          htemp = T_heatPort;
          vt_t = k * htemp / q;
          id = exlin(v / (N * vt_t), Maxexp) - 1;
          aux = (htemp / TNOM - 1) * EG / (N * vt_t);
          auxp = exp(aux);
          i = Ids * id * pow(htemp / TNOM, XTI / N) * auxp + v / R;
          LossPower = i * v;
          annotation(
            Documentation(info = "<html>
<P>
The simple diode is an electrical one port, where a heat port is added, which is
defined in the Modelica.Thermal library. It consists of the diode itself and an parallel ohmic
resistance <i>R</i>. The diode formula is:
</P>
<PRE>
                v/vt_t
  i  =  ids ( e        - 1).

</PRE>
where vt_t depends on the temperature of the heat port:
<PRE>
  vt_t = k*temp/q
</PRE>
<P>
If the exponent <i>v/vt_t</i> reaches the limit <i>maxex</i>, the diode characteristic is linearly
continued to avoid overflow.<br>
The thermal power is calculated by <i>i*v</i>.
</P>
</html>", revisions = "<html>
<ul>
<li><i> March 11, 2009   </i>
       by Christoph Clauss<br> conditional heat port added<br>
       </li>
<li><i>April 5, 2004   </i>
       by Christoph Clauss<br> implemented<br>
       </li>
</ul>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{30, 0}, {-30, 40}, {-30, -40}, {30, 0}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{-90, 0}, {40, 0}}, color = {0, 0, 255}), Line(points = {{40, 0}, {90, 0}}, color = {0, 0, 255}), Line(points = {{30, 40}, {30, -40}}, color = {0, 0, 255}), Text(extent = {{-152, 114}, {148, 74}}, textString = "%name", lineColor = {0, 0, 255}), Line(visible = useHeatPort, points = {{0, -101}, {0, -20}}, color = {127, 0, 0}, smooth = Smooth.None, pattern = LinePattern.Dot)}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{30, 0}, {-30, 40}, {-30, -40}, {30, 0}}, lineColor = {0, 0, 255}, fillColor = {255, 0, 0}, fillPattern = FillPattern.None), Line(points = {{-96, 0}, {96, 0}}, color = {0, 0, 255}), Line(points = {{30, 40}, {30, -40}}, color = {0, 0, 255})}));
        end HeatingDiode;

        model HeatingNMOS "Simple MOS Transistor with heating port"
          Modelica.Electrical.Analog.Interfaces.Pin D "Drain" annotation(
            Placement(transformation(extent = {{90, 40}, {110, 60}}, rotation = 0)));
          Modelica.Electrical.Analog.Interfaces.Pin G "Gate" annotation(
            Placement(transformation(extent = {{-90, -40}, {-110, -60}}, rotation = 0)));
          Modelica.Electrical.Analog.Interfaces.Pin S "Source" annotation(
            Placement(transformation(extent = {{90, -40}, {110, -60}}, rotation = 0)));
          Modelica.Electrical.Analog.Interfaces.Pin B "Bulk" annotation(
            Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
          parameter Modelica.SIunits.Length W = 20.e-6 "Width";
          parameter Modelica.SIunits.Length L = 6.e-6 "Length";
          parameter Modelica.SIunits.Transconductance Beta = 0.041e-3 "Transconductance parameter";
          parameter Modelica.SIunits.Voltage Vt = 0.8 "Zero bias threshold voltage";
          parameter Real K2 = 1.144 "Bulk threshold parameter";
          parameter Real K5 = 0.7311 "Reduction of pinch-off region";
          parameter Modelica.SIunits.Length dW = -2.5e-6 "Narrowing of channel";
          parameter Modelica.SIunits.Length dL = -1.5e-6 "Shortening of channel";
          parameter Modelica.SIunits.Resistance RDS = 1.e+7 "Drain-Source-Resistance";
          parameter Modelica.SIunits.Temperature Tnom = 300.15 "Parameter measurement temperature";
          parameter Real kvt = -6.96e-3 "Fitting parameter for Vt";
          parameter Real kk2 = 6.0e-4 "Fitting parameter for K2";
          extends Modelica.Electrical.Analog.Interfaces.ConditionalHeatPort(useHeatPort = true);
        protected
          Real v;
          Real uds;
          Real ubs;
          Real ugst;
          Real ud;
          Real us;
          Real id;
          Real gds;
          Real beta_t;
          Real vt_t;
          Real k2_t;
        equation
          assert(L + dL > 0, "Heating NMOS: Effective length must be positive");
          assert(W + dW > 0, "Heating NMOS: Effective width  must be positive");
          assert(T_heatPort > 0, "Heating NMOS: Temperature must be positive");
          gds = if RDS < 1.e-20 and RDS > (-1.e-20) then 1.e20 else 1 / RDS;
          v = beta_t * (W + dW) / (L + dL);
          ud = smooth(0, if D.v < S.v then S.v else D.v);
          us = smooth(0, if D.v < S.v then D.v else S.v);
          uds = ud - us;
          ubs = smooth(0, if B.v > us then 0 else B.v - us);
          ugst = (G.v - us - vt_t + k2_t * ubs) * K5;
          id = smooth(0, if ugst <= 0 then uds * gds else if ugst > uds then v * uds * (ugst - uds / 2) + uds * gds else v * ugst * ugst / 2 + uds * gds);
          beta_t = Beta * pow(T_heatPort / Tnom, -1.5);
          vt_t = Vt * (1 + (T_heatPort - Tnom) * kvt);
          k2_t = K2 * (1 + (T_heatPort - Tnom) * kk2);
          G.i = 0;
          D.i = smooth(0, if D.v < S.v then -id else id);
          S.i = smooth(0, if D.v < S.v then id else -id);
          B.i = 0;
          LossPower = D.i * (D.v - S.v);
          annotation(
            Documentation(info = "<html>
<p>The NMOS model is a simple model of a n-channel metal-oxide semiconductor FET. It differs slightly from the device used in the SPICE simulator. For more details please care for H. Spiro.
<br> A heating port is added for thermal electric simulation. The heating port is defined in the Modelica.Thermal library.
<br>The model does not consider capacitances. A high drain-source resistance RDS is included to avoid numerical difficulties.
</p>
<PRE>
  W       L      Beta         Vt      K2     K5       DW       DL
  m       m      A/V^2        V       -      -        m        m
  12.e-6  4.e-6  0.062e-3    -4.5     0.24   0.61    -1.2e-6  -0.9e-6      depletion
  60.e-6  3.e-6  0.048e-3     0.1     0.08   0.68    -1.2e-6  -0.9e-6      enhancement
  12.e-6  4.e-6  0.0625e-3   -0.8     0.21   0.78    -1.2e-6  -0.9e-6      zero
  50.e-6  8.e-6  0.0299e-3    0.24    1.144  0.7311  -5.4e-6  -4.e-6
  20.e-6  6.e-6  0.041e-3     0.8     1.144  0.7311  -2.5e-6  -1.5e-6
  30.e-6  9.e-6  0.025e-3    -4.0     0.861  0.878   -3.4e-6  -1.74e-6
  30.e-6  5.e-6  0.031e-3     0.6     1.5    0.72     0       -3.9e-6
  50.e-6  6.e-6  0.0414e-3   -3.8     0.34   0.8     -1.6e-6  -2.e-6       depletion
  50.e-6  5.e-6  0.03e-3      0.37    0.23   0.86    -1.6e-6  -2.e-6       enhancement
  50.e-6  6.e-6  0.038e-3    -0.9     0.23   0.707   -1.6e-6  -2.e-6       zero
  20.e-6  4.e-6  0.06776e-3   0.5409  0.065  0.71    -0.8e-6  -0.2e-6
  20.e-6  4.e-6  0.06505e-3   0.6209  0.065  0.71    -0.8e-6  -0.2e-6
  20.e-6  4.e-6  0.05365e-3   0.6909  0.03   0.8     -0.3e-6  -0.2e-6
  20.e-6  4.e-6  0.05365e-3   0.4909  0.03   0.8     -0.3e-6  -0.2e-6
  12.e-6  4.e-6  0.023e-3    -4.5     0.29   0.6      0        0           depletion
  60.e-6  3.e-6  0.022e-3     0.1     0.11   0.65     0        0           enhancement
  12.e-6  4.e-6  0.038e-3    -0.8     0.33   0.6      0        0           zero
  20.e-6  6.e-6  0.022e-3     0.8     1      0.66     0        0
</PRE>
<p><b>References:</b></p>
<p>Spiro, H.: Simulation integrierter Schaltungen. R. Oldenbourg Verlag Muenchen Wien 1990.</p>
</html>", revisions = "<html>
<ul>
<li><i> March 11, 2009   </i>
       by Christoph Clauss<br> conditional heat port added<br>
       </li>
<li><i>December 7, 2005   </i>
       by Christoph Clauss<br>
       error in RDS calculation deleted</li>
<li><i>March 31, 2004   </i>
       by Christoph Clauss<br> implemented<br>
       </li>
</ul>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-90, -50}, {-10, -50}}, color = {0, 0, 255}), Line(points = {{-10, -50}, {-10, 50}}, color = {0, 0, 255}), Line(points = {{10, 70}, {10, 29}}, color = {0, 0, 255}), Line(points = {{10, 20}, {10, -21}}, color = {0, 0, 255}), Line(points = {{10, -30}, {10, -71}}, color = {0, 0, 255}), Line(points = {{10, 50}, {91, 50}}, color = {0, 0, 255}), Line(points = {{10, 0}, {90, 0}}, color = {0, 0, 255}), Line(points = {{10, -50}, {90, -50}}, color = {0, 0, 255}), Polygon(points = {{40, 0}, {60, 5}, {60, -5}, {40, 0}}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Text(extent = {{-152, 118}, {148, 78}}, textString = "%name", lineColor = {0, 0, 255}), Line(visible = useHeatPort, points = {{0, -100}, {0, 0}}, color = {127, 0, 0}, smooth = Smooth.None, pattern = LinePattern.Dot)}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-96, -50}, {-10, -50}}, color = {0, 0, 255}), Line(points = {{-10, -50}, {-10, 50}}, color = {0, 0, 255}), Line(points = {{10, 70}, {10, 29}}, color = {0, 0, 255}), Line(points = {{10, 20}, {10, -21}}, color = {0, 0, 255}), Line(points = {{10, -30}, {10, -71}}, color = {0, 0, 255}), Line(points = {{10, 50}, {96, 50}}, color = {0, 0, 255}), Line(points = {{10, 0}, {96, 0}}, color = {0, 0, 255}), Line(points = {{10, -50}, {96, -50}}, color = {0, 0, 255}), Polygon(points = {{40, 0}, {60, 5}, {60, -5}, {40, 0}}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255})}));
        end HeatingNMOS;

        model HeatingPMOS "Simple PMOS Transistor with heating port"
          Modelica.Electrical.Analog.Interfaces.Pin D "Drain" annotation(
            Placement(transformation(extent = {{90, 40}, {110, 60}}, rotation = 0)));
          Modelica.Electrical.Analog.Interfaces.Pin G "Gate" annotation(
            Placement(transformation(extent = {{-90, -40}, {-110, -60}}, rotation = 0)));
          Modelica.Electrical.Analog.Interfaces.Pin S "Source" annotation(
            Placement(transformation(extent = {{90, -40}, {110, -60}}, rotation = 0)));
          Modelica.Electrical.Analog.Interfaces.Pin B "Bulk" annotation(
            Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
          parameter Modelica.SIunits.Length W = 20.0e-6 "Width";
          parameter Modelica.SIunits.Length L = 6.0e-6 "Length";
          parameter Modelica.SIunits.Transconductance Beta = 0.0105e-3 "Transconductance parameter";
          parameter Modelica.SIunits.Voltage Vt = -1.0 "Zero bias threshold voltage";
          parameter Real K2 = 0.41 "Bulk threshold parameter";
          parameter Real K5 = 0.839 "Reduction of pinch-off region";
          parameter Modelica.SIunits.Length dW = -2.5e-6 "Narrowing of channel";
          parameter Modelica.SIunits.Length dL = -2.1e-6 "Shortening of channel";
          parameter Modelica.SIunits.Resistance RDS = 1.e+7 "Drain-Source-Resistance";
          parameter Modelica.SIunits.Temperature Tnom = 300.15 "Parameter measurement temperature";
          parameter Real kvt = -2.9e-3 "Fitting parameter for Vt";
          parameter Real kk2 = 6.2e-4 "Fitting parameter for K2";
          extends Modelica.Electrical.Analog.Interfaces.ConditionalHeatPort(useHeatPort = true);
        protected
          Real v;
          Real uds;
          Real ubs;
          Real ugst;
          Real ud;
          Real us;
          Real id;
          Real gds;
          Real beta_t;
          Real vt_t;
          Real k2_t;
        equation
          assert(L + dL > 0, "HeatingPMOS: Effective length must be positive");
          assert(W + dW > 0, "HeatingPMOS: Effective width  must be positive");
          assert(T_heatPort > 0, "HeatingPMOS: Temperature must be positive");
          gds = if RDS < 1.e-20 and RDS > (-1.e-20) then 1.e20 else 1 / RDS;
          v = beta_t * (W + dW) / (L + dL);
          ud = smooth(0, if D.v > S.v then S.v else D.v);
          us = smooth(0, if D.v > S.v then D.v else S.v);
          uds = ud - us;
          ubs = smooth(0, if B.v < us then 0 else B.v - us);
          ugst = (G.v - us - vt_t + k2_t * ubs) * K5;
          id = smooth(0, if ugst >= 0 then uds * gds else if ugst < uds then (-v * uds * (ugst - uds / 2)) + uds * gds else (-v * ugst * ugst / 2) + uds * gds);
          beta_t = Beta * pow(T_heatPort / Tnom, -1.5);
          vt_t = Vt * (1 + (T_heatPort - Tnom) * kvt);
          k2_t = K2 * (1 + (T_heatPort - Tnom) * kk2);
          G.i = 0;
          D.i = smooth(0, if D.v > S.v then -id else id);
          S.i = smooth(0, if D.v > S.v then id else -id);
          B.i = 0;
          LossPower = D.i * (D.v - S.v);
          annotation(
            Documentation(info = "<html>
<p>The PMOS model is a simple model of a p-channel metal-oxide semiconductor FET. It differs slightly from the device used in the SPICE simulator. For more details please care for H. Spiro.
<br>A heating port is added for thermal electric simulation. The heating port is defined in the Modelica.Thermal library.
<br>The model does not consider capacitances. A high drain-source resistance RDS is included to avoid numerical difficulties.</p>
<dl><dt><b>References:</b> </dt>
<dd>Spiro, H.: Simulation integrierter Schaltungen. R. Oldenbourg Verlag Muenchen Wien 1990. </dd>
</dl><p>Some typical parameter sets are:</p>
<PRE>
  W       L      Beta        Vt    K2     K5      DW       DL
  m       m      A/V^2       V     -      -       m        m
  50.e-6  8.e-6  0.0085e-3  -0.15  0.41   0.839  -3.8e-6  -4.0e-6
  20.e-6  6.e-6  0.0105e-3  -1.0   0.41   0.839  -2.5e-6  -2.1e-6
  30.e-6  5.e-6  0.0059e-3  -0.3   0.98   1.01    0       -3.9e-6
  30.e-6  5.e-6  0.0152e-3  -0.69  0.104  1.1    -0.8e-6  -0.4e-6
  30.e-6  5.e-6  0.0163e-3  -0.69  0.104  1.1    -0.8e-6  -0.4e-6
  30.e-6  5.e-6  0.0182e-3  -0.69  0.086  1.06   -0.1e-6  -0.6e-6
  20.e-6  6.e-6  0.0074e-3  -1.    0.4    0.59    0        0
</PRE>
</html>", revisions = "<html>
<ul>
<li><i> March 11, 2009   </i>
       by Christoph Clauss<br> conditional heat port added<br>
       </li>
<li><i>December 7, 2005   </i>
       by Christoph Clauss<br>
       error in RDS calculation deleted</li>
<li><i>March 31, 2004   </i>
       by Christoph Clauss<br> implemented<br>
       </li>
</ul>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-90, -50}, {-10, -50}}, color = {0, 0, 255}), Line(points = {{-10, -50}, {-10, 50}}, color = {0, 0, 255}), Line(points = {{10, 70}, {10, 29}}, color = {0, 0, 255}), Line(points = {{10, 20}, {10, -21}}, color = {0, 0, 255}), Line(points = {{10, -30}, {10, -71}}, color = {0, 0, 255}), Line(points = {{10, 50}, {91, 50}}, color = {0, 0, 255}), Line(points = {{10, 0}, {90, 0}}, color = {0, 0, 255}), Line(points = {{10, -50}, {90, -50}}, color = {0, 0, 255}), Polygon(points = {{60, 0}, {40, 5}, {40, -5}, {60, 0}}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Text(extent = {{-155, 120}, {145, 80}}, textString = "%name", lineColor = {0, 0, 255}), Line(visible = useHeatPort, points = {{0, -100}, {0, 0}}, color = {127, 0, 0}, smooth = Smooth.None, pattern = LinePattern.Dot)}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-100, -50}, {-10, -50}}, color = {0, 0, 255}), Line(points = {{-10, -50}, {-10, 50}}, color = {0, 0, 255}), Line(points = {{10, 70}, {10, 29}}, color = {0, 0, 255}), Line(points = {{10, 20}, {10, -21}}, color = {0, 0, 255}), Line(points = {{10, -30}, {10, -71}}, color = {0, 0, 255}), Line(points = {{10, 50}, {96, 50}}, color = {0, 0, 255}), Line(points = {{10, 0}, {101, 0}}, color = {0, 0, 255}), Line(points = {{10, -50}, {100, -50}}, color = {0, 0, 255}), Polygon(points = {{60, 0}, {40, 5}, {40, -5}, {60, 0}}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255})}));
        end HeatingPMOS;

        model HeatingNPN "Simple NPN BJT according to Ebers-Moll with heating port"
          parameter Real Bf = 50 "Forward beta";
          parameter Real Br = 0.1 "Reverse beta";
          parameter Modelica.SIunits.Current Is = 1.e-16 "Transport saturation current";
          parameter Modelica.SIunits.InversePotential Vak = 0.02 "Early voltage (inverse), 1/Volt";
          parameter Modelica.SIunits.Time Tauf = 0.12e-9 "Ideal forward transit time";
          parameter Modelica.SIunits.Time Taur = 5e-9 "Ideal reverse transit time";
          parameter Modelica.SIunits.Capacitance Ccs = 1e-12 "Collector-substrate(ground) cap.";
          parameter Modelica.SIunits.Capacitance Cje = 0.4e-12 "Base-emitter zero bias depletion cap.";
          parameter Modelica.SIunits.Capacitance Cjc = 0.5e-12 "Base-coll. zero bias depletion cap.";
          parameter Modelica.SIunits.Voltage Phie = 0.8 "Base-emitter diffusion voltage";
          parameter Real Me = 0.4 "Base-emitter gradation exponent";
          parameter Modelica.SIunits.Voltage Phic = 0.8 "Base-collector diffusion voltage";
          parameter Real Mc = 0.333 "Base-collector gradation exponent";
          parameter Modelica.SIunits.Conductance Gbc = 1e-15 "Base-collector conductance";
          parameter Modelica.SIunits.Conductance Gbe = 1e-15 "Base-emitter conductance";
          parameter Real EMin = -100 "if x < EMin, the exp(x) function is linearized";
          parameter Real EMax = 40 "if x > EMax, the exp(x) function is linearized";
          parameter Modelica.SIunits.Temperature Tnom = 300.15 "Parameter measurement temperature";
          parameter Real XTI = 3 "Temperature exponent for effect on Is";
          parameter Real XTB = 0 "Forward and reverse beta temperature exponent";
          parameter Real EG = 1.11 "Energy gap for temperature effect on Is";
          parameter Real NF = 1.0 "Forward current emission coefficient";
          parameter Real NR = 1.0 "Reverse current emission coefficient";
          parameter Real K = 1.3806226e-23 "Boltzmann's constant";
          parameter Real q = 1.6021918e-19 "Elementary electronic charge";
          extends Modelica.Electrical.Analog.Interfaces.ConditionalHeatPort(useHeatPort = true);
          /*protected*/
          Real vbc;
          Real vbe;
          Real qbk;
          Real ibc;
          Real ibe;
          Real cbc;
          Real cbe;
          Real ExMin;
          Real ExMax;
          Real Capcje;
          Real Capcjc;
          Real is_t;
          Real br_t;
          Real bf_t;
          Real vt_t;
          Real hexp;
          Real htempexp;
        public
          Modelica.Electrical.Analog.Interfaces.Pin C "Collector" annotation(
            Placement(transformation(extent = {{90, 40}, {110, 60}}, rotation = 0)));
          Modelica.Electrical.Analog.Interfaces.Pin B "Base" annotation(
            Placement(transformation(extent = {{-90, -10}, {-110, 10}}, rotation = 0)));
          Modelica.Electrical.Analog.Interfaces.Pin E "Emitter" annotation(
            Placement(transformation(extent = {{90, -40}, {110, -60}}, rotation = 0)));
        equation
          assert(T_heatPort > 0, "temperature must be positive");
          ExMin = exp(EMin);
          ExMax = exp(EMax);
          vbc = B.v - C.v;
          vbe = B.v - E.v;
          qbk = 1 - vbc * Vak;
          hexp = (T_heatPort / Tnom - 1) * EG / vt_t;
          htempexp = smooth(1, if hexp < EMin then ExMin * (hexp - EMin + 1) else if hexp > EMax then ExMax * (hexp - EMax + 1) else exp(hexp));
          is_t = Is * pow(T_heatPort / Tnom, XTI) * htempexp;
          br_t = Br * pow(T_heatPort / Tnom, XTB);
          bf_t = Bf * pow(T_heatPort / Tnom, XTB);
          vt_t = K / q * T_heatPort;
          ibc = smooth(1, if vbc / (NR * vt_t) < EMin then is_t * (ExMin * (vbc / (NR * vt_t) - EMin + 1) - 1) + vbc * Gbc else if vbc / (NR * vt_t) > EMax then is_t * (ExMax * (vbc / (NR * vt_t) - EMax + 1) - 1) + vbc * Gbc else is_t * (exp(vbc / (NR * vt_t)) - 1) + vbc * Gbc);
          ibe = smooth(1, if vbe / (NF * vt_t) < EMin then is_t * (ExMin * (vbe / (NF * vt_t) - EMin + 1) - 1) + vbe * Gbe else if vbe / (NF * vt_t) > EMax then is_t * (ExMax * (vbe / (NF * vt_t) - EMax + 1) - 1) + vbe * Gbe else is_t * (exp(vbe / (NF * vt_t)) - 1) + vbe * Gbe);
          Capcjc = smooth(1, if vbc / Phic > 0 then Cjc * (1 + Mc * vbc / Phic) else Cjc * pow(1 - vbc / Phic, -Mc));
          Capcje = smooth(1, if vbe / Phie > 0 then Cje * (1 + Me * vbe / Phie) else Cje * pow(1 - vbe / Phie, -Me));
          cbc = smooth(1, if vbc / (NR * vt_t) < EMin then Taur * is_t / (NR * vt_t) * ExMin * (vbc / (NR * vt_t) - EMin + 1) + Capcjc else if vbc / (NR * vt_t) > EMax then Taur * is_t / (NR * vt_t) * ExMax * (vbc / (NR * vt_t) - EMax + 1) + Capcjc else Taur * is_t / (NR * vt_t) * exp(vbc / (NR * vt_t)) + Capcjc);
          cbe = smooth(1, if vbe / (NF * vt_t) < EMin then Tauf * is_t / (NF * vt_t) * ExMin * (vbe / (NF * vt_t) - EMin + 1) + Capcje else if vbe / (NF * vt_t) > EMax then Tauf * is_t / (NF * vt_t) * ExMax * (vbe / (NF * vt_t) - EMax + 1) + Capcje else Tauf * is_t / (NF * vt_t) * exp(vbe / (NF * vt_t)) + Capcje);
          C.i = (ibe - ibc) * qbk - ibc / br_t - cbc * der(vbc) + Ccs * der(C.v);
          B.i = ibe / bf_t + ibc / br_t + cbc * der(vbc) + cbe * der(vbe);
          E.i = (-B.i) - C.i + Ccs * der(C.v);
          LossPower = vbc * ibc / br_t + vbe * ibe / bf_t + (ibe - ibc) * qbk * (C.v - E.v);
          annotation(
            Documentation(info = "<html>
<p>This model is a simple model of a bipolar NPN junction transistor according to Ebers-Moll.
<br>A heating port is added for thermal electric simulation. The heating port is defined in the Modelica.Thermal library.
<br>A typical parameter set is (the parameter Vt is no longer used):</p>
<pre>  Bf  Br  Is     Vak  Tauf    Taur  Ccs   Cje     Cjc     Phie  Me   PHic   Mc     Gbc    Gbe
  -   -   A      V    s       s     F     F       F       V     -    V      -      mS     mS
  50  0.1 1e-16  0.02 0.12e-9 5e-9  1e-12 0.4e-12 0.5e-12 0.8   0.4  0.8    0.333  1e-15  1e-15</pre>
<p><b>References:</b></p>
<p>Vlach, J.; Singal, K.: Computer methods for circuit analysis and design. Van Nostrand Reinhold, New York 1983 on page 317 ff.</p>
</html>", revisions = "<html>
<ul>
<li><i> March 11, 2009   </i>
       by Christoph Clauss<br> conditional heat port added<br>
       </li>
<li><i>March 20, 2004   </i>
       by Christoph Clauss<br> implemented<br>
       </li>
</ul>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-10, 40}, {-10, -40}}, color = {0, 0, 255}), Line(points = {{-10, 0}, {-90, 0}}, color = {0, 0, 255}), Line(points = {{91, 50}, {30, 50}}, color = {0, 0, 255}), Line(points = {{30, 50}, {-10, 10}}, color = {0, 0, 255}), Line(points = {{-10, -10}, {30, -50}}, color = {0, 0, 255}), Line(points = {{30, -50}, {91, -50}}, color = {0, 0, 255}), Polygon(points = {{30, -50}, {24, -36}, {16, -44}, {30, -50}}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Text(extent = {{-162, 125}, {138, 85}}, textString = "%name", lineColor = {0, 0, 255}), Line(visible = useHeatPort, points = {{0, -100}, {0, -20}}, color = {127, 0, 0}, smooth = Smooth.None, pattern = LinePattern.Dot)}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-10, 40}, {-10, -40}}, color = {0, 0, 255}), Line(points = {{-10, 0}, {-100, 0}}, color = {0, 0, 255}), Line(points = {{30, 50}, {-10, 10}}, color = {0, 0, 255}), Line(points = {{-10, -10}, {30, -50}}, color = {0, 0, 255}), Line(points = {{30, -50}, {100, -50}}, color = {0, 0, 255}), Line(points = {{100, 50}, {30, 50}}, color = {0, 0, 255}), Polygon(points = {{30, -50}, {24, -36}, {16, -44}, {30, -50}}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255})}));
        end HeatingNPN;

        model HeatingPNP "Simple PNP BJT according to Ebers-Moll with heating port"
          parameter Real Bf = 50 "Forward beta";
          parameter Real Br = 0.1 "Reverse beta";
          parameter Modelica.SIunits.Current Is = 1.e-16 "Transport saturation current";
          parameter Modelica.SIunits.InversePotential Vak = 0.02 "Early voltage (inverse), 1/Volt";
          parameter Modelica.SIunits.Time Tauf = 0.12e-9 "Ideal forward transit time";
          parameter Modelica.SIunits.Time Taur = 5e-9 "Ideal reverse transit time";
          parameter Modelica.SIunits.Capacitance Ccs = 1e-12 "Collector-substrate(ground) cap.";
          parameter Modelica.SIunits.Capacitance Cje = 0.4e-12 "Base-emitter zero bias depletion cap.";
          parameter Modelica.SIunits.Capacitance Cjc = 0.5e-12 "Base-coll. zero bias depletion cap.";
          parameter Modelica.SIunits.Voltage Phie = 0.8 "Base-emitter diffusion voltage";
          parameter Real Me = 0.4 "Base-emitter gradation exponent";
          parameter Modelica.SIunits.Voltage Phic = 0.8 "Base-collector diffusion voltage";
          parameter Real Mc = 0.333 "Base-collector gradation exponent";
          parameter Modelica.SIunits.Conductance Gbc = 1e-15 "Base-collector conductance";
          parameter Modelica.SIunits.Conductance Gbe = 1e-15 "Base-emitter conductance";
          parameter Real EMin = -100 "if x < EMin, the exp(x) function is linearized";
          parameter Real EMax = 40 "if x > EMax, the exp(x) function is linearized";
          parameter Modelica.SIunits.Temperature Tnom = 300.15 "Parameter measurement temperature";
          parameter Real XTI = 3 "Temperature exponent for effect on Is";
          parameter Real XTB = 0 "Forward and reverse beta temperature exponent";
          parameter Real EG = 1.11 "Energy gap for temperature effect on Is";
          parameter Real NF = 1.0 "Forward current emission coefficient";
          parameter Real NR = 1.0 "Reverse current emission coefficient";
          parameter Real K = 1.3806226e-23 "Boltzmann's constant";
          parameter Real q = 1.6021918e-19 "Elementary electronic charge";
          extends Modelica.Electrical.Analog.Interfaces.ConditionalHeatPort(useHeatPort = true);
        protected
          Real vcb;
          Real veb;
          Real qbk;
          Real icb;
          Real ieb;
          Real ccb;
          Real ceb;
          Real ExMin;
          Real ExMax;
          Real Capcje;
          Real Capcjc;
          Real is_t;
          Real br_t;
          Real bf_t;
          Real vt_t;
          Real hexp;
          Real htempexp;
        public
          Modelica.Electrical.Analog.Interfaces.Pin C "Collector" annotation(
            Placement(transformation(extent = {{90, 40}, {110, 60}}, rotation = 0)));
          Modelica.Electrical.Analog.Interfaces.Pin B "Base" annotation(
            Placement(transformation(extent = {{-90, -10}, {-110, 10}}, rotation = 0)));
          Modelica.Electrical.Analog.Interfaces.Pin E "Emitter" annotation(
            Placement(transformation(extent = {{90, -40}, {110, -60}}, rotation = 0)));
        equation
          assert(T_heatPort > 0, "temperature must be positive");
          ExMin = exp(EMin);
          ExMax = exp(EMax);
          vcb = C.v - B.v;
          veb = E.v - B.v;
          qbk = 1 - vcb * Vak;
          hexp = (T_heatPort / Tnom - 1) * EG / vt_t;
          htempexp = smooth(1, if hexp < EMin then ExMin * (hexp - EMin + 1) else if hexp > EMax then ExMax * (hexp - EMax + 1) else exp(hexp));
          is_t = Is * pow(T_heatPort / Tnom, XTI) * htempexp;
          br_t = Br * pow(T_heatPort / Tnom, XTB);
          bf_t = Bf * pow(T_heatPort / Tnom, XTB);
          vt_t = K / q * T_heatPort;
          icb = smooth(1, if vcb / (NR * vt_t) < EMin then is_t * (ExMin * (vcb / (NR * vt_t) - EMin + 1) - 1) + vcb * Gbc else if vcb / (NR * vt_t) > EMax then is_t * (ExMax * (vcb / (NR * vt_t) - EMax + 1) - 1) + vcb * Gbc else is_t * (exp(vcb / (NR * vt_t)) - 1) + vcb * Gbc);
          ieb = smooth(1, if veb / (NF * vt_t) < EMin then is_t * (ExMin * (veb / (NF * vt_t) - EMin + 1) - 1) + veb * Gbe else if veb / (NF * vt_t) > EMax then is_t * (ExMax * (veb / (NF * vt_t) - EMax + 1) - 1) + veb * Gbe else is_t * (exp(veb / (NF * vt_t)) - 1) + veb * Gbe);
          Capcjc = smooth(1, if vcb / Phic > 0 then Cjc * (1 + Mc * vcb / Phic) else Cjc * pow(1 - vcb / Phic, -Mc));
          Capcje = smooth(1, if veb / Phie > 0 then Cje * (1 + Me * veb / Phie) else Cje * pow(1 - veb / Phie, -Me));
          ccb = smooth(1, if vcb / (NR * vt_t) < EMin then Taur * is_t / (NR * vt_t) * ExMin * (vcb / (NR * vt_t) - EMin + 1) + Capcjc else if vcb / (NR * vt_t) > EMax then Taur * is_t / (NR * vt_t) * ExMax * (vcb / (NR * vt_t) - EMax + 1) + Capcjc else Taur * is_t / (NR * vt_t) * exp(vcb / (NR * vt_t)) + Capcjc);
          ceb = smooth(1, if veb / (NF * vt_t) < EMin then Tauf * is_t / (NF * vt_t) * ExMin * (veb / (NF * vt_t) - EMin + 1) + Capcje else if veb / (NF * vt_t) > EMax then Tauf * is_t / (NF * vt_t) * ExMax * (veb / (NF * vt_t) - EMax + 1) + Capcje else Tauf * is_t / (NF * vt_t) * exp(veb / (NF * vt_t)) + Capcje);
          C.i = icb / br_t + ccb * der(vcb) + Ccs * der(C.v) + (icb - ieb) * qbk;
          B.i = (-ieb / bf_t) - icb / br_t - ceb * der(veb) - ccb * der(vcb);
          E.i = (-B.i) - C.i + Ccs * der(C.v);
          LossPower = vcb * icb / br_t + veb * ieb / bf_t + (icb - ieb) * qbk * (C.v - E.v);
          annotation(
            Documentation(info = "<html>
<p>This model is a simple model of a bipolar PNP junction transistor according to Ebers-Moll.
<br>A heating port is added for thermal electric simulation. The heating port is defined in the Modelica.Thermal library.
<br>A typical parameter set is (the parameter Vt is no longer used):</p>
<pre>  Bf  Br  Is     Vak  Tauf    Taur  Ccs   Cje     Cjc     Phie  Me   PHic   Mc     Gbc    Gbe
  -   -   A      V    s       s     F     F       F       V     -    V      -      mS     mS
  50  0.1 1e-16  0.02 0.12e-9 5e-9  1e-12 0.4e-12 0.5e-12 0.8   0.4  0.8    0.333  1e-15  1e-15</pre>
<p><b>References:</b></p>
<p>Vlach, J.; Singal, K.: Computer methods for circuit analysis and design. Van Nostrand Reinhold, New York 1983 on page 317 ff.</p>
</html>", revisions = "<html>
<ul>
<li><i> March 11, 2009   </i>
       by Christoph Clauss<br> conditional heat port added<br>
       </li>
<li><i>March 20, 2004   </i>
       by Christoph Clauss<br> implemented<br>
       </li>
</ul>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-10, 40}, {-10, -40}}, color = {0, 0, 255}), Line(points = {{-10, 0}, {-90, 0}}, color = {0, 0, 255}), Line(points = {{91, 50}, {30, 50}}, color = {0, 0, 255}), Line(points = {{30, 50}, {-10, 10}}, color = {0, 0, 255}), Line(points = {{-10, -10}, {30, -50}}, color = {0, 0, 255}), Line(points = {{30, -50}, {91, -50}}, color = {0, 0, 255}), Polygon(points = {{-10, -10}, {5, -17}, {-3, -25}, {-10, -10}}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Text(extent = {{-155, 117}, {145, 77}}, textString = "%name", lineColor = {0, 0, 255}), Line(visible = useHeatPort, points = {{0, -100}, {0, -20}}, color = {127, 0, 0}, smooth = Smooth.None, pattern = LinePattern.Dot)}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-10, 40}, {-10, -40}}, color = {0, 0, 255}), Line(points = {{-10, 0}, {-99, 0}}, color = {0, 0, 255}), Line(points = {{96, 50}, {30, 50}}, color = {0, 0, 255}), Line(points = {{30, 50}, {-10, 10}}, color = {0, 0, 255}), Line(points = {{-10, -10}, {30, -50}}, color = {0, 0, 255}), Line(points = {{30, -50}, {96, -50}}, color = {0, 0, 255}), Polygon(points = {{-10, -10}, {5, -17}, {-3, -25}, {-10, -10}}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255})}));
        end HeatingPNP;
      protected
        function pow "Just a helper function for x^y in order that a symbolic engine can apply some transformations more easily"
          extends Modelica.Icons.Function;
          input Real x;
          input Real y;
          output Real z;
        algorithm
          z := x ^ y;
        end pow;

        function exlin "Exponential function linearly continued for x > Maxexp"
          extends Modelica.Icons.Function;
          input Real x;
          input Real Maxexp;
          output Real z;
        algorithm
          z := if x > Maxexp then exp(Maxexp) * (1 + x - Maxexp) else exp(x);
        end exlin;
      public
        model Thyristor "Simple Thyristor Model"
          parameter Modelica.SIunits.Voltage VDRM(final min = 0) = 100 "Forward breakthrough voltage";
          parameter Modelica.SIunits.Voltage VRRM(final min = 0) = 100 "Reverse breakthrough voltage";
          parameter Modelica.SIunits.Current IDRM = 0.1 "Saturation current";
          parameter Modelica.SIunits.Voltage VTM = 1.7 "Conducting voltage";
          parameter Modelica.SIunits.Current IH = 6e-3 "Holding current";
          parameter Modelica.SIunits.Current ITM = 25 "Conducting current";
          parameter Modelica.SIunits.Voltage VGT = 0.7 "Gate trigger voltage";
          parameter Modelica.SIunits.Current IGT = 5e-3 "Gate trigger current";
          parameter Modelica.SIunits.Time TON = 1e-6 "Switch on time";
          parameter Modelica.SIunits.Time TOFF = 15e-6 "Switch off time";
          parameter Modelica.SIunits.Voltage Vt = 0.04 "Voltage equivalent of temperature (kT/qn)";
          parameter Real Nbv = 0.74 "Reverse Breakthrough emission coefficient";
          extends Modelica.Electrical.Analog.Interfaces.ConditionalHeatPort;
          Real iGK "gate current";
          Real vGK "voltage between gate and cathode";
          Real vAK "voltage between anode and cathode";
          Real vControl(start = 0);
          Real vContot;
          Real vConmain;
        public
          Modelica.Electrical.Analog.Interfaces.PositivePin Anode annotation(
            Placement(transformation(extent = {{-95, -12}, {-75, 8}}), iconTransformation(extent = {{-100, -10}, {-80, 10}})));
          Modelica.Electrical.Analog.Interfaces.NegativePin Cathode annotation(
            Placement(transformation(extent = {{80, -10}, {100, 10}}), iconTransformation(extent = {{80, -10}, {100, 10}})));
          Modelica.Electrical.Analog.Interfaces.PositivePin Gate annotation(
            Placement(transformation(extent = {{60, 80}, {80, 100}}), iconTransformation(extent = {{60, 80}, {80, 100}})));
        protected
          parameter Modelica.SIunits.Voltage Von = 5;
          parameter Modelica.SIunits.Voltage Voff = 1.5;
          parameter Modelica.SIunits.Resistance Ron = (VTM - 0.7) / ITM "Forward conducting mode resistance";
          parameter Modelica.SIunits.Resistance Roff = VDRM ^ 2 / VTM / IH "Blocking mode resistance";
        equation
//Kirchhoff's equations
          Anode.i + Gate.i + Cathode.i = 0;
          vGK = Gate.v - Cathode.v;
          vAK = Anode.v - Cathode.v;
// Gate and Control voltage
          iGK = Gate.i;
          vGK = smooth(0, if vGK < 0.65 then VGT / IGT * iGK else 0.65 ^ 2 / VGT + iGK * (VGT - 0.65) / IGT);
          vContot = vConmain + smooth(0, if iGK < 0.95 * IGT then 0 else if iGK < 0.95 * IGT + 1e-3 then 10000 * (iGK - 0.95 * IGT) * vAK else 10 * vAK);
          der(vControl) = (vContot - vControl) / (if vContot - vControl > 0 then 1.87 * TON else 0.638 * TOFF);
// Anode-Cathode characteristics
          Anode.i = smooth(1, if vAK < (-VRRM) then -VRRM / Roff * exp(-(vAK + VRRM) / (Nbv * Vt)) else if vControl < Voff then vAK / Roff else if vControl < Von then vAK / (sqrt(Ron * Roff) * (Ron / Roff) ^ ((3 * ((2 * vControl - Von - Voff) / (2 * (Von - Voff))) - 4 * ((2 * vControl - Von - Voff) / (2 * (Von - Voff))) ^ 3) / 2)) else vAK / Ron);
// holding effect and forward breakthrough
          vConmain = if Anode.i > IH or vAK > VDRM then Von else 0;
          LossPower = Anode.i * Anode.v + Cathode.i * Cathode.v + Gate.i * Gate.v;
          annotation(
            Documentation(info = "<html>
<p>This is a simple thyristor model with three pins: Anode, Cathode and Gate. There are three operating modes:conducting, blocking and reverse breakthrough.
<br>As long as the thyristor is in blocking mode it behaves like a linear resistance Roff=VDRM^2/(VTM*IH). But if the voltage between anode and cathode exceeds VDRM or a positive gate current flows for a sufficient time the mode changes to conducting mode. The model stays in conducting mode until the anode current falls below the holding current IH. There is no way to switch off the thyristor via the gate. If the voltage between anode and cathode is negative, the model represents a diode (parameters Vt, Nbv) with reverse breakthrough voltage VRRM. </p>
<p>The dV/dt switch on is not taken into account in this model. The gate circuit is not influenced by the main circuit.</p>
</html>", revisions = "<html>
<ul>
<li><i>May 12, 2009   </i>
       by Matthias Franke
       </li>
</ul>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{30, 0}, {-30, 40}, {-30, -40}, {30, 0}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{-90, 0}, {40, 0}}, color = {0, 0, 255}), Line(points = {{40, 0}, {90, 0}}, color = {0, 0, 255}), Line(points = {{30, 40}, {30, -40}}, color = {0, 0, 255}), Line(points = {{30, 20}, {70, 60}, {70, 90}}, color = {0, 0, 255}), Line(points = {{40, 50}, {60, 30}}, color = {0, 0, 255}), Text(extent = {{-142, -59}, {158, -99}}, textString = "%name", lineColor = {0, 0, 255})}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-80, 0}, {80, 0}}, color = {128, 128, 128}), Polygon(points = {{70, 4}, {80, 0}, {70, -4}, {70, 4}}, lineColor = {128, 128, 128}, fillColor = {128, 128, 128}, fillPattern = FillPattern.Solid), Line(points = {{0, 80}, {0, -80}}, color = {128, 128, 128}), Polygon(points = {{-4, 70}, {0, 80}, {4, 70}, {-4, 70}}, lineColor = {128, 128, 128}, fillColor = {128, 128, 128}, fillPattern = FillPattern.Solid), Text(extent = {{5, 81}, {15, 71}}, lineColor = {128, 128, 128}, textString = "i"), Text(extent = {{70, -10}, {80, -20}}, lineColor = {128, 128, 128}, textString = "v"), Line(points = {{20, 9}, {20, 0}}, color = {128, 128, 128}, pattern = LinePattern.Dot), Text(extent = {{25, 64}, {45, 54}}, lineColor = {128, 128, 128}, pattern = LinePattern.Dot, textString = "Ron"), Line(points = {{14, 20}, {25, 73}}, color = {0, 0, 0}, thickness = 0.5), Line(points = {{-57, -11}, {55, 9}}, color = {0, 0, 0}, smooth = Smooth.None, thickness = 0.5), Line(points = {{-57, -11}, {-59, -13}, {-60, -18}, {-63, -65}}, color = {0, 0, 0}, thickness = 0.5, smooth = Smooth.None), Text(extent = {{-67, 11}, {-47, 1}}, lineColor = {128, 128, 128}, pattern = LinePattern.Dot, textString = "VRRM"), Text(extent = {{44, -2}, {64, -12}}, lineColor = {128, 128, 128}, pattern = LinePattern.Dot, textString = "VDRM"), Line(points = {{-57, 1}, {-57, -1}}, color = {0, 0, 0}, smooth = Smooth.None), Line(points = {{54, 1}, {54, -1}}, color = {0, 0, 0}, smooth = Smooth.None), Line(points = {{55, 9}, {54, 11}, {50, 13}, {17, 17}, {15, 18}, {14, 20}}, color = {0, 0, 0}, smooth = Smooth.None, pattern = LinePattern.Dot), Text(extent = {{-28, -9}, {-8, -19}}, lineColor = {128, 128, 128}, pattern = LinePattern.Dot, textString = "Roff"), Line(points = {{2, 20}, {-2, 20}}, color = {0, 0, 0}, smooth = Smooth.None), Text(extent = {{-14, 23}, {-2, 16}}, lineColor = {128, 128, 128}, pattern = LinePattern.Dot, textString = "IH")}));
        end Thyristor;

        model SimpleTriac "Simple triac, based on Semiconductors.Thyristor model"
          parameter Modelica.SIunits.Voltage VDRM(final min = 0) = 100 "Forward breakthrough voltage";
          parameter Modelica.SIunits.Voltage VRRM(final min = 0) = 100 "Reverse breakthrough voltage";
          parameter Modelica.SIunits.Current IDRM = 0.1 "Saturation current";
          parameter Modelica.SIunits.Voltage VTM = 1.7 "Conducting voltage";
          parameter Modelica.SIunits.Current IH = 6e-3 "Holding current";
          parameter Modelica.SIunits.Current ITM = 25 "Conducting current";
          parameter Modelica.SIunits.Voltage VGT = 0.7 "Gate trigger voltage";
          parameter Modelica.SIunits.Current IGT = 5e-3 "Gate trigger current";
          parameter Modelica.SIunits.Time TON = 1e-6 "Switch on time";
          parameter Modelica.SIunits.Time TOFF = 15e-6 "Switch off time";
          parameter Modelica.SIunits.Voltage Vt = 0.04 "Voltage equivalent of temperature (kT/qn)";
          parameter Real Nbv = 0.74 "Reverse Breakthrough emission coefficient";
          Modelica.Electrical.Analog.Interfaces.NegativePin n "Cathode" annotation(
            Placement(transformation(extent = {{-110, -10}, {-90, 10}})));
          Modelica.Electrical.Analog.Interfaces.PositivePin p "Anode" annotation(
            Placement(transformation(extent = {{94, -10}, {114, 10}})));
          Modelica.Electrical.Analog.Interfaces.PositivePin g "Gate" annotation(
            Placement(transformation(extent = {{-72, -106}, {-52, -86}})));
          Modelica.Electrical.Analog.Semiconductors.Thyristor thyristor(VDRM = VDRM, VRRM = VRRM, useHeatPort = useHeatPort, T = T) annotation(
            Placement(transformation(extent = {{-20, 30}, {0, 50}})));
          Modelica.Electrical.Analog.Semiconductors.Thyristor thyristor1(VDRM = VDRM, VRRM = VRRM, useHeatPort = useHeatPort, T = T) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 180, origin = {-12, -40})));
          Modelica.Electrical.Analog.Ideal.IdealDiode idealDiode(Vknee = 0) annotation(
            Placement(transformation(extent = {{-40, 58}, {-20, 78}})));
          Modelica.Electrical.Analog.Ideal.IdealDiode idealDiode1(Vknee = 0) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 90, origin = {-20, -72})));
          parameter Boolean useHeatPort = false "=true, if HeatPort is enabled" annotation(
            Evaluate = true,
            HideResult = true,
            choices(checkBox = true));
          parameter Modelica.SIunits.Temperature T = 293.15 "Fixed device temperature if useHeatPort = false" annotation(
            Dialog(enable = not useHeatPort));
          Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a heatPort if useHeatPort annotation(
            Placement(transformation(extent = {{-10, -110}, {10, -90}}), iconTransformation(extent = {{-10, -110}, {10, -90}})));
        equation
          if useHeatPort then
            connect(heatPort, thyristor.heatPort);
            connect(heatPort, thyristor1.heatPort);
          end if;
          connect(thyristor.Anode, n) annotation(
            Line(points = {{-19, 40}, {-18, 40}, {-18, 48}, {-70, 48}, {-70, 0}, {-100, 0}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(thyristor1.Anode, p) annotation(
            Line(points = {{-3, -40}, {-2, -40}, {-2, -60}, {80, -60}, {80, 0}, {104, 0}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(thyristor1.Anode, thyristor.Cathode) annotation(
            Line(points = {{-3, -40}, {-2, -40}, {-2, 40}, {-1, 40}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(thyristor1.Cathode, thyristor.Anode) annotation(
            Line(points = {{-21, -40}, {-22, -40}, {-22, 40}, {-19, 40}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(thyristor.Gate, idealDiode.n) annotation(
            Line(points = {{-3, 49}, {-3, 59.5}, {-20, 59.5}, {-20, 68}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(idealDiode.p, g) annotation(
            Line(points = {{-40, 68}, {-82, 68}, {-82, -96}, {-62, -96}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(idealDiode1.n, thyristor1.Gate) annotation(
            Line(points = {{-20, -62}, {-20, -49}, {-19, -49}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(idealDiode1.p, g) annotation(
            Line(points = {{-20, -82}, {-42, -82}, {-42, -96}, {-62, -96}}, color = {0, 0, 255}, smooth = Smooth.None));
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{-30, 0}, {-30, -100}, {70, -50}, {-30, 0}}, lineColor = {0, 0, 255}, smooth = Smooth.None), Polygon(points = {{70, 100}, {70, 0}, {-30, 50}, {70, 100}}, lineColor = {0, 0, 255}, smooth = Smooth.None), Line(points = {{70, 0}, {70, -100}}, color = {0, 0, 255}, smooth = Smooth.None), Line(points = {{-30, 0}, {-30, 100}}, color = {0, 0, 255}, smooth = Smooth.None), Line(points = {{-30, 0}, {-90, 0}}, color = {0, 0, 255}, smooth = Smooth.None), Line(points = {{70, 0}, {110, 0}}, color = {0, 0, 255}, smooth = Smooth.None), Line(points = {{-62, -86}, {-62, -56}, {-30, -44}}, color = {0, 0, 255}, smooth = Smooth.None), Text(extent = {{-102, 130}, {98, 100}}, textString = "%name", lineColor = {0, 0, 255})}),
            Documentation(info = "<html>
<p>This is a simple TRIAC model based on the extended thyristor model Modelica.Electrical.Analog.Semiconductors.Thyristor.
<br>Two thyristors are contrarily connected in parallel, whereas each transistor is connected with a diode.
<br>Further information regarding the electrical component TRIAC can be detected in documentation of the ideal TRIAC model.
<br>As an additional information: this model is based on the Modelica.Electrical.Analog.Semiconductors.Thyristor.</p>
<p><b>Attention:</b> The model seems to be very sensitive with respect to the choice of some parameters (e.g., VDRM, VRRM). This is caused by the thyristor model. Further investigations are necessary.</p>
</html>", revisions = "<html>
<ul>
<li><i>November 25, 2009   </i> <br>

       by Susann Wolf <br><br>
       </li>
</ul>
</html>"));
        end SimpleTriac;
        annotation(
          Documentation(info = "<html>
<p>This package contains semiconductor devices:</p>
<ul>
<li>diode</li>
<li>MOS transistors</li>
<li>bipolar transistors</li>
<li>thyristor</li>
<li>triac</li>
</ul>
<p>Most of the semiconductor devices contain a conditional heat port, which is not active by default. If it is active the loss power is calculated to be used in a thermal net. The heating variants of the semiconductor devices are provided to use the thermal port temperature in the electric calculation. That means that for a true thermal electric interaction the heating device models have to be used.</p>
</html>", revisions = "<html>
<dl>
<dt>
<b>Main Authors:</b>
</dt>
<dd>
Christoph Clau&szlig;
    &lt;<a href=\"mailto:Christoph.Clauss@eas.iis.fraunhofer.de\">Christoph.Clauss@eas.iis.fraunhofer.de</a>&gt;<br>
    Andr&eacute; Schneider
    &lt;<a href=\"mailto:Andre.Schneider@eas.iis.fraunhofer.de\">Andre.Schneider@eas.iis.fraunhofer.de</a>&gt;<br>
    Fraunhofer Institute for Integrated Circuits<br>
    Design Automation Department<br>
    Zeunerstra&szlig;e 38<br>
    D-01069 Dresden
</dl>
</html>"),
          Icon(graphics = {Line(points = {{-20, 0}, {-90, 0}}, color = {0, 0, 0}), Line(points = {{-20, 60}, {-20, -60}}, color = {0, 0, 0}), Line(points = {{20, 68}, {-20, 28}}, color = {0, 0, 0}), Line(points = {{80, 68}, {20, 68}}, color = {0, 0, 0}), Line(points = {{-20, -30}, {20, -70}}, color = {0, 0, 0}), Line(points = {{20, -70}, {80, -70}}, color = {0, 0, 0})}));
      end Semiconductors;

      package Sensors "Potential, voltage, current, and power sensors"
        extends Modelica.Icons.SensorsPackage;

        model PotentialSensor "Sensor to measure the potential"
          extends Modelica.Icons.RotationalSensor;
          Interfaces.PositivePin p "pin to be measured" annotation(
            Placement(transformation(extent = {{-110, -10}, {-90, 10}}, rotation = 0)));
          Modelica.Blocks.Interfaces.RealOutput phi(unit = "V") "Absolute voltage potential as output signal" annotation(
            Placement(transformation(extent = {{100, -10}, {120, 10}}, rotation = 0)));
        equation
          p.i = 0;
          phi = p.v;
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-29, -11}, {30, -70}}, lineColor = {0, 0, 0}, textString = "V"), Line(points = {{-70, 0}, {-90, 0}}, color = {0, 0, 255}), Line(points = {{100, 0}, {70, 0}}, color = {0, 0, 127}), Text(extent = {{-150, 80}, {150, 120}}, textString = "%name", lineColor = {0, 0, 255})}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-70, 0}, {-96, 0}}, color = {0, 0, 0}), Line(points = {{100, 0}, {70, 0}}, color = {0, 0, 127})}),
            Documentation(revisions = "<html>
<ul>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>", info = "<html>
<p>The potential sensor converts the voltage of a node (with respect to the ground node) into a real valued signal. It does not influence the current sum at the node which voltage is measured, therefore, the electrical behavior is not influenced by the sensor.</p>
</html>"));
        end PotentialSensor;

        model VoltageSensor "Sensor to measure the voltage between two pins"
          extends Modelica.Icons.RotationalSensor;
          Interfaces.PositivePin p "positive pin" annotation(
            Placement(transformation(extent = {{-110, -10}, {-90, 10}}, rotation = 0)));
          Interfaces.NegativePin n "negative pin" annotation(
            Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
          Modelica.Blocks.Interfaces.RealOutput v(unit = "V") "Voltage between pin p and n (= p.v - n.v) as output signal" annotation(
            Placement(transformation(origin = {0, -100}, extent = {{10, -10}, {-10, 10}}, rotation = 90)));
        equation
          p.i = 0;
          n.i = 0;
          v = p.v - n.v;
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-29, -11}, {30, -70}}, lineColor = {0, 0, 0}, textString = "V"), Line(points = {{-70, 0}, {-90, 0}}, color = {0, 0, 255}), Line(points = {{70, 0}, {90, 0}}, color = {0, 0, 255}), Line(points = {{0, -90}, {0, -70}}, color = {0, 0, 127}), Text(extent = {{-150, 80}, {150, 120}}, textString = "%name", lineColor = {0, 0, 255})}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-70, 0}, {-96, 0}}, color = {0, 0, 0}), Line(points = {{70, 0}, {96, 0}}, color = {0, 0, 0}), Line(points = {{0, -90}, {0, -70}}, color = {0, 0, 255})}),
            Documentation(revisions = "<html>
<ul>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>", info = "<html>
<p>The voltage sensor converts the voltage between the two connectors into a real valued signal. It does not influence the current sum at the nodes in between the voltage is measured, therefore, the electrical behavior is not influenced by the sensor.</p>
</html>"));
        end VoltageSensor;

        model CurrentSensor "Sensor to measure the current in a branch"
          extends Modelica.Icons.RotationalSensor;
          Interfaces.PositivePin p "positive pin" annotation(
            Placement(transformation(extent = {{-110, -10}, {-90, 10}}, rotation = 0)));
          Interfaces.NegativePin n "negative pin" annotation(
            Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
          Modelica.Blocks.Interfaces.RealOutput i(unit = "A") "Current in the branch from p to n as output signal" annotation(
            Placement(transformation(origin = {0, -100}, extent = {{10, -10}, {-10, 10}}, rotation = 90)));
        equation
          p.v = n.v;
          p.i = i;
          n.i = -i;
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-29, -11}, {30, -70}}, lineColor = {0, 0, 0}, textString = "A"), Line(points = {{-70, 0}, {-90, 0}}, color = {0, 0, 255}), Text(extent = {{-150, 80}, {150, 120}}, textString = "%name", lineColor = {0, 0, 255}), Line(points = {{70, 0}, {90, 0}}, color = {0, 0, 255}), Line(points = {{0, -90}, {0, -70}}, color = {0, 0, 127})}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-153, 79}, {147, 119}}, textString = "%name", lineColor = {0, 0, 255}), Line(points = {{-70, 0}, {-96, 0}}, color = {0, 0, 0}), Line(points = {{70, 0}, {96, 0}}, color = {0, 0, 0}), Line(points = {{0, -90}, {0, -70}}, color = {0, 0, 255})}),
            Documentation(revisions = "<html>
<ul>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>", info = "<html>
<p>The current sensor converts the current flowing between the two connectors into a real valued signal. The two connectors are in the sensor connected like a short cut. The sensor has to be placed within an electrical connection in series.  It does not influence the current sum at the connected nodes. Therefore, the electrical behavior is not influenced by the sensor.</p>
</html>"));
        end CurrentSensor;

        model PowerSensor "Sensor to measure the power"
          extends Modelica.Icons.RotationalSensor;
          Modelica.Electrical.Analog.Interfaces.PositivePin pc "Positive pin, current path" annotation(
            Placement(transformation(extent = {{-90, -10}, {-110, 10}}, rotation = 0)));
          Modelica.Electrical.Analog.Interfaces.NegativePin nc "Negative pin, current path" annotation(
            Placement(transformation(extent = {{110, -10}, {90, 10}}, rotation = 0)));
          Modelica.Electrical.Analog.Interfaces.PositivePin pv "Positive pin, voltage path" annotation(
            Placement(transformation(extent = {{-10, 110}, {10, 90}}, rotation = 0)));
          Modelica.Electrical.Analog.Interfaces.NegativePin nv "Negative pin, voltage path" annotation(
            Placement(transformation(extent = {{10, -110}, {-10, -90}}, rotation = 0)));
          Modelica.Blocks.Interfaces.RealOutput power(unit = "W") "Instantaneous power as output signal" annotation(
            Placement(transformation(origin = {-80, -110}, extent = {{-10, 10}, {10, -10}}, rotation = 270)));
          Modelica.Electrical.Analog.Sensors.VoltageSensor voltageSensor annotation(
            Placement(transformation(origin = {0, -30}, extent = {{10, 10}, {-10, -10}}, rotation = 90)));
          Modelica.Electrical.Analog.Sensors.CurrentSensor currentSensor annotation(
            Placement(transformation(extent = {{-50, -10}, {-30, 10}}, rotation = 0)));
          Modelica.Blocks.Math.Product product annotation(
            Placement(transformation(origin = {-30, -50}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
        equation
          connect(pv, voltageSensor.p) annotation(
            Line(points = {{0, 100}, {0, -20}}, color = {0, 0, 255}));
          connect(voltageSensor.n, nv) annotation(
            Line(points = {{0, -40}, {0, -63}, {0, -100}}, color = {0, 0, 255}));
          connect(pc, currentSensor.p) annotation(
            Line(points = {{-100, 0}, {-50, 0}}, color = {0, 0, 255}));
          connect(currentSensor.n, nc) annotation(
            Line(points = {{-30, 0}, {100, 0}}, color = {0, 0, 255}));
          connect(currentSensor.i, product.u2) annotation(
            Line(points = {{-40, -10}, {-40, -30}, {-36, -30}, {-36, -38}}, color = {0, 0, 127}));
          connect(voltageSensor.v, product.u1) annotation(
            Line(points = {{-10, -30}, {-24, -30}, {-24, -38}}, color = {0, 0, 127}));
          connect(product.y, power) annotation(
            Line(points = {{-30, -61}, {-30, -80}, {-80, -80}, {-80, -110}}, color = {0, 0, 127}));
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{0, 100}, {0, 70}}, color = {0, 0, 255}), Line(points = {{0, -70}, {0, -100}}, color = {0, 0, 255}), Line(points = {{-80, -100}, {-80, -80}, {-46, -52}}, color = {0, 0, 127}), Line(points = {{-100, 0}, {100, 0}}, color = {0, 0, 255}), Text(lineColor = {0, 0, 255}, extent = {{-150, 120}, {150, 160}}, textString = "%name"), Line(points = {{0, 70}, {0, 40}}), Text(extent = {{-29, -70}, {30, -11}}, textString = "P")}),
            Documentation(info = "<html>
<p>This power sensor measures instantaneous electrical power of a singlephase system and has a separated voltage and current path. The pins of the voltage path are pv and nv, the pins of the current path are pc and nc. The internal resistance of the current path is zero, the internal resistance of the voltage path is infinite.</p>
</html>", revisions = "<html>
<ul>
<li><i>January 12, 2006</i> by Anton Haumer implemented</li>
</ul>
</html>"));
        end PowerSensor;
        annotation(
          Documentation(info = "<html>
<p>This package contains potential, voltage, and current sensors. The sensors can be used to convert voltages or currents into real signal values o be connected to components of the Blocks package. The sensors are designed in such a way that they do not influence the electrical behavior.</p>
</html>", revisions = "<html>
<dl>
<dt>
<b>Main Authors:</b>
</dt>
<dd>
Christoph Clau&szlig;
    &lt;<a href=\"mailto:Christoph.Clauss@eas.iis.fraunhofer.de\">Christoph.Clauss@eas.iis.fraunhofer.de</a>&gt;<br>
    Andr&eacute; Schneider
    &lt;<a href=\"mailto:Andre.Schneider@eas.iis.fraunhofer.de\">Andre.Schneider@eas.iis.fraunhofer.de</a>&gt;<br>
    Fraunhofer Institute for Integrated Circuits<br>
    Design Automation Department<br>
    Zeunerstra&szlig;e 38<br>
    D-01069 Dresden
</dd>
<dt>
<b>Copyright:</b>
</dt>
<dd>
Copyright &copy; 1998-2015, Modelica Association and Fraunhofer-Gesellschaft.<br>
<i>The Modelica package is <b>free</b> software; it can be redistributed and/or modified
under the terms of the <b>Modelica license</b>, see the license conditions
and the accompanying <b>disclaimer</b> in the documentation of package
Modelica in file \"Modelica/package.mo\".</i>
</dd>
</dl>
</html>"));
      end Sensors;

      package Sources "Time-dependent and controlled voltage and current sources"
        extends Modelica.Icons.SourcesPackage;

        model SignalVoltage "Generic voltage source using the input signal as source voltage"
          Interfaces.PositivePin p annotation(
            Placement(transformation(extent = {{-110, -10}, {-90, 10}}, rotation = 0)));
          Interfaces.NegativePin n annotation(
            Placement(transformation(extent = {{110, -10}, {90, 10}}, rotation = 0)));
          Modelica.Blocks.Interfaces.RealInput v(unit = "V") "Voltage between pin p and n (= p.v - n.v) as input signal" annotation(
            Placement(transformation(origin = {0, 70}, extent = {{-20, -20}, {20, 20}}, rotation = 270)));
          SI.Current i "Current flowing from pin p to pin n";
        equation
          v = p.v - n.v;
          0 = p.i + n.i;
          i = p.i;
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-50, 50}, {50, -50}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{-90, 0}, {-50, 0}}, color = {0, 0, 255}), Line(points = {{50, 0}, {90, 0}}, color = {0, 0, 255}), Line(points = {{-50, 0}, {50, 0}}, color = {0, 0, 255}), Text(extent = {{-150, -104}, {150, -64}}, textString = "%name", lineColor = {0, 0, 255}), Text(extent = {{-120, 50}, {-20, 0}}, lineColor = {0, 0, 255}, textString = "+"), Text(extent = {{20, 50}, {120, 0}}, lineColor = {0, 0, 255}, textString = "-")}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-50, 50}, {50, -50}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{-96, 0}, {-50, 0}}, color = {0, 0, 255}), Line(points = {{50, 0}, {96, 0}}, color = {0, 0, 255}), Line(points = {{-50, 0}, {50, 0}}, color = {0, 0, 255}), Line(points = {{-109, 20}, {-84, 20}}, color = {160, 160, 164}), Polygon(points = {{-94, 23}, {-84, 20}, {-94, 17}, {-94, 23}}, lineColor = {160, 160, 164}, fillColor = {160, 160, 164}, fillPattern = FillPattern.Solid), Line(points = {{91, 20}, {116, 20}}, color = {160, 160, 164}), Text(extent = {{-109, 25}, {-89, 45}}, lineColor = {160, 160, 164}, textString = "i"), Polygon(points = {{106, 23}, {116, 20}, {106, 17}, {106, 23}}, lineColor = {160, 160, 164}, fillColor = {160, 160, 164}, fillPattern = FillPattern.Solid), Text(extent = {{91, 45}, {111, 25}}, lineColor = {160, 160, 164}, textString = "i"), Line(points = {{-119, -5}, {-119, 5}}, color = {160, 160, 164}), Line(points = {{-124, 0}, {-114, 0}}, color = {160, 160, 164}), Line(points = {{116, 0}, {126, 0}}, color = {160, 160, 164})}),
            Documentation(revisions = "<html>
<ul>
<li><i> 1998   </i>
       by Martin Otter<br> initially implemented<br>
       </li>
</ul>
</html>", info = "<html>
<p>The signal voltage source is a parameterless converter of real valued signals into a the source voltage. No further effects are modeled. The real valued signal has to be provided by components of the blocks library. It can be regarded as the &quot;Opposite&quot; of a voltage sensor.</p>
</html>"));
        end SignalVoltage;

        model ConstantVoltage "Source for constant voltage"
          parameter SI.Voltage V(start = 1) "Value of constant voltage";
          extends Interfaces.OnePort;
        equation
          v = V;
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-90, 0}, {-10, 0}}, color = {0, 0, 255}), Line(points = {{-10, 60}, {-10, -60}}, color = {0, 0, 255}), Line(points = {{0, 30}, {0, -30}}, color = {0, 0, 255}), Line(points = {{0, 0}, {90, 0}}, color = {0, 0, 255}), Text(extent = {{-150, -112}, {150, -72}}, textString = "%name=%V", lineColor = {0, 0, 255}), Text(extent = {{-120, 50}, {-20, 0}}, lineColor = {0, 0, 255}, textString = "+"), Text(extent = {{20, 50}, {120, 0}}, lineColor = {0, 0, 255}, textString = "-")}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-96, 0}, {-10, 0}}, color = {0, 0, 255}), Line(points = {{-10, 60}, {-10, -60}}, color = {0, 0, 255}), Line(points = {{0, 30}, {0, -30}}, color = {0, 0, 255}), Line(points = {{0, 0}, {96, 0}}, color = {0, 0, 255})}),
            Documentation(revisions = "<html>
<ul>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>", info = "<html>
<p>The ConstantVoltage source is a simple source for an ideal constant voltage which is provided by a parameter. There is no internal resistance modeled. If it is used instead of a battery model it is not very realistic: This battery will never be unloaded.</p>
</html>"));
        end ConstantVoltage;

        model StepVoltage "Step voltage source"
          parameter SI.Voltage V(start = 1) "Height of step";
          extends Interfaces.VoltageSource(redeclare Modelica.Blocks.Sources.Step signalSource(height = V));
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-70, -70}, {0, -70}, {0, 70}, {69, 70}}, color = {192, 192, 192})}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{-80, 90}, {-88, 68}, {-72, 68}, {-80, 90}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-80, 68}, {-80, -80}}, color = {192, 192, 192}), Line(points = {{-80, -18}, {0, -18}, {0, 50}, {80, 50}}, color = {0, 0, 0}, thickness = 0.5), Line(points = {{-90, -70}, {82, -70}}, color = {192, 192, 192}), Polygon(points = {{90, -70}, {68, -62}, {68, -78}, {90, -70}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Text(extent = {{70, -80}, {94, -100}}, lineColor = {160, 160, 164}, textString = "time"), Text(extent = {{-21, -72}, {25, -90}}, lineColor = {160, 160, 164}, textString = "startTime"), Line(points = {{0, -17}, {0, -71}}, color = {192, 192, 192}, pattern = LinePattern.Dash), Text(extent = {{-68, -36}, {-22, -54}}, lineColor = {160, 160, 164}, textString = "offset"), Line(points = {{-13, 50}, {-13, -17}}, color = {192, 192, 192}, pattern = LinePattern.Solid, thickness = 0.25, arrow = {Arrow.None, Arrow.None}), Polygon(points = {{0, 50}, {-19, 50}, {0, 50}}, lineColor = {192, 192, 192}, pattern = LinePattern.Dash), Polygon(points = {{-13, -17}, {-16, -4}, {-10, -4}, {-13, -17}, {-13, -17}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Polygon(points = {{-13, 50}, {-16, 37}, {-9, 37}, {-13, 50}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Text(extent = {{-68, 26}, {-22, 8}}, lineColor = {160, 160, 164}, textString = "V"), Polygon(points = {{-13, -69}, {-16, -56}, {-10, -56}, {-13, -69}, {-13, -69}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-13, -18}, {-13, -70}}, color = {192, 192, 192}, pattern = LinePattern.Solid, thickness = 0.25, arrow = {Arrow.None, Arrow.None}), Polygon(points = {{-13, -18}, {-16, -31}, {-9, -31}, {-13, -18}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Text(extent = {{-67, 93}, {-2, 67}}, lineColor = {160, 160, 164}, textString = "v = p.v - n.v")}),
            Documentation(revisions = "<html>
<ul>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>", info = "<html>
<p>This voltage source uses the corresponding signal source of the Modelica.Blocks.Sources package. Care for the meaning of the parameters in the Blocks package. Furthermore, an offset parameter is introduced, which is added to the value calculated by the blocks source. The startTime parameter allows to shift the blocks source behavior on the time axis.</p>
</html>"));
        end StepVoltage;

        model RampVoltage "Ramp voltage source"
          parameter SI.Voltage V(start = 1) "Height of ramp";
          parameter SI.Time duration(min = Modelica.Constants.small, start = 2) "Duration of ramp";
          extends Interfaces.VoltageSource(redeclare Modelica.Blocks.Sources.Ramp signalSource(final height = V, final duration = duration));
          annotation(
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{-80, 90}, {-88, 68}, {-72, 68}, {-80, 90}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-80, 68}, {-80, -80}}, color = {192, 192, 192}), Line(points = {{-80, -20}, {-20, -20}, {50, 50}}, color = {0, 0, 0}, thickness = 0.5), Line(points = {{-90, -70}, {82, -70}}, color = {192, 192, 192}), Polygon(points = {{90, -70}, {68, -62}, {68, -78}, {90, -70}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Polygon(points = {{-40, -20}, {-42, -30}, {-38, -30}, {-40, -20}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-40, -20}, {-40, -70}}, color = {192, 192, 192}, pattern = LinePattern.Solid, thickness = 0.25, arrow = {Arrow.None, Arrow.None}), Polygon(points = {{-40, -70}, {-42, -60}, {-38, -60}, {-40, -70}, {-40, -70}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Text(extent = {{-80, -33}, {-41, -49}}, lineColor = {160, 160, 164}, textString = "offset"), Text(extent = {{-40, -70}, {6, -88}}, lineColor = {160, 160, 164}, textString = "startTime"), Text(extent = {{-67, 93}, {-2, 67}}, lineColor = {160, 160, 164}, textString = "v = p.v - n.v"), Text(extent = {{70, -80}, {94, -100}}, lineColor = {160, 160, 164}, textString = "time"), Line(points = {{-20, -20}, {-20, -70}}, color = {192, 192, 192}, pattern = LinePattern.Dash), Line(points = {{-19, -20}, {50, -20}}, color = {192, 192, 192}, pattern = LinePattern.Solid, thickness = 0.25, arrow = {Arrow.None, Arrow.None}), Line(points = {{50, 50}, {101, 50}}, color = {0, 0, 0}, thickness = 0.5), Line(points = {{50, 50}, {50, -20}}, color = {192, 192, 192}, pattern = LinePattern.Solid, thickness = 0.25, arrow = {Arrow.None, Arrow.None}), Polygon(points = {{50, -20}, {42, -18}, {42, -22}, {50, -20}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Polygon(points = {{-20, -20}, {-11, -18}, {-11, -22}, {-20, -20}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Polygon(points = {{50, 50}, {48, 40}, {52, 40}, {50, 50}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Polygon(points = {{50, -20}, {48, -10}, {52, -10}, {50, -20}, {50, -20}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Text(extent = {{53, 25}, {82, 7}}, lineColor = {160, 160, 164}, textString = "V"), Text(extent = {{0, -17}, {35, -37}}, lineColor = {160, 160, 164}, textString = "duration")}),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-80, -60}, {-50, -60}, {50, 60}, {80, 60}}, color = {192, 192, 192})}),
            Documentation(revisions = "<html>
<ul>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>", info = "<html>
<p>This voltage source uses the corresponding signal source of the Modelica.Blocks.Sources package. Care for the meaning of the parameters in the Blocks package. Furthermore, an offset parameter is introduced, which is added to the value calculated by the blocks source. The startTime parameter allows to shift the blocks source behavior on the time axis.</p>
</html>"));
        end RampVoltage;

        model SineVoltage "Sine voltage source"
          parameter SI.Voltage V(start = 1) "Amplitude of sine wave";
          parameter SI.Angle phase = 0 "Phase of sine wave";
          parameter SI.Frequency freqHz(start = 1) "Frequency of sine wave";
          extends Interfaces.VoltageSource(redeclare Modelica.Blocks.Sources.Sine signalSource(amplitude = V, freqHz = freqHz, phase = phase));
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-66, 0}, {-56.2, 29.9}, {-49.8, 46.5}, {-44.2, 58.1}, {-39.3, 65.2}, {-34.3, 69.2}, {-29.4, 69.8}, {-24.5, 67}, {-19.6, 61}, {-14.6, 52}, {-9, 38.6}, {-1.98, 18.6}, {12.79, -26.9}, {19.1, -44}, {24.8, -56.2}, {29.7, -64}, {34.6, -68.6}, {39.5, -70}, {44.5, -67.9}, {49.4, -62.5}, {54.3, -54.1}, {59.9, -41.3}, {67, -21.7}, {74, 0}}, color = {192, 192, 192})}),
            Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-80, -90}, {-80, 84}}, color = {192, 192, 192}), Polygon(points = {{-80, 100}, {-86, 84}, {-74, 84}, {-80, 100}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-99, -40}, {100, -40}}, color = {192, 192, 192}), Polygon(points = {{100, -40}, {84, -34}, {84, -46}, {100, -40}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-41, -2}, {-32.6, 32.2}, {-27.1, 51.1}, {-22.3, 64.4}, {-18.1, 72.6}, {-13.9, 77.1}, {-8, 78}, {-5.42, 74.6}, {-1.201, 67.7}, {3.02, 57.4}, {7.84, 42.1}, {13.9, 19.2}, {26.5, -32.8}, {32, -52.2}, {36.8, -66.2}, {41, -75.1}, {45.2, -80.4}, {49.5, -82}, {53.7, -79.6}, {57.9, -73.5}, {62.1, -63.9}, {66.9, -49.2}, {73, -26.8}, {79, -2}}, color = {0, 0, 0}, thickness = 0.5), Line(points = {{-41, -2}, {-80, -2}}, color = {0, 0, 0}, thickness = 0.5), Text(extent = {{-106, -11}, {-60, -29}}, lineColor = {160, 160, 164}, textString = "offset"), Line(points = {{-41, -2}, {-41, -40}}, color = {192, 192, 192}, pattern = LinePattern.Dash), Text(extent = {{-60, -43}, {-14, -61}}, lineColor = {160, 160, 164}, textString = "startTime"), Text(extent = {{76, -52}, {100, -72}}, lineColor = {160, 160, 164}, textString = "time"), Line(points = {{-8, 78}, {45, 78}}, color = {192, 192, 192}, pattern = LinePattern.Dash), Line(points = {{-41, -2}, {52, -2}}, color = {192, 192, 192}, pattern = LinePattern.Dash), Polygon(points = {{33, 78}, {30, 65}, {37, 65}, {33, 78}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Text(extent = {{37, 57}, {83, 39}}, lineColor = {160, 160, 164}, textString = "V"), Polygon(points = {{33, -2}, {30, 11}, {36, 11}, {33, -2}, {33, -2}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{33, 78}, {33, -2}}, color = {192, 192, 192}, pattern = LinePattern.Solid, thickness = 0.25, arrow = {Arrow.None, Arrow.None}), Text(extent = {{-69, 109}, {-4, 83}}, lineColor = {160, 160, 164}, textString = "v = p.v - n.v")}),
            Documentation(revisions = "<html>
<ul>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>", info = "<html>
<p>This voltage source uses the corresponding signal source of the Modelica.Blocks.Sources package. Care for the meaning of the parameters in the Blocks package. Furthermore, an offset parameter is introduced, which is added to the value calculated by the blocks source. The startTime parameter allows to shift the blocks source behavior on the time axis.</p>
</html>"));
        end SineVoltage;

        model CosineVoltage "Cosine voltage source"
          parameter SI.Voltage V(start = 1) "Amplitude of cosine wave";
          parameter SI.Angle phase = 0 "Phase of cosine wave";
          parameter SI.Frequency freqHz(start = 1) "Frequency of cosine wave";
          extends Interfaces.VoltageSource(redeclare Modelica.Blocks.Sources.Cosine signalSource(amplitude = V, freqHz = freqHz, phase = phase));
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-71, 70}, {-68.4, 69.8}, {-63.5, 67}, {-58.6, 61}, {-53.6, 52}, {-48, 38.6}, {-40.98, 18.6}, {-26.21, -26.9}, {-19.9, -44}, {-14.2, -56.2}, {-9.3, -64}, {-4.4, -68.6}, {0.5, -70}, {5.5, -67.9}, {10.4, -62.5}, {15.3, -54.1}, {20.9, -41.3}, {28, -21.7}, {35, 0}}, color = {192, 192, 192}, smooth = Smooth.Bezier), Line(points = {{35, 0}, {44.8, 29.9}, {51.2, 46.5}, {56.8, 58.1}, {61.7, 65.2}, {66.7, 69.2}, {71.6, 69.8}}, color = {192, 192, 192})}),
            Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-80, -90}, {-80, 84}}, color = {192, 192, 192}), Polygon(points = {{-80, 100}, {-86, 84}, {-74, 84}, {-80, 100}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-99, -40}, {100, -40}}, color = {192, 192, 192}), Polygon(points = {{100, -40}, {84, -34}, {84, -46}, {100, -40}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-41, 78}, {-38, 78}, {-35.42, 74.6}, {-31.201, 67.7}, {-26.98, 57.4}, {-22.16, 42.1}, {-16.1, 19.2}, {-3.5, -32.8}, {2, -52.2}, {6.8, -66.2}, {11, -75.1}, {15.2, -80.4}, {19.5, -82}, {23.7, -79.6}, {27.9, -73.5}, {32.1, -63.9}, {36.9, -49.2}, {43, -26.8}, {49, -2}, {49, -2}, {57.4, 32.2}, {62.9, 51.1}, {67.7, 64.4}, {71.9, 72.6}, {76.1, 77.1}, {80, 78}}, color = {0, 0, 0}, thickness = 0.5), Line(points = {{-41, -2}, {-80, -2}}, color = {0, 0, 0}, thickness = 0.5), Text(extent = {{-106, -11}, {-60, -29}}, lineColor = {160, 160, 164}, textString = "offset"), Line(points = {{-41, -2}, {-41, -40}}, color = {192, 192, 192}, pattern = LinePattern.Dash), Text(extent = {{-60, -43}, {-14, -61}}, lineColor = {160, 160, 164}, textString = "startTime"), Text(extent = {{76, -52}, {100, -72}}, lineColor = {160, 160, 164}, textString = "time"), Line(points = {{-8, 78}, {45, 78}}, color = {192, 192, 192}, pattern = LinePattern.Dash), Line(points = {{-41, -2}, {52, -2}}, color = {192, 192, 192}, pattern = LinePattern.Dash), Polygon(points = {{33, 78}, {30, 65}, {37, 65}, {33, 78}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Text(extent = {{37, 57}, {83, 39}}, lineColor = {160, 160, 164}, textString = "V"), Polygon(points = {{33, -2}, {30, 11}, {36, 11}, {33, -2}, {33, -2}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{33, 78}, {33, -2}}, color = {192, 192, 192}, pattern = LinePattern.Solid, thickness = 0.25, arrow = {Arrow.None, Arrow.None}), Text(extent = {{-69, 109}, {-4, 83}}, lineColor = {160, 160, 164}, textString = "v = p.v - n.v"), Line(points = {{-41, 78}, {-41, -2}}, color = {0, 0, 0}, thickness = 0.5)}),
            Documentation(revisions = "<html>
<ul>
<li>Initially implemented by Christian Kral on 2013-05-14</li>
</ul>
</html>", info = "<html>
<p>This voltage source uses the corresponding signal source of the Modelica.Blocks.Sources package. Care for the meaning of the parameters in the Blocks package. Furthermore, an offset parameter is introduced, which is added to the value calculated by the blocks source. The startTime parameter allows to shift the blocks source behavior on the time axis.</p>
</html>"));
        end CosineVoltage;

        model ExpSineVoltage "Exponentially damped sine voltage source"
          parameter SI.Voltage V(start = 1) "Amplitude of sine wave";
          parameter SI.Frequency freqHz(start = 2) "Frequency of sine wave";
          parameter SI.Angle phase = 0 "Phase of sine wave";
          parameter SI.Damping damping(start = 1) "Damping coefficient of sine wave";
          extends Interfaces.VoltageSource(redeclare Modelica.Blocks.Sources.ExpSine signalSource(amplitude = V, freqHz = freqHz, phase = phase, damping = damping));
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-64, -14}, {-59.2, 18.3}, {-56, 36.3}, {-52.7, 50.5}, {-49.5, 60.2}, {-46.3, 65.3}, {-43.1, 65.6}, {-39.9, 61.3}, {-36.7, 53.1}, {-32.6, 38.2}, {-27, 11.8}, {-19, -27.9}, {-14.2, -47.7}, {-10.1, -59.9}, {-6.1, -67.2}, {-2.1, -69.3}, {1.9, -66.5}, {5.9, -59.3}, {10.77, -46.1}, {24.44, -0.3}, {29.3, 12.4}, {34.1, 20.8}, {38.1, 24}, {42.9, 23.2}, {47.8, 17.8}, {54.2, 5.4}, {67.1, -24.5}, {73.5, -35.2}, {79.1, -39.9}, {84.7, -39.9}, {91.2, -34.5}, {96, -27.8}}, color = {192, 192, 192})}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-80, -90}, {-80, 84}}, color = {192, 192, 192}), Polygon(points = {{-80, 100}, {-86, 84}, {-74, 84}, {-80, 100}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-99, -40}, {85, -40}}, color = {192, 192, 192}), Polygon(points = {{100, -40}, {84, -34}, {84, -46}, {100, -40}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-50, 0}, {-46.1, 28.2}, {-43.5, 44}, {-40.9, 56.4}, {-38.2, 64.9}, {-35.6, 69.4}, {-33, 69.6}, {-30.4, 65.9}, {-27.8, 58.7}, {-24.5, 45.7}, {-19.9, 22.5}, {-13.4, -12.2}, {-9.5, -29.5}, {-6.23, -40.1}, {-2.96, -46.5}, {0.302, -48.4}, {3.57, -45.9}, {6.83, -39.6}, {10.8, -28.1}, {21.9, 12}, {25.8, 23.1}, {29.7, 30.5}, {33, 33.3}, {36.9, 32.5}, {40.8, 27.8}, {46, 16.9}, {56.5, -9.2}, {61.7, -18.6}, {66.3, -22.7}, {70.9, -22.6}, {76.1, -18}, {80, -12.1}}, color = {0, 0, 0}, thickness = 0.5), Text(extent = {{-78, 1}, {-55, -19}}, lineColor = {160, 160, 164}, textString = "offset"), Text(extent = {{-72, -36}, {-26, -54}}, lineColor = {160, 160, 164}, textString = "startTime"), Text(extent = {{76, -44}, {100, -64}}, lineColor = {160, 160, 164}, textString = "time"), Line(points = {{-50, 0}, {18, 0}}, color = {192, 192, 192}, pattern = LinePattern.Dash), Line(points = {{-50, 0}, {-80, 0}}, color = {0, 0, 0}, thickness = 0.5), Line(points = {{-50, 77}, {-50, 0}}, color = {192, 192, 192}, pattern = LinePattern.Dash), Line(points = {{18, 0}, {18, 76}}, color = {192, 192, 192}, pattern = LinePattern.Dash), Line(points = {{18, 73}, {-50, 73}}, color = {192, 192, 192}), Text(extent = {{-42, 88}, {9, 74}}, lineColor = {160, 160, 164}, textString = "1/freqHz"), Polygon(points = {{-50, 73}, {-41, 75}, {-41, 71}, {-50, 73}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Polygon(points = {{18, 73}, {10, 75}, {10, 71}, {18, 73}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-50, -61}, {-19, -61}}, color = {192, 192, 192}), Polygon(points = {{-18, -61}, {-26, -59}, {-26, -63}, {-18, -61}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Text(extent = {{-51, -63}, {-27, -75}}, lineColor = {160, 160, 164}, textString = "t"), Text(extent = {{-82, -67}, {108, -96}}, lineColor = {160, 160, 164}, textString = "V*exp(-damping*t)*sin(2*pi*freqHz*t+phase)"), Line(points = {{-50, 0}, {-50, -40}}, color = {192, 192, 192}, pattern = LinePattern.Dash), Line(points = {{-50, -54}, {-50, -72}}, color = {192, 192, 192}, pattern = LinePattern.Dash), Line(points = {{18, -76}, {-1, -48}}, color = {192, 192, 192}, pattern = LinePattern.Dash), Text(extent = {{-77, 105}, {-38, 92}}, lineColor = {160, 160, 164}, textString = "v = p.v - n.v")}),
            Documentation(revisions = "<html>
<ul>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>", info = "<html>
<p>This voltage source uses the corresponding signal source of the Modelica.Blocks.Sources package. Care for the meaning of the parameters in the Blocks package. Furthermore, an offset parameter is introduced, which is added to the value calculated by the blocks source. The startTime parameter allows to shift the blocks source behavior on the time axis.</p>
</html>"));
        end ExpSineVoltage;

        model ExponentialsVoltage "Rising and falling exponential voltage source"
          parameter Real vMax(start = 1) "Upper bound for rising edge";
          parameter SI.Time riseTime(min = 0, start = 0.5) "Rise time";
          parameter SI.Time riseTimeConst(min = Modelica.Constants.small, start = 0.1) "Rise time constant";
          parameter SI.Time fallTimeConst(min = Modelica.Constants.small, start = riseTimeConst) "Fall time constant";
          extends Interfaces.VoltageSource(redeclare Modelica.Blocks.Sources.Exponentials signalSource(outMax = vMax, riseTime = riseTime, riseTimeConst = riseTimeConst, fallTimeConst = fallTimeConst));
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-76, -59}, {-73.2, -44.3}, {-70.3, -31.1}, {-66.8, -16.6}, {-63.3, -4}, {-59.7, 6.92}, {-55.5, 18.18}, {-51.3, 27.7}, {-46.3, 37}, {-40.6, 45.5}, {-34.3, 53.1}, {-27.2, 59.6}, {-18.7, 65.3}, {-8.1, 70.2}, {-6, 71}, {-3.88, 58.5}, {-1.05, 43.7}, {1.78, 30.8}, {4.606, 19.45}, {8.14, 7.3}, {11.68, -3}, {15.9, -13.2}, {20.2, -21.6}, {25.1, -29.5}, {30.8, -36.4}, {37.1, -42.3}, {44.9, -47.5}, {54.8, -51.8}, {64, -54.4}}, color = {192, 192, 192})}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-100, -70}, {100, -70}}, color = {192, 192, 192}), Polygon(points = {{100, -70}, {84, -64}, {84, -76}, {100, -70}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-40, -30}, {-37.2, -15.3}, {-34.3, -2.1}, {-30.8, 12.4}, {-27.3, 25}, {-23.7, 35.92}, {-19.5, 47.18}, {-15.3, 56.7}, {-10.3, 66}, {-4.6, 74.5}, {1.7, 82.1}, {8.8, 88.6}, {17.3, 94.3}, {27.9, 99.2}, {30, 100}, {32.12, 87.5}, {34.95, 72.7}, {37.78, 59.8}, {40.606, 48.45}, {44.14, 36.3}, {47.68, 26}, {51.9, 15.8}, {56.2, 7.4}, {61.1, -0.5}, {66.8, -7.4}, {73.1, -13.3}, {80.9, -18.5}, {90.8, -22.8}, {100, -25.4}}, color = {0, 0, 0}, thickness = 0.5), Polygon(points = {{-80, 90}, {-88, 68}, {-72, 68}, {-80, 90}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-80, 68}, {-80, -80}}, color = {192, 192, 192}), Text(extent = {{-70, 91}, {-29, 71}}, lineColor = {160, 160, 164}, textString = "outPort"), Text(extent = {{-78, -43}, {-46, -56}}, lineColor = {160, 160, 164}, textString = "offset"), Polygon(points = {{-40, -70}, {-42, -60}, {-38, -60}, {-40, -70}, {-40, -70}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-40, -30}, {-40, -60}}, color = {192, 192, 192}, pattern = LinePattern.Solid, thickness = 0.25, arrow = {Arrow.None, Arrow.None}), Polygon(points = {{-40, -30}, {-42, -40}, {-38, -40}, {-40, -30}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-39, -30}, {-80, -30}}, color = {0, 0, 0}, thickness = 0.5), Text(extent = {{-59, -71}, {-13, -89}}, lineColor = {160, 160, 164}, textString = "startTime"), Polygon(points = {{-40, -30}, {-31, -28}, {-31, -32}, {-40, -30}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-39, -30}, {30, -30}}, color = {192, 192, 192}, pattern = LinePattern.Solid, thickness = 0.25, arrow = {Arrow.None, Arrow.None}), Polygon(points = {{30, -30}, {22, -28}, {22, -32}, {30, -30}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Text(extent = {{-26, -12}, {19, -28}}, lineColor = {160, 160, 164}, textString = "riseTime"), Text(extent = {{78, -76}, {102, -96}}, lineColor = {160, 160, 164}, textString = "time"), Line(points = {{-40, -30}, {-37.2, -15.3}, {-34.3, -2.1}, {-30.8, 12.4}, {-27.3, 25}, {-23.7, 35.92}, {-19.5, 47.18}, {-15.3, 56.7}, {-10.3, 66}, {-4.6, 74.5}, {1.7, 82.1}, {8.8, 88.6}, {17.3, 94.3}, {27.9, 99.2}, {30, 100}, {32.12, 87.5}, {34.95, 72.7}, {37.78, 59.8}, {40.606, 48.45}, {44.14, 36.3}, {47.68, 26}, {51.9, 15.8}, {56.2, 7.4}, {61.1, -0.5}, {66.8, -7.4}, {73.1, -13.3}, {80.9, -18.5}, {90.8, -22.8}, {100, -25.4}}, color = {0, 0, 0}, thickness = 0.5), Line(points = {{-40, -30}, {-80, -30}}, color = {0, 0, 0}, thickness = 0.5), Line(points = {{30, 100}, {30, -34}}, color = {192, 192, 192}, pattern = LinePattern.Dash)}),
            Documentation(revisions = "<html>
<ul>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>", info = "<html>
<p>This voltage source uses the corresponding signal source of the Modelica.Blocks.Sources package. Care for the meaning of the parameters in the Blocks package. Furthermore, an offset parameter is introduced, which is added to the value calculated by the blocks source. The startTime parameter allows to shift the blocks source behavior on the time axis.</p>
</html>"));
        end ExponentialsVoltage;

        model PulseVoltage "Pulse voltage source"
          parameter SI.Voltage V(start = 1) "Amplitude of pulse";
          parameter Real width(final min = Modelica.Constants.small, final max = 100, start = 50) "Width of pulse in % of period";
          parameter SI.Time period(final min = Modelica.Constants.small, start = 1) "Time for one period";
          extends Interfaces.VoltageSource(redeclare Modelica.Blocks.Sources.Pulse signalSource(amplitude = V, width = width, period = period));
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-70, -70}, {-40, -70}, {-40, 70}, {0, 70}, {0, -70}, {40, -70}, {40, 70}, {80, 70}}, color = {192, 192, 192})}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{-80, 90}, {-88, 68}, {-72, 68}, {-80, 90}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-80, 68}, {-80, -80}}, color = {192, 192, 192}), Line(points = {{-90, -70}, {82, -70}}, color = {192, 192, 192}), Polygon(points = {{90, -70}, {68, -62}, {68, -78}, {90, -70}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Polygon(points = {{-34, 0}, {-37, -13}, {-31, -13}, {-34, 0}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-34, 0}, {-34, -70}}, color = {192, 192, 192}, pattern = LinePattern.Solid, thickness = 0.25, arrow = {Arrow.None, Arrow.None}), Polygon(points = {{-34, -70}, {-37, -57}, {-31, -57}, {-34, -70}, {-34, -70}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Text(extent = {{-78, -24}, {-35, -36}}, lineColor = {160, 160, 164}, textString = "offset"), Text(extent = {{-40, -69}, {6, -87}}, lineColor = {160, 160, 164}, textString = "startTime"), Text(extent = {{70, -80}, {94, -100}}, lineColor = {160, 160, 164}, textString = "time"), Line(points = {{-20, 0}, {-20, -70}}, color = {192, 192, 192}, pattern = LinePattern.Dash), Line(points = {{-80, 0}, {-20, 0}, {-20, 50}, {21, 50}, {21, 0}, {41, 0}, {41, 50}, {79, 50}}, color = {0, 0, 0}, thickness = 0.5), Line(points = {{-20, 89}, {-20, 50}}, color = {192, 192, 192}, pattern = LinePattern.Dash), Line(points = {{21, 74}, {21, 50}}, color = {160, 160, 164}, pattern = LinePattern.Dash), Line(points = {{41, 88}, {41, 50}}, color = {192, 192, 192}, pattern = LinePattern.Dash), Line(points = {{-20, 83}, {41, 83}}, color = {192, 192, 192}), Line(points = {{-20, 69}, {21, 69}}, color = {192, 192, 192}), Text(extent = {{-11, 97}, {35, 85}}, lineColor = {160, 160, 164}, textString = "period"), Text(extent = {{-18, 81}, {21, 69}}, lineColor = {160, 160, 164}, textString = "width"), Line(points = {{-53, 50}, {-20, 50}}, color = {192, 192, 192}, pattern = LinePattern.Dash), Line(points = {{-34, 50}, {-34, 0}}, color = {192, 192, 192}, pattern = LinePattern.Solid, thickness = 0.25, arrow = {Arrow.None, Arrow.None}), Text(extent = {{-78, 34}, {-37, 20}}, lineColor = {160, 160, 164}, textString = "V"), Polygon(points = {{-34, 50}, {-37, 37}, {-31, 37}, {-34, 50}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Polygon(points = {{-34, 0}, {-37, 13}, {-31, 13}, {-34, 0}, {-34, 0}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{79, 50}, {79, 0}, {90, 0}}, color = {0, 0, 0}, thickness = 0.5), Polygon(points = {{-20, 69}, {-11, 71}, {-11, 67}, {-20, 69}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Polygon(points = {{21, 69}, {13, 71}, {13, 67}, {21, 69}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Polygon(points = {{-20, 83}, {-11, 85}, {-11, 81}, {-20, 83}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Polygon(points = {{41, 83}, {33, 85}, {33, 81}, {41, 83}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Text(extent = {{-78, 103}, {-24, 89}}, lineColor = {160, 160, 164}, textString = "v = p.v - n.v")}),
            Documentation(revisions = "<html>
<ul>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>", info = "<html>
<p>This voltage source uses the corresponding signal source of the Modelica.Blocks.Sources package. Care for the meaning of the parameters in the Blocks package. Furthermore, an offset parameter is introduced, which is added to the value calculated by the blocks source. The startTime parameter allows to shift the blocks source behavior on the time axis.</p>
</html>"));
        end PulseVoltage;

        model SawToothVoltage "Saw tooth voltage source"
          parameter SI.Voltage V(start = 1) "Amplitude of saw tooth";
          parameter SI.Time period(start = 1) "Time for one period";
          extends Interfaces.VoltageSource(redeclare Modelica.Blocks.Sources.SawTooth signalSource(amplitude = V, period = period));
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-85, -70}, {-65, -70}, {-5, 71}, {-5, -70}, {55, 71}, {55, -70}}, color = {192, 192, 192})}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{-80, 90}, {-88, 68}, {-72, 68}, {-80, 90}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-80, 68}, {-80, -80}}, color = {192, 192, 192}), Line(points = {{-90, -70}, {82, -70}}, color = {192, 192, 192}), Polygon(points = {{90, -70}, {68, -62}, {68, -78}, {90, -70}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Polygon(points = {{-34, -20}, {-37, -33}, {-31, -33}, {-34, -20}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-34, -20}, {-34, -70}}, color = {192, 192, 192}, pattern = LinePattern.Solid, thickness = 0.25, arrow = {Arrow.None, Arrow.None}), Polygon(points = {{-34, -70}, {-37, -57}, {-31, -57}, {-34, -70}, {-34, -70}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Text(extent = {{-78, -24}, {-35, -36}}, lineColor = {160, 160, 164}, textString = "offset"), Text(extent = {{-31, -69}, {15, -87}}, lineColor = {160, 160, 164}, textString = "startTime"), Text(extent = {{70, -80}, {94, -100}}, lineColor = {160, 160, 164}, textString = "time"), Line(points = {{-10, -20}, {-10, -70}}, color = {192, 192, 192}, pattern = LinePattern.Dash), Line(points = {{-10, 88}, {-10, -20}}, color = {192, 192, 192}, pattern = LinePattern.Dash), Line(points = {{30, 88}, {30, 59}}, color = {192, 192, 192}, pattern = LinePattern.Dash), Line(points = {{-10, 83}, {30, 83}}, color = {192, 192, 192}), Text(extent = {{-12, 97}, {34, 85}}, lineColor = {160, 160, 164}, textString = "period"), Line(points = {{-44, 60}, {30, 60}}, color = {192, 192, 192}, pattern = LinePattern.Dash), Line(points = {{-34, 47}, {-34, -20}}, color = {192, 192, 192}, pattern = LinePattern.Solid, thickness = 0.25, arrow = {Arrow.None, Arrow.None}), Text(extent = {{-78, 34}, {-37, 20}}, lineColor = {160, 160, 164}, textString = "V"), Polygon(points = {{-34, 60}, {-37, 47}, {-30, 47}, {-34, 60}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Polygon(points = {{-34, -20}, {-37, -7}, {-31, -7}, {-34, -20}, {-34, -20}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Polygon(points = {{-10, 83}, {-1, 85}, {-1, 81}, {-10, 83}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Polygon(points = {{30, 83}, {22, 85}, {22, 81}, {30, 83}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-80, -20}, {-10, -20}, {30, 60}, {30, -20}, {72, 60}, {72, -20}}, color = {0, 0, 0}, thickness = 0.5), Text(extent = {{-77, 100}, {-27, 88}}, lineColor = {160, 160, 164}, textString = "v = p.v - n.v")}),
            Documentation(revisions = "<html>
<ul>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>", info = "<html>
<p>This voltage source uses the corresponding signal source of the Modelica.Blocks.Sources package. Care for the meaning of the parameters in the Blocks package. Furthermore, an offset parameter is introduced, which is added to the value calculated by the blocks source. The startTime parameter allows to shift the blocks source behavior on the time axis.</p>
</html>"));
        end SawToothVoltage;

        model TrapezoidVoltage "Trapezoidal voltage source"
          parameter SI.Voltage V(start = 1) "Amplitude of trapezoid";
          parameter SI.Time rising(final min = 0, start = 0) "Rising duration of trapezoid";
          parameter SI.Time width(final min = 0, start = 0.5) "Width duration of trapezoid";
          parameter SI.Time falling(final min = 0, start = 0) "Falling duration of trapezoid";
          parameter SI.Time period(final min = Modelica.Constants.small, start = 1) "Time for one period";
          parameter Integer nperiod(start = -1) "Number of periods (< 0 means infinite number of periods)";
          extends Interfaces.VoltageSource(redeclare Modelica.Blocks.Sources.Trapezoid signalSource(amplitude = V, rising = rising, width = width, falling = falling, period = period, nperiod = nperiod));
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-81, -70}, {-60, -70}, {-30, 70}, {1, 70}, {30, -70}, {51, -70}, {80, 70}}, color = {192, 192, 192})}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{-80, 90}, {-88, 68}, {-72, 68}, {-80, 90}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-80, 68}, {-80, -80}}, color = {192, 192, 192}), Line(points = {{-90, -70}, {82, -70}}, color = {192, 192, 192}), Polygon(points = {{90, -70}, {68, -62}, {68, -78}, {90, -70}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Polygon(points = {{-45, -30}, {-47, -41}, {-43, -41}, {-45, -30}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-45, -30}, {-45, -70}}, color = {192, 192, 192}, pattern = LinePattern.Solid, thickness = 0.25, arrow = {Arrow.None, Arrow.None}), Polygon(points = {{-45, -70}, {-47, -60}, {-43, -60}, {-45, -70}, {-45, -70}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Text(extent = {{-86, -43}, {-43, -55}}, lineColor = {160, 160, 164}, textString = "offset"), Text(extent = {{-47, -69}, {-1, -87}}, lineColor = {160, 160, 164}, textString = "startTime"), Text(extent = {{70, -80}, {94, -100}}, lineColor = {160, 160, 164}, textString = "time"), Line(points = {{-30, 81}, {-30, -70}}, color = {192, 192, 192}, pattern = LinePattern.Dash), Line(points = {{-10, 59}, {-10, 40}}, color = {192, 192, 192}, pattern = LinePattern.Dash), Line(points = {{20, 59}, {20, 39}}, color = {160, 160, 164}, pattern = LinePattern.Dash), Line(points = {{40, 59}, {40, -30}}, color = {192, 192, 192}, pattern = LinePattern.Dash), Line(points = {{-30, 56}, {40, 56}}, color = {192, 192, 192}), Text(extent = {{-8, 70}, {21, 60}}, lineColor = {160, 160, 164}, textString = "width"), Line(points = {{-42, 40}, {-10, 40}}, color = {192, 192, 192}, pattern = LinePattern.Dash), Line(points = {{-39, 40}, {-39, -30}}, color = {192, 192, 192}, pattern = LinePattern.Solid, thickness = 0.25, arrow = {Arrow.None, Arrow.None}), Text(extent = {{-71, 13}, {-34, -1}}, lineColor = {160, 160, 164}, textString = "V"), Polygon(points = {{-30, 56}, {-23, 58}, {-23, 54}, {-30, 56}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Polygon(points = {{-10, 56}, {-17, 58}, {-17, 54}, {-10, 56}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-80, -30}, {-30, -30}, {-10, 40}, {20, 40}, {40, -30}, {60, -30}, {80, 46}, {100, 46}}, color = {0, 0, 0}, thickness = 0.5), Polygon(points = {{-39, 40}, {-41, 29}, {-37, 29}, {-39, 40}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Polygon(points = {{-39, -30}, {-41, -20}, {-37, -20}, {-39, -30}, {-39, -30}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{60, 81}, {60, -30}}, color = {192, 192, 192}, pattern = LinePattern.Dash), Polygon(points = {{40, 56}, {33, 58}, {33, 54}, {40, 56}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Polygon(points = {{20, 56}, {27, 58}, {27, 54}, {20, 56}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Polygon(points = {{20, 56}, {13, 58}, {13, 54}, {20, 56}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Polygon(points = {{-10, 56}, {-3, 58}, {-3, 54}, {-10, 56}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Text(extent = {{-34, 70}, {-5, 60}}, lineColor = {160, 160, 164}, textString = "rising"), Text(extent = {{16, 70}, {45, 60}}, lineColor = {160, 160, 164}, textString = "falling"), Text(extent = {{-77, 103}, {-23, 91}}, lineColor = {160, 160, 164}, textString = "v = p.v - n.v"), Line(points = {{-30, 76}, {60, 76}}, color = {192, 192, 192}), Polygon(points = {{-30, 76}, {-21, 78}, {-21, 74}, {-30, 76}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Polygon(points = {{60, 76}, {52, 78}, {52, 74}, {60, 76}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Text(extent = {{-2, 86}, {25, 77}}, lineColor = {160, 160, 164}, textString = "period")}),
            Documentation(revisions = "<html>
<ul>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>", info = "<html>
<p>This voltage source uses the corresponding signal source of the Modelica.Blocks.Sources package. Care for the meaning of the parameters in the Blocks package. Furthermore, an offset parameter is introduced, which is added to the value calculated by the blocks source. The startTime parameter allows to shift the blocks source behavior on the time axis.</p>
</html>"));
        end TrapezoidVoltage;

        model TableVoltage "Voltage source by linear interpolation in a table"
          parameter Real table[:, :] = [0, 0; 1, 1; 2, 4] "Table matrix (time = first column, voltage = second column)";
          extends Interfaces.VoltageSource(redeclare Modelica.Blocks.Sources.TimeTable signalSource(table = table));
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-58, -36}, {-58, 84}, {42, 84}, {42, -36}, {-58, -36}, {-58, -6}, {42, -6}, {42, 24}, {-58, 24}, {-58, 54}, {42, 54}, {42, 84}, {-8, 84}, {-8, -37}}, color = {192, 192, 192})}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{-80, 90}, {-88, 68}, {-72, 68}, {-80, 90}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-80, 68}, {-80, -80}}, color = {192, 192, 192}), Line(points = {{-90, -70}, {82, -70}}, color = {192, 192, 192}), Polygon(points = {{90, -70}, {68, -62}, {68, -78}, {90, -70}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-20, 90}, {30, -30}}, lineColor = {255, 255, 255}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-20, -30}, {-20, 90}, {80, 90}, {80, -30}, {-20, -30}, {-20, 0}, {80, 0}, {80, 30}, {-20, 30}, {-20, 60}, {80, 60}, {80, 90}, {30, 90}, {30, -31}}, color = {0, 0, 0}), Text(extent = {{-77, -42}, {-38, -58}}, lineColor = {160, 160, 164}, textString = "offset"), Polygon(points = {{-31, -30}, {-33, -40}, {-29, -40}, {-31, -30}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Polygon(points = {{-31, -70}, {-33, -60}, {-29, -60}, {-31, -70}, {-31, -70}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-31, -30}, {-31, -70}}, color = {192, 192, 192}, pattern = LinePattern.Solid, thickness = 0.25, arrow = {Arrow.None, Arrow.None}), Line(points = {{-20, -20}, {-20, -70}}, color = {192, 192, 192}, pattern = LinePattern.Dash), Text(extent = {{-38, -70}, {8, -88}}, lineColor = {160, 160, 164}, textString = "startTime"), Line(points = {{-20, -30}, {-80, -30}}, color = {192, 192, 192}, pattern = LinePattern.Dash), Text(extent = {{66, -81}, {91, -93}}, lineColor = {160, 160, 164}, textString = "time"), Text(extent = {{-15, 83}, {24, 68}}, lineColor = {0, 0, 0}, textString = "time"), Text(extent = {{33, 83}, {76, 67}}, lineColor = {0, 0, 0}, textString = "v"), Text(extent = {{-81, 98}, {-31, 85}}, lineColor = {160, 160, 164}, textString = "v = p.v - n.v")}),
            Documentation(info = "<html>
<p>This voltage source uses the corresponding signal source of the Modelica.Blocks.Sources package.  Furthermore, an offset parameter is introduced, which is added to the value calculated by the blocks source. The startTime parameter allows to shift the blocks source behavior on the time axis.</p>
<p><br>This block generates a voltage source by <b>linear interpolation</b> in a table. The time points and voltage values are stored in a matrix <b>table[i,j]</b>, where the first column table[:,1] contains the time points and the second column contains the voltage to be interpolated. The table interpolation has the following properties:</p>
<ul>
<li>The time points need to be <b>monotonically increasing</b>. </li>
<li><b>Discontinuities</b> are allowed, by providing the same time point twice in the table. </li>
<li>Values <b>outside</b> of the table range, are computed by <b>extrapolation</b> through the last or first two points of the table.</li>
<li>If the table has only <b>one row</b>, no interpolation is performed and the voltage value is just returned independently of the actual time instant, i.e., this is a constant voltage source.</li>
<li>Via parameters <b>startTime</b> and <b>offset</b> the curve defined by the table can be shifted both in time and in the voltage. </li>
<li>The table is implemented in a numerically sound way by generating <b>time events</b> at interval boundaries.
    This generates continuously differentiable values for the integrator.</li>
</ul>
<p>Example:</p>
<pre>   table = [0  0
            1  0
            1  1
            2  4
            3  9
            4 16]
If, e.g., time = 1.0, the voltage v =  0.0 (before event), 1.0 (after event)
    e.g., time = 1.5, the voltage v =  2.5,
    e.g., time = 2.0, the voltage v =  4.0,
    e.g., time = 5.0, the voltage v = 23.0 (i.e., extrapolation). </pre>
<p><br>  Furthermore, an offset parameter is introduced, which is added to the value calculated by the blocks source. The startTime parameter allows to shift the blocks source behavior on the time axis.</p>
</html>", revisions = "<html>
<ul>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>"));
        end TableVoltage;

        model SignalCurrent "Generic current source using the input signal as source current"
          Interfaces.PositivePin p annotation(
            Placement(transformation(extent = {{-110, -10}, {-90, 10}}, rotation = 0)));
          Interfaces.NegativePin n annotation(
            Placement(transformation(extent = {{110, -10}, {90, 10}}, rotation = 0)));
          SI.Voltage v "Voltage drop between the two pins (= p.v - n.v)";
          Modelica.Blocks.Interfaces.RealInput i(unit = "A") "Current flowing from pin p to pin n as input signal" annotation(
            Placement(transformation(origin = {0, 70}, extent = {{-20, -20}, {20, 20}}, rotation = 270)));
        equation
          v = p.v - n.v;
          0 = p.i + n.i;
          i = p.i;
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-50, 50}, {50, -50}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{-90, 0}, {-50, 0}}, color = {0, 0, 255}), Line(points = {{50, 0}, {90, 0}}, color = {0, 0, 255}), Line(points = {{0, -50}, {0, 50}}, color = {0, 0, 255}), Text(extent = {{-150, -120}, {150, -80}}, textString = "%name", lineColor = {0, 0, 255}), Polygon(points = {{90, 0}, {60, 10}, {60, -10}, {90, 0}}, lineColor = {0, 0, 255}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid)}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-50, 50}, {50, -50}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{-96, 0}, {-50, 0}}, color = {0, 0, 255}), Line(points = {{50, 0}, {96, 0}}, color = {0, 0, 255}), Line(points = {{0, -50}, {0, 50}}, color = {0, 0, 255})}),
            Documentation(revisions = "<html>
<ul>
<li><i> 1998   </i>
       by Martin Otter<br> initially implemented<br>
       </li>
</ul>
</html>", info = "<html>
<p>The signal current source is a parameterless converter of real valued signals into a the source current. No further effects are modeled. The real valued signal has to be provided by components of the blocks library. It can be regarded as the &quot;Opposite&quot; of a current sensor.</p>
</html>"));
        end SignalCurrent;

        model ConstantCurrent "Source for constant current"
          parameter SI.Current I(start = 1) "Value of constant current";
          extends Interfaces.OnePort;
        equation
          i = I;
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-50, 50}, {50, -50}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{-90, 0}, {-50, 0}}, color = {0, 0, 255}), Line(points = {{50, 0}, {90, 0}}, color = {0, 0, 255}), Line(points = {{0, -50}, {0, 50}}, color = {0, 0, 255}), Text(extent = {{-150, 80}, {150, 120}}, textString = "%name=%I", lineColor = {0, 0, 255}), Polygon(points = {{90, 0}, {60, 10}, {60, -10}, {90, 0}}, lineColor = {0, 0, 255}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid)}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-50, 50}, {50, -50}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{-96, 0}, {-50, 0}}, color = {0, 0, 255}), Line(points = {{50, 0}, {96, 0}}, color = {0, 0, 255}), Line(points = {{0, -50}, {0, 50}}, color = {0, 0, 255})}),
            Documentation(revisions = "<html>
<ul>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>", info = "<html>
<p>The ConstantCurrent source is a simple source for an ideal constant current which is provided by a parameter. There is no internal resistance modeled. No further effects are modeled. Especially, the current flow will never end.</p>
</html>"));
        end ConstantCurrent;

        model StepCurrent "Step current source"
          parameter SI.Current I(start = 1) "Height of step";
          extends Interfaces.CurrentSource(redeclare Modelica.Blocks.Sources.Step signalSource(height = I));
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-86, -70}, {-14, -70}, {-14, 70}, {57, 70}}, color = {192, 192, 192})}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{-80, 90}, {-88, 68}, {-72, 68}, {-80, 90}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-80, 68}, {-80, -80}}, color = {192, 192, 192}), Line(points = {{-80, -18}, {0, -18}, {0, 50}, {80, 50}}, color = {0, 0, 0}, thickness = 0.5), Line(points = {{-90, -70}, {82, -70}}, color = {192, 192, 192}), Polygon(points = {{90, -70}, {68, -62}, {68, -78}, {90, -70}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Text(extent = {{70, -80}, {94, -100}}, lineColor = {160, 160, 164}, textString = "time"), Text(extent = {{-21, -72}, {25, -90}}, lineColor = {160, 160, 164}, textString = "startTime"), Line(points = {{0, -17}, {0, -70}}, color = {192, 192, 192}, pattern = LinePattern.Dash), Text(extent = {{-68, -36}, {-22, -54}}, lineColor = {160, 160, 164}, textString = "offset"), Line(points = {{-13, 50}, {-13, -18}}, color = {192, 192, 192}, pattern = LinePattern.Solid, thickness = 0.25, arrow = {Arrow.None, Arrow.None}), Polygon(points = {{0, 50}, {-19, 50}, {0, 50}}, lineColor = {192, 192, 192}, pattern = LinePattern.Dash), Polygon(points = {{-13, -18}, {-16, -5}, {-10, -5}, {-13, -18}, {-13, -18}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Polygon(points = {{-13, 50}, {-16, 37}, {-9, 37}, {-13, 50}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Text(extent = {{-68, 26}, {-22, 8}}, lineColor = {160, 160, 164}, textString = "I"), Polygon(points = {{-13, -70}, {-16, -57}, {-10, -57}, {-13, -70}, {-13, -70}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-13, -18}, {-13, -70}}, color = {192, 192, 192}, pattern = LinePattern.Solid, thickness = 0.25, arrow = {Arrow.None, Arrow.None}), Polygon(points = {{-13, -18}, {-16, -31}, {-10, -31}, {-13, -18}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Text(extent = {{-73, 75}, {-53, 95}}, lineColor = {192, 192, 192}, textString = "i")}),
            Documentation(revisions = "<html>
<ul>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>", info = "<html>
<p>This current source uses the corresponding signal source of the Modelica.Blocks.Sources package. Care for the meaning of the parameters in the Blocks package. Furthermore, an offset parameter is introduced, which is added to the value calculated by the blocks source. The startTime parameter allows to shift the blocks source behavior on the time axis.</p>
</html>"));
        end StepCurrent;

        model RampCurrent "Ramp current source"
          parameter SI.Current I(start = 1) "Height of ramp";
          parameter SI.Time duration(min = Modelica.Constants.small, start = 2) "Duration of ramp";
          extends Interfaces.CurrentSource(redeclare Modelica.Blocks.Sources.Ramp signalSource(final height = I, final duration = duration));
          annotation(
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{-80, 90}, {-88, 68}, {-72, 68}, {-80, 90}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-80, 68}, {-80, -80}}, color = {192, 192, 192}), Line(points = {{-80, -20}, {-20, -20}, {50, 50}}, color = {0, 0, 0}, thickness = 0.5), Line(points = {{-90, -70}, {82, -70}}, color = {192, 192, 192}), Polygon(points = {{90, -70}, {68, -62}, {68, -78}, {90, -70}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Polygon(points = {{-40, -20}, {-42, -30}, {-38, -30}, {-40, -20}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-40, -20}, {-40, -70}}, color = {192, 192, 192}, pattern = LinePattern.Solid, thickness = 0.25, arrow = {Arrow.None, Arrow.None}), Polygon(points = {{-40, -70}, {-42, -60}, {-38, -60}, {-40, -70}, {-40, -70}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Text(extent = {{-80, -33}, {-41, -49}}, lineColor = {160, 160, 164}, textString = "offset"), Text(extent = {{-40, -70}, {6, -88}}, lineColor = {160, 160, 164}, textString = "startTime"), Text(extent = {{70, -80}, {94, -100}}, lineColor = {160, 160, 164}, textString = "time"), Line(points = {{-20, -20}, {-20, -70}}, color = {192, 192, 192}, pattern = LinePattern.Dash), Line(points = {{-19, -20}, {50, -20}}, color = {192, 192, 192}, pattern = LinePattern.Solid, thickness = 0.25, arrow = {Arrow.None, Arrow.None}), Line(points = {{50, 50}, {100, 50}}, color = {0, 0, 0}, thickness = 0.5), Line(points = {{50, 50}, {50, -20}}, color = {192, 192, 192}, pattern = LinePattern.Solid, thickness = 0.25, arrow = {Arrow.None, Arrow.None}), Polygon(points = {{50, -20}, {42, -18}, {42, -22}, {50, -20}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Polygon(points = {{-20, -20}, {-11, -18}, {-11, -22}, {-20, -20}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Polygon(points = {{50, 50}, {48, 40}, {53, 40}, {50, 50}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Polygon(points = {{50, -20}, {47, -10}, {52, -10}, {50, -20}, {50, -20}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Text(extent = {{53, 25}, {82, 7}}, lineColor = {160, 160, 164}, textString = "I"), Text(extent = {{0, -17}, {35, -37}}, lineColor = {160, 160, 164}, textString = "duration"), Text(extent = {{-73, 75}, {-53, 95}}, lineColor = {192, 192, 192}, textString = "i")}),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-80, -60}, {-50, -60}, {50, 60}, {80, 60}}, color = {192, 192, 192})}),
            Documentation(revisions = "<html>
<ul>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>", info = "<html>
<p>This current source uses the corresponding signal source of the Modelica.Blocks.Sources package. Care for the meaning of the parameters in the Blocks package. Furthermore, an offset parameter is introduced, which is added to the value calculated by the blocks source. The startTime parameter allows to shift the blocks source behavior on the time axis.</p>
</html>"));
        end RampCurrent;

        model SineCurrent "Sine current source"
          parameter SI.Current I(start = 1) "Amplitude of sine wave";
          parameter SI.Angle phase = 0 "Phase of sine wave";
          parameter SI.Frequency freqHz(start = 1) "Frequency of sine wave";
          extends Interfaces.CurrentSource(redeclare Modelica.Blocks.Sources.Sine signalSource(amplitude = I, freqHz = freqHz, phase = phase));
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-70, 0}, {-60.2, 29.9}, {-53.8, 46.5}, {-48.2, 58.1}, {-43.3, 65.2}, {-38.3, 69.2}, {-33.4, 69.8}, {-28.5, 67}, {-23.6, 61}, {-18.6, 52}, {-13, 38.6}, {-5.98, 18.6}, {8.79, -26.9}, {15.1, -44}, {20.8, -56.2}, {25.7, -64}, {30.6, -68.6}, {35.5, -70}, {40.5, -67.9}, {45.4, -62.5}, {50.3, -54.1}, {55.9, -41.3}, {63, -21.7}, {70, 0}}, color = {192, 192, 192})}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-80, -90}, {-80, 84}}, color = {192, 192, 192}), Polygon(points = {{-80, 100}, {-86, 84}, {-74, 84}, {-80, 100}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-99, -40}, {85, -40}}, color = {192, 192, 192}), Polygon(points = {{101, -40}, {85, -34}, {85, -46}, {101, -40}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-41, -2}, {-32.6, 32.2}, {-27.1, 51.1}, {-22.3, 64.4}, {-18.1, 72.6}, {-13.9, 77.1}, {-10, 78}, {-5.42, 74.6}, {-1.201, 67.7}, {3.02, 57.4}, {7.84, 42.1}, {13.9, 19.2}, {26.5, -32.8}, {32, -52.2}, {36.8, -66.2}, {41, -75.1}, {45.2, -80.4}, {49.5, -82}, {53.7, -79.6}, {57.9, -73.5}, {62.1, -63.9}, {66.9, -49.2}, {73, -26.8}, {79, -2}}, color = {0, 0, 0}, thickness = 0.5), Line(points = {{-41, -2}, {-80, -2}}, color = {0, 0, 0}, thickness = 0.5), Text(extent = {{-106, -11}, {-60, -29}}, lineColor = {160, 160, 164}, textString = "offset"), Line(points = {{-41, -2}, {-41, -40}}, color = {192, 192, 192}, pattern = LinePattern.Dash), Text(extent = {{-60, -43}, {-14, -61}}, lineColor = {160, 160, 164}, textString = "startTime"), Text(extent = {{84, -52}, {108, -72}}, lineColor = {160, 160, 164}, textString = "time"), Line(points = {{-10, 78}, {42, 78}}, color = {192, 192, 192}, pattern = LinePattern.Dash), Line(points = {{-41, -2}, {5, -2}, {5, -2}, {51, -2}}, color = {192, 192, 192}, pattern = LinePattern.Dash), Polygon(points = {{33, 78}, {30, 65}, {37, 65}, {33, 78}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Text(extent = {{37, 57}, {83, 39}}, lineColor = {160, 160, 164}, textString = "I"), Polygon(points = {{33, -2}, {30, 11}, {36, 11}, {33, -2}, {33, -2}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{33, 78}, {33, -2}}, color = {192, 192, 192}, pattern = LinePattern.Solid, thickness = 0.25, arrow = {Arrow.None, Arrow.None}), Text(extent = {{-73, 82}, {-53, 102}}, lineColor = {192, 192, 192}, textString = "i")}),
            Documentation(revisions = "<html>
<ul>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>", info = "<html>
<p>This current source uses the corresponding signal source of the Modelica.Blocks.Sources package. Care for the meaning of the parameters in the Blocks package. Furthermore, an offset parameter is introduced, which is added to the value calculated by the blocks source. The startTime parameter allows to shift the blocks source behavior on the time axis.</p>
</html>"));
        end SineCurrent;

        model CosineCurrent "Cosine current source"
          parameter SI.Current I(start = 1) "Amplitude of cosine wave";
          parameter SI.Angle phase = 0 "Phase of cosine wave";
          parameter SI.Frequency freqHz(start = 1) "Frequency of cosine wave";
          extends Interfaces.CurrentSource(redeclare Modelica.Blocks.Sources.Cosine signalSource(amplitude = I, freqHz = freqHz, phase = phase));
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-71, 70}, {-68.4, 69.8}, {-63.5, 67}, {-58.6, 61}, {-53.6, 52}, {-48, 38.6}, {-40.98, 18.6}, {-26.21, -26.9}, {-19.9, -44}, {-14.2, -56.2}, {-9.3, -64}, {-4.4, -68.6}, {0.5, -70}, {5.5, -67.9}, {10.4, -62.5}, {15.3, -54.1}, {20.9, -41.3}, {28, -21.7}, {35, 0}}, color = {192, 192, 192}, smooth = Smooth.Bezier), Line(points = {{35, 0}, {44.8, 29.9}, {51.2, 46.5}, {56.8, 58.1}, {61.7, 65.2}, {66.7, 69.2}, {71.6, 69.8}}, color = {192, 192, 192})}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-80, -90}, {-80, 84}}, color = {192, 192, 192}), Polygon(points = {{-80, 100}, {-86, 84}, {-74, 84}, {-80, 100}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-99, -40}, {85, -40}}, color = {192, 192, 192}), Polygon(points = {{101, -40}, {85, -34}, {85, -46}, {101, -40}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-41, -2}, {-80, -2}}, color = {0, 0, 0}, thickness = 0.5), Text(extent = {{-106, -11}, {-60, -29}}, lineColor = {160, 160, 164}, textString = "offset"), Line(points = {{-41, -2}, {-41, -40}}, color = {192, 192, 192}, pattern = LinePattern.Dash), Text(extent = {{-60, -43}, {-14, -61}}, lineColor = {160, 160, 164}, textString = "startTime"), Text(extent = {{84, -52}, {108, -72}}, lineColor = {160, 160, 164}, textString = "time"), Line(points = {{-10, 78}, {42, 78}}, color = {192, 192, 192}, pattern = LinePattern.Dash), Line(points = {{-41, -2}, {5, -2}, {5, -2}, {51, -2}}, color = {192, 192, 192}, pattern = LinePattern.Dash), Polygon(points = {{33, 78}, {30, 65}, {37, 65}, {33, 78}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Text(extent = {{37, 57}, {83, 39}}, lineColor = {160, 160, 164}, textString = "I"), Polygon(points = {{33, -2}, {30, 11}, {36, 11}, {33, -2}, {33, -2}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{33, 78}, {33, -2}}, color = {192, 192, 192}, pattern = LinePattern.Solid, thickness = 0.25, arrow = {Arrow.None, Arrow.None}), Text(extent = {{-73, 82}, {-53, 102}}, lineColor = {192, 192, 192}, textString = "i"), Line(points = {{-41, 78}, {-41, -2}}, color = {0, 0, 0}, thickness = 0.5), Line(points = {{-41, 78}, {-38, 78}, {-35.42, 74.6}, {-31.201, 67.7}, {-26.98, 57.4}, {-22.16, 42.1}, {-16.1, 19.2}, {-3.5, -32.8}, {2, -52.2}, {6.8, -66.2}, {11, -75.1}, {15.2, -80.4}, {19.5, -82}, {23.7, -79.6}, {27.9, -73.5}, {32.1, -63.9}, {36.9, -49.2}, {43, -26.8}, {49, -2}}, color = {0, 0, 0}, thickness = 0.5), Line(points = {{49, -2}, {57.4, 32.2}, {62.9, 51.1}, {67.7, 64.4}, {71.9, 72.6}, {76.1, 77.1}, {80, 78}}, color = {0, 0, 0}, thickness = 0.5)}),
            Documentation(revisions = "<html>
<ul>
<li>Initially implemented by Christian Kral on 2013-05-14</li>
</ul>
</html>", info = "<html>
<p>This current source uses the corresponding signal source of the Modelica.Blocks.Sources package. Care for the meaning of the parameters in the Blocks package. Furthermore, an offset parameter is introduced, which is added to the value calculated by the blocks source. The startTime parameter allows to shift the blocks source behavior on the time axis.</p>
</html>"));
        end CosineCurrent;

        model ExpSineCurrent "Exponentially damped sine current source"
          parameter Real I(start = 1) "Amplitude of sine wave";
          parameter SI.Frequency freqHz(start = 2) "Frequency of sine wave";
          parameter SI.Angle phase = 0 "Phase of sine wave";
          parameter SI.Damping damping(start = 1) "Damping coefficient of sine wave";
          extends Interfaces.CurrentSource(redeclare Modelica.Blocks.Sources.ExpSine signalSource(amplitude = I, freqHz = freqHz, phase = phase, damping = damping));
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-80, -14}, {-75.2, 18.3}, {-72, 36.3}, {-68.7, 50.5}, {-65.5, 60.2}, {-62.3, 65.3}, {-59.1, 65.6}, {-55.9, 61.3}, {-52.7, 53.1}, {-48.6, 38.2}, {-43, 11.8}, {-35, -27.9}, {-30.2, -47.7}, {-26.1, -59.9}, {-22.1, -67.2}, {-18.1, -69.3}, {-14.1, -66.5}, {-10.1, -59.3}, {-5.23, -46.1}, {8.44, -0.3}, {13.3, 12.4}, {18.1, 20.8}, {22.1, 24}, {26.9, 23.2}, {31.8, 17.8}, {38.2, 5.4}, {51.1, -24.5}, {57.5, -35.2}, {63.1, -39.9}, {68.7, -39.9}, {75.2, -34.5}, {80, -27.8}}, color = {192, 192, 192})}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-80, -90}, {-80, 84}}, color = {192, 192, 192}), Polygon(points = {{-80, 100}, {-86, 84}, {-74, 84}, {-80, 100}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-99, -40}, {100, -40}}, color = {192, 192, 192}), Polygon(points = {{100, -40}, {84, -34}, {84, -46}, {100, -40}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-50, 0}, {-46.1, 28.2}, {-43.5, 44}, {-40.9, 56.4}, {-38.2, 64.9}, {-35.6, 69.4}, {-33, 69.6}, {-30.4, 65.9}, {-27.8, 58.7}, {-24.5, 45.7}, {-19.9, 22.5}, {-13.4, -12.2}, {-9.5, -29.5}, {-6.23, -40.1}, {-2.96, -46.5}, {0.302, -48.4}, {3.57, -45.9}, {6.83, -39.6}, {10.8, -28.1}, {21.9, 12}, {25.8, 23.1}, {29.7, 30.5}, {33, 33.3}, {36.9, 32.5}, {40.8, 27.8}, {46, 16.9}, {56.5, -9.2}, {61.7, -18.6}, {66.3, -22.7}, {70.9, -22.6}, {76.1, -18}, {80, -12.1}}, color = {0, 0, 0}, thickness = 0.5), Text(extent = {{-78, 1}, {-55, -19}}, lineColor = {160, 160, 164}, textString = "offset"), Text(extent = {{-72, -36}, {-26, -54}}, lineColor = {160, 160, 164}, textString = "startTime"), Text(extent = {{76, -44}, {100, -64}}, lineColor = {160, 160, 164}, textString = "time"), Line(points = {{-50, 0}, {18, 0}}, color = {192, 192, 192}, pattern = LinePattern.Dash), Line(points = {{-50, 0}, {-80, 0}}, color = {0, 0, 0}, thickness = 0.5), Line(points = {{-50, 77}, {-50, 0}}, color = {192, 192, 192}, pattern = LinePattern.Dash), Line(points = {{18, -1}, {18, 76}}, color = {192, 192, 192}, pattern = LinePattern.Dash), Line(points = {{18, 73}, {-50, 73}}, color = {192, 192, 192}), Text(extent = {{-42, 88}, {9, 74}}, lineColor = {160, 160, 164}, textString = "1/freqHz"), Polygon(points = {{-50, 73}, {-41, 75}, {-41, 71}, {-50, 73}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Polygon(points = {{18, 73}, {10, 75}, {10, 71}, {18, 73}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-50, -61}, {-19, -61}}, color = {192, 192, 192}), Polygon(points = {{-18, -61}, {-26, -59}, {-26, -63}, {-18, -61}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Text(extent = {{-51, -63}, {-27, -75}}, lineColor = {160, 160, 164}, textString = "t"), Text(extent = {{-82, -67}, {108, -96}}, lineColor = {160, 160, 164}, textString = "I*exp(-damping*t)*sin(2*pi*freqHz*t+phase)"), Line(points = {{-50, 0}, {-50, -40}}, color = {192, 192, 192}, pattern = LinePattern.Dash), Line(points = {{-50, -54}, {-50, -72}}, color = {192, 192, 192}, pattern = LinePattern.Dash), Line(points = {{18, -76}, {-1, -48}}, color = {192, 192, 192}, pattern = LinePattern.Dash), Text(extent = {{-74, 83}, {-54, 103}}, lineColor = {192, 192, 192}, textString = "i")}),
            Documentation(revisions = "<html>
<ul>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>", info = "<html>
<p>This current source uses the corresponding signal source of the Modelica.Blocks.Sources package. Care for the meaning of the parameters in the Blocks package. Furthermore, an offset parameter is introduced, which is added to the value calculated by the blocks source. The startTime parameter allows to shift the blocks source behavior on the time axis.</p>
</html>"));
        end ExpSineCurrent;

        model ExponentialsCurrent "Rising and falling exponential current source"
          parameter Real iMax(start = 1) "Upper bound for rising edge";
          parameter SI.Time riseTime(min = 0, start = 0.5) "Rise time";
          parameter SI.Time riseTimeConst(min = Modelica.Constants.small, start = 0.1) "Rise time constant";
          parameter SI.Time fallTimeConst(min = Modelica.Constants.small, start = riseTimeConst) "Fall time constant";
          extends Interfaces.CurrentSource(redeclare Modelica.Blocks.Sources.Exponentials signalSource(outMax = iMax, riseTime = riseTime, riseTimeConst = riseTimeConst, fallTimeConst = fallTimeConst));
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-76, -59}, {-73.2, -44.3}, {-70.3, -31.1}, {-66.8, -16.6}, {-63.3, -4}, {-59.7, 6.92}, {-55.5, 18.18}, {-51.3, 27.7}, {-46.3, 37}, {-40.6, 45.5}, {-34.3, 53.1}, {-27.2, 59.6}, {-18.7, 65.3}, {-8.1, 70.2}, {-6, 71}, {-3.88, 58.5}, {-1.05, 43.7}, {1.78, 30.8}, {4.606, 19.45}, {8.14, 7.3}, {11.68, -3}, {15.9, -13.2}, {20.2, -21.6}, {25.1, -29.5}, {30.8, -36.4}, {37.1, -42.3}, {44.9, -47.5}, {54.8, -51.8}, {64, -54.4}}, color = {192, 192, 192})}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-100, -70}, {100, -70}}, color = {192, 192, 192}), Polygon(points = {{100, -70}, {84, -64}, {84, -76}, {100, -70}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-40, -30}, {-37.2, -15.3}, {-34.3, -2.1}, {-30.8, 12.4}, {-27.3, 25}, {-23.7, 35.92}, {-19.5, 47.18}, {-15.3, 56.7}, {-10.3, 66}, {-4.6, 74.5}, {1.7, 82.1}, {8.8, 88.6}, {17.3, 94.3}, {27.9, 99.2}, {30, 100}, {32.12, 87.5}, {34.95, 72.7}, {37.78, 59.8}, {40.606, 48.45}, {44.14, 36.3}, {47.68, 26}, {51.9, 15.8}, {56.2, 7.4}, {61.1, -0.5}, {66.8, -7.4}, {73.1, -13.3}, {80.9, -18.5}, {90.8, -22.8}, {100, -25.4}}, color = {0, 0, 0}, thickness = 0.5), Polygon(points = {{-80, 90}, {-88, 68}, {-72, 68}, {-80, 90}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-80, 90}, {-80, -80}}, color = {192, 192, 192}), Polygon(points = {{-40, -70}, {-42, -60}, {-38, -60}, {-40, -70}, {-40, -70}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-40, -30}, {-40, -60}}, color = {192, 192, 192}, pattern = LinePattern.Solid, thickness = 0.25, arrow = {Arrow.None, Arrow.None}), Polygon(points = {{-40, -30}, {-42, -40}, {-38, -40}, {-40, -30}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-39, -30}, {-80, -30}}, color = {0, 0, 0}, thickness = 0.5), Text(extent = {{-59, -71}, {-13, -89}}, lineColor = {160, 160, 164}, textString = "startTime"), Polygon(points = {{-40, -30}, {-31, -28}, {-31, -32}, {-40, -30}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-40, -30}, {30, -30}}, color = {192, 192, 192}, pattern = LinePattern.Solid, thickness = 0.25, arrow = {Arrow.None, Arrow.None}), Polygon(points = {{30, -30}, {22, -28}, {22, -32}, {30, -30}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Text(extent = {{-26, -12}, {19, -28}}, lineColor = {160, 160, 164}, textString = "riseTime"), Line(points = {{-40, -30}, {-80, -30}}, color = {0, 0, 0}, thickness = 0.5), Line(points = {{30, 100}, {30, -34}}, color = {192, 192, 192}, pattern = LinePattern.Dash), Text(extent = {{-73, 75}, {-53, 95}}, lineColor = {192, 192, 192}, textString = "i")}),
            Documentation(revisions = "<html>
<ul>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>", info = "<html>
<p>This current source uses the corresponding signal source of the Modelica.Blocks.Sources package. Care for the meaning of the parameters in the Blocks package. Furthermore, an offset parameter is introduced, which is added to the value calculated by the blocks source. The startTime parameter allows to shift the blocks source behavior on the time axis.</p>
</html>"));
        end ExponentialsCurrent;

        model PulseCurrent "Pulse current source"
          parameter SI.Current I(start = 1) "Amplitude of pulse";
          parameter Real width(final min = Modelica.Constants.small, final max = 100, start = 50) "Width of pulse in % of period";
          parameter SI.Time period(final min = Modelica.Constants.small, start = 1) "Time for one period";
          extends Interfaces.CurrentSource(redeclare Modelica.Blocks.Sources.Pulse signalSource(amplitude = I, width = width, period = period));
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-80, -67}, {-50, -67}, {-50, 73}, {-10, 73}, {-10, -67}, {30, -67}, {30, 73}, {70, 73}}, color = {192, 192, 192})}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{-80, 90}, {-88, 68}, {-72, 68}, {-80, 90}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-80, 68}, {-80, -80}}, color = {192, 192, 192}), Line(points = {{-90, -70}, {82, -70}}, color = {192, 192, 192}), Polygon(points = {{90, -70}, {68, -62}, {68, -78}, {90, -70}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Polygon(points = {{-34, 0}, {-37, -13}, {-31, -13}, {-34, 0}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-34, 0}, {-34, -70}}, color = {192, 192, 192}, pattern = LinePattern.Solid, thickness = 0.25, arrow = {Arrow.None, Arrow.None}), Polygon(points = {{-34, -70}, {-37, -57}, {-31, -57}, {-34, -70}, {-34, -70}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Text(extent = {{-78, -24}, {-35, -36}}, lineColor = {160, 160, 164}, textString = "offset"), Text(extent = {{-42, -69}, {4, -87}}, lineColor = {160, 160, 164}, textString = "startTime"), Text(extent = {{70, -80}, {94, -100}}, lineColor = {160, 160, 164}, textString = "time"), Line(points = {{-19, 0}, {-19, -70}}, color = {192, 192, 192}, pattern = LinePattern.Dash), Line(points = {{-80, 0}, {-19, 0}, {-19, 50}, {21, 50}, {21, 0}, {41, 0}, {41, 51}, {80, 51}}, color = {0, 0, 0}, thickness = 0.5), Line(points = {{-19, 88}, {-19, 49}}, color = {192, 192, 192}, pattern = LinePattern.Dash), Line(points = {{21, 74}, {21, 50}}, color = {160, 160, 164}, pattern = LinePattern.Dash), Line(points = {{41, 88}, {41, 51}}, color = {192, 192, 192}, pattern = LinePattern.Dash), Line(points = {{-19, 83}, {41, 83}}, color = {192, 192, 192}), Line(points = {{-19, 69}, {21, 69}}, color = {192, 192, 192}), Text(extent = {{-9, 97}, {37, 85}}, lineColor = {160, 160, 164}, textString = "period"), Text(extent = {{-18, 81}, {21, 69}}, lineColor = {160, 160, 164}, textString = "width"), Line(points = {{-52, 50}, {-19, 50}}, color = {192, 192, 192}, pattern = LinePattern.Dash), Line(points = {{-34, 50}, {-34, 1}}, color = {192, 192, 192}, pattern = LinePattern.Solid, thickness = 0.25, arrow = {Arrow.None, Arrow.None}), Text(extent = {{-78, 34}, {-37, 20}}, lineColor = {160, 160, 164}, textString = "I"), Polygon(points = {{-34, 50}, {-37, 37}, {-31, 37}, {-34, 50}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Polygon(points = {{-34, 0}, {-37, 13}, {-31, 13}, {-34, 0}, {-34, 0}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{80, 51}, {80, 0}, {90, 0}}, color = {0, 0, 0}, thickness = 0.5), Polygon(points = {{-19, 69}, {-10, 71}, {-10, 67}, {-19, 69}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Polygon(points = {{21, 69}, {13, 71}, {13, 67}, {21, 69}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Polygon(points = {{-19, 83}, {-10, 85}, {-10, 81}, {-19, 83}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Polygon(points = {{41, 83}, {33, 85}, {33, 81}, {41, 83}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Text(extent = {{-73, 75}, {-53, 95}}, lineColor = {192, 192, 192}, textString = "i")}),
            Documentation(revisions = "<html>
<ul>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>", info = "<html>
<p>This current source uses the corresponding signal source of the Modelica.Blocks.Sources package. Care for the meaning of the parameters in the Blocks package. Furthermore, an offset parameter is introduced, which is added to the value calculated by the blocks source. The startTime parameter allows to shift the blocks source behavior on the time axis.</p>
</html>"));
        end PulseCurrent;

        model SawToothCurrent "Saw tooth current source"
          parameter SI.Current I(start = 1) "Amplitude of saw tooth";
          parameter SI.Time period(start = 1) "Time for one period";
          extends Interfaces.CurrentSource(redeclare Modelica.Blocks.Sources.SawTooth signalSource(amplitude = I, period = period));
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-70, -71}, {-50, -71}, {10, 70}, {10, -71}, {70, 70}, {70, -71}}, color = {192, 192, 192})}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{-80, 90}, {-88, 68}, {-72, 68}, {-80, 90}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-80, 68}, {-80, -80}}, color = {192, 192, 192}), Line(points = {{-90, -70}, {82, -70}}, color = {192, 192, 192}), Polygon(points = {{90, -70}, {68, -62}, {68, -78}, {90, -70}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Polygon(points = {{-34, -20}, {-37, -33}, {-31, -33}, {-34, -20}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-34, -20}, {-34, -70}}, color = {192, 192, 192}, pattern = LinePattern.Solid, thickness = 0.25, arrow = {Arrow.None, Arrow.None}), Polygon(points = {{-34, -70}, {-37, -57}, {-31, -57}, {-34, -70}, {-34, -70}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Text(extent = {{-78, -24}, {-35, -36}}, lineColor = {160, 160, 164}, textString = "offset"), Text(extent = {{-31, -69}, {15, -87}}, lineColor = {160, 160, 164}, textString = "startTime"), Text(extent = {{70, -80}, {94, -100}}, lineColor = {160, 160, 164}, textString = "time"), Line(points = {{-10, -20}, {-10, -70}}, color = {192, 192, 192}, pattern = LinePattern.Dash), Line(points = {{-10, 88}, {-10, -20}}, color = {192, 192, 192}, pattern = LinePattern.Dash), Line(points = {{30, 88}, {30, 59}}, color = {192, 192, 192}, pattern = LinePattern.Dash), Line(points = {{-10, 83}, {30, 83}}, color = {192, 192, 192}), Text(extent = {{-12, 97}, {34, 85}}, lineColor = {160, 160, 164}, textString = "period"), Line(points = {{-44, 60}, {30, 60}}, color = {192, 192, 192}, pattern = LinePattern.Dash), Line(points = {{-34, 60}, {-34, -20}}, color = {192, 192, 192}, pattern = LinePattern.Solid, thickness = 0.25, arrow = {Arrow.None, Arrow.None}), Text(extent = {{-78, 34}, {-37, 20}}, lineColor = {160, 160, 164}, textString = "I"), Polygon(points = {{-34, 60}, {-37, 47}, {-31, 47}, {-34, 60}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Polygon(points = {{-34, -20}, {-37, -7}, {-31, -7}, {-34, -20}, {-34, -20}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Polygon(points = {{-10, 83}, {-1, 85}, {-1, 81}, {-10, 83}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Polygon(points = {{30, 83}, {22, 85}, {22, 81}, {30, 83}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-80, -20}, {-10, -20}, {30, 60}, {30, -20}, {72, 60}, {72, -20}}, color = {0, 0, 0}, thickness = 0.5), Text(extent = {{-73, 75}, {-53, 95}}, lineColor = {192, 192, 192}, textString = "i")}),
            Documentation(revisions = "<html>
<ul>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>", info = "<html>
<p>This current source uses the corresponding signal source of the Modelica.Blocks.Sources package. Care for the meaning of the parameters in the Blocks package. Furthermore, an offset parameter is introduced, which is added to the value calculated by the blocks source. The startTime parameter allows to shift the blocks source behavior on the time axis.</p>
</html>"));
        end SawToothCurrent;

        model TrapezoidCurrent "Trapezoidal current source"
          parameter SI.Current I(start = 1) "Amplitude of trapezoid";
          parameter SI.Time rising(final min = 0, start = 0) "Rising duration of trapezoid";
          parameter SI.Time width(final min = 0, start = 0.5) "Width duration of trapezoid";
          parameter SI.Time falling(final min = 0, start = 0) "Falling duration of trapezoid";
          parameter SI.Time period(final min = Modelica.Constants.small, start = 1) "Time for one period";
          parameter Integer nperiod(start = -1) "Number of periods (< 0 means infinite number of periods)";
          extends Interfaces.CurrentSource(redeclare Modelica.Blocks.Sources.Trapezoid signalSource(amplitude = I, rising = rising, width = width, falling = falling, period = period, nperiod = nperiod));
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-81, -66}, {-60, -66}, {-30, 74}, {1, 74}, {30, -66}, {51, -66}, {80, 74}}, color = {192, 192, 192})}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{-80, 90}, {-88, 68}, {-72, 68}, {-80, 90}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-80, 90}, {-80, -80}}, color = {192, 192, 192}), Line(points = {{-90, -70}, {90, -70}}, color = {192, 192, 192}), Polygon(points = {{90, -70}, {68, -62}, {68, -78}, {90, -70}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Polygon(points = {{-45, -30}, {-47, -41}, {-43, -41}, {-45, -30}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-45, -30}, {-45, -70}}, color = {192, 192, 192}, pattern = LinePattern.Solid, thickness = 0.25, arrow = {Arrow.None, Arrow.None}), Polygon(points = {{-45, -70}, {-47, -60}, {-43, -60}, {-45, -70}, {-45, -70}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Text(extent = {{-86, -43}, {-43, -55}}, lineColor = {160, 160, 164}, textString = "offset"), Text(extent = {{-47, -69}, {-1, -87}}, lineColor = {160, 160, 164}, textString = "startTime"), Text(extent = {{70, -80}, {94, -100}}, lineColor = {160, 160, 164}, textString = "time"), Line(points = {{-30, 79}, {-30, -70}}, color = {192, 192, 192}, pattern = LinePattern.Dash), Line(points = {{-10, 59}, {-10, 40}}, color = {192, 192, 192}, pattern = LinePattern.Dash), Line(points = {{20, 59}, {20, 40}}, color = {160, 160, 164}, pattern = LinePattern.Dash), Line(points = {{40, 59}, {40, -30}}, color = {192, 192, 192}, pattern = LinePattern.Dash), Line(points = {{-21, 75}, {60, 75}}, color = {192, 192, 192}), Line(points = {{-30, 56}, {40, 56}}, color = {192, 192, 192}), Text(extent = {{-9, 88}, {37, 76}}, lineColor = {160, 160, 164}, textString = "period"), Text(extent = {{-8, 70}, {21, 60}}, lineColor = {160, 160, 164}, textString = "width"), Line(points = {{-42, 40}, {-10, 40}}, color = {192, 192, 192}, pattern = LinePattern.Dash), Line(points = {{-39, 40}, {-39, -30}}, color = {192, 192, 192}, pattern = LinePattern.Solid, thickness = 0.25, arrow = {Arrow.None, Arrow.None}), Text(extent = {{-77, 14}, {-40, 0}}, lineColor = {160, 160, 164}, textString = "I"), Polygon(points = {{-30, 56}, {-23, 58}, {-23, 54}, {-30, 56}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Polygon(points = {{-10, 56}, {-17, 58}, {-17, 54}, {-10, 56}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Polygon(points = {{-30, 75}, {-21, 77}, {-21, 73}, {-30, 75}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Polygon(points = {{60, 75}, {52, 77}, {52, 73}, {60, 75}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-80, -30}, {-30, -30}, {-10, 40}, {20, 40}, {40, -30}, {60, -30}, {80, 40}, {100, 40}}, color = {0, 0, 0}, thickness = 0.5), Polygon(points = {{-39, 40}, {-41, 30}, {-37, 30}, {-39, 40}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Polygon(points = {{-39, -30}, {-41, -20}, {-37, -20}, {-39, -30}, {-39, -30}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{60, 80}, {60, -30}}, color = {192, 192, 192}, pattern = LinePattern.Dash), Polygon(points = {{40, 56}, {33, 58}, {33, 54}, {40, 56}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Polygon(points = {{20, 56}, {27, 58}, {27, 54}, {20, 56}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Polygon(points = {{20, 56}, {13, 58}, {13, 54}, {20, 56}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Polygon(points = {{-10, 56}, {-3, 58}, {-3, 54}, {-10, 56}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Text(extent = {{-34, 70}, {-5, 60}}, lineColor = {160, 160, 164}, textString = "rising"), Text(extent = {{16, 70}, {45, 60}}, lineColor = {160, 160, 164}, textString = "falling"), Text(extent = {{-73, 75}, {-53, 95}}, lineColor = {192, 192, 192}, textString = "i")}),
            Documentation(revisions = "<html>
<ul>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>", info = "<html>
<p>This current source uses the corresponding signal source of the Modelica.Blocks.Sources package. Care for the meaning of the parameters in the Blocks package. Furthermore, an offset parameter is introduced, which is added to the value calculated by the blocks source. The startTime parameter allows to shift the blocks source behavior on the time axis.</p>
</html>"));
        end TrapezoidCurrent;

        model TableCurrent "Current source by linear interpolation in a table"
          parameter Real table[:, :] = [0, 0; 1, 1; 2, 4] "Table matrix (time = first column, current = second column)";
          extends Interfaces.CurrentSource(redeclare Modelica.Blocks.Sources.TimeTable signalSource(table = table));
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-66, -36}, {-66, 84}, {34, 84}, {34, -36}, {-66, -36}, {-66, -6}, {34, -6}, {34, 24}, {-66, 24}, {-66, 54}, {34, 54}, {34, 84}, {-16, 84}, {-16, -37}}, color = {192, 192, 192})}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{-80, 90}, {-88, 68}, {-72, 68}, {-80, 90}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-80, 68}, {-80, -80}}, color = {192, 192, 192}), Line(points = {{-90, -70}, {82, -70}}, color = {192, 192, 192}), Polygon(points = {{90, -70}, {68, -62}, {68, -78}, {90, -70}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-20, 90}, {30, -30}}, lineColor = {255, 255, 255}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-20, -30}, {-20, 90}, {80, 90}, {80, -30}, {-20, -30}, {-20, 0}, {80, 0}, {80, 30}, {-20, 30}, {-20, 60}, {80, 60}, {80, 90}, {30, 90}, {30, -31}}, color = {0, 0, 0}), Text(extent = {{-77, -42}, {-38, -58}}, lineColor = {160, 160, 164}, textString = "offset"), Polygon(points = {{-31, -30}, {-33, -40}, {-28, -40}, {-31, -30}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Polygon(points = {{-30, -70}, {-33, -60}, {-28, -60}, {-30, -70}, {-30, -70}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-31, -31}, {-31, -70}}, color = {192, 192, 192}, pattern = LinePattern.Solid, thickness = 0.25, arrow = {Arrow.None, Arrow.None}), Line(points = {{-20, -20}, {-20, -70}}, color = {192, 192, 192}, pattern = LinePattern.Dash), Text(extent = {{-38, -70}, {8, -88}}, lineColor = {160, 160, 164}, textString = "startTime"), Line(points = {{-20, -30}, {-80, -30}}, color = {192, 192, 192}, pattern = LinePattern.Dash), Text(extent = {{66, -81}, {91, -93}}, lineColor = {160, 160, 164}, textString = "time"), Text(extent = {{-15, 83}, {24, 68}}, lineColor = {0, 0, 0}, textString = "time"), Text(extent = {{33, 83}, {76, 67}}, lineColor = {0, 0, 0}, textString = "i"), Text(extent = {{-73, 75}, {-53, 95}}, lineColor = {192, 192, 192}, textString = "i")}),
            Documentation(info = "<html>
<p>This current source uses the corresponding signal source of the Modelica.Blocks.Sources package. Furthermore, an offset parameter is introduced, which is added to the value calculated by the blocks source. The startTime parameter allows to shift the blocks source behavior on the time axis.</p>
<p><br>This block generates a current source by <b>linear interpolation</b> in a table. The time points and current values are stored in a matrix <b>table[i,j]</b>, where the first column table[:,1] contains the time points and the second column contains the current to be interpolated. The table interpolation has the following properties:</p>
<ul>
<li>The time points need to be <b>monotonically increasing</b>. </li>
<li><b>Discontinuities</b> are allowed, by providing the same time point twice in the table. </li>
<li>Values <b>outside</b> of the table range, are computed by <b>extrapolation</b> through the last or first two points of the table.</li>
<li>If the table has only <b>one row</b>, no interpolation is performed and the current value is just returned independently of the actual time instant, i.e., this is a constant current source.</li>
<li>Via parameters <b>startTime</b> and <b>offset</b> the curve defined by the table can be shifted both in time and in the current. </li>
<li>The table is implemented in a numerically sound way by generating <b>time events</b> at interval boundaries.
    This generates continuously differentiable values for the integrator.</li>
</ul>
<p>Example:</p>
<pre>   table = [0  0
            1  0
            1  1
            2  4
            3  9
            4 16]
If, e.g., time = 1.0, the current i =  0.0 (before event), 1.0 (after event)
    e.g., time = 1.5, the current i =  2.5,
    e.g., time = 2.0, the current i =  4.0,
    e.g., time = 5.0, the current i = 23.0 (i.e., extrapolation). </pre>
<p><br> Furthermore, an offset parameter is introduced, which is added to the value calculated by the blocks source. The startTime parameter allows to shift the blocks source behavior on the time axis.</p>
</html>", revisions = "<html>
<ul>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>"));
        end TableCurrent;
        annotation(
          Documentation(info = "<html>
<p>This package contains time-dependent and controlled voltage and current sources. Most of the sources use the behavior modeled in the Modelica.Blocks.Sources package. All sources are ideal in the sense that <b>no</b> internal resistances are included.</p>
</html>", revisions = "<html>
<dl>
<dt>
<b>Main Authors:</b>
</dt>
<dd>
Christoph Clau&szlig;
    &lt;<a href=\"mailto:Christoph.Clauss@eas.iis.fraunhofer.de\">Christoph.Clauss@eas.iis.fraunhofer.de</a>&gt;<br>
    Andr&eacute; Schneider
    &lt;<a href=\"mailto:Andre.Schneider@eas.iis.fraunhofer.de\">Andre.Schneider@eas.iis.fraunhofer.de</a>&gt;<br>
    Fraunhofer Institute for Integrated Circuits<br>
    Design Automation Department<br>
    Zeunerstra&szlig;e 38<br>
    D-01069 Dresden
</dd>
<dt>
<b>Copyright:</b>
</dt>
<dd>
Copyright &copy; 1998-2015, Modelica Association and Fraunhofer-Gesellschaft.<br>
<i>The Modelica package is <b>free</b> software; it can be redistributed and/or modified
under the terms of the <b>Modelica license</b>, see the license conditions
and the accompanying <b>disclaimer</b> in the documentation of package
Modelica in file \"Modelica/package.mo\".</i>
</dd>
</dl>
</html>"));
      end Sources;
      annotation(
        Documentation(info = "<html>
<p>
This package contains packages for analog electrical components:</p>
<ul>
<li>Basic: basic components (resistor, capacitor, conductor, inductor, transformer, gyrator)</li>
<li>Semiconductors: semiconductor devices (diode, bipolar and MOS transistors)</li>
<li>Lines: transmission lines (lossy and lossless)</li>
<li>Ideal: ideal elements (switches, diode, transformer, idle, short, ...)</li>
<li>Sources: time-dependent and controlled voltage and current sources</li>
<li>Sensors: sensors to measure potential, voltage, and current</li>
</ul>
<dl>
<dt>
<b>Main Authors:</b>
</dt>
<dd>
Christoph Clau&szlig;
    &lt;<a href=\"mailto:Christoph.Clauss@eas.iis.fraunhofer.de\">Christoph.Clauss@eas.iis.fraunhofer.de</a>&gt;<br>
    Andr&eacute; Schneider
    &lt;<a href=\"mailto:Andre.Schneider@eas.iis.fraunhofer.de\">Andre.Schneider@eas.iis.fraunhofer.de</a>&gt;<br>
    Fraunhofer Institute for Integrated Circuits<br>
    Design Automation Department<br>
    Zeunerstra&szlig;e 38<br>
    D-01069 Dresden, Germany
</dd>
</dl>
</html>"),
        Icon(graphics = {Line(points = {{12, 60}, {12, -60}}, color = {0, 0, 0}), Line(points = {{-12, 60}, {-12, -60}}, color = {0, 0, 0}), Line(points = {{-80, 0}, {-12, 0}}, color = {0, 0, 0}), Line(points = {{12, 0}, {80, 0}}, color = {0, 0, 0})}));
    end Analog;

    package Digital "Library for digital electrical components based on the VHDL standard with 9-valued logic and conversion to 2-,3-,4-valued logic"
      extends Modelica.Icons.Package;

      package UsersGuide "User's Guide"
        extends Modelica.Icons.Information;

        class ReleaseNotes "Release notes"
          extends Modelica.Icons.ReleaseNotes;
          annotation(
            Documentation(info = "<html>
<h4>Version 1.0.8, 2009-10-01</h4>
<ul>
<li>Package Register with Flip-Flops and Latches added</li>
<li>InertialDealySensitiveVector added</li>
</ul>
<h4>Version 1.0.7, 2005-07-01</h4>
<ul>
<li>InertialDelaySensitive improved</li>
<li>Minor errors fixed in basic and delay models (final introduced)</li>
<li>Sources.Pulse rewritten to avoid a warning</li>
<li>Minor errors in default values of sources fixed</li>
</ul>
<h4>Version 1.0.6, 2004-10-18</h4>
<ul>
<li> Missing HTML tags added (problems with mismatched pre tags fixed).</li>
<li> CVS ID string deleted.</li>
</ul>
<h4>Version 1.0.5, 2004-10-01</h4>
<ul>
<li> Wrong identifiers x0 and Tdel in HalfAdder example fixed.</li>
<li> Experiment command in FlipFlop example deleted.</li>
<li> Known issue: Pulse source causes a warning in Dymola. It is recommended to use Clock source.</li>
</ul>
<h4>Version 1.0.4, 2004-09-30</h4>
<ul>
<li> Documentation improved.</li>
</ul>
<h4>Version 1.0.3, 2004-09-21</h4>
<ul>
<li> Table names changed from \"map\" to \"Table\".</li>
<li> Icons for converters modified.</li>
<li> LogicValueType renamed to Logic. For the Electrical.Digital library
     the type Logic has a fundamental meaning. Logic is similar to
     Real, Integer or Boolean in other packages. Names for converters
     are now more consistent (LogicToBoolean, RealToLogic etc.).</li>
<li> Icons for gates and sources improved.</li>
<li> New examples added.</li>
<li> Internal names for signals and ports unified.</li>
<li> Simple Clock source added in addition to Pulse source (for convenience reasons).</li>
</ul>
<h4>Version 1.0.2, 2004-09-13</h4>
<ul>
<li> First prerelease for discussions at the 40th Modelica Design Meeting.</li>
</ul>
<h4>Version 1.0.1, 2004-06-01</h4>
<ul>
<li> Packages Tables, Basic, and Gates implemented.</li>
<li> Transport and inertial delay implemented and successfully tested.</li>
</ul>
<h4>Version 1.0.0, 2003-05-01</h4>
<ul>
<li> A first version has been implemented for case studies.</li>
</ul>
</html>"));
        end ReleaseNotes;

        class Literature "Literature"
          extends Modelica.Icons.References;
          annotation(
            Documentation(info = "<html>
<p>
The Electrical.Digital library is based on the following references:
</p>
<dl>
<dt>Ashenden, P. J.:</dt>
<dd> <b>The Designer's Guide to VHDL.</b> San Francisco: Morgan Kaufmann, 1995, 688 p. ISBN 1-55860-270-4.
     <br>&nbsp;</dd>
</dl>
<dl>
<dt>IEEE 1076-1993:</dt>
<dd> <b>IEEE Standard VHDL Language Reference Manual (ANSI).</b> 288 p. ISBN 1-55937-376-8. IEEE Ref. SH16840-NYF.
     <br>&nbsp;</dd>
</dl>
<dl>
<dt>IEEE 1164-1993:</dt>
<dd> <b>IEEE Standard Multivalue Logic System for VHDL Model Interoperability (Std_logic_1164).</b> 24 p. ISBN 1-55937-299-0. IEEE Ref. SH16097-NYF.
     <br>&nbsp;</dd>
</dl>
<dl>
<dt>Lipsett, R.; Schaefer, C.; Ussery, C.:</dt>
<dd> <b>VHDL: Hardware Description and Design.</b> Boston: Kluwer, 1989, 299 p. ISBN 079239030X.
     <br>&nbsp;</dd>
</dl>
<dl>
<dt>Navabi, Z:</dt>
<dd> <b>VHDL: Analysis and Modeling of Digital Systems.</b> New York: McGraw-Hill, 1993, 375 p. ISBN 0070464723.
     <br>&nbsp;</dd>
</dl>
</html>"));
        end Literature;

        class Contact "Contact"
          extends Modelica.Icons.Contact;
          annotation(
            Documentation(info = "<html>
<dl>
<dt><b>Main Authors:</b>
<dd>Christoph Clau&szlig;
    &lt;<A HREF=\"mailto:Christoph.Clauss@eas.iis.fraunhofer.de\">Christoph.Clauss@eas.iis.fraunhofer.de</A>&gt;<br>
    Andr&eacute; Schneider
    &lt;<A HREF=\"mailto:Andre.Schneider@eas.iis.fraunhofer.de\">Andre.Schneider@eas.iis.fraunhofer.de</A>&gt;<br>
    Ulrich Donath
    &lt;<A HREF=\"mailto:Ulrich.Donath@eas.iis.fraunhofer.de\">Ulrich.Donath@eas.iis.fraunhofer.de</A>&gt;<br>
    Fraunhofer Institute for Integrated Circuits (IIS)<br>
    Design Automation Department (EAS)<br>
    Zeunerstra&szlig;e 38<br>
    D-01069 Dresden<br>
    Germany<br>
</dl>
<br>
<br>
<dl>
<dt><b>Acknowledgements:</b></dt>
<dd>
    We thank our students Teresa Schlegel and Enrico Weber for implementing
    and carefully testing many models and examples.<br>
</dd>
</dl>
</html>"));
        end Contact;
        annotation(
          DocumentationClass = true,
          Documentation(info = "<html>
<p>
Library <b>Electrical.Digital</b> is a <b>free</b> Modelica package providing
components to model <b>digital</b> electronic
systems based on combinational and sequential logic in a convenient
way. This package contains the <b>User's Guide</b> for
the library and has the following content:
</p>
<ol>
<li><a href=\"modelica://Modelica.Electrical.Digital.UsersGuide.ReleaseNotes\">Release Notes</a>
    summarizes the differences between different versions of this library.</li>
<li><a href=\"modelica://Modelica.Electrical.Digital.UsersGuide.Literature\">Literature</a>
    provides references that have been used to design and implement this
    library.</li>
<li><a href=\"modelica://Modelica.Electrical.Digital.UsersGuide.Contact\">Contact</a>
    provides information about the authors of the library as well as
    acknowledgments.</li>
</ol>
</html>"));
      end UsersGuide;

      package Examples "Examples that demonstrate the usage of the Digital electrical components"
        extends Modelica.Icons.ExamplesPackage;

        model Multiplexer "4 to 1 Bit Multiplexer Example"
          import D = Modelica.Electrical.Digital;
          import L = Modelica.Electrical.Digital.Interfaces.Logic;
          extends Modelica.Icons.Example;
          D.Sources.Clock CLK(period = 20, startTime = 0, width = 50) annotation(
            Placement(transformation(extent = {{-80, -56}, {-60, -36}}, rotation = 0)));
          D.Sources.Table D0(y0 = L.'0', x = {L.'1', L.'0', L.'1', L.'0'}, t = {50, 100, 145, 200}) annotation(
            Placement(transformation(extent = {{-80, 58}, {-60, 78}}, rotation = 0)));
          D.Sources.Table D1(y0 = L.'0', x = {L.'1', L.'0', L.'1', L.'0'}, t = {22, 140, 150, 180}) annotation(
            Placement(transformation(extent = {{-80, 32}, {-60, 52}}, rotation = 0)));
          D.Examples.Utilities.MUX4 MUX annotation(
            Placement(transformation(extent = {{-10, 0}, {70, 80}}, rotation = 0)));
          D.Sources.Table D2(y0 = L.'0', x = {L.'1', L.'0', L.'1', L.'0'}, t = {22, 140, 150, 180}) annotation(
            Placement(transformation(extent = {{-80, 6}, {-60, 26}}, rotation = 0)));
          D.Sources.Table D3(y0 = L.'0', x = {L.'1', L.'0', L.'1', L.'0'}, t = {22, 140, 150, 180}) annotation(
            Placement(transformation(extent = {{-80, -20}, {-60, 0}}, rotation = 0)));
          D.Examples.Utilities.JKFF FF annotation(
            Placement(transformation(extent = {{-20, -62}, {0, -42}}, rotation = 0)));
          D.Sources.Set Enable(x = Modelica.Electrical.Digital.Interfaces.Logic.'1') annotation(
            Placement(transformation(extent = {{-80, -82}, {-60, -62}}, rotation = 0)));
        equation
          connect(CLK.y, FF.clk) annotation(
            Line(points = {{-60, -46}, {-36, -46}, {-36, -52}, {-20, -52}}, color = {127, 0, 127}));
          connect(Enable.y, FF.k) annotation(
            Line(points = {{-60, -72}, {-30, -72}, {-30, -59}, {-20, -59}}, color = {127, 0, 127}));
          connect(Enable.y, FF.j) annotation(
            Line(points = {{-60, -72}, {-30, -72}, {-30, -45}, {-20, -45}}, color = {127, 0, 127}));
          connect(CLK.y, MUX.a0) annotation(
            Line(points = {{-60, -46}, {-36, -46}, {-36, 22.4}, {-10, 22.4}}, color = {127, 0, 127}));
          connect(D0.y, MUX.d0) annotation(
            Line(points = {{-60, 68}, {-10, 68}}, color = {127, 0, 127}));
          connect(D1.y, MUX.d1) annotation(
            Line(points = {{-60, 42}, {-54, 42}, {-54, 57.6}, {-10, 57.6}}, color = {127, 0, 127}));
          connect(D2.y, MUX.d2) annotation(
            Line(points = {{-60, 16}, {-50, 16}, {-50, 47.2}, {-10, 47.2}}, color = {127, 0, 127}));
          connect(D3.y, MUX.d3) annotation(
            Line(points = {{-60, -10}, {-46, -10}, {-46, 36.8}, {-10, 36.8}}, color = {127, 0, 127}));
          connect(FF.q, MUX.a1) annotation(
            Line(points = {{0, -45}, {2, -45}, {2, -22}, {-20, -22}, {-20, 12}, {-10, 12}}, color = {127, 0, 127}));
          annotation(
            Documentation(info = "<html>
<p>The multiplexer converts a parallel 4 bit signal in a sequential 1 bit stream. The multiplexer component is composed by basic gates. It can be found in the Utilities subpackage. The example is designed to test and demonstrate the basic gate components.</p>
</html>"),
            experiment(StopTime = 250));
        end Multiplexer;

        model FlipFlop "Pulse Triggered Master Slave Flip-Flop"
          import D = Modelica.Electrical.Digital;
          import L = Modelica.Electrical.Digital.Interfaces.Logic;
          extends Modelica.Icons.Example;
          D.Examples.Utilities.JKFF FF annotation(
            Placement(transformation(extent = {{-10, -40}, {70, 40}}, rotation = 0)));
          D.Sources.Clock CLK(period = 10, startTime = 0, width = 50) annotation(
            Placement(transformation(extent = {{-80, -10}, {-60, 10}}, rotation = 0)));
          D.Sources.Table J(y0 = L.'0', x = {L.'1', L.'0', L.'1', L.'0'}, t = {50, 100, 145, 200}) annotation(
            Placement(transformation(extent = {{-80, 18}, {-60, 38}}, rotation = 0)));
          D.Sources.Table K(y0 = L.'0', x = {L.'1', L.'0', L.'1', L.'0'}, t = {22, 140, 150, 180}) annotation(
            Placement(transformation(extent = {{-80, -38}, {-60, -18}}, rotation = 0)));
        equation
          connect(J.y, FF.j) annotation(
            Line(points = {{-60, 28}, {-10, 28}}, color = {127, 0, 127}));
          connect(CLK.y, FF.clk) annotation(
            Line(points = {{-60, 0}, {-10, 0}}, color = {127, 0, 127}));
          connect(K.y, FF.k) annotation(
            Line(points = {{-60, -28}, {-10, -28}}, color = {127, 0, 127}));
          annotation(
            Documentation(info = "<html>
<p>A pulse-triggered master-slave flip-flop is demonstrated. The flipflop component is composed by basic gates. It can be found in the Utilities subpackage. The example is designed to test and demonstrate the basic gate components.</p>
</html>"),
            experiment(StopTime = 250));
        end FlipFlop;

        model HalfAdder "Adding circuit for binary numbers without input carry bit"
          import Modelica.Electrical.Digital;
          import L = Modelica.Electrical.Digital.Interfaces.Logic;
          extends Modelica.Icons.Example;
          Modelica.Electrical.Digital.Sources.Table a(t = {1, 2, 3, 4}, x = {L.'1', L.'0', L.'1', L.'0'}, y0 = L.'0') annotation(
            Placement(transformation(extent = {{-80, 18}, {-60, 38}}, rotation = 0)));
          Modelica.Electrical.Digital.Sources.Table b(x = {L.'1', L.'0'}, t = {2, 4}, y0 = L.'0') annotation(
            Placement(transformation(extent = {{-80, -38}, {-60, -18}}, rotation = 0)));
          Modelica.Electrical.Digital.Examples.Utilities.HalfAdder Adder(delayTime = 0.3, AND(G2(y(start = L.'U', fixed = true))), XOR(G2(y(start = L.'U', fixed = true)))) annotation(
            Placement(transformation(extent = {{-40, -40}, {40, 40}}, rotation = 0)));
          Modelica.Electrical.Digital.Converters.LogicToReal s(n = 1, value_U = 0.5, value_X = 0.5, value_0 = 0, value_1 = 1, value_Z = 0.5, value_W = 0.5, value_L = 0, value_H = 1, value_m = 0.5) annotation(
            Placement(transformation(extent = {{60, 18}, {80, 38}}, rotation = 0)));
          Modelica.Electrical.Digital.Converters.LogicToReal c(n = 1, value_U = 0.5, value_X = 0.5, value_0 = 0, value_1 = 1, value_Z = 0.5, value_W = 0.5, value_L = 0, value_H = 1, value_m = 0.5) annotation(
            Placement(transformation(extent = {{60, -38}, {80, -18}}, rotation = 0)));
        equation
          connect(b.y, Adder.b) annotation(
            Line(points = {{-60, -28}, {-40, -28}}, color = {127, 0, 127}));
          connect(a.y, Adder.a) annotation(
            Line(points = {{-60, 28}, {-40, 28}}, color = {127, 0, 127}));
          connect(Adder.s, s.x[1]) annotation(
            Line(points = {{40, 28}, {65, 28}}, color = {127, 0, 127}));
          connect(Adder.c, c.x[1]) annotation(
            Line(points = {{40, -28}, {65, -28}}, color = {127, 0, 127}));
          annotation(
            Documentation(info = "<HTML>
<p>
This example demonstrates an adding circuit for binary numbers, which internally realizes the interconnection to
And and to Xor in the final sum.</p>
<br>
<br>
1 + 0 = 1<br>
0 + 1 = 1<br>
1 + 1 = 10<br>
0 + 0 = 0
<br>
<br>
<b>a</b> + <b>b</b> = <b>s</b>
<br>(The carry of this adding is <b>c</b>.)
<br>
<br>and
<br>
<br>
<b>a</b> * <b>b</b> = <b>s</b>
<br>  (It is an interconnection to And.)
<br>
<br>
<b>a</b> * <b>b</b> + <b>a</b> * <b>b</b> = <b>a</b> Xor <b>b</b> = <b>c</b>
<br>(It is an interconnection to Xor.)
<br>
<pre>
   <b>a</b>     <b>b</b>     <b>c</b>      <b>s</b>     <b>t</b>

   1     0     1      0     1
   0     1     1      0     2
   1     1     0      1     3
   0     0     0      0     4
</pre>
<p>
<b>t</b> is the pick-up instant of the next bit(s) in the simulation.
The simulation stop time should be 5 seconds.
</p>
</HTML>"),
            experiment(StopTime = 5));
        end HalfAdder;

        model FullAdder "Full 1 Bit Adder Example"
          import D = Modelica.Electrical.Digital;
          import L = Modelica.Electrical.Digital.Interfaces.Logic;
          extends Modelica.Icons.Example;
          D.Examples.Utilities.FullAdder Adder1 annotation(
            Placement(transformation(extent = {{0, -30}, {60, 30}}, rotation = 0)));
          D.Converters.LogicToReal s(value_0 = 0, value_1 = 1, value_H = 1, value_L = 0, value_m = 0.5, value_U = 0.5, value_W = 0.5, value_X = 0.5, value_Z = 0.5, n = 1) annotation(
            Placement(transformation(extent = {{70, 12}, {90, 32}}, rotation = 0)));
          D.Converters.LogicToReal c_out(value_0 = 0, value_1 = 1, value_H = 1, value_L = 0, value_m = 0.5, value_U = 0.5, value_W = 0.5, value_X = 0.5, value_Z = 0.5, n = 1) annotation(
            Placement(transformation(extent = {{70, -32}, {90, -12}}, rotation = 0)));
          D.Examples.Utilities.Counter3 Counter annotation(
            Placement(transformation(extent = {{-60, -18}, {-20, 22}}, rotation = 0)));
          D.Sources.Set Enable(x = L.'1') annotation(
            Placement(transformation(extent = {{-90, 6}, {-70, 26}}, rotation = 0)));
          D.Sources.Clock CLK(period = 1, startTime = 0, width = 50) annotation(
            Placement(transformation(extent = {{-90, -22}, {-70, -2}}, rotation = 0)));
        equation
          connect(Adder1.s, s.x[1]) annotation(
            Line(points = {{60.3, 21}, {68, 21}, {68, 22}, {75, 22}}, color = {127, 0, 127}));
          connect(Adder1.c_out, c_out.x[1]) annotation(
            Line(points = {{60, -21}, {68, -21}, {68, -22}, {75, -22}}, color = {127, 0, 127}));
          connect(CLK.y, Counter.count) annotation(
            Line(points = {{-70, -12}, {-60, -12}}, color = {127, 0, 127}));
          connect(Enable.y, Counter.enable) annotation(
            Line(points = {{-70, 16}, {-60, 16}}, color = {127, 0, 127}));
          connect(Counter.q2, Adder1.a) annotation(
            Line(points = {{-20, 16}, {-10, 16}, {-10, 21}, {0, 21}}, color = {127, 0, 127}));
          connect(Counter.q1, Adder1.b) annotation(
            Line(points = {{-20, 2}, {-10, 2}, {-10, 9}, {0, 9}}, color = {127, 0, 127}));
          connect(Counter.q0, Adder1.c_in) annotation(
            Line(points = {{-20, -12}, {-10, -12}, {-10, -21}, {0, -21}}, color = {127, 0, 127}));
          annotation(
            Documentation(info = "<HTML>
<p>
<br>It is an adding circuit for binary numbers with input carry bit, which consists of two HalfAdders.
<br>
<br>
<b>a</b>.y, <b>b</b>.y and <b>c</b>.y are the inputs of the FullAdder.
<br>
<b>c</b>out = <b>Or1</b>.y and <b>h</b>.s are the outputs of the FullAdder.
<br>
<br>
<b>t</b> is the pick-up instant of the next bit(s) in the simulation.</p>
<pre>   <b>a</b>.y      <b>b</b>.y      <b>c</b>.y     <b>c</b>out        <b>h</b>.s        <b>t</b> </pre>

<pre>
     1        0        0        0          1        1
     0        1        0        0          1        2
     0        0        1        0          1        3
     1        1        0        1          0        4
     0        1        1        1          0        5
     1        0        1        1          0        6
     1        1        1        1          1        7
     0        0        0        0          0        8
</pre>
<p>
The simulation stop time should be 10 seconds.
</p>
</html>"),
            experiment(StopTime = 10));
        end FullAdder;

        model Adder4 "4 Bit Adder Example"
          import Modelica.Electrical.Digital;
          import L = Modelica.Electrical.Digital.Interfaces.Logic;
          extends Modelica.Icons.Example;
          Modelica.Electrical.Digital.Sources.Table b4(y0 = L.'0', x = {L.'1', L.'0'}, t = {1, 3}) annotation(
            Placement(transformation(extent = {{70, -20}, {110, 20}}, rotation = 0)));
          Modelica.Electrical.Digital.Sources.Table b1(x = {L.'1', L.'0', L.'1'}, y0 = L.'0', t = {1, 2, 3}) annotation(
            Placement(transformation(extent = {{-170, -20}, {-130, 20}}, rotation = 0)));
          Modelica.Electrical.Digital.Sources.Table b2(y0 = L.'0', x = {L.'1'}, t = {4}) annotation(
            Placement(transformation(extent = {{-90, -20}, {-50, 20}}, rotation = 0)));
          Modelica.Electrical.Digital.Sources.Table b3(y0 = L.'0', x = {L.'1'}, t = {1}) annotation(
            Placement(transformation(extent = {{-10, -20}, {30, 20}}, rotation = 0)));
          Modelica.Electrical.Digital.Sources.Table a1(y0 = L.'0', x = {L.'1', L.'0', L.'1'}, t = {1, 2, 3}) annotation(
            Placement(transformation(extent = {{-170, 40}, {-130, 80}}, rotation = 0)));
          Modelica.Electrical.Digital.Sources.Table a2(y0 = L.'0', x = {L.'1'}, t = {1}) annotation(
            Placement(transformation(extent = {{-90, 40}, {-50, 80}}, rotation = 0)));
          Modelica.Electrical.Digital.Sources.Table a3(y0 = L.'0', x = {L.'1', L.'0'}, t = {1, 4}) annotation(
            Placement(transformation(extent = {{-8, 40}, {30, 80}}, rotation = 0)));
          Modelica.Electrical.Digital.Sources.Table a4(y0 = L.'0', x = {L.'0'}, t = {1}) annotation(
            Placement(transformation(extent = {{70, 40}, {110, 80}}, rotation = 0)));
          Modelica.Electrical.Digital.Sources.Set Set(x = L.'0') annotation(
            Placement(transformation(origin = {-150, -74}, extent = {{20, 20}, {-20, -20}}, rotation = 180)));
          Modelica.Electrical.Digital.Examples.Utilities.FullAdder Adder1(Adder1(AND(G2(y(start = L.'U', fixed = true))), XOR(G2(y(start = L.'U', fixed = true)))), Adder2(AND(G2(y(start = L.'U', fixed = true))), XOR(G2(y(start = L.'U', fixed = true))))) annotation(
            Placement(transformation(extent = {{-100, -80}, {-60, -40}}, rotation = 0)));
          Modelica.Electrical.Digital.Examples.Utilities.FullAdder Adder2(Adder1(AND(G2(y(start = L.'U', fixed = true))), XOR(G2(y(start = L.'U', fixed = true)))), Adder2(AND(G2(y(start = L.'U', fixed = true))), XOR(G2(y(start = L.'U', fixed = true))))) annotation(
            Placement(transformation(extent = {{-20, -80}, {20, -40}}, rotation = 0)));
          Modelica.Electrical.Digital.Examples.Utilities.FullAdder Adder3(Adder1(AND(G2(y(start = L.'U', fixed = true))), XOR(G2(y(start = L.'U', fixed = true)))), Adder2(AND(G2(y(start = L.'U', fixed = true))), XOR(G2(y(start = L.'U', fixed = true))))) annotation(
            Placement(transformation(extent = {{60, -80}, {100, -40}}, rotation = 0)));
          Modelica.Electrical.Digital.Examples.Utilities.FullAdder Adder4(Adder1(AND(G2(y(start = L.'U', fixed = true))), XOR(G2(y(start = L.'U', fixed = true)))), Adder2(AND(G2(y(start = L.'U', fixed = true))), XOR(G2(y(start = L.'U', fixed = true))))) annotation(
            Placement(transformation(extent = {{140, -80}, {180, -40}}, rotation = 0)));
        equation
          connect(b1.y, Adder1.b) annotation(
            Line(points = {{-130, 0}, {-120, 0}, {-120, -54}, {-100, -54}}, color = {127, 0, 127}));
          connect(a1.y, Adder1.a) annotation(
            Line(points = {{-130, 60}, {-110, 60}, {-110, -46}, {-100, -46}}, color = {127, 0, 127}));
          connect(Set.y, Adder1.c_in) annotation(
            Line(points = {{-130, -74}, {-100, -74}}, color = {127, 0, 127}));
          connect(Adder1.c_out, Adder2.c_in) annotation(
            Line(points = {{-60, -74}, {-20, -74}}, color = {127, 0, 127}));
          connect(Adder2.c_out, Adder3.c_in) annotation(
            Line(points = {{20, -74}, {60, -74}}, color = {127, 0, 127}));
          connect(Adder3.c_out, Adder4.c_in) annotation(
            Line(points = {{100, -74}, {140, -74}}, color = {127, 0, 127}));
          connect(b2.y, Adder2.b) annotation(
            Line(points = {{-50, 0}, {-40, 0}, {-40, -54}, {-20, -54}}, color = {127, 0, 127}));
          connect(a2.y, Adder2.a) annotation(
            Line(points = {{-50, 60}, {-30, 60}, {-30, -46}, {-20, -46}}, color = {127, 0, 127}));
          connect(b3.y, Adder3.b) annotation(
            Line(points = {{30, 0}, {40, 0}, {40, -54}, {60, -54}}, color = {127, 0, 127}));
          connect(a3.y, Adder3.a) annotation(
            Line(points = {{30, 60}, {50, 60}, {50, -46}, {60, -46}}, color = {127, 0, 127}));
          connect(b4.y, Adder4.b) annotation(
            Line(points = {{110, 0}, {120, 0}, {120, -54}, {140, -54}}, color = {127, 0, 127}));
          connect(a4.y, Adder4.a) annotation(
            Line(points = {{110, 60}, {130, 60}, {130, -46}, {140, -46}}, color = {127, 0, 127}));
          annotation(
            Diagram(coordinateSystem(extent = {{-200, -100}, {200, 100}})),
            Documentation(info = "<HTML>
<p>
Four FullAdders are combined to built a four bit adder unit.
</p>
<br>
<br>
In dependence on time five additions are carried out:
<br>
<pre>
at t = 0                            at t = 1
 a       0 0 0 0                       a      1 1 1 0
 b    +  0 0 0 0                       b   +  1 0 1 1
 <b>s     0 0 0 0 0</b>                      <b>s     1 0 0 1 0</b>
at t = 2                             at t = 3
 a       0 1 1 0                       a      1 1 1 0
 b    +  0 0 1 1                       b   +  1 0 1 0
 <b>s     1 0 1 0 0</b>                      <b>s     0 0 0 1 1</b>

at t = 4
 a      1 1 0 0
 b   +  1 1 1 0
 <b>s    0 0 1 0 1</b>
 </pre>
<p>
To show the influence of delay a large delay time of 0.1s is chosen.
Furthermore, all signals are initialized with U, the uninitialized value.
Please remember, that the nine logic values are coded by the numbers 1,...,9.
The summands a and b can be found at the output signals of the taba and tabb sources.
The result can be seen in the output signals of the FullAdders according to:</p>
<pre>
      a                       <b>a4</b>.y      <b>a3</b>.y      <b>a2</b>.y      <b>a1</b>.y
      b                       <b>b4</b>.y      <b>b3</b>.y      <b>b2</b>.y      <b>b1</b>.y
      sum   <b>Adder4</b>.c_out  <b>Adder4.s</b>  <b>Adder3.s</b>  <b>Adder2.s</b>  <b>Adder1.s</b>
</pre>
<p>The simulation stop time has to be 5s.</p>
</html>"),
            experiment(StopTime = 5));
        end Adder4;

        model Counter3 "3 Bit Counter Example"
          import D = Modelica.Electrical.Digital;
          extends Modelica.Icons.Example;
          D.Sources.Step Enable(after = D.Interfaces.Logic.'1', before = D.Interfaces.Logic.'0', stepTime = 1) annotation(
            Placement(transformation(extent = {{-90, 8}, {-50, 48}}, rotation = 0)));
          D.Sources.Clock Clock(period = 1, startTime = 0, width = 50) annotation(
            Placement(transformation(extent = {{-90, -48}, {-50, -8}}, rotation = 0)));
          D.Examples.Utilities.Counter3 Counter annotation(
            Placement(transformation(extent = {{-30, -40}, {50, 40}}, rotation = 0)));
        equation
          connect(Enable.y, Counter.enable) annotation(
            Line(points = {{-50, 28}, {-30, 28}}, color = {127, 0, 127}));
          connect(Clock.y, Counter.count) annotation(
            Line(points = {{-50, -28}, {-30, -28}}, color = {127, 0, 127}));
          annotation(
            Documentation(info = "<html>
<p>The three bit counter example is built up by components of the Utilities package which uses components of the Gates library.</p>
<p>The counter counts the high-low slopes of the clock signal, if the enable signal is set to be true. Otherwise if it is set to zero the counter is not counting.</p>
<p>Plot Counter.count (clock signal), and Counter.enable, and the output bit signals Counter.q0, Counter.q1, and Counter.q2.</p>
</html>"),
            experiment(StopTime = 10));
        end Counter3;

        model Counter "Generic N Bit Counter Example"
          import D = Modelica.Electrical.Digital;
          extends Modelica.Icons.Example;
          D.Sources.Step Enable(after = D.Interfaces.Logic.'1', before = D.Interfaces.Logic.'0', stepTime = 1) annotation(
            Placement(transformation(extent = {{-90, 8}, {-50, 48}}, rotation = 0)));
          D.Sources.Clock Clock(period = 1, startTime = 0, width = 50) annotation(
            Placement(transformation(extent = {{-90, -48}, {-50, -8}}, rotation = 0)));
          D.Examples.Utilities.Counter Counter(n = 4) annotation(
            Placement(transformation(extent = {{-30, -40}, {50, 40}}, rotation = 0)));
          D.Converters.LogicToReal Q0(value_0 = 0, value_1 = 1, value_H = 1, value_L = 0, value_m = 0.5, value_U = 0.5, value_W = 0.5, value_X = 0.5, value_Z = 0.5, n = 1) annotation(
            Placement(transformation(extent = {{66, -40}, {86, -20}}, rotation = 0)));
          D.Converters.LogicToReal Q1(value_0 = 0, value_1 = 1, value_H = 1, value_L = 0, value_m = 0.5, value_U = 0.5, value_W = 0.5, value_X = 0.5, value_Z = 0.5, n = 1) annotation(
            Placement(transformation(extent = {{66, -20}, {86, 0}}, rotation = 0)));
          D.Converters.LogicToReal Q2(value_0 = 0, value_1 = 1, value_H = 1, value_L = 0, value_m = 0.5, value_U = 0.5, value_W = 0.5, value_X = 0.5, value_Z = 0.5, n = 1) annotation(
            Placement(transformation(extent = {{66, 0}, {86, 20}}, rotation = 0)));
          D.Converters.LogicToReal Q3(value_0 = 0, value_1 = 1, value_H = 1, value_L = 0, value_m = 0.5, value_U = 0.5, value_W = 0.5, value_X = 0.5, value_Z = 0.5, n = 1) annotation(
            Placement(transformation(extent = {{66, 20}, {86, 40}}, rotation = 0)));
        equation
          connect(Enable.y, Counter.enable) annotation(
            Line(points = {{-50, 28}, {-30, 28}}, color = {127, 0, 127}));
          connect(Clock.y, Counter.count) annotation(
            Line(points = {{-50, -28}, {-30, -28}}, color = {127, 0, 127}));
          connect(Q0.x[1], Counter.q[1]) annotation(
            Line(points = {{71, -30}, {58, -30}, {58, -24}, {50, -24}}, color = {127, 0, 127}));
          connect(Q1.x[1], Counter.q[2]) annotation(
            Line(points = {{71, -10}, {60, -10}, {60, -8}, {50, -8}}, color = {127, 0, 127}));
          connect(Q2.x[1], Counter.q[3]) annotation(
            Line(points = {{71, 10}, {60, 10}, {60, 8}, {50, 8}}, color = {127, 0, 127}));
          connect(Q3.x[1], Counter.q[4]) annotation(
            Line(points = {{71, 30}, {60, 30}, {60, 24}, {50, 24}}, color = {127, 0, 127}));
          annotation(
            Documentation(info = "<html>
<p>The counter example is built up by components of the Utilities package which uses components of the Gates library. It demonstrates the generic counter model. The parameter n is the number of counting bits. In this example it is set to 4.</p>
<p>The counter counts the high-low slopes of the clock signal, if the enable signal is set to be true. Otherwise if it is set to zero the counter is not counting.</p>
<p>Plot Counter.count (clock signal), and Counter.enable, and the output bit signals Counter.q[0], Counter.q[1], Counter.q[2], and Counter.q[3]</p>
</html>"),
            experiment(StopTime = 100));
        end Counter;

        model VectorDelay "Vector delay"
          import L = Modelica.Electrical.Digital.Interfaces.Logic;
          extends Modelica.Icons.Example;
          Modelica.Electrical.Digital.Delay.InertialDelaySensitiveVector delay(final tHL = 1, final tLH = 2, final n = 3, inertialDelaySensitive(each y(start = L.'U', fixed = true))) annotation(
            Placement(transformation(extent = {{-36, -28}, {40, 48}})));
          Modelica.Electrical.Digital.Sources.Table table(x = {L.'0', L.'1', L.'0', L.'1', L.'0'}, t = {0, 1, 5, 7, 8}) annotation(
            Placement(transformation(extent = {{-96, 40}, {-76, 60}})));
          Modelica.Electrical.Digital.Sources.Table table1(x = {L.'0', L.'1'}, t = {0, 1}) annotation(
            Placement(transformation(extent = {{-96, 0}, {-76, 20}})));
          Modelica.Electrical.Digital.Sources.Table table2(x = {L.'0', L.'1', L.'0'}, t = {0, 1, 6}) annotation(
            Placement(transformation(extent = {{-96, -50}, {-76, -30}})));
        equation
          connect(table.y, delay.x[1]) annotation(
            Line(points = {{-76, 50}, {-30.68, 50}, {-30.68, 6.13667}}, color = {127, 0, 127}, smooth = Smooth.None));
          connect(table1.y, delay.x[2]) annotation(
            Line(points = {{-76, 10}, {-30.68, 10}, {-30.68, 9.81}}, color = {127, 0, 127}, smooth = Smooth.None));
          connect(table2.y, delay.x[3]) annotation(
            Line(points = {{-76, -40}, {-30.68, -40}, {-30.68, 13.4833}}, color = {127, 0, 127}, smooth = Smooth.None));
          annotation(
            Documentation(info = "<html>
<p>This example is a simple test of the vector valued sensitive delay component. The delay times are chosen different from each other. To examine the results plot both the input vector x and the output vector y.</p>
</html>"),
            experiment(StopTime = 10));
        end VectorDelay;

        model DFFREG "Pulse triggered D-Register-Bank, high active reset"
          import L = Modelica.Electrical.Digital.Interfaces.Logic;
          extends Modelica.Icons.Example;
          Modelica.Electrical.Digital.Sources.Table clock(x = {L.'0', L.'1', L.'0', L.'1', L.'0', L.'1', L.'0'}, t = {0, 7, 8, 10, 11, 15, 16}) annotation(
            Placement(transformation(extent = {{-86, -32}, {-66, -12}})));
          Modelica.Electrical.Digital.Sources.Table data_0(x = {L.'1', L.'0'}, t = {0, 12}) annotation(
            Placement(transformation(extent = {{-86, 8}, {-66, 28}})));
          Modelica.Electrical.Digital.Sources.Table reset(x = {L.'0', L.'1', L.'0'}, t = {0, 1, 2}) annotation(
            Placement(transformation(extent = {{-86, -70}, {-66, -50}})));
          Modelica.Electrical.Digital.Sources.Table data_1(t = {0, 10}, x = {L.'H', L.'X'}) annotation(
            Placement(transformation(extent = {{-86, 40}, {-66, 60}})));
          Modelica.Electrical.Digital.Registers.DFFREG dFFREG(n = 2, tHL = 5, tLH = 6, delay(inertialDelaySensitive(each y(start = L.'U', fixed = true))), dFFR(clock(start = L.'U', fixed = true), reset(start = L.'U', fixed = true))) annotation(
            Placement(transformation(extent = {{-24, -26}, {70, 68}})));
        equation
          connect(clock.y, dFFREG.clock) annotation(
            Line(points = {{-66, -22}, {-46, -22}, {-46, 11.6}, {-20.24, 11.6}}, color = {127, 0, 127}, smooth = Smooth.None));
          connect(reset.y, dFFREG.reset) annotation(
            Line(points = {{-66, -60}, {-20.24, -60}, {-20.24, -7.2}}, color = {127, 0, 127}, smooth = Smooth.None));
          connect(data_0.y, dFFREG.dataIn[1]) annotation(
            Line(points = {{-66, 18}, {-44, 18}, {-44, 37.92}, {-20.24, 37.92}}, color = {127, 0, 127}, smooth = Smooth.None));
          connect(data_1.y, dFFREG.dataIn[2]) annotation(
            Line(points = {{-66, 50}, {-44, 50}, {-44, 41.68}, {-20.24, 41.68}}, color = {127, 0, 127}, smooth = Smooth.None));
          annotation(
            experiment(StopTime = 25),
            Documentation(info = "<html>
<p>This example is a simple test of the Registers.DFFREG component. The data width is set to two. After simulation plot both the dataIn and the dataOut vectors. To verify the results compare the the truth table which is documented in the DFFREG component.</p>
</html>"));
        end DFFREG;

        model DFFREGL "Pulse triggered D-Register-Bank, low active reset"
          import L = Modelica.Electrical.Digital.Interfaces.Logic;
          extends Modelica.Icons.Example;
          Modelica.Electrical.Digital.Sources.Table clock(x = {L.'0', L.'1', L.'0', L.'1', L.'0', L.'1', L.'0'}, t = {0, 7, 8, 10, 11, 15, 16}) annotation(
            Placement(transformation(extent = {{-86, -30}, {-66, -10}})));
          Modelica.Electrical.Digital.Sources.Table data_0(x = {L.'1', L.'0'}, t = {0, 12}) annotation(
            Placement(transformation(extent = {{-88, 10}, {-68, 30}})));
          Modelica.Electrical.Digital.Sources.Table reset(t = {0, 1, 2}, x = {L.'1', L.'0', L.'1'}) annotation(
            Placement(transformation(extent = {{-86, -70}, {-66, -50}})));
          Modelica.Electrical.Digital.Sources.Table data_1(t = {0, 10}, x = {L.'H', L.'X'}) annotation(
            Placement(transformation(extent = {{-86, 40}, {-66, 60}})));
          Modelica.Electrical.Digital.Registers.DFFREGL dFFREGL(n = 2, tHL = 5, tLH = 6, delay(inertialDelaySensitive(each y(start = L.'U', fixed = true))), dFFR(clock(start = L.'U', fixed = true), reset(start = L.'U', fixed = true))) annotation(
            Placement(transformation(extent = {{-41, -39}, {62, 65}})));
        equation
          connect(reset.y, dFFREGL.reset) annotation(
            Line(points = {{-66, -60}, {-36.88, -60}, {-36.88, -18.2}}, color = {127, 0, 127}, smooth = Smooth.None));
          connect(clock.y, dFFREGL.clock) annotation(
            Line(points = {{-66, -20}, {-62, -20}, {-62, 2.6}, {-36.88, 2.6}}, color = {127, 0, 127}, smooth = Smooth.None));
          connect(data_0.y, dFFREGL.dataIn[1]) annotation(
            Line(points = {{-68, 20}, {-52, 20}, {-52, 31.72}, {-36.88, 31.72}}, color = {127, 0, 127}, smooth = Smooth.None));
          connect(data_1.y, dFFREGL.dataIn[2]) annotation(
            Line(points = {{-66, 50}, {-52, 50}, {-52, 35.88}, {-36.88, 35.88}}, color = {127, 0, 127}, smooth = Smooth.None));
          annotation(
            experiment(StopTime = 25),
            Documentation(info = "<html>
<p>This example is a simple test of the Registers.DFFREGL component. The data width is set to two. After simulation plot both the dataIn and the dataOut vectors. To verify the results compare the the truth table which is documented in the DFFREGL component.</p>
</html>"));
        end DFFREGL;

        model DFFREGSRH "Pulse triggered D-Register-Bank, high active set and reset"
          import L = Modelica.Electrical.Digital.Interfaces.Logic;
          extends Modelica.Icons.Example;
          Modelica.Electrical.Digital.Sources.Table clock(x = {L.'0', L.'1', L.'0'}, t = {0, 10, 11}) annotation(
            Placement(transformation(extent = {{-86, -32}, {-66, -12}})));
          Modelica.Electrical.Digital.Sources.Table data_0(x = {L.'W'}, t = {0}) annotation(
            Placement(transformation(extent = {{-86, 8}, {-66, 28}})));
          Modelica.Electrical.Digital.Sources.Table reset(x = {L.'0', L.'1', L.'0'}, t = {0, 1, 2}) annotation(
            Placement(transformation(extent = {{-86, -70}, {-66, -50}})));
          Modelica.Electrical.Digital.Sources.Table data_1(x = {L.'0'}, t = {0}) annotation(
            Placement(transformation(extent = {{-86, 40}, {-66, 60}})));
          Modelica.Electrical.Digital.Sources.Table set(x = {L.'0', L.'1', L.'0'}, t = {0, 5, 6}) annotation(
            Placement(transformation(extent = {{-86, 74}, {-66, 94}})));
          Modelica.Electrical.Digital.Registers.DFFREGSRH dFFREGSRH(tHL = 2, tLH = 3, n = 2) annotation(
            Placement(transformation(extent = {{-34, -37}, {73, 71}})));
        equation
          connect(clock.y, dFFREGSRH.clock) annotation(
            Line(points = {{-66, -22}, {-50, -22}, {-50, 6.2}, {-29.72, 6.2}}, color = {127, 0, 127}, smooth = Smooth.None));
          connect(set.y, dFFREGSRH.set) annotation(
            Line(points = {{-66, 84}, {-48, 84}, {-48, 60.2}, {-29.72, 60.2}}, color = {127, 0, 127}, smooth = Smooth.None));
          connect(reset.y, dFFREGSRH.reset) annotation(
            Line(points = {{-66, -60}, {-29.72, -60}, {-29.72, -15.4}}, color = {127, 0, 127}, smooth = Smooth.None));
          connect(data_0.y, dFFREGSRH.dataIn[1]) annotation(
            Line(points = {{-66, 18}, {-48, 18}, {-48, 36.44}, {-29.72, 36.44}}, color = {127, 0, 127}, smooth = Smooth.None));
          connect(data_1.y, dFFREGSRH.dataIn[2]) annotation(
            Line(points = {{-66, 50}, {-48, 50}, {-48, 40.76}, {-29.72, 40.76}}, color = {127, 0, 127}, smooth = Smooth.None));
          annotation(
            experiment(StopTime = 15),
            Documentation(info = "<html>
<p>This example is a simple test of the Registers.DFFREGSRH component. The data width is set to two. After simulation plot both the dataIn and the dataOut vectors. To verify the results compare the the truth table which is documented in the DFFREGSRH component.</p>
</html>"));
        end DFFREGSRH;

        model DFFREGSRL "Pulse triggered D-Register-Bank, low active set and reset"
          import L = Modelica.Electrical.Digital.Interfaces.Logic;
          extends Modelica.Icons.Example;
          Modelica.Electrical.Digital.Sources.Table clock(x = {L.'0', L.'1', L.'0'}, t = {0, 10, 11}) annotation(
            Placement(transformation(extent = {{-86, -32}, {-66, -12}})));
          Modelica.Electrical.Digital.Sources.Table data_0(x = {L.'W'}, t = {0}) annotation(
            Placement(transformation(extent = {{-86, 8}, {-66, 28}})));
          Modelica.Electrical.Digital.Sources.Table reset(t = {0, 1, 2}, x = {L.'1', L.'0', L.'1'}) annotation(
            Placement(transformation(extent = {{-86, -70}, {-66, -50}})));
          Modelica.Electrical.Digital.Sources.Table data_1(x = {L.'0'}, t = {0}) annotation(
            Placement(transformation(extent = {{-86, 40}, {-66, 60}})));
          Modelica.Electrical.Digital.Sources.Table set(t = {0, 5, 6}, x = {L.'1', L.'0', L.'1'}) annotation(
            Placement(transformation(extent = {{-86, 74}, {-66, 94}})));
          Modelica.Electrical.Digital.Registers.DFFREGSRL dFFREGSRL(tHL = 2, tLH = 3, n = 2) annotation(
            Placement(transformation(extent = {{-45, -54}, {81, 72}})));
        equation
          connect(reset.y, dFFREGSRL.reset) annotation(
            Line(points = {{-66, -60}, {-39.96, -60}, {-39.96, -28.8}}, color = {127, 0, 127}, smooth = Smooth.None));
          connect(clock.y, dFFREGSRL.clock) annotation(
            Line(points = {{-66, -22}, {-56, -22}, {-56, -3.6}, {-39.96, -3.6}}, color = {127, 0, 127}, smooth = Smooth.None));
          connect(set.y, dFFREGSRL.set) annotation(
            Line(points = {{-66, 84}, {-39.96, 84}, {-39.96, 59.4}}, color = {127, 0, 127}, smooth = Smooth.None));
          connect(data_0.y, dFFREGSRL.dataIn[1]) annotation(
            Line(points = {{-66, 18}, {-56, 18}, {-56, 31.68}, {-39.96, 31.68}}, color = {127, 0, 127}, smooth = Smooth.None));
          connect(data_1.y, dFFREGSRL.dataIn[2]) annotation(
            Line(points = {{-66, 50}, {-56, 50}, {-56, 36.72}, {-39.96, 36.72}}, color = {127, 0, 127}, smooth = Smooth.None));
          annotation(
            experiment(StopTime = 15),
            Documentation(info = "<html>
<p>This example is a simple test of the Registers.DFFREGSRL component. The data width is set to two. After simulation plot both the dataIn and the dataOut vectors. To verify the results compare the the truth table which is documented in the DFFREGSRL component.</p>
</html>"));
        end DFFREGSRL;

        model DLATREG "Level sensitive D-Register-Bank, high active reset"
          import L = Modelica.Electrical.Digital.Interfaces.Logic;
          extends Modelica.Icons.Example;
          Modelica.Electrical.Digital.Sources.Table enable(x = {L.'0', L.'1', L.'0'}, t = {0, 10, 18}) annotation(
            Placement(transformation(extent = {{-86, -32}, {-66, -12}})));
          Modelica.Electrical.Digital.Sources.Table data_0(x = {L.'W', L.'1'}, t = {0, 15}) annotation(
            Placement(transformation(extent = {{-86, 8}, {-66, 28}})));
          Modelica.Electrical.Digital.Sources.Table reset(x = {L.'0', L.'1', L.'0', L.'1', L.'0'}, t = {0, 1, 2, 20, 21}) annotation(
            Placement(transformation(extent = {{-86, -70}, {-66, -50}})));
          Modelica.Electrical.Digital.Sources.Table data_1(x = {L.'0', L.'1'}, t = {0, 16}) annotation(
            Placement(transformation(extent = {{-86, 40}, {-66, 60}})));
          Modelica.Electrical.Digital.Registers.DLATREG dLATREG(n = 2, tHL = 2, tLH = 3, delay(inertialDelaySensitive(each y(start = L.'U', fixed = true)))) annotation(
            Placement(transformation(extent = {{-45, -50}, {84, 79}})));
        equation
          connect(reset.y, dLATREG.reset) annotation(
            Line(points = {{-66, -60}, {-39.84, -60}, {-39.84, -24.2}}, color = {127, 0, 127}, smooth = Smooth.None));
          connect(enable.y, dLATREG.enable) annotation(
            Line(points = {{-66, -22}, {-56, -22}, {-56, 1.6}, {-39.84, 1.6}}, color = {127, 0, 127}, smooth = Smooth.None));
          connect(data_0.y, dLATREG.dataIn[1]) annotation(
            Line(points = {{-66, 18}, {-56, 18}, {-56, 37.72}, {-39.84, 37.72}}, color = {127, 0, 127}, smooth = Smooth.None));
          connect(data_1.y, dLATREG.dataIn[2]) annotation(
            Line(points = {{-66, 50}, {-56, 50}, {-56, 42.88}, {-39.84, 42.88}}, color = {127, 0, 127}, smooth = Smooth.None));
          annotation(
            experiment(StopTime = 25),
            Documentation(info = "<html>
<p>This example is a simple test of the Registers.DLATREG component. The data width is set to two. After simulation plot both the dataIn and the dataOut vectors. To verify the results compare the the truth table which is documented in the DLATREG component.</p>
</html>"));
        end DLATREG;

        model DLATREGL "Level sensitive D-Register-Bank, low active reset"
          import L = Modelica.Electrical.Digital.Interfaces.Logic;
          extends Modelica.Icons.Example;
          Modelica.Electrical.Digital.Sources.Table enable(x = {L.'0', L.'1', L.'0'}, t = {0, 10, 18}) annotation(
            Placement(transformation(extent = {{-86, -32}, {-66, -12}})));
          Modelica.Electrical.Digital.Sources.Table data_0(x = {L.'W', L.'1'}, t = {0, 15}) annotation(
            Placement(transformation(extent = {{-86, 8}, {-66, 28}})));
          Modelica.Electrical.Digital.Sources.Table reset(t = {0, 1, 2, 20, 21}, x = {L.'1', L.'0', L.'1', L.'0', L.'1'}) annotation(
            Placement(transformation(extent = {{-86, -70}, {-66, -50}})));
          Modelica.Electrical.Digital.Sources.Table data_1(x = {L.'0', L.'1'}, t = {0, 16}) annotation(
            Placement(transformation(extent = {{-86, 40}, {-66, 60}})));
          Modelica.Electrical.Digital.Registers.DLATREGL dLATREGL(tHL = 2, tLH = 3, n = 2, delay(inertialDelaySensitive(each y(start = L.'U', fixed = true)))) annotation(
            Placement(transformation(extent = {{-45, -50}, {84, 79}})));
        equation
          connect(reset.y, dLATREGL.reset) annotation(
            Line(points = {{-66, -60}, {-39.84, -60}, {-39.84, -24.2}}, color = {127, 0, 127}, smooth = Smooth.None));
          connect(enable.y, dLATREGL.enable) annotation(
            Line(points = {{-66, -22}, {-54, -22}, {-54, 1.6}, {-39.84, 1.6}}, color = {127, 0, 127}, smooth = Smooth.None));
          connect(data_0.y, dLATREGL.dataIn[1]) annotation(
            Line(points = {{-66, 18}, {-56, 18}, {-56, 37.72}, {-39.84, 37.72}}, color = {127, 0, 127}, smooth = Smooth.None));
          connect(data_1.y, dLATREGL.dataIn[2]) annotation(
            Line(points = {{-66, 50}, {-56, 50}, {-56, 42.88}, {-39.84, 42.88}}, color = {127, 0, 127}, smooth = Smooth.None));
          annotation(
            experiment(StopTime = 25),
            Documentation(info = "<html>
<p>This example is a simple test of the Registers.DLATREGL component. The data width is set to two. After simulation plot both the dataIn and the dataOut vectors. To verify the results compare the the truth table which is documented in the DLATREGL component.</p>
</html>"));
        end DLATREGL;

        model DLATREGSRH "Level sensitive D-Register-Bank, high active set and reset"
          import L = Modelica.Electrical.Digital.Interfaces.Logic;
          extends Modelica.Icons.Example;
          Modelica.Electrical.Digital.Sources.Table enable(x = {L.'0', L.'1', L.'0'}, t = {0, 10, 18}) annotation(
            Placement(transformation(extent = {{-86, -32}, {-66, -12}})));
          Modelica.Electrical.Digital.Sources.Table data_0(x = {L.'W', L.'1'}, t = {0, 15}) annotation(
            Placement(transformation(extent = {{-86, 8}, {-66, 28}})));
          Modelica.Electrical.Digital.Sources.Table reset(x = {L.'0', L.'1', L.'0', L.'1', L.'0'}, t = {0, 1, 2, 20, 21}) annotation(
            Placement(transformation(extent = {{-86, -70}, {-66, -50}})));
          Modelica.Electrical.Digital.Sources.Table data_1(x = {L.'0', L.'1'}, t = {0, 16}) annotation(
            Placement(transformation(extent = {{-86, 40}, {-66, 60}})));
          Modelica.Electrical.Digital.Sources.Table set(x = {L.'0', L.'1', L.'0'}, t = {0, 5, 6}) annotation(
            Placement(transformation(extent = {{-86, 74}, {-66, 94}})));
          Modelica.Electrical.Digital.Registers.DLATREGSRH dLATREGSRH(tHL = 2, tLH = 3, n = 2, delay(inertialDelaySensitive(each y(start = L.'U', fixed = true)))) annotation(
            Placement(transformation(extent = {{-45, -42}, {69, 71}})));
        equation
          connect(reset.y, dLATREGSRH.reset) annotation(
            Line(points = {{-66, -60}, {-40.44, -60}, {-40.44, -19.4}}, color = {127, 0, 127}, smooth = Smooth.None));
          connect(enable.y, dLATREGSRH.enable) annotation(
            Line(points = {{-66, -22}, {-52, -22}, {-52, 3.2}, {-40.44, 3.2}}, color = {127, 0, 127}, smooth = Smooth.None));
          connect(data_0.y, dLATREGSRH.dataIn[1]) annotation(
            Line(points = {{-66, 18}, {-52, 18}, {-52, 34.84}, {-40.44, 34.84}}, color = {127, 0, 127}, smooth = Smooth.None));
          connect(data_1.y, dLATREGSRH.dataIn[2]) annotation(
            Line(points = {{-66, 50}, {-52, 50}, {-52, 39.36}, {-40.44, 39.36}}, color = {127, 0, 127}, smooth = Smooth.None));
          connect(set.y, dLATREGSRH.set) annotation(
            Line(points = {{-66, 84}, {-40.44, 84}, {-40.44, 59.7}}, color = {127, 0, 127}, smooth = Smooth.None));
          annotation(
            experiment(StopTime = 25),
            Documentation(info = "<html>
<p>This example is a simple test of the Registers.DLATREGSRH component. The data width is set to two. After simulation plot both the dataIn and the dataOut vectors. To verify the results compare the the truth table which is documented in the DLATREGSRH component.</p>
</html>"));
        end DLATREGSRH;

        model DLATREGSRL "Level sensitive D-Register-Bank, low active set and reset"
          import L = Modelica.Electrical.Digital.Interfaces.Logic;
          extends Modelica.Icons.Example;
          Modelica.Electrical.Digital.Sources.Table enable(t = {0, 10, 18}, x = {L.'0', L.'1', L.'0'}) annotation(
            Placement(transformation(extent = {{-86, -32}, {-66, -12}})));
          Modelica.Electrical.Digital.Sources.Table data_0(x = {L.'W', L.'1'}, t = {0, 15}) annotation(
            Placement(transformation(extent = {{-86, 8}, {-66, 28}})));
          Modelica.Electrical.Digital.Sources.Table reset(t = {0, 1, 2, 20, 21}, x = {L.'1', L.'0', L.'1', L.'0', L.'1'}) annotation(
            Placement(transformation(extent = {{-86, -70}, {-66, -50}})));
          Modelica.Electrical.Digital.Sources.Table data_1(x = {L.'0', L.'1'}, t = {0, 16}) annotation(
            Placement(transformation(extent = {{-86, 40}, {-66, 60}})));
          Modelica.Electrical.Digital.Sources.Table set(t = {0, 5, 6}, x = {L.'1', L.'0', L.'1'}) annotation(
            Placement(transformation(extent = {{-86, 74}, {-66, 94}})));
          Modelica.Electrical.Digital.Registers.DLATREGSRL dLATREGSRL(tHL = 2, tLH = 3, n = 2, delay(inertialDelaySensitive(each y(start = L.'U', fixed = true)))) annotation(
            Placement(transformation(extent = {{-45, -43}, {69, 71}})));
        equation
          connect(reset.y, dLATREGSRL.reset) annotation(
            Line(points = {{-66, -60}, {-40.44, -60}, {-40.44, -20.2}}, color = {127, 0, 127}, smooth = Smooth.None));
          connect(enable.y, dLATREGSRL.enable) annotation(
            Line(points = {{-66, -22}, {-56, -22}, {-56, 2.6}, {-40.44, 2.6}}, color = {127, 0, 127}, smooth = Smooth.None));
          connect(data_0.y, dLATREGSRL.dataIn[1]) annotation(
            Line(points = {{-66, 18}, {-56, 18}, {-56, 34.52}, {-40.44, 34.52}}, color = {127, 0, 127}, smooth = Smooth.None));
          connect(data_1.y, dLATREGSRL.dataIn[2]) annotation(
            Line(points = {{-66, 50}, {-56, 50}, {-56, 39.08}, {-40.44, 39.08}}, color = {127, 0, 127}, smooth = Smooth.None));
          connect(set.y, dLATREGSRL.set) annotation(
            Line(points = {{-66, 84}, {-40.44, 84}, {-40.44, 59.6}}, color = {127, 0, 127}, smooth = Smooth.None));
          annotation(
            experiment(StopTime = 25),
            Documentation(info = "<html>
<p>This example is a simple test of the Registers.DLATREGSRL component. The data width is set to two. After simulation plot both the dataIn and the dataOut vectors. To verify the results compare the the truth table which is documented in the DLATREGSRL component.</p>
</html>"));
        end DLATREGSRL;

        model NXFER "Functionality test of NXFERGATE"
          import L = Modelica.Electrical.Digital.Interfaces.Logic;
          extends Modelica.Icons.Example;
          Modelica.Electrical.Digital.Sources.Table e_table(y0 = Modelica.Electrical.Digital.Interfaces.Logic.'U', x = {L.'0', L.'1', L.'Z'}, t = {0, 5, 9}) annotation(
            Placement(transformation(extent = {{-75, 30}, {-55, 50}})));
          Modelica.Electrical.Digital.Sources.Table x_table(y0 = Modelica.Electrical.Digital.Interfaces.Logic.'U', x = {L.'1', L.'0'}, t = {1, 7}) annotation(
            Placement(transformation(extent = {{-75, -20}, {-55, 0}})));
          Modelica.Electrical.Digital.Tristates.NXFERGATE nXFERGATE(tHL = 1, tLH = 1) annotation(
            Placement(transformation(extent = {{-40, -52}, {52, 41}})));
        equation
          connect(x_table.y, nXFERGATE.x) annotation(
            Line(points = {{-55, -10}, {-45.2, -10}, {-45.2, -10.15}, {-35.4, -10.15}}, color = {127, 0, 127}, smooth = Smooth.None));
          connect(e_table.y, nXFERGATE.enable) annotation(
            Line(points = {{-55, 40}, {-35.4, 40}, {-35.4, 27.05}}, color = {127, 0, 127}, smooth = Smooth.None));
          annotation(
            experiment(StopTime = 12),
            Documentation(info = "<html>
<p>This example is a simple test of the Tristates.NXFERGATE component.</p>
</html>"));
        end NXFER;

        model NRXFER "Functionality test of NRXFERGATE"
          import L = Modelica.Electrical.Digital.Interfaces.Logic;
          extends Modelica.Icons.Example;
          Modelica.Electrical.Digital.Sources.Table e_table(y0 = Modelica.Electrical.Digital.Interfaces.Logic.'U', x = {L.'0', L.'1', L.'Z'}, t = {0, 5, 9}) annotation(
            Placement(transformation(extent = {{-75, 30}, {-55, 50}})));
          Modelica.Electrical.Digital.Sources.Table x_table(y0 = Modelica.Electrical.Digital.Interfaces.Logic.'U', x = {L.'1', L.'0'}, t = {1, 7}) annotation(
            Placement(transformation(extent = {{-75, -20}, {-55, 0}})));
          Modelica.Electrical.Digital.Tristates.NRXFERGATE nRXFERGATE(tHL = 1, tLH = 1) annotation(
            Placement(transformation(extent = {{-40, -54}, {58, 44}})));
        equation
          connect(x_table.y, nRXFERGATE.x) annotation(
            Line(points = {{-55, -10}, {-45.05, -10}, {-45.05, -9.9}, {-35.1, -9.9}}, color = {127, 0, 127}, smooth = Smooth.None));
          connect(e_table.y, nRXFERGATE.enable) annotation(
            Line(points = {{-55, 40}, {-35.1, 40}, {-35.1, 29.3}}, color = {127, 0, 127}, smooth = Smooth.None));
          annotation(
            experiment(StopTime = 12),
            Documentation(info = "<html>
<p>This example is a simple test of the Tristates.NRXFER component.  After simulation until 12 s plot x, enable, and y of the <code>nRXFERGATE </code>component. To verify the result compare to the truth table <code>NRXferTable</code>.</p>
</html>"));
        end NRXFER;

        model BUF3S "Functionality test of BUF3S"
          import L = Modelica.Electrical.Digital.Interfaces.Logic;
          extends Modelica.Icons.Example;
          Modelica.Electrical.Digital.Sources.Table e_table(y0 = Modelica.Electrical.Digital.Interfaces.Logic.'U', x = {L.'0', L.'1', L.'Z'}, t = {0, 5, 9}) annotation(
            Placement(transformation(extent = {{-75, 30}, {-55, 50}})));
          Modelica.Electrical.Digital.Sources.Table x_table(y0 = Modelica.Electrical.Digital.Interfaces.Logic.'U', x = {L.'1', L.'0'}, t = {1, 7}) annotation(
            Placement(transformation(extent = {{-75, -20}, {-55, 0}})));
          Modelica.Electrical.Digital.Tristates.BUF3S bUF3S(tHL = 1, tLH = 1, strength = Modelica.Electrical.Digital.Interfaces.Strength.'S_X01') annotation(
            Placement(transformation(extent = {{-40, -50}, {48, 38}})));
        equation
          connect(x_table.y, bUF3S.x) annotation(
            Line(points = {{-55, -10}, {-35.6, -10.4}}, color = {127, 0, 127}, smooth = Smooth.None));
          connect(e_table.y, bUF3S.enable) annotation(
            Line(points = {{-55, 40}, {-35.6, 40}, {-35.6, 24.8}}, color = {127, 0, 127}, smooth = Smooth.None));
          annotation(
            experiment(StopTime = 12),
            Documentation(info = "<html>
<p>This example is a simple test of the Tristates.BUF3S component.  After simulation until 12 s plot x, enable, and y of the <code>bUF3S</code> component. To verify the result compare to the truth table Buf3sTable.</p>
</html>"));
        end BUF3S;

        model INV3S "Functionality test of INV3S"
          import L = Modelica.Electrical.Digital.Interfaces.Logic;
          extends Modelica.Icons.Example;
          Modelica.Electrical.Digital.Sources.Table e_table(y0 = L.'U', x = {L.'0', L.'1', L.'Z'}, t = {0, 5, 9}) annotation(
            Placement(transformation(extent = {{-75, 30}, {-55, 50}})));
          Modelica.Electrical.Digital.Sources.Table x_table(y0 = L.'U', x = {L.'1', L.'0'}, t = {1, 7}) annotation(
            Placement(transformation(extent = {{-75, -20}, {-55, 0}})));
          Modelica.Electrical.Digital.Tristates.INV3S iNV3S annotation(
            Placement(transformation(extent = {{-34, -44}, {42, 32}})));
        equation
          connect(x_table.y, iNV3S.x) annotation(
            Line(points = {{-55, -10}, {-42.6, -10}, {-42.6, -9.8}, {-30.2, -9.8}}, color = {127, 0, 127}, smooth = Smooth.None));
          connect(e_table.y, iNV3S.enable) annotation(
            Line(points = {{-55, 40}, {-30.2, 40}, {-30.2, 20.6}}, color = {127, 0, 127}, smooth = Smooth.None));
          annotation(
            experiment(StopTime = 12),
            Documentation(info = "<html>
<p>This example is a simple test of the Tristates.INV3S component. After simulation until 12 s plot x, enable, and y of the <code>iNV3S</code> component. To verify the result compare to the truth table <code>T.UX01Table</code>.</p>
</html>"));
        end INV3S;

        model WiredX "Functionality test of WiredX"
          import L = Modelica.Electrical.Digital.Interfaces.Logic;
          extends Modelica.Icons.Example;
          Modelica.Electrical.Digital.Sources.Table e_table2(y0 = L.'U', x = {L.'0', L.'1', L.'0'}, t = {0, 3, 9}) annotation(
            Placement(transformation(extent = {{-85, 68}, {-65, 88}})));
          Modelica.Electrical.Digital.Sources.Table x_table2(y0 = L.'U', x = {L.'1', L.'0'}, t = {1, 7}) annotation(
            Placement(transformation(extent = {{-85, 18}, {-65, 38}})));
          Modelica.Electrical.Digital.Tristates.BUF3S bUF3S2(tHL = 1, tLH = 1, strength = Modelica.Electrical.Digital.Interfaces.Strength.'S_X01') annotation(
            Placement(transformation(extent = {{-48, -1}, {16, 63}})));
          Modelica.Electrical.Digital.Sources.Table e_table1(y0 = L.'U', t = {0, 3, 9}, x = {L.'0', L.'1', L.'0'}) annotation(
            Placement(transformation(extent = {{-85, -24}, {-65, -4}})));
          Modelica.Electrical.Digital.Sources.Table x_table1(y0 = L.'U', x = {L.'0', L.'1', L.'0'}, t = {1, 5, 7}) annotation(
            Placement(transformation(extent = {{-85, -74}, {-65, -54}})));
          Modelica.Electrical.Digital.Tristates.BUF3S bUF3S1(tHL = 1, tLH = 1, strength = Modelica.Electrical.Digital.Interfaces.Strength.'S_X01') annotation(
            Placement(transformation(extent = {{-52, -94}, {14, -28}})));
          Modelica.Electrical.Digital.Tristates.WiredX wiredX(n = 2) annotation(
            Placement(transformation(extent = {{26, -38}, {80, 16}})));
        equation
          connect(x_table2.y, bUF3S2.x) annotation(
            Line(points = {{-65, 28}, {-44.8, 27.8}}, color = {127, 0, 127}, smooth = Smooth.None));
          connect(e_table2.y, bUF3S2.enable) annotation(
            Line(points = {{-65, 78}, {-44.8, 78}, {-44.8, 53.4}}, color = {127, 0, 127}, smooth = Smooth.None));
          connect(x_table1.y, bUF3S1.x) annotation(
            Line(points = {{-65, -64}, {-48.7, -64.3}}, color = {127, 0, 127}, smooth = Smooth.None));
          connect(e_table1.y, bUF3S1.enable) annotation(
            Line(points = {{-65, -14}, {-48.7, -14}, {-48.7, -37.9}}, color = {127, 0, 127}, smooth = Smooth.None));
          connect(bUF3S1.y, wiredX.x[1]) annotation(
            Line(points = {{10.7, -64.3}, {24, -64.3}, {24, -21.8}, {36.8, -21.8}}, color = {127, 0, 127}, smooth = Smooth.None));
          connect(bUF3S2.y, wiredX.x[2]) annotation(
            Line(points = {{12.8, 27.8}, {26, 27.8}, {26, -0.2}, {36.8, -0.2}}, color = {127, 0, 127}, smooth = Smooth.None));
          annotation(
            experiment(StopTime = 12),
            Documentation(info = "<html>
<p>This example is a simple test of the Tristates.WiredX component. The input width is set to two. After simulation until 12 s plot x[1], x[2], and y of the WiredX component. To verify the result compare to the truth table  Tables.ResolutionTable.</p>
</html>"));
        end WiredX;

        model MUX2x1 "Simple Multiplexer test"
          import D = Modelica.Electrical.Digital;
          import L = Modelica.Electrical.Digital.Interfaces.Logic;
          extends Modelica.Icons.Example;
          D.Multiplexers.MUX2x1 Mux2x1 annotation(
            Placement(transformation(extent = {{-34, -48}, {56, 48}})));
          D.Sources.Table Input1(y0 = L.'U', x = {L.'X', L.'0', L.'1', L.'0', L.'X', L.'U'}, t = {2, 4, 6, 8, 10, 12}) annotation(
            Placement(transformation(extent = {{-90, 14}, {-70, 34}})));
          D.Sources.Step Select(before = L.'0', after = L.'1', stepTime = 7) annotation(
            Placement(transformation(extent = {{-90, 58}, {-70, 78}})));
          D.Sources.Table Input0(y0 = L.'U', t = {2, 4, 6, 8, 10, 12}, x = {L.'1', L.'X', L.'0', L.'X', L.'1', L.'U'}) annotation(
            Placement(transformation(extent = {{-90, -34}, {-70, -14}})));
        equation
          connect(Select.y, Mux2x1.sel) annotation(
            Line(points = {{-70, 68}, {11, 68}, {11, 43.2}}, color = {127, 0, 127}, smooth = Smooth.None));
          connect(Input0.y, Mux2x1.in0) annotation(
            Line(points = {{-70, -24}, {-29.5, -24}}, color = {127, 0, 127}, smooth = Smooth.None));
          connect(Input1.y, Mux2x1.in1) annotation(
            Line(points = {{-70, 24}, {-29.5, 24}}, color = {127, 0, 127}, smooth = Smooth.None));
          annotation(
            experiment(StopTime = 15),
            Documentation(info = "<html>
<p>This example is a simple test of a single multiplexer component with 2 inputs prescribed by sources, one select input, and one output. After simulation until 15 s plot Mux2x1.in0, Mux2x1.in1, Mux2x1.sel, and Mux2x1.out. Compare the output signal with the input signals. If the select signal changes, the output switches to the other input.</p>
</html>"));
        end MUX2x1;

        model RAM "Simple RAM test example"
          import L = Modelica.Electrical.Digital.Interfaces.Logic;
          extends Modelica.Icons.Example;
          Modelica.Electrical.Digital.Memories.DLATRAM dLATRAM annotation(
            Placement(transformation(extent = {{-11, -41}, {103, 73}})));
          Modelica.Electrical.Digital.Sources.Table addr_1(y0 = L.'U', x = {L.'X', L.'0', L.'1', L.'Z', L.'W', L.'L', L.'H', L.'-', L.'U', L.'X', L.'0', L.'1', L.'Z', L.'W', L.'L', L.'H', L.'-', L.'U', L.'X', L.'0', L.'1', L.'Z', L.'W', L.'L', L.'H', L.'-', L.'U', L.'X', L.'0', L.'1', L.'Z', L.'W', L.'L', L.'H', L.'-', L.'U', L.'X', L.'0', L.'1', L.'Z', L.'W', L.'L', L.'H', L.'-', L.'U', L.'X', L.'0', L.'1', L.'Z', L.'W', L.'L', L.'H', L.'-', L.'U', L.'X', L.'0', L.'1', L.'Z', L.'W', L.'L', L.'H', L.'-', L.'U', L.'X', L.'0', L.'1', L.'Z', L.'W', L.'L', L.'H', L.'-', L.'U', L.'X', L.'0', L.'1', L.'Z', L.'W', L.'L', L.'H', L.'-'}, t = {5, 10, 15, 20, 25, 30, 35, 40, 45, 50, 55, 60, 65, 70, 75, 80, 85, 90, 95, 100, 105, 110, 115, 120, 125, 130, 135, 140, 145, 150, 155, 160, 165, 170, 175, 180, 185, 190, 195, 200, 205, 210, 215, 220, 225, 230, 235, 240, 245, 250, 255, 260, 265, 270, 275, 280, 285, 290, 295, 300, 305, 310, 315, 320, 325, 330, 335, 340, 345, 350, 355, 360, 365, 370, 375, 380, 385, 390, 395, 400}) annotation(
            Placement(transformation(extent = {{-78, 76}, {-58, 96}})));
          Modelica.Electrical.Digital.Sources.Set data_1(x = Modelica.Electrical.Digital.Interfaces.Logic.'0') annotation(
            Placement(transformation(extent = {{-78, 20}, {-58, 40}})));
          Modelica.Electrical.Digital.Sources.Set data_0(x = Modelica.Electrical.Digital.Interfaces.Logic.'0') annotation(
            Placement(transformation(extent = {{-78, -8}, {-58, 12}})));
          Modelica.Electrical.Digital.Sources.Set WE(x = Modelica.Electrical.Digital.Interfaces.Logic.'1') annotation(
            Placement(transformation(extent = {{-78, -70}, {-58, -50}})));
          Modelica.Electrical.Digital.Sources.Table addr_0(y0 = Modelica.Electrical.Digital.Interfaces.Logic.'U', t = {45, 90, 135, 180, 225, 270, 315, 360}, x = {L.'X', L.'0', L.'1', L.'Z', L.'W', L.'L', L.'H', L.'-'}) annotation(
            Placement(transformation(extent = {{-78, 48}, {-58, 68}})));
          Modelica.Electrical.Digital.Sources.Set RE(x = Modelica.Electrical.Digital.Interfaces.Logic.'1') annotation(
            Placement(transformation(extent = {{-78, -38}, {-58, -18}})));
        equation
          connect(RE.y, dLATRAM.RE) annotation(
            Line(points = {{-58, -28}, {-28, -28}, {-28, 4.6}, {-6.44, 4.6}}, color = {127, 0, 127}, smooth = Smooth.None));
          connect(WE.y, dLATRAM.WE) annotation(
            Line(points = {{-58, -60}, {-24, -60}, {-24, -12.5}, {-6.44, -12.5}}, color = {127, 0, 127}, smooth = Smooth.None));
          connect(addr_0.y, dLATRAM.addr[1]) annotation(
            Line(points = {{-58, 58}, {-34, 58}, {-34, 47.35}, {-5.3, 47.35}}, color = {127, 0, 127}, smooth = Smooth.None));
          connect(addr_1.y, dLATRAM.addr[2]) annotation(
            Line(points = {{-58, 86}, {-32, 86}, {-32, 53.05}, {-5.3, 53.05}}, color = {127, 0, 127}, smooth = Smooth.None));
          connect(data_1.y, dLATRAM.dataIn[2]) annotation(
            Line(points = {{-58, 30}, {-32, 30}, {-32, 30.25}, {-5.3, 30.25}}, color = {127, 0, 127}, smooth = Smooth.None));
          connect(data_0.y, dLATRAM.dataIn[1]) annotation(
            Line(points = {{-58, 2}, {-34, 2}, {-34, 24.55}, {-5.3, 24.55}}, color = {127, 0, 127}, smooth = Smooth.None));
          annotation(
            experiment(StopTime = 400),
            Documentation(info = "<html>
<p>This example is a simple and uncomplete test of a single DLATRAM component . After simulation until 400 s plot dLATRAM.addr[1], dLATRAM.addr[2], and dLATRAM.dataOUT[1], dLATRAM.dataOut[2]. The address inputs are  prescribed with all possible combinations of logic values. It can be checked in which cases of address values the output is 'X' or '0'. </p>
</html>"));
        end RAM;

        package Utilities "Utility components used by package Examples"
          extends Modelica.Icons.UtilitiesPackage;

          model MUX4 "4 to 1 Bit Multiplexer"
            import D = Modelica.Electrical.Digital;
            import L = Modelica.Electrical.Digital.Interfaces.Logic;
            parameter Modelica.SIunits.Time delayTime = 0.001 "Delay time";
            parameter D.Interfaces.Logic q0 = L.'0' "Initial value";
            D.Interfaces.DigitalInput d0 annotation(
              Placement(transformation(extent = {{-110, 60}, {-90, 80}}, rotation = 0)));
            D.Interfaces.DigitalInput d1 annotation(
              Placement(transformation(extent = {{-110, 34}, {-90, 54}}, rotation = 0)));
            D.Interfaces.DigitalInput d2 annotation(
              Placement(transformation(extent = {{-110, 8}, {-90, 28}}, rotation = 0)));
            D.Interfaces.DigitalInput d3 annotation(
              Placement(transformation(extent = {{-110, -18}, {-90, 2}}, rotation = 0)));
            D.Interfaces.DigitalInput a0 annotation(
              Placement(transformation(extent = {{-110, -54}, {-90, -34}}, rotation = 0)));
            D.Interfaces.DigitalInput a1 annotation(
              Placement(transformation(extent = {{-110, -80}, {-90, -60}}, rotation = 0)));
            D.Interfaces.DigitalOutput d annotation(
              Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
            D.Basic.Or Or1(n = 4) annotation(
              Placement(transformation(extent = {{50, 20}, {70, 40}}, rotation = 0)));
            D.Basic.And And1(n = 3) annotation(
              Placement(transformation(extent = {{-20, 60}, {0, 80}}, rotation = 0)));
            D.Basic.And And2(n = 3) annotation(
              Placement(transformation(extent = {{-20, 34}, {0, 54}}, rotation = 0)));
            D.Basic.And And3(n = 3) annotation(
              Placement(transformation(extent = {{-20, 8}, {0, 28}}, rotation = 0)));
            D.Basic.And And4(n = 3) annotation(
              Placement(transformation(extent = {{-20, -18}, {0, 2}}, rotation = 0)));
            D.Basic.Not Not1 annotation(
              Placement(transformation(extent = {{-76, -54}, {-56, -34}}, rotation = 0)));
            D.Basic.Not Not2 annotation(
              Placement(transformation(extent = {{-76, -80}, {-56, -60}}, rotation = 0)));
          equation
            connect(a0, Not1.x) annotation(
              Line(points = {{-100, -44}, {-72, -44}}, color = {127, 0, 127}));
            connect(a1, Not2.x) annotation(
              Line(points = {{-100, -70}, {-72, -70}}, color = {127, 0, 127}));
            connect(d0, And1.x[2]) annotation(
              Line(points = {{-100, 70}, {-58, 70}, {-58, 70}, {-16, 70}}, color = {127, 0, 127}));
            connect(d1, And2.x[2]) annotation(
              Line(points = {{-100, 44}, {-16, 44}}, color = {127, 0, 127}));
            connect(d2, And3.x[2]) annotation(
              Line(points = {{-100, 18}, {-16, 18}}, color = {127, 0, 127}));
            connect(d3, And4.x[2]) annotation(
              Line(points = {{-100, -8}, {-58, -8}, {-58, -8}, {-16, -8}}, color = {127, 0, 127}));
            connect(And4.y, Or1.x[1]) annotation(
              Line(points = {{0, -8}, {40, -8}, {40, 24}, {54, 24}}, color = {127, 0, 127}));
            connect(And3.y, Or1.x[2]) annotation(
              Line(points = {{0, 18}, {20, 18}, {20, 28}, {54, 28}}, color = {127, 0, 127}));
            connect(And2.y, Or1.x[3]) annotation(
              Line(points = {{0, 44}, {20, 44}, {20, 32}, {54, 32}}, color = {127, 0, 127}));
            connect(And1.y, Or1.x[4]) annotation(
              Line(points = {{0, 70}, {40, 70}, {40, 36}, {54, 36}}, color = {127, 0, 127}));
            connect(Or1.y, d) annotation(
              Line(points = {{70, 30}, {80, 30}, {80, 0}, {100, 0}}, color = {127, 0, 127}));
            connect(Not1.y, And1.x[3]) annotation(
              Line(points = {{-56, -44}, {-50, -44}, {-50, 75.3333}, {-16, 75.3333}}, color = {127, 0, 127}));
            connect(Not1.y, And3.x[3]) annotation(
              Line(points = {{-56, -44}, {-50, -44}, {-50, 23.3333}, {-16, 23.3333}}, color = {127, 0, 127}));
            connect(Not2.y, And1.x[1]) annotation(
              Line(points = {{-56, -70}, {-40, -70}, {-40, 64.6667}, {-16, 64.6667}}, color = {127, 0, 127}));
            connect(Not2.y, And2.x[1]) annotation(
              Line(points = {{-56, -70}, {-40, -70}, {-40, 38.6667}, {-16, 38.6667}}, color = {127, 0, 127}));
            connect(a0, And4.x[3]) annotation(
              Line(points = {{-100, -44}, {-80, -44}, {-80, -2.66667}, {-16, -2.66667}}, color = {127, 0, 127}));
            connect(a0, And2.x[3]) annotation(
              Line(points = {{-100, -44}, {-80, -44}, {-80, 49.3333}, {-16, 49.3333}}, color = {127, 0, 127}));
            connect(a1, And4.x[1]) annotation(
              Line(points = {{-100, -70}, {-80, -70}, {-80, -90}, {-30, -90}, {-30, -13.3333}, {-16, -13.3333}}, color = {127, 0, 127}));
            connect(a1, And3.x[1]) annotation(
              Line(points = {{-100, -70}, {-80, -70}, {-80, -90}, {-30, -90}, {-30, 12.6667}, {-16, 12.6667}}, color = {127, 0, 127}));
            annotation(
              Documentation(info = "<html>
<p>MUX4 is a four bit multiplexer which is built up by And, Not, and Or gates according to the schematic.</p>
<p>The parameters delayTime and q0 are prepared but<b> not</b> yet used in the component. The MUX4 component uses standard values in its components.</p>
</html>"),
              Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-90, 100}, {90, -100}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 170}, fillPattern = FillPattern.Solid), Line(points = {{-60, 100}, {-60, -100}}, color = {0, 0, 0}, thickness = 0.5), Line(points = {{60, 100}, {60, -100}}, color = {0, 0, 0}, thickness = 0.5), Text(extent = {{-86, 80}, {-64, 60}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, textString = "D0"), Text(extent = {{64, 12}, {86, -8}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, textString = "D"), Text(extent = {{-150, -100}, {150, -160}}, lineColor = {0, 0, 255}, textString = "%name"), Text(extent = {{-60, 100}, {60, 40}}, lineColor = {0, 0, 0}, textString = "MUX"), Text(extent = {{-86, -60}, {-64, -80}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, textString = "A1"), Text(extent = {{-86, 54}, {-64, 34}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, textString = "D1"), Text(extent = {{-86, 28}, {-64, 8}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, textString = "D2"), Text(extent = {{-86, 2}, {-64, -18}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, textString = "D3"), Text(extent = {{-86, -36}, {-64, -56}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, textString = "A0")}));
          end MUX4;

          model RS "Unclocked RS FlipFlop"
            import D = Modelica.Electrical.Digital;
            import L = Modelica.Electrical.Digital.Interfaces.Logic;
            parameter Modelica.SIunits.Time delayTime = 0 "Delay time";
            parameter D.Interfaces.Logic q0 = L.'U' "Initial value of output";
            D.Basic.Nor Nor1 annotation(
              Placement(transformation(extent = {{-40, 42}, {0, 82}}, rotation = 0)));
            D.Basic.Nor Nor2 annotation(
              Placement(transformation(extent = {{-40, -82}, {0, -42}}, rotation = 0)));
            D.Interfaces.DigitalInput s annotation(
              Placement(transformation(extent = {{-110, 60}, {-90, 80}}, rotation = 0)));
            D.Interfaces.DigitalInput r annotation(
              Placement(transformation(extent = {{-110, -80}, {-90, -60}}, rotation = 0)));
            D.Interfaces.DigitalOutput q annotation(
              Placement(transformation(extent = {{90, 60}, {110, 80}}, rotation = 0)));
            D.Interfaces.DigitalOutput qn annotation(
              Placement(transformation(extent = {{90, -80}, {110, -60}}, rotation = 0)));
            D.Delay.TransportDelay TD1(delayTime = delayTime, y0 = q0) annotation(
              Placement(transformation(extent = {{-60, -64}, {-40, -44}}, rotation = 0)));
          equation
            connect(s, Nor1.x[2]) annotation(
              Line(points = {{-100, 70}, {-32, 70}}, color = {127, 0, 127}));
            connect(r, Nor2.x[1]) annotation(
              Line(points = {{-100, -70}, {-32, -70}}, color = {127, 0, 127}));
            connect(Nor2.y, Nor1.x[1]) annotation(
              Line(points = {{0, -62}, {20, -62}, {20, -20}, {-70, 20}, {-70, 54}, {-32, 54}}, color = {127, 0, 127}));
            connect(Nor1.y, qn) annotation(
              Line(points = {{0, 62}, {50, 62}, {50, -70}, {100, -70}}, color = {127, 0, 127}));
            connect(Nor2.y, q) annotation(
              Line(points = {{0, -62}, {70, -62}, {70, 70}, {100, 70}}, color = {127, 0, 127}));
            connect(TD1.y, Nor2.x[2]) annotation(
              Line(points = {{-40, -54}, {-32, -54}}, color = {127, 0, 127}));
            connect(TD1.x, Nor1.y) annotation(
              Line(points = {{-56, -54}, {-70, -54}, {-70, -20}, {20, 20}, {20, 62}, {0, 62}}, color = {127, 0, 127}));
            annotation(
              Documentation(info = "<html>
<p>RS is a basic component for e.g., the RS (set-reset) flipflop, which is built up by Nor gates according to the schematic. To avoid a numerical loop a small transport delay is inserted which delay time is a parameter of the RS component. Also its initial value can be set by parameter.</p>
</html>"),
              Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-90, 100}, {90, -100}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 170}, fillPattern = FillPattern.Solid), Text(extent = {{-100, 100}, {100, 40}}, lineColor = {0, 0, 0}, textString = "RS"), Line(points = {{-60, 100}, {-60, -100}}, color = {0, 0, 0}, thickness = 0.5), Line(points = {{60, 100}, {60, -100}}, color = {0, 0, 0}, thickness = 0.5), Text(extent = {{-86, -60}, {-64, -80}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, textString = "R"), Text(extent = {{-86, 80}, {-64, 60}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, textString = "S"), Text(extent = {{64, 80}, {86, 60}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, textString = "Q"), Text(extent = {{64, -60}, {86, -80}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, textString = "QN"), Text(extent = {{-150, -100}, {150, -160}}, lineColor = {0, 0, 255}, textString = "%name")}));
          end RS;

          model RSFF "Unclocked RS FlipFlop"
            import D = Modelica.Electrical.Digital;
            import L = Modelica.Electrical.Digital.Interfaces.Logic;
            parameter Modelica.SIunits.Time delayTime = 0.01 "Delay time";
            parameter D.Interfaces.Logic q0 = L.'U' "Initial value";
            D.Interfaces.DigitalInput s annotation(
              Placement(transformation(extent = {{-110, 60}, {-90, 80}}, rotation = 0)));
            D.Interfaces.DigitalInput r annotation(
              Placement(transformation(extent = {{-110, -80}, {-90, -60}}, rotation = 0)));
            D.Interfaces.DigitalOutput q annotation(
              Placement(transformation(extent = {{90, 60}, {110, 80}}, rotation = 0)));
            D.Interfaces.DigitalOutput qn "not Q" annotation(
              Placement(transformation(extent = {{90, -80}, {110, -60}}, rotation = 0)));
            D.Interfaces.DigitalInput clk annotation(
              Placement(transformation(extent = {{-110, -10}, {-90, 10}}, rotation = 0)));
            D.Examples.Utilities.RS RS1(delayTime = delayTime, q0 = q0) annotation(
              Placement(transformation(extent = {{-10, -40}, {70, 40}}, rotation = 0)));
            D.Basic.And And1 annotation(
              Placement(transformation(extent = {{-70, 8}, {-30, 48}}, rotation = 0)));
            D.Basic.And And2 annotation(
              Placement(transformation(extent = {{-70, -48}, {-30, -8}}, rotation = 0)));
          equation
            connect(And2.y, RS1.r) annotation(
              Line(points = {{-30, -28}, {-10, -28}}, color = {127, 0, 127}));
            connect(And1.y, RS1.s) annotation(
              Line(points = {{-30, 28}, {-10, 28}}, color = {127, 0, 127}));
            connect(s, And1.x[2]) annotation(
              Line(points = {{-100, 70}, {-70, 70}, {-70, 36}, {-62, 36}}, color = {127, 0, 127}));
            connect(clk, And1.x[1]) annotation(
              Line(points = {{-100, 0}, {-70, 0}, {-70, 20}, {-62, 20}}, color = {127, 0, 127}));
            connect(clk, And2.x[2]) annotation(
              Line(points = {{-100, 0}, {-70, 0}, {-70, -20}, {-62, -20}}, color = {127, 0, 127}));
            connect(r, And2.x[1]) annotation(
              Line(points = {{-100, -70}, {-70, -70}, {-70, -36}, {-62, -36}}, color = {127, 0, 127}));
            connect(RS1.q, q) annotation(
              Line(points = {{70, 28}, {80, 28}, {80, 70}, {100, 70}}, color = {127, 0, 127}));
            connect(RS1.qn, qn) annotation(
              Line(points = {{70, -28}, {80, -28}, {80, -70}, {100, -70}}, color = {127, 0, 127}));
            annotation(
              Documentation(info = "<html>
<p>Basing on the RS component RSFF is a RS (set-reset) flipflop composed according the schematic. Its parameter delayTime is the delay time of the RS component transport delay, q0 is the initial value of that delay.</p>
</html>"),
              Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-90, 100}, {90, -100}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 170}, fillPattern = FillPattern.Solid), Text(extent = {{-100, 100}, {100, 40}}, lineColor = {0, 0, 0}, textString = "RS"), Line(points = {{-60, 100}, {-60, -100}}, color = {0, 0, 0}, thickness = 0.5), Line(points = {{60, 100}, {60, -100}}, color = {0, 0, 0}, thickness = 0.5), Text(extent = {{-86, -60}, {-64, -80}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, textString = "R"), Text(extent = {{-86, 80}, {-64, 60}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, textString = "S"), Text(extent = {{64, 80}, {86, 60}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, textString = "Q"), Text(extent = {{64, -60}, {86, -80}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, textString = "QN"), Line(points = {{-90, 20}, {-60, 0}, {-90, -20}}, color = {0, 0, 0}), Text(extent = {{-150, -100}, {150, -160}}, lineColor = {0, 0, 255}, textString = "%name")}));
          end RSFF;

          model DFF "D FlipFlop"
            import D = Modelica.Electrical.Digital;
            import L = Modelica.Electrical.Digital.Interfaces.Logic;
            parameter Modelica.SIunits.Time Tdel = 0.01 "Delay time";
            parameter L QInit = L.'U' "Initial value";
            D.Interfaces.DigitalInput d annotation(
              Placement(transformation(extent = {{-110, 60}, {-90, 80}}, rotation = 0)));
            D.Interfaces.DigitalOutput q annotation(
              Placement(transformation(extent = {{90, 60}, {110, 80}}, rotation = 0)));
            D.Interfaces.DigitalOutput qn "not Q" annotation(
              Placement(transformation(extent = {{90, -80}, {110, -60}}, rotation = 0)));
            D.Interfaces.DigitalInput clk annotation(
              Placement(transformation(extent = {{-110, -10}, {-90, 10}}, rotation = 0)));
            D.Examples.Utilities.RSFF RSFF1 annotation(
              Placement(transformation(extent = {{-10, -40}, {70, 40}}, rotation = 0)));
            D.Basic.Not Not1 annotation(
              Placement(transformation(extent = {{-70, -48}, {-30, -8}}, rotation = 0)));
          equation
            connect(RSFF1.q, q) annotation(
              Line(points = {{70, 28}, {80, 28}, {80, 70}, {100, 70}}, color = {127, 0, 127}));
            connect(RSFF1.qn, qn) annotation(
              Line(points = {{70, -28}, {80, -28}, {80, -70}, {100, -70}}, color = {127, 0, 127}));
            connect(Not1.y, RSFF1.r) annotation(
              Line(points = {{-30, -28}, {-22, -28}, {-10, -28}}, color = {127, 0, 127}));
            connect(d, Not1.x) annotation(
              Line(points = {{-100, 70}, {-80, 70}, {-80, -28}, {-62, -28}}, color = {127, 0, 127}));
            connect(d, RSFF1.s) annotation(
              Line(points = {{-100, 70}, {-80, 70}, {-80, 28}, {-10, 28}}, color = {127, 0, 127}));
            connect(clk, RSFF1.clk) annotation(
              Line(points = {{-100, 0}, {-10, 0}}, color = {127, 0, 127}, smooth = Smooth.None));
            annotation(
              Documentation(info = "<html>
<p>Basing on the RS component DFF is a D  flipflop composed according the schematic. Its parameter delayTime is the delay time of the RS component transport delay, q0 is the initial value of that delay.</p>
</html>"),
              Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-90, 100}, {90, -100}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 170}, fillPattern = FillPattern.Solid), Line(points = {{-60, 100}, {-60, -100}}, color = {0, 0, 0}, thickness = 0.5), Line(points = {{60, 100}, {60, -100}}, color = {0, 0, 0}, thickness = 0.5), Text(extent = {{-86, 80}, {-64, 60}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, textString = "D"), Text(extent = {{64, 80}, {86, 60}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, textString = "Q"), Text(extent = {{64, -60}, {86, -80}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, textString = "QN"), Line(points = {{-90, 20}, {-60, 0}, {-90, -20}}, color = {0, 0, 0}), Text(extent = {{-150, -100}, {150, -160}}, lineColor = {0, 0, 255}, textString = "%name"), Text(extent = {{-100, 100}, {100, 40}}, lineColor = {0, 0, 0}, textString = "D")}));
          end DFF;

          model JKFF "JK FlipFlop"
            import D = Modelica.Electrical.Digital;
            import L = Modelica.Electrical.Digital.Interfaces.Logic;
            parameter Modelica.SIunits.Time delayTime = 0.001 "Delay time";
            parameter D.Interfaces.Logic q0 = L.'0' "Initial value";
            D.Interfaces.DigitalInput j annotation(
              Placement(transformation(extent = {{-110, 60}, {-90, 80}}, rotation = 0)));
            D.Interfaces.DigitalOutput q annotation(
              Placement(transformation(extent = {{90, 60}, {110, 80}}, rotation = 0)));
            D.Interfaces.DigitalOutput qn "not Q" annotation(
              Placement(transformation(extent = {{90, -80}, {110, -60}}, rotation = 0)));
            D.Interfaces.DigitalInput clk annotation(
              Placement(transformation(extent = {{-110, -10}, {-90, 10}}, rotation = 0)));
            D.Interfaces.DigitalInput k annotation(
              Placement(transformation(extent = {{-110, -80}, {-90, -60}}, rotation = 0)));
            D.Examples.Utilities.RS RS1(delayTime = delayTime, q0 = q0) annotation(
              Placement(transformation(extent = {{30, -24}, {70, 16}}, rotation = 0)));
            D.Examples.Utilities.RS RS2(delayTime = delayTime, q0 = q0) annotation(
              Placement(transformation(extent = {{-44, -20}, {-4, 20}}, rotation = 0)));
            D.Basic.And And1(n = 3) annotation(
              Placement(transformation(extent = {{-70, 4}, {-50, 24}}, rotation = 0)));
            D.Basic.And And2(n = 3) annotation(
              Placement(transformation(extent = {{-70, -24}, {-50, -4}}, rotation = 0)));
            D.Basic.And And3 annotation(
              Placement(transformation(extent = {{4, 0}, {24, 20}}, rotation = 0)));
            D.Basic.And And4 annotation(
              Placement(transformation(extent = {{4, -28}, {24, -8}}, rotation = 0)));
            D.Basic.Not Not1 annotation(
              Placement(transformation(extent = {{-34, -66}, {-14, -46}}, rotation = 0)));
          equation
            connect(And2.y, RS2.r) annotation(
              Line(points = {{-50, -14}, {-44, -14}}, color = {127, 0, 127}));
            connect(And1.y, RS2.s) annotation(
              Line(points = {{-50, 14}, {-44, 14}}, color = {127, 0, 127}));
            connect(clk, And2.x[3]) annotation(
              Line(points = {{-100, 0}, {-74, 0}, {-74, -8.66667}, {-66, -8.66667}}, color = {127, 0, 127}));
            connect(clk, And1.x[1]) annotation(
              Line(points = {{-100, 0}, {-74, 0}, {-74, 8.66667}, {-66, 8.66667}}, color = {127, 0, 127}));
            connect(k, And2.x[2]) annotation(
              Line(points = {{-100, -70}, {-74, -70}, {-74, -14}, {-66, -14}}, color = {127, 0, 127}));
            connect(And4.y, RS1.r) annotation(
              Line(points = {{24, -18}, {30, -18}}, color = {127, 0, 127}));
            connect(And3.y, RS1.s) annotation(
              Line(points = {{24, 10}, {30, 10}}, color = {127, 0, 127}));
            connect(RS2.qn, And4.x[2]) annotation(
              Line(points = {{-4, -14}, {8, -14}}, color = {127, 0, 127}));
            connect(RS2.q, And3.x[2]) annotation(
              Line(points = {{-4, 14}, {8, 14}}, color = {127, 0, 127}));
            connect(clk, Not1.x) annotation(
              Line(points = {{-100, 0}, {-80, 0}, {-80, -56}, {-30, -56}}, color = {127, 0, 127}));
            connect(Not1.y, And3.x[1]) annotation(
              Line(points = {{-14, -56}, {2, -56}, {2, 6}, {8, 6}}, color = {127, 0, 127}));
            connect(Not1.y, And4.x[1]) annotation(
              Line(points = {{-14, -56}, {2, -56}, {2, -22}, {8, -22}}, color = {127, 0, 127}));
            connect(j, And1.x[2]) annotation(
              Line(points = {{-100, 70}, {-74, 70}, {-74, 14}, {-66, 14}}, color = {127, 0, 127}));
            connect(RS1.q, And2.x[1]) annotation(
              Line(points = {{70, 10}, {80, 10}, {80, -36}, {-70, -36}, {-70, -19.3333}, {-66, -19.3333}}, color = {127, 0, 127}));
            connect(RS1.qn, And1.x[3]) annotation(
              Line(points = {{70, -18}, {86, -18}, {86, 36}, {-70, 36}, {-70, 19.3333}, {-66, 19.3333}}, color = {127, 0, 127}));
            connect(RS1.qn, q) annotation(
              Line(points = {{70, -18}, {86, -18}, {86, 70}, {100, 70}}, color = {127, 0, 127}));
            connect(RS1.q, qn) annotation(
              Line(points = {{70, 10}, {80, 10}, {80, -70}, {100, -70}}, color = {127, 0, 127}));
            annotation(
              Documentation(info = "<html>
<p>Basing on the RS component JKFF is a J-K-flipflop composed according the schematic. Its parameter delayTime is the delay time of the RS component transport delay, q0 is the initial value of that delay.</p>
</html>"),
              Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-90, 100}, {90, -100}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 170}, fillPattern = FillPattern.Solid), Line(points = {{-60, 100}, {-60, -100}}, color = {0, 0, 0}, thickness = 0.5), Line(points = {{60, 100}, {60, -100}}, color = {0, 0, 0}, thickness = 0.5), Text(extent = {{-86, 80}, {-64, 60}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, textString = "J"), Text(extent = {{64, 80}, {86, 60}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, textString = "Q"), Text(extent = {{64, -60}, {86, -80}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, textString = "QN"), Line(points = {{-90, 20}, {-60, 0}, {-90, -20}}, color = {0, 0, 0}), Text(extent = {{-150, -100}, {150, -160}}, lineColor = {0, 0, 255}, textString = "%name"), Text(extent = {{-100, 100}, {100, 40}}, lineColor = {0, 0, 0}, textString = "JK"), Text(extent = {{-86, -60}, {-64, -80}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, textString = "K")}));
          end JKFF;

          model HalfAdder "Half adder"
            import L = Modelica.Electrical.Digital.Interfaces.Logic;
            parameter Real delayTime = 0 "Delay time";
            Modelica.Electrical.Digital.Interfaces.DigitalInput b annotation(
              Placement(transformation(extent = {{-110, -80}, {-90, -60}}, rotation = 0)));
            Modelica.Electrical.Digital.Interfaces.DigitalOutput s annotation(
              Placement(transformation(extent = {{90, 60}, {110, 80}}, rotation = 0)));
            Modelica.Electrical.Digital.Interfaces.DigitalInput a annotation(
              Placement(transformation(extent = {{-110, 60}, {-90, 80}}, rotation = 0)));
            Modelica.Electrical.Digital.Interfaces.DigitalOutput c annotation(
              Placement(transformation(extent = {{90, -80}, {110, -60}}, rotation = 0)));
            Modelica.Electrical.Digital.Gates.AndGate AND(tLH = delayTime, tHL = delayTime, G2(y(start = L.'U', fixed = true))) annotation(
              Placement(transformation(extent = {{-20, -82}, {20, -42}}, rotation = 0)));
            Modelica.Electrical.Digital.Gates.XorGate XOR(tLH = delayTime, tHL = delayTime, G2(y(start = L.'U', fixed = true))) annotation(
              Placement(transformation(extent = {{-20, 42}, {20, 82}}, rotation = 0)));
          equation
            connect(AND.y, c) annotation(
              Line(points = {{20, -62}, {60, -62}, {60, -70}, {100, -70}}, color = {127, 0, 127}));
            connect(XOR.y, s) annotation(
              Line(points = {{20, 62}, {60, 62}, {60, 70}, {100, 70}}, color = {127, 0, 127}));
            connect(b, AND.x[1]) annotation(
              Line(points = {{-100, -70}, {-12, -70}}, color = {127, 0, 127}));
            connect(b, XOR.x[1]) annotation(
              Line(points = {{-100, -70}, {-30, -70}, {-30, 54}, {-12, 54}}, color = {127, 0, 127}));
            connect(a, XOR.x[2]) annotation(
              Line(points = {{-100, 70}, {-12, 70}}, color = {127, 0, 127}));
            connect(a, AND.x[2]) annotation(
              Line(points = {{-100, 70}, {-40, 70}, {-40, -54}, {-12, -54}}, color = {127, 0, 127}));
            annotation(
              Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-90, 100}, {90, -100}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 170}, fillPattern = FillPattern.Solid), Text(extent = {{-90, 80}, {-60, 60}}, lineColor = {0, 0, 0}, textString = "a"), Text(extent = {{-90, -60}, {-60, -80}}, lineColor = {0, 0, 0}, textString = "b"), Text(extent = {{60, 80}, {90, 60}}, lineColor = {0, 0, 0}, textString = "s"), Text(extent = {{60, -60}, {90, -80}}, lineColor = {0, 0, 0}, textString = "c"), Text(extent = {{-150, -100}, {150, -160}}, lineColor = {0, 0, 255}, textString = "%name"), Text(extent = {{-100, 100}, {100, 0}}, lineColor = {0, 0, 0}, textString = "+"), Line(points = {{-60, 100}, {-60, -100}}, color = {0, 0, 0}, thickness = 0.5), Line(points = {{60, 100}, {60, -100}}, color = {0, 0, 0}, thickness = 0.5)}),
              Documentation(info = "<html>
<p>HalfAdder is a two bit adder which is composed by Gates components.</p>
<p>Its logic behavior is like this:</p>
<p><b>HalfAdder behavior</b></p>
<table cellspacing=\"0\" cellpadding=\"2\" border=\"1\"><tr>
<td valign=\"top\"><h4>input a</h4></td>
<td valign=\"top\"><h4>input b</h4></td>
<td valign=\"top\"><h4>sum s</h4></td>
<td valign=\"top\"><h4>carry c</h4></td>
</tr>
<tr>
<td valign=\"top\"><p>0</p></td>
<td valign=\"top\"><p>0</p></td>
<td valign=\"top\"><p>0</p></td>
<td valign=\"top\"><p>0</p></td>
</tr>
<tr>
<td valign=\"top\"><p>1</p></td>
<td valign=\"top\"><p>0</p></td>
<td valign=\"top\"><p>1</p></td>
<td valign=\"top\"><p>0</p></td>
</tr>
<tr>
<td valign=\"top\"><p>0</p></td>
<td valign=\"top\"><p>1</p></td>
<td valign=\"top\"><p>1</p></td>
<td valign=\"top\"><p>0</p></td>
</tr>
<tr>
<td valign=\"top\"><p>1</p></td>
<td valign=\"top\"><p>1</p></td>
<td valign=\"top\"><p>0</p></td>
<td valign=\"top\"><p>1</p></td>
</tr>
</table>
<p>The parameter delayTime is the delay time (tLH=tHL) of both the components.</p>
</html>"));
          end HalfAdder;

          model FullAdder "Adding circuit for binary numbers with input carry bit"
            HalfAdder Adder2(delayTime = 0.001) annotation(
              Placement(transformation(extent = {{10, 36}, {50, 76}}, rotation = 0)));
            HalfAdder Adder1(delayTime = 0.001) annotation(
              Placement(transformation(extent = {{-60, 36}, {-20, 76}}, rotation = 0)));
            Modelica.Electrical.Digital.Interfaces.DigitalInput a annotation(
              Placement(transformation(origin = {-100, 70}, extent = {{-10, -10}, {10, 10}}, rotation = 180)));
            Modelica.Electrical.Digital.Interfaces.DigitalInput b annotation(
              Placement(transformation(origin = {-100, 30}, extent = {{-10, -10}, {10, 10}}, rotation = 180)));
            Modelica.Electrical.Digital.Interfaces.DigitalInput c_in annotation(
              Placement(transformation(origin = {-100, -70}, extent = {{-10, -10}, {10, 10}}, rotation = 180)));
            Modelica.Electrical.Digital.Interfaces.DigitalOutput s annotation(
              Placement(transformation(origin = {101, 70}, extent = {{11, -10}, {-11, 10}}, rotation = 180)));
            Modelica.Electrical.Digital.Interfaces.DigitalOutput c_out annotation(
              Placement(transformation(origin = {100, -70}, extent = {{10, -10}, {-10, 10}}, rotation = 180)));
            Modelica.Electrical.Digital.Basic.Or OR annotation(
              Placement(transformation(extent = {{10, -90}, {50, -50}}, rotation = 0)));
          equation
            connect(c_out, OR.y) annotation(
              Line(points = {{100, -70}, {50, -70}}, color = {127, 0, 127}));
            connect(Adder2.c, OR.x[2]) annotation(
              Line(points = {{50, 42}, {70, 42}, {70, -40}, {10, -40}, {10, -62}, {18, -62}}, color = {127, 0, 127}));
            connect(Adder2.s, s) annotation(
              Line(points = {{50, 70}, {101, 70}}, color = {127, 0, 127}));
            connect(Adder1.a, a) annotation(
              Line(points = {{-60, 70}, {-100, 70}}, color = {127, 0, 127}));
            connect(b, Adder1.b) annotation(
              Line(points = {{-100, 30}, {-70, 30}, {-70, 42}, {-60, 42}}, color = {127, 0, 127}));
            connect(Adder1.s, Adder2.a) annotation(
              Line(points = {{-20, 70}, {10, 70}}, color = {127, 0, 127}));
            connect(Adder1.c, OR.x[1]) annotation(
              Line(points = {{-20, 42}, {-10, 42}, {-10, -78}, {18, -78}}, color = {127, 0, 127}));
            connect(c_in, Adder2.b) annotation(
              Line(points = {{-100, -70}, {0, -70}, {0, 42}, {10, 42}}, color = {127, 0, 127}));
            annotation(
              Documentation(info = "<html>
<p>FullAdder is a two bit adder with additional carry in bit which is composed by Gates components.</p>
<p>Its logic behavior is like this:</p>
<p><b>FullAdder behavior</b></p>
<table cellspacing=\"0\" cellpadding=\"2\" border=\"1\"><tr>
<td valign=\"top\"><h4>input a</h4></td>
<td valign=\"top\"><h4>input b</h4></td>
<td valign=\"top\"><h4>input carry c_in</h4></td>
<td valign=\"top\"><h4>sum s</h4></td>
<td valign=\"top\"><h4>output carry c_out</h4></td>
</tr>
<tr>
<td valign=\"top\"><p>0</p></td>
<td valign=\"top\"><p>0</p></td>
<td valign=\"top\"><p>0</p></td>
<td valign=\"top\"><p>0</p></td>
<td valign=\"top\"><p>0</p></td>
</tr>
<tr>
<td valign=\"top\"><p>1</p></td>
<td valign=\"top\"><p>0</p></td>
<td valign=\"top\"><p>0</p></td>
<td valign=\"top\"><p>1</p></td>
<td valign=\"top\"><p>0</p></td>
</tr>
<tr>
<td valign=\"top\"><p>0</p></td>
<td valign=\"top\"><p>1</p></td>
<td valign=\"top\"><p>0</p></td>
<td valign=\"top\"><p>1</p></td>
<td valign=\"top\"><p>0</p></td>
</tr>
<tr>
<td valign=\"top\"><p>1</p></td>
<td valign=\"top\"><p>1</p></td>
<td valign=\"top\"><p>0</p></td>
<td valign=\"top\"><p>0</p></td>
<td valign=\"top\"><p>1</p></td>
</tr>
<tr>
<td valign=\"top\"><p>0</p></td>
<td valign=\"top\"><p>0</p></td>
<td valign=\"top\"><p>1</p></td>
<td valign=\"top\"><p>0</p></td>
<td valign=\"top\"><p>1</p></td>
</tr>
<tr>
<td valign=\"top\"><p>1</p></td>
<td valign=\"top\"><p>0</p></td>
<td valign=\"top\"><p>1</p></td>
<td valign=\"top\"><p>0</p></td>
<td valign=\"top\"><p>1</p></td>
</tr>
<tr>
<td valign=\"top\"><p>0</p></td>
<td valign=\"top\"><p>1</p></td>
<td valign=\"top\"><p>1</p></td>
<td valign=\"top\"><p>0</p></td>
<td valign=\"top\"><p>1</p></td>
</tr>
<tr>
<td valign=\"top\"><p>1</p></td>
<td valign=\"top\"><p>1</p></td>
<td valign=\"top\"><p>1</p></td>
<td valign=\"top\"><p>1</p></td>
<td valign=\"top\"><p>1</p></td>
</tr>
</table>
</html>"),
              Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-90, 100}, {90, -100}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 170}, fillPattern = FillPattern.Solid), Text(extent = {{-150, -96}, {150, -151}}, lineColor = {0, 0, 255}, textString = "%name"), Text(extent = {{-86, 80}, {-64, 60}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, textString = "a"), Text(extent = {{-86, 40}, {-64, 20}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, textString = "b"), Text(extent = {{-86, -60}, {-64, -80}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, textString = "c_in"), Text(extent = {{60, -60}, {90, -80}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, textString = "c_out"), Text(extent = {{64, 80}, {86, 60}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, textString = "s"), Text(extent = {{-100, 100}, {100, 0}}, lineColor = {0, 0, 0}, textString = "+"), Line(points = {{-60, 100}, {-60, -100}}, color = {0, 0, 0}, thickness = 0.5), Line(points = {{60, 100}, {60, -100}}, color = {0, 0, 0}, thickness = 0.5)}));
          end FullAdder;

          model Adder "Generic N Bit Adder"
            import Modelica.Electrical.Digital;
            parameter Integer n = 2 "Number of single adders";
            Modelica.Electrical.Digital.Examples.Utilities.FullAdder Adder[n] annotation(
              Placement(transformation(extent = {{-20, -20}, {20, 20}}, rotation = 0)));
            Modelica.Electrical.Digital.Interfaces.DigitalInput a[n] annotation(
              Placement(transformation(extent = {{-110, 60}, {-90, 80}}, rotation = 0)));
            Modelica.Electrical.Digital.Interfaces.DigitalInput b[n] annotation(
              Placement(transformation(extent = {{-110, 20}, {-90, 40}}, rotation = 0)));
            Modelica.Electrical.Digital.Interfaces.DigitalInput c_in annotation(
              Placement(transformation(extent = {{-110, -80}, {-90, -60}}, rotation = 0)));
            Modelica.Electrical.Digital.Interfaces.DigitalOutput s[n] annotation(
              Placement(transformation(extent = {{90, 60}, {110, 80}}, rotation = 0)));
            Modelica.Electrical.Digital.Interfaces.DigitalOutput c_out annotation(
              Placement(transformation(extent = {{90, -80}, {110, -60}}, rotation = 0)));
          equation
            connect(c_in, Adder[1].c_in);
            for i in 1:n loop
              connect(a[i], Adder[i].a);
              connect(b[i], Adder[i].b);
              connect(Adder[i].a, s[i]);
              if i > 1 then
                connect(Adder[i - 1].c_out, Adder[i].c_in);
              end if;
            end for;
            connect(Adder[n].c_out, c_out);
            annotation(
              Documentation(info = "<html>
<p>The Adder is a generic n bit adder which is composed as a chain of FullAdder components. n can be chosen by the user, a and b are the n bit input vectors, s is the sum vector, and c_out is the carry bit of the &quot;highes&quot; FullAdder.   All components are built up by Gate components.</p>
</html>"),
              Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-90, 100}, {90, -100}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 170}, fillPattern = FillPattern.Solid), Text(extent = {{68, 80}, {88, 60}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, textString = "S"), Text(extent = {{-150, -100}, {150, -160}}, lineColor = {0, 0, 255}, textString = "%name"), Text(extent = {{-40, 60}, {40, 20}}, lineColor = {0, 0, 0}, textString = "Adder"), Text(extent = {{48, -60}, {88, -80}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, textString = "Cout"), Text(extent = {{-90, -60}, {-50, -80}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, textString = "Cin"), Text(extent = {{-88, 80}, {-68, 60}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, textString = "A"), Text(extent = {{-88, 40}, {-68, 20}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, textString = "B")}));
          end Adder;

          model Counter3 "3 Bit Counter"
            import D = Modelica.Electrical.Digital;
            import L = Modelica.Electrical.Digital.Interfaces.Logic;
            D.Interfaces.DigitalInput enable annotation(
              Placement(transformation(extent = {{-110, 60}, {-90, 80}}, rotation = 0)));
            D.Interfaces.DigitalOutput q2 annotation(
              Placement(transformation(extent = {{90, 60}, {110, 80}}, rotation = 0)));
            D.Interfaces.DigitalInput count annotation(
              Placement(transformation(extent = {{-110, -80}, {-90, -60}}, rotation = 0)));
            D.Examples.Utilities.JKFF FF1 annotation(
              Placement(transformation(extent = {{-74, -20}, {-34, 20}}, rotation = 0)));
            D.Examples.Utilities.JKFF FF2 annotation(
              Placement(transformation(extent = {{-20, -20}, {20, 20}}, rotation = 0)));
            D.Examples.Utilities.JKFF FF3 annotation(
              Placement(transformation(extent = {{34, -20}, {74, 20}}, rotation = 0)));
            D.Interfaces.DigitalOutput q1 annotation(
              Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
            D.Interfaces.DigitalOutput q0 annotation(
              Placement(transformation(extent = {{90, -80}, {110, -60}}, rotation = 0)));
          equation
            connect(enable, FF1.j) annotation(
              Line(points = {{-100, 70}, {-80, 70}, {-80, 14}, {-74, 14}}, color = {127, 0, 127}, fillPattern = FillPattern.Solid));
            connect(enable, FF1.k) annotation(
              Line(points = {{-100, 70}, {-80, 70}, {-80, -14}, {-74, -14}}, color = {127, 0, 127}, fillPattern = FillPattern.Solid));
            connect(count, FF1.clk) annotation(
              Line(points = {{-100, -70}, {-86, -70}, {-86, 0}, {-74, 0}}, color = {127, 0, 127}, fillPattern = FillPattern.Solid));
            connect(FF1.q, FF2.clk) annotation(
              Line(points = {{-34, 14}, {-30, 14}, {-30, 0}, {-20, 0}}, color = {127, 0, 127}, fillPattern = FillPattern.Solid));
            connect(FF2.q, FF3.clk) annotation(
              Line(points = {{20, 14}, {24, 14}, {24, 0}, {34, 0}}, color = {127, 0, 127}, fillPattern = FillPattern.Solid));
            connect(FF2.j, enable) annotation(
              Line(points = {{-20, 14}, {-26, 14}, {-26, 70}, {-100, 70}}, color = {127, 0, 127}, fillPattern = FillPattern.Solid));
            connect(FF2.k, FF2.j) annotation(
              Line(points = {{-20, -14}, {-26, -14}, {-26, 14}, {-20, 14}}, color = {127, 0, 127}, fillPattern = FillPattern.Solid));
            connect(FF3.k, FF3.j) annotation(
              Line(points = {{34, -14}, {28, -14}, {28, 14}, {34, 14}}, color = {127, 0, 127}, fillPattern = FillPattern.Solid));
            connect(FF3.j, enable) annotation(
              Line(points = {{34, 14}, {28, 14}, {28, 70}, {-100, 70}}, color = {127, 0, 127}, fillPattern = FillPattern.Solid));
            connect(FF3.q, q2) annotation(
              Line(points = {{74, 14}, {80, 14}, {80, 70}, {100, 70}}, color = {127, 0, 127}, fillPattern = FillPattern.Solid));
            connect(FF1.q, q0) annotation(
              Line(points = {{-34, 14}, {-30, 14}, {-30, -70}, {100, -70}}, color = {127, 0, 127}));
            connect(FF2.q, q1) annotation(
              Line(points = {{20, 14}, {24, 14}, {24, -50}, {86, -50}, {86, 0}, {100, 0}}, color = {127, 0, 127}));
            annotation(
              Documentation(info = "<html>
<p>The Counter3 counts the high-low slopes of the count signal, if the enable signal is set to be true. It is composed by three JK flipflops. q0, q1, and q2 are the bits of the resulting number, where q0 is the lowest, and q2 the highest bit.</p>
</html>"),
              Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-90, 100}, {90, -100}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 170}, fillPattern = FillPattern.Solid), Text(extent = {{-80, 80}, {-40, 60}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, textString = "ENABLE"), Text(extent = {{64, 80}, {86, 60}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, textString = "Q2"), Text(extent = {{64, -60}, {86, -80}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, textString = "Q0"), Text(extent = {{-150, -100}, {150, -160}}, lineColor = {0, 0, 255}, textString = "%name"), Text(extent = {{-60, 40}, {60, 0}}, lineColor = {0, 0, 0}, textString = "Counter3"), Text(extent = {{-80, -60}, {-40, -80}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, textString = "COUNT"), Text(extent = {{62, 8}, {84, -12}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, textString = "Q1")}));
          end Counter3;

          model Counter "Generic N Bit Counter"
            import D = Modelica.Electrical.Digital;
            import L = Modelica.Electrical.Digital.Interfaces.Logic;
            parameter Integer n = 3 "Number of bits";
            parameter Modelica.SIunits.Time delayTime = 0.001 "Delay of each JKFF";
            parameter D.Interfaces.Logic q0 = L.'0' "Initial value";
            D.Interfaces.DigitalInput enable annotation(
              Placement(transformation(extent = {{-110, 60}, {-90, 80}}, rotation = 0)));
            D.Interfaces.DigitalInput count annotation(
              Placement(transformation(extent = {{-110, -80}, {-90, -60}}, rotation = 0)));
            D.Examples.Utilities.JKFF FF[n](each delayTime = delayTime, each q0 = q0);
            D.Interfaces.DigitalOutput q[n] annotation(
              Placement(transformation(extent = {{90, -80}, {110, 80}}, rotation = 0)));
          equation
            connect(enable, FF[1].j);
            connect(enable, FF[1].k);
            connect(count, FF[1].clk);
            connect(FF[1].q, q[1]);
            for i in 2:n loop
              connect(enable, FF[i].j);
              connect(enable, FF[i].k);
              connect(FF[i - 1].q, FF[i].clk);
              connect(FF[i].q, q[i]);
            end for;
            annotation(
              Documentation(info = "<html>
<p>The Counter is a generic component, which counts the high-low slopes of the count signal, if the enable signal is set to be true. It is composed by  n JK flipflops. q is the resulting number, where q[0] is the lowest, and q[n] the highest bit.</p>
</html>"),
              Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{90, 80}, {110, -80}}, lineColor = {127, 0, 127}, fillColor = {127, 0, 127}, fillPattern = FillPattern.Solid)}),
              Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{90, 80}, {110, -80}}, lineColor = {127, 0, 127}, fillColor = {127, 0, 127}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-90, 100}, {90, -100}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 170}, fillPattern = FillPattern.Solid), Text(extent = {{-80, 80}, {-40, 60}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, textString = "ENABLE"), Text(extent = {{66, 8}, {88, -12}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, textString = "Q"), Text(extent = {{-150, -100}, {150, -160}}, lineColor = {0, 0, 255}, textString = "%name"), Text(extent = {{-40, 40}, {40, 0}}, lineColor = {0, 0, 0}, textString = "Counter"), Text(extent = {{-80, -60}, {-40, -80}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, textString = "COUNT")}));
          end Counter;
          annotation(
            Documentation(info = "<html>
<p>This package contains utility components used by package Examples. Each component is built up hierarchically by components of the Gates package. In this way the Gates components were tested, and their usage is demonstrated.</p>
</html>"));
        end Utilities;
        annotation(
          Documentation(info = "<html>
<p>This package contains examples that demonstrate the usage of the components of the Electrical.Digital library.</p>
<p>The examples are simple to understand. They will show a typical behavior of the components, and they will give hints to users.</p>
</html>"));
      end Examples;

      package Interfaces "Basic definitions"
        extends Modelica.Icons.InterfacesPackage;
        type Logic = enumeration('U' "U  Uninitialized", 'X' "X  Forcing Unknown", '0' "0  Forcing 0", '1' "1  Forcing 1", 'Z' "Z  High Impedance", 'W' "W  Weak Unknown", 'L' "L  Weak 0", 'H' "H  Weak 1", '-' "-  Do not care") "Logic values and their coding according to IEEE 1164 STD_ULOGIC type" annotation(
          Documentation(info = "<html>
<p><b>Code Table:</b></p>

<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"><b>Logic value</b></td>
      <td valign=\"top\"><b>Meaning</b></td>
  </tr>

  <tr><td valign=\"top\">'U'</td> <td valign=\"top\">Uninitialized</td></tr>
  <tr><td valign=\"top\">'X'</td> <td valign=\"top\">Forcing Unknown</td></tr>
  <tr><td valign=\"top\">'0'</td> <td valign=\"top\">Forcing 0</td></tr>
  <tr><td valign=\"top\">'1'</td> <td valign=\"top\">Forcing 1</td></tr>
  <tr><td valign=\"top\">'Z'</td> <td valign=\"top\">High Impedance</td></tr>
  <tr><td valign=\"top\">'W'</td> <td valign=\"top\">Weak Unknown</td></tr>
  <tr><td valign=\"top\">'L'</td> <td valign=\"top\">Weak 0</td></tr>
  <tr><td valign=\"top\">'H'</td> <td valign=\"top\">Weak 1</td></tr>
  <tr><td valign=\"top\">'-'</td> <td valign=\"top\">Do not care</td></tr>
</table>

</html>"));
        type UX01 = enumeration('U' "U  Uninitialized", 'X' "X  Forcing Unknown", '0' "0  Forcing 0", '1' "1  Forcing 1") "4-valued subtype of IEEE 1164 STD_ULOGIC type" annotation(
          Documentation(info = "<html>
<p><b>Code Table:</b></p>

<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"><b>Logic value</b></td>
      <td valign=\"top\"><b>Meaning</b></td>
  </tr>

  <tr><td valign=\"top\">'U'</td> <td valign=\"top\">Uninitialized</td></tr>
  <tr><td valign=\"top\">'X'</td> <td valign=\"top\">Forcing Unknown</td></tr>
  <tr><td valign=\"top\">'0'</td> <td valign=\"top\">Forcing 0</td></tr>
  <tr><td valign=\"top\">'1'</td> <td valign=\"top\">Forcing 1</td></tr>

</table>
</html>"));
        type Strength = enumeration('S_X01', 'S_X0H', 'S_XL1', 'S_X0Z', 'S_XZ1', 'S_WLH', 'S_WLZ', 'S_WZH', 'S_W0H', 'S_WL1') "Output strengths of registers" annotation(
          Documentation(info = "<html>

<p><b>Strength Table:</b></p>

<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"><b>Strength</b></td>
      <td valign=\"top\"><b>Output conversion to</b></td>
  </tr>

  <tr><td valign=\"top\">'S_X01'</td> <td valign=\"top\">Forcing X, 0, 1</td></tr>
  <tr><td valign=\"top\">'S_X0H'</td> <td valign=\"top\">Forcing X, 0 and Weak 1</td></tr>
  <tr><td valign=\"top\">'S_XL1'</td> <td valign=\"top\">Forcing X, 1 and Weak 0</td></tr>
  <tr><td valign=\"top\">'S_X0Z'</td> <td valign=\"top\">Forcing X, 0 and High Impedance</td></tr>
  <tr><td valign=\"top\">'S_XZ1'</td> <td valign=\"top\">Forcing X, 1 and High Impedance</td></tr>
  <tr><td valign=\"top\">'S_WLH'</td> <td valign=\"top\">Weak X, 0, 1</td></tr>
  <tr><td valign=\"top\">'S_WLZ'</td> <td valign=\"top\">Weak X, 0 and High Impedance</td></tr>
  <tr><td valign=\"top\">'S_WZH'</td> <td valign=\"top\">Weak X, 1 and High Impedance</td></tr>
  <tr><td valign=\"top\">'S_W0H'</td> <td valign=\"top\">Weak X, 1 and Forcing 0</td></tr>
  <tr><td valign=\"top\">'S_WL1'</td> <td valign=\"top\">Weak X, 0 and Forcing 1</td></tr>
</table>
</html>"));
        connector DigitalSignal = Logic "Digital port (both input/output possible)" annotation(
          Documentation(info = "<html>
<p>DigitalSignal is the basic digital connector definition. A direction (input, output) is not yet defined. DigitalSignal is of type Logic. It can have the logic values (U, X, 0, 1, ...) which are internally coded by integer values by using the enumeration (c.f. the definition of type Logic).</p>
</html>"));
        connector DigitalInput = input DigitalSignal "Input DigitalSignal as connector" annotation(
          defaultComponentName = "x",
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, -100}, {100, 100}}, lineColor = {127, 0, 127}, fillColor = {127, 0, 127}, fillPattern = FillPattern.Solid)}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-150, -100}, {150, -160}}, lineColor = {127, 0, 127}, textString = "%name"), Rectangle(extent = {{-100, -100}, {100, 100}}, lineColor = {127, 0, 127}, fillColor = {127, 0, 127}, fillPattern = FillPattern.Solid)}),
          Documentation(info = "<html>
<p>DigitalInput is the  digital input connector definition. DigitalInput is of type Logic. It can have the logic values (U, X, 0, 1, ...) which are internally coded by integer values by using the enumeration (c.f. the definition of type Logic).</p>
</html>"));
        connector DigitalOutput = output DigitalSignal "Output DigitalSignal as connector" annotation(
          defaultComponentName = "y",
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{-100, 100}, {100, 0}, {-100, -100}, {-100, 100}}, lineColor = {127, 0, 127}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid)}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{-100, 100}, {100, 0}, {-100, -100}, {-100, 100}}, lineColor = {127, 0, 127}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-150, -100}, {150, -160}}, lineColor = {127, 0, 127}, textString = "%name")}),
          Documentation(info = "<html>
<p>DigitalOutput is the  digital output connector definition. DigitalOutput  is of type Logic. It can have the logic values (U, X, 0, 1, ...) which are internally coded by integer values by using the enumeration (c.f. the definition of type Logic). The arrow shape symbolizes the signal flow direction.</p>
</html>"));

        partial block SISO "Single input, single output"
          import D = Modelica.Electrical.Digital;
          D.Interfaces.DigitalInput x "Connector of Digital input signal" annotation(
            Placement(transformation(extent = {{-70, -10}, {-50, 10}}, rotation = 0)));
          D.Interfaces.DigitalOutput y "Connector of Digital output signal" annotation(
            Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-50, 100}, {50, -100}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{50, 0}, {92, 0}}, color = {127, 0, 127})}),
            Documentation(info = "<html>
<p>SISO is a partial model for the connection pattern with <b>s</b>ingle (scalar) digital <b>i</b>nput and <b>s</b>ingle (scalar) digital <b>o</b>utput. Besides the connectors it provides a rectangle for the icon which can be filled in by the component which inherits the SISO model.</p>
</html>"));
        end SISO;

        partial block MISO "Multiple input - single output"
          import D = Modelica.Electrical.Digital;
          parameter Integer n(final min = 2) = 2 "Number of inputs";
          D.Interfaces.DigitalInput x[n] "Connector of Digital input signal vector" annotation(
            Placement(transformation(extent = {{-70, -80}, {-50, 80}}, rotation = 0)));
          D.Interfaces.DigitalOutput y "Connector of Digital output signal" annotation(
            Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-50, 100}, {50, -100}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{50, 0}, {90, 0}}, color = {127, 0, 127})}),
            Documentation(info = "<html>
<p>MISO is a partial model for the connection pattern with <b>m</b>ultiple (vector) digital<b> i</b>nput and <b>s</b>ingle (scalar) digital <b>o</b>utput. Besides the connectors it provides a rectangle for the icon which can be filled in by the component which inherits the MISO model.</p>
</html>"));
        end MISO;

        partial block MIMO "Multiple input - multiple output"
          import D = Modelica.Electrical.Digital;
          parameter Integer n(final min = 1) = 1 "Number of inputs = Number of outputs";
          D.Interfaces.DigitalInput x[n] "Connector of Digital input signal vector" annotation(
            Placement(transformation(extent = {{-70, -80}, {-50, 80}}, rotation = 0)));
          D.Interfaces.DigitalOutput y[n] "Connector of Digital output signal vector" annotation(
            Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-50, 100}, {50, -100}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{50, 0}, {90, 0}}, color = {127, 0, 127}), Rectangle(extent = {{50, 80}, {70, -80}}, lineColor = {127, 33, 107}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid)}),
            Documentation(info = "<html>
<p>MIMO is a partial model for the connection pattern with <b>m</b>ultiple (vector) digital<b> i</b>nput and <b>m</b>ultiple  (vector) digital <b>o</b>utput. Besides the connectors it provides a rectangle for the icon which can be filled in by the component which inherits the MISO model.</p>
</html>"));
        end MIMO;
        annotation(
          Documentation(info = "<html>
<p>This package contains basic definitions: Type definitions of Logic and Strength,  interface definitions (connectors) for digital electrical components, and partial models for connection patterns which are often used.</p>
</html>"));
      end Interfaces;

      package Tables "Truth tables for all components of package Digital"
        extends Modelica.Icons.Package;
        import D = Modelica.Electrical.Digital;
        import L = Modelica.Electrical.Digital.Interfaces.Logic;
        import R = Modelica.Electrical.Digital.Interfaces.UX01;
        import S = Modelica.Electrical.Digital.Interfaces.Strength;
        constant D.Interfaces.Logic AndTable[L, L] = [L.'U', L.'U', L.'0', L.'U', L.'U', L.'U', L.'0', L.'U', L.'U'; L.'U', L.'X', L.'0', L.'X', L.'X', L.'X', L.'0', L.'X', L.'X'; L.'0', L.'0', L.'0', L.'0', L.'0', L.'0', L.'0', L.'0', L.'0'; L.'U', L.'X', L.'0', L.'1', L.'X', L.'X', L.'0', L.'1', L.'X'; L.'U', L.'X', L.'0', L.'X', L.'X', L.'X', L.'0', L.'X', L.'X'; L.'U', L.'X', L.'0', L.'X', L.'X', L.'X', L.'0', L.'X', L.'X'; L.'0', L.'0', L.'0', L.'0', L.'0', L.'0', L.'0', L.'0', L.'0'; L.'U', L.'X', L.'0', L.'1', L.'X', L.'X', L.'0', L.'1', L.'X'; L.'U', L.'X', L.'0', L.'X', L.'X', L.'X', L.'0', L.'X', L.'X'] "9-value logic for 'and'";
        constant D.Interfaces.Logic OrTable[L, L] = [L.'U', L.'U', L.'U', L.'1', L.'U', L.'U', L.'U', L.'1', L.'U'; L.'U', L.'X', L.'X', L.'1', L.'X', L.'X', L.'X', L.'1', L.'X'; L.'U', L.'X', L.'0', L.'1', L.'X', L.'X', L.'0', L.'1', L.'X'; L.'1', L.'1', L.'1', L.'1', L.'1', L.'1', L.'1', L.'1', L.'1'; L.'U', L.'X', L.'X', L.'1', L.'X', L.'X', L.'X', L.'1', L.'X'; L.'U', L.'X', L.'X', L.'1', L.'X', L.'X', L.'X', L.'1', L.'X'; L.'U', L.'X', L.'0', L.'1', L.'X', L.'X', L.'0', L.'1', L.'X'; L.'1', L.'1', L.'1', L.'1', L.'1', L.'1', L.'1', L.'1', L.'1'; L.'U', L.'X', L.'X', L.'1', L.'X', L.'X', L.'X', L.'1', L.'X'] "9-value logic for 'or'";
        constant D.Interfaces.Logic NotTable[L] = {L.'U', L.'X', L.'1', L.'0', L.'X', L.'X', L.'1', L.'0', L.'X'} "9-value logic for 'not'";
        constant D.Interfaces.Logic XorTable[L, L] = [L.'U', L.'U', L.'U', L.'U', L.'U', L.'U', L.'U', L.'U', L.'U'; L.'U', L.'X', L.'X', L.'X', L.'X', L.'X', L.'X', L.'X', L.'X'; L.'U', L.'X', L.'0', L.'1', L.'X', L.'X', L.'0', L.'1', L.'X'; L.'U', L.'X', L.'1', L.'0', L.'X', L.'X', L.'1', L.'0', L.'X'; L.'U', L.'X', L.'X', L.'X', L.'X', L.'X', L.'X', L.'X', L.'X'; L.'U', L.'X', L.'X', L.'X', L.'X', L.'X', L.'X', L.'X', L.'X'; L.'U', L.'X', L.'0', L.'1', L.'X', L.'X', L.'0', L.'1', L.'X'; L.'U', L.'X', L.'1', L.'0', L.'X', L.'X', L.'1', L.'0', L.'X'; L.'U', L.'X', L.'X', L.'X', L.'X', L.'X', L.'X', L.'X', L.'X'] "9-value logic for 'xor'";
        constant D.Interfaces.Logic ResolutionTable[L, L] = [L.'U', L.'U', L.'U', L.'U', L.'U', L.'U', L.'U', L.'U', L.'U'; L.'U', L.'X', L.'X', L.'X', L.'X', L.'X', L.'X', L.'X', L.'X'; L.'U', L.'X', L.'0', L.'X', L.'0', L.'0', L.'0', L.'0', L.'X'; L.'U', L.'X', L.'X', L.'1', L.'1', L.'1', L.'1', L.'1', L.'X'; L.'U', L.'X', L.'0', L.'1', L.'Z', L.'W', L.'L', L.'H', L.'X'; L.'U', L.'X', L.'0', L.'1', L.'W', L.'W', L.'W', L.'W', L.'X'; L.'U', L.'X', L.'0', L.'1', L.'L', L.'W', L.'L', L.'W', L.'X'; L.'U', L.'X', L.'0', L.'1', L.'H', L.'W', L.'W', L.'H', L.'X'; L.'U', L.'X', L.'X', L.'X', L.'X', L.'X', L.'X', L.'X', L.'X'] "9-value logic for 'wiredX'";
        constant D.Interfaces.Logic X01Table[L] = {L.'X', L.'X', L.'0', L.'1', L.'X', L.'X', L.'0', L.'1', L.'X'};
        constant D.Interfaces.Logic X01ZTable[L] = {L.'X', L.'X', L.'0', L.'1', L.'Z', L.'X', L.'0', L.'1', L.'Z'};
        constant D.Interfaces.Logic UX01Table[L] = {L.'U', L.'X', L.'0', L.'1', L.'X', L.'X', L.'0', L.'1', L.'X'};
        constant Integer DelayTable[L, L] = [0, 0, -1, 1, 0, 0, -1, 1, 0; 0, 0, -1, 1, 0, 0, -1, 1, 0; 1, 1, 0, 1, 1, 1, 0, 1, 1; -1, -1, -1, 0, -1, -1, -1, 0, -1; 0, 0, -1, 1, 0, 0, -1, 1, 0; 0, 0, -1, 1, 0, 0, -1, 1, 0; 1, 1, 0, 1, 1, 1, 0, 1, 1; -1, -1, -1, 0, -1, -1, -1, 0, -1; 0, 0, -1, 1, 0, 0, -1, 1, 0] "Delay selection for signal change by [old_signal, new_signal] reading:
              -1: High-Low delay |
               0: no delay |
               1: Low-High delay";
        constant Integer ClockMap[L, L] = [0, 0, 0, 2, 0, 0, 0, 2, 0; 0, 0, 0, 2, 0, 0, 0, 2, 0; 2, 2, 0, 1, 2, 2, 0, 1, 2; 0, 0, 0, 0, 0, 0, 0, 0, 0; 0, 0, 0, 2, 0, 0, 0, 2, 0; 0, 0, 0, 2, 0, 0, 0, 2, 0; 2, 2, 0, 1, 2, 2, 0, 1, 0; 0, 0, 0, 0, 0, 0, 0, 0, 0; 0, 0, 0, 2, 0, 0, 0, 2, 0] "Edge detection by [pre(clock), clock] reading:
            0: 0-Transition |
            1: rising edge  |
            2: X-Transition";
        constant D.Interfaces.Logic StrengthMap[L, S] = [L.'U', L.'U', L.'U', L.'U', L.'U', L.'U', L.'U', L.'U', L.'U', L.'U'; L.'X', L.'X', L.'X', L.'X', L.'X', L.'W', L.'W', L.'W', L.'W', L.'W'; L.'0', L.'0', L.'L', L.'0', L.'Z', L.'L', L.'L', L.'Z', L.'0', L.'L'; L.'1', L.'H', L.'1', L.'Z', L.'1', L.'H', L.'Z', L.'H', L.'H', L.'1'; L.'X', L.'X', L.'X', L.'X', L.'X', L.'W', L.'W', L.'W', L.'W', L.'W'; L.'X', L.'X', L.'X', L.'X', L.'X', L.'W', L.'W', L.'W', L.'W', L.'W'; L.'0', L.'0', L.'L', L.'0', L.'Z', L.'L', L.'L', L.'Z', L.'0', L.'L'; L.'1', L.'H', L.'1', L.'Z', L.'1', L.'H', L.'Z', L.'H', L.'H', L.'1'; L.'X', L.'X', L.'X', L.'X', L.'X', L.'W', L.'W', L.'W', L.'W', L.'W'] "Output strength conversion by [signal, strength] reading";
        constant D.Interfaces.Logic NXferTable[L, L] = [L.'U', L.'U', L.'U', L.'U', L.'U', L.'U', L.'U', L.'U', L.'U'; L.'U', L.'X', L.'X', L.'X', L.'X', L.'X', L.'X', L.'X', L.'X'; L.'Z', L.'Z', L.'Z', L.'Z', L.'Z', L.'Z', L.'Z', L.'Z', L.'Z'; L.'U', L.'X', L.'0', L.'1', L.'Z', L.'W', L.'L', L.'H', L.'X'; L.'U', L.'X', L.'X', L.'X', L.'X', L.'X', L.'X', L.'X', L.'X'; L.'U', L.'X', L.'X', L.'X', L.'X', L.'X', L.'X', L.'X', L.'X'; L.'Z', L.'Z', L.'Z', L.'Z', L.'Z', L.'Z', L.'Z', L.'Z', L.'Z'; L.'U', L.'X', L.'0', L.'1', L.'Z', L.'W', L.'L', L.'H', L.'X'; L.'U', L.'X', L.'X', L.'X', L.'X', L.'X', L.'X', L.'X', L.'X'] "NX Transfer by [enable, input] reading";
        constant D.Interfaces.Logic NRXferTable[L, L] = [L.'U', L.'U', L.'U', L.'U', L.'U', L.'U', L.'U', L.'U', L.'U'; L.'U', L.'W', L.'W', L.'W', L.'W', L.'W', L.'W', L.'W', L.'W'; L.'Z', L.'Z', L.'Z', L.'Z', L.'Z', L.'Z', L.'Z', L.'Z', L.'Z'; L.'U', L.'W', L.'L', L.'H', L.'Z', L.'W', L.'L', L.'H', L.'W'; L.'U', L.'W', L.'W', L.'W', L.'W', L.'W', L.'W', L.'W', L.'W'; L.'U', L.'W', L.'W', L.'W', L.'W', L.'W', L.'W', L.'W', L.'W'; L.'Z', L.'Z', L.'Z', L.'Z', L.'Z', L.'Z', L.'Z', L.'Z', L.'Z'; L.'U', L.'W', L.'L', L.'H', L.'Z', L.'W', L.'L', L.'H', L.'W'; L.'U', L.'W', L.'W', L.'W', L.'W', L.'W', L.'W', L.'W', L.'W'] "NRX Transfer by [enable, input] reading";
        constant D.Interfaces.Logic PXferTable[L, L] = [L.'U', L.'U', L.'U', L.'U', L.'U', L.'U', L.'U', L.'U', L.'U'; L.'U', L.'X', L.'X', L.'X', L.'X', L.'X', L.'X', L.'X', L.'X'; L.'U', L.'X', L.'0', L.'1', L.'Z', L.'W', L.'L', L.'H', L.'X'; L.'Z', L.'Z', L.'Z', L.'Z', L.'Z', L.'Z', L.'Z', L.'Z', L.'Z'; L.'U', L.'X', L.'X', L.'X', L.'X', L.'X', L.'X', L.'X', L.'X'; L.'U', L.'X', L.'X', L.'X', L.'X', L.'X', L.'X', L.'X', L.'X'; L.'U', L.'X', L.'0', L.'1', L.'Z', L.'W', L.'L', L.'H', L.'X'; L.'Z', L.'Z', L.'Z', L.'Z', L.'Z', L.'Z', L.'Z', L.'Z', L.'Z'; L.'U', L.'X', L.'X', L.'X', L.'X', L.'X', L.'X', L.'X', L.'X'] "PX Transfer by [enable, input] reading";
        constant D.Interfaces.Logic PRXferTable[L, L] = [L.'U', L.'U', L.'U', L.'U', L.'U', L.'U', L.'U', L.'U', L.'U'; L.'U', L.'W', L.'W', L.'W', L.'W', L.'W', L.'W', L.'W', L.'W'; L.'U', L.'W', L.'L', L.'H', L.'Z', L.'W', L.'L', L.'H', L.'W'; L.'Z', L.'Z', L.'Z', L.'Z', L.'Z', L.'Z', L.'Z', L.'Z', L.'Z'; L.'U', L.'W', L.'W', L.'W', L.'W', L.'W', L.'W', L.'W', L.'W'; L.'U', L.'W', L.'W', L.'W', L.'W', L.'W', L.'W', L.'W', L.'W'; L.'U', L.'W', L.'L', L.'H', L.'Z', L.'W', L.'L', L.'H', L.'W'; L.'Z', L.'Z', L.'Z', L.'Z', L.'Z', L.'Z', L.'Z', L.'Z', L.'Z'; L.'U', L.'W', L.'W', L.'W', L.'W', L.'W', L.'W', L.'W', L.'W'] "PRX Transfer by [enable, input] reading";
        constant D.Interfaces.Logic Buf3sTable[S, R, R] = [{{{L.'U', L.'U', L.'U', L.'U'}, {L.'U', L.'X', L.'X', L.'X'}, {L.'Z', L.'Z', L.'Z', L.'Z'}, {L.'U', L.'X', L.'0', L.'1'}}, {{L.'U', L.'U', L.'U', L.'U'}, {L.'U', L.'X', L.'X', L.'X'}, {L.'Z', L.'Z', L.'Z', L.'Z'}, {L.'U', L.'X', L.'0', L.'H'}}, {{L.'U', L.'U', L.'U', L.'U'}, {L.'U', L.'X', L.'X', L.'X'}, {L.'Z', L.'Z', L.'Z', L.'Z'}, {L.'U', L.'X', L.'L', L.'1'}}, {{L.'U', L.'U', L.'U', L.'Z'}, {L.'U', L.'X', L.'X', L.'Z'}, {L.'Z', L.'Z', L.'Z', L.'Z'}, {L.'U', L.'X', L.'0', L.'Z'}}, {{L.'U', L.'U', L.'U', L.'U'}, {L.'U', L.'X', L.'X', L.'X'}, {L.'Z', L.'Z', L.'Z', L.'Z'}, {L.'U', L.'X', L.'Z', L.'1'}}, {{L.'U', L.'U', L.'U', L.'U'}, {L.'U', L.'W', L.'W', L.'W'}, {L.'Z', L.'Z', L.'Z', L.'Z'}, {L.'U', L.'W', L.'L', L.'H'}}, {{L.'U', L.'U', L.'U', L.'U'}, {L.'U', L.'W', L.'W', L.'Z'}, {L.'Z', L.'Z', L.'Z', L.'Z'}, {L.'U', L.'W', L.'L', L.'Z'}}, {{L.'U', L.'U', L.'U', L.'U'}, {L.'U', L.'W', L.'W', L.'W'}, {L.'Z', L.'Z', L.'Z', L.'Z'}, {L.'U', L.'W', L.'Z', L.'H'}}, {{L.'U', L.'U', L.'U', L.'U'}, {L.'U', L.'W', L.'W', L.'W'}, {L.'Z', L.'Z', L.'Z', L.'Z'}, {L.'U', L.'W', L.'0', L.'H'}}, {{L.'U', L.'U', L.'U', L.'U'}, {L.'U', L.'W', L.'W', L.'W'}, {L.'Z', L.'Z', L.'Z', L.'Z'}, {L.'U', L.'W', L.'L', L.'1'}}}] "Tristate Table by [strength, enable, input] reading, high-active enable";
        constant D.Interfaces.Logic Buf3slTable[S, R, R] = [{{{L.'U', L.'U', L.'U', L.'U'}, {L.'U', L.'X', L.'X', L.'X'}, {L.'U', L.'X', L.'0', L.'1'}, {L.'Z', L.'Z', L.'Z', L.'Z'}}, {{L.'U', L.'U', L.'U', L.'U'}, {L.'U', L.'X', L.'X', L.'X'}, {L.'U', L.'X', L.'0', L.'H'}, {L.'Z', L.'Z', L.'Z', L.'Z'}}, {{L.'U', L.'U', L.'U', L.'U'}, {L.'U', L.'X', L.'X', L.'X'}, {L.'U', L.'X', L.'L', L.'1'}, {L.'Z', L.'Z', L.'Z', L.'Z'}}, {{L.'U', L.'U', L.'U', L.'Z'}, {L.'U', L.'X', L.'X', L.'Z'}, {L.'U', L.'X', L.'0', L.'Z'}, {L.'Z', L.'Z', L.'Z', L.'Z'}}, {{L.'U', L.'U', L.'U', L.'U'}, {L.'U', L.'X', L.'X', L.'X'}, {L.'U', L.'X', L.'Z', L.'1'}, {L.'Z', L.'Z', L.'Z', L.'Z'}}, {{L.'U', L.'U', L.'U', L.'U'}, {L.'U', L.'W', L.'W', L.'W'}, {L.'U', L.'W', L.'L', L.'H'}, {L.'Z', L.'Z', L.'Z', L.'Z'}}, {{L.'U', L.'U', L.'U', L.'U'}, {L.'U', L.'W', L.'W', L.'Z'}, {L.'U', L.'W', L.'L', L.'Z'}, {L.'Z', L.'Z', L.'Z', L.'Z'}}, {{L.'U', L.'U', L.'U', L.'U'}, {L.'U', L.'W', L.'W', L.'W'}, {L.'U', L.'W', L.'Z', L.'H'}, {L.'Z', L.'Z', L.'Z', L.'Z'}}, {{L.'U', L.'U', L.'U', L.'U'}, {L.'U', L.'W', L.'W', L.'W'}, {L.'U', L.'W', L.'0', L.'H'}, {L.'Z', L.'Z', L.'Z', L.'Z'}}, {{L.'U', L.'U', L.'U', L.'U'}, {L.'U', L.'W', L.'W', L.'W'}, {L.'U', L.'W', L.'L', L.'1'}, {L.'Z', L.'Z', L.'Z', L.'Z'}}}] "Tristate Table by [strength, enable, input] reading, low-active enable";
        constant D.Interfaces.Logic MUX2x1Table[R, R, R] = [{{{L.'U', L.'U', L.'U', L.'U'}, {L.'U', L.'U', L.'U', L.'U'}, {L.'U', L.'X', L.'0', L.'1'}, {L.'U', L.'U', L.'U', L.'U'}}, {{L.'U', L.'X', L.'U', L.'U'}, {L.'U', L.'X', L.'X', L.'X'}, {L.'U', L.'X', L.'0', L.'1'}, {L.'X', L.'X', L.'X', L.'X'}}, {{L.'U', L.'U', L.'0', L.'U'}, {L.'U', L.'X', L.'0', L.'X'}, {L.'U', L.'X', L.'0', L.'1'}, {L.'0', L.'0', L.'0', L.'0'}}, {{L.'U', L.'U', L.'U', L.'1'}, {L.'U', L.'X', L.'X', L.'1'}, {L.'U', L.'X', L.'0', L.'1'}, {L.'1', L.'1', L.'1', L.'1'}}}] "Multiplexer Table by [select, input1, input0] reading";
        annotation(
          Documentation(info = "<html>

</html>"));
      end Tables;

      package Delay "Delay blocks"
        extends Modelica.Icons.Package;

        partial block DelayParams "Definition of delay parameters"
          import L = Modelica.Electrical.Digital.Interfaces.Logic;
          parameter Modelica.SIunits.Time tLH(start = 0) "Rise inertial delay";
          parameter Modelica.SIunits.Time tHL(start = 0) "Fall inertial delay";
          parameter L y0 = L.'U' "Initial value of output";
          annotation(
            Documentation(info = "<html>
<p>DelayParams is a partial model for providing delay times and inertial values. It is used in components of the package Gates which need the same parameters. The partial model does not have any behavior or equations.</p>
</html>"));
        end DelayParams;

        model TransportDelay "Transport delay with initial parameter"
          import D = Modelica.Electrical.Digital;
          import L = Modelica.Electrical.Digital.Interfaces.Logic;
          extends D.Interfaces.SISO(x(start = L.'U', fixed = true));
          parameter Modelica.SIunits.Time delayTime(start = 0) "delay time";
          parameter D.Interfaces.Logic y0 = L.'U' "initial value of output";
          constant D.Interfaces.Logic LogicValues[:] = L.'U':L.'-';
        protected
          D.Interfaces.Logic x_delayed;
        equation
          x_delayed = LogicValues[integer(delay(Integer(pre(x)), delayTime))];
          y = if delayTime > 0 then if time >= delayTime then x_delayed else y0 else pre(x);
          annotation(
            Documentation(info = "<HTML>
<P>
Provide the input as output exactly delayed by <i>Tdel</i>.
If time less than <i>Tdel</i> the initial value <i>initout</i> holds.
</P>
</html>", revisions = "<HTML>
<dl>
<dt><i>August 11, 2003</i></dt>
<dd>by Christoph Clauss initially modelled.</dd>
</dl>
</HTML>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{152, -160}, {-148, -100}}, lineColor = {0, 0, 255}, textString = "%name"), Rectangle(extent = {{-50, 100}, {50, -100}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {213, 170, 255}, fillPattern = FillPattern.Solid), Polygon(points = {{-6, 60}, {-16, 40}, {4, 40}, {-6, 60}}, lineColor = {0, 0, 0}), Line(points = {{0, 60}, {20, 60}}, color = {0, 0, 0}), Line(points = {{10, 60}, {10, 40}}, color = {0, 0, 0}), Text(extent = {{-50, -40}, {50, -20}}, lineColor = {0, 0, 0}, lineThickness = 0.5, textString = "Transport"), Text(extent = {{-50, -60}, {50, -40}}, lineColor = {0, 0, 0}, lineThickness = 0.5, textString = "Delay")}));
        end TransportDelay;

        block InertialDelay "Inertial delay with initial parameter"
          import D = Modelica.Electrical.Digital;
          import L = Modelica.Electrical.Digital.Interfaces.Logic;
          extends D.Interfaces.SISO;
          parameter Modelica.SIunits.Time delayTime(start = 0) "Minimum time to hold value";
          parameter D.Interfaces.Logic y0 = L.'U' "Initial value of output y";
        protected
          D.Interfaces.Logic y_auxiliary(start = y0, fixed = true);
          D.Interfaces.Logic x_old(start = y0, fixed = true);
          discrete Modelica.SIunits.Time t_next(start = delayTime, fixed = true);
        algorithm
          when delayTime > 0 and change(x) then
            x_old := x;
            t_next := time + delayTime;
          elsewhen time >= t_next then
            y_auxiliary := x;
          end when;
          y := if delayTime > 0 then y_auxiliary else x;
          annotation(
            Documentation(info = "<HTML>
<P>
Provides the input as output delayed by <i>Tdel</i> if the input holds its value for a longer time than <i>Tdel</i>.
If time is less than <i>Tdel</i> the initial value <i>initout</i> holds.
</P>
</html>", revisions = "<HTML>
<dl>
<dt><i>August 12, 2003</i></dt>
<dd>by Christoph Clauss revised</dd>
<dt><i>March 19, 2003</i></dt>
<dd>by Martin Otter initially modelled.</dd>
</dl>
</HTML>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{152, -160}, {-148, -100}}, lineColor = {0, 0, 255}, textString = "%name"), Rectangle(extent = {{-50, 100}, {50, -100}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {213, 170, 255}, fillPattern = FillPattern.Solid), Polygon(points = {{-6, 60}, {-16, 40}, {4, 40}, {-6, 60}}, lineColor = {0, 0, 0}), Line(points = {{10, 60}, {10, 40}}, color = {0, 0, 0}), Line(points = {{0, 60}, {20, 60}}, color = {0, 0, 0}), Text(extent = {{-50, -40}, {50, -20}}, lineColor = {0, 0, 0}, lineThickness = 0.5, textString = "Inertial"), Text(extent = {{-50, -60}, {50, -40}}, lineColor = {0, 0, 0}, lineThickness = 0.5, textString = "Delay")}));
        end InertialDelay;

        model InertialDelaySensitive "Provide the input as output if it holds its value for a specific amount of time"
          import D = Modelica.Electrical.Digital;
          import L = Modelica.Electrical.Digital.Interfaces.Logic;
          extends D.Interfaces.SISO(x(start = L.'U', fixed = true), y(start = y0, fixed = true));
          parameter Modelica.SIunits.Time tLH(start = 0) "rise inertial delay";
          parameter Modelica.SIunits.Time tHL(start = 0) "fall inertial delay";
          parameter D.Interfaces.Logic y0 = L.'U' "initial value of output";
        protected
          constant Integer delayTable[:, :] = Modelica.Electrical.Digital.Tables.DelayTable "specification of delay according to signal change";
          Modelica.SIunits.Time delayTime;
          D.Interfaces.Logic y_auxiliary(start = y0, fixed = true);
          D.Interfaces.Logic y_old(start = y0, fixed = true);
          Integer lh;
          discrete Modelica.SIunits.Time t_next;
        algorithm
          when {initial(), (tLH > 0 or tHL > 0) and change(x) and not initial()} then
            y_old := if initial() or pre(y) == L.'U' then y0 else pre(y);
            lh := delayTable[y_old, x];
            delayTime := if lh > 0 then tLH else if lh < 0 then tHL else 0;
            t_next := time + delayTime;
            if lh == 0 or abs(delayTime) < Modelica.Constants.small then
              y_auxiliary := x;
            end if;
          elsewhen time >= t_next then
            y_auxiliary := x;
          end when;
          y := if tLH > 0 or tHL > 0 then y_auxiliary else x;
          annotation(
            Documentation(info = "<HTML>
<P>
Provides the input as output delayed by <i>Tdel</i> if the input holds its value for a longer time than <i>Tdel</i>.
If the time is less than <i>Tdel</i> the initial value <i>initout</i> holds.<br>
The delay <i>Tdel</i> depends on the values of the signal change. To calculate <i>Tdel</i>, the delaymap specified in
Digital.Tables is used. If the corresponding value is 1, then <i>tLH</i> is used, if it is -1, then <i>tHL</i>
is used, if it is zero, the input is not delayed.
</P>
</html>", revisions = "<HTML>
<ul>
<li><i>January 24, 2013  </i> Initial value for y set to y0
       by Kristin Majetta and Christoph Clauss<br>
       </li>
<li><i>September 8, 2009  </i> pre(y) and x are used to select <i>tHL</i> or <i>tLH</i>
       by Ulrich Donath<br>
       </li>
<li><i>January 13, 2005  </i> improved when-conditions and declaration of delayTable
       by Dynasim<br>
       </li>
<li><i>September 15, 2004  </i> color changed, names changed
       by Christoph Clauss<br>
       </li>
<li><i>May 12, 2004  </i> test <i>if Tdel=0</i> replaced
       by Christoph Clauss<br>
       </li>
<li><i>February 5, 2004 </i> handling of <i>tHL=0</i> or <i>tLH=0</i> revised
       by Christoph Clauss<br>
       </li>
<li><i>October 12, 2003   </i>
       by Christoph Clauss<br>
       initially modelled</li>
</ul>
</HTML>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{152, -160}, {-148, -100}}, lineColor = {0, 0, 255}, textString = "%name"), Rectangle(extent = {{-50, 100}, {50, -100}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {213, 170, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-50, -40}, {50, -20}}, lineColor = {0, 0, 0}, lineThickness = 0.5, textString = "Inertial"), Text(extent = {{-50, -60}, {50, -40}}, lineColor = {0, 0, 0}, lineThickness = 0.5, textString = "Delay"), Text(extent = {{-50, -80}, {50, -60}}, lineColor = {0, 0, 0}, lineThickness = 0.5, textString = "sensitive"), Polygon(points = {{-6, 60}, {-16, 40}, {4, 40}, {-6, 60}}, lineColor = {0, 0, 0}), Line(points = {{10, 60}, {10, 40}}, color = {0, 0, 0}), Line(points = {{0, 60}, {20, 60}}, color = {0, 0, 0})}));
        end InertialDelaySensitive;

        model InertialDelaySensitiveVector "Delay of a vector of digital signals"
          import D = Modelica.Electrical.Digital;
          import L = Modelica.Electrical.Digital.Interfaces.Logic;
          import S = Modelica.Electrical.Digital.Interfaces.Strength;
          import T = Modelica.Electrical.Digital.Tables;
          parameter Modelica.SIunits.Time tHL = 0 "High->Low delay";
          parameter Modelica.SIunits.Time tLH = 0 "Low->High delay";
          parameter Integer n(min = 1) = 1 "Data width";
          D.Interfaces.DigitalInput x[n] annotation(
            Placement(transformation(extent = {{-100, -12}, {-76, 12}}), iconTransformation(extent = {{-100, -15}, {-72, 14}})));
          D.Interfaces.DigitalOutput y[n] annotation(
            Placement(transformation(extent = {{72, -14}, {100, 14}}), iconTransformation(extent = {{72, -14}, {100, 14}})));
          Digital.Delay.InertialDelaySensitive inertialDelaySensitive[n](each tLH = tLH, each tHL = tHL);
        equation
          for i in 1:n loop
            connect(x[i], inertialDelaySensitive[i].x);
            connect(inertialDelaySensitive[i].y, y[i]);
          end for;
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-50, 54}, {48, -54}}, lineColor = {127, 0, 127}, lineThickness = 0.5, fillPattern = FillPattern.Solid, fillColor = {255, 255, 255}), Line(points = {{-80, 0}, {-50, 0}}, color = {127, 0, 127}, thickness = 1, smooth = Smooth.None), Line(points = {{48, 0}, {80, 0}}, color = {127, 0, 127}, thickness = 1, smooth = Smooth.None), Text(extent = {{-40, 52}, {38, -20}}, lineColor = {127, 33, 107}, textString = "DELAY"), Text(extent = {{-26, 12}, {18, -32}}, lineColor = {127, 0, 127}, textString = "signal "), Text(extent = {{-32, 0}, {28, -62}}, lineColor = {127, 0, 127}, textString = "sensitive")}),
            Documentation(info = "<html>

<p>
The delay element <b>Inertial Delay Sensitive</b> is applied to a vector of n signals.
The parameters <i>tLH</i> and <i>tHL</i> are valid for each of the n signals.
</p>
</html>", revisions = "<html>
<ul>
<li><i>September 11, 2009 </i> created by Ulrich Donath<br>
       </li>
</ul>
</html>"));
        end InertialDelaySensitiveVector;
        annotation(
          Documentation(info = "<html>
<p>The Delay package collects the delay blocks which are used in many components. Both transport and inertial delay are offered for scalar connectors. The most advanced component is the sensitive inertial delay the delay time of which is chosen in dependency of the signal values. This component is also available for vector valued input.</p>
</html>"));
      end Delay;

      package Basic "Basic logic blocks without delays"
        extends Modelica.Icons.Package;

        model Not "Not logic component without delay"
          import D = Modelica.Electrical.Digital;
          import L = Modelica.Electrical.Digital.Interfaces.Logic;
          extends D.Interfaces.SISO;
        protected
          D.Interfaces.Logic auxiliary(start = L.'0', fixed = true);
        equation
          auxiliary = Modelica.Electrical.Digital.Tables.NotTable[x];
          y = pre(auxiliary);
          annotation(
            Documentation(info = "<html>
<p>Not component with 1 input value, without delay.</p>
<p>According to the standard logic not table (Tables.NotTable) the output value is calculated.</p>
<p>To avoid loops in the numerical treatment, the pre operator is applied to the output.</p>
</html>", revisions = "<HTML>
<dl>
<dt><i>August 14, 2003</i></dt>
<dd>by Teresa Schlegel initially modelled.</dd>
</dl>
</HTML>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-50, 40}, {50, 80}}, lineColor = {0, 0, 0}, lineThickness = 0.5, textString = "1"), Text(extent = {{152, -160}, {-148, -100}}, lineColor = {0, 0, 255}, textString = "%name"), Ellipse(extent = {{50, 6}, {62, -6}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid)}));
        end Not;

        model And "And logic component with multiple input and one output"
          import D = Modelica.Electrical.Digital;
          import L = Modelica.Electrical.Digital.Interfaces.Logic;
          extends D.Interfaces.MISO;
        protected
          D.Interfaces.Logic auxiliary[n](each start = L.'U');
          D.Interfaces.Logic auxiliary_n(start = L.'U', fixed = true);
        equation
          auxiliary[1] = x[1];
          for i in 1:n - 1 loop
            auxiliary[i + 1] = Modelica.Electrical.Digital.Tables.AndTable[auxiliary[i], x[i + 1]];
          end for;
          auxiliary_n = auxiliary[n];
          y = pre(auxiliary_n);
          annotation(
            Documentation(info = "<html>
<p>And component with multiple input values and one output.</p>
<p>According to the standard logic and table (Tables.AndTable) the output value is calculated.</p>
<p>To avoid loops in the numerical treatment, the pre operator is applied to the output.</p>
</html>", revisions = "<HTML>
<ul>
<li><i>September 15, 2004</i> vector approach used for all fixed numbers of inputs
       by Christoph Clauss<br>
       </li>
<li><i>October 22, 2003</i>
       by  Teresa Schlegel<br>
       initially modelled.</li>
</ul>
</HTML>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-50, 40}, {50, 80}}, lineColor = {0, 0, 0}, lineThickness = 0.5, textString = "&"), Text(extent = {{152, -160}, {-148, -100}}, lineColor = {0, 0, 255}, textString = "%name")}));
        end And;

        model Nand "Nand logic component with multiple input and one output"
          import D = Modelica.Electrical.Digital;
          import L = Modelica.Electrical.Digital.Interfaces.Logic;
          extends D.Interfaces.MISO;
        protected
          D.Interfaces.Logic auxiliary[n](each start = L.'U');
          D.Interfaces.Logic auxiliary_n(start = L.'U', fixed = true);
        equation
          auxiliary[1] = x[1];
          for i in 1:n - 1 loop
            auxiliary[i + 1] = Modelica.Electrical.Digital.Tables.AndTable[auxiliary[i], x[i + 1]];
          end for;
          auxiliary_n = Modelica.Electrical.Digital.Tables.NotTable[auxiliary[n]];
          y = pre(auxiliary_n);
          annotation(
            Documentation(info = "<html>
<p>Nand component with multiple input values and one output.</p>
<p>According to the standard logic and table (Tables.AndTable) an intermediate value is calculated, to which the not table (Tables.NotTable) is applied.</p>
<p>To avoid loops in the numerical treatment, the pre operator is applied to the output.</p>
</html>", revisions = "<HTML>
<ul>
<li><i>September 15, 2004</i> vector approach used for all fixed numbers of inputs
       by Christoph Clauss<br>
       </li>
<li><i>October 22, 2003</i>
       by  Teresa Schlegel<br>
       initially modelled.</li>
</ul>
</HTML>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-50, 40}, {50, 80}}, lineColor = {0, 0, 0}, lineThickness = 0.5, textString = "&"), Text(extent = {{152, -160}, {-148, -100}}, lineColor = {0, 0, 255}, textString = "%name"), Ellipse(extent = {{50, 6}, {62, -6}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid)}));
        end Nand;

        model Or "Or logic component with multiple input and one output"
          import D = Modelica.Electrical.Digital;
          import L = Modelica.Electrical.Digital.Interfaces.Logic;
          extends D.Interfaces.MISO;
        protected
          D.Interfaces.Logic auxiliary[n](each start = L.'U');
          D.Interfaces.Logic auxiliary_n(start = L.'U', fixed = true);
        equation
          auxiliary[1] = x[1];
          for i in 1:n - 1 loop
            auxiliary[i + 1] = Modelica.Electrical.Digital.Tables.OrTable[auxiliary[i], x[i + 1]];
          end for;
          auxiliary_n = auxiliary[n];
          y = pre(auxiliary_n);
          annotation(
            Documentation(info = "<html>
<p>Or component with multiple input values and one output.</p>
<p>According to the standard logic or table (Tables.OrTable) the output value is calculated.</p>
<p>To avoid loops in the numerical treatment, the pre operator is applied to the output.</p>
</html>", revisions = "<HTML>
<ul>
<li><i>September 15, 2004</i> vector approach used for all fixed numbers of inputs
       by Christoph Clauss<br>
       </li>
<li><i>October 22, 2003</i>
       by  Teresa Schlegel<br>
       initially modelled.</li>
</ul>
</HTML>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-50, 40}, {50, 80}}, lineColor = {0, 0, 0}, lineThickness = 0.5, textString = ">=1"), Text(extent = {{152, -160}, {-148, -100}}, lineColor = {0, 0, 255}, textString = "%name")}));
        end Or;

        model Nor "Nor logic component with multiple input and one output"
          import D = Modelica.Electrical.Digital;
          import L = Modelica.Electrical.Digital.Interfaces.Logic;
          extends D.Interfaces.MISO;
          //  D.Interfaces.Logic auxiliary[n](each start=L.'U', each fixed=true);
        protected
          D.Interfaces.Logic auxiliary[n](each start = L.'U');
          D.Interfaces.Logic auxiliary_n(start = L.'U', fixed = true);
        equation
          auxiliary[1] = x[1];
          for i in 1:n - 1 loop
            auxiliary[i + 1] = Modelica.Electrical.Digital.Tables.OrTable[auxiliary[i], x[i + 1]];
          end for;
          auxiliary_n = Modelica.Electrical.Digital.Tables.NotTable[auxiliary[n]];
          y = pre(auxiliary_n);
          annotation(
            Documentation(info = "<html>
<p>Nor component with multiple input values and one output.</p>
<p>According to the standard logic or table (Tables.OrTable) an intermediate value is calculated, to which the not table (Tables.NotTable) is applied.</p>
<p>To avoid loops in the numerical treatment, the pre operator is applied to the output.</p>
</html>", revisions = "<HTML>
<ul>
<li><i>September 15, 2004</i> vector approach used for all fixed numbers of inputs
       by Christoph Clauss<br>
       </li>
<li><i>October 22, 2003</i>
       by Liane Jacobi<br>
       initially modelled.</li>
</ul>
</HTML>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-50, 40}, {50, 80}}, lineColor = {0, 0, 0}, lineThickness = 0.5, textString = ">=1"), Text(extent = {{152, -160}, {-148, -100}}, lineColor = {0, 0, 255}, textString = "%name"), Ellipse(extent = {{50, 6}, {62, -6}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid)}));
        end Nor;

        model Xor "Xor logic component with multiple input and one output"
          import D = Modelica.Electrical.Digital;
          import L = Modelica.Electrical.Digital.Interfaces.Logic;
          extends D.Interfaces.MISO;
        protected
          D.Interfaces.Logic auxiliary[n](each start = L.'U');
          D.Interfaces.Logic auxiliary_n(start = L.'U', fixed = true);
        equation
          auxiliary[1] = x[1];
          for i in 1:n - 1 loop
            auxiliary[i + 1] = Modelica.Electrical.Digital.Tables.XorTable[auxiliary[i], x[i + 1]];
          end for;
          auxiliary_n = auxiliary[n];
          y = pre(auxiliary_n);
          annotation(
            Documentation(info = "<html>
<p>Xor component with multiple input values and one output.</p>
<p>According to the standard logic xor table (Tables.XorTable) the output value is calculated.</p>
<p>To avoid loops in the numerical treatment, the pre operator is applied to the output.</p>
</html>", revisions = "<HTML>
<ul>
<li><i>September 15, 2004</i> vector approach used for all fixed numbers of inputs
       by Christoph Clauss<br>
       </li>
<li><i>October 22, 2003</i>
       by Liane Jacobi<br>
       initially modelled.</li>
</ul>
</HTML>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-50, 40}, {50, 80}}, lineColor = {0, 0, 0}, lineThickness = 0.5, textString = "=1"), Text(extent = {{152, -160}, {-148, -100}}, lineColor = {0, 0, 255}, textString = "%name")}));
        end Xor;

        model Xnor "Xnor logic component with multiple input and one output"
          import D = Modelica.Electrical.Digital;
          import L = Modelica.Electrical.Digital.Interfaces.Logic;
          extends D.Interfaces.MISO;
        protected
          D.Interfaces.Logic auxiliary[n](each start = L.'U');
          D.Interfaces.Logic auxiliary_n(start = L.'U', fixed = true);
        equation
          auxiliary[1] = x[1];
          for i in 1:n - 1 loop
            auxiliary[i + 1] = Modelica.Electrical.Digital.Tables.XorTable[auxiliary[i], x[i + 1]];
          end for;
          auxiliary_n = Modelica.Electrical.Digital.Tables.NotTable[auxiliary[n]];
          y = pre(auxiliary_n);
          annotation(
            Documentation(info = "<html>
<p>Xnor component with multiple input values and one output.</p>
<p>According to the standard logic xor table (Tables.XorTable)an intermediate value is calculated, to which the not table (Tables.NotTable) is applied.</p>
<p>To avoid loops in the numerical treatment, the pre operator is applied to the output.</p>
</html>", revisions = "<HTML>
<ul>
<li><i>September 15, 2004</i> vector approach used for all fixed numbers of inputs
       by Christoph Clauss<br>
       </li>
<li><i>October 22, 2003</i>
       by Liane Jacobi<br>
       initially modelled.</li>
</ul>
</HTML>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-50, 40}, {50, 80}}, lineColor = {0, 0, 0}, lineThickness = 0.5, textString = "="), Text(extent = {{152, -160}, {-148, -100}}, lineColor = {0, 0, 255}, textString = "%name")}));
        end Xnor;
        annotation(
          Documentation(info = "<html>
<p>Basic contains the basic gates according to standard logic. The components of Basic calculate their results using the corresponding truth tables. They do not contain any delay components.</p>
</html>"));
      end Basic;

      package Gates "Logic gates including delays"
        extends Modelica.Icons.Package;

        model InvGate "InvGate with 1 input value, composed by Not and sensitive inertial delay"
          import D = Modelica.Electrical.Digital;
          extends D.Delay.DelayParams;
          extends D.Interfaces.SISO;
          D.Basic.Not G1 annotation(
            Placement(transformation(extent = {{-60, -20}, {-20, 20}}, rotation = 0)));
          D.Delay.InertialDelaySensitive G2(tLH = tLH, tHL = tHL) annotation(
            Placement(transformation(extent = {{20, -20}, {60, 20}}, rotation = 0)));
        equation
          connect(G2.y, y) annotation(
            Line(points = {{60, 0}, {100, 0}}, color = {127, 0, 127}));
          connect(G1.x, x) annotation(
            Line(points = {{-52, 0}, {-60, 0}}, color = {127, 0, 127}));
          connect(G1.y, G2.x) annotation(
            Line(points = {{-20, 0}, {28, 0}}, color = {127, 0, 127}));
          annotation(
            Documentation(info = "<html>
<p>The InvGate model has a single valued input, and a single valued output. It is composed by a Basic Not and an InertialDelaySensitive. Its parameters are the delay parameters (rise and fall inertial delay time, and initial value).</p>
</html>", revisions = "<HTML>
<dl>
<dt><i>August 14, 2003</i></dt>
<dd>by Teresa Schlegel initially modelled.</dd>
</dl>
</HTML>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{152, -160}, {-148, -100}}, lineColor = {0, 0, 255}, textString = "%name"), Rectangle(extent = {{-50, 100}, {50, -100}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {213, 170, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-50, 80}, {50, 40}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 191, 127}, fillPattern = FillPattern.Solid, textString = "1"), Text(extent = {{-18, -60}, {20, -100}}, lineColor = {0, 0, 0}, textString = "Gate"), Ellipse(extent = {{50, 6}, {62, -6}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid)}));
        end InvGate;

        model AndGate "AndGate with multiple input"
          import D = Modelica.Electrical.Digital;
          extends D.Interfaces.MISO;
          extends D.Delay.DelayParams;
          D.Basic.And G1(final n = n) annotation(
            Placement(transformation(extent = {{-40, -20}, {0, 20}}, rotation = 0)));
          D.Delay.InertialDelaySensitive G2(tLH = tLH, tHL = tHL, y0 = y0) annotation(
            Placement(transformation(extent = {{20, -20}, {60, 20}}, rotation = 0)));
        equation
          connect(x, G1.x) annotation(
            Line(points = {{-60, 0}, {-32, 0}}, color = {127, 0, 127}));
          connect(y, G2.y) annotation(
            Line(points = {{100, 0}, {60, 0}}, color = {127, 0, 127}));
          connect(G1.y, G2.x) annotation(
            Line(points = {{0, 0}, {28, 0}}, color = {127, 0, 127}));
          annotation(
            Documentation(info = "<html>
<p>The AndGate model has a multiple valued (n) input vector, and a single valued output. It is composed by a Basic And and an InertialDelaySensitive. Its parameters are the delay parameters (rise and fall inertial delay time, and initial value).</p>
</html>", revisions = "<HTML>
<ul>
<li><i>September 15, 2004</i> vector approach used for all fixed numbers of inputs
       by Christoph Clauss<br>
       </li>
<li><i>October 22, 2003</i>
       by Teresa Schlegel<br>
       initially modelled.</li>
</ul>
</HTML>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{152, -160}, {-148, -100}}, lineColor = {0, 0, 255}, textString = "%name"), Rectangle(extent = {{-50, 100}, {50, -100}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {213, 170, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-50, 80}, {50, 40}}, lineColor = {0, 0, 0}, lineThickness = 0.5, textString = "&"), Text(extent = {{-20, -60}, {20, -100}}, lineColor = {0, 0, 0}, lineThickness = 0.5, textString = "Gate")}));
        end AndGate;

        model NandGate "NandGate with multiple input"
          import D = Modelica.Electrical.Digital;
          extends D.Delay.DelayParams;
          extends D.Interfaces.MISO;
          D.Basic.Nand G1(final n = n) annotation(
            Placement(transformation(extent = {{-40, -20}, {0, 20}}, rotation = 0)));
          D.Delay.InertialDelaySensitive G2(tLH = tLH, tHL = tHL, y0 = y0) annotation(
            Placement(transformation(extent = {{20, -20}, {60, 20}}, rotation = 0)));
        equation
          connect(x, G1.x) annotation(
            Line(points = {{-60, 0}, {-32, 0}}, color = {127, 0, 127}));
          connect(G1.y, G2.x) annotation(
            Line(points = {{0, 0}, {28, 0}}, color = {127, 0, 127}));
          connect(G2.y, y) annotation(
            Line(points = {{60, 0}, {100, 0}}, color = {127, 0, 127}));
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{152, -160}, {-148, -100}}, lineColor = {0, 0, 255}, textString = "%name"), Rectangle(extent = {{-50, 100}, {50, -100}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {213, 170, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-40, 40}, {40, 80}}, lineColor = {0, 0, 0}, lineThickness = 0.5, textString = "&"), Text(extent = {{-50, -64}, {50, -86}}, lineColor = {0, 0, 0}, lineThickness = 0.5, textString = "Gate"), Ellipse(extent = {{50, 6}, {62, -6}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid)}),
            Documentation(revisions = "<HTML>
<ul>
<li><i>September 15, 2004</i> vector approach used for all fixed numbers of inputs
       by Christoph Clauss<br>
       </li>
<li><i>October 22, 2003</i>
       by Teresa Schlegel<br>
       initially modelled.</li>
</ul>
</HTML>", info = "<html>
<p>The NandGate model has a multiple valued (n) input vector, and a single valued output. It is composed by a Basic Nand and an InertialDelaySensitive. Its parameters are the delay parameters (rise and fall inertial delay time, and initial value).</p>
</html>"));
        end NandGate;

        model OrGate "OrGate with multiple input"
          import D = Modelica.Electrical.Digital;
          extends D.Delay.DelayParams;
          extends D.Interfaces.MISO;
          D.Basic.Or G1(final n = n) annotation(
            Placement(transformation(extent = {{-40, -20}, {0, 20}}, rotation = 0)));
          D.Delay.InertialDelaySensitive G2(tLH = tLH, tHL = tHL, y0 = y0) annotation(
            Placement(transformation(extent = {{20, -20}, {60, 20}}, rotation = 0)));
        equation
          connect(G1.y, G2.x) annotation(
            Line(points = {{0, 0}, {28, 0}}, color = {127, 0, 127}));
          connect(x, G1.x) annotation(
            Line(points = {{-60, 0}, {-32, 0}}, color = {127, 0, 127}));
          connect(y, G2.y) annotation(
            Line(points = {{100, 0}, {60, 0}}, color = {127, 0, 127}));
          annotation(
            Documentation(info = "<html>
<p>The OrGate model has a multiple valued (n) input vector, and a single valued output. It is composed by a Basic Or and an InertialDelaySensitive. Its parameters are the delay parameters (rise and fall inertial delay time, and initial value).</p>
</html>", revisions = "<HTML>
<ul>
<li><i>September 15, 2004</i> vector approach used for all fixed numbers of inputs
       by Christoph Clauss<br>
       </li>
<li><i>October 22, 2003</i>
       by Teresa Schlegel<br>
       initially modelled.</li>
</ul>
</HTML>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{152, -160}, {-148, -100}}, lineColor = {0, 0, 255}, textString = "%name"), Rectangle(extent = {{-50, 100}, {50, -100}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {213, 170, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-50, 80}, {50, 40}}, lineColor = {0, 0, 0}, lineThickness = 0.5, textString = ">=1"), Text(extent = {{-20, -60}, {20, -100}}, lineColor = {0, 0, 0}, lineThickness = 0.5, textString = "Gate")}));
        end OrGate;

        model NorGate "NorGate with multiple input"
          import D = Modelica.Electrical.Digital;
          extends D.Delay.DelayParams;
          extends D.Interfaces.MISO;
          D.Basic.Nor G1(final n = n) annotation(
            Placement(transformation(extent = {{-40, -20}, {0, 20}}, rotation = 0)));
          D.Delay.InertialDelaySensitive G2(tLH = tLH, tHL = tHL, y0 = y0) annotation(
            Placement(transformation(extent = {{20, -20}, {60, 20}}, rotation = 0)));
        equation
          connect(G1.y, G2.x) annotation(
            Line(points = {{0, 0}, {28, 0}}, color = {127, 0, 127}));
          connect(x, G1.x) annotation(
            Line(points = {{-60, 0}, {-32, 0}}, color = {127, 0, 127}));
          connect(y, G2.y) annotation(
            Line(points = {{100, 0}, {60, 0}}, color = {127, 0, 127}));
          annotation(
            Documentation(info = "<html>
<p>The NorGate model has a multiple valued (n) input vector, and a single valued output. It is composed by a Basic Nor and an InertialDelaySensitive. Its parameters are the delay parameters (rise and fall inertial delay time, and initial value).</p>
</html>", revisions = "<HTML>
<ul>
<li><i>September 15, 2004</i> vector approach used for all fixed numbers of inputs
       by Christoph Clauss<br>
       </li>
<li><i>October 22, 2003</i>
       by Liane Jacobi<br>
       initially modelled.</li>
</ul>
</HTML>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{152, -160}, {-148, -100}}, lineColor = {0, 0, 255}, textString = "%name"), Rectangle(extent = {{-50, 100}, {50, -100}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {213, 170, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-50, 80}, {50, 40}}, lineColor = {0, 0, 0}, lineThickness = 0.5, textString = ">=1"), Text(extent = {{-20, -60}, {20, -100}}, lineColor = {0, 0, 0}, lineThickness = 0.5, textString = "Gate"), Ellipse(extent = {{50, 6}, {62, -6}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid)}));
        end NorGate;

        model XorGate "XorGate with multiple input"
          import D = Modelica.Electrical.Digital;
          extends D.Delay.DelayParams;
          extends D.Interfaces.MISO;
          D.Basic.Xor G1(final n = n) annotation(
            Placement(transformation(extent = {{-40, -20}, {0, 20}}, rotation = 0)));
          D.Delay.InertialDelaySensitive G2(tLH = tLH, tHL = tHL, y0 = y0) annotation(
            Placement(transformation(extent = {{20, -20}, {60, 20}}, rotation = 0)));
        equation
          connect(x, G1.x) annotation(
            Line(points = {{-60, 0}, {-32, 0}}, color = {127, 0, 127}));
          connect(G1.y, G2.x) annotation(
            Line(points = {{0, 0}, {28, 0}}, color = {127, 0, 127}));
          connect(G2.y, y) annotation(
            Line(points = {{60, 0}, {100, 0}}, color = {127, 0, 127}));
          annotation(
            Documentation(info = "<html>
<p>The XorGate model has a multiple valued (n) input vector, and a single valued output. It is composed by a Basic Xor and an InertialDelaySensitive. Its parameters are the delay parameters (rise and fall inertial delay time, and initial value).</p>
</html>", revisions = "<HTML>
<ul>
<li><i>September 15, 2004</i> vector approach used for all fixed numbers of inputs
       by Christoph Clauss<br>
       </li>
<li><i>October 22, 2003</i>
       by Liane Jacobi<br>
       initially modelled.</li>
</ul>
</HTML>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{152, -160}, {-148, -100}}, lineColor = {0, 0, 255}, textString = "%name"), Rectangle(extent = {{-50, 100}, {50, -100}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {213, 170, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-50, 80}, {50, 40}}, lineColor = {0, 0, 0}, lineThickness = 0.5, textString = "=1"), Text(extent = {{-20, -60}, {20, -100}}, lineColor = {0, 0, 0}, lineThickness = 0.5, textString = "Gate")}));
        end XorGate;

        model XnorGate "XnorGate with multiple input"
          import D = Modelica.Electrical.Digital;
          extends D.Delay.DelayParams;
          extends D.Interfaces.MISO;
          D.Basic.Xnor G1(final n = n) annotation(
            Placement(transformation(extent = {{-40, -20}, {0, 20}}, rotation = 0)));
          D.Delay.InertialDelaySensitive G2(tLH = tLH, tHL = tHL, y0 = y0) annotation(
            Placement(transformation(extent = {{20, -20}, {60, 20}}, rotation = 0)));
        equation
          connect(x, G1.x) annotation(
            Line(points = {{-60, 0}, {-32, 0}}, color = {127, 0, 127}));
          connect(G2.y, y) annotation(
            Line(points = {{60, 0}, {100, 0}}, color = {127, 0, 127}));
          connect(G1.y, G2.x) annotation(
            Line(points = {{0, 0}, {28, 0}}, color = {127, 0, 127}));
          annotation(
            Documentation(info = "<html>
<p>The XnorGate model has a multiple valued (n) input vector, and a single valued output. It is composed by a Basic Xnor and an InertialDelaySensitive. Its parameters are the delay parameters (rise and fall inertial delay time, and initial value).</p>
</html>", revisions = "<HTML>
<ul>
<li><i>September 15, 2004</i> vector approach used for all fixed numbers of inputs
       by Christoph Clauss<br>
       </li>
<li><i>October 22, 2003</i>
       by Liane Jacobi<br>
       initially modelled.</li>
</ul>
</HTML>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{152, -160}, {-148, -100}}, lineColor = {0, 0, 255}, textString = "%name"), Rectangle(extent = {{-50, 100}, {50, -100}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {213, 170, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-40, 80}, {40, 40}}, lineColor = {0, 0, 0}, lineThickness = 0.5, textString = "="), Text(extent = {{-20, -60}, {20, -100}}, lineColor = {0, 0, 0}, lineThickness = 0.5, textString = "Gate")}));
        end XnorGate;

        model BufGate "BufGate with 1 input value, composed by Not and sensitive inertial delay"
          import D = Modelica.Electrical.Digital;
          extends D.Delay.DelayParams;
          extends D.Interfaces.SISO;
          D.Delay.InertialDelaySensitive G1(tLH = tLH, tHL = tHL) annotation(
            Placement(transformation(extent = {{-30, -20}, {10, 20}}, rotation = 0)));
        equation
          connect(G1.y, y) annotation(
            Line(points = {{10, 0}, {100, 0}}, color = {127, 0, 127}));
          connect(G1.x, x) annotation(
            Line(points = {{-22, 0}, {-60, 0}}, color = {127, 0, 127}));
          annotation(
            Documentation(info = "<html>
<p>The BufGate model has a single valued input, and a single valued output. It consists of an InertialDelaySensitive only. Its parameters are the delay parameters (rise and fall inertial delay time, and initial value).</p>
</html>", revisions = "<HTML>
<dl>
<dt><i>September 21, 2004</i></dt>
<dd>by Andre Schneider initially modelled.</dd>
</dl>
</HTML>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{152, -160}, {-148, -100}}, lineColor = {0, 0, 255}, textString = "%name"), Rectangle(extent = {{-50, 100}, {50, -100}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {213, 170, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-18, -60}, {20, -100}}, lineColor = {0, 0, 0}, textString = "Gate"), Text(extent = {{-50, 80}, {50, 40}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 191, 127}, fillPattern = FillPattern.Solid, textString = "1")}));
        end BufGate;
        annotation(
          Documentation(info = "<html>
<p>Gates contains the basic gates according to standard logic as they are provided in the Basic package. Additionally they contain an InertialDelaySensitive component They are composed graphically, not using any equations.</p>
</html>"));
      end Gates;

      package Sources "Time-dependent digital signal sources"
        extends Modelica.Icons.SourcesPackage;

        block Set "Digital Set Source"
          import D = Modelica.Electrical.Digital;
          import L = Modelica.Electrical.Digital.Interfaces.Logic;
          parameter D.Interfaces.Logic x(start = L.'1') "Logic value to be set";
          D.Interfaces.DigitalOutput y annotation(
            Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
        algorithm
          y := x;
          annotation(
            Documentation(info = "<HTML>
<P>
Sets a nine valued digital signal, which is specified by the <i>setval</i> parameter.
</P>
<P>
To specify <i>setval</i>, the integer code has to be used.
</P>
<p><b>Code Table</b></p>

<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"><b>Logic value</b></td>
      <td valign=\"top\"><b>Integer code</b></td>
      <td valign=\"top\"><b>Meaning</b></td>
  </tr>

  <tr><td valign=\"top\">'U'</td> <td valign=\"top\">1</td> <td valign=\"top\">Uninitialized</td></tr>
  <tr><td valign=\"top\">'X'</td> <td valign=\"top\">2</td> <td valign=\"top\">Forcing Unknown</td></tr>
  <tr><td valign=\"top\">'0'</td> <td valign=\"top\">3</td> <td valign=\"top\">Forcing 0</td></tr>
  <tr><td valign=\"top\">'1'</td> <td valign=\"top\">4</td> <td valign=\"top\">Forcing 1</td></tr>
  <tr><td valign=\"top\">'Z'</td> <td valign=\"top\">5</td> <td valign=\"top\">High Impedance</td></tr>
  <tr><td valign=\"top\">'W'</td> <td valign=\"top\">6</td> <td valign=\"top\">Weak Unknown</td></tr>
  <tr><td valign=\"top\">'L'</td> <td valign=\"top\">7</td> <td valign=\"top\">Weak 0</td></tr>
  <tr><td valign=\"top\">'H'</td> <td valign=\"top\">8</td> <td valign=\"top\">Weak 1</td></tr>
  <tr><td valign=\"top\">'-'</td> <td valign=\"top\">9</td> <td valign=\"top\">Do not care</td></tr>
</table>

<P>
If the logic values are imported by <br><b> import L = Digital.Interfaces.Logic; </b> <br>
they can be used to specify the parameter, e.g., <b>L.'0'</b> for forcing 0.
</P>
</html>", revisions = "<HTML>
<dl>
<dt><i>August 20, 2003</i></dt>
<dd>by Teresa Schlegel initially modelled.</dd>
</dl>
</HTML>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-50, 100}, {50, -100}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {213, 255, 170}, fillPattern = FillPattern.Solid), Text(extent = {{-50, 40}, {50, 80}}, lineColor = {0, 0, 0}, lineThickness = 0.5, textString = "SET"), Text(extent = {{152, -160}, {-148, -100}}, lineColor = {0, 0, 255}, textString = "%name"), Line(points = {{50, 0}, {90, 0}}, color = {127, 0, 127})}));
        end Set;

        block Step "Digital Step Source"
          import D = Modelica.Electrical.Digital;
          import L = Modelica.Electrical.Digital.Interfaces.Logic;
          parameter D.Interfaces.Logic before(start = L.'0') "Logic value before step";
          parameter D.Interfaces.Logic after(start = L.'1') "Logic value after step";
          parameter Real stepTime(start = 1) "step time";
          D.Interfaces.DigitalOutput y annotation(
            Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
        algorithm
// assert(before >= L.min and before <= L.max, "Parameter is no logic value");
// assert(after >= L.min and after <= L.max, "Parameter is no logic value");
          if time >= stepTime then
            y := after;
          else
            y := before;
          end if;
          annotation(
            Documentation(info = "<HTML>
<P>
The step source output signal steps from the value <i>before</i> to the value <i>after</i>
at the time <i>stepTime</i>.
</P>
<P>
To specify the logic value parameters, the integer code has to be used.
</P>
<p><b>Code Table</b></p>
<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"><b>Logic value</b></td>
      <td valign=\"top\"><b>Integer code</b></td>
      <td valign=\"top\"><b>Meaning</b></td>
  </tr>

  <tr><td valign=\"top\">'U'</td> <td valign=\"top\">1</td> <td valign=\"top\">Uninitialized</td></tr>
  <tr><td valign=\"top\">'X'</td> <td valign=\"top\">2</td> <td valign=\"top\">Forcing Unknown</td></tr>
  <tr><td valign=\"top\">'0'</td> <td valign=\"top\">3</td> <td valign=\"top\">Forcing 0</td></tr>
  <tr><td valign=\"top\">'1'</td> <td valign=\"top\">4</td> <td valign=\"top\">Forcing 1</td></tr>
  <tr><td valign=\"top\">'Z'</td> <td valign=\"top\">5</td> <td valign=\"top\">High Impedance</td></tr>
  <tr><td valign=\"top\">'W'</td> <td valign=\"top\">6</td> <td valign=\"top\">Weak Unknown</td></tr>
  <tr><td valign=\"top\">'L'</td> <td valign=\"top\">7</td> <td valign=\"top\">Weak 0</td></tr>
  <tr><td valign=\"top\">'H'</td> <td valign=\"top\">8</td> <td valign=\"top\">Weak 1</td></tr>
  <tr><td valign=\"top\">'-'</td> <td valign=\"top\">9</td> <td valign=\"top\">Do not care</td></tr>
</table>
<P>
If the logic values are imported by <br><b> import L = Digital.Interfaces.Logic; </b> <br>
they can be used to specify the parameter, e.g., <b>L.'0'</b> for forcing 0.
</P>
</html>", revisions = "<HTML>
<dl>
<dt><i>August 20, 2003</i></dt>
<dd>by Teresa Schlegel initially modelled.</dd>
</dl>
</HTML>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-50, 100}, {50, -100}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {213, 255, 170}, fillPattern = FillPattern.Solid), Text(extent = {{152, -160}, {-148, -100}}, lineColor = {0, 0, 255}, textString = "%name"), Line(points = {{-30, 40}, {0, 40}, {0, 80}, {30, 80}}, color = {0, 0, 0}), Line(points = {{50, 0}, {90, 0}}, color = {127, 0, 127})}));
        end Step;

        block Table "Digital Tabular Source"
          import D = Modelica.Electrical.Digital;
          import L = Modelica.Electrical.Digital.Interfaces.Logic;
          parameter D.Interfaces.Logic x[:] = {L.'1'} "vector of values";
          parameter Real t[size(x, 1)] = {1} "vector of corresponding time points";
          parameter D.Interfaces.Logic y0 = L.'U' "initial output value";
          final parameter Integer n = size(x, 1) "table size";
          D.Interfaces.DigitalOutput y annotation(
            Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
        algorithm
          if initial() then
            assert(n > 0, "Invalid size of table (n < 1)");
            for i in 1:n loop
            end for;
          end if;
// assert(y0 >= L.min and y0 <= L.max, "Parameter y0 is not of type Logic");
// assert(x[i] >= L.min and x[i] <= L.max, "Table element is not of type Logic");
          y := y0;
          for i in 1:n loop
            if time >= t[i] then
              y := x[i];
            end if;
          end for;
          annotation(
            Documentation(info = "<HTML>
<P>
The table source output signal <i>y</i> steps to the values of the  <i>x</i> table at the corresponding
timepoints in the <i>t</i> table. <br>The initial value is specified by <i>y0</i>.
</P>
<P>
To specify the logic value parameters, the integer code has to be used.
</P>
<p><b>Code Table</b></p>
<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"><b>Logic value</b></td>
      <td valign=\"top\"><b>Integer code</b></td>
      <td valign=\"top\"><b>Meaning</b></td>
  </tr>

  <tr><td valign=\"top\">'U'</td> <td valign=\"top\">1</td> <td valign=\"top\">Uninitialized</td></tr>
  <tr><td valign=\"top\">'X'</td> <td valign=\"top\">2</td> <td valign=\"top\">Forcing Unknown</td></tr>
  <tr><td valign=\"top\">'0'</td> <td valign=\"top\">3</td> <td valign=\"top\">Forcing 0</td></tr>
  <tr><td valign=\"top\">'1'</td> <td valign=\"top\">4</td> <td valign=\"top\">Forcing 1</td></tr>
  <tr><td valign=\"top\">'Z'</td> <td valign=\"top\">5</td> <td valign=\"top\">High Impedance</td></tr>
  <tr><td valign=\"top\">'W'</td> <td valign=\"top\">6</td> <td valign=\"top\">Weak Unknown</td></tr>
  <tr><td valign=\"top\">'L'</td> <td valign=\"top\">7</td> <td valign=\"top\">Weak 0</td></tr>
  <tr><td valign=\"top\">'H'</td> <td valign=\"top\">8</td> <td valign=\"top\">Weak 1</td></tr>
  <tr><td valign=\"top\">'-'</td> <td valign=\"top\">9</td> <td valign=\"top\">Do not care</td></tr>
</table>
<P>
If the logic values are imported by <br><b> import L = Digital.Interfaces.Logic; </b> <br>
they can be used to specify the parameter, e.g., <b>L.'0'</b> for forcing 0.
</P>
</html>", revisions = "<HTML>
<dl>
<dt><i>August 20, 2003</i></dt>
<dd>by Teresa Schlegel initially modelled.</dd>
</dl>
</HTML>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-50, 100}, {50, -100}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {213, 255, 170}, fillPattern = FillPattern.Solid), Text(extent = {{152, -160}, {-148, -100}}, lineColor = {0, 0, 255}, textString = "%name"), Line(points = {{50, 0}, {90, 0}}, color = {127, 0, 127}), Rectangle(extent = {{-30, 80}, {32, 70}}, lineColor = {0, 0, 0}, fillColor = {215, 215, 215}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-30, 70}, {32, 40}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{0, 80}, {0, 40}}, color = {0, 0, 0}), Line(points = {{-30, 60}, {32, 60}}, color = {0, 0, 0}), Line(points = {{-30, 50}, {32, 50}}, color = {0, 0, 0})}));
        end Table;

        model Pulse "Digital Pulse Source"
          import D = Modelica.Electrical.Digital;
          import L = Modelica.Electrical.Digital.Interfaces.Logic;
          parameter Real width(final min = Modelica.Constants.small, final max = 100, start = 50) "Widths of pulses in % of periods";
          parameter Modelica.SIunits.Time period(final min = Modelica.Constants.small, start = 1) "Time for one period";
          parameter Modelica.SIunits.Time startTime(start = 0) "Output = quiet for time < startTime";
          parameter D.Interfaces.Logic pulse(start = L.'0') "pulsed value";
          parameter D.Interfaces.Logic quiet(start = L.'1') "quiet value";
          Modelica.SIunits.Time T0(final start = startTime, fixed = true) "Start time of current period";
          parameter Integer nperiod(start = -1) "Number of periods (< 0 means infinite number of periods)";
          Integer np(start = 0, fixed = true);
          D.Interfaces.DigitalOutput y annotation(
            Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
        protected
          Boolean sampling;
        equation
          sampling = nperiod <> 0 and (nperiod >= pre(np) or nperiod < 0);
          when sampling and sample(startTime, period) then
            T0 = time;
            np = if nperiod > 0 then pre(np) + 1 else pre(np);
          end when;
          if sampling then
            y = if time < startTime or time >= T0 + width * period / 100 then quiet else pulse;
          else
            y = quiet;
          end if;
          annotation(
            Documentation(info = "<HTML>
<P>
The pulse source forms pulses between the  <i>quiet</i> value and the <i>pulse</i> value.
The pulse length <i>width</i> is specified in percent of the period length <i>period</i>.
The number of periods is specified by <i>nperiod</i>. If <i>nperiod</i> is less than zero,
the number of periods is unlimited.
</P>
<P>
To specify the logic value parameters, the integer code has to be used.
</P>
<p><b>Code Table</b></p>
<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"><b>Logic value</b></td>
      <td valign=\"top\"><b>Integer code</b></td>
      <td valign=\"top\"><b>Meaning</b></td>
  </tr>

  <tr><td valign=\"top\">'U'</td> <td valign=\"top\">1</td> <td valign=\"top\">Uninitialized</td></tr>
  <tr><td valign=\"top\">'X'</td> <td valign=\"top\">2</td> <td valign=\"top\">Forcing Unknown</td></tr>
  <tr><td valign=\"top\">'0'</td> <td valign=\"top\">3</td> <td valign=\"top\">Forcing 0</td></tr>
  <tr><td valign=\"top\">'1'</td> <td valign=\"top\">4</td> <td valign=\"top\">Forcing 1</td></tr>
  <tr><td valign=\"top\">'Z'</td> <td valign=\"top\">5</td> <td valign=\"top\">High Impedance</td></tr>
  <tr><td valign=\"top\">'W'</td> <td valign=\"top\">6</td> <td valign=\"top\">Weak Unknown</td></tr>
  <tr><td valign=\"top\">'L'</td> <td valign=\"top\">7</td> <td valign=\"top\">Weak 0</td></tr>
  <tr><td valign=\"top\">'H'</td> <td valign=\"top\">8</td> <td valign=\"top\">Weak 1</td></tr>
  <tr><td valign=\"top\">'-'</td> <td valign=\"top\">9</td> <td valign=\"top\">Do not care</td></tr>
</table>
<P>
If the logic values are imported by <br><b> import L = Digital.Interfaces.Logic; </b> <br>
they can be used to specify the parameter, e.g., <b>L.'0'</b> for forcing 0.
</P>
</html>", revisions = "<HTML>
<dl>
<dt><i>September 2, 2003</i></dt>
<dd>by Christoph Clauss initially modelled.</dd>
</dl>
</HTML>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-50, 100}, {50, -100}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {213, 255, 170}, fillPattern = FillPattern.Solid), Text(extent = {{152, -160}, {-148, -100}}, lineColor = {0, 0, 255}, textString = "%name"), Line(points = {{50, 0}, {90, 0}}, color = {127, 0, 127}), Line(points = {{-36, 40}, {-30, 40}, {-30, 40}, {-18, 40}, {-18, 40}, {-14, 40}, {-14, 80}, {14, 80}, {14, 40}, {14, 40}, {14, 40}, {28, 40}, {30, 40}, {36, 40}}, color = {0, 0, 0})}));
        end Pulse;

        model Clock "Digital Clock Source"
          import D = Modelica.Electrical.Digital;
          import L = Modelica.Electrical.Digital.Interfaces.Logic;
          parameter Modelica.SIunits.Time startTime(start = 0) "Output = offset for time < startTime";
          parameter Modelica.SIunits.Time period(final min = Modelica.Constants.small, start = 1) "Time for one period";
          parameter Real width(final min = Modelica.Constants.small, final max = 100, start = 50) "Width of pulses in % of period";
          D.Interfaces.DigitalOutput y "Connector of Digital output signal" annotation(
            Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
        protected
          Modelica.SIunits.Time t_i(final start = startTime, fixed = true) "Start time of current period";
          Modelica.SIunits.Time t_width = period * width / 100;
        equation
          when sample(startTime, period) then
            t_i = time;
          end when;
          y = if not time >= startTime or time >= t_i + t_width then L.'0' else L.'1';
          annotation(
            Documentation(info = "<HTML>
<P>
The clock source forms pulses between the  <i>'0'</i> value (forcing 0) and the <i>'1'</i> value (forcing 1).
The pulse length <i>width</i> is specified in percent of the period length <i>period</i>.
The number of periods is unlimited. The first pulse starts at startTime.
</P>
<P> The clock source is a special but often used variant of the pulse source.
</P>
</html>", revisions = "<HTML>
<dl>
<dt><i>September 18, 2004</i></dt>
<dd>by Andre Schneider initially modelled.</dd>
</dl>
</HTML>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-50, 100}, {50, -100}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {213, 255, 170}, fillPattern = FillPattern.Solid), Text(extent = {{152, -160}, {-148, -100}}, lineColor = {0, 0, 255}, textString = "%name"), Line(points = {{50, 0}, {90, 0}}, color = {127, 0, 127}), Line(points = {{-36, 40}, {-30, 40}, {-30, 80}, {-18, 80}, {-18, 40}, {-6, 40}, {-6, 80}, {6, 80}, {6, 40}, {18, 40}, {18, 80}, {30, 80}, {30, 40}, {36, 40}}, color = {0, 0, 0})}));
        end Clock;
        annotation(
          Documentation(info = "<html>
<p>The sources are not taken from Standard Logic. They were added since they turned out to be quite useful, since such sources are often needed. For a better optical overview the colour green was chosen for the sources.</p>
</html>"));
      end Sources;

      package Converters "Converters between 2-,3-,4- and 9-valued logic"
        extends Modelica.Icons.Package;

        block LogicToX01 "Conversion to X01"
          import D = Modelica.Electrical.Digital;
          import T = Modelica.Electrical.Digital.Tables;
          D.Interfaces.DigitalInput x[n] annotation(
            Placement(transformation(extent = {{-60, -10}, {-40, 10}}, rotation = 0)));
          D.Interfaces.DigitalOutput y[n] annotation(
            Placement(transformation(extent = {{40, -10}, {60, 10}}, rotation = 0)));
          parameter Integer n(final min = 1, start = 1) "signal width";
        algorithm
          for i in 1:n loop
            y[i] := T.X01Table[x[i]];
          end for;
          annotation(
            Documentation(info = "<HTML>
<P>
Conversion of a nine valued digital input into a X01 digital output without
any delay according to IEEE 1164 To_X01 function.
</p>
<p><b>Conversion Table:</b></p>
<pre>
  input                  output
  'U' (coded by 1)       'X'  (coded by 2)
  'X' (coded by 2)       'X'  (coded by 2)
  '0' (coded by 3)       '0'  (coded by 3)
  '1' (coded by 4)       '1'  (coded by 4)
  'Z' (coded by 5)       'X'  (coded by 2)
  'W' (coded by 6)       'X'  (coded by 2)
  'L' (coded by 7)       '0'  (coded by 3)
  'H' (coded by 8)       '1'  (coded by 4)
  '-' (coded by 9)       'X'  (coded by 2)
</pre>
<P>
If the signal width is greater than 1 this conversion is done for each signal.
</P>
</html>", revisions = "<HTML>
<ul>
<li><i>September 15, 2004   </i>
       by Christoph Clauss name converted from cvt_to_x01 into LogicToX01<br>
       </li>
<li><i>November 5, 2003</i>
       by Christoph Clauss<br>
       initially modelled.</li>
</ul>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{152, -100}, {-148, -40}}, lineColor = {0, 0, 255}, textString = "%name"), Polygon(points = {{-40, -40}, {-40, 40}, {40, 40}, {-40, -40}}, lineColor = {127, 0, 127}, fillColor = {127, 0, 127}, fillPattern = FillPattern.Solid), Polygon(points = {{-40, -40}, {40, -40}, {40, 40}, {-40, -40}}, lineColor = {127, 0, 127}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Text(extent = {{0, -20}, {40, -40}}, lineColor = {0, 0, 0}, lineThickness = 0.5, textString = "X01"), Rectangle(extent = {{-40, 40}, {40, -40}}, lineColor = {0, 0, 0}, lineThickness = 0.5)}));
        end LogicToX01;

        block LogicToX01Z "Conversion to X01Z"
          import D = Modelica.Electrical.Digital;
          import T = Modelica.Electrical.Digital.Tables;
          D.Interfaces.DigitalInput x[n] annotation(
            Placement(transformation(extent = {{-60, -10}, {-40, 10}}, rotation = 0)));
          D.Interfaces.DigitalOutput y[n] annotation(
            Placement(transformation(extent = {{40, -10}, {60, 10}}, rotation = 0)));
          parameter Integer n(final min = 1, start = 1) "signal width";
        algorithm
          for i in 1:n loop
            y[i] := T.X01ZTable[x[i]];
          end for;
          annotation(
            Documentation(info = "<HTML>
<P>
Conversion of a nine valued digital input into a X01Z digital output without
any delay according to IEEE 1164 To_X01Z function.
</P>
<p><b>Conversion Table:</b></p>
<pre>
 input                  output
 'U' (coded by 1)       'X'  (coded by 2)
 'X' (coded by 2)       'X'  (coded by 2)
 '0' (coded by 3)       '0'  (coded by 3)
 '1' (coded by 4)       '1'  (coded by 4)
 'Z' (coded by 5)       'Z'  (coded by 5)
 'W' (coded by 6)       'X'  (coded by 2)
 'L' (coded by 7)       '0'  (coded by 3)
 'H' (coded by 8)       '1'  (coded by 4)
 '-' (coded by 9)       'X'  (coded by 2)
</pre>
<P>
If the signal width is greater than 1 this conversion is done for each signal.
</P>
</html>", revisions = "<HTML>
<ul>
<li><i>September 15, 2004   </i>
       by Christoph Clauss name converted from cvt_to_x01z into LogicToX01Z<br>
       </li>
<li><i>November 5, 2003</i>
       by Christoph Clauss<br>
       initially modelled.</li>
</ul>
</HTML>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{152, -100}, {-148, -40}}, lineColor = {0, 0, 255}, textString = "%name"), Polygon(points = {{-40, -40}, {-40, 40}, {40, 40}, {-40, -40}}, lineColor = {127, 0, 127}, fillColor = {127, 0, 127}, fillPattern = FillPattern.Solid), Polygon(points = {{-40, -40}, {40, -40}, {40, 40}, {-40, -40}}, lineColor = {127, 0, 127}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Text(extent = {{0, -20}, {40, -40}}, lineColor = {0, 0, 0}, lineThickness = 0.5, textString = "X01Z"), Rectangle(extent = {{-40, 40}, {40, -40}}, lineColor = {0, 0, 0}, lineThickness = 0.5)}));
        end LogicToX01Z;

        block LogicToUX01 "Conversion to UX01"
          import D = Modelica.Electrical.Digital;
          import T = Modelica.Electrical.Digital.Tables;
          D.Interfaces.DigitalInput x[n] annotation(
            Placement(transformation(extent = {{-60, -10}, {-40, 10}}, rotation = 0)));
          D.Interfaces.DigitalOutput y[n] annotation(
            Placement(transformation(extent = {{40, -10}, {60, 10}}, rotation = 0)));
          parameter Integer n(final min = 1, start = 1) "signal width";
        algorithm
          for i in 1:n loop
            y[i] := T.UX01Table[x[i]];
          end for;
          annotation(
            Documentation(info = "<HTML>
<P>
Conversion of a nine valued digital input into a UX01 digital output without
any delay according to IEEE 1164 To_UX01 function.
</P>
<p><b>Conversion Table:</b></p>
<pre>
 input                  output
 'U' (coded by 1)       'U'  (coded by 1)
 'X' (coded by 2)       'X'  (coded by 2)
 '0' (coded by 3)       '0'  (coded by 3)
 '1' (coded by 4)       '1'  (coded by 4)
 'Z' (coded by 5)       'X'  (coded by 2)
 'W' (coded by 6)       'X'  (coded by 2)
 'L' (coded by 7)       '0'  (coded by 3)
 'H' (coded by 8)       '1'  (coded by 4)
 '-' (coded by 9)       'X'  (coded by 2)
</pre>
<P>
If the signal width is greater than 1 this conversion is done for each signal.
</P>
</html>", revisions = "<HTML>
<ul>
<li><i>September 15, 2004   </i>
       by Christoph Clauss name converted from cvt_to_ux01 into LogicToUX01<br>
       </li>
<li><i>November 5, 2003</i>
       by Christoph Clauss<br>
       initially modelled.</li>
</ul>
</HTML>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{152, -100}, {-148, -40}}, lineColor = {0, 0, 255}, textString = "%name"), Polygon(points = {{-40, -40}, {-40, 40}, {40, 40}, {-40, -40}}, lineColor = {127, 0, 127}, fillColor = {127, 0, 127}, fillPattern = FillPattern.Solid), Polygon(points = {{-40, -40}, {40, -40}, {40, 40}, {-40, -40}}, lineColor = {127, 0, 127}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Text(extent = {{0, -20}, {40, -40}}, lineColor = {0, 0, 0}, lineThickness = 0.5, textString = "UX01"), Rectangle(extent = {{-40, 40}, {40, -40}}, lineColor = {0, 0, 0}, lineThickness = 0.5)}));
        end LogicToUX01;

        block BooleanToLogic "Boolean to Logic converter"
          import L = Modelica.Electrical.Digital.Interfaces.Logic;
          Modelica.Blocks.Interfaces.BooleanInput x[n] annotation(
            Placement(transformation(extent = {{-60, -10}, {-40, 10}}, rotation = 0)));
          parameter Integer n(final min = 1, start = 1) "signal width";
          Modelica.Electrical.Digital.Interfaces.DigitalOutput y[n] annotation(
            Placement(transformation(extent = {{40, -10}, {60, 10}}, rotation = 0)));
        equation
          for i in 1:n loop
            y[i] = if x[i] then L.'1' else L.'0';
          end for;
          annotation(
            Documentation(info = "<HTML>
<P>
Conversion of a Boolean input into a digital output without any delay according to:
</P>
<pre>
 input      output
 true       '1'  (coded by 4)
 false      '0'  (coded by 3)
</pre>
<P>
If the signal width is greater than 1 this conversion is done for each signal.
</P>
</html>", revisions = "<HTML>
<ul>
<li><i>September 15, 2004   </i>
       by Christoph Clauss colors changed<br>
       </li>
<li><i>November 4, 2003</i>
       by Christoph Clauss<br>
       initially modelled.</li>
</ul>
</HTML>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{-40, -40}, {40, 40}, {40, -40}, {-40, -40}}, lineColor = {127, 0, 127}, fillColor = {127, 0, 127}, fillPattern = FillPattern.Solid), Polygon(points = {{-40, 40}, {40, 40}, {-40, -40}, {-40, 40}}, lineColor = {255, 0, 255}, fillColor = {255, 0, 255}, fillPattern = FillPattern.Solid), Text(extent = {{152, -100}, {-148, -40}}, lineColor = {0, 0, 255}, textString = "%name"), Rectangle(extent = {{-40, 40}, {40, -40}}, lineColor = {0, 0, 0}, lineThickness = 0.5)}));
        end BooleanToLogic;

        block LogicToBoolean "Logic to Boolean converter"
          import L = Modelica.Electrical.Digital.Interfaces.Logic;
          Modelica.Electrical.Digital.Interfaces.DigitalInput x[n] annotation(
            Placement(transformation(extent = {{-60, -10}, {-40, 10}}, rotation = 0)));
          Modelica.Blocks.Interfaces.BooleanOutput y[n] annotation(
            Placement(transformation(extent = {{40, -10}, {60, 10}}, rotation = 0)));
          parameter Integer n(final min = 1, start = 2) "signal width";
        equation
          for i in 1:n loop
            y[i] = if x[i] == L.'1' or x[i] == L.'H' then true else false;
          end for;
          annotation(
            Documentation(info = "<HTML>
<P>
Conversion of a digital input into a Boolean output without any delay according to:
</P>
<pre>
 input                 output
 'U'  (coded by 1)     false
 'X'  (coded by 2)     false
 '0'  (coded by 3)     false
 '1'  (coded by 4)     true
 'Z'  (coded by 5)     false
 'W'  (coded by 6)     false
 'L'  (coded by 7)     false
 'H'  (coded by 8)     true
 '-'  (coded by 9)     false
</pre>
<P>
If the signal width is greater than 1 this conversion is done for each signal.
</P>
</html>", revisions = "<HTML>
<ul>
<li><i>September 15, 2004   </i>
       by Christoph Clauss colors changed<br>
       </li>
<li><i>November 4, 2003</i>
       by Christoph Clauss<br>
       initially modelled.</li>
</ul>
</HTML>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{-40, 40}, {40, 40}, {-40, -40}, {-40, 40}}, lineColor = {127, 0, 127}, fillColor = {127, 0, 127}, fillPattern = FillPattern.Solid), Text(extent = {{152, -100}, {-148, -40}}, lineColor = {0, 0, 255}, textString = "%name"), Polygon(points = {{-40, -40}, {40, -40}, {40, 40}, {-40, -40}}, lineColor = {255, 0, 255}, fillColor = {255, 0, 255}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-40, 40}, {40, -40}}, lineColor = {0, 0, 0}, lineThickness = 0.5)}));
        end LogicToBoolean;

        block RealToLogic "Real to Logic converter"
          import L = Modelica.Electrical.Digital.Interfaces.Logic;
          Modelica.Blocks.Interfaces.RealInput x[n] annotation(
            Placement(transformation(extent = {{-60, -10}, {-40, 10}}, rotation = 0)));
          Digital.Interfaces.DigitalOutput y[n] annotation(
            Placement(transformation(extent = {{40, -10}, {60, 10}}, rotation = 0)));
          parameter Integer n(final min = 1, start = 1) "signal width";
          parameter Real upper_limit(start = 1) "upper limit";
          parameter Real lower_limit(start = 0) "lower limit";
          parameter L upper_value(start = L.'1') "output if input > upper_limit";
          parameter L lower_value(start = L.'0') "output if input < lower_limit";
          parameter L middle_value(start = L.'X') "output else";
        equation
          for i in 1:n loop
            y[i] = if x[i] > upper_limit then upper_value else if x[i] < lower_limit then lower_value else middle_value;
          end for;
          annotation(
            Documentation(info = "<HTML>
<P>
Conversion of a real input into a digital output without any delay according to:
</P>
<pre>
                                 condition            output
      first check:               input greater upp    lupp
      second check:              input larger low     llow
                                 else                 lmid
</pre>
<P>
If the signal width is greater than 1 this conversion is done for each signal.
</P>
</html>", revisions = "<HTML>
<ul>
<li><i>September 15, 2004   </i>
       by Christoph Clauss colors changed<br>
       </li>
<li><i>November 5, 2003</i>
       by Christoph Clauss<br>
       initially modelled.</li>
</ul>
</HTML>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{-40, -40}, {40, 40}, {40, -40}, {-40, -40}}, lineColor = {127, 0, 127}, fillColor = {127, 0, 127}, fillPattern = FillPattern.Solid), Polygon(points = {{-40, 40}, {40, 40}, {-40, -40}, {-40, 40}}, lineColor = {0, 0, 255}), Text(extent = {{148, -100}, {-152, -40}}, lineColor = {0, 0, 255}, textString = "%name"), Rectangle(extent = {{-40, 40}, {40, -40}}, lineColor = {0, 0, 0}, lineThickness = 0.5)}));
        end RealToLogic;

        block LogicToReal "Logic to Real converter"
          import L = Modelica.Electrical.Digital.Interfaces.Logic;
          Modelica.Electrical.Digital.Interfaces.DigitalInput x[n] annotation(
            Placement(transformation(extent = {{-60, -10}, {-40, 10}}, rotation = 0)));
          Modelica.Blocks.Interfaces.RealOutput y[n] annotation(
            Placement(transformation(extent = {{40, -10}, {60, 10}}, rotation = 0)));
          parameter Integer n(final min = 1, start = 1) "signal width";
          parameter Real value_U(start = 0.5) "value for digital U (uninitialized)";
          parameter Real value_X(start = 0.5) "value for digital X (Forcing Unknown)";
          parameter Real value_0(start = 0) "value for digital 0 (Forcing 0)";
          parameter Real value_1(start = 1) "value for digital 1 (Forcing 1)";
          parameter Real value_Z(start = 0.5) "value for digital Z (High Impedance)";
          parameter Real value_W(start = 0.5) "value for digital W (Weak    Unknown)";
          parameter Real value_L(start = 0) "value for digital L (Weak 0)";
          parameter Real value_H(start = 1) "value for digital H (Weak 1)";
          parameter Real value_m(start = 0.5) "value for digital m (Do not care)";
        equation
          for i in 1:n loop
            y[i] = if x[i] == L.'U' then value_U else if x[i] == L.'X' then value_X else if x[i] == L.'0' then value_0 else if x[i] == L.'1' then value_1 else if x[i] == L.'Z' then value_Z else if x[i] == L.'W' then value_W else if x[i] == L.'L' then value_L else if x[i] == L.'H' then value_H else value_m;
          end for;
          annotation(
            Documentation(info = "<HTML>
<P>
Conversion of a digital input into a Real output without any delay according to:
</P>
<pre>
  input                 output
  'U'  (coded by 1)     val_U
  'X'  (coded by 2)     val_X
  '0'  (coded by 3)     val_0
  '1'  (coded by 4)     val_1
  'Z'  (coded by 5)     val_Z
  'W'  (coded by 6)     val_W
  'L'  (coded by 7)     val_L
  'H'  (coded by 8)     val_H
  '-'  (coded by 9)     val_m
</pre>
<P>
The values val... are given by parameters.</P>
<P>If the signal width is greater than 1 this conversion is done for each signal.
</P>
</html>", revisions = "<HTML>
<ul>
<li><i>September 15, 2004   </i>
       by Christoph Clauss colors changed<br>
       </li>
<li><i>November 5, 2003</i>
       by Christoph Clauss<br>
       initially modelled.</li>
</ul>
</HTML>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{-40, 40}, {40, 40}, {-40, -40}, {-40, 40}}, lineColor = {127, 0, 127}, fillColor = {127, 0, 127}, fillPattern = FillPattern.Solid), Polygon(points = {{-40, -40}, {40, -40}, {40, 40}, {-40, -40}}, lineColor = {0, 0, 255}), Text(extent = {{152, -100}, {-148, -40}}, lineColor = {0, 0, 255}, textString = "%name"), Rectangle(extent = {{-40, 40}, {40, -40}}, lineColor = {0, 0, 0}, lineThickness = 0.5)}));
        end LogicToReal;

        block LogicToXO1 "This model will be removed in future Modelica versions, use 'LogicToX01' instead!"
          // extends Modelica.Icons.ObsoleteModel;
          import D = Modelica.Electrical.Digital;
          import T = Modelica.Electrical.Digital.Tables;
          D.Interfaces.DigitalInput x[n] annotation(
            Placement(transformation(extent = {{-60, -10}, {-40, 10}}, rotation = 0)));
          D.Interfaces.DigitalOutput y[n] annotation(
            Placement(transformation(extent = {{40, -10}, {60, 10}}, rotation = 0)));
          parameter Integer n(final min = 1, start = 1) "signal width";
        algorithm
          for i in 1:n loop
            y[i] := T.X01Table[x[i]];
          end for;
          annotation(
            Documentation(info = "<HTML>
<h4>Obsoletion Note</h4>
<p>This model will be removed in future versions of the Modelica Standard Library. Instead the model <a href=\"modelica://Modelica.Electrical.Digital.Converters.LogicToX01\">LogicToX01</a> shall be used.</p>
<hr>
<P>
Conversion of a nine valued digital input into a X01 digital output without
any delay according to IEEE 1164 To_X01 function.
</p>
<p><b>Conversion Table:</b></p>
<pre>
  input                  output
  'U' (coded by 1)       'X'  (coded by 2)
  'X' (coded by 2)       'X'  (coded by 2)
  '0' (coded by 3)       '0'  (coded by 3)
  '1' (coded by 4)       '1'  (coded by 4)
  'Z' (coded by 5)       'X'  (coded by 2)
  'W' (coded by 6)       'X'  (coded by 2)
  'L' (coded by 7)       '0'  (coded by 3)
  'H' (coded by 8)       '1'  (coded by 4)
  '-' (coded by 9)       'X'  (coded by 2)
</pre>
<P>
If the signal width is greater than 1 this conversion is done for each signal.
</P>
</html>", revisions = "<HTML>
<ul>
<li><i>September 15, 2004   </i>
       by Christoph Clauss name converted from cvt_to_x01 into LogicToX01<br>
       </li>
<li><i>November 5, 2003</i>
       by Christoph Clauss<br>
       initially modelled.</li>
</ul>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{152, -100}, {-148, -40}}, lineColor = {0, 0, 255}, textString = "%name"), Polygon(points = {{-40, -40}, {-40, 40}, {40, 40}, {-40, -40}}, lineColor = {127, 0, 127}, fillColor = {127, 0, 127}, fillPattern = FillPattern.Solid), Polygon(points = {{-40, -40}, {40, -40}, {40, 40}, {-40, -40}}, lineColor = {127, 0, 127}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Text(extent = {{0, -20}, {40, -40}}, lineColor = {0, 0, 0}, lineThickness = 0.5, textString = "X01"), Rectangle(extent = {{-40, 40}, {40, -40}}, lineColor = {0, 0, 0}, lineThickness = 0.5)}));
        end LogicToXO1;

        block LogicToXO1Z "This model will be removed in future Modelica versions, use 'LogicToX01Z' instead!"
          // extends Modelica.Icons.ObsoleteModel;
          import D = Modelica.Electrical.Digital;
          import T = Modelica.Electrical.Digital.Tables;
          D.Interfaces.DigitalInput x[n] annotation(
            Placement(transformation(extent = {{-60, -10}, {-40, 10}}, rotation = 0)));
          D.Interfaces.DigitalOutput y[n] annotation(
            Placement(transformation(extent = {{40, -10}, {60, 10}}, rotation = 0)));
          parameter Integer n(final min = 1, start = 1) "signal width";
        algorithm
          for i in 1:n loop
            y[i] := T.X01ZTable[x[i]];
          end for;
          annotation(
            Documentation(info = "<HTML>
<h4>Obsoletion Note</h4>
<p>This model will be removed in future versions of the Modelica Standard Library. Instead the model <a href=\"modelica://Modelica.Electrical.Digital.Converters.LogicToX01Z\">LogicToX01Z</a> shall be used.</p>
<hr>
<P>
Conversion of a nine valued digital input into a X01Z digital output without
any delay according to IEEE 1164 To_X01Z function.
</P>
<p><b>Conversion Table:</b></p>
<pre>
 input                  output
 'U' (coded by 1)       'X'  (coded by 2)
 'X' (coded by 2)       'X'  (coded by 2)
 '0' (coded by 3)       '0'  (coded by 3)
 '1' (coded by 4)       '1'  (coded by 4)
 'Z' (coded by 5)       'Z'  (coded by 5)
 'W' (coded by 6)       'X'  (coded by 2)
 'L' (coded by 7)       '0'  (coded by 3)
 'H' (coded by 8)       '1'  (coded by 4)
 '-' (coded by 9)       'X'  (coded by 2)
</pre>
<P>
If the signal width is greater than 1 this conversion is done for each signal.
</P>
</html>", revisions = "<HTML>
<ul>
<li><i>September 15, 2004   </i>
       by Christoph Clauss name converted from cvt_to_x01z into LogicToX01Z<br>
       </li>
<li><i>November 5, 2003</i>
       by Christoph Clauss<br>
       initially modelled.</li>
</ul>
</HTML>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{152, -100}, {-148, -40}}, lineColor = {0, 0, 255}, textString = "%name"), Polygon(points = {{-40, -40}, {-40, 40}, {40, 40}, {-40, -40}}, lineColor = {127, 0, 127}, fillColor = {127, 0, 127}, fillPattern = FillPattern.Solid), Polygon(points = {{-40, -40}, {40, -40}, {40, 40}, {-40, -40}}, lineColor = {127, 0, 127}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Text(extent = {{0, -20}, {40, -40}}, lineColor = {0, 0, 0}, lineThickness = 0.5, textString = "X01Z"), Rectangle(extent = {{-40, 40}, {40, -40}}, lineColor = {0, 0, 0}, lineThickness = 0.5)}));
        end LogicToXO1Z;
        annotation(
          Documentation(info = "<html>
<p>The Converter BooleanToLogic, LogicToBoolean, RealToLogic, and LogicTo Real components are not standard logic components. They were designed to easily convert from or to Boolean or Real valued signals. The LogicToX01, LogicToX01Z and LogicTo UX01 converters correspond to standard logic functions. They transform 9-valued logic to 3- or 4-valued logic.</p>
</html>"));
      end Converters;

      package Registers "Registers with N-bit input data and output data"
        extends Modelica.Icons.Package;

        model DFFR "Edge triggered register bank with reset"
          import D = Modelica.Electrical.Digital;
          import L = Modelica.Electrical.Digital.Interfaces.Logic;
          import S = Modelica.Electrical.Digital.Interfaces.Strength;
          import T = Modelica.Electrical.Digital.Tables;
          parameter Integer ResetMap[9] = {1, 4, 3, 2, 4, 4, 3, 2, 4} "function selection, defaults for high active reset";
          parameter D.Interfaces.Strength strength = S.'S_X01' "output strength";
          parameter Integer n(min = 1) = 1 "data width";
          D.Interfaces.DigitalInput reset annotation(
            Placement(transformation(extent = {{-10, -100}, {10, -80}}), iconTransformation(extent = {{-10, -100}, {10, -80}})));
          D.Interfaces.DigitalInput clock annotation(
            Placement(transformation(extent = {{-90, -30}, {-70, -10}}), iconTransformation(extent = {{-90, -30}, {-70, -10}})));
          D.Interfaces.DigitalInput dataIn[n] annotation(
            Placement(transformation(extent = {{-90, 20}, {-70, 40}}), iconTransformation(extent = {{-90, 20}, {-70, 40}})));
          D.Interfaces.DigitalOutput dataOut[n] annotation(
            Placement(transformation(extent = {{64, 26}, {84, 46}}), iconTransformation(extent = {{69, 20}, {89, 40}})));
        protected
          Integer clock_flag(start = 0);
          // 0: 0-Transition
          // 1: rising edge
          // 2: X-Transition
          Integer reset_flag(start = 1);
          // 1: output := U
          // 2: output := 0
          // 3: output := -dataInUX
          // 4: output := U-0X
        protected
          D.Interfaces.Logic nextstate[n](start = fill(L.'U', n));
          D.Interfaces.Logic next_assign_val[n](start = fill(L.'U', n));
        algorithm
          if change(clock) or change(reset) then
            if change(clock) then
              if initial() then
                clock_flag := T.ClockMap[L.'U', clock];
              else
                clock_flag := T.ClockMap[pre(clock), clock];
              end if;
            end if;
            reset_flag := ResetMap[reset];
            for i in 1:n loop
              if reset_flag == 1 then
                nextstate[i] := L.'U';
              elseif reset_flag == 2 then
                nextstate[i] := T.StrengthMap[L.'0', strength];
              elseif reset_flag == 3 then
                if clock_flag == 0 then
                  break;
                elseif clock_flag == 1 then
                  nextstate[i] := T.StrengthMap[dataIn[i], strength];
                else
                  if next_assign_val[i] == T.StrengthMap[dataIn[i], strength] or next_assign_val[i] == L.'U' then
                    break;
                  elseif dataIn[i] == L.'U' then
                    nextstate[i] := L.'U';
                  else
                    nextstate[i] := T.StrengthMap[L.'X', strength];
                  end if;
                end if;
              elseif reset_flag == 4 then
                if next_assign_val[i] == T.StrengthMap[L.'0', strength] and (dataIn[i] == L.'0' or dataIn[i] == L.'L' or clock_flag == 0) then
                  break;
                elseif (dataIn[i] == L.'0' or dataIn[i] == L.'L') and clock_flag == 1 then
                  nextstate[i] := T.StrengthMap[L.'0', strength];
                elseif next_assign_val[i] == L.'U' and not clock_flag == 1 or dataIn[i] == L.'U' and not clock_flag == 0 then
                  nextstate[i] := L.'U';
                else
                  nextstate[i] := T.StrengthMap[L.'X', strength];
                end if;
              end if;
            end for;
          end if;
          next_assign_val := nextstate;
          dataOut := nextstate;
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-50, 60}, {50, -60}}, lineColor = {127, 33, 107}, lineThickness = 0.5, fillPattern = FillPattern.Solid, fillColor = {255, 255, 255}), Line(points = {{-70, 30}, {-50, 30}}, color = {127, 33, 107}, thickness = 1, smooth = Smooth.None), Line(points = {{50, 30}, {76, 30}}, color = {127, 33, 107}, thickness = 1, smooth = Smooth.None), Line(points = {{-80, -20}, {-50, -20}}, color = {127, 33, 107}, smooth = Smooth.None), Line(points = {{0, -84}, {0, -60}}, color = {127, 33, 107}, smooth = Smooth.None), Line(points = {{-50, -8}, {-34, -20}, {-50, -32}}, color = {127, 33, 107}, smooth = Smooth.None), Text(extent = {{-30, 54}, {20, 8}}, lineColor = {127, 33, 107}, textString = "DFFR"), Text(extent = {{-32, -14}, {-6, -26}}, lineColor = {127, 33, 107}, textString = "clock"), Text(extent = {{-14, -44}, {12, -56}}, lineColor = {127, 33, 107}, textString = "reset")}),
            Documentation(info = "<html>
<p>Description in VHDL is given by http://www.cs.sfu.ca/~ggbaker/reference/std_logic/src/std_logic_entities</p>
<p><b>Truth Table for high active reset:</b></p>

<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"><b>DataIn</b></td>
      <td valign=\"top\"><b>Clock</b></td>
      <td valign=\"top\"><b>Reset</b></td>
      <td valign=\"top\"><b>DataOut</b></td>
      <td valign=\"top\">Map</td>
  </tr>

  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">U</td > <td valign=\"top\">U</td> <td valign=\"top\">1</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">1</td>  <td valign=\"top\">0</td> <td valign=\"top\">2</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">0-Trns</td> <td valign=\"top\">0</td>  <td valign=\"top\">NC</td> <td valign=\"top\">3</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">1-Trns</td> <td valign=\"top\">0</td>  <td valign=\"top\">DataIn</td> <td valign=\"top\">3</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">X-Trns</td> <td valign=\"top\">0</td>  <td valign=\"top\">X or U or NC</td> <td valign=\"top\">3</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">X</td> <td valign=\"top\">X or U or 0 or NC</td> <td valign=\"top\">4</td> </tr>
</table>

<p><b>Truth Table for low active reset:</b></p>

<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"><b>DataIn</b></td>
      <td valign=\"top\"><b>Clock</b></td>
      <td valign=\"top\"><b>Reset</b></td>
      <td valign=\"top\"><b>DataOut</b></td>
      <td valign=\"top\">Map</td>
  </tr>

  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">U</td > <td valign=\"top\">U</td> <td valign=\"top\">1</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">0</td>  <td valign=\"top\">0</td> <td valign=\"top\">2</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">0-Trns</td> <td valign=\"top\">1</td>  <td valign=\"top\">NC</td> <td valign=\"top\">3</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">1-Trns</td> <td valign=\"top\">1</td>  <td valign=\"top\">DataIn</td> <td valign=\"top\">3</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">X-Trns</td> <td valign=\"top\">1</td>  <td valign=\"top\">X or U or NC</td> <td valign=\"top\">3</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">X</td> <td valign=\"top\">X or U or 0 or NC</td> <td valign=\"top\">4</td> </tr>
</table>

<PRE>

  *  = do not care
  U  = L.'U'
  0  = L.'0' or L.'L'
  1  = L.'1' or L.'H'
  X  = L.'X' or L.'W' or L.'Z' or L.'-'
  NC = no change

Clock transition definitions:
  1-Trns: 0 -> 1
  0-Trns: ~ -> 0 or 1 -> * or X -> X|U or U -> X|U
  X-Trns: 0 -> X|U or X|U -> 1
</PRE>
</html>", revisions = "<html>
<ul>
<li><i>September 11, 2009 </i> created by Ulrich Donath<br>
       </li>
</ul>
</html>"));
        end DFFR;

        model DFFREG "Edge triggered register bank with high active reset"
          import D = Modelica.Electrical.Digital;
          import L = Modelica.Electrical.Digital.Interfaces.Logic;
          import S = Modelica.Electrical.Digital.Interfaces.Strength;
          import T = Modelica.Electrical.Digital.Tables;
          parameter Modelica.SIunits.Time tHL = 0 "High->Low delay";
          parameter Modelica.SIunits.Time tLH = 0 "Low->High delay";
          parameter D.Interfaces.Strength strength = S.'S_X01' "output strength";
          parameter Integer n(min = 1) = 1 "data width";
        protected
          constant Integer ResetMap[9] = {1, 4, 3, 2, 4, 4, 3, 2, 4};
          // Function selection by [reset] reading
          // 1: output := U
          // 2: output := 0
          // 3: output := -dataInUX
          // 4: output := U-0X
        public
          Modelica.Electrical.Digital.Delay.InertialDelaySensitiveVector delay(tHL = tHL, tLH = tLH, n = n) annotation(
            Placement(transformation(extent = {{22, 11}, {79, 69}})));
          D.Interfaces.DigitalInput reset annotation(
            Placement(transformation(extent = {{-100, -68}, {-84, -52}}), iconTransformation(extent = {{-100, -68}, {-84, -52}})));
          D.Interfaces.DigitalInput clock annotation(
            Placement(transformation(extent = {{-100, -28}, {-84, -12}})));
          D.Interfaces.DigitalInput dataIn[n] annotation(
            Placement(transformation(extent = {{-100, 32}, {-84, 48}})));
          D.Interfaces.DigitalOutput dataOut[n] annotation(
            Placement(transformation(extent = {{84, 32}, {100, 48}}), iconTransformation(extent = {{84, 32}, {100, 48}})));
          D.Registers.DFFR dFFR(n = n, ResetMap = ResetMap, strength = strength) annotation(
            Placement(transformation(extent = {{-78, -23}, {18, 74}})));
        equation
          connect(delay.y, dataOut) annotation(
            Line(points = {{75.01, 40}, {92, 40}}, color = {127, 0, 127}, smooth = Smooth.None, thickness = 1));
          connect(dataIn, dFFR.dataIn) annotation(
            Line(points = {{-92, 40}, {-68.4, 40}, {-68.4, 40.05}}, color = {127, 0, 127}, thickness = 1, smooth = Smooth.None));
          connect(dFFR.dataOut, delay.x) annotation(
            Line(points = {{7.92, 40.05}, {25.99, 39.855}}, color = {127, 0, 127}, thickness = 1, smooth = Smooth.None));
          connect(clock, dFFR.clock) annotation(
            Line(points = {{-92, -20}, {-68.4, -20}, {-68.4, 15.8}}, color = {127, 0, 127}, smooth = Smooth.None));
          connect(reset, dFFR.reset) annotation(
            Line(points = {{-92, -60}, {-30, -60}, {-30, -18.15}}, color = {127, 0, 127}, smooth = Smooth.None));
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-50, 70}, {50, -50}}, lineColor = {127, 0, 127}, lineThickness = 0.5, fillPattern = FillPattern.Solid, fillColor = {255, 255, 255}), Line(points = {{-86, 40}, {-50, 40}}, color = {127, 0, 127}, thickness = 1, smooth = Smooth.None), Line(points = {{-84, -20}, {-50, -20}}, color = {127, 0, 127}, smooth = Smooth.None), Line(points = {{-84, -60}, {0, -60}, {0, -50}}, color = {127, 0, 127}, smooth = Smooth.None), Line(points = {{50, 40}, {84, 40}}, color = {127, 0, 127}, smooth = Smooth.None, thickness = 1), Line(points = {{-50, -10}, {-36, -20}, {-50, -30}}, color = {127, 0, 127}, smooth = Smooth.None), Text(extent = {{-32, 70}, {30, 10}}, lineColor = {127, 33, 107}, textString = "DFFREG"), Text(extent = {{-14, -34}, {12, -46}}, lineColor = {127, 33, 107}, textString = "reset"), Text(extent = {{-28, -14}, {-2, -26}}, lineColor = {127, 33, 107}, textString = "clock"), Text(extent = {{-26, 44}, {26, 2}}, lineColor = {127, 33, 107}, textString = "high active")}),
            Documentation(info = "<html>
<p>Description in VHDL is given by http://www.cs.sfu.ca/~ggbaker/reference/std_logic/src/std_logic_entities.vhd</p>
<p><b>Truth Table</b></p>

<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"><b>DataIn</b></td>
      <td valign=\"top\"><b>Clock</b></td>
      <td valign=\"top\"><b>Reset</b></td>
      <td valign=\"top\"><b>DataOut</b></td>
  </tr>

  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">U</td > <td valign=\"top\">U</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">1</td>  <td valign=\"top\">0</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">0-Trns</td> <td valign=\"top\">0</td>  <td valign=\"top\">NC</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">1-Trns</td> <td valign=\"top\">0</td>  <td valign=\"top\">DataIn</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">X-Trns</td> <td valign=\"top\">0</td>  <td valign=\"top\">X or U or NC</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">X</td> <td valign=\"top\">X or U or 0 or NC</td> </tr>
</table>

<PRE>
  *  = do not care
  U  = L.'U'
  0  = L.'0' or L.'L'
  1  = L.'1' or L.'H'
  X  = L.'X' or L.'W' or L.'Z' or L.'-'
  NC = no change

Clock transition definitions:
  1-Trns: 0 -> 1
  0-Trns: ~ -> 0 or 1 -> * or X -> X|U or U -> X|U
  X-Trns: 0 -> X|U or X|U -> 1
</PRE>
</html>", revisions = "<html>
<ul>
<li><i>September 11, 2009 </i> created by Ulrich Donath<br>
       </li>
</ul>
</html>"));
        end DFFREG;

        model DFFREGL "Edge triggered register bank with low active reset"
          extends DFFREG(final ResetMap = {1, 4, 2, 3, 4, 4, 2, 3, 4});
          // Function selection by [reset] reading
          // 1: output := U
          // 2: output := 0
          // 3: output := -dataInUX
          // 4: output := U-0X;
          annotation(
            Documentation(info = "<html>
<p>Description in VHDL is given by http://www.cs.sfu.ca/~ggbaker/reference/std_logic/src/std_logic_entities.vhd</p>
<p><b>Truth Table</b></p>

<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"><b>DataIn</b></td>
      <td valign=\"top\"><b>Clock</b></td>
      <td valign=\"top\"><b>Reset</b></td>
      <td valign=\"top\"><b>DataOut</b></td>
  </tr>

  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">U</td > <td valign=\"top\">U</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">0</td>  <td valign=\"top\">0</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">0-Trns</td> <td valign=\"top\">1</td>  <td valign=\"top\">NC</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">1-Trns</td> <td valign=\"top\">1</td>  <td valign=\"top\">DataIn</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">X-Trns</td> <td valign=\"top\">1</td>  <td valign=\"top\">X or U or NC</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">X</td> <td valign=\"top\">X or U or 0 or NC</td> </tr>
</table>

<PRE>
  *  = do not care
  U  = L.'U'
  0  = L.'0' or L.'L'
  1  = L.'1' or L.'H'
  X  = L.'X' or L.'W' or L.'Z' or L.'-'
  NC = no change

Clock transition definitions:
  1-Trns: 0 -> 1
  0-Trns: ~ -> 0 or 1 -> * or X -> X|U or U -> X|U
  X-Trns: 0 -> X|U or X|U -> 1
</PRE>
</html>", revisions = "<html>
<ul>
<li><i>September 11, 2009 </i> created by Ulrich Donath<br>
       </li>
</ul>
</html>"),
            Icon(graphics = {Rectangle(extent = {{-30, 30}, {28, 16}}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, pattern = LinePattern.None), Text(extent = {{-28, 48}, {24, -1}}, lineColor = {127, 33, 107}, textString = "low active")}));
        end DFFREGL;

        model DFFSR "Edge triggered register bank with set and reset"
          import D = Modelica.Electrical.Digital;
          import L = Modelica.Electrical.Digital.Interfaces.Logic;
          import S = Modelica.Electrical.Digital.Interfaces.Strength;
          import T = Modelica.Electrical.Digital.Tables;
          parameter Integer ResetSetMap[9, 9] = [1, 1, 1, 1, 1, 1, 1, 1, 1; 1, 4, 7, 2, 4, 4, 7, 2, 4; 1, 5, 8, 2, 5, 5, 8, 2, 5; 1, 6, 3, 2, 6, 6, 3, 2, 6; 1, 4, 7, 2, 4, 4, 7, 2, 4; 1, 4, 7, 2, 4, 4, 7, 2, 4; 1, 5, 8, 2, 5, 5, 8, 2, 5; 1, 6, 3, 2, 6, 6, 3, 2, 6; 1, 4, 7, 2, 4, 4, 7, 2, 4] "function selection by [reset, set] reading";
          /* Defaults for set and reset are active high */
          parameter D.Interfaces.Strength strength = S.'S_X01' "output strength";
          parameter Integer n(min = 1) = 1 "data width";
          D.Interfaces.DigitalInput set annotation(
            Placement(transformation(extent = {{-10, 100}, {10, 80}})));
          D.Interfaces.DigitalInput reset annotation(
            Placement(transformation(extent = {{-10, -100}, {10, -80}}), iconTransformation(extent = {{-10, -100}, {10, -80}})));
          D.Interfaces.DigitalInput clock annotation(
            Placement(transformation(extent = {{-90, -20}, {-70, 0}}), iconTransformation(extent = {{-90, -30}, {-70, -10}})));
          D.Interfaces.DigitalInput dataIn[n] annotation(
            Placement(transformation(extent = {{-90, 20}, {-70, 40}}), iconTransformation(extent = {{-90, 20}, {-70, 40}})));
          D.Interfaces.DigitalOutput dataOut[n] annotation(
            Placement(transformation(extent = {{68, 20}, {88, 40}}), iconTransformation(extent = {{69, 20}, {89, 40}})));
        protected
          Integer clock_flag(start = 0);
          // 0: 0-Transition
          // 1: rising edge
          // 2: X-Transition
          Integer reset_set_flag(start = 1);
          // 1: output := U
          // 2: output := 1
          // 3: output := 0
          // 4: output := UX
          // 5: output := -1UX
          // 6: output := X
          // 7: output := -0UX
          // 8: output := -dataInUX
          D.Interfaces.Logic nextstate[n](start = fill(L.'U', n));
          D.Interfaces.Logic next_assign_val[n](start = fill(L.'U', n));
        algorithm
          if change(clock) or change(reset) or change(set) then
            if change(clock) then
              if initial() then
                clock_flag := T.ClockMap[L.'U', clock];
              else
                clock_flag := T.ClockMap[pre(clock), clock];
              end if;
            end if;
            reset_set_flag := ResetSetMap[reset, set];
            for i in 1:n loop
              if reset_set_flag == 1 then
                nextstate[i] := L.'U';
              elseif reset_set_flag == 2 then
                nextstate[i] := T.StrengthMap[L.'1', strength];
              elseif reset_set_flag == 3 then
                nextstate[i] := T.StrengthMap[L.'0', strength];
              elseif reset_set_flag == 4 then
                if next_assign_val[i] == L.'U' and clock_flag <> 1 or dataIn[i] == L.'U' and clock_flag <> 0 then
                  nextstate[i] := L.'U';
                else
                  nextstate[i] := T.StrengthMap[L.'X', strength];
                end if;
              elseif reset_set_flag == 5 then
                if next_assign_val[i] == T.StrengthMap[L.'1', strength] and (dataIn[i] == L.'1' or dataIn[i] == L.'H' or clock_flag == 0) then
                  break;
                elseif (dataIn[i] == L.'1' or dataIn[i] == L.'H') and clock_flag == 1 then
                  nextstate[i] := T.StrengthMap[L.'1', strength];
                elseif next_assign_val[i] == L.'U' and clock_flag <> 1 or dataIn[i] == L.'U' and clock_flag <> 0 then
                  nextstate[i] := L.'U';
                else
                  nextstate[i] := T.StrengthMap[L.'X', strength];
                end if;
              elseif reset_set_flag == 6 then
                nextstate[i] := T.StrengthMap[L.'X', strength];
              elseif reset_set_flag == 7 then
                if next_assign_val[i] == T.StrengthMap[L.'0', strength] and (dataIn[i] == L.'0' or dataIn[i] == L.'L' or clock_flag == 0) then
                  break;
                elseif (dataIn[i] == L.'0' or dataIn[i] == L.'L') and clock_flag == 1 then
                  nextstate[i] := T.StrengthMap[L.'0', strength];
                elseif next_assign_val[i] == L.'U' and clock_flag <> 1 or dataIn[i] == L.'U' and clock_flag <> 0 then
                  nextstate[i] := L.'U';
                else
                  nextstate[i] := T.StrengthMap[L.'X', strength];
                end if;
              elseif reset_set_flag == 8 then
                if clock_flag == 0 then
                  break;
                elseif clock_flag == 1 then
                  nextstate[i] := T.StrengthMap[dataIn[i], strength];
                else
                  if next_assign_val[i] == T.StrengthMap[dataIn[i], strength] or next_assign_val[i] == L.'U' then
                    break;
                  elseif dataIn[i] == L.'U' then
                    nextstate[i] := L.'U';
                  else
                    nextstate[i] := T.StrengthMap[L.'X', strength];
                  end if;
                end if;
              end if;
            end for;
          end if;
          next_assign_val := nextstate;
          dataOut := nextstate;
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-50, 60}, {50, -60}}, lineColor = {127, 0, 127}, lineThickness = 0.5, fillPattern = FillPattern.Solid, fillColor = {255, 255, 255}), Line(points = {{-80, 30}, {-50, 30}}, color = {127, 0, 127}, smooth = Smooth.None, thickness = 1), Line(points = {{50, 30}, {80, 30}}, color = {127, 33, 107}, thickness = 1, smooth = Smooth.None), Line(points = {{-80, -20}, {-50, -20}}, color = {127, 33, 107}, smooth = Smooth.None), Line(points = {{0, -84}, {0, -60}}, color = {127, 33, 107}, smooth = Smooth.None), Line(points = {{-50, -8}, {-34, -20}, {-50, -32}}, color = {127, 33, 107}, smooth = Smooth.None), Text(extent = {{-36, 66}, {34, -8}}, lineColor = {127, 33, 107}, textString = "DFFSR"), Text(extent = {{-28, -14}, {-2, -26}}, lineColor = {127, 33, 107}, textString = "clock"), Text(extent = {{-14, -44}, {12, -56}}, lineColor = {127, 33, 107}, textString = "reset"), Line(points = {{0, 80}, {0, 60}}, color = {127, 0, 127}, smooth = Smooth.None), Text(extent = {{-13, 56}, {13, 44}}, lineColor = {127, 33, 107}, textString = "set")}),
            Documentation(info = "<html>
<p>Description in VHDL is given by http://www.cs.sfu.ca/~ggbaker/reference/std_logic/src/std_logic_entities.vhd</p>
<p><b>Truth Table for high active set and reset</b></p>

<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"><b>DataIn</b></td>
      <td valign=\"top\"><b>Clock</b></td>
      <td valign=\"top\"><b>Reset</b></td>
      <td valign=\"top\"><b>Set</b></td>
      <td valign=\"top\"><b>DataOut</b></td>
      <td valign=\"top\">Map</td>
  </tr>

  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">*</td>  <td valign=\"top\">U</td> <td valign=\"top\">U</td> <td valign=\"top\">1</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">U</td>  <td valign=\"top\">*</td> <td valign=\"top\">U</td> <td valign=\"top\">1</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">*</td>  <td valign=\"top\">1</td> <td valign=\"top\">1</td> <td valign=\"top\">2</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">1</td>  <td valign=\"top\">0</td> <td valign=\"top\">0</td> <td valign=\"top\">3</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">1</td>  <td valign=\"top\">X</td> <td valign=\"top\">X</td> <td valign=\"top\">6</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">X</td> <td valign=\"top\">X</td> <td valign=\"top\">X or U</td> <td valign=\"top\">4</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">0</td> <td valign=\"top\">X</td> <td valign=\"top\">X or U or 1 or NC</td> <td valign=\"top\">5</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">X</td> <td valign=\"top\">0</td> <td valign=\"top\">X or U or 0 or NC</td> <td valign=\"top\">7</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">X-Trns</td> <td valign=\"top\">0</td>  <td valign=\"top\">0</td> <td valign=\"top\">X or U or NC</td> <td valign=\"top\">8</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">1-Trns</td> <td valign=\"top\">0</td>  <td valign=\"top\">0</td> <td valign=\"top\">DataIn</td> <td valign=\"top\">8</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">0-Trns</td> <td valign=\"top\">0</td>  <td valign=\"top\">0</td> <td valign=\"top\">NC</td> <td valign=\"top\">8</td> </tr>

</table>

<p><b>Truth Table for low active set and reset </b></p>

<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"><b>DataIn</b></td>
      <td valign=\"top\"><b>Clock</b></td>
      <td valign=\"top\"><b>Reset</b></td>
      <td valign=\"top\"><b>Set</b></td>
      <td valign=\"top\"><b>DataOut</b></td>
      <td valign=\"top\">Map</td>
  </tr>

  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">*</td>  <td valign=\"top\">U</td> <td valign=\"top\">U</td> <td valign=\"top\">1</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">U</td>  <td valign=\"top\">*</td> <td valign=\"top\">U</td> <td valign=\"top\">1</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">*</td>  <td valign=\"top\">0</td> <td valign=\"top\">1</td> <td valign=\"top\">2</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">0</td>  <td valign=\"top\">1</td> <td valign=\"top\">0</td> <td valign=\"top\">3</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">0</td>  <td valign=\"top\">X</td> <td valign=\"top\">X</td> <td valign=\"top\">6</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">X</td> <td valign=\"top\">X</td> <td valign=\"top\">X or U</td> <td valign=\"top\">4</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">1</td> <td valign=\"top\">X</td> <td valign=\"top\">X or U or 1 or NC</td> <td valign=\"top\">5</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">X</td> <td valign=\"top\">1</td> <td valign=\"top\">X or U or 0 or NC</td> <td valign=\"top\">7</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">X-Trns</td> <td valign=\"top\">1</td>  <td valign=\"top\">1</td> <td valign=\"top\">X or U or NC</td> <td valign=\"top\">8</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">1-Trns</td> <td valign=\"top\">1</td>  <td valign=\"top\">1</td> <td valign=\"top\">DataIn</td> <td valign=\"top\">8</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">0-Trns</td> <td valign=\"top\">1</td>  <td valign=\"top\">1</td> <td valign=\"top\">NC</td> <td valign=\"top\">8</td> </tr>
</table>

<PRE>
  *  = do not care
  ~  = not equal
  U  = L.'U'
  0  = L.'0' or L.'L'
  1  = L.'1' or L.'H'
  X  = L.'X' or L.'W' or L.'Z' or L.'-'
  NC = no change

Clock transition definitions:
  1-Trns: 0 -> 1
  0-Trns: ~ -> 0 or 1 -> * or X -> X|U or U -> X|U
  X-Trns: 0 -> X|U or X|U -> 1
</PRE>
</html>", revisions = "<html>
<ul>
<li><i>September 11, 2009 </i> created by Ulrich Donath<br>
       </li>
</ul>
</html>"));
        end DFFSR;

        model DFFREGSRH "Edge triggered register bank with high active set and reset"
          import D = Modelica.Electrical.Digital;
          import L = Modelica.Electrical.Digital.Interfaces.Logic;
          import S = Modelica.Electrical.Digital.Interfaces.Strength;
          import T = Modelica.Electrical.Digital.Tables;
          parameter Modelica.SIunits.Time tHL = 0 "High->Low delay";
          parameter Modelica.SIunits.Time tLH = 0 "Low->High delay";
          parameter D.Interfaces.Strength strength = S.'S_X01' "output strength";
          parameter Integer n(min = 1) = 1 "data width";
        protected
          constant Integer ResetSetMap[9, 9] = [1, 1, 1, 1, 1, 1, 1, 1, 1; 1, 4, 7, 2, 4, 4, 7, 2, 4; 1, 5, 8, 2, 5, 5, 8, 2, 5; 1, 6, 3, 2, 6, 6, 3, 2, 6; 1, 4, 7, 2, 4, 4, 7, 2, 4; 1, 4, 7, 2, 4, 4, 7, 2, 4; 1, 5, 8, 2, 5, 5, 8, 2, 5; 1, 6, 3, 2, 6, 6, 3, 2, 6; 1, 4, 7, 2, 4, 4, 7, 2, 4];
          // Function selection by [reset, set] reading, active high;
        protected
          D.Delay.InertialDelaySensitiveVector delay(tHL = tHL, tLH = tLH, n = n, inertialDelaySensitive(each y(start = L.'U', fixed = true))) annotation(
            Placement(transformation(extent = {{23, 12}, {79, 68}})));
          D.Registers.DFFSR dFFSR(strength = strength, n = n, ResetSetMap = ResetSetMap, clock(start = L.'U', fixed = true), reset(start = L.'U', fixed = true), set(start = L.'U', fixed = true)) annotation(
            Placement(transformation(extent = {{-80, -25}, {20, 75}})));
        public
          D.Interfaces.DigitalInput set annotation(
            Placement(transformation(extent = {{-100, 72}, {-84, 88}})));
          D.Interfaces.DigitalInput reset annotation(
            Placement(transformation(extent = {{-100, -68}, {-84, -52}})));
          D.Interfaces.DigitalInput clock annotation(
            Placement(transformation(extent = {{-100, -28}, {-84, -12}})));
          D.Interfaces.DigitalInput dataIn[n] annotation(
            Placement(transformation(extent = {{-100, 32}, {-84, 48}})));
          D.Interfaces.DigitalOutput dataOut[n] annotation(
            Placement(transformation(extent = {{84, 32}, {100, 48}})));
        equation
          connect(dFFSR.dataOut, delay.x) annotation(
            Line(points = {{9.5, 40}, {26.92, 40}, {26.92, 39.86}}, color = {127, 0, 127}, thickness = 1, smooth = Smooth.None));
          connect(set, dFFSR.set) annotation(
            Line(points = {{-92, 80}, {-30, 80}, {-30, 70}}, color = {127, 0, 127}, smooth = Smooth.None));
          connect(reset, dFFSR.reset) annotation(
            Line(points = {{-92, -60}, {-30, -60}, {-30, -20}}, color = {127, 0, 127}, smooth = Smooth.None));
          connect(clock, dFFSR.clock) annotation(
            Line(points = {{-92, -20}, {-70, -20}, {-70, 15}}, color = {127, 0, 127}, smooth = Smooth.None));
          connect(dataIn, dFFSR.dataIn) annotation(
            Line(points = {{-92, 40}, {-70, 40}}, color = {127, 0, 127}, thickness = 1, smooth = Smooth.None));
          connect(delay.y, dataOut) annotation(
            Line(points = {{75.08, 40}, {92, 40}}, color = {127, 0, 127}, thickness = 1, smooth = Smooth.None));
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-50, 70}, {52, -50}}, lineColor = {127, 0, 127}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{-86, 40}, {-50, 40}}, color = {127, 0, 127}, thickness = 1, smooth = Smooth.None), Line(points = {{-84, -20}, {-50, -20}}, color = {127, 0, 127}, smooth = Smooth.None), Line(points = {{52, 40}, {84, 40}}, color = {127, 0, 127}, smooth = Smooth.None, thickness = 1), Line(points = {{-50, -10}, {-36, -20}, {-50, -30}}, color = {127, 0, 127}, smooth = Smooth.None), Text(extent = {{-44, 94}, {38, -14}}, lineColor = {127, 33, 107}, textString = "DFFREGSR"), Text(extent = {{-14, -32}, {12, -44}}, lineColor = {127, 33, 107}, textString = "reset"), Text(extent = {{-33, -13}, {-7, -25}}, lineColor = {127, 33, 107}, textString = "clock"), Line(points = {{-84, 80}, {0, 80}, {0, 70}}, color = {127, 0, 127}, smooth = Smooth.None), Line(points = {{-84, -60}, {0, -60}, {0, -50}}, color = {127, 0, 127}, smooth = Smooth.None), Text(extent = {{-14, 66}, {12, 54}}, lineColor = {127, 33, 107}, textString = "set"), Text(extent = {{-32, 38}, {26, 10}}, lineColor = {127, 0, 127}, textString = "high active")}),
            Documentation(info = "<html>
<p>Description in VHDL is given by http://www.cs.sfu.ca/~ggbaker/reference/std_logic/src/std_logic_entities.vhd</p>
<p><b>Truth Table</b></p>

<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"><b>DataIn</b></td>
      <td valign=\"top\"><b>Clock</b></td>
      <td valign=\"top\"><b>Reset</b></td>
      <td valign=\"top\"><b>Set</b></td>
      <td valign=\"top\"><b>DataOut</b></td>
  </tr>

  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">*</td>  <td valign=\"top\">U</td> <td valign=\"top\">U</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">U</td>  <td valign=\"top\">*</td> <td valign=\"top\">U</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">*</td>  <td valign=\"top\">1</td> <td valign=\"top\">1</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">1</td>  <td valign=\"top\">0</td> <td valign=\"top\">0</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">1</td>  <td valign=\"top\">X</td> <td valign=\"top\">X</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">X</td> <td valign=\"top\">X</td> <td valign=\"top\">X or U</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">0</td> <td valign=\"top\">X</td> <td valign=\"top\">X or U or 1 or NC</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">X</td> <td valign=\"top\">0</td> <td valign=\"top\">X or U or 0 or NC</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">X-Trns</td> <td valign=\"top\">0</td>  <td valign=\"top\">0</td> <td valign=\"top\">X or U or NC</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">1-Trns</td> <td valign=\"top\">0</td>  <td valign=\"top\">0</td> <td valign=\"top\">DataIn</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">0-Trns</td> <td valign=\"top\">0</td>  <td valign=\"top\">0</td> <td valign=\"top\">NC</td> </tr>
</table>

<PRE>
  *  = do not care
  ~  = not equal
  U  = L.'U'
  0  = L.'0' or L.'L'
  1  = L.'1' or L.'H'
  X  = L.'X' or L.'W' or L.'Z' or L.'-'
  NC = no change

Clock transition definitions:
  1-Trns: 0 -> 1
  0-Trns: ~ -> 0 or 1 -> * or X -> X|U or U -> X|U
  X-Trns: 0 -> X|U or X|U -> 1
</PRE>
</html>", revisions = "<html>
<ul>
<li><i>September 11, 2009 </i> created by Ulrich Donath<br>
       </li>
</ul>
</html>"));
        end DFFREGSRH;

        model DFFREGSRL "Edge triggered register bank with low active set and reset"
          extends Digital.Registers.DFFREGSRH(final ResetSetMap = [1, 1, 1, 1, 1, 1, 1, 1, 1; 1, 4, 2, 7, 4, 4, 2, 7, 4; 1, 6, 2, 3, 6, 6, 2, 3, 6; 1, 5, 2, 8, 5, 5, 2, 8, 5; 1, 4, 2, 7, 4, 4, 2, 7, 4; 1, 4, 2, 7, 4, 4, 2, 7, 4; 1, 6, 2, 3, 6, 6, 2, 3, 6; 1, 5, 2, 8, 5, 5, 2, 8, 5; 1, 4, 2, 7, 4, 4, 2, 7, 4]);
          // Function selection by [reset, set] reading;
          annotation(
            Icon(graphics = {Rectangle(extent = {{-36, 30}, {28, 16}}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, pattern = LinePattern.None), Text(extent = {{-30, 38}, {24, 12}}, lineColor = {127, 0, 127}, textString = "low active")}),
            Documentation(info = "<html>
<p>Description in VHDL is given by http://www.cs.sfu.ca/~ggbaker/reference/std_logic/src/std_logic_entities.vhd</p>

<p><b>Truth Table</b></p>

<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"><b>DataIn</b></td>
      <td valign=\"top\"><b>Clock</b></td>
      <td valign=\"top\"><b>Reset</b></td>
      <td valign=\"top\"><b>Set</b></td>
      <td valign=\"top\"><b>DataOut</b></td>
  </tr>

  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">*</td>  <td valign=\"top\">U</td> <td valign=\"top\">U</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">U</td>  <td valign=\"top\">*</td> <td valign=\"top\">U</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">*</td>  <td valign=\"top\">0</td> <td valign=\"top\">1</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">0</td>  <td valign=\"top\">1</td> <td valign=\"top\">0</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">0</td>  <td valign=\"top\">X</td> <td valign=\"top\">X</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">X</td> <td valign=\"top\">X</td> <td valign=\"top\">X or U</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">1</td> <td valign=\"top\">X</td> <td valign=\"top\">X or U or 1 or NC</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">X</td> <td valign=\"top\">1</td> <td valign=\"top\">X or U or 0 or NC</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">X-Trns</td> <td valign=\"top\">1</td>  <td valign=\"top\">1</td> <td valign=\"top\">X or U or NC</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">1-Trns</td> <td valign=\"top\">1</td>  <td valign=\"top\">1</td> <td valign=\"top\">DataIn</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">0-Trns</td> <td valign=\"top\">1</td>  <td valign=\"top\">1</td> <td valign=\"top\">NC</td> </tr>
</table>

<PRE>
  *  = do not care
  ~  = not equal
  U  = L.'U'
  0  = L.'0' or L.'L'
  1  = L.'1' or L.'H'
  X  = L.'X' or L.'W' or L.'Z' or L.'-'
  NC = no change

Clock transition definitions:
  1-Trns: 0 -> 1
  0-Trns: ~ -> 0 or 1 -> * or X -> X|U or U -> X|U
  X-Trns: 0 -> X|U or X|U -> 1
</PRE>
</html>", revisions = "<html>
<ul>
<li><i>September 11, 2009 </i> created by Ulrich Donath<br>
       </li>
</ul>
</html>"));
        end DFFREGSRL;

        model DLATR "Level sensitive register bank with reset"
          import D = Modelica.Electrical.Digital;
          import L = Modelica.Electrical.Digital.Interfaces.Logic;
          import S = Modelica.Electrical.Digital.Interfaces.Strength;
          import T = Modelica.Electrical.Digital.Tables;
          parameter Integer ResetMap[9] = {1, 4, 3, 2, 4, 4, 3, 2, 4} "function selection, defaults for high active reset";
          parameter D.Interfaces.Strength strength = S.'S_X01' "output strength";
          parameter Integer n(min = 1) = 1 "data width";
          D.Interfaces.DigitalInput reset annotation(
            Placement(transformation(extent = {{-10, -100}, {10, -80}}), iconTransformation(extent = {{-10, -100}, {10, -80}})));
          D.Interfaces.DigitalInput enable annotation(
            Placement(transformation(extent = {{-90, -20}, {-70, 0}}), iconTransformation(extent = {{-90, -30}, {-70, -10}})));
          D.Interfaces.DigitalInput dataIn[n] annotation(
            Placement(transformation(extent = {{-90, 20}, {-70, 40}}), iconTransformation(extent = {{-90, 20}, {-70, 40}})));
          D.Interfaces.DigitalOutput dataOut[n] annotation(
            Placement(transformation(extent = {{66, 24}, {86, 44}}), iconTransformation(extent = {{69, 20}, {89, 40}})));
        protected
          Integer enable_flag(start = 0);
          // 0: low level
          // 1: high level
          // 2: unknown
          // 3: uninitialized
          Integer reset_flag(start = 1);
          // 1: output := U
          // 2: output := 0
          // 3: output := -UdataIn
          // 4: output := U-0X
          D.Interfaces.Logic nextstate[n](start = fill(L.'U', n));
          D.Interfaces.Logic next_assign_val[n](start = fill(L.'U', n));
        algorithm
          if enable == L.'1' or enable == L.'H' then
            enable_flag := 1;
          elseif enable == L.'0' or enable == L.'L' then
            enable_flag := 0;
          elseif enable == L.'U' then
            enable_flag := 3;
          else
            enable_flag := 2;
          end if;
          reset_flag := ResetMap[reset];
          for i in 1:n loop
            if reset_flag == 1 then
              nextstate[i] := L.'U';
            elseif reset_flag == 2 then
              nextstate[i] := T.StrengthMap[L.'0', strength];
            elseif reset_flag == 3 then
              if enable_flag == 0 then
                break;
              elseif enable_flag == 3 then
                nextstate[i] := L.'U';
              elseif enable_flag == 1 then
                nextstate[i] := T.StrengthMap[dataIn[i], strength];
              else
                if next_assign_val[i] == T.StrengthMap[dataIn[i], strength] or next_assign_val[i] == L.'U' then
                  break;
                elseif dataIn[i] == L.'U' then
                  nextstate[i] := L.'U';
                else
                  nextstate[i] := T.StrengthMap[L.'X', strength];
                end if;
              end if;
            elseif reset_flag == 4 then
              if enable_flag == 3 or next_assign_val[i] == L.'U' and enable_flag <> 1 or dataIn[i] == L.'U' and enable_flag <> 0 then
                nextstate[i] := L.'U';
              elseif next_assign_val[i] == T.StrengthMap[L.'0', strength] and (dataIn[i] == L.'0' or dataIn[i] == L.'L' or enable_flag == 0) then
                break;
              elseif (dataIn[i] == L.'0' or dataIn[i] == L.'L') and enable_flag == 1 then
                nextstate[i] := T.StrengthMap[L.'0', strength];
              else
                nextstate[i] := T.StrengthMap[L.'X', strength];
              end if;
            end if;
          end for;
          next_assign_val := nextstate;
          dataOut := nextstate;
          annotation(
            Documentation(info = "<html>

<p> Description in VHDL is given by <a href=\"http://www.cs.sfu.ca/~ggbaker/reference/std_logic/src/std_logic_entities.vhd\">http://www.cs.sfu.ca/~ggbaker/reference/std_logic/src/std_logic_entities.vhd</a></p>
<p><b>Truth Table for high active reset:</b></p>

<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"><b>DataIn</b></td>
      <td valign=\"top\"><b>Enable</b></td>
      <td valign=\"top\"><b>Reset</b></td>
      <td valign=\"top\"><b>DataOut</b></td>
      <td valign=\"top\">Map</td>
  </tr>

  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">U</td > <td valign=\"top\">U</td> <td valign=\"top\">1</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">1</td>  <td valign=\"top\">0</td> <td valign=\"top\">2</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">0</td> <td valign=\"top\">0</td>  <td valign=\"top\">NC</td> <td valign=\"top\">3</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">1</td> <td valign=\"top\">0</td>  <td valign=\"top\">DataIn</td> <td valign=\"top\">3</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">X</td> <td valign=\"top\">0</td>  <td valign=\"top\">X or U or NC</td> <td valign=\"top\">3</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">U</td> <td valign=\"top\">~1</td> <td valign=\"top\">U</td> <td valign=\"top\">4</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">~U</td> <td valign=\"top\">X</td> <td valign=\"top\">X or U or 0 or NC</td> <td valign=\"top\">4</td> </tr>
</table>

<p><b>Truth Table for low active reset:</b></p>

<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"><b>DataIn</b></td>
      <td valign=\"top\"><b>Enable</b></td>
      <td valign=\"top\"><b>Reset</b></td>
      <td valign=\"top\"><b>DataOut</b></td>
      <td valign=\"top\">Map</td>
  </tr>

  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">U</td> <td valign=\"top\">U</td> <td valign=\"top\">1</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">0</td> <td valign=\"top\">0</td> <td valign=\"top\">2</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">0</td> <td valign=\"top\">1</td> <td valign=\"top\">NC</td> <td valign=\"top\">3</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">1</td> <td valign=\"top\">1</td> <td valign=\"top\">DataIn</td> <td valign=\"top\">3</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">X</td> <td valign=\"top\">1</td> <td valign=\"top\">X or U or NC</td> <td valign=\"top\">3</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">U</td> <td valign=\"top\">~0</td> <td valign=\"top\">U</td> <td valign=\"top\">4</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">~U</td> <td valign=\"top\">X</td> <td valign=\"top\">X or U or 0 or NC</td> <td valign=\"top\">4</td> </tr>
</table>

<PRE>

  *  = do not care
  ~  = not equal
  U  = L.'U'
  0  = L.'0' or L.'L'
  1  = L.'1' or L.'H'
  X  = L.'X' or L.'W' or L.'Z' or L.'-'
  NC = no change
</PRE>
</HTML>", revisions = "<html>
<ul>
<li><i>September 11, 2009 </i> created by Ulrich Donath<br>
       </li>
</ul>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-50, 60}, {50, -60}}, lineColor = {127, 0, 127}, lineThickness = 0.5, fillPattern = FillPattern.Solid, fillColor = {255, 255, 255}), Line(points = {{-80, 30}, {-50, 30}}, color = {127, 0, 127}, smooth = Smooth.None, thickness = 1), Line(points = {{50, 30}, {80, 30}}, color = {127, 33, 107}, thickness = 1, smooth = Smooth.None), Line(points = {{-80, -20}, {-50, -20}}, color = {127, 33, 107}, smooth = Smooth.None), Line(points = {{0, -84}, {0, -60}}, color = {127, 33, 107}, smooth = Smooth.None), Line(points = {{-50, -8}, {-34, -20}, {-50, -32}}, color = {127, 33, 107}, smooth = Smooth.None), Text(extent = {{-30, 58}, {26, 2}}, lineColor = {127, 33, 107}, textString = "DLATR"), Text(extent = {{-29, -8}, {6, -32}}, lineColor = {127, 33, 107}, textString = "enable"), Text(extent = {{-14, -44}, {12, -56}}, lineColor = {127, 33, 107}, textString = "reset")}));
        end DLATR;

        model DLATREG "Level sensitive register bank with reset active high"
          import D = Modelica.Electrical.Digital;
          import L = Modelica.Electrical.Digital.Interfaces.Logic;
          import S = Modelica.Electrical.Digital.Interfaces.Strength;
          import T = Modelica.Electrical.Digital.Tables;
          parameter Modelica.SIunits.Time tHL = 0 "High->Low delay";
          parameter Modelica.SIunits.Time tLH = 0 "Low->High delay";
          parameter D.Interfaces.Strength strength = S.'S_X01' "output strength";
          parameter Integer n(min = 1) = 1 "data width";
        protected
          constant Integer ResetMap[9] = {1, 4, 3, 2, 4, 4, 3, 2, 4};
          // Function selection by [reset] reading
          // 1: output := U
          // 2: output := 0
          // 3: output := -UdataIn
          // 4: output := U-0X
        public
          D.Delay.InertialDelaySensitiveVector delay(tHL = tHL, tLH = tLH, n = n) annotation(
            Placement(transformation(extent = {{20, 12}, {76, 68}})));
          D.Interfaces.DigitalInput reset annotation(
            Placement(transformation(extent = {{-100, -68}, {-84, -52}})));
          D.Interfaces.DigitalInput enable annotation(
            Placement(transformation(extent = {{-100, -28}, {-84, -12}})));
          D.Interfaces.DigitalInput dataIn[n] annotation(
            Placement(transformation(extent = {{-100, 32}, {-84, 48}})));
          D.Interfaces.DigitalOutput dataOut[n] annotation(
            Placement(transformation(extent = {{84, 32}, {100, 48}}), iconTransformation(extent = {{84, 32}, {100, 48}})));
          D.Registers.DLATR dLATR(n = n, strength = strength, ResetMap = ResetMap) annotation(
            Placement(transformation(extent = {{-78, -23}, {18, 74}})));
        equation
          connect(delay.y, dataOut) annotation(
            Line(points = {{72.08, 40}, {92, 40}}, color = {127, 0, 127}, thickness = 1, smooth = Smooth.None));
          connect(dLATR.dataOut, delay.x) annotation(
            Line(points = {{7.92, 40.05}, {15.96, 39.86}, {23.92, 39.86}}, color = {127, 0, 127}, thickness = 1, smooth = Smooth.None));
          connect(dataIn, dLATR.dataIn) annotation(
            Line(points = {{-92, 40}, {-80.2, 40}, {-80.2, 40.05}, {-68.4, 40.05}}, color = {127, 0, 127}, thickness = 1, smooth = Smooth.None));
          connect(enable, dLATR.enable) annotation(
            Line(points = {{-92, -20}, {-68.4, -20}, {-68.4, 15.8}}, color = {127, 0, 127}, smooth = Smooth.None));
          connect(reset, dLATR.reset) annotation(
            Line(points = {{-92, -60}, {-30, -60}, {-30, -18.15}}, color = {127, 0, 127}, smooth = Smooth.None));
          annotation(
            Documentation(info = "<html>
<p>Description in VHDL is given by http://www.cs.sfu.ca/~ggbaker/reference/std_logic/src/std_logic_entities.vhd</p>
<p><b>Truth Table</b></p>

<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"><b>DataIn</b></td>
      <td valign=\"top\"><b>Enable</b></td>
      <td valign=\"top\"><b>Reset</b></td>
      <td valign=\"top\"><b>DataOut</b></td>
  </tr>

  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">U</td > <td valign=\"top\">U</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">1</td>  <td valign=\"top\">0</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">0</td> <td valign=\"top\">0</td>  <td valign=\"top\">NC</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">1</td> <td valign=\"top\">0</td>  <td valign=\"top\">DataIn</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">X</td> <td valign=\"top\">0</td>  <td valign=\"top\">X or U or NC</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">U</td> <td valign=\"top\">~1</td> <td valign=\"top\">U</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">~U</td> <td valign=\"top\">X</td> <td valign=\"top\">X or U or 0 or NC</td> </tr>
</table>

<PRE>
  *  = do not care
  ~  = not equal
  U  = L.'U'
  0  = L.'0' or L.'L'
  1  = L.'1' or L.'H'
  X  = L.'X' or L.'W' or L.'Z' or L.'-'
  NC = no change
</PRE>
</HTML>", revisions = "<html>
<ul>
<li><i>September 11, 2009 </i> created by Ulrich Donath<br>
       </li>
</ul>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-50, 70}, {52, -50}}, lineColor = {127, 0, 127}, lineThickness = 0.5, fillPattern = FillPattern.Solid, fillColor = {255, 255, 255}), Line(points = {{-86, 40}, {-50, 40}}, color = {127, 0, 127}, thickness = 1, smooth = Smooth.None), Line(points = {{-84, -20}, {-50, -20}}, color = {127, 0, 127}, smooth = Smooth.None), Line(points = {{52, 40}, {84, 40}}, color = {127, 0, 127}, smooth = Smooth.None, thickness = 1), Line(points = {{-50, -10}, {-36, -20}, {-50, -30}}, color = {127, 0, 127}, smooth = Smooth.None), Text(extent = {{-32, 82}, {32, -2}}, lineColor = {127, 0, 127}, textString = "DLATREG"), Text(extent = {{-14, -33}, {12, -45}}, lineColor = {127, 33, 107}, textString = "reset"), Text(extent = {{-33, -7}, {-1, -32}}, lineColor = {127, 33, 107}, textString = "enable"), Line(points = {{-84, -60}, {0, -60}, {0, -50}}, color = {127, 0, 127}, smooth = Smooth.None), Text(extent = {{-26, 44}, {26, 2}}, lineColor = {127, 33, 107}, textString = "high active")}));
        end DLATREG;

        model DLATREGL "Level sensitive register bank with reset active low"
          extends DLATREG(final ResetMap = {1, 4, 2, 3, 4, 4, 2, 3, 4});
          // Function selection by [reset] reading
          // 1: output := U
          // 2: output := 0
          // 3: output := -UdataIn
          // 4: output := U-0X
          annotation(
            Documentation(info = "<html>
<p>Description in VHDL is given by http://www.cs.sfu.ca/~ggbaker/reference/std_logic/src/std_logic_entities.vhd</p>
<p><b>Truth Table</b></p>

<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"><b>DataIn</b></td>
      <td valign=\"top\"><b>Enable</b></td>
      <td valign=\"top\"><b>Reset</b></td>
      <td valign=\"top\"><b>DataOut</b></td>
  </tr>

  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">U</td> <td valign=\"top\">U</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">0</td> <td valign=\"top\">0</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">0</td> <td valign=\"top\">1</td> <td valign=\"top\">NC</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">1</td> <td valign=\"top\">1</td> <td valign=\"top\">DataIn</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">X</td> <td valign=\"top\">1</td> <td valign=\"top\">X or U or NC</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">U</td> <td valign=\"top\">~0</td> <td valign=\"top\">U</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">~U</td> <td valign=\"top\">X</td> <td valign=\"top\">X or U or 0 or NC</td> </tr>
</table>

<PRE>
  *  = do not care
  ~  = not equal
  U  = L.'U'
  0  = L.'0' or L.'L'
  1  = L.'1' or L.'H'
  X  = L.'X' or L.'W' or L.'Z' or L.'-'
  NC = no change
</PRE>
</HTML>", revisions = "<html>
<ul>
<li><i>September 11, 2009 </i> created by Ulrich Donath<br>
       </li>
</ul>
</html>"),
            Icon(graphics = {Rectangle(extent = {{-28, 30}, {30, 18}}, pattern = LinePattern.None, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-26, 48}, {26, -1}}, lineColor = {127, 33, 107}, textString = "low active")}));
        end DLATREGL;

        model DLATSR "Level sensitive register bank with set and reset"
          import D = Modelica.Electrical.Digital;
          import L = Modelica.Electrical.Digital.Interfaces.Logic;
          import S = Modelica.Electrical.Digital.Interfaces.Strength;
          import T = Modelica.Electrical.Digital.Tables;
          parameter Integer ResetSetMap[9, 9] = [1, 1, 1, 1, 1, 1, 1, 1, 1; 1, 4, 7, 2, 4, 4, 7, 2, 4; 1, 5, 8, 2, 5, 5, 8, 2, 5; 1, 6, 3, 2, 6, 6, 3, 2, 6; 1, 4, 7, 2, 4, 4, 7, 2, 4; 1, 4, 7, 2, 4, 4, 7, 2, 4; 1, 5, 8, 2, 5, 5, 8, 2, 5; 1, 6, 3, 2, 6, 6, 3, 2, 6; 1, 4, 7, 2, 4, 4, 7, 2, 4] "function selection by [reset, set] reading";
          /* Defaults for set and reset are active high */
          parameter D.Interfaces.Strength strength = S.'S_X01' "output strength";
          parameter Integer n(min = 1) = 1 "data width";
          D.Interfaces.DigitalInput set annotation(
            Placement(transformation(extent = {{-10, 100}, {10, 80}})));
          D.Interfaces.DigitalInput reset annotation(
            Placement(transformation(extent = {{-10, -100}, {10, -80}}), iconTransformation(extent = {{-10, -100}, {10, -80}})));
          D.Interfaces.DigitalInput enable annotation(
            Placement(transformation(extent = {{-90, -20}, {-70, 0}}), iconTransformation(extent = {{-90, -30}, {-70, -10}})));
          D.Interfaces.DigitalInput dataIn[n] annotation(
            Placement(transformation(extent = {{-90, 20}, {-70, 40}}), iconTransformation(extent = {{-90, 20}, {-70, 40}})));
          D.Interfaces.DigitalOutput dataOut[n] annotation(
            Placement(transformation(extent = {{68, 20}, {88, 40}}), iconTransformation(extent = {{69, 20}, {89, 40}})));
        protected
          Integer enable_flag(start = 0);
          // 0: low level
          // 1: high level
          // 2: unknown
          // 3: uninitialized
          Integer reset_set_flag(start = 1);
          // 1: output := U
          // 2: output := 1
          // 3: output := 0
          // 4: output := UX
          // 5: output := U-1X
          // 6: output := X
          // 7: output := U-0X
          // 8: output := -UdataInX
          D.Interfaces.Logic nextstate[n](start = fill(L.'U', n));
          D.Interfaces.Logic next_assign_val[n](start = fill(L.'U', n));
        algorithm
          if enable == L.'1' or enable == L.'H' then
            enable_flag := 1;
          elseif enable == L.'0' or enable == L.'L' then
            enable_flag := 0;
          elseif enable == L.'U' then
            enable_flag := 3;
          else
            enable_flag := 2;
          end if;
          reset_set_flag := ResetSetMap[reset, set];
          for i in 1:n loop
            if reset_set_flag == 1 then
              nextstate[i] := L.'U';
            elseif reset_set_flag == 2 then
              nextstate[i] := T.StrengthMap[L.'1', strength];
            elseif reset_set_flag == 3 then
              nextstate[i] := T.StrengthMap[L.'0', strength];
            elseif reset_set_flag == 4 then
              if enable_flag == 3 or next_assign_val[i] == L.'U' and enable_flag <> 1 or dataIn[i] == L.'U' and enable_flag <> 0 then
                nextstate[i] := L.'U';
              else
                nextstate[i] := T.StrengthMap[L.'X', strength];
              end if;
            elseif reset_set_flag == 5 then
              if enable_flag == 3 or next_assign_val[i] == L.'U' and enable_flag <> 1 or dataIn[i] == L.'U' and enable_flag <> 0 then
                nextstate[i] := L.'U';
              elseif next_assign_val[i] == T.StrengthMap[L.'1', strength] and (dataIn[i] == L.'1' or dataIn[i] == L.'H' or enable_flag == 0) then
                break;
              elseif (dataIn[i] == L.'1' or dataIn[i] == L.'H') and enable_flag == 1 then
                nextstate[i] := T.StrengthMap[L.'1', strength];
              else
                nextstate[i] := T.StrengthMap[L.'X', strength];
              end if;
            elseif reset_set_flag == 6 then
              nextstate[i] := T.StrengthMap[L.'X', strength];
            elseif reset_set_flag == 7 then
              if enable_flag == 3 or next_assign_val[i] == L.'U' and enable_flag <> 1 or dataIn[i] == L.'U' and enable_flag <> 0 then
                nextstate[i] := L.'U';
              elseif next_assign_val[i] == T.StrengthMap[L.'0', strength] and (dataIn[i] == L.'0' or dataIn[i] == L.'L' or enable_flag == 0) then
                break;
              elseif (dataIn[i] == L.'0' or dataIn[i] == L.'L') and enable_flag == 1 then
                nextstate[i] := T.StrengthMap[L.'0', strength];
              else
                nextstate[i] := T.StrengthMap[L.'X', strength];
              end if;
            elseif reset_set_flag == 8 then
              if enable_flag == 0 then
                break;
              elseif enable_flag == 3 then
                nextstate[i] := L.'U';
              elseif enable_flag == 1 then
                nextstate[i] := T.StrengthMap[dataIn[i], strength];
              else
                if next_assign_val[i] == T.StrengthMap[dataIn[i], strength] or next_assign_val[i] == L.'U' then
                  break;
                elseif dataIn[i] == L.'U' then
                  nextstate[i] := L.'U';
                else
                  nextstate[i] := T.StrengthMap[L.'X', strength];
                end if;
              end if;
            end if;
          end for;
          next_assign_val := nextstate;
          dataOut := nextstate;
          annotation(
            Documentation(info = "<html>
<p>Description in VHDL is given by http://www.cs.sfu.ca/~ggbaker/reference/std_logic/src/std_logic_entities.vhd</p>
<p><b>Truth Table for high active set and reset</b></p>

<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"><b>DataIn</b></td>
      <td valign=\"top\"><b>Enable</b></td>
      <td valign=\"top\"><b>Reset</b></td>
      <td valign=\"top\"><b>Set</b></td>
      <td valign=\"top\"><b>DataOut</b></td>
      <td valign=\"top\">Map</td>
  </tr>

  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">*</td>  <td valign=\"top\">U</td> <td valign=\"top\">U</td> <td valign=\"top\">1</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">U</td>  <td valign=\"top\">~1</td> <td valign=\"top\">U</td> <td valign=\"top\">1</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">*</td>  <td valign=\"top\">1</td> <td valign=\"top\">1</td> <td valign=\"top\">2</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">1</td>  <td valign=\"top\">0</td> <td valign=\"top\">0</td> <td valign=\"top\">3</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">1</td>  <td valign=\"top\">X</td> <td valign=\"top\">X</td> <td valign=\"top\">6</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">U</td> <td valign=\"top\">~1</td> <td valign=\"top\">~1</td> <td valign=\"top\">U</td> <td valign=\"top\">4,5,7,8</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">~U</td> <td valign=\"top\">X</td> <td valign=\"top\">X</td> <td valign=\"top\">X or U</td> <td valign=\"top\">4</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">~U</td> <td valign=\"top\">0</td> <td valign=\"top\">X</td> <td valign=\"top\">X or U or 1 or NC</td> <td valign=\"top\">5</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">~U</td> <td valign=\"top\">X</td> <td valign=\"top\">0</td> <td valign=\"top\">X or U or 0 or NC</td> <td valign=\"top\">7</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">X</td> <td valign=\"top\">0</td>  <td valign=\"top\">0</td> <td valign=\"top\">X or U or NC</td> <td valign=\"top\">8</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">1</td> <td valign=\"top\">0</td>  <td valign=\"top\">0</td> <td valign=\"top\">DataIn</td> <td valign=\"top\">8</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">0</td> <td valign=\"top\">0</td>  <td valign=\"top\">0</td> <td valign=\"top\">NC</td> <td valign=\"top\">8</td> </tr>

</table>

<p><b>Truth Table for low active set and reset </b></p>

<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"><b>DataIn</b></td>
      <td valign=\"top\"><b>Enable</b></td>
      <td valign=\"top\"><b>Reset</b></td>
      <td valign=\"top\"><b>Set</b></td>
      <td valign=\"top\"><b>DataOut</b></td>
      <td valign=\"top\">Map</td>
  </tr>

  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">*</td>  <td valign=\"top\">U</td> <td valign=\"top\">U</td> <td valign=\"top\">1</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">U</td>  <td valign=\"top\">~0</td> <td valign=\"top\">U</td> <td valign=\"top\">1</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">*</td>  <td valign=\"top\">0</td> <td valign=\"top\">1</td> <td valign=\"top\">2</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">0</td>  <td valign=\"top\">1</td> <td valign=\"top\">0</td> <td valign=\"top\">3</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">0</td>  <td valign=\"top\">X</td> <td valign=\"top\">X</td> <td valign=\"top\">6</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">U</td> <td valign=\"top\">~0</td> <td valign=\"top\">~0</td> <td valign=\"top\">U</td> <td valign=\"top\">4,5,7,8</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">~U</td> <td valign=\"top\">X</td> <td valign=\"top\">X</td> <td valign=\"top\">X or U</td> <td valign=\"top\">4</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">~U</td> <td valign=\"top\">1</td> <td valign=\"top\">X</td> <td valign=\"top\">X or U or 1 or NC</td> <td valign=\"top\">5</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">~U</td> <td valign=\"top\">X</td> <td valign=\"top\">1</td> <td valign=\"top\">X or U or 0 or NC</td> <td valign=\"top\">7</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">X</td> <td valign=\"top\">1</td>  <td valign=\"top\">1</td> <td valign=\"top\">X or U or NC</td> <td valign=\"top\">8</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">1</td> <td valign=\"top\">1</td>  <td valign=\"top\">1</td> <td valign=\"top\">DataIn</td> <td valign=\"top\">8</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">0</td> <td valign=\"top\">1</td>  <td valign=\"top\">1</td> <td valign=\"top\">NC</td> <td valign=\"top\">8</td> </tr>

</table>

<PRE>
  *  = do not care
  ~  = not equal
  U  = L.'U'
  0  = L.'0' or L.'L'
  1  = L.'1' or L.'H'
  X  = L.'X' or L.'W' or L.'Z' or L.'-'
  NC = no change
</PRE>
</HTML>", revisions = "<html>
<ul>
<li><i>September 11, 2009 </i> created by Ulrich Donath<br>
       </li>
</ul>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-50, 60}, {50, -60}}, lineColor = {127, 0, 127}, lineThickness = 0.5, fillPattern = FillPattern.Solid, fillColor = {255, 255, 255}), Line(points = {{-80, 30}, {-50, 30}}, color = {127, 0, 127}, smooth = Smooth.None, thickness = 1), Line(points = {{50, 30}, {80, 30}}, color = {127, 33, 107}, thickness = 1, smooth = Smooth.None), Line(points = {{-80, -20}, {-50, -20}}, color = {127, 33, 107}, smooth = Smooth.None), Line(points = {{0, -84}, {0, -60}}, color = {127, 33, 107}, smooth = Smooth.None), Line(points = {{-50, -8}, {-34, -20}, {-50, -32}}, color = {127, 33, 107}, smooth = Smooth.None), Text(extent = {{-34, 60}, {32, -6}}, lineColor = {127, 33, 107}, textString = "DLATSR"), Text(extent = {{-29, -8}, {5, -30}}, lineColor = {127, 33, 107}, textString = "enable"), Text(extent = {{-14, -44}, {12, -56}}, lineColor = {127, 33, 107}, textString = "reset"), Line(points = {{0, 80}, {0, 60}}, color = {127, 0, 127}, smooth = Smooth.None), Text(extent = {{-13, 56}, {13, 44}}, lineColor = {127, 33, 107}, textString = "set")}));
        end DLATSR;

        model DLATREGSRH "Level sensitive register bank with set and reset, active high"
          import D = Modelica.Electrical.Digital;
          import L = Modelica.Electrical.Digital.Interfaces.Logic;
          import S = Modelica.Electrical.Digital.Interfaces.Strength;
          import T = Modelica.Electrical.Digital.Tables;
          parameter Modelica.SIunits.Time tHL = 0 "High->Low delay";
          parameter Modelica.SIunits.Time tLH = 0 "Low->High delay";
          parameter D.Interfaces.Strength strength = S.'S_X01' "output strength";
          parameter Integer n(min = 1) = 1 "data width";
        protected
          constant Integer ResetSetMap[9, 9] = [1, 1, 1, 1, 1, 1, 1, 1, 1; 1, 4, 7, 2, 4, 4, 7, 2, 4; 1, 5, 8, 2, 5, 5, 8, 2, 5; 1, 6, 3, 2, 6, 6, 3, 2, 6; 1, 4, 7, 2, 4, 4, 7, 2, 4; 1, 4, 7, 2, 4, 4, 7, 2, 4; 1, 5, 8, 2, 5, 5, 8, 2, 5; 1, 6, 3, 2, 6, 6, 3, 2, 6; 1, 4, 7, 2, 4, 4, 7, 2, 4];
          // Function selection by [reset, set] reading, active high;
        public
          D.Delay.InertialDelaySensitiveVector delay(tHL = tHL, tLH = tLH, n = n) annotation(
            Placement(transformation(extent = {{20, 12}, {76, 68}})));
          D.Interfaces.DigitalInput set annotation(
            Placement(transformation(extent = {{-100, 72}, {-84, 88}})));
          D.Interfaces.DigitalInput reset annotation(
            Placement(transformation(extent = {{-100, -68}, {-84, -52}})));
          D.Interfaces.DigitalInput enable annotation(
            Placement(transformation(extent = {{-100, -28}, {-84, -12}})));
          D.Interfaces.DigitalInput dataIn[n] annotation(
            Placement(transformation(extent = {{-100, 32}, {-84, 48}})));
          D.Interfaces.DigitalOutput dataOut[n] annotation(
            Placement(transformation(extent = {{84, 32}, {100, 48}}), iconTransformation(extent = {{84, 32}, {100, 48}})));
          D.Registers.DLATSR dLATSR(n = n, ResetSetMap = ResetSetMap, strength = strength) annotation(
            Placement(transformation(extent = {{-78, -23}, {18, 74}})));
        equation
          connect(delay.y, dataOut) annotation(
            Line(points = {{72.08, 40}, {92, 40}}, color = {127, 0, 127}, thickness = 1, smooth = Smooth.None));
          connect(set, dLATSR.set) annotation(
            Line(points = {{-92, 80}, {-30, 80}, {-30, 69.15}}, color = {127, 0, 127}, smooth = Smooth.None));
          connect(reset, dLATSR.reset) annotation(
            Line(points = {{-92, -60}, {-30, -60}, {-30, -18.15}}, color = {127, 0, 127}, smooth = Smooth.None));
          connect(enable, dLATSR.enable) annotation(
            Line(points = {{-92, -20}, {-68.4, -20}, {-68.4, 15.8}}, color = {127, 0, 127}, smooth = Smooth.None));
          connect(dataIn, dLATSR.dataIn) annotation(
            Line(points = {{-92, 40}, {-80.2, 40}, {-80.2, 40.05}, {-68.4, 40.05}}, color = {127, 0, 127}, thickness = 1, smooth = Smooth.None));
          connect(dLATSR.dataOut, delay.x) annotation(
            Line(points = {{7.92, 40.05}, {23.92, 40.05}, {23.92, 39.86}}, color = {127, 0, 127}, thickness = 1, smooth = Smooth.None));
          annotation(
            Documentation(info = "<html>
<p>Description in VHDL is given by http://www.cs.sfu.ca/~ggbaker/reference/std_logic/src/std_logic_entities.vhd</p>
<p><b>Truth Table:</b></p>

<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"><b>DataIn</b></td>
      <td valign=\"top\"><b>Enable</b></td>
      <td valign=\"top\"><b>Reset</b></td>
      <td valign=\"top\"><b>Set</b></td>
      <td valign=\"top\"><b>DataOut</b></td>
  </tr>

  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">*</td>  <td valign=\"top\">U</td> <td valign=\"top\">U</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">U</td>  <td valign=\"top\">~1</td> <td valign=\"top\">U</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">*</td>  <td valign=\"top\">1</td> <td valign=\"top\">1</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">1</td>  <td valign=\"top\">0</td> <td valign=\"top\">0</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">1</td>  <td valign=\"top\">X</td> <td valign=\"top\">X</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">U</td> <td valign=\"top\">~1</td> <td valign=\"top\">~1</td> <td valign=\"top\">U</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">~U</td> <td valign=\"top\">X</td> <td valign=\"top\">X</td> <td valign=\"top\">X or U</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">~U</td> <td valign=\"top\">0</td> <td valign=\"top\">X</td> <td valign=\"top\">X or U or 1 or NC</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">~U</td> <td valign=\"top\">X</td> <td valign=\"top\">0</td> <td valign=\"top\">X or U or 0 or NC</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">X</td> <td valign=\"top\">0</td>  <td valign=\"top\">0</td> <td valign=\"top\">X or U or NC</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">1</td> <td valign=\"top\">0</td>  <td valign=\"top\">0</td> <td valign=\"top\">DataIn</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">0</td> <td valign=\"top\">0</td>  <td valign=\"top\">0</td> <td valign=\"top\">NC</td> </tr>

</table>

<PRE>
  *  = do not care
  ~  = not equal
  U  = L.'U'
  0  = L.'0' or L.'L'
  1  = L.'1' or L.'H'
  X  = L.'X' or L.'W' or L.'Z' or L.'-'
  NC = no change
</PRE>
</HTML>", revisions = "<html>
<ul>
<li><i>September 11, 2009 </i> created by Ulrich Donath<br>
       </li>
</ul>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-50, 70}, {52, -50}}, lineColor = {127, 0, 127}, lineThickness = 0.5, fillPattern = FillPattern.Solid, fillColor = {255, 255, 255}), Line(points = {{-86, 40}, {-50, 40}}, color = {127, 0, 127}, thickness = 1, smooth = Smooth.None), Line(points = {{-84, -20}, {-50, -20}}, color = {127, 0, 127}, smooth = Smooth.None), Line(points = {{52, 40}, {84, 40}}, color = {127, 0, 127}, smooth = Smooth.None, thickness = 1), Line(points = {{-50, -10}, {-36, -20}, {-50, -30}}, color = {127, 0, 127}, smooth = Smooth.None), Text(extent = {{-46, 96}, {46, -18}}, lineColor = {127, 0, 127}, textString = "DLATREGSR"), Text(extent = {{-14, -32}, {14, -45}}, lineColor = {127, 33, 107}, textString = "reset"), Text(extent = {{-33, -7}, {-1, -32}}, lineColor = {127, 33, 107}, textString = "enable"), Line(points = {{-84, 80}, {0, 80}, {0, 70}}, color = {127, 0, 127}, smooth = Smooth.None), Line(points = {{-84, -60}, {0, -60}, {0, -50}}, color = {127, 0, 127}, smooth = Smooth.None), Text(extent = {{-14, 67}, {12, 53}}, lineColor = {127, 33, 107}, textString = "set"), Text(extent = {{-28, 38}, {30, 10}}, lineColor = {127, 0, 127}, textString = "high active")}));
        end DLATREGSRH;

        model DLATREGSRL "Level sensitive register bank with set and reset, active low"
          extends Digital.Registers.DLATREGSRH(final ResetSetMap = [1, 1, 1, 1, 1, 1, 1, 1, 1; 1, 4, 2, 7, 4, 4, 2, 7, 4; 1, 6, 2, 3, 5, 5, 2, 3, 6; 1, 5, 2, 8, 6, 6, 2, 8, 5; 1, 4, 2, 7, 4, 4, 2, 7, 4; 1, 4, 2, 7, 4, 4, 2, 7, 4; 1, 6, 2, 3, 5, 5, 2, 3, 6; 1, 5, 2, 8, 6, 6, 2, 8, 5; 1, 4, 2, 7, 4, 4, 2, 7, 4]);
          // Function selection by [reset, set] reading;
          annotation(
            Documentation(info = "<html>
<p>Description in VHDL is given by http://www.cs.sfu.ca/~ggbaker/reference/std_logic/src/std_logic_entities.vhd</p>
<p><b>Truth Table</b></p>

<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"><b>DataIn</b></td>
      <td valign=\"top\"><b>Enable</b></td>
      <td valign=\"top\"><b>Reset</b></td>
      <td valign=\"top\"><b>Set</b></td>
      <td valign=\"top\"><b>DataOut</b></td>
  </tr>

  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">*</td>  <td valign=\"top\">U</td> <td valign=\"top\">U</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">U</td>  <td valign=\"top\">~0</td> <td valign=\"top\">U</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">*</td>  <td valign=\"top\">0</td> <td valign=\"top\">1</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">0</td>  <td valign=\"top\">1</td> <td valign=\"top\">0</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">0</td>  <td valign=\"top\">X</td> <td valign=\"top\">X</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">U</td> <td valign=\"top\">~0</td> <td valign=\"top\">~0</td> <td valign=\"top\">U</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">~U</td> <td valign=\"top\">X</td> <td valign=\"top\">X</td> <td valign=\"top\">X or U</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">~U</td> <td valign=\"top\">1</td> <td valign=\"top\">X</td> <td valign=\"top\">X or U or 1 or NC</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">~U</td> <td valign=\"top\">X</td> <td valign=\"top\">1</td> <td valign=\"top\">X or U or 0 or NC</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">X</td> <td valign=\"top\">1</td>  <td valign=\"top\">1</td> <td valign=\"top\">X or U or NC</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">1</td> <td valign=\"top\">1</td>  <td valign=\"top\">1</td> <td valign=\"top\">DataIn</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">0</td> <td valign=\"top\">1</td>  <td valign=\"top\">1</td> <td valign=\"top\">NC</td> </tr>
</table>

<PRE>
  *  = do not care
  ~  = not equal
  U  = L.'U'
  0  = L.'0' or L.'L'
  1  = L.'1' or L.'H'
  X  = L.'X' or L.'W' or L.'Z' or L.'-'
  NC = no change
</PRE>
</HTML>", revisions = "<html>
<ul>
<li><i>September 11, 2009 </i> created by Ulrich Donath<br>
       </li>
</ul>
</html>"),
            Icon(graphics = {Rectangle(extent = {{-40, 30}, {40, 16}}, pattern = LinePattern.None, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-30, 30}, {30, 16}}, lineColor = {127, 0, 127}, textString = "low active")}));
        end DLATREGSRL;
        annotation(
          Documentation(info = "<html>
<p>Registers is a collection of flipflops and latches. In the opposite to the Examples.Utilities models the Register models are a series of assignments in the algorithm part of the model. The model text is taken nearly identical from the standard logic text.</p>
</html>"));
      end Registers;

      package Tristates "Transfergates, Buffers, Inverters, and WiredX"
        extends Modelica.Icons.Package;
        import D = Modelica.Electrical.Digital;
        import L = Modelica.Electrical.Digital.Interfaces.Logic;
        import T = Modelica.Electrical.Digital.Tables;
        import S = Modelica.Electrical.Digital.Interfaces.Strength;

        model NXFERGATE "Transfergate with enable active high"
          parameter Modelica.SIunits.Time tHL = 0 "High->Low delay";
          parameter Modelica.SIunits.Time tLH = 0 "Low->High delay";
          D.Interfaces.DigitalInput enable annotation(
            Placement(transformation(extent = {{-100, 60}, {-80, 80}}), iconTransformation(extent = {{-100, 60}, {-80, 80}})));
          D.Interfaces.DigitalInput x annotation(
            Placement(transformation(extent = {{-100, -20}, {-80, 0}}), iconTransformation(extent = {{-100, -20}, {-80, 0}})));
          D.Interfaces.DigitalOutput y annotation(
            Placement(transformation(extent = {{80, -20}, {100, 0}}), iconTransformation(extent = {{80, -20}, {100, 0}})));
        protected
          D.Interfaces.Logic nextstate(start = L.'U');
          D.Interfaces.DigitalOutput yy(start = L.'U');
          D.Delay.InertialDelaySensitive inertialDelaySensitive(tLH = tLH, tHL = tHL, y(start = L.'U', fixed = true));
        algorithm
          nextstate := T.NXferTable[enable, x];
          yy := nextstate;
        equation
          connect(yy, inertialDelaySensitive.x);
          connect(inertialDelaySensitive.y, y);
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-60, 60}, {60, -20}}, lineColor = {127, 0, 127}, fillPattern = FillPattern.Solid, fillColor = {213, 170, 255}, lineThickness = 0.5), Line(points = {{-33, 16}, {31, 16}}, color = {127, 0, 127}, thickness = 1, smooth = Smooth.None), Line(points = {{-80, -10}, {-32, -10}, {-32, 10}, {30, 10}, {30, -10}, {80, -10}}, color = {127, 0, 127}, smooth = Smooth.None, thickness = 1), Text(extent = {{-7, 46}, {39, 18}}, lineColor = {127, 0, 127}, lineThickness = 1, textString = "N"), Line(points = {{-82, 70}, {0, 70}, {0, 16}}, color = {127, 0, 127}, thickness = 1, smooth = Smooth.None), Text(extent = {{-44, -24}, {30, -46}}, lineColor = {0, 0, 255}, fillColor = {213, 170, 255}, fillPattern = FillPattern.Solid, textString = "%name")}),
            Documentation(revisions = "<html>
<ul>
<li><i>January 15, 2010 </i> created by Ulrich Donath<br>
       </li>
</ul>
</html>", info = "<html>
<p>Description in VHDL is given by http://www.cs.sfu.ca/~ggbaker/reference/std_logic/src/std_logic_entities.vhd</p>
<p><b>Truth Table</b></p>
<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"><b>DataIn</b></td>
      <td valign=\"top\"><b>Enable</b></td>
      <td valign=\"top\"><b>DataOut</b></td>
  </tr>

  <tr><td valign=\"top\">*</td> <td valign=\"top\">U</td> <td valign=\"top\">U</td></tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">X</td> <td valign=\"top\">UX</td></tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">0</td> <td valign=\"top\">Z</td></tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">1</td> <td valign=\"top\">DataIn</td></tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">Z</td> <td valign=\"top\">UX</td></tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">W</td> <td valign=\"top\">UX</td></tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">L</td> <td valign=\"top\">Z</td></tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">H</td> <td valign=\"top\">DataIn</td></tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">-</td> <td valign=\"top\">UX</td></tr>
</table>

<PRE>
  UX: if dataIn == U then U else X
</PRE>
</html>"));
        end NXFERGATE;

        model NRXFERGATE "Transfergate with enable active high. Output strength reduced."
          parameter Modelica.SIunits.Time tHL = 0 "High->Low delay";
          parameter Modelica.SIunits.Time tLH = 0 "Low->High delay";
          D.Interfaces.DigitalInput enable annotation(
            Placement(transformation(extent = {{-100, 60}, {-80, 80}}), iconTransformation(extent = {{-100, 60}, {-80, 80}})));
          D.Interfaces.DigitalInput x annotation(
            Placement(transformation(extent = {{-100, -20}, {-80, 0}}), iconTransformation(extent = {{-100, -20}, {-80, 0}})));
          D.Interfaces.DigitalOutput y annotation(
            Placement(transformation(extent = {{80, -20}, {100, 0}}), iconTransformation(extent = {{80, -20}, {100, 0}})));
        protected
          D.Interfaces.Logic nextstate(start = L.'U');
          D.Interfaces.DigitalOutput yy(start = L.'U');
          D.Delay.InertialDelaySensitive inertialDelaySensitive(tLH = tLH, tHL = tHL, y(start = L.'U', fixed = true));
        algorithm
          nextstate := T.NRXferTable[enable, x];
          yy := nextstate;
        equation
          connect(yy, inertialDelaySensitive.x);
          connect(inertialDelaySensitive.y, y);
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-60, 60}, {60, -20}}, lineColor = {127, 0, 127}, fillPattern = FillPattern.Solid, fillColor = {213, 170, 255}, lineThickness = 0.5), Line(points = {{-33, 16}, {31, 16}}, color = {127, 0, 127}, thickness = 1, smooth = Smooth.None), Line(points = {{-80, -10}, {-32, -10}, {-32, 10}, {30, 10}, {30, -10}, {80, -10}}, color = {127, 0, 127}, smooth = Smooth.None, thickness = 1), Text(extent = {{-2, 46}, {44, 18}}, lineColor = {127, 0, 127}, lineThickness = 1, textString = "NR"), Line(points = {{-82, 70}, {0, 70}, {0, 16}}, color = {127, 0, 127}, thickness = 1, smooth = Smooth.None), Text(extent = {{-36, -24}, {38, -46}}, lineColor = {0, 0, 255}, fillColor = {213, 170, 255}, fillPattern = FillPattern.Solid, textString = "%name")}),
            Documentation(revisions = "<html>
<ul>
<li><i>January 15, 2010 </i> created by Ulrich Donath<br>
       </li>
</ul>
</html>", info = "<html>
<p>Description in VHDL is given by http://www.cs.sfu.ca/~ggbaker/reference/std_logic/src/std_logic_entities.vhd</p>
<p><b>Truth Table</b></p>
<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"><b>DataIn</b></td>
      <td valign=\"top\"><b>Enable</b></td>
      <td valign=\"top\"><b>DataOut</b></td>
  </tr>

  <tr><td valign=\"top\">*</td> <td valign=\"top\">U</td> <td valign=\"top\">U</td></tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">X</td> <td valign=\"top\">UW</td></tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">0</td> <td valign=\"top\">Z</td></tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">1</td> <td valign=\"top\">DataIn, Strength Reduced</td></tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">Z</td> <td valign=\"top\">UW</td></tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">W</td> <td valign=\"top\">UW</td></tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">L</td> <td valign=\"top\">Z</td></tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">H</td> <td valign=\"top\">DataIn, Strength Reduced</td></tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">-</td> <td valign=\"top\">UW</td></tr>
</table>

<PRE>
  UW: if dataIn == U then U else W
  Strength Reduced: 0 -> L, 1 -> H, X -> W
</PRE>
</html>"));
        end NRXFERGATE;

        model PXFERGATE "Transfergate with enable active low"
          parameter Modelica.SIunits.Time tHL = 0 "High->Low delay";
          parameter Modelica.SIunits.Time tLH = 0 "Low->High delay";
          D.Interfaces.DigitalInput enable annotation(
            Placement(transformation(extent = {{-100, 60}, {-80, 80}}), iconTransformation(extent = {{-100, 60}, {-80, 80}})));
          D.Interfaces.DigitalInput x annotation(
            Placement(transformation(extent = {{-100, -20}, {-80, 0}}), iconTransformation(extent = {{-100, -20}, {-80, 0}})));
          D.Interfaces.DigitalOutput y annotation(
            Placement(transformation(extent = {{80, -20}, {100, 0}}), iconTransformation(extent = {{80, -20}, {100, 0}})));
        protected
          D.Interfaces.Logic nextstate(start = L.'U');
          D.Interfaces.DigitalOutput yy(start = L.'U');
          D.Delay.InertialDelaySensitive inertialDelaySensitive(tLH = tLH, tHL = tHL);
        algorithm
          nextstate := T.PXferTable[enable, x];
          yy := nextstate;
        equation
          connect(yy, inertialDelaySensitive.x);
          connect(inertialDelaySensitive.y, y);
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-60, 60}, {60, -20}}, lineColor = {127, 0, 127}, fillPattern = FillPattern.Solid, fillColor = {213, 170, 255}, lineThickness = 0.5), Line(points = {{-33, 16}, {31, 16}}, color = {127, 0, 127}, thickness = 1, smooth = Smooth.None), Line(points = {{-80, -10}, {-32, -10}, {-32, 10}, {30, 10}, {30, -10}, {80, -10}}, color = {127, 0, 127}, smooth = Smooth.None, thickness = 1), Text(extent = {{-7, 46}, {39, 18}}, lineColor = {127, 0, 127}, lineThickness = 1, textString = "P"), Line(points = {{-82, 70}, {0, 70}, {0, 16}}, color = {127, 0, 127}, thickness = 1, smooth = Smooth.None), Text(extent = {{-36, -24}, {38, -46}}, lineColor = {0, 0, 255}, fillColor = {213, 170, 255}, fillPattern = FillPattern.Solid, textString = "%name")}),
            Documentation(revisions = "<html>
<ul>
<li><i>January 15, 2010 </i> created by Ulrich Donath<br>
       </li>
</ul>
</html>", info = "<html>
<p>Description in VHDL is given by http://www.cs.sfu.ca/~ggbaker/reference/std_logic/src/std_logic_entities.vhd</p>
<p><b>Truth Table</b></p>
<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"><b>DataIn</b></td>
      <td valign=\"top\"><b>Enable</b></td>
      <td valign=\"top\"><b>DataOut</b></td>
  </tr>

  <tr><td valign=\"top\">*</td> <td valign=\"top\">U</td> <td valign=\"top\">U</td></tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">X</td> <td valign=\"top\">UX</td></tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">0</td> <td valign=\"top\">DataIn</td></tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">1</td> <td valign=\"top\">Z</td></tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">Z</td> <td valign=\"top\">UX</td></tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">W</td> <td valign=\"top\">UX</td></tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">L</td> <td valign=\"top\">DataIn</td></tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">H</td> <td valign=\"top\">Z</td></tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">-</td> <td valign=\"top\">UX</td></tr>
</table>

<PRE>
  UX: if dataIn == U then U else X
</PRE>
</html>"));
        end PXFERGATE;

        model PRXFERGATE "Transfergate with enable active low. Output strength reduced."
          parameter Modelica.SIunits.Time tHL = 0 "High->Low delay";
          parameter Modelica.SIunits.Time tLH = 0 "Low->High delay";
          D.Interfaces.DigitalInput enable annotation(
            Placement(transformation(extent = {{-100, 60}, {-80, 80}}), iconTransformation(extent = {{-100, 60}, {-80, 80}})));
          D.Interfaces.DigitalInput x annotation(
            Placement(transformation(extent = {{-100, -20}, {-80, 0}}), iconTransformation(extent = {{-100, -20}, {-80, 0}})));
          D.Interfaces.DigitalOutput y annotation(
            Placement(transformation(extent = {{80, -20}, {100, 0}}), iconTransformation(extent = {{80, -20}, {100, 0}})));
        protected
          D.Interfaces.Logic nextstate(start = L.'U');
          D.Interfaces.DigitalOutput yy(start = L.'U');
          D.Delay.InertialDelaySensitive inertialDelaySensitive(tLH = tLH, tHL = tHL);
        algorithm
          nextstate := T.PRXferTable[enable, x];
          yy := nextstate;
        equation
          connect(yy, inertialDelaySensitive.x);
          connect(inertialDelaySensitive.y, y);
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-60, 60}, {60, -20}}, lineColor = {127, 0, 127}, fillPattern = FillPattern.Solid, fillColor = {213, 170, 255}, lineThickness = 0.5), Line(points = {{-33, 16}, {31, 16}}, color = {127, 0, 127}, thickness = 1, smooth = Smooth.None), Line(points = {{-80, -10}, {-32, -10}, {-32, 10}, {30, 10}, {30, -10}, {80, -10}}, color = {127, 0, 127}, smooth = Smooth.None, thickness = 1), Text(extent = {{-2, 46}, {44, 18}}, lineColor = {127, 0, 127}, lineThickness = 1, textString = "PR"), Line(points = {{-82, 70}, {0, 70}, {0, 16}}, color = {127, 0, 127}, thickness = 1, smooth = Smooth.None), Text(extent = {{-40, -24}, {34, -46}}, lineColor = {0, 0, 255}, fillColor = {213, 170, 255}, fillPattern = FillPattern.Solid, textString = "%name")}),
            Documentation(revisions = "<html>
<ul>
<li><i>January 15, 2010 </i> created by Ulrich Donath<br>
       </li>
</ul>
</html>", info = "<html>
<p>Description in VHDL is given by http://www.cs.sfu.ca/~ggbaker/reference/std_logic/src/std_logic_entities.vhd</p>
<p><b>Truth Table</b></p>
<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"><b>DataIn</b></td>
      <td valign=\"top\"><b>Enable</b></td>
      <td valign=\"top\"><b>DataOut</b></td>
  </tr>

  <tr><td valign=\"top\">*</td> <td valign=\"top\">U</td> <td valign=\"top\">U</td></tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">X</td> <td valign=\"top\">UW</td></tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">0</td> <td valign=\"top\">DataIn, Strength Reduced</td></tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">1</td> <td valign=\"top\">Z</td></tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">Z</td> <td valign=\"top\">UW</td></tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">W</td> <td valign=\"top\">UW</td></tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">L</td> <td valign=\"top\">DataIn, Strength Reduced</td></tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">H</td> <td valign=\"top\">Z</td></tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">-</td> <td valign=\"top\">UW</td></tr>
</table>
<p>
  UW: if dataIn == U then U else W
  Strength Reduced: 0 -> L, 1 -> H, X -> W
</p>
</html>"));
        end PRXFERGATE;

        model BUF3S "Tristate buffer with enable active high"
          parameter Modelica.SIunits.Time tHL = 0 "High->Low delay";
          parameter Modelica.SIunits.Time tLH = 0 "Low->High delay";
          parameter D.Interfaces.Strength strength = S.'S_X01' "output strength";
          D.Interfaces.DigitalInput enable annotation(
            Placement(transformation(extent = {{-100, 60}, {-80, 80}}), iconTransformation(extent = {{-100, 60}, {-80, 80}})));
          D.Interfaces.DigitalInput x annotation(
            Placement(transformation(extent = {{-100, -20}, {-80, 0}}), iconTransformation(extent = {{-100, -20}, {-80, 0}})));
          D.Interfaces.DigitalOutput y annotation(
            Placement(transformation(extent = {{80, -20}, {100, 0}}), iconTransformation(extent = {{80, -20}, {100, 0}})));
        protected
          D.Interfaces.Logic nextstate(start = L.'U');
          D.Interfaces.DigitalOutput yy(start = L.'U');
          D.Delay.InertialDelaySensitive inertialDelaySensitive(tLH = tLH, tHL = tHL, y(start = L.'U', fixed = true));
        algorithm
          nextstate := T.Buf3sTable[strength, T.UX01Table[enable], T.UX01Table[x]];
          yy := nextstate;
        equation
          connect(yy, inertialDelaySensitive.x);
          connect(inertialDelaySensitive.y, y);
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-80, 70}, {0, 70}, {0, 14}}, color = {127, 0, 127}, thickness = 1, smooth = Smooth.None), Polygon(points = {{-40, 40}, {-40, -60}, {40, -10}, {-40, 40}}, lineColor = {127, 0, 127}, smooth = Smooth.None, fillColor = {213, 170, 255}, fillPattern = FillPattern.Solid), Line(points = {{-80, -10}, {-40, -10}}, color = {127, 0, 127}, smooth = Smooth.None, thickness = 1), Line(points = {{40, -10}, {82, -10}}, color = {127, 0, 127}, smooth = Smooth.None, thickness = 1), Text(extent = {{-54, -62}, {20, -84}}, lineColor = {0, 0, 255}, fillColor = {213, 170, 255}, fillPattern = FillPattern.Solid, textString = "%name")}),
            Documentation(revisions = "<html>
<ul>
<li><i>January 22, 2010 </i> created by Ulrich Donath<br>
       </li>
</ul>
</html>", info = "<html>
<p>Description in VHDL is given by http://www.cs.sfu.ca/~ggbaker/reference/std_logic/src/std_logic_entities.vhd</p>
<p>and for tristate table http://www.cs.sfu.ca/~ggbaker/reference/std_logic/src/std_logic_misc.vhd</p>
<p><b>Truth Table</b></p>
<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"><b>DataIn</b></td>
      <td valign=\"top\"><b>Enable</b></td>
      <td valign=\"top\"><b>DataOut*</b></td>
  </tr>

  <tr><td valign=\"top\">*</td> <td valign=\"top\">U</td> <td valign=\"top\">U</td></tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">X</td> <td valign=\"top\">UX</td></tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">0</td> <td valign=\"top\">Z</td></tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">1</td> <td valign=\"top\">DataIn</td></tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">Z</td> <td valign=\"top\">UX</td></tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">W</td> <td valign=\"top\">UX</td></tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">L</td> <td valign=\"top\">Z</td></tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">H</td> <td valign=\"top\">DataIn</td></tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">-</td> <td valign=\"top\">UX</td></tr>
</table>

<PRE>
  UX: if dataIn == U then U else X
  DataOut*: Strength map for DataOut according to tristate table Buf3sTable
</PRE>
</html>"));
        end BUF3S;

        model BUF3SL "Tristate buffer with enable active low"
          parameter Modelica.SIunits.Time tHL = 0 "High->Low delay";
          parameter Modelica.SIunits.Time tLH = 0 "Low->High delay";
          parameter D.Interfaces.Strength strength = S.'S_X01' "output strength";
          D.Interfaces.DigitalInput enable annotation(
            Placement(transformation(extent = {{-100, 60}, {-80, 80}}), iconTransformation(extent = {{-100, 60}, {-80, 80}})));
          D.Interfaces.DigitalInput x annotation(
            Placement(transformation(extent = {{-100, -20}, {-80, 0}}), iconTransformation(extent = {{-100, -20}, {-80, 0}})));
          D.Interfaces.DigitalOutput y annotation(
            Placement(transformation(extent = {{80, -20}, {100, 0}}), iconTransformation(extent = {{80, -20}, {100, 0}})));
        protected
          D.Interfaces.Logic nextstate(start = L.'U');
          D.Interfaces.DigitalOutput yy(start = L.'U');
          D.Delay.InertialDelaySensitive inertialDelaySensitive(tLH = tLH, tHL = tHL);
        algorithm
          nextstate := T.Buf3slTable[strength, T.UX01Table[enable], T.UX01Table[x]];
          yy := nextstate;
        equation
          connect(yy, inertialDelaySensitive.x);
          connect(inertialDelaySensitive.y, y);
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-80, 70}, {0, 70}, {0, 14}}, color = {127, 0, 127}, thickness = 1, smooth = Smooth.None), Polygon(points = {{-40, 40}, {-40, -60}, {40, -10}, {-40, 40}}, lineColor = {127, 0, 127}, smooth = Smooth.None, fillColor = {213, 170, 255}, fillPattern = FillPattern.Solid), Line(points = {{-80, -10}, {-40, -10}}, color = {127, 0, 127}, smooth = Smooth.None, thickness = 1), Line(points = {{40, -10}, {82, -10}}, color = {127, 0, 127}, smooth = Smooth.None, thickness = 1), Text(extent = {{-54, -62}, {20, -84}}, lineColor = {0, 0, 255}, fillColor = {213, 170, 255}, fillPattern = FillPattern.Solid, textString = "%name")}),
            Documentation(revisions = "<html>
<ul>
<li><i>January 22, 2010 </i> created by Ulrich Donath<br>
       </li>
</ul>
</html>", info = "<html>
<p>Description in VHDL is given by http://www.cs.sfu.ca/~ggbaker/reference/std_logic/src/std_logic_entities.vhd</p>
<p>and for tristate table http://www.cs.sfu.ca/~ggbaker/reference/std_logic/src/std_logic_misc.vhd</p>
<p><b>Truth Table</b></p>
<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"><b>DataIn</b></td>
      <td valign=\"top\"><b>Enable</b></td>
      <td valign=\"top\"><b>DataOut*</b></td>
  </tr>

  <tr><td valign=\"top\">*</td> <td valign=\"top\">U</td> <td valign=\"top\">U</td></tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">X</td> <td valign=\"top\">UX</td></tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">0</td> <td valign=\"top\">DataIn</td></tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">1</td> <td valign=\"top\">Z</td></tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">Z</td> <td valign=\"top\">UX</td></tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">W</td> <td valign=\"top\">UX</td></tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">L</td> <td valign=\"top\">DataIn</td></tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">H</td> <td valign=\"top\">Z</td></tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">-</td> <td valign=\"top\">UX</td></tr>
</table>

<PRE>
  UX: if dataIn == U then U else X
  DataOut*: Strength map for DataOut according to tristate table Buf3slTable
</PRE>
</html>"));
        end BUF3SL;

        model INV3S "Tristate Inverter with enable active high"
          parameter Modelica.SIunits.Time tHL = 0 "High->Low delay";
          parameter Modelica.SIunits.Time tLH = 0 "Low->High delay";
          parameter D.Interfaces.Strength strength = S.'S_X01' "output strength";
          D.Interfaces.DigitalInput enable annotation(
            Placement(transformation(extent = {{-100, 60}, {-80, 80}}), iconTransformation(extent = {{-100, 60}, {-80, 80}})));
          D.Interfaces.DigitalInput x annotation(
            Placement(transformation(extent = {{-100, -20}, {-80, 0}}), iconTransformation(extent = {{-100, -20}, {-80, 0}})));
          D.Interfaces.DigitalOutput y annotation(
            Placement(transformation(extent = {{80, -20}, {100, 0}}), iconTransformation(extent = {{80, -20}, {100, 0}})));
        protected
          D.Interfaces.Logic nextstate(start = L.'U');
          D.Interfaces.DigitalOutput yy(start = L.'U');
          D.Delay.InertialDelaySensitive inertialDelaySensitive(tLH = tLH, tHL = tHL, y(start = L.'U', fixed = true));
        algorithm
          nextstate := T.Buf3sTable[strength, T.UX01Table[enable], T.NotTable[x]];
          yy := nextstate;
        equation
          connect(yy, inertialDelaySensitive.x);
          connect(inertialDelaySensitive.y, y);
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-80, 70}, {0, 70}, {0, 14}}, color = {127, 0, 127}, thickness = 1, smooth = Smooth.None), Polygon(points = {{-40, 40}, {-40, -60}, {40, -10}, {-40, 40}}, lineColor = {127, 0, 127}, smooth = Smooth.None, fillColor = {213, 170, 255}, fillPattern = FillPattern.Solid), Line(points = {{-80, -10}, {-40, -10}}, color = {127, 0, 127}, smooth = Smooth.None, thickness = 1), Line(points = {{40, -10}, {82, -10}}, color = {127, 0, 127}, smooth = Smooth.None, thickness = 1), Ellipse(extent = {{40, -3}, {54, -17}}, lineColor = {127, 0, 127}, fillColor = {213, 170, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-54, -62}, {20, -84}}, lineColor = {0, 0, 255}, fillColor = {213, 170, 255}, fillPattern = FillPattern.Solid, textString = "%name")}),
            Documentation(revisions = "<html>
<ul>
<li><i>January 22, 2010 </i> created by Ulrich Donath<br>
       </li>
</ul>
</html>", info = "<html>
<p>Description in VHDL is given by http://www.cs.sfu.ca/~ggbaker/reference/std_logic/src/std_logic_entities.vhd</p>
<p>and for tristate table http://www.cs.sfu.ca/~ggbaker/reference/std_logic/src/std_logic_misc.vhd</p>
<p><b>Truth Table</b></p>
<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"><b>DataIn</b></td>
      <td valign=\"top\"><b>Enable</b></td>
      <td valign=\"top\"><b>DataOut*</b></td>
  </tr>

  <tr><td valign=\"top\">*</td> <td valign=\"top\">U</td> <td valign=\"top\">U</td></tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">X</td> <td valign=\"top\">UX</td></tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">0</td> <td valign=\"top\">Z</td></tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">1</td> <td valign=\"top\">Not DataIn</td></tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">Z</td> <td valign=\"top\">UX</td></tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">W</td> <td valign=\"top\">UX</td></tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">L</td> <td valign=\"top\">Z</td></tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">H</td> <td valign=\"top\">Not DataIn</td></tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">-</td> <td valign=\"top\">UX</td></tr>
</table>

<PRE>
  UX: if dataIn == U then U else X
  DataOut*: Strength map for DataOut according to tristate table Buf3sTable
</PRE>
</html>"));
        end INV3S;

        model INV3SL "Tristate inverter with enable active low"
          parameter Modelica.SIunits.Time tHL = 0 "High->Low delay";
          parameter Modelica.SIunits.Time tLH = 0 "Low->High delay";
          parameter D.Interfaces.Strength strength = S.'S_X01' "output strength";
          D.Interfaces.DigitalInput enable annotation(
            Placement(transformation(extent = {{-100, 60}, {-80, 80}}), iconTransformation(extent = {{-100, 60}, {-80, 80}})));
          D.Interfaces.DigitalInput x annotation(
            Placement(transformation(extent = {{-100, -20}, {-80, 0}}), iconTransformation(extent = {{-100, -20}, {-80, 0}})));
          D.Interfaces.DigitalOutput y annotation(
            Placement(transformation(extent = {{80, -20}, {100, 0}}), iconTransformation(extent = {{80, -20}, {100, 0}})));
        protected
          D.Interfaces.Logic nextstate(start = L.'U');
          D.Interfaces.DigitalOutput yy(start = L.'U');
          D.Delay.InertialDelaySensitive inertialDelaySensitive(tLH = tLH, tHL = tHL);
        algorithm
          nextstate := T.Buf3sTable[strength, T.NotTable[enable], T.NotTable[x]];
          yy := nextstate;
        equation
          connect(yy, inertialDelaySensitive.x);
          connect(inertialDelaySensitive.y, y);
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{-40, 40}, {-40, -60}, {40, -10}, {-40, 40}}, lineColor = {127, 0, 127}, smooth = Smooth.None, fillColor = {213, 170, 255}, fillPattern = FillPattern.Solid), Line(points = {{-80, 70}, {0, 70}, {0, 14}}, color = {127, 0, 127}, thickness = 1, smooth = Smooth.None), Line(points = {{-80, -10}, {-40, -10}}, color = {127, 0, 127}, smooth = Smooth.None, thickness = 1), Line(points = {{40, -10}, {82, -10}}, color = {127, 0, 127}, smooth = Smooth.None, thickness = 1), Ellipse(extent = {{40, -3}, {54, -17}}, lineColor = {127, 0, 127}, fillColor = {213, 170, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-54, -62}, {20, -84}}, lineColor = {0, 0, 255}, fillColor = {213, 170, 255}, fillPattern = FillPattern.Solid, textString = "%name")}),
            Documentation(revisions = "<html>
<ul>
<li><i>January 22, 2010 </i> created by Ulrich Donath<br>
       </li>
</ul>
</html>", info = "<html>
<p>Description in VHDL is given by http://www.cs.sfu.ca/~ggbaker/reference/std_logic/src/std_logic_entities.vhd</p>
<p>and for tristate table http://www.cs.sfu.ca/~ggbaker/reference/std_logic/src/std_logic_misc.vhd</p>
<p><b>Truth Table</b></p>
<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"><b>DataIn</b></td>
      <td valign=\"top\"><b>Enable</b></td>
      <td valign=\"top\"><b>DataOut*</b></td>
  </tr>

  <tr><td valign=\"top\">*</td> <td valign=\"top\">U</td> <td valign=\"top\">U</td></tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">X</td> <td valign=\"top\">UX</td></tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">0</td> <td valign=\"top\">Not DataIn</td></tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">1</td> <td valign=\"top\">Z</td></tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">Z</td> <td valign=\"top\">UX</td></tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">W</td> <td valign=\"top\">UX</td></tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">L</td> <td valign=\"top\">Not DataIn</td></tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">H</td> <td valign=\"top\">Z</td></tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">-</td> <td valign=\"top\">UX</td></tr>
</table>

<PRE>
  UX: if dataIn == U then U else X
  DataOut*: Strength map for DataOut according to tristate table Buf3slTable
</PRE>
</html>"));
        end INV3SL;

        model WiredX "Wired node with multiple input and one output"
          extends D.Interfaces.MISO;
        protected
          D.Interfaces.Logic auxiliary[n](each start = L.'Z', each fixed = true);
        equation
          auxiliary[1] = x[1];
          for i in 1:n - 1 loop
            auxiliary[i + 1] = D.Tables.ResolutionTable[auxiliary[i], x[i + 1]];
          end for;
          y = pre(auxiliary[n]);
          annotation(
            Documentation(info = "<HTML>
<P>
Wires n input signals in one output signal, without delay.
</P>
<p>Resolution table is given by http://www.cs.sfu.ca/~ggbaker/reference/std_logic/src/std_logic_misc.vhd</p>
</HTML>", revisions = "<html>
<ul>
<li><i>January 22, 2010 </i> created by Ulrich Donath<br>
       </li>
</ul>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-50, 40}, {50, 80}}, lineColor = {0, 0, 0}, lineThickness = 0.5, textString = "Wire"), Text(extent = {{152, -160}, {-148, -100}}, lineColor = {0, 0, 255}, textString = "%name")}));
        end WiredX;
      end Tristates;

      package Memories
        extends Modelica.Icons.Package;

        model DLATRAM "Level sensitive Random Access Memory"
          import D = Modelica.Electrical.Digital;
          import L = Modelica.Electrical.Digital.Interfaces.Logic;
          import S = Modelica.Electrical.Digital.Interfaces.Strength;
          import T = Modelica.Electrical.Digital.Tables;
          parameter Modelica.SIunits.Time tHL = 0 "High->Low delay";
          parameter Modelica.SIunits.Time tLH = 0 "Low->High delay";
          parameter D.Interfaces.Strength strength = S.'S_X01' "output strength";
          parameter Integer n_addr(min = 1) = 2 "addr width";
          parameter Integer n_data(min = 1) = 2 "data width";
          parameter String fileName = Modelica.Utilities.Files.loadResource("modelica://Modelica/Resources/Data/Electrical/Digital/Memory_Matrix.txt") annotation(
            Dialog(group = "table data definition", enable = tableOnFile, loadSelector(filter = "Text files (*.txt);;Matlab files (*.mat)", caption = "Open file in which table is present")));
          D.Interfaces.DigitalInput RE "read enable" annotation(
            Placement(transformation(extent = {{-100, -30}, {-84, -14}}), iconTransformation(extent = {{-100, -28}, {-84, -12}})));
          D.Interfaces.DigitalInput WE "write enable" annotation(
            Placement(transformation(extent = {{-100, -60}, {-84, -44}}), iconTransformation(extent = {{-100, -58}, {-84, -42}})));
          D.Interfaces.DigitalInput addr[n_addr] "address" annotation(
            Placement(transformation(extent = {{-100, 50}, {-80, 70}}), iconTransformation(extent = {{-100, 50}, {-80, 70}})));
          D.Interfaces.DigitalInput dataIn[n_data] "data input" annotation(
            Placement(transformation(extent = {{-100, 10}, {-80, 30}})));
          D.Interfaces.DigitalOutput dataOut[n_data] "data output" annotation(
            Placement(transformation(extent = {{80, 10}, {100, 30}}), iconTransformation(extent = {{80, 10}, {100, 30}})));

          function getMemory
            extends Modelica.Icons.Function;
            input String filename;
            input Integer n_addr "addr width";
            input Integer n_data "data width";
            output D.Interfaces.Logic m[integer(2 ^ n_addr), n_data] "memory with data, lowest bit on left side";
            output String data;
            output Integer bit;
          algorithm
            for i in 1:2 ^ n_addr loop
              data := Modelica.Utilities.Streams.readLine(filename, integer(i));
              for j in 1:n_data loop
                bit := Modelica.Utilities.Strings.scanInteger(data, 2 * j - 1);
                if bit == 1 then
                  m[integer(i), j] := L.'1';
                elseif bit == 0 then
                  m[integer(i), j] := L.'0';
                else
                  m[integer(i), j] := L.'X';
                end if;
              end for;
            end for;
          end getMemory;
        public
          D.Interfaces.Logic mem[integer(2 ^ n_addr), n_data](start = fill(L.'U', integer(2 ^ n_addr), n_data));
        protected
          D.Interfaces.Logic nextstate[n_data](start = fill(L.'U', n_data));
          D.Interfaces.Logic mem_word[n_data](start = fill(L.'U', n_data));
          Integer int_addr;
          // Boolean addr_change;
          D.Interfaces.DigitalOutput yy[n_data](start = fill(L.'U', n_data));
          D.Delay.InertialDelaySensitive inertialDelaySensitive[n_data](each tLH = tLH, each tHL = tHL);

          function address
            // compute memory address
            extends Modelica.Icons.Function;
            input Integer n_addr;
            input L addr[n_addr];
            output Integer int_addr;
          protected
            L addr_bit;
          algorithm
            int_addr := 1;
            for i in 1:n_addr loop
              addr_bit := T.X01Table[addr[i]];
              if addr_bit == L.'1' then
                int_addr := int_addr + integer(2 ^ (i - 1));
              elseif addr_bit == L.'X' then
                int_addr := 0;
                break;
              end if;
            end for;
          end address;
        algorithm
          if initial() then
            mem := getMemory(fileName, n_addr, n_data);
          end if;
/* assert: no address change during write
addr_change := false;
for i in 1:n_addr loop
    if change(addr[i]) then
      addr_change := true;
    end if;
end for;

if WE == L.'1' or WE == L.'H' then
  assert(not addr_change, "Address lines changed while RAM is Write Enabled");
end if;
*/
/* write into memory */
          if WE == L.'1' or WE == L.'H' then
            int_addr := address(n_addr, addr);
            if int_addr > 0 then
              for i in 1:n_data loop
                mem_word[i] := T.X01Table[dataIn[i]];
              end for;
              mem[int_addr, 1:n_data] := mem_word;
            end if;
          elseif WE == L.'X' or WE == L.'W' or WE == L.'Z' or WE == L.'U' or WE == L.'-' then
            int_addr := address(n_addr, addr);
            if int_addr > 0 then
              mem_word := fill(L.'X', n_data);
              mem[int_addr, 1:n_data] := mem_word;
            end if;
          end if;
// write data
//assert(int_addr > 0, "Attempted write to bad RAM address");
// write X
//assert(int_addr > 0, "Attempted write to bad RAM address");
/* read from memory */
          if RE == L.'0' or RE == L.'L' then
            nextstate := fill(L.'Z', n_data);
          elseif RE == L.'1' or RE == L.'H' then
            int_addr := address(n_addr, addr);
            if int_addr > 0 then
              mem_word := mem[int_addr, 1:n_data];
              for i in 1:n_data loop
                nextstate[i] := T.StrengthMap[mem_word[i], strength];
              end for;
            else
              nextstate := fill(T.StrengthMap[L.'X', strength], n_data);
            end if;
          else
            nextstate := fill(T.StrengthMap[L.'X', strength], n_data);
          end if;
// read data
          yy := nextstate;
        equation
          for i in 1:n_data loop
            connect(yy[i], inertialDelaySensitive[i].x);
            connect(inertialDelaySensitive[i].y, dataOut[i]);
          end for;
          annotation(
            Documentation(info = "<html>
<p>
Description in VHDL is given by <a href=\"http://www.cs.sfu.ca/~ggbaker/reference/std_logic/src/std_logic_entities.vhd\">http://www.cs.sfu.ca/~ggbaker/reference/std_logic/src/std_logic_entities.vhd</a>
</p>

<p><b>Truth Table for high active read enable RE:</b></p>
<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"><b>RE</b></td>
      <td valign=\"top\"><b>Addr</b></td>
      <td valign=\"top\"><b>DataOut</b></td>
  </tr>
  <tr><td valign=\"top\">0</td>  <td valign=\"top\">*</td>              <td valign=\"top\">Z over all</td>  </tr>
  <tr><td valign=\"top\">1</td>  <td valign=\"top\">  no X in Addr</td> <td valign=\"top\">DataOut=m(Addr)</td>     </tr>
  <tr><td valign=\"top\">1</td>  <td valign=\"top\">X in Addr</td>      <td valign=\"top\">X over all</td>  </tr>
  <tr><td valign=\"top\">X</td>  <td valign=\"top\">*</td>              <td valign=\"top\">X over all</td>  </tr>
</table>
<p><b>Truth Table for high active write enable WE:</b></p>
<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"><b>WE</b></td>
      <td valign=\"top\"><b>Addr</b></td>
      <td valign=\"top\"><b>Memory</b></td>
  </tr>
  <tr><td valign=\"top\">0</td>  <td valign=\"top\">*</td>              <td valign=\"top\">no write</td>           </tr>
  <tr><td valign=\"top\">1</td>  <td valign=\"top\">no X in Addr</td>   <td valign=\"top\">m(Addr)=DataIn</td>     </tr>
  <tr><td valign=\"top\">1</td>  <td valign=\"top\">X in Addr</td>      <td valign=\"top\">no write</td>  </tr>
  <tr><td valign=\"top\">X</td>  <td valign=\"top\">no X in Addr</td>   <td valign=\"top\">m(Addr)=X over all</td> </tr>
  <tr><td valign=\"top\">X</td>  <td valign=\"top\">X in Addr</td>      <td valign=\"top\">no write</td>  </tr>
</table>

<PRE>

  *  = do not care
  0  = L.'0' or L.'L'
  1  = L.'1' or L.'H'
  X  = L.'X' or L.'W' or L.'Z' or L.'-' or L.'U'
  Z  = L.'Z'
</PRE>

<p><b>Simultaneous read/write operations are allowed.
Firstly Write is carried out, then Read.</b></p>
</html>", revisions = "<html>
<dl>
<dt><i>November 9, 2010 </i></dt>
<dd>created by Ulrich Donath</dd>
</dl>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-60, 80}, {60, -80}}, lineColor = {127, 0, 127}, lineThickness = 0.5), Text(extent = {{-46, 102}, {46, -2}}, lineColor = {127, 33, 107}, textString = "DLATRAM"), Line(points = {{-84, 60}, {-60, 60}}, color = {127, 0, 127}, thickness = 1, smooth = Smooth.None), Line(points = {{60, 20}, {84, 20}}, color = {127, 0, 127}, thickness = 1, smooth = Smooth.None), Line(points = {{-84, -20}, {-60, -20}}, color = {127, 0, 127}, smooth = Smooth.None), Line(points = {{-60, -10}, {-46, -20}, {-60, -30}}, color = {127, 0, 127}, smooth = Smooth.None), Text(extent = {{-41, -5}, {-24, -34}}, lineColor = {127, 33, 107}, textString = "RE"), Line(points = {{-60, -40}, {-46, -50}, {-60, -60}}, color = {127, 0, 127}, smooth = Smooth.None), Text(extent = {{-41, -35}, {-24, -62}}, lineColor = {127, 33, 107}, textString = "WE"), Line(points = {{-84, -50}, {-60, -50}}, color = {127, 0, 127}, smooth = Smooth.None), Line(points = {{-80, 20}, {-60, 20}}, color = {127, 0, 127}, smooth = Smooth.None, thickness = 1)}));
        end DLATRAM;

        model DLATROM "Level sensitive Read Only Memory"
          import D = Modelica.Electrical.Digital;
          import L = Modelica.Electrical.Digital.Interfaces.Logic;
          import S = Modelica.Electrical.Digital.Interfaces.Strength;
          import T = Modelica.Electrical.Digital.Tables;
          parameter Modelica.SIunits.Time tHL = 0 "High->Low delay";
          parameter Modelica.SIunits.Time tLH = 0 "Low->High delay";
          parameter D.Interfaces.Strength strength = S.'S_X01' "output strength";
          parameter Integer n_addr(min = 1) = 2 "addr width";
          parameter Integer n_data(min = 1) = 2 "data width";
          parameter String fileName = Modelica.Utilities.Files.loadResource("modelica://Modelica/Resources/Data/Electrical/Digital/Memory_Matrix.txt") "File where matrix for memory is stored" annotation(
            Dialog(group = "table data definition", enable = tableOnFile, loadSelector(filter = "Text files (*.txt);;Matlab files (*.mat)", caption = "Open file in which table is present")));
          D.Interfaces.DigitalInput RE "read enable" annotation(
            Placement(transformation(extent = {{-100, -30}, {-80, -10}}), iconTransformation(extent = {{-100, -27}, {-86, -13}})));
          D.Interfaces.DigitalInput addr[n_addr] "address" annotation(
            Placement(transformation(extent = {{-100, 50}, {-80, 70}}), iconTransformation(extent = {{-100, 52}, {-84, 68}})));
          D.Interfaces.DigitalOutput dataOut[n_data] "data output" annotation(
            Placement(transformation(extent = {{80, 8}, {100, 28}}), iconTransformation(extent = {{84, 12}, {100, 29}})));

          function getMemory
            extends Modelica.Icons.Function;
            input String filename;
            input Integer n_addr "addr width";
            input Integer n_data "data width";
            output D.Interfaces.Logic m[integer(2 ^ n_addr), n_data] "memory with data, lowest bit on left side";
            output String data;
            output Integer bit;
          algorithm
            for i in 1:2 ^ n_addr loop
              data := Modelica.Utilities.Streams.readLine(filename, integer(i));
              for j in 1:n_data loop
                bit := Modelica.Utilities.Strings.scanInteger(data, 2 * j - 1);
                if bit == 1 then
                  m[integer(i), j] := L.'1';
                elseif bit == 0 then
                  m[integer(i), j] := L.'0';
                else
                  m[integer(i), j] := L.'X';
                end if;
              end for;
            end for;
          end getMemory;
        protected
          D.Interfaces.Logic m[integer(2 ^ n_addr), n_data](start = fill(L.'U', integer(2 ^ n_addr), n_data)) "memory with data, lowest bit on left side";
          D.Interfaces.Logic nextstate[n_data](start = fill(L.'U', n_data));
          //D.Interfaces.Logic mem[integer(2^n_addr),n_data](start=fill(L.'U',integer(2^n_addr),n_data));
          D.Interfaces.Logic mem_word[n_data](start = fill(L.'U', n_data));
          D.Interfaces.Logic addr_bit;
          Integer int_addr;
          D.Interfaces.DigitalOutput yy[n_data](start = fill(L.'U', n_data));
          D.Delay.InertialDelaySensitive inertialDelaySensitive[n_data](each tLH = tLH, each tHL = tHL);
        algorithm
          if initial() then
            m := getMemory(fileName, n_addr, n_data);
          end if;
          if RE == L.'0' or RE == L.'L' then
            nextstate := fill(L.'Z', n_data);
          elseif RE == L.'1' or RE == L.'H' then
            int_addr := 1;
            for i in 1:n_addr loop
              addr_bit := T.X01Table[addr[i]];
              if addr_bit == L.'1' then
                int_addr := int_addr + integer(2 ^ (i - 1));
              elseif addr_bit == L.'X' then
                int_addr := 0;
                break;
              end if;
            end for;
            if int_addr > 0 then
              mem_word := m[int_addr, 1:n_data];
              for i in 1:n_data loop
                nextstate[i] := T.StrengthMap[mem_word[i], strength];
              end for;
            else
              nextstate := fill(T.StrengthMap[L.'X', strength], n_data);
            end if;
          else
            nextstate := fill(T.StrengthMap[L.'X', strength], n_data);
          end if;
// compute memory address
// read data
          yy := nextstate;
        equation
          for i in 1:n_data loop
            connect(yy[i], inertialDelaySensitive[i].x);
            connect(inertialDelaySensitive[i].y, dataOut[i]);
          end for;
          annotation(
            Documentation(info = "<html>
<p>
Description in VHDL is given by <a href=\"http://www.cs.sfu.ca/~ggbaker/reference/std_logic/src/std_logic_entities.vhd\">http://www.cs.sfu.ca/~ggbaker/reference/std_logic/src/std_logic_entities.vhd</a>
</p>
<p><b>Truth Table for high active read enable RE:</b></p>

<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"><b>RE</b></td>
      <td valign=\"top\"><b>Addr</b></td>
      <td valign=\"top\"><b>DataOut</b></td>
  </tr>

  <tr><td valign=\"top\">0</td> <td valign=\"top\">*</td> <td valign=\"top\">Z over all</td>  </tr>
  <tr><td valign=\"top\">1</td> <td valign=\"top\">  no X in Addr</td> <td valign=\"top\">DataOut=m(Addr)</td>  </tr>
  <tr><td valign=\"top\">1</td> <td valign=\"top\">X in Addr</td> <td valign=\"top\">X over all</td> </tr>
  <tr><td valign=\"top\">X</td> <td valign=\"top\">*</td> <td valign=\"top\">X over all</td> </tr>
</table>

<PRE>

  *  = do not care
  0  = L.'0' or L.'L'
  1  = L.'1' or L.'H'
  X  = L.'X' or L.'W' or L.'Z' or L.'-' or L.'U'
  Z  = L.'Z'
</PRE>

</html>", revisions = "<html>
<dl>
<dt><i>October 19, 2010</i></dt>
<dd>created by Ulrich Donath</dd>
</dl>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-60, 80}, {60, -80}}, lineColor = {127, 0, 127}, lineThickness = 0.5), Text(extent = {{-46, 102}, {46, -2}}, lineColor = {127, 33, 107}, textString = "DLATROM"), Line(points = {{-84, 60}, {-60, 60}}, color = {127, 0, 127}, thickness = 1, smooth = Smooth.None), Line(points = {{60, 20}, {84, 20}}, color = {127, 0, 127}, thickness = 1, smooth = Smooth.None), Line(points = {{-88, -20}, {-60, -20}}, color = {127, 0, 127}, smooth = Smooth.None), Line(points = {{-60, -10}, {-46, -20}, {-60, -30}}, color = {127, 0, 127}, smooth = Smooth.None), Text(extent = {{-41, -5}, {-24, -34}}, lineColor = {127, 33, 107}, textString = "RE")}));
        end DLATROM;
      end Memories;

      package Multiplexers
        extends Modelica.Icons.Package;

        model MUX2x1 "A two inputs MULTIPLEXER for multiple value logic
        (2 data inputs, 1 select input, 1 output)"
          import D = Modelica.Electrical.Digital;
          import L = Modelica.Electrical.Digital.Interfaces.Logic;
          import S = Modelica.Electrical.Digital.Interfaces.Strength;
          import T = Modelica.Electrical.Digital.Tables;
          parameter Modelica.SIunits.Time tHL = 0 "High->Low delay";
          parameter Modelica.SIunits.Time tLH = 0 "Low->High delay";
          parameter D.Interfaces.Strength strength = S.'S_X01' "output strength";
          D.Interfaces.DigitalInput in1 "data input 1" annotation(
            Placement(transformation(extent = {{-100, 40}, {-80, 60}}), iconTransformation(extent = {{-100, 40}, {-80, 60}})));
          D.Interfaces.DigitalInput in0 "data input 0" annotation(
            Placement(transformation(extent = {{-100, -60}, {-80, -40}}), iconTransformation(extent = {{-100, -60}, {-80, -40}})));
          D.Interfaces.DigitalInput sel "select input" annotation(
            Placement(transformation(extent = {{-10, 80}, {10, 100}}), iconTransformation(extent = {{-10, 80}, {10, 100}})));
          D.Interfaces.DigitalOutput out "output" annotation(
            Placement(transformation(extent = {{80, -10}, {100, 10}}), iconTransformation(extent = {{80, -10}, {100, 10}})));
        protected
          D.Interfaces.Logic nextstate(start = L.'U');
          D.Interfaces.DigitalOutput yy(start = L.'U');
          D.Delay.InertialDelaySensitive inertialDelaySensitive(tLH = tLH, tHL = tHL, y(start = L.'U', fixed = true));
        algorithm
          nextstate := T.MUX2x1Table[T.UX01Table[in1], T.UX01Table[sel], T.UX01Table[in0]];
          nextstate := T.StrengthMap[nextstate, strength];
          yy := nextstate;
        equation
          connect(yy, inertialDelaySensitive.x);
          connect(inertialDelaySensitive.y, out);
          annotation(
            Icon(graphics = {Line(points = {{-80, 50}, {-40, 50}}, color = {127, 0, 127}, smooth = Smooth.None, thickness = 1), Line(points = {{-62, -50}, {-62, -50}}, color = {127, 0, 127}, smooth = Smooth.None, thickness = 1), Line(points = {{-80, -50}, {-40, -50}}, color = {127, 0, 127}, thickness = 1, smooth = Smooth.None), Line(points = {{0, 60}, {0, 80}}, color = {127, 0, 127}, thickness = 1, smooth = Smooth.None), Line(points = {{40, 0}, {80, 0}}, color = {127, 0, 127}, thickness = 1, smooth = Smooth.None), Polygon(points = {{-40, 80}, {-40, -80}, {40, -40}, {40, 40}, {-40, 80}}, lineColor = {213, 170, 255}, lineThickness = 1, smooth = Smooth.None, fillColor = {213, 170, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-36, -70}, {38, -92}}, lineColor = {0, 0, 255}, fillColor = {213, 170, 255}, fillPattern = FillPattern.Solid, textString = "%name"), Text(extent = {{-38, 10}, {38, -10}}, lineColor = {127, 0, 127}, lineThickness = 1, fillColor = {127, 0, 127}, fillPattern = FillPattern.Solid, textString = "MUX2x1")}),
            Documentation(info = "<html>
<p>Description in VHDL is given by <a href=\"http://www.cs.sfu.ca/~ggbaker/reference/std_logic/src/std_logic_entities.vhd\">http://www.cs.sfu.ca/~ggbaker/reference/std_logic/src/std_logic_entities.vhd</a></p>
<p>and for Multiplexer table <a href=\"http://www.cs.sfu.ca/~ggbaker/reference/std_logic/src/std_logic_misc.vhd\">http://www.cs.sfu.ca/~ggbaker/reference/std_logic/src/std_logic_misc.vhd</a></p>
<h4>Truth Table</h4>
<table cellspacing=\"0\" cellpadding=\"2\" border=\"1\"><tr>
<td valign=\"top\"><h4>DataIn</h4></td>
<td valign=\"top\"><h4>Select</h4></td>
<td valign=\"top\"><h4>DataOut</h4></td>
</tr>
<tr>
<td valign=\"top\"><p>*</p></td>
<td valign=\"top\"><p>0</p></td>
<td valign=\"top\"><p>Input0</p></td>
</tr>
<tr>
<td valign=\"top\"><p>*</p></td>
<td valign=\"top\"><p>1</p></td>
<td valign=\"top\"><p>Input1</p></td>
</tr>
<tr>
<td valign=\"top\"><p>Inputs equal</p></td>
<td valign=\"top\"><p>U</p></td>
<td valign=\"top\"><p>Input</p></td>
</tr>
<tr>
<td valign=\"top\"><p>Inputs not equal</p></td>
<td valign=\"top\"><p>U</p></td>
<td valign=\"top\"><p>U</p></td>
</tr>
<tr>
<td valign=\"top\"><p>U in Input</p></td>
<td valign=\"top\"><p>X</p></td>
<td valign=\"top\"><p>U</p></td>
</tr>
<tr>
<td valign=\"top\"><p>Inputs equal</p></td>
<td valign=\"top\"><p>X</p></td>
<td valign=\"top\"><p>Input</p></td>
</tr>
<tr>
<td valign=\"top\"><p>no U in Input and Inputs not equal</p></td>
<td valign=\"top\"><p>X</p></td>
<td valign=\"top\"><p>X</p></td>
</tr>
</table>
<pre>

  *  = don&apos;t care
  0  = L.&apos;0&apos; or L.&apos;L&apos;
  1  = L.&apos;1&apos; or L.&apos;H&apos;
  X  = L.&apos;X&apos; or L.&apos;W&apos; or L.&apos;Z&apos; or L.&apos;-&apos;
  U  = L.&apos;U&apos; </pre>
</html>", revisions = "<html>
<dl>
<dt><i>January 24, 2011</i></dt>
<dd>created by Christian G&uuml;nther</dd>
</dl>
</html>"));
        end MUX2x1;
      end Multiplexers;
      annotation(
        Documentation(info = "<html>
<p>
This library contains packages for digital electrical components. Both, type system
and models are based on the VHDL standard (IEEE Std 1076-1987 VHDL, IEEE Std 1076-1993 VHDL,
IEEE Std 1164 Multivalue Logic System):
</p>

<ul>
<li>Interfaces: Definition of signals and interfaces</li>
<li>Tables: All truth tables needed</li>
<li>Delay: Transport and inertial delay</li>
<li>Basic: Basic logic without delay</li>
<li>Gates: Basic gates composed by basic components and inertial delay</li>
<li>Tristate: (not yet available)</li>
<li>FlipFlops: D-Flip-Flops</li>
<li>Latches: D-Latches</li>
<li>TransferGates: (not yet available)</li>
<li>Multiplexers (not yet available)</li>
<li>Memory: Ram, Rom, (not yet available)</li>
<li>Sources: Time-dependent signal sources</li>
<li>Converters</li>
<li>Examples</li>
</ul>

<p>
The logic values are coded by integer values. The following code table is necessary
for both setting of input and interpreting the output values.
</p>

<p><b>Code Table:</b></p>

<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"><b>Logic value</b></td>
      <td valign=\"top\"><b>Integer code</b></td>
      <td valign=\"top\"><b>Meaning</b></td>
  </tr>

  <tr><td valign=\"top\">'U'</td> <td valign=\"top\">1</td> <td valign=\"top\">Uninitialized</td></tr>
  <tr><td valign=\"top\">'X'</td> <td valign=\"top\">2</td> <td valign=\"top\">Forcing Unknown</td></tr>
  <tr><td valign=\"top\">'0'</td> <td valign=\"top\">3</td> <td valign=\"top\">Forcing 0</td></tr>
  <tr><td valign=\"top\">'1'</td> <td valign=\"top\">4</td> <td valign=\"top\">Forcing 1</td></tr>
  <tr><td valign=\"top\">'Z'</td> <td valign=\"top\">5</td> <td valign=\"top\">High Impedance</td></tr>
  <tr><td valign=\"top\">'W'</td> <td valign=\"top\">6</td> <td valign=\"top\">Weak Unknown</td></tr>
  <tr><td valign=\"top\">'L'</td> <td valign=\"top\">7</td> <td valign=\"top\">Weak 0</td></tr>
  <tr><td valign=\"top\">'H'</td> <td valign=\"top\">8</td> <td valign=\"top\">Weak 1</td></tr>
  <tr><td valign=\"top\">'-'</td> <td valign=\"top\">9</td> <td valign=\"top\">Do not care</td></tr>
</table>

<p>
The library will be developed in two main steps. The first step contains the basic components and
the gates. In the next step the more complicated devices will be added. Currently the first step of
the library is implemented and released for public use.
</p>

<p>
Copyright &copy; 1998-2015, Modelica Association and Fraunhofer-Gesellschaft.
</p>

<p>
<i>This Modelica package is <u>free</u> software and the use is completely at <u>your own risk</u>; it can be redistributed and/or modified under the terms of the Modelica License 2. For license conditions (including the disclaimer of warranty) see <a href=\"modelica://Modelica.UsersGuide.ModelicaLicense2\">Modelica.UsersGuide.ModelicaLicense2</a> or visit <a href=\"https://www.modelica.org/licenses/ModelicaLicense2\"> https://www.modelica.org/licenses/ModelicaLicense2</a>.</i>
</p>
</html>"),
        Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}, graphics = {Line(origin = {7, 47}, points = {{-84, -6}, {-52, -6}}), Rectangle(origin = {59, 53}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, extent = {{-104, -63}, {-64, 7}}), Rectangle(origin = {146, 34}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, extent = {{-104, -63}, {-64, 7}}), Line(origin = {7, 15}, points = {{-84, -6}, {-52, -6}}), Line(origin = {79, 30}, points = {{-84, -6}, {-37, -6}}), Line(points = {{42, -12}, {17, -12}, {17, -54}, {-71, -54}}, color = {0, 0, 0}, smooth = Smooth.None)})));
    end Digital;

    package Machines "Library for electric machines"
      extends Modelica.Icons.Package;

      package Examples "Test examples"
        extends Modelica.Icons.ExamplesPackage;

        package AsynchronousInductionMachines "Test examples of asynchronous induction machines"
          extends Modelica.Icons.ExamplesPackage;

          model AIMC_DOL "Test example: AsynchronousInductionMachineSquirrelCage direct-on-line"
            extends Modelica.Icons.Example;
            constant Integer m = 3 "Number of phases";
            parameter Modelica.SIunits.Voltage VNominal = 100 "Nominal RMS voltage per phase";
            parameter Modelica.SIunits.Frequency fNominal = 50 "Nominal frequency";
            parameter Modelica.SIunits.Time tStart1 = 0.1 "Start time";
            parameter Modelica.SIunits.Torque TLoad = 161.4 "Nominal load torque";
            parameter Modelica.SIunits.AngularVelocity wLoad(displayUnit = "1/min") = 1440.45 * 2 * Modelica.Constants.pi / 60 "Nominal load speed";
            parameter Modelica.SIunits.Inertia JLoad = 0.29 "Load's moment of inertia";
            Machines.BasicMachines.AsynchronousInductionMachines.AIM_SquirrelCage aimc(p = aimcData.p, fsNominal = aimcData.fsNominal, Rs = aimcData.Rs, TsRef = aimcData.TsRef, alpha20s(displayUnit = "1/K") = aimcData.alpha20s, Lszero = aimcData.Lszero, Lssigma = aimcData.Lssigma, Jr = aimcData.Jr, Js = aimcData.Js, frictionParameters = aimcData.frictionParameters, phiMechanical(fixed = true), wMechanical(fixed = true), statorCoreParameters = aimcData.statorCoreParameters, strayLoadParameters = aimcData.strayLoadParameters, Lm = aimcData.Lm, Lrsigma = aimcData.Lrsigma, Rr = aimcData.Rr, TrRef = aimcData.TrRef, TsOperational = 293.15, alpha20r = aimcData.alpha20r, TrOperational = 293.15) annotation(
              Placement(transformation(extent = {{-20, -50}, {0, -30}}, rotation = 0)));
            Machines.Sensors.CurrentQuasiRMSSensor currentQuasiRMSSensor annotation(
              Placement(transformation(origin = {0, 0}, extent = {{-10, 10}, {10, -10}}, rotation = 270)));
            Modelica.Electrical.MultiPhase.Sources.SineVoltage sineVoltage(final m = m, freqHz = fill(fNominal, m), V = fill(sqrt(2 / 3) * VNominal, m)) annotation(
              Placement(transformation(origin = {0, 60}, extent = {{10, -10}, {-10, 10}}, rotation = 270)));
            Modelica.Electrical.MultiPhase.Basic.Star star(final m = m) annotation(
              Placement(transformation(extent = {{-50, 80}, {-70, 100}}, rotation = 0)));
            Modelica.Electrical.Analog.Basic.Ground ground annotation(
              Placement(transformation(origin = {-90, 90}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
            Modelica.Blocks.Sources.BooleanStep booleanStep[m](each startTime = tStart1) annotation(
              Placement(transformation(extent = {{-80, 30}, {-60, 50}}, rotation = 0)));
            Modelica.Electrical.MultiPhase.Ideal.IdealClosingSwitch idealCloser(final m = m, Ron = fill(1e-5, m), Goff = fill(1e-5, m)) annotation(
              Placement(transformation(origin = {0, 30}, extent = {{-10, 10}, {10, -10}}, rotation = 270)));
            Modelica.Mechanics.Rotational.Components.Inertia loadInertia(J = JLoad) annotation(
              Placement(transformation(extent = {{40, -50}, {60, -30}}, rotation = 0)));
            Modelica.Mechanics.Rotational.Sources.QuadraticSpeedDependentTorque quadraticLoadTorque(w_nominal = wLoad, TorqueDirection = false, tau_nominal = -TLoad, useSupport = false) annotation(
              Placement(transformation(extent = {{90, -50}, {70, -30}}, rotation = 0)));
            Machines.Utilities.TerminalBox terminalBox(terminalConnection = "D") annotation(
              Placement(transformation(extent = {{-20, -30}, {0, -10}}, rotation = 0)));
            parameter Utilities.ParameterRecords.AIM_SquirrelCageData aimcData annotation(
              Placement(transformation(extent = {{-20, -80}, {0, -60}})));
          initial equation
            aimc.is = zeros(3);
            aimc.ir = zeros(2);
          equation
            connect(star.pin_n, ground.p) annotation(
              Line(points = {{-70, 90}, {-80, 90}}, color = {0, 0, 255}));
            connect(sineVoltage.plug_n, star.plug_p) annotation(
              Line(points = {{0, 70}, {0, 90}, {-50, 90}}, color = {0, 0, 255}));
            connect(sineVoltage.plug_p, idealCloser.plug_p) annotation(
              Line(points = {{0, 50}, {0, 48}, {0, 46}, {0, 40}}, color = {0, 0, 255}));
            connect(booleanStep.y, idealCloser.control) annotation(
              Line(points = {{-59, 40}, {-20, 40}, {-20, 30}, {-7, 30}}, color = {255, 0, 255}));
            connect(idealCloser.plug_n, currentQuasiRMSSensor.plug_p) annotation(
              Line(points = {{0, 20}, {0, 17}, {0, 10}}, color = {0, 0, 255}));
            connect(terminalBox.plug_sn, aimc.plug_sn) annotation(
              Line(points = {{-16, -30}, {-16, -30}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(terminalBox.plug_sp, aimc.plug_sp) annotation(
              Line(points = {{-4, -30}, {-4, -30}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(terminalBox.plugSupply, currentQuasiRMSSensor.plug_n) annotation(
              Line(points = {{-10, -28}, {-10, -20}, {0, -20}, {0, -10}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(loadInertia.flange_b, quadraticLoadTorque.flange) annotation(
              Line(points = {{60, -40}, {70, -40}}, color = {0, 0, 0}, smooth = Smooth.None));
            connect(aimc.flange, loadInertia.flange_a) annotation(
              Line(points = {{0, -40}, {40, -40}}, color = {0, 0, 0}, smooth = Smooth.None));
            annotation(
              experiment(StopTime = 1.5, Interval = 0.001),
              Documentation(info = "<HTML>
<b>Test example: Asynchronous induction machine with squirrel cage - direct on line starting</b><br>
At start time tStart three phase voltage is supplied to the asynchronous induction machine with squirrel cage;
the machine starts from standstill, accelerating inertias against load torque quadratic dependent on speed, finally reaching nominal speed.<br>
Simulate for 1.5 seconds and plot (versus time):
<ul>
<li>currentQuasiRMSSensor.I: stator current RMS</li>
<li>aimc.wMechanical: motor's speed</li>
<li>aimc.tauElectrical: motor's torque</li>
</ul>
Default machine parameters of model <i>AIM_SquirrelCage</i> are used.
</HTML>"));
          end AIMC_DOL;

          model AIMC_YD "Test example: AsynchronousInductionMachineSquirrelCage Y-D"
            extends Modelica.Icons.Example;
            constant Integer m = 3 "Number of phases";
            parameter Modelica.SIunits.Voltage VNominal = 100 "Nominal RMS voltage per phase";
            parameter Modelica.SIunits.Frequency fNominal = 50 "Nominal frequency";
            parameter Modelica.SIunits.Time tStart1 = 0.1 "Start time";
            parameter Modelica.SIunits.Time tStart2 = 2.0 "Start time from Y to D";
            parameter Modelica.SIunits.Torque TLoad = 161.4 "Nominal load torque";
            parameter Modelica.SIunits.AngularVelocity wLoad(displayUnit = "1/min") = 1440.45 * 2 * Modelica.Constants.pi / 60 "Nominal load speed";
            parameter Modelica.SIunits.Inertia JLoad = 0.29 "Load's moment of inertia";
            Machines.BasicMachines.AsynchronousInductionMachines.AIM_SquirrelCage aimc(p = aimcData.p, fsNominal = aimcData.fsNominal, Rs = aimcData.Rs, TsRef = aimcData.TsRef, alpha20s(displayUnit = "1/K") = aimcData.alpha20s, Lszero = aimcData.Lszero, Lssigma = aimcData.Lssigma, Jr = aimcData.Jr, Js = aimcData.Js, frictionParameters = aimcData.frictionParameters, phiMechanical(fixed = true), wMechanical(fixed = true), statorCoreParameters = aimcData.statorCoreParameters, strayLoadParameters = aimcData.strayLoadParameters, Lm = aimcData.Lm, Lrsigma = aimcData.Lrsigma, Rr = aimcData.Rr, TrRef = aimcData.TrRef, TsOperational = 293.15, alpha20r = aimcData.alpha20r, TrOperational = 293.15) annotation(
              Placement(transformation(extent = {{-20, -50}, {0, -30}}, rotation = 0)));
            Machines.Sensors.CurrentQuasiRMSSensor currentQuasiRMSSensor annotation(
              Placement(transformation(origin = {0, 0}, extent = {{-10, 10}, {10, -10}}, rotation = 270)));
            Modelica.Electrical.MultiPhase.Sources.SineVoltage sineVoltage(final m = m, freqHz = fill(fNominal, m), V = fill(sqrt(2 / 3) * VNominal, m)) annotation(
              Placement(transformation(origin = {0, 60}, extent = {{10, -10}, {-10, 10}}, rotation = 270)));
            Modelica.Electrical.MultiPhase.Basic.Star star(final m = m) annotation(
              Placement(transformation(extent = {{-50, 80}, {-70, 100}}, rotation = 0)));
            Modelica.Electrical.Analog.Basic.Ground ground annotation(
              Placement(transformation(origin = {-90, 90}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
            Modelica.Blocks.Sources.BooleanStep booleanStep[m](each startTime = tStart1) annotation(
              Placement(transformation(extent = {{-80, 30}, {-60, 50}}, rotation = 0)));
            Modelica.Electrical.MultiPhase.Ideal.IdealClosingSwitch idealCloser(final m = m, Ron = fill(1e-5, m), Goff = fill(1e-5, m)) annotation(
              Placement(transformation(origin = {0, 30}, extent = {{-10, 10}, {10, -10}}, rotation = 270)));
            Machines.Utilities.SwitchYD switchYD(m = m) annotation(
              Placement(transformation(extent = {{-20, -30}, {0, -10}}, rotation = 0)));
            Modelica.Blocks.Sources.BooleanStep booleanStepYD[m](each startTime = tStart2) annotation(
              Placement(transformation(extent = {{-80, -30}, {-60, -10}}, rotation = 0)));
            Modelica.Mechanics.Rotational.Components.Inertia loadInertia(J = JLoad) annotation(
              Placement(transformation(extent = {{40, -50}, {60, -30}}, rotation = 0)));
            Modelica.Mechanics.Rotational.Sources.QuadraticSpeedDependentTorque quadraticLoadTorque(w_nominal = wLoad, TorqueDirection = false, tau_nominal = -TLoad, useSupport = false) annotation(
              Placement(transformation(extent = {{90, -50}, {70, -30}}, rotation = 0)));
            parameter Utilities.ParameterRecords.AIM_SquirrelCageData aimcData annotation(
              Placement(transformation(extent = {{-20, -80}, {0, -60}})));
          initial equation
            aimc.is = zeros(3);
            aimc.ir = zeros(2);
          equation
            connect(star.pin_n, ground.p) annotation(
              Line(points = {{-70, 90}, {-80, 90}}, color = {0, 0, 255}));
            connect(sineVoltage.plug_n, star.plug_p) annotation(
              Line(points = {{0, 70}, {0, 90}, {-50, 90}}, color = {0, 0, 255}));
            connect(sineVoltage.plug_p, idealCloser.plug_p) annotation(
              Line(points = {{0, 50}, {0, 48}, {0, 46}, {0, 40}}, color = {0, 0, 255}));
            connect(loadInertia.flange_b, quadraticLoadTorque.flange) annotation(
              Line(points = {{60, -40}, {70, -40}}, color = {0, 0, 0}));
            connect(booleanStep.y, idealCloser.control) annotation(
              Line(points = {{-59, 40}, {-20, 40}, {-20, 30}, {-7, 30}}, color = {255, 0, 255}));
            connect(booleanStepYD.y, switchYD.control) annotation(
              Line(points = {{-59, -20}, {-21, -20}}, color = {255, 0, 255}));
            connect(idealCloser.plug_n, currentQuasiRMSSensor.plug_p) annotation(
              Line(points = {{0, 20}, {0, 18}, {0, 10}}, color = {0, 0, 255}));
            connect(switchYD.plug_sn, aimc.plug_sn) annotation(
              Line(points = {{-16, -30}, {-16, -30}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(switchYD.plug_sp, aimc.plug_sp) annotation(
              Line(points = {{-4, -30}, {-4, -30}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(switchYD.plugSupply, currentQuasiRMSSensor.plug_n) annotation(
              Line(points = {{-10, -10}, {0, -10}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(aimc.flange, loadInertia.flange_a) annotation(
              Line(points = {{0, -40}, {40, -40}}, color = {0, 0, 0}, smooth = Smooth.None));
            annotation(
              experiment(StopTime = 2.5, Interval = 0.001),
              Documentation(info = "<HTML>
<b>Test example: Asynchronous induction machine with squirrel cage - Y-D starting</b><br>
At start time tStart three phase voltage is supplied to the asynchronous induction machine with squirrel cage, first star-connected, then delta-connected; the machine starts from standstill, accelerating inertias against load torque quadratic dependent on speed, finally reaching nominal speed.<br>
Simulate for 2.5 seconds and plot (versus time):
<ul>
<li>currentQuasiRMSSensor.I: stator current RMS</li>
<li>aimc.wMechanical: motor's speed</li>
<li>aimc.tauElectrical: motor's torque</li>
</ul>
Default machine parameters of model <i>AIM_SquirrelCage</i> are used.
</HTML>"));
          end AIMC_YD;

          model AIMC_Transformer "Test example: AsynchronousInductionMachineSquirrelCage transformer starting"
            extends Modelica.Icons.Example;
            constant Integer m = 3 "Number of phases";
            parameter Modelica.SIunits.Voltage VNominal = 100 "Nominal RMS voltage per phase";
            parameter Modelica.SIunits.Frequency fNominal = 50 "Nominal frequency";
            parameter Modelica.SIunits.Time tStart1 = 0.1 "Start time";
            parameter Modelica.SIunits.Time tStart2 = 2.0 "Start time of bypass transformer";
            parameter Modelica.SIunits.Torque TLoad = 161.4 "Nominal load torque";
            parameter Modelica.SIunits.AngularVelocity wLoad(displayUnit = "1/min") = 1440.45 * 2 * Modelica.Constants.pi / 60 "Nominal load speed";
            parameter Modelica.SIunits.Inertia JLoad = 0.29 "Load's moment of inertia";
            Machines.BasicMachines.AsynchronousInductionMachines.AIM_SquirrelCage aimc(p = aimcData.p, fsNominal = aimcData.fsNominal, Rs = aimcData.Rs, TsRef = aimcData.TsRef, alpha20s(displayUnit = "1/K") = aimcData.alpha20s, Lszero = aimcData.Lszero, Lssigma = aimcData.Lssigma, Jr = aimcData.Jr, Js = aimcData.Js, frictionParameters = aimcData.frictionParameters, phiMechanical(fixed = true), wMechanical(fixed = true), statorCoreParameters = aimcData.statorCoreParameters, strayLoadParameters = aimcData.strayLoadParameters, Lm = aimcData.Lm, Lrsigma = aimcData.Lrsigma, Rr = aimcData.Rr, TrRef = aimcData.TrRef, TsOperational = 293.15, alpha20r = aimcData.alpha20r, TrOperational = 293.15) annotation(
              Placement(transformation(extent = {{-20, -50}, {0, -30}}, rotation = 0)));
            Machines.Sensors.CurrentQuasiRMSSensor currentQuasiRMSSensor annotation(
              Placement(transformation(origin = {0, 80}, extent = {{-10, 10}, {10, -10}}, rotation = 270)));
            Modelica.Electrical.MultiPhase.Sources.SineVoltage sineVoltage(final m = m, freqHz = fill(fNominal, m), V = fill(sqrt(2 / 3) * VNominal, m)) annotation(
              Placement(transformation(origin = {-30, 90}, extent = {{10, 10}, {-10, -10}}, rotation = 0)));
            Modelica.Electrical.MultiPhase.Basic.Star star(final m = m) annotation(
              Placement(transformation(extent = {{-50, 80}, {-70, 100}}, rotation = 0)));
            Modelica.Electrical.Analog.Basic.Ground ground annotation(
              Placement(transformation(origin = {-90, 90}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
            Modelica.Blocks.Sources.BooleanStep booleanStep1[m](each startTime = tStart1) annotation(
              Placement(transformation(extent = {{-60, 40}, {-40, 60}}, rotation = 0)));
            Modelica.Electrical.MultiPhase.Ideal.IdealClosingSwitch idealCloser(final m = m, Ron = fill(1e-5, m), Goff = fill(1e-5, m)) annotation(
              Placement(transformation(origin = {0, 50}, extent = {{-10, 10}, {10, -10}}, rotation = 270)));
            Machines.BasicMachines.Transformers.Yy.Yy00 transformer(n = transformerData.n, R1 = transformerData.R1, L1sigma = transformerData.L1sigma, R2 = transformerData.R2, L2sigma = transformerData.L2sigma, T1Ref = 293.15, alpha20_1(displayUnit = "1/K") = Modelica.Electrical.Machines.Thermal.Constants.alpha20Zero, T2Ref = 293.15, alpha20_2(displayUnit = "1/K") = Modelica.Electrical.Machines.Thermal.Constants.alpha20Zero, T1Operational = 293.15, T2Operational = 293.15) annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 270, origin = {-20, 30})));
            Modelica.Electrical.Analog.Basic.Ground ground2 annotation(
              Placement(transformation(origin = {-50, 20}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
            parameter Machines.Utilities.TransformerData transformerData(f = fNominal, V1 = VNominal, C1 = Modelica.Utilities.Strings.substring(transformer.VectorGroup, 1, 1), V2 = VNominal / sqrt(3), C2 = Modelica.Utilities.Strings.substring(transformer.VectorGroup, 2, 2), SNominal = 50E3, v_sc = 0.06, P_sc = 500) annotation(
              Placement(transformation(extent = {{-80, 20}, {-60, 40}})));
            Modelica.Blocks.Sources.BooleanStep booleanStep2[m](each startTime = tStart2) annotation(
              Placement(transformation(extent = {{-60, -10}, {-40, 10}}, rotation = 0)));
            Modelica.Electrical.MultiPhase.Ideal.IdealCommutingSwitch idealCommutingSwitch(final m = m, Ron = fill(1e-5, m), Goff = fill(50E-5, m)) annotation(
              Placement(transformation(origin = {0, 0}, extent = {{10, 10}, {-10, -10}}, rotation = 270)));
            Modelica.Mechanics.Rotational.Components.Inertia loadInertia(J = JLoad) annotation(
              Placement(transformation(extent = {{40, -50}, {60, -30}}, rotation = 0)));
            Modelica.Mechanics.Rotational.Sources.QuadraticSpeedDependentTorque quadraticLoadTorque(w_nominal = wLoad, TorqueDirection = false, tau_nominal = -TLoad, useSupport = false) annotation(
              Placement(transformation(extent = {{90, -50}, {70, -30}}, rotation = 0)));
            Machines.Utilities.TerminalBox terminalBox(terminalConnection = "D") annotation(
              Placement(transformation(extent = {{-20, -30}, {0, -10}}, rotation = 0)));
            parameter Utilities.ParameterRecords.AIM_SquirrelCageData aimcData annotation(
              Placement(transformation(extent = {{-20, -80}, {0, -60}})));
          initial equation
            aimc.is = zeros(3);
            aimc.ir = zeros(2);
            transformer.i1[1:2] = zeros(2);
          equation
            connect(star.pin_n, ground.p) annotation(
              Line(points = {{-70, 90}, {-80, 90}}, color = {0, 0, 255}));
            connect(terminalBox.plug_sn, aimc.plug_sn) annotation(
              Line(points = {{-16, -30}, {-16, -30}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(terminalBox.plug_sp, aimc.plug_sp) annotation(
              Line(points = {{-4, -30}, {-4, -30}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(loadInertia.flange_b, quadraticLoadTorque.flange) annotation(
              Line(points = {{60, -40}, {70, -40}}, color = {0, 0, 0}, smooth = Smooth.None));
            connect(aimc.flange, loadInertia.flange_a) annotation(
              Line(points = {{0, -40}, {40, -40}}, color = {0, 0, 0}, smooth = Smooth.None));
            connect(star.plug_p, sineVoltage.plug_n) annotation(
              Line(points = {{-50, 90}, {-40, 90}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(booleanStep2.y, idealCommutingSwitch.control) annotation(
              Line(points = {{-39, 0}, {-8, 0}}, color = {255, 0, 255}, smooth = Smooth.None));
            connect(transformer.starpoint2, ground2.p) annotation(
              Line(points = {{-30, 25}, {-30, 20}, {-40, 20}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(idealCommutingSwitch.plug_p, terminalBox.plugSupply) annotation(
              Line(points = {{0, -10}, {-10, -10}, {-10, -28}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(transformer.plug2, idealCommutingSwitch.plug_n1) annotation(
              Line(points = {{-20, 20}, {-5, 20}, {-5, 10}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(sineVoltage.plug_p, currentQuasiRMSSensor.plug_p) annotation(
              Line(points = {{-20, 90}, {0, 90}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(booleanStep1.y, idealCloser.control) annotation(
              Line(points = {{-39, 50}, {-7, 50}}, color = {255, 0, 255}, smooth = Smooth.None));
            connect(currentQuasiRMSSensor.plug_n, idealCloser.plug_p) annotation(
              Line(points = {{0, 70}, {0, 60}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(transformer.plug1, idealCloser.plug_n) annotation(
              Line(points = {{-20, 40}, {0, 40}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(idealCloser.plug_n, idealCommutingSwitch.plug_n2) annotation(
              Line(points = {{0, 40}, {0, 10}}, color = {0, 0, 255}, smooth = Smooth.None));
            annotation(
              experiment(StopTime = 2.5, Interval = 0.001),
              Documentation(info = "<HTML>
<b>Test example: Asynchronous induction machine with squirrel cage - transformer starting</b><br>
At start time tStart1 three phase voltage is supplied to the asynchronous induction machine with squirrel cage via the transformer;
the machine starts from standstill, accelerating inertias against load torque quadratic dependent on speed;
at start time tStart2 the machine is fed directly from the voltage source, finally reaching nominal speed.<br>
Simulate for 2.5 seconds and plot (versus time):
<ul>
<li>currentQuasiRMSSensor.I: stator current RMS</li>
<li>aimc.wMechanical: motor's speed</li>
<li>aimc.tauElectrical: motor's torque</li>
</ul>
Default machine parameters of model <i>AIM_SquirrelCage</i> are used.
</HTML>"));
          end AIMC_Transformer;

          model AIMS_Start "Test example: AsynchronousInductionMachineSlipRing"
            extends Modelica.Icons.Example;
            constant Integer m = 3 "Number of phases";
            parameter Modelica.SIunits.Voltage VNominal = 100 "Nominal RMS voltage per phase";
            parameter Modelica.SIunits.Frequency fNominal = 50 "Nominal frequency";
            parameter Modelica.SIunits.Time tStart1 = 0.1 "Start time";
            parameter Modelica.SIunits.Resistance Rstart = 0.16 / aimsData.turnsRatio ^ 2 "Starting resistance";
            parameter Modelica.SIunits.Time tStart2 = 1.0 "Start time of shorting starting resistance";
            parameter Modelica.SIunits.Torque TLoad = 161.4 "Nominal load torque";
            parameter Modelica.SIunits.AngularVelocity wLoad(displayUnit = "1/min") = 1440.45 * 2 * Modelica.Constants.pi / 60 "Nominal load speed";
            parameter Modelica.SIunits.Inertia JLoad = 0.29 "Load's moment of inertia";
            Machines.BasicMachines.AsynchronousInductionMachines.AIM_SlipRing aims(p = aimsData.p, Jr = aimsData.Jr, Js = aimsData.Js, phiMechanical(fixed = true), wMechanical(fixed = true), useTurnsRatio = aimsData.useTurnsRatio, turnsRatio = aimsData.turnsRatio, VsNominal = aimsData.VsNominal, VrLockedRotor = aimsData.VrLockedRotor, Rs = aimsData.Rs, TsRef = aimsData.TsRef, Lszero = aimsData.Lszero, Lssigma = aimsData.Lssigma, Lm = aimsData.Lm, Lrsigma = aimsData.Lrsigma, Lrzero = aimsData.Lrzero, Rr = aimsData.Rr, TrRef = aimsData.TrRef, frictionParameters = aimsData.frictionParameters, statorCoreParameters = aimsData.statorCoreParameters, strayLoadParameters = aimsData.strayLoadParameters, rotorCoreParameters = aimsData.rotorCoreParameters, fsNominal = aimsData.fsNominal, TsOperational = 293.15, alpha20s = aimsData.alpha20s, alpha20r = aimsData.alpha20r, TrOperational = 293.15) annotation(
              Placement(transformation(extent = {{-20, -50}, {0, -30}}, rotation = 0)));
            Machines.Sensors.CurrentQuasiRMSSensor currentQuasiRMSSensor annotation(
              Placement(transformation(origin = {0, 0}, extent = {{-10, 10}, {10, -10}}, rotation = 270)));
            Modelica.Electrical.MultiPhase.Sources.SineVoltage sineVoltage(final m = m, freqHz = fill(fNominal, m), V = fill(sqrt(2 / 3) * VNominal, m)) annotation(
              Placement(transformation(origin = {0, 60}, extent = {{10, -10}, {-10, 10}}, rotation = 270)));
            Modelica.Electrical.MultiPhase.Basic.Star star(final m = m) annotation(
              Placement(transformation(extent = {{-50, 80}, {-70, 100}}, rotation = 0)));
            Modelica.Electrical.Analog.Basic.Ground ground annotation(
              Placement(transformation(origin = {-90, 90}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
            Modelica.Blocks.Sources.BooleanStep booleanStep[m](each startTime = tStart1) annotation(
              Placement(transformation(extent = {{-80, 30}, {-60, 50}}, rotation = 0)));
            Modelica.Electrical.MultiPhase.Ideal.IdealClosingSwitch idealCloser(final m = m, Ron = fill(1e-5, m), Goff = fill(1e-5, m)) annotation(
              Placement(transformation(origin = {0, 30}, extent = {{-10, 10}, {10, -10}}, rotation = 270)));
            Modelica.Mechanics.Rotational.Components.Inertia loadInertia(J = JLoad) annotation(
              Placement(transformation(extent = {{40, -50}, {60, -30}}, rotation = 0)));
            Modelica.Mechanics.Rotational.Sources.QuadraticSpeedDependentTorque quadraticLoadTorque(w_nominal = wLoad, TorqueDirection = false, tau_nominal = -TLoad, useSupport = false) annotation(
              Placement(transformation(extent = {{90, -50}, {70, -30}}, rotation = 0)));
            Machines.Utilities.TerminalBox terminalBox(terminalConnection = "D") annotation(
              Placement(transformation(extent = {{-20, -30}, {0, -10}}, rotation = 0)));
            Machines.Utilities.SwitchedRheostat switchedRheostat(RStart = Rstart, tStart = tStart2, m = m) annotation(
              Placement(transformation(extent = {{-50, -50}, {-30, -30}})));
            parameter Utilities.ParameterRecords.AIM_SlipRingData aimsData annotation(
              Placement(transformation(extent = {{-20, -80}, {0, -60}})));
          initial equation
            aims.is = zeros(3);
            aims.ir = zeros(3);
          equation
            connect(star.pin_n, ground.p) annotation(
              Line(points = {{-70, 90}, {-80, 90}}, color = {0, 0, 255}));
            connect(sineVoltage.plug_n, star.plug_p) annotation(
              Line(points = {{0, 70}, {0, 90}, {-50, 90}}, color = {0, 0, 255}));
            connect(sineVoltage.plug_p, idealCloser.plug_p) annotation(
              Line(points = {{0, 50}, {0, 48}, {0, 46}, {0, 40}}, color = {0, 0, 255}));
            connect(loadInertia.flange_b, quadraticLoadTorque.flange) annotation(
              Line(points = {{60, -40}, {70, -40}}, color = {0, 0, 0}));
            connect(booleanStep.y, idealCloser.control) annotation(
              Line(points = {{-59, 40}, {-20, 40}, {-20, 30}, {-7, 30}}, color = {255, 0, 255}));
            connect(idealCloser.plug_n, currentQuasiRMSSensor.plug_p) annotation(
              Line(points = {{0, 20}, {0, 16}, {0, 10}}, color = {0, 0, 255}));
            connect(terminalBox.plugSupply, currentQuasiRMSSensor.plug_n) annotation(
              Line(points = {{-10, -28}, {-10, -20}, {0, -20}, {0, -10}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(terminalBox.plug_sn, aims.plug_sn) annotation(
              Line(points = {{-16, -30}, {-16, -30}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(terminalBox.plug_sp, aims.plug_sp) annotation(
              Line(points = {{-4, -30}, {-4, -30}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(aims.flange, loadInertia.flange_a) annotation(
              Line(points = {{0, -40}, {40, -40}}, color = {0, 0, 0}, smooth = Smooth.None));
            connect(switchedRheostat.plug_p, aims.plug_rp) annotation(
              Line(points = {{-30, -34}, {-20, -34}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(switchedRheostat.plug_n, aims.plug_rn) annotation(
              Line(points = {{-30, -46}, {-20, -46}}, color = {0, 0, 255}, smooth = Smooth.None));
            annotation(
              experiment(StopTime = 1.5, Interval = 0.001),
              Documentation(info = "<HTML>
<b>Test example: Asynchronous induction machine with slipring rotor - resistance starting</b><br>
At start time tStart1 three phase voltage is supplied to the asynchronous induction machine with sliprings;
the machine starts from standstill, accelerating inertias against load torque quadratic dependent on speed,
using a starting resistance. At time tStart2 external rotor resistance is shortened, finally reaching nominal speed.<br>
Simulate for 1.5 seconds and plot (versus time):
<ul>
<li>currentQuasiRMSSensor.I: stator current RMS</li>
<li>aims.wMechanical: motor's speed</li>
<li>aims.tauElectrical: motor's torque</li>
</ul>
Default machine parameters of model <i>AIM_SlipRing</i> are used.
</HTML>"));
          end AIMS_Start;

          model AIMC_Inverter "Test example: AsynchronousInductionMachineSquirrelCage with inverter"
            extends Modelica.Icons.Example;
            constant Integer m = 3 "Number of phases";
            parameter Modelica.SIunits.Voltage VNominal = 100 "Nominal RMS voltage per phase";
            parameter Modelica.SIunits.Frequency fNominal = 50 "Nominal frequency";
            parameter Modelica.SIunits.Frequency f = 50 "Actual frequency";
            parameter Modelica.SIunits.Time tRamp = 1 "Frequency ramp";
            parameter Modelica.SIunits.Torque TLoad = 161.4 "Nominal load torque";
            parameter Modelica.SIunits.Time tStep = 1.2 "Time of load torque step";
            parameter Modelica.SIunits.Inertia JLoad = 0.29 "Load's moment of inertia";
            Machines.BasicMachines.AsynchronousInductionMachines.AIM_SquirrelCage aimc(p = aimcData.p, fsNominal = aimcData.fsNominal, Rs = aimcData.Rs, TsRef = aimcData.TsRef, alpha20s(displayUnit = "1/K") = aimcData.alpha20s, Lszero = aimcData.Lszero, Lssigma = aimcData.Lssigma, Jr = aimcData.Jr, Js = aimcData.Js, frictionParameters = aimcData.frictionParameters, phiMechanical(fixed = true), wMechanical(fixed = true), statorCoreParameters = aimcData.statorCoreParameters, strayLoadParameters = aimcData.strayLoadParameters, Lm = aimcData.Lm, Lrsigma = aimcData.Lrsigma, Rr = aimcData.Rr, TrRef = aimcData.TrRef, TsOperational = 293.15, alpha20r = aimcData.alpha20r, TrOperational = 293.15) annotation(
              Placement(transformation(extent = {{-20, -50}, {0, -30}}, rotation = 0)));
            Machines.Sensors.CurrentQuasiRMSSensor currentQuasiRMSSensor annotation(
              Placement(transformation(origin = {0, 0}, extent = {{-10, 10}, {10, -10}}, rotation = 270)));
            Modelica.Blocks.Sources.Ramp ramp(height = f, duration = tRamp) annotation(
              Placement(transformation(extent = {{-80, 50}, {-60, 70}}, rotation = 0)));
            Machines.Utilities.VfController vfController(final m = m, VNominal = VNominal, fNominal = fNominal) annotation(
              Placement(transformation(extent = {{-40, 50}, {-20, 70}}, rotation = 0)));
            Modelica.Electrical.MultiPhase.Sources.SignalVoltage signalVoltage(final m = m) annotation(
              Placement(transformation(origin = {0, 60}, extent = {{10, 10}, {-10, -10}}, rotation = 270)));
            Modelica.Electrical.MultiPhase.Basic.Star star(final m = m) annotation(
              Placement(transformation(extent = {{-50, 80}, {-70, 100}}, rotation = 0)));
            Modelica.Electrical.Analog.Basic.Ground ground annotation(
              Placement(transformation(origin = {-90, 90}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
            Modelica.Mechanics.Rotational.Components.Inertia loadInertia(J = JLoad) annotation(
              Placement(transformation(extent = {{40, -50}, {60, -30}}, rotation = 0)));
            Modelica.Mechanics.Rotational.Sources.TorqueStep loadTorqueStep(startTime = tStep, stepTorque = -TLoad, useSupport = false, offsetTorque = 0) annotation(
              Placement(transformation(extent = {{90, -50}, {70, -30}}, rotation = 0)));
            Machines.Utilities.TerminalBox terminalBox(terminalConnection = "Y") annotation(
              Placement(transformation(extent = {{-20, -30}, {0, -10}}, rotation = 0)));
            parameter Utilities.ParameterRecords.AIM_SquirrelCageData aimcData annotation(
              Placement(transformation(extent = {{-20, -80}, {0, -60}})));
          initial equation
            aimc.is[1:2] = zeros(2);
            aimc.ir = zeros(2);
          equation
            connect(signalVoltage.plug_n, star.plug_p) annotation(
              Line(points = {{0, 70}, {0, 90}, {-50, 90}}, color = {0, 0, 255}));
            connect(star.pin_n, ground.p) annotation(
              Line(points = {{-70, 90}, {-80, 90}}, color = {0, 0, 255}));
            connect(ramp.y, vfController.u) annotation(
              Line(points = {{-59, 60}, {-42, 60}}, color = {0, 0, 255}));
            connect(vfController.y, signalVoltage.v) annotation(
              Line(points = {{-19, 60}, {-7, 60}}, color = {0, 0, 255}));
            connect(loadTorqueStep.flange, loadInertia.flange_b) annotation(
              Line(points = {{70, -40}, {60, -40}}, color = {0, 0, 0}));
            connect(signalVoltage.plug_p, currentQuasiRMSSensor.plug_p) annotation(
              Line(points = {{0, 50}, {0, 40}, {0, 10}}, color = {0, 0, 255}));
            connect(terminalBox.plugSupply, currentQuasiRMSSensor.plug_n) annotation(
              Line(points = {{-10, -28}, {-10, -20}, {0, -20}, {0, -10}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(terminalBox.plug_sn, aimc.plug_sn) annotation(
              Line(points = {{-16, -30}, {-16, -30}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(terminalBox.plug_sp, aimc.plug_sp) annotation(
              Line(points = {{-4, -30}, {-4, -30}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(aimc.flange, loadInertia.flange_a) annotation(
              Line(points = {{0, -40}, {40, -40}}, color = {0, 0, 0}, smooth = Smooth.None));
            annotation(
              experiment(StopTime = 1.5, Interval = 0.001),
              Documentation(info = "<HTML>
<b>Test example: Asynchronous induction machine with squirrel cage fed by an ideal inverter</b><br>
An ideal frequency inverter is modeled by using a VfController and a three-phase SignalVoltage.<br>
Frequency is raised by a ramp, causing the asynchronous induction machine with squirrel cage to start,
and accelerating inertias.<br>At time tStep a load step is applied.<br>
Simulate for 1.5 seconds and plot (versus time):
<ul>
<li>currentQuasiRMSSensor.I: stator current RMS</li>
<li>aimc.wMechanical: motor's speed</li>
<li>aimc.tauElectrical: motor's torque</li>
</ul>
Default machine parameters of model <i>AIM_SquirrelCage</i> are used.
</HTML>"));
          end AIMC_Inverter;

          model AIMC_Steinmetz "AsynchronousInductionMachineSquirrelCage Steinmetz-connection"
            extends Modelica.Icons.Example;
            constant Integer m = 3 "Number of phases";
            parameter Modelica.SIunits.Voltage VNominal = 100 "Nominal RMS voltage per phase";
            parameter Modelica.SIunits.Frequency fNominal = 50 "Nominal frequency";
            parameter Modelica.SIunits.Time tStart1 = 0.1 "Start time";
            parameter Modelica.SIunits.Capacitance Cr = 0.0035 "Motor's running capacitor";
            parameter Modelica.SIunits.Capacitance Cs = 5 * Cr "Motor's (additional) starting capacitor";
            parameter Modelica.SIunits.AngularVelocity wSwitch(displayUnit = "1/min") = 1350 * 2 * Modelica.Constants.pi / 60 "Speed for switching off the starting capacitor";
            parameter Modelica.SIunits.Torque TLoad = 2 / 3 * 161.4 "Nominal load torque";
            parameter Modelica.SIunits.AngularVelocity wLoad(displayUnit = "1/min") = 1462.5 * 2 * Modelica.Constants.pi / 60 "Nominal load speed";
            parameter Modelica.SIunits.Inertia JLoad = 0.29 "Load's moment of inertia";
            Machines.BasicMachines.AsynchronousInductionMachines.AIM_SquirrelCage aimc(p = aimcData.p, fsNominal = aimcData.fsNominal, Rs = aimcData.Rs, TsRef = aimcData.TsRef, alpha20s(displayUnit = "1/K") = aimcData.alpha20s, Lszero = aimcData.Lszero, Lssigma = aimcData.Lssigma, Jr = aimcData.Jr, Js = aimcData.Js, frictionParameters = aimcData.frictionParameters, phiMechanical(fixed = true), wMechanical(fixed = true), statorCoreParameters = aimcData.statorCoreParameters, strayLoadParameters = aimcData.strayLoadParameters, Lm = aimcData.Lm, Lrsigma = aimcData.Lrsigma, Rr = aimcData.Rr, TrRef = aimcData.TrRef, TsOperational = 293.15, alpha20r = aimcData.alpha20r, TrOperational = 293.15) annotation(
              Placement(transformation(extent = {{-20, -50}, {0, -30}}, rotation = 0)));
            Modelica.Electrical.Analog.Sources.SineVoltage sineVoltage(freqHz = fNominal, V = sqrt(2) * VNominal) annotation(
              Placement(transformation(extent = {{-50, 100}, {-70, 80}}, rotation = 0)));
            Modelica.Electrical.Analog.Basic.Ground ground annotation(
              Placement(transformation(origin = {-90, 90}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
            Modelica.Blocks.Sources.BooleanStep booleanStep(startTime = tStart1) annotation(
              Placement(transformation(extent = {{-50, 60}, {-30, 80}}, rotation = 0)));
            Modelica.Electrical.Analog.Ideal.IdealClosingSwitch idealCloser annotation(
              Placement(transformation(origin = {-20, 90}, extent = {{10, -10}, {-10, 10}}, rotation = 180)));
            Modelica.Mechanics.Rotational.Components.Inertia loadInertia(J = JLoad) annotation(
              Placement(transformation(extent = {{40, -50}, {60, -30}}, rotation = 0)));
            Modelica.Mechanics.Rotational.Sources.QuadraticSpeedDependentTorque quadraticLoadTorque(w_nominal = wLoad, TorqueDirection = false, tau_nominal = -TLoad, useSupport = false) annotation(
              Placement(transformation(extent = {{90, -50}, {70, -30}}, rotation = 0)));
            Machines.Utilities.TerminalBox TerminalBox1(terminalConnection = "D") annotation(
              Placement(transformation(extent = {{-20, -30}, {0, -10}}, rotation = 0)));
            Modelica.Electrical.MultiPhase.Basic.PlugToPin_p plugToPin_p3(m = m, k = 3) annotation(
              Placement(transformation(origin = {-30, 18}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
            Modelica.Electrical.MultiPhase.Basic.PlugToPin_p plugToPin_p2(m = m, k = 2) annotation(
              Placement(transformation(origin = {-10, 18}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
            Modelica.Electrical.MultiPhase.Basic.PlugToPin_p plugToPin_p1(m = m, k = 1) annotation(
              Placement(transformation(origin = {10, 18}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
            Modelica.Electrical.Analog.Basic.Capacitor cRun(C = Cr) annotation(
              Placement(transformation(origin = {10, 50}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
            Modelica.Electrical.Analog.Basic.Capacitor cStart(C = Cs) annotation(
              Placement(transformation(origin = {30, 50}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
            Modelica.Electrical.Analog.Ideal.IdealOpeningSwitch idealOpener annotation(
              Placement(transformation(origin = {30, 80}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
            Modelica.Blocks.Logical.GreaterThreshold greaterThreshold(threshold = wSwitch) annotation(
              Placement(transformation(origin = {50, 50}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
            Mechanics.Rotational.Sensors.SpeedSensor relSpeedSensor annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0, origin = {30, -30})));
            parameter Utilities.ParameterRecords.AIM_SquirrelCageData aimcData annotation(
              Placement(transformation(extent = {{-20, -80}, {0, -60}})));
            Sensors.CurrentQuasiRMSSensor currentQuasiRMSSensor annotation(
              Placement(transformation(origin = {-10, -10}, extent = {{-10, 10}, {10, -10}}, rotation = 270)));
          initial equation
            aimc.is = zeros(3);
            aimc.ir = zeros(2);
            cStart.v = 0;
            cRun.v = 0;
          equation
            connect(ground.p, sineVoltage.n) annotation(
              Line(points = {{-80, 90}, {-70, 90}}, color = {0, 0, 255}));
            connect(sineVoltage.p, idealCloser.p) annotation(
              Line(points = {{-50, 90}, {-30, 90}}, color = {0, 0, 255}));
            connect(booleanStep.y, idealCloser.control) annotation(
              Line(points = {{-29, 70}, {-20, 70}, {-20, 83}}, color = {255, 0, 255}));
            connect(plugToPin_p3.pin_p, sineVoltage.n) annotation(
              Line(points = {{-30, 20}, {-70, 20}, {-70, 90}}, color = {0, 0, 255}));
            connect(idealCloser.n, plugToPin_p2.pin_p) annotation(
              Line(points = {{-10, 90}, {-10, 20}}, color = {0, 0, 255}));
            connect(cRun.n, plugToPin_p1.pin_p) annotation(
              Line(points = {{10, 40}, {10, 20}}, color = {0, 0, 255}));
            connect(loadInertia.flange_b, quadraticLoadTorque.flange) annotation(
              Line(points = {{60, -40}, {70, -40}}, color = {0, 0, 0}));
            connect(cRun.p, idealCloser.n) annotation(
              Line(points = {{10, 60}, {10, 90}, {-10, 90}}, color = {0, 0, 255}));
            connect(plugToPin_p1.pin_p, cStart.n) annotation(
              Line(points = {{10, 20}, {30, 20}, {30, 40}}, color = {0, 0, 255}));
            connect(idealOpener.n, cStart.p) annotation(
              Line(points = {{30, 70}, {30, 67.5}, {30, 65}, {30, 60}}, color = {0, 0, 255}));
            connect(idealOpener.p, idealCloser.n) annotation(
              Line(points = {{30, 90}, {30, 90}, {-10, 90}}, color = {0, 0, 255}));
            connect(greaterThreshold.y, idealOpener.control) annotation(
              Line(points = {{50, 61}, {50, 80}, {37, 80}}, color = {255, 0, 255}));
            connect(TerminalBox1.plug_sn, aimc.plug_sn) annotation(
              Line(points = {{-16, -30}, {-16, -30}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(TerminalBox1.plug_sp, aimc.plug_sp) annotation(
              Line(points = {{-4, -30}, {-4, -30}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(aimc.flange, loadInertia.flange_a) annotation(
              Line(points = {{0, -40}, {40, -40}}, color = {0, 0, 0}, smooth = Smooth.None));
            connect(relSpeedSensor.flange, aimc.flange) annotation(
              Line(points = {{20, -30}, {20, -40}, {0, -40}}, color = {0, 0, 0}, smooth = Smooth.None));
            connect(relSpeedSensor.w, greaterThreshold.u) annotation(
              Line(points = {{41, -30}, {50, -30}, {50, 38}}, color = {0, 0, 127}, smooth = Smooth.None));
            connect(plugToPin_p3.plug_p, currentQuasiRMSSensor.plug_p) annotation(
              Line(points = {{-30, 16}, {-30, 0}, {-10, 0}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(plugToPin_p2.plug_p, currentQuasiRMSSensor.plug_p) annotation(
              Line(points = {{-10, 16}, {-10, 0}, {-10, 0}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(plugToPin_p1.plug_p, currentQuasiRMSSensor.plug_p) annotation(
              Line(points = {{10, 16}, {10, 0}, {-10, 0}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(currentQuasiRMSSensor.plug_n, TerminalBox1.plugSupply) annotation(
              Line(points = {{-10, -20}, {-10, -28}, {-10, -28}}, color = {0, 0, 255}, smooth = Smooth.None));
            annotation(
              experiment(StopTime = 1.5, Interval = 0.001),
              Documentation(info = "<HTML>
<b>Asynchronous induction machine with squirrel cage - Steinmetz-connection</b><br>
At start time tStart single phase voltage is supplied to the asynchronous induction machine with squirrel cage;
the machine starts from standstill, accelerating inertias against load torque quadratic dependent on speed, finally reaching nominal speed.<br>
Default machine parameters of model <i>AIM_SquirrelCage</i> are used.
</HTML>"));
          end AIMC_Steinmetz;

          model AIMC_withLosses "Test example: AsynchronousInductionMachineSquirrelCage with losses"
            extends Modelica.Icons.Example;
            constant Integer m = 3 "Number of phases";
            import Modelica.Constants.pi;
            import Modelica.SIunits.Conversions.from_rpm;
            import Modelica.SIunits.Conversions.to_rpm;
            import Modelica.SIunits.Conversions.from_degC;
          protected
            parameter Modelica.SIunits.Power PNominal = 18500 "Nominal output";
            parameter Modelica.SIunits.Voltage VNominal = 400 "Nominal RMS voltage";
            parameter Modelica.SIunits.Current INominal = 32.85 "Nominal RMS current";
            parameter Real pfNominal = 0.898 "Nominal power factor";
            parameter Modelica.SIunits.Frequency fNominal = 50 "Nominal frequency";
            parameter Modelica.SIunits.AngularVelocity wNominal = from_rpm(1462.5) "Nominal speed";
            parameter Modelica.SIunits.Torque TNominal = PNominal / wNominal "Nominal torque";
            parameter Modelica.SIunits.Temperature TempNominal = from_degC(90) "Nominal temperature";
            Modelica.SIunits.Power Pel = electricalPowerSensor.P;
            Modelica.SIunits.ReactivePower Qel = electricalPowerSensor.Q;
            Modelica.SIunits.ApparentPower Sel = sqrt(Pel ^ 2 + Qel ^ 2);
            parameter Real Ptable[:] = {1E-6, 1845, 3549, 5325, 7521, 9372, 11010, 12930, 14950, 16360, 18500, 18560, 20180, 22170};
            parameter Real Itable[:] = {11.0, 11.20, 12.27, 13.87, 16.41, 18.78, 21.07, 23.92, 27.05, 29.40, 32.85, 32.95, 35.92, 39.35};
            parameter Real ntable[:] = {1500, 1496, 1493, 1490, 1486, 1482, 1479, 1475, 1471, 1467, 1462, 1462, 1458, 1453};
            parameter Real ctable[:] = {0.085, 0.327, 0.506, 0.636, 0.741, 0.797, 0.831, 0.857, 0.875, 0.887, 0.896, 0.896, 0.902, 0.906};
            parameter Real etable[:] = {0, 0.7250, 0.8268, 0.8698, 0.8929, 0.9028, 0.9064, 0.9088, 0.9089, 0.9070, 0.9044, 0.9043, 0.9008, 0.8972};
          public
            output Modelica.SIunits.Power Pmech = powerSensor.power "Mechanical output";
            output Modelica.SIunits.Current I_sim = currentQuasiRMSSensor.I "Simulated current";
            output Modelica.SIunits.Current I_meas = combiTable1Ds.y[1] "Measured current";
            output Modelica.SIunits.AngularVelocity w_sim(displayUnit = "1/min") = aimc.wMechanical "Simulated speed";
            output Modelica.SIunits.AngularVelocity w_meas = combiTable1Ds.y[2] "Measured speed";
            output Real pf_sim = if noEvent(Sel > Modelica.Constants.small) then Pel / Sel else 0 "Simulated power factor";
            output Real pf_meas = combiTable1Ds.y[3] "Measured power factor";
            output Real eff_sim = if noEvent(abs(Pel) > Modelica.Constants.small) then Pmech / Pel else 0 "Simulated efficiency";
            output Real eff_meas = combiTable1Ds.y[4] "Measured efficiency";
            Machines.BasicMachines.AsynchronousInductionMachines.AIM_SquirrelCage aimc(p = aimcData.p, fsNominal = aimcData.fsNominal, Rs = aimcData.Rs, TsRef = aimcData.TsRef, alpha20s(displayUnit = "1/K") = aimcData.alpha20s, Lszero = aimcData.Lszero, Lssigma = aimcData.Lssigma, Jr = aimcData.Jr, Js = aimcData.Js, frictionParameters = aimcData.frictionParameters, phiMechanical(fixed = true), statorCoreParameters = aimcData.statorCoreParameters, strayLoadParameters = aimcData.strayLoadParameters, Lm = aimcData.Lm, Lrsigma = aimcData.Lrsigma, Rr = aimcData.Rr, TrRef = aimcData.TrRef, TsOperational = TempNominal, TrOperational = TempNominal, wMechanical(fixed = true, start = 2 * pi * aimcData.fsNominal / aimcData.p), alpha20r = aimcData.alpha20r) annotation(
              Placement(transformation(extent = {{-40, 0}, {-20, 20}}, rotation = 0)));
            Machines.Utilities.TerminalBox terminalBox(terminalConnection = "D") annotation(
              Placement(transformation(extent = {{-40, 20}, {-20, 40}}, rotation = 0)));
            Machines.Sensors.ElectricalPowerSensor electricalPowerSensor annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 270, origin = {-30, 40})));
            Machines.Sensors.CurrentQuasiRMSSensor currentQuasiRMSSensor annotation(
              Placement(transformation(origin = {-30, 70}, extent = {{-10, 10}, {10, -10}}, rotation = 270)));
            Modelica.Electrical.MultiPhase.Sources.SineVoltage sineVoltage(final m = m, freqHz = fill(fNominal, m), V = fill(sqrt(2 / 3) * VNominal, m)) annotation(
              Placement(transformation(origin = {-70, 70}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
            Modelica.Electrical.MultiPhase.Basic.Star star(final m = m) annotation(
              Placement(transformation(extent = {{10, -10}, {-10, 10}}, rotation = 90, origin = {-70, 30})));
            Modelica.Electrical.Analog.Basic.Ground ground annotation(
              Placement(transformation(origin = {-70, 0}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
            Modelica.Mechanics.Rotational.Sensors.PowerSensor powerSensor annotation(
              Placement(transformation(extent = {{-10, 0}, {10, 20}})));
            Modelica.Mechanics.Rotational.Components.Inertia loadInertia(J = aimcData.Jr) annotation(
              Placement(transformation(extent = {{20, 0}, {40, 20}}, rotation = 0)));
            Modelica.Mechanics.Rotational.Sources.Torque torque annotation(
              Placement(transformation(extent = {{70, 0}, {50, 20}})));
            Modelica.Blocks.Math.Gain gain(k = -1) annotation(
              Placement(transformation(extent = {{50, -60}, {70, -40}})));
            Modelica.Blocks.Continuous.PI PI(k = 0.01, T = 0.01, initType = Modelica.Blocks.Types.Init.InitialState) annotation(
              Placement(transformation(extent = {{20, -60}, {40, -40}})));
            Modelica.Blocks.Math.Feedback feedback annotation(
              Placement(transformation(extent = {{-10, -40}, {10, -60}})));
            Modelica.Blocks.Sources.Ramp ramp(height = 1.2 * PNominal, offset = 0, startTime = 4.5, duration = 5.5) annotation(
              Placement(transformation(extent = {{-40, -60}, {-20, -40}})));
            Modelica.Blocks.Tables.CombiTable1Ds combiTable1Ds(table = {{Ptable[j], Itable[j], ntable[j], ctable[j], etable[j]} for j in 1:size(Ptable, 1)}, smoothness = Modelica.Blocks.Types.Smoothness.ContinuousDerivative) annotation(
              Placement(transformation(extent = {{20, -30}, {40, -10}})));
            parameter Utilities.ParameterRecords.AIM_SquirrelCageData aimcData(statorCoreParameters(PRef = 410, VRef = 387.9), Jr = 0.12, Rs = 0.56, alpha20s(displayUnit = "1/K") = Modelica.Electrical.Machines.Thermal.Constants.alpha20Copper, Lssigma = 1.52 / (2 * pi * fNominal), frictionParameters(PRef = 180, wRef = wNominal), strayLoadParameters(PRef = 0.005 * sqrt(3) * VNominal * INominal * pfNominal, IRef = INominal / sqrt(3), wRef = wNominal), Lm = 66.4 / (2 * pi * fNominal), Lrsigma = 2.31 / (2 * pi * fNominal), Rr = 0.42, alpha20r(displayUnit = "1/K") = Modelica.Electrical.Machines.Thermal.Constants.alpha20Aluminium) annotation(
              Placement(transformation(extent = {{-40, -30}, {-20, -10}})));
          initial equation
            aimc.i_0_s = 0;
            der(aimc.idq_sr) = zeros(2);
            der(aimc.idq_rr) = zeros(2);
          equation
            connect(star.pin_n, ground.p) annotation(
              Line(points = {{-70, 20}, {-70, 10}}, color = {0, 0, 255}));
            connect(sineVoltage.plug_n, star.plug_p) annotation(
              Line(points = {{-70, 60}, {-70, 40}}, color = {0, 0, 255}));
            connect(terminalBox.plug_sn, aimc.plug_sn) annotation(
              Line(points = {{-36, 20}, {-36, 20}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(terminalBox.plug_sp, aimc.plug_sp) annotation(
              Line(points = {{-24, 20}, {-24, 20}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(currentQuasiRMSSensor.plug_n, electricalPowerSensor.plug_p) annotation(
              Line(points = {{-30, 60}, {-30, 57.5}, {-30, 57.5}, {-30, 55}, {-30, 50}, {-30, 50}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(electricalPowerSensor.plug_nv, star.plug_p) annotation(
              Line(points = {{-40, 40}, {-70, 40}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(electricalPowerSensor.plug_ni, terminalBox.plugSupply) annotation(
              Line(points = {{-30, 30}, {-30, 26}, {-30, 26}, {-30, 22}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(aimc.flange, powerSensor.flange_a) annotation(
              Line(points = {{-20, 10}, {-10, 10}}, color = {0, 0, 0}, smooth = Smooth.None));
            connect(powerSensor.flange_b, loadInertia.flange_a) annotation(
              Line(points = {{10, 10}, {20, 10}}, color = {0, 0, 0}, smooth = Smooth.None));
            connect(torque.flange, loadInertia.flange_b) annotation(
              Line(points = {{50, 10}, {40, 10}}, color = {0, 0, 0}, smooth = Smooth.None));
            connect(gain.y, torque.tau) annotation(
              Line(points = {{71, -50}, {80, -50}, {80, 10}, {72, 10}}, color = {0, 0, 127}, smooth = Smooth.None));
            connect(sineVoltage.plug_p, currentQuasiRMSSensor.plug_p) annotation(
              Line(points = {{-70, 80}, {-30, 80}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(powerSensor.power, feedback.u2) annotation(
              Line(points = {{-8, -1}, {-8, -20}, {0, -20}, {0, -42}}, color = {0, 0, 127}, smooth = Smooth.None));
            connect(feedback.y, PI.u) annotation(
              Line(points = {{9, -50}, {18, -50}}, color = {0, 0, 127}, smooth = Smooth.None));
            connect(PI.y, gain.u) annotation(
              Line(points = {{41, -50}, {48, -50}}, color = {0, 0, 127}, smooth = Smooth.None));
            connect(ramp.y, feedback.u1) annotation(
              Line(points = {{-19, -50}, {-8, -50}}, color = {0, 0, 127}, smooth = Smooth.None));
            connect(powerSensor.power, combiTable1Ds.u) annotation(
              Line(points = {{-8, -1}, {-8, -20}, {18, -20}}, color = {0, 0, 127}, smooth = Smooth.None));
            annotation(
              experiment(StopTime = 5.0, Interval = 0.001),
              Documentation(info = "<HTML>
<p>Test example: Asynchronous induction machine with squirrel cage - characteristics with losses</p>
<ul>
<li>Simulate for 5 seconds: The machine is started at nominal speed, flux is build up in the machine.</li>
<li>Continue the simulation for additional 5 seconds: Subsequently a load ramp is applied.</li>
<li>Compare by plotting versus Pmech:</li>
</ul>
<table>
<tr><td>Current      </td><td>I_sim   </td><td>I_meas  </td></tr>
<tr><td>Speed        </td><td>w_sim   </td><td>w_meas  </td></tr>
<tr><td>Power factor </td><td>pf_sim  </td><td>pf_meas </td></tr>
<tr><td>Efficiency   </td><td>eff_sim </td><td>eff_meas</td></tr>
</table>
<p>Machine parameters are taken from a standard 18.5 kW 400 V 50 Hz motor, simulation results are compared with measurements.</p>
<table>
<tr><td>Nominal stator current            </td><td>     32.85  </td><td>A      </td></tr>
<tr><td>Power factor                      </td><td>      0.898 </td><td>       </td></tr>
<tr><td>Speed                             </td><td>   1462.4   </td><td>rpm    </td></tr>
<tr><td>Electrical input                  </td><td> 20,443.95  </td><td>W      </td></tr>
<tr><td>Stator copper losses              </td><td>    770.13  </td><td>W      </td></tr>
<tr><td>Stator core losses                </td><td>    410.00  </td><td>W      </td></tr>
<tr><td>Rotor  copper losses              </td><td>    481.60  </td><td>W      </td></tr>
<tr><td>Stray load losses                 </td><td>    102.22  </td><td>W      </td></tr>
<tr><td>Friction losses                   </td><td>    180.00  </td><td>W      </td></tr>
<tr><td>Mechanical output                 </td><td> 18,500.00  </td><td>W      </td></tr>
<tr><td>Efficiency                        </td><td>     90.49  </td><td>%      </td></tr>
<tr><td>Nominal torque                    </td><td>    120.79  </td><td>Nm     </td></tr>
</table>
<br>
<table>
<tr><td>Stator resistance per phase       </td><td>  0.56     </td><td>&Omega;</td></tr>
<tr><td>Temperature coefficient           </td><td> copper    </td><td>       </td></tr>
<tr><td>Reference temperature             </td><td> 20        </td><td>&deg;C </td></tr>
<tr><td>Operation temperature             </td><td> 90        </td><td>&deg;C </td></tr>
<tr><td>Stator leakage reactance at 50 Hz </td><td>  1.52     </td><td>&Omega;</td></tr>
<tr><td>Main  field    reactance at 50 Hz </td><td> 66.40     </td><td>&Omega;</td></tr>
<tr><td>Rotor  leakage reactance at 50 Hz </td><td>  2.31     </td><td>&Omega;</td></tr>
<tr><td>Rotor  resistance per phase       </td><td>  0.42     </td><td>&Omega;</td></tr>
<tr><td>Temperature coefficient           </td><td> aluminium </td><td>       </td></tr>
<tr><td>Reference temperature             </td><td> 20        </td><td>&deg;C </td></tr>
<tr><td>Operation temperature             </td><td> 90        </td><td>&deg;C </td></tr>
</table>
<p>See:<br>
Anton Haumer, Christian Kral, Hansj&ouml;rg Kapeller, Thomas B&auml;uml, Johannes V. Gragger<br>
<a href=\"https://www.modelica.org/events/modelica2009/Proceedings/memorystick/pages/papers/0103/0103.pdf\">
The AdvancedMachines Library: Loss Models for Electric Machines</a><br>
Modelica 2009, 7<sup>th</sup> International Modelica Conference</p>
</HTML>"),
              Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-72, 100}, {68, 80}}, lineColor = {0, 0, 255}, textString = "Simulate for 5 seconds for starting the machine."), Text(extent = {{-100, -80}, {100, -100}}, lineColor = {0, 0, 255}, textString = "Continue the simulation for additional 5 seconds: a load ramp is applied.")}));
          end AIMC_withLosses;

          model AIMC_Initialize "Test example: Steady-State Initialization of AsynchronousInductionMachineSquirrelCage"
            extends Modelica.Icons.Example;
            import Modelica.Constants.pi;
            constant Integer m = 3 "Number of phases";
            parameter Modelica.SIunits.Voltage VNominal = 100 "Nominal RMS voltage per phase";
            parameter Modelica.SIunits.Frequency fNominal = 50 "Nominal frequency";
            parameter Modelica.SIunits.AngularVelocity wSync = 2 * pi * fNominal / aimc.p;
            parameter Modelica.SIunits.Time tStart = 0.5 "Start time";
            parameter Modelica.SIunits.Torque TLoad = 161.4 "Nominal load torque";
            parameter Modelica.SIunits.AngularVelocity wLoad(displayUnit = "1/min") = 1440.45 * 2 * Modelica.Constants.pi / 60 "Nominal load speed";
            parameter Modelica.SIunits.Inertia JLoad = 0.29 "Load's moment of inertia";
            Machines.BasicMachines.AsynchronousInductionMachines.AIM_SquirrelCage aimc(p = aimcData.p, fsNominal = aimcData.fsNominal, Rs = aimcData.Rs, TsRef = aimcData.TsRef, alpha20s(displayUnit = "1/K") = aimcData.alpha20s, Lszero = aimcData.Lszero, Lssigma = aimcData.Lssigma, Jr = aimcData.Jr, Js = aimcData.Js, frictionParameters = aimcData.frictionParameters, statorCoreParameters = aimcData.statorCoreParameters, strayLoadParameters = aimcData.strayLoadParameters, Lm = aimcData.Lm, Lrsigma = aimcData.Lrsigma, Rr = aimcData.Rr, TrRef = aimcData.TrRef, phiMechanical(fixed = true), TsOperational = 293.15, alpha20r = aimcData.alpha20r, TrOperational = 293.15) annotation(
              Placement(transformation(extent = {{-20, -50}, {0, -30}}, rotation = 0)));
            Machines.Sensors.CurrentQuasiRMSSensor currentQuasiRMSSensor annotation(
              Placement(transformation(origin = {-10, 0}, extent = {{-10, 10}, {10, -10}}, rotation = 270)));
            Modelica.Electrical.MultiPhase.Sources.SineVoltage sineVoltage(final m = m, freqHz = fill(fNominal, m), V = fill(sqrt(2 / 3) * VNominal, m)) annotation(
              Placement(transformation(origin = {-70, 10}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
            Modelica.Electrical.MultiPhase.Basic.Star star(final m = m) annotation(
              Placement(transformation(extent = {{10, -10}, {-10, 10}}, rotation = 90, origin = {-70, -20})));
            Modelica.Electrical.Analog.Basic.Ground ground annotation(
              Placement(transformation(origin = {-70, -50}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
            Modelica.Mechanics.Rotational.Components.Inertia loadInertia(J = JLoad) annotation(
              Placement(transformation(extent = {{40, -50}, {60, -30}}, rotation = 0)));
            Mechanics.Rotational.Sources.TorqueStep torqueStep(useSupport = false, startTime = tStart, stepTorque = -TLoad, offsetTorque = 0) annotation(
              Placement(transformation(extent = {{90, -50}, {70, -30}}, rotation = 0)));
            Machines.Utilities.TerminalBox terminalBox(terminalConnection = "D") annotation(
              Placement(transformation(extent = {{-20, -30}, {0, -10}}, rotation = 0)));
            parameter Utilities.ParameterRecords.AIM_SquirrelCageData aimcData annotation(
              Placement(transformation(extent = {{-20, -80}, {0, -60}})));
          initial equation
            aimc.wMechanical = wSync;
            aimc.i_0_s = 0;
            der(aimc.idq_sr) = zeros(2);
            der(aimc.idq_rr) = zeros(2);
          equation
            connect(star.pin_n, ground.p) annotation(
              Line(points = {{-70, -30}, {-70, -30}, {-70, -40}}, color = {0, 0, 255}));
            connect(sineVoltage.plug_n, star.plug_p) annotation(
              Line(points = {{-70, 0}, {-70, -10}}, color = {0, 0, 255}));
            connect(terminalBox.plug_sn, aimc.plug_sn) annotation(
              Line(points = {{-16, -30}, {-16, -30}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(terminalBox.plug_sp, aimc.plug_sp) annotation(
              Line(points = {{-4, -30}, {-4, -30}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(terminalBox.plugSupply, currentQuasiRMSSensor.plug_n) annotation(
              Line(points = {{-10, -28}, {-10, -19}, {-10, -10}, {-10, -10}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(loadInertia.flange_b, torqueStep.flange) annotation(
              Line(points = {{60, -40}, {70, -40}}, color = {0, 0, 0}, smooth = Smooth.None));
            connect(aimc.flange, loadInertia.flange_a) annotation(
              Line(points = {{0, -40}, {40, -40}}, color = {0, 0, 0}, smooth = Smooth.None));
            connect(sineVoltage.plug_p, currentQuasiRMSSensor.plug_p) annotation(
              Line(points = {{-70, 20}, {-10, 20}, {-10, 10}}, color = {0, 0, 255}, smooth = Smooth.None));
            annotation(
              experiment(StopTime = 1.5, Interval = 0.001),
              Documentation(info = "<HTML>
<b>Test example: Steady-State Initialization of Asynchronous induction machine with squirrel cage</b><br>
The asynchronous induction machine with squirrel cage is initialized in steady-state at no-load;
at time tStart a load torque step is applied.<br>
Simulate for 1.5 seconds and plot (versus time):
<ul>
<li>currentQuasiRMSSensor.I: stator current RMS</li>
<li>aimc.wMechanical: motor's speed</li>
<li>aimc.tauElectrical: motor's torque</li>
</ul>
Default machine parameters of model <i>AIM_SquirrelCage</i> are used.
</HTML>"));
          end AIMC_Initialize;
          annotation(
            Documentation(info = "<HTML>
This package contains test examples of asynchronous induction machines.
</HTML>"));
        end AsynchronousInductionMachines;

        package SynchronousInductionMachines "Test examples of synchronous induction machines"
          extends Modelica.Icons.ExamplesPackage;

          model SMR_Inverter "Test example: SynchronousInductionMachineReluctanceRotor with inverter"
            extends Modelica.Icons.Example;
            constant Integer m = 3 "Number of phases";
            parameter Modelica.SIunits.Voltage VNominal = 100 "Nominal RMS voltage per phase";
            parameter Modelica.SIunits.Frequency fNominal = 50 "Nominal frequency";
            parameter Modelica.SIunits.Frequency f = 50 "Actual frequency";
            parameter Modelica.SIunits.Time tRamp = 1 "Frequency ramp";
            parameter Modelica.SIunits.Torque TLoad = 46 "Nominal load torque";
            parameter Modelica.SIunits.Time tStep = 1.2 "Time of load torque step";
            parameter Modelica.SIunits.Inertia JLoad = 0.29 "Load's moment of inertia";
            Machines.BasicMachines.SynchronousInductionMachines.SM_ReluctanceRotor smr(p = smrData.p, fsNominal = smrData.fsNominal, Rs = smrData.Rs, TsRef = smrData.TsRef, Lszero = smrData.Lszero, Lssigma = smrData.Lssigma, Jr = smrData.Jr, Js = smrData.Js, frictionParameters = smrData.frictionParameters, phiMechanical(fixed = true), wMechanical(fixed = true), statorCoreParameters = smrData.statorCoreParameters, strayLoadParameters = smrData.strayLoadParameters, Lmd = smrData.Lmd, Lmq = smrData.Lmq, useDamperCage = smrData.useDamperCage, Lrsigmad = smrData.Lrsigmad, Lrsigmaq = smrData.Lrsigmaq, Rrd = smrData.Rrd, Rrq = smrData.Rrq, TrRef = smrData.TrRef, TsOperational = 293.15, alpha20s = smrData.alpha20s, ir(each fixed = true), TrOperational = 293.15, alpha20r = smrData.alpha20r) annotation(
              Placement(transformation(extent = {{-20, -50}, {0, -30}}, rotation = 0)));
            Machines.Sensors.CurrentQuasiRMSSensor currentQuasiRMSSensor annotation(
              Placement(transformation(origin = {0, 30}, extent = {{-10, 10}, {10, -10}}, rotation = 270)));
            Machines.Sensors.RotorDisplacementAngle rotorDisplacementAngle(p = smr.p) annotation(
              Placement(transformation(origin = {20, -40}, extent = {{10, -10}, {-10, 10}}, rotation = 90)));
            Modelica.Blocks.Sources.Ramp ramp(height = f, duration = tRamp) annotation(
              Placement(transformation(extent = {{-80, 50}, {-60, 70}}, rotation = 0)));
            Machines.Utilities.VfController vfController(final m = m, VNominal = VNominal, fNominal = fNominal) annotation(
              Placement(transformation(extent = {{-40, 50}, {-20, 70}}, rotation = 0)));
            Modelica.Electrical.MultiPhase.Sources.SignalVoltage signalVoltage(final m = m) annotation(
              Placement(transformation(origin = {0, 60}, extent = {{10, 10}, {-10, -10}}, rotation = 270)));
            Modelica.Electrical.MultiPhase.Basic.Star star(final m = m) annotation(
              Placement(transformation(extent = {{-50, 80}, {-70, 100}}, rotation = 0)));
            Modelica.Electrical.Analog.Basic.Ground ground annotation(
              Placement(transformation(origin = {-90, 90}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
            Modelica.Mechanics.Rotational.Components.Inertia loadInertia(J = JLoad) annotation(
              Placement(transformation(extent = {{40, -50}, {60, -30}}, rotation = 0)));
            Modelica.Mechanics.Rotational.Sources.TorqueStep loadTorqueStep(startTime = tStep, stepTorque = -TLoad, useSupport = false, offsetTorque = 0) annotation(
              Placement(transformation(extent = {{90, -50}, {70, -30}}, rotation = 0)));
            Machines.Utilities.TerminalBox terminalBox(terminalConnection = "Y") annotation(
              Placement(transformation(extent = {{-20, -30}, {0, -10}}, rotation = 0)));
            parameter Utilities.ParameterRecords.SM_ReluctanceRotorData smrData annotation(
              Placement(transformation(extent = {{-20, -80}, {0, -60}})));
          initial equation
            smr.is[1:2] = zeros(2);
//conditional damper cage currents are defined as fixed start values
          equation
            connect(signalVoltage.plug_n, star.plug_p) annotation(
              Line(points = {{0, 70}, {0, 90}, {-50, 90}}, color = {0, 0, 255}));
            connect(star.pin_n, ground.p) annotation(
              Line(points = {{-70, 90}, {-80, 90}}, color = {0, 0, 255}));
            connect(ramp.y, vfController.u) annotation(
              Line(points = {{-59, 60}, {-42, 60}}, color = {0, 0, 255}));
            connect(vfController.y, signalVoltage.v) annotation(
              Line(points = {{-19, 60}, {-7, 60}}, color = {0, 0, 255}));
            connect(loadInertia.flange_b, loadTorqueStep.flange) annotation(
              Line(points = {{60, -40}, {70, -40}}, color = {0, 0, 0}));
            connect(currentQuasiRMSSensor.plug_p, signalVoltage.plug_p) annotation(
              Line(points = {{0, 40}, {0, 42.5}, {0, 50}}, color = {0, 0, 255}));
            connect(smr.plug_sn, rotorDisplacementAngle.plug_n) annotation(
              Line(points = {{-16, -30}, {-16, -20}, {26, -20}, {26, -30}}, color = {0, 0, 255}));
            connect(smr.plug_sp, rotorDisplacementAngle.plug_p) annotation(
              Line(points = {{-4, -30}, {14, -30}}, color = {0, 0, 255}));
            connect(terminalBox.plugSupply, currentQuasiRMSSensor.plug_n) annotation(
              Line(points = {{-10, -28}, {-10, 0}, {0, 0}, {0, 20}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(terminalBox.plug_sp, smr.plug_sp) annotation(
              Line(points = {{-4, -30}, {-4, -30}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(terminalBox.plug_sn, smr.plug_sn) annotation(
              Line(points = {{-16, -30}, {-16, -30}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(smr.flange, rotorDisplacementAngle.flange) annotation(
              Line(points = {{0, -40}, {10, -40}}, color = {0, 0, 0}, smooth = Smooth.None));
            connect(smr.flange, loadInertia.flange_a) annotation(
              Line(points = {{0, -40}, {40, -40}}, color = {0, 0, 0}, smooth = Smooth.None));
            annotation(
              experiment(StopTime = 1.5, Interval = 0.001),
              Documentation(info = "<HTML>
<b>Test example: Synchronous induction machine with reluctance rotor fed by an ideal inverter</b><br>
An ideal frequency inverter is modeled by using a VfController and a three-phase SignalVoltage.<br>
Frequency is raised by a ramp, causing the reluctance machine to start,
and accelerating inertias.<br>At time tStep a load step is applied.<br>
Simulate for 1.5 seconds and plot (versus time):
<ul>
<li>currentQuasiRMSSensor.I: stator current RMS</li>
<li>smr.wMechanical: motor's speed</li>
<li>smr.tauElectrical: motor's torque</li>
<li>rotorDisplacementAngle.rotorDisplacementAngle: rotor displacement angle</li>
</ul>
Default machine parameters of model <i>SM_ReluctanceRotor</i> are used.
</HTML>"));
          end SMR_Inverter;

          model SMPM_Inverter "Test example: PermanentMagnetSynchronousInductionMachine with inverter"
            extends Modelica.Icons.Example;
            constant Integer m = 3 "Number of phases";
            parameter Modelica.SIunits.Voltage VNominal = 100 "Nominal RMS voltage per phase";
            parameter Modelica.SIunits.Frequency fNominal = 50 "Nominal frequency";
            parameter Modelica.SIunits.Frequency f = 50 "Actual frequency";
            parameter Modelica.SIunits.Time tRamp = 1 "Frequency ramp";
            parameter Modelica.SIunits.Torque TLoad = 181.4 "Nominal load torque";
            parameter Modelica.SIunits.Time tStep = 1.2 "Time of load torque step";
            parameter Modelica.SIunits.Inertia JLoad = 0.29 "Load's moment of inertia";
            Machines.BasicMachines.SynchronousInductionMachines.SM_PermanentMagnet smpm(p = smpmData.p, fsNominal = smpmData.fsNominal, Rs = smpmData.Rs, TsRef = smpmData.TsRef, Lszero = smpmData.Lszero, Lssigma = smpmData.Lssigma, Jr = smpmData.Jr, Js = smpmData.Js, frictionParameters = smpmData.frictionParameters, phiMechanical(fixed = true), wMechanical(fixed = true), statorCoreParameters = smpmData.statorCoreParameters, strayLoadParameters = smpmData.strayLoadParameters, VsOpenCircuit = smpmData.VsOpenCircuit, Lmd = smpmData.Lmd, Lmq = smpmData.Lmq, useDamperCage = smpmData.useDamperCage, Lrsigmad = smpmData.Lrsigmad, Lrsigmaq = smpmData.Lrsigmaq, Rrd = smpmData.Rrd, Rrq = smpmData.Rrq, TrRef = smpmData.TrRef, permanentMagnetLossParameters = smpmData.permanentMagnetLossParameters, TsOperational = 293.15, alpha20s = smpmData.alpha20s, ir(each fixed = true), TrOperational = 293.15, alpha20r = smpmData.alpha20r) annotation(
              Placement(transformation(extent = {{-20, -50}, {0, -30}}, rotation = 0)));
            Machines.Sensors.CurrentQuasiRMSSensor currentQuasiRMSSensor annotation(
              Placement(transformation(origin = {0, 30}, extent = {{-10, 10}, {10, -10}}, rotation = 270)));
            Machines.Sensors.RotorDisplacementAngle rotorDisplacementAngle(p = smpm.p) annotation(
              Placement(transformation(origin = {20, -40}, extent = {{-10, 10}, {10, -10}}, rotation = 270)));
            Modelica.Blocks.Sources.Ramp ramp(height = f, duration = tRamp) annotation(
              Placement(transformation(extent = {{-80, 50}, {-60, 70}}, rotation = 0)));
            Machines.Utilities.VfController vfController(final m = m, VNominal = VNominal, fNominal = fNominal, BasePhase = +Modelica.Constants.pi / 2) annotation(
              Placement(transformation(extent = {{-40, 50}, {-20, 70}}, rotation = 0)));
            Modelica.Electrical.MultiPhase.Sources.SignalVoltage signalVoltage(final m = m) annotation(
              Placement(transformation(origin = {0, 60}, extent = {{10, 10}, {-10, -10}}, rotation = 270)));
            Modelica.Electrical.MultiPhase.Basic.Star star(final m = m) annotation(
              Placement(transformation(extent = {{-50, 80}, {-70, 100}}, rotation = 0)));
            Modelica.Electrical.Analog.Basic.Ground ground annotation(
              Placement(transformation(origin = {-90, 90}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
            Modelica.Mechanics.Rotational.Components.Inertia loadInertia(J = JLoad) annotation(
              Placement(transformation(extent = {{40, -50}, {60, -30}}, rotation = 0)));
            Modelica.Mechanics.Rotational.Sources.TorqueStep loadTorqueStep(startTime = tStep, stepTorque = -TLoad, useSupport = false, offsetTorque = 0) annotation(
              Placement(transformation(extent = {{90, -50}, {70, -30}}, rotation = 0)));
            Machines.Utilities.TerminalBox terminalBox(terminalConnection = "Y") annotation(
              Placement(transformation(extent = {{-20, -30}, {0, -10}}, rotation = 0)));
            parameter Utilities.ParameterRecords.SM_PermanentMagnetData smpmData annotation(
              Placement(transformation(extent = {{-20, -80}, {0, -60}})));
          initial equation
            smpm.is[1:2] = zeros(2);
//conditional damper cage currents are defined as fixed start values
          equation
            connect(signalVoltage.plug_n, star.plug_p) annotation(
              Line(points = {{0, 70}, {0, 90}, {-50, 90}}, color = {0, 0, 255}));
            connect(star.pin_n, ground.p) annotation(
              Line(points = {{-70, 90}, {-80, 90}}, color = {0, 0, 255}));
            connect(ramp.y, vfController.u) annotation(
              Line(points = {{-59, 60}, {-42, 60}}, color = {0, 0, 255}));
            connect(vfController.y, signalVoltage.v) annotation(
              Line(points = {{-19, 60}, {-7, 60}}, color = {0, 0, 255}));
            connect(loadInertia.flange_b, loadTorqueStep.flange) annotation(
              Line(points = {{60, -40}, {70, -40}}, color = {0, 0, 0}));
            connect(signalVoltage.plug_p, currentQuasiRMSSensor.plug_p) annotation(
              Line(points = {{0, 50}, {0, 50}, {0, 40}}, color = {0, 0, 255}));
            connect(rotorDisplacementAngle.plug_n, smpm.plug_sn) annotation(
              Line(points = {{26, -30}, {26, -20}, {-16, -20}, {-16, -30}}, color = {0, 0, 255}));
            connect(rotorDisplacementAngle.plug_p, smpm.plug_sp) annotation(
              Line(points = {{14, -30}, {-4, -30}}, color = {0, 0, 255}));
            connect(terminalBox.plugSupply, currentQuasiRMSSensor.plug_n) annotation(
              Line(points = {{-10, -28}, {-10, 0}, {0, 0}, {0, 20}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(terminalBox.plug_sn, smpm.plug_sn) annotation(
              Line(points = {{-16, -30}, {-16, -30}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(terminalBox.plug_sp, smpm.plug_sp) annotation(
              Line(points = {{-4, -30}, {-4, -30}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(smpm.flange, rotorDisplacementAngle.flange) annotation(
              Line(points = {{0, -40}, {10, -40}}, color = {0, 0, 0}, smooth = Smooth.None));
            connect(smpm.flange, loadInertia.flange_a) annotation(
              Line(points = {{0, -40}, {40, -40}}, color = {0, 0, 0}, smooth = Smooth.None));
            annotation(
              experiment(StopTime = 1.5, Interval = 0.001),
              Documentation(info = "<HTML>
<b>Test example: Permanent magnet synchronous induction machine fed by an ideal inverter</b><br>
An ideal frequency inverter is modeled by using a VfController and a three-phase SignalVoltage.<br>
Frequency is raised by a ramp, causing the permanent magnet synchronous induction machine to start,
and accelerating inertias.<br>At time tStep a load step is applied.<br>
Simulate for 1.5 seconds and plot (versus time):
<ul>
<li>currentQuasiRMSSensor.I: stator current RMS</li>
<li>smpm.wMechanical: motor's speed</li>
<li>smpm.tauElectrical: motor's torque</li>
<li>rotorDisplacementAngle.rotorDisplacementAngle: rotor displacement angle</li>
</ul>
Default machine parameters of model <i>SM_PermanentMagnet</i> are used.
<p>
<b>In practice it is nearly impossible to drive a PMSMD without current controller.</b>
</p>
</HTML>"));
          end SMPM_Inverter;

          model SMPM_CurrentSource "Test example: PermanentMagnetSynchronousInductionMachine fed by current source"
            extends Modelica.Icons.Example;
            constant Integer m = 3 "Number of phases";
            parameter Modelica.SIunits.Voltage VNominal = 100 "Nominal RMS voltage per phase";
            parameter Modelica.SIunits.Frequency fNominal = 50 "Nominal frequency";
            parameter Modelica.SIunits.Frequency f = 50 "Actual frequency";
            parameter Modelica.SIunits.Time tRamp = 1 "Frequency ramp";
            parameter Modelica.SIunits.Torque TLoad = 181.4 "Nominal load torque";
            parameter Modelica.SIunits.Time tStep = 1.2 "Time of load torque step";
            parameter Modelica.SIunits.Inertia JLoad = 0.29 "Load's moment of inertia";
            Machines.BasicMachines.SynchronousInductionMachines.SM_PermanentMagnet smpm(p = smpmData.p, fsNominal = smpmData.fsNominal, TsOperational = 293.15, Rs = smpmData.Rs, TsRef = smpmData.TsRef, alpha20s = smpmData.alpha20s, Lszero = smpmData.Lszero, Lssigma = smpmData.Lssigma, Jr = smpmData.Jr, Js = smpmData.Js, frictionParameters = smpmData.frictionParameters, phiMechanical(fixed = true), wMechanical(fixed = true), statorCoreParameters = smpmData.statorCoreParameters, strayLoadParameters = smpmData.strayLoadParameters, TrOperational = 293.15, VsOpenCircuit = smpmData.VsOpenCircuit, Lmd = smpmData.Lmd, Lmq = smpmData.Lmq, useDamperCage = smpmData.useDamperCage, Lrsigmad = smpmData.Lrsigmad, Lrsigmaq = smpmData.Lrsigmaq, Rrd = smpmData.Rrd, Rrq = smpmData.Rrq, TrRef = smpmData.TrRef, alpha20r = smpmData.alpha20r, permanentMagnetLossParameters = smpmData.permanentMagnetLossParameters) annotation(
              Placement(transformation(extent = {{-20, -50}, {0, -30}}, rotation = 0)));
            Modelica.Electrical.MultiPhase.Sources.SignalCurrent signalCurrent(final m = m) annotation(
              Placement(transformation(origin = {-10, 50}, extent = {{-10, 10}, {10, -10}}, rotation = 270)));
            Modelica.Electrical.MultiPhase.Basic.Star star(final m = m) annotation(
              Placement(transformation(extent = {{-50, 80}, {-70, 100}}, rotation = 0)));
            Modelica.Electrical.Analog.Basic.Ground ground annotation(
              Placement(transformation(origin = {-90, 90}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
            Utilities.CurrentController currentController(p = smpm.p) annotation(
              Placement(transformation(extent = {{-50, 40}, {-30, 60}})));
            Modelica.Blocks.Sources.Constant iq(k = 84.6) annotation(
              Placement(transformation(extent = {{-90, 20}, {-70, 40}})));
            Modelica.Blocks.Sources.Constant id(k = -53.5) annotation(
              Placement(transformation(extent = {{-90, 60}, {-70, 80}})));
            Sensors.VoltageQuasiRMSSensor voltageQuasiRMSSensor annotation(
              Placement(transformation(extent = {{-10, 10}, {10, -10}}, rotation = 180, origin = {-30, -10})));
            Modelica.Electrical.MultiPhase.Basic.Star starM(final m = m) annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 180, origin = {-60, -10})));
            Modelica.Electrical.Analog.Basic.Ground groundM annotation(
              Placement(transformation(origin = {-80, -28}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
            Machines.Utilities.TerminalBox terminalBox(terminalConnection = "Y") annotation(
              Placement(transformation(extent = {{-20, -30}, {0, -10}}, rotation = 0)));
            Machines.Sensors.RotorDisplacementAngle rotorDisplacementAngle(p = smpm.p) annotation(
              Placement(transformation(origin = {20, -40}, extent = {{-10, 10}, {10, -10}}, rotation = 270)));
            Mechanics.Rotational.Sensors.AngleSensor angleSensor annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 90, origin = {10, 0})));
            Mechanics.Rotational.Sensors.TorqueSensor torqueSensor annotation(
              Placement(transformation(extent = {{10, 10}, {-10, -10}}, rotation = 180, origin = {40, -60})));
            Mechanics.Rotational.Sensors.SpeedSensor speedSensor annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 90, origin = {30, 0})));
            Mechanics.Rotational.Components.Inertia inertiaLoad(J = 0.29) annotation(
              Placement(transformation(extent = {{50, -50}, {70, -30}})));
            Mechanics.Rotational.Sources.QuadraticSpeedDependentTorque quadraticSpeedDependentTorque(tau_nominal = -181.4, w_nominal(displayUnit = "rpm") = 157.07963267949) annotation(
              Placement(transformation(extent = {{100, -50}, {80, -30}})));
            parameter Utilities.ParameterRecords.SM_PermanentMagnetData smpmData(useDamperCage = false) annotation(
              Placement(transformation(extent = {{-20, -80}, {0, -60}})));
            Sensors.CurrentQuasiRMSSensor currentQuasiRMSSensor annotation(
              Placement(transformation(origin = {-10, 0}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          equation
            connect(star.pin_n, ground.p) annotation(
              Line(points = {{-70, 90}, {-80, 90}}, color = {0, 0, 255}));
            connect(rotorDisplacementAngle.plug_n, smpm.plug_sn) annotation(
              Line(points = {{26, -30}, {26, -20}, {-16, -20}, {-16, -30}}, color = {0, 0, 255}));
            connect(rotorDisplacementAngle.plug_p, smpm.plug_sp) annotation(
              Line(points = {{14, -30}, {-4, -30}}, color = {0, 0, 255}));
            connect(terminalBox.plug_sn, smpm.plug_sn) annotation(
              Line(points = {{-16, -30}, {-16, -30}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(terminalBox.plug_sp, smpm.plug_sp) annotation(
              Line(points = {{-4, -30}, {-4, -30}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(smpm.flange, rotorDisplacementAngle.flange) annotation(
              Line(points = {{0, -40}, {10, -40}}, color = {0, 0, 0}, smooth = Smooth.None));
            connect(signalCurrent.plug_p, star.plug_p) annotation(
              Line(points = {{-10, 60}, {-10, 90}, {-50, 90}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(angleSensor.flange, rotorDisplacementAngle.flange) annotation(
              Line(points = {{10, -10}, {10, -40}}, color = {0, 0, 0}, smooth = Smooth.None));
            connect(angleSensor.phi, currentController.phi) annotation(
              Line(points = {{10, 11}, {10, 30}, {-40, 30}, {-40, 38}}, color = {0, 0, 127}, smooth = Smooth.None));
            connect(id.y, currentController.id_rms) annotation(
              Line(points = {{-69, 70}, {-60, 70}, {-60, 56}, {-52, 56}}, color = {0, 0, 127}, smooth = Smooth.None));
            connect(iq.y, currentController.iq_rms) annotation(
              Line(points = {{-69, 30}, {-60, 30}, {-60, 44}, {-52, 44}}, color = {0, 0, 127}, smooth = Smooth.None));
            connect(groundM.p, terminalBox.starpoint) annotation(
              Line(points = {{-70, -28}, {-19, -28}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(smpm.flange, torqueSensor.flange_a) annotation(
              Line(points = {{0, -40}, {30, -40}, {30, -60}}, color = {0, 0, 0}, smooth = Smooth.None));
            connect(voltageQuasiRMSSensor.plug_p, terminalBox.plugSupply) annotation(
              Line(points = {{-20, -10}, {-10, -10}, {-10, -28}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(starM.plug_p, voltageQuasiRMSSensor.plug_n) annotation(
              Line(points = {{-50, -10}, {-40, -10}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(starM.pin_n, groundM.p) annotation(
              Line(points = {{-70, -10}, {-70, -28}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(currentController.y, signalCurrent.i) annotation(
              Line(points = {{-29, 50}, {-17, 50}}, color = {0, 0, 127}, smooth = Smooth.None));
            connect(speedSensor.flange, smpm.flange) annotation(
              Line(points = {{30, -10}, {30, -40}, {0, -40}}, color = {0, 0, 0}, smooth = Smooth.None));
            connect(quadraticSpeedDependentTorque.flange, inertiaLoad.flange_b) annotation(
              Line(points = {{80, -40}, {70, -40}}, color = {0, 0, 0}, smooth = Smooth.None));
            connect(torqueSensor.flange_b, inertiaLoad.flange_a) annotation(
              Line(points = {{50, -60}, {50, -40}}, color = {0, 0, 0}, smooth = Smooth.None));
            connect(signalCurrent.plug_n, currentQuasiRMSSensor.plug_p) annotation(
              Line(points = {{-10, 40}, {-10, 10}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(currentQuasiRMSSensor.plug_n, voltageQuasiRMSSensor.plug_p) annotation(
              Line(points = {{-10, -10}, {-20, -10}}, color = {0, 0, 255}, smooth = Smooth.None));
            annotation(
              experiment(StopTime = 2.0, Interval = 0.001),
              Documentation(info = "<html>
<p>A synchronous induction machine with permanent magnets accelerates a quadratic speed dependent load from standstill.
The rms values of d- and q-current in rotor fixed coordinate system are converted to three-phase currents,
and fed to the machine. The result shows that the torque is influenced by the q-current,
whereas the stator voltage is influenced by the d-current.</p>
<p>
Default machine parameters of model <a href=\"modelica://Modelica.Electrical.Machines.BasicMachines.SynchronousInductionMachines.SM_PermanentMagnet\">SM_PermanentMagnet</a> are used.
</p>
</html>"));
          end SMPM_CurrentSource;

          model SMPM_VoltageSource "Test example: PermanentMagnetSynchronousInductionMachine fed by FOC"
            extends Modelica.Icons.Example;
            constant Integer m = 3 "Number of phases";
            parameter Modelica.SIunits.Voltage VNominal = 100 "Nominal RMS voltage per phase";
            parameter Modelica.SIunits.Frequency fNominal = 50 "Nominal frequency";
            parameter Modelica.SIunits.Frequency f = 50 "Actual frequency";
            parameter Modelica.SIunits.Time tRamp = 1 "Frequency ramp";
            parameter Modelica.SIunits.Torque TLoad = 181.4 "Nominal load torque";
            parameter Modelica.SIunits.Time tStep = 1.2 "Time of load torque step";
            parameter Modelica.SIunits.Inertia JLoad = 0.29 "Load's moment of inertia";
            Machines.BasicMachines.SynchronousInductionMachines.SM_PermanentMagnet smpm(phiMechanical(start = 0, fixed = true), wMechanical(start = 0, fixed = true), useSupport = false, useThermalPort = false, p = smpmData.p, fsNominal = smpmData.fsNominal, Rs = smpmData.Rs, TsRef = smpmData.TsRef, Lszero = smpmData.Lszero, Lssigma = smpmData.Lssigma, Jr = smpmData.Jr, Js = smpmData.Js, frictionParameters = smpmData.frictionParameters, statorCoreParameters = smpmData.statorCoreParameters, strayLoadParameters = smpmData.strayLoadParameters, VsOpenCircuit = smpmData.VsOpenCircuit, Lmd = smpmData.Lmd, Lmq = smpmData.Lmq, useDamperCage = smpmData.useDamperCage, Lrsigmad = smpmData.Lrsigmad, Lrsigmaq = smpmData.Lrsigmaq, Rrd = smpmData.Rrd, Rrq = smpmData.Rrq, TrRef = smpmData.TrRef, permanentMagnetLossParameters = smpmData.permanentMagnetLossParameters, TsOperational = 293.15, alpha20s = smpmData.alpha20s, TrOperational = 293.15, alpha20r = smpmData.alpha20r) annotation(
              Placement(transformation(extent = {{-20, -50}, {0, -30}}, rotation = 0)));
            Modelica.Electrical.MultiPhase.Sources.SignalVoltage signalVoltage(final m = m) annotation(
              Placement(transformation(origin = {-10, 50}, extent = {{10, 10}, {-10, -10}}, rotation = 270)));
            Modelica.Electrical.MultiPhase.Basic.Star star(final m = m) annotation(
              Placement(transformation(extent = {{-50, 80}, {-70, 100}}, rotation = 0)));
            Modelica.Electrical.Analog.Basic.Ground ground annotation(
              Placement(transformation(origin = {-90, 90}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
            Modelica.Blocks.Sources.Constant iq(k = 84.6) annotation(
              Placement(transformation(extent = {{-90, 20}, {-70, 40}})));
            Modelica.Blocks.Sources.Constant id(k = -53.5) annotation(
              Placement(transformation(extent = {{-90, 60}, {-70, 80}})));
            Machines.Utilities.TerminalBox terminalBox(terminalConnection = "Y") annotation(
              Placement(transformation(extent = {{-20, -30}, {0, -10}}, rotation = 0)));
            Modelica.Mechanics.Rotational.Sensors.AngleSensor angleSensor annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 90, origin = {10, 0})));
            Modelica.Mechanics.Rotational.Components.Inertia inertiaLoad(J = 0.29) annotation(
              Placement(transformation(extent = {{50, -50}, {70, -30}})));
            Modelica.Mechanics.Rotational.Sources.QuadraticSpeedDependentTorque quadraticSpeedDependentTorque(tau_nominal = -181.4, w_nominal(displayUnit = "rpm") = 157.07963267949) annotation(
              Placement(transformation(extent = {{100, -50}, {80, -30}})));
            Modelica.Electrical.MultiPhase.Sensors.CurrentSensor currentSensor(m = m) annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 270, origin = {-10, 0})));
            Machines.Utilities.VoltageController voltageController(p = smpm.p, Ld = smpm.Lssigma + smpm.Lmd, Lq = smpm.Lssigma + smpm.Lmq, Rs = Machines.Thermal.convertResistance(smpm.Rs, smpm.TsRef, smpm.alpha20s, smpm.TsOperational), fsNominal = smpm.fsNominal, VsOpenCircuit = smpm.VsOpenCircuit) annotation(
              Placement(transformation(extent = {{-50, 40}, {-30, 60}})));
            Modelica.Mechanics.Rotational.Sensors.TorqueSensor torqueSensor annotation(
              Placement(transformation(extent = {{10, 10}, {-10, -10}}, rotation = 180, origin = {40, -60})));
            Modelica.Mechanics.Rotational.Sensors.SpeedSensor speedSensor annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 90, origin = {30, 0})));
            Machines.Sensors.RotorDisplacementAngle rotorDisplacementAngle(p = smpm.p) annotation(
              Placement(transformation(origin = {20, -40}, extent = {{-10, 10}, {10, -10}}, rotation = 270)));
            Modelica.Electrical.Analog.Basic.Ground groundM annotation(
              Placement(transformation(origin = {-80, -28}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
            Modelica.Electrical.MultiPhase.Basic.Star starM(final m = m) annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 180, origin = {-60, -10})));
            Machines.Sensors.VoltageQuasiRMSSensor voltageQuasiRMSSensor annotation(
              Placement(transformation(extent = {{-10, 10}, {10, -10}}, rotation = 180, origin = {-30, -10})));
            parameter Machines.Utilities.ParameterRecords.SM_PermanentMagnetData smpmData(useDamperCage = false) annotation(
              Placement(transformation(extent = {{-20, -80}, {0, -60}})));
            Sensors.CurrentQuasiRMSSensor currentQuasiRMSSensor annotation(
              Placement(transformation(origin = {-10, 20}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          initial equation
            smpm.is[1:2] = zeros(2);
          equation
            connect(star.pin_n, ground.p) annotation(
              Line(points = {{-70, 90}, {-80, 90}}, color = {0, 0, 255}));
            connect(terminalBox.plug_sn, smpm.plug_sn) annotation(
              Line(points = {{-16, -30}, {-16, -30}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(terminalBox.plug_sp, smpm.plug_sp) annotation(
              Line(points = {{-4, -30}, {-4, -30}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(quadraticSpeedDependentTorque.flange, inertiaLoad.flange_b) annotation(
              Line(points = {{80, -40}, {70, -40}}, color = {0, 0, 0}, smooth = Smooth.None));
            connect(smpm.flange, angleSensor.flange) annotation(
              Line(points = {{0, -40}, {10, -40}, {10, -10}}, color = {0, 0, 0}, smooth = Smooth.None));
            connect(star.plug_p, signalVoltage.plug_n) annotation(
              Line(points = {{-50, 90}, {-10, 90}, {-10, 60}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(currentSensor.plug_n, terminalBox.plugSupply) annotation(
              Line(points = {{-10, -10}, {-10, -28}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(id.y, voltageController.id_rms) annotation(
              Line(points = {{-69, 70}, {-60, 70}, {-60, 56}, {-52, 56}}, color = {0, 0, 127}, smooth = Smooth.None));
            connect(iq.y, voltageController.iq_rms) annotation(
              Line(points = {{-69, 30}, {-60, 30}, {-60, 44}, {-52, 44}}, color = {0, 0, 127}, smooth = Smooth.None));
            connect(angleSensor.phi, voltageController.phi) annotation(
              Line(points = {{10, 11}, {10, 30}, {-34, 30}, {-34, 38}}, color = {0, 0, 127}, smooth = Smooth.None));
            connect(voltageController.y, signalVoltage.v) annotation(
              Line(points = {{-29, 50}, {-17, 50}}, color = {0, 0, 127}, smooth = Smooth.None));
            connect(currentSensor.i, voltageController.iActual) annotation(
              Line(points = {{-21, 1.9984e-015}, {-46, 1.9984e-015}, {-46, 38}}, color = {0, 0, 127}, smooth = Smooth.None));
            connect(inertiaLoad.flange_a, torqueSensor.flange_b) annotation(
              Line(points = {{50, -40}, {50, -60}}, color = {0, 0, 0}, smooth = Smooth.None));
            connect(torqueSensor.flange_a, smpm.flange) annotation(
              Line(points = {{30, -60}, {30, -40}, {0, -40}}, color = {0, 0, 0}, smooth = Smooth.None));
            connect(speedSensor.flange, smpm.flange) annotation(
              Line(points = {{30, -10}, {30, -40}, {0, -40}}, color = {0, 0, 0}, smooth = Smooth.None));
            connect(rotorDisplacementAngle.flange, smpm.flange) annotation(
              Line(points = {{10, -40}, {0, -40}}, color = {0, 0, 0}, smooth = Smooth.None));
            connect(rotorDisplacementAngle.plug_p, smpm.plug_sp) annotation(
              Line(points = {{14, -30}, {-4, -30}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(rotorDisplacementAngle.plug_n, smpm.plug_sn) annotation(
              Line(points = {{26, -30}, {26, -20}, {-16, -20}, {-16, -30}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(voltageQuasiRMSSensor.plug_p, currentSensor.plug_n) annotation(
              Line(points = {{-20, -10}, {-10, -10}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(starM.plug_p, voltageQuasiRMSSensor.plug_n) annotation(
              Line(points = {{-50, -10}, {-40, -10}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(groundM.p, starM.pin_n) annotation(
              Line(points = {{-70, -28}, {-70, -10}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(currentQuasiRMSSensor.plug_n, currentSensor.plug_p) annotation(
              Line(points = {{-10, 10}, {-10, 10}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(signalVoltage.plug_p, currentQuasiRMSSensor.plug_p) annotation(
              Line(points = {{-10, 40}, {-10, 30}}, color = {0, 0, 255}, smooth = Smooth.None));
            annotation(
              experiment(StopTime = 2.0, Interval = 0.001),
              Documentation(info = "<html>
<p>
A synchronous induction machine with permanent magnets accelerates a quadratic speed dependent load from standstill.
The rms values of d- and q-current in rotor fixed coordinate system are controlled by the voltageController,
and the output voltages fed to the machine. The result shows that the torque is influenced by the q-current,
whereas the stator voltage is influenced by the d-current.</p>
<p>
Default machine parameters of model <a href=\"modelica://Modelica.Electrical.Machines.BasicMachines.SynchronousInductionMachines.SM_PermanentMagnet\">SM_PermanentMagnet</a> are used.
</p>
</html>"));
          end SMPM_VoltageSource;

          model SMEE_Generator "Test example: ElectricalExcitedSynchronousInductionMachine as Generator"
            extends Modelica.Icons.Example;
            constant Integer m = 3 "Number of phases";
            parameter Modelica.SIunits.Voltage VNominal = 100 "Nominal RMS voltage per phase";
            parameter Modelica.SIunits.Frequency fNominal = 50 "Nominal frequency";
            parameter Modelica.SIunits.AngularVelocity wActual(displayUnit = "1/min") = 1499 * 2 * Modelica.Constants.pi / 60 "Actual speed";
            parameter Modelica.SIunits.Current Ie = 19 "Excitation current";
            parameter Modelica.SIunits.Current Ie0 = 10 "Initial excitation current";
            parameter Modelica.SIunits.Angle gamma0(displayUnit = "deg") = 0 "Initial rotor displacement angle";
            Machines.BasicMachines.SynchronousInductionMachines.SM_ElectricalExcited smee(phiMechanical(start = -(Modelica.Constants.pi + gamma0) / smee.p, fixed = true), fsNominal = smeeData.fsNominal, Rs = smeeData.Rs, TsRef = smeeData.TsRef, Lssigma = smeeData.Lssigma, Lmd = smeeData.Lmd, Lmq = smeeData.Lmq, Lrsigmad = smeeData.Lrsigmad, Lrsigmaq = smeeData.Lrsigmaq, Rrd = smeeData.Rrd, Rrq = smeeData.Rrq, TrRef = smeeData.TrRef, VsNominal = smeeData.VsNominal, IeOpenCircuit = smeeData.IeOpenCircuit, Re = smeeData.Re, TeRef = smeeData.TeRef, sigmae = smeeData.sigmae, p = 2, Jr = 0.29, Js = 0.29, useDamperCage = true, statorCoreParameters(VRef = 100), strayLoadParameters(IRef = 100), brushParameters(ILinear = 0.01), TsOperational = 293.15, alpha20s = smeeData.alpha20s, ir(each fixed = true), TrOperational = 293.15, alpha20r = smeeData.alpha20r, alpha20e = smeeData.alpha20e, TeOperational = 293.15) annotation(
              Placement(transformation(extent = {{-20, -50}, {0, -30}}, rotation = 0)));
            Machines.Sensors.RotorDisplacementAngle rotorDisplacementAngle(p = smee.p) annotation(
              Placement(transformation(origin = {20, -40}, extent = {{-10, 10}, {10, -10}}, rotation = 270)));
            Modelica.Electrical.Analog.Basic.Ground groundExcitation annotation(
              Placement(transformation(origin = {-90, -50}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
            Modelica.Mechanics.Rotational.Sources.ConstantSpeed constantSpeed(final w_fixed = wActual) annotation(
              Placement(transformation(extent = {{90, -50}, {70, -30}}, rotation = 0)));
            Machines.Sensors.MechanicalPowerSensor mechanicalPowerSensor annotation(
              Placement(transformation(extent = {{40, -50}, {60, -30}}, rotation = 0)));
            Machines.Sensors.ElectricalPowerSensor electricalPowerSensor annotation(
              Placement(transformation(origin = {0, 60}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
            Machines.Sensors.CurrentQuasiRMSSensor currentQuasiRMSSensor annotation(
              Placement(transformation(origin = {0, 30}, extent = {{-10, 10}, {10, -10}}, rotation = 270)));
            Modelica.Electrical.MultiPhase.Sources.SineVoltage sineVoltage(final m = m, final V = fill(VNominal * sqrt(2), m), final freqHz = fill(fNominal, m)) annotation(
              Placement(transformation(extent = {{-20, 80}, {-40, 100}}, rotation = 0)));
            Modelica.Electrical.MultiPhase.Basic.Star star(final m = m) annotation(
              Placement(transformation(extent = {{-50, 80}, {-70, 100}}, rotation = 0)));
            Modelica.Electrical.Analog.Basic.Ground ground annotation(
              Placement(transformation(origin = {-90, 90}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
            Modelica.Electrical.Analog.Sources.RampCurrent rampCurrent(duration = 0.1, I = Ie - Ie0, offset = Ie0) annotation(
              Placement(transformation(origin = {-50, -40}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
            Machines.Utilities.TerminalBox terminalBox(terminalConnection = "Y") annotation(
              Placement(transformation(extent = {{-20, -30}, {0, -10}}, rotation = 0)));
            parameter Machines.Utilities.SynchronousMachineData smeeData(SNominal = 30e3, VsNominal = 100, fsNominal = 50, IeOpenCircuit = 10, x0 = 0.1, xd = 1.6, xq = 1.6, xdTransient = 0.1375, xdSubtransient = 0.121428571, xqSubtransient = 0.148387097, Ta = 0.014171268, Td0Transient = 0.261177343, Td0Subtransient = 0.006963029, Tq0Subtransient = 0.123345081, alpha20s(displayUnit = "1/K") = Modelica.Electrical.Machines.Thermal.Constants.alpha20Zero, alpha20r(displayUnit = "1/K") = Modelica.Electrical.Machines.Thermal.Constants.alpha20Zero, alpha20e(displayUnit = "1/K") = Modelica.Electrical.Machines.Thermal.Constants.alpha20Zero, TsSpecification = 293.15, TsRef = 293.15, TrSpecification = 293.15, TrRef = 293.15, TeSpecification = 293.15, TeRef = 293.15) annotation(
              Placement(transformation(extent = {{-20, -80}, {0, -60}})));
          initial equation
            smee.is[1:2] = zeros(2);
//conditional damper cage currents are defined as fixed start values
          equation
            connect(rotorDisplacementAngle.plug_n, smee.plug_sn) annotation(
              Line(points = {{26, -30}, {26, -20}, {-16, -20}, {-16, -30}}, color = {0, 0, 255}));
            connect(rotorDisplacementAngle.plug_p, smee.plug_sp) annotation(
              Line(points = {{14, -30}, {-4, -30}}, color = {0, 0, 255}));
            connect(star.pin_n, ground.p) annotation(
              Line(points = {{-70, 90}, {-80, 90}}, color = {0, 0, 255}));
            connect(star.plug_p, sineVoltage.plug_n) annotation(
              Line(points = {{-50, 90}, {-40, 90}}, color = {0, 0, 255}));
            connect(electricalPowerSensor.plug_ni, currentQuasiRMSSensor.plug_p) annotation(
              Line(points = {{0, 50}, {0, 46}, {0, 40}}, color = {0, 0, 255}));
            connect(mechanicalPowerSensor.flange_b, constantSpeed.flange) annotation(
              Line(points = {{60, -40}, {70, -40}}, color = {0, 0, 0}));
            connect(sineVoltage.plug_p, electricalPowerSensor.plug_p) annotation(
              Line(points = {{-20, 90}, {0, 90}, {0, 70}}, color = {0, 0, 255}));
            connect(rampCurrent.p, groundExcitation.p) annotation(
              Line(points = {{-50, -50}, {-80, -50}}, color = {0, 0, 255}));
            connect(rampCurrent.p, smee.pin_en) annotation(
              Line(points = {{-50, -50}, {-40, -50}, {-40, -46}, {-20, -46}}, color = {0, 0, 255}));
            connect(rampCurrent.n, smee.pin_ep) annotation(
              Line(points = {{-50, -30}, {-40, -30}, {-40, -34}, {-20, -34}}, color = {0, 0, 255}));
            connect(electricalPowerSensor.plug_nv, smee.plug_sn) annotation(
              Line(points = {{-10, 60}, {-16, 60}, {-16, -30}}, color = {0, 0, 255}));
            connect(terminalBox.plugSupply, currentQuasiRMSSensor.plug_n) annotation(
              Line(points = {{-10, -28}, {-10, 0}, {0, 0}, {0, 20}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(terminalBox.plug_sn, smee.plug_sn) annotation(
              Line(points = {{-16, -30}, {-16, -30}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(terminalBox.plug_sp, smee.plug_sp) annotation(
              Line(points = {{-4, -30}, {-4, -30}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(smee.flange, rotorDisplacementAngle.flange) annotation(
              Line(points = {{0, -40}, {10, -40}}, color = {0, 0, 0}, smooth = Smooth.None));
            connect(smee.flange, mechanicalPowerSensor.flange_a) annotation(
              Line(points = {{0, -40}, {40, -40}}, color = {0, 0, 0}, smooth = Smooth.None));
            annotation(
              experiment(StopTime = 30, Interval = 0.005),
              Documentation(info = "<HTML>
<b>Test example: Electrical excited synchronous induction machine as generator</b><br>
An electrically excited synchronous generator is connected to the grid and driven with constant speed.
Since speed is slightly smaller than synchronous speed corresponding to mains frequency,
rotor angle is very slowly increased. This allows to see several characteristics dependent on rotor angle.
Simulate for 30 seconds and plot (versus rotorDisplacementAngle.rotorDisplacementAngle):
<ul>
<li>smee.tauElectrical</li>
<li>currentQuasiRMSSensor.I</li>
<li>electricalPowerSensor.P</li>
<li>electricalPowerSensor.Q</li>
<li>mechanicalPowerSensor.P</li>
</ul>
Default machine parameters of model <i>SM_ElectricalExcited</i> are used.
</HTML>"));
          end SMEE_Generator;

          model SMEE_LoadDump "Test example: ElectricalExcitedSynchronousInductionMachine with voltage controller"
            extends Modelica.Icons.Example;
            import Modelica.Constants.pi;
            constant Integer m = 3 "Number of phases";
            parameter Modelica.SIunits.AngularVelocity wNominal = 2 * pi * smeeData.fsNominal / smee.p "Nominal speed";
            parameter Modelica.SIunits.Impedance ZNominal = 3 * smeeData.VsNominal ^ 2 / smeeData.SNominal "Nominal load impedance";
            parameter Real powerFactor(min = 0, max = 1) = 0.8 "Load power factor";
            parameter Modelica.SIunits.Resistance RLoad = ZNominal * powerFactor "Load resistance";
            parameter Modelica.SIunits.Inductance LLoad = ZNominal * sqrt(1 - powerFactor ^ 2) / (2 * pi * smeeData.fsNominal) "Load inductance";
            parameter Modelica.SIunits.Voltage Ve0 = smee.IeOpenCircuit * Machines.Thermal.convertResistance(smee.Re, smee.TeRef, smee.alpha20e, smee.TeOperational) "No load excitation voltage";
            parameter Real k = 2 * Ve0 / smeeData.VsNominal "Voltage controller: gain";
            parameter Modelica.SIunits.Time Ti = smeeData.Td0Transient / 2 "Voltage controller: integral time constant";
            output Real controlError = (setPointGain.y - voltageQuasiRMSSensor.V) / smeeData.VsNominal;
            Machines.BasicMachines.SynchronousInductionMachines.SM_ElectricalExcited smee(fsNominal = smeeData.fsNominal, Rs = smeeData.Rs, TsRef = smeeData.TsRef, Lssigma = smeeData.Lssigma, Lmd = smeeData.Lmd, Lmq = smeeData.Lmq, Lrsigmad = smeeData.Lrsigmad, Lrsigmaq = smeeData.Lrsigmaq, Rrd = smeeData.Rrd, Rrq = smeeData.Rrq, TrRef = smeeData.TrRef, VsNominal = smeeData.VsNominal, IeOpenCircuit = smeeData.IeOpenCircuit, Re = smeeData.Re, TeRef = smeeData.TeRef, sigmae = smeeData.sigmae, useDamperCage = true, p = 2, Jr = 0.29, Js = 0.29, statorCoreParameters(VRef = 100), strayLoadParameters(IRef = 100), brushParameters(ILinear = 0.01), TsOperational = 293.15, alpha20s = smeeData.alpha20s, TrOperational = 293.15, alpha20r = smeeData.alpha20r, alpha20e = smeeData.alpha20e, TeOperational = 293.15) annotation(
              Placement(transformation(extent = {{0, -40}, {20, -20}}, rotation = 0)));
            parameter Machines.Utilities.SynchronousMachineData smeeData(SNominal = 30e3, VsNominal = 100, fsNominal = 50, IeOpenCircuit = 10, x0 = 0.1, xd = 1.6, xq = 1.6, xdTransient = 0.1375, xdSubtransient = 0.121428571, xqSubtransient = 0.148387097, Ta = 0.014171268, Td0Transient = 0.261177343, Td0Subtransient = 0.006963029, Tq0Subtransient = 0.123345081, TsSpecification = 293.15, TsRef = 293.15, alpha20s(displayUnit = "1/K") = Modelica.Electrical.Machines.Thermal.Constants.alpha20Zero, TrSpecification = 293.15, TrRef = 293.15, alpha20r(displayUnit = "1/K") = Modelica.Electrical.Machines.Thermal.Constants.alpha20Zero, TeSpecification = 293.15, TeRef = 293.15, alpha20e(displayUnit = "1/K") = Modelica.Electrical.Machines.Thermal.Constants.alpha20Zero) annotation(
              Placement(transformation(extent = {{0, -70}, {20, -50}})));
            Machines.Utilities.TerminalBox terminalBox(terminalConnection = "Y") annotation(
              Placement(transformation(extent = {{0, -20}, {20, 0}}, rotation = 0)));
            Modelica.Electrical.Analog.Basic.Ground ground annotation(
              Placement(transformation(origin = {-90, 0}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
            Modelica.Mechanics.Rotational.Sources.Speed speed annotation(
              Placement(transformation(extent = {{50, -40}, {30, -20}})));
            Modelica.Blocks.Sources.Ramp speedRamp(height = wNominal, duration = 1) annotation(
              Placement(transformation(extent = {{80, -40}, {60, -20}})));
            Modelica.Mechanics.Rotational.Sensors.SpeedSensor speedSensor annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 270, origin = {30, -50})));
            Modelica.Blocks.Math.Gain setPointGain(k = smeeData.VsNominal / wNominal / unitMagneticFlux) annotation(
              Placement(transformation(extent = {{-50, -90}, {-70, -70}})));
            Machines.Sensors.VoltageQuasiRMSSensor voltageQuasiRMSSensor(ToSpacePhasor1(y(each start = 1E-3, each fixed = true))) annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 270, origin = {0, 0})));
            Modelica.Blocks.Continuous.LimPID voltageController(controllerType = Modelica.Blocks.Types.SimpleController.PI, k = k, Ti = Ti, yMax = 2.5 * Ve0, yMin = 0, initType = Modelica.Blocks.Types.InitPID.InitialState, Td = 0.001) annotation(
              Placement(transformation(extent = {{-70, -20}, {-50, -40}})));
            Modelica.Electrical.Analog.Sources.SignalVoltage excitationVoltage annotation(
              Placement(transformation(extent = {{10, -10}, {-10, 10}}, rotation = 90, origin = {-30, -30})));
            Modelica.Electrical.Analog.Basic.Ground groundExcitation annotation(
              Placement(transformation(origin = {-30, -60}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
            Machines.Sensors.CurrentQuasiRMSSensor currentQuasiRMSSensor annotation(
              Placement(transformation(origin = {10, 30}, extent = {{-10, 10}, {10, -10}}, rotation = 270)));
            Modelica.Blocks.Sources.BooleanPulse loadControl(period = 4, startTime = 2) annotation(
              Placement(transformation(extent = {{-40, 70}, {-20, 90}})));
            Modelica.Electrical.MultiPhase.Ideal.CloserWithArc switch(m = m, Ron = fill(1e-5, m), Goff = fill(1e-5, m), V0 = fill(30, m), dVdt = fill(10e3, m), Vmax = fill(60, m)) annotation(
              Placement(transformation(extent = {{0, 40}, {-20, 60}})));
            Modelica.Electrical.MultiPhase.Basic.Resistor loadResistor(m = m, R = fill(RLoad, m)) annotation(
              Placement(transformation(extent = {{-30, 40}, {-50, 60}})));
            Modelica.Electrical.MultiPhase.Basic.Inductor loadInductor(m = m, L = fill(LLoad, m)) annotation(
              Placement(transformation(extent = {{-60, 40}, {-80, 60}})));
            Modelica.Electrical.MultiPhase.Basic.Star star(m = m) annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 270, origin = {-90, 30})));
          protected
            constant Modelica.SIunits.MagneticFlux unitMagneticFlux = 1 annotation(
              HideResult = true);
          initial equation
            smee.idq_sr = zeros(2);
//conditional damper cage currents are defined as fixed start values
            smee.ie = 0;
          equation
            connect(terminalBox.plug_sn, smee.plug_sn) annotation(
              Line(points = {{4, -20}, {4, -20}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(terminalBox.plug_sp, smee.plug_sp) annotation(
              Line(points = {{16, -20}, {16, -20}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(excitationVoltage.p, smee.pin_ep) annotation(
              Line(points = {{-30, -20}, {-20, -20}, {-20, -24}, {0, -24}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(excitationVoltage.n, smee.pin_en) annotation(
              Line(points = {{-30, -40}, {-20, -40}, {-20, -36}, {0, -36}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(excitationVoltage.n, groundExcitation.p) annotation(
              Line(points = {{-30, -40}, {-30, -50}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(voltageQuasiRMSSensor.plug_n, smee.plug_sn) annotation(
              Line(points = {{0, -10}, {4, -10}, {4, -20}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(voltageQuasiRMSSensor.plug_p, smee.plug_sp) annotation(
              Line(points = {{0, 10}, {16, 10}, {16, -20}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(terminalBox.plugSupply, currentQuasiRMSSensor.plug_n) annotation(
              Line(points = {{10, -18}, {10, 20}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(smee.flange, speed.flange) annotation(
              Line(points = {{20, -30}, {30, -30}}, color = {0, 0, 0}, smooth = Smooth.None));
            connect(speed.flange, speedSensor.flange) annotation(
              Line(points = {{30, -30}, {30, -40}}, color = {0, 0, 0}, smooth = Smooth.None));
            connect(speedRamp.y, speed.w_ref) annotation(
              Line(points = {{59, -30}, {52, -30}}, color = {0, 0, 127}, smooth = Smooth.None));
            connect(setPointGain.y, voltageController.u_s) annotation(
              Line(points = {{-71, -80}, {-80, -80}, {-80, -30}, {-72, -30}}, color = {0, 0, 127}, smooth = Smooth.None));
            connect(speedSensor.w, setPointGain.u) annotation(
              Line(points = {{30, -61}, {30, -80}, {-48, -80}}, color = {0, 0, 127}, smooth = Smooth.None));
            connect(voltageQuasiRMSSensor.V, voltageController.u_m) annotation(
              Line(points = {{-11, 0}, {-60, 0}, {-60, -18}}, color = {0, 0, 127}, smooth = Smooth.None));
            connect(voltageController.y, excitationVoltage.v) annotation(
              Line(points = {{-49, -30}, {-37, -30}}, color = {0, 0, 127}, smooth = Smooth.None));
            connect(loadInductor.plug_p, loadResistor.plug_n) annotation(
              Line(points = {{-60, 50}, {-50, 50}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(loadResistor.plug_p, switch.plug_n) annotation(
              Line(points = {{-30, 50}, {-20, 50}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(switch.plug_p, currentQuasiRMSSensor.plug_p) annotation(
              Line(points = {{0, 50}, {10, 50}, {10, 40}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(star.plug_p, loadInductor.plug_n) annotation(
              Line(points = {{-90, 40}, {-90, 50}, {-80, 50}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(loadControl.y, switch.control[1]) annotation(
              Line(points = {{-19, 80}, {-10, 80}, {-10, 57}}, color = {255, 0, 255}, smooth = Smooth.None));
            connect(loadControl.y, switch.control[2]) annotation(
              Line(points = {{-19, 80}, {-10, 80}, {-10, 57}}, color = {255, 0, 255}, smooth = Smooth.None));
            connect(loadControl.y, switch.control[3]) annotation(
              Line(points = {{-19, 80}, {-10, 80}, {-10, 57}}, color = {255, 0, 255}, smooth = Smooth.None));
            connect(star.pin_n, ground.p) annotation(
              Line(points = {{-90, 20}, {-90, 10}}, color = {0, 0, 255}, smooth = Smooth.None));
            annotation(
              experiment(StopTime = 10, Interval = 0.001),
              Documentation(info = "<html>
<b>Test example: Electrical excited synchronous induction machine with voltage controller</b><br>
An electrically excited synchronous generator is started with a speed ramp, then driven with constant speed.
Voltage is controlled, the set point depends on speed. After start-up the generator is loaded, the load is rejected.
Simulate for 10 seconds and plot:
<ul>
<li>voltageQuasiRMSSensor.V</li>
<li>smee.tauElectrical</li>
<li>smee.ie</li>
</ul>
Default machine parameters of model <i>SM_ElectricalExcited</i> are used.
One could try to optimize the controller parameters.
</html>"));
          end SMEE_LoadDump;

          model SMEE_Rectifier "Test example: ElectricalExcitedSynchronousInductionMachine with rectifier"
            extends Modelica.Icons.Example;
            import Modelica.Constants.pi;
            constant Integer m = 3 "Number of phases";
            parameter Modelica.SIunits.AngularVelocity wNominal = 2 * pi * smeeData.fsNominal / smee.p "Nominal speed";
            parameter Modelica.SIunits.Voltage VDC0 = sqrt(2 * 3) * smeeData.VsNominal "No-load DC voltage";
            parameter Modelica.SIunits.Resistance RLoad = VDC0 ^ 2 / smeeData.SNominal "Load resistance";
            parameter Modelica.SIunits.Voltage Ve0 = smee.IeOpenCircuit * Machines.Thermal.convertResistance(smee.Re, smee.TeRef, smee.alpha20e, smee.TeOperational) "No load excitation voltage";
            parameter Real k = 2 * Ve0 / smeeData.VsNominal "Voltage controller: gain";
            parameter Modelica.SIunits.Time Ti = smeeData.Td0Transient / 2 "Voltage controller: integral time constant";
            Machines.BasicMachines.SynchronousInductionMachines.SM_ElectricalExcited smee(fsNominal = smeeData.fsNominal, Rs = smeeData.Rs, TsRef = smeeData.TsRef, Lssigma = smeeData.Lssigma, Lmd = smeeData.Lmd, Lmq = smeeData.Lmq, Lrsigmad = smeeData.Lrsigmad, Lrsigmaq = smeeData.Lrsigmaq, Rrd = smeeData.Rrd, Rrq = smeeData.Rrq, TrRef = smeeData.TrRef, VsNominal = smeeData.VsNominal, IeOpenCircuit = smeeData.IeOpenCircuit, Re = smeeData.Re, TeRef = smeeData.TeRef, sigmae = smeeData.sigmae, useDamperCage = true, p = 2, Jr = 0.29, Js = 0.29, statorCoreParameters(VRef = 100), strayLoadParameters(IRef = 100), brushParameters(ILinear = 0.01), TsOperational = 293.15, alpha20s = smeeData.alpha20s, ir(each fixed = true), TrOperational = 293.15, alpha20r = smeeData.alpha20r, alpha20e = smeeData.alpha20e, TeOperational = 293.15) annotation(
              Placement(transformation(extent = {{0, -40}, {20, -20}}, rotation = 0)));
            parameter Machines.Utilities.SynchronousMachineData smeeData(SNominal = 30e3, VsNominal = 100, fsNominal = 50, IeOpenCircuit = 10, x0 = 0.1, xd = 1.6, xq = 1.6, xdTransient = 0.1375, xdSubtransient = 0.121428571, xqSubtransient = 0.148387097, Ta = 0.014171268, Td0Transient = 0.261177343, Td0Subtransient = 0.006963029, Tq0Subtransient = 0.123345081, TsSpecification = 293.15, TsRef = 293.15, alpha20s(displayUnit = "1/K") = Modelica.Electrical.Machines.Thermal.Constants.alpha20Zero, TrSpecification = 293.15, TrRef = 293.15, alpha20r(displayUnit = "1/K") = Modelica.Electrical.Machines.Thermal.Constants.alpha20Zero, TeSpecification = 293.15, TeRef = 293.15, alpha20e(displayUnit = "1/K") = Modelica.Electrical.Machines.Thermal.Constants.alpha20Zero) annotation(
              Placement(transformation(extent = {{0, -70}, {20, -50}})));
            Machines.Utilities.TerminalBox terminalBox(terminalConnection = "Y") annotation(
              Placement(transformation(extent = {{0, -20}, {20, 0}}, rotation = 0)));
            Modelica.Electrical.Analog.Basic.Ground ground annotation(
              Placement(transformation(origin = {-20, 60}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
            Modelica.Mechanics.Rotational.Sources.Speed speed annotation(
              Placement(transformation(extent = {{50, -40}, {30, -20}})));
            Blocks.Sources.Constant constantSpeed(k = wNominal) annotation(
              Placement(transformation(extent = {{80, -40}, {60, -20}})));
            Modelica.Mechanics.Rotational.Sensors.SpeedSensor speedSensor annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 270, origin = {30, -50})));
            Modelica.Blocks.Math.Gain setPointGain(k = smeeData.VsNominal / wNominal / unitMagneticFlux) annotation(
              Placement(transformation(extent = {{-50, -90}, {-70, -70}})));
            Modelica.Blocks.Continuous.LimPID voltageController(controllerType = Modelica.Blocks.Types.SimpleController.PI, k = k, Ti = Ti, yMax = 2.5 * Ve0, yMin = 0, initType = Modelica.Blocks.Types.InitPID.InitialState, Td = 0.001) annotation(
              Placement(transformation(extent = {{-70, -20}, {-50, -40}})));
            Modelica.Electrical.Analog.Sources.SignalVoltage excitationVoltage annotation(
              Placement(transformation(extent = {{10, -10}, {-10, 10}}, rotation = 90, origin = {-30, -30})));
            Modelica.Electrical.Analog.Basic.Ground groundExcitation annotation(
              Placement(transformation(origin = {-30, -60}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
            Modelica.Electrical.MultiPhase.Basic.Star star1(m = m) annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 180, origin = {-20, 90})));
            Modelica.Electrical.MultiPhase.Ideal.IdealDiode idealDiode1(m = m, Ron = fill(1E-5, m), Goff = fill(1E-5, m), Vknee = fill(0, m)) annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 90, origin = {0, 80})));
            Modelica.Electrical.MultiPhase.Ideal.IdealDiode idealDiode2(m = m, Ron = fill(1E-5, m), Goff = fill(1E-5, m), Vknee = fill(0, m)) annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 90, origin = {0, 40})));
            Modelica.Electrical.MultiPhase.Basic.Star star2(m = m) annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 180, origin = {-20, 30})));
            Modelica.Electrical.Analog.Basic.Capacitor capacitor1(C = 2 * 10E-6, v(fixed = true, start = 0)) annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 270, origin = {-40, 80})));
            Modelica.Electrical.Analog.Basic.Capacitor capacitor2(C = 2 * 10E-6, v(fixed = true, start = 0)) annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 270, origin = {-40, 40})));
            Modelica.Electrical.Analog.Basic.Resistor resistor(R = RLoad) annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 270, origin = {-50, 60})));
            Modelica.Electrical.Analog.Sensors.VoltageSensor voltageSensor annotation(
              Placement(transformation(extent = {{-10, 10}, {10, -10}}, rotation = 270, origin = {-80, 60})));
            Modelica.Blocks.Continuous.Filter filter(analogFilter = Modelica.Blocks.Types.AnalogFilter.CriticalDamping, filterType = Modelica.Blocks.Types.FilterType.LowPass, order = 2, f_cut = 20, gain = 1 / sqrt(2 * 3), normalized = true) annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 270, origin = {-60, 0})));
          protected
            constant Modelica.SIunits.MagneticFlux unitMagneticFlux = 1 annotation(
              HideResult = true);
          initial equation
            smee.is[1:2] = zeros(2);
//conditional damper cage currents are defined as fixed start values
            smee.ie = 0;
          equation
            connect(terminalBox.plug_sn, smee.plug_sn) annotation(
              Line(points = {{4, -20}, {4, -20}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(terminalBox.plug_sp, smee.plug_sp) annotation(
              Line(points = {{16, -20}, {16, -20}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(excitationVoltage.p, smee.pin_ep) annotation(
              Line(points = {{-30, -20}, {-20, -20}, {-20, -24}, {0, -24}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(excitationVoltage.n, smee.pin_en) annotation(
              Line(points = {{-30, -40}, {-20, -40}, {-20, -36}, {0, -36}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(excitationVoltage.n, groundExcitation.p) annotation(
              Line(points = {{-30, -40}, {-30, -50}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(smee.flange, speed.flange) annotation(
              Line(points = {{20, -30}, {30, -30}}, color = {0, 0, 0}, smooth = Smooth.None));
            connect(speed.flange, speedSensor.flange) annotation(
              Line(points = {{30, -30}, {30, -40}}, color = {0, 0, 0}, smooth = Smooth.None));
            connect(constantSpeed.y, speed.w_ref) annotation(
              Line(points = {{59, -30}, {52, -30}}, color = {0, 0, 127}, smooth = Smooth.None));
            connect(setPointGain.y, voltageController.u_s) annotation(
              Line(points = {{-71, -80}, {-80, -80}, {-80, -30}, {-72, -30}}, color = {0, 0, 127}, smooth = Smooth.None));
            connect(speedSensor.w, setPointGain.u) annotation(
              Line(points = {{30, -61}, {30, -80}, {-48, -80}}, color = {0, 0, 127}, smooth = Smooth.None));
            connect(voltageController.y, excitationVoltage.v) annotation(
              Line(points = {{-49, -30}, {-37, -30}}, color = {0, 0, 127}, smooth = Smooth.None));
            connect(idealDiode1.plug_p, idealDiode2.plug_n) annotation(
              Line(points = {{0, 70}, {0, 50}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(idealDiode2.plug_p, star2.plug_p) annotation(
              Line(points = {{0, 30}, {-10, 30}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(idealDiode1.plug_n, star1.plug_p) annotation(
              Line(points = {{0, 90}, {-10, 90}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(capacitor2.n, star2.pin_n) annotation(
              Line(points = {{-40, 30}, {-30, 30}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(capacitor1.p, star1.pin_n) annotation(
              Line(points = {{-40, 90}, {-30, 90}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(capacitor1.n, capacitor2.p) annotation(
              Line(points = {{-40, 70}, {-40, 50}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(capacitor1.n, ground.p) annotation(
              Line(points = {{-40, 70}, {-40, 60}, {-30, 60}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(filter.y, voltageController.u_m) annotation(
              Line(points = {{-60, -11}, {-60, -18}}, color = {0, 0, 127}, smooth = Smooth.None));
            connect(voltageSensor.v, filter.u) annotation(
              Line(points = {{-70, 60}, {-60, 60}, {-60, 12}}, color = {0, 0, 127}, smooth = Smooth.None));
            connect(terminalBox.plugSupply, idealDiode2.plug_n) annotation(
              Line(points = {{10, -18}, {10, 60}, {0, 60}, {0, 50}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(resistor.p, capacitor1.p) annotation(
              Line(points = {{-50, 70}, {-50, 90}, {-40, 90}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(resistor.n, capacitor2.n) annotation(
              Line(points = {{-50, 50}, {-50, 30}, {-40, 30}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(voltageSensor.n, capacitor2.n) annotation(
              Line(points = {{-80, 50}, {-80, 30}, {-40, 30}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(voltageSensor.p, capacitor1.p) annotation(
              Line(points = {{-80, 70}, {-80, 90}, {-40, 90}}, color = {0, 0, 255}, smooth = Smooth.None));
            annotation(
              experiment(StopTime = 1.1, Interval = 0.0001),
              Documentation(info = "<html>
<p><b>Test example: Electrical excited synchronous induction machine with voltage controller</b></p><p>An electrically excited synchronous generator is driven with constant speed. Voltage is controlled, the set point depends on speed. The generator is loaded with a rectifier.</p>
<p>Default machine parameters of model <i>SM_ElectricalExcited</i> are used. </p>
</html>"));
          end SMEE_Rectifier;
          annotation(
            Documentation(info = "<HTML>
This package contains test examples of synchronous induction machines.
</HTML>"));
        end SynchronousInductionMachines;

        package DCMachines "Test examples of DC machines"
          extends Modelica.Icons.ExamplesPackage;

          model DCPM_Start "Test example: DC with permanent magnet starting with voltage ramp"
            extends Modelica.Icons.Example;
            parameter Modelica.SIunits.Voltage Va = 100 "Actual armature voltage";
            parameter Modelica.SIunits.Time tStart = 0.2 "Start of armature voltage ramp";
            parameter Modelica.SIunits.Time tRamp = 0.8 "Armature voltage ramp";
            parameter Modelica.SIunits.Torque TLoad = 63.66 "Nominal load torque";
            parameter Modelica.SIunits.Time tStep = 1.5 "Time of load torque step";
            parameter Modelica.SIunits.Inertia JLoad = 0.15 "Load's moment of inertia";
            Machines.BasicMachines.DCMachines.DC_PermanentMagnet dcpm(VaNominal = dcpmData.VaNominal, IaNominal = dcpmData.IaNominal, wNominal = dcpmData.wNominal, TaNominal = dcpmData.TaNominal, Ra = dcpmData.Ra, TaRef = dcpmData.TaRef, La = dcpmData.La, Jr = dcpmData.Jr, useSupport = false, Js = dcpmData.Js, frictionParameters = dcpmData.frictionParameters, coreParameters = dcpmData.coreParameters, strayLoadParameters = dcpmData.strayLoadParameters, brushParameters = dcpmData.brushParameters, TaOperational = 293.15, alpha20a = dcpmData.alpha20a, phiMechanical(fixed = true), wMechanical(fixed = true), ia(fixed = true)) annotation(
              Placement(transformation(extent = {{-20, -50}, {0, -30}}, rotation = 0)));
            Modelica.Blocks.Sources.Ramp ramp(duration = tRamp, height = Va, startTime = tStart) annotation(
              Placement(transformation(extent = {{-80, 60}, {-60, 80}}, rotation = 0)));
            Modelica.Electrical.Analog.Sources.SignalVoltage signalVoltage annotation(
              Placement(transformation(extent = {{0, 30}, {-20, 50}}, rotation = 0)));
            Modelica.Electrical.Analog.Basic.Ground ground annotation(
              Placement(transformation(origin = {-70, 40}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
            Modelica.Mechanics.Rotational.Components.Inertia loadInertia(J = JLoad) annotation(
              Placement(transformation(extent = {{40, -50}, {60, -30}}, rotation = 0)));
            Modelica.Mechanics.Rotational.Sources.TorqueStep loadTorqueStep(startTime = tStep, stepTorque = -TLoad, useSupport = false, offsetTorque = 0) annotation(
              Placement(transformation(extent = {{90, -50}, {70, -30}}, rotation = 0)));
            parameter Utilities.ParameterRecords.DcPermanentMagnetData dcpmData annotation(
              Placement(transformation(extent = {{-20, -80}, {0, -60}})));
          equation
            connect(ramp.y, signalVoltage.v) annotation(
              Line(points = {{-59, 70}, {-10, 70}, {-10, 47}}, color = {0, 0, 255}));
            connect(signalVoltage.p, dcpm.pin_ap) annotation(
              Line(points = {{0, 40}, {0, -20}, {-4, -20}, {-4, -30}}, color = {0, 0, 255}));
            connect(signalVoltage.n, ground.p) annotation(
              Line(points = {{-20, 40}, {-60, 40}}, color = {0, 0, 255}));
            connect(dcpm.pin_an, signalVoltage.n) annotation(
              Line(points = {{-16, -30}, {-16, -20}, {-20, -20}, {-20, 40}}, color = {0, 0, 255}));
            connect(loadInertia.flange_b, loadTorqueStep.flange) annotation(
              Line(points = {{60, -40}, {70, -40}}, color = {0, 0, 0}));
            connect(dcpm.flange, loadInertia.flange_a) annotation(
              Line(points = {{0, -40}, {40, -40}}, color = {0, 0, 0}, smooth = Smooth.None));
            annotation(
              experiment(StopTime = 2.0, Interval = 0.001),
              Documentation(info = "<HTML>
<b>Test example: Permanent magnet DC machine started with an armature voltage ramp</b><br>
A voltage ramp is applied to the armature, causing the DC machine to start,
and accelerating inertias.<br>At time tStep a load step is applied.<br>
Simulate for 2 seconds and plot (versus time):
<ul>
<li>dcpm.ia: armature current</li>
<li>dcpm.wMechanical: motor's speed</li>
<li>dcpm.tauElectrical: motor's torque</li>
</ul>
Default machine parameters of model <i>DC_PermanentMagnet</i> are used.
</HTML>"));
          end DCPM_Start;

          model DCEE_Start "Test example: DC with electrical excitation starting with voltage ramp"
            extends Modelica.Icons.Example;
            parameter Modelica.SIunits.Voltage Va = 100 "Actual armature voltage";
            parameter Modelica.SIunits.Time tStart = 0.2 "Start of armature voltage ramp";
            parameter Modelica.SIunits.Time tRamp = 0.8 "Armature voltage ramp";
            parameter Modelica.SIunits.Voltage Ve = 100 "Actual excitation voltage";
            parameter Modelica.SIunits.Torque TLoad = 63.66 "Nominal load torque";
            parameter Modelica.SIunits.Time tStep = 1.5 "Time of load torque step";
            parameter Modelica.SIunits.Inertia JLoad = 0.15 "Load's moment of inertia";
            Machines.BasicMachines.DCMachines.DC_ElectricalExcited dcee(VaNominal = dceeData.VaNominal, IaNominal = dceeData.IaNominal, wNominal = dceeData.wNominal, TaNominal = dceeData.TaNominal, Ra = dceeData.Ra, TaRef = dceeData.TaRef, La = dceeData.La, Jr = dceeData.Jr, useSupport = false, Js = dceeData.Js, frictionParameters = dceeData.frictionParameters, coreParameters = dceeData.coreParameters, strayLoadParameters = dceeData.strayLoadParameters, brushParameters = dceeData.brushParameters, IeNominal = dceeData.IeNominal, Re = dceeData.Re, TeRef = dceeData.TeRef, Le = dceeData.Le, sigmae = dceeData.sigmae, TaOperational = 293.15, alpha20a = dceeData.alpha20a, phiMechanical(fixed = true), wMechanical(fixed = true), ia(fixed = true), alpha20e = dceeData.alpha20e, TeOperational = 293.15, ie(fixed = true)) annotation(
              Placement(transformation(extent = {{-20, -50}, {0, -30}}, rotation = 0)));
            Modelica.Blocks.Sources.Ramp ramp(duration = tRamp, height = Va, startTime = tStart) annotation(
              Placement(transformation(extent = {{-80, 60}, {-60, 80}}, rotation = 0)));
            Modelica.Electrical.Analog.Sources.SignalVoltage signalVoltage annotation(
              Placement(transformation(extent = {{0, 30}, {-20, 50}}, rotation = 0)));
            Modelica.Electrical.Analog.Basic.Ground ground annotation(
              Placement(transformation(origin = {-70, 40}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
            Modelica.Electrical.Analog.Sources.ConstantVoltage constantVoltage(V = Ve) annotation(
              Placement(transformation(origin = {-40, -40}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
            Modelica.Electrical.Analog.Basic.Ground groundExcitation annotation(
              Placement(transformation(origin = {-70, -50}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
            Modelica.Mechanics.Rotational.Components.Inertia loadInertia(J = JLoad) annotation(
              Placement(transformation(extent = {{40, -50}, {60, -30}}, rotation = 0)));
            Modelica.Mechanics.Rotational.Sources.TorqueStep loadTorqueStep(startTime = tStep, stepTorque = -TLoad, useSupport = false, offsetTorque = 0) annotation(
              Placement(transformation(extent = {{90, -50}, {70, -30}}, rotation = 0)));
            parameter Utilities.ParameterRecords.DcElectricalExcitedData dceeData annotation(
              Placement(transformation(extent = {{-20, -80}, {0, -60}})));
          equation
            connect(ramp.y, signalVoltage.v) annotation(
              Line(points = {{-59, 70}, {-10, 70}, {-10, 47}}, color = {0, 0, 255}));
            connect(signalVoltage.p, dcee.pin_ap) annotation(
              Line(points = {{0, 40}, {0, -20}, {-4, -20}, {-4, -30}}, color = {0, 0, 255}));
            connect(signalVoltage.n, ground.p) annotation(
              Line(points = {{-20, 40}, {-60, 40}}, color = {0, 0, 255}));
            connect(dcee.pin_an, ground.p) annotation(
              Line(points = {{-16, -30}, {-16, -20}, {-20, -20}, {-20, 40}, {-60, 40}}, color = {0, 0, 255}));
            connect(constantVoltage.n, groundExcitation.p) annotation(
              Line(points = {{-40, -50}, {-60, -50}}, color = {0, 0, 255}));
            connect(dcee.pin_ep, constantVoltage.p) annotation(
              Line(points = {{-20, -34}, {-30, -34}, {-30, -30}, {-40, -30}}, color = {0, 0, 255}));
            connect(dcee.pin_en, constantVoltage.n) annotation(
              Line(points = {{-20, -46}, {-30, -46}, {-30, -50}, {-40, -50}}, color = {0, 0, 255}));
            connect(loadInertia.flange_b, loadTorqueStep.flange) annotation(
              Line(points = {{60, -40}, {70, -40}}, color = {0, 0, 0}));
            connect(dcee.flange, loadInertia.flange_a) annotation(
              Line(points = {{0, -40}, {40, -40}}, color = {0, 0, 0}, smooth = Smooth.None));
            annotation(
              experiment(StopTime = 2.0, Interval = 0.001),
              Documentation(info = "<HTML>
<b>Test example: Electrically separate excited DC machine started with an armature voltage ramp</b><br>
A voltage ramp is applied to the armature, causing the DC machine to start,
and accelerating inertias.<br>At time tStep a load step is applied.<br>
Simulate for 2 seconds and plot (versus time):
<ul>
<li>dcee.ia: armature current</li>
<li>dcee.wMechanical: motor's speed</li>
<li>dcee.tauElectrical: motor's torque</li>
<li>dcee.ie: excitation current</li>
</ul>
Default machine parameters of model <i>DC_ElectricalExcited</i> are used.
</HTML>"));
          end DCEE_Start;

          model DCSE_Start "Test example: DC with serial excitation starting with voltage ramp"
            extends Modelica.Icons.Example;
            parameter Modelica.SIunits.Voltage Va = 100 "Actual armature voltage";
            parameter Modelica.SIunits.Time tStart = 0.1 "Start of resistance ramp";
            parameter Modelica.SIunits.Time tRamp = 0.9 "Resistance ramp";
            parameter Modelica.SIunits.Torque TLoad = 63.66 "Nominal load torque";
            parameter Modelica.SIunits.AngularVelocity wLoad(displayUnit = "1/min") = 1410 * 2 * Modelica.Constants.pi / 60 "Nominal load speed";
            parameter Modelica.SIunits.Inertia JLoad = 0.15 "Load's moment of inertia";
            Machines.BasicMachines.DCMachines.DC_SeriesExcited dcse(VaNominal = dcseData.VaNominal, IaNominal = dcseData.IaNominal, wNominal = dcseData.wNominal, TaNominal = dcseData.TaNominal, TeNominal = dcseData.TeNominal, Ra = dcseData.Ra, TaRef = dcseData.TaRef, La = dcseData.La, Jr = dcseData.Jr, useSupport = false, Js = dcseData.Js, frictionParameters = dcseData.frictionParameters, coreParameters = dcseData.coreParameters, strayLoadParameters = dcseData.strayLoadParameters, brushParameters = dcseData.brushParameters, Re = dcseData.Re, TeRef = dcseData.TeRef, Le = dcseData.Le, sigmae = dcseData.sigmae, TaOperational = 293.15, alpha20a = dcseData.alpha20a, phiMechanical(fixed = true), wMechanical(fixed = true), ia(fixed = true), alpha20e = dcseData.alpha20e, TeOperational = 293.15) annotation(
              Placement(transformation(extent = {{-20, -50}, {0, -30}}, rotation = 0)));
            Modelica.Blocks.Sources.Ramp ramp(duration = tRamp, startTime = tStart, height = -1, offset = 1) annotation(
              Placement(transformation(extent = {{60, 0}, {40, 20}}, rotation = 0)));
            Modelica.Electrical.Analog.Sources.ConstantVoltage constantVoltage(V = Va) annotation(
              Placement(transformation(extent = {{0, 50}, {-20, 30}}, rotation = 0)));
            Modelica.Electrical.Analog.Basic.Ground ground annotation(
              Placement(transformation(origin = {-70, 40}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
            Modelica.Mechanics.Rotational.Components.Inertia loadInertia(J = JLoad) annotation(
              Placement(transformation(extent = {{40, -50}, {60, -30}}, rotation = 0)));
            Modelica.Mechanics.Rotational.Sources.QuadraticSpeedDependentTorque quadraticLoadTorque(w_nominal = wLoad, TorqueDirection = false, tau_nominal = -TLoad, useSupport = false) annotation(
              Placement(transformation(extent = {{90, -50}, {70, -30}}, rotation = 0)));
            Modelica.Electrical.Analog.Basic.VariableResistor variableResistor annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 270, origin = {0, 10})));
            parameter Utilities.ParameterRecords.DcSeriesExcitedData dcseData annotation(
              Placement(transformation(extent = {{-20, -80}, {0, -60}})));
          equation
            connect(constantVoltage.n, ground.p) annotation(
              Line(points = {{-20, 40}, {-60, 40}}, color = {0, 0, 255}));
            connect(loadInertia.flange_b, quadraticLoadTorque.flange) annotation(
              Line(points = {{60, -40}, {70, -40}}, color = {0, 0, 0}));
            connect(dcse.pin_an, dcse.pin_ep) annotation(
              Line(points = {{-16, -30}, {-20, -30}, {-20, -34}}, color = {0, 0, 255}));
            connect(dcse.pin_en, constantVoltage.n) annotation(
              Line(points = {{-20, -46}, {-30, -46}, {-30, -20}, {-20, -20}, {-20, 40}}, color = {0, 0, 255}));
            connect(dcse.flange, loadInertia.flange_a) annotation(
              Line(points = {{0, -40}, {40, -40}}, color = {0, 0, 0}, smooth = Smooth.None));
            connect(constantVoltage.p, variableResistor.p) annotation(
              Line(points = {{0, 40}, {0, 20}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(variableResistor.n, dcse.pin_ap) annotation(
              Line(points = {{0, 0}, {0, -20}, {-4, -20}, {-4, -30}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(ramp.y, variableResistor.R) annotation(
              Line(points = {{39, 10}, {11, 10}}, color = {0, 0, 127}, smooth = Smooth.None));
            annotation(
              experiment(StopTime = 2.0, Interval = 0.001),
              Documentation(info = "<HTML>
<b>Test example: Series excited DC machine started with a series resistor</b><br>
At constant source voltage, a series resistor limiting the armature current, is reduced according to a ramp, causing the DC machine to start,
and accelerating inertias against load torque quadratic dependent on speed, finally reaching nominal speed.<br>
Simulate for 2 seconds and plot (versus time):
<ul>
<li>dcse.ia: armature current</li>
<li>dcse.wMechanical: motor's speed</li>
<li>dcse.tauElectrical: motor's torque</li>
</ul>
Default machine parameters of model <i>DC_SeriesExcited</i> are used.
</HTML>"));
          end DCSE_Start;

          model DCSE_SinglePhase "Test example: DC with serial excitation starting with voltage ramp"
            extends Modelica.Icons.Example;
            parameter Modelica.SIunits.Voltage Va = 100 "Actual armature voltage RMS";
            parameter Modelica.SIunits.Time tStart = 0.1 "Start of resistance ramp";
            parameter Modelica.SIunits.Time tRamp = 0.9 "Resistance ramp";
            parameter Modelica.SIunits.Torque TLoad = 63.66 "Nominal load torque";
            parameter Modelica.SIunits.AngularVelocity wLoad(displayUnit = "1/min") = 1410 * 2 * Modelica.Constants.pi / 60 "Nominal load speed";
            parameter Modelica.SIunits.Inertia JLoad = 0.15 "Load's moment of inertia";
            Machines.BasicMachines.DCMachines.DC_SeriesExcited dcse(VaNominal = dcseData.VaNominal, IaNominal = dcseData.IaNominal, wNominal = dcseData.wNominal, TaNominal = dcseData.TaNominal, TeNominal = dcseData.TeNominal, Ra = dcseData.Ra, TaRef = dcseData.TaRef, La = dcseData.La, Jr = dcseData.Jr, useSupport = false, Js = dcseData.Js, frictionParameters = dcseData.frictionParameters, coreParameters = dcseData.coreParameters, strayLoadParameters = dcseData.strayLoadParameters, brushParameters = dcseData.brushParameters, Re = dcseData.Re, TeRef = dcseData.TeRef, Le = dcseData.Le, sigmae = dcseData.sigmae, TaOperational = 293.15, alpha20a = dcseData.alpha20a, phiMechanical(fixed = true), wMechanical(fixed = true), ia(fixed = true), alpha20e = dcseData.alpha20e, TeOperational = 293.15) annotation(
              Placement(transformation(extent = {{-20, -50}, {0, -30}}, rotation = 0)));
            Modelica.Blocks.Sources.Ramp ramp(duration = tRamp, startTime = tStart, height = -1, offset = 1) annotation(
              Placement(transformation(extent = {{60, 0}, {40, 20}}, rotation = 0)));
            Modelica.Electrical.Analog.Sources.SineVoltage constantVoltage(V = sqrt(2) * Va, freqHz = 50) annotation(
              Placement(transformation(extent = {{0, 50}, {-20, 30}}, rotation = 0)));
            Modelica.Electrical.Analog.Basic.Ground ground annotation(
              Placement(transformation(origin = {-70, 40}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
            Modelica.Mechanics.Rotational.Components.Inertia loadInertia(J = JLoad) annotation(
              Placement(transformation(extent = {{40, -50}, {60, -30}}, rotation = 0)));
            Modelica.Mechanics.Rotational.Sources.QuadraticSpeedDependentTorque quadraticLoadTorque(w_nominal = wLoad, TorqueDirection = false, tau_nominal = -TLoad, useSupport = false) annotation(
              Placement(transformation(extent = {{90, -50}, {70, -30}}, rotation = 0)));
            Modelica.Electrical.Analog.Basic.VariableResistor variableResistor annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 270, origin = {0, 10})));
            parameter Utilities.ParameterRecords.DcSeriesExcitedData dcseData annotation(
              Placement(transformation(extent = {{-20, -80}, {0, -60}})));
          equation
            connect(constantVoltage.n, ground.p) annotation(
              Line(points = {{-20, 40}, {-60, 40}}, color = {0, 0, 255}));
            connect(loadInertia.flange_b, quadraticLoadTorque.flange) annotation(
              Line(points = {{60, -40}, {70, -40}}, color = {0, 0, 0}));
            connect(dcse.pin_an, dcse.pin_ep) annotation(
              Line(points = {{-16, -30}, {-20, -30}, {-20, -34}}, color = {0, 0, 255}));
            connect(dcse.pin_en, constantVoltage.n) annotation(
              Line(points = {{-20, -46}, {-30, -46}, {-30, -20}, {-20, -20}, {-20, 40}}, color = {0, 0, 255}));
            connect(dcse.flange, loadInertia.flange_a) annotation(
              Line(points = {{0, -40}, {40, -40}}, color = {0, 0, 0}, smooth = Smooth.None));
            connect(constantVoltage.p, variableResistor.p) annotation(
              Line(points = {{0, 40}, {0, 20}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(variableResistor.n, dcse.pin_ap) annotation(
              Line(points = {{0, 0}, {0, -20}, {-4, -20}, {-4, -30}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(ramp.y, variableResistor.R) annotation(
              Line(points = {{39, 10}, {11, 10}}, color = {0, 0, 127}, smooth = Smooth.None));
            annotation(
              experiment(StopTime = 2.0, Interval = 0.0005),
              Documentation(info = "<HTML>
<b>Test example: Series excited DC machine at singlephase AC voltage started with a series resistor</b><br>
At sinusoidal source voltage, a series resistor limiting the armature current, is reduced according to a ramp, causing the DC machine to start,
and accelerating inertias against load torque quadratic dependent on speed, finally reaching nominal speed.<br>
Simulate for 2 seconds and plot (versus time):
<ul>
<li>dcse.ia: armature current</li>
<li>dcse.wMechanical: motor's speed</li>
<li>dcse.tauElectrical: motor's torque</li>
</ul>
Default machine parameters of model <i>DC_SeriesExcited</i> are used.<br>
<b>Note:</b><br>
Since both the field and the armature current are sinusoidal, the waveform of the torque is the square of sine.
Due to the additional inductive voltage drops, output of the motor is lower, compared to the same motor (DCSE_Start) at DC voltage.
</HTML>"));
          end DCSE_SinglePhase;

          /*
            model DC_CompareCharacteristics
              "Test example: Compare torque-speed characteristic of DC motors"
              extends Modelica.Icons.Example;
              parameter Modelica.SIunits.Voltage Va=100 "Actual armature voltage";
              parameter Modelica.SIunits.Voltage Ve=100 "Actual excitation voltage";
              parameter Modelica.SIunits.AngularVelocity w0=Modelica.SIunits.Conversions.from_rpm(1500)
                "No-load speed";
              parameter Modelica.SIunits.Torque TLoad=63.66 "Nominal load torque";
              parameter Modelica.SIunits.Time tStart=0.5 "Start of load torque ramp ramp";
              parameter Modelica.SIunits.Time tRamp=2.0 "Load torque ramp";
              parameter Modelica.SIunits.Inertia JLoad=0.15 "Load's moment of inertia";

              Machines.BasicMachines.DCMachines.DC_ElectricalExcited dcee(wMechanical(start=
                     w0, fixed=true), ie(start=1, fixed=true))
                annotation (Placement(transformation(extent={{-20,-10},{0,10}},
                      rotation=0)));
              Modelica.Electrical.Analog.Sources.ConstantVoltage armatureVoltage(V=Va)
                annotation (Placement(transformation(extent={{10,-10},{-10,10}},
                                                                               rotation=90,
                    origin={-80,70})));
              Modelica.Electrical.Analog.Basic.Ground groundArmature
                annotation (Placement(transformation(
                    origin={-80,40},
                    extent={{-10,-10},{10,10}},
                    rotation=0)));
              Modelica.Electrical.Analog.Sources.ConstantVoltage excitationVoltage(V=Ve)
                annotation (Placement(transformation(
                    origin={-80,-50},
                    extent={{-10,-10},{10,10}},
                    rotation=270)));
              Modelica.Electrical.Analog.Basic.Ground groundExcitation
                annotation (Placement(transformation(
                    origin={-80,-80},
                    extent={{-10,-10},{10,10}},
                    rotation=0)));
              Modelica.Mechanics.Rotational.Components.Inertia loadInertia1(J=JLoad)
                annotation (Placement(transformation(extent={{10,-10},{30,10}},
                      rotation=0)));
              Modelica.Mechanics.Rotational.Sources.Torque loadTorque1(useSupport=false)
                annotation (Placement(transformation(extent={{60,-10},{40,10}}, rotation=0)));
              Modelica.Blocks.Sources.Ramp ramp(
                startTime=tStart,
                height=-TLoad,
                duration=tRamp,
                offset=0) annotation (Placement(transformation(extent={{100,-10},{80,10}})));
            equation
              connect(loadInertia1.flange_b, loadTorque1.flange)
                annotation (Line(points={{30,0},{30,0},{40,0}},
                                                             color={0,0,0}));
              connect(dcee.flange, loadInertia1.flange_a)
                                                         annotation (Line(
                  points={{0,0},{10,0}},
                  color={0,0,0},
                  smooth=Smooth.None));

              connect(ramp.y, loadTorque1.tau)
                                              annotation (Line(
                  points={{79,0},{62,0}},
                  color={0,0,127},
                  smooth=Smooth.None));
              connect(armatureVoltage.n, groundArmature.p) annotation (Line(
                  points={{-80,60},{-80,50}},
                  color={0,0,255},
                  smooth=Smooth.None));
              connect(excitationVoltage.n, groundExcitation.p) annotation (Line(
                  points={{-80,-60},{-80,-70}},
                  color={0,0,255},
                  smooth=Smooth.None));
              connect(armatureVoltage.p, dcee.pin_ap) annotation (Line(
                  points={{-80,80},{-30,80},{-30,20},{-4,20},{-4,10}},
                  color={0,0,255},
                  smooth=Smooth.None));
              connect(armatureVoltage.n, dcee.pin_an) annotation (Line(
                  points={{-80,60},{-40,60},{-40,10},{-16,10}},
                  color={0,0,255},
                  smooth=Smooth.None));
              connect(excitationVoltage.p, dcee.pin_ep) annotation (Line(
                  points={{-80,-40},{-60,-40},{-60,6},{-20,6}},
                  color={0,0,255},
                  smooth=Smooth.None));
              connect(excitationVoltage.n, dcee.pin_en) annotation (Line(
                  points={{-80,-60},{-50,-60},{-50,-6},{-20,-6}},
                  color={0,0,255},
                  smooth=Smooth.None));
              annotation (
                experiment(StopTime=3, Interval=0.001),
                Documentation(info="<HTML>
          <b>Test example: Compare characteristic of DC motors</b><br>
          The motors are started at no-load speed, then a load ramp is applied.<br>
          Simulate for 3 seconds and plot (versus time):
          <ul>
          <li>dcxx.ia: armature current</li>
          <li>dcxx.wMechanical: motor's speed</li>
          <li>dcxx.tauElectrical: motor's torque</li>
          </ul>
          Default machine parameters are used.
          </HTML>"));
            end DC_CompareCharacteristics;
          */

          model DCPM_Temperature "Test example: Investigate temperature dependency of a DCPM motor"
            extends Modelica.Icons.Example;
            parameter Modelica.SIunits.Voltage Va = 100 "Actual armature voltage";
            parameter Modelica.SIunits.Voltage Ve = 100 "Actual excitation voltage";
            parameter Modelica.SIunits.AngularVelocity w0 = Modelica.SIunits.Conversions.from_rpm(1500) "No-load speed";
            parameter Modelica.SIunits.Torque TLoad = 63.66 "Nominal load torque";
            parameter Modelica.SIunits.Inertia JLoad = 0.15 "Load's moment of inertia";
            Machines.BasicMachines.DCMachines.DC_PermanentMagnet dcpm(wMechanical(start = w0, fixed = true), VaNominal = dcpmData.VaNominal, IaNominal = dcpmData.IaNominal, wNominal = dcpmData.wNominal, TaNominal = dcpmData.TaNominal, Ra = dcpmData.Ra, TaRef = dcpmData.TaRef, La = dcpmData.La, Jr = dcpmData.Jr, useSupport = false, Js = dcpmData.Js, frictionParameters = dcpmData.frictionParameters, coreParameters = dcpmData.coreParameters, strayLoadParameters = dcpmData.strayLoadParameters, brushParameters = dcpmData.brushParameters, phiMechanical(fixed = true), ia(fixed = true), TaOperational = 293.15, alpha20a = dcpmData.alpha20a, useThermalPort = true) annotation(
              Placement(transformation(extent = {{-20, -10}, {0, 10}}, rotation = 0)));
            Modelica.Electrical.Analog.Sources.ConstantVoltage armatureVoltage(V = Va) annotation(
              Placement(transformation(extent = {{10, -10}, {-10, 10}}, rotation = 90, origin = {-80, 70})));
            Modelica.Electrical.Analog.Basic.Ground groundArmature annotation(
              Placement(transformation(origin = {-80, 40}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
            Modelica.Mechanics.Rotational.Components.Inertia loadInertia(J = JLoad) annotation(
              Placement(transformation(extent = {{10, -10}, {30, 10}}, rotation = 0)));
            Modelica.Mechanics.Rotational.Sources.TorqueStep loadTorque(useSupport = false, stepTorque = -TLoad, offsetTorque = 0, startTime = 0.1) annotation(
              Placement(transformation(extent = {{60, -10}, {40, 10}}, rotation = 0)));
            Machines.Thermal.DCMachines.ThermalAmbientDCPM thermalAmbientDCPM(useTemperatureInputs = true, Ta = 293.15, Tpm = 293.15) annotation(
              Placement(transformation(extent = {{-20, -40}, {0, -20}})));
            Modelica.Blocks.Sources.Exponentials exponential(offset = 293.15, outMax = 60, riseTime = 3600, riseTimeConst = 0.5, fallTimeConst = 0.5, startTime = 0.1) annotation(
              Placement(transformation(extent = {{-60, -60}, {-40, -40}})));
            Modelica.Blocks.Sources.Constant const(k = 293.15) annotation(
              Placement(transformation(extent = {{40, -60}, {20, -40}})));
            parameter Utilities.ParameterRecords.DcPermanentMagnetData dcpmData(alpha20a(displayUnit = "1/K") = Modelica.Electrical.Machines.Thermal.Constants.alpha20Copper, TaNominal = 353.15, TaRef = 353.15) annotation(
              Placement(transformation(extent = {{-20, -80}, {0, -60}})));
          equation
            connect(loadInertia.flange_b, loadTorque.flange) annotation(
              Line(points = {{30, 0}, {30, 0}, {40, 0}}, color = {0, 0, 0}));
            connect(dcpm.flange, loadInertia.flange_a) annotation(
              Line(points = {{0, 0}, {10, 0}}, color = {0, 0, 0}, smooth = Smooth.None));
            connect(armatureVoltage.n, groundArmature.p) annotation(
              Line(points = {{-80, 60}, {-80, 50}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(armatureVoltage.p, dcpm.pin_ap) annotation(
              Line(points = {{-80, 80}, {-4, 80}, {-4, 10}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(armatureVoltage.n, dcpm.pin_an) annotation(
              Line(points = {{-80, 60}, {-16, 60}, {-16, 10}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(exponential.y, thermalAmbientDCPM.TArmature) annotation(
              Line(points = {{-39, -50}, {-18, -50}, {-18, -40}}, color = {0, 0, 127}, smooth = Smooth.None));
            connect(const.y, thermalAmbientDCPM.TPermanentMagnet) annotation(
              Line(points = {{19, -50}, {-10, -50}, {-10, -40}}, color = {0, 0, 127}, smooth = Smooth.None));
            connect(dcpm.thermalPort, thermalAmbientDCPM.thermalPort) annotation(
              Line(points = {{-10, -10}, {-10, -20}}, color = {195, 0, 0}, smooth = Smooth.None));
            annotation(
              experiment(StopTime = 3.0, Interval = 0.001),
              Documentation(info = "<HTML>
<b>Test example: Investigate influence of armature temperature on a DCPM motor</b><br>
The motor starts at no-load speed, then a load step is applied.<br>
Beginning with the load step, the armature temperature rises exponentially from 20 degC to 80 degC.<br>
Simulate for 3 seconds and plot (versus time):
<ul>
<li>dcpm.ia: armature current</li>
<li>dcpm.wMechanical: motor's speed</li>
<li>dcpm.tauElectrical: motor's torque</li>
<li>thermalAmbientDCPM.Q_flow_a: motor's armature losses</li>
</ul>
Default machine parameters are used, but:
<ul>
<li>The armature winding material is set to Copper.</li>
<li>Armature reference temperature is set to 80 degC.</li>
<li>Nominal armature temperature is set to 80 degC.</li>
</ul>
So the machine is at the beginning in cold condition, ending in warm condition
(with the same armature resistance as the unmodified machine).
</HTML>"));
          end DCPM_Temperature;

          model DCPM_Cooling "Test example: Cooling of a DCPM motor"
            extends Modelica.Icons.Example;
            parameter Modelica.SIunits.Voltage Va = 100 "Actual armature voltage";
            parameter Modelica.SIunits.Voltage Ve = 100 "Actual excitation voltage";
            parameter Modelica.SIunits.AngularVelocity w0 = Modelica.SIunits.Conversions.from_rpm(1500) "No-load speed";
            parameter Modelica.SIunits.Torque TLoad = 63.66 "Nominal load torque";
            parameter Modelica.SIunits.Inertia JLoad = 0.15 "Load's moment of inertia";
            parameter Modelica.SIunits.Temperature TAmbient = 293.15 "Ambient temperature";
            parameter Modelica.SIunits.HeatCapacity Ca = 20 "Armature's heat capacity";
            parameter Modelica.SIunits.HeatCapacity Cc = 50 "Core's heat capacity";
            final parameter Modelica.SIunits.Power Losses = dcpm.Ra * dcpm.IaNominal ^ 2 "Nominal Losses";
            final parameter Modelica.SIunits.Temperature T0 = 293.15 "Reference temperature 20 degC";
            final parameter Modelica.SIunits.TemperatureDifference dTCoolant = 10 "Coolant's temperature rise";
            final parameter Modelica.SIunits.TemperatureDifference dTArmature = dcpm.TaNominal - T0 - dTCoolant / 2 "Armature's temperature rise over coolant";
            parameter Modelica.SIunits.ThermalConductance G_armature_core = 2 * Losses / dTArmature "Heat conductance armature - core";
            parameter Modelica.SIunits.ThermalConductance G_core_cooling = 2 * Losses / dTArmature "Heat conductance core - cooling";
            parameter Modelica.SIunits.VolumeFlowRate CoolantFlow = 50 "Coolant flow";
            Machines.BasicMachines.DCMachines.DC_PermanentMagnet dcpm(wMechanical(start = w0, fixed = true), VaNominal = dcpmData.VaNominal, IaNominal = dcpmData.IaNominal, wNominal = dcpmData.wNominal, TaNominal = dcpmData.TaNominal, Ra = dcpmData.Ra, TaRef = dcpmData.TaRef, La = dcpmData.La, Jr = dcpmData.Jr, useSupport = false, Js = dcpmData.Js, frictionParameters = dcpmData.frictionParameters, coreParameters = dcpmData.coreParameters, strayLoadParameters = dcpmData.strayLoadParameters, brushParameters = dcpmData.brushParameters, phiMechanical(fixed = true), ia(fixed = true), TaOperational = 293.15, alpha20a = dcpmData.alpha20a, useThermalPort = true) annotation(
              Placement(transformation(extent = {{-20, 20}, {0, 40}}, rotation = 0)));
            Modelica.Electrical.Analog.Sources.ConstantVoltage armatureVoltage(V = Va) annotation(
              Placement(transformation(extent = {{10, -10}, {-10, 10}}, rotation = 90, origin = {-80, 70})));
            Modelica.Electrical.Analog.Basic.Ground groundArmature annotation(
              Placement(transformation(origin = {-80, 40}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
            Modelica.Mechanics.Rotational.Components.Inertia loadInertia(J = JLoad) annotation(
              Placement(transformation(extent = {{10, 20}, {30, 40}}, rotation = 0)));
            Modelica.Mechanics.Rotational.Sources.Torque loadTorque(useSupport = false) annotation(
              Placement(transformation(extent = {{60, 20}, {40, 40}}, rotation = 0)));
            Modelica.Blocks.Sources.Pulse pulse(amplitude = -1.5 * TLoad, offset = 0, period = 1) annotation(
              Placement(transformation(extent = {{90, 20}, {70, 40}})));
            Modelica.Thermal.HeatTransfer.Components.HeatCapacitor armature(C = Ca, T(start = TAmbient, fixed = true)) annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 180, origin = {-50, -40})));
            Modelica.Thermal.HeatTransfer.Components.ThermalConductor armatureCore(G = G_armature_core) annotation(
              Placement(transformation(extent = {{-40, -40}, {-20, -20}})));
            Modelica.Thermal.HeatTransfer.Components.HeatCapacitor core(C = Cc, T(start = TAmbient, fixed = true)) annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 180, origin = {-10, -40})));
            Modelica.Thermal.HeatTransfer.Components.ThermalConductor coreCooling(G = G_core_cooling) annotation(
              Placement(transformation(extent = {{0, -40}, {20, -20}})));
            Modelica.Thermal.FluidHeatFlow.Sources.Ambient inlet(constantAmbientTemperature = TAmbient, constantAmbientPressure = 0) annotation(
              Placement(transformation(extent = {{-10, -80}, {-30, -60}})));
            Modelica.Thermal.FluidHeatFlow.Sources.VolumeFlow volumeFlow(T0 = TAmbient, constantVolumeFlow = CoolantFlow, m = 0) annotation(
              Placement(transformation(extent = {{0, -80}, {20, -60}})));
            Modelica.Thermal.FluidHeatFlow.Components.HeatedPipe cooling(tapT = 0.5, T0 = TAmbient, m = 0, h_g = 0, V_flowLaminar = 0.1, dpLaminar(displayUnit = "Pa") = 0.1, V_flowNominal = 1, dpNominal(displayUnit = "Pa") = 1, T0fixed = false) annotation(
              Placement(transformation(extent = {{30, -60}, {50, -80}})));
            Modelica.Thermal.FluidHeatFlow.Sources.Ambient outlet(constantAmbientTemperature = TAmbient, constantAmbientPressure = 0) annotation(
              Placement(transformation(extent = {{60, -80}, {80, -60}})));
            Modelica.Thermal.HeatTransfer.Sources.FixedTemperature fixedTemperature(T = TAmbient) annotation(
              Placement(transformation(extent = {{42, -10}, {22, 10}})));
          protected
            Machines.Interfaces.DCMachines.ThermalPortDCPM thermalPort annotation(
              Placement(transformation(extent = {{-14, -4}, {-6, 4}})));
          public
            parameter Utilities.ParameterRecords.DcPermanentMagnetData dcpmData(alpha20a(displayUnit = "1/K") = Modelica.Electrical.Machines.Thermal.Constants.alpha20Copper, TaNominal = 353.15, TaRef = 353.15) annotation(
              Placement(transformation(extent = {{-60, 20}, {-40, 40}})));
          equation
            connect(loadInertia.flange_b, loadTorque.flange) annotation(
              Line(points = {{30, 30}, {30, 30}, {40, 30}}, color = {0, 0, 0}));
            connect(dcpm.flange, loadInertia.flange_a) annotation(
              Line(points = {{0, 30}, {10, 30}}, color = {0, 0, 0}, smooth = Smooth.None));
            connect(armatureVoltage.n, groundArmature.p) annotation(
              Line(points = {{-80, 60}, {-80, 50}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(armatureVoltage.p, dcpm.pin_ap) annotation(
              Line(points = {{-80, 80}, {-4, 80}, {-4, 40}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(armatureVoltage.n, dcpm.pin_an) annotation(
              Line(points = {{-80, 60}, {-16, 60}, {-16, 40}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(armature.port, armatureCore.port_a) annotation(
              Line(points = {{-50, -30}, {-40, -30}}, color = {191, 0, 0}, smooth = Smooth.None));
            connect(armatureCore.port_b, core.port) annotation(
              Line(points = {{-20, -30}, {-10, -30}}, color = {191, 0, 0}, smooth = Smooth.None));
            connect(core.port, coreCooling.port_a) annotation(
              Line(points = {{-10, -30}, {0, -30}}, color = {191, 0, 0}, smooth = Smooth.None));
            connect(pulse.y, loadTorque.tau) annotation(
              Line(points = {{69, 30}, {62, 30}}, color = {0, 0, 127}, smooth = Smooth.None));
            connect(coreCooling.port_b, cooling.heatPort) annotation(
              Line(points = {{20, -30}, {40, -30}, {40, -60}}, color = {191, 0, 0}, smooth = Smooth.None));
            connect(cooling.flowPort_b, outlet.flowPort) annotation(
              Line(points = {{50, -70}, {60, -70}}, color = {255, 0, 0}, smooth = Smooth.None));
            connect(inlet.flowPort, volumeFlow.flowPort_a) annotation(
              Line(points = {{-10, -70}, {0, -70}}, color = {255, 0, 0}, smooth = Smooth.None));
            connect(volumeFlow.flowPort_b, cooling.flowPort_a) annotation(
              Line(points = {{20, -70}, {30, -70}}, color = {255, 0, 0}, smooth = Smooth.None));
            connect(dcpm.thermalPort, thermalPort) annotation(
              Line(points = {{-10, 20}, {-10, 0}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(armature.port, thermalPort.heatPortArmature) annotation(
              Line(points = {{-50, -30}, {-50, 0}, {-10, 0}}, color = {191, 0, 0}, smooth = Smooth.None));
            connect(core.port, thermalPort.heatPortCore) annotation(
              Line(points = {{-10, -30}, {-10, 0}}, color = {191, 0, 0}, smooth = Smooth.None));
            connect(fixedTemperature.port, thermalPort.heatPortStrayLoad) annotation(
              Line(points = {{22, 0}, {-10, 0}}, color = {191, 0, 0}, smooth = Smooth.None));
            connect(fixedTemperature.port, thermalPort.heatPortFriction) annotation(
              Line(points = {{22, 0}, {-10, 0}}, color = {191, 0, 0}, smooth = Smooth.None));
            connect(fixedTemperature.port, thermalPort.heatPortBrush) annotation(
              Line(points = {{22, 0}, {-10, 0}}, color = {191, 0, 0}, smooth = Smooth.None));
            connect(fixedTemperature.port, thermalPort.heatPortPermanentMagnet) annotation(
              Line(points = {{22, 0}, {-10, 0}}, color = {191, 0, 0}, smooth = Smooth.None));
            annotation(
              experiment(StopTime = 25, Interval = 0.001),
              Documentation(info = "<HTML>
<b>Test example: Demonstrate cooling of a DCPM motor</b><br>
The motor starts at no-load speed, then load pulses are applied.<br>
The cooling circuit consists of armature's thermal capacitance,
a thermal conductance between armature and core, core's thermal capacitance and
a thermal conductance between core and coolant.
The coolant flow circuit consists of inlet, volume flow, a pipe connected to the core and the outlet.<br>
<b>Please note:</b>
<ul>
<li>All unused heat ports of the thermal port (i.e., without loss sources in the machine: brush, stray, friction, permanent magnet) have to be connected to a constant temperature source.</li>
<li>The thermal capacitances (i.e., time constants) are unusual small to provide short simulation time!</li>
<li>The coolant is a theoretical coolant with specific mass = 1 kg/m3 and cp = 1 J/kg.K.</li>
<li>The thermal conductances as well as the coolant flow are parametrized such way, that:</li>
</ul>
<ol>
<li>the total coolant's temperature rise is 10 K (over coolant inlet)</li>
<li>the core's temperature rise is 27.5 K (over coolant's average temperature between inlet and outlet)</li>
<li>the armature's temperature rise is 55 K (over coolant's average temperature between inlet and outlet)</li>
</ol>
Simulate for 25 seconds and plot (versus time):
<ul>
<li>armature.T: armature temperature</li>
<li>core.T: core temperature</li>
<li>cooling.T: coolant temperature at outlet</li>
</ul>
Therefore the armature temperature would reach nominal armature temperature at constant nominal load.<br>
Default machine parameters are used, but:
<ul>
<li>The armature winding material is set to Copper.</li>
<li>Armature reference temperature is set to 80 degC.</li>
<li>Nominal armature temperature is set to 80 degC.</li>
</ul>
</HTML>"));
          end DCPM_Cooling;

          model DCPM_QuasiStationary "Test example: Compare DCPM motors transient - quasistationary"
            extends Modelica.Icons.Example;
            parameter Modelica.SIunits.Voltage Va = 100 "Actual armature voltage";
            parameter Modelica.SIunits.Voltage Ve = 100 "Actual excitation voltage";
            parameter Modelica.SIunits.AngularVelocity w0 = Modelica.SIunits.Conversions.from_rpm(1500) "No-load speed";
            parameter Modelica.SIunits.Torque TLoad = 63.66 "Nominal load torque";
            parameter Modelica.SIunits.Inertia JLoad = 0.15 "Load's moment of inertia";
            Machines.BasicMachines.DCMachines.DC_PermanentMagnet dcpm1(VaNominal = dcpmData.VaNominal, IaNominal = dcpmData.IaNominal, wNominal = dcpmData.wNominal, TaNominal = dcpmData.TaNominal, Ra = dcpmData.Ra, TaRef = dcpmData.TaRef, La = dcpmData.La, Jr = dcpmData.Jr, useSupport = false, Js = dcpmData.Js, frictionParameters = dcpmData.frictionParameters, coreParameters = dcpmData.coreParameters, strayLoadParameters = dcpmData.strayLoadParameters, brushParameters = dcpmData.brushParameters, phiMechanical(fixed = true), ia(fixed = true), TaOperational = 293.15, alpha20a = dcpmData.alpha20a, wMechanical(fixed = true, start = w0)) annotation(
              Placement(transformation(extent = {{-20, 20}, {0, 40}}, rotation = 0)));
            Modelica.Electrical.Analog.Sources.ConstantVoltage armatureVoltage(V = Va) annotation(
              Placement(transformation(extent = {{10, -10}, {-10, 10}}, rotation = 90, origin = {-80, 50})));
            Modelica.Electrical.Analog.Basic.Ground groundArmature annotation(
              Placement(transformation(origin = {-80, 20}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
            Modelica.Mechanics.Rotational.Components.Inertia loadInertia1(J = JLoad) annotation(
              Placement(transformation(extent = {{10, 20}, {30, 40}}, rotation = 0)));
            Modelica.Mechanics.Rotational.Sources.Torque loadTorque1(useSupport = false) annotation(
              Placement(transformation(extent = {{60, 20}, {40, 40}}, rotation = 0)));
            Modelica.Blocks.Sources.Pulse pulse(amplitude = -1.5 * TLoad, offset = 0, period = 1) annotation(
              Placement(transformation(extent = {{100, -10}, {80, 10}})));
            Machines.BasicMachines.QuasiStationaryDCMachines.DC_PermanentMagnet dcpm2(VaNominal = dcpmData.VaNominal, IaNominal = dcpmData.IaNominal, wNominal = dcpmData.wNominal, TaNominal = dcpmData.TaNominal, Ra = dcpmData.Ra, TaRef = dcpmData.TaRef, La = dcpmData.La, Jr = dcpmData.Jr, useSupport = false, Js = dcpmData.Js, frictionParameters = dcpmData.frictionParameters, coreParameters = dcpmData.coreParameters, strayLoadParameters = dcpmData.strayLoadParameters, brushParameters = dcpmData.brushParameters, phiMechanical(fixed = true), wMechanical(fixed = true, start = w0), TaOperational = 293.15, alpha20a = dcpmData.alpha20a) annotation(
              Placement(transformation(extent = {{-20, -40}, {0, -20}}, rotation = 0)));
            Modelica.Mechanics.Rotational.Components.Inertia loadInertia2(J = JLoad) annotation(
              Placement(transformation(extent = {{10, -40}, {30, -20}}, rotation = 0)));
            Modelica.Mechanics.Rotational.Sources.Torque loadTorque2(useSupport = false) annotation(
              Placement(transformation(extent = {{60, -40}, {40, -20}}, rotation = 0)));
            parameter Utilities.ParameterRecords.DcPermanentMagnetData dcpmData annotation(
              Placement(transformation(extent = {{-20, -80}, {0, -60}})));
          equation
            connect(loadInertia1.flange_b, loadTorque1.flange) annotation(
              Line(points = {{30, 30}, {30, 30}, {40, 30}}, color = {0, 0, 0}));
            connect(dcpm1.flange, loadInertia1.flange_a) annotation(
              Line(points = {{0, 30}, {10, 30}}, color = {0, 0, 0}, smooth = Smooth.None));
            connect(armatureVoltage.n, groundArmature.p) annotation(
              Line(points = {{-80, 40}, {-80, 30}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(armatureVoltage.p, dcpm1.pin_ap) annotation(
              Line(points = {{-80, 60}, {-4, 60}, {-4, 40}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(armatureVoltage.n, dcpm1.pin_an) annotation(
              Line(points = {{-80, 40}, {-16, 40}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(pulse.y, loadTorque1.tau) annotation(
              Line(points = {{79, 0}, {70, 0}, {70, 30}, {62, 30}}, color = {0, 0, 127}, smooth = Smooth.None));
            connect(loadInertia2.flange_b, loadTorque2.flange) annotation(
              Line(points = {{30, -30}, {30, -30}, {40, -30}}, color = {0, 0, 0}));
            connect(dcpm2.flange, loadInertia2.flange_a) annotation(
              Line(points = {{0, -30}, {10, -30}}, color = {0, 0, 0}, smooth = Smooth.None));
            connect(pulse.y, loadTorque2.tau) annotation(
              Line(points = {{79, 0}, {70, 0}, {70, -30}, {62, -30}}, color = {0, 0, 127}, smooth = Smooth.None));
            connect(armatureVoltage.p, dcpm2.pin_ap) annotation(
              Line(points = {{-80, 60}, {-40, 60}, {-40, 0}, {-4, 0}, {-4, -20}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(armatureVoltage.n, dcpm2.pin_an) annotation(
              Line(points = {{-80, 40}, {-60, 40}, {-60, -20}, {-16, -20}}, color = {0, 0, 255}, smooth = Smooth.None));
            annotation(
              experiment(StopTime = 2.0, Interval = 0.001),
              Documentation(info = "<HTML>
<b>Test example: Compare DCPM motors transient and quasistationary</b><br>
The motors start at no-load speed, then load pulses are applied.<br>
Simulate for 2 seconds and plot (versus time):
<ul>
<li>dcpm1.ia: armature current of transient model</li>
<li>dcpm1.wMechanical: motor's speed of transient model</li>
<li>dcpm1.tauElectrical: motor's torque of transient model</li>
<li>dcpm2.ia: armature current of quasistationary model</li>
<li>dcpm2.wMechanical: motor's speed of quasistationary model</li>
<li>dcpm2.tauElectrical: motor's torque of quasistationary model</li>
</ul>
</HTML>"));
          end DCPM_QuasiStationary;

          model DCPM_withLosses "Test example: Investigate influence of losses on DCPM motor performance"
            extends Modelica.Icons.Example;
            import Modelica.Constants.pi;
            parameter Modelica.SIunits.Voltage Va = 100 "Actual armature voltage";
            parameter Modelica.SIunits.Time tStart = 0.2 "Start of armature voltage ramp";
            parameter Modelica.SIunits.Time tRamp = 0.8 "Armature voltage ramp";
            parameter Modelica.SIunits.Torque TLoad1 = 63.66 "Nominal load torque";
            parameter Modelica.SIunits.AngularVelocity wLoad1 = 1425 * 2 * pi / 60 "Nominal load speed";
            parameter Modelica.SIunits.Torque TLoad2 = 61.30 "Nominal load torque";
            parameter Modelica.SIunits.AngularVelocity wLoad2 = 1417.5 * 2 * pi / 60 "Nominal load speed";
            parameter Modelica.SIunits.Inertia JLoad = 0.15 "Load's moment of inertia";
            Machines.BasicMachines.DCMachines.DC_PermanentMagnet dcpm1(VaNominal = dcpmData1.VaNominal, IaNominal = dcpmData1.IaNominal, wNominal = dcpmData1.wNominal, TaNominal = dcpmData1.TaNominal, Ra = dcpmData1.Ra, TaRef = dcpmData1.TaRef, La = dcpmData1.La, Jr = dcpmData1.Jr, useSupport = false, Js = dcpmData1.Js, frictionParameters = dcpmData1.frictionParameters, coreParameters = dcpmData1.coreParameters, strayLoadParameters = dcpmData1.strayLoadParameters, brushParameters = dcpmData1.brushParameters, TaOperational = 293.15, alpha20a = dcpmData1.alpha20a, phiMechanical(fixed = true), wMechanical(fixed = true), ia(fixed = true)) annotation(
              Placement(transformation(extent = {{0, 0}, {20, 20}}, rotation = 0)));
            Modelica.Blocks.Sources.Ramp ramp(duration = tRamp, height = Va, startTime = tStart) annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0, origin = {-70, 70})));
            Modelica.Electrical.Analog.Sources.SignalVoltage signalVoltage annotation(
              Placement(transformation(extent = {{10, -10}, {-10, 10}}, rotation = 90, origin = {-40, 70})));
            Modelica.Electrical.Analog.Basic.Ground ground annotation(
              Placement(transformation(origin = {-60, 30}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
            Modelica.Mechanics.Rotational.Components.Inertia loadInertia1(J = JLoad) annotation(
              Placement(transformation(extent = {{30, 0}, {50, 20}}, rotation = 0)));
            Modelica.Mechanics.Rotational.Sources.QuadraticSpeedDependentTorque loadTorque1(useSupport = false, tau_nominal = -TLoad1, TorqueDirection = false, w_nominal = wLoad1) annotation(
              Placement(transformation(extent = {{80, 0}, {60, 20}}, rotation = 0)));
            Machines.BasicMachines.DCMachines.DC_PermanentMagnet dcpm2(VaNominal = dcpmData2.VaNominal, IaNominal = dcpmData2.IaNominal, wNominal = dcpmData2.wNominal, TaNominal = dcpmData2.TaNominal, Ra = dcpmData2.Ra, TaRef = dcpmData2.TaRef, La = dcpmData2.La, Jr = dcpmData2.Jr, useSupport = false, Js = dcpmData2.Js, frictionParameters = dcpmData2.frictionParameters, coreParameters = dcpmData2.coreParameters, strayLoadParameters = dcpmData2.strayLoadParameters, brushParameters = dcpmData2.brushParameters, alpha20a = dcpmData2.alpha20a, phiMechanical(fixed = true), wMechanical(fixed = true), ia(fixed = true), TaOperational = 368.15, core(v(start = 0))) annotation(
              Placement(transformation(extent = {{0, -60}, {20, -40}}, rotation = 0)));
            Modelica.Mechanics.Rotational.Components.Inertia loadInertia2(J = JLoad) annotation(
              Placement(transformation(extent = {{30, -60}, {50, -40}}, rotation = 0)));
            Modelica.Mechanics.Rotational.Sources.QuadraticSpeedDependentTorque loadTorque2(useSupport = false, tau_nominal = -TLoad2, TorqueDirection = false, w_nominal = wLoad2) annotation(
              Placement(transformation(extent = {{80, -60}, {60, -40}}, rotation = 0)));
            parameter Utilities.ParameterRecords.DcPermanentMagnetData dcpmData1 annotation(
              Placement(transformation(extent = {{0, 60}, {20, 80}})));
            parameter Utilities.ParameterRecords.DcPermanentMagnetData dcpmData2(Ra = 0.03864, alpha20a(displayUnit = "1/K") = Modelica.Electrical.Machines.Thermal.Constants.alpha20Copper, wNominal = 148.44025288212, TaNominal = 368.15, frictionParameters(PRef = 100), coreParameters(PRef = 200), strayLoadParameters(PRef = 50), brushParameters(V = 0.5)) annotation(
              Placement(transformation(extent = {{0, -100}, {20, -80}})));
          equation
            connect(ramp.y, signalVoltage.v) annotation(
              Line(points = {{-59, 70}, {-47, 70}}, color = {0, 0, 255}));
            connect(signalVoltage.n, ground.p) annotation(
              Line(points = {{-40, 60}, {-40, 60}, {-40, 40}, {-60, 40}}, color = {0, 0, 255}));
            connect(loadInertia1.flange_b, loadTorque1.flange) annotation(
              Line(points = {{50, 10}, {60, 10}}, color = {0, 0, 0}));
            connect(dcpm1.flange, loadInertia1.flange_a) annotation(
              Line(points = {{20, 10}, {30, 10}}, color = {0, 0, 0}, smooth = Smooth.None));
            connect(loadInertia2.flange_b, loadTorque2.flange) annotation(
              Line(points = {{50, -50}, {60, -50}}, color = {0, 0, 0}));
            connect(dcpm2.flange, loadInertia2.flange_a) annotation(
              Line(points = {{20, -50}, {30, -50}}, color = {0, 0, 0}, smooth = Smooth.None));
            connect(signalVoltage.p, dcpm1.pin_ap) annotation(
              Line(points = {{-40, 80}, {-20, 80}, {-20, 40}, {16, 40}, {16, 20}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(signalVoltage.p, dcpm2.pin_ap) annotation(
              Line(points = {{-40, 80}, {-20, 80}, {-20, -20}, {16, -20}, {16, -40}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(signalVoltage.n, dcpm1.pin_an) annotation(
              Line(points = {{-40, 60}, {-40, 20}, {4, 20}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(signalVoltage.n, dcpm2.pin_an) annotation(
              Line(points = {{-40, 60}, {-40, -40}, {4, -40}}, color = {0, 0, 255}, smooth = Smooth.None));
            annotation(
              experiment(StopTime = 2.0, Interval = 0.001),
              Documentation(info = "<HTML>
<b>Test example: Investigate influence of losses on DCPM motor performance</b><br>
Both motors are started with a voltage ramp applied to the armature, causing the DC machines to start,
and accelerating inertias. Both machines are loading with a quadratic speed dependent load torque.<br>
The first machine <code>dcpm1</code> uses default machine parameters of model <i>DC_PermanentMagnet</i>,
the second machine <code>dcpm2</code> is parametrized with additional losses:<br>
<table>
<tr><td>                            </td><td>   dcpm1</td><td>   dcpm2</td><td>     </td></tr>
<tr><td>Armature voltage            </td><td>     100</td><td>     100</td><td>    V</td></tr>
<tr><td>Armature current            </td><td>     100</td><td>     100</td><td>    A</td></tr>
<tr><td>Inner voltage               </td><td>    95.0</td><td>    94.5</td><td>    V</td></tr>
<tr><td>Nominal speed               </td><td>  1425.0</td><td>  1417.5</td><td>  rpm</td></tr>
<tr><td>Armature resistance         </td><td> 0.05000</td><td> 0.03864</td><td>  Ohm</td></tr>
<tr><td>Temperature coefficient     </td><td>     n/a</td><td> 0.00392</td><td>  1/K</td></tr>
<tr><td>Reference temperature       </td><td>     n/a</td><td>      20</td><td> degC</td></tr>
<tr><td>Operation temperature       </td><td>     n/a</td><td>      95</td><td> degC</td></tr>
<tr><td>Brush voltage drop          </td><td>     n/a</td><td>     0.5</td><td>    V</td></tr>
<tr><td>Electrical input            </td><td>  10,000</td><td>  10,000</td><td>    W</td></tr>
<tr><td>Armature copper losses      </td><td>     500</td><td>     500</td><td>    W</td></tr>
<tr><td>Core losses                 </td><td>     n/a</td><td>     200</td><td>    W</td></tr>
<tr><td>Stray load losses           </td><td>     n/a</td><td>      50</td><td>    W</td></tr>
<tr><td>Friction losses             </td><td>     n/a</td><td>     100</td><td>    W</td></tr>
<tr><td>Brush losses                </td><td>     n/a</td><td>      50</td><td>    W</td></tr>
<tr><td>Mechanical output           </td><td>   9,500</td><td>   9,100</td><td>    W</td></tr>
<tr><td>Nominal torque              </td><td>   63,66</td><td>   61,30</td><td>   Nm</td></tr>
</table>
<br>
Note: The reference values (voltage, current, speed) are already propagated to the loss records,
using the nominal operation point.<br>
See:<br>
Anton Haumer, Christian Kral, Hansj&ouml;rg Kapeller, Thomas B&auml;uml, Johannes V. Gragger<br>
<a href=\"https://www.modelica.org/events/modelica2009/Proceedings/memorystick/pages/papers/0103/0103.pdf\">
The AdvancedMachines Library: Loss Models for Electric Machines</a><br>
Modelica 2009, 7<sup>th</sup> International Modelica Conference
</HTML>"));
          end DCPM_withLosses;
          annotation(
            Documentation(info = "<HTML>
This package contains test examples of DC machines.
</HTML>"));
        end DCMachines;

        package Transformers "Test examples of transformers"
          extends Modelica.Icons.ExamplesPackage;

          model TransformerTestbench "Transformer Testbench"
            extends Modelica.Icons.Example;
            parameter Modelica.SIunits.Resistance RL[3] = fill(1 / 3, 3) "Load resistance";
            Modelica.Electrical.MultiPhase.Sources.SineVoltage source(freqHz = fill(50, 3), V = fill(sqrt(2 / 3) * 100, 3)) annotation(
              Placement(transformation(origin = {-90, -10}, extent = {{-10, 10}, {10, -10}}, rotation = 270)));
            Modelica.Electrical.MultiPhase.Basic.Star starS annotation(
              Placement(transformation(origin = {-90, -40}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
            Modelica.Electrical.Analog.Basic.Ground groundS annotation(
              Placement(transformation(extent = {{-100, -80}, {-80, -60}}, rotation = 0)));
            Machines.Sensors.ElectricalPowerSensor electricalPowerSensorS annotation(
              Placement(transformation(extent = {{-90, 0}, {-70, 20}}, rotation = 0)));
            Machines.Sensors.CurrentQuasiRMSSensor currentQuasiRMSSensorS annotation(
              Placement(transformation(extent = {{-60, 20}, {-40, 0}}, rotation = 0)));
            Machines.Sensors.VoltageQuasiRMSSensor voltageQuasiRMSSensorS annotation(
              Placement(transformation(origin = {-50, -30}, extent = {{-10, 10}, {10, -10}}, rotation = 180)));
            Modelica.Electrical.MultiPhase.Basic.Delta deltaS annotation(
              Placement(transformation(origin = {-50, -10}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
            Modelica.Electrical.Analog.Basic.Resistor earth(R = 1e6) annotation(
              Placement(transformation(origin = {0, -40}, extent = {{-10, 10}, {10, -10}}, rotation = 270)));
            Modelica.Electrical.Analog.Basic.Ground groundT annotation(
              Placement(transformation(extent = {{-10, -80}, {10, -60}}, rotation = 0)));
            Machines.Sensors.VoltageQuasiRMSSensor voltageRMSSensorL annotation(
              Placement(transformation(origin = {50, -30}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
            Modelica.Electrical.MultiPhase.Basic.Delta deltaL annotation(
              Placement(transformation(origin = {50, -10}, extent = {{-10, 10}, {10, -10}}, rotation = 180)));
            Machines.Sensors.CurrentQuasiRMSSensor currentQuasiRMSSensorL annotation(
              Placement(transformation(extent = {{40, 20}, {60, 0}}, rotation = 0)));
            Machines.Sensors.ElectricalPowerSensor electricalPowerSensorL annotation(
              Placement(transformation(extent = {{70, 0}, {90, 20}}, rotation = 0)));
            Modelica.Electrical.MultiPhase.Basic.Resistor load(R = RL) annotation(
              Placement(transformation(origin = {90, -10}, extent = {{-10, 10}, {10, -10}}, rotation = 270)));
            Modelica.Electrical.MultiPhase.Basic.Star starL annotation(
              Placement(transformation(origin = {90, -40}, extent = {{-10, 10}, {10, -10}}, rotation = 270)));
            Modelica.Electrical.Analog.Basic.Ground groundL annotation(
              Placement(transformation(extent = {{80, -80}, {100, -60}}, rotation = 0)));
            parameter Machines.Utilities.TransformerData transformerData(C1 = Modelica.Utilities.Strings.substring(transformer.VectorGroup, 1, 1), C2 = Modelica.Utilities.Strings.substring(transformer.VectorGroup, 2, 2), f = 50, V1 = 100, V2 = 100, SNominal = 30E3, v_sc = 0.05, P_sc = 300) annotation(
              Placement(transformation(extent = {{-10, 40}, {10, 60}}, rotation = 0)));
            BasicMachines.Transformers.Dy.Dy01 transformer(n = transformerData.n, R1 = transformerData.R1, L1sigma = transformerData.L1sigma, R2 = transformerData.R2, L2sigma = transformerData.L2sigma, T1Ref = 293.15, alpha20_1(displayUnit = "1/K") = Modelica.Electrical.Machines.Thermal.Constants.alpha20Zero, T2Ref = 293.15, alpha20_2(displayUnit = "1/K") = Modelica.Electrical.Machines.Thermal.Constants.alpha20Zero, T1Operational = 293.15, T2Operational = 293.15) annotation(
              Placement(transformation(extent = {{-20, -10}, {20, 30}}, rotation = 0)));
          initial equation
            transformer.i2[1:2] = zeros(2);
          equation
            connect(starS.pin_n, groundS.p) annotation(
              Line(points = {{-90, -50}, {-90, -60}}, color = {0, 0, 255}));
            connect(source.plug_n, starS.plug_p) annotation(
              Line(points = {{-90, -20}, {-90, -30}}, color = {0, 0, 255}));
            connect(starL.pin_n, groundL.p) annotation(
              Line(points = {{90, -50}, {90, -60}}, color = {0, 0, 255}));
            connect(load.plug_n, starL.plug_p) annotation(
              Line(points = {{90, -20}, {90, -30}}, color = {0, 0, 255}));
            connect(earth.n, groundT.p) annotation(
              Line(points = {{0, -50}, {0, -50}, {0, -60}}, color = {0, 0, 255}));
            connect(electricalPowerSensorS.plug_nv, starS.plug_p) annotation(
              Line(points = {{-80, 0}, {-80, -30}, {-90, -30}}, color = {0, 0, 255}));
            connect(source.plug_p, electricalPowerSensorS.plug_p) annotation(
              Line(points = {{-90, 0}, {-90, 10}}, color = {0, 0, 255}));
            connect(electricalPowerSensorS.plug_ni, currentQuasiRMSSensorS.plug_p) annotation(
              Line(points = {{-70, 10}, {-60, 10}}, color = {0, 0, 255}));
            connect(currentQuasiRMSSensorL.plug_n, electricalPowerSensorL.plug_p) annotation(
              Line(points = {{60, 10}, {70, 10}}, color = {0, 0, 255}));
            connect(electricalPowerSensorL.plug_ni, load.plug_p) annotation(
              Line(points = {{90, 10}, {90, 0}}, color = {0, 0, 255}));
            connect(electricalPowerSensorL.plug_nv, starL.plug_p) annotation(
              Line(points = {{80, 0}, {80, -30}, {90, -30}}, color = {0, 0, 255}));
            connect(currentQuasiRMSSensorS.plug_n, transformer.plug1) annotation(
              Line(points = {{-40, 10}, {-20, 10}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(transformer.plug2, currentQuasiRMSSensorL.plug_p) annotation(
              Line(points = {{20, 10}, {40, 10}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(deltaS.plug_p, voltageQuasiRMSSensorS.plug_n) annotation(
              Line(points = {{-60, -10}, {-60, -30}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(deltaS.plug_n, voltageQuasiRMSSensorS.plug_p) annotation(
              Line(points = {{-40, -10}, {-40, -30}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(currentQuasiRMSSensorS.plug_n, deltaS.plug_n) annotation(
              Line(points = {{-40, 10}, {-40, -10}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(currentQuasiRMSSensorL.plug_p, deltaL.plug_n) annotation(
              Line(points = {{40, 10}, {40, -10}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(deltaL.plug_n, voltageRMSSensorL.plug_p) annotation(
              Line(points = {{40, -10}, {40, -30}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(deltaL.plug_p, voltageRMSSensorL.plug_n) annotation(
              Line(points = {{60, -10}, {60, -30}}, color = {0, 0, 255}, smooth = Smooth.None));
            annotation(
              Documentation(info = "<HTML>
<h4>Transformer testbench:</h4>
<p>
You may choose different connections as well as vary the load (even not symmetrical).
</p>
<p>
<b>Please pay attention</b> to proper grounding of the primary and secondary part of the whole circuit.<br>
The primary and secondary starpoint are available as connectors, if the connection is not delta (D or d).
</p>
<p>
In some cases it may be necessary to ground the transformer's starpoint even though the source's or load's starpoint are grounded:
</p>
<ul>
<li>Yy ... Grounding of transformer's primary or secondary starpoint with reasonable high earthing resistance is necessary.</li>
<li>Yd ... No grounding necessary.</li>
<li>Yz ... Grounding of transformer's primary starpoint with reasonable high earthing resistance is necessary.</li>
<li>Dy ... No grounding necessary.</li>
<li>Dd ... No grounding necessary.</li>
<li>Dz ... No grounding necessary.</li>
</ul>
</HTML>"),
              experiment(StopTime = 0.1, Interval = 0.001));
          end TransformerTestbench;

          model AsymmetricalLoad "AsymmetricalLoad"
            extends Modelica.Icons.Example;
            parameter Modelica.SIunits.Resistance RL = 1 "Load resistance";
            Modelica.Electrical.MultiPhase.Sources.SineVoltage source(freqHz = fill(50, 3), V = fill(sqrt(2 / 3) * 100, 3)) annotation(
              Placement(transformation(origin = {-90, -10}, extent = {{-10, 10}, {10, -10}}, rotation = 270)));
            Modelica.Electrical.MultiPhase.Basic.Star starS annotation(
              Placement(transformation(origin = {-90, -40}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
            Modelica.Electrical.Analog.Basic.Ground groundS annotation(
              Placement(transformation(extent = {{-100, -80}, {-80, -60}}, rotation = 0)));
            Modelica.Electrical.MultiPhase.Sensors.CurrentSensor currentSensorS annotation(
              Placement(transformation(extent = {{-60, 20}, {-40, 0}}, rotation = 0)));
            Modelica.Electrical.Analog.Basic.Ground groundL annotation(
              Placement(transformation(extent = {{0, -80}, {20, -60}}, rotation = 0)));
            parameter Machines.Utilities.TransformerData transformerData(C1 = Modelica.Utilities.Strings.substring(transformer.VectorGroup, 1, 1), C2 = Modelica.Utilities.Strings.substring(transformer.VectorGroup, 2, 2), f = 50, V1 = 100, V2 = 100, SNominal = 30E3, v_sc = 0.05, P_sc = 300) annotation(
              Placement(transformation(extent = {{-10, 40}, {10, 60}}, rotation = 0)));
            BasicMachines.Transformers.Dy.Dy01 transformer(n = transformerData.n, R1 = transformerData.R1, L1sigma = transformerData.L1sigma, R2 = transformerData.R2, L2sigma = transformerData.L2sigma, T1Ref = 293.15, alpha20_1(displayUnit = "1/K") = Modelica.Electrical.Machines.Thermal.Constants.alpha20Zero, T2Ref = 293.15, alpha20_2(displayUnit = "1/K") = Modelica.Electrical.Machines.Thermal.Constants.alpha20Zero, T1Operational = 293.15, T2Operational = 293.15) annotation(
              Placement(transformation(extent = {{-20, -10}, {20, 30}}, rotation = 0)));
            Modelica.Electrical.MultiPhase.Basic.PlugToPin_n plugToPin_n(k = 1) annotation(
              Placement(transformation(extent = {{20, 0}, {40, 20}})));
            Analog.Basic.Resistor load(R = RL) annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 270, origin = {50, 0})));
            Analog.Basic.Resistor earth(R = 1e6) annotation(
              Placement(transformation(origin = {-10, -40}, extent = {{-10, 10}, {10, -10}}, rotation = 270)));
            Analog.Basic.Ground groundT annotation(
              Placement(transformation(extent = {{-20, -80}, {0, -60}}, rotation = 0)));
          initial equation
            transformer.i2[1] = 0;
          equation
            connect(starS.pin_n, groundS.p) annotation(
              Line(points = {{-90, -50}, {-90, -60}}, color = {0, 0, 255}));
            connect(source.plug_n, starS.plug_p) annotation(
              Line(points = {{-90, -20}, {-90, -30}}, color = {0, 0, 255}));
            connect(currentSensorS.plug_n, transformer.plug1) annotation(
              Line(points = {{-40, 10}, {-20, 10}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(transformer.plug2, plugToPin_n.plug_n) annotation(
              Line(points = {{20, 10}, {28, 10}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(transformer.starpoint2, groundL.p) annotation(
              Line(points = {{10, -10}, {10, -60}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(load.p, plugToPin_n.pin_n) annotation(
              Line(points = {{50, 10}, {32, 10}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(transformer.starpoint2, load.n) annotation(
              Line(points = {{10, -10}, {50, -10}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(source.plug_p, currentSensorS.plug_p) annotation(
              Line(points = {{-90, 0}, {-90, 10}, {-60, 10}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(earth.n, groundT.p) annotation(
              Line(points = {{-10, -50}, {-10, -50}, {-10, -60}}, color = {0, 0, 255}));
            annotation(
              Documentation(info = "<HTML>
<h4>Asymmetrical (singlephase) load:</h4>
<p>
You may choose different connections.
</p>
<p>
<b>Please pay attention</b> to proper grounding of the primary and secondary part of the whole circuit.<br>
The primary and secondary starpoint are available as connectors, if the connection is not delta (D or d).
</p>
<p>
In some cases it may be necessary to ground the transformer's starpoint even though the source's or load's starpoint are grounded:
</p>
<ul>
<li>Yy with primary starpoint connected to source's starpoint: primary current in only one phase</li>
<li>Yy primary starpoint  not connected to source's starpoint: secondary voltage breaks down</li>
<li>Yz ... Grounding of transformer's primary starpoint with reasonable high earthing resistance is necessary.</li>
<li>Dy ... Load current in two   primary phases.</li>
<li>Dz ... Load current in three primary phases.</li>
</ul>
</HTML>"),
              experiment(StopTime = 0.1, Interval = 0.001));
          end AsymmetricalLoad;

          model Rectifier6pulse "6-pulse rectifier with 1 transformer"
            extends Modelica.Icons.Example;
            constant Integer m = 3 "Number of phases";
            parameter Modelica.SIunits.Voltage V = 100 * sqrt(2 / 3) "Amplitude of star-voltage";
            parameter Modelica.SIunits.Frequency f = 50 "Frequency";
            parameter Modelica.SIunits.Resistance RL = 0.4 "Load resistance";
            parameter Modelica.SIunits.Capacitance C = 0.005 "Total DC-capacitance";
            parameter Modelica.SIunits.Voltage VC0 = sqrt(3) * V "Initial voltage of capacitance";
            Modelica.Electrical.MultiPhase.Sources.SineVoltage source(m = m, V = fill(V, m), freqHz = fill(f, m)) annotation(
              Placement(transformation(origin = {-90, -10}, extent = {{-10, 10}, {10, -10}}, rotation = 270)));
            Modelica.Electrical.MultiPhase.Basic.Star starAC(m = m) annotation(
              Placement(transformation(origin = {-90, -40}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
            Modelica.Electrical.Analog.Basic.Ground groundAC annotation(
              Placement(transformation(extent = {{-100, -80}, {-80, -60}}, rotation = 0)));
            Modelica.Electrical.MultiPhase.Sensors.CurrentSensor currentSensor(m = m) annotation(
              Placement(transformation(extent = {{-80, -10}, {-60, 10}}, rotation = 0)));
            Modelica.Electrical.MultiPhase.Ideal.IdealDiode diode1(m = m, Ron = fill(1e-5, m), Goff = fill(1e-5, m), Vknee = fill(0, m)) annotation(
              Placement(transformation(origin = {-20, 60}, extent = {{-10, 10}, {10, -10}}, rotation = 90)));
            Modelica.Electrical.MultiPhase.Basic.Star star1(m = m) annotation(
              Placement(transformation(origin = {0, 70}, extent = {{10, 10}, {-10, -10}}, rotation = 180)));
            Modelica.Electrical.MultiPhase.Ideal.IdealDiode diode2(m = m, Ron = fill(1e-5, m), Goff = fill(1e-5, m), Vknee = fill(0, m)) annotation(
              Placement(transformation(origin = {-20, 20}, extent = {{-10, 10}, {10, -10}}, rotation = 90)));
            Modelica.Electrical.MultiPhase.Basic.Star star2(m = m) annotation(
              Placement(transformation(extent = {{-10, 0}, {10, 20}}, rotation = 0)));
            Modelica.Electrical.Analog.Basic.Resistor load(R = RL) annotation(
              Placement(transformation(origin = {50, 0}, extent = {{-10, 10}, {10, -10}}, rotation = 270)));
            Modelica.Electrical.Analog.Basic.Capacitor cDC1(C = 2 * C, v(start = VC0 / 2)) annotation(
              Placement(transformation(origin = {70, 20}, extent = {{-10, 10}, {10, -10}}, rotation = 270)));
            Modelica.Electrical.Analog.Basic.Capacitor cDC2(C = 2 * C, v(start = VC0 / 2)) annotation(
              Placement(transformation(origin = {70, -20}, extent = {{-10, 10}, {10, -10}}, rotation = 270)));
            Modelica.Electrical.Analog.Basic.Ground groundDC annotation(
              Placement(transformation(extent = {{80, -20}, {100, 0}}, rotation = 0)));
            parameter Machines.Utilities.TransformerData transformerData1(C1 = Modelica.Utilities.Strings.substring(transformer1.VectorGroup, 1, 1), C2 = Modelica.Utilities.Strings.substring(transformer1.VectorGroup, 2, 2), f = 50, V1 = 100, V2 = 100, SNominal = 30E3, v_sc = 0.05, P_sc = 300) annotation(
              Placement(transformation(extent = {{-60, 60}, {-40, 80}}, rotation = 0)));
            Machines.BasicMachines.Transformers.Dy.Dy01 transformer1(n = transformerData1.n, R1 = transformerData1.R1, L1sigma = transformerData1.L1sigma, R2 = transformerData1.R2, L2sigma = transformerData1.L2sigma, T1Ref = 293.15, alpha20_1(displayUnit = "1/K") = Modelica.Electrical.Machines.Thermal.Constants.alpha20Zero, T2Ref = 293.15, alpha20_2(displayUnit = "1/K") = Modelica.Electrical.Machines.Thermal.Constants.alpha20Zero, T1Operational = 293.15, T2Operational = 293.15) annotation(
              Placement(transformation(extent = {{-50, 30}, {-30, 50}}, rotation = 0)));
          initial equation
            cDC1.v = VC0 / 2;
            cDC2.v = VC0 / 2;
            transformer1.i2[1:2] = zeros(2);
          equation
            connect(cDC1.n, cDC2.p) annotation(
              Line(points = {{70, 10}, {70, -10}}, color = {0, 0, 255}));
            connect(cDC1.n, groundDC.p) annotation(
              Line(points = {{70, 10}, {70, 0}, {90, 0}}, color = {0, 0, 255}));
            connect(starAC.plug_p, source.plug_n) annotation(
              Line(points = {{-90, -30}, {-90, -20}}, color = {0, 0, 255}));
            connect(diode1.plug_n, star1.plug_p) annotation(
              Line(points = {{-20, 70}, {-10, 70}}, color = {0, 0, 255}));
            connect(diode2.plug_p, star2.plug_p) annotation(
              Line(points = {{-20, 10}, {-10, 10}}, color = {0, 0, 255}));
            connect(diode2.plug_n, diode1.plug_p) annotation(
              Line(points = {{-20, 30}, {-20, 50}}, color = {0, 0, 255}));
            connect(starAC.pin_n, groundAC.p) annotation(
              Line(points = {{-90, -50}, {-90, -60}}, color = {0, 0, 255}));
            connect(source.plug_p, currentSensor.plug_p) annotation(
              Line(points = {{-90, 0}, {-85, 0}, {-80, 0}}, color = {0, 0, 255}));
            connect(load.p, cDC1.p) annotation(
              Line(points = {{50, 10}, {50, 30}, {70, 30}}, color = {0, 0, 255}));
            connect(load.n, cDC2.n) annotation(
              Line(points = {{50, -10}, {50, -30}, {70, -30}}, color = {0, 0, 255}));
            connect(star1.pin_n, cDC1.p) annotation(
              Line(points = {{10, 70}, {70, 70}, {70, 30}}, color = {0, 0, 255}));
            connect(star2.pin_n, cDC2.n) annotation(
              Line(points = {{10, 10}, {20, 10}, {20, -70}, {70, -70}, {70, -30}}, color = {0, 0, 255}));
            connect(transformer1.plug1, currentSensor.plug_n) annotation(
              Line(points = {{-50, 40}, {-60, 40}, {-60, 0}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(transformer1.plug2, diode1.plug_p) annotation(
              Line(points = {{-30, 40}, {-20, 40}, {-20, 50}}, color = {0, 0, 255}, smooth = Smooth.None));
            annotation(
              Documentation(info = "<HTML>
Test example with multiphase components:<br>
Star-connected voltage source feeds via a transformer a diode bridge rectifier with a DC burden.<br>
Using f=50 Hz, simulate for 0.1 seconds (5 periods) and compare voltages and currents of source and DC burden,
neglecting initial transient.
</HTML>"),
              experiment(StopTime = 0.1, Interval = 0.0001));
          end Rectifier6pulse;

          model Rectifier12pulse "12-pulse rectifier with 2 transformers"
            extends Machines.Examples.Transformers.Rectifier6pulse(RL = 0.2);
            Modelica.Electrical.MultiPhase.Ideal.IdealDiode diode3(m = m, Ron = fill(1e-5, m), Goff = fill(1e-5, m), Vknee = fill(0, m)) annotation(
              Placement(transformation(origin = {-20, -20}, extent = {{-10, 10}, {10, -10}}, rotation = 90)));
            Modelica.Electrical.MultiPhase.Basic.Star star3(m = m) annotation(
              Placement(transformation(extent = {{-10, 0}, {10, -20}}, rotation = 0)));
            Modelica.Electrical.MultiPhase.Ideal.IdealDiode diode4(m = m, Ron = fill(1e-5, m), Goff = fill(1e-5, m), Vknee = fill(0, m)) annotation(
              Placement(transformation(origin = {-20, -60}, extent = {{-10, 10}, {10, -10}}, rotation = 90)));
            Modelica.Electrical.MultiPhase.Basic.Star star4(m = m) annotation(
              Placement(transformation(extent = {{-10, -60}, {10, -80}}, rotation = 0)));
            Machines.BasicMachines.Transformers.Dd.Dd00 transformer2(n = transformerData2.n, R1 = transformerData2.R1, L1sigma = transformerData2.L1sigma, R2 = transformerData2.R2, L2sigma = transformerData2.L2sigma, T1Ref = 293.15, alpha20_1(displayUnit = "1/K") = Modelica.Electrical.Machines.Thermal.Constants.alpha20Zero, T2Ref = 293.15, alpha20_2(displayUnit = "1/K") = Modelica.Electrical.Machines.Thermal.Constants.alpha20Zero, T1Operational = 293.15, T2Operational = 293.15) annotation(
              Placement(transformation(extent = {{-50, -50}, {-30, -30}}, rotation = 0)));
            parameter Machines.Utilities.TransformerData transformerData2(C1 = Modelica.Utilities.Strings.substring(transformer2.VectorGroup, 1, 1), C2 = Modelica.Utilities.Strings.substring(transformer2.VectorGroup, 2, 2), f = 50, V1 = 100, V2 = 100, SNominal = 30E3, v_sc = 0.05, P_sc = 300) annotation(
              Placement(transformation(extent = {{-60, -80}, {-40, -60}}, rotation = 0)));
          initial equation
            transformer2.core.plug_p1.pin[1:3].i = zeros(3);
          equation
            connect(diode3.plug_n, star3.plug_p) annotation(
              Line(points = {{-20, -10}, {-10, -10}}, color = {0, 0, 255}));
            connect(diode4.plug_p, star4.plug_p) annotation(
              Line(points = {{-20, -70}, {-10, -70}}, color = {0, 0, 255}));
            connect(diode4.plug_n, diode3.plug_p) annotation(
              Line(points = {{-20, -50}, {-20, -30}}, color = {0, 0, 255}));
            connect(star4.pin_n, cDC2.n) annotation(
              Line(points = {{10, -70}, {70, -70}, {70, -30}}, color = {0, 0, 255}));
            connect(star3.pin_n, cDC1.p) annotation(
              Line(points = {{10, -10}, {18, -10}, {18, -8}, {22, -8}, {22, -10}, {30, -10}, {30, 70}, {70, 70}, {70, 30}}, color = {0, 0, 255}));
            connect(transformer2.plug2, diode4.plug_n) annotation(
              Line(points = {{-30, -40}, {-20, -40}, {-20, -50}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(transformer2.plug1, currentSensor.plug_n) annotation(
              Line(points = {{-50, -40}, {-60, -40}, {-60, 0}}, color = {0, 0, 255}, smooth = Smooth.None));
            annotation(
              Documentation(info = "<HTML>
Test example with multiphase components:<br>
Star-connected voltage source feeds via two transformers (Dd0 and Dy1) two diode bridge rectifiers with a single DC burden.<br>
Using f=50 Hz, simulate for 0.1 seconds (5 periods) and compare voltages and currents of source and DC burden,
neglecting initial transient.
</HTML>"),
              experiment(StopTime = 0.1, Interval = 0.0001));
          end Rectifier12pulse;

          model AIMC_Transformer "Test example: AsynchronousInductionMachineSquirrelCage transformer starting"
            extends Machines.Examples.AsynchronousInductionMachines.AIMC_Transformer;
            annotation(
              experiment(StopTime = 2.5, Interval = 0.001),
              Documentation(info = "<HTML>
<b>Test example: Asynchronous induction machine with squirrel cage - transformer starting</b><br>
At start time tStart1 three phase voltage is supplied to the asynchronous induction machine with squirrel cage via the transformer;
the machine starts from standstill, accelerating inertias against load torque quadratic dependent on speed;
at start time tStart2 the machine is fed directly from the voltage source, finally reaching nominal speed.<br>
Simulate for 2.5 seconds and plot (versus time):
<ul>
<li>currentQuasiRMSSensor.I: stator current RMS</li>
<li>aimc.wMechanical: motor's speed</li>
<li>aimc.tauElectrical: motor's torque</li>
</ul>
Default machine parameters of model <i>AIM_SquirrelCage</i> are used.
</HTML>"));
          end AIMC_Transformer;
          annotation(
            Documentation(info = "<HTML>
This package contains test examples of DC machines.
</HTML>"));
        end Transformers;
        annotation(
          Documentation(info = "<HTML>
This package contains test examples of electric machines.
</HTML>", revisions = "<HTML>
<dl>
  <dt><b>Main Authors:</b></dt>
  <dd>
  <a href=\"http://www.haumer.at/\">Anton Haumer</a><br>
  Technical Consulting &amp; Electrical Engineering<br>
  A-3423 St.Andrae-Woerdern<br>Austria<br>
  email: <a href=\"mailto:a.haumer@haumer.at\">a.haumer@haumer.at</a>
  </dd>
  <dt><b>Copyright:</b></dt>
  <dd>Copyright &copy; 1998-2015, Modelica Association and Anton Haumer.<br>
  <i>The Modelica package is <b>free</b> software; it can be redistributed and/or modified
  under the terms of the <b>Modelica license</b>, see the license conditions
  and the accompanying <b>disclaimer</b> in the documentation of package
  Modelica in file \"Modelica/package.mo\".</i></dd>
</dl>
  <ul>
  <li> v1.00 2004/09/16 Anton Haumer</li>
  <li> v1.01 2004/09/18 Anton Haumer<br>
       adapted to improved MoveToRotational</li>
  <li> v1.02 2004/09/19 Anton Haumer<br>
       added examples for DC machines</li>
  <li> v1.03 2004/09/24 Anton Haumer<br>
       usage of Sensors.CurrentRMSSensor<br>
       added example for DC machine with series excitation</li>
  <li> v1.1  2004/10/01 Anton Haumer<br>
       changed naming and structure<br>
       issued to Modelica Standard Library 2.1</li>
  <li> v1.3.1 2004/11/06 Anton Haumer<br>
       small changes in Utilities.VfController</li>
  <li> v1.52 2005/10/12 Anton Haumer<br>
       new example for electrical excited synchronous induction machine</li>
  <li> v1.6.1 2004/11/22 Anton Haumer<br>
       introduced Utilities.TerminalBox</li>
  <li> v2.1.2 2010/02/09 Anton Haumer<br>
       included new Examples (AIMC_Transformer, DC_Comparison)</li>
  </ul>
</HTML>"));
      end Examples;

      package BasicMachines "Basic machine models"
        extends Modelica.Icons.Package;

        package AsynchronousInductionMachines "Models of asynchronous induction machines"
          extends Modelica.Icons.VariantsPackage;

          model AIM_SquirrelCage "Asynchronous induction machine with squirrel cage rotor"
            extends Machines.Interfaces.PartialBasicInductionMachine(final idq_ss = airGapS.i_ss, final idq_sr = airGapS.i_sr, final idq_rs = airGapS.i_rs, final idq_rr = airGapS.i_rr, redeclare final Machines.Thermal.AsynchronousInductionMachines.ThermalAmbientAIMC thermalAmbient(final Tr = TrOperational), redeclare final Machines.Interfaces.InductionMachines.ThermalPortAIMC thermalPort, redeclare final Machines.Interfaces.InductionMachines.ThermalPortAIMC internalThermalPort, redeclare final Machines.Interfaces.InductionMachines.PowerBalanceAIMC powerBalance(final lossPowerRotorWinding = squirrelCageR.LossPower, final lossPowerRotorCore = 0), statorCore(final w = statorCoreParameters.wRef));
            output Modelica.SIunits.Current ir[2] = -squirrelCageR.spacePhasor_r.i_ "Rotor cage currents";
            Machines.BasicMachines.Components.AirGapS airGapS(final p = p, final Lm = Lm, final m = m) annotation(
              Placement(transformation(origin = {0, 0}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
            parameter Modelica.SIunits.Inductance Lm(start = 3 * sqrt(1 - 0.0667) / (2 * pi * fsNominal)) "Stator main field inductance per phase" annotation(
              Dialog(tab = "Nominal resistances and inductances", groupImage = "modelica://Modelica/Resources/Images/Electrical/Machines/IMC.png"));
            parameter Modelica.SIunits.Inductance Lrsigma(start = 3 * (1 - sqrt(1 - 0.0667)) / (2 * pi * fsNominal)) "Rotor stray inductance per phase (equivalent three phase winding)" annotation(
              Dialog(tab = "Nominal resistances and inductances"));
            parameter Modelica.SIunits.Resistance Rr(start = 0.04) "Rotor resistance per phase (equivalent three phase winding) at TRef" annotation(
              Dialog(tab = "Nominal resistances and inductances"));
            parameter Modelica.SIunits.Temperature TrRef(start = 293.15) "Reference temperature of rotor resistance" annotation(
              Dialog(tab = "Nominal resistances and inductances"));
            parameter Machines.Thermal.LinearTemperatureCoefficient20 alpha20r(start = 0) "Temperature coefficient of rotor resistance at 20 degC" annotation(
              Dialog(tab = "Nominal resistances and inductances"));
            parameter Modelica.SIunits.Temperature TrOperational(start = 293.15) "Operational temperature of rotor resistance" annotation(
              Dialog(group = "Operational temperatures", enable = not useThermalPort));
            Machines.BasicMachines.Components.SquirrelCage squirrelCageR(final Lrsigma = Lrsigma, final Rr = Rr, final useHeatPort = true, final T_ref = TrRef, final T = TrRef, final alpha = Machines.Thermal.convertAlpha(alpha20r, TrRef)) annotation(
              Placement(transformation(origin = {0, -40}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          equation
            connect(airGapS.spacePhasor_r, squirrelCageR.spacePhasor_r) annotation(
              Line(points = {{10, -10}, {10, -30}}, color = {0, 0, 255}));
            connect(airGapS.flange, inertiaRotor.flange_a) annotation(
              Line(points = {{10, 0}, {36, 0}, {36, 0}, {70, 0}}, color = {0, 0, 0}, smooth = Smooth.None));
            connect(lssigma.spacePhasor_b, airGapS.spacePhasor_s) annotation(
              Line(points = {{20, 10}, {10, 10}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(squirrelCageR.heatPort, internalThermalPort.heatPortRotorWinding) annotation(
              Line(points = {{-10, -40}, {-10, -60}, {0, -60}, {0, -80}}, color = {191, 0, 0}, smooth = Smooth.None));
            connect(airGapS.support, internalSupport) annotation(
              Line(points = {{-10, 0}, {-40, 0}, {-40, -90}, {60, -90}, {60, -100}}, color = {0, 0, 0}, smooth = Smooth.None));
            annotation(
              defaultComponentName = "aimc",
              Documentation(info = "<HTML>
<p><b>Model of a three phase asynchronous induction machine with squirrel cage.</b><br>
Resistance and stray inductance of stator is modeled directly in stator phases, then using space phasor transformation. Resistance and stray inductance of rotor's squirrel cage is modeled in two axis of the rotor-fixed coordinate system. Both together connected via a stator-fixed <i>AirGap</i> model. The machine models take the following loss effects into account:
</p>

<ul>
<li>heat losses in the temperature dependent stator winding resistances</li>
<li>heat losses in the temperature dependent cage resistances</li>
<li>friction losses</li>
<li>core losses (only eddy current losses, no hysteresis losses)</li>
<li>stray load losses</li>
</ul>

<p><b>Default values for machine's parameters (a realistic example) are:</b><br>
<table>
<tr>
<td valign=\"top\">number of pole pairs p</td>
<td valign=\"top\">2</td><td valign=\"top\"> </td>
</tr>
<tr>
<td valign=\"top\">stator's moment of inertia</td>
<td valign=\"top\">0.29</td><td valign=\"top\">kg.m2</td>
</tr>
<tr>
<td valign=\"top\">rotor's moment of inertia</td>
<td valign=\"top\">0.29</td><td valign=\"top\">kg.m2</td>
</tr>
<tr>
<td valign=\"top\">nominal frequency fNominal</td>
<td valign=\"top\">50</td><td valign=\"top\">Hz</td>
</tr>
<tr>
<td valign=\"top\">nominal voltage per phase</td>
<td valign=\"top\">100</td><td valign=\"top\">V RMS</td>
</tr>
<tr>
<td valign=\"top\">nominal current per phase</td>
<td valign=\"top\">100</td><td valign=\"top\">A RMS</td>
</tr>
<tr>
<td valign=\"top\">nominal torque</td>
<td valign=\"top\">161.4</td><td valign=\"top\">Nm</td>
</tr>
<tr>
<td valign=\"top\">nominal speed</td>
<td valign=\"top\">1440.45</td><td valign=\"top\">rpm</td>
</tr>
<tr>
<td valign=\"top\">nominal mechanical output</td>
<td valign=\"top\">24.346</td><td valign=\"top\">kW</td>
</tr>
<tr>
<td valign=\"top\">efficiency</td>
<td valign=\"top\">92.7</td><td valign=\"top\">%</td>
</tr>
<tr>
<td valign=\"top\">power factor</td>
<td valign=\"top\">0.875</td><td valign=\"top\"> </td>
</tr>
<tr>
<td valign=\"top\">stator resistance</td>
<td valign=\"top\">0.03</td><td valign=\"top\">Ohm per phase at reference temperature</td>
</tr>
<tr>
<td valign=\"top\">reference temperature TsRef</td>
<td valign=\"top\">20</td><td valign=\"top\">&deg;C</td>
</tr>
<tr>
<td valign=\"top\">temperature coefficient alpha20s </td>
<td valign=\"top\">0</td><td valign=\"top\">1/K</td>
</tr>
<tr>
<td valign=\"top\">rotor resistance</td>
<td valign=\"top\">0.04</td><td valign=\"top\">Ohm at reference temperature</td>
</tr>
<tr>
<td valign=\"top\">reference temperature TrRef</td>
<td valign=\"top\">20</td><td valign=\"top\">&deg;C</td>
</tr>
<tr>
<td valign=\"top\">temperature coefficient alpha20r </td>
<td valign=\"top\">0</td><td valign=\"top\">1/K</td>
</tr>
<tr>
<td valign=\"top\">stator reactance Xs</td>
<td valign=\"top\">3</td><td valign=\"top\">Ohm per phase</td>
</tr>
<tr>
<td valign=\"top\">rotor reactance Xr</td>
<td valign=\"top\">3</td><td valign=\"top\">Ohm</td>
</tr>
<tr>
<td valign=\"top\">total stray coefficient sigma</td>
<td valign=\"top\">0.0667</td><td valign=\"top\"> </td>
</tr>
<tr>
<td valign=\"top\">stator operational temperature TsOperational</td>
<td valign=\"top\">20</td><td valign=\"top\">&deg;C</td>
</tr>
<tr>
<td valign=\"top\">rotor operational temperature TrOperational</td>
<td valign=\"top\">20</td><td valign=\"top\">&deg;C</td>
</tr>
<tr>
<td valign=\"top\">These values give the following inductances:</td>
<td valign=\"top\"> </td><td valign=\"top\"> </td>
</tr>
<tr>
<td valign=\"top\">stator stray inductance per phase</td>
<td valign=\"top\">Xs * (1 - sqrt(1-sigma))/(2*pi*fNominal)</td><td valign=\"top\"> </td>
</tr>
<tr>
<td valign=\"top\">rotor stray inductance</td>
<td valign=\"top\">Xr * (1 - sqrt(1-sigma))/(2*pi*fNominal)</td><td valign=\"top\"> </td>
</tr>
<tr>
<td valign=\"top\">main field inductance per phase</td>
<td valign=\"top\">sqrt(Xs*Xr * (1-sigma))/(2*pi*fNominal)</td><td valign=\"top\"> </td>
</tr>
</table>
</HTML>"));
          end AIM_SquirrelCage;

          model AIM_SlipRing "Asynchronous induction machine with slipring rotor"
            extends Machines.Interfaces.PartialBasicInductionMachine(final idq_ss = airGapS.i_ss, final idq_sr = airGapS.i_sr, final idq_rs = airGapS.i_rs, final idq_rr = airGapS.i_rr, redeclare final Machines.Thermal.AsynchronousInductionMachines.ThermalAmbientAIMS thermalAmbient(final Tr = TrOperational, final mr = m), redeclare final Machines.Interfaces.InductionMachines.ThermalPortAIMS thermalPort(final mr = m), redeclare final Machines.Interfaces.InductionMachines.ThermalPortAIMS internalThermalPort(final mr = m), redeclare final Machines.Interfaces.InductionMachines.PowerBalanceAIMS powerBalance(final lossPowerRotorWinding = sum(rr.resistor.LossPower), final lossPowerRotorCore = rotorCore.lossPower, final lossPowerBrush = 0, final powerRotor = Machines.SpacePhasors.Functions.activePower(vr, ir)), statorCore(final w = statorCoreParameters.wRef));
            Machines.BasicMachines.Components.AirGapS airGapS(final p = p, final Lm = Lm, final m = m) annotation(
              Placement(transformation(origin = {0, 0}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
            parameter Modelica.SIunits.Inductance Lm(start = 3 * sqrt(1 - 0.0667) / (2 * pi * fsNominal)) "Stator main field inductance per phase" annotation(
              Dialog(tab = "Nominal resistances and inductances", groupImage = "modelica://Modelica/Resources/Images/Electrical/Machines/IMS.png"));
            parameter Modelica.SIunits.Inductance Lrsigma(start = 3 * (1 - sqrt(1 - 0.0667)) / (2 * pi * fsNominal)) "Rotor stray inductance per phase w.r.t. rotor side" annotation(
              Dialog(tab = "Nominal resistances and inductances"));
            parameter Modelica.SIunits.Inductance Lrzero = Lrsigma "Rotor zero sequence inductance w.r.t. rotor side" annotation(
              Dialog(tab = "Nominal resistances and inductances"));
            parameter Modelica.SIunits.Resistance Rr(start = 0.04) "Rotor resistance per phase at TRef w.r.t. rotor side" annotation(
              Dialog(tab = "Nominal resistances and inductances"));
            parameter Modelica.SIunits.Temperature TrRef(start = 293.15) "Reference temperature of rotor resistance" annotation(
              Dialog(tab = "Nominal resistances and inductances"));
            parameter Machines.Thermal.LinearTemperatureCoefficient20 alpha20r(start = 0) "Temperature coefficient of rotor resistance at 20 degC" annotation(
              Dialog(tab = "Nominal resistances and inductances"));
            parameter Boolean useTurnsRatio(start = true) "Use turnsRatio or calculate from locked-rotor voltage?";
            parameter Real turnsRatio(final min = Modelica.Constants.small, start = 1) "Effective number of stator turns / effective number of rotor turns" annotation(
              Dialog(enable = useTurnsRatio));
            parameter Modelica.SIunits.Voltage VsNominal(start = 100) "Nominal stator voltage per phase" annotation(
              Dialog(enable = not useTurnsRatio));
            parameter Modelica.SIunits.Voltage VrLockedRotor(start = 100 * (2 * pi * fsNominal * Lm) / sqrt(Rs ^ 2 + (2 * pi * fsNominal * (Lm + Lssigma)) ^ 2)) "Locked-rotor voltage per phase" annotation(
              Dialog(enable = not useTurnsRatio));
            parameter Modelica.SIunits.Temperature TrOperational(start = 293.15) "Operational temperature of rotor resistance" annotation(
              Dialog(group = "Operational temperatures", enable = not useThermalPort));
            parameter Machines.Losses.CoreParameters rotorCoreParameters(final m = 3, PRef = 0, VRef(start = 1) = 1, wRef(start = 1) = 1) "Rotor core loss parameter record; all parameters refer to rotor side" annotation(
              Dialog(tab = "Losses"));
            output Modelica.SIunits.Current i_0_r(stateSelect = StateSelect.prefer) = spacePhasorR.zero.i "Rotor zero-sequence current";
            output Modelica.SIunits.Voltage vr[m] = plug_rp.pin.v - plug_rn.pin.v "Rotor instantaneous voltages";
            output Modelica.SIunits.Current ir[m] = plug_rp.pin.i "Rotor instantaneous currents";
          protected
            final parameter Real internalTurnsRatio = if useTurnsRatio then turnsRatio else VsNominal / VrLockedRotor * (2 * pi * fsNominal * Lm) / sqrt(Rs ^ 2 + (2 * pi * fsNominal * (Lm + Lssigma)) ^ 2);
          public
            Machines.SpacePhasors.Components.SpacePhasor spacePhasorR(final turnsRatio = internalTurnsRatio) annotation(
              Placement(transformation(origin = {-70, -50}, extent = {{10, 10}, {-10, -10}}, rotation = 180)));
            Modelica.Electrical.MultiPhase.Basic.Resistor rr(final m = m, final R = fill(Rr, m), final T_ref = fill(TrRef, m), final alpha = fill(Machines.Thermal.convertAlpha(alpha20r, TrRef), m), final useHeatPort = true, final T = fill(TrRef, m)) annotation(
              Placement(transformation(extent = {{10, -10}, {-10, 10}}, rotation = 90, origin = {-80, 40})));
            Modelica.Electrical.MultiPhase.Interfaces.PositivePlug plug_rp(final m = m) "Positive rotor plug" annotation(
              Placement(transformation(extent = {{-110, 70}, {-90, 50}}, rotation = 0)));
            Modelica.Electrical.MultiPhase.Interfaces.NegativePlug plug_rn(final m = m) "Negative rotor plug" annotation(
              Placement(transformation(extent = {{-110, -50}, {-90, -70}}, rotation = 0)));
            Machines.BasicMachines.Components.Inductor lrsigma(final L = fill(internalTurnsRatio ^ 2 * Lrsigma, 2)) annotation(
              Placement(transformation(extent = {{10, -10}, {-10, 10}}, rotation = 270, origin = {20, -20})));
            Modelica.Electrical.Analog.Basic.Inductor lrzero(final L = internalTurnsRatio ^ 2 * Lrzero) annotation(
              Placement(transformation(extent = {{10, 10}, {-10, -10}}, rotation = 90, origin = {-50, -60})));
            Machines.Losses.InductionMachines.Core rotorCore(final coreParameters = rotorCoreParameters, final w = rotorCoreParameters.wRef, final useHeatPort = true, final turnsRatio = internalTurnsRatio) annotation(
              Placement(transformation(extent = {{-10, 10}, {10, -10}}, rotation = 180, origin = {0, -30})));
          equation
            connect(lssigma.spacePhasor_b, airGapS.spacePhasor_s) annotation(
              Line(points = {{20, 10}, {10, 10}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(lrsigma.spacePhasor_b, airGapS.spacePhasor_r) annotation(
              Line(points = {{20, -10}, {10, -10}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(rr.plug_n, spacePhasorR.plug_p) annotation(
              Line(points = {{-80, 30}, {-80, -40}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(spacePhasorR.plug_n, plug_rn) annotation(
              Line(points = {{-80, -60}, {-100, -60}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(spacePhasorR.zero, lrzero.p) annotation(
              Line(points = {{-60, -50}, {-50, -50}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(spacePhasorR.ground, lrzero.n) annotation(
              Line(points = {{-60, -60}, {-60, -70}, {-50, -70}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(spacePhasorR.spacePhasor, lrsigma.spacePhasor_a) annotation(
              Line(points = {{-60, -40}, {-20, -40}, {-20, -50}, {20, -50}, {20, -30}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(rotorCore.spacePhasor, lrsigma.spacePhasor_a) annotation(
              Line(points = {{10, -30}, {20, -30}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(rotorCore.heatPort, internalThermalPort.heatPortRotorCore) annotation(
              Line(points = {{10, -40}, {0, -40}, {0, -80}}, color = {191, 0, 0}, smooth = Smooth.None));
            connect(rr.heatPort, internalThermalPort.heatPortRotorWinding) annotation(
              Line(points = {{-70, 40}, {50, 40}, {50, -80}, {0, -80}}, color = {191, 0, 0}, smooth = Smooth.None));
            connect(plug_rp, rr.plug_p) annotation(
              Line(points = {{-100, 60}, {-80, 60}, {-80, 50}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(airGapS.flange, inertiaRotor.flange_a) annotation(
              Line(points = {{10, 0}, {70, 0}}, color = {0, 0, 0}, smooth = Smooth.None));
            connect(fixed.flange, internalSupport) annotation(
              Line(points = {{50, -100}, {60, -100}}, color = {0, 0, 0}, smooth = Smooth.None));
            connect(internalSupport, airGapS.support) annotation(
              Line(points = {{60, -100}, {60, -90}, {-40, -90}, {-40, 0}, {-10, 0}}, color = {0, 0, 0}, smooth = Smooth.None));
            annotation(
              defaultComponentName = "aims",
              Documentation(info = "<HTML>
<p><b>Model of a three phase asynchronous induction machine with slipring rotor.</b><br>
Resistance and stray inductance of stator and rotor are modeled directly in stator respectively rotor phases, then using space phasor transformation and a stator-fixed <i>AirGap</i> model. The machine models take the following loss effects into account:
</p>

<ul>
<li>heat losses in the temperature dependent stator winding resistances</li>
<li>heat losses in the temperature dependent rotor winding resistances</li>
<li>friction losses</li>
<li>core losses (only eddy current losses, no hysteresis losses)</li>
<li>stray load losses</li>
</ul>

<p><b>Default values for machine's parameters (a realistic example) are:</b><br>
<table>
<tr>
<td valign=\"top\">number of pole pairs p</td>
<td valign=\"top\">2</td><td valign=\"top\"> </td>
</tr>
<tr>
<td valign=\"top\">stator's moment of inertia</td>
<td valign=\"top\">0.29</td><td valign=\"top\">kg.m2</td>
</tr>
<tr>
<td valign=\"top\">rotor's moment of inertia</td>
<td valign=\"top\">0.29</td><td valign=\"top\">kg.m2</td>
</tr>
<tr>
<td valign=\"top\">nominal frequency fNominal</td>
<td valign=\"top\">50</td><td valign=\"top\">Hz</td>
</tr>
<tr>
<td valign=\"top\">nominal voltage per phase</td>
<td valign=\"top\">100</td><td valign=\"top\">V RMS</td>
</tr>
<tr>
<td valign=\"top\">nominal current per phase</td>
<td valign=\"top\">100</td><td valign=\"top\">A RMS</td>
</tr>
<tr>
<td valign=\"top\">nominal torque</td>
<td valign=\"top\">161.4</td><td valign=\"top\">Nm</td>
</tr>
<tr>
<td valign=\"top\">nominal speed</td>
<td valign=\"top\">1440.45</td><td valign=\"top\">rpm</td>
</tr>
<tr>
<td valign=\"top\">nominal mechanical output</td>
<td valign=\"top\">24.346</td><td valign=\"top\">kW</td>
</tr>
<tr>
<td valign=\"top\">efficiency</td>
<td valign=\"top\">92.7</td><td valign=\"top\">%</td>
</tr>
<tr>
<td valign=\"top\">power factor</td>
<td valign=\"top\">0.875</td><td valign=\"top\"> </td>
</tr>
<tr>
<td valign=\"top\">stator resistance</td>
<td valign=\"top\">0.03</td><td valign=\"top\">Ohm per phase at reference temperature</td>
</tr>
<tr>
<td valign=\"top\">reference temperature TsRef</td>
<td valign=\"top\">20</td><td valign=\"top\">&deg;C</td>
</tr>
<tr>
<td valign=\"top\">temperature coefficient alpha20s </td>
<td valign=\"top\">0</td><td valign=\"top\">1/K</td>
</tr>
<tr>
<td valign=\"top\">rotor resistance</td>
<td valign=\"top\">0.04</td><td valign=\"top\">Ohm per phase at reference temperature</td>
</tr>
<tr>
<td valign=\"top\">reference temperature TrRef</td>
<td valign=\"top\">20</td><td valign=\"top\">&deg;C</td>
</tr>
<tr>
<td valign=\"top\">temperature coefficient alpha20r </td>
<td valign=\"top\">0</td><td valign=\"top\">1/K</td>
</tr>
<tr>
<td valign=\"top\">stator reactance Xs</td>
<td valign=\"top\">3</td><td valign=\"top\">Ohm per phase</td>
</tr>
<tr>
<td valign=\"top\">rotor reactance Xr</td>
<td valign=\"top\">3</td><td valign=\"top\">Ohm per phase</td>
</tr>
<tr>
<td valign=\"top\">total stray coefficient sigma</td>
<td valign=\"top\">0.0667</td><td valign=\"top\"> </td>
</tr>
<tr>
<td valign=\"top\">turnsRatio</td>
<td valign=\"top\">1</td><td valign=\"top\">effective ratio of stator and rotor current</td>
</tr>
<tr>
<td valign=\"top\">stator operational temperature TsOperational</td>
<td valign=\"top\">20</td><td valign=\"top\">&deg;C</td>
</tr>
<tr>
<td valign=\"top\">rotor operational temperature TrOperational</td>
<td valign=\"top\">20</td><td valign=\"top\">&deg;C</td>
</tr>
<tr>
<td valign=\"top\">These values give the following inductances:</td>
<td valign=\"top\"> </td><td valign=\"top\"> </td>
</tr>
<tr>
<td valign=\"top\">stator stray inductance per phase</td>
<td valign=\"top\">Xs * (1 - sqrt(1-sigma))/(2*pi*fNominal)</td><td valign=\"top\"> </td>
</tr>
<tr>
<td valign=\"top\">rotor stray inductance</td>
<td valign=\"top\">Xr * (1 - sqrt(1-sigma))/(2*pi*fNominal)</td><td valign=\"top\"> </td>
</tr>
<tr>
<td valign=\"top\">main field inductance per phase</td>
<td valign=\"top\">sqrt(Xs*Xr * (1-sigma))/(2*pi*f)</td><td valign=\"top\"> </td>
</tr>
</table>
<p>
Parameter turnsRatio could be obtained from the following relationship
at standstill with open rotor circuit at nominal voltage and nominal frequency, <br>
using the locked-rotor voltage VR, no-load stator current I0 and powerfactor PF0:<br>
turnsRatio * <u>V</u><sub>R</sub> = <u>V</u><sub>s</sub> - (R<sub>s</sub> + j X<sub>s,sigma</sub>) <u>I</u><sub>0</sub>
</p>
</HTML>"),
              Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-100, 50}, {-100, 20}, {-60, 20}}, color = {0, 0, 255}), Line(points = {{-100, -50}, {-100, -20}, {-60, -20}}, color = {0, 0, 255})}));
          end AIM_SlipRing;
          annotation(
            Documentation(info = "<HTML>
This package contains models of asynchronous induction machines, based on space phasor theory:
<ul>
<li>AIM_SquirrelCage: asynchronous induction machine with squirrel cage</li>
<li>AIM_SlipRing: asynchronous induction machine with wound rotor</li>
</ul>
These models use package SpacePhasors.
</HTML>", revisions = "<HTML>
<dl>
  <dt><b>Main Authors:</b></dt>
  <dd>
  <a href=\"http://www.haumer.at/\">Anton Haumer</a><br>
  Technical Consulting &amp; Electrical Engineering<br>
  A-3423 St.Andrae-Woerdern<br>Austria<br>
  email: <a href=\"mailto:a.haumer@haumer.at\">a.haumer@haumer.at</a>
  </dd>
  <dt><b>Copyright:</b></dt>
  <dd>Copyright &copy; 1998-2015, Modelica Association and Anton Haumer.<br>
  <i>The Modelica package is <b>free</b> software; it can be redistributed and/or modified
  under the terms of the <b>Modelica license</b>, see the license conditions
  and the accompanying <b>disclaimer</b> in the documentation of package
  Modelica in file \"Modelica/package.mo\".</i></dd>
</dl>
  <ul>
  <li> v1.02 2004/09/19 Anton Haumer</li>
  <li> v1.03 2004/09/24 Anton Haumer<br>
       consistent naming of inductors and resistors in machine models</li>
  <li> v1.1  2004/10/01 Anton Haumer<br>
       changed naming and structure<br>
       issued to Modelica Standard Library 2.1</li>
  <li> v1.2  2004/10/27 Anton Haumer<br>
       fixed a bug with support (formerly bearing)</li>
  <li> v1.3.2 2004/11/10 Anton Haumer<br>
       ReluctanceRotor moved to SynchronousMachines</li>
  <li> v1.4   2004/11/11 Anton Haumer<br>
       removed mechanical flange support<br>
       to ease the implementation of a 3D-frame in a future release</li>
  <li> v1.6.3 2005/11/25 Anton Haumer<br>
       easier parameterization of SlipRing model</li>
  <li> v2.2.0 2011/02/10 Anton Haumer<br>
       conditional ThermalPort for all machines</li>
  </ul>
</HTML>"));
        end AsynchronousInductionMachines;

        package SynchronousInductionMachines "Models of synchronous induction machines"
          extends Modelica.Icons.VariantsPackage;

          model SM_PermanentMagnet "Permanent magnet synchronous induction machine"
            extends Machines.Interfaces.PartialBasicInductionMachine(Lssigma(start = 0.1 / (2 * pi * fsNominal)), final idq_ss = airGapR.i_ss, final idq_sr = airGapR.i_sr, final idq_rs = airGapR.i_rs, final idq_rr = airGapR.i_rr, redeclare final Machines.Thermal.SynchronousInductionMachines.ThermalAmbientSMPM thermalAmbient(final useDamperCage = useDamperCage, final Tr = TrOperational, final Tpm = TpmOperational), redeclare final Machines.Interfaces.InductionMachines.ThermalPortSMPM thermalPort(final useDamperCage = useDamperCage), redeclare final Machines.Interfaces.InductionMachines.ThermalPortSMPM internalThermalPort(final useDamperCage = useDamperCage), redeclare final Machines.Interfaces.InductionMachines.PowerBalanceSMPM powerBalance(final lossPowerRotorWinding = damperCageLossPower, final lossPowerRotorCore = 0, final lossPowerPermanentMagnet = permanentMagnet.lossPower), statorCore(final w = statorCoreParameters.wRef));
            Modelica.Blocks.Interfaces.RealOutput ir[2](start = zeros(2), each final quantity = "ElectricCurrent", each final unit = "A") if useDamperCage "Damper cage currents" annotation(
              Dialog(showStartAttribute = true));
            Modelica.Blocks.Interfaces.RealOutput idq_dr[2](each stateSelect = StateSelect.prefer, each final quantity = "ElectricCurrent", each final unit = "A") if useDamperCage "Damper space phasor current / rotor fixed frame";
            Machines.BasicMachines.Components.AirGapR airGapR(final p = p, final Lmd = Lmd, final Lmq = Lmq, final m = m) annotation(
              Placement(transformation(origin = {0, 0}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
            final parameter Modelica.SIunits.Temperature TpmOperational = 293.15 "Operational temperature of permanent magnet" annotation(
              Dialog(group = "Operational temperatures"));
            parameter Modelica.SIunits.Temperature TrOperational(start = 293.15) "Operational temperature of (optional) damper cage" annotation(
              Dialog(group = "Operational temperatures", enable = not useThermalPort and useDamperCage));
            parameter Modelica.SIunits.Voltage VsOpenCircuit(start = 112.3) "Open circuit RMS voltage per phase @ fsNominal";
            parameter Modelica.SIunits.Inductance Lmd(start = 0.3 / (2 * pi * fsNominal)) "Stator main field inductance per phase in d-axis" annotation(
              Dialog(tab = "Nominal resistances and inductances", groupImage = "modelica://Modelica/Resources/Images/Electrical/Machines/SMPM.png"));
            parameter Modelica.SIunits.Inductance Lmq(start = 0.3 / (2 * pi * fsNominal)) "Stator main field inductance per phase in q-axis" annotation(
              Dialog(tab = "Nominal resistances and inductances"));
            parameter Boolean useDamperCage(start = true) "Enable / disable damper cage" annotation(
              Evaluate = true,
              Dialog(tab = "Nominal resistances and inductances", group = "DamperCage"));
            parameter Modelica.SIunits.Inductance Lrsigmad(start = 0.05 / (2 * pi * fsNominal)) "Damper stray inductance in d-axis" annotation(
              Dialog(tab = "Nominal resistances and inductances", group = "DamperCage", enable = useDamperCage));
            parameter Modelica.SIunits.Inductance Lrsigmaq = Lrsigmad "Damper stray inductance in q-axis" annotation(
              Dialog(tab = "Nominal resistances and inductances", group = "DamperCage", enable = useDamperCage));
            parameter Modelica.SIunits.Resistance Rrd(start = 0.04) "Damper resistance in d-axis at TRef" annotation(
              Dialog(tab = "Nominal resistances and inductances", group = "DamperCage", enable = useDamperCage));
            parameter Modelica.SIunits.Resistance Rrq = Rrd "Damper resistance in q-axis at TRef" annotation(
              Dialog(tab = "Nominal resistances and inductances", group = "DamperCage", enable = useDamperCage));
            parameter Modelica.SIunits.Temperature TrRef(start = 293.15) "Reference temperature of damper resistances in d- and q-axis" annotation(
              Dialog(tab = "Nominal resistances and inductances", group = "DamperCage", enable = useDamperCage));
            parameter Machines.Thermal.LinearTemperatureCoefficient20 alpha20r(start = 0) "Temperature coefficient of damper resistances in d- and q-axis" annotation(
              Dialog(tab = "Nominal resistances and inductances", group = "DamperCage", enable = useDamperCage));
            parameter Machines.Losses.PermanentMagnetLossParameters permanentMagnetLossParameters(IRef(start = 100), wRef(start = 2 * pi * fsNominal / p)) "Permanent magnet loss parameter record" annotation(
              Dialog(tab = "Losses"));
            Components.PermanentMagnetWithLosses permanentMagnet(final Ie = Ie, final useHeatPort = true, final m = m, final permanentMagnetLossParameters = permanentMagnetLossParameters, final is = is) annotation(
              Placement(transformation(origin = {30, -30}, extent = {{10, 10}, {-10, -10}}, rotation = 180)));
            Machines.BasicMachines.Components.DamperCage damperCage(final Lrsigmad = Lrsigmad, final Lrsigmaq = Lrsigmaq, final Rrd = Rrd, final Rrq = Rrq, final T_ref = TrRef, final alpha = Machines.Thermal.convertAlpha(alpha20r, TrRef), final T = TrRef, final useHeatPort = true) if useDamperCage annotation(
              Placement(transformation(origin = {0, -40}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          protected
            final parameter Modelica.SIunits.Current Ie = sqrt(2) * VsOpenCircuit / (Lmd * 2 * pi * fsNominal) "Equivalent excitation current";
            Modelica.Blocks.Interfaces.RealOutput damperCageLossPower(final quantity = "Power", final unit = "W") "Damper losses";
          equation
            connect(ir, damperCage.i);
            connect(idq_dr, damperCage.i);
            connect(damperCageLossPower, damperCage.lossPower);
            if not useDamperCage then
              damperCageLossPower = 0;
            end if;
            connect(airGapR.spacePhasor_r, damperCage.spacePhasor_r) annotation(
              Line(points = {{10, -10}, {10, -30}}, color = {0, 0, 255}));
            connect(airGapR.spacePhasor_r, permanentMagnet.spacePhasor_r) annotation(
              Line(points = {{10, -10}, {10, -20}, {20, -20}}, color = {0, 0, 255}));
            connect(airGapR.support, internalSupport) annotation(
              Line(points = {{-10, 0}, {-40, 0}, {-40, -90}, {60, -90}, {60, -100}}, color = {0, 0, 0}, smooth = Smooth.None));
            connect(lssigma.spacePhasor_b, airGapR.spacePhasor_s) annotation(
              Line(points = {{20, 10}, {10, 10}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(airGapR.flange, inertiaRotor.flange_a) annotation(
              Line(points = {{10, 0}, {70, 0}}, color = {0, 0, 0}, smooth = Smooth.None));
            connect(permanentMagnet.heatPort, internalThermalPort.heatPortPermanentMagnet) annotation(
              Line(points = {{20, -40}, {20, -80}, {0, -80}}, color = {191, 0, 0}, smooth = Smooth.None));
            connect(permanentMagnet.flange, inertiaRotor.flange_b) annotation(
              Line(points = {{30, -20}, {90, -20}, {90, -1.33227e-015}}, color = {0, 0, 0}, smooth = Smooth.None));
            connect(damperCage.heatPort, internalThermalPort.heatPortRotorWinding) annotation(
              Line(points = {{-10, -40}, {-10, -80}, {0, -80}, {0, -80}}, color = {191, 0, 0}, smooth = Smooth.None));
            connect(internalSupport, permanentMagnet.support) annotation(
              Line(points = {{60, -100}, {60, -100}, {60, -90}, {30, -90}, {30, -40}, {30, -40}}, color = {0, 0, 0}, smooth = Smooth.None));
            annotation(
              defaultComponentName = "smpm",
              Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-130, 10}, {-100, -10}}, lineColor = {0, 0, 0}, fillColor = {0, 255, 0}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-100, 10}, {-70, -10}}, lineColor = {0, 0, 0}, fillColor = {255, 0, 0}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-134, 34}, {-66, -34}}, lineColor = {0, 0, 255})}),
              Documentation(info = "<HTML>
<p><b>Model of a three phase permanent magnet synchronous induction machine.</b><br>
Resistance and stray inductance of stator is modeled directly in stator phases, then using space phasor transformation and a rotor-fixed <i>AirGap</i> model. Resistance and stray inductance of rotor's squirrel cage is modeled in two axis of the rotor-fixed coordinate system. Permanent magnet excitation is modelled by a constant equivalent excitation current feeding the d-axis. The machine models take the following loss effects into account:
</p>

<ul>
<li>heat losses in the temperature dependent stator winding resistances</li>
<li>optional, when enabled: heat losses in the temperature dependent damper cage resistances</li>
<li>friction losses</li>
<li>core losses (only eddy current losses, no hysteresis losses)</li>
<li>stray load losses</li>
<li>permanent magnet losses</li>
</ul>

<p>Whether a damper cage is present or not, can be selected with Boolean parameter useDamperCage (default = true).
<br><b>Default values for machine's parameters (a realistic example) are:</b><br>
<table>
<tr>
<td valign=\"top\">number of pole pairs p</td>
<td valign=\"top\">2</td><td valign=\"top\"> </td>
</tr>
<tr>
<td valign=\"top\">stator's moment of inertia</td>
<td valign=\"top\">0.29</td><td valign=\"top\">kg.m2</td>
</tr>
<tr>
<td valign=\"top\">rotor's moment of inertia</td>
<td valign=\"top\">0.29</td><td valign=\"top\">kg.m2</td>
</tr>
<tr>
<td valign=\"top\">nominal frequency fNominal</td>
<td valign=\"top\">50</td><td valign=\"top\">Hz</td>
</tr>
<tr>
<td valign=\"top\">nominal voltage per phase</td>
<td valign=\"top\">100</td><td valign=\"top\">V RMS</td>
</tr>
<tr>
<td valign=\"top\">no-load voltage per phase</td>
<td valign=\"top\">112.3</td><td valign=\"top\">V RMS @ nominal speed</td>
</tr>
<tr>
<td valign=\"top\">nominal current per phase</td>
<td valign=\"top\">100</td><td valign=\"top\">A RMS</td>
</tr>
<tr>
<td valign=\"top\">nominal torque</td>
<td valign=\"top\">181.4</td><td valign=\"top\">Nm</td>
</tr>
<tr>
<td valign=\"top\">nominal speed</td>
<td valign=\"top\">1500</td><td valign=\"top\">rpm</td>
</tr>
<tr>
<td valign=\"top\">nominal mechanical output</td>
<td valign=\"top\">28.5</td><td valign=\"top\">kW</td>
</tr>
<tr>
<td valign=\"top\">nominal rotor angle</td>
<td valign=\"top\">20.75</td><td valign=\"top\">degree</td>
</tr>
<tr>
<td valign=\"top\">efficiency</td>
<td valign=\"top\">95.0</td><td valign=\"top\">%</td>
</tr>
<tr>
<td valign=\"top\">power factor</td>
<td valign=\"top\">0.98</td><td valign=\"top\"> </td>
</tr>
<tr>
<td valign=\"top\">stator resistance</td>
<td valign=\"top\">0.03</td><td valign=\"top\">Ohm per phase at reference temperature</td>
</tr>
<tr>
<td valign=\"top\">reference temperature TsRef</td>
<td valign=\"top\">20</td><td valign=\"top\">&deg;C</td>
</tr>
<tr>
<td valign=\"top\">temperature coefficient alpha20s </td>
<td valign=\"top\">0</td><td valign=\"top\">1/K</td>
</tr>
<tr>
<td valign=\"top\">stator reactance Xd</td>
<td valign=\"top\">0.4</td><td valign=\"top\">Ohm per phase in d-axis</td>
</tr>
<tr>
<td valign=\"top\">stator reactance Xq</td>
<td valign=\"top\">0.4</td><td valign=\"top\">Ohm per phase in q-axis</td>
</tr>
<tr>
<td valign=\"top\">stator stray reactance Xss</td>
<td valign=\"top\">0.1</td><td valign=\"top\">Ohm per phase</td>
</tr>
<tr>
<td valign=\"top\">damper resistance in d-axis</td>
<td valign=\"top\">0.04</td><td valign=\"top\">Ohm at reference temperature</td>
</tr>
<tr>
<td valign=\"top\">damper resistance in q-axis</td>
<td valign=\"top\">same as d-axis</td><td valign=\"top\"> </td>
</tr>
<tr>
<td valign=\"top\">reference temperature TrRef</td>
<td valign=\"top\">20</td><td valign=\"top\">&deg;C</td>
</tr>
<tr>
<td valign=\"top\">temperature coefficient alpha20r </td>
<td valign=\"top\">0</td><td valign=\"top\">1/K</td>
</tr>
<tr>
<td valign=\"top\">damper stray reactance in d-axis XDds</td>
<td valign=\"top\">0.05</td><td valign=\"top\">Ohm</td>
</tr>
<tr>
<td valign=\"top\">damper stray reactance in q-axis XDqs</td>
<td valign=\"top\">same as d-axis</td><td valign=\"top\"> </td>
</tr>
<tr>
<td valign=\"top\">stator operational temperature TsOperational</td>
<td valign=\"top\">20</td><td valign=\"top\">&deg;C</td>
</tr>
<tr>
<td valign=\"top\">damper operational temperature TrOperational</td>
<td valign=\"top\">20</td><td valign=\"top\">&deg;C</td>
</tr>
<tr>
<td valign=\"top\">These values give the following inductances:</td>
<td valign=\"top\"> </td><td valign=\"top\"> </td>
</tr>
<tr>
<td valign=\"top\">main field inductance in d-axis</td>
<td valign=\"top\">(Xd - Xss)/(2*pi*fNominal)</td><td valign=\"top\"> </td>
</tr>
<tr>
<td valign=\"top\">main field inductance in q-axis</td>
<td valign=\"top\">(Xq - Xss)/(2*pi*fNominal)</td><td valign=\"top\"> </td>
</tr>
<tr>
<td valign=\"top\">stator stray inductance per phase</td>
<td valign=\"top\">Xss/(2*pi*fNominal)</td><td valign=\"top\"> </td>
</tr>
<tr>
<td valign=\"top\">damper stray inductance in d-axis</td>
<td valign=\"top\">XDds/(2*pi*fNominal)</td><td valign=\"top\"> </td>
</tr>
<tr>
<td valign=\"top\">damper stray inductance in q-axis</td>
<td valign=\"top\">XDqs/(2*pi*fNominal)</td><td valign=\"top\"> </td>
</tr>
</table>
</HTML>"));
          end SM_PermanentMagnet;

          model SM_ElectricalExcited "Electrical excited synchronous induction machine with damper cage"
            extends Machines.Interfaces.PartialBasicInductionMachine(Lssigma(start = 0.1 / (2 * pi * fsNominal)), final idq_ss = airGapR.i_ss, final idq_sr = airGapR.i_sr, final idq_rs = airGapR.i_rs, final idq_rr = airGapR.i_rr, redeclare final Machines.Thermal.SynchronousInductionMachines.ThermalAmbientSMEE thermalAmbient(final useDamperCage = useDamperCage, final Te = TeOperational, final Tr = TrOperational), redeclare final Machines.Interfaces.InductionMachines.ThermalPortSMEE thermalPort(final useDamperCage = useDamperCage), redeclare final Machines.Interfaces.InductionMachines.ThermalPortSMEE internalThermalPort(final useDamperCage = useDamperCage), redeclare final Machines.Interfaces.InductionMachines.PowerBalanceSMEE powerBalance(final lossPowerRotorWinding = damperCageLossPower, final powerExcitation = ve * ie, final lossPowerExcitation = re.LossPower, final lossPowerBrush = brush.lossPower, final lossPowerRotorCore = 0), statorCore(final w = statorCoreParameters.wRef));
            Modelica.Blocks.Interfaces.RealOutput ir[2](start = zeros(2), each final quantity = "ElectricCurrent", each final unit = "A") if useDamperCage "Damper cage currents" annotation(
              Dialog(showStartAttribute = true));
            Modelica.Blocks.Interfaces.RealOutput idq_dr[2](each stateSelect = StateSelect.prefer, each final quantity = "ElectricCurrent", each final unit = "A") if useDamperCage "Damper space phasor current / rotor fixed frame";
            Machines.BasicMachines.Components.AirGapR airGapR(final p = p, final Lmd = Lmd, final Lmq = Lmq, final m = m) annotation(
              Placement(transformation(origin = {0, 0}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
            parameter Modelica.SIunits.Temperature TrOperational(start = 293.15) "Operational temperature of (optional) damper cage" annotation(
              Dialog(group = "Operational temperatures", enable = not useThermalPort and useDamperCage));
            parameter Modelica.SIunits.Inductance Lmd(start = 1.5 / (2 * pi * fsNominal)) "Stator main field inductance per phase in d-axis" annotation(
              Dialog(tab = "Nominal resistances and inductances", groupImage = "modelica://Modelica/Resources/Images/Electrical/Machines/SMEE.png"));
            parameter Modelica.SIunits.Inductance Lmq(start = 1.5 / (2 * pi * fsNominal)) "Stator main field inductance per phase in q-axis" annotation(
              Dialog(tab = "Nominal resistances and inductances"));
            parameter Boolean useDamperCage(start = true) "Enable / disable damper cage" annotation(
              Evaluate = true,
              Dialog(tab = "Nominal resistances and inductances", group = "DamperCage"));
            parameter Modelica.SIunits.Inductance Lrsigmad(start = 0.05 / (2 * pi * fsNominal)) "Damper stray inductance in d-axis" annotation(
              Dialog(tab = "Nominal resistances and inductances", group = "DamperCage", enable = useDamperCage));
            parameter Modelica.SIunits.Inductance Lrsigmaq = Lrsigmad "Damper stray inductance in q-axis" annotation(
              Dialog(tab = "Nominal resistances and inductances", group = "DamperCage", enable = useDamperCage));
            parameter Modelica.SIunits.Resistance Rrd(start = 0.04) "Damper resistance in d-axis at TRef" annotation(
              Dialog(tab = "Nominal resistances and inductances", group = "DamperCage", enable = useDamperCage));
            parameter Modelica.SIunits.Resistance Rrq = Rrd "Damper resistance in q-axis at TRef" annotation(
              Dialog(tab = "Nominal resistances and inductances", group = "DamperCage", enable = useDamperCage));
            parameter Modelica.SIunits.Temperature TrRef(start = 293.15) "Reference temperature of damper resistances in d- and q-axis" annotation(
              Dialog(tab = "Nominal resistances and inductances", group = "DamperCage", enable = useDamperCage));
            parameter Machines.Thermal.LinearTemperatureCoefficient20 alpha20r(start = 0) "Temperature coefficient of damper resistances in d- and q-axis" annotation(
              Dialog(tab = "Nominal resistances and inductances", group = "DamperCage", enable = useDamperCage));
            parameter Modelica.SIunits.Voltage VsNominal(start = 100) "Nominal stator RMS voltage per phase" annotation(
              Dialog(tab = "Excitation"));
            parameter Modelica.SIunits.Current IeOpenCircuit(start = 10) "Open circuit excitation current @ nominal voltage and frequency" annotation(
              Dialog(tab = "Excitation"));
            parameter Modelica.SIunits.Resistance Re(start = 2.5) "Excitation resistance at TRef" annotation(
              Dialog(tab = "Excitation"));
            parameter Modelica.SIunits.Temperature TeRef(start = 293.15) "Reference temperature of excitation resistance" annotation(
              Dialog(tab = "Excitation"));
            parameter Machines.Thermal.LinearTemperatureCoefficient20 alpha20e(start = 0) "Temperature coefficient of excitation resistance" annotation(
              Dialog(tab = "Excitation"));
            parameter Real sigmae(min = 0, max = 0.99, start = 0.025) "Stray fraction of total excitation inductance" annotation(
              Dialog(tab = "Excitation"));
            parameter Modelica.SIunits.Temperature TeOperational(start = 293.15) "Operational excitation temperature" annotation(
              Dialog(group = "Operational temperatures", enable = not useThermalPort));
            parameter Machines.Losses.BrushParameters brushParameters "Brush loss parameter record" annotation(
              Dialog(tab = "Losses"));
            output Modelica.SIunits.Voltage ve = pin_ep.v - pin_en.v "Excitation voltage";
            output Modelica.SIunits.Current ie = pin_ep.i "Excitation current";
            Machines.BasicMachines.Components.DamperCage damperCage(final Lrsigmad = Lrsigmad, final Lrsigmaq = Lrsigmaq, final Rrd = Rrd, final Rrq = Rrq, final T_ref = TrRef, final alpha = Machines.Thermal.convertAlpha(alpha20r, TrRef), final T = TrRef, final useHeatPort = true) if useDamperCage annotation(
              Placement(transformation(origin = {0, -40}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
            Machines.BasicMachines.Components.ElectricalExcitation electricalExcitation(final turnsRatio = turnsRatio) annotation(
              Placement(transformation(origin = {-70, -50}, extent = {{-10, 10}, {10, -10}}, rotation = 180)));
            Modelica.Electrical.Analog.Basic.Resistor re(final R = Re, final T_ref = TeRef, final alpha = Machines.Thermal.convertAlpha(alpha20e, TeRef), final useHeatPort = true, final T = TeRef) annotation(
              Placement(transformation(origin = {-80, 10}, extent = {{-10, 10}, {10, -10}}, rotation = 270)));
            Modelica.Electrical.Analog.Basic.Inductor lesigma(final L = Lesigma) annotation(
              Placement(transformation(extent = {{10, -10}, {-10, 10}}, rotation = 90, origin = {-80, -20})));
            Modelica.Electrical.Analog.Interfaces.PositivePin pin_ep "Positive excitation pin" annotation(
              Placement(transformation(extent = {{-110, 70}, {-90, 50}}, rotation = 0)));
            Modelica.Electrical.Analog.Interfaces.NegativePin pin_en "Negative excitation pin" annotation(
              Placement(transformation(extent = {{-90, -50}, {-110, -70}}, rotation = 0)));
            Machines.Losses.DCMachines.Brush brush(final brushParameters = brushParameters, final useHeatPort = true) annotation(
              Placement(transformation(extent = {{10, -10}, {-10, 10}}, rotation = 90, origin = {-80, 40})));
          protected
            final parameter Real turnsRatio = sqrt(2) * VsNominal / (2 * pi * fsNominal * Lmd * IeOpenCircuit) "Stator current / excitation current";
            final parameter Modelica.SIunits.Inductance Lesigma = Lmd * turnsRatio ^ 2 * 3 / 2 * sigmae / (1 - sigmae);
            Modelica.Blocks.Interfaces.RealOutput damperCageLossPower(final quantity = "Power", final unit = "W") "Damper losses";
          equation
            connect(ir, damperCage.i);
            connect(idq_dr, damperCage.i);
            connect(damperCageLossPower, damperCage.lossPower);
            if not useDamperCage then
              damperCageLossPower = 0;
            end if;
            connect(airGapR.spacePhasor_r, damperCage.spacePhasor_r) annotation(
              Line(points = {{10, -10}, {10, -30}}, color = {0, 0, 255}));
            connect(airGapR.spacePhasor_r, electricalExcitation.spacePhasor_r) annotation(
              Line(points = {{10, -10}, {10, -10}, {10, -20}, {-60, -20}, {-60, -40}}, color = {0, 0, 255}));
            connect(electricalExcitation.pin_en, pin_en) annotation(
              Line(points = {{-80, -60}, {-100, -60}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(pin_ep, brush.p) annotation(
              Line(points = {{-100, 60}, {-80, 60}, {-80, 50}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(brush.n, re.p) annotation(
              Line(points = {{-80, 30}, {-80, 20}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(re.n, lesigma.p) annotation(
              Line(points = {{-80, 0}, {-80, -10}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(lesigma.n, electricalExcitation.pin_ep) annotation(
              Line(points = {{-80, -30}, {-80, -40}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(lssigma.spacePhasor_b, airGapR.spacePhasor_s) annotation(
              Line(points = {{20, 10}, {10, 10}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(brush.heatPort, internalThermalPort.heatPortBrush) annotation(
              Line(points = {{-70, 50}, {-60, 50}, {-60, 40}, {50, 40}, {50, -80}, {0, -80}}, color = {191, 0, 0}, smooth = Smooth.None));
            connect(re.heatPort, internalThermalPort.heatPortExcitation) annotation(
              Line(points = {{-70, 10}, {-60, 10}, {-60, 40}, {50, 40}, {50, -80}, {0, -80}}, color = {191, 0, 0}, smooth = Smooth.None));
            connect(airGapR.flange, inertiaRotor.flange_a) annotation(
              Line(points = {{10, 0}, {70, 0}}, color = {0, 0, 0}, smooth = Smooth.None));
            connect(airGapR.support, internalSupport) annotation(
              Line(points = {{-10, 1.77636e-015}, {-26, 0}, {-40, 0}, {-40, -90}, {60, -90}, {60, -100}}, color = {0, 0, 0}, smooth = Smooth.None));
            connect(damperCage.heatPort, internalThermalPort.heatPortRotorWinding) annotation(
              Line(points = {{-10, -40}, {-10, -80}, {0, -80}, {0, -80}}, color = {191, 0, 0}, smooth = Smooth.None));
            annotation(
              defaultComponentName = "smee",
              Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-134, 34}, {-66, -34}}, lineColor = {0, 0, 255}), Line(points = {{-100, 50}, {-100, 20}, {-130, 20}, {-130, -4}}, color = {0, 0, 255}), Line(points = {{-130, -4}, {-129, 1}, {-125, 5}, {-120, 6}, {-115, 5}, {-111, 1}, {-110, -4}}, color = {0, 0, 255}), Line(points = {{-110, -4}, {-109, 1}, {-105, 5}, {-100, 6}, {-95, 5}, {-91, 1}, {-90, -4}}, color = {0, 0, 255}), Line(points = {{-90, -4}, {-89, 1}, {-85, 5}, {-80, 6}, {-75, 5}, {-71, 1}, {-70, -4}}, color = {0, 0, 255}), Line(points = {{-100, -50}, {-100, -20}, {-70, -20}, {-70, -2}}, color = {0, 0, 255})}),
              Documentation(info = "<HTML>
<p><b>Model of a three phase electrical excited synchronous induction machine with damper cage.</b><br>
Resistance and stray inductance of stator is modeled directly in stator phases, then using space phasor transformation and a rotor-fixed <i>AirGap</i> model. Resistance and stray inductance of rotor's squirrel cage is modeled in two axis of the rotor-fixed coordinate system. Electrical excitation is modelled by converting excitation current and voltage to d-axis space phasors. The machine models take the following loss effects into account:
</p>

<ul>
<li>heat losses in the temperature dependent stator winding resistances</li>
<li>heat losses in the temperature dependent excitation winding resistance</li>
<li>optional, when enabled: heat losses in the temperature dependent damper cage resistances</li>
<li>brush losses in the excitation circuit</li>
<li>friction losses</li>
<li>core losses (only eddy current losses, no hysteresis losses)</li>
<li>stray load losses</li>
</ul>

<p>Whether a damper cage is present or not, can be selected with Boolean parameter useDamperCage (default = true).
<br><b>Default values for machine's parameters (a realistic example) are:</b><br>
<table>
<tr>
<td valign=\"top\">number of pole pairs p</td>
<td valign=\"top\">2</td><td valign=\"top\"> </td>
</tr>
<tr>
<td valign=\"top\">stator's moment of inertia</td>
<td valign=\"top\">0.29</td><td valign=\"top\">kg.m2</td>
</tr>
<tr>
<td valign=\"top\">rotor's moment of inertia</td>
<td valign=\"top\">0.29</td><td valign=\"top\">kg.m2</td>
</tr>
<tr>
<td valign=\"top\">nominal frequency fNominal</td>
<td valign=\"top\">50</td><td valign=\"top\">Hz</td>
</tr>
<tr>
<td valign=\"top\">nominal voltage per phase</td>
<td valign=\"top\">100</td><td valign=\"top\">V RMS</td>
</tr>
<tr>
<td valign=\"top\">no-load excitation current<br>
    @ nominal voltage and frequency</td>
<td valign=\"top\">10</td><td valign=\"top\">A DC</td>
</tr>
<tr>
<td valign=\"top\">warm excitation resistance</td>
<td valign=\"top\">2.5</td><td valign=\"top\">Ohm</td>
</tr>
<tr>
<td valign=\"top\">nominal current per phase</td>
<td valign=\"top\">100</td><td valign=\"top\">A RMS</td>
</tr>
<tr>
<td valign=\"top\">nominal apparent power</td>
<td valign=\"top\">-30000</td><td valign=\"top\">VA</td>
</tr>
<tr>
<td valign=\"top\">power factor</td>
<td valign=\"top\">-1.0</td><td valign=\"top\">ind./cap.</td>
</tr>
<tr>
<td valign=\"top\">nominal excitation current</td>
<td valign=\"top\">19</td><td valign=\"top\">A</td>
</tr>
<tr>
<td valign=\"top\">efficiency w/o excitation</td>
<td valign=\"top\">97.1</td><td valign=\"top\">%</td>
</tr>
<tr>
<td valign=\"top\">nominal torque</td>
<td valign=\"top\">-196.7</td><td valign=\"top\">Nm</td>
</tr>
<tr>
<td valign=\"top\">nominal speed</td>
<td valign=\"top\">1500</td><td valign=\"top\">rpm</td>
</tr>
<tr>
<td valign=\"top\">nominal rotor angle</td>
<td valign=\"top\">-57.23</td><td valign=\"top\">degree</td>
</tr>
<tr>
<td valign=\"top\">stator resistance</td>
<td valign=\"top\">0.03</td><td valign=\"top\">Ohm per phase at reference temperature</td>
</tr>
<tr>
<td valign=\"top\">reference temperature TsRef</td>
<td valign=\"top\">20</td><td valign=\"top\">&deg;C</td>
</tr>
<tr>
<td valign=\"top\">temperature coefficient alpha20s </td>
<td valign=\"top\">0</td><td valign=\"top\">1/K</td>
</tr>
<tr>
<td valign=\"top\">stator reactance Xd</td>
<td valign=\"top\">1.6</td><td valign=\"top\">Ohm per phase in d-axis</td>
</tr>
<tr>
<td valign=\"top\">giving Kc</td>
<td valign=\"top\">0.625</td><td valign=\"top\"> </td>
</tr>
<tr>
<td valign=\"top\">stator reactance Xq</td>
<td valign=\"top\">1.6</td><td valign=\"top\">Ohm per phase in q-axis</td>
</tr>
<tr>
<td valign=\"top\">stator stray reactance Xss</td>
<td valign=\"top\">0.1</td><td valign=\"top\">Ohm per phase</td>
</tr>
<tr>
<td valign=\"top\">damper resistance in d-axis</td>
<td valign=\"top\">0.04</td><td valign=\"top\">Ohm at reference temperature</td>
</tr>
<tr>
<td valign=\"top\">damper resistance in q-axis</td>
<td valign=\"top\">same as d-axis</td><td valign=\"top\"> </td>
</tr>
<tr>
<td valign=\"top\">reference temperature TrRef</td>
<td valign=\"top\">20</td><td valign=\"top\">&deg;C</td>
</tr>
<tr>
<td valign=\"top\">temperature coefficient alpha20r </td>
<td valign=\"top\">0</td><td valign=\"top\">1/K</td>
</tr>
<tr>
<td valign=\"top\">damper stray reactance in d-axis XDds</td>
<td valign=\"top\">0.05</td><td valign=\"top\">Ohm</td>
</tr>
<tr>
<td valign=\"top\">damper stray reactance in q-axis XDqs</td>
<td valign=\"top\">same as d-axis</td><td valign=\"top\"> </td>
</tr>
<tr>
<td valign=\"top\">excitation resistance</td>
<td valign=\"top\">2.5</td><td valign=\"top\">Ohm at reference temperature</td>
</tr>
<tr>
<td valign=\"top\">reference temperature TeRef</td>
<td valign=\"top\">20</td><td valign=\"top\">&deg;C</td>
</tr>
<tr>
<td valign=\"top\">temperature coefficient alpha20e </td>
<td valign=\"top\">0</td><td valign=\"top\">1/K</td>
</tr>
<tr>
<td valign=\"top\">excitation stray inductance</td>
<td valign=\"top\">2.5</td><td valign=\"top\">% of total excitation inductance</td>
</tr>
<tr>
<td valign=\"top\">stator operational temperature TsOperational</td>
<td valign=\"top\">20</td><td valign=\"top\">&deg;C</td>
</tr>
<tr>
<td valign=\"top\">damper operational temperature TrOperational</td>
<td valign=\"top\">20</td><td valign=\"top\">&deg;C</td>
</tr>
<tr>
<td valign=\"top\">excitation operational temperature TeOperational</td>
<td valign=\"top\">20</td><td valign=\"top\">&deg;C</td>
</tr>
<tr>
<td valign=\"top\">These values give the following inductances:</td>
<td valign=\"top\"> </td><td valign=\"top\"> </td>
</tr>
<tr>
<td valign=\"top\">main field inductance in d-axis</td>
<td valign=\"top\">(Xd - Xss)/(2*pi*fNominal)</td><td valign=\"top\"> </td>
</tr>
<tr>
<td valign=\"top\">main field inductance in q-axis</td>
<td valign=\"top\">(Xq - Xss)/(2*pi*fNominal)</td><td valign=\"top\"> </td>
</tr>
<tr>
<td valign=\"top\">stator stray inductance per phase</td>
<td valign=\"top\">Xss/(2*pi*fNominal)</td><td valign=\"top\"> </td>
</tr>
<tr>
<td valign=\"top\">damper stray inductance in d-axis</td>
<td valign=\"top\">XDds/(2*pi*fNominal)</td><td valign=\"top\"> </td>
</tr>
<tr>
<td valign=\"top\">damper stray inductance in q-axis</td>
<td valign=\"top\">XDqs/(2*pi*fNominal)</td><td valign=\"top\"> </td>
</tr>
</table>
</HTML>"));
          end SM_ElectricalExcited;

          model SM_ReluctanceRotor "Synchronous induction machine with reluctance rotor and damper cage"
            extends Machines.Interfaces.PartialBasicInductionMachine(Lssigma(start = 0.1 / (2 * pi * fsNominal)), final idq_ss = airGapR.i_ss, final idq_sr = airGapR.i_sr, final idq_rs = airGapR.i_rs, final idq_rr = airGapR.i_rr, redeclare final Machines.Thermal.SynchronousInductionMachines.ThermalAmbientSMR thermalAmbient(final useDamperCage = useDamperCage, final Tr = TrOperational), redeclare final Machines.Interfaces.InductionMachines.ThermalPortSMR thermalPort(final useDamperCage = useDamperCage), redeclare final Machines.Interfaces.InductionMachines.ThermalPortSMR internalThermalPort(final useDamperCage = useDamperCage), redeclare final Machines.Interfaces.InductionMachines.PowerBalanceSMR powerBalance(final lossPowerRotorWinding = damperCageLossPower, final lossPowerRotorCore = 0), statorCore(final w = statorCoreParameters.wRef));
            Modelica.Blocks.Interfaces.RealOutput ir[2](start = zeros(2), each final quantity = "ElectricCurrent", each final unit = "A") if useDamperCage "Damper cage currents" annotation(
              Dialog(showStartAttribute = true));
            Modelica.Blocks.Interfaces.RealOutput idq_dr[2](each stateSelect = StateSelect.prefer, each final quantity = "ElectricCurrent", each final unit = "A") if useDamperCage "Damper space phasor current / rotor fixed frame";
            Machines.BasicMachines.Components.AirGapR airGapR(final p = p, final Lmd = Lmd, final Lmq = Lmq, final m = m) annotation(
              Placement(transformation(origin = {0, 0}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
            parameter Modelica.SIunits.Temperature TrOperational(start = 293.15) "Operational temperature of (optional) damper cage" annotation(
              Dialog(group = "Operational temperatures", enable = not useThermalPort and useDamperCage));
            parameter Modelica.SIunits.Inductance Lmd(start = 2.9 / (2 * pi * fsNominal)) "Stator main field inductance per phase in d-axis" annotation(
              Dialog(tab = "Nominal resistances and inductances", groupImage = "modelica://Modelica/Resources/Images/Electrical/Machines/SMR.png"));
            parameter Modelica.SIunits.Inductance Lmq(start = 0.9 / (2 * pi * fsNominal)) "Stator main field inductance per phase in q-axis" annotation(
              Dialog(tab = "Nominal resistances and inductances"));
            parameter Boolean useDamperCage(start = true) "Enable / disable damper cage" annotation(
              Evaluate = true,
              Dialog(tab = "Nominal resistances and inductances", group = "DamperCage"));
            parameter Modelica.SIunits.Inductance Lrsigmad(start = 0.05 / (2 * pi * fsNominal)) "Damper stray inductance in d-axis" annotation(
              Dialog(tab = "Nominal resistances and inductances", group = "DamperCage", enable = useDamperCage));
            parameter Modelica.SIunits.Inductance Lrsigmaq = Lrsigmad "Damper stray inductance in q-axis" annotation(
              Dialog(tab = "Nominal resistances and inductances", group = "DamperCage", enable = useDamperCage));
            parameter Modelica.SIunits.Resistance Rrd(start = 0.04) "Damper resistance in d-axis at TRef" annotation(
              Dialog(tab = "Nominal resistances and inductances", group = "DamperCage", enable = useDamperCage));
            parameter Modelica.SIunits.Resistance Rrq = Rrd "Damper resistance in q-axis at TRef" annotation(
              Dialog(tab = "Nominal resistances and inductances", group = "DamperCage", enable = useDamperCage));
            parameter Modelica.SIunits.Temperature TrRef(start = 293.15) "Reference temperature of damper resistances in d- and q-axis" annotation(
              Dialog(tab = "Nominal resistances and inductances", group = "DamperCage", enable = useDamperCage));
            parameter Machines.Thermal.LinearTemperatureCoefficient20 alpha20r(start = 0) "Temperature coefficient of damper resistances in d- and q-axis" annotation(
              Dialog(tab = "Nominal resistances and inductances", group = "DamperCage", enable = useDamperCage));
            Machines.BasicMachines.Components.DamperCage damperCage(final Lrsigmad = Lrsigmad, final Lrsigmaq = Lrsigmaq, final Rrd = Rrd, final Rrq = Rrq, final T_ref = TrRef, final alpha = Machines.Thermal.convertAlpha(alpha20r, TrRef), final T = TrRef, final useHeatPort = true) if useDamperCage annotation(
              Placement(transformation(origin = {0, -40}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          protected
            Modelica.Blocks.Interfaces.RealOutput damperCageLossPower(final quantity = "Power", final unit = "W") "Damper losses";
          equation
            connect(ir, damperCage.i);
            connect(idq_dr, damperCage.i);
            connect(damperCageLossPower, damperCage.lossPower);
            if not useDamperCage then
              damperCageLossPower = 0;
            end if;
            connect(airGapR.spacePhasor_r, damperCage.spacePhasor_r) annotation(
              Line(points = {{10, -10}, {10, -15}, {10, -30}}, color = {0, 0, 255}));
            connect(airGapR.support, internalSupport) annotation(
              Line(points = {{-10, 0}, {-40, 0}, {-40, -90}, {60, -90}, {60, -100}}, color = {0, 0, 0}, smooth = Smooth.None));
            connect(lssigma.spacePhasor_b, airGapR.spacePhasor_s) annotation(
              Line(points = {{20, 10}, {10, 10}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(airGapR.flange, inertiaRotor.flange_a) annotation(
              Line(points = {{10, 0}, {70, 0}}, color = {0, 0, 0}, smooth = Smooth.None));
            connect(damperCage.heatPort, internalThermalPort.heatPortRotorWinding) annotation(
              Line(points = {{-10, -40}, {-10, -60}, {-10, -60}, {-10, -80}, {0, -80}}, color = {191, 0, 0}, smooth = Smooth.None));
            annotation(
              defaultComponentName = "smr",
              Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-130, 10}, {-100, -10}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-100, 10}, {-70, -10}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-134, 34}, {-66, -34}}, lineColor = {0, 0, 255})}),
              Documentation(info = "<HTML>
<p><b>Model of a three phase synchronous induction machine with reluctance rotor and damper cage.</b><br>
Resistance and stray inductance of stator is modeled directly in stator phases, then using space phasor transformation. Resistance and stray inductance of rotor's squirrel cage is modeled in two axis of the rotor-fixed coordinate system. Both together connected via a rotor-fixed <i>AirGap</i> model. The machine models take the following loss effects into account:
</p>

<ul>
<li>heat losses in the temperature dependent stator winding resistances</li>
<li>optional, when enabled: heat losses in the temperature dependent damper cage resistances</li>
<li>friction losses</li>
<li>core losses (only eddy current losses, no hysteresis losses)</li>
<li>stray load losses</li>
</ul>

<p>Whether a damper cage is present or not, can be selected with Boolean parameter useDamperCage (default = true).
<br><b>Default values for machine's parameters (a realistic example) are:</b><br>
<table>
<tr>
<td valign=\"top\">number of pole pairs p</td>
<td valign=\"top\">2</td><td valign=\"top\"> </td>
</tr>
<tr>
<td valign=\"top\">stator's moment of inertia</td>
<td valign=\"top\">0.29</td><td valign=\"top\">kg.m2</td>
</tr>
<tr>
<td valign=\"top\">rotor's moment of inertia</td>
<td valign=\"top\">0.29</td><td valign=\"top\">kg.m2</td>
</tr>
<tr>
<td valign=\"top\">nominal frequency fNominal</td>
<td valign=\"top\">50</td><td valign=\"top\">Hz</td>
</tr>
<tr>
<td valign=\"top\">nominal voltage per phase</td>
<td valign=\"top\">100</td><td valign=\"top\">V RMS</td>
</tr>
<tr>
<td valign=\"top\">nominal current per phase</td>
<td valign=\"top\">50</td><td valign=\"top\">A RMS</td>
</tr>
<tr>
<td valign=\"top\">nominal torque</td>
<td valign=\"top\"> 46</td><td valign=\"top\">Nm</td>
</tr>
<tr>
<td valign=\"top\">nominal speed</td>
<td valign=\"top\">1500</td><td valign=\"top\">rpm</td>
</tr>
<tr>
<td valign=\"top\">nominal mechanical output</td>
<td valign=\"top\"> 7.23</td><td valign=\"top\">kW</td>
</tr>
<tr>
<td valign=\"top\">efficiency</td>
<td valign=\"top\">96.98</td><td valign=\"top\">%</td>
</tr>
<tr>
<td valign=\"top\">power factor</td>
<td valign=\"top\">0.497</td><td valign=\"top\"> </td>
</tr>
<tr>
<td valign=\"top\">stator resistance</td>
<td valign=\"top\">0.03</td><td valign=\"top\">Ohm per phase at reference temperature</td>
</tr>
<tr>
<td valign=\"top\">reference temperature TsRef</td>
<td valign=\"top\">20</td><td valign=\"top\">&deg;C</td>
</tr>
<tr>
<td valign=\"top\">temperature coefficient alpha20s </td>
<td valign=\"top\">0</td><td valign=\"top\">1/K</td>
</tr>
<tr>
<td valign=\"top\">rotor resistance in d-axis</td>
<td valign=\"top\">0.04</td><td valign=\"top\">Ohm at reference temperature</td>
</tr>
<tr>
<td valign=\"top\">rotor resistance in q-axis</td>
<td valign=\"top\">same as d-axis</td><td valign=\"top\"> </td>
</tr>
<tr>
<td valign=\"top\">reference temperature TrRef</td>
<td valign=\"top\">20</td><td valign=\"top\">&deg;C</td>
</tr>
<tr>
<td valign=\"top\">temperature coefficient alpha20r </td>
<td valign=\"top\">0</td><td valign=\"top\">1/K</td>
</tr>
<tr>
<td valign=\"top\">stator reactance Xsd in d-axis</td>
<td valign=\"top\">3</td><td valign=\"top\">Ohm per phase</td>
</tr>
<tr>
<td valign=\"top\">stator reactance Xsq in q-axis</td>
<td valign=\"top\">1</td><td valign=\"top\">Ohm</td>
</tr>
<tr>
<td valign=\"top\">stator stray reactance Xss</td>
<td valign=\"top\">0.1</td><td valign=\"top\">Ohm per phase</td>
</tr>
<tr>
<td valign=\"top\">rotor stray reactance in d-axis Xrds</td>
<td valign=\"top\">0.05</td><td valign=\"top\">Ohm per phase</td>
</tr>
<tr>
<td valign=\"top\">rotor stray reactance in q-axis Xrqs</td>
<td valign=\"top\">same as d-axis</td><td valign=\"top\"> </td>
</tr>
<tr>
<td valign=\"top\">stator operational temperature TsOperational</td>
<td valign=\"top\">20</td><td valign=\"top\">&deg;C</td>
</tr>
<tr>
<td valign=\"top\">damper operational temperature TrOperational</td>
<td valign=\"top\">20</td><td valign=\"top\">&deg;C</td>
</tr>
<tr>
<td valign=\"top\">These values give the following inductances:</td>
<td valign=\"top\"> </td><td valign=\"top\"> </td>
</tr>
<tr>
<td valign=\"top\">stator stray inductance per phase</td>
<td valign=\"top\">Xss/(2*pi*fNominal)</td><td valign=\"top\"> </td>
</tr>
<tr>
<td valign=\"top\">rotor stray inductance in d-axis</td>
<td valign=\"top\">Xrds/(2*pi*fNominal)</td><td valign=\"top\"> </td>
</tr>
<tr>
<td valign=\"top\">rotor stray inductance in q-axis</td>
<td valign=\"top\">Xrqs/(2*pi*fNominal)</td><td valign=\"top\"> </td>
</tr>
<tr>
<td valign=\"top\">main field inductance per phase in d-axis</td>
<td valign=\"top\">(Xsd-Xss)/(2*pi*fNominal)</td><td valign=\"top\"> </td>
</tr>
<tr>
<td valign=\"top\">main field inductance per phase in q-axis</td>
<td valign=\"top\">(Xsq-Xss)/(2*pi*fNominal)</td><td valign=\"top\"> </td>
</tr>
</table>
</HTML>"));
          end SM_ReluctanceRotor;
          annotation(
            Documentation(info = "<HTML>
This package contains models of synchronous induction machines, based on space phasor theory:
<ul>
<li>SM_PermanentMagnet: synchronous induction machine with permanent magnet excitation, with damper cage</li>
<li>SM_ElectricalExcited: synchronous induction machine with electrical excitation
    and damper cage</li>
<li>SM_ReluctanceRotor: induction machine with reluctance rotor and damper cage<br>
i.e., a squirrel cage rotor with magnetic poles due to different airgap width</li>
</ul>
These models use package SpacePhasors.
<br><b>Please keep in mind:</b><br>
<ul>
<li>We keep the same reference system as for motors, i.e.:<br>
    Positive RotorDisplacementAngle means acting as motor,<br>
    with positive electric power consumption and positive mechanical power output.
<li>ElectricalAngle = p * MechanicalAngle</li>
<li>real axis = d-axis<br>
    imaginary= q-axis</li>
<li>Voltage induced by the magnet wheel (d-axis) is located in the q-axis.</li>
</ul>
</HTML>", revisions = "<HTML>
<dl>
  <dt><b>Main Authors:</b></dt>
  <dd>
  <a href=\"http://www.haumer.at/\">Anton Haumer</a><br>
  Technical Consulting &amp; Electrical Engineering<br>
  A-3423 St.Andrae-Woerdern<br>Austria<br>
  email: <a href=\"mailto:a.haumer@haumer.at\">a.haumer@haumer.at</a>
  </dd>
  <dt><b>Copyright:</b></dt>
  <dd>Copyright &copy; 1998-2015, Modelica Association and Anton Haumer.<br>
  <i>The Modelica package is <b>free</b> software; it can be redistributed and/or modified
  under the terms of the <b>Modelica license</b>, see the license conditions
  and the accompanying <b>disclaimer</b> in the documentation of package
  Modelica in file \"Modelica/package.mo\".</i></dd>
</dl>
  <ul>
  <li> v1.02  2004/09/19 Anton Haumer</li>
  <li> v1.03  2004/09/24 Anton Haumer<br>
       consistent naming of inductors and resistors in machine models</li>
  <li> v1.1   2004/10/01 Anton Haumer<br>
       changed naming and structure<br>
       issued to Modelica Standard Library 2.1</li>
  <li> v1.2   2004/10/27 Anton Haumer<br>
       fixed a bug with support (formerly bearing)</li>
  <li> v1.3.2 2004/11/10 Anton Haumer<br>
       ReluctanceRotor moved to SynchronousMachines</li>
  <li> v1.4   2004/11/11 Anton Haumer<br>
       removed mechanical flange support<br>
       to ease the implementation of a 3D-frame in a future release</li>
  <li> v1.52  2005/10/12 Anton Haumer<br>
       added SM_ElectricalExcited</li>
  <li> v1.53  2005/10/14 Anton Haumer<br>
       introduced unsymmetrical DamperCage for Synchronous Machines</li>
  <li> v1.6.2 2005/10/23 Anton Haumer<br>
       selectable DamperCage for Synchronous Machines</li>
  <li> v1.7.1 2006/02/06 Anton Haumer<br>
       changed some naming of synchronous machines, not affecting existing models</li>
  <li> v2.2.0 2011/02/10 Anton Haumer<br>
       conditional ThermalPort for all machines</li>
  </ul>
</HTML>"));
        end SynchronousInductionMachines;

        package DCMachines "Models of DC machines"
          extends Modelica.Icons.VariantsPackage;

          model DC_PermanentMagnet "Permanent magnet DC machine"
            extends Machines.Interfaces.PartialBasicDCMachine(final ViNominal = VaNominal - Machines.Thermal.convertResistance(Ra, TaRef, alpha20a, TaNominal) * IaNominal - Machines.Losses.DCMachines.brushVoltageDrop(brushParameters, IaNominal), final psi_eNominal = Lme * IeNominal, redeclare final Machines.Thermal.DCMachines.ThermalAmbientDCPM thermalAmbient(final Tpm = TpmOperational), redeclare final Machines.Interfaces.DCMachines.ThermalPortDCPM thermalPort, redeclare final Machines.Interfaces.DCMachines.ThermalPortDCPM internalThermalPort, redeclare final Machines.Interfaces.DCMachines.PowerBalanceDCPM powerBalance(final lossPowerPermanentMagnet = 0), core(final w = airGapDC.w));
            final parameter Modelica.SIunits.Temperature TpmOperational = 293.15 "Operational temperature of permanent magnet" annotation(
              Dialog(group = "Operational temperatures"));
            Machines.BasicMachines.Components.AirGapDC airGapDC(final turnsRatio = turnsRatio, final Le = Lme, final quasiStationary = quasiStationary) annotation(
              Placement(transformation(origin = {0, 0}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
            Modelica.Electrical.Analog.Basic.Ground eGround annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 270, origin = {-20, -30})));
            Modelica.Electrical.Analog.Sources.ConstantCurrent ie(I = IeNominal) annotation(
              Placement(transformation(origin = {0, -40}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
          protected
            constant Modelica.SIunits.Inductance Lme = 1 "Field excitation inductance";
            constant Modelica.SIunits.Current IeNominal = 1 "Equivalent excitation current";
          equation
            connect(eGround.p, ie.p) annotation(
              Line(points = {{-10, -30}, {-10, -30}, {-10, -40}}, color = {0, 0, 255}));
            connect(airGapDC.pin_ep, ie.n) annotation(
              Line(points = {{10, -10}, {10, -40}}, color = {0, 0, 255}));
            connect(airGapDC.pin_en, eGround.p) annotation(
              Line(points = {{-10, -10}, {-10, -20}, {-10, -30}}, color = {0, 0, 255}));
            connect(airGapDC.pin_ap, la.n) annotation(
              Line(points = {{10, 10}, {10, 60}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(airGapDC.support, internalSupport) annotation(
              Line(points = {{-10, 0}, {-40, 0}, {-40, -90}, {60, -90}, {60, -100}}, color = {0, 0, 0}, smooth = Smooth.None));
            connect(airGapDC.flange, inertiaRotor.flange_a) annotation(
              Line(points = {{10, 0}, {36, 0}, {36, 0}, {70, 0}}, color = {0, 0, 0}, smooth = Smooth.None));
            connect(airGapDC.pin_an, brush.p) annotation(
              Line(points = {{-10, 10}, {-10, 60}}, color = {0, 0, 255}, smooth = Smooth.None));
            annotation(
              defaultComponentName = "dcpm",
              Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-130, 10}, {-100, -10}}, lineColor = {0, 0, 0}, fillColor = {0, 255, 0}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-100, 10}, {-70, -10}}, lineColor = {0, 0, 0}, fillColor = {255, 0, 0}, fillPattern = FillPattern.Solid)}),
              Documentation(info = "<HTML>
<p><b>Model of a DC Machine with permanent magnets.</b><br>
Armature resistance and inductance are modeled directly after the armature pins, then using a <i>AirGapDC</i> model. Permanent magnet excitation is modelled by a constant equivalent excitation current feeding AirGapDC. The machine models take the following loss effects into account:
</p>

<ul>
<li>heat losses in the temperature dependent armature winding resistance</li>
<li>brush losses in the armature circuit</li>
<li>friction losses</li>
<li>core losses (only eddy current losses, no hysteresis losses)</li>
<li>stray load losses</li>
</ul>

<p>No saturation is modelled.
<br><b>Default values for machine's parameters (a realistic example) are:</b><br>
<table>
<tr>
<td valign=\"top\">stator's moment of inertia</td>
<td valign=\"top\">0.29</td><td valign=\"top\">kg.m2</td>
</tr>
<tr>
<td valign=\"top\">rotor's moment of inertia</td>
<td valign=\"top\">0.15</td><td valign=\"top\">kg.m2</td>
</tr>
<tr>
<td valign=\"top\">nominal armature voltage</td>
<td valign=\"top\">100</td><td valign=\"top\">V</td>
</tr>
<tr>
<td valign=\"top\">nominal armature current</td>
<td valign=\"top\">100</td><td valign=\"top\">A</td>
</tr>
<tr>
<td valign=\"top\">nominal speed</td>
<td valign=\"top\">1425</td><td valign=\"top\">rpm</td>
</tr>
<tr>
<td valign=\"top\">nominal torque</td>
<td valign=\"top\">63.66</td><td valign=\"top\">Nm</td>
</tr>
<tr>
<td valign=\"top\">nominal mechanical output</td>
<td valign=\"top\">9.5</td><td valign=\"top\">kW</td>
</tr>
<tr>
<td valign=\"top\">efficiency</td>
<td valign=\"top\">95.0</td><td valign=\"top\">%</td>
</tr>
<tr>
<td valign=\"top\">armature resistance</td>
<td valign=\"top\">0.05</td><td valign=\"top\">Ohm at reference temperature</td>
</tr>
<tr>
<td valign=\"top\">reference temperature TaRef</td>
<td valign=\"top\">20</td><td valign=\"top\">&deg;C</td>
</tr>
<tr>
<td valign=\"top\">temperature coefficient alpha20a </td>
<td valign=\"top\">0</td><td valign=\"top\">1/K</td>
</tr>
<tr>
<td valign=\"top\">armature inductance</td>
<td valign=\"top\">0.0015</td><td valign=\"top\">H</td>
</tr>
<tr>
<td valign=\"top\">armature nominal temperature TaNominal</td>
<td valign=\"top\">20</td><td valign=\"top\">&deg;C</td>
</tr>
<tr>
<td valign=\"top\">armature operational temperature TaOperational</td>
<td valign=\"top\">20</td><td valign=\"top\">&deg;C</td>
</tr>
</table>
Armature resistance resp. inductance include resistance resp. inductance of commutating pole winding and compensation winding, if present.
</HTML>"));
          end DC_PermanentMagnet;

          model DC_ElectricalExcited "Electrical shunt/separate excited linear DC machine"
            extends Machines.Interfaces.PartialBasicDCMachine(final ViNominal = VaNominal - Machines.Thermal.convertResistance(Ra, TaRef, alpha20a, TaNominal) * IaNominal - Machines.Losses.DCMachines.brushVoltageDrop(brushParameters, IaNominal), final psi_eNominal = Lme * IeNominal, redeclare final Machines.Thermal.DCMachines.ThermalAmbientDCEE thermalAmbient(final Te = TeOperational), redeclare final Machines.Interfaces.DCMachines.ThermalPortDCEE thermalPort, redeclare final Machines.Interfaces.DCMachines.ThermalPortDCEE internalThermalPort, redeclare final Machines.Interfaces.DCMachines.PowerBalanceDCEE powerBalance(final powerExcitation = ve * ie, final lossPowerExcitation = re.LossPower), core(final w = airGapDC.w));
            parameter Modelica.SIunits.Current IeNominal(start = 1) "Nominal excitation current" annotation(
              Dialog(tab = "Excitation"));
            parameter Modelica.SIunits.Resistance Re(start = 100) "Field excitation resistance at TRef" annotation(
              Dialog(tab = "Excitation"));
            parameter Modelica.SIunits.Temperature TeRef(start = 293.15) "Reference temperature of excitation resistance" annotation(
              Dialog(tab = "Excitation"));
            parameter Machines.Thermal.LinearTemperatureCoefficient20 alpha20e(start = 0) "Temperature coefficient of excitation resistance" annotation(
              Dialog(tab = "Excitation"));
            parameter Modelica.SIunits.Inductance Le(start = 1) "Total field excitation inductance" annotation(
              Dialog(tab = "Excitation"));
            parameter Real sigmae(min = 0, max = 0.99, start = 0) "Stray fraction of total excitation inductance" annotation(
              Dialog(tab = "Excitation"));
            parameter Modelica.SIunits.Temperature TeOperational(start = 293.15) "Operational (shunt) excitation temperature" annotation(
              Dialog(group = "Operational temperatures", enable = not useThermalPort));
            output Modelica.SIunits.Voltage ve = pin_ep.v - pin_en.v "Field excitation voltage";
            output Modelica.SIunits.Current ie(start = 0) = pin_ep.i "Field excitation current";
            Machines.BasicMachines.Components.AirGapDC airGapDC(final turnsRatio = turnsRatio, final Le = Lme, final quasiStationary = quasiStationary) annotation(
              Placement(transformation(origin = {0, 0}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
            Modelica.Electrical.Analog.Basic.Ground ground annotation(
              Placement(transformation(extent = {{-30, -30}, {-10, -10}})));
            Machines.BasicMachines.Components.CompoundDCExcitation compoundDCExcitation(final excitationTurnsRatio = 1) annotation(
              Placement(transformation(extent = {{-10, -30}, {10, -10}})));
            Modelica.Electrical.Analog.Basic.Ground groundSE annotation(
              Placement(transformation(extent = {{-30, -50}, {-10, -30}})));
            Modelica.Electrical.Analog.Basic.Resistor re(final R = Re, final T_ref = TeRef, final alpha = Machines.Thermal.convertAlpha(alpha20e, TeRef), final useHeatPort = true, final T = TeRef) annotation(
              Placement(transformation(origin = {-80, 50}, extent = {{10, -10}, {-10, 10}}, rotation = 90)));
            Machines.BasicMachines.Components.InductorDC lesigma(final L = Lesigma, final quasiStationary = quasiStationary) annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 270, origin = {-80, 20})));
            Modelica.Electrical.Analog.Interfaces.PositivePin pin_ep "Positive excitation pin" annotation(
              Placement(transformation(extent = {{-110, 70}, {-90, 50}}, rotation = 0)));
            Modelica.Electrical.Analog.Interfaces.NegativePin pin_en "Negative excitation pin" annotation(
              Placement(transformation(extent = {{-90, -50}, {-110, -70}}, rotation = 0)));
          protected
            final parameter Modelica.SIunits.Inductance Lme = Le * (1 - sigmae) "Main part of excitation inductance";
            final parameter Modelica.SIunits.Inductance Lesigma = Le * sigmae "Stray part of excitation inductance" annotation(
              Evaluate = true);
          equation
            connect(airGapDC.pin_ap, la.n) annotation(
              Line(points = {{10, 10}, {10, 60}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(airGapDC.support, internalSupport) annotation(
              Line(points = {{-10, 0}, {-40, 0}, {-40, -90}, {60, -90}, {60, -100}}, color = {0, 0, 0}, smooth = Smooth.None));
            connect(airGapDC.flange, inertiaRotor.flange_a) annotation(
              Line(points = {{10, 0}, {35, 0}, {35, 0}, {70, 0}}, color = {0, 0, 0}, smooth = Smooth.None));
            connect(re.p, pin_ep) annotation(
              Line(points = {{-80, 60}, {-100, 60}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(re.n, lesigma.p) annotation(
              Line(points = {{-80, 40}, {-80, 30}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(ground.p, airGapDC.pin_en) annotation(
              Line(points = {{-20, -10}, {-10, -10}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(airGapDC.pin_en, compoundDCExcitation.pin_n) annotation(
              Line(points = {{-10, -10}, {-10, -10}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(airGapDC.pin_ep, compoundDCExcitation.pin_p) annotation(
              Line(points = {{10, -10}, {10, -10}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(groundSE.p, compoundDCExcitation.pin_sen) annotation(
              Line(points = {{-20, -30}, {-10, -30}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(pin_en, compoundDCExcitation.pin_en) annotation(
              Line(points = {{-100, -60}, {2, -60}, {2, -30}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(compoundDCExcitation.pin_ep, lesigma.n) annotation(
              Line(points = {{10, -29.8}, {10, -40}, {-80, -40}, {-80, 10}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(airGapDC.pin_an, brush.p) annotation(
              Line(points = {{-10, 10}, {-10, 60}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(re.heatPort, internalThermalPort.heatPortExcitation) annotation(
              Line(points = {{-70, 50}, {-60, 50}, {-60, 40}, {50, 40}, {50, -80}, {0, -80}}, color = {191, 0, 0}, smooth = Smooth.None));
            annotation(
              defaultComponentName = "dcee",
              Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-130, -4}, {-129, 1}, {-125, 5}, {-120, 6}, {-115, 5}, {-111, 1}, {-110, -4}}, color = {0, 0, 255}), Line(points = {{-110, -4}, {-109, 1}, {-105, 5}, {-100, 6}, {-95, 5}, {-91, 1}, {-90, -4}}, color = {0, 0, 255}), Line(points = {{-90, -4}, {-89, 1}, {-85, 5}, {-80, 6}, {-75, 5}, {-71, 1}, {-70, -4}}, color = {0, 0, 255}), Line(points = {{-100, -50}, {-100, -20}, {-70, -20}, {-70, -2}}, color = {0, 0, 255}), Line(points = {{-100, 50}, {-100, 20}, {-130, 20}, {-130, -4}}, color = {0, 0, 255})}),
              Documentation(info = "<HTML>
<p><b>Model of a DC Machine with electrical shunt or separate excitation.</b><br>
Armature resistance and inductance are modeled directly after the armature pins, then using a <i>AirGapDC</i> model.<br>
The machine models take the following loss effects into account:
</p>

<ul>
<li>heat losses in the temperature dependent armature winding resistance</li>
<li>heat losses in the temperature dependent excitation winding resistance</li>
<li>brush losses in the armature circuit</li>
<li>friction losses</li>
<li>core losses (only eddy current losses, no hysteresis losses)</li>
<li>stray load losses</li>
</ul>

<p>No saturation is modelled.<br>
Shunt or separate excitation is defined by the user's external circuit.
<br><b>Default values for machine's parameters (a realistic example) are:</b><br>
<table>
<tr>
<td valign=\"top\">stator's moment of inertia</td>
<td valign=\"top\">0.29</td><td valign=\"top\">kg.m2</td>
</tr>
<tr>
<td valign=\"top\">rotor's moment of inertia</td>
<td valign=\"top\">0.15</td><td valign=\"top\">kg.m2</td>
</tr>
<tr>
<td valign=\"top\">nominal armature voltage</td>
<td valign=\"top\">100</td><td valign=\"top\">V</td>
</tr>
<tr>
<td valign=\"top\">nominal armature current</td>
<td valign=\"top\">100</td><td valign=\"top\">A</td>
</tr>
<tr>
<td valign=\"top\">nominal torque</td>
<td valign=\"top\">63.66</td><td valign=\"top\">Nm</td>
</tr>
<tr>
<td valign=\"top\">nominal speed</td>
<td valign=\"top\">1425</td><td valign=\"top\">rpm</td>
</tr>
<tr>
<td valign=\"top\">nominal mechanical output</td>
<td valign=\"top\">9.5</td><td valign=\"top\">kW</td>
</tr>
<tr>
<td valign=\"top\">efficiency</td>
<td valign=\"top\">95.0</td><td valign=\"top\">% only armature</td>
</tr>
<tr>
<td valign=\"top\">efficiency</td>
<td valign=\"top\">94.06</td><td valign=\"top\">% including excitation</td>
</tr>
<tr>
<td valign=\"top\">armature resistance</td>
<td valign=\"top\">0.05</td><td valign=\"top\">Ohm at reference temperature</td>
</tr>
<tr>
<td valign=\"top\">reference temperature TaRef</td>
<td valign=\"top\">20</td><td valign=\"top\">&deg;C</td>
</tr>
<tr>
<td valign=\"top\">temperature coefficient alpha20a </td>
<td valign=\"top\">0</td><td valign=\"top\">1/K</td>
</tr>
<tr>
<td valign=\"top\">armature inductance</td>
<td valign=\"top\">0.0015</td><td valign=\"top\">H</td>
</tr>
<tr>
<td valign=\"top\">nominal excitation voltage</td>
<td valign=\"top\">100</td><td valign=\"top\">V</td>
</tr>
<tr>
<td valign=\"top\">nominal excitation current</td>
<td valign=\"top\">1</td><td valign=\"top\">A</td>
</tr>
<tr>
<td valign=\"top\">excitation resistance</td>
<td valign=\"top\">100</td><td valign=\"top\">Ohm at reference temperature</td>
</tr>
<tr>
<td valign=\"top\">reference temperature TeRef</td>
<td valign=\"top\">20</td><td valign=\"top\">&deg;C</td>
</tr>
<tr>
<td valign=\"top\">temperature coefficient alpha20e </td>
<td valign=\"top\">0</td><td valign=\"top\">1/K</td>
</tr>
<tr>
<td valign=\"top\">excitation inductance</td>
<td valign=\"top\">1</td><td valign=\"top\">H</td>
</tr>
<tr>
<td valign=\"top\">stray part of excitation inductance</td>
<td valign=\"top\">0</td><td valign=\"top\"> </td>
</tr>
<tr>
<td valign=\"top\">armature nominal temperature TaNominal</td>
<td valign=\"top\">20</td><td valign=\"top\">&deg;C</td>
</tr>
<tr>
<td valign=\"top\">armature operational temperature TaOperational</td>
<td valign=\"top\">20</td><td valign=\"top\">&deg;C</td>
</tr>
<tr>
<td valign=\"top\">(shunt) excitation operational temperature TeOperational</td>
<td valign=\"top\">20</td><td valign=\"top\">&deg;C</td>
</tr>
</table>
Armature resistance resp. inductance include resistance resp. inductance of commutating pole winding and
compensation winding, if present.<br>
Armature current does not cover excitation current of a shunt excitation; in this case total current drawn from the grid = armature current + excitation current.
</HTML>"));
          end DC_ElectricalExcited;

          model DC_SeriesExcited "Series excited linear DC machine"
            extends Machines.Interfaces.PartialBasicDCMachine(wNominal(start = 1410 * 2 * pi / 60), final ViNominal = VaNominal - (Machines.Thermal.convertResistance(Ra, TaRef, alpha20a, TaNominal) + Machines.Thermal.convertResistance(Re, TeRef, alpha20e, TeNominal)) * IaNominal - Machines.Losses.DCMachines.brushVoltageDrop(brushParameters, IaNominal), final psi_eNominal = Lme * abs(IaNominal), redeclare final Machines.Thermal.DCMachines.ThermalAmbientDCSE thermalAmbient(final Tse = TeOperational), redeclare final Machines.Interfaces.DCMachines.ThermalPortDCSE thermalPort, redeclare final Machines.Interfaces.DCMachines.ThermalPortDCSE internalThermalPort, redeclare final Machines.Interfaces.DCMachines.PowerBalanceDCSE powerBalance(final powerSeriesExcitation = ve * ie, final lossPowerSeriesExcitation = re.LossPower), core(final w = airGapDC.w));
            parameter Modelica.SIunits.Resistance Re(start = 0.01) "Series excitation resistance at TRef" annotation(
              Dialog(tab = "Excitation"));
            parameter Modelica.SIunits.Temperature TeRef(start = 293.15) "Reference temperature of excitation resistance" annotation(
              Dialog(tab = "Excitation"));
            parameter Machines.Thermal.LinearTemperatureCoefficient20 alpha20e(start = 0) "Temperature coefficient of excitation resistance" annotation(
              Dialog(tab = "Excitation"));
            parameter Modelica.SIunits.Inductance Le(start = 0.0005) "Total field excitation inductance" annotation(
              Dialog(tab = "Excitation"));
            parameter Real sigmae(min = 0, max = 0.99, start = 0) "Stray fraction of total excitation inductance" annotation(
              Dialog(tab = "Excitation"));
            parameter Modelica.SIunits.Temperature TeNominal(start = 293.15) "Nominal series excitation temperature" annotation(
              Dialog(tab = "Nominal parameters"));
            parameter Modelica.SIunits.Temperature TeOperational(start = 293.15) "Operational series excitation temperature" annotation(
              Dialog(group = "Operational temperatures", enable = not useThermalPort));
            output Modelica.SIunits.Voltage ve = pin_ep.v - pin_en.v "Field excitation voltage";
            output Modelica.SIunits.Current ie = pin_ep.i "Field excitation current";
            Machines.BasicMachines.Components.AirGapDC airGapDC(final turnsRatio = turnsRatio, final Le = Lme, final quasiStationary = quasiStationary) annotation(
              Placement(transformation(origin = {0, 0}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
            Machines.BasicMachines.Components.CompoundDCExcitation compoundDCExcitation(final excitationTurnsRatio = 1) annotation(
              Placement(transformation(extent = {{-10, -30}, {10, -10}})));
            Modelica.Electrical.Analog.Basic.Ground ground annotation(
              Placement(transformation(extent = {{-30, -30}, {-10, -10}})));
            Modelica.Electrical.Analog.Basic.Ground groundE annotation(
              Placement(transformation(extent = {{0, -60}, {20, -40}})));
            Modelica.Electrical.Analog.Basic.Resistor re(final R = Re, final T_ref = TeRef, final alpha = Machines.Thermal.convertAlpha(alpha20e, TeRef), final useHeatPort = true, final T = TeRef) annotation(
              Placement(transformation(origin = {-80, 50}, extent = {{-10, 10}, {10, -10}}, rotation = 270)));
            Machines.BasicMachines.Components.InductorDC lesigma(final L = Lesigma, final quasiStationary = quasiStationary) annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 270, origin = {-80, 20})));
            Modelica.Electrical.Analog.Interfaces.PositivePin pin_ep "Positive series excitation pin" annotation(
              Placement(transformation(extent = {{-110, 70}, {-90, 50}}, rotation = 0)));
            Modelica.Electrical.Analog.Interfaces.NegativePin pin_en "Negative series excitation pin" annotation(
              Placement(transformation(extent = {{-90, -50}, {-110, -70}}, rotation = 0)));
          protected
            final parameter Modelica.SIunits.Inductance Lme = Le * (1 - sigmae) "Main part of excitation inductance";
            final parameter Modelica.SIunits.Inductance Lesigma = Le * sigmae "Stray part of excitation inductance" annotation(
              Evaluate = true);
          equation
            connect(airGapDC.pin_ap, la.n) annotation(
              Line(points = {{10, 10}, {10, 60}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(airGapDC.support, internalSupport) annotation(
              Line(points = {{-10, 0}, {-40, 0}, {-40, -90}, {60, -90}, {60, -100}}, color = {0, 0, 0}, smooth = Smooth.None));
            connect(airGapDC.flange, inertiaRotor.flange_a) annotation(
              Line(points = {{10, 0}, {70, 0}, {70, 0}}, color = {0, 0, 0}, smooth = Smooth.None));
            connect(pin_ep, re.p) annotation(
              Line(points = {{-100, 60}, {-80, 60}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(re.n, lesigma.p) annotation(
              Line(points = {{-80, 40}, {-80, 30}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(airGapDC.pin_en, compoundDCExcitation.pin_n) annotation(
              Line(points = {{-10, -10}, {-10, -10}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(compoundDCExcitation.pin_p, airGapDC.pin_ep) annotation(
              Line(points = {{10, -10}, {10, -10}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(airGapDC.pin_en, ground.p) annotation(
              Line(points = {{-10, -10}, {-20, -10}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(compoundDCExcitation.pin_sen, pin_en) annotation(
              Line(points = {{-10, -30}, {-10, -60}, {-100, -60}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(compoundDCExcitation.pin_sep, lesigma.n) annotation(
              Line(points = {{-2, -30}, {-2, -40}, {-80, -40}, {-80, 10}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(compoundDCExcitation.pin_en, groundE.p) annotation(
              Line(points = {{2, -30}, {2, -40}, {10, -40}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(airGapDC.pin_an, brush.p) annotation(
              Line(points = {{-10, 10}, {-10, 60}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(re.heatPort, internalThermalPort.heatPortSeriesExcitation) annotation(
              Line(points = {{-70, 50}, {-60, 50}, {-60, 40}, {50, 40}, {50, -80}, {0, -80}}, color = {191, 0, 0}, smooth = Smooth.None));
            annotation(
              defaultComponentName = "dcse",
              Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-100, -10}, {-105, -9}, {-109, -5}, {-110, 0}, {-109, 5}, {-105, 9}, {-100, 10}}, color = {0, 0, 255}), Line(points = {{-100, -30}, {-105, -29}, {-109, -25}, {-110, -20}, {-109, -15}, {-105, -11}, {-100, -10}}, color = {0, 0, 255}), Line(points = {{-100, 10}, {-105, 11}, {-109, 15}, {-110, 20}, {-109, 25}, {-105, 29}, {-100, 30}}, color = {0, 0, 255}), Line(points = {{-100, 50}, {-100, 30}}, color = {0, 0, 255}), Line(points = {{-100, -30}, {-100, -50}}, color = {0, 0, 255})}),
              Documentation(info = "<HTML>
<p><b>Model of a DC Machine with series excitation.</b><br>
Armature resistance and inductance are modeled directly after the armature pins, then using a <i>AirGapDC</i> model.<br>
The machine models take the following loss effects into account:
</p>

<ul>
<li>heat losses in the temperature dependent armature winding resistance</li>
<li>heat losses in the temperature dependent excitation winding resistance</li>
<li>brush losses in the armature circuit</li>
<li>friction losses</li>
<li>core losses (only eddy current losses, no hysteresis losses)</li>
<li>stray load losses</li>
</ul>

<p>No saturation is modelled.<br>
Series excitation has to be connected by the user's external circuit.
<br><b>Default values for machine's parameters (a realistic example) are:</b><br>
<table>
<tr>
<td valign=\"top\">stator's moment of inertia</td>
<td valign=\"top\">0.29</td><td valign=\"top\">kg.m2</td>
</tr>
<tr>
<td valign=\"top\">rotor's moment of inertia</td>
<td valign=\"top\">0.15</td><td valign=\"top\">kg.m2</td>
</tr>
<tr>
<td valign=\"top\">nominal armature voltage</td>
<td valign=\"top\">100</td><td valign=\"top\">V</td>
</tr>
<tr>
<td valign=\"top\">nominal armature current</td>
<td valign=\"top\">100</td><td valign=\"top\">A</td>
</tr>
<tr>
<td valign=\"top\">nominal torque</td>
<td valign=\"top\">63.66</td><td valign=\"top\">Nm</td>
</tr>
<tr>
<td valign=\"top\">nominal speed</td>
<td valign=\"top\">1410</td><td valign=\"top\">rpm</td>
</tr>
<tr>
<td valign=\"top\">nominal mechanical output</td>
<td valign=\"top\">9.4</td><td valign=\"top\">kW</td>
</tr>
<tr>
<td valign=\"top\">efficiency</td>
<td valign=\"top\">94.0</td><td valign=\"top\">% only armature</td>
</tr>
<tr>
<td valign=\"top\">armature resistance</td>
<td valign=\"top\">0.05</td><td valign=\"top\">Ohm at reference temperature</td>
</tr>
<tr>
<td valign=\"top\">reference temperature TaRef</td>
<td valign=\"top\">20</td><td valign=\"top\">&deg;C</td>
</tr>
<tr>
<td valign=\"top\">temperature coefficient alpha20a </td>
<td valign=\"top\">0</td><td valign=\"top\">1/K</td>
</tr>
<tr>
<td valign=\"top\">armature inductance</td>
<td valign=\"top\">0.0015</td><td valign=\"top\">H</td>
</tr>
<tr>
<td valign=\"top\">excitation resistance</td>
<td valign=\"top\">0.01</td><td valign=\"top\">Ohm at reference temperature</td>
</tr>
<tr>
<td valign=\"top\">reference temperature TeRef</td>
<td valign=\"top\">20</td><td valign=\"top\">&deg;C</td>
</tr>
<tr>
<td valign=\"top\">temperature coefficient alpha20e</td>
<td valign=\"top\">0</td><td valign=\"top\">1/K</td>
</tr>
<tr>
<td valign=\"top\">excitation inductance</td>
<td valign=\"top\">0.0005</td><td valign=\"top\">H</td>
</tr>
<tr>
<td valign=\"top\">stray part of excitation inductance</td>
<td valign=\"top\">0</td><td valign=\"top\"> </td>
</tr>
<tr>
<td valign=\"top\">armature nominal temperature TaNominal</td>
<td valign=\"top\">20</td><td valign=\"top\">&deg;C</td>
</tr>
<tr>
<td valign=\"top\">series excitation nominal temperature TeNominal</td>
<td valign=\"top\">20</td><td valign=\"top\">&deg;C</td>
</tr>
<tr>
<td valign=\"top\">armature operational temperature TaOperational</td>
<td valign=\"top\">20</td><td valign=\"top\">&deg;C</td>
</tr>
<tr>
<td valign=\"top\">series excitation operational temperature TeOperational</td>
<td valign=\"top\">20</td><td valign=\"top\">&deg;C</td>
</tr>
</table>
Armature resistance resp. inductance include resistance resp. inductance of commutating pole winding and
compensation winding, if present.<br>
Parameter nominal armature voltage includes voltage drop of series excitation;<br>
but for output the voltage is split into:<br>
va = armature voltage without voltage drop of series excitation<br>
ve = voltage drop of series excitation
</HTML>"));
          end DC_SeriesExcited;
          annotation(
            Documentation(info = "<HTML>
This package contains models of DC machines:
<ul>
<li>DC_PermanentMagnet: DC machine with permanent magnet excitation</li>
<li>DC_ElectricalExcited: DC machine with electrical shunt or separate excitation</li>
<li>DC_SeriesExcited: DC machine with series excitation</li>
</ul>
</html>", revisions = "<HTML>
<dl>
  <dt><b>Main Authors:</b></dt>
  <dd>
  <a href=\"http://www.haumer.at/\">Anton Haumer</a><br>
  Technical Consulting &amp; Electrical Engineering<br>
  A-3423 St.Andrae-Woerdern<br>Austria<br>
  email: <a href=\"mailto:a.haumer@haumer.at\">a.haumer@haumer.at</a>
  </dd>
  <dt><b>Copyright:</b></dt>
  <dd>Copyright &copy; 1998-2015, Modelica Association and Anton Haumer.<br>
  <i>The Modelica package is <b>free</b> software; it can be redistributed and/or modified
  under the terms of the <b>Modelica license</b>, see the license conditions
  and the accompanying <b>disclaimer</b> in the documentation of package
  Modelica in file \"Modelica/package.mo\".</i></dd>
</dl>
  <ul>
  <li> v1.02 2004/09/19 Anton Haumer</li>
  <li> v1.03 2004/09/24 Anton Haumer<br>
       added DC machine with series excitation</li>
  <li> v1.1  2004/10/01 Anton Haumer<br>
       changed naming and structure<br>
       issued to Modelica Standard Library 2.1</li>
  <li> v1.2  2004/10/27 Anton Haumer<br>
       fixed a bug with support (formerly bearing)</li>
  <li> v1.4   2004/11/11 Anton Haumer<br>
       removed mechanical flange support<br>
       to ease the implementation of a 3D-frame in a future release</li>
  <li> v2.2.0 2011/02/10 Anton Haumer<br>
       conditional ThermalPort for all machines</li>
  </ul>
</HTML>"));
        end DCMachines;

        package QuasiStationaryDCMachines "Models of quasistationary DC machines"
          extends Modelica.Icons.VariantsPackage;

          model DC_PermanentMagnet "Quasistationary permanent magnet DC machine"
            extends Machines.BasicMachines.DCMachines.DC_PermanentMagnet(final quasiStationary = true);
            extends Machines.Icons.QuasiStationaryMachine;
            annotation(
              defaultComponentName = "dcpm",
              Documentation(info = "<HTML>
<b>Quasistaionary model of a DC Machine with permanent magnets.</b><br>
This model is fully compatible with the
<a href=\"modelica://Modelica.Electrical.Machines.BasicMachines.DCMachines.DC_PermanentMagnet\">transient machine model of a DC machine with permanent magnets</a>;
the only difference is that electrical transients are neglected.
</HTML>"));
          end DC_PermanentMagnet;

          model DC_ElectricalExcited "Quasistationary electrical shunt/separate excited linear DC machine"
            extends Machines.BasicMachines.DCMachines.DC_ElectricalExcited(final quasiStationary = true);
            extends Machines.Icons.QuasiStationaryMachine;
            annotation(
              defaultComponentName = "dcee",
              Documentation(info = "<HTML>
<b>Quasistaionary model of a DC Machine with electrical shunt or separate excitation.</b><br>
This model is fully compatible with the
<a href=\"modelica://Modelica.Electrical.Machines.BasicMachines.DCMachines.DC_ElectricalExcited\">transient machine model of a DC machine with electrical shunt or separate excitation</a>;
the only difference is that electrical transients are neglected.
</HTML>"));
          end DC_ElectricalExcited;

          model DC_SeriesExcited "Quasistationary series excited linear DC machine"
            extends Machines.BasicMachines.DCMachines.DC_SeriesExcited(final quasiStationary = true);
            extends Machines.Icons.QuasiStationaryMachine;
            annotation(
              defaultComponentName = "dcse",
              Documentation(info = "<HTML>
<b>Quasistaionary model of a DC Machine with Series excitation.</b><br>
This model is fully compatible with the
<a href=\"modelica://Modelica.Electrical.Machines.BasicMachines.DCMachines.DC_SeriesExcited\">transient machine model of a DC machine with with series excitation</a>;
the only difference is that electrical transients are neglected.
</HTML>"));
          end DC_SeriesExcited;
          annotation(
            Documentation(info = "<HTML>
<p>
This package contains quasistationary models of DC machines;
these models are fully compatible with the
<a href=\"modelica://Modelica.Electrical.Machines.BasicMachines.DCMachines\">transient machine models of DC machines</a>;
the only difference is that electrical transients are neglected.
</p>
<h4>Please note:</h4>
<p>
Quasistationary DC machine models are basically different from quasistationary induction machine models:
Quasistaionary DC machine models neglect electrical transients, i.e., setting <code>der(i) = 0</code>,
whereas quasistationary induction machine models are based on time phasor theory,
see <a href=\"modelica://Modelica.Electrical.QuasiStationary\">QuasiStationary Library</a>,
where e.g., <code>L*der(i)</code> is replaced by <code>j*omega*L*(I_re+j*I_im)</code>.
</p>
</html>", revisions = "<HTML>
<dl>
  <dt><b>Main Authors:</b></dt>
  <dd>
  <a href=\"http://www.haumer.at/\">Anton Haumer</a><br>
  Technical Consulting &amp; Electrical Engineering<br>
  A-3423 St.Andrae-Woerdern<br>Austria<br>
  email: <a href=\"mailto:a.haumer@haumer.at\">a.haumer@haumer.at</a>
  </dd>
  <dt><b>Copyright:</b></dt>
  <dd>Copyright &copy; 1998-2015, Modelica Association and Anton Haumer.<br>
  <i>The Modelica package is <b>free</b> software; it can be redistributed and/or modified
  under the terms of the <b>Modelica license</b>, see the license conditions
  and the accompanying <b>disclaimer</b> in the documentation of package
  Modelica in file \"Modelica/package.mo\".</i></dd>
</dl>
  <ul>
  <li> v2.3.0 2010/02/16 Anton Haumer<br>
       first implementation</li>
  </ul>
</HTML>"));
        end QuasiStationaryDCMachines;

        package Transformers "Library for technical 3phase transformers"
          extends Modelica.Icons.Package;

          package Yy "Transformers: primary Y / secondary y"
            extends Modelica.Icons.VariantsPackage;

            model Yy00 "Transformer Yy0"
              extends Machines.Interfaces.PartialBasicTransformer(final VectorGroup = "Yy00");
              Modelica.Electrical.MultiPhase.Basic.Star star1(final m = m) annotation(
                Placement(transformation(origin = {-10, -80}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
              Modelica.Electrical.Analog.Interfaces.NegativePin starpoint1 annotation(
                Placement(transformation(extent = {{-60, -110}, {-40, -90}}, rotation = 0)));
              Modelica.Electrical.MultiPhase.Basic.Star star2(final m = m) annotation(
                Placement(transformation(origin = {10, -80}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
              Modelica.Electrical.Analog.Interfaces.NegativePin starpoint2 annotation(
                Placement(transformation(extent = {{40, -110}, {60, -90}}, rotation = 0)));
            equation
              connect(star1.pin_n, starpoint1) annotation(
                Line(points = {{-10, -90}, {-10, -100}, {-50, -100}}, color = {0, 0, 255}));
              connect(star2.pin_n, starpoint2) annotation(
                Line(points = {{10, -90}, {10, -100}, {50, -100}}, color = {0, 0, 255}));
              connect(l1sigma.plug_n, core.plug_p1) annotation(
                Line(points = {{-50, 0}, {-50, 20}, {-10, 20}, {-10, 5}}, color = {0, 0, 255}));
              connect(core.plug_n1, star1.plug_p) annotation(
                Line(points = {{-10, -5}, {-10, -37.5}, {-10, -70}, {-10, -70}}, color = {0, 0, 255}));
              connect(core.plug_n2, core.plug_p3) annotation(
                Line(points = {{10, 4}, {10, -4}}, color = {0, 0, 255}));
              connect(core.plug_p2, l2sigma.plug_n) annotation(
                Line(points = {{10, 10}, {10, 20}, {50, 20}, {50, 0}}, color = {0, 0, 255}));
              connect(core.plug_n3, star2.plug_p) annotation(
                Line(points = {{10, -10}, {10, -70}, {10, -70}}, color = {0, 0, 255}));
              annotation(
                defaultComponentName = "transformer",
                Documentation(info = "<html>
Transformer Yy0
<br>Typical parameters see:
<a href=\"modelica://Modelica.Electrical.Machines.Interfaces.PartialBasicTransformer\">PartialBasicTransformer</a>
</html>"));
            end Yy00;

            model Yy02 "Transformer Yy2"
              extends Machines.Interfaces.PartialBasicTransformer(final VectorGroup = "Yy02");
              Modelica.Electrical.MultiPhase.Basic.Star star1(final m = m) annotation(
                Placement(transformation(origin = {-10, -80}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
              Modelica.Electrical.Analog.Interfaces.NegativePin starpoint1 annotation(
                Placement(transformation(extent = {{-60, -110}, {-40, -90}}, rotation = 0)));
              Modelica.Electrical.MultiPhase.Basic.Star star2(final m = m) annotation(
                Placement(transformation(origin = {10, -80}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
              Modelica.Electrical.Analog.Interfaces.NegativePin starpoint2 annotation(
                Placement(transformation(extent = {{40, -110}, {60, -90}}, rotation = 0)));
              Modelica.Electrical.MultiPhase.Basic.Delta Rot2(final m = m) annotation(
                Placement(transformation(extent = {{50, -30}, {30, -10}}, rotation = 0)));
            equation
              connect(star1.pin_n, starpoint1) annotation(
                Line(points = {{-10, -90}, {-10, -100}, {-50, -100}}, color = {0, 0, 255}));
              connect(star2.pin_n, starpoint2) annotation(
                Line(points = {{10, -90}, {10, -100}, {50, -100}}, color = {0, 0, 255}));
              connect(l2sigma.plug_n, Rot2.plug_p) annotation(
                Line(points = {{50, 0}, {50, -20}}, color = {0, 0, 255}));
              connect(l1sigma.plug_n, core.plug_p1) annotation(
                Line(points = {{-50, 0}, {-50, 20}, {-10, 20}, {-10, 5}}, color = {0, 0, 255}));
              connect(core.plug_n2, core.plug_p3) annotation(
                Line(points = {{10, 4}, {10, -4}, {10, -4}}, color = {0, 0, 255}));
              connect(core.plug_n1, star1.plug_p) annotation(
                Line(points = {{-10, -5}, {-10, -37.5}, {-10, -70}, {-10, -70}}, color = {0, 0, 255}));
              connect(core.plug_n3, Rot2.plug_n) annotation(
                Line(points = {{10, -10}, {10, -20}, {30, -20}}, color = {0, 0, 255}));
              connect(core.plug_p2, star2.plug_p) annotation(
                Line(points = {{10, 10}, {20, 10}, {20, -70}, {10, -70}}, color = {0, 0, 255}));
              annotation(
                defaultComponentName = "transformer",
                Documentation(info = "<html>
Transformer Yy2
<br>Typical parameters see:
<a href=\"modelica://Modelica.Electrical.Machines.Interfaces.PartialBasicTransformer\">PartialBasicTransformer</a>
</html>"));
            end Yy02;

            model Yy04 "Transformer Yy4"
              extends Machines.Interfaces.PartialBasicTransformer(final VectorGroup = "Yy04");
              Modelica.Electrical.MultiPhase.Basic.Star star1(final m = m) annotation(
                Placement(transformation(origin = {-10, -80}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
              Modelica.Electrical.Analog.Interfaces.NegativePin starpoint1 annotation(
                Placement(transformation(extent = {{-60, -110}, {-40, -90}}, rotation = 0)));
              Modelica.Electrical.MultiPhase.Basic.Star star2(final m = m) annotation(
                Placement(transformation(origin = {10, -80}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
              Modelica.Electrical.Analog.Interfaces.NegativePin starpoint2 annotation(
                Placement(transformation(extent = {{40, -110}, {60, -90}}, rotation = 0)));
              Modelica.Electrical.MultiPhase.Basic.Delta Rot2(final m = m) annotation(
                Placement(transformation(extent = {{30, 10}, {50, 30}}, rotation = 0)));
            equation
              connect(star1.pin_n, starpoint1) annotation(
                Line(points = {{-10, -90}, {-10, -100}, {-50, -100}}, color = {0, 0, 255}));
              connect(star2.pin_n, starpoint2) annotation(
                Line(points = {{10, -90}, {10, -100}, {50, -100}}, color = {0, 0, 255}));
              connect(Rot2.plug_n, l2sigma.plug_n) annotation(
                Line(points = {{50, 20}, {50, 0}}, color = {0, 0, 255}));
              connect(l1sigma.plug_n, core.plug_p1) annotation(
                Line(points = {{-50, 0}, {-50, 20}, {-10, 20}, {-10, 5}}, color = {0, 0, 255}));
              connect(core.plug_n1, star1.plug_p) annotation(
                Line(points = {{-10, -5}, {-10, -37.5}, {-10, -70}, {-10, -70}}, color = {0, 0, 255}));
              connect(core.plug_n2, core.plug_p3) annotation(
                Line(points = {{10, 4}, {10, -4}}, color = {0, 0, 255}));
              connect(core.plug_p2, Rot2.plug_p) annotation(
                Line(points = {{10, 10}, {10, 20}, {30, 20}}, color = {0, 0, 255}));
              connect(core.plug_n3, star2.plug_p) annotation(
                Line(points = {{10, -10}, {10, -70}, {10, -70}}, color = {0, 0, 255}));
              annotation(
                defaultComponentName = "transformer",
                Documentation(info = "<html>
Transformer Yy4
<br>Typical parameters see:
<a href=\"modelica://Modelica.Electrical.Machines.Interfaces.PartialBasicTransformer\">PartialBasicTransformer</a>
</html>"));
            end Yy04;

            model Yy06 "Transformer Yy6"
              extends Machines.Interfaces.PartialBasicTransformer(final VectorGroup = "Yy06");
              Modelica.Electrical.MultiPhase.Basic.Star star1(final m = m) annotation(
                Placement(transformation(origin = {-10, -80}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
              Modelica.Electrical.Analog.Interfaces.NegativePin starpoint1 annotation(
                Placement(transformation(extent = {{-60, -110}, {-40, -90}}, rotation = 0)));
              Modelica.Electrical.MultiPhase.Basic.Star star2(final m = m) annotation(
                Placement(transformation(origin = {10, -80}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
              Modelica.Electrical.Analog.Interfaces.NegativePin starpoint2 annotation(
                Placement(transformation(extent = {{40, -110}, {60, -90}}, rotation = 0)));
            equation
              connect(star1.pin_n, starpoint1) annotation(
                Line(points = {{-10, -90}, {-10, -100}, {-50, -100}}, color = {0, 0, 255}));
              connect(star2.pin_n, starpoint2) annotation(
                Line(points = {{10, -90}, {10, -100}, {50, -100}}, color = {0, 0, 255}));
              connect(l1sigma.plug_n, core.plug_p1) annotation(
                Line(points = {{-50, 0}, {-50, 20}, {-10, 20}, {-10, 5}}, color = {0, 0, 255}));
              connect(core.plug_n1, star1.plug_p) annotation(
                Line(points = {{-10, -5}, {-10, -37.5}, {-10, -70}, {-10, -70}}, color = {0, 0, 255}));
              connect(core.plug_n2, core.plug_p3) annotation(
                Line(points = {{10, 4}, {10, -4}}, color = {0, 0, 255}));
              connect(l2sigma.plug_n, core.plug_n3) annotation(
                Line(points = {{50, 0}, {50, -20}, {10, -20}, {10, -10}}, color = {0, 0, 255}));
              connect(core.plug_p2, star2.plug_p) annotation(
                Line(points = {{10, 10}, {20, 10}, {20, -70}, {10, -70}}, color = {0, 0, 255}));
              annotation(
                defaultComponentName = "transformer",
                Documentation(info = "<html>
Transformer Yy6
<br>Typical parameters see:
<a href=\"modelica://Modelica.Electrical.Machines.Interfaces.PartialBasicTransformer\">PartialBasicTransformer</a>
</html>"));
            end Yy06;

            model Yy08 "Transformer Yy8"
              extends Machines.Interfaces.PartialBasicTransformer(final VectorGroup = "Yy08");
              Modelica.Electrical.MultiPhase.Basic.Star star1(final m = m) annotation(
                Placement(transformation(origin = {-10, -80}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
              Modelica.Electrical.Analog.Interfaces.NegativePin starpoint1 annotation(
                Placement(transformation(extent = {{-60, -110}, {-40, -90}}, rotation = 0)));
              Modelica.Electrical.MultiPhase.Basic.Star star2(final m = m) annotation(
                Placement(transformation(origin = {10, -80}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
              Modelica.Electrical.Analog.Interfaces.NegativePin starpoint2 annotation(
                Placement(transformation(extent = {{40, -110}, {60, -90}}, rotation = 0)));
              Modelica.Electrical.MultiPhase.Basic.Delta Rot2(final m = m) annotation(
                Placement(transformation(extent = {{50, 10}, {30, 30}}, rotation = 0)));
            equation
              connect(star1.pin_n, starpoint1) annotation(
                Line(points = {{-10, -90}, {-10, -100}, {-50, -100}}, color = {0, 0, 255}));
              connect(star2.pin_n, starpoint2) annotation(
                Line(points = {{10, -90}, {10, -100}, {50, -100}}, color = {0, 0, 255}));
              connect(Rot2.plug_p, l2sigma.plug_n) annotation(
                Line(points = {{50, 20}, {50, 0}}, color = {0, 0, 255}));
              connect(l1sigma.plug_n, core.plug_p1) annotation(
                Line(points = {{-50, 0}, {-50, 20}, {-10, 20}, {-10, 5}}, color = {0, 0, 255}));
              connect(core.plug_n1, star1.plug_p) annotation(
                Line(points = {{-10, -5}, {-10, -37.5}, {-10, -70}, {-10, -70}}, color = {0, 0, 255}));
              connect(core.plug_n2, core.plug_p3) annotation(
                Line(points = {{10, 4}, {10, -4}}, color = {0, 0, 255}));
              connect(core.plug_p2, Rot2.plug_n) annotation(
                Line(points = {{10, 10}, {10, 20}, {30, 20}}, color = {0, 0, 255}));
              connect(core.plug_n3, star2.plug_p) annotation(
                Line(points = {{10, -10}, {10, -70}, {10, -70}, {10, -70}}, color = {0, 0, 255}));
              annotation(
                defaultComponentName = "transformer",
                Documentation(info = "<html>
Transformer Yy8
<br>Typical parameters see:
<a href=\"modelica://Modelica.Electrical.Machines.Interfaces.PartialBasicTransformer\">PartialBasicTransformer</a>
</html>"));
            end Yy08;

            model Yy10 "Transformer Yy10"
              extends Machines.Interfaces.PartialBasicTransformer(final VectorGroup = "Yy10");
              Modelica.Electrical.MultiPhase.Basic.Star star1(final m = m) annotation(
                Placement(transformation(origin = {-10, -80}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
              Modelica.Electrical.Analog.Interfaces.NegativePin starpoint1 annotation(
                Placement(transformation(extent = {{-60, -110}, {-40, -90}}, rotation = 0)));
              Modelica.Electrical.MultiPhase.Basic.Star star2(final m = m) annotation(
                Placement(transformation(origin = {10, -80}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
              Modelica.Electrical.Analog.Interfaces.NegativePin starpoint2 annotation(
                Placement(transformation(extent = {{40, -110}, {60, -90}}, rotation = 0)));
              Modelica.Electrical.MultiPhase.Basic.Delta Rot2(final m = m) annotation(
                Placement(transformation(extent = {{30, -30}, {50, -10}}, rotation = 0)));
            equation
              connect(star1.pin_n, starpoint1) annotation(
                Line(points = {{-10, -90}, {-10, -100}, {-50, -100}}, color = {0, 0, 255}));
              connect(star2.pin_n, starpoint2) annotation(
                Line(points = {{10, -90}, {10, -100}, {50, -100}}, color = {0, 0, 255}));
              connect(l2sigma.plug_n, Rot2.plug_n) annotation(
                Line(points = {{50, 0}, {50, -20}}, color = {0, 0, 255}));
              connect(l1sigma.plug_n, core.plug_p1) annotation(
                Line(points = {{-50, 0}, {-50, 20}, {-10, 20}, {-10, 5}}, color = {0, 0, 255}));
              connect(core.plug_n1, star1.plug_p) annotation(
                Line(points = {{-10, -5}, {-10, -37.5}, {-10, -70}, {-10, -70}}, color = {0, 0, 255}));
              connect(core.plug_n2, core.plug_p3) annotation(
                Line(points = {{10, 4}, {10, -4}}, color = {0, 0, 255}));
              connect(core.plug_n3, Rot2.plug_p) annotation(
                Line(points = {{10, -10}, {10, -20}, {30, -20}}, color = {0, 0, 255}));
              connect(core.plug_p2, star2.plug_p) annotation(
                Line(points = {{10, 10}, {20, 10}, {20, -70}, {10, -70}}, color = {0, 0, 255}));
              annotation(
                defaultComponentName = "transformer",
                Documentation(info = "<html>
Transformer Yy10
<br>Typical parameters see:
<a href=\"modelica://Modelica.Electrical.Machines.Interfaces.PartialBasicTransformer\">PartialBasicTransformer</a>
</html>"));
            end Yy10;
            annotation(
              Documentation(info = "<HTML>
This package contains transformers primary Y connected / secondary y connected in all possible vector groups.
</HTML>", revisions = "<HTML>
<dl>
  <dt><b>Main Authors:</b></dt>
  <dd>
  <a href=\"http://www.haumer.at/\">Anton Haumer</a><br>
  Technical Consulting &amp; Electrical Engineering<br>
  A-3423 St.Andrae-Woerdern<br>Austria<br>
  email: <a href=\"mailto:a.haumer@haumer.at\">a.haumer@haumer.at</a>
  </dd>
  <dt><b>Copyright:</b></dt>
  <dd>Copyright &copy; 1998-2015, Modelica Association and Anton Haumer.<br>
  <i>The Modelica package is <b>free</b> software; it can be redistributed and/or modified
  under the terms of the <b>Modelica license</b>, see the license conditions
  and the accompanying <b>disclaimer</b> in the documentation of package
  Modelica in file \"Modelica/package.mo\".</i></dd>
</dl>
  <ul>
  <li> v1.0.0 2006/11/19 Anton Haumer</li>
  </ul>
</HTML>"));
          end Yy;

          package Yd "Transformers: primary Y / secondary d"
            extends Modelica.Icons.VariantsPackage;

            model Yd01 "Transformer Yd1"
              extends Machines.Interfaces.PartialBasicTransformer(final VectorGroup = "Yd01");
              Modelica.Electrical.MultiPhase.Basic.Star star1(final m = m) annotation(
                Placement(transformation(origin = {-10, -80}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
              Modelica.Electrical.Analog.Interfaces.NegativePin starpoint1 annotation(
                Placement(transformation(extent = {{-60, -110}, {-40, -90}}, rotation = 0)));
              Modelica.Electrical.MultiPhase.Basic.Delta Delta2(final m = m) annotation(
                Placement(transformation(extent = {{50, -30}, {30, -10}}, rotation = 0)));
            equation
              connect(star1.pin_n, starpoint1) annotation(
                Line(points = {{-10, -90}, {-10, -100}, {-50, -100}}, color = {0, 0, 255}));
              connect(Delta2.plug_p, r2.plug_p) annotation(
                Line(points = {{50, -20}, {90, -20}, {90, 0}}, color = {0, 0, 255}));
              connect(l1sigma.plug_n, core.plug_p1) annotation(
                Line(points = {{-50, 0}, {-50, 20}, {-10, 20}, {-10, 5}}, color = {0, 0, 255}));
              connect(core.plug_n1, star1.plug_p) annotation(
                Line(points = {{-10, -5}, {-10, -70}, {-10, -70}}, color = {0, 0, 255}));
              connect(core.plug_n2, core.plug_p3) annotation(
                Line(points = {{10, 4}, {10, -4}}, color = {0, 0, 255}));
              connect(core.plug_n3, Delta2.plug_n) annotation(
                Line(points = {{10, -10}, {10, -20}, {30, -20}}, color = {0, 0, 255}));
              connect(core.plug_p2, l2sigma.plug_n) annotation(
                Line(points = {{10, 10}, {10, 20}, {50, 20}, {50, 0}}, color = {0, 0, 255}));
              annotation(
                defaultComponentName = "transformer",
                Documentation(info = "<html>
Transformer Yd1
<br>Typical parameters see:
<a href=\"modelica://Modelica.Electrical.Machines.Interfaces.PartialBasicTransformer\">PartialBasicTransformer</a>
</html>"));
            end Yd01;

            model Yd03 "Transformer Yd3"
              extends Machines.Interfaces.PartialBasicTransformer(final VectorGroup = "Yd03");
              Modelica.Electrical.MultiPhase.Basic.Star star1(final m = m) annotation(
                Placement(transformation(origin = {-10, -80}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
              Modelica.Electrical.Analog.Interfaces.NegativePin starpoint1 annotation(
                Placement(transformation(extent = {{-60, -110}, {-40, -90}}, rotation = 0)));
              Modelica.Electrical.MultiPhase.Basic.Delta Delta2(final m = m) annotation(
                Placement(transformation(extent = {{30, 10}, {50, 30}}, rotation = 0)));
              Modelica.Electrical.MultiPhase.Basic.Delta Rot2(final m = m) annotation(
                Placement(transformation(extent = {{50, -30}, {30, -10}}, rotation = 0)));
            equation
              connect(star1.pin_n, starpoint1) annotation(
                Line(points = {{-10, -90}, {-10, -100}, {-50, -100}}, color = {0, 0, 255}));
              connect(Delta2.plug_n, r2.plug_p) annotation(
                Line(points = {{50, 20}, {90, 20}, {90, 0}}, color = {0, 0, 255}));
              connect(l2sigma.plug_n, Rot2.plug_p) annotation(
                Line(points = {{50, 0}, {50, -20}}, color = {0, 0, 255}));
              connect(l1sigma.plug_n, core.plug_p1) annotation(
                Line(points = {{-50, 0}, {-50, 20}, {-10, 20}, {-10, 5}}, color = {0, 0, 255}));
              connect(core.plug_n1, star1.plug_p) annotation(
                Line(points = {{-10, -5}, {-10, -70}, {-10, -70}}, color = {0, 0, 255}));
              connect(core.plug_n2, core.plug_p3) annotation(
                Line(points = {{10, 4}, {10, -4}}, color = {0, 0, 255}));
              connect(core.plug_p2, Delta2.plug_p) annotation(
                Line(points = {{10, 10}, {10, 20}, {30, 20}}, color = {0, 0, 255}));
              connect(core.plug_n3, Rot2.plug_n) annotation(
                Line(points = {{10, -10}, {10, -20}, {30, -20}}, color = {0, 0, 255}));
              annotation(
                defaultComponentName = "transformer",
                Documentation(info = "<html>
Transformer Yd3
<br>Typical parameters see:
<a href=\"modelica://Modelica.Electrical.Machines.Interfaces.PartialBasicTransformer\">PartialBasicTransformer</a>
</html>"));
            end Yd03;

            model Yd05 "Transformer Yd5"
              extends Machines.Interfaces.PartialBasicTransformer(final VectorGroup = "Yd05");
              Modelica.Electrical.MultiPhase.Basic.Star star1(final m = m) annotation(
                Placement(transformation(origin = {-10, -80}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
              Modelica.Electrical.Analog.Interfaces.NegativePin starpoint1 annotation(
                Placement(transformation(extent = {{-60, -110}, {-40, -90}}, rotation = 0)));
              Modelica.Electrical.MultiPhase.Basic.Delta Delta2(final m = m) annotation(
                Placement(transformation(extent = {{30, 10}, {50, 30}}, rotation = 0)));
            equation
              connect(star1.pin_n, starpoint1) annotation(
                Line(points = {{-10, -90}, {-10, -100}, {-50, -100}}, color = {0, 0, 255}));
              connect(Delta2.plug_n, r2.plug_p) annotation(
                Line(points = {{50, 20}, {90, 20}, {90, 0}}, color = {0, 0, 255}));
              connect(l1sigma.plug_n, core.plug_p1) annotation(
                Line(points = {{-50, 0}, {-50, 20}, {-10, 20}, {-10, 5}}, color = {0, 0, 255}));
              connect(core.plug_n1, star1.plug_p) annotation(
                Line(points = {{-10, -5}, {-10, -70}, {-10, -70}}, color = {0, 0, 255}));
              connect(core.plug_n2, core.plug_p3) annotation(
                Line(points = {{10, 4}, {10, -4}}, color = {0, 0, 255}));
              connect(core.plug_n3, l2sigma.plug_n) annotation(
                Line(points = {{10, -10}, {10, -20}, {50, -20}, {50, 0}}, color = {0, 0, 255}));
              connect(core.plug_p2, Delta2.plug_p) annotation(
                Line(points = {{10, 10}, {10, 20}, {30, 20}}, color = {0, 0, 255}));
              annotation(
                defaultComponentName = "transformer",
                Documentation(info = "<html>
Transformer Yd5
<br>Typical parameters see:
<a href=\"modelica://Modelica.Electrical.Machines.Interfaces.PartialBasicTransformer\">PartialBasicTransformer</a>
</html>"));
            end Yd05;

            model Yd07 "Transformer Yd7"
              extends Machines.Interfaces.PartialBasicTransformer(final VectorGroup = "Yd07");
              Modelica.Electrical.MultiPhase.Basic.Star star1(final m = m) annotation(
                Placement(transformation(origin = {-10, -80}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
              Modelica.Electrical.Analog.Interfaces.NegativePin starpoint1 annotation(
                Placement(transformation(extent = {{-60, -110}, {-40, -90}}, rotation = 0)));
              Modelica.Electrical.MultiPhase.Basic.Delta Delta2(final m = m) annotation(
                Placement(transformation(extent = {{50, 10}, {30, 30}}, rotation = 0)));
            equation
              connect(star1.pin_n, starpoint1) annotation(
                Line(points = {{-10, -90}, {-10, -100}, {-50, -100}}, color = {0, 0, 255}));
              connect(Delta2.plug_p, r2.plug_p) annotation(
                Line(points = {{50, 20}, {90, 20}, {90, 0}}, color = {0, 0, 255}));
              connect(l1sigma.plug_n, core.plug_p1) annotation(
                Line(points = {{-50, 0}, {-50, 20}, {-10, 20}, {-10, 5}}, color = {0, 0, 255}));
              connect(core.plug_n1, star1.plug_p) annotation(
                Line(points = {{-10, -5}, {-10, -37.5}, {-10, -70}, {-10, -70}}, color = {0, 0, 255}));
              connect(core.plug_n2, core.plug_p3) annotation(
                Line(points = {{10, 4}, {10, -4}}, color = {0, 0, 255}));
              connect(core.plug_p2, Delta2.plug_n) annotation(
                Line(points = {{10, 10}, {10, 20}, {30, 20}}, color = {0, 0, 255}));
              connect(l2sigma.plug_n, core.plug_n3) annotation(
                Line(points = {{50, 0}, {50, -20}, {10, -20}, {10, -10}}, color = {0, 0, 255}));
              annotation(
                defaultComponentName = "transformer",
                Documentation(info = "<html>
Transformer Yd7
<br>Typical parameters see:
<a href=\"modelica://Modelica.Electrical.Machines.Interfaces.PartialBasicTransformer\">PartialBasicTransformer</a>
</html>"));
            end Yd07;

            model Yd09 "Transformer Yd9"
              extends Machines.Interfaces.PartialBasicTransformer(final VectorGroup = "Yd09");
              Modelica.Electrical.MultiPhase.Basic.Star star1(final m = m) annotation(
                Placement(transformation(origin = {-10, -80}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
              Modelica.Electrical.Analog.Interfaces.NegativePin starpoint1 annotation(
                Placement(transformation(extent = {{-60, -110}, {-40, -90}}, rotation = 0)));
              Modelica.Electrical.MultiPhase.Basic.Delta Delta2(final m = m) annotation(
                Placement(transformation(extent = {{30, -30}, {50, -10}}, rotation = 0)));
              Modelica.Electrical.MultiPhase.Basic.Delta Rot2(final m = m) annotation(
                Placement(transformation(extent = {{50, 10}, {30, 30}}, rotation = 0)));
            equation
              connect(star1.pin_n, starpoint1) annotation(
                Line(points = {{-10, -90}, {-10, -100}, {-50, -100}}, color = {0, 0, 255}));
              connect(Rot2.plug_p, l2sigma.plug_n) annotation(
                Line(points = {{50, 20}, {50, 0}}, color = {0, 0, 255}));
              connect(Delta2.plug_n, r2.plug_p) annotation(
                Line(points = {{50, -20}, {90, -20}, {90, 0}}, color = {0, 0, 255}));
              connect(l1sigma.plug_n, core.plug_p1) annotation(
                Line(points = {{-50, 0}, {-50, 20}, {-10, 20}, {-10, 5}}, color = {0, 0, 255}));
              connect(core.plug_n1, star1.plug_p) annotation(
                Line(points = {{-10, -5}, {-10, -37.5}, {-10, -70}, {-10, -70}}, color = {0, 0, 255}));
              connect(core.plug_n2, core.plug_p3) annotation(
                Line(points = {{10, 4}, {10, -4}}, color = {0, 0, 255}));
              connect(core.plug_p2, Rot2.plug_n) annotation(
                Line(points = {{10, 10}, {10, 20}, {30, 20}}, color = {0, 0, 255}));
              connect(core.plug_n3, Delta2.plug_p) annotation(
                Line(points = {{10, -10}, {10, -20}, {30, -20}}, color = {0, 0, 255}));
              annotation(
                defaultComponentName = "transformer",
                Documentation(info = "<html>
Transformer Yd9
<br>Typical parameters see:
<a href=\"modelica://Modelica.Electrical.Machines.Interfaces.PartialBasicTransformer\">PartialBasicTransformer</a>
</html>"));
            end Yd09;

            model Yd11 "Transformer Yd11"
              extends Machines.Interfaces.PartialBasicTransformer(final VectorGroup = "Yd11");
              Modelica.Electrical.MultiPhase.Basic.Star star1(final m = m) annotation(
                Placement(transformation(origin = {-10, -80}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
              Modelica.Electrical.Analog.Interfaces.NegativePin starpoint1 annotation(
                Placement(transformation(extent = {{-60, -110}, {-40, -90}}, rotation = 0)));
              Modelica.Electrical.MultiPhase.Basic.Delta Delta2(final m = m) annotation(
                Placement(transformation(extent = {{30, -30}, {50, -10}}, rotation = 0)));
            equation
              connect(star1.pin_n, starpoint1) annotation(
                Line(points = {{-10, -90}, {-10, -100}, {-50, -100}}, color = {0, 0, 255}));
              connect(Delta2.plug_n, r2.plug_p) annotation(
                Line(points = {{50, -20}, {90, -20}, {90, 0}}, color = {0, 0, 255}));
              connect(l1sigma.plug_n, core.plug_p1) annotation(
                Line(points = {{-50, 0}, {-50, 20}, {-10, 20}, {-10, 5}}, color = {0, 0, 255}));
              connect(core.plug_n1, star1.plug_p) annotation(
                Line(points = {{-10, -5}, {-10, -70}, {-10, -70}}, color = {0, 0, 255}));
              connect(core.plug_n2, core.plug_p3) annotation(
                Line(points = {{10, 4}, {10, -4}}, color = {0, 0, 255}));
              connect(core.plug_n3, Delta2.plug_p) annotation(
                Line(points = {{10, -10}, {10, -20}, {30, -20}}, color = {0, 0, 255}));
              connect(core.plug_p2, l2sigma.plug_n) annotation(
                Line(points = {{10, 10}, {10, 20}, {50, 20}, {50, 0}}, color = {0, 0, 255}));
              annotation(
                defaultComponentName = "transformer",
                Documentation(info = "<html>
Transformer Yd11
<br>Typical parameters see:
<a href=\"modelica://Modelica.Electrical.Machines.Interfaces.PartialBasicTransformer\">PartialBasicTransformer</a>
</html>"));
            end Yd11;
            annotation(
              Documentation(info = "<HTML>
This package contains transformers primary Y connected / secondary d connected in all possible vector groups.
</HTML>", revisions = "<HTML>
<dl>
  <dt><b>Main Authors:</b></dt>
  <dd>
  <a href=\"http://www.haumer.at/\">Anton Haumer</a><br>
  Technical Consulting &amp; Electrical Engineering<br>
  A-3423 St.Andrae-Woerdern<br>Austria<br>
  email: <a href=\"mailto:a.haumer@haumer.at\">a.haumer@haumer.at</a>
  </dd>
  <dt><b>Copyright:</b></dt>
  <dd>Copyright &copy; 1998-2015, Modelica Association and Anton Haumer.<br>
  <i>The Modelica package is <b>free</b> software; it can be redistributed and/or modified
  under the terms of the <b>Modelica license</b>, see the license conditions
  and the accompanying <b>disclaimer</b> in the documentation of package
  Modelica in file \"Modelica/package.mo\".</i></dd>
</dl>
  <ul>
  <li> v1.0.0 2006/11/19 Anton Haumer</li>
  </ul>
</HTML>"));
          end Yd;

          package Yz "Transformers: primary Y / secondary zig-zag"
            extends Modelica.Icons.VariantsPackage;

            model Yz01 "Transformer Yz1"
              extends Machines.Interfaces.PartialBasicTransformer(final VectorGroup = "Yz01");
              Modelica.Electrical.MultiPhase.Basic.Star star1(final m = m) annotation(
                Placement(transformation(origin = {-10, -80}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
              Modelica.Electrical.Analog.Interfaces.NegativePin starpoint1 annotation(
                Placement(transformation(extent = {{-60, -110}, {-40, -90}}, rotation = 0)));
              Modelica.Electrical.MultiPhase.Basic.Star star2(final m = m) annotation(
                Placement(transformation(origin = {10, -80}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
              Modelica.Electrical.Analog.Interfaces.NegativePin starpoint2 annotation(
                Placement(transformation(extent = {{40, -110}, {60, -90}}, rotation = 0)));
              Modelica.Electrical.MultiPhase.Basic.Delta Rot21(final m = m) annotation(
                Placement(transformation(origin = {30, 0}, extent = {{-10, 10}, {10, -10}}, rotation = 90)));
              Modelica.Electrical.MultiPhase.Basic.Delta Rot22(final m = m) annotation(
                Placement(transformation(origin = {40, 20}, extent = {{-10, 10}, {10, -10}}, rotation = 180)));
            equation
              connect(star1.pin_n, starpoint1) annotation(
                Line(points = {{-10, -90}, {-10, -100}, {-50, -100}}, color = {0, 0, 255}));
              connect(star2.pin_n, starpoint2) annotation(
                Line(points = {{10, -90}, {10, -100}, {50, -100}}, color = {0, 0, 255}));
              connect(l1sigma.plug_n, core.plug_p1) annotation(
                Line(points = {{-50, 0}, {-50, 20}, {-10, 20}, {-10, 5}}, color = {0, 0, 255}));
              connect(core.plug_n1, star1.plug_p) annotation(
                Line(points = {{-10, -5}, {-10, -70}, {-10, -70}}, color = {0, 0, 255}));
              connect(core.plug_p2, Rot21.plug_n) annotation(
                Line(points = {{10, 10}, {30, 10}}, color = {0, 0, 255}));
              connect(core.plug_p3, Rot21.plug_p) annotation(
                Line(points = {{10, -4}, {20, -4}, {20, -10}, {30, -10}}, color = {0, 0, 255}));
              connect(l2sigma.plug_n, Rot22.plug_p) annotation(
                Line(points = {{50, 0}, {50, 20}}, color = {0, 0, 255}));
              connect(Rot22.plug_n, core.plug_n2) annotation(
                Line(points = {{30, 20}, {20, 20}, {20, 4}, {10, 4}}, color = {0, 0, 255}));
              connect(core.plug_n3, star2.plug_p) annotation(
                Line(points = {{10, -10}, {10, -70}, {10, -70}}, color = {0, 0, 255}));
              annotation(
                defaultComponentName = "transformer",
                Documentation(info = "<html>
Transformer Yz1
<br>Typical parameters see:
<a href=\"modelica://Modelica.Electrical.Machines.Interfaces.PartialBasicTransformer\">PartialBasicTransformer</a>
</html>"));
            end Yz01;

            model Yz03 "Transformer Yz3"
              extends Machines.Interfaces.PartialBasicTransformer(final VectorGroup = "Yz03");
              Modelica.Electrical.MultiPhase.Basic.Star star1(final m = m) annotation(
                Placement(transformation(origin = {-10, -80}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
              Modelica.Electrical.Analog.Interfaces.NegativePin starpoint1 annotation(
                Placement(transformation(extent = {{-60, -110}, {-40, -90}}, rotation = 0)));
              Modelica.Electrical.MultiPhase.Basic.Star star2(final m = m) annotation(
                Placement(transformation(origin = {10, -80}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
              Modelica.Electrical.Analog.Interfaces.NegativePin starpoint2 annotation(
                Placement(transformation(extent = {{40, -110}, {60, -90}}, rotation = 0)));
              Modelica.Electrical.MultiPhase.Basic.Delta Rot21(final m = m) annotation(
                Placement(transformation(origin = {30, 0}, extent = {{10, -10}, {-10, 10}}, rotation = 90)));
              Modelica.Electrical.MultiPhase.Basic.Delta Rot22(final m = m) annotation(
                Placement(transformation(origin = {40, 20}, extent = {{-10, 10}, {10, -10}}, rotation = 180)));
            equation
              connect(star1.pin_n, starpoint1) annotation(
                Line(points = {{-10, -90}, {-10, -100}, {-50, -100}}, color = {0, 0, 255}));
              connect(star2.pin_n, starpoint2) annotation(
                Line(points = {{10, -90}, {10, -100}, {50, -100}}, color = {0, 0, 255}));
              connect(l1sigma.plug_n, core.plug_p1) annotation(
                Line(points = {{-50, 0}, {-50, 20}, {-10, 20}, {-10, 5}}, color = {0, 0, 255}));
              connect(core.plug_n1, star1.plug_p) annotation(
                Line(points = {{-10, -5}, {-10, -37.5}, {-10, -70}, {-10, -70}}, color = {0, 0, 255}));
              connect(core.plug_p2, Rot21.plug_p) annotation(
                Line(points = {{10, 10}, {30, 10}}, color = {0, 0, 255}));
              connect(core.plug_p3, Rot21.plug_n) annotation(
                Line(points = {{10, -4}, {20, -4}, {20, -10}, {30, -10}}, color = {0, 0, 255}));
              connect(l2sigma.plug_n, Rot22.plug_p) annotation(
                Line(points = {{50, 0}, {50, 20}}, color = {0, 0, 255}));
              connect(Rot22.plug_n, core.plug_n2) annotation(
                Line(points = {{30, 20}, {20, 20}, {20, 4}, {10, 4}}, color = {0, 0, 255}));
              connect(core.plug_n3, star2.plug_p) annotation(
                Line(points = {{10, -10}, {10, -70}, {10, -70}}, color = {0, 0, 255}));
              annotation(
                defaultComponentName = "transformer",
                Documentation(info = "<html>
Transformer Yz3
<br>Typical parameters see:
<a href=\"modelica://Modelica.Electrical.Machines.Interfaces.PartialBasicTransformer\">PartialBasicTransformer</a>
</html>"));
            end Yz03;

            model Yz05 "Transformer Yz5"
              extends Machines.Interfaces.PartialBasicTransformer(final VectorGroup = "Yz05");
              Modelica.Electrical.MultiPhase.Basic.Star star1(final m = m) annotation(
                Placement(transformation(origin = {-10, -80}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
              Modelica.Electrical.Analog.Interfaces.NegativePin starpoint1 annotation(
                Placement(transformation(extent = {{-60, -110}, {-40, -90}}, rotation = 0)));
              Modelica.Electrical.MultiPhase.Basic.Star star2(final m = m) annotation(
                Placement(transformation(origin = {10, -80}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
              Modelica.Electrical.Analog.Interfaces.NegativePin starpoint2 annotation(
                Placement(transformation(extent = {{40, -110}, {60, -90}}, rotation = 0)));
              Modelica.Electrical.MultiPhase.Basic.Delta Rot21(final m = m) annotation(
                Placement(transformation(origin = {30, 0}, extent = {{-10, 10}, {10, -10}}, rotation = 90)));
            equation
              connect(star1.pin_n, starpoint1) annotation(
                Line(points = {{-10, -90}, {-10, -100}, {-50, -100}}, color = {0, 0, 255}));
              connect(star2.pin_n, starpoint2) annotation(
                Line(points = {{10, -90}, {10, -100}, {50, -100}}, color = {0, 0, 255}));
              connect(l1sigma.plug_n, core.plug_p1) annotation(
                Line(points = {{-50, 0}, {-50, 20}, {-10, 20}, {-10, 5}}, color = {0, 0, 255}));
              connect(core.plug_n1, star1.plug_p) annotation(
                Line(points = {{-10, -5}, {-10, -70}, {-10, -70}}, color = {0, 0, 255}));
              connect(core.plug_p2, Rot21.plug_n) annotation(
                Line(points = {{10, 10}, {30, 10}}, color = {0, 0, 255}));
              connect(core.plug_p3, Rot21.plug_p) annotation(
                Line(points = {{10, -4}, {20, -4}, {20, -10}, {30, -10}}, color = {0, 0, 255}));
              connect(core.plug_n3, star2.plug_p) annotation(
                Line(points = {{10, -10}, {10, -40}, {10, -70}, {10, -70}}, color = {0, 0, 255}));
              connect(l2sigma.plug_n, core.plug_n2) annotation(
                Line(points = {{50, 0}, {50, 20}, {20, 20}, {20, 4}, {10, 4}}, color = {0, 0, 255}));
              annotation(
                defaultComponentName = "transformer",
                Documentation(info = "<html>
Transformer Yz5
<br>Typical parameters see:
<a href=\"modelica://Modelica.Electrical.Machines.Interfaces.PartialBasicTransformer\">PartialBasicTransformer</a>
</html>"));
            end Yz05;

            model Yz07 "Transformer Yz7"
              extends Machines.Interfaces.PartialBasicTransformer(final VectorGroup = "Yz07");
              Modelica.Electrical.MultiPhase.Basic.Star star1(final m = m) annotation(
                Placement(transformation(origin = {-10, -80}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
              Modelica.Electrical.Analog.Interfaces.NegativePin starpoint1 annotation(
                Placement(transformation(extent = {{-60, -110}, {-40, -90}}, rotation = 0)));
              Modelica.Electrical.MultiPhase.Basic.Star star2(final m = m) annotation(
                Placement(transformation(origin = {10, -80}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
              Modelica.Electrical.Analog.Interfaces.NegativePin starpoint2 annotation(
                Placement(transformation(extent = {{40, -110}, {60, -90}}, rotation = 0)));
              Modelica.Electrical.MultiPhase.Basic.Delta Rot21(final m = m) annotation(
                Placement(transformation(origin = {30, 0}, extent = {{10, -10}, {-10, 10}}, rotation = 90)));
            equation
              connect(star1.pin_n, starpoint1) annotation(
                Line(points = {{-10, -90}, {-10, -100}, {-50, -100}}, color = {0, 0, 255}));
              connect(star2.pin_n, starpoint2) annotation(
                Line(points = {{10, -90}, {10, -100}, {50, -100}}, color = {0, 0, 255}));
              connect(l1sigma.plug_n, core.plug_p1) annotation(
                Line(points = {{-50, 0}, {-50, 20}, {-10, 20}, {-10, 5}}, color = {0, 0, 255}));
              connect(core.plug_n1, star1.plug_p) annotation(
                Line(points = {{-10, -5}, {-10, -37.5}, {-10, -70}, {-10, -70}}, color = {0, 0, 255}));
              connect(core.plug_p2, Rot21.plug_p) annotation(
                Line(points = {{10, 10}, {30, 10}}, color = {0, 0, 255}));
              connect(core.plug_p3, Rot21.plug_n) annotation(
                Line(points = {{10, -4}, {20, -4}, {20, -10}, {30, -10}}, color = {0, 0, 255}));
              connect(l2sigma.plug_n, core.plug_n2) annotation(
                Line(points = {{50, 0}, {50, 20}, {20, 20}, {20, 4}, {10, 4}}, color = {0, 0, 255}));
              connect(core.plug_n3, star2.plug_p) annotation(
                Line(points = {{10, -10}, {10, -40}, {10, -70}, {10, -70}}, color = {0, 0, 255}));
              annotation(
                defaultComponentName = "transformer",
                Documentation(info = "<html>
Transformer Yz7
<br>Typical parameters see:
<a href=\"modelica://Modelica.Electrical.Machines.Interfaces.PartialBasicTransformer\">PartialBasicTransformer</a>
</html>"));
            end Yz07;

            model Yz09 "Transformer Yz9"
              extends Machines.Interfaces.PartialBasicTransformer(final VectorGroup = "Yz09");
              Modelica.Electrical.MultiPhase.Basic.Star star1(final m = m) annotation(
                Placement(transformation(origin = {-10, -80}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
              Modelica.Electrical.Analog.Interfaces.NegativePin starpoint1 annotation(
                Placement(transformation(extent = {{-60, -110}, {-40, -90}}, rotation = 0)));
              Modelica.Electrical.MultiPhase.Basic.Star star2(final m = m) annotation(
                Placement(transformation(origin = {10, -80}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
              Modelica.Electrical.Analog.Interfaces.NegativePin starpoint2 annotation(
                Placement(transformation(extent = {{40, -110}, {60, -90}}, rotation = 0)));
              Modelica.Electrical.MultiPhase.Basic.Delta Rot21(final m = m) annotation(
                Placement(transformation(origin = {30, 0}, extent = {{-10, 10}, {10, -10}}, rotation = 90)));
              Modelica.Electrical.MultiPhase.Basic.Delta Rot22(final m = m) annotation(
                Placement(transformation(origin = {40, 20}, extent = {{10, 10}, {-10, -10}}, rotation = 180)));
            equation
              connect(star1.pin_n, starpoint1) annotation(
                Line(points = {{-10, -90}, {-10, -100}, {-50, -100}}, color = {0, 0, 255}));
              connect(star2.pin_n, starpoint2) annotation(
                Line(points = {{10, -90}, {10, -100}, {50, -100}}, color = {0, 0, 255}));
              connect(l1sigma.plug_n, core.plug_p1) annotation(
                Line(points = {{-50, 0}, {-50, 20}, {-10, 20}, {-10, 5}}, color = {0, 0, 255}));
              connect(core.plug_n1, star1.plug_p) annotation(
                Line(points = {{-10, -5}, {-10, -70}, {-10, -70}}, color = {0, 0, 255}));
              connect(core.plug_p2, Rot21.plug_n) annotation(
                Line(points = {{10, 10}, {30, 10}}, color = {0, 0, 255}));
              connect(core.plug_p3, Rot21.plug_p) annotation(
                Line(points = {{10, -4}, {20, -4}, {20, -10}, {30, -10}}, color = {0, 0, 255}));
              connect(core.plug_n3, star2.plug_p) annotation(
                Line(points = {{10, -10}, {10, -70}, {10, -70}}, color = {0, 0, 255}));
              connect(l2sigma.plug_n, Rot22.plug_n) annotation(
                Line(points = {{50, 0}, {50, 20}, {50, 20}}, color = {0, 0, 255}));
              connect(Rot22.plug_p, core.plug_n2) annotation(
                Line(points = {{30, 20}, {20, 20}, {20, 4}, {10, 4}}, color = {0, 0, 255}));
              annotation(
                defaultComponentName = "transformer",
                Documentation(info = "<html>
Transformer Yz9
<br>Typical parameters see:
<a href=\"modelica://Modelica.Electrical.Machines.Interfaces.PartialBasicTransformer\">PartialBasicTransformer</a>
</html>"));
            end Yz09;

            model Yz11 "Transformer Yz11"
              extends Machines.Interfaces.PartialBasicTransformer(final VectorGroup = "Yz11");
              Modelica.Electrical.MultiPhase.Basic.Star star1(final m = m) annotation(
                Placement(transformation(origin = {-10, -80}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
              Modelica.Electrical.Analog.Interfaces.NegativePin starpoint1 annotation(
                Placement(transformation(extent = {{-60, -110}, {-40, -90}}, rotation = 0)));
              Modelica.Electrical.MultiPhase.Basic.Star star2(final m = m) annotation(
                Placement(transformation(origin = {10, -80}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
              Modelica.Electrical.Analog.Interfaces.NegativePin starpoint2 annotation(
                Placement(transformation(extent = {{40, -110}, {60, -90}}, rotation = 0)));
              Modelica.Electrical.MultiPhase.Basic.Delta Rot21(final m = m) annotation(
                Placement(transformation(origin = {30, 0}, extent = {{10, -10}, {-10, 10}}, rotation = 90)));
              Modelica.Electrical.MultiPhase.Basic.Delta Rot22(final m = m) annotation(
                Placement(transformation(origin = {40, 20}, extent = {{10, 10}, {-10, -10}}, rotation = 180)));
            equation
              connect(star1.pin_n, starpoint1) annotation(
                Line(points = {{-10, -90}, {-10, -100}, {-50, -100}}, color = {0, 0, 255}));
              connect(star2.pin_n, starpoint2) annotation(
                Line(points = {{10, -90}, {10, -100}, {50, -100}}, color = {0, 0, 255}));
              connect(l1sigma.plug_n, core.plug_p1) annotation(
                Line(points = {{-50, 0}, {-50, 20}, {-10, 20}, {-10, 5}}, color = {0, 0, 255}));
              connect(core.plug_n1, star1.plug_p) annotation(
                Line(points = {{-10, -5}, {-10, -37.5}, {-10, -70}, {-10, -70}}, color = {0, 0, 255}));
              connect(core.plug_p2, Rot21.plug_p) annotation(
                Line(points = {{10, 10}, {30, 10}}, color = {0, 0, 255}));
              connect(core.plug_p3, Rot21.plug_n) annotation(
                Line(points = {{10, -4}, {20, -4}, {20, -10}, {30, -10}}, color = {0, 0, 255}));
              connect(l2sigma.plug_n, Rot22.plug_n) annotation(
                Line(points = {{50, 0}, {50, 20}}, color = {0, 0, 255}));
              connect(Rot22.plug_p, core.plug_n2) annotation(
                Line(points = {{30, 20}, {20, 20}, {20, 4}, {10, 4}}, color = {0, 0, 255}));
              connect(core.plug_n3, star2.plug_p) annotation(
                Line(points = {{10, -10}, {10, -70}, {10, -70}}, color = {0, 0, 255}));
              annotation(
                defaultComponentName = "transformer",
                Documentation(info = "<html>
Transformer Yz11
<br>Typical parameters see:
<a href=\"modelica://Modelica.Electrical.Machines.Interfaces.PartialBasicTransformer\">PartialBasicTransformer</a>
</html>"));
            end Yz11;
            annotation(
              Documentation(info = "<HTML>
This package contains transformers primary Y connected / secondary zig-zag connected in all possible vector groups.
</HTML>", revisions = "<HTML>
<dl>
  <dt><b>Main Authors:</b></dt>
  <dd>
  <a href=\"http://www.haumer.at/\">Anton Haumer</a><br>
  Technical Consulting &amp; Electrical Engineering<br>
  A-3423 St.Andrae-Woerdern<br>Austria<br>
  email: <a href=\"mailto:a.haumer@haumer.at\">a.haumer@haumer.at</a>
  </dd>
  <dt><b>Copyright:</b></dt>
  <dd>Copyright &copy; 1998-2015, Modelica Association and Anton Haumer.<br>
  <i>The Modelica package is <b>free</b> software; it can be redistributed and/or modified
  under the terms of the <b>Modelica license</b>, see the license conditions
  and the accompanying <b>disclaimer</b> in the documentation of package
  Modelica in file \"Modelica/package.mo\".</i></dd>
</dl>
  <ul>
  <li> v1.0.0 2006/11/19 Anton Haumer</li>
  </ul>
</HTML>"));
          end Yz;

          package Dy "Transformers: primary D / secondary y"
            extends Modelica.Icons.VariantsPackage;

            model Dy01 "Transformer Dy1"
              extends Machines.Interfaces.PartialBasicTransformer(final VectorGroup = "Dy01");
              Modelica.Electrical.MultiPhase.Basic.Star star2(final m = m) annotation(
                Placement(transformation(origin = {10, -80}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
              Modelica.Electrical.Analog.Interfaces.NegativePin starpoint2 annotation(
                Placement(transformation(extent = {{40, -110}, {60, -90}}, rotation = 0)));
              Modelica.Electrical.MultiPhase.Basic.Delta Delta1(final m = m) annotation(
                Placement(transformation(extent = {{-50, -30}, {-30, -10}}, rotation = 0)));
              Modelica.Electrical.MultiPhase.Basic.Delta Rot2(final m = m) annotation(
                Placement(transformation(extent = {{50, -30}, {30, -10}}, rotation = 0)));
            equation
              connect(star2.pin_n, starpoint2) annotation(
                Line(points = {{10, -90}, {10, -100}, {50, -100}}, color = {0, 0, 255}));
              connect(l2sigma.plug_n, Rot2.plug_p) annotation(
                Line(points = {{50, 0}, {50, -20}}, color = {0, 0, 255}));
              connect(Delta1.plug_p, r1.plug_p) annotation(
                Line(points = {{-50, -20}, {-90, -20}, {-90, 0}}, color = {0, 0, 255}));
              connect(Delta1.plug_n, core.plug_n1) annotation(
                Line(points = {{-30, -20}, {-10, -20}, {-10, -5}}, color = {0, 0, 255}));
              connect(l1sigma.plug_n, core.plug_p1) annotation(
                Line(points = {{-50, 0}, {-50, 20}, {-10, 20}, {-10, 5}}, color = {0, 0, 255}));
              connect(core.plug_n2, core.plug_p3) annotation(
                Line(points = {{10, 4}, {10, -4}}, color = {0, 0, 255}));
              connect(core.plug_n3, Rot2.plug_n) annotation(
                Line(points = {{10, -10}, {10, -20}, {30, -20}}, color = {0, 0, 255}));
              connect(core.plug_p2, star2.plug_p) annotation(
                Line(points = {{10, 10}, {20, 10}, {20, -70}, {10, -70}}, color = {0, 0, 255}));
              annotation(
                defaultComponentName = "transformer",
                Documentation(info = "<html>
Transformer Dy1
<br>Typical parameters see:
<a href=\"modelica://Modelica.Electrical.Machines.Interfaces.PartialBasicTransformer\">PartialBasicTransformer</a>
</html>"));
            end Dy01;

            model Dy03 "Transformer Dy3"
              extends Machines.Interfaces.PartialBasicTransformer(final VectorGroup = "Dy03");
              Modelica.Electrical.MultiPhase.Basic.Star star2(final m = m) annotation(
                Placement(transformation(origin = {10, -80}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
              Modelica.Electrical.Analog.Interfaces.NegativePin starpoint2 annotation(
                Placement(transformation(extent = {{40, -110}, {60, -90}}, rotation = 0)));
              Modelica.Electrical.MultiPhase.Basic.Delta Delta1(final m = m) annotation(
                Placement(transformation(extent = {{-50, -30}, {-30, -10}}, rotation = 0)));
              Modelica.Electrical.MultiPhase.Basic.Delta Rot2(final m = m) annotation(
                Placement(transformation(extent = {{30, 10}, {50, 30}}, rotation = 0)));
            equation
              connect(star2.pin_n, starpoint2) annotation(
                Line(points = {{10, -90}, {10, -100}, {50, -100}}, color = {0, 0, 255}));
              connect(Rot2.plug_n, l2sigma.plug_n) annotation(
                Line(points = {{50, 20}, {50, 0}}, color = {0, 0, 255}));
              connect(Delta1.plug_p, r1.plug_p) annotation(
                Line(points = {{-50, -20}, {-90, -20}, {-90, 0}}, color = {0, 0, 255}));
              connect(l1sigma.plug_n, core.plug_p1) annotation(
                Line(points = {{-50, 0}, {-50, 20}, {-10, 20}, {-10, 5}}, color = {0, 0, 255}));
              connect(Delta1.plug_n, core.plug_n1) annotation(
                Line(points = {{-30, -20}, {-10, -20}, {-10, -5}}, color = {0, 0, 255}));
              connect(core.plug_p2, Rot2.plug_p) annotation(
                Line(points = {{10, 10}, {10, 20}, {30, 20}}, color = {0, 0, 255}));
              connect(core.plug_n2, core.plug_p3) annotation(
                Line(points = {{10, 4}, {10, -4}}, color = {0, 0, 255}));
              connect(core.plug_n3, star2.plug_p) annotation(
                Line(points = {{10, -10}, {10, -40}, {10, -70}, {10, -70}}, color = {0, 0, 255}));
              annotation(
                defaultComponentName = "transformer",
                Documentation(info = "<html>
Transformer Dy3
<br>Typical parameters see:
<a href=\"modelica://Modelica.Electrical.Machines.Interfaces.PartialBasicTransformer\">PartialBasicTransformer</a>
</html>"));
            end Dy03;

            model Dy05 "Transformer Dy5"
              extends Machines.Interfaces.PartialBasicTransformer(final VectorGroup = "Dy05");
              Modelica.Electrical.MultiPhase.Basic.Star star2(final m = m) annotation(
                Placement(transformation(origin = {10, -80}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
              Modelica.Electrical.Analog.Interfaces.NegativePin starpoint2 annotation(
                Placement(transformation(extent = {{40, -110}, {60, -90}}, rotation = 0)));
              Modelica.Electrical.MultiPhase.Basic.Delta Delta1(final m = m) annotation(
                Placement(transformation(extent = {{-50, -30}, {-30, -10}}, rotation = 0)));
            equation
              connect(star2.pin_n, starpoint2) annotation(
                Line(points = {{10, -90}, {10, -100}, {50, -100}}, color = {0, 0, 255}));
              connect(Delta1.plug_p, r1.plug_p) annotation(
                Line(points = {{-50, -20}, {-90, -20}, {-90, 0}}, color = {0, 0, 255}));
              connect(Delta1.plug_n, core.plug_n1) annotation(
                Line(points = {{-30, -20}, {-10, -20}, {-10, -5}}, color = {0, 0, 255}));
              connect(l1sigma.plug_n, core.plug_p1) annotation(
                Line(points = {{-50, 0}, {-50, 20}, {-10, 20}, {-10, 5}}, color = {0, 0, 255}));
              connect(core.plug_n2, core.plug_p3) annotation(
                Line(points = {{10, 4}, {10, -4}}, color = {0, 0, 255}));
              connect(l2sigma.plug_n, core.plug_n3) annotation(
                Line(points = {{50, 0}, {50, -20}, {10, -20}, {10, -10}}, color = {0, 0, 255}));
              connect(core.plug_p2, star2.plug_p) annotation(
                Line(points = {{10, 10}, {20, 10}, {20, -70}, {10, -70}}, color = {0, 0, 255}));
              annotation(
                defaultComponentName = "transformer",
                Documentation(info = "<html>
Transformer Dy5
<br>Typical parameters see:
<a href=\"modelica://Modelica.Electrical.Machines.Interfaces.PartialBasicTransformer\">PartialBasicTransformer</a>
</html>"));
            end Dy05;

            model Dy07 "Transformer Dy7"
              extends Machines.Interfaces.PartialBasicTransformer(final VectorGroup = "Dy07");
              Modelica.Electrical.MultiPhase.Basic.Star star2(final m = m) annotation(
                Placement(transformation(origin = {10, -80}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
              Modelica.Electrical.Analog.Interfaces.NegativePin starpoint2 annotation(
                Placement(transformation(extent = {{40, -110}, {60, -90}}, rotation = 0)));
              Modelica.Electrical.MultiPhase.Basic.Delta Delta1(final m = m) annotation(
                Placement(transformation(extent = {{-50, -30}, {-30, -10}}, rotation = 0)));
              Modelica.Electrical.MultiPhase.Basic.Delta Rot2(final m = m) annotation(
                Placement(transformation(extent = {{50, 10}, {30, 30}}, rotation = 0)));
            equation
              connect(star2.pin_n, starpoint2) annotation(
                Line(points = {{10, -90}, {10, -100}, {50, -100}}, color = {0, 0, 255}));
              connect(r1.plug_p, Delta1.plug_p) annotation(
                Line(points = {{-90, 0}, {-90, -20}, {-50, -20}}, color = {0, 0, 255}));
              connect(Rot2.plug_p, l2sigma.plug_n) annotation(
                Line(points = {{50, 20}, {50, 0}}, color = {0, 0, 255}));
              connect(l1sigma.plug_n, core.plug_p1) annotation(
                Line(points = {{-50, 0}, {-50, 20}, {-10, 20}, {-10, 5}}, color = {0, 0, 255}));
              connect(Delta1.plug_n, core.plug_n1) annotation(
                Line(points = {{-30, -20}, {-10, -20}, {-10, -5}}, color = {0, 0, 255}));
              connect(core.plug_n2, core.plug_p3) annotation(
                Line(points = {{10, 4}, {10, -4}}, color = {0, 0, 255}));
              connect(core.plug_n3, star2.plug_p) annotation(
                Line(points = {{10, -10}, {10, -40}, {10, -70}, {10, -70}}, color = {0, 0, 255}));
              connect(core.plug_p2, Rot2.plug_n) annotation(
                Line(points = {{10, 10}, {10, 20}, {30, 20}}, color = {0, 0, 255}));
              annotation(
                defaultComponentName = "transformer",
                Documentation(info = "<html>
Transformer Dy7
<br>Typical parameters see:
<a href=\"modelica://Modelica.Electrical.Machines.Interfaces.PartialBasicTransformer\">PartialBasicTransformer</a>
</html>"));
            end Dy07;

            model Dy09 "Transformer Dy9"
              extends Machines.Interfaces.PartialBasicTransformer(final VectorGroup = "Dy09");
              Modelica.Electrical.MultiPhase.Basic.Star star2(final m = m) annotation(
                Placement(transformation(origin = {10, -80}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
              Modelica.Electrical.Analog.Interfaces.NegativePin starpoint2 annotation(
                Placement(transformation(extent = {{40, -110}, {60, -90}}, rotation = 0)));
              Modelica.Electrical.MultiPhase.Basic.Delta Delta1(final m = m) annotation(
                Placement(transformation(extent = {{-50, -30}, {-30, -10}}, rotation = 0)));
              Modelica.Electrical.MultiPhase.Basic.Delta Rot2(final m = m) annotation(
                Placement(transformation(extent = {{30, -30}, {50, -10}}, rotation = 0)));
            equation
              connect(star2.pin_n, starpoint2) annotation(
                Line(points = {{10, -90}, {10, -100}, {50, -100}}, color = {0, 0, 255}));
              connect(r1.plug_p, Delta1.plug_p) annotation(
                Line(points = {{-90, 0}, {-90, -20}, {-50, -20}}, color = {0, 0, 255}));
              connect(l2sigma.plug_n, Rot2.plug_n) annotation(
                Line(points = {{50, 0}, {50, -20}}, color = {0, 0, 255}));
              connect(l1sigma.plug_n, core.plug_p1) annotation(
                Line(points = {{-50, 0}, {-50, 20}, {-10, 20}, {-10, 5}, {-10, 5}}, color = {0, 0, 255}));
              connect(Delta1.plug_n, core.plug_n1) annotation(
                Line(points = {{-30, -20}, {-10, -20}, {-10, -5}}, color = {0, 0, 255}));
              connect(core.plug_n2, core.plug_p3) annotation(
                Line(points = {{10, 4}, {10, -4}}, color = {0, 0, 255}));
              connect(core.plug_n3, Rot2.plug_p) annotation(
                Line(points = {{10, -10}, {10, -20}, {30, -20}}, color = {0, 0, 255}));
              connect(core.plug_p2, star2.plug_p) annotation(
                Line(points = {{10, 10}, {20, 10}, {20, -70}, {10, -70}}, color = {0, 0, 255}));
              annotation(
                defaultComponentName = "transformer",
                Documentation(info = "<html>
Transformer Dy9
<br>Typical parameters see:
<a href=\"modelica://Modelica.Electrical.Machines.Interfaces.PartialBasicTransformer\">PartialBasicTransformer</a>
</html>"));
            end Dy09;

            model Dy11 "Transformer Dy11"
              extends Machines.Interfaces.PartialBasicTransformer(final VectorGroup = "Dy11");
              Modelica.Electrical.MultiPhase.Basic.Star star2(final m = m) annotation(
                Placement(transformation(origin = {10, -80}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
              Modelica.Electrical.Analog.Interfaces.NegativePin starpoint2 annotation(
                Placement(transformation(extent = {{40, -110}, {60, -90}}, rotation = 0)));
              Modelica.Electrical.MultiPhase.Basic.Delta Delta1(final m = m) annotation(
                Placement(transformation(extent = {{-50, -30}, {-30, -10}}, rotation = 0)));
            equation
              connect(star2.pin_n, starpoint2) annotation(
                Line(points = {{10, -90}, {10, -100}, {50, -100}}, color = {0, 0, 255}));
              connect(r1.plug_p, Delta1.plug_p) annotation(
                Line(points = {{-90, 0}, {-90, -20}, {-50, -20}}, color = {0, 0, 255}));
              connect(l1sigma.plug_n, core.plug_p1) annotation(
                Line(points = {{-50, 0}, {-50, 20}, {-10, 20}, {-10, 5}}, color = {0, 0, 255}));
              connect(Delta1.plug_n, core.plug_n1) annotation(
                Line(points = {{-30, -20}, {-10, -20}, {-10, -5}}, color = {0, 0, 255}));
              connect(core.plug_n2, core.plug_p3) annotation(
                Line(points = {{10, 4}, {10, -4}}, color = {0, 0, 255}));
              connect(core.plug_n3, star2.plug_p) annotation(
                Line(points = {{10, -10}, {10, -40}, {10, -70}, {10, -70}}, color = {0, 0, 255}));
              connect(core.plug_p2, l2sigma.plug_n) annotation(
                Line(points = {{10, 10}, {10, 20}, {50, 20}, {50, 0}}, color = {0, 0, 255}));
              annotation(
                defaultComponentName = "transformer",
                Documentation(info = "<html>
Transformer Dy11
<br>Typical parameters see:
<a href=\"modelica://Modelica.Electrical.Machines.Interfaces.PartialBasicTransformer\">PartialBasicTransformer</a>
</html>"));
            end Dy11;
            annotation(
              Documentation(info = "<HTML>
This package contains transformers primary D connected / secondary y connected in all possible vector groups.
</HTML>", revisions = "<HTML>
<dl>
  <dt><b>Main Authors:</b></dt>
  <dd>
  <a href=\"http://www.haumer.at/\">Anton Haumer</a><br>
  Technical Consulting &amp; Electrical Engineering<br>
  A-3423 St.Andrae-Woerdern<br>Austria<br>
  email: <a href=\"mailto:a.haumer@haumer.at\">a.haumer@haumer.at</a>
  </dd>
  <dt><b>Copyright:</b></dt>
  <dd>Copyright &copy; 1998-2015, Modelica Association and Anton Haumer.<br>
  <i>The Modelica package is <b>free</b> software; it can be redistributed and/or modified
  under the terms of the <b>Modelica license</b>, see the license conditions
  and the accompanying <b>disclaimer</b> in the documentation of package
  Modelica in file \"Modelica/package.mo\".</i></dd>
</dl>
  <ul>
  <li> v1.0.0 2006/11/19 Anton Haumer</li>
  </ul>
</HTML>"));
          end Dy;

          package Dd "Transformers: primary D / secondary d"
            extends Modelica.Icons.VariantsPackage;

            model Dd00 "Transformer Dd0"
              extends Machines.Interfaces.PartialBasicTransformer(final VectorGroup = "Dd00");
              Modelica.Electrical.MultiPhase.Basic.Delta Delta1(final m = m) annotation(
                Placement(transformation(extent = {{-50, -30}, {-30, -10}}, rotation = 0)));
              Modelica.Electrical.MultiPhase.Basic.Delta Delta2(final m = m) annotation(
                Placement(transformation(extent = {{50, -30}, {30, -10}}, rotation = 0)));
            equation
              connect(r1.plug_p, Delta1.plug_p) annotation(
                Line(points = {{-90, 0}, {-90, -20}, {-50, -20}}, color = {0, 0, 255}));
              connect(Delta2.plug_p, r2.plug_p) annotation(
                Line(points = {{50, -20}, {90, -20}, {90, 0}}, color = {0, 0, 255}));
              connect(l1sigma.plug_n, core.plug_p1) annotation(
                Line(points = {{-50, 0}, {-50, 20}, {-10, 20}, {-10, 5}}, color = {0, 0, 255}));
              connect(Delta1.plug_n, core.plug_n1) annotation(
                Line(points = {{-30, -20}, {-10, -20}, {-10, -5}}, color = {0, 0, 255}));
              connect(core.plug_n2, core.plug_p3) annotation(
                Line(points = {{10, 4}, {10, -4}}, color = {0, 0, 255}));
              connect(core.plug_p2, l2sigma.plug_n) annotation(
                Line(points = {{10, 10}, {10, 20}, {50, 20}, {50, 0}}, color = {0, 0, 255}));
              connect(core.plug_n3, Delta2.plug_n) annotation(
                Line(points = {{10, -10}, {10, -20}, {30, -20}}, color = {0, 0, 255}));
              annotation(
                defaultComponentName = "transformer",
                Documentation(info = "<html>
Transformer Dd0
<br>Typical parameters see:
<a href=\"modelica://Modelica.Electrical.Machines.Interfaces.PartialBasicTransformer\">PartialBasicTransformer</a>
</html>"));
            end Dd00;

            model Dd02 "Transformer Dd2"
              extends Machines.Interfaces.PartialBasicTransformer(final VectorGroup = "Dd02");
              Modelica.Electrical.MultiPhase.Basic.Delta Delta1(final m = m) annotation(
                Placement(transformation(extent = {{-50, -30}, {-30, -10}}, rotation = 0)));
              Modelica.Electrical.MultiPhase.Basic.Delta Delta2(final m = m) annotation(
                Placement(transformation(extent = {{30, 10}, {50, 30}}, rotation = 0)));
              Modelica.Electrical.MultiPhase.Basic.Delta Rot2(final m = m) annotation(
                Placement(transformation(extent = {{50, -30}, {30, -10}}, rotation = 0)));
            equation
              connect(r1.plug_p, Delta1.plug_p) annotation(
                Line(points = {{-90, 0}, {-90, -20}, {-50, -20}}, color = {0, 0, 255}));
              connect(Delta2.plug_n, r2.plug_p) annotation(
                Line(points = {{50, 20}, {90, 20}, {90, 0}}, color = {0, 0, 255}));
              connect(l2sigma.plug_n, Rot2.plug_p) annotation(
                Line(points = {{50, 0}, {50, -20}}, color = {0, 0, 255}));
              connect(l1sigma.plug_n, core.plug_p1) annotation(
                Line(points = {{-50, 0}, {-50, 20}, {-10, 20}, {-10, 5}}, color = {0, 0, 255}));
              connect(Delta1.plug_n, core.plug_n1) annotation(
                Line(points = {{-30, -20}, {-10, -20}, {-10, -5}}, color = {0, 0, 255}));
              connect(core.plug_n2, core.plug_p3) annotation(
                Line(points = {{10, 4}, {10, -4}}, color = {0, 0, 255}));
              connect(core.plug_p2, Delta2.plug_p) annotation(
                Line(points = {{10, 10}, {10, 20}, {30, 20}}, color = {0, 0, 255}));
              connect(core.plug_n3, Rot2.plug_n) annotation(
                Line(points = {{10, -10}, {10, -20}, {30, -20}}, color = {0, 0, 255}));
              annotation(
                defaultComponentName = "transformer",
                Documentation(info = "<html>
Transformer Dd2
<br>Typical parameters see:
<a href=\"modelica://Modelica.Electrical.Machines.Interfaces.PartialBasicTransformer\">PartialBasicTransformer</a>
</html>"));
            end Dd02;

            model Dd04 "Transformer Dd4"
              extends Machines.Interfaces.PartialBasicTransformer(final VectorGroup = "Dd04");
              Modelica.Electrical.MultiPhase.Basic.Delta Delta1(final m = m) annotation(
                Placement(transformation(extent = {{-50, -30}, {-30, -10}}, rotation = 0)));
              Modelica.Electrical.MultiPhase.Basic.Delta Delta2(final m = m) annotation(
                Placement(transformation(extent = {{30, 10}, {50, 30}}, rotation = 0)));
            equation
              connect(r1.plug_p, Delta1.plug_p) annotation(
                Line(points = {{-90, 0}, {-90, -20}, {-50, -20}}, color = {0, 0, 255}));
              connect(Delta2.plug_n, r2.plug_p) annotation(
                Line(points = {{50, 20}, {90, 20}, {90, 0}}, color = {0, 0, 255}));
              connect(l1sigma.plug_n, core.plug_p1) annotation(
                Line(points = {{-50, 0}, {-50, 20}, {-10, 20}, {-10, 5}}, color = {0, 0, 255}));
              connect(Delta1.plug_n, core.plug_n1) annotation(
                Line(points = {{-30, -20}, {-10, -20}, {-10, -5}}, color = {0, 0, 255}));
              connect(core.plug_n2, core.plug_p3) annotation(
                Line(points = {{10, 4}, {10, 1}, {10, 1}, {10, -4}}, color = {0, 0, 255}));
              connect(core.plug_p2, Delta2.plug_p) annotation(
                Line(points = {{10, 10}, {10, 20}, {30, 20}}, color = {0, 0, 255}));
              connect(core.plug_n3, l2sigma.plug_n) annotation(
                Line(points = {{10, -10}, {10, -20}, {50, -20}, {50, 0}}, color = {0, 0, 255}));
              annotation(
                defaultComponentName = "transformer",
                Documentation(info = "<html>
Transformer Dd4
<br>Typical parameters see:
<a href=\"modelica://Modelica.Electrical.Machines.Interfaces.PartialBasicTransformer\">PartialBasicTransformer</a>
</html>"));
            end Dd04;

            model Dd06 "Transformer Dd6"
              extends Machines.Interfaces.PartialBasicTransformer(final VectorGroup = "Dd06");
              Modelica.Electrical.MultiPhase.Basic.Delta Delta1(final m = m) annotation(
                Placement(transformation(extent = {{-50, -30}, {-30, -10}}, rotation = 0)));
              Modelica.Electrical.MultiPhase.Basic.Delta Delta2(final m = m) annotation(
                Placement(transformation(extent = {{50, 10}, {30, 30}}, rotation = 0)));
            equation
              connect(Delta2.plug_p, r2.plug_p) annotation(
                Line(points = {{50, 20}, {90, 20}, {90, 0}}, color = {0, 0, 255}));
              connect(Delta1.plug_p, r1.plug_p) annotation(
                Line(points = {{-50, -20}, {-90, -20}, {-90, 0}}, color = {0, 0, 255}));
              connect(l1sigma.plug_n, core.plug_p1) annotation(
                Line(points = {{-50, 0}, {-50, 20}, {-10, 20}, {-10, 5}}, color = {0, 0, 255}));
              connect(Delta1.plug_n, core.plug_n1) annotation(
                Line(points = {{-30, -20}, {-10, -20}, {-10, -5}}, color = {0, 0, 255}));
              connect(core.plug_n2, core.plug_p3) annotation(
                Line(points = {{10, 4}, {10, -4}}, color = {0, 0, 255}));
              connect(Delta2.plug_n, core.plug_p2) annotation(
                Line(points = {{30, 20}, {10, 20}, {10, 10}}, color = {0, 0, 255}));
              connect(core.plug_n3, l2sigma.plug_n) annotation(
                Line(points = {{10, -10}, {10, -20}, {50, -20}, {50, 0}}, color = {0, 0, 255}));
              annotation(
                defaultComponentName = "transformer",
                Documentation(info = "<html>
Transformer Dd6
<br>Typical parameters see:
<a href=\"modelica://Modelica.Electrical.Machines.Interfaces.PartialBasicTransformer\">PartialBasicTransformer</a>
</html>"));
            end Dd06;

            model Dd08 "Transformer Dd8"
              extends Machines.Interfaces.PartialBasicTransformer(final VectorGroup = "Dd08");
              Modelica.Electrical.MultiPhase.Basic.Delta Delta1(final m = m) annotation(
                Placement(transformation(extent = {{-50, -30}, {-30, -10}}, rotation = 0)));
              Modelica.Electrical.MultiPhase.Basic.Delta Rot2(final m = m) annotation(
                Placement(transformation(extent = {{50, 10}, {30, 30}}, rotation = 0)));
              Modelica.Electrical.MultiPhase.Basic.Delta Delta2(final m = m) annotation(
                Placement(transformation(extent = {{30, -30}, {50, -10}}, rotation = 0)));
            equation
              connect(r1.plug_p, Delta1.plug_p) annotation(
                Line(points = {{-90, 0}, {-90, -20}, {-50, -20}}, color = {0, 0, 255}));
              connect(Delta2.plug_n, r2.plug_p) annotation(
                Line(points = {{50, -20}, {90, -20}, {90, 0}}, color = {0, 0, 255}));
              connect(Rot2.plug_p, l2sigma.plug_n) annotation(
                Line(points = {{50, 20}, {50, 0}}, color = {0, 0, 255}));
              connect(l1sigma.plug_n, core.plug_p1) annotation(
                Line(points = {{-50, 0}, {-50, 20}, {-10, 20}, {-10, 5}}, color = {0, 0, 255}));
              connect(core.plug_n1, Delta1.plug_n) annotation(
                Line(points = {{-10, -5}, {-10, -20}, {-30, -20}}, color = {0, 0, 255}));
              connect(core.plug_n2, core.plug_p3) annotation(
                Line(points = {{10, 4}, {10, -4}}, color = {0, 0, 255}));
              connect(core.plug_p2, Rot2.plug_n) annotation(
                Line(points = {{10, 10}, {10, 20}, {30, 20}}, color = {0, 0, 255}));
              connect(core.plug_n3, Delta2.plug_p) annotation(
                Line(points = {{10, -10}, {10, -20}, {30, -20}}, color = {0, 0, 255}));
              annotation(
                defaultComponentName = "transformer",
                Documentation(info = "<html>
Transformer Dd8
<br>Typical parameters see:
<a href=\"modelica://Modelica.Electrical.Machines.Interfaces.PartialBasicTransformer\">PartialBasicTransformer</a>
</html>"));
            end Dd08;

            model Dd10 "Transformer Dd10"
              extends Machines.Interfaces.PartialBasicTransformer(final VectorGroup = "Dd10");
              Modelica.Electrical.MultiPhase.Basic.Delta Delta1(final m = m) annotation(
                Placement(transformation(extent = {{-50, -30}, {-30, -10}}, rotation = 0)));
              Modelica.Electrical.MultiPhase.Basic.Delta Delta2(final m = m) annotation(
                Placement(transformation(extent = {{30, -30}, {50, -10}}, rotation = 0)));
            equation
              connect(r1.plug_p, Delta1.plug_p) annotation(
                Line(points = {{-90, 0}, {-90, -20}, {-50, -20}}, color = {0, 0, 255}));
              connect(Delta2.plug_n, r2.plug_p) annotation(
                Line(points = {{50, -20}, {90, -20}, {90, 0}}, color = {0, 0, 255}));
              connect(l1sigma.plug_n, core.plug_p1) annotation(
                Line(points = {{-50, 0}, {-50, 20}, {-10, 20}, {-10, 5}}, color = {0, 0, 255}));
              connect(Delta1.plug_n, core.plug_n1) annotation(
                Line(points = {{-30, -20}, {-10, -20}, {-10, -5}}, color = {0, 0, 255}));
              connect(core.plug_n2, core.plug_p3) annotation(
                Line(points = {{10, 4}, {10, -4}}, color = {0, 0, 255}));
              connect(core.plug_p2, l2sigma.plug_n) annotation(
                Line(points = {{10, 10}, {10, 20}, {50, 20}, {50, 0}}, color = {0, 0, 255}));
              connect(core.plug_n3, Delta2.plug_p) annotation(
                Line(points = {{10, -10}, {10, -20}, {30, -20}}, color = {0, 0, 255}));
              annotation(
                defaultComponentName = "transformer",
                Documentation(info = "<html>
Transformer Dd10
<br>Typical parameters see:
<a href=\"modelica://Modelica.Electrical.Machines.Interfaces.PartialBasicTransformer\">PartialBasicTransformer</a>
</html>"));
            end Dd10;
            annotation(
              Documentation(info = "<HTML>
This package contains transformers primary D connected / secondary d connected in all possible vector groups.
</HTML>", revisions = "<HTML>
<dl>
  <dt><b>Main Authors:</b></dt>
  <dd>
  <a href=\"http://www.haumer.at/\">Anton Haumer</a><br>
  Technical Consulting &amp; Electrical Engineering<br>
  A-3423 St.Andrae-Woerdern<br>Austria<br>
  email: <a href=\"mailto:a.haumer@haumer.at\">a.haumer@haumer.at</a>
  </dd>
  <dt><b>Copyright:</b></dt>
  <dd>Copyright &copy; 1998-2015, Modelica Association and Anton Haumer.<br>
  <i>The Modelica package is <b>free</b> software; it can be redistributed and/or modified
  under the terms of the <b>Modelica license</b>, see the license conditions
  and the accompanying <b>disclaimer</b> in the documentation of package
  Modelica in file \"Modelica/package.mo\".</i></dd>
</dl>
  <ul>
  <li> v1.0.0 2006/11/19 Anton Haumer</li>
  </ul>
</HTML>"));
          end Dd;

          package Dz "Transformers: primary D / secondary zig-zag"
            extends Modelica.Icons.VariantsPackage;

            model Dz00 "Transformer Dz0"
              extends Machines.Interfaces.PartialBasicTransformer(final VectorGroup = "Dz00");
              Modelica.Electrical.MultiPhase.Basic.Star star2(final m = m) annotation(
                Placement(transformation(origin = {10, -80}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
              Modelica.Electrical.Analog.Interfaces.NegativePin starpoint2 annotation(
                Placement(transformation(extent = {{40, -110}, {60, -90}}, rotation = 0)));
              Modelica.Electrical.MultiPhase.Basic.Delta Rot21(final m = m) annotation(
                Placement(transformation(origin = {30, 0}, extent = {{-10, 10}, {10, -10}}, rotation = 90)));
              Modelica.Electrical.MultiPhase.Basic.Delta Rot22(final m = m) annotation(
                Placement(transformation(origin = {40, 20}, extent = {{-10, 10}, {10, -10}}, rotation = 180)));
              Modelica.Electrical.MultiPhase.Basic.Delta Delta1(final m = m) annotation(
                Placement(transformation(extent = {{-50, -30}, {-30, -10}}, rotation = 0)));
            equation
              connect(star2.pin_n, starpoint2) annotation(
                Line(points = {{10, -90}, {10, -100}, {50, -100}}, color = {0, 0, 255}));
              connect(r1.plug_p, Delta1.plug_p) annotation(
                Line(points = {{-90, 0}, {-90, -20}, {-50, -20}}, color = {0, 0, 255}));
              connect(l1sigma.plug_n, core.plug_p1) annotation(
                Line(points = {{-50, 0}, {-50, 20}, {-10, 20}, {-10, 5}, {-10, 5}}, color = {0, 0, 255}));
              connect(Delta1.plug_n, core.plug_n1) annotation(
                Line(points = {{-30, -20}, {-10, -20}, {-10, -5}}, color = {0, 0, 255}));
              connect(core.plug_p2, Rot21.plug_n) annotation(
                Line(points = {{10, 10}, {30, 10}}, color = {0, 0, 255}));
              connect(core.plug_p3, Rot21.plug_p) annotation(
                Line(points = {{10, -4}, {20, -4}, {20, -10}, {30, -10}}, color = {0, 0, 255}));
              connect(l2sigma.plug_n, Rot22.plug_p) annotation(
                Line(points = {{50, 0}, {50, 20}}, color = {0, 0, 255}));
              connect(Rot22.plug_n, core.plug_n2) annotation(
                Line(points = {{30, 20}, {20, 20}, {20, 4}, {10, 4}}, color = {0, 0, 255}));
              connect(core.plug_n3, star2.plug_p) annotation(
                Line(points = {{10, -10}, {10, -70}, {10, -70}}, color = {0, 0, 255}));
              annotation(
                defaultComponentName = "transformer",
                Documentation(info = "<html>
Transformer Dz0
<br>Typical parameters see:
<a href=\"modelica://Modelica.Electrical.Machines.Interfaces.PartialBasicTransformer\">PartialBasicTransformer</a>
</html>"));
            end Dz00;

            model Dz02 "Transformer Dz2"
              extends Machines.Interfaces.PartialBasicTransformer(final VectorGroup = "Dz02");
              Modelica.Electrical.MultiPhase.Basic.Star star2(final m = m) annotation(
                Placement(transformation(origin = {10, -80}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
              Modelica.Electrical.Analog.Interfaces.NegativePin starpoint2 annotation(
                Placement(transformation(extent = {{40, -110}, {60, -90}}, rotation = 0)));
              Modelica.Electrical.MultiPhase.Basic.Delta Rot21(final m = m) annotation(
                Placement(transformation(origin = {30, 0}, extent = {{10, -10}, {-10, 10}}, rotation = 90)));
              Modelica.Electrical.MultiPhase.Basic.Delta Rot22(final m = m) annotation(
                Placement(transformation(origin = {40, 20}, extent = {{-10, 10}, {10, -10}}, rotation = 180)));
              Modelica.Electrical.MultiPhase.Basic.Delta Delta1(final m = m) annotation(
                Placement(transformation(extent = {{-50, -30}, {-30, -10}}, rotation = 0)));
            equation
              connect(star2.pin_n, starpoint2) annotation(
                Line(points = {{10, -90}, {10, -100}, {50, -100}}, color = {0, 0, 255}));
              connect(r1.plug_p, Delta1.plug_p) annotation(
                Line(points = {{-90, 0}, {-90, -20}, {-50, -20}}, color = {0, 0, 255}));
              connect(l1sigma.plug_n, core.plug_p1) annotation(
                Line(points = {{-50, 0}, {-50, 20}, {-10, 20}, {-10, 5}}, color = {0, 0, 255}));
              connect(Delta1.plug_n, core.plug_n1) annotation(
                Line(points = {{-30, -20}, {-10, -20}, {-10, -5}}, color = {0, 0, 255}));
              connect(core.plug_p2, Rot21.plug_p) annotation(
                Line(points = {{10, 10}, {30, 10}}, color = {0, 0, 255}));
              connect(core.plug_p3, Rot21.plug_n) annotation(
                Line(points = {{10, -4}, {20, -4}, {20, -10}, {30, -10}}, color = {0, 0, 255}));
              connect(l2sigma.plug_n, Rot22.plug_p) annotation(
                Line(points = {{50, 0}, {50, 20}}, color = {0, 0, 255}));
              connect(Rot22.plug_n, core.plug_n2) annotation(
                Line(points = {{30, 20}, {20, 20}, {20, 4}, {10, 4}}, color = {0, 0, 255}));
              connect(core.plug_n3, star2.plug_p) annotation(
                Line(points = {{10, -10}, {10, -40}, {10, -70}, {10, -70}}, color = {0, 0, 255}));
              annotation(
                defaultComponentName = "transformer",
                Documentation(info = "<html>
Transformer Dz2
<br>Typical parameters see:
<a href=\"modelica://Modelica.Electrical.Machines.Interfaces.PartialBasicTransformer\">PartialBasicTransformer</a>
</html>"));
            end Dz02;

            model Dz04 "Transformer Dz4"
              extends Machines.Interfaces.PartialBasicTransformer(final VectorGroup = "Dz04");
              Modelica.Electrical.MultiPhase.Basic.Star star2(final m = m) annotation(
                Placement(transformation(origin = {10, -80}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
              Modelica.Electrical.Analog.Interfaces.NegativePin starpoint2 annotation(
                Placement(transformation(extent = {{40, -110}, {60, -90}}, rotation = 0)));
              Modelica.Electrical.MultiPhase.Basic.Delta Rot21(final m = m) annotation(
                Placement(transformation(origin = {30, 0}, extent = {{-10, 10}, {10, -10}}, rotation = 90)));
              Modelica.Electrical.MultiPhase.Basic.Delta Delta1(final m = m) annotation(
                Placement(transformation(extent = {{-50, -30}, {-30, -10}}, rotation = 0)));
            equation
              connect(star2.pin_n, starpoint2) annotation(
                Line(points = {{10, -90}, {10, -100}, {50, -100}}, color = {0, 0, 255}));
              connect(r1.plug_p, Delta1.plug_p) annotation(
                Line(points = {{-90, 0}, {-90, -20}, {-50, -20}}, color = {0, 0, 255}));
              connect(l1sigma.plug_n, core.plug_p1) annotation(
                Line(points = {{-50, 0}, {-50, 20}, {-10, 20}, {-10, 5}}, color = {0, 0, 255}));
              connect(Delta1.plug_n, core.plug_n1) annotation(
                Line(points = {{-30, -20}, {-10, -20}, {-10, -5}}, color = {0, 0, 255}));
              connect(core.plug_p2, Rot21.plug_n) annotation(
                Line(points = {{10, 10}, {30, 10}}, color = {0, 0, 255}));
              connect(core.plug_p3, Rot21.plug_p) annotation(
                Line(points = {{10, -4}, {20, -4}, {20, -10}, {30, -10}}, color = {0, 0, 255}));
              connect(l2sigma.plug_n, core.plug_n2) annotation(
                Line(points = {{50, 0}, {50, 20}, {20, 20}, {20, 4}, {10, 4}}, color = {0, 0, 255}));
              connect(core.plug_n3, star2.plug_p) annotation(
                Line(points = {{10, -10}, {10, -70}, {10, -70}}, color = {0, 0, 255}));
              annotation(
                defaultComponentName = "transformer",
                Documentation(info = "<html>
Transformer Dz4
<br>Typical parameters see:
<a href=\"modelica://Modelica.Electrical.Machines.Interfaces.PartialBasicTransformer\">PartialBasicTransformer</a>
</html>"));
            end Dz04;

            model Dz06 "Transformer Dz6"
              extends Machines.Interfaces.PartialBasicTransformer(final VectorGroup = "Dz06");
              Modelica.Electrical.MultiPhase.Basic.Star star2(final m = m) annotation(
                Placement(transformation(origin = {10, -80}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
              Modelica.Electrical.Analog.Interfaces.NegativePin starpoint2 annotation(
                Placement(transformation(extent = {{40, -110}, {60, -90}}, rotation = 0)));
              Modelica.Electrical.MultiPhase.Basic.Delta Rot21(final m = m) annotation(
                Placement(transformation(origin = {30, 0}, extent = {{10, -10}, {-10, 10}}, rotation = 90)));
              Modelica.Electrical.MultiPhase.Basic.Delta Delta1(final m = m) annotation(
                Placement(transformation(extent = {{-50, -30}, {-30, -10}}, rotation = 0)));
            equation
              connect(star2.pin_n, starpoint2) annotation(
                Line(points = {{10, -90}, {10, -100}, {50, -100}}, color = {0, 0, 255}));
              connect(r1.plug_p, Delta1.plug_p) annotation(
                Line(points = {{-90, 0}, {-90, -20}, {-50, -20}}, color = {0, 0, 255}));
              connect(l1sigma.plug_n, core.plug_p1) annotation(
                Line(points = {{-50, 0}, {-50, 20}, {-10, 20}, {-10, 5}}, color = {0, 0, 255}));
              connect(Delta1.plug_n, core.plug_n1) annotation(
                Line(points = {{-30, -20}, {-10, -20}, {-10, -5}}, color = {0, 0, 255}));
              connect(core.plug_p2, Rot21.plug_p) annotation(
                Line(points = {{10, 10}, {30, 10}}, color = {0, 0, 255}));
              connect(core.plug_p3, Rot21.plug_n) annotation(
                Line(points = {{10, -4}, {20, -4}, {20, -10}, {30, -10}}, color = {0, 0, 255}));
              connect(core.plug_n3, star2.plug_p) annotation(
                Line(points = {{10, -10}, {10, -40}, {10, -70}, {10, -70}}, color = {0, 0, 255}));
              connect(l2sigma.plug_n, core.plug_n2) annotation(
                Line(points = {{50, 0}, {50, 20}, {20, 20}, {20, 4}, {10, 4}}, color = {0, 0, 255}));
              annotation(
                defaultComponentName = "transformer",
                Documentation(info = "<html>
Transformer Dz6
<br>Typical parameters see:
<a href=\"modelica://Modelica.Electrical.Machines.Interfaces.PartialBasicTransformer\">PartialBasicTransformer</a>
</html>"));
            end Dz06;

            model Dz08 "Transformer Dz8"
              extends Machines.Interfaces.PartialBasicTransformer(final VectorGroup = "Dz08");
              Modelica.Electrical.MultiPhase.Basic.Star star2(final m = m) annotation(
                Placement(transformation(origin = {10, -80}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
              Modelica.Electrical.Analog.Interfaces.NegativePin starpoint2 annotation(
                Placement(transformation(extent = {{40, -110}, {60, -90}}, rotation = 0)));
              Modelica.Electrical.MultiPhase.Basic.Delta Rot21(final m = m) annotation(
                Placement(transformation(origin = {30, 0}, extent = {{-10, 10}, {10, -10}}, rotation = 90)));
              Modelica.Electrical.MultiPhase.Basic.Delta Rot22(final m = m) annotation(
                Placement(transformation(origin = {40, 20}, extent = {{10, 10}, {-10, -10}}, rotation = 180)));
              Modelica.Electrical.MultiPhase.Basic.Delta Delta1(final m = m) annotation(
                Placement(transformation(extent = {{-50, -30}, {-30, -10}}, rotation = 0)));
            equation
              connect(star2.pin_n, starpoint2) annotation(
                Line(points = {{10, -90}, {10, -100}, {50, -100}}, color = {0, 0, 255}));
              connect(r1.plug_p, Delta1.plug_p) annotation(
                Line(points = {{-90, 0}, {-90, -20}, {-50, -20}}, color = {0, 0, 255}));
              connect(l1sigma.plug_n, core.plug_p1) annotation(
                Line(points = {{-50, 0}, {-50, 20}, {-10, 20}, {-10, 5}, {-10, 5}}, color = {0, 0, 255}));
              connect(Delta1.plug_n, core.plug_n1) annotation(
                Line(points = {{-30, -20}, {-10, -20}, {-10, -5}}, color = {0, 0, 255}));
              connect(core.plug_p2, Rot21.plug_n) annotation(
                Line(points = {{10, 10}, {30, 10}}, color = {0, 0, 255}));
              connect(core.plug_p3, Rot21.plug_p) annotation(
                Line(points = {{10, -4}, {20, -4}, {20, -10}, {30, -10}}, color = {0, 0, 255}));
              connect(core.plug_n3, star2.plug_p) annotation(
                Line(points = {{10, -10}, {10, -70}, {10, -70}}, color = {0, 0, 255}));
              connect(l2sigma.plug_n, Rot22.plug_n) annotation(
                Line(points = {{50, 0}, {50, 20}, {50, 20}}, color = {0, 0, 255}));
              connect(Rot22.plug_p, core.plug_n2) annotation(
                Line(points = {{30, 20}, {20, 20}, {20, 4}, {10, 4}}, color = {0, 0, 255}));
              annotation(
                defaultComponentName = "transformer",
                Documentation(info = "<html>
Transformer Dz8
<br>Typical parameters see:
<a href=\"modelica://Modelica.Electrical.Machines.Interfaces.PartialBasicTransformer\">PartialBasicTransformer</a>
</html>"));
            end Dz08;

            model Dz10 "Transformer Dz10"
              extends Machines.Interfaces.PartialBasicTransformer(final VectorGroup = "Dz10");
              Modelica.Electrical.MultiPhase.Basic.Star star2(final m = m) annotation(
                Placement(transformation(origin = {10, -80}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
              Modelica.Electrical.Analog.Interfaces.NegativePin starpoint2 annotation(
                Placement(transformation(extent = {{40, -110}, {60, -90}}, rotation = 0)));
              Modelica.Electrical.MultiPhase.Basic.Delta Rot21(final m = m) annotation(
                Placement(transformation(origin = {30, 0}, extent = {{10, -10}, {-10, 10}}, rotation = 90)));
              Modelica.Electrical.MultiPhase.Basic.Delta Rot22(final m = m) annotation(
                Placement(transformation(origin = {40, 20}, extent = {{10, 10}, {-10, -10}}, rotation = 180)));
              Modelica.Electrical.MultiPhase.Basic.Delta Delta1(final m = m) annotation(
                Placement(transformation(extent = {{-50, -30}, {-30, -10}}, rotation = 0)));
            equation
              connect(star2.pin_n, starpoint2) annotation(
                Line(points = {{10, -90}, {10, -100}, {50, -100}}, color = {0, 0, 255}));
              connect(r1.plug_p, Delta1.plug_p) annotation(
                Line(points = {{-90, 0}, {-90, -20}, {-50, -20}}, color = {0, 0, 255}));
              connect(l1sigma.plug_n, core.plug_p1) annotation(
                Line(points = {{-50, 0}, {-50, 20}, {-10, 20}, {-10, 5}}, color = {0, 0, 255}));
              connect(Delta1.plug_n, core.plug_n1) annotation(
                Line(points = {{-30, -20}, {-10, -20}, {-10, -5}}, color = {0, 0, 255}));
              connect(core.plug_p2, Rot21.plug_p) annotation(
                Line(points = {{10, 10}, {30, 10}}, color = {0, 0, 255}));
              connect(core.plug_p3, Rot21.plug_n) annotation(
                Line(points = {{10, -4}, {20, -4}, {20, -10}, {30, -10}}, color = {0, 0, 255}));
              connect(core.plug_n3, star2.plug_p) annotation(
                Line(points = {{10, -10}, {10, -70}, {10, -70}}, color = {0, 0, 255}));
              connect(l2sigma.plug_n, Rot22.plug_n) annotation(
                Line(points = {{50, 0}, {50, 20}}, color = {0, 0, 255}));
              connect(Rot22.plug_p, core.plug_n2) annotation(
                Line(points = {{30, 20}, {20, 20}, {20, 4}, {10, 4}}, color = {0, 0, 255}));
              annotation(
                defaultComponentName = "transformer",
                Documentation(info = "<html>
Transformer Dz10
<br>Typical parameters see:
<a href=\"modelica://Modelica.Electrical.Machines.Interfaces.PartialBasicTransformer\">PartialBasicTransformer</a>
</html>"));
            end Dz10;
            annotation(
              Documentation(info = "<HTML>
This package contains transformers primary D connected / secondary d connected in all possible vector groups.
</HTML>", revisions = "<HTML>
<dl>
  <dt><b>Main Authors:</b></dt>
  <dd>
  <a href=\"http://www.haumer.at/\">Anton Haumer</a><br>
  Technical Consulting &amp; Electrical Engineering<br>
  A-3423 St.Andrae-Woerdern<br>Austria<br>
  email: <a href=\"mailto:a.haumer@haumer.at\">a.haumer@haumer.at</a>
  </dd>
  <dt><b>Copyright:</b></dt>
  <dd>Copyright &copy; 1998-2015, Modelica Association and Anton Haumer.<br>
  <i>The Modelica package is <b>free</b> software; it can be redistributed and/or modified
  under the terms of the <b>Modelica license</b>, see the license conditions
  and the accompanying <b>disclaimer</b> in the documentation of package
  Modelica in file \"Modelica/package.mo\".</i></dd>
</dl>
  <ul>
  <li> v1.0.0 2006/11/19 Anton Haumer</li>
  </ul>
</HTML>"));
          end Dz;
          annotation(
            Documentation(info = "<HTML>
This package contains components to model technical three-phase transformers:
<ul>
<li>Transformer: transformer model to choose connection / vector group</li>
<li>Yy: Transformers with primary primary Y / secondary y</li>
<li>Yd: Transformers with primary primary Y / secondary d</li>
<li>Yz: Transformers with primary primary Y / secondary zig-zag</li>
<li>Dy: Transformers with primary primary D / secondary y</li>
<li>Dd: Transformers with primary D / secondary d</li>
<li>Dz: Transformers with primary D / secondary zig-zag</li>
</ul>
<p>
Transformers are modeled by an ideal transformer, adding primary and secondary winding resistances and stray inductances.<br>
All transformers extend from the base model <i>PartialTransformer</i>, adding the primary and secondary connection.<br>
<b>VectorGroup</b> defines the phase shift between primary and secondary voltages, expressed by a number phase shift/30 degree
(i.e., the hour on a clock face). Therefore each transformer is identified by two characters and a two-digit number,
e.g., Yd11 ... primary connection Y (star), secondary connection d (delta), vector group 11 (phase shift 330 degree)<br>
With the \"supermodel\" <i>Transformer</i>&nbsp; the user may choose primary and secondary connection as well as the vector group.<br>
It calculates winding ratio as well as primary and secondary winding resistances and stray inductances,
distributing them equally to primary and secondary winding, from the following parameters:
</p>
<ul>
<li>nominal frequency</li>
<li>primary voltage (RMS line-to-line)</li>
<li>secondary voltage (RMS line-to-line)</li>
<li>nominal apparent power</li>
<li>impedance voltage drop</li>
<li>short-circuit copper losses</li>
</ul>
The <b>impedance voltage drop</b> indicates the (absolute value of the) voltage drop at nominal load (current) as well as
the voltage we have to apply to the primary winding to achieve nominal current in the short-circuited secondary winding.
<p>
<b>Please pay attention</b> to proper grounding of the primary and secondary part of the whole circuit.<br>
The primary and secondary starpoint are available as connectors, if the connection is not delta (D or d).<br>
<b>In some cases (Yy or Yz) it may be necessary to ground one of the transformer's starpoints
even though the source's and/or load's starpoint are grounded; you may use a reasonable high earthing resistance.</b>
</p>
<b>Limitations and assumptions:</b><br>
<ul>
<li>number of phases is limited to 3, therefore definition as a constant m=3</li>
<li>symmetry of the 3 phases resp. limbs</li>
<li>saturation is neglected, i.e., inductances are constant</li>
<li>magnetizing current is neglected</li>
<li>magnetizing losses are neglected</li>
<li>additional (stray) losses are neglected</li>
</ul>
<b>Further development:</b>
<ul>
<li>modeling magnetizing current, including saturation</li>
<li>temperature dependency of winding resistances</li>
</ul>
<dl>
  <dt><b>Main Authors:</b></dt>
  <dd>
  <a href=\"http://www.haumer.at/\">Anton Haumer</a><br>
  Technical Consulting &amp; Electrical Engineering<br>
  A-3423 St.Andrae-Woerdern<br>Austria<br>
  email: <a href=\"mailto:a.haumer@haumer.at\">a.haumer@haumer.at</a>
  </dd>
</dl>
<p>
Copyright &copy; 1998-2015, Modelica Association and Anton Haumer.
</p>

<p>
<i>This Modelica package is <u>free</u> software and the use is completely at <u>your own risk</u>; it can be redistributed and/or modified under the terms of the Modelica License 2. For license conditions (including the disclaimer of warranty) see <a href=\"modelica://Modelica.UsersGuide.ModelicaLicense2\">Modelica.UsersGuide.ModelicaLicense2</a> or visit <a href=\"https://www.modelica.org/licenses/ModelicaLicense2\"> https://www.modelica.org/licenses/ModelicaLicense2</a>.</i>
</p>
</HTML>", revisions = "<HTML>
  <ul>
  <li> v1.0.0 2006/11/19 Anton Haumer<br>
       first stable release</li>
  <li> v2.2.0 2011/02/10 Anton Haumer<br>
       conditional ThermalPort for all machines</li>
  </ul>
</HTML>"),
            Icon(coordinateSystem(extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(origin = {10, 10}, fillColor = {135, 135, 135}, fillPattern = FillPattern.VerticalCylinder, points = {{-80, 50}, {-60, 30}, {-60, -50}, {-80, -70}, {-80, 50}}), Polygon(origin = {10, 10}, fillColor = {135, 135, 135}, fillPattern = FillPattern.VerticalCylinder, points = {{60, 50}, {40, 30}, {40, -50}, {60, -70}, {60, 50}}), Polygon(origin = {10, 10}, fillColor = {135, 135, 135}, fillPattern = FillPattern.VerticalCylinder, points = {{-10, 40}, {-20, 30}, {-20, -50}, {-10, -60}, {0, -50}, {0, 30}, {-10, 40}}), Polygon(origin = {10, 10}, fillColor = {135, 135, 135}, fillPattern = FillPattern.VerticalCylinder, points = {{-80, 50}, {60, 50}, {40, 30}, {0, 30}, {-10, 40}, {-20, 30}, {-60, 30}, {-80, 50}}), Polygon(origin = {10, 10}, fillColor = {135, 135, 135}, fillPattern = FillPattern.VerticalCylinder, points = {{-80, -70}, {60, -70}, {40, -50}, {0, -50}, {-10, -60}, {-20, -50}, {-60, -50}, {-80, -70}}), Rectangle(origin = {10, 10}, lineColor = {128, 0, 255}, fillColor = {128, 0, 255}, fillPattern = FillPattern.VerticalCylinder, extent = {{-88, -46}, {-52, 26}}), Rectangle(origin = {10, 10}, lineColor = {0, 128, 255}, fillColor = {0, 128, 255}, fillPattern = FillPattern.VerticalCylinder, extent = {{-94, -38}, {-46, 18}}), Rectangle(origin = {10, 10}, lineColor = {128, 0, 255}, fillColor = {128, 0, 255}, fillPattern = FillPattern.VerticalCylinder, extent = {{-28, -46}, {8, 26}}), Rectangle(origin = {10, 10}, lineColor = {0, 128, 255}, fillColor = {0, 128, 255}, fillPattern = FillPattern.VerticalCylinder, extent = {{-34, -38}, {14, 18}}), Rectangle(origin = {10, 10}, lineColor = {128, 0, 255}, fillColor = {128, 0, 255}, fillPattern = FillPattern.VerticalCylinder, extent = {{32, -46}, {68, 26}}), Rectangle(origin = {10, 10}, lineColor = {0, 128, 255}, fillColor = {0, 128, 255}, fillPattern = FillPattern.VerticalCylinder, extent = {{26, -38}, {74, 18}})}));
        end Transformers;

        package Components "Machine components like AirGaps"
          extends Modelica.Icons.Package;

          partial model PartialAirGap "Partial airgap model"
            parameter Integer m = 3 "Number of phases";
            parameter Integer p(min = 1) "Number of pole pairs";
            output Modelica.SIunits.Torque tauElectrical;
            Modelica.SIunits.Angle gamma "Rotor displacement angle";
            Modelica.SIunits.Current i_ss[2] "Stator current space phasor with respect to the stator fixed frame";
            Modelica.SIunits.Current i_sr[2] "Stator current space phasor with respect to the rotor fixed frame";
            Modelica.SIunits.Current i_rs[2] "Rotor current space phasor with respect to the stator fixed frame";
            Modelica.SIunits.Current i_rr[2] "Rotor current space phasor with respect to the rotor fixed frame";
            Modelica.SIunits.MagneticFlux psi_ms[2] "Magnetizing flux phasor with respect to the stator fixed frame";
            Modelica.SIunits.MagneticFlux psi_mr[2] "Magnetizing flux phasor with respect to the rotor fixed frame";
            Real RotationMatrix[2, 2] "Matrix of rotation from rotor to stator";
          public
            Modelica.Mechanics.Rotational.Interfaces.Flange_a flange annotation(
              Placement(transformation(extent = {{-10, 110}, {10, 90}}, rotation = 0)));
            Modelica.Mechanics.Rotational.Interfaces.Flange_a support "Support at which the reaction torque is acting" annotation(
              Placement(transformation(extent = {{-10, -110}, {10, -90}}, rotation = 0)));
            Machines.Interfaces.SpacePhasor spacePhasor_s annotation(
              Placement(transformation(extent = {{-110, 90}, {-90, 110}}, rotation = 0)));
            Machines.Interfaces.SpacePhasor spacePhasor_r annotation(
              Placement(transformation(extent = {{90, 90}, {110, 110}}, rotation = 0)));
            /*
              Modelica.SIunits.AngularVelocity omegaPsi_ms
                "Angular velocity of main flux with respect to the stator fixed frame";
              Modelica.SIunits.AngularVelocity omegaPsi_mr
                "Angular velocity of main flux with respect to the rotor fixed frame";
              Modelica.SIunits.Current i_sm[2]
                "Stator current space phasor with respect to the main flux fixed frame";
              Modelica.SIunits.Current i_rm[2]
                "Rotor current space phasor with respect to the main flux fixed frame";
            protected
              Modelica.SIunits.MagneticFlux psi_msAbs "Length of main flux phasor";
              Modelica.SIunits.Angle psi_msArg
                "(Wrapped) angle of main flux phasor with respect to the stator fixed frame";
              Modelica.SIunits.Angle psi_mrArg
                "(Wrapped) angle of main flux phasor with respect to the rotor fixed frame";
            initial equation
              i_sm = Machines.SpacePhasors.Functions.Rotator(i_ss, psi_msArg);
              i_rm = Machines.SpacePhasors.Functions.Rotator(i_rr, psi_mrArg);
            equation
              // AngularVelocity of main flux phasor
              (psi_msAbs, psi_msArg) = Machines.SpacePhasors.Functions.ToPolar(psi_ms);
              psi_mrArg = psi_msArg - gamma;
              omegaPsi_ms = if noEvent(psi_msAbs<Modelica.Constants.small) then 0 else
                (spacePhasor_s.v_[2]*cos(psi_msArg) - spacePhasor_s.v_[1]*sin(psi_msArg))/psi_msAbs;
              omegaPsi_mr = omegaPsi_ms - der(gamma);
              // stator and rotor current w.r.t. main flux fixed frame
              der(i_sm) = Machines.SpacePhasors.Functions.Rotator(
                {der(i_ss[1]) + omegaPsi_ms*i_ss[2], der(i_ss[2]) - omegaPsi_ms*i_ss[1]}, psi_msArg);
              der(i_rm) = Machines.SpacePhasors.Functions.Rotator(
                {der(i_rr[1]) + omegaPsi_mr*i_rr[2], der(i_rr[2]) - omegaPsi_mr*i_rr[1]}, psi_mrArg);
            */
          equation
// mechanical angle of the rotor of an equivalent 2-pole machine
            gamma = p * (flange.phi - support.phi);
            RotationMatrix = {{+cos(gamma), -sin(gamma)}, {+sin(gamma), +cos(gamma)}};
            i_ss = spacePhasor_s.i_;
            i_ss = RotationMatrix * i_sr;
            i_rr = spacePhasor_r.i_;
            i_rs = RotationMatrix * i_rr;
// Stator voltage induction
            spacePhasor_s.v_ = der(psi_ms);
// Rotor voltage induction
            spacePhasor_r.v_ = der(psi_mr);
// Electromechanical torque (cross product of current and flux space phasor)
            tauElectrical = m / 2 * p * (spacePhasor_s.i_[2] * psi_ms[1] - spacePhasor_s.i_[1] * psi_ms[2]);
            flange.tau = -tauElectrical;
            support.tau = tauElectrical;
            annotation(
              Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-90, 90}, {90, -92}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-80, 80}, {80, -80}}, lineColor = {0, 0, 255}), Rectangle(extent = {{-10, 90}, {10, -80}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.VerticalCylinder, fillColor = {128, 128, 128}), Text(extent = {{-150, -90}, {150, -150}}, lineColor = {0, 0, 255}, textString = "%name")}),
              Documentation(info = "<HTML>
Partial model of the airgap, using only equations.
</HTML>"));
          end PartialAirGap;

          model AirGapS "Airgap in stator-fixed coordinate system"
            parameter Modelica.SIunits.Inductance Lm "Main field inductance";
            extends PartialAirGap;
            Modelica.SIunits.Current i_ms[2] "Magnetizing current space phasor with respect to the stator fixed frame";
          protected
            parameter Modelica.SIunits.Inductance L[2, 2] = {{Lm, 0}, {0, Lm}} "Inductance matrix";
          equation
// Magnetizing current with respect to the stator reference frame
            i_ms = i_ss + i_rs;
// Magnetizing flux linkage with respect to the stator reference frame
            psi_ms = L * i_ms;
// Magnetizing flux linkage with respect to the rotor reference frame
            psi_mr = transpose(RotationMatrix) * psi_ms;
            annotation(
              defaultComponentName = "airGap",
              Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-80, 40}, {0, -40}}, lineColor = {0, 0, 0}, textString = "S"), Text(extent = {{-150, -90}, {150, -150}}, lineColor = {0, 0, 255}, textString = "%name")}),
              Documentation(info = "<HTML>
Model of the airgap in stator-fixed coordinate system, using only equations.
</HTML>"));
          end AirGapS;

          model AirGapR "Airgap in rotor-fixed coordinate system"
            parameter Modelica.SIunits.Inductance Lmd "Main field inductance d-axis";
            parameter Modelica.SIunits.Inductance Lmq "Main field inductance q-axis";
            extends PartialAirGap;
            Modelica.SIunits.Current i_mr[2] "Magnetizing current space phasor with respect to the rotor fixed frame";
          protected
            parameter Modelica.SIunits.Inductance L[2, 2] = {{Lmd, 0}, {0, Lmq}} "Inductance matrix";
          equation
// Magnetizing current with respect to the rotor reference frame
            i_mr = i_sr + i_rr;
// Main flux linkage with respect to the rotor reference frame
            psi_mr = L * i_mr;
// Main flux linkage with respect to the stator reference frame
            psi_ms = RotationMatrix * psi_mr;
            annotation(
              defaultComponentName = "airGap",
              Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{0, 40}, {80, -40}}, lineColor = {0, 0, 0}, textString = "R"), Text(extent = {{-150, -90}, {150, -150}}, lineColor = {0, 0, 255}, textString = "%name")}),
              Documentation(info = "<HTML>
Model of the airgap in rotor-fixed coordinate system, using only equations.
</HTML>"));
          end AirGapR;

          model Inductor "Space phasor inductor"
            parameter Modelica.SIunits.Inductance L[2] "Inductance of both axes";
            Modelica.SIunits.Voltage v_[2];
            Modelica.SIunits.Current i_[2];
            Machines.Interfaces.SpacePhasor spacePhasor_a annotation(
              Placement(transformation(extent = {{-110, -10}, {-90, 10}}, rotation = 0)));
            Machines.Interfaces.SpacePhasor spacePhasor_b annotation(
              Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
          equation
            spacePhasor_a.i_ + spacePhasor_b.i_ = zeros(2);
            v_ = spacePhasor_a.v_ - spacePhasor_b.v_;
            i_ = spacePhasor_a.i_;
            v_[1] = L[1] * der(i_[1]);
            v_[2] = L[2] * der(i_[2]);
            annotation(
              Documentation(info = "<html>
This is a model of an inductor, described with space phasors.
</html>"),
              Icon(graphics = {Ellipse(extent = {{-60, -15}, {-30, 15}}, lineColor = {0, 0, 255}), Ellipse(extent = {{-30, -15}, {0, 15}}, lineColor = {0, 0, 255}), Ellipse(extent = {{0, -15}, {30, 15}}, lineColor = {0, 0, 255}), Ellipse(extent = {{30, -15}, {60, 15}}, lineColor = {0, 0, 255}), Rectangle(extent = {{-60, -30}, {60, 0}}, lineColor = {255, 255, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{-60, 0}, {-90, 0}}, color = {0, 0, 255}, smooth = Smooth.None), Line(points = {{90, 0}, {60, 0}}, color = {0, 0, 255}, smooth = Smooth.None), Text(extent = {{-100, 100}, {100, 60}}, lineColor = {0, 0, 255}, textString = "%name")}));
          end Inductor;

          model SquirrelCage "Squirrel Cage"
            parameter Modelica.SIunits.Inductance Lrsigma "Rotor stray inductance per phase translated to stator";
            parameter Modelica.SIunits.Resistance Rr "Rotor resistance per phase translated to stator at T_ref";
            parameter Modelica.SIunits.Temperature T_ref = 293.15 "Reference temperature";
            parameter Modelica.SIunits.LinearTemperatureCoefficient alpha = 0 "Temperature coefficient of resistance at T_ref";
            extends Modelica.Electrical.Analog.Interfaces.ConditionalHeatPort(T = T_ref);
            Modelica.SIunits.Resistance Rr_actual "Actual resistance = Rr*(1 + alpha*(T_heatPort - T_ref))";
            Machines.Interfaces.SpacePhasor spacePhasor_r annotation(
              Placement(transformation(extent = {{-110, 90}, {-90, 110}}, rotation = 0)));
          equation
            assert(1 + alpha * (T_heatPort - T_ref) >= Modelica.Constants.eps, "Temperature outside scope of model!");
            Rr_actual = Rr * (1 + alpha * (T_heatPort - T_ref));
            spacePhasor_r.v_ = Rr_actual * spacePhasor_r.i_ + Lrsigma * der(spacePhasor_r.i_);
            2 / 3 * LossPower = Rr_actual * (spacePhasor_r.i_[1] * spacePhasor_r.i_[1] + spacePhasor_r.i_[2] * spacePhasor_r.i_[2]);
            annotation(
              Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-60, 45}, {-30, 75}}, lineColor = {0, 0, 255}), Ellipse(extent = {{-30, 45}, {0, 75}}, lineColor = {0, 0, 255}), Ellipse(extent = {{0, 45}, {30, 75}}, lineColor = {0, 0, 255}), Ellipse(extent = {{30, 45}, {60, 75}}, lineColor = {0, 0, 255}), Rectangle(extent = {{-60, 30}, {60, 60}}, lineColor = {255, 255, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{-100, 60}, {-60, 60}}, color = {0, 0, 255}), Line(points = {{60, 60}, {80, 60}, {80, 40}}, color = {0, 0, 255}), Rectangle(extent = {{60, 40}, {100, -40}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{80, -40}, {80, -60}}, color = {0, 0, 255}), Line(points = {{60, -60}, {100, -60}}, color = {0, 0, 255}), Line(points = {{70, -80}, {90, -80}}, color = {0, 0, 255}), Line(points = {{66, -70}, {94, -70}}, color = {0, 0, 255}), Text(extent = {{-150, -90}, {150, -150}}, lineColor = {0, 0, 255}, textString = "%name"), Line(points = {{-100, 90}, {-100, 60}}, color = {0, 0, 255})}),
              Documentation(info = "<HTML>
<p>
Model of a squirrel cage / symmetrical damper cage in two axis.
</p>
<p>
The squirrel cage has an optional (conditional) HeatPort,
which can be enabled or disabled by the Boolean parameter useHeatPort.
Temperatures of both axis are the same, both losses are added.
Material properties alpha of both axis are the same.
</p>
</HTML>"));
          end SquirrelCage;

          model DamperCage "Squirrel Cage"
            parameter Modelica.SIunits.Inductance Lrsigmad "Stray inductance in d-axis per phase translated to stator";
            parameter Modelica.SIunits.Inductance Lrsigmaq "Stray inductance in q-axis per phase translated to stator";
            parameter Modelica.SIunits.Resistance Rrd "Resistance in d-axis per phase translated to stator at T_ref";
            parameter Modelica.SIunits.Resistance Rrq "Resistance in q-axis per phase translated to stator at T_ref";
            parameter Modelica.SIunits.Temperature T_ref = 293.15 "Reference temperature of both resistances in d- and q-axis";
            parameter Modelica.SIunits.LinearTemperatureCoefficient alpha = 0 "Temperature coefficient of both resistances in d- and q-axis at T_ref";
            extends Modelica.Electrical.Analog.Interfaces.ConditionalHeatPort(T = T_ref);
            Modelica.SIunits.Resistance Rrd_actual "Actual resistance = Rrd*(1 + alpha*(T_heatPort - T_ref))";
            Modelica.SIunits.Resistance Rrq_actual "Actual resistance = Rrq*(1 + alpha*(T_heatPort - T_ref))";
            Modelica.Blocks.Interfaces.RealOutput i[2](each final quantity = "ElectricCurrent", each final unit = "A") = -spacePhasor_r.i_ "Currents out from damper";
            Modelica.Blocks.Interfaces.RealOutput lossPower(final quantity = "Power", final unit = "W") = LossPower "Damper losses";
            Machines.Interfaces.SpacePhasor spacePhasor_r annotation(
              Placement(transformation(extent = {{-110, 90}, {-90, 110}}, rotation = 0)));
          equation
            assert(1 + alpha * (T_heatPort - T_ref) >= Modelica.Constants.eps, "Temperature outside scope of model!");
            Rrd_actual = Rrd * (1 + alpha * (T_heatPort - T_ref));
            Rrq_actual = Rrq * (1 + alpha * (T_heatPort - T_ref));
            spacePhasor_r.v_[1] = Rrd_actual * spacePhasor_r.i_[1] + Lrsigmad * der(spacePhasor_r.i_[1]);
            spacePhasor_r.v_[2] = Rrq_actual * spacePhasor_r.i_[2] + Lrsigmaq * der(spacePhasor_r.i_[2]);
            2 / 3 * LossPower = Rrd_actual * spacePhasor_r.i_[1] * spacePhasor_r.i_[1] + Rrq_actual * spacePhasor_r.i_[2] * spacePhasor_r.i_[2];
            annotation(
              Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{60, 60}, {100, 60}}, color = {0, 0, 255}), Line(points = {{70, 40}, {90, 40}}, color = {0, 0, 255}), Line(points = {{66, 50}, {94, 50}}, color = {0, 0, 255}), Text(extent = {{-150, -90}, {150, -150}}, lineColor = {0, 0, 255}, textString = "%name"), Ellipse(extent = {{-60, 90}, {-40, 70}}, lineColor = {0, 0, 255}), Ellipse(extent = {{-40, 90}, {-20, 70}}, lineColor = {0, 0, 255}), Ellipse(extent = {{-20, 90}, {0, 70}}, lineColor = {0, 0, 255}), Line(points = {{-80, 0}, {-80, -20}}, color = {0, 0, 255}), Rectangle(extent = {{20, 90}, {60, 70}}, lineColor = {0, 0, 255}), Line(points = {{60, 80}, {80, 80}, {80, 60}}, color = {0, 0, 255}), Line(points = {{-100, 90}, {-100, 80}, {-60, 80}}, color = {0, 0, 255}), Line(points = {{-100, 90}, {-100, 80}, {-80, 80}, {-80, 60}}, color = {0, 0, 255}), Ellipse(extent = {{-90, 60}, {-70, 40}}, lineColor = {0, 0, 255}), Ellipse(extent = {{-90, 40}, {-70, 20}}, lineColor = {0, 0, 255}), Ellipse(extent = {{-90, 20}, {-70, 0}}, lineColor = {0, 0, 255}), Rectangle(extent = {{-80, 0}, {-70, 60}}, lineColor = {255, 255, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{0, 80}, {20, 80}}, color = {0, 0, 255}), Rectangle(extent = {{-90, -20}, {-70, -60}}, lineColor = {0, 0, 255}), Line(points = {{-100, -80}, {-60, -80}}, color = {0, 0, 255}), Line(points = {{-90, -100}, {-70, -100}}, color = {0, 0, 255}), Line(points = {{-94, -90}, {-66, -90}}, color = {0, 0, 255}), Line(points = {{-80, -60}, {-80, -80}}, color = {0, 0, 255}), Rectangle(extent = {{0, 70}, {-60, 80}}, lineColor = {255, 255, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid)}),
              Documentation(info = "<HTML>
<p>
Model of an asymmetrical damper cage in two axis.
</p>
<p>
The damper cage has an optional (conditional) HeatPort,
which can be enabled or disabled by the Boolean parameter useHeatPort.
Temperatures of both axis are the same, both losses are added.
Material properties alpha can be set differently for both d- and q-axis,
although reference temperature for both resistances is the same.
</p>
</HTML>"));
          end DamperCage;

          model ElectricalExcitation "Electrical excitation"
            parameter Real turnsRatio(start = 1) "Ratio stator current / excitation current";
            Modelica.SIunits.Current ie "Excitation current";
            Modelica.SIunits.Voltage ve "Excitation voltage";
            Machines.Interfaces.SpacePhasor spacePhasor_r annotation(
              Placement(transformation(extent = {{-110, 90}, {-90, 110}}, rotation = 0)));
            Modelica.Electrical.Analog.Interfaces.PositivePin pin_ep annotation(
              Placement(transformation(extent = {{90, 110}, {110, 90}}, rotation = 0)));
            Modelica.Electrical.Analog.Interfaces.NegativePin pin_en annotation(
              Placement(transformation(extent = {{110, -110}, {90, -90}}, rotation = 0)));
          equation
            pin_ep.i + pin_en.i = 0;
            ie = +pin_ep.i;
            ve = pin_ep.v - pin_en.v;
            spacePhasor_r.i_ = {-ie * turnsRatio, 0};
            ve = spacePhasor_r.v_[1] * turnsRatio * 3 / 2;
            annotation(
              Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-150, -90}, {150, -150}}, lineColor = {0, 0, 255}, textString = "%name"), Polygon(points = {{-90, 100}, {-70, 106}, {-70, 94}, {-90, 100}}, lineColor = {0, 0, 255}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-70, 40}, {10, -40}}, lineColor = {0, 0, 255}), Ellipse(extent = {{-10, 40}, {70, -40}}, lineColor = {0, 0, 255}), Line(points = {{-30, 0}, {-54, 32}, {-50, 20}, {-44, 26}, {-54, 32}}, color = {0, 0, 255}), Line(points = {{-30, 0}, {-54, -32}, {-50, -20}, {-44, -26}, {-54, -32}}, color = {0, 0, 255}), Line(points = {{-54, 32}, {-54, 100}, {-70, 100}}, color = {0, 0, 255}), Line(points = {{90, 100}, {30, 100}, {30, 40}}, color = {0, 0, 255}), Line(points = {{30, -40}, {30, -100}, {90, -100}}, color = {0, 0, 255})}),
              Documentation(info = "<HTML>
Model of an electrical excitation, converting excitation to space phasor.
</HTML>"));
          end ElectricalExcitation;

          model PermanentMagnet "Permanent magnet excitation"
            parameter Modelica.SIunits.Current Ie "Equivalent excitation current";
            Machines.Interfaces.SpacePhasor spacePhasor_r annotation(
              Placement(transformation(extent = {{-110, 90}, {-90, 110}}, rotation = 0)));
          equation
            spacePhasor_r.i_ = {-Ie, 0};
            annotation(
              Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-60, 60}, {60, 20}}, lineColor = {255, 0, 0}, fillColor = {255, 0, 0}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-60, -20}, {60, -60}}, lineColor = {0, 255, 0}, fillColor = {0, 255, 0}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-60, 40}, {60, 0}}, lineColor = {255, 0, 0}, fillColor = {255, 0, 0}, fillPattern = FillPattern.Solid), Polygon(points = {{-90, 100}, {-70, 106}, {-70, 94}, {-90, 100}}, lineColor = {0, 0, 255}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-60, 0}, {60, -40}}, lineColor = {0, 255, 0}, pattern = LinePattern.None, lineThickness = 0.5, fillColor = {0, 255, 0}, fillPattern = FillPattern.Solid), Line(points = {{-60, -40}, {-60, 0}}, color = {0, 255, 0}), Line(points = {{60, -40}, {60, 0}}, color = {0, 255, 0}), Line(points = {{0, 100}, {-70, 100}}, color = {0, 0, 255}), Line(points = {{0, 100}, {0, 60}}, color = {255, 0, 0}), Text(extent = {{-150, -90}, {150, -150}}, lineColor = {0, 0, 255}, textString = "%name")}),
              Documentation(info = "<HTML>
Model of a permanent magnet excitation, characterized by an equivalent excitation current.
</HTML>"));
          end PermanentMagnet;

          model PermanentMagnetWithLosses "Permanent magnet excitation"
            extends Machines.BasicMachines.Components.PermanentMagnet;
            extends Machines.Losses.InductionMachines.PermanentMagnetLosses;
          end PermanentMagnetWithLosses;

          model InductorDC "Ideal linear electrical inductor for electrical DC machines"
            extends Modelica.Electrical.Analog.Interfaces.OnePort;
            parameter Modelica.SIunits.Inductance L(start = 1) "Inductance";
            parameter Boolean quasiStationary(start = false) "No electrical transients if true" annotation(
              Evaluate = true);
          equation
            v = if quasiStationary then 0 else L * der(i);
            annotation(
              Documentation(info = "<html>
<p>The linear inductor connects the branch voltage <i>v</i> with the branch current <i>i</i> by <i>v = L * di/dt</i>.
If <code>quasiStationary == false</code>, the electrical transients are neglected, i.e., the voltage drop is zero.</p>
</html>"),
              Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-60, -15}, {-30, 15}}, lineColor = {0, 0, 255}), Ellipse(extent = {{-30, -15}, {0, 15}}, lineColor = {0, 0, 255}), Ellipse(extent = {{0, -15}, {30, 15}}, lineColor = {0, 0, 255}), Ellipse(extent = {{30, -15}, {60, 15}}, lineColor = {0, 0, 255}), Rectangle(extent = {{-60, -30}, {60, 0}}, lineColor = {255, 255, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{60, 0}, {90, 0}}, color = {0, 0, 255}), Line(points = {{-90, 0}, {-60, 0}}, color = {0, 0, 255}), Text(extent = {{-138, -60}, {144, -94}}, lineColor = {0, 0, 0}, textString = "L=%L"), Text(extent = {{-152, 79}, {148, 39}}, textString = "%name", lineColor = {0, 0, 255}), Rectangle(visible = quasiStationary, extent = {{-70, 30}, {70, -30}}, lineColor = {0, 0, 255}, fillColor = {85, 170, 255}, fillPattern = FillPattern.Solid)}),
              Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-60, -15}, {-30, 15}}, lineColor = {0, 0, 255}), Ellipse(extent = {{-30, -15}, {0, 15}}, lineColor = {0, 0, 255}), Ellipse(extent = {{0, -15}, {30, 15}}, lineColor = {0, 0, 255}), Ellipse(extent = {{30, -15}, {60, 15}}, lineColor = {0, 0, 255}), Rectangle(extent = {{-60, -30}, {60, 0}}, lineColor = {255, 255, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{60, 0}, {96, 0}}, color = {0, 0, 255}), Line(points = {{-96, 0}, {-60, 0}}, color = {0, 0, 255})}));
          end InductorDC;

          partial model PartialAirGapDC "Partial airgap model of a DC machine"
            parameter Boolean quasiStationary(start = false) "No electrical transients if true" annotation(
              Evaluate = true);
            parameter Real turnsRatio "Ratio of armature turns over number of turns of the excitation winding";
            output Modelica.SIunits.AngularVelocity w "Angular velocity";
            Modelica.SIunits.Voltage vei "Voltage drop across field excitation inductance";
            Modelica.SIunits.Current ie "Excitation current";
            Modelica.SIunits.MagneticFlux psi_e "Excitation flux";
            Modelica.SIunits.Voltage vai "Induced armature voltage";
            Modelica.SIunits.Current ia "Armature current";
            output Modelica.SIunits.Torque tauElectrical;
            Modelica.Mechanics.Rotational.Interfaces.Flange_a flange annotation(
              Placement(transformation(extent = {{-10, 110}, {10, 90}}, rotation = 0)));
            Modelica.Mechanics.Rotational.Interfaces.Flange_a support "Support at which the reaction torque is acting" annotation(
              Placement(transformation(extent = {{-10, -110}, {10, -90}}, rotation = 0)));
            Modelica.Electrical.Analog.Interfaces.PositivePin pin_ap annotation(
              Placement(transformation(extent = {{-110, 110}, {-90, 90}}, rotation = 0)));
            Modelica.Electrical.Analog.Interfaces.PositivePin pin_ep annotation(
              Placement(transformation(extent = {{90, 110}, {110, 90}}, rotation = 0)));
            Modelica.Electrical.Analog.Interfaces.NegativePin pin_an annotation(
              Placement(transformation(extent = {{-110, -110}, {-90, -90}}, rotation = 0)));
            Modelica.Electrical.Analog.Interfaces.NegativePin pin_en annotation(
              Placement(transformation(extent = {{90, -110}, {110, -90}}, rotation = 0)));
          equation
// armature pins
            vai = pin_ap.v - pin_an.v;
            ia = +pin_ap.i;
            ia = -pin_an.i;
// excitation pins
            vei = pin_ep.v - pin_en.v;
            ie = +pin_ep.i;
            ie = -pin_en.i;
// induced voltage across field excitation inductance
            vei = if quasiStationary then 0 else der(psi_e);
// mechanical speed
            w = der(flange.phi) - der(support.phi);
// induced armature voltage
            vai = turnsRatio * psi_e * w;
// electrical torque (ia is perpendicular to flux)
            tauElectrical = turnsRatio * psi_e * ia;
            flange.tau = -tauElectrical;
            support.tau = tauElectrical;
            annotation(
              Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-90, 90}, {90, -92}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-80, 80}, {80, -80}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-10, 90}, {10, -80}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.VerticalCylinder, fillColor = {128, 128, 128}), Text(extent = {{0, 40}, {80, -40}}, lineColor = {0, 0, 0}, textString = "E"), Text(extent = {{-150, -100}, {150, -160}}, lineColor = {0, 0, 255}, textString = "%name"), Text(extent = {{-80, 40}, {0, -40}}, lineColor = {0, 0, 0}, textString = "A"), Rectangle(visible = quasiStationary, extent = {{-10, 90}, {10, -80}}, lineColor = {170, 213, 255}, fillPattern = FillPattern.VerticalCylinder, fillColor = {170, 213, 255})}),
              Documentation(info = "<HTML>
Linear model of the airgap (without saturation effects) of a DC machine, using only equations.<br>
Induced excitation voltage is calculated from der(flux), where flux is defined by excitation inductance times excitation current.
If <code>quasiStationary == false</code>, the electrical transients are neglected, i.e., the induced excitation voltage is zero.<br>
Induced armature voltage is calculated from flux times angular velocity.
</HTML>"));
          end PartialAirGapDC;

          model AirGapDC "Linear airgap model of a DC machine"
            extends PartialAirGapDC;
            parameter Modelica.SIunits.Inductance Le "Excitation inductance";
          equation
// excitation flux: linearly dependent on excitation current
            psi_e = Le * ie;
            annotation(
              defaultComponentName = "airGap",
              Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-150, -100}, {150, -160}}, lineColor = {0, 0, 255}, textString = "%name")}),
              Documentation(info = "<HTML>
Linear model of the airgap (without saturation effects) of a DC machine, using only equations.<br>
Induced excitation voltage is calculated from der(flux), where flux is defined by excitation inductance times excitation current.<br>
Induced armature voltage is calculated from flux times angular velocity.
</HTML>"));
          end AirGapDC;

          model CompoundDCExcitation "Compound excitation = shunt + series"
            parameter Real excitationTurnsRatio "Ratio of series excitation turns over shunt excitation turns";
            Modelica.SIunits.Voltage v = pin_p.v - pin_n.v;
            Modelica.SIunits.Current i = pin_p.i;
            Modelica.SIunits.Voltage ve = pin_ep.v - pin_en.v;
            Modelica.SIunits.Current ie = pin_ep.i;
            Modelica.SIunits.Voltage vse = pin_sep.v - pin_sen.v;
            Modelica.SIunits.Current ise = pin_sep.i;
            Modelica.Electrical.Analog.Interfaces.PositivePin pin_p "Positive pin to airgap" annotation(
              Placement(transformation(extent = {{90, 110}, {110, 90}})));
            Modelica.Electrical.Analog.Interfaces.NegativePin pin_n "Negative pin to airgap" annotation(
              Placement(transformation(extent = {{-110, 110}, {-90, 90}})));
            Modelica.Electrical.Analog.Interfaces.PositivePin pin_ep "Positive pin to shunt excitation" annotation(
              Placement(transformation(extent = {{90, -108}, {110, -88}})));
            Modelica.Electrical.Analog.Interfaces.NegativePin pin_en "Negative pin to shunt excitation" annotation(
              Placement(transformation(extent = {{10, -110}, {30, -90}})));
            Modelica.Electrical.Analog.Interfaces.PositivePin pin_sep "Positive pin to series excitation" annotation(
              Placement(transformation(extent = {{-30, -110}, {-10, -90}})));
            Modelica.Electrical.Analog.Interfaces.NegativePin pin_sen "Negative pin to series excitation" annotation(
              Placement(transformation(extent = {{-110, -110}, {-90, -90}})));
          equation
//current balances
            pin_p.i + pin_n.i = 0;
            pin_ep.i + pin_en.i = 0;
            pin_sep.i + pin_sen.i = 0;
//compound currents
            -i = ie + excitationTurnsRatio * ise;
//induced voltages
            ve = v;
            vse = v * excitationTurnsRatio;
            annotation(
              Icon(graphics = {Polygon(points = {{-60, -40}, {-40, -40}, {0, 4}, {40, -40}, {60, -40}, {10, 20}, {10, 60}, {20, 60}, {0, 80}, {-20, 60}, {-10, 60}, {-10, 20}, {-60, -40}}, lineColor = {0, 0, 255}, smooth = Smooth.None, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-80, -40}, {-40, -80}}, lineColor = {0, 0, 255}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid, textString = "S"), Text(extent = {{40, -40}, {80, -80}}, lineColor = {0, 0, 255}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid, textString = "E")}),
              Documentation(info = "<HTML>
Model to compound the shunt excitation current and the series excitation current to the total excitation current w.r.t. shunt excitation.
This model is intended to be placed between shunt and series excitation pins and the airgap;
the connection to airgap has to be grounded at one point.
</HTML>"));
          end CompoundDCExcitation;

          partial model PartialCore "Partial model of transformer core with 3 windings"
            parameter Integer m(final min = 1) = 3 "Number of phases";
            parameter Real n12(start = 1) "Turns ratio 1:2";
            parameter Real n13(start = 1) "Turns ratio 1:3";
            Modelica.SIunits.Voltage v1[m] = plug_p1.pin.v - plug_n1.pin.v;
            Modelica.SIunits.Current i1[m] = plug_p1.pin.i;
            Modelica.SIunits.Voltage v2[m] = plug_p2.pin.v - plug_n2.pin.v;
            Modelica.SIunits.Current i2[m] = plug_p2.pin.i;
            Modelica.SIunits.Voltage v3[m] = plug_p3.pin.v - plug_n3.pin.v;
            Modelica.SIunits.Current i3[m] = plug_p3.pin.i;
            Modelica.SIunits.Current im[m] = i1 + i2 / n12 + i3 / n13 "Magnetizing current";
            Modelica.Electrical.MultiPhase.Interfaces.PositivePlug plug_p1(final m = m) annotation(
              Placement(transformation(extent = {{-110, 40}, {-90, 60}}, rotation = 0)));
            Modelica.Electrical.MultiPhase.Interfaces.NegativePlug plug_n1(final m = m) annotation(
              Placement(transformation(extent = {{-110, -60}, {-90, -40}}, rotation = 0)));
            Modelica.Electrical.MultiPhase.Interfaces.PositivePlug plug_p2(final m = m) annotation(
              Placement(transformation(extent = {{90, 90}, {110, 110}}, rotation = 0)));
            Modelica.Electrical.MultiPhase.Interfaces.NegativePlug plug_n2(final m = m) annotation(
              Placement(transformation(extent = {{90, 30}, {110, 50}}, rotation = 0)));
            Modelica.Electrical.MultiPhase.Interfaces.PositivePlug plug_p3(final m = m) annotation(
              Placement(transformation(extent = {{90, -30}, {110, -50}}, rotation = 0)));
            Modelica.Electrical.MultiPhase.Interfaces.NegativePlug plug_n3(final m = m) annotation(
              Placement(transformation(extent = {{90, -90}, {110, -110}}, rotation = 0)));
          equation
            plug_p1.pin.i + plug_n1.pin.i = zeros(m);
            plug_p2.pin.i + plug_n2.pin.i = zeros(m);
            plug_p3.pin.i + plug_n3.pin.i = zeros(m);
            annotation(
              Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-100, 130}, {100, 110}}, textString = "%name", lineColor = {0, 0, 255}), Ellipse(extent = {{-45, -50}, {-20, -25}}, lineColor = {0, 0, 255}), Ellipse(extent = {{-45, -25}, {-20, 0}}, lineColor = {0, 0, 255}), Ellipse(extent = {{-45, 0}, {-20, 25}}, lineColor = {0, 0, 255}), Ellipse(extent = {{-45, 25}, {-20, 50}}, lineColor = {0, 0, 255}), Rectangle(extent = {{-46, -50}, {-34, 50}}, lineColor = {255, 255, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{-90, 50}, {-32, 50}}, color = {0, 0, 255}), Line(points = {{-90, -50}, {-32, -50}}, color = {0, 0, 255}), Ellipse(extent = {{20, 40}, {45, 65}}, lineColor = {0, 0, 255}), Ellipse(extent = {{20, 65}, {45, 90}}, lineColor = {0, 0, 255}), Rectangle(extent = {{36, 40}, {46, 90}}, lineColor = {255, 255, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{32, 90}, {90, 90}}, color = {0, 0, 255}), Line(points = {{34, 40}, {92, 40}}, color = {0, 0, 255}), Ellipse(extent = {{20, -90}, {45, -65}}, lineColor = {0, 0, 255}), Ellipse(extent = {{20, -65}, {45, -40}}, lineColor = {0, 0, 255}), Rectangle(extent = {{36, -90}, {46, -40}}, lineColor = {255, 255, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{32, -40}, {90, -40}}, color = {0, 0, 255}), Line(points = {{34, -90}, {92, -90}}, color = {0, 0, 255})}),
              Documentation(info = "<html>
Partial model of transformer core with 3 windings; saturation function flux versus magnetizing current has to be defined.
</html>"));
          end PartialCore;

          model IdealCore "Ideal transformer with 3 windings"
            extends PartialCore;
          equation
            im = zeros(m);
            v1 = n12 * v2;
            v1 = n13 * v3;
            annotation(
              defaultComponentName = "core",
              Documentation(info = "<html>
Ideal transformer with 3 windings: no magnetizing current.
</html>"));
          end IdealCore;

          partial model BasicTransformer "Partial model of three-phase transformer"
            extends Machines.Interfaces.PartialBasicTransformer;
            //dummy will be removed when conversion script is applicable
            annotation(
              Documentation(info = "<html>
Partial model of a three-phase transformer, containing primary and secondary resistances and stray inductances, as well as the iron core.
Circuit layout (vector group) of primary and secondary windings have to be defined.<br>
Exactly the same as Interfaces.PartialBasicTransformer, included for compatibility reasons.
</html>"));
          end BasicTransformer;
          annotation(
            Documentation(info = "<HTML>
This package contains components for modeling electrical machines, specially three-phase induction machines, based on space phasor theory.
These models use package SpacePhasors.
</HTML>", revisions = "<HTML>
<dl>
  <dt><b>Main Authors:</b></dt>
  <dd>
  <a href=\"http://www.haumer.at/\">Anton Haumer</a><br>
  Technical Consulting &amp; Electrical Engineering<br>
  A-3423 St.Andrae-Woerdern<br>Austria<br>
  email: <a href=\"mailto:a.haumer@haumer.at\">a.haumer@haumer.at</a>
  </dd>
  <dt><b>Copyright:</b></dt>
  <dd>Copyright &copy; 1998-2015, Modelica Association and Anton Haumer.<br>
  <i>The Modelica package is <b>free</b> software; it can be redistributed and/or modified
  under the terms of the <b>Modelica license</b>, see the license conditions
  and the accompanying <b>disclaimer</b> in the documentation of package
  Modelica in file \"Modelica/package.mo\".</i></dd>
</dl>
  <ul>
  <li> v1.00 2004/09/16 Anton Haumer</li>
  <li> v1.02 2004/09/19 Anton Haumer<br>
       added AirGapDC models</li>
  <li> v1.2  2004/10/27 Anton Haumer<br>
       fixed a bug with support (formerly bearing)</li>
  <li> v1.52 2005/10/12 Anton Haumer<br>
       added electrical excitation</li>
  <li> v1.53 Beta 2005/10/14 Anton Haumer<br>
       introduced unsymmetrical DamperCage for Synchronous Machines</li>
  <li> v2.1.3 2010/02/10 Anton Haumer<br>
       prepared ConditionalHeatPort of SquirrelCage and DamperCage</li>
  <li> v2.2.0 2011/02/10 Anton Haumer<br>
       conditional ThermalPort for all machines</li>
  </ul>
</HTML>"));
        end Components;
        annotation(
          Documentation(info = "<HTML>
This package contains components for modeling electrical machines, specially three-phase induction machines, based on space phasor theory:
<ul>
<li>package AsynchronousInductionMachines: models of three phase asynchronous induction machines</li>
<li>package SynchronousInductionMachines: models of three phase synchronous induction machines</li>
<li>package DCMachines: models of DC machines with different excitation</li>
<li>package Transformers: Threephase transformers (see detailed documentation in subpackage)</li>
<li>package Components: components for modeling machines and transformers</li>
</ul>
The induction machine models use package SpacePhasors.
</HTML>", revisions = "<HTML>
<dl>
  <dt><b>Main Authors:</b></dt>
  <dd>
  <a href=\"http://www.haumer.at/\">Anton Haumer</a><br>
  Technical Consulting &amp; Electrical Engineering<br>
  A-3423 St.Andrae-Woerdern<br>Austria<br>
  email: <a href=\"mailto:a.haumer@haumer.at\">a.haumer@haumer.at</a>
  </dd>
  <dt><b>Copyright:</b></dt>
  <dd>Copyright &copy; 1998-2015, Modelica Association and Anton Haumer.<br>
  <i>The Modelica package is <b>free</b> software; it can be redistributed and/or modified
  under the terms of the <b>Modelica license</b>, see the license conditions
  and the accompanying <b>disclaimer</b> in the documentation of package
  Modelica in file \"Modelica/package.mo\".</i></dd>
</dl>
  <ul>
  <li> v1.00  2004/09/16 Anton Haumer</li>
  <li> v1.01  2004/09/18 Anton Haumer<br>
       moved common equations from machine models to PartialMachine</li>
  <li> v1.02  2004/09/19 Anton Haumer<br>
       new package structure for machine types<br>
       added DCMachine models</li>
  <li> v1.03  2004/09/24 Anton Haumer<br>
       added DC machine with series excitation</li>
  <li> v1.1   2004/10/01 Anton Haumer<br>
       changed naming and structure<br>
       issued to Modelica Standard Library 2.1</li>
  <li> v1.2   2004/10/27 Anton Haumer<br>
       fixed a bug with support (formerly bearing)</li>
  <li> v1.3.2 2004/11/10 Anton Haumer<br>
       ReluctanceRotor moved to SynchronousMachines</li>
  <li> v1.4   2004/11/11 Anton Haumer<br>
       removed mechanical flange support<br>
       to ease the implementation of a 3D-frame in a future release</li>
  <li> v1.53  2005/10/14 Anton Haumer<br>
       introduced unsymmetrical DamperCage for Synchronous Machines</li>
  <li> v1.6.2 2005/10/23 Anton Haumer<br>
       selectable DamperCage for Synchronous Machines</li>
  <li> v1.6.3 2005/11/25 Anton Haumer<br>
       easier parametrization of AsynchronousInductionMachines.AIM_SlipRing model</li>
  <li> v1.7.1 2006/02/06 Anton Haumer<br>
       changed some naming of synchronous machines, not affecting existing models</li>
  <li> v2.1.3 2010/02/10 Anton Haumer<br>
       prepared conditionalHeatPort of SquirrelCage and DamperCage</li>
  <li> v2.2.0 2011/02/10 Anton Haumer<br>
       conditional ThermalPort for all machines</li>
  </ul>
</HTML>"),
          Icon(coordinateSystem(extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(origin = {2.835, 10}, fillColor = {0, 128, 255}, fillPattern = FillPattern.HorizontalCylinder, extent = {{-60, -60}, {60, 60}}), Rectangle(origin = {2.835, 10}, fillColor = {128, 128, 128}, fillPattern = FillPattern.HorizontalCylinder, extent = {{-80, -60}, {-60, 60}}), Rectangle(origin = {2.835, 10}, fillColor = {95, 95, 95}, fillPattern = FillPattern.HorizontalCylinder, extent = {{60, -10}, {80, 10}}), Rectangle(origin = {2.835, 10}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid, extent = {{-60, 50}, {20, 70}}), Polygon(origin = {2.835, 10}, fillPattern = FillPattern.Solid, points = {{-70, -90}, {-60, -90}, {-30, -20}, {20, -20}, {50, -90}, {60, -90}, {60, -100}, {-70, -100}, {-70, -90}})}));
      end BasicMachines;

      package Sensors "Sensors for machine modelling"
        extends Modelica.Icons.SensorsPackage;

        model VoltageQuasiRMSSensor "Length of space phasor -> RMS voltage"
          extends Modelica.Icons.RotationalSensor;
          constant Integer m(final min = 1) = 3 "Number of phases";
          Modelica.Blocks.Interfaces.RealOutput V(final quantity = "ElectricPotential", final unit = "V") annotation(
            Placement(transformation(origin = {0, -110}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          Modelica.Electrical.MultiPhase.Interfaces.PositivePlug plug_p(final m = m) annotation(
            Placement(transformation(extent = {{-110, -10}, {-90, 10}}, rotation = 0)));
          Modelica.Electrical.MultiPhase.Interfaces.NegativePlug plug_n(final m = m) annotation(
            Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
          Modelica.Electrical.MultiPhase.Sensors.VoltageSensor VoltageSensor1(final m = m) annotation(
            Placement(transformation(extent = {{-10, 40}, {10, 60}}, rotation = 0)));
          Modelica.Blocks.Math.Gain Gain1(final k = 1 / sqrt(2)) annotation(
            Placement(transformation(origin = {0, -70}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          Machines.SpacePhasors.Blocks.ToSpacePhasor ToSpacePhasor1 annotation(
            Placement(transformation(origin = {0, 10}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          Machines.SpacePhasors.Blocks.ToPolar ToPolar1 annotation(
            Placement(transformation(origin = {0, -30}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
        equation
          connect(plug_p, VoltageSensor1.plug_p) annotation(
            Line(points = {{-100, 0}, {-100, 50}, {-10, 50}}, color = {0, 0, 255}));
          connect(VoltageSensor1.plug_n, plug_n) annotation(
            Line(points = {{10, 50}, {100, 50}, {100, 0}}, color = {0, 0, 255}));
          connect(VoltageSensor1.v, ToSpacePhasor1.u) annotation(
            Line(points = {{0, 39}, {0, 22}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(ToSpacePhasor1.y, ToPolar1.u) annotation(
            Line(points = {{0, -1}, {0, -18}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(ToPolar1.y[1], Gain1.u) annotation(
            Line(points = {{-0.5, -41}, {-0.5, -49.5}, {0, -49.5}, {0, -58}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(Gain1.y, V) annotation(
            Line(points = {{0, -81}, {0, -110}}, color = {0, 0, 127}, smooth = Smooth.None));
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-90, 0}, {-70, 0}}, color = {0, 0, 255}), Line(points = {{70, 0}, {90, 0}}, color = {0, 0, 255}), Line(points = {{0, -70}, {0, -100}}, color = {0, 0, 127}), Text(lineColor = {0, 0, 255}, extent = {{-40, -60}, {40, -20}}, textString = "V RMS")}),
            Documentation(info = "<HTML>
Measured 3-phase instantaneous voltages are transformed to the corresponding space phasor; <br>
output is length of the space phasor divided by sqrt(2), thus giving in sinusoidal stationary state RMS voltage.
</HTML>"));
        end VoltageQuasiRMSSensor;

        model CurrentQuasiRMSSensor "Length of space phasor -> RMS current"
          extends Modelica.Icons.RotationalSensor;
          constant Integer m(final min = 1) = 3 "Number of phases";
          Modelica.Blocks.Interfaces.RealOutput I(final quantity = "ElectricCurrent", final unit = "A") annotation(
            Placement(transformation(origin = {0, -110}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          Modelica.Electrical.MultiPhase.Interfaces.PositivePlug plug_p(final m = m) annotation(
            Placement(transformation(extent = {{-110, -10}, {-90, 10}}, rotation = 0)));
          Modelica.Electrical.MultiPhase.Interfaces.NegativePlug plug_n(final m = m) annotation(
            Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
          Modelica.Electrical.MultiPhase.Sensors.CurrentSensor CurrentSensor1(final m = m) annotation(
            Placement(transformation(extent = {{-10, 40}, {10, 60}}, rotation = 0)));
          Modelica.Blocks.Math.Gain Gain1(final k = 1 / sqrt(2)) annotation(
            Placement(transformation(origin = {0, -70}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          Machines.SpacePhasors.Blocks.ToSpacePhasor ToSpacePhasor1 annotation(
            Placement(transformation(origin = {0, 10}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          Machines.SpacePhasors.Blocks.ToPolar ToPolar1 annotation(
            Placement(transformation(origin = {0, -30}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
        equation
          connect(plug_p, CurrentSensor1.plug_p) annotation(
            Line(points = {{-100, 0}, {-100, 50}, {-10, 50}}, color = {0, 0, 255}));
          connect(CurrentSensor1.plug_n, plug_n) annotation(
            Line(points = {{10, 50}, {100, 50}, {100, 0}}, color = {0, 0, 255}));
          connect(CurrentSensor1.i, ToSpacePhasor1.u) annotation(
            Line(points = {{0, 39}, {0, 22}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(ToSpacePhasor1.y, ToPolar1.u) annotation(
            Line(points = {{0, -1}, {0, -18}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(ToPolar1.y[1], Gain1.u) annotation(
            Line(points = {{-0.5, -41}, {-0.5, -50.5}, {0, -50.5}, {0, -58}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(Gain1.y, I) annotation(
            Line(points = {{0, -81}, {0, -110}}, color = {0, 0, 127}, smooth = Smooth.None));
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-90, 0}, {-70, 0}}, color = {0, 0, 255}), Line(points = {{70, 0}, {90, 0}}, color = {0, 0, 255}), Line(points = {{0, -70}, {0, -100}}, color = {0, 0, 127}), Text(lineColor = {0, 0, 255}, extent = {{-40, -60}, {40, -20}}, textString = "A RMS")}),
            Documentation(info = "<HTML>
Measured 3-phase instantaneous currents are transformed to the corresponding space phasor; <br>
output is length of the space phasor divided by sqrt(2), thus giving in sinusoidal stationary state RMS current.
</HTML>"));
        end CurrentQuasiRMSSensor;

        model ElectricalPowerSensor "Instantaneous power from space phasors"
          extends Modelica.Icons.RotationalSensor;
          constant Integer m(final min = 1) = 3 "Number of phases";
          Modelica.Blocks.Interfaces.RealOutput P(final quantity = "Power", final unit = "W") annotation(
            Placement(transformation(origin = {-50, 110}, extent = {{10, -10}, {-10, 10}}, rotation = 270)));
          Modelica.Blocks.Interfaces.RealOutput Q(final quantity = "Power", final unit = "var") annotation(
            Placement(transformation(origin = {50, 110}, extent = {{10, -10}, {-10, 10}}, rotation = 270)));
          Modelica.Electrical.MultiPhase.Interfaces.PositivePlug plug_p(final m = m) annotation(
            Placement(transformation(extent = {{-110, -10}, {-90, 10}}, rotation = 0)));
          Modelica.Electrical.MultiPhase.Interfaces.NegativePlug plug_ni(final m = m) annotation(
            Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
          Modelica.Electrical.MultiPhase.Interfaces.NegativePlug plug_nv(final m = m) annotation(
            Placement(transformation(extent = {{-10, -110}, {10, -90}}, rotation = 0)));
        protected
          Modelica.SIunits.Voltage v_[2];
          Modelica.SIunits.Current i_[2];
        equation
          plug_p.pin.v = plug_ni.pin.v;
          plug_p.pin.i + plug_ni.pin.i = zeros(m);
          plug_nv.pin.i = zeros(m);
          v_ = Machines.SpacePhasors.Functions.ToSpacePhasor(plug_p.pin.v - plug_nv.pin.v);
          i_ = Machines.SpacePhasors.Functions.ToSpacePhasor(plug_p.pin.i);
          2 / 3 * P = (+v_[1] * i_[1]) + v_[2] * i_[2];
          2 / 3 * Q = (-v_[1] * i_[2]) + v_[2] * i_[1];
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-90, 0}, {-70, 0}}, color = {0, 0, 255}), Line(points = {{70, 0}, {90, 0}}, color = {0, 0, 255}), Line(points = {{0, -70}, {0, -90}}, color = {0, 0, 255}), Line(points = {{-10, 70}, {-10, 80}, {-50, 80}, {-50, 100}}, color = {0, 0, 127}), Line(points = {{10, 70}, {10, 80}, {50, 80}, {50, 100}}, color = {0, 0, 127}), Text(lineColor = {0, 0, 255}, extent = {{-40, -60}, {40, -20}}, textString = "P Q")}),
            Documentation(info = "<HTML>
3-phase instantaneous voltages (plug_p - plug_nv) and currents (plug_p - plug_ni) are transformed to the corresponding space phasors, <br>
which are used to calculate power quantities:<br>
P = instantaneous power, thus giving in stationary state active power.<br>
Q = giving in stationary state reactive power.<br>
</HTML>"));
        end ElectricalPowerSensor;

        model MechanicalPowerSensor "Mechanical power = torque x speed"
          extends Modelica.Icons.RotationalSensor;
          extends Modelica.Mechanics.Rotational.Interfaces.PartialTwoFlanges;
          parameter Boolean useSupport = false "Use support or fixed housing" annotation(
            Evaluate = true);
          Modelica.Blocks.Interfaces.RealOutput P(final quantity = "Power", final unit = "W") annotation(
            Placement(transformation(origin = {0, 110}, extent = {{10, -10}, {-10, 10}}, rotation = 270)));
          Modelica.Mechanics.Rotational.Sensors.TorqueSensor torqueSensor annotation(
            Placement(transformation(extent = {{20, -10}, {40, 10}}, rotation = 0)));
          Modelica.Blocks.Math.Product product annotation(
            Placement(transformation(origin = {0, 30}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
          Modelica.Mechanics.Rotational.Sensors.RelSpeedSensor relSpeedSensor annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 90, origin = {-30, -30})));
          Modelica.Mechanics.Rotational.Components.Fixed fixed if not useSupport annotation(
            Placement(transformation(extent = {{-40, -100}, {-20, -80}}, rotation = 0)));
          Modelica.Mechanics.Rotational.Interfaces.Flange_a support if useSupport "Support at which the reaction torque is acting" annotation(
            Placement(transformation(extent = {{-10, -110}, {10, -90}}, rotation = 0)));
        equation
          connect(flange_a, torqueSensor.flange_a) annotation(
            Line(points = {{-100, 0}, {-70, 0}, {-40, 0}, {20, 0}}, color = {0, 0, 0}));
          connect(torqueSensor.flange_b, flange_b) annotation(
            Line(points = {{40, 0}, {100, 0}}, color = {0, 0, 0}));
          connect(flange_a, relSpeedSensor.flange_b) annotation(
            Line(points = {{-100, 0}, {-30, 0}, {-30, -20}}, color = {0, 0, 0}, smooth = Smooth.None));
          connect(relSpeedSensor.flange_a, fixed.flange) annotation(
            Line(points = {{-30, -40}, {-30, -90}}, color = {0, 0, 0}, smooth = Smooth.None));
          connect(relSpeedSensor.flange_a, support) annotation(
            Line(points = {{-30, -40}, {-30, -80}, {0, -80}, {0, -100}}, color = {0, 0, 0}, smooth = Smooth.None));
          connect(relSpeedSensor.w_rel, product.u1) annotation(
            Line(points = {{-19, -30}, {-6, -30}, {-6, 18}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(torqueSensor.tau, product.u2) annotation(
            Line(points = {{22, -11}, {22, -20}, {6, -20}, {6, 18}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(product.y, P) annotation(
            Line(points = {{0, 41}, {0, 110}}, color = {0, 0, 127}, smooth = Smooth.None));
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-70, 0}, {-90, 0}}), Line(points = {{70, 0}, {90, 0}}), Line(points = {{0, 70}, {0, 100}}, color = {0, 0, 127}), Rectangle(visible = useSupport, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid, extent = {{-20, -120}, {20, -80}}), Line(visible = not useSupport, points = {{-20, -100}, {20, -100}}), Line(visible = not useSupport, points = {{-10, -100}, {-20, -120}}), Line(visible = not useSupport, points = {{0, -100}, {-10, -120}}), Line(visible = not useSupport, points = {{10, -100}, {0, -120}}), Line(visible = not useSupport, points = {{20, -100}, {10, -120}}), Text(lineColor = {0, 0, 255}, extent = {{-40, -60}, {40, -20}}, textString = "Pmech")}),
            Documentation(info = "<HTML>
Calculates (mechanical) power from torque times angular speed.
</HTML>"));
        end MechanicalPowerSensor;

        model RotorDisplacementAngle "Rotor lagging angle"
          constant Integer m = 3 "Number of phases";
          parameter Integer p(min = 1) "Number of pole pairs";
          parameter Boolean useSupport = false "Use support or fixed housing" annotation(
            Evaluate = true);
          Modelica.Blocks.Interfaces.RealOutput rotorDisplacementAngle(final quantity = "Angle", final unit = "rad") annotation(
            Placement(transformation(extent = {{100, -10}, {120, 10}}, rotation = 0)));
          Modelica.Electrical.MultiPhase.Interfaces.PositivePlug plug_p(final m = m) annotation(
            Placement(transformation(extent = {{-110, 50}, {-90, 70}}, rotation = 0)));
          Modelica.Electrical.MultiPhase.Interfaces.NegativePlug plug_n(final m = m) annotation(
            Placement(transformation(extent = {{-110, -70}, {-90, -50}}, rotation = 0)));
          Modelica.Electrical.MultiPhase.Sensors.VoltageSensor VoltageSensor1(final m = m) annotation(
            Placement(transformation(origin = {-80, -10}, extent = {{10, -10}, {-10, 10}}, rotation = 90)));
          Machines.SpacePhasors.Blocks.ToSpacePhasor ToSpacePhasorVS annotation(
            Placement(transformation(extent = {{-60, -20}, {-40, 0}}, rotation = 0)));
          Modelica.Mechanics.Rotational.Interfaces.Flange_a flange annotation(
            Placement(transformation(extent = {{-10, 90}, {10, 110}}, rotation = 0)));
          Modelica.Mechanics.Rotational.Sensors.RelAngleSensor relativeAngleSensor annotation(
            Placement(transformation(extent = {{40, 70}, {20, 90}}, rotation = 0)));
          Modelica.Blocks.Sources.Constant constant_(final k = Modelica.Constants.pi / 2) annotation(
            Placement(transformation(extent = {{-50, 30}, {-30, 50}}, rotation = 0)));
          Modelica.Blocks.Math.Add add(final k2 = 1, final k1 = p) annotation(
            Placement(transformation(origin = {-10, 20}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          Machines.SpacePhasors.Blocks.Rotator rotatorVS2R annotation(
            Placement(transformation(extent = {{-20, 0}, {0, -20}}, rotation = 0)));
          Machines.SpacePhasors.Blocks.ToPolar ToPolarVSR annotation(
            Placement(transformation(extent = {{20, -20}, {40, 0}}, rotation = 0)));
          Modelica.Blocks.Routing.DeMultiplex2 deMultiplex2(final n1 = 1, final n2 = 1) annotation(
            Placement(transformation(extent = {{60, -20}, {80, 0}}, rotation = 0)));
          Modelica.Mechanics.Rotational.Interfaces.Flange_a support if useSupport "support at which the reaction torque is acting" annotation(
            Placement(transformation(extent = {{90, 90}, {110, 110}}, rotation = 0)));
          Modelica.Mechanics.Rotational.Components.Fixed fixed if not useSupport annotation(
            Placement(transformation(extent = {{90, 70}, {110, 90}}, rotation = 0)));
        equation
          connect(plug_p, VoltageSensor1.plug_p) annotation(
            Line(points = {{-100, 60}, {-80, 60}, {-80, 0}}, color = {0, 0, 255}));
          connect(plug_n, VoltageSensor1.plug_n) annotation(
            Line(points = {{-100, -60}, {-80, -60}, {-80, -20}}, color = {0, 0, 255}));
          connect(relativeAngleSensor.flange_b, flange) annotation(
            Line(points = {{20, 80}, {0, 80}, {0, 100}}, color = {0, 0, 0}));
          connect(relativeAngleSensor.flange_a, support) annotation(
            Line(points = {{40, 80}, {60, 80}, {60, 100}, {100, 100}}, color = {0, 0, 0}, smooth = Smooth.None));
          connect(relativeAngleSensor.flange_a, fixed.flange) annotation(
            Line(points = {{40, 80}, {100, 80}}, color = {0, 0, 0}, smooth = Smooth.None));
          connect(relativeAngleSensor.phi_rel, add.u1) annotation(
            Line(points = {{30, 69}, {30, 40}, {-4, 40}, {-4, 32}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(constant_.y, add.u2) annotation(
            Line(points = {{-29, 40}, {-16, 40}, {-16, 32}, {-16, 32}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(VoltageSensor1.v, ToSpacePhasorVS.u) annotation(
            Line(points = {{-69, -10}, {-62, -10}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(ToSpacePhasorVS.y, rotatorVS2R.u) annotation(
            Line(points = {{-39, -10}, {-22, -10}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(rotatorVS2R.y, ToPolarVSR.u) annotation(
            Line(points = {{1, -10}, {18, -10}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(add.y, rotatorVS2R.angle) annotation(
            Line(points = {{-10, 9}, {-10, 2}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(ToPolarVSR.y, deMultiplex2.u) annotation(
            Line(points = {{41, -10}, {58, -10}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(deMultiplex2.y2[1], rotorDisplacementAngle) annotation(
            Line(points = {{81, -16}, {90, -16}, {90, 0}, {110, 0}}, color = {0, 0, 127}, smooth = Smooth.None));
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-60, 80}, {60, 40}}, lineColor = {0, 255, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-60, -40}, {60, -80}}, lineColor = {0, 255, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-60, 60}, {60, 40}}, lineColor = {0, 255, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-60, -40}, {60, -60}}, lineColor = {0, 255, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-40, 40}, {40, -40}}, lineColor = {0, 255, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{0, 0}, {-80, 80}}, color = {0, 0, 255}), Polygon(points = {{-80, 80}, {-68, 76}, {-76, 68}, {-80, 80}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{0, 0}, {0, 80}}, color = {0, 255, 0}), Polygon(points = {{0, 84}, {4, 72}, {-4, 72}, {0, 84}}, lineColor = {0, 255, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Rectangle(extent = {{80, 120}, {120, 80}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(visible = not useSupport, points = {{80, 100}, {120, 100}}, color = {0, 0, 0}, smooth = Smooth.None), Line(visible = not useSupport, points = {{90, 120}, {80, 100}}, color = {0, 0, 0}, smooth = Smooth.None), Line(visible = not useSupport, points = {{100, 120}, {90, 100}}, color = {0, 0, 0}, smooth = Smooth.None), Line(visible = not useSupport, points = {{110, 120}, {100, 100}}, color = {0, 0, 0}, smooth = Smooth.None), Line(visible = not useSupport, points = {{120, 120}, {110, 100}}, color = {0, 0, 0}, smooth = Smooth.None)}),
            Documentation(info = "<HTML>
Calculates rotor lagging angle by measuring the stator phase voltages, transforming them to the corresponding space phasor in stator-fixed coordinate system, <br>
rotating the space phasor to the rotor-fixed coordinate system and calculating the angle of this space phasor.
<p>
The sensor's housing can be implicitly fixed (useSupport=false).<br>
If the machine's stator also implicitly fixed (useSupport=false), the angle at the flange
is equal to the angle of the machine's rotor against the stator.<br>
Otherwise, the sensor's support has to be connected to the machine's support.
</p>
</HTML>"));
        end RotorDisplacementAngle;
        annotation(
          Documentation(info = "<HTML>
This package contains sensors that are useful when modelling machines.
</HTML>", revisions = "<HTML>
<dl>
  <dt><b>Main Authors:</b></dt>
  <dd>
  <a href=\"http://www.haumer.at/\">Anton Haumer</a><br>
  Technical Consulting &amp; Electrical Engineering<br>
  A-3423 St.Andrae-Woerdern<br>Austria<br>
  email: <a href=\"mailto:a.haumer@haumer.at\">a.haumer@haumer.at</a>
  </dd>
  <dt><b>Copyright:</b></dt>
  <dd>Copyright &copy; 1998-2015, Modelica Association and Anton Haumer.<br>
  <i>The Modelica package is <b>free</b> software; it can be redistributed and/or modified
  under the terms of the <b>Modelica license</b>, see the license conditions
  and the accompanying <b>disclaimer</b> in the documentation of package
  Modelica in file \"Modelica/package.mo\".</i></dd>
</dl>
  <ul>
  <li> v1.03 2004/09/24 Anton Haumer</li>
  <li> v1.1  2004/10/01 Anton Haumer<br>
       changed RotorDisplacementAngle</li>
  <li> v1.4   2004/11/11 Anton Haumer<br>
       removed mechanical flange support, also in sensor RotorDisplacementAngle<br>
       to ease the implementation of a 3D-frame in a future release</li>
  </ul>
</HTML>"));
      end Sensors;

      package SpacePhasors "Library with space phasor-models"
        extends Modelica.Icons.Package;

        package Components "Basic space phasor models"
          extends Modelica.Icons.Package;

          model SpacePhasor "Physical transformation: three phase <-> space phasors"
            constant Integer m = 3 "Number of phases";
            constant Real pi = Modelica.Constants.pi;
            parameter Real turnsRatio = 1 "Turns ratio";
            Modelica.SIunits.Voltage v[m] "Instantaneous phase voltages";
            Modelica.SIunits.Current i[m] "Instantaneous phase currents";
          protected
            parameter Real TransformationMatrix[2, m] = 2 / m * {{cos(+(k - 1) / m * 2 * pi) for k in 1:m}, {+sin(+(k - 1) / m * 2 * pi) for k in 1:m}};
            parameter Real InverseTransformation[m, 2] = {{cos(-(k - 1) / m * 2 * pi), -sin(-(k - 1) / m * 2 * pi)} for k in 1:m};
          public
            Modelica.Electrical.MultiPhase.Interfaces.PositivePlug plug_p(final m = m) annotation(
              Placement(transformation(extent = {{-110, 90}, {-90, 110}}, rotation = 0)));
            Modelica.Electrical.MultiPhase.Interfaces.NegativePlug plug_n(final m = m) annotation(
              Placement(transformation(extent = {{-110, -110}, {-90, -90}}, rotation = 0)));
            Modelica.Electrical.Analog.Interfaces.PositivePin zero annotation(
              Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
            Modelica.Electrical.Analog.Interfaces.NegativePin ground annotation(
              Placement(transformation(extent = {{90, -110}, {110, -90}}, rotation = 0)));
            Machines.Interfaces.SpacePhasor spacePhasor annotation(
              Placement(transformation(extent = {{90, 90}, {110, 110}}, rotation = 0)));
          equation
            v / turnsRatio = plug_p.pin.v - plug_n.pin.v;
            i * turnsRatio = +plug_p.pin.i;
            i * turnsRatio = -plug_n.pin.i;
            m * zero.v = sum(v);
            spacePhasor.v_ = TransformationMatrix * v;
//v  = fill(zero.v,m) + InverseTransformation*spacePhasor.v_;
            -m * zero.i = sum(i);
            -spacePhasor.i_ = TransformationMatrix * i;
//-i  = fill(zero.i,m) + InverseTransformation*spacePhasor.i_;
            ground.v = 0;
            annotation(
              Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{0, 0}, {80, 80}, {60, 72}, {72, 60}, {80, 80}}, color = {0, 0, 255}), Line(points = {{0, 0}, {80, -80}, {72, -60}, {60, -72}, {80, -80}}, color = {0, 0, 255}), Line(points = {{-80, 0}, {-73.33, 10}, {-66.67, 17.32}, {-60, 20}, {-53.33, 17.32}, {-46.67, 10}, {-40, 0}, {-33.33, -10}, {-26.67, -17.32}, {-20, -20}, {-13.33, -17.32}, {-6.67, -10}, {0, 0}}, color = {0, 0, 255}, smooth = Smooth.Bezier), Line(points = {{-90, 0}, {-83.33, 10}, {-76.67, 17.32}, {-70, 20}, {-63.33, 17.32}, {-56.67, 10}, {-50, 0}, {-43.33, -10}, {-36.67, -17.32}, {-30, -20}, {-23.33, -17.32}, {-16.67, -10}, {-10, 0}}, color = {0, 0, 255}, smooth = Smooth.Bezier), Line(points = {{-70, 0}, {-63.33, 10}, {-56.67, 17.32}, {-50, 20}, {-43.33, 17.32}, {-36.67, 10}, {-30, 0}, {-23.33, -10}, {-16.67, -17.32}, {-10, -20}, {-3.33, -17.32}, {3.33, -10}, {10, 0}}, color = {0, 0, 255}, smooth = Smooth.Bezier), Text(extent = {{-150, -100}, {150, -160}}, lineColor = {0, 0, 255}, textString = "%name"), Text(extent = {{40, 10}, {90, -10}}, lineColor = {0, 0, 255}, textString = "zero"), Line(points = {{90, -100}, {60, -100}}, color = {0, 0, 255}), Line(points = {{60, -84}, {60, -116}}, color = {0, 0, 255}), Line(points = {{50, -90}, {50, -110}}, color = {0, 0, 255}), Line(points = {{40, -96}, {40, -104}}, color = {0, 0, 255})}),
              Documentation(info = "<HTML>
Physical transformation of voltages and currents: three phases &lt;-&gt; space phasors:<br>
x[k] = X0 + {cos(-(k - 1)/m*2*pi),-sin(-(k - 1)/m*2*pi)}*X[Re,Im]<br>
and vice versa:<br>
X0 = sum(x[k])/m<br>
X[Re,Im] = sum(2/m*{cos((k - 1)/m*2*pi),sin((k - 1)/m*2*pi)}*x[k])<br>
were x designates three phase values, X[Re,Im] designates the space phasor and X0 designates the zero sequence system.<br>
<i>Physical transformation</i> means that both voltages and currents are transformed in both directions.<br>
Zero-sequence voltage and current are present at pin zero. An additional zero-sequence impedance could be connected between pin zero and pin ground.
</HTML>"));
          end SpacePhasor;

          model Rotator "Rotates space phasor"
            constant Real pi = Modelica.Constants.pi;
          protected
            Real RotationMatrix[2, 2] = {{+cos(-angle), -sin(-angle)}, {+sin(-angle), +cos(-angle)}};
            //Real InverseRotator[2,2] = {{+cos(+angle),-sin(+angle)},{+sin(+angle),+cos(+angle)}};
          public
            Machines.Interfaces.SpacePhasor spacePhasor_a annotation(
              Placement(transformation(extent = {{-110, 90}, {-90, 110}}, rotation = 0)));
            Machines.Interfaces.SpacePhasor spacePhasor_b annotation(
              Placement(transformation(extent = {{90, 90}, {110, 110}}, rotation = 0)));
            Modelica.Blocks.Interfaces.RealInput angle annotation(
              Placement(transformation(origin = {0, -120}, extent = {{-20, -20}, {20, 20}}, rotation = 90)));
          equation
            spacePhasor_b.v_ = RotationMatrix * spacePhasor_a.v_;
//spacePhasor_a.v_ = InverseRotator*spacePhasor_b.v_;
            spacePhasor_b.i_ + RotationMatrix * spacePhasor_a.i_ = zeros(2);
//spacePhasor_a.i_ + InverseRotator*spacePhasor_b.i_ = zeros(2);
            annotation(
              Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{50, -2}, {-2, -50}}, pattern = LinePattern.None, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Line(points = {{0, 0}, {80, 0}, {60, 10}, {60, -10}, {80, 0}}, color = {0, 0, 255}), Line(points = {{0, 0}, {0, 80}, {-10, 60}, {10, 60}, {0, 80}}, color = {0, 0, 255}), Polygon(points = {{50, 0}, {42, 14}, {54, 16}, {50, 0}}, lineColor = {0, 0, 255}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-50, 50}, {50, -50}}, lineColor = {0, 0, 255}), Rectangle(extent = {{-51, 51}, {-1, -51}}, pattern = LinePattern.None, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Rectangle(extent = {{51, -1}, {-1, -51}}, pattern = LinePattern.None, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255})}),
              Documentation(info = "<HTML>
Rotates space phasors of left connector to right connector by the angle provided by the input signal \"angle\" from one coordinate system into another.
</HTML>"));
          end Rotator;
          annotation(
            Documentation(info = "<HTML>
This package contains basic space phasor models.<br>
Real and imaginary part of voltage space phasor are the potentials v_[2] of the space phasor connector; (implicit grounded).<br>
Real and imaginary part of current space phasor are the currents i_[2] at the space phasor connector;
a ground has to be used where necessary for currents flowing back.
</HTML>", revisions = "<HTML>
<dl>
  <dt><b>Main Authors:</b></dt>
  <dd>
  <a href=\"http://www.haumer.at/\">Anton Haumer</a><br>
  Technical Consulting &amp; Electrical Engineering<br>
  A-3423 St.Andrae-Woerdern<br>Austria<br>
  email: <a href=\"mailto:a.haumer@haumer.at\">a.haumer@haumer.at</a>
  </dd>
  <dt><b>Copyright:</b></dt>
  <dd>Copyright &copy; 1998-2015, Modelica Association and Anton Haumer.<br>
  <i>The Modelica package is <b>free</b> software; it can be redistributed and/or modified
  under the terms of the <b>Modelica license</b>, see the license conditions
  and the accompanying <b>disclaimer</b> in the documentation of package
  Modelica in file \"Modelica/package.mo\".</i></dd>
</dl>
  <ul>
  <li> v1.00 2004/09/16 Anton Haumer</li>
  <li> v1.60 2005/11/03 Anton Haumer<br>
       added Rotator</li>
  <li> v1.6.1 2005/11/10 Anton Haumer<br>
       improved Transformation and Rotation</li>
  </ul>
</HTML>"));
        end Components;

        package Blocks "Blocks for space phasor transformation"
          extends Modelica.Icons.Package;

          block ToSpacePhasor "Conversion: three phase -> space phasor"
            extends Modelica.Blocks.Interfaces.MIMO(final nin = m, final nout = 2);
            constant Integer m = 3 "Number of phases";
            constant Real pi = Modelica.Constants.pi;
          protected
            parameter Real TransformationMatrix[2, m] = 2 / m * {{cos(+(k - 1) / m * 2 * pi) for k in 1:m}, {+sin(+(k - 1) / m * 2 * pi) for k in 1:m}};
            parameter Real InverseTransformation[m, 2] = {{cos(-(k - 1) / m * 2 * pi), -sin(-(k - 1) / m * 2 * pi)} for k in 1:m};
          public
            Modelica.Blocks.Interfaces.RealOutput zero annotation(
              Placement(transformation(extent = {{100, -70}, {120, -90}}, rotation = 0)));
          equation
            m * zero = sum(u);
            y = TransformationMatrix * u;
//u = fill(zero,m) + InverseTransformation*y;
            annotation(
              Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{0, 0}, {80, 80}, {60, 72}, {72, 60}, {80, 80}}, color = {0, 0, 255}), Line(points = {{0, 0}, {80, -80}, {72, -60}, {60, -72}, {80, -80}}, color = {0, 0, 255}), Line(points = {{-80, 0}, {-73.33, 10}, {-66.67, 17.32}, {-60, 20}, {-53.33, 17.32}, {-46.67, 10}, {-40, 0}, {-33.33, -10}, {-26.67, -17.32}, {-20, -20}, {-13.33, -17.32}, {-6.67, -10}, {0, 0}}, color = {0, 0, 255}, smooth = Smooth.Bezier), Line(points = {{-90, 0}, {-83.33, 10}, {-76.67, 17.32}, {-70, 20}, {-63.33, 17.32}, {-56.67, 10}, {-50, 0}, {-43.33, -10}, {-36.67, -17.32}, {-30, -20}, {-23.33, -17.32}, {-16.67, -10}, {-10, 0}}, color = {0, 0, 255}, smooth = Smooth.Bezier), Line(points = {{-70, 0}, {-63.33, 10}, {-56.67, 17.32}, {-50, 20}, {-43.33, 17.32}, {-36.67, 10}, {-30, 0}, {-23.33, -10}, {-16.67, -17.32}, {-10, -20}, {-3.33, -17.32}, {3.33, -10}, {10, 0}}, color = {0, 0, 255}, smooth = Smooth.Bezier), Text(extent = {{-12, -74}, {64, -86}}, lineColor = {0, 0, 0}, textString = "zero")}),
              Documentation(info = "<HTML>
Transformation of three-phase values (voltages or currents) to space phasor and zero sequence value.
</HTML>"));
          end ToSpacePhasor;

          block FromSpacePhasor "Conversion: space phasor -> three phase"
            extends Modelica.Blocks.Interfaces.MIMO(final nin = 2, final nout = m);
            constant Integer m = 3 "Number of phases";
            constant Real pi = Modelica.Constants.pi;
          protected
            parameter Real TransformationMatrix[2, m] = 2 / m * {{cos(+(k - 1) / m * 2 * pi) for k in 1:m}, {+sin(+(k - 1) / m * 2 * pi) for k in 1:m}};
            parameter Real InverseTransformation[m, 2] = {{cos(-(k - 1) / m * 2 * pi), -sin(-(k - 1) / m * 2 * pi)} for k in 1:m};
          public
            Modelica.Blocks.Interfaces.RealInput zero annotation(
              Placement(transformation(extent = {{-140, -60}, {-100, -100}}, rotation = 0)));
          equation
            y = fill(zero, m) + InverseTransformation * u;
//m*zero = sum(y);
//u = TransformationMatrix *y;
            annotation(
              Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{0, 0}, {-80, 80}, {-60, 72}, {-72, 60}, {-80, 80}}, color = {0, 0, 255}), Line(points = {{0, 0}, {-80, -80}, {-72, -60}, {-60, -72}, {-80, -80}}, color = {0, 0, 255}), Line(points = {{0, 0}, {6.67, 10}, {13.33, 17.32}, {20, 20}, {26.67, 17.32}, {33.33, 10}, {40, 0}, {46.67, -10}, {53.33, -17.32}, {60, -20}, {66.67, -17.32}, {73.33, -10}, {80, 0}}, color = {0, 0, 255}, smooth = Smooth.Bezier), Line(points = {{-10, 0}, {-3.33, 10}, {3.33, 17.32}, {10, 20}, {16.67, 17.32}, {23.33, 10}, {30, 0}, {36.67, -10}, {43.33, -17.32}, {50, -20}, {56.67, -17.32}, {63.33, -10}, {70, 0}}, color = {0, 0, 255}, smooth = Smooth.Bezier), Line(points = {{10, 0}, {16.67, 10}, {23.33, 17.32}, {30, 20}, {36.67, 17.32}, {43.33, 10}, {50, 0}, {56.67, -10}, {63.33, -17.32}, {70, -20}, {76.67, -17.32}, {83.33, -10}, {90, 0}}, color = {0, 0, 255}, smooth = Smooth.Bezier), Text(extent = {{-62, -74}, {14, -86}}, lineColor = {0, 0, 0}, textString = "zero")}),
              Documentation(info = "<HTML>
Transformation of space phasor and zero sequence value to three-phase values (voltages or currents).
</HTML>"));
          end FromSpacePhasor;

          block Rotator "Rotates space phasor"
            extends Modelica.Blocks.Interfaces.MIMOs(final n = 2);
          protected
            Real RotationMatrix[2, 2] = {{+cos(-angle), -sin(-angle)}, {+sin(-angle), +cos(-angle)}};
            //Real InverseRotator[2,2] = {{+cos(+angle),-sin(+angle)},{+sin(+angle),+cos(+angle)}};
          public
            Modelica.Blocks.Interfaces.RealInput angle annotation(
              Placement(transformation(origin = {0, -120}, extent = {{-20, -20}, {20, 20}}, rotation = 90)));
          equation
            y = RotationMatrix * u;
//u = InverseRotator*y;
            annotation(
              Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{0, 0}, {0, 80}, {-10, 60}, {10, 60}, {0, 80}}, color = {0, 0, 255}), Line(points = {{0, 0}, {80, 0}, {60, 10}, {60, -10}, {80, 0}}, color = {0, 0, 255}), Ellipse(extent = {{-50, 50}, {50, -50}}, lineColor = {0, 0, 255}), Rectangle(extent = {{-51, 51}, {-1, -51}}, pattern = LinePattern.None, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Rectangle(extent = {{51, -1}, {-1, -51}}, pattern = LinePattern.None, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Polygon(points = {{50, 0}, {42, 14}, {54, 16}, {50, 0}}, lineColor = {0, 0, 255}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-88, -72}, {84, -92}}, lineColor = {0, 0, 0}, textString = "angle")}),
              Documentation(info = "<HTML>
Rotates a space phasor (voltage or current) by the angle provided by the input signal \"angle\" from one coordinate system into another.
</HTML>"));
          end Rotator;

          block ToPolar "Converts a space phasor to polar coordinates"
            extends Modelica.Blocks.Interfaces.MIMOs(final n = 2);
            constant Real small = Modelica.Constants.small;
          equation
            y[1] = sqrt(u[1] ^ 2 + u[2] ^ 2);
            y[2] = if noEvent(y[1] <= small) then 0 else Modelica.Math.atan2(u[2], u[1]);
/*
  if y[1] <= small then
    y[2] = 0;
  else
    if u[2] >= 0 then
      y[2] =  Modelica.Math.acos(u[1]/y[1]);
    else
      y[2] = -Modelica.Math.acos(u[1]/y[1]);
    end if;
  end if;
*/
            annotation(
              Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-60, 60}, {-60, -60}, {60, -60}}, color = {0, 0, 255}), Line(points = {{-100, -100}, {100, 100}}, color = {0, 0, 255}), Polygon(points = {{26, 26}, {14, 20}, {20, 14}, {26, 26}}, lineColor = {0, 0, 255}), Line(points = {{-18, -18}, {-14, -22}, {-10, -28}, {-6, -34}, {-2, -44}, {0, -52}, {0, -60}}, color = {0, 0, 255}, smooth = Smooth.Bezier), Polygon(points = {{-18, -18}, {-14, -26}, {-10, -22}, {-18, -18}}, lineColor = {0, 0, 255}), Text(extent = {{-100, 100}, {-6, -6}}, lineColor = {0, 0, 255}, textString = "R"), Text(extent = {{6, 6}, {100, -100}}, lineColor = {0, 0, 255}, textString = "P")}),
              Documentation(info = "<HTML>
Converts a space phasor from rectangular coordinates to polar coordinates.
</HTML>"));
          end ToPolar;

          block FromPolar "Converts a space phasor from polar coordinates"
            extends Modelica.Blocks.Interfaces.MIMOs(final n = 2);
          equation
            y = u[1] * {cos(u[2]), sin(u[2])};
            annotation(
              Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-60, 60}, {-60, -60}, {60, -60}}, color = {0, 0, 255}), Line(points = {{-100, -100}, {100, 100}}, color = {0, 0, 255}), Polygon(points = {{26, 26}, {14, 20}, {20, 14}, {26, 26}}, lineColor = {0, 0, 255}), Line(points = {{-18, -18}, {-14, -22}, {-10, -28}, {-6, -34}, {-2, -44}, {0, -52}, {0, -60}}, color = {0, 0, 255}, smooth = Smooth.Bezier), Polygon(points = {{-18, -18}, {-14, -26}, {-10, -22}, {-18, -18}}, lineColor = {0, 0, 255}), Text(extent = {{-100, 100}, {-6, -6}}, lineColor = {0, 0, 255}, textString = "P"), Text(extent = {{6, 6}, {100, -100}}, lineColor = {0, 0, 255}, textString = "R")}),
              Documentation(info = "<HTML>
Converts a space phasor from polar coordinates to rectangular coordinates.
</HTML>"));
          end FromPolar;
          annotation(
            Documentation(info = "<HTML>
This package contains space phasor transformation blocks for use in controllers:
<ul>
<li>ToSpacePhasor: transforms a set of three-phase values to space phasor and zero sequence system</li>
<li>FromSpacePhasor: transforms a space phasor and zero sequence system to a set of three-phase values</li>
<li>Rotator: rotates a space phasor (from one coordinate system into another)</li>
<li>ToPolar: Converts a space phasor from rectangular coordinates to polar coordinates</li>
<li>FromPolar: Converts a space phasor from polar coordinates to rectangular coordinates</li>
</ul>
<p>
Space phasors are defined as vectors of length = 2,
the first element representing the real part and the second element representing the imaginary part of the space phasor.
</p>
</HTML>", revisions = "<HTML>
  <dl>
  <dt><b>Main Authors:</b></dt>
  <dd>
  <a href=\"http://www.haumer.at/\">Anton Haumer</a><br>
  Technical Consulting &amp; Electrical Engineering<br>
  A-3423 St.Andrae-Woerdern<br>Austria<br>
  email: <a href=\"mailto:a.haumer@haumer.at\">a.haumer@haumer.at</a>
  </dd>
  <dt><b>Copyright:</b></dt>
  <dd>Copyright &copy; 1998-2015, Modelica Association and Anton Haumer.<br>
  <i>The Modelica package is <b>free</b> software; it can be redistributed and/or modified
  under the terms of the <b>Modelica license</b>, see the license conditions
  and the accompanying <b>disclaimer</b> in the documentation of package
  Modelica in file \"Modelica/package.mo\".</i></dd>
</dl>
  <ul>
  <li> v1.00 2004/09/16 Anton Haumer</li>
  <li> v1.30 2004/11/05 Anton Haumer<br>
       several improvements in SpacePhasors.Blocks</li>
  <li> v1.6.1 2005/11/10 Anton Haumer<br>
       improved Transformation and Rotation</li>
  </ul>
</HTML>"));
        end Blocks;

        package Functions "Functions for space phasor transformation"
          extends Modelica.Icons.Package;

          function ToSpacePhasor "Conversion: three phase -> space phasor"
            extends Modelica.Icons.Function;
            input Real x[3];
            output Real y[2];
            output Real y0;
          protected
            constant Integer m = 3 "Number of phases";
            constant Modelica.SIunits.Angle pi = Modelica.Constants.pi;
          algorithm
            y := zeros(2);
            for k in 1:m loop
              y := y + 2 / m * {+cos((k - 1) / m * 2 * pi), +sin(+(k - 1) / m * 2 * pi)} * x[k];
            end for;
            y0 := 1 / m * sum(x);
            annotation(
              Inline = true,
              Documentation(info = "<HTML>
Transformation of three phase values (voltages or currents) to space phasor and zero sequence value:<br>
y[k] = X0 + {cos(-(k - 1)/m*2*pi),-sin(-(k - 1)/m*2*pi)}*X[Re,Im]<br>
were y designates three phase values, X[Re,Im] designates the space phasor and X0 designates the zero sequence system.
</HTML>"));
          end ToSpacePhasor;

          function FromSpacePhasor "Conversion: space phasor -> three phase"
            extends Modelica.Icons.Function;
            input Real x[2];
            input Real x0;
            output Real y[3];
          protected
            constant Integer m = 3 "Number of phases";
            constant Modelica.SIunits.Angle pi = Modelica.Constants.pi;
          algorithm
            for k in 1:m loop
              y[k] := x0 + {cos(-(k - 1) / m * 2 * pi), -sin(-(k - 1) / m * 2 * pi)} * x;
            end for;
            annotation(
              Inline = true,
              Documentation(info = "<HTML>
Transformation of space phasor and zero sequence value to three phase values (voltages or currents):<br>
Y0 = sum(x[k])/m<br>
Y[Re,Im] = sum(2/m*{cos((k - 1)/m*2*pi),sin((k - 1)/m*2*pi)}*x[k])<br>
were x designates three phase values, Y[Re,Im] designates the space phasor and Y0 designates the zero sequence system.
</HTML>"));
          end FromSpacePhasor;

          function Rotator "Rotates space phasor"
            extends Modelica.Icons.Function;
            input Real x[2];
            input Modelica.SIunits.Angle angle;
            output Real y[2];
          protected
            Real RotationMatrix[2, 2] = {{+cos(-angle), -sin(-angle)}, {+sin(-angle), +cos(-angle)}};
          algorithm
            y := RotationMatrix * x;
            annotation(
              Inline = true,
              Documentation(info = "<HTML>
Rotates a space phasor (voltage or current) by the angle provided by input argument \"angle\" from one coordinate system into another:<br>
y[Re,Im] := {{+cos(-angle),-sin(-angle)},{+sin(-angle),+cos(-angle)}}*x[Re,Im]<br>
where y[Re,Im] designates the space phasor in the new coordinate system (twisted by angle against old coordinate system) and y[Re,Im] designates the space phasor in the old coordinate system.
</HTML>"));
          end Rotator;

          function ToPolar "Converts a space phasor to polar coordinates"
            extends Modelica.Icons.Function;
            input Real x[2];
            output Real absolute;
            output Modelica.SIunits.Angle angle;
          protected
            constant Real small = Modelica.Constants.small;
          algorithm
            absolute := sqrt(x[1] ^ 2 + x[2] ^ 2);
            angle := if absolute <= small then 0 else Modelica.Math.atan2(x[2], x[1]);
/*
  if absolute <= small then
    angle := 0;
  else
    if x[2] >= 0 then
      angle :=  Modelica.Math.acos(x[1]/absolute);
    else
      angle := -Modelica.Math.acos(x[1]/absolute);
    end if;
  end if;
*/
            annotation(
              Inline = true,
              Documentation(info = "<HTML>
Converts a space phasor from rectangular coordinates to polar coordinates, providing angle=0 for {0,0}.
</HTML>"));
          end ToPolar;

          function FromPolar "Converts a space phasor from polar coordinates"
            extends Modelica.Icons.Function;
            input Real absolute;
            input Modelica.SIunits.Angle angle;
            output Real x[2];
          protected
            constant Modelica.SIunits.Angle pi = Modelica.Constants.pi;
            constant Real small = Modelica.Constants.small;
          algorithm
            x := absolute * {cos(angle), sin(angle)};
            annotation(
              Inline = true,
              Documentation(info = "<HTML>
Converts a space phasor from polar coordinates to rectangular coordinates.
</HTML>"));
          end FromPolar;

          function quasiRMS "Calculate quasi-RMS value of input"
            extends Modelica.Icons.Function;
            input Real x[3];
            output Real y;
          protected
            constant Integer m = 3 "Number of phases";
            constant Modelica.SIunits.Angle pi = Modelica.Constants.pi;
            Real h[2];
          algorithm
            h := zeros(2);
            for k in 1:m loop
              h := h + 2 / m * {+cos((k - 1) / m * 2 * pi), +sin(+(k - 1) / m * 2 * pi)} * x[k];
            end for;
            y := sqrt(h[1] ^ 2 + h[2] ^ 2) / sqrt(2);
            annotation(
              Inline = true,
              Documentation(info = "<HTML>
Transformation of three phase values (voltages or currents) to space phasor and calculate length of space phasor.
</HTML>"));
          end quasiRMS;

          function activePower "Calculate active power of voltage and current input"
            extends Modelica.Icons.Function;
            input Modelica.SIunits.Voltage v[m] "phase voltages";
            input Modelica.SIunits.Current i[m] "phase currents";
            output Modelica.SIunits.Power p "Active power";
          protected
            constant Integer m = 3 "Number of phases";
            constant Modelica.SIunits.Angle pi = Modelica.Constants.pi;
            Modelica.SIunits.Voltage v_[2] "Voltage space phasor";
            Modelica.SIunits.Current i_[2] "Current space phasor";
          algorithm
            v_ := zeros(2);
            i_ := zeros(2);
            for k in 1:m loop
              v_ := v_ + 2 / m * {+cos((k - 1) / m * 2 * pi), +sin(+(k - 1) / m * 2 * pi)} * v[k];
              i_ := i_ + 2 / m * {+cos((k - 1) / m * 2 * pi), +sin(+(k - 1) / m * 2 * pi)} * i[k];
            end for;
            p := m / 2 * ((+v_[1] * i_[1]) + v_[2] * i_[2]);
            annotation(
              Inline = true,
              Documentation(info = "<HTML>
Transformation of three phase voltages and currents to space phasors and calculate active power.
</HTML>"));
          end activePower;
          annotation(
            Documentation(info = "<HTML>
This package contains space phasor transformation functions for use in calculations:
<ul>
<li>ToSpacePhasor: transforms a set of three-phase values to space phasor and zero sequence system</li>
<li>FromSpacePhasor: transforms a space phasor and zero sequence system to a set of three-phase values</li>
<li>Rotator: rotates a space phasor (from one coordinate system into another)</li>
<li>ToPolar: Converts a space phasor from rectangular coordinates to polar coordinates</li>
<li>FromPolar: Converts a space phasor from polar coordinates to rectangular coordinates</li>
</ul>
<p>
Space phasors are defined as vectors of length = 2,
the first element representing the real part and the second element representing the imaginary part of the space phasor.
</p>
</HTML>", revisions = "<HTML>
<dl>
  <dt><b>Main Authors:</b></dt>
  <dd>
  <a href=\"http://www.haumer.at/\">Anton Haumer</a><br>
  Technical Consulting &amp; Electrical Engineering<br>
  A-3423 St.Andrae-Woerdern<br>Austria<br>
  email: <a href=\"mailto:a.haumer@haumer.at\">a.haumer@haumer.at</a>
  </dd>

  <dt><b>Release Notes:</b></dt>
  <dd>
  <ul>
  <li> v1.00 2004/09/16 Anton Haumer</li>
  <li> v1.6.1 2005/11/10 Anton Haumer<br>
       improved Transformation and Rotation</li>
  </ul>
  </dd>

  <dt><b>Copyright:</b></dt>
  <dd>Copyright &copy; 1998-2015, Modelica Association and Anton Haumer.<br>
  <i>The Modelica package is <b>free</b> software; it can be redistributed and/or modified
  under the terms of the <b>Modelica license</b>, see the license conditions
  and the accompanying <b>disclaimer</b> in the documentation of package
  Modelica in file \"Modelica/package.mo\".</i></dd>
</dl>
</HTML>"));
        end Functions;
        annotation(
          Documentation(info = "<HTML>
<p>
This package contains components, blocks and functions to utilize space phasor theory.
</p>
<p>
Space phasors are defined as vectors of length = 2,
the first element representing the real part and the second element representing the imaginary part of the space phasor.
</p>
<p>
You may have a look at a short summary of space phasor theory at <a href=\"http://www.haumer.at/refimg/SpacePhasors.pdf\">http://www.haumer.at/refimg/SpacePhasors.pdf</a>
</p>
</HTML>", revisions = "<HTML>
<dl>
  <dt><b>Main Authors:</b></dt>
  <dd>
  <a href=\"http://www.haumer.at/\">Anton Haumer</a><br>
  Technical Consulting &amp; Electrical Engineering<br>
  A-3423 St.Andrae-Woerdern<br>Austria<br>
  email: <a href=\"mailto:a.haumer@haumer.at\">a.haumer@haumer.at</a>
  </dd>
  <dt><b>Copyright:</b></dt>
  <dd>Copyright &copy; 1998-2015, Modelica Association and Anton Haumer.<br>
  <i>The Modelica package is <b>free</b> software; it can be redistributed and/or modified
  under the terms of the <b>Modelica license</b>, see the license conditions
  and the accompanying <b>disclaimer</b> in the documentation of package
  Modelica in file \"Modelica/package.mo\".</i></dd>
</dl>
  <ul>
  <li> v1.00 2004/09/16 Anton Haumer</li>
  <li> v1.30 2004/11/05 Anton Haumer<br>
       several improvements in SpacePhasors.Blocks</li>
  <li> v1.60 2005/11/03 Anton Haumer<br>
       added Components.Rotator</li>
  <li> v1.6.1 2005/11/10 Anton Haumer<br>
       improved Transformation and Rotation</li>
  </ul>
</HTML>"));
      end SpacePhasors;

      package Losses "Loss models for electric machines"
        extends Modelica.Icons.Package;

        record FrictionParameters "Parameter record for friction losses"
          extends Modelica.Icons.Record;
          parameter Modelica.SIunits.Power PRef(min = 0) = 0 "Reference friction losses at wRef";
          parameter Modelica.SIunits.AngularVelocity wRef(displayUnit = "1/min", min = Modelica.Constants.small) "Reference angular velocity that the PRef refer to";
          parameter Real power_w(min = Modelica.Constants.small) = 2 "Exponent of friction torque w.r.t. angular velocity";
          final parameter Modelica.SIunits.Torque tauRef = if PRef <= 0 then 0 else PRef / wRef "Reference friction torque at reference angular velocity";
          final parameter Real linear = 0.001 "Linear angular velocity range with respect to reference angular velocity";
          final parameter Modelica.SIunits.AngularVelocity wLinear = linear * wRef "Linear angular velocity range";
          final parameter Modelica.SIunits.Torque tauLinear = if PRef <= 0 then 0 else tauRef * (wLinear / wRef) ^ power_w "Torque corresponding with linear angular velocity range";
          annotation(
            defaultComponentPrefixes = "parameter ",
            Documentation(info = "<html>
<p>
Parameter record for <a href=\"modelica://Modelica.Electrical.Machines.Losses.Friction\">Friction</a> losses.
</p>
</html>"));
        end FrictionParameters;

        record BrushParameters "Parameter record for brush losses"
          extends Modelica.Icons.Record;
          parameter Modelica.SIunits.Voltage V = 0 "Total voltage drop of brushes for currents > ILinear";
          parameter Modelica.SIunits.Current ILinear(start = 0.01) "Current indicating linear voltage region of brush voltage drop";
          annotation(
            defaultComponentPrefixes = "parameter ",
            Documentation(info = "<html>
<p>
Parameter record for <a href=\"modelica://Modelica.Electrical.Machines.Losses.InductionMachines.Brush\">three-phase Brush</a> and
<a href=\"modelica://Modelica.Electrical.Machines.Losses.DCMachines.Brush\">DC Brush</a> losses.
</p>
</html>"));
        end BrushParameters;

        record StrayLoadParameters "Parameter record for stray load losses"
          extends Modelica.Icons.Record;
          parameter Modelica.SIunits.Power PRef(min = 0) = 0 "Reference stray load losses at IRef and wRef";
          parameter Modelica.SIunits.Current IRef(min = Modelica.Constants.small) "Reference RMS current that PRef refers to";
          parameter Modelica.SIunits.AngularVelocity wRef(displayUnit = "1/min", min = Modelica.Constants.small) "Reference angular velocity that PRef refers to";
          parameter Real power_w(min = Modelica.Constants.small) = 1 "Exponent of stray load loss torque w.r.t. angular velocity";
          final parameter Modelica.SIunits.Torque tauRef = if PRef <= 0 then 0 else PRef / wRef "Reference stray load torque at reference angular velocity and reference current";
          annotation(
            defaultComponentPrefixes = "parameter ",
            Documentation(info = "<html>
<p>
Parameter record for <a href=\"modelica://Modelica.Electrical.Machines.Losses.InductionMachines.StrayLoad\">three-phase</a> and
<a href=\"modelica://Modelica.Electrical.Machines.Losses.DCMachines.StrayLoad\">DC</a> stray load losses.
</p>
</html>"));
        end StrayLoadParameters;

        record CoreParameters "Parameter record for core losses"
          extends Modelica.Icons.Record;
          parameter Integer m "Number of phases (1 for DC, 3 for induction machines)";
          parameter Modelica.SIunits.Power PRef(min = 0) = 0 "Reference core losses at reference inner voltage VRef";
          parameter Modelica.SIunits.Voltage VRef(min = Modelica.Constants.small) "Reference inner RMS voltage that reference core losses PRef refer to";
          parameter Modelica.SIunits.AngularVelocity wRef(min = Modelica.Constants.small) "Reference angular velocity that reference core losses PRef refer to";
          // In the current implementation ratioHysterisis = 0 since hysteresis losses are not implemented yet
          final parameter Real ratioHysteresis(min = 0, max = 1, start = 0.775) = 0 "Ratio of hysteresis losses with respect to the total core losses at VRef and fRef";
          final parameter Modelica.SIunits.Conductance GcRef = if PRef <= 0 then 0 else PRef / VRef ^ 2 / m "Reference conductance at reference frequency and voltage";
          final parameter Modelica.SIunits.AngularVelocity wMin = 1e-6 * wRef;
          annotation(
            defaultComponentPrefixes = "parameter ",
            Documentation(info = "<html>
<p>
Parameter record for <a href=\"modelica://Modelica.Electrical.Machines.Losses.InductionMachines.Core\">core losses of induction machines</a>
and <a href=\"modelica://Modelica.Electrical.Machines.Losses.DCMachines.Core\">core losses of DC machines</a>.
</p>
</html>"));
        end CoreParameters;

        record PermanentMagnetLossParameters "Parameter record for permanent magnet losses"
          extends Modelica.Icons.Record;
          parameter Modelica.SIunits.Power PRef(min = 0) = 0 "Reference permanent magnet losses at IRef and wRef";
          parameter Real c(min = 0, max = 1) = 0 "Part of permanent magnet losses at current = 0, i.e. independent of current";
          parameter Modelica.SIunits.Current IRef(min = Modelica.Constants.small) "Reference stator RMS current that PRef refers to";
          parameter Real power_I(min = Modelica.Constants.small) = 2 "Exponent of permanent magnet loss torque w.r.t. stator current";
          parameter Modelica.SIunits.AngularVelocity wRef(displayUnit = "1/min", min = Modelica.Constants.small) "Reference angular velocity that PRef refers to";
          parameter Real power_w(min = Modelica.Constants.small) = 1 "Exponent of permanent magnet loss torque w.r.t. angular velocity";
          final parameter Modelica.SIunits.Torque tauRef = if PRef <= 0 then 0 else PRef / wRef "Reference permanent magnet loss torque at reference angular velocity and reference current";
          annotation(
            defaultComponentPrefixes = "parameter ",
            Documentation(info = "<html>
<p>
Parameter record for <a href=\"modelica://Modelica.Electrical.Machines.Losses.InductionMachines.PermanentMagnetLosses\">permanent magnet losses</a>.
</p>
</html>"));
        end PermanentMagnetLossParameters;

        model Friction "Model of angular velocity dependent friction losses"
          extends Machines.Interfaces.FlangeSupport;
          parameter FrictionParameters frictionParameters "Friction loss parameters";
          extends Modelica.Thermal.HeatTransfer.Interfaces.PartialElementaryConditionalHeatPortWithoutT(useHeatPort = false);
        equation
          if frictionParameters.PRef <= 0 then
            tau = 0;
          else
            tau = -smooth(1, if w >= (+frictionParameters.wLinear) then +frictionParameters.tauRef * (+w / frictionParameters.wRef) ^ frictionParameters.power_w else if w <= (-frictionParameters.wLinear) then -frictionParameters.tauRef * (-w / frictionParameters.wRef) ^ frictionParameters.power_w else frictionParameters.tauLinear * (w / frictionParameters.wLinear));
          end if;
          lossPower = -tau * w;
          annotation(
            Icon(graphics = {Ellipse(extent = {{-60, 60}, {60, -60}}, lineColor = {0, 0, 0}, fillColor = {175, 175, 175}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-50, 50}, {50, -50}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-12, 50}, {8, 30}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.Sphere, fillColor = {135, 135, 135}), Ellipse(extent = {{-10, -30}, {10, -50}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.Sphere, fillColor = {135, 135, 135}), Ellipse(extent = {{24, -10}, {44, -30}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.Sphere, fillColor = {135, 135, 135}), Ellipse(extent = {{22, 34}, {42, 14}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.Sphere, fillColor = {135, 135, 135}), Ellipse(extent = {{-44, 30}, {-24, 10}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.Sphere, fillColor = {135, 135, 135}), Ellipse(extent = {{-44, -12}, {-24, -32}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.Sphere, fillColor = {135, 135, 135}), Ellipse(extent = {{-30, 30}, {30, -30}}, lineColor = {0, 0, 0}, fillColor = {175, 175, 175}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-20, 20}, {20, -20}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(visible = useHeatPort, points = {{-40, 0}, {-100, 0}, {-100, -90}}, color = {255, 0, 0}, smooth = Smooth.None), Line(points = {{0, 90}, {0, 0}}, color = {95, 95, 95}, smooth = Smooth.None), Line(points = {{0, -60}, {0, -90}}, color = {95, 95, 95}, smooth = Smooth.None), Text(extent = {{-100, 90}, {100, 60}}, lineColor = {0, 0, 255}, textString = "%name")}),
            Documentation(info = "<html>
<p>
The friction losses are considered by the equations
</p>
<pre>
   tau / tauRef = (+w / wRef) ^ power_w    for w &gt; +wLinear
 - tau / tauRef = (-w / wRef) ^ power_w    for w &lt; -wLinear
</pre>
<p>
with
</p>
<pre>
  tauRef * wRef = PRef
</pre>
<p>
being the friction torque at the reference angular velocity
<code>wRef</code>. The exponent <code>power_w</code> is
approximately 1.5 for axial ventilation and approximately 2.0 for radial ventilation.
</p>
<p>
For stability reasons the friction torque <code>tau</code> is approximated by a linear curve
</p>
<pre>
  tau / tauLinear = w / wLinear
</pre>
<p>
with
</p>
<pre>
  tauLinear = tauRef*(wLinear/wRef) ^ power_w
</pre>
<p>
in the range <code> -wLinear &le; w &le; wLinear</code> with <code>wLinear = 0.001 * wRef</code>. The relationship of torque
and angular velocity is depicted in Fig. 1
</p>
<table border=0 cellspacing=0 cellpadding=1>
  <tr><td> <img src=\"modelica://Modelica/Resources/Images/Electrical/Machines/frictiontorque.png\"
                alt=\"frictiontorque.png\"> </td>
  </tr>
  <tr><td> <b> Fig. 1: </b>Friction loss torque versus angular velocity for <code>power_w = 2</code></td>
  </tr>
</table>
<h4>See also</h4>
<p>
<a href=\"modelica://Modelica.Electrical.Machines.Losses.FrictionParameters\">FrictionParameters</a>
</p>
<p>
If it is desired to neglect friction losses, set <code>frictionParameters.PRef = 0</code> (this is the default).
</p>
</html>"));
        end Friction;

        package InductionMachines "Loss models for induction machines"
          extends Modelica.Icons.VariantsPackage;

          model Brush "Model considering voltage drop of carbon brushes"
            extends Modelica.Electrical.MultiPhase.Interfaces.TwoPlug;
            parameter Machines.Losses.BrushParameters brushParameters "Brush loss parameters";
            extends Modelica.Thermal.HeatTransfer.Interfaces.PartialConditionalHeatPort(useHeatPort = false, final T = 293.15);
            DCMachines.Brush brush[3](each final brushParameters = brushParameters, each final useHeatPort = true) annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}})));
          equation
            connect(plug_p.pin, brush.p) annotation(
              Line(points = {{-100, 0}, {-10, 0}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(brush.n, plug_n.pin) annotation(
              Line(points = {{10, 0}, {100, 0}}, color = {0, 0, 255}, smooth = Smooth.None));
            for j in 1:m loop
              connect(brush[j].heatPort, internalHeatPort) annotation(
                Line(points = {{-10, -10}, {-10, -60}, {-100, -60}, {-100, -80}}, color = {191, 0, 0}, smooth = Smooth.None));
            end for;
            annotation(
              Icon(graphics = {Line(points = {{-100, -100}, {-92, -80}, {-80, -60}, {-60, -40}, {-40, -28}, {-20, -22}, {0, -20}, {20, -22}, {40, -28}, {60, -40}, {80, -60}, {92, -80}, {100, -100}}, color = {0, 0, 255}), Polygon(points = {{-20, -22}, {-40, -28}, {-40, 20}, {40, 20}, {40, -28}, {20, -22}, {0, -20}, {-20, -22}}, lineColor = {0, 0, 255}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid), Line(points = {{-90, 0}, {-40, 0}}, color = {0, 0, 255}), Line(points = {{40, 0}, {90, 0}}, color = {0, 0, 255}), Text(extent = {{-100, 70}, {100, 40}}, lineColor = {0, 0, 255}, textString = "%name")}),
              Documentation(info = "<html>
<p>
Model of voltage drop and losses of carbon brushes. This three-phase model uses three
<a href=\"modelica://Modelica.Electrical.Machines.Losses.DCMachines.Brush\">DC Brush</a> loss models.
</p>
</html>"));
          end Brush;

          model StrayLoad "Model of stray load losses dependent on current and speed"
            extends Modelica.Electrical.MultiPhase.Interfaces.OnePort;
            extends Machines.Interfaces.FlangeSupport;
            import Modelica.Electrical.MultiPhase.Functions.quasiRMS;
            parameter Machines.Losses.StrayLoadParameters strayLoadParameters "Stray load loss parameters";
            extends Modelica.Thermal.HeatTransfer.Interfaces.PartialElementaryConditionalHeatPortWithoutT(useHeatPort = false);
            Modelica.SIunits.Current iRMS = quasiRMS(i);
          equation
            v = zeros(m);
            if strayLoadParameters.PRef <= 0 then
              tau = 0;
            else
              tau = -strayLoadParameters.tauRef * (iRMS / strayLoadParameters.IRef) ^ 2 * smooth(1, if w >= 0 then +(+w / strayLoadParameters.wRef) ^ strayLoadParameters.power_w else -(-w / strayLoadParameters.wRef) ^ strayLoadParameters.power_w);
            end if;
            lossPower = -tau * w;
            annotation(
              Icon(graphics = {Line(points = {{-90, 0}, {90, 0}}, color = {0, 0, 255}), Rectangle(extent = {{-70, 30}, {70, -30}}, lineColor = {0, 0, 255}, pattern = LinePattern.Dot), Text(extent = {{-100, 70}, {100, 40}}, lineColor = {0, 0, 255}, textString = "%name")}),
              Documentation(info = "<html>
<p>
Stray load losses are modeled similar to standards EN 60034-2 and IEEE 112, i.e., they are dependent on square of current,
but without scaling them to zero at no-load current.
</p>
<p>
For an estimation of dependency on varying angular velocity see:<br>
W. Lang, &Uuml;ber die Bemessung verlustarmer Asynchronmotoren mit K&auml;figl&auml;ufer f&uuml;r Pulsumrichterspeisung,
Doctoral Thesis, Technical University of Vienna, 1984.
</p>
<p>
The stray load losses are modeled such way that they do not cause a voltage drop in the electric circuit.
Instead, the dissipated losses are considered through an equivalent braking torque at the shaft.
</p>
<p>
The stray load loss torque is
</p>
<pre>
  tau = PRef/wRef * (i/IRef)^2 * (w/wRef)^power_w
</pre>
<p>
where <code>i</code> is the current of the machine and <code>w</code> is the actual angular velocity.
The dependency of the stray load torque on the angular velocity is modeled by the exponent <code>power_w</code>.
</p>
<h4>See also</h4>
<p>
<a href=\"modelica://Modelica.Electrical.Machines.Losses.StrayLoadParameters\">StrayLoad parameters</a>
</p>
<p>
If it is desired to neglect stray load losses, set <code>strayLoadParameters.PRef = 0</code> (this is the default).
</p>
</html>"));
          end StrayLoad;

          model PermanentMagnetLosses "Model of permanent magnet losses dependent on current and speed"
            extends Machines.Interfaces.FlangeSupport;
            import Modelica.Electrical.MultiPhase.Functions.quasiRMS;
            parameter Integer m(min = 1) = 3 "Number of phases";
            parameter Machines.Losses.PermanentMagnetLossParameters permanentMagnetLossParameters "Permanent magnet loss parameters";
            extends Modelica.Thermal.HeatTransfer.Interfaces.PartialElementaryConditionalHeatPortWithoutT(useHeatPort = false);
            input Modelica.SIunits.Current is[m] "Instantaneous stator currents";
            Modelica.SIunits.Current iRMS = quasiRMS(is);
          equation
            if permanentMagnetLossParameters.PRef <= 0 then
              tau = 0;
            else
              tau = -permanentMagnetLossParameters.tauRef * (permanentMagnetLossParameters.c + (1 - permanentMagnetLossParameters.c) * (iRMS / permanentMagnetLossParameters.IRef) ^ permanentMagnetLossParameters.power_I) * smooth(1, if w >= 0 then +(+w / permanentMagnetLossParameters.wRef) ^ permanentMagnetLossParameters.power_w else -(-w / permanentMagnetLossParameters.wRef) ^ permanentMagnetLossParameters.power_w);
            end if;
            lossPower = -tau * w;
            annotation(
              Icon(graphics = {Ellipse(extent = {{-40, -40}, {40, 40}}, lineColor = {200, 0, 0})}),
              Documentation(info = "<html>
<p>
Permanent magnet losses are modeled dependent on current and speed.
</p>
<p>
The permanent magnet losses are modeled such way that they do not cause a voltage drop in the electric circuit.
Instead, the dissipated losses are considered through an equivalent braking torque at the shaft.
</p>
<p>
The permanent magnet loss torque is
</p>
<pre>
  tau = PRef/wRef * (c + (1 - c) * (i/IRef)^power_I) * (w/wRef)^power_w
</pre>
<p>
where <code>i</code> is the current of the machine and <code>w</code> is the actual angular velocity.
The parameter <code>c</code> designates the part of the permanent magnet losses that are present even at current = 0, i.e. independent of current.
The dependency of the permanent magnet loss torque on the stator current is modeled by the exponent <code>power_I</code>.
The dependency of the permanent magnet loss torque on the angular velocity is modeled by the exponent <code>power_w</code>.
</p>
<h4>See also</h4>
<p>
<a href=\"modelica://Modelica.Electrical.Machines.Losses.PermanentMagnetLossParameters\">Permanent magnet loss parameters</a>
</p>
<p>
If it is desired to neglect permanent magnet losses, set <code>strayLoadParameters.PRef = 0</code> (this is the default).
</p>
</html>"));
          end PermanentMagnetLosses;

          model Core "Model of core losses"
            parameter Machines.Losses.CoreParameters coreParameters;
            final parameter Integer m = coreParameters.m "Number of phases";
            parameter Real turnsRatio(final min = Modelica.Constants.small) "Effective number of stator turns / effective number of rotor turns (if used as rotor core)";
            extends Modelica.Thermal.HeatTransfer.Interfaces.PartialElementaryConditionalHeatPortWithoutT(useHeatPort = false);
            Machines.Interfaces.SpacePhasor spacePhasor annotation(
              Placement(transformation(extent = {{-110, -10}, {-90, 10}}, rotation = 0), iconTransformation(extent = {{-110, -10}, {-90, 10}})));
            input Modelica.SIunits.AngularVelocity w "Remagnetization angular velocity" annotation(
              Dialog(group = "Losses"));
            Modelica.SIunits.Conductance Gc "Variable core loss conductance";
          protected
            Modelica.SIunits.AngularVelocity wLimit = noEvent(max(noEvent(abs(w)), coreParameters.wMin)) "Limited angular velocity";
          equation
            if coreParameters.PRef <= 0 then
              Gc = 0;
              spacePhasor.i_ = zeros(2);
            else
              Gc = coreParameters.GcRef;
//  * (coreParameters.wRef/wLimit*coreParameters.ratioHysteresis + 1 - coreParameters.ratioHysteresis);
              spacePhasor.i_ = Gc * spacePhasor.v_;
            end if;
            lossPower = 3 / 2 * ((+spacePhasor.v_[1] * spacePhasor.i_[1]) + spacePhasor.v_[2] * spacePhasor.i_[2]);
            annotation(
              Icon(graphics = {Rectangle(extent = {{-70, 30}, {70, -30}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{-90, 0}, {-70, 0}}), Line(points = {{-70, 10}, {70, 10}}, color = {0, 0, 255}), Line(points = {{-70, -30}, {70, -30}}, color = {0, 0, 255}), Line(points = {{-70, -10}, {70, -10}}, color = {0, 0, 255}), Line(points = {{70, 0}, {80, 0}}, color = {0, 0, 255}, smooth = Smooth.None), Line(points = {{80, 20}, {80, -20}}, color = {0, 0, 255}, smooth = Smooth.None), Line(points = {{90, 14}, {90, -14}}, color = {0, 0, 255}, smooth = Smooth.None), Line(points = {{100, 8}, {100, -8}}, color = {0, 0, 255}, smooth = Smooth.None), Text(extent = {{-100, 70}, {100, 40}}, lineColor = {0, 0, 255}, textString = "%name")}),
              Documentation(info = "<html>
<p>
Core losses can be separated into <b>eddy current</b> and <b>hysteresis</b> losses. The total core losses
can thus be expressed as
</p>
<pre>
  P = PRef * (ratioHysteresis * (wRef / w) + 1 - ratioHysteresis) * (V / VRef)^2
</pre>
<p>
where <code>w</code> is the actual angular remagnetization velocity and <code>V</code> is the actual voltage.
The term <code>ratioHysteresis</code> is the ratio of the hysteresis losses with respect to the total core losses for the reference voltage and frequency.
</p>

<p>
In the current implementation hysteresis losses are <b>not considered</b> since complex numbers are not provided in Modelica.
Therefore, implicitly <code>ratioHysteresis = 0</code> is set. For the voltage and frequency range with respect to Fig.&nbsp;1,
the dependency of total core losses on the parameter <code>ratioHysteresis</code> is depicted in Fig.&nbsp;2.
The current implementation has thus the drawback over a model that considers <code>ratioHysteresis &gt; 0</code>:
</p>
<ul>
<li>underestimation of the losses in the constant field region (<code>w</code> &lt; <code>wRef</code>)</li>
<li>overestimation of the losses in the field weakening region (<code>w</code> &gt; <code>wRef</code>)</li>
</ul>

<table border=0 cellspacing=0 cellpadding=1>
  <tr><td> <img src=\"modelica://Modelica/Resources/Images/Electrical/Machines/corelossesVw.png\"
                alt=\"corelossesVw.png\"> </td>
  </tr>
  <tr><td> <b> Fig. 1: </b>Voltage versus angular velocity</td>
  </tr>
</table>

<table border=0 cellspacing=0 cellpadding=1>
  <tr><td> <img src=\"modelica://Modelica/Resources/Images/Electrical/Machines/corelossesPcw.png\"
                alt=\"corelossesPcw.png\"> </td>
  </tr>
  <tr><td> <b> Fig. 2: </b>Core losses versus angular velocity with parameter <code>ratioHysteresis</code></td>
  </tr>
</table>

<h4>Note</h4>
<p>In the current implementation it is assumed that <code>ratioHysteresis = 0</code>. This parameter cannot be changed due to compatibility reasons.</p>

<h4>See also</h4>
<p>
<a href=\"modelica://Modelica.Electrical.Machines.Losses.CoreParameters\">Core loss parameters</a>
</p>

</html>"));
          end Core;
          annotation(
            Documentation(info = "<HTML>
This package contains loss models used for induction machine models.
</HTML>", revisions = "<HTML>
<dl>
  <dt><b>Main Authors:</b></dt>
  <dd>
  <a href=\"http://www.haumer.at/\">Anton Haumer</a><br>
  Technical Consulting &amp; Electrical Engineering<br>
  A-3423 St.Andrae-Woerdern<br>Austria<br>
  email: <a href=\"mailto:a.haumer@haumer.at\">a.haumer@haumer.at</a>
  </dd>
  <dt><b>Copyright:</b></dt>
  <dd>Copyright &copy; 1998-2015, Modelica Association and Anton Haumer.<br>
  <i>The Modelica package is <b>free</b> software; it can be redistributed and/or modified
  under the terms of the <b>Modelica license</b>, see the license conditions
  and the accompanying <b>disclaimer</b> in the documentation of package
  Modelica in file \"Modelica/package.mo\".</i></dd>
</dl>
  <ul>
  <li> v2.4.0 2010/04/20 Anton Haumer<br>
       first implementation</li>
  </ul>
</HTML>"));
        end InductionMachines;

        package DCMachines "Loss models for DC machines"
          extends Modelica.Icons.VariantsPackage;

          model Brush "Model considering voltage drop of carbon brushes"
            extends Modelica.Electrical.Analog.Interfaces.OnePort;
            parameter Machines.Losses.BrushParameters brushParameters "Brush loss parameters";
            extends Modelica.Thermal.HeatTransfer.Interfaces.PartialElementaryConditionalHeatPortWithoutT(useHeatPort = false);
          equation
            if brushParameters.V <= 0 then
              v = 0;
            else
              v = smooth(1, if i > (+brushParameters.ILinear) then +brushParameters.V else if i < (-brushParameters.ILinear) then -brushParameters.V else brushParameters.V * i / brushParameters.ILinear);
            end if;
            lossPower = v * i;
            annotation(
              Icon(graphics = {Line(points = {{-100, -100}, {-92, -80}, {-80, -60}, {-60, -40}, {-40, -28}, {-20, -22}, {0, -20}, {20, -22}, {40, -28}, {60, -40}, {80, -60}, {92, -80}, {100, -100}}, color = {0, 0, 255}), Polygon(points = {{-20, -22}, {-40, -28}, {-40, 20}, {40, 20}, {40, -28}, {20, -22}, {0, -20}, {-20, -22}}, lineColor = {0, 0, 255}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid), Line(points = {{-90, 0}, {-40, 0}}, color = {0, 0, 255}), Line(points = {{40, 0}, {90, 0}}, color = {0, 0, 255}), Text(extent = {{-100, 70}, {100, 40}}, lineColor = {0, 0, 255}, textString = "%name")}),
              Documentation(info = "<html>
<p>
Model of voltage drop and losses of carbon brushes. For currents between <code>-ILinear</code> and <code>ILinear</code>
 the voltage drop shows a linear behavior as depicted in Fig. 1.
 For positive currents greater or equal than <code>ILinear</code> the voltage drop equals <code>V</code>.
 For negative currents less or equal than <code>-ILinear</code> the voltage drop equals <code>-V</code>.
</p>

<table border=0 cellspacing=0 cellpadding=1>
  <tr><td> <img src=\"modelica://Modelica/Resources/Images/Electrical/Machines/brush.png\"
                alt=\"brush.png\"> </td> </tr>
  <tr><td> <b> Fig. 1: </b>Model of voltage drop of carbon brushes</td> </tr>
</table>
<h4>Note:</h4>
<p>
The voltage drop <code>v</code> is the total voltage drop of all series connected brushes.
</p>

<h4>See also</h4>

<p>
<a href=\"modelica://Modelica.Electrical.Machines.Losses.BrushParameters\">BrushParameters</a>
</p>
<p>
If it is desired to neglect brush losses, set <code>brushParameters.V = 0</code> (this is the default).
</p>
</html>"));
          end Brush;

          function brushVoltageDrop "Voltage drop of carbon brushes"
            extends Modelica.Icons.Function;
            input Machines.Losses.BrushParameters brushParameters "Brush loss parameters";
            input Modelica.SIunits.Current i "Actual current";
            output Modelica.SIunits.Voltage v "Voltage drop";
          algorithm
            if brushParameters.V <= 0 then
              v := 0;
            else
              v := if i > (+brushParameters.ILinear) then +brushParameters.V else if i < (-brushParameters.ILinear) then -brushParameters.V else brushParameters.V * i / brushParameters.ILinear;
            end if;
            annotation(
              Documentation(info = "<html>
<p>
Calculates the voltage drop of carbon brushes, according to
<a href=\"modelica://Modelica.Electrical.Machines.Losses.DCMachines.Brush\">Brush</a> losses,
e.g., used for initial equations.
</p>
</html>"));
          end brushVoltageDrop;

          model StrayLoad "Model of stray load losses dependent on current and speed"
            extends Modelica.Electrical.Analog.Interfaces.OnePort;
            extends Machines.Interfaces.FlangeSupport;
            parameter Machines.Losses.StrayLoadParameters strayLoadParameters "Stray load loss parameters";
            extends Modelica.Thermal.HeatTransfer.Interfaces.PartialElementaryConditionalHeatPortWithoutT(useHeatPort = false);
          equation
            v = 0;
            if strayLoadParameters.PRef <= 0 then
              tau = 0;
            else
              tau = -strayLoadParameters.tauRef * (i / strayLoadParameters.IRef) ^ 2 * smooth(1, if w >= 0 then +(+w / strayLoadParameters.wRef) ^ strayLoadParameters.power_w else -(-w / strayLoadParameters.wRef) ^ strayLoadParameters.power_w);
            end if;
            lossPower = -tau * w;
            annotation(
              Icon(graphics = {Line(points = {{-90, 0}, {90, 0}}, color = {0, 0, 255}), Rectangle(extent = {{-70, 30}, {70, -30}}, lineColor = {0, 0, 255}, pattern = LinePattern.Dot), Text(extent = {{-100, 70}, {100, 40}}, lineColor = {0, 0, 255}, textString = "%name")}),
              Documentation(info = "<html>
<p>
The stray load loss torque is
</p>
<pre>
  tau = PRef/wRef * (i/IRef)^2 * (w/wRef)^power_w
</pre>
<p>
where <code>i</code> is the current of the machine and <code>w</code> is the actual angular velocity.
The dependency of the stray load torque on the angular velocity is modeled by the exponent <code>power_w</code>.
</p>
<p>
The stray load losses are modeled such way that they do not cause a voltage drop in the electric circuit.
Instead, the dissipated losses are considered through an equivalent braking torque at the shaft.
</p>
<h4>See also</h4>
<p>
<a href=\"modelica://Modelica.Electrical.Machines.Losses.StrayLoadParameters\">StrayLoad parameters</a>
</p>
<p>
If it is desired to neglect stray load losses, set <code>strayLoadParameters.PRef = 0</code> (this is the default).
</p>
</html>"));
          end StrayLoad;

          model Core "Model of core losses"
            extends Modelica.Electrical.Analog.Interfaces.OnePort;
            parameter Machines.Losses.CoreParameters coreParameters "Armature core losses";
            extends Modelica.Thermal.HeatTransfer.Interfaces.PartialElementaryConditionalHeatPortWithoutT(useHeatPort = false);
            input Modelica.SIunits.AngularVelocity w "Remagnetization angular velocity" annotation(
              Dialog(group = "Losses"));
            Modelica.SIunits.Conductance Gc "Variable core loss conductance";
          protected
            Modelica.SIunits.AngularVelocity wLimit = noEvent(max(noEvent(abs(w)), coreParameters.wMin)) "Limited angular velocity";
          equation
            if coreParameters.PRef <= 0 then
              Gc = 0;
              i = 0;
            else
              Gc = coreParameters.GcRef;
// * (coreParameters.wRef/wLimit*coreParameters.ratioHysteresis + 1 - coreParameters.ratioHysteresis);
              i = Gc * v;
            end if;
            lossPower = v * i;
            annotation(
              Icon(graphics = {Rectangle(extent = {{-70, 30}, {70, -30}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{70, 0}, {90, 0}}), Line(points = {{-90, 0}, {-70, 0}}), Line(points = {{-70, 10}, {70, 10}}, color = {0, 0, 255}), Line(points = {{-70, -30}, {70, -30}}, color = {0, 0, 255}), Line(points = {{-70, -10}, {70, -10}}, color = {0, 0, 255}), Text(extent = {{-100, 70}, {100, 40}}, lineColor = {0, 0, 255}, textString = "%name")}),
              Documentation(info = "<html>
<p>
Core losses can be separated into <i>eddy current</i> and <i>hysteresis</i> losses. The total core losses
can thus be expressed as
</p>
<pre>
  p = PRef * (ratioHysteresis * (wRef / w) + 1 - ratioHysteresis) * (v / VRef)^2
</pre>
<p>
where <code>w</code> is the actual angular velocity and <code>v</code> is the actual voltage. The term <code>ratioHysteresis</code> is the ratio
of the hysteresis losses with respect to the total core losses for reference inner voltage and reference angular velocity.
</p>

<p>
For the voltage and angular velocity range with respect to Fig.&nbsp;1,
the dependency of total core losses on the parameter <code>ratioHysteresis</code> is depicted in Fig.&nbsp;2.
</p>

<table border=0 cellspacing=0 cellpadding=1>
  <tr><td> <img src=\"modelica://Modelica/Resources/Images/Electrical/Machines/corelossesVw.png\"
                alt=\"corelossesVw.png\"> </td>
  </tr>
  <tr><td> <b> Fig. 1: </b>Voltage versus angular velocity</td>
  </tr>
</table>

<table border=0 cellspacing=0 cellpadding=1>
  <tr><td> <img src=\"modelica://Modelica/Resources/Images/Electrical/Machines/corelossesPcw.png\"
                alt=\"corelossesPcw.png\"> </td>
  </tr>
  <tr><td> <b> Fig. 2: </b>Core losses versus angular velocity with parameter <code>ratioHysteresis</code></td>
  </tr>
</table>

<h4>Note</h4>
<p>In the current implementation it is assumed that <code>ratioHysteresis = 0</code>. This parameter cannot be changed due to compatibility reasons.</p>

<h4>See also</h4>
<p>
<a href=\"modelica://Modelica.Electrical.Machines.Losses.CoreParameters\">Core loss parameters</a>
</p>

</html>"));
          end Core;
          annotation(
            Documentation(info = "<HTML>
This package contains loss models used for DC machine models.
</HTML>", revisions = "<HTML>
<dl>
  <dt><b>Main Authors:</b></dt>
  <dd>
  <a href=\"http://www.haumer.at/\">Anton Haumer</a><br>
  Technical Consulting &amp; Electrical Engineering<br>
  A-3423 St.Andrae-Woerdern<br>Austria<br>
  email: <a href=\"mailto:a.haumer@haumer.at\">a.haumer@haumer.at</a>
  </dd>
  <dt><b>Copyright:</b></dt>
  <dd>Copyright &copy; 1998-2015, Modelica Association and Anton Haumer.<br>
  <i>The Modelica package is <b>free</b> software; it can be redistributed and/or modified
  under the terms of the <b>Modelica license</b>, see the license conditions
  and the accompanying <b>disclaimer</b> in the documentation of package
  Modelica in file \"Modelica/package.mo\".</i></dd>
</dl>
  <ul>
  <li> v2.4.0 2010/04/20 Anton Haumer<br>
       first implementation</li>
  </ul>
</HTML>"));
        end DCMachines;
        annotation(
          Documentation(info = "<HTML>
This package contains loss models and their parameter records used for machine models.
</HTML>", revisions = "<HTML>
<dl>
  <dt><b>Main Authors:</b></dt>
  <dd>
  <a href=\"http://www.haumer.at/\">Anton Haumer</a><br>
  Technical Consulting &amp; Electrical Engineering<br>
  A-3423 St.Andrae-Woerdern<br>Austria<br>
  email: <a href=\"mailto:a.haumer@haumer.at\">a.haumer@haumer.at</a>
  </dd>
  <dt><b>Copyright:</b></dt>
  <dd>Copyright &copy; 1998-2015, Modelica Association and Anton Haumer.<br>
  <i>The Modelica package is <b>free</b> software; it can be redistributed and/or modified
  under the terms of the <b>Modelica license</b>, see the license conditions
  and the accompanying <b>disclaimer</b> in the documentation of package
  Modelica in file \"Modelica/package.mo\".</i></dd>
</dl>
  <ul>
  <li> v2.4.0 2010/04/20 Anton Haumer<br>
       first implementation</li>
  </ul>
</HTML>"));
      end Losses;

      package Thermal "Library with models for connecting thermal models"
        extends Modelica.Icons.Package;
        type LinearTemperatureCoefficient20 = Modelica.SIunits.LinearTemperatureCoefficient "Linear temperature coefficient with choices" annotation(
          choices(choice = Modelica.Electrical.Machines.Thermal.Constants.alpha20Zero "Not temperature dependent", choice = Modelica.Electrical.Machines.Thermal.Constants.alpha20Aluminium "Aluminium", choice = Modelica.Electrical.Machines.Thermal.Constants.alpha20Brass "Brass", choice = Modelica.Electrical.Machines.Thermal.Constants.alpha20Bronze "Bronze", choice = Modelica.Electrical.Machines.Thermal.Constants.alpha20Copper "Copper"));

        package Constants "Material Constants"
          extends Modelica.Icons.Package;
          constant Modelica.SIunits.LinearTemperatureCoefficient alpha20Zero = 0 "Not temperature dependent";
          constant Modelica.SIunits.LinearTemperatureCoefficient alpha20Aluminium = 4.000e-3 "Aluminium";
          constant Modelica.SIunits.LinearTemperatureCoefficient alpha20Brass = 1.100e-3 "Brass";
          constant Modelica.SIunits.LinearTemperatureCoefficient alpha20Bronze = 0.800e-3 "Bronze";
          constant Modelica.SIunits.LinearTemperatureCoefficient alpha20Copper = 3.920e-3 "Copper";
          annotation(
            Icon(coordinateSystem(extent = {{-100.0, -100.0}, {100.0, 100.0}}), graphics = {Polygon(origin = {-9.2597, 25.6673}, fillColor = {102, 102, 102}, pattern = LinePattern.None, fillPattern = FillPattern.Solid, points = {{48.017, 11.336}, {48.017, 11.336}, {10.766, 11.336}, {-25.684, 10.95}, {-34.944, -15.111}, {-34.944, -15.111}, {-32.298, -15.244}, {-32.298, -15.244}, {-22.112, 0.168}, {11.292, 0.234}, {48.267, -0.097}, {48.267, -0.097}}, smooth = Smooth.Bezier), Polygon(origin = {-19.9923, -8.3993}, fillColor = {102, 102, 102}, pattern = LinePattern.None, fillPattern = FillPattern.Solid, points = {{3.239, 37.343}, {3.305, 37.343}, {-0.399, 2.683}, {-16.936, -20.071}, {-7.808, -28.604}, {6.811, -22.519}, {9.986, 37.145}, {9.986, 37.145}}, smooth = Smooth.Bezier), Polygon(origin = {23.753, -11.5422}, fillColor = {102, 102, 102}, pattern = LinePattern.None, fillPattern = FillPattern.Solid, points = {{-10.873, 41.478}, {-10.873, 41.478}, {-14.048, -4.162}, {-9.352, -24.8}, {7.912, -24.469}, {16.247, 0.27}, {16.247, 0.27}, {13.336, 0.071}, {13.336, 0.071}, {7.515, -9.983}, {-3.134, -7.271}, {-2.671, 41.214}, {-2.671, 41.214}}, smooth = Smooth.Bezier)}),
            Documentation(info = "<HTML>
Material constants, especially linear temperature coefficients of commonly used conductor materials
</HTML>"));
        end Constants;

        function convertAlpha "Converts alpha from temperature 1 (default 20 degC) to temperature 2"
          extends Modelica.Icons.Function;
          input Modelica.SIunits.LinearTemperatureCoefficient alpha1 "Temperature coefficient at temperature 1 (default: 20 degC)";
          input Modelica.SIunits.Temperature T2 "Temperature 2";
          input Modelica.SIunits.Temperature T1 = 293.15 "Temperature 1 (default: 20 degC)";
          output Modelica.SIunits.LinearTemperatureCoefficient alpha2 "Temperature coefficient at TRef";
        algorithm
          alpha2 := alpha1 / (1 + alpha1 * (T2 - T1));
          annotation(
            Inline = true,
            Documentation(info = "<html>
<p>
From the temperature coefficient <code>alpha1</code> at temperature <code>T1</code> (default 20 degC = 293.15 K)
the temperature coefficient <code>alpha2</code> at temperature <code>T2</code> is calculated:
</p>
<pre>
                alpha1
  alpha2 = ------------------------
            1 + alpha1 * (T2 - T1)
</pre>
</html>"));
        end convertAlpha;

        function convertResistance "Converts resistance from reference temperature to an actual temperature"
          extends Modelica.Icons.Function;
          input Modelica.SIunits.Resistance RRef "Resistance at TRef";
          input Modelica.SIunits.Temperature TRef "Reference temperature";
          input Modelica.SIunits.LinearTemperatureCoefficient alpha20 "Temperature coefficient at 20 degC";
          input Modelica.SIunits.Temperature T "Actual temperature";
          output Modelica.SIunits.Resistance R "Actual resistance at T";
        algorithm
          R := RRef * (1 + Machines.Thermal.convertAlpha(alpha20, TRef) * (T - TRef));
          annotation(
            Inline = true,
            Documentation(info = "<html>
<p>
From the temperature coefficient <code>alpha20</code> at 20 degC (equals to 293.15 K) the parameter <code>alphaRef</code> at <code>TRef</code>
</p>
<pre>
                        alpha20
  alphaRef = -------------------------------
              1 + alpha20 * (TRef - 293.15)
</pre>
<p>
is determined; using this value, actual resistance <code>R</code> with respect to the actual temperature <code>T</code> is calculated by
</p>
<pre>
   R
  ------ = 1 + alphaRef * (T - TRef)
   RRef
</pre>
<p>
where <code>RRef</code> is the resistance at the reference temperature <code>TRef</code>.
</p>
</html>"));
        end convertResistance;

        package AsynchronousInductionMachines "Thermal parts of asynchronous induction machines"
          extends Modelica.Icons.VariantsPackage;

          model ThermalAmbientAIMC "Thermal ambient for asynchronous induction machine with squirrel cage"
            extends Machines.Interfaces.InductionMachines.PartialThermalAmbientInductionMachines(redeclare final Machines.Interfaces.InductionMachines.ThermalPortAIMC thermalPort);
            parameter Modelica.SIunits.Temperature Tr(start = TDefault) "Temperature of rotor (squirrel cage)" annotation(
              Dialog(enable = not useTemperatureInputs));
            output Modelica.SIunits.HeatFlowRate Q_flowRotorWinding = temperatureRotorWinding.port.Q_flow "Heat flow rate of rotor (squirrel cage)";
            output Modelica.SIunits.HeatFlowRate Q_flowTotal = Q_flowStatorWinding + Q_flowRotorWinding + Q_flowStatorCore + Q_flowRotorCore + Q_flowStrayLoad + Q_flowFriction;
            Modelica.Thermal.HeatTransfer.Sources.PrescribedTemperature temperatureRotorWinding annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 90, origin = {-50, 30})));
            Modelica.Blocks.Interfaces.RealInput TRotorWinding if useTemperatureInputs "Temperature of squirrel cage" annotation(
              Placement(transformation(extent = {{-20, -20}, {20, 20}}, rotation = 90, origin = {80, -100})));
            Modelica.Blocks.Sources.Constant constTr(final k = Tr) if not useTemperatureInputs annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 90, origin = {-50, -10})));
          equation
            connect(constTr.y, temperatureRotorWinding.T) annotation(
              Line(points = {{-50, 1}, {-50, 18}}, color = {0, 0, 127}, smooth = Smooth.None));
            connect(temperatureRotorWinding.port, thermalPort.heatPortRotorWinding) annotation(
              Line(points = {{-50, 40}, {-50, 100}, {0, 100}}, color = {191, 0, 0}, smooth = Smooth.None));
            connect(TRotorWinding, temperatureRotorWinding.T) annotation(
              Line(points = {{80, -100}, {80, 8}, {-50, 8}, {-50, 18}}, color = {0, 0, 127}, smooth = Smooth.None));
            annotation(
              Icon(graphics = {Text(extent = {{-100, -20}, {100, -80}}, lineColor = {0, 0, 0}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid, textString = "AIMC")}),
              Documentation(info = "<HTML>
Thermal ambient for asynchronous induction machines with squirrel cage to prescribe winding temperatures either constant or via signal connectors.
Additionally, all losses = heat flows are recorded.
</HTML>"));
          end ThermalAmbientAIMC;

          model ThermalAmbientAIMS "Thermal ambient for asynchronous induction machine with slipring"
            extends Machines.Interfaces.InductionMachines.PartialThermalAmbientInductionMachines(redeclare final Machines.Interfaces.InductionMachines.ThermalPortAIMS thermalPort(final mr = mr));
            parameter Integer mr = m "Number of rotor phases";
            parameter Modelica.SIunits.Temperature Tr(start = TDefault) "Temperature of rotor windings" annotation(
              Dialog(enable = not useTemperatureInputs));
            output Modelica.SIunits.HeatFlowRate Q_flowRotorWinding = temperatureRotorWinding.port.Q_flow "Heat flow rate of rotor (squirrel cage)";
            output Modelica.SIunits.HeatFlowRate Q_flowBrush = temperatureBrush.port.Q_flow "Heat flow rate of brushes";
            output Modelica.SIunits.HeatFlowRate Q_flowTotal = Q_flowStatorWinding + Q_flowRotorWinding + Q_flowStatorCore + Q_flowRotorCore + Q_flowStrayLoad + Q_flowFriction + Q_flowBrush;
            Modelica.Thermal.HeatTransfer.Sources.PrescribedTemperature temperatureRotorWinding annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 90, origin = {-50, 30})));
            Modelica.Blocks.Interfaces.RealInput TRotorWinding if useTemperatureInputs "Temperature of rotor windings" annotation(
              Placement(transformation(extent = {{-20, -20}, {20, 20}}, rotation = 90, origin = {80, -100})));
            Modelica.Blocks.Sources.Constant constTr(final k = Tr) if not useTemperatureInputs annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 90, origin = {-50, -10})));
            Modelica.Thermal.HeatTransfer.Components.ThermalCollector thermalCollectorRotor(final m = mr) annotation(
              Placement(transformation(extent = {{-60, 60}, {-40, 80}})));
            Modelica.Thermal.HeatTransfer.Sources.FixedTemperature temperatureBrush(final T = TDefault) annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 90, origin = {0, 50})));
          equation
            connect(constTr.y, temperatureRotorWinding.T) annotation(
              Line(points = {{-50, 1}, {-50, 18}}, color = {0, 0, 127}, smooth = Smooth.None));
            connect(thermalCollectorRotor.port_b, temperatureRotorWinding.port) annotation(
              Line(points = {{-50, 60}, {-50, 40}}, color = {191, 0, 0}, smooth = Smooth.None));
            connect(thermalCollectorRotor.port_a, thermalPort.heatPortRotorWinding) annotation(
              Line(points = {{-50, 80}, {-50, 100}, {0, 100}}, color = {191, 0, 0}, smooth = Smooth.None));
            connect(temperatureBrush.port, thermalPort.heatPortBrush) annotation(
              Line(points = {{0, 60}, {0, 60}, {0, 100}}, color = {191, 0, 0}, smooth = Smooth.None));
            connect(TRotorWinding, temperatureRotorWinding.T) annotation(
              Line(points = {{80, -100}, {80, 8}, {-50, 8}, {-50, 18}}, color = {0, 0, 127}, smooth = Smooth.None));
            annotation(
              Icon(graphics = {Text(extent = {{-100, -20}, {100, -80}}, lineColor = {0, 0, 0}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid, textString = "AIMS")}),
              Documentation(info = "<HTML>
Thermal ambient for asynchronous induction machines with slipring rotor to prescribe winding temperatures either constant or via signal connectors.
Additionally, all losses = heat flows are recorded.
</HTML>"));
          end ThermalAmbientAIMS;
          annotation(
            Documentation(info = "<HTML>
Thermal parts for asynchronous induction machines
</HTML>"));
        end AsynchronousInductionMachines;

        package SynchronousInductionMachines "Thermal parts of synchronous induction machines"
          extends Modelica.Icons.VariantsPackage;

          model ThermalAmbientSMPM "Thermal ambient for synchronous induction machine with permanent magnets"
            import Modelica;
            parameter Boolean useDamperCage(start = true) "Enable / disable damper cage" annotation(
              Evaluate = true);
            extends Machines.Interfaces.InductionMachines.PartialThermalAmbientInductionMachines(redeclare final Machines.Interfaces.InductionMachines.ThermalPortSMPM thermalPort(final useDamperCage = useDamperCage));
            parameter Modelica.SIunits.Temperature Tpm(start = TDefault) "Temperature of permanent magnet" annotation(
              Dialog(enable = not useTemperatureInputs));
            parameter Modelica.SIunits.Temperature Tr(start = TDefault) "Temperature of damper cage (optional)" annotation(
              Dialog(enable = not useTemperatureInputs and useDamperCage));
            output Modelica.SIunits.HeatFlowRate Q_flowRotorWinding = temperatureRotorWinding.port.Q_flow "Heat flow rate of damper cage (optional)";
            output Modelica.SIunits.HeatFlowRate Q_flowPermanentMagnet = temperaturePermanentMagnet.port.Q_flow "Heat flow rate of permanent magnets";
            output Modelica.SIunits.HeatFlowRate Q_flowTotal = Q_flowStatorWinding + Q_flowRotorWinding + Q_flowPermanentMagnet + Q_flowStatorCore + Q_flowRotorCore + Q_flowStrayLoad + Q_flowFriction;
            Modelica.Thermal.HeatTransfer.Sources.PrescribedTemperature temperatureRotorWinding annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 90, origin = {-20, 30})));
            Modelica.Blocks.Interfaces.RealInput TRotorWinding if useTemperatureInputs and useDamperCage "Temperature of damper cage (optional)" annotation(
              Placement(transformation(extent = {{-20, -20}, {20, 20}}, rotation = 90, origin = {80, -100})));
            Modelica.Blocks.Sources.Constant constTr(final k = if useDamperCage then Tr else TDefault) if not useTemperatureInputs or not useDamperCage annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 90, origin = {-20, -10})));
            Modelica.Thermal.HeatTransfer.Sources.PrescribedTemperature temperaturePermanentMagnet annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 90, origin = {-50, 30})));
            Modelica.Blocks.Sources.Constant constTpm(final k = Tpm) if not useTemperatureInputs annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 90, origin = {-50, -10})));
            Modelica.Blocks.Interfaces.RealInput TPermanentMagnet if useTemperatureInputs "Temperature of permanent magnet" annotation(
              Placement(transformation(extent = {{-20, -20}, {20, 20}}, rotation = 90, origin = {0, -100})));
          equation
            connect(constTr.y, temperatureRotorWinding.T) annotation(
              Line(points = {{-20, 1}, {-20, 18}}, color = {0, 0, 127}, smooth = Smooth.None));
            connect(temperatureRotorWinding.port, thermalPort.heatPortRotorWinding) annotation(
              Line(points = {{-20, 40}, {-20, 100}, {0, 100}}, color = {191, 0, 0}, smooth = Smooth.None));
            connect(TRotorWinding, temperatureRotorWinding.T) annotation(
              Line(points = {{80, -100}, {80, 8}, {-20, 8}, {-20, 18}}, color = {0, 0, 127}, smooth = Smooth.None));
            connect(temperaturePermanentMagnet.port, thermalPort.heatPortPermanentMagnet) annotation(
              Line(points = {{-50, 40}, {-50, 100}, {0, 100}}, color = {191, 0, 0}, smooth = Smooth.None));
            connect(constTpm.y, temperaturePermanentMagnet.T) annotation(
              Line(points = {{-50, 1}, {-50, 18}}, color = {0, 0, 127}, smooth = Smooth.None));
            connect(TPermanentMagnet, temperaturePermanentMagnet.T) annotation(
              Line(points = {{0, -100}, {0, -60}, {-60, -60}, {-60, 8}, {-50, 8}, {-50, 18}}, color = {0, 0, 127}, smooth = Smooth.None));
            annotation(
              Icon(graphics = {Text(extent = {{-100, -20}, {100, -80}}, lineColor = {0, 0, 0}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid, textString = "SMPM")}),
              Documentation(info = "<HTML>
Thermal ambient for synchronous induction machines with permanent magnets to prescribe winding temperatures either constant or via signal connectors.
Additionally, all losses = heat flows are recorded.
</HTML>"));
          end ThermalAmbientSMPM;

          model ThermalAmbientSMEE "Thermal ambient for synchronous induction machine with electrical excitation"
            parameter Boolean useDamperCage(start = true) "Enable / disable damper cage" annotation(
              Evaluate = true);
            extends Machines.Interfaces.InductionMachines.PartialThermalAmbientInductionMachines(redeclare final Machines.Interfaces.InductionMachines.ThermalPortSMEE thermalPort(final useDamperCage = useDamperCage));
            parameter Modelica.SIunits.Temperature Tr(start = TDefault) "Temperature of damper cage (optional)" annotation(
              Dialog(enable = not useTemperatureInputs and useDamperCage));
            parameter Modelica.SIunits.Temperature Te(start = TDefault) "Temperature of excitation" annotation(
              Dialog(enable = not useTemperatureInputs));
            output Modelica.SIunits.HeatFlowRate Q_flowRotorWinding = temperatureRotorWinding.port.Q_flow "Heat flow rate of damper cage (optional)";
            output Modelica.SIunits.HeatFlowRate Q_flowExcitation = temperatureExcitation.port.Q_flow "Heat flow rate of excitation";
            output Modelica.SIunits.HeatFlowRate Q_flowTotal = Q_flowStatorWinding + Q_flowRotorWinding + Q_flowExcitation + Q_flowStatorCore + Q_flowRotorCore + Q_flowStrayLoad + Q_flowFriction;
            Modelica.Thermal.HeatTransfer.Sources.PrescribedTemperature temperatureRotorWinding annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 90, origin = {-20, 30})));
            Modelica.Blocks.Interfaces.RealInput TRotorWinding if useTemperatureInputs and useDamperCage "Temperature of damper cage (optional)" annotation(
              Placement(transformation(extent = {{-20, -20}, {20, 20}}, rotation = 90, origin = {80, -100})));
            Modelica.Blocks.Sources.Constant constTr(final k = if useDamperCage then Tr else TDefault) if not useTemperatureInputs or not useDamperCage annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 90, origin = {-20, -10})));
            Modelica.Thermal.HeatTransfer.Sources.PrescribedTemperature temperatureExcitation annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 90, origin = {-50, 30})));
            Modelica.Blocks.Interfaces.RealInput TExcitation if useTemperatureInputs "Temperature of excitation" annotation(
              Placement(transformation(extent = {{-20, -20}, {20, 20}}, rotation = 90, origin = {0, -100})));
            Modelica.Blocks.Sources.Constant constTe(final k = Te) if not useTemperatureInputs annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 90, origin = {-50, -10})));
            Modelica.Thermal.HeatTransfer.Sources.FixedTemperature temperatureBrush(final T = TDefault) annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 90, origin = {0, 50})));
          equation
            connect(constTr.y, temperatureRotorWinding.T) annotation(
              Line(points = {{-20, 1}, {-20, 18}}, color = {0, 0, 127}, smooth = Smooth.None));
            connect(temperatureRotorWinding.port, thermalPort.heatPortRotorWinding) annotation(
              Line(points = {{-20, 40}, {-20, 100}, {0, 100}}, color = {191, 0, 0}, smooth = Smooth.None));
            connect(TRotorWinding, temperatureRotorWinding.T) annotation(
              Line(points = {{80, -100}, {80, 8}, {-20, 8}, {-20, 18}}, color = {0, 0, 127}, smooth = Smooth.None));
            connect(constTe.y, temperatureExcitation.T) annotation(
              Line(points = {{-50, 1}, {-50, 18}}, color = {0, 0, 127}, smooth = Smooth.None));
            connect(TExcitation, temperatureExcitation.T) annotation(
              Line(points = {{0, -100}, {0, -60}, {-60, -60}, {-60, 8}, {-50, 8}, {-50, 18}}, color = {0, 0, 127}, smooth = Smooth.None));
            connect(temperatureExcitation.port, thermalPort.heatPortExcitation) annotation(
              Line(points = {{-50, 40}, {-50, 100}, {0, 100}}, color = {191, 0, 0}, smooth = Smooth.None));
            connect(temperatureBrush.port, thermalPort.heatPortBrush) annotation(
              Line(points = {{0, 60}, {0, 100}, {0, 100}}, color = {191, 0, 0}, smooth = Smooth.None));
            annotation(
              Icon(graphics = {Text(extent = {{-100, -20}, {100, -80}}, lineColor = {0, 0, 0}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid, textString = "SMEE")}),
              Documentation(info = "<HTML>
Thermal ambient for synchronous induction machines with electrical excitation to prescribe winding temperatures either constant or via signal connectors.
Additionally, all losses = heat flows are recorded.
</HTML>"));
          end ThermalAmbientSMEE;

          model ThermalAmbientSMR "Thermal ambient for synchronous induction machine with reluctance rotor"
            parameter Boolean useDamperCage(start = true) "Enable / disable damper cage" annotation(
              Evaluate = true);
            extends Machines.Interfaces.InductionMachines.PartialThermalAmbientInductionMachines(redeclare final Machines.Interfaces.InductionMachines.ThermalPortSMR thermalPort(final useDamperCage = useDamperCage));
            parameter Modelica.SIunits.Temperature Tr(start = TDefault) "Temperature of damper cage (optional)" annotation(
              Dialog(enable = not useTemperatureInputs and useDamperCage));
            output Modelica.SIunits.HeatFlowRate Q_flowRotorWinding = temperatureRotorWinding.port.Q_flow "Heat flow rate of damper cage (optional))";
            output Modelica.SIunits.HeatFlowRate Q_flowTotal = Q_flowStatorWinding + Q_flowRotorWinding + Q_flowStatorCore + Q_flowRotorCore + Q_flowStrayLoad + Q_flowFriction;
            Modelica.Thermal.HeatTransfer.Sources.PrescribedTemperature temperatureRotorWinding annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 90, origin = {-20, 30})));
            Modelica.Blocks.Interfaces.RealInput TRotorWinding if useTemperatureInputs and useDamperCage "Temperature of damper cage (optional)" annotation(
              Placement(transformation(extent = {{-20, -20}, {20, 20}}, rotation = 90, origin = {80, -100})));
            Modelica.Blocks.Sources.Constant constTr(final k = if useDamperCage then Tr else TDefault) if not useTemperatureInputs or not useDamperCage annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 90, origin = {-20, -10})));
          equation
            connect(constTr.y, temperatureRotorWinding.T) annotation(
              Line(points = {{-20, 1}, {-20, 18}}, color = {0, 0, 127}, smooth = Smooth.None));
            connect(temperatureRotorWinding.port, thermalPort.heatPortRotorWinding) annotation(
              Line(points = {{-20, 40}, {-20, 100}, {0, 100}}, color = {191, 0, 0}, smooth = Smooth.None));
            connect(TRotorWinding, temperatureRotorWinding.T) annotation(
              Line(points = {{80, -100}, {80, 10}, {-20, 10}, {-20, 18}}, color = {0, 0, 127}, smooth = Smooth.None));
            annotation(
              Icon(graphics = {Text(extent = {{-100, -20}, {100, -80}}, lineColor = {0, 0, 0}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid, textString = "SMR")}),
              Documentation(info = "<HTML>
Thermal ambient for synchronous induction machines with reluctance rotor to prescribe winding temperatures either constant or via signal connectors.
Additionally, all losses = heat flows are recorded.
</HTML>"));
          end ThermalAmbientSMR;
          annotation(
            Documentation(info = "<HTML>
Thermal parts for synchronous induction machines
</HTML>"));
        end SynchronousInductionMachines;

        package DCMachines "Thermal parts of DC machines"
          extends Modelica.Icons.VariantsPackage;

          model ThermalAmbientDCPM "Thermal ambient for DC machine with permanent magnets"
            import Modelica;
            extends Machines.Interfaces.DCMachines.PartialThermalAmbientDCMachines(redeclare final Machines.Interfaces.DCMachines.ThermalPortDCPM thermalPort);
            parameter Modelica.SIunits.Temperature Tpm(start = TDefault) "Temperature of permanent magnet" annotation(
              Dialog(enable = not useTemperatureInputs));
            output Modelica.SIunits.HeatFlowRate Q_flowPermanentMagnet = temperaturePermanentMagnet.port.Q_flow "Heat flow rate of permanent magnets";
            output Modelica.SIunits.HeatFlowRate Q_flowTotal = Q_flowArmature + Q_flowCore + Q_flowStrayLoad + Q_flowFriction + Q_flowBrush + Q_flowPermanentMagnet;
            Modelica.Thermal.HeatTransfer.Sources.PrescribedTemperature temperaturePermanentMagnet annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 90, origin = {-20, 30})));
            Modelica.Blocks.Sources.Constant constTpm(final k = Tpm) if not useTemperatureInputs annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 90, origin = {-20, -10})));
            Modelica.Blocks.Interfaces.RealInput TPermanentMagnet if useTemperatureInputs "Temperature of permanent magnet" annotation(
              Placement(transformation(extent = {{-20, -20}, {20, 20}}, rotation = 90, origin = {0, -100})));
          equation
            connect(temperaturePermanentMagnet.port, thermalPort.heatPortPermanentMagnet) annotation(
              Line(points = {{-20, 40}, {-20, 100}, {0, 100}}, color = {191, 0, 0}, smooth = Smooth.None));
            connect(constTpm.y, temperaturePermanentMagnet.T) annotation(
              Line(points = {{-20, 1}, {-20, 18}}, color = {0, 0, 127}, smooth = Smooth.None));
            connect(TPermanentMagnet, temperaturePermanentMagnet.T) annotation(
              Line(points = {{0, -100}, {0, -60}, {-40, -60}, {-40, 8}, {-20, 8}, {-20, 18}}, color = {0, 0, 127}, smooth = Smooth.None));
            annotation(
              Icon(graphics = {Text(extent = {{-100, -20}, {100, -80}}, lineColor = {0, 0, 0}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid, textString = "DCPM")}),
              Documentation(info = "<HTML>
Thermal ambient for DC machines with permanent magnets to prescribe winding temperatures either constant or via signal connectors.
Additionally, all losses = heat flows are recorded.
</HTML>"));
          end ThermalAmbientDCPM;

          model ThermalAmbientDCEE "Thermal ambient for DC machine with electrical excitation"
            extends Machines.Interfaces.DCMachines.PartialThermalAmbientDCMachines(redeclare final Machines.Interfaces.DCMachines.ThermalPortDCEE thermalPort);
            parameter Modelica.SIunits.Temperature Te(start = TDefault) "Temperature of (shunt) excitation" annotation(
              Dialog(enable = not useTemperatureInputs));
            output Modelica.SIunits.HeatFlowRate Q_flowExcitation = temperatureExcitation.port.Q_flow "Heat flow rate of (shunt) excitation";
            output Modelica.SIunits.HeatFlowRate Q_flowTotal = Q_flowArmature + Q_flowCore + Q_flowStrayLoad + Q_flowFriction + Q_flowBrush + Q_flowExcitation;
            Modelica.Thermal.HeatTransfer.Sources.PrescribedTemperature temperatureExcitation annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 90, origin = {-20, 30})));
            Modelica.Blocks.Sources.Constant constTe(final k = Te) if not useTemperatureInputs annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 90, origin = {-20, -10})));
            Modelica.Blocks.Interfaces.RealInput TExcitation if useTemperatureInputs "Temperature of (shunt) excitation" annotation(
              Placement(transformation(extent = {{-20, -20}, {20, 20}}, rotation = 90, origin = {0, -100})));
          equation
            connect(constTe.y, temperatureExcitation.T) annotation(
              Line(points = {{-20, 1}, {-20, 18}}, color = {0, 0, 127}, smooth = Smooth.None));
            connect(TExcitation, temperatureExcitation.T) annotation(
              Line(points = {{0, -100}, {0, -60}, {-40, -60}, {-40, 8}, {-20, 8}, {-20, 18}}, color = {0, 0, 127}, smooth = Smooth.None));
            connect(temperatureExcitation.port, thermalPort.heatPortExcitation) annotation(
              Line(points = {{-20, 40}, {-20, 100}, {0, 100}}, color = {191, 0, 0}, smooth = Smooth.None));
            annotation(
              Icon(graphics = {Text(extent = {{-100, -20}, {100, -80}}, lineColor = {0, 0, 0}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid, textString = "DCEE")}),
              Documentation(info = "<HTML>
Thermal ambient for DC machines with electrical (shunt) excitation to prescribe winding temperatures either constant or via signal connectors.
Additionally, all losses = heat flows are recorded.
</HTML>"));
          end ThermalAmbientDCEE;

          model ThermalAmbientDCSE "Thermal ambient for DC machine with series excitation"
            extends Machines.Interfaces.DCMachines.PartialThermalAmbientDCMachines(redeclare final Machines.Interfaces.DCMachines.ThermalPortDCSE thermalPort);
            parameter Modelica.SIunits.Temperature Tse(start = TDefault) "Temperature of series excitation" annotation(
              Dialog(enable = not useTemperatureInputs));
            output Modelica.SIunits.HeatFlowRate Q_flowSeriesExcitation = temperatureSeriesExcitation.port.Q_flow "Heat flow rate of series excitation";
            output Modelica.SIunits.HeatFlowRate Q_flowTotal = Q_flowArmature + Q_flowCore + Q_flowStrayLoad + Q_flowFriction + Q_flowBrush + Q_flowSeriesExcitation;
            Modelica.Thermal.HeatTransfer.Sources.PrescribedTemperature temperatureSeriesExcitation annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 90, origin = {-50, 30})));
            Modelica.Blocks.Sources.Constant constTse(final k = Tse) if not useTemperatureInputs annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 90, origin = {-50, -10})));
            Modelica.Blocks.Interfaces.RealInput T_se if useTemperatureInputs "Temperature of series excitation" annotation(
              Placement(transformation(extent = {{-20, -20}, {20, 20}}, rotation = 90, origin = {-40, -100})));
          equation
            connect(constTse.y, temperatureSeriesExcitation.T) annotation(
              Line(points = {{-50, 1}, {-50, 18}}, color = {0, 0, 127}, smooth = Smooth.None));
            connect(T_se, temperatureSeriesExcitation.T) annotation(
              Line(points = {{-40, -100}, {-40, -60}, {-70, -60}, {-70, 8}, {-50, 8}, {-50, 18}}, color = {0, 0, 127}, smooth = Smooth.None));
            connect(temperatureSeriesExcitation.port, thermalPort.heatPortSeriesExcitation) annotation(
              Line(points = {{-50, 40}, {-50, 100}, {0, 100}}, color = {191, 0, 0}, smooth = Smooth.None));
            annotation(
              Icon(graphics = {Text(extent = {{-100, -20}, {100, -80}}, lineColor = {0, 0, 0}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid, textString = "DCSE")}),
              Documentation(info = "<HTML>
Thermal ambient for DC machines with serial excitation to prescribe winding temperatures either constant or via signal connectors.
Additionally, all losses = heat flows are recorded.
</HTML>"));
          end ThermalAmbientDCSE;

          model ThermalAmbientDCCE "Thermal ambient for DC machine with compound excitation"
            extends Machines.Interfaces.DCMachines.PartialThermalAmbientDCMachines(redeclare final Machines.Interfaces.DCMachines.ThermalPortDCCE thermalPort);
            parameter Modelica.SIunits.Temperature Te(start = TDefault) "Temperature of (shunt) excitation" annotation(
              Dialog(enable = not useTemperatureInputs));
            parameter Modelica.SIunits.Temperature Tse(start = TDefault) "Temperature of series excitation" annotation(
              Dialog(enable = not useTemperatureInputs));
            output Modelica.SIunits.HeatFlowRate Q_flowShuntExcitation = temperatureShuntExcitation.port.Q_flow "Heat flow rate of (shunt) excitation";
            output Modelica.SIunits.HeatFlowRate Q_flowSeriesExcitation = temperatureSeriesExcitation.port.Q_flow "Heat flow rate of series excitation";
            output Modelica.SIunits.HeatFlowRate Q_flowTotal = Q_flowArmature + Q_flowCore + Q_flowStrayLoad + Q_flowFriction + Q_flowBrush + Q_flowShuntExcitation + Q_flowSeriesExcitation;
            Modelica.Thermal.HeatTransfer.Sources.PrescribedTemperature temperatureShuntExcitation annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 90, origin = {-20, 30})));
            Modelica.Blocks.Sources.Constant constTe(final k = Te) if not useTemperatureInputs annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 90, origin = {-20, -10})));
            Modelica.Blocks.Interfaces.RealInput T_e if useTemperatureInputs "Temperature of (shunt) excitation" annotation(
              Placement(transformation(extent = {{-20, -20}, {20, 20}}, rotation = 90, origin = {0, -100})));
            Modelica.Thermal.HeatTransfer.Sources.PrescribedTemperature temperatureSeriesExcitation annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 90, origin = {-50, 30})));
            Modelica.Blocks.Sources.Constant constTse(final k = Tse) if not useTemperatureInputs annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 90, origin = {-50, -10})));
            Modelica.Blocks.Interfaces.RealInput T_se if useTemperatureInputs "Temperature of series excitation" annotation(
              Placement(transformation(extent = {{-20, -20}, {20, 20}}, rotation = 90, origin = {-40, -100})));
          equation
            connect(constTe.y, temperatureShuntExcitation.T) annotation(
              Line(points = {{-20, 1}, {-20, 18}}, color = {0, 0, 127}, smooth = Smooth.None));
            connect(T_e, temperatureShuntExcitation.T) annotation(
              Line(points = {{0, -100}, {0, -60}, {-40, -60}, {-40, 8}, {-20, 8}, {-20, 18}}, color = {0, 0, 127}, smooth = Smooth.None));
            connect(constTse.y, temperatureSeriesExcitation.T) annotation(
              Line(points = {{-50, 1}, {-50, 18}}, color = {0, 0, 127}, smooth = Smooth.None));
            connect(T_se, temperatureSeriesExcitation.T) annotation(
              Line(points = {{-40, -100}, {-40, -60}, {-70, -60}, {-70, 8}, {-50, 8}, {-50, 18}}, color = {0, 0, 127}, smooth = Smooth.None));
            connect(temperatureSeriesExcitation.port, thermalPort.heatPortSeriesExcitation) annotation(
              Line(points = {{-50, 40}, {-50, 100}, {0, 100}}, color = {191, 0, 0}, smooth = Smooth.None));
            connect(temperatureShuntExcitation.port, thermalPort.heatPortShuntExcitation) annotation(
              Line(points = {{-20, 40}, {-20, 100}, {0, 100}}, color = {191, 0, 0}, smooth = Smooth.None));
            annotation(
              Icon(graphics = {Text(extent = {{-100, -20}, {100, -80}}, lineColor = {0, 0, 0}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid, textString = "DCCE")}),
              Documentation(info = "<HTML>
Thermal ambient for DC machines with compound excitation to prescribe winding temperatures either constant or via signal connectors.
Additionally, all losses = heat flows are recorded.
</HTML>"));
          end ThermalAmbientDCCE;
          annotation(
            Documentation(info = "<HTML>
Thermal parts for DC machines
</HTML>"));
        end DCMachines;

        model ThermalAmbientTransformer "Thermal ambient for transformers"
          parameter Integer m(min = 1) = 3 "Number of phases";
          parameter Boolean useTemperatureInputs = false "If true, temperature inputs are used; else, temperatures are constant" annotation(
            Evaluate = true);
          constant Modelica.SIunits.Temperature TDefault = 293.15 "Default temperature";
          parameter Modelica.SIunits.Temperature T1(start = TDefault) "Temperature of primary windings" annotation(
            Dialog(enable = not useTemperatureInputs));
          parameter Modelica.SIunits.Temperature T2(start = TDefault) "Temperature of secondary windings" annotation(
            Dialog(enable = not useTemperatureInputs));
          output Modelica.SIunits.HeatFlowRate Q_flow1 = temperature1.port.Q_flow "Heat flow rate of primary windings";
          output Modelica.SIunits.HeatFlowRate Q_flow2 = temperature2.port.Q_flow "Heat flow rate of secondary windings";
          output Modelica.SIunits.HeatFlowRate Q_flowCore = temperatureCore.port.Q_flow "Heat flow rate of core losses";
          output Modelica.SIunits.HeatFlowRate Q_flowTotal = Q_flow1 + Q_flow2 + Q_flowCore;
          Machines.Interfaces.ThermalPortTransformer thermalPort(final m = m) annotation(
            Placement(transformation(extent = {{-10, 90}, {10, 110}})));
          Modelica.Thermal.HeatTransfer.Sources.PrescribedTemperature temperature1 annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 90, origin = {-80, 30})));
          Modelica.Blocks.Sources.Constant constT1(final k = T1) if not useTemperatureInputs annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 90, origin = {-80, -10})));
          Modelica.Blocks.Interfaces.RealInput TPrimary if useTemperatureInputs "Temperature of primary windings" annotation(
            Placement(transformation(extent = {{-20, -20}, {20, 20}}, rotation = 90, origin = {-80, -100})));
          Modelica.Thermal.HeatTransfer.Sources.PrescribedTemperature temperature2 annotation(
            Placement(transformation(extent = {{-10, 10}, {10, -10}}, rotation = 90, origin = {80, 30})));
          Modelica.Blocks.Sources.Constant constT2(final k = T2) if not useTemperatureInputs annotation(
            Placement(transformation(extent = {{-10, 10}, {10, -10}}, rotation = 90, origin = {80, -10})));
          Modelica.Blocks.Interfaces.RealInput TSecondary if useTemperatureInputs "Temperature of secondary windings" annotation(
            Placement(transformation(extent = {{-20, 20}, {20, -20}}, rotation = 90, origin = {80, -100})));
          Modelica.Thermal.HeatTransfer.Sources.FixedTemperature temperatureCore(final T = TDefault) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 90, origin = {0, 30})));
          Modelica.Thermal.HeatTransfer.Components.ThermalCollector thermalCollector1(final m = m) annotation(
            Placement(transformation(extent = {{-90, 60}, {-70, 80}})));
          Modelica.Thermal.HeatTransfer.Components.ThermalCollector thermalCollector2(final m = m) annotation(
            Placement(transformation(extent = {{70, 60}, {90, 80}})));
        equation
          connect(TPrimary, temperature1.T) annotation(
            Line(points = {{-80, -100}, {-80, -60}, {-100, -60}, {-100, 8}, {-80, 8}, {-80, 18}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(constT1.y, temperature1.T) annotation(
            Line(points = {{-80, 1}, {-80, 18}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(TSecondary, temperature2.T) annotation(
            Line(points = {{80, -100}, {80, -60}, {100, -60}, {100, 8}, {80, 8}, {80, 18}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(constT2.y, temperature2.T) annotation(
            Line(points = {{80, 1}, {80, 18}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(temperatureCore.port, thermalPort.heatPortCore) annotation(
            Line(points = {{0, 40}, {0, 100}}, color = {191, 0, 0}, smooth = Smooth.None));
          connect(thermalCollector1.port_b, temperature1.port) annotation(
            Line(points = {{-80, 60}, {-80, 40}}, color = {191, 0, 0}, smooth = Smooth.None));
          connect(thermalCollector2.port_b, temperature2.port) annotation(
            Line(points = {{80, 60}, {80, 40}}, color = {191, 0, 0}, smooth = Smooth.None));
          connect(thermalCollector1.port_a, thermalPort.heatPort1) annotation(
            Line(points = {{-80, 80}, {-80, 100}, {0, 100}}, color = {191, 0, 0}, smooth = Smooth.None));
          connect(thermalCollector2.port_a, thermalPort.heatPort2) annotation(
            Line(points = {{80, 80}, {80, 100}, {0, 100}}, color = {191, 0, 0}, smooth = Smooth.None));
          annotation(
            Icon(graphics = {Rectangle(extent = {{-100, 100}, {100, -100}}, lineColor = {0, 0, 0}, pattern = LinePattern.None, fillColor = {159, 159, 223}, fillPattern = FillPattern.Backward), Line(points = {{-14, 0}, {54, 0}}, color = {191, 0, 0}, thickness = 0.5, origin = {0, -6}, rotation = 90), Polygon(points = {{-20, -20}, {-20, 20}, {20, 0}, {-20, -20}}, lineColor = {191, 0, 0}, fillColor = {191, 0, 0}, fillPattern = FillPattern.Solid, origin = {0, 68}, rotation = 90), Text(extent = {{-100, -20}, {100, -80}}, lineColor = {0, 0, 0}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid, textString = "Trafo")}),
            Documentation(info = "<HTML>
Thermal ambient for transformers to prescribe winding temperatures either constant or via signal connectors.
Additionally, all losses = heat flows are recorded.
</HTML>"));
        end ThermalAmbientTransformer;
        annotation(
          Icon(graphics = {Ellipse(extent = {{-65, -63}, {65, 63}}, lineColor = {191, 0, 0}, fillColor = {191, 0, 0}, fillPattern = FillPattern.Solid)}),
          Documentation(info = "<HTML>
<h4>Thermal concept</h4>
<p>
Each machine model is equipped with a machine-specific conditional <code>thermalPort</code>.
If <code>useThermalPort == false</code>, a machine-specific thermal ambient prescribing constant temperatures is used inside the machine.
If <code>useThermalPort == true</code>, a thermal model or machine-specific thermal ambient prescribing the temperatures has to be connected from outside.
On the other hand, all losses are dissipated to this internal or external thermal ambient.
</p>
<p>
The machine specific thermal connector contains <a href=\"modelica://Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a\">heatPort</a>s
for all relevant loss sources of the machine type, although some of the loss sources are not yet implemented;
these heatPorts are left unconnected inside the machine, i.e., the HeatFlowRate is zero,
but they have to be connected to a constant temperature source in the internal or external thermal ambient.
Simple machine-specific thermal ambients for constant temperatures (<code>useTemperatureInputs == false</code>)
or temperatures prescribed via signal inputs (<code>useTemperatureInputs == true</code>) are provided in this package.
</p>
<h4>Loss sources</h4>
<p>
Up to now, only Ohmic losses in stator and rotor windings are implemented.
They are modeled as <a href=\"modelica://Modelica.Electrical.Analog.Basic.Resistor\">linearly temperature dependent resistors</a>:
</p>
<pre>
   ROperational = RRef * (1 + alphaRef * (TOperational - TRef))
</pre>
<h5>Parameters:</h5>
<ul>
<li>Resistance <code>RRef</code> at reference temperature</li>
<li>Reference temperature <code>TRef</code></li>
<li>Linear temperature coefficient <code>alpha20</code> at 20&deg;C</li>
<li>Operational temperature <code>TOperational</code>
(if <code>useThermalPort == false</code>; otherwise, the operational temperature is provided via the heatPort)</li>
<li>Nominal temperature <code>TNominal</code>
(required for DC machines to calculate the turns ratio)</li>
</ul>
<p>
The linear temperature coefficient <code>alpha20</code> at 20&deg;C = 293.15 K has to be converted to reference temperature <code>TRef</code>:
</p>
<pre>
                        alpha20
  alphaRef = -------------------------------
              1 + alpha20 * (TRef - 293.15)
</pre>
<p>
For this reason, the function <a href=\"modelica://Modelica.Electrical.Machines.Thermal.convertAlpha\">convertAlpha</a> is provided.
In sub-package <a href=\"modelica://Modelica.Electrical.Machines.Thermal.Constants\">Constants</a> linear temperature coefficients at 20&deg;C for commonly used materials are defined.
</p>
<h4>Backwards compatibility</h4>
<ul>
<li>The default / start values of all resistances are left unchanged.</li>
<li>The default / start values of all reference temperatures are set to 20&deg;C.</li>
<li>The default / start values of all linear temperature coefficients are set to 0.</li>
<li>The default / start values of all operational temperatures are set to 20&deg;C.</li>
<li>The default / start values of all nominal temperatures are set to 20&deg;C.</li>
</ul>
<h4>Machine specific thermalPorts</h4>
<h5><a href=\"modelica://Modelica.Electrical.Machines.BasicMachines.AsynchronousInductionMachines.AIM_SquirrelCage\">Asynchronous induction machine with squirrel cage</a></h5>
<ul>
<li><code>heatPortStatorWinding[m]</code>: m=3 heatPorts for the m=3 stator phases</li>
<li><code>heatPortRotorWinding</code>: heatPort for the rotor cage</li>
<li><code>heatPortStatorCore</code>: stator core losses (not yet fully implemented)</li>
<li><code>heatPortRotorCore</code>: rotor core losses (not yet connected/implemented)</li>
<li><code>heatPortStrayLoad</code>: stray load losses</li>
<li><code>heatPortFriction</code>: friction losses</li>
</ul>
<h5><a href=\"modelica://Modelica.Electrical.Machines.BasicMachines.AsynchronousInductionMachines.AIM_SlipRing\">Asynchronous induction machine with slipring rotor</a></h5>
<ul>
<li><code>heatPortStatorWinding[m]</code>: m=3 heatPorts for the m=3 stator phases</li>
<li><code>heatPortRotorWinding[m]</code>: m=3 heatPorts for the m=3 rotor phases</li>
<li><code>heatPortBrush</code>: brush losses (not yet connected/implemented)</li>
<li><code>heatPortStatorCore</code>: stator core losses (not yet fully implemented)</li>
<li><code>heatPortRotorCore</code>: rotor core losses (not yet fully implemented)</li>
<li><code>heatPortStrayLoad</code>: stray load losses</li>
<li><code>heatPortFriction</code>: friction losses</li>
</ul>
<h5><a href=\"modelica://Modelica.Electrical.Machines.BasicMachines.SynchronousInductionMachines.SM_PermanentMagnet\">Synchronous induction machine with permanent magnets</a></h5>
<ul>
<li><code>heatPortStatorWinding[m]</code>: m=3 heatPorts for the m=3 stator phases</li>
<li><code>heatPortRotorWinding</code>: conditional (<code>useDamperCage=true/false</code>) heatPort for the damper cage</li>
<li><code>heatPortPermanentMagnet</code>: permanent magnet losses (not yet connected/implemented)</li>
<li><code>heatPortStatorCore</code>: stator core losses (not yet fully implemented)</li>
<li><code>heatPortRotorCore</code>: rotor core losses (not yet connected/implemented)</li>
<li><code>heatPortStrayLoad</code>: stray load losses</li>
<li><code>heatPortFriction</code>: friction losses</li>
</ul>
<h5><a href=\"modelica://Modelica.Electrical.Machines.BasicMachines.SynchronousInductionMachines.SM_ElectricalExcited\">Synchronous induction machine with electrical excitation</a></h5>
<ul>
<li><code>heatPortStatorWinding[m]</code>: m=3 heatPorts for the m=3 stator phases</li>
<li><code>heatPortRotorWinding</code>: conditional (<code>useDamperCage=true/false</code>) heatPort for the damper cage</li>
<li><code>heatPortExcitation</code>: electrical excitation</li>
<li><code>heatPortBrush</code>: brush losses</li>
<li><code>heatPortStatorCore</code>: stator core losses (not yet fully implemented)</li>
<li><code>heatPortRotorCore</code>: rotor core losses (not yet connected/implemented)</li>
<li><code>heatPortStrayLoad</code>: stray load losses</li>
<li><code>heatPortFriction</code>: friction losses</li>
</ul>
<h5><a href=\"modelica://Modelica.Electrical.Machines.BasicMachines.SynchronousInductionMachines.SM_ReluctanceRotor\">Synchronous induction machine with reluctance rotor</a></h5>
<ul>
<li><code>heatPortStatorWinding[m]</code>: m=3 heatPorts for the m=3 stator phases</li>
<li><code>heatPortRotorWinding</code>: conditional (<code>useDamperCage=true/false</code>) heatPort for the damper cage</li>
<li><code>heatPortStatorCore</code>: stator core losses (not yet fully implemented)</li>
<li><code>heatPortRotorCore</code>: rotor core losses (not yet connected/implemented)</li>
<li><code>heatPortStrayLoad</code>: stray load losses</li>
<li><code>heatPortFriction</code>: friction losses</li>
</ul>
<h5><a href=\"modelica://Modelica.Electrical.Machines.BasicMachines.DCMachines.DC_PermanentMagnet\">DC machine with permanent magnets</a></h5>
<ul>
<li><code>heatPortArmature</code>: armature losses</li>
<li><code>heatPortPermanentMagnet</code>: permanent magnet losses (not yet connected/implemented)</li>
<li><code>heatPortBrush</code>: brush losses</li>
<li><code>heatPortCore</code>: armature core losses</li>
<li><code>heatPortStrayLoad</code>: stray load losses</li>
<li><code>heatPortFriction</code>: friction losses</li>
</ul>
<h5><a href=\"modelica://Modelica.Electrical.Machines.BasicMachines.DCMachines.DC_ElectricalExcited\">DC machine with electrical (shunt) excitation</a></h5>
<ul>
<li><code>heatPortArmature</code>: armature losses</li>
<li><code>heatPortExcitation</code>: electrical (shunt) excitation</li>
<li><code>heatPortBrush</code>: brush losses</li>
<li><code>heatPortCore</code>: armature core losses</li>
<li><code>heatPortStrayLoad</code>: stray load losses</li>
<li><code>heatPortFriction</code>: friction losses</li>
</ul>
<h5><a href=\"modelica://Modelica.Electrical.Machines.BasicMachines.DCMachines.DC_SeriesExcited\">DC machine with serial excitation</a></h5>
<ul>
<li><code>heatPortArmature</code>: armature losses</li>
<li><code>heatPortSeriesExcitation</code>: electrical series excitation</li>
<li><code>heatPortBrush</code>: brush losses</li>
<li><code>heatPortCore</code>: armature core losses</li>
<li><code>heatPortStrayLoad</code>: stray load losses</li>
<li><code>heatPortFriction</code>: friction losses</li>
</ul>
<h5><!--<a href=\"modelica://Modelica.Electrical.Machines.BasicMachines.DCMachines.DC_Compound\">-->DC machine with compound excitation (not yet implemented)<!--</a>--></h5>
<ul>
<li><code>heatPortArmature</code>: armature losses</li>
<li><code>heatPortShuntExcitation</code>: electrical (shunt) excitation</li>
<li><code>heatPortSeriesExcitation</code>: electrical series excitation</li>
<li><code>heatPortBrush</code>: brush losses</li>
<li><code>heatPortCore</code>: armature core losses</li>
<li><code>heatPortStrayLoad</code>: stray load losses</li>
<li><code>heatPortFriction</code>: friction losses</li>
</ul>
<h5><a href=\"modelica://Modelica.Electrical.Machines.BasicMachines.Transformers\">Transformers</a></h5>
<ul>
<li><code>heatPort1[m]</code>: m=3 heatPorts for the m=3 primary phases</li>
<li><code>heatPort2[m]</code>: m=3 heatPorts for the m=3 secondary phases</li>
<li><code>heatPortCore</code>: iron core losses (not yet connected/implemented)</li>
</ul>
</HTML>", revisions = "<HTML>
  <ul>
  <li> v2.2.0 2011/02/10 Anton Haumer<br>
       first stable release of this subpackage:<br>
       conditional ThermalPort for all machines</li>
  </ul>
</HTML>"));
      end Thermal;

      package Interfaces "SpacePhasor connector and PartialMachines"
        extends Modelica.Icons.InterfacesPackage;

        connector SpacePhasor "Connector for Space Phasors"
          Modelica.SIunits.Voltage v_[2] "1=real, 2=imaginary part";
          flow Modelica.SIunits.Current i_[2] "1=real, 2=imaginary part";
          annotation(
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{0, 100}, {-100, 0}, {0, -100}, {100, 0}, {0, 100}}, lineColor = {0, 0, 255}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-150, -90}, {150, -150}}, lineColor = {0, 0, 255}, textString = "%name")}),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{0, 100}, {-100, 0}, {0, -100}, {100, 0}, {0, 100}}, lineColor = {0, 0, 255}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid)}),
            Documentation(info = "<HTML>
Connector for Space Phasors:
<ul>
<li>Voltage v_[2] ... Real and Imaginary part of voltage space phasor</li>
<li>Current i_[2] ... Real and Imaginary part of current space phasor</li>
</ul>
</HTML>"));
        end SpacePhasor;

        partial model PartialBasicMachine "Partial model for all machines"
          extends Machines.Icons.TransientMachine;
          constant Modelica.SIunits.Angle pi = Modelica.Constants.pi;
          parameter Modelica.SIunits.Inertia Jr "Rotor's moment of inertia";
          parameter Boolean useSupport = false "Enable / disable (=fixed stator) support" annotation(
            Evaluate = true);
          parameter Modelica.SIunits.Inertia Js(start = Jr) "Stator's moment of inertia" annotation(
            Dialog(enable = useSupport));
          parameter Boolean useThermalPort = false "Enable / disable (=fixed temperatures) thermal port" annotation(
            Evaluate = true);
          parameter Machines.Losses.FrictionParameters frictionParameters "Friction loss parameter record" annotation(
            Dialog(tab = "Losses"));
          output Modelica.SIunits.Angle phiMechanical(start = 0) = flange.phi - internalSupport.phi "Mechanical angle of rotor against stator";
          output Modelica.SIunits.AngularVelocity wMechanical(displayUnit = "1/min", start = 0) = der(phiMechanical) "Mechanical angular velocity of rotor against stator";
          output Modelica.SIunits.Torque tauElectrical = inertiaRotor.flange_a.tau "Electromagnetic torque";
          output Modelica.SIunits.Torque tauShaft = -flange.tau "Shaft torque";
          Modelica.Mechanics.Rotational.Interfaces.Flange_a flange "Shaft" annotation(
            Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
          Modelica.Mechanics.Rotational.Components.Inertia inertiaRotor(final J = Jr) annotation(
            Placement(transformation(origin = {80, 0}, extent = {{10, 10}, {-10, -10}}, rotation = 180)));
          Modelica.Mechanics.Rotational.Interfaces.Flange_a support if useSupport "Support at which the reaction torque is acting" annotation(
            Placement(transformation(extent = {{90, -110}, {110, -90}}, rotation = 0)));
          Modelica.Mechanics.Rotational.Components.Inertia inertiaStator(final J = Js) annotation(
            Placement(transformation(origin = {80, -100}, extent = {{10, 10}, {-10, -10}}, rotation = 180)));
          Modelica.Mechanics.Rotational.Components.Fixed fixed if not useSupport annotation(
            Placement(transformation(extent = {{10, 10}, {-10, -10}}, rotation = 180, origin = {50, -100})));
          Machines.Losses.Friction friction(final frictionParameters = frictionParameters) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0, origin = {90, -40})));
        protected
          Modelica.Mechanics.Rotational.Interfaces.Support internalSupport annotation(
            Placement(transformation(extent = {{56, -104}, {64, -96}}, rotation = 0)));
        equation
          connect(inertiaRotor.flange_b, flange) annotation(
            Line(points = {{90, 0}, {92, 0}, {92, 0}, {100, 0}}, color = {0, 0, 0}));
          connect(inertiaStator.flange_b, support) annotation(
            Line(points = {{90, -100}, {100, -100}}, color = {0, 0, 0}));
          connect(internalSupport, fixed.flange) annotation(
            Line(points = {{60, -100}, {50, -100}}, color = {0, 0, 0}, smooth = Smooth.None));
          connect(internalSupport, inertiaStator.flange_a) annotation(
            Line(points = {{60, -100}, {70, -100}}, color = {0, 0, 0}, smooth = Smooth.None));
          connect(inertiaRotor.flange_b, friction.flange) annotation(
            Line(points = {{90, 0}, {90, -30}}, color = {0, 0, 0}, smooth = Smooth.None));
          connect(friction.support, internalSupport) annotation(
            Line(points = {{90, -50}, {90, -90}, {60, -90}, {60, -100}}, color = {0, 0, 0}, smooth = Smooth.None));
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-150, -120}, {150, -180}}, lineColor = {0, 0, 255}, textString = "%name"), Rectangle(extent = {{80, -80}, {120, -120}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(visible = not useSupport, points = {{80, -100}, {120, -100}}, color = {0, 0, 0}, smooth = Smooth.None), Line(visible = not useSupport, points = {{90, -100}, {80, -120}}, color = {0, 0, 0}, smooth = Smooth.None), Line(visible = not useSupport, points = {{100, -100}, {90, -120}}, color = {0, 0, 0}, smooth = Smooth.None), Line(visible = not useSupport, points = {{110, -100}, {100, -120}}, color = {0, 0, 0}, smooth = Smooth.None), Line(visible = not useSupport, points = {{120, -100}, {110, -120}}, color = {0, 0, 0}, smooth = Smooth.None)}),
            Documentation(info = "<HTML>
Base partial model DC machines:
<ul>
<li>main parts of the icon</li>
<li>mechanical shaft</li>
<li>mechanical support</li>
</ul>
Besides the mechanical connector <i>flange</i> (i.e., the shaft) the machines have a second mechanical connector <i>support</i>.<br>
If <i>useSupport</i> = false, it is assumed that the stator is fixed.<br>
Otherwise reaction torque (i.e., airGap torque, minus acceleration torque for stator's moment of inertia) can be measured at <i>support</i>.<br>
One may also fix the the shaft and let rotate the stator; parameter Js is only of importance when the stator is rotating.
</HTML>"));
        end PartialBasicMachine;

        partial model PartialBasicInductionMachine "Partial model for induction machine"
          final parameter Integer m = 3 "Number of phases";
          parameter Integer p(min = 1, start = 2) "Number of pole pairs (Integer)";
          parameter Modelica.SIunits.Frequency fsNominal(start = 50) "Nominal frequency";
          parameter Modelica.SIunits.Temperature TsOperational(start = 293.15) "Operational temperature of stator resistance" annotation(
            Dialog(group = "Operational temperatures", enable = not useThermalPort));
          parameter Modelica.SIunits.Resistance Rs(start = 0.03) "Stator resistance per phase at TRef" annotation(
            Dialog(tab = "Nominal resistances and inductances"));
          parameter Modelica.SIunits.Temperature TsRef(start = 293.15) "Reference temperature of stator resistance" annotation(
            Dialog(tab = "Nominal resistances and inductances"));
          parameter Machines.Thermal.LinearTemperatureCoefficient20 alpha20s(start = 0) "Temperature coefficient of stator resistance at 20 degC" annotation(
            Dialog(tab = "Nominal resistances and inductances"));
          parameter Modelica.SIunits.Inductance Lszero = Lssigma "Stator zero sequence inductance" annotation(
            Dialog(tab = "Nominal resistances and inductances"));
          parameter Modelica.SIunits.Inductance Lssigma(start = 3 * (1 - sqrt(1 - 0.0667)) / (2 * pi * fsNominal)) "Stator stray inductance per phase" annotation(
            Dialog(tab = "Nominal resistances and inductances"));
          extends PartialBasicMachine(Jr(start = 0.29), frictionParameters(wRef = 2 * pi * fsNominal / p), friction(final useHeatPort = true));
          parameter Machines.Losses.CoreParameters statorCoreParameters(final m = 3, VRef(start = 100), wRef = 2 * pi * fsNominal) "Stator core loss parameter record; all parameters refer to stator side" annotation(
            Dialog(tab = "Losses"));
          parameter Machines.Losses.StrayLoadParameters strayLoadParameters(IRef(start = 100), wRef = 2 * pi * fsNominal / p) "Stray load loss parameter record" annotation(
            Dialog(tab = "Losses"));
          replaceable output Machines.Interfaces.InductionMachines.PartialPowerBalanceInductionMachines powerBalance(final powerStator = Machines.SpacePhasors.Functions.activePower(vs, is), final powerMechanical = wMechanical * tauShaft, final powerInertiaStator = inertiaStator.J * inertiaStator.a * inertiaStator.w, final powerInertiaRotor = inertiaRotor.J * inertiaRotor.a * inertiaRotor.w, final lossPowerStatorWinding = sum(rs.resistor.LossPower), final lossPowerStatorCore = statorCore.lossPower, final lossPowerStrayLoad = strayLoad.lossPower, final lossPowerFriction = friction.lossPower) "Power balance";
          output Modelica.SIunits.Voltage vs[m] = plug_sp.pin.v - plug_sn.pin.v "Stator instantaneous voltages";
          output Modelica.SIunits.Current is[m] = plug_sp.pin.i "Stator instantaneous currents";
          output Modelica.SIunits.Current i_0_s(stateSelect = StateSelect.prefer) = spacePhasorS.zero.i "Stator zero-sequence current";
          // to be defined in model that extends from this partial (since airgap isn't included here)
          input Modelica.SIunits.Current idq_ss[2] "Stator space phasor current / stator fixed frame";
          input Modelica.SIunits.Current idq_sr[2](each stateSelect = StateSelect.prefer) "Stator space phasor current / rotor fixed frame";
          input Modelica.SIunits.Current idq_rs[2] "Rotor space phasor current / stator fixed frame";
          input Modelica.SIunits.Current idq_rr[2](each stateSelect = StateSelect.prefer) "Rotor space phasor current / rotor fixed frame";
          Modelica.Electrical.MultiPhase.Interfaces.PositivePlug plug_sp(final m = m) "Positive stator plug" annotation(
            Placement(transformation(extent = {{50, 90}, {70, 110}}, rotation = 0)));
          Modelica.Electrical.MultiPhase.Interfaces.NegativePlug plug_sn(final m = m) "Negative stator plug" annotation(
            Placement(transformation(extent = {{-70, 90}, {-50, 110}}, rotation = 0)));
          Modelica.Electrical.MultiPhase.Basic.Resistor rs(final m = m, final R = fill(Rs, m), final T_ref = fill(TsRef, m), final alpha = fill(Machines.Thermal.convertAlpha(alpha20s, TsRef), m), final useHeatPort = true, final T = fill(TsRef, m)) annotation(
            Placement(transformation(extent = {{60, 70}, {40, 90}}, rotation = 0)));
          Machines.BasicMachines.Components.Inductor lssigma(final L = fill(Lssigma, 2)) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 270, origin = {20, 20})));
          Modelica.Electrical.Analog.Basic.Inductor lszero(final L = Lszero) annotation(
            Placement(transformation(extent = {{0, 40}, {-20, 60}})));
          Machines.Losses.InductionMachines.Core statorCore(final coreParameters = statorCoreParameters, final useHeatPort = true, final turnsRatio = 1) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 180, origin = {0, 30})));
          Machines.SpacePhasors.Components.SpacePhasor spacePhasorS(final turnsRatio = 1) annotation(
            Placement(transformation(origin = {0, 70}, extent = {{10, 10}, {-10, -10}}, rotation = 90)));
          Machines.Losses.InductionMachines.StrayLoad strayLoad(final strayLoadParameters = strayLoadParameters, final useHeatPort = true, final m = m) annotation(
            Placement(transformation(extent = {{90, 70}, {70, 90}})));
          replaceable Machines.Interfaces.InductionMachines.PartialThermalPortInductionMachines thermalPort(final m = m) if useThermalPort annotation(
            Placement(transformation(extent = {{-10, -110}, {10, -90}})));
          replaceable Machines.Interfaces.InductionMachines.PartialThermalAmbientInductionMachines thermalAmbient(final useTemperatureInputs = false, final Ts = TsOperational, final m = m) if not useThermalPort annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 270, origin = {-30, -80})));
        protected
          replaceable Machines.Interfaces.InductionMachines.PartialThermalPortInductionMachines internalThermalPort(final m = m) annotation(
            Placement(transformation(extent = {{-4, -84}, {4, -76}})));
        equation
          connect(spacePhasorS.plug_n, plug_sn) annotation(
            Line(points = {{-10, 80}, {-60, 80}, {-60, 100}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(thermalPort, internalThermalPort) annotation(
            Line(points = {{0, -100}, {0, -80}}, color = {199, 0, 0}, smooth = Smooth.None));
          connect(thermalAmbient.thermalPort, internalThermalPort) annotation(
            Line(points = {{-20, -80}, {0, -80}}, color = {199, 0, 0}, smooth = Smooth.None));
          connect(strayLoad.plug_n, rs.plug_p) annotation(
            Line(points = {{70, 80}, {60, 80}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(strayLoad.plug_p, plug_sp) annotation(
            Line(points = {{90, 80}, {90, 100}, {60, 100}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(strayLoad.support, internalSupport) annotation(
            Line(points = {{80, 70}, {80, 50}, {60, 50}, {60, -100}}, color = {0, 0, 0}, smooth = Smooth.None));
          connect(spacePhasorS.plug_p, rs.plug_n) annotation(
            Line(points = {{10, 80}, {40, 80}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(spacePhasorS.zero, lszero.p) annotation(
            Line(points = {{0, 60}, {0, 50}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(lszero.n, spacePhasorS.ground) annotation(
            Line(points = {{-20, 50}, {-20, 60}, {-10, 60}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(spacePhasorS.spacePhasor, lssigma.spacePhasor_a) annotation(
            Line(points = {{10, 60}, {20, 60}, {20, 30}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(statorCore.spacePhasor, lssigma.spacePhasor_a) annotation(
            Line(points = {{10, 30}, {20, 30}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(statorCore.heatPort, internalThermalPort.heatPortStatorCore) annotation(
            Line(points = {{10, 40}, {50, 40}, {50, -80}, {0, -80}}, color = {191, 0, 0}, smooth = Smooth.None));
          connect(strayLoad.heatPort, internalThermalPort.heatPortStrayLoad) annotation(
            Line(points = {{90, 70}, {90, 60}, {50, 60}, {50, -80}, {0, -80}}, color = {191, 0, 0}, smooth = Smooth.None));
          connect(rs.heatPort, internalThermalPort.heatPortStatorWinding) annotation(
            Line(points = {{50, 70}, {50, -80}, {0, -80}}, color = {191, 0, 0}, smooth = Smooth.None));
          connect(friction.heatPort, internalThermalPort.heatPortFriction) annotation(
            Line(points = {{80, -50}, {50, -50}, {50, -80}, {0, -80}}, color = {191, 0, 0}, smooth = Smooth.None));
          connect(strayLoad.flange, inertiaRotor.flange_b) annotation(
            Line(points = {{80, 90}, {100, 90}, {100, 40}, {90, 40}, {90, 0}}, color = {0, 0, 0}, smooth = Smooth.None));
          annotation(
            Documentation(info = "<HTML>
Partial model for induction machine models
</HTML>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-50, 100}, {-20, 100}, {-20, 70}}, color = {0, 0, 255}), Line(points = {{50, 100}, {20, 100}, {20, 70}}, color = {0, 0, 255})}));
        end PartialBasicInductionMachine;

        package InductionMachines "Interfaces and partial models for induction machines"
          extends Modelica.Icons.VariantsPackage;

          connector PartialThermalPortInductionMachines "Partial thermal port of induction machines"
            parameter Integer m = 3 "Number of stator phases";
            Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a heatPortStatorWinding[m] "Heat port of stator windings" annotation(
              Placement(transformation(extent = {{-20, 10}, {0, 30}})));
            Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a heatPortStatorCore "Heat port of (optional) stator core losses" annotation(
              Placement(transformation(extent = {{0, 10}, {20, 30}})));
            Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a heatPortRotorCore "Heat port of (optional) rotor core losses" annotation(
              Placement(transformation(extent = {{0, -30}, {20, -10}})));
            Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a heatPortStrayLoad "Heat port of (optional) stray losses" annotation(
              Placement(transformation(extent = {{0, -10}, {20, 10}})));
            Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a heatPortFriction "Heat port of (optional) friction losses" annotation(
              Placement(transformation(extent = {{-10, -50}, {10, -30}})));
            annotation(
              Diagram(graphics = {Rectangle(extent = {{-60, 60}, {60, -60}}, lineColor = {199, 0, 0}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-50, 50}, {50, -50}}, lineColor = {191, 0, 0}, fillColor = {191, 0, 0}, fillPattern = FillPattern.Solid)}),
              Icon(graphics = {Rectangle(extent = {{-110, 110}, {110, -110}}, lineColor = {199, 0, 0}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-80, 80}, {80, -80}}, lineColor = {191, 0, 0}, fillColor = {191, 0, 0}, fillPattern = FillPattern.Solid)}),
              Documentation(info = "<HTML>
Partial thermal port for induction machines
</HTML>"));
          end PartialThermalPortInductionMachines;

          model PartialThermalAmbientInductionMachines "Partial thermal ambient for induction machines"
            parameter Integer m = 3 "Number of stator phases";
            parameter Boolean useTemperatureInputs = false "If true, temperature inputs are used; else, temperatures are constant" annotation(
              Evaluate = true);
            constant Modelica.SIunits.Temperature TDefault = 293.15 "Default temperature";
            parameter Modelica.SIunits.Temperature Ts(start = TDefault) "Temperature of stator windings" annotation(
              Dialog(enable = not useTemperatureInputs));
            output Modelica.SIunits.HeatFlowRate Q_flowStatorWinding = temperatureStatorWinding.port.Q_flow "Heat flow rate of stator windings";
            output Modelica.SIunits.HeatFlowRate Q_flowStatorCore = temperatureStatorCore.port.Q_flow "Heat flow rate of stator core losses";
            output Modelica.SIunits.HeatFlowRate Q_flowRotorCore = temperatureRotorCore.port.Q_flow "Heat flow rate of stator core losses";
            output Modelica.SIunits.HeatFlowRate Q_flowStrayLoad = temperatureStrayLoad.port.Q_flow "Heat flow rate of stray load losses";
            output Modelica.SIunits.HeatFlowRate Q_flowFriction = temperatureFriction.port.Q_flow "Heat flow rate of friction losses";
            replaceable Machines.Interfaces.InductionMachines.PartialThermalPortInductionMachines thermalPort(final m = m) annotation(
              Placement(transformation(extent = {{-10, 90}, {10, 110}})));
            Modelica.Thermal.HeatTransfer.Sources.PrescribedTemperature temperatureStatorWinding annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 90, origin = {-80, 30})));
            Modelica.Thermal.HeatTransfer.Sources.FixedTemperature temperatureStatorCore(final T = TDefault) annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 90, origin = {20, 50})));
            Modelica.Thermal.HeatTransfer.Sources.FixedTemperature temperatureRotorCore(final T = TDefault) annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 90, origin = {40, 30})));
            Modelica.Thermal.HeatTransfer.Sources.FixedTemperature temperatureStrayLoad(final T = TDefault) annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 90, origin = {60, 50})));
            Modelica.Thermal.HeatTransfer.Sources.FixedTemperature temperatureFriction(final T = TDefault) annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 90, origin = {80, 30})));
            Modelica.Blocks.Interfaces.RealInput TStatorWinding if useTemperatureInputs "Temperature of stator windings" annotation(
              Placement(transformation(extent = {{-20, -20}, {20, 20}}, rotation = 90, origin = {-80, -100})));
            Modelica.Blocks.Sources.Constant constTs(final k = Ts) if not useTemperatureInputs annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 90, origin = {-80, -10})));
            Modelica.Thermal.HeatTransfer.Components.ThermalCollector thermalCollectorStator(final m = m) annotation(
              Placement(transformation(extent = {{-90, 60}, {-70, 80}})));
          equation
            connect(constTs.y, temperatureStatorWinding.T) annotation(
              Line(points = {{-80, 1}, {-80, 18}}, color = {0, 0, 127}, smooth = Smooth.None));
            connect(TStatorWinding, temperatureStatorWinding.T) annotation(
              Line(points = {{-80, -100}, {-80, -60}, {-100, -60}, {-100, 8}, {-80, 8}, {-80, 18}}, color = {0, 0, 127}, smooth = Smooth.None));
            connect(temperatureStrayLoad.port, thermalPort.heatPortStrayLoad) annotation(
              Line(points = {{60, 60}, {60, 100}, {0, 100}}, color = {191, 0, 0}, smooth = Smooth.None));
            connect(temperatureFriction.port, thermalPort.heatPortFriction) annotation(
              Line(points = {{80, 40}, {80, 100}, {0, 100}}, color = {191, 0, 0}, smooth = Smooth.None));
            connect(thermalCollectorStator.port_b, temperatureStatorWinding.port) annotation(
              Line(points = {{-80, 60}, {-80, 40}}, color = {191, 0, 0}, smooth = Smooth.None));
            connect(thermalCollectorStator.port_a, thermalPort.heatPortStatorWinding) annotation(
              Line(points = {{-80, 80}, {-80, 100}, {0, 100}}, color = {191, 0, 0}, smooth = Smooth.None));
            connect(temperatureStatorCore.port, thermalPort.heatPortStatorCore) annotation(
              Line(points = {{20, 60}, {20, 100}, {0, 100}}, color = {191, 0, 0}, smooth = Smooth.None));
            connect(temperatureRotorCore.port, thermalPort.heatPortRotorCore) annotation(
              Line(points = {{40, 40}, {40, 100}, {0, 100}}, color = {191, 0, 0}, smooth = Smooth.None));
            annotation(
              Icon(graphics = {Rectangle(extent = {{-100, 100}, {100, -100}}, lineColor = {0, 0, 0}, pattern = LinePattern.None, fillColor = {159, 159, 223}, fillPattern = FillPattern.Backward), Line(points = {{-14, 0}, {54, 0}}, color = {191, 0, 0}, thickness = 0.5, origin = {0, -6}, rotation = 90), Polygon(points = {{-20, -20}, {-20, 20}, {20, 0}, {-20, -20}}, lineColor = {191, 0, 0}, fillColor = {191, 0, 0}, fillPattern = FillPattern.Solid, origin = {0, 68}, rotation = 90)}),
              Documentation(info = "<HTML>
Partial thermal ambient for induction machines
</HTML>"));
          end PartialThermalAmbientInductionMachines;

          record PartialPowerBalanceInductionMachines "Partial power balance of induction machines"
            extends Modelica.Icons.Record;
            Modelica.SIunits.Power powerStator = 0 "Electrical power (stator)";
            Modelica.SIunits.Power powerMechanical = 0 "Mechanical power";
            Modelica.SIunits.Power powerInertiaStator = 0 "Stator inertia power";
            Modelica.SIunits.Power powerInertiaRotor = 0 "Rotor inertia power";
            Modelica.SIunits.Power lossPowerTotal = 0 "Total loss power";
            Modelica.SIunits.Power lossPowerStatorWinding = 0 "Stator copper losses";
            Modelica.SIunits.Power lossPowerStatorCore = 0 "Stator core losses";
            Modelica.SIunits.Power lossPowerRotorCore = 0 "Rotor core losses";
            Modelica.SIunits.Power lossPowerStrayLoad = 0 "Stray load losses";
            Modelica.SIunits.Power lossPowerFriction = 0 "Friction losses";
            annotation(
              defaultComponentPrefixes = "output",
              Documentation(info = "<HTML>
Partial power balance of induction machines.
 </HTML>"));
          end PartialPowerBalanceInductionMachines;

          connector ThermalPortAIMC "Thermal port of asynchronous induction machine with squirrel cage"
            extends Machines.Interfaces.InductionMachines.PartialThermalPortInductionMachines;
            Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a heatPortRotorWinding "Heat port of rotor (squirrel cage)" annotation(
              Placement(transformation(extent = {{-20, -30}, {0, -10}})));
            annotation(
              Documentation(info = "<HTML>
Thermal port for asynchronous induction machine with squirrel cage
</HTML>"));
          end ThermalPortAIMC;

          record PowerBalanceAIMC "Power balance of asynchronous induction machines with squirrel cage"
            extends Machines.Interfaces.InductionMachines.PartialPowerBalanceInductionMachines(final lossPowerTotal = lossPowerStatorWinding + lossPowerStatorCore + lossPowerRotorCore + lossPowerStrayLoad + lossPowerFriction + lossPowerRotorWinding);
            Modelica.SIunits.Power lossPowerRotorWinding "Rotor copper losses";
            annotation(
              defaultComponentPrefixes = "output",
              Documentation(info = "<HTML>
Power balance of asynchronous induction machines with squirrel cage.
 </HTML>"));
          end PowerBalanceAIMC;

          connector ThermalPortAIMS "Thermal port of asynchronous induction machine with slipring"
            extends Machines.Interfaces.InductionMachines.PartialThermalPortInductionMachines;
            parameter Integer mr = m "Number of rotor phases";
            Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a heatPortRotorWinding[mr] "Heat port of rotor windings" annotation(
              Placement(transformation(extent = {{-20, -30}, {0, -10}})));
            Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a heatPortBrush "Heat port of (optional) brush losses" annotation(
              Placement(transformation(extent = {{-10, 30}, {10, 50}})));
            annotation(
              Documentation(info = "<HTML>
Thermal port for asynchronous induction machine with slipring rotor
</HTML>"));
          end ThermalPortAIMS;

          record PowerBalanceAIMS "Power balance of asynchronous induction machines with slipring"
            extends Machines.Interfaces.InductionMachines.PartialPowerBalanceInductionMachines(final lossPowerTotal = lossPowerStatorWinding + lossPowerStatorCore + lossPowerRotorCore + lossPowerStrayLoad + lossPowerFriction + lossPowerRotorWinding + lossPowerBrush);
            Modelica.SIunits.Power lossPowerRotorWinding "Rotor copper losses";
            Modelica.SIunits.Power lossPowerBrush "Brush losses";
            Modelica.SIunits.Power powerRotor "Electrical power (rotor)";
            annotation(
              defaultComponentPrefixes = "output",
              Documentation(info = "<HTML>
Power balance of asynchronous induction machines with slipring.
 </HTML>"));
          end PowerBalanceAIMS;

          connector ThermalPortSMPM "Thermal port of synchronous induction machine with permanent magnets"
            extends Machines.Interfaces.InductionMachines.PartialThermalPortInductionMachines;
            parameter Boolean useDamperCage(start = true) "Enable / disable damper cage" annotation(
              Evaluate = true);
            Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a heatPortRotorWinding if useDamperCage "Heat port of damper cage (optional)" annotation(
              Placement(transformation(extent = {{-20, -30}, {0, -10}})));
            Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a heatPortPermanentMagnet "Heat port of permanent magnets" annotation(
              Placement(transformation(extent = {{-20, -10}, {0, 10}})));
            annotation(
              Documentation(info = "<HTML>
Thermal port for synchronous induction machine with permanent magnets
</HTML>"));
          end ThermalPortSMPM;

          record PowerBalanceSMPM "Power balance of synchronous induction machines with permanent magnet"
            extends Machines.Interfaces.InductionMachines.PartialPowerBalanceInductionMachines(final lossPowerTotal = lossPowerStatorWinding + lossPowerStatorCore + lossPowerRotorCore + lossPowerStrayLoad + lossPowerFriction + lossPowerRotorWinding + lossPowerPermanentMagnet);
            Modelica.SIunits.Power lossPowerRotorWinding "Rotor copper losses";
            Modelica.SIunits.Power lossPowerPermanentMagnet "Permanent magnet losses";
            annotation(
              defaultComponentPrefixes = "output",
              Documentation(info = "<HTML>
Power balance of synchronous induction machines with permanent magnet.
 </HTML>"));
          end PowerBalanceSMPM;

          connector ThermalPortSMEE "Thermal port of synchronous induction machine with electrical excitation"
            extends Machines.Interfaces.InductionMachines.PartialThermalPortInductionMachines;
            parameter Boolean useDamperCage(start = true) "Enable / disable damper cage" annotation(
              Evaluate = true);
            Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a heatPortRotorWinding if useDamperCage "Heat port of damper cage (optional)" annotation(
              Placement(transformation(extent = {{-20, -30}, {0, -10}})));
            Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a heatPortExcitation "Heat port of excitation" annotation(
              Placement(transformation(extent = {{-20, -10}, {0, 10}})));
            Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a heatPortBrush "Heat port of (optional) brush losses" annotation(
              Placement(transformation(extent = {{-10, 30}, {10, 50}})));
            annotation(
              Documentation(info = "<HTML>
Thermal port for synchronous induction machine with electrical excitation
</HTML>"));
          end ThermalPortSMEE;

          record PowerBalanceSMEE "Power balance of synchronous induction machines with electrical excitation"
            extends Machines.Interfaces.InductionMachines.PartialPowerBalanceInductionMachines(final lossPowerTotal = lossPowerStatorWinding + lossPowerStatorCore + lossPowerRotorCore + lossPowerStrayLoad + lossPowerFriction + lossPowerRotorWinding + lossPowerExcitation + lossPowerBrush);
            Modelica.SIunits.Power lossPowerRotorWinding "Rotor copper losses";
            Modelica.SIunits.Power powerExcitation "Electrical excitation power";
            Modelica.SIunits.Power lossPowerExcitation "Excitation losses";
            Modelica.SIunits.Power lossPowerBrush "Brush losses";
            annotation(
              defaultComponentPrefixes = "output",
              Documentation(info = "<HTML>
Power balance of synchronous induction machines with electrical excitation.
 </HTML>"));
          end PowerBalanceSMEE;

          connector ThermalPortSMR "Thermal port of synchronous induction machine with reluctance rotor"
            extends Machines.Interfaces.InductionMachines.PartialThermalPortInductionMachines;
            parameter Boolean useDamperCage(start = true) "Enable / disable damper cage" annotation(
              Evaluate = true);
            Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a heatPortRotorWinding if useDamperCage "Heat port of damper cage (optional)" annotation(
              Placement(transformation(extent = {{-20, -30}, {0, -10}})));
            annotation(
              Documentation(info = "<HTML>
Thermal port for synchronous induction machine with reluctance rotor
</HTML>"));
          end ThermalPortSMR;

          record PowerBalanceSMR "Power balance of synchronous induction machines with reluctance rotor"
            extends Machines.Interfaces.InductionMachines.PartialPowerBalanceInductionMachines(final lossPowerTotal = lossPowerStatorWinding + lossPowerStatorCore + lossPowerRotorCore + lossPowerStrayLoad + lossPowerFriction + lossPowerRotorWinding);
            Modelica.SIunits.Power lossPowerRotorWinding "Rotor copper losses";
            annotation(
              defaultComponentPrefixes = "output",
              Documentation(info = "<HTML>
Power balance of synchronous induction machines with reluctance rotor.
 </HTML>"));
          end PowerBalanceSMR;
          annotation(
            Documentation(info = "<HTML>
Interfaces and partial models for induction machines
</HTML>"));
        end InductionMachines;

        partial model PartialBasicDCMachine "Partial model for DC machine"
          parameter Modelica.SIunits.Temperature TaOperational(start = 293.15) "Operational armature temperature" annotation(
            Dialog(group = "Operational temperatures", enable = not useThermalPort));
          parameter Modelica.SIunits.Voltage VaNominal(start = 100) "Nominal armature voltage" annotation(
            Dialog(tab = "Nominal parameters"));
          parameter Modelica.SIunits.Current IaNominal(start = 100) "Nominal armature current (>0..Motor, <0..Generator)" annotation(
            Dialog(tab = "Nominal parameters"));
          parameter Modelica.SIunits.AngularVelocity wNominal(displayUnit = "1/min", start = 1425 * 2 * pi / 60) "Nominal speed" annotation(
            Dialog(tab = "Nominal parameters"));
          parameter Modelica.SIunits.Temperature TaNominal(start = 293.15) "Nominal armature temperature" annotation(
            Dialog(tab = "Nominal parameters"));
          parameter Modelica.SIunits.Resistance Ra(start = 0.05) "Armature resistance at TRef" annotation(
            Dialog(tab = "Nominal resistances and inductances"));
          parameter Modelica.SIunits.Temperature TaRef(start = 293.15) "Reference temperature of armature resistance" annotation(
            Dialog(tab = "Nominal resistances and inductances"));
          parameter Machines.Thermal.LinearTemperatureCoefficient20 alpha20a(start = 0) "Temperature coefficient of armature resistance" annotation(
            Dialog(tab = "Nominal resistances and inductances"));
          parameter Modelica.SIunits.Inductance La(start = 0.0015) "Armature inductance" annotation(
            Dialog(tab = "Nominal resistances and inductances"));
          extends PartialBasicMachine(Jr(start = 0.15), frictionParameters(wRef = wNominal), friction(final useHeatPort = true));
          parameter Machines.Losses.CoreParameters coreParameters(final m = 1, VRef = ViNominal, wRef = wNominal) "Armature core loss parameter record" annotation(
            Dialog(tab = "Losses"));
          parameter Machines.Losses.StrayLoadParameters strayLoadParameters(IRef = IaNominal, wRef = wNominal) "Stray load loss parameter record" annotation(
            Dialog(tab = "Losses"));
          parameter Machines.Losses.BrushParameters brushParameters(ILinear = 0.01 * IaNominal) "Brush loss parameter record" annotation(
            Dialog(tab = "Losses"));
          replaceable output Machines.Interfaces.DCMachines.PartialPowerBalanceDCMachines powerBalance(final powerArmature = va * ia, final powerMechanical = wMechanical * tauShaft, final powerInertiaStator = inertiaStator.J * inertiaStator.a * inertiaStator.w, final powerInertiaRotor = inertiaRotor.J * inertiaRotor.a * inertiaRotor.w, final lossPowerArmature = ra.LossPower, final lossPowerCore = core.lossPower, final lossPowerStrayLoad = strayLoad.lossPower, final lossPowerFriction = friction.lossPower, final lossPowerBrush = brush.lossPower) "Power balance";
          output Modelica.SIunits.Voltage va = pin_ap.v - pin_an.v "Armature voltage";
          output Modelica.SIunits.Current ia(start = 0) = pin_ap.i "Armature current";
          Modelica.Electrical.Analog.Interfaces.PositivePin pin_ap "Positive armature pin" annotation(
            Placement(transformation(extent = {{50, 110}, {70, 90}}, rotation = 0)));
          Modelica.Electrical.Analog.Interfaces.NegativePin pin_an "Negative armature pin" annotation(
            Placement(transformation(extent = {{-70, 110}, {-50, 90}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Resistor ra(final R = Ra, final T_ref = TaRef, final alpha = Machines.Thermal.convertAlpha(alpha20a, TaRef), final useHeatPort = true, final T = TaRef) annotation(
            Placement(transformation(extent = {{60, 50}, {40, 70}}, rotation = 0)));
          Machines.BasicMachines.Components.InductorDC la(final L = La, final quasiStationary = quasiStationary) annotation(
            Placement(transformation(extent = {{30, 50}, {10, 70}}, rotation = 0)));
          Machines.Losses.DCMachines.Brush brush(final brushParameters = brushParameters, final useHeatPort = true) annotation(
            Placement(transformation(extent = {{-10, 10}, {10, -10}}, rotation = 180, origin = {-20, 60})));
          Machines.Losses.DCMachines.Core core(final coreParameters = coreParameters, final useHeatPort = true) annotation(
            Placement(transformation(extent = {{10, 70}, {-10, 90}})));
          Machines.Losses.DCMachines.StrayLoad strayLoad(final strayLoadParameters = strayLoadParameters, final useHeatPort = true) annotation(
            Placement(transformation(extent = {{90, 50}, {70, 70}})));
          replaceable Machines.Interfaces.DCMachines.PartialThermalPortDCMachines thermalPort if useThermalPort annotation(
            Placement(transformation(extent = {{-10, -110}, {10, -90}})));
          replaceable Machines.Interfaces.DCMachines.PartialThermalAmbientDCMachines thermalAmbient(final useTemperatureInputs = false, final Ta = TaOperational) if not useThermalPort annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 270, origin = {-30, -80})));
        protected
          constant Boolean quasiStationary = false "No electrical transients if true" annotation(
            Evaluate = true);
          parameter Modelica.SIunits.Voltage ViNominal "Nominal induced Voltage";
          parameter Modelica.SIunits.MagneticFlux psi_eNominal "Nominal magnetic flux";
          parameter Real turnsRatio = ViNominal / (wNominal * psi_eNominal) "Ratio of armature turns over number of turns of the excitation winding";
          replaceable Machines.Interfaces.DCMachines.PartialThermalPortDCMachines internalThermalPort annotation(
            Placement(transformation(extent = {{-4, -84}, {4, -76}})));
        equation
          assert(ViNominal > Modelica.Constants.eps, "VaNominal has to be > (Ra[+Rse])*IaNominal");
          connect(la.p, ra.n) annotation(
            Line(points = {{30, 60}, {40, 60}}, color = {0, 0, 255}));
          connect(thermalPort, internalThermalPort) annotation(
            Line(points = {{0, -100}, {0, -80}}, color = {199, 0, 0}, smooth = Smooth.None));
          connect(thermalAmbient.thermalPort, internalThermalPort) annotation(
            Line(points = {{-20, -80}, {0, -80}}, color = {199, 0, 0}, smooth = Smooth.None));
          connect(brush.n, pin_an) annotation(
            Line(points = {{-30, 60}, {-60, 60}, {-60, 100}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(pin_ap, strayLoad.p) annotation(
            Line(points = {{60, 100}, {90, 100}, {90, 60}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(strayLoad.n, ra.p) annotation(
            Line(points = {{70, 60}, {60, 60}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(strayLoad.support, internalSupport) annotation(
            Line(points = {{80, 50}, {80, 30}, {60, 30}, {60, -100}}, color = {0, 0, 0}, smooth = Smooth.None));
          connect(la.p, core.p) annotation(
            Line(points = {{30, 60}, {30, 80}, {10, 80}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(core.n, brush.p) annotation(
            Line(points = {{-10, 80}, {-10, 60}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(core.heatPort, internalThermalPort.heatPortCore) annotation(
            Line(points = {{10, 70}, {10, 40}, {50, 40}, {50, -80}, {0, -80}}, color = {191, 0, 0}, smooth = Smooth.None));
          connect(brush.heatPort, internalThermalPort.heatPortBrush) annotation(
            Line(points = {{-10, 50}, {-10, 40}, {50, 40}, {50, -80}, {0, -80}}, color = {191, 0, 0}, smooth = Smooth.None));
          connect(strayLoad.heatPort, internalThermalPort.heatPortStrayLoad) annotation(
            Line(points = {{90, 50}, {90, 40}, {50, 40}, {50, -80}, {0, -80}}, color = {191, 0, 0}, smooth = Smooth.None));
          connect(friction.heatPort, internalThermalPort.heatPortFriction) annotation(
            Line(points = {{80, -50}, {50, -50}, {50, -80}, {0, -80}}, color = {191, 0, 0}, smooth = Smooth.None));
          connect(ra.heatPort, internalThermalPort.heatPortArmature) annotation(
            Line(points = {{50, 50}, {50, -80}, {0, -80}}, color = {191, 0, 0}, smooth = Smooth.None));
          connect(inertiaRotor.flange_b, strayLoad.flange) annotation(
            Line(points = {{90, 0}, {92, 0}, {92, 30}, {100, 30}, {100, 70}, {80, 70}}, color = {0, 0, 0}, smooth = Smooth.None));
          annotation(
            Documentation(info = "<HTML>
Partial model for DC machine models.
</HTML>"),
            Icon(graphics = {Line(points = {{-40, 70}, {-60, 70}, {-60, 90}}, color = {0, 0, 255}), Line(points = {{40, 70}, {60, 70}, {60, 90}}, color = {0, 0, 255})}));
        end PartialBasicDCMachine;

        package DCMachines "Thermal ports of DC machines"
          extends Modelica.Icons.VariantsPackage;

          connector PartialThermalPortDCMachines "Partial thermal port of DC machines"
            Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a heatPortArmature "Heat port of armature" annotation(
              Placement(transformation(extent = {{-20, 10}, {0, 30}})));
            Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a heatPortCore "Heat port of (optional) core losses" annotation(
              Placement(transformation(extent = {{0, 10}, {20, 30}})));
            Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a heatPortStrayLoad "Heat port of (optional) stray losses" annotation(
              Placement(transformation(extent = {{0, -10}, {20, 10}})));
            Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a heatPortFriction "Heat port of (optional) friction losses" annotation(
              Placement(transformation(extent = {{0, -30}, {20, -10}})));
            Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a heatPortBrush "Heat port of (optional) brush losses" annotation(
              Placement(transformation(extent = {{-10, 30}, {10, 50}})));
            annotation(
              Diagram(graphics = {Rectangle(extent = {{-60, 60}, {60, -60}}, lineColor = {199, 0, 0}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-50, 50}, {50, -50}}, lineColor = {191, 0, 0}, fillColor = {191, 0, 0}, fillPattern = FillPattern.Solid)}),
              Icon(graphics = {Rectangle(extent = {{-110, 110}, {110, -110}}, lineColor = {199, 0, 0}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-82, 80}, {78, -80}}, lineColor = {191, 0, 0}, fillColor = {191, 0, 0}, fillPattern = FillPattern.Solid)}),
              Documentation(info = "<HTML>
Partial thermal port for DC machines
</HTML>"));
          end PartialThermalPortDCMachines;

          model PartialThermalAmbientDCMachines "Partial thermal ambient for DC machines"
            parameter Boolean useTemperatureInputs = false "If true, temperature inputs are used; else, temperatures are constant" annotation(
              Evaluate = true);
            constant Modelica.SIunits.Temperature TDefault = 293.15 "Default temperature";
            parameter Modelica.SIunits.Temperature Ta(start = TDefault) "Temperature of armature" annotation(
              Dialog(enable = not useTemperatureInputs));
            output Modelica.SIunits.HeatFlowRate Q_flowArmature = temperatureArmature.port.Q_flow "Heat flow rate of armature";
            output Modelica.SIunits.HeatFlowRate Q_flowCore = temperatureCore.port.Q_flow "Heat flow rate of core losses";
            output Modelica.SIunits.HeatFlowRate Q_flowStrayLoad = temperatureStrayLoad.port.Q_flow "Heat flow rate of stray load losses";
            output Modelica.SIunits.HeatFlowRate Q_flowFriction = temperatureFriction.port.Q_flow "Heat flow rate of friction losses";
            output Modelica.SIunits.HeatFlowRate Q_flowBrush = temperatureBrush.port.Q_flow "Heat flow rate of brushes";
            replaceable Machines.Interfaces.DCMachines.PartialThermalPortDCMachines thermalPort annotation(
              Placement(transformation(extent = {{-10, 90}, {10, 110}})));
            Modelica.Thermal.HeatTransfer.Sources.PrescribedTemperature temperatureArmature annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 90, origin = {-80, 30})));
            Modelica.Thermal.HeatTransfer.Sources.FixedTemperature temperatureCore(final T = TDefault) annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 90, origin = {40, 30})));
            Modelica.Thermal.HeatTransfer.Sources.FixedTemperature temperatureStrayLoad(final T = TDefault) annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 90, origin = {60, 10})));
            Modelica.Thermal.HeatTransfer.Sources.FixedTemperature temperatureFriction(final T = TDefault) annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 90, origin = {80, 30})));
            Modelica.Blocks.Interfaces.RealInput TArmature if useTemperatureInputs "Temperature of armature" annotation(
              Placement(transformation(extent = {{-20, -20}, {20, 20}}, rotation = 90, origin = {-80, -100})));
            Modelica.Blocks.Sources.Constant constTa(final k = Ta) if not useTemperatureInputs annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 90, origin = {-80, -10})));
            Modelica.Thermal.HeatTransfer.Sources.FixedTemperature temperatureBrush(final T = TDefault) annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 90, origin = {20, 10})));
          equation
            connect(constTa.y, temperatureArmature.T) annotation(
              Line(points = {{-80, 1}, {-80, 18}}, color = {0, 0, 127}, smooth = Smooth.None));
            connect(TArmature, temperatureArmature.T) annotation(
              Line(points = {{-80, -100}, {-80, -60}, {-100, -60}, {-100, 8}, {-80, 8}, {-80, 18}}, color = {0, 0, 127}, smooth = Smooth.None));
            connect(temperatureArmature.port, thermalPort.heatPortArmature) annotation(
              Line(points = {{-80, 40}, {-80, 100}, {0, 100}}, color = {191, 0, 0}, smooth = Smooth.None));
            connect(temperatureBrush.port, thermalPort.heatPortBrush) annotation(
              Line(points = {{20, 20}, {20, 100}, {0, 100}}, color = {191, 0, 0}, smooth = Smooth.None));
            connect(temperatureCore.port, thermalPort.heatPortCore) annotation(
              Line(points = {{40, 40}, {40, 100}, {0, 100}}, color = {191, 0, 0}, smooth = Smooth.None));
            connect(temperatureStrayLoad.port, thermalPort.heatPortStrayLoad) annotation(
              Line(points = {{60, 20}, {60, 100}, {0, 100}}, color = {191, 0, 0}, smooth = Smooth.None));
            connect(temperatureFriction.port, thermalPort.heatPortFriction) annotation(
              Line(points = {{80, 40}, {80, 100}, {0, 100}}, color = {191, 0, 0}, smooth = Smooth.None));
            annotation(
              Icon(graphics = {Rectangle(extent = {{-100, 100}, {100, -100}}, lineColor = {0, 0, 0}, pattern = LinePattern.None, fillColor = {159, 159, 223}, fillPattern = FillPattern.Backward), Line(points = {{-14, 0}, {54, 0}}, color = {191, 0, 0}, thickness = 0.5, origin = {0, -6}, rotation = 90), Polygon(points = {{-20, -20}, {-20, 20}, {20, 0}, {-20, -20}}, lineColor = {191, 0, 0}, fillColor = {191, 0, 0}, fillPattern = FillPattern.Solid, origin = {0, 68}, rotation = 90)}),
              Documentation(info = "<HTML>
Partial thermal ambient for induction machines
</HTML>"));
          end PartialThermalAmbientDCMachines;

          record PartialPowerBalanceDCMachines "Partial power balance of DC machines"
            extends Modelica.Icons.Record;
            Modelica.SIunits.Power powerArmature = 0 "Electrical armature power";
            Modelica.SIunits.Power powerMechanical = 0 "Mechanical power";
            Modelica.SIunits.Power powerInertiaStator = 0 "Stator inertia power";
            Modelica.SIunits.Power powerInertiaRotor = 0 "Rotor inertia power";
            Modelica.SIunits.Power lossPowerTotal = 0 "Total loss power";
            Modelica.SIunits.Power lossPowerArmature = 0 "Armature copper losses";
            Modelica.SIunits.Power lossPowerCore = 0 "Core losses";
            Modelica.SIunits.Power lossPowerStrayLoad = 0 "Stray load losses";
            Modelica.SIunits.Power lossPowerFriction = 0 "Friction losses";
            Modelica.SIunits.Power lossPowerBrush = 0 "Brush losses";
            annotation(
              defaultComponentPrefixes = "output",
              Documentation(info = "<HTML>
Partial power balance of DC machines.
 </HTML>"));
          end PartialPowerBalanceDCMachines;

          connector ThermalPortDCPM "Thermal port of DC machine with permanent magnets"
            extends Machines.Interfaces.DCMachines.PartialThermalPortDCMachines;
            Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a heatPortPermanentMagnet "Heat port of permanent magnets" annotation(
              Placement(transformation(extent = {{-20, -30}, {0, -10}})));
            annotation(
              Documentation(info = "<HTML>
Thermal port for DC machine with permanent magnets
</HTML>"));
          end ThermalPortDCPM;

          record PowerBalanceDCPM "Power balance of DC machines with permanent magnet"
            extends Machines.Interfaces.DCMachines.PartialPowerBalanceDCMachines(final lossPowerTotal = lossPowerArmature + lossPowerCore + lossPowerStrayLoad + lossPowerFriction + lossPowerBrush + lossPowerPermanentMagnet);
            Modelica.SIunits.Power lossPowerPermanentMagnet "Permanent magnet losses";
            annotation(
              defaultComponentPrefixes = "output",
              Documentation(info = "<HTML>
Power balance of DC machines with permanent magnet.
 </HTML>"));
          end PowerBalanceDCPM;

          connector ThermalPortDCEE "Thermal port of DC machine with electrical excitation"
            extends Machines.Interfaces.DCMachines.PartialThermalPortDCMachines;
            Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a heatPortExcitation "Heat port of (shunt) excitation" annotation(
              Placement(transformation(extent = {{-20, -30}, {0, -10}})));
            annotation(
              Documentation(info = "<HTML>
Thermal port for DC machine with electrical (shunt) excitation
</HTML>"));
          end ThermalPortDCEE;

          record PowerBalanceDCEE "Power balance of DC machines with electrical excitation"
            extends Machines.Interfaces.DCMachines.PartialPowerBalanceDCMachines(final lossPowerTotal = lossPowerArmature + lossPowerCore + lossPowerStrayLoad + lossPowerFriction + lossPowerBrush + lossPowerExcitation);
            Modelica.SIunits.Power powerExcitation "Electrical (shunt) excitation power";
            Modelica.SIunits.Power lossPowerExcitation "Excitation losses";
            annotation(
              defaultComponentPrefixes = "output",
              Documentation(info = "<HTML>
Power balance of DC machines with electrical excitation.
 </HTML>"));
          end PowerBalanceDCEE;

          connector ThermalPortDCSE "Thermal port of DC machine with series excitation"
            extends Machines.Interfaces.DCMachines.PartialThermalPortDCMachines;
            Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a heatPortSeriesExcitation "Heat port of series excitation" annotation(
              Placement(transformation(extent = {{-20, -10}, {0, 10}})));
            annotation(
              Documentation(info = "<HTML>
Thermal port for DC machine with serial excitation
</HTML>"));
          end ThermalPortDCSE;

          record PowerBalanceDCSE "Power balance of DC machines with series excitation"
            extends Machines.Interfaces.DCMachines.PartialPowerBalanceDCMachines(final lossPowerTotal = lossPowerArmature + lossPowerCore + lossPowerStrayLoad + lossPowerFriction + lossPowerBrush + lossPowerSeriesExcitation);
            Modelica.SIunits.Power powerSeriesExcitation "Electrical series excitation power";
            Modelica.SIunits.Power lossPowerSeriesExcitation "Series excitation losses";
            annotation(
              defaultComponentPrefixes = "output",
              Documentation(info = "<HTML>
Power balance of DC machines with series excitation.
 </HTML>"));
          end PowerBalanceDCSE;

          connector ThermalPortDCCE "Thermal port of DC machine with compound excitation"
            extends Machines.Interfaces.DCMachines.PartialThermalPortDCMachines;
            Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a heatPortShuntExcitation "Heat port of (shunt) excitation" annotation(
              Placement(transformation(extent = {{-20, -30}, {0, -10}})));
            Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a heatPortSeriesExcitation "Heat port of series excitation" annotation(
              Placement(transformation(extent = {{-20, -10}, {0, 10}})));
            annotation(
              Documentation(info = "<HTML>
Thermal port for DC machine with compound excitation
</HTML>"));
          end ThermalPortDCCE;

          record PowerBalanceDCCE "Power balance of DC machines with compound excitation"
            extends Machines.Interfaces.DCMachines.PartialPowerBalanceDCMachines(final lossPowerTotal = lossPowerArmature + lossPowerCore + lossPowerStrayLoad + lossPowerFriction + lossPowerBrush + lossPowerShuntExcitation + lossPowerSeriesExcitation);
            Modelica.SIunits.Power powerShuntExcitation "Electrical (shunt) excitation power";
            Modelica.SIunits.Power powerSeriesExcitation "Electrical series excitation power";
            Modelica.SIunits.Power lossPowerShuntExcitation "(Shunt) excitation losses";
            Modelica.SIunits.Power lossPowerSeriesExcitation "Series excitation losses";
            annotation(
              defaultComponentPrefixes = "output",
              Documentation(info = "<HTML>
Power balance of DC machines with compound excitation.
 </HTML>"));
          end PowerBalanceDCCE;
          annotation(
            Documentation(info = "<HTML>
Thermal ports for DC machines
</HTML>"));
        end DCMachines;

        partial model PartialBasicTransformer "Partial model of three-phase transformer"
          extends Machines.Icons.TransientTransformer;
          final parameter Integer m(min = 1) = 3 "Number of phases";
          constant String VectorGroup = "Yy00";
          parameter Real n(start = 1) "Ratio primary voltage (line-to-line) / secondary voltage (line-to-line)";
          parameter Modelica.SIunits.Resistance R1(start = 5E-3 / (if C1 == "D" then 1 else 3)) "Primary resistance per phase at TRef" annotation(
            Dialog(tab = "Nominal resistances and inductances"));
          parameter Modelica.SIunits.Temperature T1Ref(start = 293.15) "Reference temperature of primary resistance" annotation(
            Dialog(tab = "Nominal resistances and inductances"));
          parameter Machines.Thermal.LinearTemperatureCoefficient20 alpha20_1(start = 0) "Temperature coefficient of primary resistance at 20 degC" annotation(
            Dialog(tab = "Nominal resistances and inductances"));
          parameter Modelica.SIunits.Inductance L1sigma(start = 78E-6 / (if C1 == "D" then 1 else 3)) "Primary stray inductance per phase" annotation(
            Dialog(tab = "Nominal resistances and inductances"));
          parameter Modelica.SIunits.Resistance R2(start = 5E-3 / (if C2 == "d" then 1 else 3)) "Secondary resistance per phase at TRef" annotation(
            Dialog(tab = "Nominal resistances and inductances"));
          parameter Modelica.SIunits.Temperature T2Ref(start = 293.15) "Reference temperature of secondary resistance" annotation(
            Dialog(tab = "Nominal resistances and inductances"));
          parameter Machines.Thermal.LinearTemperatureCoefficient20 alpha20_2(start = 0) "Temperature coefficient of secondary resistance at 20 degC" annotation(
            Dialog(tab = "Nominal resistances and inductances"));
          parameter Modelica.SIunits.Inductance L2sigma(start = 78E-6 / (if C2 == "d" then 1 else 3)) "Secondary stray inductance per phase" annotation(
            Dialog(tab = "Nominal resistances and inductances"));
          parameter Boolean useThermalPort = false "Enable / disable (=fixed temperatures) thermal port" annotation(
            Evaluate = true);
          parameter Modelica.SIunits.Temperature T1Operational(start = 293.15) "Operational temperature of primary resistance" annotation(
            Dialog(group = "Operational temperatures", enable = not useThermalPort));
          parameter Modelica.SIunits.Temperature T2Operational(start = 293.15) "Operational temperature of secondary resistance" annotation(
            Dialog(group = "Operational temperatures", enable = not useThermalPort));
          output Machines.Interfaces.PowerBalanceTransformer powerBalance(final power1 = Machines.SpacePhasors.Functions.activePower(v1, i1), final power2 = Machines.SpacePhasors.Functions.activePower(v2, i2), final lossPower1 = sum(r1.resistor.LossPower), final lossPower2 = sum(r2.resistor.LossPower), final lossPowerCore = 0) "Power balance";
          output Modelica.SIunits.Voltage v1[m] = plug1.pin.v "Primary voltage";
          output Modelica.SIunits.Current i1[m] = plug1.pin.i "Primary current";
          output Modelica.SIunits.Voltage v2[m] = plug2.pin.v "Secondary voltage";
          output Modelica.SIunits.Current i2[m] = plug2.pin.i "Secondary current";
        protected
          constant String C1 = Modelica.Utilities.Strings.substring(VectorGroup, 1, 1);
          constant String C2 = Modelica.Utilities.Strings.substring(VectorGroup, 2, 2);
          parameter Real ni = n * (if C2 == "z" then sqrt(3) else 2) * (if C2 == "d" then 1 else sqrt(3)) / (if C1 == "D" then 1 else sqrt(3));
        public
          Modelica.Electrical.MultiPhase.Interfaces.PositivePlug plug1(final m = m) "Primary plug" annotation(
            Placement(transformation(extent = {{-110, -10}, {-90, 10}}, rotation = 0)));
          Modelica.Electrical.MultiPhase.Interfaces.NegativePlug plug2(final m = m) "Secondary plug" annotation(
            Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
          Modelica.Electrical.MultiPhase.Basic.Resistor r1(final m = m, final R = fill(R1, m), final T_ref = fill(T1Ref, m), final alpha = fill(Machines.Thermal.convertAlpha(alpha20_1, T1Ref), m), final useHeatPort = true, final T = fill(T1Ref, m)) annotation(
            Placement(transformation(extent = {{-90, 10}, {-70, -10}}, rotation = 0)));
          Modelica.Electrical.MultiPhase.Basic.Inductor l1sigma(final m = m, final L = fill(L1sigma, m)) annotation(
            Placement(transformation(extent = {{-70, -10}, {-50, 10}}, rotation = 0)));
          Modelica.Electrical.MultiPhase.Basic.Resistor r2(final m = m, final R = fill(R2, m), final T_ref = fill(T2Ref, m), final alpha = fill(Machines.Thermal.convertAlpha(alpha20_2, T2Ref), m), final useHeatPort = true, final T = fill(T2Ref, m)) annotation(
            Placement(transformation(extent = {{90, 10}, {70, -10}}, rotation = 0)));
          Modelica.Electrical.MultiPhase.Basic.Inductor l2sigma(final m = m, final L = fill(L2sigma, m)) annotation(
            Placement(transformation(extent = {{70, -10}, {50, 10}}, rotation = 0)));
          Machines.BasicMachines.Components.IdealCore core(final m = m, final n12 = ni, final n13 = ni) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0)));
          Machines.Interfaces.ThermalPortTransformer thermalPort(final m = m) if useThermalPort annotation(
            Placement(transformation(extent = {{-10, 90}, {10, 110}})));
          Machines.Thermal.ThermalAmbientTransformer thermalAmbient(final useTemperatureInputs = false, final T1 = T1Operational, final T2 = T2Operational, final m = m) if not useThermalPort annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 270, origin = {-30, 80})));
        protected
          Machines.Interfaces.ThermalPortTransformer internalThermalPort(final m = m) annotation(
            Placement(transformation(extent = {{-4, 76}, {4, 84}})));
        equation
          connect(r1.plug_n, l1sigma.plug_p) annotation(
            Line(points = {{-70, 0}, {-70, 0}}, color = {0, 0, 255}));
          connect(plug1, r1.plug_p) annotation(
            Line(points = {{-100, 0}, {-90, 0}}, color = {0, 0, 255}));
          connect(thermalPort, internalThermalPort) annotation(
            Line(points = {{0, 100}, {0, 80}}, color = {199, 0, 0}, smooth = Smooth.None));
          connect(thermalAmbient.thermalPort, internalThermalPort) annotation(
            Line(points = {{-20, 80}, {0, 80}}, color = {199, 0, 0}, smooth = Smooth.None));
          connect(r1.heatPort, internalThermalPort.heatPort1) annotation(
            Line(points = {{-80, 10}, {-80, 60}, {0, 60}, {0, 80}}, color = {191, 0, 0}, smooth = Smooth.None));
          connect(r2.heatPort, internalThermalPort.heatPort2) annotation(
            Line(points = {{80, 10}, {80, 60}, {0, 60}, {0, 80}}, color = {191, 0, 0}, smooth = Smooth.None));
          connect(r2.plug_p, plug2) annotation(
            Line(points = {{90, 0}, {100, 0}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(l2sigma.plug_p, r2.plug_n) annotation(
            Line(points = {{70, 0}, {70, 0}}, color = {0, 0, 255}, smooth = Smooth.None));
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{0, -60}, {0, -100}}, lineColor = {0, 0, 255}, textString = "%VectorGroup"), Text(extent = {{0, 100}, {0, 60}}, lineColor = {0, 0, 255}, textString = "%name")}),
            Documentation(info = "<html>
Partial model of a three-phase transformer, containing primary and secondary resistances and stray inductances, as well as the iron core.
Circuit layout (vector group) of primary and secondary windings have to be defined.
<br><b>Default values for transformer's parameters (a realistic example) are:</b><br>
<table>
<tr>
<td valign=\"top\">turns ratio n</td>
<td valign=\"top\">1</td><td valign=\"top\"> </td>
</tr>
<tr>
<td valign=\"top\">nominal frequency fNominal</td>
<td valign=\"top\">50</td><td valign=\"top\">Hz</td>
</tr>
<tr>
<td valign=\"top\">nominal voltage per phase</td>
<td valign=\"top\">100</td><td valign=\"top\">V RMS</td>
</tr>
<tr>
<td valign=\"top\">nominal current per phase</td>
<td valign=\"top\">100</td><td valign=\"top\">A RMS</td>
</tr>
<tr>
<td valign=\"top\">nominal apparent power</td>
<td valign=\"top\">30</td><td valign=\"top\">kVA</td>
</tr>
<tr>
<td valign=\"top\">primary resistance R1</td>
<td valign=\"top\">0.005</td><td valign=\"top\">Ohm per phase at reference temperature</td>
</tr>
<tr>
<td valign=\"top\">reference temperature T1Ref</td>
<td valign=\"top\">20</td><td valign=\"top\">&deg;C</td>
</tr>
<tr>
<td valign=\"top\">temperature coefficient alpha20_1 </td>
<td valign=\"top\">0</td><td valign=\"top\">1/K</td>
</tr>
<tr>
<td valign=\"top\">primary stray inductance L1sigma</td>
<td valign=\"top\">78E-6</td><td valign=\"top\">H per phase</td>
</tr>
<tr>
<td valign=\"top\">secondary resistance R2</td>
<td valign=\"top\">0.005</td><td valign=\"top\">Ohm per phase at reference temperature</td>
</tr>
<tr>
<td valign=\"top\">reference temperature T2Ref</td>
<td valign=\"top\">20</td><td valign=\"top\">&deg;C</td>
</tr>
<tr>
<td valign=\"top\">temperature coefficient alpha20_2 </td>
<td valign=\"top\">0</td><td valign=\"top\">1/K</td>
</tr>
<tr>
<td valign=\"top\">secondary stray inductance L2sigma</td>
<td valign=\"top\">78E-6</td><td valign=\"top\">H per phase</td>
</tr>
<tr>
<td valign=\"top\">operational temperature T1Operational</td>
<td valign=\"top\">20</td><td valign=\"top\">&deg;C</td>
</tr>
<tr>
<td valign=\"top\">operational temperature T2Operational</td>
<td valign=\"top\">20</td><td valign=\"top\">&deg;C</td>
</tr>
<tr>
<td valign=\"top\">These values give the operational parameters:</td>
<td valign=\"top\"> </td><td valign=\"top\"> </td>
</tr>
<tr>
<td valign=\"top\">nominal voltage drop</td>
<td valign=\"top\">0.05</td><td valign=\"top\">p.u.</td>
</tr>
<tr>
<td valign=\"top\">nominal copper losses</td>
<td valign=\"top\">300</td><td valign=\"top\">W</td>
</tr>
</table>
</html>"));
        end PartialBasicTransformer;

        connector ThermalPortTransformer "Thermal port of transformers"
          parameter Integer m = 3 "number of phases";
          Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a heatPort1[m] "Heat port of primary windings" annotation(
            Placement(transformation(extent = {{-20, 10}, {0, 30}})));
          Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a heatPort2[m] "Heat port of secondary windings" annotation(
            Placement(transformation(extent = {{-20, -30}, {0, -10}})));
          Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a heatPortCore "Heat port of (optional) core losses" annotation(
            Placement(transformation(extent = {{0, -10}, {20, 10}})));
          annotation(
            Diagram(graphics = {Rectangle(extent = {{-60, 60}, {60, -60}}, lineColor = {199, 0, 0}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-50, 50}, {50, -50}}, lineColor = {191, 0, 0}, fillColor = {191, 0, 0}, fillPattern = FillPattern.Solid)}),
            Icon(graphics = {Rectangle(extent = {{-110, 110}, {110, -110}}, lineColor = {199, 0, 0}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-80, 80}, {80, -80}}, lineColor = {191, 0, 0}, fillColor = {191, 0, 0}, fillPattern = FillPattern.Solid)}),
            Documentation(info = "<HTML>
Thermal port for transformers
</HTML>"));
        end ThermalPortTransformer;

        record PowerBalanceTransformer "Power balance of transformers"
          extends Modelica.Icons.Record;
          Modelica.SIunits.Power power1 "Primary power";
          Modelica.SIunits.Power power2 "Secondary power";
          Modelica.SIunits.Power lossPowerTotal = lossPower1 + lossPower2 + lossPowerCore "Total loss power";
          Modelica.SIunits.Power lossPower1 "Primary copper losses";
          Modelica.SIunits.Power lossPower2 "Secondary copper losses";
          Modelica.SIunits.Power lossPowerCore "Core losses";
          annotation(
            defaultComponentPrefixes = "output",
            Documentation(info = "<HTML>
Power balance of transformers.
 </HTML>"));
        end PowerBalanceTransformer;

        partial model FlangeSupport "Shaft and support"
          Modelica.Mechanics.Rotational.Interfaces.Flange_a flange "Shaft end" annotation(
            Placement(transformation(extent = {{-10, 110}, {10, 90}}, rotation = 0)));
          Modelica.Mechanics.Rotational.Interfaces.Flange_a support "Housing and support" annotation(
            Placement(transformation(extent = {{-10, -110}, {10, -90}}, rotation = 0)));
          Modelica.SIunits.Angle phi "Angle between shaft and support";
          Modelica.SIunits.Torque tau "Torque";
          Modelica.SIunits.AngularVelocity w "Relative angular velocity of flange and support";
        equation
          phi = flange.phi - support.phi;
          w = der(phi);
          tau = -flange.tau;
          tau = support.tau;
          annotation(
            Documentation(info = "<html>
<p>
This partial model defines shaft and housing connector for loss models.
Positive torque <code>tau</code> acts as braking torque.
</p>
</html>"),
            Icon(graphics = {Rectangle(extent = {{-20, -80}, {20, -120}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid)}));
        end FlangeSupport;
        annotation(
          Documentation(info = "<HTML>
This package contains the space phasor connector and partial models for machine models.
</HTML>", revisions = "<HTML>
<dl>
  <dt><b>Main Authors:</b></dt>
  <dd>
  <a href=\"http://www.haumer.at/\">Anton Haumer</a><br>
  Technical Consulting &amp; Electrical Engineering<br>
  A-3423 St.Andrae-Woerdern<br>Austria<br>
  email: <a href=\"mailto:a.haumer@haumer.at\">a.haumer@haumer.at</a>
  </dd>
  <dt><b>Copyright:</b></dt>
  <dd>Copyright &copy; 1998-2015, Modelica Association and Anton Haumer.<br>
  <i>The Modelica package is <b>free</b> software; it can be redistributed and/or modified
  under the terms of the <b>Modelica license</b>, see the license conditions
  and the accompanying <b>disclaimer</b> in the documentation of package
  Modelica in file \"Modelica/package.mo\".</i></dd>
</dl>
  <ul>
  <li> v1.00 2004/09/16 Anton Haumer</li>
  <li> v1.01 2004/09/18 Anton Haumer<br>
       moved common equations from machine models to PartialMachine</li>
  <li> v1.02 2004/09/19 Anton Haumer<br>
       added PartialDCMachine</li>
  <li> v1.2  2004/10/27 Anton Haumer<br>
       fixed a bug with support (formerly bearing)</li>
  <li> v1.4   2004/11/11 Anton Haumer<br>
       removed mechanical flange support<br>
       to ease the implementation of a 3D-frame in a future release</li>
  <li> v1.51 Beta 2005/02/01 Anton Haumer<br>
       changed parameter polePairs to Integer</li>
  <li> v2.2.0 2011/02/10 Anton Haumer<br>
       conditional ThermalPort for all machines</li>
  </ul>
</HTML>"));
      end Interfaces;

      package Icons "Icons for electrical machines"
        extends Modelica.Icons.IconsPackage;

        partial model TransientMachine
          annotation(
            Icon(graphics = {Rectangle(extent = {{-40, 60}, {80, -60}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {0, 128, 255}), Rectangle(extent = {{-40, 60}, {-60, -60}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {128, 128, 128}), Rectangle(extent = {{80, 10}, {100, -10}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {95, 95, 95}), Rectangle(extent = {{-40, 70}, {40, 50}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Polygon(points = {{-50, -90}, {-40, -90}, {-10, -20}, {40, -20}, {70, -90}, {80, -90}, {80, -100}, {-50, -100}, {-50, -90}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid)}),
            Documentation(info = "<html>
<p>
This icon is designed for a <b>transient machine</b> model.
</p>
</html>"));
        end TransientMachine;

        partial model QuasiStationaryMachine
          annotation(
            Icon(graphics = {Rectangle(extent = {{-40, 60}, {80, -60}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {170, 213, 255}), Rectangle(extent = {{-40, 60}, {-60, -60}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {128, 128, 128}), Rectangle(extent = {{80, 10}, {100, -10}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {95, 95, 95}), Rectangle(extent = {{-40, 70}, {40, 50}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Polygon(points = {{-50, -90}, {-40, -90}, {-10, -20}, {40, -20}, {70, -90}, {80, -90}, {80, -100}, {-50, -100}, {-50, -90}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid)}),
            Documentation(info = "<html>
<p>
This icon is designed for a <b>quasistationary machine</b> model.
</p>
</html>"));
        end QuasiStationaryMachine;

        partial model FundamentalWaveMachine
          annotation(
            Icon(graphics = {Rectangle(extent = {{-40, 60}, {80, -60}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {255, 128, 0}), Rectangle(extent = {{-40, 60}, {-60, -60}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {128, 128, 128}), Rectangle(extent = {{80, 10}, {100, -10}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {95, 95, 95}), Rectangle(extent = {{-40, 70}, {40, 50}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Polygon(points = {{-50, -90}, {-40, -90}, {-10, -20}, {40, -20}, {70, -90}, {80, -90}, {80, -100}, {-50, -100}, {-50, -90}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid)}),
            Documentation(info = "<html>
<p>
This icon is designed for a <b>FundamentalWave machine</b> model.
</p>
</html>"));
        end FundamentalWaveMachine;

        model TransientTransformer
          annotation(
            Icon(graphics = {Polygon(points = {{-70, 60}, {-50, 40}, {-50, -40}, {-70, -60}, {-70, 60}}, lineColor = {0, 0, 0}, smooth = Smooth.None, fillColor = {135, 135, 135}, fillPattern = FillPattern.VerticalCylinder), Polygon(points = {{70, 60}, {50, 40}, {50, -40}, {70, -60}, {70, 60}}, lineColor = {0, 0, 0}, smooth = Smooth.None, fillColor = {135, 135, 135}, fillPattern = FillPattern.VerticalCylinder), Polygon(points = {{0, 50}, {-10, 40}, {-10, -40}, {0, -50}, {10, -40}, {10, 40}, {0, 50}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.VerticalCylinder, smooth = Smooth.None, fillColor = {135, 135, 135}), Polygon(points = {{-70, 60}, {70, 60}, {50, 40}, {10, 40}, {0, 50}, {-10, 40}, {-50, 40}, {-70, 60}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.VerticalCylinder, smooth = Smooth.None, fillColor = {135, 135, 135}), Polygon(points = {{-70, -60}, {70, -60}, {50, -40}, {10, -40}, {0, -50}, {-10, -40}, {-50, -40}, {-70, -60}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.VerticalCylinder, smooth = Smooth.None, fillColor = {135, 135, 135}), Rectangle(extent = {{-78, 36}, {-42, -36}}, lineColor = {128, 0, 255}, fillPattern = FillPattern.VerticalCylinder, fillColor = {128, 0, 255}), Rectangle(extent = {{-84, 28}, {-36, -28}}, lineColor = {0, 128, 255}, fillPattern = FillPattern.VerticalCylinder, fillColor = {0, 128, 255}), Rectangle(extent = {{-18, 36}, {18, -36}}, lineColor = {128, 0, 255}, fillPattern = FillPattern.VerticalCylinder, fillColor = {128, 0, 255}), Rectangle(extent = {{-24, 28}, {24, -28}}, lineColor = {0, 128, 255}, fillPattern = FillPattern.VerticalCylinder, fillColor = {0, 128, 255}), Rectangle(extent = {{42, 36}, {78, -36}}, lineColor = {128, 0, 255}, fillPattern = FillPattern.VerticalCylinder, fillColor = {128, 0, 255}), Rectangle(extent = {{36, 28}, {84, -28}}, lineColor = {0, 128, 255}, fillPattern = FillPattern.VerticalCylinder, fillColor = {0, 128, 255})}),
            Documentation(info = "<html>
<p>
This icon is designed for a <b>transient transformer</b> model.
</p>
</html>"));
        end TransientTransformer;

        partial model QuasiStationaryTransformer
          annotation(
            Icon(graphics = {Polygon(points = {{-70, 60}, {-50, 40}, {-50, -40}, {-70, -60}, {-70, 60}}, lineColor = {0, 0, 0}, smooth = Smooth.None, fillColor = {135, 135, 135}, fillPattern = FillPattern.VerticalCylinder), Polygon(points = {{70, 60}, {50, 40}, {50, -40}, {70, -60}, {70, 60}}, lineColor = {0, 0, 0}, smooth = Smooth.None, fillColor = {135, 135, 135}, fillPattern = FillPattern.VerticalCylinder), Polygon(points = {{0, 50}, {-10, 40}, {-10, -40}, {0, -50}, {10, -40}, {10, 40}, {0, 50}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.VerticalCylinder, smooth = Smooth.None, fillColor = {135, 135, 135}), Polygon(points = {{-70, 60}, {70, 60}, {50, 40}, {10, 40}, {0, 50}, {-10, 40}, {-50, 40}, {-70, 60}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.VerticalCylinder, smooth = Smooth.None, fillColor = {135, 135, 135}), Polygon(points = {{-70, -60}, {70, -60}, {50, -40}, {10, -40}, {0, -50}, {-10, -40}, {-50, -40}, {-70, -60}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.VerticalCylinder, smooth = Smooth.None, fillColor = {135, 135, 135}), Rectangle(extent = {{-78, 36}, {-42, -36}}, lineColor = {213, 170, 255}, fillPattern = FillPattern.VerticalCylinder, fillColor = {213, 170, 255}), Rectangle(extent = {{-84, 28}, {-36, -28}}, lineColor = {170, 213, 255}, fillPattern = FillPattern.VerticalCylinder, fillColor = {170, 213, 255}), Rectangle(extent = {{-18, 36}, {18, -36}}, lineColor = {213, 170, 255}, fillPattern = FillPattern.VerticalCylinder, fillColor = {213, 170, 255}), Rectangle(extent = {{-24, 28}, {24, -28}}, lineColor = {170, 213, 255}, fillPattern = FillPattern.VerticalCylinder, fillColor = {170, 213, 255}), Rectangle(extent = {{42, 36}, {78, -36}}, lineColor = {213, 170, 255}, fillPattern = FillPattern.VerticalCylinder, fillColor = {213, 170, 255}), Rectangle(extent = {{36, 28}, {84, -28}}, lineColor = {170, 213, 255}, fillPattern = FillPattern.VerticalCylinder, fillColor = {170, 213, 255})}),
            Documentation(info = "<html>
<p>
This icon is designed for a <b>quasistationary transformer</b> model.
</p>
</html>"));
        end QuasiStationaryTransformer;
        annotation(
          Documentation(info = "<html>
<p>
This package contains definitions for the graphical layout of machines.
The icons can be utilized by inheriting them in the desired class using \"extends\" or by directly copying the \"icon\" layer.
</p>
</html>"));
      end Icons;

      package Utilities "Library with auxiliary models for testing"
        extends Modelica.Icons.UtilitiesPackage;

        package ParameterRecords "Parameter records"
          extends Modelica.Icons.MaterialPropertiesPackage;

          record InductionMachineData "Common parameters for induction machines"
            extends Modelica.Icons.Record;
            import Modelica.Constants.pi;
            final parameter Integer m = 3 "Number of phases";
            parameter Modelica.SIunits.Inertia Jr = 0.29 "Rotor's moment of inertia";
            parameter Modelica.SIunits.Inertia Js = Jr "Stator's moment of inertia";
            parameter Integer p(min = 1) = 2 "Number of pole pairs (Integer)";
            parameter Modelica.SIunits.Frequency fsNominal = 50 "Nominal frequency";
            parameter Modelica.SIunits.Resistance Rs = 0.03 "Stator resistance per phase at TRef" annotation(
              Dialog(tab = "Nominal resistances and inductances"));
            parameter Modelica.SIunits.Temperature TsRef = 293.15 "Reference temperature of stator resistance" annotation(
              Dialog(tab = "Nominal resistances and inductances"));
            parameter Machines.Thermal.LinearTemperatureCoefficient20 alpha20s = 0 "Temperature coefficient of stator resistance at 20 degC" annotation(
              Dialog(tab = "Nominal resistances and inductances"));
            parameter Modelica.SIunits.Inductance Lszero = Lssigma "Stator zero sequence inductance" annotation(
              Dialog(tab = "Nominal resistances and inductances"));
            parameter Modelica.SIunits.Inductance Lssigma = 3 * (1 - sqrt(1 - 0.0667)) / (2 * pi * fsNominal) "Stator stray inductance" annotation(
              Dialog(tab = "Nominal resistances and inductances"));
            parameter Machines.Losses.FrictionParameters frictionParameters(PRef = 0, wRef = 2 * pi * fsNominal / p) "Friction losses" annotation(
              Dialog(tab = "Losses"));
            parameter Machines.Losses.CoreParameters statorCoreParameters(final m = m, PRef = 0, VRef = 100, wRef = 2 * pi * fsNominal) "Stator core losses; all parameters refer tpo stator side" annotation(
              Dialog(tab = "Losses"));
            parameter Machines.Losses.StrayLoadParameters strayLoadParameters(PRef = 0, IRef = 100, wRef = 2 * pi * fsNominal / p) "Stray load losses" annotation(
              Dialog(tab = "Losses"));
            annotation(
              defaultComponentName = "inductionMachineData",
              defaultComponentPrefixes = "parameter",
              Documentation(info = "<HTML>
<p>Basic parameters of induction machines are predefined with default values.</p>
</HTML>"));
          end InductionMachineData;

          record AIM_SquirrelCageData "Common parameters for asynchronous induction machines with squirrel cage"
            extends InductionMachineData;
            import Modelica.Constants.pi;
            parameter Modelica.SIunits.Inductance Lm = 3 * sqrt(1 - 0.0667) / (2 * pi * fsNominal) "Stator main field inductance" annotation(
              Dialog(tab = "Nominal resistances and inductances"));
            parameter Modelica.SIunits.Inductance Lrsigma = 3 * (1 - sqrt(1 - 0.0667)) / (2 * pi * fsNominal) "Rotor stray inductance (equivalent three phase winding)" annotation(
              Dialog(tab = "Nominal resistances and inductances"));
            parameter Modelica.SIunits.Resistance Rr = 0.04 "Rotor resistance (equivalent three phase winding) at TRef" annotation(
              Dialog(tab = "Nominal resistances and inductances"));
            parameter Modelica.SIunits.Temperature TrRef = 293.15 "Reference temperature of rotor resistance" annotation(
              Dialog(tab = "Nominal resistances and inductances"));
            parameter Machines.Thermal.LinearTemperatureCoefficient20 alpha20r = 0 "Temperature coefficient of rotor resistance at 20 degC" annotation(
              Dialog(tab = "Nominal resistances and inductances"));
            annotation(
              defaultComponentName = "aimcData",
              defaultComponentPrefixes = "parameter",
              Documentation(info = "<HTML>
<p>Basic parameters of asynchronous induction machines with squirrel cage are predefined with default values.</p>
</HTML>"));
          end AIM_SquirrelCageData;

          record AIM_SlipRingData "Common parameters for asynchronous induction machines with slip ring"
            extends InductionMachineData;
            import Modelica.Constants.pi;
            parameter Modelica.SIunits.Inductance Lm = 3 * sqrt(1 - 0.0667) / (2 * pi * fsNominal) "Stator main field inductance" annotation(
              Dialog(tab = "Nominal resistances and inductances"));
            parameter Modelica.SIunits.Inductance Lrsigma = 3 * (1 - sqrt(1 - 0.0667)) / (2 * pi * fsNominal) / turnsRatio ^ 2 "Rotor stray inductance w.r.t. rotor side" annotation(
              Dialog(tab = "Nominal resistances and inductances"));
            parameter Modelica.SIunits.Inductance Lrzero = Lrsigma / turnsRatio ^ 2 "Rotor zero sequence inductance w.r.t. rotor side" annotation(
              Dialog(tab = "Nominal resistances and inductances"));
            parameter Modelica.SIunits.Resistance Rr = 0.04 / turnsRatio ^ 2 "Rotor resistance per phase at TRef w.r.t. rotor side" annotation(
              Dialog(tab = "Nominal resistances and inductances"));
            parameter Modelica.SIunits.Temperature TrRef = 293.15 "Reference temperature of rotor resistance" annotation(
              Dialog(tab = "Nominal resistances and inductances"));
            parameter Machines.Thermal.LinearTemperatureCoefficient20 alpha20r = 0 "Temperature coefficient of rotor resistance at 20 degC" annotation(
              Dialog(tab = "Nominal resistances and inductances"));
            parameter Boolean useTurnsRatio = true "Use turnsRatio or calculate from locked-rotor voltage?";
            parameter Real turnsRatio(final min = Modelica.Constants.small) = VsNominal / VrLockedRotor * (2 * pi * fsNominal * Lm) / sqrt(Rs ^ 2 + (2 * pi * fsNominal * (Lm + Lssigma)) ^ 2) "Effective number of stator turns / effective number of rotor turns" annotation(
              Dialog(enable = useTurnsRatio));
            parameter Modelica.SIunits.Voltage VsNominal = 100 "Nominal stator voltage per phase" annotation(
              Dialog(enable = not useTurnsRatio));
            parameter Modelica.SIunits.Voltage VrLockedRotor = 100 * (2 * pi * fsNominal * Lm) / sqrt(Rs ^ 2 + (2 * pi * fsNominal * (Lm + Lssigma)) ^ 2) "Locked-rotor voltage per phase" annotation(
              Dialog(enable = not useTurnsRatio));
            parameter Machines.Losses.CoreParameters rotorCoreParameters(final m = 3, PRef = 0, VRef = 1, wRef = 1) "Rotor core losses; all parameters refer to rotor side" annotation(
              Dialog(tab = "Losses"));
            annotation(
              defaultComponentName = "aimsData",
              defaultComponentPrefixes = "parameter",
              Documentation(info = "<HTML>
<p>Basic parameters of asynchronous induction machines with slip ring are predefined with default values.</p>
</HTML>"));
          end AIM_SlipRingData;

          record SM_PermanentMagnetData "Common parameters for synchronous induction machines with permanent magnet"
            extends SM_ReluctanceRotorData(Lmd = 0.3 / (2 * pi * fsNominal), Lmq = 0.3 / (2 * pi * fsNominal));
            import Modelica.Constants.pi;
            parameter Modelica.SIunits.Voltage VsOpenCircuit = 112.3 "Open circuit RMS voltage per phase @ fsNominal";
            parameter Machines.Losses.PermanentMagnetLossParameters permanentMagnetLossParameters(PRef = 0, IRef = 100, wRef = 2 * pi * fsNominal / p) "Permanent magnet loss losses" annotation(
              Dialog(tab = "Losses"));
            annotation(
              defaultComponentName = "smpmData",
              defaultComponentPrefixes = "parameter",
              Documentation(info = "<HTML>
<p>Basic parameters of synchronous induction machines with permanent magnet are predefined with default values.</p>
</HTML>"));
          end SM_PermanentMagnetData;

          record SM_ElectricalExcitedData "Common parameters for synchronous induction machines with electrical excitation"
            extends SM_ReluctanceRotorData(Lmd = 1.5 / (2 * pi * fsNominal), Lmq = 1.5 / (2 * pi * fsNominal));
            import Modelica.Constants.pi;
            parameter Modelica.SIunits.Voltage VsNominal = 100 "Nominal stator RMS voltage per phase" annotation(
              Dialog(tab = "Excitation"));
            parameter Modelica.SIunits.Current IeOpenCircuit = 10 "Open circuit excitation current @ nominal voltage and frequency" annotation(
              Dialog(tab = "Excitation"));
            parameter Modelica.SIunits.Resistance Re = 2.5 "Excitation resistance at TRef" annotation(
              Dialog(tab = "Excitation"));
            parameter Modelica.SIunits.Temperature TeRef = 293.15 "Reference temperature of excitation resistance" annotation(
              Dialog(tab = "Excitation"));
            parameter Machines.Thermal.LinearTemperatureCoefficient20 alpha20e = 0 "Temperature coefficient of excitation resistance" annotation(
              Dialog(tab = "Excitation"));
            parameter Real sigmae(min = 0, max = 0.99) = 0.025 "Stray fraction of total excitation inductance" annotation(
              Dialog(tab = "Excitation"));
            parameter Machines.Losses.BrushParameters brushParameters(V = 0, ILinear = 0.01) "Brush losses" annotation(
              Dialog(tab = "Losses"));
            annotation(
              defaultComponentName = "smeeData",
              defaultComponentPrefixes = "parameter",
              Documentation(info = "<HTML>
<p>Basic parameters of synchronous induction machines with electrical excitation are predefined with default values.</p>
</HTML>"));
          end SM_ElectricalExcitedData;

          record SM_ReluctanceRotorData "Common parameters for synchronous induction machines with reluctance rotor"
            extends InductionMachineData(Lssigma = 0.1 / (2 * pi * fsNominal));
            import Modelica.Constants.pi;
            parameter Modelica.SIunits.Inductance Lmd = 2.9 / (2 * pi * fsNominal) "Stator main field inductance in d-axis" annotation(
              Dialog(tab = "Nominal resistances and inductances"));
            parameter Modelica.SIunits.Inductance Lmq = 0.9 / (2 * pi * fsNominal) "Stator main field inductance in q-axis" annotation(
              Dialog(tab = "Nominal resistances and inductances"));
            parameter Boolean useDamperCage = true "Enable / disable damper cage" annotation(
              Evaluate = true,
              Dialog(tab = "Nominal resistances and inductances", group = "DamperCage"));
            parameter Modelica.SIunits.Inductance Lrsigmad = 0.05 / (2 * pi * fsNominal) "Damper stray inductance in d-axis" annotation(
              Dialog(tab = "Nominal resistances and inductances", group = "DamperCage", enable = useDamperCage));
            parameter Modelica.SIunits.Inductance Lrsigmaq = Lrsigmad "Damper stray inductance in q-axis" annotation(
              Dialog(tab = "Nominal resistances and inductances", group = "DamperCage", enable = useDamperCage));
            parameter Modelica.SIunits.Resistance Rrd = 0.04 "Damper resistance in d-axis at TRef" annotation(
              Dialog(tab = "Nominal resistances and inductances", group = "DamperCage", enable = useDamperCage));
            parameter Modelica.SIunits.Resistance Rrq = Rrd "Damper resistance in q-axis at TRef" annotation(
              Dialog(tab = "Nominal resistances and inductances", group = "DamperCage", enable = useDamperCage));
            parameter Modelica.SIunits.Temperature TrRef = 293.15 "Reference temperature of damper resistances in d- and q-axis" annotation(
              Dialog(tab = "Nominal resistances and inductances", group = "DamperCage", enable = useDamperCage));
            parameter Machines.Thermal.LinearTemperatureCoefficient20 alpha20r = 0 "Temperature coefficient of damper resistances in d- and q-axis" annotation(
              Dialog(tab = "Nominal resistances and inductances", group = "DamperCage", enable = useDamperCage));
            annotation(
              defaultComponentName = "smrData",
              defaultComponentPrefixes = "parameter",
              Documentation(info = "<HTML>
<p>Basic parameters of synchronous induction machines with reluctance rotor are predefined with default values.</p>
</HTML>"));
          end SM_ReluctanceRotorData;

          record DcPermanentMagnetData "Common parameters for DC machines"
            extends Modelica.Icons.Record;
            import Modelica.Constants.pi;
            parameter Modelica.SIunits.Inertia Jr = 0.15 "Rotor's moment of inertia";
            parameter Modelica.SIunits.Inertia Js = Jr "Stator's moment of inertia";
            parameter Modelica.SIunits.Voltage VaNominal = 100 "Nominal armature voltage" annotation(
              Dialog(tab = "Nominal parameters"));
            parameter Modelica.SIunits.Current IaNominal = 100 "Nominal armature current (>0..Motor, <0..Generator)" annotation(
              Dialog(tab = "Nominal parameters"));
            parameter Modelica.SIunits.AngularVelocity wNominal(displayUnit = "1/min") = 1425 * 2 * pi / 60 "Nominal speed" annotation(
              Dialog(tab = "Nominal parameters"));
            parameter Modelica.SIunits.Temperature TaNominal = 293.15 "Nominal armature temperature" annotation(
              Dialog(tab = "Nominal parameters"));
            parameter Modelica.SIunits.Resistance Ra = 0.05 "Armature resistance at TRef" annotation(
              Dialog(tab = "Nominal resistances and inductances"));
            parameter Modelica.SIunits.Temperature TaRef = 293.15 "Reference temperature of armature resistance" annotation(
              Dialog(tab = "Nominal resistances and inductances"));
            parameter Machines.Thermal.LinearTemperatureCoefficient20 alpha20a = 0 "Temperature coefficient of armature resistance" annotation(
              Dialog(tab = "Nominal resistances and inductances"));
            parameter Modelica.SIunits.Inductance La = 0.0015 "Armature inductance" annotation(
              Dialog(tab = "Nominal resistances and inductances"));
            parameter Machines.Losses.FrictionParameters frictionParameters(PRef = 0, wRef = wNominal) "Friction losses" annotation(
              Dialog(tab = "Losses"));
            parameter Modelica.SIunits.Voltage ViNominal = VaNominal - Machines.Thermal.convertResistance(Ra, TaRef, alpha20a, TaNominal) * IaNominal - Machines.Losses.DCMachines.brushVoltageDrop(brushParameters, IaNominal);
            parameter Machines.Losses.CoreParameters coreParameters(final m = 1, PRef = 0, VRef = ViNominal, wRef = wNominal) "Armature core losses" annotation(
              Dialog(tab = "Losses"));
            parameter Machines.Losses.StrayLoadParameters strayLoadParameters(PRef = 0, IRef = IaNominal, wRef = wNominal) "Stray load losses" annotation(
              Dialog(tab = "Losses"));
            parameter Machines.Losses.BrushParameters brushParameters(V = 0, ILinear = 0.01 * IaNominal) "Brush losses" annotation(
              Dialog(tab = "Losses"));
            annotation(
              defaultComponentName = "dcpmData",
              defaultComponentPrefixes = "parameter",
              Documentation(info = "<HTML>
<p>Basic parameters of DC machines are predefined with default values.</p>
</HTML>"));
          end DcPermanentMagnetData;

          record DcElectricalExcitedData "Common parameters for DC machines"
            extends DcPermanentMagnetData;
            parameter Modelica.SIunits.Current IeNominal = 1 "Nominal excitation current" annotation(
              Dialog(tab = "Excitation"));
            parameter Modelica.SIunits.Resistance Re = 100 "Field excitation resistance at TRef" annotation(
              Dialog(tab = "Excitation"));
            parameter Modelica.SIunits.Temperature TeRef = 293.15 "Reference temperature of excitation resistance" annotation(
              Dialog(tab = "Excitation"));
            parameter Machines.Thermal.LinearTemperatureCoefficient20 alpha20e = 0 "Temperature coefficient of excitation resistance" annotation(
              Dialog(tab = "Excitation"));
            parameter Modelica.SIunits.Inductance Le = 1 "Total field excitation inductance" annotation(
              Dialog(tab = "Excitation"));
            parameter Real sigmae(min = 0, max = 0.99) = 0 "Stray fraction of total excitation inductance" annotation(
              Dialog(tab = "Excitation"));
            annotation(
              defaultComponentName = "dceeData",
              defaultComponentPrefixes = "parameter",
              Documentation(info = "<HTML>
<p>Basic parameters of DC machines are predefined with default values.</p>
</HTML>"));
          end DcElectricalExcitedData;

          record DcSeriesExcitedData "Common parameters for DC machines"
            extends DcPermanentMagnetData(wNominal = 1410 * 2 * pi / 60);
            import Modelica.Constants.pi;
            parameter Modelica.SIunits.Resistance Re = 0.01 "Series excitation resistance at TRef" annotation(
              Dialog(tab = "Excitation"));
            parameter Modelica.SIunits.Temperature TeRef = 293.15 "Reference temperature of excitation resistance" annotation(
              Dialog(tab = "Excitation"));
            parameter Machines.Thermal.LinearTemperatureCoefficient20 alpha20e = 0 "Temperature coefficient of excitation resistance" annotation(
              Dialog(tab = "Excitation"));
            parameter Modelica.SIunits.Inductance Le = 0.0005 "Total field excitation inductance" annotation(
              Dialog(tab = "Excitation"));
            parameter Real sigmae(min = 0, max = 0.99) = 0 "Stray fraction of total excitation inductance" annotation(
              Dialog(tab = "Excitation"));
            parameter Modelica.SIunits.Temperature TeNominal = 293.15 "Nominal series excitation temperature" annotation(
              Dialog(tab = "Nominal parameters"));
            annotation(
              defaultComponentName = "dcseData",
              defaultComponentPrefixes = "parameter",
              Documentation(info = "<HTML>
<p>Basic parameters of DC machines are predefined with default values.</p>
</HTML>"));
          end DcSeriesExcitedData;

          record TransformerData "Common parameters for transformers"
            extends Modelica.Icons.Record;
            parameter Real n_phase = 1 "Ratio primary voltage per phase / secondary voltage per phase";
            parameter String C1 = "Y" "Choose primary connection" annotation(
              choices(choice = "Y" "Star connection", choice = "D" "Delta connection"));
            parameter String C2 = "y" "Choose secondary connection" annotation(
              choices(choice = "y" "Star connection", choice = "d" "Delta connection", choice = "z" "Zig-zag connection"));
            parameter Real n = n_phase * (if C1 == "D" and (C2 == "y" or C2 == "z") then 1 / sqrt(3) else if C1 == "Y" and C2 == "d" then sqrt(3) else 1) "Ratio primary voltage (line-to-line) / secondary voltage (line-to-line)" annotation(
              Dialog(enable = false));
            parameter Modelica.SIunits.Resistance R1 = 5E-3 / 3 "Primary resistance per phase at TRef" annotation(
              Dialog(tab = "Nominal resistances and inductances"));
            parameter Modelica.SIunits.Temperature T1Ref = 293.15 "Reference temperature of primary resistance" annotation(
              Dialog(tab = "Nominal resistances and inductances"));
            parameter Machines.Thermal.LinearTemperatureCoefficient20 alpha20_1 = 0 "Temperature coefficient of primary resistance at 20 degC" annotation(
              Dialog(tab = "Nominal resistances and inductances"));
            parameter Modelica.SIunits.Inductance L1sigma = 78E-6 / 3 "Primary stray inductance per phase" annotation(
              Dialog(tab = "Nominal resistances and inductances"));
            parameter Modelica.SIunits.Resistance R2 = 5E-3 / 3 "Secondary resistance per phase at TRef" annotation(
              Dialog(tab = "Nominal resistances and inductances"));
            parameter Modelica.SIunits.Temperature T2Ref = 293.15 "Reference temperature of secondary resistance" annotation(
              Dialog(tab = "Nominal resistances and inductances"));
            parameter Machines.Thermal.LinearTemperatureCoefficient20 alpha20_2 = 0 "Temperature coefficient of secondary resistance at 20 degC" annotation(
              Dialog(tab = "Nominal resistances and inductances"));
            parameter Modelica.SIunits.Inductance L2sigma = 78E-6 / 3 "Secondary stray inductance per phase" annotation(
              Dialog(tab = "Nominal resistances and inductances"));
            annotation(
              defaultComponentName = "transformerData",
              defaultComponentPrefixes = "parameter",
              Documentation(info = "<HTML>
<p>Basic parameters of transformers are predefined with default values.</p>
<p>Note: ratio n is defined between primary / secondary line-to-line voltage;
   therefore the user has to take into account primary and secondary connection!</p>
</HTML>"));
          end TransformerData;
        end ParameterRecords;

        block VfController "Voltage-Frequency-Controller"
          extends Modelica.Blocks.Interfaces.SIMO(final nout = m);
          constant Modelica.SIunits.Angle pi = Modelica.Constants.pi;
          parameter Integer m = 3 "Number of phases";
          parameter Modelica.SIunits.Angle orientation[m] = -Modelica.Electrical.MultiPhase.Functions.symmetricOrientation(m) "Orientation of phases";
          parameter Modelica.SIunits.Voltage VNominal "Nominal RMS voltage per phase";
          parameter Modelica.SIunits.Frequency fNominal "Nominal frequency";
          parameter Modelica.SIunits.Angle BasePhase = 0 "Common phase shift";
          output Modelica.SIunits.Angle x(start = 0, fixed = true) "Integrator state";
          output Modelica.SIunits.Voltage amplitude;
        equation
//amplitude = sqrt(2)*VNominal*min(abs(u)/fNominal, 1);
          amplitude = sqrt(2) * VNominal * (if abs(u) < fNominal then abs(u) / fNominal else 1);
          der(x) = 2 * pi * u;
          y = amplitude * sin(fill(x + BasePhase, m) + orientation);
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-100, -100}, {0, 60}, {80, 60}}, color = {0, 0, 255}), Line(points = {{-70, 0}, {-60.2, 29.9}, {-53.8, 46.5}, {-48.2, 58.1}, {-43.3, 65.2}, {-38.3, 69.2}, {-33.4, 69.8}, {-28.5, 67}, {-23.6, 61}, {-18.6, 52}, {-13, 38.6}, {-5.98, 18.6}, {8.79, -26.9}, {15.1, -44}, {20.8, -56.2}, {25.7, -64}, {30.6, -68.6}, {35.5, -70}, {40.5, -67.9}, {45.4, -62.5}, {50.3, -54.1}, {55.9, -41.3}, {63, -21.7}, {70, 0}}, color = {192, 192, 192}, smooth = Smooth.Bezier), Line(points = {{-40, 0}, {-30.2, 29.9}, {-23.8, 46.5}, {-18.2, 58.1}, {-13.3, 65.2}, {-8.3, 69.2}, {-3.4, 69.8}, {1.5, 67}, {6.4, 61}, {11.4, 52}, {17, 38.6}, {24.02, 18.6}, {38.79, -26.9}, {45.1, -44}, {50.8, -56.2}, {55.7, -64}, {60.6, -68.6}, {65.5, -70}, {70.5, -67.9}, {75.4, -62.5}, {80.3, -54.1}, {85.9, -41.3}, {93, -21.7}, {100, 0}}, color = {192, 192, 192}, smooth = Smooth.Bezier), Line(points = {{-100, 0}, {-90.2, 29.9}, {-83.8, 46.5}, {-78.2, 58.1}, {-73.3, 65.2}, {-68.3, 69.2}, {-63.4, 69.8}, {-58.5, 67}, {-53.6, 61}, {-48.6, 52}, {-43, 38.6}, {-35.98, 18.6}, {-21.21, -26.9}, {-14.9, -44}, {-9.2, -56.2}, {-4.3, -64}, {0.6, -68.6}, {5.5, -70}, {10.5, -67.9}, {15.4, -62.5}, {20.3, -54.1}, {25.9, -41.3}, {33, -21.7}, {40, 0}}, color = {192, 192, 192}, smooth = Smooth.Bezier)}),
            Documentation(info = "<HTML>
Simple Voltage-Frequency-Controller.<br>
Amplitude of voltage is linear dependent (VNominal/fNominal) on frequency (input signal \"u\"), but limited by VNominal (nominal RMS voltage per phase).<br>
m sine-waves with amplitudes as described above are provided as output signal \"y\".<br>
The sine-waves are intended to feed a m-phase SignalVoltage.<br>
Phase shifts between sine-waves may be chosen by the user; default values are <i>(k-1)/m*pi for k in 1:m</i>.
</HTML>"));
        end VfController;

        model CurrentController "Current controller"
          constant Integer m = 3 "Number of phases";
          parameter Integer p "Number of pole pairs";
          extends Modelica.Blocks.Interfaces.MO(final nout = m);
          Modelica.Blocks.Interfaces.RealInput id_rms annotation(
            Placement(transformation(extent = {{-140, 40}, {-100, 80}}, rotation = 0)));
          Modelica.Blocks.Interfaces.RealInput iq_rms annotation(
            Placement(transformation(extent = {{-140, -80}, {-100, -40}}, rotation = 0)));
          Modelica.Blocks.Interfaces.RealInput phi annotation(
            Placement(transformation(origin = {0, -120}, extent = {{20, -20}, {-20, 20}}, rotation = 270)));
          Modelica.Blocks.Math.Gain toPeak_d(k = sqrt(2)) annotation(
            Placement(transformation(extent = {{-60, 50}, {-40, 70}}, rotation = 0)));
          Modelica.Blocks.Math.Gain toPeak_q(k = sqrt(2)) annotation(
            Placement(transformation(extent = {{-60, -70}, {-40, -50}}, rotation = 0)));
          Modelica.Blocks.Math.Gain toGamma(k = -p) annotation(
            Placement(transformation(origin = {0, -50}, extent = {{10, -10}, {-10, 10}}, rotation = 270)));
          Machines.SpacePhasors.Blocks.Rotator rotator annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0)));
          Modelica.Blocks.Sources.Constant i0(k = 0) annotation(
            Placement(transformation(extent = {{-10, 50}, {10, 30}}, rotation = 0)));
          Machines.SpacePhasors.Blocks.FromSpacePhasor fromSpacePhasor annotation(
            Placement(transformation(extent = {{40, 10}, {60, -10}}, rotation = 0)));
        equation
          connect(iq_rms, toPeak_q.u) annotation(
            Line(points = {{-120, -60}, {-62, -60}}, color = {0, 0, 127}));
          connect(phi, toGamma.u) annotation(
            Line(points = {{0, -120}, {0, -62}}, color = {0, 0, 127}));
          connect(rotator.angle, toGamma.y) annotation(
            Line(points = {{0, -12}, {0, -39}, {0, -39}}, color = {0, 0, 127}));
          connect(rotator.y, fromSpacePhasor.u) annotation(
            Line(points = {{11, 0}, {24, 0}, {38, 0}}, color = {0, 0, 127}));
          connect(toPeak_d.u, id_rms) annotation(
            Line(points = {{-62, 60}, {-120, 60}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(toPeak_d.y, rotator.u[1]) annotation(
            Line(points = {{-39, 60}, {-30, 60}, {-30, -1}, {-12, -1}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(toPeak_q.y, rotator.u[2]) annotation(
            Line(points = {{-39, -60}, {-30, -60}, {-30, 1}, {-12, 1}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(i0.y, fromSpacePhasor.zero) annotation(
            Line(points = {{11, 40}, {20, 40}, {20, 8}, {38, 8}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(fromSpacePhasor.y, y) annotation(
            Line(points = {{61, 0}, {110, 0}}, color = {0, 0, 127}, smooth = Smooth.None));
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-100, 60}, {20, 40}}, lineColor = {0, 0, 255}, textString = "id_rms"), Text(extent = {{-100, -40}, {20, -60}}, lineColor = {0, 0, 255}, textString = "iq_rms")}),
            Documentation(info = "<html>
<p>
Simple Current-Controller.
</p>
<p>
The desired rms values of d- and q-component of the space phasor current in rotor fixed coordinate system are given by inputs \"id_rms\" and \"iq_rms\".
Using the given rotor position (input \"phi\"), the correct three-phase currents (output \"y[3]\") are calculated.
They can be used to feed a current source which in turn feeds an induction machine.
</p>
</HTML>"));
        end CurrentController;

        model VoltageController "Voltage controller"
          import Modelica.Constants.pi;
          constant Integer m = 3 "Number of phases";
          parameter Integer p "Number of pole pairs";
          parameter Modelica.SIunits.Frequency fsNominal "Nominal frequency";
          parameter Modelica.SIunits.Voltage VsOpenCircuit "Open circuit RMS voltage per phase @ fsNominal";
          parameter Modelica.SIunits.Resistance Rs "Stator resistance per phase";
          parameter Modelica.SIunits.Inductance Ld "Inductance in d-axis";
          parameter Modelica.SIunits.Inductance Lq "Inductance in q-axis";
          //Decoupling
          parameter Boolean decoupling = false "Use decoupling network";
          final parameter Modelica.SIunits.MagneticFlux psiM = sqrt(2) * VsOpenCircuit / (2 * pi * fsNominal);
          Modelica.SIunits.AngularVelocity omega = p * der(phi);
          Modelica.SIunits.Voltage Vd = sqrt(2) * (Rs * id_rms - omega * Lq * iq_rms);
          Modelica.SIunits.Voltage Vq = sqrt(2) * (Rs * iq_rms + omega * Ld * id_rms) + omega * psiM;
          extends Modelica.Blocks.Interfaces.MO(final nout = m);
          Modelica.Blocks.Interfaces.RealInput id_rms annotation(
            Placement(transformation(extent = {{-140, 40}, {-100, 80}}, rotation = 0)));
          Modelica.Blocks.Interfaces.RealInput iq_rms annotation(
            Placement(transformation(extent = {{-140, -80}, {-100, -40}}, rotation = 0)));
          Modelica.Blocks.Interfaces.RealInput phi annotation(
            Placement(transformation(origin = {60, -120}, extent = {{20, -20}, {-20, 20}}, rotation = 270)));
          Modelica.Blocks.Interfaces.RealInput iActual[3] annotation(
            Placement(transformation(origin = {-60, -120}, extent = {{20, -20}, {-20, 20}}, rotation = 270)));
          Machines.Utilities.FromDQ fromDQ(final p = p) annotation(
            Placement(transformation(extent = {{70, -10}, {90, 10}})));
          Machines.Utilities.ToDQ toDQ(final p = p) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 90, origin = {-60, -80})));
          Modelica.Blocks.Math.Gain toPeak_d(final k = sqrt(2)) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0, origin = {-70, 60})));
          Modelica.Blocks.Math.Gain toPeak_q(final k = sqrt(2)) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0, origin = {-70, 0})));
          Modelica.Blocks.Math.Feedback feedback_d annotation(
            Placement(transformation(extent = {{-38, 50}, {-18, 70}})));
          Modelica.Blocks.Math.Feedback feedback_q annotation(
            Placement(transformation(extent = {{-40, -10}, {-20, 10}})));
          Modelica.Blocks.Continuous.PI PI_d(final k = unitResistance / Rs, final T = Ld / Rs, initType = Modelica.Blocks.Types.Init.InitialOutput) annotation(
            Placement(transformation(extent = {{-10, 50}, {10, 70}})));
          Modelica.Blocks.Continuous.PI PI_q(final k = unitResistance / Rs, final T = Lq / Rs, initType = Modelica.Blocks.Types.Init.InitialOutput) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}})));
          Modelica.Blocks.Math.Add add[2](final k1 = fill(+1, 2), final k2 = fill(if decoupling then +1 else 0, 2)) annotation(
            Placement(transformation(extent = {{32, -10}, {52, 10}})));
          Modelica.Blocks.Sources.RealExpression deCoupling[2](y = {Vd, Vq}) annotation(
            Placement(transformation(extent = {{-10, -40}, {10, -20}})));
        protected
          constant Modelica.SIunits.Resistance unitResistance = 1 annotation(
            HideResult = true);
        equation
          connect(fromDQ.y, y) annotation(
            Line(points = {{91, 0}, {110, 0}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(phi, fromDQ.phi) annotation(
            Line(points = {{60, -120}, {60, -80}, {80, -80}, {80, -12}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(iActual, toDQ.u) annotation(
            Line(points = {{-60, -120}, {-60, -92}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(phi, toDQ.phi) annotation(
            Line(points = {{60, -120}, {60, -80}, {-48, -80}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(toPeak_d.u, id_rms) annotation(
            Line(points = {{-82, 60}, {-120, 60}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(toPeak_q.u, iq_rms) annotation(
            Line(points = {{-82, 0}, {-90, 0}, {-90, -60}, {-120, -60}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(toPeak_q.y, feedback_q.u1) annotation(
            Line(points = {{-59, 0}, {-38, 0}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(toPeak_d.y, feedback_d.u1) annotation(
            Line(points = {{-59, 60}, {-36, 60}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(feedback_d.y, PI_d.u) annotation(
            Line(points = {{-19, 60}, {-12, 60}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(feedback_q.y, PI_q.u) annotation(
            Line(points = {{-21, 0}, {-12, 0}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(toDQ.y[1], feedback_d.u2) annotation(
            Line(points = {{-59.5, -69}, {-59.5, -60}, {-50, -60}, {-50, 40}, {-28, 40}, {-28, 52}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(toDQ.y[2], feedback_q.u2) annotation(
            Line(points = {{-60.5, -69}, {-60.5, -60}, {-50, -60}, {-50, -20}, {-30, -20}, {-30, -8}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(add.y, fromDQ.u) annotation(
            Line(points = {{53, 0}, {68, 0}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(PI_d.y, add[1].u1) annotation(
            Line(points = {{11, 60}, {20, 60}, {20, 6}, {30, 6}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(PI_q.y, add[2].u1) annotation(
            Line(points = {{11, 0}, {20, 0}, {20, 6}, {30, 6}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(deCoupling.y, add.u2) annotation(
            Line(points = {{11, -30}, {20, -30}, {20, -6}, {30, -6}}, color = {0, 0, 127}, smooth = Smooth.None));
          annotation(
            Icon(graphics = {Text(extent = {{-100, 60}, {20, 40}}, lineColor = {0, 0, 255}, textString = "id_rms"), Text(extent = {{-100, -40}, {20, -60}}, lineColor = {0, 0, 255}, textString = "iq_rms")}),
            Documentation(info = "<html>
<p>
Simple Voltage-Controller
</p>
<p>
The desired rms values of d- and q-component of the space phasor current in rotor fixed coordinate system are given by inputs \"id_rms\" and \"iq_rms\".
Using the given rotor position (input \"phi\"), the actual threephase currents are measured and transformed to the d-q coordinate system.
Two PI-controller determine the necessary d- and q- voltages, which are transformed back to threephase (output \"y[3]\").
They can be used to feed a voltage source which in turn feeds a permanent magnet synchronous machine.
</p>
<p>
Note: No care is taken for current or voltage limiting, as well as for field weakening.
</p>
</html>"));
        end VoltageController;

        block ToDQ
          extends Modelica.Blocks.Icons.Block;
          parameter Integer p "Number of pole pairs";
          Modelica.Blocks.Math.Gain toGamma(final k = p) annotation(
            Placement(transformation(origin = {0, -50}, extent = {{10, -10}, {-10, 10}}, rotation = 270)));
          Machines.SpacePhasors.Blocks.Rotator rotator2 annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0)));
          Modelica.Blocks.Interfaces.RealOutput y[2] "Connector of Real output signals" annotation(
            Placement(transformation(extent = {{100, -10}, {120, 10}}, rotation = 0)));
          Modelica.Blocks.Interfaces.RealInput phi annotation(
            Placement(transformation(origin = {0, -120}, extent = {{20, -20}, {-20, 20}}, rotation = 270)));
          Modelica.Blocks.Interfaces.RealInput u[3] "Connector of Real input signals" annotation(
            Placement(transformation(extent = {{-140, -20}, {-100, 20}})));
          Machines.SpacePhasors.Blocks.ToSpacePhasor toSpacePhasor annotation(
            Placement(transformation(extent = {{-60, -10}, {-40, 10}}, rotation = 0)));
        equation
          connect(phi, toGamma.u) annotation(
            Line(points = {{0, -120}, {-2.22045e-015, -120}, {-2.22045e-015, -62}}, color = {0, 0, 127}));
          connect(rotator2.angle, toGamma.y) annotation(
            Line(points = {{0, -12}, {0, -39}, {2.22045e-015, -39}}, color = {0, 0, 127}));
          connect(rotator2.y, y) annotation(
            Line(points = {{11, 0}, {110, 0}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(u, toSpacePhasor.u) annotation(
            Line(points = {{-120, 0}, {-62, 0}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(toSpacePhasor.y, rotator2.u) annotation(
            Line(points = {{-39, 0}, {-12, 0}}, color = {0, 0, 127}, smooth = Smooth.None));
          annotation(
            Documentation(info = "<html>
<p>
Threepase values <code>u[3]</code> are transformed to the corresponding spacePhasor which is rotated to the rotor-fixed coordinate system,
using the provided mechanical rotor angle phi. The ouput is the rsulting d- / q- components <code>y[2]</code>.
</p>
</html>"));
        end ToDQ;

        block FromDQ
          extends Modelica.Blocks.Icons.Block;
          parameter Integer p "Number of pole pairs";
          Modelica.Blocks.Math.Gain toGamma(final k = -p) annotation(
            Placement(transformation(origin = {0, -50}, extent = {{10, -10}, {-10, 10}}, rotation = 270)));
          Machines.SpacePhasors.Blocks.Rotator rotator2 annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0)));
          Modelica.Blocks.Sources.Constant i0(final k = 0) annotation(
            Placement(transformation(extent = {{20, -20}, {40, -40}}, rotation = 0)));
          Machines.SpacePhasors.Blocks.FromSpacePhasor fromSpacePhasor annotation(
            Placement(transformation(extent = {{60, -10}, {80, 10}}, rotation = 0)));
          Modelica.Blocks.Interfaces.RealOutput y[3] "Connector of Real output signals" annotation(
            Placement(transformation(extent = {{100, -10}, {120, 10}}, rotation = 0)));
          Modelica.Blocks.Interfaces.RealInput phi annotation(
            Placement(transformation(origin = {0, -120}, extent = {{20, -20}, {-20, 20}}, rotation = 270)));
          Modelica.Blocks.Interfaces.RealInput u[2] "Connector of Real input signals" annotation(
            Placement(transformation(extent = {{-140, -20}, {-100, 20}})));
        equation
          connect(phi, toGamma.u) annotation(
            Line(points = {{0, -120}, {-2.22045e-015, -120}, {-2.22045e-015, -62}}, color = {0, 0, 127}));
          connect(rotator2.angle, toGamma.y) annotation(
            Line(points = {{0, -12}, {0, -39}, {2.22045e-015, -39}}, color = {0, 0, 127}));
          connect(rotator2.y, fromSpacePhasor.u) annotation(
            Line(points = {{11, 0}, {34, 0}, {58, 0}}, color = {0, 0, 127}));
          connect(i0.y, fromSpacePhasor.zero) annotation(
            Line(points = {{41, -30}, {50, -30}, {50, -8}, {58, -8}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(fromSpacePhasor.y, y) annotation(
            Line(points = {{81, 0}, {110, 0}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(rotator2.u, u) annotation(
            Line(points = {{-12, 0}, {-120, 0}}, color = {0, 0, 127}, smooth = Smooth.None));
          annotation(
            Documentation(info = "<html>
<p>
The d- / q- components <code>u[2]</code> are rotated back to the stator-fixed coordinate system,
using the provided mechanical rotor angle phi. The output is the result of the back-transformation to threephase values <code>y[3]</code>.
</p>
</html>"));
        end FromDQ;

        model SwitchYD "Y-D-switch"
          parameter Integer m = 3 "Number of phases";
          Modelica.Electrical.MultiPhase.Interfaces.PositivePlug plugSupply(final m = m) "To grid" annotation(
            Placement(transformation(extent = {{-10, 90}, {10, 110}}, rotation = 0)));
          Modelica.Electrical.MultiPhase.Interfaces.PositivePlug plug_sp(final m = m) "To positive stator plug" annotation(
            Placement(transformation(extent = {{50, -90}, {70, -110}}, rotation = 0)));
          Modelica.Electrical.MultiPhase.Interfaces.NegativePlug plug_sn(final m = m) "To negative stator plug" annotation(
            Placement(transformation(extent = {{-70, -90}, {-50, -110}}, rotation = 0)));
          Modelica.Electrical.MultiPhase.Basic.Star star(final m = m) annotation(
            Placement(transformation(extent = {{20, -50}, {40, -30}}, rotation = 0)));
          Modelica.Electrical.MultiPhase.Basic.Delta delta(final m = m) annotation(
            Placement(transformation(extent = {{40, -80}, {20, -60}}, rotation = 0)));
          Modelica.Electrical.MultiPhase.Ideal.IdealCommutingSwitch idealCommutingSwitch(final m = m, Ron = fill(1e-5, m), Goff = fill(1e-5, m)) annotation(
            Placement(transformation(extent = {{-10, -70}, {10, -50}}, rotation = 0)));
          Modelica.Blocks.Interfaces.BooleanInput control[m] annotation(
            Placement(transformation(extent = {{-120, -10}, {-100, 10}}, rotation = 0)));
        equation
          connect(delta.plug_p, plugSupply) annotation(
            Line(points = {{40, -70}, {60, -70}, {60, 100}, {0, 100}}, color = {0, 0, 255}));
          connect(delta.plug_p, plug_sp) annotation(
            Line(points = {{40, -70}, {60, -70}, {60, -100}}, color = {0, 0, 255}));
          connect(idealCommutingSwitch.plug_n2, delta.plug_n) annotation(
            Line(points = {{10, -60}, {10, -70}, {20, -70}}, color = {0, 0, 255}));
          connect(idealCommutingSwitch.plug_n1, star.plug_p) annotation(
            Line(points = {{10, -55}, {10, -40}, {20, -40}}, color = {0, 0, 255}));
          connect(idealCommutingSwitch.plug_p, plug_sn) annotation(
            Line(points = {{-10, -60}, {-60, -60}, {-60, -100}}, color = {0, 0, 255}));
          connect(control, idealCommutingSwitch.control) annotation(
            Line(points = {{-110, 0}, {0, 0}, {0, -52}}, color = {255, 0, 255}));
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{-100, -60}, {-60, -100}, {58, -100}, {100, -60}, {100, 60}, {60, 100}, {-60, 100}, {-100, 60}, {-100, -60}}, lineColor = {175, 175, 175}, smooth = Smooth.None, fillColor = {215, 215, 215}, fillPattern = FillPattern.Solid), Line(points = {{-80, 52}, {-40, 0}, {-2, 52}, {-40, 0}, {-40, -52}}, color = {0, 0, 255}, thickness = 0.5), Line(points = {{0, 40}, {0, -40}, {80, 0}, {0, 40}}, color = {0, 0, 255}, thickness = 0.5)}),
            Documentation(info = "<HTML>
Simple Star-Delta-switch.<br>
If <i>control</i> is false, plug_sp and plug_sn are star connected and plug_sp connected to the supply plug.<br>
If <i>control</i> is true, plug_sp and plug_sn are delta connected and they are connected to the supply plug.
</HTML>"));
        end SwitchYD;

        model TerminalBox "Terminal box Y/D-connection"
          parameter Integer m = 3 "Number of phases";
          parameter String terminalConnection(start = "Y") "Choose Y=star/D=delta" annotation(
            choices(choice = "Y" "Star connection", choice = "D" "Delta connection"));
          Modelica.Electrical.MultiPhase.Interfaces.PositivePlug plug_sp(final m = m) "To positive stator plug" annotation(
            Placement(transformation(extent = {{50, -90}, {70, -110}}, rotation = 0)));
          Modelica.Electrical.MultiPhase.Interfaces.NegativePlug plug_sn(final m = m) "To negative stator plug" annotation(
            Placement(transformation(extent = {{-70, -90}, {-50, -110}}, rotation = 0)));
          Modelica.Electrical.MultiPhase.Basic.Star star(final m = m) if terminalConnection <> "D" annotation(
            Placement(transformation(origin = {-70, -80}, extent = {{-10, 10}, {10, -10}}, rotation = 180)));
          Modelica.Electrical.MultiPhase.Basic.Delta delta(final m = m) if terminalConnection == "D" annotation(
            Placement(transformation(extent = {{-20, -70}, {-40, -50}}, rotation = 0)));
          Modelica.Electrical.MultiPhase.Interfaces.PositivePlug plugSupply(final m = m) "To grid" annotation(
            Placement(transformation(extent = {{-10, -70}, {10, -90}}, rotation = 0)));
          Modelica.Electrical.Analog.Interfaces.NegativePin starpoint if terminalConnection <> "D" annotation(
            Placement(transformation(extent = {{-100, -90}, {-80, -70}}, rotation = 0)));
        equation
          connect(plug_sn, star.plug_p) annotation(
            Line(points = {{-60, -100}, {-60, -80}}, color = {0, 0, 255}));
          connect(plug_sn, delta.plug_n) annotation(
            Line(points = {{-60, -100}, {-40, -100}, {-40, -60}}, color = {0, 0, 255}));
          connect(delta.plug_p, plug_sp) annotation(
            Line(points = {{-20, -60}, {60, -60}, {60, -100}}, color = {0, 0, 255}));
          connect(plug_sp, plugSupply) annotation(
            Line(points = {{60, -100}, {0, -100}, {0, -80}}, color = {0, 0, 255}));
          connect(star.pin_n, starpoint) annotation(
            Line(points = {{-80, -80}, {-90, -80}}, color = {0, 0, 255}));
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{-80, -80}, {-80, -84}, {-80, -120}, {-40, -140}, {40, -140}, {80, -110}, {80, -84}, {76, -80}, {-80, -80}}, lineColor = {95, 95, 95}, fillColor = {135, 135, 135}, fillPattern = FillPattern.Solid), Text(extent = {{-40, -90}, {40, -130}}, lineColor = {0, 0, 0}, textString = "%terminalConnection")}),
            Documentation(info = "<html>
TerminalBox: at the bottom connected to both machine plugs, connect at the top to the grid as usual,<br>
choosing Y-connection (StarDelta=Y) or D-connection (StarDelta=D).
</html>"));
        end TerminalBox;

        model SwitchedRheostat "Rheostat which is shortened after a given time"
          parameter Integer m = 3 "Number of phases";
          Modelica.Electrical.MultiPhase.Interfaces.PositivePlug plug_p(final m = m) "To positive rotor plug" annotation(
            Placement(transformation(extent = {{90, 70}, {110, 50}}, rotation = 0)));
          Modelica.Electrical.MultiPhase.Interfaces.NegativePlug plug_n(final m = m) "To negative rotor plug" annotation(
            Placement(transformation(extent = {{90, -50}, {110, -70}}, rotation = 0)));
          parameter Modelica.SIunits.Resistance RStart "Starting resistance";
          parameter Modelica.SIunits.Time tStart "Duration of switching on the starting resistor";
          Modelica.Electrical.MultiPhase.Basic.Star star(final m = m) annotation(
            Placement(transformation(extent = {{-40, -70}, {-60, -50}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Ground ground annotation(
            Placement(transformation(origin = {-80, -60}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          Modelica.Electrical.MultiPhase.Ideal.IdealCommutingSwitch idealCommutingSwitch(final m = m, Ron = fill(1e-5, m), Goff = fill(1e-5, m)) annotation(
            Placement(transformation(origin = {40, 20}, extent = {{-10, 10}, {10, -10}}, rotation = 270)));
          Modelica.Electrical.MultiPhase.Basic.Resistor rheostat(final m = m, final R = fill(RStart, m)) annotation(
            Placement(transformation(extent = {{0, -30}, {-20, -10}}, rotation = 0)));
          Modelica.Electrical.MultiPhase.Basic.Star starRheostat(final m = m) annotation(
            Placement(transformation(extent = {{-40, -30}, {-60, -10}}, rotation = 0)));
          Modelica.Blocks.Sources.BooleanStep booleanStep[m](final startTime = fill(tStart, m), final startValue = fill(false, m)) annotation(
            Placement(transformation(extent = {{-60, 10}, {-40, 30}}, rotation = 0)));
        equation
          connect(plug_p, idealCommutingSwitch.plug_p) annotation(
            Line(points = {{100, 60}, {40, 60}, {40, 30}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(idealCommutingSwitch.plug_n2, plug_n) annotation(
            Line(points = {{40, 10}, {40, -60}, {100, -60}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(rheostat.plug_p, idealCommutingSwitch.plug_n1) annotation(
            Line(points = {{0, -20}, {35, -20}, {35, 10}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(idealCommutingSwitch.plug_n2, star.plug_p) annotation(
            Line(points = {{40, 10}, {40, -60}, {-40, -60}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(rheostat.plug_n, starRheostat.plug_p) annotation(
            Line(points = {{-20, -20}, {-40, -20}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(starRheostat.pin_n, star.pin_n) annotation(
            Line(points = {{-60, -20}, {-60, -60}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(star.pin_n, ground.p) annotation(
            Line(points = {{-60, -60}, {-70, -60}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(booleanStep.y, idealCommutingSwitch.control) annotation(
            Line(points = {{-39, 20}, {32, 20}}, color = {255, 0, 255}, smooth = Smooth.None));
          annotation(
            Icon(graphics = {Rectangle(extent = {{26, 40}, {54, -40}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{100, 60}, {-40, 60}, {-40, 40}}, color = {0, 0, 255}), Line(points = {{100, -60}, {-40, -60}, {-40, -40}}, color = {0, 0, 255}), Ellipse(extent = {{-44, 40}, {-36, 32}}, lineColor = {0, 0, 255}), Ellipse(extent = {{-44, -32}, {-36, -40}}, lineColor = {0, 0, 255}), Line(points = {{-80, 40}, {-42, -34}}, color = {0, 0, 255}), Line(points = {{40, 40}, {40, 42}, {40, 60}}, color = {0, 0, 255}), Line(points = {{40, -40}, {40, -60}}, color = {0, 0, 255}), Line(points = {{10, -80}, {70, -80}}, color = {0, 0, 255}), Line(points = {{40, -60}, {40, -80}}, color = {0, 0, 255}), Line(points = {{20, -90}, {60, -90}}, color = {0, 0, 255}), Line(points = {{30, -100}, {50, -100}}, color = {0, 0, 255})}),
            Documentation(info = "<HTML>
<p>Switched rheostat, used for starting asynchronous induction motors with slipring rotor:</p>
<p>The external rotor resistance <code>RStart</code> is shortened at time <code>tStart</code>.</p>
</HTML>"));
        end SwitchedRheostat;

        model RampedRheostat "Rheostat with linearly decreasing resistance"
          parameter Integer m = 3 "Number of phases";
          Modelica.Electrical.MultiPhase.Interfaces.PositivePlug plug_p(final m = m) "To positive rotor plug" annotation(
            Placement(transformation(extent = {{90, 70}, {110, 50}}, rotation = 0)));
          Modelica.Electrical.MultiPhase.Interfaces.NegativePlug plug_n(final m = m) "To negative rotor plug" annotation(
            Placement(transformation(extent = {{90, -50}, {110, -70}}, rotation = 0)));
          parameter Modelica.SIunits.Resistance RStart "Starting resistance";
          parameter Modelica.SIunits.Time tStart "Time instance of reducing the rheostat";
          parameter Modelica.SIunits.Time tRamp "Duration of ramp";
          Modelica.Electrical.MultiPhase.Basic.Star star(final m = m) annotation(
            Placement(transformation(extent = {{-20, -70}, {-40, -50}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Ground ground annotation(
            Placement(transformation(origin = {-70, -60}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          Modelica.Electrical.MultiPhase.Basic.VariableResistor rheostat(final m = m) annotation(
            Placement(transformation(origin = {60, 0}, extent = {{-10, 10}, {10, -10}}, rotation = 270)));
          Modelica.Blocks.Sources.Ramp ramp[m](final height = fill(-RStart, m), final duration = fill(tRamp, m), final offset = fill(RStart, m), final startTime = fill(tStart, m)) annotation(
            Placement(transformation(extent = {{-20, -10}, {0, 10}}, rotation = 0)));
        equation
          connect(plug_p, rheostat.plug_p) annotation(
            Line(points = {{100, 60}, {60, 60}, {60, 10}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(rheostat.plug_n, plug_n) annotation(
            Line(points = {{60, -10}, {60, -60}, {100, -60}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(rheostat.R, ramp.y) annotation(
            Line(points = {{50, 0}, {26, 0}, {26, 0}, {1, 0}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(rheostat.plug_n, star.plug_p) annotation(
            Line(points = {{60, -10}, {60, -60}, {-20, -60}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(star.pin_n, ground.p) annotation(
            Line(points = {{-40, -60}, {-60, -60}}, color = {0, 0, 255}, smooth = Smooth.None));
          annotation(
            Icon(graphics = {Rectangle(extent = {{26, 40}, {54, -40}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{100, -60}, {40, -60}, {40, -40}}, color = {0, 0, 255}), Line(points = {{10, -80}, {70, -80}}, color = {0, 0, 255}), Line(points = {{40, -60}, {40, -80}}, color = {0, 0, 255}), Line(points = {{20, -90}, {60, -90}}, color = {0, 0, 255}), Line(points = {{30, -100}, {50, -100}}, color = {0, 0, 255}), Line(points = {{40, 40}, {40, 60}, {100, 60}}, color = {0, 0, 255}), Rectangle(extent = {{-100, 40}, {-20, -40}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{-90, 24}, {-72, 24}, {-30, -30}}, color = {0, 0, 255}), Polygon(points = {{-20, 6}, {-10, 0}, {-20, -6}, {-20, 6}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Polygon(points = {{10, 6}, {20, 0}, {10, -6}, {10, 6}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{-10, 0}, {10, 0}}, color = {0, 0, 255}), Line(points = {{20, 0}, {26, 0}}, color = {0, 0, 255})}),
            Documentation(info = "<HTML>
<p>Ramped rheostat, used for starting asynchronous induction motors with slipring rotor:</p>
<p>The external rotor resistance <code>RStart</code> is reduced to zero,
starting at time <code>tStart</code> with a linear ramp <code>tRamp</code>.</p>
</HTML>"));
        end RampedRheostat;

        record SynchronousMachineData "Computes machine parameter from usual datasheet"
          extends Modelica.Icons.Record;
          import Modelica.Constants.pi;
          parameter Modelica.SIunits.ApparentPower SNominal(start = 30E3) "Nominal apparent power";
          parameter Modelica.SIunits.Voltage VsNominal(start = 100) "Nominal stator voltage per phase";
          final parameter Modelica.SIunits.Current IsNominal = SNominal / (3 * VsNominal) "Nominal stator current per phase";
          final parameter Modelica.SIunits.Impedance ZReference = VsNominal / IsNominal "Reference impedance";
          parameter Modelica.SIunits.Frequency fsNominal(start = 50) "Nominal stator frequency";
          final parameter Modelica.SIunits.AngularVelocity omega = 2 * pi * fsNominal "Nominal angular frequency";
          parameter Modelica.SIunits.Current IeOpenCircuit(start = 10) "Open circuit excitation current @ nominal voltage and frequency";
          final parameter Real turnsRatio = sqrt(2) * VsNominal / (omega * Lmd * IeOpenCircuit) "Stator current / excitation current";
          parameter Real x0(start = 0.1) "Stator stray inductance per phase (approximately zero impedance) [pu]";
          parameter Real xd(start = 1.6) "Synchronous reactance per phase, d-axis [pu]";
          parameter Real xq(start = 1.6) "Synchronous reactance per phase, q-axis [pu]";
          parameter Real xdTransient(start = 0.1375) "Transient reactance per phase, d-axis [pu]";
          parameter Real xdSubtransient(start = 0.121428571) "Subtransient reactance per phase, d-axis [pu]";
          parameter Real xqSubtransient(start = 0.148387097) "Subtransient reactance per phase, q-axis [pu]";
          parameter Modelica.SIunits.Time Ta(start = 0.014171268) "Armature time constant";
          parameter Modelica.SIunits.Time Td0Transient(start = 0.261177343) "Open circuit field time constant Td0'";
          parameter Modelica.SIunits.Time Td0Subtransient(start = 0.006963029) "Open circuit subtransient time constant Td0'', d-axis";
          parameter Modelica.SIunits.Time Tq0Subtransient(start = 0.123345081) "Open circuit subtransient time constant Tq0'', q-axis";
          parameter Modelica.SIunits.Temperature TsSpecification(start = 293.15) "Specification temperature of stator resistance" annotation(
            Dialog(tab = "Material"));
          parameter Modelica.SIunits.Temperature TsRef(start = 293.15) "Reference temperature of stator resistance" annotation(
            Dialog(tab = "Material"));
          parameter Machines.Thermal.LinearTemperatureCoefficient20 alpha20s(start = 0) "Temperature coefficient of stator resistance at 20 degC" annotation(
            Dialog(tab = "Material"));
          parameter Modelica.SIunits.Temperature TrSpecification(start = 293.15) "Specification temperature of (optional) damper cage" annotation(
            Dialog(tab = "Material"));
          parameter Modelica.SIunits.Temperature TrRef(start = 293.15) "Reference temperature of damper resistances in d- and q-axis" annotation(
            Dialog(tab = "Material"));
          parameter Machines.Thermal.LinearTemperatureCoefficient20 alpha20r(start = 0) "Temperature coefficient of damper resistances in d- and q-axis" annotation(
            Dialog(tab = "Material"));
          parameter Modelica.SIunits.Temperature TeSpecification(start = 293.15) "Specification excitation temperature" annotation(
            Dialog(tab = "Material"));
          parameter Modelica.SIunits.Temperature TeRef(start = 293.15) "Reference temperature of excitation resistance" annotation(
            Dialog(tab = "Material"));
          parameter Machines.Thermal.LinearTemperatureCoefficient20 alpha20e(start = 0) "Temperature coefficient of excitation resistance" annotation(
            Dialog(tab = "Material"));
          final parameter Real xmd = xd - x0 "Main field reactance per phase, d-axis [pu]";
          final parameter Real xmq = xq - x0 "Main field reactance per phase, q-axis [pu]";
          final parameter Real xe = xmd ^ 2 / (xd - xdTransient) "Excitation reactance [pu]";
          final parameter Real xrd = xmd ^ 2 / (xdTransient - xdSubtransient) * (1 - xmd / xe) ^ 2 + xmd ^ 2 / xe "Damper reactance per phase, d-axis [pu]";
          final parameter Real xrq = xmq ^ 2 / (xq - xqSubtransient) "Damper reactance per phase, d-axis [pu]";
          final parameter Real rs = 2 / (1 / xdSubtransient + 1 / xqSubtransient) / (omega * Ta) "Stator resistance per phase at specification temperature [pu]";
          final parameter Real rrd = (xrd - xmd ^ 2 / xe) / (omega * Td0Subtransient) "Damper resistance per phase at specification temperature, d-axis [pu]";
          final parameter Real rrq = xrq / (omega * Tq0Subtransient) "Damper resistance per phase at specification temperature, q-axis [pu]";
          final parameter Real re = xe / (omega * Td0Transient) "Excitation resistance per phase at specification temperature [pu]";
          parameter Modelica.SIunits.Resistance Rs = Machines.Thermal.convertResistance(rs * ZReference, TsSpecification, alpha20s, TsRef) "Stator resistance per phase at TRef" annotation(
            Dialog(tab = "Result", enable = false));
          parameter Modelica.SIunits.Inductance Lssigma = x0 * ZReference / omega "Stator stray inductance per phase" annotation(
            Dialog(tab = "Result", enable = false));
          parameter Modelica.SIunits.Inductance Lmd = xmd * ZReference / omega "Main field inductance per phase in d-axis" annotation(
            Dialog(tab = "Result", enable = false));
          parameter Modelica.SIunits.Inductance Lmq = xmq * ZReference / omega "Main field inductance per phase in q-axis" annotation(
            Dialog(tab = "Result", enable = false));
          parameter Modelica.SIunits.Inductance Lrsigmad = (xrd - xmd) * ZReference / omega "Damper stray inductance in d-axis" annotation(
            Dialog(tab = "Result", enable = false));
          parameter Modelica.SIunits.Inductance Lrsigmaq = (xrq - xmq) * ZReference / omega "Damper stray inductance in q-axis" annotation(
            Dialog(tab = "Result", enable = false));
          parameter Modelica.SIunits.Resistance Rrd = Machines.Thermal.convertResistance(rrd * ZReference, TrSpecification, alpha20r, TrRef) "Damper resistance in d-axis at TRef" annotation(
            Dialog(tab = "Result", enable = false));
          parameter Modelica.SIunits.Resistance Rrq = Machines.Thermal.convertResistance(rrq * ZReference, TrSpecification, alpha20r, TrRef) "Damper resistance in q-axis at TRef" annotation(
            Dialog(tab = "Result", enable = false));
          parameter Modelica.SIunits.Resistance Re = 3 / 2 * turnsRatio ^ 2 * Machines.Thermal.convertResistance(re * ZReference, TeSpecification, alpha20e, TeRef) "Excitation resistance at TRef" annotation(
            Dialog(tab = "Result", enable = false));
          parameter Real sigmae = 1 - xmd / xe "Stray fraction of total excitation inductance" annotation(
            Dialog(tab = "Result", enable = false));
          annotation(
            defaultComponentName = "smeeData",
            defaultComponentPrefixes = "parameter",
            Documentation(info = "<html>
<p>The parameters of the
<a href=\"modelica://Modelica.Electrical.Machines.BasicMachines.SynchronousInductionMachines.SM_ElectricalExcited\">
synchronous machine model with electrical excitation (and damper)</a> are calculated from parameters
normally given in a technical description, according to the standard EN&nbsp;60034-4:2008&nbsp;Appendix&nbsp;C.</p>
</html>"));
        end SynchronousMachineData;

        record TransformerData "Calculates Impedances from nominal values"
          extends Modelica.Icons.Record;
          parameter Modelica.SIunits.Frequency f(start = 50) "Nominal frequency";
          parameter Modelica.SIunits.Voltage V1(start = 100) "Primary nominal line-to-line voltage (RMS)";
          parameter String C1(start = "Y") "Choose primary connection" annotation(
            choices(choice = "Y" "Star connection", choice = "D" "Delta connection"));
          parameter Modelica.SIunits.Voltage V2(start = 100) "Secondary open circuit line-to-line voltage (RMS) @ primary nominal voltage";
          parameter String C2(start = "y") "Choose secondary connection" annotation(
            choices(choice = "y" "Star connection", choice = "d" "Delta connection", choice = "z" "Zig-zag connection"));
          parameter Modelica.SIunits.ApparentPower SNominal(start = 30E3) "Nominal apparent power";
          parameter Real v_sc(final min = 0, final max = 1, start = 0.05) "Impedance voltage drop pu";
          parameter Modelica.SIunits.Power P_sc(start = 300) "Short-circuit (copper) losses";
          parameter Real n = V1 / V2 "Ratio primary voltage (line-to-line) / secondary voltage (line-to-line)" annotation(
            Dialog(tab = "Result", enable = false));
          final parameter Modelica.SIunits.Voltage V1ph = V1 / (if C1 == "D" then 1 else sqrt(3)) "Primary phase voltage (RMS)";
          final parameter Modelica.SIunits.Current I1ph = SNominal / (3 * V1ph) "Primary phase current (RMS)";
          final parameter Modelica.SIunits.Voltage V2ph = V2 / (if C2 == "d" then 1 else sqrt(3)) "Secondary phase voltage (RMS)";
          final parameter Modelica.SIunits.Current I2ph = SNominal / (3 * V2ph) "Secondary phase current (RMS)";
          final parameter Modelica.SIunits.Impedance Z1ph = 0.5 * v_sc * V1ph / I1ph "Primary impedance per phase";
          parameter Modelica.SIunits.Resistance R1 = 0.5 * P_sc / (3 * I1ph ^ 2) "Warm primary resistance per phase" annotation(
            Dialog(tab = "Result", enable = false));
          parameter Modelica.SIunits.Inductance L1sigma = sqrt(Z1ph ^ 2 - R1 ^ 2) / (2 * Modelica.Constants.pi * f) "Primary stray inductance per phase" annotation(
            Dialog(tab = "Result", enable = false));
          final parameter Modelica.SIunits.Impedance Z2ph = 0.5 * v_sc * V2ph / I2ph "Secondary impedance per phase";
          parameter Modelica.SIunits.Resistance R2 = 0.5 * P_sc / (3 * I2ph ^ 2) "Warm secondary resistance per phase" annotation(
            Dialog(tab = "Result", enable = false));
          parameter Modelica.SIunits.Inductance L2sigma = sqrt(Z2ph ^ 2 - R2 ^ 2) / (2 * Modelica.Constants.pi * f) "Secondary stray inductance per phase" annotation(
            Dialog(tab = "Result", enable = false));
          annotation(
            defaultComponentPrefixes = "parameter",
            Documentation(info = "<HTML>
<p>The parameters of the transformer models are calculated from parameters normally given in a technical description.</p>
</HTML>"));
        end TransformerData;
        annotation(
          Documentation(info = "<HTML>
This package contains utility components for testing examples.
</HTML>", revisions = "<HTML>
<dl>
  <dt><b>Main Authors:</b></dt>
  <dd>
  <a href=\"http://www.haumer.at/\">Anton Haumer</a><br>
  Technical Consulting &amp; Electrical Engineering<br>
  A-3423 St.Andrae-Woerdern<br>Austria<br>
  email: <a href=\"mailto:a.haumer@haumer.at\">a.haumer@haumer.at</a>
  </dd>
  <dt><b>Copyright:</b></dt>
  <dd>Copyright &copy; 1998-2015, Modelica Association and Anton Haumer.<br>
  <i>The Modelica package is <b>free</b> software; it can be redistributed and/or modified
  under the terms of the <b>Modelica license</b>, see the license conditions
  and the accompanying <b>disclaimer</b> in the documentation of package
  Modelica in file \"Modelica/package.mo\".</i></dd>
</dl>
  <ul>
  <li> v1.00 2004/09/16 Anton Haumer</li>
  <li> v1.1  2004/10/01 Anton Haumer<br>
       changed naming and structure<br>
       issued to Modelica Standard Library 2.1</li>
  <li> v1.3.1 2004/11/06 Anton Haumer<br>
       small changes in VfController</li>
  <li> v1.6.1 2004/11/22 Anton Haumer<br>
       introduced TerminalBox</li>
  <li> v2.1.1 2010/02/05 Anton Haumer<br>
       included rheostats (designed by Christian Kral)</li>
  </ul>
</HTML>"));
      end Utilities;
      annotation(
        preferredView = "info",
        Documentation(info = "<HTML>
<p>This package contains electric machine models and components for modeling these machines.</p>
<b>Limitations and assumptions:</b>
<ul>
<li>number of phases (of induction machines) is limited to 3, therefore definition as a constant m=3</li>
<li>phase symmetric windings as well as symmetry of the whole machine structure</li>
<li>all values are used in physical units, no scaling to p.u. is done</li>
<li>only basic harmonics (in space) are taken into account</li>
<li>waveform (with respect to time) of voltages and currents is not restricted</li>
<li>constant parameters, i.e., no saturation, no skin effect</li>
</ul>
<p>
You may have a look at a short summary of space phasor theory at <a href=\"http://www.haumer.at/refimg/SpacePhasors.pdf\">http://www.haumer.at/refimg/SpacePhasors.pdf</a>
</p>
<b>Further development:</b>
<ul>
<li>generalizing space phasor theory to m phases with arbitrary spatial angle of the coils</li>
<li>generalizing space phasor theory to arbitrary number of windings and winding factor of the coils</li>
<li>MachineModels: other machine types</li>
<li>effects: saturation, skin-effect, ...</li>
</ul>
<p>
<b>In memoriam Prof. Hans Kleinrath (1928-03-07 - 2010-04-05)</b>
</p>
<dl>
  <dt><b>Main Authors:</b></dt>
  <dd>
  <p>
  <a href=\"http://www.haumer.at/\">Anton Haumer</a><br>
  Technical Consulting &amp; Electrical Engineering<br>
  A-3423 St.Andrae-Woerdern<br>Austria<br>
  email: <a href=\"mailto:a.haumer@haumer.at\">a.haumer@haumer.at</a>
</p>
  <p>
  Dr. Christian Kral<br>
  Electric Machines, Drives and Systems<br>
  A-1060 Vienna, Austria<br>
  email: <a href=\"mailto:dr.christian.kral@gmail.com\">dr.christian.kral@gmail.com</a>
</p>
  </dd>
</dl>
<p>
Copyright &copy; 1998-2015, Modelica Association, Anton Haumer, Christian Kral and AIT.
</p>
<p>
<i>This Modelica package is <u>free</u> software and the use is completely at <u>your own risk</u>; it can be redistributed and/or modified under the terms of the Modelica License 2. For license conditions (including the disclaimer of warranty) see <a href=\"modelica://Modelica.UsersGuide.ModelicaLicense2\">Modelica.UsersGuide.ModelicaLicense2</a> or visit <a href=\"https://www.modelica.org/licenses/ModelicaLicense2\"> https://www.modelica.org/licenses/ModelicaLicense2</a>.</i>
</p>
</HTML>", revisions = "<HTML>
  <ul>
  <li> v1.00  2004/09/16 Anton Haumer<br>
       first stable release</li>
  <li> v1.01  2004/09/18 Anton Haumer<br>
       moved common equations from machine models to PartialMachine<br>
       improved MoveToRotational</li>
  <li> v1.02  2004/09/19 Anton Haumer<br>
       new package structure for machine types<br>
       added DC machine models</li>
  <li> v1.03  2004/09/24 Anton Haumer<br>
       added package Sensors<br>
       added DC machine with series excitation<br>
       debugged and improved MoveToRotational</li>
  <li> v1.1   2004/10/01 Anton Haumer<br>
       changed naming and structure<br>
       issued to Modelica Standard Library 2.1</li>
  <li> v1.2   2004/10/27 Anton Haumer<br>
       fixed a bug with support (formerly bearing)</li>
  <li> v1.3   2004/11/05 Anton Haumer<br>
       several improvements in SpacePhasors.Blocks</li>
  <li> v1.3.1 2004/11/06 Anton Haumer<br>
       small changes in Examples.Utilities.VfController</li>
  <li> v1.3.2 2004/11/10 Anton Haumer<br>
       ReluctanceRotor moved to SynchronousMachines</li>
  <li> v1.4   2004/11/11 Anton Haumer<br>
       removed mechanical flange support<br>
       to ease the implementation of a 3D-frame in a future release</li>
  <li> v1.51  2005/02/01 Anton Haumer<br>
       changed parameter polePairs to Integer</li>
  <li> v1.52  2005/10/12 Anton Haumer<br>
       added BasicMachines.SynchronousInductionMachines.SM_ElectricalExcited<br>
       using new basicMachines.Components.ElectricalExcitation<br>
       as well as a new example.</li>
  <li> v1.53  2005/10/14 Anton Haumer<br>
       introduced unsymmetrical DamperCage for Synchronous Machines</li>
  <li> v1.60  2005/11/04 Anton Haumer<br>
       added SpacePhasors.Components.Rotator<br>
       corrected consistent naming of parameters and variables</li>
  <li> v1.6.1 2005/11/22 Anton Haumer<br>
       improved Transformation and Rotation in SpacePhasor.<br>
       introduced Examples.Utilities.TerminalBox</li>
  <li> v1.6.2 2005/10/23 Anton Haumer<br>
       selectable DamperCage for Synchronous Machines</li>
  <li> v1.6.3 2005/11/25 Anton Haumer<br>
       easier parameterization of AsynchronousInductionMachines.AIM_SlipRing model</li>
  <li> v1.7.0 2005/12/15 Anton Haumer<br>
       back-changed the naming to ensure backward compatibility</li>
  <li> v1.7.1 2006/02/06 Anton Haumer<br>
       changed some naming of synchronous machines, not affecting existing models</li>
  <li> v1.8.0 2006/11/26 Anton Haumer<br>
       introduced package Transformers<br>
       moved common parameters and functionality to partial models,<br>
       keeping Interfaces.PartialBasicInductionMachines resp. PartialBasicDCMachine for compatibility reasons.<br>
       implemented support showing reaction torque if connected</li>
  <li> v1.8.1 2006/12/01 Anton Haumer<br>
       resolved a compatibility issue with airGap</li>
  <li> v1.8.2 2007/01/15 Anton Haumer<br>
       resolved a bug in electrical excited synchronous induction machine</li>
  <li> v1.8.3 2007/06/08 Anton Haumer<br>
       documentation update</li>
  <li> v1.8.4 2007/06/25 Anton Haumer<br>
       corrected some typos in documentation</li>
  <li> v1.8.5 2007/06/26 Anton Haumer<br>
       consistent parameters of DCSE</li>
  <li> v1.8.6 2007/08/12 Anton Haumer<br>
       improved documentation</li>
  <li> v1.8.7 2007/08/20 Anton Haumer<br>
       corrected typo in documentation</li>
  <li> v1.8.8 2007/08/20 Anton Haumer<br>
       improved documentation</li>
  <li> v1.9.0 2007/08/24 Anton Haumer<br>
       removed redeclare type SignalType</li>
  <li> v1.9.1 2007/10/15 Anton Haumer<br>
       solved a bug with replaceable airgap / partial machines</li>
  <li> v1.9.2 2007/10/15 Anton Haumer<br>
       changed some names to be more concise (see conversion script)</li>
  <li> v2.0.0 2007/11/13 Anton Haumer<br>
       removed replaceable from airgaps<br>
       removed cardinality from support, using a Boolean parameter<br>
       removed all nonSIunits</li>
  <li> v2.1.0 2009/08/26 Anton Haumer<br>
       set all useHeatPort=false</li>
  <li> v2.1.1 2010/02/05 Anton Haumer<br>
       included Utilities.rheostats (designed by Christian Kral)</li>
  <li> v2.1.2 2010/02/09 Anton Haumer<br>
       included new Examples (AIMC_Transformer, DC_Comparison)</li>
  <li> v2.1.3 2010/02/10 Anton Haumer<br>
       prepared conditionalHeatPort of SquirrelCage and DamperCage</li>
  <li> v2.2.0 2010/02/10 Anton Haumer<br>
       conditional ThermalPort for all machines</li>
  <li> v2.3.0 2010/02/16 Anton Haumer<br>
       quasistationary DC machine models</li>
  <li> v2.4.0 2010/04/20 Anton Haumer<br>
       loss models</li>
  <li> v2.5.0 2012/xx/xx Anton Haumer<br>
       permanent magnet losses</li>
  <li> v2.6.0 2013/02/25 Anton Haumer<br>
       corrected turnsRatio bug in asynchronous induction machine with slipring; corrected parameter descriptions</li>
  <li> v3.2.1 2014/10/05 Anton Haumer, Christian Kral<br>
       corrected bug in wrong orientation of rotor current, see #1226<br>
       fixed bugs of wrong sign of power calculation and orientation of secondary components; see #1474<br>
       fixed bug in temperature propagation of transformer ambient model; see #1579</li>
</li>
  </ul>
</HTML>"),
        Icon(coordinateSystem(extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(origin = {2.835, 10}, fillColor = {0, 128, 255}, fillPattern = FillPattern.HorizontalCylinder, extent = {{-60, -60}, {60, 60}}), Rectangle(origin = {2.835, 10}, fillColor = {128, 128, 128}, fillPattern = FillPattern.HorizontalCylinder, extent = {{-80, -60}, {-60, 60}}), Rectangle(origin = {2.835, 10}, fillColor = {95, 95, 95}, fillPattern = FillPattern.HorizontalCylinder, extent = {{60, -10}, {80, 10}}), Rectangle(origin = {2.835, 10}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid, extent = {{-60, 50}, {20, 70}}), Polygon(origin = {2.835, 10}, fillPattern = FillPattern.Solid, points = {{-70, -90}, {-60, -90}, {-30, -20}, {20, -20}, {50, -90}, {60, -90}, {60, -100}, {-70, -100}, {-70, -90}})}));
    end Machines;

    package MultiPhase "Library for electrical components with 2, 3 or more phases"
      extends Modelica.Icons.Package;

      package Examples "Multiphase test examples"
        extends Modelica.Icons.ExamplesPackage;

        model TransformerYY "Test example with multiphase components"
          extends Modelica.Icons.Example;
          parameter Integer m = 3 "Number of phases";
          parameter Modelica.SIunits.Voltage V = 1 "Amplitude of Star-Voltage";
          parameter Modelica.SIunits.Frequency f = 5 "Frequency";
          parameter Modelica.SIunits.Inductance Lm = 1 "Transformer main inductance";
          parameter Modelica.SIunits.Inductance LT = 0.003 "Transformer stray inductance";
          parameter Modelica.SIunits.Resistance RT = 0.05 "Transformer resistance";
          parameter Modelica.SIunits.Resistance RL = 1 "Load Resistance";
          parameter Real nT = 1 "Transformer ratio";
          Sources.SineVoltage sineVoltage(V = fill(V, m), freqHz = fill(f, m), m = m) annotation(
            Placement(transformation(origin = {-80, 20}, extent = {{-10, -10}, {10, 10}}, rotation = 180)));
          Basic.Star starS(m = m) annotation(
            Placement(transformation(origin = {-90, -62}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          Modelica.Electrical.Analog.Basic.Ground groundS annotation(
            Placement(transformation(extent = {{-100, -100}, {-80, -80}}, rotation = 0)));
          Ideal.IdealTransformer idealTransformer(m = m, Lm1 = fill(Lm, m), n = fill(nT, m)) annotation(
            Placement(transformation(extent = {{-40, 0}, {-20, 20}}, rotation = 0)));
          Basic.Star starT1(m = m) annotation(
            Placement(transformation(origin = {-40, -62}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          Basic.Star starT2(m = m) annotation(
            Placement(transformation(origin = {-20, -62}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          Modelica.Electrical.Analog.Basic.Ground groundT2 annotation(
            Placement(transformation(extent = {{-30, -100}, {-10, -80}}, rotation = 0)));
          Basic.Resistor transformerR(m = m, R = fill(RT, m)) annotation(
            Placement(transformation(extent = {{0, 10}, {20, 30}}, rotation = 0)));
          Basic.Inductor transformerL(m = m, L = fill(LT, m)) annotation(
            Placement(transformation(extent = {{30, 10}, {50, 30}}, rotation = 0)));
          Basic.Resistor loadR(m = m, R = fill(RL, m)) annotation(
            Placement(transformation(extent = {{70, 10}, {90, 30}}, rotation = 0)));
          Basic.Star starL(m = m) annotation(
            Placement(transformation(origin = {90, -62}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          Modelica.Electrical.Analog.Basic.Ground groundT1 annotation(
            Placement(transformation(extent = {{-50, -100}, {-30, -80}}, rotation = 0)));
        initial equation
          transformerL.i[1:m - 1] = zeros(m - 1) "Y-connection";
        equation
          connect(starS.pin_n, groundS.p) annotation(
            Line(points = {{-90, -72}, {-90, -80}}, color = {0, 0, 255}));
          connect(starT1.pin_n, groundT1.p) annotation(
            Line(points = {{-40, -72}, {-40, -80}}, color = {0, 0, 255}));
          connect(starT2.pin_n, groundT2.p) annotation(
            Line(points = {{-20, -72}, {-20, -76}, {-20, -76}, {-20, -80}}, color = {0, 0, 255}));
          connect(starS.plug_p, sineVoltage.plug_n) annotation(
            Line(points = {{-90, -52}, {-90, 20}}, color = {0, 0, 255}));
          connect(sineVoltage.plug_p, idealTransformer.plug_p1) annotation(
            Line(points = {{-70, 20}, {-40, 20}}, color = {0, 0, 255}));
          connect(idealTransformer.plug_n1, starT1.plug_p) annotation(
            Line(points = {{-40, 0}, {-40, -52}}, color = {0, 0, 255}));
          connect(starT2.plug_p, idealTransformer.plug_n2) annotation(
            Line(points = {{-20, -52}, {-20, -26}, {-20, 0}}, color = {0, 0, 255}));
          connect(idealTransformer.plug_p2, transformerR.plug_p) annotation(
            Line(points = {{-20, 20}, {0, 20}}, color = {0, 0, 255}));
          connect(transformerR.plug_n, transformerL.plug_p) annotation(
            Line(points = {{20, 20}, {30, 20}}, color = {0, 0, 255}));
          connect(transformerL.plug_n, loadR.plug_p) annotation(
            Line(points = {{50, 20}, {70, 20}}, color = {0, 0, 255}));
          connect(loadR.plug_n, starL.plug_p) annotation(
            Line(points = {{90, 20}, {90, -52}}, color = {0, 0, 255}));
          annotation(
            Documentation(info = "<HTML>
<p>
Test example with multiphase components:<br>
Star-connected voltage source feeds via a Y-Y-transformer with internal impedance (RT, LT) a load resistor RT.<br>
Using f=5 Hz LT=3mH defines nominal voltage drop of approximately 10 %.<br>
Simulate for 1 second (2 periods) and compare voltages and currents of source, transformer and load.
</p>
</HTML>"),
            experiment(StopTime = 1.0, Interval = 0.001));
        end TransformerYY;

        model TransformerYD "Test example with multiphase components"
          extends Modelica.Icons.Example;
          parameter Integer m = 3 "Number of phases";
          parameter Modelica.SIunits.Voltage V = 1 "Amplitude of Star-Voltage";
          parameter Modelica.SIunits.Frequency f = 5 "Frequency";
          parameter Modelica.SIunits.Inductance Lm = 1 "Transformer main inductance";
          parameter Modelica.SIunits.Inductance LT = 0.003 "Transformer stray inductance";
          parameter Modelica.SIunits.Resistance RT = 0.05 "Transformer resistance";
          parameter Modelica.SIunits.Resistance RL = 1 "Load Resistance";
          parameter Real nT = 1 / sqrt((1 - Modelica.Math.cos(2 * Modelica.Constants.pi / m)) ^ 2 + Modelica.Math.sin(2 * Modelica.Constants.pi / m) ^ 2) "Transformer ratio";
          Sources.SineVoltage sineVoltage(V = fill(V, m), freqHz = fill(f, m), m = m) annotation(
            Placement(transformation(origin = {-80, 20}, extent = {{-10, -10}, {10, 10}}, rotation = 180)));
          Basic.Star starS(m = m) annotation(
            Placement(transformation(origin = {-90, -62}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          Modelica.Electrical.Analog.Basic.Ground groundS annotation(
            Placement(transformation(extent = {{-100, -100}, {-80, -80}}, rotation = 0)));
          Ideal.IdealTransformer idealTransformer(m = m, n = fill(nT, m), Lm1 = fill(Lm, m)) annotation(
            Placement(transformation(extent = {{-40, 0}, {-20, 20}}, rotation = 0)));
          Basic.Star starT(m = m) annotation(
            Placement(transformation(origin = {-40, -62}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          Basic.Delta deltaT2(m = m) annotation(
            Placement(transformation(origin = {50, 10}, extent = {{-10, 10}, {10, -10}}, rotation = 270)));
          Modelica.Electrical.Analog.Basic.Ground groundT annotation(
            Placement(transformation(extent = {{-50, -100}, {-30, -80}}, rotation = 0)));
          Basic.Resistor transformerR(m = m, R = fill(RT / nT ^ 2, m)) annotation(
            Placement(transformation(extent = {{-10, 10}, {10, 30}}, rotation = 0)));
          Basic.Inductor transformerL(m = m, L = fill(LT / nT ^ 2, m)) annotation(
            Placement(transformation(extent = {{20, 10}, {40, 30}}, rotation = 0)));
          Basic.Resistor loadR(m = m, R = fill(RL, m)) annotation(
            Placement(transformation(extent = {{70, 10}, {90, 30}}, rotation = 0)));
          Basic.Star starL(m = m) annotation(
            Placement(transformation(origin = {90, -62}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          Modelica.Electrical.Analog.Basic.Ground groundL annotation(
            Placement(transformation(extent = {{80, -100}, {100, -80}}, rotation = 0)));
        initial equation
          transformerL.i[1:m] = zeros(m) "D-connection";
        equation
          connect(groundS.p, starS.pin_n) annotation(
            Line(points = {{-90, -80}, {-90, -72}}, color = {0, 0, 255}));
          connect(groundT.p, starT.pin_n) annotation(
            Line(points = {{-40, -80}, {-40, -72}}, color = {0, 0, 255}));
          connect(starS.plug_p, sineVoltage.plug_n) annotation(
            Line(points = {{-90, -52}, {-90, 20}}, color = {0, 0, 255}));
          connect(sineVoltage.plug_p, idealTransformer.plug_p1) annotation(
            Line(points = {{-70, 20}, {-40, 20}}, color = {0, 0, 255}));
          connect(idealTransformer.plug_n1, starT.plug_p) annotation(
            Line(points = {{-40, 0}, {-40, -52}}, color = {0, 0, 255}));
          connect(idealTransformer.plug_p2, transformerR.plug_p) annotation(
            Line(points = {{-20, 20}, {-10, 20}}, color = {0, 0, 255}));
          connect(transformerR.plug_n, transformerL.plug_p) annotation(
            Line(points = {{10, 20}, {20, 20}}, color = {0, 0, 255}));
          connect(transformerL.plug_n, deltaT2.plug_p) annotation(
            Line(points = {{40, 20}, {50, 20}}, color = {0, 0, 255}));
          connect(deltaT2.plug_n, idealTransformer.plug_n2) annotation(
            Line(points = {{50, 0}, {-20, 0}}, color = {0, 0, 255}));
          connect(deltaT2.plug_p, loadR.plug_p) annotation(
            Line(points = {{50, 20}, {70, 20}}, color = {0, 0, 255}));
          connect(loadR.plug_n, starL.plug_p) annotation(
            Line(points = {{90, 20}, {90, -52}}, color = {0, 0, 255}));
          connect(starL.pin_n, groundL.p) annotation(
            Line(points = {{90, -72}, {90, -80}}, color = {0, 0, 255}));
          annotation(
            Documentation(info = "<HTML>
<p>
Test example with multiphase components:<br>
Star-connected voltage source feeds via a Y-D-transformer with internal impedance (RT, LT) a load resistor RT.<br>
Using f=5 Hz LT=3mH defines nominal voltage drop of approximately 10 %.<br>
Simulate for 1 second (2 periods) and compare voltages and currents of source, transformer and load.
</p>
</HTML>"),
            experiment(StopTime = 1.0, Interval = 0.001));
        end TransformerYD;

        model Rectifier "Test example with multiphase components"
          extends Modelica.Icons.Example;
          parameter Integer m = 3 "Number of phases";
          parameter Modelica.SIunits.Voltage V = 1 "Amplitude of Star-Voltage";
          parameter Modelica.SIunits.Frequency f = 5 "Frequency";
          parameter Modelica.SIunits.Inductance L = 0.001 "Line Inductance";
          parameter Modelica.SIunits.Resistance RL = 2 "Load Resistance";
          parameter Modelica.SIunits.Capacitance C = 0.05 "Total DC-Capacitance";
          parameter Modelica.SIunits.Resistance RE = 1E6 "Earthing Resistance";
          parameter Modelica.SIunits.Resistance Ron = 1.E-5 "Closed diode resistance";
          parameter Modelica.SIunits.Conductance Goff = 1.E-5 "Opened diode conductance";
          parameter Modelica.SIunits.Voltage Vknee = 0 "Threshold diode voltage";
          Sources.SineVoltage sineVoltage(m = m, V = fill(V, m), freqHz = fill(f, m)) annotation(
            Placement(transformation(extent = {{-70, 10}, {-90, -10}}, rotation = 0)));
          Basic.Star starS(m = m) annotation(
            Placement(transformation(origin = {-90, -50}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          Basic.Inductor supplyL(m = m, L = fill(L, m)) annotation(
            Placement(transformation(extent = {{-52, -10}, {-32, 10}}, rotation = 0)));
          Ideal.IdealDiode idealDiode1(m = m, Ron = fill(Ron, m), Goff = fill(Goff, m), Vknee = fill(Vknee, m)) annotation(
            Placement(transformation(origin = {10, 20}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
          Basic.Star star1(m = m) annotation(
            Placement(transformation(origin = {10, 50}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
          Ideal.IdealDiode idealDiode2(m = m, Ron = fill(Ron, m), Goff = fill(Goff, m), Vknee = fill(Vknee, m)) annotation(
            Placement(transformation(origin = {10, -20}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
          Basic.Star star2(m = m) annotation(
            Placement(transformation(origin = {10, -50}, extent = {{-10, 10}, {10, -10}}, rotation = 270)));
          Modelica.Electrical.Analog.Basic.Resistor loadR(R = RL) annotation(
            Placement(transformation(origin = {50, 0}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          Modelica.Electrical.Analog.Basic.Capacitor cDC1(C = 2 * C) annotation(
            Placement(transformation(origin = {70, 30}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          Modelica.Electrical.Analog.Basic.Capacitor cDC2(C = 2 * C) annotation(
            Placement(transformation(origin = {70, -30}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          Modelica.Electrical.Analog.Basic.Ground groundDC annotation(
            Placement(transformation(extent = {{80, -80}, {100, -60}}, rotation = 0)));
        initial equation
          cDC1.v = 0;
          cDC2.v = 0;
          supplyL.i[1:m - 1] = zeros(m - 1) "Y-connection";
        equation
          connect(cDC1.n, cDC2.p) annotation(
            Line(points = {{70, 20}, {70, -20}}, color = {0, 0, 255}));
          connect(cDC1.n, groundDC.p) annotation(
            Line(points = {{70, 20}, {70, 0}, {90, 0}, {90, -60}}, color = {0, 0, 255}));
          connect(starS.plug_p, sineVoltage.plug_n) annotation(
            Line(points = {{-90, -40}, {-90, 0}}, color = {0, 0, 255}));
          connect(sineVoltage.plug_p, supplyL.plug_p) annotation(
            Line(points = {{-70, 0}, {-52, 0}}, color = {0, 0, 255}));
          connect(idealDiode1.plug_p, supplyL.plug_n) annotation(
            Line(points = {{10, 10}, {10, 0}, {-32, 0}}, color = {0, 0, 255}));
          connect(idealDiode2.plug_n, supplyL.plug_n) annotation(
            Line(points = {{10, -10}, {10, 0}, {-32, 0}}, color = {0, 0, 255}));
          connect(idealDiode1.plug_n, star1.plug_p) annotation(
            Line(points = {{10, 30}, {10, 40}}, color = {0, 0, 255}));
          connect(idealDiode2.plug_p, star2.plug_p) annotation(
            Line(points = {{10, -30}, {10, -35}, {10, -40}}, color = {0, 0, 255}));
          connect(star2.pin_n, loadR.n) annotation(
            Line(points = {{10, -60}, {50, -60}, {50, -10}}, color = {0, 0, 255}));
          connect(star2.pin_n, cDC2.n) annotation(
            Line(points = {{10, -60}, {70, -60}, {70, -40}}, color = {0, 0, 255}));
          connect(star1.pin_n, loadR.p) annotation(
            Line(points = {{10, 60}, {50, 60}, {50, 10}}, color = {0, 0, 255}));
          connect(star1.pin_n, cDC1.p) annotation(
            Line(points = {{10, 60}, {70, 60}, {70, 40}}, color = {0, 0, 255}));
          annotation(
            Documentation(info = "<HTML>
<p>
Test example with multiphase components:<br>
Star-connected voltage source feeds via a line reactor a diode bridge rectifier with a DC burden.<br>
Using f=5 Hz, simulate for 1 second (2 periods) and compare voltages and currents of source and DC burden,
neglecting initial transient.
</p>
</HTML>"),
            experiment(StopTime = 1.0, Interval = 0.0001));
        end Rectifier;

        model TestSensors
          extends Modelica.Icons.Example;
          constant Integer m = 3 "Number of phases";
          import Modelica.Constants.pi;
          parameter Modelica.SIunits.Voltage VRMS = 100 "Nominal RMS voltage per phase";
          parameter Modelica.SIunits.Frequency f = 50 "Frequency";
          parameter Modelica.SIunits.Resistance R = 1 / sqrt(2) "Load resistance";
          parameter Modelica.SIunits.Inductance L = 1 / sqrt(2) / (2 * pi * f) "Load inductance";
          final parameter Modelica.SIunits.Current IRMS = VRMS / sqrt(R ^ 2 + (2 * pi * f * L) ^ 2) "Steady state RMS current";
          Modelica.Electrical.MultiPhase.Sources.SineVoltage sineVoltage(final m = m, V = fill(sqrt(2) * VRMS, m), freqHz = fill(f, m)) annotation(
            Placement(transformation(origin = {-20, 70}, extent = {{10, -10}, {-10, 10}}, rotation = 90)));
          Modelica.Electrical.MultiPhase.Basic.Star star(final m = m) annotation(
            Placement(transformation(extent = {{10, -10}, {-10, 10}}, rotation = 90, origin = {-20, -70})));
          Modelica.Electrical.Analog.Basic.Ground ground annotation(
            Placement(transformation(origin = {-20, -100}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
          Modelica.Electrical.MultiPhase.Basic.Resistor resistor(m = m, R = fill(R, m)) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 270, origin = {20, -10})));
          Modelica.Electrical.MultiPhase.Basic.Inductor inductor(m = m, L = fill(L, m)) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 270, origin = {20, -40})));
          Modelica.Electrical.MultiPhase.Basic.Star starLoad(m = m) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 270, origin = {20, -70})));
          Modelica.Electrical.MultiPhase.Sensors.CurrentQuasiRMSSensor currentQuasiRMSSensor(m = m) annotation(
            Placement(transformation(extent = {{-10, 10}, {10, -10}}, rotation = 270, origin = {20, 70})));
          Modelica.Blocks.Math.Feedback feedbackI annotation(
            Placement(transformation(extent = {{70, 60}, {90, 80}})));
          Modelica.Blocks.Sources.Constant constI(k = IRMS) annotation(
            Placement(transformation(extent = {{10, -10}, {-10, 10}}, rotation = 270, origin = {80, 40})));
          Sensors.VoltageQuasiRMSSensor voltageQuasiRMSSensor(m = m) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 270, origin = {-50, 70})));
          Modelica.Blocks.Math.Feedback feedbackV annotation(
            Placement(transformation(extent = {{-70, 60}, {-90, 80}})));
          Modelica.Blocks.Sources.Constant constV(k = VRMS) annotation(
            Placement(transformation(extent = {{10, -10}, {-10, 10}}, rotation = 270, origin = {-80, 40})));
          Sensors.PowerSensor powerSensor(m = m) annotation(
            Placement(transformation(extent = {{10, -10}, {-10, 10}}, rotation = 90, origin = {20, 20})));
          Modelica.Blocks.Math.Feedback feedbackP annotation(
            Placement(transformation(extent = {{50, 10}, {70, 30}})));
          Modelica.Blocks.Sources.RealExpression realExpression(y = Modelica.Electrical.MultiPhase.Functions.activePower(voltageQuasiRMSSensor.v, currentQuasiRMSSensor.i)) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 90, origin = {60, -10})));
        initial equation
          for k in 1:m - 1 loop
            inductor.i[k] = 0;
          end for;
        equation
          connect(star.pin_n, ground.p) annotation(
            Line(points = {{-20, -80}, {-20, -90}}, color = {0, 0, 255}));
          connect(sineVoltage.plug_n, star.plug_p) annotation(
            Line(points = {{-20, 60}, {-20, -60}}, color = {0, 0, 255}));
          connect(resistor.plug_n, inductor.plug_p) annotation(
            Line(points = {{20, -20}, {20, -30}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(inductor.plug_n, starLoad.plug_p) annotation(
            Line(points = {{20, -50}, {20, -60}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(currentQuasiRMSSensor.I, feedbackI.u1) annotation(
            Line(points = {{30, 70}, {72, 70}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(feedbackI.u2, constI.y) annotation(
            Line(points = {{80, 62}, {80, 51}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(sineVoltage.plug_p, currentQuasiRMSSensor.plug_p) annotation(
            Line(points = {{-20, 80}, {-20, 90}, {20, 90}, {20, 80}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(voltageQuasiRMSSensor.plug_n, sineVoltage.plug_n) annotation(
            Line(points = {{-50, 60}, {-20, 60}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(voltageQuasiRMSSensor.plug_p, sineVoltage.plug_p) annotation(
            Line(points = {{-50, 80}, {-20, 80}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(feedbackV.u1, voltageQuasiRMSSensor.V) annotation(
            Line(points = {{-72, 70}, {-60, 70}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(constV.y, feedbackV.u2) annotation(
            Line(points = {{-80, 51}, {-80, 62}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(powerSensor.nc, resistor.plug_p) annotation(
            Line(points = {{20, 10}, {20, 0}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(currentQuasiRMSSensor.plug_n, powerSensor.pc) annotation(
            Line(points = {{20, 60}, {20, 30}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(powerSensor.pc, powerSensor.pv) annotation(
            Line(points = {{20, 30}, {10, 30}, {10, 20}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(powerSensor.nv, starLoad.plug_p) annotation(
            Line(points = {{30, 20}, {30, -60}, {20, -60}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(powerSensor.power, feedbackP.u1) annotation(
            Line(points = {{31, 28}, {40, 28}, {40, 20}, {52, 20}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(realExpression.y, feedbackP.u2) annotation(
            Line(points = {{60, 1}, {60, 12}}, color = {0, 0, 127}, smooth = Smooth.None));
          annotation(
            experiment(StopTime = 0.1, Interval = 0.0001),
            Documentation(info = "<HTML>
<p>
Test multiphase quasiRMS sensors: A sinusoidal source feeds a load consisting of resistor and inductor.
</p>
</HTML>"));
        end TestSensors;
        annotation(
          Documentation(info = "<HTML>
<p>
This package contains test examples of analog electrical multiphase circuits.
</p>

</HTML>", revisions = "<html>
<dl>
  <dt><b>Main Authors:</b></dt>
  <dd>
  <a href=\"http://www.haumer.at/\">Anton Haumer</a><br>
  Technical Consulting &amp; Electrical Engineering<br>
  A-3423 St.Andrae-Woerdern<br>Austria<br>
  email: <a href=\"mailto:a.haumer@haumer.at\">a.haumer@haumer.at</a>
  </dd>
  <dt><b>Release Notes:</b></dt>
  <dd>
  <ul>
  <li> v1.0 2004/10/01 Anton Haumer</li>
  </ul>
  </dd>
  <dt><b>Copyright:</b></dt>
  <dd>Copyright &copy; 1998-2015, Modelica Association and Anton Haumer.<br>
  <i>The Modelica package is <b>free</b> software; it can be redistributed and/or modified
  under the terms of the <b>Modelica license</b>, see the license conditions
  and the accompanying <b>disclaimer</b> in the documentation of package
  Modelica in file \"Modelica/package.mo\".</i></dd>
</dl>
</html>"));
      end Examples;

      package Basic "Basic components for electrical multiphase models"
        extends Modelica.Icons.Package;

        model Star "Star-connection"
          parameter Integer m(final min = 1) = 3 "Number of phases";
          Interfaces.PositivePlug plug_p(final m = m) annotation(
            Placement(transformation(extent = {{-110, -10}, {-90, 10}}, rotation = 0)));
          Modelica.Electrical.Analog.Interfaces.NegativePin pin_n annotation(
            Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
        equation
          for j in 1:m loop
            plug_p.pin[j].v = pin_n.v;
          end for;
          sum(plug_p.pin.i) + pin_n.i = 0;
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-150, 60}, {150, 120}}, textString = "%name", lineColor = {0, 0, 255}), Line(points = {{80, 0}, {0, 0}}, thickness = 0.5, color = {0, 0, 255}), Line(points = {{0, 0}, {-39, 68}}, thickness = 0.5, color = {0, 0, 255}), Line(points = {{0, 0}, {-38, -69}}, thickness = 0.5, color = {0, 0, 255}), Text(extent = {{-100, -110}, {100, -70}}, lineColor = {0, 0, 0}, textString = "m=%m"), Line(points = {{-90, 0}, {-40, 0}}, color = {0, 0, 255}), Line(points = {{80, 0}, {90, 0}}, color = {0, 0, 255})}),
            Documentation(info = "<HTML>
<p>
Connects all pins of plug_p to pin_n, thus establishing a so-called star-connection.
</p>
</HTML>"));
        end Star;

        model Delta "Delta (polygon) connection"
          parameter Integer m(final min = 2) = 3 "Number of phases";
          Interfaces.PositivePlug plug_p(final m = m) annotation(
            Placement(transformation(extent = {{-110, -10}, {-90, 10}}, rotation = 0)));
          Interfaces.NegativePlug plug_n(final m = m) annotation(
            Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
        equation
          for j in 1:m loop
            if j < m then
              plug_n.pin[j].v = plug_p.pin[j + 1].v;
              plug_n.pin[j].i + plug_p.pin[j + 1].i = 0;
            else
              plug_n.pin[j].v = plug_p.pin[1].v;
              plug_n.pin[j].i + plug_p.pin[1].i = 0;
            end if;
          end for;
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-150, 60}, {150, 120}}, lineColor = {0, 0, 255}, textString = "%name"), Line(points = {{-40, 68}, {-40, -70}, {79, 0}, {-40, 68}, {-40, 67}}, thickness = 0.5, color = {0, 0, 255}), Text(extent = {{-100, -110}, {100, -70}}, lineColor = {0, 0, 0}, textString = "m=%m"), Line(points = {{-90, 0}, {-40, 0}}, color = {0, 0, 255}), Line(points = {{80, 0}, {90, 0}}, color = {0, 0, 255})}),
            Documentation(info = "<HTML>
<p>
Connects in a cyclic way plug_n.pin[j] to plug_p.pin[j+1],
thus establishing a so-called delta (or polygon) connection
when used in parallel to another component.
</p>
</HTML>"));
        end Delta;

        model PlugToPin_p "Connect one (positive) Pin"
          parameter Integer m(final min = 1) = 3 "Number of phases";
          parameter Integer k(final min = 1, final max = m, start = 1) "Phase index";
          Interfaces.PositivePlug plug_p(final m = m) annotation(
            Placement(transformation(extent = {{-30, -10}, {-10, 10}}, rotation = 0)));
          Modelica.Electrical.Analog.Interfaces.PositivePin pin_p annotation(
            Placement(transformation(extent = {{10, -10}, {30, 10}}, rotation = 0)));
        equation
          pin_p.v = plug_p.pin[k].v;
          for j in 1:m loop
            plug_p.pin[j].i = if j == k then -pin_p.i else 0;
          end for;
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-150, 100}, {150, 40}}, textString = "%name", lineColor = {0, 0, 255}), Text(extent = {{-100, -60}, {100, -100}}, lineColor = {0, 0, 0}, textString = "k = %k"), Line(points = {{-20, 20}, {40, 20}, {40, -20}, {-20, -20}}, color = {0, 0, 0}), Rectangle(extent = {{-20, 20}, {40, -20}}, pattern = LinePattern.None, fillColor = {215, 215, 215}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Ellipse(extent = {{-40, 20}, {0, -20}}, lineColor = {0, 0, 0}, fillColor = {215, 215, 215}, fillPattern = FillPattern.Solid), Line(points = {{-20, 20}, {40, 20}, {40, -20}, {-20, -20}}, color = {0, 0, 0})}),
            Documentation(info = "<HTML>
<p>
Connects pin <i>k</i> of plug_p to pin_p, leaving the other pins of plug_p unconnected.
</p>
</HTML>"));
        end PlugToPin_p;

        model PlugToPin_n "Connect one (negative) Pin"
          parameter Integer m(final min = 1) = 3 "Number of phases";
          parameter Integer k(final min = 1, final max = m, start = 1) "Phase index";
          Interfaces.NegativePlug plug_n(final m = m) annotation(
            Placement(transformation(extent = {{-30, -10}, {-10, 10}}, rotation = 0)));
          Modelica.Electrical.Analog.Interfaces.NegativePin pin_n annotation(
            Placement(transformation(extent = {{10, -10}, {30, 10}}, rotation = 0)));
        equation
          pin_n.v = plug_n.pin[k].v;
          for j in 1:m loop
            plug_n.pin[j].i = if j == k then -pin_n.i else 0;
          end for;
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-150, 100}, {150, 40}}, textString = "%name", lineColor = {0, 0, 255}), Text(extent = {{-100, -60}, {100, -100}}, lineColor = {0, 0, 0}, textString = "k = %k"), Line(points = {{-20, 20}, {40, 20}, {40, -20}, {-20, -20}}, color = {0, 0, 0}), Rectangle(extent = {{-20, 20}, {40, -20}}, pattern = LinePattern.None, fillColor = {215, 215, 215}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Ellipse(extent = {{-40, 20}, {0, -20}}, lineColor = {0, 0, 0}, fillColor = {215, 215, 215}, fillPattern = FillPattern.Solid), Line(points = {{-20, 20}, {40, 20}, {40, -20}, {-20, -20}}, color = {0, 0, 0})}),
            Documentation(info = "<HTML>
<p>
Connects pin <i>k</i> of plug_n to pin_n, leaving the other pins of plug_n unconnected.
</p>
</HTML>"));
        end PlugToPin_n;

        model PlugToPins_p "Connect all (positive) Pins"
          parameter Integer m(final min = 1) = 3 "Number of phases";
          Interfaces.PositivePlug plug_p(final m = m) annotation(
            Placement(transformation(extent = {{-30, -10}, {-10, 10}}, rotation = 0)));
          Modelica.Electrical.Analog.Interfaces.PositivePin pin_p[m] annotation(
            Placement(transformation(extent = {{10, -10}, {30, 10}}, rotation = 0)));
        equation
          pin_p.v = plug_p.pin.v;
          plug_p.pin.i = -pin_p.i;
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-150, 100}, {150, 40}}, textString = "%name", lineColor = {0, 0, 255}), Line(points = {{-20, 20}, {40, 20}, {40, -20}, {-20, -20}}, color = {0, 0, 0}), Rectangle(extent = {{-20, 20}, {40, -20}}, pattern = LinePattern.None, fillColor = {170, 255, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Ellipse(extent = {{-40, 20}, {0, -20}}, lineColor = {0, 0, 0}, fillColor = {170, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{-20, 20}, {40, 20}, {40, -20}, {-20, -20}}, color = {0, 0, 0})}),
            Documentation(info = "<HTML>
<p>
Connects all pins of plug_p to the pin array pin_p.
</p>
</HTML>"));
        end PlugToPins_p;

        model PlugToPins_n "Connect all (negative) Pins"
          parameter Integer m(final min = 1) = 3 "Number of phases";
          Interfaces.NegativePlug plug_n(final m = m) annotation(
            Placement(transformation(extent = {{-30, -10}, {-10, 10}}, rotation = 0)));
          Modelica.Electrical.Analog.Interfaces.NegativePin pin_n[m] annotation(
            Placement(transformation(extent = {{10, -10}, {30, 10}}, rotation = 0)));
        equation
          pin_n.v = plug_n.pin.v;
          plug_n.pin.i = -pin_n.i;
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-150, 100}, {150, 40}}, textString = "%name", lineColor = {0, 0, 255}), Line(points = {{-20, 20}, {40, 20}, {40, -20}, {-20, -20}}, color = {0, 0, 0}), Rectangle(extent = {{-20, 20}, {40, -20}}, pattern = LinePattern.None, fillColor = {170, 255, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Ellipse(extent = {{-40, 20}, {0, -20}}, lineColor = {0, 0, 0}, fillColor = {170, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{-20, 20}, {40, 20}, {40, -20}, {-20, -20}}, color = {0, 0, 0})}),
            Documentation(info = "<HTML>
<p>
Connects all pins of plug_n to the pin array pin_n.
</p>
</HTML>"));
        end PlugToPins_n;

        model Resistor "Ideal linear electrical resistors"
          extends Interfaces.TwoPlug;
          parameter Modelica.SIunits.Resistance R[m](start = fill(1, m)) "Resistances R_ref at temperatures T_ref";
          parameter Modelica.SIunits.Temperature T_ref[m] = fill(300.15, m) "Reference temperatures";
          parameter Modelica.SIunits.LinearTemperatureCoefficient alpha[m] = zeros(m) "Temperature coefficients of resistances at reference temperatures";
          extends Modelica.Electrical.MultiPhase.Interfaces.ConditionalHeatPort(final mh = m, T = T_ref);
          Modelica.Electrical.Analog.Basic.Resistor resistor[m](final R = R, final T_ref = T_ref, final alpha = alpha, each final useHeatPort = useHeatPort, final T = T) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0)));
        equation
          connect(resistor.p, plug_p.pin) annotation(
            Line(points = {{-10, 0}, {-100, 0}}, color = {0, 0, 255}));
          connect(resistor.n, plug_n.pin) annotation(
            Line(points = {{10, 0}, {100, 0}}, color = {0, 0, 255}));
          connect(resistor.heatPort, heatPort) annotation(
            Line(points = {{0, -10}, {0, -100}}, color = {191, 0, 0}, smooth = Smooth.None));
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-70, 30}, {70, -30}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{-90, 0}, {-70, 0}}, color = {0, 0, 255}), Line(points = {{70, 0}, {90, 0}}, color = {0, 0, 255}), Text(extent = {{-150, 40}, {150, 100}}, textString = "%name", lineColor = {0, 0, 255}), Text(extent = {{-100, -100}, {100, -60}}, lineColor = {0, 0, 0}, textString = "m=%m")}),
            Documentation(info = "<HTML>
<p>
Contains m resistors (Modelica.Electrical.Analog.Basic.Resistor)
</p>
</HTML>"));
        end Resistor;

        model Conductor "Ideal linear electrical conductors"
          extends Interfaces.TwoPlug;
          parameter Modelica.SIunits.Conductance G[m](start = fill(1, m)) "Conductances G_ref at temperatures T_ref";
          parameter Modelica.SIunits.Temperature T_ref[m] = fill(300.15, m) "Reference temperatures";
          parameter Modelica.SIunits.LinearTemperatureCoefficient alpha[m] = zeros(m) "Temperature coefficients of conductances at reference temperatures";
          extends Modelica.Electrical.MultiPhase.Interfaces.ConditionalHeatPort(final mh = m, T = T_ref);
          Modelica.Electrical.Analog.Basic.Conductor conductor[m](final G = G, final T_ref = T_ref, final alpha = alpha, each final useHeatPort = useHeatPort, final T = T) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0)));
        equation
          connect(plug_p.pin, conductor.p) annotation(
            Line(points = {{-100, 0}, {-10, 0}}, color = {0, 0, 255}));
          connect(plug_n.pin, conductor.n) annotation(
            Line(points = {{100, 0}, {10, 0}}, color = {0, 0, 255}));
          connect(conductor.heatPort, heatPort) annotation(
            Line(points = {{0, -10}, {0, -100}}, color = {191, 0, 0}, smooth = Smooth.None));
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-70, 30}, {70, -30}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{-90, 0}, {-70, 0}}, color = {0, 0, 255}), Line(points = {{70, 0}, {90, 0}}, color = {0, 0, 255}), Text(extent = {{-150, 40}, {150, 100}}, textString = "%name", lineColor = {0, 0, 255}), Text(extent = {{-100, -100}, {100, -60}}, lineColor = {0, 0, 0}, textString = "m=%m")}),
            Documentation(info = "<HTML>
<p>
Contains m conductors (Modelica.Electrical.Analog.Basic.Conductor)
</p>
</HTML>"));
        end Conductor;

        model Capacitor "Ideal linear electrical capacitors"
          extends Interfaces.TwoPlug;
          parameter Modelica.SIunits.Capacitance C[m](start = fill(1, m)) "Capacitance";
          Modelica.Electrical.Analog.Basic.Capacitor capacitor[m](final C = C) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0)));
        equation
          connect(capacitor.p, plug_p.pin) annotation(
            Line(points = {{-10, 0}, {-100, 0}}, color = {0, 0, 255}));
          connect(capacitor.n, plug_n.pin) annotation(
            Line(points = {{10, 0}, {100, 0}}, color = {0, 0, 255}));
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-150, 40}, {150, 100}}, textString = "%name", lineColor = {0, 0, 255}), Line(points = {{-14, 28}, {-14, -28}}, thickness = 0.5, color = {0, 0, 255}), Line(points = {{14, 28}, {14, -28}}, thickness = 0.5, color = {0, 0, 255}), Line(points = {{-90, 0}, {-14, 0}}, color = {0, 0, 255}), Line(points = {{14, 0}, {90, 0}}, color = {0, 0, 255}), Text(extent = {{-100, -100}, {100, -60}}, lineColor = {0, 0, 0}, textString = "m=%m")}),
            Documentation(info = "<HTML>
<p>
Contains m capacitors (Modelica.Electrical.Analog.Basic.Capacitor)
</p>
</HTML>"));
        end Capacitor;

        model Inductor "Ideal linear electrical inductors"
          extends Interfaces.TwoPlug;
          parameter Modelica.SIunits.Inductance L[m](start = fill(1, m)) "Inductance";
          Modelica.Electrical.Analog.Basic.Inductor inductor[m](final L = L) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0)));
        equation
          connect(inductor.p, plug_p.pin) annotation(
            Line(points = {{-10, 0}, {-100, 0}}, color = {0, 0, 255}));
          connect(inductor.n, plug_n.pin) annotation(
            Line(points = {{10, 0}, {100, 0}}, color = {0, 0, 255}));
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-60, -15}, {-30, 15}}, lineColor = {0, 0, 255}), Ellipse(extent = {{-30, -15}, {0, 15}}, lineColor = {0, 0, 255}), Ellipse(extent = {{0, -15}, {30, 15}}, lineColor = {0, 0, 255}), Ellipse(extent = {{30, -15}, {60, 15}}, lineColor = {0, 0, 255}), Rectangle(extent = {{-60, -30}, {60, 0}}, lineColor = {255, 255, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{60, 0}, {90, 0}}, color = {0, 0, 255}), Line(points = {{-90, 0}, {-60, 0}}, color = {0, 0, 255}), Text(extent = {{-150, 40}, {150, 100}}, textString = "%name", lineColor = {0, 0, 255}), Text(extent = {{-100, -100}, {100, -60}}, lineColor = {0, 0, 0}, textString = "m=%m")}),
            Documentation(info = "<HTML>
<p>
Contains m inductors (Modelica.Electrical.Analog.Basic.Inductor)
</p>
</HTML>"));
        end Inductor;

        model SaturatingInductor "Simple model of inductors with saturation"
          extends Interfaces.TwoPlug;
          parameter Modelica.SIunits.Current Inom[m](start = fill(1, m)) "Nominal current";
          parameter Modelica.SIunits.Inductance Lnom[m](start = fill(1, m)) "Nominal inductance at Nominal current";
          parameter Modelica.SIunits.Inductance Lzer[m](start = {2 * Lnom[j] for j in 1:m}) "Inductance near current=0";
          parameter Modelica.SIunits.Inductance Linf[m](start = {Lnom[j] / 2 for j in 1:m}) "Inductance at large currents";
          Modelica.Electrical.Analog.Basic.SaturatingInductor saturatingInductor[m](final Inom = Inom, final Lnom = Lnom, final Lzer = Lzer, final Linf = Linf) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0)));
        equation
          connect(saturatingInductor.p, plug_p.pin) annotation(
            Line(points = {{-10, 0}, {-100, 0}}, color = {0, 0, 255}));
          connect(saturatingInductor.n, plug_n.pin) annotation(
            Line(points = {{10, 0}, {100, 0}}, color = {0, 0, 255}));
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-60, -15}, {-30, 15}}, lineColor = {0, 0, 255}), Ellipse(extent = {{-30, -15}, {0, 15}}, lineColor = {0, 0, 255}), Ellipse(extent = {{0, -15}, {30, 15}}, lineColor = {0, 0, 255}), Ellipse(extent = {{30, -15}, {60, 15}}, lineColor = {0, 0, 255}), Rectangle(extent = {{-60, -20}, {60, 0}}, lineColor = {255, 255, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{60, 0}, {90, 0}}, color = {0, 0, 255}), Line(points = {{-90, 0}, {-60, 0}}, color = {0, 0, 255}), Rectangle(extent = {{-60, -10}, {60, -20}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.Sphere, fillColor = {0, 0, 255}), Text(extent = {{-150, 40}, {150, 100}}, textString = "%name", lineColor = {0, 0, 255}), Text(extent = {{-100, -100}, {100, -60}}, lineColor = {0, 0, 0}, textString = "m=%m")}),
            Documentation(info = "<HTML>
<p>
Contains m saturating inductors (Modelica.Electrical.Analog.Basic.SaturatingInductor)
</p>
<p>
<b>Attention!!!</b><br>
Each element of the array of saturatingInductors is only dependent on the current flowing through this element.
</p>
</HTML>"));
        end SaturatingInductor;

        model MutualInductor "Linear mutual inductor"
          extends Modelica.Electrical.MultiPhase.Interfaces.OnePort;
          constant Real epsilon = 1e-9 "Relative accuracy tolerance of matrix symmetry";
          parameter Integer m = 3 "Number of phases";
          parameter Modelica.SIunits.Inductance L[m, m] "Mutual inductance matrix";
        initial equation
          if abs(Modelica.Math.Matrices.det(L)) < epsilon then
            Modelica.Utilities.Streams.print("Warning: mutual inductance matrix singular!");
          end if;
        equation
          assert(sum(abs(L - transpose(L))) < epsilon * sum(abs(L)), "Mutual inductance matrix is not symmetric");
          for j in 1:m loop
            v[j] = sum(L[j, k] * der(i[k]) for k in 1:m);
          end for;
          annotation(
            Documentation(info = "<html>
<p>
Model of a multi phase inductor providing a mutual inductance matrix model.
</p>
<H4>Implementation</H4>
<pre>
  v[1] = L[1,1]*der(i[1]) + L[1,2]*der(i[2]) + ... + L[1,m]*der(i[m])
  v[2] = L[2,1]*der(i[1]) + L[2,2]*der(i[2]) + ... + L[2,m]*der(i[m])
    :              :                         :                                :
  v[m] = L[m,1]*der(i[1]) + L[m,2]*der(i[2]) + ... + L[m,m]*der(i[m])
</pre>

</html>"),
            Icon(graphics = {Ellipse(extent = {{30, -50}, {60, 10}}, lineColor = {0, 0, 255}), Ellipse(extent = {{0, -50}, {30, 10}}, lineColor = {0, 0, 255}), Ellipse(extent = {{-30, -50}, {0, 10}}, lineColor = {0, 0, 255}), Ellipse(extent = {{-60, -50}, {-30, 10}}, lineColor = {0, 0, 255}), Line(points = {{-80, 20}, {-80, -20}, {-60, -20}}, color = {0, 0, 255}), Line(points = {{-80, 20}, {-60, 20}}, color = {0, 0, 255}), Ellipse(extent = {{-60, -10}, {-30, 50}}, lineColor = {0, 0, 255}), Ellipse(extent = {{-30, -10}, {0, 50}}, lineColor = {0, 0, 255}), Ellipse(extent = {{0, -10}, {30, 50}}, lineColor = {0, 0, 255}), Ellipse(extent = {{30, -10}, {60, 50}}, lineColor = {0, 0, 255}), Line(points = {{60, 20}, {80, 20}}, color = {0, 0, 255}), Line(points = {{80, 20}, {80, -20}, {60, -20}}, color = {0, 0, 255}), Rectangle(extent = {{-60, 0}, {60, 20}}, lineColor = {255, 255, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-60, -20}, {60, 0}}, lineColor = {255, 255, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{-90, 0}, {-80, 0}}, color = {0, 0, 255}), Line(points = {{80, 0}, {90, 0}}, color = {0, 0, 255}), Text(extent = {{0, 60}, {0, 100}}, textString = "%name", lineColor = {0, 0, 255}), Text(extent = {{-100, -100}, {100, -60}}, lineColor = {0, 0, 0}, textString = "m=%m")}));
        end MutualInductor;

        model ZeroInductor "Linear zero sequence inductor"
          extends Modelica.Electrical.MultiPhase.Interfaces.OnePort;
          parameter Modelica.SIunits.Inductance Lzero "Zero sequence inductance";
          Modelica.SIunits.Current i0;
          Modelica.SIunits.Voltage v0;
        equation
          m * i0 = sum(i);
          v0 = Lzero * der(i0);
          v = fill(v0, m);
          annotation(
            Documentation(info = "<html>
<p>
Model of a multi phase zero sequence inductor.
</p>
<H4>Implementation</H4>
<pre>
  v = Lzero*sum(der(i)) = Lzero*der(sum(i))
</pre>

</html>"),
            Icon(graphics = {Text(extent = {{0, 60}, {0, 100}}, textString = "%name", lineColor = {0, 0, 255}), Text(extent = {{-100, -100}, {100, -60}}, lineColor = {0, 0, 0}, textString = "m=%m"), Line(points = {{-90, 0}, {-50, 0}}, color = {0, 0, 255}), Line(points = {{52, 0}, {90, 0}}, color = {0, 0, 255}), Ellipse(extent = {{-50, 30}, {-8, -32}}, lineColor = {0, 0, 255}), Ellipse(extent = {{-20, 30}, {22, -32}}, lineColor = {0, 0, 255}), Ellipse(extent = {{10, 30}, {52, -32}}, lineColor = {0, 0, 255})}));
        end ZeroInductor;

        model Transformer "Multiphase Transformer"
          extends Interfaces.FourPlug;
          parameter Modelica.SIunits.Inductance L1[m](start = fill(1, m)) "Primary inductance";
          parameter Modelica.SIunits.Inductance L2[m](start = fill(1, m)) "Secondary inductance";
          parameter Modelica.SIunits.Inductance M[m](start = fill(1, m)) "Coupling inductance";
          Modelica.Electrical.Analog.Basic.Transformer transformer[m](final L1 = L1, final L2 = L2, final M = M) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0)));
        equation
          connect(plug_p1.pin, transformer.p1) annotation(
            Line(points = {{-100, 100}, {-10, 100}, {-10, 5}}, color = {0, 0, 255}));
          connect(plug_p2.pin, transformer.p2) annotation(
            Line(points = {{100, 100}, {10, 100}, {10, 5}}, color = {0, 0, 255}));
          connect(plug_n1.pin, transformer.n1) annotation(
            Line(points = {{-100, -100}, {-10, -100}, {-10, -5}}, color = {0, 0, 255}));
          connect(plug_n2.pin, transformer.n2) annotation(
            Line(points = {{100, -100}, {10, -100}, {10, -5}}, color = {0, 0, 255}));
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-45, -50}, {-20, -25}}, lineColor = {0, 0, 255}), Ellipse(extent = {{-45, -25}, {-20, 0}}, lineColor = {0, 0, 255}), Ellipse(extent = {{-45, 0}, {-20, 25}}, lineColor = {0, 0, 255}), Ellipse(extent = {{-45, 25}, {-20, 50}}, lineColor = {0, 0, 255}), Rectangle(extent = {{-72, -60}, {-33, 60}}, lineColor = {255, 255, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{-90, 50}, {-32, 50}}, color = {0, 0, 255}), Line(points = {{-90, -50}, {-32, -50}}, color = {0, 0, 255}), Ellipse(extent = {{20, -50}, {45, -25}}, lineColor = {0, 0, 255}), Ellipse(extent = {{20, -25}, {45, 0}}, lineColor = {0, 0, 255}), Ellipse(extent = {{20, 0}, {45, 25}}, lineColor = {0, 0, 255}), Ellipse(extent = {{20, 25}, {45, 50}}, lineColor = {0, 0, 255}), Rectangle(extent = {{33, -60}, {72, 60}}, lineColor = {255, 255, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{32, 50}, {90, 50}}, color = {0, 0, 255}), Line(points = {{32, -50}, {90, -50}}, color = {0, 0, 255}), Text(extent = {{-100, 20}, {-58, -20}}, textString = "L1", lineColor = {0, 0, 255}), Text(extent = {{60, 20}, {100, -20}}, textString = "L2", lineColor = {0, 0, 255}), Text(extent = {{-20, 20}, {20, -20}}, textString = "M", lineColor = {0, 0, 255}), Text(extent = {{-80, -100}, {80, -60}}, lineColor = {0, 0, 0}, textString = "m=%m"), Text(extent = {{-100, 50}, {100, 110}}, textString = "%name", lineColor = {0, 0, 255})}),
            Documentation(info = "<HTML>
<p>
Contains m transformers (Modelica.Electrical.Analog.Basic.Transformer)
</p>
</HTML>"));
        end Transformer;

        model VariableResistor "Ideal linear electrical resistors with variable resistance"
          extends Interfaces.TwoPlug;
          parameter Modelica.SIunits.Temperature T_ref[m] = fill(300.15, m) "Reference temperatures";
          parameter Modelica.SIunits.LinearTemperatureCoefficient alpha[m] = zeros(m) "Temperature coefficients of resistances at reference temperatures";
          extends Modelica.Electrical.MultiPhase.Interfaces.ConditionalHeatPort(final mh = m, T = T_ref);
          Modelica.Blocks.Interfaces.RealInput R[m](each unit = "Ohm") annotation(
            Placement(transformation(origin = {0, 100}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          Modelica.Electrical.Analog.Basic.VariableResistor variableResistor[m](final T_ref = T_ref, final alpha = alpha, each final useHeatPort = useHeatPort, final T = T) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0)));
        equation
          connect(variableResistor.p, plug_p.pin) annotation(
            Line(points = {{-10, 0}, {-100, 0}}, color = {0, 0, 255}));
          connect(variableResistor.n, plug_n.pin) annotation(
            Line(points = {{10, 0}, {100, 0}}, color = {0, 0, 255}));
          connect(R, variableResistor.R) annotation(
            Line(points = {{0, 100}, {0, 11}}, color = {0, 0, 255}));
          connect(variableResistor.heatPort, heatPort) annotation(
            Line(points = {{0, -10}, {0, -100}}, color = {191, 0, 0}, smooth = Smooth.None));
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-90, 0}, {-70, 0}}, color = {0, 0, 255}), Rectangle(extent = {{-70, 30}, {70, -30}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{70, 0}, {90, 0}}, color = {0, 0, 255}), Line(points = {{0, 90}, {0, 30}}, color = {0, 0, 255}), Text(extent = {{-150, -100}, {150, -40}}, textString = "%name", lineColor = {0, 0, 255}), Text(extent = {{-100, 40}, {100, 80}}, lineColor = {0, 0, 0}, textString = "m=%m")}),
            Documentation(info = "<HTML>
<p>
Contains m variable resistors (Modelica.Electrical.Analog.Basic.VariableResistor)
</p>
<P>
<b>Attention!!!</b><br>
  It is recommended that none of the R_Port signals should not cross the zero value.
  Otherwise depending on the surrounding circuit the probability of singularities is high.
</P>
</HTML>"));
        end VariableResistor;

        model VariableConductor "Ideal linear electrical conductors with variable conductance"
          extends Interfaces.TwoPlug;
          parameter Modelica.SIunits.Temperature T_ref[m] = fill(300.15, m) "Reference temperatures";
          parameter Modelica.SIunits.LinearTemperatureCoefficient alpha[m] = zeros(m) "Temperature coefficients of conductances at reference temperatures";
          extends Modelica.Electrical.MultiPhase.Interfaces.ConditionalHeatPort(final mh = m, T = T_ref);
          Modelica.Blocks.Interfaces.RealInput G[m](each unit = "S") annotation(
            Placement(transformation(origin = {0, 100}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          Modelica.Electrical.Analog.Basic.VariableConductor variableConductor[m](final T_ref = T_ref, final alpha = alpha, each final useHeatPort = useHeatPort, final T = T) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0)));
        equation
          connect(variableConductor.p, plug_p.pin) annotation(
            Line(points = {{-10, 0}, {-100, 0}}, color = {0, 0, 255}));
          connect(variableConductor.n, plug_n.pin) annotation(
            Line(points = {{10, 0}, {100, 0}}, color = {0, 0, 255}));
          connect(G, variableConductor.G) annotation(
            Line(points = {{0, 100}, {0, 11}}, color = {0, 0, 255}));
          connect(variableConductor.heatPort, heatPort) annotation(
            Line(points = {{0, -10}, {0, -100}}, color = {191, 0, 0}, smooth = Smooth.None));
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-90, 0}, {-70, 0}}, color = {0, 0, 255}), Rectangle(extent = {{-70, 30}, {70, -30}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{70, 0}, {90, 0}}, color = {0, 0, 255}), Line(points = {{0, 90}, {0, 30}}, color = {0, 0, 255}), Text(extent = {{-150, -100}, {150, -40}}, textString = "%name", lineColor = {0, 0, 255}), Text(extent = {{-100, 40}, {100, 80}}, lineColor = {0, 0, 0}, textString = "m=%m")}),
            Documentation(info = "<HTML>
<p>
Contains m variable conductors (Modelica.Electrical.Analog.Basic.VariableConductor)
</p>
<P>
<b>Attention!!!</b><br>
  It is recommended that none of the G_Port signals should not cross the zero value.
  Otherwise depending on the surrounding circuit the probability of singularities is high.
</P>
</HTML>"));
        end VariableConductor;

        model VariableCapacitor "Ideal linear electrical capacitors with variable capacitance"
          extends Interfaces.TwoPlug;
          parameter Modelica.SIunits.Capacitance Cmin[m] = fill(Modelica.Constants.eps, m) "minimum Capacitance";
          Modelica.Blocks.Interfaces.RealInput C[m](each unit = "F") annotation(
            Placement(transformation(origin = {0, 100}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          Modelica.Electrical.Analog.Basic.VariableCapacitor variableCapacitor[m](final Cmin = Cmin) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0)));
        equation
          connect(variableCapacitor.p, plug_p.pin) annotation(
            Line(points = {{-10, 0}, {-100, 0}}, color = {0, 0, 255}));
          connect(variableCapacitor.n, plug_n.pin) annotation(
            Line(points = {{10, 0}, {100, 0}}, color = {0, 0, 255}));
          connect(C, variableCapacitor.C) annotation(
            Line(points = {{0, 100}, {0, 11}}, color = {0, 0, 255}));
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{0, 90}, {0, 30}}, color = {0, 0, 255}), Text(extent = {{-150, -100}, {150, -40}}, textString = "%name", lineColor = {0, 0, 255}), Text(extent = {{-100, 40}, {100, 80}}, lineColor = {0, 0, 0}, textString = "m=%m"), Line(points = {{-90, 0}, {-14, 0}}, color = {0, 0, 255}), Line(points = {{14, 0}, {90, 0}}, color = {0, 0, 255}), Line(points = {{-14, 28}, {-14, -28}}, thickness = 0.5, color = {0, 0, 255}), Line(points = {{14, 28}, {14, -28}}, thickness = 0.5, color = {0, 0, 255})}),
            Documentation(info = "<HTML>
<p>
Contains m variable capacitors (Modelica.Electrical.Analog.Basic.VariableCapacitor)
</p>
<P>
It is required that each C_Port.signal &ge; 0, otherwise an
assertion is raised. To avoid a variable index system,<br>
C = Cmin, if 0 &le; C_Port.signal &lt; Cmin, where
Cmin is a parameter with default value Modelica.Constants.eps.
</P>
</HTML>"));
        end VariableCapacitor;

        model VariableInductor "Ideal linear electrical inductors with variable inductance"
          extends Interfaces.TwoPlug;
          parameter Modelica.SIunits.Inductance Lmin[m] = fill(Modelica.Constants.eps, m) "minimum Inductance";
          Modelica.Blocks.Interfaces.RealInput L[m](each unit = "H") annotation(
            Placement(transformation(origin = {0, 100}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          Modelica.Electrical.Analog.Basic.VariableInductor variableInductor[m](final Lmin = Lmin) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0)));
        equation
          connect(variableInductor.p, plug_p.pin) annotation(
            Line(points = {{-10, 0}, {-100, 0}}, color = {0, 0, 255}));
          connect(variableInductor.n, plug_n.pin) annotation(
            Line(points = {{10, 0}, {100, 0}}, color = {0, 0, 255}));
          connect(L, variableInductor.L) annotation(
            Line(points = {{0, 100}, {0, 10.8}}, color = {0, 0, 255}));
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-150, -100}, {150, -40}}, textString = "%name", lineColor = {0, 0, 255}), Text(extent = {{-100, 40}, {100, 80}}, lineColor = {0, 0, 0}, textString = "m=%m"), Line(points = {{-90, 0}, {-60, 0}}, color = {0, 0, 255}), Line(points = {{60, 0}, {90, 0}}, color = {0, 0, 255}), Line(points = {{0, 90}, {0, 8}}, color = {0, 0, 255}), Ellipse(extent = {{-60, -15}, {-30, 15}}, lineColor = {0, 0, 255}), Ellipse(extent = {{-30, -15}, {0, 15}}, lineColor = {0, 0, 255}), Ellipse(extent = {{0, -15}, {30, 15}}, lineColor = {0, 0, 255}), Ellipse(extent = {{30, -15}, {60, 15}}, lineColor = {0, 0, 255}), Rectangle(extent = {{-60, -30}, {60, 0}}, lineColor = {255, 255, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid)}),
            Documentation(info = "<HTML>
<p>
Contains m variable inductors (Modelica.Electrical.Analog.Basic.VariableInductor)
</p>
<P>
It is required that each L_Port.signal &ge; 0, otherwise an
assertion is raised. To avoid a variable index system,<br>
L = Lmin, if 0 &le; L_Port.signal &lt; Lmin, where
Lmin is a parameter with default value Modelica.Constants.eps.
</P>
</HTML>"));
        end VariableInductor;
        annotation(
          Documentation(info = "<HTML>
<p>
This package contains basic analog electrical multiphase components.
</p>

</HTML>", revisions = "<html>
<dl>
  <dt><b>Main Authors:</b></dt>
  <dd>
  <a href=\"http://www.haumer.at/\">Anton Haumer</a><br>
  Technical Consulting &amp; Electrical Engineering<br>
  A-3423 St.Andrae-Woerdern<br>Austria<br>
  email: <a href=\"mailto:a.haumer@haumer.at\">a.haumer@haumer.at</a>
  </dd>
  <dt><b>Release Notes:</b></dt>
  <dd>
  <ul>
  <li> v1.0 2004/10/01 Anton Haumer</li>
  </ul>
  </dd>
  <dt><b>Copyright:</b></dt>
  <dd>Copyright &copy; 1998-2015, Modelica Association and Anton Haumer.<br>
  <i>The Modelica package is <b>free</b> software; it can be redistributed and/or modified
  under the terms of the <b>Modelica license</b>, see the license conditions
  and the accompanying <b>disclaimer</b> in the documentation of package
  Modelica in file \"Modelica/package.mo\".</i></dd>
</dl>
</html>"),
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(origin = {11.626, 40}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, extent = {{-80, -70}, {60, -10}}), Line(origin = {11.626, 40}, points = {{60, -40}, {80, -40}}, color = {0, 0, 255}), Line(points = {{-88.374, 0}, {-68.374, 0}}, color = {0, 0, 255})}));
      end Basic;

      package Ideal "Multiphase components with idealized behaviour"
        extends Modelica.Icons.Package;

        model IdealDiode "Multiphase ideal diode"
          extends Interfaces.TwoPlug;
          parameter Modelica.SIunits.Resistance Ron[m](final min = zeros(m), start = fill(1.E-5, m)) "Closed diode resistance";
          parameter Modelica.SIunits.Conductance Goff[m](final min = zeros(m), start = fill(1.E-5, m)) "Opened diode conductance";
          parameter Modelica.SIunits.Voltage Vknee[m](final min = zeros(m), start = zeros(m)) "Threshold voltage";
          extends Modelica.Electrical.MultiPhase.Interfaces.ConditionalHeatPort(final mh = m, final T = fill(293.15, m));
          Modelica.Electrical.Analog.Ideal.IdealDiode idealDiode[m](final Ron = Ron, final Goff = Goff, final Vknee = Vknee, each final useHeatPort = useHeatPort) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0)));
        equation
          connect(plug_p.pin, idealDiode.p) annotation(
            Line(points = {{-100, 0}, {-10, 0}}, color = {0, 0, 255}));
          connect(idealDiode.n, plug_n.pin) annotation(
            Line(points = {{10, 0}, {100, 0}}, color = {0, 0, 255}));
          connect(idealDiode.heatPort, heatPort) annotation(
            Line(points = {{0, -10}, {0, -100}}, color = {191, 0, 0}, smooth = Smooth.None));
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-150, -40}, {150, -100}}, textString = "%name", lineColor = {0, 0, 255}), Line(points = {{-90, 0}, {40, 0}}, color = {0, 0, 255}), Polygon(points = {{30, 0}, {-30, 40}, {-30, -40}, {30, 0}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255}), Line(points = {{30, 40}, {30, -40}}, color = {0, 0, 255}), Line(points = {{40, 0}, {90, 0}}, color = {0, 0, 255}), Text(extent = {{-80, 100}, {80, 60}}, lineColor = {0, 0, 0}, textString = "m=%m")}),
            Documentation(info = "<HTML>
<p>
Contains m ideal diodes (Modelica.Electrical.Analog.Ideal.IdealDiode).
</p>
</HTML>"));
        end IdealDiode;

        model IdealThyristor "Multiphase ideal thyristor"
          extends Interfaces.TwoPlug;
          parameter Modelica.SIunits.Resistance Ron[m](final min = zeros(m), start = fill(1.E-5, m)) "Closed thyristor resistance";
          parameter Modelica.SIunits.Conductance Goff[m](final min = zeros(m), start = fill(1.E-5, m)) "Opened thyristor conductance";
          parameter Modelica.SIunits.Voltage Vknee[m](final min = zeros(m), start = zeros(m)) "Threshold voltage";
          Boolean off[m](start = fill(true, m)) "Alias of boolean thyristor off; type, e.g.: off(start=fill(true,m))" annotation(
            Dialog);
          extends Modelica.Electrical.MultiPhase.Interfaces.ConditionalHeatPort(final mh = m, final T = fill(293.15, m));
          Modelica.Blocks.Interfaces.BooleanInput fire[m] annotation(
            Placement(transformation(origin = {70, 110}, extent = {{-20, -20}, {20, 20}}, rotation = 270)));
          Modelica.Electrical.Analog.Ideal.IdealThyristor idealThyristor[m](final Ron = Ron, final Goff = Goff, final Vknee = Vknee, each final useHeatPort = useHeatPort) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0)));
        equation
          off = idealThyristor.off;
          connect(plug_p.pin, idealThyristor.p) annotation(
            Line(points = {{-100, 8.88178e-16}, {-60, 8.88178e-16}, {-60, 0}, {-38, 0}, {-38, 8.88178e-16}, {-10, 8.88178e-16}}, color = {0, 0, 255}));
          connect(idealThyristor.n, plug_n.pin) annotation(
            Line(points = {{10, 0}, {100, 0}}, color = {0, 0, 255}));
          connect(fire, idealThyristor.fire) annotation(
            Line(points = {{70, 110}, {70, 80}, {7, 80}, {7, 11}}, color = {255, 0, 255}));
          connect(idealThyristor.heatPort, heatPort) annotation(
            Line(points = {{0, -10}, {0, -100}}, color = {191, 0, 0}, smooth = Smooth.None));
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-150, -40}, {150, -100}}, textString = "%name", lineColor = {0, 0, 255}), Line(points = {{-90, 0}, {40, 0}}, color = {0, 0, 255}), Polygon(points = {{30, 0}, {-30, 40}, {-30, -40}, {30, 0}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255}), Line(points = {{30, 40}, {30, -40}}, color = {0, 0, 255}), Line(points = {{40, 0}, {90, 0}}, color = {0, 0, 255}), Line(points = {{40, 50}, {60, 30}}, color = {0, 0, 255}), Line(points = {{30, 20}, {70, 60}, {70, 90}}, color = {0, 0, 255}), Text(extent = {{-100, 100}, {60, 60}}, lineColor = {0, 0, 0}, textString = "m=%m")}),
            Documentation(info = "<HTML>
<p>
Contains m ideal thyristors (Modelica.Electrical.Analog.Ideal.IdealThyristor).
</p>
</HTML>"));
        end IdealThyristor;

        model IdealGTOThyristor "Multiphase ideal GTO thyristor"
          extends Interfaces.TwoPlug;
          parameter Modelica.SIunits.Resistance Ron[m](final min = zeros(m), start = fill(1.E-5, m)) "Closed thyristor resistance";
          parameter Modelica.SIunits.Conductance Goff[m](final min = zeros(m), start = fill(1.E-5, m)) "Opened thyristor conductance";
          parameter Modelica.SIunits.Voltage Vknee[m](final min = zeros(m), start = zeros(m)) "Threshold voltage";
          extends Modelica.Electrical.MultiPhase.Interfaces.ConditionalHeatPort(final mh = m, final T = fill(293.15, m));
          Modelica.Blocks.Interfaces.BooleanInput fire[m] annotation(
            Placement(transformation(origin = {70, 110}, extent = {{-20, -20}, {20, 20}}, rotation = 270)));
          Modelica.Electrical.Analog.Ideal.IdealGTOThyristor idealGTOThyristor[m](final Ron = Ron, final Goff = Goff, final Vknee = Vknee, each final useHeatPort = useHeatPort) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0)));
        equation
          connect(idealGTOThyristor.p, plug_p.pin) annotation(
            Line(points = {{-10, 0}, {-100, 0}}, color = {0, 0, 255}));
          connect(idealGTOThyristor.n, plug_n.pin) annotation(
            Line(points = {{10, 0}, {100, 0}}, color = {0, 0, 255}));
          connect(fire, idealGTOThyristor.fire) annotation(
            Line(points = {{70, 110}, {70, 80}, {7, 80}, {7, 11}}, color = {255, 0, 255}));
          connect(idealGTOThyristor.heatPort, heatPort) annotation(
            Line(points = {{0, -10}, {0, -100}}, color = {191, 0, 0}, smooth = Smooth.None));
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-150, -40}, {150, -100}}, textString = "%name", lineColor = {0, 0, 255}), Line(points = {{-90, 0}, {40, 0}}, color = {0, 0, 255}), Polygon(points = {{30, 0}, {-30, 40}, {-30, -40}, {30, 0}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255}), Line(points = {{30, 40}, {30, -40}}, color = {0, 0, 255}), Line(points = {{40, 0}, {90, 0}}, color = {0, 0, 255}), Line(points = {{40, 50}, {60, 30}}, color = {0, 0, 255}), Line(points = {{30, 20}, {70, 60}, {70, 90}}, color = {0, 0, 255}), Text(extent = {{-100, 100}, {60, 60}}, lineColor = {0, 0, 0}, textString = "m=%m")}),
            Documentation(info = "<HTML>
<p>
Contains m ideal GTO thyristors (Modelica.Electrical.Analog.Ideal.IdealGTOThyristor).
</p>
</HTML>"));
        end IdealGTOThyristor;

        model IdealCommutingSwitch "Multiphase ideal commuting switch"
          parameter Integer m(final min = 1) = 3 "Number of phases";
          parameter Modelica.SIunits.Resistance Ron[m](final min = zeros(m), start = fill(1.E-5, m)) "Closed switch resistance";
          parameter Modelica.SIunits.Conductance Goff[m](final min = zeros(m), start = fill(1.E-5, m)) "Opened switch conductance";
          extends Modelica.Electrical.MultiPhase.Interfaces.ConditionalHeatPort(final mh = m, final T = fill(293.15, m));
          Modelica.Blocks.Interfaces.BooleanInput control[m] "true => p--n2 connected, false => p--n1 connected" annotation(
            Placement(transformation(origin = {0, 80}, extent = {{-20, -20}, {20, 20}}, rotation = 270)));
          Interfaces.PositivePlug plug_p(final m = m) annotation(
            Placement(transformation(extent = {{-110, -10}, {-90, 10}}, rotation = 0)));
          Interfaces.NegativePlug plug_n2(final m = m) annotation(
            Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
          Interfaces.NegativePlug plug_n1(final m = m) annotation(
            Placement(transformation(extent = {{90, 40}, {110, 60}}, rotation = 0)));
          Modelica.Electrical.Analog.Ideal.IdealCommutingSwitch idealCommutingSwitch[m](final Ron = Ron, final Goff = Goff, each final useHeatPort = useHeatPort) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0)));
        equation
          connect(plug_p.pin, idealCommutingSwitch.p) annotation(
            Line(points = {{-100, 0}, {-10, 0}}, color = {0, 0, 255}));
          connect(idealCommutingSwitch.n2, plug_n2.pin) annotation(
            Line(points = {{10, 0}, {100, 0}}, color = {0, 0, 255}));
          connect(idealCommutingSwitch.n1, plug_n1.pin) annotation(
            Line(points = {{10, 5}, {10, 50}, {100, 50}}, color = {0, 0, 255}));
          connect(control, idealCommutingSwitch.control) annotation(
            Line(points = {{0, 80}, {0, 8}}, color = {255, 0, 255}));
          connect(idealCommutingSwitch.heatPort, heatPort) annotation(
            Line(points = {{0, -10}, {0, -100}}, color = {191, 0, 0}, smooth = Smooth.None));
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-150, -40}, {150, -100}}, textString = "%name", lineColor = {0, 0, 255}), Text(extent = {{-100, 100}, {-20, 60}}, lineColor = {0, 0, 0}, textString = "m="), Text(extent = {{20, 100}, {100, 60}}, lineColor = {0, 0, 0}, textString = "%m"), Ellipse(extent = {{-44, 4}, {-36, -4}}, lineColor = {0, 0, 255}), Line(points = {{-90, 0}, {-44, 0}}, color = {0, 0, 255}), Line(points = {{-37, 2}, {40, 50}}, color = {0, 0, 255}), Line(points = {{40, 50}, {90, 50}}, color = {0, 0, 255}), Line(points = {{0, 90}, {0, 25}}, color = {0, 0, 255}), Line(points = {{40, 0}, {90, 0}}, color = {0, 0, 255})}),
            Documentation(info = "<HTML>
<p>
Contains m ideal commuting switches (Modelica.Electrical.Analog.Ideal.IdealCommutingSwitch).
</p>
</HTML>"));
        end IdealCommutingSwitch;

        model IdealIntermediateSwitch "Multiphase ideal intermediate switch"
          parameter Integer m(final min = 1) = 3 "Number of phases";
          parameter Modelica.SIunits.Resistance Ron[m](final min = zeros(m), start = fill(1.E-5, m)) "Closed switch resistance";
          parameter Modelica.SIunits.Conductance Goff[m](final min = zeros(m), start = fill(1.E-5, m)) "Opened switch conductance";
          extends Modelica.Electrical.MultiPhase.Interfaces.ConditionalHeatPort(final mh = m, final T = fill(293.15, m));
          Modelica.Blocks.Interfaces.BooleanInput control[m] "true => p1--n2, p2--n1 connected, otherwise p1--n1, p2--n2 connected" annotation(
            Placement(transformation(origin = {0, 80}, extent = {{-20, -20}, {20, 20}}, rotation = 270)));
          Interfaces.PositivePlug plug_p1(final m = m) annotation(
            Placement(transformation(extent = {{-110, 40}, {-90, 60}}, rotation = 0)));
          Interfaces.PositivePlug plug_p2(final m = m) annotation(
            Placement(transformation(extent = {{-110, -10}, {-90, 10}}, rotation = 0)));
          Interfaces.NegativePlug plug_n2(final m = m) annotation(
            Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
          Interfaces.NegativePlug plug_n1(final m = m) annotation(
            Placement(transformation(extent = {{90, 40}, {110, 60}}, rotation = 0)));
          Modelica.Electrical.Analog.Ideal.IdealIntermediateSwitch idealIntermediateSwitch[m](final Ron = Ron, final Goff = Goff, each final useHeatPort = useHeatPort) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0)));
        equation
          connect(plug_p2.pin, idealIntermediateSwitch.p2) annotation(
            Line(points = {{-100, 0}, {-10, 0}}, color = {0, 0, 255}));
          connect(idealIntermediateSwitch.n2, plug_n2.pin) annotation(
            Line(points = {{10, 0}, {100, 0}}, color = {0, 0, 255}));
          connect(idealIntermediateSwitch.p1, plug_p1.pin) annotation(
            Line(points = {{-10, 5}, {-10, 50}, {-100, 50}}, color = {0, 0, 255}));
          connect(idealIntermediateSwitch.n1, plug_n1.pin) annotation(
            Line(points = {{10, 5}, {10, 50}, {100, 50}}, color = {0, 0, 255}));
          connect(control, idealIntermediateSwitch.control) annotation(
            Line(points = {{0, 80}, {0, 8}}, color = {255, 0, 255}));
          connect(idealIntermediateSwitch.heatPort, heatPort) annotation(
            Line(points = {{0, -10}, {0, -100}}, color = {191, 0, 0}, smooth = Smooth.None));
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-150, -40}, {150, -100}}, textString = "%name", lineColor = {0, 0, 255}), Text(extent = {{-100, 100}, {-20, 60}}, lineColor = {0, 0, 0}, textString = "m="), Text(extent = {{20, 100}, {100, 60}}, lineColor = {0, 0, 0}, textString = "%m"), Ellipse(extent = {{-4, 30}, {4, 22}}, lineColor = {0, 0, 255}), Line(points = {{-90, 0}, {-44, 0}}, color = {0, 0, 255}), Line(points = {{-90, 50}, {-44, 50}}, color = {0, 0, 255}), Line(points = {{-44, 0}, {40, 50}}, color = {0, 0, 255}), Line(points = {{-44, 50}, {40, 0}}, color = {0, 0, 255}), Line(points = {{40, 50}, {90, 50}}, color = {0, 0, 255}), Line(points = {{0, 90}, {0, 25}}, color = {0, 0, 255}), Line(points = {{40, 0}, {90, 0}}, color = {0, 0, 255})}),
            Documentation(info = "<HTML>
<p>
Contains m ideal intermediate switches (Modelica.Electrical.Analog.Ideal.IdealIntermediateSwitch).
</p>
</HTML>"));
        end IdealIntermediateSwitch;

        model IdealTransformer "Multiphase ideal transformer"
          extends Interfaces.FourPlug;
          parameter Real n[m](start = fill(1, m)) "Turns ratio";
          parameter Boolean considerMagnetization = false "Choice of considering magnetization";
          parameter SIunits.Inductance Lm1[m](start = fill(1, m)) "Magnetization inductances w.r.t. primary side";
          Modelica.Electrical.Analog.Ideal.IdealTransformer idealTransformer[m](final n = n, each final considerMagnetization = considerMagnetization, final Lm1 = Lm1) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0)));
        equation
          connect(plug_p1.pin, idealTransformer.p1) annotation(
            Line(points = {{-100, 100}, {-10, 100}, {-10, 5}}, color = {0, 0, 255}));
          connect(plug_p2.pin, idealTransformer.p2) annotation(
            Line(points = {{100, 100}, {10, 100}, {10, 5}}, color = {0, 0, 255}));
          connect(plug_n1.pin, idealTransformer.n1) annotation(
            Line(points = {{-100, -100}, {-10, -100}, {-10, -5}}, color = {0, 0, 255}));
          connect(plug_n2.pin, idealTransformer.n2) annotation(
            Line(points = {{100, -100}, {10, -100}, {10, -5}}, color = {0, 0, 255}));
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-80, 100}, {80, 60}}, lineColor = {0, 0, 0}, textString = "m=%m"), Text(extent = {{-100, -100}, {100, -40}}, lineColor = {0, 0, 255}, textString = "%name"), Ellipse(extent = {{-80, 50}, {20, -50}}, lineColor = {0, 0, 255}), Ellipse(extent = {{-20, 50}, {80, -50}}, lineColor = {0, 0, 255}), Text(extent = {{-20, 20}, {20, -20}}, lineColor = {0, 0, 255}, textString = "="), Line(points = {{-96, 50}, {-30, 50}}, color = {0, 0, 255}, smooth = Smooth.None), Line(points = {{30, 50}, {96, 50}}, color = {0, 0, 255}, smooth = Smooth.None), Line(points = {{30, -50}, {96, -50}}, color = {0, 0, 255}, smooth = Smooth.None), Line(points = {{-96, -50}, {-30, -50}}, color = {0, 0, 255}, smooth = Smooth.None)}),
            Documentation(info = "<HTML>
<p>
Contains m ideal transformers (Modelica.Electrical.Analog.Ideal.IdealTransformer).
</p>
<p>
<b>Note:</b> Due to the above equations, also DC signals will be transformed!
</p>
</HTML>"));
        end IdealTransformer;

        model Idle "Multiphase idle branch"
          extends Interfaces.TwoPlug;
          Modelica.Electrical.Analog.Ideal.Idle idle[m] annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0)));
        equation
          connect(plug_p.pin, idle.p) annotation(
            Line(points = {{-100, 0}, {-10, 0}}, color = {0, 0, 255}));
          connect(idle.n, plug_n.pin) annotation(
            Line(points = {{10, 0}, {100, 0}}, color = {0, 0, 255}));
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-150, 50}, {150, 110}}, textString = "%name", lineColor = {0, 0, 255}), Text(extent = {{-100, -100}, {100, -60}}, lineColor = {0, 0, 0}, textString = "m=%m"), Rectangle(extent = {{-60, 60}, {60, -60}}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Line(points = {{-90, 0}, {-41, 0}}, color = {0, 0, 255}), Line(points = {{91, 0}, {40, 0}}, color = {0, 0, 255})}),
            Documentation(info = "<HTML>
<p>
Contains m idles (Modelica.Electrical.Analog.Ideal.Idle)
</p>
</HTML>"));
        end Idle;

        model Short "Multiphase short cut branch"
          extends Interfaces.TwoPlug;
          Modelica.Electrical.Analog.Ideal.Short short[m] annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0)));
        equation
          connect(plug_p.pin, short.p) annotation(
            Line(points = {{-100, 0}, {-10, 0}}, color = {0, 0, 255}));
          connect(short.n, plug_n.pin) annotation(
            Line(points = {{10, 0}, {100, 0}}, color = {0, 0, 255}));
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-150, 50}, {150, 110}}, textString = "%name", lineColor = {0, 0, 255}), Text(extent = {{-100, -100}, {100, -60}}, lineColor = {0, 0, 0}, textString = "m=%m"), Rectangle(extent = {{-60, 60}, {60, -60}}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Line(points = {{-90, 0}, {90, 0}}, color = {0, 0, 255})}),
            Documentation(info = "<HTML>
<p>
Contains m short cuts (Modelica.Electrical.Analog.Ideal.Short)
</p>
</HTML>"));
        end Short;

        model IdealOpeningSwitch "Multiphase ideal opener"
          extends Interfaces.TwoPlug;
          parameter Modelica.SIunits.Resistance Ron[m](final min = zeros(m), start = fill(1.E-5, m)) "Closed switch resistance";
          parameter Modelica.SIunits.Conductance Goff[m](final min = zeros(m), start = fill(1.E-5, m)) "Opened switch conductance";
          extends Modelica.Electrical.MultiPhase.Interfaces.ConditionalHeatPort(final mh = m, final T = fill(293.15, m));
          Modelica.Blocks.Interfaces.BooleanInput control[m] "true => switch open, false => p--n connected" annotation(
            Placement(transformation(origin = {0, 70}, extent = {{-20, -20}, {20, 20}}, rotation = 270)));
          Modelica.Electrical.Analog.Ideal.IdealOpeningSwitch idealOpeningSwitch[m](final Ron = Ron, final Goff = Goff, each final useHeatPort = useHeatPort) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0)));
        equation
          connect(plug_p.pin, idealOpeningSwitch.p) annotation(
            Line(points = {{-100, 0}, {-10, 0}}, color = {0, 0, 255}));
          connect(idealOpeningSwitch.n, plug_n.pin) annotation(
            Line(points = {{10, 0}, {100, 0}}, color = {0, 0, 255}));
          connect(control, idealOpeningSwitch.control) annotation(
            Line(points = {{0, 70}, {0, 7}}, color = {255, 0, 255}));
          connect(idealOpeningSwitch.heatPort, heatPort) annotation(
            Line(points = {{0, -10}, {0, -100}}, color = {191, 0, 0}, smooth = Smooth.None));
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-150, -40}, {150, -100}}, textString = "%name", lineColor = {0, 0, 255}), Line(points = {{-90, 0}, {-44, 0}}, color = {0, 0, 255}), Ellipse(extent = {{-44, 4}, {-36, -4}}, lineColor = {0, 0, 255}), Line(points = {{-37, 2}, {40, 50}}, color = {0, 0, 255}), Line(points = {{0, 88}, {0, 26}}, color = {0, 0, 255}), Line(points = {{40, 0}, {90, 0}}, color = {0, 0, 255}), Text(extent = {{-100, 100}, {-20, 60}}, lineColor = {0, 0, 0}, textString = "m="), Text(extent = {{20, 100}, {100, 60}}, lineColor = {0, 0, 0}, textString = "%m"), Line(points = {{40, 20}, {40, 0}}, color = {0, 0, 255})}),
            Documentation(info = "<HTML>
<p>
Contains m ideal opening switches (Modelica.Electrical.Analog.Ideal.IdealOpeningSwitch).
</p>
</HTML>"));
        end IdealOpeningSwitch;

        model IdealClosingSwitch "Multiphase ideal closer"
          extends Interfaces.TwoPlug;
          parameter Modelica.SIunits.Resistance Ron[m](final min = zeros(m), start = fill(1.E-5, m)) "Closed switch resistance";
          parameter Modelica.SIunits.Conductance Goff[m](final min = zeros(m), start = fill(1.E-5, m)) "Opened switch conductance";
          extends Modelica.Electrical.MultiPhase.Interfaces.ConditionalHeatPort(final mh = m, final T = fill(293.15, m));
          Modelica.Blocks.Interfaces.BooleanInput control[m] "true => p--n connected, false => switch open" annotation(
            Placement(transformation(origin = {0, 70}, extent = {{-20, -20}, {20, 20}}, rotation = 270)));
          Modelica.Electrical.Analog.Ideal.IdealClosingSwitch idealClosingSwitch[m](final Ron = Ron, final Goff = Goff, each final useHeatPort = useHeatPort) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0)));
        equation
          connect(plug_p.pin, idealClosingSwitch.p) annotation(
            Line(points = {{-100, 0}, {-10, 0}}, color = {0, 0, 255}));
          connect(idealClosingSwitch.n, plug_n.pin) annotation(
            Line(points = {{10, 0}, {100, 0}}, color = {0, 0, 255}));
          connect(control, idealClosingSwitch.control) annotation(
            Line(points = {{0, 70}, {0, 7}}, color = {255, 0, 255}));
          connect(idealClosingSwitch.heatPort, heatPort) annotation(
            Line(points = {{0, -10}, {0, -100}}, color = {191, 0, 0}, smooth = Smooth.None));
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-150, -40}, {150, -100}}, textString = "%name", lineColor = {0, 0, 255}), Line(points = {{-90, 0}, {-44, 0}}, color = {0, 0, 255}), Ellipse(extent = {{-44, 4}, {-36, -4}}, lineColor = {0, 0, 255}), Line(points = {{-37, 2}, {40, 50}}, color = {0, 0, 255}), Line(points = {{0, 88}, {0, 26}}, color = {0, 0, 255}), Line(points = {{40, 0}, {90, 0}}, color = {0, 0, 255}), Text(extent = {{-100, 100}, {-20, 60}}, lineColor = {0, 0, 0}, textString = "m="), Text(extent = {{20, 100}, {100, 60}}, lineColor = {0, 0, 0}, textString = "%m")}),
            Documentation(info = "<HTML>
<p>
Contains m ideal closing switches (Modelica.Electrical.Analog.Ideal.IdealClosingSwitch).
</p><
</HTML>"));
        end IdealClosingSwitch;

        model OpenerWithArc "Multiphase opener with arc"
          extends Interfaces.TwoPlug;
          parameter Modelica.SIunits.Resistance Ron[m](final min = zeros(m), start = fill(1.E-5, m)) "Closed switch resistance";
          parameter Modelica.SIunits.Conductance Goff[m](final min = zeros(m), start = fill(1.E-5, m)) "Opened switch conductance";
          parameter Modelica.SIunits.Voltage V0[m](start = fill(30, m)) "Initial arc voltage";
          parameter Modelica.SIunits.VoltageSlope dVdt[m](start = fill(10E3, m)) "Arc voltage slope";
          parameter Modelica.SIunits.Voltage Vmax[m](start = fill(60, m)) "Max. arc voltage";
          extends Modelica.Electrical.MultiPhase.Interfaces.ConditionalHeatPort(final mh = m, final T = fill(293.15, m));
          Modelica.Blocks.Interfaces.BooleanInput control[m] "true => switch open, false => p--n connected" annotation(
            Placement(transformation(origin = {0, 70}, extent = {{-20, -20}, {20, 20}}, rotation = 270)));
          Modelica.Electrical.Analog.Ideal.OpenerWithArc openerWithArc[m](final Ron = Ron, final Goff = Goff, final V0 = V0, final dVdt = dVdt, final Vmax = Vmax, each final useHeatPort = useHeatPort) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0)));
        equation
          connect(plug_p.pin, openerWithArc.p) annotation(
            Line(points = {{-100, 0}, {-10, 0}}, color = {0, 0, 255}));
          connect(openerWithArc.n, plug_n.pin) annotation(
            Line(points = {{10, 0}, {100, 0}}, color = {0, 0, 255}));
          connect(control, openerWithArc.control) annotation(
            Line(points = {{0, 70}, {0, 10}}, color = {255, 0, 255}));
          connect(openerWithArc.heatPort, heatPort) annotation(
            Line(points = {{0, -10}, {0, -100}}, color = {191, 0, 0}, smooth = Smooth.None));
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-150, -40}, {150, -100}}, textString = "%name", lineColor = {0, 0, 255}), Line(points = {{-90, 0}, {-44, 0}}, color = {0, 0, 255}), Ellipse(extent = {{-44, 4}, {-36, -4}}, lineColor = {0, 0, 255}), Line(points = {{-37, 2}, {40, 50}}, color = {0, 0, 255}), Line(points = {{0, 88}, {0, 26}}, color = {0, 0, 255}), Line(points = {{40, 0}, {90, 0}}, color = {0, 0, 255}), Text(extent = {{-100, 100}, {-20, 60}}, lineColor = {0, 0, 0}, textString = "m="), Text(extent = {{20, 100}, {100, 60}}, lineColor = {0, 0, 0}, textString = "%m"), Line(points = {{40, 20}, {40, 0}}, color = {0, 0, 255}), Line(points = {{40, 50}, {32, 34}, {48, 30}, {40, 20}}, color = {255, 0, 0})}),
            Documentation(info = "<html>
<p>Contains m opening switches with arc (Modelica.Electrical.Analog.Ideal.OpenerWithArc).</p>
</html>"));
        end OpenerWithArc;

        model CloserWithArc "Multiphase closer with arc"
          extends Interfaces.TwoPlug;
          parameter Modelica.SIunits.Resistance Ron[m](final min = zeros(m), start = fill(1.E-5, m)) "Closed switch resistance";
          parameter Modelica.SIunits.Conductance Goff[m](final min = zeros(m), start = fill(1.E-5, m)) "Opened switch conductance";
          parameter Modelica.SIunits.Voltage V0[m](start = fill(30, m)) "Initial arc voltage";
          parameter Modelica.SIunits.VoltageSlope dVdt[m](start = fill(10E3, m)) "Arc voltage slope";
          parameter Modelica.SIunits.Voltage Vmax[m](start = fill(60, m)) "Max. arc voltage";
          extends Modelica.Electrical.MultiPhase.Interfaces.ConditionalHeatPort(final mh = m, final T = fill(293.15, m));
          Modelica.Blocks.Interfaces.BooleanInput control[m] "true => switch open, false => p--n connected" annotation(
            Placement(transformation(origin = {0, 70}, extent = {{-20, -20}, {20, 20}}, rotation = 270)));
          Modelica.Electrical.Analog.Ideal.CloserWithArc closerWithArc[m](final Ron = Ron, final Goff = Goff, final V0 = V0, final dVdt = dVdt, final Vmax = Vmax, each final useHeatPort = useHeatPort) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0)));
        equation
          connect(plug_p.pin, closerWithArc.p) annotation(
            Line(points = {{-100, 0}, {-10, 0}}, color = {0, 0, 255}));
          connect(closerWithArc.n, plug_n.pin) annotation(
            Line(points = {{10, 0}, {100, 0}}, color = {0, 0, 255}));
          connect(control, closerWithArc.control) annotation(
            Line(points = {{0, 70}, {0, 10}}, color = {255, 0, 255}));
          connect(closerWithArc.heatPort, heatPort) annotation(
            Line(points = {{0, -10}, {0, -100}}, color = {191, 0, 0}, smooth = Smooth.None));
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-150, -40}, {150, -100}}, textString = "%name", lineColor = {0, 0, 255}), Line(points = {{-90, 0}, {-44, 0}}, color = {0, 0, 255}), Ellipse(extent = {{-44, 4}, {-36, -4}}, lineColor = {0, 0, 255}), Line(points = {{-37, 2}, {40, 50}}, color = {0, 0, 255}), Line(points = {{0, 88}, {0, 26}}, color = {0, 0, 255}), Line(points = {{40, 0}, {90, 0}}, color = {0, 0, 255}), Text(extent = {{-100, 100}, {-20, 60}}, lineColor = {0, 0, 0}, textString = "m="), Text(extent = {{20, 100}, {100, 60}}, lineColor = {0, 0, 0}, textString = "%m"), Line(points = {{40, 50}, {32, 24}, {48, 28}, {40, 0}}, color = {255, 0, 0})}),
            Documentation(info = "<html>
<p>Contains m closing switches with arc (Modelica.Electrical.Analog.Ideal.CloserWithArc).</p>
</html>"));
        end CloserWithArc;
        annotation(
          Documentation(info = "<HTML>
<p>
This package contains analog electrical multiphase components with idealized behaviour,
like thyristor, diode, switch, transformer.
</p>

</HTML>", revisions = "<html>
<dl>
  <dt><b>Main Authors:</b></dt>
  <dd>
  <a href=\"http://www.haumer.at/\">Anton Haumer</a><br>
  Technical Consulting &amp; Electrical Engineering<br>
  A-3423 St.Andrae-Woerdern<br>Austria<br>
  email: <a href=\"mailto:a.haumer@haumer.at\">a.haumer@haumer.at</a>
  </dd>
  <dt><b>Release Notes:</b></dt>
  <dd>
  <ul>
  <li> v1.0 2004/10/01 Anton Haumer</li>
  </ul>
  </dd>
  <dt><b>Copyright:</b></dt>
  <dd>Copyright &copy; 1998-2015, Modelica Association and Anton Haumer.<br>
  <i>The Modelica package is <b>free</b> software; it can be redistributed and/or modified
  under the terms of the <b>Modelica license</b>, see the license conditions
  and the accompanying <b>disclaimer</b> in the documentation of package
  Modelica in file \"Modelica/package.mo\".</i></dd>
</dl>
</html>"),
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(origin = {10, 40}, points = {{-100, -40}, {80, -40}}, color = {0, 0, 255}), Polygon(origin = {10, 40}, fillColor = {255, 255, 255}, points = {{20, -40}, {-40, 0}, {-40, -80}, {20, -40}}), Line(origin = {-10, 0}, points = {{40, 40}, {40, -40}}, color = {0, 0, 255})}));
      end Ideal;

      package Blocks "Blocks for multi phase systems"
        extends Modelica.Icons.Package;

        block QuasiRMS
          import Modelica;
          extends Modelica.Blocks.Interfaces.SO;
          parameter Integer m(min = 2) = 3 "Number of phases";
          Modelica.Blocks.Interfaces.RealInput u[m] annotation(
            Placement(transformation(extent = {{-140, -20}, {-100, 20}})));
        equation
          y = Modelica.Electrical.MultiPhase.Functions.quasiRMS(u);
          annotation(
            Documentation(info = "<HTML>
<p>
This block determines the continuous quasi <a href=\"Modelica://Modelica.Blocks.Math.RootMeanSquare\">RMS</a> value of a multi phase system, representiong an equivalent RMS vector or phasor. If the waveform of the input deviates from a sine curve, the output of the sensor will not be exactly the average RMS value.
</p>
<pre>
 y = sqrt(sum(u[k]^2 for k in 1:m)/m)
</pre>
</HTML>"));
        end QuasiRMS;
      end Blocks;

      package Functions "Functions for multi phase systems"
        extends Modelica.Icons.Package;

        function quasiRMS "Calculate continuous quasi RMS value of input"
          extends Modelica.Icons.Function;
          input Real x[:];
          output Real y;
        algorithm
          y := sqrt(sum(x .^ 2 / size(x, 1)));
          annotation(
            Inline = true,
            Documentation(info = "<HTML>
<p>
This function determines the continuous quasi <a href=\"Modelica://Modelica.Blocks.Math.RootMeanSquare\">RMS</a> value of a multi phase system, representiong an equivalent RMS vector or phasor. If the waveform of the input deviates from a sine curve, the output of the sensor will not be exactly the average RMS value.
</p>
<pre>
 y = sqrt(sum(u[k]^2 for k in 1:m)/m)
</pre>
</HTML>"));
        end quasiRMS;

        function activePower "Calculate active power of voltage and current input"
          extends Modelica.Icons.Function;
          input Modelica.SIunits.Voltage v[:] "phase voltages";
          input Modelica.SIunits.Current i[size(v, 1)] "phase currents";
          output Modelica.SIunits.Power p "Active power";
        algorithm
          p := sum(v .* i);
          annotation(
            Inline = true,
            Documentation(info = "<HTML>
<p>
Calculates instantaneous power from multiphase voltages and currents.
In quasistaionary operation, instantaneous power equals active power;
</p>
</HTML>"));
        end activePower;

        function symmetricOrientation "Orientations of the resulting fundamental wave field phasors"
          extends Modelica.Icons.Function;
          input Integer m "Number of phases";
          output Modelica.SIunits.Angle orientation[m] "Orientation of the resulting fundamental wave field phasors";
          import Modelica.Constants.pi;
        algorithm
          if mod(m, 2) == 0 then
            if m == 2 then
              orientation[1] := 0;
              orientation[2] := +pi / 2;
            else
              orientation[1:integer(m / 2)] := symmetricOrientation(integer(m / 2));
              orientation[integer(m / 2) + 1:m] := symmetricOrientation(integer(m / 2)) - fill(pi / m, integer(m / 2));
            end if;
          else
            orientation := {(k - 1) * 2 * pi / m for k in 1:m};
          end if;
// Even number of phases
// Special case two phase machine
// Odd number of phases
          annotation(
            Documentation(info = "<html>
<p>
This function determines the orientation of the symmetrical winding with <img src=\"modelica://Modelica/Resources/Images/Magnetic/FundamentalWave/m.png\"> phases. For an odd number of phases the difference of the windings angles of two adjacent phases is <img src=\"modelica://Modelica/Resources/Images/Magnetic/FundamentalWave/2pi_over_m.png\">. In case of an even number of phases the aligned orientation of winding is not modeled since they do not add any information. Instead the <img src=\"modelica://Modelica/Resources/Images/Magnetic/FundamentalWave/m.png\"> windings are divided into two different groups. The first group refers to the indices <img src=\"modelica://Modelica/Resources/Images/Magnetic/FundamentalWave/k_le_m_over_2.png\">. The second group covers the indices <img src=\"modelica://Modelica/Resources/Images/Magnetic/FundamentalWave/k_gt_m_over_2.png\">. The difference of the windings angles of two adjacent phases - of both the first and the second group, respectively - is <img src=\"modelica://Modelica/Resources/Images/Magnetic/FundamentalWave/4pi_over_m.png\">. The phase shift of the two groups is <img src=\"modelica://Modelica/Resources/Images/Magnetic/FundamentalWave/pi_over_m.png\">.
</p>
<h4>See also</h4>
<p>
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.UsersGuide.MultiPhase\">User's guide on multi phase winding</a>,
</p>
</html>"));
        end symmetricOrientation;
      end Functions;

      package Sensors "Multiphase potential, voltage and current Sensors"
        extends Modelica.Icons.SensorsPackage;

        model PotentialSensor "Multiphase potential sensor"
          extends Modelica.Icons.RotationalSensor;
          parameter Integer m(final min = 1) = 3 "Number of phases";
          Interfaces.PositivePlug plug_p(final m = m) annotation(
            Placement(transformation(extent = {{-110, -10}, {-90, 10}}, rotation = 0)));
          Modelica.Blocks.Interfaces.RealOutput phi[m] "Absolute voltage potential as output signal" annotation(
            Placement(transformation(extent = {{100, -10}, {120, 10}}, rotation = 0)));
          Modelica.Electrical.Analog.Sensors.PotentialSensor potentialSensor[m] annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0)));
        equation
          connect(potentialSensor.p, plug_p.pin) annotation(
            Line(points = {{-10, 0}, {-100, 0}}, color = {0, 0, 255}));
          connect(potentialSensor.phi, phi) annotation(
            Line(points = {{11, 0}, {110, 0}}, color = {0, 0, 127}, smooth = Smooth.None));
          annotation(
            Icon(graphics = {Line(points = {{70, 0}, {100, 0}}, color = {0, 0, 127}), Line(points = {{-70, 0}, {-90, 0}}, color = {0, 0, 0}), Text(extent = {{-100, -70}, {100, -110}}, lineColor = {0, 0, 0}, textString = "m=%m"), Text(extent = {{-150, 60}, {150, 120}}, textString = "%name", lineColor = {0, 0, 255})}),
            Documentation(info = "<HTML>
<p>
Contains m potential sensors (Modelica.Electrical.Analog.Sensors.PotentialSensor),
thus measuring the m potentials <i>phi[m]</i> of the m pins of plug_p.
</p>
</HTML>"));
        end PotentialSensor;

        model VoltageSensor "Multiphase voltage sensor"
          extends Modelica.Icons.RotationalSensor;
          parameter Integer m(final min = 1) = 3 "Number of phases";
          Interfaces.PositivePlug plug_p(final m = m) annotation(
            Placement(transformation(extent = {{-110, -10}, {-90, 10}}, rotation = 0)));
          Interfaces.NegativePlug plug_n(final m = m) annotation(
            Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
          Modelica.Blocks.Interfaces.RealOutput v[m] "Voltage between pin p and n (= p.v - n.v) as output signal" annotation(
            Placement(transformation(origin = {0, -110}, extent = {{10, -10}, {-10, 10}}, rotation = 90)));
          Modelica.Electrical.Analog.Sensors.VoltageSensor voltageSensor[m] annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0)));
        equation
          connect(voltageSensor.n, plug_n.pin) annotation(
            Line(points = {{10, 0}, {100, 0}}, color = {0, 0, 255}));
          connect(voltageSensor.p, plug_p.pin) annotation(
            Line(points = {{-10, 0}, {-100, 0}}, color = {0, 0, 255}));
          connect(voltageSensor.v, v) annotation(
            Line(points = {{0, -10}, {0, -110}}, color = {0, 0, 127}, smooth = Smooth.None));
          annotation(
            Icon(graphics = {Text(extent = {{-29, -11}, {30, -70}}, lineColor = {0, 0, 0}, textString = "V"), Text(extent = {{-100, 60}, {100, 120}}, textString = "%name", lineColor = {0, 0, 255}), Text(extent = {{-100, -60}, {-20, -100}}, lineColor = {0, 0, 0}, textString = "m="), Line(points = {{-70, 0}, {-90, 0}}, color = {0, 0, 0}), Line(points = {{70, 0}, {90, 0}}, color = {0, 0, 0}), Line(points = {{0, -100}, {0, -70}}, color = {0, 0, 127}), Text(extent = {{20, -60}, {100, -100}}, lineColor = {0, 0, 0}, textString = "%m")}),
            Documentation(info = "<HTML>
<p>
Contains m voltage sensors (Modelica.Electrical.Analog.Sensors.VoltageSensor),
thus measuring the m potential differences <i>v[m]</i> between the m pins of plug_p and plug_n.
</p>
</HTML>"));
        end VoltageSensor;

        model VoltageQuasiRMSSensor "Continuous quasi voltage RMS sensor for multi phase system"
          import Modelica;
          extends Modelica.Icons.RotationalSensor;
          extends Modelica.Electrical.MultiPhase.Interfaces.TwoPlug;
          parameter Integer m(min = 1) = 3 "Number of phases";
          Modelica.Blocks.Interfaces.RealOutput V "Continuous quasi RMS of voltage" annotation(
            Placement(transformation(origin = {0, -100}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          Modelica.Electrical.MultiPhase.Sensors.VoltageSensor voltageSensor(final m = m) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0)));
          Modelica.Electrical.MultiPhase.Blocks.QuasiRMS quasiRMS(final m = m) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 270, origin = {0, -50})));
        equation
          connect(plug_p, voltageSensor.plug_p) annotation(
            Line(points = {{-100, 5.55112e-016}, {-100, 0}, {-10, 0}}, color = {0, 0, 255}));
          connect(voltageSensor.plug_n, plug_n) annotation(
            Line(points = {{10, 0}, {100, 0}, {100, 5.55112e-016}}, color = {0, 0, 255}));
          connect(voltageSensor.v, quasiRMS.u) annotation(
            Line(points = {{0, -11}, {0, -38}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(quasiRMS.y, V) annotation(
            Line(points = {{0, -61}, {0, -100}}, color = {0, 0, 127}, smooth = Smooth.None));
          annotation(
            Icon(graphics = {Line(points = {{0, -70}, {0, -90}}, color = {0, 0, 255}), Line(points = {{-90, 0}, {-70, 0}}, color = {0, 0, 255}), Line(points = {{70, 0}, {90, 0}}, color = {0, 0, 255}), Text(extent = {{-100, 60}, {100, 120}}, textString = "%name", lineColor = {0, 0, 255}), Text(extent = {{-100, -60}, {-20, -100}}, lineColor = {0, 0, 0}, textString = "m="), Text(extent = {{20, -60}, {100, -100}}, lineColor = {0, 0, 0}, textString = "%m")}),
            Documentation(revisions = "<html>
</html>", info = "<html>
<p>
This sensor determines the continuous quasi <a href=\"Modelica://Modelica.Blocks.Math.RootMeanSquare\">RMS</a> value of a multi phase voltage system, representiong an equivalent RMS voltage <code>V</code> vector or phasor. If the voltage waveform deviates from a sine curve, the output of the sensor will not be exactly the average RMS value.
</p>
<pre>
 V = sqrt(sum(v[k]^2 for k in 1:m)/m)
</pre>
</html>"));
        end VoltageQuasiRMSSensor;

        model CurrentSensor "Multiphase current sensor"
          extends Modelica.Icons.RotationalSensor;
          parameter Integer m(final min = 1) = 3 "Number of phases";
          Interfaces.PositivePlug plug_p(final m = m) annotation(
            Placement(transformation(extent = {{-110, -10}, {-90, 10}}, rotation = 0)));
          Interfaces.NegativePlug plug_n(final m = m) annotation(
            Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
          Modelica.Blocks.Interfaces.RealOutput i[m] "current in the branch from p to n as output signal" annotation(
            Placement(transformation(origin = {0, -110}, extent = {{10, -10}, {-10, 10}}, rotation = 90)));
          Modelica.Electrical.Analog.Sensors.CurrentSensor currentSensor[m] annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0)));
        equation
          connect(plug_p.pin, currentSensor.p) annotation(
            Line(points = {{-100, 0}, {-10, 0}}, color = {0, 0, 255}));
          connect(currentSensor.n, plug_n.pin) annotation(
            Line(points = {{10, 0}, {100, 0}}, color = {0, 0, 255}));
          connect(currentSensor.i, i) annotation(
            Line(points = {{0, -10}, {0, -110}}, color = {0, 0, 127}, smooth = Smooth.None));
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-29, -11}, {30, -70}}, lineColor = {0, 0, 0}, textString = "A"), Text(extent = {{-150, 60}, {150, 120}}, textString = "%name", lineColor = {0, 0, 255}), Text(extent = {{-100, -60}, {-20, -100}}, lineColor = {0, 0, 0}, textString = "m="), Line(points = {{-70, 0}, {-90, 0}}, color = {0, 0, 0}), Line(points = {{70, 0}, {90, 0}}, color = {0, 0, 0}), Line(points = {{0, -100}, {0, -70}}, color = {0, 0, 127}), Text(extent = {{20, -60}, {100, -100}}, lineColor = {0, 0, 0}, textString = "%m")}),
            Documentation(info = "<HTML>
<p>
Contains m current sensors (Modelica.Electrical.Analog.Sensors.CurrentSensor),
thus measuring the m currents <i>i[m]</i> flowing from the m pins of plug_p to the m pins of plug_n.
</p>
</HTML>"));
        end CurrentSensor;

        model CurrentQuasiRMSSensor "Continuous quasi current RMS sensor for multi phase system"
          import Modelica;
          extends Modelica.Icons.RotationalSensor;
          extends Modelica.Electrical.MultiPhase.Interfaces.TwoPlug;
          parameter Integer m(min = 1) = 3 "Number of phases";
          Modelica.Blocks.Interfaces.RealOutput I "Continuous quasi average RMS of current" annotation(
            Placement(transformation(origin = {0, -100}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          Modelica.Electrical.MultiPhase.Sensors.CurrentSensor currentSensor(final m = m) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0)));
          Modelica.Electrical.MultiPhase.Blocks.QuasiRMS quasiRMS(final m = m) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 270, origin = {0, -50})));
        equation
          connect(plug_p, currentSensor.plug_p) annotation(
            Line(points = {{-100, 5.55112e-016}, {-100, 0}, {-10, 0}}, color = {0, 0, 255}));
          connect(currentSensor.plug_n, plug_n) annotation(
            Line(points = {{10, 0}, {100, 0}, {100, 5.55112e-016}}, color = {0, 0, 255}));
          connect(currentSensor.i, quasiRMS.u) annotation(
            Line(points = {{0, -11}, {0, -38}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(quasiRMS.y, I) annotation(
            Line(points = {{0, -61}, {0, -100}}, color = {0, 0, 127}, smooth = Smooth.None));
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-90, 0}, {-70, 0}}, color = {0, 0, 255}), Line(points = {{0, -70}, {0, -90}}, color = {0, 0, 255}), Text(extent = {{-100, 60}, {100, 120}}, textString = "%name", lineColor = {0, 0, 255}), Text(extent = {{-100, -60}, {-20, -100}}, lineColor = {0, 0, 0}, textString = "m="), Text(extent = {{20, -60}, {100, -100}}, lineColor = {0, 0, 0}, textString = "%m"), Line(points = {{70, 0}, {90, 0}}, color = {0, 0, 255})}),
            Documentation(revisions = "<html>
</html>", info = "<html>
<p>
This sensor determines the continuous quasi <a href=\"Modelica://Modelica.Blocks.Math.RootMeanSquare\">RMS</a> value of a multi phase current system, representiong an equivalent RMS current vector <code>I</code> or phasor. If the current waveform deviates from a sine curve, the output of the sensor will not be exactly the average RMS value.
</p>
<pre>
 I = sqrt(sum(i[k]^2 for k in 1:m)/m)
</pre>
</html>"));
        end CurrentQuasiRMSSensor;

        model PowerSensor "Multiphase instantaneous power sensor"
          extends Modelica.Icons.RotationalSensor;
          parameter Integer m(min = 1) = 3 "Number of phases";
          MultiPhase.Interfaces.PositivePlug pc(final m = m) "Positive plug, current path" annotation(
            Placement(transformation(extent = {{-110, 10}, {-90, -10}}, rotation = 0)));
          MultiPhase.Interfaces.NegativePlug nc(final m = m) "Negative plug, current path" annotation(
            Placement(transformation(extent = {{90, 10}, {110, -10}}, rotation = 0)));
          MultiPhase.Interfaces.PositivePlug pv(final m = m) "Positive plug, voltage path" annotation(
            Placement(transformation(extent = {{-10, 90}, {10, 110}}, rotation = 0)));
          MultiPhase.Interfaces.NegativePlug nv(final m = m) "Negative plug, voltage path" annotation(
            Placement(transformation(extent = {{-10, -90}, {10, -110}}, rotation = 0)));
          Modelica.Blocks.Interfaces.RealOutput power annotation(
            Placement(transformation(origin = {-80, -110}, extent = {{10, -10}, {-10, 10}}, rotation = 90)));
          MultiPhase.Sensors.VoltageSensor voltageSensor(final m = m) annotation(
            Placement(transformation(origin = {0, -20}, extent = {{10, 10}, {-10, -10}}, rotation = 90)));
          MultiPhase.Sensors.CurrentSensor currentSensor(final m = m) annotation(
            Placement(transformation(extent = {{-50, -10}, {-30, 10}}, rotation = 0)));
          Modelica.Blocks.Math.Product product[m] annotation(
            Placement(transformation(origin = {-30, -40}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          Modelica.Blocks.Math.Sum sum(final nin = m, final k = ones(m)) annotation(
            Placement(transformation(origin = {-30, -70}, extent = {{10, -10}, {-10, 10}}, rotation = 90)));
        equation
          connect(pc, currentSensor.plug_p) annotation(
            Line(points = {{-100, 0}, {-50, 0}}, color = {0, 0, 255}));
          connect(currentSensor.plug_n, nc) annotation(
            Line(points = {{-30, 0}, {100, 0}}, color = {0, 0, 255}));
          connect(voltageSensor.plug_p, pv) annotation(
            Line(points = {{6.12303e-016, -10}, {0, -10}, {0, 100}}, color = {0, 0, 255}));
          connect(voltageSensor.plug_n, nv) annotation(
            Line(points = {{-6.12303e-016, -30}, {0, -30}, {0, -100}}, color = {0, 0, 255}));
          connect(voltageSensor.v, product.u1) annotation(
            Line(points = {{-11, -20}, {-24, -20}, {-24, -28}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(currentSensor.i, product.u2) annotation(
            Line(points = {{-40, -11}, {-40, -20}, {-36, -20}, {-36, -28}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(product.y, sum.u) annotation(
            Line(points = {{-30, -51}, {-30, -58}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(sum.y, power) annotation(
            Line(points = {{-30, -81}, {-30, -90}, {-80, -90}, {-80, -110}}, color = {0, 0, 127}, smooth = Smooth.None));
          annotation(
            Icon(graphics = {Line(points = {{0, 100}, {0, 70}}, color = {0, 0, 255}), Line(points = {{0, -70}, {0, -100}}, color = {0, 0, 255}), Line(points = {{-80, -100}, {-80, 0}}, color = {0, 0, 127}), Text(lineColor = {0, 0, 255}, extent = {{-150, 120}, {150, 160}}, textString = "%name"), Text(extent = {{0, -120}, {140, -80}}, textString = "m=%m"), Ellipse(fillPattern = FillPattern.Solid, extent = {{-5, -5}, {5, 5}}), Text(extent = {{-29, -70}, {30, -11}}, textString = "P"), Line(points = {{-100, 0}, {100, 0}}, color = {0, 0, 255})}),
            Documentation(info = "<html><p>
This power sensor measures instantaneous electrical power of a multiphase system and has a separated voltage and current path. The plugs of the voltage path are <code>pv</code> and <code>nv</code>, the plugs of the current path are <code>pc</code> and <code>nc</code>. The internal resistance of each current path is zero, the internal resistance of each voltage path is infinite.
</p></html>"));
        end PowerSensor;
        annotation(
          Documentation(info = "<HTML>
<p>
This package contains multiphase potential, voltage, and current sensors.
</p>

</HTML>", revisions = "<html>
<dl>
  <dt><b>Main Authors:</b></dt>
  <dd>
  <a href=\"http://www.haumer.at/\">Anton Haumer</a><br>
  Technical Consulting &amp; Electrical Engineering<br>
  A-3423 St.Andrae-Woerdern<br>Austria<br>
  email: <a href=\"mailto:a.haumer@haumer.at\">a.haumer@haumer.at</a>
  </dd>
  <dt><b>Release Notes:</b></dt>
  <dd>
  <ul>
  <li>v1.0 2004/10/01 Anton Haumer</li>
  <li>v1.1 2006/01/12 Anton Haumer<br>
      added PowerSensor</li>
  </ul>
  </dd>
  <dt><b>Copyright:</b></dt>
  <dd>Copyright &copy; 1998-2015, Modelica Association and Anton Haumer.<br>
  <i>The Modelica package is <b>free</b> software; it can be redistributed and/or modified
  under the terms of the <b>Modelica license</b>, see the license conditions
  and the accompanying <b>disclaimer</b> in the documentation of package
  Modelica in file \"Modelica/package.mo\".</i></dd>
</dl>
</html>"));
      end Sensors;

      package Sources "Multiphase voltage and current sources"
        extends Modelica.Icons.SourcesPackage;

        model SignalVoltage "Multiphase signal voltage source"
          parameter Integer m(min = 1) = 3 "Number of phases";
          Modelica.SIunits.Current i[m] = plug_p.pin.i "Currents flowing into positive plugs";
          Interfaces.PositivePlug plug_p(final m = m) annotation(
            Placement(transformation(extent = {{-110, -10}, {-90, 10}}, rotation = 0)));
          Interfaces.NegativePlug plug_n(final m = m) annotation(
            Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
          Modelica.Blocks.Interfaces.RealInput v[m](each unit = "V") "Voltage between pin p and n (= p.v - n.v) as input signal" annotation(
            Placement(transformation(origin = {0, 70}, extent = {{-20, -20}, {20, 20}}, rotation = 270)));
          Modelica.Electrical.Analog.Sources.SignalVoltage signalVoltage[m] annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0)));
        equation
          connect(signalVoltage.p, plug_p.pin) annotation(
            Line(points = {{-10, 0}, {-100, 0}}, color = {0, 0, 255}));
          connect(signalVoltage.n, plug_n.pin) annotation(
            Line(points = {{10, 0}, {100, 0}}, color = {0, 0, 255}));
          connect(v, signalVoltage.v) annotation(
            Line(points = {{0, 70}, {0, 7}}, color = {0, 0, 255}));
          annotation(
            Icon(graphics = {Line(points = {{-90, 0}, {-50, 0}}, color = {0, 0, 255}), Line(points = {{50, 0}, {90, 0}}, color = {0, 0, 255}), Ellipse(extent = {{-50, 50}, {50, -50}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{-50, 0}, {50, 0}}, color = {0, 0, 255}), Text(extent = {{-150, -110}, {150, -50}}, textString = "%name", lineColor = {0, 0, 255}), Text(extent = {{20, 100}, {100, 60}}, lineColor = {0, 0, 0}, textString = "%m"), Text(extent = {{30, 60}, {110, 0}}, lineColor = {0, 0, 255}, textString = "-"), Text(extent = {{-110, 60}, {-30, 0}}, lineColor = {0, 0, 255}, textString = "+")}),
            Documentation(info = "<HTML>
<p>
Contains m signal controlled voltage sources (Modelica.Electrical.Analog.Sources.SignalVoltage)
</p>
</HTML>"));
        end SignalVoltage;

        model ConstantVoltage "Multiphase constant voltage source"
          extends Interfaces.TwoPlug;
          parameter Modelica.SIunits.Voltage V[m](start = fill(1, m)) "Values of constant voltages";
          Modelica.Electrical.Analog.Sources.ConstantVoltage constantVoltage[m](final V = V) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0)));
        equation
          connect(constantVoltage.p, plug_p.pin) annotation(
            Line(points = {{-10, 0}, {-100, 0}}, color = {0, 0, 255}));
          connect(constantVoltage.n, plug_n.pin) annotation(
            Line(points = {{10, 0}, {100, 0}}, color = {0, 0, 255}));
          annotation(
            Icon(graphics = {Line(points = {{-90, 0}, {-50, 0}}, color = {0, 0, 255}), Line(points = {{50, 0}, {90, 0}}, color = {0, 0, 255}), Ellipse(extent = {{-50, 50}, {50, -50}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{-50, 0}, {50, 0}}, color = {0, 0, 255}), Text(extent = {{-150, -110}, {150, -50}}, textString = "%name", lineColor = {0, 0, 255}), Text(extent = {{-100, 100}, {100, 60}}, lineColor = {0, 0, 0}, textString = "m=%m"), Text(extent = {{30, 60}, {110, 0}}, lineColor = {0, 0, 255}, textString = "-"), Text(extent = {{-110, 60}, {-30, 0}}, lineColor = {0, 0, 255}, textString = "+")}),
            Documentation(info = "<HTML>
<p>
Contains m constant voltage sources (Modelica.Electrical.Analog.Sources.ConstantVoltage)
</p>
</HTML>"));
        end ConstantVoltage;

        model SineVoltage "Multiphase sine voltage source"
          extends Interfaces.TwoPlug;
          parameter Modelica.SIunits.Voltage V[m](start = fill(1, m)) "Amplitudes of sine waves";
          parameter Modelica.SIunits.Angle phase[m] = -Modelica.Electrical.MultiPhase.Functions.symmetricOrientation(m) "Phases of sine waves";
          parameter Modelica.SIunits.Frequency freqHz[m](start = fill(1, m)) "Frequencies of sine waves";
          parameter Modelica.SIunits.Voltage offset[m] = zeros(m) "Voltage offsets";
          parameter Modelica.SIunits.Time startTime[m] = zeros(m) "Time offsets";
          Modelica.Electrical.Analog.Sources.SineVoltage sineVoltage[m](final V = V, final phase = phase, final freqHz = freqHz, final offset = offset, final startTime = startTime) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0)));
        equation
          connect(sineVoltage.p, plug_p.pin) annotation(
            Line(points = {{-10, 0}, {-100, 0}}, color = {0, 0, 255}));
          connect(sineVoltage.n, plug_n.pin) annotation(
            Line(points = {{10, 0}, {100, 0}}, color = {0, 0, 255}));
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-90, 0}, {-50, 0}}, color = {0, 0, 255}), Line(points = {{50, 0}, {90, 0}}, color = {0, 0, 255}), Ellipse(extent = {{-50, 50}, {50, -50}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{-50, 0}, {50, 0}}, color = {0, 0, 255}), Line(points = {{-70, 0}, {-60.2, 29.9}, {-53.8, 46.5}, {-48.2, 58.1}, {-43.3, 65.2}, {-38.3, 69.2}, {-33.4, 69.8}, {-28.5, 67}, {-23.6, 61}, {-18.6, 52}, {-13, 38.6}, {-5.98, 18.6}, {8.79, -26.9}, {15.1, -44}, {20.8, -56.2}, {25.7, -64}, {30.6, -68.6}, {35.5, -70}, {40.5, -67.9}, {45.4, -62.5}, {50.3, -54.1}, {55.9, -41.3}, {63, -21.7}, {70, 0}}, color = {192, 192, 192}, smooth = Smooth.Bezier), Text(extent = {{-150, -110}, {150, -50}}, textString = "%name", lineColor = {0, 0, 255}), Text(extent = {{-100, 100}, {100, 60}}, lineColor = {0, 0, 0}, textString = "m=%m"), Text(extent = {{30, 60}, {110, 0}}, lineColor = {0, 0, 255}, textString = "-"), Text(extent = {{-110, 60}, {-30, 0}}, lineColor = {0, 0, 255}, textString = "+")}),
            Documentation(info = "<HTML>
<p>
Contains m sine voltage sources (Modelica.Electrical.Analog.Sources.SineVoltage)
with a default phase shift determined by
<a href=\"modelica://Modelica.Electrical.MultiPhase.Functions.symmetricOrientation\">symmetricOrientation</a>.
</p>
</HTML>"));
        end SineVoltage;

        model CosineVoltage "Multiphase cosine voltage source"
          extends Interfaces.TwoPlug;
          parameter Modelica.SIunits.Voltage V[m](start = fill(1, m)) "Amplitudes of cosine waves";
          parameter Modelica.SIunits.Angle phase[m] = -Modelica.Electrical.MultiPhase.Functions.symmetricOrientation(m) "Phases of cosine waves";
          parameter Modelica.SIunits.Frequency freqHz[m](start = fill(1, m)) "Frequencies of cosine waves";
          parameter Modelica.SIunits.Voltage offset[m] = zeros(m) "Voltage offsets";
          parameter Modelica.SIunits.Time startTime[m] = zeros(m) "Time offsets";
          Analog.Sources.CosineVoltage cosineVoltage[m](final V = V, final phase = phase, final freqHz = freqHz, final offset = offset, final startTime = startTime) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0)));
        equation
          connect(cosineVoltage.p, plug_p.pin) annotation(
            Line(points = {{-10, 0}, {-100, 0}}, color = {0, 0, 255}));
          connect(cosineVoltage.n, plug_n.pin) annotation(
            Line(points = {{10, 0}, {100, 0}}, color = {0, 0, 255}));
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-90, 0}, {-50, 0}}, color = {0, 0, 255}), Line(points = {{50, 0}, {90, 0}}, color = {0, 0, 255}), Ellipse(extent = {{-50, 50}, {50, -50}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{-50, 0}, {50, 0}}, color = {0, 0, 255}), Text(extent = {{-150, -110}, {150, -50}}, textString = "%name", lineColor = {0, 0, 255}), Text(extent = {{-100, 100}, {100, 60}}, lineColor = {0, 0, 0}, textString = "m=%m"), Text(extent = {{30, 60}, {110, 0}}, lineColor = {0, 0, 255}, textString = "-"), Text(extent = {{-110, 60}, {-30, 0}}, lineColor = {0, 0, 255}, textString = "+"), Line(points = {{-71, 70}, {-68.4, 69.8}, {-63.5, 67}, {-58.6, 61}, {-53.6, 52}, {-48, 38.6}, {-40.98, 18.6}, {-26.21, -26.9}, {-19.9, -44}, {-14.2, -56.2}, {-9.3, -64}, {-4.4, -68.6}, {0.5, -70}, {5.5, -67.9}, {10.4, -62.5}, {15.3, -54.1}, {20.9, -41.3}, {28, -21.7}, {35, 0}}, color = {192, 192, 192}, smooth = Smooth.Bezier), Line(points = {{35, 0}, {44.8, 29.9}, {51.2, 46.5}, {56.8, 58.1}, {61.7, 65.2}, {66.7, 69.2}, {71.6, 69.8}}, color = {192, 192, 192})}),
            Documentation(info = "<HTML>
<p>
Contains m cosine voltage sources (Modelica.Electrical.Analog.Sources.CosineVoltage)
with a default phase shift determined by
<a href=\"modelica://Modelica.Electrical.MultiPhase.Functions.symmetricOrientation\">symmetricOrientation</a>.
</p>
</HTML>"));
        end CosineVoltage;

        model SignalCurrent "Multiphase sine current source"
          parameter Integer m(min = 1) = 3 "Number of phases";
          Modelica.SIunits.Voltage v[m] = plug_p.pin.v - plug_n.pin.v "Voltage drops between the two plugs";
          Interfaces.PositivePlug plug_p(final m = m) annotation(
            Placement(transformation(extent = {{-110, -10}, {-90, 10}}, rotation = 0)));
          Interfaces.NegativePlug plug_n(final m = m) annotation(
            Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
          Modelica.Blocks.Interfaces.RealInput i[m](each unit = "A") "Current flowing from pin p to pin n as input signal" annotation(
            Placement(transformation(origin = {0, 70}, extent = {{-20, -20}, {20, 20}}, rotation = 270)));
          Modelica.Electrical.Analog.Sources.SignalCurrent signalCurrent[m] annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0)));
        equation
          connect(signalCurrent.p, plug_p.pin) annotation(
            Line(points = {{-10, 0}, {-100, 0}}, color = {0, 0, 255}));
          connect(signalCurrent.n, plug_n.pin) annotation(
            Line(points = {{10, 0}, {100, 0}}, color = {0, 0, 255}));
          connect(i, signalCurrent.i) annotation(
            Line(points = {{0, 70}, {0, 7}}, color = {0, 0, 255}));
          annotation(
            Icon(graphics = {Line(points = {{-90, 0}, {-50, 0}}, color = {0, 0, 255}), Line(points = {{50, 0}, {90, 0}}, color = {0, 0, 255}), Ellipse(extent = {{-50, 50}, {50, -50}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{0, 50}, {0, -50}}, color = {0, 0, 255}), Text(extent = {{-150, -110}, {150, -50}}, textString = "%name", lineColor = {0, 0, 255}), Text(extent = {{-100, 100}, {-20, 60}}, lineColor = {0, 0, 0}, textString = "m="), Text(extent = {{20, 100}, {100, 60}}, lineColor = {0, 0, 0}, textString = "%m"), Polygon(points = {{90, 0}, {60, 10}, {60, -10}, {90, 0}}, lineColor = {0, 0, 255}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid)}),
            Documentation(info = "<HTML>
<p>
Contains m signal controlled current sources (Modelica.Electrical.Analog.Sources.SignalCurrent)
</p>
</HTML>"));
        end SignalCurrent;

        model ConstantCurrent "Multiphase constant current source"
          extends Interfaces.TwoPlug;
          parameter Modelica.SIunits.Current I[m](start = fill(1, m)) "Values of constant currents";
          Modelica.Electrical.Analog.Sources.ConstantCurrent constantCurrent[m](final I = I) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0)));
        equation
          connect(constantCurrent.p, plug_p.pin) annotation(
            Line(points = {{-10, 0}, {-100, 0}}, color = {0, 0, 255}));
          connect(constantCurrent.n, plug_n.pin) annotation(
            Line(points = {{10, 0}, {100, 0}}, color = {0, 0, 255}));
          annotation(
            Icon(graphics = {Line(points = {{-90, 0}, {-50, 0}}, color = {0, 0, 255}), Line(points = {{50, 0}, {90, 0}}, color = {0, 0, 255}), Ellipse(extent = {{-50, 50}, {50, -50}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{0, 50}, {0, -50}}, color = {0, 0, 255}), Text(extent = {{-150, -110}, {150, -50}}, textString = "%name", lineColor = {0, 0, 255}), Text(extent = {{-100, 100}, {100, 60}}, lineColor = {0, 0, 0}, textString = "m=%m"), Polygon(points = {{90, 0}, {60, 10}, {60, -10}, {90, 0}}, lineColor = {0, 0, 255}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid)}),
            Documentation(info = "<HTML>
<p>
Contains m constant current sources (Modelica.Electrical.Analog.Sources.ConstantCurrent)
</p>
</HTML>"));
        end ConstantCurrent;

        model SineCurrent "Multiphase sine current source"
          extends Interfaces.TwoPlug;
          parameter Modelica.SIunits.Current I[m](start = fill(1, m)) "Amplitudes of sine waves";
          parameter Modelica.SIunits.Angle phase[m] = -Modelica.Electrical.MultiPhase.Functions.symmetricOrientation(m) "Phases of sine waves";
          parameter Modelica.SIunits.Frequency freqHz[m](start = fill(1, m)) "Frequencies of sine waves";
          parameter Modelica.SIunits.Current offset[m] = zeros(m) "Current offsets";
          parameter Modelica.SIunits.Time startTime[m] = zeros(m) "Time offsets";
          Modelica.Electrical.Analog.Sources.SineCurrent sineCurrent[m](final I = I, final phase = phase, final freqHz = freqHz, final offset = offset, final startTime = startTime) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0)));
        equation
          connect(sineCurrent.p, plug_p.pin) annotation(
            Line(points = {{-10, 0}, {-100, 0}}, color = {0, 0, 255}));
          connect(sineCurrent.n, plug_n.pin) annotation(
            Line(points = {{10, 0}, {100, 0}}, color = {0, 0, 255}));
          annotation(
            Icon(graphics = {Line(points = {{-90, 0}, {-50, 0}}, color = {0, 0, 255}), Line(points = {{50, 0}, {90, 0}}, color = {0, 0, 255}), Ellipse(extent = {{-50, 50}, {50, -50}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{0, 50}, {0, -50}}, color = {0, 0, 255}), Line(points = {{-70, 0}, {-60.2, 29.9}, {-53.8, 46.5}, {-48.2, 58.1}, {-43.3, 65.2}, {-38.3, 69.2}, {-33.4, 69.8}, {-28.5, 67}, {-23.6, 61}, {-18.6, 52}, {-13, 38.6}, {-5.98, 18.6}, {8.79, -26.9}, {15.1, -44}, {20.8, -56.2}, {25.7, -64}, {30.6, -68.6}, {35.5, -70}, {40.5, -67.9}, {45.4, -62.5}, {50.3, -54.1}, {55.9, -41.3}, {63, -21.7}, {70, 0}}, color = {192, 192, 192}, smooth = Smooth.Bezier), Text(extent = {{-150, -110}, {150, -50}}, textString = "%name", lineColor = {0, 0, 255}), Text(extent = {{-100, 100}, {100, 60}}, lineColor = {0, 0, 0}, textString = "m=%m"), Polygon(points = {{90, 0}, {60, 10}, {60, -10}, {90, 0}}, lineColor = {0, 0, 255}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid)}),
            Documentation(info = "<HTML>
<p>
Contains m sine current sources (Modelica.Electrical.Analog.Sources.SineCurrent)
with a default phase shift determined by
<a href=\"modelica://Modelica.Electrical.MultiPhase.Functions.symmetricOrientation\">symmetricOrientation</a>.
</p>
</HTML>"));
        end SineCurrent;

        model CosineCurrent "Multiphase cosine current source"
          extends Interfaces.TwoPlug;
          parameter Modelica.SIunits.Current I[m](start = fill(1, m)) "Amplitudes of cosine waves";
          parameter Modelica.SIunits.Angle phase[m] = -Modelica.Electrical.MultiPhase.Functions.symmetricOrientation(m) "Phases of cosine waves";
          parameter Modelica.SIunits.Frequency freqHz[m](start = fill(1, m)) "Frequencies of cosine waves";
          parameter Modelica.SIunits.Current offset[m] = zeros(m) "Current offsets";
          parameter Modelica.SIunits.Time startTime[m] = zeros(m) "Time offsets";
          Analog.Sources.CosineCurrent cosineCurrent[m](final I = I, final phase = phase, final freqHz = freqHz, final offset = offset, final startTime = startTime) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0)));
        equation
          connect(cosineCurrent.p, plug_p.pin) annotation(
            Line(points = {{-10, 0}, {-100, 0}}, color = {0, 0, 255}));
          connect(cosineCurrent.n, plug_n.pin) annotation(
            Line(points = {{10, 0}, {100, 0}}, color = {0, 0, 255}));
          annotation(
            Icon(graphics = {Line(points = {{-90, 0}, {-50, 0}}, color = {0, 0, 255}), Line(points = {{50, 0}, {90, 0}}, color = {0, 0, 255}), Ellipse(extent = {{-50, 50}, {50, -50}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{0, 50}, {0, -50}}, color = {0, 0, 255}), Text(extent = {{-150, -110}, {150, -50}}, textString = "%name", lineColor = {0, 0, 255}), Text(extent = {{-100, 100}, {100, 60}}, lineColor = {0, 0, 0}, textString = "m=%m"), Polygon(points = {{90, 0}, {60, 10}, {60, -10}, {90, 0}}, lineColor = {0, 0, 255}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid), Line(points = {{35, 0}, {44.8, 29.9}, {51.2, 46.5}, {56.8, 58.1}, {61.7, 65.2}, {66.7, 69.2}, {71.6, 69.8}}, color = {192, 192, 192}), Line(points = {{-71, 70}, {-68.4, 69.8}, {-63.5, 67}, {-58.6, 61}, {-53.6, 52}, {-48, 38.6}, {-40.98, 18.6}, {-26.21, -26.9}, {-19.9, -44}, {-14.2, -56.2}, {-9.3, -64}, {-4.4, -68.6}, {0.5, -70}, {5.5, -67.9}, {10.4, -62.5}, {15.3, -54.1}, {20.9, -41.3}, {28, -21.7}, {35, 0}}, color = {192, 192, 192}, smooth = Smooth.Bezier)}),
            Documentation(info = "<HTML>
<p>
Contains m cosine current sources (Modelica.Electrical.Analog.Sources.CosineCurrent)
with a default phase shift determined by
<a href=\"modelica://Modelica.Electrical.MultiPhase.Functions.symmetricOrientation\">symmetricOrientation</a>.
</p>
</HTML>"));
        end CosineCurrent;
        annotation(
          Documentation(info = "<HTML>
<p>
This package contains time-dependent and controlled multiphase voltage and current sources:
</p>
<ul>
<li>SignalVoltage: fed by Modelica.Blocks.Sources arbitrary waveforms of voltages are possible</li>
<li>ConstantVoltage: constant multi phase voltages</li>
<li>SineVoltage : phase shift between consecutive voltages by default given by
<a href=\"modelica://Modelica.Electrical.MultiPhase.Functions.symmetricOrientation\">symmetricOrientation</a></li>
<li>CosineVoltage : phase shift between consecutive voltages by default given by
<a href=\"modelica://Modelica.Electrical.MultiPhase.Functions.symmetricOrientation\">symmetricOrientation</a></li>
<li>SignalCurrent: fed by Modelica.Blocks.Sources arbitrary waveforms of currents are possible</li>
<li>ConstantCurrent: constant multi phase currents</li>
<li>SineCurrent : phase shift between consecutive currents by default given by
<a href=\"modelica://Modelica.Electrical.MultiPhase.Functions.symmetricOrientation\">symmetricOrientation</a></li>
<li>CosineCurrent : phase shift between consecutive currents by default given by
<a href=\"modelica://Modelica.Electrical.MultiPhase.Functions.symmetricOrientation\">symmetricOrientation</a></li>
</ul>
</HTML>", revisions = "<html>
<dl>
  <dt><b>Main Authors:</b></dt>
  <dd>
  <a href=\"http://www.haumer.at/\">Anton Haumer</a><br>
  Technical Consulting &amp; Electrical Engineering<br>
  A-3423 St.Andrae-Woerdern<br>Austria<br>
  email: <a href=\"mailto:a.haumer@haumer.at\">a.haumer@haumer.at</a>
  </dd>
  <dt><b>Release Notes:</b></dt>
  <dd>
  <ul>
  <li> v1.0 2004/10/01 Anton Haumer</li>
  </ul>
  </dd>
  <dt><b>Copyright:</b></dt>
  <dd>Copyright &copy; 1998-2015, Modelica Association and Anton Haumer.<br>
  <i>The Modelica package is <b>free</b> software; it can be redistributed and/or modified
  under the terms of the <b>Modelica license</b>, see the license conditions
  and the accompanying <b>disclaimer</b> in the documentation of package
  Modelica in file \"Modelica/package.mo\".</i></dd>
</dl>
</html>"));
      end Sources;

      package Interfaces "Interfaces for electrical multiphase models"
        extends Modelica.Icons.InterfacesPackage;

        connector Plug "Plug with m pins for an electric component"
          parameter Integer m(final min = 1) = 3 "Number of phases";
          Modelica.Electrical.Analog.Interfaces.Pin pin[m];
          annotation(
            Documentation(info = "<HTML>
<p>
Connectors PositivePlug and NegativePlug are nearly identical.
The only difference is that the icons are different in order
to identify more easily the plugs of a component.
Usually, connector PositivePlug is used for the positive and
connector NegativePlug for the negative plug of an electrical component.<br>
Connector Plug is a composite connector containing m Pins (Modelica.Electrical.Analog.Interfaces.Pin).
</p>
</HTML>"),
            Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-100, -99}, {100, -179}}, lineColor = {0, 0, 255}, textString = "%name")}));
        end Plug;

        connector PositivePlug "Positive plug with m pins"
          extends Plug;
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-100, 100}, {100, -100}}, lineColor = {0, 0, 255}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid)}),
            Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-40, 40}, {40, -40}}, lineColor = {0, 0, 255}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid)}),
            Documentation(info = "<HTML>
<p>
Connectors PositivePlug and NegativePlug are nearly identical.
The only difference is that the icons are different in order
to identify more easily the plugs of a component.
Usually, connector PositivePlug is used for the positive and
connector NegativePlug for the negative plug of an electrical component.<br>
Connector Plug is a composite connector containing m Pins (Modelica.Electrical.Analog.Interfaces.Pin).
</p>
</HTML>"));
        end PositivePlug;

        connector NegativePlug "Negative plug with m pins"
          extends Plug;
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-100, 100}, {100, -100}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid)}),
            Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-40, 40}, {40, -40}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid)}),
            Documentation(info = "<HTML>
<p>
Connectors PositivePlug and NegativePlug are nearly identical.
The only difference is that the icons are different in order
to identify more easily the plugs of a component.
Usually, connector PositivePlug is used for the positive and
connector NegativePlug for the negative plug of an electrical component.<br>
Connector Plug is a composite connector containing m Pins (Modelica.Electrical.Analog.Interfaces.Pin).
</p>
</HTML>"));
        end NegativePlug;

        partial model ConditionalHeatPort "Partial model to include conditional HeatPorts in order to describe the power loss via a thermal network"
          parameter Integer mh(min = 1) = 3 "Number of heatPorts=number of phases";
          parameter Boolean useHeatPort = false "=true, if all HeatPorts are enabled" annotation(
            Evaluate = true,
            HideResult = true,
            choices(checkBox = true));
          parameter Modelica.SIunits.Temperature T[mh] = fill(293.15, mh) "Fixed device temperatures if useHeatPort = false" annotation(
            Dialog(enable = not useHeatPort));
          Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a heatPort[mh] if useHeatPort annotation(
            Placement(transformation(extent = {{-10, -110}, {10, -90}}), iconTransformation(extent = {{-10, -110}, {10, -90}})));
          annotation(
            Documentation(revisions = "<html>
<ul>
<li><i>August 26, 2009 </i>by Anton Haumer initially implemented</li>
</ul>
</html>", info = "<html>
<p>
This partial model provides conditional heat ports for the connection to a thermal network.
</p>
<ul>
<li> If <b>useHeatPort</b> is set to <b>false</b> (default), no heat port is available, and the thermal
     loss power flows internally to the ground. In this case, the parameter <b>T</b> specifies
     the fixed device temperatures.</li>
<li> If <b>useHeatPort</b> is set to <b>true</b>, all heat ports are available.</li>
</ul>
</html>"));
        end ConditionalHeatPort;

        partial model TwoPlug "Component with one m-phase electric port"
          parameter Integer m(min = 1) = 3 "Number of phases";
          Modelica.SIunits.Voltage v[m] "Voltage drops between the two plugs";
          Modelica.SIunits.Current i[m] "Currents flowing into positive plugs";
          PositivePlug plug_p(final m = m) annotation(
            Placement(transformation(extent = {{-110, -10}, {-90, 10}}, rotation = 0)));
          NegativePlug plug_n(final m = m) annotation(
            Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
        equation
          v = plug_p.pin.v - plug_n.pin.v;
          i = plug_p.pin.i;
          annotation(
            Documentation(info = "<HTML>
<p>
Superclass of elements which have <b>two</b> electrical plugs:
the positive plug connector <i>plug_p</i>, and the negative plug connector <i>plug_n</i>.
The currents flowing into plug_p are provided explicitly as currents i[m].
</p>
</HTML>"));
        end TwoPlug;

        partial model OnePort "Component with two electrical plugs and currents from plug_p to plug_n"
          extends TwoPlug;
        equation
          plug_p.pin.i + plug_n.pin.i = zeros(m);
          annotation(
            Documentation(info = "<HTML>
<p>
Superclass of elements which have <b>two</b> electrical plugs:
the positive plug connector <i>plug_p</i>, and the negative plug connector <i>plug_n</i>.
The currents flowing into plug_p are provided explicitly as currents i[m].
It is assumed that the currents flowing into plug_p are identical to the currents flowing out of plug_n.
</p>
</HTML>"));
        end OnePort;

        partial model FourPlug "Component with two m-phase electric ports"
          parameter Integer m(final min = 1) = 3 "Number of phases";
          Modelica.SIunits.Voltage v1[m] "Voltage drops over the left port";
          Modelica.SIunits.Voltage v2[m] "Voltage drops over the right port";
          Modelica.SIunits.Current i1[m] "Current flowing into positive plug of the left port";
          Modelica.SIunits.Current i2[m] "Current flowing into positive plug of the right port";
          PositivePlug plug_p1(final m = m) annotation(
            Placement(transformation(extent = {{-110, 90}, {-90, 110}}, rotation = 0)));
          PositivePlug plug_p2(final m = m) annotation(
            Placement(transformation(extent = {{90, 90}, {110, 110}}, rotation = 0)));
          NegativePlug plug_n1(final m = m) annotation(
            Placement(transformation(extent = {{-110, -110}, {-90, -90}}, rotation = 0)));
          NegativePlug plug_n2(final m = m) annotation(
            Placement(transformation(extent = {{90, -110}, {110, -90}}, rotation = 0)));
        equation
          v1 = plug_p1.pin.v - plug_n1.pin.v;
          v2 = plug_p2.pin.v - plug_n2.pin.v;
          i1 = plug_p1.pin.i;
          i2 = plug_p2.pin.i;
          annotation(
            Documentation(info = "<HTML>
<p>
Superclass of elements which have <b>four</b> electrical plugs.
</p>
</HTML>"));
        end FourPlug;

        partial model TwoPort "Component with two m-phase electric ports, including currents"
          extends FourPlug;
        equation
          plug_p1.pin.i + plug_n1.pin.i = zeros(m);
          plug_p2.pin.i + plug_n2.pin.i = zeros(m);
          annotation(
            Documentation(info = "<HTML>
<p>
Superclass of elements which have <b>four</b> electrical plugs.
It is assumed that the currents flowing into plug_p1 are identical to the currents flowing out of plug_n1,
and that the currents flowing into plug_p2 are identical to the currents flowing out of plug_n2.
</p>
</HTML>"));
        end TwoPort;
        annotation(
          Documentation(info = "<HTML>
<p>
This package contains connectors and interfaces (partial models) for
electrical multiphase components, based on Modelica.Electrical.Analog.
</p>

</HTML>", revisions = "<html>
<dl>
  <dt><b>Main Authors:</b></dt>
  <dd>
  <a href=\"http://www.haumer.at/\">Anton Haumer</a><br>
  Technical Consulting &amp; Electrical Engineering<br>
  A-3423 St.Andrae-Woerdern<br>Austria<br>
  email: <a href=\"mailto:a.haumer@haumer.at\">a.haumer@haumer.at</a>
  </dd>
  <dt><b>Release Notes:</b></dt>
  <dd>
  <ul>
  <li>v1.0 2004/10/01 Anton Haumer</li>
  <li>v1.2 2006/05/12 Anton Haumer<br>
      removed annotation from pin of Interfaces.Plug</li>
  </ul>
  </dd>
  <dt><b>Copyright:</b></dt>
  <dd>Copyright &copy; 1998-2015, Modelica Association and Anton Haumer.<br>
  <i>The Modelica package is <b>free</b> software; it can be redistributed and/or modified
  under the terms of the <b>Modelica license</b>, see the license conditions
  and the accompanying <b>disclaimer</b> in the documentation of package
  Modelica in file \"Modelica/package.mo\".</i></dd>
</dl>
</html>"));
      end Interfaces;
      annotation(
        Documentation(info = "<HTML>
<p>
This package contains packages for electrical multiphase components, based on Modelica.Electrical.Analog:
</p>
<ul>
<li>Basic: basic components (resistor, capacitor, inductor, ...)</li>
<li>Ideal: ideal elements (switches, diode, transformer, ...)</li>
<li>Sensors: sensors to measure potentials, voltages, and currents</li>
<li>Sources: time-dependent and controlled voltage and current sources</li>
</ul>
<p>
This package is intended to be used the same way as Modelica.Electrical.Analog
but to make design of multiphase models easier.<br>
The package is based on the plug: a composite connector containing m pins.<br>
It is possible to connect plugs to plugs or single pins of a plug to single pins.<br>
Potentials may be accessed as <code>plug.pin[].v</code>, currents may be accessed as <code>plug.pin[].i</code>.
</p>
<p>
Further development:
</p>

<ul>
<li>temperature-dependent resistor</li>
<li>lines (m-phase models)</li>
</ul>

<dl>
  <dt><b>Main Authors:</b></dt>
  <dd>
  <a href=\"http://www.haumer.at/\">Anton Haumer</a><br>
  Technical Consulting &amp; Electrical Engineering<br>
  A-3423 St.Andrae-Woerdern<br>Austria<br>
  email: <a href=\"mailto:a.haumer@haumer.at\">a.haumer@haumer.at</a>
  </dd>
  <dt><b>Release Notes:</b></dt>
  <dd>
  <ul>
  <li> v1.0 2004/10/01 Anton Haumer</li>
  </ul>
  </dd>
  <dt><b>Copyright:</b></dt>
  <dd>Copyright &copy; 1998-2015, Modelica Association and Anton Haumer.<br>
  <i>The Modelica package is <b>free</b> software; it can be redistributed and/or modified
  under the terms of the <b>Modelica license</b>, see the license conditions
  and the accompanying <b>disclaimer</b> in the documentation of package
  Modelica in file \"Modelica/package.mo\".</i></dd>
</dl>
</HTML>", revisions = "<html>
  <ul>
  <li>v1.0 2004/10/01 Anton Haumer</li>
  <li>v1.1 2006/01/12 Anton Haumer<br>
      added Sensors.PowerSensor</li>
  <li>v1.2 2006/07/05 Anton Haumer<br>
      removed annotation from pin of Interfaces.Plug<br>
      corrected usage of resistance/conductance</li>
  <li>v1.3.0 2007/01/23 Anton Haumer<br>
      improved some icons</li>
  <li>v1.3.1 2007/08/12 Anton Haumer<br>
      improved documentation</li>
  <li>v1.3.2 2007/08/24 Anton Haumer<br>
      removed redeclare type SignalType</li>
  <li>v1.4.0 2009/08/26 Anton Haumer<br>
      added conditional HeatPorts as Electrical.Analog<br>
      added switches with arc as Electrical.Analog</li>
  </ul>
</html>"),
        Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(origin = {14, 56}, lineColor = {95, 95, 95}, extent = {{-84, -126}, {56, 14}}), Ellipse(origin = {0, 42}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid, extent = {{-40, -34}, {-20, -14}}), Ellipse(origin = {20, 40}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid, extent = {{0, -34}, {20, -14}}), Ellipse(origin = {10, 34}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid, extent = {{-20, -74}, {0, -54}})}));
    end MultiPhase;

    package QuasiStationary "Library for quasi-stationary electrical singlephase and multiphase AC simulation"
      extends Modelica.Icons.Package;

      package UsersGuide "User's guide"
        extends Modelica.Icons.Information;

        package Overview "Overview"
          extends Modelica.Icons.Information;

          class Introduction "Introduction to phasors"
            extends Modelica.Icons.Information;
            annotation(
              Documentation(info = "<html>

<p>
The purely sinusoidal voltage
</p>

<p>
<IMG
 BORDER=\"0\"
 src=\"modelica://Modelica/Resources/Images/Electrical/QuasiStationary/UsersGuide/Overview/Introduction/img1.png\"
 ALT=\"v=\\sqrt{2}V_{\\mathrm{RMS}}\\cos(\\omega t+\\varphi_{v})\">
</p>

<p>
in the time domain can be represented by a complex
<a href=\"modelica://Modelica.Electrical.QuasiStationary.UsersGuide.Glossar\">rms</a> phasor
</p>

<p>
<IMG
 BORDER=\"0\"
 src=\"modelica://Modelica/Resources/Images/Electrical/QuasiStationary/UsersGuide/Overview/Introduction/img2.png\"
 ALT=\"\\underline{v}=V_{\\mathrm{RMS}}e^{j\\varphi_{v}}.\">
</p>

<p>For these quasi stationary
phasor the following relationship applies:</p>

<p>
<IMG
 BORDER=\"0\"
 src=\"modelica://Modelica/Resources/Images/Electrical/QuasiStationary/UsersGuide/Overview/Introduction/img3.png\"
 ALT=\"\\begin{displaymath}
v=\\mathrm{Re}(\\sqrt{2}\\underline{v}e^{j\\omega t})\\end{displaymath}\">
</p>

<p>
This equation is also illustrated in Fig. 1.
</p>

<table border=\"0\" cellspacing=\"0\" cellpadding=\"2\">
  <tr>
    <td>
      <img src=\"modelica://Modelica/Resources/Images/Electrical/QuasiStationary/UsersGuide/Overview/Introduction/phasor_voltage.png\"
           alt=\"phasor_voltage.png\">
    </td>
  </tr>
  <caption align=\"bottom\">Fig. 1: Relationship between voltage phasor and time domain voltage</caption>
</table>

<p>
From the above equation it is obvious that for <i>t</i> = 0
the time domain voltage is <i>v</i> = cos(<i>&phi;<sub>v</sub></i>).
The complex representation of the phasor corresponds with this instance, too, since
the phasor is leading the real axis by the angle <i>&phi;<sub>v</sub></i>.
</p>

<p>
The explanation given for sinusoidal voltages can certainly also be applied
to sinusoidal currents.</p>

<h4>See also</h4>
<a href=\"modelica://Modelica.Electrical.QuasiStationary.UsersGuide.Overview.ACCircuit\">
          AC circuit</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.UsersGuide.Overview.Power\">
          Power</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.UsersGuide.Overview.ReferenceSystem\">
          Reference system</a>

</html>"));
          end Introduction;

          class ACCircuit "AC circuit"
            extends Modelica.Icons.Information;
            annotation(
              Documentation(info = "<html>
<p>
A simple
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Examples.SeriesResonance\">
          example</a> of a series connection of a resistor, an inductor and a capacitor
as depicted in Fig. 1 should be explained in the following. For various frequencies,
the voltage drops across the resistor, the inductor and the capacitor should be determined.
</p>

<table border=\"0\" cellspacing=\"0\" cellpadding=\"2\">
  <tr>
    <td>
      <img src=\"modelica://Modelica/Resources/Images/Electrical/QuasiStationary/UsersGuide/Overview/ACCircuit/resonance_circuit.png\"
           alt=\"resonance_circuit.png\">
    </td>
  </tr>
  <caption align=\"bottom\">Fig. 1: Series AC circuit of a resistor and an inductor at variable frequency</caption>
</table>

<p>
The voltage drop across the resistor
</p>

<p>
<IMG
 BORDER=\"0\"
 src=\"modelica://Modelica/Resources/Images/Electrical/QuasiStationary/UsersGuide/Overview/ACCircuit/img1.png\"
 ALT=\"
\\underline{v}_{r}=R\\underline{i}\">
</p>

<p>
and the inductor
</p>

<p>
<IMG
 BORDER=\"0\"
 src=\"modelica://Modelica/Resources/Images/Electrical/QuasiStationary/UsersGuide/Overview/ACCircuit/img2.png\"
 ALT=\"
\\underline{v}_{l}=j\\omega L\\underline{i}\">
</p>

<p>
and the capacitor
</p>

<p>
<IMG
 BORDER=\"0\"
 src=\"modelica://Modelica/Resources/Images/Electrical/QuasiStationary/UsersGuide/Overview/ACCircuit/img3.png\"
 ALT=\"
\\underline{v}_{l}=j\\omega L\\underline{i}\">
</p>

<p>
add up to the total voltage
</p>

<p>
<IMG
 BORDER=\"0\"
 src=\"modelica://Modelica/Resources/Images/Electrical/QuasiStationary/UsersGuide/Overview/ACCircuit/img4.png\"
 ALT=\"
\\underline{v}=\\underline{v}_{r}+\\underline{v}_{l}\">
</p>

<p>
as illustrated in the phasor diagram of Fig. 2.
</p>

<table border=\"0\" cellspacing=\"0\" cellpadding=\"2\">
  <tr>
    <td>
      <img src=\"modelica://Modelica/Resources/Images/Electrical/QuasiStationary/UsersGuide/Overview/ACCircuit/phasor_diagram.png\"
           alt=\"phasor_diagram.png\">
    </td>
  </tr>
  <caption align=\"bottom\">Fig. 2: Phasor diagram of a resistor and inductance series connection</caption>
</table>

<p>Due to the series connection of the resistor, inductor and capacitor, the three currents are all equal:</p>

<p>
<IMG BORDER=\"0\"
 src=\"modelica://Modelica/Resources/Images/Electrical/QuasiStationary/UsersGuide/Overview/ACCircuit/img5.png\"
 alt=\"img5.png\">
</p>

<h4>See also</h4>
<a href=\"modelica://Modelica.Electrical.QuasiStationary.UsersGuide.Overview.Introduction\">
          Introduction</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.UsersGuide.Overview.Power\">
          Power</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.UsersGuide.Overview.ReferenceSystem\">
          Reference system</a>

</html>"));
          end ACCircuit;

          class Power "Real and reactive power"
            extends Modelica.Icons.Information;
            annotation(
              Documentation(info = "<html>

<p>For periodic waveforms, the average value of the instantaneous power is <b>real power</b> <i>P</i>.
<b>Reactive power</b> <i>Q</i> is a term
associated with inductors and capacitors. For pure inductors and capacitors, real power is equal to zero.
Yet, there is instantaneous power exchanged with connecting network.
</p>

The
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Examples.SeriesResonance\">
          series resonance circuit</a> which was also addressed in the
<a href=\"modelica://Modelica.Electrical.QuasiStationary.UsersGuide.Overview.ACCircuit\">
          AC circuit</a>
will be investigated.

<h5>Power of a resistor</h5>

<p>
The instantaneous voltage and current are in phase:
</p>
<p>
<IMG BORDER=\"0\"  src=\"modelica://Modelica/Resources/Images/Electrical/QuasiStationary/UsersGuide/Overview/Power/v_r.png\"
                   alt=\"v_r.png\"> <br>
<IMG BORDER=\"0\"  src=\"modelica://Modelica/Resources/Images/Electrical/QuasiStationary/UsersGuide/Overview/Power/i_r.png\"
                   alt=\"i_r.png\">
</p>

<p>
Therefore, the instantaneous power is
</p>
<p>
<IMG BORDER=\"0\"  src=\"modelica://Modelica/Resources/Images/Electrical/QuasiStationary/UsersGuide/Overview/Power/power_r.png\"
                   alt=\"power_r.png\">
</p>

<p>A graphical representation of these equations is depicted in Fig. 1</p>

<table border=\"0\" cellspacing=\"0\" cellpadding=\"2\">
  <tr>
    <td>
      <img src=\"modelica://Modelica/Resources/Images/Electrical/QuasiStationary/UsersGuide/Overview/Power/power_resistor.png\"
           alt=\"power_resistor.png\">
    </td>
  </tr>
  <caption align=\"bottom\">Fig. 1: Instantaneous voltage, current of power of a resistor</caption>
</table>

<p>Real power of the resistor is the average of instantaneous power:</p>
<p>
<IMG BORDER=\"0\"  src=\"modelica://Modelica/Resources/Images/Electrical/QuasiStationary/UsersGuide/Overview/Power/p_r.png\"
                   alt=\"p_r.png\">
</p>

<h5>Power of an inductor</h5>

<p>
The instantaneous voltage leads the current by a quarter of the period:
</p>
<p>
<IMG BORDER=\"0\"  src=\"modelica://Modelica/Resources/Images/Electrical/QuasiStationary/UsersGuide/Overview/Power/v_l.png\"
                   alt=\"v_l.png\"> <br>
<IMG BORDER=\"0\"  src=\"modelica://Modelica/Resources/Images/Electrical/QuasiStationary/UsersGuide/Overview/Power/i_l.png\"
                   alt=\"i_l.png\">
</p>

<p>
Therefore, the instantaneous power is
</p>
<p>
<IMG BORDER=\"0\"  src=\"modelica://Modelica/Resources/Images/Electrical/QuasiStationary/UsersGuide/Overview/Power/power_l.png\"
                   alt=\"power_l.png\">
</p>

<p>A graphical representation of these equations is depicted in Fig. 2</p>

<table border=\"0\" cellspacing=\"0\" cellpadding=\"2\">
  <tr>
    <td>
      <img src=\"modelica://Modelica/Resources/Images/Electrical/QuasiStationary/UsersGuide/Overview/Power/power_inductor.png\"
           alt=\"power_inductor.png\">
    </td>
  </tr>
  <caption align=\"bottom\">Fig. 2: Instantaneous voltage, current of power of an inductor</caption>
</table>

<p>Reactive power of the inductor is:</p>
<p>
<IMG BORDER=\"0\"  src=\"modelica://Modelica/Resources/Images/Electrical/QuasiStationary/UsersGuide/Overview/Power/q_l.png\"
                   alt=\"q_l.png\">
</p>

<h5>Power of a capacitor</h5>

<p>
The instantaneous voltage lags the current by a quarter of the period:
</p>
<p>
<IMG BORDER=\"0\"  src=\"modelica://Modelica/Resources/Images/Electrical/QuasiStationary/UsersGuide/Overview/Power/v_c.png\"
                   alt=\"v_c.png\"> <br>
<IMG BORDER=\"0\"  src=\"modelica://Modelica/Resources/Images/Electrical/QuasiStationary/UsersGuide/Overview/Power/i_c.png\"
                   alt=\"i_c.png\">
</p>

<p>
Therefore, the instantaneous power is
</p>
<p>
<IMG BORDER=\"0\"  src=\"modelica://Modelica/Resources/Images/Electrical/QuasiStationary/UsersGuide/Overview/Power/power_c.png\"
                   alt=\"power_c.png\">
</p>

<p>A graphical representation of these equations is depicted in Fig. 3</p>

<table border=\"0\" cellspacing=\"0\" cellpadding=\"2\">
  <tr>
    <td>
      <img src=\"modelica://Modelica/Resources/Images/Electrical/QuasiStationary/UsersGuide/Overview/Power/power_capacitor.png\"
           alt=\"power_capacitor.png\">
    </td>
  </tr>
  <caption align=\"bottom\">Fig. 3: Instantaneous voltage, current of power of a capacitor</caption>
</table>

<p>Reactive power of the capacitor is:</p>
<p>
<IMG BORDER=\"0\"  src=\"modelica://Modelica/Resources/Images/Electrical/QuasiStationary/UsersGuide/Overview/Power/q_c.png\"
                   alt=\"q_c.png\">
</p>

<h5>Complex apparent power</h5>

<p>For an arbitrary component with two pins, real and reactive power can be determined by the complex phasors:</p>
<p>
<IMG BORDER=\"0\"  src=\"modelica://Modelica/Resources/Images/Electrical/QuasiStationary/UsersGuide/Overview/Power/s.png\"
                   alt=\"s.png\">
</p>

<p>
In this equation <sup>*</sup> represents the conjugate complex operator
</p>

<h4>See also</h4>
<a href=\"modelica://Modelica.Electrical.QuasiStationary.UsersGuide.Overview.Introduction\">
          Introduction</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.UsersGuide.Overview.ACCircuit\">
          AC circuit</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.UsersGuide.Overview.ReferenceSystem\">
          Reference system</a>
</html>"));
          end Power;

          class ReferenceSystem "Reference system"
            extends Modelica.Icons.Information;
            annotation(
              Documentation(info = "<html>
<p>
The reference angle <code>gamma</code>:
</p>
<ul>
  <li>defines the angular frequency <code>omega</code> of the voltages and currents
      of a circuit by means of <code>omega = der(gamma)</code>. </li>
  <li>is not a global quantity since it propagated through the connector.
      Therefore, independent circuits of different frequencies can be modeled in one model.</li>
  <li>is present only once in a multiphase connector;
      a multiphase component has only one reference angle common to all phases.</li>
  <li>can be either constant or variable, but it has to be consistent in one contiguous circuit.</li>
  <li>is defined by the sources.</li>
</ul>
<p>
Designing new components, the guidelines of the Modelica Specification dealing with
Overconstrained Equation Operators for Connection Graphs have to be taken into account.
</p>

<h4>See also</h4>
<a href=\"modelica://Modelica.Electrical.QuasiStationary.UsersGuide.Overview.Introduction\">
          Introduction</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.UsersGuide.Overview.ACCircuit\">
          AC circuit</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.UsersGuide.Overview.Power\">
          Power</a>

</html>"));
          end ReferenceSystem;
          annotation(
            Documentation(info = "<html>
<p>
The <a href=\"modelica://Modelica.Electrical.QuasiStationary\">Modelica.Electrical.QuasiStationary</a>
library addresses the analysis of electrical circuits with purely sinusoidal
voltages and currents. The main characteristics of the library are:
</p>

<ul>
  <li>Only pure sinusoidal voltages and currents are taken into account.
      Higher harmonic voltages and currents are not considered.</li>
  <li>Any electrical transient effects are neglected.</li>
  <li>The electrical components of this library are strictly linear.</li>
  <li>The angular frequency <code>omega</code> of the voltages and currents of
      a circuit are determined from a reference angle <code>gamma</code> by means of
      <code>omega = der(gamma)</code>. </li>
  <li>The reference angle <code>gamma</code> is not a global quantity
      since it propagated through the connector.
      Therefore, independent circuits of different frequencies can be modeled in one model.</li>
  <li>The connectors contain the real and the imaginary part of the voltage and the current
      <a href=\"modelica://Modelica.Electrical.QuasiStationary.UsersGuide.Overview.Introduction\">RMS phasors</a></li>

</ul>

<p>
The main intention of this library is the modeling of quasi stationary behavior
of single and multi phase
<a href=\"modelica://Modelica.Electrical.QuasiStationary.UsersGuide.Overview.ACCircuit\">AC circuits</a>
with fixed and variable frequency. Quasi stationary theory and applications can be
found in
[<a href=\"modelica://Modelica.Electrical.QuasiStationary.UsersGuide.References\">Dorf1993</a>],
[<a href=\"modelica://Modelica.Electrical.QuasiStationary.UsersGuide.References\">Burton1994</a>],
[<a href=\"modelica://Modelica.Electrical.QuasiStationary.UsersGuide.References\">Landolt1936</a>],
[<a href=\"modelica://Modelica.Electrical.QuasiStationary.UsersGuide.References\">Philippow1967</a>],
[<a href=\"modelica://Modelica.Electrical.QuasiStationary.UsersGuide.References\">Weyh1967</a>],
[<a href=\"modelica://Modelica.Electrical.QuasiStationary.UsersGuide.References\">Vaske1973</a>].
</p>

<h4>Note</h4>
<p>
A general electrical circuit can be a DC circuit, an AC circuit with periodic sinusoidal or non-sinusoidal voltages and currents
or a transient circuit without particular waveform of voltages and currents.
Therefore a coupling model between a quasi stationary circuit and a general (transient) electrical circuit
has to be designed carefully taking the specific application into account.
As an example, you may look at the <a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Utilities.IdealACDCConverter\">
ideal AC DC converter</a>, which is used in the
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Examples.Rectifier\">rectifier example</a>.
</p>

</html>"));
        end Overview;

        class ReleaseNotes "Release notes"
          extends Modelica.Icons.ReleaseNotes;
          annotation(
            Documentation(info = "<html>
</html>", revisions = "<html>
<h4>Version 1.0.0</h4>

<p>First official release</p>
</html>"));
        end ReleaseNotes;

        class References "References"
          extends Modelica.Icons.References;
          annotation(
            Documentation(info = "<html>

<table border=\"0\" cellspacing=\"0\" cellpadding=\"2\">
    <tr>
      <td valign=\"top\">[Dorf1993]</td>
      <td valign=\"top\">R. C. Dorf
        <i>The Electrical Engineering</i>,
        VDE, 1993.</td>
    </tr>

    <tr>
      <td valign=\"top\">[Boas1966]</td>
      <td valign=\"top\">M. L. Boas
        <i>Mathematical Methods in the Physical Sciences</i>,
        J. Wiley &amp; Sons, New York, 1966.</td>
    </tr>

    <tr>
      <td valign=\"top\">[Burton1994]</td>
      <td valign=\"top\">T. Burton
        <i>Introduction to Dynamic Systems Analysis</i>,
        McGraw Hill, New York, 1994.</td>
    </tr>

    <tr>
      <td valign=\"top\">[Landolt1936]</td>
      <td valign=\"top\">M. Landolt
        <i>Komplexe Zahlen und Zeiger in der Wechselstromlehre</i>,
        Springer Verlag, Berlin, 1936</td>
    </tr>

    <tr>
      <td valign=\"top\">[Philippow1967]</td>
      <td valign=\"top\">E. Philippow
        <i>Grundlagen der Elektrotechnik</i>,
       Akademischer Verlag, Leipzig, 1967.</td>
    </tr>

    <tr>
      <td valign=\"top\">[Weyh1967]</td>
      <td valign=\"top\">Weyh and Benzinger
        <i>Die Grundlagen der Wechselstromlehre</i>,
       R. Oldenbourg Verlag, 1967.</td>
    </tr>

    <tr>
      <td valign=\"top\">[Vaske1973]</td>
      <td valign=\"top\">P. Vaske
        <i>Berechnung von Drehstromschaltungen</i>,
       B.G. Teubner Verlag, 1973.</td>
    </tr>

</table>

</html>"));
        end References;

        class Contact "Contact"
          extends Modelica.Icons.Contact;
          annotation(
            Documentation(info = "<html>

<p>This package is developed an maintained by the following contributors</p>

  <table border=\"1\" cellspacing=\"0\" cellpadding=\"2\">
    <tr>
      <th></th>
      <th>Name</th>
      <th>Affiliation</th>
    </tr>
    <tr>
      <td valign=\"top\">Library officer</td>
      <td valign=\"top\">
      <a href=\"mailto:a.haumer@haumer.at\">A. Haumer</a>
      </td>
      <td valign=\"top\">
        <a href=\"http://www.haumer.at\">Technical Consulting &amp; Electrical Engineering</a><br>
        3423 St.Andrae-Woerdern<br>
        Austria
      </td>
    </tr>
    <tr>
      <td valign=\"top\">Library officer</td>
      <td valign=\"top\">
        <a href=\"mailto:dr.christian.kral@gmail.com\">C. Kral</a>
      </td>
      <td valign=\"top\">
        <a href=\"http://christiankral.net\">Electric Machines, Drives and Systems</a><br>
        1060 Vienna<br>
        Austria
      </td>
    </tr>
  </table>

</html>"));
        end Contact;

        class Glossar "Glossar"
          extends Modelica.Icons.Information;
          annotation(
            Documentation(info = "<html>
<table border=\"1\" cellspacing=\"0\" cellpadding=\"2\">
  <tr>
    <th>Abbreviation</th>
    <th>Comment</th>
  </tr>
  <tr>
    <td>AC</td>
    <td>alternating current</td>
  </tr>
  <tr>
    <td>RMS</td>
    <td>root mean square</td>
  </tr>
</table>
</html>"));
        end Glossar;
        annotation(
          DocumentationClass = true,
          Documentation(info = "<html>
</html>"));
      end UsersGuide;

      package SinglePhase "Single phase AC components"
        extends Modelica.Icons.Package;

        package Examples "Test examples"
          extends Modelica.Icons.ExamplesPackage;

          model SeriesResonance "Series resonance circuit"
            extends Modelica.Icons.Example;
            Modelica.Blocks.Sources.Constant V(k = 1) annotation(
              Placement(transformation(origin = {-40, 50}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
            Modelica.Blocks.Sources.Constant phi(k = 0) annotation(
              Placement(transformation(origin = {-80, 50}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
            Modelica.Blocks.Sources.Ramp f(height = 2, duration = 1, offset = 1e-6) annotation(
              Placement(transformation(origin = {-60, -50}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
            QuasiStationary.SinglePhase.Sources.VariableVoltageSource voltageSource annotation(
              Placement(transformation(origin = {-30, -20}, extent = {{-10, 10}, {10, -10}}, rotation = 270)));
            QuasiStationary.SinglePhase.Basic.Ground ground annotation(
              Placement(transformation(extent = {{-40, -60}, {-20, -40}}, rotation = 0)));
            QuasiStationary.SinglePhase.Basic.Resistor resistor(R_ref = 0.1) annotation(
              Placement(transformation(extent = {{10, -10}, {30, 10}}, rotation = 0)));
            QuasiStationary.SinglePhase.Basic.Inductor inductor(L = 1 / (2 * Modelica.Constants.pi)) annotation(
              Placement(transformation(extent = {{40, -10}, {60, 10}}, rotation = 0)));
            QuasiStationary.SinglePhase.Basic.Capacitor capacitor(C = 1 / (2 * Modelica.Constants.pi)) annotation(
              Placement(transformation(extent = {{70, -10}, {90, 10}}, rotation = 0)));
            QuasiStationary.SinglePhase.Sensors.CurrentSensor currentSensor annotation(
              Placement(transformation(extent = {{-20, 10}, {0, -10}}, rotation = 0)));
            Modelica.ComplexBlocks.ComplexMath.PolarToComplex polarToComplex annotation(
              Placement(transformation(origin = {-60, 10}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
            Modelica.ComplexBlocks.ComplexMath.ComplexToPolar complexToPolar annotation(
              Placement(transformation(origin = {-10, 30}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
          initial equation
            voltageSource.pin_p.reference.gamma = 0;
          equation
            connect(f.y, voltageSource.f) annotation(
              Line(points = {{-60, -39}, {-60, -24}, {-40, -24}}, color = {0, 0, 127}));
            connect(polarToComplex.y, voltageSource.V) annotation(
              Line(points = {{-60, -1}, {-60, -16}, {-40, -16}}, color = {85, 170, 255}));
            connect(ground.pin, voltageSource.pin_n) annotation(
              Line(points = {{-30, -40}, {-30, -35}, {-30, -30}}, color = {85, 170, 255}));
            connect(voltageSource.pin_p, currentSensor.pin_p) annotation(
              Line(points = {{-30, -10}, {-30, 0}, {-20, 0}}, color = {85, 170, 255}));
            connect(currentSensor.pin_n, resistor.pin_p) annotation(
              Line(points = {{0, 0}, {2.5, 0}, {5, 0}, {10, 0}}, color = {85, 170, 255}));
            connect(resistor.pin_n, inductor.pin_p) annotation(
              Line(points = {{30, 0}, {32.5, 0}, {35, 0}, {40, 0}}, color = {85, 170, 255}));
            connect(inductor.pin_n, capacitor.pin_p) annotation(
              Line(points = {{60, 0}, {62.5, 0}, {65, 0}, {70, 0}}, color = {85, 170, 255}));
            connect(capacitor.pin_n, ground.pin) annotation(
              Line(points = {{90, 0}, {90, -40}, {-30, -40}}, color = {85, 170, 255}));
            connect(phi.y, polarToComplex.phi) annotation(
              Line(points = {{-80, 39}, {-80, 30}, {-66, 30}, {-66, 22}}, color = {0, 0, 127}));
            connect(V.y, polarToComplex.len) annotation(
              Line(points = {{-40, 39}, {-40, 30}, {-54, 30}, {-54, 22}}, color = {0, 0, 127}, smooth = Smooth.None));
            connect(currentSensor.y, complexToPolar.u) annotation(
              Line(points = {{-10, 11}, {-10, 18}}, color = {85, 170, 255}, smooth = Smooth.None));
            annotation(
              Documentation(info = "<html>
<p>
The frequency of the voltage source is varied by a ramp.
Plot length and angle of the current phasor, i.e., complexToPolar.len and .phi, versus time resp. frequency.
</p>
</html>"),
              experiment(StopTime = 1.0, Interval = 0.001));
          end SeriesResonance;

          model ParallelResonance "Parallel resonance circuit"
            extends Modelica.Icons.Example;
            Modelica.Blocks.Sources.Constant I(k = 1) annotation(
              Placement(transformation(origin = {-80, -50}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
            Modelica.Blocks.Sources.Constant phi(k = 0) annotation(
              Placement(transformation(origin = {-40, -50}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
            Modelica.Blocks.Sources.Ramp f(height = 2, duration = 1, offset = 1e-6) annotation(
              Placement(transformation(origin = {-60, 50}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
            QuasiStationary.SinglePhase.Sources.VariableCurrentSource currentSource annotation(
              Placement(transformation(origin = {-30, 20}, extent = {{10, 10}, {-10, -10}}, rotation = 270)));
            QuasiStationary.SinglePhase.Basic.Ground ground annotation(
              Placement(transformation(extent = {{-40, -20}, {-20, 0}}, rotation = 0)));
            QuasiStationary.SinglePhase.Basic.Resistor resistor(R_ref = 10) annotation(
              Placement(transformation(origin = {-10, 20}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
            QuasiStationary.SinglePhase.Basic.Inductor inductor(L = 1 / (2 * Modelica.Constants.pi)) annotation(
              Placement(transformation(origin = {10, 20}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
            QuasiStationary.SinglePhase.Basic.Capacitor capacitor(C = 1 / (2 * Modelica.Constants.pi)) annotation(
              Placement(transformation(origin = {30, 20}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
            QuasiStationary.SinglePhase.Sensors.VoltageSensor voltageSensor annotation(
              Placement(transformation(origin = {50, 20}, extent = {{10, -10}, {-10, 10}}, rotation = 90)));
            Modelica.ComplexBlocks.ComplexMath.PolarToComplex polarToComplex annotation(
              Placement(transformation(origin = {-60, -10}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
            Modelica.ComplexBlocks.ComplexMath.ComplexToPolar complexToPolar annotation(
              Placement(transformation(extent = {{70, 10}, {90, 30}}, rotation = 0)));
          initial equation
            currentSource.pin_p.reference.gamma = 0;
          equation
            connect(currentSource.pin_n, resistor.pin_p) annotation(
              Line(points = {{-30, 30}, {-30, 40}, {-10, 40}, {-10, 30}}, color = {85, 170, 255}));
            connect(currentSource.pin_n, inductor.pin_p) annotation(
              Line(points = {{-30, 30}, {-30, 40}, {10, 40}, {10, 30}}, color = {85, 170, 255}));
            connect(currentSource.pin_n, capacitor.pin_p) annotation(
              Line(points = {{-30, 30}, {-30, 40}, {30, 40}, {30, 30}}, color = {85, 170, 255}));
            connect(currentSource.pin_n, voltageSensor.pin_p) annotation(
              Line(points = {{-30, 30}, {-30, 40}, {50, 40}, {50, 30}}, color = {85, 170, 255}));
            connect(currentSource.pin_p, ground.pin) annotation(
              Line(points = {{-30, 10}, {-30, 5}, {-30, 0}}, color = {85, 170, 255}));
            connect(resistor.pin_n, ground.pin) annotation(
              Line(points = {{-10, 10}, {-10, 0}, {-30, 0}}, color = {85, 170, 255}));
            connect(inductor.pin_n, ground.pin) annotation(
              Line(points = {{10, 10}, {10, 0}, {-30, 0}}, color = {85, 170, 255}));
            connect(capacitor.pin_n, ground.pin) annotation(
              Line(points = {{30, 10}, {30, 0}, {-30, 0}}, color = {85, 170, 255}));
            connect(voltageSensor.pin_n, ground.pin) annotation(
              Line(points = {{50, 10}, {50, 0}, {-30, 0}}, color = {85, 170, 255}));
            connect(f.y, currentSource.f) annotation(
              Line(points = {{-60, 39}, {-60, 24}, {-40, 24}}, color = {0, 0, 127}));
            connect(polarToComplex.y, currentSource.I) annotation(
              Line(points = {{-60, 1}, {-60, 16}, {-40, 16}}, color = {85, 170, 255}));
            connect(phi.y, polarToComplex.phi) annotation(
              Line(points = {{-40, -39}, {-40, -32}, {-54, -32}, {-54, -22}}, color = {0, 0, 127}));
            connect(I.y, polarToComplex.len) annotation(
              Line(points = {{-80, -39}, {-80, -32}, {-66, -32}, {-66, -22}}, color = {0, 0, 127}, smooth = Smooth.None));
            connect(voltageSensor.y, complexToPolar.u) annotation(
              Line(points = {{61, 20}, {68, 20}}, color = {85, 170, 255}, smooth = Smooth.None));
            annotation(
              Documentation(info = "<html>
<p>
The frequency of the current source is varied by a ramp.
Plot length and angle of the voltage phasor, i.e., complexToPolar.len and .phi, versus time resp. frequency.
</p>
</html>"),
              experiment(StopTime = 1.0, Interval = 0.001));
          end ParallelResonance;

          model Rectifier "Rectifier example"
            extends Modelica.Icons.Example;
            import Modelica.Constants.pi;
            parameter Modelica.SIunits.Voltage VAC = 100 "AC rms voltage";
            parameter Real conversionFactor = 1 "Ratio of DC voltage / AC rms voltage";
            Sources.VoltageSource voltageQS(f = 50, V = VAC, phi = 0, i(re(start = 0), im(start = 0))) annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 270, origin = {-80, 50})));
            Basic.Resistor resistorQS(R_ref = 50E-3) annotation(
              Placement(transformation(extent = {{-72, 50}, {-52, 70}})));
            Sensors.CurrentSensor currentSensorQS annotation(
              Placement(transformation(extent = {{-40, 70}, {-20, 50}})));
            ComplexBlocks.ComplexMath.ComplexToPolar iQS annotation(
              Placement(transformation(extent = {{-20, 70}, {0, 90}})));
            Basic.Ground groundQS annotation(
              Placement(transformation(extent = {{-20, 10}, {0, 30}})));
            Utilities.IdealACDCConverter rectifierQS(conversionFactor = conversionFactor) annotation(
              Placement(transformation(extent = {{-10, 40}, {10, 60}})));
            Analog.Basic.Ground groundDC1 annotation(
              Placement(transformation(extent = {{0, 10}, {20, 30}})));
            Analog.Sensors.CurrentSensor iDC1 annotation(
              Placement(transformation(extent = {{20, 70}, {40, 50}})));
            Analog.Basic.VariableConductor load1 annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 270, origin = {50, 50})));
            Analog.Sources.SineVoltage voltageAC(V = sqrt(2) * VAC, freqHz = 50, phase = pi / 2) annotation(
              Placement(transformation(extent = {{-10, 10}, {10, -10}}, rotation = 270, origin = {-80, -50})));
            Analog.Basic.Resistor resistorAC(R = 50E-3) annotation(
              Placement(transformation(extent = {{-70, -50}, {-50, -30}})));
            Analog.Sensors.CurrentSensor currentSensorAC annotation(
              Placement(transformation(extent = {{-40, -30}, {-20, -50}})));
            Modelica.Blocks.Math.Harmonic iAC(f = 50, k = 1) annotation(
              Placement(transformation(extent = {{-20, -30}, {0, -10}})));
            Analog.Basic.Ground groundAC annotation(
              Placement(transformation(extent = {{-20, -90}, {0, -70}})));
            Utilities.GraetzRectifier rectifierAC annotation(
              Placement(transformation(extent = {{-10, -60}, {10, -40}})));
            Analog.Sensors.CurrentSensor currentSensorDC2 annotation(
              Placement(transformation(extent = {{20, -30}, {40, -50}})));
            Modelica.Blocks.Math.RootMeanSquare iDC2(f = 50) annotation(
              Placement(transformation(extent = {{40, -30}, {60, -10}})));
            Analog.Basic.VariableConductor load2 annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 270, origin = {50, -50})));
            Blocks.Sources.Ramp ramp(height = 1, duration = 0.8, startTime = 0.1) annotation(
              Placement(transformation(extent = {{100, -10}, {80, 10}})));
          initial equation
            voltageQS.pin_p.reference.gamma = 0;
          equation
            connect(voltageQS.pin_p, resistorQS.pin_p) annotation(
              Line(points = {{-80, 60}, {-72, 60}}, color = {85, 170, 255}, smooth = Smooth.None));
            connect(voltageQS.pin_n, rectifierQS.pin_nQS) annotation(
              Line(points = {{-80, 40}, {-10, 40}}, color = {85, 170, 255}, smooth = Smooth.None));
            connect(rectifierQS.pin_nQS, groundQS.pin) annotation(
              Line(points = {{-10, 40}, {-10, 30}}, color = {85, 170, 255}, smooth = Smooth.None));
            connect(rectifierQS.pin_nDC, groundDC1.p) annotation(
              Line(points = {{10, 40}, {10, 30}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(load1.n, rectifierQS.pin_nDC) annotation(
              Line(points = {{50, 40}, {10, 40}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(voltageAC.p, resistorAC.p) annotation(
              Line(points = {{-80, -40}, {-70, -40}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(voltageAC.n, rectifierAC.pin_nAC) annotation(
              Line(points = {{-80, -60}, {-10, -60}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(rectifierAC.pin_nAC, groundAC.p) annotation(
              Line(points = {{-10, -60}, {-10, -70}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(rectifierAC.pin_nDC, load2.n) annotation(
              Line(points = {{10, -60}, {50, -60}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(ramp.y, load1.G) annotation(
              Line(points = {{79, 0}, {70, 0}, {70, 50}, {61, 50}}, color = {0, 0, 127}, smooth = Smooth.None));
            connect(ramp.y, load2.G) annotation(
              Line(points = {{79, 0}, {70, 0}, {70, -50}, {61, -50}}, color = {0, 0, 127}, smooth = Smooth.None));
            connect(resistorQS.pin_n, currentSensorQS.pin_p) annotation(
              Line(points = {{-52, 60}, {-40, 60}}, color = {85, 170, 255}, smooth = Smooth.None));
            connect(currentSensorQS.pin_n, rectifierQS.pin_pQS) annotation(
              Line(points = {{-20, 60}, {-10, 60}}, color = {85, 170, 255}, smooth = Smooth.None));
            connect(currentSensorQS.y, iQS.u) annotation(
              Line(points = {{-30, 71}, {-30, 80}, {-22, 80}}, color = {85, 170, 255}, smooth = Smooth.None));
            connect(rectifierQS.pin_pDC, iDC1.p) annotation(
              Line(points = {{10, 60}, {20, 60}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(iDC1.n, load1.p) annotation(
              Line(points = {{40, 60}, {50, 60}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(resistorAC.n, currentSensorAC.p) annotation(
              Line(points = {{-50, -40}, {-40, -40}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(currentSensorAC.n, rectifierAC.pin_pAC) annotation(
              Line(points = {{-20, -40}, {-10, -40}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(currentSensorAC.i, iAC.u) annotation(
              Line(points = {{-30, -30}, {-30, -20}, {-22, -20}}, color = {0, 0, 127}, smooth = Smooth.None));
            connect(currentSensorDC2.i, iDC2.u) annotation(
              Line(points = {{30, -30}, {30, -20}, {38, -20}}, color = {0, 0, 127}, smooth = Smooth.None));
            connect(currentSensorDC2.p, rectifierAC.pin_pDC) annotation(
              Line(points = {{20, -40}, {10, -40}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(currentSensorDC2.n, load2.p) annotation(
              Line(points = {{40, -40}, {50, -40}}, color = {0, 0, 255}, smooth = Smooth.None));
            annotation(
              Documentation(info = "<html>
<p>
This example demonstrates coupling a quasi stationary circuit with a DC circuit.
The QS voltage is rectified (using an
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Utilities.IdealACDCConverter\">
ideal AC DC converter</a>), loaded by a variable load conductor.
The <i>conversionFactor = DC voltage / AC rms voltage</i> in this case is the root mean square of a rectified sine, i.e., 1.
You may compare the quasi stationary results with that of a fully transient model (using a
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Utilities.GraetzRectifier\">
Graetz rectifier</a>), plotting:
</p>
<ul>
<li>QS: AC rms           current = iQS.len</li>
<li>AC: AC instantaneous current = iAC.u</li>
<li>AC: AC rms           current = iAC.y_rms</li>
<li>QS: DC current               = iDC1.i</li>
<li>AC: DC instantaneous current = iDC2.u</li>
<li>AC: DC rms           current = iDC2.y</li>
</ul>
<p>
It can be seen that at the DC side the current is represented by its averaged value, at the AC side by its rms value.
</p>
<h4>Note</h4>
<p>
The quasi stationary model needs a grounding at the QS side as well as the DC side,
whereas the transient model may have only one ground since AC side and DC side are connected via the diodes.
</p>
</html>"),
              experiment(StopTime = 1.0, Interval = 0.0001));
          end Rectifier;
          annotation(
            Documentation(info = "<html>
Examples to demonstrate the usage of quasistationary electric components.
</html>"));
        end Examples;

        package Basic "Basic components for AC singlephase models"
          extends Modelica.Icons.Package;

          model Ground "Electrical ground"
            Interfaces.PositivePin pin annotation(
              Placement(transformation(extent = {{-10, 90}, {10, 110}}, rotation = 0)));
          equation
            Connections.potentialRoot(pin.reference, 256);
            if Connections.isRoot(pin.reference) then
              pin.reference.gamma = 0;
            end if;
            pin.v = Complex(0);
            annotation(
              Icon(graphics = {Line(points = {{-60, 50}, {60, 50}}, color = {85, 170, 255}), Line(points = {{-40, 30}, {40, 30}}, color = {85, 170, 255}), Line(points = {{-20, 10}, {20, 10}}, color = {85, 170, 255}), Line(points = {{0, 90}, {0, 50}}, color = {85, 170, 255}), Text(extent = {{100, -60}, {-100, -20}}, textString = "%name", lineColor = {0, 0, 255})}),
              Documentation(info = "<html>
<p>
Ground of a single phase circuit. The potential at the ground node is zero.
Every electrical circuit, e.g., a series resonance
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Examples.SeriesResonance\">
          example</a>, has to contain at least one ground object.
</p>

</html>"));
          end Ground;

          model Resistor "Singlephase linear resistor"
            extends Interfaces.OnePort;
            import Modelica.ComplexMath.real;
            import Modelica.ComplexMath.conj;
            parameter Modelica.SIunits.Resistance R_ref(start = 1) "Reference resistance at T_ref";
            parameter Modelica.SIunits.Temperature T_ref = 293.15 "Reference temperature";
            parameter Modelica.SIunits.LinearTemperatureCoefficient alpha_ref = 0 "Temperature coefficient of resistance (R_actual = R_ref*(1 + alpha_ref*(heatPort.T - T_ref))";
            extends Modelica.Electrical.Analog.Interfaces.ConditionalHeatPort(T = T_ref);
            Modelica.SIunits.Resistance R_actual "Resistance = R_ref*(1 + alpha_ref*(heatPort.T - T_ref))";
          equation
            assert(1 + alpha_ref * (T_heatPort - T_ref) >= Modelica.Constants.eps, "Temperature outside scope of model!");
            R_actual = R_ref * (1 + alpha_ref * (T_heatPort - T_ref));
            v = R_actual * i;
            LossPower = real(v * conj(i));
            annotation(
              Icon(graphics = {Text(extent = {{100, 60}, {-100, 100}}, textString = "%name", lineColor = {0, 0, 255}), Line(points = {{60, 0}, {90, 0}}, color = {85, 170, 255}), Line(points = {{-90, 0}, {-60, 0}}, color = {85, 170, 255}), Rectangle(extent = {{-70, 30}, {70, -30}}, lineColor = {85, 170, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Text(extent = {{100, -80}, {-100, -40}}, textString = "R=%R_ref")}),
              Documentation(info = "<html>
<p>
The linear resistor connects the complex voltage <i><u>v</u></i> with the complex
current <i><u>i</u></i> by <i><u>i</u>*R = <u>v</u></i>.
The resistance <i>R</i> is allowed to be positive, zero, or negative.
</p>

<p>
The resistor model also has an optional
<a href=\"modelica://Modelica.Electrical.Analog.Interfaces.ConditionalHeatPort\">conditional heat port</a>.
A linear temperature dependency of the resistance for an enabled heat port is also taken into account.
</p>

<h4>See also</h4>
<p>
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Basic.Conductor\">Conductor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Basic.Capacitor\">Capacitor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Basic.Inductor\">Inductor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Basic.VariableResistor\">Variable resistor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Basic.VariableConductor\">Variable conductor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Basic.VariableCapacitor\">Variable capacitor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Basic.VariableInductor\">Variable inductor</a>
</p>
</html>"));
          end Resistor;

          model Conductor "Singlephase linear conductor"
            extends Interfaces.OnePort;
            import Modelica.ComplexMath.real;
            import Modelica.ComplexMath.conj;
            parameter Modelica.SIunits.Conductance G_ref(start = 1) "Reference conductance at T_ref";
            parameter Modelica.SIunits.Temperature T_ref = 293.15 "Reference temperature";
            parameter Modelica.SIunits.LinearTemperatureCoefficient alpha_ref = 0 "Temperature coefficient of conductance (G_actual = G_ref/(1 + alpha_ref*(heatPort.T - T_ref))";
            extends Modelica.Electrical.Analog.Interfaces.ConditionalHeatPort(T = T_ref);
            Modelica.SIunits.Conductance G_actual "Conductance = G_ref/(1 + alpha_ref*(heatPort.T - T_ref))";
          equation
            assert(1 + alpha_ref * (T_heatPort - T_ref) >= Modelica.Constants.eps, "Temperature outside scope of model!");
            G_actual = G_ref / (1 + alpha_ref * (T_heatPort - T_ref));
            i = G_actual * v;
            LossPower = real(v * conj(i));
            annotation(
              Icon(graphics = {Text(extent = {{100, 60}, {-100, 100}}, textString = "%name", lineColor = {0, 0, 255}), Line(points = {{60, 0}, {90, 0}}, color = {85, 170, 255}), Line(points = {{-90, 0}, {-60, 0}}, color = {85, 170, 255}), Rectangle(extent = {{-70, 30}, {70, -30}}, lineColor = {85, 170, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Text(extent = {{100, -80}, {-100, -40}}, textString = "G=%G_ref")}),
              Documentation(info = "<html>

<p>
The linear conductor connects the voltage <i><u>v</u></i> with the
current <i><u>i</u></i> by <i><u>i</u> = <u>v</u>*G</i>.
The conductance <i>G</i> is allowed to be positive, zero, or negative.
</p>

<p>
The conductor model also has an optional
<a href=\"modelica://Modelica.Electrical.Analog.Interfaces.ConditionalHeatPort\">conditional heat port</a>.
A linear temperature dependency of the resistance for an enabled heat port is also taken into account.
</p>

<h4>See also</h4>
<p>
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Basic.Resistor\">Resistor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Basic.Capacitor\">Capacitor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Basic.Inductor\">Inductor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Basic.VariableResistor\">Variable resistor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Basic.VariableConductor\">Variable conductor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Basic.VariableCapacitor\">Variable capacitor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Basic.VariableInductor\">Variable inductor</a>
</p>
</html>"));
          end Conductor;

          model Capacitor "Singlephase linear capacitor"
            extends Interfaces.OnePort;
            import Modelica.ComplexMath.j;
            parameter Modelica.SIunits.Capacitance C(start = 1) "Capacitance";
          equation
            i = j * omega * C * v;
            annotation(
              Icon(graphics = {Text(extent = {{100, 60}, {-100, 100}}, textString = "%name", lineColor = {0, 0, 255}), Line(points = {{-14, 28}, {-14, -28}}, color = {85, 170, 255}, thickness = 0.5), Line(points = {{14, 28}, {14, -28}}, color = {85, 170, 255}, thickness = 0.5), Line(points = {{-90, 0}, {-14, 0}}, color = {85, 170, 255}), Line(points = {{14, 0}, {90, 0}}, color = {85, 170, 255}), Text(extent = {{100, -80}, {-100, -40}}, textString = "C=%C")}),
              Documentation(info = "<html>

<p>
The linear capacitor connects the voltage <i><u>v</u></i> with the
current <i><u>i</u></i> by <i><u>i</u> = j*&omega;*C*<u>v</u></i>.
The capacitance <i>C</i> is allowed to be positive, zero, or negative.
</p>

<h4>See also</h4>
<p>
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Basic.Resistor\">Resistor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Basic.Conductor\">Conductor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Basic.Inductor\">Inductor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Basic.VariableResistor\">Variable resistor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Basic.VariableConductor\">Variable conductor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Basic.VariableCapacitor\">Variable capacitor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Basic.VariableInductor\">Variable inductor</a>
</p>
</html>"));
          end Capacitor;

          model Inductor "Singlephase linear inductor"
            extends Interfaces.OnePort;
            import Modelica.ComplexMath.j;
            parameter Modelica.SIunits.Inductance L(start = 1) "Inductance";
          equation
            v = j * omega * L * i;
            annotation(
              Icon(graphics = {Text(extent = {{100, 60}, {-100, 100}}, textString = "%name", lineColor = {0, 0, 255}), Ellipse(extent = {{-60, -15}, {-30, 15}}, lineColor = {85, 170, 255}), Ellipse(extent = {{-30, -15}, {0, 15}}, lineColor = {85, 170, 255}), Ellipse(extent = {{0, -15}, {30, 15}}, lineColor = {85, 170, 255}), Ellipse(extent = {{30, -15}, {60, 15}}, lineColor = {85, 170, 255}), Rectangle(extent = {{-60, -30}, {60, 0}}, lineColor = {255, 255, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{60, 0}, {90, 0}}, color = {85, 170, 255}), Line(points = {{-90, 0}, {-60, 0}}, color = {85, 170, 255}), Text(extent = {{100, -80}, {-100, -40}}, textString = "L=%L")}),
              Documentation(info = "<html>

<p>
The linear inductor connects the voltage <i><u>v</u></i> with the
current <i><u>i</u></i> by  <i><u>v</u> = j*&omega;*L*<u>i</u></i>.
The Inductance <i>L</i> is allowed to be positive, zero, or negative.
</p>

<h4>See also</h4>
<p>
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Basic.Resistor\">Resistor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Basic.Conductor\">Conductor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Basic.Capacitor\">Capacitor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Basic.VariableResistor\">Variable resistor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Basic.VariableConductor\">Variable conductor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Basic.VariableCapacitor\">Variable capacitor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Basic.VariableInductor\">Variable inductor</a>
</p>
</html>"));
          end Inductor;

          model VariableResistor "Singlephase variable resistor"
            extends Interfaces.OnePort;
            import Modelica.ComplexMath.real;
            import Modelica.ComplexMath.conj;
            parameter Modelica.SIunits.Temperature T_ref = 293.15 "Reference temperature";
            parameter Modelica.SIunits.LinearTemperatureCoefficient alpha_ref = 0 "Temperature coefficient of resistance (R_actual = R_ref*(1 + alpha_ref*(heatPort.T - T_ref))";
            extends Modelica.Electrical.Analog.Interfaces.ConditionalHeatPort(T = T_ref);
            Modelica.SIunits.Resistance R_actual "Resistance = R_ref*(1 + alpha_ref*(heatPort.T - T_ref))";
            Modelica.Blocks.Interfaces.RealInput R_ref(unit = "Ohm") annotation(
              Placement(transformation(origin = {0, 110}, extent = {{-20, -20}, {20, 20}}, rotation = 270)));
          equation
            assert(1 + alpha_ref * (T_heatPort - T_ref) >= Modelica.Constants.eps, "Temperature outside scope of model!");
            R_actual = R_ref * (1 + alpha_ref * (T_heatPort - T_ref));
            v = R_actual * i;
            LossPower = real(v * conj(i));
            annotation(
              Icon(graphics = {Text(extent = {{100, -80}, {-100, -40}}, textString = "%name", lineColor = {0, 0, 255}), Line(points = {{60, 0}, {90, 0}}, color = {85, 170, 255}), Line(points = {{-90, 0}, {-60, 0}}, color = {85, 170, 255}), Rectangle(extent = {{-70, 30}, {70, -30}}, lineColor = {85, 170, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{0, 90}, {0, 30}}, color = {85, 170, 255}, smooth = Smooth.None)}),
              Documentation(info = "<html>

<p>
The linear resistor connects the voltage <i><u>v</u></i> with the
current <i><u>i</u></i> by <i><u>i</u>*R = <u>v</u></i>.
The resistance <i>R</i> is given as input signal.
</p>

<p>
The variable resistor model also has an optional
<a href=\"modelica://Modelica.Electrical.Analog.Interfaces.ConditionalHeatPort\">conditional heat port</a>.
A linear temperature dependency of the resistance for an enabled heat port is also taken into account.
</p>

<h4>Note</h4>
<p>
A zero crossing of the R signal could cause singularities due to the actual structure of the connected network.
</p>

<p>
The variable resistor model also has an optional
<a href=\"modelica://Modelica.Electrical.Analog.Interfaces.ConditionalHeatPort\">conditional heat port</a>.
A linear temperature dependency of the resistance for an enabled heat port is also taken into account.
</p>

<h4>See also</h4>
<p>
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Basic.Resistor\">Resistor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Basic.Conductor\">Conductor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Basic.Capacitor\">Capacitor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Basic.Inductor\">Inductor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Basic.VariableConductor\">Variable conductor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Basic.VariableCapacitor\">Variable capacitor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Basic.VariableInductor\">Variable inductor</a>
</p>
</html>"));
          end VariableResistor;

          model VariableConductor "Singlephase variable conductor"
            extends Interfaces.OnePort;
            import Modelica.ComplexMath.real;
            import Modelica.ComplexMath.conj;
            parameter Modelica.SIunits.Temperature T_ref = 293.15 "Reference temperature";
            parameter Modelica.SIunits.LinearTemperatureCoefficient alpha_ref = 0 "Temperature coefficient of conductance (G_actual = G_ref/(1 + alpha_ref*(heatPort.T - T_ref))";
            extends Modelica.Electrical.Analog.Interfaces.ConditionalHeatPort(T = T_ref);
            Modelica.SIunits.Conductance G_actual "Conductance = G_ref/(1 + alpha_ref*(heatPort.T - T_ref))";
            Modelica.Blocks.Interfaces.RealInput G_ref(unit = "S") annotation(
              Placement(transformation(origin = {0, 110}, extent = {{-20, -20}, {20, 20}}, rotation = 270)));
          equation
            assert(1 + alpha_ref * (T_heatPort - T_ref) >= Modelica.Constants.eps, "Temperature outside scope of model!");
            G_actual = G_ref / (1 + alpha_ref * (T_heatPort - T_ref));
            i = G_actual * v;
            LossPower = real(v * conj(i));
            annotation(
              Icon(graphics = {Text(extent = {{100, -80}, {-100, -40}}, textString = "%name", lineColor = {0, 0, 255}), Line(points = {{60, 0}, {90, 0}}, color = {85, 170, 255}), Line(points = {{-90, 0}, {-60, 0}}, color = {85, 170, 255}), Rectangle(extent = {{-70, 30}, {70, -30}}, lineColor = {85, 170, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{0, 90}, {0, 30}}, color = {85, 170, 255}, smooth = Smooth.None)}),
              Documentation(info = "<html>

<p>
The linear conductor connects the voltage <i><u>v</u></i> with the
current <i><u>i</u></i> by <i><u>i</u> = G*<u>v</u></i>.
The conductance <i>G</i> is given as input signal.
</p>

<p>
The variable conductor model also has an optional
<a href=\"modelica://Modelica.Electrical.Analog.Interfaces.ConditionalHeatPort\">conditional heat port</a>.
A linear temperature dependency of the resistance for an enabled heat port is also taken into account.
</p>

<h4>See also</h4>
<p>
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Basic.Resistor\">Resistor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Basic.Conductor\">Conductor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Basic.Capacitor\">Capacitor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Basic.Inductor\">Inductor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Basic.VariableResistor\">Variable resistor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Basic.VariableCapacitor\">Variable capacitor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Basic.VariableInductor\">Variable inductor</a>
</p>
</html>"));
          end VariableConductor;

          model VariableCapacitor "Singlephase variable capacitor"
            extends Interfaces.OnePort;
            import Modelica.ComplexMath.j;
            Modelica.Blocks.Interfaces.RealInput C(unit = "F") annotation(
              Placement(transformation(origin = {0, 110}, extent = {{-20, -20}, {20, 20}}, rotation = 270)));
          equation
            i = j * omega * C * v;
            annotation(
              Icon(graphics = {Text(extent = {{100, -80}, {-100, -40}}, textString = "%name", lineColor = {0, 0, 255}), Line(points = {{-14, 28}, {-14, -28}}, color = {85, 170, 255}, thickness = 0.5), Line(points = {{14, 28}, {14, -28}}, color = {85, 170, 255}, thickness = 0.5), Line(points = {{-90, 0}, {-14, 0}}, color = {85, 170, 255}), Line(points = {{14, 0}, {90, 0}}, color = {85, 170, 255}), Line(points = {{0, 90}, {0, 30}}, color = {85, 170, 255})}),
              Documentation(info = "<html>

<p>
The linear capacitor connects the voltage <i><u>v</u></i> with the
current <i><u>i</u></i> by <i><u>i</u> = j*&omega;*C*<u>v</u></i>.
The capacitance <i>C</i> is given as input signal.
</p>

<h4>Note</h4>
<p>
The abstraction of a variable capacitor at quasi stationary operation assumes:
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Electrical/QuasiStationary/SinglePhase/Basic/dc_dt.png\"
     alt=\"dc_dt.png\">.
</p>

<h4>See also</h4>
<p>
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Basic.Resistor\">Resistor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Basic.Conductor\">Conductor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Basic.Capacitor\">Capacitor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Basic.Inductor\">Inductor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Basic.VariableResistor\">Variable resistor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Basic.VariableConductor\">Variable conductor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Basic.VariableInductor\">Variable inductor</a>
</p>
</html>"));
          end VariableCapacitor;

          model VariableInductor "Singlephase variable inductor"
            extends Interfaces.OnePort;
            import Modelica.ComplexMath.j;
            Modelica.Blocks.Interfaces.RealInput L(unit = "H") annotation(
              Placement(transformation(origin = {0, 108}, extent = {{-20, -20}, {20, 20}}, rotation = 270)));
          equation
            v = j * omega * L * i;
            annotation(
              Icon(graphics = {Text(extent = {{100, -80}, {-100, -40}}, textString = "%name", lineColor = {0, 0, 255}), Ellipse(extent = {{-60, -15}, {-30, 15}}, lineColor = {85, 170, 255}), Ellipse(extent = {{-30, -15}, {0, 15}}, lineColor = {85, 170, 255}), Ellipse(extent = {{0, -15}, {30, 15}}, lineColor = {85, 170, 255}), Ellipse(extent = {{30, -15}, {60, 15}}, lineColor = {85, 170, 255}), Rectangle(extent = {{-60, -30}, {60, 0}}, lineColor = {255, 255, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{60, 0}, {90, 0}}, color = {85, 170, 255}), Line(points = {{-90, 0}, {-60, 0}}, color = {85, 170, 255}), Line(points = {{0, 90}, {0, 8}}, color = {85, 170, 255})}),
              Documentation(info = "<html>

<p>
The linear inductor connects the branch voltage <i><u>v</u></i> with the
branch current <i><u>i</u></i> by <i><u>v</u> = j*&omega;*L*<u>i</u></i>. The inductance <i>L</i> is given as input signal.
</p>

<h4>Note</h4>
<p>
The abstraction of a variable inductor at quasi stationary operation assumes:
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Electrical/QuasiStationary/SinglePhase/Basic/dl_dt.png\"
     alt=\"dl_dt.png\">
</p>

<h4>See also</h4>
<p>
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Basic.Resistor\">Resistor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Basic.Conductor\">Conductor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Basic.Capacitor\">Capacitor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Basic.Inductor\">Inductor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Basic.VariableResistor\">Variable resistor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Basic.VariableConductor\">Variable conductor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Basic.VariableCapacitor\">Variable capacitor</a>
</p>
</html>"));
          end VariableInductor;
          annotation(
            Icon(graphics = {Line(origin = {10, 40}, points = {{-100, -40}, {-80, -40}}), Line(origin = {10, 40}, points = {{60, -40}, {80, -40}}), Rectangle(lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, extent = {{-70, -30}, {70, 30}})}, coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = true)),
            Documentation(info = "<html>
<p>This package hosts basic models for quasi stationary single phase circuits.
Quasi stationary theory for single phase circuits can be found in the
<a href=\"modelica://Modelica.Electrical.QuasiStationary.UsersGuide.References\">references</a>.
</p>
<h4>See also</h4>

<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Basic\">MultiPhase.Basic</a>

</html>"));
        end Basic;

        package Ideal "Ideal components for AC singlephase models"
          extends Modelica.Icons.Package;

          model Idle "Idle branch"
            extends Interfaces.OnePort;
          equation
            i = Complex(0);
            annotation(
              Icon(graphics = {Rectangle(extent = {{-60, 60}, {60, -60}}, lineColor = {85, 170, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{-90, 0}, {-41, 0}}, color = {85, 170, 255}), Line(points = {{91, 0}, {40, 0}}, color = {85, 170, 255}), Text(extent = {{-100, 100}, {100, 70}}, textString = "%name", lineColor = {0, 0, 255})}),
              Documentation(info = "<html>
<p>
This model is a simple idle branch considering the complex current <i><u>i</u></i> = 0.
</p>

<h4>See also</h4>
<p>
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Ideal.Short\">Short</a>
</p>
</html>"));
          end Idle;

          model Short "Short cut branch"
            extends Interfaces.OnePort;
          equation
            v = Complex(0);
            annotation(
              Icon(graphics = {Rectangle(extent = {{-60, 60}, {60, -60}}, lineColor = {85, 170, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{91, 0}, {-90, 0}}, color = {85, 170, 255}), Text(extent = {{-100, 100}, {100, 70}}, textString = "%name", lineColor = {0, 0, 255})}),
              Documentation(info = "<html>
<p>
This model is a simple short cut branch considering the complex voltage <i><u>v</u></i> = 0.
</p>

<h4>See also</h4>
<p>
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Ideal.Idle\">Idle</a>
</p>
</html>"));
          end Short;

          model IdealCommutingSwitch "Ideal commuting switch"
            import Modelica.ComplexMath.real;
            import Modelica.ComplexMath.conj;
            parameter Modelica.SIunits.Resistance Ron(final min = 0) = 1.E-5 "Closed switch resistance";
            parameter Modelica.SIunits.Conductance Goff(final min = 0) = 1.E-5 "Opened switch conductance";
            extends Modelica.Electrical.Analog.Interfaces.ConditionalHeatPort(final T = 293.15);
            Modelica.Electrical.QuasiStationary.SinglePhase.Interfaces.PositivePin p annotation(
              Placement(transformation(extent = {{-110, -10}, {-90, 10}}, rotation = 0)));
            Modelica.Electrical.QuasiStationary.SinglePhase.Interfaces.NegativePin n2 annotation(
              Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
            Modelica.Electrical.QuasiStationary.SinglePhase.Interfaces.NegativePin n1 annotation(
              Placement(transformation(extent = {{90, 40}, {110, 60}}, rotation = 0)));
            Modelica.Blocks.Interfaces.BooleanInput control "true => p--n2 connected, false => p--n1 connected" annotation(
              Placement(transformation(origin = {0, 80}, extent = {{-20, -20}, {20, 20}}, rotation = 270)));
          protected
            Complex s1(re(final unit = "1"), im(final unit = "1"));
            Complex s2(re(final unit = "1"), im(final unit = "1")) "Auxiliary variables";
            constant Modelica.SIunits.ComplexVoltage unitVoltage = Complex(1, 0) annotation(
              HideResult = true);
            constant Modelica.SIunits.ComplexCurrent unitCurrent = Complex(1, 0) annotation(
              HideResult = true);
          equation
            Connections.branch(p.reference, n1.reference);
            p.reference.gamma = n1.reference.gamma;
            Connections.branch(p.reference, n2.reference);
            p.reference.gamma = n2.reference.gamma;
            p.i + n2.i + n1.i = Complex(0, 0);
            p.v - n1.v = s1 * unitCurrent * (if control then 1 else Ron);
            n1.i = -s1 * unitVoltage * (if control then Goff else 1);
            p.v - n2.v = s2 * unitCurrent * (if control then Ron else 1);
            n2.i = -s2 * unitVoltage * (if control then 1 else Goff);
            LossPower = real(p.v * conj(p.i)) + real(n1.v * conj(n1.i)) + real(n2.v * conj(n2.i));
            annotation(
              Documentation(info = "<HTML>
<P>
The commuting switch has a positive pin p and two negative pins n1 and n2.
The switching behaviour is controlled
by the input signal control. If control is true, the pin p is connected
with the negative pin n2. Otherwise, the pin p is connected to the negative pin n1.
</P>
<P>
In order to prevent singularities during switching, the opened
switch has a (very low) conductance Goff
and the closed switch has a (very low) resistance Ron.
The limiting case is also allowed, i.e., the resistance Ron of the
closed switch could be exactly zero and the conductance Goff of the
open switch could be also exactly zero. Note, there are circuits,
where a description with zero Ron or zero Goff is not possible.
<br> <br>
<b>Please note:</b>
In case of useHeatPort=true the temperature dependence of the electrical
behavior is <b> not </b> modelled. The parameters are not temperature dependent.
</P>
<p>
<b>Use with care:</b>
This switch is only intended to be used for structural changes, not for fast switching sequences, due to the quasistationary formulation.
</p>
</HTML>"),
              Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-44, 4}, {-36, -4}}, lineColor = {85, 170, 255}), Text(extent = {{-80, 0}, {-60, 22}}, textString = "p", lineColor = {0, 0, 255}), Text(extent = {{60, 50}, {80, 72}}, textString = "n1", lineColor = {0, 0, 255}), Text(extent = {{60, 0}, {80, 22}}, textString = "n2", lineColor = {0, 0, 255}), Line(points = {{-90, 0}, {-44, 0}}, color = {85, 170, 255}), Line(points = {{-37, 2}, {40, 50}}, color = {85, 170, 255}), Line(points = {{40, 50}, {90, 50}}, color = {85, 170, 255}), Line(points = {{0, 90}, {0, 25}}, color = {0, 0, 255}), Line(points = {{40, 0}, {90, 0}}, color = {85, 170, 255}), Line(visible = useHeatPort, points = {{0, -100}, {0, 25}}, color = {127, 0, 0}, smooth = Smooth.None, pattern = LinePattern.Dot), Text(extent = {{-148, -22}, {152, -62}}, textString = "%name", lineColor = {0, 0, 255})}),
              Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-44, 4}, {-36, -4}}, lineColor = {0, 0, 255}), Line(points = {{-96, 0}, {-44, 0}}, color = {0, 0, 255}), Line(points = {{-37, 2}, {40, 50}}, color = {0, 0, 255}), Line(points = {{40, 50}, {96, 50}}, color = {0, 0, 255}), Line(points = {{0, 60}, {0, 25}}, color = {0, 0, 255}), Line(points = {{40, 0}, {96, 0}}, color = {0, 0, 255})}));
          end IdealCommutingSwitch;

          model IdealIntermediateSwitch "Ideal intermediate switch"
            import Modelica.ComplexMath.real;
            import Modelica.ComplexMath.conj;
            parameter Modelica.SIunits.Resistance Ron(final min = 0) = 1.E-5 "Closed switch resistance";
            parameter Modelica.SIunits.Conductance Goff(final min = 0) = 1.E-5 "Opened switch conductance";
            extends Modelica.Electrical.Analog.Interfaces.ConditionalHeatPort(final T = 293.15);
            Modelica.Electrical.QuasiStationary.SinglePhase.Interfaces.PositivePin p1 annotation(
              Placement(transformation(extent = {{-110, 40}, {-90, 60}}, rotation = 0)));
            Modelica.Electrical.QuasiStationary.SinglePhase.Interfaces.PositivePin p2 annotation(
              Placement(transformation(extent = {{-110, -10}, {-90, 10}}, rotation = 0)));
            Modelica.Electrical.QuasiStationary.SinglePhase.Interfaces.NegativePin n1 annotation(
              Placement(transformation(extent = {{90, 40}, {110, 60}}, rotation = 0)));
            Modelica.Electrical.QuasiStationary.SinglePhase.Interfaces.NegativePin n2 annotation(
              Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
            Modelica.Blocks.Interfaces.BooleanInput control "true => p1--n2, p2--n1 connected, otherwise p1--n1, p2--n2  connected" annotation(
              Placement(transformation(origin = {0, 80}, extent = {{-20, -20}, {20, 20}}, rotation = 270)));
          protected
            Complex s1(re(final unit = "1"), im(final unit = "1"));
            Complex s2(re(final unit = "1"), im(final unit = "1"));
            Complex s3(re(final unit = "1"), im(final unit = "1"));
            Complex s4(re(final unit = "1"), im(final unit = "1")) "Auxiliary variables";
            constant Modelica.SIunits.ComplexVoltage unitVoltage = Complex(1, 0) annotation(
              HideResult = true);
            constant Modelica.SIunits.ComplexCurrent unitCurrent = Complex(1, 0) annotation(
              HideResult = true);
          equation
            Connections.branch(p1.reference, n1.reference);
            p1.reference.gamma = n1.reference.gamma;
            Connections.branch(p2.reference, n2.reference);
            p2.reference.gamma = n2.reference.gamma;
            Connections.branch(n1.reference, n2.reference);
            n1.reference.gamma = n2.reference.gamma;
            p1.v - n1.v = s1 * unitCurrent * (if control then 1 else Ron);
            p2.v - n2.v = s2 * unitCurrent * (if control then 1 else Ron);
            p1.v - n2.v = s3 * unitCurrent * (if control then Ron else 1);
            p2.v - n1.v = s4 * unitCurrent * (if control then Ron else 1);
            p1.i = if control then s1 * unitVoltage * Goff + s3 * unitCurrent else s1 * unitCurrent + s3 * unitVoltage * Goff;
            p2.i = if control then s2 * unitVoltage * Goff + s4 * unitCurrent else s2 * unitCurrent + s4 * unitVoltage * Goff;
            n1.i = if control then (-s1 * unitVoltage * Goff) - s4 * unitCurrent else (-s1 * unitCurrent) - s4 * unitVoltage * Goff;
            n2.i = if control then (-s2 * unitVoltage * Goff) - s3 * unitCurrent else (-s2 * unitCurrent) - s3 * unitVoltage * Goff;
            LossPower = real(p1.v * conj(p1.i)) + real(p2.v * conj(p2.i)) + real(n1.v * conj(n1.i)) + real(n2.v * conj(n2.i));
            annotation(
              Documentation(info = "<HTML>
<P>
The intermediate switch has four switching contact pins p1, p2, n1, and n2.
The switching behaviour is controlled by the input signal control. If control
is true, the pin p1 is connected to pin n2, and the pin p2 is
connected to the pin n2. Otherwise, the pin p1 is connected to n1, and
p2 is connected to n2.
</P>

<p>
<IMG src=\"modelica://Modelica/Resources/Images/Electrical/QuasiStationary/SinglePhase/Ideal/IdealIntermediateSwitch1.png\" ALT=\"IdealIntermediateSwitch1\">
</p>

<P>
In order to prevent singularities during switching, the opened
switch has a (very low) conductance Goff
and the closed switch has a (very low) resistance Ron.
</P>

<p>
<IMG src=\"modelica://Modelica/Resources/Images/Electrical/QuasiStationary/SinglePhase/Ideal/IdealIntermediateSwitch2.png\" ALT=\"IdealIntermediateSwitch2\">
</p>

<P>
The limiting case is also allowed, i.e., the resistance Ron of the
closed switch could be exactly zero and the conductance Goff of the
open switch could be also exactly zero. Note, there are circuits,
where a description with zero Ron or zero Goff is not possible.
<br> <br>
<b>Please note:</b>
In case of useHeatPort=true the temperature dependence of the electrical
behavior is <b> not </b> modelled. The parameters are not temperature dependent.
</P>
<p>
<b>Use with care:</b>
This switch is only intended to be used for structural changes, not for fast switching sequences, due to the quasistationary formulation.
</p>
</HTML>"),
              Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-4, 30}, {4, 22}}, lineColor = {85, 170, 255}), Text(extent = {{-80, 50}, {-60, 72}}, textString = "p1", lineColor = {0, 0, 255}), Text(extent = {{-80, 0}, {-60, 22}}, textString = "p2", lineColor = {0, 0, 255}), Text(extent = {{60, 50}, {80, 72}}, textString = "n1", lineColor = {0, 0, 255}), Text(extent = {{60, 0}, {80, 22}}, textString = "n2", lineColor = {0, 0, 255}), Line(points = {{-90, 0}, {-40, 0}}, color = {85, 170, 255}), Line(points = {{-90, 50}, {-40, 50}}, color = {85, 170, 255}), Line(points = {{-40, 0}, {40, 50}}, color = {85, 170, 255}), Line(points = {{-40, 50}, {40, 0}}, color = {85, 170, 255}), Line(points = {{40, 50}, {90, 50}}, color = {85, 170, 255}), Line(points = {{0, 90}, {0, 25}}, color = {85, 170, 255}), Line(points = {{40, 0}, {90, 0}}, color = {85, 170, 255}), Text(extent = {{-151, -24}, {149, -64}}, textString = "%name", lineColor = {0, 0, 255})}),
              Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-4, 29}, {4, 21}}, lineColor = {0, 0, 255}), Line(points = {{-96, 0}, {-40, 0}}, color = {0, 0, 255}), Line(points = {{-96, 50}, {-40, 50}}, color = {0, 0, 255}), Line(points = {{-40, 0}, {40, 50}}, color = {0, 0, 255}), Line(points = {{-40, 50}, {40, 0}}, color = {0, 0, 255}), Line(points = {{40, 50}, {96, 50}}, color = {0, 0, 255}), Line(points = {{0, 60}, {0, 25}}, color = {0, 0, 255}), Line(points = {{40, 0}, {96, 0}}, color = {0, 0, 255})}));
          end IdealIntermediateSwitch;

          model IdealOpeningSwitch "Ideal electrical opener"
            import Modelica.ComplexMath.real;
            import Modelica.ComplexMath.conj;
            extends Modelica.Electrical.QuasiStationary.SinglePhase.Interfaces.OnePort;
            parameter Modelica.SIunits.Resistance Ron(final min = 0) = 1.E-5 "Closed switch resistance" annotation(
              Placement(transformation(extent = {{-56.6667, 10}, {-10, 56.6667}}, rotation = 0)));
            parameter Modelica.SIunits.Conductance Goff(final min = 0) = 1.E-5 "Opened switch conductance" annotation(
              Placement(transformation(extent = {{10, 10}, {56.6667, 56.6667}}, rotation = 0)));
            extends Modelica.Electrical.Analog.Interfaces.ConditionalHeatPort(final T = 293.15);
            Modelica.Blocks.Interfaces.BooleanInput control "true => switch open, false => p--n connected" annotation(
              Placement(transformation(origin = {0, 70}, extent = {{-20, -20}, {20, 20}}, rotation = 270)));
          protected
            Complex s(re(final unit = "1"), im(final unit = "1")) "Auxiliary variable";
            constant Modelica.SIunits.ComplexVoltage unitVoltage = Complex(1, 0) annotation(
              HideResult = true);
            constant Modelica.SIunits.ComplexCurrent unitCurrent = Complex(1, 0) annotation(
              HideResult = true);
          equation
            v = s * unitCurrent * (if control then 1 else Ron);
            i = s * unitVoltage * (if control then Goff else 1);
            LossPower = real(v * conj(i));
            annotation(
              Documentation(info = "<HTML>
<P>
The ideal opening switch has a positive pin p and a negative pin n.
The switching behaviour is controlled by the input signal control.
If control is true, pin p is not connected
with negative pin n. Otherwise, pin p is connected
with negative pin n.
</P>
<P>
In order to prevent singularities during switching, the opened
switch has a (very low) conductance Goff
and the closed switch has a (very low) resistance Ron.
The limiting case is also allowed, i.e., the resistance Ron of the
closed switch could be exactly zero and the conductance Goff of the
open switch could be also exactly zero. Note, there are circuits,
where a description with zero Ron or zero Goff is not possible.
<br> <br>
<b>Please note:</b>
In case of useHeatPort=true the temperature dependence of the electrical
behavior is <b> not </b> modelled. The parameters are not temperature dependent.
</P>
<p>
<b>Use with care:</b>
This switch is only intended to be used for structural changes, not for fast switching sequences, due to the quasistationary formulation.
</p>
</HTML>"),
              Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-44, 4}, {-36, -4}}, lineColor = {85, 170, 255}), Line(points = {{-90, 0}, {-44, 0}}, color = {85, 170, 255}), Line(points = {{-37, 2}, {40, 50}}, color = {85, 170, 255}), Line(points = {{40, 0}, {90, 0}}, color = {85, 170, 255}), Line(points = {{0, 51}, {0, 26}}, color = {85, 170, 255}), Line(points = {{40, 20}, {40, 0}}, color = {85, 170, 255}), Line(visible = useHeatPort, points = {{0, -100}, {0, 25}}, color = {127, 0, 0}, smooth = Smooth.None, pattern = LinePattern.Dot), Text(extent = {{-151, -21}, {149, -61}}, textString = "%name", lineColor = {0, 0, 255})}),
              Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-44, 4}, {-36, -4}}, lineColor = {0, 0, 255}), Line(points = {{-96, 0}, {-44, 0}}, color = {0, 0, 255}), Line(points = {{-37, 2}, {40, 50}}, color = {0, 0, 255}), Line(points = {{40, 0}, {96, 0}}, color = {0, 0, 255}), Text(extent = {{-100, -40}, {100, -79}}, textString = "%name", lineColor = {0, 0, 255}), Line(points = {{0, 51}, {0, 26}}, color = {0, 0, 255}), Line(points = {{40, 20}, {40, 0}}, color = {0, 0, 255})}));
          end IdealOpeningSwitch;

          model IdealClosingSwitch "Ideal electrical closer"
            import Modelica.ComplexMath.real;
            import Modelica.ComplexMath.conj;
            extends Modelica.Electrical.QuasiStationary.SinglePhase.Interfaces.OnePort;
            parameter Modelica.SIunits.Resistance Ron(final min = 0) = 1.E-5 "Closed switch resistance" annotation(
              Placement(transformation(extent = {{-56.6667, 10}, {-10, 56.6667}}, rotation = 0)));
            parameter Modelica.SIunits.Conductance Goff(final min = 0) = 1.E-5 "Opened switch conductance" annotation(
              Placement(transformation(extent = {{10, 10}, {56.6667, 56.6667}}, rotation = 0)));
            extends Modelica.Electrical.Analog.Interfaces.ConditionalHeatPort(final T = 293.15);
            Modelica.Blocks.Interfaces.BooleanInput control "true => p--n connected, false => switch open" annotation(
              Placement(transformation(origin = {0, 70}, extent = {{-20, -20}, {20, 20}}, rotation = 270)));
          protected
            Complex s(re(final unit = "1"), im(final unit = "1")) "Auxiliary variable";
            constant Modelica.SIunits.ComplexVoltage unitVoltage = Complex(1, 0) annotation(
              HideResult = true);
            constant Modelica.SIunits.ComplexCurrent unitCurrent = Complex(1, 0) annotation(
              HideResult = true);
          equation
            v = s * unitCurrent * (if control then Ron else 1);
            i = s * unitVoltage * (if control then 1 else Goff);
            LossPower = real(v * conj(i));
            annotation(
              Documentation(info = "<HTML>
<P>
The ideal closing switch has a positive pin p and a negative pin n.
The switching behaviour is controlled by input signal control.
If control is true, pin p is connected
with negative pin n. Otherwise, pin p is not connected
with negative pin n.
</P>
<P>
In order to prevent singularities during switching, the opened
switch has a (very low) conductance Goff
and the closed switch has a (very low) resistance Ron.
The limiting case is also allowed, i.e., the resistance Ron of the
closed switch could be exactly zero and the conductance Goff of the
open switch could be also exactly zero. Note, there are circuits,
where a description with zero Ron or zero Goff is not possible.
<br> <br>
<b>Please note:</b>
In case of useHeatPort=true the temperature dependence of the electrical
behavior is <b> not </b> modelled. The parameters are not temperature dependent.
</P>
<p>
<b>Use with care:</b>
This switch is only intended to be used for structural changes, not for fast switching sequences, due to the quasistationary formulation.
</p>
</HTML>"),
              Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-44, 4}, {-36, -4}}, lineColor = {85, 170, 255}), Line(points = {{-90, 0}, {-44, 0}}, color = {85, 170, 255}), Line(points = {{-37, 2}, {40, 50}}, color = {85, 170, 255}), Line(points = {{40, 0}, {90, 0}}, color = {85, 170, 255}), Line(points = {{0, 51}, {0, 26}}, color = {85, 170, 255}), Line(visible = useHeatPort, points = {{0, -100}, {0, 25}}, color = {127, 0, 0}, smooth = Smooth.None, pattern = LinePattern.Dot), Text(extent = {{-152, -28}, {148, -68}}, textString = "%name", lineColor = {0, 0, 255})}),
              Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-44, 4}, {-36, -4}}, lineColor = {0, 0, 255}), Line(points = {{-96, 0}, {-44, 0}}, color = {0, 0, 255}), Line(points = {{-37, 2}, {40, 50}}, color = {0, 0, 255}), Line(points = {{40, 0}, {96, 0}}, color = {0, 0, 255}), Text(extent = {{-100, -40}, {100, -79}}, textString = "%name", lineColor = {0, 0, 255}), Line(points = {{0, 51}, {0, 26}}, color = {0, 0, 255})}));
          end IdealClosingSwitch;
          annotation(
            Icon(coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = true), graphics = {Line(origin = {10, 34}, points = {{-100, -60}, {-54, -60}}), Ellipse(origin = {10, 34}, extent = {{-54, -64}, {-46, -56}}), Line(origin = {10, 34}, points = {{-47, -58}, {30, -10}}), Line(origin = {10, 34}, points = {{30, -40}, {30, -60}}), Line(origin = {10, 34}, points = {{30, -60}, {80, -60}})}),
            Documentation(info = "<html>
<p>This package hosts ideal models for quasi stationary single phase circuits.
Quasi stationary theory for single phase circuits can be found in the
<a href=\"modelica://Modelica.Electrical.QuasiStationary.UsersGuide.References\">references</a>.
</p>
<h4>See also</h4>

<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Ideal\">MultiPhase.Ideal</a>

</html>"));
        end Ideal;

        package Sensors "AC singlephase sensors"
          extends Modelica.Icons.SensorsPackage;

          model FrequencySensor "Frequency sensor"
            extends Interfaces.AbsoluteSensor;
            import Modelica.Constants.pi;
            Blocks.Interfaces.RealOutput y annotation(
              Placement(transformation(extent = {{100, -10}, {120, 10}}, rotation = 0)));
          equation
            2 * pi * y = omega;
            annotation(
              Icon(graphics = {Text(extent = {{-29, -11}, {30, -70}}, lineColor = {0, 0, 0}, textString = "f")}),
              Documentation(info = "<html>

<p>
This sensor can be used to measure the frequency of the reference system.
</p>

<h4>See also</h4>

<p>
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Sensors.PotentialSensor\">PotentialSensor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Sensors.VoltageSensor\">VoltageSensor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Sensors.CurrentSensor\">CurrentSensor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Sensors.PowerSensor\">PowerSensor</a>
</p>

</html>"));
          end FrequencySensor;

          model PotentialSensor "Potential sensor"
            extends Interfaces.AbsoluteSensor;
            ComplexBlocks.Interfaces.ComplexOutput y annotation(
              Placement(transformation(extent = {{100, -10}, {120, 10}}, rotation = 0)));
          equation
            y = pin.v;
            annotation(
              Icon(graphics = {Text(extent = {{-29, -11}, {30, -70}}, lineColor = {0, 0, 0}, textString = "V")}),
              Documentation(info = "<html>

<p>
This sensor can be used to measure the complex potential.
</p>

<h4>See also</h4>

<p>
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Sensors.VoltageSensor\">VoltageSensor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Sensors.CurrentSensor\">CurrentSensor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Sensors.PowerSensor\">PowerSensor</a>
</p>

</html>"));
          end PotentialSensor;

          model VoltageSensor "Voltage sensor"
            extends Interfaces.RelativeSensor;
          equation
            i = Complex(0);
            y = v;
            annotation(
              Icon(graphics = {Text(extent = {{-29, -11}, {30, -70}}, lineColor = {0, 0, 0}, textString = "V")}),
              Documentation(info = "<html>
<p>
This sensor can be used to measure the complex voltage.
</p>

<h4>See also</h4>

<p>
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Sensors.PotentialSensor\">PotentialSensor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Sensors.CurrentSensor\">CurrentSensor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Sensors.PowerSensor\">PowerSensor</a>
</p>

</html>"));
          end VoltageSensor;

          model CurrentSensor "Current sensor"
            extends Interfaces.RelativeSensor;
          equation
            v = Complex(0);
            y = i;
            annotation(
              Icon(graphics = {Text(extent = {{-29, -11}, {30, -70}}, lineColor = {0, 0, 0}, textString = "I")}),
              Documentation(info = "<html>
<p>
This sensor can be used to measure the complex current.
</p>

<h4>See also</h4>

<p>
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Sensors.PotentialSensor\">PotentialSensor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Sensors.VoltageSensor\">VoltageSensor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Sensors.PowerSensor\">PowerSensor</a>
</p>

</html>"));
          end CurrentSensor;

          model PowerSensor "Power sensor"
            import Modelica.ComplexMath.conj;
            extends Modelica.Icons.RotationalSensor;
            Interfaces.PositivePin currentP annotation(
              Placement(transformation(extent = {{-110, -10}, {-90, 10}}, rotation = 0)));
            Interfaces.NegativePin currentN annotation(
              Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
            Interfaces.PositivePin voltageP annotation(
              Placement(transformation(extent = {{-10, 90}, {10, 110}}, rotation = 0)));
            Interfaces.NegativePin voltageN annotation(
              Placement(transformation(extent = {{-10, -110}, {10, -90}}, rotation = 0)));
            output Modelica.SIunits.ComplexCurrent i;
            output Modelica.SIunits.ComplexVoltage v;
            Modelica.ComplexBlocks.Interfaces.ComplexOutput y annotation(
              Placement(transformation(origin = {-80, -110}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          equation
            Connections.branch(currentP.reference, currentN.reference);
            currentP.reference.gamma = currentN.reference.gamma;
            Connections.branch(voltageP.reference, voltageN.reference);
            voltageP.reference.gamma = voltageN.reference.gamma;
            Connections.branch(currentP.reference, voltageP.reference);
            currentP.reference.gamma = voltageP.reference.gamma;
            currentP.i + currentN.i = Complex(0);
            currentP.v - currentN.v = Complex(0);
            i = currentP.i;
            voltageP.i + voltageN.i = Complex(0);
            voltageP.i = Complex(0);
            v = voltageP.v - voltageN.v;
//P + j*Q = v * conj(i);
            y = v * conj(i);
            annotation(
              Icon(graphics = {Line(points = {{0, 100}, {0, 70}}, color = {0, 0, 255}), Line(points = {{0, -70}, {0, -100}}, color = {0, 0, 255}), Text(extent = {{-29, -70}, {30, -11}}, textString = "P"), Line(points = {{-80, -100}, {-80, 0}}, color = {85, 170, 255}), Text(textColor = {0, 0, 255}, extent = {{-100, 110}, {100, 150}}, textString = "%name"), Line(points = {{-100, 0}, {100, 0}}, color = {0, 0, 255})}),
              Documentation(info = "<html>

<p>
This sensor can be used to measure the complex apparent power.
</p>

<h4>See also</h4>

<p>
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Sensors.PotentialSensor\">PotentialSensor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Sensors.VoltageSensor\">VoltageSensor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Sensors.CurrentSensor\">CurrentSensor</a>,
</p>

</html>"));
          end PowerSensor;
          annotation(
            Documentation(info = "<html>
<p>This package hosts sensors for quasi stationary single phase circuits.
Quasi stationary theory for single phase circuits can be found in the
<a href=\"modelica://Modelica.Electrical.QuasiStationary.UsersGuide.References\">references</a>.
</p>
<h4>See also</h4>

<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Sensors\">MultiPhase.Sensors</a>

</html>"));
        end Sensors;

        package Sources "AC singlephase sources"
          extends Modelica.Icons.SourcesPackage;

          model VoltageSource "Constant AC voltage"
            extends Interfaces.Source;
            parameter Modelica.SIunits.Frequency f(start = 1) "frequency of the source";
            parameter Modelica.SIunits.Voltage V(start = 1) "RMS voltage of the source";
            parameter Modelica.SIunits.Angle phi(start = 0) "phase shift of the source";
          equation
            omega = 2 * Modelica.Constants.pi * f;
            v = Complex(V * cos(phi), V * sin(phi));
            annotation(
              Icon(graphics = {Text(extent = {{-120, 50}, {-20, 0}}, lineColor = {0, 0, 255}, textString = "+"), Text(extent = {{20, 50}, {120, 0}}, lineColor = {0, 0, 255}, textString = "-"), Line(points = {{-50, 0}, {50, 0}}, color = {0, 0, 0})}),
              Documentation(info = "<html>

<p>
This is a constant voltage source, specifying the complex voltage by the RMS voltage and the phase shift.
</p>

<h4>See also</h4>

<p>
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Sources.VariableVoltageSource\">VariableVoltageSource</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Sources.CurrentSource\">CurrentSource</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Sources.VariableCurrentSource\">VariableCurrentSource</a>
</p>
</html>"));
          end VoltageSource;

          model VariableVoltageSource "Variable AC voltage"
            extends Interfaces.Source;
            Modelica.Blocks.Interfaces.RealInput f annotation(
              Placement(transformation(origin = {40, 100}, extent = {{-20, -20}, {20, 20}}, rotation = 270)));
            Modelica.ComplexBlocks.Interfaces.ComplexInput V annotation(
              Placement(transformation(origin = {-40, 100}, extent = {{-20, -20}, {20, 20}}, rotation = 270)));
          equation
            omega = 2 * Modelica.Constants.pi * f;
            v = V;
            annotation(
              Icon(graphics = {Text(extent = {{-120, 50}, {-20, 0}}, lineColor = {0, 0, 255}, textString = "+"), Text(extent = {{20, 50}, {120, 0}}, lineColor = {0, 0, 255}, textString = "-"), Line(points = {{50, 0}, {-50, 0}}, color = {0, 0, 0})}),
              Documentation(info = "<html>

<p>
This is a voltage source with a complex signal input, specifying the complex voltage by the complex RMS voltage components.
Additionally, the frequency of the voltage source is defined by a real signal input.
</p>

<h4>See also</h4>

<p>
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Sources.VoltageSource\">VoltageSource</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Sources.CurrentSource\">CurrentSource</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Sources.VariableCurrentSource\">VariableCurrentSource</a>
</p>
</html>"));
          end VariableVoltageSource;

          model CurrentSource "Constant AC current"
            extends Interfaces.Source;
            parameter Modelica.SIunits.Frequency f(start = 1) "frequency of the source";
            parameter Modelica.SIunits.Current I(start = 1) "RMS current of the source";
            parameter Modelica.SIunits.Angle phi(start = 0) "phase shift of the source";
          equation
            omega = 2 * Modelica.Constants.pi * f;
            i = Complex(I * cos(phi), I * sin(phi));
            annotation(
              Icon(graphics = {Line(points = {{0, -50}, {0, 50}}, color = {0, 0, 0}), Line(points = {{-60, 60}, {60, 60}}, color = {0, 0, 255}), Polygon(points = {{60, 60}, {30, 70}, {30, 50}, {60, 60}}, lineColor = {0, 0, 255}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid)}),
              Documentation(info = "<html>

<p>
This is a constant current source, specifying the complex current by the RMS current and the phase shift.
</p>

<h4>See also</h4>

<p>
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Sources.VoltageSource\">VoltageSource</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Sources.VariableVoltageSource\">VariableVoltageSource</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Sources.VariableCurrentSource\">VariableCurrentSource</a>
</p>
</html>"));
          end CurrentSource;

          model VariableCurrentSource "Variable AC current"
            extends Interfaces.Source;
            Modelica.Blocks.Interfaces.RealInput f annotation(
              Placement(transformation(origin = {40, 100}, extent = {{-20, -20}, {20, 20}}, rotation = 270)));
            Modelica.ComplexBlocks.Interfaces.ComplexInput I annotation(
              Placement(transformation(origin = {-40, 100}, extent = {{-20, -20}, {20, 20}}, rotation = 270)));
          equation
            omega = 2 * Modelica.Constants.pi * f;
            i = I;
            annotation(
              Icon(graphics = {Line(points = {{0, -50}, {0, 50}}, color = {0, 0, 0}), Line(points = {{-60, 60}, {60, 60}}, color = {0, 0, 255}), Polygon(points = {{60, 60}, {30, 70}, {30, 50}, {60, 60}}, lineColor = {0, 0, 255}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid)}),
              Documentation(info = "<html>

<p>
This is a current source with a complex signal input, specifying the complex current by the complex RMS current components.
Additionally, the frequency of the voltage source is defined by a real signal input.
</p>

<h4>See also</h4>

<p>
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Sources.VoltageSource\">VoltageSource</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Sources.VariableVoltageSource\">VariableVoltageSource</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Sources.CurrentSource\">CurrentSource</a>,
</p>
</html>"));
          end VariableCurrentSource;
          annotation(
            Documentation(info = "<html>
<p>This package hosts sources for quasi stationary single phase circuits.
Quasi stationary theory for single phase circuits can be found in the
<a href=\"modelica://Modelica.Electrical.QuasiStationary.UsersGuide.References\">references</a>.
</p>
<h4>See also</h4>

<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Sources\">MultiPhase.Sources</a>

</html>"));
        end Sources;

        package Interfaces "Interfaces for AC singlephase models"
          extends Modelica.Icons.InterfacesPackage;

          connector Pin "Basic connector"
            Modelica.SIunits.ComplexVoltage v "Complex potential at the node";
            flow Modelica.SIunits.ComplexCurrent i "Complex current flowing into the pin";
            annotation(
              Documentation(info = "<html>
<p>
The potential of this connector is the complex voltage and the flow variable is the complex current.
The <a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Interfaces.PositivePin\">positive</a> and
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Interfaces.NegativePin\">negative pin</a> are
derived from this base connector.
</p>

<h4>See also</h4>

<p>
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Interfaces.PositivePin\">PositivePin</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Interfaces.NegativePin\">NegativePin</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Interfaces.Plug\">Plug</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Interfaces.PositivePlug\">PositivePlug</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Interfaces.NegativePlug\">NegativePlug</a>
</p>

</html>"));
          end Pin;

          connector PositivePin "Positive connector"
            extends Pin;
            QuasiStationary.Types.Reference reference "Reference";
            annotation(
              Diagram(graphics = {Text(extent = {{-100, 100}, {100, 60}}, lineColor = {0, 0, 255}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid, textString = "%name"), Rectangle(extent = {{-40, 40}, {40, -40}}, lineColor = {85, 170, 255}, fillColor = {85, 170, 255}, fillPattern = FillPattern.Solid)}),
              Icon(graphics = {Rectangle(extent = {{-100, 100}, {100, -100}}, lineColor = {85, 170, 255}, fillColor = {85, 170, 255}, fillPattern = FillPattern.Solid)}),
              Documentation(info = "<html>

<p>
The positive pin is based on <a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Interfaces.Pin\">Pin</a>.
Additionally the reference angle is specified in the connector. The time derivative of the reference angle is the actual angular velocity of the quasi stationary voltage and current. The symbol is also designed such way to look different than the <a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Interfaces.NegativePin\">negative pin</a>.
</p>

<h4>See also</h4>

<p>
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Interfaces.Pin\">Pin</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Interfaces.NegativePin\">NegativePin</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Interfaces.Plug\">Plug</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Interfaces.PositivePlug\">PositivePlug</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Interfaces.NegativePlug\">NegativePlug</a>
</p>
</html>"));
          end PositivePin;

          connector NegativePin "Negative Connector"
            extends Pin;
            QuasiStationary.Types.Reference reference "Reference";
            annotation(
              Diagram(graphics = {Text(extent = {{-100, 100}, {100, 60}}, lineColor = {0, 0, 255}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid, textString = "%name"), Rectangle(extent = {{-40, 40}, {40, -40}}, lineColor = {85, 170, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid)}),
              Icon(graphics = {Rectangle(extent = {{-100, 100}, {100, -100}}, lineColor = {85, 170, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid)}),
              Documentation(info = "<html>

<p>
The negative pin is based on <a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Interfaces.Pin\">Pin</a>.
Additionally the reference angle is specified in the connector. The time derivative of the reference angle is the actual angular velocity of the quasi stationary voltage and current. The symbol is also designed such way to look different than the <a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Interfaces.PositivePin\">positive pin</a>.
</p>

<h4>See also</h4>

<p>
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Interfaces.Pin\">Pin</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Interfaces.PositivePin\">PositivePin</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Interfaces.Plug\">Plug</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Interfaces.PositivePlug\">PositivePlug</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Interfaces.NegativePlug\">NegativePlug</a>
</p>
</html>"));
          end NegativePin;

          partial model TwoPin "Two pins"
            Modelica.SIunits.ComplexVoltage v;
            Modelica.SIunits.ComplexCurrent i;
            Modelica.SIunits.AngularVelocity omega;
            PositivePin pin_p "Positive pin" annotation(
              Placement(transformation(extent = {{-110, -10}, {-90, 10}}, rotation = 0)));
            NegativePin pin_n "Negative pin" annotation(
              Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
          equation
            Connections.branch(pin_p.reference, pin_n.reference);
            pin_p.reference.gamma = pin_n.reference.gamma;
            omega = der(pin_p.reference.gamma);
            v = pin_p.v - pin_n.v;
            i = pin_p.i;
            annotation(
              Documentation(info = "<html>
<p>
This partial model uses a <a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Interfaces.PositivePin\">positive</a>
and <a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Interfaces.NegativePin\">negative pin</a> and defines the complex voltage difference as well as the complex current (into the positive pin). Additionally, the angular velocity of the quasi stationary system is explicitly defined as variable. This model is mainly intended to be used with graphical representation of user models.
</p>

<h4>See also</h4>

<p>
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Interfaces.PositivePin\">PositivePin</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Interfaces.NegativePin\">NegativePin</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Interfaces.OnePort\">OnePort</a>
</p>
</html>"));
          end TwoPin;

          partial model OnePort "Two pins, current through"
            extends TwoPin;
          equation
            pin_p.i + pin_n.i = Complex(0);
            annotation(
              Documentation(info = "<html>
<p>
This partial model is based on <a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Interfaces.TwoPin\">TwoPin</a> and
additionally considers the complex current balance of the
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Interfaces.PositivePin\">positive</a> and the
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Interfaces.NegativePin\">negative pin</a>.
This model is intended to be used with textual representation of user models.
</p>

<h4>See also</h4>

<p>
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Interfaces.PositivePin\">PositivePin</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Interfaces.NegativePin\">NegativePin</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Interfaces.TwoPin\">TwoPin</a>
</p>
</html>"));
          end OnePort;

          partial model AbsoluteSensor "Partial potential sensor"
            extends Modelica.Icons.RotationalSensor;
            Modelica.SIunits.AngularVelocity omega;
            PositivePin pin "Pin" annotation(
              Placement(transformation(extent = {{-110, -10}, {-90, 10}}, rotation = 0)));
          equation
            omega = der(pin.reference.gamma);
            pin.i = Complex(0);
            annotation(
              Icon(graphics = {Line(points = {{-70, 0}, {-94, 0}}, color = {0, 0, 0}), Text(extent = {{-100, 100}, {100, 70}}, lineColor = {0, 0, 255}, fillColor = {170, 85, 255}, fillPattern = FillPattern.Solid, textString = "%name"), Line(points = {{100, 0}, {70, 0}}, color = {0, 0, 0})}),
              Documentation(info = "<html>
<p>
The absolute sensor partial model provides a single
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Interfaces.PositivePin\">positive pin</a> to measure the complex voltage. Additionally this model contains a proper icon and a definition of the angular velocity.
</p>

<h4>See also</h4>

<p>
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Interfaces.RelativeSensor\">RelativeSensor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Sensors.PotentialSensor\">PotentialSensor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Interfaces.AbsoluteSensor\">MultiPhase.Interfaces.AbsoluteSensor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Interfaces.RelativeSensor\">MultiPhase.Interfaces.RelativeSensor</a>
</p>

</html>"));
          end AbsoluteSensor;

          partial model RelativeSensor "Partial voltage / current sensor"
            extends Modelica.Icons.RotationalSensor;
            extends OnePort;
            Modelica.ComplexBlocks.Interfaces.ComplexOutput y annotation(
              Placement(transformation(origin = {0, -110}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
            annotation(
              Icon(graphics = {Line(points = {{-70, 0}, {-94, 0}}, color = {0, 0, 0}), Line(points = {{70, 0}, {94, 0}}, color = {0, 0, 0}), Text(extent = {{-100, 100}, {100, 70}}, lineColor = {0, 0, 255}, fillColor = {170, 85, 255}, fillPattern = FillPattern.Solid, textString = "%name"), Line(points = {{0, -70}, {0, -80}, {0, -90}, {0, -100}}, color = {85, 170, 255})}),
              Documentation(info = "<html>
<p>
The relative sensor partial model relies on the
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Interfaces.OnePort\">OnePort</a> to measure the complex voltage, current or power. Additionally this model contains a proper icon and a definition of the angular velocity.
</p>

<h4>See also</h4>

<p>
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Interfaces.AbsoluteSensor\">AbsoluteSensor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Sensors.VoltageSensor\">VoltageSensor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Sensors.CurrentSensor\">CurrentSensor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Sensors.PowerSensor\">PowerSensor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Interfaces.AbsoluteSensor\">MultiPhase.Interfaces.AbsoluteSensor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Interfaces.RelativeSensor\">MultiPhase.Interfaces.RelativeSensor</a>
</p>

</html>"));
          end RelativeSensor;

          partial model Source "Partial voltage / current source"
            extends OnePort;
          equation
            Connections.root(pin_p.reference);
            annotation(
              Icon(graphics = {Ellipse(extent = {{-50, 50}, {50, -50}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Text(extent = {{100, -100}, {-100, -60}}, textString = "%name", lineColor = {0, 0, 255}), Line(points = {{-90, 0}, {-50, 0}}, color = {0, 0, 0}), Line(points = {{50, 0}, {90, 0}}, color = {0, 0, 0})}),
              Documentation(info = "<html>
<p>
The source partial model relies on the
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Interfaces.OnePort\">OnePort</a> and contains a proper icon.
</p>

<h4>See also</h4>

<p>
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Sources.VoltageSource\">VoltageSource</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Sources.VariableVoltageSource\">VariableVoltageSource</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Sources.CurrentSource\">CurrentSource</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Sources.VariableCurrentSource\">VariableCurrentSource</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Interfaces.Source\">MultiPhase.Interfaces.Source</a>.
</p>
</html>"));
          end Source;
          annotation(
            Documentation(info = "<html>
<p>This package contains connector specifications and partial models for more complex components.</p>
</html>"));
        end Interfaces;

        package Utilities "Library with auxiliary models for testing"
          extends Modelica.Icons.UtilitiesPackage;

          model IdealACDCConverter "Ideal AC DC converter"
            parameter Real conversionFactor "Ratio of DC voltage / QS rms voltage";
            import Modelica.ComplexMath.real;
            import Modelica.ComplexMath.imag;
            import Modelica.ComplexMath.conj;
            import Modelica.ComplexMath.'abs';
            import Modelica.ComplexMath.arg;
            Modelica.SIunits.ComplexVoltage vQS = pin_pQS.v - pin_nQS.v "AC QS voltage";
            Modelica.SIunits.ComplexCurrent iQS = pin_pQS.i "AC QS current";
            output Modelica.SIunits.Voltage vQSabs = 'abs'(vQS) "Abs(AC QS voltage)";
            output Modelica.SIunits.Current iQSabs = 'abs'(iQS) "Abs(AC QS current)";
            Modelica.SIunits.ComplexPower sQS = vQS * conj(iQS) "AC QS apparent power";
            Modelica.SIunits.ActivePower pQS = real(sQS) "AC QS active power";
            Modelica.SIunits.ReactivePower qQS = imag(sQS) "AC QS reactive power";
            Modelica.SIunits.Voltage vDC = pin_pDC.v - pin_nDC.v "DC voltage";
            Modelica.SIunits.Current iDC = pin_pDC.i "DC current";
            Modelica.SIunits.Power pDC = vDC * iDC "DC power";
            Interfaces.PositivePin pin_pQS annotation(
              Placement(transformation(extent = {{-110, 110}, {-90, 90}}), iconTransformation(extent = {{-110, 110}, {-90, 90}})));
            Interfaces.NegativePin pin_nQS annotation(
              Placement(transformation(extent = {{-110, -110}, {-90, -90}}), iconTransformation(extent = {{-110, -110}, {-90, -90}})));
            Analog.Interfaces.PositivePin pin_pDC annotation(
              Placement(transformation(extent = {{90, 110}, {110, 90}}), iconTransformation(extent = {{90, 110}, {110, 90}})));
            Analog.Interfaces.NegativePin pin_nDC annotation(
              Placement(transformation(extent = {{90, -110}, {110, -90}}), iconTransformation(extent = {{90, -110}, {110, -90}})));
          equation
//QS balances
            Connections.branch(pin_pQS.reference, pin_nQS.reference);
            pin_pQS.reference.gamma = pin_nQS.reference.gamma;
            pin_pQS.i + pin_nQS.i = Complex(0);
//DC current balance
            pin_pDC.i + pin_nDC.i = 0;
//voltage relation
            vDC = 'abs'(vQS) * conversionFactor;
//power balance
            pQS + pDC = 0;
//define reactive power
            qQS = 0;
            annotation(
              Icon(graphics = {Line(points = {{2, 100}, {2, 60}, {82, 60}, {2, 60}, {82, -60}, {2, -60}, {2, 60}, {2, -100}}, color = {0, 0, 255}, smooth = Smooth.None), Text(extent = {{40, 40}, {100, 0}}, lineColor = {0, 0, 255}, textString = "DC"), Line(points = {{-2, 100}, {-2, 60}, {-82, 60}, {-2, 60}, {-82, -60}, {-2, -60}, {-2, 60}, {-2, -100}}, color = {85, 170, 255}, smooth = Smooth.None), Text(extent = {{-100, 40}, {-40, 0}}, lineColor = {85, 170, 255}, textString = "QS"), Text(extent = {{-100, 92}, {100, 60}}, lineColor = {0, 0, 255}, textString = "%name"), Text(extent = {{-100, -60}, {100, -92}}, lineColor = {0, 0, 255}, textString = "%conversionFactor")}),
              Documentation(info = "<html>
<p>
This is an ideal AC DC converter, based on a power balance between QS circuit and DC side.
The parameter <i>conversionFactor</i> defines the ratio between averaged DC voltage and QS rms voltage.
Furthermore, reactive power at the QS side is set to 0.
</p>
<h4>Note:</h4>
<p>
Of course no voltage or current ripple is present, neither at the QS side nor at the DC side.
At the QS side, only base harmonics of voltage and current are taken into account.
At the DC side, only the mean of voltage and current are taken into account.
</p>
</html>"));
          end IdealACDCConverter;

          model GraetzRectifier "Graetz rectifier bridge"
            Modelica.SIunits.Voltage vAC = pin_pAC.v - pin_nAC.v "AC voltage";
            Modelica.SIunits.Current iAC = pin_pAC.i "AC current";
            Modelica.SIunits.ActivePower pAC = vAC * iAC "AC power";
            Modelica.SIunits.Voltage vDC = pin_pDC.v - pin_nDC.v "DC voltage";
            Modelica.SIunits.Current iDC = pin_pDC.i "DC current";
            Modelica.SIunits.Power pDC = vDC * iDC "DC power";
            Analog.Interfaces.PositivePin pin_pAC annotation(
              Placement(transformation(extent = {{-110, 110}, {-90, 90}}), iconTransformation(extent = {{-110, 110}, {-90, 90}})));
            Analog.Interfaces.NegativePin pin_nAC annotation(
              Placement(transformation(extent = {{-110, -110}, {-90, -90}}), iconTransformation(extent = {{-110, -110}, {-90, -90}})));
            Analog.Interfaces.PositivePin pin_pDC annotation(
              Placement(transformation(extent = {{90, 110}, {110, 90}}), iconTransformation(extent = {{90, 110}, {110, 90}})));
            Analog.Interfaces.NegativePin pin_nDC annotation(
              Placement(transformation(extent = {{90, -110}, {110, -90}}), iconTransformation(extent = {{90, -110}, {110, -90}})));
            Analog.Ideal.IdealDiode idealDiode1(Vknee = 0) annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 90, origin = {-40, 30})));
            Analog.Ideal.IdealDiode idealDiode2(Vknee = 0) annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 90, origin = {40, 30})));
            Analog.Ideal.IdealDiode idealDiode3(Vknee = 0) annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 90, origin = {-40, -30})));
            Analog.Ideal.IdealDiode idealDiode4(Vknee = 0) annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 90, origin = {40, -30})));
          equation
            connect(idealDiode1.p, idealDiode3.n) annotation(
              Line(points = {{-40, 20}, {-40, -20}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(idealDiode2.p, idealDiode4.n) annotation(
              Line(points = {{40, 20}, {40, -20}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(idealDiode3.p, idealDiode4.p) annotation(
              Line(points = {{-40, -40}, {40, -40}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(idealDiode1.n, idealDiode2.n) annotation(
              Line(points = {{-40, 40}, {40, 40}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(pin_pAC, idealDiode1.p) annotation(
              Line(points = {{-100, 100}, {-100, 10}, {-40, 10}, {-40, 20}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(pin_nAC, idealDiode4.n) annotation(
              Line(points = {{-100, -100}, {-100, -10}, {40, -10}, {40, -20}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(idealDiode2.n, pin_pDC) annotation(
              Line(points = {{40, 40}, {40, 100}, {100, 100}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(idealDiode4.p, pin_nDC) annotation(
              Line(points = {{40, -40}, {40, -100}, {100, -100}}, color = {0, 0, 255}, smooth = Smooth.None));
            annotation(
              Icon(graphics = {Line(points = {{2, 100}, {2, 60}, {82, 60}, {2, 60}, {82, -60}, {2, -60}, {2, 60}, {2, -100}}, color = {0, 0, 255}, smooth = Smooth.None), Text(extent = {{40, 40}, {100, 0}}, lineColor = {0, 0, 255}, textString = "DC"), Line(points = {{-2, 100}, {-2, 60}, {-82, 60}, {-2, 60}, {-82, -60}, {-2, -60}, {-2, 60}, {-2, -100}}, color = {0, 0, 255}, smooth = Smooth.None), Text(extent = {{-100, 40}, {-40, 0}}, lineColor = {0, 0, 255}, textString = "AC"), Text(extent = {{-100, 92}, {100, 60}}, lineColor = {0, 0, 255}, textString = "%name")}),
              Documentation(info = "<html>
<p>
This is a so called Graetz-bridge, a single phase rectifier built from 4 diodes.
</p>
</html>"));
          end GraetzRectifier;
          annotation(
            Documentation(info = "<html>
<p>This package hosts utilities for test examples of quasi stationary single phase circuits.
Quasi stationary theory for single phase circuits can be found in the
<a href=\"modelica://Modelica.Electrical.QuasiStationary.UsersGuide.References\">references</a>.
</p>
<h4>See also</h4>

<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Examples\">Examples</a>

</html>"));
        end Utilities;
        annotation(
          Icon(graphics = {Rectangle(lineColor = {0, 0, 255}, extent = {{-50, -50}, {50, 50}}), Rectangle(fillColor = {170, 213, 255}, fillPattern = FillPattern.Solid, extent = {{-20, -20}, {20, 20}})}, coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = true)),
          Documentation(info = "<html>
<p>This package hosts models for quasi stationary single phase circuits.
Quasi stationary theory for single phase circuits can be found in the
<a href=\"modelica://Modelica.Electrical.QuasiStationary.UsersGuide.References\">references</a>.
</p>
<h4>See also</h4>

<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase\">MultiPhase</a>

</html>"));
      end SinglePhase;

      package Machines "Quasistationary machine models"
        extends Modelica.Icons.Package;

        package Examples "Test examples"
          extends Modelica.Icons.ExamplesPackage;

          model TransformerTestbench "Transformer Testbench"
            extends Modelica.Icons.Example;
            parameter Integer m = 3 "Number of phases";
            parameter Modelica.SIunits.Resistance RL[m] = fill(1 / 3, m) "Load resistance";
            QuasiStationary.MultiPhase.Sources.VoltageSource source(f = 50, V = fill(100 / sqrt(3), 3), m = m) annotation(
              Placement(transformation(origin = {-90, -10}, extent = {{-10, 10}, {10, -10}}, rotation = 270)));
            QuasiStationary.MultiPhase.Basic.Star starS(m = m) annotation(
              Placement(transformation(origin = {-90, -40}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
            QuasiStationary.SinglePhase.Basic.Ground groundS annotation(
              Placement(transformation(extent = {{-100, -80}, {-80, -60}}, rotation = 0)));
            QuasiStationary.MultiPhase.Sensors.PowerSensor electricalPowerSensorS(m = m) annotation(
              Placement(transformation(extent = {{-90, 0}, {-70, 20}}, rotation = 0)));
            QuasiStationary.MultiPhase.Sensors.CurrentSensor currentSensorS(m = m) annotation(
              Placement(transformation(extent = {{-60, 20}, {-40, 0}}, rotation = 0)));
            ComplexBlocks.ComplexMath.ComplexToPolar polarIS[m] annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 90, origin = {-50, 40})));
            QuasiStationary.MultiPhase.Sensors.VoltageSensor voltageSensorS(m = m) annotation(
              Placement(transformation(origin = {-50, -30}, extent = {{-10, 10}, {10, -10}}, rotation = 180)));
            ComplexBlocks.ComplexMath.ComplexToPolar polarVS[m] annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 270, origin = {-50, -60})));
            QuasiStationary.MultiPhase.Basic.Delta deltaS(m = m) annotation(
              Placement(transformation(origin = {-50, -10}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
            QuasiStationary.SinglePhase.Basic.Resistor earth(R_ref = 1e6) annotation(
              Placement(transformation(origin = {0, -40}, extent = {{-10, 10}, {10, -10}}, rotation = 270)));
            QuasiStationary.SinglePhase.Basic.Ground groundT annotation(
              Placement(transformation(extent = {{-10, -80}, {10, -60}}, rotation = 0)));
            QuasiStationary.MultiPhase.Sensors.VoltageSensor voltageSensorL(m = m) annotation(
              Placement(transformation(origin = {50, -30}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
            ComplexBlocks.ComplexMath.ComplexToPolar polarVL[m] annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 270, origin = {50, -60})));
            QuasiStationary.MultiPhase.Basic.Delta deltaL(m = m) annotation(
              Placement(transformation(origin = {50, -10}, extent = {{-10, 10}, {10, -10}}, rotation = 180)));
            QuasiStationary.MultiPhase.Sensors.CurrentSensor currentSensorL(m = m) annotation(
              Placement(transformation(extent = {{40, 20}, {60, 0}}, rotation = 0)));
            ComplexBlocks.ComplexMath.ComplexToPolar polarIL[m] annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 90, origin = {50, 40})));
            QuasiStationary.MultiPhase.Sensors.PowerSensor electricalPowerSensorL(m = m) annotation(
              Placement(transformation(extent = {{70, 0}, {90, 20}}, rotation = 0)));
            QuasiStationary.MultiPhase.Basic.Resistor load(R_ref = RL, m = m) annotation(
              Placement(transformation(origin = {90, -10}, extent = {{-10, 10}, {10, -10}}, rotation = 270)));
            QuasiStationary.MultiPhase.Basic.Star starL(m = m) annotation(
              Placement(transformation(origin = {90, -40}, extent = {{-10, 10}, {10, -10}}, rotation = 270)));
            QuasiStationary.SinglePhase.Basic.Ground groundL annotation(
              Placement(transformation(extent = {{80, -80}, {100, -60}}, rotation = 0)));
            Modelica.Electrical.Machines.Utilities.TransformerData transformerData(C1 = Modelica.Utilities.Strings.substring(transformer.VectorGroup, 1, 1), C2 = Modelica.Utilities.Strings.substring(transformer.VectorGroup, 2, 2), f = 50, V1 = 100, V2 = 100, SNominal = 30E3, v_sc = 0.05, P_sc = 300) annotation(
              Placement(transformation(extent = {{-10, 40}, {10, 60}}, rotation = 0)));
            QuasiStationary.Machines.BasicMachines.Transformers.Yd.Yd01 transformer(n = transformerData.n, R1 = transformerData.R1, L1sigma = transformerData.L1sigma, R2 = transformerData.R2, L2sigma = transformerData.L2sigma, T1Ref = 293.15, alpha20_1(displayUnit = "1/K") = Modelica.Electrical.Machines.Thermal.Constants.alpha20Zero, T2Ref = 293.15, alpha20_2(displayUnit = "1/K") = Modelica.Electrical.Machines.Thermal.Constants.alpha20Zero, T1Operational = 293.15, T2Operational = 293.15) annotation(
              Placement(transformation(extent = {{-20, -10}, {20, 30}}, rotation = 0)));
          initial equation
            source.voltageSource.pin_p.reference.gamma = zeros(m);
          equation
            connect(starS.pin_n, groundS.pin) annotation(
              Line(points = {{-90, -50}, {-90, -60}}, color = {85, 170, 255}, smooth = Smooth.None));
            connect(source.plug_n, starS.plug_p) annotation(
              Line(points = {{-90, -20}, {-90, -30}}, color = {85, 170, 255}, smooth = Smooth.None));
            connect(starL.pin_n, groundL.pin) annotation(
              Line(points = {{90, -50}, {90, -60}}, color = {85, 170, 255}, smooth = Smooth.None));
            connect(load.plug_n, starL.plug_p) annotation(
              Line(points = {{90, -20}, {90, -30}}, color = {85, 170, 255}, smooth = Smooth.None));
            connect(source.plug_p, electricalPowerSensorS.currentP) annotation(
              Line(points = {{-90, 0}, {-90, 10}}, color = {85, 170, 255}, smooth = Smooth.None));
            connect(electricalPowerSensorS.currentP, electricalPowerSensorS.voltageP) annotation(
              Line(points = {{-90, 10}, {-90, 20}, {-80, 20}}, color = {85, 170, 255}, smooth = Smooth.None));
            connect(electricalPowerSensorS.currentN, currentSensorS.plug_p) annotation(
              Line(points = {{-70, 10}, {-60, 10}}, color = {85, 170, 255}, smooth = Smooth.None));
            connect(currentSensorS.plug_n, transformer.plug1) annotation(
              Line(points = {{-40, 10}, {-20, 10}}, color = {85, 170, 255}, smooth = Smooth.None));
            connect(transformer.plug2, currentSensorL.plug_p) annotation(
              Line(points = {{20, 10}, {40, 10}}, color = {85, 170, 255}, smooth = Smooth.None));
            connect(currentSensorL.plug_n, electricalPowerSensorL.currentP) annotation(
              Line(points = {{60, 10}, {70, 10}}, color = {85, 170, 255}, smooth = Smooth.None));
            connect(electricalPowerSensorL.currentP, electricalPowerSensorL.voltageP) annotation(
              Line(points = {{70, 10}, {70, 20}, {80, 20}}, color = {85, 170, 255}, smooth = Smooth.None));
            connect(electricalPowerSensorL.currentN, load.plug_p) annotation(
              Line(points = {{90, 10}, {90, 0}}, color = {85, 170, 255}, smooth = Smooth.None));
            connect(electricalPowerSensorL.voltageN, starL.plug_p) annotation(
              Line(points = {{80, 0}, {80, -30}, {90, -30}}, color = {85, 170, 255}, smooth = Smooth.None));
            connect(electricalPowerSensorS.voltageN, starS.plug_p) annotation(
              Line(points = {{-80, 0}, {-80, -30}, {-90, -30}}, color = {85, 170, 255}, smooth = Smooth.None));
            connect(earth.pin_n, groundT.pin) annotation(
              Line(points = {{0, -50}, {0, -60}}, color = {85, 170, 255}, smooth = Smooth.None));
            connect(deltaS.plug_p, voltageSensorS.plug_n) annotation(
              Line(points = {{-60, -10}, {-60, -30}}, color = {85, 170, 255}, smooth = Smooth.None));
            connect(deltaS.plug_n, voltageSensorS.plug_p) annotation(
              Line(points = {{-40, -10}, {-40, -30}}, color = {85, 170, 255}, smooth = Smooth.None));
            connect(currentSensorS.plug_n, deltaS.plug_n) annotation(
              Line(points = {{-40, 10}, {-40, -10}}, color = {85, 170, 255}, smooth = Smooth.None));
            connect(deltaL.plug_p, voltageSensorL.plug_n) annotation(
              Line(points = {{60, -10}, {60, -30}}, color = {85, 170, 255}, smooth = Smooth.None));
            connect(deltaL.plug_n, voltageSensorL.plug_p) annotation(
              Line(points = {{40, -10}, {40, -30}}, color = {85, 170, 255}, smooth = Smooth.None));
            connect(currentSensorL.plug_p, deltaL.plug_n) annotation(
              Line(points = {{40, 10}, {40, -10}}, color = {85, 170, 255}, smooth = Smooth.None));
            connect(polarIS.u, currentSensorS.y) annotation(
              Line(points = {{-50, 28}, {-50, 21}}, color = {85, 170, 255}, smooth = Smooth.None));
            connect(polarIL.u, currentSensorL.y) annotation(
              Line(points = {{50, 28}, {50, 21}}, color = {85, 170, 255}, smooth = Smooth.None));
            connect(voltageSensorS.y, polarVS.u) annotation(
              Line(points = {{-50, -41}, {-50, -48}}, color = {85, 170, 255}, smooth = Smooth.None));
            connect(voltageSensorL.y, polarVL.u) annotation(
              Line(points = {{50, -41}, {50, -48}}, color = {85, 170, 255}, smooth = Smooth.None));
            annotation(
              Documentation(info = "<HTML>
Transformer testbench:<br>
You may choose different connections as well as vary the load (even not symmetrical).<br>
<b>Please pay attention</b> to proper grounding of the primary and secondary part of the whole circuit.<br>
The primary and secondary starpoint are available as connectors, if the connection is not delta (D or d).<br>
In some cases it may be necessary to ground the transformer's starpoint
even though the source's or load's starpoint are grounded; you may use a reasonable high earthing resistance.
</HTML>"),
              experiment(StopTime = 0.1, Interval = 0.001));
          end TransformerTestbench;
          annotation(
            Documentation(info = "<html>
Examples to demonstrate the usage of quasistationary electric components.
</html>"));
        end Examples;

        package BasicMachines "Basic machine models"
          extends Modelica.Icons.Package;

          package Transformers "Library for technical 3phase transformers"
            extends Modelica.Icons.Package;

            package Yy "Transformers: primary Y / secondary y"
              extends Modelica.Icons.VariantsPackage;

              model Yy00 "Transformer Yy0"
                extends QuasiStationary.Machines.Interfaces.PartialBasicTransformer(final VectorGroup = "Yy00");
                QuasiStationary.MultiPhase.Basic.Star star1(final m = m) annotation(
                  Placement(transformation(origin = {-10, -80}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
                QuasiStationary.SinglePhase.Interfaces.NegativePin starpoint1 annotation(
                  Placement(transformation(extent = {{-60, -110}, {-40, -90}}, rotation = 0)));
                QuasiStationary.MultiPhase.Basic.Star star2(final m = m) annotation(
                  Placement(transformation(origin = {10, -80}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
                QuasiStationary.SinglePhase.Interfaces.NegativePin starpoint2 annotation(
                  Placement(transformation(extent = {{40, -110}, {60, -90}}, rotation = 0)));
              equation
                connect(star1.pin_n, starpoint1) annotation(
                  Line(points = {{-10, -90}, {-10, -100}, {-50, -100}}, color = {85, 170, 255}));
                connect(star2.pin_n, starpoint2) annotation(
                  Line(points = {{10, -90}, {10, -100}, {50, -100}}, color = {85, 170, 255}));
                connect(l1sigma.plug_n, core.plug_p1) annotation(
                  Line(points = {{-50, 0}, {-50, 20}, {-10, 20}, {-10, 5}}, color = {85, 170, 255}));
                connect(core.plug_n1, star1.plug_p) annotation(
                  Line(points = {{-10, -5}, {-10, -37.5}, {-10, -70}, {-10, -70}}, color = {85, 170, 255}));
                connect(core.plug_n2, core.plug_p3) annotation(
                  Line(points = {{10, 4}, {10, -4}}, color = {85, 170, 255}));
                connect(core.plug_p2, l2sigma.plug_p) annotation(
                  Line(points = {{10, 10}, {10, 20}, {50, 20}, {50, 0}}, color = {85, 170, 255}));
                connect(core.plug_n3, star2.plug_p) annotation(
                  Line(points = {{10, -10}, {10, -70}, {10, -70}}, color = {85, 170, 255}));
                annotation(
                  defaultComponentName = "transformer",
                  Documentation(info = "<html>
Transformer Yy0
<br>Typical parameters see:
<a href=\"modelica://Modelica.Electrical.Machines.Interfaces.PartialBasicTransformer\">PartialBasicTransformer</a>
</html>"));
              end Yy00;

              model Yy02 "Transformer Yy2"
                extends QuasiStationary.Machines.Interfaces.PartialBasicTransformer(final VectorGroup = "Yy02");
                QuasiStationary.MultiPhase.Basic.Star star1(final m = m) annotation(
                  Placement(transformation(origin = {-10, -80}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
                QuasiStationary.SinglePhase.Interfaces.NegativePin starpoint1 annotation(
                  Placement(transformation(extent = {{-60, -110}, {-40, -90}}, rotation = 0)));
                QuasiStationary.MultiPhase.Basic.Star star2(final m = m) annotation(
                  Placement(transformation(origin = {10, -80}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
                QuasiStationary.SinglePhase.Interfaces.NegativePin starpoint2 annotation(
                  Placement(transformation(extent = {{40, -110}, {60, -90}}, rotation = 0)));
                QuasiStationary.MultiPhase.Basic.Delta Rot2(final m = m) annotation(
                  Placement(transformation(extent = {{50, -30}, {30, -10}}, rotation = 0)));
              equation
                connect(star1.pin_n, starpoint1) annotation(
                  Line(points = {{-10, -90}, {-10, -100}, {-50, -100}}, color = {85, 170, 255}));
                connect(star2.pin_n, starpoint2) annotation(
                  Line(points = {{10, -90}, {10, -100}, {50, -100}}, color = {85, 170, 255}));
                connect(l2sigma.plug_p, Rot2.plug_p) annotation(
                  Line(points = {{50, 0}, {50, -20}}, color = {85, 170, 255}));
                connect(l1sigma.plug_n, core.plug_p1) annotation(
                  Line(points = {{-50, 0}, {-50, 20}, {-10, 20}, {-10, 5}}, color = {85, 170, 255}));
                connect(core.plug_n2, core.plug_p3) annotation(
                  Line(points = {{10, 4}, {10, -4}, {10, -4}}, color = {85, 170, 255}));
                connect(core.plug_n1, star1.plug_p) annotation(
                  Line(points = {{-10, -5}, {-10, -37.5}, {-10, -70}, {-10, -70}}, color = {85, 170, 255}));
                connect(core.plug_n3, Rot2.plug_n) annotation(
                  Line(points = {{10, -10}, {10, -20}, {30, -20}}, color = {85, 170, 255}));
                connect(core.plug_p2, star2.plug_p) annotation(
                  Line(points = {{10, 10}, {20, 10}, {20, -70}, {10, -70}}, color = {85, 170, 255}));
                annotation(
                  defaultComponentName = "transformer",
                  Documentation(info = "<html>
Transformer Yy2
<br>Typical parameters see:
<a href=\"modelica://Modelica.Electrical.Machines.Interfaces.PartialBasicTransformer\">PartialBasicTransformer</a>
</html>"));
              end Yy02;

              model Yy04 "Transformer Yy4"
                extends QuasiStationary.Machines.Interfaces.PartialBasicTransformer(final VectorGroup = "Yy04");
                QuasiStationary.MultiPhase.Basic.Star star1(final m = m) annotation(
                  Placement(transformation(origin = {-10, -80}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
                QuasiStationary.SinglePhase.Interfaces.NegativePin starpoint1 annotation(
                  Placement(transformation(extent = {{-60, -110}, {-40, -90}}, rotation = 0)));
                QuasiStationary.MultiPhase.Basic.Star star2(final m = m) annotation(
                  Placement(transformation(origin = {10, -80}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
                QuasiStationary.SinglePhase.Interfaces.NegativePin starpoint2 annotation(
                  Placement(transformation(extent = {{40, -110}, {60, -90}}, rotation = 0)));
                QuasiStationary.MultiPhase.Basic.Delta Rot2(final m = m) annotation(
                  Placement(transformation(extent = {{30, 10}, {50, 30}}, rotation = 0)));
              equation
                connect(star1.pin_n, starpoint1) annotation(
                  Line(points = {{-10, -90}, {-10, -100}, {-50, -100}}, color = {85, 170, 255}));
                connect(star2.pin_n, starpoint2) annotation(
                  Line(points = {{10, -90}, {10, -100}, {50, -100}}, color = {85, 170, 255}));
                connect(Rot2.plug_n, l2sigma.plug_p) annotation(
                  Line(points = {{50, 20}, {50, 0}}, color = {85, 170, 255}));
                connect(l1sigma.plug_n, core.plug_p1) annotation(
                  Line(points = {{-50, 0}, {-50, 20}, {-10, 20}, {-10, 5}}, color = {85, 170, 255}));
                connect(core.plug_n1, star1.plug_p) annotation(
                  Line(points = {{-10, -5}, {-10, -37.5}, {-10, -70}, {-10, -70}}, color = {85, 170, 255}));
                connect(core.plug_n2, core.plug_p3) annotation(
                  Line(points = {{10, 4}, {10, -4}}, color = {85, 170, 255}));
                connect(core.plug_p2, Rot2.plug_p) annotation(
                  Line(points = {{10, 10}, {10, 20}, {30, 20}}, color = {85, 170, 255}));
                connect(core.plug_n3, star2.plug_p) annotation(
                  Line(points = {{10, -10}, {10, -70}, {10, -70}}, color = {85, 170, 255}));
                annotation(
                  defaultComponentName = "transformer",
                  Documentation(info = "<html>
Transformer Yy4
<br>Typical parameters see:
<a href=\"modelica://Modelica.Electrical.Machines.Interfaces.PartialBasicTransformer\">PartialBasicTransformer</a>
</html>"));
              end Yy04;

              model Yy06 "Transformer Yy6"
                extends QuasiStationary.Machines.Interfaces.PartialBasicTransformer(final VectorGroup = "Yy06");
                QuasiStationary.MultiPhase.Basic.Star star1(final m = m) annotation(
                  Placement(transformation(origin = {-10, -80}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
                QuasiStationary.SinglePhase.Interfaces.NegativePin starpoint1 annotation(
                  Placement(transformation(extent = {{-60, -110}, {-40, -90}}, rotation = 0)));
                QuasiStationary.MultiPhase.Basic.Star star2(final m = m) annotation(
                  Placement(transformation(origin = {10, -80}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
                QuasiStationary.SinglePhase.Interfaces.NegativePin starpoint2 annotation(
                  Placement(transformation(extent = {{40, -110}, {60, -90}}, rotation = 0)));
              equation
                connect(star1.pin_n, starpoint1) annotation(
                  Line(points = {{-10, -90}, {-10, -100}, {-50, -100}}, color = {85, 170, 255}));
                connect(star2.pin_n, starpoint2) annotation(
                  Line(points = {{10, -90}, {10, -100}, {50, -100}}, color = {85, 170, 255}));
                connect(l1sigma.plug_n, core.plug_p1) annotation(
                  Line(points = {{-50, 0}, {-50, 20}, {-10, 20}, {-10, 5}}, color = {85, 170, 255}));
                connect(core.plug_n1, star1.plug_p) annotation(
                  Line(points = {{-10, -5}, {-10, -37.5}, {-10, -70}, {-10, -70}}, color = {85, 170, 255}));
                connect(core.plug_n2, core.plug_p3) annotation(
                  Line(points = {{10, 4}, {10, -4}}, color = {85, 170, 255}));
                connect(l2sigma.plug_p, core.plug_n3) annotation(
                  Line(points = {{50, 0}, {50, -20}, {10, -20}, {10, -10}}, color = {85, 170, 255}));
                connect(core.plug_p2, star2.plug_p) annotation(
                  Line(points = {{10, 10}, {20, 10}, {20, -70}, {10, -70}}, color = {85, 170, 255}));
                annotation(
                  defaultComponentName = "transformer",
                  Documentation(info = "<html>
Transformer Yy6
<br>Typical parameters see:
<a href=\"modelica://Modelica.Electrical.Machines.Interfaces.PartialBasicTransformer\">PartialBasicTransformer</a>
</html>"));
              end Yy06;

              model Yy08 "Transformer Yy8"
                extends QuasiStationary.Machines.Interfaces.PartialBasicTransformer(final VectorGroup = "Yy08");
                QuasiStationary.MultiPhase.Basic.Star star1(final m = m) annotation(
                  Placement(transformation(origin = {-10, -80}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
                QuasiStationary.SinglePhase.Interfaces.NegativePin starpoint1 annotation(
                  Placement(transformation(extent = {{-60, -110}, {-40, -90}}, rotation = 0)));
                QuasiStationary.MultiPhase.Basic.Star star2(final m = m) annotation(
                  Placement(transformation(origin = {10, -80}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
                QuasiStationary.SinglePhase.Interfaces.NegativePin starpoint2 annotation(
                  Placement(transformation(extent = {{40, -110}, {60, -90}}, rotation = 0)));
                QuasiStationary.MultiPhase.Basic.Delta Rot2(final m = m) annotation(
                  Placement(transformation(extent = {{50, 10}, {30, 30}}, rotation = 0)));
              equation
                connect(star1.pin_n, starpoint1) annotation(
                  Line(points = {{-10, -90}, {-10, -100}, {-50, -100}}, color = {85, 170, 255}));
                connect(star2.pin_n, starpoint2) annotation(
                  Line(points = {{10, -90}, {10, -100}, {50, -100}}, color = {85, 170, 255}));
                connect(Rot2.plug_p, l2sigma.plug_p) annotation(
                  Line(points = {{50, 20}, {50, 0}}, color = {85, 170, 255}));
                connect(l1sigma.plug_n, core.plug_p1) annotation(
                  Line(points = {{-50, 0}, {-50, 20}, {-10, 20}, {-10, 5}}, color = {85, 170, 255}));
                connect(core.plug_n1, star1.plug_p) annotation(
                  Line(points = {{-10, -5}, {-10, -37.5}, {-10, -70}, {-10, -70}}, color = {85, 170, 255}));
                connect(core.plug_n2, core.plug_p3) annotation(
                  Line(points = {{10, 4}, {10, -4}}, color = {85, 170, 255}));
                connect(core.plug_p2, Rot2.plug_n) annotation(
                  Line(points = {{10, 10}, {10, 20}, {30, 20}}, color = {85, 170, 255}));
                connect(core.plug_n3, star2.plug_p) annotation(
                  Line(points = {{10, -10}, {10, -70}, {10, -70}, {10, -70}}, color = {85, 170, 255}));
                annotation(
                  defaultComponentName = "transformer",
                  Documentation(info = "<html>
Transformer Yy8
<br>Typical parameters see:
<a href=\"modelica://Modelica.Electrical.Machines.Interfaces.PartialBasicTransformer\">PartialBasicTransformer</a>
</html>"));
              end Yy08;

              model Yy10 "Transformer Yy10"
                extends QuasiStationary.Machines.Interfaces.PartialBasicTransformer(final VectorGroup = "Yy10");
                QuasiStationary.MultiPhase.Basic.Star star1(final m = m) annotation(
                  Placement(transformation(origin = {-10, -80}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
                QuasiStationary.SinglePhase.Interfaces.NegativePin starpoint1 annotation(
                  Placement(transformation(extent = {{-60, -110}, {-40, -90}}, rotation = 0)));
                QuasiStationary.MultiPhase.Basic.Star star2(final m = m) annotation(
                  Placement(transformation(origin = {10, -80}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
                QuasiStationary.SinglePhase.Interfaces.NegativePin starpoint2 annotation(
                  Placement(transformation(extent = {{40, -110}, {60, -90}}, rotation = 0)));
                QuasiStationary.MultiPhase.Basic.Delta Rot2(final m = m) annotation(
                  Placement(transformation(extent = {{30, -30}, {50, -10}}, rotation = 0)));
              equation
                connect(star1.pin_n, starpoint1) annotation(
                  Line(points = {{-10, -90}, {-10, -100}, {-50, -100}}, color = {85, 170, 255}));
                connect(star2.pin_n, starpoint2) annotation(
                  Line(points = {{10, -90}, {10, -100}, {50, -100}}, color = {85, 170, 255}));
                connect(l2sigma.plug_p, Rot2.plug_n) annotation(
                  Line(points = {{50, 0}, {50, -20}}, color = {85, 170, 255}));
                connect(l1sigma.plug_n, core.plug_p1) annotation(
                  Line(points = {{-50, 0}, {-50, 20}, {-10, 20}, {-10, 5}}, color = {85, 170, 255}));
                connect(core.plug_n1, star1.plug_p) annotation(
                  Line(points = {{-10, -5}, {-10, -37.5}, {-10, -70}, {-10, -70}}, color = {85, 170, 255}));
                connect(core.plug_n2, core.plug_p3) annotation(
                  Line(points = {{10, 4}, {10, -4}}, color = {85, 170, 255}));
                connect(core.plug_n3, Rot2.plug_p) annotation(
                  Line(points = {{10, -10}, {10, -20}, {30, -20}}, color = {85, 170, 255}));
                connect(core.plug_p2, star2.plug_p) annotation(
                  Line(points = {{10, 10}, {20, 10}, {20, -70}, {10, -70}}, color = {85, 170, 255}));
                annotation(
                  defaultComponentName = "transformer",
                  Documentation(info = "<html>
Transformer Yy10
<br>Typical parameters see:
<a href=\"modelica://Modelica.Electrical.Machines.Interfaces.PartialBasicTransformer\">PartialBasicTransformer</a>
</html>"));
              end Yy10;
              annotation(
                Documentation(info = "<HTML>
This package contains transformers primary Y connected / secondary y connected in all possible vector groups.
</HTML>", revisions = "<HTML>
<dl>
  <dt><b>Main Authors:</b></dt>
  <dd>
  <a href=\"http://www.haumer.at/\">Anton Haumer</a><br>
  Technical Consulting &amp; Electrical Engineering<br>
  A-3423 St.Andrae-Woerdern<br>Austria<br>
  email: <a href=\"mailto:a.haumer@haumer.at\">a.haumer@haumer.at</a>
  </dd>
  <dt><b>Copyright:</b></dt>
  <dd>Copyright &copy; 1998-2015, Modelica Association and Anton Haumer.<br>
  <i>The Modelica package is <b>free</b> software; it can be redistributed and/or modified
  under the terms of the <b>Modelica license</b>, see the license conditions
  and the accompanying <b>disclaimer</b> in the documentation of package
  Modelica in file \"Modelica/package.mo\".</i></dd>
</dl>
  <ul>
  <li> v1.0.0 2006/11/19 Anton Haumer</li>
  </ul>
</HTML>"));
            end Yy;

            package Yd "Transformers: primary Y / secondary d"
              extends Modelica.Icons.VariantsPackage;

              model Yd01 "Transformer Yd1"
                extends QuasiStationary.Machines.Interfaces.PartialBasicTransformer(final VectorGroup = "Yd01");
                QuasiStationary.MultiPhase.Basic.Star star1(final m = m) annotation(
                  Placement(transformation(origin = {-10, -80}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
                QuasiStationary.SinglePhase.Interfaces.NegativePin starpoint1 annotation(
                  Placement(transformation(extent = {{-60, -110}, {-40, -90}}, rotation = 0)));
                QuasiStationary.MultiPhase.Basic.Delta Delta2(final m = m) annotation(
                  Placement(transformation(extent = {{50, -30}, {30, -10}}, rotation = 0)));
              equation
                connect(star1.pin_n, starpoint1) annotation(
                  Line(points = {{-10, -90}, {-10, -100}, {-50, -100}}, color = {85, 170, 255}));
                connect(Delta2.plug_p, r2.plug_n) annotation(
                  Line(points = {{50, -20}, {90, -20}, {90, 0}}, color = {85, 170, 255}));
                connect(l1sigma.plug_n, core.plug_p1) annotation(
                  Line(points = {{-50, 0}, {-50, 20}, {-10, 20}, {-10, 5}}, color = {85, 170, 255}));
                connect(core.plug_n1, star1.plug_p) annotation(
                  Line(points = {{-10, -5}, {-10, -70}, {-10, -70}}, color = {85, 170, 255}));
                connect(core.plug_n2, core.plug_p3) annotation(
                  Line(points = {{10, 4}, {10, -4}}, color = {85, 170, 255}));
                connect(core.plug_n3, Delta2.plug_n) annotation(
                  Line(points = {{10, -10}, {10, -20}, {30, -20}}, color = {85, 170, 255}));
                connect(core.plug_p2, l2sigma.plug_p) annotation(
                  Line(points = {{10, 10}, {10, 20}, {50, 20}, {50, 0}}, color = {85, 170, 255}));
                annotation(
                  defaultComponentName = "transformer",
                  Documentation(info = "<html>
Transformer Yd1
<br>Typical parameters see:
<a href=\"modelica://Modelica.Electrical.Machines.Interfaces.PartialBasicTransformer\">PartialBasicTransformer</a>
</html>"));
              end Yd01;

              model Yd03 "Transformer Yd3"
                extends QuasiStationary.Machines.Interfaces.PartialBasicTransformer(final VectorGroup = "Yd03");
                QuasiStationary.MultiPhase.Basic.Star star1(final m = m) annotation(
                  Placement(transformation(origin = {-10, -80}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
                QuasiStationary.SinglePhase.Interfaces.NegativePin starpoint1 annotation(
                  Placement(transformation(extent = {{-60, -110}, {-40, -90}}, rotation = 0)));
                QuasiStationary.MultiPhase.Basic.Delta Delta2(final m = m) annotation(
                  Placement(transformation(extent = {{30, 10}, {50, 30}}, rotation = 0)));
                QuasiStationary.MultiPhase.Basic.Delta Rot2(final m = m) annotation(
                  Placement(transformation(extent = {{50, -30}, {30, -10}}, rotation = 0)));
              equation
                connect(star1.pin_n, starpoint1) annotation(
                  Line(points = {{-10, -90}, {-10, -100}, {-50, -100}}, color = {85, 170, 255}));
                connect(Delta2.plug_n, r2.plug_n) annotation(
                  Line(points = {{50, 20}, {90, 20}, {90, 0}}, color = {85, 170, 255}));
                connect(l2sigma.plug_p, Rot2.plug_p) annotation(
                  Line(points = {{50, 0}, {50, -20}}, color = {85, 170, 255}));
                connect(l1sigma.plug_n, core.plug_p1) annotation(
                  Line(points = {{-50, 0}, {-50, 20}, {-10, 20}, {-10, 5}}, color = {85, 170, 255}));
                connect(core.plug_n1, star1.plug_p) annotation(
                  Line(points = {{-10, -5}, {-10, -70}, {-10, -70}}, color = {85, 170, 255}));
                connect(core.plug_n2, core.plug_p3) annotation(
                  Line(points = {{10, 4}, {10, -4}}, color = {85, 170, 255}));
                connect(core.plug_p2, Delta2.plug_p) annotation(
                  Line(points = {{10, 10}, {10, 20}, {30, 20}}, color = {85, 170, 255}));
                connect(core.plug_n3, Rot2.plug_n) annotation(
                  Line(points = {{10, -10}, {10, -20}, {30, -20}}, color = {85, 170, 255}));
                annotation(
                  defaultComponentName = "transformer",
                  Documentation(info = "<html>
Transformer Yd3
<br>Typical parameters see:
<a href=\"modelica://Modelica.Electrical.Machines.Interfaces.PartialBasicTransformer\">PartialBasicTransformer</a>
</html>"));
              end Yd03;

              model Yd05 "Transformer Yd5"
                extends QuasiStationary.Machines.Interfaces.PartialBasicTransformer(final VectorGroup = "Yd05");
                QuasiStationary.MultiPhase.Basic.Star star1(final m = m) annotation(
                  Placement(transformation(origin = {-10, -80}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
                QuasiStationary.SinglePhase.Interfaces.NegativePin starpoint1 annotation(
                  Placement(transformation(extent = {{-60, -110}, {-40, -90}}, rotation = 0)));
                QuasiStationary.MultiPhase.Basic.Delta Delta2(final m = m) annotation(
                  Placement(transformation(extent = {{30, 10}, {50, 30}}, rotation = 0)));
              equation
                connect(star1.pin_n, starpoint1) annotation(
                  Line(points = {{-10, -90}, {-10, -100}, {-50, -100}}, color = {85, 170, 255}));
                connect(Delta2.plug_n, r2.plug_n) annotation(
                  Line(points = {{50, 20}, {90, 20}, {90, 0}}, color = {85, 170, 255}));
                connect(l1sigma.plug_n, core.plug_p1) annotation(
                  Line(points = {{-50, 0}, {-50, 20}, {-10, 20}, {-10, 5}}, color = {85, 170, 255}));
                connect(core.plug_n1, star1.plug_p) annotation(
                  Line(points = {{-10, -5}, {-10, -70}, {-10, -70}}, color = {85, 170, 255}));
                connect(core.plug_n2, core.plug_p3) annotation(
                  Line(points = {{10, 4}, {10, -4}}, color = {85, 170, 255}));
                connect(core.plug_n3, l2sigma.plug_p) annotation(
                  Line(points = {{10, -10}, {10, -20}, {50, -20}, {50, 0}}, color = {85, 170, 255}));
                connect(core.plug_p2, Delta2.plug_p) annotation(
                  Line(points = {{10, 10}, {10, 20}, {30, 20}}, color = {85, 170, 255}));
                annotation(
                  defaultComponentName = "transformer",
                  Documentation(info = "<html>
Transformer Yd5
<br>Typical parameters see:
<a href=\"modelica://Modelica.Electrical.Machines.Interfaces.PartialBasicTransformer\">PartialBasicTransformer</a>
</html>"));
              end Yd05;

              model Yd07 "Transformer Yd7"
                extends QuasiStationary.Machines.Interfaces.PartialBasicTransformer(final VectorGroup = "Yd07");
                QuasiStationary.MultiPhase.Basic.Star star1(final m = m) annotation(
                  Placement(transformation(origin = {-10, -80}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
                QuasiStationary.SinglePhase.Interfaces.NegativePin starpoint1 annotation(
                  Placement(transformation(extent = {{-60, -110}, {-40, -90}}, rotation = 0)));
                QuasiStationary.MultiPhase.Basic.Delta Delta2(final m = m) annotation(
                  Placement(transformation(extent = {{50, 10}, {30, 30}}, rotation = 0)));
              equation
                connect(star1.pin_n, starpoint1) annotation(
                  Line(points = {{-10, -90}, {-10, -100}, {-50, -100}}, color = {85, 170, 255}));
                connect(Delta2.plug_p, r2.plug_n) annotation(
                  Line(points = {{50, 20}, {90, 20}, {90, 0}}, color = {85, 170, 255}));
                connect(l1sigma.plug_n, core.plug_p1) annotation(
                  Line(points = {{-50, 0}, {-50, 20}, {-10, 20}, {-10, 5}}, color = {85, 170, 255}));
                connect(core.plug_n1, star1.plug_p) annotation(
                  Line(points = {{-10, -5}, {-10, -37.5}, {-10, -70}, {-10, -70}}, color = {85, 170, 255}));
                connect(core.plug_n2, core.plug_p3) annotation(
                  Line(points = {{10, 4}, {10, -4}}, color = {85, 170, 255}));
                connect(core.plug_p2, Delta2.plug_n) annotation(
                  Line(points = {{10, 10}, {10, 20}, {30, 20}}, color = {85, 170, 255}));
                connect(l2sigma.plug_p, core.plug_n3) annotation(
                  Line(points = {{50, 0}, {50, -20}, {10, -20}, {10, -10}}, color = {85, 170, 255}));
                annotation(
                  defaultComponentName = "transformer",
                  Documentation(info = "<html>
Transformer Yd7
<br>Typical parameters see:
<a href=\"modelica://Modelica.Electrical.Machines.Interfaces.PartialBasicTransformer\">PartialBasicTransformer</a>
</html>"));
              end Yd07;

              model Yd09 "Transformer Yd9"
                extends QuasiStationary.Machines.Interfaces.PartialBasicTransformer(final VectorGroup = "Yd09");
                QuasiStationary.MultiPhase.Basic.Star star1(final m = m) annotation(
                  Placement(transformation(origin = {-10, -80}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
                QuasiStationary.SinglePhase.Interfaces.NegativePin starpoint1 annotation(
                  Placement(transformation(extent = {{-60, -110}, {-40, -90}}, rotation = 0)));
                QuasiStationary.MultiPhase.Basic.Delta Delta2(final m = m) annotation(
                  Placement(transformation(extent = {{30, -30}, {50, -10}}, rotation = 0)));
                QuasiStationary.MultiPhase.Basic.Delta Rot2(final m = m) annotation(
                  Placement(transformation(extent = {{50, 10}, {30, 30}}, rotation = 0)));
              equation
                connect(star1.pin_n, starpoint1) annotation(
                  Line(points = {{-10, -90}, {-10, -100}, {-50, -100}}, color = {85, 170, 255}));
                connect(Rot2.plug_p, l2sigma.plug_p) annotation(
                  Line(points = {{50, 20}, {50, 0}}, color = {85, 170, 255}));
                connect(Delta2.plug_n, r2.plug_n) annotation(
                  Line(points = {{50, -20}, {90, -20}, {90, 0}}, color = {85, 170, 255}));
                connect(l1sigma.plug_n, core.plug_p1) annotation(
                  Line(points = {{-50, 0}, {-50, 20}, {-10, 20}, {-10, 5}}, color = {85, 170, 255}));
                connect(core.plug_n1, star1.plug_p) annotation(
                  Line(points = {{-10, -5}, {-10, -37.5}, {-10, -70}, {-10, -70}}, color = {85, 170, 255}));
                connect(core.plug_n2, core.plug_p3) annotation(
                  Line(points = {{10, 4}, {10, -4}}, color = {85, 170, 255}));
                connect(core.plug_p2, Rot2.plug_n) annotation(
                  Line(points = {{10, 10}, {10, 20}, {30, 20}}, color = {85, 170, 255}));
                connect(core.plug_n3, Delta2.plug_p) annotation(
                  Line(points = {{10, -10}, {10, -20}, {30, -20}}, color = {85, 170, 255}));
                annotation(
                  defaultComponentName = "transformer",
                  Documentation(info = "<html>
Transformer Yd9
<br>Typical parameters see:
<a href=\"modelica://Modelica.Electrical.Machines.Interfaces.PartialBasicTransformer\">PartialBasicTransformer</a>
</html>"));
              end Yd09;

              model Yd11 "Transformer Yd11"
                extends QuasiStationary.Machines.Interfaces.PartialBasicTransformer(final VectorGroup = "Yd11");
                QuasiStationary.MultiPhase.Basic.Star star1(final m = m) annotation(
                  Placement(transformation(origin = {-10, -80}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
                QuasiStationary.SinglePhase.Interfaces.NegativePin starpoint1 annotation(
                  Placement(transformation(extent = {{-60, -110}, {-40, -90}}, rotation = 0)));
                QuasiStationary.MultiPhase.Basic.Delta Delta2(final m = m) annotation(
                  Placement(transformation(extent = {{30, -30}, {50, -10}}, rotation = 0)));
              equation
                connect(star1.pin_n, starpoint1) annotation(
                  Line(points = {{-10, -90}, {-10, -100}, {-50, -100}}, color = {85, 170, 255}));
                connect(Delta2.plug_n, r2.plug_n) annotation(
                  Line(points = {{50, -20}, {90, -20}, {90, 0}}, color = {85, 170, 255}));
                connect(l1sigma.plug_n, core.plug_p1) annotation(
                  Line(points = {{-50, 0}, {-50, 20}, {-10, 20}, {-10, 5}}, color = {85, 170, 255}));
                connect(core.plug_n1, star1.plug_p) annotation(
                  Line(points = {{-10, -5}, {-10, -70}, {-10, -70}}, color = {85, 170, 255}));
                connect(core.plug_n2, core.plug_p3) annotation(
                  Line(points = {{10, 4}, {10, -4}}, color = {85, 170, 255}));
                connect(core.plug_n3, Delta2.plug_p) annotation(
                  Line(points = {{10, -10}, {10, -20}, {30, -20}}, color = {85, 170, 255}));
                connect(core.plug_p2, l2sigma.plug_p) annotation(
                  Line(points = {{10, 10}, {10, 20}, {50, 20}, {50, 0}}, color = {85, 170, 255}));
                annotation(
                  defaultComponentName = "transformer",
                  Documentation(info = "<html>
Transformer Yd11
<br>Typical parameters see:
<a href=\"modelica://Modelica.Electrical.Machines.Interfaces.PartialBasicTransformer\">PartialBasicTransformer</a>
</html>"));
              end Yd11;
              annotation(
                Documentation(info = "<HTML>
This package contains transformers primary Y connected / secondary d connected in all possible vector groups.
</HTML>", revisions = "<HTML>
<dl>
  <dt><b>Main Authors:</b></dt>
  <dd>
  <a href=\"http://www.haumer.at/\">Anton Haumer</a><br>
  Technical Consulting &amp; Electrical Engineering<br>
  A-3423 St.Andrae-Woerdern<br>Austria<br>
  email: <a href=\"mailto:a.haumer@haumer.at\">a.haumer@haumer.at</a>
  </dd>
  <dt><b>Copyright:</b></dt>
  <dd>Copyright &copy; 1998-2015, Modelica Association and Anton Haumer.<br>
  <i>The Modelica package is <b>free</b> software; it can be redistributed and/or modified
  under the terms of the <b>Modelica license</b>, see the license conditions
  and the accompanying <b>disclaimer</b> in the documentation of package
  Modelica in file \"Modelica/package.mo\".</i></dd>
</dl>
  <ul>
  <li> v1.0.0 2006/11/19 Anton Haumer</li>
  </ul>
</HTML>"));
            end Yd;

            package Yz "Transformers: primary Y / secondary zig-zag"
              extends Modelica.Icons.VariantsPackage;

              model Yz01 "Transformer Yz1"
                extends QuasiStationary.Machines.Interfaces.PartialBasicTransformer(final VectorGroup = "Yz01");
                QuasiStationary.MultiPhase.Basic.Star star1(final m = m) annotation(
                  Placement(transformation(origin = {-10, -80}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
                QuasiStationary.SinglePhase.Interfaces.NegativePin starpoint1 annotation(
                  Placement(transformation(extent = {{-60, -110}, {-40, -90}}, rotation = 0)));
                QuasiStationary.MultiPhase.Basic.Star star2(final m = m) annotation(
                  Placement(transformation(origin = {10, -80}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
                QuasiStationary.SinglePhase.Interfaces.NegativePin starpoint2 annotation(
                  Placement(transformation(extent = {{40, -110}, {60, -90}}, rotation = 0)));
                QuasiStationary.MultiPhase.Basic.Delta Rot21(final m = m) annotation(
                  Placement(transformation(origin = {30, 0}, extent = {{-10, 10}, {10, -10}}, rotation = 90)));
                QuasiStationary.MultiPhase.Basic.Delta Rot22(final m = m) annotation(
                  Placement(transformation(origin = {40, 20}, extent = {{-10, 10}, {10, -10}}, rotation = 180)));
              equation
                connect(star1.pin_n, starpoint1) annotation(
                  Line(points = {{-10, -90}, {-10, -100}, {-50, -100}}, color = {85, 170, 255}));
                connect(star2.pin_n, starpoint2) annotation(
                  Line(points = {{10, -90}, {10, -100}, {50, -100}}, color = {85, 170, 255}));
                connect(l1sigma.plug_n, core.plug_p1) annotation(
                  Line(points = {{-50, 0}, {-50, 20}, {-10, 20}, {-10, 5}}, color = {85, 170, 255}));
                connect(core.plug_n1, star1.plug_p) annotation(
                  Line(points = {{-10, -5}, {-10, -70}, {-10, -70}}, color = {85, 170, 255}));
                connect(core.plug_p2, Rot21.plug_n) annotation(
                  Line(points = {{10, 10}, {30, 10}}, color = {85, 170, 255}));
                connect(core.plug_p3, Rot21.plug_p) annotation(
                  Line(points = {{10, -4}, {20, -4}, {20, -10}, {30, -10}}, color = {85, 170, 255}));
                connect(l2sigma.plug_p, Rot22.plug_p) annotation(
                  Line(points = {{50, 0}, {50, 20}}, color = {85, 170, 255}));
                connect(Rot22.plug_n, core.plug_n2) annotation(
                  Line(points = {{30, 20}, {20, 20}, {20, 4}, {10, 4}}, color = {85, 170, 255}));
                connect(core.plug_n3, star2.plug_p) annotation(
                  Line(points = {{10, -10}, {10, -70}, {10, -70}}, color = {85, 170, 255}));
                annotation(
                  defaultComponentName = "transformer",
                  Documentation(info = "<html>
Transformer Yz1
<br>Typical parameters see:
<a href=\"modelica://Modelica.Electrical.Machines.Interfaces.PartialBasicTransformer\">PartialBasicTransformer</a>
</html>"));
              end Yz01;

              model Yz03 "Transformer Yz3"
                extends QuasiStationary.Machines.Interfaces.PartialBasicTransformer(final VectorGroup = "Yz03");
                QuasiStationary.MultiPhase.Basic.Star star1(final m = m) annotation(
                  Placement(transformation(origin = {-10, -80}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
                QuasiStationary.SinglePhase.Interfaces.NegativePin starpoint1 annotation(
                  Placement(transformation(extent = {{-60, -110}, {-40, -90}}, rotation = 0)));
                QuasiStationary.MultiPhase.Basic.Star star2(final m = m) annotation(
                  Placement(transformation(origin = {10, -80}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
                QuasiStationary.SinglePhase.Interfaces.NegativePin starpoint2 annotation(
                  Placement(transformation(extent = {{40, -110}, {60, -90}}, rotation = 0)));
                QuasiStationary.MultiPhase.Basic.Delta Rot21(final m = m) annotation(
                  Placement(transformation(origin = {30, 0}, extent = {{10, -10}, {-10, 10}}, rotation = 90)));
                QuasiStationary.MultiPhase.Basic.Delta Rot22(final m = m) annotation(
                  Placement(transformation(origin = {40, 20}, extent = {{-10, 10}, {10, -10}}, rotation = 180)));
              equation
                connect(star1.pin_n, starpoint1) annotation(
                  Line(points = {{-10, -90}, {-10, -100}, {-50, -100}}, color = {85, 170, 255}));
                connect(star2.pin_n, starpoint2) annotation(
                  Line(points = {{10, -90}, {10, -100}, {50, -100}}, color = {85, 170, 255}));
                connect(l1sigma.plug_n, core.plug_p1) annotation(
                  Line(points = {{-50, 0}, {-50, 20}, {-10, 20}, {-10, 5}}, color = {85, 170, 255}));
                connect(core.plug_n1, star1.plug_p) annotation(
                  Line(points = {{-10, -5}, {-10, -37.5}, {-10, -70}, {-10, -70}}, color = {85, 170, 255}));
                connect(core.plug_p2, Rot21.plug_p) annotation(
                  Line(points = {{10, 10}, {30, 10}}, color = {85, 170, 255}));
                connect(core.plug_p3, Rot21.plug_n) annotation(
                  Line(points = {{10, -4}, {20, -4}, {20, -10}, {30, -10}}, color = {85, 170, 255}));
                connect(l2sigma.plug_p, Rot22.plug_p) annotation(
                  Line(points = {{50, 0}, {50, 20}}, color = {85, 170, 255}));
                connect(Rot22.plug_n, core.plug_n2) annotation(
                  Line(points = {{30, 20}, {20, 20}, {20, 4}, {10, 4}}, color = {85, 170, 255}));
                connect(core.plug_n3, star2.plug_p) annotation(
                  Line(points = {{10, -10}, {10, -70}, {10, -70}}, color = {85, 170, 255}));
                annotation(
                  defaultComponentName = "transformer",
                  Documentation(info = "<html>
Transformer Yz3
<br>Typical parameters see:
<a href=\"modelica://Modelica.Electrical.Machines.Interfaces.PartialBasicTransformer\">PartialBasicTransformer</a>
</html>"));
              end Yz03;

              model Yz05 "Transformer Yz5"
                extends QuasiStationary.Machines.Interfaces.PartialBasicTransformer(final VectorGroup = "Yz05");
                QuasiStationary.MultiPhase.Basic.Star star1(final m = m) annotation(
                  Placement(transformation(origin = {-10, -80}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
                QuasiStationary.SinglePhase.Interfaces.NegativePin starpoint1 annotation(
                  Placement(transformation(extent = {{-60, -110}, {-40, -90}}, rotation = 0)));
                QuasiStationary.MultiPhase.Basic.Star star2(final m = m) annotation(
                  Placement(transformation(origin = {10, -80}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
                QuasiStationary.SinglePhase.Interfaces.NegativePin starpoint2 annotation(
                  Placement(transformation(extent = {{40, -110}, {60, -90}}, rotation = 0)));
                QuasiStationary.MultiPhase.Basic.Delta Rot21(final m = m) annotation(
                  Placement(transformation(origin = {30, 0}, extent = {{-10, 10}, {10, -10}}, rotation = 90)));
              equation
                connect(star1.pin_n, starpoint1) annotation(
                  Line(points = {{-10, -90}, {-10, -100}, {-50, -100}}, color = {85, 170, 255}));
                connect(star2.pin_n, starpoint2) annotation(
                  Line(points = {{10, -90}, {10, -100}, {50, -100}}, color = {85, 170, 255}));
                connect(l1sigma.plug_n, core.plug_p1) annotation(
                  Line(points = {{-50, 0}, {-50, 20}, {-10, 20}, {-10, 5}}, color = {85, 170, 255}));
                connect(core.plug_n1, star1.plug_p) annotation(
                  Line(points = {{-10, -5}, {-10, -70}, {-10, -70}}, color = {85, 170, 255}));
                connect(core.plug_p2, Rot21.plug_n) annotation(
                  Line(points = {{10, 10}, {30, 10}}, color = {85, 170, 255}));
                connect(core.plug_p3, Rot21.plug_p) annotation(
                  Line(points = {{10, -4}, {20, -4}, {20, -10}, {30, -10}}, color = {85, 170, 255}));
                connect(core.plug_n3, star2.plug_p) annotation(
                  Line(points = {{10, -10}, {10, -40}, {10, -70}, {10, -70}}, color = {85, 170, 255}));
                connect(l2sigma.plug_p, core.plug_n2) annotation(
                  Line(points = {{50, 0}, {50, 20}, {20, 20}, {20, 4}, {10, 4}}, color = {85, 170, 255}));
                annotation(
                  defaultComponentName = "transformer",
                  Documentation(info = "<html>
Transformer Yz5
<br>Typical parameters see:
<a href=\"modelica://Modelica.Electrical.Machines.Interfaces.PartialBasicTransformer\">PartialBasicTransformer</a>
</html>"));
              end Yz05;

              model Yz07 "Transformer Yz7"
                extends QuasiStationary.Machines.Interfaces.PartialBasicTransformer(final VectorGroup = "Yz07");
                QuasiStationary.MultiPhase.Basic.Star star1(final m = m) annotation(
                  Placement(transformation(origin = {-10, -80}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
                QuasiStationary.SinglePhase.Interfaces.NegativePin starpoint1 annotation(
                  Placement(transformation(extent = {{-60, -110}, {-40, -90}}, rotation = 0)));
                QuasiStationary.MultiPhase.Basic.Star star2(final m = m) annotation(
                  Placement(transformation(origin = {10, -80}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
                QuasiStationary.SinglePhase.Interfaces.NegativePin starpoint2 annotation(
                  Placement(transformation(extent = {{40, -110}, {60, -90}}, rotation = 0)));
                QuasiStationary.MultiPhase.Basic.Delta Rot21(final m = m) annotation(
                  Placement(transformation(origin = {30, 0}, extent = {{10, -10}, {-10, 10}}, rotation = 90)));
              equation
                connect(star1.pin_n, starpoint1) annotation(
                  Line(points = {{-10, -90}, {-10, -100}, {-50, -100}}, color = {85, 170, 255}));
                connect(star2.pin_n, starpoint2) annotation(
                  Line(points = {{10, -90}, {10, -100}, {50, -100}}, color = {85, 170, 255}));
                connect(l1sigma.plug_n, core.plug_p1) annotation(
                  Line(points = {{-50, 0}, {-50, 20}, {-10, 20}, {-10, 5}}, color = {85, 170, 255}));
                connect(core.plug_n1, star1.plug_p) annotation(
                  Line(points = {{-10, -5}, {-10, -37.5}, {-10, -70}, {-10, -70}}, color = {85, 170, 255}));
                connect(core.plug_p2, Rot21.plug_p) annotation(
                  Line(points = {{10, 10}, {30, 10}}, color = {85, 170, 255}));
                connect(core.plug_p3, Rot21.plug_n) annotation(
                  Line(points = {{10, -4}, {20, -4}, {20, -10}, {30, -10}}, color = {85, 170, 255}));
                connect(l2sigma.plug_p, core.plug_n2) annotation(
                  Line(points = {{50, 0}, {50, 20}, {20, 20}, {20, 4}, {10, 4}}, color = {85, 170, 255}));
                connect(core.plug_n3, star2.plug_p) annotation(
                  Line(points = {{10, -10}, {10, -40}, {10, -70}, {10, -70}}, color = {85, 170, 255}));
                annotation(
                  defaultComponentName = "transformer",
                  Documentation(info = "<html>
Transformer Yz7
<br>Typical parameters see:
<a href=\"modelica://Modelica.Electrical.Machines.Interfaces.PartialBasicTransformer\">PartialBasicTransformer</a>
</html>"));
              end Yz07;

              model Yz09 "Transformer Yz9"
                extends QuasiStationary.Machines.Interfaces.PartialBasicTransformer(final VectorGroup = "Yz09");
                QuasiStationary.MultiPhase.Basic.Star star1(final m = m) annotation(
                  Placement(transformation(origin = {-10, -80}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
                QuasiStationary.SinglePhase.Interfaces.NegativePin starpoint1 annotation(
                  Placement(transformation(extent = {{-60, -110}, {-40, -90}}, rotation = 0)));
                QuasiStationary.MultiPhase.Basic.Star star2(final m = m) annotation(
                  Placement(transformation(origin = {10, -80}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
                QuasiStationary.SinglePhase.Interfaces.NegativePin starpoint2 annotation(
                  Placement(transformation(extent = {{40, -110}, {60, -90}}, rotation = 0)));
                QuasiStationary.MultiPhase.Basic.Delta Rot21(final m = m) annotation(
                  Placement(transformation(origin = {30, 0}, extent = {{-10, 10}, {10, -10}}, rotation = 90)));
                QuasiStationary.MultiPhase.Basic.Delta Rot22(final m = m) annotation(
                  Placement(transformation(origin = {40, 20}, extent = {{10, 10}, {-10, -10}}, rotation = 180)));
              equation
                connect(star1.pin_n, starpoint1) annotation(
                  Line(points = {{-10, -90}, {-10, -100}, {-50, -100}}, color = {85, 170, 255}));
                connect(star2.pin_n, starpoint2) annotation(
                  Line(points = {{10, -90}, {10, -100}, {50, -100}}, color = {85, 170, 255}));
                connect(l1sigma.plug_n, core.plug_p1) annotation(
                  Line(points = {{-50, 0}, {-50, 20}, {-10, 20}, {-10, 5}}, color = {85, 170, 255}));
                connect(core.plug_n1, star1.plug_p) annotation(
                  Line(points = {{-10, -5}, {-10, -70}, {-10, -70}}, color = {85, 170, 255}));
                connect(core.plug_p2, Rot21.plug_n) annotation(
                  Line(points = {{10, 10}, {30, 10}}, color = {85, 170, 255}));
                connect(core.plug_p3, Rot21.plug_p) annotation(
                  Line(points = {{10, -4}, {20, -4}, {20, -10}, {30, -10}}, color = {85, 170, 255}));
                connect(core.plug_n3, star2.plug_p) annotation(
                  Line(points = {{10, -10}, {10, -70}, {10, -70}}, color = {85, 170, 255}));
                connect(l2sigma.plug_p, Rot22.plug_n) annotation(
                  Line(points = {{50, 0}, {50, 20}, {50, 20}}, color = {85, 170, 255}));
                connect(Rot22.plug_p, core.plug_n2) annotation(
                  Line(points = {{30, 20}, {20, 20}, {20, 4}, {10, 4}}, color = {85, 170, 255}));
                annotation(
                  defaultComponentName = "transformer",
                  Documentation(info = "<html>
Transformer Yz9
<br>Typical parameters see:
<a href=\"modelica://Modelica.Electrical.Machines.Interfaces.PartialBasicTransformer\">PartialBasicTransformer</a>
</html>"));
              end Yz09;

              model Yz11 "Transformer Yz11"
                extends QuasiStationary.Machines.Interfaces.PartialBasicTransformer(final VectorGroup = "Yz11");
                QuasiStationary.MultiPhase.Basic.Star star1(final m = m) annotation(
                  Placement(transformation(origin = {-10, -80}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
                QuasiStationary.SinglePhase.Interfaces.NegativePin starpoint1 annotation(
                  Placement(transformation(extent = {{-60, -110}, {-40, -90}}, rotation = 0)));
                QuasiStationary.MultiPhase.Basic.Star star2(final m = m) annotation(
                  Placement(transformation(origin = {10, -80}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
                QuasiStationary.SinglePhase.Interfaces.NegativePin starpoint2 annotation(
                  Placement(transformation(extent = {{40, -110}, {60, -90}}, rotation = 0)));
                QuasiStationary.MultiPhase.Basic.Delta Rot21(final m = m) annotation(
                  Placement(transformation(origin = {30, 0}, extent = {{10, -10}, {-10, 10}}, rotation = 90)));
                QuasiStationary.MultiPhase.Basic.Delta Rot22(final m = m) annotation(
                  Placement(transformation(origin = {40, 20}, extent = {{10, 10}, {-10, -10}}, rotation = 180)));
              equation
                connect(star1.pin_n, starpoint1) annotation(
                  Line(points = {{-10, -90}, {-10, -100}, {-50, -100}}, color = {85, 170, 255}));
                connect(star2.pin_n, starpoint2) annotation(
                  Line(points = {{10, -90}, {10, -100}, {50, -100}}, color = {85, 170, 255}));
                connect(l1sigma.plug_n, core.plug_p1) annotation(
                  Line(points = {{-50, 0}, {-50, 20}, {-10, 20}, {-10, 5}}, color = {85, 170, 255}));
                connect(core.plug_n1, star1.plug_p) annotation(
                  Line(points = {{-10, -5}, {-10, -37.5}, {-10, -70}, {-10, -70}}, color = {85, 170, 255}));
                connect(core.plug_p2, Rot21.plug_p) annotation(
                  Line(points = {{10, 10}, {30, 10}}, color = {85, 170, 255}));
                connect(core.plug_p3, Rot21.plug_n) annotation(
                  Line(points = {{10, -4}, {20, -4}, {20, -10}, {30, -10}}, color = {85, 170, 255}));
                connect(l2sigma.plug_p, Rot22.plug_n) annotation(
                  Line(points = {{50, 0}, {50, 20}}, color = {85, 170, 255}));
                connect(Rot22.plug_p, core.plug_n2) annotation(
                  Line(points = {{30, 20}, {20, 20}, {20, 4}, {10, 4}}, color = {85, 170, 255}));
                connect(core.plug_n3, star2.plug_p) annotation(
                  Line(points = {{10, -10}, {10, -70}, {10, -70}}, color = {85, 170, 255}));
                annotation(
                  defaultComponentName = "transformer",
                  Documentation(info = "<html>
Transformer Yz11
<br>Typical parameters see:
<a href=\"modelica://Modelica.Electrical.Machines.Interfaces.PartialBasicTransformer\">PartialBasicTransformer</a>
</html>"));
              end Yz11;
              annotation(
                Documentation(info = "<HTML>
This package contains transformers primary Y connected / secondary zig-zag connected in all possible vector groups.
</HTML>", revisions = "<HTML>
<dl>
  <dt><b>Main Authors:</b></dt>
  <dd>
  <a href=\"http://www.haumer.at/\">Anton Haumer</a><br>
  Technical Consulting &amp; Electrical Engineering<br>
  A-3423 St.Andrae-Woerdern<br>Austria<br>
  email: <a href=\"mailto:a.haumer@haumer.at\">a.haumer@haumer.at</a>
  </dd>
  <dt><b>Copyright:</b></dt>
  <dd>Copyright &copy; 1998-2015, Modelica Association and Anton Haumer.<br>
  <i>The Modelica package is <b>free</b> software; it can be redistributed and/or modified
  under the terms of the <b>Modelica license</b>, see the license conditions
  and the accompanying <b>disclaimer</b> in the documentation of package
  Modelica in file \"Modelica/package.mo\".</i></dd>
</dl>
  <ul>
  <li> v1.0.0 2006/11/19 Anton Haumer</li>
  </ul>
</HTML>"));
            end Yz;

            package Dy "Transformers: primary D / secondary y"
              extends Modelica.Icons.VariantsPackage;

              model Dy01 "Transformer Dy1"
                extends QuasiStationary.Machines.Interfaces.PartialBasicTransformer(final VectorGroup = "Dy01");
                QuasiStationary.MultiPhase.Basic.Star star2(final m = m) annotation(
                  Placement(transformation(origin = {10, -80}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
                QuasiStationary.SinglePhase.Interfaces.NegativePin starpoint2 annotation(
                  Placement(transformation(extent = {{40, -110}, {60, -90}}, rotation = 0)));
                QuasiStationary.MultiPhase.Basic.Delta Delta1(final m = m) annotation(
                  Placement(transformation(extent = {{-50, -30}, {-30, -10}}, rotation = 0)));
                QuasiStationary.MultiPhase.Basic.Delta Rot2(final m = m) annotation(
                  Placement(transformation(extent = {{50, -30}, {30, -10}}, rotation = 0)));
              equation
                connect(star2.pin_n, starpoint2) annotation(
                  Line(points = {{10, -90}, {10, -100}, {50, -100}}, color = {85, 170, 255}));
                connect(l2sigma.plug_p, Rot2.plug_p) annotation(
                  Line(points = {{50, 0}, {50, -20}}, color = {85, 170, 255}));
                connect(Delta1.plug_p, r1.plug_p) annotation(
                  Line(points = {{-50, -20}, {-90, -20}, {-90, 0}}, color = {85, 170, 255}));
                connect(Delta1.plug_n, core.plug_n1) annotation(
                  Line(points = {{-30, -20}, {-10, -20}, {-10, -5}}, color = {85, 170, 255}));
                connect(l1sigma.plug_n, core.plug_p1) annotation(
                  Line(points = {{-50, 0}, {-50, 20}, {-10, 20}, {-10, 5}}, color = {85, 170, 255}));
                connect(core.plug_n2, core.plug_p3) annotation(
                  Line(points = {{10, 4}, {10, -4}}, color = {85, 170, 255}));
                connect(core.plug_n3, Rot2.plug_n) annotation(
                  Line(points = {{10, -10}, {10, -20}, {30, -20}}, color = {85, 170, 255}));
                connect(core.plug_p2, star2.plug_p) annotation(
                  Line(points = {{10, 10}, {20, 10}, {20, -70}, {10, -70}}, color = {85, 170, 255}));
                annotation(
                  defaultComponentName = "transformer",
                  Documentation(info = "<html>
Transformer Dy1
<br>Typical parameters see:
<a href=\"modelica://Modelica.Electrical.Machines.Interfaces.PartialBasicTransformer\">PartialBasicTransformer</a>
</html>"));
              end Dy01;

              model Dy03 "Transformer Dy3"
                extends QuasiStationary.Machines.Interfaces.PartialBasicTransformer(final VectorGroup = "Dy03");
                QuasiStationary.MultiPhase.Basic.Star star2(final m = m) annotation(
                  Placement(transformation(origin = {10, -80}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
                QuasiStationary.SinglePhase.Interfaces.NegativePin starpoint2 annotation(
                  Placement(transformation(extent = {{40, -110}, {60, -90}}, rotation = 0)));
                QuasiStationary.MultiPhase.Basic.Delta Delta1(final m = m) annotation(
                  Placement(transformation(extent = {{-50, -30}, {-30, -10}}, rotation = 0)));
                QuasiStationary.MultiPhase.Basic.Delta Rot2(final m = m) annotation(
                  Placement(transformation(extent = {{30, 10}, {50, 30}}, rotation = 0)));
              equation
                connect(star2.pin_n, starpoint2) annotation(
                  Line(points = {{10, -90}, {10, -100}, {50, -100}}, color = {85, 170, 255}));
                connect(Rot2.plug_n, l2sigma.plug_p) annotation(
                  Line(points = {{50, 20}, {50, 0}}, color = {85, 170, 255}));
                connect(Delta1.plug_p, r1.plug_p) annotation(
                  Line(points = {{-50, -20}, {-90, -20}, {-90, 0}}, color = {85, 170, 255}));
                connect(l1sigma.plug_n, core.plug_p1) annotation(
                  Line(points = {{-50, 0}, {-50, 20}, {-10, 20}, {-10, 5}}, color = {85, 170, 255}));
                connect(Delta1.plug_n, core.plug_n1) annotation(
                  Line(points = {{-30, -20}, {-10, -20}, {-10, -5}}, color = {85, 170, 255}));
                connect(core.plug_p2, Rot2.plug_p) annotation(
                  Line(points = {{10, 10}, {10, 20}, {30, 20}}, color = {85, 170, 255}));
                connect(core.plug_n2, core.plug_p3) annotation(
                  Line(points = {{10, 4}, {10, -4}}, color = {85, 170, 255}));
                connect(core.plug_n3, star2.plug_p) annotation(
                  Line(points = {{10, -10}, {10, -40}, {10, -70}, {10, -70}}, color = {85, 170, 255}));
                annotation(
                  defaultComponentName = "transformer",
                  Documentation(info = "<html>
Transformer Dy3
<br>Typical parameters see:
<a href=\"modelica://Modelica.Electrical.Machines.Interfaces.PartialBasicTransformer\">PartialBasicTransformer</a>
</html>"));
              end Dy03;

              model Dy05 "Transformer Dy5"
                extends QuasiStationary.Machines.Interfaces.PartialBasicTransformer(final VectorGroup = "Dy05");
                QuasiStationary.MultiPhase.Basic.Star star2(final m = m) annotation(
                  Placement(transformation(origin = {10, -80}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
                QuasiStationary.SinglePhase.Interfaces.NegativePin starpoint2 annotation(
                  Placement(transformation(extent = {{40, -110}, {60, -90}}, rotation = 0)));
                QuasiStationary.MultiPhase.Basic.Delta Delta1(final m = m) annotation(
                  Placement(transformation(extent = {{-50, -30}, {-30, -10}}, rotation = 0)));
              equation
                connect(star2.pin_n, starpoint2) annotation(
                  Line(points = {{10, -90}, {10, -100}, {50, -100}}, color = {85, 170, 255}));
                connect(Delta1.plug_p, r1.plug_p) annotation(
                  Line(points = {{-50, -20}, {-90, -20}, {-90, 0}}, color = {85, 170, 255}));
                connect(Delta1.plug_n, core.plug_n1) annotation(
                  Line(points = {{-30, -20}, {-10, -20}, {-10, -5}}, color = {85, 170, 255}));
                connect(l1sigma.plug_n, core.plug_p1) annotation(
                  Line(points = {{-50, 0}, {-50, 20}, {-10, 20}, {-10, 5}}, color = {85, 170, 255}));
                connect(core.plug_n2, core.plug_p3) annotation(
                  Line(points = {{10, 4}, {10, -4}}, color = {85, 170, 255}));
                connect(l2sigma.plug_p, core.plug_n3) annotation(
                  Line(points = {{50, 0}, {50, -20}, {10, -20}, {10, -10}}, color = {85, 170, 255}));
                connect(core.plug_p2, star2.plug_p) annotation(
                  Line(points = {{10, 10}, {20, 10}, {20, -70}, {10, -70}}, color = {85, 170, 255}));
                annotation(
                  defaultComponentName = "transformer",
                  Documentation(info = "<html>
Transformer Dy5
<br>Typical parameters see:
<a href=\"modelica://Modelica.Electrical.Machines.Interfaces.PartialBasicTransformer\">PartialBasicTransformer</a>
</html>"));
              end Dy05;

              model Dy07 "Transformer Dy7"
                extends QuasiStationary.Machines.Interfaces.PartialBasicTransformer(final VectorGroup = "Dy07");
                QuasiStationary.MultiPhase.Basic.Star star2(final m = m) annotation(
                  Placement(transformation(origin = {10, -80}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
                QuasiStationary.SinglePhase.Interfaces.NegativePin starpoint2 annotation(
                  Placement(transformation(extent = {{40, -110}, {60, -90}}, rotation = 0)));
                QuasiStationary.MultiPhase.Basic.Delta Delta1(final m = m) annotation(
                  Placement(transformation(extent = {{-50, -30}, {-30, -10}}, rotation = 0)));
                QuasiStationary.MultiPhase.Basic.Delta Rot2(final m = m) annotation(
                  Placement(transformation(extent = {{50, 10}, {30, 30}}, rotation = 0)));
              equation
                connect(star2.pin_n, starpoint2) annotation(
                  Line(points = {{10, -90}, {10, -100}, {50, -100}}, color = {85, 170, 255}));
                connect(r1.plug_p, Delta1.plug_p) annotation(
                  Line(points = {{-90, 0}, {-90, -20}, {-50, -20}}, color = {85, 170, 255}));
                connect(Rot2.plug_p, l2sigma.plug_p) annotation(
                  Line(points = {{50, 20}, {50, 0}}, color = {85, 170, 255}));
                connect(l1sigma.plug_n, core.plug_p1) annotation(
                  Line(points = {{-50, 0}, {-50, 20}, {-10, 20}, {-10, 5}}, color = {85, 170, 255}));
                connect(Delta1.plug_n, core.plug_n1) annotation(
                  Line(points = {{-30, -20}, {-10, -20}, {-10, -5}}, color = {85, 170, 255}));
                connect(core.plug_n2, core.plug_p3) annotation(
                  Line(points = {{10, 4}, {10, -4}}, color = {85, 170, 255}));
                connect(core.plug_n3, star2.plug_p) annotation(
                  Line(points = {{10, -10}, {10, -40}, {10, -70}, {10, -70}}, color = {85, 170, 255}));
                connect(core.plug_p2, Rot2.plug_n) annotation(
                  Line(points = {{10, 10}, {10, 20}, {30, 20}}, color = {85, 170, 255}));
                annotation(
                  defaultComponentName = "transformer",
                  Documentation(info = "<html>
Transformer Dy7
<br>Typical parameters see:
<a href=\"modelica://Modelica.Electrical.Machines.Interfaces.PartialBasicTransformer\">PartialBasicTransformer</a>
</html>"));
              end Dy07;

              model Dy09 "Transformer Dy9"
                extends QuasiStationary.Machines.Interfaces.PartialBasicTransformer(final VectorGroup = "Dy09");
                QuasiStationary.MultiPhase.Basic.Star star2(final m = m) annotation(
                  Placement(transformation(origin = {10, -80}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
                QuasiStationary.SinglePhase.Interfaces.NegativePin starpoint2 annotation(
                  Placement(transformation(extent = {{40, -110}, {60, -90}}, rotation = 0)));
                QuasiStationary.MultiPhase.Basic.Delta Delta1(final m = m) annotation(
                  Placement(transformation(extent = {{-50, -30}, {-30, -10}}, rotation = 0)));
                QuasiStationary.MultiPhase.Basic.Delta Rot2(final m = m) annotation(
                  Placement(transformation(extent = {{30, -30}, {50, -10}}, rotation = 0)));
              equation
                connect(star2.pin_n, starpoint2) annotation(
                  Line(points = {{10, -90}, {10, -100}, {50, -100}}, color = {85, 170, 255}));
                connect(r1.plug_p, Delta1.plug_p) annotation(
                  Line(points = {{-90, 0}, {-90, -20}, {-50, -20}}, color = {85, 170, 255}));
                connect(l2sigma.plug_p, Rot2.plug_n) annotation(
                  Line(points = {{50, 0}, {50, -20}}, color = {85, 170, 255}));
                connect(l1sigma.plug_n, core.plug_p1) annotation(
                  Line(points = {{-50, 0}, {-50, 20}, {-10, 20}, {-10, 5}, {-10, 5}}, color = {85, 170, 255}));
                connect(Delta1.plug_n, core.plug_n1) annotation(
                  Line(points = {{-30, -20}, {-10, -20}, {-10, -5}}, color = {85, 170, 255}));
                connect(core.plug_n2, core.plug_p3) annotation(
                  Line(points = {{10, 4}, {10, -4}}, color = {85, 170, 255}));
                connect(core.plug_n3, Rot2.plug_p) annotation(
                  Line(points = {{10, -10}, {10, -20}, {30, -20}}, color = {85, 170, 255}));
                connect(core.plug_p2, star2.plug_p) annotation(
                  Line(points = {{10, 10}, {20, 10}, {20, -70}, {10, -70}}, color = {85, 170, 255}));
                annotation(
                  defaultComponentName = "transformer",
                  Documentation(info = "<html>
Transformer Dy9
<br>Typical parameters see:
<a href=\"modelica://Modelica.Electrical.Machines.Interfaces.PartialBasicTransformer\">PartialBasicTransformer</a>
</html>"));
              end Dy09;

              model Dy11 "Transformer Dy11"
                extends QuasiStationary.Machines.Interfaces.PartialBasicTransformer(final VectorGroup = "Dy11");
                QuasiStationary.MultiPhase.Basic.Star star2(final m = m) annotation(
                  Placement(transformation(origin = {10, -80}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
                QuasiStationary.SinglePhase.Interfaces.NegativePin starpoint2 annotation(
                  Placement(transformation(extent = {{40, -110}, {60, -90}}, rotation = 0)));
                QuasiStationary.MultiPhase.Basic.Delta Delta1(final m = m) annotation(
                  Placement(transformation(extent = {{-50, -30}, {-30, -10}}, rotation = 0)));
              equation
                connect(star2.pin_n, starpoint2) annotation(
                  Line(points = {{10, -90}, {10, -100}, {50, -100}}, color = {85, 170, 255}));
                connect(r1.plug_p, Delta1.plug_p) annotation(
                  Line(points = {{-90, 0}, {-90, -20}, {-50, -20}}, color = {85, 170, 255}));
                connect(l1sigma.plug_n, core.plug_p1) annotation(
                  Line(points = {{-50, 0}, {-50, 20}, {-10, 20}, {-10, 5}}, color = {85, 170, 255}));
                connect(Delta1.plug_n, core.plug_n1) annotation(
                  Line(points = {{-30, -20}, {-10, -20}, {-10, -5}}, color = {85, 170, 255}));
                connect(core.plug_n2, core.plug_p3) annotation(
                  Line(points = {{10, 4}, {10, -4}}, color = {85, 170, 255}));
                connect(core.plug_n3, star2.plug_p) annotation(
                  Line(points = {{10, -10}, {10, -40}, {10, -70}, {10, -70}}, color = {85, 170, 255}));
                connect(core.plug_p2, l2sigma.plug_p) annotation(
                  Line(points = {{10, 10}, {10, 20}, {50, 20}, {50, 0}}, color = {85, 170, 255}));
                annotation(
                  defaultComponentName = "transformer",
                  Documentation(info = "<html>
Transformer Dy11
<br>Typical parameters see:
<a href=\"modelica://Modelica.Electrical.Machines.Interfaces.PartialBasicTransformer\">PartialBasicTransformer</a>
</html>"));
              end Dy11;
              annotation(
                Documentation(info = "<HTML>
This package contains transformers primary D connected / secondary y connected in all possible vector groups.
</HTML>", revisions = "<HTML>
<dl>
  <dt><b>Main Authors:</b></dt>
  <dd>
  <a href=\"http://www.haumer.at/\">Anton Haumer</a><br>
  Technical Consulting &amp; Electrical Engineering<br>
  A-3423 St.Andrae-Woerdern<br>Austria<br>
  email: <a href=\"mailto:a.haumer@haumer.at\">a.haumer@haumer.at</a>
  </dd>
  <dt><b>Copyright:</b></dt>
  <dd>Copyright &copy; 1998-2015, Modelica Association and Anton Haumer.<br>
  <i>The Modelica package is <b>free</b> software; it can be redistributed and/or modified
  under the terms of the <b>Modelica license</b>, see the license conditions
  and the accompanying <b>disclaimer</b> in the documentation of package
  Modelica in file \"Modelica/package.mo\".</i></dd>
</dl>
  <ul>
  <li> v1.0.0 2006/11/19 Anton Haumer</li>
  </ul>
</HTML>"));
            end Dy;

            package Dd "Transformers: primary D / secondary d"
              extends Modelica.Icons.VariantsPackage;

              model Dd00 "Transformer Dd0"
                extends QuasiStationary.Machines.Interfaces.PartialBasicTransformer(final VectorGroup = "Dd00");
                QuasiStationary.MultiPhase.Basic.Delta Delta1(final m = m) annotation(
                  Placement(transformation(extent = {{-50, -30}, {-30, -10}}, rotation = 0)));
                QuasiStationary.MultiPhase.Basic.Delta Delta2(final m = m) annotation(
                  Placement(transformation(extent = {{50, -30}, {30, -10}}, rotation = 0)));
              equation
                connect(r1.plug_p, Delta1.plug_p) annotation(
                  Line(points = {{-90, 0}, {-90, -20}, {-50, -20}}, color = {85, 170, 255}));
                connect(Delta2.plug_p, r2.plug_n) annotation(
                  Line(points = {{50, -20}, {90, -20}, {90, 0}}, color = {85, 170, 255}));
                connect(l1sigma.plug_n, core.plug_p1) annotation(
                  Line(points = {{-50, 0}, {-50, 20}, {-10, 20}, {-10, 5}}, color = {85, 170, 255}));
                connect(Delta1.plug_n, core.plug_n1) annotation(
                  Line(points = {{-30, -20}, {-10, -20}, {-10, -5}}, color = {85, 170, 255}));
                connect(core.plug_n2, core.plug_p3) annotation(
                  Line(points = {{10, 4}, {10, -4}}, color = {85, 170, 255}));
                connect(core.plug_p2, l2sigma.plug_p) annotation(
                  Line(points = {{10, 10}, {10, 20}, {50, 20}, {50, 0}}, color = {85, 170, 255}));
                connect(core.plug_n3, Delta2.plug_n) annotation(
                  Line(points = {{10, -10}, {10, -20}, {30, -20}}, color = {85, 170, 255}));
                annotation(
                  defaultComponentName = "transformer",
                  Documentation(info = "<html>
Transformer Dd0
<br>Typical parameters see:
<a href=\"modelica://Modelica.Electrical.Machines.Interfaces.PartialBasicTransformer\">PartialBasicTransformer</a>
</html>"));
              end Dd00;

              model Dd02 "Transformer Dd2"
                extends QuasiStationary.Machines.Interfaces.PartialBasicTransformer(final VectorGroup = "Dd02");
                QuasiStationary.MultiPhase.Basic.Delta Delta1(final m = m) annotation(
                  Placement(transformation(extent = {{-50, -30}, {-30, -10}}, rotation = 0)));
                QuasiStationary.MultiPhase.Basic.Delta Delta2(final m = m) annotation(
                  Placement(transformation(extent = {{30, 10}, {50, 30}}, rotation = 0)));
                QuasiStationary.MultiPhase.Basic.Delta Rot2(final m = m) annotation(
                  Placement(transformation(extent = {{50, -30}, {30, -10}}, rotation = 0)));
              equation
                connect(r1.plug_p, Delta1.plug_p) annotation(
                  Line(points = {{-90, 0}, {-90, -20}, {-50, -20}}, color = {85, 170, 255}));
                connect(Delta2.plug_n, r2.plug_n) annotation(
                  Line(points = {{50, 20}, {90, 20}, {90, 0}}, color = {85, 170, 255}));
                connect(l2sigma.plug_p, Rot2.plug_p) annotation(
                  Line(points = {{50, 0}, {50, -20}}, color = {85, 170, 255}));
                connect(l1sigma.plug_n, core.plug_p1) annotation(
                  Line(points = {{-50, 0}, {-50, 20}, {-10, 20}, {-10, 5}}, color = {85, 170, 255}));
                connect(Delta1.plug_n, core.plug_n1) annotation(
                  Line(points = {{-30, -20}, {-10, -20}, {-10, -5}}, color = {85, 170, 255}));
                connect(core.plug_n2, core.plug_p3) annotation(
                  Line(points = {{10, 4}, {10, -4}}, color = {85, 170, 255}));
                connect(core.plug_p2, Delta2.plug_p) annotation(
                  Line(points = {{10, 10}, {10, 20}, {30, 20}}, color = {85, 170, 255}));
                connect(core.plug_n3, Rot2.plug_n) annotation(
                  Line(points = {{10, -10}, {10, -20}, {30, -20}}, color = {85, 170, 25}));
                annotation(
                  defaultComponentName = "transformer",
                  Documentation(info = "<html>
Transformer Dd2
<br>Typical parameters see:
<a href=\"modelica://Modelica.Electrical.Machines.Interfaces.PartialBasicTransformer\">PartialBasicTransformer</a>
</html>"));
              end Dd02;

              model Dd04 "Transformer Dd4"
                extends QuasiStationary.Machines.Interfaces.PartialBasicTransformer(final VectorGroup = "Dd04");
                QuasiStationary.MultiPhase.Basic.Delta Delta1(final m = m) annotation(
                  Placement(transformation(extent = {{-50, -30}, {-30, -10}}, rotation = 0)));
                QuasiStationary.MultiPhase.Basic.Delta Delta2(final m = m) annotation(
                  Placement(transformation(extent = {{30, 10}, {50, 30}}, rotation = 0)));
              equation
                connect(r1.plug_p, Delta1.plug_p) annotation(
                  Line(points = {{-90, 0}, {-90, -20}, {-50, -20}}, color = {85, 170, 25}));
                connect(Delta2.plug_n, r2.plug_n) annotation(
                  Line(points = {{50, 20}, {90, 20}, {90, 0}}, color = {85, 170, 255}));
                connect(l1sigma.plug_n, core.plug_p1) annotation(
                  Line(points = {{-50, 0}, {-50, 20}, {-10, 20}, {-10, 5}}, color = {85, 170, 255}));
                connect(Delta1.plug_n, core.plug_n1) annotation(
                  Line(points = {{-30, -20}, {-10, -20}, {-10, -5}}, color = {85, 170, 25}));
                connect(core.plug_n2, core.plug_p3) annotation(
                  Line(points = {{10, 4}, {10, 1}, {10, 1}, {10, -4}}, color = {85, 170, 25}));
                connect(core.plug_p2, Delta2.plug_p) annotation(
                  Line(points = {{10, 10}, {10, 20}, {30, 20}}, color = {85, 170, 255}));
                connect(core.plug_n3, l2sigma.plug_p) annotation(
                  Line(points = {{10, -10}, {10, -20}, {50, -20}, {50, 0}}, color = {85, 170, 255}));
                annotation(
                  defaultComponentName = "transformer",
                  Documentation(info = "<html>
Transformer Dd4
<br>Typical parameters see:
<a href=\"modelica://Modelica.Electrical.Machines.Interfaces.PartialBasicTransformer\">PartialBasicTransformer</a>
</html>"));
              end Dd04;

              model Dd06 "Transformer Dd6"
                extends QuasiStationary.Machines.Interfaces.PartialBasicTransformer(final VectorGroup = "Dd06");
                QuasiStationary.MultiPhase.Basic.Delta Delta1(final m = m) annotation(
                  Placement(transformation(extent = {{-50, -30}, {-30, -10}}, rotation = 0)));
                QuasiStationary.MultiPhase.Basic.Delta Delta2(final m = m) annotation(
                  Placement(transformation(extent = {{50, 10}, {30, 30}}, rotation = 0)));
              equation
                connect(Delta2.plug_p, r2.plug_n) annotation(
                  Line(points = {{50, 20}, {90, 20}, {90, 0}}, color = {85, 170, 255}));
                connect(Delta1.plug_p, r1.plug_p) annotation(
                  Line(points = {{-50, -20}, {-90, -20}, {-90, 0}}, color = {85, 170, 25}));
                connect(l1sigma.plug_n, core.plug_p1) annotation(
                  Line(points = {{-50, 0}, {-50, 20}, {-10, 20}, {-10, 5}}, color = {85, 170, 255}));
                connect(Delta1.plug_n, core.plug_n1) annotation(
                  Line(points = {{-30, -20}, {-10, -20}, {-10, -5}}, color = {85, 170, 25}));
                connect(core.plug_n2, core.plug_p3) annotation(
                  Line(points = {{10, 4}, {10, -4}}, color = {85, 170, 255}));
                connect(Delta2.plug_n, core.plug_p2) annotation(
                  Line(points = {{30, 20}, {10, 20}, {10, 10}}, color = {85, 170, 255}));
                connect(core.plug_n3, l2sigma.plug_p) annotation(
                  Line(points = {{10, -10}, {10, -20}, {50, -20}, {50, 0}}, color = {85, 170, 255}));
                annotation(
                  defaultComponentName = "transformer",
                  Documentation(info = "<html>
Transformer Dd6
<br>Typical parameters see:
<a href=\"modelica://Modelica.Electrical.Machines.Interfaces.PartialBasicTransformer\">PartialBasicTransformer</a>
</html>"));
              end Dd06;

              model Dd08 "Transformer Dd8"
                extends QuasiStationary.Machines.Interfaces.PartialBasicTransformer(final VectorGroup = "Dd08");
                QuasiStationary.MultiPhase.Basic.Delta Delta1(final m = m) annotation(
                  Placement(transformation(extent = {{-50, -30}, {-30, -10}}, rotation = 0)));
                QuasiStationary.MultiPhase.Basic.Delta Rot2(final m = m) annotation(
                  Placement(transformation(extent = {{50, 10}, {30, 30}}, rotation = 0)));
                QuasiStationary.MultiPhase.Basic.Delta Delta2(final m = m) annotation(
                  Placement(transformation(extent = {{30, -30}, {50, -10}}, rotation = 0)));
              equation
                connect(r1.plug_p, Delta1.plug_p) annotation(
                  Line(points = {{-90, 0}, {-90, -20}, {-50, -20}}, color = {85, 170, 25}));
                connect(Delta2.plug_n, r2.plug_n) annotation(
                  Line(points = {{50, -20}, {90, -20}, {90, 0}}, color = {85, 170, 255}));
                connect(Rot2.plug_p, l2sigma.plug_p) annotation(
                  Line(points = {{50, 20}, {50, 0}}, color = {85, 170, 255}));
                connect(l1sigma.plug_n, core.plug_p1) annotation(
                  Line(points = {{-50, 0}, {-50, 20}, {-10, 20}, {-10, 5}}, color = {85, 170, 255}));
                connect(core.plug_n1, Delta1.plug_n) annotation(
                  Line(points = {{-10, -5}, {-10, -20}, {-30, -20}}, color = {85, 170, 255}));
                connect(core.plug_n2, core.plug_p3) annotation(
                  Line(points = {{10, 4}, {10, -4}}, color = {85, 170, 255}));
                connect(core.plug_p2, Rot2.plug_n) annotation(
                  Line(points = {{10, 10}, {10, 20}, {30, 20}}, color = {85, 170, 255}));
                connect(core.plug_n3, Delta2.plug_p) annotation(
                  Line(points = {{10, -10}, {10, -20}, {30, -20}}, color = {85, 170, 255}));
                annotation(
                  defaultComponentName = "transformer",
                  Documentation(info = "<html>
Transformer Dd8
<br>Typical parameters see:
<a href=\"modelica://Modelica.Electrical.Machines.Interfaces.PartialBasicTransformer\">PartialBasicTransformer</a>
</html>"));
              end Dd08;

              model Dd10 "Transformer Dd10"
                extends QuasiStationary.Machines.Interfaces.PartialBasicTransformer(final VectorGroup = "Dd10");
                QuasiStationary.MultiPhase.Basic.Delta Delta1(final m = m) annotation(
                  Placement(transformation(extent = {{-50, -30}, {-30, -10}}, rotation = 0)));
                QuasiStationary.MultiPhase.Basic.Delta Delta2(final m = m) annotation(
                  Placement(transformation(extent = {{30, -30}, {50, -10}}, rotation = 0)));
              equation
                connect(r1.plug_p, Delta1.plug_p) annotation(
                  Line(points = {{-90, 0}, {-90, -20}, {-50, -20}}, color = {85, 170, 255}));
                connect(Delta2.plug_n, r2.plug_n) annotation(
                  Line(points = {{50, -20}, {90, -20}, {90, 0}}, color = {85, 170, 255}));
                connect(l1sigma.plug_n, core.plug_p1) annotation(
                  Line(points = {{-50, 0}, {-50, 20}, {-10, 20}, {-10, 5}}, color = {85, 170, 255}));
                connect(Delta1.plug_n, core.plug_n1) annotation(
                  Line(points = {{-30, -20}, {-10, -20}, {-10, -5}}, color = {85, 170, 255}));
                connect(core.plug_n2, core.plug_p3) annotation(
                  Line(points = {{10, 4}, {10, -4}}, color = {85, 170, 255}));
                connect(core.plug_p2, l2sigma.plug_p) annotation(
                  Line(points = {{10, 10}, {10, 20}, {50, 20}, {50, 0}}, color = {85, 170, 255}));
                connect(core.plug_n3, Delta2.plug_p) annotation(
                  Line(points = {{10, -10}, {10, -20}, {30, -20}}, color = {85, 170, 255}));
                annotation(
                  defaultComponentName = "transformer",
                  Documentation(info = "<html>
Transformer Dd10
<br>Typical parameters see:
<a href=\"modelica://Modelica.Electrical.Machines.Interfaces.PartialBasicTransformer\">PartialBasicTransformer</a>
</html>"));
              end Dd10;
              annotation(
                Documentation(info = "<HTML>
This package contains transformers primary D connected / secondary d connected in all possible vector groups.
</HTML>", revisions = "<HTML>
<dl>
  <dt><b>Main Authors:</b></dt>
  <dd>
  <a href=\"http://www.haumer.at/\">Anton Haumer</a><br>
  Technical Consulting &amp; Electrical Engineering<br>
  A-3423 St.Andrae-Woerdern<br>Austria<br>
  email: <a href=\"mailto:a.haumer@haumer.at\">a.haumer@haumer.at</a>
  </dd>
  <dt><b>Copyright:</b></dt>
  <dd>Copyright &copy; 1998-2015, Modelica Association and Anton Haumer.<br>
  <i>The Modelica package is <b>free</b> software; it can be redistributed and/or modified
  under the terms of the <b>Modelica license</b>, see the license conditions
  and the accompanying <b>disclaimer</b> in the documentation of package
  Modelica in file \"Modelica/package.mo\".</i></dd>
</dl>
  <ul>
  <li> v1.0.0 2006/11/19 Anton Haumer</li>
  </ul>
</HTML>"));
            end Dd;

            package Dz "Transformers: primary D / secondary zig-zag"
              extends Modelica.Icons.VariantsPackage;

              model Dz00 "Transformer Dz0"
                extends QuasiStationary.Machines.Interfaces.PartialBasicTransformer(final VectorGroup = "Dz00");
                QuasiStationary.MultiPhase.Basic.Star star2(final m = m) annotation(
                  Placement(transformation(origin = {10, -80}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
                QuasiStationary.SinglePhase.Interfaces.NegativePin starpoint2 annotation(
                  Placement(transformation(extent = {{40, -110}, {60, -90}}, rotation = 0)));
                QuasiStationary.MultiPhase.Basic.Delta Rot21(final m = m) annotation(
                  Placement(transformation(origin = {30, 0}, extent = {{-10, 10}, {10, -10}}, rotation = 90)));
                QuasiStationary.MultiPhase.Basic.Delta Rot22(final m = m) annotation(
                  Placement(transformation(origin = {40, 20}, extent = {{-10, 10}, {10, -10}}, rotation = 180)));
                QuasiStationary.MultiPhase.Basic.Delta Delta1(final m = m) annotation(
                  Placement(transformation(extent = {{-50, -30}, {-30, -10}}, rotation = 0)));
              equation
                connect(star2.pin_n, starpoint2) annotation(
                  Line(points = {{10, -90}, {10, -100}, {50, -100}}, color = {85, 170, 255}));
                connect(r1.plug_p, Delta1.plug_p) annotation(
                  Line(points = {{-90, 0}, {-90, -20}, {-50, -20}}, color = {85, 170, 255}));
                connect(l1sigma.plug_n, core.plug_p1) annotation(
                  Line(points = {{-50, 0}, {-50, 20}, {-10, 20}, {-10, 5}, {-10, 5}}, color = {85, 170, 255}));
                connect(Delta1.plug_n, core.plug_n1) annotation(
                  Line(points = {{-30, -20}, {-10, -20}, {-10, -5}}, color = {85, 170, 255}));
                connect(core.plug_p2, Rot21.plug_n) annotation(
                  Line(points = {{10, 10}, {30, 10}}, color = {85, 170, 255}));
                connect(core.plug_p3, Rot21.plug_p) annotation(
                  Line(points = {{10, -4}, {20, -4}, {20, -10}, {30, -10}}, color = {85, 170, 255}));
                connect(l2sigma.plug_p, Rot22.plug_p) annotation(
                  Line(points = {{50, 0}, {50, 20}}, color = {85, 170, 255}));
                connect(Rot22.plug_n, core.plug_n2) annotation(
                  Line(points = {{30, 20}, {20, 20}, {20, 4}, {10, 4}}, color = {85, 170, 255}));
                connect(core.plug_n3, star2.plug_p) annotation(
                  Line(points = {{10, -10}, {10, -70}, {10, -70}}, color = {85, 170, 255}));
                annotation(
                  defaultComponentName = "transformer",
                  Documentation(info = "<html>
Transformer Dz0
<br>Typical parameters see:
<a href=\"modelica://Modelica.Electrical.Machines.Interfaces.PartialBasicTransformer\">PartialBasicTransformer</a>
</html>"));
              end Dz00;

              model Dz02 "Transformer Dz2"
                extends QuasiStationary.Machines.Interfaces.PartialBasicTransformer(final VectorGroup = "Dz02");
                QuasiStationary.MultiPhase.Basic.Star star2(final m = m) annotation(
                  Placement(transformation(origin = {10, -80}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
                QuasiStationary.SinglePhase.Interfaces.NegativePin starpoint2 annotation(
                  Placement(transformation(extent = {{40, -110}, {60, -90}}, rotation = 0)));
                QuasiStationary.MultiPhase.Basic.Delta Rot21(final m = m) annotation(
                  Placement(transformation(origin = {30, 0}, extent = {{10, -10}, {-10, 10}}, rotation = 90)));
                QuasiStationary.MultiPhase.Basic.Delta Rot22(final m = m) annotation(
                  Placement(transformation(origin = {40, 20}, extent = {{-10, 10}, {10, -10}}, rotation = 180)));
                QuasiStationary.MultiPhase.Basic.Delta Delta1(final m = m) annotation(
                  Placement(transformation(extent = {{-50, -30}, {-30, -10}}, rotation = 0)));
              equation
                connect(star2.pin_n, starpoint2) annotation(
                  Line(points = {{10, -90}, {10, -100}, {50, -100}}, color = {85, 170, 255}));
                connect(r1.plug_p, Delta1.plug_p) annotation(
                  Line(points = {{-90, 0}, {-90, -20}, {-50, -20}}, color = {85, 170, 255}));
                connect(l1sigma.plug_n, core.plug_p1) annotation(
                  Line(points = {{-50, 0}, {-50, 20}, {-10, 20}, {-10, 5}}, color = {85, 170, 255}));
                connect(Delta1.plug_n, core.plug_n1) annotation(
                  Line(points = {{-30, -20}, {-10, -20}, {-10, -5}}, color = {85, 170, 255}));
                connect(core.plug_p2, Rot21.plug_p) annotation(
                  Line(points = {{10, 10}, {30, 10}}, color = {85, 170, 255}));
                connect(core.plug_p3, Rot21.plug_n) annotation(
                  Line(points = {{10, -4}, {20, -4}, {20, -10}, {30, -10}}, color = {85, 170, 255}));
                connect(l2sigma.plug_p, Rot22.plug_p) annotation(
                  Line(points = {{50, 0}, {50, 20}}, color = {85, 170, 255}));
                connect(Rot22.plug_n, core.plug_n2) annotation(
                  Line(points = {{30, 20}, {20, 20}, {20, 4}, {10, 4}}, color = {85, 170, 255}));
                connect(core.plug_n3, star2.plug_p) annotation(
                  Line(points = {{10, -10}, {10, -40}, {10, -70}, {10, -70}}, color = {85, 170, 255}));
                annotation(
                  defaultComponentName = "transformer",
                  Documentation(info = "<html>
Transformer Dz2
<br>Typical parameters see:
<a href=\"modelica://Modelica.Electrical.Machines.Interfaces.PartialBasicTransformer\">PartialBasicTransformer</a>
</html>"));
              end Dz02;

              model Dz04 "Transformer Dz4"
                extends QuasiStationary.Machines.Interfaces.PartialBasicTransformer(final VectorGroup = "Dz04");
                QuasiStationary.MultiPhase.Basic.Star star2(final m = m) annotation(
                  Placement(transformation(origin = {10, -80}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
                QuasiStationary.SinglePhase.Interfaces.NegativePin starpoint2 annotation(
                  Placement(transformation(extent = {{40, -110}, {60, -90}}, rotation = 0)));
                QuasiStationary.MultiPhase.Basic.Delta Rot21(final m = m) annotation(
                  Placement(transformation(origin = {30, 0}, extent = {{-10, 10}, {10, -10}}, rotation = 90)));
                QuasiStationary.MultiPhase.Basic.Delta Delta1(final m = m) annotation(
                  Placement(transformation(extent = {{-50, -30}, {-30, -10}}, rotation = 0)));
              equation
                connect(star2.pin_n, starpoint2) annotation(
                  Line(points = {{10, -90}, {10, -100}, {50, -100}}, color = {85, 170, 255}));
                connect(r1.plug_p, Delta1.plug_p) annotation(
                  Line(points = {{-90, 0}, {-90, -20}, {-50, -20}}, color = {85, 170, 255}));
                connect(l1sigma.plug_n, core.plug_p1) annotation(
                  Line(points = {{-50, 0}, {-50, 20}, {-10, 20}, {-10, 5}}, color = {85, 170, 255}));
                connect(Delta1.plug_n, core.plug_n1) annotation(
                  Line(points = {{-30, -20}, {-10, -20}, {-10, -5}}, color = {85, 170, 255}));
                connect(core.plug_p2, Rot21.plug_n) annotation(
                  Line(points = {{10, 10}, {30, 10}}, color = {85, 170, 255}));
                connect(core.plug_p3, Rot21.plug_p) annotation(
                  Line(points = {{10, -4}, {20, -4}, {20, -10}, {30, -10}}, color = {85, 170, 255}));
                connect(l2sigma.plug_p, core.plug_n2) annotation(
                  Line(points = {{50, 0}, {50, 20}, {20, 20}, {20, 4}, {10, 4}}, color = {85, 170, 255}));
                connect(core.plug_n3, star2.plug_p) annotation(
                  Line(points = {{10, -10}, {10, -70}, {10, -70}}, color = {85, 170, 255}));
                annotation(
                  defaultComponentName = "transformer",
                  Documentation(info = "<html>
Transformer Dz4
<br>Typical parameters see:
<a href=\"modelica://Modelica.Electrical.Machines.Interfaces.PartialBasicTransformer\">PartialBasicTransformer</a>
</html>"));
              end Dz04;

              model Dz06 "Transformer Dz6"
                extends QuasiStationary.Machines.Interfaces.PartialBasicTransformer(final VectorGroup = "Dz06");
                QuasiStationary.MultiPhase.Basic.Star star2(final m = m) annotation(
                  Placement(transformation(origin = {10, -80}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
                QuasiStationary.SinglePhase.Interfaces.NegativePin starpoint2 annotation(
                  Placement(transformation(extent = {{40, -110}, {60, -90}}, rotation = 0)));
                QuasiStationary.MultiPhase.Basic.Delta Rot21(final m = m) annotation(
                  Placement(transformation(origin = {30, 0}, extent = {{10, -10}, {-10, 10}}, rotation = 90)));
                QuasiStationary.MultiPhase.Basic.Delta Delta1(final m = m) annotation(
                  Placement(transformation(extent = {{-50, -30}, {-30, -10}}, rotation = 0)));
              equation
                connect(star2.pin_n, starpoint2) annotation(
                  Line(points = {{10, -90}, {10, -100}, {50, -100}}, color = {85, 170, 255}));
                connect(r1.plug_p, Delta1.plug_p) annotation(
                  Line(points = {{-90, 0}, {-90, -20}, {-50, -20}}, color = {85, 170, 255}));
                connect(l1sigma.plug_n, core.plug_p1) annotation(
                  Line(points = {{-50, 0}, {-50, 20}, {-10, 20}, {-10, 5}}, color = {85, 170, 255}));
                connect(Delta1.plug_n, core.plug_n1) annotation(
                  Line(points = {{-30, -20}, {-10, -20}, {-10, -5}}, color = {85, 170, 255}));
                connect(core.plug_p2, Rot21.plug_p) annotation(
                  Line(points = {{10, 10}, {30, 10}}, color = {85, 170, 255}));
                connect(core.plug_p3, Rot21.plug_n) annotation(
                  Line(points = {{10, -4}, {20, -4}, {20, -10}, {30, -10}}, color = {85, 170, 255}));
                connect(core.plug_n3, star2.plug_p) annotation(
                  Line(points = {{10, -10}, {10, -40}, {10, -70}, {10, -70}}, color = {85, 170, 255}));
                connect(l2sigma.plug_p, core.plug_n2) annotation(
                  Line(points = {{50, 0}, {50, 20}, {20, 20}, {20, 4}, {10, 4}}, color = {85, 170, 255}));
                annotation(
                  defaultComponentName = "transformer",
                  Documentation(info = "<html>
Transformer Dz6
<br>Typical parameters see:
<a href=\"modelica://Modelica.Electrical.Machines.Interfaces.PartialBasicTransformer\">PartialBasicTransformer</a>
</html>"));
              end Dz06;

              model Dz08 "Transformer Dz8"
                extends QuasiStationary.Machines.Interfaces.PartialBasicTransformer(final VectorGroup = "Dz08");
                QuasiStationary.MultiPhase.Basic.Star star2(final m = m) annotation(
                  Placement(transformation(origin = {10, -80}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
                QuasiStationary.SinglePhase.Interfaces.NegativePin starpoint2 annotation(
                  Placement(transformation(extent = {{40, -110}, {60, -90}}, rotation = 0)));
                QuasiStationary.MultiPhase.Basic.Delta Rot21(final m = m) annotation(
                  Placement(transformation(origin = {30, 0}, extent = {{-10, 10}, {10, -10}}, rotation = 90)));
                QuasiStationary.MultiPhase.Basic.Delta Rot22(final m = m) annotation(
                  Placement(transformation(origin = {40, 20}, extent = {{10, 10}, {-10, -10}}, rotation = 180)));
                QuasiStationary.MultiPhase.Basic.Delta Delta1(final m = m) annotation(
                  Placement(transformation(extent = {{-50, -30}, {-30, -10}}, rotation = 0)));
              equation
                connect(star2.pin_n, starpoint2) annotation(
                  Line(points = {{10, -90}, {10, -100}, {50, -100}}, color = {85, 170, 255}));
                connect(r1.plug_p, Delta1.plug_p) annotation(
                  Line(points = {{-90, 0}, {-90, -20}, {-50, -20}}, color = {85, 170, 255}));
                connect(l1sigma.plug_n, core.plug_p1) annotation(
                  Line(points = {{-50, 0}, {-50, 20}, {-10, 20}, {-10, 5}, {-10, 5}}, color = {85, 170, 255}));
                connect(Delta1.plug_n, core.plug_n1) annotation(
                  Line(points = {{-30, -20}, {-10, -20}, {-10, -5}}, color = {85, 170, 255}));
                connect(core.plug_p2, Rot21.plug_n) annotation(
                  Line(points = {{10, 10}, {30, 10}}, color = {85, 170, 255}));
                connect(core.plug_p3, Rot21.plug_p) annotation(
                  Line(points = {{10, -4}, {20, -4}, {20, -10}, {30, -10}}, color = {85, 170, 255}));
                connect(core.plug_n3, star2.plug_p) annotation(
                  Line(points = {{10, -10}, {10, -70}, {10, -70}}, color = {85, 170, 255}));
                connect(l2sigma.plug_p, Rot22.plug_n) annotation(
                  Line(points = {{50, 0}, {50, 20}, {50, 20}}, color = {85, 170, 255}));
                connect(Rot22.plug_p, core.plug_n2) annotation(
                  Line(points = {{30, 20}, {20, 20}, {20, 4}, {10, 4}}, color = {85, 170, 255}));
                annotation(
                  defaultComponentName = "transformer",
                  Documentation(info = "<html>
Transformer Dz8
<br>Typical parameters see:
<a href=\"modelica://Modelica.Electrical.Machines.Interfaces.PartialBasicTransformer\">PartialBasicTransformer</a>
</html>"));
              end Dz08;

              model Dz10 "Transformer Dz10"
                extends QuasiStationary.Machines.Interfaces.PartialBasicTransformer(final VectorGroup = "Dz10");
                QuasiStationary.MultiPhase.Basic.Star star2(final m = m) annotation(
                  Placement(transformation(origin = {10, -80}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
                QuasiStationary.SinglePhase.Interfaces.NegativePin starpoint2 annotation(
                  Placement(transformation(extent = {{40, -110}, {60, -90}}, rotation = 0)));
                QuasiStationary.MultiPhase.Basic.Delta Rot21(final m = m) annotation(
                  Placement(transformation(origin = {30, 0}, extent = {{10, -10}, {-10, 10}}, rotation = 90)));
                QuasiStationary.MultiPhase.Basic.Delta Rot22(final m = m) annotation(
                  Placement(transformation(origin = {40, 20}, extent = {{10, 10}, {-10, -10}}, rotation = 180)));
                QuasiStationary.MultiPhase.Basic.Delta Delta1(final m = m) annotation(
                  Placement(transformation(extent = {{-50, -30}, {-30, -10}}, rotation = 0)));
              equation
                connect(star2.pin_n, starpoint2) annotation(
                  Line(points = {{10, -90}, {10, -100}, {50, -100}}, color = {85, 170, 255}));
                connect(r1.plug_p, Delta1.plug_p) annotation(
                  Line(points = {{-90, 0}, {-90, -20}, {-50, -20}}, color = {85, 170, 255}));
                connect(l1sigma.plug_n, core.plug_p1) annotation(
                  Line(points = {{-50, 0}, {-50, 20}, {-10, 20}, {-10, 5}}, color = {85, 170, 255}));
                connect(Delta1.plug_n, core.plug_n1) annotation(
                  Line(points = {{-30, -20}, {-10, -20}, {-10, -5}}, color = {85, 170, 255}));
                connect(core.plug_p2, Rot21.plug_p) annotation(
                  Line(points = {{10, 10}, {30, 10}}, color = {85, 170, 255}));
                connect(core.plug_p3, Rot21.plug_n) annotation(
                  Line(points = {{10, -4}, {20, -4}, {20, -10}, {30, -10}}, color = {85, 170, 255}));
                connect(core.plug_n3, star2.plug_p) annotation(
                  Line(points = {{10, -10}, {10, -70}, {10, -70}}, color = {85, 170, 255}));
                connect(l2sigma.plug_p, Rot22.plug_n) annotation(
                  Line(points = {{50, 0}, {50, 20}}, color = {85, 170, 255}));
                connect(Rot22.plug_p, core.plug_n2) annotation(
                  Line(points = {{30, 20}, {20, 20}, {20, 4}, {10, 4}}, color = {85, 170, 255}));
                annotation(
                  defaultComponentName = "transformer",
                  Documentation(info = "<html>
Transformer Dz10
<br>Typical parameters see:
<a href=\"modelica://Modelica.Electrical.Machines.Interfaces.PartialBasicTransformer\">PartialBasicTransformer</a>
</html>"));
              end Dz10;
              annotation(
                Documentation(info = "<HTML>
This package contains transformers primary D connected / secondary d connected in all possible vector groups.
</HTML>", revisions = "<HTML>
<dl>
  <dt><b>Main Authors:</b></dt>
  <dd>
  <a href=\"http://www.haumer.at/\">Anton Haumer</a><br>
  Technical Consulting &amp; Electrical Engineering<br>
  A-3423 St.Andrae-Woerdern<br>Austria<br>
  email: <a href=\"mailto:a.haumer@haumer.at\">a.haumer@haumer.at</a>
  </dd>
  <dt><b>Copyright:</b></dt>
  <dd>Copyright &copy; 1998-2015, Modelica Association and Anton Haumer.<br>
  <i>The Modelica package is <b>free</b> software; it can be redistributed and/or modified
  under the terms of the <b>Modelica license</b>, see the license conditions
  and the accompanying <b>disclaimer</b> in the documentation of package
  Modelica in file \"Modelica/package.mo\".</i></dd>
</dl>
  <ul>
  <li> v1.0.0 2006/11/19 Anton Haumer</li>
  </ul>
</HTML>"));
            end Dz;
            annotation(
              Documentation(info = "<HTML>
This package contains components to model technical three-phase transformers:
<ul>
<li>Transformer: transformer model to choose connection / vector group</li>
<li>Yy: Transformers with primary primary Y / secondary y</li>
<li>Yd: Transformers with primary primary Y / secondary d</li>
<li>Yz: Transformers with primary primary Y / secondary zig-zag</li>
<li>Dy: Transformers with primary primary D / secondary y</li>
<li>Dd: Transformers with primary D / secondary d</li>
<li>Dz: Transformers with primary D / secondary zig-zag</li>
</ul>
<p>
Transformers are modeled by an ideal transformer, adding primary and secondary winding resistances and stray inductances.<br>
All transformers extend from the base model <i>PartialTransformer</i>, adding the primary and secondary connection.<br>
<b>VectorGroup</b> defines the phase shift between primary and secondary voltages, expressed by a number phase shift/30 degree
(i.e., the hour on a clock face). Therefore each transformer is identified by two characters and a two-digit number,
e.g., Yd11 ... primary connection Y (star), secondary connection d (delta), vector group 11 (phase shift 330 degree)<br>
With the \"supermodel\" <i>Transformer</i>&nbsp; the user may choose primary and secondary connection as well as the vector group.<br>
It calculates winding ratio as well as primary and secondary winding resistances and stray inductances,
distributing them equally to primary and secondary winding, from the following parameters:
</p>
<ul>
<li>nominal frequency</li>
<li>primary voltage (RMS line-to-line)</li>
<li>secondary voltage (RMS line-to-line)</li>
<li>nominal apparent power</li>
<li>impedance voltage drop</li>
<li>short-circuit copper losses</li>
</ul>
The <b>impedance voltage drop</b> indicates the (absolute value of the) voltage drop at nominal load (current) as well as
the voltage we have to apply to the primary winding to achieve nominal current in the short-circuited secondary winding.
<p>
<b>Please pay attention</b> to proper grounding of the primary and secondary part of the whole circuit.<br>
The primary and secondary starpoint are available as connectors, if the connection is not delta (D or d).<br>
<b>In some cases (Yy or Yz) it may be necessary to ground one of the transformer's starpoints
even though the source's and/or load's starpoint are grounded; you may use a reasonable high earthing resistance.</b>
</p>
<b>Limitations and assumptions:</b><br>
<ul>
<li>number of phases is limited to 3, therefore definition as a constant m=3</li>
<li>symmetry of the 3 phases resp. limbs</li>
<li>saturation is neglected, i.e., inductances are constant</li>
<li>magnetizing current is neglected</li>
<li>magnetizing losses are neglected</li>
<li>additional (stray) losses are neglected</li>
</ul>
<b>Further development:</b>
<ul>
<li>modeling magnetizing current, including saturation</li>
<li>temperature dependency of winding resistances</li>
</ul>
<dl>
  <dt><b>Main Authors:</b></dt>
  <dd>
  <a href=\"http://www.haumer.at/\">Anton Haumer</a><br>
  Technical Consulting &amp; Electrical Engineering<br>
  A-3423 St.Andrae-Woerdern<br>Austria<br>
  email: <a href=\"mailto:a.haumer@haumer.at\">a.haumer@haumer.at</a>
  </dd>
</dl>
<p>
Copyright &copy; 1998-2015, Modelica Association and Anton Haumer.
</p>
<p>
<i>This Modelica package is <u>free</u> software and the use is completely at <u>your own risk</u>; it can be redistributed and/or modified under the terms of the Modelica License 2. For license conditions (including the disclaimer of warranty) see <a href=\"modelica://Modelica.UsersGuide.ModelicaLicense2\">Modelica.UsersGuide.ModelicaLicense2</a> or visit <a href=\"https://www.modelica.org/licenses/ModelicaLicense2\"> https://www.modelica.org/licenses/ModelicaLicense2</a>.</i>
</p>
</HTML>", revisions = "<HTML>
  <ul>
  <li> v1.0.0 2006/11/19 Anton Haumer<br>
       first stable release</li>
  <li> v2.2.0 2011/02/10 Anton Haumer<br>
       conditional ThermalPort for all machines</li>
  </ul>
</HTML>"),
              Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(origin = {10, 10}, fillColor = {135, 135, 135}, fillPattern = FillPattern.VerticalCylinder, points = {{-80, 50}, {-60, 30}, {-60, -50}, {-80, -70}, {-80, 50}}), Polygon(origin = {10, 10}, fillColor = {135, 135, 135}, fillPattern = FillPattern.VerticalCylinder, points = {{60, 50}, {40, 30}, {40, -50}, {60, -70}, {60, 50}}), Polygon(origin = {10, 10}, fillColor = {135, 135, 135}, fillPattern = FillPattern.VerticalCylinder, points = {{-10, 40}, {-20, 30}, {-20, -50}, {-10, -60}, {0, -50}, {0, 30}, {-10, 40}}), Polygon(origin = {10, 10}, fillColor = {135, 135, 135}, fillPattern = FillPattern.VerticalCylinder, points = {{-80, 50}, {60, 50}, {40, 30}, {0, 30}, {-10, 40}, {-20, 30}, {-60, 30}, {-80, 50}}), Polygon(origin = {10, 10}, fillColor = {135, 135, 135}, fillPattern = FillPattern.VerticalCylinder, points = {{-80, -70}, {60, -70}, {40, -50}, {0, -50}, {-10, -60}, {-20, -50}, {-60, -50}, {-80, -70}}), Rectangle(origin = {10, 10}, lineColor = {213, 170, 255}, fillColor = {213, 170, 255}, fillPattern = FillPattern.VerticalCylinder, extent = {{-88, -46}, {-52, 26}}), Rectangle(origin = {10, 10}, lineColor = {170, 213, 255}, fillColor = {170, 213, 255}, fillPattern = FillPattern.VerticalCylinder, extent = {{-94, -38}, {-46, 18}}), Rectangle(origin = {10, 10}, lineColor = {213, 170, 255}, fillColor = {213, 170, 255}, fillPattern = FillPattern.VerticalCylinder, extent = {{-28, -46}, {8, 26}}), Rectangle(origin = {10, 10}, lineColor = {170, 213, 255}, fillColor = {170, 213, 255}, fillPattern = FillPattern.VerticalCylinder, extent = {{-34, -38}, {14, 18}}), Rectangle(origin = {10, 10}, lineColor = {213, 170, 255}, fillColor = {213, 170, 255}, fillPattern = FillPattern.VerticalCylinder, extent = {{32, -46}, {68, 26}}), Rectangle(origin = {10, 10}, lineColor = {170, 213, 255}, fillColor = {170, 213, 255}, fillPattern = FillPattern.VerticalCylinder, extent = {{26, -38}, {74, 18}})}));
          end Transformers;

          package Components "Machine components like AirGaps"
            extends Modelica.Icons.Package;

            partial model PartialCore "Partial model of transformer core with 3 windings"
              parameter Integer m(final min = 1) = 3 "Number of phases";
              parameter Real n12(start = 1) "Turns ratio 1:2";
              parameter Real n13(start = 1) "Turns ratio 1:3";
              Modelica.SIunits.ComplexVoltage v1[m];
              Modelica.SIunits.ComplexCurrent i1[m];
              Modelica.SIunits.ComplexVoltage v2[m];
              Modelica.SIunits.ComplexCurrent i2[m];
              Modelica.SIunits.ComplexVoltage v3[m];
              Modelica.SIunits.ComplexCurrent i3[m];
              Modelica.SIunits.ComplexCurrent im[m] "Magnetizing current";
              QuasiStationary.MultiPhase.Interfaces.PositivePlug plug_p1(final m = m) annotation(
                Placement(transformation(extent = {{-110, 40}, {-90, 60}}, rotation = 0)));
              QuasiStationary.MultiPhase.Interfaces.NegativePlug plug_n1(final m = m) annotation(
                Placement(transformation(extent = {{-110, -60}, {-90, -40}}, rotation = 0)));
              QuasiStationary.MultiPhase.Interfaces.PositivePlug plug_p2(final m = m) annotation(
                Placement(transformation(extent = {{90, 90}, {110, 110}}, rotation = 0)));
              QuasiStationary.MultiPhase.Interfaces.NegativePlug plug_n2(final m = m) annotation(
                Placement(transformation(extent = {{90, 30}, {110, 50}}, rotation = 0)));
              QuasiStationary.MultiPhase.Interfaces.PositivePlug plug_p3(final m = m) annotation(
                Placement(transformation(extent = {{90, -30}, {110, -50}}, rotation = 0)));
              QuasiStationary.MultiPhase.Interfaces.NegativePlug plug_n3(final m = m) annotation(
                Placement(transformation(extent = {{90, -90}, {110, -110}}, rotation = 0)));
            equation
              v1 = plug_p1.pin.v - plug_n1.pin.v;
              i1 = plug_p1.pin.i;
              v2 = plug_p2.pin.v - plug_n2.pin.v;
              i2 = plug_p2.pin.i;
              v3 = plug_p3.pin.v - plug_n3.pin.v;
              i3 = plug_p3.pin.i;
              im = i1 + i2 / n12 + i3 / n13;
//branches p1-n1, p2-n2, p3-n3
              Connections.branch(plug_p1.reference, plug_n1.reference);
              plug_p1.reference.gamma = plug_n1.reference.gamma;
              Connections.branch(plug_p2.reference, plug_n2.reference);
              plug_p2.reference.gamma = plug_n2.reference.gamma;
              Connections.branch(plug_p3.reference, plug_n3.reference);
              plug_p3.reference.gamma = plug_n3.reference.gamma;
//Define p1.reference.gamme = p2.reference.gamma = p3.reference.gamma
              Connections.branch(plug_p1.reference, plug_p2.reference);
              plug_p1.reference.gamma = plug_p2.reference.gamma;
              Connections.branch(plug_p1.reference, plug_p3.reference);
              plug_p1.reference.gamma = plug_p3.reference.gamma;
//Define p1, p2 and p3 as potential roots
//Note: transformer could be fed from primary or secondary side
              Connections.potentialRoot(plug_p1.reference);
              Connections.potentialRoot(plug_p2.reference);
              Connections.potentialRoot(plug_p3.reference);
//Current balances
              plug_p1.pin.i + plug_n1.pin.i = fill(Complex(0), m);
              plug_p2.pin.i + plug_n2.pin.i = fill(Complex(0), m);
              plug_p3.pin.i + plug_n3.pin.i = fill(Complex(0), m);
              annotation(
                Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-100, 130}, {100, 110}}, textString = "%name", lineColor = {0, 0, 255}), Ellipse(extent = {{-45, -50}, {-20, -25}}, lineColor = {0, 0, 255}), Ellipse(extent = {{-45, -25}, {-20, 0}}, lineColor = {0, 0, 255}), Ellipse(extent = {{-45, 0}, {-20, 25}}, lineColor = {0, 0, 255}), Ellipse(extent = {{-45, 25}, {-20, 50}}, lineColor = {0, 0, 255}), Rectangle(extent = {{-46, -50}, {-34, 50}}, lineColor = {255, 255, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{-90, 50}, {-32, 50}}, color = {0, 0, 255}), Line(points = {{-90, -50}, {-32, -50}}, color = {0, 0, 255}), Ellipse(extent = {{20, 40}, {45, 65}}, lineColor = {0, 0, 255}), Ellipse(extent = {{20, 65}, {45, 90}}, lineColor = {0, 0, 255}), Rectangle(extent = {{36, 40}, {46, 90}}, lineColor = {255, 255, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{32, 90}, {90, 90}}, color = {0, 0, 255}), Line(points = {{34, 40}, {92, 40}}, color = {0, 0, 255}), Ellipse(extent = {{20, -90}, {45, -65}}, lineColor = {0, 0, 255}), Ellipse(extent = {{20, -65}, {45, -40}}, lineColor = {0, 0, 255}), Rectangle(extent = {{36, -90}, {46, -40}}, lineColor = {255, 255, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{32, -40}, {90, -40}}, color = {0, 0, 255}), Line(points = {{34, -90}, {92, -90}}, color = {0, 0, 255})}),
                Documentation(info = "<html>
Partial model of transformer core with 3 windings; saturation function flux versus magnetizing current has to be defined.
</html>"));
            end PartialCore;

            model IdealCore "Ideal transformer with 3 windings"
              extends PartialCore;
            equation
              im = fill(Complex(0), m);
              v1 = n12 * v2;
              v1 = n13 * v3;
              annotation(
                defaultComponentName = "core",
                Documentation(info = "<html>
Ideal transformer with 3 windings: no magnetizing current.
</html>"));
            end IdealCore;
            annotation(
              Documentation(info = "<html>
<p>
This package contains components for modeling electrical machines, specially three-phase induction machines, based on space phasor theory.
</p>
</html>"));
          end Components;
          annotation(
            Icon(graphics = {Rectangle(origin = {0, 14.817}, fillColor = {170, 213, 255}, fillPattern = FillPattern.HorizontalCylinder, extent = {{-54.179, -59.817}, {65.821, 60.183}}), Rectangle(origin = {5.821, 15}, fillColor = {128, 128, 128}, fillPattern = FillPattern.HorizontalCylinder, extent = {{-80, -60}, {-60, 60}}), Rectangle(origin = {5.821, 15}, fillColor = {95, 95, 95}, fillPattern = FillPattern.HorizontalCylinder, extent = {{60, -10}, {80, 10}}), Rectangle(origin = {5.821, 15}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid, extent = {{-60, 50}, {20, 70}}), Polygon(origin = {5.821, 15}, fillPattern = FillPattern.Solid, points = {{-70, -90}, {-60, -90}, {-30, -20}, {20, -20}, {50, -90}, {60, -90}, {60, -100}, {-70, -100}, {-70, -90}})}),
            Documentation(info = "<html>
This package contains components for modeling quasi stationary electrical induction machines machines, based on space phasor theory:
<ul>
<li>package AsynchronousInductionMachines: quasi stationary models of three phase asynchronous induction machines</li>
<li>package SynchronousInductionMachines: quasi stationary models of three phase synchronous induction machines</li>
<li>package Transformers: quasi stationary three-phase transformers (see detailed documentation in subpackage)</li>
<li>package Components: components for quasi stationary modeling machines and transformers</li>
</ul>
</html>"));
        end BasicMachines;

        package Interfaces "SpacePhasor connector and PartialMachines"
          extends Modelica.Icons.InterfacesPackage;

          partial model PartialBasicTransformer "Partial model of three-phase transformer"
            extends Modelica.Electrical.Machines.Icons.QuasiStationaryTransformer;
            constant Integer m(min = 1) = 3 "Number of phases";
            constant String VectorGroup = "Yy00";
            parameter Real n(start = 1) "Ratio primary voltage (line-to-line) / secondary voltage (line-to-line)";
            parameter Modelica.SIunits.Resistance R1(start = 5E-3 / (if C1 == "D" then 1 else 3)) "Primary resistance per phase at TRef" annotation(
              Dialog(tab = "Nominal resistances and inductances"));
            parameter Modelica.SIunits.Temperature T1Ref(start = 293.15) "Reference temperature of primary resistance" annotation(
              Dialog(tab = "Nominal resistances and inductances"));
            parameter Modelica.Electrical.Machines.Thermal.LinearTemperatureCoefficient20 alpha20_1(start = 0) "Temperature coefficient of primary resistance at 20 degC" annotation(
              Dialog(tab = "Nominal resistances and inductances"));
            parameter Modelica.SIunits.Inductance L1sigma(start = 78E-6 / (if C1 == "D" then 1 else 3)) "Primary stray inductance per phase" annotation(
              Dialog(tab = "Nominal resistances and inductances"));
            parameter Modelica.SIunits.Resistance R2(start = 5E-3 / (if C2 == "d" then 1 else 3)) "Secondary resistance per phase at TRef" annotation(
              Dialog(tab = "Nominal resistances and inductances"));
            parameter Modelica.SIunits.Temperature T2Ref(start = 293.15) "Reference temperature of secondary resistance" annotation(
              Dialog(tab = "Nominal resistances and inductances"));
            parameter Modelica.Electrical.Machines.Thermal.LinearTemperatureCoefficient20 alpha20_2(start = 0) "Temperature coefficient of secondary resistance at 20 degC" annotation(
              Dialog(tab = "Nominal resistances and inductances"));
            parameter Modelica.SIunits.Inductance L2sigma(start = 78E-6 / (if C2 == "d" then 1 else 3)) "Secondary stray inductance per phase" annotation(
              Dialog(tab = "Nominal resistances and inductances"));
            parameter Boolean useThermalPort = false "Enable / disable (=fixed temperatures) thermal port" annotation(
              Evaluate = true);
            parameter Modelica.SIunits.Temperature T1Operational(start = 293.15) "Operational temperature of primary resistance" annotation(
              Dialog(group = "Operational temperatures", enable = not useThermalPort));
            parameter Modelica.SIunits.Temperature T2Operational(start = 293.15) "Operational temperature of secondary resistance" annotation(
              Dialog(group = "Operational temperatures", enable = not useThermalPort));
            output Modelica.Electrical.Machines.Interfaces.PowerBalanceTransformer powerBalance(final power1 = sum({Modelica.ComplexMath.real(v1[j] * Modelica.ComplexMath.conj(i1[j])) for j in 1:m}), final power2 = sum({Modelica.ComplexMath.real(v2[j] * Modelica.ComplexMath.conj(-i2[j])) for j in 1:m}), final lossPower1 = -sum(r1.resistor.LossPower), final lossPower2 = -sum(r2.resistor.LossPower), final lossPowerCore = 0) "Power balance";
            output Modelica.SIunits.ComplexVoltage v1[m] = plug1.pin.v "Primary voltage";
            output Modelica.SIunits.ComplexCurrent i1[m] = plug1.pin.i "Primary current";
            output Modelica.SIunits.ComplexVoltage v2[m] = plug2.pin.v "Secondary voltage";
            output Modelica.SIunits.ComplexCurrent i2[m] = plug2.pin.i "Secondary current";
          protected
            constant String C1 = Modelica.Utilities.Strings.substring(VectorGroup, 1, 1);
            constant String C2 = Modelica.Utilities.Strings.substring(VectorGroup, 2, 2);
            parameter Real ni = n * (if C2 == "z" then sqrt(3) else 2) * (if C2 == "d" then 1 else sqrt(3)) / (if C1 == "D" then 1 else sqrt(3));
          public
            QuasiStationary.MultiPhase.Interfaces.PositivePlug plug1(final m = m) "Primary plug" annotation(
              Placement(transformation(extent = {{-110, -10}, {-90, 10}}, rotation = 0)));
            QuasiStationary.MultiPhase.Interfaces.NegativePlug plug2(final m = m) "Secondary plug" annotation(
              Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
            QuasiStationary.MultiPhase.Basic.Resistor r1(final m = m, final R_ref = fill(R1, m), final T_ref = fill(T1Ref, m), final alpha_ref = fill(Modelica.Electrical.Machines.Thermal.convertAlpha(alpha20_1, T1Ref), m), final useHeatPort = true, final T = fill(T1Ref, m)) annotation(
              Placement(transformation(extent = {{-90, 10}, {-70, -10}}, rotation = 0)));
            QuasiStationary.MultiPhase.Basic.Inductor l1sigma(final m = m, final L = fill(L1sigma, m)) annotation(
              Placement(transformation(extent = {{-70, -10}, {-50, 10}}, rotation = 0)));
            QuasiStationary.MultiPhase.Basic.Resistor r2(final m = m, final R_ref = fill(R2, m), final T_ref = fill(T2Ref, m), final alpha_ref = fill(Modelica.Electrical.Machines.Thermal.convertAlpha(alpha20_2, T2Ref), m), final useHeatPort = true, final T = fill(T2Ref, m)) annotation(
              Placement(transformation(extent = {{70, 10}, {90, -10}}, rotation = 0)));
            QuasiStationary.MultiPhase.Basic.Inductor l2sigma(final m = m, final L = fill(L2sigma, m)) annotation(
              Placement(transformation(extent = {{50, -10}, {70, 10}}, rotation = 0)));
            QuasiStationary.Machines.BasicMachines.Components.IdealCore core(final m = m, final n12 = ni, final n13 = ni) annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0)));
            Modelica.Electrical.Machines.Interfaces.ThermalPortTransformer thermalPort if useThermalPort annotation(
              Placement(transformation(extent = {{-10, 90}, {10, 110}})));
            Modelica.Electrical.Machines.Thermal.ThermalAmbientTransformer thermalAmbient(final useTemperatureInputs = false, final T1 = T1Operational, final T2 = T2Operational) if not useThermalPort annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 270, origin = {-30, 80})));
          protected
            Modelica.Electrical.Machines.Interfaces.ThermalPortTransformer internalThermalPort annotation(
              Placement(transformation(extent = {{-4, 76}, {4, 84}})));
          equation
            connect(r1.plug_n, l1sigma.plug_p) annotation(
              Line(points = {{-70, 0}, {-70, 0}}, color = {85, 170, 255}));
            connect(l2sigma.plug_n, r2.plug_p) annotation(
              Line(points = {{70, 0}, {70, 0}}, color = {85, 170, 255}));
            connect(plug1, r1.plug_p) annotation(
              Line(points = {{-100, 0}, {-90, 0}}, color = {85, 170, 255}));
            connect(r2.plug_n, plug2) annotation(
              Line(points = {{90, 0}, {100, 0}}, color = {85, 170, 255}));
            connect(thermalPort, internalThermalPort) annotation(
              Line(points = {{0, 100}, {0, 80}}, color = {199, 0, 0}, smooth = Smooth.None));
            connect(thermalAmbient.thermalPort, internalThermalPort) annotation(
              Line(points = {{-20, 80}, {0, 80}}, color = {199, 0, 0}, smooth = Smooth.None));
            connect(r1.heatPort, internalThermalPort.heatPort1) annotation(
              Line(points = {{-80, 10}, {-80, 60}, {0, 60}, {0, 80}}, color = {191, 0, 0}, smooth = Smooth.None));
            connect(r2.heatPort, internalThermalPort.heatPort2) annotation(
              Line(points = {{80, 10}, {80, 60}, {0, 60}, {0, 80}}, color = {191, 0, 0}, smooth = Smooth.None));
            annotation(
              Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{0, -60}, {0, -100}}, lineColor = {0, 0, 0}, textString = "%VectorGroup"), Text(extent = {{0, 100}, {0, 60}}, lineColor = {0, 0, 255}, textString = "%name")}),
              Documentation(info = "<html>
Partial model of a three-phase transformer, containing primary and secondary resistances and stray inductances, as well as the iron core.
Circuit layout (vector group) of primary and secondary windings have to be defined.
<br><b>Default values for transformer's parameters (a realistic example) are:</b><br>
<table>
<tr>
<td valign=\"top\">turns ratio n</td>
<td valign=\"top\">1</td><td valign=\"top\"> </td>
</tr>
<tr>
<td valign=\"top\">nominal frequency fNominal</td>
<td valign=\"top\">50</td><td valign=\"top\">Hz</td>
</tr>
<tr>
<td valign=\"top\">nominal voltage per phase</td>
<td valign=\"top\">100</td><td valign=\"top\">V RMS</td>
</tr>
<tr>
<td valign=\"top\">nominal current per phase</td>
<td valign=\"top\">100</td><td valign=\"top\">A RMS</td>
</tr>
<tr>
<td valign=\"top\">nominal apparent power</td>
<td valign=\"top\">30</td><td valign=\"top\">kVA</td>
</tr>
<tr>
<td valign=\"top\">primary resistance R1</td>
<td valign=\"top\">0.005</td><td valign=\"top\">Ohm per phase at reference temperature</td>
</tr>
<tr>
<td valign=\"top\">reference temperature T1Ref</td>
<td valign=\"top\">20</td><td valign=\"top\">&deg;C</td>
</tr>
<tr>
<td valign=\"top\">temperature coefficient alpha20_1 </td>
<td valign=\"top\">0</td><td valign=\"top\">1/K</td>
</tr>
<tr>
<td valign=\"top\">primary stray inductance L1sigma</td>
<td valign=\"top\">78E-6</td><td valign=\"top\">H per phase</td>
</tr>
<tr>
<td valign=\"top\">secondary resistance R2</td>
<td valign=\"top\">0.005</td><td valign=\"top\">Ohm per phase at reference temperature</td>
</tr>
<tr>
<td valign=\"top\">reference temperature T2Ref</td>
<td valign=\"top\">20</td><td valign=\"top\">&deg;C</td>
</tr>
<tr>
<td valign=\"top\">temperature coefficient alpha20_2 </td>
<td valign=\"top\">0</td><td valign=\"top\">1/K</td>
</tr>
<tr>
<td valign=\"top\">secondary stray inductance L2sigma</td>
<td valign=\"top\">78E-6</td><td valign=\"top\">H per phase</td>
</tr>
<tr>
<td valign=\"top\">operational temperature T1Operational</td>
<td valign=\"top\">20</td><td valign=\"top\">&deg;C</td>
</tr>
<tr>
<td valign=\"top\">operational temperature T2Operational</td>
<td valign=\"top\">20</td><td valign=\"top\">&deg;C</td>
</tr>
<tr>
<td valign=\"top\">These values give the operational parameters:</td>
<td valign=\"top\"> </td><td valign=\"top\"> </td>
</tr>
<tr>
<td valign=\"top\">nominal voltage drop</td>
<td valign=\"top\">0.05</td><td valign=\"top\">p.u.</td>
</tr>
<tr>
<td valign=\"top\">nominal copper losses</td>
<td valign=\"top\">300</td><td valign=\"top\">W</td>
</tr>
</table>
</html>"));
          end PartialBasicTransformer;
          annotation(
            Documentation(info = "<html>
<p>
This package contains the quasi stationary space phasor connector and partial models for quasi stationary machine models.
</p>
</html>"));
        end Interfaces;
        annotation(
          Icon(graphics = {Rectangle(origin = {0, 14.817}, fillColor = {170, 213, 255}, fillPattern = FillPattern.HorizontalCylinder, extent = {{-54.179, -59.817}, {65.821, 60.183}}), Rectangle(origin = {5.821, 15}, fillColor = {128, 128, 128}, fillPattern = FillPattern.HorizontalCylinder, extent = {{-80, -60}, {-60, 60}}), Rectangle(origin = {5.821, 15}, fillColor = {95, 95, 95}, fillPattern = FillPattern.HorizontalCylinder, extent = {{60, -10}, {80, 10}}), Rectangle(origin = {5.821, 15}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid, extent = {{-60, 50}, {20, 70}}), Polygon(origin = {5.821, 15}, fillPattern = FillPattern.Solid, points = {{-70, -90}, {-60, -90}, {-30, -20}, {20, -20}, {50, -90}, {60, -90}, {60, -100}, {-70, -100}, {-70, -90}})}),
          Documentation(info = "<html>
<p>This package hosts models for quasi stationary induction machines and transformers.
</p>
<h4>Please note</h4>
<p>
Quasi stationary DC machines are still operated with Dc voltage and current, whereas the quasi stationary induction machines and transformers
are operated with sinusoidal voltages and currents represented by time phasors. Quasi stationary theory can be found in the
<a href=\"modelica://Modelica.Electrical.QuasiStationary.UsersGuide.References\">references</a>.
Quasi stationary DC machine models therefore are part of the
<a href=\"modelica://Modelica.Electrical.Machines.BasicMachines.QuasiStationaryDCMachines\">machines library</a>.
</p>

</html>"));
      end Machines;

      package MultiPhase "Multiphase AC components"
        extends Modelica.Icons.Package;

        package Examples "Test examples"
          extends Modelica.Icons.ExamplesPackage;

          model BalancingStar "Balancing an unsymmetrical star-connected load"
            extends Modelica.Icons.Example;
            parameter Integer m = 3 "Number of phases";
            parameter Modelica.SIunits.Voltage V = 100 "Source Voltage";
            parameter Modelica.SIunits.Frequency f = 50 "Source frequency";
            parameter Modelica.SIunits.Resistance R = 10 "Load resistance";
            parameter Modelica.SIunits.Inductance L = R * sqrt(3) / (2 * Modelica.Constants.pi * f) "Load inductance";
            parameter Modelica.SIunits.Capacitance C = 1 / (R * sqrt(3)) / (2 * Modelica.Constants.pi * f) "Load capacitance";
            QuasiStationary.MultiPhase.Sources.VoltageSource voltageSource(m = m, f = f, V = fill(V, m), phi = -Modelica.Electrical.MultiPhase.Functions.symmetricOrientation(m)) annotation(
              Placement(transformation(origin = {-80, -20}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
            QuasiStationary.MultiPhase.Basic.Star star(m = m) annotation(
              Placement(transformation(origin = {-80, -50}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
            QuasiStationary.SinglePhase.Basic.Ground ground annotation(
              Placement(transformation(extent = {{-90, -90}, {-70, -70}}, rotation = 0)));
            QuasiStationary.MultiPhase.Basic.PlugToPin_p plugToPin_p1(m = m, k = 1) annotation(
              Placement(transformation(extent = {{-10, 40}, {10, 60}}, rotation = 0)));
            QuasiStationary.MultiPhase.Basic.PlugToPin_p plugToPin_p2(k = 2, m = m) annotation(
              Placement(transformation(extent = {{-10, 0}, {10, 20}}, rotation = 0)));
            QuasiStationary.MultiPhase.Basic.PlugToPin_p plugToPin_p3(k = 3, m = m) annotation(
              Placement(transformation(extent = {{-10, -40}, {10, -20}}, rotation = 0)));
            QuasiStationary.MultiPhase.Sensors.PowerSensor powerSensor(m = m) annotation(
              Placement(transformation(extent = {{-70, 0}, {-50, 20}}, rotation = 0)));
            QuasiStationary.MultiPhase.Sensors.CurrentSensor currentSensor(m = m) annotation(
              Placement(transformation(extent = {{-40, 0}, {-20, 20}}, rotation = 0)));
            QuasiStationary.SinglePhase.Basic.Resistor resistor(R_ref = R) annotation(
              Placement(transformation(extent = {{20, -40}, {40, -20}}, rotation = 0)));
            QuasiStationary.SinglePhase.Basic.Capacitor capacitor(C = C) annotation(
              Placement(transformation(extent = {{20, 0}, {40, 20}}, rotation = 0)));
            QuasiStationary.SinglePhase.Basic.Inductor inductor(L = L) annotation(
              Placement(transformation(extent = {{20, 40}, {40, 60}}, rotation = 0)));
            QuasiStationary.MultiPhase.Basic.Star star2(m = m) annotation(
              Placement(transformation(origin = {80, -50}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
            QuasiStationary.MultiPhase.Basic.PlugToPin_n plugToPin_n1(m = m, k = 1) annotation(
              Placement(transformation(origin = {60, 50}, extent = {{-10, -10}, {10, 10}}, rotation = 180)));
            QuasiStationary.MultiPhase.Basic.PlugToPin_n plugToPin_n2(k = 2, m = m) annotation(
              Placement(transformation(origin = {60, 10}, extent = {{-10, -10}, {10, 10}}, rotation = 180)));
            QuasiStationary.MultiPhase.Basic.PlugToPin_n plugToPin_n3(k = 3, m = m) annotation(
              Placement(transformation(origin = {60, -30}, extent = {{-10, -10}, {10, 10}}, rotation = 180)));
            QuasiStationary.SinglePhase.Sensors.CurrentSensor currentSensor0 annotation(
              Placement(transformation(origin = {-30, -60}, extent = {{-10, -10}, {10, 10}}, rotation = 180)));
          initial equation
            voltageSource.voltageSource.pin_p.reference.gamma = zeros(m);
          equation
            connect(ground.pin, star.pin_n) annotation(
              Line(points = {{-80, -70}, {-80, -60}}, color = {85, 170, 255}));
            connect(star.plug_p, voltageSource.plug_n) annotation(
              Line(points = {{-80, -40}, {-80, -30}}, color = {85, 170, 255}));
            connect(voltageSource.plug_p, powerSensor.currentP) annotation(
              Line(points = {{-80, -10}, {-80, 10}, {-70, 10}}, color = {85, 170, 255}));
            connect(powerSensor.currentN, currentSensor.plug_p) annotation(
              Line(points = {{-50, 10}, {-40, 10}}, color = {85, 170, 255}));
            connect(powerSensor.voltageP, powerSensor.currentP) annotation(
              Line(points = {{-60, 20}, {-70, 20}, {-70, 10}}, color = {85, 170, 255}));
            connect(powerSensor.voltageN, star.plug_p) annotation(
              Line(points = {{-60, 0}, {-60, -40}, {-80, -40}}, color = {85, 170, 255}));
            connect(currentSensor.plug_n, plugToPin_p2.plug_p) annotation(
              Line(points = {{-20, 10}, {-2, 10}}, color = {85, 170, 255}));
            connect(currentSensor.plug_n, plugToPin_p3.plug_p) annotation(
              Line(points = {{-20, 10}, {-20, -30}, {-2, -30}}, color = {85, 170, 255}));
            connect(currentSensor.plug_n, plugToPin_p1.plug_p) annotation(
              Line(points = {{-20, 10}, {-20, 50}, {-2, 50}}, color = {85, 170, 255}));
            connect(plugToPin_p1.pin_p, inductor.pin_p) annotation(
              Line(points = {{2, 50}, {20, 50}}, color = {85, 170, 255}));
            connect(plugToPin_p2.pin_p, capacitor.pin_p) annotation(
              Line(points = {{2, 10}, {20, 10}}, color = {85, 170, 255}));
            connect(plugToPin_p3.pin_p, resistor.pin_p) annotation(
              Line(points = {{2, -30}, {20, -30}}, color = {85, 170, 255}));
            connect(inductor.pin_n, plugToPin_n1.pin_n) annotation(
              Line(points = {{40, 50}, {58, 50}}, color = {85, 170, 255}));
            connect(capacitor.pin_n, plugToPin_n2.pin_n) annotation(
              Line(points = {{40, 10}, {58, 10}}, color = {85, 170, 255}));
            connect(resistor.pin_n, plugToPin_n3.pin_n) annotation(
              Line(points = {{40, -30}, {58, -30}}, color = {85, 170, 255}));
            connect(plugToPin_n1.plug_n, star2.plug_p) annotation(
              Line(points = {{62, 50}, {80, 50}, {80, -40}}, color = {85, 170, 255}));
            connect(plugToPin_n2.plug_n, star2.plug_p) annotation(
              Line(points = {{62, 10}, {80, 10}, {80, -40}}, color = {85, 170, 255}));
            connect(plugToPin_n3.plug_n, star2.plug_p) annotation(
              Line(points = {{62, -30}, {80, -30}, {80, -40}}, color = {85, 170, 255}));
            connect(star2.pin_n, currentSensor0.pin_p) annotation(
              Line(points = {{80, -60}, {-20, -60}}, color = {85, 170, 255}));
            connect(currentSensor0.pin_n, star.pin_n) annotation(
              Line(points = {{-40, -60}, {-80, -60}}, color = {85, 170, 255}));
            annotation(
              Documentation(info = "<html>
<p>
For the unsymmetrical load (resistor, capacitor and inductor) the parameters inductance L and capacitance C
are chosen such way that the neutral current (see currentSensor0) is zero.
</p>
<p>
P.Vaske, Berechnung von Drehstromschaltungen (German, Calculation of polyphase circuits), Teubner 1973, page 42, example 18
</p>
</html>"),
              experiment(StopTime = 1.0, Interval = 0.001));
          end BalancingStar;

          model BalancingDelta "Balancing an unsymmetrical delta-connected load"
            extends Modelica.Icons.Example;
            parameter Integer m = 3 "Number of phases";
            parameter Modelica.SIunits.Voltage V_LL = 100 "Source voltage line-to-line";
            parameter Modelica.SIunits.Frequency f = 50 "Source frequency";
            parameter Modelica.SIunits.Resistance R = 10 "Load resistance";
            parameter Modelica.SIunits.Inductance L = R * sqrt(3) / (2 * Modelica.Constants.pi * f) "Load inductance";
            parameter Modelica.SIunits.Capacitance C = 1 / (R * sqrt(3)) / (2 * Modelica.Constants.pi * f) "Load capacitance";
            QuasiStationary.MultiPhase.Sources.VoltageSource voltageSource(m = m, f = f, V = fill(V_LL, m), phi = -Modelica.Electrical.MultiPhase.Functions.symmetricOrientation(m)) annotation(
              Placement(transformation(origin = {-70, -20}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
            QuasiStationary.MultiPhase.Basic.Star star(m = m) annotation(
              Placement(transformation(origin = {-70, -50}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
            QuasiStationary.SinglePhase.Basic.Ground ground annotation(
              Placement(transformation(extent = {{-80, -90}, {-60, -70}}, rotation = 0)));
            QuasiStationary.MultiPhase.Basic.PlugToPin_p plugToPin_p1(m = m, k = 1) annotation(
              Placement(transformation(extent = {{12, 70}, {32, 90}}, rotation = 0)));
            QuasiStationary.MultiPhase.Basic.PlugToPin_p plugToPin_p2(k = 2, m = m) annotation(
              Placement(transformation(extent = {{10, 0}, {30, 20}}, rotation = 0)));
            QuasiStationary.MultiPhase.Basic.PlugToPin_p plugToPin_p3(k = 3, m = m) annotation(
              Placement(transformation(extent = {{10, -70}, {30, -50}}, rotation = 0)));
            QuasiStationary.MultiPhase.Sensors.PowerSensor powerSensor(m = m) annotation(
              Placement(transformation(extent = {{-60, 0}, {-40, 20}}, rotation = 0)));
            QuasiStationary.MultiPhase.Sensors.CurrentSensor currentSensor(m = m) annotation(
              Placement(transformation(extent = {{-30, 0}, {-10, 20}}, rotation = 0)));
            QuasiStationary.SinglePhase.Basic.Resistor resistor(R_ref = R) annotation(
              Placement(transformation(origin = {70, 30}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
            QuasiStationary.SinglePhase.Basic.Capacitor capacitor(C = C) annotation(
              Placement(transformation(origin = {40, 32}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
            QuasiStationary.SinglePhase.Basic.Inductor inductor(L = L) annotation(
              Placement(transformation(origin = {40, -38}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
            QuasiStationary.SinglePhase.Sensors.CurrentSensor currentSensor12 annotation(
              Placement(transformation(origin = {40, 62}, extent = {{-10, 10}, {10, -10}}, rotation = 270)));
            QuasiStationary.SinglePhase.Sensors.CurrentSensor currentSensor23 annotation(
              Placement(transformation(origin = {40, -8}, extent = {{-10, 10}, {10, -10}}, rotation = 270)));
            QuasiStationary.SinglePhase.Sensors.CurrentSensor currentSensor31 annotation(
              Placement(transformation(origin = {70, -10}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
            Modelica.ComplexBlocks.ComplexMath.ComplexToPolar complexToPolar[m] annotation(
              Placement(transformation(origin = {-20, -20}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          initial equation
            voltageSource.voltageSource.pin_p.reference.gamma = zeros(m);
          equation
            connect(ground.pin, star.pin_n) annotation(
              Line(points = {{-70, -70}, {-70, -60}}, color = {85, 170, 255}));
            connect(star.plug_p, voltageSource.plug_n) annotation(
              Line(points = {{-70, -40}, {-70, -30}}, color = {85, 170, 255}));
            connect(voltageSource.plug_p, powerSensor.currentP) annotation(
              Line(points = {{-70, -10}, {-70, 10}, {-60, 10}}, color = {85, 170, 255}));
            connect(powerSensor.currentP, powerSensor.voltageP) annotation(
              Line(points = {{-60, 10}, {-60, 20}, {-50, 20}}, color = {85, 170, 255}));
            connect(powerSensor.currentN, currentSensor.plug_p) annotation(
              Line(points = {{-40, 10}, {-30, 10}}, color = {85, 170, 255}));
            connect(powerSensor.voltageN, star.plug_p) annotation(
              Line(points = {{-50, 0}, {-50, -40}, {-70, -40}}, color = {85, 170, 255}));
            connect(currentSensor.plug_n, plugToPin_p2.plug_p) annotation(
              Line(points = {{-10, 10}, {18, 10}}, color = {85, 170, 255}));
            connect(currentSensor.plug_n, plugToPin_p1.plug_p) annotation(
              Line(points = {{-10, 10}, {0, 10}, {0, 80}, {20, 80}}, color = {85, 170, 255}));
            connect(currentSensor.plug_n, plugToPin_p3.plug_p) annotation(
              Line(points = {{-10, 10}, {0, 10}, {0, -60}, {18, -60}}, color = {85, 170, 255}));
            connect(currentSensor12.pin_p, plugToPin_p1.pin_p) annotation(
              Line(points = {{40, 72}, {40, 80}, {24, 80}}, color = {85, 170, 255}));
            connect(currentSensor12.pin_n, capacitor.pin_p) annotation(
              Line(points = {{40, 52}, {40, 42}}, color = {85, 170, 255}));
            connect(capacitor.pin_n, plugToPin_p2.pin_p) annotation(
              Line(points = {{40, 22}, {40, 10}, {22, 10}}, color = {85, 170, 255}));
            connect(plugToPin_p2.pin_p, currentSensor23.pin_p) annotation(
              Line(points = {{22, 10}, {40, 10}, {40, 2}}, color = {85, 170, 255}));
            connect(currentSensor23.pin_n, inductor.pin_p) annotation(
              Line(points = {{40, -18}, {40, -28}}, color = {85, 170, 255}));
            connect(inductor.pin_n, plugToPin_p3.pin_p) annotation(
              Line(points = {{40, -48}, {40, -60}, {22, -60}}, color = {85, 170, 255}));
            connect(plugToPin_p1.pin_p, resistor.pin_n) annotation(
              Line(points = {{24, 80}, {70, 80}, {70, 40}}, color = {85, 170, 255}));
            connect(resistor.pin_p, currentSensor31.pin_n) annotation(
              Line(points = {{70, 20}, {70, 0}}, color = {85, 170, 255}));
            connect(currentSensor31.pin_p, plugToPin_p3.pin_p) annotation(
              Line(points = {{70, -20}, {70, -60}, {22, -60}}, color = {85, 170, 255}));
            connect(currentSensor.y, complexToPolar.u) annotation(
              Line(points = {{-20, -1}, {-20, -8}}, color = {85, 170, 255}, smooth = Smooth.None));
            annotation(
              Documentation(info = "<html>
<p>
For the unsymmetrical load (resistor, capacitor and inductor) the parameters inductance L and capacitance C
are chosen such way that the magnitudes of the three phase currents (see currentSensor12, currentSensor23, currentSensor31) are equal.
</p>
<p>
P.Vaske, Berechnung von Drehstromschaltungen (German, Calculation of polyphase circuits), Teubner 1973, page 43, example 23
</p>
</html>"),
              experiment(StopTime = 1.0, Interval = 0.001));
          end BalancingDelta;
          annotation(
            Documentation(info = "<html>
Examples to demonstrate the usage of quasistationary electric components.
</html>"));
        end Examples;

        package Basic "Basic components for AC multiphase models"
          extends Modelica.Icons.Package;

          model Star "Star connection"
            parameter Integer m(final min = 1) = 3 "Number of phases";
            Interfaces.PositivePlug plug_p(final m = m) annotation(
              Placement(transformation(extent = {{-110, -10}, {-90, 10}}, rotation = 0)));
            QuasiStationary.SinglePhase.Interfaces.NegativePin pin_n annotation(
              Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
            PlugToPins_p plugToPins_p(final m = m) annotation(
              Placement(transformation(extent = {{-80, -10}, {-60, 10}}, rotation = 0)));
          equation
            for j in 1:m loop
              connect(plugToPins_p.pin_p[j], pin_n);
            end for;
            connect(plug_p, plugToPins_p.plug_p) annotation(
              Line(points = {{-100, 0}, {-93, 0}, {-86, 0}, {-72, 0}}, color = {85, 170, 255}));
            annotation(
              Icon(graphics = {Text(extent = {{-150, 60}, {150, 120}}, textString = "%name", lineColor = {0, 0, 255}), Line(points = {{80, 0}, {0, 0}}, color = {85, 170, 255}, thickness = 0.5), Line(points = {{0, 0}, {-39, 68}}, color = {85, 170, 255}, thickness = 0.5), Line(points = {{0, 0}, {-38, -69}}, color = {85, 170, 255}, thickness = 0.5), Text(extent = {{-100, -110}, {100, -70}}, lineColor = {0, 0, 0}, textString = "m=%m"), Line(points = {{-90, 0}, {-40, 0}}, color = {85, 170, 255}), Line(points = {{80, 0}, {90, 0}}, color = {0, 0, 255})}),
              Documentation(info = "<html>
<p>
Star (wye) connection of a multi phase circuit. The potentials at the star points are the same.
</p>
<h4>See also</h4>
<p>
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Basic.Delta\">Delta</a>
</p>
</html>"));
          end Star;

          model Delta "Delta (polygon) connection"
            parameter Integer m(final min = 2) = 3 "Number of phases";
            Interfaces.PositivePlug plug_p(final m = m) annotation(
              Placement(transformation(extent = {{-110, -10}, {-90, 10}}, rotation = 0)));
            Interfaces.NegativePlug plug_n(final m = m) annotation(
              Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
            PlugToPins_p plugToPins_p(final m = m) annotation(
              Placement(transformation(extent = {{-80, -10}, {-60, 10}}, rotation = 0)));
            PlugToPins_n plugToPins_n(final m = m) annotation(
              Placement(transformation(extent = {{80, -10}, {60, 10}}, rotation = 0)));
          equation
            for j in 1:m loop
              if j < m then
                connect(plugToPins_n.pin_n[j], plugToPins_p.pin_p[j + 1]);
              else
                connect(plugToPins_n.pin_n[j], plugToPins_p.pin_p[1]);
              end if;
            end for;
            connect(plug_p, plugToPins_p.plug_p) annotation(
              Line(points = {{-100, 0}, {-93, 0}, {-86, 0}, {-72, 0}}, color = {85, 170, 255}));
            connect(plugToPins_n.plug_n, plug_n) annotation(
              Line(points = {{72, 0}, {79, 0}, {79, 0}, {86, 0}, {86, 0}, {100, 0}}, color = {85, 170, 255}));
            annotation(
              Icon(graphics = {Text(extent = {{-150, 60}, {150, 120}}, lineColor = {0, 0, 255}, textString = "%name"), Line(points = {{-40, 68}, {-40, -70}, {79, 0}, {-40, 68}, {-40, 67}}, color = {85, 170, 255}, thickness = 0.5), Text(extent = {{-100, -110}, {100, -70}}, lineColor = {0, 0, 0}, textString = "m=%m"), Line(points = {{-90, 0}, {-40, 0}}, color = {85, 170, 255}), Line(points = {{80, 0}, {90, 0}}, color = {85, 170, 255})}),
              Documentation(info = "<html>
<p>
Delta (polygon) connection of a multi phase circuit.
</p>
<h4>See also</h4>
<p>
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Basic.Star\">Star</a>
</p>
</html>"));
          end Delta;

          model PlugToPin_p "Connect one (positive) pin"
            parameter Integer m(final min = 1) = 3 "Number of phases";
            parameter Integer k(final min = 1, final max = m) = 1 "Phase index";
            Interfaces.PositivePlug plug_p(final m = m) annotation(
              Placement(transformation(extent = {{-30, -10}, {-10, 10}}, rotation = 0)));
            QuasiStationary.SinglePhase.Interfaces.PositivePin pin_p annotation(
              Placement(transformation(extent = {{10, -10}, {30, 10}}, rotation = 0)));
          equation
            Connections.branch(plug_p.reference, pin_p.reference);
//Connections.potentialRoot(plug_p.reference);
//Connections.potentialRoot(pin_p.reference);
            plug_p.reference.gamma = pin_p.reference.gamma;
            pin_p.v = plug_p.pin[k].v;
            for j in 1:m loop
              plug_p.pin[j].i = if j == k then -pin_p.i else Complex(0);
            end for;
            annotation(
              Icon(graphics = {Rectangle(extent = {{-20, 20}, {40, -20}}, lineColor = {0, 0, 0}, fillColor = {215, 215, 215}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-40, 20}, {0, -20}}, lineColor = {0, 0, 0}, fillColor = {215, 215, 215}, fillPattern = FillPattern.Solid), Text(extent = {{-100, 100}, {100, 40}}, textString = "%name", lineColor = {0, 0, 255}), Text(extent = {{-100, -60}, {100, -100}}, lineColor = {0, 0, 0}, textString = "k = %k")}),
              Documentation(info = "<html>
<p>
Connects the single phase (positive) pin <i>k</i> of the multi phase (positive) plug to a single phase (positive) pin.
</p>
<h4>See also</h4>
<p>
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Basic.PlugToPin_n\">PlugToPin_n</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Basic.PlugToPins_p\">PlutToPins_p</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Basic.PlugToPins_n\">PlugToPins_n</a>
</p>
</html>"));
          end PlugToPin_p;

          model PlugToPin_n "Connect one (negative) pin"
            parameter Integer m(final min = 1) = 3 "Number of phases";
            parameter Integer k(final min = 1, final max = m) = 1 "Phase index";
            Interfaces.NegativePlug plug_n(final m = m) annotation(
              Placement(transformation(extent = {{-30, -10}, {-10, 10}}, rotation = 0)));
            QuasiStationary.SinglePhase.Interfaces.NegativePin pin_n annotation(
              Placement(transformation(extent = {{10, -10}, {30, 10}}, rotation = 0)));
          equation
            Connections.branch(plug_n.reference, pin_n.reference);
//Connections.potentialRoot(plug_n.reference);
//Connections.potentialRoot(pin_n.reference);
            plug_n.reference.gamma = pin_n.reference.gamma;
            pin_n.v = plug_n.pin[k].v;
            for j in 1:m loop
              plug_n.pin[j].i = if j == k then -pin_n.i else Complex(0);
            end for;
            annotation(
              Icon(graphics = {Rectangle(extent = {{-20, 20}, {40, -20}}, lineColor = {0, 0, 0}, fillColor = {215, 215, 215}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-40, 20}, {0, -20}}, lineColor = {0, 0, 0}, fillColor = {215, 215, 215}, fillPattern = FillPattern.Solid), Text(extent = {{-100, 100}, {100, 40}}, textString = "%name", lineColor = {0, 0, 255}), Text(extent = {{-100, -60}, {100, -100}}, lineColor = {0, 0, 0}, textString = "k = %k")}),
              Documentation(info = "<html>
<p>
Connects the single phase (negative) pin <i>k</i> of the multi phase (negative) plug to a single phase (negative) pin.
</p>
<h4>See also</h4>
<p>
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Basic.PlugToPin_p\">PlugToPin_p</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Basic.PlugToPins_p\">PlutToPins_p</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Basic.PlugToPins_n\">PlugToPins_n</a>
</p>
</html>"));
          end PlugToPin_n;

          model PlugToPins_p "Connect all (positive) pins"
            parameter Integer m(final min = 1) = 3 "number of phases";
            Interfaces.PositivePlug plug_p(final m = m) annotation(
              Placement(transformation(extent = {{-30, -10}, {-10, 10}}, rotation = 0)));
            QuasiStationary.SinglePhase.Interfaces.PositivePin pin_p[m] annotation(
              Placement(transformation(extent = {{10, -10}, {30, 10}}, rotation = 0)));
            PlugToPin_p plugToPin_p[m](each final m = m, final k = {j for j in 1:m}) annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}})));
          equation
            for j in 1:m loop
/*
    Connections.branch(plug_p.reference, pin_p[j].reference);
    plug_p.reference.gamma = pin_p[j].reference.gamma;
    plug_p.pin[j].v = pin_p[j].v;
    plug_p.pin[j].i = -pin_p[j].i;
*/
              connect(plug_p, plugToPin_p[j].plug_p);
              connect(plugToPin_p[j].pin_p, pin_p[j]);
            end for;
            annotation(
              Icon(graphics = {Rectangle(extent = {{-20, 20}, {40, -20}}, lineColor = {0, 0, 0}, fillColor = {170, 255, 255}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-40, 20}, {0, -20}}, lineColor = {0, 0, 0}, fillColor = {170, 255, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-100, 100}, {100, 40}}, textString = "%name", lineColor = {0, 0, 255})}),
              Documentation(info = "<html>
<p>
Connects all <i>m</i> single phase (positive) pins of the multi phase (positive) plug to an array of <i>m</i> single phase (positive) pins.
</p>
<h4>See also</h4>
<p>
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Basic.PlugToPin_p\">PlugToPin_p</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Basic.PlugToPin_n\">PlugToPin_n</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Basic.PlugToPins_n\">PlugToPins_n</a>
</p>
</html>"));
          end PlugToPins_p;

          model PlugToPins_n "Connect all (negative) pins"
            parameter Integer m(final min = 1) = 3 "number of phases";
            Interfaces.NegativePlug plug_n(final m = m) annotation(
              Placement(transformation(extent = {{-30, -10}, {-10, 10}}, rotation = 0)));
            QuasiStationary.SinglePhase.Interfaces.NegativePin pin_n[m] annotation(
              Placement(transformation(extent = {{10, -10}, {30, 10}}, rotation = 0)));
            PlugToPin_n plugToPin_n[m](each final m = m, final k = {j for j in 1:m}) annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}})));
          equation
            for j in 1:m loop
/*
    Connections.branch(plug_n.reference, pin_n[j].reference);
    plug_n.reference.gamma = pin_n[j].reference.gamma;
    plug_n.pin[j].v = pin_n[j].v;
    plug_n.pin[j].i = -pin_n[j].i;
*/
              connect(plug_n, plugToPin_n[j].plug_n);
              connect(plugToPin_n[j].pin_n, pin_n[j]);
            end for;
            annotation(
              Icon(graphics = {Rectangle(extent = {{-20, 20}, {40, -20}}, lineColor = {0, 0, 0}, fillColor = {170, 255, 255}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-40, 20}, {0, -20}}, lineColor = {0, 0, 0}, fillColor = {170, 255, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-100, 100}, {100, 40}}, textString = "%name", lineColor = {0, 0, 255})}),
              Documentation(info = "<html>
<p>
Connects all <i>m</i> single phase (negative) pins of the multi phase (negative) plug to an array of <i>m</i> single phase (negative) pins.
</p>
<h4>See also</h4>
<p>
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Basic.PlugToPin_p\">PlugToPin_p</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Basic.PlugToPin_n\">PlugToPin_n</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Basic.PlugToPins_p\">PlugToPins_p</a>
</p>
</html>"));
          end PlugToPins_n;

          model Resistor "Multiphase linear resistor"
            extends Interfaces.TwoPlug;
            parameter Modelica.SIunits.Resistance R_ref[m](start = fill(1, m)) "Reference resistances at T_ref";
            parameter Modelica.SIunits.Temperature T_ref[m] = fill(293.15, m) "Reference temperatures";
            parameter Modelica.SIunits.LinearTemperatureCoefficient alpha_ref[m] = zeros(m) "Temperature coefficient of resistance (R_actual = R_ref*(1 + alpha_ref*(heatPort.T - T_ref))";
            extends Modelica.Electrical.MultiPhase.Interfaces.ConditionalHeatPort(final mh = m, T = T_ref);
            QuasiStationary.SinglePhase.Basic.Resistor resistor[m](final R_ref = R_ref, final T_ref = T_ref, final alpha_ref = alpha_ref, each final useHeatPort = useHeatPort, final T = T) annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0)));
          equation
            connect(plugToPins_p.pin_p, resistor.pin_p) annotation(
              Line(points = {{-68, 0}, {-53.5, 0}, {-39, 0}, {-10, 0}}, color = {85, 170, 255}));
            connect(resistor.pin_n, plugToPins_n.pin_n) annotation(
              Line(points = {{10, 0}, {39, 0}, {68, 0}}, color = {85, 170, 255}));
            connect(resistor.heatPort, heatPort) annotation(
              Line(points = {{0, -10}, {0, -32.5}, {0, -55}, {0, -100}}, color = {191, 0, 0}));
            annotation(
              Icon(graphics = {Text(extent = {{100, 60}, {-100, 100}}, textString = "%name", lineColor = {0, 0, 255}), Line(points = {{60, 0}, {90, 0}}, color = {85, 170, 255}), Line(points = {{-90, 0}, {-60, 0}}, color = {85, 170, 255}), Rectangle(extent = {{-70, 30}, {70, -30}}, lineColor = {85, 170, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Text(extent = {{100, -80}, {-100, -40}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid, textString = "m=%m")}),
              Documentation(info = "<html>
<p>
The linear resistor connects the complex voltages <i><u>v</u></i> with the complex
currents <i><u>i</u></i> by <i><u>i</u>*R = <u>v</u></i>,
using <i>m</i> <a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Basic.Resistor\">single phase Resistors</a>.
</p>

<p>
The resistor model also has <i>m</i> optional
<a href=\"modelica://Modelica.Electrical.MultiPhase.Interfaces.ConditionalHeatPort\">conditional heat ports</a>.
A linear temperature dependency of the resistances for enabled heat ports is also taken into account.
</p>

<h4>See also</h4>
<p>
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Basic.Resistor\">Resistor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Basic.Conductor\">Conductor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Basic.Capacitor\">Capacitor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Basic.Inductor\">Inductor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Basic.VariableResistor\">Variable resistor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Basic.VariableConductor\">Variable conductor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Basic.VariableCapacitor\">Variable capacitor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Basic.VariableInductor\">Variable inductor</a>
</p>
</html>"));
          end Resistor;

          model Conductor "Multiphase linear conductor"
            extends Interfaces.TwoPlug;
            parameter Modelica.SIunits.Conductance G_ref[m](start = fill(1, m)) "Reference conductances at T_ref";
            parameter Modelica.SIunits.Temperature T_ref[m] = fill(293.15, m) "Reference temperatures";
            parameter Modelica.SIunits.LinearTemperatureCoefficient alpha_ref[m] = zeros(m) "Temperature coefficient of conductance (G_actual = G_ref/(1 + alpha_ref*(heatPort.T - T_ref))";
            extends Modelica.Electrical.MultiPhase.Interfaces.ConditionalHeatPort(final mh = m, T = T_ref);
            QuasiStationary.SinglePhase.Basic.Conductor conductor[m](final G_ref = G_ref, final T_ref = T_ref, final alpha_ref = alpha_ref, each final useHeatPort = useHeatPort, final T = T) annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0)));
          equation
            connect(plugToPins_p.pin_p, conductor.pin_p) annotation(
              Line(points = {{-68, 0}, {-53.5, 0}, {-53.5, 0}, {-39, 0}, {-39, 0}, {-10, 0}}, color = {85, 170, 255}));
            connect(conductor.pin_n, plugToPins_n.pin_n) annotation(
              Line(points = {{10, 0}, {39, 0}, {39, 0}, {68, 0}}, color = {85, 170, 255}));
            connect(conductor.heatPort, heatPort) annotation(
              Line(points = {{0, -10}, {0, -32.5}, {0, -32.5}, {0, -55}, {0, -55}, {0, -100}}, color = {191, 0, 0}));
            annotation(
              Icon(graphics = {Text(extent = {{100, 60}, {-100, 100}}, textString = "%name", lineColor = {0, 0, 255}), Line(points = {{60, 0}, {90, 0}}, color = {85, 170, 255}), Line(points = {{-90, 0}, {-60, 0}}, color = {85, 170, 255}), Rectangle(extent = {{-70, 30}, {70, -30}}, lineColor = {85, 170, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Text(extent = {{100, -80}, {-100, -40}}, lineColor = {0, 0, 0}, textString = "m=%m")}),
              Documentation(info = "<html>
<p>
The linear resistor connects the complex currents <i><u>i</u></i> with the complex
voltages <i><u>v</u></i> by <i><u>v</u>*G = <u>i</u></i>,
using <i>m</i> <a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Basic.Conductor\">single phase Conductors</a>.
</p>

<p>
The conductor model also has <i>m</i> optional
<a href=\"modelica://Modelica.Electrical.MultiPhase.Interfaces.ConditionalHeatPort\">conditional heat ports</a>.
A linear temperature dependency of the conductances for enabled heat ports is also taken into account.
</p>

<h4>See also</h4>
<p>
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Basic.Conductor\">Conductor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Basic.Resistor\">Resistor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Basic.Capacitor\">Capacitor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Basic.Inductor\">Inductor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Basic.VariableResistor\">Variable resistor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Basic.VariableConductor\">Variable conductor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Basic.VariableCapacitor\">Variable capacitor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Basic.VariableInductor\">Variable inductor</a>
</p>
</html>"));
          end Conductor;

          model Capacitor "Multiphase linear capacitor"
            extends Interfaces.TwoPlug;
            parameter Modelica.SIunits.Capacitance C[m](start = fill(1, m)) "Capacitances";
            QuasiStationary.SinglePhase.Basic.Capacitor capacitor[m](final C = C) annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0)));
          equation
            connect(plugToPins_p.pin_p, capacitor.pin_p) annotation(
              Line(points = {{-68, 0}, {-53.5, 0}, {-53.5, 0}, {-39, 0}, {-39, 0}, {-10, 0}}, color = {85, 170, 255}));
            connect(capacitor.pin_n, plugToPins_n.pin_n) annotation(
              Line(points = {{10, 0}, {39, 0}, {39, 0}, {68, 0}}, color = {85, 170, 255}));
            annotation(
              Icon(graphics = {Text(extent = {{100, 60}, {-100, 100}}, textString = "%name", lineColor = {0, 0, 255}), Line(points = {{-14, 28}, {-14, -28}}, color = {85, 170, 255}, thickness = 0.5), Line(points = {{14, 28}, {14, -28}}, color = {85, 170, 255}, thickness = 0.5), Line(points = {{-90, 0}, {-14, 0}}, color = {85, 170, 255}), Line(points = {{14, 0}, {90, 0}}, color = {85, 170, 255}), Text(extent = {{100, -80}, {-100, -40}}, lineColor = {0, 0, 0}, textString = "m=%m")}),
              Documentation(info = "<html>
<p>
The linear capacitor connects the complex currents <i><u>i</u></i> with the complex
voltages <i><u>v</u></i> by <i><u>v</u>*j*&omega;*C = <u>i</u></i>,
using <i>m</i> <a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Basic.Capacitor\">single phase Capacitors</a>.
</p>

<h4>See also</h4>
<p>
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Basic.Capacitor\">Capacitor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Basic.Resistor\">Resistor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Basic.Conductor\">Conductor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Basic.Inductor\">Inductor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Basic.VariableResistor\">Variable resistor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Basic.VariableConductor\">Variable conductor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Basic.VariableCapacitor\">Variable capacitor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Basic.VariableInductor\">Variable inductor</a>
</p>
</html>"));
          end Capacitor;

          model Inductor "Multiphase linear inductor"
            extends Interfaces.TwoPlug;
            parameter Modelica.SIunits.Inductance L[m](start = fill(1, m)) "Inductances";
            QuasiStationary.SinglePhase.Basic.Inductor inductor[m](final L = L) annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0)));
          equation
            connect(plugToPins_p.pin_p, inductor.pin_p) annotation(
              Line(points = {{-68, 0}, {-53.5, 0}, {-53.5, 0}, {-39, 0}, {-39, 0}, {-10, 0}}, color = {85, 170, 255}));
            connect(inductor.pin_n, plugToPins_n.pin_n) annotation(
              Line(points = {{10, 0}, {39, 0}, {39, 0}, {68, 0}}, color = {85, 170, 255}));
            annotation(
              Icon(graphics = {Text(extent = {{100, 60}, {-100, 100}}, textString = "%name", lineColor = {0, 0, 255}), Ellipse(extent = {{-60, -15}, {-30, 15}}, lineColor = {85, 170, 255}), Ellipse(extent = {{-30, -15}, {0, 15}}, lineColor = {85, 170, 255}), Ellipse(extent = {{0, -15}, {30, 15}}, lineColor = {85, 170, 255}), Ellipse(extent = {{30, -15}, {60, 15}}, lineColor = {85, 170, 255}), Rectangle(extent = {{-60, -30}, {60, 0}}, lineColor = {255, 255, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{60, 0}, {90, 0}}, color = {85, 170, 255}), Line(points = {{-90, 0}, {-60, 0}}, color = {85, 170, 255}), Text(extent = {{100, -80}, {-100, -40}}, lineColor = {0, 0, 0}, textString = "m=%m")}),
              Documentation(info = "<html>
<p>
The linear inductor connects the complex voltages <i><u>v</u></i> with the complex
currents <i><u>i</u></i> by <i><u>i</u>*j*&omega;*L = <u>v</u></i>,
using <i>m</i> <a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Basic.Inductor\">single phase Inductors</a>.
</p>

<h4>See also</h4>
<p>
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Basic.Inductor\">Inductor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Basic.Resistor\">Resistor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Basic.Conductor\">Conductor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Basic.Capacitor\">Capacitor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Basic.VariableResistor\">Variable resistor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Basic.VariableConductor\">Variable conductor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Basic.VariableCapacitor\">Variable capacitor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Basic.VariableInductor\">Variable inductor</a>
</p>
</html>"));
          end Inductor;

          model VariableResistor "Multiphase variable resistor"
            extends Interfaces.TwoPlug;
            parameter Modelica.SIunits.Temperature T_ref[m] = fill(293.15, m) "Reference temperatures";
            parameter Modelica.SIunits.LinearTemperatureCoefficient alpha_ref[m] = zeros(m) "Temperature coefficient of resistance (R_actual = R_ref*(1 + alpha_ref*(heatPort.T - T_ref))";
            extends Modelica.Electrical.MultiPhase.Interfaces.ConditionalHeatPort(final mh = m, T = T_ref);
            Modelica.Blocks.Interfaces.RealInput R_ref[m](each unit = "Ohm") annotation(
              Placement(transformation(origin = {0, 110}, extent = {{-20, -20}, {20, 20}}, rotation = 270)));
            QuasiStationary.SinglePhase.Basic.VariableResistor variableResistor[m](final T_ref = T_ref, final alpha_ref = alpha_ref, each final useHeatPort = useHeatPort, final T = T) annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0)));
          equation
            connect(variableResistor.pin_p, plugToPins_p.pin_p) annotation(
              Line(points = {{-10, 0}, {-24.5, 0}, {-24.5, 0}, {-39, 0}, {-39, 0}, {-68, 0}}, color = {85, 170, 255}));
            connect(variableResistor.pin_n, plugToPins_n.pin_n) annotation(
              Line(points = {{10, 0}, {39, 0}, {39, 0}, {68, 0}}, color = {85, 170, 255}));
            connect(variableResistor.heatPort, heatPort) annotation(
              Line(points = {{0, -10}, {0, -10}, {0, -100}}, color = {191, 0, 0}, smooth = Smooth.None));
            connect(R_ref, variableResistor.R_ref) annotation(
              Line(points = {{0, 110}, {0, 62}, {0, 62}, {0, 11}}, color = {0, 0, 127}, smooth = Smooth.None));
            annotation(
              Icon(graphics = {Text(extent = {{100, -80}, {-100, -40}}, textString = "%name", lineColor = {0, 0, 255}), Line(points = {{60, 0}, {90, 0}}, color = {85, 170, 255}), Line(points = {{-90, 0}, {-60, 0}}, color = {85, 170, 255}), Rectangle(extent = {{-70, 30}, {70, -30}}, lineColor = {85, 170, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{0, 90}, {0, 30}}, color = {85, 170, 255}, smooth = Smooth.None), Text(extent = {{100, 40}, {-100, 80}}, lineColor = {0, 0, 0}, textString = "m=%m")}),
              Documentation(info = "<html>
<p>
The linear resistor connects the complex voltages <i><u>v</u></i> with the complex
currents <i><u>i</u></i> by <i><u>i</u>*R = <u>v</u></i>,
using <i>m</i> <a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Basic.VariableResistor\">single phase variable Resistors</a>.
The resistances <i>R</i> are given as <i>m</i> input signals.
</p>

<p>
The resistor model also has <i>m</i> optional
<a href=\"modelica://Modelica.Electrical.MultiPhase.Interfaces.ConditionalHeatPort\">conditional heat ports</a>.
A linear temperature dependency of the resistances for enabled heat ports is also taken into account.
</p>

<h4>See also</h4>
<p>
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Basic.VariableResistor\">VariableResistor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Basic.Resistor\">Resistor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Basic.Conductor\">Conductor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Basic.Capacitor\">Capacitor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Basic.Inductor\">Inductor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Basic.VariableConductor\">Variable conductor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Basic.VariableCapacitor\">Variable capacitor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Basic.VariableInductor\">Variable inductor</a>
</p>
</html>"));
          end VariableResistor;

          model VariableConductor "Multiphase variable conductor"
            extends Interfaces.TwoPlug;
            parameter Modelica.SIunits.Temperature T_ref[m] = fill(293.15, m) "Reference temperatures";
            parameter Modelica.SIunits.LinearTemperatureCoefficient alpha_ref[m] = zeros(m) "Temperature coefficient of resistance (G_actual = G_ref/(1 + alpha_ref*(heatPort.T - T_ref))";
            extends Modelica.Electrical.MultiPhase.Interfaces.ConditionalHeatPort(final mh = m, T = T_ref);
            Modelica.Blocks.Interfaces.RealInput G_ref[m](each unit = "S") annotation(
              Placement(transformation(origin = {0, 110}, extent = {{-20, -20}, {20, 20}}, rotation = 270)));
            QuasiStationary.SinglePhase.Basic.VariableConductor variableResistor[m](final T_ref = T_ref, final alpha_ref = alpha_ref, each final useHeatPort = useHeatPort, final T = T) annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0)));
          equation
            connect(variableResistor.pin_p, plugToPins_p.pin_p) annotation(
              Line(points = {{-10, 0}, {-24.5, 0}, {-24.5, 0}, {-39, 0}, {-39, 0}, {-68, 0}}, color = {85, 170, 255}));
            connect(variableResistor.pin_n, plugToPins_n.pin_n) annotation(
              Line(points = {{10, 0}, {39, 0}, {39, 0}, {68, 0}}, color = {85, 170, 255}));
            connect(variableResistor.heatPort, heatPort) annotation(
              Line(points = {{0, -10}, {0, -32.5}, {0, -32.5}, {0, -55}, {0, -55}, {0, -100}}, color = {191, 0, 0}));
            connect(G_ref, variableResistor.G_ref) annotation(
              Line(points = {{0, 110}, {0, 85.25}, {0, 85.25}, {0, 60.5}, {0, 60.5}, {0, 11}}, color = {0, 0, 127}));
            annotation(
              Icon(graphics = {Text(extent = {{100, -80}, {-100, -40}}, textString = "%name", lineColor = {0, 0, 255}), Line(points = {{60, 0}, {90, 0}}, color = {85, 170, 255}), Line(points = {{-90, 0}, {-60, 0}}, color = {85, 170, 255}), Rectangle(extent = {{-70, 30}, {70, -30}}, lineColor = {85, 170, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{0, 90}, {0, 30}}, color = {85, 170, 255}, smooth = Smooth.None), Text(extent = {{100, 40}, {-100, 80}}, lineColor = {0, 0, 0}, textString = "m=%m")}),
              Documentation(info = "<html>
<p>
The linear resistor connects the complex currents <i><u>i</u></i> with the complex
voltages <i><u>v</u></i> by <i><u>v</u>*G = <u>i</u></i>,
using <i>m</i> <a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Basic.VariableConductor\">single phase variable Conductors</a>.
The conductances <i>G</i> are given as <i>m</i> input signals.
</p>

<p>
The conductor model also has <i>m</i> optional
<a href=\"modelica://Modelica.Electrical.MultiPhase.Interfaces.ConditionalHeatPort\">conditional heat ports</a>.
A linear temperature dependency of the conductances for enabled heat ports is also taken into account.
</p>

<h4>See also</h4>
<p>
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Basic.VariableConductor\">VariableConductor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Basic.Resistor\">Resistor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Basic.Conductor\">Conductor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Basic.Capacitor\">Capacitor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Basic.Inductor\">Inductor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Basic.VariableResistor\">Variable resistor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Basic.VariableCapacitor\">Variable capacitor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Basic.VariableInductor\">Variable inductor</a>
</p>
</html>"));
          end VariableConductor;

          model VariableCapacitor "Multiphase variable capacitor"
            extends Interfaces.TwoPlug;
            Modelica.Blocks.Interfaces.RealInput C[m](each unit = "F") annotation(
              Placement(transformation(origin = {0, 110}, extent = {{-20, -20}, {20, 20}}, rotation = 270)));
            QuasiStationary.SinglePhase.Basic.VariableCapacitor variableCapacitor[m] annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0)));
          equation
            connect(variableCapacitor.pin_p, plugToPins_p.pin_p) annotation(
              Line(points = {{-10, 0}, {-24.5, 0}, {-24.5, 0}, {-39, 0}, {-39, 0}, {-68, 0}}, color = {85, 170, 255}));
            connect(variableCapacitor.pin_n, plugToPins_n.pin_n) annotation(
              Line(points = {{10, 0}, {39, 0}, {39, 0}, {68, 0}}, color = {85, 170, 255}));
            connect(C, variableCapacitor.C) annotation(
              Line(points = {{0, 110}, {0, 61}, {0, 61}, {0, 11}}, color = {0, 0, 127}, smooth = Smooth.None));
            annotation(
              Icon(graphics = {Text(extent = {{100, -80}, {-100, -40}}, textString = "%name", lineColor = {0, 0, 255}), Line(points = {{-14, 28}, {-14, -28}}, color = {85, 170, 255}, thickness = 0.5), Line(points = {{14, 28}, {14, -28}}, color = {85, 170, 255}, thickness = 0.5), Line(points = {{-90, 0}, {-14, 0}}, color = {85, 170, 255}), Line(points = {{14, 0}, {90, 0}}, color = {85, 170, 255}), Line(points = {{0, 90}, {0, 30}}, color = {85, 170, 255}), Text(extent = {{100, 40}, {-100, 80}}, lineColor = {0, 0, 0}, textString = "m=%m")}),
              Documentation(info = "<html>
<p>
The linear capacitor connects the complex currents <i><u>i</u></i> with the complex
voltages <i><u>v</u></i> by <i><u>v</u>*j*&omega;*C = <u>i</u></i>,
using <i>m</i> <a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Basic.VariableCapacitor\">single phase variable Capacitors</a>.
The capacitances <i>C</i> are given as <i>m</i> input signals.
</p>

<h4>See also</h4>
<p>
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Basic.VariableCapacitor\">VariableCapacitor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Basic.Resistor\">Resistor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Basic.Conductor\">Conductor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Basic.Capacitor\">Capacitor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Basic.Inductor\">Inductor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Basic.VariableResistor\">Variable resistor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Basic.VariableConductor\">Variable conductor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Basic.VariableInductor\">Variable inductor</a>
</p>
</html>"));
          end VariableCapacitor;

          model VariableInductor "Multiphase variable inductor"
            extends Interfaces.TwoPlug;
            Modelica.Blocks.Interfaces.RealInput L[m](each unit = "H") annotation(
              Placement(transformation(origin = {0, 110}, extent = {{-20, -20}, {20, 20}}, rotation = 270)));
            QuasiStationary.SinglePhase.Basic.VariableInductor variableInductor[m] annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0)));
          equation
            connect(variableInductor.pin_p, plugToPins_p.pin_p) annotation(
              Line(points = {{-10, 0}, {-24.5, 0}, {-24.5, 0}, {-39, 0}, {-39, 0}, {-68, 0}}, color = {85, 170, 255}));
            connect(variableInductor.pin_n, plugToPins_n.pin_n) annotation(
              Line(points = {{10, 0}, {39, 0}, {39, 0}, {68, 0}}, color = {85, 170, 255}));
            connect(variableInductor.L, L) annotation(
              Line(points = {{0, 10.8}, {0, 35.6}, {0, 35.6}, {0, 60.4}, {0, 60.4}, {0, 110}}, color = {0, 0, 127}));
            annotation(
              Icon(graphics = {Text(extent = {{100, -80}, {-100, -40}}, textString = "%name", lineColor = {0, 0, 255}), Ellipse(extent = {{-60, -15}, {-30, 15}}, lineColor = {85, 170, 255}), Ellipse(extent = {{-30, -15}, {0, 15}}, lineColor = {85, 170, 255}), Ellipse(extent = {{0, -15}, {30, 15}}, lineColor = {85, 170, 255}), Ellipse(extent = {{30, -15}, {60, 15}}, lineColor = {85, 170, 255}), Rectangle(extent = {{-60, -30}, {60, 0}}, lineColor = {255, 255, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{60, 0}, {90, 0}}, color = {85, 170, 255}), Line(points = {{-90, 0}, {-60, 0}}, color = {85, 170, 255}), Line(points = {{0, 90}, {0, 8}}, color = {85, 170, 255}), Text(extent = {{100, 40}, {-100, 80}}, lineColor = {0, 0, 0}, textString = "m=%m")}),
              Documentation(info = "<html>
<p>
The linear inductor connects the complex voltages <i><u>v</u></i> with the complex
currents <i><u>i</u></i> by <i><u>i</u>*j*&omega;*L = <u>v</u></i>,
using <i>m</i> <a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Basic.VariableInductor\">single phase variable Inductors</a>.
The inductances <i>L</i> are given as <i>m</i> input signals.
</p>

<h4>See also</h4>
<p>
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Basic.Inductor\">Inductor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Basic.Resistor\">Resistor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Basic.Conductor\">Conductor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Basic.Capacitor\">Capacitor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Basic.Inductor\">Inductor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Basic.VariableResistor\">Variable resistor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Basic.VariableConductor\">Variable conductor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Basic.VariableCapacitor\">Variable capacitor</a>
</p>
</html>"));
          end VariableInductor;
          annotation(
            Icon(graphics = {Line(origin = {10, 40}, points = {{-100, -40}, {-80, -40}}), Line(origin = {10, 40}, points = {{60, -40}, {80, -40}}), Rectangle(lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, extent = {{-70, -30}, {70, 30}})}, coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = true)),
            Documentation(info = "<html>
<p>This package hosts basic models for quasi stationary multiphase circuits.
Quasi stationary theory can be found in the
<a href=\"modelica://Modelica.Electrical.QuasiStationary.UsersGuide.References\">references</a>.
</p>
<h4>See also</h4>

<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Basic\">SinglePhase.Basic</a>

</html>"));
        end Basic;

        package Ideal "Ideal components for AC multiphase models"
          extends Modelica.Icons.Package;

          model Idle "Idle branch"
            extends Interfaces.TwoPlug;
            QuasiStationary.SinglePhase.Ideal.Idle idle[m] annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0)));
          equation
            connect(plugToPins_p.pin_p, idle.pin_p) annotation(
              Line(points = {{-68, 0}, {-10, 0}}, color = {85, 170, 255}));
            connect(idle.pin_n, plugToPins_n.pin_n) annotation(
              Line(points = {{10, 0}, {39, 0}, {39, 0}, {68, 0}}, color = {0, 127, 0}));
            annotation(
              Icon(graphics = {Rectangle(extent = {{-60, 60}, {60, -60}}, lineColor = {85, 170, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{-90, 0}, {-41, 0}}, color = {85, 170, 255}), Line(points = {{91, 0}, {40, 0}}, color = {85, 170, 255}), Text(extent = {{-100, 100}, {100, 70}}, textString = "%name", lineColor = {0, 0, 255})}),
              Documentation(info = "<html>
<p>
This model describes <i>m</i> simple idle branches considering the complex currents <i><u>i</u></i> = 0;
it uses <i>m</i> <a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Ideal.Idle\">single phase idle branches</a>.
</p>

<h4>See also</h4>
<p>
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Ideal.Idle\">Idle</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Ideal.Short\">Short</a>
</p>
</html>"));
          end Idle;

          model Short "Short cut branch"
            extends Interfaces.TwoPlug;
            QuasiStationary.SinglePhase.Ideal.Short short[m] annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0)));
          equation
            connect(plugToPins_p.pin_p, short.pin_p) annotation(
              Line(points = {{-68, 0}, {-10, 0}}, color = {85, 170, 255}));
            connect(short.pin_n, plugToPins_n.pin_n) annotation(
              Line(points = {{10, 0}, {39, 0}, {39, 0}, {68, 0}}, color = {0, 127, 0}));
            annotation(
              Icon(graphics = {Rectangle(extent = {{-60, 60}, {60, -60}}, lineColor = {85, 170, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-100, 100}, {100, 70}}, textString = "%name", lineColor = {0, 0, 255}), Line(points = {{91, 0}, {-90, 0}}, color = {85, 170, 255})}),
              Documentation(info = "<html>
<p>
This model describes <i>m</i> simple short branches considering the complex voltages <i><u>v</u></i> = 0;
it uses <i>m</i> <a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Ideal.Short\">single phase short branches</a>.
</p>

<h4>See also</h4>
<p>
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Ideal.Short\">Short</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Ideal.Idle\">Idle</a>
</p>
</html>"));
          end Short;

          model IdealCommutingSwitch "Multiphase ideal commuting switch"
            parameter Integer m(final min = 1) = 3 "Number of phases";
            parameter Modelica.SIunits.Resistance Ron[m](final min = zeros(m), start = fill(1.E-5, m)) "Closed switch resistance";
            parameter Modelica.SIunits.Conductance Goff[m](final min = zeros(m), start = fill(1.E-5, m)) "Opened switch conductance";
            extends Modelica.Electrical.MultiPhase.Interfaces.ConditionalHeatPort(final mh = m, final T = fill(293.15, m));
            Modelica.Blocks.Interfaces.BooleanInput control[m] "true => p--n2 connected, false => p--n1 connected" annotation(
              Placement(transformation(origin = {0, 80}, extent = {{-20, -20}, {20, 20}}, rotation = 270)));
            Modelica.Electrical.QuasiStationary.MultiPhase.Interfaces.PositivePlug plug_p(final m = m) annotation(
              Placement(transformation(extent = {{-110, -10}, {-90, 10}}, rotation = 0)));
            Modelica.Electrical.QuasiStationary.MultiPhase.Interfaces.NegativePlug plug_n2(final m = m) annotation(
              Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
            Modelica.Electrical.QuasiStationary.MultiPhase.Interfaces.NegativePlug plug_n1(final m = m) annotation(
              Placement(transformation(extent = {{90, 40}, {110, 60}}, rotation = 0)));
            Modelica.Electrical.QuasiStationary.SinglePhase.Ideal.IdealCommutingSwitch idealCommutingSwitch[m](final Ron = Ron, final Goff = Goff, each final useHeatPort = useHeatPort) annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0)));
            Modelica.Electrical.QuasiStationary.MultiPhase.Basic.PlugToPins_p plugToPins_p(final m = m) annotation(
              Placement(transformation(extent = {{-90, -10}, {-70, 10}}, rotation = 0)));
            Modelica.Electrical.QuasiStationary.MultiPhase.Basic.PlugToPins_n plugToPins_n1(final m = m) annotation(
              Placement(transformation(origin = {80, 50}, extent = {{-10, -10}, {10, 10}}, rotation = 180)));
            Modelica.Electrical.QuasiStationary.MultiPhase.Basic.PlugToPins_n plugToPins_n2(final m = m) annotation(
              Placement(transformation(origin = {80, 0}, extent = {{-10, -10}, {10, 10}}, rotation = 180)));
          equation
            connect(control, idealCommutingSwitch.control) annotation(
              Line(points = {{0, 80}, {0, 8}}, color = {255, 0, 255}));
            connect(idealCommutingSwitch.heatPort, heatPort) annotation(
              Line(points = {{0, -10}, {0, -100}}, color = {191, 0, 0}, pattern = LinePattern.Solid, smooth = Smooth.None));
            connect(plugToPins_n1.plug_n, plug_n1) annotation(
              Line(points = {{82, 50}, {100, 50}}, color = {85, 170, 255}, smooth = Smooth.None));
            connect(plugToPins_n2.plug_n, plug_n2) annotation(
              Line(points = {{82, 0}, {90, 0}, {90, 0}, {100, 0}}, color = {85, 170, 255}, smooth = Smooth.None));
            connect(plugToPins_n2.pin_n, idealCommutingSwitch.n2) annotation(
              Line(points = {{78, 0}, {44, 0}, {44, 0}, {10, 0}}, color = {85, 170, 255}, smooth = Smooth.None));
            connect(idealCommutingSwitch.n1, plugToPins_n1.pin_n) annotation(
              Line(points = {{10, 5}, {10, 50}, {78, 50}}, color = {85, 170, 255}, smooth = Smooth.None));
            connect(plugToPins_p.plug_p, plug_p) annotation(
              Line(points = {{-82, 0}, {-100, 0}}, color = {85, 170, 255}, smooth = Smooth.None));
            connect(idealCommutingSwitch.p, plugToPins_p.pin_p) annotation(
              Line(points = {{-10, 0}, {-78, 0}}, color = {85, 170, 255}, smooth = Smooth.None));
            annotation(
              Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-150, -40}, {150, -100}}, textString = "%name", lineColor = {0, 0, 255}), Text(extent = {{-100, 100}, {-20, 60}}, lineColor = {0, 0, 0}, textString = "m="), Text(extent = {{20, 100}, {100, 60}}, lineColor = {0, 0, 0}, textString = "%m"), Ellipse(extent = {{-44, 4}, {-36, -4}}, lineColor = {85, 170, 255}), Line(points = {{-90, 0}, {-44, 0}}, color = {85, 170, 255}), Line(points = {{-37, 2}, {40, 50}}, color = {85, 170, 255}), Line(points = {{40, 50}, {90, 50}}, color = {85, 170, 255}), Line(points = {{0, 90}, {0, 25}}, color = {85, 170, 255}), Line(points = {{40, 0}, {90, 0}}, color = {85, 170, 255})}),
              Documentation(info = "<HTML>
<p>
Contains m singlephase ideal commuting switches (Modelica.Electrical.QuasiStationary.SinglePhase.Ideal.IdealCommutingSwitch).
</p>
<p>
<b>Use with care:</b>
This switch is only intended to be used for structural changes, not fast switching sequences, due to the quasistationary formulation.
</p>
</HTML>"));
          end IdealCommutingSwitch;

          model IdealIntermediateSwitch "Multiphase ideal intermediate switch"
            parameter Integer m(final min = 1) = 3 "Number of phases";
            parameter Modelica.SIunits.Resistance Ron[m](final min = zeros(m), start = fill(1.E-5, m)) "Closed switch resistance";
            parameter Modelica.SIunits.Conductance Goff[m](final min = zeros(m), start = fill(1.E-5, m)) "Opened switch conductance";
            extends Modelica.Electrical.MultiPhase.Interfaces.ConditionalHeatPort(final mh = m, final T = fill(293.15, m));
            Modelica.Blocks.Interfaces.BooleanInput control[m] "true => p1--n2, p2--n1 connected, otherwise p1--n1, p2--n2 connected" annotation(
              Placement(transformation(origin = {0, 80}, extent = {{-20, -20}, {20, 20}}, rotation = 270)));
            Modelica.Electrical.QuasiStationary.MultiPhase.Interfaces.PositivePlug plug_p1(final m = m) annotation(
              Placement(transformation(extent = {{-110, 40}, {-90, 60}}, rotation = 0)));
            Modelica.Electrical.QuasiStationary.MultiPhase.Interfaces.PositivePlug plug_p2(final m = m) annotation(
              Placement(transformation(extent = {{-110, -10}, {-90, 10}}, rotation = 0)));
            Modelica.Electrical.QuasiStationary.MultiPhase.Interfaces.NegativePlug plug_n2(final m = m) annotation(
              Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
            Modelica.Electrical.QuasiStationary.MultiPhase.Interfaces.NegativePlug plug_n1(final m = m) annotation(
              Placement(transformation(extent = {{90, 40}, {110, 60}}, rotation = 0)));
            Modelica.Electrical.QuasiStationary.SinglePhase.Ideal.IdealIntermediateSwitch idealIntermediateSwitch[m](final Ron = Ron, final Goff = Goff, each final useHeatPort = useHeatPort) annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0)));
            Modelica.Electrical.QuasiStationary.MultiPhase.Basic.PlugToPins_p plugToPins_p1(final m = m) annotation(
              Placement(transformation(extent = {{-90, 40}, {-70, 60}}, rotation = 0)));
            Modelica.Electrical.QuasiStationary.MultiPhase.Basic.PlugToPins_n plugToPins_n1(final m = m) annotation(
              Placement(transformation(origin = {80, 50}, extent = {{-10, -10}, {10, 10}}, rotation = 180)));
            Modelica.Electrical.QuasiStationary.MultiPhase.Basic.PlugToPins_p plugToPins_p2(final m = m) annotation(
              Placement(transformation(extent = {{-90, -10}, {-70, 10}}, rotation = 0)));
            Modelica.Electrical.QuasiStationary.MultiPhase.Basic.PlugToPins_n plugToPins_n2(final m = m) annotation(
              Placement(transformation(origin = {80, 0}, extent = {{-10, -10}, {10, 10}}, rotation = 180)));
          equation
            connect(control, idealIntermediateSwitch.control) annotation(
              Line(points = {{0, 80}, {0, 8}}, color = {255, 0, 255}));
            connect(idealIntermediateSwitch.heatPort, heatPort) annotation(
              Line(points = {{0, -10}, {0, -100}}, color = {191, 0, 0}, pattern = LinePattern.Solid, smooth = Smooth.None));
            connect(plug_p1, plugToPins_p1.plug_p) annotation(
              Line(points = {{-100, 50}, {-92, 50}, {-92, 50}, {-82, 50}}, color = {85, 170, 255}, smooth = Smooth.None));
            connect(plug_p2, plugToPins_p2.plug_p) annotation(
              Line(points = {{-100, 0}, {-82, 0}}, color = {85, 170, 255}, smooth = Smooth.None));
            connect(plugToPins_n1.plug_n, plug_n1) annotation(
              Line(points = {{82, 50}, {100, 50}}, color = {85, 170, 255}, smooth = Smooth.None));
            connect(plugToPins_n2.plug_n, plug_n2) annotation(
              Line(points = {{82, 0}, {90, 0}, {90, 0}, {100, 0}}, color = {85, 170, 255}, smooth = Smooth.None));
            connect(idealIntermediateSwitch.p2, plugToPins_p2.pin_p) annotation(
              Line(points = {{-10, 0}, {-78, 0}}, color = {85, 170, 255}, smooth = Smooth.None));
            connect(idealIntermediateSwitch.n2, plugToPins_n2.pin_n) annotation(
              Line(points = {{10, 0}, {44, 0}, {44, 0}, {78, 0}}, color = {85, 170, 255}, smooth = Smooth.None));
            connect(idealIntermediateSwitch.n1, plugToPins_n1.pin_n) annotation(
              Line(points = {{10, 5}, {10, 50}, {78, 50}}, color = {85, 170, 255}, smooth = Smooth.None));
            connect(idealIntermediateSwitch.p1, plugToPins_p1.pin_p) annotation(
              Line(points = {{-10, 5}, {-10, 50}, {-78, 50}}, color = {85, 170, 255}, smooth = Smooth.None));
            annotation(
              Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-150, -40}, {150, -100}}, textString = "%name", lineColor = {0, 0, 255}), Text(extent = {{-100, 100}, {-20, 60}}, lineColor = {0, 0, 0}, textString = "m="), Text(extent = {{20, 100}, {100, 60}}, lineColor = {0, 0, 0}, textString = "%m"), Ellipse(extent = {{-4, 30}, {4, 22}}, lineColor = {85, 170, 255}), Line(points = {{-90, 0}, {-40, 0}}, color = {85, 170, 255}), Line(points = {{-90, 50}, {-40, 50}}, color = {85, 170, 255}), Line(points = {{-40, 0}, {40, 50}}, color = {85, 170, 255}), Line(points = {{-40, 50}, {40, 0}}, color = {85, 170, 255}), Line(points = {{40, 50}, {90, 50}}, color = {85, 170, 255}), Line(points = {{0, 90}, {0, 25}}, color = {85, 170, 255}), Line(points = {{40, 0}, {90, 0}}, color = {85, 170, 255})}),
              Documentation(info = "<HTML>
<p>
Contains m ideal intermediate switches (Modelica.Electrical.QuasiStationary.SinglePhase.Ideal.IdealIntermediateSwitch).
</p>
<p>
<b>Use with care:</b>
This switch is only intended to be used for structural changes, not fast switching sequences, due to the quasistationary formulation.
</p>
</HTML>"));
          end IdealIntermediateSwitch;

          model IdealOpeningSwitch "Multiphase ideal opener"
            extends Modelica.Electrical.QuasiStationary.MultiPhase.Interfaces.TwoPlug;
            parameter Modelica.SIunits.Resistance Ron[m](final min = zeros(m), start = fill(1.E-5, m)) "Closed switch resistance";
            parameter Modelica.SIunits.Conductance Goff[m](final min = zeros(m), start = fill(1.E-5, m)) "Opened switch conductance";
            extends Modelica.Electrical.MultiPhase.Interfaces.ConditionalHeatPort(final mh = m, final T = fill(293.15, m));
            Modelica.Blocks.Interfaces.BooleanInput control[m] "true => switch open, false => p--n connected" annotation(
              Placement(transformation(origin = {0, 70}, extent = {{-20, -20}, {20, 20}}, rotation = 270)));
            Modelica.Electrical.QuasiStationary.SinglePhase.Ideal.IdealOpeningSwitch idealOpeningSwitch[m](final Ron = Ron, final Goff = Goff, each final useHeatPort = useHeatPort) annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0)));
          equation
            connect(control, idealOpeningSwitch.control) annotation(
              Line(points = {{0, 70}, {0, 7}}, color = {255, 0, 255}));
            connect(idealOpeningSwitch.heatPort, heatPort) annotation(
              Line(points = {{0, -10}, {0, -100}}, color = {191, 0, 0}, smooth = Smooth.None));
            connect(idealOpeningSwitch.pin_n, plugToPins_n.pin_n) annotation(
              Line(points = {{10, 0}, {39, 0}, {39, 0}, {68, 0}}, color = {85, 170, 255}, smooth = Smooth.None));
            connect(plugToPins_p.pin_p, idealOpeningSwitch.pin_p) annotation(
              Line(points = {{-68, 0}, {-10, 0}}, color = {85, 170, 255}, smooth = Smooth.None));
            annotation(
              Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-150, -40}, {150, -100}}, textString = "%name", lineColor = {0, 0, 255}), Line(points = {{-90, 0}, {-44, 0}}, color = {85, 170, 255}), Ellipse(extent = {{-44, 4}, {-36, -4}}, lineColor = {85, 170, 255}), Line(points = {{-37, 2}, {40, 50}}, color = {85, 170, 255}), Line(points = {{0, 88}, {0, 26}}, color = {85, 170, 255}), Line(points = {{40, 0}, {90, 0}}, color = {85, 170, 255}), Text(extent = {{-100, 100}, {-20, 60}}, lineColor = {0, 0, 0}, textString = "m="), Text(extent = {{20, 100}, {100, 60}}, lineColor = {0, 0, 0}, textString = "%m"), Line(points = {{40, 20}, {40, 0}}, color = {85, 170, 255})}),
              Documentation(info = "<HTML>
<p>
Contains m ideal opening switches (Modelica.Electrical.QuasiStationary.SinglePhase.Ideal.IdealOpeningSwitch).
</p>
<p>
<b>Use with care:</b>
This switch is only intended to be used for structural changes, not fast switching sequences, due to the quasistationary formulation.
</p>
</HTML>"));
          end IdealOpeningSwitch;

          model IdealClosingSwitch "Multiphase ideal closer"
            extends Modelica.Electrical.QuasiStationary.MultiPhase.Interfaces.TwoPlug;
            parameter Modelica.SIunits.Resistance Ron[m](final min = zeros(m), start = fill(1.E-5, m)) "Closed switch resistance";
            parameter Modelica.SIunits.Conductance Goff[m](final min = zeros(m), start = fill(1.E-5, m)) "Opened switch conductance";
            extends Modelica.Electrical.MultiPhase.Interfaces.ConditionalHeatPort(final mh = m, final T = fill(293.15, m));
            Modelica.Blocks.Interfaces.BooleanInput control[m] "true => p--n connected, false => switch open" annotation(
              Placement(transformation(origin = {0, 70}, extent = {{-20, -20}, {20, 20}}, rotation = 270)));
            Modelica.Electrical.QuasiStationary.SinglePhase.Ideal.IdealClosingSwitch idealClosingSwitch[m](final Ron = Ron, final Goff = Goff, each final useHeatPort = useHeatPort) annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0)));
          equation
            connect(control, idealClosingSwitch.control) annotation(
              Line(points = {{0, 70}, {0, 7}}, color = {255, 0, 255}));
            connect(idealClosingSwitch.heatPort, heatPort) annotation(
              Line(points = {{0, -10}, {0, -100}}, color = {191, 0, 0}, smooth = Smooth.None));
            connect(idealClosingSwitch.pin_p, plugToPins_p.pin_p) annotation(
              Line(points = {{-10, 0}, {-68, 0}}, color = {85, 170, 255}, smooth = Smooth.None));
            connect(idealClosingSwitch.pin_n, plugToPins_n.pin_n) annotation(
              Line(points = {{10, 0}, {39, 0}, {39, 0}, {68, 0}}, color = {85, 170, 255}, smooth = Smooth.None));
            annotation(
              Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-150, -40}, {150, -100}}, textString = "%name", lineColor = {0, 0, 255}), Line(points = {{-90, 0}, {-44, 0}}, color = {85, 170, 255}), Ellipse(extent = {{-44, 4}, {-36, -4}}, lineColor = {85, 170, 255}), Line(points = {{-37, 2}, {40, 50}}, color = {85, 170, 255}), Line(points = {{0, 88}, {0, 26}}, color = {85, 170, 255}), Line(points = {{40, 0}, {90, 0}}, color = {85, 170, 255}), Text(extent = {{-100, 100}, {-20, 60}}, lineColor = {0, 0, 0}, textString = "m="), Text(extent = {{20, 100}, {100, 60}}, lineColor = {0, 0, 0}, textString = "%m")}),
              Documentation(info = "<HTML>
<p>
Contains m ideal closing switches (Modelica.Electrical.QuasiStationary.SinglePhase.Ideal.IdealClosingSwitch).
</p>
<p>
<b>Use with care:</b>
This switch is only intended to be used for structural changes, not fast switching sequences, due to the quasistationary formulation.
</p>
</HTML>"));
          end IdealClosingSwitch;
          annotation(
            Icon(coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = true), graphics = {Line(origin = {10, 34}, points = {{-100, -60}, {-54, -60}}), Ellipse(origin = {10, 34}, extent = {{-54, -64}, {-46, -56}}), Line(origin = {10, 34}, points = {{-47, -58}, {30, -10}}), Line(origin = {10, 34}, points = {{30, -40}, {30, -60}}), Line(origin = {10, 34}, points = {{30, -60}, {80, -60}})}),
            Documentation(info = "<html>
<p>This package hosts ideal models for quasi stationary multiphase circuits.
Quasi stationary theory can be found in the
<a href=\"modelica://Modelica.Electrical.QuasiStationary.UsersGuide.References\">references</a>.
</p>
<h4>See also</h4>

<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Ideal\">SinglePhase.Ideal</a>

</html>"));
        end Ideal;

        package Sensors "AC multiphase sensors"
          extends Modelica.Icons.SensorsPackage;

          model FrequencySensor "Frequency sensor"
            extends Interfaces.AbsoluteSensor;
            SinglePhase.Sensors.FrequencySensor frequencySensor annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0)));
            Basic.PlugToPin_p plugToPin_p(final m = m, final k = 1) annotation(
              Placement(transformation(extent = {{-80, -10}, {-60, 10}}, rotation = 0)));
            Blocks.Interfaces.RealOutput y annotation(
              Placement(transformation(extent = {{100, -10}, {120, 10}}, rotation = 0)));
          equation
            connect(plug_p, plugToPin_p.plug_p) annotation(
              Line(points = {{-100, 0}, {-72, 0}}, color = {85, 170, 255}, smooth = Smooth.None));
            connect(plugToPin_p.pin_p, frequencySensor.pin) annotation(
              Line(points = {{-68, 0}, {-10, 0}}, color = {85, 170, 255}, smooth = Smooth.None));
            connect(frequencySensor.y, y) annotation(
              Line(points = {{11, 0}, {110, 0}}, color = {0, 0, 127}, smooth = Smooth.None));
            annotation(
              Icon(graphics = {Line(points = {{70, 0}, {80, 0}, {90, 0}, {100, 0}}, color = {0, 0, 127}), Text(extent = {{-29, -11}, {30, -70}}, lineColor = {0, 0, 0}, textString = "f")}),
              Documentation(info = "<html>

<p>
This sensor can be used to measure the frequency of the reference system, using <i>1</i>
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Sensors.FrequencySensor\">single phase FrequencySensor</a>.
</p>

<h4>See also</h4>

<p>
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Sensors.FrequencySensor\">SinglePhase.FrequencySensor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Sensors.PotentialSensor\">PotentialSensor</a>
</p>

</html>"));
          end FrequencySensor;

          model PotentialSensor "Potential sensor"
            extends Interfaces.AbsoluteSensor;
            QuasiStationary.SinglePhase.Sensors.PotentialSensor potentialSensor[m] annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0)));
            Basic.PlugToPins_p plugToPins_p(final m = m) annotation(
              Placement(transformation(extent = {{-80, -10}, {-60, 10}}, rotation = 0)));
            ComplexBlocks.Interfaces.ComplexOutput y[m] annotation(
              Placement(transformation(extent = {{100, -10}, {120, 10}}, rotation = 0)));
          equation
            connect(potentialSensor.y, y) annotation(
              Line(points = {{11, 0}, {35.75, 0}, {35.75, 0}, {60.5, 0}, {60.5, 0}, {110, 0}}, color = {85, 170, 255}));
            connect(plug_p, plugToPins_p.plug_p) annotation(
              Line(points = {{-100, 0}, {-72, 0}}, color = {85, 170, 255}, smooth = Smooth.None));
            connect(plugToPins_p.pin_p, potentialSensor.pin) annotation(
              Line(points = {{-68, 0}, {-10, 0}}, color = {85, 170, 255}, smooth = Smooth.None));
            annotation(
              Icon(graphics = {Text(extent = {{-29, -11}, {30, -70}}, lineColor = {0, 0, 0}, textString = "V")}),
              Documentation(info = "<html>

<p>
This sensor can be used to measure <i>m</i> complex potentials, using <i>m</i>
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Sensors.PotentialSensor\">single phase PotentialSensors</a>.
</p>

<h4>See also</h4>

<p>
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Sensors.PotentialSensor\">SinglePhase.PotentialSensor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Sensors.VoltageSensor\">VoltageSensor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Sensors.CurrentSensor\">CurrentSensor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Sensors.PowerSensor\">PowerSensor</a>
</p>

</html>"));
          end PotentialSensor;

          model VoltageSensor "Voltage sensor"
            extends Interfaces.RelativeSensor;
            QuasiStationary.SinglePhase.Sensors.VoltageSensor voltageSensor[m] annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0)));
          equation
            connect(plugToPins_p.pin_p, voltageSensor.pin_p) annotation(
              Line(points = {{-68, 0}, {-53.5, 0}, {-53.5, 0}, {-39, 0}, {-39, 0}, {-10, 0}}, color = {85, 170, 255}));
            connect(voltageSensor.pin_n, plugToPins_n.pin_n) annotation(
              Line(points = {{10, 0}, {39, 0}, {39, 0}, {68, 0}}, color = {85, 170, 255}));
            connect(voltageSensor.y, y) annotation(
              Line(points = {{0, -11}, {0, -35.75}, {0, -35.75}, {0, -60.5}, {0, -60.5}, {0, -110}}, color = {85, 170, 255}));
            annotation(
              Icon(graphics = {Text(extent = {{-29, -11}, {30, -70}}, lineColor = {0, 0, 0}, textString = "V")}),
              Documentation(info = "<html>

<p>
This sensor can be used to measure <i>m</i> complex voltages, using <i>m</i>
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Sensors.VoltageSensor\">single phase VoltageSensors</a>.
</p>

<h4>See also</h4>

<p>
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Sensors.VoltageSensor\">SinglePhase.VoltageSensor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Sensors.PotentialSensor\">PotentialSensor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Sensors.CurrentSensor\">CurrentSensor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Sensors.PowerSensor\">PowerSensor</a>
</p>

</html>"));
          end VoltageSensor;

          model CurrentSensor "Current Sensor"
            extends Interfaces.RelativeSensor;
            QuasiStationary.SinglePhase.Sensors.CurrentSensor currentSensor[m] annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0)));
          equation
            connect(plugToPins_p.pin_p, currentSensor.pin_p) annotation(
              Line(points = {{-68, 0}, {-53.5, 0}, {-53.5, 0}, {-39, 0}, {-39, 0}, {-10, 0}}, color = {85, 170, 255}));
            connect(currentSensor.pin_n, plugToPins_n.pin_n) annotation(
              Line(points = {{10, 0}, {39, 0}, {39, 0}, {68, 0}}, color = {85, 170, 255}));
            connect(currentSensor.y, y) annotation(
              Line(points = {{0, -11}, {0, -35.75}, {0, -35.75}, {0, -60.5}, {0, -60.5}, {0, -110}}, color = {85, 170, 255}));
            annotation(
              Icon(graphics = {Text(extent = {{-29, -11}, {30, -70}}, lineColor = {0, 0, 0}, textString = "I")}),
              Documentation(info = "<html>

<p>
This sensor can be used to measure <i>m</i> complex currents, using <i>m</i>
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Sensors.CurrentSensor\">single phase CurrentSensors</a>.
</p>

<h4>See also</h4>

<p>
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Sensors.CurrentSensor\">SinglePhase.CurrentSensor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Sensors.PotentialSensor\">PotentialSensor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Sensors.VoltageSensor\">VoltageSensor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Sensors.PowerSensor\">PowerSensor</a>
</p>

</html>"));
          end CurrentSensor;

          model PowerSensor "Power sensor"
            extends Modelica.Icons.RotationalSensor;
            parameter Integer m(min = 1) = 3 "number of phases";
            Modelica.SIunits.AngularVelocity omega = der(currentP.reference.gamma);
            Interfaces.PositivePlug currentP(final m = m) annotation(
              Placement(transformation(extent = {{-110, -10}, {-90, 10}}, rotation = 0)));
            Interfaces.NegativePlug currentN(final m = m) annotation(
              Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
            Interfaces.PositivePlug voltageP(final m = m) annotation(
              Placement(transformation(extent = {{-10, 90}, {10, 110}}, rotation = 0)));
            Interfaces.NegativePlug voltageN(final m = m) annotation(
              Placement(transformation(extent = {{-10, -110}, {10, -90}}, rotation = 0)));
            Modelica.ComplexBlocks.Interfaces.ComplexOutput y annotation(
              Placement(transformation(origin = {-80, -110}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
            Basic.PlugToPins_p plugToPinsCurrentP(final m = m) annotation(
              Placement(transformation(extent = {{-80, -10}, {-60, 10}}, rotation = 0)));
            Basic.PlugToPins_p plugToPinsVoltageP(final m = m) annotation(
              Placement(transformation(origin = {0, 70}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
            Basic.PlugToPins_n plugToPinsCurrentN(final m = m) annotation(
              Placement(transformation(extent = {{80, -10}, {60, 10}}, rotation = 0)));
            Basic.PlugToPins_n plugToPinsVoltageN(final m = m) annotation(
              Placement(transformation(origin = {0, -70}, extent = {{-10, 10}, {10, -10}}, rotation = 90)));
            QuasiStationary.SinglePhase.Sensors.PowerSensor powerSensor[m] annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0)));
            Modelica.ComplexBlocks.ComplexMath.Sum sum(final nin = m) annotation(
              Placement(transformation(origin = {-80, -70}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          equation
            connect(plugToPinsCurrentP.plug_p, currentP) annotation(
              Line(points = {{-72, 0}, {-79, 0}, {-86, 0}, {-100, 0}}, color = {85, 170, 255}));
            connect(currentN, plugToPinsCurrentN.plug_n) annotation(
              Line(points = {{100, 0}, {93, 0}, {86, 0}, {72, 0}}, color = {85, 170, 255}));
            connect(voltageP, plugToPinsVoltageP.plug_p) annotation(
              Line(points = {{0, 100}, {0, 100}, {0, 72}}, color = {85, 170, 255}));
            connect(plugToPinsVoltageN.plug_n, voltageN) annotation(
              Line(points = {{0, -72}, {0, -72}, {0, -100}}, color = {85, 170, 255}));
            connect(plugToPinsCurrentP.pin_p, powerSensor.currentP) annotation(
              Line(points = {{-68, 0}, {-53.5, 0}, {-53.5, 0}, {-39, 0}, {-39, 0}, {-10, 0}}, color = {85, 170, 255}));
            connect(powerSensor.currentN, plugToPinsCurrentN.pin_n) annotation(
              Line(points = {{10, 0}, {24.5, 0}, {24.5, 0}, {39, 0}, {39, 0}, {68, 0}}, color = {85, 170, 255}));
            connect(powerSensor.voltageP, plugToPinsVoltageP.pin_p) annotation(
              Line(points = {{0, 10}, {0, 10}, {0, 68}, {0, 68}}, color = {85, 170, 255}));
            connect(powerSensor.voltageN, plugToPinsVoltageN.pin_n) annotation(
              Line(points = {{0, -10}, {0, -10}, {0, -68}}, color = {85, 170, 255}));
            connect(powerSensor.y, sum.u) annotation(
              Line(points = {{-8, -11}, {-8, -40}, {-80, -40}, {-80, -58}}, color = {85, 170, 255}));
            connect(sum.y, y) annotation(
              Line(points = {{-80, -81}, {-80, -110}}, color = {85, 170, 255}));
            annotation(
              Icon(graphics = {Line(points = {{0, 100}, {0, 70}}, color = {0, 0, 255}), Line(points = {{0, -70}, {0, -100}}, color = {0, 0, 255}), Text(extent = {{-29, -70}, {30, -11}}, textString = "P"), Line(points = {{-80, -100}, {-80, 0}}, color = {85, 170, 255}), Text(textColor = {0, 0, 255}, extent = {{-100, 110}, {100, 150}}, textString = "%name"), Line(points = {{-100, 0}, {100, 0}}, color = {0, 0, 255})}),
              Documentation(info = "<html>

<p>
This sensor can be used to measure <i>m</i> complex apparent power values, using <i>m</i>
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Sensors.PowerSensor\">single phase PowerSensors</a>.
</p>

<h4>See also</h4>

<p>
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Sensors.PowerSensor\">SinglePhase.PowerSensor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Sensors.PotentialSensor\">PotentialSensor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Sensors.VoltageSensor\">VoltageSensor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Sensors.CurrentSensor\">CurrentSensor</a>
</p>

</html>"));
          end PowerSensor;
          annotation(
            Documentation(info = "<html>
<p>This package hosts sensors for quasi stationary multiphase circuits.
Quasi stationary theory can be found in the
<a href=\"modelica://Modelica.Electrical.QuasiStationary.UsersGuide.References\">references</a>.
</p>
<h4>See also</h4>

<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Sensors\">SinglePhase.Sensors</a>

</html>"));
        end Sensors;

        package Sources "AC multiphase sources"
          extends Modelica.Icons.SourcesPackage;

          model VoltageSource "Constant multiphase AC voltage"
            extends Interfaces.Source;
            parameter Modelica.SIunits.Frequency f(start = 1) "Frequency of the source";
            parameter Modelica.SIunits.Voltage V[m](start = fill(1, m)) "RMS voltage of the source";
            parameter Modelica.SIunits.Angle phi[m] = -Modelica.Electrical.MultiPhase.Functions.symmetricOrientation(m) "Phase shift of the source";
            QuasiStationary.SinglePhase.Sources.VoltageSource voltageSource[m](each final f = f, final V = V, final phi = phi) annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0)));
          equation
            connect(plugToPins_p.pin_p, voltageSource.pin_p) annotation(
              Line(points = {{-68, 0}, {-53.5, 0}, {-53.5, 0}, {-39, 0}, {-39, 0}, {-10, 0}}, color = {85, 170, 255}));
            connect(voltageSource.pin_n, plugToPins_n.pin_n) annotation(
              Line(points = {{10, 0}, {39, 0}, {39, 0}, {68, 0}}, color = {85, 170, 255}));
            annotation(
              Icon(graphics = {Line(points = {{50, 0}, {-50, 0}}, color = {0, 0, 0}), Text(extent = {{-120, 50}, {-20, 0}}, lineColor = {0, 0, 255}, textString = "+"), Text(extent = {{20, 50}, {120, 0}}, lineColor = {0, 0, 255}, textString = "-")}),
              Documentation(info = "<html>

<p>
This model describes <i>m</i> constant voltage sources, specifying the complex voltages by the RMS voltages and the phase shifts
(defaults are
<a href=\"modelica://Modelica.Electrical.MultiPhase.Functions.symmetricOrientation\">-symmetricOrientation</a>).
<i>m</i> <a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Sources.VoltageSource\">single phase VoltageSources</a> are used.
</p>

<h4>See also</h4>

<p>
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Sources.VoltageSource\">SinglePhase.VoltageSource</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Sources.VariableVoltageSource\">VariableVoltageSource</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Sources.CurrentSource\">CurrentSource</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Sources.VariableCurrentSource\">VariableCurrentSource</a>
</p>
</html>"));
          end VoltageSource;

          model VariableVoltageSource "Variable multiphase AC voltage"
            extends Interfaces.Source;
            QuasiStationary.SinglePhase.Sources.VariableVoltageSource variableVoltageSource[m] annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0)));
            Modelica.Blocks.Interfaces.RealInput f annotation(
              Placement(transformation(origin = {40, 100}, extent = {{-20, -20}, {20, 20}}, rotation = 270)));
            Modelica.ComplexBlocks.Interfaces.ComplexInput V[m] annotation(
              Placement(transformation(origin = {-40, 100}, extent = {{-20, -20}, {20, 20}}, rotation = 270)));
          equation
            for j in 1:m loop
              connect(f, variableVoltageSource[j].f) annotation(
                Line(points = {{40, 100}, {40, 60}, {4, 60}, {4, 10}}, color = {0, 0, 127}));
            end for;
            connect(plugToPins_p.pin_p, variableVoltageSource.pin_p) annotation(
              Line(points = {{-68, 0}, {-53.5, 0}, {-53.5, 0}, {-39, 0}, {-39, 0}, {-10, 0}}, color = {85, 170, 255}));
            connect(variableVoltageSource.pin_n, plugToPins_n.pin_n) annotation(
              Line(points = {{10, 0}, {39, 0}, {39, 0}, {68, 0}}, color = {85, 170, 255}));
            connect(V, variableVoltageSource.V) annotation(
              Line(points = {{-40, 100}, {-40, 60}, {-4, 60}, {-4, 10}}, color = {85, 170, 255}));
            annotation(
              Icon(graphics = {Line(points = {{50, 0}, {-50, 0}}, color = {0, 0, 0}), Text(extent = {{-120, 50}, {-20, 0}}, lineColor = {0, 0, 255}, textString = "+"), Text(extent = {{20, 50}, {120, 0}}, lineColor = {0, 0, 255}, textString = "-")}),
              Documentation(info = "<html>

<p>
This model describes <i>m</i> variable voltage sources, with <i>m</i> complex signal inputs,
specifying the complex voltages by the complex RMS voltage components.
Additionally, the frequency of the voltage source is defined by a real signal input.
<i>m</i> <a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Sources.VariableVoltageSource\">single phase VariableVoltageSources</a> are used.
</p>

<h4>See also</h4>

<p>
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Sources.VoltageSource\">SinglePhase.VoltageSource</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Sources.VoltageSource\">VoltageSource</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Sources.CurrentSource\">CurrentSource</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Sources.VariableCurrentSource\">VariableCurrentSource</a>
</p>
</html>"));
          end VariableVoltageSource;

          model CurrentSource "Constant multiphase AC current"
            extends Interfaces.Source;
            parameter Modelica.SIunits.Frequency f(start = 1) "Frequency of the source";
            parameter Modelica.SIunits.Current I[m](start = fill(1, m)) "RMS current of the source";
            parameter Modelica.SIunits.Angle phi[m] = -Modelica.Electrical.MultiPhase.Functions.symmetricOrientation(m) "Phase shift of the source";
            QuasiStationary.SinglePhase.Sources.CurrentSource currentSource[m](each final f = f, final phi = phi, final I = I) annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0)));
          equation
            connect(plugToPins_p.pin_p, currentSource.pin_p) annotation(
              Line(points = {{-68, 0}, {-10, 0}}, color = {85, 170, 255}));
            connect(currentSource.pin_n, plugToPins_n.pin_n) annotation(
              Line(points = {{10, 0}, {39, 0}, {39, 0}, {68, 0}}, color = {85, 170, 255}));
            annotation(
              Icon(graphics = {Line(points = {{-60, 60}, {60, 60}}, color = {0, 0, 255}), Polygon(points = {{60, 60}, {30, 70}, {30, 50}, {60, 60}}, lineColor = {0, 0, 255}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid), Line(points = {{0, -50}, {0, 50}}, color = {0, 0, 0})}),
              Documentation(info = "<html>

<p>
This model describes <i>m</i> constant current sources, specifying the complex currents by the RMS currents and the the phase shifts
(defaults are
<a href=\"modelica://Modelica.Electrical.MultiPhase.Functions.symmetricOrientation\">-symmetricOrientation</a>).
<i>m</i> <a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Sources.CurrentSource\">single phase CurrentSources</a> are used.
</p>

<h4>See also</h4>

<p>
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Sources.CurrentSource\">SinglePhase.CurrentSource</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Sources.VoltageSource\">VoltageSource</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Sources.VariableVoltageSource\">VariableVoltageSource</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Sources.VariableCurrentSource\">VariableCurrentSource</a>
</p>
</html>"));
          end CurrentSource;

          model VariableCurrentSource "Variable multiphase AC current"
            extends Interfaces.Source;
            QuasiStationary.SinglePhase.Sources.VariableCurrentSource variableCurrentSource[m] annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0)));
            Modelica.Blocks.Interfaces.RealInput f annotation(
              Placement(transformation(origin = {40, 100}, extent = {{-20, -20}, {20, 20}}, rotation = 270)));
            Modelica.ComplexBlocks.Interfaces.ComplexInput I[m] annotation(
              Placement(transformation(origin = {-40, 100}, extent = {{-20, -20}, {20, 20}}, rotation = 270)));
          equation
            for j in 1:m loop
              connect(f, variableCurrentSource[j].f) annotation(
                Line(points = {{40, 100}, {40, 60}, {4, 60}, {4, 10}}, color = {0, 0, 127}));
            end for;
            connect(plugToPins_p.pin_p, variableCurrentSource.pin_p) annotation(
              Line(points = {{-68, 0}, {-53.5, 0}, {-53.5, 0}, {-39, 0}, {-39, 0}, {-10, 0}}, color = {85, 170, 255}));
            connect(variableCurrentSource.pin_n, plugToPins_n.pin_n) annotation(
              Line(points = {{10, 0}, {39, 0}, {39, 0}, {68, 0}}, color = {85, 170, 255}));
            connect(I, variableCurrentSource.I) annotation(
              Line(points = {{-40, 100}, {-40, 60}, {-4, 60}, {-4, 10}}, color = {85, 170, 255}));
            annotation(
              Icon(graphics = {Line(points = {{-60, 60}, {60, 60}}, color = {85, 170, 255}), Polygon(points = {{60, 60}, {30, 70}, {30, 50}, {60, 60}}, lineColor = {0, 0, 255}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid), Line(points = {{0, -50}, {0, 50}}, color = {0, 0, 0})}),
              Documentation(info = "<html>

<p>
This model describes <i>m</i> variable current sources, with <i>m</i> complex signal inputs,
specifying the complex current by the complex RMS voltage components.
Additionally, the frequency of the current source is defined by a real signal input.
<i>m</i> <a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Sources.VariableCurrentSource\">single phase VariableCurrentSources</a> are used.
</p>

<h4>See also</h4>

<p>
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Sources.VoltageSource\">SinglePhase.VoltageSource</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Sources.VoltageSource\">VoltageSource</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Sources.VariableVoltageSource\">VariableVoltageSource</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Sources.CurrentSource\">CurrentSource</a>.
</p>
</html>"));
          end VariableCurrentSource;
          annotation(
            Documentation(info = "<html>
<p>This package hosts sources for quasi stationary multiphase circuits.
Quasi stationary theory can be found in the
<a href=\"modelica://Modelica.Electrical.QuasiStationary.UsersGuide.References\">references</a>.
</p>
<h4>See also</h4>

<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Sources\">SinglePhase.Sources</a>

</html>"));
        end Sources;

        package Interfaces "Interfaces for AC multiphase models"
          extends Modelica.Icons.InterfacesPackage;

          connector Plug "Basic multiphase plug"
            parameter Integer m = 3 "number of phases";
            QuasiStationary.SinglePhase.Interfaces.Pin pin[m];
            annotation(
              Documentation(info = "<html>

<p>
This multiphase plug contains a vector of <i>m</i> <a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Interfaces.Pin\">single phase pins</a>.
The <a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Interfaces.PositivePlug\">positive</a> and
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Interfaces.NegativePlug\">negative plug</a> are
derived from this base connector.
</p>

<h4>See also</h4>

<p>
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Interfaces.Pin\">Pin</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Interfaces.PositivePin\">PositivePin</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Interfaces.NegativePin\">NegativePin</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Interfaces.PositivePlug\">PositivePlug</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Interfaces.NegativePlug\">NegativePlug</a>
</p>
</html>"));
          end Plug;

          connector PositivePlug "Positive multiphase connector"
            extends Plug;
            QuasiStationary.Types.Reference reference;
            annotation(
              Icon(graphics = {Ellipse(extent = {{-100, 100}, {100, -100}}, lineColor = {85, 170, 255}, fillColor = {85, 170, 255}, fillPattern = FillPattern.Solid)}),
              Diagram(graphics = {Ellipse(extent = {{-40, 40}, {40, -40}}, lineColor = {85, 170, 255}, fillColor = {85, 170, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-100, 100}, {100, 60}}, lineColor = {0, 0, 255}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid, textString = "%name")}),
              Documentation(info = "<html>

<p>
The positive plug is based on <a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Interfaces.Plug\">Plug</a>.
Additionally the reference angle is specified in the connector. The time derivative of the reference angle is the actual angular velocity of each quasi stationary voltage and current. The symbol is also designed such way to look different than the <a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Interfaces.NegativePlug\">negative plug</a>.
</p>

<h4>See also</h4>

<p>
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Interfaces.Pin\">Pin</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Interfaces.PositivePin\">PositivePin</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Interfaces.NegativePin\">NegativePin</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Interfaces.Plug\">Plug</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Interfaces.NegativePlug\">NegativePlug</a>
</p>
</html>"));
          end PositivePlug;

          connector NegativePlug "Negative multiphase connector"
            extends Plug;
            QuasiStationary.Types.Reference reference;
            annotation(
              Icon(graphics = {Ellipse(extent = {{-100, 100}, {100, -100}}, lineColor = {85, 170, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid)}),
              Diagram(graphics = {Ellipse(extent = {{-40, 40}, {40, -40}}, lineColor = {85, 170, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-100, 100}, {100, 60}}, lineColor = {0, 0, 255}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid, textString = "%name")}),
              Documentation(info = "<html>

<p>
The negative plug is based on <a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Interfaces.Plug\">Plug</a>.
Additionally the reference angle is specified in the connector. The time derivative of the reference angle is the actual angular velocity of each quasi stationary voltage and current. The symbol is also designed such way to look different than the <a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Interfaces.PositivePlug\">positive plug</a>.
</p>

<h4>See also</h4>

<p>
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Interfaces.Pin\">Pin</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Interfaces.PositivePin\">PositivePin</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Interfaces.NegativePin\">NegativePin</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Interfaces.Plug\">Plug</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Interfaces.PositivePlug\">PositivePlug</a>,
</p>
</html>"));
          end NegativePlug;

          partial model TwoPlug "Two plugs with pin-adapter"
            parameter Integer m(min = 1) = 3 "Number of phases";
            Modelica.SIunits.ComplexVoltage v[m];
            Modelica.SIunits.ComplexCurrent i[m];
            Modelica.SIunits.AngularVelocity omega;
            PositivePlug plug_p(final m = m) "Positive quasi stationary multi phase plug" annotation(
              Placement(transformation(extent = {{-110, -10}, {-90, 10}}, rotation = 0)));
            NegativePlug plug_n(final m = m) "Negative quasi stationary multi phase plug" annotation(
              Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
            Basic.PlugToPins_p plugToPins_p(final m = m) annotation(
              Placement(transformation(extent = {{-80, -10}, {-60, 10}}, rotation = 0)));
            Basic.PlugToPins_n plugToPins_n(final m = m) annotation(
              Placement(transformation(origin = {70, 0}, extent = {{-10, -10}, {10, 10}}, rotation = 180)));
          equation
            omega = der(plug_p.reference.gamma);
            v = plug_p.pin.v - plug_n.pin.v;
            i = plug_p.pin.i;
            connect(plug_p, plugToPins_p.plug_p) annotation(
              Line(points = {{-100, 0}, {-93, 0}, {-86, 0}, {-72, 0}}, color = {85, 170, 255}));
            connect(plugToPins_n.plug_n, plug_n) annotation(
              Line(points = {{72, 0}, {86, 0}, {100, 0}}, color = {85, 170, 255}));
            annotation(
              Documentation(info = "<html>
<p>
This partial model uses a <a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Interfaces.PositivePlug\">positive</a>
and <a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Interfaces.NegativePlug\">negative plug</a> and defines the complex voltage differences as well as the complex currents (into the positive plug). A <a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Basic.PlugToPins_p\">positive</a> and
a <a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Basic.PlugToPins_n\">negative adapter</a> are used to give easy access to the single pins of both plugs. Additionally, the angular velocity of the quasi stationary system is explicitly defined as variable. This model is mainly intended to be used with graphical representation of user models.
</p>

<h4>See also</h4>

<p>
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Interfaces.PositivePlug\">PositivePlug</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Interfaces.NegativePlug\">NegativePlug</a>,
</p>
</html>"));
          end TwoPlug;

          partial model OnePort
            parameter Integer m(min = 1) = 3 "Number of phases";
            Modelica.SIunits.ComplexVoltage v[m];
            Modelica.SIunits.ComplexCurrent i[m];
            Modelica.SIunits.AngularVelocity omega;
            PositivePlug plug_p(final m = m) "Positive quasi stationary multi phase plug" annotation(
              Placement(transformation(extent = {{-110, -10}, {-90, 10}}, rotation = 0)));
            NegativePlug plug_n(final m = m) "Negative quasi stationary multi phase plug" annotation(
              Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
          equation
            Connections.branch(plug_p.reference, plug_n.reference);
            plug_p.reference.gamma = plug_n.reference.gamma;
            omega = der(plug_p.reference.gamma);
            v = plug_p.pin.v - plug_n.pin.v;
            i = plug_p.pin.i;
            plug_p.pin.i + plug_n.pin.i = fill(Complex(0), m);
          end OnePort;

          partial model AbsoluteSensor "Partial potential sensor"
            extends Modelica.Icons.RotationalSensor;
            parameter Integer m(min = 1) = 3 "number of phases";
            Modelica.SIunits.AngularVelocity omega;
            PositivePlug plug_p(final m = m) "Positive quasi stationary multi phase plug" annotation(
              Placement(transformation(extent = {{-110, -10}, {-90, 10}}, rotation = 0)));
          equation
            omega = der(plug_p.reference.gamma);
            annotation(
              Icon(graphics = {Line(points = {{-70, 0}, {-94, 0}}, color = {85, 170, 255}), Line(points = {{70, 0}, {80, 0}, {90, 0}, {100, 0}}, color = {85, 170, 255}), Text(extent = {{100, -100}, {-100, -70}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid, textString = "m=%m")}),
              Documentation(info = "<html>

<p>
The absolute sensor partial model relies on the a
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Interfaces.PositivePlug\">positive plug</a> to measure the complex potential. Additionally this model contains a proper icon and a definition of the angular velocity.
</p>

<h4>See also</h4>

<p>
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Interfaces.RelativeSensor\">RelativeSensor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Interfaces.AbsoluteSensor\">SinglePhase.Interfaces.AbsoluteSensor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Interfaces.RelativeSensor\">SinglePhase.Interfaces.RelativeSensor</a>
</p>

</html>"));
          end AbsoluteSensor;

          partial model RelativeSensor "Partial voltage / current sensor"
            extends Modelica.Icons.RotationalSensor;
            extends TwoPlug;
            Modelica.ComplexBlocks.Interfaces.ComplexOutput y[m] annotation(
              Placement(transformation(origin = {0, -110}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
            annotation(
              Icon(graphics = {Line(points = {{-70, 0}, {-94, 0}}, color = {0, 0, 0}), Line(points = {{70, 0}, {94, 0}}, color = {0, 0, 0}), Line(points = {{0, -70}, {0, -80}, {0, -90}, {0, -100}}, color = {85, 170, 255}), Text(extent = {{100, -100}, {-100, -70}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid, textString = "m=%m")}),
              Documentation(info = "<html>
<p>
The relative sensor partial model relies on the
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Interfaces.TwoPlug\">TwoPlug</a> to measure the complex voltages, currents or power. Additionally this model contains a proper icon and a definition of the angular velocity.
</p>

<h4>See also</h4>

<p>
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Interfaces.AbsoluteSensor\">AbsoluteSensor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Interfaces.AbsoluteSensor\">SinglePhase.Interfaces.AbsoluteSensor</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Interfaces.RelativeSensor\">SinglePhase.Interfaces.RelativeSensor</a>
</p>

</html>"));
          end RelativeSensor;

          partial model Source "Partial voltage / current source"
            extends TwoPlug;
            constant Modelica.SIunits.Angle pi = Modelica.Constants.pi;
            annotation(
              Icon(graphics = {Ellipse(extent = {{-50, 50}, {50, -50}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Text(extent = {{100, -100}, {-100, -60}}, textString = "%name", lineColor = {0, 0, 255}), Line(points = {{-90, 0}, {-50, 0}}, color = {0, 0, 0}), Line(points = {{50, 0}, {90, 0}}, color = {0, 0, 0}), Text(extent = {{100, 60}, {-100, 100}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid, textString = "m=%m")}),
              Documentation(info = "<html>
<p>
The source partial model relies on the
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Interfaces.TwoPlug\">TwoPlug</a> and contains a proper icon.
</p>

<h4>See also</h4>

<p>
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Sources.VoltageSource\">VoltageSource</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Sources.VariableVoltageSource\">VariableVoltageSource</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Sources.CurrentSource\">CurrentSource</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.MultiPhase.Sources.VariableCurrentSource\">VariableCurrentSource</a>,
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase.Interfaces.Source\">SinglePhase.Interfaces.Source</a>.
</p>
</html>"));
          end Source;
        end Interfaces;
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(origin = {14, 56}, lineColor = {0, 0, 255}, extent = {{-84, -126}, {56, 14}}), Ellipse(origin = {-0, 40}, lineColor = {0, 0, 255}, fillColor = {170, 213, 255}, fillPattern = FillPattern.Solid, extent = {{-40, -34}, {-20, -14}}), Ellipse(origin = {20, 40}, lineColor = {0, 0, 255}, fillColor = {170, 213, 255}, fillPattern = FillPattern.Solid, extent = {{0, -34}, {20, -14}}), Ellipse(origin = {10, 34}, lineColor = {0, 0, 255}, fillColor = {170, 213, 255}, fillPattern = FillPattern.Solid, extent = {{-20, -74}, {0, -54}})}),
          Documentation(info = "<html>
<p>This package hosts models for quasi stationary multi phase circuits.
Quasi stationary theory can be found in
[<a href=\"modelica://Modelica.Electrical.QuasiStationary.UsersGuide.References\">Vaske1973</a>]
and other
<a href=\"modelica://Modelica.Electrical.QuasiStationary.UsersGuide.References\">references</a>.
</p>
<h4>See also</h4>
<a href=\"modelica://Modelica.Electrical.QuasiStationary.SinglePhase\">SinglePhase</a>

</html>"));
      end MultiPhase;

      package Types "Definition of types for quasistationary AC models"
        extends Modelica.Icons.TypesPackage;

        record Reference "Reference angle"
          Modelica.SIunits.Angle gamma;

          function equalityConstraint "Equality constraint for reference angle"
            input Reference reference1;
            input Reference reference2;
            output Real residue[0];
          algorithm
            assert(abs(reference1.gamma - reference2.gamma) < 1E-6 * 2 * Modelica.Constants.pi, "Reference angles should be equal!");
            annotation(
              Documentation(info = "<html>
Equality constraint for the reference angle, according to the Modelica Specification (Overdetermined connection sets).
</html>"));
          end equalityConstraint;
          annotation(
            Documentation(info = "<html>
Reference angle, used in the AC connectors.
</html>"));
        end Reference;
        annotation(
          Documentation(info = "<html>
Type definitions needed for quasistationary AC models.
</html>"));
      end Types;
      annotation(
        preferredView = "info",
        Documentation(info = "<HTML>
<dl>
  <dt><b>Main Authors:</b></dt>
  <dd>
 <p>
  Anton Haumer<br>
  <a href=\"http://www.haumer.at/\">Technical Consulting &amp; Electrical Engineering</a><br>
  A-3423 St.Andrae-Woerdern, Austria<br>
  email: <a href=\"mailto:a.haumer@haumer.at\">a.haumer@haumer.at</a>
</p>
<p>
  Dr. Christian Kral<br>
  <a href=\"http://christiankral.net/\">Electric Machines, Drives and Systems</a><br>
  A-1060 Vienna, Austria<br>
  email: <a href=\"mailto:dr.christian.kral@gmail.com\">dr.christian.kral@gmail.com</a>
</p>
  </dd>
</dl>
<p>
Copyright &copy; 1998-2015, Modelica Association, Anton Haumer, Christian Kral and AIT.
</p>
<p>
<i>This Modelica package is <u>free</u> software and the use is completely at <u>your own risk</u>; it can be redistributed and/or modified under the terms of the Modelica License 2. For license conditions (including the disclaimer of warranty) see <a href=\"modelica://Modelica.UsersGuide.ModelicaLicense2\">Modelica.UsersGuide.ModelicaLicense2</a> or visit <a href=\"https://www.modelica.org/licenses/ModelicaLicense2\"> https://www.modelica.org/licenses/ModelicaLicense2</a>.</i>
</p>
</HTML>", revisions = "<HTML>
<table border=\"1\" cellspacing=\"0\" cellpadding=\"2\">
    <tr>
      <th>Version</th>
      <th>Revision</th>
      <th>Date</th>
      <th>Author</th>
      <th>Comment</th>
    </tr>
    <tr>
      <td valign=\"top\">1.0.0</td>
      <td valign=\"top\"> </td>
      <td valign=\"top\">2010-01-30</td>
      <td valign=\"top\">A. Haumer<br>C. Kral</td>
      <td valign=\"top\"></td>
    </tr>
</table>
</HTML>"),
        Icon(coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = true), graphics = {Line(origin = {10, 20}, points = {{-90, -20}, {-78.7, 14.2}, {-71.5, 33.1}, {-65.1, 46.4}, {-59.4, 54.6}, {-53.8, 59.1}, {-48.2, 59.8}, {-42.6, 56.6}, {-36.9, 49.7}, {-31.3, 39.4}, {-24.9, 24.1}, {-16.83, 1.2}, {0.1, -50.8}, {7.3, -70.2}, {13.7, -84.2}, {19.3, -93.1}, {25, -98.4}, {30.6, -100}, {36.2, -97.6}, {41.9, -91.5}, {47.5, -81.9}, {53.9, -67.2}, {62, -44.8}, {70, -20}}, smooth = Smooth.Bezier)}));
    end QuasiStationary;

    package Spice3 "Library for components of the Berkeley SPICE3 simulator"
      import SI = Modelica.SIunits;
      extends Modelica.Icons.Package;

      package UsersGuide "User's Guide"
        extends Modelica.Icons.Information;

        class Overview "Overview"
          extends Modelica.Icons.Information;
          annotation(
            Documentation(info = "<html>
<h4>Overview of Spice3 library</h4>
<p>The Spice3 library is a Modelica library that contains some models of the Berkeley SPICE3 analog simulator.</p>
<p><u>General information about the analog simulator SPICE3 </u></p>
<p>SPICE (Simulation Program with Integrated Circuit Emphasis) is a simulator for analog electrical circuits. It was developed as one of the first analog simulators in the university of Berkeley. SPICE netlists, which contain the circuit that shall be simulated, are a de-facto-standard up to now. For nearly every electrical circuit a SPICE netlist exists. Today the current version of SPICE is SPICE3e/SPICE3f. SPICE contains basic elements (resistor, inductor, capacitor), sources and semiconductor devices (diode, bipolar transistors, junction field effect transistors, MOS-field effect transistors) as well as models of lines. Out of this offered pool of elements, the circuits that shall be simulated are build as SPICE netlists.</p>
<p><u>The Spice3-library for Modelica</u></p>
<p>The Spice3 library was extracted from original SPICE3 C++ code. To be sure the Modelica models are correct the simulation results were compared to SPICE3. This way was chosen since SPICE3 is the only open source Spice simulator.</p>
<p>The Spice3-library was built in accordance to the model structure in SPICE. It contains the following packages:</p>
<ul>
<li>Examples</li>
<li>Basic (R, C, L, controlled sources) </li>
<li>Semiconductors (MOS (P, N), BJT(NPN PNP), Diode, semiconductor resistor)</li>
<li>Sources (constant, sinusoidal, exponential, pulse, piece wise linear, single-frequency FM, respectively for V and I)</li>
<li>Additionals (useful features from SPICE2)</li>
<li>Interfaces</li>
<li>Internal (functions and data needed to model the semiconductor devices)<br></li>
</ul>
<p>Since the semiconductor models, especially MOS and BJT, are very complex models, many functions, data and parameters were needed for their description. Therefore a special Package called Internal was created that contains all the functions and records with data and parameters that are needed for the semiconductor models. It is not necessary that a user of the library works inside this package, so it is not for user access. The package Additionals is also a special one. It is not part of the original SPICE3. Nevertheless it contains useful models or features like the polynomial sources of SPICE2 that are often asked for.</p>
<p>There are many commercial SPICE simulators (PSPICE, NgSPICE, HSPICE, ...) which are derived from the Berkeley SPICE or are in some relation to it. Netlists of such SPICE derivatives can differ from Berkeley SPICE3 netlists. This has to be taken into account if netlists (their parameter names) are used with this package.</p>
</html>"));
        end Overview;

        class Useofsemiconductors "Use of Semiconductors"
          extends Modelica.Icons.Information;
          annotation(
            Documentation(info = "<html>
<p>Within the semiconductor devices SPICE3 differentiates between technology parameters and device parameters. Device parameters can be chosen for every single model instance, e.g., the channel length of a transistor. Technology parameters which are specified in a model card (.model) are adjustable for more than one element simultaneously, e.g. the type of transistors. As usually done in Modelica the parameters of the modelcard can be set in a parameter list.</p>
<p>To parametrize more than one model two ways are possible:</p>
<ol>
<li>Apart record:<br>For each transistor in the circuit a record with the technologieparameters is made available as an instance of the record modelcardMOS. In the example<br>&quot;inverterApartRecord&quot; this way is explained more in detail.</li>
<li>Extended model:<br>For each set of technologyparameters a apart model has to be defined. In the example &quot;inverterExtendedModel&quot; this way is explained more in detail.</li>
</ol>
</html>"));
        end Useofsemiconductors;

        class Spicenetlist "SPICE3 netlists"
          extends Modelica.Icons.Information;
          annotation(
            Documentation(info = "<html>
<h4>Translation of SPICE3 netlists to Modelica </h4>
<p>Since SPICE3 netlists are available for nearly every electrical circuit a desirable feature would be to translate SPICE3 netlists to Modelica. With the help of the example of an inverter circuits a possible way of the translation will be explained.</p>

<table cellspacing=\"0\" cellpadding=\"0\" border=\"1\">
<caption>Table 1: Translation of the SPICE3 netlist (left side) to Modelica (right side)</caption>
<tr>
<td valign=\"top\"><pre>
inverter

Mp1 11 1 13 11 MPmos
Mp2 11 13 2 11 MPmos
Mn1 13 1 0 0 MNmos
Mn2 2 13 0 0 MNmos
Vgate 1 0 PULSE(0 5 2s 1s)
Vdrain 11 0 PULSE(0 5 0s 1s)
.model MPmos PMOS (gamma=0.37)
.model MNmos NMOS (gamma=0.37 lambda=0.02)















.tran 0.01 5
.end</pre></td>
<td valign=\"top\"><pre>
model inverter
  Spice3.Basic.Ground g;
  Spice3&hellip;M Mp1(mtype=true, M(GAMMA=0.37));
  Spice3&hellip;M Mp2(mtype=true, M(GAMMA=0.37));
  Spice3&hellip;M Mn1(M(LAMBDA=0.02, GAMMA=0.37));
  Spice3&hellip;M Mn2(p(LAMBDA=0.02, GAMMA=0.37));
  Spice3&hellip;V_pulse vdrain(V1=0, V2=5, TD=0, TR=1);
  Spice3&hellip;V_pulse vdrain(V1=0, V2=5, TD=0, TR=1);


  Spice3.Interfaces.Pin p_in, p_out;
protected
  Spice3.Interfaces.Pin n0, n1, n2, n11, n13;
equation
  connect(p_in, n1);    connect(p_out, n2);
  connect(g.p, n0);
  connect(vdrain.n,n0); connect(vdrain.p,n11);
  connect(Mp1.B,n11);   connect(Mp1.D, n11);
  connect(Mp1.G, n1);   connect(Mp1.S, n13);
  connect(Mp2.B,n11);   connect(Mp2.D, n11);
  connect(Mp2.G, n13);  connect(Mp2.S, n2);
  connect(Mn1.B,n0);    connect(Mn1.D, n13);
  connect(Mn1.G, n1);   connect(Mn1.S, n0);
  connect(Mn2.B,n0);    connect(Mn2.D, n2);
  connect(Mn2.G, n13);  connect(Mn2.S, n0);

end inverter;
</pre></td>
</tr>
</table>

<p>Given is a SPICE3 netlist that contains two inverter circuits. This netlist should be translated to Modelica in which the input voltage of the first inverter (node number 1) and the output voltage of the second inverter (node number 2) will later be connected with the surrounding circuit.</p>
<p>The following steps are necessary:</p>
<ol>
<li>A name for the Modelica model has to be chosen. It could be taken from the first line of the SPICE3 netlist.</li>
<li>The ground node has to be instantiated (i.e., <code>Spice3.Basic.Ground</code>).</li>
<li>For each component of the netlist an instant has to be created. According to the first letter of the SPICE3 model identifier in the netlist, the needed component has to be chosen, instantiated and according to the given parameters parametrized, e.g., the SPICE lineVdrain 11 0 PULSE(0 5 0 1)becomes the following Modelica line: <code>Spice3&hellip;V_pulse vdrain(V1=0, V2=5, TD=0, TR=1);</code></li>
<li>For all node numbers an internal pin has to be created. For example the node number 2 from the SPICE3 netlist becomes
<pre>
protected Spice3.Interfaces.Pin n2;
</pre>
in Modelica. The code letter (here <code>n</code>) is needed because a single number is no name in Modelica.</li>
<li>According to the netlist the internal pins have to be connected with the components, e.g., <code>connect(Mp1.D, n11)</code>.</li>
<li>In the last step the external pins have to be allocated ant connected to the according internal pin. In Table&nbsp;1 this is done as follows:
<pre>
Spice3.Interfaces.Pin p_in, p_out;
connect(p_in, n1);
connect(p_out, n2);
</pre>
</li>
</ol>
</html>"));
        end Spicenetlist;

        class NamingPrinciple "Naming principle"
          extends Modelica.Icons.Information;
          annotation(
            Documentation(info = "<html>
<p>In SPICE3 we have a predefined model pool. Each model device has got a code letter (e.g., resistor - R). In analogy to the SPICE3 models the models of the Spice3 library also got the according code letter in their names. The following examples shows the relationship:</p>
<p>A typical SPICE3 line could be:</p>
<p><b>C</b>1 3 2 1pF</p>
<p>The first letter is the code letter (here <b>C</b>). It specifies the type of the model component (here capacitance). To see the analogy to the SPICE3 models in the Spice3 library the transformed capacitance has got the name <b>C</b>_Capacitance. According to that naming rule the components of the Spice3 library have the following names (the first letter is the code letter that has to be used in SPICE3):</p>
<ul>
<li>R_Resistor</li>
<li>C_Capacitance</li>
<li>L_Inductor</li>
<li>E_VCV, E_VCV_POLY</li>
<li>G_VCC, G_VCC_POLY</li>
<li>H_CCV, H_CCV_POLY</li>
<li>F_CCC, F_CCC_POLY</li>
<li>M_PMOS</li>
<li>M_NMOS</li>
<li>Q_NPNBJT</li>
<li>Q_PNPBJT</li>
<li>D_Diode</li>
<li>V_constant, I_constant</li>
<li>V_sin, I_sin</li>
<li>V_exp, I_exp</li>
<li>V_pulse, I_pulse</li>
<li>V_pwl, I_pwl</li>
<li>V_sffm, I_sffm<br/><b><br/></b></li>
</ul>
</html>"));
        end NamingPrinciple;

        class ParameterHandling "Parameter handling"
          extends Modelica.Icons.Information;
          annotation(
            Documentation(info = "<html>
<p>In SPICE3 it is important to know whether a parameter was set by the user or not because the calculation of some values depends on that information and can be different. Since in Modelica there is no possibility to check that, a circumvention was chosen. The relevant parameters get an unrealistic value (-1e40) as their default value. Within a function it is checked if the parameter has still got this value (the parameter was not set by the user) of if it has a new value (parameter was set by the user).</p>
</html>"));
        end ParameterHandling;

        class Literature "Literature"
          extends Modelica.Icons.References;
          annotation(
            Documentation(info = "<html>
<table cellspacing=\"0\" cellpadding=\"2\" border=\"0\"><tr>
<td valign=\"top\"><p>[B&ouml;hme2009]</p></td>
<td valign=\"top\"><p>S. B&ouml;hme, K. Majetta, C. Clauss, P. Schneider, &quot;Spice3 Modelica Library,&quot; <i>7th Modelica Conference</i>, Como, Italy (2009)</p></td>
<td></td>
</tr>
<tr>
<td valign=\"top\"><p>[Antognetti1988]</p></td>
<td valign=\"top\"><p>P. Antognetti, G. Massobrio, <i>Semiconductor Device Modeling with SPICE.</i>, McGraw-Hill Book Company, USA, 1988</p></td>
<td></td>
</tr>
<tr>
<td valign=\"top\"><p>[Connelly1992]</p></td>
<td valign=\"top\"><p>A. Connelly, A, P. Choi, <i>Macromodeling with SPICE.</i>, Prentice-Hall, New Jersey, USA (1992)</p></td>
<td></td>
</tr>
<tr>
<td valign=\"top\"><p>[Johnson1991]</p></td>
<td valign=\"top\"><p>B. Johnson, T. Quarles, A.R. Newton, D. O. Pederson, A. Sangiovanni-Vincentelli, <i>SPICE3 Version 3f User's Manual.</i>, University of Berkeley, Department of Electrical Engineering and Computer Sciences, USA (1991): <a href=\"modelica://Modelica/Resources/Documentation/Electrical/Spice3/Spice_3f3_Users_Manual.pdf\">SPICE3 user's manual</a> (&copy; Regents of the University of California)</p></td>
<td></td>
</tr>
<tr>
<td valign=\"top\"><p>[Kielkowski1994]</p></td>
<td valign=\"top\"><p>R. Kielkowski, <i>Inside SPICE - Overcoming the obstacles of circuit simulation.</i>, McGraw-Hill, USA (1994)</p></td>
</tr>
</table>
</html>"));
        end Literature;

        class ReleaseNotes "Release notes"
          extends Modelica.Icons.ReleaseNotes;
          annotation(
            Documentation(info = "<html>
<p>This section summarizes the changes that have been performed on the Spice3 library.</p>
<ul>
<li>Version 1.0 (2010-02-18): first version of the library was released</li>
</ul>
</html>", revisions = "<html>
<ul>
<li><i>15th March 2012 by Kristin Majetta</i><br/>SPICE3 benchmark RTL Inverter</li>
<li><i>14th March 2012 by Kristin Majetta</i><br/>SPICE3 benchmark Mosfet characterisation</li>
<li><i>14th March 2012 by Kristin Majetta</i><br/>SPICE3 benchmark Differential Pair added</li>
<li><i>12th March 2012 by Kristin Majetta</i><br/>BJT model improved</li>
<li><i>09th March 2012 by Kristin Majetta</i><br/>MOS Level 2 model added</li>
<li><i>24th February 2012 by Kristin Majetta</i><br/>JFET model added</li>
<li><i>23rd February 2012 by Kristin Majetta</i><br/>Semiconductor Capacitor added</li>
<li><i>21st February 2012</i> by Kristin Majetta<br/>CoupledInductors (K) added</li>
<li><i>March 2010</i> by Kristin Majetta<br/>Guidelines applied, User&apos;s Guide added</li>
<li><i>February 2010</i> by Kristin Majetta<br/>Spice3 library added to MSL and examples revised</li>
<li><i>September 2009 </i>by Kristin Majetta <br/>Bipolar transistor implemented</li>
<li><i>August 2009 </i>by Jonathan Kress <br/>default values in sources improved</li>
<li><i>August 2009 </i>by Kristin Majetta <br/>Bipolar transistor started</li>
<li><i>April 2009 </i>by Kristin Majetta <br/>Semiconductor Resistor implemented</li>
<li><i>March 2009 </i>by Kristin Majetta <br/>DIODE implemented</li>
<li><i>25th February 2009 </i>by Kristin Majetta <br/>MOS Level 2 implemented</li>
<li><i>15th October 2008 </i>by Kristin Majetta <br/>minor errors fixed in L_Inductor, I_Pulse and SpiceRoot</li>
<li><i>April, 2008 </i>by Sandra Boehme <br/>initially implemented<br/></li>
</ul>
</html>"));
        end ReleaseNotes;

        class Contact "Contact"
          extends Modelica.Icons.Contact;
          annotation(
            Documentation(info = "<html>

<h4>Main Authors:</h4>
<dl>
<dt><b>Kristin Majetta</b></dt>
<dd>email: <a href=\"mailto:Kristin.Majetta@eas.iis.fraunhofer.de\">Kristin Majetta@eas.iis.fraunhofer.de</a></dd>
<dt><b>Christoph Clauss</b></dt>
<dd>email: <a href=\"mailto:Christoph.Clauss@eas.iis.fraunhofer.de\">Christoph.Clauss@eas.iis.fraunhofer.de</a></dd>
<dt><b>Sandra Boehme</b></dt>
<dd>email: <a href=\"mailto:Sandra.Boehme@eas.iis.fraunhofer.de\">Sandra.Boehme@eas.iis.fraunhofer.de</a></dd>
</dl>

<dl>
<dt>Address</dt>
<dd>Fraunhofer Institute Integrated Circuits<br />
Design Automation Division<br />
Zeunerstrasse 38<br />
01069 Dresden, Germany</dd>
</dl>

<h4>Acknowledgements:</h4>
<ul>
<li>The development of this library was done within the European ITEA2 projects EUROSYSLIB and MODELISAR. </li>
<li>For his contribution we thank Mr. Jonathan Gerbet.</li>
</ul>
</html>"));
        end Contact;
        annotation(
          DocumentationClass = true,
          Documentation(info = "<html>
<p>Package Spice3 is a <b>free</b> Modelica package</p>
<p>This is a short <b>User's Guide</b> for the overall library.</p>
</html>", revisions = "<html>
<ul>
<li><i>Feb 2010 </i>by Kristin Majetta initially written</li>
</ul>
</html>"));
      end UsersGuide;

      package Examples "Example circuits"
        extends Modelica.Icons.ExamplesPackage;

        model Inverter "Simple inverter circuit"
          //--------------------------------------------------------------------------------------------------------------
          //--------------------------------------------------------------------------------------------------------------
          extends Modelica.Icons.Example;
          Semiconductors.M_PMOS mp(modelcard(RD = 0, RS = 0, CBD = 0, CBS = 0), Sinternal(start = 0), IC = -1e40) annotation(
            Placement(transformation(extent = {{-14, 8}, {6, 28}})));
          Semiconductors.M_NMOS mn(modelcard(RD = 0, RS = 0, CBD = 0, CBS = 0), IC = -1e40) annotation(
            Placement(transformation(extent = {{-14, -34}, {6, -14}})));
          Basic.Ground ground annotation(
            Placement(transformation(extent = {{-14, -60}, {6, -40}})));
          Sources.V_pulse vin(V2 = 5, TD = 4e-12, TR = 0.1e-12, TF = 0.1e-12, PW = 1e-12, PER = 2e-12) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 270, origin = {-40, -16})));
          Sources.V_pulse v(V2 = 5, TR = 0.1e-12) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 270, origin = {40, -4})));
        equation
          connect(mn.D, mp.S) annotation(
            Line(points = {{-4, -14}, {-4, 8}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(mp.G, mn.G) annotation(
            Line(points = {{-14, 17.9}, {-14, -24.1}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(mn.S, mn.B) annotation(
            Line(points = {{-4, -34}, {6, -34}, {6, -24}}, color = {0, 0, 0}, smooth = Smooth.None));
          connect(mp.B, mp.D) annotation(
            Line(points = {{6, 18}, {6, 28}, {-4, 28}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(mn.S, ground.p) annotation(
            Line(points = {{-4, -34}, {-4, -40}}, color = {0, 0, 0}, smooth = Smooth.None));
          connect(v.p, mp.D) annotation(
            Line(points = {{40, 6}, {40, 28}, {-4, 28}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(v.n, ground.p) annotation(
            Line(points = {{40, -14}, {40, -40}, {-4, -40}}, color = {0, 0, 0}, smooth = Smooth.None));
          connect(vin.p, mp.G) annotation(
            Line(points = {{-40, -6}, {-40, 17.9}, {-14, 17.9}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(vin.n, ground.p) annotation(
            Line(points = {{-40, -26}, {-40, -40}, {-4, -40}}, color = {0, 0, 0}, smooth = Smooth.None));
          annotation(
            experiment(StopTime = 1e-11, Interval = 5e-15, Tolerance = 1e-7),
            Documentation(info = "<html>
<p>An inverter is an electrical circuit that consists of a PMOS and a NMOS transistor. Its task is to turn the input voltage from high potential to low potential or the other way round.</p>
<p>Simulate until 1.e-11 s. Display the input voltage Vin.p.v as well as the output voltage mp.S.v. It shows that the input voltage is inverted.</p>
</html>", revisions = "<html>
<ul>
<li><i>March 2009 </i>by Kristin Majetta initially implemented</li>
</ul>
</html>"));
        end Inverter;

        model InvertersApartRecord "Two inverters where transistor models use different modelcard instances"
          extends Modelica.Icons.Example;
          Basic.Ground ground annotation(
            Placement(transformation(extent = {{-38, -80}, {-18, -60}}, rotation = 0)));
          //--------------------------------------------------------------------------------------------------------------
          /*apart record: For each transistor in the circuit a record with the technologieparameters is made available
            as an instance of the record modelcardMOS */
          parameter Semiconductors.ModelcardMOS MPmos(GAMMA = 0.37, CBD = 0, CBS = 0) "Specified modelcardMOS for MPmos";
          //instance of record modelcardMOS
          parameter Semiconductors.ModelcardMOS MNmos(GAMMA = 0.37, LAMBDA = 0.02, CBD = 0, CBS = 0) "Specified modelcardMOS for MNmos";
          //instance of record modelcardMOS
          Semiconductors.M_PMOS mp1(modelcard = MPmos, IC = -1e40) annotation(
            Placement(transformation(extent = {{-38, 20}, {-18, 40}}, rotation = 0)));
          Semiconductors.M_NMOS mn1(modelcard = MNmos, IC = -1e40) annotation(
            Placement(transformation(extent = {{-38, -30}, {-18, -10}}, rotation = 0)));
          Semiconductors.M_PMOS mp2(modelcard = MPmos, IC = -1e40) annotation(
            Placement(transformation(extent = {{2, 20}, {22, 40}}, rotation = 0)));
          Semiconductors.M_NMOS mn2(modelcard = MNmos, IC = -1e40) annotation(
            Placement(transformation(extent = {{2, -30}, {22, -10}}, rotation = 0)));
          //--------------------------------------------------------------------------------------------------------------
          Basic.C_Capacitor c1(C = 1e-5, IC = 0, UIC = true) annotation(
            Placement(transformation(origin = {-8, -30}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          Basic.C_Capacitor c2(C = 1e-5, IC = 0, UIC = true) annotation(
            Placement(transformation(origin = {34, -30}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          Sources.V_pulse vin(V2 = 5, TD = 2, TR = 1) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 270, origin = {-60, -32})));
          Sources.V_pulse v(V2 = 5, TR = 1) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 270, origin = {60, -32})));
        equation
          connect(mp1.B, mp1.D) annotation(
            Line(points = {{-18, 30}, {-18, 40}, {-28, 40}}, color = {0, 0, 255}));
          connect(mn1.S, ground.p) annotation(
            Line(points = {{-28, -30}, {-28, -60}}, color = {0, 0, 255}));
          connect(mp1.S, mn1.D) annotation(
            Line(points = {{-28, 20}, {-28, -10}}, color = {0, 0, 255}));
          connect(mn1.G, mp1.G) annotation(
            Line(points = {{-38, -20.1}, {-38, 29.9}}, color = {0, 0, 255}));
          connect(mn1.B, mn1.S) annotation(
            Line(points = {{-18, -20}, {-18, -30}, {-28, -30}}, color = {0, 0, 255}));
          connect(mp2.B, mp2.D) annotation(
            Line(points = {{22, 30}, {22, 40}, {12, 40}}, color = {0, 0, 255}));
          connect(mn2.S, ground.p) annotation(
            Line(points = {{12, -30}, {12, -60}, {-28, -60}}, color = {0, 0, 255}));
          connect(mp2.S, mn2.D) annotation(
            Line(points = {{12, 20}, {12, -10}}, color = {0, 0, 255}));
          connect(mn2.G, mp2.G) annotation(
            Line(points = {{2, -20.1}, {2, 29.9}}, color = {0, 0, 255}));
          connect(mn2.B, mn2.S) annotation(
            Line(points = {{22, -20}, {22, -30}, {12, -30}}, color = {0, 0, 255}));
          connect(mp2.G, mn1.D) annotation(
            Line(points = {{2, 29.9}, {2, 0}, {-28, 0}, {-28, -10}}, color = {0, 0, 255}));
          connect(c1.p, mn1.D) annotation(
            Line(points = {{-8, -20}, {-8, 0}, {-28, 0}, {-28, -10}}, color = {0, 0, 255}));
          connect(mn2.D, c2.p) annotation(
            Line(points = {{12, -10}, {12, 0}, {34, 0}, {34, -20}}, color = {0, 0, 255}));
          connect(c2.n, ground.p) annotation(
            Line(points = {{34, -40}, {34, -60}, {-28, -60}}, color = {0, 0, 255}));
          connect(c1.n, ground.p) annotation(
            Line(points = {{-8, -40}, {-8, -60}, {-28, -60}}, color = {0, 0, 255}));
          connect(mp1.G, vin.p) annotation(
            Line(points = {{-38, 29.9}, {-60, 29.9}, {-60, -22}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(vin.n, ground.p) annotation(
            Line(points = {{-60, -42}, {-60, -60}, {-28, -60}}, color = {0, 0, 0}, smooth = Smooth.None));
          connect(v.p, mp2.D) annotation(
            Line(points = {{60, -22}, {60, 40}, {12, 40}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(mp1.D, mp2.D) annotation(
            Line(points = {{-28, 40}, {12, 40}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(v.n, ground.p) annotation(
            Line(points = {{60, -42}, {60, -60}, {-28, -60}}, color = {0, 0, 0}, smooth = Smooth.None));
          annotation(
            experiment(StopTime = 5),
            Documentation(info = "<html>
<p>An inverter is an electrical circuit that consists of a PMOS and a NMOS. Its task is to turn the input voltage from high potential to low potential or the other way round. This circuit<i> InverterApartModel</i> contains two inverters. The input voltage of the first inverter is nearly equal to the output voltage of the second inverter. Capacities cause some differences.</p>
<p>To see the typical behavior of the circuit the input voltages and the output voltages should be plotted. Besides that it can be interesting to watch the output voltage of the first inverter. Simulated until t=5s.</p>
<p>Input voltages: vin.p.v and v.p.v</p>
<p>Output voltage of the first inverter: mn1.ND.v</p>
<p>Output voltage of the second Inverter: mn2.ND.v</p>
<p>This example shows one possibility to make the record of the technology parameters available for more than one transistor. For each transistor in the circuit a record with the technology parameters is made available as an instance of the record modelcardMOS. In this circuit we need two different records for technology parameters, one for PMOS (MPmos) and one for NMOS (MNmos). This instances of the record for the technology parameters were made available for every transistor as one of theirs parameters (Spice3.Repository.MOS mn1(mtype=0, modelcard=MNmos).</p>
</html>", revisions = "<html>
<ul>
<li><i>April 2009 </i>by Kristin Majetta initially implemented</li>
</ul>
</html>"));
        end InvertersApartRecord;

        model InvertersExtendedModel "Two inverters with MOS models defined by inheritance"
          extends Modelica.Icons.Example;
          Basic.Ground ground annotation(
            Placement(transformation(extent = {{-40, -80}, {-20, -60}}, rotation = 0)));
          //--------------------------------------------------------------------------------------------------------------
          /*extended model: For each set of technology parameters an apart model has to be defined. Every transistor extends
            this model. In this process the required technology parameters are specified. */

          model MPmos "PMOS transistor with specified modelcard"
            parameter Semiconductors.ModelcardMOS M(GAMMA = 0.37, CBD = 0, CBS = 0);
            extends Semiconductors.M_PMOS(modelcard = M);
            annotation(
              Documentation(info = "<html>
<p>This model MPmos is inherited by the model <i>InverterExtendedModel</i> to build an inverter circuit. For detailed information</p><p>please see <i>InverterExtendedModel</i>.</p>
</html>"));
          end MPmos;

          model MNmos "NMOS transistor with specified modelcard"
            parameter Semiconductors.ModelcardMOS M(GAMMA = 0.37, LAMBDA = 0.02, CBD = 0, CBS = 0);
            extends Semiconductors.M_NMOS(modelcard = M);
            annotation(
              Documentation(info = "<html>
<p>This model MNmos is inherited by the model <i>InverterExtendedModel</i> to build an inverter circuit. For detailed information</p><p>please see <i>InverterExtendedModel</i>.</p>
</html>"));
          end MNmos;

          MPmos mp1(IC = -1e40) annotation(
            Placement(transformation(extent = {{-40, 20}, {-20, 40}}, rotation = 0)));
          MNmos mn1(IC = -1e40) annotation(
            Placement(transformation(extent = {{-40, -30}, {-20, -10}}, rotation = 0)));
          MPmos mp2(IC = -1e40) annotation(
            Placement(transformation(extent = {{0, 20}, {20, 40}}, rotation = 0)));
          MNmos mn2(IC = -1e40) annotation(
            Placement(transformation(extent = {{0, -30}, {20, -10}}, rotation = 0)));
          //--------------------------------------------------------------------------------------------------------------
          Basic.C_Capacitor c1(C = 1e-5, IC = 0, UIC = true) annotation(
            Placement(transformation(origin = {-10, -30}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          Basic.C_Capacitor c2(C = 1e-5, IC = 0, UIC = true) annotation(
            Placement(transformation(origin = {32, -30}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          Sources.V_pulse vin(V2 = 5, TD = 2, TR = 1) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 270, origin = {-60, -22})));
          Sources.V_pulse v(V2 = 5, TR = 1) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 270, origin = {56, -22})));
        equation
          connect(mp1.B, mp1.D) annotation(
            Line(points = {{-20, 30}, {-20, 40}, {-30, 40}}, color = {0, 0, 255}));
          connect(mn1.S, ground.p) annotation(
            Line(points = {{-30, -30}, {-30, -60}}, color = {0, 0, 255}));
          connect(mp1.S, mn1.D) annotation(
            Line(points = {{-30, 20}, {-30, -10}}, color = {0, 0, 255}));
          connect(mn1.G, mp1.G) annotation(
            Line(points = {{-40, -20.1}, {-40, 29.9}}, color = {0, 0, 255}));
          connect(mn1.B, mn1.S) annotation(
            Line(points = {{-20, -20}, {-20, -30}, {-30, -30}}, color = {0, 0, 255}));
          connect(mp2.B, mp2.D) annotation(
            Line(points = {{20, 30}, {20, 40}, {10, 40}}, color = {0, 0, 255}));
          connect(mn2.S, ground.p) annotation(
            Line(points = {{10, -30}, {10, -60}, {-30, -60}}, color = {0, 0, 255}));
          connect(mp2.S, mn2.D) annotation(
            Line(points = {{10, 20}, {10, -10}}, color = {0, 0, 255}));
          connect(mn2.G, mp2.G) annotation(
            Line(points = {{0, -20.1}, {0, 29.9}}, color = {0, 0, 255}));
          connect(mn2.B, mn2.S) annotation(
            Line(points = {{20, -20}, {20, -30}, {10, -30}}, color = {0, 0, 255}));
          connect(mp2.G, mn1.D) annotation(
            Line(points = {{0, 29.9}, {0, 0}, {-30, 0}, {-30, -10}}, color = {0, 0, 255}));
          connect(c1.p, mn1.D) annotation(
            Line(points = {{-10, -20}, {-10, 0}, {-30, 0}, {-30, -10}}, color = {0, 0, 255}));
          connect(mn2.D, c2.p) annotation(
            Line(points = {{10, -10}, {10, 0}, {32, 0}, {32, -20}}, color = {0, 0, 255}));
          connect(c2.n, ground.p) annotation(
            Line(points = {{32, -40}, {32, -60}, {-30, -60}}, color = {0, 0, 255}));
          connect(c1.n, ground.p) annotation(
            Line(points = {{-10, -40}, {-10, -60}, {-30, -60}}, color = {0, 0, 255}));
          connect(vin.p, mp1.G) annotation(
            Line(points = {{-60, -12}, {-60, 26}, {-40, 26}, {-40, 29.9}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(vin.n, ground.p) annotation(
            Line(points = {{-60, -32}, {-60, -60}, {-30, -60}}, color = {0, 0, 0}, smooth = Smooth.None));
          connect(v.p, mp2.D) annotation(
            Line(points = {{56, -12}, {56, 40}, {10, 40}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(mp2.D, mp1.D) annotation(
            Line(points = {{10, 40}, {-30, 40}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(v.n, ground.p) annotation(
            Line(points = {{56, -32}, {56, -60}, {-30, -60}}, color = {0, 0, 0}, smooth = Smooth.None));
          annotation(
            experiment(StopTime = 5),
            Documentation(info = "<html>
<p>An inverter is an electrical circuit that consists of a PMOS and a NMOS. Its task is to turn the input voltage from high potential to low potential or the other way round. This circuit<i> InverterExtendedModel</i> contains two inverters. The input voltage of the first inverter is nearly equal to the output voltage of the second inverter. Capacities cause some differences.</p>
<p>To see the typical behavior of the circuit the input voltages and the output voltages should be plotted. Besides that it can be interesting to watch the output voltage of the first inverter. Simulated until t=5s.</p>
<p>Input voltages: vin.p.v and v.p.v</p>
<p>Output voltage of the first inverter: mn1.ND.v</p>
<p>Output voltage of the second Inverter: mn2.ND.v</p>
<p>This example shows one possibility to make the record of the technology parameters available for more than one transistor. For each set of technology parameters an apart model has to be defined (in this example: MPmos ans MNmos). Inside the model definition the technology parameters are appointed (Spice3.Semiconductors.modelcardMOS M(GAMMA=0.37, LAMBDA=0.02)). Every model extends a transistor. In this process the required technologieparameters are specified (extends Spice3.Repository.MOS(final mtype=1, modelcard=M). To make transistors available in the circuit instances of the defined models are applied (MPmos mp1; MNmos mn1; MPmos mp2; MNmos mn2;).</p>
</html>", revisions = "<html>
<ul>
<li><i>April 2009 </i>by Kristin Majetta initially implemented</li>
</ul>
</html>"));
        end InvertersExtendedModel;

        model FourInverters "Four inverters with MOSFET level 1, using private record as model card"
          extends Modelica.Icons.Example;
          Basic.Ground ground annotation(
            Placement(transformation(extent = {{-74, -80}, {-54, -60}}, rotation = 0)));
          parameter Semiconductors.ModelcardMOS modp(CBD = 0, CBS = 0) "private PMOS modelcard";
          parameter Semiconductors.ModelcardMOS modn(CBD = 0, CBS = 0) "private NMOS modelcard";
          Semiconductors.M_PMOS mp1(modelcard = modp, IC = -1e40) annotation(
            Placement(transformation(extent = {{-74, 20}, {-54, 40}}, rotation = 0)));
          Semiconductors.M_NMOS mn1(modelcard = modn, IC = -1e40) annotation(
            Placement(transformation(extent = {{-74, -30}, {-54, -10}}, rotation = 0)));
          Semiconductors.M_PMOS mp2(modelcard = modp, IC = -1e40) annotation(
            Placement(transformation(extent = {{-34, 20}, {-14, 40}}, rotation = 0)));
          Semiconductors.M_NMOS mn2(modelcard = modn, IC = -1e40) annotation(
            Placement(transformation(extent = {{-34, -30}, {-14, -10}}, rotation = 0)));
          Semiconductors.M_PMOS mp3(modelcard = modp, IC = -1e40) annotation(
            Placement(transformation(extent = {{6, 20}, {26, 40}}, rotation = 0)));
          Semiconductors.M_PMOS mp4(modelcard = modn, IC = -1e40) annotation(
            Placement(transformation(extent = {{46, 20}, {66, 40}}, rotation = 0)));
          Semiconductors.M_NMOS mn3(modelcard = modp, IC = -1e40) annotation(
            Placement(transformation(extent = {{6, -30}, {26, -10}}, rotation = 0)));
          Semiconductors.M_NMOS mn4(modelcard = modn, IC = -1e40) annotation(
            Placement(transformation(extent = {{46, -30}, {66, -10}}, rotation = 0)));
          Basic.C_Capacitor c1(C = 10e-6, IC = 0, UIC = true) annotation(
            Placement(transformation(origin = {-44, -30}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          Basic.C_Capacitor c2(C = 10e-6, IC = 0, UIC = true) annotation(
            Placement(transformation(origin = {-2, -30}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          Basic.C_Capacitor c3(C = 10e-6, IC = 0, UIC = true) annotation(
            Placement(transformation(origin = {36, -30}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          Basic.C_Capacitor c4(C = 10e-6, IC = 0, UIC = true) annotation(
            Placement(transformation(origin = {76, -30}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          Sources.V_pulse vin(V2 = 5, TD = 2, TR = 1) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 270, origin = {-86, -42})));
          Sources.V_pulse v(V2 = 5, TR = 1) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 270, origin = {92, -48})));
        equation
          connect(mp1.B, mp1.D) annotation(
            Line(points = {{-54, 30}, {-54, 40}, {-64, 40}}, color = {0, 0, 255}));
          connect(mn1.S, ground.p) annotation(
            Line(points = {{-64, -30}, {-64, -60}}, color = {0, 0, 255}));
          connect(mp1.S, mn1.D) annotation(
            Line(points = {{-64, 20}, {-64, -10}}, color = {0, 0, 255}));
          connect(mn1.G, mp1.G) annotation(
            Line(points = {{-74, -20.1}, {-74, 29.9}}, color = {0, 0, 255}));
          connect(mn1.B, mn1.S) annotation(
            Line(points = {{-54, -20}, {-54, -30}, {-64, -30}}, color = {0, 0, 255}));
          connect(mp2.B, mp2.D) annotation(
            Line(points = {{-14, 30}, {-14, 40}, {-24, 40}}, color = {0, 0, 255}));
          connect(mn2.S, ground.p) annotation(
            Line(points = {{-24, -30}, {-24, -60}, {-64, -60}}, color = {0, 0, 255}));
          connect(mp2.S, mn2.D) annotation(
            Line(points = {{-24, 20}, {-24, -10}}, color = {0, 0, 255}));
          connect(mn2.G, mp2.G) annotation(
            Line(points = {{-34, -20.1}, {-34, 29.9}}, color = {0, 0, 255}));
          connect(mn2.B, mn2.S) annotation(
            Line(points = {{-14, -20}, {-14, -30}, {-24, -30}}, color = {0, 0, 255}));
          connect(c1.p, mn1.D) annotation(
            Line(points = {{-44, -20}, {-44, 0}, {-64, 0}, {-64, -10}}, color = {0, 0, 255}));
          connect(mn2.D, c2.p) annotation(
            Line(points = {{-24, -10}, {-24, 0}, {-2, 0}, {-2, -20}}, color = {0, 0, 255}));
          connect(c2.n, ground.p) annotation(
            Line(points = {{-2, -40}, {-2, -60}, {-64, -60}}, color = {0, 0, 255}));
          connect(c1.n, ground.p) annotation(
            Line(points = {{-44, -40}, {-44, -60}, {-64, -60}}, color = {0, 0, 255}));
          connect(c3.n, ground.p) annotation(
            Line(points = {{36, -40}, {36, -60}, {-64, -60}}, color = {0, 0, 255}));
          connect(c4.n, ground.p) annotation(
            Line(points = {{76, -40}, {76, -60}, {-64, -60}}, color = {0, 0, 255}));
          connect(mn4.B, mn4.S) annotation(
            Line(points = {{66, -20}, {66, -30}, {56, -30}}, color = {0, 0, 255}));
          connect(mn3.B, mn3.S) annotation(
            Line(points = {{26, -20}, {26, -30}, {16, -30}}, color = {0, 0, 255}));
          connect(mp3.B, mp3.D) annotation(
            Line(points = {{26, 30}, {26, 40}, {16, 40}}, color = {0, 0, 255}));
          connect(mp4.B, mp4.D) annotation(
            Line(points = {{66, 30}, {66, 40}, {56, 40}}, color = {0, 0, 255}));
          connect(mp3.S, mn3.D) annotation(
            Line(points = {{16, 20}, {16, -10}}, color = {0, 0, 255}));
          connect(mp4.S, mn4.D) annotation(
            Line(points = {{56, 20}, {56, -10}}, color = {0, 0, 255}));
          connect(mn3.S, ground.p) annotation(
            Line(points = {{16, -30}, {16, -60}, {-64, -60}}, color = {0, 0, 255}));
          connect(mn4.S, ground.p) annotation(
            Line(points = {{56, -30}, {56, -60}, {-64, -60}}, color = {0, 0, 255}));
          connect(c3.p, mn3.D) annotation(
            Line(points = {{36, -20}, {36, 0}, {16, 0}, {16, -10}}, color = {0, 0, 255}));
          connect(c4.p, mn4.D) annotation(
            Line(points = {{76, -20}, {76, 0}, {56, 0}, {56, -10}}, color = {0, 0, 255}));
          connect(c2.p, mn3.G) annotation(
            Line(points = {{-2, -20}, {2, -20}, {2, -20.1}, {6, -20.1}}, color = {0, 0, 255}));
          connect(mn3.G, mp3.G) annotation(
            Line(points = {{6, -20.1}, {6, 29.9}}, color = {0, 0, 255}));
          connect(c3.p, mn4.G) annotation(
            Line(points = {{36, -20}, {41, -20}, {41, -20.1}, {46, -20.1}}, color = {0, 0, 255}));
          connect(mn4.G, mp4.G) annotation(
            Line(points = {{46, -20.1}, {46, 29.9}}, color = {0, 0, 255}));
          connect(c1.p, mn2.G) annotation(
            Line(points = {{-44, -20}, {-39, -20}, {-39, -20.1}, {-34, -20.1}}, color = {0, 0, 255}));
          connect(vin.p, mn1.G) annotation(
            Line(points = {{-86, -32}, {-86, 0}, {-74, 0}, {-74, -20.1}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(vin.n, ground.p) annotation(
            Line(points = {{-86, -52}, {-86, -60}, {-64, -60}}, color = {0, 0, 0}, smooth = Smooth.None));
          connect(v.p, mp4.D) annotation(
            Line(points = {{92, -38}, {92, 40}, {56, 40}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(v.n, ground.p) annotation(
            Line(points = {{92, -58}, {92, -60}, {-64, -60}}, color = {0, 0, 0}, smooth = Smooth.None));
          connect(mp3.D, mp4.D) annotation(
            Line(points = {{16, 40}, {56, 40}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(mp2.D, mp3.D) annotation(
            Line(points = {{-24, 40}, {16, 40}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(mp1.D, mp2.D) annotation(
            Line(points = {{-64, 40}, {-24, 40}}, color = {0, 0, 255}, smooth = Smooth.None));
          annotation(
            experiment(StopTime = 5),
            Documentation(info = "<html>
<p>This circuit that contains four inverter was designed to show the functionality of the MOS transistor models. To see the behavior of the circuit the output voltages of each inverter should be displayed (mp1.NS.v, mp2.NS.v, mp3.NS.v, mp4.NS.v). The output voltages of the second an fourth inverter and the input voltage of the first inverter have the same potential. The output voltages of the first and third inverter have the opposite potential compared with inverter 2 and 4.</p>
<p>Simulate until t=5s. The output values should be: mp1.NS.v, mp2.NS.v, mp3.NS.v, mp4.NS.v and vgate.p.v</p>
</html>", revisions = "<html>
<ul>
<li><i>April 2009 </i>by Kristin Majetta initially implemented</li>
</ul>
</html>"));
        end FourInverters;

        model Nand "MOS Nand gate circuit"
          extends Modelica.Icons.Example;
          Semiconductors.M_PMOS mp1(L = 2e-5, W = 1e-5, modelcard(PHI = 0.7, CBD = 0, CBS = 0), Sinternal(start = 0), IC = -1e40) annotation(
            Placement(transformation(extent = {{-22, 24}, {-2, 44}}, rotation = 0)));
          Semiconductors.M_PMOS mp2(modelcard(PHI = 0.7, CBD = 0, CBS = 0), IC = -1e40) annotation(
            Placement(transformation(extent = {{24, 24}, {44, 44}}, rotation = 0)));
          Semiconductors.M_NMOS mn2(Dinternal(start = 0), modelcard(CBD = 0, CBS = 0), IC = -1e40) annotation(
            Placement(transformation(extent = {{-24, -44}, {-4, -24}}, rotation = 0)));
          Semiconductors.M_NMOS mn1(modelcard(CBD = 0, CBS = 0), IC = -1e40) annotation(
            Placement(transformation(extent = {{-24, -10}, {-4, 10}}, rotation = 0)));
          Sources.V_constant vconstant(V = 5) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 270, origin = {62, 34})));
          Basic.Ground ground annotation(
            Placement(transformation(extent = {{20, -100}, {40, -80}}, rotation = 0)));
          Sources.V_pulse vin1(TR = 1e-9, TF = 1e-9, V2 = 5, TD = 2e-8, PW = 4e-8, PER = 8e-8) annotation(
            Placement(transformation(origin = {-50, 24}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          Sources.V_pulse vin2(V2 = 5, TR = 1e-9, TF = 1e-9, TD = 1e-8, PW = 4e-8, PER = 8e-8) annotation(
            Placement(transformation(origin = {-52, -44}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
        equation
          connect(mp1.B, mp1.D) annotation(
            Line(points = {{-2, 34}, {-2, 44}, {-12, 44}}, color = {0, 0, 255}));
          connect(mp2.B, mp2.D) annotation(
            Line(points = {{44, 34}, {44, 44}, {34, 44}}, color = {0, 0, 255}));
          connect(vconstant.p, mp2.D) annotation(
            Line(points = {{62, 44}, {62, 44}, {34, 44}}, color = {0, 0, 255}));
          connect(mp2.D, mp1.D) annotation(
            Line(points = {{34, 44}, {-12, 44}}, color = {0, 0, 255}));
          connect(mp1.G, mn1.G) annotation(
            Line(points = {{-22, 33.9}, {-22, 16}, {-24, 16}, {-24, -0.1}}, color = {0, 0, 255}));
          connect(mp1.G, vin1.p) annotation(
            Line(points = {{-22, 33.9}, {-38, 33.9}, {-38, 34}, {-50, 34}}, color = {0, 0, 255}));
          connect(vin1.n, ground.p) annotation(
            Line(points = {{-50, 14}, {-62, 14}, {-62, -80}, {30, -80}}, color = {0, 0, 255}));
          connect(mp2.G, mn2.G) annotation(
            Line(points = {{24, 33.9}, {24, 14}, {-32, 14}, {-32, -34.1}, {-24, -34.1}}, color = {0, 0, 255}));
          connect(mn2.G, vin2.p) annotation(
            Line(points = {{-24, -34.1}, {-38, -34.1}, {-38, -34}, {-52, -34}}, color = {0, 0, 255}));
          connect(vin2.n, ground.p) annotation(
            Line(points = {{-52, -54}, {-52, -80}, {30, -80}}, color = {0, 0, 255}));
          connect(mn2.S, ground.p) annotation(
            Line(points = {{-14, -44}, {-14, -80}, {30, -80}}, color = {0, 0, 255}));
          connect(mn1.B, mn2.B) annotation(
            Line(points = {{-4, 0}, {-4, -34}}, color = {0, 0, 255}));
          connect(mn2.B, mn2.S) annotation(
            Line(points = {{-4, -34}, {-4, -44}, {-14, -44}}, color = {0, 0, 255}));
          connect(mn1.S, mn2.D) annotation(
            Line(points = {{-14, -10}, {-14, -24}}, color = {0, 0, 255}));
          connect(mp1.S, mn1.D) annotation(
            Line(points = {{-12, 24}, {-12, 18}, {-14, 18}, {-14, 10}}, color = {0, 0, 255}));
          connect(mp2.S, mn1.D) annotation(
            Line(points = {{34, 24}, {34, 10}, {-14, 10}}, color = {0, 0, 255}));
          connect(vconstant.n, ground.p) annotation(
            Line(points = {{62, 24}, {62, -80}, {30, -80}}, color = {0, 0, 255}));
          annotation(
            Documentation(info = "<html>
<p>In nearly every electronic the basic circuit &quot;nand&quot; are used. A nand contains two PMOS and two NMOS. The faulty wiring can be seen in the graphical mode. If and only if the two input voltages have high potential, the output voltage has low potential, otherwise the output voltage has high potential.</p>
<p>Nand truth table (1 means true, it is represented by the 5V voltage):</p>
<table cellspacing=\"2\" cellpadding=\"0\" border=\"1\"><tr>
<td><p>input voltage vin1</p></td>
<td><p>input voltage vin2</p></td>
<td><p>output voltage mNMOS1.NS</p></td>
</tr>
<tr>
<td><p>0</p></td>
<td><p>0</p></td>
<td><p>1</p></td>
</tr>
<tr>
<td><p>0</p></td>
<td><p>1</p></td>
<td><p>1</p></td>
</tr>
<tr>
<td><p>1</p></td>
<td><p>0</p></td>
<td><p>1</p></td>
</tr>
<tr>
<td><p>1</p></td>
<td><p>1</p></td>
<td><p>0</p></td>
</tr>
</table>
<p>Simulate until t=2e-7s. Display the two input voltages vin1.p.v and vin2.p.v and the output voltage mNMOS1.ND.v, which becomes zero only if both input values are high.</p>
</html>", revisions = "<html>
<ul>
<li><i>May 2009 </i>by Kristin Majetta initially implemented</li>
</ul>
</html>"),
            experiment(StopTime = 2e-007));
        end Nand;

        model Nor "MOS NOR gate circuit"
          extends Modelica.Icons.Example;
          Semiconductors.M_PMOS mp1(modelcard(RD = 1e-4, RS = 1e-4, CBD = 1e-5, CBS = 1e-5, CGSO = 1e-5, CGDO = 1e-5, CGBO = 1e-5), Dinternal(start = 0, fixed = true), Sinternal(start = 0, fixed = true), IC = -1e40) annotation(
            Placement(transformation(extent = {{-16, 24}, {4, 44}}, rotation = 0)));
          Semiconductors.M_PMOS mp2(modelcard(RD = 1e-4, RS = 1e-4, CBD = 1e-5, CBS = 1e-5, CGSO = 1e-5, CGDO = 1e-5, CGBO = 1e-5), Dinternal(start = 0, fixed = true), Sinternal(start = 0, fixed = true), IC = -1e40) annotation(
            Placement(transformation(extent = {{-16, -6}, {4, 14}}, rotation = 0)));
          Semiconductors.M_NMOS mn1(modelcard(RD = 1e-4, RS = 1e-4, CBD = 1e-5, CBS = 1e-5, CGSO = 1e-5, CGDO = 1e-5, CGBO = 1e-5), Dinternal(start = 0, fixed = true), Sinternal(start = 0, fixed = true), IC = -1e40) annotation(
            Placement(transformation(extent = {{-16, -44}, {4, -24}}, rotation = 0)));
          Semiconductors.M_NMOS mn2(modelcard(RD = 1e-4, RS = 1e-4, CBD = 1e-5, CBS = 1e-5, CGSO = 1e-5, CGDO = 1e-5, CGBO = 1e-5), Dinternal(start = 0, fixed = true), Sinternal(start = 0, fixed = true), IC = -1e40) annotation(
            Placement(transformation(extent = {{32, -44}, {52, -24}}, rotation = 0)));
          Basic.Ground ground annotation(
            Placement(transformation(extent = {{28, -100}, {48, -80}}, rotation = 0)));
          Sources.V_pulse vin1(V2 = 5, TR = 0.001, TF = 0.001, PW = 2, PER = 10, TD = 2) annotation(
            Placement(transformation(origin = {-42, 24}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          Sources.V_pulse vin2(V2 = 5, TR = 0.001, TF = 0.001, PW = 2, PER = 10, TD = 1) annotation(
            Placement(transformation(origin = {-44, -44}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          Sources.V_pulse v(TD = 0.5, TR = 0.1, V2 = 5) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 270, origin = {62, 34})));
        equation
          connect(mn1.B, mn1.S) annotation(
            Line(points = {{4, -34}, {4, -44}, {-6, -44}}, color = {0, 0, 255}));
          connect(mn2.B, mn2.S) annotation(
            Line(points = {{52, -34}, {52, -44}, {42, -44}}, color = {0, 0, 255}));
          connect(mn2.S, ground.p) annotation(
            Line(points = {{42, -44}, {38, -44}, {38, -80}}, color = {0, 0, 255}));
          connect(mn1.S, ground.p) annotation(
            Line(points = {{-6, -44}, {-6, -80}, {38, -80}}, color = {0, 0, 255}));
          connect(vin2.p, mn1.G) annotation(
            Line(points = {{-44, -34}, {-29, -34}, {-29, -34.1}, {-16, -34.1}}, color = {0, 0, 255}));
          connect(vin2.n, ground.p) annotation(
            Line(points = {{-44, -54}, {-44, -80}, {38, -80}}, color = {0, 0, 255}));
          connect(vin1.p, mp1.G) annotation(
            Line(points = {{-42, 34}, {-29, 34}, {-29, 33.9}, {-16, 33.9}}, color = {0, 0, 255}));
          connect(vin1.n, ground.p) annotation(
            Line(points = {{-42, 14}, {-64, 14}, {-64, -80}, {38, -80}}, color = {0, 0, 255}));
          connect(mp1.S, mp2.D) annotation(
            Line(points = {{-6, 24}, {-6, 14}}, color = {0, 0, 255}));
          connect(mp2.S, mn1.D) annotation(
            Line(points = {{-6, -6}, {-6, -24}}, color = {0, 0, 255}));
          connect(mn2.D, mn1.D) annotation(
            Line(points = {{42, -24}, {-6, -24}}, color = {0, 0, 255}));
          connect(vin2.p, mp2.G) annotation(
            Line(points = {{-44, -34}, {-44, 4}, {-16, 4}, {-16, 3.9}}, color = {0, 0, 255}));
          connect(vin1.p, mn2.G) annotation(
            Line(points = {{-42, 34}, {-28, 34}, {-28, -18}, {32, -18}, {32, -34.1}}, color = {0, 0, 255}));
          connect(mp1.B, mp1.D) annotation(
            Line(points = {{4, 34}, {4, 44}, {-6, 44}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(mp2.B, mp1.B) annotation(
            Line(points = {{4, 4}, {4, 34}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(v.p, mp1.D) annotation(
            Line(points = {{62, 44}, {-6, 44}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(v.n, ground.p) annotation(
            Line(points = {{62, 24}, {62, -80}, {38, -80}}, color = {0, 0, 0}, smooth = Smooth.None));
          annotation(
            Documentation(info = "<html>
<p>In nearly every electronic the basic circuit &quot;nor&quot; is used. A nor contains two PMOS and two NMOS. The faulty wiring can be seen in the graphical mode. If and only if the two input voltages have low potential, the output voltage has high potential, otherwise the output voltage has low potential.</p>
<p>Nor truth table (1 means true, it is represented by the 5V voltage):</p>
<table cellspacing=\"2\" cellpadding=\"0\" border=\"1\"><tr>
<td><p>input voltage vin1</p></td>
<td><p>input voltage vin2</p></td>
<td><p>output voltage m_PMOS1.NS</p></td>
</tr>
<tr>
<td><p>0</p></td>
<td><p>0</p></td>
<td><p>1</p></td>
</tr>
<tr>
<td><p>0</p></td>
<td><p>1</p></td>
<td><p>0</p></td>
</tr>
<tr>
<td><p>1</p></td>
<td><p>0</p></td>
<td><p>0</p></td>
</tr>
<tr>
<td><p>1</p></td>
<td><p>1</p></td>
<td><p>0</p></td>
</tr>
</table>
<p>Simulate until t=5s. Display the two input voltages vin1.p.v and vin2.p.v and the output voltage mPMOS1.NS.v.</p>
<p>The output value in the example shows a behaviour &quot;near&quot; the one of the truth table, since the capacitances are huge. Therefore loading is not finished before the next input changes.</p>
</html>", revisions = "<html>
<ul>
<li><i>March 2009 </i>by Kristin Majetta initially implemented</li>
</ul>
</html>"),
            experiment(StopTime = 5));
        end Nor;

        model Graetz "Graetz rectifier circuit"
          extends Modelica.Icons.Example;
          Semiconductors.D_DIODE D1(IC = -1e40, SENS_AREA = false, modelcarddiode(CJO = 1e-7), pin(start = 0, fixed = true)) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0, origin = {0, 14})));
          Semiconductors.D_DIODE D3(IC = -1e40, SENS_AREA = false, modelcarddiode(CJO = 1e-7), n(v(start = 0, fixed = true))) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 270, origin = {20, -8})));
          Semiconductors.D_DIODE D4(IC = -1e40, SENS_AREA = false, modelcarddiode(CJO = 1e-7)) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0, origin = {1, -30})));
          Semiconductors.D_DIODE D2(IC = -1e40, SENS_AREA = false, modelcarddiode(CJO = 1e-7)) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 270, origin = {-20, -8})));
          Semiconductors.R_Resistor rout(R = 10) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 90, origin = {42, -7})));
          Sources.V_sin vsin(VA = 10, FREQ = 200) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 270, origin = {-44, -8})));
          Basic.Ground ground annotation(
            Placement(transformation(extent = {{-30, -60}, {-10, -40}})));
        equation
          connect(D1.n, D3.p) annotation(
            Line(points = {{10, 14}, {20, 14}, {20, 2}}, color = {0, 0, 0}, smooth = Smooth.None));
          connect(D2.p, D1.p) annotation(
            Line(points = {{-20, 2}, {-20, 14}, {-10, 14}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(D4.n, D3.n) annotation(
            Line(points = {{11, -30}, {20, -30}, {20, -18}}, color = {0, 0, 0}, smooth = Smooth.None));
          connect(D4.p, D2.n) annotation(
            Line(points = {{-9, -30}, {-20, -30}, {-20, -18}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(D4.p, ground.p) annotation(
            Line(points = {{-9, -30}, {-20, -30}, {-20, -40}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(vsin.n, ground.p) annotation(
            Line(points = {{-44, -18}, {-44, -40}, {-20, -40}}, color = {0, 0, 0}, smooth = Smooth.None));
          connect(D4.n, rout.p) annotation(
            Line(points = {{11, -30}, {42, -30}, {42, -17}}, color = {0, 0, 0}, smooth = Smooth.None));
          connect(rout.n, D1.p) annotation(
            Line(points = {{42, 3}, {42, 26}, {-20, 26}, {-20, 14}, {-10, 14}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(D3.p, vsin.p) annotation(
            Line(points = {{20, 2}, {20, 40}, {-44, 40}, {-44, 2}}, color = {0, 0, 255}, smooth = Smooth.None));
          annotation(
            experiment(StopTime = 0.025),
            Documentation(info = "<html>
<p>The Graetz rectifier circuit is to show the behaviour of diodes.</p>
<p>Simulator until 0.025 s. Then display the input voltage vsin.p.v. The rout.p.v voltage is the rectified voltage result, which is pulsing mostly in the positive range.</p>
</html>", revisions = "<html>
<ul>
<li><i>Jan. 2010 </i>by Kristin Majetta initially implemented</li>
</ul>
</html>"));
        end Graetz;

        model Oscillator "Oscillator circuit"
          extends Modelica.Icons.Example;
          Basic.R_Resistor r(R = 1000) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 90, origin = {-36, 20})));
          Basic.C_Capacitor c(C = 1e-7) annotation(
            Placement(transformation(extent = {{-32, 0}, {-12, 20}})));
          Basic.C_Capacitor c1(C = 1e-7) annotation(
            Placement(transformation(extent = {{24, 0}, {44, 20}})));
          Basic.R_Resistor r1(R = 22000) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 90, origin = {-6, 20})));
          Basic.R_Resistor r2(R = 22000) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 90, origin = {18, 20})));
          Basic.R_Resistor r3(R = 1000) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 90, origin = {50, 20})));
          Semiconductors.Q_NPNBJT T1(modelcard(CJE = 1e-9, CJC = 1e-9), vbe(start = 0, fixed = true)) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 180, origin = {-30, -24})));
          Semiconductors.Q_NPNBJT T2(modelcard(CJE = 1e-9, CJC = 1e-9), vbe(start = 0, fixed = true)) annotation(
            Placement(transformation(extent = {{30, -40}, {50, -20}})));
          Basic.Ground ground1 annotation(
            Placement(transformation(extent = {{-78, -34}, {-58, -14}})));
          Basic.Ground ground2 annotation(
            Placement(transformation(extent = {{50, -60}, {70, -40}})));
          Basic.R_Resistor r4(R = 10000) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 270, origin = {64, -2})));
          Sources.V_pulse v(V2 = 8, TD = 0.0005, TR = 0.01, PW = 1000, PER = 1000, TF = 0) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 270, origin = {-68, 20})));
        equation
          connect(r.n, r1.n) annotation(
            Line(points = {{-36, 30}, {-6, 30}}, color = {0, 0, 0}, smooth = Smooth.None));
          connect(r1.n, r2.n) annotation(
            Line(points = {{-6, 30}, {18, 30}}, color = {0, 0, 0}, smooth = Smooth.None));
          connect(r2.n, r3.n) annotation(
            Line(points = {{18, 30}, {50, 30}}, color = {0, 0, 0}, smooth = Smooth.None));
          connect(r.p, c.p) annotation(
            Line(points = {{-36, 10}, {-32, 10}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(c.n, r1.p) annotation(
            Line(points = {{-12, 10}, {-6, 10}}, color = {0, 0, 0}, smooth = Smooth.None));
          connect(r2.p, c1.p) annotation(
            Line(points = {{18, 10}, {24, 10}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(c1.n, r3.p) annotation(
            Line(points = {{44, 10}, {50, 10}}, color = {0, 0, 0}, smooth = Smooth.None));
          connect(r1.p, T2.B) annotation(
            Line(points = {{-6, 10}, {-6, -30}, {30.4, -30}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(r2.p, T1.B) annotation(
            Line(points = {{18, 10}, {18, -24}, {-20.4, -24}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(T1.C, r.p) annotation(
            Line(points = {{-33, -34}, {-48, -34}, {-48, 10}, {-36, 10}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(T1.E, ground1.p) annotation(
            Line(points = {{-33, -14}, {-68, -14}}, color = {0, 0, 0}, smooth = Smooth.None));
          connect(T2.E, ground2.p) annotation(
            Line(points = {{43, -40}, {60, -40}}, color = {0, 0, 0}, smooth = Smooth.None));
          connect(r3.p, T2.C) annotation(
            Line(points = {{50, 10}, {50, -20}, {43, -20}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(r3.p, r4.p) annotation(
            Line(points = {{50, 10}, {58, 10}, {58, 8}, {64, 8}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(ground2.p, r4.n) annotation(
            Line(points = {{60, -40}, {60, -12}, {64, -12}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(v.p, r.n) annotation(
            Line(points = {{-68, 30}, {-36, 30}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(v.n, ground1.p) annotation(
            Line(points = {{-68, 10}, {-68, -14}}, color = {0, 0, 0}, smooth = Smooth.None));
          annotation(
            experiment(StopTime = 0.025),
            Documentation(info = "<html>
<p>The oscillator circuit demonstrates the usage of BJT transistors.</p>
<p>Simulate until 0.025 s. Display v.p.v, which is rising until 5 V. Furthermore display r4.p.v, which starts oscillating.</p>
</html>", revisions = "<html>
<ul>
<li><i>Jan. 2010 </i>by Jonathan Gerbet initially implemented</li>
</ul>
</html>"));
        end Oscillator;

        model CoupledInductors
          extends Modelica.Icons.Example;
          Modelica.Electrical.Spice3.Basic.Ground ground annotation(
            Placement(transformation(extent = {{-100, -100}, {-80, -80}})));
          Modelica.Electrical.Spice3.Sources.V_sin sineVoltage(VA = 220, FREQ = 50) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 270, origin = {-90, 20})));
          Modelica.Electrical.Spice3.Basic.R_Resistor R1(R = 1) annotation(
            Placement(transformation(extent = {{-80, 70}, {-60, 90}})));
          Modelica.Electrical.Spice3.Basic.L_Inductor L1(L = 1, UIC = false) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 270, origin = {-40, 40})));
          Modelica.Electrical.Spice3.Basic.L_Inductor L2(L = 0.01) annotation(
            Placement(transformation(extent = {{-10, 10}, {10, -10}}, rotation = 270, origin = {20, 60})));
          Modelica.Electrical.Spice3.Basic.R_Resistor R2(R = 1) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 180, origin = {42, 80})));
          Modelica.Electrical.Spice3.Basic.K_CoupledInductors k1(k = 0.1) annotation(
            Placement(transformation(extent = {{-20, 50}, {0, 70}})));
          Modelica.Electrical.Spice3.Basic.R_Resistor R3(R = 1000) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 270, origin = {60, 50})));
          Modelica.Electrical.Spice3.Basic.C_Capacitor C1(C = 1.e-6) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 270, origin = {80, 50})));
          Modelica.Electrical.Spice3.Basic.L_Inductor L3(L = 0.01) annotation(
            Placement(transformation(extent = {{-10, 10}, {10, -10}}, rotation = 270, origin = {20, -40})));
          Modelica.Electrical.Spice3.Basic.R_Resistor R4(R = 1) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 180, origin = {42, -20})));
          Modelica.Electrical.Spice3.Basic.R_Resistor R5(R = 1000) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 270, origin = {60, -50})));
          Modelica.Electrical.Spice3.Basic.C_Capacitor C2(C = 2.e-3) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 270, origin = {80, -50})));
          Modelica.Electrical.Spice3.Basic.K_CoupledInductors k2(k = 0.05) annotation(
            Placement(transformation(extent = {{-20, -50}, {0, -30}})));
          Modelica.Electrical.Spice3.Basic.K_CoupledInductors k3(k = 0.05) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 90, origin = {6, 10})));
        equation
          connect(sineVoltage.n, ground.p) annotation(
            Line(points = {{-90, 10}, {-90, -80}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(R1.p, sineVoltage.p) annotation(
            Line(points = {{-80, 80}, {-90, 80}, {-90, 30}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(R1.n, L1.p) annotation(
            Line(points = {{-60, 80}, {-40, 80}, {-40, 50}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(L1.n, ground.p) annotation(
            Line(points = {{-40, 30}, {-40, -80}, {-90, -80}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(C1.n, R3.n) annotation(
            Line(points = {{80, 40}, {60, 40}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(L2.n, R3.n) annotation(
            Line(points = {{20, 50}, {20, 40}, {60, 40}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(C1.p, R2.p) annotation(
            Line(points = {{80, 60}, {80, 80}, {52, 80}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(R3.p, R2.p) annotation(
            Line(points = {{60, 60}, {60, 80}, {52, 80}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(R2.n, L2.p) annotation(
            Line(points = {{32, 80}, {20, 80}, {20, 70}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(C2.n, R5.n) annotation(
            Line(points = {{80, -60}, {60, -60}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(L3.n, R5.n) annotation(
            Line(points = {{20, -50}, {20, -60}, {60, -60}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(C2.p, R4.p) annotation(
            Line(points = {{80, -40}, {80, -20}, {52, -20}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(R5.p, R4.p) annotation(
            Line(points = {{60, -40}, {60, -20}, {52, -20}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(R4.n, L3.p) annotation(
            Line(points = {{32, -20}, {20, -20}, {20, -30}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(C1.n, ground.p) annotation(
            Line(points = {{80, 40}, {90, 40}, {90, -80}, {-90, -80}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(C2.n, ground.p) annotation(
            Line(points = {{80, -60}, {90, -60}, {90, -80}, {-90, -80}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(L1.ICP, k2.inductiveCouplePin1) annotation(
            Line(points = {{-31.6, 40}, {-28, 40}, {-28, -40}, {-20, -40}}, color = {170, 85, 255}, smooth = Smooth.None));
          connect(k2.inductiveCouplePin2, L3.ICP) annotation(
            Line(points = {{0, -40}, {11.6, -40}}, color = {170, 85, 255}, smooth = Smooth.None));
          connect(L3.ICP, k3.inductiveCouplePin1) annotation(
            Line(points = {{11.6, -40}, {6, -40}, {6, 0}}, color = {170, 85, 255}, smooth = Smooth.None));
          connect(k3.inductiveCouplePin2, L2.ICP) annotation(
            Line(points = {{6, 20}, {6, 60}, {11.6, 60}}, color = {170, 85, 255}, smooth = Smooth.None));
          connect(L2.ICP, k1.inductiveCouplePin2) annotation(
            Line(points = {{11.6, 60}, {0, 60}}, color = {170, 85, 255}, smooth = Smooth.None));
          connect(L1.ICP, k1.inductiveCouplePin1) annotation(
            Line(points = {{-31.6, 40}, {-28, 40}, {-28, 60}, {-20, 60}}, color = {170, 85, 255}, smooth = Smooth.None));
          annotation(
            experiment(StopTime = 0.2),
            Documentation(info = "<html>
<p>The coupled inductor circuit demonstrates how different inductors can be coupled using the Component K_CoupledInductors from package Basic</p>
<p>Simulate until 0.2s, and display the behaviour of the parts that are coupled via <code>K</code>, such as <code>C1.p.v</code>, and <code>C2.p.v</code>.</p>
</html>"));
        end CoupledInductors;

        model CascodeCircuit
          extends Modelica.Icons.Example;
          Modelica.Electrical.Spice3.Semiconductors.J_NJFJFET J1(S(v(start = 0)), modelcard(CGS = 0, CGD = 0), AREA = 1) annotation(
            Placement(transformation(extent = {{-54, 38}, {-34, 58}})));
          Modelica.Electrical.Spice3.Semiconductors.J_NJFJFET J2(modelcard(CGS = 0, CGD = 0), AREA = 1) annotation(
            Placement(transformation(extent = {{-54, 4}, {-34, 24}})));
          Modelica.Electrical.Spice3.Basic.R_Resistor RC(R = 200, v(start = 0)) annotation(
            Placement(transformation(extent = {{-44, 60}, {-24, 80}})));
          Modelica.Electrical.Spice3.Basic.Ground ground annotation(
            Placement(transformation(extent = {{-54, -26}, {-34, -6}})));
          Modelica.Electrical.Spice3.Sources.V_constant UDD(V = 10) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 270, origin = {-10, 60})));
          Modelica.Electrical.Spice3.Sources.V_constant U0(V = 2) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 270, origin = {-72, 38})));
          Modelica.Electrical.Spice3.Sources.V_sin v_sin(FREQ = 10, VA = 2) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 270, origin = {-66, 4})));
        equation
          connect(J2.D, J1.S) annotation(
            Line(points = {{-44, 24}, {-44, 38}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(ground.p, J2.S) annotation(
            Line(points = {{-44, -6}, {-44, 4}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(v_sin.p, J2.G) annotation(
            Line(points = {{-66, 14}, {-60, 14}, {-60, 13.9}, {-54, 13.9}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(v_sin.n, ground.p) annotation(
            Line(points = {{-66, -6}, {-44, -6}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(U0.p, J1.G) annotation(
            Line(points = {{-72, 48}, {-63, 48}, {-63, 47.9}, {-54, 47.9}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(U0.n, ground.p) annotation(
            Line(points = {{-72, 28}, {-80, 28}, {-80, -6}, {-44, -6}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(J1.D, RC.p) annotation(
            Line(points = {{-44, 58}, {-44, 70}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(UDD.p, RC.n) annotation(
            Line(points = {{-10, 70}, {-24, 70}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(UDD.n, ground.p) annotation(
            Line(points = {{-10, 50}, {-10, -6}, {-44, -6}}, color = {0, 0, 255}, smooth = Smooth.None));
          annotation(
            Documentation(info = "<html>
<p>This model is a simple JFET cascode circuit. The J2 gate variation (v_sin.p.v) is transformed to the J2 drain variation (J2.D.v). </p>
<p><br/>Simulate until 0.2s, and display the mentioned voltages.</p>
</html>", revisions = "<html>
<ul>
<li><i>Aug. 2011 </i>by Kristin Majetta initially implemented</li>
</ul>
</html>"),
            experiment(StopTime = 0.2, Interval = 2e-4, Tolerance = 1e-007));
        end CascodeCircuit;

        model Spice3BenchmarkDifferentialPair "Simple differential pair"
          extends Modelica.Icons.Example;
          Sources.V_pulse VCC(V1 = 0, V2 = 12, TD = 0, TR = 2e-009, TF = 2e-009, PW = 3);
          Sources.V_pulse VEE(V1 = 0, V2 = -12, TD = 0, TR = 2e-009, TF = 2e-009, PW = 3);
          Sources.V_sin VIN(VO = 0, VA = 0.01, FREQ = 5);
          Basic.R_Resistor RS1(R = 1000, v(start = 0));
          Basic.R_Resistor RS2(R = 1000);
          Semiconductors.Q_NPNBJT Q1(modelcard = MOD1, Binternal(start = 0), vbe(start = 0, fixed = true));
          Semiconductors.Q_NPNBJT Q2(modelcard = MOD1, icapbe(start = 0), vbc(start = 0), vbe(start = 0, fixed = true));
          Basic.R_Resistor RC1(R = 10000, v(start = 0));
          Basic.R_Resistor RC2(R = 10000);
          Basic.R_Resistor RE(R = 10000);
          Sources.V_constant VIE(V = 0);
          parameter Semiconductors.ModelcardBJT MOD1(BF = 50, VAF = 50, IS = 1e-012, RB = 100, CJC = 5e-09, TF = 6e-010);
          Basic.Ground g;
          Real OutputVoltage;
        protected
          Modelica.Electrical.Analog.Interfaces.Pin n7;
          Modelica.Electrical.Analog.Interfaces.Pin n0;
          Modelica.Electrical.Analog.Interfaces.Pin n8;
          Modelica.Electrical.Analog.Interfaces.Pin n1;
          Modelica.Electrical.Analog.Interfaces.Pin n2;
          Modelica.Electrical.Analog.Interfaces.Pin n6;
          Modelica.Electrical.Analog.Interfaces.Pin n3;
          Modelica.Electrical.Analog.Interfaces.Pin n42;
          Modelica.Electrical.Analog.Interfaces.Pin n5;
          Modelica.Electrical.Analog.Interfaces.Pin n41;
        initial equation
          der(Q1.vbc) = 0;
          der(Q2.vbc) = 0;
        equation
          OutputVoltage = Q2.C.v - Q1.C.v;
          connect(g.p, n0);
          connect(VCC.p, n7);
          connect(VCC.n, n0);
          connect(VEE.p, n8);
          connect(VEE.n, n0);
          connect(VIN.p, n1);
          connect(VIN.n, n0);
          connect(RS1.p, n1);
          connect(RS1.n, n2);
          connect(RS2.p, n6);
          connect(RS2.n, n0);
          connect(Q1.C, n3);
          connect(Q1.B, n2);
          connect(Q1.E, n42);
          connect(Q2.C, n5);
          connect(Q2.B, n6);
          connect(Q2.E, n42);
          connect(RC1.p, n7);
          connect(RC1.n, n3);
          connect(RC2.p, n7);
          connect(RC2.n, n5);
          connect(RE.p, n41);
          connect(RE.n, n8);
          connect(VIE.p, n41);
          connect(VIE.n, n42);
          annotation(
            experiment(StopTime = 1, Interval = 0.001, Tolerance = 1e-005),
            Documentation(info = "<html>
<p>This Differential pair model is one of the five benchmark circuits described in the SPICE3 Version e3 User&apos;s Manual (see information of package Spice3).</p>
<p>The differential pair circuit is operating in the differential mode. This means the input voltage VIN that is only applied at one transistor
is amplified. To comprehend this behavior the user is recommended to simulate from t=0 to t=1s and observe \"VIN.p.v\" which is the input voltage and \"Outputvoltage\" which is the
amplified output voltage.</p>
<p>Original SPICE3 netlist of the Differential pair:</p>
<pre>
SIMPLE DIFFERENTIAL PAIR<br/>
VCC 7 0 12<br/>
VEE 8 0 -12<br/>
VIN 1 0 AC 1<br/>
RS1 1 2 1K<br/>
RS2 6 0 1K<br/>
Q1 3 2 4 MOD1<br/>
Q2 5 6 4 MOD1<br/>
RC1 7 3 10K<br/>
RC2 7 5 10K<br/>
RE 4 8 10K<br/>
.MODEL MOD1 NPN BF=50 VAF=50 IS=1.E-12 RB=100 CJC=.5PF TF=.6NS<br/>
.TF V(5) VIN<br/>
.AC DEC 10 1 100MEG<br/>
.END
</pre>
<p>In the Modelica representation the tiny capacity CJC was set to 1.e-9F to get a higher numerical
robustness. In future the numerical behavior will be analysed to be able to apply the original value
CJC=.5PF.</p>
</html>"));
        end Spice3BenchmarkDifferentialPair;

        model Spice3BenchmarkMosfetCharacterization "Mos output characteristics"
          extends Modelica.Icons.Example;
          Sources.V_pulse VDS(V1 = 0, V2 = 10, TD = 0, TR = 1e-008, TF = 1e-008, PW = 1, PER = 1);
          Sources.V_pulse VGS(V1 = 0, V2 = 5, TD = 4e-008, TR = 1e-009, TF = 1e-009, PW = 1e-008, PER = 2e-008);
          Semiconductors.M_NMOS M1(modelcard = MOD1, L = 4e-006, W = 6e-006, AD = 1e-011, AS = 1e-011, IC = -1e40);
          // * VIDS MEASURES ID, WE COULD HAVE USED VDS, BUT ID WOULD BE NEGATIVE
          Sources.V_constant VIDS(V = 0);
          parameter Semiconductors.ModelcardMOS MOD1(VTO = -2, NSUB = 1e+015, UO = 550, CBD = 0, CBS = 0);

          record SpiceConstants
            // NODE
            // NOPAGE
          end SpiceConstants;

          Basic.Ground g;
        protected
          Modelica.Electrical.Analog.Interfaces.Pin n3;
          Modelica.Electrical.Analog.Interfaces.Pin n0;
          Modelica.Electrical.Analog.Interfaces.Pin n2;
          Modelica.Electrical.Analog.Interfaces.Pin n1;
        equation
          connect(g.p, n0);
          connect(VDS.p, n3);
          connect(VDS.n, n0);
          connect(VGS.p, n2);
          connect(VGS.n, n0);
          connect(M1.D, n1);
          connect(M1.G, n2);
          connect(M1.S, n0);
          connect(M1.B, n0);
          connect(VIDS.p, n3);
          connect(VIDS.n, n1);
          annotation(
            experiment(StopTime = 1e-007, Interval = 1e-009),
            Documentation(info = "<html>
<p>This Mosfet Characterization model is one of the five benchmark circuits described in the SPICE3 Version e3 User&apos;s Manual (see information of package Spice3). </p>
<p>This circuit is a very simple one than consists of an NMOS transistor level 1 that is connected to voltage sources at gate and drain node whereas the drain voltage source supplies the
operating voltage. The user is recommended to simulate from t=0 to t=1e-7s and observe the gate node voltage (\"VGS.p.v\") and the transistor current (\"M1.D.i\").
It can be seen that the current is increasing with increasing gate voltage which means the conductivity of the transistor is increasing. The opposite case occurs for decreasing gate voltage. </p>
<p>Original SPICE3 netlist of the MOSFET characterization circuit:</p>
<pre>
MOS OUTPUT CHARACTERISTICS<br/>
.OPTIONS NODE NOPAGE<br/>
VDS 3 0<br/>
VGS 2 0<br/>
M1 1 2 0 0 MOD1 L=4U W=6U AD=10P AS=10P<br/>
*VIDS MEASURES ID, WE COULD HAVE USED VDS, BUT IT WOULD BE NEGATIVE VIDS 3 1
.MODEL MOD1 NMOS VTO=-2 NSUB=1.0E15 UO=550<br/>
.DC VDS 0 10 .5 VGS 0 5 1<br/>
.END
</pre>
</html>"));
        end Spice3BenchmarkMosfetCharacterization;

        model Spice3BenchmarkRtlInverter "Simple RTL inverter"
          extends Modelica.Icons.Example;
          Sources.V_constant VCC(V = 5);
          Sources.V_pulse VIN(V1 = 0, V2 = 5, TD = 2e-009, TR = 2e-009, TF = 2e-009, PW = 3e-008);
          Basic.R_Resistor RB(R = 10000);
          Semiconductors.Q_NPNBJT Q1(modelcard = Q11, vbc(start = 0, fixed = true), vbe(start = 0, fixed = true));
          Basic.R_Resistor RC(R = 1000);
          parameter Semiconductors.ModelcardBJT Q11(BF = 20, RB = 100, TF = 1e-010, CJC = 2e-012);
          Basic.Ground g;
        protected
          Modelica.Electrical.Analog.Interfaces.Pin n4;
          Modelica.Electrical.Analog.Interfaces.Pin n0;
          Modelica.Electrical.Analog.Interfaces.Pin n1;
          Modelica.Electrical.Analog.Interfaces.Pin n2;
          Modelica.Electrical.Analog.Interfaces.Pin n3;
        equation
          connect(g.p, n0);
          connect(VCC.p, n4);
          connect(VCC.n, n0);
          connect(VIN.p, n1);
          connect(VIN.n, n0);
          connect(RB.p, n1);
          connect(RB.n, n2);
          connect(Q1.C, n3);
          connect(Q1.B, n2);
          connect(Q1.E, n0);
          connect(RC.p, n3);
          connect(RC.n, n4);
          annotation(
            experiment(StopTime = 1e-007, Interval = 1e-010, Tolerance = 1e-008),
            Documentation(info = "<html>
<p>This RTL Inverter model is one of the five benchmark circuits described in the SPICE3 Version e3 User&apos;s Manual (see information of package Spice3). </p>
<p>This simple RTL inverter (resistor transistor logic) circuits inverts the input voltage which means the output voltage has high potential if the input voltage
has low potential and the other way round. To comprehend this behaviour the user is recommended to simulate from t=0 to t=1e-7s and observe the input voltage (VIN.p.v)
and the output voltage (Q1.C.v)</p>
<p>Original SPICE3 netlist of the RTL inverter:</p>
<pre>
SIMPLE RTL INVERTER<br/>
VCC 4 0 5<br/>
VIN 1 0 PULSE 0 5 2NS 2NS 2NS 30NS<br/>
RB 1 2 10K<br/>
Q1 3 2 0 Q1<br/>
RC 3 4 1K<br/>
.MODEL Q1 NPN BF 20 RB 100 TF .1NS CJC 2PF<br/>
.DC VIN 0 5 0.1<br/>
.TRAN 1NS 100NS<br/>
.END
</pre>
</html>"));
        end Spice3BenchmarkRtlInverter;

        model Spice3BenchmarkFourBitBinaryAdder "ADDER - 4 BIT ALL-NAND-GATE BINARY ADDER"
          extends Modelica.Icons.Example;
          import Modelica.Electrical.Spice3.*;
          output Real X1_p9_v = X1.p9.v;
          output Real X1_p10_v = X1.p10.v;
          output Real X1_p11_v = X1.p11.v;
          output Real X1_p12_v = X1.p12.v;
          output Real X1_p14_v = X1.p14.v;
          output Real X1_p1_v = X1.p1.v;
          output Real X1_p2_v = X1.p2.v;
          output Real X1_p3_v = X1.p3.v;
          output Real X1_p4_v = X1.p4.v;
          output Real X1_p5_v = X1.p5.v;
          output Real X1_p6_v = X1.p6.v;
          output Real X1_p7_v = X1.p7.v;
          output Real X1_p8_v = X1.p8.v;
          // *** SUBCIRCUIT DEFINITIONS

          model NAND ".SUBCKT NAND 1 2 3 4"
            parameter Semiconductors.ModelcardDIODE DMOD;
            parameter Semiconductors.ModelcardBJT QMOD(BF = 75, RB = 100, CJE = 1e-012, CJC = 3e-012);
            // *   NODES:  INPUT(2), OUTPUT, VCC
            Semiconductors.Q_NPNBJT Q1(modelcard = QMOD);
            Semiconductors.D_DIODE D1CLAMP(modelcarddiode = DMOD, SENS_AREA = false, IC = -1e40);
            Semiconductors.Q_NPNBJT Q2(modelcard = QMOD);
            Semiconductors.D_DIODE D2CLAMP(modelcarddiode = DMOD, SENS_AREA = false, IC = -1e40);
            Basic.R_Resistor RB(R = 4000);
            Basic.R_Resistor R1(R = 1600);
            Semiconductors.Q_NPNBJT Q3(modelcard = QMOD);
            Basic.R_Resistor R2(R = 1000);
            Basic.R_Resistor RC(R = 130);
            Semiconductors.Q_NPNBJT Q4(modelcard = QMOD);
            Semiconductors.D_DIODE DVBEDROP(modelcarddiode = DMOD, SENS_AREA = false, IC = -1e40);
            Semiconductors.Q_NPNBJT Q5(modelcard = QMOD);
            Basic.Ground g;
            Modelica.Electrical.Analog.Interfaces.Pin p1;
            Modelica.Electrical.Analog.Interfaces.Pin p2;
            Modelica.Electrical.Analog.Interfaces.Pin p3;
            Modelica.Electrical.Analog.Interfaces.Pin p4;
          protected
            Modelica.Electrical.Analog.Interfaces.Pin n1;
            Modelica.Electrical.Analog.Interfaces.Pin n2;
            Modelica.Electrical.Analog.Interfaces.Pin n3;
            Modelica.Electrical.Analog.Interfaces.Pin n4;
            Modelica.Electrical.Analog.Interfaces.Pin n9;
            Modelica.Electrical.Analog.Interfaces.Pin n5;
            Modelica.Electrical.Analog.Interfaces.Pin n0;
            Modelica.Electrical.Analog.Interfaces.Pin n6;
            Modelica.Electrical.Analog.Interfaces.Pin n8;
            Modelica.Electrical.Analog.Interfaces.Pin n7;
            Modelica.Electrical.Analog.Interfaces.Pin n10;
          equation
            connect(g.p, n0);
            connect(p1, n1);
            connect(p2, n2);
            connect(p3, n3);
            connect(p4, n4);
            connect(Q1.C, n9);
            connect(Q1.B, n5);
            connect(Q1.E, n1);
            connect(D1CLAMP.p, n0);
            connect(D1CLAMP.n, n1);
            connect(Q2.C, n9);
            connect(Q2.B, n5);
            connect(Q2.E, n2);
            connect(D2CLAMP.p, n0);
            connect(D2CLAMP.n, n2);
            connect(RB.p, n4);
            connect(RB.n, n5);
            connect(R1.p, n4);
            connect(R1.n, n6);
            connect(Q3.C, n6);
            connect(Q3.B, n9);
            connect(Q3.E, n8);
            connect(R2.p, n8);
            connect(R2.n, n0);
            connect(RC.p, n4);
            connect(RC.n, n7);
            connect(Q4.C, n7);
            connect(Q4.B, n6);
            connect(Q4.E, n10);
            connect(DVBEDROP.p, n10);
            connect(DVBEDROP.n, n3);
            connect(Q5.C, n3);
            connect(Q5.B, n8);
            connect(Q5.E, n0);
          end NAND;

          model ONEBIT ".SUBCKT ONEBIT 1 2 3 4 5 6"
            parameter Semiconductors.ModelcardDIODE DMOD;
            parameter Semiconductors.ModelcardBJT QMOD(BF = 75, RB = 100, CJE = 1e-012, CJC = 3e-012);
            // *   NODES:  INPUT(2), CARRY-IN, OUTPUT, CARRY-OUT, VCC
            NAND X1;
            NAND X2;
            NAND X3;
            NAND X4;
            NAND X5;
            NAND X6;
            NAND X7;
            NAND X8;
            NAND X9;
            Modelica.Electrical.Analog.Interfaces.Pin p1;
            Modelica.Electrical.Analog.Interfaces.Pin p2;
            Modelica.Electrical.Analog.Interfaces.Pin p3;
            Modelica.Electrical.Analog.Interfaces.Pin p4;
            Modelica.Electrical.Analog.Interfaces.Pin p5;
            Modelica.Electrical.Analog.Interfaces.Pin p6;
          protected
            Modelica.Electrical.Analog.Interfaces.Pin n1;
            Modelica.Electrical.Analog.Interfaces.Pin n2;
            Modelica.Electrical.Analog.Interfaces.Pin n3;
            Modelica.Electrical.Analog.Interfaces.Pin n4;
            Modelica.Electrical.Analog.Interfaces.Pin n5;
            Modelica.Electrical.Analog.Interfaces.Pin n6;
            Modelica.Electrical.Analog.Interfaces.Pin n7;
            Modelica.Electrical.Analog.Interfaces.Pin n8;
            Modelica.Electrical.Analog.Interfaces.Pin n9;
            Modelica.Electrical.Analog.Interfaces.Pin n10;
            Modelica.Electrical.Analog.Interfaces.Pin n11;
            Modelica.Electrical.Analog.Interfaces.Pin n12;
            Modelica.Electrical.Analog.Interfaces.Pin n13;
          equation
            connect(p1, n1);
            connect(p2, n2);
            connect(p3, n3);
            connect(p4, n4);
            connect(p5, n5);
            connect(p6, n6);
            connect(X1.p1, n1);
            connect(X1.p2, n2);
            connect(X1.p3, n7);
            connect(X1.p4, n6);
            connect(X2.p1, n1);
            connect(X2.p2, n7);
            connect(X2.p3, n8);
            connect(X2.p4, n6);
            connect(X3.p1, n2);
            connect(X3.p2, n7);
            connect(X3.p3, n9);
            connect(X3.p4, n6);
            connect(X4.p1, n8);
            connect(X4.p2, n9);
            connect(X4.p3, n10);
            connect(X4.p4, n6);
            connect(X5.p1, n3);
            connect(X5.p2, n10);
            connect(X5.p3, n11);
            connect(X5.p4, n6);
            connect(X6.p1, n3);
            connect(X6.p2, n11);
            connect(X6.p3, n12);
            connect(X6.p4, n6);
            connect(X7.p1, n10);
            connect(X7.p2, n11);
            connect(X7.p3, n13);
            connect(X7.p4, n6);
            connect(X8.p1, n12);
            connect(X8.p2, n13);
            connect(X8.p3, n4);
            connect(X8.p4, n6);
            connect(X9.p1, n11);
            connect(X9.p2, n7);
            connect(X9.p3, n5);
            connect(X9.p4, n6);
          end ONEBIT;

          model TWOBIT ".SUBCKT TWOBIT 1 2 3 4 5 6 7 8 9"
            parameter Semiconductors.ModelcardDIODE DMOD;
            parameter Semiconductors.ModelcardBJT QMOD(BF = 75, RB = 100, CJE = 1e-012, CJC = 3e-012);
            // *   NODES:  INPUT - BIT0(2) / BIT1(2), OUTPUT - BIT0 / BIT1,
            // *           CARRY-IN, CARRY-OUT, VCC
            ONEBIT X1;
            ONEBIT X2;
            Modelica.Electrical.Analog.Interfaces.Pin p1;
            Modelica.Electrical.Analog.Interfaces.Pin p2;
            Modelica.Electrical.Analog.Interfaces.Pin p3;
            Modelica.Electrical.Analog.Interfaces.Pin p4;
            Modelica.Electrical.Analog.Interfaces.Pin p5;
            Modelica.Electrical.Analog.Interfaces.Pin p6;
            Modelica.Electrical.Analog.Interfaces.Pin p7;
            Modelica.Electrical.Analog.Interfaces.Pin p8;
            Modelica.Electrical.Analog.Interfaces.Pin p9;
          protected
            Modelica.Electrical.Analog.Interfaces.Pin n1;
            Modelica.Electrical.Analog.Interfaces.Pin n2;
            Modelica.Electrical.Analog.Interfaces.Pin n3;
            Modelica.Electrical.Analog.Interfaces.Pin n4;
            Modelica.Electrical.Analog.Interfaces.Pin n5;
            Modelica.Electrical.Analog.Interfaces.Pin n6;
            Modelica.Electrical.Analog.Interfaces.Pin n7;
            Modelica.Electrical.Analog.Interfaces.Pin n8;
            Modelica.Electrical.Analog.Interfaces.Pin n9;
            Modelica.Electrical.Analog.Interfaces.Pin n10;
          equation
            connect(p1, n1);
            connect(p2, n2);
            connect(p3, n3);
            connect(p4, n4);
            connect(p5, n5);
            connect(p6, n6);
            connect(p7, n7);
            connect(p8, n8);
            connect(p9, n9);
            connect(X1.p1, n1);
            connect(X1.p2, n2);
            connect(X1.p3, n7);
            connect(X1.p4, n5);
            connect(X1.p5, n10);
            connect(X1.p6, n9);
            connect(X2.p1, n3);
            connect(X2.p2, n4);
            connect(X2.p3, n10);
            connect(X2.p4, n6);
            connect(X2.p5, n8);
            connect(X2.p6, n9);
          end TWOBIT;

          model FOURBIT ".SUBCKT FOURBIT 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15"
            parameter Semiconductors.ModelcardDIODE DMOD;
            parameter Semiconductors.ModelcardBJT QMOD(BF = 75, RB = 100, CJE = 1e-012, CJC = 3e-012);
            // *   NODES:  INPUT - BIT0(2) / BIT1(2) / BIT2(2) / BIT3(2),
            // *           OUTPUT - BIT0 / BIT1 / BIT2 / BIT3, CARRY-IN, CARRY-OUT, VCC
            TWOBIT X1;
            TWOBIT X2;
            Modelica.Electrical.Analog.Interfaces.Pin p1;
            Modelica.Electrical.Analog.Interfaces.Pin p2;
            Modelica.Electrical.Analog.Interfaces.Pin p3;
            Modelica.Electrical.Analog.Interfaces.Pin p4;
            Modelica.Electrical.Analog.Interfaces.Pin p5;
            Modelica.Electrical.Analog.Interfaces.Pin p6;
            Modelica.Electrical.Analog.Interfaces.Pin p7;
            Modelica.Electrical.Analog.Interfaces.Pin p8;
            Modelica.Electrical.Analog.Interfaces.Pin p9;
            Modelica.Electrical.Analog.Interfaces.Pin p10;
            Modelica.Electrical.Analog.Interfaces.Pin p11;
            Modelica.Electrical.Analog.Interfaces.Pin p12;
            Modelica.Electrical.Analog.Interfaces.Pin p13;
            Modelica.Electrical.Analog.Interfaces.Pin p14;
            Modelica.Electrical.Analog.Interfaces.Pin p15;
          protected
            Modelica.Electrical.Analog.Interfaces.Pin n1;
            Modelica.Electrical.Analog.Interfaces.Pin n2;
            Modelica.Electrical.Analog.Interfaces.Pin n3;
            Modelica.Electrical.Analog.Interfaces.Pin n4;
            Modelica.Electrical.Analog.Interfaces.Pin n5;
            Modelica.Electrical.Analog.Interfaces.Pin n6;
            Modelica.Electrical.Analog.Interfaces.Pin n7;
            Modelica.Electrical.Analog.Interfaces.Pin n8;
            Modelica.Electrical.Analog.Interfaces.Pin n9;
            Modelica.Electrical.Analog.Interfaces.Pin n10;
            Modelica.Electrical.Analog.Interfaces.Pin n11;
            Modelica.Electrical.Analog.Interfaces.Pin n12;
            Modelica.Electrical.Analog.Interfaces.Pin n13;
            Modelica.Electrical.Analog.Interfaces.Pin n14;
            Modelica.Electrical.Analog.Interfaces.Pin n15;
            Modelica.Electrical.Analog.Interfaces.Pin n16;
          equation
            connect(p1, n1);
            connect(p2, n2);
            connect(p3, n3);
            connect(p4, n4);
            connect(p5, n5);
            connect(p6, n6);
            connect(p7, n7);
            connect(p8, n8);
            connect(p9, n9);
            connect(p10, n10);
            connect(p11, n11);
            connect(p12, n12);
            connect(p13, n13);
            connect(p14, n14);
            connect(p15, n15);
            connect(X1.p1, n1);
            connect(X1.p2, n2);
            connect(X1.p3, n3);
            connect(X1.p4, n4);
            connect(X1.p5, n9);
            connect(X1.p6, n10);
            connect(X1.p7, n13);
            connect(X1.p8, n16);
            connect(X1.p9, n15);
            connect(X2.p1, n5);
            connect(X2.p2, n6);
            connect(X2.p3, n7);
            connect(X2.p4, n8);
            connect(X2.p5, n11);
            connect(X2.p6, n12);
            connect(X2.p7, n16);
            connect(X2.p8, n14);
            connect(X2.p9, n15);
          end FOURBIT;

          // *** DEFINE NOMINAL CIRCUIT
          parameter Semiconductors.ModelcardDIODE DMOD;
          parameter Semiconductors.ModelcardBJT QMOD(BF = 75, RB = 100, CJE = 1e-012, CJC = 3e-012);
          Sources.V_constant VCC(V = 5);
          Sources.V_pulse VIN1A(V1 = 0, V2 = 3, TD = 0, TR = 1e-008, TF = 1e-008, PW = 1e-008, PER = 5e-008);
          Sources.V_pulse VIN1B(V1 = 0, V2 = 3, TD = 0, TR = 1e-008, TF = 1e-008, PW = 2e-008, PER = 1e-007);
          Sources.V_pulse VIN2A(V1 = 0, V2 = 3, TD = 0, TR = 1e-008, TF = 1e-008, PW = 4e-008, PER = 2e-007);
          Sources.V_pulse VIN2B(V1 = 0, V2 = 3, TD = 0, TR = 1e-008, TF = 1e-008, PW = 8e-008, PER = 4e-007);
          Sources.V_pulse VIN3A(V1 = 0, V2 = 3, TD = 0, TR = 1e-008, TF = 1e-008, PW = 1.6e-007, PER = 8e-007);
          Sources.V_pulse VIN3B(V1 = 0, V2 = 3, TD = 0, TR = 1e-008, TF = 1e-008, PW = 3.2e-007, PER = 1.6e-006);
          Sources.V_pulse VIN4A(V1 = 0, V2 = 3, TD = 0, TR = 1e-008, TF = 1e-008, PW = 6.4e-007, PER = 3.2e-006);
          Sources.V_pulse VIN4B(V1 = 0, V2 = 3, TD = 0, TR = 1e-008, TF = 1e-008, PW = 1.28e-006, PER = 6.4e-006);
          FOURBIT X1(X1(X1(X1(Q1(vbe(start = 0, fixed = true)), Q2(vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q3(Binternal(start = 0), icapbe(start = 0), vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q4(Binternal(start = 0), icapbc(start = 0), vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q5(vbc(start = 0, fixed = true), vbe(start = 0))), X2(Q1(vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q2(vbe(start = 0, fixed = true)), Q3(Binternal(start = 0), icapbe(start = 0), vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q4(Binternal(start = 0), icapbe(start = 0), vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q5(vbc(start = 0, fixed = true), vbe(start = 0, fixed = true))), X3(Q1(vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q2(vbe(start = 0, fixed = true)), Q3(Binternal(start = 0), icapbe(start = 0), vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q4(Binternal(start = 0), icapbe(start = 0), vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q5(vbe(start = 0, fixed = true))), X4(Q1(vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q2(vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q3(Binternal(start = 0), icapbe(start = 0), vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q4(Binternal(start = 0), icapbe(start = 0), vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q5(vbe(start = 0, fixed = true))), X5(Q1(vbe(start = 0, fixed = true)), Q2(vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q3(Binternal(start = 0), icapbe(start = 0), vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q4(Binternal(start = 0), icapbc(start = 0), vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q5(vbe(start = 0, fixed = true))), X6(Q1(vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q2(vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q3(Binternal(start = 0), icapbe(start = 0), vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q4(Binternal(start = 0), icapbe(start = 0), vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q5(vbc(start = 0, fixed = true), vbe(start = 0))), X7(Q1(vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q2(vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q3(Binternal(start = 0), icapbe(start = 0), vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q4(icapbe(start = 0), vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q5(vbe(start = 0, fixed = true), vbc(start = 0, fixed = true)), RC(v(start = 0))), X8(Q1(vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q2(vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q3(Binternal(start = 0), icapbe(start = 0), vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q4(Binternal(start = 0), icapbe(start = 0), vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q5(vbc(start = 0, fixed = true), vbe(start = 0, fixed = true))), X9(Q1(vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q2(vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q3(Binternal(start = 0), icapbe(start = 0), vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q4(icapbe(start = 0), vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q5(vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), RC(v(start = 0)))), X2(X1(Q1(vbc(start = 0, fixed = true), vbe(start = 0, fixed = false)), Q2(vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q3(Binternal(start = 0), icapbe(start = 0), vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q4(Binternal(start = 0), icapbc(start = 0), vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q5(vbc(start = 0, fixed = true))), X2(Q1(vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q2(vbe(start = 0, fixed = true)), Q3(Binternal(start = 0), icapbe(start = 0), vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q4(icapbe(start = 0), vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q5(vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), RC(v(start = 0))), X3(Q1(vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q2(vbe(start = 0, fixed = true)), Q3(Binternal(start = 0), icapbe(start = 0), vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q4(icapbe(start = 0), vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q5(vbe(start = 0, fixed = true)), RC(v(start = 0))), X4(Q1(vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q2(vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q3(Binternal(start = 0), icapbe(start = 0), vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q4(Binternal(start = 0), icapbe(start = 0), vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q5(vbc(start = 0, fixed = true), vbe(start = 0, fixed = true))), X5(Q1(vbe(start = 0, fixed = true)), Q2(vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q3(Binternal(start = 0), icapbe(start = 0), vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q4(Binternal(start = 0), icapbe(start = 0), vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q5(vbe(start = 0, fixed = true))), X6(Q1(vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q2(vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q3(Binternal(start = 0), icapbe(start = 0), vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q4(Binternal(start = 0), icapbe(start = 0), vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q5(vbe(start = 0, fixed = true))), X7(Q1(vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q2(vbc(start = 0, fixed = true)), Q3(Binternal(start = 0), icapbe(start = 0), vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q4(icapbe(start = 0), vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q5(vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), RC(v(start = 0))), X8(Q1(vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q2(vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q3(Binternal(start = 0), icapbe(start = 0), vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q4(Binternal(start = 0), icapbe(start = 0), vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q5(vbc(start = 0, fixed = true), vbe(start = 0, fixed = true))), X9(Q1(vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q2(vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q3(Binternal(start = 0), icapbe(start = 0), vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q4(icapbe(start = 0), vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q5(vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), RC(v(start = 0))))), X2(X1(X1(Q1(vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q2(vbe(start = 0, fixed = true)), Q3(Binternal(start = 0), icapbe(start = 0), vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q4(Binternal(start = 0), icapbc(start = 0), vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q5(vbc(start = 0, fixed = true))), X2(Q1(vbe(start = 0, fixed = true)), Q2(vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q3(Binternal(start = 0), icapbe(start = 0), vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q4(icapbe(start = 0), vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q5(vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), RC(v(start = 0))), X3(Q1(vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q2(vbe(start = 0, fixed = true)), Q3(Binternal(start = 0), icapbe(start = 0), vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q4(icapbe(start = 0), vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q5(vbe(start = 0, fixed = true)), RC(v(start = 0))), X4(Q1(vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q2(vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q3(Binternal(start = 0), icapbe(start = 0), vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q4(Binternal(start = 0), icapbe(start = 0), vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q5(vbc(start = 0, fixed = true), vbe(start = 0, fixed = true))), X5(Q1(vbe(start = 0, fixed = true)), Q2(vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q3(Binternal(start = 0), icapbe(start = 0), vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q4(Binternal(start = 0), icapbe(start = 0), vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q5(vbe(start = 0, fixed = true))), X6(Q1(vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q2(vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q3(Binternal(start = 0), icapbe(start = 0), vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q4(Binternal(start = 0), icapbe(start = 0), vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q5(vbe(start = 0, fixed = true))), X7(Q1(vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q2(vbc(start = 0, fixed = true)), Q3(Binternal(start = 0), icapbe(start = 0), vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q4(icapbe(start = 0), vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q5(vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), RC(v(start = 0))), X8(Q1(vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q2(vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q3(Binternal(start = 0), icapbe(start = 0), vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q4(Binternal(start = 0), icapbe(start = 0), vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q5(vbc(start = 0, fixed = true), vbe(start = 0, fixed = true))), X9(Q1(vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q2(vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q3(Binternal(start = 0), icapbe(start = 0), vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q4(icapbe(start = 0), vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q5(vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), RC(v(start = 0)))), X2(X1(Q1(vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q2(vbe(start = 0, fixed = true)), Q3(Binternal(start = 0), icapbe(start = 0), vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q4(icapbc(start = 0), vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q5(vbc(start = 0, fixed = true)), RC(v(start = 0))), X2(Q1(vbe(start = 0, fixed = true)), Q2(vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q3(Binternal(start = 0), icapbe(start = 0), vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q4(icapbe(start = 0), vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q5(vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), RC(v(start = 0))), X3(Q1(vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q2(vbe(start = 0, fixed = true)), Q3(Binternal(start = 0), icapbe(start = 0), vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q4(icapbe(start = 0), vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q5(vbe(start = 0, fixed = true)), RC(v(start = 0))), X4(Q1(vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q2(vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q3(Binternal(start = 0), icapbe(start = 0), vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q4(Binternal(start = 0), icapbe(start = 0), vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q5(vbc(start = 0, fixed = true), vbe(start = 0, fixed = true))), X5(Q1(vbe(start = 0, fixed = true)), Q2(vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q3(Binternal(start = 0), icapbe(start = 0), vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q4(Binternal(start = 0), icapbe(start = 0), vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q5(vbe(start = 0, fixed = true))), X6(Q1(vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q2(vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q3(Binternal(start = 0), icapbe(start = 0), vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q4(Binternal(start = 0), icapbe(start = 0), vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q5(vbe(start = 0, fixed = true))), X7(Q1(vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q2(vbc(start = 0, fixed = true)), Q3(Binternal(start = 0), icapbe(start = 0), vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q4(icapbe(start = 0), vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q5(vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), RC(v(start = 0))), X8(Q1(vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q2(vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q3(Binternal(start = 0), icapbe(start = 0), vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q4(Binternal(start = 0), icapbe(start = 0), vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q5(vbc(start = 0, fixed = true), vbe(start = 0, fixed = true))), X9(Q1(vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q2(vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q3(Binternal(start = 0), icapbe(start = 0), vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q4(Binternal(start = 0), icapbe(start = 0), vbc(start = 0, fixed = true), vbe(start = 0, fixed = true)), Q5(vbc(start = 0, fixed = true), vbe(start = 0, fixed = true))))));
          Basic.R_Resistor RBIT0(R = 1000);
          Basic.R_Resistor RBIT1(R = 1000);
          Basic.R_Resistor RBIT2(R = 1000);
          Basic.R_Resistor RBIT3(R = 1000);
          Basic.R_Resistor RCOUT(R = 1000);
          // *** (FOR THOSE WITH MONEY (AND MEMORY) TO BURN)
          Basic.Ground g;
        protected
          Modelica.Electrical.Analog.Interfaces.Pin n99;
          Modelica.Electrical.Analog.Interfaces.Pin n0;
          Modelica.Electrical.Analog.Interfaces.Pin n1;
          Modelica.Electrical.Analog.Interfaces.Pin n2;
          Modelica.Electrical.Analog.Interfaces.Pin n3;
          Modelica.Electrical.Analog.Interfaces.Pin n4;
          Modelica.Electrical.Analog.Interfaces.Pin n5;
          Modelica.Electrical.Analog.Interfaces.Pin n6;
          Modelica.Electrical.Analog.Interfaces.Pin n7;
          Modelica.Electrical.Analog.Interfaces.Pin n8;
          Modelica.Electrical.Analog.Interfaces.Pin n9;
          Modelica.Electrical.Analog.Interfaces.Pin n10;
          Modelica.Electrical.Analog.Interfaces.Pin n11;
          Modelica.Electrical.Analog.Interfaces.Pin n12;
          Modelica.Electrical.Analog.Interfaces.Pin n13;
        equation
          connect(g.p, n0);
          connect(VCC.p, n99);
          connect(VCC.n, n0);
          connect(VIN1A.p, n1);
          connect(VIN1A.n, n0);
          connect(VIN1B.p, n2);
          connect(VIN1B.n, n0);
          connect(VIN2A.p, n3);
          connect(VIN2A.n, n0);
          connect(VIN2B.p, n4);
          connect(VIN2B.n, n0);
          connect(VIN3A.p, n5);
          connect(VIN3A.n, n0);
          connect(VIN3B.p, n6);
          connect(VIN3B.n, n0);
          connect(VIN4A.p, n7);
          connect(VIN4A.n, n0);
          connect(VIN4B.p, n8);
          connect(VIN4B.n, n0);
          connect(X1.p1, n1);
          connect(X1.p2, n2);
          connect(X1.p3, n3);
          connect(X1.p4, n4);
          connect(X1.p5, n5);
          connect(X1.p6, n6);
          connect(X1.p7, n7);
          connect(X1.p8, n8);
          connect(X1.p9, n9);
          connect(X1.p10, n10);
          connect(X1.p11, n11);
          connect(X1.p12, n12);
          connect(X1.p13, n0);
          connect(X1.p14, n13);
          connect(X1.p15, n99);
          connect(RBIT0.p, n9);
          connect(RBIT0.n, n0);
          connect(RBIT1.p, n10);
          connect(RBIT1.n, n0);
          connect(RBIT2.p, n11);
          connect(RBIT2.n, n0);
          connect(RBIT3.p, n12);
          connect(RBIT3.n, n0);
          connect(RCOUT.p, n13);
          connect(RCOUT.n, n0);
          annotation(
            experiment(StopTime = 1e-006, Interval = 1e-009),
            Documentation(info = "<html>
<p>This Four Bit Binary Adder model is one of the five benchmark circuits described in the SPICE3 Version e3 User&apos;s Manual (see information of package Spice3). </p>
<p>The model adds two 4-bit numbers (Number A and Number B). It has eight inputs where the first one is the lowest-order bit of the first number (Number A Bit0), the second is the lowest-order bit of the second number (Number B Bit0), the third one is the second-order bit of the first number (Number A Bit1) and so on. The Four Bit Binary Adder has four outputs where the first one (Sum Bit0) is the lowest-order bit, the second and the third one (Sum Bit1 and Sum Bit2) are the next two bits and the last one (Sum Bit3) is the highest-order bit. The picture illustrates the pin-assignment.</p>
<blockquote>
<img src=\"modelica://Modelica/Resources/Images/Electrical/Spice3/FourBitBinaryAdder.jpg\"
     alt=\"segment.png\">
</blockquote>

<p>Inside the model the names of the inputs refer to the names in the picture as follows:</p>
<p>X1_p1_v --> Number A Bit0</p>
<p>X1_p2_v --> Number B Bit0</p>
<p>X1_p3_v --> Number A Bit1</p>
<p>X1_p4_v --> Number B Bit1</p>
<p>X1_p5_v --> Number A Bit2</p>
<p>X1_p6_v --> Number B Bit2</p>
<p>X1_p7_v --> Number A Bit3</p>
<p>X1_p8_v --> Number B Bit3</p>
<p>X1_p9_v --> Sum Bit0</p>
<p>X1_p10_v --> Sum Bit1</p>
<p>X1_p11_v --> Sum Bit2</p>
<p>X1_p12_v --> Sum Bit3</p>
<p>X1_p14_v --> Cout</p>
<p>The Four Bit Binary Adder is built out of two two bit adders which respectively are built out of two one bit adders. One one bit adder is build out of nine NAND circuits. </p>
<p>Please note, that the simulation time of the Four Bit Binary Adder can take several hours due to its immense size (e.g. 11387 equations).</p>
<p>The user is recommended to simulate from t=0 to t=1e-6s and observe the eight inputs (X1_p1_v, ..., X1_p8_v) and the four outputs (X1_p9_v, ..., X1_p12_v) and the carryout output (X1_p14_v).</p>
<p><b>The timing of the single transistors of the adder causes a delay which makes it hard to recognize the adder behaviour. Since the Four Bit Binary Adder is a SPICE3 benchmark, the circuit is not changed in order to see the adder behaviour in a better way.</b></p>
<p>Original SPICE3 netlist of the Four Bit Binary Adder:</p>
<pre>
ADDER - 4 BIT ALL-NAND-GATE BINARY ADDER

*** SUBCIRCUIT DEFINITIONS
.SUBCKT NAND 1 2 3 4
*   NODES:  INPUT(2), OUTPUT, VCC
Q1        9  5  1 QMOD
D1CLAMP   0  1    DMOD
Q2        9  5  2 QMOD
D2CLAMP   0  2    DMOD
RB        4  5    4K
R1        4  6    1.6K
Q3        6  9  8 QMOD
R2        8  0    1K
RC        4  7    130
Q4        7  6 10 QMOD
DVBEDROP 10  3    DMOD
Q5        3  8  0 QMOD
.ENDS NAND

.SUBCKT ONEBIT 1 2 3 4 5 6
*   NODES:  INPUT(2), CARRY-IN, OUTPUT, CARRY-OUT, VCC
X1   1  2  7  6   NAND
X2   1  7  8  6   NAND
X3   2  7  9  6   NAND
X4   8  9 10  6   NAND
X5   3 10 11  6   NAND
X6   3 11 12  6   NAND
X7  10 11 13  6   NAND
X8  12 13  4  6   NAND
X9  11  7  5  6   NAND
.ENDS ONEBIT

.SUBCKT TWOBIT 1 2 3 4 5 6 7 8 9
*   NODES:  INPUT - BIT0(2) / BIT1(2), OUTPUT - BIT0 / BIT1,
*           CARRY-IN, CARRY-OUT, VCC
X1   1  2  7  5 10  9   ONEBIT
X2   3  4 10  6  8  9   ONEBIT
.ENDS TWOBIT

.SUBCKT FOURBIT 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15
*   NODES:  INPUT - BIT0(2) / BIT1(2) / BIT2(2) / BIT3(2),
*           OUTPUT - BIT0 / BIT1 / BIT2 / BIT3, CARRY-IN, CARRY-OUT, VCC
X1   1  2  3  4  9 10 13 16 15   TWOBIT
X2   5  6  7  8 11 12 16 14 15   TWOBIT
.ENDS FOURBIT

*** DEFINE NOMINAL CIRCUIT
.MODEL DMOD D
.MODEL QMOD NPN(BF=75 RB=100 CJE=1PF CJC=3PF)
VCC   99  0   DC 5V
VIN1A  1  0   PULSE(0 3 0 10NS 10NS   10NS   50NS)
VIN1B  2  0   PULSE(0 3 0 10NS 10NS   20NS  100NS)
VIN2A  3  0   PULSE(0 3 0 10NS 10NS   40NS  200NS)
VIN2B  4  0   PULSE(0 3 0 10NS 10NS   80NS  400NS)
VIN3A  5  0   PULSE(0 3 0 10NS 10NS  160NS  800NS)
VIN3B  6  0   PULSE(0 3 0 10NS 10NS  320NS 1600NS)
VIN4A  7  0   PULSE(0 3 0 10NS 10NS  640NS 3200NS)
VIN4B  8  0   PULSE(0 3 0 10NS 10NS 1280NS 6400NS)
X1     1  2  3  4  5  6  7  8  9 10 11 12  0 13 99 FOURBIT
RBIT0  9  0   1K
RBIT1 10  0   1K
RBIT2 11  0   1K
RBIT3 12  0   1K
RCOUT 13  0   1K

*** (FOR THOSE WITH MONEY (AND MEMORY) TO BURN)
.TRAN 1NS 6400NS UIC

.control
run
set options no break

*plot v(1) v(2)
*plot v(3) v(4)
*plot v(5) v(6)
*plot v(7) v(8)
*plot v(9) v(10)
*plot v(11) v(12)
*plot v(13)
*print v(9) v(10)
print v(11) v(12) v(13)

.endc

.END
</pre>
<p>The model is built out of several subcircuits which were described only ones and used several times. </p>
</html>"));
        end Spice3BenchmarkFourBitBinaryAdder;
        annotation(
          Documentation(info = "<html>
<p>This package Example circuits contains some useful examples to demonstrate how the library is working and how the models can be used.</p>
</html>"));
      end Examples;

      package Basic "Basic electrical components"
        extends Modelica.Icons.Package;

        model Ground "Ground node"
          Modelica.Electrical.Analog.Interfaces.Pin p "Ground pin" annotation(
            Placement(transformation(origin = {0, 100}, extent = {{10, -10}, {-10, 10}}, rotation = 270)));
        equation
          p.v = 0;
          annotation(
            Documentation(info = "<HTML>
<P>
Ground of an electrical circuit. The potential at the
ground node is zero. Every electrical circuit has to contain
at least one ground object.
</P>
<P>
SPICE does not have an element for the ground node (mass). In SPICE
netlists the ground is specified by the node number 0.
This Modelica SPICE library demands to describe the ground node
by this ground element.
</P>
</HTML>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{0, 100}, {0, 40}}, color = {0, 0, 255}, smooth = Smooth.None), Line(points = {{-60, 40}, {60, 40}}, color = {0, 0, 255}, smooth = Smooth.None), Line(points = {{-40, 20}, {40, 20}}, color = {0, 0, 255}, smooth = Smooth.None), Line(points = {{-20, 0}, {20, 0}}, color = {0, 0, 255}, smooth = Smooth.None), Text(extent = {{-50, -14}, {58, -54}}, lineColor = {0, 0, 255}, textString = "%name")}));
        end Ground;

        model R_Resistor "Ideal linear electrical resistor"
          extends Modelica.Electrical.Analog.Interfaces.OnePort;
          parameter SI.Resistance R(start = 1000) "Resistance";
        equation
          R * i = v;
          annotation(
            Documentation(info = "<HTML>
<P>
The linear resistor connects the branch voltage <i>v</i> with the
branch current <i>i</i> by <i>i*R = v</i>.
The Resistance <i>R</i> is allowed to be positive, zero, or negative.
</P>
</HTML>"),
            Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-70, 30}, {70, -30}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{-90, 0}, {-70, 0}}, color = {0, 0, 255}), Line(points = {{70, 0}, {90, 0}}, color = {0, 0, 255}), Text(extent = {{-136, -46}, {140, -62}}, lineColor = {0, 0, 0}, textString = "R=%R"), Text(extent = {{-136, 34}, {138, 54}}, textString = "%name")}));
        end R_Resistor;

        model C_Capacitor "Ideal linear electrical capacitor"
          extends Modelica.Electrical.Analog.Interfaces.OnePort;
          parameter SI.Capacitance C(start = 0) "Capacitance";
          parameter SI.Voltage IC = 0 "Initial value";
          parameter Boolean UIC = false "Use initial conditions: true, if initial condition is used";
        protected
          SI.Voltage vinternal;
          //(start=IC, fixed=UIC);
        initial equation
          vinternal = IC;
        equation
          vinternal = p.v - n.v;
          i = C * der(vinternal);
          annotation(
            Documentation(info = "<HTML>
<p>
The linear capacitor connects the branch voltage <i>v</i> with the
branch current <i>i</i> by <i>i = C * dv/dt</i>.
The Capacitance <i>C</i> is allowed to be positive, zero, or negative.
</p>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-6, 28}, {-6, -28}}, color = {0, 0, 255}), Line(points = {{6, 28}, {6, -28}}, color = {0, 0, 255}), Line(points = {{-90, 0}, {-6, 0}}), Line(points = {{6, 0}, {90, 0}}), Text(extent = {{-130, -40}, {134, -60}}, lineColor = {0, 0, 0}, textString = "C=%C"), Text(extent = {{-138, 42}, {136, 62}}, textString = "%name")}));
        end C_Capacitor;

        model L_Inductor "Ideal linear electrical inductor"
          extends Modelica.Electrical.Analog.Interfaces.OnePort;
          parameter SI.Inductance L(start = 0) "Inductance";
          parameter SI.Current IC = 0 "Initial value; used, if UIC is true";
          parameter Boolean UIC = false "Use initial conditions";
          SI.Current iinternal;
          Modelica.Electrical.Spice3.Interfaces.InductiveCouplePinOut ICP "Pin to couple inductances via K" annotation(
            Placement(transformation(extent = {{-20, -20}, {20, 20}}, rotation = -90, origin = {0, 80}), iconTransformation(extent = {{-16, -16}, {16, 16}}, rotation = 270, origin = {0, 84})));
        initial equation
          if UIC then
            iinternal = IC;
          else
            der(iinternal) = 0;
          end if;
        equation
          iinternal = p.i;
          L * der(iinternal) = v + ICP.v;
          ICP.L = L;
          ICP.di = der(iinternal);
          annotation(
            Documentation(info = "<html>
<p>The linear inductor connects the branch voltage <i>v</i> with the branch current <i>i</i> by <i>v = L * di/dt</i>. The inductance <i>L</i> is allowed to be positive, zero, or negative.</p>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-60, -15}, {-30, 15}}), Ellipse(extent = {{-30, -15}, {0, 15}}), Ellipse(extent = {{0, -15}, {30, 15}}), Ellipse(extent = {{30, -15}, {60, 15}}), Rectangle(extent = {{-60, -30}, {60, 0}}, lineColor = {255, 255, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{60, 0}, {90, 0}}), Line(points = {{-90, 0}, {-60, 0}}), Text(extent = {{-142, -40}, {142, -60}}, lineColor = {0, 0, 0}, textString = "L=%L"), Text(extent = {{-136, 36}, {138, 56}}, textString = "%name")}));
        end L_Inductor;

        model K_CoupledInductors "Inductive coupling via coupling factor K"
          parameter Real k(start = 0) "Coupling Factor";
          Modelica.Electrical.Spice3.Interfaces.InductiveCouplePinIn inductiveCouplePin1 "Couple pin for inductances" annotation(
            Placement(transformation(extent = {{-110, -10}, {-90, 10}})));
          Modelica.Electrical.Spice3.Interfaces.InductiveCouplePinIn inductiveCouplePin2 "Couple pin for inductances" annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 180, origin = {100, 0}), iconTransformation(extent = {{-10, -10}, {10, 10}}, rotation = 180, origin = {100, 0})));
          Modelica.SIunits.Inductance M "mutual inductance";
        equation
          assert(k >= 0, "Coupling factor must be not negative");
          assert(k < 1, "coupling factor must be less than one");
          M = k * sqrt(inductiveCouplePin1.L * inductiveCouplePin2.L);
          inductiveCouplePin1.v = -M * inductiveCouplePin2.di;
          inductiveCouplePin2.v = -M * inductiveCouplePin1.di;
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{-60, 0}, {0, 20}, {60, 0}, {0, -20}, {-60, 0}}, lineColor = {170, 85, 255}, smooth = Smooth.None), Line(points = {{-60, 0}, {-96, 0}, {-98, 0}}, color = {170, 85, 255}, smooth = Smooth.None), Line(points = {{60, 0}, {100, 0}}, color = {170, 85, 255}, smooth = Smooth.None), Text(extent = {{-74, 76}, {74, 30}}, lineColor = {170, 85, 255}, fillPattern = FillPattern.Solid, textString = "%name"), Text(extent = {{-72, -24}, {76, -70}}, lineColor = {170, 85, 255}, fillPattern = FillPattern.Solid, textString = "k=%k")}),
            Documentation(info = "<html>
<p>
<code>K_CoupledInductors</code> is a component that allows the coupling of two inductors.
<code>K</code> is the coefficient of coupling which must be greater than or equal to zero and less than one.
</p>
<p>
The usage is demonstrated in the example <a href=\"modelica://Modelica.Electrical.Spice3.Examples.CoupledInductors\">CoupledInductors</a>.
</p>
</html>"));
        end K_CoupledInductors;

        model E_VCV "Linear voltage-controlled voltage source"
          extends Interfaces.TwoPortControlledSources;
          parameter Real gain(start = 0) "Voltage gain";
        equation
          v2 = v1 * gain;
          i1 = 0;
          annotation(
            Documentation(info = "<HTML>
<p>
The linear voltage-controlled voltage source is a TwoPort.
The right port voltage at pin p2 (=p2.v) is controlled by the left port voltage at pin p1 (=p1.v)
via
</p>
<pre>
    p2.v = p1.v * gain.
</pre>
<p>
The left port current is zero. Any voltage gain can be chosen.
</p>
The corresponding SPICE description
<pre>
    Ename N+ N- NC+ NC- VALUE
</pre>
<p>is translated to Modelica:</p>
<pre>
    Ename -> Spice3.Basic.E_VCV Ename
    (Ename is the name of the Modelica instance)
    N+ -> p2.v
    N- -> n2.v
    NC+ -> p1.v
    NC- -> n1.v
    VALUE -> gain
</pre>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-70, 70}, {70, -70}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-99, -79}, {100, -129}}, textString = "%name"), Line(points = {{-90, 50}, {-30, 50}}), Line(points = {{-30, -50}, {-90, -50}}), Line(points = {{100, 50}, {30, 50}, {30, -50}, {100, -50}}), Ellipse(extent = {{10, 20}, {50, -20}}), Line(points = {{-20, 60}, {20, 60}}), Polygon(points = {{20, 60}, {10, 63}, {10, 57}, {20, 60}}, lineColor = {0, 0, 255}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid)}));
        end E_VCV;

        model G_VCC "Linear voltage-controlled current source"
          extends Interfaces.TwoPortControlledSources;
          parameter SI.Conductance transConductance(start = 0) "Transconductance";
        equation
          i2 = v1 * transConductance;
          i1 = 0;
          annotation(
            Documentation(info = "<HTML>
<p>
The linear voltage-controlled current source is a TwoPort.
The right port current at pin p2 (=p2.i) is controlled by the left port voltage at pin p1 (p1.v)
via
</p>
<pre>
    p2.i = p1.v * transConductance.
</pre>
<p>
The left port current is zero. Any transConductance can be chosen.
</p>
The corresponding SPICE description
<pre>
    Gname N+ N- NC+ NC- VALUE
</pre>
<p>is translated to Modelica:</p>
<pre>

    Gname -> Spice3.Basic.G_VCC Gname
    (Gname is the name of the Modelica instance)
    N+ -> p2.i
    N- -> n2.i
    NC+ -> p1 .v
    NC- -> n1.v
    VALUE -> transConductance
</pre>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-71, 70}, {69, -70}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-100, -80}, {99, -129}}, textString = "%name"), Line(points = {{-91, 50}, {-31, 50}}), Line(points = {{-31, -50}, {-91, -50}}), Ellipse(extent = {{9, 20}, {49, -20}}), Line(points = {{-21, 60}, {19, 60}}), Polygon(points = {{19, 60}, {9, 63}, {9, 57}, {19, 60}}, lineColor = {0, 0, 255}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid), Line(points = {{89, 50}, {29, 50}, {29, 20}}), Line(points = {{90, -50}, {29, -50}, {29, -20}}), Line(points = {{9, 0}, {49, 0}})}));
        end G_VCC;

        model H_CCV "Linear current-controlled voltage source"
          extends Interfaces.TwoPortControlledSources;
          parameter SI.Resistance transResistance(start = 0) "Transresistance";
        equation
          v2 = i1 * transResistance;
          v1 = 0;
          annotation(
            Documentation(info = "<html>
<p>The linear current-controlled voltage source is a TwoPort. The &quot;right&quot; port voltage at pin 2 (=p2.v) is controlled by the &quot;left&quot; port current at pin p1(=p1.i) via</p>
<pre>    p2.v = p1.i * transResistance.</pre>
<p>The controlling port voltage is zero. Any transResistance can be chosen.</p>
<p>The corresponding SPICE description</p>
<pre>    Hname N+ N- VNAM VALUE</pre>
<p>is translated to Modelica:</p>
<pre>    Hname -&gt; Spice3.Basic.H_CCV Hname
    (Hname is the name of the Modelica instance)
    N+ -&gt; p2.v
    N- -&gt; n2.v  </pre>
<p>The voltage source VNAM has the two nodes NV+ and NV-:</p>
<pre>                   VNAM VN+ VN- VALUE_V</pre>
<p>The current through VNAM hast to be led through the CCV.</p><p>Therefore VNAM has to be disconnected and an additional</p><p>node NV_AD has to be added.</p>
<pre>    NV_AD -&gt; p1.i
    NV- -&gt; n1.i</pre>
<p>On this way the current, that flows through the voltage source VNAM, flows through the CCV.</p>
<pre>
    VALUE -&gt; transResistance </pre>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-70, 70}, {70, -70}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-99, -80}, {100, -130}}, textString = "%name"), Line(points = {{100, 50}, {30, 50}, {30, -50}, {100, -50}}), Ellipse(extent = {{10, 20}, {50, -20}}), Line(points = {{-20, 60}, {20, 60}}), Polygon(points = {{20, 60}, {10, 63}, {10, 57}, {20, 60}}, lineColor = {0, 0, 255}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid), Line(points = {{-90, 50}, {-20, 50}, {-20, -50}, {-90, -50}})}));
        end H_CCV;

        model F_CCC "Linear current-controlled current source"
          extends Interfaces.TwoPortControlledSources;
          parameter Real gain(start = 0) "Current gain";
        equation
          i2 = i1 * gain;
          v1 = 0;
          annotation(
            Documentation(info = "<html>
<p>The linear current-controlled current source is a TwoPort. The &quot;right&quot; port current at pin 2 (=p2.i) is controlled by the &quot;left&quot; port current at pin p1(=p1.i) via</p>
<pre>    p2.i = p1.i * gain.</pre>
<p>The controlling port voltage is zero. Any current gain can be chosen.</p>
<p>The corresponding SPICE description</p>
<pre>    Fname N+ N- VNAM VALUE</pre>
<p>is translated to Modelica:</p>
<pre>    Fname -&gt; Spice3.Basic.F_CCC Fname
    (Fname is the name of the Modelica instance)
    N+ -&gt; p2.i
    N- -&gt; n2.i  </pre>
<p>The voltage source VNAM has the two nodes NV+ and NV-:</p>
<pre>                   VNAM NV+ NV- VALUE_V</pre>
<p>The current through VNAM hast to be led through the CCC.</p>
<p>Therefore VNAM has to be disconnected and an additional</p>
<p>node NV_AD has to be added.</p>
<pre>    NV_AD -&gt; p1.i
    NV- -&gt; n1.i</pre>
<p>On this way the current, that flows through the voltage source VNAM, flows through the CCC.</p>
<pre>    VALUE -&gt; gain </pre>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-70, 70}, {70, -70}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-104, -76}, {97, -127}}, textString = "%name"), Line(points = {{-100, 50}, {-30, 50}, {-30, -50}, {-100, -50}}), Ellipse(extent = {{10, 20}, {50, -20}}), Line(points = {{-20, 60}, {20, 60}}), Polygon(points = {{20, 60}, {10, 63}, {10, 57}, {20, 60}}, lineColor = {0, 0, 255}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid), Line(points = {{90, 50}, {30, 50}, {30, 20}}), Line(points = {{91, -50}, {30, -50}, {30, -20}}), Line(points = {{10, 0}, {50, 0}})}));
        end F_CCC;
        annotation(
          preferredView = "info",
          Documentation(info = "<html>
<p>This Package contains the basic components of the SPICE3 models. The first letter of the</p><p>name of the component shows the SPICE name, e.g., <b>R</b>_Resistor: <b>R</b> is the SPICE-name of the component</p><p>resistor which is used in SPICE-Netlists.</p>
</html>", revisions = "<html>
<dl>
<dt>
<b>Main Authors:</b>
<dd>
<a href=\"http://people.eas.iis.fhg.de/Christoph.Clauss/\">Christoph Clau&szlig;</a>
    &lt;<a href=\"mailto:clauss@eas.iis.fhg.de\">clauss@eas.iis.fhg.de</a>&gt;<br>

    Fraunhofer Institute for Integrated Circuits<br>
    Design Automation Department<br>
    Zeunerstra&szlig;e 38<br>
    D-01069 Dresden
</dd>
</dl>
</html>"));
      end Basic;

      package Semiconductors "Semiconductor devices and model cards"
        extends Modelica.Icons.Package;

        model M_PMOS "PMOS MOSFET device"
          extends Modelica.Electrical.Spice3.Internal.MOS(final mtype = 1);
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{60, 0}, {40, 5}, {40, -5}, {60, 0}}, lineColor = {0, 0, 255}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid)}),
            Documentation(info = "<html>
<p>The model M_PMOS is a P channel MOSFET transistor with fixed level 1: Shichman-Hodges model</p>
<p>The models from the package Semiconductors accesses to the package Repository where all functions,</p>
<p>records and data are stored and modeled that are needed for the semiconductor models.</p>
<p>The package Semiconductors is for user access but not the package Repository.</p>
</html>", revisions = "<html>
<ul>
<li><i>March 2008 </i>by Kristin Majetta <br/>initially implemented</li>
</ul>
</html>"));
        end M_PMOS;

        model M_NMOS "NMOS MOSFET device"
          extends Modelica.Electrical.Spice3.Internal.MOS(final mtype = 0);
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{40, 0}, {60, 5}, {60, -5}, {40, 0}}, lineColor = {0, 0, 255}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid)}),
            Documentation(info = "<html>
<p>The model M_NMOS is a N channel MOSFET transistor with fixed level 1: Shichman-Hodges model</p>
<p>The models from the package Semiconductors accesses to the package Repository where all functions,</p>
<p>records and data are stored and modeled that are needed for the semiconductor models.</p>
<p>The package Semiconductors is for user access but not the package Repository.</p>
</html>", revisions = "<html>
<ul>
<li><i>March 2008 </i>by Kristin Majetta <br/>initially implemented</li>
</ul>
</html>"));
        end M_NMOS;

        record ModelcardMOS "Record for the specification of modelcard parameters"
          extends Modelica.Icons.Record;
          extends Modelica.Electrical.Spice3.Internal.ModelcardMOS;
          annotation(
            Documentation(info = "<html>
<p>Technology model parameters of MOSFET transistor with fixed level 1: Shichman-Hodges model</p>
<p>In modelcards, that are typical for SPICE3, the so called technology parameters are stored. These parameters are usually set for more than one semiconductor device in a circuit, e.g., the temperature of a whole electrical circuit.</p>
</html>"));
        end ModelcardMOS;

        model M_NMOS2 "NMOS MOSFET device"
          extends Modelica.Electrical.Spice3.Internal.MOS2(final mtype = 0);
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{40, 0}, {60, 5}, {60, -5}, {40, 0}}, lineColor = {0, 0, 255}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid)}),
            Documentation(info = "<html>
<p>The model M_NMOS is a N channel MOSFET transistor with fixed level 2: </p>
<p>The models from the package Semiconductors accesses to the package Internal where all functions,</p>
<p>records and data are stored and modeled that are needed for the semiconductor models.</p>
<p>The package Semiconductors is for user access but not the package Internal.</p>
</html>", revisions = "<html>
<ul>
<li><i>March 2008 </i>by Kristin Majetta <br/>initially implemented</li>
</ul>
</html>"));
        end M_NMOS2;

        model M_PMOS2 "PMOS MOSFET device"
          extends Modelica.Electrical.Spice3.Internal.MOS2(final mtype = 1);
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{60, 0}, {40, 5}, {40, -5}, {60, 0}}, lineColor = {0, 0, 255}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid)}),
            Documentation(info = "<html>
<p>The model M_PMOS is a P channel MOSFET transistor with fixed level 2:</p>
<p>The models from the package Semiconductors accesses to the package Internal where all functions,</p>
<p>records and data are stored and modeled that are needed for the semiconductor models.</p>
<p>The package Semiconductors is for user access but not the package Internal.</p>
</html>", revisions = "<html>
<ul>
<li><i>March 2008 </i>by Kristin Majetta <br/>initially implemented</li>
</ul>
</html>"));
        end M_PMOS2;

        record ModelcardMOS2 "Record for the specification of modelcard parameters"
          extends Modelica.Icons.Record;
          extends Modelica.Electrical.Spice3.Internal.ModelcardMOS2;
          annotation(
            Documentation(info = "<html>
<p>Technology model parameters of MOSFET transistor with fixed level 1: Shichman-Hodges model</p>
<p>In modelcards, that are typical for SPICE3, the so called technology parameters are stored. These parameters are usually set for more than one semiconductor device in a circuit, e.g., the temperature of a whole electrical circuit.</p>
</html>"));
        end ModelcardMOS2;

        model Q_NPNBJT "Bipolar junction transistor"
          extends Modelica.Electrical.Spice3.Internal.BJT2(final TBJT = 1);
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{8, -68}, {2, -55}, {-4, -62}, {8, -68}}, lineColor = {0, 0, 255}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid)}),
            Documentation(info = "<html>
<p>The model Q_NPNBJT is a NPN bipolar junction transistor model: Modified Gummel-Poon.</p>
<p>The models from the package Semiconductors accesses to the package Internal where all functions,</p>
<p>records and data are stored and modeled that are needed for the semiconductor models.</p>
<p>The package Semiconductors is for user access but not the package Internal.</p>
</html>", revisions = "<html>
<ul>
<li><i>August 2009 </i>by Kristin Majetta <br/>initially implemented</li>
</ul>
</html>"));
        end Q_NPNBJT;

        model Q_PNPBJT "Bipolar junction transistor"
          extends Modelica.Electrical.Spice3.Internal.BJT2(final TBJT = -1);
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{0, -60}, {14, -67}, {8, -74}, {0, -60}}, lineColor = {0, 0, 255}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid)}),
            Documentation(info = "<html>
<p>The model Q_PNPBJT is a PNP bipolar junction transistor model: Modified Gummel-Poon.</p>
<p>The models from the package Semiconductors accesses to the package Internal where all functions,</p>
<p>records and data are stored and modeled that are needed for the semiconductor models.</p>
<p>The package Semiconductors is for user access but not the package Internal.</p>
</html>", revisions = "<html>
<ul>
<li><i>August 2009 </i>by Kristin Majetta <br/>initially implemented</li>
</ul>
</html>"));
        end Q_PNPBJT;

        record ModelcardBJT "Record for the specification of modelcard parameters"
          extends Modelica.Icons.Record;
          extends Modelica.Electrical.Spice3.Internal.ModelcardBJT2;
          annotation(
            Documentation(info = "<html>
<p>In modelcards, that are typical for SPICE3, the so called technology parameters are stored. These parameters are usually set for more than one semiconductor device in a circuit, e.g., the temperature of a whole electrical circuit.</p>
<p>Technology parameters of the modified Gummel-Poon bipolar junction transistor model</p>
</html>"));
        end ModelcardBJT;

        model J_PJFJFET "P-channel Junction Field-Effect Transistor model (JFET)"
          extends Modelica.Electrical.Spice3.Internal.JFET(final mtype = 1);
          annotation(
            Documentation(info = "<html>
<p>J_PJFJFET is a P-channel junction field-effect transistor.</p>
<p>The junction field-effect transistor is derived from the FET model of Shichman and Hodges.</p>
<p>The models from the package Semiconductors accesses to the package Internal where all functions, records and data are stored and modeled that are needed for the semiconductor models. The package Semiconductors is for user access, but not the package Internal.</p>
</html>", revisions = "<html>
<ul>
<li><i>September 2011 </i>revised by Sandra B&ouml;hme</br/>
<li><i>August 2009 </i>by Kristin Majetta <br/>initially implemented</li>
</ul>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{8, -68}, {2, -55}, {-4, -62}, {8, -68}}, lineColor = {0, 0, 255}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid)}));
        end J_PJFJFET;

        model J_NJFJFET "N-channel Junction Field-Effect Transistor model (JFET)"
          extends Modelica.Electrical.Spice3.Internal.JFET(final mtype = 0);
          annotation(
            Documentation(info = "<html>
<p>J_NJFJFET is a N-channel junction field-effect transistor.</p>
<p>The junction field-effect transistor is derived from the FET model of Shichman and Hodges.</p>
<p>The models from the package Semiconductors accesses to the package Internal where all functions, records and data are stored and modeled that are needed for the semiconductor models. The package Semiconductors is for user access, but not the package Internal.</p>
</html>", revisions = "<html>
<ul>
<li><i>September 2011 </i>revised by Sandra B&ouml;hme</br/>
<li><i>August 2009 </i>by Kristin Majetta <br/>initially implemented</li>
</ul>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{8, -68}, {2, -55}, {-4, -62}, {8, -68}}, lineColor = {0, 0, 255}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid)}));
        end J_NJFJFET;

        record ModelcardJFET "Record for the specification of modelcard parameters for JFET"
          extends Modelica.Icons.Record;
          extends Modelica.Electrical.Spice3.Internal.ModelcardJFET;
          annotation(
            Documentation(info = "<html>
<p>Technology parameters of the junction field-effect transistor model.</p>
<p>In modelcards, that are typical for SPICE3, the so called technology parameters are stored. These parameters are usually set for more than one semiconductor device in a circuit, e.g., the temperature of a whole electrical circuit.</p>
</html>"));
        end ModelcardJFET;

        model D_DIODE "Diode model"
          extends Modelica.Electrical.Spice3.Internal.DIODE;
          annotation(
            Documentation(info = "<html>
<p>The model D_DIODE is a Junction diode model</p>
<p>The models from the package Semiconductors accesses to the package Repository where all functions,</p>
<p>records and data are stored and modeled that are needed for the semiconductor models.</p>
<p>The package Semiconductors is for user access but not the package Repository.</p>
</html>", revisions = "<html>
<ul>
<li><i>Nov. 2008 </i>by Kristin Majetta <br/>initially implemented</li>
</ul>
</html>"));
        end D_DIODE;

        record ModelcardDIODE "Record for the specification of modelcard parameters"
          extends Modelica.Icons.Record;
          extends Modelica.Electrical.Spice3.Internal.ModelcardDIODE;
          annotation(
            Documentation(info = "<html>
<p>In modelcards, that are typical for SPICE3, the so called technology parameters are stored. These parameters are usually set for more than one semiconductor device in a circuit, e.g., the temperature of a whole electrical circuit.</p>
<p>Technology parameters of the junction diode model</p>
</html>"));
        end ModelcardDIODE;

        model R_Resistor "Semiconductor resistor from SPICE3"
          extends Modelica.Electrical.Spice3.Internal.R_SEMI;
          annotation(
            Documentation(info = "<html>
<p>The model R_Resistor is a Semiconductor resistor model.</p>
<p>The models from the package Semiconductors accesses to the package Repository where all functions,</p>
<p>records and data are stored and modeled that are needed for the semiconductor models.</p>
<p>The package Semiconductors is for user access but not the package Repository.</p>
</html>", revisions = "<html>
<ul>
<li><i>April 2009 </i>by Kristin Majetta <br/>initially implemented</li>
</ul>
</html>"));
        end R_Resistor;

        record ModelcardRESISTOR "Record for the specification of modelcard parameters"
          extends Modelica.Icons.Record;
          extends Modelica.Electrical.Spice3.Internal.ModelcardR;
          annotation(
            Documentation(info = "<html>
<p>In modelcards, that are typical for SPICE3, the so called technology parameters are stored. These parameters are usually set for more than one semiconductor device in a circuit, e.g., the temperature of a whole electrical circuit.</p>
<p>Technology parameters of the semiconductor resistor model</p>
</html>"));
        end ModelcardRESISTOR;

        model C_Capacitor "Semiconductor capacitor"
          extends Modelica.Electrical.Spice3.Internal.C_SEMI;
          annotation(
            Documentation(info = "<html>
<p>C_Capacitor is a Semiconductor Capacitor model.</p>
<p>This capacitor model allows the calculation of the actual capacitance value from strictly geometric information and the specification of the process.</p>
<p>The models from the package Semiconductors accesses to the package Repository where all functions, records and data are stored and modeled that are needed for the semiconductor models. The package Semiconductors is for user access, but not the package Repository.</p>
</html>", revisions = "<html>
<ul>
<li><i>September 2011 </i>revised by Sandra B&ouml;hme</br/>
<li><i>April 2009 </i>by Kristin Majetta <br/>initially implemented</li>
</ul>
</html>"));
        end C_Capacitor;

        record ModelcardCAPACITOR "Record for the specification of modelcard parameters for Semiconductor Capacitor"
          extends Modelica.Icons.Record;
          extends Modelica.Electrical.Spice3.Internal.ModelcardC;
          annotation(
            Documentation(info = "<html>
<p>Technology parameters of the semiconductor capacitor model.</p>
<p>In modelcards, that are typical for SPICE3, the so called technology parameters are stored. These parameters are usually set for more than one semiconductor device in a circuit, e.g., the temperature of a whole electrical circuit.</p>
</html>"));
        end ModelcardCAPACITOR;
        annotation(
          preferredView = "info",
          Documentation(info = "<html>
<p>This package contains both the semiconductor devices models of SPICE3, which are available, and their modelcards. The user should apply the models of this package.</p>
<p>All models of this package extend models of the package Repository, which contains the functions, parameters and data which are necessary to model the behaviour of the semiconductor devices.The modelcard records contain the SPICE3 technology parameters, which can be adjusted for more than one MOS simultaneously.</p>
</html>"));
      end Semiconductors;

      package Sources "Time dependent SPICE3 voltage and current sources"
        extends Modelica.Icons.SourcesPackage;

        model V_constant "Constant independent voltage sources"
          parameter SI.Voltage V = 1 "Value of constant voltage";
          extends Modelica.Electrical.Analog.Interfaces.OnePort;
        equation
          v = V;
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-90, 0}, {-10, 0}}, color = {0, 0, 0}), Line(points = {{-10, 60}, {-10, -60}}, color = {0, 0, 0}), Line(points = {{0, 30}, {0, -30}}, color = {0, 0, 0}), Line(points = {{0, 0}, {90, 0}}, color = {0, 0, 0}), Text(extent = {{-100, -120}, {100, -80}}, textString = "%name=%V")}),
            Documentation(info = "<html>
<p>The V_constant source is a  source is a simple constant voltage source for an ideal constant voltage which is provided by a parameter.</p>
</html>"));
        end V_constant;

        model V_sin "Sinusoidal voltage source"
          extends Modelica.Electrical.Analog.Interfaces.OnePort;
          parameter SI.Voltage VO = 0.0 "Offset";
          parameter SI.Voltage VA = 0.0 "Amplitude";
          parameter SI.Frequency FREQ(start = 1) "Frequency";
          parameter SI.Time TD = 0.0 "Delay";
          parameter SI.Damping THETA = 0.0 "Damping factor";
        protected
          constant Real pi = Modelica.Constants.pi;
        equation
          assert(FREQ > 0, "Frequency less or equal zero");
          v = VO + (if time < TD then 0 else VA * Modelica.Math.exp(-(time - TD) * THETA) * Modelica.Math.sin(2 * pi * FREQ * (time - TD)));
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-122, 53}, {-22, 3}}, lineColor = {0, 0, 255}, textString = "+"), Text(extent = {{19, 59}, {119, 9}}, lineColor = {0, 0, 255}, textString = "-"), Ellipse(extent = {{-51, 50}, {49, -50}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{-90, 0}, {50, 0}}, color = {0, 0, 255}), Line(points = {{50, 0}, {90, 0}}, color = {0, 0, 255}), Line(points = {{-80, 4}, {-75.2, 36.3}, {-72, 54.3}, {-68.7, 68.5}, {-65.5, 78.2}, {-62.3, 83.3}, {-59.1, 83.6}, {-55.9, 79.3}, {-52.7, 71.1}, {-48.6, 56.2}, {-43, 29.8}, {-35, -9.9}, {-30.2, -29.7}, {-26.1, -41.9}, {-22.1, -49.2}, {-18.1, -51.3}, {-14.1, -48.5}, {-10.1, -41.3}, {-5.23, -28.1}, {8.44, 17.7}, {13.3, 30.4}, {18.1, 38.8}, {22.1, 42}, {26.9, 41.2}, {31.8, 35.8}, {38.2, 23.4}, {51.1, -6.5}, {57.5, -17.2}, {63.1, -21.9}, {68.7, -21.9}, {75.2, -16.5}, {80, -9.8}}, color = {192, 192, 192}), Text(extent = {{-150, -56}, {150, -96}}, textString = "%name", lineColor = {0, 0, 255})}),
            Documentation(info = "<html>
<p>Damped sinusoidal source</p>
<h4>Note:</h4>
<ul>
<li>All parameters of sources should be set explicitly.</li>
<li>since TSTEP and TSTOP are not available for modeling in Modelica, differences to SPICE may occur if not all parameters are set.<br></li>
</ul>
</html>"));
        end V_sin;

        model V_exp "Exponential voltage source"
          extends Modelica.Electrical.Analog.Interfaces.OnePort;
          parameter SI.Voltage V1 = 0 "Initial value";
          parameter SI.Voltage V2 = 0 "Pulsed value";
          parameter SI.Time TD1 = 0.0 "Rise delay time";
          parameter SI.Time TAU1 = 1 "Rise time constant";
          parameter SI.Time TD2 = 1 "Fall delay time";
          parameter SI.Time TAU2 = 1 "Fall time constant";
        equation
          v = V1 + (if time < TD1 then 0 else if time < TD2 then (V2 - V1) * (1 - Modelica.Math.exp(-(time - TD1) / TAU1)) else (V2 - V1) * (1 - Modelica.Math.exp(-(time - TD1) / TAU1)) + (V1 - V2) * (1 - Modelica.Math.exp(-(time - TD2) / TAU2)));
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-117, 57}, {-17, 7}}, lineColor = {0, 0, 255}, textString = "+"), Text(extent = {{22, 60}, {122, 10}}, lineColor = {0, 0, 255}, textString = "-"), Text(extent = {{-153, -58}, {147, -98}}, textString = "%name", lineColor = {0, 0, 255}), Ellipse(extent = {{-51, 50}, {49, -50}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{-90, 0}, {50, 0}}, color = {0, 0, 255}), Line(points = {{50, 0}, {90, 0}}, color = {0, 0, 255}), Line(points = {{-70, -48}, {-67.2, -33.3}, {-64.3, -20.1}, {-60.8, -5.6}, {-57.3, 7}, {-53.7, 17.92}, {-49.5, 29.18}, {-45.3, 38.7}, {-40.3, 48}, {-34.6, 56.5}, {-28.3, 64.1}, {-21.2, 70.6}, {-12.7, 76.3}, {-2.1, 81.2}, {0, 82}, {2.12, 69.5}, {4.95, 54.7}, {7.78, 41.8}, {10, 31}, {14.14, 18.3}, {17.68, 8}, {21.9, -2.2}, {26.2, -10.6}, {31.1, -18.5}, {36.8, -25.4}, {43.1, -31.3}, {50.9, -36.5}, {60.8, -40.8}, {70, -43.4}}, color = {192, 192, 192})}),
            Documentation(info = "<html>
<p>Rising and falling exponential source.</p>
<h4>Note:</h4>
<ul>
<li>All parameters of sources should be set explicitly.</li>
<li>since TSTEP and TSTOP are not available for modeling in Modelica, differences to SPICE may occur if not all parameters are set.- it should be set all the parameters definitely <br>- normally, there exist differences between Dymola and Spice, because TSTEP and TSTOP are not available. <br></li>
</ul>
</html>"));
        end V_exp;

        model V_pulse "Pulse voltage source"
          extends Modelica.Electrical.Analog.Interfaces.OnePort;
          parameter SI.Voltage V1 = 0 "Initial value";
          parameter SI.Voltage V2 = 0 "Pulsed value";
          parameter SI.Time TD = 0.0 "Delay time";
          parameter SI.Time TR(start = 1) "Rise time";
          parameter SI.Time TF = TR "Fall time";
          parameter SI.Time PW = Modelica.Constants.inf "Pulse width";
          parameter SI.Time PER = Modelica.Constants.inf "Period";
        protected
          parameter SI.Time Trising = TR "End time of rising phase within one period";
          parameter SI.Time Twidth = Trising + PW "End time of width phase within one period";
          parameter SI.Time Tfalling = Twidth + TF "End time of falling phase within one period";
          SI.Time T0(final start = TD, fixed = true) "Start time of current period";
          Integer counter(start = -1, fixed = true) "Period counter";
          Integer counter2(start = -1, fixed = true);
        equation
          when pre(counter2) <> 0 and sample(TD, PER) then
            T0 = time;
            counter2 = pre(counter);
            counter = pre(counter) - (if pre(counter) > 0 then 1 else 0);
          end when;
          v = V1 + (if time < TD or counter2 == 0 or time >= T0 + Tfalling then 0 else if time < T0 + Trising then (time - T0) * (V2 - V1) / Trising else if time < T0 + Twidth then V2 - V1 else (T0 + Tfalling - time) * (V2 - V1) / (Tfalling - Twidth));
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-120, 49}, {-20, -1}}, lineColor = {0, 0, 255}, textString = "+"), Text(extent = {{25, 53}, {125, 3}}, lineColor = {0, 0, 255}, textString = "-"), Text(extent = {{-157, -62}, {143, -102}}, textString = "%name", lineColor = {0, 0, 255}), Ellipse(extent = {{-51, 50}, {49, -50}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{-90, 0}, {50, 0}}, color = {0, 0, 255}), Line(points = {{50, 0}, {90, 0}}, color = {0, 0, 255}), Line(points = {{-86, -74}, {-65, -74}, {-35, 66}, {-4, 66}, {25, -74}, {46, -74}, {75, 66}}, color = {192, 192, 192})}),
            Documentation(info = "<html>
<p>Periodic pulse source with not limited number of periods.</p>
<p>A single pulse is described by the following table:</p>
<table cellspacing=\"2\" cellpadding=\"0\" border=\"0\"><tr>
<td><h4>time</h4></td>
<td><h4>value</h4></td>
</tr>
<tr>
<td><p>0</p></td>
<td><p>V1</p></td>
</tr>
<tr>
<td><p>TD</p></td>
<td><p>V1</p></td>
</tr>
<tr>
<td><p>TD+TR</p></td>
<td><p>V2</p></td>
</tr>
<tr>
<td><p>TD+TR+PW</p></td>
<td><p>V2</p></td>
</tr>
<tr>
<td><p>TD+TR+PW+TF</p></td>
<td><p>V1</p></td>
</tr>
<tr>
<td><p>TSTOP</p></td>
<td><p>V1</p></td>
</tr>
</table>
<p>Intermediate points are determined by linear interpolation.</p>
<p>A pulse it looks like a saw tooth, use this parameters e.g.:</p>
<table cellspacing=\"2\" cellpadding=\"2\" border=\"1\"><tr>
<td><h4>Parameter</h4></td>
<td><h4>Value</h4></td>
</tr>
<tr>
<td><p>V1</p></td>
<td><p>0</p></td>
</tr>
<tr>
<td><p>V2</p></td>
<td><p>1</p></td>
</tr>
<tr>
<td><p>TD</p></td>
<td><p>0</p></td>
</tr>
<tr>
<td><p>TR</p></td>
<td><p>1</p></td>
</tr>
<tr>
<td><p>TF</p></td>
<td><p>1</p></td>
</tr>
<tr>
<td><p>PW</p></td>
<td><p>2</p></td>
</tr>
<tr>
<td><p>PER</p></td>
<td><p>1</p></td>
</tr>
</table>
<h4>Note:</h4>
<ul>
<li>All parameters of sources should be set explicitly.</li>
<li>since TSTEP and TSTOP are not available for modeling in Modelica, differences to SPICE may occur if not all parameters are set.</li>
</ul>
</html>"));
        end V_pulse;

        model V_pwl "Piece-wise linear voltage source"
          extends Modelica.Electrical.Analog.Interfaces.OnePort;
          parameter Real table[:, :] = [0, 0; 1, 1; 2, 4] "Table matrix (time = first column, voltage = second column)";
        protected
          parameter Integer x = size(table, 1);
          parameter Real tlast = table[x, 1] + 1;
          parameter Real valuelast = table[x, 2];
          parameter Real lastvaluematrix[1, 2] = [tlast, valuelast];
          parameter Real tablenew[:, 2] = cat(1, table, lastvaluematrix);
          Modelica.Blocks.Sources.TimeTable tab(table = tablenew);
        equation
          v = tab.y;
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-150, -56}, {150, -96}}, textString = "%name", lineColor = {0, 0, 255}), Text(extent = {{-120, 49}, {-20, -1}}, lineColor = {0, 0, 255}, textString = "+"), Text(extent = {{25, 53}, {125, 3}}, lineColor = {0, 0, 255}, textString = "-"), Ellipse(extent = {{-51, 50}, {49, -50}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{-90, 0}, {50, 0}}, color = {0, 0, 255}), Line(points = {{50, 0}, {90, 0}}, color = {0, 0, 255}), Line(points = {{-40, -50}, {-40, 70}, {60, 70}, {60, -50}, {-40, -50}, {-40, -20}, {60, -20}, {60, 10}, {-40, 10}, {-40, 40}, {60, 40}, {60, 70}, {10, 70}, {10, -51}}, color = {192, 192, 192})}),
            Documentation(info = "<html>
<p>This model generates a voltage by <b>linear interpolation</b> in a given table. The time points and voltage values are stored in a matrix <b>table[i,j]</b>, where the first column table[:,1] contains the time points and the second column contains the voltage to be interpolated. The table interpolation has the following properties:</p>
<ul>
<li>The time points need to be <b>monotonically increasing</b>. </li>
<li><b>Discontinuities</b> are allowed, by providing the same time point twice in the table. </li>
<li>Values <b>outside</b> of the table range, are computed by <b>extrapolation</b> through the last or first two points of the table.</li>
<li>If the table has only <b>one row</b>, no interpolation is performed and the voltage value is just returned independently of the actual time instant, i.e., this is a constant voltage source.</li>
<li>Via parameters <b>startTime</b> and <b>offset</b> the curve defined by the table can be shifted both in time and in the voltage. </li>
<li>The table is implemented in a numerically sound way by generating <b>time events</b> at interval boundaries.
    This generates continuously differentiable values for the integrator.</li>
</ul>
<p>Example:</p>
<pre>   table = [0  0
            1  0
            1  1
            2  4
            3  9
            4 16]
If, e.g., time = 1.0, the voltage v =  0.0 (before event), 1.0 (after event)
    e.g., time = 1.5, the voltage v =  2.5,
    e.g., time = 2.0, the voltage v =  4.0,
    e.g., time = 5.0, the voltage v = 23.0 (i.e., extrapolation).
</pre>
</html>"));
        end V_pwl;

        model V_sffm "Single-frequency FM voltage source"
          extends Modelica.Electrical.Analog.Interfaces.OnePort;
          parameter SI.Voltage VO = 0 "Offset";
          parameter SI.Voltage VA = 0 "Amplitude";
          parameter SI.Frequency FC(start = 0) "Carrier frequency";
          parameter Real MDI = 0 "Modulation index";
          parameter SI.Frequency FS = FC "Signal frequency";
        protected
          constant Real pi = Modelica.Constants.pi;
        equation
          v = VO + VA * Modelica.Math.sin(2 * pi * FC * time + MDI * Modelica.Math.sin(2 * pi * FS * time));
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-150, -56}, {150, -96}}, textString = "%name", lineColor = {0, 0, 255}), Text(extent = {{-120, 49}, {-20, -1}}, lineColor = {0, 0, 255}, textString = "+"), Text(extent = {{25, 53}, {125, 3}}, lineColor = {0, 0, 255}, textString = "-"), Ellipse(extent = {{-51, 50}, {49, -50}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{-90, 0}, {50, 0}}, color = {0, 0, 255}), Line(points = {{50, 0}, {90, 0}}, color = {0, 0, 255}), Text(extent = {{-51, 57}, {55, 18}}, lineColor = {0, 0, 255}, textString = "SFFM")}),
            Documentation(info = "<html>
<p>The single-frequency frequency modulation source generates a carrier signal of the frequency FC. This signal is modulated by the signal frequency FS. See the formula in the Modelica text.</p>
<h4>Attention:</h4>
<ul>
<li>All parameters of sources should be set explicitly.</li>
<li>since TSTEP and TSTOP are not available for modeling in Modelica, differences to SPICE may occur if not all parameters are set.</li>
</ul>
</html>"));
        end V_sffm;

        model I_constant "Constant independent current sources"
          parameter SI.Current I = 1 "Value of constant voltage";
          extends Modelica.Electrical.Analog.Interfaces.OnePort;
        equation
          i = I;
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-94, -106}, {106, -66}}, textString = "%name=%I"), Ellipse(extent = {{-51, 50}, {49, -50}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{-90, 0}, {-50, 0}}, color = {0, 0, 255}), Line(points = {{50, 0}, {90, 0}}, color = {0, 0, 255}), Line(points = {{-1, -50}, {-1, 50}}, color = {0, 0, 255})}),
            Documentation(info = "<html>
<p>The I_constant source is a simple constant current source for an ideal constant current which is provided by a parameter.</p>
</html>"));
        end I_constant;

        model I_sin "Sinusoidal current source"
          extends Modelica.Electrical.Analog.Interfaces.OnePort;
          parameter SI.Current IO = 0 "Offset";
          parameter SI.Current IA = 0 "Amplitude";
          parameter SI.Frequency FREQ(start = 1) "Frequency";
          parameter SI.Time TD = 0.0 "Delay";
          parameter SI.Damping THETA = 0.0 "Damping factor";
        protected
          constant Real pi = Modelica.Constants.pi;
        equation
          assert(FREQ > 0, "Frequency less or equal zero");
          i = IO + (if time < TD then 0 else IA * Modelica.Math.exp(-(time - TD) * THETA) * Modelica.Math.sin(2 * pi * FREQ * (time - TD)));
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{90, 0}, {60, 10}, {60, -10}, {90, 0}}, lineColor = {0, 0, 255}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-51, 50}, {49, -50}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{-90, 0}, {-50, 0}}, color = {0, 0, 255}), Line(points = {{50, 0}, {90, 0}}, color = {0, 0, 255}), Line(points = {{-1, -50}, {-1, 50}}, color = {0, 0, 255}), Text(extent = {{-144, -64}, {156, -104}}, textString = "%name", lineColor = {0, 0, 255}), Line(points = {{-77, -12}, {-72.2, 20.3}, {-69, 38.3}, {-65.7, 52.5}, {-62.5, 62.2}, {-59.3, 67.3}, {-56.1, 67.6}, {-52.9, 63.3}, {-49.7, 55.1}, {-45.6, 40.2}, {-40, 13.8}, {-32, -25.9}, {-27.2, -45.7}, {-23.1, -57.9}, {-19.1, -65.2}, {-15.1, -67.3}, {-11.1, -64.5}, {-7.1, -57.3}, {-2.23, -44.1}, {11.44, 1.7}, {16.3, 14.4}, {21.1, 22.8}, {25.1, 26}, {29.9, 25.2}, {34.8, 19.8}, {41.2, 7.4}, {54.1, -22.5}, {60.5, -33.2}, {66.1, -37.9}, {71.7, -37.9}, {78.2, -32.5}, {83, -25.8}}, color = {192, 192, 192})}),
            Documentation(info = "<html>
<p>Damped sinusoidal source</p>
<h4>Note:</h4>
<ul>
<li>All parameters of sources should be set explicitly.</li>
<li>since TSTEP and TSTOP are not available for modeling in Modelica, differences to SPICE may occur if not all parameters are set.</li>
</ul>
</html>"));
        end I_sin;

        model I_exp "Exponential current source"
          extends Modelica.Electrical.Analog.Interfaces.OnePort;
          parameter SI.Current I1 = 0 "Initial value";
          parameter SI.Current I2 = 0 "Pulsed value";
          parameter SI.Time TD1 = 0.0 "Rise delay time";
          parameter SI.Time TAU1 = 1 "Rise time constant";
          parameter SI.Time TD2 = 2 "Fall delay time";
          parameter SI.Time TAU2 = 1 "Fall time constant";
        equation
          i = I1 + (if time < TD1 then 0 else if time < TD2 then (I2 - I1) * (1 - Modelica.Math.exp(-(time - TD1) / TAU1)) else (I2 - I1) * (1 - Modelica.Math.exp(-(time - TD1) / TAU1)) + (I1 - I2) * (1 - Modelica.Math.exp(-(time - TD2) / TAU2)));
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{90, 0}, {60, 10}, {60, -10}, {90, 0}}, lineColor = {0, 0, 255}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-50, 50}, {50, -50}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{-89, 0}, {-49, 0}}, color = {0, 0, 255}), Line(points = {{51, 0}, {91, 0}}, color = {0, 0, 255}), Line(points = {{0, -50}, {0, 50}}, color = {0, 0, 255}), Text(extent = {{-145, -60}, {155, -100}}, textString = "%name", lineColor = {0, 0, 255}), Line(points = {{-79, -53}, {-76.2, -38.3}, {-73.3, -25.1}, {-69.8, -10.6}, {-66.3, 2}, {-62.7, 12.92}, {-58.5, 24.18}, {-54.3, 33.7}, {-49.3, 43}, {-43.6, 51.5}, {-37.3, 59.1}, {-30.2, 65.6}, {-21.7, 71.3}, {-11.1, 76.2}, {-9, 77}, {-6.88, 64.5}, {-4.05, 49.7}, {-1.22, 36.8}, {1.606, 25.45}, {5.14, 13.3}, {8.68, 3}, {12.9, -7.2}, {17.2, -15.6}, {22.1, -23.5}, {27.8, -30.4}, {34.1, -36.3}, {41.9, -41.5}, {51.8, -45.8}, {61, -48.4}}, color = {192, 192, 192})}),
            Documentation(info = "<html>
<p>Rising and falling exponential source.</p>
<h4>Note:</h4>
<ul>
<li>All parameters of sources should be set explicitly.</li>
<li>since TSTEP and TSTOP are not available for modeling in Modelica, differences to SPICE may occur if not all parameters are set.</li>
</ul>
</html>"));
        end I_exp;

        model I_pulse "Pulse current source"
          extends Modelica.Electrical.Analog.Interfaces.OnePort;
          parameter SI.Current I1 = 0 "Initial value";
          parameter SI.Current I2 = 0 "Pulsed value";
          parameter SI.Time TD = 0.0 "Delay time";
          parameter SI.Time TR(start = 1) "Rise time";
          parameter SI.Time TF = TR "Fall time";
          parameter SI.Time PW = Modelica.Constants.inf "Pulse width";
          parameter SI.Time PER = Modelica.Constants.inf "Period";
        protected
          parameter SI.Time Trising = TR "End time of rising phase within one period";
          parameter SI.Time Twidth = Trising + PW "End time of width phase within one period";
          parameter SI.Time Tfalling = Twidth + TF "End time of falling phase within one period";
          SI.Time T0(final start = TD, fixed = true) "Start time of current period";
          Integer counter(start = -1) "Period counter";
          Integer counter2(start = -1);
        equation
          when pre(counter2) <> 0 and sample(TD, PER) then
            T0 = time;
            counter2 = pre(counter);
            counter = pre(counter) - (if pre(counter) > 0 then 1 else 0);
          end when;
          i = I1 + (if time < TD or counter2 == 0 or time >= T0 + Tfalling then 0 else if time < T0 + Trising then (time - T0) * (I2 - I1) / Trising else if time < T0 + Twidth then I2 - I1 else (T0 + Tfalling - time) * (I2 - I1) / (Tfalling - Twidth));
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{90, 0}, {60, 10}, {60, -10}, {90, 0}}, lineColor = {0, 0, 255}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-51, 50}, {49, -50}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{-90, 0}, {-50, 0}}, color = {0, 0, 255}), Line(points = {{50, 0}, {90, 0}}, color = {0, 0, 255}), Line(points = {{-1, -50}, {-1, 50}}, color = {0, 0, 255}), Text(extent = {{-150, -58}, {150, -98}}, textString = "%name", lineColor = {0, 0, 255}), Line(points = {{-85, -60}, {-64, -60}, {-34, 80}, {-3, 80}, {26, -60}, {47, -60}, {76, 80}}, color = {192, 192, 192})}),
            Documentation(info = "<html>
<p>Periodic pulse source with not limited number of periods.</p>
<p>A single pulse is described by the following table:</p>
<table cellspacing=\"2\" cellpadding=\"0\" border=\"0\"><tr>
<td><h4>time</h4></td>
<td><h4>value</h4></td>
</tr>
<tr>
<td><p>0</p></td>
<td><p>I1</p></td>
</tr>
<tr>
<td><p>TD</p></td>
<td><p>I1</p></td>
</tr>
<tr>
<td><p>TD+TR</p></td>
<td><p>I2</p></td>
</tr>
<tr>
<td><p>TD+TR+PW</p></td>
<td><p>I2</p></td>
</tr>
<tr>
<td><p>TD+TR+PW+TF</p></td>
<td><p>I1</p></td>
</tr>
<tr>
<td><p>TSTOP</p></td>
<td><p>I1</p></td>
</tr>
</table>
<p>Intermediate points are determined by linear interpolation.</p>
<p>A pulse it looks like a saw tooth, use this parameters e.g.:</p>
<table cellspacing=\"2\" cellpadding=\"2\" border=\"1\"><tr>
<td><h4>Parameter</h4></td>
<td><h4>Value</h4></td>
</tr>
<tr>
<td><p>I1</p></td>
<td><p>0</p></td>
</tr>
<tr>
<td><p>I2</p></td>
<td><p>1</p></td>
</tr>
<tr>
<td><p>TD</p></td>
<td><p>0</p></td>
</tr>
<tr>
<td><p>TR</p></td>
<td><p>1</p></td>
</tr>
<tr>
<td><p>TF</p></td>
<td><p>1</p></td>
</tr>
<tr>
<td><p>PW</p></td>
<td><p>2</p></td>
</tr>
<tr>
<td><p>PER</p></td>
<td><p>1</p></td>
</tr>
</table>
<h4>Note:</h4>
<ul>
<li>All parameters of sources should be set explicitly.</li>
<li>since TSTEP and TSTOP are not available for modeling in Modelica, differences to SPICE may occur if not all parameters are set.</li>
</ul>
</html>"));
        end I_pulse;

        model I_pwl "Piece-wise linear current source"
          extends Modelica.Electrical.Analog.Interfaces.OnePort;
          parameter Real table[:, :] = [0, 0; 1, 1; 2, 4] "Table matrix (time = first column, voltage = second column)";
        protected
          parameter Integer x = size(table, 1);
          parameter Real tlast = table[x, 1] + 1;
          parameter Real valuelast = table[x, 2];
          parameter Real lastvaluematrix[1, 2] = [tlast, valuelast];
          parameter Real tablenew[:, 2] = cat(1, table, lastvaluematrix);
          Modelica.Blocks.Sources.TimeTable tab(table = tablenew);
        equation
          i = tab.y;
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-51, 50}, {49, -50}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{-90, 0}, {-50, 0}}, color = {0, 0, 255}), Line(points = {{50, 0}, {90, 0}}, color = {0, 0, 255}), Line(points = {{-1, -50}, {-1, 50}}, color = {0, 0, 255}), Text(extent = {{-148, -57}, {152, -97}}, textString = "%name", lineColor = {0, 0, 255}), Line(points = {{-44, -39}, {-44, 81}, {56, 81}, {56, -39}, {-44, -39}, {-44, -9}, {56, -9}, {56, 21}, {-44, 21}, {-44, 51}, {56, 51}, {56, 81}, {6, 81}, {6, -40}}, color = {192, 192, 192}), Polygon(points = {{90, 0}, {60, 10}, {60, -10}, {90, 0}}, lineColor = {0, 0, 255}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid)}),
            Documentation(info = "<html>
<p>This model generates a current  by <b>linear interpolation</b> in a given table. The time points and current values are stored in a matrix <b>table[i,j]</b>, where the first column table[:,1] contains the time points and the second column contains the current to be interpolated. The table interpolation has the following properties:</p>

<ul>
<li>The time points need to be <b>monotonically increasing</b>. </li>
<li><b>Discontinuities</b> are allowed, by providing the same time point twice in the table. </li>
<li>Values <b>outside</b> of the table range, are computed by <b>extrapolation</b> through the last or first two points of the table.</li>
<li>If the table has only <b>one row</b>, no interpolation is performed and the current value is just returned independently of the actual time instant, i.e., this is a constant current source.</li>
<li>Via parameters <b>startTime</b> and <b>offset</b> the curve defined by the table can be shifted both in time and in the current. </li>
<li>The table is implemented in a numerically sound way by generating <b>time events</b> at interval boundaries.
    This generates continuously differentiable values for the integrator.</li>
</ul>

<p>Example:</p>
<pre>   table = [0  0
            1  0
            1  1
            2  4
            3  9
            4 16]
If, e.g., time = 1.0, the current i =  0.0 (before event), 1.0 (after event)
    e.g., time = 1.5, the current i =  2.5,
    e.g., time = 2.0, the current i =  4.0,
    e.g., time = 5.0, the current i = 23.0 (i.e., extrapolation).
</pre>
</html>"));
        end I_pwl;

        model I_sffm "Single-frequency FM current source"
          extends Modelica.Electrical.Analog.Interfaces.OnePort;
          parameter SI.Current IO = 0 "Offset";
          parameter SI.Current IA = 0 "Amplitude";
          parameter SI.Frequency FC(start = 0) "Carrier frequency";
          parameter Real MDI = 0 "Modulation index";
          parameter SI.Frequency FS = FC "Signal frequency";
        protected
          constant Real pi = Modelica.Constants.pi;
        equation
          i = IO + IA * Modelica.Math.sin(2 * pi * FC * time + MDI * Modelica.Math.sin(2 * pi * FS * time));
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-150, -56}, {150, -96}}, textString = "%name", lineColor = {0, 0, 255}), Ellipse(extent = {{-51, 50}, {49, -50}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{50, 0}, {90, 0}}, color = {0, 0, 255}), Line(points = {{-92, 0}, {-52, 0}}, color = {0, 0, 255}), Line(points = {{0, -50}, {0, 50}}, color = {0, 0, 255}), Polygon(points = {{90, 0}, {60, 10}, {60, -10}, {90, 0}}, lineColor = {0, 0, 255}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-58, 53}, {48, 14}}, lineColor = {0, 0, 255}, textString = "SFFM")}),
            Documentation(info = "<html>
<p>The single-frequency frequency modulation source generates a carrier signal of the frequency FC. This signal is modulated by the signal frequency FS. See the formula in the Modelica text.</p>
<h4>Note:</h4>
<ul>
<li>All parameters of sources should be set explicitly.</li>
<li>since TSTEP and TSTOP are not available for modeling in Modelica, differences to SPICE may occur if not all parameters are set.</li>
</ul>
</html>"));
        end I_sffm;
        annotation(
          preferredView = "info",
          Documentation(info = "<html>
<p>This package contains the SPICE sources.</p>
<p><b>Note:</b> There are differences between SPICE3 and Modelica concerning the default values of the parameter. Therefore it is recommended to specify <b>all</b> parameters of the source.</p>
</html>", revisions = "<html>
<ul>
<li><i>August 2009 </i>default values improved by Jonathan Kress<br/></li>
<li><i>October 2008 </i>by Christoph Clauss initially implemented.</li>
</ul>
</html>"));
      end Sources;

      package Additionals "Some useful additional models, e.g., from SPICE2 the polynomial sources"
        extends Modelica.Icons.Package;

        function poly "POLY function of SPICE2"
          extends Modelica.Icons.Function;
          input Real s[:] "Variables";
          input Real a[:] "Coefficients";
          output Real v "Value of polynomial";
        protected
          Integer n "number of polynomial variables, like POLY(n)";
          Integer na "number of polynomial coefficients, like POLY(n)";
          Integer ia "state of the usage of a";
        algorithm
          n := size(s, 1);
          na := size(a, 1);
          assert(n > 0, "poly: number of variables zero");
          assert(na > 0, "poly: number of coefficients zero");
          ia := 0;
// case one coefficient
          if na == 1 then
            v := a[1] * s[1];
            return;
          end if;
// absolute term
          v := a[1];
          ia := 1;
// linear terms
          for i1 in 1:n loop
            ia := ia + 1;
            if ia > na then
              return;
            end if;
            v := v + a[ia] * s[i1];
          end for;
// quadratic terms
          for i1 in 1:n loop
            for i2 in i1:n loop
              ia := ia + 1;
              if ia > na then
                return;
              end if;
              v := v + a[ia] * s[i1] * s[i2];
            end for;
          end for;
// cubic terms
          for i1 in 1:n loop
            for i2 in i1:n loop
              for i3 in i2:n loop
                ia := ia + 1;
                if ia > na then
                  return;
                end if;
                v := v + a[ia] * s[i1] * s[i2] * s[i3];
              end for;
            end for;
          end for;
// fourth potential terms
          for i1 in 1:n loop
            for i2 in i1:n loop
              for i3 in i2:n loop
                for i4 in i3:n loop
                  ia := ia + 1;
                  if ia > na then
                    return;
                  end if;
                  v := v + a[ia] * s[i1] * s[i2] * s[i3] * s[i4];
                end for;
              end for;
            end for;
          end for;
// fifth potential terms
          for i1 in 1:n loop
            for i2 in i1:n loop
              for i3 in i2:n loop
                for i4 in i3:n loop
                  for i5 in i4:n loop
                    ia := ia + 1;
                    if ia > na then
                      return;
                    end if;
                    v := v + a[ia] * s[i1] * s[i2] * s[i3] * s[i4] * s[i5];
                  end for;
                end for;
              end for;
            end for;
          end for;
          v := na;
          annotation(
            Documentation(info = "<html>
<p>Function needed for polynomial interpolation of POLY controlled sources:</p>
<ul>
<li>E_VCV_POLY</li>
<li>G_VCC_POLY</li>
<li>H_CCV_POLY</li>
<li>F_CCC_POLY</li>
</ul>
</html>"));
        end poly;

        model E_VCV_POLY "Polynomial voltage controlled voltage source, like SPICE2"
          parameter Integer N(final min = 1) = 1 "Number of controlling voltages";
          parameter Real coeff[:] = {1} "Coefficients of polynomial";
          Modelica.Electrical.Analog.Interfaces.PositivePin p "Positive pin of the controlled (normally right) port (potential p2.v > n2.v for positive voltage drop v2)" annotation(
            Placement(transformation(extent = {{110, 40}, {90, 60}}, rotation = 0), iconTransformation(extent = {{110, 40}, {90, 60}})));
          Modelica.Electrical.Analog.Interfaces.NegativePin n "Negative pin of the controlled (normally right) port" annotation(
            Placement(transformation(extent = {{90, -60}, {110, -40}}, rotation = 0), iconTransformation(extent = {{90, -60}, {110, -40}})));
          Modelica.Electrical.Analog.Interfaces.PositivePin pc[2 * N] "Pin vector of controlling pins (normally left)" annotation(
            Placement(transformation(extent = {{-90, -80}, {-70, 80}}, rotation = 0), iconTransformation(extent = {{-90, -80}, {-70, 80}})));
          Real control[N];
        equation
          p.i + n.i = 0;
          for i in 1:2 * N loop
            pc[i].i = 0;
          end for;
          for i in 1:N loop
            control[i] = pc[2 * i - 1].v - pc[2 * i].v;
          end for;
          p.v - n.v = poly(control, coeff);
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-70, 70}, {70, -70}}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Text(extent = {{-100, -62}, {99, -112}}, textString = "%name", lineColor = {0, 0, 255}), Line(points = {{100, 50}, {30, 50}, {30, -50}, {100, -50}}, color = {0, 0, 255}), Ellipse(extent = {{10, 20}, {50, -20}}, lineColor = {0, 0, 255}), Line(points = {{-20, 50}, {20, 50}}, color = {0, 0, 255}), Polygon(points = {{20, 50}, {10, 53}, {10, 47}, {20, 50}}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Text(extent = {{-54, -26}, {22, -60}}, lineColor = {0, 0, 255}, textString = "VCV")}),
            Documentation(info = "<html>
<p>The polynomial source is a SPICE2 model, which is also known in other SPICE derivates.</p>
<p>Nonlinear voltage controlled voltage source. The &quot;right&quot; port voltage between pin p2 and n2 (=p2.v - n2.v) is controlled by the &quot;left&quot; port vector of voltages at the pin vector pc[:] via</p>
<pre>    p2.v - n2.v = f(pc[1].v - pc[2].v, pc[3].v - pc[4].v,...)</pre>
<p>The controlling port (left) current vector is zero.</p>
<p>f is a polynomial in N variables s1...sN of the following form with M+1 coefficients a0, a1, a2,...aM.</p>
<pre>f = a0 +
    a1s1 + a2s2 + ... + aNsN +
    a(N+1)s1&sup2; + a(N+2)s1s2 + ... + a(.)s1sN +
    a(.)s2&sup2; + a(.)s2s3 + ... + a(.)s2sN +
    a(.)s3&sup2; + s3s4 + ... + a(.)s4sN +
    ... +
    a(.)sN&sup2; +
    a(.)s1&sup3; + a(.)s1&sup2;s2 + a(.)s1&sup2;s3 + ... + a(.)s1&sup2;sN +
    a(.)s1s2&sup2; + a(.)s1s2s3 + ... + a(.)s1s2sN +
    ... +
    a(.)sN&sup3; + ... </pre>
<p>The Coefficients a(.) are counted in this order. Reaching M, the particular sum is canceled.</p>
<p>In connection with the VCV, s1...sN are the voltages of the controlling side: s1=pc[1].v - pc[2].v, s2=pc[3].v - pc[4].v, s3=...</p>
<p>The corresponding SPICE description of the VCV polynomial source is the following:</p>
<pre>    Ename A1 A2 POLY(N) E11 E21 ... E1N E2N P0 P1...</pre>
<p>where Ename is the name of the instance, A1 and A2 are the nodes between them the controlled voltage is gripped,</p>
<p>N is the number of the controlling voltages, E11 E12 ... E1N E2N are pairs of nodes between them the controlling voltages</p>
<p>are gripped, and P0, P1... are the coefficients that are called a0, a1, ... aM in the description of the polynomial f above.</p>
<p>To describe the SPICE line in Modelica, the following explanation would be useful:</p>
<pre>Ename -&gt; E_VCV_POLY name
A1, A2 -&gt; pins name.p2, name.p1
N -&gt; parameter N
E11 -&gt; name.pc[2]
E12 -&gt; name.pc[1]
...
E1N -&gt; name.pc[N]
E2N -&gt; name.pc[N-1]
P0, P1 -&gt; polynomial coefficients name.coeff(coeff={P0,P1,...})</pre>
</html>", revisions = "<html>
<ul>
<li><i>Sept 2008 </i>by Kristin Majetta <br/>initially implemented</li>
</ul>
</html>"));
        end E_VCV_POLY;

        model G_VCC_POLY "Polynomial voltage controlled current source, like SPICE2"
          parameter Integer N(final min = 1) = 1 "Number of controlling voltages";
          parameter Real coeff[:] = {1} "Coefficients of polynomial";
          Modelica.Electrical.Analog.Interfaces.PositivePin p "Positive pin of the right port (potential p2.v > n2.v for positive voltage drop v2)" annotation(
            Placement(transformation(extent = {{110, 40}, {90, 60}}, rotation = 0), iconTransformation(extent = {{110, 40}, {90, 60}})));
          Modelica.Electrical.Analog.Interfaces.NegativePin n "Negative pin of the right port" annotation(
            Placement(transformation(extent = {{90, -60}, {110, -40}}, rotation = 0), iconTransformation(extent = {{90, -60}, {110, -40}})));
          Modelica.Electrical.Analog.Interfaces.PositivePin pc[2 * N] "Pin vector of controlling pins" annotation(
            Placement(transformation(extent = {{-90, -80}, {-70, 80}}, rotation = 0), iconTransformation(extent = {{-90, -80}, {-70, 80}})));
          Real control[N];
        equation
          p.i + n.i = 0;
          for i in 1:2 * N loop
            pc[i].i = 0;
          end for;
          for i in 1:N loop
            control[i] = pc[2 * i - 1].v - pc[2 * i].v;
          end for;
          p.i = poly(control, coeff);
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-70, 70}, {70, -70}}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Text(extent = {{-100, -62}, {99, -112}}, textString = "%name", lineColor = {0, 0, 255}), Line(points = {{100, 50}, {30, 50}, {30, -50}, {100, -50}}, color = {0, 0, 255}), Ellipse(extent = {{10, 20}, {50, -20}}, lineColor = {0, 0, 255}), Line(points = {{-20, 50}, {20, 50}}, color = {0, 0, 255}), Polygon(points = {{20, 50}, {10, 53}, {10, 47}, {20, 50}}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Text(extent = {{-60, -24}, {16, -58}}, lineColor = {0, 0, 255}, textString = "VCC")}),
            Documentation(info = "<html>
<p>The polynomial source is a SPICE2 model, which is also known in other SPICE derivates.</p>
<p>Nonlinear voltage controlled current source. The right port current at pin p2 (=p2.i) is controlled by the left port vector of voltages at the pin vector pc[:] via</p>
<pre>    p2.i = f(pc[1].v - pc[2].v, pc[3].v - pc[4].v,...)</pre>
<p>The controlling port (left) current vector is zero.</p>
<p>f is a polynomial in N variables s1...sN of the following form with M+1 coefficients a0, a1, a2,...aM.</p>
<pre>f = a0 +
    a1s1 + a2s2 + ... + aNsN +
    a(N+1)s1&sup2; + a(N+2)s1s2 + ... + a(.)s1sN +
    a(.)s2&sup2; + a(.)s2s3 + ... + a(.)s2sN +
    a(.)s3&sup2; + s3s4 + ... + a(.)s4sN +
    ... +
    a(.)sN&sup2; +
    a(.)s1&sup3; + a(.)s1&sup2;s2 + a(.)s1&sup2;s3 + ... + a(.)s1&sup2;sN +
    a(.)s1s2&sup2; + a(.)s1s2s3 + ... + a(.)s1s2sN +
    ... +
    a(.)sN&sup3; + ... </pre>
<p>The Coefficients a(.) are counted in this order. Reaching M, the particular sum is canceled.</p>
<p>In connection with the VCC, s1...sN are the voltages of the controlling side: s1=pc[1].v - pc[2].v, s2=pc[3].v - pc[4].v, s3=...</p>
<p>The corresponding SPICE description of the VCC polynomial source is the following:</p>
<pre>    Gname A1 A2 POLY(N) E11 E21 ... E1N E2N P0 P1...</pre>
<p>where Gname is the name of the instance, A1 and A2 are the nodes between them the current source is arranged, whose current is calculated,</p>
<p>N is the number of the controlling voltages, E11 E12 ... E1N E2N are pairs of nodes between them the controlling voltages</p>
<p>are gripped, and P0, P1... are the coefficients that are called a0, a1, ... aM in the description of the polynomial f above.</p>
<p>To describe the SPICE line in Modelica, the following explanation would be useful:</p>
<pre>Gname -&gt; G_VCC_POLY name
A1, A2 -&gt; pins name.p2, name.p1
N -&gt; parameter N
E11 -&gt; name.pc[2]
E12 -&gt; name.pc[1]
...
E1N -&gt; name.pc[N]
E2N -&gt; name.pc[N-1]
P0, P1 -&gt; polynomial coefficients name.coeff(coeff={P0,P1,...}) </pre>
</html>", revisions = "<html>
<ul>
<li><i>Sept 2008 </i>by Kristin Majetta <br/>initially implemented</li>
</ul>
</html>"));
        end G_VCC_POLY;

        model H_CCV_POLY "Polynomial current controlled voltage source, like SPICE2"
          parameter Integer N(final min = 1) = 1 "Number of controlling voltages";
          parameter Real coeff[:] = {1} "Coefficients of polynomial";
          Modelica.Electrical.Analog.Interfaces.PositivePin p "Positive pin of the right port (potential p2.v > n2.v for positive voltage drop v2)" annotation(
            Placement(transformation(extent = {{110, 40}, {90, 60}}, rotation = 0), iconTransformation(extent = {{110, 40}, {90, 60}})));
          Modelica.Electrical.Analog.Interfaces.NegativePin n "Negative pin of the right port" annotation(
            Placement(transformation(extent = {{90, -60}, {110, -40}}, rotation = 0), iconTransformation(extent = {{90, -60}, {110, -40}})));
          Modelica.Electrical.Analog.Interfaces.PositivePin pc[2 * N] "Pin vector of controlling pins" annotation(
            Placement(transformation(extent = {{-90, -80}, {-70, 80}}, rotation = 0), iconTransformation(extent = {{-90, -80}, {-70, 80}})));
          Real control[N];
        equation
          p.i + n.i = 0;
          for i in 1:N loop
            pc[2 * i - 1].i + pc[2 * i].i = 0;
            pc[2 * i - 1].v - pc[2 * i].v = 0;
          end for;
          for i in 1:N loop
            control[i] = pc[2 * i - 1].i;
          end for;
          p.v - n.v = poly(control, coeff);
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-70, 70}, {70, -70}}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Text(extent = {{-100, -62}, {99, -112}}, textString = "%name", lineColor = {0, 0, 255}), Line(points = {{100, 50}, {30, 50}, {30, -50}, {100, -50}}, color = {0, 0, 255}), Ellipse(extent = {{10, 20}, {50, -20}}, lineColor = {0, 0, 255}), Line(points = {{-20, 50}, {20, 50}}, color = {0, 0, 255}), Polygon(points = {{20, 50}, {10, 53}, {10, 47}, {20, 50}}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Text(extent = {{-60, -26}, {16, -60}}, lineColor = {0, 0, 255}, textString = "CCV")}),
            Documentation(info = "<html>
<p>The polynomial source is a SPICE2 model, which is also known in other SPICE derivates.</p>
<p>Nonlinear current controlled voltage source. The right port voltage between pin p2 and n2 (=p2.v - n2.v) is controlled by the left port vector of currents at pin pc (=pc.i) via</p>
<pre>    p2.v - n2.v = f(pc[2].i, pc[4].i,...)</pre>
<p>The controlling port (left) current vector is zero.</p>
<p>The corresponding SPICE description</p>
<pre>    Hname A1 A2 POLY(N) V1...VN P0 P1...</pre>
<p>f is a polynomial in N variables s1...sN of the following form with M+1 coefficients a0, a1, a2,...aM.</p>
<pre>f = a0 +
    a1s1 + a2s2 + ... + aNsN +
    a(N+1)s1&sup2; + a(N+2)s1s2 + ... + a(.)s1sN +
    a(.)s2&sup2; + a(.)s2s3 + ... + a(.)s2sN +
    a(.)s3&sup2; + s3s4 + ... + a(.)s4sN +
    ... +
    a(.)sN&sup2; +
    a(.)s1&sup3; + a(.)s1&sup2;s2 + a(.)s1&sup2;s3 + ... + a(.)s1&sup2;sN +
    a(.)s1s2&sup2; + a(.)s1s2s3 + ... + a(.)s1s2sN +
    ... +
    a(.)sN&sup3; + ... </pre>
<p>The Coefficients a(.) are counted in this order. Reaching M, the particular sum is canceled.</p><p>In Modelica the controlling pins have to be connected to the CCV in that way, that the required currents flow through the according pins of the CCV:</p><p>s1 = pc[2].i, s2 = pc[4].i, s3 = pc[6].i,...</p><p>The pairs pc[1].i and pc[2].i, pc[3].i and pc[4].i...form ports with pc[2].i + pc[1].i = 0, pc[4].i + pc[3].i = 0, ...</p><p>The corresponding SPICE description of the CCV polynomial source is the following:</p>
<pre>    Hname A1 A2 POLY(N) V1...VN P0 P1...</pre>
<p>where Hname is the name of the instance, A1 and A2 are the nodes between them the controlled voltage is gripped.</p><p>N is the number of the controlling currents, V1...VN are the voltage sources, that are necessary in SPICE to supply the controlling currents,</p><p>and P0, P1... are the coefficients that are called a0, a1, ... aM in the description of the polynomial f above.</p>
<p>To describe the SPICE line in Modelica, the following explanation would be useful:</p>
<pre>Hname -&gt; H_CCV_POLY name
A1, A2 -&gt; pins name.p2, name.p1
N -&gt; parameter N</pre>
<p>V1 (...VN) is declared in SPICE:</p>
<pre>   V1 V1+ V1- type of voltage source (constant, pulse, sin...)</pre>
<p>In Modelica the currents through V1...VN has to be led through the CCV. Therefore V1...VN have to be disconnected and additional nodes</p>
<pre>   V1_AD...VN_AD</pre>
<p>have to be added. In the case, that the SPICE source is</p>
<pre>   V1 n+ n- 0,</pre>
<p>this source can be eliminated.</p>
<pre>V1_AD -&gt; name.pc[2]
V1- -&gt; name.pc[1]
...
VN_AD -&gt; name.pc[N]
VN- -&gt; name.pc[N-1]
P0, P1 -&gt; polynomial coefficients name.coeff(coeff={P0,P1,...})
</pre>
</html>", revisions = "<html>
<ul>
<li><i>Sept 2008 </i>by Kristin Majetta <br/>initially implemented</li>
</ul>
</html>"));
        end H_CCV_POLY;

        model F_CCC_POLY "Polynomial current controlled current source, like SPICE2"
          parameter Integer N(final min = 1) = 1 "Number of controlling voltages";
          parameter Real coeff[:] = {1} "Coefficients of polynomial";
          Modelica.Electrical.Analog.Interfaces.PositivePin p "Positive pin of the right port (potential p2.v > n2.v for positive voltage drop v2)" annotation(
            Placement(transformation(extent = {{110, 40}, {90, 60}}, rotation = 0), iconTransformation(extent = {{110, 40}, {90, 60}})));
          Modelica.Electrical.Analog.Interfaces.NegativePin n "Negative pin of the right port" annotation(
            Placement(transformation(extent = {{90, -60}, {110, -40}}, rotation = 0), iconTransformation(extent = {{90, -60}, {110, -40}})));
          Modelica.Electrical.Analog.Interfaces.PositivePin pc[2 * N] "Pin vector of controlling pins" annotation(
            Placement(transformation(extent = {{-90, -80}, {-70, 80}}, rotation = 0), iconTransformation(extent = {{-90, -80}, {-70, 80}})));
          Real control[N];
        equation
          p.i + n.i = 0;
          for i in 1:N loop
            pc[2 * i - 1].i + pc[2 * i].i = 0;
            pc[2 * i - 1].v - pc[2 * i].v = 0;
          end for;
          for i in 1:N loop
            control[i] = pc[2 * i - 1].i;
          end for;
          p.i = poly(control, coeff);
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-70, 70}, {70, -70}}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Text(extent = {{-100, -62}, {99, -112}}, textString = "%name", lineColor = {0, 0, 255}), Line(points = {{100, 50}, {30, 50}, {30, -50}, {100, -50}}, color = {0, 0, 255}), Ellipse(extent = {{10, 20}, {50, -20}}, lineColor = {0, 0, 255}), Line(points = {{-20, 50}, {20, 50}}, color = {0, 0, 255}), Polygon(points = {{20, 50}, {10, 53}, {10, 47}, {20, 50}}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Text(extent = {{-60, -24}, {16, -58}}, lineColor = {0, 0, 255}, textString = "CCC")}),
            Documentation(info = "<html>
<p>The polynomial source is a SPICE2 model, which is also known in other SPICE derivates.</p>
<p>Nonlinear current controlled current source. The &quot;right&quot; port current at pin p2 (=p2.i) is controlled by the &quot;left&quot; port vector of currents at pin pc[:] via</p>
<pre>    p2.i = f(pc[2].i, pc[4].i,...)</pre>
<p>The controlling port (left) voltage is zero for each pair: pc[2].v - pc[1].v = 0, ...</p><p>Furthermore the currents of each pair are pc[2].i + pc[1].i = 0, ...</p>
<p>f is a polynomial in N variables s1...sN of the following form with M+1 coefficients a0, a1, a2,...aM.</p>
<pre>f = a0 +
    a1s1 + a2s2 + ... + aNsN +
    a(N+1)s1&sup2; + a(N+2)s1s2 + ... + a(.)s1sN +
    a(.)s2&sup2; + a(.)s2s3 + ... + a(.)s2sN +
    a(.)s3&sup2; + s3s4 + ... + a(.)s4sN +
    ... +
    a(.)sN&sup2; +
    a(.)s1&sup3; + a(.)s1&sup2;s2 + a(.)s1&sup2;s3 + ... + a(.)s1&sup2;sN +
    a(.)s1s2&sup2; + a(.)s1s2s3 + ... + a(.)s1s2sN +
    ... +
    a(.)sN&sup3; + ... </pre>
<p>The Coefficients a(.) are counted in this order. Reaching M, the particular sum is canceled.</p><p>In Modelica the controlling pins have to be connected to the CCC in that way, that the required currents flow through the according pins of the CCC:</p><p>s1=pc[2].i, s2=pc[4].i, s3=pc[6].i,...</p><p>The pairs pc[1].i and pc[2].i, pc[3].i and pc[4].i...form ports with pc[2].i + pc[1].i = 0, pc[4].i + pc[3].i = 0, ...</p><p>The corresponding SPICE description of the CCC polynomial source is the following:</p>
<pre>    Fname A1 A2 POLY(N) V1...VN P0 P1...</pre>
<p>where Fname is the name of the instance, A1 and A2 are the nodes between them the current source is arranged, whose current is calculated.</p><p>N is the number of the controlling currents, V1...VN are the voltage sources, that are necessary in SPICE to supply the controlling currents,</p><p>and P0, P1... are the coefficients that are called a0, a1, ... aM in the description of the polynomial f above.</p>
<p>To describe the SPICE line in Modelica, the following explanation would be useful:</p>
<pre>Fname -&gt; F_CCC_POLY name
A1, A2 -&gt; pins name.p2, name.p1
N -&gt; parameter N</pre>
<p>V1 (...VN) is declared in SPICE:</p>
<pre>   V1 V1+ V1- type of voltage source (constant, pulse, sin...)</pre>
<p>In Modelica the currents through V1...VN has to be led through the CCC. Therefore V1...VN have to be disconnected and additional nodes</p>
<pre>   V1_AD...VN_AD</pre>
<p>have to be added. In the case, that the SPICE source is</p>
<pre>   V1 n+ n- 0,</pre>
<p>this source can be eliminated.</p>
<pre>V1_AD -&gt; name.pc[2]
V1- -&gt; name.pc[1]
...
VN_AD -&gt; name.pc[N]
VN- -&gt; name.pc[N-1]
P0, P1 -&gt; polynomial coefficients name.coeff(coeff={P0,P1,...})
</pre>
</html>", revisions = "<html>
<ul>
<li><i>Sept 2008 </i>by Kristin Majetta <br/>initially implemented</li>
</ul>
</html>"));
        end F_CCC_POLY;
        annotation(
          Documentation(info = "<html>
<p>This package contains additional useful models which do not belong to the original SPICE3 model set.</p>
</html>"));
      end Additionals;

      package Interfaces "Connectors, Interfaces, and partial models"
        extends Modelica.Icons.InterfacesPackage;

        partial model TwoPortControlledSources "Component with two electrical ports, including current"
          SI.Voltage v1 "Voltage drop over the controlling port";
          SI.Voltage v2 "Voltage drop over the controlled port";
          SI.Current i1 "Current flowing from pos. to neg. pin of the controlling port";
          SI.Current i2 "Current flowing from pos. to neg. pin of the controlled port";
          Modelica.Electrical.Analog.Interfaces.PositivePin p1 "Positive pin of the controlling port" annotation(
            Placement(transformation(extent = {{-110, 40}, {-90, 60}}, rotation = 0)));
          Modelica.Electrical.Analog.Interfaces.NegativePin n1 "Negative pin of the controlling port" annotation(
            Placement(transformation(extent = {{-90, -60}, {-110, -40}}, rotation = 0)));
          Modelica.Electrical.Analog.Interfaces.PositivePin p2 "Positive pin of the controlled port" annotation(
            Placement(transformation(extent = {{110, 40}, {90, 60}}, rotation = 0)));
          Modelica.Electrical.Analog.Interfaces.NegativePin n2 "Negative pin of the controlled port" annotation(
            Placement(transformation(extent = {{90, -60}, {110, -40}}, rotation = 0)));
        equation
          v1 = p1.v - n1.v;
          v2 = p2.v - n2.v;
          0 = p1.i + n1.i;
          0 = p2.i + n2.i;
          i1 = p1.i;
          i2 = p2.i;
          annotation(
            Documentation(info = "<html>
<p>TwoPort is a partial model that consists of two ports. It is assumed that the current flowing into the positive pin is identical to the current flowing out of pin n. This currents of each port are provided explicitly as currents i1 and i2, the voltages respectively as v1 and v2.</p>
</html>"));
        end TwoPortControlledSources;

        connector InductiveCouplePinIn "Pin to couple inductances via K, which gets the value of inductance"
          input Modelica.SIunits.Inductance L;
          SI.CurrentSlope di "di/dt";
          flow SI.Voltage v;
          annotation(
            Icon(graphics = {Polygon(points = {{0, 0}, {0, 100}, {100, 0}, {0, -100}, {0, -100}, {0, 0}}, lineColor = {170, 85, 255}, smooth = Smooth.None, fillColor = {170, 85, 255}, fillPattern = FillPattern.Solid)}));
        end InductiveCouplePinIn;

        connector InductiveCouplePinOut "Pin to couple inductances via K, which sets the value of inductance"
          output Modelica.SIunits.Inductance L;
          SI.CurrentSlope di "di/dt";
          flow SI.Voltage v;
          annotation(
            Icon(graphics = {Polygon(points = {{-100, 0}, {0, 100}, {0, 0}, {0, -100}, {-2, -98}, {-100, 0}}, lineColor = {170, 85, 255}, smooth = Smooth.None, fillColor = {170, 85, 255}, fillPattern = FillPattern.Solid)}));
        end InductiveCouplePinOut;

        partial model ConditionalSubstrate "Partial model to include a conditional substrate node"
          parameter Boolean useSubstrateNode = false "=true, if SubstrateNode is enabled" annotation(
            Evaluate = true,
            HideResult = true,
            choices(checkBox = true));
          Modelica.Electrical.Analog.Interfaces.PositivePin S(v = substrateVoltage, i = -substrateCurrent) if useSubstrateNode annotation(
            Placement(transformation(extent = {{90, 0}, {110, 20}}), iconTransformation(extent = {{90, -10}, {110, 10}})));
          Modelica.SIunits.Voltage substrateVoltage;
          Modelica.SIunits.Current substrateCurrent;
        equation
          if not useSubstrateNode then
            substrateVoltage = 0;
          end if;
          annotation(
            Documentation(revisions = "<html>
<ul>
<li><i> February 17, 2009   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>", info = "<html>
<p>
This partial model provides a conditional heating port for the connection to a thermal network.
</p>
<ul>
<li> If <b>useHeatPort</b> is set to <b>false</b> (default), no heat port is available, and the thermal
     loss power flows internally to the ground. In this case, the parameter <b>T</b> specifies
     the fixed device temperature (the default for T = 20<sup>o</sup>C).</li>
<li> If <b>useHeatPort</b> is set to <b>true</b>, a heat port is available.</li>
</ul>

<p>
If this model is used, the loss power has to be provided by an equation in the model which inherits from
ConditionalHeatingPort model (<b>lossPower = ...</b>). As device temperature
<b>T_heatPort</b> can be used to describe the influence of the device temperature
on the model behaviour.
</p>
</html>"));
        end ConditionalSubstrate;
        annotation(
          preferredView = "info",
          Documentation(info = "<html>
<p>The SPICE3 package uses the Modelica.Electrical.Analog interfaces. Only special partial models used in the SPICE3 package are located in this Interfaces package.</p>
</html>"));
      end Interfaces;

      package Types "Additional Spice3 type definitions"
        extends Modelica.Icons.TypesPackage;
        type VoltageSquare = Real(final quantity = "ElectricalPotential2", final unit = "V2");
        type GapEnergyPerTemperature = Real(final quantity = "Energy per Temperature", final unit = "eV/K");
        type GapEnergyPerEnergy = Real(final quantity = "Energy per Energy", final unit = "eV/J");
        type PerVolume = Real(final quantity = "PerVolume", final unit = "1/m3");
        type InverseElectricCurrent = Real(final quantity = "InverseElectricCurrent", final unit = "1/A") "Inverse of electric current";
        type ElectricFieldStrength_cm = Real(final quantity = "ElectricFieldStrength", final unit = "V/cm");
        annotation(
          Documentation(info = "<html>
<p>This package Types contains units that are needed in the models of the Spice3 package.</p>
</html>"));
      end Types;

      package Internal "Collection of functions and records derived from the C++ Spice library"
        extends Modelica.Icons.InternalPackage;

        model MOS "Metal-Oxide Semiconductor Field-Effect Transistor"
          Modelica.Electrical.Analog.Interfaces.PositivePin G "gate node" annotation(
            Placement(transformation(extent = {{-110, -12}, {-90, 10}}, rotation = 0)));
          Modelica.Electrical.Analog.Interfaces.PositivePin D "drain node" annotation(
            Placement(transformation(extent = {{-10, 90}, {10, 110}}, rotation = 0)));
          Modelica.Electrical.Analog.Interfaces.NegativePin S "source node" annotation(
            Placement(transformation(extent = {{-10, -110}, {10, -90}}, rotation = 0)));
          Modelica.Electrical.Analog.Interfaces.PositivePin B "bulk node" annotation(
            Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
          parameter Integer mtype(start = 0) "MOSFET type: 0 - N channel, 1 - P channel";
          parameter SI.Length L = 1e-4 "Length";
          parameter SI.Length W = 1e-4 "Width";
          parameter SI.Area AD = 0 "Area of the drain diffusion";
          parameter SI.Area AS = 0 "Area of the source diffusion";
          parameter SI.Length PD = 0 "Perimeter of the drain junction";
          parameter SI.Length PS = 0 "Perimeter of the source junction";
          parameter Real NRD = 1 "Number of squares of the drain diffusions";
          parameter Real NRS = 1 "Number of squares of the source diffusions";
          parameter Integer OFF = 0 "Optional initial condition: 0 - IC not used, 1 - IC used, not implemented yet";
          parameter SI.Voltage IC(start = -1e40) "Initial condition values, not implemented yet";
          parameter SI.Temp_C TEMP = 27 "Operating temperature of the device";
          parameter ModelcardMOS modelcard "MOSFET modelcard" annotation(
            Evaluate = true);
          constant SpiceConstants C "General constants of SPICE simulator";
          final parameter Mos1.Mos1ModelLineParams p = Mos1.mos1RenameParameters(modelcard, C) "Model line parameters" annotation(
            Evaluate = true);
          final parameter Mosfet.Mosfet m = Mos1.mos1RenameParametersDev(modelcard, mtype, W, L, AS, AS, PD, PS, NRD, NRS, OFF, IC, TEMP) "Renamed parameters" annotation(
            Evaluate = true);
          final parameter Integer m_type = if m.m_bPMOS > 0.5 then -1 else 1 "Type of the transistor";
          final parameter Mos.MosModelLineVariables vp = Mos1.mos1ModelLineParamsInitEquations(p, C, m_type) "Model line variables";
          final parameter Mos1.Mos1Calc c1 = Mos.mosCalcInitEquations(p, C, vp, m) "Precalculated parameters";
          final parameter Mos1.Mos1Calc c2 = Mos.mosCalcCalcTempDependencies(p, C, vp, m, c1, m_type) "Precalculated parameters";
          Mos.CurrrentsCapacitances cc;
          constant Boolean m_bInit = false;
          SI.Voltage Dinternal;
          //internal drain node
          SI.Voltage Sinternal;
          //internal source node
          SI.Current ird;
          SI.Current irs;
          SI.Current ibdgmin;
          SI.Current ibsgmin;
          SI.Current icBD;
          SI.Current icBS;
          SI.Current icGB;
          SI.Current icGS;
          SI.Current icGD;
          SI.Voltage vDS "Drain - source voltage";
          SI.Voltage vGS "Gate - source voltage";
        equation
          assert(NRD > 0, "NRD, length of drain in squares, must be greater than zero");
          assert(NRS > 0, "NRS, length of source in squares, must be greater than zero");
          vDS = D.v - S.v;
          vGS = G.v - S.v;
          cc = Mos.mosCalcNoBypassCode(m, m_type, c2, p, C, vp, m_bInit, {G.v, B.v, Dinternal, Sinternal});
// drain- and sourceresistances
// ----------------------------
          ird * c1.m_drainResistance = D.v - Dinternal;
          irs * p.m_sourceResistance = S.v - Sinternal;
// capacitances
// ------------
          icBD = cc.cBD * (der(B.v) - der(Dinternal));
          icBS = cc.cBS * (der(B.v) - der(Sinternal));
          icGB = cc.cGB * (der(G.v) - der(B.v));
          icGD = cc.cGD * (der(G.v) - der(Dinternal));
          icGS = cc.cGS * (der(G.v) - der(Sinternal));
// currents
// --------
          ibsgmin = SpiceConstants.CKTgmin * (B.v - Sinternal);
          ibdgmin = SpiceConstants.CKTgmin * (B.v - Dinternal);
          G.i = icGB + icGD + icGS;
          B.i = cc.iBD + cc.iBS + ibdgmin + ibsgmin - icGB + icBD + icBS;
          D.i = ird;
          S.i = irs;
//currentsum at inner node
//------------------------
          0 = (-ird) + cc.idrain - cc.iBD - ibdgmin - icGD - icBD;
          0 = (-irs) - cc.idrain - cc.iBS - ibsgmin - icGS - icBS;
          annotation(
            Documentation(info = "<html>
<p>MOSFET model, both N and P channel, LEVEL 1: Shichman-Hodges</p>
<p>The package Repository is not for user access. There all function, records and data are stored, that are needed for the semiconductor models of the package Semiconductors.</p>
</html>", revisions = "<html>
<ul>
<li><i>March 2008 </i>by Kristin Majetta <br/>initially implemented</li>
</ul>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{0, 92}, {0, 40}, {-12, 40}, {-12, -40}, {0, -40}, {0, -94}}, color = {0, 0, 255}), Line(points = {{-92, 0}, {-20, 0}}, color = {0, 0, 255}), Line(points = {{-12, 0}, {94, 0}}, color = {0, 0, 255}), Line(points = {{-20, 40}, {-20, -40}}, color = {0, 0, 255}), Text(extent = {{8, -34}, {92, -86}}, lineColor = {0, 0, 255}, textString = "%name")}));
        end MOS;

        record ModelcardMOS "Record with technological parameters (.model)"
          extends Modelica.Icons.Record;
          parameter SI.Voltage VTO = -1e40 "Zero-bias threshold voltage, default 0";
          parameter SI.Transconductance KP = -1e40 "Transconductance parameter, default 2e-5";
          parameter Real GAMMA = -1e40 "Bulk threshold parameter, default 0";
          parameter SI.Voltage PHI = -1e40 "Surface potential, default 0.6";
          //Substrate Sperrschicht Potential
          parameter SI.InversePotential LAMBDA = 0 "Channel-length modulation, default 0";
          parameter SI.Resistance RD = -1e40 "Drain ohmic resistance, default 0";
          parameter SI.Resistance RS = -1e40 "Source ohmic resistance, default 0";
          parameter SI.Capacitance CBD = -1e40 "Zero-bias B-D junction capacitance, default 0";
          parameter SI.Capacitance CBS = -1e40 "Zero-bias B-S junction capacitance, default 0";
          parameter SI.Current IS = 1.e-14 "Bulk junction saturation current";
          parameter SI.Voltage PB = 0.8 "Bulk junction potential";
          parameter SI.Permittivity CGSO = 0.0 "Gate-source overlap capacitance per meter channel width";
          parameter SI.Permittivity CGDO = 0.0 "Gate-drain overlap capacitance per meter channel width";
          parameter SI.Permittivity CGBO = 0.0 "Gate-bulk overlap capacitance per meter channel width";
          parameter SI.Resistance RSH = 0.0 "Drain and source diffusion sheet resistance";
          parameter SI.CapacitancePerArea CJ = 0.0 "Zero-bias bulk junction bottom cap. per sq-meter of junction area";
          parameter Real MJ = 0.5 "Bulk junction bottom grading coefficient";
          parameter SI.Permittivity CJSW = 0.0 "Zero-bias junction sidewall cap. per meter of junction perimeter";
          parameter Real MJSW = 0.5 "Bulk junction sidewall grading coefficient";
          parameter SI.CurrentDensity JS = 0.0 "Bulk junction saturation current per sq-meter of junction area";
          parameter SI.Length TOX = -1e40 "Oxide thickness, default 1e-7";
          parameter Real NSUB = -1e40 "Substrate doping, default 0";
          parameter SI.Conversions.NonSIunits.PerArea_cm NSS = 0.0 "Surface state density";
          parameter Real TPG = 1.0 "Type of gate material: +1 opp. to substrate, -1 same as substrate, 0 Al gate";
          parameter SI.Length LD = 0.0 "Lateral diffusion";
          parameter SI.Conversions.NonSIunits.Area_cmPerVoltageSecond UO = 600 "Surface mobility";
          parameter Real KF = 0 "Flicker noise coefficient";
          parameter Real AF = 1.0 "Flicker noise exponent";
          parameter Real FC = 0.5 "Coefficient for forward-bias depletion capacitance formula";
          parameter SI.Temp_C TNOM = 27 "Parameter measurement temperature, default 27";
          constant Integer LEVEL = 1 "Model level: Shichman-Hodges";
          annotation(
            Documentation(info = "<html>
<p>Modelcard parameters for MOSFET model, both N and P channel, LEVEL 1: Shichman-Hodges</p>
<p>The package Repository is not for user access. There all function, records and data are stored, that are needed for the semiconductor models of the package Semiconductors.</p>
</html>"));
        end ModelcardMOS;

        model MOS2 "Metal-Oxide Semiconductor Field-Effect Transistor"
          Modelica.Electrical.Analog.Interfaces.PositivePin G "gate node" annotation(
            Placement(transformation(extent = {{-110, -12}, {-90, 10}}, rotation = 0)));
          Modelica.Electrical.Analog.Interfaces.PositivePin D "drain node" annotation(
            Placement(transformation(extent = {{-10, 90}, {10, 110}}, rotation = 0)));
          Modelica.Electrical.Analog.Interfaces.NegativePin S "source node" annotation(
            Placement(transformation(extent = {{-10, -110}, {10, -90}}, rotation = 0)));
          Modelica.Electrical.Analog.Interfaces.PositivePin B "bulk node" annotation(
            Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
          parameter Integer mtype(start = 0) "MOSFET type: 0 - N channel, 1 - P channel";
          parameter SI.Length L = 1e-4 "Length";
          parameter SI.Length W = 1e-4 "Width";
          parameter SI.Area AD = 0 "Area of the drain diffusion";
          parameter SI.Area AS = 0 "Area of the source diffusion";
          parameter SI.Length PD = 0 "Perimeter of the drain junction";
          parameter SI.Length PS = 0 "Perimeter of the source junction";
          parameter Real NRD = 1 "Number of squares of the drain diffusions";
          parameter Real NRS = 1 "Number of squares of the source diffusions";
          //--------------------------------------------------------------------------------------
          //need to be Boolean instead of integer, will be changed with the next Modelica version
          //via converting script
          parameter Integer OFF = 0 "Optional initial condition: 0 - IC not used, 1 - IC used, not implemented yet";
          //--------------------------------------------------------------------------------------
          parameter SI.Voltage IC_VDS = -1e40 "Initial condition value (VDS, not implemented yet)";
          parameter SI.Voltage IC_VGS = -1e40 "Initial condition value (VGS, not implemented yet)";
          parameter SI.Voltage IC_VBS = -1e40 "Initial condition value (VBS, not implemented yet)";
          parameter Boolean UIC = false "Use initial conditions: true, if initial condition is used";
          parameter SI.Temp_C TEMP = 27 "Operating temperature of the device";
          parameter Spice3.Internal.ModelcardMOS2 modelcard "MOSFET modelcard" annotation(
            Evaluate = true);
          final parameter Spice3.Internal.Mos2.Mos2ModelLineParams p = Spice3.Internal.Mos2.mos2RenameParametersRevised(modelcard) "Model line parameters" annotation(
            Evaluate = true);
          final parameter Spice3.Internal.Mosfet.Mosfet m = Spice3.Internal.Mosfet.mosfetRenameParametersDev(W, L, AD, AS, PD, PS, NRD, NRS, OFF, IC_VDS, IC_VGS, IC_VBS, UIC, TEMP) "Renamed parameters" annotation(
            Evaluate = true);
          final parameter Spice3.Internal.Mosfet.Mosfet m1 = Spice3.Internal.Mosfet.mosfetInitEquations(m);
          final parameter Integer m_type = if m.m_bPMOS > 0.5 then -1 else 1 "Type of the transistor";
          final parameter Spice3.Internal.Mos2.Mos2ModelLineParams p1 = Spice3.Internal.Mos2.mos2ModelLineParamsInitEquationsRevised(p, m_type) "Model line variables";
          final parameter Spice3.Internal.Mos2.Mos2Calc c11 = Spice3.Internal.Mos.mos2CalcInitEquationsRevised(p1, m1) "Precalculated parameters";
          final parameter Spice3.Internal.Mos2.Mos2Calc c22 = Spice3.Internal.Mos.mos2CalcCalcTempDependenciesRevised(p1, m1, c11, m_type) "Precalculated parameters";
          Spice3.Internal.Mos.CurrrentsCapacitances cc;
          constant Boolean m_bInit = false;
          SI.Voltage Dinternal;
          SI.Voltage Sinternal;
          SI.Voltage vBD;
          SI.Voltage vBS;
          SI.Voltage vGB;
          SI.Voltage vGD;
          SI.Voltage vGS;
          SI.Current ird;
          SI.Current irs;
          SI.Current ibdgmin;
          SI.Current ibsgmin;
          SI.Current icBD;
          SI.Current icBS;
          SI.Current icGB;
          SI.Current icGS;
          SI.Current icGD;
          //-------------------------------obsolete-----------------------------------------------------------------------------------------------
          parameter SI.Voltage IC(start = -1e40) "Initial condition values, not implemented yet";
          Real icqmGB;
          Real icqmGS;
          Real icqmGD;
          SI.Voltage vDS "Drain - source voltage";
          Spice3.Internal.Mos.DEVqmeyer qm;
          Spice3.Internal.Mos.CurrrentsCapacitances cc_obsolete;
          final parameter Spice3.Internal.Mos2.Mos2ModelLineVariables vp = Spice3.Internal.Mos2.mos2ModelLineParamsInitEquations(p, C, m_type) "Model line variables";
          final parameter Spice3.Internal.Mos2.Mos2Calc c1 = Spice3.Internal.Mos.mos2CalcInitEquations(p, C, vp, m) "Precalculated parameters";
          final parameter Spice3.Internal.Mos2.Mos2Calc c2 = Spice3.Internal.Mos.mos2CalcCalcTempDependencies(p, C, vp, m, c1, m_type) "Precalculated parameters";
          final parameter Spice3.Internal.Mos2.Mos2ModelLineParams p_obsolete = Spice3.Internal.Mos2.mos2RenameParameters(modelcard, C) "Model line parameters" annotation(
            Evaluate = true);
          constant Spice3.Internal.SpiceConstants C "General constants of SPICE simulator";
          Real MOScapgd = qm.qm_capgd;
          Real MOScapgs = qm.qm_capgs;
          Real MOScapgb = qm.qm_capgb;
        equation
          assert(NRD > 0, "NRD, length of drain in squares, must be greater than zero");
          assert(NRS > 0, "NRS, length of source in squares, must be greater than zero");
          cc = Spice3.Internal.Mos.mos2CalcNoBypassCodeRevised(m1, m_type, c22, p1, m_bInit, {G.v, B.v, Dinternal, Sinternal});
// voltages
// --------
          vBD = B.v - Dinternal;
          vBS = B.v - Sinternal;
          vGB = G.v - B.v;
          vGD = G.v - Dinternal;
          vGS = G.v - Sinternal;
// drain- and sourceresistances
// ----------------------------
          ird * c11.m_drainResistance = D.v - Dinternal;
          irs * c11.m_sourceResistance = S.v - Sinternal;
// capacitances
// ------------
          icBD = cc.cBD * der(vBD);
          icBS = cc.cBS * der(vBS);
          icGB = cc.cGB * der(vGB);
          icGD = cc.cGD * der(vGD);
          icGS = cc.cGS * der(vGS);
// currents
// --------
          ibsgmin = Spice3.Internal.SpiceConstants.CKTgmin * (B.v - Sinternal);
          ibdgmin = Spice3.Internal.SpiceConstants.CKTgmin * (B.v - Dinternal);
          G.i = icGB + icGD + icGS;
          B.i = cc.iBD + cc.iBS + ibdgmin + ibsgmin - icGB + icBD + icBS;
          D.i = ird;
          S.i = irs;
//currentsum at inner node
//------------------------
          0 = (-ird) + cc.idrain - cc.iBD - ibdgmin - icGD - icBD;
          0 = (-irs) - cc.idrain - cc.iBS - ibsgmin - icGS - icBS;
//----------------------obsolete--------------------------------------------------
          vDS = D.v - S.v;
          (cc_obsolete, qm) = Spice3.Internal.Mos.mos2CalcNoBypassCode(m, m_type, c2, p, C, vp, m_bInit, {G.v, B.v, Dinternal, Sinternal});
          icqmGB = 0;
          icqmGS = 0;
          icqmGD = 0;
          annotation(
            Documentation(info = "<html>
<p>MOSFET model, both N and P channel, LEVEL 2</p>
<p>The package Repository is not for user access. There all function, records and data are stored, that are needed for the semiconductor models of the package Semiconductors.</p>
</html>", revisions = "<html>
<ul>
<li><i>January 2009 </i>by Kristin Majetta <br/>initially implemented</li>
</ul>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{0, 92}, {0, 40}, {-12, 40}, {-12, -40}, {0, -40}, {0, -94}}, color = {0, 0, 255}), Line(points = {{-92, 0}, {-20, 0}}, color = {0, 0, 255}), Line(points = {{-12, 0}, {94, 0}}, color = {0, 0, 255}), Line(points = {{-20, 40}, {-20, -40}}, color = {0, 0, 255}), Text(extent = {{8, -34}, {92, -86}}, lineColor = {0, 0, 255}, textString = "%name")}));
        end MOS2;

        record ModelcardMOS2 "Record with technological parameters (.model)"
          extends Modelica.Icons.Record;
          extends Spice3.Internal.ModelcardMOS(MJSW = 0.33);
          parameter SI.Conversions.NonSIunits.PerArea_cm NFS = 0.0 "Fast surface state density";
          parameter SI.Length XJ = 0.0 "Metallurgical junction depth";
          parameter Types.ElectricFieldStrength_cm UCRIT = 1.e4 "Critical field for mobility degradation (MOS2 only)";
          parameter Real UEXP = 0.0 "Critical field exponent in mobility degradation (MOS2 only)";
          parameter SI.Velocity VMAX = 0.0 "Maximum drift velocity of carries";
          parameter Real NEFF = 1.0 "Total channel charge (fixed and mobile) coefficient (MOS2 only)";
          parameter Real DELTA = 0.0 "Width effect on threshold voltage";
          annotation(
            Documentation(info = "<html>
<p>Modelcard parameters for MOSFET model, both N and P channel, LEVEL 2</p>
<p>The package Repository is not for user access. There all function, records and data are stored, that are needed for the semiconductor models of the package Semiconductors.</p>
</html>"));
        end ModelcardMOS2;

        model BJT2 "Bipolar junction transistor"
          Modelica.Electrical.Analog.Interfaces.PositivePin B "Base node" annotation(
            Placement(transformation(extent = {{-108, -10}, {-88, 10}}), iconTransformation(extent = {{-106, -10}, {-86, 10}})));
          Modelica.Electrical.Analog.Interfaces.PositivePin C "Collector node" annotation(
            Placement(transformation(extent = {{10, 88}, {30, 108}}), iconTransformation(extent = {{20, 90}, {40, 110}})));
          Modelica.Electrical.Analog.Interfaces.NegativePin E "Emitter node" annotation(
            Placement(transformation(extent = {{10, -108}, {30, -88}}), iconTransformation(extent = {{20, -110}, {40, -90}})));
          extends Spice3.Interfaces.ConditionalSubstrate;
          parameter Real TBJT(start = 1) "Type of transistor (NPN=1, PNP=-1)";
          parameter Real AREA = 1.0 "Area factor";
          parameter Boolean OFF = false "Optional initial condition: false - IC not used, true - IC used, not implemented yet";
          parameter SI.Voltage IC_VBE = -1e40 "Initial condition value (VBC, not implemented yet)";
          parameter SI.Voltage IC_VCE = -1e40 "Initial condition value (VBE, not implemented yet)";
          parameter Boolean UIC = false "Use initial conditions: true, if initial condition is used";
          parameter SI.Temp_C TEMP = 27 "Operating temperature of the device";
          parameter Boolean SENS_AREA = false "Flag to request sensitivity WRT area, not implemented yet";
          parameter Modelica.Electrical.Spice3.Internal.ModelcardBJT2 modelcard "BJT modelcard" annotation(
            Evaluate = true);
          final parameter Spice3.Internal.Bjt.BjtModelLineParams p = Spice3.Internal.Bjt.bjtRenameParameters(modelcard, TBJT) "Model line parameters" annotation(
            Evaluate = true);
          final parameter Spice3.Internal.Bjt.Bjt dev = Spice3.Internal.Bjt.bjtRenameParametersDev(AREA, OFF, IC_VBE, IC_VCE, UIC, SENS_AREA, TEMP) "Renamed parameters" annotation(
            Evaluate = true);
          final parameter Spice3.Internal.Bjt.BjtModelLineParams p1 = Spice3.Internal.Bjt.bjtModelLineInitEquations(p) "Model line variables";
          final parameter Spice3.Internal.Bjt.Bjt dev1 = Spice3.Internal.Bjt.bjtInitEquations(dev, p1) "Precalculated parameters";
          final parameter Spice3.Internal.Bjt.BjtCalc c = Spice3.Internal.Bjt.bjtCalcTempDependencies(dev1, p1) "Precalculated parameters";
          constant Boolean m_bInit = false;
          Spice3.Internal.Bjt.CurrentsCapacitances cc;
          SI.Voltage Cinternal;
          // inner collector node
          SI.Voltage Binternal;
          // inner base node
          SI.Voltage Einternal;
          // inner emitter node
          SI.Voltage vbe;
          SI.Voltage vbc;
          SI.Voltage vbx;
          SI.Voltage vcs;
          SI.Current irc;
          SI.Current ire;
          SI.Current irb;
          SI.Current ibcgmin;
          SI.Current ibegmin;
          SI.Current icapbe;
          SI.Current icapbc;
          SI.Current icapbx;
          SI.Current icapcs;
          SI.Current icaptt;
        equation
          cc = Spice3.Internal.Bjt.bjtNoBypassCode(dev1, p1, c, {C.v, B.v, E.v, Cinternal, Binternal, Einternal});
// voltages
          vbe = Binternal - Einternal;
          vbc = Binternal - Cinternal;
          vbx = B.v - Cinternal;
          vcs = Cinternal - substrateVoltage;
// currents through capacitances
          icapbe = if m_bInit then 0.0 else cc.capbe * der(vbe);
          icapbc = if m_bInit then 0.0 else cc.capbc * der(vbc);
          icaptt = if m_bInit then 0.0 else cc.captt * der(vbc);
          icapbx = if m_bInit then 0.0 else cc.capbx * der(vbx);
          icapcs = if m_bInit then 0.0 else cc.capcs * der(vcs);
// resistances
          irc * p1.m_collectorResist = C.v - Cinternal;
          ire * p1.m_emitterResist = E.v - Einternal;
//  irb * cc.rx = (B.v - Binternal);  // exact resistance
          irb * p.m_baseResist = B.v - Binternal;
// faulty resistance
// currents
          ibcgmin = Spice3.Internal.SpiceConstants.CKTgmin * (Binternal - Cinternal);
          ibegmin = Spice3.Internal.SpiceConstants.CKTgmin * (Binternal - Einternal);
          C.i = irc;
          E.i = ire;
          B.i = irb + icapbx;
//current sum at inner nodes
          0 = ibcgmin + irc - cc.iCC + cc.iBCN + cc.iBC + icapbc + icapbx - icapcs + icaptt;
//current sum for inner node Cinternal
          0 = ibegmin + ire + cc.iCC + cc.iBEN + cc.iBE + icapbe;
//current sum for inner node Einternal
          0 = (-ibcgmin) - ibegmin + irb - cc.iBC - cc.iBE - cc.iBCN - cc.iBEN - icapbc - icapbe - icaptt;
//current sum for inner node Binternal
          substrateCurrent = icapcs;
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-20, 60}, {-20, -60}}, color = {0, 0, 255}, smooth = Smooth.None), Line(points = {{-20, 0}, {-86, 0}}, color = {0, 0, 255}, smooth = Smooth.None), Line(points = {{34, 94}, {-20, 40}}, color = {0, 0, 255}, smooth = Smooth.None), Line(points = {{-20, -40}, {32, -92}}, color = {0, 0, 255}, smooth = Smooth.None), Text(extent = {{-94, 56}, {206, 16}}, textString = "%name", lineColor = {0, 0, 255})}),
            Documentation(info = "<html>
<p>Bipolar junction transistor model, both NPN and PNP</p>
<p>The package Internal is not for user access. There all function, records and data are stored, that are needed for the semiconductor models of the package Semiconductors.</p>
</html>", revisions = "<html>
<ul>
<li><i>August 2009 </i>by Kristin Majetta <br/>initially implemented</li>
</ul>
</html>"));
        end BJT2;

        record ModelcardBJT2 "Record with technological parameters (.model)"
          extends Modelica.Icons.Record;
          parameter SI.Temp_C TNOM = 27 "Parameter measurement temperature";
          parameter SI.Current IS = 1e-16 "Transport saturation current";
          parameter Real BF = 100.00 "Ideal maximum forward beta F";
          parameter Real NF = 1.0 "Forward current emission coefficientF";
          parameter Real NE = 1.5 "B-E leakage emission coefficient";
          parameter SI.Current ISE = -1e40 "B-E leakage saturation current, default = 0";
          constant Real C2 = -1e40 "Obsolete parameter name, default = 0";
          parameter SI.Current ISC = -1e40 "B-C leakage saturation current, default = 0";
          constant Real C4 = -1e40 "Obsolete parameter name, default = 0";
          parameter Real BR = 1.0 "Ideal maximum reverse beta";
          parameter Real NR = 1.0 "Reverse current emission coefficient";
          parameter Real NC = 2.0 "B-C leakage emission coefficient";
          parameter SI.Voltage VAF = 0.0 "Forward Early voltage";
          parameter SI.Current IKF = 0.0 "Forward beta roll-off corner current";
          parameter SI.Voltage VAR = 0.0 "Reverse Early voltage";
          parameter SI.Current IKR = 0.0 "Reverse beta roll-off corner current";
          parameter SI.Resistance RE = 0.0 "Emitter resistance";
          parameter SI.Resistance RC = 0.0 "Collector resistance";
          parameter SI.Current IRB = 0.0 "Current for base resistance = (rb+rbm)/2";
          parameter SI.Resistance RB = 0.0 "Zero bias base resistance";
          parameter SI.Resistance RBM = 0.0 "Minimum base resistance, default = 0.0";
          parameter SI.Capacitance CJE = 0.0 "Zero bias B-E depletion capacitance";
          parameter SI.Voltage VJE = 0.75 "B-E built in potential";
          parameter Real MJE = 0.33 "B-E junction exponential factor";
          parameter SI.Time TF = 0.0 "Ideal forward transit time";
          parameter Real XTF = 0.0 "Coefficient for bias dependence of TF";
          parameter SI.Current ITF = 0.0 "High current dependence of TF,";
          parameter SI.Voltage VTF = 0.0 "Voltage giving VBC dependence of TF";
          parameter SI.Frequency PTF = 0.0 "Excess phase at freq=1/(TF*2*Pi) Hz";
          parameter SI.Capacitance CJC = 0.0 "Zero bias B-C depletion capacitance";
          parameter SI.Voltage VJC = 0.75 "B-C built in potential";
          parameter Real MJC = 0.33 "B-C junction grading coefficient";
          parameter Real XCJC = 1.0 "Fraction of B-C cap to internal base";
          parameter SI.Time TR = 0.0 "Ideal reverse transit time";
          parameter SI.Capacitance CJS = 0.0 "Zero bias C-S capacitance";
          parameter SI.Voltage VJS = 0.75 "Substrate junction built-in potential";
          parameter Real MJS = 0.0 "Substrate junction grading coefficient";
          parameter Real XTB = 0.0 "Forward and reverse beta temperature exponent";
          parameter SI.GapEnergy EG = 1.11 "Energy gap for IS temperature effect on IS";
          parameter Real XTI = 3.0 "Temperature exponent for IS";
          parameter Real KF = 0.0 "Flicker Noise Coefficient";
          parameter Real AF = 1.0 "Flicker Noise Exponent";
          parameter Real FC = 0.5 "Forward bias junction fit parameter";
          annotation(
            Documentation(info = "<html>
<p>Modelcard parameters for BJT model, both PNP and NPN</p>
<p>The package Internal is not for user access. There all function, records and data are stored, that are needed for the semiconductor models of the package Semiconductors.</p>
</html>"));
        end ModelcardBJT2;

        model BJT "Bipolar junction transistor, obsolete, use model BJT2"
          extends Modelica.Icons.ObsoleteModel;
          Modelica.Electrical.Analog.Interfaces.PositivePin B "Base node" annotation(
            Placement(transformation(extent = {{-108, -10}, {-88, 10}}), iconTransformation(extent = {{-106, -10}, {-86, 10}})));
          Modelica.Electrical.Analog.Interfaces.PositivePin C "Collector node" annotation(
            Placement(transformation(extent = {{10, 88}, {30, 108}}), iconTransformation(extent = {{20, 90}, {40, 110}})));
          Modelica.Electrical.Analog.Interfaces.NegativePin E "Emitter node" annotation(
            Placement(transformation(extent = {{10, -108}, {30, -88}}), iconTransformation(extent = {{20, -110}, {40, -90}})));
          parameter Real TBJT(start = 1) "Type of transistor (NPN=1, PNP=-1)";
          parameter Real AREA = 1.0 "Area factor";
          parameter Boolean OFF = false "Optional initial condition: false - IC not used, true - IC used, not implemented yet";
          parameter SI.Voltage IC_VCE(start = -1e40) "Initial condition value (VBE, not implemented yet";
          parameter SI.Voltage IC_VBE(start = -1e40) "Initial condition value (VBC, not implemented yet";
          parameter SI.Temp_C TEMP = 27 "Operating temperature of the device";
          parameter Boolean SENS_AREA = false "Flag to request sensitivity WRT area, not implemented yet";
          parameter Modelica.Electrical.Spice3.Internal.ModelcardBJT modelcard "BJT modelcard" annotation(
            Evaluate = true);
          final parameter Spice3.Internal.Bjt3.BjtModelLineParams p = Spice3.Internal.Bjt3.bjtRenameParameters(modelcard, Con, TBJT) "Model line parameters" annotation(
            Evaluate = true);
          constant Spice3.Internal.SpiceConstants Con "General constants of SPICE simulator";
          final parameter Spice3.Internal.Bjt3.Bjt p1 = Spice3.Internal.Bjt3.bjtRenameParametersDev(AREA, OFF, IC_VBE, IC_VCE, SENS_AREA) "Renamed parameters" annotation(
            Evaluate = true);
          final parameter Spice3.Internal.Model.Model m = Spice3.Internal.Bjt3.bjtRenameParametersDevTemp(TEMP) "Renamed parameters" annotation(
            Evaluate = true);
          final parameter Spice3.Internal.Bjt3.BjtModelLineVariables vl = Spice3.Internal.Bjt3.bjtModelLineInitEquations(p) "Model line variables";
          final parameter Spice3.Internal.Bjt3.Bjt3Calc c = Spice3.Internal.Bjt3.bjt3CalcTempDependencies(p1, p, m, vl) "Precalculated parameters";
          final parameter Spice3.Internal.Bjt3.BjtVariables v = Spice3.Internal.Bjt3.bjtInitEquations(p1, p, vl) "Precalculated parameters";
          constant Boolean m_bInit = false;
          Spice3.Internal.Bjt3.CurrentsCapacitances cc;
          Real Cinternal;
          //inner collector node
          Real Binternal;
          //inner base node
          Real Einternal;
          //inner emitter node
          Real irc;
          Real ire;
          Real irb;
          Real ibcgmin;
          Real ibegmin;
          Real capbe;
          Real icapbe;
          Real capbc;
          Real icapbc;
          Real capbx;
          Real icapbx;
          SI.Voltage vBE "Base - emitter voltage";
          SI.Voltage vCE "Collector - emitter voltage";
          SI.Voltage vBC "Base - collector voltage";
        equation
          vBE = B.v - E.v;
          vCE = C.v - E.v;
          vBC = B.v - C.v;
          (cc, capbe, capbc, capbx) = Spice3.Internal.Bjt3.bjtNoBypassCode(m, p1, p, c, v, vl, {C.v, B.v, E.v, Cinternal, Binternal, Einternal}, m_bInit);
//currents through capacitances
          icapbe = if m_bInit then 0.0 else capbe * (der(Binternal) - der(Einternal));
          icapbc = if m_bInit then 0.0 else capbc * (der(Binternal) - der(Cinternal));
          icapbx = if m_bInit then 0.0 else capbx * (der(B.v) - der(Cinternal));
//Resistances
          irc * p.m_collectorResist = C.v - Cinternal;
          ire * p.m_emitterResist = E.v - Einternal;
          irb * p.m_baseResist = B.v - Binternal;
//currents
          ibcgmin = Spice3.Internal.SpiceConstants.CKTgmin * (Binternal - Cinternal);
          ibegmin = Spice3.Internal.SpiceConstants.CKTgmin * (Binternal - Einternal);
          C.i = irc;
          E.i = ire;
          B.i = irb + icapbx;
//current sum at inner nodes
          0 = ibcgmin + irc - cc.iCC + cc.iBCN + cc.iBC + icapbc + icapbx;
//current sum for inner node Cinternal
          0 = ibegmin + ire + cc.iCC + cc.iBEN + cc.iBE + icapbe;
//current sum for inner node Einternal
          0 = (-ibcgmin) - ibegmin + irb - cc.iBC - cc.iBE - cc.iBCN - cc.iBEN - icapbc - icapbe;
//current sum for inner node Binternal
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-20, 60}, {-20, -60}}, color = {0, 0, 255}, smooth = Smooth.None), Line(points = {{-20, 0}, {-86, 0}}, color = {0, 0, 255}, smooth = Smooth.None), Line(points = {{34, 94}, {-20, 40}}, color = {0, 0, 255}, smooth = Smooth.None), Line(points = {{-20, -40}, {32, -92}}, color = {0, 0, 255}, smooth = Smooth.None), Text(extent = {{-94, 56}, {206, 16}}, textString = "%name", lineColor = {0, 0, 255})}),
            Documentation(info = "<html>
<p>This model is obsolete, please use BJT2</p>
<p>Bipolar junction transistor model</p>
<p>The package Internal is not for user access. There all function, records and data are stored, that are needed for the semiconductor models of the package Semiconductors.</p>
</html>", revisions = "<html>
<ul>
<li><i>August 2009 </i>by Kristin Majetta <br/>initially implemented</li>
</ul>
</html>"));
        end BJT;

        record ModelcardBJT "Record with technological parameters (.model), obsolete model, please use ModelcardBJT2"
          extends Modelica.Icons.Record;
          extends Modelica.Icons.ObsoleteModel;
          parameter SI.Temp_C TNOM = -1e40 "Parameter measurement temperature, default 27";
          parameter SI.Current IS = 1e-16 "Transport saturation current";
          parameter Real BF = 100.00 "Ideal maximum forward beta F";
          parameter Real NF = 1.0 "Forward current emission coefficientF";
          parameter Real NE = 1.5 "B-E leakage emission coefficient";
          parameter SI.Current ISE = -1e40 "B-E leakage saturation current, default = 0";
          constant Real C2 = -1e40 "Obsolete parameter name, default = 0";
          parameter SI.Current ISC = -1e40 "B-C leakage saturation current, default = 0";
          constant Real C4 = -1e40 "Obsolete parameter name, default = 0";
          parameter Real BR = 1.0 "Ideal maximum reverse beta";
          parameter Real NR = 1.0 "Reverse current emission coefficient";
          parameter Real NC = 2.0 "B-C leakage emission coefficient";
          parameter SI.Voltage VAF = 0.0 "Forward Early voltage";
          parameter SI.Current IKF = 0.0 "Forward beta roll-off corner current";
          parameter SI.Voltage VAR = 0.0 "Reverse Early voltage";
          parameter SI.Current IKR = 0.0 "Reverse beta roll-off corner current";
          parameter SI.Resistance RE = 0.0 "Emitter resistance";
          parameter SI.Resistance RC = 0.0 "Collector resistance";
          parameter SI.Current IRB = 0.0 "Current for base resistance = (rb+rbm)/2";
          parameter SI.Resistance RB = 0.0 "Zero bias base resistance";
          parameter SI.Resistance RBM = -1e40 "Minimum base resistance, default = 0.0";
          parameter SI.Capacitance CJE = 0.0 "Zero bias B-E depletion capacitance";
          parameter SI.Voltage VJE = 0.75 "B-E built in potential";
          parameter Real MJE = 0.33 "B-E junction exponential factor";
          parameter SI.Time TF = 0.0 "Ideal forward transit time";
          parameter Real XTF = 0.0 "Coefficient for bias dependence of TF";
          parameter SI.Current ITF = 0.0 "High current dependence of TF,";
          parameter SI.Voltage VTF = 0.0 "Voltage giving VBC dependence of TF";
          parameter SI.Temp_C PTF = 0.0 "Excess phase at freq=1/(TF*2*Pi) Hz";
          parameter SI.Capacitance CJC = 0.0 "Zero bias B-C depletion capacitance";
          parameter SI.Voltage VJC = 0.75 "B-C built in potential";
          parameter Real MJC = 0.33 "B-C junction grading coefficient";
          parameter Real XCJC = 1.0 "Fraction of B-C cap to internal base";
          parameter SI.Time TR = 0.0 "Ideal reverse transit time";
          parameter SI.Capacitance CJS = 0.0 "Zero bias C-S capacitance";
          parameter SI.Voltage VJS = 0.75 "Substrate junction built-in potential";
          parameter Real MJS = 0.0 "Substrate junction grading coefficient";
          parameter Real XTB = 0.0 "Forward and reverse beta temperature exponent";
          parameter SI.GapEnergy EG = 1.11 "Energy gap for IS temperature effect on IS";
          parameter Real XTI = 3.0 "Temperature exponent for IS";
          parameter Real KF = 0.0 "Flicker Noise Coefficient";
          parameter Real AF = 1.0 "Flicker Noise Exponent";
          parameter Real FC = 0.5 "Forward bias junction fit parameter";
          annotation(
            Documentation(info = "<html>
<p>This model is obsolete, please use BJT2</p>
<p>Modelcard parameters for BJT model, both PNP and NPN</p>
<p>The package Internal is not for user access. There all function, records and data are stored, that are needed for the semiconductor models of the package Semiconductors.</p>
</html>"));
        end ModelcardBJT;

        model JFET "Junction Field-Effect Transistor"
          Modelica.Electrical.Analog.Interfaces.PositivePin G "gate node" annotation(
            Placement(transformation(extent = {{-110, -12}, {-90, 10}}, rotation = 0)));
          Modelica.Electrical.Analog.Interfaces.PositivePin D "drain node" annotation(
            Placement(transformation(extent = {{-10, 90}, {10, 110}}, rotation = 0)));
          Modelica.Electrical.Analog.Interfaces.NegativePin S "source node" annotation(
            Placement(transformation(extent = {{-10, -110}, {10, -90}}, rotation = 0)));
          parameter Integer mtype(start = 0) "JFET type: 0 - N channel, 1 - P channel";
          parameter Real AREA(start = 1) "Number of parallel connected identical elements";
          parameter Boolean OFF = false "Optional initial condition: 0 - IC not used, 1 - IC used, not implemented yet";
          parameter SI.Voltage IC_VDS = -1e40 "Initial condition value (VDS, not implemented yet)";
          parameter SI.Voltage IC_VGS = -1e40 "Initial condition value (VGS, not implemented yet)";
          parameter Boolean UIC = false "Use initial conditions: true, if initial condition is used";
          parameter SI.Temp_C TEMP = 27 "Operating temperature of the device";
          parameter Modelica.Electrical.Spice3.Internal.ModelcardJFET modelcard "JFET modelcard" annotation(
            Evaluate = true);
          final parameter Modelica.Electrical.Spice3.Internal.Jfet.JfetModelLine p = Modelica.Electrical.Spice3.Internal.Jfet.jfetRenameParameters(modelcard) "Model line parameters" annotation(
            Evaluate = true);
          final parameter Modelica.Electrical.Spice3.Internal.Fet.Fet m = Modelica.Electrical.Spice3.Internal.Fet.fetRenameParametersDev(AREA, OFF, IC_VDS, IC_VGS, UIC, TEMP) "Renamed parameters" annotation(
            Evaluate = true);
          final parameter Integer m_type = if mtype > 0.5 then -1 else 1 "Type of the transistor";
          //    enum fet_type { NFET = 1, PFET = -1 };
          final parameter Modelica.Electrical.Spice3.Internal.Jfet.JfetModelLine p1 = Modelica.Electrical.Spice3.Internal.Jfet.jfetInitEquations(m, p) "Precalculated parameters";
          final parameter Modelica.Electrical.Spice3.Internal.Jfet.JfetModelLine p2 = Modelica.Electrical.Spice3.Internal.Jfet.jfetModelLineInitEquations(p1) "Model line variables";
          final parameter Modelica.Electrical.Spice3.Internal.Fet.Fet m1 = Modelica.Electrical.Spice3.Internal.Jfet.jfetCalcTempDependencies(m, p2) "Precalculated parameters";
          Modelica.Electrical.Spice3.Internal.Fet.CurrrentsCapacitances cc;
          constant Boolean m_bInit = false;
          SI.Voltage Dinternal;
          //internal drain node
          SI.Voltage Sinternal;
          //internal source node
          SI.Voltage vGD;
          SI.Voltage vGS;
          SI.Current ird;
          SI.Current irs;
          SI.Current igdgmin;
          SI.Current igsgmin;
          SI.Current icGS;
          SI.Current icGD;
        equation
          cc = Modelica.Electrical.Spice3.Internal.Jfet.jfetNoBypassCode(m1, p2, m_type, m_bInit, {G.v, Dinternal, Sinternal});
// voltages
// --------
          vGD = G.v - Dinternal;
          vGS = G.v - Sinternal;
// drain- and sourceresistances
// ----------------------------
          ird * p2.m_drainResist = D.v - Dinternal;
          irs * p2.m_sourceResist = S.v - Sinternal;
// capacitances
// ------------
          icGD = cc.cGD * der(vGD);
          icGS = cc.cGS * der(vGS);
// currents
// --------
          igsgmin = Modelica.Electrical.Spice3.Internal.SpiceConstants.CKTgmin * (G.v - Sinternal);
          igdgmin = Modelica.Electrical.Spice3.Internal.SpiceConstants.CKTgmin * (G.v - Dinternal);
          G.i = icGD + icGS + cc.iGD + igdgmin + cc.iGS + igsgmin;
          D.i = ird;
          S.i = irs;
//currentsum at inner node
//------------------------
          0 = (-ird) + cc.idrain - cc.iGD - igdgmin - icGD;
          0 = (-irs) - cc.idrain - cc.iGS - igsgmin - icGS;
          annotation(
            Documentation(info = "<html>
<p>JFET model, both N and P channel</p>
<p>The package Internal is not for user access. There all function, records and data are stored, that are needed for modeling the semiconductor models of the package Semiconductors.</p>
</html>", revisions = "<html>
<ul>
<li><i>March 2008 </i>by Kristin Majetta <br/>initially implemented</li>
</ul>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{0, 92}, {0, 40}, {-12, 40}, {-12, -40}, {0, -40}, {0, -94}}, color = {0, 0, 255}), Line(points = {{-92, 0}, {-20, 0}}, color = {0, 0, 255}), Line(points = {{-20, 40}, {-20, -40}}, color = {0, 0, 255}), Text(extent = {{8, -34}, {96, -54}}, lineColor = {0, 0, 255}, textString = "%name")}));
        end JFET;

        record ModelcardJFET "Record with technological parameters (.model)"
          extends Modelica.Icons.Record;
          parameter SI.Capacitance CGS = -1e40 "Zero-bias G-S junction capacitance, default 0";
          parameter SI.Capacitance CGD = -1e40 "Zero-bias G-D junction capacitance, default 0";
          parameter SI.Current IS = 1.e-14 "Saturation current of pn junctions";
          parameter Real FC = 0.5 "Coefficient for forward-bias depletion capacitance formula";
          parameter SI.Resistance RD = -1e40 "Drain ohmic resistance, default 0";
          parameter SI.Resistance RS = -1e40 "Source ohmic resistance, default 0";
          parameter SI.Temp_C TNOM = 27 "Parameter measurement temperature";
          parameter SI.Voltage VTO = -1e40 "Zero-bias threshold voltage, default -2";
          parameter SI.InversePotential B = -1e40 "Dotierungsverlauf parameter, default 1";
          parameter Real BETA = -1e40 "Output admittance parameter, default 1e-4";
          parameter SI.InversePotential LAMBDA = 0 "Channel-length modulation, default 0";
          parameter SI.Voltage PB = 1.0 "Junction potential of pn junctions";
          parameter Real AF = 1.0 "Flicker noise exponent";
          parameter Real KF = 0 "Flicker noise coefficient";
          annotation(
            Documentation(info = "<html>
<p>Modelcard parameters for JFET model, both N and P channel</p>
<p>The package Internal is not for user access. There all function, records and data are stored, that are needed for modeling the semiconductor models of the package Semiconductors.</p>
</html>"));
        end ModelcardJFET;

        model DIODE "Diode model"
          extends Modelica.Electrical.Analog.Interfaces.TwoPin;
          parameter Real AREA = 1 "Area factor";
          parameter Boolean OFF = false "Optional initial condition: false - IC not used, true - IC used, not implemented yet";
          parameter SI.Voltage IC(start = -1e40) "Initial condition value (VD, not implemented yet";
          parameter SI.Temp_C TEMP = 27 "Operating temperature of the device";
          parameter Boolean SENS_AREA(start = false) "Flag to request sensitivity WRT area, not implemented yet";
          parameter ModelcardDIODE modelcarddiode "DIODE modelcard" annotation(
            Evaluate = true);
          constant SpiceConstants C "General constants of SPICE simulator";
          final parameter Diode.DiodeModelLineParams param = Diode.diodeRenameParameters(modelcarddiode, C) "Model line parameters";
          final parameter Diode.DiodeParams dp = Diode.diodeRenameParametersDev(TEMP, AREA, IC, OFF, SENS_AREA) "Renamed parameters" annotation(
            Evaluate = true);
          final parameter Model.Model m = Diode.diodeRenameParametersDevTemp(TEMP) "Renamed parameters" annotation(
            Evaluate = true);
          final parameter Diode.DiodeVariables c1 = Diode.diodeInitEquations(param) "Precalculated values";
          final parameter Diode.DiodeCalc c2 = Diode.diodeCalcTempDependencies(param, dp, m, c1) "Precalculated values";
          constant Boolean m_mbInit = false;
          Diode.CurrentsCapacitances cc;
          Real icap;
          Real m_dCap;
          Real pin;
          Real ir;
          Real igmin;
        equation
          (cc, m_dCap) = Diode.diodeNoBypassCode(param, dp, c2, m, m_mbInit, {pin, n.v});
//current through capacitance
          icap = if m_mbInit then 0.0 else m_dCap * (der(pin) - der(n.v));
//resistance
          ir * param.m_resist = p.v - pin;
//gmin
          igmin = SpiceConstants.CKTgmin * (pin - n.v);
          p.i = ir;
          n.i = (-(cc.m_dCurrent + igmin)) - icap;
//currentsum at inner node
          0 = (-ir) + cc.m_dCurrent + igmin + icap;
          annotation(
            Icon(graphics = {Line(points = {{90, 0}, {-90, 0}}, color = {0, 0, 255}), Polygon(points = {{30, 0}, {-30, 40}, {-30, -40}, {30, 0}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{-90, 0}, {40, 0}}, color = {0, 0, 255}), Line(points = {{40, 0}, {90, 0}}, color = {0, 0, 255}), Line(points = {{30, 40}, {30, -40}}, color = {0, 0, 255}), Text(extent = {{-154, 100}, {146, 60}}, textString = "%name", lineColor = {0, 0, 255})}),
            Documentation(info = "<html>
<p>DIODE model</p>
<p>The package Repository is not for user access. There all function, records and data are stored, that are needed for the semiconductor models of the package Semiconductors.</p>
</html>", revisions = "<html>
<ul>
<li><i>Nov. 2008 </i>by Kristin Majetta <br/>initially implemented</li>
</ul>
</html>"));
        end DIODE;

        record ModelcardDIODE "Record with technological parameters (.model)"
          extends Modelica.Icons.Record;
          parameter SI.Current IS = 1e-14 "Saturation Current";
          parameter SI.Resistance RS = 0.0 "Ohmic resistance";
          parameter Real N = 1.0 "Emission coefficient";
          parameter SI.Time TT = 0.0 "Transit time";
          parameter SI.Capacitance CJO = 0.0 "Junction capacitance";
          parameter SI.Voltage VJ = 1.0 "Junction Potential";
          parameter Real M = 0.5 "Grading coefficient";
          parameter SI.ActivationEnergy EG = 1.11 "Activation Energy";
          parameter Real XTI = 3.0 "Saturation current temperature exponent";
          parameter Real FC = 0.5 "Forward bias junction fit parameter";
          parameter SI.Voltage BV = -1e40 "Reverse breakdown voltage, default infinity";
          parameter SI.Current IBV = 1e-3 "Current at reverse breakdown voltage";
          parameter SI.Temp_C TNOM = 27 "Parameter measurement temperature";
          parameter Real KF = 0.0 "Flicker noise coefficient";
          parameter Real AF = 1.0 "Flicker noise exponent";
          parameter SI.Conductance G = 0 "Ohmic conductance";
          annotation(
            Documentation(info = "<html>
<p>Modelcard parameters for DIODE model</p>
<p>The package Repository is not for user access. There all function, records and data are stored, that are needed for the semiconductor models of the package Semiconductors.</p>
</html>"));
        end ModelcardDIODE;

        model R_SEMI "Semiconductor resistor"
          extends Modelica.Electrical.Analog.Interfaces.OnePort;
          parameter SI.Resistance R = -1e40 "Resistance, if specified, geometrical information is overwritten";
          parameter SI.Temp_C TEMP = -1e40 "Temperature of resistor";
          parameter SI.Length L = -1e40 "Lenght of the resistor";
          parameter SI.Length W = -1e40 "Width of the resistor, default DEFW (modelcard)";
          parameter Boolean SENS_AREA = false "Parameter for sensitivity analyses, not implemented yet";
          parameter ModelcardR modelcard "Resistor modelcard";
          constant SpiceConstants C "General constants of SPICE simulator";
          final parameter Rsemiconductor.ResistorModelLineParams lp = Rsemiconductor.resistorRenameParameters(modelcard, C) "Model Line Parameters";
          final parameter Rsemiconductor.ResistorParams rp = Rsemiconductor.resistorRenameParametersDev(R, W, L, TEMP, SENS_AREA, C) "Renamed parameters";
          Rsemiconductor.ResistorVariables vp;
        algorithm
          vp := Rsemiconductor.resistorInitEquations(rp, lp);
          (vp.m_dConduct, vp.m_dCond_dTemp) := Modelica.Electrical.Spice3.Internal.Functions.resDepTemp(vp.m_dResist, rp.m_dTemp, lp.m_dTnom, lp.m_dTC1, lp.m_dTC2);
          i := vp.m_dConduct * v;
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-70, 28}, {70, -32}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{-70, 0}, {-90, 0}}, color = {0, 0, 255}, smooth = Smooth.None), Line(points = {{70, 0}, {90, 0}}, color = {0, 0, 255}, smooth = Smooth.None), Text(extent = {{-150, 64}, {150, 24}}, textString = "%name", lineColor = {0, 0, 255})}),
            Documentation(revisions = "<html>
<dl>
<dt><i> April 2009 </i></dt>
<dd>by Kristin Majetta initially implemented</dd>
</dl>
</html>", info = "<html>
<p>Semiconductor resistance model</p>
<p>The package Repository is not for user access. There all function, records and data are stored, that are needed for the semiconductor models of the package Semiconductors.</p>
</html>"));
        end R_SEMI;

        record ModelcardR "Record with technological parameters (.model)"
          extends Modelica.Icons.Record;
          parameter SI.Conversions.NonSIunits.FirstOrderTemperaturCoefficient TC1 = 0.0 "First order temperature coefficient";
          parameter SI.Conversions.NonSIunits.SecondOrderTemperaturCoefficient TC2 = 0.0 "In Ohm/(deg C*deg C), Second2 order temperature coefficient";
          parameter SI.Resistance RSH = -1e40 "Sheet resistance";
          parameter SI.Temp_C TNOM = -1e40 "Parameter measurement temperature, default 27";
          parameter SI.Length DEFW = 1e-5 "Default device width";
          parameter SI.Length NARROW = 0 "Narrowing of resistor due to side etching";
          annotation(
            Documentation(info = "<html>
<p>Modelcard parameters for semiconductor resistance model</p>
<p>The package Repository is not for user access. There all function, records and data are stored, that are needed for the semiconductor models of the package Semiconductors.</p>
</html>"));
        end ModelcardR;

        model C_SEMI "Semiconductor capacitor"
          extends Modelica.Electrical.Analog.Interfaces.OnePort;
          parameter SI.Capacitance C = -1e40 "Capacitance, if specified, geometrical information is overwritten";
          parameter SI.Temp_C TEMP = 27 "Temperature of capacitor";
          parameter SI.Length L(start = 0) "Lenght of the capacitor";
          parameter SI.Length W = -1e40 "Width of the capacitor, default DEFW (modelcard)";
          parameter Boolean SENS_AREA = false "Parameter for sensitivity analyses, not implemented yet";
          parameter SI.Voltage IC = 0 "Initial value";
          parameter Boolean UIC = false "Use initial conditions: true, if initial condition is used";
          parameter Modelica.Electrical.Spice3.Internal.ModelcardC modelcard "Capacitor modelcard";
          final parameter Modelica.Electrical.Spice3.Internal.Csemiconductor.CapacitorModelLineParams lp = Modelica.Electrical.Spice3.Internal.Csemiconductor.capacitorRenameParameters(modelcard) "Model Line Parameters";
          final parameter Modelica.Electrical.Spice3.Internal.Csemiconductor.Capacitor cp = Modelica.Electrical.Spice3.Internal.Csemiconductor.capacitorRenameParametersDev(C, W, L, TEMP, SENS_AREA, lp) "Renamed parameters";
          Modelica.Electrical.Spice3.Internal.Csemiconductor.Capacitor vp;
        protected
          SI.Voltage vinternal;
        initial equation
          if UIC then
            vinternal = IC;
          else
            der(vinternal) = 0;
          end if;
        algorithm
          if cp.m_dCapIsGiven < 0.5 then
            assert(L > 0, "Length of capacitor must be greater then zero");
          end if;
          vp := Modelica.Electrical.Spice3.Internal.Csemiconductor.capacitorInitEquations(cp, lp);
          vinternal := p.v - n.v;
          i := vp.m_dCapac * der(vinternal);
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-14, 28}, {-14, -28}}, color = {0, 0, 255}, thickness = 0.5), Line(points = {{14, 28}, {14, -28}}, color = {0, 0, 255}, thickness = 0.5), Line(points = {{-90, 0}, {-14, 0}}), Line(points = {{14, 0}, {90, 0}}), Text(extent = {{-130, -40}, {134, -60}}, lineColor = {0, 0, 0}, textString = "C=%C"), Text(extent = {{-138, 42}, {136, 62}}, textString = "%name")}),
            Documentation(revisions = "<html>
<ul>
<li><i>April 2009</i> by Kristin Majetta
    <br>initially implemented</li>
</ul>
</html>", info = "<html>
<p>Semiconductor capacitance model</p>
<p><br/>The package Repository is not for user access. There all function, records and data are stored, that are needed for the semiconductor models of the package Semiconductors.</p>
</html>"));
        end C_SEMI;

        record ModelcardC "Record with technological parameters (.model)"
          extends Modelica.Icons.Record;
          parameter SI.CapacitancePerArea CJ = 0.0 "Junction bottom capacitance     F/meters2";
          parameter SI.Permittivity CJSW = 0.0 "Junction sidewall capacitance   F/meters";
          parameter SI.Length DEFW = 1e-5 "Default device width";
          parameter SI.Length NARROW = 0 "Narrowing due to side etching";
          annotation(
            Documentation(info = "<html>
<p>Modelcard parameters for semiconductor capacitance model</p>
<p><br/>The package Repository is not for user access. There all function, records and data are stored, that are needed for the semiconductor models of the package Semiconductors.</p>
</html>"));
        end ModelcardC;

        record SpiceConstants "General constants of SPICE simulator"
          extends Modelica.Icons.Record;
          constant Real EPSSIL = 11.7 * 8.854214871e-12;
          constant Real EPSOX = 3.453133e-11;
          constant SI.Charge CHARGE = 1.6021918e-19;
          constant SI.Temp_K CONSTCtoK = 273.15;
          constant SI.HeatCapacity CONSTboltz = 1.3806226e-23;
          // J/K
          constant SI.Temp_K REFTEMP = 300.15;
          /* 27 deg C */
          constant Real CONSTroot2 = sqrt(2.0);
          constant Real CONSTvt0(final unit = "(J/K)/(A.s)") = Modelica.Constants.k * Modelica.SIunits.Conversions.from_degC(27) / CHARGE;
          // deg C
          constant Real CONSTKoverQ(final unit = "(J/K)/(A.s)") = Modelica.Constants.k / CHARGE;
          constant Real CONSTe = exp(1.0);
          // options
          constant SI.Conductance CKTgmin = 1e-12;
          constant SI.Temp_K CKTnomTemp = 300.15;
          constant SI.Temp_K CKTtemp = 300.15;
          constant SI.Area CKTdefaultMosAD = 0.0;
          constant SI.Area CKTdefaultMosAS = 0.0;
          constant SI.Length CKTdefaultMosL = 100e-6;
          constant SI.Length CKTdefaultMosW = 100e-6;
          constant Real CKTreltol = 1e-10;
          constant Real CKTabstol = 1e-15;
          constant Real CKTvolttol = 1e-10;
          constant Real CKTtemptol = 1e-3;
          annotation(
            Documentation(info = "<html>
<p>General constants used by SPICE</p>
<p>The package Internal is not for user access. There all function, records and data are stored, that are needed for the semiconductor models of the package Semiconductors.</p>
</html>"));
        end SpiceConstants;

        record MaterialParameters
          extends Modelica.Icons.Record;
          // energy gap for silicon
          constant SI.GapEnergy EnergyGapSi = 1.16;
          // first band correction factor of silicon
          constant Types.GapEnergyPerTemperature FirstBandCorrFactorSi = 7.02e-4;
          // second band correction factor of silicon
          constant SI.Temperature SecondBandCorrFactorSi = 1108;
          // band correction factor for T = 300K
          constant SI.GapEnergy BandCorrFactorT300 = 1.1150877;
          // intrinsic conduction carrier density
          constant Types.PerVolume IntCondCarrDensity = 1.45e16;
          annotation(
            Documentation(info = "<html>
<p>Definition of Material parameters</p>
<p>The package Repository is not for user access. There all function, records and data are stored, that are needed for the semiconductor models of the package Semiconductors.</p>
</html>"));
        end MaterialParameters;

        package Functions "Equations for semiconductor calculation"
          extends Modelica.Icons.InternalPackage;

          function junctionPotDepTemp "Temperature dependency of junction potential"
            extends Modelica.Icons.Function;
            input Modelica.SIunits.Voltage phi0;
            input Modelica.SIunits.Temp_K temp "Device Temperature";
            input Modelica.SIunits.Temp_K tnom "Nominal Temperature";
            output Modelica.SIunits.Voltage ret "Output voltage";
          protected
            Modelica.SIunits.Voltage phibtemp;
            Modelica.SIunits.Voltage phibtnom;
            Modelica.SIunits.Voltage vt;
          algorithm
            phibtemp := Modelica.Electrical.Spice3.Internal.Functions.energyGapDepTemp(temp);
            phibtnom := Modelica.Electrical.Spice3.Internal.Functions.energyGapDepTemp(tnom);
            vt := Spice3.Internal.SpiceConstants.CONSTKoverQ * temp;
            ret := (phi0 - phibtnom) * temp / tnom + phibtemp + vt * 3 * Modelica.Math.log(tnom / temp);
            annotation(
              Documentation(info = "<html>
<p>This internal function calculates the temperature dependent junction potential based on the actual and the nominal temperature.</p>
</html>"));
          end junctionPotDepTemp;

          function saturationCurDepTempSPICE3MOSFET "Temperature dependency of saturation current"
            extends Modelica.Icons.Function;
            input Modelica.SIunits.Current satcur0 "Saturation current";
            input Modelica.SIunits.Temp_K temp "Device Temperature";
            input Modelica.SIunits.Temp_K tnom "Nominal Temperature";
            output Real ret "Output current";
            //unit Current
          protected
            Modelica.SIunits.Voltage vt;
            Modelica.SIunits.Voltage vtnom;
            Modelica.SIunits.Voltage energygaptnom;
            Modelica.SIunits.Voltage energygaptemp;
          algorithm
            vt := Spice3.Internal.SpiceConstants.CONSTKoverQ * temp;
            vtnom := Spice3.Internal.SpiceConstants.CONSTKoverQ * tnom;
            energygaptnom := Modelica.Electrical.Spice3.Internal.Functions.energyGapDepTemp(tnom);
            energygaptemp := Modelica.Electrical.Spice3.Internal.Functions.energyGapDepTemp(temp);
            ret := satcur0 * exp(energygaptnom / vtnom - energygaptemp / vt);
            annotation(
              Documentation(info = "<html>
<p>This internal function calculates the temperature dependent saturation current based on the actual and the nominal temperature.</p>
</html>"));
          end saturationCurDepTempSPICE3MOSFET;

          function junctionVCrit "Voltage limitation"
            extends Modelica.Icons.Function;
            input Modelica.SIunits.Temp_K temp "temperature";
            input Real ncoeff;
            input Modelica.SIunits.Current satcur "Saturation current";
            output Real ret "Output value";
          protected
            Modelica.SIunits.Voltage vte;
          algorithm
            vte := Spice3.Internal.SpiceConstants.CONSTKoverQ * temp * ncoeff;
            ret := vte * Modelica.Math.log(vte / (sqrt(2) * satcur));
            ret := if ret > 1e10 then 1e10 else ret;
            annotation(
              Documentation(info = "<html>
<p>This internal function limits the junction voltage. If it increases 1.e10, it is hold to be constant at that value.</p>
</html>"));
          end junctionVCrit;

          function junctionParamDepTempSPICE3 "Temperature dependency of junction parameters"
            extends Modelica.Icons.Function;
            input Modelica.SIunits.Voltage phi0;
            input Real cap0;
            input Real mcoeff;
            input Modelica.SIunits.Temp_K temp "Device temperature";
            input Modelica.SIunits.Temp_K tnom "Nominal temperature";
            output Modelica.SIunits.Voltage junctionpot "Junction potential";
            output Real jucntioncap "Junction capacitance";
          protected
            Modelica.SIunits.Voltage phibtemp;
            Modelica.SIunits.Voltage phibtnom;
            Modelica.SIunits.Voltage vt;
            Modelica.SIunits.Voltage vtnom;
            Real arg;
            Real fact2;
            Real pbfact;
            Real arg1;
            Real fact1;
            Real pbfact1;
            Real pbo;
            Real gmaold;
            Real gmanew;
          algorithm
            phibtemp := Modelica.Electrical.Spice3.Internal.Functions.energyGapDepTemp(temp);
            phibtnom := Modelica.Electrical.Spice3.Internal.Functions.energyGapDepTemp(tnom);
            vt := Spice3.Internal.SpiceConstants.CONSTKoverQ * temp;
            vtnom := Spice3.Internal.SpiceConstants.CONSTKoverQ * tnom;
            arg := (-phibtemp / (2 * Modelica.Constants.k * temp)) + 1.1150877 / (Modelica.Constants.k * (2 * Spice3.Internal.SpiceConstants.REFTEMP));
            fact2 := temp / Spice3.Internal.SpiceConstants.REFTEMP;
            pbfact := -2 * vt * (1.5 * Modelica.Math.log(fact2) + Spice3.Internal.SpiceConstants.CHARGE * arg);
            arg1 := (-phibtnom / (Modelica.Constants.k * 2 * tnom)) + 1.1150877 / (2 * Modelica.Constants.k * Spice3.Internal.SpiceConstants.REFTEMP);
            fact1 := tnom / Spice3.Internal.SpiceConstants.REFTEMP;
            pbfact1 := -2 * vtnom * (1.5 * Modelica.Math.log(fact1) + Spice3.Internal.SpiceConstants.CHARGE * arg1);
            pbo := (phi0 - pbfact1) / fact1;
            junctionpot := pbfact + fact2 * pbo;
            gmaold := (phi0 - pbo) / pbo;
            gmanew := (junctionpot - pbo) / pbo;
            jucntioncap := cap0 / (1 + mcoeff * (400e-6 * (tnom - Spice3.Internal.SpiceConstants.REFTEMP) - gmaold)) * (1 + mcoeff * (400e-6 * (temp - Spice3.Internal.SpiceConstants.REFTEMP) - gmanew));
            annotation(
              Documentation(info = "<html>
<p>This internal function calculates several temperature dependent junction parameters based on the actual and the nominal temperature.</p>
</html>"));
          end junctionParamDepTempSPICE3;

          function junctionCapCoeffs "Coefficient calculation"
            extends Modelica.Icons.Function;
            input Real mj;
            input Real fc;
            input Modelica.SIunits.Voltage phij;
            output Modelica.SIunits.Voltage f1;
            output Real f2;
            output Real f3;
          protected
            Real xfc;
          algorithm
            xfc := Modelica.Math.log(1 - fc);
            f1 := phij * (1 - exp((1 - mj) * xfc)) / (1 - mj);
            f2 := exp((1 + mj) * xfc);
            f3 := 1 - fc * (1 + mj);
            annotation(
              Documentation(info = "<html>
<p>This internal auxiliary function calculates some coefficients which are necessary for the calculation of junction capacities.</p>
</html>"));
          end junctionCapCoeffs;

          function junction2SPICE3MOSFETRevised "Junction current and conductance calculation"
            extends Modelica.Icons.Function;
            input SI.Voltage voltage "Input voltage";
            input SI.Temp_K temp "Device Temperature";
            input Real ncoeff;
            input SI.Current satcur "Saturation current";
            output SI.Current out_current "Calculated current";
            output SI.Conductance out_cond "Calculated conductance";
          protected
            SI.Voltage vte;
            Real max_exponent;
            Real evbd;
            Real evd;
            constant Real max_exp = 50.;
            constant SI.Current max_current = 1.e4;
          algorithm
            if satcur > 1e-101 then
              vte := Spice3.Internal.SpiceConstants.CONSTKoverQ * temp * ncoeff;
              max_exponent := Modelica.Math.log(max_current / satcur);
              max_exponent := min(max_exp, max_exponent);
              if voltage <= 0 then
                out_cond := satcur / vte;
                out_current := out_cond * voltage;
                out_cond := out_cond + Spice3.Internal.SpiceConstants.CKTgmin;
              elseif voltage >= max_exponent * vte then
                evd := exp(max_exponent);
                out_cond := satcur * evd / vte;
                out_current := satcur * (evd - 1) + out_cond * (voltage - max_exponent * vte);
              else
                evbd := exp(voltage / vte);
                out_cond := satcur * evbd / vte + Spice3.Internal.SpiceConstants.CKTgmin;
                out_current := satcur * (evbd - 1);
              end if;
            else
              out_current := 0.;
              out_cond := 0.;
            end if;
            annotation(
              Documentation(info = "<html>
<p>This internal function calculates both the junction current and the junction conductance dependent from the given voltage.</p>
</html>"));
          end junction2SPICE3MOSFETRevised;

          function junction2SPICE3MOSFET "Junction current and conductance calculation, obsolete, use junction2SPICE3MOSFETRevised"
            extends Modelica.Icons.Function;
            extends Modelica.Icons.ObsoleteModel;
            input Modelica.SIunits.Current current "Input current";
            input Modelica.SIunits.Conductance cond "Input conductance";
            input Modelica.SIunits.Voltage voltage "Input voltage";
            input Modelica.SIunits.Temp_K temp "Device Temperature";
            input Real ncoeff;
            input Modelica.SIunits.Current satcur "Saturation current";
            output Modelica.SIunits.Current out_current "Calculated current";
            output Modelica.SIunits.Conductance out_cond "Calculated conductance";
          protected
            Modelica.SIunits.Voltage vte;
            Real max_exponent;
            Real evbd;
            Real evd;
            constant Real max_exp = 50.;
            constant Modelica.SIunits.Current max_current = 1.e4;
          algorithm
            out_current := current;
            out_cond := cond;
            if satcur > 1e-101 then
              vte := Spice3.Internal.SpiceConstants.CONSTKoverQ * temp * ncoeff;
              max_exponent := Modelica.Math.log(max_current / satcur);
              max_exponent := min(max_exp, max_exponent);
              if voltage <= 0 then
                out_cond := satcur / vte;
                out_current := out_cond * voltage;
                out_cond := out_cond + Spice3.Internal.SpiceConstants.CKTgmin;
              elseif voltage >= max_exponent * vte then
                evd := exp(max_exponent);
                out_cond := satcur * evd / vte;
                out_current := satcur * (evd - 1) + out_cond * (voltage - max_exponent * vte);
              else
                evbd := exp(voltage / vte);
                out_cond := satcur * evbd / vte + Spice3.Internal.SpiceConstants.CKTgmin;
                out_current := satcur * (evbd - 1);
              end if;
            else
              out_current := 0.;
              out_cond := 0.;
            end if;
            annotation(
              Documentation(info = "<html>
<p>This internal function calculates both the junction current and the junction conductance dependent from the given voltage.</p>
</html>"));
          end junction2SPICE3MOSFET;

          function junctionCapRevised "Junction capacity"
            extends Modelica.Icons.Function;
            input SI.Capacitance capin "Input capacitance";
            input SI.Voltage voltage "Input voltage";
            input SI.Voltage depcap;
            input Real mj;
            input Real phij;
            input SI.Voltage f1;
            input Real f2;
            input Real f3;
            output SI.Capacitance capout "Output capacitance";
            output SI.Charge charge "Output charge";
          protected
            Real arg;
            Real sarg;
            Real czof2;
          algorithm
            if voltage < depcap then
              arg := 1 - voltage / phij;
              if mj == 0.5 then
                sarg := 1 / sqrt(arg);
              else
                sarg := exp(-1 * mj * Modelica.Math.log(arg));
              end if;
              capout := capin * sarg;
              charge := phij * (capin * (1 - arg * sarg) / (1 - mj));
            else
              czof2 := capin / f2;
              capout := czof2 * (f3 + mj * voltage / phij);
              charge := capin * f1 + czof2 * (f3 * (voltage - depcap) + mj / (2 * phij) * (voltage ^ 2 - depcap ^ 2));
            end if;
            annotation(
              Documentation(info = "<html>
<p>This internal function calculates the charge and the capacitance of the junction capacity dependent from the given voltage.</p>
</html>"));
          end junctionCapRevised;

          function junctionCap "Junction capacity, obsolete, use JunctionCapRevised"
            extends Modelica.Icons.Function;
            extends Modelica.Icons.ObsoleteModel;
            input Modelica.SIunits.Capacitance capin "Input capacitance";
            input Modelica.SIunits.Voltage voltage "Input voltage";
            input Modelica.SIunits.Voltage depcap;
            input Real mj;
            input Real phij;
            input Modelica.SIunits.Voltage f1;
            input Real f2;
            input Real f3;
            output Modelica.SIunits.Capacitance capout "Output capacitance";
            output Modelica.SIunits.Charge charge "Output charge";
          protected
            Real arg;
            Real sarg;
            Real czof2;
          algorithm
            if voltage < depcap then
              arg := 1 - voltage / phij;
              if mj == 0.5 then
                sarg := 1 / sqrt(arg);
              else
                sarg := exp(-1 * mj * Modelica.Math.log(arg));
              end if;
              capout := capin * sarg;
              charge := phij * (capin * (1 - arg * sarg) / (1 - mj));
            else
              czof2 := capin / f2;
              capout := czof2 * (f3 + mj * voltage / phij);
              charge := capin * f1 + czof2 * (f3 * (voltage - depcap) + mj / (2 * phij) * (voltage ^ 2 - depcap ^ 2));
            end if;
            annotation(
              Documentation(info = "<html>
<p>This internal function calculates the charge and the capacitance of the junction capacity dependent from the given voltage.</p>
</html>"));
          end junctionCap;

          function saturationCurDepTempSPICE3 "Temperature dependency of saturation current"
            extends Modelica.Icons.Function;
            input Modelica.SIunits.Current satcur0 "Saturation current";
            input Modelica.SIunits.Temp_K temp "Device Temperature";
            input Modelica.SIunits.Temp_K tnom "Nominal Temperature";
            input Real emissioncoeff;
            input Real energygap;
            input Real satcurexp;
            output Modelica.SIunits.Current ret "Output value";
          protected
            Modelica.SIunits.Voltage vt;
            Modelica.SIunits.Voltage vte;
          algorithm
            vt := Spice3.Internal.SpiceConstants.CONSTKoverQ * temp;
            vte := emissioncoeff * vt;
            ret := satcur0 * exp((temp / tnom - 1) * energygap / vte + satcurexp / emissioncoeff * Modelica.Math.log(temp / tnom));
            annotation(
              Documentation(info = "<html>
<p>This internal function calculates the temperature dependent saturation current based on the actual and the nominal temperature.</p>
</html>"));
          end saturationCurDepTempSPICE3;

          function junctionVoltage23SPICE3 "Junction Voltage"
            extends Modelica.Icons.Function;
            input Modelica.SIunits.Voltage vb;
            input Modelica.SIunits.Current ivb;
            input Modelica.SIunits.Current satcur "Saturation current";
            input Modelica.SIunits.Temp_K temp "Device temperature";
            input Real ncoeff;
            output Modelica.SIunits.Voltage v23 "Output value";
          protected
            Modelica.SIunits.Voltage vt;
            Modelica.SIunits.Current cbv;
            Real tol;
            Integer iter;
          algorithm
            vt := Spice3.Internal.SpiceConstants.CONSTKoverQ * temp;
            v23 := vb;
            cbv := ivb;
            if cbv < satcur * vb / vt then
              cbv := satcur * vb / vt;
            else
              tol := Spice3.Internal.SpiceConstants.CKTreltol * cbv;
              v23 := vb - vt * Modelica.Math.log(1 + cbv / satcur);
              for iter in 0:24 loop
                v23 := vb - vt * Modelica.Math.log(cbv / satcur + 1 - v23 / vt);
                if abs(satcur * (exp((vb - v23) / vt) - 1 + v23 / vt) - cbv) <= tol then
                end if;
              end for;
            end if;
            annotation(
              Documentation(info = "<html>
<p>This internal function calculates the junction voltage based on the actual temperature, voltage and saturation current.</p>
</html>"));
          end junctionVoltage23SPICE3;

          function junction3 "Junction current and conductance calculation"
            extends Modelica.Icons.Function;
            input Modelica.SIunits.Voltage voltage "Input voltage";
            input Modelica.SIunits.Temp_K temp "Device Temperature";
            input Real ncoeff;
            input Modelica.SIunits.Current satcur "Saturation current";
            input Modelica.SIunits.Voltage v23;
            output Modelica.SIunits.Current current "Output current";
            output Modelica.SIunits.Conductance cond "Output conductance";
          protected
            constant Real max_exp = 50.0;
            constant Modelica.SIunits.Current max_current = 1.0e4;
            Modelica.SIunits.Voltage vte;
            Real max_exponent;
            Real evd;
            Real arg;
            Real evrev;
            Modelica.SIunits.Voltage vr;
          algorithm
            if satcur > 1.0e-101 then
              vte := Spice3.Internal.SpiceConstants.CONSTKoverQ * temp * ncoeff;
              max_exponent := Modelica.Math.log(max_current / satcur);
              max_exponent := min(max_exp, max_exponent);
              if voltage >= max_exponent * vte then
                evd := exp(max_exponent);
                cond := satcur * evd / vte;
                current := satcur * (evd - 1) + cond * (voltage - max_exponent * vte);
              elseif voltage >= (-3 * vte) then
                evd := exp(voltage / vte);
                current := satcur * (evd - 1) + Spice3.Internal.SpiceConstants.CKTgmin * voltage;
                cond := satcur * evd / vte + Spice3.Internal.SpiceConstants.CKTgmin;
              elseif voltage >= (-v23) then
                arg := 3 * vte / (voltage * Spice3.Internal.SpiceConstants.CONSTe);
                arg := arg * arg * arg;
                current := (-1. * satcur * (1 + arg)) + Spice3.Internal.SpiceConstants.CKTgmin * voltage;
                cond := satcur * 3 * arg / voltage + Spice3.Internal.SpiceConstants.CKTgmin;
              else
                vr := -(v23 + voltage);
                if vr > max_exponent * vte then
                  evd := exp(max_exponent);
                  cond := satcur * evd / vte;
                  current := -1. * (satcur * (evd - 1) + cond * (vr - max_exponent * vte));
                else
                  evrev := exp(vr / vte);
                  current := (-1. * satcur * evrev) + Spice3.Internal.SpiceConstants.CKTgmin * voltage;
                  cond := satcur * evrev / vte + Spice3.Internal.SpiceConstants.CKTgmin;
                end if;
              end if;
            else
              current := 0.0;
              cond := 0.0;
            end if;
            annotation(
              Documentation(info = "<html>
<p>This internal function calculates both the junction current and the junction conductance dependent from the given voltage.</p>
</html>"));
          end junction3;

          function junctionCapTransTime "Junction capacitance transittime calculation"
            extends Modelica.Icons.Function;
            input Modelica.SIunits.Capacitance capin "Input capacitance";
            input Modelica.SIunits.Voltage voltage "Input voltage";
            input Modelica.SIunits.Voltage depcap;
            input Real mj;
            input Real phij;
            input Modelica.SIunits.Voltage f1;
            input Real f2;
            input Real f3;
            input Modelica.SIunits.Time transittime;
            input Modelica.SIunits.Conductance conduct "Input conductance";
            input Modelica.SIunits.Current current "Input current";
            output Modelica.SIunits.Capacitance capout "Output capacitance";
            output Modelica.SIunits.Charge charge "Output charge";
          algorithm
            (capout, charge) := junctionCap(capin, voltage, depcap, mj, phij, f1, f2, f3);
            capout := capout + transittime * conduct;
            charge := charge + transittime * current;
            annotation(
              Documentation(info = "<html>
<p>This internal function calculates the capacitance and the charge dependent on the transittime.</p>
</html>"));
          end junctionCapTransTime;

          function junction2 "Junction current and conductance calculation"
            extends Modelica.Icons.Function;
            input Modelica.SIunits.Voltage voltage "Input Voltage";
            input Modelica.SIunits.Temp_K temp "Device Temperature";
            input Real ncoeff;
            input Modelica.SIunits.Current satcur "Saturation current";
            output Modelica.SIunits.Current current "Output current";
            output Modelica.SIunits.Conductance cond "Output conductance";
          protected
            constant Real max_exp = 50.0;
            constant Real max_current = 1.0e4;
            Modelica.SIunits.Voltage vte;
            Real max_exponent;
            Real evd;
            Real arg;
          algorithm
            if satcur > 1.0e-101 then
              vte := Spice3.Internal.SpiceConstants.CONSTKoverQ * temp * ncoeff;
              max_exponent := Modelica.Math.log(max_current / satcur);
              max_exponent := min(max_exp, max_exponent);
              if voltage >= max_exponent * vte then
                evd := exp(max_exponent);
                cond := satcur * evd / vte;
                current := satcur * (evd - 1) + cond * (voltage - max_exponent * vte);
              elseif voltage >= (-5 * vte) then
                evd := exp(voltage / vte);
                current := satcur * (evd - 1) + Spice3.Internal.SpiceConstants.CKTgmin * voltage;
                cond := satcur * evd / vte + Spice3.Internal.SpiceConstants.CKTgmin;
              else
                arg := 3 * vte / (voltage * Spice3.Internal.SpiceConstants.CONSTe);
                arg := arg * arg * arg;
                current := (-1 * satcur * (1 + arg)) + Spice3.Internal.SpiceConstants.CKTgmin * voltage;
                cond := satcur * 3 * arg / voltage + Spice3.Internal.SpiceConstants.CKTgmin;
              end if;
            else
              current := 0.0;
              cond := 0.0;
            end if;
            annotation(
              Documentation(info = "<html>
<p>This internal function calculates both the junction current and the junction conductance dependent from the given voltage.</p>
</html>"));
          end junction2;

          function resDepTemp "Temperature dependent conductance"
            extends Modelica.Icons.Function;
            input Modelica.SIunits.Resistance resist "Input resistance";
            input Modelica.SIunits.Temp_K temp "Device temperature";
            input Modelica.SIunits.Temp_K tnom "Nominal temperature";
            input Real tc1;
            input Real tc2;
            output Modelica.SIunits.Conductance conduct "Output conductance";
            output Real dCond_dTemp "Output value";
          protected
            Real difference;
            Real factor;
          algorithm
            difference := temp - tnom;
            factor := 1.0 + tc1 * difference + tc2 * difference * difference;
            conduct := 1.0 / (resist * factor);
            dCond_dTemp := (tc1 + 2 * tc2 * difference) * conduct * conduct;
            annotation(
              Documentation(info = "<html>
<p>This internal function calculates the conductance in dependency from the temperature.</p>
</html>"));
          end resDepTemp;

          function resDepGeom "Resistance dependent from width and narrow"
            extends Modelica.Icons.Function;
            input Real rsh "Input sheet resistance";
            input Modelica.SIunits.Length width "Input transistor width";
            input Modelica.SIunits.Length length "Input transistor length";
            input Modelica.SIunits.Length narrow "Input narrow";
            output Real out "Output value";
          algorithm
            out := rsh * (length - narrow) / (width - narrow);
            annotation(
              Documentation(info = "<html>
<p>This internal function calculates the resistance in dependency from the geometrical values (width, narrow) and resistivity.</p>
</html>"));
          end resDepGeom;

          function saturationCurDepTempSPICE3JFET "Temperature dependency of saturation current"
            extends Modelica.Icons.Function;
            input SI.Current satcur0 "Saturation current";
            input SI.Temp_K temp "Device Temperature";
            input SI.Temp_K tnom "Nominal Temperature";
            output SI.Current ret "Output value";
          protected
            SI.Voltage vt;
          algorithm
            vt := Spice3.Internal.SpiceConstants.CONSTKoverQ * temp;
            ret := satcur0 * exp((temp / tnom - 1) * 1.11 / vt);
          end saturationCurDepTempSPICE3JFET;

          function capDepGeom "Capacitance dependent from width and narrow"
            extends Modelica.Icons.Function;
            input SI.CapacitancePerArea cap0 "Junction bottom capacitance";
            input SI.Permittivity capsw0 "Junction sidewall capacitance";
            input SI.Length width "Input capacitor width";
            input SI.Length length "Input capacitor length";
            input SI.Length narrow "Input narrow";
            output SI.Capacitance out "Output value";
          algorithm
            out := cap0 * (width - narrow) * (length - narrow) + capsw0 * 2 * (length - narrow + width - narrow);
          end capDepGeom;

          function energyGapDepTemp "Temperature dependency of energy gap"
            extends Modelica.Icons.Function;
            input SI.Temp_K temp "Temperature";
            output SI.Voltage ret "Output voltage";
          algorithm
            ret := Spice3.Internal.MaterialParameters.EnergyGapSi - Spice3.Internal.MaterialParameters.FirstBandCorrFactorSi * temp * temp / (temp + Spice3.Internal.MaterialParameters.SecondBandCorrFactorSi);
            annotation(
              Documentation(info = "<html>
<p>This internal function calculates the temperature dependent energy gap based on the actual temperature, and two coefficients given as input to the function.</p>
</html>"));
          end energyGapDepTemp;

          function junction2SPICE3BJT "Junction current and conductance calculation"
            extends Modelica.Icons.Function;
            input SI.Voltage voltage "Input Voltage";
            input SI.Temp_K temp "Device Temperature";
            input Real ncoeff1;
            input Real ncoeff2;
            input SI.Current satcur1 "Saturation current";
            input SI.Current satcur2 "Saturation current";
            output SI.Current current1 "Output current";
            output SI.Conductance cond1 "Output conductance";
            output SI.Current current2 "Output current";
            output SI.Conductance cond2 "Output conductance";
          protected
            SI.Voltage vte;
            SI.Voltage vtn;
            Real evd;
          algorithm
            vtn := Spice3.Internal.SpiceConstants.CONSTKoverQ * temp * ncoeff1;
            vte := Spice3.Internal.SpiceConstants.CONSTKoverQ * temp * ncoeff2;
            if voltage > (-5 * vtn) then
              evd := exp(voltage / vtn);
              current1 := satcur1 * (evd - 1) + Spice3.Internal.SpiceConstants.CKTgmin * voltage;
              cond1 := satcur1 * evd / vtn + Spice3.Internal.SpiceConstants.CKTgmin;
              if satcur2 == 0 then
                current2 := 0;
                cond2 := 0;
              else
                evd := exp(voltage / vte);
                current2 := satcur2 * (evd - 1);
                cond2 := satcur2 * evd / vte;
              end if;
            else
              cond1 := (-satcur1 / voltage) + Spice3.Internal.SpiceConstants.CKTgmin;
              current1 := cond1 * voltage;
              cond2 := -satcur2 / voltage;
              current2 := -satcur2;
            end if;
          end junction2SPICE3BJT;

          function energyGapDepTemp_old "Temperature dependency of energy gap"
            extends Modelica.Icons.Function;
            input Modelica.SIunits.Temp_K temp "Temperature";
            output Modelica.SIunits.Voltage ret "Output voltage";
          protected
            Modelica.SIunits.Voltage gap0 = 1.16;
            Real coeff1(final unit = "V/K") = 7.02e-4;
            Modelica.SIunits.Temp_K coeff2 = 1108.0;
          algorithm
            ret := gap0 - coeff1 * temp * temp / (temp + coeff2);
            annotation(
              Documentation(info = "<html>
<p>This internal function calculates the temperature dependent energy gap based on the actual temperature, and two coefficients given as input to the function.</p>
</html>"));
          end energyGapDepTemp_old;
          annotation(
            Documentation(info = "<html>
<p>The package Equation contains functions that are needed to model the semiconductor models. Some of these functions are used by several semiconductor models.</p>
</html>"));
        end Functions;

        package SpiceRoot "Basic records and functions"
          extends Modelica.Icons.InternalPackage;

          record SpiceRoot "Data for insertion to matrices"
            extends Modelica.Icons.Record;
            Modelica.SIunits.Current[6] m_pCurrentValues(start = zeros(6));
            Real[36] m_pResJacobi(start = zeros(36));
            Real[36] m_pCapJacobi(start = zeros(36));
            annotation(
              Documentation(info = "<html>
<p>This data are to collect intermediate results to be inserted into SPICE-like matrices for linear systems of equations.</p>
</html>"));
          end SpiceRoot;

          function useInitialConditions "Initial condition handling"
            extends Modelica.Icons.Function;
            output Boolean ret;
          algorithm
            ret := false;
            annotation(
              Documentation(info = "<html>
<p>This function useInitialConditions appoints whether the initial conditions that are given in the description are used or not.</p>
</html>"));
          end useInitialConditions;

          function initJunctionVoltages "Choice of junction voltage handling, obsolete, use initJunctionVoltageRevised"
            extends Modelica.Icons.Function;
            extends Modelica.Icons.ObsoleteModel;
            output Boolean ret;
          algorithm
            ret := false;
            annotation(
              Documentation(info = "<html>
<p>This internal function is provided to choose the junction voltage handling which is at the current library version fixed to false.</p>
</html>"));
          end initJunctionVoltages;

          function limitJunctionVoltage "Limitation of junction voltage, obsolete, use limitJunctionVoltageRevised"
            extends Modelica.Icons.Function;
            extends Modelica.Icons.ObsoleteModel;
            input Modelica.SIunits.Voltage voltage "Input voltage";
            output Modelica.SIunits.Voltage ret;
          algorithm
            ret := voltage;
            annotation(
              Documentation(info = "<html>
<p>This internal function is provided to limit the junction voltage which is at the current library version fixed to false.</p>
</html>"));
          end limitJunctionVoltage;

          function limitJunctionVoltageRevised "Limitation of junction voltage"
            extends Modelica.Icons.Function;
            input SI.Voltage voltage "Input voltage";
            output SI.Voltage ret;
          algorithm
            ret := voltage;
            annotation(
              Documentation(info = "<html>
<p>This internal function is provided to limit the junction voltage which is at the current library version fixed to false.</p>
</html>"));
          end limitJunctionVoltageRevised;

          function initJunctionVoltagesRevised "Choice of junction voltage handling"
            extends Modelica.Icons.Function;
            output Boolean ret;
          algorithm
            ret := false;
            annotation(
              Documentation(info = "<html>
<p>This internal function is provided to choose the junction voltage handling which is at the current library version fixed to false.</p>
</html>"));
          end initJunctionVoltagesRevised;
          annotation(
            Documentation(info = "<html>
<p>The package SpiceRoot contains basic records and functions that are needed in SPICE3.</p>
</html>"));
        end SpiceRoot;

        package Model "Device Temperature"
          extends Modelica.Icons.InternalPackage;

          record Model "Device Temperature"
            extends Modelica.Icons.Record;
            Modelica.SIunits.Temp_K m_dTemp(start = SpiceConstants.CKTnomTemp) "TEMP, Device Temperature";
            annotation(
              Documentation(info = "<html>
<p>The record Model includes the device temperature which has a default value of 27&deg;C.</p>
</html>"));
          end Model;
          annotation(
            Documentation(info = "<html>
<p>The package Model contains the record Model that includes the device temperature.</p>
</html>"));
        end Model;

        package Mosfet "Functions and records for MOSFETs"
          extends Modelica.Icons.InternalPackage;

          record Mosfet "Record for Mosfet parameters"
            extends Spice3.Internal.Model.Model;
            Modelica.SIunits.Length m_len(start = 1e-4) "L, length of channel region";
            Modelica.SIunits.Length m_width(start = 1e-4) "W, width of channel region";
            Modelica.SIunits.Area m_drainArea(start = Spice3.Internal.SpiceConstants.CKTdefaultMosAD) "AD, area of drain diffusion";
            Modelica.SIunits.Area m_sourceArea(start = Spice3.Internal.SpiceConstants.CKTdefaultMosAS) "AS, area of source diffusion";
            Real m_drainSquares(start = 1.0) "NRD, length of drain in squares";
            Real m_sourceSquares(start = 1.0) "NRS, length of source in squares";
            Modelica.SIunits.Length m_drainPerimeter(start = 0.0) "PD, Drain perimeter";
            Modelica.SIunits.Length m_sourcePerimeter(start = 0.0) "PS, Source perimeter";
            Modelica.SIunits.Voltage m_dICVDS(start = 0.0) "IC_VDS, Initial D-S voltage";
            Real m_dICVDSIsGiven "IC_VDS, IsGivenValue";
            Modelica.SIunits.Voltage m_dICVGS(start = 0.0) "IC_VGS, Initial G-S voltage";
            Real m_dICVGSIsGiven "IC_VGS, IsGivenValue";
            Modelica.SIunits.Voltage m_dICVBS(start = 0.0) "IC_VBS, Initial B-S voltage";
            Real m_dICVBSIsGiven "IC_VBS, IsGivenValue";
            Integer m_off(start = 0) "Device initially off, non-zero to indicate device is off for dc analysis";
            //----------------------obsolete-----------------------------------
            Integer m_bPMOS(start = 0) "P type MOSFET model";
            Integer m_nLevel(start = 1) "MOS model level";
            Modelica.SIunits.Length m_drainPerimiter(start = 0.0) "PD, Drain perimeter";
            Modelica.SIunits.Length m_sourcePerimiter(start = 0.0) "PS, Source perimeter";
            //-----------------------------------------------------------------
            Boolean m_uic;
            annotation(
              Documentation(info = "<html>
<p>This record Mosfet contains parameters that are used for all types of Mosfet transistors in SPICE3.</p>
</html>"));
          end Mosfet;

          record MosfetModelLineParams "Record for Mosfet model line parameters"
            extends Modelica.Icons.Record;
            Real m_jctSatCurDensity(start = 0.0) "JS, Bulk jct. sat. current density, input - use tSatCurDens";
            Modelica.SIunits.Resistance m_sheetResistance(start = 0.0) "RSH, Sheet resistance";
            Real m_bulkJctPotential(start = 0.8) "PB, Bulk junction potential, input - use tBulkPot";
            Modelica.SIunits.LinearTemperatureCoefficient m_bulkJctBotGradingCoeff(start = 0.5) "MJ, Bottom grading coefficient";
            //unit checked by maj
            Modelica.SIunits.LinearTemperatureCoefficient m_bulkJctSideGradingCoeff(start = 0.5) "MJSW, Side grading coefficient";
            //unit checked by maj
            Real m_oxideThickness(start = 1.0e-7) "TOX, Oxide thickness unit: micron";
            //--------------------------obsolete------------------------
            Real m_oxideThicknessIsGiven "TOX, IsGiven value";
            //-----------------------------------------------------------
            Real m_gateSourceOverlapCapFactor(start = 0.0) "CGS0, Gate-source overlap cap";
            Real m_gateDrainOverlapCapFactor(start = 0.0) "CGD0, Gate-drain overlap cap";
            Real m_gateBulkOverlapCapFactor(start = 0.0) "CGB0, Gate-bulk overlap cap";
            Real m_fNcoef(start = 0.0) "KF, Flicker noise coefficient";
            Real m_fNexp(start = 1.0) "AF, Flicker noise exponent";
            Real m_mjswIsGiven "MJSW, IsGivenValue";
            Real m_cgsoIsGiven "CGSO, IsGivenValue";
            Real m_cgdoIsGiven "CGDO, IsGivenValue";
            Real m_cgboIsGiven "CGBO, IsGivenValue";
            Real m_pbIsGiven "PB, IsGivenValue";
            annotation(
              Documentation(info = "<html>
<p>This record MosfetModelLineParams contains the model line parameters that are used for all kinds of Mosfet transistors in SPICE3.</p>
</html>"));
          end MosfetModelLineParams;

          record MosfetModelLine "Type of the transistor"
            extends Modelica.Icons.Record;
            Integer m_type(start = 1) "device type : 1 = n,  -1 = p";
            annotation(
              Documentation(info = "<html>
<p>This record contains only one variable and it provides the information on the transistor type (PMOS or nmos).</p>
</html>"));
          end MosfetModelLine;

          record MosfetCalc "Mosfet Variables"
            extends Modelica.Icons.Record;
            Modelica.SIunits.Voltage m_vds "Vds, Drain-Source voltage";
            Modelica.SIunits.Voltage m_vgs "Vgs, Gate-Source voltage";
            Modelica.SIunits.Voltage m_vbs "Vbs, Bulk-Source voltage";
            Modelica.SIunits.Current m_cbs "Ibs, B-S junction current";
            Modelica.SIunits.Conductance m_gbs "Gbs, Bulk-Source conductance";
            Modelica.SIunits.Current m_cbd "Ibd, B-D junction current";
            Modelica.SIunits.Conductance m_gbd "Gbd, Bulk-Drain conductance";
            Modelica.SIunits.Current m_cdrain "Ids";
            Modelica.SIunits.Conductance m_gds "Gds, Drain-Source conductance";
            Modelica.SIunits.Transconductance m_gm "Gm, Transconductance";
            Modelica.SIunits.Transconductance m_gmbs "Gmbs, Bulk-Source transconductance";
            Modelica.SIunits.Capacitance m_capbsb "Cbsb";
            Modelica.SIunits.Charge m_chargebsb "Qbsb";
            Modelica.SIunits.Capacitance m_capbss "Cbss";
            Modelica.SIunits.Charge m_chargebss "Qbss";
            Modelica.SIunits.Capacitance m_capbdb "Cbdb";
            Modelica.SIunits.Charge m_chargebdb "Qbdb";
            Modelica.SIunits.Capacitance m_capbds "Cbds";
            Modelica.SIunits.Charge m_chargebds "Qbds";
            Real m_Beta "Beta";
            Modelica.SIunits.Capacitance m_capGSovl "Cgso, Gate-source overlap cap.";
            Modelica.SIunits.Capacitance m_capGDovl "Cgdo, Gate-drain overlap cap.";
            Modelica.SIunits.Capacitance m_capGBovl "Cgbo, Gate-bulk overlap cap.";
            Modelica.SIunits.Capacitance m_capOx "Cox";
            Modelica.SIunits.Voltage m_von "Von, Turn-on voltage";
            Modelica.SIunits.Voltage m_vdsat "Vdsat";
            Integer m_mode(start = 1) "Mode";
            Modelica.SIunits.Length m_lEff;
            Modelica.SIunits.Resistance m_sourceResistance "Rs";
            Modelica.SIunits.Resistance m_drainResistance "Rd";
            annotation(
              Documentation(info = "<html>
<p>This record MosfetCalc contains variables that are needed for calculation within modeling the semiconductor models.</p>
</html>"));
          end MosfetCalc;

          function mosfetInitEquations "MOSFET initial precalculations"
            extends Modelica.Icons.Function;
            input Mosfet in_m "Input parameter set";
            output Mosfet out_m "Changed parameter set";
          algorithm
            out_m := in_m;
            if out_m.m_drainSquares == 0 then
              out_m.m_drainSquares := 1.;
            end if;
            if out_m.m_sourceSquares == 0 then
              out_m.m_sourceSquares := 1.;
            end if;
            annotation(
              Documentation(info = "<html>
<p>This function initially precalculates some values for transistor area which can be used for all transistor models.</p>
</html>"));
          end mosfetInitEquations;

          function mosfetModelLineInitEquations "Type transcription"
            extends Modelica.Icons.Function;
            input Mosfet in_m "Input parameter set";
            output MosfetModelLine out_ml "Changed parameter set";
          algorithm
            out_ml.m_type := if in_m.m_bPMOS > 0.5 then -1 else 1;
// -1: PMOS ; 1: NMOS
            annotation(
              Documentation(info = "<html>
<p>In this function during the initialization phase the transistor type is transcribed to another parameter for further usage.</p>
</html>"));
          end mosfetModelLineInitEquations;

          function getNumberOfElectricalPins "Number of Pins"
            extends Modelica.Icons.Function;
            output Integer ret "Number of pins";
          algorithm
            ret := 4;
            annotation(
              Documentation(info = "<html>
<p>This function getNumberOfElectricalPins identifies the number of electrical pins. At the current library version it is fixed to 4.</p>
</html>"));
          end getNumberOfElectricalPins;

          function mosfetRenameParametersDev "Device parameter renaming to internal names"
            extends Modelica.Icons.Function;
            input SI.Length W "Channel Width";
            input SI.Length L "Channel Length";
            input SI.Area AD "Area of the drain diffusion";
            input SI.Area AS "Area of the source diffusion";
            input SI.Length PD "Perimeter of the drain junction";
            input SI.Length PS "Perimeter of the source junction";
            input Real NRD "Number of squares of the drain diffusions";
            input Real NRS "Number of squares of the source diffusions";
            input Integer OFF "Optional initial condition: 0 - IC not used, 1 - IC used, not implemented yet";
            input SI.Voltage IC_VDS "Initial condition value VDS, not implemented yet";
            input SI.Voltage IC_VGS "Initial condition value VGS, not implemented yet";
            input SI.Voltage IC_VBS "Initial condition value VBS, not implemented yet";
            input Boolean UIC "Use initial condition, UIC";
            input SI.Temp_C TEMP "Temperature";
            output Modelica.Electrical.Spice3.Internal.Mosfet.Mosfet dev "Output record Mosfet";
          algorithm
/*device parameters*/
            dev.m_len := L "L, length of channel region";
            dev.m_width := W "W, width of channel region";
            dev.m_drainArea := AD "AD, area of drain diffusion";
            dev.m_sourceArea := AS "AS, area of source diffusion";
            dev.m_drainSquares := NRD "NRD, length of drain in squares";
            dev.m_sourceSquares := NRS "NRS, length of source in squares";
            dev.m_drainPerimeter := PD "PD, Drain perimeter";
            dev.m_sourcePerimeter := PS "PS, Source perimeter";
            dev.m_dICVDSIsGiven := if IC_VDS > (-1e40) then 1 else 0 "IC_VDS IsGivenValue";
            dev.m_dICVDS := if IC_VDS > (-1e40) then IC_VDS else 0 "Initial condition of VDS";
            dev.m_dICVGSIsGiven := if IC_VGS > (-1e40) then 1 else 0 "IC_VGS IsGivenValue";
            dev.m_dICVGS := if IC_VGS > (-1e40) then IC_VGS else 0 "Initial condition of VGS";
            dev.m_dICVBSIsGiven := if IC_VBS > (-1e40) then 1 else 0 "IC_VBS IsGivenValue";
            dev.m_dICVBS := if IC_VBS > (-1e40) then IC_VBS else 0 "Initial condition of VBS";
            dev.m_off := OFF "Non-zero to indicate device is off for dc analysis";
            dev.m_uic := UIC "Use initial conditions";
            dev.m_dTemp := TEMP + Spice3.Internal.SpiceConstants.CONSTCtoK "Device temperature";
            annotation(
              Documentation(info = "<html>
<pre>This function mos1RenameParametersDev assigns the external (given by the user) device parameters to the internal parameters. It also does the analysis of the IsGiven values (level 1).</pre>
</html>"));
          end mosfetRenameParametersDev;
          annotation(
            Documentation(info = "<html>
<p>The package Mosfet contains all functions and records that are used for all types of Mosfet transistors in SPICE3.</p>
</html>"));
        end Mosfet;

        package Mos "Records and functions for MOSFETs level 1,2,3,6"
          extends Modelica.Icons.InternalPackage;

          record MosModelLineParams "Record for Mosfet model line parameters (for level 1, 2, 3 and 6)"
            extends Spice3.Internal.Mosfet.MosfetModelLineParams;
            Real m_oxideCapFactor(start = 0.0);
            Modelica.SIunits.Voltage m_vt0(start = 0.0) "VTO, Threshold voltage";
            Real m_vtOIsGiven "VTO IsGivenValue";
            Modelica.SIunits.Capacitance m_capBD(start = 0.0) "CBD, B-D junction capacitance";
            Real m_capBDIsGiven "CapBD IsGivenValue";
            Modelica.SIunits.Capacitance m_capBS(start = 0.0) "CBS, B-S junction capacitance";
            Real m_capBSIsGiven "CapBS IsGivenValue";
            Modelica.SIunits.CapacitancePerArea m_bulkCapFactor(start = 0.0) "CJ, Bottom junction cap per area";
            Real m_bulkCapFactorIsGiven "Bulk cap factor IsGivenValue";
            Modelica.SIunits.Permittivity m_sideWallCapFactor(start = 0.0) "CJSW, Side grading coefficient";
            Real m_fwdCapDepCoeff(start = 0.5) "FC, Forward bias junction fit parameter";
            Modelica.SIunits.Voltage m_phi(start = 0.6) "PHI, Surface potential";
            Real m_phiIsGiven "Phi IsGivenValue";
            Modelica.SIunits.Voltage m_gamma(start = 0.0) "GAMMA, Bulk threshold parameter";
            Real m_gammaIsGiven "Gamma IsGivenValue";
            Modelica.SIunits.InversePotential m_lambda "Channel-length modulation";
            Real m_substrateDoping(start = 0.0) "NSUB, Substrate doping";
            Real m_substrateDopingIsGiven "Substrate doping IsGivenValue";
            Real m_gateType(start = 1.0) "TPG, Gate type";
            Modelica.SIunits.Conversions.NonSIunits.PerArea_cm m_surfaceStateDensity(start = 0.0) "NSS, Gate type";
            //-----------------obsolete--------------------------------------------
            Real m_surfaceStateDensityIsGiven(start = 0) "surfaceStateDensityIsGivenValue";
            //---------------------------------------------------------------------
            Modelica.SIunits.Conversions.NonSIunits.Area_cmPerVoltageSecond m_surfaceMobility(start = 600.0) "UO, Surface mobility";
            Modelica.SIunits.Length m_latDiff(start = 0.0) "LD, Lateral diffusion";
            Modelica.SIunits.Current m_jctSatCur(start = 1.0e-14) "IS, Bulk junction sat. current";
            Modelica.SIunits.Resistance m_drainResistance(start = 0) "RD, Drain ohmic resistance";
            Real m_drainResistanceIsGiven "Drain resistance IsGivenValue";
            Modelica.SIunits.Resistance m_sourceResistance(start = 0) "RS, Source ohmic resistance";
            Real m_sourceResistanceIsGiven "Source resistance IsGivenValue";
            Modelica.SIunits.Transconductance m_transconductance "input - use tTransconductance";
            Real m_transconductanceIsGiven "Transconductance IsGivenValue";
            Modelica.SIunits.Temp_K m_tnom(start = Spice3.Internal.SpiceConstants.CKTnomTemp) "TNOM, Parameter measurement temperature";
            annotation(
              Documentation(info = "<html>
<p>This record MosModelLineParams contains the model line parameters that are used for the MOSFET transistors level 1, 2, 3 and 6 in SPICE3.</p>
</html>"));
          end MosModelLineParams;

          record MosModelLineVariables "Record for Mosfet model line variables (for level 1)"
            extends Modelica.Icons.Record;
            Real m_oxideCapFactor;
            Modelica.SIunits.Voltage m_vt0;
            Modelica.SIunits.Voltage m_phi;
            Real m_gamma;
            Modelica.SIunits.Transconductance m_transconductance;
            annotation(
              Documentation(info = "<html>
<p>This record MosModelLineVariables contains the model line variables that are used for the MOSFET transistors level 1 SPICE3.</p>
</html>"));
          end MosModelLineVariables;

          record MosCalc "Further MOSFET variables (for level 1, 2, 3 and 6)"
            extends Spice3.Internal.Mosfet.MosfetCalc;
            SI.Transconductance m_tTransconductance(start = 0.);
            SI.Conversions.NonSIunits.Area_cmPerVoltageSecond m_tSurfMob(start = 0.);
            SI.Voltage m_tPhi(start = 0.7);
            SI.Voltage m_tVto(start = 1.);
            SI.CurrentDensity m_tSatCurDens(start = 0.);
            SI.Current m_tDrainSatCur(start = 0.);
            SI.Current m_tSourceSatCur(start = 0.);
            SI.Capacitance m_tCBDb(start = 0.);
            SI.Capacitance m_tCBDs(start = 0.);
            SI.Capacitance m_tCBSb(start = 0.);
            SI.Capacitance m_tCBSs(start = 0.);
            SI.CapacitancePerArea m_tCj(start = 0.);
            SI.Permittivity m_tCjsw(start = 0.);
            SI.Voltage m_tBulkPot(start = 0.7);
            SI.Voltage m_tDepCap(start = 0.35);
            SI.Voltage m_tVbi(start = 1.);
            SI.Voltage m_VBScrit(start = 0.7);
            SI.Voltage m_VBDcrit(start = 0.7);
            SI.Voltage m_f1b(start = 0.);
            Real m_f2b(start = 0.);
            Real m_f3b(start = 0.);
            SI.Voltage m_f1s(start = 0.);
            Real m_f2s(start = 0.);
            Real m_f3s(start = 0.);
            SI.Voltage m_dVt(start = 0.);
            SI.Capacitance m_capgd(start = 0.);
            SI.Capacitance m_capgs(start = 0.);
            SI.Capacitance m_capgb(start = 0.);
            SI.Charge m_qgs(start = 0.);
            SI.Charge m_qgd(start = 0.);
            SI.Charge m_qgb(start = 0.);
            annotation(
              Documentation(info = "<html>
<pre>This record MosCalc contains further MOSFET variables (for level 1, 2, 3 and 6).</pre>
</html>"));
          end MosCalc;

          record DEVqmeyer "Meyer capacities and charge"
            extends Modelica.Icons.Record;
            Modelica.SIunits.Capacitance qm_capgb(start = 0);
            Modelica.SIunits.Capacitance qm_capgs(start = 0);
            Modelica.SIunits.Capacitance qm_capgd(start = 0);
            Modelica.SIunits.Charge qm_qgs(start = 0);
            Modelica.SIunits.Charge qm_qgb(start = 0);
            Modelica.SIunits.Charge qm_qgd(start = 0);
            Modelica.SIunits.Voltage qm_vgs(start = 0);
            Modelica.SIunits.Voltage qm_vgb(start = 0);
            Modelica.SIunits.Voltage qm_vgd(start = 0);
            annotation(
              Documentation(info = "<html>
<p>This record DEVqmeyer contains values that are needed for the calculation of the Meyer capacities and charge.</p>
</html>"));
          end DEVqmeyer;

          record CurrrentsCapacitances "Currents and Capacities"
            extends Modelica.Icons.Record;
            Modelica.SIunits.Current idrain(start = 0);
            Modelica.SIunits.Current iBD(start = 0);
            Modelica.SIunits.Current iBS(start = 0);
            Modelica.SIunits.Capacitance cGS(start = 0);
            Modelica.SIunits.Capacitance cGB(start = 0);
            Modelica.SIunits.Capacitance cGD(start = 0);
            Modelica.SIunits.Capacitance cBS(start = 0);
            Modelica.SIunits.Capacitance cBD(start = 0);
            Modelica.SIunits.Capacitance m_capgd;
            annotation(
              Documentation(info = "<html>
<p>This record CurrentsCapacities contains values for the currents and the capacities inside the MOSFET models level 1, 2, 3 and 6.</p>
</html>"));
          end CurrrentsCapacitances;

          function mosCalcInitEquations "Mosfet initial precalculations (level 1)"
            extends Modelica.Icons.Function;
            input Spice3.Internal.Mos1.Mos1ModelLineParams in_p "Input record model line parameters for MOS1";
            input Spice3.Internal.SpiceConstants in_C "Input record SPICE constants";
            input MosModelLineVariables in_vp "Input record model line variables";
            input Spice3.Internal.Mosfet.Mosfet in_m "Input record MOSFET parameters";
            output Spice3.Internal.Mos1.Mos1Calc out_c "Output record Mos1 calculated values";
          algorithm
            out_c.m_drainResistance := if in_p.m_drainResistanceIsGiven > 0.5 then in_p.m_drainResistance else in_p.m_sheetResistance * in_m.m_drainSquares;
            out_c.m_sourceResistance := if in_p.m_sourceResistanceIsGiven > 0.5 then in_p.m_sourceResistance else in_p.m_sheetResistance * in_m.m_sourceSquares;
            out_c.m_lEff := in_m.m_len - 2 * in_p.m_latDiff;
            if abs(out_c.m_lEff) < 1e-18 then
              out_c.m_lEff := 1e-6;
            end if;
            out_c.m_capGSovl := in_p.m_gateSourceOverlapCapFactor * in_m.m_width;
            out_c.m_capGDovl := in_p.m_gateDrainOverlapCapFactor * in_m.m_width;
            out_c.m_capGBovl := in_p.m_gateBulkOverlapCapFactor * out_c.m_lEff;
            out_c.m_capOx := in_vp.m_oxideCapFactor * out_c.m_lEff * in_m.m_width;
            out_c.m_tTransconductance := 0;
            out_c.m_tSurfMob := 0;
            out_c.m_tPhi := 0.7;
            out_c.m_tVto := 1.;
            out_c.m_tSatCurDens := 0;
            out_c.m_tDrainSatCur := 0;
            out_c.m_tSourceSatCur := 0;
            out_c.m_tCBDb := 0;
            out_c.m_tCBDs := 0;
            out_c.m_tCBSb := 0;
            out_c.m_tCBSs := 0;
            out_c.m_tCj := 0;
            out_c.m_tCjsw := 0;
            out_c.m_tBulkPot := 0.7;
            out_c.m_tDepCap := 0.35;
            out_c.m_tVbi := 1.;
            out_c.m_VBScrit := 0.7;
            out_c.m_VBDcrit := 0.7;
            out_c.m_f1b := 0;
            out_c.m_f2b := 0;
            out_c.m_f3b := 0;
            out_c.m_f1s := 0;
            out_c.m_f2s := 0;
            out_c.m_f3s := 0;
            out_c.m_dVt := 0;
            out_c.m_capgd := 0;
            out_c.m_capgs := 0;
            out_c.m_capgb := 0;
            out_c.m_qgs := 0;
            out_c.m_qgd := 0;
            out_c.m_qgb := 0;
            out_c.m_vds := 0;
            out_c.m_vgs := 0;
            out_c.m_vbs := 0;
            out_c.m_cbs := 0;
            out_c.m_gbs := 0;
            out_c.m_cbd := 0;
            out_c.m_gbd := 0;
            out_c.m_cdrain := 0;
            out_c.m_gds := 0;
            out_c.m_gm := 0;
            out_c.m_gmbs := 0;
            out_c.m_capbsb := 0;
            out_c.m_chargebsb := 0;
            out_c.m_capbss := 0;
            out_c.m_chargebss := 0;
            out_c.m_capbdb := 0;
            out_c.m_chargebdb := 0;
            out_c.m_capbds := 0;
            out_c.m_chargebds := 0;
            out_c.m_Beta := 0;
            out_c.m_von := 0;
            out_c.m_vdsat := 0;
            out_c.m_mode := 1;
            annotation(
              Documentation(info = "<html>
<p>This function mosCalcInitEquations does the initial precalculation of the MOSFET parameters (level 1).</p>
</html>"));
          end mosCalcInitEquations;

          function mosCalcCalcTempDependencies "Precalculation relating to temperature"
            extends Modelica.Icons.Function;
            input Spice3.Internal.Mos1.Mos1ModelLineParams in_p "Input record model line parameters for MOS1";
            input Spice3.Internal.SpiceConstants in_C "Input record SPICE constants";
            input MosModelLineVariables in_vp "Input record model line variables";
            input Spice3.Internal.Mosfet.Mosfet in_m "Input record MOSFET parameters";
            input Spice3.Internal.Mos1.Mos1Calc in_c "Input record Mos1Calc";
            input Integer in_m_type "Type of MOS transistor";
            output Spice3.Internal.Mos1.Mos1Calc out_c "Output record with calculated values";
          protected
            Real ratio;
            Real ratio4;
            Real res;
          algorithm
            out_c := in_c;
            ratio := in_m.m_dTemp / in_p.m_tnom;
            ratio4 := ratio * sqrt(ratio);
            out_c.m_tTransconductance := in_vp.m_transconductance / ratio4;
            out_c.m_Beta := out_c.m_tTransconductance * in_m.m_width / out_c.m_lEff;
            out_c.m_tSurfMob := in_p.m_surfaceMobility / ratio4;
            out_c.m_tPhi := Spice3.Internal.Functions.junctionPotDepTemp(in_vp.m_phi, in_m.m_dTemp, in_p.m_tnom);
            out_c.m_tVbi := in_vp.m_vt0 - in_m_type * (in_vp.m_gamma * sqrt(in_vp.m_phi)) + 0.5 * (Spice3.Internal.Functions.energyGapDepTemp_old(in_p.m_tnom) - Spice3.Internal.Functions.energyGapDepTemp_old(in_m.m_dTemp)) + in_m_type * 0.5 * (out_c.m_tPhi - in_vp.m_phi);
            out_c.m_tVto := out_c.m_tVbi + in_m_type * in_vp.m_gamma * sqrt(out_c.m_tPhi);
            out_c.m_tBulkPot := Spice3.Internal.Functions.junctionPotDepTemp(in_p.m_bulkJctPotential, in_m.m_dTemp, in_p.m_tnom);
            out_c.m_tDepCap := in_p.m_fwdCapDepCoeff * out_c.m_tBulkPot;
            if in_p.m_jctSatCurDensity == 0.0 or in_m.m_sourceArea == 0.0 or in_m.m_drainArea == 0.0 then
              out_c.m_tDrainSatCur := Spice3.Internal.Functions.saturationCurDepTempSPICE3MOSFET(in_p.m_jctSatCur, in_m.m_dTemp, in_p.m_tnom);
              out_c.m_tSourceSatCur := out_c.m_tDrainSatCur;
              out_c.m_VBScrit := Spice3.Internal.Functions.junctionVCrit(in_m.m_dTemp, 1.0, out_c.m_tSourceSatCur);
              out_c.m_VBDcrit := out_c.m_VBScrit;
            else
              out_c.m_tSatCurDens := Spice3.Internal.Functions.saturationCurDepTempSPICE3MOSFET(in_p.m_jctSatCurDensity, in_m.m_dTemp, in_p.m_tnom);
              out_c.m_tDrainSatCur := out_c.m_tSatCurDens * in_m.m_drainArea;
              out_c.m_tSourceSatCur := out_c.m_tSatCurDens * in_m.m_sourceArea;
              out_c.m_VBScrit := Spice3.Internal.Functions.junctionVCrit(in_m.m_dTemp, 1.0, out_c.m_tSourceSatCur);
              out_c.m_VBDcrit := Spice3.Internal.Functions.junctionVCrit(in_m.m_dTemp, 1.0, out_c.m_tDrainSatCur);
            end if;
            if not in_p.m_capBDIsGiven > 0.5 or not in_p.m_capBSIsGiven > 0.5 then
              (res, out_c.m_tCj) := Spice3.Internal.Functions.junctionParamDepTempSPICE3(in_p.m_bulkJctPotential, in_p.m_bulkCapFactor, in_p.m_bulkJctBotGradingCoeff, in_m.m_dTemp, in_p.m_tnom);
              (res, out_c.m_tCjsw) := Spice3.Internal.Functions.junctionParamDepTempSPICE3(in_p.m_bulkJctPotential, in_p.m_sideWallCapFactor, in_p.m_bulkJctSideGradingCoeff, in_m.m_dTemp, in_p.m_tnom);
              (out_c.m_f1s, out_c.m_f2s, out_c.m_f3s) := Spice3.Internal.Functions.junctionCapCoeffs(in_p.m_bulkJctSideGradingCoeff, in_p.m_fwdCapDepCoeff, out_c.m_tBulkPot);
            end if;
            if in_p.m_capBDIsGiven > 0.5 then
              (res, out_c.m_tCBDb) := Spice3.Internal.Functions.junctionParamDepTempSPICE3(in_p.m_bulkJctPotential, in_p.m_capBD, in_p.m_bulkJctBotGradingCoeff, in_m.m_dTemp, in_p.m_tnom);
              out_c.m_tCBDs := 0.0;
            else
              out_c.m_tCBDb := out_c.m_tCj * in_m.m_drainArea;
              out_c.m_tCBDs := out_c.m_tCjsw * in_m.m_drainPerimeter;
            end if;
            if in_p.m_capBSIsGiven > 0.5 then
              (res, out_c.m_tCBSb) := Spice3.Internal.Functions.junctionParamDepTempSPICE3(in_p.m_bulkJctPotential, in_p.m_capBS, in_p.m_bulkJctBotGradingCoeff, in_m.m_dTemp, in_p.m_tnom);
              out_c.m_tCBSs := 0.0;
            else
              out_c.m_tCBSb := out_c.m_tCj * in_m.m_sourceArea;
              out_c.m_tCBSs := out_c.m_tCjsw * in_m.m_sourcePerimeter;
            end if;
            (out_c.m_f1b, out_c.m_f2b, out_c.m_f3b) := Spice3.Internal.Functions.junctionCapCoeffs(in_p.m_bulkJctBotGradingCoeff, in_p.m_fwdCapDepCoeff, out_c.m_tBulkPot);
            out_c.m_dVt := in_m.m_dTemp * Spice3.Internal.SpiceConstants.CONSTKoverQ;
            annotation(
              Documentation(info = "<html>
<p>This function mosCalcCalcTempDependencies does precalculation relating to the temperature (level 1).</p>
</html>"));
          end mosCalcCalcTempDependencies;

          function mosCalcNoBypassCode "Calculation of currents and capacities (level 1)"
            extends Modelica.Icons.Function;
            input Spice3.Internal.Mosfet.Mosfet in_m "Input record MOSFET parameters";
            input Integer in_m_type "Type of MOS transistor";
            input Spice3.Internal.Mos1.Mos1Calc in_c "Input record Mos1Calc";
            input Spice3.Internal.Mos1.Mos1ModelLineParams in_p "Input record model line parameters for MOS1";
            input Spice3.Internal.SpiceConstants in_C "Input record SPICE constants";
            input MosModelLineVariables in_vp "Input record model line variables";
            input Boolean in_m_bInit;
            input Modelica.SIunits.Voltage[4] in_m_pVoltageValues;
            /* gate bulk drain source */
            output CurrrentsCapacitances out_cc;
          protected
            Modelica.SIunits.Voltage vbd;
            Modelica.SIunits.Voltage vgd;
            Modelica.SIunits.Voltage vgb;
            Modelica.SIunits.Current cur;
            Integer n;
            DEVqmeyer qm;
            Spice3.Internal.Mos1.Mos1Calc int_c;
            Real hlp;
          algorithm
            int_c := in_c;
            out_cc.m_capgd := 0;
            int_c.m_vgs := in_m_type * (in_m_pVoltageValues[1] - in_m_pVoltageValues[4]);
// ( G , SP)
            int_c.m_vbs := in_m_type * (in_m_pVoltageValues[2] - in_m_pVoltageValues[4]);
// ( B , SP)
            int_c.m_vds := in_m_type * (in_m_pVoltageValues[3] - in_m_pVoltageValues[4]);
// ( DP, SP)
            if Spice3.Internal.SpiceRoot.useInitialConditions() and in_m.m_dICVBSIsGiven > 0.5 then
              int_c.m_vbs := in_m_type * in_m.m_dICVBS;
            elseif Spice3.Internal.SpiceRoot.initJunctionVoltages() then
              int_c.m_vbs := if in_m.m_off > 0.5 then 0. else int_c.m_VBScrit;
            end if;
            if Spice3.Internal.SpiceRoot.useInitialConditions() and in_m.m_dICVDSIsGiven > 0.5 then
              int_c.m_vds := in_m_type * in_m.m_dICVDS;
            elseif Spice3.Internal.SpiceRoot.initJunctionVoltages() then
              int_c.m_vds := if in_m.m_off > 0.5 then 0. else int_c.m_VBDcrit - int_c.m_VBScrit;
            end if;
            if Spice3.Internal.SpiceRoot.useInitialConditions() and in_m.m_dICVGSIsGiven > 0.5 then
              int_c.m_vgs := in_m_type * in_m.m_dICVGS;
            elseif Spice3.Internal.SpiceRoot.initJunctionVoltages() then
              if in_m.m_off > 0.5 then
                int_c.m_vgs := 0.;
              end if;
            end if;
            vbd := int_c.m_vbs - int_c.m_vds;
            vgd := int_c.m_vgs - int_c.m_vds;
            if int_c.m_vds >= 0 then
              vbd := int_c.m_vbs - int_c.m_vds;
            else
              int_c.m_vbs := vbd + int_c.m_vds;
            end if;
            vgb := int_c.m_vgs - int_c.m_vbs;
            (int_c.m_cbd, int_c.m_gbd) := Spice3.Internal.Functions.junction2SPICE3MOSFET(int_c.m_cbd, int_c.m_gbd, vbd, in_m.m_dTemp, 1.0, int_c.m_tDrainSatCur);
            out_cc.iBD := in_m_type * int_c.m_cbd;
            (int_c.m_cbs, int_c.m_gbs) := Spice3.Internal.Functions.junction2SPICE3MOSFET(int_c.m_cbs, int_c.m_gbs, int_c.m_vbs, in_m.m_dTemp, 1.0, int_c.m_tSourceSatCur);
            out_cc.iBS := in_m_type * int_c.m_cbs;
            int_c.m_mode := if int_c.m_vds >= 0 then 1 else -1;
// 1: normal mode, -1: inverse mode
            if int_c.m_mode == 1 then
              int_c := Spice3.Internal.Mos1.drainCur(int_c.m_vbs, int_c.m_vgs, int_c.m_vds, int_c, in_p, in_C, in_vp, in_m_type);
            else
              int_c := Spice3.Internal.Mos1.drainCur(vbd, vgd, -int_c.m_vds, int_c, in_p, in_C, in_vp, in_m_type);
            end if;
            n := if int_c.m_mode == 1 then 6 else 5;
            out_cc.idrain := in_m_type * int_c.m_cdrain * int_c.m_mode;
            int_c.m_capbss := 0.0;
            int_c.m_chargebss := 0.0;
            int_c.m_capbds := 0.0;
            int_c.m_chargebds := 0.0;
            (int_c.m_capbsb, int_c.m_chargebsb) := Spice3.Internal.Functions.junctionCap(int_c.m_tCBSb, int_c.m_vbs, int_c.m_tDepCap, in_p.m_bulkJctBotGradingCoeff, int_c.m_tBulkPot, int_c.m_f1b, int_c.m_f2b, int_c.m_f3b);
            (int_c.m_capbdb, int_c.m_chargebdb) := Spice3.Internal.Functions.junctionCap(int_c.m_tCBDb, vbd, int_c.m_tDepCap, in_p.m_bulkJctBotGradingCoeff, int_c.m_tBulkPot, int_c.m_f1b, int_c.m_f2b, int_c.m_f3b);
            if not in_p.m_capBSIsGiven > 0.5 then
              (int_c.m_capbss, int_c.m_chargebss) := Spice3.Internal.Functions.junctionCap(int_c.m_tCBSs, int_c.m_vbs, int_c.m_tDepCap, in_p.m_bulkJctSideGradingCoeff, int_c.m_tBulkPot, int_c.m_f1s, int_c.m_f2s, int_c.m_f3s);
            end if;
            if not in_p.m_capBDIsGiven > 0.5 then
              (int_c.m_capbds, int_c.m_chargebds) := Spice3.Internal.Functions.junctionCap(int_c.m_tCBDs, vbd, int_c.m_tDepCap, in_p.m_bulkJctSideGradingCoeff, int_c.m_tBulkPot, int_c.m_f1s, int_c.m_f2s, int_c.m_f3s);
            end if;
            out_cc.cBS := if in_m_bInit then 1e-15 else int_c.m_capbsb + int_c.m_capbss;
            out_cc.cBD := if in_m_bInit then 1e-15 else int_c.m_capbdb + int_c.m_capbds;
            if int_c.m_mode > 0 then
              qm := mosCalcDEVqmeyer(int_c.m_vgs, vgd, vgb, int_c);
            else
              qm := mosCalcDEVqmeyer(vgd, int_c.m_vgs, vgb, int_c);
              hlp := qm.qm_capgd;
              qm.qm_capgd := qm.qm_capgs;
              qm.qm_capgs := hlp;
            end if;
            int_c.m_capgd := 2 * qm.qm_capgd + int_c.m_capGDovl;
            int_c.m_capgs := 2 * qm.qm_capgs + int_c.m_capGSovl;
            int_c.m_capgb := 2 * qm.qm_capgb + int_c.m_capGBovl;
            out_cc.cGB := if in_m_bInit then -1e40 else int_c.m_capgb;
            out_cc.cGD := if in_m_bInit then -1e40 else int_c.m_capgd;
            out_cc.cGS := if in_m_bInit then -1e40 else int_c.m_capgs;
            annotation(
              Documentation(info = "<html>
<p>This function NoBypassCode calculates the currents (and the capacitances) that are necessary for the currents sum in the toplevelmodel (level 1).</p>
</html>"));
          end mosCalcNoBypassCode;

          function mosCalcDEVqmeyer "Calculation of Meyer capacities"
            extends Modelica.Icons.Function;
            input SI.Voltage vgs;
            input SI.Voltage vgd;
            input SI.Voltage vgb;
            input MosCalc in_c "Input variable set";
            output DEVqmeyer out_qm "Qmeyer values";
          protected
            SI.Voltage vds;
            SI.Voltage vddif;
            SI.Voltage vddif1;
            Types.VoltageSquare vddif2;
            SI.Voltage vgst;
          algorithm
            vgst := vgs - in_c.m_von;
            if vgst <= (-in_c.m_tPhi) then
              out_qm.qm_capgb := in_c.m_capOx / 2.;
              out_qm.qm_capgs := 0.;
              out_qm.qm_capgd := 0.;
            elseif vgst <= (-in_c.m_tPhi / 2.) then
              out_qm.qm_capgb := -vgst * in_c.m_capOx / (2. * in_c.m_tPhi);
              out_qm.qm_capgs := 0.;
              out_qm.qm_capgd := 0.;
            elseif vgst <= 0. then
              out_qm.qm_capgb := -vgst * in_c.m_capOx / (2. * in_c.m_tPhi);
              out_qm.qm_capgs := vgst * in_c.m_capOx / (1.5 * in_c.m_tPhi) + in_c.m_capOx / 3.;
              out_qm.qm_capgd := 0.;
            else
              vds := vgs - vgd;
              if in_c.m_vdsat <= vds then
                out_qm.qm_capgs := in_c.m_capOx / 3.;
                out_qm.qm_capgd := 0.;
                out_qm.qm_capgb := 0.;
              else
                vddif := 2.0 * in_c.m_vdsat - vds;
                vddif1 := in_c.m_vdsat - vds;
                vddif2 := vddif * vddif;
                out_qm.qm_capgd := in_c.m_capOx * (1. - in_c.m_vdsat * in_c.m_vdsat / vddif2) / 3.;
                out_qm.qm_capgs := in_c.m_capOx * (1. - vddif1 * vddif1 / vddif2) / 3.;
                out_qm.qm_capgb := 0.;
              end if;
            end if;
            out_qm.qm_qgs := 0.0;
            out_qm.qm_qgb := 0.0;
            out_qm.qm_qgd := 0.0;
            out_qm.qm_vgs := 0.0;
            out_qm.qm_vgb := 0.0;
            out_qm.qm_vgd := 0.0;
            annotation(
              Documentation(info = "<html>
<p>This function mosCalcDEVqmeyer calculates the Meyer capacities and charge for the Meyer model.</p>
</html>"));
          end mosCalcDEVqmeyer;

          function mos2CalcInitEquationsRevised "Mosfet initial precalculations (level 2)"
            extends Modelica.Icons.Function;
            input Spice3.Internal.Mos2.Mos2ModelLineParams in_p "Input record Mos2 values";
            input Spice3.Internal.Mosfet.Mosfet in_m "Input record MOSFET parameters";
            output Spice3.Internal.Mos2.Mos2Calc out_c "Output record Mos2 calculated values";
          algorithm
            out_c.m_drainResistance := if in_p.m_drainResistanceIsGiven > 0.5 then in_p.m_drainResistance else in_p.m_sheetResistance * in_m.m_drainSquares;
            out_c.m_sourceResistance := if in_p.m_sourceResistanceIsGiven > 0.5 then in_p.m_sourceResistance else in_p.m_sheetResistance * in_m.m_sourceSquares;
            out_c.m_lEff := in_m.m_len - 2 * in_p.m_latDiff;
            if abs(out_c.m_lEff) < 1e-18 then
              out_c.m_lEff := 1e-6;
            end if;
            out_c.m_capGSovl := in_p.m_gateSourceOverlapCapFactor * in_m.m_width;
            out_c.m_capGDovl := in_p.m_gateDrainOverlapCapFactor * in_m.m_width;
            out_c.m_capGBovl := in_p.m_gateBulkOverlapCapFactor * out_c.m_lEff;
            out_c.m_capOx := in_p.m_oxideCapFactor * out_c.m_lEff * in_m.m_width;
            annotation(
              Documentation(info = "<html>
<p>This function mosCalcInitEquations does the initial precalculation of the MOSFET parameters (level 2).</p>
</html>"));
          end mos2CalcInitEquationsRevised;

          function mos2CalcInitEquations "Mosfet initial precalculations (level 1) obsolete, use mos2CalcInitEquationsRevised"
            extends Modelica.Icons.Function;
            extends Modelica.Icons.ObsoleteModel;
            input Spice3.Internal.Mos2.Mos2ModelLineParams in_p "Input record Mos2 values";
            input Spice3.Internal.SpiceConstants in_C "Spice constants";
            input Spice3.Internal.Mos2.Mos2ModelLineVariables in_vp "Input record model line variables";
            input Spice3.Internal.Mosfet.Mosfet in_m "Input record MOSFET parameters";
            output Spice3.Internal.Mos2.Mos2Calc out_c "Output record Mos2 calculated values";
          algorithm
            out_c.m_drainResistance := if in_p.m_drainResistanceIsGiven > 0.5 then in_p.m_drainResistance else in_p.m_sheetResistance * in_m.m_drainSquares;
            out_c.m_sourceResistance := if in_p.m_sourceResistanceIsGiven > 0.5 then in_p.m_sourceResistance else in_p.m_sheetResistance * in_m.m_sourceSquares;
            out_c.m_lEff := in_m.m_len - 2 * in_p.m_latDiff;
            if abs(out_c.m_lEff) < 1e-18 then
              out_c.m_lEff := 1e-6;
            end if;
            out_c.m_capGSovl := in_p.m_gateSourceOverlapCapFactor * in_m.m_width;
            out_c.m_capGDovl := in_p.m_gateDrainOverlapCapFactor * in_m.m_width;
            out_c.m_capGBovl := in_p.m_gateBulkOverlapCapFactor * out_c.m_lEff;
            out_c.m_capOx := in_vp.m_oxideCapFactor * out_c.m_lEff * in_m.m_width;
            annotation(
              Documentation(info = "<html>
<p>This function mosCalcInitEquations does the initial precalculation of the MOSFET parameters (level 2).</p>
</html>"));
          end mos2CalcInitEquations;

          function mos2CalcCalcTempDependenciesRevised "Precalculation relating to temperature"
            extends Modelica.Icons.Function;
            input Spice3.Internal.Mos2.Mos2ModelLineParams in_p "Output record Mos1 calculated values";
            input Spice3.Internal.Mosfet.Mosfet in_m "Input record MOSFET parameters";
            input Spice3.Internal.Mos2.Mos2Calc in_c "Input record Mos2Calc";
            input Integer in_m_type "Type of MOS transistor";
            output Spice3.Internal.Mos2.Mos2Calc out_c "Output record with calculated values";
          protected
            Real ratio;
            Real ratio4;
            Real res;
          algorithm
            out_c := in_c;
            ratio := in_m.m_dTemp / in_p.m_tnom;
            ratio4 := ratio * sqrt(ratio);
            out_c.m_tTransconductance := in_p.m_transconductance / ratio4;
            out_c.m_Beta := out_c.m_tTransconductance * in_m.m_width / out_c.m_lEff;
            out_c.m_tSurfMob := in_p.m_surfaceMobility / ratio4;
            out_c.m_tPhi := Spice3.Internal.Functions.junctionPotDepTemp(in_p.m_phi, in_m.m_dTemp, in_p.m_tnom);
            out_c.m_tVbi := in_p.m_vt0 - in_m_type * (in_p.m_gamma * sqrt(in_p.m_phi)) + 0.5 * (Spice3.Internal.Functions.energyGapDepTemp(in_p.m_tnom) - Spice3.Internal.Functions.energyGapDepTemp(in_m.m_dTemp)) + in_m_type * 0.5 * (out_c.m_tPhi - in_p.m_phi);
            out_c.m_tVto := out_c.m_tVbi + in_m_type * in_p.m_gamma * sqrt(out_c.m_tPhi);
            out_c.m_tBulkPot := Spice3.Internal.Functions.junctionPotDepTemp(in_p.m_bulkJctPotential, in_m.m_dTemp, in_p.m_tnom);
            out_c.m_tDepCap := in_p.m_fwdCapDepCoeff * out_c.m_tBulkPot;
            if in_p.m_jctSatCurDensity == 0.0 or in_m.m_sourceArea == 0.0 or in_m.m_drainArea == 0.0 then
              out_c.m_tDrainSatCur := Spice3.Internal.Functions.saturationCurDepTempSPICE3MOSFET(in_p.m_jctSatCur, in_m.m_dTemp, in_p.m_tnom);
              out_c.m_tSourceSatCur := out_c.m_tDrainSatCur;
              out_c.m_VBScrit := Spice3.Internal.Functions.junctionVCrit(in_m.m_dTemp, 1.0, out_c.m_tSourceSatCur);
              out_c.m_VBDcrit := out_c.m_VBScrit;
            else
              out_c.m_tSatCurDens := Spice3.Internal.Functions.saturationCurDepTempSPICE3MOSFET(in_p.m_jctSatCurDensity, in_m.m_dTemp, in_p.m_tnom);
              out_c.m_tDrainSatCur := out_c.m_tSatCurDens * in_m.m_drainArea;
              out_c.m_tSourceSatCur := out_c.m_tSatCurDens * in_m.m_sourceArea;
              out_c.m_VBScrit := Spice3.Internal.Functions.junctionVCrit(in_m.m_dTemp, 1.0, out_c.m_tSourceSatCur);
              out_c.m_VBDcrit := Spice3.Internal.Functions.junctionVCrit(in_m.m_dTemp, 1.0, out_c.m_tDrainSatCur);
            end if;
            if not in_p.m_capBDIsGiven > 0.5 or not in_p.m_capBSIsGiven > 0.5 then
              (res, out_c.m_tCj) := Spice3.Internal.Functions.junctionParamDepTempSPICE3(in_p.m_bulkJctPotential, in_p.m_bulkCapFactor, in_p.m_bulkJctBotGradingCoeff, in_m.m_dTemp, in_p.m_tnom);
              (res, out_c.m_tCjsw) := Spice3.Internal.Functions.junctionParamDepTempSPICE3(in_p.m_bulkJctPotential, in_p.m_sideWallCapFactor, in_p.m_bulkJctSideGradingCoeff, in_m.m_dTemp, in_p.m_tnom);
              (out_c.m_f1s, out_c.m_f2s, out_c.m_f3s) := Spice3.Internal.Functions.junctionCapCoeffs(in_p.m_bulkJctSideGradingCoeff, in_p.m_fwdCapDepCoeff, out_c.m_tBulkPot);
            end if;
            if in_p.m_capBDIsGiven > 0.5 then
              (res, out_c.m_tCBDb) := Spice3.Internal.Functions.junctionParamDepTempSPICE3(in_p.m_bulkJctPotential, in_p.m_capBD, in_p.m_bulkJctBotGradingCoeff, in_m.m_dTemp, in_p.m_tnom);
              out_c.m_tCBDs := 0.0;
            else
              out_c.m_tCBDb := out_c.m_tCj * in_m.m_drainArea;
              out_c.m_tCBDs := out_c.m_tCjsw * in_m.m_drainPerimeter;
            end if;
            if in_p.m_capBSIsGiven > 0.5 then
              (res, out_c.m_tCBSb) := Spice3.Internal.Functions.junctionParamDepTempSPICE3(in_p.m_bulkJctPotential, in_p.m_capBS, in_p.m_bulkJctBotGradingCoeff, in_m.m_dTemp, in_p.m_tnom);
              out_c.m_tCBSs := 0.0;
            else
              out_c.m_tCBSb := out_c.m_tCj * in_m.m_sourceArea;
              out_c.m_tCBSs := out_c.m_tCjsw * in_m.m_sourcePerimeter;
            end if;
            (out_c.m_f1b, out_c.m_f2b, out_c.m_f3b) := Spice3.Internal.Functions.junctionCapCoeffs(in_p.m_bulkJctBotGradingCoeff, in_p.m_fwdCapDepCoeff, out_c.m_tBulkPot);
            out_c.m_dVt := in_m.m_dTemp * Spice3.Internal.SpiceConstants.CONSTKoverQ;
            annotation(
              Documentation(info = "<html>
<p>This function mosCalcCalcTempDependencies does precalculation relating to the temperature (level 2).</p>
</html>"));
          end mos2CalcCalcTempDependenciesRevised;

          function mos2CalcCalcTempDependencies "Precalculation relating to temperature obsolete, use mos2CalcCalcTempDependenciesRevised"
            extends Modelica.Icons.Function;
            extends Modelica.Icons.ObsoleteModel;
            input Spice3.Internal.Mos2.Mos2ModelLineParams in_p "Output record Mos1 calculated values";
            input Spice3.Internal.SpiceConstants in_C "Spice constants";
            input Spice3.Internal.Mos2.Mos2ModelLineVariables in_vp "Input record model line variables";
            input Spice3.Internal.Mosfet.Mosfet in_m "Input record MOSFET parameters";
            input Spice3.Internal.Mos2.Mos2Calc in_c "Input record Mos2Calc";
            input Integer in_m_type "Type of MOS transistor";
            output Spice3.Internal.Mos2.Mos2Calc out_c "Output record with calculated values";
          protected
            Real ratio;
            Real ratio4;
          algorithm
            out_c := in_c;
            ratio := in_m.m_dTemp / in_p.m_tnom;
            ratio4 := ratio * sqrt(ratio);
            out_c.m_tTransconductance := in_vp.m_transconductance / ratio4;
            out_c.m_Beta := out_c.m_tTransconductance * in_m.m_width / out_c.m_lEff;
            out_c.m_tSurfMob := in_p.m_surfaceMobility / ratio4;
            out_c.m_tPhi := Spice3.Internal.Functions.junctionPotDepTemp(in_vp.m_phi, in_m.m_dTemp, in_p.m_tnom);
            out_c.m_tVbi := in_vp.m_vt0 - in_m_type * (in_vp.m_gamma * sqrt(in_vp.m_phi)) + 0.5 * (Spice3.Internal.Functions.energyGapDepTemp(in_p.m_tnom) - Spice3.Internal.Functions.energyGapDepTemp(in_m.m_dTemp)) + in_m_type * 0.5 * (out_c.m_tPhi - in_vp.m_phi);
            out_c.m_tVto := out_c.m_tVbi + in_m_type * in_vp.m_gamma * sqrt(out_c.m_tPhi);
            out_c.m_tBulkPot := Spice3.Internal.Functions.junctionPotDepTemp(in_p.m_bulkJctPotential, in_m.m_dTemp, in_p.m_tnom);
            out_c.m_tDepCap := in_p.m_fwdCapDepCoeff * out_c.m_tBulkPot;
            if in_p.m_jctSatCurDensity == 0.0 or in_m.m_sourceArea == 0.0 or in_m.m_drainArea == 0.0 then
              out_c.m_tDrainSatCur := Spice3.Internal.Functions.saturationCurDepTempSPICE3MOSFET(in_p.m_jctSatCur, in_m.m_dTemp, in_p.m_tnom);
              out_c.m_tSourceSatCur := out_c.m_tDrainSatCur;
              out_c.m_VBScrit := Spice3.Internal.Functions.junctionVCrit(in_m.m_dTemp, 1.0, out_c.m_tSourceSatCur);
              out_c.m_VBDcrit := out_c.m_VBScrit;
            else
              out_c.m_tSatCurDens := Spice3.Internal.Functions.saturationCurDepTempSPICE3MOSFET(in_p.m_jctSatCurDensity, in_m.m_dTemp, in_p.m_tnom);
              out_c.m_tDrainSatCur := out_c.m_tSatCurDens * in_m.m_drainArea;
              out_c.m_tSourceSatCur := out_c.m_tSatCurDens * in_m.m_sourceArea;
              out_c.m_VBScrit := Spice3.Internal.Functions.junctionVCrit(in_m.m_dTemp, 1.0, out_c.m_tSourceSatCur);
              out_c.m_VBDcrit := Spice3.Internal.Functions.junctionVCrit(in_m.m_dTemp, 1.0, out_c.m_tDrainSatCur);
            end if;
            if not in_p.m_capBDIsGiven > 0.5 or not in_p.m_capBSIsGiven > 0.5 then
              out_c.m_tCj := Spice3.Internal.Functions.junctionParamDepTempSPICE3(in_p.m_bulkJctPotential, in_p.m_bulkCapFactor, in_p.m_bulkJctBotGradingCoeff, in_m.m_dTemp, in_p.m_tnom);
              out_c.m_tCjsw := Spice3.Internal.Functions.junctionParamDepTempSPICE3(in_p.m_bulkJctPotential, in_p.m_sideWallCapFactor, in_p.m_bulkJctSideGradingCoeff, in_m.m_dTemp, in_p.m_tnom);
              (out_c.m_f1s, out_c.m_f2s, out_c.m_f3s) := Spice3.Internal.Functions.junctionCapCoeffs(in_p.m_bulkJctSideGradingCoeff, in_p.m_fwdCapDepCoeff, out_c.m_tBulkPot);
            end if;
            if in_p.m_capBDIsGiven > 0.5 then
              out_c.m_tCBDb := Spice3.Internal.Functions.junctionParamDepTempSPICE3(in_p.m_bulkJctPotential, in_p.m_capBD, in_p.m_bulkJctBotGradingCoeff, in_m.m_dTemp, in_p.m_tnom);
              out_c.m_tCBDs := 0.0;
            else
              out_c.m_tCBDb := out_c.m_tCj * in_m.m_drainArea;
              out_c.m_tCBDs := out_c.m_tCjsw * in_m.m_drainPerimeter;
            end if;
            if in_p.m_capBSIsGiven > 0.5 then
              out_c.m_tCBSb := Spice3.Internal.Functions.junctionParamDepTempSPICE3(in_p.m_bulkJctPotential, in_p.m_capBS, in_p.m_bulkJctBotGradingCoeff, in_m.m_dTemp, in_p.m_tnom);
              out_c.m_tCBSs := 0.0;
            else
              out_c.m_tCBSb := out_c.m_tCj * in_m.m_sourceArea;
              out_c.m_tCBSs := out_c.m_tCjsw * in_m.m_sourcePerimeter;
            end if;
            (out_c.m_f1b, out_c.m_f2b, out_c.m_f3b) := Spice3.Internal.Functions.junctionCapCoeffs(in_p.m_bulkJctBotGradingCoeff, in_p.m_fwdCapDepCoeff, out_c.m_tBulkPot);
            out_c.m_dVt := in_m.m_dTemp * Spice3.Internal.SpiceConstants.CONSTKoverQ;
            annotation(
              Documentation(info = "<html>
<p>This function mosCalcCalcTempDependencies does precalculation relating to the temperature (level 2).</p>
</html>"));
          end mos2CalcCalcTempDependencies;

          function mos2CalcNoBypassCodeRevised "Calculation of currents and capacities (level 2)"
            extends Modelica.Icons.Function;
            input Spice3.Internal.Mosfet.Mosfet in_m "Input record MOSFET parameters";
            input Integer in_m_type "Type of MOS transistor";
            input Spice3.Internal.Mos2.Mos2Calc in_c "Input record Mos2Calc";
            input Spice3.Internal.Mos2.Mos2ModelLineParams in_p "Input record model line parameters";
            input Boolean in_m_bInit;
            input SI.Voltage[4] in_m_pVoltageValues;
            // gate bulk drain source
            output Modelica.Electrical.Spice3.Internal.Mos.CurrrentsCapacitances out_cc "Calculated currents and capacitances";
          protected
            SI.Voltage vbd "Voltage";
            SI.Voltage vgd "Voltage";
            SI.Voltage vgb "Voltage";
            Modelica.Electrical.Spice3.Internal.Mos.DEVqmeyer qm "Qmeyer capacitances";
            Spice3.Internal.Mos2.Mos2Calc int_c "Record Mos2Calc";
            Real hlp;
          algorithm
            int_c := in_c;
            int_c.m_vgs := in_m_type * (in_m_pVoltageValues[1] - in_m_pVoltageValues[4]);
// ( G , SP)
            int_c.m_vbs := in_m_type * (in_m_pVoltageValues[2] - in_m_pVoltageValues[4]);
// ( B , SP)
            int_c.m_vds := in_m_type * (in_m_pVoltageValues[3] - in_m_pVoltageValues[4]);
// ( DP, SP)
            if in_m.m_uic and in_m.m_dICVBSIsGiven > 0.5 then
              int_c.m_vbs := in_m_type * in_m.m_dICVBS;
            elseif Spice3.Internal.SpiceRoot.initJunctionVoltagesRevised() then
              int_c.m_vbs := if in_m.m_off == 1 then 0. else int_c.m_VBScrit;
            end if;
            if in_m.m_uic and in_m.m_dICVDSIsGiven > 0.5 then
              int_c.m_vds := in_m_type * in_m.m_dICVDS;
            elseif Spice3.Internal.SpiceRoot.initJunctionVoltagesRevised() then
              int_c.m_vds := if in_m.m_off == 1 then 0. else int_c.m_VBDcrit - int_c.m_VBScrit;
            end if;
            if in_m.m_uic and in_m.m_dICVGSIsGiven > 0.5 then
              int_c.m_vgs := in_m_type * in_m.m_dICVGS;
            elseif Spice3.Internal.SpiceRoot.initJunctionVoltagesRevised() then
              if in_m.m_off == 1 then
                int_c.m_vgs := 0.;
              end if;
            end if;
            if int_c.m_vds == 0 and int_c.m_vgs == 0 and int_c.m_vbs == 0 and not in_m.m_uic and in_m.m_off == 0 then
              int_c.m_vbs := -1;
              int_c.m_vgs := in_m_type * int_c.m_tVto;
              int_c.m_vds := 0;
            end if;
            vbd := int_c.m_vbs - int_c.m_vds;
            vgd := int_c.m_vgs - int_c.m_vds;
            if int_c.m_vds >= 0 then
              int_c.m_vbs := Spice3.Internal.SpiceRoot.limitJunctionVoltageRevised(int_c.m_vbs);
              vbd := int_c.m_vbs - int_c.m_vds;
            else
              vbd := Spice3.Internal.SpiceRoot.limitJunctionVoltageRevised(vbd);
              int_c.m_vbs := vbd + int_c.m_vds;
            end if;
            vgb := int_c.m_vgs - int_c.m_vbs;
            (int_c.m_cbd, int_c.m_gbd) := Spice3.Internal.Functions.junction2SPICE3MOSFETRevised(vbd, in_m.m_dTemp, 1.0, int_c.m_tDrainSatCur);
            out_cc.iBD := in_m_type * int_c.m_cbd;
            (int_c.m_cbs, int_c.m_gbs) := Spice3.Internal.Functions.junction2SPICE3MOSFETRevised(int_c.m_vbs, in_m.m_dTemp, 1.0, int_c.m_tSourceSatCur);
            out_cc.iBS := in_m_type * int_c.m_cbs;
            int_c.m_mode := if int_c.m_vds >= 0 then 1 else -1;
// 1: normal mode, -1: inverse mode
            if int_c.m_mode == 1 then
              int_c := Spice3.Internal.Mos2.drainCurRevised(int_c.m_vbs, int_c.m_vgs, int_c.m_vds, in_m, int_c, in_p, in_m_type);
            else
              int_c := Spice3.Internal.Mos2.drainCurRevised(vbd, vgd, -int_c.m_vds, in_m, int_c, in_p, in_m_type);
            end if;
            out_cc.idrain := in_m_type * int_c.m_cdrain * int_c.m_mode;
            int_c.m_capbss := 0.0;
            int_c.m_chargebss := 0.0;
            int_c.m_capbds := 0.0;
            int_c.m_chargebds := 0.0;
            (int_c.m_capbsb, int_c.m_chargebsb) := Spice3.Internal.Functions.junctionCapRevised(int_c.m_tCBSb, int_c.m_vbs, int_c.m_tDepCap, in_p.m_bulkJctBotGradingCoeff, int_c.m_tBulkPot, int_c.m_f1b, int_c.m_f2b, int_c.m_f3b);
            (int_c.m_capbdb, int_c.m_chargebdb) := Spice3.Internal.Functions.junctionCapRevised(int_c.m_tCBDb, vbd, int_c.m_tDepCap, in_p.m_bulkJctBotGradingCoeff, int_c.m_tBulkPot, int_c.m_f1b, int_c.m_f2b, int_c.m_f3b);
            if not in_p.m_capBSIsGiven > 0.5 then
              (int_c.m_capbss, int_c.m_chargebss) := Spice3.Internal.Functions.junctionCapRevised(int_c.m_tCBSs, int_c.m_vbs, int_c.m_tDepCap, in_p.m_bulkJctSideGradingCoeff, int_c.m_tBulkPot, int_c.m_f1s, int_c.m_f2s, int_c.m_f3s);
            end if;
            if not in_p.m_capBDIsGiven > 0.5 then
              (int_c.m_capbds, int_c.m_chargebds) := Spice3.Internal.Functions.junctionCapRevised(int_c.m_tCBDs, vbd, int_c.m_tDepCap, in_p.m_bulkJctSideGradingCoeff, int_c.m_tBulkPot, int_c.m_f1s, int_c.m_f2s, int_c.m_f3s);
            end if;
            out_cc.cBS := if in_m_bInit then 1e-15 else int_c.m_capbsb + int_c.m_capbss;
            out_cc.cBD := if in_m_bInit then 1e-15 else int_c.m_capbdb + int_c.m_capbds;
            if int_c.m_mode > 0 then
              qm := Modelica.Electrical.Spice3.Internal.Mos.mosCalcDEVqmeyer(int_c.m_vgs, vgd, vgb, int_c);
            else
              qm := Modelica.Electrical.Spice3.Internal.Mos.mosCalcDEVqmeyer(vgd, int_c.m_vgs, vgb, int_c);
              hlp := qm.qm_capgd;
              qm.qm_capgd := qm.qm_capgs;
              qm.qm_capgs := hlp;
            end if;
// Inverser Betrieb -> Drain und Source vertauschen
// Tausch der Spannungen bei Aufruf von DEVqmeyer
// Tausch der Kapazitaeten GD und GS
            int_c.m_capgd := 2 * qm.qm_capgd + int_c.m_capGDovl;
            int_c.m_capgs := 2 * qm.qm_capgs + int_c.m_capGSovl;
            int_c.m_capgb := 2 * qm.qm_capgb + int_c.m_capGBovl;
            out_cc.cGB := if in_m_bInit then -1e40 else int_c.m_capgb;
            out_cc.cGD := if in_m_bInit then -1e40 else int_c.m_capgd;
            out_cc.cGS := if in_m_bInit then -1e40 else int_c.m_capgs;
            annotation(
              Documentation(info = "<html>
<p>This function NoBypassCode calculates the currents (and the capacitances) that are necessary for the currents sum in the toplevelmodel (level 2).</p>
</html>"));
          end mos2CalcNoBypassCodeRevised;

          function mos2CalcNoBypassCode "Calculation of currents and capacities (level 2) obsolete, use mos2CalcNoBypassCodeRevised"
            extends Modelica.Icons.Function;
            extends Modelica.Icons.ObsoleteModel;
            input Spice3.Internal.Mosfet.Mosfet in_m "Input record MOSFET parameters";
            input Integer in_m_type "Type of MOS transistor";
            input Spice3.Internal.Mos2.Mos2Calc in_c "Input record Mos2Calc";
            input Spice3.Internal.Mos2.Mos2ModelLineParams in_p "Input record model line parameters";
            input Spice3.Internal.SpiceConstants in_C "Spice constants";
            input Spice3.Internal.Mos2.Mos2ModelLineVariables in_vp "Input record model line variables";
            input Boolean in_m_bInit;
            input Modelica.SIunits.Voltage[4] in_m_pVoltageValues;
            /* gate bulk drain source */
            output CurrrentsCapacitances out_cc "Calculated currents and capacitances";
            output DEVqmeyer qm "Calculated qmeyer values";
          protected
            Modelica.SIunits.Voltage vbd "Voltage";
            Modelica.SIunits.Voltage vgd "Voltage";
            Modelica.SIunits.Voltage vgb "Voltage";
            Modelica.SIunits.Current cur "Voltage";
            Integer n;
            DEVqmeyer in_qm(qm_capgb = 0, qm_capgs = 0, qm_capgd = 0, qm_qgs = 0, qm_qgb = 0, qm_qgd = 0, qm_vgs = 0, qm_vgb = 0, qm_vgd = 0) "Qmeyer capacitances";
            Spice3.Internal.Mos2.Mos2Calc int_c "Record Mos2Calc";
            Spice3.Internal.Mosfet.Mosfet int_m(m_len = 1e-4, m_width = 1e-4, m_drainArea = Spice3.Internal.SpiceConstants.CKTdefaultMosAD, m_sourceArea = Spice3.Internal.SpiceConstants.CKTdefaultMosAS, m_drainSquares = 1.0, m_sourceSquares = 1.0, m_drainPerimeter = 0.0, m_sourcePerimeter = 0.0, m_dICVDS = 0.0, m_dICVDSIsGiven = 0.0, m_dICVGS = 0.0, m_dICVGSIsGiven = 0.0, m_dICVBS = 0.0, m_dICVBSIsGiven = 0.0, m_off = 0, m_bPMOS = 0, m_nLevel = 1, m_drainPerimiter = 0.0, m_sourcePerimiter = 0.0, m_uic = false) "Record Mosfet";
            Real hlp;
          algorithm
            int_c := in_c;
            int_c.m_vgs := in_m_type * (in_m_pVoltageValues[1] - in_m_pVoltageValues[4]);
// ( G , SP)
            int_c.m_vbs := in_m_type * (in_m_pVoltageValues[2] - in_m_pVoltageValues[4]);
// ( B , SP)
            int_c.m_vds := in_m_type * (in_m_pVoltageValues[3] - in_m_pVoltageValues[4]);
// ( DP, SP)
            if Spice3.Internal.SpiceRoot.useInitialConditions() and in_m.m_dICVBSIsGiven > 0.5 then
              int_c.m_vbs := in_m_type * in_m.m_dICVBS;
            elseif Spice3.Internal.SpiceRoot.initJunctionVoltages() then
              int_c.m_vbs := if in_m.m_off > 0.5 then 0. else int_c.m_VBScrit;
            end if;
            if Spice3.Internal.SpiceRoot.useInitialConditions() and in_m.m_dICVDSIsGiven > 0.5 then
              int_c.m_vds := in_m_type * in_m.m_dICVDS;
            elseif Spice3.Internal.SpiceRoot.initJunctionVoltages() then
              int_c.m_vds := if in_m.m_off > 0.5 then 0. else int_c.m_VBDcrit - int_c.m_VBScrit;
            end if;
            if Spice3.Internal.SpiceRoot.useInitialConditions() and in_m.m_dICVGSIsGiven > 0.5 then
              int_c.m_vgs := in_m_type * in_m.m_dICVGS;
            elseif Spice3.Internal.SpiceRoot.initJunctionVoltages() then
              if in_m.m_off > 0.5 then
                int_c.m_vgs := 0.;
              end if;
            end if;
            if int_c.m_vds <> 0 and int_c.m_vgs <> 0 and int_c.m_vbs <> 0 and not Spice3.Internal.SpiceRoot.useInitialConditions() and in_m.m_off <> 0 then
              int_c.m_vbs := -1;
              int_c.m_vgs := in_m_type * int_c.m_tVto;
              int_c.m_vds := 0;
            end if;
            vbd := int_c.m_vbs - int_c.m_vds;
            vgd := int_c.m_vgs - int_c.m_vds;
            if int_c.m_vds >= 0 then
              int_c.m_vbs := Spice3.Internal.SpiceRoot.limitJunctionVoltage(int_c.m_vbs);
              vbd := int_c.m_vbs - int_c.m_vds;
            else
              vbd := Spice3.Internal.SpiceRoot.limitJunctionVoltage(vbd);
              int_c.m_vbs := vbd + int_c.m_vds;
            end if;
            vgb := int_c.m_vgs - int_c.m_vbs;
            (int_c.m_cbd, int_c.m_gbd) := Spice3.Internal.Functions.junction2SPICE3MOSFET(int_c.m_cbd, int_c.m_gbd, vbd, in_m.m_dTemp, 1.0, int_c.m_tDrainSatCur);
            out_cc.iBD := in_m_type * int_c.m_cbd;
            (int_c.m_cbs, int_c.m_gbs) := Spice3.Internal.Functions.junction2SPICE3MOSFET(int_c.m_cbs, int_c.m_gbs, int_c.m_vbs, in_m.m_dTemp, 1.0, int_c.m_tSourceSatCur);
            out_cc.iBS := in_m_type * int_c.m_cbs;
            int_c.m_mode := if int_c.m_vds >= 0 then 1 else -1;
// 1: normal mode, -1: inverse mode
            if int_c.m_mode == 1 then
              int_c := Spice3.Internal.Mos2.drainCur(int_c.m_vbs, int_c.m_vgs, int_c.m_vds, int_m, int_c, in_p, in_C, in_vp, in_m_type);
            else
              int_c := Spice3.Internal.Mos2.drainCur(vbd, vgd, -int_c.m_vds, int_m, int_c, in_p, in_C, in_vp, in_m_type);
            end if;
            n := if int_c.m_mode == 1 then 6 else 5;
            out_cc.idrain := in_m_type * int_c.m_cdrain * int_c.m_mode;
            int_c.m_capbss := 0.0;
            int_c.m_chargebss := 0.0;
            int_c.m_capbds := 0.0;
            int_c.m_chargebds := 0.0;
            (int_c.m_capbsb, int_c.m_chargebsb) := Spice3.Internal.Functions.junctionCap(int_c.m_tCBSb, int_c.m_vbs, int_c.m_tDepCap, in_p.m_bulkJctBotGradingCoeff, int_c.m_tBulkPot, int_c.m_f1b, int_c.m_f2b, int_c.m_f3b);
            (int_c.m_capbdb, int_c.m_chargebdb) := Spice3.Internal.Functions.junctionCap(int_c.m_tCBDb, vbd, int_c.m_tDepCap, in_p.m_bulkJctBotGradingCoeff, int_c.m_tBulkPot, int_c.m_f1b, int_c.m_f2b, int_c.m_f3b);
            if not in_p.m_capBSIsGiven > 0.5 then
              (int_c.m_capbss, int_c.m_chargebss) := Spice3.Internal.Functions.junctionCap(int_c.m_tCBSs, int_c.m_vbs, int_c.m_tDepCap, in_p.m_bulkJctSideGradingCoeff, int_c.m_tBulkPot, int_c.m_f1s, int_c.m_f2s, int_c.m_f3s);
            end if;
            if not in_p.m_capBDIsGiven > 0.5 then
              (int_c.m_capbds, int_c.m_chargebds) := Spice3.Internal.Functions.junctionCap(int_c.m_tCBDs, vbd, int_c.m_tDepCap, in_p.m_bulkJctSideGradingCoeff, int_c.m_tBulkPot, int_c.m_f1s, int_c.m_f2s, int_c.m_f3s);
            end if;
            out_cc.cBS := if in_m_bInit then 1e-15 else int_c.m_capbsb + int_c.m_capbss;
            out_cc.cBD := if in_m_bInit then 1e-15 else int_c.m_capbdb + int_c.m_capbds;
            if int_c.m_mode > 0 then
              qm := mosCalcDEVqmeyer(int_c.m_vgs, vgd, vgb, int_c);
            else
              qm := mosCalcDEVqmeyer(vgd, int_c.m_vgs, vgb, int_c);
              hlp := qm.qm_capgd;
              qm.qm_capgd := qm.qm_capgs;
              qm.qm_capgs := hlp;
            end if;
            if in_m_bInit then
              int_c.m_capgd := 2 * qm.qm_capgd + int_c.m_capGDovl;
              int_c.m_capgs := 2 * qm.qm_capgs + int_c.m_capGSovl;
              int_c.m_capgb := 2 * qm.qm_capgb + int_c.m_capGBovl;
              int_c.m_qgs := int_c.m_capgs * int_c.m_vgs;
              int_c.m_qgb := int_c.m_capgb * vgb;
              int_c.m_qgd := int_c.m_capgd * vgd;
            else
              int_c.m_capgd := qm.qm_capgd + in_qm.qm_capgd + int_c.m_capGDovl;
              int_c.m_capgs := qm.qm_capgs + in_qm.qm_capgs + int_c.m_capGSovl;
              int_c.m_capgb := qm.qm_capgb + in_qm.qm_capgb + int_c.m_capGBovl;
              int_c.m_qgs := (int_c.m_vgs - in_qm.qm_vgs) * int_c.m_capgs + in_qm.qm_qgs;
              int_c.m_qgb := (vgb - in_qm.qm_vgb) * int_c.m_capgb + in_qm.qm_qgb;
              int_c.m_qgd := (vgd - in_qm.qm_vgd) * int_c.m_capgd + in_qm.qm_qgd;
            end if;
            out_cc.m_capgd := int_c.m_capgd;
            qm.qm_qgs := int_c.m_qgs;
            qm.qm_qgb := int_c.m_qgb;
            qm.qm_qgd := int_c.m_qgd;
            qm.qm_vgs := int_c.m_vgs;
            qm.qm_vgb := vgb;
            qm.qm_vgd := vgd;
            out_cc.cGB := if in_m_bInit then -1e40 else int_c.m_capgb;
            out_cc.cGD := if in_m_bInit then -1e40 else out_cc.m_capgd;
            out_cc.cGS := if in_m_bInit then -1e40 else int_c.m_capgs;
            annotation(
              Documentation(info = "<html>
<p>This function NoBypassCode calculates the currents (and the capacitances) that are necessary for the currents sum in the toplevelmodel (level 2).</p>
</html>"));
          end mos2CalcNoBypassCode;
          annotation(
            Documentation(info = "<html>
<p>This package Mos contains functions and records with data of the MOSFET models level 1, 2, 3 and 6.</p>
</html>"));
        end Mos;

        package Mos1 "Records and functions for MOSFETs level 1"
          extends Modelica.Icons.InternalPackage;

          record Mos1ModelLineParams "Record for Mosfet model line parameters (for level 1)"
            extends Mos.MosModelLineParams(m_lambda(start = 0.0), m_transconductance(start = 2.0e-5));
            annotation(
              Documentation(info = "<html>
<p>This record Mos1ModelLineParams contains the model line parameters that are used for the MOSFET transistors level 1 in SPICE3.</p>
</html>"));
          end Mos1ModelLineParams;

          record Mos1Calc "Further MOSFET variables (for level 1)"
            extends Mos.MosCalc;
            annotation(
              Documentation(info = "<html>
<p>This record Mos1Calc contains further MOSFET variables (for level 1) that are needed for the calculations.</p>
</html>"));
          end Mos1Calc;

          function mos1ModelLineParamsInitEquations "Initial precalculation"
            extends Modelica.Icons.Function;
            input Mos1ModelLineParams in_p "Input record model line parameters for MOS1";
            input SpiceConstants in_C "Spice constants";
            input Integer in_m_type "Type of MOS transistor";
            output Mos.MosModelLineVariables out_v "Output record model line variables";
          protected
            Modelica.SIunits.Voltage vtnom;
            Modelica.SIunits.Voltage fermis;
            Real fermig;
            Real wkfng;
            Real wkfngs;
            Real egfet1;
            Real vfb;
          algorithm
            out_v.m_oxideCapFactor := in_p.m_oxideCapFactor;
            out_v.m_transconductance := in_p.m_transconductance;
            out_v.m_phi := in_p.m_phi;
            out_v.m_gamma := in_p.m_gamma;
            out_v.m_vt0 := in_p.m_vt0;
            vtnom := in_p.m_tnom * SpiceConstants.CONSTKoverQ;
            egfet1 := 1.16 - 7.02e-4 * in_p.m_tnom * in_p.m_tnom / (in_p.m_tnom + 1108);
            if not in_p.m_oxideThicknessIsGiven > 0.5 or in_p.m_oxideThickness == 0 then
              if in_p.m_oxideThickness == 0 then
                out_v.m_oxideCapFactor := 0;
              end if;
            else
              out_v.m_oxideCapFactor := 3.9 * 8.854214871e-12 / in_p.m_oxideThickness;
              if out_v.m_oxideCapFactor <> 0 then
                if not in_p.m_transconductanceIsGiven > 0.5 then
                  out_v.m_transconductance := in_p.m_surfaceMobility * out_v.m_oxideCapFactor * 1e-4;
                end if;
                if in_p.m_substrateDopingIsGiven > 0.5 then
                  if in_p.m_substrateDoping * 1e6 > 1.45e16 then
                    if not in_p.m_phiIsGiven > 0.5 then
                      out_v.m_phi := 2 * vtnom * Modelica.Math.log(in_p.m_substrateDoping * 1e6 / 1.45e16);
                      out_v.m_phi := max(0.1, out_v.m_phi);
                    end if;
                    fermis := in_m_type * 0.5 * out_v.m_phi;
                    wkfng := 3.2;
                    if in_p.m_gateType <> 0 then
                      fermig := in_m_type * in_p.m_gateType * 0.5 * egfet1;
                      wkfng := 3.25 + 0.5 * egfet1 - fermig;
                    end if;
                    wkfngs := wkfng - (3.25 + 0.5 * egfet1 + fermis);
                    if not in_p.m_gammaIsGiven > 0.5 then
                      out_v.m_gamma := sqrt(2 * 11.70 * 8.854214871e-12 * SpiceConstants.CHARGE * in_p.m_substrateDoping * 1e6 / out_v.m_oxideCapFactor);
                    end if;
                    if not in_p.m_vtOIsGiven > 0.5 then
                      vfb := wkfngs - in_p.m_surfaceStateDensity * 1e4 * SpiceConstants.CHARGE / out_v.m_oxideCapFactor;
                      out_v.m_vt0 := vfb + in_m_type * (out_v.m_gamma * sqrt(out_v.m_phi) + out_v.m_phi);
                    end if;
                  end if;
                end if;
              end if;
            end if;
// (m**2/cm**2)
// (cm**3/m**3)
// (cm**3/m**3)
// (cm**3/m**3)
// (cm**2/m**2)
            annotation(
              Documentation(info = "<html>
<p>This function mos1ModelLineParamsInitEquation does the initial precalculation of the MOSFET model line parameters for level 1.</p>
</html>"));
          end mos1ModelLineParamsInitEquations;

          function drainCur "Drain current calculation"
            extends Modelica.Icons.Function;
            input Modelica.SIunits.Voltage vb;
            input Modelica.SIunits.Voltage vg;
            input Modelica.SIunits.Voltage vds;
            input Mos1Calc in_c "Input record Mos1Calc";
            input Mos1ModelLineParams in_p "Input record model line parameters for MOS1";
            input SpiceConstants in_C "Spice constants";
            input Mos.MosModelLineVariables in_vp "Input record model line variables";
            input Integer in_m_type "Type of Mos transistor";
            output Mos1Calc out_c "Output record Mos1Calc";
          protected
            Real arg;
            Real betap;
            Real sarg;
            Modelica.SIunits.Voltage vgst;
          algorithm
            out_c := in_c;
            if vb <= 0 then
              sarg := sqrt(out_c.m_tPhi - vb);
            else
              sarg := sqrt(out_c.m_tPhi);
              sarg := sarg - vb / (sarg + sarg);
              sarg := max(0., sarg);
            end if;
            out_c.m_von := out_c.m_tVbi * in_m_type + in_vp.m_gamma * sarg;
            vgst := vg - out_c.m_von;
            out_c.m_vdsat := max(vgst, 0.);
            arg := if sarg <= 0 then 0 else in_vp.m_gamma / (sarg + sarg);
            if vgst <= 0 then
              out_c.m_cdrain := 0;
              out_c.m_gm := 0;
              out_c.m_gds := 0;
              out_c.m_gmbs := 0;
            else
              betap := out_c.m_Beta * (1 + in_p.m_lambda * vds);
              if vgst <= vds then
                out_c.m_cdrain := betap * vgst * vgst * 0.5;
                out_c.m_gm := betap * vgst;
                out_c.m_gds := in_p.m_lambda * out_c.m_Beta * vgst * vgst * 0.5;
                out_c.m_gmbs := out_c.m_gm * arg;
              else
                out_c.m_cdrain := betap * vds * (vgst - 0.5 * vds);
                out_c.m_gm := betap * vds;
                out_c.m_gds := betap * (vgst - vds) + in_p.m_lambda * out_c.m_Beta * vds * (vgst - 0.5 * vds);
                out_c.m_gmbs := out_c.m_gm * arg;
              end if;
            end if;
/* cutoff region */
/* saturation region */
/* linear region */
            annotation(
              Documentation(info = "<html>
<p>This function drainCur calculates the main currents that flows from drain node to source node (level 1).</p>
</html>"));
          end drainCur;

          function mos1RenameParameters "Parameter renaming to internal names"
            extends Modelica.Icons.Function;
            input ModelcardMOS ex "Modelcard with technologieparameters";
            input SpiceConstants con "Spice constants";
            output Mos.MosModelLineParams intern "Output record model line parameters";
          algorithm
            intern.m_cgboIsGiven := 0;
            intern.m_cgdoIsGiven := 0;
            intern.m_cgsoIsGiven := 0;
            intern.m_mjswIsGiven := 0;
            intern.m_pbIsGiven := 0;
            intern.m_surfaceStateDensityIsGiven := 0;
            intern.m_oxideCapFactor := 0;
            intern.m_vtOIsGiven := if ex.VTO > (-1e40) then 1 else 0;
            intern.m_vt0 := if ex.VTO > (-1e40) then ex.VTO else 0;
            intern.m_capBDIsGiven := if ex.CBD > (-1e40) then 1 else 0;
            intern.m_capBD := if ex.CBD > (-1e40) then ex.CBD else 0;
            intern.m_capBSIsGiven := if ex.CBS > (-1e40) then 1 else 0;
            intern.m_capBS := if ex.CBS > (-1e40) then ex.CBS else 0;
            intern.m_bulkCapFactorIsGiven := if ex.CJ > (-1e40) then 1 else 0;
            intern.m_bulkCapFactor := if ex.CJ > (-1e40) then ex.CJ else 0;
            intern.m_sideWallCapFactor := ex.CJSW "F/m zero-bias junction sidewall cap. per meter of junction perimeter (default 0)";
            intern.m_fwdCapDepCoeff := ex.FC "Coefficient for forward-bias depletion capacitance formula (default 0.5)";
            intern.m_phiIsGiven := if ex.PHI > (-1e40) then 1 else 0;
            intern.m_phi := if ex.PHI > (-1e40) then ex.PHI else 0.6;
            intern.m_gammaIsGiven := if ex.GAMMA > (-1e40) then 1 else 0;
            intern.m_gamma := if ex.GAMMA > (-1e40) then ex.GAMMA else 0;
            intern.m_lambda := ex.LAMBDA "1/V channel-length modulation (default 0)";
            intern.m_substrateDopingIsGiven := if ex.NSUB > (-1e40) then 1 else 0;
            intern.m_substrateDoping := if ex.NSUB > (-1e40) then ex.NSUB else 0;
            intern.m_gateType := ex.TPG "Type of gate material: +1 opp. to substrate, -1 same as substrate, 0 Al gate (default 1)";
            intern.m_surfaceStateDensity := ex.NSS "IN 1/(cm*cm) surface state density (default 0)";
            intern.m_surfaceMobility := ex.UO "In (cm*cm)/(Vs) surface mobility (default 600)";
            intern.m_latDiff := ex.LD "In m lateral diffusion (default 0)";
            intern.m_jctSatCur := ex.IS "A bulk junction saturation current (default 1e-14)";
            intern.m_drainResistanceIsGiven := if ex.RD > (-1e40) then 1 else 0;
            intern.m_drainResistance := if ex.RD > (-1e40) then ex.RD else 0;
            intern.m_sourceResistanceIsGiven := if ex.RS > (-1e40) then 1 else 0;
            intern.m_sourceResistance := if ex.RS > (-1e40) then ex.RS else 0;
            intern.m_transconductanceIsGiven := if ex.KP > (-1e40) then 1 else 0;
            intern.m_transconductance := if ex.KP > (-1e40) then ex.KP else 2e-5;
            intern.m_tnom := if ex.TNOM > (-1e40) then ex.TNOM + SpiceConstants.CONSTCtoK else 300.15 "parameter measurement temperature (default 27 deg C)";
            intern.m_jctSatCurDensity := ex.JS "A/(m*m) bulk junction saturation current per sq-meter of junction area (default 0)";
            intern.m_sheetResistance := ex.RSH "Ohm drain and source diffusion sheet resistance (default 0)";
            intern.m_bulkJctPotential := ex.PB "V bulk junction potential (default 0.8)";
            intern.m_bulkJctBotGradingCoeff := ex.MJ "bulk junction bottom grading coeff. (default 0.5)";
            intern.m_bulkJctSideGradingCoeff := ex.MJSW "bulk junction sidewall grading coeff. (default 0.5)";
            intern.m_oxideThicknessIsGiven := if ex.TOX > (-1e40) then 1 else 0;
            intern.m_oxideThickness := if ex.TOX > (-1e40) then ex.TOX else 0;
            intern.m_gateSourceOverlapCapFactor := ex.CGSO "F/m gate-source overlap capacitance per meter channel width (default 0)";
            intern.m_gateDrainOverlapCapFactor := ex.CGDO "F/m gate-drain overlap capacitance per meter channel width (default 0)";
            intern.m_gateBulkOverlapCapFactor := ex.CGBO "F/m gate-bulk overlap capacitance per meter channel width (default 0)";
            intern.m_fNcoef := ex.KF "Flicker-noise coefficient (default 0)";
            intern.m_fNexp := ex.AF "Flicker-noise exponent (default 1)";
            annotation(
              Documentation(info = "<html>
<p>This function mos1RenameParameters assigns the external (given by the user, e.g., RD) technology parameters
to the internal parameters (e.g., m_drainResistance). It also does the analysis of the IsGiven values (level 1).</p>
</html>"));
          end mos1RenameParameters;

          function mos1RenameParametersDev "Device parameter renaming to internal names"
            extends Modelica.Icons.Function;
            input ModelcardMOS ex;
            input Integer mtype;
            input Modelica.SIunits.Length W "Channel Width";
            input Modelica.SIunits.Length L "Channel Length";
            input Modelica.SIunits.Area AD "Area of the drain diffusion";
            input Modelica.SIunits.Area AS "Area of the source diffusion";
            input Modelica.SIunits.Length PD "Perimeter of the drain junction";
            input Modelica.SIunits.Length PS "Perimeter of the source junction";
            input Real NRD "Number of squares of the drain diffusions";
            input Real NRS "Number of squares of the source diffusions";
            input Integer OFF "Optional initial condition: 0 - IC not used, 1 - IC used, not implemented yet";
            input Real IC "Initial condition values, not implemented yet";
            input Modelica.SIunits.Temp_C TEMP "Temperature";
            output Mosfet.Mosfet dev "Output record Mosfet";
          algorithm
/*device parameters*/
            dev.m_len := L "L, length of channel region";
            dev.m_width := W "W, width of channel region";
            dev.m_drainArea := AD "AD, area of drain diffusion";
            dev.m_sourceArea := AS "AS, area of source diffusion";
            dev.m_drainSquares := NRD "NRD, length of drain in squares";
            dev.m_sourceSquares := NRS "NRS, length of source in squares";
            dev.m_drainPerimeter := PD "PD, Drain perimeter";
            dev.m_sourcePerimeter := PS "PS, Source perimeter";
            dev.m_dICVDSIsGiven := if IC > (-1e40) then 1 else 0 "ICVDS IsGivenValue";
            dev.m_dICVDS := if IC > (-1e40) then IC else 0 "Initial condition of VDS";
            dev.m_dICVGSIsGiven := if IC > (-1e40) then 1 else 0 "ICVGS IsGivenValue";
            dev.m_dICVGS := if IC > (-1e40) then IC else 0 "Initial condition of VGS";
            dev.m_dICVBSIsGiven := if IC > (-1e40) then 1 else 0 "ICVBS IsGivenValue";
            dev.m_dICVBS := if IC > (-1e40) then IC else 0 "Initial condition of VBS";
            dev.m_off := OFF "Non-zero to indicate device is off for dc analysis";
            dev.m_bPMOS := mtype "P type MOSFET model";
            dev.m_nLevel := ex.LEVEL "Level";
            assert(ex.LEVEL == 1, "only MOS Level1 implemented");
            dev.m_dTemp := TEMP + SpiceConstants.CONSTCtoK "Device temperature";
            dev.m_drainPerimiter := 0;
            dev.m_sourcePerimiter := 0;
            dev.m_uic := false;
            annotation(
              Documentation(info = "<html>
<p>This function mos1RenameParametersDev assigns the external (given by the user) device parameters to the internal parameters. It also does the analysis of the IsGiven values (level 1).</p>
</html>"));
          end mos1RenameParametersDev;
          annotation(
            Documentation(info = "<html>
<p>This package Mos1 contains functions and record with data of the MOSFET model level 1.</p>
</html>"));
        end Mos1;

        package Mos2 "Records and functions for MOSFETs level 2"
          extends Modelica.Icons.InternalPackage;

          record Mos2ModelLineParams "Record for Mosfet model line parameters (for level 2)"
            extends Spice3.Internal.Mos.MosModelLineParams(m_lambda(start = 0.0), m_transconductance(start = 2.0e-5), m_bulkJctSideGradingCoeff(start = 0.33), m_oxideThickness(start = 1.0e-7));
            Real m_narrowFactor(start = 0.0) "DELTA, Width effect on threshold";
            Real m_critFieldExp(start = 0.0) "UEXP, Crit. field exp for mob. deg";
            Types.ElectricFieldStrength_cm m_critField(start = 1.0e4) "UCRIT, Crit. field for mob. degradation";
            SI.Velocity m_maxDriftVel(start = 0.0) "VMAX, Maximum carrier drift velocity";
            SI.Length m_junctionDepth(start = 0.0) "XJ, Junction depth";
            Modelica.SIunits.Charge m_channelCharge(start = 1.0) "NEFF, Total channel charge coeff";
            SI.Conversions.NonSIunits.PerArea_cm m_fastSurfaceStateDensity(start = 0.0) "NFS, Fast surface state density";
            Real m_xd;
            // unit m/V(-0.5) -> m/Wurzel V
            annotation(
              Documentation(info = "<html>
<p>This record Mos1ModelLineParams contains the model line parameters that are used for the MOSFET transistors level 2 in SPICE3.</p>
</html>"));
          end Mos2ModelLineParams;

          record Mos2ModelLineVariables "Record for Mosfet model line variables (for level 2)"
            extends Spice3.Internal.Mos.MosModelLineVariables;
            Real m_bulkCapFactor;
            Real m_substrateDoping;
            Real m_xd;
            annotation(
              Documentation(info = "<html>
<p>This record MosModelLineVariables contains the model line variables that are used for the MOSFET transistors level 2 SPICE3.</p>
</html>"));
          end Mos2ModelLineVariables;

          record Mos2Calc "Further MOSFET variables (for level 2)"
            extends Spice3.Internal.Mos.MosCalc;
            annotation(
              Documentation(info = "<html>
<p>This record Mos1Calc contains further MOSFET variables (for level 2) that are needed for the calculations.</p>
</html>"));
          end Mos2Calc;

          function mos2ModelLineParamsInitEquations "Initial precalculation
           obsolete, use mos2ModelLineParamsInitEquationsRevised"
            extends Modelica.Icons.Function;
            extends Modelica.Icons.ObsoleteModel;
            input Mos2ModelLineParams in_p "Input record model line parameters for MOS2";
            input Spice3.Internal.SpiceConstants in_C "Spice constants";
            input Integer in_m_type "Type of MOS transistor";
            output Mos2ModelLineVariables out_v "Output record with model line variables";
          protected
            Modelica.SIunits.Voltage vtnom;
            Modelica.SIunits.Voltage fermis;
            Real fermig;
            Real wkfng;
            Real wkfngs;
            Real egfet1;
            Real vfb;
          algorithm
            out_v.m_oxideCapFactor := in_p.m_oxideCapFactor;
            out_v.m_transconductance := in_p.m_transconductance;
            out_v.m_phi := in_p.m_phi;
            out_v.m_gamma := in_p.m_gamma;
            out_v.m_vt0 := in_p.m_vt0;
            out_v.m_substrateDoping := in_p.m_substrateDoping;
            out_v.m_bulkCapFactor := in_p.m_bulkCapFactor;
            vtnom := in_p.m_tnom * in_C.CONSTKoverQ;
            egfet1 := 1.16 - 7.02e-4 * in_p.m_tnom * in_p.m_tnom / (in_p.m_tnom + 1108);
            out_v.m_oxideCapFactor := 3.9 * 8.854214871e-12 / in_p.m_oxideThickness;
            if not in_p.m_transconductanceIsGiven > 0.5 then
              out_v.m_transconductance := in_p.m_surfaceMobility * 1.0e-4 * out_v.m_oxideCapFactor;
            end if;
            if in_p.m_substrateDopingIsGiven > 0.5 then
              if out_v.m_substrateDoping * 1.0e6 > 1.45e16 then
                if not in_p.m_phiIsGiven > 0.5 then
                  out_v.m_phi := 2 * vtnom * Modelica.Math.log(out_v.m_substrateDoping * 1.0e6 / 1.45e16);
                  out_v.m_phi := max(0.1, out_v.m_phi);
                end if;
                fermis := in_m_type * 0.5 * out_v.m_phi;
                wkfng := 3.2;
                if in_p.m_gateType <> 0 then
                  fermig := in_m_type * in_p.m_gateType * 0.5 * egfet1;
                  wkfng := 3.25 + 0.5 * egfet1 - fermig;
                end if;
                wkfngs := wkfng - (3.25 + 0.5 * egfet1 + fermis);
                if not in_p.m_gammaIsGiven > 0.5 then
                  out_v.m_gamma := sqrt(2.0 * 11.70 * 8.854214871e-12 * in_C.CHARGE * out_v.m_substrateDoping * 1.0e6) / out_v.m_oxideCapFactor;
                end if;
                if not in_p.m_vtOIsGiven > 0.5 then
                  vfb := wkfngs - in_p.m_surfaceStateDensity * 1.0e4 * in_C.CHARGE / out_v.m_oxideCapFactor;
                  out_v.m_vt0 := vfb + in_m_type * (out_v.m_gamma * sqrt(out_v.m_phi) + out_v.m_phi);
                else
                  vfb := out_v.m_vt0 - in_m_type * (out_v.m_gamma * sqrt(out_v.m_phi) + out_v.m_phi);
                end if;
                out_v.m_xd := sqrt((in_C.EPSSIL + in_C.EPSSIL) / (in_C.CHARGE * out_v.m_substrateDoping * 1.0e6));
              else
                out_v.m_substrateDoping := 0.0;
              end if;
            end if;
            if not in_p.m_bulkCapFactorIsGiven > 0.5 then
              out_v.m_bulkCapFactor := sqrt(in_C.EPSSIL * in_C.CHARGE * out_v.m_substrateDoping * 1e6 / (2 * in_p.m_bulkJctPotential));
            end if;
            annotation(
              Documentation(info = "<html>
<p>This function mos1ModelLineParamsInitEquation does the initial precalculation of the MOSFET model line parameters for level 2.</p>
</html>"));
          end mos2ModelLineParamsInitEquations;

          function mos2ModelLineParamsInitEquationsRevised "Initial precalculation"
            extends Modelica.Icons.Function;
            input Modelica.Electrical.Spice3.Internal.Mos2.Mos2ModelLineParams in_p "Input record model line parameters for MOS2";
            input Integer in_m_type "Type of MOS transistor";
            output Modelica.Electrical.Spice3.Internal.Mos2.Mos2ModelLineParams out_p "Input record model line parameters for MOS2";
          protected
            SI.Voltage vtnom;
            SI.Voltage fermis;
            Real fermig;
            Real wkfng;
            Real wkfngs;
            Real egfet1;
            Real vfb;
          algorithm
            out_p := in_p;
            vtnom := out_p.m_tnom * Spice3.Internal.SpiceConstants.CONSTKoverQ;
            egfet1 := Spice3.Internal.MaterialParameters.EnergyGapSi - Modelica.Electrical.Spice3.Internal.MaterialParameters.FirstBandCorrFactorSi * out_p.m_tnom * out_p.m_tnom / (out_p.m_tnom + Spice3.Internal.MaterialParameters.SecondBandCorrFactorSi);
            out_p.m_oxideCapFactor := Spice3.Internal.SpiceConstants.EPSOX / out_p.m_oxideThickness;
            if not out_p.m_transconductanceIsGiven > 0.5 then
              out_p.m_transconductance := out_p.m_surfaceMobility * 1.0e-4 * out_p.m_oxideCapFactor;
            end if;
            if out_p.m_substrateDopingIsGiven > 0.5 then
              if out_p.m_substrateDoping * 1.0e6 > Modelica.Electrical.Spice3.Internal.MaterialParameters.IntCondCarrDensity then
                if not out_p.m_phiIsGiven > 0.5 then
                  out_p.m_phi := 2 * vtnom * Modelica.Math.log(out_p.m_substrateDoping * 1.0e6 / Spice3.Internal.MaterialParameters.IntCondCarrDensity);
                  out_p.m_phi := max(0.1, out_p.m_phi);
                end if;
                fermis := in_m_type * 0.5 * out_p.m_phi;
                wkfng := 3.2;
                if out_p.m_gateType <> 0 then
                  fermig := in_m_type * out_p.m_gateType * 0.5 * egfet1;
                  wkfng := 3.25 + 0.5 * egfet1 - fermig;
                end if;
                wkfngs := wkfng - (3.25 + 0.5 * egfet1 + fermis);
                if not out_p.m_gammaIsGiven > 0.5 then
                  out_p.m_gamma := sqrt(2.0 * Spice3.Internal.SpiceConstants.EPSSIL * Spice3.Internal.SpiceConstants.CHARGE * out_p.m_substrateDoping * 1.0e6) / out_p.m_oxideCapFactor;
                end if;
                if not out_p.m_vtOIsGiven > 0.5 then
                  vfb := wkfngs - out_p.m_surfaceStateDensity * 1.0e4 * Spice3.Internal.SpiceConstants.CHARGE / out_p.m_oxideCapFactor;
                  out_p.m_vt0 := vfb + in_m_type * (out_p.m_gamma * sqrt(out_p.m_phi) + out_p.m_phi);
                else
                  vfb := out_p.m_vt0 - in_m_type * (out_p.m_gamma * sqrt(out_p.m_phi) + out_p.m_phi);
                end if;
                out_p.m_xd := sqrt((Spice3.Internal.SpiceConstants.EPSSIL + Spice3.Internal.SpiceConstants.EPSSIL) / (Spice3.Internal.SpiceConstants.CHARGE * out_p.m_substrateDoping * 1.0e6));
              else
                out_p.m_substrateDoping := 0.0;
              end if;
            end if;
            if not out_p.m_bulkCapFactorIsGiven > 0.5 then
              out_p.m_bulkCapFactor := sqrt(Spice3.Internal.SpiceConstants.EPSSIL * Spice3.Internal.SpiceConstants.CHARGE * out_p.m_substrateDoping * 1e6 / (2 * out_p.m_bulkJctPotential));
            end if;
            annotation(
              Documentation(info = "<html>
<p>This function mos1ModelLineParamsInitEquation does the initial precalculation of the MOSFET model line parameters for level 2.</p>
</html>"));
          end mos2ModelLineParamsInitEquationsRevised;

          function drainCurRevised "Drain current calculation"
            extends Modelica.Icons.Function;
            input SI.Voltage vbs;
            input SI.Voltage vgs;
            input SI.Voltage vds;
            input Spice3.Internal.Mosfet.Mosfet in_m "Record MOSFET";
            input Modelica.Electrical.Spice3.Internal.Mos2.Mos2Calc in_c "Input record Mos2Calc";
            input Modelica.Electrical.Spice3.Internal.Mos2.Mos2ModelLineParams in_p "Input record model line parameters for MOS2";
            input Integer in_m_type "Type of MOS transistor";
            output Modelica.Electrical.Spice3.Internal.Mos2.Mos2Calc out_c "Output record Mos2Calc";
          protected
            Real vt;
            // K * T / Q
            Real beta1;
            Real dsrgdb;
            Real d2sdb2;
            Real sphi = 0.0;
            Real sphi3 = 1.0;
            // square root of phi
            Real barg;
            Real sarg;
            Real bsarg = 0.0;
            Real sarg3;
            Real d2bdb2;
            Real factor;
            Real dbrgdb;
            Real eta;
            Real vbin;
            Real vth;
            Real dgddb2;
            Real dgddvb;
            Real dgdvds;
            Real gamasd;
            Real gammad;
            Real xn = 1.0;
            Real argg = 0.0;
            Real vgst;
            Real vgsx;
            Real dgdvbs;
            Real body;
            Real bodys = 0.0;
            Real gdbdv;
            Real dodvbs;
            Real dodvds = 0.0;
            Real dxndvd = 0.0;
            Real dxndvb = 0.0;
            Real dudvgs;
            Real dudvds;
            Real dudvbs;
            Real ufact;
            Real ueff;
            Real dsdvgs;
            Real dsdvbs;
            Real dbsrdb;
            Real gdbdvs = 0.0;
            Real dldvgs;
            Real dldvds;
            Real dldvbs;
            Real clfact;
            Real xleff;
            Real deltal;
            Real xwb;
            Real xld;
            Real xlamda = in_p.m_lambda;
            Real phiMinVbs;
            Real tmp;
            Real argss;
            Real argsd;
            Real args = 0.0;
            Real argd = 0.0;
            Real argxs = 0.0;
            Real argxd = 0.0;
            Real dbargs;
            Real dbargd;
            Real dbxws;
            Real dbxwd;
            Real xwd;
            Real xws;
            Real daddb2;
            Real dasdb2;
            Real ddxwd;
            Real cfs;
            Real cdonco;
            Real argv;
            Real gammd2;
            Real arg;
            Real y3;
            Real xvalid = 0.0;
            Real[4] sig1;
            Real[4] sig2;
            Real[4] a4;
            Real[4] b4;
            Real[8] x4;
            Real[8] poly4;
            Real delta4;
            Integer j;
            Integer iknt = 0;
            Integer i;
            Integer jknt = 0;
            Real v1;
            Real v2;
            Real xv;
            Real a1;
            Real b1;
            Real c1;
            Real d1;
            Real b2;
            Real r1;
            Real s1;
            Real s2;
            Real p1;
            Real p0;
            Real p2;
            Real a3;
            Real b3;
            Real sargv;
            Real dldsat;
            Real xlfact;
            Real xdv;
            Real xlv;
            Real vqchan;
            Real dqdsat;
            Real vl;
            Real dfunds;
            Real dfundg;
            Real dfundb;
            Real xls;
            Real dfact;
            Real vdson;
            Real cdson;
            Real gdson;
            Real didvds;
            Real gmw;
            Real gbson;
            Real expg;
          algorithm
            out_c := in_c;
            vt := Spice3.Internal.SpiceConstants.CONSTKoverQ * Spice3.Internal.SpiceConstants.REFTEMP;
            phiMinVbs := out_c.m_tPhi - vbs;
            if vbs <= 0.0 then
              sarg := sqrt(phiMinVbs);
              dsrgdb := -0.5 / sarg;
              d2sdb2 := 0.5 * dsrgdb / phiMinVbs;
            else
              sphi := sqrt(out_c.m_tPhi);
              sphi3 := out_c.m_tPhi * sphi;
              sarg := sphi / (1.0 + 0.5 * vbs / out_c.m_tPhi);
              tmp := sarg / sphi3;
              dsrgdb := -0.5 * sarg * tmp;
              d2sdb2 := -dsrgdb * tmp;
            end if;
            if vds - vbs >= 0 then
              barg := sqrt(phiMinVbs + vds);
              dbrgdb := -0.5 / barg;
              d2bdb2 := 0.5 * dbrgdb / (phiMinVbs + vds);
            else
              barg := sphi / (1.0 + 0.5 * (vbs - vds) / out_c.m_tPhi);
              tmp := barg / sphi3;
              dbrgdb := -0.5 * barg * tmp;
              d2bdb2 := -dbrgdb * tmp;
            end if;
            factor := 0.125 * in_p.m_narrowFactor * 2.0 * Modelica.Constants.pi * Spice3.Internal.SpiceConstants.EPSSIL / out_c.m_capOx * out_c.m_lEff;
            eta := 1.0 + factor;
            vbin := out_c.m_tVbi * in_m_type + factor * phiMinVbs;
            if in_p.m_gamma > 0.0 or in_p.m_substrateDoping > 0.0 then
              xwd := in_p.m_xd * barg;
              xws := in_p.m_xd * sarg;
              argss := 0.0;
              argsd := 0.0;
              dbargs := 0.0;
              dbargd := 0.0;
              dgdvds := 0.0;
              dgddb2 := 0.0;
              if in_p.m_junctionDepth > 0 then
                tmp := 2.0 / in_p.m_junctionDepth;
                argxs := 1.0 + xws * tmp;
                argxd := 1.0 + xwd * tmp;
                args := sqrt(argxs);
                argd := sqrt(argxd);
                tmp := 0.5 * in_p.m_junctionDepth / out_c.m_lEff;
                argss := tmp * (args - 1.0);
                argsd := tmp * (argd - 1.0);
              end if;
              gamasd := in_p.m_gamma * (1.0 - argss - argsd);
              dbxwd := in_p.m_xd * dbrgdb;
              dbxws := in_p.m_xd * dsrgdb;
              if in_p.m_junctionDepth > 0 then
                tmp := 0.5 / out_c.m_lEff;
                dbargs := tmp * dbxws / args;
                dbargd := tmp * dbxwd / argd;
                dasdb2 := -in_p.m_xd * (d2sdb2 + dsrgdb * dsrgdb * in_p.m_xd / (in_p.m_junctionDepth * argxs)) / (out_c.m_lEff * args);
                daddb2 := -in_p.m_xd * (d2bdb2 + dbrgdb * dbrgdb * in_p.m_xd / (in_p.m_junctionDepth * argxd)) / (out_c.m_lEff * argd);
                dgddb2 := -0.5 * in_p.m_gamma * (dasdb2 + daddb2);
              end if;
              dgddvb := -in_p.m_gamma * (dbargs + dbargd);
              if in_p.m_junctionDepth > 0 then
                ddxwd := -dbxwd;
                dgdvds := -in_p.m_gamma * 0.5 * ddxwd / (out_c.m_lEff * argd);
              end if;
            else
              gamasd := in_p.m_gamma;
              gammad := in_p.m_gamma;
              dgddvb := 0.0;
              dgdvds := 0.0;
              dgddb2 := 0.0;
            end if;
            out_c.m_von := vbin + gamasd * sarg;
            vth := out_c.m_von;
            out_c.m_vdsat := 0.0;
            if in_p.m_fastSurfaceStateDensity <> 0.0 and out_c.m_capOx <> 0.0 then
              cfs := Spice3.Internal.SpiceConstants.CHARGE * in_p.m_fastSurfaceStateDensity * 1.0e4;
              cdonco := (-(gamasd * dsrgdb + dgddvb * sarg)) + factor;
              xn := 1.0 + cfs / out_c.m_capOx * in_m.m_width * out_c.m_lEff + cdonco;
              tmp := vt * xn;
              out_c.m_von := out_c.m_von + tmp;
              argg := 1.0 / tmp;
              vgst := vgs - out_c.m_von;
            else
              vgst := vgs - out_c.m_von;
              if vgs <= out_c.m_von then
                out_c.m_gds := 0.0;
                out_c.m_cdrain := 0.0;
                out_c.m_gm := 0.0;
                out_c.m_gmbs := 0.0;
                return;
              end if;
            end if;
// cutoff region
            sarg3 := sarg * sarg * sarg;
            gammad := gamasd;
            dgdvbs := dgddvb;
            body := barg * barg * barg - sarg3;
            gdbdv := 2.0 * gammad * (barg * barg * dbrgdb - sarg * sarg * dsrgdb);
            dodvbs := (-factor) + dgdvbs * sarg + gammad * dsrgdb;
            if in_p.m_fastSurfaceStateDensity <> 0.0 and out_c.m_capOx <> 0.0 then
              dxndvb := 2.0 * dgdvbs * dsrgdb + gammad * d2sdb2 + dgddb2 * sarg;
              dodvbs := dodvbs + vt * dxndvb;
              dxndvd := dgdvds * dsrgdb;
              dodvds := dgdvds * sarg + vt * dxndvd;
            end if;
// evaluate effective mobility and its derivatives
            ufact := 1.0;
            ueff := in_p.m_surfaceMobility * 1e-4;
            dudvgs := 0.0;
            dudvds := 0.0;
            dudvbs := 0.0;
            if out_c.m_capOx > 0.0 then
              tmp := in_p.m_critField * Spice3.Internal.SpiceConstants.EPSSIL * 100 / in_p.m_oxideCapFactor;
              if vgst > tmp then
                ufact := exp(in_p.m_critFieldExp * Modelica.Math.log(tmp / vgst));
                ueff := in_p.m_surfaceMobility * 1.0e-4 * ufact;
                dudvgs := -ufact * in_p.m_critFieldExp / vgst;
                dudvds := 0.0;
                dudvbs := in_p.m_critFieldExp * ufact * dodvbs / vgst;
              end if;
            end if;
// evaluate saturation voltage and its derivatives according to
// Grove-Frohman equation
            vgsx := vgs;
            gammad := gamasd / eta;
            dgdvbs := dgddvb;
            if in_p.m_fastSurfaceStateDensity <> 0 and out_c.m_capOx <> 0 then
              vgsx := max(vgs, out_c.m_von);
            end if;
            if gammad > 0 then
              gammd2 := gammad * gammad;
              argv := (vgsx - vbin) / eta + phiMinVbs;
              if argv <= 0.0 then
                out_c.m_vdsat := 0.0;
                dsdvgs := 0.0;
                dsdvbs := 0.0;
              else
                arg := sqrt(1.0 + 4.0 * argv / gammd2);
                out_c.m_vdsat := (vgsx - vbin) / eta + gammd2 * (1.0 - arg) / 2.0;
                out_c.m_vdsat := max(out_c.m_vdsat, 0.0);
                dsdvgs := (1.0 - 1.0 / arg) / eta;
                dsdvbs := (gammad * (1.0 - arg) + 2.0 * argv / (gammad * arg)) / eta * dgdvbs + 1.0 / arg + factor * dsdvgs;
              end if;
            else
              out_c.m_vdsat := (vgsx - vbin) / eta;
              out_c.m_vdsat := max(out_c.m_vdsat, 0.0);
              dsdvgs := 1.0;
              dsdvbs := 0.0;
            end if;
            if in_p.m_maxDriftVel > 0 then
              v1 := (vgsx - vbin) / eta + phiMinVbs;
              v2 := phiMinVbs;
              xv := in_p.m_maxDriftVel * out_c.m_lEff / ueff;
              a1 := gammad / 0.75;
              b1 := -2.0 * (v1 + xv);
              c1 := -2.0 * gammad * xv;
              d1 := 2.0 * v1 * (v2 + xv) - v2 * v2 - 4.0 / 3.0 * gammad * sarg3;
              b2 := a1 * c1 - 4.0 * d1;
              r1 := (-b1 * b1 / 3.0) + b2;
              s1 := 2.0 * b1 * b1 * (-b1) / 27.0 + b1 * b2 / 3.0 + (-d1) * (a1 * a1 - 4.0 * b1) - c1 * c1;
              s2 := s1 * s1;
              p1 := s2 / 4.0 + r1 * r1 * r1 / 27.0;
              p0 := abs(p1);
              p2 := sqrt(p0);
              sig1[1] := 1.0;
              sig1[2] := -1.0;
              sig1[3] := 1.0;
              sig1[4] := -1.0;
              sig2[1] := 1.0;
              sig2[2] := 1.0;
              sig2[3] := -1.0;
              sig2[4] := -1.0;
              if p1 < 0 then
                y3 := 2.0 * exp(Modelica.Math.log(sqrt(s2 / 4.0 + p0)) / 3.0) * cos(Modelica.Math.atan(-2.0 * p2 / s1) / 3.0) + b1 / 3.0;
              else
                y3 := exp(Modelica.Math.log(abs((-s1 / 2.0) + p2)) / 3.0) + exp(Modelica.Math.log(abs((-s1 / 2.0) - p2)) / 3.0) + b1 / 3.0;
              end if;
              a3 := sqrt(a1 * a1 / 4.0 - b1 + y3);
              b3 := sqrt(y3 * y3 / 4.0 - d1);
              for i in 1:4 loop
                a4[i] := a1 / 2.0 + sig1[i] * a3;
                b4[i] := y3 / 2.0 + sig2[i] * b3;
                delta4 := a4[i] * a4[i] / 4.0 - b4[i];
                if delta4 >= 0 then
                  iknt := iknt + 1;
                  tmp := sqrt(delta4);
                  x4[iknt] := (-a4[i] / 2.0) + tmp;
                  iknt := iknt + 1;
                  x4[iknt] := (-a4[i] / 2.0) - tmp;
                end if;
              end for;
              jknt := 0;
              for j in 1:iknt loop
                if x4[j] > 0 then
                  poly4[j] := x4[j] * x4[j] * x4[j] * x4[j] + a1 * x4[j] * x4[j] * x4[j];
                  poly4[j] := poly4[j] + b1 * x4[j] * x4[j] + c1 * x4[j] + d1;
                  if abs(poly4[j]) <= 1.0e-6 then
                    jknt := jknt + 1;
                    if jknt <= 1 then
                      xvalid := x4[j];
                    end if;
                    if x4[j] <= xvalid then
                      xvalid := x4[j];
                    end if;
                  end if;
                end if;
              end for;
              if jknt > 0 then
                out_c.m_vdsat := xvalid * xvalid - phiMinVbs;
              end if;
            end if;
// evaluate saturation voltage and its derivatives
// according to Baum's theory of scattering velocity saturation
// evaluate effective channel length and its derivatives
            dldvgs := 0.0;
            dldvds := 0.0;
            dldvbs := 0.0;
            if vds <> 0.0 then
              gammad := gamasd;
              if vbs - out_c.m_vdsat <= 0 then
                bsarg := sqrt(out_c.m_vdsat + phiMinVbs);
                dbsrdb := -0.5 / bsarg;
              else
                bsarg := sphi / (1.0 + 0.5 * (vbs - out_c.m_vdsat) / out_c.m_tPhi);
                dbsrdb := -0.5 * bsarg * bsarg / sphi3;
              end if;
              bodys := bsarg * bsarg * bsarg - sarg3;
              gdbdvs := 2.0 * gammad * (bsarg * bsarg * dbsrdb - sarg * sarg * dsrgdb);
              if in_p.m_maxDriftVel <= 0 then
                if in_p.m_substrateDoping <> 0.0 and xlamda <= 0.0 then
                  argv := (vds - out_c.m_vdsat) / 4.0;
                  sargv := sqrt(1.0 + argv * argv);
                  arg := sqrt(argv + sargv);
                  xlfact := in_p.m_xd / (out_c.m_lEff * vds);
                  xlamda := xlfact * arg;
                  dldsat := vds * xlamda / (8.0 * sargv);
                  dldvgs := dldsat * dsdvgs;
                  dldvds := (-xlamda) + dldsat;
                  dldvbs := dldsat * dsdvbs;
                end if;
              else
                argv := (vgsx - vbin) / eta - out_c.m_vdsat;
                xdv := in_p.m_xd / sqrt(in_p.m_channelCharge);
                xlv := in_p.m_maxDriftVel * xdv / (2.0 * ueff);
                vqchan := argv - gammad * bsarg;
                dqdsat := (-1.0) + gammad * dbsrdb;
                vl := in_p.m_maxDriftVel * out_c.m_lEff;
                dfunds := vl * dqdsat - ueff * vqchan;
                dfundg := (vl - ueff * out_c.m_vdsat) / eta;
                dfundb := (-vl * (1.0 + dqdsat - factor / eta)) + ueff * (gdbdvs - dgdvbs * bodys / 1.5) / eta;
                dsdvgs := -dfundg / dfunds;
                dsdvbs := -dfundb / dfunds;
                if in_p.m_substrateDoping <> 0.0 and xlamda <= 0.0 then
                  argv := vds - out_c.m_vdsat;
                  argv := max(argv, 0.0);
                  xls := sqrt(xlv * xlv + argv);
                  dldsat := xdv / (2.0 * xls);
                  xlfact := xdv / (out_c.m_lEff * vds);
                  xlamda := xlfact * (xls - xlv);
                  dldsat := dldsat / out_c.m_lEff;
                  dldvgs := dldsat * dsdvgs;
                  dldvds := (-xlamda) + dldsat;
                  dldvbs := dldsat * dsdvbs;
                end if;
              end if;
            end if;
// limit channel shortening at punch-through
            xwb := in_p.m_xd * sqrt(out_c.m_tBulkPot);
            xld := out_c.m_lEff - xwb;
            clfact := 1.0 - xlamda * vds;
            dldvds := (-xlamda) - dldvds;
            xleff := out_c.m_lEff * clfact;
            deltal := xlamda * vds * out_c.m_lEff;
            if in_p.m_substrateDoping == 0.0 then
              xwb := 0.25e-6;
            end if;
            if xleff < xwb then
              xleff := xwb / (1.0 + (deltal - xld) / xwb);
              clfact := xleff / out_c.m_lEff;
              dfact := xleff * xleff / (xwb * xwb);
              dldvgs := dfact * dldvgs;
              dldvds := dfact * dldvds;
              dldvbs := dfact * dldvbs;
            end if;
// evaluate effective beta (effective kp)
            beta1 := out_c.m_Beta * ufact / clfact;
// test for mode of operation and branch appropriately
            gammad := gamasd;
            dgdvbs := dgddvb;
            if vds <= 1.0e-10 then
              if vgs <= out_c.m_von then
                if in_p.m_fastSurfaceStateDensity == 0.0 or out_c.m_capOx == 0.0 then
                  out_c.m_gds := 0.0;
                else
                  out_c.m_gds := beta1 * (out_c.m_von - vbin - gammad * sarg) * exp(argg * (vgs - out_c.m_von));
                end if;
              else
                out_c.m_gds := beta1 * (vgs - vbin - gammad * sarg);
              end if;
              out_c.m_cdrain := 0.0;
              out_c.m_gm := 0.0;
              out_c.m_gmbs := 0.0;
              return;
            end if;
            if vgs <= out_c.m_von then
              if out_c.m_vdsat <= 0 then
                out_c.m_gds := 0.0;
                if vgs > vth then
                  return;
                end if;
                out_c.m_cdrain := 0.0;
                out_c.m_gm := 0.0;
                out_c.m_gmbs := 0.0;
                return;
              end if;
              vdson := min(out_c.m_vdsat, vds);
              if vds > out_c.m_vdsat then
                barg := bsarg;
                dbrgdb := dbsrdb;
                body := bodys;
                gdbdv := gdbdvs;
              end if;
              cdson := beta1 * ((out_c.m_von - vbin - eta * vdson * 0.5) * vdson - gammad * body / 1.5);
              didvds := beta1 * (out_c.m_von - vbin - eta * vdson - gammad * barg);
              gdson := (-cdson * dldvds / clfact) - beta1 * dgdvds * body / 1.5;
              if vds < out_c.m_vdsat then
                gdson := gdson + didvds;
              end if;
              gbson := (-cdson * dldvbs / clfact) + beta1 * (dodvbs * vdson + factor * vdson - dgdvbs * body / 1.5 - gdbdv);
              if vds > out_c.m_vdsat then
                gbson := gbson + didvds * dsdvbs;
              end if;
              expg := exp(argg * (vgs - out_c.m_von));
              out_c.m_cdrain := cdson * expg;
              gmw := out_c.m_cdrain * argg;
              out_c.m_gm := gmw;
              if vds > out_c.m_vdsat then
                out_c.m_gm := gmw + didvds * dsdvgs * expg;
              end if;
              tmp := gmw * (vgs - out_c.m_von) / xn;
              out_c.m_gds := gdson * expg - out_c.m_gm * dodvds - tmp * dxndvd;
              out_c.m_gmbs := gbson * expg - out_c.m_gm * dodvbs - tmp * dxndvb;
            elseif vds <= out_c.m_vdsat then
              out_c.m_cdrain := beta1 * ((vgs - vbin - eta * vds / 2.0) * vds - gammad * body / 1.5);
              arg := out_c.m_cdrain * (dudvgs / ufact - dldvgs / clfact);
              out_c.m_gm := arg + beta1 * vds;
              arg := out_c.m_cdrain * (dudvds / ufact - dldvds / clfact);
              out_c.m_gds := arg + beta1 * (vgs - vbin - eta * vds - gammad * barg - dgdvds * body / 1.5);
              arg := out_c.m_cdrain * (dudvbs / ufact - dldvbs / clfact);
              out_c.m_gmbs := arg - beta1 * (gdbdv + dgdvbs * body / 1.5 - factor * vds);
            else
              out_c.m_cdrain := beta1 * ((vgs - vbin - eta * out_c.m_vdsat / 2.0) * out_c.m_vdsat - gammad * bodys / 1.5);
              arg := out_c.m_cdrain * (dudvgs / ufact - dldvgs / clfact);
              out_c.m_gm := arg + beta1 * out_c.m_vdsat + beta1 * (vgs - vbin - eta * out_c.m_vdsat - gammad * bsarg) * dsdvgs;
              out_c.m_gds := (-out_c.m_cdrain * dldvds / clfact) - beta1 * dgdvds * bodys / 1.5;
              arg := out_c.m_cdrain * (dudvbs / ufact - dldvbs / clfact);
              out_c.m_gmbs := arg - beta1 * (gdbdvs + dgdvbs * bodys / 1.5 - factor * out_c.m_vdsat) + beta1 * (vgs - vbin - eta * out_c.m_vdsat - gammad * bsarg) * dsdvbs;
            end if;
// subthreshold region
// linear region
// saturation region
            annotation(
              Documentation(info = "<html>
<p>This function drainCur calculates the main currents that flows from drain node to source node (level 2).</p>
</html>"));
          end drainCurRevised;

          function drainCur "Drain current calculation obsolete, use drainCurRevised"
            extends Modelica.Icons.Function;
            extends Modelica.Icons.ObsoleteModel;
            input Modelica.SIunits.Voltage vbs;
            input Modelica.SIunits.Voltage vgs;
            input Modelica.SIunits.Voltage vds;
            input Spice3.Internal.Mosfet.Mosfet in_m "Record MOSFET";
            input Mos2Calc in_c "Input record Mos2Calc";
            input Mos2ModelLineParams in_p "Input record model line parameters for MOS2";
            input Spice3.Internal.SpiceConstants in_C "Spice constants";
            input Mos2ModelLineVariables in_vp "Input record model line variables";
            input Integer in_m_type "Type of MOS transistor";
            output Mos2Calc out_c "Output record Mos2Calc";
          protected
            Real vt;
            // K * T / Q
            Real beta1;
            Real dsrgdb;
            Real d2sdb2;
            Real sphi = 0.0;
            Real sphi3 = 1.0;
            // square root of phi
            Real barg;
            Real sarg;
            Real bsarg = 0.0;
            Real sarg3;
            Real d2bdb2;
            Real factor;
            Real dbrgdb;
            Real eta;
            Real vbin;
            Real vth;
            Real dgddb2;
            Real dgddvb;
            Real dgdvds;
            Real gamasd;
            Real gammad;
            Real xn = 1.0;
            Real argg = 0.0;
            Real vgst;
            Real vgsx;
            Real dgdvbs;
            Real body;
            Real bodys = 0.0;
            Real gdbdv;
            Real dodvbs;
            Real dodvds = 0.0;
            Real dxndvd = 0.0;
            Real dxndvb = 0.0;
            Real dudvgs;
            Real dudvds;
            Real dudvbs;
            Real ufact;
            Real ueff;
            Real dsdvgs;
            Real dsdvbs;
            Real dbsrdb;
            Real gdbdvs = 0.0;
            Real dldvgs;
            Real dldvds;
            Real dldvbs;
            Real clfact;
            Real xleff;
            Real deltal;
            Real xwb;
            Real xld;
            Real xlamda = in_p.m_lambda;
            Real phiMinVbs;
            Real tmp;
            Real argss;
            Real argsd;
            Real args = 0.0;
            Real argd = 0.0;
            Real argxs = 0.0;
            Real argxd = 0.0;
            Real dbargs;
            Real dbargd;
            Real dbxws;
            Real dbxwd;
            Real xwd;
            Real xws;
            Real daddb2;
            Real dasdb2;
            Real ddxwd;
            Real cfs;
            Real cdonco;
            Real argv;
            Real gammd2;
            Real arg;
            Real y3;
            Real xvalid = 0.0;
            Real[4] sig1;
            Real[4] sig2;
            Real[4] a4;
            Real[4] b4;
            Real[8] x4;
            Real[8] poly4;
            Real delta4;
            Integer j;
            Integer iknt = 0;
            Integer i;
            Integer jknt = 0;
            Real v1;
            Real v2;
            Real xv;
            Real a1;
            Real b1;
            Real c1;
            Real d1;
            Real b2;
            Real r1;
            Real s1;
            Real s2;
            Real p1;
            Real p0;
            Real p2;
            Real a3;
            Real b3;
            Real sargv;
            Real dldsat;
            Real xlfact;
            Real xdv;
            Real xlv;
            Real vqchan;
            Real dqdsat;
            Real vl;
            Real dfunds;
            Real dfundg;
            Real dfundb;
            Real xls;
            Real dfact;
            Real vdson;
            Real cdson;
            Real gdson;
            Real didvds;
            Real gmw;
            Real gbson;
            Real expg;
          algorithm
            out_c := in_c;
            vt := in_C.CONSTKoverQ * in_C.REFTEMP;
            phiMinVbs := out_c.m_tPhi - vbs;
            if vbs <= 0.0 then
              sarg := sqrt(phiMinVbs);
              dsrgdb := -0.5 / sarg;
              d2sdb2 := 0.5 * dsrgdb / phiMinVbs;
            else
              sphi := sqrt(out_c.m_tPhi);
              sphi3 := out_c.m_tPhi * sphi;
              sarg := sphi / (1.0 + 0.5 * vbs / out_c.m_tPhi);
              tmp := sarg / sphi3;
              dsrgdb := -0.5 * sarg * tmp;
              d2sdb2 := -dsrgdb * tmp;
            end if;
            if vds - vbs >= 0 then
              barg := sqrt(phiMinVbs + vds);
              dbrgdb := -0.5 / barg;
              d2bdb2 := 0.5 * dbrgdb / (phiMinVbs + vds);
            else
              barg := sphi / (1.0 + 0.5 * (vbs - vds) / out_c.m_tPhi);
              tmp := barg / sphi3;
              dbrgdb := -0.5 * barg * tmp;
              d2bdb2 := -dbrgdb * tmp;
            end if;
            factor := 0.125 * in_p.m_narrowFactor * 2.0 * Modelica.Constants.pi * in_C.EPSSIL / out_c.m_capOx * out_c.m_lEff;
            eta := 1.0 + factor;
            vbin := out_c.m_tVbi * in_m_type + factor * phiMinVbs;
            if in_vp.m_gamma > 0.0 or in_vp.m_substrateDoping > 0.0 then
              xwd := in_vp.m_xd * barg;
              xws := in_vp.m_xd * sarg;
              argss := 0.0;
              argsd := 0.0;
              dbargs := 0.0;
              dbargd := 0.0;
              dgdvds := 0.0;
              dgddb2 := 0.0;
              if in_p.m_junctionDepth > 0 then
                tmp := 2.0 / in_p.m_junctionDepth;
                argxs := 1.0 + xws * tmp;
                argxd := 1.0 + xwd * tmp;
                args := sqrt(argxs);
                argd := sqrt(argxd);
                tmp := 0.5 * in_p.m_junctionDepth / out_c.m_lEff;
                argss := tmp * (args - 1.0);
                argsd := tmp * (argd - 1.0);
              end if;
              gamasd := in_vp.m_gamma * (1.0 - argss - argsd);
              dbxwd := in_vp.m_xd * dbrgdb;
              dbxws := in_vp.m_xd * dsrgdb;
              if in_p.m_junctionDepth > 0 then
                tmp := 0.5 / out_c.m_lEff;
                dbargs := tmp * dbxws / args;
                dbargd := tmp * dbxwd / argd;
                dasdb2 := -in_vp.m_xd * (d2sdb2 + dsrgdb * dsrgdb * in_vp.m_xd / (in_p.m_junctionDepth * argxs)) / (out_c.m_lEff * args);
                daddb2 := -in_vp.m_xd * (d2bdb2 + dbrgdb * dbrgdb * in_vp.m_xd / (in_p.m_junctionDepth * argxd)) / (out_c.m_lEff * argd);
                dgddb2 := -0.5 * in_vp.m_gamma * (dasdb2 + daddb2);
              end if;
              dgddvb := -in_vp.m_gamma * (dbargs + dbargd);
              if in_p.m_junctionDepth > 0 then
                ddxwd := -dbxwd;
                dgdvds := -in_vp.m_gamma * 0.5 * ddxwd / (out_c.m_lEff * argd);
              end if;
            else
              gamasd := in_vp.m_gamma;
              gammad := in_vp.m_gamma;
              dgddvb := 0.0;
              dgdvds := 0.0;
              dgddb2 := 0.0;
            end if;
            out_c.m_von := vbin + gamasd * sarg;
            vth := out_c.m_von;
            out_c.m_vdsat := 0.0;
            if in_p.m_fastSurfaceStateDensity <> 0.0 and out_c.m_capOx <> 0.0 then
              cfs := in_C.CHARGE * in_p.m_fastSurfaceStateDensity * 1.0e4;
              cdonco := (-(gamasd * dsrgdb + dgddvb * sarg)) + factor;
              xn := 1.0 + cfs / out_c.m_capOx * in_m.m_width * out_c.m_lEff + cdonco;
              tmp := vt * xn;
              out_c.m_von := out_c.m_von + tmp;
              argg := 1.0 / tmp;
              vgst := vgs - out_c.m_von;
            else
              vgst := vgs - out_c.m_von;
              if vgs <= out_c.m_von then
                out_c.m_gds := 0.0;
                out_c.m_cdrain := 0.0;
                out_c.m_gm := 0.0;
                out_c.m_gmbs := 0.0;
                return;
              end if;
            end if;
// cutoff region
            sarg3 := sarg * sarg * sarg;
            gammad := gamasd;
            dgdvbs := dgddvb;
            body := barg * barg * barg - sarg3;
            gdbdv := 2.0 * gammad * (barg * barg * dbrgdb - sarg * sarg * dsrgdb);
            dodvbs := (-factor) + dgdvbs * sarg + gammad * dsrgdb;
            if in_p.m_fastSurfaceStateDensity <> 0.0 and out_c.m_capOx <> 0.0 then
              dxndvb := 2.0 * dgdvbs * dsrgdb + gammad * d2sdb2 + dgddb2 * sarg;
              dodvbs := dodvbs + vt * dxndvb;
              dxndvd := dgdvds * dsrgdb;
              dodvds := dgdvds * sarg + vt * dxndvd;
            end if;
// evaluate effective mobility and its derivatives
            ufact := 1.0;
            ueff := in_p.m_surfaceMobility * 1e-4;
            dudvgs := 0.0;
            dudvds := 0.0;
            dudvbs := 0.0;
            if out_c.m_capOx > 0.0 then
              tmp := in_p.m_critField * in_C.EPSSIL * 100 / in_vp.m_oxideCapFactor;
              if vgst > tmp then
                ufact := exp(in_p.m_critFieldExp * Modelica.Math.log(tmp / vgst));
                ueff := in_p.m_surfaceMobility * 1.0e-4 * ufact;
                dudvgs := -ufact * in_p.m_critFieldExp / vgst;
                dudvds := 0.0;
                dudvbs := in_p.m_critFieldExp * ufact * dodvbs / vgst;
              end if;
            end if;
// evaluate saturation voltage and its derivatives according to
// Grove-Frohman equation
            vgsx := vgs;
            gammad := gamasd / eta;
            dgdvbs := dgddvb;
            if in_p.m_fastSurfaceStateDensity <> 0 and out_c.m_capOx <> 0 then
              vgsx := max(vgs, out_c.m_von);
            end if;
            if gammad > 0 then
              gammd2 := gammad * gammad;
              argv := (vgsx - vbin) / eta + phiMinVbs;
              if argv <= 0.0 then
                out_c.m_vdsat := 0.0;
                dsdvgs := 0.0;
                dsdvbs := 0.0;
              else
                arg := sqrt(1.0 + 4.0 * argv / gammd2);
                out_c.m_vdsat := (vgsx - vbin) / eta + gammd2 * (1.0 - arg) / 2.0;
                out_c.m_vdsat := max(out_c.m_vdsat, 0.0);
                dsdvgs := (1.0 - 1.0 / arg) / eta;
                dsdvbs := (gammad * (1.0 - arg) + 2.0 * argv / (gammad * arg)) / eta * dgdvbs + 1.0 / arg + factor * dsdvgs;
              end if;
            else
              out_c.m_vdsat := (vgsx - vbin) / eta;
              out_c.m_vdsat := max(out_c.m_vdsat, 0.0);
              dsdvgs := 1.0;
              dsdvbs := 0.0;
            end if;
            if in_p.m_maxDriftVel > 0 then
              v1 := (vgsx - vbin) / eta + phiMinVbs;
              v2 := phiMinVbs;
              xv := in_p.m_maxDriftVel * out_c.m_lEff / ueff;
              a1 := gammad / 0.75;
              b1 := -2.0 * (v1 + xv);
              c1 := -2.0 * gammad * xv;
              d1 := 2.0 * v1 * (v2 + xv) - v2 * v2 - 4.0 / 3.0 * gammad * sarg3;
              b2 := a1 * c1 - 4.0 * d1;
              r1 := (-b1 * b1 / 3.0) + b2;
              s1 := 2.0 * b1 * b1 * (-b1) / 27.0 + b1 * b2 / 3.0 + (-d1) * (a1 * a1 - 4.0 * b1) - c1 * c1;
              s2 := s1 * s1;
              p1 := s2 / 4.0 + r1 * r1 * r1 / 27.0;
              p0 := abs(p1);
              p2 := sqrt(p0);
              sig1[1] := 1.0;
              sig1[2] := -1.0;
              sig1[3] := 1.0;
              sig1[4] := -1.0;
              sig2[1] := 1.0;
              sig2[2] := 1.0;
              sig2[3] := -1.0;
              sig2[4] := -1.0;
              if p1 < 0 then
                y3 := 2.0 * exp(Modelica.Math.log(sqrt(s2 / 4.0 + p0)) / 3.0) * cos(Modelica.Math.atan(-2.0 * p2 / s1) / 3.0) + b1 / 3.0;
              else
                y3 := exp(Modelica.Math.log(abs((-s1 / 2.0) + p2)) / 3.0) + exp(Modelica.Math.log(abs((-s1 / 2.0) - p2)) / 3.0) + b1 / 3.0;
              end if;
              a3 := sqrt(a1 * a1 / 4.0 - b1 + y3);
              b3 := sqrt(y3 * y3 / 4.0 - d1);
              for i in 1:4 loop
                a4[i] := a1 / 2.0 + sig1[i] * a3;
                b4[i] := y3 / 2.0 + sig2[i] * b3;
                delta4 := a4[i] * a4[i] / 4.0 - b4[i];
                if delta4 >= 0 then
                  iknt := iknt + 1;
                  tmp := sqrt(delta4);
                  x4[iknt] := (-a4[i] / 2.0) + tmp;
                  iknt := iknt + 1;
                  x4[iknt] := (-a4[i] / 2.0) - tmp;
                end if;
              end for;
              jknt := 0;
              for j in 1:iknt loop
                if x4[j] > 0 then
                  poly4[j] := x4[j] * x4[j] * x4[j] * x4[j] + a1 * x4[j] * x4[j] * x4[j];
                  poly4[j] := poly4[j] + b1 * x4[j] * x4[j] + c1 * x4[j] + d1;
                  if abs(poly4[j]) <= 1.0e-6 then
                    jknt := jknt + 1;
                    if jknt <= 1 then
                      xvalid := x4[j];
                    end if;
                    if x4[j] <= xvalid then
                      xvalid := x4[j];
                    end if;
                  end if;
                end if;
              end for;
              if jknt > 0 then
                out_c.m_vdsat := xvalid * xvalid - phiMinVbs;
              end if;
            end if;
// evaluate saturation voltage and its derivatives
// according to Baum's theory of scattering velocity saturation
// evaluate effective channel length and its derivatives
            dldvgs := 0.0;
            dldvds := 0.0;
            dldvbs := 0.0;
            if vds <> 0.0 then
              gammad := gamasd;
              if vbs - out_c.m_vdsat <= 0 then
                bsarg := sqrt(out_c.m_vdsat + phiMinVbs);
                dbsrdb := -0.5 / bsarg;
              else
                bsarg := sphi / (1.0 + 0.5 * (vbs - out_c.m_vdsat) / out_c.m_tPhi);
                dbsrdb := -0.5 * bsarg * bsarg / sphi3;
              end if;
              bodys := bsarg * bsarg * bsarg - sarg3;
              gdbdvs := 2.0 * gammad * (bsarg * bsarg * dbsrdb - sarg * sarg * dsrgdb);
              if in_p.m_maxDriftVel <= 0 then
                if in_vp.m_substrateDoping <> 0.0 and xlamda <= 0.0 then
                  argv := (vds - out_c.m_vdsat) / 4.0;
                  sargv := sqrt(1.0 + argv * argv);
                  arg := sqrt(argv + sargv);
                  xlfact := in_vp.m_xd / (out_c.m_lEff * vds);
                  xlamda := xlfact * arg;
                  dldsat := vds * xlamda / (8.0 * sargv);
                  dldvgs := dldsat * dsdvgs;
                  dldvds := (-xlamda) + dldsat;
                  dldvbs := dldsat * dsdvbs;
                end if;
              else
                argv := (vgsx - vbin) / eta - out_c.m_vdsat;
                xdv := in_vp.m_xd / sqrt(in_p.m_channelCharge);
                xlv := in_p.m_maxDriftVel * xdv / (2.0 * ueff);
                vqchan := argv - gammad * bsarg;
                dqdsat := (-1.0) + gammad * dbsrdb;
                vl := in_p.m_maxDriftVel * out_c.m_lEff;
                dfunds := vl * dqdsat - ueff * vqchan;
                dfundg := (vl - ueff * out_c.m_vdsat) / eta;
                dfundb := (-vl * (1.0 + dqdsat - factor / eta)) + ueff * (gdbdvs - dgdvbs * bodys / 1.5) / eta;
                dsdvgs := -dfundg / dfunds;
                dsdvbs := -dfundb / dfunds;
                if in_vp.m_substrateDoping <> 0.0 and xlamda <= 0.0 then
                  argv := vds - out_c.m_vdsat;
                  argv := max(argv, 0.0);
                  xls := sqrt(xlv * xlv + argv);
                  dldsat := xdv / (2.0 * xls);
                  xlfact := xdv / (out_c.m_lEff * vds);
                  xlamda := xlfact * (xls - xlv);
                  dldsat := dldsat / out_c.m_lEff;
                  dldvgs := dldsat * dsdvgs;
                  dldvds := (-xlamda) + dldsat;
                  dldvbs := dldsat * dsdvbs;
                end if;
              end if;
            end if;
// limit channel shortening at punch-through
            xwb := in_vp.m_xd * sqrt(out_c.m_tBulkPot);
            xld := out_c.m_lEff - xwb;
            clfact := 1.0 - xlamda * vds;
            dldvds := (-xlamda) - dldvds;
            xleff := out_c.m_lEff * clfact;
            deltal := xlamda * vds * out_c.m_lEff;
            if in_vp.m_substrateDoping == 0.0 then
              xwb := 0.25e-6;
            end if;
            if xleff < xwb then
              xleff := xwb / (1.0 + (deltal - xld) / xwb);
              clfact := xleff / out_c.m_lEff;
              dfact := xleff * xleff / (xwb * xwb);
              dldvgs := dfact * dldvgs;
              dldvds := dfact * dldvds;
              dldvbs := dfact * dldvbs;
            end if;
// evaluate effective beta (effective kp)
            beta1 := out_c.m_Beta * ufact / clfact;
// test for mode of operation and branch appropriately
            gammad := gamasd;
            dgdvbs := dgddvb;
            if vds <= 1.0e-10 then
              if vgs <= out_c.m_von then
                if in_p.m_fastSurfaceStateDensity == 0.0 or out_c.m_capOx == 0.0 then
                  out_c.m_gds := 0.0;
                else
                  out_c.m_gds := beta1 * (out_c.m_von - vbin - gammad * sarg) * exp(argg * (vgs - out_c.m_von));
                end if;
              else
                out_c.m_gds := beta1 * (vgs - vbin - gammad * sarg);
              end if;
              out_c.m_cdrain := 0.0;
              out_c.m_gm := 0.0;
              out_c.m_gmbs := 0.0;
              return;
            end if;
            if vgs <= out_c.m_von then
              if out_c.m_vdsat <= 0 then
                out_c.m_gds := 0.0;
                if vgs > vth then
                  return;
                end if;
                out_c.m_cdrain := 0.0;
                out_c.m_gm := 0.0;
                out_c.m_gmbs := 0.0;
                return;
              end if;
              vdson := min(out_c.m_vdsat, vds);
              if vds > out_c.m_vdsat then
                barg := bsarg;
                dbrgdb := dbsrdb;
                body := bodys;
                gdbdv := gdbdvs;
              end if;
              cdson := beta1 * ((out_c.m_von - vbin - eta * vdson * 0.5) * vdson - gammad * body / 1.5);
              didvds := beta1 * (out_c.m_von - vbin - eta * vdson - gammad * barg);
              gdson := (-cdson * dldvds / clfact) - beta1 * dgdvds * body / 1.5;
              if vds < out_c.m_vdsat then
                gdson := gdson + didvds;
              end if;
              gbson := (-cdson * dldvbs / clfact) + beta1 * (dodvbs * vdson + factor * vdson - dgdvbs * body / 1.5 - gdbdv);
              if vds > out_c.m_vdsat then
                gbson := gbson + didvds * dsdvbs;
              end if;
              expg := exp(argg * (vgs - out_c.m_von));
              out_c.m_cdrain := cdson * expg;
              gmw := out_c.m_cdrain * argg;
              out_c.m_gm := gmw;
              if vds > out_c.m_vdsat then
                out_c.m_gm := gmw + didvds * dsdvgs * expg;
              end if;
              tmp := gmw * (vgs - out_c.m_von) / xn;
              out_c.m_gds := gdson * expg - out_c.m_gm * dodvds - tmp * dxndvd;
              out_c.m_gmbs := gbson * expg - out_c.m_gm * dodvbs - tmp * dxndvb;
            elseif vds <= out_c.m_vdsat then
              out_c.m_cdrain := beta1 * ((vgs - vbin - eta * vds / 2.0) * vds - gammad * body / 1.5);
              arg := out_c.m_cdrain * (dudvgs / ufact - dldvgs / clfact);
              out_c.m_gm := arg + beta1 * vds;
              arg := out_c.m_cdrain * (dudvds / ufact - dldvds / clfact);
              out_c.m_gds := arg + beta1 * (vgs - vbin - eta * vds - gammad * barg - dgdvds * body / 1.5);
              arg := out_c.m_cdrain * (dudvbs / ufact - dldvbs / clfact);
              out_c.m_gmbs := arg - beta1 * (gdbdv + dgdvbs * body / 1.5 - factor * vds);
            else
              out_c.m_cdrain := beta1 * ((vgs - vbin - eta * out_c.m_vdsat / 2.0) * out_c.m_vdsat - gammad * bodys / 1.5);
              arg := out_c.m_cdrain * (dudvgs / ufact - dldvgs / clfact);
              out_c.m_gm := arg + beta1 * out_c.m_vdsat + beta1 * (vgs - vbin - eta * out_c.m_vdsat - gammad * bsarg) * dsdvgs;
              out_c.m_gds := (-out_c.m_cdrain * dldvds / clfact) - beta1 * dgdvds * bodys / 1.5;
              arg := out_c.m_cdrain * (dudvbs / ufact - dldvbs / clfact);
              out_c.m_gmbs := arg - beta1 * (gdbdvs + dgdvbs * bodys / 1.5 - factor * out_c.m_vdsat) + beta1 * (vgs - vbin - eta * out_c.m_vdsat - gammad * bsarg) * dsdvbs;
            end if;
// subthreshold region
// linear region
// saturation region
            annotation(
              Documentation(info = "<html>
<p>This function drainCur calculates the main currents that flows from drain node to source node (level 2).</p>
</html>"));
          end drainCur;

          function mos2RenameParametersRevised "Parameter renaming to internal names"
            extends Modelica.Icons.Function;
            input Spice3.Internal.ModelcardMOS2 ex "Modelcard with technology parameters";
            output Modelica.Electrical.Spice3.Internal.Mos2.Mos2ModelLineParams intern "Output record model line parameters";
          algorithm
            intern.m_narrowFactor := ex.DELTA;
// DELTA, Width effect on threshold
            intern.m_critFieldExp := ex.UEXP;
// UEXP, Crit. field exp for mob. deg
            intern.m_critField := ex.UCRIT;
// UCRIT, Crit. field for mob. degradation
            intern.m_maxDriftVel := ex.VMAX;
// VMAX, Maximum carrier drift velocity
            intern.m_junctionDepth := ex.XJ;
// XJ, Junction depth
            intern.m_channelCharge := ex.NEFF;
// NEFF, Total channel charge coeff
            intern.m_fastSurfaceStateDensity := ex.NFS;
// NFS, Fast surface state density
// MosModelLineParams
            intern.m_oxideCapFactor := 0;
            intern.m_vtOIsGiven := if ex.VTO > (-1e40) then 1 else 0;
            intern.m_vt0 := if ex.VTO > (-1e40) then ex.VTO else 0;
            intern.m_capBDIsGiven := if ex.CBD > (-1e40) then 1 else 0;
            intern.m_capBD := if ex.CBD > (-1e40) then ex.CBD else 0;
            intern.m_capBSIsGiven := if ex.CBS > (-1e40) then 1 else 0;
            intern.m_capBS := if ex.CBS > (-1e40) then ex.CBS else 0;
            intern.m_bulkCapFactor := ex.CJ;
// F/(m*m) zero-bias bulk junction bottom cap. per sq-meter of junction area (default 0)
            intern.m_sideWallCapFactor := ex.CJSW;
// F/m zero-bias junction sidewall cap. per meter of junction perimeter (default 0)
            intern.m_fwdCapDepCoeff := ex.FC;
// coefficient for forward-bias depletion capacitance formula (default 0.5)
            intern.m_phiIsGiven := if ex.PHI > (-1e40) then 1 else 0;
            intern.m_phi := if ex.PHI > (-1e40) then ex.PHI else 0.6;
            intern.m_gammaIsGiven := if ex.GAMMA > (-1e40) then 1 else 0;
            intern.m_gamma := if ex.GAMMA > (-1e40) then ex.GAMMA else 0;
            intern.m_lambda := ex.LAMBDA;
// 1/V channel-length modulation (default 0)
            intern.m_substrateDopingIsGiven := if ex.NSUB > (-1e40) then 1 else 0;
            intern.m_substrateDoping := if ex.NSUB > (-1e40) then ex.NSUB else 0;
            intern.m_gateType := ex.TPG;
// type of gate material: +1 opp. to substrate, -1 same as substrate, 0 Al gate (default 1)
            intern.m_surfaceStateDensity := ex.NSS;
// 1/(cm*cm) surface state density (default 0)
            intern.m_surfaceMobility := ex.UO;
// (cm*cm)/(Vs) surface mobility (default 600)
            intern.m_latDiff := ex.LD;
// m lateral diffusion (default 0)
            intern.m_jctSatCur := ex.IS;
// A bulk junction saturation current (default 1e-14)
            intern.m_drainResistanceIsGiven := if ex.RD > (-1e40) then 1 else 0;
            intern.m_drainResistance := if ex.RD > (-1e40) then ex.RD else 0;
            intern.m_sourceResistanceIsGiven := if ex.RS > (-1e40) then 1 else 0;
            intern.m_sourceResistance := if ex.RS > (-1e40) then ex.RS else 0;
            intern.m_transconductanceIsGiven := if ex.KP > (-1e40) then 1 else 0;
            intern.m_transconductance := if ex.KP > (-1e40) then ex.KP else 2e-5;
            intern.m_tnom := ex.TNOM + Spice3.Internal.SpiceConstants.CONSTCtoK;
// MosfetModelLineParams
            intern.m_jctSatCurDensity := ex.JS;
// A/(m*m) bulk junction saturation current per sq-meter of junction area (default 0)
            intern.m_sheetResistance := ex.RSH;
// Ohm drain and source diffusion sheet resistance (default 0)
            intern.m_bulkJctPotential := ex.PB;
// V bulk junction potential (default 0.8)
            intern.m_bulkJctBotGradingCoeff := ex.MJ;
// bulk junction bottom grading coeff. (default 0.5)
            intern.m_bulkJctSideGradingCoeff := ex.MJSW;
// bulk junction sidewall grading coeff. (default 0.5)
            intern.m_oxideThickness := if ex.TOX > (-1e40) then ex.TOX else 1e-7;
// m oxide thickness (default 1e-7)
            intern.m_gateSourceOverlapCapFactor := ex.CGSO;
// F/m gate-source overlap capacitance per meter channel width (default 0)
            intern.m_gateDrainOverlapCapFactor := ex.CGDO;
// F/m gate-drain overlap capacitance per meter channel width (default 0)
            intern.m_gateBulkOverlapCapFactor := ex.CGBO;
// F/m gate-bulk overlap capacitance per meter channel width (default 0)
            intern.m_fNcoef := ex.KF;
// flicker-noise coefficient (default 0)
            intern.m_fNexp := ex.AF;
// flicker-noise exponent (default 1)
            annotation(
              Documentation(info = "<html>
<p>This function mos2RenameParameters assigns the external (given by the user, e.g., RD) technology parameters
to the internal parameters (e.g., m_drainResistance). It also does the analysis of the IsGiven values (level 2).</p>
</html>"));
          end mos2RenameParametersRevised;

          function mos2RenameParameters "Parameter renaming to internal names
             obsolete, use mos2RenameParametersRevised"
            extends Modelica.Icons.Function;
            extends Modelica.Icons.ObsoleteModel;
            input Spice3.Internal.ModelcardMOS2 ex "Modelcard with technology parameters";
            input Spice3.Internal.SpiceConstants con "Spice constants";
            output Mos2ModelLineParams intern "Output record model line parameters";
          algorithm
            intern.m_narrowFactor := ex.DELTA;
// DELTA, Width effect on threshold
            intern.m_critFieldExp := ex.UEXP;
// UEXP, Crit. field exp for mob. deg
            intern.m_critField := ex.UCRIT;
// UCRIT, Crit. field for mob. degradation
            intern.m_maxDriftVel := ex.VMAX;
// VMAX, Maximum carrier drift velocity
            intern.m_junctionDepth := ex.XJ;
// XJ, Junction depth
            intern.m_channelCharge := ex.NEFF;
// NEFF, Total channel charge coeff
            intern.m_fastSurfaceStateDensity := ex.NFS;
// NFS, Fast surface state density
            intern.m_oxideCapFactor := 0;
            intern.m_vtOIsGiven := if ex.VTO > (-1e40) then 1 else 0;
            intern.m_vt0 := if ex.VTO > (-1e40) then ex.VTO else 0;
            intern.m_capBDIsGiven := if ex.CBD > (-1e40) then 1 else 0;
            intern.m_capBD := if ex.CBD > (-1e40) then ex.CBD else 0;
            intern.m_capBSIsGiven := if ex.CBS > (-1e40) then 1 else 0;
            intern.m_capBS := if ex.CBS > (-1e40) then ex.CBS else 0;
            intern.m_bulkCapFactor := ex.CJ;
// F/(m*m) zero-bias bulk junction bottom cap. per sq-meter of junction area (default 0)
            intern.m_sideWallCapFactor := ex.CJSW;
// F/m zero-bias junction sidewall cap. per meter of junction perimeter (default 0)
            intern.m_fwdCapDepCoeff := ex.FC;
// coefficient for forward-bias depletion capacitance formula (default 0.5)
            intern.m_phiIsGiven := if ex.PHI > (-1e40) then 1 else 0;
            intern.m_phi := if ex.PHI > (-1e40) then ex.PHI else 0.6;
            intern.m_gammaIsGiven := if ex.GAMMA > (-1e40) then 1 else 0;
            intern.m_gamma := if ex.GAMMA > (-1e40) then ex.GAMMA else 0;
            intern.m_lambda := ex.LAMBDA;
// 1/V channel-length modulation (default 0)
            intern.m_substrateDopingIsGiven := if ex.NSUB > (-1e40) then 1 else 0;
            intern.m_substrateDoping := if ex.NSUB > (-1e40) then ex.NSUB else 0;
            intern.m_gateType := ex.TPG;
// type of gate material: +1 opp. to substrate, -1 same as substrate, 0 Al gate (default 1)
            intern.m_surfaceStateDensity := ex.NSS;
// 1/(cm*cm) surface state density (default 0)
            intern.m_surfaceMobility := ex.UO;
// (cm*cm)/(Vs) surface mobility (default 600)
            intern.m_latDiff := ex.LD;
// m lateral diffusion (default 0)
            intern.m_jctSatCur := ex.IS;
// A bulk junction saturation current (default 1e-14)
            intern.m_drainResistanceIsGiven := if ex.RD > (-1e40) then 1 else 0;
            intern.m_drainResistance := if ex.RD > (-1e40) then ex.RD else 0;
            intern.m_sourceResistanceIsGiven := if ex.RS > (-1e40) then 1 else 0;
            intern.m_sourceResistance := if ex.RS > (-1e40) then ex.RS else 0;
            intern.m_transconductanceIsGiven := if ex.KP > (-1e40) then 1 else 0;
            intern.m_transconductance := if ex.KP > (-1e40) then ex.KP else 2e-5;
            intern.m_tnom := if ex.TNOM > (-1e40) then ex.TNOM + Spice3.Internal.SpiceConstants.CONSTCtoK else 300.15;
            intern.m_jctSatCurDensity := ex.JS;
// A/(m*m) bulk junction saturation current per sq-meter of junction area (default 0)
            intern.m_sheetResistance := ex.RSH;
// Ohm drain and source diffusion sheet resistance (default 0)
            intern.m_bulkJctPotential := ex.PB;
// V bulk junction potential (default 0.8)
            intern.m_bulkJctBotGradingCoeff := ex.MJ;
// bulk junction bottom grading coeff. (default 0.5)
            intern.m_bulkJctSideGradingCoeff := ex.MJSW;
// bulk junction sidewall grading coeff. (default 0.5)
            intern.m_oxideThickness := ex.TOX;
// m oxide thickness (default 1e-7)
            intern.m_oxideThicknessIsGiven := if ex.TOX > (-1e40) then 1 else 0;
            intern.m_oxideThickness := if ex.TOX > (-1e40) then ex.TOX else 1e-7;
            intern.m_gateSourceOverlapCapFactor := ex.CGSO;
// F/m gate-source overlap capacitance per meter channel width (default 0)
            intern.m_gateDrainOverlapCapFactor := ex.CGDO;
// F/m gate-drain overlap capacitance per meter channel width (default 0)
            intern.m_gateBulkOverlapCapFactor := ex.CGBO;
// F/m gate-bulk overlap capacitance per meter channel width (default 0)
            intern.m_fNcoef := ex.KF;
// flicker-noise coefficient (default 0)
            intern.m_fNexp := ex.AF;
// flicker-noise exponent (default 1)
            annotation(
              Documentation(info = "<html>
<p>This function mos2RenameParameters assigns the external (given by the user, e.g., RD) technology parameters
to the internal parameters (e.g., m_drainResistance). It also does the analysis of the IsGiven values (level 2).</p>
</html>"));
          end mos2RenameParameters;

          function mos2RenameParametersDev "Device parameter renaming to internal names"
            extends Modelica.Icons.Function;
            input Spice3.Internal.ModelcardMOS2 ex;
            input Integer mtype;
            input Modelica.SIunits.Length W "Width of channel region";
            input Modelica.SIunits.Length L "Length of channel region";
            input Modelica.SIunits.Area AD "Area of drain diffusion";
            input Modelica.SIunits.Area AS "Area of source diffusion";
            input Modelica.SIunits.Length PD "Drain perimeter";
            input Modelica.SIunits.Length PS "Source perimeter";
            input Real NRD "Length of drain squares";
            input Real NRS "Length of Source squares";
            input Integer OFF "Optional initial condition: 0 - IC not used, 1 - IC used, not implemented yet";
            input Real IC "Initial condition values, not implemented yet";
            input Modelica.SIunits.Temp_C TEMP "Temperature";
            output Spice3.Internal.Mosfet.Mosfet dev "Output record Mosfet";
          algorithm
/*device parameters*/
            dev.m_len := L;
// L, length of channel region
            dev.m_width := W;
// W, width of channel region
            dev.m_drainArea := AD;
// AD, area of drain diffusion
            dev.m_sourceArea := AS;
// AS, area of source diffusion
            dev.m_drainSquares := NRD;
// NRD, length of drain in squares
            dev.m_sourceSquares := NRS;
// NRS, length of source in squares
            dev.m_drainPerimiter := PD;
// PD, Drain perimeter;
            dev.m_sourcePerimiter := PS;
// PS, Source perimeter
            dev.m_dICVDSIsGiven := if IC > (-1e40) then 1 else 0;
            dev.m_dICVDS := if IC > (-1e40) then IC else 0;
            dev.m_dICVGSIsGiven := if IC > (-1e40) then 1 else 0;
            dev.m_dICVGS := if IC > (-1e40) then IC else 0;
            dev.m_dICVBSIsGiven := if IC > (-1e40) then 1 else 0;
            dev.m_dICVBS := if IC > (-1e40) then IC else 0;
            dev.m_off := OFF;
// non-zero to indicate device is off for dc analysis
            dev.m_bPMOS := mtype;
// P type MOSFET model
            dev.m_nLevel := ex.LEVEL;
            assert(ex.LEVEL == 1, "only MOS Level1 implemented");
            dev.m_dTemp := TEMP + Spice3.Internal.SpiceConstants.CONSTCtoK;
            annotation(
              Documentation(info = "<html>
<p>This function mos2RenameParameters assigns the external (given by the user) device parameters to the internal parameters. It also does the analysis of the IsGiven values (level 2).</p>
</html>"));
          end mos2RenameParametersDev;
          annotation(
            Documentation(info = "<html>
<p>This package Mos2 contains functions and records with data of the MOSFET model level 2.</p>
</html>"));
        end Mos2;

        package Diode "Records and functions for diode model"
          extends Modelica.Icons.InternalPackage;

          record DiodeModelLineParams "Record for Diode model line parameters"
            extends Modelica.Icons.Record;
            Modelica.SIunits.Current m_satCur(start = 1.0e-14) "IS, Saturation current";
            Modelica.SIunits.Resistance m_resist(start = 0.0) "RS, Ohmic resistance";
            Real m_emissionCoeff(start = 1.0) "N, Emission Coefficient";
            Modelica.SIunits.Time m_transitTime(start = 0.0) "TT, Transit Time";
            Modelica.SIunits.Capacitance m_junctionCap(start = 0.0) "CJO, Junction capacitance";
            Modelica.SIunits.Voltage m_junctionPot(start = 1.0) "VJ, Junction potential";
            Modelica.SIunits.LinearTemperatureCoefficient m_gradingCoeff(start = 0.5) "M, Grading coefficient";
            //unit check by maj
            Modelica.SIunits.ActivationEnergy m_activationEnergy(start = 1.11) "EG, Activation energy";
            Real m_saturationCurrentExp(start = 3.0) "XTI, Saturation current temperature exp.";
            Real m_depletionCapCoeff(start = 0.5) "FC, Forward bias junction fit parameter";
            Modelica.SIunits.Voltage m_breakdownVoltage "BV, Reverse breakdown voltage";
            Real m_pBvIsGiven "BV is given value";
            Modelica.SIunits.Current m_breakdownCurrent(start = 1.0e-3) "IBV, Current at reverse breakdown voltage";
            Modelica.SIunits.Temp_K m_nomTemp(start = SpiceConstants.CKTnomTemp) "TNOM, Parameter measurement temperature";
            Real m_fNcoef(start = 0.0) "KF, flicker noise coefficient";
            Real m_fNexp(start = 1.0) "AF, flicker noise exponent";
            Modelica.SIunits.Conductance m_conductance "G, Ohmic conductance";
            annotation(
              Documentation(info = "<html>
<p>This record contains the model line (also called model card) parameters that are used for the diode model  in SPICE3.</p>
</html>"));
          end DiodeModelLineParams;

          record DiodeModelLineVariables "Record for Diode model line variables"
            extends Modelica.Icons.Record;
            Real m_gradingCoeff;
            Real m_activationEnergy;
            Real m_depletionCapCoeff;
            Modelica.SIunits.Conductance m_conductance;
            annotation(
              Documentation(info = "<html>
<p>This record contains the model line (also called model card) variables  that are used for the diode model in SPICE3.</p>
</html>"));
          end DiodeModelLineVariables;

          record DiodeParams "Record for Diode device parameters"
            extends Modelica.Icons.Record;
            Real m_area(start = 1.0) "AREA, Area factor";
            Boolean m_bOff(start = false) "OFF, Initially off";
            Modelica.SIunits.Voltage m_dIC(start = 0.0) "IC, Initial device voltage";
            Real m_pIcIsGiven "IC is given value";
            Boolean m_bSensArea(start = false) "SENS_AREA, flag to request sensitivity WRT area";
            annotation(
              Documentation(info = "<html>
<p>This record contains the device parameters that are used for the diode model in SPICE3.</p>
</html>"));
          end DiodeParams;

          record DiodeVariables "Variables for the diode model"
            extends Modelica.Icons.Record;
            Real m_pBvIsGiven "Additional BV is given variable";
            annotation(
              Documentation(info = "<html>
<p>This record contains the model  variables that are used for the diode model in SPICE3.</p>
</html>"));
          end DiodeVariables;

          record DiodeCalc "Diode variables"
            extends Modelica.Icons.Record;
            Modelica.SIunits.Voltage m_tJctPot;
            Modelica.SIunits.Capacitance m_tJctCap;
            Real m_tF1;
            Real m_f2;
            Real m_f3;
            Modelica.SIunits.Current m_tSatCur;
            Modelica.SIunits.Voltage m_tVcrit;
            Real m_dVte;
            Modelica.SIunits.Voltage m_tBrkdwnV;
            annotation(
              Documentation(info = "<html>
<p>This record contains the model variables that are used for the diode model in SPICE3.</p>
</html>"));
          end DiodeCalc;

          record CurrentsCapacitances "Diode variables"
            extends Modelica.Icons.Record;
            Modelica.SIunits.Current m_dCurrent;
            annotation(
              Documentation(info = "<html>
<p>This record contains the model variables that are used for the diode model in SPICE3.</p>
</html>"));
          end CurrentsCapacitances;

          function diodeModelLineInitEquations "Initial precalculation of model line parameters"
            extends Modelica.Icons.Function;
            input DiodeModelLineParams in_p "Input record with diode model line parameters";
            output DiodeModelLineVariables out_v "Output record with diode model line variables";
          algorithm
// limit grading coeff to max of 0.9
            if in_p.m_gradingCoeff > 0.9 then
              out_v.m_gradingCoeff := 0.9;
            end if;
// limit activation energy to min of 0.1
            if in_p.m_activationEnergy < 0.1 then
              out_v.m_activationEnergy := 0.1;
            end if;
// limit depletion cap coeff to max of 0.95
            if in_p.m_depletionCapCoeff > 0.95 then
              out_v.m_depletionCapCoeff := 0.95;
            end if;
            out_v.m_conductance := if in_p.m_resist == 0.0 then 0.0 else 1.0 / in_p.m_resist;
            annotation(
              Documentation(info = "<html>
<p>Within this function some parameters are initially precalculated from model line parameters.</p>
</html>"));
          end diodeModelLineInitEquations;

          function diodeInitEquations "Initial calculation"
            extends Modelica.Icons.Function;
            input DiodeModelLineParams in_p "Input record with diode model line parameters";
            output DiodeVariables out_v "Output record with diode variables";
          algorithm
            out_v.m_pBvIsGiven := in_p.m_pBvIsGiven;
            if out_v.m_pBvIsGiven > 0.5 then
              if in_p.m_breakdownVoltage > 1.0e+100 then
                out_v.m_pBvIsGiven := 1.0e-41;
              end if;
            end if;
// set to false
            annotation(
              Documentation(info = "<html>
<p>In this function some initial calculations for the diode model are done, especially concerning the handling of the breakthrough voltage.</p>
</html>"));
          end diodeInitEquations;

          function diodeCalcTempDependencies "Temperature dependency calculation"
            extends Modelica.Icons.Function;
            input DiodeModelLineParams in_p "Input record diode model line parameters";
            input DiodeParams in_dp "Input record diode parameters";
            input Model.Model in_m "Input record Model";
            input DiodeVariables in_v "Input record diode variables";
            output DiodeCalc out_c "Output record with calculated values";
          algorithm
            (out_c.m_tJctPot, out_c.m_tJctCap) := Modelica.Electrical.Spice3.Internal.Functions.junctionParamDepTempSPICE3(in_p.m_junctionPot, in_p.m_junctionCap, in_p.m_gradingCoeff, in_m.m_dTemp, in_p.m_nomTemp);
            out_c.m_tJctCap := in_dp.m_area * out_c.m_tJctCap;
            (out_c.m_tF1, out_c.m_f2, out_c.m_f3) := Modelica.Electrical.Spice3.Internal.Functions.junctionCapCoeffs(in_p.m_gradingCoeff, in_p.m_depletionCapCoeff, out_c.m_tJctPot);
            out_c.m_tSatCur := Modelica.Electrical.Spice3.Internal.Functions.saturationCurDepTempSPICE3(in_p.m_satCur, in_m.m_dTemp, in_p.m_nomTemp, in_p.m_emissionCoeff, in_p.m_activationEnergy, in_p.m_saturationCurrentExp);
            out_c.m_tVcrit := Modelica.Electrical.Spice3.Internal.Functions.junctionVCrit(in_m.m_dTemp, in_p.m_emissionCoeff, out_c.m_tSatCur);
            out_c.m_dVte := in_m.m_dTemp * SpiceConstants.CONSTKoverQ * in_p.m_emissionCoeff;
            if in_v.m_pBvIsGiven > 0.5 then
              out_c.m_tBrkdwnV := Modelica.Electrical.Spice3.Internal.Functions.junctionVoltage23SPICE3(in_p.m_breakdownVoltage, in_p.m_breakdownCurrent, out_c.m_tSatCur, in_m.m_dTemp, in_p.m_emissionCoeff);
            end if;
            out_c.m_tSatCur := in_dp.m_area * out_c.m_tSatCur;
            annotation(
              Documentation(info = "<html>
<p>In this function for the diode model temperature dependencies are calculated using temperature treating functions from the equation package.</p>
</html>"));
          end diodeCalcTempDependencies;

          function diodeNoBypassCode "Calculation of currents"
            extends Modelica.Icons.Function;
            input DiodeModelLineParams in_p "Input record model line parameters for diode";
            input DiodeParams in_dp "Input record with parameters for diode";
            input DiodeCalc in_c "Input record DiodeCalc";
            input Model.Model in_m "Input record model";
            input Boolean in_m_mbInit;
            input Modelica.SIunits.Voltage[2] in_m_pVoltageValues;
            /* DPP, DN */
            output CurrentsCapacitances out_cc "Output record with calculated currents and capacitances";
            output Modelica.SIunits.Capacitance m_dCap "Output capacitance";
          protected
            Modelica.SIunits.Voltage m_dPNVoltage "Voltage";
            Modelica.SIunits.Current m_dCurrent "Current";
            Modelica.SIunits.Conductance m_dCond "Conductance";
            Modelica.SIunits.Charge m_dCharge "Charge";
            Modelica.SIunits.Current m_dCapCurrent "Current";
          algorithm
            m_dPNVoltage := in_m_pVoltageValues[1] - in_m_pVoltageValues[2];
            if SpiceRoot.useInitialConditions() and in_dp.m_pIcIsGiven > 0.5 then
              m_dPNVoltage := in_dp.m_dIC;
            elseif SpiceRoot.initJunctionVoltages() then
              if in_dp.m_bOff then
                m_dPNVoltage := 0.0;
              else
                m_dPNVoltage := in_c.m_tVcrit;
              end if;
            end if;
            if in_p.m_pBvIsGiven > 0.5 then
              (out_cc.m_dCurrent, m_dCond) := Modelica.Electrical.Spice3.Internal.Functions.junction3(m_dPNVoltage, in_m.m_dTemp, in_p.m_emissionCoeff, in_c.m_tSatCur, in_c.m_tBrkdwnV);
            else
              (out_cc.m_dCurrent, m_dCond) := Modelica.Electrical.Spice3.Internal.Functions.junction2(m_dPNVoltage, in_m.m_dTemp, in_p.m_emissionCoeff, in_c.m_tSatCur);
            end if;
            m_dCurrent := out_cc.m_dCurrent;
            (m_dCap, m_dCharge) := Modelica.Electrical.Spice3.Internal.Functions.junctionCapTransTime(in_c.m_tJctCap, m_dPNVoltage, in_c.m_tJctPot * in_p.m_depletionCapCoeff, in_p.m_gradingCoeff, in_p.m_junctionPot, in_c.m_tF1, in_c.m_f2, in_c.m_f3, in_p.m_transitTime, m_dCond, m_dCurrent);
            annotation(
              Documentation(info = "<html>
<p>This function NoBypassCode calculates the currents (and the capacitances) that are necessary for the currents to be used in the top-level model.</p>
</html>"));
          end diodeNoBypassCode;

          function diodeCalcAdditionalValues "Calculation of additional values"
            extends Modelica.Icons.Function;
            input DiodeVariables in_v "Input record with diode variables";
            input DiodeModelLineParams in_p "Input record with diode model line parameters";
            input DiodeParams in_dp "Input record with diode parameters";
            input Modelica.SIunits.Voltage[2] in_m_pVoltageValues;
            /* DP, DPP */
            output DiodeVariables out_v "Output record with diode variables";
          algorithm
            out_v := in_v;
            annotation(
              Documentation(info = "<html>
<p>This function is prepared for additional calculations but it is plain (inputs are written to the outputs) in the actual version of this library.</p>
</html>"));
          end diodeCalcAdditionalValues;

          function diodeRenameParameters "Technology parameter renaming"
            extends Modelica.Icons.Function;
            input ModelcardDIODE ex "Modelcard with technology parameters";
            input SpiceConstants con "Spice constants";
            output DiodeModelLineParams intern "Output record diode model line parameters";
          algorithm
            intern.m_satCur := ex.IS;
            intern.m_resist := ex.RS;
            intern.m_emissionCoeff := ex.N;
            intern.m_transitTime := ex.TT;
            intern.m_junctionCap := ex.CJO;
            intern.m_junctionPot := ex.VJ;
            intern.m_gradingCoeff := if ex.M > 0.9 then 0.9 else ex.M;
            intern.m_activationEnergy := if ex.EG < 0.1 then 0.1 else ex.EG;
            intern.m_saturationCurrentExp := ex.XTI;
            intern.m_depletionCapCoeff := if ex.FC > 0.95 then 0.95 else ex.FC;
            intern.m_pBvIsGiven := if ex.BV > (-1e40) then 1 else 0;
            intern.m_breakdownVoltage := if ex.BV > (-1e40) then ex.BV else 0;
            intern.m_breakdownCurrent := ex.IBV;
            intern.m_nomTemp := ex.TNOM + SpiceConstants.CONSTCtoK;
            intern.m_fNcoef := ex.KF;
            intern.m_fNexp := ex.AF;
            intern.m_conductance := if ex.RS == 0 then 0 else 1 / ex.RS;
            annotation(
              Documentation(info = "<html>
<p>This function assigns the external (given by the user, e.g., IS) technology parameters
to the internal parameters (e.g., m_satCur). It also does the analysis of the IsGiven values.</p>
</html>"));
          end diodeRenameParameters;

          function diodeRenameParametersDev "Temperature calculation"
            extends Modelica.Icons.Function;
            input Modelica.SIunits.Temp_K TEMP "Temperature";
            input Real AREA "Area factor";
            input Real IC "Initial condition value (VD, not implemented yet";
            input Boolean OFF "Optional initial condition: false - IC not used, true - IC used, not implemented yet";
            input Boolean SENS_AREA "Flag to request sensitivity WRT area, not implemented yet";
            output DiodeParams dev "Output record with calculated diode parameters";
          algorithm
/*device parameters*/
            dev.m_area := AREA;
            dev.m_pIcIsGiven := if IC > (-1e40) then 1 else 0;
            dev.m_dIC := if IC > (-1e40) then IC else 0;
            dev.m_bOff := OFF;
            dev.m_bSensArea := SENS_AREA;
            annotation(
              Documentation(info = "<html>
<p>This function assigns the external (given by the user, e.g., AREA) device parameters
to the internal parameters (e.g., m_area). It also does the analysis of the IsGiven values.</p>
</html>"));
          end diodeRenameParametersDev;

          function diodeRenameParametersDevTemp "Temperature calculation"
            extends Modelica.Icons.Function;
            input Modelica.SIunits.Temp_C TEMP "Temperature";
            output Model.Model dev_temp "Input record Model";
          algorithm
            dev_temp.m_dTemp := TEMP + SpiceConstants.CONSTCtoK;
            annotation(
              Documentation(info = "<html>
<p>This function calculates device parameters which are temperature dependent.</p>
</html>"));
          end diodeRenameParametersDevTemp;
          annotation(
            Documentation(info = "<html>
<p>This package Diode contains functions and record with data of the semiconductor diode model.</p>
</html>"));
        end Diode;

        package Rsemiconductor "Records and functions for semiconductor resistor model"
          extends Modelica.Icons.InternalPackage;

          record ResistorParams "Resistor device parameters"
            extends Modelica.Icons.Record;
            Modelica.SIunits.Resistance m_dResist(start = 1000) "Device is a resistor model";
            Real m_dResIsGiven "Resistance is given value";
            Modelica.SIunits.Length m_dWidth(start = 0) "Width";
            Real m_dWidthIsGiven "Width is given value";
            Modelica.SIunits.Length m_dLength "Length";
            Real m_dLengthIsGiven "Length is given value";
            Boolean m_bSensResist(start = false) "flag to request sensitivity WRT resistance";
            Modelica.SIunits.Temp_C m_dTemp(start = 27) "Resistor device temperature";
            annotation(
              Documentation(info = "<html>
<p>This record contains the device parameters that are used for the resistor model in SPICE3.</p>
</html>"));
          end ResistorParams;

          record ResistorModelLineParams "Record for Diode model line parameters"
            extends Modelica.Icons.Record;
            Modelica.SIunits.Conversions.NonSIunits.FirstOrderTemperaturCoefficient m_dTC1 "First order temp, coefficient";
            Modelica.SIunits.Conversions.NonSIunits.SecondOrderTemperaturCoefficient m_dTC2 "Second order temp, coefficient";
            Modelica.SIunits.Resistance m_dRsh "Sheet resistance";
            Real m_dRshIsGiven;
            Modelica.SIunits.Length m_dDefW "Default device width";
            Modelica.SIunits.Length m_dNarrow "Narrowing of resistor";
            Modelica.SIunits.Temp_C m_dTnom "Parameter measurement temperature";
            annotation(
              Documentation(info = "<html>
<p>This record contains the model line (also called model card) parameters that are used for the resistor model in SPICE3.</p>
</html>"));
          end ResistorModelLineParams;

          record ResistorVariables "Variables for the diode model"
            extends Modelica.Icons.Record;
            Modelica.SIunits.Length m_dWidth;
            Modelica.SIunits.Length m_dLength;
            Modelica.SIunits.Resistance m_dResist;
            Modelica.SIunits.Conductance m_dConduct;
            Modelica.SIunits.Temp_C m_dCond_dTemp;
            annotation(
              Documentation(info = "<html>
<p>This record contains the model variables that are used for the resistor model in SPICE3.</p>
</html>"));
          end ResistorVariables;

          function resistorRenameParameters "Technology parameter renaming"
            extends Modelica.Icons.Function;
            input ModelcardR ex "Modelcard with technology parameters";
            input SpiceConstants con "Spice constants";
            output ResistorModelLineParams intern "Output record with resistor model line parameters";
          algorithm
            intern.m_dTC1 := ex.TC1;
            intern.m_dTC2 := ex.TC2;
            intern.m_dRshIsGiven := if ex.RSH > (-1e40) then 1 else 0;
            intern.m_dRsh := if ex.RSH > (-1e40) then ex.RSH else 0;
            intern.m_dDefW := ex.DEFW;
            intern.m_dNarrow := ex.NARROW;
            intern.m_dTnom := if ex.TNOM > (-1e40) then ex.TNOM + SpiceConstants.CONSTCtoK else 300.15;
            annotation(
              Documentation(info = "<html>
<p>This function assigns the external (given by the user, e.g., N) technology parameters
to the internal parameters (e.g., m_emissionCoeff). It also does the analysis of the IsGiven values.</p>
</html>"));
          end resistorRenameParameters;

          function resistorRenameParametersDev "Device parameter renaming"
            extends Modelica.Icons.Function;
            input Modelica.SIunits.Resistance R "Resistance";
            input Modelica.SIunits.Length W "Width";
            input Modelica.SIunits.Length L "Length";
            input Modelica.SIunits.Temp_C TEMP "Temperature";
            input Boolean SENS_AREA "Parameter for sensitivity analyses, not implemented yet";
            input SpiceConstants con "Spice constants";
            output ResistorParams intern "Output record with resistor parameters";
          algorithm
            intern.m_dResIsGiven := if R > (-1e40) then 1 else 0;
            intern.m_dResist := if R > (-1e40) then R else 0;
            intern.m_dWidthIsGiven := if W > (-1e40) then 1 else 0;
            intern.m_dWidth := if W > (-1e40) then W else 0;
            intern.m_dLengthIsGiven := if L > (-1e40) then 1 else 0;
            intern.m_dLength := if L > (-1e40) then L else 0;
            intern.m_bSensResist := SENS_AREA;
            intern.m_dTemp := if TEMP > (-1e40) then TEMP + SpiceConstants.CONSTCtoK else 300.15;
            annotation(
              Documentation(info = "<html>
<p>This function assigns the external (given by the user, e.g., AREA) device parameters
to the internal parameters (e.g., m_area). It also does the analysis of the IsGiven values.</p>
</html>"));
          end resistorRenameParametersDev;

          function resistorInitEquations "Initial calculation"
            extends Modelica.Icons.Function;
            input ResistorParams in_p "Input record with resistor parameters";
            input ResistorModelLineParams in_p2 "Input record with resistor model line parameters";
            output ResistorVariables out "Output record with resistor variables";
          algorithm
            out.m_dLength := 0;
            out.m_dConduct := 0;
            out.m_dCond_dTemp := 0;
            out.m_dWidth := in_p.m_dWidth;
            if in_p.m_dResIsGiven < 0.5 then
              if abs(in_p.m_dLength) > 1e-8 and abs(in_p2.m_dRsh) > 1e-25 then
                if not in_p.m_dWidthIsGiven > 0.5 then
                  out.m_dWidth := in_p2.m_dDefW;
                end if;
                out.m_dResist := Modelica.Electrical.Spice3.Internal.Functions.resDepGeom(in_p2.m_dRsh, out.m_dWidth, in_p.m_dLength, in_p2.m_dNarrow);
              else
                out.m_dResist := 1000;
              end if;
            end if;
            if in_p.m_dResist < 1e-12 and in_p.m_dResIsGiven > 0.5 then
              out.m_dResist := 1e-12;
            end if;
            if in_p.m_dResist > 1e-12 and in_p.m_dResIsGiven > 0.5 then
              out.m_dResist := in_p.m_dResist;
            end if;
            annotation(
              Documentation(info = "<html>
<p>In this function some initial calculations for the resistor model are done, especially concerning the handling of the breakthrough voltage.</p>
</html>"));
          end resistorInitEquations;
          annotation(
            Documentation(info = "<html>
<p>This package Rsemiconductor contains functions and records with data of the semiconductor resistor model.</p>
</html>"));
        end Rsemiconductor;

        package Bjt "Records and functions for bjt model"
          extends Modelica.Icons.InternalPackage;

          record Bjt "Record for bjt device parameters"
            extends Spice3.Internal.Model.Model;
            Real m_area(start = 1.0) "AREA";
            Boolean m_bOff(start = false) "OFF";
            Modelica.SIunits.Voltage m_dICvbe(start = 0.0) "IC_VBE";
            Real m_bICvbeIsGiven(start = 0.0);
            Modelica.SIunits.Voltage m_dICvce(start = 0.0) "IC_VCE";
            Real m_bICvceIsGiven(start = 0.0);
            Boolean m_uic "Use initial conditions, UIC";
            Boolean m_bSensArea(start = false) "SENS_AREA";
            Modelica.SIunits.Current m_transitTimeHighCurrentF(start = 0.0);
            Types.InverseElectricCurrent m_invRollOffF(start = 0);
            Types.InverseElectricCurrent m_invRollOffR(start = 0);
            Modelica.SIunits.Capacitance m_CScap(start = 0);
            annotation(
              Documentation(info = "<html>
<p>This record contains the device parameters that are used for the bipolar transistor bjt model in SPICE3.</p>
</html>"));
          end Bjt;

          record BjtModelLineParams "Record for bjt model line parameters"
            extends Modelica.Icons.Record;
            Real m_type(start = 1) "device type : 1 = n,  -1 = p";
            Modelica.SIunits.Temp_K m_tnom(start = Spice3.Internal.SpiceConstants.CKTnomTemp) "TNOM, Parameter measurement temperature";
            Modelica.SIunits.Current m_satCur(start = 1.0e-16) "IS, Saturation Current";
            Real m_betaF(start = 100.0) "BF, Ideal forward beta";
            Real m_emissionCoeffF(start = 1.0) "NF, Forward emission coefficient";
            Real m_leakBEemissionCoeff(start = 1.5) "NE, B-E leakage emission coefficient";
            Modelica.SIunits.Current m_leakBEcurrent(start = 0.) "ISE, B-E leakage saturation current";
            Real m_c2(start = 0.) "C2, Obsolete parameter name";
            Modelica.SIunits.Current m_leakBCcurrent(start = 0.) "ISC, B-C leakage saturation current";
            Real m_c4(start = 0.) "C4, Obsolete parameter name";
            Real m_leakBEcurrentIsGiven;
            Real m_c2IsGiven;
            Real m_leakBCcurrentIsGiven;
            Real m_c4IsGiven;
            Real m_betaR(start = 1.0) "BR, Ideal reverse beta";
            Real m_emissionCoeffR(start = 1.0) "NR, Reverse emission coefficient";
            Real m_leakBCemissionCoeff(start = 2.0) "NC, B-C leakage emission coefficient";
            Modelica.SIunits.Voltage m_earlyVoltF(start = 0.0) "VAF, Forward Early voltage";
            Modelica.SIunits.Current m_rollOffF(start = 0.0) "IKF, Forward beta roll-off corner current";
            Modelica.SIunits.Voltage m_earlyVoltR(start = 0.0) "VAR, Reverse Early voltage";
            Modelica.SIunits.Current m_rollOffR(start = 0.0) "IKR, reverse beta roll-off corner current";
            Modelica.SIunits.Resistance m_emitterResist(start = 0.0) "RE, Emitter resistance";
            Modelica.SIunits.Resistance m_collectorResist(start = 0.0) "RC, Collector resistance";
            Modelica.SIunits.Current m_baseCurrentHalfResist(start = 0.0) "IRB, Current for base resistance=(rb+rbm)/2";
            Modelica.SIunits.Resistance m_baseResist(start = 0.0) "RB, Zero bias base resistance";
            Modelica.SIunits.Resistance m_minBaseResist(start = 0.0) "RBM, Minimum base resistance";
            Real m_minBaseResistIsGiven;
            Modelica.SIunits.Capacitance m_depletionCapBE(start = 0.0) "CJE, Zero bias B-E depletion capacitance";
            Modelica.SIunits.Voltage m_potentialBE(start = 0.75) "VJE, B-E built in potential";
            Real m_junctionExpBE(start = 0.33) "MJE, B-E built in potential";
            Modelica.SIunits.Time m_transitTimeF(start = 0.0) "TF, Ideal forward transit time";
            Real m_transitTimeBiasCoeffF(start = 0.0) "XTF, Coefficient for bias dependence of TF";
            Modelica.SIunits.Current m_transitTimeHighCurrentF(start = 0.0) "ITF, High current dependence of TF";
            Modelica.SIunits.Voltage m_transitTimeFVBC(start = 0.0) "VTF, Voltage giving VBC dependence of TF";
            Modelica.SIunits.Frequency m_excessPhase(start = 0.0) "PTF, Excess phase";
            Modelica.SIunits.Capacitance m_depletionCapBC(start = 0.0) "CJC, Zero bias B-C depletion capacitance";
            Modelica.SIunits.Voltage m_potentialBC(start = 0.75) "VJC, B-C built in potential";
            Real m_junctionExpBC(start = 0.33) "MJC, B-C junction grading coefficient";
            Real m_baseFractionBCcap(start = 1.0) "XCJC, Fraction of B-C cap to internal base";
            Modelica.SIunits.Time m_transitTimeR(start = 0.0) "TR, Ideal reverse transit time";
            Modelica.SIunits.Capacitance m_capCS(start = 0.0) "CJS, Zero bias C-S capacitance";
            Modelica.SIunits.Voltage m_potentialSubstrate(start = 0.75) "VJS, Zero bias C-S capacitance";
            Real m_exponentialSubstrate(start = 0.0) "MJS, Substrate junction grading coefficient";
            Real m_betaExp(start = 0.0) "XTB, Forward and reverse beta temp. exp.";
            Modelica.SIunits.GapEnergy m_energyGap(start = 1.11) "EG, Energy gap for IS temp. dependency";
            Real m_tempExpIS(start = 3.0) "XTI,Temp. exponent for IS";
            Real m_fNcoef(start = 0.0) "KF, Flicker Noise Coefficient";
            Real m_fNexp(start = 1.0) "AF, Flicker Noise Exponent";
            Real m_depletionCapCoeff(start = 0.5) "FC, Forward bias junction fit parameter";
            Modelica.SIunits.Conductance m_collectorConduct(start = 0.0);
            Modelica.SIunits.Conductance m_emitterConduct(start = 0.0);
            Modelica.SIunits.InversePotential m_transitTimeVBCFactor(start = 0.0);
            Real m_excessPhaseFactor(start = 0.0);
            Modelica.SIunits.InversePotential m_invEarlyVoltF(start = 0.0);
            Types.InverseElectricCurrent m_invRollOffF(start = 0.0);
            Modelica.SIunits.InversePotential m_invEarlyVoltR(start = 0.0);
            Types.InverseElectricCurrent m_invRollOffR(start = 0.0);
            //obsolete
            Real m_bNPN = 1;
            Real m_bPNP = 1;
            Real m_area = 1.0;
            Boolean m_bOff = false;
            Modelica.SIunits.Voltage m_dICvbe = 0;
            Real m_bICvbeIsGiven = 0;
            Modelica.SIunits.Voltage m_dICvce = 0;
            Real m_bICvceIsGiven = 0;
            Boolean m_bSensArea = false;
            Real m_dTemp = 1;
            annotation(
              Documentation(info = "<html>
<p>This record contains the model line (also called model card) parameters that are used for the bipolar transistor model in SPICE3.</p>
</html>"));
          end BjtModelLineParams;

          record BjtCalc "Bjt variables"
            extends Modelica.Icons.Record;
            Modelica.SIunits.Current m_tSatCur(start = 0);
            Real m_tBetaF(start = 1);
            Real m_tBetaR(start = 1);
            Modelica.SIunits.Current m_tBEleakCur(start = 1e-14);
            Modelica.SIunits.Current m_tBCleakCur(start = 1e-14);
            Modelica.SIunits.Capacitance m_tBEcap(start = 0);
            Modelica.SIunits.Voltage m_tBEpot(start = 0.7);
            Modelica.SIunits.Capacitance m_tBCcap(start = 0);
            Modelica.SIunits.Voltage m_tBCpot(start = 0.7);
            Modelica.SIunits.Voltage m_tDepCapBE(start = 0.7);
            Modelica.SIunits.Voltage m_tDepCapBC(start = 0.7);
            Modelica.SIunits.Voltage m_tVcrit(start = 0.7);
            Modelica.SIunits.Voltage m_dVt(start = 0.025);
            Modelica.SIunits.Voltage m_tF1c(start = 0);
            Real m_f2c(start = 0);
            Real m_f3c(start = 0);
            Modelica.SIunits.Voltage m_tF1e(start = 0);
            Real m_f2e(start = 0);
            Real m_f3e(start = 0);
            annotation(
              Documentation(info = "<html>
<p>This record contains the model variables that are used for the bipolar transistor model in SPICE3.</p>
</html>"));
          end BjtCalc;

          record CurrentsCapacitances "Bjt variables"
            extends Modelica.Icons.Record;
            Modelica.SIunits.Current iBE(start = 0.0);
            //current through diode dE1 (ideal part)
            Modelica.SIunits.Current iBEN(start = 0.0);
            //current through diode dE2 (non ideal part)
            Modelica.SIunits.Current iBC(start = 0.0);
            //current through diode dC1 (ideal part)
            Modelica.SIunits.Current iBCN(start = 0.0);
            //current through diode dC2 (non ideal part)
            Modelica.SIunits.Current iCC(start = 0.0);
            //channel current
            Modelica.SIunits.Capacitance capbc(start = 0.0);
            Modelica.SIunits.Capacitance capbe(start = 0.0);
            Modelica.SIunits.Capacitance capbx(start = 0.0);
            Modelica.SIunits.Capacitance captt(start = 0.0);
            Modelica.SIunits.Capacitance capcs(start = 0.0);
            Modelica.SIunits.Resistance rx(start = 0.0);
            //obsolete
            Real iXX;
            annotation(
              Documentation(info = "<html>
<p>This record contains the model variables that are used for the bipoar transistor model in SPICE3.</p>
</html>"));
          end CurrentsCapacitances;

          function bjtModelLineInitEquations "Initial calculation"
            extends Modelica.Icons.Function;
            input BjtModelLineParams in_p "Input record with Bjt model line parameters";
            output BjtModelLineParams out_p "Output record with Bjt model line variables";
          algorithm
            out_p := in_p;
            if not out_p.m_leakBEcurrentIsGiven > 0.5 and out_p.m_c2IsGiven > 0.5 then
              out_p.m_leakBEcurrent := out_p.m_c2 * out_p.m_satCur;
            end if;
            if not out_p.m_leakBCcurrentIsGiven > 0.5 and out_p.m_c4IsGiven > 0.5 then
              out_p.m_leakBCcurrent := out_p.m_c4 * out_p.m_satCur;
            end if;
            if out_p.m_earlyVoltF <> 0 then
              out_p.m_invEarlyVoltF := 1 / out_p.m_earlyVoltF;
            end if;
            if out_p.m_rollOffF <> 0 then
              out_p.m_invRollOffF := 1 / out_p.m_rollOffF;
            end if;
            if out_p.m_earlyVoltR <> 0 then
              out_p.m_invEarlyVoltR := 1 / out_p.m_earlyVoltR;
            end if;
            if out_p.m_rollOffR <> 0 then
              out_p.m_invRollOffR := 1 / out_p.m_rollOffR;
            end if;
            if out_p.m_collectorResist <> 0 then
              out_p.m_collectorConduct := 1 / out_p.m_collectorResist;
            end if;
            if out_p.m_emitterResist <> 0 then
              out_p.m_emitterConduct := 1 / out_p.m_emitterResist;
            end if;
            if out_p.m_transitTimeFVBC <> 0 then
              out_p.m_transitTimeVBCFactor := 1 / (out_p.m_transitTimeFVBC * 1.44);
            end if;
            out_p.m_excessPhaseFactor := out_p.m_excessPhase / (180.0 / Modelica.Constants.pi) * out_p.m_transitTimeF;
            if out_p.m_depletionCapCoeff > 0.9999 then
              out_p.m_depletionCapCoeff := 0.9999;
            end if;
            annotation(
              Documentation(info = "<html>
<p>Within this function some parameters are initially precalculated from model line parameters.</p>
</html>"));
          end bjtModelLineInitEquations;

          function bjtInitEquations "Initial calculation"
            extends Modelica.Icons.Function;
            input Bjt in_p "Input record Bjt";
            input BjtModelLineParams in_pml "Input record with Bjt model line parameters";
            output Bjt out_v "Output record with Bjt";
          algorithm
            out_v := in_p;
// calculate the parameters that depend on the area factor
            out_v.m_transitTimeHighCurrentF := in_pml.m_transitTimeHighCurrentF * out_v.m_area;
            out_v.m_invRollOffF := in_pml.m_invRollOffF / out_v.m_area;
            out_v.m_invRollOffR := in_pml.m_invRollOffR / out_v.m_area;
            out_v.m_CScap := in_pml.m_capCS * out_v.m_area;
            annotation(
              Documentation(info = "<html>
<p>Within this function some parameters are initially precalculated from model line parameters.</p>
</html>"));
          end bjtInitEquations;

          function bjtCalcTempDependencies "Temperature dependency calculation"
            extends Modelica.Icons.Function;
            input Bjt in_p3 "Input record Bjt";
            input BjtModelLineParams in_p "Input record with Bjt model line parameters";
            output BjtCalc out_c "Output record BjtCalc";
          protected
            Real fact1;
            Real fact2;
            Modelica.SIunits.Voltage vt;
            Modelica.SIunits.GapEnergy egfet;
            Types.GapEnergyPerEnergy arg;
            Modelica.SIunits.Voltage pbfact;
            Real ratlog;
            Real ratio1;
            Real factlog;
            Real factor;
            Real bfactor;
            Modelica.SIunits.Voltage pbo;
            Real gmanew;
            Real gmaold;
          algorithm
            fact1 := in_p.m_tnom / Spice3.Internal.SpiceConstants.REFTEMP;
            vt := in_p3.m_dTemp * Spice3.Internal.SpiceConstants.CONSTKoverQ;
            fact2 := in_p3.m_dTemp / Spice3.Internal.SpiceConstants.REFTEMP;
            egfet := Spice3.Internal.MaterialParameters.EnergyGapSi - Spice3.Internal.MaterialParameters.FirstBandCorrFactorSi * in_p3.m_dTemp * in_p3.m_dTemp / (in_p3.m_dTemp + Spice3.Internal.MaterialParameters.SecondBandCorrFactorSi);
            arg := (-egfet / (2 * Spice3.Internal.SpiceConstants.CONSTboltz * in_p3.m_dTemp)) + Spice3.Internal.MaterialParameters.BandCorrFactorT300 / (Spice3.Internal.SpiceConstants.CONSTboltz * (Spice3.Internal.SpiceConstants.REFTEMP + Spice3.Internal.SpiceConstants.REFTEMP));
            pbfact := -2 * vt * (1.5 * Modelica.Math.log(fact2) + Spice3.Internal.SpiceConstants.CHARGE * arg);
            ratlog := Modelica.Math.log(in_p3.m_dTemp / in_p.m_tnom);
            ratio1 := in_p3.m_dTemp / in_p.m_tnom - 1;
            factlog := ratio1 * in_p.m_energyGap / vt + in_p.m_tempExpIS * ratlog;
            factor := exp(factlog);
            bfactor := exp(ratlog * in_p.m_betaExp);
            pbo := (in_p.m_potentialBE - pbfact) / fact1;
            gmaold := (in_p.m_potentialBE - pbo) / pbo;
            out_c.m_tSatCur := in_p.m_satCur * factor * in_p3.m_area;
            out_c.m_tBetaF := in_p.m_betaF * bfactor;
            out_c.m_tBetaR := in_p.m_betaR * bfactor;
            out_c.m_tBEleakCur := in_p.m_leakBEcurrent * exp(factlog / in_p.m_leakBEemissionCoeff) / bfactor * in_p3.m_area;
            out_c.m_tBCleakCur := in_p.m_leakBCcurrent * exp(factlog / in_p.m_leakBCemissionCoeff) / bfactor * in_p3.m_area;
            out_c.m_tBEcap := in_p.m_depletionCapBE / (1 + in_p.m_junctionExpBE * (4e-4 * (in_p.m_tnom - Spice3.Internal.SpiceConstants.REFTEMP) - gmaold));
            out_c.m_tBEpot := fact2 * pbo + pbfact;
            gmanew := (out_c.m_tBEpot - pbo) / pbo;
            out_c.m_tBEcap := out_c.m_tBEcap * (1 + in_p.m_junctionExpBE * (4e-4 * (in_p3.m_dTemp - Spice3.Internal.SpiceConstants.REFTEMP) - gmanew));
            pbo := (in_p.m_potentialBC - pbfact) / fact1;
            gmaold := (in_p.m_potentialBC - pbo) / pbo;
            out_c.m_tBCcap := in_p.m_depletionCapBC / (1 + in_p.m_junctionExpBC * (4e-4 * (in_p.m_tnom - Spice3.Internal.SpiceConstants.REFTEMP) - gmaold));
            out_c.m_tBCpot := fact2 * pbo + pbfact;
            gmanew := (out_c.m_tBCpot - pbo) / pbo;
            out_c.m_tBCcap := out_c.m_tBCcap * (1 + in_p.m_junctionExpBC * (4e-4 * (in_p3.m_dTemp - Spice3.Internal.SpiceConstants.REFTEMP) - gmanew));
            out_c.m_tDepCapBE := in_p.m_depletionCapCoeff * out_c.m_tBEpot;
            out_c.m_tDepCapBC := in_p.m_depletionCapCoeff * out_c.m_tBCpot;
            out_c.m_tVcrit := vt * Modelica.Math.log(vt / (Spice3.Internal.SpiceConstants.CONSTroot2 * in_p.m_satCur));
            out_c.m_dVt := vt;
// calculate the parameters that depend on the area factor
            out_c.m_tBEcap := out_c.m_tBEcap * in_p3.m_area;
            out_c.m_tBCcap := out_c.m_tBCcap * in_p3.m_area;
            (out_c.m_tF1c, out_c.m_f2c, out_c.m_f3c) := Spice3.Internal.Functions.junctionCapCoeffs(in_p.m_junctionExpBC, in_p.m_depletionCapCoeff, out_c.m_tBCpot);
            (out_c.m_tF1e, out_c.m_f2e, out_c.m_f3e) := Spice3.Internal.Functions.junctionCapCoeffs(in_p.m_junctionExpBE, in_p.m_depletionCapCoeff, out_c.m_tBEpot);
            annotation(
              Documentation(info = "<html>
<p>In this function for the bipolar transistor model temperature dependencies are calculated using temperature treating functions from the equation package.</p>
</html>"));
          end bjtCalcTempDependencies;

          function bjtNoBypassCode "Calculation of currents"
            extends Modelica.Icons.Function;
            input Bjt in_p3 "Input record Bjt";
            input BjtModelLineParams in_p "Input record with Bjt model line parameters";
            input BjtCalc in_c "Input record BjtCalc";
            input Modelica.SIunits.Voltage[6] in_m_pVoltageValues;
            /* 1 Col; 2 Base; 3 Emit; 4 ColP; 5 BaseP; 6 EmitP */
            output CurrentsCapacitances out_cc "Output record with calculated currents and capacitances";
          protected
            Modelica.SIunits.Voltage vce;
            Modelica.SIunits.Voltage vbe;
            Modelica.SIunits.Voltage vbx;
            Modelica.SIunits.Voltage vbc;
            Modelica.SIunits.Conductance gbe;
            // Modelica.SIunits.Current cbe;
            Real cbe;
            Modelica.SIunits.Conductance gbc;
            Modelica.SIunits.Current cbc;
            Modelica.SIunits.Conductance gben;
            Modelica.SIunits.Current cben;
            Modelica.SIunits.Conductance gbcn;
            Modelica.SIunits.Current cbcn;
            Modelica.SIunits.Current cjbe;
            Modelica.SIunits.Current cjbc;
            Real dqbdve;
            Real dqbdvc;
            Real qb;
            Real q1;
            Real q2;
            Real arg;
            Real sqarg;
            Real cc;
            Modelica.SIunits.Current cex;
            Modelica.SIunits.Conductance gex;
            Modelica.SIunits.Time ttime;
            Real step;
            Real laststep;
            Modelica.SIunits.Current bcex0;
            Modelica.SIunits.Current bcex1;
            Real arg1;
            Real arg2;
            Real denom;
            Real arg3;
            Real rbpr;
            Real rbpi;
            Real gx;
            Real xjrb;
            Real go;
            Real gm;
            Modelica.SIunits.Capacitance captt;
            Modelica.SIunits.Charge chargebe;
            Modelica.SIunits.Charge chargebc;
            Modelica.SIunits.Charge chargebx;
            Real argtf;
            Real exponent;
            Real temp;
            Real aux1;
            Real aux2;
            Modelica.SIunits.Charge chargecs;
            Modelica.SIunits.Voltage vcs;
            Real sarg;
          algorithm
            temp := 0;
            vce := in_p.m_type * (in_m_pVoltageValues[4] - in_m_pVoltageValues[6]);
// ( ColP, EmitP);
            vbe := in_p.m_type * (in_m_pVoltageValues[5] - in_m_pVoltageValues[6]);
// ( BaseP, EmitP);
            vbx := in_p.m_type * (in_m_pVoltageValues[2] - in_m_pVoltageValues[4]);
// ( Base, ColP);
            if in_p3.m_uic then
              if in_p3.m_bICvbeIsGiven > 0.5 then
                vbe := in_p.m_type * in_p3.m_dICvbe;
              end if;
              if in_p3.m_bICvceIsGiven > 0.5 then
                vce := in_p.m_type * in_p3.m_dICvce;
              end if;
              vbx := vbe - vce;
            elseif Spice3.Internal.SpiceRoot.initJunctionVoltages() then
              if in_p3.m_bOff then
                vbe := 0.0;
                vce := 0.0;
                vbx := 0.0;
              else
                vbe := in_c.m_tVcrit;
                vce := vbe;
                vbx := 0.0;
              end if;
            end if;
            vbc := vbe - vce;
// junction current
            (cbe, gbe, cben, gben) := Spice3.Internal.Functions.junction2SPICE3BJT(vbe, in_p3.m_dTemp, in_p.m_emissionCoeffF, in_p.m_leakBEemissionCoeff, in_c.m_tSatCur, in_c.m_tBEleakCur);
            out_cc.iBE := in_p.m_type * cbe / in_c.m_tBetaF;
            out_cc.iBEN := in_p.m_type * cben;
            (cbc, gbc, cbcn, gbcn) := Spice3.Internal.Functions.junction2SPICE3BJT(vbc, in_p3.m_dTemp, in_p.m_emissionCoeffR, in_p.m_leakBCemissionCoeff, in_c.m_tSatCur, in_c.m_tBCleakCur);
            out_cc.iBC := in_p.m_type * cbc / in_c.m_tBetaR;
            out_cc.iBCN := in_p.m_type * cbcn;
            cjbe := cbe / in_c.m_tBetaF + cben;
            cjbc := cbc / in_c.m_tBetaR + cbcn;
// determine base charge terms
            q1 := 1.0 / (1.0 - in_p.m_invEarlyVoltF * vbc - in_p.m_invEarlyVoltR * vbe);
            if in_p.m_invRollOffF == 0 and in_p.m_invRollOffR == 0 then
              qb := q1;
              dqbdve := q1 * qb * in_p.m_invEarlyVoltR;
              dqbdvc := q1 * qb * in_p.m_invEarlyVoltF;
            else
              q2 := in_p.m_invRollOffF * cbe + in_p.m_invRollOffR * cbc;
              arg := max(0.0, 1 + 4 * q2);
              sqarg := 1;
              if arg <> 0 then
                sqarg := sqrt(arg);
              end if;
              qb := q1 * (1 + sqarg) / 2;
              dqbdve := q1 * (qb * in_p.m_invEarlyVoltR + in_p.m_invRollOffF * gbe / sqarg);
              dqbdvc := q1 * (qb * in_p.m_invEarlyVoltF + in_p.m_invRollOffR * gbc / sqarg);
            end if;
// determine dc incremental conductances - Weil's approximation
            cc := 0.0;
            cex := cbe;
            gex := gbe;
            ttime := 1;
            if in_p.m_excessPhaseFactor <> 0 and ttime > 0.0 then
              step := 0;
              laststep := 1;
              bcex0 := 0;
              bcex1 := 0;
              if bcex1 == 0.0 then
                bcex1 := cbe / qb;
                bcex0 := bcex1;
              end if;
              arg1 := step / in_p.m_excessPhaseFactor;
              arg2 := 3 * arg1;
              arg1 := arg2 * arg1;
              denom := 1 + arg1 + arg2;
              arg3 := arg1 / denom;
              cc := (bcex0 * (1 + step / laststep + arg2) - bcex1 * step / laststep) / denom;
              cex := cbe * arg3;
              gex := gbe * arg3;
            end if;
            cc := cc + (cex - cbc) / qb;
// resistances
            rbpr := in_p.m_minBaseResist / in_p3.m_area;
            rbpi := in_p.m_baseResist / in_p3.m_area - rbpr;
            gx := rbpr + rbpi / qb;
            xjrb := in_p.m_baseCurrentHalfResist * in_p3.m_area;
            if xjrb <> 0 then
              arg1 := max((cjbe + cjbc) / xjrb, 1e-9);
              arg2 := ((-1) + sqrt(1 + 14.59025 * arg1)) / 2.4317 / sqrt(arg1);
              arg1 := tan(arg2);
              gx := rbpr + 3 * rbpi * (arg1 - arg2) / arg2 / arg1 / arg1;
            end if;
// z s. Gl. (11-44a)
// keine if-Abfrage falls gx als Widerstand in Top_level genutzt wird !!!
// ----------------------------------------------------------------------
            if gx <> 0 then
              gx := 1 / gx;
            end if;
            out_cc.rx := gx;
// determine dc incremental conductances
            go := (gbc + (cex - cbc) * dqbdvc / qb) / qb;
            gm := (gex - (cex - cbc) * dqbdve / qb) / qb - go;
            out_cc.iCC := in_p.m_type * cc;
// charge storage elements and transit time calculation
            captt := 0.0;
            if in_p.m_transitTimeF <> 0.0 and vbe > 0.0 then
              argtf := 0.0;
              arg2 := 0.0;
              arg3 := 0.0;
              if in_p.m_transitTimeBiasCoeffF <> 0.0 then
                argtf := in_p.m_transitTimeBiasCoeffF;
                if in_p.m_transitTimeVBCFactor <> 0.0 then
                  exponent := min(50., vbc * in_p.m_transitTimeVBCFactor);
                  argtf := argtf * exp(exponent);
                end if;
                arg2 := argtf;
                if in_p.m_transitTimeHighCurrentF <> 0 then
                  temp := cbe / (cbe + in_p.m_transitTimeHighCurrentF);
                  argtf := argtf * temp * temp;
                  arg2 := argtf * (3 - temp - temp);
                end if;
                arg3 := cbe * argtf * in_p.m_transitTimeVBCFactor;
              end if;
              cbe := cbe * (1 + argtf) / qb;
              gbe := (gbe * (1 + arg2) - cbe * dqbdve) / qb;
              captt := in_p.m_transitTimeF * (arg3 - cbe * dqbdvc) / qb;
            end if;
            out_cc.captt := captt;
// (BaseP, ColP)
            (out_cc.capbe, chargebe) := Spice3.Internal.Functions.junctionCapTransTime(in_c.m_tBEcap, vbe, in_c.m_tDepCapBE, in_p.m_junctionExpBE, in_c.m_tBEpot, in_c.m_tF1e, in_c.m_f2e, in_c.m_f3e, in_p.m_transitTimeF, gbe, cbe);
            aux1 := in_c.m_tBCcap * in_p.m_baseFractionBCcap;
            (out_cc.capbc, chargebc) := Spice3.Internal.Functions.junctionCapTransTime(aux1, vbc, in_c.m_tDepCapBC, in_p.m_junctionExpBC, in_c.m_tBCpot, in_c.m_tF1c, in_c.m_f2c, in_c.m_f3c, in_p.m_transitTimeR, gbc, cbc);
            aux2 := in_c.m_tBCcap * (1. - in_p.m_baseFractionBCcap);
            (out_cc.capbx, chargebx) := Spice3.Internal.Functions.junctionCap(aux2, vbx, in_c.m_tDepCapBC, in_p.m_junctionExpBC, in_c.m_tBCpot, in_c.m_tF1c, in_c.m_f2c, in_c.m_f3c);
            out_cc.capcs := 0;
            chargecs := 0;
            vcs := in_p.m_type * (0 - in_m_pVoltageValues[4]);
// ( Subst,  ColP);
            if vcs < 0 then
              arg := 1 - vcs / in_p.m_potentialSubstrate;
              sarg := exp(-in_p.m_exponentialSubstrate * Modelica.Math.log(arg));
              out_cc.capcs := in_p3.m_CScap * sarg;
              chargecs := in_p.m_potentialSubstrate * in_p3.m_CScap * (1 - arg * sarg) / (1 - in_p.m_exponentialSubstrate);
            else
              out_cc.capcs := in_p3.m_CScap * (1 + in_p.m_exponentialSubstrate * vcs / in_p.m_potentialSubstrate);
              chargecs := vcs * in_p3.m_CScap * (1 + in_p.m_exponentialSubstrate * vcs / (2 * in_p.m_potentialSubstrate));
            end if;
//obsolete --> for backward compatibility
            out_cc.iXX := 1;
            annotation(
              smoothOrder(normallyConstant = in_p3) = 1,
              Documentation(info = "<html>
<p>This function NoBypassCode calculates the currents (and the capacitances) that are necessary for the currents to be used in the top-level model.</p>
</html>"));
          end bjtNoBypassCode;

          function bjtRenameParameters "Technology parameter renaming"
            extends Modelica.Icons.Function;
            input Modelica.Electrical.Spice3.Internal.ModelcardBJT2 ex "Modelcard with technology parameters";
            input Real TBJT;
            output BjtModelLineParams intern "Output record with Bjt model line parameters";
          algorithm
            intern.m_minBaseResistIsGiven := 0;
            intern.m_collectorConduct := 0;
            intern.m_emitterConduct := 0;
            intern.m_transitTimeVBCFactor := 0;
            intern.m_excessPhaseFactor := 0;
            intern.m_invEarlyVoltF := 0;
            intern.m_invRollOffF := 0;
            intern.m_invEarlyVoltR := 0;
            intern.m_invRollOffR := 0;
//obsolete
            intern.m_bNPN := 1;
            intern.m_bPNP := 1;
            intern.m_area := 1.0;
            intern.m_bOff := false;
            intern.m_dICvbe := 0;
            intern.m_bICvbeIsGiven := 0;
            intern.m_dICvce := 0;
            intern.m_bICvceIsGiven := 0;
            intern.m_bSensArea := false;
            intern.m_dTemp := 1;
            intern.m_satCur := ex.IS;
            intern.m_betaF := ex.BF;
            intern.m_emissionCoeffF := ex.NF;
            intern.m_leakBEemissionCoeff := ex.NE;
            intern.m_leakBEcurrentIsGiven := if ex.ISE > (-1e40) then 1 else 0;
            intern.m_leakBEcurrent := if ex.ISE > (-1e40) then ex.ISE else 0;
            intern.m_c2IsGiven := if ex.C2 > (-1e40) then 1 else 0;
            intern.m_c2 := if ex.C2 > (-1e40) then ex.C2 else 0;
            intern.m_leakBCcurrentIsGiven := if ex.ISC > (-1e40) then 1 else 0;
            intern.m_leakBCcurrent := if ex.ISC > (-1e40) then ex.ISC else 0;
            intern.m_c4IsGiven := if ex.C4 > (-1e40) then 1 else 0;
            intern.m_c4 := if ex.C4 > (-1e40) then ex.C4 else 0;
            intern.m_betaR := ex.BR;
            intern.m_emissionCoeffR := ex.NR;
            intern.m_leakBCemissionCoeff := ex.NC;
            intern.m_earlyVoltF := ex.VAF;
            intern.m_rollOffF := ex.IKF;
            intern.m_earlyVoltR := ex.VAR;
            intern.m_rollOffR := ex.IKR;
            intern.m_emitterResist := ex.RE;
            intern.m_collectorResist := ex.RC;
            intern.m_baseCurrentHalfResist := ex.IRB;
            intern.m_baseResist := ex.RB;
            intern.m_minBaseResist := if ex.RBM > (-1e40) then ex.RBM else intern.m_baseResist;
            intern.m_depletionCapBE := ex.CJE;
            intern.m_potentialBE := ex.VJE;
            intern.m_junctionExpBE := ex.MJE;
            intern.m_transitTimeF := ex.TF;
            intern.m_transitTimeBiasCoeffF := ex.XTF;
            intern.m_transitTimeHighCurrentF := ex.ITF;
            intern.m_transitTimeFVBC := ex.VTF;
            intern.m_excessPhase := ex.PTF;
            intern.m_depletionCapBC := ex.CJC;
            intern.m_potentialBC := ex.VJC;
            intern.m_junctionExpBC := ex.MJC;
            intern.m_baseFractionBCcap := ex.XCJC;
            intern.m_transitTimeR := ex.TR;
            intern.m_capCS := ex.CJS;
            intern.m_potentialSubstrate := ex.VJS;
            intern.m_exponentialSubstrate := ex.MJS;
            intern.m_betaExp := ex.XTB;
            intern.m_energyGap := ex.EG;
            intern.m_tempExpIS := ex.XTI;
            intern.m_fNcoef := ex.KF;
            intern.m_fNexp := ex.AF;
            intern.m_depletionCapCoeff := ex.FC;
            intern.m_tnom := ex.TNOM + Spice3.Internal.SpiceConstants.CONSTCtoK;
            intern.m_type := TBJT;
            annotation(
              Documentation(info = "<html>
<p>This function assigns the external (given by the user, e.g. IS) technology parameters</p>
<p>to the internal parameters (e.g. m_satCur). It also does the analysis of the IsGiven values.</p>
</html>"));
          end bjtRenameParameters;

          function bjtRenameParametersDev "Temperature calculation"
            extends Modelica.Icons.Function;
            input Real AREA "Area factor";
            input Boolean OFF "Optional initial condition: false - IC not used, true - IC used, not implemented yet";
            input Modelica.SIunits.Voltage IC_VBE "Initial condition value, not yet implemented";
            input Modelica.SIunits.Voltage IC_VCE "Initial condition value, not yet implemented";
            input Boolean UIC "Use initial conditions, UIC";
            input Boolean SENS_AREA "Flag for sensitivity analysis, not yet implemented";
            input Modelica.SIunits.Temp_C TEMP "Temperature";
            output Bjt dev "Output record Bjt";
          algorithm
            dev.m_area := AREA;
            dev.m_bOff := OFF;
            dev.m_bICvbeIsGiven := if IC_VBE > (-1e40) then 1 else 0;
            dev.m_dICvbe := if IC_VBE > (-1e40) then IC_VBE else 0;
            dev.m_bICvceIsGiven := if IC_VCE > (-1e40) then 1 else 0;
            dev.m_dICvce := if IC_VCE > (-1e40) then IC_VCE else 0;
            dev.m_uic := UIC;
            dev.m_bSensArea := SENS_AREA;
            dev.m_dTemp := TEMP + Spice3.Internal.SpiceConstants.CONSTCtoK "Device temperature";
            dev.m_transitTimeHighCurrentF := 0;
            dev.m_invRollOffF := 0;
            dev.m_invRollOffR := 0;
            dev.m_CScap := 0;
            annotation(
              Documentation(info = "<html>
<p>This function assigns the external (given by the user, e.g. AREA) device parameters</p>
<p>to the internal parameters (e.g. m_area). It also does the analysis of the IsGiven values.</p>
</html>"));
          end bjtRenameParametersDev;

          record BjtModelLineVariables "Record for bjt model line variables, obsolete"
            extends Modelica.Icons.Record;
            extends Modelica.Icons.ObsoleteModel;
            Modelica.SIunits.Current m_leakBEcurrent = 1;
            Modelica.SIunits.Current m_leakBCcurrent = 1;
            Modelica.SIunits.Resistance m_minBaseResist = 1;
            Real m_invEarlyVoltF = 1;
            Real m_invRollOffF = 1;
            Real m_invEarlyVoltR = 1;
            Real m_invRollOffR = 1;
            Modelica.SIunits.Conductance m_collectorConduct = 1;
            Modelica.SIunits.Conductance m_emitterConduct = 1;
            Real m_transitTimeVBCFactor = 1;
            Real m_excessPhaseFactor = 1;
            Integer m_type = 1;
            annotation(
              Documentation(info = "<html>
<p>This record contains the model line (also called model card) variables that are used for the bipolar transistor model in SPICE3.</p>
</html>"));
          end BjtModelLineVariables;

          record BjtVariables "Variables for the bjt model, obsolete"
            extends Modelica.Icons.Record;
            extends Modelica.Icons.ObsoleteModel;
            extends Bjt3Variables;
            Real m_CScap = 1;
            annotation(
              Documentation(info = "<html>
<p>This record contains the model variables that are used for the bipoar transistor bjt model in SPICE3.</p>
</html>"));
          end BjtVariables;

          record Bjt3Variables "Variables for the bjt3 model, obsolete"
            extends Modelica.Icons.Record;
            extends Modelica.Icons.ObsoleteModel;
            Modelica.SIunits.Current m_transitTimeHighCurrentF = 1;
            Real m_invRollOffF = 1;
            Real m_invRollOffR = 1;
            annotation(
              Documentation(info = "<html>
<p>This record contains the model variables that are used for the bipoar transistor bjt3 model in SPICE3.</p>
</html>"));
          end Bjt3Variables;
          annotation(
            Documentation(info = "<html>
<p>This package Bjt contains functions and records with data of the Bjt bipolar transistor models.</p>
</html>"));
        end Bjt;

        package Bjt3 "Records and functions for bjt model, obsolete"
          extends Modelica.Icons.InternalPackage;
          extends Modelica.Icons.ObsoleteModel;

          record BjtModelLineParams "Record for bjt model line parameters, obsolete"
            extends Modelica.Icons.Record;
            extends Modelica.Icons.ObsoleteModel;
            Real m_type(start = 1) "device type : 1 = n,  -1 = p";
            Boolean m_bNPN(start = true) "NPN type device";
            Boolean m_bPNP(start = false) "PNP type device";
            Modelica.SIunits.Temp_C m_tnom(start = Spice3.Internal.SpiceConstants.CKTnomTemp) "TNOM, Parameter measurement temperature";
            Modelica.SIunits.Current m_satCur(start = 1.0e-16) "IS, Saturation Current";
            Real m_betaF(start = 100.0) "BF, Ideal forward beta";
            Real m_emissionCoeffF(start = 1.0) "NF, Forward emission coefficient";
            Real m_leakBEemissionCoeff(start = 1.5) "NE, B-E leakage emission coefficient";
            Modelica.SIunits.Current m_leakBEcurrent(start = 0.) "ISE, B-E leakage saturation current";
            Real m_c2(start = 0.) "C2, Obsolete parameter name";
            Modelica.SIunits.Current m_leakBCcurrent(start = 0.) "ISC, B-C leakage saturation current";
            Real m_c4(start = 0.) "C4, Obsolete parameter name";
            Real m_leakBEcurrentIsGiven;
            Real m_c2IsGiven;
            Real m_leakBCcurrentIsGiven;
            Real m_c4IsGiven;
            Real m_betaR(start = 1.0) "BR, Ideal reverse beta";
            Real m_emissionCoeffR(start = 1.0) "NR, Reverse emission coefficient";
            Real m_leakBCemissionCoeff(start = 2.0) "NC, B-C leakage emission coefficient";
            Modelica.SIunits.Voltage m_earlyVoltF(start = 0.0) "VAF, Forward Early voltage";
            Modelica.SIunits.Current m_rollOffF(start = 0.0) "IKF, Forward beta roll-off corner current";
            Modelica.SIunits.Voltage m_earlyVoltR(start = 0.0) "VAR, Reverse Early voltage";
            Modelica.SIunits.Current m_rollOffR(start = 0.0) "IKR, reverse beta roll-off corner current";
            Modelica.SIunits.Resistance m_emitterResist(start = 0.0) "RE, Emitter resistance";
            Modelica.SIunits.Resistance m_collectorResist(start = 0.0) "RC, Collector resistance";
            Modelica.SIunits.Current m_baseCurrentHalfResist(start = 0.0) "IRB, Current for base resistance=(rb+rbm)/2";
            Modelica.SIunits.Resistance m_baseResist(start = 0.0) "RB, Zero bias base resistance";
            Modelica.SIunits.Resistance m_minBaseResist(start = 0.0) "RBM, Minimum base resistance";
            Real m_minBaseResistIsGiven;
            Modelica.SIunits.Capacitance m_depletionCapBE(start = 0.0) "CJE, Zero bias B-E depletion capacitance";
            Modelica.SIunits.Voltage m_potentialBE(start = 0.75) "VJE, B-E built in potential";
            Real m_junctionExpBE(start = 0.33) "MJE, B-E built in potential";
            Modelica.SIunits.Time m_transitTimeF(start = 0.0) "TF, Ideal forward transit time";
            Real m_transitTimeBiasCoeffF(start = 0.0) "XTF, Coefficient for bias dependence of TF";
            Modelica.SIunits.Current m_transitTimeHighCurrentF(start = 0.0) "ITF, High current dependence of TF";
            Modelica.SIunits.Voltage m_transitTimeFVBC(start = 0.0) "VTF, Voltage giving VBC dependence of TF";
            Modelica.SIunits.Temp_C m_excessPhase(start = 0.0) "PTF, Excess phase";
            Modelica.SIunits.Capacitance m_depletionCapBC(start = 0.0) "CJC, Zero bias B-C depletion capacitance";
            Modelica.SIunits.Voltage m_potentialBC(start = 0.75) "VJC, B-C built in potential";
            Real m_junctionExpBC(start = 0.33) "MJC, B-C junction grading coefficient";
            Real m_baseFractionBCcap(start = 1.0) "XCJC, Fraction of B-C cap to internal base";
            Modelica.SIunits.Time m_transitTimeR(start = 0.0) "TR, Ideal reverse transit time";
            Modelica.SIunits.Capacitance m_capCS(start = 0.0) "CJS, Zero bias C-S capacitance";
            Modelica.SIunits.Voltage m_potentialSubstrate(start = 0.75) "VJS, Zero bias C-S capacitance";
            Real m_exponentialSubstrate(start = 0.0) "MJS, Substrate junction grading coefficient";
            Real m_betaExp(start = 0.0) "XTB, Forward and reverse beta temp. exp.";
            Modelica.SIunits.GapEnergy m_energyGap(start = 1.11) "EG, Energy gap for IS temp. dependency";
            Real m_tempExpIS(start = 3.0) "XTI,Temp. exponent for IS";
            Real m_fNcoef(start = 0.0) "KF, Flicker Noise Coefficient";
            Real m_fNexp(start = 1.0) "AF, Flicker Noise Exponent";
            Real m_depletionCapCoeff(start = 0.5) "FC, Forward bias junction fit parameter";
            Real m_collectorConduct(start = 0.0);
            Real m_emitterConduct(start = 0.0);
            Real m_transitTimeVBCFactor(start = 0.0);
            Real m_excessPhaseFactor(start = 0.0);
            Real m_invEarlyVoltF(start = 0.0);
            Types.InverseElectricCurrent m_invRollOffF(start = 0.0);
            Real m_invEarlyVoltR(start = 0.0);
            Types.InverseElectricCurrent m_invRollOffR(start = 0.0);
            annotation(
              Documentation(info = "<html>
<p>This record contains the model line (also called model card) parameters that are used for the bipolar transistor model in SPICE3.</p>
</html>"));
          end BjtModelLineParams;

          record BjtModelLineVariables "Record for bjt model line variables, obsolete"
            extends Modelica.Icons.Record;
            extends Modelica.Icons.ObsoleteModel;
            Modelica.SIunits.Current m_leakBEcurrent;
            Modelica.SIunits.Current m_leakBCcurrent;
            Modelica.SIunits.Resistance m_minBaseResist;
            Real m_invEarlyVoltF;
            Real m_invRollOffF;
            Real m_invEarlyVoltR;
            Real m_invRollOffR;
            Modelica.SIunits.Conductance m_collectorConduct;
            Modelica.SIunits.Conductance m_emitterConduct;
            Real m_transitTimeVBCFactor;
            Real m_excessPhaseFactor;
            Integer m_type;
            annotation(
              Documentation(info = "<html>
<p>This record contains the model line (also called model card) variables that are used for the bipolar transistor model in SPICE3.</p>
</html>"));
          end BjtModelLineVariables;

          function bjtModelLineInitEquations "Initial calculation, obsolete"
            extends Modelica.Icons.Function;
            extends Modelica.Icons.ObsoleteModel;
            input BjtModelLineParams in_p "Input record with Bjt model line parameters";
            output BjtModelLineVariables out_v "Output record with Bjt model line variables";
          protected
            Real xfc;
          algorithm
            out_v.m_type := if in_p.m_bPNP then -1 else 1;
            if not in_p.m_leakBEcurrentIsGiven > 0.5 and in_p.m_c2IsGiven > 0.5 then
              out_v.m_leakBEcurrent := in_p.m_c2 * in_p.m_satCur;
            end if;
            if not in_p.m_leakBCcurrentIsGiven > 0.5 and in_p.m_c4IsGiven > 0.5 then
              out_v.m_leakBCcurrent := in_p.m_c4 * in_p.m_satCur;
            end if;
            if not in_p.m_minBaseResistIsGiven > 0.5 then
              out_v.m_minBaseResist := in_p.m_baseResist;
            end if;
            if in_p.m_earlyVoltF <> 0 then
              out_v.m_invEarlyVoltF := 1 / in_p.m_earlyVoltF;
            end if;
            if in_p.m_rollOffF <> 0 then
              out_v.m_invRollOffF := 1 / in_p.m_rollOffF;
            end if;
            if in_p.m_earlyVoltR <> 0 then
              out_v.m_invEarlyVoltR := 1 / in_p.m_earlyVoltR;
            end if;
            if in_p.m_rollOffR <> 0 then
              out_v.m_invRollOffR := 1 / in_p.m_rollOffR;
            end if;
            if in_p.m_collectorResist <> 0 then
              out_v.m_collectorConduct := 1 / in_p.m_collectorResist;
            end if;
            if in_p.m_emitterResist <> 0 then
              out_v.m_emitterConduct := 1 / in_p.m_emitterResist;
            end if;
            if in_p.m_transitTimeFVBC <> 0 then
              out_v.m_transitTimeVBCFactor := 1 / (in_p.m_transitTimeFVBC * 1.44);
            end if;
            out_v.m_excessPhaseFactor := in_p.m_excessPhase / (180.0 / Modelica.Constants.pi) * in_p.m_transitTimeF;
            xfc := Modelica.Math.log(1 - in_p.m_depletionCapCoeff);
            annotation(
              Documentation(info = "<html>
<p>Within this function some parameters are initially precalculated from model line parameters.</p>
</html>"));
          end bjtModelLineInitEquations;

          record Bjt3 "Record for bjt3 device parameters, obsolete"
            extends Spice3.Internal.Model.Model;
            extends Modelica.Icons.Record;
            extends Modelica.Icons.ObsoleteModel;
            Real m_area(start = 1.0) "AREA";
            Boolean m_bOff(start = false) "OFF";
            Modelica.SIunits.Voltage m_dICvbe(start = 0.0) "IC_VBE";
            Real m_bICvbeIsGiven(start = 0.0);
            Modelica.SIunits.Voltage m_dICvce(start = 0.0) "IC_VCE";
            Real m_bICvceIsGiven(start = 0.0);
            Boolean m_bSensArea(start = false) "SENS_AREA";
            Real m_transitTimeHighCurrentF(start = 0.0);
            Types.InverseElectricCurrent m_invRollOffF(start = 0);
            Types.InverseElectricCurrent m_invRollOffR(start = 0);
            annotation(
              Documentation(info = "<html>
<p>This record contains the device parameters that are used for the bipolar transistor bjt3 model in SPICE3.</p>
</html>"));
          end Bjt3;

          record Bjt3Variables "Variables for the bjt3 model, obsolete"
            extends Modelica.Icons.Record;
            extends Modelica.Icons.ObsoleteModel;
            Modelica.SIunits.Current m_transitTimeHighCurrentF(start = 0.0);
            Real m_invRollOffF(start = 0.0);
            Real m_invRollOffR(start = 0.0);
            annotation(
              Documentation(info = "<html>
<p>This record contains the model variables that are used for the bipoar transistor bjt3 model in SPICE3.</p>
</html>"));
          end Bjt3Variables;

          record Bjt3Calc "Bjt3 variables, obsolete"
            extends Modelica.Icons.Record;
            extends Modelica.Icons.ObsoleteModel;
            Modelica.SIunits.Current m_tSatCur = 0;
            Real m_tBetaF = 1;
            Real m_tBetaR = 1;
            Modelica.SIunits.Current m_tBEleakCur = 1e-14;
            Modelica.SIunits.Current m_tBCleakCur = 1e-14;
            Real m_tBEcap = 0;
            Real m_tBEpot = 0.7;
            Real m_tBCcap = 0;
            Real m_tBCpot = 0.7;
            Real m_tDepCapBE = 0.7;
            Real m_tDepCapBC = 0.7;
            Real m_tVcrit = 0.7;
            Real m_dVt = 0.25;
            Real m_tF1c = 0;
            Real m_f2c = 0;
            Real m_f3c = 0;
            Real m_tF1e = 0;
            Real m_f2e = 0;
            Real m_f3e = 0;
            annotation(
              Documentation(info = "<html>
<p>This record contains the model variables that are used for the bipolar transistor model in SPICE3.</p>
</html>"));
          end Bjt3Calc;

          record CurrentsCapacitances "Bjt3 variables, obsolete"
            extends Modelica.Icons.Record;
            extends Modelica.Icons.ObsoleteModel;
            Modelica.SIunits.Current iBE(start = 0.0);
            //current through diode dE1 (ideal part)
            Modelica.SIunits.Current iBEN(start = 0.0);
            //current through diode dE2 (non ideal part)
            Modelica.SIunits.Current iBC(start = 0.0);
            //current through diode dC1 (ideal part)
            Modelica.SIunits.Current iBCN(start = 0.0);
            //current through diode dC2 (non ideal part)
            Modelica.SIunits.Current iCC(start = 0.0);
            //channel current
            Modelica.SIunits.Current capbc(start = 0.0);
            Modelica.SIunits.Capacitance capbe(start = 0.0);
            Modelica.SIunits.Capacitance capbx(start = 0.0);
            Real iXX(start = 0.0);
            Modelica.SIunits.Capacitance capcs(start = 0.0);
            annotation(
              Documentation(info = "<html>
<p>This record contains the model variables that are used for the bipoar transistor model in SPICE3.</p>
</html>"));
          end CurrentsCapacitances;

          function bjt3InitEquations "Initial calculation, obsolete"
            extends Modelica.Icons.Function;
            extends Modelica.Icons.ObsoleteModel;
            input Bjt3 in_p "Input record Bjt3";
            input BjtModelLineParams in_pml "Input record with Bjt model line parameters";
            input BjtModelLineVariables in_vl "Input record with Bjt model line variables";
            output Bjt3Variables out_v "Output record with Bjt variables";
          algorithm
// calculate the parameters that depend on the area factor
            out_v.m_transitTimeHighCurrentF := in_pml.m_transitTimeHighCurrentF * in_p.m_area;
            out_v.m_invRollOffF := in_vl.m_invRollOffF / in_p.m_area;
            out_v.m_invRollOffR := in_vl.m_invRollOffR / in_p.m_area;
            annotation(
              Documentation(info = "<html>
<p>Within this function some parameters are initially precalculated from model line parameters.</p>
</html>"));
          end bjt3InitEquations;

          function bjt3CalcTempDependencies "Temperature dependency calculation, obsolete"
            extends Modelica.Icons.Function;
            extends Modelica.Icons.ObsoleteModel;
            input Bjt3 in_p3 "Input record Bjt3";
            input BjtModelLineParams in_p "Input record with Bjt model line parameters";
            input Spice3.Internal.Model.Model m "Input record model";
            input BjtModelLineVariables in_vl "Input record with Bjt model line variables";
            output Bjt3Calc out_c "Output record Bjt3Calc";
          protected
            Real xfc;
            Real gmanew;
            Real fact1;
            Modelica.SIunits.Voltage vt;
            Real fact2;
            Real egfet;
            Types.GapEnergyPerEnergy arg;
            Real pbfact;
            Real ratlog;
            Real ratio1;
            Real factlog;
            Real factor;
            Real bfactor;
            Real pbo;
            Real gmaold;
          algorithm
            fact1 := in_p.m_tnom / Spice3.Internal.SpiceConstants.REFTEMP;
            vt := m.m_dTemp * Spice3.Internal.SpiceConstants.CONSTKoverQ;
            fact2 := m.m_dTemp / Spice3.Internal.SpiceConstants.REFTEMP;
            egfet := 1.16 - 7.02e-4 * m.m_dTemp * m.m_dTemp / (m.m_dTemp + 1108);
            arg := (-egfet / (2 * Modelica.Constants.k * m.m_dTemp)) + 1.1150877 / (Modelica.Constants.k * (Spice3.Internal.SpiceConstants.REFTEMP + Spice3.Internal.SpiceConstants.REFTEMP));
            pbfact := -2 * vt * (1.5 * Modelica.Math.log(fact2) + Spice3.Internal.SpiceConstants.CHARGE * arg);
            ratlog := Modelica.Math.log(m.m_dTemp / in_p.m_tnom);
            ratio1 := m.m_dTemp / in_p.m_tnom - 1;
            factlog := ratio1 * in_p.m_energyGap / vt + in_p.m_tempExpIS * ratlog;
            factor := exp(factlog);
            bfactor := exp(ratlog * in_p.m_betaExp);
            pbo := (in_p.m_potentialBE - pbfact) / fact1;
            gmaold := (in_p.m_potentialBE - pbo) / pbo;
            out_c.m_tSatCur := in_p.m_satCur * factor * in_p3.m_area;
            out_c.m_tBetaF := in_p.m_betaF * bfactor;
            out_c.m_tBetaR := in_p.m_betaR * bfactor;
            out_c.m_tBEleakCur := in_vl.m_leakBEcurrent * exp(factlog / in_p.m_leakBEemissionCoeff) / bfactor * in_p3.m_area;
            out_c.m_tBCleakCur := in_vl.m_leakBCcurrent * exp(factlog / in_p.m_leakBCemissionCoeff) / bfactor * in_p3.m_area;
            out_c.m_tBEcap := in_p.m_depletionCapBE / (1 + in_p.m_junctionExpBE * (4e-4 * (in_p.m_tnom - Spice3.Internal.SpiceConstants.REFTEMP) - gmaold));
            out_c.m_tBEpot := fact2 * pbo + pbfact;
            gmanew := (out_c.m_tBEpot - pbo) / pbo;
            out_c.m_tBEcap := out_c.m_tBEcap * (1 + in_p.m_junctionExpBE * (4e-4 * (m.m_dTemp - Spice3.Internal.SpiceConstants.REFTEMP) - gmanew));
            pbo := (in_p.m_potentialBC - pbfact) / fact1;
            gmaold := (in_p.m_potentialBC - pbo) / pbo;
            out_c.m_tBCcap := in_p.m_depletionCapBC / (1 + in_p.m_junctionExpBC * (4e-4 * (in_p.m_tnom - Spice3.Internal.SpiceConstants.REFTEMP) - gmaold));
            out_c.m_tBCpot := fact2 * pbo + pbfact;
            gmanew := (out_c.m_tBCpot - pbo) / pbo;
            out_c.m_tBCcap := out_c.m_tBCcap * (1 + in_p.m_junctionExpBC * (4e-4 * (m.m_dTemp - Spice3.Internal.SpiceConstants.REFTEMP) - gmanew));
            out_c.m_tDepCapBE := in_p.m_depletionCapCoeff * out_c.m_tBEpot;
            out_c.m_tDepCapBC := in_p.m_depletionCapCoeff * out_c.m_tBCpot;
            xfc := Modelica.Math.log(1 - in_p.m_depletionCapCoeff);
            out_c.m_tVcrit := vt * Modelica.Math.log(vt / (Spice3.Internal.SpiceConstants.CONSTroot2 * in_p.m_satCur));
            out_c.m_dVt := vt;
// calculate the parameters that depend on the area factor
            out_c.m_tBEcap := out_c.m_tBEcap * in_p3.m_area;
            out_c.m_tBCcap := out_c.m_tBCcap * in_p3.m_area;
            (out_c.m_tF1c, out_c.m_f2c, out_c.m_f3c) := Spice3.Internal.Functions.junctionCapCoeffs(in_p.m_junctionExpBC, in_p.m_depletionCapCoeff, out_c.m_tBCpot);
            (out_c.m_tF1e, out_c.m_f2e, out_c.m_f3e) := Spice3.Internal.Functions.junctionCapCoeffs(in_p.m_junctionExpBE, in_p.m_depletionCapCoeff, out_c.m_tBEpot);
            annotation(
              Documentation(info = "<html>
<p>In this function for the bipolar transistor model temperature dependencies are calculated using temperature treating functions from the equation package.</p>
</html>"));
          end bjt3CalcTempDependencies;

          function bjt3NoBypassCode "Calculation of currents, obsolete"
            extends Modelica.Icons.Function;
            extends Modelica.Icons.ObsoleteModel;
            input Spice3.Internal.Model.Model in_m "Input record model";
            input Bjt in_p3 "Input record Bjt3";
            input BjtModelLineParams in_p "Input record with Bjt model line parameters";
            input Bjt3Calc in_c "Input record Bjt3Calc";
            input BjtModelLineVariables in_vl "Input record with Bjt model line variables";
            input Modelica.SIunits.Voltage[6] in_m_pVoltageValues;
            /* 1 Col; 2 Base; 3 Emit; 4 ColP; 5 BaseP; 6 EmitP */
            input Boolean in_m_bInit;
            output CurrentsCapacitances out_cc "Output record with calculated currents and capacitances";
            output Modelica.SIunits.Capacitance capbe "Capacitance";
            output Modelica.SIunits.Capacitance capbc "Capacitance";
            output Modelica.SIunits.Capacitance capbx "Capacitance";
          protected
            Modelica.SIunits.Voltage vce;
            Modelica.SIunits.Voltage vbe;
            Modelica.SIunits.Voltage vbx;
            Modelica.SIunits.Voltage vbc;
            Modelica.SIunits.Conductance gbe;
            Modelica.SIunits.Current cbe;
            Modelica.SIunits.Conductance gbc;
            Modelica.SIunits.Current cbc;
            Modelica.SIunits.Conductance gben;
            Modelica.SIunits.Current cben;
            Modelica.SIunits.Conductance gbcn;
            Modelica.SIunits.Current cbcn;
            Modelica.SIunits.Current cjbe;
            Modelica.SIunits.Current cjbc;
            Real dqbdve;
            Real dqbdvc;
            Real qb;
            Real q1;
            Real q2;
            Real arg;
            Real sqarg;
            Real cc;
            Modelica.SIunits.Current cex;
            Modelica.SIunits.Conductance gex;
            Modelica.SIunits.Time ttime;
            Real step;
            Real laststep;
            Modelica.SIunits.Current bcex0;
            Modelica.SIunits.Current bcex1;
            Real arg1;
            Real arg2;
            Real denom;
            Real arg3;
            Real rbpr;
            Real rbpi;
            Real gx;
            Real xjrb;
            Real go;
            Real gm;
            Real captt;
            Modelica.SIunits.Charge chargebe;
            Modelica.SIunits.Charge chargebc;
            Modelica.SIunits.Charge chargebx;
            Real argtf;
            Real exponent;
            //  Modelica.SIunits.Temp_K temp;
            Real temp;
            Real aux1;
            Real aux2;
          algorithm
            vce := in_p.m_type * (in_m_pVoltageValues[4] - in_m_pVoltageValues[6]);
// ( ColP, EmitP);
            vbe := in_p.m_type * (in_m_pVoltageValues[5] - in_m_pVoltageValues[6]);
// ( BaseP, EmitP);
            vbx := in_p.m_type * (in_m_pVoltageValues[2] - in_m_pVoltageValues[4]);
// ( Base, ColP);
            if Spice3.Internal.SpiceRoot.useInitialConditions() then
              if in_p3.m_bICvbeIsGiven > 0.5 then
                vbe := in_p.m_type * in_p3.m_dICvbe;
              end if;
              if in_p3.m_bICvceIsGiven > 0.5 then
                vce := in_p.m_type * in_p3.m_dICvce;
              end if;
              vbx := vbe - vce;
            elseif Spice3.Internal.SpiceRoot.initJunctionVoltages() then
              if in_p3.m_bOff then
                vbe := 0.0;
                vce := 0.0;
                vbx := 0.0;
              else
                vbe := in_c.m_tVcrit;
                vce := vbe;
                vbx := 0.0;
              end if;
            end if;
            vbc := vbe - vce;
// junction current
            (cbe, gbe) := Spice3.Internal.Functions.junction2(vbe, in_m.m_dTemp, in_p.m_emissionCoeffF, in_c.m_tSatCur);
            out_cc.iBE := in_p.m_type * cbe / in_c.m_tBetaF;
            (cben, gben) := Spice3.Internal.Functions.junction2(vbe, in_m.m_dTemp, in_p.m_leakBEemissionCoeff, in_c.m_tBEleakCur);
            out_cc.iBEN := in_p.m_type * cben;
            (cbc, gbc) := Spice3.Internal.Functions.junction2(vbc, in_m.m_dTemp, in_p.m_emissionCoeffR, in_c.m_tSatCur);
            out_cc.iBC := in_p.m_type * cbc / in_c.m_tBetaR;
            (cbcn, gbcn) := Spice3.Internal.Functions.junction2(vbc, in_m.m_dTemp, in_p.m_leakBCemissionCoeff, in_c.m_tBCleakCur);
            out_cc.iBCN := in_p.m_type * cbcn;
            cjbe := cbe / in_c.m_tBetaF + cben;
            cjbc := cbc / in_c.m_tBetaR + cbcn;
// determine base charge terms
            q1 := 1.0 / (1.0 - in_p.m_invEarlyVoltF * vbc - in_p.m_invEarlyVoltR * vbe);
            if in_vl.m_invRollOffF == 0 and in_vl.m_invRollOffR == 0 then
              qb := q1;
              dqbdve := q1 * qb * in_p.m_invEarlyVoltR;
              dqbdvc := q1 * qb * in_p.m_invEarlyVoltF;
            else
              q2 := in_vl.m_invRollOffF * cbe + in_vl.m_invRollOffR * cbc;
              arg := max(0.0, 1 + 4 * q2);
              sqarg := 1;
              if arg <> 0 then
                sqarg := sqrt(arg);
              end if;
              qb := q1 * (1 + sqarg) / 2;
              dqbdve := q1 * (qb * in_p.m_invEarlyVoltR + in_vl.m_invRollOffF * gbe / sqarg);
              dqbdvc := q1 * (qb * in_p.m_invEarlyVoltF + in_vl.m_invRollOffR * gbc / sqarg);
            end if;
// determine dc incremental conductances - Weil's approximation
            cc := 0.0;
            cex := cbe;
            gex := gbe;
            ttime := 1;
            if in_vl.m_excessPhaseFactor <> 0 and ttime > 0.0 then
              step := 0;
              laststep := 1;
              bcex0 := 0;
              bcex1 := 0;
              if bcex1 == 0.0 then
                bcex1 := cbe / qb;
                bcex0 := bcex1;
              end if;
              arg1 := step / in_vl.m_excessPhaseFactor;
              arg2 := 3 * arg1;
              arg1 := arg2 * arg1;
              denom := 1 + arg1 + arg2;
              arg3 := arg1 / denom;
              cc := (bcex0 * (1 + step / laststep + arg2) - bcex1 * step / laststep) / denom;
              cex := cbe * arg3;
              gex := gbe * arg3;
            end if;
            cc := cc + (cex - cbc) / qb;
// resistances
            rbpr := in_vl.m_minBaseResist / in_p3.m_area;
            rbpi := in_p.m_baseResist / in_p3.m_area - rbpr;
            gx := rbpr + rbpi / qb;
            xjrb := in_p.m_baseCurrentHalfResist * in_p3.m_area;
            if xjrb <> 0 then
              arg1 := max((cjbe + cjbc) / xjrb, 1e-9);
              arg2 := ((-1) + sqrt(1 + 14.59025 * arg1)) / 2.4317 / sqrt(arg1);
              arg1 := tan(arg2);
              gx := rbpr + 3 * rbpi * (arg1 - arg2) / arg2 / arg1 / arg1;
            end if;
            if gx <> 0 then
              gx := 1 / gx;
            end if;
// determine dc incremental conductances
            go := (gbc + (cex - cbc) * dqbdvc / qb) / qb;
            gm := (gex - (cex - cbc) * dqbdve / qb) / qb - go;
            out_cc.iCC := in_p.m_type * cc;
// charge storage elements and transit time calculation
            captt := 0.0;
            if in_p.m_transitTimeF <> 0.0 and vbe > 0.0 then
              argtf := 0.0;
              arg2 := 0.0;
              arg3 := 0.0;
              if in_p.m_transitTimeBiasCoeffF <> 0.0 then
                argtf := in_p.m_transitTimeBiasCoeffF;
                if in_vl.m_transitTimeVBCFactor <> 0.0 then
                  exponent := min(50., vbc * in_vl.m_transitTimeVBCFactor);
                  argtf := argtf * exp(exponent);
                end if;
                arg2 := argtf;
                if in_p.m_transitTimeHighCurrentF <> 0 then
                  temp := cbe / (cbe + in_p.m_transitTimeHighCurrentF);
                  argtf := argtf * temp * temp;
                  arg2 := argtf * (3 - temp - temp);
                end if;
                arg3 := cbe * argtf * in_vl.m_transitTimeVBCFactor;
              end if;
              cbe := cbe * (1 + argtf) / qb;
              gbe := (gbe * (1 + arg2) - cbe * dqbdve) / qb;
              captt := in_p.m_transitTimeF * (arg3 - cbe * dqbdvc) / qb;
            end if;
            (capbe, chargebe) := Spice3.Internal.Functions.junctionCapTransTime(in_c.m_tBEcap, vbe, in_c.m_tDepCapBE, in_p.m_junctionExpBE, in_c.m_tBEpot, in_c.m_tF1e, in_c.m_f2e, in_c.m_f3e, in_p.m_transitTimeF, gbe, cbe);
            out_cc.iXX := 0;
            aux1 := in_c.m_tBCcap * in_p.m_baseFractionBCcap;
            (capbc, chargebc) := Spice3.Internal.Functions.junctionCapTransTime(aux1, vbc, in_c.m_tDepCapBC, in_p.m_junctionExpBC, in_c.m_tBCpot, in_c.m_tF1c, in_c.m_f2c, in_c.m_f3c, in_p.m_transitTimeR, gbc, cbc);
            aux2 := in_c.m_tBCcap * (1. - in_p.m_baseFractionBCcap);
            (capbx, chargebx) := Spice3.Internal.Functions.junctionCap(aux2, vbx, in_c.m_tDepCapBC, in_p.m_junctionExpBC, in_c.m_tBCpot, in_c.m_tF1c, in_c.m_f2c, in_c.m_f3c);
            annotation(
              Documentation(info = "<html>
<p>This function NoBypassCode calculates the currents (and the capacitances) that are necessary for the currents to be used in the top-level model.</p>
</html>"));
          end bjt3NoBypassCode;

          record Bjt "Record for bjt device parameters, obsolete"
            extends Bjt3;
            annotation(
              Documentation(info = "<html>
<p>This record contains the device parameters that are used for the bipolar transistor bjt model in SPICE3.</p>
</html>"));
          end Bjt;

          record BjtVariables "Variables for the bjt model, obsolete"
            extends Bjt3Variables;
            Real m_CScap;
            annotation(
              Documentation(info = "<html>
<p>This record contains the model variables that are used for the bipoar transistor bjt model in SPICE3.</p>
</html>"));
          end BjtVariables;

          function bjtInitEquations "Initial calculation, obsolete"
            extends Modelica.Icons.Function;
            extends Modelica.Icons.ObsoleteModel;
            input Bjt in_p "Input record Bjt";
            input BjtModelLineParams in_pml "Input record with Bjt model line parameters";
            input BjtModelLineVariables in_vl "Input record with Bjt model line variables";
            output BjtVariables out_v "Output record with Bjt variables";
          protected
            Bjt3Variables v3 "Record with Bjt variables";
          algorithm
            v3 := bjt3InitEquations(in_p, in_pml, in_vl);
// calculate the parameters that depend on the area factor
            out_v.m_CScap := in_pml.m_capCS * in_p.m_area;
            annotation(
              Documentation(info = "<html>
<p>Within this function some parameters are initially precalculated from model line parameters.</p>
</html>"));
          end bjtInitEquations;

          function bjtNoBypassCode "Calculation of currents, obsolete"
            extends Modelica.Icons.Function;
            extends Modelica.Icons.ObsoleteModel;
            input Spice3.Internal.Model.Model in_m "Input record model";
            input Bjt3 in_p3 "Input record Bjt3";
            input BjtModelLineParams in_p "Input record with Bjt model line parameters";
            input Bjt3Calc in_c "Input record Bjt3Calc";
            input BjtVariables in_v "Input record with Bjt variables";
            input BjtModelLineVariables in_vl;
            input Modelica.SIunits.Voltage[6] in_m_pVoltageValues;
            /* 1 Col; 2 Base; 3 Emit; 4 Subst; 5 ColP; 6 BaseP; 7 EmitP */
            input Boolean in_m_bInit;
            output CurrentsCapacitances out_cc "Output record with calculated currents and capacitances";
            output Modelica.SIunits.Capacitance capbe "Capacitance";
            output Modelica.SIunits.Capacitance capbc "Capacitance";
            output Modelica.SIunits.Capacitance capbx "Capacitance";
          protected
            Modelica.SIunits.Voltage[6] bjt3_VoltageValues;
            /* 1 Col; 2 Base; 3 Emit; 4 ColP; 5 BaseP; 6 EmitP */
            Integer i;
            Modelica.SIunits.Capacitance capcs;
            Modelica.SIunits.Charge chargecs;
            Modelica.SIunits.Voltage vcs;
            Real arg;
            Real sarg;
          algorithm
// Bjt3 has got only 6 pins, therefore new vector VoltageValues
            for i in 1:3 loop
              bjt3_VoltageValues[i] := in_m_pVoltageValues[i];
            end for;
            for i in 4:6 loop
              bjt3_VoltageValues[i] := in_m_pVoltageValues[i];
            end for;
            (out_cc, capbe, capbc, capbx) := bjt3NoBypassCode(in_m, in_p3, in_p, in_c, in_vl, bjt3_VoltageValues, in_m_bInit);
            capcs := 0;
            chargecs := 0;
            vcs := in_p.m_type * (0 - in_m_pVoltageValues[4]);
// ( Subst,  ColP);
            if vcs < 0 then
              arg := 1 - vcs / in_p.m_potentialSubstrate;
              sarg := exp(-in_p.m_exponentialSubstrate * Modelica.Math.log(arg));
              capcs := in_v.m_CScap * sarg;
              chargecs := in_p.m_potentialSubstrate * in_v.m_CScap * (1 - arg * sarg) / (1 - in_p.m_exponentialSubstrate);
            else
              capcs := in_v.m_CScap * (1 + in_p.m_exponentialSubstrate * vcs / in_p.m_potentialSubstrate);
              chargecs := vcs * in_v.m_CScap * (1 + in_p.m_exponentialSubstrate * vcs / (2 * in_p.m_potentialSubstrate));
            end if;
            annotation(
              Documentation(info = "<html>
<p>This function NoBypassCode calculates the currents (and the capacitances) that are necessary for the currents to be used in the top-level model.</p>
</html>"));
          end bjtNoBypassCode;

          function bjtRenameParameters "Technology parameter renaming, obsolete"
            extends Modelica.Icons.Function;
            extends Modelica.Icons.ObsoleteModel;
            input Modelica.Electrical.Spice3.Internal.ModelcardBJT ex "Modelcard with technology parameters";
            input Spice3.Internal.SpiceConstants con "Spice constants";
            input Real TBJT "Type";
            output BjtModelLineParams intern "Output record with Bjt model line parameters";
          algorithm
            intern.m_type := TBJT;
            intern.m_satCur := ex.IS;
            intern.m_betaF := ex.BF;
            intern.m_emissionCoeffF := ex.NF;
            intern.m_leakBEemissionCoeff := ex.NE;
            intern.m_leakBEcurrentIsGiven := if ex.ISE > (-1e40) then 1 else 0;
            intern.m_leakBEcurrent := if ex.ISE > (-1e40) then ex.ISE else 0;
            intern.m_c2IsGiven := if ex.C2 > (-1e40) then 1 else 0;
            intern.m_c2 := if ex.C2 > (-1e40) then ex.C2 else 0;
            intern.m_leakBCcurrentIsGiven := if ex.ISC > (-1e40) then 1 else 0;
            intern.m_leakBCcurrent := if ex.ISC > (-1e40) then ex.ISC else 0;
            intern.m_c4IsGiven := if ex.C4 > (-1e40) then 1 else 0;
            intern.m_c4 := if ex.C4 > (-1e40) then ex.C4 else 0;
            intern.m_betaR := ex.BR;
            intern.m_emissionCoeffR := ex.NR;
            intern.m_leakBCemissionCoeff := ex.NC;
            intern.m_earlyVoltF := ex.VAF;
            intern.m_rollOffF := ex.IKF;
            intern.m_earlyVoltR := ex.VAR;
            intern.m_rollOffR := ex.IKR;
            intern.m_emitterResist := ex.RE;
            intern.m_collectorResist := ex.RC;
            intern.m_baseCurrentHalfResist := ex.IRB;
            intern.m_baseResist := ex.RB;
            intern.m_minBaseResistIsGiven := if ex.RBM > (-1e40) then 1 else 0;
            intern.m_minBaseResist := if ex.RBM > (-1e40) then ex.RBM else 0;
            intern.m_depletionCapBE := ex.CJE;
            intern.m_potentialBE := ex.VJE;
            intern.m_junctionExpBE := ex.MJE;
            intern.m_transitTimeF := ex.TF;
            intern.m_transitTimeBiasCoeffF := ex.XTF;
            intern.m_transitTimeHighCurrentF := ex.ITF;
            intern.m_transitTimeFVBC := ex.VTF;
            intern.m_excessPhase := ex.PTF;
            intern.m_depletionCapBC := ex.CJC;
            intern.m_potentialBC := ex.VJC;
            intern.m_junctionExpBC := ex.MJC;
            intern.m_baseFractionBCcap := ex.XCJC;
            intern.m_transitTimeR := ex.TR;
            intern.m_capCS := ex.CJS;
            intern.m_potentialSubstrate := ex.VJS;
            intern.m_exponentialSubstrate := ex.MJS;
            intern.m_betaExp := ex.XTB;
            intern.m_energyGap := ex.EG;
            intern.m_tempExpIS := ex.XTI;
            intern.m_fNcoef := ex.KF;
            intern.m_fNexp := ex.AF;
            intern.m_depletionCapCoeff := ex.FC;
            intern.m_tnom := if ex.TNOM > (-1e40) then ex.TNOM + Spice3.Internal.SpiceConstants.CONSTCtoK else 300.15;
            annotation(
              Documentation(info = "<html>
<p>This function assigns the external (given by the user, e.g., IS) technology parameters
to the internal parameters (e.g., m_satCur). It also does the analysis of the IsGiven values.</p>
</html>"));
          end bjtRenameParameters;

          function bjtRenameParametersDev "Temperature calculation, obsolete"
            extends Modelica.Icons.Function;
            extends Modelica.Icons.ObsoleteModel;
            input Real AREA "Area factor";
            input Boolean OFF "Optional initial condition: false - IC not used, true - IC used, not implemented yet";
            input Modelica.SIunits.Voltage IC_VBE "Initial condition value, not yet implemented";
            input Modelica.SIunits.Voltage IC_VCE "Initial condition value, not yet implemented";
            input Boolean SENS_AREA "Flag for sensitivity analysis, not yet implemented";
            output Bjt3 dev "Output record Bjt3";
          algorithm
            dev.m_area := AREA;
            dev.m_bOff := OFF;
            dev.m_bICvbeIsGiven := if IC_VBE > (-1e40) then 1 else 0;
            dev.m_dICvbe := if IC_VBE > (-1e40) then IC_VBE else 0;
            dev.m_bICvceIsGiven := if IC_VCE > (-1e40) then 1 else 0;
            dev.m_dICvce := if IC_VCE > (-1e40) then IC_VCE else 0;
            dev.m_bSensArea := SENS_AREA;
            annotation(
              Documentation(info = "<html>
<p>This function assigns the external (given by the user, e.g., AREA) device parameters to the internal parameters (e.g., m_area). It also does the analysis of the IsGiven values.</p>
</html>"));
          end bjtRenameParametersDev;

          function bjtRenameParametersDevTemp "Temperature calculation, obsolete"
            extends Modelica.Icons.Function;
            extends Modelica.Icons.ObsoleteModel;
            input Modelica.SIunits.Temp_C TEMP "Temperature";
            output Spice3.Internal.Model.Model m "Output record model";
          algorithm
            m.m_dTemp := TEMP + Spice3.Internal.SpiceConstants.CONSTCtoK;
            annotation(
              Documentation(info = "<html>
<p>This function calculates device parameters which are temperature dependent.</p>
</html>"));
          end bjtRenameParametersDevTemp;

          function bjtRenameParametersType "Type, obsolete"
            extends Modelica.Icons.Function;
            extends Modelica.Icons.ObsoleteModel;
            input Real TBJT "Type";
            output BjtModelLineParams dev_type "Output record Bjt3";
          algorithm
            dev_type.m_type := TBJT;
            annotation(
              Documentation(info = "<html>
<p>This function assigns the external (given by the user, e.g. AREA) device parameters to the internal parameters (e.g. m_area). It also does the analysis of the IsGiven values.</p>
</html>"));
          end bjtRenameParametersType;
          annotation(
            Documentation(info = "<html>
<p>The package Bjt3 is obsolete. Please use package Bjt instead.</p>
<p>This package Bjt3 contains functions and records with data of the Bjt3 bipolar transistor models.</p>
</html>"));
        end Bjt3;

        package Fet
          extends Modelica.Icons.InternalPackage;

          record Fet "Record for Fet parameters"
            extends Modelica.Electrical.Spice3.Internal.Model.Model;
            Real m_area(start = 1.0) "AREA, Area factor";
            Boolean m_off(start = false) "OFF, Device initially off";
            SI.Voltage m_dICVDS(start = 0.0) "IC_VDS";
            Real m_bICVDSIsGiven "IC_VDS, IsGivenValue";
            SI.Voltage m_dICVGS(start = 0.0) "IC_VGS";
            Real m_bICVGSIsGiven "IC_VGS, IsGivenValue";
            Boolean m_uic "Use initial conditions, UIC";
            SI.Current m_tSatCur(start = 0);
            SI.Voltage m_tGatePot(start = 0);
            SI.Capacitance m_tCGS(start = 0);
            SI.Capacitance m_tCGD(start = 0);
            SI.Voltage m_corDepCap(start = 0);
            SI.Voltage m_vcrit(start = 0);
            SI.Voltage m_f1(start = 0);
            Real m_f2(start = 0);
            Real m_f3(start = 0);
            SI.Voltage m_dVt(start = 0);
            SI.Voltage m_vgs "Vgs, Voltage G-S";
            SI.Voltage m_vgd "Vgd, Voltage G-D";
            SI.Voltage m_vds "Vds";
            SI.Current m_cgd "Igd";
            SI.Conductance m_ggd "Ggd";
            SI.Current m_cgs "Igs";
            SI.Conductance m_ggs "Ggs";
            SI.Charge m_chargegd "Qgd";
            SI.Capacitance m_capgd "Cgd, G-D junction cap";
            SI.Charge m_chargegs "Qgs";
            SI.Capacitance m_capgs "Cgs, G-S junction capactance";
            SI.Current m_cdrain "Idrain";
            SI.Conductance m_gm "Gm";
            SI.Conductance m_gds "Gds";
          end Fet;

          record FetModelLine "Record for Fet model line parameters"
            extends Modelica.Icons.Record;
            SI.Voltage m_threshold(start = -2.0) "VTO";
            Real m_beta(start = 1e-4) "BETA";
            SI.InversePotential m_lModulation(start = 0.0) "LAMBDA";
            SI.Resistance m_drainResist(start = 0.0) "RD";
            SI.Resistance m_sourceResist(start = 0) "RS";
            SI.Capacitance m_capGS(start = 0) "CGS";
            SI.Capacitance m_capGD(start = 0) "CGD";
            SI.Voltage m_gatePotential(start = 1.0) "PB";
            SI.Current m_gateSatCurrent(start = 1e-14) "IS";
            Real m_depletionCapCoeff(start = 0.5) "FC";
            Real m_b(start = 1.0) "B";
            Real m_fNcoef(start = 0.0) "KF";
            Real m_fNexp(start = 1.0) "AF";
            SI.Conductance m_drainConduct(start = 0);
            SI.Conductance m_sourceConduct(start = 0);
            SI.Temp_K m_tnom(start = Modelica.Electrical.Spice3.Internal.SpiceConstants.CKTnomTemp) "TNOM";
          end FetModelLine;

          record CurrrentsCapacitances "Currents and Capacities"
            extends Modelica.Icons.Record;
            SI.Current idrain(start = 0);
            SI.Current iGD(start = 0);
            SI.Current iGS(start = 0);
            SI.Capacitance cGS(start = 0);
            SI.Capacitance cGD(start = 0);
            annotation(
              Documentation(info = "<html>
<p>This record CurrentsCapacities contains values for the currents and the capacities inside the model Jfet.</p>
</html>"));
          end CurrrentsCapacitances;

          function fetRenameParametersDev "Device Parameter renaming to internal names"
            extends Modelica.Icons.Function;
            input Real AREA "Number of parallel connected identical elements";
            input Boolean OFF "Optional initial condition: 0 - IC not used, 1 - IC used, not implemented yet";
            input SI.Voltage IC_VDS "Initial condition value VDS, not implemented yet";
            input SI.Voltage IC_VGS "Initial condition value VGS, not implemented yet";
            input Boolean UIC "Use initial conditions, UIC";
            input SI.Temp_C TEMP "Temperature";
            output Fet dev "Output record MESFET";
          algorithm
            dev.m_capgd := 0;
            dev.m_capgs := 0;
            dev.m_cdrain := 0;
            dev.m_cgd := 0;
            dev.m_cgs := 0;
            dev.m_chargegd := 0;
            dev.m_chargegs := 0;
            dev.m_corDepCap := 0;
            dev.m_dVt := 0;
            dev.m_f1 := 0;
            dev.m_f2 := 0;
            dev.m_f3 := 0;
            dev.m_gds := 0;
            dev.m_ggd := 0;
            dev.m_ggs := 0;
            dev.m_gm := 0;
            dev.m_tCGD := 0;
            dev.m_tCGS := 0;
            dev.m_tGatePot := 0;
            dev.m_tSatCur := 0;
            dev.m_vcrit := 0;
            dev.m_vds := 0;
            dev.m_vgd := 0;
            dev.m_vgs := 0;
            dev.m_bICVDSIsGiven := if IC_VDS > (-1e40) then 1 else 0;
            dev.m_dICVDS := if IC_VDS > (-1e40) then IC_VDS else 0;
            dev.m_bICVGSIsGiven := if IC_VGS > (-1e40) then 1 else 0;
            dev.m_dICVGS := if IC_VGS > (-1e40) then IC_VGS else 0;
            dev.m_off := OFF;
// non-zero to indicate device is off for dc analysis
            dev.m_uic := UIC;
            dev.m_dTemp := TEMP + Modelica.Electrical.Spice3.Internal.SpiceConstants.CONSTCtoK;
            dev.m_area := AREA;
// Number of parallel connected identical elements
          end fetRenameParametersDev;
        end Fet;

        package Jfet "Records and functions for Jfet"
          extends Modelica.Icons.InternalPackage;

          record JfetModelLine "Record for Jfet model line parameters"
            extends Modelica.Electrical.Spice3.Internal.Fet.FetModelLine;
            SI.InversePotential m_bFac;
          end JfetModelLine;

          function jfetInitEquations "FET initial precalculations"
            extends Modelica.Icons.Function;
            input Modelica.Electrical.Spice3.Internal.Fet.Fet in_f;
            input JfetModelLine in_fm;
            output JfetModelLine out_fm;
          algorithm
            out_fm := in_fm;
            out_fm.m_beta := in_fm.m_beta * in_f.m_area;
            out_fm.m_drainConduct := in_fm.m_drainConduct * in_f.m_area;
            out_fm.m_sourceConduct := in_fm.m_sourceConduct * in_f.m_area;
            out_fm.m_gateSatCurrent := in_fm.m_gateSatCurrent * in_f.m_area;
          end jfetInitEquations;

          function jfetModelLineInitEquations "Initial precalculation"
            extends Modelica.Icons.Function;
            input JfetModelLine in_fm;
            output JfetModelLine out_fm;
          algorithm
            out_fm := in_fm;
//  Fet_Model_Line::InitEquations();
//  algorithm aus FET.fetModelLineInitEquations kopiert, statt Funktionsaufruf
            if in_fm.m_drainResist <> 0 then
              out_fm.m_drainConduct := 1 / in_fm.m_drainResist;
            end if;
            if in_fm.m_sourceResist <> 0 then
              out_fm.m_sourceConduct := 1 / in_fm.m_sourceResist;
            end if;
            if out_fm.m_depletionCapCoeff > 0.95 then
              out_fm.m_depletionCapCoeff := 0.95;
            end if;
            out_fm.m_bFac := (1 - out_fm.m_b) / (out_fm.m_gatePotential - out_fm.m_threshold);
          end jfetModelLineInitEquations;

          function drainCur "Drain current calculation"
            extends Modelica.Icons.Function;
            input SI.Voltage vds;
            input SI.Voltage vgs;
            input SI.Voltage vgd;
            input Modelica.Electrical.Spice3.Internal.Fet.Fet in_f;
            input JfetModelLine in_fm;
            output Modelica.Electrical.Spice3.Internal.Fet.Fet out_f;
          protected
            SI.Voltage vto;
            SI.Voltage vgst;
            Real betap;
            //  SI.InversePotential Bfac;
            Real Bfac;
            Real apart;
            Real cpart;
            SI.Voltage vgdt;
          algorithm
            out_f := in_f;
// Modification for Sydney University JFET model
            vto := in_fm.m_threshold;
            if vds >= 0 then
              vgst := vgs - vto;
              if vgst <= 0 then
                out_f.m_cdrain := 0;
                out_f.m_gm := 0;
                out_f.m_gds := 0;
              else
                betap := in_fm.m_beta * (1 + in_fm.m_lModulation * vds);
                Bfac := in_fm.m_bFac;
                if vgst >= vds then
                  apart := 2 * in_fm.m_b + 3 * Bfac * (vgst - vds);
                  cpart := vds * (vds * (Bfac * vds - in_fm.m_b) + vgst * apart);
                  out_f.m_cdrain := betap * cpart;
                  out_f.m_gm := betap * vds * (apart + 3 * Bfac * vgst);
                  out_f.m_gds := betap * (vgst - vds) * apart + in_fm.m_beta * in_fm.m_lModulation * cpart;
                else
                  Bfac := vgst * Bfac;
                  out_f.m_gm := betap * vgst * (2 * in_fm.m_b + 3 * Bfac);
                  cpart := vgst * vgst * (in_fm.m_b + Bfac);
                  out_f.m_cdrain := betap * cpart;
                  out_f.m_gds := in_fm.m_lModulation * in_fm.m_beta * cpart;
                end if;
              end if;
            else
              vgdt := vgd - vto;
              if vgdt <= 0 then
                out_f.m_cdrain := 0;
                out_f.m_gm := 0;
                out_f.m_gds := 0;
              else
                betap := in_fm.m_beta * (1 - in_fm.m_lModulation * vds);
                Bfac := in_fm.m_bFac;
                if vgdt + vds >= 0 then
                  apart := 2 * in_fm.m_b + 3 * Bfac * (vgdt + vds);
                  cpart := vds * ((-vds * ((-Bfac * vds) - in_fm.m_b)) + vgdt * apart);
                  out_f.m_cdrain := betap * cpart;
                  out_f.m_gm := betap * vds * (apart + 3 * Bfac * vgdt);
                  out_f.m_gds := betap * (vgdt + vds) * apart - in_fm.m_beta * in_fm.m_lModulation * cpart - out_f.m_gm;
                else
                  Bfac := vgdt * Bfac;
                  out_f.m_gm := -betap * vgdt * (2 * in_fm.m_b + 3 * Bfac);
                  cpart := vgdt * vgdt * (in_fm.m_b + Bfac);
                  out_f.m_cdrain := -betap * cpart;
                  out_f.m_gds := in_fm.m_lModulation * in_fm.m_beta * cpart - out_f.m_gm;
                end if;
              end if;
            end if;
// compute drain current and derivatives for normal mode
// normal mode, cutoff region
// normal mode, linear region
// normal mode, saturation region
// compute drain current and derivatives for inverse mode
// inverse mode, cutoff region
// inverse mode, linear region
// inverse mode, saturation region
          end drainCur;

          function calculateGateCap "Gate capactance calculation"
            extends Modelica.Icons.Function;
            input Modelica.Electrical.Spice3.Internal.Fet.Fet in_f;
            input SI.Voltage vgs "Input voltage gate source";
            input SI.Voltage vgd "Input voltage gate drain";
            output SI.Capacitance capgs "Output capacitance gate source";
            output SI.Charge chargegs "Output charge gate source";
            output SI.Capacitance capgd "Output capacitance gate drain";
            output SI.Charge chargegd "Output charge gate drain";
          algorithm
            (capgs, chargegs) := Modelica.Electrical.Spice3.Internal.Functions.junctionCap(in_f.m_tCGS, vgs, in_f.m_corDepCap, 0.5, in_f.m_tGatePot, in_f.m_f1, in_f.m_f2, in_f.m_f3);
            (capgd, chargegd) := Modelica.Electrical.Spice3.Internal.Functions.junctionCap(in_f.m_tCGD, vgd, in_f.m_corDepCap, 0.5, in_f.m_tGatePot, in_f.m_f1, in_f.m_f2, in_f.m_f3);
          end calculateGateCap;

          function jfetCalcTempDependencies "Precalculations relating to temperature"
            extends Modelica.Icons.Function;
            input Modelica.Electrical.Spice3.Internal.Fet.Fet in_f;
            input JfetModelLine in_fm;
            output Modelica.Electrical.Spice3.Internal.Fet.Fet out_f;
          algorithm
            out_f := in_f;
            (out_f.m_tGatePot, out_f.m_tCGS) := Modelica.Electrical.Spice3.Internal.Functions.junctionParamDepTempSPICE3(in_fm.m_gatePotential, in_fm.m_capGS, 0.5, out_f.m_dTemp, in_fm.m_tnom);
            out_f.m_tCGS := out_f.m_area * out_f.m_tCGS;
            (out_f.m_tGatePot, out_f.m_tCGD) := Modelica.Electrical.Spice3.Internal.Functions.junctionParamDepTempSPICE3(in_fm.m_gatePotential, in_fm.m_capGD, 0.5, out_f.m_dTemp, in_fm.m_tnom);
            out_f.m_tCGD := out_f.m_area * out_f.m_tCGD;
            (out_f.m_f1, out_f.m_f2, out_f.m_f3) := Modelica.Electrical.Spice3.Internal.Functions.junctionCapCoeffs(0.5, in_fm.m_depletionCapCoeff, out_f.m_tGatePot);
            out_f.m_tSatCur := Modelica.Electrical.Spice3.Internal.Functions.saturationCurDepTempSPICE3JFET(in_fm.m_gateSatCurrent, out_f.m_dTemp, in_fm.m_tnom);
            out_f.m_vcrit := Modelica.Electrical.Spice3.Internal.Functions.junctionVCrit(out_f.m_dTemp, 1.0, out_f.m_tSatCur);
            out_f.m_dVt := out_f.m_dTemp * Modelica.Electrical.Spice3.Internal.SpiceConstants.CONSTKoverQ;
            out_f.m_corDepCap := in_fm.m_depletionCapCoeff * out_f.m_tGatePot;
          end jfetCalcTempDependencies;

          function jfetNoBypassCode "Calculations of currents and capacities"
            extends Modelica.Icons.Function;
            input Modelica.Electrical.Spice3.Internal.Fet.Fet in_f "Input record fet parameters";
            input JfetModelLine in_fm "Input record model line parameters";
            input Integer in_m_type "Type of MOS transistor";
            input Boolean in_m_bInit;
            input SI.Voltage[3] in_m_pVoltageValues;
            // gate drain source
            output Modelica.Electrical.Spice3.Internal.Fet.CurrrentsCapacitances out_cc "Calculated currents and capacitances";
          protected
            Modelica.Electrical.Spice3.Internal.Fet.Fet int_f "Record Fet";
          algorithm
            int_f := in_f;
            int_f.m_vgd := in_m_type * (in_m_pVoltageValues[1] - in_m_pVoltageValues[2]);
// ( G,  DP);
            int_f.m_vgs := in_m_type * (in_m_pVoltageValues[1] - in_m_pVoltageValues[3]);
// ( G,  SP);
            if in_f.m_uic and int_f.m_bICVGSIsGiven > 0.5 then
              int_f.m_vgs := in_m_type * int_f.m_dICVGS;
            elseif Modelica.Electrical.Spice3.Internal.SpiceRoot.initJunctionVoltages() then
              if int_f.m_off == true then
                int_f.m_vgs := 0;
              else
                int_f.m_vgs := int_f.m_vcrit;
              end if;
            end if;
            if in_f.m_uic and int_f.m_bICVDSIsGiven > 0.5 then
              int_f.m_vgd := in_m_type * int_f.m_dICVDS + int_f.m_vgs;
            elseif Modelica.Electrical.Spice3.Internal.SpiceRoot.initJunctionVoltages() then
              if int_f.m_off == true then
                int_f.m_vgd := 0;
              else
                int_f.m_vgd := int_f.m_vcrit;
              end if;
            end if;
            int_f.m_vgs := Modelica.Electrical.Spice3.Internal.SpiceRoot.limitJunctionVoltage(int_f.m_vgs);
            int_f.m_vgd := Modelica.Electrical.Spice3.Internal.SpiceRoot.limitJunctionVoltage(int_f.m_vgd);
            int_f.m_vds := int_f.m_vgs - int_f.m_vgd;
//////////////////////////////////////////////////////////////////////
// junction currents
            (int_f.m_cgd, int_f.m_ggd) := Modelica.Electrical.Spice3.Internal.Functions.junction2(int_f.m_vgd, int_f.m_dTemp, 1.0, int_f.m_tSatCur);
            out_cc.iGD := in_m_type * int_f.m_cgd;
            (int_f.m_cgs, int_f.m_ggs) := Modelica.Electrical.Spice3.Internal.Functions.junction2(int_f.m_vgs, int_f.m_dTemp, 1.0, int_f.m_tSatCur);
            out_cc.iGS := in_m_type * int_f.m_cgs;
//////////////////////////////////////////////////////////////////////
// channel current calculation
// Let the actual instance calculate the values
            int_f := drainCur(int_f.m_vds, int_f.m_vgs, int_f.m_vgd, int_f, in_fm);
            out_cc.idrain := in_m_type * int_f.m_cdrain;
//////////////////////////////////////////////////////////////////////
// channel charge calculation
            if not in_m_bInit then
              (int_f.m_capgs, int_f.m_chargegs, int_f.m_capgd, int_f.m_chargegd) := calculateGateCap(int_f, int_f.m_vgs, int_f.m_vgd);
              out_cc.cGD := if in_m_bInit then -1e40 else int_f.m_capgd;
              out_cc.cGS := if in_m_bInit then -1e40 else int_f.m_capgs;
            end if;
          end jfetNoBypassCode;

          function jfetRenameParameters "Parameter renaming to internal names"
            extends Modelica.Icons.Function;
            input Modelica.Electrical.Spice3.Internal.ModelcardJFET ex "Modelcard with technologieparameters";
            output JfetModelLine intern "Output record model line parameters";
          algorithm
            intern.m_drainConduct := 0;
            intern.m_sourceConduct := 0;
            intern.m_bFac := 0;
            intern.m_capGS := if ex.CGS > (-1e40) then ex.CGS else 0;
// Zero-bias G-S junction capacitance, default 0
            intern.m_capGD := if ex.CGD > (-1e40) then ex.CGD else 0;
// Zero-bias G-D junction capacitance, default 0
            intern.m_gateSatCurrent := ex.IS;
// Saturation current of pn junctions
            intern.m_depletionCapCoeff := ex.FC;
// Coefficient for forward-bias depletion capacitance formula
            intern.m_drainResist := if ex.RD > (-1e40) then ex.RD else 0;
// Drain ohmic resistance, default 0
            intern.m_sourceResist := if ex.RS > (-1e40) then ex.RS else 0;
// Source ohmic resistance, default 0
            intern.m_tnom := ex.TNOM + Modelica.Electrical.Spice3.Internal.SpiceConstants.CONSTCtoK;
// Parameter measurement temperature, default 27
            intern.m_threshold := if ex.VTO > (-1e40) then ex.VTO else -2.0;
// Zero-bias threshold voltage, default -2
            intern.m_b := if ex.B > (-1e40) then ex.B else 1.0;
// Dotierungsverlauf parameter, default 1
            intern.m_beta := if ex.BETA > (-1e40) then ex.BETA else 1e-4;
// Output admittance parameter, default 1e4
            intern.m_lModulation := ex.LAMBDA;
// Channel-length modulation, default 0
            intern.m_gatePotential := ex.PB;
// Junction potential of pn junctions
            intern.m_fNexp := ex.AF;
// Flicker noise exponent
            intern.m_fNcoef := ex.KF;
// Flicker noise coefficient
            annotation(
              Documentation(info = "<html>
<p>This function jfetRenameParameters assigns the external (given by the user, e.g. RD) technology parameters
to the internal parameters (e.g. m_drainResistance). It also does the analysis of the IsGiven values.</p>
</html>"));
          end jfetRenameParameters;
        end Jfet;

        package Csemiconductor
          extends Modelica.Icons.InternalPackage;

          record Capacitor
            extends Modelica.Electrical.Spice3.Internal.Model.Model;
            SI.Capacitance m_dCapac(start = 1e-9) "Device is a Capacitor model";
            Real m_dCapIsGiven "Capacitor is given value";
            SI.Length m_dWidth(start = 0) "Width";
            SI.Length m_dLength(start = 0) "Length";
            Boolean m_bSensCapac(start = false) "flag to request sensitivity WRT Capacitor";
          end Capacitor;

          record CapacitorModelLineParams "Record for Capacitor model line parameters"
            extends Modelica.Icons.Record;
            SI.CapacitancePerArea m_dCj "Junction bottom capacitance";
            SI.Permittivity m_dCjsw "Junction sidewall capacitance";
            SI.Length m_dDefW "Default device width";
            SI.Length m_dNarrow "Narrowing due to side etching";
          end CapacitorModelLineParams;

          function capacitorInitEquations
            extends Modelica.Icons.Function;
            input Capacitor in_p "Input record with capacitor parameters";
            input CapacitorModelLineParams in_p2 "Input record with capacitor model line parameters";
            output Capacitor out "Output record with capacitor variables";
          algorithm
            out := in_p;
            if in_p.m_dCapIsGiven < 0.5 then
              if abs(in_p.m_dLength) > 1e-18 and abs(in_p2.m_dCj) > 1e-25 then
                out.m_dCapac := Modelica.Electrical.Spice3.Internal.Functions.capDepGeom(in_p2.m_dCj, in_p2.m_dCjsw, out.m_dWidth, in_p.m_dLength, in_p2.m_dNarrow);
              end if;
            end if;
          end capacitorInitEquations;

          function capacitorRenameParameters
            extends Modelica.Icons.Function;
            input Modelica.Electrical.Spice3.Internal.ModelcardC ex "Modelcard with technology parameters";
            output CapacitorModelLineParams intern "Output record with capacitor model line parameters";
          algorithm
            intern.m_dCj := ex.CJ;
            intern.m_dCjsw := ex.CJSW;
            intern.m_dDefW := ex.DEFW;
            intern.m_dNarrow := ex.NARROW;
          end capacitorRenameParameters;

          function capacitorRenameParametersDev
            extends Modelica.Icons.Function;
            input SI.Capacitance C "Capacitance";
            input SI.Length W "Width";
            input SI.Length L "Length";
            input SI.Temp_C TEMP "Temperature";
            input Boolean SENS_AREA "Parameter for sensitivity analyses, not implemented yet";
            input CapacitorModelLineParams p;
            output Capacitor intern "Output record with capacitor parameters";
          algorithm
            intern.m_dCapIsGiven := if C > (-1e40) then 1 else 0;
            intern.m_dCapac := if C > (-1e40) then C else 1e-9;
            intern.m_dWidth := if W > (-1e40) then W else p.m_dDefW;
            intern.m_dLength := L;
            intern.m_bSensCapac := SENS_AREA;
            intern.m_dTemp := TEMP + Modelica.Electrical.Spice3.Internal.SpiceConstants.CONSTCtoK;
          end capacitorRenameParametersDev;
        end Csemiconductor;
      end Internal;
      annotation(
        Documentation(info = "<html>
<p>This package contains all function, parameters and data of semiconductor models, that are transformed from SPICE3 into Modelica. The models of the package semiconductors access to repository models. This package should not be used via direct access by a user of the Spice-Library for Modelica. It is restricted to the development.</p>
</html>"),
        Icon(graphics = {Line(points = {{-20, 40}, {-20, -40}}, color = {0, 0, 0}), Line(points = {{-90, 0}, {-20, 0}}, color = {0, 0, 0}), Line(points = {{0, 0}, {90, 0}}, color = {0, 0, 0}), Line(points = {{20, 90}, {20, 40}, {0, 40}, {0, -40}, {20, -40}, {20, -90}}, color = {0, 0, 0})}));
    end Spice3;
    annotation(
      Documentation(info = "<html>
<p>
This library contains electrical components to build up analog and digital circuits,
as well as machines to model electrical motors and generators,
especially three phase induction machines such as an asynchronous motor.
</p>

</html>"),
      Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100.0, -100.0}, {100.0, 100.0}}), graphics = {Rectangle(origin = {20.3125, 82.8571}, extent = {{-45.3125, -57.8571}, {4.6875, -27.8571}}), Line(origin = {8.0, 48.0}, points = {{32.0, -58.0}, {72.0, -58.0}}), Line(origin = {9.0, 54.0}, points = {{31.0, -49.0}, {71.0, -49.0}}), Line(origin = {-2.0, 55.0}, points = {{-83.0, -50.0}, {-33.0, -50.0}}), Line(origin = {-3.0, 45.0}, points = {{-72.0, -55.0}, {-42.0, -55.0}}), Line(origin = {1.0, 50.0}, points = {{-61.0, -45.0}, {-61.0, -10.0}, {-26.0, -10.0}}), Line(origin = {7.0, 50.0}, points = {{18.0, -10.0}, {53.0, -10.0}, {53.0, -45.0}}), Line(origin = {6.2593, 48.0}, points = {{53.7407, -58.0}, {53.7407, -93.0}, {-66.2593, -93.0}, {-66.2593, -58.0}})}));
  end Electrical;

  package Magnetic "Library of magnetic models"
    extends Modelica.Icons.Package;

    package FluxTubes "Library for modelling of electromagnetic devices with lumped magnetic networks"
      import SI = Modelica.SIunits;
      import Modelica.Constants.pi;
      import mu_0 = Modelica.Constants.mue_0;
      extends Modelica.Icons.Package;

      package UsersGuide "User's Guide"
        extends Modelica.Icons.Information;

        class FluxTubeConcept "Flux tube concept"
          extends Modelica.Icons.Information;
          annotation(
            Documentation(info = "<html>
<h4>Overview of the Concept of Magnetic Flux Tubes</h4>
<p>
Following below, the concept of magnetic flux tubes is outlined in short. For a detailed description of flux tube elements, please have a look at the listed literature. Magnetic flux tubes enable for modeling of magnetic fields with lumped networks. The figure below and the following equations illustrate the transition from the original magnetic field quantities described by <i>Maxwell</i>'s equations to network elements with a flow variable and an across variable:
</p>

<p>
<IMG src=\"modelica://Modelica/Resources/Images/Magnetic/FluxTubes/UsersGuide/FluxTubeConcept/magnetic_flux_tube_schematic.png\" ALT=\"Magnetic flux tube\">
</p>

<p>
For a region with an approximately homogeneous distribution of the magnetic field strength <b>H</b> and the magnetic flux density <b>B</b> through cross sectional area <i>A</i> at each length coordinate <i>s</i>  (<i>A</i> perpendicular to the direction of the magnetic field lines), a magnetic reluctance <i>R<sub>m</sub></i> can be defined:</p>

<p>
<IMG src=\"modelica://Modelica/Resources/Images/Magnetic/FluxTubes/UsersGuide/FluxTubeConcept/eq_transition_reluctance_flowAcross_IntegralQuantities.png\" ALT=\"Transition from field quantities to flow- and across variables\">
</p>

<p>
With the definition of the magnetic potential difference <i>V<sub>m</sub></i> as an across variable and the magnetic flux <i>&Phi;</i> as flow variable, a reluctance element <i>R<sub>m</sub></i> can be defined similar to resistive network elements in other physical domains. Using <i>Maxwell</i>'s constitutive equation
</p>

<p>
<IMG src=\"modelica://Modelica/Resources/Images/Magnetic/FluxTubes/UsersGuide/FluxTubeConcept/eq_MaxwellConstitutive.png\" ALT=\"Maxwell's constitutive equation\">
</p>

<p>the general formula for the calculation of a magnetic reluctance <i>R<sub>m</sub></i> from its geometric and material properties is:</p>

<p>
<IMG src=\"modelica://Modelica/Resources/Images/Magnetic/FluxTubes/UsersGuide/FluxTubeConcept/eq_reluctance_general.png\" ALT=\"General formula for calculation of a magnetic reluctance\">
</p>

<p>
For a prismatic or cylindrical volume of length <i>l</i> and cross sectional area <i>A</i> with the magnetic flux entering and leaving the region through its end planes, the above equation simplifies to:</p>

<p>
<IMG src=\"modelica://Modelica/Resources/Images/Magnetic/FluxTubes/UsersGuide/FluxTubeConcept/eq_reluctance_prismatic.png\" ALT=\"Magnetic reluctance of a prismatic or cylindrical volume\">
</p>

<p>
Similar equations can be derived for other geometries. In cases where a direct integration is not possible, the reluctance can be calculated on base of average length, average cross sectional area and volume <i>V</i> respectively:
</p>

<p>
<IMG src=\"modelica://Modelica/Resources/Images/Magnetic/FluxTubes/UsersGuide/FluxTubeConcept/eq_reluctanceFromAverageGeometry.png\" ALT=\"Reluctance calculation from average geometric quantities\">
</p>

<p>
Network elements for sources of a magnetic potential difference or magnetomotive force, i.e., coils or permanent magnets can be formulated as well. The resulting magnetic network models of actuators reflect the main dimensions of these devices as well as the normally nonlinear characteristics of their magnetically active materials.
</p>

</html>"));
        end FluxTubeConcept;

        class ReluctanceForceCalculation "Reluctance forces"
          extends Modelica.Icons.Information;
          annotation(
            Documentation(info = "<html>
<h4>Calculation of reluctance forces from lumped magnetic network models</h4>

<p>
Generally, the thrust <i>F</i> developed by a translatory electro-magneto-mechanical actuator (similar for the rotational case with torque and angular position) is equal to the change of magnetic co-energy <i>W<sub>m</sub><sup>*</sup></i> with armature position <i>x</i> according to
</p>

<p>
<IMG src=\"modelica://Modelica/Resources/Images/Magnetic/FluxTubes/UsersGuide/ReluctanceForceCalculation/eq_CoEnergy_general.png\" ALT=\"Equation for force calculation from change of magnetic co-energy with armature position\">
</p>

<p>
(<i>&Psi;</i> flux linkage, <i>i</i> actuator current). In lumped magnetic network models, the above equation simplifies to
</p>

<p>
<IMG src=\"modelica://Modelica/Resources/Images/Magnetic/FluxTubes/UsersGuide/ReluctanceForceCalculation/eq_forceFromPermeance_network.png\" ALT=\"Equation for force calculation in lumped magnetic network models\">
</p>

<p>
where <i>n<sub>linear</sub></i> is the number of flux tube elements with constant relative permeability that change its permeance <i>G<sub>m i</sub></i> with armature position (index <i>i</i>), <i>V<sub>m i</sub></i> the magnetic voltage across each respective flux tube and <i>dG<sub>m i</sub>/dx</i> the derivative of the respective permeances with respect to armature position. Transition from the general formula based on magnetic co-energy to the latter one is outlined in <a href=\"modelica://Modelica.Magnetic.FluxTubes.UsersGuide.Literature\">[Ka08]</a> for the reciprocal of the permeance, i.e., for the magnetic reluctance <i>R<sub>m</sub></i>. Note that
</p>

<p>
<IMG src=\"modelica://Modelica/Resources/Images/Magnetic/FluxTubes/UsersGuide/ReluctanceForceCalculation/eq_transition_forceReluctancePermeance.png\" ALT=\"Transition from force calculation based on reluctance to calculation based on permeance\">
</p>

<p>with <i>&Phi;<sub>i</sub></i> being the magnetic flux through each respective flux tube element.</p>

<p>
Flux tube elements with <i>non-linear</i> material characteristics <i>&mu;<sub>r</sub></i>(<i>B</i>) in magnetic network models do not restrict the usability of the above equation. However, it is required that these nonlinear flux tube elements do not change its shape with armature motion (e.g., portion of a solenoid plunger where the magnetic flux passes through in axial direction). This limitation is not a strong one, since the permeance of nonlinear, but highly permeable ferromagnetic flux tube elements and its change with armature position compared to that of air gap flux tubes can be neglected in most cases. Because of this constraint, the dimensions of possibly nonlinear flux tube elements in sub-package <a href=\"modelica://Modelica.Magnetic.FluxTubes.Shapes.FixedShape\">Shapes.FixedShape</a> are fixed, whereas the dimension in direction of motion of the linear flux tube elements in sub-package <a href=\"modelica://Modelica.Magnetic.FluxTubes.Shapes.Force\">Shapes.Force</a> can vary during simulation. For the flux tubes defined in this package with their rather simple shapes, the derivative <i>dG<sub>m</sub>/dx</i> is given analytically. For more complex shapes and variations of dimensions with armature motion, it must be provided analytically during model development, preferably by extending the partial model <a href=\"modelica://Modelica.Magnetic.FluxTubes.Interfaces.PartialForce\">Interfaces.PartialForce</a>.
</p>

<p>
The sub-package <a href=\"modelica://Modelica.Magnetic.FluxTubes.Shapes.Leakage\">Shapes.Leakage</a> contains flux tube shapes typical for leakage flux around prismatic or cylindrical poles. Since the permeance of these flux tubes does not change with armature position, they do not contribute to a reluctance actuator's thrust.
</p>

</html>"));
        end ReluctanceForceCalculation;

        class Literature "Literature"
          extends Modelica.Icons.References;
          annotation(
            Documentation(info = "<html>
<h4>Literature</h4>
<ul>
<li> Usage of the Modelica Magnetic library is described in:
<dl>
<dt>[B&ouml;08] B&ouml;drich, T.:</dt>
<dd> <b>Electromagnetic Actuator Modelling with the Extended Modelica Magnetic Library</b>.
     Modelica 2008 Conference, Bielefeld, Germany,
     pp. 221-227, March 3-4, 2008.
     Download from:
     <a href=\"https://www.modelica.org/events/modelica2008/Proceedings/sessions/session2d2.pdf\">https://www.modelica.org/events/modelica2008/Proceedings/sessions/session2d2.pdf</a>
     </dd>
</dl>
</li>
<li> The method of magnetic flux tubes as well as derivation of the permeance of many flux tube shapes is explained in detail in:
<dl>
<dt>[Ro41] Roters, H.:</dt>
<dd> <b>Electromagnetic Devices</b>.
New York: John Wiley &amp; Sons 1941 (8th Printing 1961)
</dd>
</dl>
</li>
<li> Structure, properties, applications and design of electromagnetic (reluctance type) actuators are thoroughly described in:
<dl>
<dt>[Ka08] Kallenbach, E.; Eick, R.; Quendt, P.; Str&ouml;hla, T.; Feindt, K.; Kallenbach, M.:</dt>
<dd><b>Elektromagnete: Grundlagen, Berechnung, Entwurf und Anwendung</b>.
3rd ed., Wiesbaden: Vieweg Teubner 2008 (in German)
<br>&nbsp;</dd>
<dt>[Ro00] Roschke, T.:</dt>
<dd><b>Entwurf geregelter elektromagnetischer Antriebe f&uuml;r Luftsch&uuml;tze</b>.
    Fortschritt-Berichte VDI, Reihe 21, Nr. 293, D&uuml;sseldorf: VDI-Verlag 2000 (in German)</dd>
</dl>
</li>
<li> Application of the method of magnetic flux tubes to the design of rotational electrical machines is explained for example in:
<dl>
<dt>[HM94] Hendershot, J.R. Jr.; Miller, T.J.E.:</dt>
<dd> <b>Design of Brushless Permanent-Magnet Motors</b>.
Magna Physics Publishing and Oxford University Press 1994
</dd>
</dl>
</li>

</ul>

</html>"));
        end Literature;

        class Contact "Contact"
          extends Modelica.Icons.Contact;
          annotation(
            Documentation(info = "<html>
<h4>Contact</h4>

<dl>
<dt><b>Main Author:</b></dt>
<dd>
    <a href=\"http://www.ifte.de/mitarbeiter/boedrich.html\">Thomas B&ouml;drich</a><br>
    Dresden University of Technology<br>
    Institute of Electromechanical and Electronic Design<br>
    01062 Dresden, Germany<br>
    Phone: +49 - 351 - 463 36296<br>
    Fax: +49 - 351 - 463 37183<br>
    email: <A HREF=\"mailto:Thomas.Boedrich@tu-dresden.de\">Thomas.Boedrich@tu-dresden.de</A><br></dd>
</dl>

<p><b>Acknowledgements:</b></p>
<ul>
<li> The magnetisation characteristics of the included soft magnetic materials were compiled and measured respectively by Thomas Roschke, now with Johnson Electric. Provision of this data is highly appreciated. He also formulated the approximation function used for description of the magnetisation characteristics of these materials.
</li>
<li> Andr&eacute; Klick of then Dresden University of Technology, Dresden, Germany gave valuable support on the implementation of this library. His contribution is highly appreciated, too.
</li>
</ul>
</html>"));
        end Contact;
        annotation(
          DocumentationClass = true,
          Documentation(info = "<html>
<p>
This library contains components for modelling of electromagnetic devices with lumped magnetic networks. Those models are suited for both rough design of the magnetic subsystem of a device as well as for efficient dynamic simulation at system level together with neighbouring subsystems. At present, components and examples for modelling of <i>translatory</i> electromagnetic and electrodynamic actuators are provided. If needed, these components can be adapted to network modelling of <i>rotational</i> electrical machines.
</p>
<p>
This user's guide gives a short introduction to the underlying concept of <b>magnetic flux tubes</b>, summarizes the calculation of magnetic <b>reluctance forces</b> from lumped magnetic network models and lists <b>reference literature</b>.
</p>
<p>
<a href=\"modelica://Modelica.Magnetic.FluxTubes.Examples\">Examples</a> illustrates the usage of magnetic network models with simple models from different fields of application.</p>
</html>"));
      end UsersGuide;

      package Examples "Illustration of component usage with simple models of various devices"
        extends Modelica.Icons.ExamplesPackage;

        model SaturatedInductor "Inductor with saturation in the ferromagnetic core"
          extends Modelica.Icons.Example;
          Modelica.Magnetic.FluxTubes.Basic.Ground ground_m annotation(
            Placement(transformation(extent = {{50, -30}, {70, -10}}, rotation = 0)));
          Modelica.Electrical.Analog.Sources.SineVoltage source(freqHz = 50, phase = pi / 2, V = 230 * sqrt(2)) "Voltage applied to inductor" annotation(
            Placement(transformation(origin = {-80, 10}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          Modelica.Electrical.Analog.Basic.Resistor r(R = 7.5) "Inductor coil resistance" annotation(
            Placement(transformation(extent = {{-61, 10}, {-41, 30}}, rotation = 0)));
          Modelica.Magnetic.FluxTubes.Basic.ElectroMagneticConverter coil(N = 600, i(fixed = true)) "Inductor coil" annotation(
            Placement(transformation(extent = {{-30, 0}, {-10, 20}}, rotation = 0)));
          Basic.ConstantReluctance r_mLeak(R_m = 1.2e6) "Constant leakage reluctance" annotation(
            Placement(transformation(origin = {10, 10}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          Modelica.Magnetic.FluxTubes.Shapes.FixedShape.Cuboid r_mAirPar(a = 0.025, b = 0.025, nonLinearPermeability = false, mu_rConst = 1, l = 0.0001) "Reluctance of small parasitic air gap (ferromagnetic core packeted from single sheets)" annotation(
            Placement(transformation(extent = {{26, 10}, {46, 30}}, rotation = 0)));
          Modelica.Magnetic.FluxTubes.Shapes.FixedShape.Cuboid r_mFe(mu_rConst = 1000, a = 0.025, b = 0.025, nonLinearPermeability = true, l = 4 * 0.065, material = Modelica.Magnetic.FluxTubes.Material.SoftMagnetic.ElectricSheet.M350_50A(), B(start = 0)) "Reluctance of ferromagnetic inductor core" annotation(
            Placement(transformation(origin = {60, 10}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          Modelica.Electrical.Analog.Basic.Ground ground annotation(
            Placement(transformation(extent = {{-90, -30}, {-70, -10}}, rotation = 0)));
        equation
          connect(source.p, r.p) annotation(
            Line(points = {{-80, 20}, {-61, 20}}, color = {0, 0, 255}));
          connect(r.n, coil.p) annotation(
            Line(points = {{-41, 20}, {-30, 20}, {-30, 16}}, color = {0, 0, 255}));
          connect(source.n, coil.n) annotation(
            Line(points = {{-80, 0}, {-30, 0}, {-30, 4}}, color = {0, 0, 255}));
          connect(coil.port_p, r_mLeak.port_p) annotation(
            Line(points = {{-10, 16}, {-10, 20}, {10, 20}}, color = {255, 127, 0}));
          connect(r_mLeak.port_p, r_mAirPar.port_p) annotation(
            Line(points = {{10, 20}, {26, 20}}, color = {255, 127, 0}));
          connect(r_mAirPar.port_n, r_mFe.port_p) annotation(
            Line(points = {{46, 20}, {54, 20}, {60, 20}}, color = {255, 127, 0}));
          connect(r_mFe.port_n, r_mLeak.port_n) annotation(
            Line(points = {{60, 0}, {47.5, 0}, {35, 0}, {10, 0}}, color = {255, 127, 0}));
          connect(r_mFe.port_n, coil.port_n) annotation(
            Line(points = {{60, 0}, {-10, 0}, {-10, 4}}, color = {255, 127, 0}));
          connect(ground.p, source.n) annotation(
            Line(points = {{-80, -10}, {-80, 0}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(ground_m.port, r_mFe.port_n) annotation(
            Line(points = {{60, -10}, {60, 0}}, color = {255, 127, 0}, smooth = Smooth.None));
          annotation(
            experiment(StopTime = 0.1, Tolerance = 1e-007),
            Documentation(info = "<html>
<p>
This model demonstrates the effects of non-linear magnetisation characteristics of soft magnetic materials (hysteresis neglected). A sinusoidal voltage is applied to an inductor with a closed ferromagnetic core of rectangular shape. Set the <b>tolerance</b> to <b>1e-7</b>, <b>simulate for 0.1 s</b> and plot for example:
</p>

<pre>
    coil.i vs. time           // non-harmonic current due to saturation of the core material
    r_mFe.mu_r vs. r_mFe.B    // relative permeability vs. flux density inside core
    r_mFe.B vs. r_mFe.H       // magnetisation curve B(H); hysteresis neglected
</pre>

<p>
The magnetisation characteristics of the flux tube element representing the ferromagnetic core can easily be changed from simplified linear behaviour (nonLinearPermeability set to false and R_mFe.mu_rConst set to a positive value, preferably mu_rConst >> 1) to non-linear behaviour (e.g., selection of one of the electric sheets in <a href=\"modelica://Modelica.Magnetic.FluxTubes.Material.SoftMagnetic\">Material.SoftMagnetic</a> with nonLinearPermeability set to true). This enables for convenient inital design of magnetic circuits with linear material characteristics prior to simulation with non-linear behaviour.
</p>

<h4>Note</h4>

<p>
If the supply voltage has a zero-crossing when applied to the inductor at time t=0 (i.e., source.phase set to zero instead of &pi;/2), then the inrush current that is typical for switching of inductive loads can be observed.
</p>
</html>"));
        end SaturatedInductor;

        package MovingCoilActuator "Two translatory electrodynamic actuator models of different modelling depth and their comparison"
          extends Modelica.Icons.ExamplesPackage;

          model ForceCurrentBehaviour "Comparison of the force-current characteristics of both converter models with armature blocked at mid-position"
            extends Modelica.Icons.Example;
            Modelica.Electrical.Analog.Basic.Ground pmGround annotation(
              Placement(transformation(extent = {{-70, -100}, {-50, -80}}, rotation = 0)));
            Modelica.Magnetic.FluxTubes.Examples.MovingCoilActuator.Components.PermeanceActuator pmActuator(x(start = 0), material = Modelica.Magnetic.FluxTubes.Material.HardMagnetic.PlasticNdFeB()) "Moving coil actuator described with permeance model" annotation(
              Placement(transformation(extent = {{-20, -70}, {0, -50}}, rotation = 0)));
            Modelica.Mechanics.Translational.Components.Fixed pmFixedPos(s0 = 0) "Fixed armature position" annotation(
              Placement(transformation(extent = {{10, -70}, {30, -50}}, rotation = 0)));
            Modelica.Electrical.Analog.Sources.RampCurrent pmRampCurrent(I = -6, duration = 6, offset = 3) "Ideal current source" annotation(
              Placement(transformation(origin = {-60, -60}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
            Modelica.Electrical.Analog.Basic.Ground cGround annotation(
              Placement(transformation(extent = {{-70, -20}, {-50, 0}}, rotation = 0)));
            Modelica.Mechanics.Translational.Components.Fixed cFixedPos(s0 = 0) "Fixed armature position" annotation(
              Placement(transformation(extent = {{10, 10}, {30, 30}}, rotation = 0)));
            Modelica.Electrical.Analog.Sources.RampCurrent cRampCurrent(I = -6, duration = 6, offset = 3) "Ideal current source" annotation(
              Placement(transformation(origin = {-60, 20}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
            Modelica.Magnetic.FluxTubes.Examples.MovingCoilActuator.Components.ConstantActuator cActuator "Moving coil actuator described with converter constant" annotation(
              Placement(transformation(extent = {{-20, 10}, {0, 30}}, rotation = 0)));
            Modelica.Blocks.Tables.CombiTable1Ds comparisonWithFEA(table = [-3, -9.65653; -2.5, -8.28587; -2, -6.82002; -1.5, -5.25898; -1, -3.60274; -0.5, -1.85131; 0, -0.00468; 0.5, 1.93714; 1, 3.97415; 1.5, 6.10636; 2, 8.33376; 2.5, 10.65636; 3, 13.07415]) "Column 1: current, col. 2: force; mu_rFe=const.=1000 in FEA model" annotation(
              Placement(transformation(extent = {{40, 50}, {60, 70}}, rotation = 0)));
            Modelica.Electrical.Analog.Sensors.CurrentSensor iSensor "Input value for look-up table with FEA results" annotation(
              Placement(transformation(extent = {{-50, 40}, {-30, 20}}, rotation = 0)));
          equation
            connect(pmFixedPos.flange, pmActuator.flange) annotation(
              Line(points = {{20, -60}, {0, -60}}, color = {0, 127, 0}));
            connect(pmRampCurrent.p, pmActuator.p) annotation(
              Line(points = {{-60, -50}, {-30, -50}, {-30, -54}, {-20, -54}}, color = {0, 0, 255}));
            connect(pmActuator.n, pmRampCurrent.n) annotation(
              Line(points = {{-20, -66}, {-30, -66}, {-30, -70}, {-60, -70}}, color = {0, 0, 255}));
            connect(pmGround.p, pmRampCurrent.n) annotation(
              Line(points = {{-60, -80}, {-60, -70}}, color = {0, 0, 255}));
            connect(cGround.p, cRampCurrent.n) annotation(
              Line(points = {{-60, 0}, {-60, 10}}, color = {0, 0, 255}));
            connect(cActuator.flange, cFixedPos.flange) annotation(
              Line(points = {{0, 20}, {20, 20}}, color = {0, 127, 0}));
            connect(cRampCurrent.n, cActuator.n) annotation(
              Line(points = {{-60, 10}, {-30, 10}, {-30, 14}, {-20, 14}}, color = {0, 0, 255}));
            connect(cRampCurrent.p, iSensor.p) annotation(
              Line(points = {{-60, 30}, {-50, 30}}, color = {0, 0, 255}));
            connect(cActuator.p, iSensor.n) annotation(
              Line(points = {{-20, 26}, {-30, 26}, {-30, 30}}, color = {0, 0, 255}));
            connect(iSensor.i, comparisonWithFEA.u) annotation(
              Line(points = {{-40, 40}, {-40, 60}, {38, 60}}, color = {0, 0, 127}));
            annotation(
              Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-98, 96}, {100, 86}}, lineColor = {0, 0, 255}, textString = "Comparison of the force-current characteristics of both converter models"), Text(extent = {{-98, 88}, {2, 78}}, lineColor = {0, 0, 255}, textString = "with armature blocked at mid-position")}),
              experiment(StopTime = 6, Tolerance = 1e-007),
              Documentation(info = "<html>
<p>
Have a look at <a href=\"modelica://Modelica.Magnetic.FluxTubes.Examples.MovingCoilActuator.Components.ConstantActuator\">ConstantActuator</a> and at <a href=\"modelica://Modelica.Magnetic.FluxTubes.Examples.MovingCoilActuator.Components.PermeanceActuator\">PermeanceActuator</a> for an explanation of both converter models. <br>
</p>
<p>
Simulation of the force-current characteristics of both converter models with the armature blocked at mid-position x=0 reveals the difference between the two models. In the <a href=\"modelica://Modelica.Magnetic.FluxTubes.Examples.MovingCoilActuator.Components.ConstantActuator\">ConstantActuator</a>, force is proportional to current. In the simple <a href=\"modelica://Modelica.Magnetic.FluxTubes.Examples.MovingCoilActuator.Components.PermeanceActuator\">PermeanceActuator</a> there is an additional non-linear force component that is due to the dependency of the inductance on the armature position. Comparison with FEA results validates the higher accuracy of the <a href=\"modelica://Modelica.Magnetic.FluxTubes.Examples.MovingCoilActuator.Components.PermeanceActuator\">PermeanceActuator</a>. In the FEA model, the relative permeability of the stator iron was set to mu_rFe=const.=1000 in order to avoid additional non-linear force components due to saturation. <b>Simulate for 6 s</b> and <b>plot vs. current (e.g., iSensor.i)</b></p>
<pre>
    pmFixedPos.flange_b.f       // force of permeance model
    cFixedPos.flange_b.f        // force of converter constant model
    comparisonWithFEA.y[1]      // force of FEA model for comparison
</pre>
</html>"));
          end ForceCurrentBehaviour;

          model ForceStrokeBehaviour "Force-stroke characteristic of the permeance model at constant current"
            extends Modelica.Icons.Example;
            Modelica.Electrical.Analog.Basic.Ground ground annotation(
              Placement(transformation(extent = {{-70, -30}, {-50, -10}}, rotation = 0)));
            Modelica.Magnetic.FluxTubes.Examples.MovingCoilActuator.Components.PermeanceActuator actuator(x(start = 0), material = Modelica.Magnetic.FluxTubes.Material.HardMagnetic.PlasticNdFeB()) annotation(
              Placement(transformation(extent = {{-30, 0}, {-10, 20}}, rotation = 0)));
            Modelica.Electrical.Analog.Sources.ConstantCurrent source(I = 3) annotation(
              Placement(transformation(origin = {-60, 10}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
            Modelica.Blocks.Sources.Ramp sweepX(height = 7.99e-3, duration = 8, offset = -3.995e-3, startTime = -4) annotation(
              Placement(transformation(extent = {{70, 0}, {50, 20}}, rotation = 0)));
            Modelica.Mechanics.Translational.Sources.Position feedX(exact = true) annotation(
              Placement(transformation(extent = {{30, 0}, {10, 20}}, rotation = 0)));
            Modelica.Blocks.Tables.CombiTable1Ds comparisonWithFEA(table = [-0.004, -8.8729, -9.07503, 0.00332; -0.0035, -9.05239, -9.25042, 0.00352; -0.003, -9.1915, -9.38558, 0.00371; -0.0025, -9.28247, -9.47266, 0.0039; -0.002, -9.3587, -9.54503, 0.00409; -0.0015, -9.41568, -9.59782, 0.00429; -0.001, -9.45496, -9.6331, 0.00448; -0.0005, -9.47427, -9.64839, 0.00467; 0, -9.48639, -9.65616, 0.00486; 0.0005, -9.48623, -9.65174, 0.00505; 0.001, -9.4732, -9.63435, 0.00524; 0.0015, -9.44143, -9.59825, 0.00543; 0.002, -9.39915, -9.55226, 0.00562; 0.0025, -9.33166, -9.47988, 0.00581; 0.003, -9.23707, -9.38112, 0.006; 0.0035, -9.09497, -9.23417, 0.00619; 0.004, -8.91839, -9.05337, 0.00638]) "Column 1: position, col.2: force with non-linear stator iron, col.3: force with mu_rFe=const.=1000, col.4: inductance with mu_rFe=const.=1000" annotation(
              Placement(transformation(extent = {{50, 40}, {70, 60}}, rotation = 0)));
          equation
            connect(ground.p, source.n) annotation(
              Line(points = {{-60, -10}, {-60, 0}}, color = {0, 0, 255}));
            connect(source.n, actuator.n) annotation(
              Line(points = {{-60, 0}, {-46, 0}, {-46, 4}, {-30, 4}}, color = {0, 0, 255}));
            connect(source.p, actuator.p) annotation(
              Line(points = {{-60, 20}, {-46, 20}, {-46, 16}, {-30, 16}}, color = {0, 0, 255}));
            connect(sweepX.y, feedX.s_ref) annotation(
              Line(points = {{49, 10}, {32, 10}}, color = {0, 0, 127}));
            connect(feedX.flange, actuator.flange) annotation(
              Line(points = {{10, 10}, {-10, 10}}, color = {0, 127, 0}));
            connect(feedX.s_ref, comparisonWithFEA.u) annotation(
              Line(points = {{32, 10}, {32, 50}, {48, 50}}, color = {0, 0, 127}));
            annotation(
              experiment(StartTime = -4, StopTime = 4, Tolerance = 1e-007),
              Documentation(info = "<html>
<p>
Have a look at <a href=\"modelica://Modelica.Magnetic.FluxTubes.Examples.MovingCoilActuator.Components.ConstantActuator\">ConstantActuator</a> and at <a href=\"modelica://Modelica.Magnetic.FluxTubes.Examples.MovingCoilActuator.Components.PermeanceActuator\">PermeanceActuator</a> for an explanation of both converter models. <br>
</p>
<p>
Simulation of the force-stroke characteristic of the <a href=\"modelica://Modelica.Magnetic.FluxTubes.Examples.MovingCoilActuator.Components.PermeanceActuator\">PermeanceActuator</a> with a constant current I=3A and a forced armature movement (similar to measurements in reality) shows the dependency of both force-generating permeances G_ma and G_mb as well as inductance L on armature position x. <b>Simulate for 8 s</b> and <b>plot vs.</b> armature position <b>feedX.flange_b.s</b> (same physical quantities together in a common diagram for comparison):</p>
<pre>
    feedX.flange_b.f            // force of permeance model (permeance of stator iron neglected in this model)
    comparisonWithFEA.y[1]      // force of FEA model with non-linear stator iron 1.0718
    comparisonWithFEA.y[2]      // force of FEA model with mu_rFe=const.=1000
    actuator.g_ma.G_m           // permeance G_ma
    actuator.g_mb.G_m           // permeance G_mb
    actuator.L                  // inductance of permeance model
    comparisonWithFEA.y[3]      // inductance of FEA model for comparison (mu_rFe=const.=1000).
</pre>
</html>"));
          end ForceStrokeBehaviour;

          model ArmatureStroke "Armature stroke of both moving coil actuator models after a voltage step at time t=0"
            extends Modelica.Icons.Example;
            Modelica.Electrical.Analog.Basic.Ground pmGround annotation(
              Placement(transformation(extent = {{-80, -70}, {-60, -50}}, rotation = 0)));
            Modelica.Electrical.Analog.Sources.StepVoltage pmSource(startTime = 0, V = pmActuator.R * 1.5) "Steady state current 1.5A" annotation(
              Placement(transformation(origin = {-70, -30}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
            Modelica.Magnetic.FluxTubes.Examples.MovingCoilActuator.Components.PermeanceActuator pmActuator(material = Modelica.Magnetic.FluxTubes.Material.HardMagnetic.PlasticNdFeB(), x(start = pmActuator.x_min, fixed = true), armature(v(fixed = true)), coil(i(fixed = true))) "Moving coil actuator described with permeance model" annotation(
              Placement(transformation(extent = {{-40, -40}, {-20, -20}}, rotation = 0)));
            Modelica.Mechanics.Translational.Components.Mass pmLoad(m = 0.05) "Load to be moved in addition to the armature mass" annotation(
              Placement(transformation(extent = {{0, -40}, {20, -20}}, rotation = 0)));
            Modelica.Electrical.Analog.Basic.Ground cGround annotation(
              Placement(transformation(extent = {{-80, 0}, {-60, 20}}, rotation = 0)));
            Modelica.Electrical.Analog.Sources.StepVoltage cSource(startTime = 0, V = cActuator.R * 1.5) "Steady state current 1.5A" annotation(
              Placement(transformation(origin = {-70, 40}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
            Modelica.Magnetic.FluxTubes.Examples.MovingCoilActuator.Components.ConstantActuator cActuator(x(start = cActuator.x_min, fixed = true), armature(v(fixed = true)), l(i(start = 0, fixed = true))) "Moving coil actuator described with converter constant" annotation(
              Placement(transformation(extent = {{-40, 30}, {-20, 50}}, rotation = 0)));
            Modelica.Mechanics.Translational.Components.Mass cLoad(m = 0.05) "Load to be moved in addition to the armature mass" annotation(
              Placement(transformation(extent = {{0, 30}, {20, 50}}, rotation = 0)));
          equation
            connect(pmLoad.flange_a, pmActuator.flange) annotation(
              Line(points = {{0, -30}, {-20, -30}}, color = {0, 127, 0}));
            connect(cGround.p, cSource.n) annotation(
              Line(points = {{-70, 20}, {-70, 30}}, color = {0, 0, 255}));
            connect(cLoad.flange_a, cActuator.flange) annotation(
              Line(points = {{0, 40}, {-20, 40}}, color = {0, 127, 0}));
            connect(cSource.p, cActuator.p) annotation(
              Line(points = {{-70, 50}, {-56, 50}, {-56, 46}, {-40, 46}}, color = {0, 0, 255}));
            connect(cSource.n, cActuator.n) annotation(
              Line(points = {{-70, 30}, {-56, 30}, {-56, 34}, {-40, 34}}, color = {0, 0, 255}));
            connect(pmSource.n, pmGround.p) annotation(
              Line(points = {{-70, -40}, {-70, -50}}, color = {0, 0, 255}));
            connect(pmSource.n, pmActuator.n) annotation(
              Line(points = {{-70, -40}, {-54, -40}, {-54, -36}, {-40, -36}}, color = {0, 0, 255}));
            connect(pmSource.p, pmActuator.p) annotation(
              Line(points = {{-70, -20}, {-55, -20}, {-55, -24}, {-40, -24}}, color = {0, 0, 255}));
            annotation(
              Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-98, 98}, {48, 86}}, lineColor = {0, 0, 255}, textString = "Comparison of a pull-in stroke of both converter models"), Text(extent = {{-98, 90}, {-30, 80}}, lineColor = {0, 0, 255}, textString = "after a voltage step at t=0")}),
              experiment(StopTime = 0.05, Tolerance = 1e-007),
              Documentation(info = "<html>
<p>
Have a look at <a href=\"modelica://Modelica.Magnetic.FluxTubes.Examples.MovingCoilActuator.Components.ConstantActuator\">ConstantActuator</a> and at <a href=\"modelica://Modelica.Magnetic.FluxTubes.Examples.MovingCoilActuator.Components.PermeanceActuator\">PermeanceActuator</a>
for an explanation of both actuator models.</p>

<p>
A voltage step at time t=0 is applied to both actuator models. In each model, the armature and an attached load mass perform a stroke between the two stoppers included in cActuator.armature and pmActuator.armature respectively. <b>Simulate for 0.05 s</b> and plot vs. time (same physical quantities together in a common diagram for comparison):
</p>
<pre>
    cActuator.p.i                     // input current to converter constant model
    pmActuator.p.i                    // input current to permeance model
    cActuator.armature.flange_a.f     // actuator force of converter constant model
    pmActuator.armature.flange_a.f    // actuator force of permeance model
    cActuator.x                       // armature position of converter constant model
    pmActuator.x                      // armature position of permeance model
    cActuator.L                       // inductance of converter constant model
    pmActuator.L                      // inductance of permeance model
</pre>
<p>
The initial current rise in both actuator models is due to the inductance of the actuator coil. After acceleration of the armature and the load, the current decreases due to the motion-induced back-emf. Bouncing occurs when the armatures of both models arrive at the stopper at maximum armature position. The bouncing is rather intense due to the absence of any kind of external friction in this simple example (apart from the nonlinear damping in the stopper elements). After decay of the bouncing, both actuators operate under conditions valid for a blocked armature.
</p>
<p>
Whereas the steady state current is the same in both models, the steady state actuator force is not due to the neglection of the non-linear force component in the converter constant model. Differences in the current rise of both models are due to the neglection of the coil inductance variation in the converter constant model.
</p>
</html>"));
          end ArmatureStroke;

          package Components "Components to be used in examples"
            extends Modelica.Icons.Package;

            model PermeanceActuator "Detailed actuator model for rough magnetic design of actuator and system simulation"
              parameter Real N = 140 "Number of turns" annotation(
                Dialog(group = "Parameters", groupImage = "modelica://Modelica/Resources/Images/Magnetic/FluxTubes/Examples/MovingCoilActuator/MovingCoilActuator_dimensions.png"));
              parameter SI.Resistance R = 2.86 "Coil resistance";
              parameter SI.Radius r_core = 12.5e-3 "Radius of ferromagnetic stator core";
              parameter SI.Length l_PM = 3.5e-3 "Radial thickness of permanent magnet ring";
              parameter SI.Length t = 0.02 "Axial length of permanent magnet ring and air gap respectively";
              parameter SI.Length l_air = 3e-3 "Total radial length of armature air gap";
              parameter SI.Length l_FeOut = 4e-3 "Radial thickness of outer back iron (for estimation of leakage permeance)";
              parameter Modelica.Magnetic.FluxTubes.Material.HardMagnetic.BaseData material = Modelica.Magnetic.FluxTubes.Material.HardMagnetic.BaseData() "Ferromagnetic material characteristics" annotation(
                choicesAllMatching = true,
                Dialog(group = "Material"));
              parameter SI.Mass m_a = 0.012 "Mass of armature" annotation(
                Dialog(group = "Armature and stopper"));
              parameter SI.TranslationalSpringConstant c = 1e11 "Spring stiffness between impact partners" annotation(
                Dialog(group = "Armature and stopper"));
              parameter SI.TranslationalDampingConstant d = 400 "Damping coefficient between impact partners" annotation(
                Dialog(group = "Armature and stopper"));
              parameter SI.Position x_min = -4e-3 "Position of stopper at minimum armature position" annotation(
                Dialog(group = "Armature and stopper"));
              parameter SI.Position x_max = 4e-3 "Position of stopper at maximum armature position" annotation(
                Dialog(group = "Armature and stopper"));
              SI.Position x(start = x_min, stateSelect = StateSelect.prefer) "Armature position, alias for flange position";
              SI.Inductance L "Coil inductance";
              Modelica.Magnetic.FluxTubes.Sources.ConstantMagneticPotentialDifference mmf_PM(final V_m = material.H_cB * l_PM) "Permanent magnet's magnetomotive force" annotation(
                Placement(transformation(origin = {10, 20}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
              Modelica.Magnetic.FluxTubes.Examples.Utilities.TranslatoryArmatureAndStopper armature(final L = 0, final m = m_a, final c = c, final d = d, n = 2, final x_max = x_max, final x_min = x_min) "Inertia of moving coil + coil carrier; stoppers at end of stroke range" annotation(
                Placement(transformation(extent = {{60, -10}, {80, 10}}, rotation = 0)));
              Modelica.Electrical.Analog.Basic.Resistor r(final R = R) annotation(
                Placement(transformation(origin = {-80, 30}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
              Basic.Ground ground annotation(
                Placement(transformation(extent = {{30, -38}, {50, -18}}, rotation = 0)));
              Modelica.Magnetic.FluxTubes.Basic.ElectroMagneticConverter coil(final N = N) annotation(
                Placement(transformation(origin = {0, -20}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
              Modelica.Magnetic.FluxTubes.Shapes.Force.HollowCylinderRadialFlux g_ma(final dlBydx = 1, final l = t / 2 + x, final r_i = r_core, final r_o = r_core + l_air + l_PM, final mu_r = 1.05, final useSupport = false) annotation(
                Placement(transformation(origin = {-20, 0}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
              Modelica.Magnetic.FluxTubes.Shapes.Force.HollowCylinderRadialFlux g_mb(final r_i = r_core, final r_o = r_core + l_air + l_PM, final dlBydx = -1, final l = t / 2 - x, final mu_r = 1.05, final useSupport = false) annotation(
                Placement(transformation(origin = {30, 0}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
              Modelica.Magnetic.FluxTubes.Shapes.Leakage.CoaxCylindersEndFaces g_mLeak1(final r_1 = r_core + l_air + l_PM, final r_2 = r_core + l_air + l_PM + l_FeOut, final r_0 = r_core) "Leakage between coaxial end planes of ferromagnetic stator core and outer back iron" annotation(
                Placement(transformation(origin = {-60, 10}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
              Modelica.Magnetic.FluxTubes.Shapes.Leakage.HalfCylinder g_mLeak2(final l = 2 * pi * (r_core + (l_air + l_PM) / 2)) "Leakage between edges of ferromagnetic stator core and outer back iron" annotation(
                Placement(transformation(origin = {-40, 10}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
              Modelica.Electrical.Analog.Interfaces.PositivePin p "Electrical connector" annotation(
                Placement(transformation(extent = {{-110, 50}, {-90, 70}}, rotation = 0)));
              Modelica.Electrical.Analog.Interfaces.NegativePin n "Electrical connector" annotation(
                Placement(transformation(extent = {{-90, -70}, {-110, -50}}, rotation = 0)));
              Modelica.Mechanics.Translational.Interfaces.Flange_b flange "Flange of component" annotation(
                Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
            equation
              x = flange.s;
              L = coil.N ^ 2 * (g_ma.G_m + g_mLeak1.G_m + g_mLeak2.G_m);
              connect(armature.flange_b, flange) annotation(
                Line(points = {{80, 0}, {100, 0}}, color = {0, 127, 0}));
              connect(r.p, p) annotation(
                Line(points = {{-80, 40}, {-80, 60}, {-100, 60}}, color = {0, 0, 255}));
              connect(armature.flange_a, g_mb.flange) annotation(
                Line(points = {{60, 0}, {40, 0}}, color = {0, 127, 0}));
              connect(g_mb.flange, g_ma.flange) annotation(
                Line(points = {{40, 0}, {40, 40}, {-10, 40}, {-10, 0}}, color = {0, 127, 0}));
              connect(g_mLeak1.port_n, g_ma.port_n) annotation(
                Line(points = {{-60, 0}, {-60, -10}, {-20, -10}}, color = {255, 127, 0}));
              connect(g_mLeak2.port_n, g_ma.port_n) annotation(
                Line(points = {{-40, 0}, {-40, -10}, {-20, -10}}, color = {255, 127, 0}));
              connect(g_ma.port_n, coil.port_p) annotation(
                Line(points = {{-20, -10}, {-18, -10}, {-18, -12}, {-14, -12}, {-14, -10}, {-6, -10}}, color = {255, 127, 0}));
              connect(coil.port_n, g_mb.port_n) annotation(
                Line(points = {{6, -10}, {30, -10}}, color = {255, 127, 0}));
              connect(g_mb.port_n, mmf_PM.port_p) annotation(
                Line(points = {{30, -10}, {50, -10}, {50, 30}, {10, 30}}, color = {255, 127, 0}));
              connect(mmf_PM.port_p, g_mLeak2.port_p) annotation(
                Line(points = {{10, 30}, {-40, 30}, {-40, 20}}, color = {255, 127, 0}));
              connect(mmf_PM.port_p, g_mLeak1.port_p) annotation(
                Line(points = {{10, 30}, {-60, 30}, {-60, 20}}, color = {255, 127, 0}));
              connect(g_ma.port_p, mmf_PM.port_n) annotation(
                Line(points = {{-20, 10}, {10, 10}}, color = {255, 127, 0}));
              connect(mmf_PM.port_n, g_mb.port_p) annotation(
                Line(points = {{10, 10}, {30, 10}}, color = {255, 127, 0}));
              connect(r.n, coil.p) annotation(
                Line(points = {{-80, 20}, {-80, -30}, {-6, -30}}, color = {0, 0, 255}));
              connect(coil.n, n) annotation(
                Line(points = {{6, -30}, {6, -60}, {-100, -60}}, color = {0, 0, 255}));
              connect(ground.port, g_mb.port_n) annotation(
                Line(points = {{40, -18}, {40, -10}, {30, -10}}, color = {255, 127, 0}, smooth = Smooth.None));
              annotation(
                Documentation(info = "<html>
<p>
In the <a href=\"modelica://Modelica.Magnetic.FluxTubes.Examples.MovingCoilActuator.Components.ConstantActuator\">ConstantActuator</a> model the force F is strictly proportional to the current i as indicated by the converter constant c. However, there is an additional non-linear force component in such an actuator that is due to the dependency of the coil inductance L on the armature position x. The inductance increases as the armature moves into the stator. The total force is
</p>

<pre>
        1  2 dL
    F = - i  --  + c i
        2    dx
</pre>

<p>
Both force components are properly considered with a simple permeance model as shown in the figures below. Figure (a) illustrates the dimensions of the axisymmetric moving coil actuator that are needed in the permeance model. Figure (b) shows partitioning into flux tubes and the permanent magnetic field without current. G_ma and G_mb both are the permeances resulting from a series connection of the permanent magnet and air gap sections. The field plot of the coil-imposed mmf is shown in figure (c) without the permanent magnetic mmf (H_cB=0). The placement of the magnetic network components in figure (d) retains the geometric structure of the actuator. In figure (e), the permeance model is restructured and thus simplified.
</p>

<p>
<IMG src=\"modelica://Modelica/Resources/Images/Magnetic/FluxTubes/Examples/MovingCoilActuator/MovingCoilActuator_PermeanceModel.png\" ALT=\"Structure, assigned flux tubes and field plots of the moving coil actuator\">
</p>
</html>"),
                Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-90, 100}, {90, -100}}, lineColor = {255, 128, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-90, 100}, {-50, -100}}, lineColor = {0, 0, 255}, pattern = LinePattern.None, fillColor = {255, 128, 0}, fillPattern = FillPattern.Solid), Rectangle(extent = {{90, 80}, {-90, 100}}, lineColor = {0, 0, 255}, pattern = LinePattern.None, fillColor = {255, 128, 0}, fillPattern = FillPattern.Solid), Rectangle(extent = {{90, -80}, {-90, -100}}, lineColor = {0, 0, 255}, pattern = LinePattern.None, fillColor = {255, 128, 0}, fillPattern = FillPattern.Solid), Rectangle(extent = {{70, 34}, {-90, -34}}, lineColor = {0, 0, 255}, pattern = LinePattern.None, fillColor = {255, 128, 0}, fillPattern = FillPattern.Solid), Rectangle(extent = {{90, 52}, {-16, 64}}, lineColor = {0, 0, 255}, pattern = LinePattern.None, fillColor = {255, 213, 170}, fillPattern = FillPattern.Solid), Rectangle(extent = {{90, -64}, {-12, -52}}, lineColor = {0, 0, 255}, pattern = LinePattern.None, fillColor = {255, 213, 170}, fillPattern = FillPattern.Solid), Text(extent = {{0, 160}, {0, 120}}, lineColor = {0, 0, 255}, textString = "%name")}));
            end PermeanceActuator;

            model ConstantActuator "Simple behavioural actuator model for system simulation"
              parameter SI.ElectricalForceConstant k = 3.88 "Converter constant";
              parameter SI.Resistance R = 2.86 "Coil resistance";
              parameter SI.Inductance L = 0.0051 "Coil inductance at mid-stroke";
              parameter SI.Mass m_a = 0.012 "Armature mass" annotation(
                Dialog(group = "Armature and stopper"));
              parameter Modelica.SIunits.TranslationalSpringConstant c = 1e11 "Spring stiffness between impact partners" annotation(
                Dialog(group = "Armature and stopper"));
              parameter Modelica.SIunits.TranslationalDampingConstant d = 400 "Damping coefficient between impact partners" annotation(
                Dialog(group = "Armature and stopper"));
              parameter SI.Position x_min = -4e-3 "Minimum armature position" annotation(
                Dialog(group = "Armature and stopper"));
              parameter SI.Position x_max = 4e-3 "Maximum armature position" annotation(
                Dialog(group = "Armature and stopper"));
              SI.Position x(start = x_min, stateSelect = StateSelect.prefer) "Armature position, alias for flange position";
              Modelica.Electrical.Analog.Basic.Resistor r(final R = R) "Coil resistance" annotation(
                Placement(transformation(extent = {{-90, 50}, {-70, 70}}, rotation = 0)));
              Modelica.Magnetic.FluxTubes.Examples.Utilities.TranslatoryArmatureAndStopper armature(final m = m_a, final x_max = x_max, final x_min = x_min, final L = 0, final c = c, final d = d, n = 2) "Armature inertia with stoppers at end of stroke range" annotation(
                Placement(transformation(extent = {{60, -10}, {80, 10}}, rotation = 0)));
              Modelica.Electrical.Analog.Basic.Inductor l(final L = L) "Coil inductance" annotation(
                Placement(transformation(extent = {{-60, 50}, {-40, 70}}, rotation = 0)));
              Modelica.Electrical.Analog.Basic.TranslationalEMF electroTranslationalConverter(final k = k) annotation(
                Placement(transformation(extent = {{-40, -10}, {-20, 10}}, rotation = 0)));
              Modelica.Electrical.Analog.Interfaces.PositivePin p "Electrical connector" annotation(
                Placement(transformation(extent = {{-110, 50}, {-90, 70}}, rotation = 0)));
              Modelica.Electrical.Analog.Interfaces.NegativePin n "Electrical connector" annotation(
                Placement(transformation(extent = {{-90, -70}, {-110, -50}}, rotation = 0)));
              Modelica.Mechanics.Translational.Interfaces.Flange_b flange "Flange of component" annotation(
                Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
            equation
              flange.s = x;
              connect(r.p, p) annotation(
                Line(points = {{-90, 60}, {-100, 60}}, color = {0, 0, 255}));
              connect(l.p, r.n) annotation(
                Line(points = {{-60, 60}, {-70, 60}}, color = {0, 0, 255}));
              connect(armature.flange_b, flange) annotation(
                Line(points = {{80, 0}, {85, 0}, {90, 0}, {100, 0}}, color = {0, 127, 0}));
              connect(l.n, electroTranslationalConverter.p) annotation(
                Line(points = {{-40, 60}, {-30, 60}, {-30, 10}}, color = {0, 0, 255}));
              connect(n, electroTranslationalConverter.n) annotation(
                Line(points = {{-100, -60}, {-30, -60}, {-30, -10}}, color = {0, 0, 255}));
              connect(electroTranslationalConverter.flange, armature.flange_a) annotation(
                Line(points = {{-20, 0}, {0, 0}, {20, 0}, {60, 0}}, color = {0, 127, 0}));
              annotation(
                Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-80, 100}, {80, -100}}, lineColor = {255, 128, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-70, 80}, {72, -80}}, lineColor = {255, 128, 0}, textString = "c"), Line(points = {{-90, 60}, {-80, 60}}, color = {0, 0, 255}), Line(points = {{-90, -60}, {-80, -60}}, color = {0, 0, 255}), Line(points = {{80, 0}, {90, 0}}, color = {0, 127, 0}), Text(extent = {{0, 160}, {0, 120}}, lineColor = {0, 0, 255}, textString = "%name")}),
                Documentation(info = "<html>
<p>
Similar to rotational DC-Motors, the electro-mechanical energy conversion of translatory electrodynamic actuators and generators of moving coil and moving magnet type can be described with the following two converter equations:
</p>

<pre>
      F = c * i
    V_i = c * v
</pre>

<p>
with electrodynamic or <i>Lorentz</i> force F, converter constant c, current i, induced back-emf V_i and armature velocity v. The model is very similar to the well-known behavioural model of a rotational one-phase DC-Machine, except that it is for translatory motion. For a moving coil actuator with a coil inside an air gap with flux density B and a total wire length l inside the magnetic field, the converter constant becomes
</p>

<pre>
    c = B * l.
</pre>

<p>
The converter constant c as well as coil resistance R and inductance L are assumed to be known, e.g., from measurements or catalogue data. Hence this model is well-suited for system simulation together with neighbouring subsystems, but not for actuator design, where the motor constant is to be found on base of the magnetic circuit's geometry, material properties and winding data. See <a href=\"modelica://Modelica.Magnetic.FluxTubes.Examples.MovingCoilActuator.Components.PermeanceActuator\">PermeanceActuator</a> for a more accurate model of this actuator that is based on a magnetic network. Due to identical connectors, both models can be used in system simulation, e.g. to simulate a stroke as demonstrated in <a href=\"modelica://Modelica.Magnetic.FluxTubes.Examples.MovingCoilActuator.ArmatureStroke\">ArmatureStroke</a>.
</p>
</html>"));
            end ConstantActuator;
          end Components;
          annotation(
            Documentation(info = "<html>
<p>
Moving coil actuators are often called electrodynamic actuators and a proportional behaviour between force and current is expressed by a converter constant (see <a href=\"modelica://Modelica.Magnetic.FluxTubes.Examples.MovingCoilActuator.Components.ConstantActuator\">ConstantActuator</a>). However, in a simple moving coil actuator as presented in this example there is an additional non-linear force component that is due to the increase of the inductance when the armature coil moves into the ferromagnetic stator. A simple <a href=\"modelica://Modelica.Magnetic.FluxTubes.Examples.MovingCoilActuator.Components.PermeanceActuator\">PermeanceActuator</a> can be used to describe this non-linear force component.
</p>
</html>"));
        end MovingCoilActuator;

        package SolenoidActuator "Two models of a reluctance actuator of different modelling depth and their comparison and usage"
          extends Modelica.Icons.ExamplesPackage;

          model ComparisonQuasiStationary "Slow forced armature motion of both solenoid models so that electromagnetic field and current are quasi-stationary"
            extends Modelica.Icons.Example;
            parameter SI.Voltage v_step = 12 "Applied voltage";
            Modelica.Blocks.Sources.Ramp x_set(duration = 10, height = -(advancedSolenoid.x_max - advancedSolenoid.x_min), offset = advancedSolenoid.x_max) "Prescribed armature position, slow enforced motion from x_max to x_min" annotation(
              Placement(transformation(extent = {{80, -10}, {60, 10}}, rotation = 0)));
            Modelica.Electrical.Analog.Basic.Ground advancedGround annotation(
              Placement(transformation(extent = {{-80, 10}, {-60, 30}}, rotation = 0)));
            Modelica.Electrical.Analog.Sources.StepVoltage advancedSource(V = v_step) annotation(
              Placement(transformation(origin = {-70, 50}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
            Modelica.Magnetic.FluxTubes.Examples.SolenoidActuator.Components.AdvancedSolenoid advancedSolenoid annotation(
              Placement(transformation(extent = {{-40, 40}, {-20, 60}}, rotation = 0)));
            Modelica.Mechanics.Translational.Sources.Position advancedFeed_x(f_crit = 1000, exact = false) annotation(
              Placement(transformation(origin = {0, 50}, extent = {{-10, -10}, {10, 10}}, rotation = 180)));
            Modelica.Electrical.Analog.Basic.Ground simpleGround annotation(
              Placement(transformation(extent = {{-80, -90}, {-60, -70}}, rotation = 0)));
            Modelica.Electrical.Analog.Sources.StepVoltage simpleSource(V = v_step) annotation(
              Placement(transformation(origin = {-70, -50}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
            Modelica.Magnetic.FluxTubes.Examples.SolenoidActuator.Components.SimpleSolenoid simpleSolenoid annotation(
              Placement(transformation(extent = {{-40, -60}, {-20, -40}}, rotation = 0)));
            Modelica.Mechanics.Translational.Sources.Position simpleFeed_x(f_crit = 1000, exact = false) annotation(
              Placement(transformation(origin = {0, -50}, extent = {{-10, -10}, {10, 10}}, rotation = 180)));
            Modelica.Blocks.Tables.CombiTable1Ds comparisonWithFEA(table = [0.00025, -85.8619, 0.00014821, 0.11954; 0.0005, -59.9662, 0.00013931, 0.11004; 0.00075, -41.0806, 0.0001277, 0.098942; 0.001, -28.88, 0.00011587, 0.088425; 0.00125, -21.4113, 0.00010643, 0.08015; 0.0015, -16.8003, 9.9406e-005, 0.073992; 0.00175, -13.6942, 9.3416e-005, 0.068792; 0.002, -11.1188, 8.8564e-005, 0.064492; 0.00225, -9.6603, 8.4505e-005, 0.060917; 0.0025, -8.4835, 8.1215e-005, 0.058017; 0.00275, -7.4658, 7.7881e-005, 0.055125; 0.003, -6.5591, 7.5197e-005, 0.052733; 0.00325, -5.9706, 7.2447e-005, 0.05035; 0.0035, -5.5013, 7.0342e-005, 0.048525; 0.00375, -5.0469, 6.8527e-005, 0.046867; 0.004, -4.6573, 6.6526e-005, 0.045158; 0.00425, -4.2977, 6.4425e-005, 0.043442; 0.0045, -4.0912, 6.2747e-005, 0.04205; 0.00475, -3.7456, 6.1231e-005, 0.040733; 0.005, -3.5869, 5.9691e-005, 0.039467]) "Valid for u_source=12V only; column 1: position, col.2: force, col.3: armature flux, col.4: inductance" annotation(
              Placement(transformation(extent = {{60, 60}, {80, 80}}, rotation = 0)));
          equation
            connect(advancedGround.p, advancedSource.n) annotation(
              Line(points = {{-70, 30}, {-70, 40}}, color = {0, 0, 255}));
            connect(x_set.y, advancedFeed_x.s_ref) annotation(
              Line(points = {{59, 0}, {20, 0}, {20, 50}, {12, 50}}, color = {0, 0, 127}));
            connect(simpleSolenoid.p, simpleSource.p) annotation(
              Line(points = {{-40, -44}, {-50, -44}, {-50, -40}, {-70, -40}}, color = {0, 0, 255}));
            connect(simpleSolenoid.n, simpleSource.n) annotation(
              Line(points = {{-40, -56}, {-50, -56}, {-50, -60}, {-70, -60}}, color = {0, 0, 255}));
            connect(simpleSolenoid.flange, simpleFeed_x.flange) annotation(
              Line(points = {{-20, -50}, {-10, -50}}, color = {0, 127, 0}));
            connect(advancedSolenoid.n, advancedSource.n) annotation(
              Line(points = {{-40, 44}, {-50, 44}, {-50, 40}, {-70, 40}}, color = {0, 0, 255}));
            connect(simpleFeed_x.s_ref, x_set.y) annotation(
              Line(points = {{12, -50}, {20, -50}, {20, 0}, {59, 0}}, color = {0, 0, 127}));
            connect(x_set.y, comparisonWithFEA.u) annotation(
              Line(points = {{59, 0}, {50, 0}, {50, 70}, {58, 70}}, color = {0, 0, 127}));
            connect(advancedFeed_x.flange, advancedSolenoid.flange) annotation(
              Line(points = {{-10, 50}, {-20, 50}}, color = {0, 127, 0}));
            connect(advancedSource.p, advancedSolenoid.p) annotation(
              Line(points = {{-70, 60}, {-50, 60}, {-50, 56}, {-40, 56}}, color = {0, 0, 255}));
            connect(simpleGround.p, simpleSource.n) annotation(
              Line(points = {{-70, -70}, {-70, -60}}, color = {0, 0, 255}));
            annotation(
              experiment(StopTime = 10, Tolerance = 1e-007),
              Documentation(info = "<html>
<p>
Have a look at <a href=\"modelica://Modelica.Magnetic.FluxTubes.Examples.SolenoidActuator\">SolenoidActuator</a> for general comments and at <a href=\"modelica://Modelica.Magnetic.FluxTubes.Examples.SolenoidActuator.Components.SimpleSolenoid\">SimpleSolenoid</a> and <a href=\"modelica://Modelica.Magnetic.FluxTubes.Examples.SolenoidActuator.Components.AdvancedSolenoid\">AdvancedSolenoid</a> for a detailed description of both magnetic network models.
</p>

<p>
Similar to static force-stroke measurements on real actuators, the armatures of both actuator models are forced to move slowly here. Hence, the dynamics of the electrical subsystems due to coil inductance and armature motion can be neglected and the static force-stroke characteristics are obtained. To illustrate the accuracy to be expected from the lumped magnetic network models, results obtained with stationary FEA are included as reference (position-dependent force, armature flux and actuator inductance). Note that these reference values are valid for the default supply voltage v_step=12V DC only!
</p>

<p>
Set the <b>tolerance</b> to <b>1e-7</b> and <b>simulate for 10 s</b>. Plot in one common window the electromagnetic force of the two magnetic network models and the FEA reference <b>vs. armature position x_set.y</b>:
</p>

<pre>
    simpleSolenoid.armature.flange_a.f     // electromagnetic force of simple magnetic network model
    advancedSolenoid.armature.flange_a.f   // electromagnetic force of advanced magnetic network model
    comparisonWithFEA.y[1]                 // electromagnetic force obtained with FEA as reference
</pre>

<p>
Electromagnetic or reluctance forces always act towards a decrease of air gap lengths. With the defined armature position coordinate x, the forces of the models are negative.
</p>

<p>
The magnetic flux through the armature and the actuator's static inductance both illustrate the differences between the two magnetic network models. Similar to the forces, compare these quantities in one common plot window for each variable (plot vs. armature position x_set.y):
</p>

<pre>
    simpleSolenoid.G_mFeArm.Phi            // magnetic flux through armature of simple magnetic network model
    advancedSolenoid.G_mFeArm.Phi          // magnetic flux through armature of advanced magnetic network model
    comparisonWithFEA.y[2]                 // magnetic flux obtained with FEA as reference

    simpleSolenoid.coil.L_stat             // static inductance of simple magnetic network model
    advancedSolenoid.L_statTot             // series connection of both partial coils of advanced network model
    comparisonWithFEA.y[3]                 // static inductance obtained with FEA as reference
</pre>

<p>
As mentioned in the description of both magnetic network models, one can tell the higher armature flux and inductance of the advanced solenoid model at large air gaps compared to that of the simple model. The effect of this difference on dynamic model behaviour can be analysed in <a href=\"modelica://Modelica.Magnetic.FluxTubes.Examples.SolenoidActuator.ComparisonPullInStroke\">ComparisonPullInStroke</a>.
</p>
</html>"));
          end ComparisonQuasiStationary;

          model ComparisonPullInStroke "Pull-in stroke of both solenoid models after a voltage step at time t=0"
            extends Modelica.Icons.Example;
            parameter SI.Voltage v_step = 12 "Applied voltage";
            Modelica.Electrical.Analog.Basic.Ground advancedGround annotation(
              Placement(transformation(extent = {{-70, -10}, {-50, 10}}, rotation = 0)));
            Modelica.Electrical.Analog.Sources.StepVoltage advancedSource(V = v_step) annotation(
              Placement(transformation(origin = {-60, 30}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
            Modelica.Magnetic.FluxTubes.Examples.SolenoidActuator.Components.AdvancedSolenoid advancedSolenoid(x(fixed = true)) annotation(
              Placement(transformation(extent = {{-20, 20}, {0, 40}}, rotation = 0)));
            Modelica.Mechanics.Translational.Components.Mass advancedLoad(m = 0.01) "translatory load to be pulled horizontally" annotation(
              Placement(transformation(extent = {{20, 20}, {40, 40}}, rotation = 0)));
            Modelica.Electrical.Analog.Basic.Ground simpleGround annotation(
              Placement(transformation(extent = {{-70, -90}, {-50, -70}}, rotation = 0)));
            Modelica.Electrical.Analog.Sources.StepVoltage simpleSource(V = v_step) annotation(
              Placement(transformation(origin = {-60, -50}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
            Modelica.Magnetic.FluxTubes.Examples.SolenoidActuator.Components.SimpleSolenoid simpleSolenoid(x(fixed = true)) annotation(
              Placement(transformation(extent = {{-20, -60}, {0, -40}}, rotation = 0)));
            Modelica.Mechanics.Translational.Components.Mass simpleLoad(m = 0.01) "translatory load to be pulled horizontally" annotation(
              Placement(transformation(extent = {{20, -60}, {40, -40}}, rotation = 0)));
            Modelica.Blocks.Sources.CombiTimeTable comparisonWithFEA(table = [0, 0, 0, 0.005; 2.61165e-007, 7.93537e-005, -1.97914e-005, 0.005; 2.61165e-007, 7.93537e-005, -1.97914e-005, 0.005; 0.0001, 0.0300045, -0.00748335, 0.005; 0.0002, 0.05926, -0.0147799, 0.005; 0.0003, 0.0877841, -0.021894, 0.00499999; 0.0004, 0.115593, -0.036608, 0.00499997; 0.0005, 0.142707, -0.0568957, 0.00499994; 0.0006, 0.169143, -0.076676, 0.00499988; 0.0007, 0.194915, -0.0959614, 0.0049998; 0.0008, 0.220042, -0.124763, 0.00499968; 0.0009, 0.244539, -0.155317, 0.00499951; 0.001, 0.26842, -0.185107, 0.00499928; 0.0011, 0.291701, -0.214153, 0.00499898; 0.0012, 0.314394, -0.249655, 0.0049986; 0.0013, 0.336514, -0.288306, 0.00499812; 0.0014, 0.358074, -0.325991, 0.00499754; 0.0015, 0.379086, -0.362735, 0.00499682; 0.0016, 0.399562, -0.398563, 0.00499597; 0.0017, 0.419514, -0.44324, 0.00499496; 0.0018, 0.438955, -0.487015, 0.00499378; 0.0019, 0.457893, -0.529698, 0.00499242; 0.002, 0.47634, -0.571317, 0.00499085; 0.0021, 0.494305, -0.611901, 0.00498906; 0.0022, 0.511799, -0.657374, 0.00498704; 0.0023, 0.528832, -0.704491, 0.00498476; 0.0024, 0.545412, -0.750434, 0.00498221; 0.0025, 0.561548, -0.795237, 0.00497937; 0.0026, 0.577248, -0.83893, 0.00497623; 0.0027, 0.592521, -0.881543, 0.00497277; 0.0028, 0.607375, -0.926803, 0.00496896; 0.0029, 0.62182, -0.974598, 0.0049648; 0.003, 0.63586, -1.02121, 0.00496027; 0.0031, 0.649503, -1.06667, 0.00495534; 0.0032, 0.662756, -1.11102, 0.00495; 0.0033, 0.675625, -1.15428, 0.00494424; 0.0034, 0.688119, -1.19648, 0.00493803; 0.0035, 0.700242, -1.23778, 0.00493136; 0.0036, 0.712005, -1.28391, 0.00492421; 0.0037, 0.72341, -1.32891, 0.00491657; 0.0038, 0.734463, -1.3728, 0.00490842; 0.0039, 0.74517, -1.41563, 0.00489974; 0.004, 0.755536, -1.45743, 0.00489052; 0.0041, 0.765568, -1.49822, 0.00488074; 0.0042, 0.775269, -1.53803, 0.00487038; 0.0043, 0.784646, -1.57689, 0.00485943; 0.0044, 0.793704, -1.61483, 0.00484787; 0.0045, 0.80245, -1.65314, 0.00483569; 0.0046, 0.810888, -1.69366, 0.00482288; 0.0047, 0.81902, -1.7332, 0.00480941; 0.0048, 0.826851, -1.77179, 0.00479528; 0.0049, 0.834387, -1.80945, 0.00478046; 0.005, 0.841631, -1.84622, 0.00476495; 0.0051, 0.84859, -1.88259, 0.00474873; 0.0052, 0.855304, -1.92429, 0.00473179; 0.0053, 0.861739, -1.96564, 0.0047141; 0.0054, 0.8679, -2.00668, 0.00469566; 0.0055, 0.873791, -2.04743, 0.00467645; 0.0056, 0.879419, -2.08794, 0.00465645; 0.0057, 0.884782, -2.1282, 0.00463565; 0.0058, 0.889885, -2.16824, 0.00461403; 0.0059, 0.894731, -2.20808, 0.00459157; 0.006, 0.899322, -2.24774, 0.00456827; 0.0061, 0.903661, -2.28927, 0.0045441; 0.0062, 0.907752, -2.33091, 0.00451905; 0.0063, 0.911603, -2.37014, 0.0044931; 0.0064, 0.915232, -2.40274, 0.00446624; 0.0065, 0.91862, -2.43469, 0.00443846; 0.0066, 0.92177, -2.466, 0.00440974; 0.0067, 0.924686, -2.49668, 0.00438007; 0.0068, 0.927368, -2.52672, 0.00434945; 0.0069, 0.929822, -2.55615, 0.00431785; 0.007, 0.93205, -2.58498, 0.00428527; 0.0071, 0.934052, -2.61318, 0.00425169; 0.0072, 0.935241, -2.64973, 0.00421711; 0.0073, 0.936164, -2.68643, 0.00418151; 0.0074, 0.936854, -2.7228, 0.00414488; 0.0075, 0.937309, -2.7588, 0.0041072; 0.0076, 0.937532, -2.7944, 0.00406845; 0.0077, 0.937522, -2.82958, 0.00402864; 0.0078, 0.937411, -2.866, 0.00398773; 0.0079, 0.937385, -2.90613, 0.00394572; 0.008, 0.937133, -2.94589, 0.0039026; 0.0081, 0.936656, -2.98525, 0.00385834; 0.0082, 0.935953, -3.02414, 0.00381293; 0.0083, 0.935024, -3.06251, 0.00376636; 0.0084, 0.934308, -3.10824, 0.00371862; 0.0085, 0.933608, -3.15783, 0.00366967; 0.0086, 0.93269, -3.20708, 0.00361952; 0.0087, 0.931553, -3.25592, 0.00356812; 0.0088, 0.930194, -3.30427, 0.00351548; 0.0089, 0.928473, -3.35247, 0.00346157; 0.009, 0.926467, -3.40014, 0.00340636; 0.0091, 0.924232, -3.44698, 0.00334985; 0.0092, 0.921766, -3.49289, 0.00329202; 0.0093, 0.918579, -3.53879, 0.00323283; 0.0094, 0.913925, -3.5856, 0.00317229; 0.0095, 0.909004, -3.63034, 0.00311037; 0.0096, 0.903809, -3.67275, 0.00304706; 0.0097, 0.89859, -3.72881, 0.00298233; 0.0098, 0.893783, -3.82589, 0.00291616; 0.0099, 0.888707, -3.92096, 0.00284852; 0.01, 0.883343, -4.01357, 0.00277938; 0.0101, 0.876979, -4.10734, 0.00270869; 0.0102, 0.869783, -4.19987, 0.00263642; 0.0103, 0.862246, -4.28752, 0.00256254; 0.0104, 0.854574, -4.37627, 0.00248701; 0.0105, 0.847614, -4.49154, 0.00240979; 0.0106, 0.840302, -4.60102, 0.00233085; 0.0107, 0.832625, -4.70399, 0.00225014; 0.0108, 0.822938, -4.82647, 0.00216761; 0.0109, 0.812813, -4.93752, 0.00208323; 0.011, 0.802204, -5.04175, 0.00199695; 0.0111, 0.78997, -5.30274, 0.00190873; 0.0112, 0.777197, -5.54515, 0.00181846; 0.0113, 0.763521, -5.78149, 0.00172606; 0.0114, 0.748272, -6.039, 0.00163144; 0.0115, 0.73235, -6.25778, 0.0015345; 0.0116, 0.715211, -6.57852, 0.00143514; 0.0117, 0.696998, -6.91971, 0.00133326; 0.0118, 0.677065, -7.30735, 0.00122872; 0.0119, 0.652791, -7.88085, 0.00112136; 0.012, 0.62734, -8.29718, 0.00101097; 0.0121, 0.597125, -9.13179, 0.000897364; 0.0122, 0.564919, -9.82427, 0.000780251; 0.0123, 0.527838, -11.1684, 0.000659331; 0.0124, 0.487477, -12.1609, 0.000534142; 0.0125, 0.436631, -14.9103, 0.000404205; 0.0126, 0.379243, -16.2449, 0.000268616; 0.0126134, 0.371242, -16.2777, 0.00025; 0.0126134, 0.371242, -16.2777, 0.00025; 0.0126868, 0.350822, -16.2554, 0.000198624; 0.0126868, 0.350822, -16.2554, 0.000198624; 0.0127, 0.351869, -16.3218, 0.000199455; 0.0128, 0.37695, -17.0338, 0.000241587; 0.0128157, 0.381787, -17.1198, 0.00025; 0.0128157, 0.381787, -17.1198, 0.00025; 0.0129, 0.406591, -17.48, 0.000292352; 0.013, 0.433421, -17.8191, 0.000336402; 0.0131, 0.457261, -17.8337, 0.000373609; 0.0132, 0.477911, -17.6706, 0.000403962; 0.0133, 0.495294, -17.4605, 0.00042752; 0.0134, 0.509353, -17.3988, 0.000444358; 0.0135, 0.520015, -17.4878, 0.0004545; 0.0136, 0.527192, -17.7433, 0.000457911; 0.0136003, 0.527207, -17.7443, 0.000457911; 0.0136003, 0.527207, -17.7443, 0.000457911; 0.0137, 0.530748, -18.1997, 0.000454491; 0.0138, 0.530517, -18.8646, 0.000444064; 0.0139, 0.526294, -19.7142, 0.000426376; 0.014, 0.517828, -20.6871, 0.000401101; 0.0141, 0.504836, -21.6765, 0.000367869; 0.0142, 0.487037, -22.6627, 0.000326301; 0.0143, 0.464073, -23.4017, 0.000276025; 0.0143458, 0.451744, -23.5657, 0.00025; 0.0143458, 0.451744, -23.5657, 0.00025; 0.0144, 0.439383, -23.6302, 0.000223375; 0.0144518, 0.438001, -23.8106, 0.00021654; 0.0144518, 0.438001, -23.8106, 0.00021654; 0.0145, 0.442437, -24.0882, 0.000220288; 0.0146, 0.459291, -24.7355, 0.000241352; 0.014643, 0.466338, -24.9736, 0.00025; 0.014643, 0.466338, -24.9736, 0.00025; 0.0147, 0.47417, -25.2545, 0.000258795; 0.0148, 0.483493, -25.7045, 0.000266567; 0.0148288, 0.485111, -25.8323, 0.00026698; 0.0148288, 0.485111, -25.8323, 0.00026698; 0.0149, 0.486998, -26.1506, 0.000264454; 0.015, 0.484444, -26.5924, 0.000252282; 0.0150127, 0.483671, -26.6456, 0.00025; 0.0150127, 0.483671, -26.6456, 0.00025; 0.0151, 0.477935, -26.9803, 0.000233764; 0.0151954, 0.478678, -27.3825, 0.000227777; 0.0151954, 0.478678, -27.3825, 0.000227777; 0.0152, 0.478896, -27.404, 0.000227786; 0.0153, 0.486112, -27.9096, 0.000231723; 0.0154, 0.494618, -28.4114, 0.000237745; 0.0154716, 0.499054, -28.7526, 0.000239402; 0.0154716, 0.499054, -28.7526, 0.000239402; 0.0155, 0.500242, -28.8872, 0.000239151; 0.0156, 0.502893, -29.3755, 0.000235871; 0.0157, 0.505639, -29.8643, 0.000232816; 0.0158, 0.509736, -30.3772, 0.000231912; 0.0158118, 0.51029, -30.4396, 0.000231905; 0.0158118, 0.51029, -30.4396, 0.000231905; 0.0159, 0.514622, -30.9065, 0.000232198; 0.016, 0.519654, -31.4343, 0.000232755; 0.016048, 0.521947, -31.6846, 0.000232849; 0.016048, 0.521947, -31.6846, 0.000232849; 0.0161, 0.524291, -31.9527, 0.000232753; 0.0162, 0.528618, -32.4638, 0.000232328; 0.0163, 0.53296, -32.9726, 0.000231976; 0.0164, 0.537374, -33.4793, 0.000231787; 0.0165, 0.541801, -33.9827, 0.000231672; 0.0166, 0.546199, -34.4828, 0.000231561; 0.0167, 0.550555, -34.9795, 0.000231435; 0.0168, 0.554875, -35.4729, 0.0002313; 0.0169, 0.559164, -35.9631, 0.000231166; 0.017, 0.56344, -36.4518, 0.000231035; 0.0171, 0.567726, -36.9417, 0.000230906; 0.0172, 0.571982, -37.4284, 0.000230779; 0.0173, 0.576209, -37.9119, 0.000230653; 0.0174, 0.580407, -38.3923, 0.000230528; 0.0175, 0.584575, -38.8695, 0.000230405; 0.0176, 0.588716, -39.3436, 0.000230284; 0.0177, 0.593137, -39.8493, 0.000230163; 0.0178, 0.59757, -40.357, 0.000230038; 0.0179, 0.601967, -40.8716, 0.000229911; 0.018, 0.60633, -41.3953, 0.000229783; 0.0181, 0.610659, -41.9153, 0.000229654; 0.0182, 0.614955, -42.4317, 0.000229526; 0.0183, 0.619218, -42.9441, 0.0002294; 0.0184, 0.623441, -43.452, 0.000229276; 0.0185, 0.627634, -43.9562, 0.000229154; 0.0186, 0.631795, -44.4569, 0.000229034; 0.0187, 0.635926, -44.954, 0.000228915; 0.0188, 0.640026, -45.4476, 0.000228797; 0.0189, 0.644096, -45.9377, 0.000228681; 0.019, 0.648136, -46.4242, 0.000228566; 0.0191, 0.652146, -46.9074, 0.000228453; 0.0192, 0.656126, -47.387, 0.000228341; 0.0193, 0.660077, -47.8633, 0.000228231; 0.0194, 0.663999, -48.3362, 0.000228122; 0.0195, 0.667892, -48.8057, 0.000228014; 0.0196, 0.671756, -49.2718, 0.000227908; 0.0197, 0.675592, -49.7347, 0.000227802; 0.0198, 0.67979, -50.2404, 0.000227697; 0.0199, 0.684118, -50.7623, 0.000227586; 0.02, 0.688404, -51.2799, 0.000227471; 0.0201, 0.692654, -51.7933, 0.000227355; 0.0202, 0.696868, -52.3025, 0.000227241; 0.0203, 0.701047, -52.8002, 0.00022713; 0.0204, 0.705193, -53.2717, 0.000227022; 0.0205, 0.709307, -53.7394, 0.000226918; 0.0206, 0.713479, -54.2135, 0.000226817; 0.0207, 0.717635, -54.686, 0.000226716; 0.0208, 0.721755, -55.1544, 0.000226615; 0.0209, 0.725839, -55.619, 0.000226515; 0.021, 0.729888, -56.0796, 0.000226416; 0.0211, 0.733903, -56.5364, 0.000226319; 0.0212, 0.737883, -56.9893, 0.000226222; 0.0213, 0.741829, -57.4383, 0.000226127; 0.0214, 0.745732, -57.8827, 0.000226033; 0.0215, 0.749587, -58.3217, 0.000225941; 0.0216, 0.75341, -58.7569, 0.00022585; 0.0217, 0.757199, -59.1885, 0.00022576; 0.0218, 0.760956, -59.6164, 0.000225671; 0.0219, 0.764681, -60.0407, 0.000225583; 0.022, 0.768373, -60.4614, 0.000225497; 0.0221, 0.772034, -60.8786, 0.000225411; 0.0222, 0.775663, -61.2922, 0.000225326; 0.0223, 0.779579, -61.7378, 0.000225242; 0.0224, 0.784355, -62.2802, 0.000225151; 0.0225, 0.789065, -62.8168, 0.000225046; 0.0226, 0.793716, -63.3474, 0.000224938; 0.0227, 0.798315, -63.8721, 0.000224831; 0.0228, 0.802863, -64.3256, 0.000224728; 0.0229, 0.80737, -64.7356, 0.000224637; 0.023, 0.811833, -65.1406, 0.000224555; 0.0231, 0.816247, -65.541, 0.000224477; 0.0232, 0.820611, -65.9369, 0.000224399; 0.0233, 0.824909, -66.3269, 0.000224322; 0.0234, 0.829106, -66.7079, 0.000224246; 0.0235, 0.833258, -67.0845, 0.000224172; 0.0236, 0.837362, -67.457, 0.000224099; 0.0237, 0.84142, -67.8252, 0.000224027; 0.0238, 0.845433, -68.1893, 0.000223957; 0.0239, 0.8494, -68.5494, 0.000223887; 0.024, 0.853323, -68.9053, 0.000223818; 0.0241, 0.857201, -69.2573, 0.00022375; 0.0242, 0.861036, -69.6053, 0.000223683; 0.0243, 0.864828, -69.9494, 0.000223617; 0.0244, 0.868577, -70.2896, 0.000223552; 0.0245, 0.873541, -70.7381, 0.000223484; 0.0246, 0.878506, -71.1879, 0.000223404; 0.0247, 0.883389, -71.6312, 0.00022332; 0.0248, 0.888198, -72.0678, 0.000223236; 0.0249, 0.892935, -72.4978, 0.000223154; 0.025, 0.8976, -72.9212, 0.000223074; 0.0251, 0.902194, -73.2832, 0.000222997; 0.0252, 0.906729, -73.5797, 0.00022293; 0.0253, 0.911203, -73.8711, 0.000222873; 0.0254, 0.915611, -74.1579, 0.00022282; 0.0255, 0.919953, -74.4404, 0.000222768; 0.0256, 0.924227, -74.7185, 0.000222716; 0.0257, 0.928436, -74.9923, 0.000222665; 0.0258, 0.932872, -75.2805, 0.000222615; 0.0259, 0.937419, -75.5759, 0.000222563; 0.026, 0.941886, -75.8664, 0.000222509; 0.0261, 0.946276, -76.1519, 0.000222456; 0.0262, 0.950592, -76.4326, 0.000222404; 0.0263, 0.954834, -76.7084, 0.000222354; 0.0264, 0.959005, -76.9795, 0.000222304; 0.0265, 0.963104, -77.246, 0.000222255; 0.0266, 0.967134, -77.5079, 0.000222207; 0.0267, 0.971094, -77.7654, 0.00022216; 0.0268, 0.974988, -78.0184, 0.000222114; 0.0269, 0.978815, -78.2671, 0.000222068; 0.027, 0.982577, -78.5115, 0.000222024; 0.0271, 0.986275, -78.7518, 0.00022198; 0.0272, 0.98991, -78.9879, 0.000221937; 0.0273, 0.993484, -79.2201, 0.000221895; 0.0274, 0.996996, -79.4482, 0.000221854; 0.0275, 1.00082, -79.6845, 0.000221813; 0.0276, 1.00486, -79.8903, 0.000221773; 0.0277, 1.00883, -80.0919, 0.000221735; 0.0278, 1.01272, -80.2892, 0.000221699; 0.0279, 1.01653, -80.4824, 0.000221665; 0.028, 1.02026, -80.6717, 0.000221631; 0.0281, 1.02392, -80.8572, 0.000221597; 0.0282, 1.0275, -81.0389, 0.000221565; 0.0283, 1.03101, -81.2168, 0.000221533; 0.0284, 1.03445, -81.3911, 0.000221501; 0.0285, 1.03781, -81.5619, 0.000221471; 0.0286, 1.04111, -81.7292, 0.000221441; 0.0287, 1.04434, -81.893, 0.000221412; 0.0288, 1.04751, -82.0535, 0.000221383; 0.0289, 1.05061, -82.2107, 0.000221355; 0.029, 1.05365, -82.3647, 0.000221328; 0.0291, 1.05663, -82.5155, 0.000221301; 0.0292, 1.05954, -82.6633, 0.000221275; 0.0293, 1.0624, -82.808, 0.000221249; 0.0294, 1.0652, -82.9498, 0.000221224; 0.0295, 1.06794, -83.0887, 0.000221199; 0.0296, 1.07063, -83.2248, 0.000221175; 0.0297, 1.07326, -83.3581, 0.000221151; 0.0298, 1.07584, -83.4886, 0.000221128; 0.0299, 1.07836, -83.6165, 0.000221106; 0.03, 1.08088, -83.7439, 0.000221083; 0.0301, 1.08376, -83.8895, 0.000221061; 0.0302, 1.08657, -84.0316, 0.000221037; 0.0303, 1.08931, -84.1703, 0.000221012; 0.0304, 1.09198, -84.3057, 0.000220988; 0.0305, 1.09459, -84.4378, 0.000220965; 0.0306, 1.09714, -84.5667, 0.000220942; 0.0307, 1.09962, -84.6924, 0.00022092; 0.0308, 1.10205, -84.7987, 0.000220899; 0.0309, 1.10442, -84.8994, 0.00022088; 0.031, 1.10673, -84.9975, 0.000220862; 0.0311, 1.10898, -85.0932, 0.000220846; 0.0312, 1.11119, -85.1866, 0.000220829; 0.0313, 1.11333, -85.2778, 0.000220813; 0.0314, 1.11543, -85.3668, 0.000220798; 0.0315, 1.11748, -85.4536, 0.000220782; 0.0316, 1.11947, -85.5382, 0.000220768; 0.0317, 1.12142, -85.6209, 0.000220753; 0.0318, 1.12332, -85.7015, 0.000220739; 0.0319, 1.12518, -85.7802, 0.000220725; 0.032, 1.12699, -85.857, 0.000220712; 0.0321, 1.12875, -85.9319, 0.000220699; 0.0322, 1.13048, -86.005, 0.000220686; 0.0323, 1.13216, -86.0763, 0.000220673; 0.0324, 1.1338, -86.1459, 0.000220661; 0.0325, 1.1354, -86.2138, 0.000220649; 0.0326, 1.13696, -86.28, 0.000220638; 0.0327, 1.13849, -86.3447, 0.000220627; 0.0328, 1.13997, -86.4078, 0.000220616; 0.0329, 1.14143, -86.4693, 0.000220605; 0.033, 1.14284, -86.5294, 0.000220594; 0.0331, 1.14423, -86.588, 0.000220584; 0.0332, 1.14558, -86.6452, 0.000220574; 0.0333, 1.14689, -86.701, 0.000220564; 0.0334, 1.14818, -86.7555, 0.000220555; 0.0335, 1.14943, -86.8086, 0.000220546; 0.0336, 1.15065, -86.8605, 0.000220537; 0.0337, 1.15185, -86.9111, 0.000220528; 0.0338, 1.15301, -86.9605, 0.000220519; 0.0339, 1.15415, -87.0086, 0.000220511; 0.034, 1.15526, -87.0556, 0.000220503; 0.0341, 1.15634, -87.1015, 0.000220495; 0.0342, 1.1574, -87.1463, 0.000220487; 0.0343, 1.15843, -87.19, 0.000220479; 0.0344, 1.15943, -87.2326, 0.000220472; 0.0345, 1.16041, -87.2742, 0.000220465; 0.0346, 1.16137, -87.3148, 0.000220458; 0.0347, 1.16231, -87.3544, 0.000220451; 0.0348, 1.16322, -87.3931, 0.000220444; 0.0349, 1.16411, -87.4308, 0.000220438; 0.035, 1.16498, -87.4676, 0.000220431; 0.0351, 1.16582, -87.5035, 0.000220425; 0.0352, 1.16665, -87.5385, 0.000220419; 0.0353, 1.16746, -87.5727, 0.000220413; 0.0354, 1.16824, -87.6061, 0.000220407; 0.0355, 1.16901, -87.6386, 0.000220402; 0.0356, 1.16976, -87.6704, 0.000220396; 0.0357, 1.17049, -87.7014, 0.000220391; 0.0358, 1.17121, -87.7316, 0.000220386; 0.0359, 1.1719, -87.7612, 0.00022038; 0.036, 1.17258, -87.79, 0.000220375; 0.0361, 1.17325, -87.8181, 0.000220371; 0.0362, 1.1739, -87.8455, 0.000220366; 0.0363, 1.17453, -87.8722, 0.000220361; 0.0364, 1.17514, -87.8984, 0.000220357; 0.0365, 1.17574, -87.9238, 0.000220352; 0.0366, 1.17633, -87.9487, 0.000220348; 0.0367, 1.1769, -87.9729, 0.000220344; 0.0368, 1.17746, -87.9966, 0.00022034; 0.0369, 1.17801, -88.0197, 0.000220336; 0.037, 1.17858, -88.0441, 0.000220332; 0.0371, 1.17922, -88.0712, 0.000220328; 0.0372, 1.17985, -88.0975, 0.000220323; 0.0373, 1.18045, -88.123, 0.000220319; 0.0374, 1.18103, -88.1477, 0.000220314; 0.0375, 1.1816, -88.1717, 0.00022031; 0.0376, 1.18215, -88.195, 0.000220306; 0.0377, 1.18268, -88.2176, 0.000220302; 0.0378, 1.1832, -88.2395, 0.000220299; 0.0379, 1.1837, -88.2607, 0.000220295; 0.038, 1.18419, -88.2814, 0.000220291; 0.0381, 1.18466, -88.3014, 0.000220288; 0.0382, 1.18512, -88.3208, 0.000220284; 0.0383, 1.18556, -88.3396, 0.000220281; 0.0384, 1.18599, -88.3578, 0.000220278; 0.0385, 1.18641, -88.3756, 0.000220275; 0.0386, 1.18682, -88.3928, 0.000220272; 0.0387, 1.18721, -88.4094, 0.000220269; 0.0388, 1.18759, -88.4256, 0.000220266; 0.0389, 1.18796, -88.4413, 0.000220264; 0.039, 1.18832, -88.4565, 0.000220261; 0.0391, 1.18867, -88.4713, 0.000220258; 0.0392, 1.18901, -88.4856, 0.000220256; 0.0393, 1.18934, -88.4995, 0.000220253; 0.0394, 1.18965, -88.513, 0.000220251; 0.0395, 1.18996, -88.5261, 0.000220249; 0.0396, 1.19026, -88.5388, 0.000220247; 0.0397, 1.19055, -88.5511, 0.000220245; 0.0398, 1.19084, -88.563, 0.000220242; 0.0399, 1.19111, -88.5746, 0.00022024; 0.04, 1.19137, -88.5859, 0.000220239; 0.0401, 1.19163, -88.5968, 0.000220237; 0.0402, 1.19188, -88.6074, 0.000220235; 0.0403, 1.19212, -88.6176, 0.000220233; 0.0404, 1.19236, -88.6276, 0.000220231; 0.0405, 1.19259, -88.6373, 0.00022023; 0.0406, 1.19281, -88.6466, 0.000220228; 0.0407, 1.19302, -88.6557, 0.000220226; 0.0408, 1.19323, -88.6646, 0.000220225; 0.0409, 1.19343, -88.6731, 0.000220223; 0.041, 1.19363, -88.6814, 0.000220222; 0.0411, 1.19382, -88.6895, 0.000220221; 0.0412, 1.19401, -88.6973, 0.000220219; 0.0413, 1.19418, -88.7049, 0.000220218; 0.0414, 1.19436, -88.7122, 0.000220217; 0.0415, 1.19453, -88.7194, 0.000220215; 0.0416, 1.19469, -88.7263, 0.000220214; 0.0417, 1.19485, -88.733, 0.000220213; 0.0418, 1.195, -88.7395, 0.000220212; 0.0419, 1.19515, -88.7459, 0.000220211; 0.042, 1.1953, -88.752, 0.00022021; 0.0421, 1.19544, -88.7579, 0.000220209; 0.0422, 1.19557, -88.7637, 0.000220208; 0.0423, 1.19571, -88.7693, 0.000220207; 0.0424, 1.19583, -88.7747, 0.000220206; 0.0425, 1.19596, -88.78, 0.000220205; 0.0426, 1.19608, -88.7851, 0.000220204; 0.0427, 1.1962, -88.7901, 0.000220203; 0.0428, 1.19631, -88.7949, 0.000220202; 0.0429, 1.19642, -88.7996, 0.000220202; 0.043, 1.19653, -88.8041, 0.000220201; 0.0431, 1.19663, -88.8085, 0.0002202; 0.0432, 1.19673, -88.8127, 0.000220199; 0.0433, 1.19683, -88.8169, 0.000220199; 0.0434, 1.19692, -88.8209, 0.000220198; 0.0435, 1.19702, -88.8248, 0.000220197; 0.0436, 1.1971, -88.8286, 0.000220197; 0.0437, 1.19719, -88.8322, 0.000220196; 0.0438, 1.19728, -88.8358, 0.000220195; 0.0439, 1.19736, -88.8392, 0.000220195; 0.044, 1.19744, -88.8426, 0.000220194; 0.0441, 1.19751, -88.8458, 0.000220194; 0.0442, 1.19759, -88.8489, 0.000220193; 0.0443, 1.19766, -88.852, 0.000220192; 0.0444, 1.19773, -88.855, 0.000220192; 0.0445, 1.1978, -88.8578, 0.000220191; 0.0446, 1.19786, -88.8606, 0.000220191; 0.0447, 1.19793, -88.8633, 0.000220191; 0.0448, 1.19799, -88.8659, 0.00022019; 0.0449, 1.19805, -88.8685, 0.00022019; 0.045, 1.19811, -88.871, 0.000220189; 0.0451, 1.19816, -88.8734, 0.000220189; 0.0452, 1.19822, -88.8757, 0.000220188; 0.0453, 1.19827, -88.8779, 0.000220188; 0.0454, 1.19832, -88.8801, 0.000220188; 0.0455, 1.19837, -88.8822, 0.000220187; 0.0456, 1.19842, -88.8843, 0.000220187; 0.0457, 1.19847, -88.8863, 0.000220187; 0.0458, 1.19851, -88.8882, 0.000220186; 0.0459, 1.19856, -88.8901, 0.000220186; 0.046, 1.1986, -88.8919, 0.000220186; 0.0461, 1.19864, -88.8937, 0.000220185; 0.0462, 1.19868, -88.8954, 0.000220185; 0.0463, 1.19872, -88.897, 0.000220185; 0.0464, 1.19876, -88.8987, 0.000220184; 0.0465, 1.1988, -88.9002, 0.000220184; 0.0466, 1.19883, -88.9017, 0.000220184; 0.0467, 1.19887, -88.9032, 0.000220184; 0.0468, 1.1989, -88.9046, 0.000220183; 0.0469, 1.19893, -88.906, 0.000220183; 0.047, 1.19897, -88.9074, 0.000220183; 0.0471, 1.199, -88.9087, 0.000220183; 0.0472, 1.19903, -88.91, 0.000220182; 0.0473, 1.19906, -88.9112, 0.000220182; 0.0474, 1.19908, -88.9124, 0.000220182; 0.0475, 1.19911, -88.9135, 0.000220182; 0.0476, 1.19914, -88.9146, 0.000220182; 0.0477, 1.19916, -88.9157, 0.000220181; 0.0478, 1.19919, -88.9168, 0.000220181; 0.0479, 1.19921, -88.9178, 0.000220181; 0.048, 1.19924, -88.9188, 0.000220181; 0.0481, 1.19926, -88.9198, 0.000220181; 0.0482, 1.19928, -88.9207, 0.000220181; 0.0483, 1.1993, -88.9216, 0.00022018; 0.0484, 1.19932, -88.9225, 0.00022018; 0.0485, 1.19934, -88.9233, 0.00022018; 0.0486, 1.19936, -88.9242, 0.00022018; 0.0487, 1.19938, -88.925, 0.00022018; 0.0488, 1.1994, -88.9258, 0.00022018; 0.0489, 1.19942, -88.9265, 0.00022018; 0.049, 1.19944, -88.9273, 0.000220179; 0.0491, 1.19945, -88.928, 0.000220179; 0.0492, 1.19947, -88.9287, 0.000220179; 0.0493, 1.19948, -88.9293, 0.000220179; 0.0494, 1.1995, -88.93, 0.000220179; 0.0495, 1.19951, -88.9306, 0.000220179; 0.0496, 1.19953, -88.9312, 0.000220179; 0.0497, 1.19954, -88.9318, 0.000220179; 0.0498, 1.19956, -88.9324, 0.000220179; 0.0499, 1.19957, -88.933, 0.000220178; 0.05, 1.19958, -88.9335, 0.000220178; 0.05, 1.19958, -88.9335, 0.000220178], tableOnFile = false, columns = 2:4, extrapolation = Modelica.Blocks.Types.Extrapolation.HoldLastPoint) "Valid for u_source=12VDC and m_load=0.01kg only; column 2: current, col.3: force, col.4: position" annotation(
              Placement(transformation(extent = {{-40, 60}, {-20, 80}}, rotation = 0)));
          equation
            connect(advancedGround.p, advancedSource.n) annotation(
              Line(points = {{-60, 10}, {-60, 20}}, color = {0, 0, 255}));
            connect(advancedSource.p, advancedSolenoid.p) annotation(
              Line(points = {{-60, 40}, {-40, 40}, {-40, 36}, {-20, 36}}, color = {0, 0, 255}));
            connect(advancedSolenoid.n, advancedSource.n) annotation(
              Line(points = {{-20, 24}, {-40, 24}, {-40, 20}, {-60, 20}}, color = {0, 0, 255}));
            connect(advancedSolenoid.flange, advancedLoad.flange_a) annotation(
              Line(points = {{0, 30}, {20, 30}}, color = {0, 127, 0}));
            connect(simpleGround.p, simpleSource.n) annotation(
              Line(points = {{-60, -70}, {-60, -60}}, color = {0, 0, 255}));
            connect(simpleSource.p, simpleSolenoid.p) annotation(
              Line(points = {{-60, -40}, {-40, -40}, {-40, -44}, {-20, -44}}, color = {0, 0, 255}));
            connect(simpleSolenoid.n, simpleSource.n) annotation(
              Line(points = {{-20, -56}, {-40, -56}, {-40, -60}, {-60, -60}}, color = {0, 0, 255}));
            connect(simpleSolenoid.flange, simpleLoad.flange_a) annotation(
              Line(points = {{0, -50}, {20, -50}}, color = {0, 127, 0}));
            annotation(
              experiment(StopTime = 0.05, Tolerance = 1e-007),
              Documentation(info = "<html>
<p>
Have a look at <a href=\"modelica://Modelica.Magnetic.FluxTubes.Examples.SolenoidActuator\">SolenoidActuator</a> for general comments and at <a href=\"modelica://Modelica.Magnetic.FluxTubes.Examples.SolenoidActuator.Components.SimpleSolenoid\">SimpleSolenoid</a> and <a href=\"modelica://Modelica.Magnetic.FluxTubes.Examples.SolenoidActuator.Components.AdvancedSolenoid\">AdvancedSolenoid</a> for a detailed description of both magnetic network models.
</p>

<p>
A voltage step is applied to both solenoid models at time t=0. The armatures of both models and therewith connected loads are pulled from their rest position at maximum air gap length to their minimum position that is due to a stopper. As a reference, simulation results obtained with a dynamic model based on stationary FEA look-up tables (not part of this library) are included. Note that these reference results are valid for the default supply voltage v_step=12V DC and the default load mass m_load=0.01kg only!
</p>

<p>
Set the <b>tolerance</b> to <b>1e-7</b> and <b>simulate for 0.05 s</b>. Plot actuator current, force and position of the two magnetic network models and the FEA-based reference <b>vs. time</b> (each quantity in one common plot window):
</p>

<pre>
Plot window for current:
    simpleSolenoid.p.i          // rapid current rise indicates low inductance of simple network model
    advancedSolenoid.p.i        // current rise slower, better match with FEA reference
    comparisonWithFEA.y[1]      // current obtained from dynamic model based on stationary FEA look-up tables

Plot window for force:
    simpleSolenoid.armature.flange_a.f       // reluctance force of simple actuator model
    advancedSolenoid.armature.flange_a.f     // reluctance force of advanced actuator model
    comparisonWithFEA.y[2]      // force obtained from dynamic model based on stationary FEA look-up tables

Plot window for position:
    simpleSolenoid.x            // armature position of simple actuator model
    advancedSolenoid.x          // armature position of advanced actuator model
    comparisonWithFEA.y[3]      // position obtained from dynamic model based on stationary FEA look-up tables
</pre>

<p>
The characteristic current drop during pull-in is due to both armature motion and increasing inductance with decreasing air gap length. Bouncing occurs when  armature and load of each model arrive at the stopper at minimum position. Although the pull-in times of the two magnetic network models are relatively close to the time obtained with the reference model, the accuracy of the advanced solenoid model is better, as one can tell from a comparison of the current rise at the beginning of the stroke.
</p>
</html>"));
          end ComparisonPullInStroke;

          package Components "Components to be used in examples"
            extends Modelica.Icons.Package;

            model SimpleSolenoid "Simple network model of a lifting magnet with planar armature end face"
              parameter SI.Resistance R = 10 "Armature coil resistance";
              parameter Real N = 957 "Number of turns";
              //yoke
              parameter SI.Radius r_yokeOut = 15e-3 "Outer yoke radius" annotation(
                Dialog(group = "Parameters", groupImage = "modelica://Modelica/Resources/Images/Magnetic/FluxTubes/Examples/SolenoidActuator/Solenoid_dimensions.png"));
              parameter SI.Radius r_yokeIn = 13.5e-3 "Inner yoke radius";
              parameter SI.Length l_yoke = 35e-3 "Axial yoke length";
              parameter SI.Length t_yokeBot = 3.5e-3 "Axial thickness of yoke bottom";
              //pole
              parameter SI.Length l_pole = 6.5e-3 "Axial length of pole";
              parameter SI.Length t_poleBot = 3.5e-3 "Axial thickness of bottom at pole side";
              parameter SI.Length t_airPar = 0.65e-3 "Radial thickness of parasitic air gap due to slide guiding";
              parameter Modelica.Magnetic.FluxTubes.Material.SoftMagnetic.BaseData material = Modelica.Magnetic.FluxTubes.Material.SoftMagnetic.Steel.Steel_9SMnPb28() "Ferromagnetic material characteristics" annotation(
                choicesAllMatching = true,
                Dialog(group = "Material"));
              //armature
              parameter SI.Radius r_arm = 5e-3 "Armature radius = pole radius" annotation(
                Dialog(group = "Armature and stopper"));
              parameter SI.Length l_arm = 26e-3 "Armature length" annotation(
                Dialog(group = "Armature and stopper"));
              parameter Modelica.SIunits.TranslationalSpringConstant c = 1e11 "Spring stiffness between impact partners" annotation(
                Dialog(group = "Armature and stopper"));
              parameter Modelica.SIunits.TranslationalDampingConstant d = 400 "Damping coefficient between impact partners" annotation(
                Dialog(group = "Armature and stopper"));
              parameter SI.Position x_min = 0.25e-3 "Stopper at minimum armature position" annotation(
                Dialog(group = "Armature and stopper"));
              parameter SI.Position x_max = 5e-3 "Stopper at maximum armature position" annotation(
                Dialog(group = "Armature and stopper"));
              SI.Position x(start = x_max, stateSelect = StateSelect.prefer) "Armature position, alias for flange position (identical with length of working air gap)";
            protected
              parameter SI.Density rho_steel = 7853 "Density for calculation of armature mass from geometry";
            public
              Modelica.Magnetic.FluxTubes.Basic.Ground ground annotation(
                Placement(transformation(extent = {{50, 10}, {70, 30}}, rotation = 0)));
              Modelica.Magnetic.FluxTubes.Basic.ElectroMagneticConverter coil(final N = N, i(fixed = true)) "Electro-magnetic converter" annotation(
                Placement(transformation(origin = {0, 20}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
              Modelica.Electrical.Analog.Basic.Resistor r(final R = R) "Coil resistance" annotation(
                Placement(transformation(extent = {{-70, -30}, {-50, -10}}, rotation = 0)));
              Modelica.Magnetic.FluxTubes.Shapes.FixedShape.HollowCylinderAxialFlux g_mFeYokeSide(final nonLinearPermeability = true, final material = material, final l = l_yoke - (t_poleBot + t_yokeBot) / 2, final r_i = r_yokeIn, final r_o = r_yokeOut) "Permeance of of hollow cylindric section of ferromagnetic yoke" annotation(
                Placement(transformation(extent = {{-10, 80}, {10, 100}}, rotation = 0)));
              Modelica.Magnetic.FluxTubes.Shapes.FixedShape.HollowCylinderAxialFlux g_mFeArm(final nonLinearPermeability = true, final material = material, final l = l_yoke - (t_yokeBot + t_poleBot) / 2 - l_pole - (x_max + x_min) / 2, final r_i = 0, final r_o = r_arm) "Permeance of ferromagnetic armature" annotation(
                Placement(transformation(origin = {16, 40}, extent = {{-10, -10}, {10, 10}}, rotation = 180)));
              Modelica.Magnetic.FluxTubes.Shapes.Force.HollowCylinderAxialFlux g_mAirWork(final mu_r = 1, final dlBydx = 1, final r_i = 0, final r_o = r_arm, final useSupport = false, final l = flange.s) "Permeance of working air gap (between armature and pole end faces)" annotation(
                Placement(transformation(origin = {-30, 30}, extent = {{-10, 10}, {10, -10}}, rotation = 180)));
              Modelica.Magnetic.FluxTubes.Shapes.FixedShape.HollowCylinderRadialFlux g_mFeYokeBot(final nonLinearPermeability = true, final material = material, final l = t_yokeBot, final r_i = r_arm + t_airPar, final r_o = r_yokeIn) "Permeance of bottom side of ferromagnetic yoke" annotation(
                Placement(transformation(origin = {80, 80}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
              Modelica.Magnetic.FluxTubes.Shapes.FixedShape.HollowCylinderRadialFlux g_mAirPar(final nonLinearPermeability = false, final mu_rConst = 1, final l = t_yokeBot, final r_i = r_arm, final r_o = r_arm + t_airPar) "Permeance of parasitic radial air gap due to slide guiding" annotation(
                Placement(transformation(origin = {80, 50}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
              Modelica.Magnetic.FluxTubes.Shapes.FixedShape.HollowCylinderRadialFlux g_mFePoleBot(final l = t_poleBot, final r_i = r_arm, final r_o = r_yokeIn, final nonLinearPermeability = true, final material = material) "Permeance of bottom side of pole" annotation(
                Placement(transformation(origin = {-72, 80}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
              Modelica.Magnetic.FluxTubes.Shapes.FixedShape.HollowCylinderAxialFlux g_mFePole(final nonLinearPermeability = true, final material = material, final l = l_pole, final r_i = 0, final r_o = r_arm) "Permeance of ferromagnetic pole" annotation(
                Placement(transformation(origin = {-72, 40}, extent = {{10, -10}, {-10, 10}}, rotation = 270)));
              Modelica.Magnetic.FluxTubes.Examples.Utilities.TranslatoryArmatureAndStopper armature(final m = rho_steel * l_arm * pi * r_arm ^ 2, final x_max = x_max, final x_min = x_min, final L = 0, final c = c, final d = d, n = 2, v(fixed = true)) "Inertia of armature and stoppers at end of stroke range" annotation(
                Placement(transformation(extent = {{64, -10}, {84, 10}}, rotation = 0)));
              Modelica.Magnetic.FluxTubes.Shapes.Leakage.QuarterCylinder g_mLeak1(l = 2 * pi * (r_arm + t_airPar / 2)) "Leakage permeance between inner edge of yoke bore and armature side face" annotation(
                Placement(transformation(origin = {60, 50}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
              Modelica.Magnetic.FluxTubes.Shapes.Leakage.QuarterHollowCylinder g_mLeak2(final l = 2 * pi * r_arm, final ratio = 8) "Leakage permeance between inner side of yoke bottom and armature side (r_i = t_airPar)" annotation(
                Placement(transformation(origin = {40, 50}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
              Modelica.Magnetic.FluxTubes.Shapes.Force.LeakageAroundPoles g_mLeakWork(final mu_r = 1, final dlBydx = 1, final w = 2 * pi * (r_arm + 0.0015), final r = 0.003, final l = flange.s, final useSupport = false) "Permeance of leakage air gap around working air gap (between armature and pole side faces)" annotation(
                Placement(transformation(origin = {-30, 70}, extent = {{-10, 10}, {10, -10}}, rotation = 180)));
              Modelica.Electrical.Analog.Interfaces.PositivePin p "Electrical connector" annotation(
                Placement(transformation(extent = {{-110, 50}, {-90, 70}}, rotation = 0)));
              Modelica.Electrical.Analog.Interfaces.NegativePin n "Electrical connector" annotation(
                Placement(transformation(extent = {{-90, -70}, {-110, -50}}, rotation = 0)));
              Modelica.Mechanics.Translational.Interfaces.Flange_b flange "Flange of component" annotation(
                Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
            equation
              x = flange.s;
              connect(r.p, p) annotation(
                Line(points = {{-70, -20}, {-90, -20}, {-90, 60}, {-100, 60}}, color = {0, 0, 255}));
              connect(armature.flange_b, flange) annotation(
                Line(points = {{84, 0}, {88, 0}, {92, 0}, {100, 0}}, color = {0, 127, 0}));
              connect(armature.flange_a, g_mAirWork.flange) annotation(
                Line(points = {{64, 0}, {34, 0}, {34, 52}, {-30, 52}, {-30, 40}}, color = {0, 127, 0}));
              connect(g_mAirWork.flange, g_mLeakWork.flange) annotation(
                Line(points = {{-30, 40}, {-30, 52}, {34, 52}, {34, 80}, {-30, 80}}, color = {0, 127, 0}));
              connect(r.n, coil.p) annotation(
                Line(points = {{-50, -20}, {-6, -20}, {-6, 10}}, color = {0, 0, 255}));
              connect(coil.n, n) annotation(
                Line(points = {{6, 10}, {6, -60}, {-100, -60}}, color = {0, 0, 255}));
              connect(coil.port_p, g_mAirWork.port_p) annotation(
                Line(points = {{-6, 30}, {-20, 30}}, color = {255, 127, 0}));
              connect(g_mAirWork.port_p, g_mLeakWork.port_p) annotation(
                Line(points = {{-20, 30}, {-20, 70}}, color = {255, 127, 0}));
              connect(g_mAirWork.port_n, g_mLeakWork.port_n) annotation(
                Line(points = {{-40, 30}, {-40, 70}}, color = {255, 127, 0}));
              connect(g_mFePole.port_p, g_mAirWork.port_n) annotation(
                Line(points = {{-72, 30}, {-40, 30}}, color = {255, 127, 0}));
              connect(g_mFePoleBot.port_p, g_mFePole.port_n) annotation(
                Line(points = {{-72, 70}, {-72, 50}}, color = {255, 127, 0}));
              connect(g_mFePoleBot.port_n, g_mFeYokeSide.port_p) annotation(
                Line(points = {{-72, 90}, {-10, 90}}, color = {255, 127, 0}));
              connect(g_mFeYokeSide.port_n, g_mFeYokeBot.port_n) annotation(
                Line(points = {{10, 90}, {80, 90}}, color = {255, 127, 0}));
              connect(g_mFeYokeBot.port_p, g_mAirPar.port_n) annotation(
                Line(points = {{80, 70}, {80, 60}}, color = {255, 127, 0}));
              connect(g_mFeArm.port_p, g_mLeak2.port_p) annotation(
                Line(points = {{26, 40}, {40, 40}}, color = {255, 127, 0}));
              connect(g_mLeak2.port_p, g_mLeak1.port_p) annotation(
                Line(points = {{40, 40}, {60, 40}}, color = {255, 127, 0}));
              connect(g_mLeak1.port_p, g_mAirPar.port_p) annotation(
                Line(points = {{60, 40}, {80, 40}}, color = {255, 127, 0}));
              connect(g_mLeak2.port_n, g_mLeak1.port_n) annotation(
                Line(points = {{40, 60}, {60, 60}}, color = {255, 127, 0}));
              connect(g_mLeak1.port_n, g_mAirPar.port_n) annotation(
                Line(points = {{60, 60}, {80, 60}}, color = {255, 127, 0}));
              connect(g_mFeArm.port_n, coil.port_n) annotation(
                Line(points = {{6, 40}, {6, 35}, {6, 30}}, color = {255, 127, 0}));
              connect(ground.port, g_mLeak1.port_p) annotation(
                Line(points = {{60, 30}, {60, 40}}, color = {255, 127, 0}, smooth = Smooth.None));
              annotation(
                Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-90, 100}, {90, -100}}, lineColor = {255, 255, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Rectangle(extent = {{90, -30}, {-4, 30}}, lineColor = {255, 128, 0}, fillColor = {255, 128, 0}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-40, -30}, {-90, 30}}, lineColor = {255, 128, 0}, fillColor = {255, 128, 0}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-80, -100}, {-90, 100}}, lineColor = {255, 128, 0}, fillColor = {255, 128, 0}, fillPattern = FillPattern.Solid), Rectangle(extent = {{90, 90}, {-90, 100}}, lineColor = {255, 128, 0}, fillColor = {255, 128, 0}, fillPattern = FillPattern.Solid), Rectangle(extent = {{90, -100}, {-90, -90}}, lineColor = {255, 128, 0}, fillColor = {255, 128, 0}, fillPattern = FillPattern.Solid), Rectangle(extent = {{90, 40}, {80, 100}}, lineColor = {255, 128, 0}, fillColor = {255, 128, 0}, fillPattern = FillPattern.Solid), Rectangle(extent = {{90, -100}, {80, -40}}, lineColor = {255, 128, 0}, fillColor = {255, 128, 0}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-70, 80}, {70, 40}}, lineColor = {255, 213, 170}, fillColor = {255, 213, 170}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-70, -40}, {70, -80}}, lineColor = {255, 213, 170}, fillColor = {255, 213, 170}, fillPattern = FillPattern.Solid), Text(extent = {{0, 160}, {0, 120}}, lineColor = {0, 0, 255}, textString = "%name")}),
                Documentation(info = "<html>
<p>
Please refer to the <b>Parameters</b> section for a schematic drawing of this axisymmetric lifting magnet.
In the half-section below, the flux tube elements of the actuator's magnetic circuit are superimposed on a field plot obtained with FEA. The magnetomotive force imposed by the coil is modelled as one lumped element. As a result, the radial leakage flux between armature and yoke that occurs especially at large working air gaps can not be considered properly. This leads to a a higher total reluctance and lower inductance respectively compared to FEA for large working air gaps (i.e., armature close to x_max). Please have a look at the comments associated with the individual model components for a short explanation of their purpose in the model.
</p>

<p>
<IMG src=\"modelica://Modelica/Resources/Images/Magnetic/FluxTubes/Examples/SolenoidActuator/SimpleSolenoidModel_fluxTubePartitioning.png\" ALT=\"Field lines and assigned flux tubes of the simple solenoid model\">
</p>

<p>
The coupling coefficient c_coupl in the coil is set to 1 in this example, since leakage flux is accounted for explicitly with the flux tube element G_mLeakWork. Although this leakage model is rather simple, it describes the reluctance force due to the leakage field sufficiently, especially at large air gaps. With decreasing air gap length, the influence of the leakage flux on the actuator's net reluctance force decreases due to the increasing influence of the main working air gap G_mAirWork.
</p>

<p>
During model-based actuator design, the radii and lengths of the flux tube elements (and hence their cross-sectional areas and flux densities) should be assigned with parametric equations so that common design rules are met (e.g., allowed flux density in ferromagnetic parts, allowed current density and required cross-sectional area of winding). For simplicity, those equations are omitted in the example. Instead, the found values are assigned to the model elements directly.
</p>
</html>"));
            end SimpleSolenoid;

            model AdvancedSolenoid "Advanced network model of a lifting magnet with planar armature end face, split magnetomotive force"
              parameter Real N = 957 "Number of turns" annotation(
                Dialog(group = "Parameters", groupImage = "modelica://Modelica/Resources/Images/Magnetic/FluxTubes/Examples/SolenoidActuator/Solenoid_dimensions.png"));
              parameter SI.Resistance R = 5 "Coil resistance";
              parameter SI.Resistance R_par = 1e5 "Resistance parallel to the coil, in series to C_par";
              parameter SI.Capacitance C_par = 1e-9 "Capacitance parallel to the coil, in series to R_par";
              //yoke
              parameter SI.Radius r_yokeOut = 15e-3 "Outer yoke radius";
              parameter SI.Radius r_yokeIn = 13.5e-3 "Inner yoke radius";
              parameter SI.Length l_yoke = 35e-3 "Axial yoke length";
              parameter SI.Length t_yokeBot = 3.5e-3 "Axial thickness of yoke bottom";
              //pole
              parameter SI.Length l_pole = 6.5e-3 "Axial length of pole";
              parameter SI.Length t_poleBot = 3.5e-3 "Axial thickness of bottom at pole side";
              parameter SI.Length t_airPar = 0.65e-3 "Radial thickness of parasitic air gap due to slide guiding";
              parameter Modelica.Magnetic.FluxTubes.Material.SoftMagnetic.BaseData material = Modelica.Magnetic.FluxTubes.Material.SoftMagnetic.Steel.Steel_9SMnPb28() "Ferromagnetic material characteristics" annotation(
                choicesAllMatching = true,
                Dialog(group = "Material"));
              //armature
              parameter SI.Radius r_arm = 5e-3 "Armature radius = pole radius" annotation(
                Dialog(group = "Armature and stopper"));
              parameter SI.Length l_arm = 26e-3 "Armature length" annotation(
                Dialog(group = "Armature and stopper"));
              parameter Modelica.SIunits.TranslationalSpringConstant c = 1e11 "Spring stiffness between impact partners" annotation(
                Dialog(group = "Armature and stopper"));
              parameter Modelica.SIunits.TranslationalDampingConstant d = 400 "Damping coefficient between impact partners" annotation(
                Dialog(group = "Armature and stopper"));
              parameter SI.Position x_min = 0.25e-3 "Stopper at minimum armature position" annotation(
                Dialog(group = "Armature and stopper"));
              parameter SI.Position x_max = 5e-3 "Stopper at maximum armature position" annotation(
                Dialog(group = "Armature and stopper"));
              SI.Position x(start = x_max, stateSelect = StateSelect.prefer) "Armature position";
              SI.MagneticFlux Psi_tot "Total flux linkage for information only";
              SI.Inductance L_statTot "Total static inductance for information only";
            protected
              parameter SI.Density rho_steel = 7853 "Density for calculation of armature mass from geometry";
            public
              Modelica.Magnetic.FluxTubes.Basic.Ground ground annotation(
                Placement(transformation(extent = {{42, 2}, {62, 22}}, rotation = 0)));
              Modelica.Magnetic.FluxTubes.Basic.ElectroMagneticConverter coil1(final N = N / 2, i(fixed = true)) "Electro-magnetic conversion in first half of coil" annotation(
                Placement(transformation(origin = {-46, 20}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
              Modelica.Electrical.Analog.Basic.Resistor r_1(final R = R) "Resistance of first half of coil" annotation(
                Placement(transformation(extent = {{-84, -30}, {-64, -10}}, rotation = 0)));
              Modelica.Magnetic.FluxTubes.Shapes.FixedShape.HollowCylinderAxialFlux g_mFeYokeSide1(final l = l_yoke / 2 - t_poleBot / 2, final r_i = r_yokeIn, final r_o = r_yokeOut, final nonLinearPermeability = true, final material = material) "Permeance of of first half of yoke's hollow cylindric section" annotation(
                Placement(transformation(extent = {{-50, 70}, {-30, 90}}, rotation = 0)));
              Modelica.Magnetic.FluxTubes.Shapes.FixedShape.HollowCylinderAxialFlux g_mFeArm(final r_i = 0, final l = l_yoke - (t_yokeBot + t_poleBot) / 2 - l_pole - (x_max + x_min) / 2, final r_o = r_arm, final nonLinearPermeability = true, final material = material) "Permeance of ferromagnetic armature" annotation(
                Placement(transformation(origin = {10, 30}, extent = {{-10, 10}, {10, -10}}, rotation = 180)));
              Modelica.Magnetic.FluxTubes.Shapes.Force.HollowCylinderAxialFlux g_mAirWork(final r_o = r_arm, final useSupport = false, final mu_r = 1, final dlBydx = 1, final r_i = 0, final l = flange.s) "Permeance of working air gap (between armature and pole end faces)" annotation(
                Placement(transformation(origin = {-20, 30}, extent = {{-10, 10}, {10, -10}}, rotation = 180)));
              Modelica.Magnetic.FluxTubes.Shapes.FixedShape.HollowCylinderRadialFlux g_mFeYokeBot(final l = t_yokeBot, final r_i = r_arm + t_airPar, final r_o = r_yokeIn, final nonLinearPermeability = true, final material = material) "Permeance of bottom side of ferromagnetic yoke" annotation(
                Placement(transformation(origin = {74, 70}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
              Modelica.Magnetic.FluxTubes.Shapes.FixedShape.HollowCylinderRadialFlux g_mAirPar(final l = t_yokeBot, final r_i = r_arm, final r_o = r_arm + t_airPar, final nonLinearPermeability = false, final mu_rConst = 1) "Permeance of parasitic radial air gap due to slide guiding" annotation(
                Placement(transformation(origin = {74, 40}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
              Modelica.Magnetic.FluxTubes.Shapes.FixedShape.HollowCylinderRadialFlux g_mFePoleBot(final l = t_poleBot, final r_i = r_arm, final r_o = r_yokeIn, final nonLinearPermeability = true, final material = material) "Permeance of bottom side of pole" annotation(
                Placement(transformation(origin = {-78, 56}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
              Modelica.Magnetic.FluxTubes.Shapes.FixedShape.HollowCylinderAxialFlux g_mFePole(final l = l_pole, final r_o = r_arm, final nonLinearPermeability = true, final material = material, r_i = 0) "Permeance of ferromagnetic pole" annotation(
                Placement(transformation(origin = {-68, 30}, extent = {{-10, 10}, {10, -10}}, rotation = 180)));
              Modelica.Magnetic.FluxTubes.Examples.Utilities.TranslatoryArmatureAndStopper armature(final x_max = x_max, final x_min = x_min, final m = rho_steel * l_arm * pi * r_arm ^ 2, final L = 0, final c = c, final d = d, n = 2, v(fixed = true)) "Inertia of armature and stoppers at end of stroke range" annotation(
                Placement(transformation(extent = {{64, -10}, {84, 10}}, rotation = 0)));
              Modelica.Magnetic.FluxTubes.Shapes.Leakage.QuarterCylinder g_mLeak1(final l = 2 * pi * (r_arm + t_airPar / 2)) "Leakage permeance between inner edge of yoke bore and armature side face" annotation(
                Placement(transformation(origin = {60, 40}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
              Modelica.Magnetic.FluxTubes.Shapes.Leakage.QuarterHollowCylinder g_mLeak2(final ratio = 8, final l = 2 * pi * r_arm) "Leakage permeance between inner side of yoke bottom and armature side (r_i = t_airPar)" annotation(
                Placement(transformation(origin = {46, 40}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
              Modelica.Magnetic.FluxTubes.Basic.ElectroMagneticConverter coil2(final N = N / 2, i(fixed = true)) "Electro-magnetic conversion in first half of coil" annotation(
                Placement(transformation(origin = {30, 20}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
              Modelica.Electrical.Analog.Basic.Capacitor c_par1(final C = C_par, v(start = 0, fixed = true)) "Parasitic capacitance assigned to first half of coil" annotation(
                Placement(transformation(extent = {{-60, -50}, {-40, -30}}, rotation = 0)));
              Modelica.Magnetic.FluxTubes.Shapes.FixedShape.HollowCylinderRadialFlux G_mLeakRad(final mu_rConst = 1, final r_i = r_arm, final r_o = r_yokeIn, final l = l_yoke / 4, final nonLinearPermeability = false) "Permeance of radial leakage flux tube between armature side and yoke side" annotation(
                Placement(transformation(origin = {0, 56}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
              Modelica.Magnetic.FluxTubes.Shapes.FixedShape.HollowCylinderAxialFlux g_mFeYokeSide2(final l = l_yoke / 2 - t_yokeBot / 2, final r_i = r_yokeIn, r_o = r_yokeOut, final nonLinearPermeability = true, final material = material) "Permeance of of second half of yoke's hollow cylindric section" annotation(
                Placement(transformation(extent = {{20, 70}, {40, 90}}, rotation = 0)));
              Modelica.Electrical.Analog.Basic.Capacitor c_par2(final C = C_par, v(start = 0, fixed = true)) "Parasitic capacitance assigned to second half of coil" annotation(
                Placement(transformation(extent = {{16, -50}, {36, -30}}, rotation = 0)));
              Modelica.Electrical.Analog.Basic.Resistor r_par1(final R = R_par) "Parasitic resistance assigned to first half of coil" annotation(
                Placement(transformation(extent = {{-84, -50}, {-64, -30}}, rotation = 0)));
              Modelica.Electrical.Analog.Basic.Resistor r_par2(final R = R_par) "Parasitic resistance assigned to second half of coil" annotation(
                Placement(transformation(extent = {{-8, -50}, {12, -30}}, rotation = 0)));
              Modelica.Electrical.Analog.Basic.Resistor r_2(final R = R) "Resistance of second half of coil" annotation(
                Placement(transformation(extent = {{-8, -30}, {12, -10}}, rotation = 0)));
              Modelica.Magnetic.FluxTubes.Shapes.Leakage.QuarterCylinder g_mLeak3(final l = 2 * pi * (r_arm + t_airPar / 2)) "Leakage permeance between outer edge of yoke bore and armature side face" annotation(
                Placement(transformation(origin = {88, 40}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
              Modelica.Magnetic.FluxTubes.Shapes.Force.LeakageAroundPoles g_mLeakWork(final w = 2 * pi * (r_arm + 0.0015), final r = 0.003, final useSupport = false, final mu_r = 1, final dlBydx = 1, final l = flange.s) "Permeance of leakage air gap around working air gap (between armature and pole side faces)" annotation(
                Placement(transformation(origin = {-20, 64}, extent = {{-10, 10}, {10, -10}}, rotation = 180)));
              Modelica.Electrical.Analog.Interfaces.PositivePin p "Electrical connector" annotation(
                Placement(transformation(extent = {{-110, 50}, {-90, 70}}, rotation = 0)));
              Modelica.Electrical.Analog.Interfaces.NegativePin n "Electrical connector" annotation(
                Placement(transformation(extent = {{-90, -70}, {-110, -50}}, rotation = 0)));
              Modelica.Mechanics.Translational.Interfaces.Flange_b flange "Flange of component" annotation(
                Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
            equation
              x = flange.s;
              Psi_tot = coil1.Psi + coil2.Psi;
              L_statTot = coil1.L_stat + coil2.L_stat;
              connect(armature.flange_b, flange) annotation(
                Line(points = {{84, 0}, {88, 0}, {92, 0}, {100, 0}}, color = {0, 127, 0}));
              connect(r_par1.n, c_par1.p) annotation(
                Line(points = {{-64, -40}, {-60, -40}}, color = {0, 0, 255}));
              connect(r_par1.p, r_1.p) annotation(
                Line(points = {{-84, -40}, {-84, -20}}, color = {0, 0, 255}));
              connect(c_par2.p, r_par2.n) annotation(
                Line(points = {{16, -40}, {12, -40}}, color = {0, 0, 255}));
              connect(r_par2.p, r_2.p) annotation(
                Line(points = {{-8, -40}, {-8, -20}}, color = {0, 0, 255}));
              connect(r_1.p, p) annotation(
                Line(points = {{-84, -20}, {-92, -20}, {-92, 60}, {-100, 60}}, color = {0, 0, 255}));
              connect(g_mLeakWork.flange, g_mAirWork.flange) annotation(
                Line(points = {{-20, 74}, {-6, 74}, {-6, 40}, {-20, 40}}, color = {0, 127, 0}));
              connect(g_mAirWork.flange, armature.flange_a) annotation(
                Line(points = {{-20, 40}, {-6, 40}, {-6, 0}, {64, 0}}, color = {0, 127, 0}));
              connect(n, c_par2.n) annotation(
                Line(points = {{-100, -60}, {36, -60}, {36, -40}}, color = {0, 0, 255}));
              connect(coil2.port_p, g_mFeArm.port_p) annotation(
                Line(points = {{24, 30}, {20, 30}}, color = {255, 127, 0}));
              connect(G_mLeakRad.port_p, g_mFeArm.port_n) annotation(
                Line(points = {{-6.12323e-016, 46}, {0, 46}, {0, 30}}, color = {255, 127, 0}));
              connect(g_mAirWork.port_p, g_mFeArm.port_n) annotation(
                Line(points = {{-10, 30}, {0, 30}}, color = {255, 127, 0}));
              connect(coil1.port_n, g_mAirWork.port_n) annotation(
                Line(points = {{-40, 30}, {-30, 30}}, color = {255, 127, 0}));
              connect(g_mAirWork.port_n, g_mLeakWork.port_n) annotation(
                Line(points = {{-30, 30}, {-30, 64}}, color = {255, 127, 0}));
              connect(g_mLeakWork.port_p, g_mAirWork.port_p) annotation(
                Line(points = {{-10, 64}, {-10, 30}}, color = {255, 127, 0}));
              connect(coil1.port_p, g_mFePole.port_p) annotation(
                Line(points = {{-52, 30}, {-58, 30}}, color = {255, 127, 0}));
              connect(g_mFePole.port_n, g_mFePoleBot.port_p) annotation(
                Line(points = {{-78, 30}, {-78, 46}}, color = {255, 127, 0}));
              connect(g_mFePoleBot.port_n, g_mFeYokeSide1.port_p) annotation(
                Line(points = {{-78, 66}, {-78, 80}, {-50, 80}}, color = {255, 127, 0}));
              connect(g_mFeYokeSide1.port_n, G_mLeakRad.port_n) annotation(
                Line(points = {{-30, 80}, {6.12323e-016, 80}, {6.12323e-016, 66}}, color = {255, 127, 0}));
              connect(g_mFeYokeSide1.port_n, g_mFeYokeSide2.port_p) annotation(
                Line(points = {{-30, 80}, {20, 80}}, color = {255, 127, 0}));
              connect(g_mFeYokeSide2.port_n, g_mFeYokeBot.port_n) annotation(
                Line(points = {{40, 80}, {74, 80}}, color = {255, 127, 0}));
              connect(coil2.port_n, g_mLeak2.port_p) annotation(
                Line(points = {{36, 30}, {46, 30}}, color = {255, 127, 0}));
              connect(g_mLeak2.port_p, g_mLeak1.port_p) annotation(
                Line(points = {{46, 30}, {60, 30}}, color = {255, 127, 0}));
              connect(g_mLeak1.port_p, g_mAirPar.port_p) annotation(
                Line(points = {{60, 30}, {74, 30}}, color = {255, 127, 0}));
              connect(g_mAirPar.port_p, g_mLeak3.port_p) annotation(
                Line(points = {{74, 30}, {88, 30}}, color = {255, 127, 0}));
              connect(g_mLeak2.port_n, g_mLeak1.port_n) annotation(
                Line(points = {{46, 50}, {60, 50}}, color = {255, 127, 0}));
              connect(g_mLeak1.port_n, g_mAirPar.port_n) annotation(
                Line(points = {{60, 50}, {74, 50}}, color = {255, 127, 0}));
              connect(g_mAirPar.port_n, g_mLeak3.port_n) annotation(
                Line(points = {{74, 50}, {88, 50}}, color = {255, 127, 0}));
              connect(g_mFeYokeBot.port_p, g_mAirPar.port_n) annotation(
                Line(points = {{74, 60}, {74, 50}}, color = {255, 127, 0}));
              connect(coil2.p, r_2.n) annotation(
                Line(points = {{24, 10}, {24, -20}, {12, -20}}, color = {0, 0, 255}));
              connect(coil2.n, c_par2.n) annotation(
                Line(points = {{36, 10}, {36, -40}}, color = {0, 0, 255}));
              connect(coil1.n, c_par1.n) annotation(
                Line(points = {{-40, 10}, {-40, -40}}, color = {0, 0, 255}));
              connect(coil1.n, r_2.p) annotation(
                Line(points = {{-40, 10}, {-40, -20}, {-8, -20}}, color = {0, 0, 255}));
              connect(r_1.n, coil1.p) annotation(
                Line(points = {{-64, -20}, {-52, -20}, {-52, 10}}, color = {0, 0, 255}));
              connect(ground.port, g_mLeak2.port_p) annotation(
                Line(points = {{52, 22}, {52, 30}, {46, 30}}, color = {255, 127, 0}, smooth = Smooth.None));
              annotation(
                Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-90, 100}, {90, -100}}, lineColor = {255, 255, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Rectangle(extent = {{90, -30}, {-4, 30}}, lineColor = {255, 128, 0}, fillColor = {255, 128, 0}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-40, -30}, {-90, 30}}, lineColor = {255, 128, 0}, fillColor = {255, 128, 0}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-80, -100}, {-90, 100}}, lineColor = {255, 128, 0}, fillColor = {255, 128, 0}, fillPattern = FillPattern.Solid), Rectangle(extent = {{90, 90}, {-90, 100}}, lineColor = {255, 128, 0}, fillColor = {255, 128, 0}, fillPattern = FillPattern.Solid), Rectangle(extent = {{90, -100}, {-90, -90}}, lineColor = {255, 128, 0}, fillColor = {255, 128, 0}, fillPattern = FillPattern.Solid), Rectangle(extent = {{90, 40}, {80, 100}}, lineColor = {255, 128, 0}, fillColor = {255, 128, 0}, fillPattern = FillPattern.Solid), Rectangle(extent = {{90, -100}, {80, -40}}, lineColor = {255, 128, 0}, fillColor = {255, 128, 0}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-70, 80}, {70, 40}}, lineColor = {255, 213, 170}, fillColor = {255, 213, 170}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-70, -40}, {70, -80}}, lineColor = {255, 213, 170}, fillColor = {255, 213, 170}, fillPattern = FillPattern.Solid), Line(points = {{4, 30}, {4, 32}, {2, 38}, {-4, 48}, {-14, 60}, {-22, 72}, {-24, 80}, {-24, 90}}, color = {255, 128, 0}), Line(points = {{22, 30}, {22, 32}, {20, 38}, {14, 48}, {4, 60}, {-4, 72}, {-6, 80}, {-6, 90}}, color = {255, 128, 0}), Line(points = {{40, 30}, {40, 32}, {38, 38}, {32, 48}, {22, 60}, {14, 72}, {12, 80}, {12, 90}}, color = {255, 128, 0}), Text(extent = {{0, 160}, {0, 120}}, lineColor = {0, 0, 255}, textString = "%name")}),
                Documentation(info = "<html>
<p>
Please have a look at <a href=\"modelica://Modelica.Magnetic.FluxTubes.Examples.SolenoidActuator.Components.SimpleSolenoid\">SimpleSolenoid</a> for a general description of this actuator. Unlike in that simple magnetic network model, the coil is split into two lumped elements here. This enables for more realistic modelling of the radial leakage flux between armature and yoke (leakage permeance G_mLeakRad). Especially for large air gaps, the influence of this leakage flux on the actuator's inductance and its electromagnetic force is rather strong. Please have a look at <a href=\"modelica://Modelica.Magnetic.FluxTubes.Examples.SolenoidActuator.ComparisonQuasiStationary\">ComparisonQuasiStationary</a> for a comparison of both models with FEA-based results included as reference.
</p>

<p>
<IMG src=\"modelica://Modelica/Resources/Images/Magnetic/FluxTubes/Examples/SolenoidActuator/AdvancedSolenoidModel_fluxTubePartitioning.png\" ALT=\"Assigned flux tubes and field plot of the solenoid actuator\">
</p>

<p>
The parasitic capacitances c_par1 and c_par2 across both partial coils assure that the voltages across these coils are well-defined during simulation.
</p>
</html>"));
            end AdvancedSolenoid;
          end Components;
          annotation(
            Documentation(info = "<html>
<p>
In electromagnetic or reluctance actuators, a thrust or reluctance force is generated due to a non-zero gradient of the relative magnetic permeability mu_r at surfaces between regions of different permeability (non-saturated ferromagnetic material: mu_r>>1, adjacent air: mu_r=1). In lumped magnetic network models, this force can be calculated as shortly outlined in <a href=\"modelica://Modelica.Magnetic.FluxTubes.UsersGuide.ReluctanceForceCalculation\">Reluctance Forces</a> of the User's Guide.
</p>

<p>
As an example of a reluctance actuator, a simple axisymmetric lifting magnet with planar end planes of armature and pole is shown. Often, a <a href=\"modelica://Modelica.Magnetic.FluxTubes.Examples.SolenoidActuator.Components.SimpleSolenoid\">SimpleSolenoid</a> model is sufficient for initial rough design of such an actuator's magnetic subsystem. Higher accuracy can be gained from an <a href=\"modelica://Modelica.Magnetic.FluxTubes.Examples.SolenoidActuator.Components.AdvancedSolenoid\">AdvancedSolenoid</a> model where the coil-imposed magnetomotive force is split and the leakage flux between armature and yoke is accounted for more precisely.
</p>

<p>
The differences between these two models in static behaviour can be analysed and compared to results obtained with a more accurate finite element analysis (FEA) in <a href=\"modelica://Modelica.Magnetic.FluxTubes.Examples.SolenoidActuator.ComparisonQuasiStationary\">ComparisonQuasiStationary</a>. The resulting differences in dynamic behaviour can be analysed and compared to FEA results with simulation of a pull-in stroke in <a href=\"modelica://Modelica.Magnetic.FluxTubes.Examples.SolenoidActuator.ComparisonPullInStroke\">ComparisonPullInStroke</a>.
</p>
</html>"));
        end SolenoidActuator;

        package Utilities "Utilities to be used in examples"
          extends Modelica.Icons.UtilitiesPackage;

          model TranslatoryArmatureAndStopper "Mass with free travel between two stoppers"
            parameter SI.Length L(start = 0) "Length of component from left flange to right flange (= flange_b.s - flange_a.s)";
            parameter SI.Mass m(start = 1) "Armature mass";
            parameter Modelica.SIunits.TranslationalSpringConstant c(start = 1e11) "Spring stiffness between impact partners";
            parameter Modelica.SIunits.TranslationalDampingConstant d(start = 2e7) "Damping coefficient between impact partners";
            parameter Real n(final min = 1) = 2 "Exponent of spring forces (f_c = c*|s_rel|^n)" annotation(
              Evaluate = true);
            parameter SI.Position x_max(start = 10e-3) "Position of stopper at maximum armature position";
            parameter SI.Position x_min(start = 0) "Position of stopper at minimum armature position";
            Modelica.SIunits.Position s(start = 0) "Absolute position of center of component (= flange_a.s + L/2)";
            Modelica.SIunits.Velocity v(start = 0) "Absolute velocity of components (= der(s))";
            Modelica.SIunits.Acceleration a(start = 0) "Absolute acceleration of components (= der(v))";
            Modelica.Mechanics.Translational.Components.Mass mass(final L = L, final m = m) annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0)));
            Modelica.Mechanics.Translational.Interfaces.Flange_a flange_a annotation(
              Placement(transformation(extent = {{-110, -10}, {-90, 10}}, rotation = 0)));
            Modelica.Mechanics.Translational.Interfaces.Flange_b flange_b annotation(
              Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
            Modelica.Mechanics.Translational.Components.Fixed limit_xMin(s0 = x_min) annotation(
              Placement(transformation(extent = {{-80, -50}, {-60, -30}}, rotation = 0)));
            Modelica.Mechanics.Translational.Components.Fixed limit_xMax(s0 = x_max) annotation(
              Placement(transformation(extent = {{60, -50}, {80, -30}}, rotation = 0)));
            Modelica.Mechanics.Translational.Components.ElastoGap stopper_xMax(final c = c, final d = d, final n = n, final s_rel0 = 0) annotation(
              Placement(transformation(extent = {{50, -30}, {70, -10}}, rotation = 0)));
            Modelica.Mechanics.Translational.Components.ElastoGap stopper_xMin(final c = c, final d = d, final n = n, final s_rel0 = 0) annotation(
              Placement(transformation(extent = {{-70, -30}, {-50, -10}}, rotation = 0)));
          equation
            mass.s = s;
            mass.v = v;
            mass.a = a;
            connect(mass.flange_a, stopper_xMin.flange_b) annotation(
              Line(points = {{-10, 0}, {-40, 0}, {-40, -20}, {-50, -20}}, color = {0, 127, 0}));
            connect(limit_xMax.flange, stopper_xMax.flange_b) annotation(
              Line(points = {{70, -40}, {70, -20}}, color = {0, 191, 0}));
            connect(stopper_xMax.flange_a, mass.flange_b) annotation(
              Line(points = {{50, -20}, {40, -20}, {40, 0}, {10, 0}}, color = {0, 127, 0}));
            connect(mass.flange_a, flange_a) annotation(
              Line(points = {{-10, 0}, {-32.5, 0}, {-55, 0}, {-100, 0}}, color = {0, 127, 0}));
            connect(limit_xMin.flange, stopper_xMin.flange_a) annotation(
              Line(points = {{-70, -40}, {-70, -20}}, color = {0, 127, 0}));
            connect(flange_b, mass.flange_b) annotation(
              Line(points = {{100, 0}, {10, 0}}, color = {0, 127, 0}));
            annotation(
              defaultComponentName = "armature",
              Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-80, -60}, {-80, -80}}, color = {160, 160, 164}, thickness = 0.5), Line(points = {{-80, -76}, {-40, -76}}, color = {160, 160, 164}, thickness = 0.5, arrow = {Arrow.None, Arrow.Filled}), Text(extent = {{-84, -82}, {-76, -92}}, lineColor = {160, 160, 164}, pattern = LinePattern.Dash, lineThickness = 0.5, textString = "0"), Text(extent = {{-46, -82}, {-38, -92}}, lineColor = {160, 160, 164}, pattern = LinePattern.Dash, lineThickness = 0.5, textString = "x"), Ellipse(extent = {{-82, -78}, {-78, -74}}, lineColor = {175, 175, 175}, fillColor = {175, 175, 175}, fillPattern = FillPattern.Solid)}),
              Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-90, 10}, {90, -10}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {255, 127, 0}), Text(extent = {{-120, 140}, {120, 100}}, lineColor = {0, 0, 255}, textString = "%name"), Rectangle(extent = {{-50, 60}, {50, -60}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {255, 127, 0}), Rectangle(extent = {{-80, -20}, {-88, -80}}, lineColor = {255, 128, 0}, fillColor = {255, 128, 0}, fillPattern = FillPattern.Solid), Rectangle(extent = {{88, -20}, {80, -80}}, lineColor = {255, 128, 0}, fillColor = {255, 128, 0}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-80, 80}, {-88, 20}}, lineColor = {255, 128, 0}, fillColor = {255, 128, 0}, fillPattern = FillPattern.Solid), Rectangle(extent = {{88, 80}, {80, 20}}, lineColor = {255, 128, 0}, fillColor = {255, 128, 0}, fillPattern = FillPattern.Solid), Text(extent = {{-100, -100}, {100, -140}}, lineColor = {0, 0, 0}, textString = "m=%m"), Line(points = {{-50, -80}, {30, -80}}, color = {0, 0, 0}), Polygon(points = {{60, -80}, {30, -70}, {30, -90}, {60, -80}}, lineColor = {128, 128, 128}, fillColor = {128, 128, 128}, fillPattern = FillPattern.Solid)}),
              Documentation(info = "<html>
<p>
In translatory actuators with limited stroke, the armature with its inertia can travel between two stoppers.
</p>
</html>"));
          end TranslatoryArmatureAndStopper;

          record CoilDesign "Calculation of winding parameters (wire diameter, number of turns et al.) and recalculation with optionally chosen parameters; to be adapted to particular design tasks"
            extends Modelica.Icons.Record;
            parameter SI.Resistivity rho_20 = 0.0178e-6 "Resistivity of conductor material at 20 degC (default: Copper)";
            parameter Modelica.SIunits.LinearTemperatureCoefficient alpha_20 = 0.0039 "Temperature coefficient of conductor material's resistivity at 20 degC (default: Copper)";
            parameter SI.Temperature T_op = 293.15 "Operating temperature of winding";
            final parameter SI.Resistivity rho = rho_20 * (1 + alpha_20 * (T_op - (20 - Modelica.Constants.T_zero))) "Resistivity at operating temperature";
            parameter SI.Length h_w "Height of winding cross-section";
            parameter SI.Length b_w "Width of winding cross-section";
            final parameter SI.Area A_w = h_w * b_w "Cross-section area of winding";
            parameter SI.Length l_avg "Average length of one turn";
            parameter SI.Voltage V_op "Operating voltage (nominal/ minimum/ maximum voltage depending on design objective)";
            parameter SI.CurrentDensity J_desired = 4e6 "DESIRED current density at operating temperature and voltage resp.";
            parameter Real c_condFillChosen = 0.6 "CHOSEN conductor filling factor = total conductor area without insulation/ total winding area";
            final parameter Real N_calculated = V_op / (rho * l_avg * J_desired) "CALCULATED number of turns";
            final parameter SI.Diameter d_wireCalculated = sqrt(4 * A_w * c_condFillChosen / (pi * N_calculated)) "CALCULATED wire diameter (without insulation)";
            final parameter SI.Area A_wireCalculated = pi * d_wireCalculated ^ 2 / 4 "Calculated wire cross-section area";
            final parameter SI.Resistance R_calculated = rho * N_calculated * l_avg / A_wireCalculated "Winding resistance at operating temperature and voltage resp. with CALCULATED number of turns and wire diameter";
            final parameter SI.Power P_calculated = V_op ^ 2 / R_calculated "Winding's ohmic losses at operating temperature and voltage resp. with CALCULATED number of turns and wire diameter";
            parameter SI.Diameter d_wireChosen = d_wireCalculated "CHOSEN available wire diameter (without insulation)" annotation(
              Dialog(group = "Chosen feasible parameters (optional)"));
            parameter Real N_chosen = N_calculated "CHOSEN number of turns" annotation(
              Dialog(group = "Chosen feasible parameters (optional)"));
            final parameter SI.Area A_wireChosen = pi * d_wireChosen ^ 2 / 4 "Wire cross-section area resulting from CHOSEN wire diameter";
            final parameter SI.Resistance R_actual = rho * N_chosen * l_avg / A_wireChosen "Winding resistance at operating temperature and voltage resp. resulting from CHOSEN number of turns and wire diameter";
            final parameter SI.Power P_actual = V_op ^ 2 / R_actual "Winding's ohmic losses at operating temperature and voltage resp. resulting from CHOSEN number of turns and wire diameter";
            final parameter SI.CurrentDensity J_actual = V_op * 4 / (R_actual * pi * d_wireChosen ^ 2) "Current density at operating temperature and voltage resp. resulting from CHOSEN number of turns and wire diameter";
            final parameter Real c_condFillActual = N_chosen * pi * d_wireChosen ^ 2 / (4 * A_w) "Conductor filling factor resulting from CHOSEN number of turns and wire diameter";
            annotation(
              Documentation(info = "<html>
<p>
This model example shows dimensioning of a winding (wire diameter, number of turns) based on desired operating conditions (voltage, temperature, current density, conductor filling factor) for a given cross-section area of the winding. It can be modified according to the parameters given and sought after for a particular design project.
</p>

<p>
The calculated winding resistance and number of turns can be used as input parameters to the electrical subsystem
of a device to be modelled. Operating voltage V_op can be minimum, nominal and maximum voltage respectively as specified for a particular design project. In conjunction with the setting of the operating temperature T_op, this enables for analysis of the device under worst-case conditions (e.g., minimum required magnetomotive force, maximum allowed ohmic losses, minimum and maximum force respectively).
</p>

<p>
For manufacturing of a winding, the obtained wire diameter d_wireCalculated must be rounded to that of an available wire. In order to analyse the influence of this rounding, one can enter the chosen wire diameter d_wireChosen and number of turns N_chosen as optional input. Calculation of the resulting winding parameters enables for comparison with the ones obtained otherwise.
</p>
</html>"));
          end CoilDesign;
        end Utilities;
        annotation(
          Documentation(info = "<html>
<p>
This package contains examples to demonstrate the usage of the flux tubes components.
</p>
</html>"));
      end Examples;

      package Basic "Basic elements of magnetic network models"
        extends Modelica.Icons.Package;

        model Ground "Zero magnetic potential"
          Modelica.Magnetic.FluxTubes.Interfaces.PositiveMagneticPort port annotation(
            Placement(transformation(extent = {{-10, 110}, {10, 90}}, rotation = -0)));
        equation
          port.V_m = 0;
          annotation(
            Documentation(info = "<html>
<p>
The magnetic potential at the magnetic ground node is zero. Every magnetic network model must contain at least one magnetic ground object.
</p>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-60, 50}, {60, 50}}, color = {255, 127, 0}), Line(points = {{-40, 30}, {40, 30}}, color = {255, 127, 0}), Line(points = {{-20, 10}, {20, 10}}, color = {255, 127, 0}), Line(points = {{0, 90}, {0, 50}}, color = {255, 127, 0}), Text(extent = {{-150, -40}, {150, 0}}, lineColor = {0, 0, 255}, textString = "%name")}),
            Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-60, 50}, {60, 50}}, color = {255, 127, 0}, thickness = 0.5), Line(points = {{-40, 30}, {40, 30}}, color = {255, 127, 0}, thickness = 0.5), Line(points = {{-20, 10}, {20, 10}}, color = {255, 127, 0}, thickness = 0.5), Line(points = {{0, 100}, {0, 50}}, color = {255, 127, 0}, thickness = 0.5), Text(extent = {{-40, -40}, {40, 20}}, lineColor = {0, 0, 255}, textString = "port.V_m = 0")}));
        end Ground;

        model ElectroMagneticConverter "Electro-magnetic energy conversion"
          Modelica.Magnetic.FluxTubes.Interfaces.PositiveMagneticPort port_p "Positive magnetic port" annotation(
            Placement(transformation(extent = {{90, 50}, {110, 70}}, rotation = 0)));
          Modelica.Magnetic.FluxTubes.Interfaces.NegativeMagneticPort port_n "Negative magnetic port" annotation(
            Placement(transformation(extent = {{110, -70}, {90, -50}}, rotation = 0)));
          Modelica.Electrical.Analog.Interfaces.PositivePin p "Positive electric pin" annotation(
            Placement(transformation(extent = {{-90, 50}, {-110, 70}}, rotation = 0)));
          Modelica.Electrical.Analog.Interfaces.NegativePin n "Negative electric pin" annotation(
            Placement(transformation(extent = {{-110, -70}, {-90, -50}}, rotation = 0)));
          SI.Voltage v "Voltage";
          SI.Current i(start = 0, stateSelect = StateSelect.prefer) "Current";
          SI.MagneticPotentialDifference V_m "Magnetic potential difference";
          SI.MagneticFlux Phi "Magnetic flux coupled into magnetic circuit";
          parameter Real N = 1 "Number of turns";
          //for information only:
          SI.MagneticFlux Psi "Flux linkage for information only";
          SI.Inductance L_stat "Static inductance abs(Psi/i) for information only";
        protected
          Real eps = 100 * Modelica.Constants.eps;
        equation
          v = p.v - n.v;
          0 = p.i + n.i;
          i = p.i;
          V_m = port_p.V_m - port_n.V_m;
          0 = port_p.Phi + port_n.Phi;
          Phi = port_p.Phi;
//converter equations:
          V_m = i * N;
// Ampere's law
          N * der(Phi) = -v;
// Faraday's law
//for information only:
          Psi = N * Phi;
//use of abs() for positive results; due to Modelica sign conventions for flow into connectors:
          L_stat = noEvent(if abs(i) > eps then abs(Psi / i) else abs(Psi / eps));
          annotation(
            defaultComponentName = "converter",
            Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{-134, 63}, {-124, 60}, {-134, 57}, {-134, 63}}, lineColor = {160, 160, 164}, fillColor = {160, 160, 164}, fillPattern = FillPattern.Solid), Line(points = {{-150, 60}, {-125, 60}}, color = {160, 160, 164}), Polygon(points = {{141, -57}, {151, -60}, {141, -63}, {141, -57}}, lineColor = {160, 160, 164}, fillColor = {160, 160, 164}, fillPattern = FillPattern.Solid), Line(points = {{125, -60}, {150, -60}}, color = {160, 160, 164}), Text(extent = {{128, -56}, {144, -41}}, lineColor = {160, 160, 164}, textString = "Phi"), Text(extent = {{128, 64}, {145, 79}}, lineColor = {0, 0, 0}, textString = "Phi"), Line(points = {{-150, -59}, {-125, -59}}, color = {160, 160, 164}), Polygon(points = {{-140, -56}, {-150, -59}, {-140, -62}, {-140, -56}}, lineColor = {160, 160, 164}, fillColor = {160, 160, 164}, fillPattern = FillPattern.Solid), Text(extent = {{-141, -56}, {-124, -41}}, lineColor = {160, 160, 164}, textString = "i"), Text(extent = {{-150, 63}, {-133, 78}}, lineColor = {160, 160, 164}, textString = "i"), Line(points = {{124, 61}, {149, 61}}, color = {160, 160, 164}), Polygon(points = {{134, 64}, {124, 61}, {134, 58}, {134, 64}}, lineColor = {160, 160, 164}, fillColor = {160, 160, 164}, fillPattern = FillPattern.Solid)}),
            Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-70, 100}, {70, -100}}, lineColor = {0, 0, 255}, pattern = LinePattern.None, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-50, 0}, {-30, 20}}, lineColor = {0, 0, 255}), Line(points = {{-40, 60}, {-40, 40}}, color = {0, 0, 255}), Ellipse(extent = {{-50, 20}, {-30, 40}}, lineColor = {0, 0, 255}), Ellipse(extent = {{-50, -20}, {-30, 0}}, lineColor = {0, 0, 255}), Ellipse(extent = {{-50, -40}, {-30, -20}}, lineColor = {0, 0, 255}), Line(points = {{-40, -40}, {-40, -60}}, color = {0, 0, 255}), Rectangle(extent = {{-54, 40}, {-40, -40}}, lineColor = {255, 255, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{-40, 60}, {-92, 60}}, color = {0, 0, 255}), Line(points = {{-40, -60}, {-90, -60}}, color = {0, 0, 255}), Line(points = {{0, 100}, {-70, 100}}, color = {0, 0, 255}, pattern = LinePattern.Dash), Line(points = {{-70, 100}, {-70, -100}}, color = {0, 0, 255}, pattern = LinePattern.Dash), Line(points = {{0, -100}, {-70, -100}}, color = {0, 0, 255}, pattern = LinePattern.Dash), Line(points = {{70, 100}, {0, 100}}, color = {255, 127, 0}, pattern = LinePattern.Dash), Line(points = {{70, -100}, {0, -100}}, color = {255, 127, 0}, pattern = LinePattern.Dash), Line(points = {{70, 100}, {70, -100}}, color = {255, 127, 0}, pattern = LinePattern.Dash), Ellipse(extent = {{-4, -34}, {64, 34}}, lineColor = {255, 127, 0}), Line(points = {{30, -60}, {30, -34}}, color = {255, 127, 0}), Line(points = {{18, 0}, {42, 0}}, color = {255, 127, 0}), Line(points = {{42, 10}, {42, -12}}, color = {255, 127, 0}), Line(points = {{30, 34}, {30, 60}}, color = {255, 127, 0}), Line(points = {{30, 60}, {100, 60}}, color = {255, 127, 0}), Line(points = {{30, -60}, {90, -60}}, color = {255, 127, 0}), Text(extent = {{-150, 150}, {150, 110}}, lineColor = {0, 0, 255}, textString = "%name"), Line(points = {{18, 10}, {18, -12}}, color = {255, 127, 0}), Line(points = {{-90, 30}, {-90, -30}}, color = {0, 0, 255}), Polygon(points = {{-90, -30}, {-84, -10}, {-96, -10}, {-90, -30}}, lineColor = {0, 0, 255}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid), Line(points = {{90, 30}, {90, -30}}, color = {255, 128, 0}), Polygon(points = {{90, -30}, {96, -10}, {84, -10}, {90, -30}}, lineColor = {255, 128, 0}, fillColor = {255, 128, 0}, fillPattern = FillPattern.Solid)}),
            Documentation(info = "<html>
<p>
The electro-magnetic energy conversion is given by <i>Ampere</i>'s law and <i>Faraday</i>'s law respectively:
</p>

<pre>
    V_m = i * N
    N * d&Phi;/dt = -v
</pre>

<p>
V_m is the magnetomotive force that is supplied to the connected magnetic circuit, &Phi; is the magnetic flux through the associated branch of this magnetic circuit. The negative sign of the induced voltage v is due to <i>Lenz</i>'s law.
</p>

<p>
The flux linkage &Psi; and the static inductance L_stat = |&Psi;/i| are calculated for information only. Note that L_stat is set to |&Psi;/eps| if |i| &lt; eps
(= 100*Modelica.Constants.eps).
</p>
</html>"));
        end ElectroMagneticConverter;

        model ConstantReluctance "Constant reluctance"
          extends Modelica.Magnetic.FluxTubes.Interfaces.PartialTwoPorts;
          parameter SI.Reluctance R_m = 1 "Magnetic reluctance";
        equation
          V_m = Phi * R_m;
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-70, 30}, {70, -30}}, lineColor = {255, 128, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{-70, 0}, {-90, 0}}, color = {255, 128, 0}), Line(points = {{70, 0}, {90, 0}}, color = {255, 128, 0}), Text(extent = {{-100, -100}, {100, -62}}, textString = "%name", lineColor = {0, 0, 255})}),
            Documentation(info = "<html>
<p>
This constant reluctance is provided for test purposes and simple magnetic network models. The reluctance is not calculated from geometry and permeability of a flux tube, but is provided as a parameter.
</p>
</html>"));
        end ConstantReluctance;

        model LeakageWithCoefficient "Leakage reluctance with respect to the reluctance of a useful flux path (not for dynamic simulation of actuators)"
          extends Modelica.Magnetic.FluxTubes.Interfaces.PartialLeakage;
          parameter SI.CouplingCoefficient c_usefulFlux = 0.7 "Ratio useful flux/(leakage flux + useful flux) = useful flux/total flux";
          input SI.Reluctance R_mUsefulTot "Total reluctance of useful flux path as reference" annotation(
            Dialog(group = "Reference reluctance", groupImage = "modelica://Modelica/Resources/Images/Magnetic/FluxTubes/Basic/LeakageWithCoefficient.png"));
        equation
          (1 - c_usefulFlux) * R_m = c_usefulFlux * R_mUsefulTot;
// Generalized Kirchhoff's current law
          annotation(
            Documentation(info = "<html>
<p>
Differently from the flux tube elements of package <a href=\"modelica://Modelica.Magnetic.FluxTubes.Shapes.Leakage\">Shapes.Leakage</a>
that are calculated from their geometry, this leakage reluctance is calculated with reference to the total reluctance of a useful flux path. Please refer to the <b>Parameters</b> section for an illustration of the resulting magnetic network. Exploiting <i>Kirchhoff</i>'s generalized current law, the leakage reluctance is calculated by means of a coupling coefficient c_usefulFlux.
</p>

<h4>Attention:</h4>

<p>
This element must <b>not</b> be used <b>for dynamic simulation of</b> electro-magneto-mechanical <b>actuators</b>, where the shape of at least one flux tube element with reluctance force generation in the useful flux path changes with armature motion (e.g., air gap). This change results in a non-zero derivative dG_m/dx of those elements permeance G_m with respect to armature position x, which in turn will lead to a non-zero derivative of the leakage permeance with respect to armature position. This would generate a reluctance force in the leakage element that is not accounted for properly. <a href=\"modelica://Modelica.Magnetic.FluxTubes.Shapes.Force.LeakageAroundPoles\">Shapes.Force.LeakageAroundPoles</a> provides a simple leakage reluctance with force generation.
</p>
</html>"));
        end LeakageWithCoefficient;

        model EddyCurrent "For modelling of eddy current in a conductive magnetic flux tube"
          extends Modelica.Magnetic.FluxTubes.Interfaces.PartialTwoPorts;
          extends Modelica.Electrical.Analog.Interfaces.ConditionalHeatPort(final T = 273.15);
          parameter SI.Resistivity rho = 0.098e-6 "Resistivity of flux tube material (default: Iron at 20degC)";
          parameter SI.Length l = 1 "Average length of eddy current path";
          parameter SI.Area A = 1 "Cross sectional area of eddy current path";
          final parameter SI.Resistance R = rho * l / A "Electrical resistance of eddy current path";
        equation
          LossPower = V_m * der(Phi);
          V_m = 1 / R * der(Phi);
//Magnetic voltage drop in magnetic network due to eddy current
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-70, 30}, {70, -30}}, lineColor = {255, 128, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{-70, 0}, {-90, 0}}, color = {255, 128, 0}), Line(points = {{70, 0}, {90, 0}}, color = {255, 128, 0}), Rectangle(extent = {{-70, 30}, {70, -30}}, lineColor = {255, 128, 0}, fillColor = {255, 128, 0}, fillPattern = FillPattern.Solid), Text(extent = {{-100, -98}, {100, -60}}, textString = "%name", lineColor = {0, 0, 255})}),
            Documentation(info = "<html>
<p>
Eddy currents are induced in a conductive magnetic flux tube when the flux changes with time. This causes a magnetic voltage drop in addition to the voltage drop that is due to the reluctance of this flux tube. The eddy current component can be thought of as a short-circuited secondary winding of a transformer with only one turn. Its resistance is calculated from the geometry and resistivity of the eddy current path.
</p>

<p>
Partitioning of a solid conductive cylinder or prism into several hollow cylinders or separate nested prisms and modelling of each of these flux tubes connected in parallel with a series connection of a reluctance element and an eddy current component can model the delayed buildup of the magnetic field in the complete flux tube from the outer to the inner sections. Please refer to <a href=\"modelica://Modelica.Magnetic.FluxTubes.UsersGuide.Literature\">[Ka08]</a> for an illustration.
</p>
</html>"));
        end EddyCurrent;
      end Basic;

      package Shapes "Reluctance and permeance elements respectively based on geometric shapes"
        extends Modelica.Icons.Package;

        package FixedShape "Flux tubes with fixed shape during simulation and linear or non-linear material characteristics"
          extends Modelica.Icons.VariantsPackage;

          model HollowCylinderAxialFlux "(Hollow) cylinder with axial flux; fixed shape; linear or non-linear material characteristics"
            extends Modelica.Magnetic.FluxTubes.Interfaces.PartialFixedShape;
            parameter SI.Length l = 0.01 "Axial length (in direction of flux)" annotation(
              Dialog(group = "Fixed geometry", groupImage = "modelica://Modelica/Resources/Images/Magnetic/FluxTubes/Shapes/HollowCylinderAxialFlux.png"));
            parameter SI.Radius r_i = 0 "Inner radius of hollow cylinder (zero for cylinder)" annotation(
              Dialog(group = "Fixed geometry"));
            parameter SI.Radius r_o = 0.01 "Outer radius of (hollow) cylinder" annotation(
              Dialog(group = "Fixed geometry"));
          equation
            A = pi * (r_o ^ 2 - r_i ^ 2);
            G_m = mu_0 * mu_r * A / l;
            annotation(
              Documentation(info = "<html>
<p>
Please refer to the enclosing sub-package <a href=\"modelica://Modelica.Magnetic.FluxTubes.Shapes.FixedShape\">FixedShape</a> for a description of all elements of this package and to <a href=\"modelica://Modelica.Magnetic.FluxTubes.UsersGuide.Literature\">[Ro41]</a> for derivation and/or coefficients of the equation for permeance G_m.
</p>

<p>
Set the inner radius r_i=0 for modelling of a solid cylindric flux tube.
</p>
</html>"));
          end HollowCylinderAxialFlux;

          model HollowCylinderRadialFlux "Hollow cylinder with radial flux; fixed shape; linear or non-linear material characteristics"
            extends Modelica.Magnetic.FluxTubes.Interfaces.PartialFixedShape;
            parameter SI.Length l = 0.01 "Width (orthogonal to flux direction)" annotation(
              Dialog(group = "Fixed geometry", groupImage = "modelica://Modelica/Resources/Images/Magnetic/FluxTubes/Shapes/HollowCylinderRadialFlux.png"));
            parameter SI.Radius r_i = 0.01 "Inner radius of hollow cylinder" annotation(
              Dialog(group = "Fixed geometry"));
            parameter SI.Radius r_o = 0.02 "Outer radius of hollow cylinder" annotation(
              Dialog(group = "Fixed geometry"));
          equation
            A = l * pi * (r_o + r_i);
// Area at arithmetic mean radius for calculation of average flux density
            G_m = 2 * pi * mu_0 * mu_r * l / Modelica.Math.log(r_o / r_i);
            annotation(
              Documentation(info = "<html>
<p>
Please refer to the enclosing sub-package <a href=\"modelica://Modelica.Magnetic.FluxTubes.Shapes.FixedShape\">FixedShape</a> for a description of all elements of this package and to <a href=\"modelica://Modelica.Magnetic.FluxTubes.UsersGuide.Literature\">[Ro41]</a> for derivation and/or coefficients of the equation for permeance G_m.
</p>

<p>
For hollow cylindric flux tubes with a radial magnetic flux, the flux density is a function of the radius. For that reason, the characteristic mu_r(B) is evaluated for the flux density at the flux tube's mean radius.
</p>

<p>
For those flux tube sections of a magnetic device that have a nonlinear material characteristic mu_r(B) and a large aspect ratio of outer to inner radius r_o/r_i, the section can be split up in a series connection of several hollow cylindric flux tubes with radial flux. This allows for more realistic modelling of the dependence of flux density on the radius compared to modelling with just one flux tube element.
</p>
</html>"));
          end HollowCylinderRadialFlux;

          model Cuboid "Flux tube with rectangular cross-section; fixed shape; linear or non-linear material characteristics"
            extends Modelica.Magnetic.FluxTubes.Interfaces.PartialFixedShape;
            parameter SI.Length l = 0.01 "Length in direction of flux" annotation(
              Dialog(group = "Fixed geometry", groupImage = "modelica://Modelica/Resources/Images/Magnetic/FluxTubes/Shapes/CuboidParallelFlux.png"));
            parameter SI.Length a = 0.01 "Width of rectangular cross-section" annotation(
              Dialog(group = "Fixed geometry"));
            parameter SI.Length b = 0.01 "Height of rectangular cross-section" annotation(
              Dialog(group = "Fixed geometry"));
          equation
            A = a * b;
            G_m = mu_0 * mu_r * A / l;
            annotation(
              Documentation(info = "<html>
<p>
Please refer to the enclosing sub-package <a href=\"modelica://Modelica.Magnetic.FluxTubes.Shapes.FixedShape\">FixedShape</a> for a description of all elements of this package and to <a href=\"modelica://Modelica.Magnetic.FluxTubes.UsersGuide.Literature\">[Ro41]</a> for derivation and/or coefficients of the equation for permeance G_m.
</p>
</html>"));
          end Cuboid;
          annotation(
            Documentation(info = "<html>
<p>
Please have a look at <a href=\"modelica://Modelica.Magnetic.FluxTubes.UsersGuide.ReluctanceForceCalculation\">UsersGuide.ReluctanceForceCalculation</a> for an explanation of the different flux tube categories and resulting sub-packages.
</p>

<p>
Due to the restrictions on reluctance force calculation outlined there, flux tube elements with a possibly non-linear material characteristic mu_r(B) must have a fixed shape during simulation of converter motion. Hence, the dimensions of these flux tubes are defined as parameters in the model components that extend the base class <a href=\"modelica://Modelica.Magnetic.FluxTubes.Interfaces.PartialFixedShape\">Interfaces.PartialFixedShape</a>.</p>

<p>
For initial design of magnetic circuits, the relative permeability of possibly non-linear flux tube elements can easily be set to a constant value mu_rConst (non-linearPermeability set to false). In some cases, this can simplify the rough geometric design of a device's magnetic circuit. Once an initial geometry is found, the magnetic subsystem can be simulated and fine-tuned with more realistic non-linear characteristics of ferromagnetic materials. Doing so requires setting of the parameter non-linearPermeability to true and selection of one of the soft magnetic materials of <a href=\"modelica://Modelica.Magnetic.FluxTubes.Material.SoftMagnetic\">Material.SoftMagnetic</a>.
</p>
</html>"));
        end FixedShape;

        package Force "Flux tubes with reluctance force generation; constant permeability"
          extends Modelica.Icons.VariantsPackage;

          model HollowCylinderAxialFlux "(Hollow) cylinder with axial flux; constant permeability"
            extends Modelica.Magnetic.FluxTubes.Interfaces.PartialForce;
            SI.Length l = s "Axial length (in direction of flux)" annotation(
              Dialog(group = "Variable geometry", groupImage = "modelica://Modelica/Resources/Images/Magnetic/FluxTubes/Shapes/HollowCylinderAxialFlux.png"));
            parameter SI.Radius r_i = 0 "Inner radius of (hollow) cylinder";
            parameter SI.Radius r_o = 0.01 "Outer radius of (hollow) cylinder";
            SI.MagneticFluxDensity B "Homogeneous flux density";
          protected
            parameter SI.Area A = pi * (r_o ^ 2 - r_i ^ 2) "Cross-sectional area orthogonal to direction of flux";
          equation
            G_m = mu_0 * mu_r * A / l;
            dGmBydx = -1 * mu_0 * mu_r * A / l ^ 2 * dlBydx;
            B = Phi / A;
            annotation(
              Documentation(info = "<html>
<p>
Please refer to the enclosing sub-package <a href=\"modelica://Modelica.Magnetic.FluxTubes.Shapes.Force\">Force</a> for a description of all elements of this package and to <a href=\"modelica://Modelica.Magnetic.FluxTubes.UsersGuide.Literature\">[Ro41]</a> for derivation and/or coefficients of the equation for permeance G_m.
</p>
</html>"));
          end HollowCylinderAxialFlux;

          model HollowCylinderRadialFlux "Hollow cylinder with radial flux; constant permeability"
            extends Modelica.Magnetic.FluxTubes.Interfaces.PartialForce;
            SI.Length l = s "Axial length (orthogonal to direction of flux)" annotation(
              Dialog(group = "Variable geometry", groupImage = "modelica://Modelica/Resources/Images/Magnetic/FluxTubes/Shapes/HollowCylinderRadialFlux.png"));
            parameter SI.Radius r_i = 0.01 "Inner radius of hollow cylinder";
            parameter SI.Radius r_o = 0.015 "Outer radius of hollow cylinder";
            SI.MagneticFluxDensity B_avg "Average flux density (at arithmetic mean radius)";
          protected
            SI.Area A_avg "Average cross-sectional area orthogonal to direction of flux (at arithmetic mean radius)";
          equation
            G_m = mu_0 * mu_r * 2 * pi * l / Modelica.Math.log(r_o / r_i);
            dGmBydx = mu_0 * mu_r * 2 * pi / Modelica.Math.log(r_o / r_i) * dlBydx;
            A_avg = pi * (r_i + r_o) * l;
            B_avg = Phi / A_avg;
            annotation(
              Documentation(info = "<html>
<p>
Please refer to the enclosing sub-package <a href=\"modelica://Modelica.Magnetic.FluxTubes.Shapes.Force\">Force</a> for a description of all elements of this package and to <a href=\"modelica://Modelica.Magnetic.FluxTubes.UsersGuide.Literature\">[Ro41]</a> for derivation and/or coefficients of the equation for permeance G_m.
</p>
</html>"));
          end HollowCylinderRadialFlux;

          model CuboidParallelFlux "Cuboid with flux in direction of motion, e.g., air gap with rectangular cross-section; constant permeability"
            extends Modelica.Magnetic.FluxTubes.Interfaces.PartialForce;
            SI.Length l = s "Axial length (in direction of flux)" annotation(
              Dialog(group = "Variable geometry", groupImage = "modelica://Modelica/Resources/Images/Magnetic/FluxTubes/Shapes/CuboidParallelFlux.png"));
            parameter SI.Length a = 0.01 "Width of rectangular cross-section";
            parameter SI.Length b = 0.01 "Height of rectangular cross-section";
            SI.MagneticFluxDensity B "Homogeneous flux density";
          protected
            parameter SI.Area A = a * b "Cross-sectional area orthogonal to direction of flux";
          equation
            G_m = mu_0 * mu_r * A / l;
            dGmBydx = -1 * mu_0 * mu_r * A / l ^ 2 * dlBydx;
            B = Phi / A;
            annotation(
              Documentation(info = "<html>
<p>
Please refer to the enclosing sub-package <a href=\"modelica://Modelica.Magnetic.FluxTubes.Shapes.Force\">Force</a> for a description of all elements of this package and to <a href=\"modelica://Modelica.Magnetic.FluxTubes.UsersGuide.Literature\">[Ro41]</a> for derivation and/or coefficients of the equation for permeance G_m.
</p>
</html>"));
          end CuboidParallelFlux;

          model CuboidOrthogonalFlux "Cuboid with flux orthogonal to direction of motion; constant permeability"
            extends Modelica.Magnetic.FluxTubes.Interfaces.PartialForce;
            SI.Length l = s "Length in direction of motion (orthogonal to flux)" annotation(
              Dialog(group = "Variable geometry", groupImage = "modelica://Modelica/Resources/Images/Magnetic/FluxTubes/Shapes/CuboidOrthogonalFlux.png"));
            parameter SI.Length a = 0.01 "Width of rectangular cross-section";
            parameter SI.Length b = 0.01 "Height of rectangular cross-section (in flux direction)";
            SI.MagneticFluxDensity B "Homogeneous flux density";
          protected
            SI.Area A "Cross-sectional area orthogonal to direction of flux";
          equation
            A = a * l;
            G_m = mu_0 * mu_r * A / b;
            dGmBydx = mu_0 * mu_r * a / b * dlBydx;
            B = Phi / A;
            annotation(
              Documentation(info = "<html>
<p>
Please refer to the enclosing sub-package <a href=\"modelica://Modelica.Magnetic.FluxTubes.Shapes.Force\">Force</a> for a description of all elements of this package and to <a href=\"modelica://Modelica.Magnetic.FluxTubes.UsersGuide.Literature\">[Ro41]</a> for derivation and/or coefficients of the equation for permeance G_m.
</p>
</html>"));
          end CuboidOrthogonalFlux;

          model LeakageAroundPoles "Leakage flux tube around cylindrical or prismatic poles"
            extends Modelica.Magnetic.FluxTubes.Interfaces.PartialForce;
            SI.Length l = s "Axial length (in direction of flux)" annotation(
              Dialog(group = "Variable geometry", groupImage = "modelica://Modelica/Resources/Images/Magnetic/FluxTubes/Shapes/LeakageAroundPoles.png"));
            parameter SI.Length w = 0.1 "Width orthogonal to flux; mean circumference of flux tube in case of cylindrical poles";
            parameter SI.Radius r = 0.01 "Radius of leakage field";
          equation
//adapted from [Ka08], but corrected
//(in [Ka08] equation accidentally swapped with that of a similar element)
            G_m = mu_0 * w / pi * Modelica.Math.log(1 + pi * r / l);
//derivative at full length:
//  dGmBydx = mu_0 * w /pi * 1/(1 + pi * r/l) * (-1)*pi*r/l^2  * dlBydx;
//simplified:
            dGmBydx = -mu_0 * w * r * dlBydx / (l ^ 2 * (1 + pi * r / l));
            annotation(
              Documentation(info = "<html>
<p>
Please refer to the description of  the enclosing sub-package <a href=\"modelica://Modelica.Magnetic.FluxTubes.Shapes.Force\">Force</a> for a description of all elements of this package.
</p>

<p>
Leakage flux around a prismatic or cylindric air gap between to poles can be described with this model. Due to its constant radius of the leakage field r, the model is rather simple. Whereas in reality the leakage radius is approximately constant for air gap lengths l greater than this radius, it decreases with air gap lengths less than the leakage radius. This decrease for small air gaps is neglected here, since the influence of the leakage flux tube compared to that of the enclosed main air gap (connected in parallel) decreases for decreasing air gap length l.
</p>

<p>
Note that in <a href=\"modelica://Modelica.Magnetic.FluxTubes.UsersGuide.Literature\">[Ka08]</a> the equation for G_m is accidentally swapped with that of a similar element.
</p>
</html>"));
          end LeakageAroundPoles;
          annotation(
            Documentation(info = "<html>
<p>
Please have a look at <a href=\"modelica://Modelica.Magnetic.FluxTubes.UsersGuide.ReluctanceForceCalculation\">UsersGuide.ReluctanceForceCalculation</a> for an explanation of the different flux tube categories and resulting sub-packages.
</p>
<p>
Flux tube elements with generation of a reluctance force are intended for modelling of position-dependent air gap sections and permanent magnet sections respectively of translatory actuators. By default, the position co-ordinate of the mechanical connector flange.s is identical with the dimension l of the package's flux tube elements. l is the dimension changes with armature motion. If needed, the identity l=flange.s can be replaced by an actuator-specific equation, for example, when a flux tube length increases with decreasing armature position. The position co-ordinate of an element's translatory connector flange.s in turn will be identical with the armature position x in most cases, as the examples illustrate.</p>
<p>
The derivative of each element's permeance with respect to armature position <code>dGmBydx</code> is calculated from the derivative of the flux tube's permeance with respect to its varying dimension dGmBydl and the derivative of this dimension with respect to armature position <code>dlBydx</code>:</p>

<pre>
    dG_m   dG_m   dl
    ---- = ---- * --
     dx     dl    dx
</pre>

<p>
The parameter <code>dlBydx</code> must be set in each flux tube element to +1 or -1 according to the definition of the armature co-ordinate and the position of the element in a device's magnetic circuit. Proper match between armature motion and resulting variation of the flux tube length assures that the element's reluctance force acts in the right direction.
</p>
<p>
The shapes of the flux tubes defined in this package are rather simple. Only one dimension varies with armature motion. Flux tubes with more complex variations of dimensions with armature motion can be defined by extending the base class <a href=\"modelica://Modelica.Magnetic.FluxTubes.Interfaces.PartialForce\">Interfaces.PartialForce</a>, if needed. Determination of the analytic derivative dGmBydl could become more complex for those flux tubes.
</p>
</html>"));
        end Force;

        package Leakage "Leakage flux tubes with position-independent permeance and hence no force generation; mu_r=1"
          extends Modelica.Icons.VariantsPackage;

          model QuarterCylinder "Leakage flux from one edge to the opposite plane through a quarter cylinder"
            extends Modelica.Magnetic.FluxTubes.Interfaces.PartialLeakage;
            parameter SI.Length l = 0.1 "Axial length orthogonal to flux (=2*pi*r for cylindrical pole and r>>distance between edge and plane)" annotation(
              Dialog(group = "Parameters", groupImage = "modelica://Modelica/Resources/Images/Magnetic/FluxTubes/Shapes/Leakage/QuarterCylinder.png"));
          equation
            G_m = mu_0 * 0.52 * l;
            annotation(
              Documentation(info = "<html>
<p>
Please refer to the enclosing sub-package <a href=\"modelica://Modelica.Magnetic.FluxTubes.Shapes.Leakage\">Leakage</a> for a description of all elements of this package and to <a href=\"modelica://Modelica.Magnetic.FluxTubes.UsersGuide.Literature\">[Ro41]</a> for derivation and/or coefficients of the equation for permeance G_m.
</p>
</html>"));
          end QuarterCylinder;

          model QuarterHollowCylinder "Leakage flux in circumferential direction through a quarter hollow cylinder"
            extends Modelica.Magnetic.FluxTubes.Interfaces.PartialLeakage;
            parameter SI.Length l = 0.1 "Axial length orthogonal to flux (=2*pi*r for cylindrical pole and r>>r_i)" annotation(
              Dialog(group = "Parameters", groupImage = "modelica://Modelica/Resources/Images/Magnetic/FluxTubes/Shapes/Leakage/QuarterHollowCylinder.png"));
            parameter Real ratio(start = 1) "Constant ratio t/r_i";
          equation
            G_m = 2 * mu_0 * l * Modelica.Math.log(1 + ratio) / pi;
            annotation(
              Documentation(info = "<html>
<p>
Please refer to the enclosing sub-package <a href=\"modelica://Modelica.Magnetic.FluxTubes.Shapes.Leakage\">Leakage</a> for a description of all elements of this package and to <a href=\"modelica://Modelica.Magnetic.FluxTubes.UsersGuide.Literature\">[Ro41]</a> for derivation and/or coefficients of the equation for permeance G_m.
</p>
</html>"));
          end QuarterHollowCylinder;

          model HalfCylinder "Leakage flux through the edges of a half cylinder"
            extends Modelica.Magnetic.FluxTubes.Interfaces.PartialLeakage;
            parameter SI.Length l = 0.1 "Axial length orthogonal to flux (=2*pi*r for cylindrical pole and r>>distance between edges)" annotation(
              Dialog(group = "Parameters", groupImage = "modelica://Modelica/Resources/Images/Magnetic/FluxTubes/Shapes/Leakage/HalfCylinder.png"));
          equation
            G_m = mu_0 * 0.26 * l;
            annotation(
              Documentation(info = "<html>
<p>
Please refer to the enclosing sub-package <a href=\"modelica://Modelica.Magnetic.FluxTubes.Shapes.Leakage\">Leakage</a> for a description of all elements of this package and to <a href=\"modelica://Modelica.Magnetic.FluxTubes.UsersGuide.Literature\">[Ro41]</a> for derivation and/or coefficients of the equation for permeance G_m.
</p>
</html>"));
          end HalfCylinder;

          model HalfHollowCylinder "Leakage flux in circumferential direction through a half hollow cylinder"
            extends Modelica.Magnetic.FluxTubes.Interfaces.PartialLeakage;
            parameter SI.Length l = 0.1 "Axial length orthogonal to flux (=2*pi*r for cylindrical pole and r>>r_i)" annotation(
              Dialog(group = "Parameters", groupImage = "modelica://Modelica/Resources/Images/Magnetic/FluxTubes/Shapes/Leakage/HalfHollowCylinder.png"));
            parameter Real ratio(start = 1) "Constant ratio t/r_i";
          equation
            G_m = mu_0 * l * Modelica.Math.log(1 + ratio) / pi;
            annotation(
              Documentation(info = "<html>
<p>
Please refer to the enclosing sub-package <a href=\"modelica://Modelica.Magnetic.FluxTubes.Shapes.Leakage\">Leakage</a> for a description of all elements of this package and to <a href=\"modelica://Modelica.Magnetic.FluxTubes.UsersGuide.Literature\">[Ro41]</a> for derivation and/or coefficients of the equation for permeance G_m.
</p>
</html>"));
          end HalfHollowCylinder;

          model QuarterSphere "Leakage flux through the corners of a quarter sphere"
            extends Modelica.Magnetic.FluxTubes.Interfaces.PartialLeakage;
            parameter SI.Radius r = 0.005 "Radius of quarter sphere" annotation(
              Dialog(group = "Parameters", groupImage = "modelica://Modelica/Resources/Images/Magnetic/FluxTubes/Shapes/Leakage/QuarterSphere.png"));
          equation
            G_m = mu_0 * 0.077 * 2 * r;
            annotation(
              Documentation(info = "<html>
<p>
Please refer to the enclosing sub-package <a href=\"modelica://Modelica.Magnetic.FluxTubes.Shapes.Leakage\">Leakage</a> for a description of all elements of this package and to <a href=\"modelica://Modelica.Magnetic.FluxTubes.UsersGuide.Literature\">[Ro41]</a> for derivation and/or coefficients of the equation for permeance G_m.
</p>
</html>"));
          end QuarterSphere;

          model QuarterHollowSphere "Leakage flux through the edges of a quarter hollow sphere"
            extends Modelica.Magnetic.FluxTubes.Interfaces.PartialLeakage;
            parameter SI.Length t(start = 0.01) "Thickness of spherical shell" annotation(
              Dialog(group = "Parameters", groupImage = "modelica://Modelica/Resources/Images/Magnetic/FluxTubes/Shapes/Leakage/QuarterHollowSphere.png"));
          equation
            G_m = mu_0 * 0.25 * t;
            annotation(
              Documentation(info = "<html>
<p>
Please refer to the enclosing sub-package <a href=\"modelica://Modelica.Magnetic.FluxTubes.Shapes.Leakage\">Leakage</a> for a description of all elements of this package and to <a href=\"modelica://Modelica.Magnetic.FluxTubes.UsersGuide.Literature\">[Ro41]</a> for derivation and/or coefficients of the equation for permeance G_m.
</p>
</html>"));
          end QuarterHollowSphere;

          model EighthOfSphere "Leakage flux through one edge and the opposite plane of an eighth of a sphere"
            extends Modelica.Magnetic.FluxTubes.Interfaces.PartialLeakage;
            parameter SI.Radius r = 0.01 "Radius of eighth of sphere" annotation(
              Dialog(group = "Parameters", groupImage = "modelica://Modelica/Resources/Images/Magnetic/FluxTubes/Shapes/Leakage/EighthOfSphere.png"));
          equation
            G_m = mu_0 * 0.308 * r;
            annotation(
              Documentation(info = "<html>
<p>
Please refer to the enclosing sub-package <a href=\"modelica://Modelica.Magnetic.FluxTubes.Shapes.Leakage\">Leakage</a> for a description of all elements of this package and to <a href=\"modelica://Modelica.Magnetic.FluxTubes.UsersGuide.Literature\">[Ro41]</a> for derivation and/or coefficients of the equation for permeance G_m.
</p>
</html>"));
          end EighthOfSphere;

          model EighthOfHollowSphere "Leakage flux through one edge and the opposite plane of an eighth of a hollow sphere"
            extends Modelica.Magnetic.FluxTubes.Interfaces.PartialLeakage;
            parameter SI.Length t(start = 0.01) "Thickness of spherical shell" annotation(
              Dialog(group = "Parameters", groupImage = "modelica://Modelica/Resources/Images/Magnetic/FluxTubes/Shapes/Leakage/EighthOfHollowSphere.png"));
          equation
            G_m = mu_0 * 0.5 * t;
            annotation(
              Documentation(info = "<html>
<p>
Please refer to the enclosing sub-package <a href=\"modelica://Modelica.Magnetic.FluxTubes.Shapes.Leakage\">Leakage</a> for a description of all elements of this package and to <a href=\"modelica://Modelica.Magnetic.FluxTubes.UsersGuide.Literature\">[Ro41]</a> for derivation and/or coefficients of the equation for permeance G_m.
</p>
</html>"));
          end EighthOfHollowSphere;

          model CoaxCylindersEndFaces "Leakage flux between the end planes of a inner solid cylinder and a coaxial outer hollow cylinder"
            extends Modelica.Magnetic.FluxTubes.Interfaces.PartialLeakage;
            parameter SI.Radius r_0 = 10e-3 "Radius of inner solid cylinder" annotation(
              Dialog(group = "Parameters", groupImage = "modelica://Modelica/Resources/Images/Magnetic/FluxTubes/Shapes/Leakage/CoaxCylindersEndFaces.png"));
            parameter SI.Radius r_1 = 17e-3 "Inner radius of outer hollow cylinder";
            parameter SI.Radius r_2 = 20e-3 "Outer radius of outer hollow cylinder";
            final parameter SI.Distance l_g = r_1 - r_0 "Radial gap length between both cylinders";
            final parameter SI.Length t = r_2 - r_1 "Radial thickness of outer hollow cylinder";
          equation
// [Ro41], p. 139, Eq. (22)
            G_m = if t <= r_0 then 2 * mu_0 * (r_0 + l_g / 2) * Modelica.Math.log(1 + 2 * t / l_g) else 2 * mu_0 * (r_0 + l_g / 2) * Modelica.Math.log(1 + 2 * r_0 / l_g);
            annotation(
              Documentation(info = "<html>
<p>
Please refer to the enclosing sub-package <a href=\"modelica://Modelica.Magnetic.FluxTubes.Shapes.Leakage\">Leakage</a> for a description of all elements of this package and to <a href=\"modelica://Modelica.Magnetic.FluxTubes.UsersGuide.Literature\">[Ro41]</a> for derivation and/or coefficients of the equation for permeance G_m.
</p>
</html>"));
          end CoaxCylindersEndFaces;
          annotation(
            Documentation(info = "<html>
<p>
Please have a look at <a href=\"modelica://Modelica.Magnetic.FluxTubes.UsersGuide.ReluctanceForceCalculation\">UsersGuide.ReluctanceForceCalculation</a> for an explanation of the different flux tube categories and resulting sub-packages.
</p>

<p>
The permeances of all elements of this package are calculated from their geometry. These flux tube elements are intended for modelling of leakage fields through vacuum, air and other media with a relative permeability mu_r=1. <a href=\"modelica://Modelica.Magnetic.FluxTubes.Basic.LeakageWithCoefficient\">Basic.LeakageWithCoefficient</a> accounts for leakage not by the geometry of flux tubes, but by a coupling coefficient c_usefulFlux.
</p>

<p>
All dimensions are defined as parameters. As a result, the shape of these elements will remain constant during dynamic simulation of actuators and reluctance forces will not be generated in these flux tube elements. A simple leakage flux tube with reluctance force generation is provided with the element <a href=\"modelica://Modelica.Magnetic.FluxTubes.Shapes.Force.LeakageAroundPoles\">Force.LeakageAroundPoles</a>. In cases where the accuracy of that element is not sufficient, the leakage elements of this package can be adapted and extended so that they are able to change their shape with armature motion and to generate reluctance forces. This requires an extension of the partial model <a href=\"modelica://Modelica.Magnetic.FluxTubes.Interfaces.PartialForce\">Interfaces.PartialForce</a>, a higher variability of the variables representing the flux tube's dimensions, definition of a relationship between armature position and these dimensions and determination of the analytic derivative dG_m/dx of the flux tube's permeance G_m with respect to armature position x.
</p>
</html>"));
        end Leakage;
      end Shapes;

      package Material "Magnetisation characteristics of common soft magnetic and hard magnetic materials"
        extends Modelica.Icons.MaterialPropertiesPackage;

        package SoftMagnetic "Characteristics mu_r(B) of common soft magnetic materials; hysteresis neglected"
          extends Modelica.Icons.MaterialPropertiesPackage;

          record BaseData "Coefficients for approximation of soft magnetic materials"
            extends Modelica.Icons.Record;
            parameter SI.RelativePermeability mu_i = 1 "Initial relative permeability at B=0";
            parameter SI.MagneticFluxDensity B_myMax = 1 "Flux density at maximum relative permeability";
            parameter Real c_a = 1 "Coefficient of approximation function";
            parameter Real c_b = 1 "Coefficient of approximation function";
            parameter Real n = 1 "Exponent of approximation function";
            annotation(
              Documentation(info = "<html>
<p>
The parameters needed for <a href=\"modelica://Modelica.Magnetic.FluxTubes.Material.SoftMagnetic.mu_rApprox\">approximation of the magnetisation characteristics</a> of included soft magnetic materials are declared in this record.
</p>
</html>"));
          end BaseData;

          package Steel "Various ferromagnetic steels"
            extends Modelica.Icons.MaterialPropertiesPackage;

            record Steel_9SMnPb28 "9SMnPb28 (1.0718)"
              extends Modelica.Magnetic.FluxTubes.Material.SoftMagnetic.BaseData(mu_i = 400, B_myMax = 1.488, c_a = 1200, c_b = 3, n = 12.5);
              annotation(
                Documentation(info = "<html>
<p>
Please refer to the description of  the enclosing package <a href=\"modelica://Modelica.Magnetic.FluxTubes.Material.SoftMagnetic\">SoftMagnetic</a> for a description of all soft magnetic material characteristics of this package.
</p>
</html>"));
            end Steel_9SMnPb28;

            record Steel_9SMn28K "9SMn28k (1.0715)"
              extends Modelica.Magnetic.FluxTubes.Material.SoftMagnetic.BaseData(mu_i = 500, B_myMax = 1.036, c_a = 43414, c_b = 35.8, n = 14);
              annotation(
                Documentation(info = "<html>
<p>
Please refer to the description of  the enclosing package <a href=\"modelica://Modelica.Magnetic.FluxTubes.Material.SoftMagnetic\">SoftMagnetic</a> for a description of all soft magnetic material characteristics of this package.
</p>
</html>"));
            end Steel_9SMn28K;

            record DC01 "DC01 (1.0330, previously St2)"
              extends Modelica.Magnetic.FluxTubes.Material.SoftMagnetic.BaseData(mu_i = 5, B_myMax = 1.1, c_a = 6450, c_b = 3.65, n = 7.7);
              annotation(
                Documentation(info = "<html>
<p>
Please refer to the description of  the enclosing package <a href=\"modelica://Modelica.Magnetic.FluxTubes.Material.SoftMagnetic\">SoftMagnetic</a> for a description of all soft magnetic material characteristics of this package.
</p>
</html>"));
            end DC01;

            record DC03 "DC03 (1.0347, previously St3)"
              extends Modelica.Magnetic.FluxTubes.Material.SoftMagnetic.BaseData(mu_i = 0, B_myMax = 1.05, c_a = 27790, c_b = 16, n = 10.4);
              annotation(
                Documentation(info = "<html>
<p>
Please refer to the description of  the enclosing package <a href=\"modelica://Modelica.Magnetic.FluxTubes.Material.SoftMagnetic\">SoftMagnetic</a> for a description of all soft magnetic material characteristics of this package.
</p>
</html>"));
            end DC03;

            record X6Cr17 "X6Cr17 (1.4016)"
              extends Modelica.Magnetic.FluxTubes.Material.SoftMagnetic.BaseData(mu_i = 274, B_myMax = 1.1, c_a = 970, c_b = 1.2, n = 8.3);
              annotation(
                Documentation(info = "<html>
<p>
Please refer to the description of  the enclosing package <a href=\"modelica://Modelica.Magnetic.FluxTubes.Material.SoftMagnetic\">SoftMagnetic</a> for a description of all soft magnetic material characteristics of this package.
</p>
</html>"));
            end X6Cr17;

            record AISI_1008 "AISI 1008 (1.0204)"
              extends Modelica.Magnetic.FluxTubes.Material.SoftMagnetic.BaseData(mu_i = 200, B_myMax = 1.17, c_a = 8100, c_b = 2.59, n = 10);
              annotation(
                Documentation(info = "<html>
<p>
Please refer to the description of  the enclosing package <a href=\"modelica://Modelica.Magnetic.FluxTubes.Material.SoftMagnetic\">SoftMagnetic</a> for a description of all soft magnetic material characteristics of this package.
</p>
</html>"));
            end AISI_1008;

            record AISI_12L14 "AISI 12L14 (1.0718)"
              extends Modelica.Magnetic.FluxTubes.Material.SoftMagnetic.BaseData(mu_i = 10, B_myMax = 0.94, c_a = 5900, c_b = 4.19, n = 6.4);
              annotation(
                Documentation(info = "<html>
<p>
Please refer to the description of  the enclosing package <a href=\"modelica://Modelica.Magnetic.FluxTubes.Material.SoftMagnetic\">SoftMagnetic</a> for a description of all soft magnetic material characteristics of this package.
</p>
</html>"));
            end AISI_12L14;
            annotation(
              Documentation(info = "<html>
<p>
Please refer to the description of  the enclosing package <a href=\"modelica://Modelica.Magnetic.FluxTubes.Material.SoftMagnetic\">SoftMagnetic</a> for a description of all soft magnetic material characteristics of this package.
</p>
</html>"));
          end Steel;

          package ElectricSheet "Various electric sheets"
            extends Modelica.Icons.MaterialPropertiesPackage;

            record M330_50A "M330-50A (1.0809) @ 50Hz"
              extends Modelica.Magnetic.FluxTubes.Material.SoftMagnetic.BaseData(mu_i = 500, B_myMax = 0.7, c_a = 24000, c_b = 9.38, n = 9.6);
              annotation(
                Documentation(info = "<html>
<p>
Please refer to the description of  the enclosing package <a href=\"modelica://Modelica.Magnetic.FluxTubes.Material.SoftMagnetic\">SoftMagnetic</a> for a description of all soft magnetic material characteristics of this package.
</p>
<p>
Sample: complete core after machining and packet assembling<br>
</p>
</html>"));
            end M330_50A;

            record M350_50A "M350-50A (1.0810) @ 50Hz"
              extends Modelica.Magnetic.FluxTubes.Material.SoftMagnetic.BaseData(mu_i = 1210, B_myMax = 1.16, c_a = 24630, c_b = 2.44, n = 14);
              annotation(
                Documentation(info = "<html>
<p>
Please refer to the description of  the enclosing package <a href=\"modelica://Modelica.Magnetic.FluxTubes.Material.SoftMagnetic\">SoftMagnetic</a> for a description of all soft magnetic material characteristics of this package.
</p>
<p>
Sample: sheet strip<br>
Measurement: Epstein frame
</p>
</html>"));
            end M350_50A;

            record M530_50A "M530-50A (1.0813) @ 50Hz"
              extends Modelica.Magnetic.FluxTubes.Material.SoftMagnetic.BaseData(mu_i = 2120, B_myMax = 1.25, c_a = 12400, c_b = 1.6, n = 13.5);
              annotation(
                Documentation(info = "<html>
<p>
Please refer to the description of  the enclosing package <a href=\"modelica://Modelica.Magnetic.FluxTubes.Material.SoftMagnetic\">SoftMagnetic</a> for a description of all soft magnetic material characteristics of this package.
</p>
<p>
Sample: sheet strip<br>
Measurement: Epstein frame
</p>
</html>"));
            end M530_50A;

            record M700_100A "M700-100A (1.0826) @ 50Hz"
              extends Modelica.Magnetic.FluxTubes.Material.SoftMagnetic.BaseData(mu_i = 1120, B_myMax = 1.2, c_a = 20750, c_b = 3.55, n = 13.15);
              annotation(
                Documentation(info = "<html>
<p>
Please refer to the description of  the enclosing package <a href=\"modelica://Modelica.Magnetic.FluxTubes.Material.SoftMagnetic\">SoftMagnetic</a> for a description of all soft magnetic material characteristics of this package.
</p>
<p>
Sample: sheet strip<br>
Measurement: Epstein frame
</p>
</html>"));
            end M700_100A;

            record M940_100A "M940-100A @ 50Hz"
              extends Modelica.Magnetic.FluxTubes.Material.SoftMagnetic.BaseData(mu_i = 680, B_myMax = 1.26, c_a = 17760, c_b = 3.13, n = 13.9);
              annotation(
                Documentation(info = "<html>
<p>
Please refer to the description of  the enclosing package <a href=\"modelica://Modelica.Magnetic.FluxTubes.Material.SoftMagnetic\">SoftMagnetic</a> for a description of all soft magnetic material characteristics of this package.
</p>
<p>
Sample: sheet strip<br>
Measurement: Epstein frame
</p>
</html>"));
            end M940_100A;
            annotation(
              Documentation(info = "<html>
<p>
Please refer to the description of  the enclosing package <a href=\"modelica://Modelica.Magnetic.FluxTubes.Material.SoftMagnetic\">SoftMagnetic</a> for a description of all soft magnetic material characteristics of this package.
</p>
</html>"));
          end ElectricSheet;

          package PureIron "Pure iron"
            extends Modelica.Icons.MaterialPropertiesPackage;

            record RFe80 "Hyperm 0 (RFe80)"
              extends Modelica.Magnetic.FluxTubes.Material.SoftMagnetic.BaseData(mu_i = 123, B_myMax = 1.27, c_a = 44410, c_b = 6.4, n = 10);
              annotation(
                Documentation(info = "<html>
<p>
Please refer to the description of  the enclosing package <a href=\"modelica://Modelica.Magnetic.FluxTubes.Material.SoftMagnetic\">SoftMagnetic</a> for a description of all soft magnetic material characteristics of this package.
</p>
<p>
Source of B(H) characteristics: Product catalogue <i>Magnequench</i>, 2000
</p>
</html>"));
            end RFe80;

            record VacoferS2 "VACOFER S2 (99.95% Fe)"
              extends Modelica.Magnetic.FluxTubes.Material.SoftMagnetic.BaseData(mu_i = 2666, B_myMax = 1.15, c_a = 187000, c_b = 4.24, n = 19);
              annotation(
                Documentation(info = "<html>
<p>
Please refer to the description of  the enclosing package <a href=\"modelica://Modelica.Magnetic.FluxTubes.Material.SoftMagnetic\">SoftMagnetic</a> for a description of all soft magnetic material characteristics of this package.
</p>
<dl>
<dt>Source of B(H) characteristics:</dt>
    <dd><p><i>Boll, R.</i>: Weichmagnetische Werkstoffe: Einf&uuml;hrung in den Magnetismus, VAC-Werkstoffe und ihre Anwendungen. 4th ed. Berlin, M&uuml;nchen: Siemens Aktiengesellschaft 1990</p>
    </dd>
</dl>
</html>"));
            end VacoferS2;
          end PureIron;

          package CobaltIron "Cobalt iron"
            extends Modelica.Icons.MaterialPropertiesPackage;

            record Vacoflux50 "Vacoflux 50 (50% CoFe)"
              extends Modelica.Magnetic.FluxTubes.Material.SoftMagnetic.BaseData(mu_i = 3850, B_myMax = 1.75, c_a = 11790, c_b = 2.63, n = 15.02);
              annotation(
                Documentation(info = "<html>
<p>
Please refer to the description of  the enclosing package <a href=\"modelica://Modelica.Magnetic.FluxTubes.Material.SoftMagnetic\">SoftMagnetic</a> for a description of all soft magnetic material characteristics of this package.
</p>
<p>
Source of B(H) characteristics: VACUUMSCHMELZE GmbH &amp; Co. KG, Germany
</p>
</html>"));
            end Vacoflux50;
          end CobaltIron;

          package NickelIron "Nickel iron"
            extends Modelica.Icons.MaterialPropertiesPackage;

            record MuMetall "MUMETALL (77% NiFe)"
              extends Modelica.Magnetic.FluxTubes.Material.SoftMagnetic.BaseData(mu_i = 27300, B_myMax = 0.46, c_a = 1037500, c_b = 3.67, n = 10);
              annotation(
                Documentation(info = "<html>
<p>
Please refer to the description of  the enclosing package <a href=\"modelica://Modelica.Magnetic.FluxTubes.Material.SoftMagnetic\">SoftMagnetic</a> for a description of all soft magnetic material characteristics of this package.
</p>
<p>
Source of B(H) characteristics:
</p>
<ul>
<li><i>Boll, R.</i>: Weichmagnetische Werkstoffe: Einf&uuml;hrung in den Magnetismus, VAC-Werkstoffe und ihre Anwendungen. 4th ed. Berlin, M&uuml;nchen: Siemens Aktiengesellschaft 1990</li>
</ul>
</html>"));
            end MuMetall;

            record Permenorm3601K3 "PERMENORM 3601 K3 (36% NiFe)"
              extends Modelica.Magnetic.FluxTubes.Material.SoftMagnetic.BaseData(mu_i = 3000, B_myMax = 0.67, c_a = 50000, c_b = 2.39, n = 9.3);
              annotation(
                Documentation(info = "<html>
<p>
Please refer to the description of  the enclosing package <a href=\"modelica://Modelica.Magnetic.FluxTubes.Material.SoftMagnetic\">SoftMagnetic</a> for a description of all soft magnetic material characteristics of this package.
</p>
<p>
Source of B(H) characteristics:
</p>
<ul>
<li><i>Boll, R.</i>: Weichmagnetische Werkstoffe: Einf&uuml;hrung in den Magnetismus, VAC-Werkstoffe und ihre Anwendungen. 4th ed. Berlin, M&uuml;nchen: Siemens Aktiengesellschaft 1990</li>
</ul>
</html>"));
            end Permenorm3601K3;
          end NickelIron;

          function mu_rApprox "Approximation of relative permeability mu_r as a function of flux density B for soft magnetic materials"
            extends Modelica.Icons.Function;
            input Modelica.SIunits.MagneticFluxDensity B "Flux density in ferromagnetic flux tube element";
            //Material specific parameter set:
            input Modelica.SIunits.RelativePermeability mu_i "Initial relative permeability at B=0";
            input Modelica.SIunits.MagneticFluxDensity B_myMax "Flux density at maximum relative permeability";
            input Real c_a "Coefficient of approximation function";
            input Real c_b "Coefficient of approximation function";
            input Real n "Exponent of approximation function";
            output Modelica.SIunits.RelativePermeability mu_r "Relative magnetic permeability of ferromagnetic flux tube element";
          protected
            Real B_N "Flux density B normalized to flux density at maximum relative permeability B_myMax";
          algorithm
            B_N := abs(B / B_myMax);
            mu_r := 1 + (mu_i - 1 + c_a * B_N) / (1 + c_b * B_N + B_N ^ n);
            annotation(
              Documentation(info = "<html>
<p>
The relative permeability mu_r as a function of flux density B for all soft magnetic materials currently included in this library is approximated with the following function <a href=\"modelica://Modelica.Magnetic.FluxTubes.UsersGuide.Literature\">[Ro00]</a>:
</p>

<p>
<IMG src=\"modelica://Modelica/Resources/Images/Magnetic/FluxTubes/Material/SoftMagnetic/eq_mu_rApprox.png\" ALT=\"Equation for approximation mu_r(B)\"/>
</p>

<p>
Two of the five parameters of this equation have a physical meaning, namely the initial relative permeability mu_i at B=0 and the magnetic flux density at maximum permeability B_myMax. B_N is the flux density normalized to latter parameter.</p>
</html>"));
          end mu_rApprox;
          annotation(
            Documentation(info = "<html>
<p>
The magnetisation characteristics mu_r(B) of all soft magnetic materials currently included in this library are approximated with a <a href=\"modelica://Modelica.Magnetic.FluxTubes.Material.SoftMagnetic.mu_rApprox\">function</a>. Each material is characterised by the five parameters of this function. The approximated characteristics mu_r(B) for most of the ferromagnetic materials currently included are shown in the plots below (solid lines) together with the original data points compiled from measurements and literature.
</p>

<p>
<IMG src=\"modelica://Modelica/Resources/Images/Magnetic/FluxTubes/Material/SoftMagnetic/Steel.png\" ALT=\"Approximated magnetization characteristics of selected steels\"><br>
<IMG src=\"modelica://Modelica/Resources/Images/Magnetic/FluxTubes/Material/SoftMagnetic/Miscellaneous.png\" ALT=\"Approximated magnetization characteristics of miscellaneous soft magnetic materials\"><br>
<IMG src=\"modelica://Modelica/Resources/Images/Magnetic/FluxTubes/Material/SoftMagnetic/ElectricSheet.png\" ALT=\"Approximated magnetization characteristics of included electric sheets\"><br>
</p>

<p>
For the nonlinear curve fit, data points for high flux densities (approximately B>1T) have been weighted higher than the ones for low flux densities. This is due to the large impact of saturated ferromagnetic sections in a magnetic circuit compared to that of non-saturated sections with relative permeabilities mu_r>>1.
</p>

<p>
Note that the magnetisation characteristics largely depend on possible previous machining and on measurement conditions. A virgin material normally has a considerably higher permeability than the same material after machining (and packet assembling in case of electric sheets). This is indicated in the above plots by different magnetisation curves for similar materials. In most cases, the original data points represent commutating curves obtained with measurements at 50Hz.
</p>

<p>
Additional user-specific materials can be defined as needed. This requires determination of the approximation parameters from the original data points, preferably with a nonlinear curve fit.
</p>
</html>"));
        end SoftMagnetic;

        package HardMagnetic "Characteristics of common permanent magnetic materials (temperature dependence considered)"
          extends Modelica.Icons.MaterialPropertiesPackage;

          record BaseData "Record for permanent magnetic material data"
            extends Modelica.Icons.Record;
            parameter SI.MagneticFieldStrength H_cBRef = 1 "Coercivity at reference temperature";
            parameter SI.MagneticFluxDensity B_rRef = 1 "Remanence at reference temperature";
            parameter SI.Temperature T_ref = 293.15 "Reference temperature";
            parameter Modelica.SIunits.LinearTemperatureCoefficient alpha_Br = 0 "Temperature coefficient of remanence at reference temperature";
            parameter SI.Temperature T_op = 293.15 "Operating temperature";
            final parameter SI.MagneticFluxDensity B_r = B_rRef * (1 + alpha_Br * (T_op - T_ref)) "Remanence at operating temperature";
            final parameter SI.MagneticFieldStrength H_cB = H_cBRef * (1 + alpha_Br * (T_op - T_ref)) "Coercivity at operating temperature";
            final parameter SI.RelativePermeability mu_r = B_r / (mu_0 * H_cB) "Relative permeability";
            annotation(
              Documentation(info = "<html>
<p>
Please refer to the description of  the enclosing package <a href=\"modelica://Modelica.Magnetic.FluxTubes.Material.HardMagnetic\">HardMagnetic</a> for a description of all permanent magnetic material characteristics of this package.
</p>
</html>"));
          end BaseData;

          record NdFeB "NdFeB sintered; exemplary values"
            extends Modelica.Magnetic.FluxTubes.Material.HardMagnetic.BaseData(H_cBRef = 900000, B_rRef = 1.2, T_ref = 20 + 273.15, alpha_Br = -0.001);
            annotation(
              Documentation(info = "<html>
<p>
Please refer to the description of  the enclosing package <a href=\"modelica://Modelica.Magnetic.FluxTubes.Material.HardMagnetic\">HardMagnetic</a> for a description of all permanent magnetic material characteristics of this package.
</p>
</html>"));
          end NdFeB;

          record Sm2Co17 "Sm2Co17 sintered, exemplary values"
            extends Modelica.Magnetic.FluxTubes.Material.HardMagnetic.BaseData(H_cBRef = 750000, B_rRef = 1.02, T_ref = 20 + 273.15, alpha_Br = -0.0003);
            annotation(
              Documentation(info = "<html>
<p>
Please refer to the description of  the enclosing package <a href=\"modelica://Modelica.Magnetic.FluxTubes.Material.HardMagnetic\">HardMagnetic</a> for a description of all permanent magnetic material characteristics of this package.
</p>
</html>"));
          end Sm2Co17;

          record SmCo5 "SmCo5 sintered, exemplary values"
            extends Modelica.Magnetic.FluxTubes.Material.HardMagnetic.BaseData(H_cBRef = 720000, B_rRef = 0.95, T_ref = 20 + 273.15, alpha_Br = -0.0004);
            annotation(
              Documentation(info = "<html>
<p>
Please refer to the description of  the enclosing package <a href=\"modelica://Modelica.Magnetic.FluxTubes.Material.HardMagnetic\">HardMagnetic</a> for a description of all permanent magnetic material characteristics of this package.
</p>
</html>"));
          end SmCo5;

          record PlasticNdFeB "Plastic-bonded NdFeB, exemplary values"
            extends Modelica.Magnetic.FluxTubes.Material.HardMagnetic.BaseData(H_cBRef = 400000, B_rRef = 0.58, T_ref = 20 + 273.15, alpha_Br = -0.001);
            annotation(
              Documentation(info = "<html>
<p>
Please refer to the description of  the enclosing package <a href=\"modelica://Modelica.Magnetic.FluxTubes.Material.HardMagnetic\">HardMagnetic</a> for a description of all permanent magnetic material characteristics of this package.
</p>
</html>"));
          end PlasticNdFeB;

          record PlasticSmCo "Plastic-bonded Sm-Co, exemplary values"
            extends Modelica.Magnetic.FluxTubes.Material.HardMagnetic.BaseData(H_cBRef = 385000, B_rRef = 0.57, T_ref = 20 + 273.15, alpha_Br = -0.0004);
            annotation(
              Documentation(info = "<html>
<p>
Please refer to the description of  the enclosing package <a href=\"modelica://Modelica.Magnetic.FluxTubes.Material.HardMagnetic\">HardMagnetic</a> for a description of all permanent magnetic material characteristics of this package.
</p>
</html>"));
          end PlasticSmCo;

          record HardFerrite "Hard ferrite sintered, exemplary values"
            extends Modelica.Magnetic.FluxTubes.Material.HardMagnetic.BaseData(H_cBRef = 170000, B_rRef = 0.38, T_ref = 20 + 273.15, alpha_Br = -0.002);
            annotation(
              Documentation(info = "<html>
<p>
Please refer to the description of  the enclosing package <a href=\"modelica://Modelica.Magnetic.FluxTubes.Material.HardMagnetic\">HardMagnetic</a> for a description of all permanent magnetic material characteristics of this package.
</p>
</html>"));
          end HardFerrite;

          record PlasticHardFerrite "Plastic-bonded hard ferrite, exemplary values"
            extends Modelica.Magnetic.FluxTubes.Material.HardMagnetic.BaseData(H_cBRef = 130000, B_rRef = 0.21, T_ref = 20 + 273.15, alpha_Br = -0.002);
            annotation(
              Documentation(info = "<html>
<p>
Please refer to the description of  the enclosing package <a href=\"modelica://Modelica.Magnetic.FluxTubes.Material.HardMagnetic\">HardMagnetic</a> for a description of all permanent magnetic material characteristics of this package.
</p>
</html>"));
          end PlasticHardFerrite;
          annotation(
            Documentation(info = "<html>
<p>
Typical values for remanence, coercivity and the temperature coefficient of remanence are provided for the common permanent magnetic materials illustrated below.
<dl>
<dd>
<IMG src=\"modelica://Modelica/Resources/Images/Magnetic/FluxTubes/Material/HardMagnetic/HardMagneticMaterials.png\" ALT=\"Demagnetization characteristics of included permanent magnetic materials\">
</dd>
</dl>
<p>
Linear demagnetization curves are modelled. The characteristic, temperature-dependent \"knee\" of many permanent magnetic materials is not considered, since proper design of permanent magnetic circuits should avoid operation of permanent magnets \"below\" that point due to partial demagnetization. As a result, the temperature coefficient of coercivity is not considered. Only the temperature coefficient of remanence alpha_Br is accounted for, since it describes the dependence of the demagnetization curve on the temperature sufficiently for the region \"above the knee-point\".
</p>
<p>
Additional user-specific materials can be defined as needed.
</p>
</html>"));
        end HardMagnetic;
        annotation(
          Documentation(info = "<html>
</html>"));
      end Material;

      package Interfaces "Interfaces of magnetic network components"
        extends Modelica.Icons.InterfacesPackage;

        connector MagneticPort "Generic magnetic port"
          SI.MagneticPotential V_m "Magnetic potential at the port";
          flow SI.MagneticFlux Phi "Magnetic flux flowing into the port";
          annotation(
            defaultComponentName = "mag");
        end MagneticPort;

        connector PositiveMagneticPort "Positive magnetic port"
          extends Modelica.Magnetic.FluxTubes.Interfaces.MagneticPort;
          annotation(
            defaultComponentName = "port_p",
            Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, 100}, {100, -100}}, lineColor = {255, 127, 0}, fillColor = {255, 127, 0}, fillPattern = FillPattern.Solid)}),
            Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-100, 160}, {-100, 100}}, lineColor = {255, 127, 0}, textString = "%name"), Rectangle(extent = {{-40, 40}, {40, -40}}, lineColor = {255, 127, 0}, fillColor = {255, 127, 0}, fillPattern = FillPattern.Solid)}));
        end PositiveMagneticPort;

        connector NegativeMagneticPort "Negative magnetic port"
          extends Modelica.Magnetic.FluxTubes.Interfaces.MagneticPort;
          annotation(
            defaultComponentName = "port_n",
            Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, 100}, {100, -100}}, lineColor = {255, 127, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid)}),
            Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{100, 160}, {100, 100}}, lineColor = {255, 127, 0}, textString = "%name"), Rectangle(extent = {{-40, 40}, {40, -40}}, lineColor = {255, 127, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid)}));
        end NegativeMagneticPort;

        partial model PartialTwoPortsElementary "Partial component with two magnetic ports p and n for textual programming"
          Modelica.Magnetic.FluxTubes.Interfaces.PositiveMagneticPort port_p "Positive magnetic port" annotation(
            Placement(transformation(extent = {{-110, -10}, {-90, 10}}, rotation = 0)));
          Modelica.Magnetic.FluxTubes.Interfaces.NegativeMagneticPort port_n "Negative magnetic port" annotation(
            Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
          annotation(
            Documentation(info = "<html>
<p>
Partial model of a flux tube component with two magnetic ports:
the positive port connector port_p, and the negative port
connector port_n.
</p>
</html>"));
        end PartialTwoPortsElementary;

        partial model PartialTwoPorts "Partial component with magnetic potential difference between two magnetic ports p and n and magnetic flux Phi from p to n"
          extends Modelica.Magnetic.FluxTubes.Interfaces.PartialTwoPortsElementary;
          SI.MagneticPotentialDifference V_m "Magnetic potential difference between both ports";
          SI.MagneticFlux Phi(start = 0) "Magnetic flux from port_p to port_n";
        equation
          V_m = port_p.V_m - port_n.V_m;
          Phi = port_p.Phi;
          0 = port_p.Phi + port_n.Phi;
          annotation(
            Documentation(info = "<html>
<p>
It is assumed that the magnetic flux flowing into port_p is identical to the flux flowing out of port_n.
This magnetic flux is provided explicitly as flux Phi.
</p>
</html>"));
        end PartialTwoPorts;

        partial model PartialFixedShape "Base class for flux tubes with fixed shape during simulation; linear or non-linear material characteristics"
          extends Modelica.Magnetic.FluxTubes.Interfaces.PartialTwoPorts;
          parameter Boolean nonLinearPermeability = true "= true, if non-linear rel. permeability is used, otherwise constant rel. permeability" annotation(
            Dialog(group = "Material"),
            Evaluate = true);
          parameter SI.RelativePermeability mu_rConst = 1 "Constant relative permeability; used if nonLinearPermeability = false" annotation(
            Dialog(group = "Material", enable = not nonLinearPermeability));
          parameter Modelica.Magnetic.FluxTubes.Material.SoftMagnetic.BaseData material = Modelica.Magnetic.FluxTubes.Material.SoftMagnetic.BaseData() "Ferromagnetic material characteristics; used if nonLinearPermeability = true" annotation(
            choicesAllMatching = true,
            Dialog(group = "Material", enable = nonLinearPermeability));
          SI.Reluctance R_m "Magnetic reluctance";
          SI.Permeance G_m "Magnetic permeance";
          SI.MagneticFluxDensity B "Magnetic flux density";
          SI.CrossSection A "Cross-sectional area penetrated by magnetic flux";
          SI.MagneticFieldStrength H "Magnetic field strength";
          SI.RelativePermeability mu_r "Relative magnetic permeability";
        equation
          mu_r = if nonLinearPermeability then Modelica.Magnetic.FluxTubes.Material.SoftMagnetic.mu_rApprox(B, material.mu_i, material.B_myMax, material.c_a, material.c_b, material.n) else mu_rConst;
          R_m = 1 / G_m;
          V_m = Phi * R_m;
          B = Phi / A;
          H = B / (mu_0 * mu_r);
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-70, 30}, {70, -30}}, lineColor = {255, 128, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{-70, 0}, {-90, 0}}, color = {255, 128, 0}), Line(points = {{70, 0}, {90, 0}}, color = {255, 128, 0}), Text(extent = {{-150, -100}, {150, -60}}, textString = "%name", lineColor = {0, 0, 255})}),
            Documentation(info = "<html>
<p>
Please refer to the description of  the sub-package <a href=\"modelica://Modelica.Magnetic.FluxTubes.Shapes.FixedShape\">Shapes.FixedShape</a> for utilisation of this partial model.
</p>
</html>"));
        end PartialFixedShape;

        partial model PartialForce "Base class for flux tubes with reluctance force generation; constant permeability"
          extends Modelica.Magnetic.FluxTubes.Interfaces.PartialTwoPorts;
          parameter Boolean useSupport = false "= true, if support flange enabled, otherwise implicitly grounded" annotation(
            Evaluate = true,
            HideResult = true);
          parameter SI.RelativePermeability mu_r(start = 1) "Relative magnetic permeability";
          SI.Force F_m "Reluctance force";
          Modelica.SIunits.Length s = flange.s - s_support "Distance between flange and support";
          SI.Reluctance R_m "Magnetic reluctance";
          SI.Permeance G_m "Magnetic permeance";
          SI.Permeability dGmBydx "Derivative of permeance with respect to armature position";
          parameter Integer dlBydx = 1 "Derivative of flux tube's varying dimension with respect to armature position; set to +1 or -1";
          Modelica.Mechanics.Translational.Interfaces.Flange_b flange "Generated reluctance force at armature position" annotation(
            Placement(transformation(extent = {{-10, 90}, {10, 110}}, rotation = 0)));
          Modelica.Mechanics.Translational.Interfaces.Support support(s = s_support, f = -flange.f) if useSupport "Support/housing of component" annotation(
            Placement(transformation(extent = {{-10, -110}, {10, -90}}, rotation = 0)));
        protected
          Modelica.SIunits.Length s_support "Absolute position of support flange";
        equation
          V_m = Phi * R_m;
          R_m = 1 / G_m;
          F_m = 0.5 * V_m ^ 2 * dGmBydx;
          if not useSupport then
            s_support = 0;
          end if;
          flange.f = -F_m;
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{30, 30}, {70, -30}}, lineColor = {255, 128, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{-70, 0}, {-90, 0}}, color = {255, 128, 0}), Line(points = {{70, 0}, {90, 0}}, color = {255, 128, 0}), Text(extent = {{-150, -80}, {150, -40}}, textString = "%name", lineColor = {0, 0, 255}), Line(points = {{-10, -100}, {-30, -120}}, color = {0, 0, 0}), Line(points = {{-30, -100}, {-50, -120}}, color = {0, 0, 0}), Line(points = {{-30, -100}, {30, -100}}, color = {0, 0, 0}), Line(points = {{10, -100}, {-10, -120}}, color = {0, 0, 0}), Line(points = {{30, -100}, {10, -120}}, color = {0, 0, 0}), Rectangle(extent = {{-70, 30}, {-30, -30}}, lineColor = {255, 128, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-10, 80}, {10, -30}}, lineColor = {0, 0, 0}, fillColor = {0, 127, 0}, fillPattern = FillPattern.Solid)}),
            Documentation(info = "<html>
<p>
Please refer to the description of  the sub-package <a href=\"modelica://Modelica.Magnetic.FluxTubes.Shapes.Force\">Shapes.Force</a> for utilisation of this partial model.
</p>
</html>"));
        end PartialForce;

        partial model PartialLeakage "Base class for leakage flux tubes with position-independent permeance and hence no force generation; mu_r=1"
          extends Modelica.Magnetic.FluxTubes.Interfaces.PartialTwoPorts;
          SI.Reluctance R_m "Magnetic reluctance";
          SI.Permeance G_m "Magnetic permeance";
        equation
          V_m = Phi * R_m;
          R_m = 1 / G_m;
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-70, 30}, {70, -30}}, lineColor = {255, 128, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{-70, 0}, {-90, 0}}, color = {255, 128, 0}), Line(points = {{70, 0}, {90, 0}}, color = {255, 128, 0}), Text(extent = {{-150, -100}, {150, -60}}, textString = "%name", lineColor = {0, 0, 255})}),
            Documentation(info = "<html>
<p>
Please refer to the description of  the sub-package <a href=\"modelica://Modelica.Magnetic.FluxTubes.Shapes.Leakage\">Shapes.Leakage</a> for utilisation of this partial model.
</p>
</html>"));
        end PartialLeakage;
        annotation(
          Documentation(info = "<html>
<p>
This package contains connectors for the magnetic domain and partial models for lumped magnetic network components.
</p>

</html>"));
      end Interfaces;

      package Sources "Sources of different complexity of magnetomotive force and magnetic flux"
        extends Modelica.Icons.SourcesPackage;

        model ConstantMagneticPotentialDifference "Constant magnetomotive force"
          extends Modelica.Magnetic.FluxTubes.Interfaces.PartialTwoPortsElementary;
          parameter SI.MagneticPotentialDifference V_m "Magnetic potential difference";
          SI.MagneticFlux Phi "Magnetic flux from port_p to port_n";
        equation
          V_m = port_p.V_m - port_n.V_m;
          Phi = port_p.Phi;
          0 = port_p.Phi + port_n.Phi;
          annotation(
            defaultComponentName = "constantSource",
            Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-50, -50}, {50, 50}}, lineColor = {255, 127, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{100, 0}, {50, 0}}, color = {255, 127, 0}), Line(points = {{-50, 0}, {-100, 0}}, color = {255, 127, 0}), Text(extent = {{-80, -20}, {-80, -40}}, lineColor = {255, 128, 0}, textString = "+"), Text(extent = {{80, -20}, {80, -40}}, lineColor = {255, 128, 0}, textString = "-"), Text(extent = {{-150, -110}, {150, -70}}, textString = "%name", lineColor = {0, 0, 255}), Line(points = {{-50, 0}, {50, 0}}, color = {255, 127, 0})}),
            Documentation(info = "<html>
<p>
Magnetic circuits under steady-state conditions, i.e., with stationary magnetic fields (change of magnetic flux  d&Phi;/dt = 0) can be described with constant sources of a magnetic potential difference or magnetomotive force (mmf). Constant magnetic potential differences are imposed by
</p>
<ul>
<li>coils with stationary current (di / dt = 0) and </li>
<li>permanent magnets modelled with <i>Th&eacute;venin</i>'s equivalent magnetic circuit. </li>
</ul>
<p>
For modelling of reluctance actuators with this source component it is assumed that the armature is fixed so that no motion-induced flux change d&Phi;/dt can occur.
</p>
</html>"));
        end ConstantMagneticPotentialDifference;

        model SignalMagneticPotentialDifference "Signal-controlled magnetomotive force"
          extends Modelica.Magnetic.FluxTubes.Interfaces.PartialTwoPortsElementary;
          Modelica.Blocks.Interfaces.RealInput V_m(unit = "A") "Magnetic potential difference" annotation(
            Placement(transformation(origin = {0, 90}, extent = {{10, -10}, {-10, 10}}, rotation = 90)));
          SI.MagneticFlux Phi "Magnetic flux from port_p to port_n";
        equation
          V_m = port_p.V_m - port_n.V_m;
          Phi = port_p.Phi;
          0 = port_p.Phi + port_n.Phi;
          annotation(
            defaultComponentName = "signalSource",
            Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-100, 0}, {-50, 0}}, color = {255, 127, 0}), Line(points = {{50, 0}, {100, 0}}, color = {255, 127, 0}), Text(extent = {{-80, -20}, {-80, -40}}, lineColor = {255, 128, 0}, textString = "+"), Text(extent = {{80, -20}, {80, -40}}, lineColor = {255, 128, 0}, textString = "-"), Line(points = {{0, 100}, {0, 50}}, color = {255, 127, 0}), Text(extent = {{-150, -110}, {150, -70}}, textString = "%name", lineColor = {0, 0, 255}), Ellipse(extent = {{-50, -50}, {50, 50}}, lineColor = {255, 127, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{-50, 0}, {50, 0}}, color = {255, 127, 0})}),
            Documentation(info = "<html>
<p>
In electromagnetic devices, a change of a coil's magnetic flux linkage &Psi; reacts on the electrical subsystem in that a voltage v is induced due to <i>Faraday</i>'s law:
</p>
<pre>
    v = - d&Psi;/dt
</pre>
<p>This reaction can possibly be neglected for</p>
<ul>
<li>modelling of electromagnetic actuators under quasi-stationary conditions (slow current change, slow armature motion),
<li>modelling of current-controlled electromagnetic actuators (ideal current source) and</li>
<li>for system simulation where the system dynamics is not governed by an electromagnetic actuator, but by the surrounding subsystems.</li>
</ul>
<p>
In these cases, the magnetic potential difference or magnetomotive force imposed by a coil can easily be modelled with a signal-controlled source. Except for the neglected dynamics, steady-state actuator forces will be calculated properly in actuator models based on these sources.
</p>
</html>"));
        end SignalMagneticPotentialDifference;

        model ConstantMagneticFlux "Source of constant magnetic flux"
          extends Modelica.Magnetic.FluxTubes.Interfaces.PartialTwoPortsElementary;
          parameter SI.MagneticFlux Phi = 1 "Magnetic flux";
          SI.MagneticPotentialDifference V_m "Magnetic potential difference between both ports";
        equation
          V_m = port_p.V_m - port_n.V_m;
          Phi = port_p.Phi;
          0 = port_p.Phi + port_n.Phi;
          annotation(
            defaultComponentName = "constantSource",
            Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-150, -110}, {150, -70}}, textString = "%name", lineColor = {0, 0, 255}), Polygon(points = {{80, 0}, {60, 6}, {60, -6}, {80, 0}}, lineColor = {255, 128, 0}, fillColor = {255, 128, 0}, fillPattern = FillPattern.Solid), Line(points = {{-100, 0}, {-50, 0}}, color = {255, 127, 0}), Line(points = {{50, 0}, {100, 0}}, color = {255, 127, 0}), Ellipse(extent = {{-50, -50}, {50, 50}}, lineColor = {255, 127, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{0, 50}, {0, -50}}, color = {255, 127, 0})}),
            Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-125, 0}, {-115, 0}}, color = {160, 160, 164}), Line(points = {{-120, -5}, {-120, 5}}, color = {160, 160, 164}), Line(points = {{115, 0}, {125, 0}}, color = {160, 160, 164})}),
            Documentation(info = "<html>
<p>
Sources of a constant magnetic flux are useful for modelling of permanent magnets with <i>Norton</i>'s magnetic equivalent circuit.
</p>
</html>"));
        end ConstantMagneticFlux;

        model SignalMagneticFlux "Signal-controlled magnetic flux source"
          extends Modelica.Magnetic.FluxTubes.Interfaces.PartialTwoPortsElementary;
          Modelica.Blocks.Interfaces.RealInput Phi(unit = "Wb") "Magnetic flux" annotation(
            Placement(transformation(origin = {0, 90}, extent = {{10, -10}, {-10, 10}}, rotation = 90)));
          SI.MagneticPotentialDifference V_m "Magnetic potential difference between both ports";
        equation
          V_m = port_p.V_m - port_n.V_m;
          Phi = port_p.Phi;
          0 = port_p.Phi + port_n.Phi;
          annotation(
            defaultComponentName = "signalSource",
            Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{80, 0}, {60, 6}, {60, -6}, {80, 0}}, lineColor = {255, 128, 0}, fillColor = {255, 128, 0}, fillPattern = FillPattern.Solid), Text(extent = {{-150, -110}, {150, -70}}, textString = "%name", lineColor = {0, 0, 255}), Line(points = {{-100, 0}, {-50, 0}}, color = {255, 127, 0}), Line(points = {{50, 0}, {100, 0}}, color = {255, 127, 0}), Line(points = {{0, 100}, {0, 50}}, color = {255, 127, 0}), Ellipse(extent = {{-50, -50}, {50, 50}}, lineColor = {255, 127, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{0, 50}, {0, -50}}, color = {255, 127, 0})}),
            Documentation(info = "<html>
<p>
This source of a magnetic flux is intended for test purposes, e.g., for simulation and subsequent plotting of a softmagnetic material's magnetisation characteristics if used together with a non-linear reluctance element.
</p>
</html>"));
        end SignalMagneticFlux;
        annotation(
          Documentation(info = "<html>
<p>
This package contains sources of a magnetic potential difference or a magnetic flux:
</p>
</html>"));
      end Sources;

      package Sensors "Sensors to measure variables in magnetic networks"
        extends Modelica.Icons.SensorsPackage;

        model MagneticPotentialDifferenceSensor "Sensor to measure magnetic potential difference"
          extends Modelica.Icons.RotationalSensor;
          extends Modelica.Magnetic.FluxTubes.Interfaces.PartialTwoPortsElementary;
          Modelica.Blocks.Interfaces.RealOutput V_m(final quantity = "MagneticPotential", final unit = "A") "Magnetic potential difference between ports p and n as output signal" annotation(
            Placement(transformation(origin = {0, -100}, extent = {{10, -10}, {-10, 10}}, rotation = 90)));
          SI.MagneticFlux Phi "Magnetic flux from port_p to port_n";
        equation
          V_m = port_p.V_m - port_n.V_m;
          Phi = port_p.Phi;
          Phi = 0;
          0 = port_p.Phi + port_n.Phi;
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-52, 1}, {48, -57}}, lineColor = {0, 0, 0}, textString = "V_m"), Line(points = {{-70, 0}, {-90, 0}}, color = {0, 0, 0}), Line(points = {{70, 0}, {90, 0}}, color = {0, 0, 0}), Line(points = {{0, -90}, {0, -70}}, color = {0, 0, 0}), Text(extent = {{-150, 120}, {150, 80}}, textString = "%name", lineColor = {0, 0, 255})}),
            Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-70, 0}, {-100, 0}}, color = {255, 128, 0}), Line(points = {{70, 0}, {100, 0}}, color = {255, 128, 0}), Line(points = {{0, -100}, {0, -70}}, color = {0, 0, 0})}));
        end MagneticPotentialDifferenceSensor;

        model MagneticFluxSensor "Sensor to measure magnetic flux"
          extends Modelica.Magnetic.FluxTubes.Interfaces.PartialTwoPortsElementary;
          extends Modelica.Icons.RotationalSensor;
          Modelica.Blocks.Interfaces.RealOutput Phi(final quantity = "MagneticFlux", final unit = "Wb") "Magnetic flux from port p to port n as output signal" annotation(
            Placement(transformation(origin = {0, -100}, extent = {{10, -10}, {-10, 10}}, rotation = 90)));
        equation
          port_p.V_m = port_n.V_m;
          Phi = port_p.Phi;
          0 = port_p.Phi + port_n.Phi;
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{0, -100}, {0, -70}}, color = {0, 0, 0}), Line(points = {{-70, 0}, {-90, 0}}, color = {0, 0, 0}), Line(points = {{70, 0}, {90, 0}}, color = {0, 0, 0}), Text(extent = {{-29, -11}, {30, -70}}, textString = "Phi"), Text(extent = {{-150, 120}, {150, 80}}, textString = "%name", lineColor = {0, 0, 255}), Line(points = {{0, -90}, {0, -70}})}));
        end MagneticFluxSensor;
        annotation(
          Documentation(info = "<html>
<p>
For analysis of magnetic networks, only magnetic potential differences and magnetic flux are variables of interest. For that reason, a magnetic potential sensor is not provided.
</p>
</html>"));
      end Sensors;
      annotation(
        Documentation(info = "<html>
<p>
This library contains components for modelling of electromagnetic devices with lumped magnetic networks. Those models are suited for both rough design of the magnetic subsystem of a device as well as for efficient dynamic simulation at system level together with neighbouring subsystems. At present, components and examples for modelling of <i>translatory</i> electromagnetic and electrodynamic actuators are provided. If needed, these components can be adapted to network modelling of <i>rotational</i> electrical machines.
</p>
<p>
<a href=\"modelica://Modelica.Magnetic.FluxTubes.UsersGuide\">User's Guide</a> gives a short introduction to the underlying concept of <b>magnetic flux tubes</b>, summarizes the calculation of magnetic <b>reluctance forces</b> from lumped magnetic network models and lists <b>reference literature</b>.
</p>
<p>
<a href=\"modelica://Modelica.Magnetic.FluxTubes.Examples\">Examples</a> illustrates the usage of magnetic network models with simple models from different fields of application.
</p>

<p>
Copyright &copy; 2005-2015, Modelica Association and Thomas B&ouml;drich.
</p>
<p>
<i>This Modelica package is <u>free</u> software and the use is completely at <u>your own risk</u>; it can be redistributed and/or modified under the terms of the Modelica License 2. For license conditions (including the disclaimer of warranty) see <a href=\"modelica://Modelica.UsersGuide.ModelicaLicense2\">Modelica.UsersGuide.ModelicaLicense2</a> or visit <a href=\"https://www.modelica.org/licenses/ModelicaLicense2\"> https://www.modelica.org/licenses/ModelicaLicense2</a>.</i>
</p>
</html>", revisions = "<html>
<table border=1>

<thead>
<tr><td>Version</td> <td>Date</td>  <td>Author(s)</td> <td>Comments</td></tr>
</thead>

<tbody>
<tr><td>1.5</td>  <td>2013-01-04</td>
    <td>Martin&nbsp;Otter<br>Thomas&nbsp;B&ouml;drich<br>Johannes&nbsp;Ziske</td> <td>Added missing initial conditions<br>
        Fixed initial parameter values</td> </tr>
<tr><td>1.4</td>  <td>2011-08-01</td>
    <td>Thomas&nbsp;B&ouml;drich</td> <td>MagneticPort declared with MagneticPotential instead of MagneticPotentialDifference</td> </tr>
<tr><td>1.3</td>  <td>2010-04-22</td>
    <td>Christian&nbsp;Kral</td> <td>Added conditional heat port to EddyCurrent model</td> </tr>
<tr><td>1.2</td>  <td>2009-08-11</td>
    <td>Christian&nbsp;Kral<br>
        Anton&nbsp;Haumer<br>
        Thomas&nbsp;B&ouml;drich&nbsp;<br>
        Martin&nbsp;Otter</td> <td>Update and improvement for inclusion in the
        Modelica Standard Library.</td> </tr>
<tr><td>1.1</td>  <td>2009-05-19</td>  <td>Thomas B&ouml;drich </td>
    <td> Coupling coefficient in Basic.ElectroMagneticConverter Removed.<br>
         Basic.EddyCurrent added .<br>
         Example MovingCoilActuator, especially PermeanceModel, completely revised. <br>
         Leakage coefficient replaced by coupling coefficient in Basic.LeakageWithCoefficient. <br>
         Utilities.CoilDesign: parameter U renamed to V_op,CoilDesign moved to Utilities.<br>
         Reference direction for magnetic flux added in all sources.<br>
         degC replaced by K for compatibility with Modelica 3.0.<br>
         redeclare in Sensors for compatibility with Modelica 3.0 removed.<br>
         Partial flux tube components moved to Interfaces and basic elements moved to new package Basic. </td> </tr>
<tr><td>1.0</td>  <td>2007-10-11</td>  <td>Thomas B&ouml;drich </td> <td>Release of version 1.0 of the library </td> </tr>
<tr><td></td>     <td>2005</td>        <td>Thomas B&ouml;drich </td> <td>First release of a Modelica magnetic library</td> </tr>
</tbody>

</table>

</html>"),
        Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100.0, -100.0}, {100.0, 100.0}}), graphics = {Polygon(origin = {-3.75, 0.0}, fillColor = {160, 160, 164}, fillPattern = FillPattern.Solid, points = {{33.75, 50.0}, {-46.25, 50.0}, {-46.25, -50.0}, {33.75, -50.0}, {33.75, -30.0}, {-21.25, -30.0}, {-21.25, 30.0}, {33.75, 30.0}}), Ellipse(origin = {10.4708, 41.6771}, extent = {{-86.0, -24.0}, {-78.0, -16.0}}), Line(origin = {10.4708, 41.6771}, points = {{-64.0, -20.0}, {-78.0, -20.0}}), Line(origin = {10.4708, 41.6771}, points = {{-64.1812, -31.6229}, {-32.0, -40.0}}), Line(origin = {10.4708, 41.6771}, points = {{-64.0, -20.0}, {-32.0, -28.0}}), Ellipse(origin = {10.4708, 41.6771}, extent = {{-86.0, -60.0}, {-78.0, -52.0}}), Line(origin = {10.4708, 41.6771}, points = {{-64.0, -56.0}, {-78.0, -56.0}}), Line(origin = {10.4708, 41.6771}, points = {{-64.0, -44.0}, {-32.0, -52.0}}), Line(origin = {10.4708, 41.6771}, points = {{-64.0, -56.0}, {-32.0, -64.0}}), Rectangle(origin = {62.5, 0.0}, fillColor = {160, 160, 164}, fillPattern = FillPattern.Solid, extent = {{-12.5, -50.0}, {12.5, 50.0}})}));
    end FluxTubes;

    package FundamentalWave "Library for magnetic fundamental wave effects in electric machines"
      extends Modelica.Icons.Package;

      package UsersGuide "User's Guide"
        extends Modelica.Icons.Information;

        class Concept "Fundamental wave concept"
          extends Modelica.Icons.Information;
          annotation(
            Documentation(info = "<html>

<h4>Overview of the concept of fundamental waves</h4>

<p>
The exact magnetic field in the air gap of an electric machine is usually determined by an electro magnetic finite element analysis. The waveform of the magnetic field, e.g., the magnetic potential difference <img src=\"modelica://Modelica/Resources/Images/Magnetic/FundamentalWave/V_m_phi.png\">, consists of a spatial fundamental wave - with respect to an equivalent two pole machine - and additional harmonic waves of different order. The fundamental wave is however dominant in the air gap of an electric machine.
</p>

<table border=\"0\" cellspacing=\"0\" cellpadding=\"2\">
  <caption align=\"bottom\"><b>Fig. 1:</b> Field lines of a four pole induction machine</caption>
  <tr>
    <td>
      <img src=\"modelica://Modelica/Resources/Images/Magnetic/FundamentalWave/UsersGuide/Concept/aimc_fem.png\">
    </td>
  </tr>
</table>

<p>
In the fundamental wave theory only a pure sinusoidal distribution of magnetic quantities is assumed. It is thus assumed that all other harmonic wave effects are not taken into account.</p>

<table border=\"0\" cellspacing=\"0\" cellpadding=\"2\">
  <caption align=\"bottom\"><b>Fig. 2:</b> Magnetic potential difference of a four pole machine, where <img src=\"modelica://Modelica/Resources/Images/Magnetic/FundamentalWave/varphi.png\"> is the angle of the spatial domain with respect to one pole pair</caption>
  <tr>
    <td>
      <img src=\"modelica://Modelica/Resources/Images/Magnetic/FundamentalWave/UsersGuide/Concept/four_pole_V_m.png\">
    </td>
  </tr>
</table>

<p>
The waveforms of the magnetic field quantities, e.g., the magnetic potential difference <img src=\"modelica://Modelica/Resources/Images/Magnetic/FundamentalWave/V_m_phi.png\">, can be represented by complex phasor, e.g., <img src=\"modelica://Modelica/Resources/Images/Magnetic/FundamentalWave/V_m.png\"> according to:
</p>

<p>
&nbsp;&nbsp;<img src=\"modelica://Modelica/Resources/Images/Magnetic/FundamentalWave/V_m_real_complex.png\">
</p>

<p>It is important to note that the magnetic potential used in this library <b>always</b> refers to an equivalent two pole machine.</p>

<table border=\"0\" cellspacing=\"0\" cellpadding=\"2\">
  <caption align=\"bottom\"><b>Fig. 3:</b> Spatial distribution of the magnetic potential difference (red shade = positive sine wave, blue shade = negative sine wave) including complex phasor representing this spatial distribution</caption>
  <tr>
    <td>
      <img src=\"modelica://Modelica/Resources/Images/Magnetic/FundamentalWave/UsersGuide/Concept/electrical_reference_V_m.png\">
    </td>
  </tr>
</table>

<p>
The potential and flow quantities of this library are the complex magnetic potential difference and the complex magnetic flux as defined in the basic <a href=\"modelica://Modelica.Magnetic.FundamentalWave.Interfaces.MagneticPort\">magnetic port</a>. Due to the sinusoidal distribution of magnetic potential and flux, such a complex phasor representation can be used. This way, the FundamentalWave library can be seen as a spatial extension of the <a href=\"modelica://Modelica.Magnetic.FluxTubes\">FluxTubes</a> library.
</p>

<p>
The specific arrangement of windings in electric machines with <img src=\"modelica://Modelica/Resources/Images/Magnetic/FundamentalWave/p.png\"> pole pairs give rise to sinusoidal dominant magnetic potential wave. The spatial period of this wave is determined by one pole pair
[<a href=\"modelica://Modelica.Magnetic.FundamentalWave.UsersGuide.References\">Mueller70</a>,
 <a href=\"modelica://Modelica.Magnetic.FundamentalWave.UsersGuide.References\">Spaeth73</a>].
</p>

<p>
The main components of an electric machine model based on the FundamentalWave library are <a href=\"modelica://Modelica.Magnetic.FundamentalWave.BasicMachines.Components.SymmetricMultiPhaseWinding\">multi phase</a> and <a href=\"modelica://Modelica.Magnetic.FundamentalWave.BasicMachines.Components.SinglePhaseWinding\">single phase windings</a>, <a href=\"modelica://Modelica.Magnetic.FundamentalWave.BasicMachines.Components.RotorSaliencyAirGap\">air gap</a> as well as <a href=\"modelica://Modelica.Magnetic.FundamentalWave.BasicMachines.Components.SymmetricMultiPhaseWinding\">symmetric</a> or <a href=\"modelica://Modelica.Magnetic.FundamentalWave.BasicMachines.Components.SaliencyCageWinding\">salient cage</a> models.
The electric machine models provided in this library are based on symmetrical windings in the stator and equivalent two or three phase windings in squirrel cage rotors. Slip ring induction machines may have different phase numbers in the stator and rotor.
</p>

<h4>Assumptions</h4>

<p>
The machine models of the FundamentalWave library are currently based on the following assumptions
</p>

<ul>
<li>The number of stator phases is greater or equal to three
    [<a href=\"modelica://Modelica.Magnetic.FundamentalWave.UsersGuide.References\">Eckhardt82</a>]
    </li>
<li>The phase windings are assumed to be symmetrical; an extension to this approach can be considered</li>
<li>Only fundamental wave effects are taken into account</li>
<li>The magnetic potential difference refers to an equivalent two pole machine</li>
<li>There are no restrictions on the waveforms of voltages and currents</li>
<li>All resistances and inductances are modeled as constant quantities; saturation effects, cross coupling effects
    [<a href=\"modelica://Modelica.Magnetic.FundamentalWave.UsersGuide.References\">Li07</a>], temperature dependency of resistances and deep bar effects could be considered in an extension to this library</li>
<li>Hysteresis losses are currently not considered [<a href=\"modelica://Modelica.Magnetic.FundamentalWave.UsersGuide.References\">Haumer09</a>]</li>
<li>The losses dissipated in the electric machine models are
    <ul>
    <li>ohmic heat losses,</li>
    <li>eddy current losses in the stator core,</li>
    <li>stray load losses,</li>
    <li>friction losses.</li>
    </ul>
</li>
</ul>

<h4>Note</h4>

<p>
The term <b>fundamental wave</b> refers to spatial waves of the electro magnetic quantities. This library has no limitations with respect to the waveforms of the time domain signals of any voltages, currents, etc.
</p>
</html>"));
        end Concept;

        class MultiPhase "Multi phase windings"
          extends Modelica.Icons.Information;
          annotation(
            Documentation(info = "<html>
<h4>Symmetrical three phase system</h4>

<p>
Symmetrical three phases systems of currents (or voltages) consists of three sinusoidal
sine waves with with an angular displacement of
<img src=\"modelica://Modelica/Resources/Images/Magnetic/FundamentalWave/UsersGuide/MultiPhase/2pi_3.png\"/>.
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Magnetic/FundamentalWave/UsersGuide/MultiPhase/i123.png\"/>,
</p>

<p>
Electrical three phase machines have (usually) symmetrical three phase windings which
excite spatial magnetic potential with a spacial displacement of
<img src=\"modelica://Modelica/Resources/Images/Magnetic/FundamentalWave/UsersGuide/MultiPhase/2pi_3.png\"/>
- with respect to the fundamental wave,
see [<a href=\"modelica://Modelica.Magnetic.FundamentalWave.UsersGuide.References\">Laughton02</a>].
Such a symmetrical three phase system of currents (or voltages) can be represented by
<a href=\"http://en.wikipedia.org/wiki/Phasor\">phasors</a>, as depicted in Fig. 1(a).
The associated three phase winding is depicted in Fig. 2(a). The winding axis are displaced by
<img src=\"modelica://Modelica/Resources/Images/Magnetic/FundamentalWave/UsersGuide/MultiPhase/2pi_3.png\"/>:
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Magnetic/FundamentalWave/UsersGuide/MultiPhase/orientationk_3.png\"/>
</p>

<p>
So there is is a strong coherence between angular displacement in the time and
spatial domain which also applies to multi phase systems.
</p>

<table border=\"0\" cellspacing=\"0\" cellpadding=\"2\">
  <caption align=\"bottom\"><strong>Fig. 1: </strong>Symmetrical (a) three phase and (b) five phase current system</caption>
  <tr>
    <td>
      <img src=\"modelica://Modelica/Resources/Images/Magnetic/FundamentalWave/UsersGuide/MultiPhase/phase35.png\"
           alt=\"phase35.png\">
    </td>
  </tr>
</table>

<table border=\"0\" cellspacing=\"0\" cellpadding=\"2\">
  <caption align=\"bottom\"><strong>Fig. 2: </strong>Symmetrical (a) three phase and (b) five phase winding</caption>
  <tr>
    <td>
      <img src=\"modelica://Modelica/Resources/Images/Magnetic/FundamentalWave/UsersGuide/MultiPhase/winding35.png\"
           alt=\"winding35.png\">
    </td>
  </tr>
</table>

<h4>Symmetrical multi phase system</h4>

<p>
In symmetrical multi phase systems odd and even phase numbers have to be distinguished.
</p>

<h5>Odd number of phases</h5>

<p>
For a symmetrical multi phase system with <img src=\"modelica://Modelica/Resources/Images/Magnetic/FundamentalWave/UsersGuide/MultiPhase/m.png\"/>
phases the displacement in the time and spatial domain is
<img src=\"modelica://Modelica/Resources/Images/Magnetic/FundamentalWave/UsersGuide/MultiPhase/2pi_m.png\"/>,
as depicted in Fig. 1 and 2.
</p>

<p>
Mathematically, this symmetry is expressd in terms of phase currents by:
</p>

<p><img src=\"modelica://Modelica/Resources/Images/Magnetic/FundamentalWave/UsersGuide/MultiPhase/ik_odd.png\"/> </p>

<p>
The orientation of the winding axis of such winding is given by:
</p>

<p><img src=\"modelica://Modelica/Resources/Images/Magnetic/FundamentalWave/UsersGuide/MultiPhase/orientationk_m.png\"/> </p>

<h5>Even number of phases</h5>

<p>
In the current implementation of the FundamentalWave library, phase numbers equal
to the power of two are not supported. However, any other multi phase system with even
an phase number, <img src=\"modelica://Modelica/Resources/Images/Magnetic/FundamentalWave/UsersGuide/MultiPhase/m.png\"/>,
can be recursively split into various symmetrical systems with odd phase numbers, as depicted in Fig. 3 and 4.
The displacement between the two symmetrical systems is
<img src=\"modelica://Modelica/Resources/Images/Magnetic/FundamentalWave/UsersGuide/MultiPhase/pi_m.png\"/>.
A function for calculating the <a href=\"modelica://Modelica.Electrical.MultiPhase.Functions.symmetricOrientation\">symmetricOrientation</a> is available.
</p>

<table border=\"0\" cellspacing=\"0\" cellpadding=\"2\">
  <caption align=\"bottom\"><strong>Fig. 3: </strong>Symmetrical (a) six and (b) ten phase current system</caption>
  <tr>
    <td>
      <img src=\"modelica://Modelica/Resources/Images/Magnetic/FundamentalWave/UsersGuide/MultiPhase/phase610.png\"
           alt=\"phase610.png\">
    </td>
  </tr>
</table>

<table border=\"0\" cellspacing=\"0\" cellpadding=\"2\">
  <caption align=\"bottom\"><strong>Fig. 4: </strong>Symmetrical (a) six and (b) ten phase winding</caption>
  <tr>
    <td>
      <img src=\"modelica://Modelica/Resources/Images/Magnetic/FundamentalWave/UsersGuide/MultiPhase/winding610.png\"
           alt=\"winding610.png\">
    </td>
  </tr>
</table>


<h4>Note</h4>

<p>
In a fully symmetrical machine, the orientation of the winding axes and the symmetrical currents (or voltages)
<a href=\"http://en.wikipedia.org/wiki/Phasor\">phasors</a> have different signs; see Fig. 1 and 2 for odd phase numbers
and Fig. 3 and 4 for even phase numbes.
</p>
</html>"));
        end MultiPhase;

        class Parameters "Parameters of equivalent machines models"
          extends Modelica.Icons.Information;
          annotation(
            Documentation(info = "<html>

<h4>Stator main inductance</h4>

<p>
The stator main inductance
<img src=\"modelica://Modelica/Resources/Images/Magnetic/FundamentalWave/UsersGuide/Parameters/Lm_m.png\">
of an
<img src=\"modelica://Modelica/Resources/Images/Magnetic/FundamentalWave/UsersGuide/Parameters/m.png\">
phase induction machine is related with the self inductance of on stator phase,
<img src=\"modelica://Modelica/Resources/Images/Magnetic/FundamentalWave/UsersGuide/Parameters/Lss.png\">,
by:</p>
<p>
<img src=\"modelica://Modelica/Resources/Images/Magnetic/FundamentalWave/UsersGuide/Parameters/Lm_m_Lss.png\">
</p>

<h4>Parameters of equivalent multi phase induction machines models</h4>

<p>Assume a set parameters,
<img src=\"modelica://Modelica/Resources/Images/Magnetic/FundamentalWave/UsersGuide/Parameters/Rs_3.png\">,
<img src=\"modelica://Modelica/Resources/Images/Magnetic/FundamentalWave/UsersGuide/Parameters/Lssigma_3.png\">,
<img src=\"modelica://Modelica/Resources/Images/Magnetic/FundamentalWave/UsersGuide/Parameters/L0_3.png\">,
<img src=\"modelica://Modelica/Resources/Images/Magnetic/FundamentalWave/UsersGuide/Parameters/Lm_3.png\">,
of a three phase induction machine and a set of parameters,
<img src=\"modelica://Modelica/Resources/Images/Magnetic/FundamentalWave/UsersGuide/Parameters/Rs_m.png\">,
<img src=\"modelica://Modelica/Resources/Images/Magnetic/FundamentalWave/UsersGuide/Parameters/Lssigma_m.png\">,
<img src=\"modelica://Modelica/Resources/Images/Magnetic/FundamentalWave/UsersGuide/Parameters/L0_m.png\">,
<img src=\"modelica://Modelica/Resources/Images/Magnetic/FundamentalWave/UsersGuide/Parameters/Lm_m.png\">,
of an
<img src=\"modelica://Modelica/Resources/Images/Magnetic/FundamentalWave/UsersGuide/Parameters/m.png\">
phase induction machine. It is also assumed that</p>
<ul>
<li>the nominal phase voltages</li>
<li>the nominal stator frequences</li>
</ul>
<p>of the three and
<img src=\"modelica://Modelica/Resources/Images/Magnetic/FundamentalWave/UsersGuide/Parameters/m.png\">
phase induction machine are equal. In this case the two parameter sets are related by:</p>
<p>
<img src=\"modelica://Modelica/Resources/Images/Magnetic/FundamentalWave/UsersGuide/Parameters/Rs_3m.png\"><br>
<img src=\"modelica://Modelica/Resources/Images/Magnetic/FundamentalWave/UsersGuide/Parameters/Lssigma_3m.png\"><br>
<img src=\"modelica://Modelica/Resources/Images/Magnetic/FundamentalWave/UsersGuide/Parameters/L0_3m.png\"><br>
<img src=\"modelica://Modelica/Resources/Images/Magnetic/FundamentalWave/UsersGuide/Parameters/Lm_3m.png\"></p>
<p>
This way the same torque is generated and the machine currents are related by:
</p>
<p>
<img src=\"modelica://Modelica/Resources/Images/Magnetic/FundamentalWave/UsersGuide/Parameters/is_3m.png\">
</p>

<p>
The same applies for the rotor parameters of a
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.BasicMachines.AsynchronousInductionMachines.AIM_SlipRing\">
slip ring induction machine</a>, where the phase number
<img src=\"modelica://Modelica/Resources/Images/Magnetic/FundamentalWave/UsersGuide/Parameters/m.png\">
is simply replaced by
<img src=\"modelica://Modelica/Resources/Images/Magnetic/FundamentalWave/UsersGuide/Parameters/mr.png\">
for transforming equivalent three phase to
<img src=\"modelica://Modelica/Resources/Images/Magnetic/FundamentalWave/UsersGuide/Parameters/mr.png\">
phase winding parameters -- at the same nominal rotor voltage and frequency.
</p>

<h4>See also</h4>
<p>
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.Examples.BasicMachines.AIMC_DOL_MultiPhase\">AIMC_DOL_MultiPhase</a>,
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.Examples.BasicMachines.AIMS_Start_MultiPhase\">AIMS_Start_MultiPhase</a>,
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.Examples.BasicMachines.SMPM_Inverter_MultiPhase\">SMPM_Inverter_MultiPhase</a>,
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.Examples.BasicMachines.SMEE_Generator_MultiPhase\">SMEE_Generator_MultiPhase</a>,
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.Examples.BasicMachines.SMR_Inverter_MultiPhase\">SMR_Inverter_MultiPhase</a>
</p>

</html>"));
        end Parameters;

        class Contact "Contact"
          extends Modelica.Icons.Contact;
          annotation(
            Documentation(info = "<html>
<h4>Contact</h4>

<p>
Christian Kral<br>
Electric Machines, Drives and Systems<br>
1060 Vienna, Austria<br>
email: <a HREF=\"mailto:dr.christian.kral@gmail.com\">dr.christian.kral@gmail.com</a><br>
</p>

<p>
Anton Haumer<br>
<a href=\"http://www.haumer.at\">Technical Consulting &amp; Electrical Engineering</a><br>
3423 St. Andrae-Woerdern, Austria<br>
email: <a HREF=\"mailto:a.haumer@haumer.at\">a.haumer@haumer.at</a><br>
</p>

<h4>Acknowledgements</h4>

<p>
Based on an original idea of Michael Beuschel this library was developed
[<a href=\"modelica://Modelica.Magnetic.FundamentalWave.UsersGuide.References\">Beuschel00</a>].
The authors of the FundamentalWave library would like to thank Michael Beuschel
for contributing his source code to this library.
</p>

<p>
The research leading to version 2.0.0 has received funding from the ENIAC Joint Undertaking under grant
agreement no. 270693-2 and from the &Ouml;sterreichische
Forschungsf&ouml;rderungsgesellschaft mbH under project
no. 829420.
</p>
</html>"));
        end Contact;

        class ReleaseNotes "Release Notes"
          extends Modelica.Icons.ReleaseNotes;
          annotation(
            Documentation(info = "<html>

<h5>Version 3.2.1, 2013-07-31</h5>

<ul>
<li>Bug fix of wrong orientaton of squirrel and damper cage modells, see ticket #1226; this also includes the removal of the rotor cage heat sensor which previously has been used</li>
<li>Bug fix of conditional initialization of examples, see ticket #1223</li>
<li>Fixed missing default parameter TpmOperational in ambient of PM synchronous machine, see ticket #1216</li>
<li>Added voltages, currents, complex flux and magnetic potential difference as global variables in
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.Components.MultiPhaseElectroMagneticConverter\">multi phase converter</a></li>
<li>Added two more component examples, showing the equivalent nature of electrical and magnetic domain</li>
</ul>

<h5>Version 2.0.0, 2013-03-10</h5>

<ul>
<li>Corrected wrong parameter descriptions, see ticket #1003</li>
<li>Extended machine models towards phase numbers to greater or equal than three, see ticket #990</li>
</ul>

<h5>Version 1.7.3, 2013-02-25</h5>

<ul>
<li>Corrected wrong parameter description</li>
</ul>


<h5>Version 1.7.2, 2011-06-28</h5>

<ul>
<li>Corrected bug in calculation of core conductance in
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.BasicMachines.Components.SymmetricMultiPhaseWinding\">SymmetricMultiPhaseWinding</a>:
the wrong calculation <code>G=(m/2)*GcRef/effectiveTurns^2</code> is now replaced by <code>G=(m/2)*GcRef*effectiveTurns^2</code></li>
</ul>

<h5>Version 1.7.1, 2010-09-03</h5>

<ul>
<li>Naming and documentation of
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.Interfaces.PartialTwoPort\">
PartialTwoPort</a> is exchanged by
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.Interfaces.PartialTwoPortElementary\">
PartialTwoPortElementary</a> to match the naming conventions of
<a href=\"modelica://Modelica.Mechanics.Rotational.Interfaces\">
Rotational.Interfaces</a>
and <a href=\"modelica://Modelica.Mechanics.Translational.Interfaces\">
Translational.Interfaces</a></li>
<li>Fixed a broken link and updated documentation</li>
<li>Adaptions to Complex SIunits</li>
</ul>

<h5>Version 1.7.0, 2010-05-31</h5>

<ul>
<li>Changed <a href=\"modelica://Modelica.Magnetic.FundamentalWave.BasicMachines.Components.SymmetricMultiPhaseWinding\">symmetric multi phase winding</a> model
  <ul>
      <li>Added zero sequence inductance based on
          <a href=\"modelica://Modelica.Electrical.MultiPhase.Basic.ZeroInductor\">zero inductor</a></li>
      <li>Replaced electrical model of stray
          <a href=\"modelica://Modelica.Electrical.MultiPhase.Basic.Inductor\">inductor</a> by stray
          <a href=\"modelica://Modelica.Magnetic.FundamentalWave.Components.Reluctance\">reluctance</a> model</li>
      <li>Integrated cores <a href=\"modelica://Modelica.Magnetic.FundamentalWave.Components.EddyCurrent\">losses</a>
          and <a href=\"modelica://Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a\">heat port</a></li>
  </ul></li>
  <li>Added rotor core loss parameters in asynchronous induction machine with slip rings</li>
  <li>Renamed heat ports of <a href=\"modelica://Modelica.Magnetic.FundamentalWave.BasicMachines.Components.SinglePhaseWinding\">single phase winding</a> and <a href=\"modelica://Modelica.Magnetic.FundamentalWave.BasicMachines.Components.SymmetricMultiPhaseWinding\">symmetric multi phase winding</a>
     </li>
  <li>Relocated core losses between zero inductor and stray reluctance model in the magnetic domain</li>
  <li>Renamed instances of stator and rotor (winding) models in each machines</li>
  <li>Added magnetic potential sensor</li>
  <li>Removed state selections</li>
  <li>Updates due to changed loss variable and heat port names in
      <a href=\"modelica://Modelica.Electrical.Machines\">Electrical.Machines</a></li>
  <li>Added machine specific output records to summarize power and loss balance</li>
  <li>Updated images of Users Guide</li>
  <li>Improved performance due to <code>annotation(Evaluate=true)</code> added to the parameters of the
      <a href=\"modelica://Modelica.Magnetic.FundamentalWave.BasicMachines.Components.SinglePhaseWinding\">single phase winding</a>
  <li>Reduced number of states in <a href=\"modelica://Modelica.Magnetic.FundamentalWave.BasicMachines.Components.SymmetricMultiPhaseCageWinding\">symmetric cage</a> model by introducing an additional non-grounded star connection</li>
</ul>

<h5>Version 1.6.0, 2010-05-05</h5>

<ul>
<li>Renamed all parameters <code>windingAngle</code> to <code>orientation</code>. The following classes are affected:
<ul>
<li><a href=\"modelica://Modelica.Magnetic.FundamentalWave.Components.SinglePhaseElectroMagneticConverter\">SinglePhaseElectroMagneticConverter</a></li>
<li><a href=\"modelica://Modelica.Magnetic.FundamentalWave.Components.MultiPhaseElectroMagneticConverter\">MultiPhaseElectroMagneticConverter</a></li>
<li><a href=\"modelica://Modelica.Magnetic.FundamentalWave.BasicMachines.Components.SinglePhaseWinding\">SinglePhaseWinding</a></li>
<li><a href=\"modelica://Modelica.Electrical.MultiPhase.Functions.symmetricOrientation\">symmetricOrientation</a></li>
</ul></li>
<li>Update due to changed class names in <a href=\"modelica://Modelica.Electrical.Machines.Icons\">Machines.Icons</a></li>
<li>Using <a href=\"modelica://Modelica.Thermal.HeatTransfer.Interfaces.PartialElementaryConditionalHeatPort\">HeatTransfer.Interfaces.PartialElementaryConditionalHeatPort</a> instead of
    <a href=\"modelica://Modelica.Electrical.Analog.Interfaces.ConditionalHeatPort\">Analog.Interfaces.ConditionalHeatPort</a> in
    <a href=\"modelica://Modelica.Magnetic.FundamentalWave.Components.EddyCurrent\">EddyCurrent</a> </li>
<li>Added <code>modelica://</code> to all Modelica hyper links</li>
<li>Fixed bug in displayed parameters of <a href=\"modelica://Modelica.Magnetic.FundamentalWave.Components.EddyCurrent\">EddyCurrent</a> </li>
<li>Updated some images (and renamed image file <code>LossPower.png</code> to <code>lossPower.png</code>)</li>
<li>Exchanged positive and negative stator ports of <a href=\"modelica://Modelica.Magnetic.FundamentalWave.BasicMachines.Components.RotorSaliencyAirGap\">RotorSaliencyAirGap</a>
model, adapted equations accordingly and updated code documentation.</li>
</ul>

<h5>Version 1.5.0, 2010-04-28</h5>

<ul>
<li>Added stator core, friction, stray load and brush losses to all machine types based on
<a href=\"modelica://Modelica.Electrical.Machines.Losses\">loss models</a> of the <a href=\"modelica://Modelica.Electrical.Machines\">Machines</a> library.</li>
<li>Changed parameter of
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.Components.EddyCurrent\">EddyCurrent</a>
model from R to G</li>
<li>Fixed wrong sign of internal quantity <code>tauElectrical</code>, model behavior does not change</li>
<li>Rewrote equations of electromagnetic coupling to look more elegant</li>
</ul>

<h5>Version 1.4.0, 2010-04-22</h5>

<ul>
<li>Added eddy current model in accordance to <a href=\"modelica://Modelica.Magnetic.FluxTubes\">FluxTubes</a> library</li>
<li>Added thermal heat port to eddy current model</li>
<li>Minor updates due to dependencies of <a href=\"modelica://Modelica.Electrical.Machines\">Machines</a>
</ul>

<h5>Version 1.3.0, 2010-02-26</h5>

<ul>
<li>Changed some icon references</li>
<li>Added state selections for the machine models</li>
<li>Restructured partial machine model</li>
<li>Added copyright information</li>
</ul>

<h5>Version 1.2.0, 2010-02-17</h5>

<ul>
<li>Renamed Machines to BasicMachines</li>
<li>Updated dependencies due to renamed class
<a href=\"modelica://Modelica.Electrical.Machines.Thermal.LinearTemperatureCoefficient20\">LinearTemperatureCoefficient20</a>
<li>Added release notes in User's Guide</li>
</ul>

<h5>Version 1.1.0, 2010-02-15</h5>

<ul>
<li>Added thermal connectors and temperature dependent resistances</li>
</ul>

<h5>Version 1.0.0, 2010-02-04</h5>

<ul>
<li>Integrated the library into the MSL</li>
</ul>

<h5>Version 0.4.0, 2009-10-29</h5>

<ul>
<li>Corrected bug in magnetic potential calculation</li>
</ul>

<h5>Version 0.3.0, 2009-10-28</h5>

<ul>
<li>Renamed number of turns and winding angles</li>
</ul>

<h5>Version 0.2.0, 2009-10-20</h5>

<ul>
<li>Added idle model</li>
</ul>

<h5>Version 0.1.0, 2009-07-22</h5>

<ul>
<li>First version based on the concept of the FluxTubes library and the Magnetics library of Michael Beuschel
[<a href=\"modelica://Modelica.Magnetic.FundamentalWave.UsersGuide.References\">Beuschel00</a>]</li>
</ul>
</html>"));
        end ReleaseNotes;

        class References "References"
          extends Modelica.Icons.References;
          annotation(
            Documentation(info = "<html>
<h4>References</h4>

<table border=\"0\" cellspacing=\"0\" cellpadding=\"2\">
    <tr>
      <td valign=\"top\">[Beuschel00]</td>
      <td valign=\"top\">M. Beuschel,
        &quot;<a href=\"https://www.modelica.org/events/workshop2000/proceedings/Beuschel.pdf\">
        A uniform approach for modelling electrical machines</a>,&quot;
        <i>Modelica Workshop</i>,
        pp. 101-108, October 23-24, 2000.</td>
    </tr>

    <tr>
      <td valign=\"top\">[Eckhardt82]</td>
      <td valign=\"top\">H. Eckhardt,
        <i>Grundz&uuml;ge der elektrischen Maschinen</i> (in German),
        B. G. Teubner Verlag, Stuttgart, 1982.</td>
    </tr>

    <tr>
      <td valign=\"top\">[Haumer09]</td>
      <td valign=\"top\">A. Haumer, and C. Kral,
        &quot;<a href=\"https://www.modelica.org/events/modelica2009/Proceedings/memorystick/pages/papers/0103/0103.pdf\">The
        AdvancedMachines Library: Loss Models for Electric Machines</a>,&quot;
        <i>Modelica Conference</i>, 2009.</td>
    </tr>

    <tr>
      <td valign=\"top\">[Lang84]</td>
      <td valign=\"top\">W. Lang,
        <i>&Uuml;ber die Bemessung verlustarmer Asynchronmotoren mit K&auml;figl&auml;ufer f&uuml;r Pulsumrichterspeisung</i>
       (in German),
       Doctoral Thesis, Technical University of Vienna, 1984.</td>
    </tr>

    <tr>
      <td valign=\"top\">[Laughton02]</td>
      <td valign=\"top\">M.A. Laughton, D.F. Warne
        <i>Electrical Engineer's Reference Book</i>
        Butterworth Heinemann, 16th edition, ISBN 978-0750646376, 2002</td>
    </tr>

    <tr>
      <td valign=\"top\">[Li07]</td>
      <td valign=\"top\">Y. Li, Z. Q. Zhu, D. Howe, and C. M. Bingham,
        &quot;Modeling of Cross-Coupling Magnetic Saturation in Signal-Injection-Based
        Sensorless Control of Permanent-Magnet Brushless AC Motors,&quot;
        <i>IEEE Transactions on Magnetics</i>,
        vol. 43, no. 6, pp. 2552-2554, June 2007.</td>
    </tr>

    <tr>
      <td valign=\"top\">[Mueller70]</td>
      <td valign=\"top\">G, M&uuml;ller,
        <i>Elektrische Maschinen -- Grundlagen, Aufbau und Wirkungsweise</i> (in German),
        VEB Verlag Technik Berlin, 4th edition, 1970.</td>
    </tr>

    <tr>
      <td valign=\"top\">[Spaeth73]</td>
      <td valign=\"top\">H. Sp&auml;th,
        <i>Elektrische Maschinen -- Eine Einf&uuml;hrung in die Theorie des Betriebsverhaltens</i> (in German),
        Springer-Verlag, Berlin, Heidelberg, New York, 1973.</td>
    </tr>

</table>
</html>"));
        end References;
        annotation(
          Documentation(info = "<html>
<p>
This library contains components for modelling of electromagnetic fundamental wave
models for the application in multi phase phase
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.BasicMachines\">electric machines</a>.
The number of phases is not restricted to three. DC machines are (currently) not included
in this library. The FundamentalWave library is an alternative approach to the
<a href=\"modelica://Modelica.Electrical.Machines\">Modelica.Electrical.Machines</a> library.
A great advantage of this library is the strict object orientation of the electrical and
magnetic components that the electric machines models are composed of.
From a didactic point of view this library is very beneficial for students in the
field of electrical engineering.
</p>

<p>
For more details see the <a href=\"modelica://Modelica.Magnetic.FundamentalWave.UsersGuide.Concept\">concept.</a>
</p>

<h5>Note</h5>

<ul>
<li>All the machine models provided in this library are equivalent two pole machines.
The magnetic potential difference of the connector therefore also refers to an equivalent two pole machine</li>
<li>In machines with <strong>more than three phases</strong> only effects of currents and voltages on the magnetic <strong>fundamental waves</strong> are considered. Other magnetic effects due to higher harmonic are not taken into account.</li>
</ul>


</html>"));
      end UsersGuide;

      package Examples "Examples of electric machines based on the FundamentalWave concept"
        extends Modelica.Icons.ExamplesPackage;

        package Components "Examples of components of the FundamentalWave library"
          extends Modelica.Icons.ExamplesPackage;

          model EddyCurrentLosses "Comparison of equivalent circuits of eddy current loss models"
            extends Modelica.Icons.Example;
            parameter Integer m = 3 "Number of phases";
            parameter Modelica.SIunits.Resistance R = 0.1 "Resistance of leader cables";
            parameter Modelica.SIunits.Conductance Gc = 1 "Loss conductance";
            parameter Modelica.SIunits.Reluctance R_m = 1 "Reluctance of the magnetic circuit";
            parameter Real N = 1 "Number of turns";
            output Modelica.SIunits.Power lossPower_e = sum(loss_e.conductor.LossPower);
            output Modelica.SIunits.Power lossPower_m = loss_m.lossPower;
            Modelica.Electrical.Analog.Basic.Ground ground_e annotation(
              Placement(transformation(extent = {{-90, 0}, {-70, 20}})));
            Modelica.Electrical.Analog.Basic.Ground ground_m annotation(
              Placement(transformation(extent = {{-90, -90}, {-70, -70}})));
            Modelica.Electrical.MultiPhase.Basic.Star star_e(m = m) annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 270, origin = {-80, 30})));
            Modelica.Electrical.MultiPhase.Basic.Star star_m(m = m) annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 270, origin = {-80, -60})));
            Modelica.Electrical.MultiPhase.Sources.SineVoltage sineVoltage_e(m = m, V = fill(1, m), freqHz = fill(1, m)) annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 270, origin = {-80, 60})));
            Modelica.Electrical.MultiPhase.Sources.SineVoltage sineVoltage_m(m = m, V = fill(1, m), freqHz = fill(1, m)) annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 270, origin = {-80, -30})));
            Modelica.Electrical.MultiPhase.Basic.Resistor resistor_e(m = m, R = fill(R, m)) annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0, origin = {-60, 70})));
            Modelica.Electrical.MultiPhase.Basic.Resistor resistor_m(m = m, R = fill(R, m)) annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0, origin = {-60, -20})));
            Modelica.Magnetic.FundamentalWave.Components.MultiPhaseElectroMagneticConverter converter_e(m = m, effectiveTurns = fill(N, m), orientation = Modelica.Electrical.MultiPhase.Functions.symmetricOrientation(m)) annotation(
              Placement(transformation(extent = {{20, 50}, {40, 70}})));
            Modelica.Magnetic.FundamentalWave.Components.MultiPhaseElectroMagneticConverter converter_m(m = m, effectiveTurns = fill(N, m), orientation = Modelica.Electrical.MultiPhase.Functions.symmetricOrientation(m)) annotation(
              Placement(transformation(extent = {{20, -40}, {40, -20}})));
            Modelica.Magnetic.FundamentalWave.Components.Reluctance reluctance_e(R_m(d = R_m, q = R_m)) annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 270, origin = {80, 60})));
            Modelica.Magnetic.FundamentalWave.Components.Reluctance reluctance_m(R_m(d = R_m, q = R_m)) annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 270, origin = {80, -30})));
            Modelica.Magnetic.FundamentalWave.Components.Ground mground_e annotation(
              Placement(transformation(extent = {{30, 0}, {50, 20}})));
            Modelica.Magnetic.FundamentalWave.Components.Ground mground_m annotation(
              Placement(transformation(extent = {{30, -90}, {50, -70}})));
            Electrical.MultiPhase.Basic.Conductor loss_e(m = m, G = fill(Gc, m)) annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 270, origin = {-10, 60})));
            Modelica.Magnetic.FundamentalWave.Components.EddyCurrent loss_m(G = m * N ^ 2 * Gc / 2) annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0, origin = {60, -20})));
            Modelica.Electrical.MultiPhase.Sensors.PowerSensor powerb_e(m = m) annotation(
              Placement(transformation(extent = {{-40, 60}, {-20, 80}})));
            Modelica.Electrical.MultiPhase.Sensors.PowerSensor powerb_m(m = m) annotation(
              Placement(transformation(extent = {{-40, -30}, {-20, -10}})));
          initial equation
            reluctance_e.Phi = Complex(0, 0);
            reluctance_m.Phi = Complex(0, 0);
          equation
            connect(sineVoltage_e.plug_n, converter_e.plug_n) annotation(
              Line(points = {{-80, 50}, {20, 50}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(sineVoltage_e.plug_n, star_e.plug_p) annotation(
              Line(points = {{-80, 50}, {-80, 40}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(sineVoltage_m.plug_n, star_m.plug_p) annotation(
              Line(points = {{-80, -40}, {-80, -50}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(star_e.pin_n, ground_e.p) annotation(
              Line(points = {{-80, 20}, {-80, 20}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(star_m.pin_n, ground_m.p) annotation(
              Line(points = {{-80, -70}, {-80, -70}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(sineVoltage_m.plug_n, converter_m.plug_n) annotation(
              Line(points = {{-80, -40}, {20, -40}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(converter_e.port_p, reluctance_e.port_p) annotation(
              Line(points = {{40, 70}, {80, 70}}, color = {255, 128, 0}, smooth = Smooth.None));
            connect(converter_e.port_n, reluctance_e.port_n) annotation(
              Line(points = {{40, 50}, {80, 50}}, color = {255, 128, 0}, smooth = Smooth.None));
            connect(converter_e.port_n, mground_e.port_p) annotation(
              Line(points = {{40, 50}, {40, 20}}, color = {255, 128, 0}, smooth = Smooth.None));
            connect(converter_m.port_n, reluctance_m.port_n) annotation(
              Line(points = {{40, -40}, {80, -40}}, color = {255, 128, 0}, smooth = Smooth.None));
            connect(converter_m.port_p, loss_m.port_p) annotation(
              Line(points = {{40, -20}, {50, -20}}, color = {255, 128, 0}, smooth = Smooth.None));
            connect(loss_m.port_n, reluctance_m.port_p) annotation(
              Line(points = {{70, -20}, {80, -20}}, color = {255, 128, 0}, smooth = Smooth.None));
            connect(converter_m.port_n, mground_m.port_p) annotation(
              Line(points = {{40, -40}, {40, -70}}, color = {255, 128, 0}, smooth = Smooth.None));
            connect(resistor_e.plug_p, sineVoltage_e.plug_p) annotation(
              Line(points = {{-70, 70}, {-80, 70}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(sineVoltage_m.plug_p, resistor_m.plug_p) annotation(
              Line(points = {{-80, -20}, {-70, -20}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(resistor_e.plug_n, powerb_e.pc) annotation(
              Line(points = {{-50, 70}, {-40, 70}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(powerb_e.pv, powerb_e.pc) annotation(
              Line(points = {{-30, 80}, {-40, 80}, {-40, 70}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(powerb_e.nc, loss_e.plug_p) annotation(
              Line(points = {{-20, 70}, {-10, 70}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(powerb_e.nv, sineVoltage_e.plug_n) annotation(
              Line(points = {{-30, 60}, {-30, 50}, {-80, 50}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(resistor_m.plug_n, powerb_m.pc) annotation(
              Line(points = {{-50, -20}, {-40, -20}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(powerb_m.pc, powerb_m.pv) annotation(
              Line(points = {{-40, -20}, {-40, -10}, {-30, -10}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(powerb_m.nc, converter_m.plug_p) annotation(
              Line(points = {{-20, -20}, {20, -20}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(powerb_m.nv, sineVoltage_m.plug_n) annotation(
              Line(points = {{-30, -30}, {-30, -40}, {-80, -40}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(loss_e.plug_n, sineVoltage_e.plug_n) annotation(
              Line(points = {{-10, 50}, {-80, 50}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(loss_e.plug_p, converter_e.plug_p) annotation(
              Line(points = {{-10, 70}, {20, 70}}, color = {0, 0, 255}, smooth = Smooth.None));
            annotation(
              experiment(StopTime = 40, Interval = 0.01),
              Documentation(info = "<html>
<p>
In this example the eddy current losses are implemented in two different ways. Compare the loss dissipation <code>powerb_e.power</code> and <code>powerb_m.power</code> of the two models indicated by power meters.</p>
</html>"));
          end EddyCurrentLosses;

          model SinglePhaseInductance "Single phase inductance"
            extends Modelica.Icons.Example;
            parameter Modelica.SIunits.Frequency f = 1 "Supply frequency";
            parameter Modelica.SIunits.Voltage VRMS = 100 "RMS supply voltage";
            parameter Modelica.SIunits.Resistance R = 0.1 "Leader cable resistance";
            parameter Modelica.SIunits.Inductance L = 1 "Load inductance";
            parameter Real effectiveTurns = 5 "Effective number of turns";
            final parameter Modelica.SIunits.Reluctance R_m = effectiveTurns ^ 2 / L "Equivalent magnetic reluctance";
            Modelica.Electrical.Analog.Basic.Ground ground_e annotation(
              Placement(transformation(extent = {{-70, 20}, {-50, 40}})));
            Modelica.Electrical.Analog.Basic.Ground ground_m annotation(
              Placement(transformation(extent = {{-70, -80}, {-50, -60}})));
            Modelica.Electrical.Analog.Sources.SineVoltage voltageSource_e(freqHz = f, V = sqrt(2) * VRMS, phase = Modelica.Constants.pi / 2) annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 270, origin = {-60, 70})));
            Modelica.Electrical.Analog.Sources.SineVoltage voltageSource_m(freqHz = f, V = sqrt(2) * VRMS, phase = Modelica.Constants.pi / 2) annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 270, origin = {-60, -30})));
            Modelica.Electrical.Analog.Basic.Resistor resistor_e(R = R) annotation(
              Placement(transformation(extent = {{-40, 70}, {-20, 90}})));
            Modelica.Electrical.Analog.Basic.Resistor resistor_m(R = R) annotation(
              Placement(transformation(extent = {{-40, -30}, {-20, -10}})));
            Modelica.Electrical.Analog.Basic.Inductor inductor_e(L = L) annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 270, origin = {0, 70})));
            Modelica.Magnetic.FundamentalWave.Components.SinglePhaseElectroMagneticConverter converter_m(effectiveTurns = effectiveTurns, orientation = 0) annotation(
              Placement(transformation(extent = {{0, -40}, {20, -20}})));
            Modelica.Magnetic.FundamentalWave.Components.Reluctance reluctance_m(R_m(d = R_m, q = R_m)) annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 270, origin = {60, -30})));
            Modelica.Magnetic.FundamentalWave.Components.Ground groundM_m annotation(
              Placement(transformation(extent = {{10, -80}, {30, -60}})));
          initial equation
            resistor_e.i = 0;
            resistor_m.i = 0;
          equation
            connect(converter_m.port_p, reluctance_m.port_p) annotation(
              Line(points = {{20, -20}, {60, -20}}, color = {255, 128, 0}, smooth = Smooth.None));
            connect(converter_m.port_n, reluctance_m.port_n) annotation(
              Line(points = {{20, -40}, {60, -40}}, color = {255, 128, 0}, smooth = Smooth.None));
            connect(converter_m.port_n, groundM_m.port_p) annotation(
              Line(points = {{20, -40}, {20, -60}}, color = {255, 128, 0}, smooth = Smooth.None));
            connect(voltageSource_e.n, inductor_e.n) annotation(
              Line(points = {{-60, 60}, {0, 60}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(voltageSource_e.n, ground_e.p) annotation(
              Line(points = {{-60, 60}, {-60, 40}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(voltageSource_m.n, converter_m.pin_n) annotation(
              Line(points = {{-60, -40}, {0, -40}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(voltageSource_m.n, ground_m.p) annotation(
              Line(points = {{-60, -40}, {-60, -60}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(voltageSource_e.p, resistor_e.p) annotation(
              Line(points = {{-60, 80}, {-40, 80}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(resistor_e.n, inductor_e.p) annotation(
              Line(points = {{-20, 80}, {0, 80}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(voltageSource_m.p, resistor_m.p) annotation(
              Line(points = {{-60, -20}, {-40, -20}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(resistor_m.n, converter_m.pin_p) annotation(
              Line(points = {{-20, -20}, {-4.44089e-16, -20}}, color = {0, 0, 255}, smooth = Smooth.None));
            annotation(
              experiment(StopTime = 100, Interval = 0.01));
          end SinglePhaseInductance;

          model MultiPhaseInductance "Multi phase inductance"
            extends Modelica.Icons.Example;
            parameter Integer m = 3 "Number of phases";
            parameter Modelica.SIunits.Frequency f = 1 "Supply frequency";
            parameter Modelica.SIunits.Voltage VRMS = 100 "RMS supply voltage";
            parameter Modelica.SIunits.Resistance R = 0.1 "Leader cable resistance";
            parameter Real effectiveTurns = 5 "Effective number of turns";
            parameter Modelica.SIunits.Inductance L = 1 "Load inductance";
            final parameter Modelica.SIunits.Reluctance R_m = m * effectiveTurns ^ 2 / 2 / L "Equivalent magnetic reluctance";
            Modelica.Electrical.Analog.Basic.Ground ground_e annotation(
              Placement(transformation(extent = {{-70, 10}, {-50, 30}})));
            Modelica.Electrical.Analog.Basic.Ground ground_m annotation(
              Placement(transformation(extent = {{-70, -90}, {-50, -70}})));
            Modelica.Electrical.MultiPhase.Basic.Star star_e(m = m) annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 270, origin = {-60, 40})));
            Modelica.Electrical.MultiPhase.Basic.Star star_m(m = m) annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 270, origin = {-60, -60})));
            Modelica.Electrical.MultiPhase.Sources.SineVoltage voltageSource_e(m = m, freqHz = fill(f, m), V = fill(sqrt(2) * VRMS, m), phase = (-Modelica.Electrical.MultiPhase.Functions.symmetricOrientation(m)) + fill(Modelica.Constants.pi / 2, m)) annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 270, origin = {-60, 70})));
            Modelica.Electrical.MultiPhase.Sources.SineVoltage voltageSource_m(m = m, freqHz = fill(f, m), V = fill(sqrt(2) * VRMS, m), phase = (-Modelica.Electrical.MultiPhase.Functions.symmetricOrientation(m)) + fill(Modelica.Constants.pi / 2, m)) annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 270, origin = {-60, -30})));
            Modelica.Electrical.MultiPhase.Basic.Resistor resistor_e(m = m, R = fill(R, m)) annotation(
              Placement(transformation(extent = {{-40, 70}, {-20, 90}})));
            Modelica.Electrical.MultiPhase.Basic.Resistor resistor_m(m = m, R = fill(R, m)) annotation(
              Placement(transformation(extent = {{-40, -30}, {-20, -10}})));
            Modelica.Electrical.MultiPhase.Basic.Inductor inductor_e(m = m, L = fill(L, m)) annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 270, origin = {0, 70})));
            Modelica.Magnetic.FundamentalWave.Components.MultiPhaseElectroMagneticConverter converter_m(m = m, orientation = Modelica.Electrical.MultiPhase.Functions.symmetricOrientation(m), effectiveTurns = fill(effectiveTurns, m)) annotation(
              Placement(transformation(extent = {{0, -40}, {20, -20}})));
            Modelica.Magnetic.FundamentalWave.Components.Reluctance reluctance_m(R_m(d = R_m, q = R_m)) annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 270, origin = {60, -30})));
            Modelica.Magnetic.FundamentalWave.Components.Ground groundM_m annotation(
              Placement(transformation(extent = {{10, -90}, {30, -70}})));
          initial equation
            resistor_e.i = zeros(m);
            resistor_m.i[1:2] = zeros(2);
          equation
            connect(star_e.plug_p, voltageSource_e.plug_n) annotation(
              Line(points = {{-60, 50}, {-60, 60}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(voltageSource_e.plug_n, inductor_e.plug_n) annotation(
              Line(points = {{-60, 60}, {-1.33227e-15, 60}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(converter_m.port_p, reluctance_m.port_p) annotation(
              Line(points = {{20, -20}, {60, -20}}, color = {255, 128, 0}, smooth = Smooth.None));
            connect(converter_m.port_n, reluctance_m.port_n) annotation(
              Line(points = {{20, -40}, {60, -40}}, color = {255, 128, 0}, smooth = Smooth.None));
            connect(converter_m.port_n, groundM_m.port_p) annotation(
              Line(points = {{20, -40}, {20, -70}}, color = {255, 128, 0}, smooth = Smooth.None));
            connect(voltageSource_m.plug_n, star_m.plug_p) annotation(
              Line(points = {{-60, -40}, {-60, -50}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(voltageSource_m.plug_n, converter_m.plug_n) annotation(
              Line(points = {{-60, -40}, {-4.44089e-16, -40}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(voltageSource_e.plug_p, resistor_e.plug_p) annotation(
              Line(points = {{-60, 80}, {-40, 80}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(resistor_e.plug_n, inductor_e.plug_p) annotation(
              Line(points = {{-20, 80}, {2.66454e-15, 80}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(voltageSource_m.plug_p, resistor_m.plug_p) annotation(
              Line(points = {{-60, -20}, {-40, -20}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(resistor_m.plug_n, converter_m.plug_p) annotation(
              Line(points = {{-20, -20}, {-4.44089e-16, -20}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(star_e.pin_n, ground_e.p) annotation(
              Line(points = {{-60, 30}, {-60, 30}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(star_m.pin_n, ground_m.p) annotation(
              Line(points = {{-60, -70}, {-60, -70}}, color = {0, 0, 255}, smooth = Smooth.None));
            annotation(
              experiment(StopTime = 100, Interval = 0.01));
          end MultiPhaseInductance;
        end Components;

        package BasicMachines "Examples of machines of the FundamentalWave library"
          extends Modelica.Icons.ExamplesPackage;

          model AIMC_DOL "Direct on line start of asynchronous induction machine with squirrel cage"
            extends Modelica.Icons.Example;
            constant Integer m = 3 "Number of stator phases";
            parameter Modelica.SIunits.Voltage VsNominal = 100 "Nominal RMS voltage per phase";
            parameter Modelica.SIunits.Frequency fNominal = 50 "Nominal frequency";
            parameter Modelica.SIunits.Time tOn = 0.1 "Start time of machine";
            parameter Modelica.SIunits.Torque T_Load = 161.4 "Nominal load torque";
            parameter Modelica.SIunits.AngularVelocity w_Load(displayUnit = "1/min") = 1440.45 * 2 * Modelica.Constants.pi / 60 "Nominal load speed";
            parameter Modelica.SIunits.Inertia J_Load = 0.29 "Load inertia";
            parameter Integer p = 2 "Number of pole pairs";
            Modelica.Electrical.Analog.Basic.Ground ground annotation(
              Placement(transformation(origin = {-90, 90}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
            Modelica.Electrical.MultiPhase.Basic.Star star(final m = m) annotation(
              Placement(transformation(extent = {{-50, 80}, {-70, 100}}, rotation = 0)));
            Modelica.Electrical.MultiPhase.Sources.SineVoltage sineVoltage(final m = m, freqHz = fill(fNominal, m), V = fill(sqrt(2.0 / 3.0) * VsNominal, m)) annotation(
              Placement(transformation(origin = {-30, 90}, extent = {{10, -10}, {-10, 10}}, rotation = 0)));
            Electrical.MultiPhase.Ideal.IdealClosingSwitch idealCloser(final m = m, Ron = fill(1e-5, m), Goff = fill(1e-5, m)) annotation(
              Placement(transformation(origin = {0, 60}, extent = {{-10, 10}, {10, -10}}, rotation = 270)));
            Blocks.Sources.BooleanStep booleanStep[m](each startTime = tOn, each startValue = false) annotation(
              Placement(transformation(extent = {{-60, 50}, {-40, 70}}, rotation = 0)));
            Modelica.Electrical.Machines.Sensors.CurrentQuasiRMSSensor currentRMSsensorM annotation(
              Placement(transformation(origin = {0, 30}, extent = {{-10, 10}, {10, -10}}, rotation = 270)));
            Modelica.Electrical.Machines.Sensors.CurrentQuasiRMSSensor currentRMSsensorE annotation(
              Placement(transformation(origin = {-60, 30}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
            Modelica.Electrical.Machines.Utilities.TerminalBox terminalBoxM(terminalConnection = "D") annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0)));
            Modelica.Electrical.Machines.Utilities.TerminalBox terminalBoxE(terminalConnection = "D") annotation(
              Placement(transformation(extent = {{-10, -70}, {10, -50}}, rotation = 0)));
            Modelica.Magnetic.FundamentalWave.BasicMachines.AsynchronousInductionMachines.AIM_SquirrelCage aimcM(Jr = aimcData.Jr, Js = aimcData.Js, p = aimcData.p, fsNominal = aimcData.fsNominal, Rs = aimcData.Rs, TsRef = aimcData.TsRef, alpha20s(displayUnit = "1/K") = aimcData.alpha20s, Lssigma = aimcData.Lssigma, Lszero = aimcData.Lszero, frictionParameters = aimcData.frictionParameters, statorCoreParameters = aimcData.statorCoreParameters, strayLoadParameters = aimcData.strayLoadParameters, Lm = aimcData.Lm, Lrsigma = aimcData.Lrsigma, Rr = aimcData.Rr, TrRef = aimcData.TrRef, alpha20r(displayUnit = "1/K") = aimcData.alpha20r, phiMechanical(fixed = true), wMechanical(fixed = true), TsOperational = 293.15, TrOperational = 293.15) annotation(
              Placement(transformation(extent = {{-10, -30}, {10, -10}}, rotation = 0)));
            Modelica.Electrical.Machines.BasicMachines.AsynchronousInductionMachines.AIM_SquirrelCage aimcE(p = aimcData.p, fsNominal = aimcData.fsNominal, Rs = aimcData.Rs, TsRef = aimcData.TsRef, alpha20s(displayUnit = "1/K") = aimcData.alpha20s, Lszero = aimcData.Lszero, Lssigma = aimcData.Lssigma, Jr = aimcData.Jr, Js = aimcData.Js, frictionParameters = aimcData.frictionParameters, phiMechanical(fixed = true), wMechanical(fixed = true), statorCoreParameters = aimcData.statorCoreParameters, strayLoadParameters = aimcData.strayLoadParameters, Lm = aimcData.Lm, Lrsigma = aimcData.Lrsigma, Rr = aimcData.Rr, alpha20r(displayUnit = "1/K") = aimcData.alpha20r, TsOperational = 293.15, TrRef = aimcData.TrRef, TrOperational = 293.15) annotation(
              Placement(transformation(extent = {{-10, -90}, {10, -70}}, rotation = 0)));
            Modelica.Mechanics.Rotational.Components.Inertia loadInertiaM(J = J_Load) annotation(
              Placement(transformation(extent = {{50, -30}, {70, -10}}, rotation = 0)));
            Modelica.Mechanics.Rotational.Components.Inertia loadInertiaE(J = J_Load) annotation(
              Placement(transformation(extent = {{50, -90}, {70, -70}}, rotation = 0)));
            Modelica.Mechanics.Rotational.Sources.QuadraticSpeedDependentTorque quadraticLoadTorqueM(w_nominal = w_Load, tau_nominal = -T_Load, TorqueDirection = false, useSupport = false) annotation(
              Placement(transformation(extent = {{100, -30}, {80, -10}}, rotation = 0)));
            Modelica.Mechanics.Rotational.Sources.QuadraticSpeedDependentTorque quadraticLoadTorqueE(w_nominal = w_Load, tau_nominal = -T_Load, TorqueDirection = false, useSupport = false) annotation(
              Placement(transformation(extent = {{100, -90}, {80, -70}}, rotation = 0)));
            parameter Electrical.Machines.Utilities.ParameterRecords.AIM_SquirrelCageData aimcData annotation(
              Placement(transformation(extent = {{-40, -100}, {-20, -80}})));
            Electrical.MultiPhase.Sensors.CurrentQuasiRMSSensor currentQuasiRMSSensor annotation(
              Placement(transformation(extent = {{-10, 10}, {10, -10}}, rotation = 270, origin = {0, 10})));
          initial equation
            aimcE.is = zeros(m);
            aimcE.ir = zeros(2);
            aimcM.is = zeros(m);
            aimcM.ir[1:2] = zeros(2);
          equation
            connect(star.pin_n, ground.p) annotation(
              Line(points = {{-70, 90}, {-80, 90}}, color = {0, 0, 255}));
            connect(sineVoltage.plug_n, star.plug_p) annotation(
              Line(points = {{-40, 90}, {-40, 90}, {-50, 90}}, color = {0, 0, 255}));
            connect(aimcM.flange, loadInertiaM.flange_a) annotation(
              Line(points = {{10, -20}, {10, -20}, {50, -20}}, color = {0, 0, 0}));
            connect(loadInertiaM.flange_b, quadraticLoadTorqueM.flange) annotation(
              Line(points = {{70, -20}, {80, -20}}, color = {0, 0, 0}));
            connect(terminalBoxM.plug_sn, aimcM.plug_sn) annotation(
              Line(points = {{-6, -10}, {-6, -10}}, color = {0, 0, 255}));
            connect(terminalBoxM.plug_sp, aimcM.plug_sp) annotation(
              Line(points = {{6, -10}, {6, -10}}, color = {0, 0, 255}));
            connect(aimcE.flange, loadInertiaE.flange_a) annotation(
              Line(points = {{10, -80}, {10, -80}, {50, -80}}, color = {0, 0, 0}));
            connect(loadInertiaE.flange_b, quadraticLoadTorqueE.flange) annotation(
              Line(points = {{70, -80}, {80, -80}}, color = {0, 0, 0}));
            connect(terminalBoxE.plug_sn, aimcE.plug_sn) annotation(
              Line(points = {{-6, -70}, {-6, -70}}, color = {0, 0, 255}));
            connect(terminalBoxE.plug_sp, aimcE.plug_sp) annotation(
              Line(points = {{6, -70}, {6, -70}}, color = {0, 0, 255}));
            connect(currentRMSsensorE.plug_n, terminalBoxE.plugSupply) annotation(
              Line(points = {{-60, 20}, {-60, -60}, {6.10623e-16, -60}, {6.10623e-16, -68}}, color = {0, 0, 255}));
            connect(sineVoltage.plug_p, idealCloser.plug_p) annotation(
              Line(points = {{-20, 90}, {2.33651e-15, 90}, {2.33651e-15, 70}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(idealCloser.plug_n, currentRMSsensorM.plug_p) annotation(
              Line(points = {{-1.33731e-15, 50}, {-1.33731e-15, 47.5}, {9.992e-16, 47.5}, {9.992e-16, 45}, {2.33651e-15, 45}, {2.33651e-15, 40}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(booleanStep.y, idealCloser.control) annotation(
              Line(points = {{-39, 60}, {-7, 60}}, color = {255, 0, 255}, smooth = Smooth.None));
            connect(currentRMSsensorE.plug_p, idealCloser.plug_n) annotation(
              Line(points = {{-60, 40}, {-1.33731e-15, 40}, {-1.33731e-15, 50}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(currentRMSsensorM.plug_n, currentQuasiRMSSensor.plug_p) annotation(
              Line(points = {{-1.33731e-15, 20}, {2.33651e-15, 20}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(terminalBoxM.plugSupply, currentQuasiRMSSensor.plug_n) annotation(
              Line(points = {{6.10623e-16, -8}, {6.10623e-16, -5.55112e-16}, {-1.33731e-15, -5.55112e-16}}, color = {0, 0, 255}, smooth = Smooth.None));
            annotation(
              experiment(StopTime = 1.5, Interval = 0.0002, Tolerance = 1e-05),
              Documentation(info = "<HTML>
<h4>Direct on line (DOL) starting of an asynchronous induction machine with squirrel cage</h4>
<p>
At start time tStart three phase voltage is supplied to the
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.BasicMachines.AsynchronousInductionMachines.AIM_SquirrelCage\">asynchronous induction machine with squirrel cage</a>.
The machine starts from standstill, accelerating
inertias against load torque quadratic dependent on speed, finally reaching nominal speed.</p>

<p>
Simulate for 1.5 seconds and plot (versus time):
</p>

<ul>
<li><code>currentRMSsensorM|E.I</code>: equivalent RMS stator current</li>
<li><code>aimcM|E.wMechanical</code>: machine speed</li>
<li><code>aimcM|E.tauElectrical</code>: machine torque</li>
</ul>
</HTML>"));
          end AIMC_DOL;

          model AIMC_DOL_MultiPhase "Direct on line start of multi phase asynchronous induction machine with squirrel cage"
            extends Modelica.Icons.Example;
            constant Integer m3 = 3 "Number of stator phases of threephase system";
            parameter Integer m = 5 "Number of stator phases";
            parameter Modelica.SIunits.Voltage VsNominal = 100 "Nominal RMS voltage per phase";
            parameter Modelica.SIunits.Frequency fNominal = 50 "Nominal frequency";
            parameter Modelica.SIunits.Time tOn = 0.1 "Start time of machine";
            parameter Modelica.SIunits.Torque T_Load = 161.4 "Nominal load torque";
            parameter Modelica.SIunits.AngularVelocity w_Load(displayUnit = "1/min") = 1440.45 * 2 * Modelica.Constants.pi / 60 "Nominal load speed";
            parameter Modelica.SIunits.Inertia J_Load = 0.29 "Load inertia";
            parameter Integer p = 2 "Number of pole pairs";
            Modelica.Electrical.Analog.Basic.Ground groundM annotation(
              Placement(transformation(origin = {-90, 50}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
            Modelica.Electrical.MultiPhase.Basic.Star starM(final m = m) annotation(
              Placement(transformation(extent = {{-70, 62}, {-90, 82}}, rotation = 0)));
            Modelica.Electrical.MultiPhase.Sources.SineVoltage sineVoltageM(final m = m, freqHz = fill(fNominal, m), V = fill(sqrt(2.0) * VsNominal, m), phase = -Modelica.Electrical.MultiPhase.Functions.symmetricOrientation(m)) annotation(
              Placement(transformation(origin = {-50, 72}, extent = {{10, -10}, {-10, 10}}, rotation = 0)));
            Electrical.MultiPhase.Ideal.IdealClosingSwitch idealCloserM(final m = m, Ron = fill(1e-5 * m / m3, m), Goff = fill(1e-5 * m3 / m, m)) annotation(
              Placement(transformation(origin = {-20, 72}, extent = {{-10, 10}, {10, -10}}, rotation = 0)));
            Blocks.Sources.BooleanStep booleanStepM[m](each startTime = tOn, each startValue = false) annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 90, origin = {-20, 40})));
            Electrical.MultiPhase.Sensors.CurrentQuasiRMSSensor currentRMSsensorM(m = m) annotation(
              Placement(transformation(origin = {10, 72}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
            Modelica.Electrical.Machines.Utilities.TerminalBox terminalBoxM(m = m, terminalConnection = "Y") annotation(
              Placement(transformation(extent = {{20, 60}, {40, 80}}, rotation = 0)));
            Modelica.Magnetic.FundamentalWave.BasicMachines.AsynchronousInductionMachines.AIM_SquirrelCage aimcM(Jr = aimcData.Jr, Js = aimcData.Js, p = aimcData.p, fsNominal = aimcData.fsNominal, TsRef = aimcData.TsRef, alpha20s(displayUnit = "1/K") = aimcData.alpha20s, frictionParameters = aimcData.frictionParameters, statorCoreParameters = aimcData.statorCoreParameters, strayLoadParameters = aimcData.strayLoadParameters, TrRef = aimcData.TrRef, alpha20r(displayUnit = "1/K") = aimcData.alpha20r, phiMechanical(fixed = true), wMechanical(fixed = true), m = m, Rs = aimcData.Rs * m / 3, Lssigma = aimcData.Lssigma * m / 3, Lszero = aimcData.Lszero * m / 3, Lm = aimcData.Lm * m / 3, TsOperational = 293.15, Lrsigma = aimcData.Lrsigma * m / 3, Rr = aimcData.Rr * m / 3, TrOperational = 293.15) annotation(
              Placement(transformation(extent = {{20, 40}, {40, 60}}, rotation = 0)));
            Modelica.Mechanics.Rotational.Components.Inertia loadInertiaM(J = J_Load) annotation(
              Placement(transformation(extent = {{50, 40}, {70, 60}}, rotation = 0)));
            Modelica.Mechanics.Rotational.Sources.QuadraticSpeedDependentTorque quadraticLoadTorqueM(w_nominal = w_Load, tau_nominal = -T_Load, TorqueDirection = false, useSupport = false) annotation(
              Placement(transformation(extent = {{100, 40}, {80, 60}}, rotation = 0)));
            parameter Electrical.Machines.Utilities.ParameterRecords.AIM_SquirrelCageData aimcData annotation(
              Placement(transformation(extent = {{-100, -100}, {-80, -80}})));
            Modelica.Electrical.Analog.Basic.Ground ground3 annotation(
              Placement(transformation(origin = {-90, -52}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
            Modelica.Electrical.MultiPhase.Basic.Star star3(final m = m3) annotation(
              Placement(transformation(extent = {{-70, -40}, {-90, -20}}, rotation = 0)));
            Modelica.Electrical.MultiPhase.Sources.SineVoltage sineVoltage3(final m = m3, V = fill(sqrt(2.0) * VsNominal, m3), phase = -Modelica.Electrical.MultiPhase.Functions.symmetricOrientation(m3), freqHz = fill(fNominal, m3)) annotation(
              Placement(transformation(origin = {-50, -30}, extent = {{10, -10}, {-10, 10}}, rotation = 0)));
            Electrical.MultiPhase.Ideal.IdealClosingSwitch idealCloser3(final m = m3, Ron = fill(1e-5, m3), Goff = fill(1e-5, m3)) annotation(
              Placement(transformation(origin = {-20, -30}, extent = {{-10, 10}, {10, -10}}, rotation = 0)));
            Blocks.Sources.BooleanStep booleanStep3[m3](each startTime = tOn, each startValue = false) annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 90, origin = {-20, -62})));
            Electrical.Machines.Sensors.CurrentQuasiRMSSensor currentRMSsensor3(m = 3) annotation(
              Placement(transformation(origin = {10, -30}, extent = {{-10, 10}, {10, -10}}, rotation = 0)));
            Modelica.Electrical.Machines.Utilities.TerminalBox terminalBox3(terminalConnection = "Y", m = m3) annotation(
              Placement(transformation(extent = {{20, -42}, {40, -22}}, rotation = 0)));
            Modelica.Magnetic.FundamentalWave.BasicMachines.AsynchronousInductionMachines.AIM_SquirrelCage aimc3(Jr = aimcData.Jr, Js = aimcData.Js, p = aimcData.p, fsNominal = aimcData.fsNominal, Rs = aimcData.Rs, TsRef = aimcData.TsRef, alpha20s(displayUnit = "1/K") = aimcData.alpha20s, Lssigma = aimcData.Lssigma, Lszero = aimcData.Lszero, frictionParameters = aimcData.frictionParameters, statorCoreParameters = aimcData.statorCoreParameters, strayLoadParameters = aimcData.strayLoadParameters, Lm = aimcData.Lm, Lrsigma = aimcData.Lrsigma, Rr = aimcData.Rr, TrRef = aimcData.TrRef, alpha20r(displayUnit = "1/K") = aimcData.alpha20r, phiMechanical(fixed = true), wMechanical(fixed = true), m = 3, TsOperational = 293.15, TrOperational = 293.15) annotation(
              Placement(transformation(extent = {{20, -62}, {40, -42}}, rotation = 0)));
            Modelica.Mechanics.Rotational.Components.Inertia loadInertia3(J = J_Load) annotation(
              Placement(transformation(extent = {{50, -62}, {70, -42}}, rotation = 0)));
            Modelica.Mechanics.Rotational.Sources.QuadraticSpeedDependentTorque quadraticLoadTorque3(w_nominal = w_Load, tau_nominal = -T_Load, TorqueDirection = false, useSupport = false) annotation(
              Placement(transformation(extent = {{100, -62}, {80, -42}}, rotation = 0)));
            Blocks.Math.Gain gain(k = m / m3) annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 270, origin = {10, 32})));
            Blocks.Math.Feedback feedback annotation(
              Placement(transformation(extent = {{80, 10}, {100, -10}})));
          initial equation
            aimc3.is[1:2] = zeros(2);
            aimc3.ir[1:2] = zeros(2);
            aimcM.is[1:2] = zeros(2);
            aimcM.ir[1:m - 1] = zeros(m - 1);
          equation
            connect(starM.pin_n, groundM.p) annotation(
              Line(points = {{-90, 72}, {-90, 70}, {-90, 60}}, color = {0, 0, 255}));
            connect(sineVoltageM.plug_n, starM.plug_p) annotation(
              Line(points = {{-60, 72}, {-60, 72}, {-70, 72}}, color = {0, 0, 255}));
            connect(aimcM.flange, loadInertiaM.flange_a) annotation(
              Line(points = {{40, 50}, {40, 50}, {50, 50}}, color = {0, 0, 0}));
            connect(terminalBoxM.plug_sn, aimcM.plug_sn) annotation(
              Line(points = {{24, 60}, {24, 60}}, color = {0, 0, 255}));
            connect(terminalBoxM.plug_sp, aimcM.plug_sp) annotation(
              Line(points = {{36, 60}, {36, 60}}, color = {0, 0, 255}));
            connect(sineVoltageM.plug_p, idealCloserM.plug_p) annotation(
              Line(points = {{-40, 72}, {-30, 72}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(idealCloserM.plug_n, currentRMSsensorM.plug_p) annotation(
              Line(points = {{-10, 72}, {-5.55112e-16, 72}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(booleanStepM.y, idealCloserM.control) annotation(
              Line(points = {{-20, 51}, {-20, 65}}, color = {255, 0, 255}, smooth = Smooth.None));
            connect(star3.pin_n, ground3.p) annotation(
              Line(points = {{-90, -30}, {-90, -42}}, color = {0, 0, 255}));
            connect(sineVoltage3.plug_n, star3.plug_p) annotation(
              Line(points = {{-60, -30}, {-60, -30}, {-70, -30}}, color = {0, 0, 255}));
            connect(aimc3.flange, loadInertia3.flange_a) annotation(
              Line(points = {{40, -52}, {40, -52}, {50, -52}}, color = {0, 0, 0}));
            connect(loadInertia3.flange_b, quadraticLoadTorque3.flange) annotation(
              Line(points = {{70, -52}, {74, -52}, {76, -52}, {80, -52}}, color = {0, 0, 0}));
            connect(terminalBox3.plug_sn, aimc3.plug_sn) annotation(
              Line(points = {{24, -42}, {24, -42}}, color = {0, 0, 255}));
            connect(terminalBox3.plug_sp, aimc3.plug_sp) annotation(
              Line(points = {{36, -42}, {36, -42}}, color = {0, 0, 255}));
            connect(terminalBox3.plugSupply, currentRMSsensor3.plug_n) annotation(
              Line(points = {{30, -40}, {30, -30}, {20, -30}}, color = {0, 0, 255}));
            connect(sineVoltage3.plug_p, idealCloser3.plug_p) annotation(
              Line(points = {{-40, -30}, {-30, -30}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(idealCloser3.plug_n, currentRMSsensor3.plug_p) annotation(
              Line(points = {{-10, -30}, {-5.55112e-16, -30}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(booleanStep3.y, idealCloser3.control) annotation(
              Line(points = {{-20, -51}, {-20, -37}}, color = {255, 0, 255}, smooth = Smooth.None));
            connect(currentRMSsensorM.I, gain.u) annotation(
              Line(points = {{10, 62}, {10, 44}}, color = {0, 0, 127}, smooth = Smooth.None));
            connect(loadInertiaM.flange_b, quadraticLoadTorqueM.flange) annotation(
              Line(points = {{70, 50}, {80, 50}}, color = {0, 0, 0}, smooth = Smooth.None));
            connect(currentRMSsensor3.I, feedback.u1) annotation(
              Line(points = {{10, -19}, {10, -6.66134e-16}, {82, -6.66134e-16}}, color = {0, 0, 127}, smooth = Smooth.None));
            connect(feedback.u2, gain.y) annotation(
              Line(points = {{90, 8}, {90, 16}, {10, 16}, {10, 21}}, color = {0, 0, 127}, smooth = Smooth.None));
            connect(currentRMSsensorM.plug_n, terminalBoxM.plugSupply) annotation(
              Line(points = {{20, 72}, {30, 72}, {30, 62}}, color = {0, 0, 255}, smooth = Smooth.None));
            annotation(
              experiment(StopTime = 1.5, Interval = 0.0002, Tolerance = 1e-05),
              Documentation(info = "<HTML>
<h4>Direct on line (DOL) starting of an asynchronous induction machine with squirrel cage</h4>
<p>
At start time tStart voltages are supplied to the
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.BasicMachines.AsynchronousInductionMachines.AIM_SquirrelCage\">multi phase asynchronous induction machines with squirrel cage</a>.
The machines starts from standstill, accelerating
inertias against load torque quadratic dependent on speed, finally reaching nominal speed. Two equivalent machines with different numbers of phases are compared and their equal behavior is demonstrated.</p>

<p>
Simulate for 1.5 seconds and plot (versus time):
</p>

<ul>
<li><code>aimcM|M3.tauElectrical</code>: machine torque</li>
<li><code>aimsM/M3.wMechanical</code>: machine speed</li>
<li><code>feedback.y</code>: zero since difference of three phase current phasor and scaled multi phase current phasor are equal</li>
</ul>
</HTML>"),
              Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, 100}, {100, 20}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 170}, fillPattern = FillPattern.Solid, pattern = LinePattern.Dash), Rectangle(extent = {{-100, -10}, {100, -80}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 170}, fillPattern = FillPattern.Solid, pattern = LinePattern.Dash), Text(extent = {{20, 88}, {100, 80}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 170}, fillPattern = FillPattern.Solid, textString = "%m phase machine
              ", textStyle = {TextStyle.Bold}), Text(extent = {{20, -22}, {100, -30}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 170}, fillPattern = FillPattern.Solid, textString = "3 phase machine
              ", textStyle = {TextStyle.Bold})}));
          end AIMC_DOL_MultiPhase;

          model AIMS_Start "Starting of asynchronous induction machine with slip rings"
            extends Modelica.Icons.Example;
            constant Integer m = 3 "Number of phases";
            parameter Modelica.SIunits.Voltage VsNominal = 100 "Nominal RMS voltage per phase";
            parameter Modelica.SIunits.Frequency fNominal = 50 "Nominal frequency";
            parameter Modelica.SIunits.Time tOn = 0.1 "Start time of machine";
            parameter Modelica.SIunits.Resistance RStart = 0.16 / aimsData.turnsRatio ^ 2 "Starting resistance";
            parameter Modelica.SIunits.Time tRheostat = 1.0 "Time of shortening the rheostat";
            parameter Modelica.SIunits.Torque T_Load = 161.4 "Nominal load torque";
            parameter Modelica.SIunits.AngularVelocity w_Load(displayUnit = "1/min") = Modelica.SIunits.Conversions.from_rpm(1440.45) "Nominal load speed";
            parameter Modelica.SIunits.Inertia J_Load = 0.29 "Load inertia";
            Modelica.Electrical.Analog.Basic.Ground ground annotation(
              Placement(transformation(origin = {-90, 90}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
            Modelica.Electrical.MultiPhase.Basic.Star star(final m = m) annotation(
              Placement(transformation(extent = {{-50, 80}, {-70, 100}}, rotation = 0)));
            Modelica.Electrical.MultiPhase.Sources.SineVoltage sineVoltage(final m = m, V = fill(sqrt(2.0 / 3.0) * VsNominal, m), freqHz = fill(fNominal, m)) annotation(
              Placement(transformation(origin = {-30, 90}, extent = {{10, -10}, {-10, 10}}, rotation = 0)));
            Modelica.Electrical.MultiPhase.Ideal.IdealClosingSwitch idealCloser(final m = m, Ron = fill(1e-5, m), Goff = fill(1e-5, m)) annotation(
              Placement(transformation(origin = {0, 60}, extent = {{-10, 10}, {10, -10}}, rotation = 270)));
            Modelica.Blocks.Sources.BooleanStep booleanStep[m](each startTime = tOn) annotation(
              Placement(transformation(extent = {{-60, 50}, {-40, 70}}, rotation = 0)));
            Electrical.MultiPhase.Sensors.CurrentQuasiRMSSensor currentRMSsensorM annotation(
              Placement(transformation(origin = {0, 20}, extent = {{-10, 10}, {10, -10}}, rotation = 270)));
            Modelica.Electrical.Machines.Sensors.CurrentQuasiRMSSensor currentRMSsensorE annotation(
              Placement(transformation(origin = {-60, 20}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
            Modelica.Electrical.Machines.Utilities.TerminalBox terminalBoxM(terminalConnection = "D") annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0)));
            Modelica.Electrical.Machines.Utilities.TerminalBox terminalBoxE(terminalConnection = "D") annotation(
              Placement(transformation(extent = {{-10, -70}, {10, -50}}, rotation = 0)));
            Modelica.Magnetic.FundamentalWave.BasicMachines.AsynchronousInductionMachines.AIM_SlipRing aimsM(Jr = aimsData.Jr, Js = aimsData.Js, p = aimsData.p, fsNominal = aimsData.fsNominal, Rs = aimsData.Rs, TsRef = aimsData.TsRef, alpha20s(displayUnit = "1/K") = aimsData.alpha20s, Lssigma = aimsData.Lssigma, Lszero = aimsData.Lszero, frictionParameters = aimsData.frictionParameters, statorCoreParameters = aimsData.statorCoreParameters, strayLoadParameters = aimsData.strayLoadParameters, phiMechanical(fixed = true), wMechanical(fixed = true), Lm = aimsData.Lm, Lrsigma = aimsData.Lrsigma, Lrzero = aimsData.Lrzero, Rr = aimsData.Rr, TrRef = aimsData.TrRef, alpha20r(displayUnit = "1/K") = aimsData.alpha20r, useTurnsRatio = aimsData.useTurnsRatio, VsNominal = aimsData.VsNominal, VrLockedRotor = aimsData.VrLockedRotor, rotorCoreParameters = aimsData.rotorCoreParameters, TurnsRatio = aimsData.turnsRatio, TsOperational = 293.15, TrOperational = 293.15) annotation(
              Placement(transformation(extent = {{-10, -30}, {10, -10}}, rotation = 0)));
            Modelica.Electrical.Machines.BasicMachines.AsynchronousInductionMachines.AIM_SlipRing aimsE(p = aimsData.p, fsNominal = aimsData.fsNominal, Rs = aimsData.Rs, TsRef = aimsData.TsRef, alpha20s(displayUnit = "1/K") = aimsData.alpha20s, Lszero = aimsData.Lszero, Lssigma = aimsData.Lssigma, Jr = aimsData.Jr, Js = aimsData.Js, frictionParameters = aimsData.frictionParameters, phiMechanical(fixed = true), wMechanical(fixed = true), statorCoreParameters = aimsData.statorCoreParameters, strayLoadParameters = aimsData.strayLoadParameters, Lm = aimsData.Lm, Lrsigma = aimsData.Lrsigma, Lrzero = aimsData.Lrzero, Rr = aimsData.Rr, TrRef = aimsData.TrRef, alpha20r(displayUnit = "1/K") = aimsData.alpha20r, useTurnsRatio = aimsData.useTurnsRatio, VsNominal = aimsData.VsNominal, VrLockedRotor = aimsData.VrLockedRotor, rotorCoreParameters = aimsData.rotorCoreParameters, TsOperational = 566.3, turnsRatio = aimsData.turnsRatio, TrOperational = 566.3) annotation(
              Placement(transformation(extent = {{-10, -90}, {10, -70}}, rotation = 0)));
            Electrical.Machines.Utilities.SwitchedRheostat rheostatM(RStart = RStart, tStart = tRheostat, m = m) annotation(
              Placement(transformation(extent = {{-40, -30}, {-20, -10}}, rotation = 0)));
            Electrical.Machines.Utilities.SwitchedRheostat rheostatE(RStart = RStart, tStart = tRheostat, m = m) annotation(
              Placement(transformation(extent = {{-40, -90}, {-20, -70}}, rotation = 0)));
            Modelica.Mechanics.Rotational.Components.Inertia loadInertiaM(J = J_Load) annotation(
              Placement(transformation(extent = {{50, -30}, {70, -10}}, rotation = 0)));
            Modelica.Mechanics.Rotational.Components.Inertia loadInertiaE(J = J_Load) annotation(
              Placement(transformation(extent = {{50, -90}, {70, -70}}, rotation = 0)));
            Modelica.Mechanics.Rotational.Sources.QuadraticSpeedDependentTorque quadraticLoadTorqueM(tau_nominal = -T_Load, TorqueDirection = false, useSupport = false, w_nominal = w_Load) annotation(
              Placement(transformation(extent = {{100, -30}, {80, -10}}, rotation = 0)));
            Modelica.Mechanics.Rotational.Sources.QuadraticSpeedDependentTorque quadraticLoadTorqueE(tau_nominal = -T_Load, TorqueDirection = false, useSupport = false, w_nominal = w_Load) annotation(
              Placement(transformation(extent = {{100, -90}, {80, -70}}, rotation = 0)));
            parameter Electrical.Machines.Utilities.ParameterRecords.AIM_SlipRingData aimsData annotation(
              Placement(transformation(extent = {{-60, -100}, {-40, -80}})));
          initial equation
            aimsE.is = zeros(3);
            aimsE.ir = zeros(3);
            aimsM.is = zeros(3);
            aimsM.ir = zeros(3);
          equation
            connect(star.pin_n, ground.p) annotation(
              Line(points = {{-70, 90}, {-80, 90}}, color = {0, 0, 255}));
            connect(sineVoltage.plug_n, star.plug_p) annotation(
              Line(points = {{-40, 90}, {-40, 90}, {-50, 90}}, color = {0, 0, 255}));
            connect(loadInertiaE.flange_b, quadraticLoadTorqueE.flange) annotation(
              Line(points = {{70, -80}, {80, -80}}, color = {0, 0, 0}));
            connect(aimsE.flange, loadInertiaE.flange_a) annotation(
              Line(points = {{10, -80}, {50, -80}}, color = {0, 0, 0}));
            connect(booleanStep.y, idealCloser.control) annotation(
              Line(points = {{-39, 60}, {-7, 60}}, color = {255, 0, 255}));
            connect(terminalBoxE.plug_sn, aimsE.plug_sn) annotation(
              Line(points = {{-6, -70}, {-6, -70}}, color = {0, 0, 255}));
            connect(terminalBoxE.plug_sp, aimsE.plug_sp) annotation(
              Line(points = {{6, -70}, {6, -70}}, color = {0, 0, 255}));
            connect(rheostatE.plug_p, aimsE.plug_rp) annotation(
              Line(points = {{-20, -74}, {-18, -74}, {-10, -74}}, color = {0, 0, 255}));
            connect(rheostatE.plug_n, aimsE.plug_rn) annotation(
              Line(points = {{-20, -86}, {-18, -86}, {-10, -86}}, color = {0, 0, 255}));
            connect(loadInertiaM.flange_b, quadraticLoadTorqueM.flange) annotation(
              Line(points = {{70, -20}, {70, -20}, {80, -20}}, color = {0, 0, 0}));
            connect(aimsM.flange, loadInertiaM.flange_a) annotation(
              Line(points = {{10, -20}, {10, -20}, {50, -20}}, color = {0, 0, 0}));
            connect(terminalBoxM.plug_sp, aimsM.plug_sp) annotation(
              Line(points = {{6, -10}, {6, -10}}, color = {0, 0, 255}));
            connect(terminalBoxM.plug_sn, aimsM.plug_sn) annotation(
              Line(points = {{-6, -10}, {-6, -10}}, color = {0, 0, 255}));
            connect(currentRMSsensorM.plug_n, terminalBoxM.plugSupply) annotation(
              Line(points = {{-1.33731e-15, 10}, {-1.33731e-15, 5.5}, {-7.26687e-16, 5.5}, {-7.26687e-16, 1}, {6.10623e-16, 1}, {6.10623e-16, -8}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(rheostatM.plug_p, aimsM.plug_rp) annotation(
              Line(points = {{-20, -14}, {-10, -14}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(rheostatM.plug_n, aimsM.plug_rn) annotation(
              Line(points = {{-20, -26}, {-10, -26}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(currentRMSsensorE.plug_n, terminalBoxE.plugSupply) annotation(
              Line(points = {{-60, 10}, {-60, -60}, {6.10623e-16, -60}, {6.10623e-16, -68}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(idealCloser.plug_n, currentRMSsensorM.plug_p) annotation(
              Line(points = {{-1.33731e-15, 50}, {-1.33731e-15, 45}, {9.992e-16, 45}, {9.992e-16, 40}, {2.33651e-15, 40}, {2.33651e-15, 30}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(currentRMSsensorE.plug_p, idealCloser.plug_n) annotation(
              Line(points = {{-60, 30}, {-1.33731e-15, 30}, {-1.33731e-15, 50}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(idealCloser.plug_p, sineVoltage.plug_p) annotation(
              Line(points = {{2.33651e-15, 70}, {2.33651e-15, 90}, {-20, 90}}, color = {0, 0, 255}, smooth = Smooth.None));
            annotation(
              experiment(StopTime = 1.5, Interval = 0.001, Tolerance = 1e-05),
              Documentation(info = "<HTML>
<h4>Starting of an asynchronous induction machine with slipring rotor resistance starting</h4>
<p>
At start time <code>tOn</code> three phase voltage is supplied to the
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.BasicMachines.AsynchronousInductionMachines.AIM_SlipRing\">asynchronous induction machine with sliprings</a>.
The machine starts from standstill, accelerating inertias against load torque quadratic dependent on speed,
using a starting resistance. At time tRheostat external rotor resistance is shortened, finally reaching nominal speed.</p>

<p>
Simulate for 1.5 seconds and plot (versus time):
</p>

<ul>
<li><code>currentRMSsensorM|E.I</code>: equivalent RMS stator current</li>
<li><code>aimsM/E.wMechanical</code>: machine speed</li>
<li><code>aimsM|E.tauElectrical</code>: machine torque</li>
</ul>
</HTML>"));
          end AIMS_Start;

          model AIMS_Start_MultiPhase "Starting of multi phase asynchronous induction machine with slip rings"
            import Modelica;
            extends Modelica.Icons.Example;
            constant Integer m3 = 3 "Number of stator phases of threephase system";
            parameter Integer m = 5 "Number of stator phases";
            parameter Integer mr = 5 "Number of rotor phases";
            parameter Modelica.SIunits.Voltage VsNominal = 100 "Nominal RMS voltage per phase";
            parameter Modelica.SIunits.Frequency fNominal = 50 "Nominal frequency";
            parameter Modelica.SIunits.Time tOn = 0.1 "Start time of machine";
            parameter Modelica.SIunits.Resistance RStart = 0.16 / aimsData.turnsRatio ^ 2 "Starting resistance";
            parameter Modelica.SIunits.Time tRheostat = 1.0 "Time of shortening the rheostat";
            parameter Modelica.SIunits.Torque T_Load = 161.4 "Nominal load torque";
            parameter Modelica.SIunits.AngularVelocity w_Load(displayUnit = "1/min") = Modelica.SIunits.Conversions.from_rpm(1440.45) "Nominal load speed";
            parameter Modelica.SIunits.Inertia J_Load = 0.29 "Load inertia";
            Modelica.Electrical.Machines.Utilities.TerminalBox terminalBoxM(m = m, terminalConnection = "Y") annotation(
              Placement(transformation(extent = {{20, 50}, {40, 70}}, rotation = 0)));
            Modelica.Electrical.Machines.Utilities.TerminalBox terminalBox3(terminalConnection = "Y", m = m3) annotation(
              Placement(transformation(extent = {{20, -50}, {40, -30}}, rotation = 0)));
            Modelica.Magnetic.FundamentalWave.BasicMachines.AsynchronousInductionMachines.AIM_SlipRing aimsM(Jr = aimsData.Jr, Js = aimsData.Js, p = aimsData.p, fsNominal = aimsData.fsNominal, TsRef = aimsData.TsRef, alpha20s(displayUnit = "1/K") = aimsData.alpha20s, frictionParameters = aimsData.frictionParameters, statorCoreParameters = aimsData.statorCoreParameters, strayLoadParameters = aimsData.strayLoadParameters, phiMechanical(fixed = true), wMechanical(fixed = true), TrRef = aimsData.TrRef, alpha20r(displayUnit = "1/K") = aimsData.alpha20r, useTurnsRatio = aimsData.useTurnsRatio, VsNominal = aimsData.VsNominal, VrLockedRotor = aimsData.VrLockedRotor, rotorCoreParameters = aimsData.rotorCoreParameters, TurnsRatio = aimsData.turnsRatio, mr = mr, m = m, TsOperational = 293.15, Rs = aimsData.Rs * m / 3, Lssigma = aimsData.Lssigma * m / 3, Lszero = aimsData.Lszero * m / 3, Lm = aimsData.Lm * m / 3, Lrsigma = aimsData.Lrsigma * mr / 3, Lrzero = aimsData.Lrzero * mr / 3, Rr = aimsData.Rr * mr / 3, TrOperational = 293.15) annotation(
              Placement(transformation(extent = {{20, 30}, {40, 50}}, rotation = 0)));
            Modelica.Magnetic.FundamentalWave.BasicMachines.AsynchronousInductionMachines.AIM_SlipRing aims3(p = aimsData.p, fsNominal = aimsData.fsNominal, Rs = aimsData.Rs, TsRef = aimsData.TsRef, alpha20s(displayUnit = "1/K") = aimsData.alpha20s, Lszero = aimsData.Lszero, Lssigma = aimsData.Lssigma, Jr = aimsData.Jr, Js = aimsData.Js, frictionParameters = aimsData.frictionParameters, phiMechanical(fixed = true), wMechanical(fixed = true), statorCoreParameters = aimsData.statorCoreParameters, strayLoadParameters = aimsData.strayLoadParameters, Lm = aimsData.Lm, Lrsigma = aimsData.Lrsigma, Lrzero = aimsData.Lrzero, Rr = aimsData.Rr, TrRef = aimsData.TrRef, alpha20r(displayUnit = "1/K") = aimsData.alpha20r, useTurnsRatio = aimsData.useTurnsRatio, VsNominal = aimsData.VsNominal, VrLockedRotor = aimsData.VrLockedRotor, rotorCoreParameters = aimsData.rotorCoreParameters, m = m3, TsOperational = 566.3, TrOperational = 566.3, TurnsRatio = aimsData.turnsRatio) annotation(
              Placement(transformation(extent = {{20, -70}, {40, -50}}, rotation = 0)));
            Electrical.Machines.Utilities.SwitchedRheostat rheostatM(tStart = tRheostat, m = mr, RStart = RStart * mr / m3) annotation(
              Placement(transformation(extent = {{-10, 30}, {10, 50}}, rotation = 0)));
            Electrical.Machines.Utilities.SwitchedRheostat rheostat3(RStart = RStart, tStart = tRheostat, m = m3) annotation(
              Placement(transformation(extent = {{-10, -70}, {10, -50}}, rotation = 0)));
            Modelica.Mechanics.Rotational.Components.Inertia loadInertiaM(J = J_Load) annotation(
              Placement(transformation(extent = {{50, 30}, {70, 50}}, rotation = 0)));
            Modelica.Mechanics.Rotational.Components.Inertia loadInertia3(J = J_Load) annotation(
              Placement(transformation(extent = {{50, -70}, {70, -50}}, rotation = 0)));
            Modelica.Mechanics.Rotational.Sources.QuadraticSpeedDependentTorque quadraticLoadTorqueM(tau_nominal = -T_Load, TorqueDirection = false, useSupport = false, w_nominal = w_Load) annotation(
              Placement(transformation(extent = {{100, 30}, {80, 50}}, rotation = 0)));
            Modelica.Mechanics.Rotational.Sources.QuadraticSpeedDependentTorque quadraticLoadTorque3(tau_nominal = -T_Load, TorqueDirection = false, useSupport = false, w_nominal = w_Load) annotation(
              Placement(transformation(extent = {{100, -70}, {80, -50}}, rotation = 0)));
            parameter Electrical.Machines.Utilities.ParameterRecords.AIM_SlipRingData aimsData annotation(
              Placement(transformation(extent = {{-100, -100}, {-80, -80}})));
            Electrical.Analog.Basic.Ground groundM annotation(
              Placement(transformation(origin = {-90, 50}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
            Electrical.MultiPhase.Basic.Star starM(final m = m) annotation(
              Placement(transformation(extent = {{-70, 62}, {-90, 82}}, rotation = 0)));
            Electrical.MultiPhase.Sources.SineVoltage sineVoltageM(final m = m, freqHz = fill(fNominal, m), V = fill(sqrt(2.0) * VsNominal, m), phase = -Modelica.Electrical.MultiPhase.Functions.symmetricOrientation(m)) annotation(
              Placement(transformation(origin = {-50, 72}, extent = {{10, -10}, {-10, 10}}, rotation = 0)));
            Electrical.MultiPhase.Ideal.IdealClosingSwitch idealCloserM(final m = m, Ron = fill(1e-5 * m / 3, m), Goff = fill(1e-5 * m3 / m, m)) annotation(
              Placement(transformation(origin = {-20, 72}, extent = {{-10, 10}, {10, -10}}, rotation = 0)));
            Blocks.Sources.BooleanStep booleanStepM[m](each startTime = tOn, each startValue = false) annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0, origin = {-40, 40})));
            Modelica.Electrical.MultiPhase.Sensors.CurrentQuasiRMSSensor currentRMSsensorM(m = m) annotation(
              Placement(transformation(origin = {10, 72}, extent = {{-10, 10}, {10, -10}}, rotation = 0)));
            Electrical.Analog.Basic.Ground ground3 annotation(
              Placement(transformation(origin = {-90, -52}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
            Electrical.MultiPhase.Basic.Star star3(final m = m3) annotation(
              Placement(transformation(extent = {{-70, -40}, {-90, -20}}, rotation = 0)));
            Electrical.MultiPhase.Sources.SineVoltage sineVoltage3(final m = m3, V = fill(sqrt(2.0) * VsNominal, m3), phase = -Modelica.Electrical.MultiPhase.Functions.symmetricOrientation(m3), freqHz = fill(fNominal, m3)) annotation(
              Placement(transformation(origin = {-50, -30}, extent = {{10, -10}, {-10, 10}}, rotation = 0)));
            Electrical.MultiPhase.Ideal.IdealClosingSwitch idealCloser3(final m = m3, Ron = fill(1e-5, m3), Goff = fill(1e-5, m3)) annotation(
              Placement(transformation(origin = {-20, -30}, extent = {{-10, 10}, {10, -10}}, rotation = 0)));
            Blocks.Sources.BooleanStep booleanStep3[m3](each startTime = tOn, each startValue = false) annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0, origin = {-40, -60})));
            Modelica.Electrical.Machines.Sensors.CurrentQuasiRMSSensor currentRMSsensor3(m = 3) annotation(
              Placement(transformation(origin = {10, -30}, extent = {{-10, 10}, {10, -10}}, rotation = 0)));
            Blocks.Math.Gain gain(k = m / m3) annotation(
              Placement(transformation(extent = {{40, 80}, {60, 100}})));
            Modelica.Blocks.Math.Feedback feedback annotation(
              Placement(transformation(extent = {{80, 10}, {100, -10}})));
          initial equation
            aims3.is[1:2] = zeros(2);
            aims3.ir[1:3] = zeros(3);
            aimsM.is[1:2] = zeros(2);
            aimsM.ir[1:mr - 2] = zeros(mr - 2);
          equation
            connect(loadInertia3.flange_b, quadraticLoadTorque3.flange) annotation(
              Line(points = {{70, -60}, {76, -60}, {80, -60}}, color = {0, 0, 0}));
            connect(aims3.flange, loadInertia3.flange_a) annotation(
              Line(points = {{40, -60}, {40, -60}, {50, -60}}, color = {0, 0, 0}));
            connect(terminalBox3.plug_sn, aims3.plug_sn) annotation(
              Line(points = {{24, -50}, {24, -50}}, color = {0, 0, 255}));
            connect(terminalBox3.plug_sp, aims3.plug_sp) annotation(
              Line(points = {{36, -50}, {36, -50}}, color = {0, 0, 255}));
            connect(rheostat3.plug_p, aims3.plug_rp) annotation(
              Line(points = {{10, -54}, {10, -54}, {20, -54}}, color = {0, 0, 255}));
            connect(rheostat3.plug_n, aims3.plug_rn) annotation(
              Line(points = {{10, -66}, {10, -66}, {20, -66}}, color = {0, 0, 255}));
            connect(loadInertiaM.flange_b, quadraticLoadTorqueM.flange) annotation(
              Line(points = {{70, 40}, {70, 40}, {80, 40}}, color = {0, 0, 0}));
            connect(aimsM.flange, loadInertiaM.flange_a) annotation(
              Line(points = {{40, 40}, {40, 40}, {50, 40}}, color = {0, 0, 0}));
            connect(terminalBoxM.plug_sp, aimsM.plug_sp) annotation(
              Line(points = {{36, 50}, {36, 50}}, color = {0, 0, 255}));
            connect(terminalBoxM.plug_sn, aimsM.plug_sn) annotation(
              Line(points = {{24, 50}, {24, 50}}, color = {0, 0, 255}));
            connect(rheostatM.plug_p, aimsM.plug_rp) annotation(
              Line(points = {{10, 46}, {20, 46}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(rheostatM.plug_n, aimsM.plug_rn) annotation(
              Line(points = {{10, 34}, {20, 34}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(starM.pin_n, groundM.p) annotation(
              Line(points = {{-90, 72}, {-90, 72}, {-90, 60}}, color = {0, 0, 255}));
            connect(sineVoltageM.plug_n, starM.plug_p) annotation(
              Line(points = {{-60, 72}, {-60, 72}, {-70, 72}}, color = {0, 0, 255}));
            connect(sineVoltageM.plug_p, idealCloserM.plug_p) annotation(
              Line(points = {{-40, 72}, {-30, 72}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(idealCloserM.plug_n, currentRMSsensorM.plug_p) annotation(
              Line(points = {{-10, 72}, {0, 72}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(booleanStepM.y, idealCloserM.control) annotation(
              Line(points = {{-29, 40}, {-20, 40}, {-20, 65}}, color = {255, 0, 255}, smooth = Smooth.None));
            connect(star3.pin_n, ground3.p) annotation(
              Line(points = {{-90, -30}, {-90, -42}}, color = {0, 0, 255}));
            connect(sineVoltage3.plug_n, star3.plug_p) annotation(
              Line(points = {{-60, -30}, {-60, -30}, {-70, -30}}, color = {0, 0, 255}));
            connect(sineVoltage3.plug_p, idealCloser3.plug_p) annotation(
              Line(points = {{-40, -30}, {-30, -30}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(idealCloser3.plug_n, currentRMSsensor3.plug_p) annotation(
              Line(points = {{-10, -30}, {-5.55112e-16, -30}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(booleanStep3.y, idealCloser3.control) annotation(
              Line(points = {{-29, -60}, {-20, -60}, {-20, -37}}, color = {255, 0, 255}, smooth = Smooth.None));
            connect(currentRMSsensorM.I, gain.u) annotation(
              Line(points = {{10, 82}, {10, 90}, {38, 90}}, color = {0, 0, 127}, smooth = Smooth.None));
            connect(currentRMSsensor3.plug_n, terminalBox3.plugSupply) annotation(
              Line(points = {{20, -30}, {30, -30}, {30, -48}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(currentRMSsensorM.plug_n, terminalBoxM.plugSupply) annotation(
              Line(points = {{20, 72}, {30, 72}, {30, 52}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(currentRMSsensor3.I, feedback.u1) annotation(
              Line(points = {{10, -19}, {10, 8.88178e-016}, {82, 8.88178e-016}}, color = {0, 0, 127}, smooth = Smooth.None));
            connect(feedback.u2, gain.y) annotation(
              Line(points = {{90, 8}, {90, 16}, {100, 16}, {100, 90}, {61, 90}}, color = {0, 0, 127}, smooth = Smooth.None));
            annotation(
              experiment(StopTime = 1.5, Interval = 0.001, Tolerance = 1e-05),
              Documentation(info = "<HTML>
<h4>Starting of an asynchronous induction machine with slipring rotor resistance starting</h4>
<p>
At start time <code>tOn</code> voltages are supplied to the
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.BasicMachines.AsynchronousInductionMachines.AIM_SlipRing\">asynchronous induction machines with sliprings</a>.
The two machine start from standstill, accelerating inertias against load torque quadratic dependent on speed,
using a starting resistance. At time tRheostat external rotor resistance is shortened, finally reaching nominal speed. Two equivalent machines with different numbers of phases are compared and their equal behavior is demonstrated.</p>

<p>
Simulate for 1.5 seconds and plot (versus time):
</p>

<ul>
<li><code>aimcM|M3.tauElectrical</code>: machine torque</li>
<li><code>aimsM|M3.wMechanical</code>: machine speed</li>
<li><code>feedback.y</code>: zero since difference of three phase current phasor and scaled multi phase current phasor are equal</li>
</ul>

</HTML>"),
              Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, 100}, {100, 20}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 170}, fillPattern = FillPattern.Solid, pattern = LinePattern.Dash), Rectangle(extent = {{-100, -10}, {100, -80}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 170}, fillPattern = FillPattern.Solid, pattern = LinePattern.Dash), Text(extent = {{40, 68}, {100, 60}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 170}, fillPattern = FillPattern.Solid, textString = "%m phase machine
              ", textStyle = {TextStyle.Bold}), Text(extent = {{40, -32}, {100, -40}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 170}, fillPattern = FillPattern.Solid, textString = "3 phase machine
              ", textStyle = {TextStyle.Bold})}));
          end AIMS_Start_MultiPhase;

          model SMPM_Inverter "Starting of permanent magnet synchronous machine with inverter"
            extends Modelica.Icons.Example;
            constant Integer m = 3 "Number of phases";
            parameter Modelica.SIunits.Voltage VsNominal = 100 "Nominal RMS voltage per phase";
            parameter Modelica.SIunits.Frequency fsNominal = 50 "Nominal frequency";
            parameter Modelica.SIunits.Frequency fKnee = 50 "Knee frequency of V/f curve";
            parameter Modelica.SIunits.Time tRamp = 1 "Frequency ramp";
            parameter Modelica.SIunits.Torque T_Load = 181.4 "Nominal load torque";
            parameter Modelica.SIunits.Time tStep = 1.2 "Time of load torque step";
            parameter Modelica.SIunits.Inertia J_Load = 0.29 "Load inertia";
            Modelica.Electrical.Analog.Basic.Ground ground annotation(
              Placement(transformation(origin = {-90, 90}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
            Modelica.Electrical.MultiPhase.Basic.Star star(final m = m) annotation(
              Placement(transformation(extent = {{-50, 80}, {-70, 100}}, rotation = 0)));
            Modelica.Electrical.MultiPhase.Sources.SignalVoltage signalVoltage(final m = m) annotation(
              Placement(transformation(origin = {0, 60}, extent = {{10, 10}, {-10, -10}}, rotation = 270)));
            Modelica.Blocks.Sources.Ramp ramp(height = fKnee, duration = tRamp) annotation(
              Placement(transformation(extent = {{-80, 50}, {-60, 70}}, rotation = 0)));
            Modelica.Electrical.Machines.Utilities.VfController vfController(final m = m, VNominal = VsNominal, fNominal = fsNominal, BasePhase = +Modelica.Constants.pi / 2) annotation(
              Placement(transformation(extent = {{-40, 50}, {-20, 70}}, rotation = 0)));
            Electrical.MultiPhase.Sensors.CurrentQuasiRMSSensor currentRMSsensorM annotation(
              Placement(transformation(origin = {0, 30}, extent = {{-10, 10}, {10, -10}}, rotation = 270)));
            Modelica.Electrical.Machines.Sensors.CurrentQuasiRMSSensor currentRMSsensorE annotation(
              Placement(transformation(origin = {-60, 30}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
            Modelica.Electrical.Machines.Utilities.TerminalBox terminalBoxM(terminalConnection = "Y") annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0)));
            Modelica.Electrical.Machines.Utilities.TerminalBox terminalBoxE(terminalConnection = "Y") annotation(
              Placement(transformation(extent = {{-10, -70}, {10, -50}}, rotation = 0)));
            Modelica.Magnetic.FundamentalWave.BasicMachines.SynchronousInductionMachines.SM_PermanentMagnet smpmM(Jr = smpmData.Jr, Js = smpmData.Js, p = smpmData.p, fsNominal = smpmData.fsNominal, Rs = smpmData.Rs, TsRef = smpmData.TsRef, alpha20s(displayUnit = "1/K") = smpmData.alpha20s, Lssigma = smpmData.Lssigma, Lszero = smpmData.Lszero, phiMechanical(fixed = true), wMechanical(fixed = true), Lmd = smpmData.Lmd, Lmq = smpmData.Lmq, useDamperCage = smpmData.useDamperCage, Lrsigmad = smpmData.Lrsigmad, Lrsigmaq = smpmData.Lrsigmaq, Rrd = smpmData.Rrd, Rrq = smpmData.Rrq, TrRef = smpmData.TrRef, alpha20r(displayUnit = "1/K") = smpmData.alpha20r, VsOpenCircuit = smpmData.VsOpenCircuit, frictionParameters = smpmData.frictionParameters, statorCoreParameters = smpmData.statorCoreParameters, strayLoadParameters = smpmData.strayLoadParameters, permanentMagnetLossParameters = smpmData.permanentMagnetLossParameters, TsOperational = 293.15, TrOperational = 293.15, ir(each fixed = true)) annotation(
              Placement(transformation(extent = {{-10, -30}, {10, -10}}, rotation = 0)));
            Modelica.Electrical.Machines.BasicMachines.SynchronousInductionMachines.SM_PermanentMagnet smpmE(p = smpmData.p, fsNominal = smpmData.fsNominal, Rs = smpmData.Rs, TsRef = smpmData.TsRef, alpha20s(displayUnit = "1/K") = smpmData.alpha20s, Lszero = smpmData.Lszero, Lssigma = smpmData.Lssigma, Jr = smpmData.Jr, Js = smpmData.Js, frictionParameters = smpmData.frictionParameters, phiMechanical(fixed = true), wMechanical(fixed = true), statorCoreParameters = smpmData.statorCoreParameters, strayLoadParameters = smpmData.strayLoadParameters, VsOpenCircuit = smpmData.VsOpenCircuit, Lmd = smpmData.Lmd, Lmq = smpmData.Lmq, useDamperCage = smpmData.useDamperCage, Lrsigmad = smpmData.Lrsigmad, Lrsigmaq = smpmData.Lrsigmaq, Rrd = smpmData.Rrd, Rrq = smpmData.Rrq, TrRef = smpmData.TrRef, alpha20r(displayUnit = "1/K") = smpmData.alpha20r, permanentMagnetLossParameters = smpmData.permanentMagnetLossParameters, TsOperational = 293.15, ir(each fixed = true), TrOperational = 293.15) annotation(
              Placement(transformation(extent = {{-10, -90}, {10, -70}}, rotation = 0)));
            Modelica.Electrical.Machines.Sensors.RotorDisplacementAngle rotorAngleM(p = smpmM.p) annotation(
              Placement(transformation(origin = {30, -20}, extent = {{-10, 10}, {10, -10}}, rotation = 270)));
            Modelica.Electrical.Machines.Sensors.RotorDisplacementAngle rotorAngleE(p = smpmE.p) annotation(
              Placement(transformation(origin = {30, -80}, extent = {{-10, 10}, {10, -10}}, rotation = 270)));
            Modelica.Mechanics.Rotational.Components.Inertia loadInertiaM(J = J_Load) annotation(
              Placement(transformation(extent = {{50, -30}, {70, -10}}, rotation = 0)));
            Modelica.Mechanics.Rotational.Components.Inertia loadInertiaE(J = J_Load) annotation(
              Placement(transformation(extent = {{50, -90}, {70, -70}}, rotation = 0)));
            Modelica.Mechanics.Rotational.Sources.TorqueStep torqueStepM(startTime = tStep, stepTorque = -T_Load, useSupport = false, offsetTorque = 0) annotation(
              Placement(transformation(extent = {{100, -30}, {80, -10}}, rotation = 0)));
            Modelica.Mechanics.Rotational.Sources.TorqueStep torqueStepE(startTime = tStep, stepTorque = -T_Load, useSupport = false, offsetTorque = 0) annotation(
              Placement(transformation(extent = {{100, -90}, {80, -70}}, rotation = 0)));
            parameter Electrical.Machines.Utilities.ParameterRecords.SM_PermanentMagnetData smpmData annotation(
              Placement(transformation(extent = {{-40, -100}, {-20, -80}})));
          initial equation
            smpmE.is[1:2] = zeros(2);
            smpmM.is[1:2] = zeros(2);
//conditional damper cage currents are defined as fixed start values
          equation
            connect(signalVoltage.plug_n, star.plug_p) annotation(
              Line(points = {{1.22629e-15, 70}, {1.22629e-15, 90}, {-50, 90}}, color = {0, 0, 255}));
            connect(star.pin_n, ground.p) annotation(
              Line(points = {{-70, 90}, {-80, 90}}, color = {0, 0, 255}));
            connect(ramp.y, vfController.u) annotation(
              Line(points = {{-59, 60}, {-42, 60}}, color = {0, 0, 255}));
            connect(vfController.y, signalVoltage.v) annotation(
              Line(points = {{-19, 60}, {-7, 60}}, color = {0, 0, 255}));
            connect(loadInertiaM.flange_b, torqueStepM.flange) annotation(
              Line(points = {{70, -20}, {80, -20}}, color = {0, 0, 0}));
            connect(signalVoltage.plug_p, currentRMSsensorM.plug_p) annotation(
              Line(points = {{-2.44753e-15, 50}, {2.33651e-15, 50}, {2.33651e-15, 40}}, color = {0, 0, 255}));
            connect(rotorAngleM.plug_n, smpmM.plug_sn) annotation(
              Line(points = {{36, -10}, {36, 0}, {-6, 0}, {-6, -10}}, color = {0, 0, 255}));
            connect(rotorAngleM.plug_p, smpmM.plug_sp) annotation(
              Line(points = {{24, -10}, {15, -10}, {6, -10}}, color = {0, 0, 255}));
            connect(rotorAngleM.flange, smpmM.flange) annotation(
              Line(points = {{20, -20}, {10, -20}}, color = {0, 0, 0}));
            connect(smpmM.flange, loadInertiaM.flange_a) annotation(
              Line(points = {{10, -20}, {50, -20}}, color = {0, 0, 0}));
            connect(terminalBoxM.plug_sn, smpmM.plug_sn) annotation(
              Line(points = {{-6, -10}, {-6, -10}}, color = {0, 0, 255}));
            connect(terminalBoxM.plug_sp, smpmM.plug_sp) annotation(
              Line(points = {{6, -10}, {6, -10}}, color = {0, 0, 255}));
            connect(terminalBoxM.plugSupply, currentRMSsensorM.plug_n) annotation(
              Line(points = {{6.10623e-16, -8}, {6.10623e-16, -1}, {-7.26686e-16, -1}, {-7.26686e-16, 6}, {-1.33731e-15, 6}, {-1.33731e-15, 20}}, color = {0, 0, 255}));
            connect(loadInertiaE.flange_b, torqueStepE.flange) annotation(
              Line(points = {{70, -80}, {80, -80}}, color = {0, 0, 0}));
            connect(rotorAngleE.plug_n, smpmE.plug_sn) annotation(
              Line(points = {{36, -70}, {36, -60}, {-6, -60}, {-6, -70}}, color = {0, 0, 255}));
            connect(rotorAngleE.plug_p, smpmE.plug_sp) annotation(
              Line(points = {{24, -70}, {6, -70}}, color = {0, 0, 255}));
            connect(rotorAngleE.flange, smpmE.flange) annotation(
              Line(points = {{20, -80}, {10, -80}}, color = {0, 0, 0}));
            connect(smpmE.flange, loadInertiaE.flange_a) annotation(
              Line(points = {{10, -80}, {50, -80}}, color = {0, 0, 0}));
            connect(terminalBoxE.plug_sn, smpmE.plug_sn) annotation(
              Line(points = {{-6, -70}, {-6, -70}}, color = {0, 0, 255}));
            connect(terminalBoxE.plug_sp, smpmE.plug_sp) annotation(
              Line(points = {{6, -70}, {6, -70}}, color = {0, 0, 255}));
            connect(currentRMSsensorE.plug_n, terminalBoxE.plugSupply) annotation(
              Line(points = {{-60, 20}, {-60, -50}, {6.10623e-16, -50}, {6.10623e-16, -68}}, color = {0, 0, 255}));
            connect(currentRMSsensorE.plug_p, signalVoltage.plug_p) annotation(
              Line(points = {{-60, 40}, {-2.44753e-15, 40}, {-2.44753e-15, 50}}, color = {0, 0, 255}, smooth = Smooth.None));
            annotation(
              experiment(StopTime = 1.5, Interval = 0.0005, Tolerance = 1e-005),
              Documentation(info = "<HTML>
<h4>Permanent magnet synchronous induction machine fed by an ideal inverter</h4>
<p>

An ideal frequency inverter is modeled by using a
<a href=\"modelica://Modelica.Electrical.Machines.Utilities.VfController\">VfController</a>
and a three-phase <a href=\"modelica://Modelica.Electrical.MultiPhase.Sources.SignalVoltage\">SignalVoltage</a>.
Frequency is raised by a ramp, causing the
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.BasicMachines.SynchronousInductionMachines.SM_PermanentMagnet\">permanent magnet synchronous induction machine</a> to start,
and accelerate the inertias.</p>

<p>At time tStep a load step is applied. Simulate for 1.5 seconds and plot (versus time):</p>

<ul>
<li><code>currentRMSsensorM|E.I</code>: equivalent RMS stator current</li>
<li><code>smpmM|E.wMechanical</code>: machine speed</li>
<li><code>smpmM|E.tauElectrical</code>: machine torque</li>
<li><code>rotorAnglepmsmM|E.rotorDisplacementAngle</code>: rotor displacement angle</li>
</ul>
</HTML>"));
          end SMPM_Inverter;

          model SMPM_Inverter_MultiPhase "Starting of multi phase permanent magnet synchronous machine with inverter"
            import Modelica;
            extends Modelica.Icons.Example;
            constant Integer m3 = 3 "Number of stator phases of threephase system";
            parameter Integer m = 5 "Number of stator phases";
            parameter Modelica.SIunits.Voltage VsNominal = 100 "Nominal RMS voltage per phase";
            parameter Modelica.SIunits.Frequency fsNominal = 50 "Nominal frequency";
            parameter Modelica.SIunits.Frequency fKnee = 50 "Knee frequency of V/f curve";
            parameter Modelica.SIunits.Time tRamp = 1 "Frequency ramp";
            parameter Modelica.SIunits.Torque T_Load = 181.4 "Nominal load torque";
            parameter Modelica.SIunits.Time tStep = 1.2 "Time of load torque step";
            parameter Modelica.SIunits.Inertia J_Load = 0.29 "Load inertia";
            Modelica.Electrical.Analog.Basic.Ground ground3 annotation(
              Placement(transformation(origin = {-90, 90}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
            Modelica.Electrical.MultiPhase.Basic.Star star3(final m = m3) annotation(
              Placement(transformation(extent = {{-50, 80}, {-70, 100}}, rotation = 0)));
            Modelica.Electrical.MultiPhase.Sources.SignalVoltage signalVoltage3(final m = m3) annotation(
              Placement(transformation(origin = {0, 70}, extent = {{10, 10}, {-10, -10}}, rotation = 270)));
            Modelica.Blocks.Sources.Ramp ramp(height = fKnee, duration = tRamp) annotation(
              Placement(transformation(extent = {{-80, 60}, {-60, 80}}, rotation = 0)));
            Modelica.Electrical.Machines.Utilities.VfController vfController3(VNominal = VsNominal, fNominal = fsNominal, BasePhase = +Modelica.Constants.pi / 2, final m = m3, orientation = -Modelica.Electrical.MultiPhase.Functions.symmetricOrientation(m3)) annotation(
              Placement(transformation(extent = {{-40, 60}, {-20, 80}}, rotation = 0)));
            Modelica.Electrical.MultiPhase.Sensors.CurrentQuasiRMSSensor currentRMSsensorM(m = m) annotation(
              Placement(transformation(origin = {30, 20}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
            Modelica.Electrical.Machines.Sensors.CurrentQuasiRMSSensor currentRMSsensor3 annotation(
              Placement(transformation(origin = {-80, 0}, extent = {{-10, 10}, {10, -10}}, rotation = 270)));
            Modelica.Electrical.Machines.Utilities.TerminalBox terminalBoxM(terminalConnection = "Y", m = m) annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0)));
            Modelica.Electrical.Machines.Utilities.TerminalBox terminalBox3(terminalConnection = "Y", m = m3) annotation(
              Placement(transformation(extent = {{-10, -70}, {10, -50}}, rotation = 0)));
            Modelica.Magnetic.FundamentalWave.BasicMachines.SynchronousInductionMachines.SM_PermanentMagnet smpmM(Jr = smpmData.Jr, Js = smpmData.Js, p = smpmData.p, fsNominal = smpmData.fsNominal, TsRef = smpmData.TsRef, alpha20s(displayUnit = "1/K") = smpmData.alpha20s, phiMechanical(fixed = true), wMechanical(fixed = true), useDamperCage = smpmData.useDamperCage, Lrsigmad = smpmData.Lrsigmad, Lrsigmaq = smpmData.Lrsigmaq, Rrd = smpmData.Rrd, Rrq = smpmData.Rrq, TrRef = smpmData.TrRef, alpha20r(displayUnit = "1/K") = smpmData.alpha20r, VsOpenCircuit = smpmData.VsOpenCircuit, frictionParameters = smpmData.frictionParameters, statorCoreParameters = smpmData.statorCoreParameters, strayLoadParameters = smpmData.strayLoadParameters, permanentMagnetLossParameters = smpmData.permanentMagnetLossParameters, m = m, Rs = smpmData.Rs * m / 3, Lssigma = smpmData.Lssigma * m / 3, Lszero = smpmData.Lszero * m / 3, Lmd = smpmData.Lmd * m / 3, Lmq = smpmData.Lmq * m / 3, TsOperational = 293.15, TrOperational = 293.15, ir(each fixed = true)) annotation(
              Placement(transformation(extent = {{-10, -30}, {10, -10}}, rotation = 0)));
            Modelica.Magnetic.FundamentalWave.BasicMachines.SynchronousInductionMachines.SM_PermanentMagnet smpm3(p = smpmData.p, fsNominal = smpmData.fsNominal, Rs = smpmData.Rs, TsRef = smpmData.TsRef, alpha20s(displayUnit = "1/K") = smpmData.alpha20s, Lszero = smpmData.Lszero, Lssigma = smpmData.Lssigma, Jr = smpmData.Jr, Js = smpmData.Js, frictionParameters = smpmData.frictionParameters, phiMechanical(fixed = true), wMechanical(fixed = true), statorCoreParameters = smpmData.statorCoreParameters, strayLoadParameters = smpmData.strayLoadParameters, VsOpenCircuit = smpmData.VsOpenCircuit, Lmd = smpmData.Lmd, Lmq = smpmData.Lmq, useDamperCage = smpmData.useDamperCage, Lrsigmad = smpmData.Lrsigmad, Lrsigmaq = smpmData.Lrsigmaq, Rrd = smpmData.Rrd, Rrq = smpmData.Rrq, TrRef = smpmData.TrRef, alpha20r(displayUnit = "1/K") = smpmData.alpha20r, permanentMagnetLossParameters = smpmData.permanentMagnetLossParameters, m = m3, TsOperational = 293.15, TrOperational = 293.15, ir(each fixed = true)) annotation(
              Placement(transformation(extent = {{-10, -90}, {10, -70}}, rotation = 0)));
            Modelica.Electrical.Machines.Sensors.RotorDisplacementAngle rotorAngle3(p = smpm3.p) annotation(
              Placement(transformation(origin = {30, -80}, extent = {{-10, 10}, {10, -10}}, rotation = 270)));
            Modelica.Mechanics.Rotational.Components.Inertia loadInertiaM(J = J_Load) annotation(
              Placement(transformation(extent = {{50, -30}, {70, -10}}, rotation = 0)));
            Modelica.Mechanics.Rotational.Components.Inertia loadInertia3(J = J_Load) annotation(
              Placement(transformation(extent = {{50, -90}, {70, -70}}, rotation = 0)));
            Modelica.Mechanics.Rotational.Sources.TorqueStep torqueStepM(startTime = tStep, stepTorque = -T_Load, useSupport = false, offsetTorque = 0) annotation(
              Placement(transformation(extent = {{100, -30}, {80, -10}}, rotation = 0)));
            Modelica.Mechanics.Rotational.Sources.TorqueStep torqueStep3(startTime = tStep, stepTorque = -T_Load, useSupport = false, offsetTorque = 0) annotation(
              Placement(transformation(extent = {{100, -90}, {80, -70}}, rotation = 0)));
            parameter Electrical.Machines.Utilities.ParameterRecords.SM_PermanentMagnetData smpmData annotation(
              Placement(transformation(extent = {{-100, -100}, {-80, -80}})));
            Modelica.Electrical.MultiPhase.Sources.SignalVoltage signalVoltageM(final m = m) annotation(
              Placement(transformation(origin = {30, 50}, extent = {{10, 10}, {-10, -10}}, rotation = 270)));
            Modelica.Electrical.MultiPhase.Basic.Star starM(final m = m) annotation(
              Placement(transformation(extent = {{10, -10}, {-10, 10}}, rotation = 180, origin = {60, 90})));
            Modelica.Electrical.Analog.Basic.Ground groundM annotation(
              Placement(transformation(origin = {90, 90}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
            Modelica.Electrical.Machines.Utilities.VfController vfController(VNominal = VsNominal, fNominal = fsNominal, BasePhase = +Modelica.Constants.pi / 2, final m = m, orientation = -Modelica.Electrical.MultiPhase.Functions.symmetricOrientation(m)) annotation(
              Placement(transformation(extent = {{10, 10}, {-10, -10}}, rotation = 180, origin = {-30, 50})));
            Modelica.Blocks.Math.Feedback feedback annotation(
              Placement(transformation(extent = {{-10, 10}, {10, -10}}, rotation = 0, origin = {-50, 0})));
            Modelica.Blocks.Math.Gain gain(k = m / m3) annotation(
              Placement(transformation(extent = {{-20, 10}, {-40, 30}})));
          initial equation
            smpm3.is[1:2] = zeros(2);
            smpmM.is[1:2] = zeros(2);
//conditional damper cage currents are defined as fixed start values
          equation
            connect(signalVoltage3.plug_n, star3.plug_p) annotation(
              Line(points = {{1.22629e-15, 80}, {1.22629e-15, 90}, {-50, 90}}, color = {0, 0, 255}));
            connect(star3.pin_n, ground3.p) annotation(
              Line(points = {{-70, 90}, {-80, 90}}, color = {0, 0, 255}));
            connect(ramp.y, vfController3.u) annotation(
              Line(points = {{-59, 70}, {-42, 70}}, color = {0, 0, 255}));
            connect(vfController3.y, signalVoltage3.v) annotation(
              Line(points = {{-19, 70}, {-7, 70}}, color = {0, 0, 255}));
            connect(loadInertiaM.flange_b, torqueStepM.flange) annotation(
              Line(points = {{70, -20}, {80, -20}}, color = {0, 0, 0}));
            connect(smpmM.flange, loadInertiaM.flange_a) annotation(
              Line(points = {{10, -20}, {50, -20}}, color = {0, 0, 0}));
            connect(terminalBoxM.plug_sn, smpmM.plug_sn) annotation(
              Line(points = {{-6, -10}, {-6, -10}}, color = {0, 0, 255}));
            connect(terminalBoxM.plug_sp, smpmM.plug_sp) annotation(
              Line(points = {{6, -10}, {6, -10}}, color = {0, 0, 255}));
            connect(terminalBoxM.plugSupply, currentRMSsensorM.plug_n) annotation(
              Line(points = {{6.10623e-16, -8}, {6.10623e-16, -1}, {-7.26686e-16, -1}, {-7.26686e-16, 10}, {30, 10}}, color = {0, 0, 255}));
            connect(loadInertia3.flange_b, torqueStep3.flange) annotation(
              Line(points = {{70, -80}, {80, -80}}, color = {0, 0, 0}));
            connect(rotorAngle3.plug_n, smpm3.plug_sn) annotation(
              Line(points = {{36, -70}, {36, -64}, {-6, -64}, {-6, -70}}, color = {0, 0, 255}));
            connect(rotorAngle3.plug_p, smpm3.plug_sp) annotation(
              Line(points = {{24, -70}, {6, -70}}, color = {0, 0, 255}));
            connect(rotorAngle3.flange, smpm3.flange) annotation(
              Line(points = {{20, -80}, {10, -80}}, color = {0, 0, 0}));
            connect(smpm3.flange, loadInertia3.flange_a) annotation(
              Line(points = {{10, -80}, {50, -80}}, color = {0, 0, 0}));
            connect(terminalBox3.plug_sn, smpm3.plug_sn) annotation(
              Line(points = {{-6, -70}, {-6, -70}}, color = {0, 0, 255}));
            connect(terminalBox3.plug_sp, smpm3.plug_sp) annotation(
              Line(points = {{6, -70}, {6, -70}}, color = {0, 0, 255}));
            connect(currentRMSsensor3.plug_n, terminalBox3.plugSupply) annotation(
              Line(points = {{-80, -10}, {-80, -60}, {6.10623e-16, -60}, {6.10623e-16, -68}}, color = {0, 0, 255}));
            connect(signalVoltage3.plug_p, currentRMSsensor3.plug_p) annotation(
              Line(points = {{-2.44753e-15, 60}, {-2.44753e-15, 34}, {-80, 34}, {-80, 10}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(signalVoltageM.plug_n, starM.plug_p) annotation(
              Line(points = {{30, 60}, {30, 90}, {50, 90}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(starM.pin_n, groundM.p) annotation(
              Line(points = {{70, 90}, {80, 90}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(vfController.y, signalVoltageM.v) annotation(
              Line(points = {{-19, 50}, {23, 50}}, color = {0, 0, 127}, smooth = Smooth.None));
            connect(vfController.u, ramp.y) annotation(
              Line(points = {{-42, 50}, {-50, 50}, {-50, 70}, {-59, 70}}, color = {0, 0, 127}, smooth = Smooth.None));
            connect(signalVoltageM.plug_p, currentRMSsensorM.plug_p) annotation(
              Line(points = {{30, 40}, {30, 30}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(currentRMSsensor3.I, feedback.u1) annotation(
              Line(points = {{-69, -2.63122e-15}, {-64.5, -2.63122e-15}, {-64.5, -6.66134e-16}, {-58, -6.66134e-16}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(gain.y, feedback.u2) annotation(
              Line(points = {{-41, 20}, {-50, 20}, {-50, 8}}, color = {0, 0, 127}, smooth = Smooth.None));
            connect(gain.u, currentRMSsensorM.I) annotation(
              Line(points = {{-18, 20}, {20, 20}}, color = {0, 0, 127}, smooth = Smooth.None));
            annotation(
              experiment(StopTime = 1.5, Interval = 0.0005, Tolerance = 1e-005),
              Documentation(info = "<HTML>
<h4>Permanent magnet synchronous induction machine fed by an ideal inverter</h4>
<p>

An ideal frequency inverter is modeled by using
<a href=\"modelica://Modelica.Electrical.Machines.Utilities.VfController\">VfController</a>s
and <a href=\"modelica://Modelica.Electrical.MultiPhase.Sources.SignalVoltage\">SignalVoltages</a>s.
Frequency is raised by a ramp, causing the
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.BasicMachines.SynchronousInductionMachines.SM_PermanentMagnet\">permanent magnet synchronous induction machines</a> to start,
and accelerate the inertias. Two equivalent machines with different numbers of phases are compared and their equal behavior is demonstrated.</p>

<p>At time tStep a load step is applied. Simulate for 1.5 seconds and plot (versus time):</p>

<ul>
<li><code>aimcM|M3.tauElectrical</code>: machine torque</li>
<li><code>aimsM|M3.wMechanical</code>: machine speed</li>
<li><code>feedback.y</code>: zero since difference of three phase current phasor and scaled multi phase current phasor are equal</li>
</ul>

</HTML>"),
              Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-20, 0}, {100, -40}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 170}, fillPattern = FillPattern.Solid, pattern = LinePattern.Dash), Rectangle(extent = {{-20, -60}, {100, -100}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 170}, fillPattern = FillPattern.Solid, pattern = LinePattern.Dash), Text(extent = {{40, -54}, {100, -62}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 170}, fillPattern = FillPattern.Solid, textStyle = {TextStyle.Bold}, textString = "3 phase machine
              "), Text(extent = {{40, -44}, {100, -52}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 170}, fillPattern = FillPattern.Solid, textStyle = {TextStyle.Bold}, textString = "%m phase machine
              ")}));
          end SMPM_Inverter_MultiPhase;

          model SMEE_Generator "Electrical excited synchronous machine operating as generator"
            extends Modelica.Icons.Example;
            import Modelica.Constants.pi;
            constant Integer m = 3 "Number of stator phases";
            parameter Modelica.SIunits.Voltage VsNominal = 100 "Nominal RMS voltage per phase";
            parameter Modelica.SIunits.Frequency fsNominal = 50 "Nominal frequency";
            parameter Modelica.SIunits.AngularVelocity w = Modelica.SIunits.Conversions.from_rpm(1499) "Nominal speed";
            parameter Modelica.SIunits.Current Ie = 19 "Excitation current";
            parameter Modelica.SIunits.Current Ie0 = 10 "Initial excitation current";
            parameter Modelica.SIunits.Angle gamma0(displayUnit = "deg") = 0 "Initial rotor displacement angle";
            parameter Integer p = 2 "Number of pole pairs";
            parameter Modelica.SIunits.Resistance Rs = 0.03 "Warm stator resistance per phase";
            parameter Modelica.SIunits.Inductance Lssigma = 0.1 / (2 * Modelica.Constants.pi * fsNominal) "Stator stray inductance per phase";
            parameter Modelica.SIunits.Inductance Lmd = 1.5 / (2 * Modelica.Constants.pi * fsNominal) "Main field inductance in d-axis";
            parameter Modelica.SIunits.Inductance Lmq = 1.5 / (2 * Modelica.Constants.pi * fsNominal) "Main field inductance in q-axis";
            parameter Modelica.SIunits.Inductance Lrsigmad = 0.05 / (2 * Modelica.Constants.pi * fsNominal) "Damper stray inductance (equivalent three phase winding) d-axis";
            parameter Modelica.SIunits.Inductance Lrsigmaq = Lrsigmad "Damper stray inductance (equivalent three phase winding) q-axis";
            parameter Modelica.SIunits.Resistance Rrd = 0.04 "Warm damper resistance (equivalent three phase winding) d-axis";
            parameter Modelica.SIunits.Resistance Rrq = Rrd "Warm damper resistance (equivalent three phase winding) q-axis";
            Modelica.Electrical.MultiPhase.Basic.Star star(final m = m) annotation(
              Placement(transformation(extent = {{-50, 80}, {-70, 100}}, rotation = 0)));
            Modelica.Electrical.Analog.Basic.Ground ground annotation(
              Placement(transformation(origin = {-90, 90}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
            Modelica.Electrical.MultiPhase.Sources.SineVoltage sineVoltage(final m = m, final V = fill(VsNominal * sqrt(2), m), final freqHz = fill(fsNominal, m)) annotation(
              Placement(transformation(extent = {{-20, 80}, {-40, 100}}, rotation = 0)));
            Electrical.Machines.Sensors.ElectricalPowerSensor electricalPowerSensorM annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 270, origin = {0, 30})));
            Electrical.Machines.Sensors.ElectricalPowerSensor electricalPowerSensorE annotation(
              Placement(transformation(extent = {{-10, 10}, {10, -10}}, rotation = 270, origin = {-60, 30})));
            Modelica.Electrical.Machines.Utilities.TerminalBox terminalBoxM(terminalConnection = "Y") annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0)));
            Modelica.Electrical.Machines.Utilities.TerminalBox terminalBoxE(terminalConnection = "Y") annotation(
              Placement(transformation(extent = {{-10, -70}, {10, -50}}, rotation = 0)));
            Modelica.Magnetic.FundamentalWave.BasicMachines.SynchronousInductionMachines.SM_ElectricalExcited smeeM(phiMechanical(start = -(Modelica.Constants.pi + gamma0) / p, fixed = true), Jr = 0.29, Js = 0.29, p = 2, fsNominal = smeeData.fsNominal, Rs = smeeData.Rs, TsRef = smeeData.TsRef, alpha20s(displayUnit = "1/K") = smeeData.alpha20s, Lssigma = smeeData.Lssigma, Lmd = smeeData.Lmd, Lmq = smeeData.Lmq, useDamperCage = true, Lrsigmad = smeeData.Lrsigmad, Lrsigmaq = smeeData.Lrsigmaq, Rrd = smeeData.Rrd, Rrq = smeeData.Rrq, TrRef = smeeData.TrRef, alpha20r(displayUnit = "1/K") = smeeData.alpha20r, VsNominal = smeeData.VsNominal, IeOpenCircuit = smeeData.IeOpenCircuit, Re = smeeData.Re, TeRef = smeeData.TeRef, alpha20e(displayUnit = "1/K") = smeeData.alpha20e, sigmae = smeeData.sigmae, statorCoreParameters(VRef = 100), strayLoadParameters(IRef = 100), brushParameters(ILinear = 0.01), TsOperational = 293.15, TrOperational = 293.15, TeOperational = 293.15, ir(each fixed = true)) annotation(
              Placement(transformation(extent = {{-10, -30}, {10, -10}}, rotation = 0)));
            Modelica.Electrical.Machines.BasicMachines.SynchronousInductionMachines.SM_ElectricalExcited smeeE(phiMechanical(start = -(Modelica.Constants.pi + gamma0) / p, fixed = true), p = 2, fsNominal = smeeData.fsNominal, Rs = smeeData.Rs, TsRef = smeeData.TsRef, alpha20s(displayUnit = "1/K") = smeeData.alpha20s, Lssigma = smeeData.Lssigma, Jr = 0.29, Js = 0.29, frictionParameters(PRef = 0), statorCoreParameters(PRef = 0, VRef = 100), strayLoadParameters(PRef = 0, IRef = 100), Lmd = smeeData.Lmd, Lmq = smeeData.Lmq, useDamperCage = true, Lrsigmad = smeeData.Lrsigmad, Rrd = smeeData.Rrd, Rrq = smeeData.Rrq, alpha20r(displayUnit = "1/K") = smeeData.alpha20r, VsNominal = smeeData.VsNominal, IeOpenCircuit = smeeData.IeOpenCircuit, Re = smeeData.Re, TeRef = smeeData.TeRef, alpha20e(displayUnit = "1/K") = smeeData.alpha20e, sigmae = smeeData.sigmae, brushParameters(V = 0, ILinear = 0.01), Lrsigmaq = smeeData.Lrsigmaq, TrRef = smeeData.TrRef, TsOperational = 293.15, ir(each fixed = true), TrOperational = 293.15, TeOperational = 293.15) annotation(
              Placement(transformation(extent = {{-10, -90}, {10, -70}}, rotation = 0)));
            Modelica.Electrical.Analog.Basic.Ground groundM annotation(
              Placement(transformation(origin = {-50, -30}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
            Modelica.Electrical.Analog.Basic.Ground groundE annotation(
              Placement(transformation(origin = {-50, -90}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
            Modelica.Electrical.Analog.Sources.RampCurrent rampCurrentM(duration = 0.1, I = Ie - Ie0, offset = Ie0) annotation(
              Placement(transformation(origin = {-30, -20}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
            Modelica.Electrical.Analog.Sources.RampCurrent rampCurrentE(duration = 0.1, I = Ie - Ie0, offset = Ie0) annotation(
              Placement(transformation(origin = {-30, -80}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
            Modelica.Electrical.Machines.Sensors.RotorDisplacementAngle rotorAngleM(p = p) annotation(
              Placement(transformation(origin = {30, -20}, extent = {{-10, 10}, {10, -10}}, rotation = 270)));
            Modelica.Electrical.Machines.Sensors.RotorDisplacementAngle rotorAngleE(p = p) annotation(
              Placement(transformation(origin = {30, -80}, extent = {{-10, 10}, {10, -10}}, rotation = 270)));
            Modelica.Electrical.Machines.Sensors.MechanicalPowerSensor mechanicalPowerSensorM annotation(
              Placement(transformation(extent = {{50, -30}, {70, -10}}, rotation = 0)));
            Modelica.Electrical.Machines.Sensors.MechanicalPowerSensor mechanicalPowerSensorE annotation(
              Placement(transformation(extent = {{50, -90}, {70, -70}}, rotation = 0)));
            Modelica.Mechanics.Rotational.Sources.ConstantSpeed constantSpeedM(final w_fixed = w, useSupport = false) annotation(
              Placement(transformation(extent = {{100, -30}, {80, -10}}, rotation = 0)));
            Modelica.Mechanics.Rotational.Sources.ConstantSpeed constantSpeedE(final w_fixed = w, useSupport = false) annotation(
              Placement(transformation(extent = {{100, -90}, {80, -70}}, rotation = 0)));
            parameter Electrical.Machines.Utilities.SynchronousMachineData smeeData(SNominal = 30e3, VsNominal = 100, fsNominal = 50, IeOpenCircuit = 10, x0 = 0.1, xd = 1.6, xq = 1.6, xdTransient = 0.1375, xdSubtransient = 0.121428571, xqSubtransient = 0.148387097, Ta = 0.014171268, Td0Transient = 0.261177343, Td0Subtransient = 0.006963029, Tq0Subtransient = 0.123345081, alpha20s(displayUnit = "1/K") = Modelica.Electrical.Machines.Thermal.Constants.alpha20Zero, alpha20r(displayUnit = "1/K") = Modelica.Electrical.Machines.Thermal.Constants.alpha20Zero, alpha20e(displayUnit = "1/K") = Modelica.Electrical.Machines.Thermal.Constants.alpha20Zero, TsSpecification = 293.15, TsRef = 293.15, TrSpecification = 293.15, TrRef = 293.15, TeSpecification = 293.15, TeRef = 293.15) annotation(
              Placement(transformation(extent = {{-80, -80}, {-60, -60}})));
          initial equation
            smeeE.is[1:2] = zeros(2);
            smeeM.is[1:2] = zeros(2);
//conditional damper cage currents are defined as fixed start values
          equation
            connect(rotorAngleE.plug_n, smeeE.plug_sn) annotation(
              Line(points = {{36, -70}, {36, -60}, {-6, -60}, {-6, -70}}, color = {0, 0, 255}));
            connect(rotorAngleE.plug_p, smeeE.plug_sp) annotation(
              Line(points = {{24, -70}, {6, -70}}, color = {0, 0, 255}));
            connect(rotorAngleE.flange, smeeE.flange) annotation(
              Line(points = {{20, -80}, {10, -80}}, color = {0, 0, 0}));
            connect(star.pin_n, ground.p) annotation(
              Line(points = {{-70, 90}, {-80, 90}}, color = {0, 0, 255}));
            connect(star.plug_p, sineVoltage.plug_n) annotation(
              Line(points = {{-50, 90}, {-40, 90}}, color = {0, 0, 255}));
            connect(smeeE.flange, mechanicalPowerSensorE.flange_a) annotation(
              Line(points = {{10, -80}, {50, -80}}, color = {0, 0, 0}));
            connect(mechanicalPowerSensorE.flange_b, constantSpeedE.flange) annotation(
              Line(points = {{70, -80}, {80, -80}}, color = {0, 0, 0}));
            connect(rampCurrentE.p, groundE.p) annotation(
              Line(points = {{-30, -90}, {-40, -90}}, color = {0, 0, 255}));
            connect(rampCurrentE.p, smeeE.pin_en) annotation(
              Line(points = {{-30, -90}, {-20, -90}, {-20, -86}, {-10, -86}}, color = {0, 0, 255}));
            connect(rampCurrentE.n, smeeE.pin_ep) annotation(
              Line(points = {{-30, -70}, {-20, -70}, {-20, -74}, {-10, -74}}, color = {0, 0, 255}));
            connect(smeeE.plug_sn, terminalBoxE.plug_sn) annotation(
              Line(points = {{-6, -70}, {-6, -70}}, color = {0, 0, 255}));
            connect(smeeE.plug_sp, terminalBoxE.plug_sp) annotation(
              Line(points = {{6, -70}, {6, -70}}, color = {0, 0, 255}));
            connect(rotorAngleM.plug_n, smeeM.plug_sn) annotation(
              Line(points = {{36, -10}, {36, 0}, {-6, 0}, {-6, -10}}, color = {0, 0, 255}));
            connect(rotorAngleM.plug_p, smeeM.plug_sp) annotation(
              Line(points = {{24, -10}, {24, -10}, {6, -10}}, color = {0, 0, 255}));
            connect(rotorAngleM.flange, smeeM.flange) annotation(
              Line(points = {{20, -20}, {15, -20}, {10, -20}}, color = {0, 0, 0}));
            connect(smeeM.flange, mechanicalPowerSensorM.flange_a) annotation(
              Line(points = {{10, -20}, {50, -20}}, color = {0, 0, 0}));
            connect(mechanicalPowerSensorM.flange_b, constantSpeedM.flange) annotation(
              Line(points = {{70, -20}, {80, -20}}, color = {0, 0, 0}));
            connect(rampCurrentM.p, groundM.p) annotation(
              Line(points = {{-30, -30}, {-35, -30}, {-35, -30}, {-40, -30}}, color = {0, 0, 255}));
            connect(rampCurrentM.p, smeeM.pin_en) annotation(
              Line(points = {{-30, -30}, {-20, -30}, {-20, -26}, {-10, -26}}, color = {0, 0, 255}));
            connect(rampCurrentM.n, smeeM.pin_ep) annotation(
              Line(points = {{-30, -10}, {-20, -10}, {-20, -14}, {-10, -14}}, color = {0, 0, 255}));
            connect(smeeM.plug_sn, terminalBoxM.plug_sn) annotation(
              Line(points = {{-6, -10}, {-6, -10}}, color = {0, 0, 255}));
            connect(smeeM.plug_sp, terminalBoxM.plug_sp) annotation(
              Line(points = {{6, -10}, {6, -10}}, color = {0, 0, 255}));
            connect(electricalPowerSensorM.plug_p, sineVoltage.plug_p) annotation(
              Line(points = {{2.44753e-15, 40}, {2.44753e-15, 90}, {-20, 90}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(electricalPowerSensorE.plug_p, sineVoltage.plug_p) annotation(
              Line(points = {{-60, 40}, {0, 40}, {0, 90}, {-20, 90}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(electricalPowerSensorM.plug_ni, terminalBoxM.plugSupply) annotation(
              Line(points = {{-1.22629e-15, 20}, {-1.22629e-15, 13}, {-6.15667e-16, 13}, {-6.15667e-16, 6}, {6.10623e-16, 6}, {6.10623e-16, -8}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(electricalPowerSensorE.plug_ni, terminalBoxE.plugSupply) annotation(
              Line(points = {{-60, 20}, {-60, -50}, {6.10623e-16, -50}, {6.10623e-16, -68}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(electricalPowerSensorE.plug_nv, star.plug_p) annotation(
              Line(points = {{-50, 30}, {-50, 90}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(electricalPowerSensorM.plug_nv, star.plug_p) annotation(
              Line(points = {{-10, 30}, {-50, 30}, {-50, 90}}, color = {0, 0, 255}, smooth = Smooth.None));
            annotation(
              experiment(StopTime = 30, Interval = 0.005, Tolerance = 1e-06),
              Documentation(info = "<HTML>
<h4>Electrical excited synchronous induction machine as generator</h4>
<p>
An
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.BasicMachines.SynchronousInductionMachines.SM_ElectricalExcited\">electrically excited synchronous generator</a> is connected to the grid and driven with constant speed.
Since speed is slightly smaller than synchronous speed corresponding to mains frequency,
rotor angle is very slowly increased. This allows to see several characteristics dependent on rotor angle.
</p>

<p>
Simulate for 30 seconds and plot (versus <code>rotorAngleM.rotorDisplacementAngle</code>):
</p>

<ul>
<li><code>speedM|E.tauElectrical</code>: machine torque</li>
<li><code>mechanicalPowerSensorM|E.P</code>: mechanical power</li>
</ul>
</HTML>"));
          end SMEE_Generator;

          model SMEE_Generator_MultiPhase "Electrical excited multi phase synchronous machine operating as generator"
            import Modelica;
            extends Modelica.Icons.Example;
            import Modelica.Constants.pi;
            constant Integer m3 = 3 "Number of stator phases of threephase system";
            parameter Integer m = 5 "Number of stator phases";
            parameter Modelica.SIunits.Voltage VsNominal = 100 "Nominal RMS voltage per phase";
            parameter Modelica.SIunits.Frequency fsNominal = 50 "Nominal frequency";
            parameter Modelica.SIunits.AngularVelocity w = Modelica.SIunits.Conversions.from_rpm(1499) "Nominal speed";
            parameter Modelica.SIunits.Current Ie = 19 "Excitation current";
            parameter Modelica.SIunits.Current Ie0 = 10 "Initial excitation current";
            parameter Modelica.SIunits.Angle gamma0(displayUnit = "deg") = 0 "Initial rotor displacement angle";
            parameter Integer p = 2 "Number of pole pairs";
            parameter Modelica.SIunits.Resistance Rs = 0.03 "Warm stator resistance per phase";
            parameter Modelica.SIunits.Inductance Lssigma = 0.1 / (2 * Modelica.Constants.pi * fsNominal) "Stator stray inductance per phase";
            parameter Modelica.SIunits.Inductance Lmd = 1.5 / (2 * Modelica.Constants.pi * fsNominal) "Main field inductance in d-axis";
            parameter Modelica.SIunits.Inductance Lmq = 1.5 / (2 * Modelica.Constants.pi * fsNominal) "Main field inductance in q-axis";
            parameter Modelica.SIunits.Inductance Lrsigmad = 0.05 / (2 * Modelica.Constants.pi * fsNominal) "Damper stray inductance (equivalent three phase winding) d-axis";
            parameter Modelica.SIunits.Inductance Lrsigmaq = Lrsigmad "Damper stray inductance (equivalent three phase winding) q-axis";
            parameter Modelica.SIunits.Resistance Rrd = 0.04 "Warm damper resistance (equivalent three phase winding) d-axis";
            parameter Modelica.SIunits.Resistance Rrq = Rrd "Warm damper resistance (equivalent three phase winding) q-axis";
            Modelica.Electrical.MultiPhase.Basic.Star star3(final m = m3) annotation(
              Placement(transformation(extent = {{-50, -30}, {-70, -10}}, rotation = 0)));
            Modelica.Electrical.Analog.Basic.Ground ground3 annotation(
              Placement(transformation(origin = {-90, -20}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
            Modelica.Electrical.MultiPhase.Sources.SineVoltage sineVoltage3(final m = m3, final V = fill(VsNominal * sqrt(2), m3), phase = -Modelica.Electrical.MultiPhase.Functions.symmetricOrientation(m3), final freqHz = fill(fsNominal, m3)) annotation(
              Placement(transformation(extent = {{-20, -30}, {-40, -10}}, rotation = 0)));
            Modelica.Electrical.MultiPhase.Sensors.PowerSensor electricalPowerSensorM(m = m) annotation(
              Placement(transformation(extent = {{10, -10}, {-10, 10}}, rotation = 180, origin = {-2, 80})));
            Electrical.Machines.Sensors.ElectricalPowerSensor electricalPowerSensor3 annotation(
              Placement(transformation(extent = {{-10, 10}, {10, -10}}, rotation = 0, origin = {0, -20})));
            Modelica.Electrical.Machines.Utilities.TerminalBox terminalBoxM(terminalConnection = "Y", m = m) annotation(
              Placement(transformation(extent = {{-10, 50}, {10, 70}}, rotation = 0)));
            Modelica.Electrical.Machines.Utilities.TerminalBox terminalBox3(terminalConnection = "Y", m = m3) annotation(
              Placement(transformation(extent = {{-10, -70}, {10, -50}}, rotation = 0)));
            Modelica.Magnetic.FundamentalWave.BasicMachines.SynchronousInductionMachines.SM_ElectricalExcited smeeM(phiMechanical(start = -(Modelica.Constants.pi + gamma0) / p, fixed = true), Jr = 0.29, Js = 0.29, p = 2, fsNominal = smeeData.fsNominal, TsRef = smeeData.TsRef, alpha20s(displayUnit = "1/K") = smeeData.alpha20s, useDamperCage = true, Lrsigmad = smeeData.Lrsigmad, Lrsigmaq = smeeData.Lrsigmaq, Rrd = smeeData.Rrd, Rrq = smeeData.Rrq, TrRef = smeeData.TrRef, alpha20r(displayUnit = "1/K") = smeeData.alpha20r, VsNominal = smeeData.VsNominal, IeOpenCircuit = smeeData.IeOpenCircuit, Re = smeeData.Re, TeRef = smeeData.TeRef, alpha20e(displayUnit = "1/K") = smeeData.alpha20e, sigmae = smeeData.sigmae, m = m, Rs = smeeData.Rs * m / 3, Lssigma = smeeData.Lssigma * m / 3, Lmd = smeeData.Lmd * m / 3, Lmq = smeeData.Lmq * m / 3, statorCoreParameters(VRef = 100), strayLoadParameters(IRef = 100), brushParameters(ILinear = 0.01), TsOperational = 293.15, TrOperational = 293.15, TeOperational = 293.15, ir(each fixed = true)) annotation(
              Placement(transformation(extent = {{-10, 30}, {10, 50}}, rotation = 0)));
            Modelica.Magnetic.FundamentalWave.BasicMachines.SynchronousInductionMachines.SM_ElectricalExcited smee3(phiMechanical(start = -(Modelica.Constants.pi + gamma0) / p, fixed = true), p = 2, fsNominal = smeeData.fsNominal, Rs = smeeData.Rs, TsRef = smeeData.TsRef, alpha20s(displayUnit = "1/K") = smeeData.alpha20s, Lssigma = smeeData.Lssigma, Jr = 0.29, Js = 0.29, frictionParameters(PRef = 0), statorCoreParameters(PRef = 0, VRef = 100), strayLoadParameters(PRef = 0, IRef = 100), Lmd = smeeData.Lmd, Lmq = smeeData.Lmq, useDamperCage = true, Lrsigmad = smeeData.Lrsigmad, Rrd = smeeData.Rrd, Rrq = smeeData.Rrq, alpha20r(displayUnit = "1/K") = smeeData.alpha20r, VsNominal = smeeData.VsNominal, IeOpenCircuit = smeeData.IeOpenCircuit, Re = smeeData.Re, TeRef = smeeData.TeRef, alpha20e(displayUnit = "1/K") = smeeData.alpha20e, sigmae = smeeData.sigmae, brushParameters(V = 0, ILinear = 0.01), Lrsigmaq = smeeData.Lrsigmaq, TrRef = smeeData.TrRef, m = m3, TsOperational = 293.15, TrOperational = 293.15, TeOperational = 293.15, ir(each fixed = true)) annotation(
              Placement(transformation(extent = {{-10, -90}, {10, -70}}, rotation = 0)));
            Modelica.Electrical.Analog.Basic.Ground groundRM annotation(
              Placement(transformation(origin = {-20, 20}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
            Modelica.Electrical.Analog.Basic.Ground groundR3 annotation(
              Placement(transformation(origin = {-20, -100}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
            Modelica.Electrical.Analog.Sources.RampCurrent rampCurrentM(duration = 0.1, I = Ie - Ie0, offset = Ie0) annotation(
              Placement(transformation(origin = {-30, 40}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
            Modelica.Electrical.Analog.Sources.RampCurrent rampCurrent3(duration = 0.1, I = Ie - Ie0, offset = Ie0) annotation(
              Placement(transformation(origin = {-30, -80}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
            Modelica.Electrical.Machines.Sensors.RotorDisplacementAngle rotorAngle3(p = p) annotation(
              Placement(transformation(origin = {30, -80}, extent = {{-10, 10}, {10, -10}}, rotation = 270)));
            Modelica.Electrical.Machines.Sensors.MechanicalPowerSensor mechanicalPowerSensorM annotation(
              Placement(transformation(extent = {{20, 30}, {40, 50}}, rotation = 0)));
            Modelica.Electrical.Machines.Sensors.MechanicalPowerSensor mechanicalPowerSensor3 annotation(
              Placement(transformation(extent = {{50, -90}, {70, -70}}, rotation = 0)));
            Modelica.Mechanics.Rotational.Sources.ConstantSpeed constantSpeedM(final w_fixed = w, useSupport = false) annotation(
              Placement(transformation(extent = {{70, 30}, {50, 50}}, rotation = 0)));
            Modelica.Mechanics.Rotational.Sources.ConstantSpeed constantSpeed3(final w_fixed = w, useSupport = false) annotation(
              Placement(transformation(extent = {{100, -90}, {80, -70}}, rotation = 0)));
            parameter Electrical.Machines.Utilities.SynchronousMachineData smeeData(SNominal = 30e3, VsNominal = 100, fsNominal = 50, IeOpenCircuit = 10, x0 = 0.1, xd = 1.6, xq = 1.6, xdTransient = 0.1375, xdSubtransient = 0.121428571, xqSubtransient = 0.148387097, Ta = 0.014171268, Td0Transient = 0.261177343, Td0Subtransient = 0.006963029, Tq0Subtransient = 0.123345081, alpha20s(displayUnit = "1/K") = Modelica.Electrical.Machines.Thermal.Constants.alpha20Zero, alpha20r(displayUnit = "1/K") = Modelica.Electrical.Machines.Thermal.Constants.alpha20Zero, alpha20e(displayUnit = "1/K") = Modelica.Electrical.Machines.Thermal.Constants.alpha20Zero, TsSpecification = 293.15, TsRef = 293.15, TrSpecification = 293.15, TrRef = 293.15, TeSpecification = 293.15, TeRef = 293.15) annotation(
              Placement(transformation(extent = {{-100, -100}, {-80, -80}})));
            Modelica.Electrical.MultiPhase.Sensors.CurrentQuasiRMSSensor currentRMSsensorM(m = m) annotation(
              Placement(transformation(origin = {30, 80}, extent = {{-10, 10}, {10, -10}}, rotation = 0)));
            Modelica.Electrical.Machines.Sensors.CurrentQuasiRMSSensor currentRMSsensor3 annotation(
              Placement(transformation(origin = {30, -20}, extent = {{-10, 10}, {10, -10}}, rotation = 0)));
            Modelica.Blocks.Math.Gain gain(k = m / m3) annotation(
              Placement(transformation(extent = {{10, -10}, {-10, 10}}, rotation = 90, origin = {90, 30})));
            Modelica.Blocks.Math.Feedback feedback annotation(
              Placement(transformation(extent = {{-10, 10}, {10, -10}}, rotation = 0, origin = {90, 0})));
            Modelica.Electrical.MultiPhase.Basic.Star starM(final m = m) annotation(
              Placement(transformation(extent = {{-50, 70}, {-70, 90}}, rotation = 0)));
            Modelica.Electrical.Analog.Basic.Ground groundM annotation(
              Placement(transformation(origin = {-90, 80}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
            Modelica.Electrical.MultiPhase.Sources.SineVoltage sineVoltageM(final V = fill(VsNominal * sqrt(2), m), final freqHz = fill(fsNominal, m), final m = m, phase = -Modelica.Electrical.MultiPhase.Functions.symmetricOrientation(m)) annotation(
              Placement(transformation(extent = {{-20, 70}, {-40, 90}}, rotation = 0)));
          initial equation
            smee3.is[1:2] = zeros(2);
            smeeM.is[1:2] = zeros(2);
//conditional damper cage currents are defined as fixed start values
          equation
            connect(rotorAngle3.plug_n, smee3.plug_sn) annotation(
              Line(points = {{36, -70}, {36, -60}, {-6, -60}, {-6, -70}}, color = {0, 0, 255}));
            connect(rotorAngle3.plug_p, smee3.plug_sp) annotation(
              Line(points = {{24, -70}, {6, -70}}, color = {0, 0, 255}));
            connect(rotorAngle3.flange, smee3.flange) annotation(
              Line(points = {{20, -80}, {10, -80}}, color = {0, 0, 0}));
            connect(star3.pin_n, ground3.p) annotation(
              Line(points = {{-70, -20}, {-75, -20}, {-80, -20}}, color = {0, 0, 255}));
            connect(star3.plug_p, sineVoltage3.plug_n) annotation(
              Line(points = {{-50, -20}, {-48, -20}, {-46, -20}, {-40, -20}}, color = {0, 0, 255}));
            connect(smee3.flange, mechanicalPowerSensor3.flange_a) annotation(
              Line(points = {{10, -80}, {50, -80}}, color = {0, 0, 0}));
            connect(mechanicalPowerSensor3.flange_b, constantSpeed3.flange) annotation(
              Line(points = {{70, -80}, {80, -80}}, color = {0, 0, 0}));
            connect(rampCurrent3.p, groundR3.p) annotation(
              Line(points = {{-30, -90}, {-26, -90}, {-20, -90}}, color = {0, 0, 255}));
            connect(rampCurrent3.p, smee3.pin_en) annotation(
              Line(points = {{-30, -90}, {-20, -90}, {-20, -86}, {-10, -86}}, color = {0, 0, 255}));
            connect(rampCurrent3.n, smee3.pin_ep) annotation(
              Line(points = {{-30, -70}, {-20, -70}, {-20, -74}, {-10, -74}}, color = {0, 0, 255}));
            connect(smee3.plug_sn, terminalBox3.plug_sn) annotation(
              Line(points = {{-6, -70}, {-6, -70}}, color = {0, 0, 255}));
            connect(smee3.plug_sp, terminalBox3.plug_sp) annotation(
              Line(points = {{6, -70}, {6, -70}}, color = {0, 0, 255}));
            connect(smeeM.flange, mechanicalPowerSensorM.flange_a) annotation(
              Line(points = {{10, 40}, {10, 40}, {20, 40}}, color = {0, 0, 0}));
            connect(mechanicalPowerSensorM.flange_b, constantSpeedM.flange) annotation(
              Line(points = {{40, 40}, {44, 40}, {46, 40}, {50, 40}}, color = {0, 0, 0}));
            connect(rampCurrentM.p, groundRM.p) annotation(
              Line(points = {{-30, 30}, {-30, 30}, {-20, 30}}, color = {0, 0, 255}));
            connect(rampCurrentM.p, smeeM.pin_en) annotation(
              Line(points = {{-30, 30}, {-20, 30}, {-20, 34}, {-10, 34}}, color = {0, 0, 255}));
            connect(rampCurrentM.n, smeeM.pin_ep) annotation(
              Line(points = {{-30, 50}, {-20, 50}, {-20, 46}, {-10, 46}}, color = {0, 0, 255}));
            connect(smeeM.plug_sn, terminalBoxM.plug_sn) annotation(
              Line(points = {{-6, 50}, {-6, 50}}, color = {0, 0, 255}));
            connect(smeeM.plug_sp, terminalBoxM.plug_sp) annotation(
              Line(points = {{6, 50}, {6, 50}}, color = {0, 0, 255}));
            connect(electricalPowerSensor3.plug_p, sineVoltage3.plug_p) annotation(
              Line(points = {{-10, -20}, {-20, -20}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(electricalPowerSensor3.plug_nv, star3.plug_p) annotation(
              Line(points = {{6.10623e-16, -10}, {6.10623e-16, -8}, {-50, -8}, {-50, -20}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(gain.y, feedback.u2) annotation(
              Line(points = {{90, 19}, {90, 8}}, color = {0, 0, 127}, smooth = Smooth.None));
            connect(electricalPowerSensor3.plug_ni, currentRMSsensor3.plug_p) annotation(
              Line(points = {{10, -20}, {20, -20}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(currentRMSsensor3.plug_n, terminalBox3.plugSupply) annotation(
              Line(points = {{40, -20}, {40, -40}, {6.10623e-16, -40}, {6.10623e-16, -68}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(currentRMSsensor3.I, feedback.u1) annotation(
              Line(points = {{30, -9}, {30, -6.66134e-016}, {82, -6.66134e-016}}, color = {0, 0, 127}, smooth = Smooth.None));
            connect(terminalBoxM.plugSupply, currentRMSsensorM.plug_n) annotation(
              Line(points = {{6.10623e-16, 52}, {6.10623e-16, 60}, {40, 60}, {40, 80}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(currentRMSsensorM.I, gain.u) annotation(
              Line(points = {{30, 90}, {30, 100}, {90, 100}, {90, 42}}, color = {0, 0, 127}, smooth = Smooth.None));
            connect(starM.pin_n, groundM.p) annotation(
              Line(points = {{-70, 80}, {-75, 80}, {-80, 80}}, color = {0, 0, 255}));
            connect(starM.plug_p, sineVoltageM.plug_n) annotation(
              Line(points = {{-50, 80}, {-48, 80}, {-40, 80}}, color = {0, 0, 255}));
            connect(sineVoltageM.plug_p, electricalPowerSensorM.pc) annotation(
              Line(points = {{-20, 80}, {-12, 80}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(electricalPowerSensorM.pc, electricalPowerSensorM.pv) annotation(
              Line(points = {{-12, 80}, {-12, 70}, {-2, 70}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(electricalPowerSensorM.nc, currentRMSsensorM.plug_p) annotation(
              Line(points = {{8, 80}, {20, 80}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(electricalPowerSensorM.nv, starM.plug_p) annotation(
              Line(points = {{-2, 90}, {-2, 94}, {-50, 94}, {-50, 80}}, color = {0, 0, 255}, smooth = Smooth.None));
            annotation(
              experiment(StopTime = 30, Interval = 0.005, Tolerance = 1e-06),
              Documentation(info = "<HTML>
<h4>Electrical excited synchronous induction machine as generator</h4>
<p>
Two
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.BasicMachines.SynchronousInductionMachines.SM_ElectricalExcited\">electrically excited synchronous generators</a> are connected to grids and driven with constant speed.
Since speed is slightly smaller than synchronous speed corresponding to mains frequency,
rotor angle is very slowly increased. Two equivalent machines with different numbers of phases are compared and their equal behavior is demonstrated.
</p>

<p>
Simulate for 30 seconds and plot (versus <code>rotorAngleM3.rotorDisplacementAngle</code>):
</p>

<ul>
<li><code>aimcM|M3.tauElectrical</code>: machine torque</li>
<li><code>aimsM|M3.wMechanical</code>: machine speed</li>
<li><code>feedback.y</code>: zero since difference of three phase current phasor and scaled multi phase current phasor are equal</li>
</ul>

</HTML>"),
              Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-50, 60}, {70, 20}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 170}, fillPattern = FillPattern.Solid, pattern = LinePattern.Dash), Text(extent = {{10, 16}, {70, 8}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 170}, fillPattern = FillPattern.Solid, textStyle = {TextStyle.Bold}, textString = "%m phase machine
              "), Text(extent = {{10, -52}, {70, -60}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 170}, fillPattern = FillPattern.Solid, textStyle = {TextStyle.Bold}, textString = "3 phase machine
              "), Rectangle(extent = {{-50, -60}, {100, -100}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 170}, fillPattern = FillPattern.Solid, pattern = LinePattern.Dash)}));
          end SMEE_Generator_MultiPhase;

          model SMR_Inverter "Starting of synchronous reluctance machine with inverter"
            extends Modelica.Icons.Example;
            constant Integer m = 3 "Number of stator phases";
            parameter Modelica.SIunits.Voltage VsNominal = 100 "Nominal RMS voltage per phase";
            parameter Modelica.SIunits.Frequency fsNominal = 50 "Nominal frequency";
            parameter Modelica.SIunits.Frequency fKnee = 50 "Knee frequency of V/f curve";
            parameter Modelica.SIunits.Time tRamp = 1 "Frequency ramp";
            parameter Modelica.SIunits.Torque T_Load = 46 "Nominal load torque";
            parameter Modelica.SIunits.Time tStep = 1.2 "Time of load torque step";
            parameter Modelica.SIunits.Inertia J_Load = 0.29 "Load inertia";
            Modelica.Electrical.Analog.Basic.Ground ground annotation(
              Placement(transformation(origin = {-90, 90}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
            Modelica.Electrical.MultiPhase.Basic.Star star(final m = m) annotation(
              Placement(transformation(extent = {{-50, 80}, {-70, 100}}, rotation = 0)));
            Modelica.Electrical.MultiPhase.Sources.SignalVoltage signalVoltage(final m = m) annotation(
              Placement(transformation(origin = {0, 60}, extent = {{10, 10}, {-10, -10}}, rotation = 270)));
            Modelica.Blocks.Sources.Ramp ramp(height = fKnee, duration = tRamp) annotation(
              Placement(transformation(extent = {{-80, 50}, {-60, 70}}, rotation = 0)));
            Modelica.Electrical.Machines.Utilities.VfController vfController(final m = m, VNominal = VsNominal, fNominal = fsNominal) annotation(
              Placement(transformation(extent = {{-40, 50}, {-20, 70}}, rotation = 0)));
            Electrical.MultiPhase.Sensors.CurrentQuasiRMSSensor currentRMSsensorM annotation(
              Placement(transformation(origin = {0, 20}, extent = {{-10, 10}, {10, -10}}, rotation = 270)));
            Modelica.Electrical.Machines.Sensors.CurrentQuasiRMSSensor currentRMSsensorE annotation(
              Placement(transformation(origin = {-60, 20}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
            Modelica.Electrical.Machines.Utilities.TerminalBox terminalBoxM(terminalConnection = "Y") annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0)));
            Modelica.Electrical.Machines.Utilities.TerminalBox terminalBoxE(terminalConnection = "Y") annotation(
              Placement(transformation(extent = {{-10, -70}, {10, -50}}, rotation = 0)));
            Modelica.Magnetic.FundamentalWave.BasicMachines.SynchronousInductionMachines.SM_ReluctanceRotor smrM(Jr = smrData.Jr, Js = smrData.Js, p = smrData.p, fsNominal = smrData.fsNominal, Rs = smrData.Rs, TsRef = smrData.TsRef, alpha20s(displayUnit = "1/K") = smrData.alpha20s, Lssigma = smrData.Lssigma, Lszero = smrData.Lszero, frictionParameters = smrData.frictionParameters, statorCoreParameters = smrData.statorCoreParameters, strayLoadParameters = smrData.strayLoadParameters, phiMechanical(fixed = true), wMechanical(fixed = true), Lmd = smrData.Lmd, Lmq = smrData.Lmq, useDamperCage = smrData.useDamperCage, Lrsigmad = smrData.Lrsigmad, Lrsigmaq = smrData.Lrsigmaq, Rrd = smrData.Rrd, Rrq = smrData.Rrq, TrRef = smrData.TrRef, alpha20r(displayUnit = "1/K") = smrData.alpha20r, TsOperational = 293.15, TrOperational = 293.15, ir(each fixed = true)) annotation(
              Placement(transformation(extent = {{-10, -30}, {10, -10}}, rotation = 0)));
            Modelica.Electrical.Machines.BasicMachines.SynchronousInductionMachines.SM_ReluctanceRotor smrE(p = smrData.p, fsNominal = smrData.fsNominal, Rs = smrData.Rs, TsRef = smrData.TsRef, alpha20s(displayUnit = "1/K") = smrData.alpha20s, Lszero = smrData.Lszero, Lssigma = smrData.Lssigma, Jr = smrData.Jr, Js = smrData.Js, frictionParameters = smrData.frictionParameters, phiMechanical(fixed = true), wMechanical(fixed = true), statorCoreParameters = smrData.statorCoreParameters, strayLoadParameters = smrData.strayLoadParameters, Lmd = smrData.Lmd, Lmq = smrData.Lmq, useDamperCage = smrData.useDamperCage, Lrsigmad = smrData.Lrsigmad, Lrsigmaq = smrData.Lrsigmaq, Rrd = smrData.Rrd, Rrq = smrData.Rrq, TrRef = smrData.TrRef, alpha20r(displayUnit = "1/K") = smrData.alpha20r, TsOperational = 293.15, ir(each fixed = true), TrOperational = 293.15) annotation(
              Placement(transformation(extent = {{-10, -90}, {10, -70}}, rotation = 0)));
            Modelica.Electrical.Machines.Sensors.RotorDisplacementAngle rotorAngleM(p = smrM.p) annotation(
              Placement(transformation(origin = {30, -20}, extent = {{10, -10}, {-10, 10}}, rotation = 90)));
            Modelica.Electrical.Machines.Sensors.RotorDisplacementAngle rotorAngleE(p = smrE.p) annotation(
              Placement(transformation(origin = {30, -80}, extent = {{10, -10}, {-10, 10}}, rotation = 90)));
            Modelica.Mechanics.Rotational.Components.Inertia loadInertiaM(J = J_Load) annotation(
              Placement(transformation(extent = {{50, -30}, {70, -10}}, rotation = 0)));
            Modelica.Mechanics.Rotational.Components.Inertia loadInertiaE(J = J_Load) annotation(
              Placement(transformation(extent = {{50, -90}, {70, -70}}, rotation = 0)));
            Modelica.Mechanics.Rotational.Sources.TorqueStep torqueStepM(startTime = tStep, stepTorque = -T_Load, useSupport = false, offsetTorque = 0) annotation(
              Placement(transformation(extent = {{100, -30}, {80, -10}}, rotation = 0)));
            Modelica.Mechanics.Rotational.Sources.TorqueStep torqueStepE(startTime = tStep, stepTorque = -T_Load, useSupport = false, offsetTorque = 0) annotation(
              Placement(transformation(extent = {{100, -90}, {80, -70}}, rotation = 0)));
            parameter Electrical.Machines.Utilities.ParameterRecords.SM_ReluctanceRotorData smrData annotation(
              Placement(transformation(extent = {{-40, -100}, {-20, -80}})));
          initial equation
            smrE.is[1:2] = zeros(2);
            smrM.is[1:2] = zeros(2);
//conditional damper cage currents are defined as fixed start values
          equation
            connect(signalVoltage.plug_n, star.plug_p) annotation(
              Line(points = {{1.22629e-15, 70}, {1.22629e-15, 90}, {-50, 90}}, color = {0, 0, 255}));
            connect(star.pin_n, ground.p) annotation(
              Line(points = {{-70, 90}, {-80, 90}}, color = {0, 0, 255}));
            connect(smrE.flange, loadInertiaE.flange_a) annotation(
              Line(points = {{10, -80}, {50, -80}}, color = {0, 0, 0}));
            connect(ramp.y, vfController.u) annotation(
              Line(points = {{-59, 60}, {-42, 60}}, color = {0, 0, 255}));
            connect(vfController.y, signalVoltage.v) annotation(
              Line(points = {{-19, 60}, {-7, 60}}, color = {0, 0, 255}));
            connect(loadInertiaE.flange_b, torqueStepE.flange) annotation(
              Line(points = {{70, -80}, {80, -80}}, color = {0, 0, 0}));
            connect(smrE.plug_sn, rotorAngleE.plug_n) annotation(
              Line(points = {{-6, -70}, {-6, -60}, {36, -60}, {36, -70}}, color = {0, 0, 255}));
            connect(smrE.plug_sp, rotorAngleE.plug_p) annotation(
              Line(points = {{6, -70}, {24, -70}}, color = {0, 0, 255}));
            connect(smrE.flange, rotorAngleE.flange) annotation(
              Line(points = {{10, -80}, {20, -80}}, color = {0, 0, 0}));
            connect(terminalBoxE.plug_sp, smrE.plug_sp) annotation(
              Line(points = {{6, -70}, {6, -70}}, color = {0, 0, 255}));
            connect(terminalBoxE.plug_sn, smrE.plug_sn) annotation(
              Line(points = {{-6, -70}, {-6, -70}}, color = {0, 0, 255}));
            connect(terminalBoxE.plugSupply, currentRMSsensorE.plug_n) annotation(
              Line(points = {{6.10623e-16, -68}, {6.10623e-16, -50}, {-60, -50}, {-60, 10}}, color = {0, 0, 255}));
            connect(smrM.flange, loadInertiaM.flange_a) annotation(
              Line(points = {{10, -20}, {10, -20}, {50, -20}}, color = {0, 0, 0}));
            connect(loadInertiaM.flange_b, torqueStepM.flange) annotation(
              Line(points = {{70, -20}, {80, -20}}, color = {0, 0, 0}));
            connect(smrM.plug_sn, rotorAngleM.plug_n) annotation(
              Line(points = {{-6, -10}, {-6, 0}, {36, 0}, {36, -10}}, color = {0, 0, 255}));
            connect(smrM.plug_sp, rotorAngleM.plug_p) annotation(
              Line(points = {{6, -10}, {6, -10}, {24, -10}}, color = {0, 0, 255}));
            connect(smrM.flange, rotorAngleM.flange) annotation(
              Line(points = {{10, -20}, {10, -20}, {20, -20}}, color = {0, 0, 0}));
            connect(terminalBoxM.plug_sp, smrM.plug_sp) annotation(
              Line(points = {{6, -10}, {6, -10}}, color = {0, 0, 255}));
            connect(terminalBoxM.plug_sn, smrM.plug_sn) annotation(
              Line(points = {{-6, -10}, {-6, -10}}, color = {0, 0, 255}));
            connect(currentRMSsensorM.plug_n, terminalBoxM.plugSupply) annotation(
              Line(points = {{-1.33731e-15, 10}, {-1.33731e-15, 5.5}, {-7.26687e-16, 5.5}, {-7.26687e-16, 1}, {6.10623e-16, 1}, {6.10623e-16, -8}}, color = {0, 0, 255}));
            connect(signalVoltage.plug_p, currentRMSsensorM.plug_p) annotation(
              Line(points = {{-2.44753e-15, 50}, {-2.44753e-15, 45}, {-1.1102e-16, 45}, {-1.1102e-16, 40}, {2.33651e-15, 40}, {2.33651e-15, 30}}, color = {0, 0, 255}));
            connect(signalVoltage.plug_p, currentRMSsensorE.plug_p) annotation(
              Line(points = {{-2.44753e-15, 50}, {0, 50}, {0, 30}, {-60, 30}}, color = {0, 0, 255}));
            annotation(
              experiment(StopTime = 1.5, Interval = 0.001, Tolerance = 1e-06),
              Documentation(info = "<HTML>
<h4>Synchronous induction machine with reluctance rotor fed by an ideal inverter</h4>
<p>
An ideal frequency inverter is modeled by using a
<a href=\"modelica://Modelica.Electrical.Machines.Utilities.VfController\">VfController</a>
and a three-phase <a href=\"modelica://Modelica.Electrical.MultiPhase.Sources.SignalVoltage\">SignalVoltage</a>.
Frequency is raised by a ramp, causing the
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.BasicMachines.SynchronousInductionMachines.SM_ReluctanceRotor\">reluctance machine</a> to start,
and accelerating inertias. At time <code>tStep</code> a load step is applied.
</p>

<p>
Simulate for 1.5 seconds and plot (versus time):
</p>

<ul>
<li><code>currentRMSsensorM|E.I</code>: equivalent RMS stator current</li>
<li><code>smrM|E.wMechanical</code>: machine speed</li>
<li><code>smrM|E.tauElectrical</code>: machine torque</li>
<li><code>rotorAngleM|R.rotorDisplacementAngle</code>: rotor displacement angle</li>
</ul>
</HTML>"));
          end SMR_Inverter;

          model SMR_Inverter_MultiPhase "Starting of multi phase synchronous reluctance machine with inverter"
            import Modelica;
            extends Modelica.Icons.Example;
            constant Integer m3 = 3 "Number of stator phases of threephase system";
            parameter Integer m = 5 "Number of stator phases";
            parameter Modelica.SIunits.Voltage VsNominal = 100 "Nominal RMS voltage per phase";
            parameter Modelica.SIunits.Frequency fsNominal = 50 "Nominal frequency";
            parameter Modelica.SIunits.Frequency fKnee = 50 "Knee frequency of V/f curve";
            parameter Modelica.SIunits.Time tRamp = 1 "Frequency ramp";
            parameter Modelica.SIunits.Torque T_Load = 46 "Nominal load torque";
            parameter Modelica.SIunits.Time tStep = 1.2 "Time of load torque step";
            parameter Modelica.SIunits.Inertia J_Load = 0.29 "Load inertia";
            Modelica.Electrical.Machines.Utilities.TerminalBox terminalBoxM(terminalConnection = "Y", m = m) annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0)));
            Modelica.Electrical.Machines.Utilities.TerminalBox terminalBox3(terminalConnection = "Y", m = m3) annotation(
              Placement(transformation(extent = {{-10, -70}, {10, -50}}, rotation = 0)));
            Modelica.Magnetic.FundamentalWave.BasicMachines.SynchronousInductionMachines.SM_ReluctanceRotor smrM(Jr = smrData.Jr, Js = smrData.Js, p = smrData.p, fsNominal = smrData.fsNominal, TsRef = smrData.TsRef, alpha20s(displayUnit = "1/K") = smrData.alpha20s, frictionParameters = smrData.frictionParameters, statorCoreParameters = smrData.statorCoreParameters, strayLoadParameters = smrData.strayLoadParameters, phiMechanical(fixed = true), wMechanical(fixed = true), useDamperCage = smrData.useDamperCage, Lrsigmad = smrData.Lrsigmad, Lrsigmaq = smrData.Lrsigmaq, Rrd = smrData.Rrd, Rrq = smrData.Rrq, TrRef = smrData.TrRef, alpha20r(displayUnit = "1/K") = smrData.alpha20r, Rs = smrData.Rs * m / 3, Lssigma = smrData.Lssigma * m / 3, Lszero = smrData.Lszero * m / 3, Lmd = smrData.Lmd * m / 3, Lmq = smrData.Lmq * m / 3, m = m, TsOperational = 293.15, TrOperational = 293.15, ir(each fixed = true)) annotation(
              Placement(transformation(extent = {{-10, -30}, {10, -10}}, rotation = 0)));
            Modelica.Magnetic.FundamentalWave.BasicMachines.SynchronousInductionMachines.SM_ReluctanceRotor smr3(p = smrData.p, fsNominal = smrData.fsNominal, Rs = smrData.Rs, TsRef = smrData.TsRef, alpha20s(displayUnit = "1/K") = smrData.alpha20s, Lszero = smrData.Lszero, Lssigma = smrData.Lssigma, Jr = smrData.Jr, Js = smrData.Js, frictionParameters = smrData.frictionParameters, phiMechanical(fixed = true), wMechanical(fixed = true), statorCoreParameters = smrData.statorCoreParameters, strayLoadParameters = smrData.strayLoadParameters, Lmd = smrData.Lmd, Lmq = smrData.Lmq, useDamperCage = smrData.useDamperCage, Lrsigmad = smrData.Lrsigmad, Lrsigmaq = smrData.Lrsigmaq, Rrd = smrData.Rrd, Rrq = smrData.Rrq, TrRef = smrData.TrRef, alpha20r(displayUnit = "1/K") = smrData.alpha20r, m = m3, TsOperational = 293.15, TrOperational = 293.15, ir(each fixed = true)) annotation(
              Placement(transformation(extent = {{-10, -90}, {10, -70}}, rotation = 0)));
            Modelica.Electrical.Machines.Sensors.RotorDisplacementAngle rotorAngle3(p = smr3.p) annotation(
              Placement(transformation(origin = {30, -80}, extent = {{10, -10}, {-10, 10}}, rotation = 90)));
            Modelica.Mechanics.Rotational.Components.Inertia loadInertiaM(J = J_Load) annotation(
              Placement(transformation(extent = {{50, -30}, {70, -10}}, rotation = 0)));
            Modelica.Mechanics.Rotational.Components.Inertia loadInertia3(J = J_Load) annotation(
              Placement(transformation(extent = {{50, -90}, {70, -70}}, rotation = 0)));
            Modelica.Mechanics.Rotational.Sources.TorqueStep torqueStepM(startTime = tStep, stepTorque = -T_Load, useSupport = false, offsetTorque = 0) annotation(
              Placement(transformation(extent = {{100, -30}, {80, -10}}, rotation = 0)));
            Modelica.Mechanics.Rotational.Sources.TorqueStep torqueStep3(startTime = tStep, stepTorque = -T_Load, useSupport = false, offsetTorque = 0) annotation(
              Placement(transformation(extent = {{100, -90}, {80, -70}}, rotation = 0)));
            parameter Electrical.Machines.Utilities.ParameterRecords.SM_ReluctanceRotorData smrData annotation(
              Placement(transformation(extent = {{-100, -100}, {-80, -80}})));
            Modelica.Electrical.Analog.Basic.Ground ground3 annotation(
              Placement(transformation(origin = {-90, 90}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
            Modelica.Electrical.MultiPhase.Basic.Star star3(final m = m3) annotation(
              Placement(transformation(extent = {{-50, 80}, {-70, 100}}, rotation = 0)));
            Modelica.Electrical.MultiPhase.Sources.SignalVoltage signalVoltage3(final m = m3) annotation(
              Placement(transformation(origin = {0, 70}, extent = {{10, 10}, {-10, -10}}, rotation = 270)));
            Modelica.Blocks.Sources.Ramp ramp(height = fKnee, duration = tRamp) annotation(
              Placement(transformation(extent = {{-80, 60}, {-60, 80}}, rotation = 0)));
            Modelica.Electrical.Machines.Utilities.VfController vfController3(VNominal = VsNominal, fNominal = fsNominal, BasePhase = +Modelica.Constants.pi / 2, final m = m3, orientation = -Modelica.Electrical.MultiPhase.Functions.symmetricOrientation(m3)) annotation(
              Placement(transformation(extent = {{-40, 60}, {-20, 80}}, rotation = 0)));
            Modelica.Electrical.MultiPhase.Sensors.CurrentQuasiRMSSensor currentRMSsensorM(m = m) annotation(
              Placement(transformation(origin = {30, 20}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
            Modelica.Electrical.Machines.Sensors.CurrentQuasiRMSSensor currentRMSsensor3 annotation(
              Placement(transformation(origin = {-80, 0}, extent = {{-10, 10}, {10, -10}}, rotation = 270)));
            Modelica.Electrical.MultiPhase.Sources.SignalVoltage signalVoltageM(final m = m) annotation(
              Placement(transformation(origin = {30, 50}, extent = {{10, 10}, {-10, -10}}, rotation = 270)));
            Modelica.Electrical.MultiPhase.Basic.Star starM(final m = m) annotation(
              Placement(transformation(extent = {{10, -10}, {-10, 10}}, rotation = 180, origin = {60, 90})));
            Modelica.Electrical.Analog.Basic.Ground groundM annotation(
              Placement(transformation(origin = {90, 90}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
            Modelica.Electrical.Machines.Utilities.VfController vfControllerM(VNominal = VsNominal, fNominal = fsNominal, BasePhase = +Modelica.Constants.pi / 2, final m = m, orientation = -Modelica.Electrical.MultiPhase.Functions.symmetricOrientation(m)) annotation(
              Placement(transformation(extent = {{10, 10}, {-10, -10}}, rotation = 180, origin = {-30, 50})));
            Modelica.Blocks.Math.Feedback feedback annotation(
              Placement(transformation(extent = {{-10, 10}, {10, -10}}, rotation = 0, origin = {-50, 0})));
            Modelica.Blocks.Math.Gain gain(k = m / m3) annotation(
              Placement(transformation(extent = {{-20, 10}, {-40, 30}})));
          initial equation
            smr3.is[1:2] = zeros(2);
            smrM.is[1:2] = zeros(2);
//conditional damper cage currents are defined as fixed start values
          equation
            connect(smr3.flange, loadInertia3.flange_a) annotation(
              Line(points = {{10, -80}, {50, -80}}, color = {0, 0, 0}));
            connect(loadInertia3.flange_b, torqueStep3.flange) annotation(
              Line(points = {{70, -80}, {80, -80}}, color = {0, 0, 0}));
            connect(smr3.plug_sn, rotorAngle3.plug_n) annotation(
              Line(points = {{-6, -70}, {-6, -60}, {36, -60}, {36, -70}}, color = {0, 0, 255}));
            connect(smr3.plug_sp, rotorAngle3.plug_p) annotation(
              Line(points = {{6, -70}, {24, -70}}, color = {0, 0, 255}));
            connect(smr3.flange, rotorAngle3.flange) annotation(
              Line(points = {{10, -80}, {20, -80}}, color = {0, 0, 0}));
            connect(terminalBox3.plug_sp, smr3.plug_sp) annotation(
              Line(points = {{6, -70}, {6, -70}}, color = {0, 0, 255}));
            connect(terminalBox3.plug_sn, smr3.plug_sn) annotation(
              Line(points = {{-6, -70}, {-6, -70}}, color = {0, 0, 255}));
            connect(smrM.flange, loadInertiaM.flange_a) annotation(
              Line(points = {{10, -20}, {10, -20}, {50, -20}}, color = {0, 0, 0}));
            connect(loadInertiaM.flange_b, torqueStepM.flange) annotation(
              Line(points = {{70, -20}, {80, -20}}, color = {0, 0, 0}));
            connect(terminalBoxM.plug_sp, smrM.plug_sp) annotation(
              Line(points = {{6, -10}, {6, -10}}, color = {0, 0, 255}));
            connect(terminalBoxM.plug_sn, smrM.plug_sn) annotation(
              Line(points = {{-6, -10}, {-6, -10}}, color = {0, 0, 255}));
            connect(signalVoltage3.plug_n, star3.plug_p) annotation(
              Line(points = {{0, 80}, {0, 90}, {-50, 90}}, color = {0, 0, 255}));
            connect(star3.pin_n, ground3.p) annotation(
              Line(points = {{-70, 90}, {-80, 90}}, color = {0, 0, 255}));
            connect(ramp.y, vfController3.u) annotation(
              Line(points = {{-59, 70}, {-42, 70}}, color = {0, 0, 255}));
            connect(vfController3.y, signalVoltage3.v) annotation(
              Line(points = {{-19, 70}, {-7, 70}}, color = {0, 0, 255}));
            connect(terminalBoxM.plugSupply, currentRMSsensorM.plug_n) annotation(
              Line(points = {{4.44089e-16, -8}, {4.44089e-16, -1}, {0, -1}, {0, 10}, {30, 10}}, color = {0, 0, 255}));
            connect(signalVoltage3.plug_p, currentRMSsensor3.plug_p) annotation(
              Line(points = {{0, 60}, {0, 34}, {-80, 34}, {-80, 10}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(signalVoltageM.plug_n, starM.plug_p) annotation(
              Line(points = {{30, 60}, {30, 90}, {50, 90}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(starM.pin_n, groundM.p) annotation(
              Line(points = {{70, 90}, {80, 90}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(vfControllerM.y, signalVoltageM.v) annotation(
              Line(points = {{-19, 50}, {23, 50}}, color = {0, 0, 127}, smooth = Smooth.None));
            connect(vfControllerM.u, ramp.y) annotation(
              Line(points = {{-42, 50}, {-50, 50}, {-50, 70}, {-59, 70}}, color = {0, 0, 127}, smooth = Smooth.None));
            connect(signalVoltageM.plug_p, currentRMSsensorM.plug_p) annotation(
              Line(points = {{30, 40}, {30, 30}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(gain.y, feedback.u2) annotation(
              Line(points = {{-41, 20}, {-50, 20}, {-50, 8}}, color = {0, 0, 127}, smooth = Smooth.None));
            connect(gain.u, currentRMSsensorM.I) annotation(
              Line(points = {{-18, 20}, {20, 20}}, color = {0, 0, 127}, smooth = Smooth.None));
            connect(currentRMSsensor3.plug_n, terminalBox3.plugSupply) annotation(
              Line(points = {{-80, -10}, {-80, -60}, {4.44089e-16, -60}, {4.44089e-16, -68}}, color = {0, 0, 255}));
            connect(currentRMSsensor3.I, feedback.u1) annotation(
              Line(points = {{-69, 0}, {-58, 0}}, color = {0, 0, 127}, smooth = Smooth.None));
            annotation(
              experiment(StopTime = 1.5, Interval = 0.001, Tolerance = 1e-06),
              Documentation(info = "<HTML>
<h4>Synchronous induction machine with reluctance rotor fed by an ideal inverter</h4>
<p>
Ideal frequency inverters are modeled by using a
<a href=\"modelica://Modelica.Electrical.Machines.Utilities.VfController\">VfController</a>
and phase <a href=\"modelica://Modelica.Electrical.MultiPhase.Sources.SignalVoltage\">SignalVoltage</a>s.
Frequency is raised by a ramp, causing the
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.BasicMachines.SynchronousInductionMachines.SM_ReluctanceRotor\">reluctance machine</a> to start,
and accelerating inertias. At time <code>tStep</code> a load step is applied. Two equivalent machines with different numbers of phases are compared and their equal behavior is demonstrated.
</p>

<p>
Simulate for 1.5 seconds and plot (versus time):
</p>

<ul>
<li><code>aimcM|M3.tauElectrical</code>: machine torque</li>
<li><code>aimsM|M3.wMechanical</code>: machine speed</li>
<li><code>feedback.y</code>: zero since difference of three phase current phasor and scaled multi phase current phasor are equal</li>
</ul>

</HTML>"),
              Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-20, 0}, {100, -40}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 170}, fillPattern = FillPattern.Solid, pattern = LinePattern.Dash), Text(extent = {{40, -44}, {100, -52}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 170}, fillPattern = FillPattern.Solid, textStyle = {TextStyle.Bold}, textString = "%m phase machine
              "), Text(extent = {{40, -54}, {100, -62}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 170}, fillPattern = FillPattern.Solid, textStyle = {TextStyle.Bold}, textString = "3 phase machine
              "), Rectangle(extent = {{-20, -60}, {100, -100}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 170}, fillPattern = FillPattern.Solid, pattern = LinePattern.Dash)}));
          end SMR_Inverter_MultiPhase;
        end BasicMachines;
        annotation(
          Documentation(info = "<html>
<p>Examples comparing the models of
<a href=\"modelica://Modelica.Electrical.Machines.BasicMachines\">Electrical.Machines.BasicMachines</a> with
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.BasicMachines\">Magnetic.FundamentalWave.BasicMachines</a>.
</p>
</html>"));
      end Examples;

      package Components "Basic fundamental wave components"
        extends Modelica.Icons.Package;

        model Ground "Magnetic ground"
          Interfaces.PositiveMagneticPort port_p "Complex magnetic port" annotation(
            Placement(transformation(extent = {{-10, 90}, {10, 110}}, rotation = 0)));
        equation
          port_p.V_m = Complex(0, 0);
          annotation(
            Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{0, 100}, {0, 50}}, color = {255, 128, 0}), Line(points = {{-60, 50}, {60, 50}}, color = {255, 128, 0}), Line(points = {{-40, 30}, {40, 30}}, color = {255, 128, 0}), Line(points = {{-20, 10}, {20, 10}}, color = {255, 128, 0})}),
            Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{0, 100}, {0, 50}}, color = {255, 128, 0}), Line(points = {{-60, 50}, {60, 50}}, color = {255, 128, 0}), Line(points = {{-40, 30}, {40, 30}}, color = {255, 128, 0}), Line(points = {{-20, 10}, {20, 10}}, color = {255, 128, 0}), Text(extent = {{-144, -9}, {156, -49}}, textString = "%name", lineColor = {0, 0, 255})}),
            Documentation(info = "<html>

<p>
Grounding of the complex magnetic potential. Each magnetic circuit has to be grounded at least one point of the circuit.
</p>

</html>"));
        end Ground;

        model Reluctance "Salient reluctance"
          import Modelica.Constants.pi;
          extends Modelica.Magnetic.FundamentalWave.Interfaces.PartialTwoPortElementary;
          parameter Modelica.Magnetic.FundamentalWave.Types.SalientReluctance R_m(d(start = 1), q(start = 1)) "Magnetic reluctance in d=re and q=im axis";
        equation
          pi / 2 * V_m.re = R_m.d * Phi.re;
          pi / 2 * V_m.im = R_m.q * Phi.im;
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-70, 30}, {70, -30}}, lineColor = {255, 128, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{-96, 0}, {-70, 0}}, color = {255, 128, 0}), Line(points = {{70, 0}, {96, 0}}, color = {255, 128, 0}), Text(extent = {{0, 60}, {0, 100}}, lineColor = {0, 0, 255}, textString = "%name"), Text(extent = {{0, -70}, {0, -110}}, lineColor = {0, 0, 0}, textString = "R_m.d=%R_m.d, R_m.q=%R_m.q")}),
            Documentation(info = "<html>
<p>
The salient reluctance models the relationship between the complex magnetic potential difference
<img src=\"modelica://Modelica/Resources/Images/Magnetic/FundamentalWave/V_m.png\" alt=\"V_m.png\"> and the complex magnetic flux <img src=\"modelica://Modelica/Resources/Images/Magnetic/FundamentalWave/Phi.png\">,
</p>

<blockquote>
<img src=\"modelica://Modelica/Resources/Images/Magnetic/FundamentalWave/Components/reluctance.png\"
      alt=\"reluctance.png\">
</blockquote>

<p>which can also be expressed in terms complex phasors:</p>

<blockquote>
<img src=\"modelica://Modelica/Resources/Images/Magnetic/FundamentalWave/Components/reluctance_alt.png\"
     alt=\"reluctance_alt.png\">
</blockquote>

</html>"));
        end Reluctance;

        model EddyCurrent "Constant loss model under sinusoidal magnetic conditions"
          import Modelica.Constants.pi;
          extends Modelica.Magnetic.FundamentalWave.Interfaces.PartialTwoPortElementary;
          parameter Modelica.SIunits.Conductance G(min = 0) "Equivalent symmetric loss conductance";
          extends Modelica.Thermal.HeatTransfer.Interfaces.PartialElementaryConditionalHeatPort(final T = 273.15);
        equation
          lossPower = pi / 2 * (V_m.re * der(Phi.re) + V_m.im * der(Phi.im));
          if G > 0 then
            pi / 2 * V_m.re = G * der(Phi.re);
            pi / 2 * V_m.im = G * der(Phi.im);
          else
            V_m.re = 0;
            V_m.im = 0;
          end if;
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-70, 30}, {70, -30}}, lineColor = {255, 128, 0}, fillColor = {255, 128, 0}, fillPattern = FillPattern.Solid), Line(points = {{-96, 0}, {-70, 0}}, color = {255, 128, 0}), Line(points = {{70, 0}, {96, 0}}, color = {255, 128, 0}), Text(extent = {{0, 60}, {0, 100}}, lineColor = {0, 0, 255}, textString = "%name"), Text(extent = {{0, -40}, {0, -80}}, lineColor = {0, 0, 0}, textString = "G=%G")}),
            Documentation(info = "<html>
<p>
The eddy current loss model with respect to fundamental wave effects is designed in accordance to
<a href=\"modelica://Modelica.Magnetic.FluxTubes.Basic.EddyCurrent\">FluxTubes.Basic.EddyCurrent</a>.
</p>

<blockquote>
<img src=\"modelica://Modelica/Resources/Images/Magnetic/FundamentalWave/Components/eddycurrent.png\"
     alt=\"eddycurrent.png\">
</blockquote>

<table border=\"0\" cellspacing=\"0\" cellpadding=\"2\">
  <caption align=\"bottom\">Fig. 1: equivalent models of eddy current losses</caption>
  <tr>
    <td>
      <img src=\"modelica://Modelica/Resources/Images/Magnetic/FundamentalWave/Components/eddycurrent_electric.png\"
           alt=\"eddycurrent_electric.png\">
    </td>
  </tr>
</table>

<p>Due to the nature of eddy current losses, which can be represented by symmetric
conductors in an equivalent electric circuit (Fig. 1), the respective
number of phases <img src=\"modelica://Modelica/Resources/Images/Magnetic/FundamentalWave/m.png\" alt=\"m\"> has to be taken into account.
Assume that the <img src=\"modelica://Modelica/Resources/Images/Magnetic/FundamentalWave/m.png\" alt=\"m\"> conductances
of the equivalent circuit are <img src=\"modelica://Modelica/Resources/Images/Magnetic/FundamentalWave/Components/Gc.png\" alt=\"G_c\">,
the conductance for the eddy current loss model is determined by</p>

<blockquote>
<img src=\"modelica://Modelica/Resources/Images/Magnetic/FundamentalWave/Components/GGc.png\"
     alt=\"GGc\">
</blockquote>

<p>
where <img src=\"modelica://Modelica/Resources/Images/Magnetic/FundamentalWave/N.png\" alt=\"N\"> is the number of turns of the symmetric electro magnetic coupling.
</p>

<p>For such an <img src=\"modelica://Modelica/Resources/Images/Magnetic/FundamentalWave/m.png\" alt=\"m\"> phase system
the relationship between the voltage and current <a href=\"http://www.haumer.at/refimg/SpacePhasors.pdf\">space phasors</a>
and the magnetic flux and magnetic potential difference phasor is
</p>

<blockquote>
<img src=\"modelica://Modelica/Resources/Images/Magnetic/FundamentalWave/Components/vPhi.png\" alt=\"vPhi\">,<br>
<img src=\"modelica://Modelica/Resources/Images/Magnetic/FundamentalWave/Components/iV_m.png\" alt=\"iV_m\">,
</blockquote>

<p>
where <img src=\"modelica://Modelica/Resources/Images/Magnetic/FundamentalWave/v_k.png\" alt=\"v_k\">
and <img src=\"modelica://Modelica/Resources/Images/Magnetic/FundamentalWave/i_k.png\" alt=\"i_k\">
are the phase voltages and currents, respectively.
</p>

<p>
The dissipated loss power
</p>
<blockquote>
<img src=\"modelica://Modelica/Resources/Images/Magnetic/FundamentalWave/Components/lossPower.png\" alt=\"lossPower\">
</blockquote>
<p>
can be determined for the <a href=\"http://www.haumer.at/refimg/SpacePhasors.pdf\">space phasor</a>
relationship of the voltage and current space phasor.
</p>
<h4>See also</h4>

<p><a href=\"modelica://Modelica.Magnetic.FluxTubes.Basic.EddyCurrent\">FluxTubes.Basic.EddyCurrent</a></p>

</html>"));
        end EddyCurrent;

        model SinglePhaseElectroMagneticConverter "Single phase electro magnetic converter"
          import Modelica.Constants.pi;
          Modelica.Electrical.Analog.Interfaces.PositivePin pin_p "Positive pin" annotation(
            Placement(transformation(origin = {-100, 100}, extent = {{-10, -10}, {10, 10}}, rotation = 180)));
          Modelica.Electrical.Analog.Interfaces.NegativePin pin_n "Negative pin" annotation(
            Placement(transformation(origin = {-100, -100}, extent = {{-10, -10}, {10, 10}}, rotation = 180)));
          Interfaces.PositiveMagneticPort port_p "Positive complex magnetic port" annotation(
            Placement(transformation(extent = {{90, 90}, {110, 110}}, rotation = 0)));
          Interfaces.NegativeMagneticPort port_n "Negative complex magnetic port" annotation(
            Placement(transformation(extent = {{90, -110}, {110, -90}}, rotation = 0)));
          parameter Real effectiveTurns "Effective number of turns" annotation(
            Evaluate = true);
          parameter Modelica.SIunits.Angle orientation "Orientation of the resulting fundamental wave V_m phasor" annotation(
            Evaluate = true);
          // Local electric single phase quantities
          Modelica.SIunits.Voltage v "Voltage drop";
          Modelica.SIunits.Current i "Current";
          // Local electromagnetic fundamental wave quantities
          Modelica.SIunits.ComplexMagneticPotentialDifference V_m "Complex magnetic potential difference";
          Modelica.SIunits.ComplexMagneticFlux Phi "Complex magnetic flux";
          final parameter Complex N = effectiveTurns * Modelica.ComplexMath.exp(Complex(0, orientation)) "Complex number of turns";
        equation
// Magnetic flux and flux balance of the magnetic ports
          port_p.Phi = Phi;
          port_p.Phi + port_n.Phi = Complex(0, 0);
// Magnetic potential difference of the magnetic ports
          port_p.V_m - port_n.V_m = V_m;
// Voltage drop between the electrical pins
          v = pin_p.v - pin_n.v;
// Current and current balance of the electric pins
          i = pin_p.i;
          pin_p.i + pin_n.i = 0;
// Complex magnetic potential difference is determined from currents, number
// of turns and angles of orientation of winding
// V_m.re = (2/pi) * effectiveTurns*cos(orientation)*i;
// V_m.im = (2/pi) * effectiveTurns*sin(orientation)*i;
          V_m = 2.0 / pi * N * i;
// Induced voltages is determined from complex magnetic flux, number of turns
// and angles of orientation of winding
// -v = effectiveTurns*cos(orientation)*der(Phi.re)
//    + effectiveTurns*sin(orientation)*der(Phi.im);
          -v = Modelica.ComplexMath.real(Modelica.ComplexMath.conj(N) * Complex(der(Phi.re), der(Phi.im)));
          annotation(
            defaultComponentName = "converter",
            Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-60, 60}, {58, 0}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-58, 0}, {60, -60}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-60, 60}, {0, -60}}, lineColor = {0, 0, 255}, pattern = LinePattern.None, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{100, -100}, {94, -100}, {84, -98}, {76, -94}, {64, -86}, {50, -72}, {42, -58}, {36, -40}, {30, -18}, {30, 0}, {30, 18}, {34, 36}, {46, 66}, {62, 84}, {78, 96}, {90, 100}, {100, 100}}, color = {255, 128, 0}), Line(points = {{0, 60}, {-100, 60}, {-100, 100}}, color = {0, 0, 255}), Line(points = {{0, -60}, {-100, -60}, {-100, -98}}, color = {0, 0, 255}), Text(extent = {{0, 160}, {0, 120}}, lineColor = {0, 0, 255}, fillColor = {255, 128, 0}, fillPattern = FillPattern.Solid, textString = "%name")}),
            Documentation(info = "<html>
<p>
The single phase winding has an effective number of turns, <img src=\"modelica://Modelica/Resources/Images/Magnetic/FundamentalWave/effectiveTurns.png\"> and a respective orientation of the winding, <img src=\"modelica://Modelica/Resources/Images/Magnetic/FundamentalWave/orientation.png\">. The current in winding is <img src=\"modelica://Modelica/Resources/Images/Magnetic/FundamentalWave/i.png\">.
</p>

<p>
The total complex magnetic potential difference of the single phase winding is determined by:
</p>

<p>
&nbsp;&nbsp;<img src=\"modelica://Modelica/Resources/Images/Magnetic/FundamentalWave/Components/singlephaseconverter_vm.png\">
</p>

<p>
In this equation the magneto motive force is aligned with the orientation of the winding.
</p>

<p>
The voltage <img src=\"modelica://Modelica/Resources/Images/Magnetic/FundamentalWave/v.png\"> induced in the winding depends on the cosine between the orientation of the winding and the angle of the complex magnetic flux. Additionally, the magnitudes of the induced voltages are proportional to the respective number of turns. This relationship can be modeled by means of</p>

<p>
&nbsp;&nbsp;<img src=\"modelica://Modelica/Resources/Images/Magnetic/FundamentalWave/Components/singlephaseconverter_phi.png\">
</p>

<p>The single phase electromagnetic converter is a special case of the
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.Components.MultiPhaseElectroMagneticConverter\">MultiPhaseElectroMagneticConverter</a>
</p>

<h4>See also</h4>
<p>
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.Components.MultiPhaseElectroMagneticConverter\">MultiPhaseElectroMagneticConverter</a>
</p>

</html>"));
        end SinglePhaseElectroMagneticConverter;

        model MultiPhaseElectroMagneticConverter "Multi phase electro magnetic converter"
          // Global plug and port variables
          Modelica.SIunits.Voltage v[m] = plug_p.pin.v - plug_n.pin.v "Voltages";
          Modelica.SIunits.Current i[m] = plug_p.pin.i "Currents";
          Modelica.SIunits.ComplexMagneticPotentialDifference V_m = port_p.V_m - port_n.V_m "Magnetic potential difference";
          Modelica.SIunits.ComplexMagneticFlux Phi = port_p.Phi "Magnetic flux";
          import Modelica.Constants.pi;
          Modelica.Electrical.MultiPhase.Interfaces.PositivePlug plug_p(final m = m) "Positive plug" annotation(
            Placement(transformation(origin = {-100, 100}, extent = {{-10, -10}, {10, 10}}, rotation = 180)));
          Modelica.Electrical.MultiPhase.Interfaces.NegativePlug plug_n(final m = m) "Negative plug" annotation(
            Placement(transformation(origin = {-100, -100}, extent = {{-10, -10}, {10, 10}}, rotation = 180)));
          Modelica.Magnetic.FundamentalWave.Interfaces.PositiveMagneticPort port_p "Positive complex magnetic port" annotation(
            Placement(transformation(extent = {{90, 90}, {110, 110}}, rotation = 0)));
          Modelica.Magnetic.FundamentalWave.Interfaces.NegativeMagneticPort port_n "Negative complex magnetic port" annotation(
            Placement(transformation(extent = {{90, -110}, {110, -90}}, rotation = 0)));
          parameter Integer m = 3 "Number of phases";
          parameter Real effectiveTurns[m] "Effective number of turns";
          parameter Modelica.SIunits.Angle orientation[m] "Orientation of the resulting fundamental wave field phasor";
          Modelica.Magnetic.FundamentalWave.Components.SinglePhaseElectroMagneticConverter singlePhaseElectroMagneticConverter[m](final effectiveTurns = effectiveTurns, final orientation = orientation) annotation(
            Placement(transformation(extent = {{-8, -10}, {12, 10}})));
        equation
          connect(plug_p.pin, singlePhaseElectroMagneticConverter.pin_p) annotation(
            Line(points = {{-100, 100}, {-8, 100}, {-8, 10}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(singlePhaseElectroMagneticConverter.pin_n, plug_n.pin) annotation(
            Line(points = {{-8, -10}, {-8, -100}, {-100, -100}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(singlePhaseElectroMagneticConverter[1].port_p, port_p) annotation(
            Line(points = {{12, 10}, {12, 100}, {100, 100}}, color = {255, 128, 0}, smooth = Smooth.None));
          for k in 2:m loop
            connect(singlePhaseElectroMagneticConverter[k - 1].port_n, singlePhaseElectroMagneticConverter[k].port_p);
          end for;
          connect(singlePhaseElectroMagneticConverter[m].port_n, port_n) annotation(
            Line(points = {{12, -10}, {12, -100}, {100, -100}}, color = {255, 128, 0}, smooth = Smooth.None));
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-60, 60}, {58, 0}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-58, 0}, {60, -60}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-60, 60}, {0, -60}}, lineColor = {0, 0, 255}, pattern = LinePattern.None, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{100, -100}, {94, -100}, {84, -98}, {76, -94}, {64, -86}, {50, -72}, {42, -58}, {36, -40}, {30, -18}, {30, 0}, {30, 18}, {34, 36}, {46, 66}, {62, 84}, {78, 96}, {90, 100}, {100, 100}}, color = {255, 128, 0}), Line(points = {{0, 60}, {-100, 60}, {-100, 100}}, color = {0, 0, 255}), Line(points = {{0, -60}, {-100, -60}, {-100, -98}}, color = {0, 0, 255}), Text(extent = {{0, 160}, {0, 120}}, lineColor = {0, 0, 255}, fillColor = {255, 128, 0}, fillPattern = FillPattern.Solid, textString = "%name")}),
            Documentation(info = "<html>
<p>
Each phase <img src=\"modelica://Modelica/Resources/Images/Magnetic/FundamentalWave/k.png\"> of an <img src=\"modelica://Modelica/Resources/Images/Magnetic/FundamentalWave/m.png\"> phase winding has an effective number of turns, <img src=\"modelica://Modelica/Resources/Images/Magnetic/FundamentalWave/effectiveTurns_k.png\"> and an respective winging angle <img src=\"modelica://Modelica/Resources/Images/Magnetic/FundamentalWave/orientation_k.png\"> and a phase current <img src=\"modelica://Modelica/Resources/Images/Magnetic/FundamentalWave/i_k.png\">.
</p>

<p>
The total complex magnetic potential difference of the multi phase winding is determined by:
</p>

<p>
&nbsp;&nbsp;<img src=\"modelica://Modelica/Resources/Images/Magnetic/FundamentalWave/Components/multiphaseconverter_vm.png\">
</p>

<p>
In this equation each contribution of a winding magneto motive force on the total complex magnetic potential difference is aligned with the respective orientation of the winding.
</p>

<p>
The voltages <img src=\"modelica://Modelica/Resources/Images/Magnetic/FundamentalWave/v_k.png\"> induced in each winding depend on the cosines between the orientation of the winding and the angle of the complex magnetic flux. Additionally, the magnitudes of the induced voltages are proportional to the respective number of turns. This relationship can be modeled by means of</p>

<p>
&nbsp;&nbsp;<img src=\"modelica://Modelica/Resources/Images/Magnetic/FundamentalWave/Components/multiphaseconverter_phi.png\">
</p>

<p>for <img src=\"modelica://Modelica/Resources/Images/Magnetic/FundamentalWave/k_in_1_m.png\"> and is also illustrated by the following figure:</p>

<table border=\"0\" cellspacing=\"0\" cellpadding=\"2\">
  <caption align=\"bottom\"><b>Fig:</b> Orientation of winding and location of complex magnetic flux</caption>
  <tr>
    <td>
      <img src=\"modelica://Modelica/Resources/Images/Magnetic/FundamentalWave/Components/coupling.png\">
    </td>
  </tr>
</table>

<h4>See also</h4>
<p>
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.Components.SinglePhaseElectroMagneticConverter\">SinglePhaseElectroMagneticConverter</a>
</p>
</html>"),
            defaultComponentName = "converter");
        end MultiPhaseElectroMagneticConverter;

        model Idle "Salient reluctance"
          extends Modelica.Magnetic.FundamentalWave.Interfaces.PartialTwoPortElementary;
        equation
          Phi = Complex(0, 0);
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{0, 60}, {0, 100}}, lineColor = {0, 0, 255}, textString = "%name"), Rectangle(extent = {{-100, 40}, {100, -40}}, lineColor = {255, 255, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{-100, 0}, {-40, 0}}, color = {255, 128, 0}), Line(points = {{40, 0}, {100, 0}}, color = {255, 128, 0})}),
            Documentation(info = "<html>
<p>
This is a simple idle running branch.
</p>

<h4>See also</h4>
<p>
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.Components.Short\">Short</a>
</p>

</html>"),
            Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-100, 0}, {-60, 0}}, color = {255, 128, 0}), Line(points = {{60, 0}, {100, 0}}, color = {255, 128, 0}), Line(points = {{-60, 0}, {-40, 2}, {-18, 6}, {0, 14}, {12, 26}}, color = {255, 128, 0}), Line(points = {{60, 0}, {40, -2}, {18, -6}, {0, -14}, {-12, -26}}, color = {255, 128, 0})}));
        end Idle;

        model Short "Salient reluctance"
          extends Modelica.Magnetic.FundamentalWave.Interfaces.PartialTwoPort;
        equation
          connect(port_p, port_n) annotation(
            Line(points = {{-100, 0}, {-1, 0}, {-1, 0}, {100, 0}}, color = {255, 128, 0}));
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{0, 60}, {0, 100}}, lineColor = {0, 0, 255}, textString = "%name"), Rectangle(extent = {{-100, 40}, {100, -40}}, lineColor = {255, 255, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{-100, 0}, {100, 0}}, color = {255, 128, 0})}),
            Documentation(info = "<html>
<p>
This is a simple short cut branch.
</p>

<h4>See also</h4>
<p>
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.Components.Idle\">Idle</a>
</p>

</html>"));
        end Short;
        annotation(
          Documentation(info = "<html>
<p>Basic components of the FundamentalWave library for modeling magnetic circuits. Machine specific components are
located at <a href=\"modelica://Modelica.Magnetic.FundamentalWave.BasicMachines.Components\">Machines.Components</a>.</p>
</html>"));
      end Components;

      package BasicMachines "Machine components and modelsElectric machine models based on FundamentalWave package"
        extends Modelica.Icons.Package;

        package AsynchronousInductionMachines "Asynchronous induction machines"
          extends Modelica.Icons.VariantsPackage;

          model AIM_SquirrelCage "Asynchronous induction machine with squirrel cage"
            extends Modelica.Magnetic.FundamentalWave.Interfaces.PartialBasicInductionMachine(is(start = zeros(m)), Rs(start = 0.03), Lssigma(start = 3 * (1 - sqrt(1 - 0.0667)) / (2 * pi * fsNominal)), final L0(d = 2.0 * Lm / m / effectiveStatorTurns ^ 2, q = 2.0 * Lm / m / effectiveStatorTurns ^ 2), redeclare final Modelica.Electrical.Machines.Thermal.AsynchronousInductionMachines.ThermalAmbientAIMC thermalAmbient(final Tr = TrOperational), redeclare final Modelica.Electrical.Machines.Interfaces.InductionMachines.ThermalPortAIMC thermalPort, redeclare final Modelica.Electrical.Machines.Interfaces.InductionMachines.ThermalPortAIMC internalThermalPort, redeclare final Modelica.Electrical.Machines.Interfaces.InductionMachines.PowerBalanceAIMC powerBalance(final lossPowerRotorWinding = sum(rotorCage.resistor.resistor.LossPower), final lossPowerRotorCore = 0));
            parameter Modelica.SIunits.Inductance Lm(start = 3 * sqrt(1 - 0.0667) / (2 * pi * fsNominal)) "Stator main field inductance per phase" annotation(
              Dialog(tab = "Nominal resistances and inductances", groupImage = "modelica://Modelica/Resources/Images/Electrical/Machines/IMC.png"));
            parameter Modelica.SIunits.Inductance Lrsigma(start = 3 * (1 - sqrt(1 - 0.0667)) / (2 * pi * fsNominal)) "Rotor leakage inductance of equivalent m phase winding w.r.t. stator side" annotation(
              Dialog(tab = "Nominal resistances and inductances"));
            parameter Modelica.SIunits.Resistance Rr(start = 0.04) "Rotor resistance of equivalent m phase winding w.r.t. stator side" annotation(
              Dialog(tab = "Nominal resistances and inductances"));
            parameter Modelica.SIunits.Temperature TrRef(start = 293.15) "Reference temperature of rotor resistance" annotation(
              Dialog(tab = "Nominal resistances and inductances"));
            parameter Modelica.Electrical.Machines.Thermal.LinearTemperatureCoefficient20 alpha20r(start = 0) "Temperature coefficient of rotor resistance at 20 degC" annotation(
              Dialog(tab = "Nominal resistances and inductances"));
            parameter Modelica.SIunits.Temperature TrOperational(start = 293.15) "Operational temperature of rotor resistance" annotation(
              Dialog(group = "Operational temperatures", enable = not useThermalPort));
            output Modelica.SIunits.Current ir[m] = rotorCage.winding.plug_p.pin.i "Rotor cage currents";
            Modelica.Magnetic.FundamentalWave.BasicMachines.Components.SymmetricMultiPhaseCageWinding rotorCage(final Lsigma = Lrsigma, final effectiveTurns = effectiveStatorTurns, final useHeatPort = true, final RRef = Rr, final TRef = TrRef, final TOperational = TrRef, final m = m, final alpha20 = alpha20r) "Symmetric rotor cage winding including resistances and stray inductances" annotation(
              Placement(transformation(extent = {{-10, -40}, {10, -20}}, rotation = 0)));
          equation
            connect(rotorCage.heatPortWinding, internalThermalPort.heatPortRotorWinding) annotation(
              Line(points = {{-6.10623e-16, -40}, {-40, -40}, {-40, -90}}, color = {191, 0, 0}, smooth = Smooth.None));
            connect(airGap.port_rn, rotorCage.port_p) annotation(
              Line(points = {{-10, -10}, {-10, -30}}, color = {255, 128, 0}, smooth = Smooth.None));
            connect(airGap.port_rp, rotorCage.port_n) annotation(
              Line(points = {{10, -10}, {10, -30}}, color = {255, 128, 0}, smooth = Smooth.None));
            annotation(
              defaultComponentName = "aimc",
              Documentation(info = "<html>
<p>
Resistances and stray inductances of the machine refer to an <code>m</code> phase stator. The symmetry of the stator and rotor is assumed. The machine models take the following loss effects into account:
</p>

<ul>
<li>heat losses in the temperature dependent stator winding resistances</li>
<li>heat losses in the temperature dependent cage resistances</li>
<li>friction losses</li>
<li>core losses (only eddy current losses, no hysteresis losses)</li>
<li>stray load losses</li>
</ul>

<h4>See also</h4>
<p>
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.BasicMachines.AsynchronousInductionMachines.AIM_SlipRing\">AIM_SlipRing</a>,
</p>
</html>"));
          end AIM_SquirrelCage;

          model AIM_SlipRing "Asynchronous induction machine with slip ring rotor"
            parameter Integer mr(min = 3) = m "Number of rotor phases";
            extends Modelica.Magnetic.FundamentalWave.Interfaces.PartialBasicInductionMachine(is(start = zeros(m)), Rs(start = 0.03), Lssigma(start = 3 * (1 - sqrt(1 - 0.0667)) / (2 * pi * fsNominal)), final L0(d = 2.0 * Lm / m / effectiveStatorTurns ^ 2, q = 2.0 * Lm / m / effectiveStatorTurns ^ 2), redeclare final Electrical.Machines.Thermal.AsynchronousInductionMachines.ThermalAmbientAIMS thermalAmbient(final Tr = TrOperational, final mr = mr), redeclare final Electrical.Machines.Interfaces.InductionMachines.ThermalPortAIMS thermalPort(final mr = mr), redeclare final Electrical.Machines.Interfaces.InductionMachines.ThermalPortAIMS internalThermalPort(final mr = mr), redeclare final Modelica.Electrical.Machines.Interfaces.InductionMachines.PowerBalanceAIMS powerBalance(final lossPowerRotorWinding = sum(rotor.resistor.resistor.LossPower), final lossPowerRotorCore = rotor.core.lossPower, final lossPowerBrush = 0, final powerRotor = Modelica.Electrical.MultiPhase.Functions.activePower(vr, ir)));
            Modelica.Electrical.MultiPhase.Interfaces.NegativePlug plug_rn(final m = mr) "Negative plug of rotor" annotation(
              Placement(transformation(extent = {{-110, -50}, {-90, -70}}, rotation = 0)));
            Modelica.Electrical.MultiPhase.Interfaces.PositivePlug plug_rp(final m = mr) "Positive plug of rotor" annotation(
              Placement(transformation(extent = {{-110, 70}, {-90, 50}}, rotation = 0)));
            parameter Modelica.SIunits.Inductance Lm(start = 3 * sqrt(1 - 0.0667) / (2 * pi * fsNominal)) "Stator main field inductance per phase" annotation(
              Dialog(tab = "Nominal resistances and inductances", groupImage = "modelica://Modelica/Resources/Images/Electrical/Machines/IMS.png"));
            parameter Modelica.SIunits.Inductance Lrsigma(start = 3 * (1 - sqrt(1 - 0.0667)) / (2 * pi * fsNominal)) "Rotor leakage inductance per phase w.r.t. rotor side" annotation(
              Dialog(tab = "Nominal resistances and inductances"));
            parameter Modelica.SIunits.Inductance Lrzero = Lrsigma "Rotor zero inductance w.r.t. rotor side" annotation(
              Dialog(tab = "Nominal resistances and inductances"));
            parameter Modelica.SIunits.Resistance Rr(start = 0.04) "Rotor resistance per phase w.r.t. rotor side" annotation(
              Dialog(tab = "Nominal resistances and inductances"));
            parameter Modelica.SIunits.Temperature TrRef(start = 293.15) "Reference temperature of rotor resistance" annotation(
              Dialog(tab = "Nominal resistances and inductances"));
            parameter Modelica.Electrical.Machines.Thermal.LinearTemperatureCoefficient20 alpha20r(start = 0) "Temperature coefficient of rotor resistance at 20 degC" annotation(
              Dialog(tab = "Nominal resistances and inductances"));
            parameter Modelica.SIunits.Temperature TrOperational(start = 293.15) "Operational temperature of rotor resistance" annotation(
              Dialog(group = "Operational temperatures", enable = not useThermalPort));
            parameter Boolean useTurnsRatio(start = true) "Use TurnsRatio or calculate from locked-rotor voltage?";
            parameter Real TurnsRatio(final min = Modelica.Constants.small, start = 1) "Effective number of stator turns / effective number of rotor turns" annotation(
              Dialog(enable = useTurnsRatio));
            parameter Modelica.SIunits.Voltage VsNominal(start = 100) "Nominal stator voltage per phase" annotation(
              Dialog(enable = not useTurnsRatio));
            parameter Modelica.SIunits.Voltage VrLockedRotor(start = 100 * (2 * pi * fsNominal * Lm) / sqrt(Rs ^ 2 + (2 * pi * fsNominal * (Lm + Lssigma)) ^ 2)) "Locked rotor voltage per phase" annotation(
              Dialog(enable = not useTurnsRatio));
            parameter Modelica.Electrical.Machines.Losses.CoreParameters rotorCoreParameters(final m = 3, PRef = 0, VRef(start = 1) = 1, wRef(start = 1) = 1) "Rotor core loss parameter record, all quantities refer to rotor side" annotation(
              Dialog(tab = "Losses"));
            output Modelica.SIunits.Voltage vr[mr] = plug_rp.pin.v - plug_rn.pin.v "Rotor instantaneous voltages";
            output Modelica.SIunits.Current ir[mr] = plug_rp.pin.i "Rotor instantaneous currents";
          protected
            final parameter Real internalTurnsRatio = if useTurnsRatio then TurnsRatio else VsNominal / VrLockedRotor * (2 * pi * fsNominal * Lm) / sqrt(Rs ^ 2 + (2 * pi * fsNominal * (Lm + Lssigma)) ^ 2);
          public
            Components.SymmetricMultiPhaseWinding rotor(final Lsigma = Lrsigma, final effectiveTurns = effectiveStatorTurns / internalTurnsRatio, final useHeatPort = true, final RRef = Rr, final TRef = TrRef, final TOperational = TrOperational, final Lzero = Lrsigma, final GcRef = rotorCoreParameters.GcRef, final m = mr, final alpha20 = alpha20r) "Symmetric rotor winding including resistances, zero and stray inductances and zero core losses" annotation(
              Placement(transformation(origin = {0, -40}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
            /* previously used: state selection, now commented
              FundamentalWave.Interfaces.StateSelector stateSelectorR(
                final m=m,
                final xi=ir,
                final gamma=0) "State selection of rotor currents"
                annotation (Placement(transformation(extent={{-10,-10},{10,10}},
                    rotation=90,
                    origin={-90,0})));
              */
          equation
            connect(rotor.plug_n, plug_rn) annotation(
              Line(points = {{10, -50}, {10, -60}, {-100, -60}}, color = {0, 0, 255}));
            connect(rotor.heatPortCore, internalThermalPort.heatPortRotorCore) annotation(
              Line(points = {{10, -36}, {16, -36}, {16, -36}, {20, -36}, {20, -80}, {-40, -80}, {-40, -90}}, color = {191, 0, 0}, smooth = Smooth.None));
            connect(rotor.heatPortWinding, internalThermalPort.heatPortRotorWinding) annotation(
              Line(points = {{10, -44}, {20, -44}, {20, -80}, {-40, -80}, {-40, -90}}, color = {191, 0, 0}, smooth = Smooth.None));
            connect(plug_rp, rotor.plug_p) annotation(
              Line(points = {{-100, 60}, {-80, 60}, {-80, -50}, {-10, -50}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(airGap.port_rn, rotor.port_p) annotation(
              Line(points = {{-10, -10}, {-10, -30}}, color = {255, 128, 0}, smooth = Smooth.None));
            connect(airGap.port_rp, rotor.port_n) annotation(
              Line(points = {{10, -10}, {10, -30}}, color = {255, 128, 0}, smooth = Smooth.None));
            annotation(
              defaultComponentName = "aims",
              Icon(graphics = {Line(points = {{-100, 50}, {-100, 20}, {-60, 20}}, color = {0, 0, 255}), Line(points = {{-100, -50}, {-100, -20}, {-60, -20}}, color = {0, 0, 255})}),
              Documentation(info = "<html>
<p>
Resistances and stray inductances of the machine always refer to either stator or rotor. The symmetry of the stator and rotor is assumed. The number of stator and rotor phases may be different. The machine models take the following loss effects into account:
</p>

<ul>
<li>heat losses in the temperature dependent stator winding resistances</li>
<li>heat losses in the temperature dependent rotor winding resistances</li>
<li>friction losses</li>
<li>stator and rotor core losses (only eddy current losses, no hysteresis losses)</li>
<li>stray load losses</li>
</ul>

<h4>See also</h4>
<p>
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.BasicMachines.AsynchronousInductionMachines.AIM_SquirrelCage\">AIM_SquirrelCage</a>,
</p>
</html>"));
          end AIM_SlipRing;
          annotation(
            Documentation(info = "<html>
<p>This package provides squirrel cage and slip ring induction machine models.</p>

<h4>See also</h4>

<p>
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.BasicMachines.SynchronousInductionMachines\">SynchronousInductionMachines</a>
</p>
</html>"));
        end AsynchronousInductionMachines;

        package SynchronousInductionMachines "Synchronous machines"
          extends Modelica.Icons.VariantsPackage;

          model SM_PermanentMagnet "Permanent magnet synchronous machine with optional damper cage"
            extends Modelica.Magnetic.FundamentalWave.Interfaces.PartialBasicInductionMachine(is(start = zeros(m)), Rs(start = 0.03), Lssigma(start = 0.1 / (2 * pi * fsNominal)), final L0(d = 2.0 * Lmd / m / effectiveStatorTurns ^ 2, q = 2.0 * Lmq / m / effectiveStatorTurns ^ 2), redeclare final Modelica.Electrical.Machines.Thermal.SynchronousInductionMachines.ThermalAmbientSMPM thermalAmbient(final useDamperCage = useDamperCage, final Tr = TrOperational, final Tpm = TpmOperational), redeclare final Modelica.Electrical.Machines.Interfaces.InductionMachines.ThermalPortSMPM thermalPort(final useDamperCage = useDamperCage), redeclare final Modelica.Electrical.Machines.Interfaces.InductionMachines.ThermalPortSMPM internalThermalPort(final useDamperCage = useDamperCage), redeclare final Modelica.Electrical.Machines.Interfaces.InductionMachines.PowerBalanceSMPM powerBalance(final lossPowerRotorWinding = damperCageLossPower, final lossPowerRotorCore = 0, final lossPowerPermanentMagnet = permanentMagnet.lossPower));
            parameter Modelica.SIunits.Inductance Lmd(start = 0.3 / (2 * pi * fsNominal)) "Stator main field inductance per phase, d-axis" annotation(
              Dialog(tab = "Nominal resistances and inductances", groupImage = "modelica://Modelica/Resources/Images/Electrical/Machines/SMPM.png"));
            parameter Modelica.SIunits.Inductance Lmq(start = 0.3 / (2 * pi * fsNominal)) "Stator main field inductance per phase, q-axis" annotation(
              Dialog(tab = "Nominal resistances and inductances"));
            // Rotor cage parameters
            parameter Boolean useDamperCage(start = true) "Enable/disable damper cage" annotation(
              Dialog(tab = "Nominal resistances and inductances", group = "Damper cage"));
            parameter Modelica.SIunits.Inductance Lrsigmad(start = 0.05 / (2 * pi * fsNominal)) "Rotor leakage inductance, d-axis, w.r.t. stator side" annotation(
              Dialog(tab = "Nominal resistances and inductances", group = "Damper cage", enable = useDamperCage));
            parameter Modelica.SIunits.Inductance Lrsigmaq = Lrsigmad "Rotor leakage inductance, q-axis, w.r.t. stator side" annotation(
              Dialog(tab = "Nominal resistances and inductances", group = "Damper cage", enable = useDamperCage));
            parameter Modelica.SIunits.Resistance Rrd(start = 0.04) "Rotor resistance, d-axis, w.r.t. stator side" annotation(
              Dialog(tab = "Nominal resistances and inductances", group = "Damper cage", enable = useDamperCage));
            parameter Modelica.SIunits.Resistance Rrq = Rrd "Rotor resistance , q-axis, w.r.t. stator side" annotation(
              Dialog(tab = "Nominal resistances and inductances", group = "Damper cage", enable = useDamperCage));
            parameter Modelica.SIunits.Temperature TrRef(start = 293.15) "Reference temperature of damper resistances in d- and q-axis" annotation(
              Dialog(tab = "Nominal resistances and inductances", group = "Damper cage", enable = useDamperCage));
            parameter Modelica.Electrical.Machines.Thermal.LinearTemperatureCoefficient20 alpha20r(start = 0) "Temperature coefficient of damper resistances in d- and q-axis" annotation(
              Dialog(tab = "Nominal resistances and inductances", group = "Damper cage", enable = useDamperCage));
            parameter Modelica.SIunits.Voltage VsOpenCircuit(start = 112.3) "Open circuit RMS voltage per phase @ fsNominal";
            final parameter Modelica.SIunits.Temperature TpmOperational = 293.15 "Operational temperature of permanent magnet" annotation(
              Dialog(group = "Operational temperatures"));
            parameter Modelica.SIunits.Temperature TrOperational(start = 293.15) "Operational temperature of (optional) damper cage" annotation(
              Dialog(group = "Operational temperatures", enable = not useThermalPort and useDamperCage));
            parameter Modelica.Electrical.Machines.Losses.PermanentMagnetLossParameters permanentMagnetLossParameters(IRef(start = 100), wRef(start = 2 * pi * fsNominal / p)) "Permanent magnet loss parameter record" annotation(
              Dialog(tab = "Losses"));
            Modelica.Blocks.Interfaces.RealOutput ir[2](start = zeros(2), each final quantity = "ElectricCurrent", each final unit = "A") if useDamperCage "Damper cage currents" annotation(
              Dialog(showStartAttribute = true));
            Modelica.Magnetic.FundamentalWave.Components.Ground groundR "Ground of rotor magnetic circuit" annotation(
              Placement(transformation(extent = {{-40, -30}, {-20, -10}}, rotation = 0)));
            Modelica.Magnetic.FundamentalWave.Components.Short short if not useDamperCage "Magnetic connection in case the damper cage is not present" annotation(
              Placement(transformation(origin = {10, -40}, extent = {{10, 10}, {-10, -10}}, rotation = 270)));
            Modelica.Magnetic.FundamentalWave.BasicMachines.Components.SaliencyCageWinding rotorCage(final RRef(d = Rrd, q = Rrq), final Lsigma(d = Lrsigmad, q = Lrsigmaq), final effectiveTurns = sqrt(3.0 / 2.0) * effectiveStatorTurns, final useHeatPort = true, final TRef = TrRef, final alpha20 = alpha20r, final TOperational = TrOperational) if useDamperCage "Symmetric rotor cage winding including resistances and stray inductances" annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 90, origin = {20, -40})));
            Components.PermanentMagnet permanentMagnet(final V_m = Complex(V_mPM, 0), final m = m, final permanentMagnetLossParameters = permanentMagnetLossParameters, final useHeatPort = true, final is = is) "Magnetic potential difference of permanent magnet" annotation(
              Placement(transformation(origin = {-10, -40}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          protected
            final parameter Modelica.SIunits.MagneticPotentialDifference V_mPM = 2 / pi * sqrt(2) * (m / 2) * VsOpenCircuit / effectiveStatorTurns / (Lmd / effectiveStatorTurns ^ 2 * 2 * pi * fsNominal) "Equivalent excitation magnetic potential difference";
            Modelica.Blocks.Interfaces.RealOutput damperCageLossPower(final quantity = "Power", final unit = "W") "Damper losses";
          equation
            connect(ir, rotorCage.i);
            connect(damperCageLossPower, rotorCage.lossPower);
            if not useDamperCage then
              damperCageLossPower = 0;
            end if;
            connect(permanentMagnet.port_p, airGap.port_rn) annotation(
              Line(points = {{-10, -30}, {-10, -10}}, color = {255, 128, 0}, smooth = Smooth.None));
            connect(permanentMagnet.support, airGap.support) annotation(
              Line(points = {{-20, -40}, {-50, -40}, {-50, 0}, {-10, 0}}, color = {0, 0, 0}, smooth = Smooth.None));
            connect(permanentMagnet.heatPort, internalThermalPort.heatPortPermanentMagnet) annotation(
              Line(points = {{-20, -30}, {-40, -30}, {-40, -90}}, color = {191, 0, 0}, smooth = Smooth.None));
            connect(permanentMagnet.flange, inertiaRotor.flange_b) annotation(
              Line(points = {{0, -40}, {0, -20}, {90, -20}, {90, -1.33227e-015}}, color = {0, 0, 0}, smooth = Smooth.None));
            connect(airGap.port_rp, rotorCage.port_n) annotation(
              Line(points = {{10, -10}, {10, -10}, {10, -30}, {10, -30}, {10, -30}, {20, -30}, {20, -30}}, color = {255, 128, 0}, smooth = Smooth.None));
            connect(short.port_n, airGap.port_rp) annotation(
              Line(points = {{10, -30}, {10, -10}}, color = {255, 128, 0}, smooth = Smooth.None));
            connect(short.port_p, permanentMagnet.port_n) annotation(
              Line(points = {{10, -50}, {-10, -50}}, color = {255, 128, 0}, smooth = Smooth.None));
            connect(rotorCage.port_p, permanentMagnet.port_n) annotation(
              Line(points = {{20, -50}, {-10, -50}}, color = {255, 128, 0}, smooth = Smooth.None));
            connect(rotorCage.heatPortWinding, internalThermalPort.heatPortRotorWinding) annotation(
              Line(points = {{30, -40}, {40, -40}, {40, -80}, {-40, -80}, {-40, -90}}, color = {191, 0, 0}, smooth = Smooth.None));
            annotation(
              defaultComponentName = "smpm",
              Icon(graphics = {Rectangle(extent = {{-130, 10}, {-100, -10}}, lineColor = {0, 0, 0}, fillColor = {0, 255, 0}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-100, 10}, {-70, -10}}, lineColor = {0, 0, 0}, fillColor = {255, 0, 0}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-134, 34}, {-66, -34}}, lineColor = {0, 0, 255})}),
              Documentation(info = "<html>
<p>
Resistances and stray inductances of the machine refer to an <code>m</code> phase stator. The symmetry of the stator is assumed. For rotor asymmetries can be taken into account by different resistances and stray inductances in the d- and q-axis. The machine models take the following loss effects into account:
</p>

<ul>
<li>heat losses in the temperature dependent stator winding resistances</li>
<li>optional, when enabled: heat losses in the temperature dependent damper cage resistances</li>
<li>friction losses</li>
<li>core losses (only eddy current losses, no hysteresis losses)</li>
<li>stray load losses</li>
<li>permanent magnet losses</li>
</ul>

<h4>See also</h4>
<p>
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.BasicMachines.SynchronousInductionMachines.SM_ElectricalExcited\">SM_ElectricalExcited</a>,
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.BasicMachines.SynchronousInductionMachines.SM_ReluctanceRotor\">SM_ReluctanceRotor</a>,
</p>
</html>"));
          end SM_PermanentMagnet;

          model SM_ElectricalExcited "Electrical excited synchronous machine with optional damper cage"
            extends Modelica.Magnetic.FundamentalWave.Interfaces.PartialBasicInductionMachine(is(start = zeros(m)), Rs(start = 0.03), Lssigma(start = 0.1 / (2 * pi * fsNominal)), final L0(d = 2.0 * Lmd / m / effectiveStatorTurns ^ 2, q = 2.0 * Lmq / m / effectiveStatorTurns ^ 2), redeclare final Modelica.Electrical.Machines.Thermal.SynchronousInductionMachines.ThermalAmbientSMEE thermalAmbient(final useDamperCage = useDamperCage, final Te = TeOperational, final Tr = TrOperational), redeclare final Modelica.Electrical.Machines.Interfaces.InductionMachines.ThermalPortSMEE thermalPort(final useDamperCage = useDamperCage), redeclare final Modelica.Electrical.Machines.Interfaces.InductionMachines.ThermalPortSMEE internalThermalPort(final useDamperCage = useDamperCage), redeclare final Modelica.Electrical.Machines.Interfaces.InductionMachines.PowerBalanceSMEE powerBalance(final lossPowerRotorWinding = damperCageLossPower, final powerExcitation = ve * ie, final lossPowerExcitation = excitation.resistor.LossPower, final lossPowerBrush = brush.lossPower, final lossPowerRotorCore = 0));
            parameter Modelica.SIunits.Inductance Lmd(start = 1.5 / (2 * pi * fsNominal)) "Stator main field inductance per phase, d-axis" annotation(
              Dialog(tab = "Nominal resistances and inductances", groupImage = "modelica://Modelica/Resources/Images/Electrical/Machines/SMEE.png"));
            parameter Modelica.SIunits.Inductance Lmq(start = 1.5 / (2 * pi * fsNominal)) "Stator main field inductance per phase, q-axis" annotation(
              Dialog(tab = "Nominal resistances and inductances"));
            // Rotor cage parameters
            parameter Boolean useDamperCage(start = true) "Enable/disable damper cage" annotation(
              Dialog(tab = "Nominal resistances and inductances", group = "DamperCage"));
            parameter Modelica.SIunits.Inductance Lrsigmad(start = 0.05 / (2 * pi * fsNominal)) "Rotor leakage inductance, d-axis, w.r.t. stator side" annotation(
              Dialog(tab = "Nominal resistances and inductances", group = "DamperCage", enable = useDamperCage));
            parameter Modelica.SIunits.Inductance Lrsigmaq = Lrsigmad "Rotor leakage inductance, q-axis, w.r.t. stator side" annotation(
              Dialog(tab = "Nominal resistances and inductances", group = "DamperCage", enable = useDamperCage));
            parameter Modelica.SIunits.Resistance Rrd(start = 0.04) "Rotor resistance, d-axis, w.r.t. stator side" annotation(
              Dialog(tab = "Nominal resistances and inductances", group = "DamperCage", enable = useDamperCage));
            parameter Modelica.SIunits.Resistance Rrq = Rrd "Rotor resistance , q-axis, w.r.t. stator side" annotation(
              Dialog(tab = "Nominal resistances and inductances", group = "DamperCage", enable = useDamperCage));
            parameter Modelica.SIunits.Temperature TrRef(start = 293.15) "Reference temperature of damper resistances in d- and q-axis" annotation(
              Dialog(tab = "Nominal resistances and inductances", group = "DamperCage", enable = useDamperCage));
            parameter Modelica.Electrical.Machines.Thermal.LinearTemperatureCoefficient20 alpha20r(start = 0) "Temperature coefficient of damper resistances in d- and q-axis" annotation(
              Dialog(tab = "Nominal resistances and inductances", group = "DamperCage", enable = useDamperCage));
            // Operational temperature
            parameter Modelica.SIunits.Temperature TrOperational(start = 293.15) "Operational temperature of (optional) damper cage" annotation(
              Dialog(group = "Operational temperatures", enable = not useThermalPort and useDamperCage));
            parameter Modelica.SIunits.Temperature TeOperational(start = 293.15) "Operational excitation temperature" annotation(
              Dialog(group = "Operational temperatures", enable = not useThermalPort));
            // Excitation parameters
            parameter Modelica.SIunits.Voltage VsNominal(start = 100) "Nominal stator voltage" annotation(
              Dialog(tab = "Excitation"));
            parameter Modelica.SIunits.Current IeOpenCircuit(start = 10) "Open circuit excitation current @ nominal voltage and frequency" annotation(
              Dialog(tab = "Excitation"));
            parameter Modelica.SIunits.Resistance Re(start = 2.5) "Warm excitation resistance" annotation(
              Dialog(tab = "Excitation"));
            parameter Modelica.SIunits.Temperature TeRef(start = 293.15) "Reference temperture of excitation resistance" annotation(
              Dialog(tab = "Excitation"));
            parameter Modelica.Electrical.Machines.Thermal.LinearTemperatureCoefficient20 alpha20e(start = 0) "Temperature coefficient of excitation resistance" annotation(
              Dialog(tab = "Excitation"));
            parameter Real sigmae(min = 0, max = 1, start = 0.025) "Stray fraction of total excitation inductance" annotation(
              Dialog(tab = "Excitation"));
            parameter Modelica.Electrical.Machines.Losses.BrushParameters brushParameters "Brush loss parameter record" annotation(
              Dialog(tab = "Losses"));
            output Modelica.SIunits.Voltage ve = pin_ep.v - pin_en.v "Excitation voltage";
            output Modelica.SIunits.Current ie = pin_ep.i "Excitation current";
            Modelica.Blocks.Interfaces.RealOutput ir[2](start = zeros(2), each final quantity = "ElectricCurrent", each final unit = "A") if useDamperCage "Damper cage currents" annotation(
              Dialog(showStartAttribute = true));
            FundamentalWave.Components.Short short if not useDamperCage "Magnetic connection in case the damper cage is not present" annotation(
              Placement(transformation(origin = {10, -40}, extent = {{10, 10}, {-10, -10}}, rotation = 270)));
            Components.SaliencyCageWinding rotorCage(final Lsigma(d = Lrsigmad, q = Lrsigmaq), final effectiveTurns = sqrt(3.0 / 2.0) * effectiveStatorTurns, final useHeatPort = true, final TRef = TrRef, final TOperational = TrOperational, final RRef(d = Rrd, q = Rrq), final alpha20 = alpha20r) if useDamperCage "Symmetric rotor cage winding including resistances and stray inductances" annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 90, origin = {20, -40})));
            Components.SinglePhaseWinding excitation(final orientation = 0, final RRef = Re, final TRef = TeRef, final Lsigma = Lesigma, final effectiveTurns = effectiveStatorTurns * turnsRatio * m / 2, final useHeatPort = true, final TOperational = TeOperational, final alpha20 = alpha20e) "Excitation winding including resistance and stray inductance" annotation(
              Placement(transformation(extent = {{-30, -50}, {-10, -30}}, rotation = 0)));
            Modelica.Electrical.Analog.Interfaces.PositivePin pin_ep "Positive pin of excitation" annotation(
              Placement(transformation(extent = {{-110, 70}, {-90, 50}}, rotation = 0)));
            Modelica.Electrical.Analog.Interfaces.NegativePin pin_en "Negative pin of excitation" annotation(
              Placement(transformation(extent = {{-90, -50}, {-110, -70}}, rotation = 0)));
            Electrical.Machines.Losses.DCMachines.Brush brush(final brushParameters = brushParameters, final useHeatPort = true) annotation(
              Placement(transformation(extent = {{10, -10}, {-10, 10}}, rotation = 90, origin = {-80, 40})));
          protected
            final parameter Real turnsRatio = sqrt(2) * VsNominal / (2 * pi * fsNominal * Lmd * IeOpenCircuit) "Stator current / excitation current";
            final parameter Modelica.SIunits.Inductance Lesigma = Lmd * turnsRatio ^ 2 * 3 / 2 * sigmae / (1 - sigmae) "Leakage inductance of the excitation winding";
            Modelica.Blocks.Interfaces.RealOutput damperCageLossPower(final quantity = "Power", final unit = "W") "Damper losses";
          equation
            connect(ir, rotorCage.i);
            connect(damperCageLossPower, rotorCage.lossPower);
            if not useDamperCage then
              damperCageLossPower = 0;
            end if;
            connect(pin_en, excitation.pin_n) annotation(
              Line(points = {{-100, -60}, {-100, -60}, {-100, -50}, {-30, -50}}, color = {0, 0, 255}));
            connect(airGap.port_rn, excitation.port_p) annotation(
              Line(points = {{-10, -10}, {-10, -20}, {-10, -20}, {-10, -30}}, color = {255, 128, 0}, smooth = Smooth.None));
            connect(pin_ep, brush.p) annotation(
              Line(points = {{-100, 60}, {-80, 60}, {-80, 50}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(brush.n, excitation.pin_p) annotation(
              Line(points = {{-80, 30}, {-80, -30}, {-30, -30}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(brush.heatPort, internalThermalPort.heatPortBrush) annotation(
              Line(points = {{-70, 50}, {-40, 50}, {-40, -90}}, color = {191, 0, 0}, smooth = Smooth.None));
            connect(excitation.heatPortWinding, internalThermalPort.heatPortExcitation) annotation(
              Line(points = {{-20, -50}, {-20, -80}, {-40, -80}, {-40, -90}}, color = {191, 0, 0}, smooth = Smooth.None));
            connect(airGap.port_rp, rotorCage.port_n) annotation(
              Line(points = {{10, -10}, {10, -30}, {20, -30}}, color = {255, 128, 0}, smooth = Smooth.None));
            connect(short.port_n, airGap.port_rp) annotation(
              Line(points = {{10, -30}, {10, -10}}, color = {255, 128, 0}, smooth = Smooth.None));
            connect(rotorCage.port_p, excitation.port_n) annotation(
              Line(points = {{20, -50}, {-10, -50}}, color = {255, 128, 0}, smooth = Smooth.None));
            connect(short.port_p, excitation.port_n) annotation(
              Line(points = {{10, -50}, {-10, -50}}, color = {255, 128, 0}, smooth = Smooth.None));
            connect(rotorCage.heatPortWinding, internalThermalPort.heatPortRotorWinding) annotation(
              Line(points = {{30, -40}, {40, -40}, {40, -80}, {-40, -80}, {-40, -90}}, color = {191, 0, 0}, smooth = Smooth.None));
            annotation(
              defaultComponentName = "smee",
              Icon(graphics = {Ellipse(extent = {{-134, 34}, {-66, -34}}, lineColor = {0, 0, 255}), Line(points = {{-100, 50}, {-100, 20}, {-130, 20}, {-130, -4}}, color = {0, 0, 255}), Line(points = {{-130, -4}, {-129, 1}, {-125, 5}, {-120, 6}, {-115, 5}, {-111, 1}, {-110, -4}}, color = {0, 0, 255}), Line(points = {{-110, -4}, {-109, 1}, {-105, 5}, {-100, 6}, {-95, 5}, {-91, 1}, {-90, -4}}, color = {0, 0, 255}), Line(points = {{-90, -4}, {-89, 1}, {-85, 5}, {-80, 6}, {-75, 5}, {-71, 1}, {-70, -4}}, color = {0, 0, 255}), Line(points = {{-100, -50}, {-100, -20}, {-70, -20}, {-70, -2}}, color = {0, 0, 255})}),
              Documentation(info = "<html>
<p>
The symmetry of the stator is assumed. For rotor asymmetries can be taken into account by different resistances and stray inductances in the d- and q-axis. The machine models take the following loss effects into account:
</p>

<ul>
<li>heat losses in the temperature dependent stator winding resistances</li>
<li>heat losses in the temperature dependent excitation winding resistance</li>
<li>optional, when enabled: heat losses in the temperature dependent damper cage resistances</li>
<li>brush losses in the excitation circuit</li>
<li>friction losses</li>
<li>core losses (only eddy current losses, no hysteresis losses)</li>
<li>stray load losses</li>
</ul>

<h4>See also</h4>
<p>
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.BasicMachines.SynchronousInductionMachines.SM_PermanentMagnet\">SM_PermanentMagnet</a>,
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.BasicMachines.SynchronousInductionMachines.SM_ReluctanceRotor\">SM_ReluctanceRotor</a>,
</p>
</html>"));
          end SM_ElectricalExcited;

          model SM_ReluctanceRotor "Reluctance machine with optional damper cage"
            extends Modelica.Magnetic.FundamentalWave.Interfaces.PartialBasicInductionMachine(is(start = zeros(m)), Rs(start = 0.03), Lssigma(start = 0.1 / (2 * pi * fsNominal)), final L0(d = 2.0 * Lmd / m / effectiveStatorTurns ^ 2, q = 2.0 * Lmq / m / effectiveStatorTurns ^ 2), redeclare final Modelica.Electrical.Machines.Thermal.SynchronousInductionMachines.ThermalAmbientSMR thermalAmbient(final useDamperCage = useDamperCage, final Tr = TrOperational), redeclare final Modelica.Electrical.Machines.Interfaces.InductionMachines.ThermalPortSMR thermalPort(final useDamperCage = useDamperCage), redeclare final Modelica.Electrical.Machines.Interfaces.InductionMachines.ThermalPortSMR internalThermalPort(final useDamperCage = useDamperCage), redeclare final Modelica.Electrical.Machines.Interfaces.InductionMachines.PowerBalanceSMR powerBalance(final lossPowerRotorWinding = damperCageLossPower, final lossPowerRotorCore = 0));
            parameter Modelica.SIunits.Temperature TrOperational(start = 293.15) "Operational temperature of (optional) damper cage" annotation(
              Dialog(group = "Operational temperatures", enable = not useThermalPort and useDamperCage));
            parameter Modelica.SIunits.Inductance Lmd(start = 2.9 / (2 * pi * fsNominal)) "Stator main field inductance per phase, d-axis" annotation(
              Dialog(tab = "Nominal resistances and inductances", groupImage = "modelica://Modelica/Resources/Images/Electrical/Machines/SMR.png"));
            parameter Modelica.SIunits.Inductance Lmq(start = 0.9 / (2 * pi * fsNominal)) "Stator main field inductance per phase, q-axis" annotation(
              Dialog(tab = "Nominal resistances and inductances"));
            // Rotor cage parameters
            parameter Boolean useDamperCage(start = true) "Enable/disable damper cage" annotation(
              Dialog(tab = "Nominal resistances and inductances", group = "DamperCage"));
            parameter Modelica.SIunits.Inductance Lrsigmad(start = 0.05 / (2 * pi * fsNominal)) "Rotor leakage inductance, d-axis, w.r.t. stator side" annotation(
              Dialog(tab = "Nominal resistances and inductances", group = "DamperCage", enable = useDamperCage));
            parameter Modelica.SIunits.Inductance Lrsigmaq = Lrsigmad "Rotor leakage inductance, q-axis, w.r.t. stator side" annotation(
              Dialog(tab = "Nominal resistances and inductances", group = "DamperCage", enable = useDamperCage));
            parameter Modelica.SIunits.Resistance Rrd(start = 0.04) "Rotor resistance, d-axis, w.r.t. stator side" annotation(
              Dialog(tab = "Nominal resistances and inductances", group = "DamperCage", enable = useDamperCage));
            parameter Modelica.SIunits.Resistance Rrq = Rrd "Rotor resistance , q-axis, w.r.t. stator side" annotation(
              Dialog(tab = "Nominal resistances and inductances", group = "DamperCage", enable = useDamperCage));
            parameter Modelica.SIunits.Temperature TrRef(start = 293.15) "Reference temperature of damper resistances in d- and q-axis" annotation(
              Dialog(tab = "Nominal resistances and inductances", group = "DamperCage", enable = useDamperCage));
            parameter Modelica.Electrical.Machines.Thermal.LinearTemperatureCoefficient20 alpha20r(start = 0) "Temperature coefficient of damper resistances in d- and q-axis" annotation(
              Dialog(tab = "Nominal resistances and inductances", group = "DamperCage", enable = useDamperCage));
            Modelica.Blocks.Interfaces.RealOutput ir[2](start = zeros(2), each final quantity = "ElectricCurrent", each final unit = "A") if useDamperCage "Damper cage currents" annotation(
              Dialog(showStartAttribute = true));
            Modelica.Magnetic.FundamentalWave.Components.Ground groundR "Ground of rotor magnetic circuit" annotation(
              Placement(transformation(extent = {{-40, -30}, {-20, -10}}, rotation = 0)));
            Modelica.Magnetic.FundamentalWave.Components.Short short if not useDamperCage "Magnetic connection in case the damper cage is not present" annotation(
              Placement(transformation(origin = {10, -40}, extent = {{10, 10}, {-10, -10}}, rotation = 270)));
            Modelica.Magnetic.FundamentalWave.BasicMachines.Components.SaliencyCageWinding rotorCage(final RRef(d = Rrd, q = Rrq), final Lsigma(d = Lrsigmad, q = Lrsigmaq), final effectiveTurns = sqrt(3.0 / 2.0) * effectiveStatorTurns, final useHeatPort = true, final TRef = TrRef, final alpha20 = alpha20r, final TOperational = TrOperational) if useDamperCage "Symmetric rotor cage winding including resistances and stray inductances" annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 90, origin = {20, -40})));
          protected
            Modelica.Blocks.Interfaces.RealOutput damperCageLossPower(final quantity = "Power", final unit = "W") "Damper losses";
          equation
            connect(ir, rotorCage.i);
            connect(damperCageLossPower, rotorCage.lossPower);
            if not useDamperCage then
              damperCageLossPower = 0;
            end if;
            connect(rotorCage.port_n, airGap.port_rp) annotation(
              Line(points = {{20, -30}, {20, -30}, {10, -30}, {10, -30}, {10, -30}, {10, -10}, {10, -10}}, color = {255, 128, 0}, smooth = Smooth.None));
            connect(short.port_n, airGap.port_rp) annotation(
              Line(points = {{10, -30}, {10, -10}}, color = {255, 128, 0}, smooth = Smooth.None));
            connect(rotorCage.port_p, airGap.port_rn) annotation(
              Line(points = {{20, -50}, {-10, -50}, {-10, -50}, {-10, -50}, {-10, -10}, {-10, -10}}, color = {255, 128, 0}, smooth = Smooth.None));
            connect(short.port_p, airGap.port_rn) annotation(
              Line(points = {{10, -50}, {-10, -50}, {-10, -10}}, color = {255, 128, 0}, smooth = Smooth.None));
            connect(rotorCage.heatPortWinding, internalThermalPort.heatPortRotorWinding) annotation(
              Line(points = {{30, -40}, {40, -40}, {40, -80}, {-40, -80}, {-40, -90}}, color = {191, 0, 0}, smooth = Smooth.None));
            annotation(
              defaultComponentName = "smr",
              Icon(graphics = {Rectangle(extent = {{-130, 10}, {-100, -10}}, lineColor = {0, 0, 0}), Rectangle(extent = {{-100, 10}, {-70, -10}}, lineColor = {0, 0, 0}), Ellipse(extent = {{-134, 34}, {-66, -34}}, lineColor = {0, 0, 255})}),
              Documentation(info = "<html>
<p>
The symmetry of the stator is assumed. For rotor asymmetries can be taken into account by different resistances and stray inductances in the d- and q-axis. The machine models take the following loss effects into account:
</p>

<ul>
<li>heat losses in the temperature dependent stator winding resistances</li>
<li>optional, when enabled: heat losses in the temperature dependent damper cage resistances</li>
<li>friction losses</li>
<li>core losses (only eddy current losses, no hysteresis losses)</li>
<li>stray load losses</li>
</ul>

<h4>See also</h4>
<p>
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.BasicMachines.SynchronousInductionMachines.SM_ElectricalExcited\">SM_ElectricalExcited</a>,
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.BasicMachines.SynchronousInductionMachines.SM_PermanentMagnet\">SM_PermanentMagnet</a>,
</p>
</html>"));
          end SM_ReluctanceRotor;
          annotation(
            Documentation(info = "<html>
<p>This package contains various synchronous induction machine models.</p>

<h4>See also</h4>

<p>
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.BasicMachines.AsynchronousInductionMachines\">AsynchronousInductionMachines</a>
</p>
</html>"));
        end SynchronousInductionMachines;

        package Components "Components specially for electric machines"
          model SinglePhaseWinding "Symmetric winding model coupling electrical and magnetic domain"
            Modelica.Electrical.Analog.Interfaces.PositivePin pin_p "Positive pin" annotation(
              Placement(transformation(origin = {-100, 100}, extent = {{-10, -10}, {10, 10}}, rotation = 180)));
            Modelica.Electrical.Analog.Interfaces.NegativePin pin_n "Negative pin" annotation(
              Placement(transformation(origin = {-100, -100}, extent = {{-10, -10}, {10, 10}}, rotation = 180)));
            Modelica.Magnetic.FundamentalWave.Interfaces.NegativeMagneticPort port_n "Negative complex magnetic port" annotation(
              Placement(transformation(extent = {{90, -110}, {110, -90}}, rotation = 0)));
            Modelica.Magnetic.FundamentalWave.Interfaces.PositiveMagneticPort port_p "Positive complex magnetic port" annotation(
              Placement(transformation(extent = {{90, 90}, {110, 110}}, rotation = 0)));
            parameter Boolean useHeatPort = false "Enable / disable (=fixed temperatures) thermal port" annotation(
              Evaluate = true);
            parameter Modelica.SIunits.Resistance RRef "Winding resistance per phase at TRef";
            parameter Modelica.SIunits.Temperature TRef(start = 293.15) "Reference temperature of winding";
            parameter Modelica.Electrical.Machines.Thermal.LinearTemperatureCoefficient20 alpha20(start = 0) "Temperature coefficient of winding at 20 degC";
            final parameter Modelica.SIunits.LinearTemperatureCoefficient alphaRef = Modelica.Electrical.Machines.Thermal.convertAlpha(alpha20, TRef, 293.15);
            parameter Modelica.SIunits.Temperature TOperational(start = 293.15) "Operational temperature of winding" annotation(
              Dialog(enable = not useHeatPort));
            parameter Modelica.SIunits.Inductance Lsigma "Winding stray inductance per phase";
            parameter Real effectiveTurns = 1 "Effective number of turns per phase";
            parameter Modelica.SIunits.Angle orientation "Orientation of the resulting fundamental wave field phasor";
            Modelica.Electrical.Analog.Basic.Resistor resistor(final useHeatPort = useHeatPort, final R = RRef, final T_ref = TRef, final alpha = alphaRef, final T = TOperational) annotation(
              Placement(transformation(origin = {-10, 70}, extent = {{10, 10}, {-10, -10}}, rotation = 90)));
            Modelica.Magnetic.FundamentalWave.Components.SinglePhaseElectroMagneticConverter electroMagneticConverter(final effectiveTurns = effectiveTurns, final orientation = orientation) annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0)));
            Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a heatPortWinding if useHeatPort "Heat ports of winding resistor" annotation(
              Placement(transformation(extent = {{-10, -110}, {10, -90}})));
            Modelica.Magnetic.FundamentalWave.Components.Reluctance reluctance(final R_m(d = effectiveTurns ^ 2 / Lsigma, q = effectiveTurns ^ 2 / Lsigma)) annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 270, origin = {30, 0})));
          equation
            connect(pin_p, resistor.p) annotation(
              Line(points = {{-100, 100}, {-10, 100}, {-10, 80}}, color = {0, 0, 255}));
            connect(electroMagneticConverter.pin_n, pin_n) annotation(
              Line(points = {{-10, -10}, {-10, -100}, {-100, -100}}, color = {0, 0, 255}));
            connect(electroMagneticConverter.port_p, port_p) annotation(
              Line(points = {{10, 10}, {10, 100}, {100, 100}}, color = {255, 128, 0}));
            connect(electroMagneticConverter.port_n, port_n) annotation(
              Line(points = {{10, -10}, {10, -100}, {100, -100}}, color = {255, 128, 0}));
            connect(heatPortWinding, resistor.heatPort) annotation(
              Line(points = {{5.55112e-16, -100}, {5.55112e-16, -60}, {-40, -60}, {-40, 70}, {-20, 70}}, color = {191, 0, 0}, smooth = Smooth.None));
            connect(resistor.n, electroMagneticConverter.pin_p) annotation(
              Line(points = {{-10, 60}, {-10, 10}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(electroMagneticConverter.port_p, reluctance.port_p) annotation(
              Line(points = {{10, 10}, {15, 10}, {15, 10}, {20, 10}, {20, 10}, {30, 10}}, color = {255, 128, 0}, smooth = Smooth.None));
            connect(electroMagneticConverter.port_n, reluctance.port_n) annotation(
              Line(points = {{10, -10}, {30, -10}}, color = {255, 128, 0}, smooth = Smooth.None));
            annotation(
              Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, 60}, {100, -60}}, lineColor = {0, 0, 255}, pattern = LinePattern.None, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{100, -100}, {94, -100}, {84, -98}, {76, -94}, {64, -86}, {50, -72}, {42, -58}, {36, -40}, {30, -18}, {30, 0}, {30, 18}, {34, 36}, {46, 66}, {62, 84}, {78, 96}, {90, 100}, {100, 100}}, color = {255, 128, 0}), Line(points = {{40, 60}, {-100, 60}, {-100, 100}}, color = {0, 0, 255}), Line(points = {{40, -60}, {-100, -60}, {-100, -98}}, color = {0, 0, 255}), Line(points = {{40, 60}, {100, 20}, {40, -20}, {0, -20}, {-40, 0}, {0, 20}, {40, 20}, {100, -20}, {40, -60}}, color = {0, 0, 255}), Text(extent = {{0, 160}, {0, 120}}, lineColor = {0, 0, 255}, fillColor = {255, 128, 0}, fillPattern = FillPattern.Solid, textString = "%name")}),
              Documentation(info = "<html>
<p>
The single phase winding consists of a winding
<a href=\"modelica://Modelica.Electrical.Analog.Basic.Resistor\">resistor</a>, a
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.Components.SinglePhaseElectroMagneticConverter\">single phase electromagnetic coupling</a> and a <a href=\"modelica://Modelica.Magnetic.FundamentalWave.Components.Reluctance\">stray reluctance</a>.
</p>

<h4>See also</h4>
<p>
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.BasicMachines.Components.SymmetricMultiPhaseWinding\">SymmetricMultiPhaseWinding</a>,
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.BasicMachines.Components.SymmetricMultiPhaseCageWinding\">SymmetricMultiPhaseCageWinding</a>,
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.BasicMachines.Components.SaliencyCageWinding\">SaliencyCageWinding</a>
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.BasicMachines.Components.RotorSaliencyAirGap\">RotorSaliencyAirGap</a>
</p>
</html>"));
          end SinglePhaseWinding;

          extends Modelica.Icons.Package;

          model SymmetricMultiPhaseWinding "Symmetric winding model coupling electrical and magnetic domain"
            // Orientation changed
            Modelica.Electrical.MultiPhase.Interfaces.PositivePlug plug_p(final m = m) "Positive plug" annotation(
              Placement(transformation(origin = {-100, 100}, extent = {{-10, -10}, {10, 10}}, rotation = 180)));
            Modelica.Electrical.MultiPhase.Interfaces.NegativePlug plug_n(final m = m) "Negative plug" annotation(
              Placement(transformation(origin = {-100, -100}, extent = {{-10, -10}, {10, 10}}, rotation = 180)));
            Modelica.Magnetic.FundamentalWave.Interfaces.NegativeMagneticPort port_n "Negative complex magnetic port" annotation(
              Placement(transformation(extent = {{90, -110}, {110, -90}}, rotation = 0)));
            Modelica.Magnetic.FundamentalWave.Interfaces.PositiveMagneticPort port_p "Positive complex magnetic port" annotation(
              Placement(transformation(extent = {{90, 90}, {110, 110}}, rotation = 0)));
            parameter Integer m = 3 "Number of phases";
            parameter Boolean useHeatPort = false "Enable / disable (=fixed temperatures) thermal port" annotation(
              Evaluate = true);
            // Resistor model
            parameter Modelica.SIunits.Resistance RRef "Winding resistance per phase at TRef";
            parameter Modelica.SIunits.Temperature TRef(start = 293.15) "Reference temperature of winding";
            parameter Modelica.Electrical.Machines.Thermal.LinearTemperatureCoefficient20 alpha20(start = 0) "Temperature coefficient of winding at 20 degC";
            final parameter Modelica.SIunits.LinearTemperatureCoefficient alphaRef = Modelica.Electrical.Machines.Thermal.convertAlpha(alpha20, TRef, 293.15);
            parameter Modelica.SIunits.Temperature TOperational(start = 293.15) "Operational temperature of winding" annotation(
              Dialog(enable = not useHeatPort));
            parameter Modelica.SIunits.Inductance Lsigma "Winding stray inductance per phase";
            parameter Modelica.SIunits.Inductance Lzero "Zero sequence inductance of winding";
            parameter Real effectiveTurns = 1 "Effective number of turns per phase";
            parameter Modelica.SIunits.Conductance GcRef "Electrical reference core loss reluctance";
            Modelica.Magnetic.FundamentalWave.Components.MultiPhaseElectroMagneticConverter electroMagneticConverter(final m = m, final effectiveTurns = fill(effectiveTurns, m), final orientation = Modelica.Electrical.MultiPhase.Functions.symmetricOrientation(m)) annotation(
              Placement(transformation(extent = {{-10, -40}, {10, -20}}, rotation = 0)));
            Modelica.Electrical.MultiPhase.Basic.ZeroInductor zeroInductor(final m = m, final Lzero = Lzero) "Zero sequence inductance of winding" annotation(
              Placement(transformation(origin = {-20, 30}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
            Modelica.Electrical.MultiPhase.Basic.Resistor resistor(final m = m, final useHeatPort = useHeatPort, final R = fill(RRef, m), final T_ref = fill(TRef, m), final alpha = fill(alphaRef, m), final T = fill(TOperational, m)) "Winding resistor" annotation(
              Placement(transformation(origin = {-20, 70}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
            Thermal.HeatTransfer.Interfaces.HeatPort_a heatPortWinding[m] if useHeatPort "Heat ports of winding resistors" annotation(
              Placement(transformation(extent = {{-50, -110}, {-30, -90}})));
            Thermal.HeatTransfer.Interfaces.HeatPort_a heatPortCore if useHeatPort "Heat port of core" annotation(
              Placement(transformation(extent = {{30, -110}, {50, -90}})));
            Modelica.Magnetic.FundamentalWave.Components.EddyCurrent core(final useHeatPort = useHeatPort, final G = m / 2 * GcRef * effectiveTurns ^ 2) "Core loss model (currently eddy currents only)" annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0, origin = {50, -40})));
            Modelica.Magnetic.FundamentalWave.Components.Reluctance strayReluctance(final R_m(d = m * effectiveTurns ^ 2 / 2 / Lsigma, q = m * effectiveTurns ^ 2 / 2 / Lsigma)) "Stray reluctance equivalent to ideally coupled stray inductances" annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 270, origin = {80, 30})));
          equation
            connect(plug_p, resistor.plug_p) annotation(
              Line(points = {{-100, 100}, {-20, 100}, {-20, 80}}, color = {0, 0, 255}));
            connect(resistor.plug_n, zeroInductor.plug_p) annotation(
              Line(points = {{-20, 60}, {-20, 55}, {-20, 50}, {-20, 40}}, color = {0, 0, 255}));
            connect(zeroInductor.plug_n, electroMagneticConverter.plug_p) annotation(
              Line(points = {{-20, 20}, {-20, -20}, {-10, -20}}, color = {0, 0, 255}));
            connect(electroMagneticConverter.plug_n, plug_n) annotation(
              Line(points = {{-10, -40}, {-10, -40}, {-100, -40}, {-100, -100}}, color = {0, 0, 255}));
            connect(electroMagneticConverter.port_p, port_p) annotation(
              Line(points = {{10, -20}, {10, 100}, {100, 100}}, color = {255, 128, 0}));
            connect(resistor.heatPort, heatPortWinding) annotation(
              Line(points = {{-30, 70}, {-40, 70}, {-40, -100}}, color = {191, 0, 0}, smooth = Smooth.None));
            connect(electroMagneticConverter.port_n, core.port_p) annotation(
              Line(points = {{10, -40}, {40, -40}}, color = {255, 128, 0}, smooth = Smooth.None));
            connect(core.port_n, port_n) annotation(
              Line(points = {{60, -40}, {100, -40}, {100, -100}}, color = {255, 128, 0}, smooth = Smooth.None));
            connect(core.heatPort, heatPortCore) annotation(
              Line(points = {{40, -50}, {40, -100}}, color = {191, 0, 0}, smooth = Smooth.None));
            connect(strayReluctance.port_n, core.port_n) annotation(
              Line(points = {{80, 20}, {80, -40}, {60, -40}}, color = {255, 128, 0}, smooth = Smooth.None));
            connect(strayReluctance.port_p, electroMagneticConverter.port_p) annotation(
              Line(points = {{80, 40}, {80, 100}, {10, 100}, {10, -20}}, color = {255, 128, 0}, smooth = Smooth.None));
            annotation(
              Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, 60}, {100, -60}}, lineColor = {0, 0, 255}, pattern = LinePattern.None, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{100, -100}, {94, -100}, {84, -98}, {76, -94}, {64, -86}, {50, -72}, {42, -58}, {36, -40}, {30, -18}, {30, 0}, {30, 18}, {34, 36}, {46, 66}, {62, 84}, {78, 96}, {90, 100}, {100, 100}}, color = {255, 128, 0}), Line(points = {{40, 60}, {-100, 60}, {-100, 100}}, color = {0, 0, 255}), Line(points = {{40, -60}, {-100, -60}, {-100, -98}}, color = {0, 0, 255}), Line(points = {{40, 60}, {100, 20}, {40, -20}, {0, -20}, {-40, 0}, {0, 20}, {40, 20}, {100, -20}, {40, -60}}, color = {0, 0, 255}), Text(extent = {{0, 160}, {0, 120}}, lineColor = {0, 0, 255}, fillColor = {255, 128, 0}, fillPattern = FillPattern.Solid, textString = "%name")}),
              Documentation(info = "<html>
<p>
The symmetrical multi phase winding consists of a symmetrical winding
<a href=\"modelica://Modelica.Electrical.MultiPhase.Basic.Resistor\">resistor</a>, a
<a href=\"modelica://Modelica.Electrical.MultiPhase.Basic.ZeroInductor\">zero inductor</a> as well as a symmetrical
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.Components.MultiPhaseElectroMagneticConverter\">multi phase electromagnetic coupling</a> and a
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.Components.Reluctance\">stray reluctance</a> and a
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.Components.EddyCurrent\">core loss</a> model including
heat <a href=\"modelica://Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a\">port</a>.
</p>

<h4>See also</h4>
<p>
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.BasicMachines.Components.SinglePhaseWinding\">SinglePhaseWinding</a>,
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.BasicMachines.Components.SymmetricMultiPhaseCageWinding\">SymmetricMultiPhaseCageWinding</a>,
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.BasicMachines.Components.SaliencyCageWinding\">SaliencyCageWinding</a>
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.BasicMachines.Components.RotorSaliencyAirGap\">RotorSaliencyAirGap</a>
</p>
</html>"));
          end SymmetricMultiPhaseWinding;

          model RotorSaliencyAirGap "Air gap model with rotor saliency"
            import Modelica.Constants.pi;
            Interfaces.PositiveMagneticPort port_sp "Positive complex magnetic stator port" annotation(
              Placement(transformation(extent = {{-110, -110}, {-90, -90}}, rotation = 0)));
            Interfaces.NegativeMagneticPort port_sn "Negative complex magnetic stator port" annotation(
              Placement(transformation(extent = {{-110, 90}, {-90, 110}}, rotation = 0)));
            Interfaces.PositiveMagneticPort port_rp "Positive complex magnetic rotor port" annotation(
              Placement(transformation(extent = {{90, 90}, {110, 110}}, rotation = 0)));
            Interfaces.NegativeMagneticPort port_rn "Negative complex magnetic rotor port" annotation(
              Placement(transformation(extent = {{90, -110}, {110, -90}}, rotation = 0)));
            Modelica.Mechanics.Rotational.Interfaces.Flange_a flange_a "Flange of the rotor" annotation(
              Placement(transformation(extent = {{-10, 110}, {10, 90}}, rotation = 0)));
            Modelica.Mechanics.Rotational.Interfaces.Flange_a support "Support at which the reaction torque is acting" annotation(
              Placement(transformation(extent = {{-10, -110}, {10, -90}}, rotation = 0)));
            parameter Integer p "Number of pole pairs";
            parameter Modelica.Magnetic.FundamentalWave.Types.SalientInductance L0(d(start = 1), q(start = 1)) "Salient inductance of a single unchorded coil w.r.t. the fundamental wave";
            final parameter Modelica.Magnetic.FundamentalWave.Types.SalientReluctance R_m(d = 1 / L0.d, q = 1 / L0.q) "Reluctance of the air gap model";
            // Complex phasors of magnetic potential differences
            Modelica.SIunits.ComplexMagneticPotentialDifference V_mss "Complex magnetic potential difference of stator w.r.t. stator reference frame";
            Modelica.SIunits.ComplexMagneticPotentialDifference V_msr "Complex magnetic potential difference of stator w.r.t. rotor reference frame";
            Modelica.SIunits.ComplexMagneticPotentialDifference V_mrr "Complex magnetic potential difference of rotor w.r.t. rotor reference frame";
            // Modelica.SIunits.ComplexMagneticPotentialDifference V_mrs
            //   "Complex magnetic potential difference of rotor w.r.t. stator reference frame";
            // Complex phasors of magnetic fluxes
            Modelica.SIunits.ComplexMagneticFlux Phi_ss "Complex magnetic potential difference of stator w.r.t. stator reference frame";
            Modelica.SIunits.ComplexMagneticFlux Phi_sr "Complex magnetic potential difference of stator w.r.t. rotor reference frame";
            Modelica.SIunits.ComplexMagneticFlux Phi_rr "Complex magnetic potential difference of rotor w.r.t. rotor reference frame";
            // Modelica.SIunits.ComplexMagneticFlux Phi_rs
            //   "Complex magnetic potential difference of rotor w.r.t. stator reference frame";
            // Electrical torque and mechanical angle
            Modelica.SIunits.Torque tauElectrical "Electrical torque";
            // Modelica.SIunits.Torque tauTemp "Electrical torque";
            Modelica.SIunits.Angle gamma "Electrical angle between rotor and stator";
            Complex rotator "Equivalent vector representation of orientation";
          equation
// Stator flux into positive stator port
            port_sp.Phi = Phi_ss;
// Balance of stator flux
            port_sp.Phi + port_sn.Phi = Complex(0, 0);
// Rotor flux into positive rotor port
            port_rp.Phi = Phi_rr;
// Balance of rotor flux
            port_rp.Phi + port_rn.Phi = Complex(0, 0);
// Magneto motive force of stator
            port_sp.V_m - port_sn.V_m = V_mss;
// Magneto motive force of stator
            port_rp.V_m - port_rn.V_m = V_mrr;
// Transformation of fluxes between stator and rotor fixed frame, if wanted
// Phi_rs.re = + Phi_rr.re * cos(gamma) - Phi_rr.im * sin(gamma);
// Phi_rs.im = + Phi_rr.re * sin(gamma) + Phi_rr.im * cos(gamma);
// Alternative transformation
// Phi_rr.re = + Phi_rs.re * cos(gamma) + Phi_rs.im * sin(gamma);
// Phi_rr.im = - Phi_rs.re * sin(gamma) + Phi_rs.im * cos(gamma);
// Transformed stator flux is not needed
// Phi_sr.re = + Phi_ss.re * cos(gamma) + Phi_ss.im * sin(gamma);
// Phi_sr.im = - Phi_ss.re * sin(gamma) + Phi_ss.im * cos(gamma);
            Phi_sr = Phi_ss * Modelica.ComplexMath.conj(rotator);
// Alternative transformation
// Phi_ss.re = + Phi_sr.re * cos(gamma) - Phi_sr.im * sin(gamma);
// Phi_ss.im = + Phi_sr.re * sin(gamma) + Phi_sr.im * cos(gamma);
// Stator flux w.r.t. the rotor fixed frame and rotor flux are equal
            Phi_sr = Phi_rr;
// Transformation of magnetic potential difference between stator and rotor fixed frame
// V_mrs.re = + V_mrr.re * cos(gamma) - V_mrr.im * sin(gamma);
// V_mrs.im = + V_mrr.re * sin(gamma) + V_mrr.im * cos(gamma);
// V_mrr.re = + V_mrs.re * cos(gamma) + V_mrs.im * sin(gamma);
// V_mrr.im = - V_mrs.re * sin(gamma) + V_mrs.im * cos(gamma);
// V_msr.re = + V_mss.re * cos(gamma) + V_mss.im * sin(gamma);
// V_msr.im = - V_mss.re * sin(gamma) + V_mss.im * cos(gamma);
            V_msr = V_mss * Modelica.ComplexMath.conj(rotator);
// V_msr.re = + V_mss.re * cos(gamma) + V_mss.im * sin(gamma);
// V_msr.im = - V_mss.re * sin(gamma) + V_mss.im * cos(gamma);
// Local balance of magneto motive force
            pi / 2.0 * (V_mrr.re + V_msr.re) = Phi_rr.re * R_m.d;
            pi / 2.0 * (V_mrr.im + V_msr.im) = Phi_rr.im * R_m.q;
// Torque
            tauElectrical = -pi * p / 2.0 * (Phi_ss.im * V_mss.re - Phi_ss.re * V_mss.im);
            flange_a.tau = -tauElectrical;
            support.tau = tauElectrical;
// Electrical angle between stator and rotor
            gamma = p * (flange_a.phi - support.phi);
            rotator = Modelica.ComplexMath.exp(Complex(0, gamma));
            annotation(
              Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-100, 100}, {100, -100}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{-100, 90}, {-100, 60}, {-80, 60}}, color = {255, 128, 0}), Line(points = {{-100, -90}, {-100, -60}, {-80, -60}}, color = {255, 128, 0}), Line(points = {{40, 60}, {100, 60}, {100, 90}}, color = {255, 128, 0}), Line(points = {{40, -60}, {100, -60}, {100, -90}}, color = {255, 128, 0}), Ellipse(extent = {{-60, 80}, {60, -80}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{0, 80}, {0, 90}}, color = {0, 0, 0})}),
              Documentation(info = "<html>
<p>
This salient air gap model can be used for machines with uniform airgaps and for machines with rotor saliencies. The air gap model is not symmetrical towards stator and rotor since it is assumed the saliency always refers to the rotor. The saliency of the air gap is represented by a main field inductance in the d- and q-axis.
</p>

<p>
For the mechanical interaction of the air gap model with the stator and the rotor it is equipped with to
<a href=\"modelica://Modelica.Mechanics.Rotational.Interfaces.Flange_a\">rotational connectors</a>. The torques acting on both connectors have the same absolute values but different signs. The difference between the stator and the rotor angle,
<img src=\"modelica://Modelica/Resources/Images/Magnetic/FundamentalWave/gamma.png\">, is required for the transformation of the magnetic stator quantities to the rotor side.</p>

<p>
The air gap model has two magnetic stator and two magnetic rotor
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.Interfaces.MagneticPort\">ports</a>. The magnetic potential difference and the magnetic flux of the stator (superscript s) are transformed to the rotor fixed reference frame (superscript r). The effective reluctances of the main field with respect to the d- and q-axis are considered then in the balance equations
</p>

<p>
&nbsp;&nbsp;<img src=\"modelica://Modelica/Resources/Images/Magnetic/FundamentalWave/Machines/Components/airgap.png\">
</p>

<p>
according to the following figure.
</p>
<table border=\"0\" cellspacing=\"0\" cellpadding=\"2\">
  <caption align=\"bottom\"><b>Fig:</b> Magnetic equivalent circuit of the air gap model</caption>
  <tr>
    <td>
      <img src=\"modelica://Modelica/Resources/Images/Magnetic/FundamentalWave/Machines/Components/airgap_phasors.png\">
    </td>
  </tr>
</table>

<h4>See also</h4>
<p>
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.BasicMachines.Components.SinglePhaseWinding\">SinglePhaseWinding</a>,
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.BasicMachines.Components.SymmetricMultiPhaseWinding\">SymmetricMultiPhaseWinding</a>,
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.BasicMachines.Components.SymmetricMultiPhaseCageWinding\">SymmetricMultiPhaseCageWinding</a>
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.BasicMachines.Components.SaliencyCageWinding\">SaliencyCageWinding</a>
</p>

</html>"));
          end RotorSaliencyAirGap;

          model SymmetricMultiPhaseCageWinding "Symmetrical rotor cage"
            import Modelica.Constants.pi;
            extends Modelica.Magnetic.FundamentalWave.Interfaces.PartialTwoPort;
            parameter Integer m = 3 "Number of phases";
            parameter Boolean useHeatPort = false "Enable / disable (=fixed temperatures) thermal port" annotation(
              Evaluate = true);
            parameter Modelica.SIunits.Resistance RRef "Winding resistance per phase at TRef";
            parameter Modelica.SIunits.Temperature TRef(start = 293.15) "Reference temperature of winding";
            parameter Modelica.Electrical.Machines.Thermal.LinearTemperatureCoefficient20 alpha20(start = 0) "Temperature coefficient of winding at 20 degC";
            final parameter Modelica.SIunits.LinearTemperatureCoefficient alphaRef = Modelica.Electrical.Machines.Thermal.convertAlpha(alpha20, TRef, 293.15);
            parameter Modelica.SIunits.Temperature TOperational(start = 293.15) "Operational temperature of winding" annotation(
              Dialog(enable = not useHeatPort));
            parameter Modelica.SIunits.Inductance Lsigma "Cage stray inductance";
            parameter Real effectiveTurns = 1 "Effective number of turns";
            Modelica.SIunits.Current i[m] = strayInductor.i "Cage currents";
            Modelica.Magnetic.FundamentalWave.Components.MultiPhaseElectroMagneticConverter winding(final m = m, final effectiveTurns = fill(effectiveTurns, m), final orientation = Modelica.Electrical.MultiPhase.Functions.symmetricOrientation(m)) "Symmetric winding" annotation(
              Placement(transformation(origin = {0, -10}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
            Modelica.Electrical.MultiPhase.Basic.Inductor strayInductor(final m = m, final L = fill(Lsigma, m)) annotation(
              Placement(transformation(origin = {-20, -30}, extent = {{10, -10}, {-10, 10}}, rotation = 90)));
            Modelica.Electrical.MultiPhase.Basic.Resistor resistor(final useHeatPort = useHeatPort, final m = m, final R = fill(RRef, m), final T_ref = fill(TRef, m), final alpha = fill(alphaRef, m), final T = fill(TRef, m)) annotation(
              Placement(transformation(origin = {-20, -70}, extent = {{10, 10}, {-10, -10}}, rotation = 90)));
            Modelica.Electrical.MultiPhase.Basic.Star star(final m = m) annotation(
              Placement(transformation(extent = {{30, -30}, {50, -10}}, rotation = 0)));
            Modelica.Electrical.Analog.Basic.Ground ground annotation(
              Placement(transformation(origin = {70, -20}, extent = {{-10, 10}, {10, -10}}, rotation = 270)));
            Thermal.HeatTransfer.Interfaces.HeatPort_a heatPortWinding if useHeatPort "Heat ports of winding resistor" annotation(
              Placement(transformation(extent = {{-10, -110}, {10, -90}})));
            Thermal.HeatTransfer.Components.ThermalCollector thermalCollector(final m = m) if useHeatPort "Connector of thermal rotor resistance heat ports" annotation(
              Placement(transformation(extent = {{-50, -90}, {-30, -70}})));
            Modelica.Electrical.MultiPhase.Basic.Star starAuxiliary(final m = m) annotation(
              Placement(transformation(extent = {{30, -90}, {50, -70}}, rotation = 0)));
          equation
            connect(port_p, winding.port_p) annotation(
              Line(points = {{-100, 5.55112e-16}, {-10, 5.55112e-16}, {-10, 1.16747e-15}}, color = {255, 128, 0}));
            connect(winding.port_n, port_n) annotation(
              Line(points = {{10, -5.72459e-17}, {100, -5.72459e-17}, {100, 5.55112e-16}}, color = {255, 128, 0}));
            connect(ground.p, star.pin_n) annotation(
              Line(points = {{60, -20}, {56, -20}, {50, -20}}, color = {0, 0, 255}));
            connect(strayInductor.plug_n, resistor.plug_p) annotation(
              Line(points = {{-20, -40}, {-20, -60}}, color = {0, 0, 255}));
            connect(strayInductor.plug_p, winding.plug_p) annotation(
              Line(points = {{-20, -20}, {-10, -20}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(star.plug_p, winding.plug_n) annotation(
              Line(points = {{30, -20}, {10, -20}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(thermalCollector.port_a, resistor.heatPort) annotation(
              Line(points = {{-40, -70}, {-30, -70}}, color = {191, 0, 0}, smooth = Smooth.None));
            connect(thermalCollector.port_b, heatPortWinding) annotation(
              Line(points = {{-40, -90}, {-40, -100}, {5.55112e-16, -100}}, color = {191, 0, 0}, smooth = Smooth.None));
            connect(resistor.plug_n, starAuxiliary.plug_p) annotation(
              Line(points = {{-20, -80}, {30, -80}}, color = {0, 0, 255}, smooth = Smooth.None));
            annotation(
              Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-80, 80}, {80, -80}}, lineColor = {0, 0, 0}, fillColor = {175, 175, 175}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-20, 76}, {20, 36}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Ellipse(extent = {{28, 46}, {68, 6}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Ellipse(extent = {{28, -8}, {68, -48}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-20, -36}, {20, -76}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-68, -6}, {-28, -46}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-66, 50}, {-26, 10}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{-80, 0}, {-100, 0}}, color = {255, 128, 0}), Line(points = {{100, 0}, {80, 0}}, color = {255, 128, 0}), Text(extent = {{0, 100}, {0, 140}}, lineColor = {0, 0, 255}, textString = "%name")}),
              Documentation(info = "<html>
<p>
<img src=\"modelica://Modelica/Resources/Images/Magnetic/FundamentalWave/Machines/Components/rotorcage.png\">
</p>
<p>
The symmetric rotor cage model of this library does not consist of rotor bars and end rings. Instead the symmetric cage is modeled by an equivalent symmetrical winding. The rotor cage model consists of
<img src=\"modelica://Modelica/Resources/Images/Magnetic/FundamentalWave/m.png\"> phases. If the cage is modeled by equivalent stator winding parameters, the number of effective turns, <img src=\"modelica://Modelica/Resources/Images/Magnetic/FundamentalWave/effectiveTurns.png\">, has to be chosen equivalent to the effective number of stator turns.
</p>

<h4>See also</h4>
<p>
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.BasicMachines.Components.SinglePhaseWinding\">SinglePhaseWinding</a>,
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.BasicMachines.Components.SymmetricMultiPhaseWinding\">SymmetricMultiPhaseWinding</a>,
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.BasicMachines.Components.SaliencyCageWinding\">SaliencyCageWinding</a>,
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.BasicMachines.Components.RotorSaliencyAirGap\">RotorSaliencyAirGap</a>
</p>
</html>"));
          end SymmetricMultiPhaseCageWinding;

          model SaliencyCageWinding "Rotor cage with saliency in d- and q-axis"
            extends Modelica.Magnetic.FundamentalWave.Interfaces.PartialTwoPort;
            parameter Boolean useHeatPort = false "Enable / disable (=fixed temperatures) thermal port" annotation(
              Evaluate = true);
            parameter Modelica.Magnetic.FundamentalWave.Types.SalientResistance RRef(d(start = 1), q(start = 1)) "Salient cage resistance";
            parameter Modelica.SIunits.Temperature TRef(start = 293.15) "Reference temperature of winding";
            parameter Modelica.Electrical.Machines.Thermal.LinearTemperatureCoefficient20 alpha20(start = 0) "Temperature coefficient of winding at 20 degC";
            final parameter Modelica.SIunits.LinearTemperatureCoefficient alphaRef = Modelica.Electrical.Machines.Thermal.convertAlpha(alpha20, TRef, 293.15);
            parameter Modelica.SIunits.Temperature TOperational(start = 293.15) "Operational temperature of winding" annotation(
              Dialog(enable = not useHeatPort));
            parameter Modelica.Magnetic.FundamentalWave.Types.SalientInductance Lsigma(d(start = 1), q(start = 1)) "Salient cage stray inductance";
            parameter Real effectiveTurns = 1 "Effective number of turns";
            Modelica.Blocks.Interfaces.RealOutput i[2](each final quantity = "ElectricCurrent", each final unit = "A") = -strayInductor.i "Currents out from damper";
            Modelica.Blocks.Interfaces.RealOutput lossPower(final quantity = "Power", final unit = "W") = sum(resistor.resistor.LossPower) "Damper losses";
            Modelica.Magnetic.FundamentalWave.Components.MultiPhaseElectroMagneticConverter winding(final m = 2, final orientation = {0, Modelica.Constants.pi / 2}, final effectiveTurns = fill(effectiveTurns, 2)) "Symmetric winding" annotation(
              Placement(transformation(origin = {0, -10}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
            Modelica.Electrical.MultiPhase.Basic.Inductor strayInductor(final m = 2, final L = {Lsigma.d, Lsigma.q}) annotation(
              Placement(transformation(origin = {-20, -30}, extent = {{10, -10}, {-10, 10}}, rotation = 90)));
            Modelica.Electrical.MultiPhase.Basic.Resistor resistor(final useHeatPort = useHeatPort, final m = 2, final R = {RRef.d, RRef.q}, final T_ref = fill(TRef, 2), final alpha = fill(alphaRef, 2), final T = fill(TOperational, 2)) annotation(
              Placement(transformation(origin = {-20, -70}, extent = {{10, 10}, {-10, -10}}, rotation = 90)));
            Modelica.Electrical.MultiPhase.Basic.Star star(final m = 2) annotation(
              Placement(transformation(extent = {{30, -90}, {50, -70}}, rotation = 0)));
            Modelica.Electrical.Analog.Basic.Ground ground annotation(
              Placement(transformation(origin = {70, -80}, extent = {{-10, 10}, {10, -10}}, rotation = 270)));
            Thermal.HeatTransfer.Interfaces.HeatPort_a heatPortWinding if useHeatPort "Heat ports of winding resistor" annotation(
              Placement(transformation(extent = {{-10, -110}, {10, -90}})));
            Thermal.HeatTransfer.Components.ThermalCollector thermalCollector(final m = 2) if useHeatPort "Connector of thermal rotor resistance heat ports" annotation(
              Placement(transformation(extent = {{-50, -90}, {-30, -70}})));
          equation
            connect(port_p, winding.port_p) annotation(
              Line(points = {{-100, 0}, {-10, 0}}, color = {255, 128, 0}));
            connect(winding.port_n, port_n) annotation(
              Line(points = {{10, 0}, {100, 0}}, color = {255, 128, 0}));
            connect(ground.p, star.pin_n) annotation(
              Line(points = {{60, -80}, {50, -80}}, color = {0, 0, 255}));
            connect(strayInductor.plug_n, resistor.plug_p) annotation(
              Line(points = {{-20, -40}, {-20, -60}}, color = {0, 0, 255}));
            connect(winding.plug_n, resistor.plug_n) annotation(
              Line(points = {{10, -20}, {20, -20}, {20, -80}, {-20, -80}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(star.plug_p, winding.plug_n) annotation(
              Line(points = {{30, -80}, {20, -80}, {20, -20}, {10, -20}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(strayInductor.plug_p, winding.plug_p) annotation(
              Line(points = {{-20, -20}, {-10, -20}}, color = {0, 0, 255}, smooth = Smooth.None));
            connect(thermalCollector.port_b, heatPortWinding) annotation(
              Line(points = {{-40, -90}, {-40, -100}, {0, -100}}, color = {191, 0, 0}, smooth = Smooth.None));
            connect(resistor.heatPort, thermalCollector.port_a) annotation(
              Line(points = {{-30, -70}, {-40, -70}}, color = {191, 0, 0}, smooth = Smooth.None));
            annotation(
              Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-80, 80}, {80, -80}}, lineColor = {0, 0, 0}, fillColor = {175, 175, 175}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-20, 76}, {20, 36}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Ellipse(extent = {{28, 46}, {68, 6}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Ellipse(extent = {{28, -8}, {68, -48}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-20, -36}, {20, -76}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-68, -6}, {-28, -46}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-66, 50}, {-26, 10}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{-80, 0}, {-100, 0}}, color = {255, 128, 0}), Line(points = {{100, 0}, {80, 0}}, color = {255, 128, 0}), Text(extent = {{0, 100}, {0, 140}}, lineColor = {0, 0, 255}, textString = "%name")}),
              Documentation(info = "<html>

<p>
The salient cage model is a two axis model with two phases. The electromagnetic coupling therefore is also two phase coupling model. The angles of the two orientations are 0 and <img src=\"modelica://Modelica/Resources/Images/Magnetic/FundamentalWave/pi_over_2.png\">. This way an asymmetrical rotor cage with different resistances and stray inductances in d- and q-axis can be modeled.
</p>

<h4>See also</h4>
<p>
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.BasicMachines.Components.SinglePhaseWinding\">SinglePhaseWinding</a>,
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.BasicMachines.Components.SymmetricMultiPhaseWinding\">SymmetricMultiPhaseWinding</a>,
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.BasicMachines.Components.SymmetricMultiPhaseCageWinding\">SymmetricMultiPhaseCageWinding</a>
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.BasicMachines.Components.RotorSaliencyAirGap\">RotorSaliencyAirGap</a>
</p>
</html>"));
          end SaliencyCageWinding;

          model PermanentMagnet
            extends Modelica.Magnetic.FundamentalWave.Sources.ConstantMagneticPotentialDifference;
            extends Modelica.Electrical.Machines.Losses.InductionMachines.PermanentMagnetLosses;
            annotation(
              Documentation(info = "<html>
<p>
Simple model of a permanent magnet, containing:
</p>
<ul>
<li><a href=\"modelica://Modelica.Magnetic.FundamentalWave.Sources.ConstantMagneticPotentialDifference\">constant magnetomotive force</a></li>
<li><a href=\"modelica://Modelica.Electrical.Machines.Losses.InductionMachines.PermanentMagnetLosses\">loss model</a></li>
</ul>
</html>"));
          end PermanentMagnet;
        end Components;
        annotation(
          Documentation(info = "<html>
<p>
This package contains electric machine models and
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.BasicMachines.Components\">components</a>
for electric machines.
</p>
</html>"));
      end BasicMachines;

      package Sources "Sources"
        extends Modelica.Icons.SourcesPackage;

        model ConstantMagneticPotentialDifference "Source with constant magnetic potential difference"
          extends Modelica.Magnetic.FundamentalWave.Interfaces.PartialTwoPort;
          parameter Modelica.SIunits.ComplexMagneticPotentialDifference V_m = Complex(re = 1, im = 0) "Complex magnetic potential difference";
          Modelica.SIunits.ComplexMagneticFlux Phi "Complex magnetic flux";
        equation
// Flux into positive port
          port_p.V_m - port_n.V_m = V_m;
// Magneto motive force
          port_p.Phi = Phi;
// Local flux balance
          port_p.Phi + port_n.Phi = Complex(0, 0);
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-80, -20}, {-80, -40}}, lineColor = {255, 128, 0}, textString = "+"), Text(extent = {{80, -20}, {80, -40}}, lineColor = {255, 128, 0}, textString = "-"), Ellipse(extent = {{-50, -50}, {50, 50}}, lineColor = {255, 128, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{100, 0}, {50, 0}}, color = {255, 128, 0}), Line(points = {{-50, 0}, {-100, 0}}, color = {255, 128, 0}), Line(points = {{-50, 0}, {50, 0}}, color = {255, 128, 0}), Text(extent = {{0, -120}, {0, -80}}, textString = "%name", lineColor = {0, 0, 255})}),
            Documentation(info = "<html>
<p>
Source of constant magneto motive force.
</p>

<h4>See also</h4>
<p>
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.Sources.SignalMagneticPotentialDifference\">SignalMagneticPotentialDifference</a>,
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.Sources.ConstantFlux\">ConstantFlux</a>,
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.Sources.SignalFlux\">SignalFlux</a>
</p>
</html>"));
        end ConstantMagneticPotentialDifference;

        model SignalMagneticPotentialDifference "Source of magnetic potential difference with signal input"
          extends Modelica.Magnetic.FundamentalWave.Interfaces.PartialTwoPort;
          Modelica.ComplexBlocks.Interfaces.ComplexInput V_m "Complex signal input of magnetic potential difference" annotation(
            Placement(transformation(origin = {0, 100}, extent = {{-20, -20}, {20, 20}}, rotation = 270)));
          Modelica.SIunits.ComplexMagneticFlux Phi "Complex magnetic flux";
        equation
// Flux into positive port
          port_p.V_m - port_n.V_m = V_m;
// Magneto motive force
          port_p.Phi = Phi;
// Local flux balance
          port_p.Phi + port_n.Phi = Complex(0, 0);
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{80, -20}, {80, -40}}, lineColor = {255, 128, 0}, textString = "-"), Ellipse(extent = {{-50, -50}, {50, 50}}, lineColor = {255, 128, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{100, 0}, {50, 0}}, color = {255, 128, 0}), Line(points = {{-50, 0}, {-100, 0}}, color = {255, 128, 0}), Line(points = {{-50, 0}, {50, 0}}, color = {255, 128, 0}), Line(points = {{0, 100}, {0, 50}}, color = {255, 128, 0}), Text(extent = {{0, -120}, {0, -80}}, textString = "%name", lineColor = {0, 0, 255})}),
            Documentation(info = "<html>
<p>
Source of magneto motive force with complex signal input.
</p>

<h4>See also</h4>
<p>
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.Sources.ConstantMagneticPotentialDifference\">ConstantMagneticPotentialDifference</a>,
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.Sources.ConstantFlux\">ConstantFlux</a>,
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.Sources.SignalFlux\">SignalFlux</a>
</p>

</html>"));
        end SignalMagneticPotentialDifference;

        model ConstantFlux "Source of constant magnetic flux"
          extends Modelica.Magnetic.FundamentalWave.Interfaces.PartialTwoPort;
          Modelica.SIunits.ComplexMagneticPotentialDifference V_m "Complex magnetic potential difference";
          parameter Modelica.SIunits.ComplexMagneticFlux Phi = Complex(re = 1, im = 0) "Complex magnetic flux";
        equation
// Flux into positive port
          port_p.V_m - port_n.V_m = V_m;
// Magneto motive force
          port_p.Phi = Phi;
// Local flux balance
          port_p.Phi + port_n.Phi = Complex(0, 0);
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-50, -50}, {50, 50}}, lineColor = {255, 128, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{100, 0}, {50, 0}}, color = {255, 128, 0}), Line(points = {{-50, 0}, {-100, 0}}, color = {255, 128, 0}), Line(points = {{0, 50}, {0, -50}}, color = {255, 128, 0}), Polygon(points = {{80, 0}, {60, 6}, {60, -6}, {80, 0}}, lineColor = {255, 128, 0}, fillColor = {255, 128, 0}, fillPattern = FillPattern.Solid), Text(extent = {{0, -120}, {0, -80}}, textString = "%name", lineColor = {0, 0, 255})}),
            Documentation(info = "<html>
<p>
Source of constant magnetic flux.
</p>

<h4>See also</h4>
<p>
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.Sources.ConstantMagneticPotentialDifference\">
   ConstantMagneticPotentialDifference</a>,
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.Sources.SignalMagneticPotentialDifference\">
   SignalMagneticPotentialDifference</a>,
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.Sources.SignalFlux\">SignalFlux</a>
</p>

</html>"));
        end ConstantFlux;

        model SignalFlux "Source of constant magnetic flux"
          extends Modelica.Magnetic.FundamentalWave.Interfaces.PartialTwoPort;
          Modelica.SIunits.ComplexMagneticPotentialDifference V_m "Complex magnetic potential difference";
          Modelica.ComplexBlocks.Interfaces.ComplexInput Phi "Complex signal input of magnetic flux" annotation(
            Placement(transformation(origin = {0, 100}, extent = {{-20, -20}, {20, 20}}, rotation = 270)));
        equation
// Flux into positive port
          port_p.V_m - port_n.V_m = V_m;
// Magneto motive force
          port_p.Phi = Phi;
// Local flux balance
          port_p.Phi + port_n.Phi = Complex(0, 0);
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-50, -50}, {50, 50}}, lineColor = {255, 128, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{100, 0}, {50, 0}}, color = {255, 128, 0}), Line(points = {{-50, 0}, {-100, 0}}, color = {255, 128, 0}), Line(points = {{0, 50}, {0, -50}}, color = {255, 128, 0}), Polygon(points = {{80, 0}, {60, 6}, {60, -6}, {80, 0}}, lineColor = {255, 128, 0}, fillColor = {255, 128, 0}, fillPattern = FillPattern.Solid), Line(points = {{0, 100}, {0, 50}}, color = {255, 128, 0}), Text(extent = {{0, -120}, {0, -80}}, textString = "%name", lineColor = {0, 0, 255})}),
            Documentation(info = "<html>
<p>
Source of magnetic flux with complex signal input.
</p>

<h4>See also</h4>
<p>
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.Sources.ConstantMagneticPotentialDifference\">
   ConstantMagneticPotentialDifference</a>,
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.Sources.SignalMagneticPotentialDifference\">
   SignalMagneticPotentialDifference</a>,
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.Sources.ConstantFlux\">ConstantFlux</a>,
</p>

</html>"));
        end SignalFlux;
        annotation(
          Documentation(info = "<html>
<p>This package provides sources of magnetic potential difference and magnetic flux.</p>
</html>"));
      end Sources;

      package Sensors "Sensors to measure variables in magnetic networks"
        extends Modelica.Icons.SensorsPackage;

        model MagneticFluxSensor "Sensor to measure magnetic flux"
          extends Modelica.Icons.RotationalSensor;
          extends Modelica.Magnetic.FundamentalWave.Interfaces.PartialTwoPort;
          Modelica.SIunits.ComplexMagneticPotentialDifference V_m "Complex magnetic potential difference";
          Modelica.ComplexBlocks.Interfaces.ComplexOutput Phi "Complex magnetic flux from por_ p to port_n as output signal" annotation(
            Placement(transformation(origin = {0, -100}, extent = {{10, -10}, {-10, 10}}, rotation = 90)));
        equation
// Flux into positive port
          port_p.V_m - port_n.V_m = V_m;
// Magneto motive force
          port_p.Phi = Phi;
// Local flux balance
          port_p.Phi + port_n.Phi = Complex(0, 0);
// No magnetic potential difference at sensor
          V_m = Complex(0, 0);
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-29, -11}, {30, -70}}, lineColor = {0, 0, 0}, textString = "Phi"), Line(points = {{-72, 0}, {-90, 0}}, color = {0, 0, 0}), Text(extent = {{-140, 120}, {140, 80}}, textString = "%name", lineColor = {0, 0, 255}), Line(points = {{70, 0}, {90, 0}}, color = {0, 0, 0}), Line(points = {{0, -90}, {0, -70}})}),
            Documentation(info = "<html>
<p>Sensor for magnetic flux.</p>

<h4>See also</h4>
<p>
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.Sensors.MagneticPotentialDifferenceSensor\">MagneticPotentialDifferenceSensor</a>
</p>
</html>"));
        end MagneticFluxSensor;

        model MagneticPotentialDifferenceSensor "Sensor to measure magnetic potential difference"
          extends Modelica.Icons.RotationalSensor;
          extends Modelica.Magnetic.FundamentalWave.Interfaces.PartialTwoPort;
          Modelica.ComplexBlocks.Interfaces.ComplexOutput V_m "Complex magnetic potential difference between port_p and port_n as output signal" annotation(
            Placement(transformation(origin = {0, -100}, extent = {{10, -10}, {-10, 10}}, rotation = 90)));
          Modelica.SIunits.ComplexMagneticFlux Phi "Complex magnetic flux";
        equation
// Flux into positive port
          port_p.V_m - port_n.V_m = V_m;
// Magneto motive force
          port_p.Phi = Phi;
// Local flux balance
          port_p.Phi + port_n.Phi = Complex(0, 0);
// No magnetic flux through sensor
          Phi = Complex(0, 0);
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-52, 1}, {48, -57}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid, textString = "V_m"), Line(points = {{-70, 0}, {-90, 0}}, color = {0, 0, 0}), Line(points = {{70, 0}, {90, 0}}, color = {0, 0, 0}), Line(points = {{0, -90}, {0, -70}}), Text(extent = {{-140, 120}, {140, 80}}, textString = "%name", lineColor = {0, 0, 255})}),
            Documentation(info = "<html>
<p>Sensor for magnetic potential difference.</p>

<h4>See also</h4>
<p>
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.Sensors.MagneticFluxSensor\">MagneticFluxSensor</a>
</p></html>"));
        end MagneticPotentialDifferenceSensor;

        model MagneticPotentialSensor "Sensor to measure magnetic potential"
          extends Modelica.Icons.RotationalSensor;
          Modelica.ComplexBlocks.Interfaces.ComplexOutput V_m "Complex magnetic potential as output signal" annotation(
            Placement(transformation(origin = {0, -100}, extent = {{10, -10}, {-10, 10}}, rotation = 90)));
          Interfaces.PositiveMagneticPort port_p "Magnetic connector of sensor" annotation(
            Placement(transformation(extent = {{-110, -10}, {-90, 10}})));
        equation
// No magnetic flux through sensor
          port_p.Phi = Complex(0, 0);
// Magnetic potential
          V_m = port_p.V_m;
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-52, 1}, {48, -57}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid, textString = "V_m"), Line(points = {{-70, 0}, {-90, 0}}, color = {0, 0, 0}), Line(points = {{0, -90}, {0, -70}}), Text(extent = {{-140, 120}, {140, 80}}, textString = "%name", lineColor = {0, 0, 255})}),
            Documentation(info = "<html>
<p>Sensor for magnetic potential difference.</p>

<h4>See also</h4>
<p>
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.Sensors.MagneticFluxSensor\">MagneticFluxSensor</a>
</p></html>"));
        end MagneticPotentialSensor;
        annotation(
          Documentation(info = "<html>
<p>
This package provides sensors for the magnetic potential difference and the magnetic flux in magnetic circuit.
</p>
</html>"));
      end Sensors;

      package Interfaces "Interfaces and partial models"
        extends Modelica.Icons.InterfacesPackage;

        connector MagneticPort "Complex magnetic port"
          Modelica.SIunits.ComplexMagneticPotentialDifference V_m "Complex magnetic potential difference";
          flow Modelica.SIunits.ComplexMagneticFlux Phi "Complex magnetic flux";
          annotation(
            Documentation(info = "<html>
<p>
The potential quantity of the magnetic port is the complex magnetic potential difference <img src=\"modelica://Modelica/Resources/Images/Magnetic/FundamentalWave/V_m.png\">. The corresponding flow quantity is the magnetic flux <img src=\"modelica://Modelica/Resources/Images/Magnetic/FundamentalWave/Phi.png\">.
</p>

<h4>See also</h4>
<p>
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.Interfaces.PositiveMagneticPort\">PositiveMagneticPort</a>,
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.Interfaces.NegativeMagneticPort\">NegativeMagneticPort</a>
</p>

</html>"));
        end MagneticPort;

        connector NegativeMagneticPort "Negative complex magnetic port"
          extends Modelica.Magnetic.FundamentalWave.Interfaces.MagneticPort;
          annotation(
            defaultComponentName = "port_n",
            Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-100, 100}, {100, -100}}, lineColor = {255, 128, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid)}),
            Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-60, 100}, {-60, 60}}, lineColor = {255, 128, 0}, textString = "%name"), Ellipse(extent = {{-50, 50}, {50, -50}}, lineColor = {255, 128, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid)}),
            Documentation(info = "<html>
<p>
Negative magnetic <a href=\"modelica://Modelica.Magnetic.FundamentalWave.Interfaces.MagneticPort\">port</a>.
</p>

<h4>See also</h4>
<p>
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.Interfaces.MagneticPort\">MagneticPort</a>,
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.Interfaces.PositiveMagneticPort\">PositiveMagneticPort</a>
</p>
</html>"));
        end NegativeMagneticPort;

        connector PositiveMagneticPort "Positive complex magnetic port"
          extends Modelica.Magnetic.FundamentalWave.Interfaces.MagneticPort;
          annotation(
            defaultComponentName = "port_p",
            Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-100, 100}, {100, -100}}, lineColor = {255, 128, 0}, fillColor = {255, 128, 0}, fillPattern = FillPattern.Solid)}),
            Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-60, 100}, {-60, 60}}, lineColor = {255, 128, 0}, textString = "%name"), Ellipse(extent = {{-50, 50}, {50, -50}}, lineColor = {255, 128, 0}, fillColor = {255, 128, 0}, fillPattern = FillPattern.Solid)}),
            Documentation(info = "<html>
<p>
Positive magnetic <a href=\"modelica://Modelica.Magnetic.FundamentalWave.Interfaces.MagneticPort\">port</a>.
</p>

<h4>See also</h4>
<p>
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.Interfaces.MagneticPort\">MagneticPort</a>,
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.Interfaces.NegativeMagneticPort\">NegativeMagneticPort</a>
</p>

</html>"));
        end PositiveMagneticPort;

        partial model PartialTwoPort "Two magnetic ports for graphical modeling"
          PositiveMagneticPort port_p "Positive complex magnetic port" annotation(
            Placement(transformation(extent = {{-110, -10}, {-90, 10}}, rotation = 0)));
          NegativeMagneticPort port_n "Negative complex magnetic port" annotation(
            Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
          annotation(
            Documentation(info = "<html>
<p>
This magnetic two port element consists of a
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.Interfaces.PositiveMagneticPort\">positive</a> and a
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.Interfaces.NegativeMagneticPort\">negative magnetic port</a> and
considers the flux balance of the two ports. Additionally the magnetic potential difference (of the positive and the negative port) and the magnetic flux (into the positive magnetic port) are defined. This model is mainly to used to extend from in order build more complex - graphical - models.
</p>

<h4>See also</h4>
<p>
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.Interfaces.PositiveMagneticPort\">PositiveMagneticPort</a>,
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.Interfaces.NegativeMagneticPort\">NegativeMagneticPort</a>,
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.Interfaces.PartialTwoPortElementary\">PartialTwoPortElementary</a>
</p></html>"));
        end PartialTwoPort;

        partial model PartialTwoPortElementary "Two magnetic ports for textual modeling"
          extends Modelica.Magnetic.FundamentalWave.Interfaces.PartialTwoPort;
          Modelica.SIunits.ComplexMagneticPotentialDifference V_m "Complex magnetic potential difference";
          Modelica.SIunits.ComplexMagneticFlux Phi "Complex magnetic flux";
        equation
// Flux into positive port
          V_m = port_p.V_m - port_n.V_m;
// Magneto motive force
          port_p.Phi = Phi;
// Local flux balance
          port_p.Phi + port_n.Phi = Complex(0, 0);
          annotation(
            Documentation(info = "<html>
<p>
This magnetic two port element only consists of a
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.Interfaces.PositiveMagneticPort\">positive</a> and a
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.Interfaces.NegativeMagneticPort\">negative magnetic port</a>.
This model is mainly used to extend from in order build more complex - equation based - models.
</p>

<h4>See also</h4>
<p>
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.Interfaces.PositiveMagneticPort\">PositiveMagneticPort</a>,
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.Interfaces.NegativeMagneticPort\">NegativeMagneticPort</a>,
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.Interfaces.PartialTwoPort\">PartialTwoPort</a>
</p>
</html>"));
        end PartialTwoPortElementary;

        partial model PartialBasicInductionMachine "Partial model for induction machine"
          extends Modelica.Electrical.Machines.Icons.FundamentalWaveMachine;
          constant Modelica.SIunits.Angle pi = Modelica.Constants.pi;
          parameter Integer m(min = 3) = 3 "Number of stator phases";
          // Mechanical parameters
          parameter Modelica.SIunits.Inertia Jr(start = 0.29) "Rotor inertia";
          parameter Boolean useSupport = false "Enable / disable (=fixed stator) support" annotation(
            Evaluate = true);
          parameter Modelica.SIunits.Inertia Js(start = Jr) "Stator inertia" annotation(
            Dialog(enable = useSupport));
          parameter Boolean useThermalPort = false "Enable / disable (=fixed temperatures) thermal port" annotation(
            Evaluate = true);
          parameter Integer p(min = 1, start = 2) "Number of pole pairs (Integer)";
          parameter Modelica.SIunits.Frequency fsNominal(start = 50) "Nominal frequency";
          parameter Modelica.SIunits.Temperature TsOperational(start = 293.15) "Operational temperature of stator resistance" annotation(
            Dialog(group = "Operational temperatures", enable = not useThermalPort));
          parameter Modelica.SIunits.Resistance Rs(start = 0.03) "Stator resistance per phase at TRef" annotation(
            Dialog(tab = "Nominal resistances and inductances"));
          parameter Modelica.SIunits.Temperature TsRef(start = 293.15) "Reference temperature of stator resistance" annotation(
            Dialog(tab = "Nominal resistances and inductances"));
          parameter Modelica.Electrical.Machines.Thermal.LinearTemperatureCoefficient20 alpha20s(start = 0) "Temperature coefficient of stator resistance at 20 degC" annotation(
            Dialog(tab = "Nominal resistances and inductances"));
          parameter Real effectiveStatorTurns = 1 "Effective number of stator turns";
          parameter Modelica.SIunits.Inductance Lssigma(start = 3 * (1 - sqrt(1 - 0.0667)) / (2 * pi * fsNominal)) "Stator stray inductance per phase" annotation(
            Dialog(tab = "Nominal resistances and inductances"));
          parameter Modelica.SIunits.Inductance Lszero = Lssigma "Stator zero inductance" annotation(
            Dialog(tab = "Nominal resistances and inductances"));
          parameter Modelica.Magnetic.FundamentalWave.Types.SalientInductance L0(d(start = 1), q(start = 1)) "Salient inductance of an unchorded coil" annotation(
            Dialog(tab = "Nominal resistances and inductances"));
          parameter Modelica.Electrical.Machines.Losses.FrictionParameters frictionParameters(wRef = 2 * pi * fsNominal / p) "Friction loss parameter record" annotation(
            Dialog(tab = "Losses"));
          parameter Modelica.Electrical.Machines.Losses.CoreParameters statorCoreParameters(final m = 3, wRef = 2 * pi * fsNominal / p, VRef(start = 100)) "Stator core loss parameter record; all parameters refer to stator side" annotation(
            Dialog(tab = "Losses"));
          parameter Modelica.Electrical.Machines.Losses.StrayLoadParameters strayLoadParameters(IRef(start = 100), wRef = 2 * pi * fsNominal / p) "Stray load loss parameter record" annotation(
            Dialog(tab = "Losses"));
          // Mechanical quantities
          output Modelica.SIunits.Angle phiMechanical(start = 0) = flange.phi - internalSupport.phi "Mechanical angle of rotor against stator";
          output Modelica.SIunits.AngularVelocity wMechanical(start = 0, displayUnit = "1/min") = der(phiMechanical) "Mechanical angular velocity of rotor against stator";
          output Modelica.SIunits.Torque tauElectrical = inertiaRotor.flange_a.tau "Electromagnetic torque";
          output Modelica.SIunits.Torque tauShaft = -flange.tau "Shaft torque";
          replaceable output Modelica.Electrical.Machines.Interfaces.InductionMachines.PartialPowerBalanceInductionMachines powerBalance(final powerStator = Modelica.Electrical.MultiPhase.Functions.activePower(vs, is), final powerMechanical = wMechanical * tauShaft, final powerInertiaStator = inertiaStator.J * inertiaStator.a * inertiaStator.w, final powerInertiaRotor = inertiaRotor.J * inertiaRotor.a * inertiaRotor.w, final lossPowerStatorWinding = sum(stator.resistor.resistor.LossPower), final lossPowerStatorCore = stator.core.lossPower, final lossPowerStrayLoad = strayLoad.lossPower, final lossPowerFriction = friction.lossPower) "Power balance";
          // Stator voltages and currents
          output Modelica.SIunits.Voltage vs[m] = plug_sp.pin.v - plug_sn.pin.v "Stator instantaneous voltages";
          output Modelica.SIunits.Current is[m] = plug_sp.pin.i "Stator instantaneous currents";
          Modelica.Mechanics.Rotational.Interfaces.Flange_a flange "Shaft" annotation(
            Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
          Modelica.Mechanics.Rotational.Components.Inertia inertiaRotor(final J = Jr) annotation(
            Placement(transformation(origin = {80, 0}, extent = {{10, 10}, {-10, -10}}, rotation = 180)));
          Modelica.Mechanics.Rotational.Interfaces.Flange_a support if useSupport "Support at which the reaction torque is acting" annotation(
            Placement(transformation(extent = {{90, -110}, {110, -90}}, rotation = 0)));
          Modelica.Mechanics.Rotational.Components.Inertia inertiaStator(final J = Js) annotation(
            Placement(transformation(origin = {80, -100}, extent = {{10, 10}, {-10, -10}}, rotation = 180)));
          Modelica.Mechanics.Rotational.Components.Fixed fixed if not useSupport annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 180, origin = {70, -90})));
          Modelica.Electrical.MultiPhase.Interfaces.PositivePlug plug_sp(final m = m) "Positive plug of stator" annotation(
            Placement(transformation(extent = {{50, 90}, {70, 110}}, rotation = 0)));
          Modelica.Electrical.MultiPhase.Interfaces.NegativePlug plug_sn(final m = m) "Negative plug of stator" annotation(
            Placement(transformation(extent = {{-70, 90}, {-50, 110}}, rotation = 0)));
          BasicMachines.Components.SymmetricMultiPhaseWinding stator(final useHeatPort = true, final m = m, final RRef = Rs, final TRef = TsRef, final Lsigma = Lssigma, final effectiveTurns = effectiveStatorTurns, final TOperational = TsOperational, final GcRef = statorCoreParameters.GcRef, final alpha20 = alpha20s, final Lzero = Lszero) "Symmetric stator winding including resistances, zero and stray inductances and core losses" annotation(
            Placement(transformation(origin = {0, 40}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          replaceable Modelica.Electrical.Machines.Interfaces.InductionMachines.PartialThermalAmbientInductionMachines thermalAmbient(final useTemperatureInputs = false, final Ts = TsOperational, final m = m) if not useThermalPort annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 270, origin = {-70, -90})));
          replaceable Modelica.Electrical.Machines.Interfaces.InductionMachines.PartialThermalPortInductionMachines thermalPort(final m = m) if useThermalPort "Thermal port of induction machines" annotation(
            Placement(transformation(extent = {{-10, -110}, {10, -90}})));
          Modelica.Magnetic.FundamentalWave.Components.Ground groundS "Ground of stator magnetic circuit" annotation(
            Placement(transformation(extent = {{-40, 30}, {-20, 10}}, rotation = 0)));
          Modelica.Magnetic.FundamentalWave.BasicMachines.Components.RotorSaliencyAirGap airGap(final p = p, final L0 = L0) annotation(
            Placement(transformation(origin = {0, 0}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          Modelica.Magnetic.FundamentalWave.Components.Ground groundR "Ground of rotor magnetic circuit" annotation(
            Placement(transformation(extent = {{-40, -30}, {-20, -10}}, rotation = 0)));
          /* previously used: state selection, now commented
            FundamentalWave.Interfaces.StateSelector stateSelectorS(
              final m=m,
              final xi=is,
              final gamma=p*phiMechanical) "State selection of stator currents"
              annotation (Placement(transformation(extent={{-10,80},{10,100}})));
            */
          Electrical.Machines.Losses.InductionMachines.StrayLoad strayLoad(final strayLoadParameters = strayLoadParameters, final useHeatPort = true, final m = m) annotation(
            Placement(transformation(extent = {{60, 60}, {40, 80}})));
          Electrical.Machines.Losses.Friction friction(final frictionParameters = frictionParameters, final useHeatPort = true) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0, origin = {90, -30})));
        protected
          replaceable Modelica.Electrical.Machines.Interfaces.InductionMachines.PartialThermalPortInductionMachines internalThermalPort(final m = m) annotation(
            Placement(transformation(extent = {{-44, -94}, {-36, -86}})));
          Modelica.Mechanics.Rotational.Interfaces.Support internalSupport annotation(
            Placement(transformation(extent = {{56, -104}, {64, -96}}, rotation = 0)));
        initial algorithm
          assert(not Modelica.Math.isPowerOf2(m), String(m) + " phases are currently not supported in this version of FundametalWave");
        equation
          connect(stator.plug_n, plug_sn) annotation(
            Line(points = {{-10, 50}, {-10, 70}, {-60, 70}, {-60, 100}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(thermalPort, internalThermalPort) annotation(
            Line(points = {{5.55112e-16, -100}, {5.55112e-16, -90}, {-40, -90}}, color = {199, 0, 0}, smooth = Smooth.None));
          connect(thermalAmbient.thermalPort, internalThermalPort) annotation(
            Line(points = {{-60, -90}, {-40, -90}}, color = {199, 0, 0}, smooth = Smooth.None));
          connect(inertiaRotor.flange_b, flange) annotation(
            Line(points = {{90, -1.72421e-15}, {100, -1.72421e-15}, {100, 5.55112e-16}}, color = {0, 0, 0}));
          connect(internalSupport, inertiaStator.flange_a) annotation(
            Line(points = {{60, -100}, {70, -100}}, color = {0, 0, 0}, smooth = Smooth.None));
          connect(internalSupport, fixed.flange) annotation(
            Line(points = {{60, -100}, {60, -90}, {70, -90}}, color = {0, 0, 0}, smooth = Smooth.None));
          connect(inertiaStator.flange_b, support) annotation(
            Line(points = {{90, -100}, {90, -100}, {100, -100}}, color = {0, 0, 0}));
          connect(airGap.flange_a, inertiaRotor.flange_a) annotation(
            Line(points = {{10, -1.33731e-15}, {25, -1.33731e-15}, {25, -6.12304e-16}, {40, -6.12304e-16}, {40, 7.25006e-16}, {70, 7.25006e-16}}, color = {0, 0, 0}, smooth = Smooth.None));
          connect(airGap.support, internalSupport) annotation(
            Line(points = {{-10, 2.33651e-15}, {-50, 2.33651e-15}, {-50, -70}, {60, -70}, {60, -100}}, color = {0, 0, 0}, smooth = Smooth.None));
          connect(groundR.port_p, airGap.port_rn) annotation(
            Line(points = {{-30, -10}, {-10, -10}}, color = {255, 128, 0}));
          connect(stator.plug_p, strayLoad.plug_n) annotation(
            Line(points = {{10, 50}, {10, 70}, {40, 70}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(plug_sp, strayLoad.plug_p) annotation(
            Line(points = {{60, 100}, {60, 70}}, color = {0, 0, 255}, smooth = Smooth.None));
          connect(strayLoad.support, internalSupport) annotation(
            Line(points = {{50, 60}, {50, 50}, {60, 50}, {60, -100}}, color = {0, 0, 0}, smooth = Smooth.None));
          connect(strayLoad.heatPort, internalThermalPort.heatPortStrayLoad) annotation(
            Line(points = {{60, 60}, {60, 50}, {50, 50}, {50, -80}, {-40, -80}, {-40, -90}}, color = {191, 0, 0}, smooth = Smooth.None));
          connect(friction.support, internalSupport) annotation(
            Line(points = {{90, -40}, {90, -70}, {60, -70}, {60, -100}}, color = {0, 0, 0}, smooth = Smooth.None));
          connect(strayLoad.flange, inertiaRotor.flange_b) annotation(
            Line(points = {{50, 80}, {90, 80}, {90, -1.72421e-15}}, color = {0, 0, 0}, smooth = Smooth.None));
          connect(friction.flange, inertiaRotor.flange_b) annotation(
            Line(points = {{90, -20}, {90, -1.72421e-15}}, color = {0, 0, 0}, smooth = Smooth.None));
          connect(friction.heatPort, internalThermalPort.heatPortFriction) annotation(
            Line(points = {{80, -40}, {50, -40}, {50, -80}, {-40, -80}, {-40, -90}}, color = {191, 0, 0}, smooth = Smooth.None));
          connect(groundS.port_p, airGap.port_sp) annotation(
            Line(points = {{-30, 10}, {-10, 10}}, color = {255, 128, 0}, smooth = Smooth.None));
          connect(stator.port_n, airGap.port_sp) annotation(
            Line(points = {{-10, 30}, {-10, 10}}, color = {255, 128, 0}, smooth = Smooth.None));
          connect(stator.port_p, airGap.port_sn) annotation(
            Line(points = {{10, 30}, {10, 10}}, color = {255, 128, 0}, smooth = Smooth.None));
          connect(stator.heatPortWinding, internalThermalPort.heatPortStatorWinding) annotation(
            Line(points = {{-10, 44}, {-40, 44}, {-40, -90}}, color = {191, 0, 0}, smooth = Smooth.None));
          connect(stator.heatPortCore, internalThermalPort.heatPortStatorCore) annotation(
            Line(points = {{-10, 36}, {-40, 36}, {-40, -90}}, color = {191, 0, 0}, smooth = Smooth.None));
          annotation(
            Documentation(info = "<HTML>
<p>This partial model for induction machines contains elements common in all machine models.</p>
</HTML>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{80, -80}, {120, -120}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-50, 100}, {-20, 100}, {-20, 70}}, color = {0, 0, 255}), Line(points = {{50, 100}, {20, 100}, {20, 70}}, color = {0, 0, 255}), Text(extent = {{-150, -120}, {150, -180}}, lineColor = {0, 0, 255}, textString = "%name"), Line(visible = not useSupport, points = {{80, -100}, {120, -100}}, color = {0, 0, 0}, smooth = Smooth.None), Line(visible = not useSupport, points = {{90, -100}, {80, -120}}, color = {0, 0, 0}, smooth = Smooth.None), Line(visible = not useSupport, points = {{100, -100}, {90, -120}}, color = {0, 0, 0}, smooth = Smooth.None), Line(visible = not useSupport, points = {{110, -100}, {100, -120}}, color = {0, 0, 0}, smooth = Smooth.None), Line(visible = not useSupport, points = {{120, -100}, {110, -120}}, color = {0, 0, 0}, smooth = Smooth.None)}));
        end PartialBasicInductionMachine;

        model StateSelector "Transform instantaneous values to space phasors and select states"
          import Modelica.Constants.pi;
          parameter Integer m(min = 3) = 3 "Number of phases";
          input Real xi[m](each stateSelect = StateSelect.avoid) "Instantaneous values" annotation(
            Dialog);
          input Modelica.SIunits.Angle gamma "Angle of rotation" annotation(
            Dialog);
          parameter StateSelect x0StateSelect = StateSelect.prefer "Priority to use zero systems as states";
          parameter StateSelect xrStateSelect = StateSelect.prefer "Priority to use space phasors w.r.t. rotating frame as states";
          Real x0(stateSelect = x0StateSelect) = 1 / sqrt(m) * sum(xi) "Zero system";
          Real x00(stateSelect = x0StateSelect) = 1 / sqrt(m) * sum({xi[2 * l - 1] - xi[2 * l] for l in 1:integer(m / 2)}) if m == 2 * integer(m / 2) "Second zero system, if present (mp even)";
          final parameter Integer np = integer((m - 1) / 2) "Number of space phasors";
          Complex xf[np](each re(stateSelect = StateSelect.avoid), each im(stateSelect = StateSelect.avoid)) "Space phasors w.r.t. fixed frame";
          Complex xr[np](each re(stateSelect = xrStateSelect), each im(stateSelect = xrStateSelect)) "Space phasors w.r.t. rotating frame";
        equation
//space phasor transformations
          for k in 1:np loop
            xf[k].re = 1 / sqrt(m) * sum({cos(k * (l - 1) * 2 * pi / m) * xi[l] for l in 1:m});
            xf[k].im = 1 / sqrt(m) * sum({sin(k * (l - 1) * 2 * pi / m) * xi[l] for l in 1:m});
            xr[k] = xf[k] * Modelica.ComplexMath.conj(Modelica.ComplexMath.exp(Complex(0, gamma)));
          end for;
          annotation(
            Documentation(info = "<html>
<p>
Transforms instantaneous values into space phasors and zero system currents,
rotates space phasors and sets stateSelect modifiers in order to choose states w.r.t. rotating frame,
i.e., with small derivatives.
</p>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-60, 60}, {60, -60}}, lineColor = {170, 213, 255}, fillColor = {170, 213, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-60, 60}, {60, -60}}, textString = "S", lineColor = {0, 0, 255}, fillColor = {255, 170, 85}, fillPattern = FillPattern.Solid), Text(extent = {{0, -60}, {0, -100}}, lineColor = {0, 0, 255}, textString = "%name")}));
        end StateSelector;
        annotation(
          Documentation(info = "<html>
<p>
This package contains interface definitions of the magnetic ports as well as partial models.
</p>
</html>"));
      end Interfaces;

      package Types "Definition of salient types"
        extends Modelica.Icons.TypesPackage;

        record Salient "Base record of saliency with d and q component"
          replaceable Real d "Component of d (direct) axis, aligned to real part";
          replaceable Real q "Component of q (quadrature) axis, aligned to imaginary part";
          annotation(
            Documentation(info = "<html>
<p>
Definition of saliency with respect to the orthogonal d- and q-axis. Saliency, however, refers to different properties in d- and q-axis and thus considers the anisotropic behavior.
</p>

<h4>See also</h4>
<p>
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.Types.SalientCurrent\">SalientCurrent</a>,
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.Types.SalientVoltage\">SalientVoltage</a>,
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.Types.SalientInductance\">SalientInductance</a>,
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.Types.SalientReluctance\">SalientReluctance</a>,
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.Types.SalientResistance\">SalientResistance</a>
</p>

</html>"));
        end Salient;

        record SalientCurrent = Salient(redeclare Modelica.SIunits.Current d, redeclare Modelica.SIunits.Current q) "Salient current" annotation(
          Documentation(info = "<html>
<p>Type representing the d- and q-axis of a current phasor.</p>

<h4>See also</h4>
<p>
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.Types.Salient\">Salient</a>,
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.Types.SalientVoltage\">SalientVoltage</a>,
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.Types.SalientInductance\">SalientInductance</a>,
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.Types.SalientReluctance\">SalientReluctance</a>,
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.Types.SalientResistance\">SalientResistance</a>
</p>

</html>"));
        record SalientVoltage = Salient(redeclare Modelica.SIunits.Voltage d, redeclare Modelica.SIunits.Voltage q) "Salient voltage" annotation(
          Documentation(info = "<html>
<p>Type representing the d- and q-axis of a voltage phasor.</p>

<h4>See also</h4>
<p>
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.Types.Salient\">Salient</a>,
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.Types.SalientCurrent\">SalientCurrent</a>,
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.Types.SalientInductance\">SalientInductance</a>,
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.Types.SalientReluctance\">SalientReluctance</a>,
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.Types.SalientResistance\">SalientResistance</a>
</p>

</html>"));
        record SalientInductance = Salient(redeclare Modelica.SIunits.Inductance d, redeclare Modelica.SIunits.Inductance q) "Salient inductance" annotation(
          Documentation(info = "<html>
<p>Type representing the d- and q-axis of an inductance with respect to the fundamental wave.</p>

<h4>See also</h4>
<p>
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.Types.Salient\">Salient</a>,
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.Types.SalientCurrent\">SalientCurrent</a>,
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.Types.SalientVoltage\">SalientVoltage</a>,
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.Types.SalientReluctance\">SalientReluctance</a>,
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.Types.SalientResistance\">SalientResistance</a>
</p>

</html>"));
        record SalientReluctance = Salient(redeclare Modelica.SIunits.Reluctance d, redeclare Modelica.SIunits.Reluctance q) "Salient reluctance" annotation(
          Documentation(info = "<html>
<p>Type representing the d- and q-axis of an reluctance with respect to the fundamental wave.</p>

<h4>See also</h4>
<p>
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.Types.Salient\">Salient</a>,
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.Types.SalientCurrent\">SalientCurrent</a>,
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.Types.SalientVoltage\">SalientVoltage</a>,
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.Types.SalientInductance\">SalientInductance</a>,
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.Types.SalientResistance\">SalientResistance</a>
</p>

</html>"));
        record SalientResistance = Salient(redeclare Modelica.SIunits.Resistance d, redeclare Modelica.SIunits.Resistance q) "Salient resistance" annotation(
          Documentation(info = "<html>
<p>Type representing the d- and q-axis of an resistance with respect to the fundamental wave.</p>

<h4>See also</h4>
<p>
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.Types.Salient\">Salient</a>,
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.Types.SalientCurrent\">SalientCurrent</a>,
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.Types.SalientVoltage\">SalientVoltage</a>,
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.Types.SalientInductance\">SalientInductance</a>,
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.Types.SalientReluctance\">SalientReluctance</a>
</p>

</html>"));
        annotation(
          Documentation(info = "<html>
<p>This package provides types for modeling anisotropic saliency effects in electric machines. These saliencies are usually considered by a d- (direct) and q-axis (quadrature) components in the respective axis (of the rotor).</p>
</html>"));
      end Types;
      annotation(
        preferredView = "info",
        Documentation(revisions = "<html>

<p>The table below refers to internal version numbers of the FundamentalWave library. A more detailed list of changes is summarized in the <a href=\"modelica://Modelica.Magnetic.FundamentalWave.UsersGuide.ReleaseNotes\">release notes</a>.</p>


<table border=\"1\" rules=\"groups\" cellpadding=\"2\">
<thead>
<tr><td>Version</td> <td>Revision</td> <td>Date</td> <td>Authors</td> <td>Comments</td></tr>
</thead>
<tbody>
<tr><td>3.2.1</td><td></td>  <td>2013-07-31</td>  <td>C. Kral<br>A. Haumer</td>  <td>New component examples, added variables in multi phase converter models, bug fixes #1216, #1223 and #1226</td></tr>
<tr><td>2.0.0</td><td>6083</td>  <td>2013-03-10</td>  <td>C. Kral<br>A. Haumer</td>  <td>Extended machines models for phase numbers greater or equal than three</td></tr>
<tr><td>1.7.3</td><td>6029</td>  <td>2013-02-25</td>  <td>C. Kral</td>  <td>Corrected wrong parameter description</td></tr>
<tr><td>1.7.2</td><td>4621</td>  <td>2011-06-28</td>  <td>C. Kral<br>A. Haumer</td>  <td>Corrected bug in parameterization of symmetrical multi phase winding model<br>Necessary adaptions due to conditional heatPorts of loss models (backwards compatibility)</td></tr>
<tr><td>1.7.1</td><td>4170</td>  <td>2010-09-13</td>  <td>C. Kral</td>  <td>Corrected bug in partial one port models</td></tr>
<tr><td>1.7.0</td><td>3899</td>  <td>2010-05-31</td>  <td>C. Kral<br>A. Haumer</td>  <td>Changed single phase and symmetrical multi phase winding model<br>Relocated core losses</td></tr>
<tr><td>1.6.0</td><td>3837</td>  <td>2010-05-05</td>  <td>C. Kral</td>  <td>Renamed all parameters windingAngle to orientation<br>Update due to changed class names in Machines.Icons<br>Exchanged positive and negative stator ports of air gap model</td></tr>
<tr><td>1.5.0</td><td>3802</td>  <td>2010-04-28</td>  <td>C. Kral</td>  <td>Added stator core, friction, stray load and brush loss models and changed parameter of EddyCurrent model</td></tr>
<tr><td>1.4.0</td><td>3763</td>  <td>2010-04-22</td>  <td>C. Kral</td>  <td>Added eddy current loss model with thermal heat port</td></tr>
<tr><td>1.3.0</td><td></td>  <td>2010-02-26</td>  <td>A. Haumer<br>C. Kral</td>  <td>New state selection, icons and copyright included</td></tr>
<tr><td>1.2.0</td><td>3468</td>   <td>2010-02-17</td>  <td>C. Kral</td>  <td>Renamed Machines to BasicMachines and updated references to Electrical.Machines</td></tr>
<tr><td>1.1.0</td><td>3424</td>  <td>2010-02-15</td>  <td>C. Kral</td>  <td>Added thermal connectors and temperature dependent resistances</td></tr>
<tr><td>1.0.0</td><td>3426</td>  <td>2010-02-04</td>  <td>C. Kral</td>  <td>Integrated the library into the MSL</td></tr>
<tr><td>0.4.0</td> <td></td>  <td>2009-10-29</td>  <td>C. Kral<br>A.&nbsp;Haumer</td>  <td>Corrected bug in magnetic potential calculation</td></tr>
<tr><td>0.3.0</td> <td></td>  <td>2009-10-28</td>  <td>C. Kral</td>  <td>Renamed number of turns and winding angles</td></tr>
<tr><td>0.2.0</td> <td></td>  <td>2009-10-20</td>  <td>C. Kral</td>  <td>Added idle model</td></tr>
<tr><td>0.1.0</td> <td></td>  <td>2009-07-22</td>  <td>C. Kral</td>  <td>First version based on the concept of the FluxTubes library and the Magnetics library of Michael Beuschel</td></tr>
</tbody>
</table>
</html>", info = "<html>
<p>
Copyright &copy; 2009-2015, <a href=\"modelica://Modelica.Magnetic.FundamentalWave.UsersGuide.Contact\">Christian Kral</a> and
<a href=\"modelica://Modelica.Magnetic.FundamentalWave.UsersGuide.Contact\">Anton Haumer</a>
</p>
<p>
<i>This Modelica package is <u>free</u> software and the use is completely at <u>your own risk</u>; it can be redistributed and/or modified under the terms of the Modelica License 2. For license conditions (including the disclaimer of warranty) see <a href=\"modelica://Modelica.UsersGuide.ModelicaLicense2\">Modelica.UsersGuide.ModelicaLicense2</a> or visit <a href=\"https://www.modelica.org/licenses/ModelicaLicense2\"> https://www.modelica.org/licenses/ModelicaLicense2</a>.</i>
</p>
</html>"));
    end FundamentalWave;
    annotation(
      Documentation(info = "<html>
<p>
This library contains magnetic components to build especially
electro-magnetic devices.
</p>

</html>"),
      Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100.0, -100.0}, {100.0, 100.0}}), graphics = {Polygon(origin = {-3.75, 0.0}, fillColor = {160, 160, 164}, fillPattern = FillPattern.Solid, points = {{33.75, 50.0}, {-46.25, 50.0}, {-46.25, -50.0}, {33.75, -50.0}, {33.75, -30.0}, {-21.25, -30.0}, {-21.25, 30.0}, {33.75, 30.0}}), Ellipse(origin = {10.4708, 41.6771}, extent = {{-86.0, -24.0}, {-78.0, -16.0}}), Line(origin = {10.4708, 41.6771}, points = {{-64.0, -20.0}, {-78.0, -20.0}}), Line(origin = {10.4708, 41.6771}, points = {{-64.1812, -31.6229}, {-32.0, -40.0}}), Line(origin = {10.4708, 41.6771}, points = {{-64.0, -20.0}, {-32.0, -28.0}}), Ellipse(origin = {10.4708, 41.6771}, extent = {{-86.0, -60.0}, {-78.0, -52.0}}), Line(origin = {10.4708, 41.6771}, points = {{-64.0, -56.0}, {-78.0, -56.0}}), Line(origin = {10.4708, 41.6771}, points = {{-64.0, -44.0}, {-32.0, -52.0}}), Line(origin = {10.4708, 41.6771}, points = {{-64.0, -56.0}, {-32.0, -64.0}}), Rectangle(origin = {62.5, 0.0}, fillColor = {160, 160, 164}, fillPattern = FillPattern.Solid, extent = {{-12.5, -50.0}, {12.5, 50.0}})}));
  end Magnetic;

  package Mechanics "Library of 1-dim. and 3-dim. mechanical components (multi-body, rotational, translational)"
    extends Modelica.Icons.Package;

    package MultiBody "Library to model 3-dimensional mechanical systems"
      extends Modelica.Icons.Package;
      import SI = Modelica.SIunits;
      import Cv = Modelica.SIunits.Conversions;
      import C = Modelica.Constants;

      package UsersGuide "User's Guide of MultiBody Library"
        extends Modelica.Icons.Information;

        package Tutorial "Tutorial"
          extends Modelica.Icons.Information;

          class OverView "Overview of MultiBody library"
            extends Modelica.Icons.Information;
            annotation(
              Documentation(info = "<HTML>
<p>
Library <b>MultiBody</b> is a <b>free</b> Modelica package providing
3-dimensional mechanical components to model in a convenient way
<b>mechanical systems</b>, such as robots, mechanisms, vehicles.
A basic feature is that all components have <b>animation</b> information
with appropriate default sizes and colors. A typical screenshot of the
animation of a double pendulum is shown in the figure below, together
with its schematic.
</p>

<p>
<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Examples/Elementary/DoublePendulumSmall.png\">

<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/UsersGuide/Tutorial/DoublePendulumDiagramSmall.png\" ALT=\"double pendulum (diagram layer)\">
</p>

<p>
Note, that all components - the coordinate system of the world frame,
the gravity acceleration vector, the revolute joints and the
bodies - are visualized in the animation.<br>
This library replaces the long available ModelicaAdditions.MultiBody
library, since it is much more easier to use and more powerful.
The main features of the library are:
</p>
<ul>
<li>About <b>60 main components</b>, i.e., joint, force, part,
    body, sensor and visualizer components that are ready to use
    and have useful default animation properties. One-dimensional
    force laws can be defined with components of the Modelica.Mechanics.Rotational
    and of the Modelica.Mechanics.Translational library and can be
    connected via available flange connectors to MultiBody
    components.
<li>About <b>75 functions</b> to operate in a convenient way on
    orientation objects, e.g., to transform vector quantities between
    frames, or compute the orientation object of a planar rotation.
    The basic idea is to hide the actual definition of an <b>orientation</b>
    by providing essentially an <b>Orientation</b> type together with
    <b>functions</b> operating on instances of this type. Orientation
    objects based on a 3x3 transformation matrix and on quaternions
    are provided. As a side effect, the equations in all other
    components are simpler and easier to understand.</li>
<li> <b>A World model</b> has to be present in every model on top
     level. Here the gravity field is defined (currently: no gravity,
     uniform gravity, point gravity), the visualization of the
     world coordinate system and default settings for animation.
     If a world model is not present, it is automatically provided
     together with a warning message.
<li><b>Built-in animation properties</b> of all components, such as
    joints, forces, bodies, sensors. This allows an easy visual
    check of the constructed model. Animation of every component
    can be switched off via a parameter. The animation of a complete
    system can be switched off via one parameter in the <b>world</b>
    model. If animation is switched off, all equations related
    to animation are removed from the generated code. This is especially
    important for real-time simulation.</li>
<li><b>Automatic handling of kinematic loops</b>.
    Components can be connected together in a nearly arbitrary fashion.
    It does not matter whether components are flipped. This does not
    influence the efficiency. If kinematic loop structures occur,
    this is automatically handled in an efficient way by a new
    technique to transform a certain class of overdetermined sets of
    differential algebraic equations symbolically to a system where
    the number of equations and unknowns are the same (the user need
    <b>not</b> cut loops with special cut-joints to construct a
    tree-structure). </li>
<li><b>Automatic state selection from joints and bodies</b>.
    Most joints and all bodies have potential states. A Modelica
    translator, such as Dymola, will use the generalized coordinates
    of joints as states if possible. If this is not possible,
    states are selected from body coordinates. As a consequence,
    strange joints with 6 degrees of freedom are not necessary
    to define a body moving freely in space. An advanced user
    may select states manually from the <b>Advanced</b> menu of the
    corresponding components or use a Modelica parameter modification
    to set the \"stateSelect\" attribute directly. </li>
<li> <b>Analytic solution of kinematic loops</b>. The non-linear
    equations occurring in kinematic loops are solved <b>analytically</b>
    for a large class of mechanisms, such as a 4 bar mechanism,
    a slider-crank mechanism or a MacPherson suspension. This is performed
    by constructing such loops with assembly joints JointXXX,
    available in the Modelica.Mechanics.MultiBody.Joints package. Assembly joints consist
    of 3 joints that have together 6 degrees of freedom, i.e., no
    constraints.They do not have potential states. When the motion
    of the two frame connectors are provided, a non-linear system of
    equation is solved analytically to compute the motion of the 3 joints.
    Analytic loop handling is especially important for real-time simulation.</li>
<li> <b>Line force components may have mass</b>.
     Masses of line force components are located on the line on which
     the force is acting. They approximate the mass properties of
     a real physical device by one or two point masses. For example,
     a spring has often significant mass that has to be taken into
     account. If masses are set to zero, the additional code to handle
     these point masses is removed. If the masses are taken into
     account, the calculation overhead is small (the reason is that
     the occurring kinematic loops are analytically solved).</li>
<li> <b>Force components may be connected directly together</b>, e.g.,
     3-dimensional springs in series connection. Usually,
     multi-body programs have the restriction that force components
     can only be connected between two bodies. Such restrictions are
     not present in the Modelica multi-body library, since it is
     a fully object-oriented, equation based library. Usually, if
     force components are connected directly together, non-linear
     systems of equations occur. The advantage is often, that this
     may avoid stiff systems that would occur if a small mass has
     to be put in between the two force elements.</li>
<li><b>Initialization definition is available via menus</b>.
    Initialization of states in joints and bodies can be
    performed in the parameter menu, <b>without</b> typing Modelica
    statements. For non-standard initialization, the usual
    Modelica commands can be used.</li>
<li><b>Multi-body specific error messages</b>. Annotations
    and assert statements have been introduced that provide
    in many cases warning or error messages that are related
    to the library components (and not to specific equations
    as it is usual in Modelica libraries). This requires
    appropriate tool support, as it is, e.g., available in Dymola.</li>
<li><b>Inverse models</b> of mechanical systems can be easily
    defined by using motion generators, e.g.,
    Modelica.Mechanics.Rotational.Position. Also, non-standard
    inverse models can be generated, e.g., when elasticity is present
    it might be necessary to differentiate equations several times.</li>
</ul>
</html>"));
          end OverView;

          class FirstExample "A first example"
            extends Modelica.Icons.Information;
            annotation(
              Documentation(info = "<HTML>
<p>
As a first example it shall be demonstrated how to build up, simulate
and animate a <b>simple pendulum</b>.
</p>
<p>
A simple pendulum consisting of a <b>body</b> and a <b>revolute</b> joint
with <b>linear damping</b> in the joint, is first build-up as
Modelica composition diagram, resulting in:
</p>

<p>
<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/UsersGuide/Tutorial/FirstExample/PenulumSchematic1.png\"
ALT=\"Modelica composition diagram of simple pendulum\">
</p>

<p>
In the following figure the location of the used
model components is shown. Drag these components in the diagram layer
and connect them according to the figure:
</p>

<p>
<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/UsersGuide/Tutorial/FirstExample/PenulumSchematic2.png\">
</p>

<p>
Every model that uses model components from the MultiBody library
must have an instance of the Modelica.Mechanics.MultiBody.World model on highest level.
The reason is that in the world object the gravity field is defined
(uniform gravity or point gravity), as well as the default sizes of
animation shapes and this information is reported to all used
components. If the World object is missing, a warning message is
printed and an instance of the World object with default settings is
automatically utilized (this feature is defined with annotations and
is, e.g., supported by Dymola).
</p>
<p>
In a second step the parameters of the dragged components need to be
defined. Some parameters are vectors that have to be defined with respect
to a local coordinate system of the corresponding component. The easiest
way to perform this is to define a <b>reference configuration</b> of your
multi-body model: In this configuration, the relative coordinates of
all joints are zero. This means that all coordinate systems on all
components are parallel to each other. Therefore, this just means
that all vectors are resolved in the world frame in this configuration.
</p>
<p>
The reference configuration for the simple pendulum shall be defined
in the following way: The y-axis of the world frame is directed
upwards, i.e., the opposite direction of the gravity acceleration.
The x-axis of the world frame is orthogonal to it. The revolute joint
is placed in the origin of the world frame. The rotation axis of the revolute
joint is directed along the z-axis of the world frame. The body is
placed on the x-axis of the world frame (i.e., the rotation angle of the
revolute joint is zero, when the body is on the x-axis).
In the following figures the definition of this reference configuration
is shown in the parameter menus of the revolute joint and the body:
</p>

<p>
<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/UsersGuide/Tutorial/FirstExample/ActuatedRevolute.png\">

<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/UsersGuide/Tutorial/FirstExample/Body.png\">
</p>

<p>
Translate and simulate the model, e.g., with Dymola.
Automatically, all defined components are visualized in
an animation using default absolute or relative sizes
of the components. For example, a body is visualized as
a sphere and as a cylinder. The default size of the sphere is defined
as parameter in the world object. You may change this size
in the \"Animation\" parameter menu of the body (see parameter menu
above). The default size of the cylinder is defined relatively
to the size of the sphere (half of the sphere size).
With default settings, the following animation is defined:
</p>

<p>
<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/UsersGuide/Tutorial/FirstExample/Pendulum.png\">
</p>

<p>
The world coordinate system is visualized as coordinate system
with axes labels. The direction of the gravity acceleration
vector is shown as green arrow. The red cylinder represents
the rotation axis of the revolute joint and the light blue
shapes represent the body. The center of mass of the body is
in the middle of the light blue sphere.
</p>
</html>"));
          end FirstExample;

          package LoopStructures "Loop structures"
            extends Modelica.Icons.Information;

            class Introduction "Introduction"
              extends Modelica.Icons.Information;
              annotation(
                DocumentationClass = true,
                Documentation(info = "<HTML>
<p>
In principal, now special action is needed, if
loop structures occur (contrary to the ModelicaAdditions.MultiBody library).
An example is presented in the figure below. It is available as
<a href=\"modelica://Modelica.Mechanics.MultiBody.Examples.Loops.Fourbar1\">
MultiBody.Examples.Loops.Fourbar1</a>
</p>

<p>
<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/UsersGuide/Tutorial/LoopStructures/Fourbar1a.png\">
</p>

<p>
This mechanism consists of 6 revolute joints, 1 prismatic joint and forms a
kinematical loop.
It has has one degree of freedom. In the next figure the default animation
is shown. Note, that the axes of the revolute joints are represented by the
red cylinders and that the axis of the prismatic joint is represented by the
red box on the lower right side.
</p>

<p>
<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/UsersGuide/Tutorial/LoopStructures/Fourbar1b.png\" width=\"205\" height=\"107\">
</p>

<p>
Whenever loop structures occur, non-linear algebraic
equations are present on \"position level\". It is then usually not possible by
structural analysis to select states during translation (which is possible for
non-loop structures). In the example above, Dymola detects a non-linear
algebraic loop of 57 equations and reduces this to a system of 7 coupled
algebraic equations. Note, that this is performed without using any
\"cut-joints\" as it is usually done in multi-body programs, but by just
appropriate symbolic equation manipulation. Via the dynamic dummy derivative
method the generalized coordinates on position and velocity level from one of
the 7 joints are dynamically selected as states during simulation. Whenever,
these two states are no longer appropriate, states from one of the other
joints are selected during simulation.
</p>
<p>
The efficiency of loop structures can usually be
enhanced, if states are statically fixed at translation time. For this
mechanism, the generalized coordinates of joint j1 (i.e., the
rotation angle of the revolute joint and its derivative) can always be used as
states. This can be stated by setting parameter \"enforceStates = <b>true</b>\"
in the \"Advanced\" menu of the desired joint. This flag sets the attribute
stateSelect of the generalized coordinates of the corresponding joint to
\"StateSelect.always\". When setting this flag to <b>true</b> for joint j1 in
the four bar mechanism, Dymola detects a non-linear algebraic loop of 40
equations and reduces this to a system of 5 coupled non-linear algebraic
equations.
</p>
<p>
In many mechanisms it is possible to solve the non-linear algebraic
equations analytically. For a certain class of systems this can
be performed also with the MultiBody library. This technique
is described in section
\"<a href=\"modelica://Modelica.Mechanics.MultiBody.UsersGuide.Tutorial.LoopStructures.AnalyticLoopHandling\">Analytic loop handling</a>\".
</p>
</HTML>"));
            end Introduction;

            class PlanarLoops "Planar loops"
              extends Modelica.Icons.Information;
              annotation(
                Documentation(info = "<HTML>
<p>
In the figure below, the model of a V6 engine is
shown that has a simple combustion model. It is available as
<a href=\"modelica://Modelica.Mechanics.MultiBody.Examples.Loops.EngineV6\">
MultiBody.Examples.Loops.EngineV6</a>.
</p>

<p>
<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/UsersGuide/Tutorial/LoopStructures/EngineV6_1.png\">
</p>

<p>
The Modelica schematic of one cylinder
is given in the figure below. Connecting 6 instances of this
cylinder appropriately together results in the engine schematic displayed
above.
</p>

<p>
<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/UsersGuide/Tutorial/LoopStructures/EngineV6_2.png\">
</p>

<p>
In the next figure the animation of the engine is
shown. Every cylinder consists essentially of 1 prismatic and 2 revolute
joints that form a planar loop, since the axes of the two revolute joints are
parallel to each other and the axis of the prismatic joint is orthogonal to
the revolute joint axes. All 6 cylinders together form a coupled set of 6
loops that have together 1 degree of freedom.
</p>

<p>
<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/UsersGuide/Tutorial/LoopStructures/EngineV6_3.png\" width=\"303\" height=\"136\">
</p>

<p>
All planar loops, and especially the engine, result in
a DAE (= Differential-Algebraic Equation system)
that does not have a unique solution. The reason is that, e.g., the cut
forces in direction of the axes of the revolute joints cannot be uniquely
computed. Any value fulfills the DAE equations. This is a structural property
that is determined by the symbolic algorithms. Since they detect that the DAE is
structurally singular, a further processing is not possible. Without
additional information it is also impossible that the symbolic algorithms
could be enhanced because if the axes of rotations of the revolute joints are
only slightly changed such that they are no longer parallel to each other, the
planar loop can no longer move and has 0 degrees of freedom. Algorithms based
on pure structural information cannot distinguish these two cases.
</p>
<p>
The usual remedy is to remove superfluous constraints,
e.g., along the axis of rotation of <b>one</b> revolute joint. Since this is
not easy for an inexperienced modeler, the special joint:
<a href=\"modelica://Modelica.Mechanics.MultiBody.Joints.RevolutePlanarLoopConstraint\">
RevolutePlanarLoopConstraint</a> is provided that removes these constraints.
Exactly one revolute joint in a every planar loop must be replaced by this
joint type. In the engine example, this special joint is used for
the revolute joint B2 in the cylinder model above. The icon of the joint is
slightly different to other revolute joints to visualize this case.
</p>
<p>
If a modeler is not aware of the problems with planar
loops and models them without special consideration, a Modelica
translator, such as Dymola, displays an error
message and points out that a planar loop may be the reason and suggests to
use the RevolutePlanarLoopConstraint joint.
This error message is due to an annotation in
the Frame connector.
</p>
<pre>
  <b>connector</b> Frame
     ...
     <b>flow</b> SI.Force f[3] <b>annotation</b>(unassignedMessage=\"..\");
  <b>end</b> Frame;
</pre>
<p>
If no assignment can be found for some
forces in a connector, the \"unassignedMessage\" is displayed. In most cases the
reason for this is a planar loop or two joints that constrain the same motion.
Both cases are discussed in the error message.
</p>
<p>
Note, that the non-linear algebraic equations occurring
in planar loops can be solved analytically in most cases and therefore it is
highly recommended to use the techniques discussed in section
\"<a href=\"modelica://Modelica.Mechanics.MultiBody.UsersGuide.Tutorial.LoopStructures.AnalyticLoopHandling\">Analytic loop handling</a>\"
for such systems.
</p>
</HTML>"));
            end PlanarLoops;

            class AnalyticLoopHandling "Analytic loop handling"
              extends Modelica.Icons.Information;
              annotation(
                Documentation(info = "<HTML>
<p>
It is well known that the non-linear
algebraic equations of most mechanical loops in technical devices can be
solved analytically. It is, however, difficult to perform this fully
automatically and therefore none of the commercial, general purpose multi-body
programs, such as MSC ADAMS, LMS DADS, SIMPACK, have this feature.
These programs solve loop structures with pure numerical methods. Multi-body
programs that are designed for real-time simulation of the dynamics of
specific vehicles, such as ve-DYNA, usually contain manual implementations
of a particular multi-body system (the vehicle) where the occurring loops are
either analytically solved, if this is possible, or are treated by table
look-up where the tables are constructed in a pre-processing phase. Without
these features the required real-time capability would be difficult to
achieve.
</p>
<p>
In a series of papers and dissertations
Prof. Hiller and his group in Duisburg, Germany,
have developed systematic methods to handle mechanical
loops analytically. The \"characteristic pair of joints\" method
basically cuts a loop at two joints and uses geometric
invariants to reduce the number of algebraic
equations, often down to one equation that can be solved analytically. Also
several multi-body codes have been developed that are based on this method,
e.g., MOBILE. Besides the very desired feature to solve non-linear
algebraic equations analytically, i.e., efficiently and in a robust way, there
are several drawbacks: It is difficult to apply this method automatically.
Even if this would be possible in a good way, there is always the problem that
it cannot be guaranteed that the statically selected states lead to no
singularity during simulation. Therefore, the \"characteristic pair of joints\"
method is usually manually applied which requires know-how and experience.
</p>
<p>
In the MultiBody library the \"characteristic pair of
joints\" method is supported in a restricted form such that it can be applied
also by non-specialists. The idea is to provide aggregations of joints in package
<a href=\"modelica://Modelica.Mechanics.MultiBody.Joints.Assemblies\">
MultiBody.Joints.Assemblies</a>.
as one object that either have <b>6</b> degrees of freedom or
<b>3</b> degrees of freedom (for usage in planar loops).
</p>
<p>
As an example, a variant of the four bar mechanism is given in
the figure below.
</p>

<p>
<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/UsersGuide/Tutorial/LoopStructures/FourbarAnalytic1.png\">
</p>

<p>
Here, the mechanism is modeled with one revolute
joint, two spherical joints and one prismatic joint.
In the figure below, the two spherical joints
and the prismatic joint are collected together in an assembly object
called \"jointSSP\" from
<a href=\"modelica://Modelica.Mechanics.MultiBody.Joints.Assemblies.JointSSP\">
MultiBody.Joints.Assemblies.JointSSP</a>.
</p>

<p>
<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/UsersGuide/Tutorial/LoopStructures/FourbarAnalytic2.png\">
</p>

<p>
The JointSSP joint aggregation has a frame at the
left side of the left spherical joint (frame_a) and a frame at the right side
of the prismatic joint (frame_b). JointSSP, as all other objects from the
Joints.Assemblies package, has the property, that the <b>generalized
coordinates, and all other frames
defined in the assembly, can be calculated given the movement of frame_a and
of frame_b</b>. This is performed by <b>analytically</b> solving non-linear
systems of equations (details are given in section xxx). From a
structural point of view, the equations in an assembly object are written in
the form
</p>
<p align=\"left\">
  <b>q</b> = <b>
  f</b><sub>1</sub>(<b>r</b><sup>a</sup>, <b>R</b><sup>a</sup>,<b> r</b><sup>b</sup>,
  <b>R</b><sup>b</sup>)
</p>
<p>
where<b> r</b><sup>a</sup>, <b>R</b><sup>a</sup>,<b>
r</b><sup>b</sup>, <b>R</b><sup>b</sup> are the variables defining the
position and orientation of the frame_a and frame_b connector,
<b>q</b> are the generalized positional coordinates inside the
assembly, e.g., the angle of a revolute joint. Given angle
<span style=\"font-family:Symbol\">j</span> of revolute joint j1 from the four
bar mechanism, frame_a and frame_b of the assembly object can be computed by a
forward recursion
</p>
<p align=\"left\">
(<b>r</b><sup>a</sup>,
<b>R</b><sup>a</sup>,<b> r</b><sup>b</sup>, <b>R</b><sup>b</sup>) = <b>f</b>(<span style=\"font-family:Symbol\">j)</span>
</p>
<p>
Since this is a structural property, the
symbolic algorithms can automatically select <span style=\"font-family:Symbol\">
j</span> and its derivative as states and then all positional variables can be
computed in a forwards sequence. It is now understandable that a Modelica
translator, such as Dymola, can
transform the equations of the four bar mechanism to a recursive sequence of
statements that has no non-linear algebraic loops anymore(remember,
the previous \"straightforward\" solution with 6 revolute joints and 1
prismatic joint has a nonlinear system of equations of order 5).
</p>
<p>
The aggregated joint
objects consist of a combination of either a revolute or prismatic joint and
of a rod that has either two spherical joints at its two ends or a spherical
and a universal joint, respectively. For all combinations, analytic solutions
can be determined. For planar loops, combinations of 1, 2 or 3 revolute joints
with parallel axes and of 2 or 1 prismatic joint with axes that are orthogonal
to the revolute joints can be treated analytically. The currently supported
combinations are listed in the table below.
The missing combinations (such as JointSUP
or Joint RPP) will be added in one of the next releases.
</p>
<div align=\"left\">
    <table border=\"1\" cellspacing=\"0\" cellpadding=\"0\">
      <tr>
        <td colspan=\"2\"> <b>3-dimensional Loops:</b></td>
      </tr>
      <tr>
        <td valign=\"top\">JointSSR</td>
        <td valign=\"top\">Spherical - Spherical - Revolute</td>
      </tr>
      <tr>
        <td valign=\"top\">JointSSP</td>
        <td valign=\"top\">Spherical - Spherical - Prismatic</td>
      </tr>
      <tr>
        <td valign=\"top\">JointUSR</td>
        <td valign=\"top\">Universal - Spherical - Revolute</td>
      </tr>
      <tr>
        <td valign=\"top\">JointUSP</td>
        <td valign=\"top\">Universal - Spherical - Prismatic</td>
      </tr>
      <tr>
        <td valign=\"top\">JointUPS</td>
        <td valign=\"top\">Universal - Prismatic - Spherical</td>
      </tr>
      <tr>
        <td colspan=\"2\"><b>Planar Loops:</b></td>
      </tr>
      <tr>
        <td valign=\"top\">JointRRR</td>
        <td valign=\"top\">Revolute - Revolute - Revolute</td>
      </tr>
      <tr>
        <td valign=\"top\">JointRRP</td>
        <td valign=\"top\">Revolute - Revolute - Prismatic</td>
      </tr>
</table>
</div>
<p>
On first view this seems to be quite restrictive. However, mechanical devices are usually built up with rods connected by spherical joints on each end,
and additionally with revolute and prismatic joints.
Therefore, the combinations of the above table occur frequently.
The universal joint is usually not present in actual devices but is used
(a) if two JointXXX components can be connected such that a revolute
and a universal joint together form a spherical joint
and (b) if the orientation of the connecting rod between two
spherical joints is needed, e.g., since a body shall be attached.
In this case one of the spherical joints might be replaced by a
universal joint. This approximation is fine as long as the mass
and inertia of the rod is not significant.
</p>
<p>
Let us discuss item (a) in more detail: The
MacPherson suspension in the next figure
is from the Modelica VehicleDynamics library.

<p>
The JointSSP joint aggregation has a frame at the
left side of the left spherical joint (frame_a) and a frame at the right side
of the prismatic joint (frame_b). JointSSP, as all other objects from the
Joints.Assemblies package, has the property, that the <b>generalized
coordinates, and all other frames
defined in the assembly, can be calculated given the movement of frame_a and
of frame_b</b>. This is performed by <b>analytically</b> solving non-linear
systems of equations (details are given in section xxx). From a
structural point of view, the equations in an assembly object are written in
the form
</p>
<p align=\"left\">
  <b>q</b> = <b>
  f</b><sub>1</sub>(<b>r</b><sup>a</sup>, <b>R</b><sup>a</sup>,<b> r</b><sup>b</sup>,
  <b>R</b><sup>b</sup>)
</p>
<p>
where<b> r</b><sup>a</sup>, <b>R</b><sup>a</sup>,<b>
r</b><sup>b</sup>, <b>R</b><sup>b</sup> are the variables defining the
position and orientation of the frame_a and frame_b connector,
<b>q</b> are the generalized positional coordinates inside the
assembly, e.g., the angle of a revolute joint. Given angle
<span style=\"font-family:Symbol\">j</span> of revolute joint j1 from the four
bar mechanism, frame_a and frame_b of the assembly object can be computed by a
forward recursion
</p>
<p align=\"left\">
(<b>r</b><sup>a</sup>,
<b>R</b><sup>a</sup>,<b> r</b><sup>b</sup>, <b>R</b><sup>b</sup>) = <b>f</b>(<span style=\"font-family:Symbol\">j)</span>
</p>
<p>
Since this is a structural property, the
symbolic algorithms can automatically select <span style=\"font-family:Symbol\">
j</span> and its derivative as states and then all positional variables can be
computed in a forwards sequence. It is now understandable that a Modelica
translator, such as Dymola, can
transform the equations of the four bar mechanism to a recursive sequence of
statements that has no non-linear algebraic loops anymore(remember,
the previous \"straightforward\" solution with 6 revolute joints and 1
prismatic joint has a nonlinear system of equations of order 5).
</p>
<p>
The aggregated joint
objects consist of a combination of either a revolute or prismatic joint and
of a rod that has either two spherical joints at its two ends or a spherical
and a universal joint, respectively. For all combinations, analytic solutions
can be determined. For planar loops, combinations of 1, 2 or 3 revolute joints
with parallel axes and of 2 or 1 prismatic joint with axes that are orthogonal
to the revolute joints can be treated analytically. The currently supported
combinations are listed in the table below.
The missing combinations (such as JointSUP
or Joint RPP) will be added in one of the next releases.
</p>
<div align=\"left\">
    <table border=\"1\" cellspacing=\"0\" cellpadding=\"0\">
      <tr>
        <td colspan=\"2\"> <b>3-dimensional Loops:</b></td>
      </tr>
      <tr>
        <td valign=\"top\">JointSSR</td>
        <td valign=\"top\">Spherical - Spherical - Revolute</td>
      </tr>
      <tr>
        <td valign=\"top\">JointSSP</td>
        <td valign=\"top\">Spherical - Spherical - Prismatic</td>
      </tr>
      <tr>
        <td valign=\"top\">JointUSR</td>
        <td valign=\"top\">Universal - Spherical - Revolute</td>
      </tr>
      <tr>
        <td valign=\"top\">JointUSP</td>
        <td valign=\"top\">Universal - Spherical - Prismatic</td>
      </tr>
      <tr>
        <td valign=\"top\">JointUPS</td>
        <td valign=\"top\">Universal - Prismatic - Spherical</td>
      </tr>
      <tr>
        <td colspan=\"2\"><b>Planar Loops:</b></td>
      </tr>
      <tr>
        <td valign=\"top\">JointRRR</td>
        <td valign=\"top\">Revolute - Revolute - Revolute</td>
      </tr>
      <tr>
        <td valign=\"top\">JointRRP</td>
        <td valign=\"top\">Revolute - Revolute - Prismatic</td>
      </tr>
</table>
</div>
<p>
On first view this seems to be quite restrictive. However, mechanical devices are usually built up with rods connected by spherical joints on each end,
and additionally with revolute and prismatic joints.
Therefore, the combinations of the above table occur frequently.
The universal joint is usually not present in actual devices but is used
(a) if two JointXXX components can be connected such that a revolute
and a universal joint together form a spherical joint
and (b) if the orientation of the connecting rod between two
spherical joints is needed, e.g., since a body shall be attached.
In this case one of the spherical joints might be replaced by a
universal joint. This approximation is fine as long as the mass
and inertia of the rod is not significant.
</p>
<p>
Let us discuss item (a) in more detail: The
MacPherson suspension in the next figure
is from the Modelica VehicleDynamics library.
</p>

<p>
<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/UsersGuide/Tutorial/LoopStructures/MacPherson1.png\">
</p>

<p>
It has three frame
connectors. The lower left one (frame_C) is fixed in the vehicle chassis. The
upper left one (frame_S) is driven by the steering mechanism, i.e., the
movement of both frames are given. The frame connector on the right (frame_U)
drives the wheel. The three frames are connected by a mechanism consisting
essentially of two rods with spherical joints on both ends. These are built up
by a jointUPS and a jointSSR assembly.
As can be seen, the universal joint from the jointUPS
assembly is connected to the revolute joint of the jointSSR assembly.
Therefore, we have 3 revolute joints connected together at one point and if
the axes of rotations are chosen appropriately, this describes a spherical
joint. In other words, the two connected assemblies define the desired two
rods with spherical joints on each ends.
</p>
<p>
The movement of the chassis, frame_C, is computed
somewhere else. When the generalized coordinates of revolute joint
\"innerJoint\" (lower left part in figure) are used as states, then frame_a and
frame_b of the jointUPS joint can be calculated. After the non-linear loop
with jointUPS is (analytically) solved, all frames on this assembly are
known, especially,
the one connected to frame_b of the jointSSR assembly. Since frame_b of
jointSSR is connected to frame_S which is computed from the steering
mechanism, again the two required frame movements of the jointSSR assembly are
calculated, meaning in turn that also all other frames on the jointSSR
assembly can be computed, especially, the one connected to frame_U that drives
the wheel. From this analysis it is clear that a tool is able to solve these
coupled loops analytically.
</p>
<p>
Another example is the model of the V6 engine,
see next figure for an animation view and the original definition
of one cylinder with elementary joints.
</p>

<p>
<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/UsersGuide/Tutorial/LoopStructures/EngineV6_3.png\" width=\"303\" height=\"136\">

<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/UsersGuide/Tutorial/LoopStructures/EngineV6_2.png\">
</p>

<p>
It is sufficient to rewrite the basic cylinder model
by replacing the joints with a JointRRP object that has two
revolute and one prismatic joint, see next figure.
</p>

<p>
<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/UsersGuide/Tutorial/LoopStructures/EngineV6_4.png\">
</p>

<p>
Since 6
cylinders are connected together, 6 coupled loops with 6 JointRRP objects are
present. This model is available as
<a href=\"modelica://Modelica.Mechanics.MultiBody.Examples.Loops.EngineV6_analytic\">
MultiBody.Examples.Loops.EngineV6_analytic</a>.
</p>
<p>
The composition diagram of the connected 6 cylinders is
shown in the next figure
</p>

<p>
<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/UsersGuide/Tutorial/LoopStructures/EngineV6_1.png\">
</p>
<p>
It can be seen that the revolute joint
of the crank shaft (joint \"bearing\" in left part of figure) might be
selected as degree of freedom. Then the 4 connector frames of all cylinders
can be computed. As a result the computations of the cylinders are decoupled
from each other. Within one cylinder
the position of frame_a and frame_b of the jointRRP assembly
can be computed and therefore the generalized coordinates of the two revolute
and the prismatic joint in the jointRRP object can be determined. From this
analysis it is not surprising that a Modelica translator, such as Dymola,
is able to transform the DAE
equations into a sequential evaluation without any non-linear loop.
Compare this nice result with the model using only elementary joints
that leads to a DAE with 6 algebraic loops and 5 non-linear equations per
loop. Additionally, a linear system of equations of order 43 is present.
The simulation time is about 5 times faster with the analytic loop handling.
</p>
</html>"));
            end AnalyticLoopHandling;
            annotation(
              Documentation(info = "<HTML>
<p>
The MultiBody library has the feature that all components
can be connected together in a nearly arbitrary fashion.
Therefore, kinematic loop structures pose in principal
no problems. In this section several examples are given,
the special treatment of planar loops is discussed and it is explained
how a kinematic loop structure can be modeled such that the
occurring non-linear algebraic equation systems are solved
analytically. There are the following sub-chapters:
</p>
<ol>
<li><a href=\"modelica://Modelica.Mechanics.MultiBody.UsersGuide.Tutorial.LoopStructures.Introduction\">Introduction</a>
</li>
<li><a href=\"modelica://Modelica.Mechanics.MultiBody.UsersGuide.Tutorial.LoopStructures.PlanarLoops\">Planar loops</a>.
</li>
<li><a href=\"modelica://Modelica.Mechanics.MultiBody.UsersGuide.Tutorial.LoopStructures.AnalyticLoopHandling\">Analytic loop handling</a>.
</li>
</ol>
</HTML>"));
          end LoopStructures;

          class ConnectionOfLineForces "Connection of LineForces"
            extends Modelica.Icons.Information;
            annotation(
              Documentation(info = "<html>
<p>
Line force elements, such as a
<a href=\"modelica://Modelica.Mechanics.MultiBody.Forces.Spring\">Spring</a>,
are usually connected between two parts. In fact, this is the only possibility
in most multi-body programs. In an equation based system like Modelica, more
general connections are possible. In particular
<b>3-dimensional line force</b> elements can be connected together
in <b>series</b> without having a body with mass at the
connection point. This is advantageous since stiff systems can be avoided, say, due to
a stiff spring and a small mass at the connection point.
For an example, see model
<a href=\"modelica://Modelica.Mechanics.MultiBody.Examples.Elementary.ThreeSprings\">ThreeSprings</a>:
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Examples/Elementary/ThreeSprings.png\">
</p>

<p>
Here, three springs are connected together at one point, without having a body
at the connection point of the springs. There is one difficulty: In such a situation
the orientation object at the connection point is undefined, because
the springs do not transmit torques. Translation will therefore fail, if three springs
and a body are connected together in this way. To handle such a case, all line force
elements have flags \"fixedRotationAtFrame_a\" and \"fixedRotationAtFrame_b\" in their
\"Advanced\" parameter menu. For example, if \"fixedRotationAtFrame_b = <b>true</b>\",
the orientation object at frame_b is explicitly set to a null rotation, i.e.,
</p>

<blockquote><pre>
frame_b.R = Modelica.Mechanics.MultiBody.Frames.nullRotation();
</pre></blockquote>

<p>
This means that the coordinate system in the connection point of the three springs
is always parallel to the world frame. When this option is selected, the corresponding
frame in the line force icon is marked with a red circle and with the text \"R=0\".
This is shown in the next figure, where this option is selected for spring3.frame_b:
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/UsersGuide/ThreeSpringsDiagramLayer.png\">
</p>

<p>
Note, if this flag is not set to <b>true</b>, a translation error will occur.
Due to the usage of overdetermined connectors in the MultiBody library, the error
message will be something like: .
</p>

<blockquote><p>
\"The overdetermined connectors &lt;...&gt; are connected but do not have any root defined\"
</p></blockquote>

<p>
The two flags \"fixedRotationAtFrame_a\" and \"fixedRotationAtFrame_b must be very carefully
set, because a wrong definition can lead to a model that simulates, but the simulation
result is wrong. This is the case, whenever the movement of the resulting system depends
on the orientation object that was arbitrarily set in parallel to the world frame.
A typical example is shown in the next figure:
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/UsersGuide/ThreeSprings2DiagramLayer.png\">
</p>

<p>
Here, spring3.frame_b.R is defined to be in parallel to the world frame.
However, this is then also the orientation of fixedTranslation.frame_a, and this
in turn means that the left part of the fixedTranslation object is always in parallel
to the world frame. Since this is not correct, this model
will result in a <b>wrong simulation result</b>
This system is mathematically not well-defined and does not have a solution.
The only way to model such a system is by providing a mass and an inertia tensor
to fixedTranslation. Then, the flags are not needed, because the \"connection\"
point of the springs is a body where the absolution position vector and the
orientation matrix of the body-fixed coordinate system are used as states.
</p>

</html>"));
          end ConnectionOfLineForces;
          annotation(
            DocumentationClass = true,
            Documentation(info = "<HTML>
<p>
This tutorial provides an introduction into the
MultiBody library.
</p>
<ol>
<li><a href=\"modelica://Modelica.Mechanics.MultiBody.UsersGuide.Tutorial.OverView\">Overview of
MultiBody library</a> summarizes the most important aspects.
</li>
<li><a href=\"modelica://Modelica.Mechanics.MultiBody.UsersGuide.Tutorial.FirstExample\">A first example</a>
  describes in detail all the steps to build a simple pendulum model.
</li>
<li><a href=\"modelica://Modelica.Mechanics.MultiBody.UsersGuide.Tutorial.LoopStructures\">Loop structures</a>
  explains how to model kinematic loops, especially by analytically
  solving non-linear equations.
</li>
<li><a href=\"modelica://Modelica.Mechanics.MultiBody.UsersGuide.Tutorial.ConnectionOfLineForces\">ConnectionOfLineForces</a>
  explains how to connect line force components directly together.
</li>
</ol>
</HTML>"));
        end Tutorial;

        class Upgrade "Upgrade from Former Versions"
          extends Modelica.Icons.Information;
          annotation(
            Documentation(info = "<HTML>
<p>
If different versions of the MultiBody library are not
compatible to each other, corresponding conversion scripts are
provided. As a result, models build with an older version
of the MultiBody library are automatically converted to the
new version when the model is loaded. The user is prompted
whether automatic conversion shall take place or not.
Problems are not to be expected. Still one should first make
a copy of such a model as backup before the conversion
is performed.
</p>
<p>
<b>Upgrade from ModelicaAdditions.MultiBody</b>
</p>
<p>
There is now also a conversion script from the \"old\"
<b>ModelicaAdditions.MultiBody</b> library to the
\"new\" Modelica.Mechanics.MultiBody library. This script is also
automatically invoked. Since the differences between the \"old\" and the
\"new\" MultiBody library are so large, not everything is
converted and it might be that some pieces have to
be adapted manually. Still, this script is useful, since
many class names, parameters and modifiers are
automatically converted.
</p>
<p>
Components from the following sublibraries
are automatically converted
to the Modelica.Mechanics.MultiBody library:
</p>
<ul>
<li> ModelicaAdditions.MultiBody.Parts</li>
<li> ModelicaAdditions.MultiBody.Joints</li>
<li> ModelicaAdditions.MultiBody.Forces</li>
<li> Part of ModelicaAdditions.MultiBody.Interfaces</li>
</ul>
<p>
Models using the ModelicaAdditions.MultiBody library
that are programmed with <b>equations</b> are only partly converted:
The Frame connectors will be converted to the \"new\"
Frame connectors of the MultiBody library, but the equations
that reference variables of the Frame connectors will
<b>not</b> be converted. For a manual conversion, the following
table might be helpful showing how the <b>variables</b> of the
\"old\" and the \"new\" <b>Frame connectors</b> are
related to each other (resolve2 and angularVelocity2 are
functions from library Modelica.Mechanics.MultiBody.Frames):
</p>
<table border=1 cellspacing=0 cellpadding=2>
<tr><th><b>ModelicaAdditions.MultiBody.<br>
           Interfaces.Frame_a</b></th>
    <th><b>MultiBody.Interfaces.Frame_a</b></th></tr>
<tr>
  <td valign=\"top\">frame_a.<b>r0</b></td>
  <td valign=\"top\">= frame_a.r_0 (is converted)</td>
</tr>
<tr>
  <td valign=\"top\">frame_a.<b>S</b></td>
  <td valign=\"top\">= transpose(frame_a.R)</td>
</tr>
<tr>
  <td valign=\"top\">frame_a.<b>v</b></td>
  <td valign=\"top\">= resolve2(frame_a.R, <b>der</b>(frame_a.r_0))</td>
</tr>
<tr>
  <td valign=\"top\">frame_a.<b>w</b></td>
  <td valign=\"top\">= angularVelocity2(frame_a.R)</td>
</tr>
<tr>
  <td valign=\"top\">frame_a.<b>a</b></td>
  <td valign=\"top\">= resolve2(frame_a.R, <b>der</b>(v_0)); v_0 = der(r_0)</td>
</tr>
<tr>
  <td valign=\"top\">frame_a.<b>z</b></td>
  <td valign=\"top\">= <b>der</b>(w);  w = angulaVelocity2(frame_a.R)</td>
</tr>
<tr>
  <td valign=\"top\">frame_a.<b>f</b></td>
  <td valign=\"top\">= frame_a.f (no conversion needed)</td>
</tr>
<tr>
  <td valign=\"top\">frame_a.<b>t</b></td>
  <td valign=\"top\">= frame_a.t (no conversion needed)</td>
</tr>
</table>
<p>
<b>Upgrade from MultiBody 0.99 (and earlier) to 1.0 (and later)</b>
<p>
The conversion from MultiBody 0.99 to 1.0 does not work in some rare
cases, where own components are implemented using functions of the
MultiBody.Frames package. In this case, the conversion has to be
performed manually. The changes in 1.0 with regards to 0.99 are:
</p>
<p>
The definition of the Modelica.Mechanics.MultiBody.Frames.Orientation object has changed.
In 0.99 this was just an alias type for a transformation matrix
(now Modelica.Mechanics.MultiBody.Frames.TransformationMatrices.Orientation).
In 1.0 the orientation object is a record holding the
transformation matrix from frame 1 to frame 2 and the angular
velocity of the transformation matrix resolved in frame 2.
The reason is that this allows to compute the angular velocity
in many cases by standard recursive formulas and not by
differentiation of the transformation matrix. This is usually
much more efficient. As a consequence, the following
calls in 0.99 should be changed:
</p>
<pre>
   Frames.angularVelocity1(T,der(T)) -> Frames.angularVelocity1(T)
   Frames.angularVelocity2(T,der(T)) -> Frames.angularVelocity2(T)
   Frames.from_T(T)                  -> Frames.from_T2(T,der(T))
</pre>
</html>"));
        end Upgrade;

        class Literature "Literature"
          extends Modelica.Icons.References;
          annotation(
            Documentation(info = "<html>
<ul>
<li>Technical details of this library are described in the
20 page paper:
<dl>
<dt>Otter M., Elmqvist H., and Mattsson S.E.:</dt>
<dd> <b>The New Modelica MultiBody Library</b>.
     Modelica 2003 Conference, Link&ouml;ping, Sweden,
     pp. 311-330, Nov. 3-4, 2003.
     Download from:
     <a href=\"https://www.modelica.org/events/Conference2003/papers/h37_Otter_multibody.pdf\">https://www.modelica.org/events/Conference2003/papers/h37_Otter_multibody.pdf</a>
     </dd>
</dl>
</li>
<li>The method how to describe drive trains with 1-dimensional mechanics
and to mount them on 3-dimensional components without neglecting
dynamical effects is described in:
<dl>
<dt>Schweiger C., and Otter M.:</dt>
<dd> <b>Modelling 3-dim. Mechanical Effects of 1-dim. Powertrains</b>.
     Modelica 2003 Conference, Link&ouml;ping, Sweden,
     pp. 149-158, Nov. 3-4, 2003.
     Download from:
<a href=\"https://www.modelica.org/events/Conference2003/papers/h06_Schweiger_powertrains_v5.pdf\">https://www.modelica.org/events/Conference2003/papers/h06_Schweiger_powertrains_v5.pdf</a>
     </dd>
</dl>
</li>
<li> The method to solve a certain class of kinematic loops
     analytically is based on:
<dl>
<dt>Woernle C.:</dt>
<dd><b>Ein systematisches Verfahren zur Aufstellung der geometrischen
    Schliessbedingungen in kinematischen Schleifen mit Anwendung
    bei der R&uuml;ckw&auml;rtstransformation f&uuml;r
    Industrieroboter.</b><br>
    Fortschritt-Berichte VDI, Reihe 18, Nr. 59, Duesseldorf: VDI-Verlag 1988,
    ISBN 3-18-145918-6.<br>&nbsp;</dd>
<dt>Hiller M., and Woernle C.:</dt>
<dd><b>A Systematic Approach for Solving the Inverse Kinematic
    Problem of Robot Manipulators</b>.<br>
    Proceedings 7th World Congress Th. Mach. Mech., Sevilla 1987. </dd>
</dl>
</li>
</ul>
</html>"));
        end Literature;

        class Contact "Contact"
          extends Modelica.Icons.Contact;
          annotation(
            Documentation(info = "<html>
<dl>
<dt><b>Library Officer:</b>
<dd><a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a><br>
    Deutsches Zentrum f&uuml;r Luft und Raumfahrt e.V. (DLR)<br>
    Institut f&uuml;r Robotik und Mechatronik<br>
    Abteilung f&uuml;r Entwurfsorientierte Regelungstechnik<br>
    Postfach 1116<br>
    D-82230 Wessling<br>
    Germany<br>
    email: <A HREF=\"mailto:Martin.Otter@dlr.de\">Martin.Otter@dlr.de</A><br>
</dl>
<p><b>Acknowledgements:</b></p>
<ul>
<li> The central idea to handle a certain class of overdetermined, consistent
     set of differential algebraic equations (i.e., there are more equations than
     unknowns) with symbolic transformation algorithms was developed together
     with Hilding Elmqvist and Sven Erik Mattsson from Dassault Syst&egrave;mes AB, Lund, Sweden.
     The MultiBody library is heavily relying on this feature which is a
     prerequisite for a truly \"object-oriented\" multi-body systems library,
     where components can be connected together in any meaningful way.</li>
<li> The Examples.Loops.EngineV6 demo of a six cylinder V6 engine with
     6 planar loops and 1 degree of freedom is from Hilding Elmqvist and
     Sven Erik Mattsson.</li>
<li> Modelica.Mechanics.MultiBody.Forces.LineForceWithMass is based on model
     \"RelativeDistance\" from the Modelica VehicleDynamics library of
     Johan Andreasson from Royal Institute of Technology, Stockholm, Sweden.</li>
<li> The 1-dim. components (Parts.Rotor1D, Parts.BevelGear1D, Mounting1D) and
     Joints.GearConstraints are from Christian Schweiger.</li>
<li> The design of this library is based on work carried out
     in the EU RealSim project (Real-time Simulation for Design of
     Multi-physics Systems) funded by the European Commission within
     the Information Societies Technology (IST) programme under
     contract number IST 1999-11979.
     </li>
</ul>
</html>"));
        end Contact;
        annotation(
          DocumentationClass = true,
          Documentation(info = "<HTML>
<p>
Library <b>MultiBody</b> is a <b>free</b> Modelica package providing
3-dimensional mechanical components to model in a convenient way
<b>mechanical systems</b>, such as robots, mechanisms, vehicles.
This package contains the User's Guide for the MultiBody library.
</p>
<ol>
<li> <a href=\"modelica://Modelica.Mechanics.MultiBody.UsersGuide.Tutorial\">Tutorial</a>
gives an introduction into the most important aspects of the library.
</li>
<li> <a href=\"modelica://Modelica.Mechanics.MultiBody.UsersGuide.Upgrade\">Upgrade</a> describes
  how to upgrade from former versions, especially from the \"old\"
  ModelicaAdditions.MultiBody library.
</li>
<li> <a href=\"modelica://Modelica.Mechanics.MultiBody.UsersGuide.Literature\">Literature</a> provides
  references that have been used to design and implement this library.
</li>
<li> <a href=\"modelica://Modelica.Mechanics.MultiBody.UsersGuide.Contact\">Contact</a> provides
  information about the author of the library as well as
  acknowledgments.
</li>
</ol>
</HTML>"));
      end UsersGuide;

      model World "World coordinate system + gravity field + default animation definition"
        import Modelica.Mechanics.MultiBody.Types.GravityTypes;
        import Modelica.Mechanics.MultiBody.Types;
        Interfaces.Frame_b frame_b "Coordinate system fixed in the origin of the world frame" annotation(
          Placement(transformation(extent = {{84, -16}, {116, 16}}, rotation = 0)));
        parameter Boolean enableAnimation = true "= true, if animation of all components is enabled";
        parameter Boolean animateWorld = true "= true, if world coordinate system shall be visualized" annotation(
          Dialog(enable = enableAnimation));
        parameter Boolean animateGravity = true "= true, if gravity field shall be visualized (acceleration vector or field center)" annotation(
          Dialog(enable = enableAnimation));
        parameter Types.AxisLabel label1 = "x" "Label of horizontal axis in icon";
        parameter Types.AxisLabel label2 = "y" "Label of vertical axis in icon";
        parameter Types.GravityTypes gravityType = GravityTypes.UniformGravity "Type of gravity field" annotation(
          Evaluate = true);
        parameter SI.Acceleration g = 9.81 "Constant gravity acceleration" annotation(
          Dialog(enable = gravityType == Modelica.Mechanics.MultiBody.Types.GravityTypes.UniformGravity));
        parameter Types.Axis n = {0, -1, 0} "Direction of gravity resolved in world frame (gravity = g*n/length(n))" annotation(
          Evaluate = true,
          Dialog(enable = gravityType == Modelica.Mechanics.MultiBody.Types.GravityTypes.UniformGravity));
        parameter Real mue(unit = "m3/s2", min = 0) = 3.986e14 "Gravity field constant (default = field constant of earth)" annotation(
          Dialog(enable = gravityType == Modelica.Mechanics.MultiBody.Types.GravityTypes.PointGravity));
        parameter Boolean driveTrainMechanics3D = true "= true, if 3-dim. mechanical effects of Parts.Mounting1D/Rotor1D/BevelGear1D shall be taken into account";
        parameter SI.Distance axisLength = nominalLength / 2 "Length of world axes arrows" annotation(
          Dialog(tab = "Animation", group = "if animateWorld = true", enable = enableAnimation and animateWorld));
        parameter SI.Distance axisDiameter = axisLength / defaultFrameDiameterFraction "Diameter of world axes arrows" annotation(
          Dialog(tab = "Animation", group = "if animateWorld = true", enable = enableAnimation and animateWorld));
        parameter Boolean axisShowLabels = true "= true, if labels shall be shown" annotation(
          Dialog(tab = "Animation", group = "if animateWorld = true", enable = enableAnimation and animateWorld));
        input Types.Color axisColor_x = Modelica.Mechanics.MultiBody.Types.Defaults.FrameColor "Color of x-arrow" annotation(
          Dialog(colorSelector = true, tab = "Animation", group = "if animateWorld = true", enable = enableAnimation and animateWorld));
        input Types.Color axisColor_y = axisColor_x annotation(
          Dialog(colorSelector = true, tab = "Animation", group = "if animateWorld = true", enable = enableAnimation and animateWorld));
        input Types.Color axisColor_z = axisColor_x "Color of z-arrow" annotation(
          Dialog(colorSelector = true, tab = "Animation", group = "if animateWorld = true", enable = enableAnimation and animateWorld));
        parameter SI.Position gravityArrowTail[3] = {0, 0, 0} "Position vector from origin of world frame to arrow tail, resolved in world frame" annotation(
          Dialog(tab = "Animation", group = "if animateGravity = true and gravityType = UniformGravity", enable = enableAnimation and animateGravity and gravityType == GravityTypes.UniformGravity));
        parameter SI.Length gravityArrowLength = axisLength / 2 "Length of gravity arrow" annotation(
          Dialog(tab = "Animation", group = "if animateGravity = true and gravityType = UniformGravity", enable = enableAnimation and animateGravity and gravityType == GravityTypes.UniformGravity));
        parameter SI.Diameter gravityArrowDiameter = gravityArrowLength / defaultWidthFraction "Diameter of gravity arrow" annotation(
          Dialog(tab = "Animation", group = "if animateGravity = true and gravityType = UniformGravity", enable = enableAnimation and animateGravity and gravityType == GravityTypes.UniformGravity));
        input Types.Color gravityArrowColor = {0, 230, 0} "Color of gravity arrow" annotation(
          Dialog(colorSelector = true, tab = "Animation", group = "if animateGravity = true and gravityType = UniformGravity", enable = enableAnimation and animateGravity and gravityType == GravityTypes.UniformGravity));
        parameter SI.Diameter gravitySphereDiameter = 12742000 "Diameter of sphere representing gravity center (default = mean diameter of earth)" annotation(
          Dialog(tab = "Animation", group = "if animateGravity = true and gravityType = PointGravity", enable = enableAnimation and animateGravity and gravityType == GravityTypes.PointGravity));
        input Types.Color gravitySphereColor = {0, 230, 0} "Color of gravity sphere" annotation(
          Dialog(colorSelector = true, tab = "Animation", group = "if animateGravity = true and gravityType = PointGravity", enable = enableAnimation and animateGravity and gravityType == GravityTypes.PointGravity));
        parameter SI.Length nominalLength = 1 "\"Nominal\" length of multi-body system" annotation(
          Dialog(tab = "Defaults"));
        parameter SI.Length defaultAxisLength = nominalLength / 5 "Default for length of a frame axis (but not world frame)" annotation(
          Dialog(tab = "Defaults"));
        parameter SI.Length defaultJointLength = nominalLength / 10 "Default for the fixed length of a shape representing a joint" annotation(
          Dialog(tab = "Defaults"));
        parameter SI.Length defaultJointWidth = nominalLength / 20 "Default for the fixed width of a shape representing a joint" annotation(
          Dialog(tab = "Defaults"));
        parameter SI.Length defaultForceLength = nominalLength / 10 "Default for the fixed length of a shape representing a force (e.g., damper)" annotation(
          Dialog(tab = "Defaults"));
        parameter SI.Length defaultForceWidth = nominalLength / 20 "Default for the fixed width of a shape representing a force (e.g., spring, bushing)" annotation(
          Dialog(tab = "Defaults"));
        parameter SI.Length defaultBodyDiameter = nominalLength / 9 "Default for diameter of sphere representing the center of mass of a body" annotation(
          Dialog(tab = "Defaults"));
        parameter Real defaultWidthFraction = 20 "Default for shape width as a fraction of shape length (e.g., for Parts.FixedTranslation)" annotation(
          Dialog(tab = "Defaults"));
        parameter SI.Length defaultArrowDiameter = nominalLength / 40 "Default for arrow diameter (e.g., of forces, torques, sensors)" annotation(
          Dialog(tab = "Defaults"));
        parameter Real defaultFrameDiameterFraction = 40 "Default for arrow diameter of a coordinate system as a fraction of axis length" annotation(
          Dialog(tab = "Defaults"));
        parameter Real defaultSpecularCoefficient(min = 0) = 0.7 "Default reflection of ambient light (= 0: light is completely absorbed)" annotation(
          Dialog(tab = "Defaults"));
        parameter Real defaultN_to_m(unit = "N/m", min = 0) = 1000 "Default scaling of force arrows (length = force/defaultN_to_m)" annotation(
          Dialog(tab = "Defaults"));
        parameter Real defaultNm_to_m(unit = "N.m/m", min = 0) = 1000 "Default scaling of torque arrows (length = torque/defaultNm_to_m)" annotation(
          Dialog(tab = "Defaults"));
        replaceable function gravityAcceleration = Modelica.Mechanics.MultiBody.Forces.Internal.standardGravityAcceleration(gravityType = gravityType, g = g * Modelica.Math.Vectors.normalizeWithAssert(n), mue = mue) constrainedby Modelica.Mechanics.MultiBody.Interfaces.partialGravityAcceleration "Function to compute the gravity acceleration, resolved in world frame" annotation(
           choicesAllMatching = true,
           Dialog(enable = gravityType == Modelica.Mechanics.MultiBody.Types.GravityTypes.NoGravity),
           Documentation(info = "<html>
<p>Replaceable function to define the gravity field.
   Default is function
   <a href=\"modelica://Modelica.Mechanics.MultiBody.Forces.Internal.standardGravityAcceleration\">standardGravityAcceleration</a>
   that provides some simple gravity fields (no gravity, constant parallel gravity field,
   point gravity field).
   By redeclaring this function, any type of gravity field can be defined, see example
     <a href=\"modelica://Modelica.Mechanics.MultiBody.Examples.Elementary.UserDefinedGravityField\">Examples.Elementary.UserDefinedGravityField</a>.
</p>
</html>"));
        /* The World object can only use the Modelica.Mechanics.MultiBody.Visualizers.Advanced.Shape model, but no
             other models in package Modelica.Mechanics.MultiBody.Visualizers, since the other models access
             data of the "outer Modelica.Mechanics.MultiBody.World world" object, i.e., there are
             mutually dependent classes. For this reason, the higher level visualization
             objects cannot be used.
          */
      protected
        parameter Integer ndim = if enableAnimation and animateWorld then 1 else 0;
        parameter Integer ndim2 = if enableAnimation and animateWorld and axisShowLabels then 1 else 0;
        // Parameters to define axes
        parameter SI.Length headLength = min(axisLength, axisDiameter * Types.Defaults.FrameHeadLengthFraction);
        parameter SI.Length headWidth = axisDiameter * Types.Defaults.FrameHeadWidthFraction;
        parameter SI.Length lineLength = max(0, axisLength - headLength);
        parameter SI.Length lineWidth = axisDiameter;
        // Parameters to define axes labels
        parameter SI.Length scaledLabel = Modelica.Mechanics.MultiBody.Types.Defaults.FrameLabelHeightFraction * axisDiameter;
        parameter SI.Length labelStart = 1.05 * axisLength;
        // x-axis
        Modelica.Mechanics.MultiBody.Visualizers.Advanced.Shape x_arrowLine(shapeType = "cylinder", length = lineLength, width = lineWidth, height = lineWidth, lengthDirection = {1, 0, 0}, widthDirection = {0, 1, 0}, color = axisColor_x, specularCoefficient = 0) if enableAnimation and animateWorld;
        Modelica.Mechanics.MultiBody.Visualizers.Advanced.Shape x_arrowHead(shapeType = "cone", length = headLength, width = headWidth, height = headWidth, lengthDirection = {1, 0, 0}, widthDirection = {0, 1, 0}, color = axisColor_x, r = {lineLength, 0, 0}, specularCoefficient = 0) if enableAnimation and animateWorld;
        Modelica.Mechanics.MultiBody.Visualizers.Internal.Lines x_label(lines = scaledLabel * {[0, 0; 1, 1], [0, 1; 1, 0]}, diameter = axisDiameter, color = axisColor_x, r_lines = {labelStart, 0, 0}, n_x = {1, 0, 0}, n_y = {0, 1, 0}, specularCoefficient = 0) if enableAnimation and animateWorld and axisShowLabels;
        // y-axis
        Modelica.Mechanics.MultiBody.Visualizers.Advanced.Shape y_arrowLine(shapeType = "cylinder", length = lineLength, width = lineWidth, height = lineWidth, lengthDirection = {0, 1, 0}, widthDirection = {1, 0, 0}, color = axisColor_y, specularCoefficient = 0) if enableAnimation and animateWorld;
        Modelica.Mechanics.MultiBody.Visualizers.Advanced.Shape y_arrowHead(shapeType = "cone", length = headLength, width = headWidth, height = headWidth, lengthDirection = {0, 1, 0}, widthDirection = {1, 0, 0}, color = axisColor_y, r = {0, lineLength, 0}, specularCoefficient = 0) if enableAnimation and animateWorld;
        Modelica.Mechanics.MultiBody.Visualizers.Internal.Lines y_label(lines = scaledLabel * {[0, 0; 1, 1.5], [0, 1.5; 0.5, 0.75]}, diameter = axisDiameter, color = axisColor_y, r_lines = {0, labelStart, 0}, n_x = {0, 1, 0}, n_y = {-1, 0, 0}, specularCoefficient = 0) if enableAnimation and animateWorld and axisShowLabels;
        // z-axis
        Modelica.Mechanics.MultiBody.Visualizers.Advanced.Shape z_arrowLine(shapeType = "cylinder", length = lineLength, width = lineWidth, height = lineWidth, lengthDirection = {0, 0, 1}, widthDirection = {0, 1, 0}, color = axisColor_z, specularCoefficient = 0) if enableAnimation and animateWorld;
        Modelica.Mechanics.MultiBody.Visualizers.Advanced.Shape z_arrowHead(shapeType = "cone", length = headLength, width = headWidth, height = headWidth, lengthDirection = {0, 0, 1}, widthDirection = {0, 1, 0}, color = axisColor_z, r = {0, 0, lineLength}, specularCoefficient = 0) if enableAnimation and animateWorld;
        Modelica.Mechanics.MultiBody.Visualizers.Internal.Lines z_label(lines = scaledLabel * {[0, 0; 1, 0], [0, 1; 1, 1], [0, 1; 1, 0]}, diameter = axisDiameter, color = axisColor_z, r_lines = {0, 0, labelStart}, n_x = {0, 0, 1}, n_y = {0, 1, 0}, specularCoefficient = 0) if enableAnimation and animateWorld and axisShowLabels;
        // Uniform gravity visualization
        parameter SI.Length gravityHeadLength = min(gravityArrowLength, gravityArrowDiameter * Types.Defaults.ArrowHeadLengthFraction);
        parameter SI.Length gravityHeadWidth = gravityArrowDiameter * Types.Defaults.ArrowHeadWidthFraction;
        parameter SI.Length gravityLineLength = max(0, gravityArrowLength - gravityHeadLength);
        Modelica.Mechanics.MultiBody.Visualizers.Advanced.Shape gravityArrowLine(shapeType = "cylinder", length = gravityLineLength, width = gravityArrowDiameter, height = gravityArrowDiameter, lengthDirection = n, widthDirection = {0, 1, 0}, color = gravityArrowColor, r_shape = gravityArrowTail, specularCoefficient = 0) if enableAnimation and animateGravity and gravityType == GravityTypes.UniformGravity;
        Modelica.Mechanics.MultiBody.Visualizers.Advanced.Shape gravityArrowHead(shapeType = "cone", length = gravityHeadLength, width = gravityHeadWidth, height = gravityHeadWidth, lengthDirection = n, widthDirection = {0, 1, 0}, color = gravityArrowColor, r_shape = gravityArrowTail + Modelica.Math.Vectors.normalize(n) * gravityLineLength, specularCoefficient = 0) if enableAnimation and animateGravity and gravityType == GravityTypes.UniformGravity;
        // Point gravity visualization
        parameter Integer ndim_pointGravity = if enableAnimation and animateGravity and gravityType == GravityTypes.UniformGravity then 1 else 0;
        Modelica.Mechanics.MultiBody.Visualizers.Advanced.Shape gravitySphere(shapeType = "sphere", r_shape = {-gravitySphereDiameter / 2, 0, 0}, lengthDirection = {1, 0, 0}, length = gravitySphereDiameter, width = gravitySphereDiameter, height = gravitySphereDiameter, color = gravitySphereColor, specularCoefficient = 0) if enableAnimation and animateGravity and gravityType == GravityTypes.PointGravity;
        /*
          function gravityAcceleration = gravityAccelerationTypes (
              gravityType=gravityType,
              g=g*Modelica.Math.Vectors.normalize(
                                             n),
              mue=mue);
        */
      equation
        Connections.root(frame_b.R);
        assert(Modelica.Math.Vectors.length(n) > 1.e-10, "Parameter n of World object is wrong (length(n) > 0 required)");
        frame_b.r_0 = zeros(3);
        frame_b.R = Frames.nullRotation();
        annotation(
          defaultComponentName = "world",
          defaultComponentPrefixes = "inner",
          missingInnerMessage = "No \"world\" component is defined. A default world
component with the default gravity field will be used
(g=9.81 in negative y-axis). If this is not desired,
drag Modelica.Mechanics.MultiBody.World into the top level of your model.",
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, 100}, {100, -100}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{-100, -118}, {-100, 61}}, color = {0, 0, 0}, thickness = 0.5), Polygon(points = {{-100, 100}, {-120, 60}, {-80, 60}, {-100, 100}, {-100, 100}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Line(points = {{-119, -100}, {59, -100}}, color = {0, 0, 0}, thickness = 0.5), Polygon(points = {{99, -100}, {59, -80}, {59, -120}, {99, -100}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Text(extent = {{-150, 145}, {150, 105}}, textString = "%name", lineColor = {0, 0, 255}), Text(extent = {{95, -113}, {144, -162}}, lineColor = {0, 0, 0}, textString = "%label1"), Text(extent = {{-170, 127}, {-119, 77}}, lineColor = {0, 0, 0}, textString = "%label2"), Line(points = {{-56, 78}, {-56, -26}}, color = {0, 0, 255}), Polygon(points = {{-68, -26}, {-56, -66}, {-44, -26}, {-68, -26}}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Line(points = {{2, 78}, {2, -26}}, color = {0, 0, 255}), Polygon(points = {{-10, -26}, {2, -66}, {14, -26}, {-10, -26}}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Line(points = {{66, 80}, {66, -26}}, color = {0, 0, 255}), Polygon(points = {{54, -26}, {66, -66}, {78, -26}, {54, -26}}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255})}),
          Documentation(info = "<HTML>
<p>
Model <b>World</b> represents a global coordinate system fixed in
ground. This model serves several purposes:
<ul>
<li> It is used as <b>inertial system</b> in which
     the equations of all elements of the MultiBody library
     are defined.</li>
<li> It is the world frame of an <b>animation window</b> in which
     all elements of the MultiBody library are visualized.</li>
<li> It is used to define the <b>gravity field</b> in which a
     multi-body model is present. Default is a uniform gravity
     field where the gravity acceleration vector g is the
     same at every position. Additionally, a point gravity field or no
     gravity can be selected. Also, function gravityAcceleration can
     be redeclared to a user-defined function that computes the gravity
     acceleration, see example
     <a href=\"modelica://Modelica.Mechanics.MultiBody.Examples.Elementary.UserDefinedGravityField\">Examples.Elementary.UserDefinedGravityField</a>.
     </li>
<li> It is used to define <b>default settings</b> of animation properties
     (e.g., the diameter of a sphere representing by default
     the center of mass of a body, or the diameters of the cylinders
     representing a revolute joint).</li>
<li> It is used to define a <b>visual representation</b> of the
     world model (= 3 coordinate axes with labels) and of the defined
     gravity field.<br>
    <IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/world.png\" ALT=\"MultiBody.World\">
</li>
</ul>
<p>
Since the gravity field function is required from all bodies with mass
and the default settings of animation properties are required
from nearly every component, exactly one instance of model World needs
to be present in every model on the top level. The basic declaration
needs to be:
</p>
<pre>
    <b>inner</b> Modelica.Mechanics.MultiBody.World world
</pre>
<p>
Note, it must be an <b>inner</b> declaration with instance name <b>world</b>
in order that this world object can be accessed from all objects in the
model. When dragging the \"World\" object from the package browser into
the diagram layer, this declaration is automatically generated
(this is defined via annotations in model World).
</p>
<p>
All vectors and tensors of a mechanical system are resolved in a
frame that is local to the corresponding component. Usually,
if all relative joint coordinates vanish, the local frames
of all components are parallel to each other, as well as to the
world frame (this holds as long as a Parts.FixedRotation,
component is <b>not</b> used). In this \"reference configuration\"
it is therefore
alternatively possible to resolve all vectors in the world
frame, since all frames are parallel to each other.
This is often very convenient. In order to give some visual
support in such a situation, in the icon of a World instance
two axes of the world frame are shown and the labels
of these axes can be set via parameters.
</p>
</html>"));
      end World;

      package Examples "Examples that demonstrate the usage of the MultiBody library"
        extends Modelica.Icons.ExamplesPackage;

        package Elementary "Elementary examples to demonstrate various features of the MultiBody library"
          extends Modelica.Icons.ExamplesPackage;

          model DoublePendulum "Simple double pendulum with two revolute joints and two bodies"
            extends Modelica.Icons.Example;
            inner Modelica.Mechanics.MultiBody.World world annotation(
              Placement(transformation(extent = {{-88, 0}, {-68, 20}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Joints.Revolute revolute1(useAxisFlange = true, phi(fixed = true), w(fixed = true)) annotation(
              Placement(transformation(extent = {{-48, 0}, {-28, 20}}, rotation = 0)));
            Modelica.Mechanics.Rotational.Components.Damper damper(d = 0.1) annotation(
              Placement(transformation(extent = {{-48, 40}, {-28, 60}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Parts.BodyBox boxBody1(r = {0.5, 0, 0}, width = 0.06) annotation(
              Placement(transformation(extent = {{-10, 0}, {10, 20}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Joints.Revolute revolute2(phi(fixed = true), w(fixed = true)) annotation(
              Placement(transformation(extent = {{32, 0}, {52, 20}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Parts.BodyBox boxBody2(r = {0.5, 0, 0}, width = 0.06) annotation(
              Placement(transformation(extent = {{74, 0}, {94, 20}}, rotation = 0)));
          equation
            connect(damper.flange_b, revolute1.axis) annotation(
              Line(points = {{-28, 50}, {-24, 50}, {-24, 28}, {-38, 28}, {-38, 20}}, color = {0, 0, 0}));
            connect(revolute1.support, damper.flange_a) annotation(
              Line(points = {{-44, 20}, {-44, 28}, {-58, 28}, {-58, 50}, {-48, 50}}, color = {0, 0, 0}));
            connect(revolute1.frame_b, boxBody1.frame_a) annotation(
              Line(points = {{-28, 10}, {-10, 10}}, color = {95, 95, 95}, thickness = 0.5));
            connect(revolute2.frame_b, boxBody2.frame_a) annotation(
              Line(points = {{52, 10}, {74, 10}}, color = {95, 95, 95}, thickness = 0.5));
            connect(boxBody1.frame_b, revolute2.frame_a) annotation(
              Line(points = {{10, 10}, {32, 10}}, color = {95, 95, 95}, thickness = 0.5));
            connect(world.frame_b, revolute1.frame_a) annotation(
              Line(points = {{-68, 10}, {-48, 10}}, color = {95, 95, 95}, thickness = 0.5));
            annotation(
              experiment(StopTime = 3),
              Documentation(info = "<html>
<p>
This example demonstrates that by using joint and body
elements animation is automatically available. Also the revolute
joints are animated. Note, that animation of every component
can be switched of by setting the first parameter <b>animation</b>
to <b>false</b> or by setting <b>enableAnimation</b> in the <b>world</b>
object to <b>false</b> to switch off animation of all components.
</p>

<table border=0 cellspacing=0 cellpadding=0><tr><td valign=\"top\">
<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Examples/Elementary/DoublePendulum.png\"
ALT=\"model Examples.Elementary.DoublePendulum\">
</td></tr></table>

</HTML>"));
          end DoublePendulum;

          model DoublePendulumInitTip "Demonstrate how to initialize a double pendulum so that its tip starts at a predefined position"
            extends Modelica.Icons.Example;
            inner World world annotation(
              Placement(transformation(extent = {{-100, 0}, {-80, 20}}, rotation = 0)));
            Joints.Revolute revolute1(useAxisFlange = true) annotation(
              Placement(transformation(extent = {{-60, 0}, {-40, 20}}, rotation = 0)));
            Rotational.Components.Damper damper(d = 0.1) annotation(
              Placement(transformation(extent = {{-60, 40}, {-40, 60}}, rotation = 0)));
            Parts.BodyBox boxBody1(r = {0.5, 0, 0}, width = 0.06) annotation(
              Placement(transformation(extent = {{-22, 0}, {-2, 20}}, rotation = 0)));
            Joints.Revolute revolute2 annotation(
              Placement(transformation(extent = {{20, 0}, {40, 20}}, rotation = 0)));
            Parts.BodyBox boxBody2(r = {0.5, 0, 0}, width = 0.06) annotation(
              Placement(transformation(extent = {{62, 0}, {82, 20}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Joints.FreeMotionScalarInit freeMotionScalarInit(use_r = true, r_rel_a_1(start = 0.7, fixed = true), r_rel_a_2(start = 0.3, fixed = true), use_v = true, v_rel_a_1(fixed = true), v_rel_a_2(fixed = true)) annotation(
              Placement(transformation(extent = {{-20, -40}, {0, -20}})));
          equation
            connect(damper.flange_b, revolute1.axis) annotation(
              Line(points = {{-40, 50}, {-36, 50}, {-36, 28}, {-50, 28}, {-50, 20}}, color = {0, 0, 0}));
            connect(revolute1.support, damper.flange_a) annotation(
              Line(points = {{-56, 20}, {-56, 28}, {-70, 28}, {-70, 50}, {-60, 50}}, color = {0, 0, 0}));
            connect(revolute1.frame_b, boxBody1.frame_a) annotation(
              Line(points = {{-40, 10}, {-22, 10}}, color = {95, 95, 95}, thickness = 0.5));
            connect(revolute2.frame_b, boxBody2.frame_a) annotation(
              Line(points = {{40, 10}, {62, 10}}, color = {95, 95, 95}, thickness = 0.5));
            connect(boxBody1.frame_b, revolute2.frame_a) annotation(
              Line(points = {{-2, 10}, {20, 10}}, color = {95, 95, 95}, thickness = 0.5));
            connect(world.frame_b, revolute1.frame_a) annotation(
              Line(points = {{-80, 10}, {-60, 10}}, color = {95, 95, 95}, thickness = 0.5));
            connect(world.frame_b, freeMotionScalarInit.frame_a) annotation(
              Line(points = {{-80, 10}, {-66, 10}, {-66, -30}, {-20, -30}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            connect(freeMotionScalarInit.frame_b, boxBody2.frame_b) annotation(
              Line(points = {{0, -30}, {88, -30}, {88, 10}, {82, 10}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            annotation(
              experiment(StopTime = 5),
              Documentation(info = "<html>
<p>
This example demonstrates at hand of a double pendulum,
how no-standard initialization can be defined:
The absolute position of the pendulum tip, and its absolute speed
shall be initially defined. This can be performed with the
<a href=\"modelica://Modelica.Mechanics.MultiBody.Joints.FreeMotionScalarInit\">Joints.FreeMotionScalarInit</a>
joint that allows to initialize individual elements of its relative vectors.
In this case, the x-, and y-coordinates of the relative position vector
(visualized by the yellow arrow in the figure below) and of its
derivative shall have a defined value at initial time.
The configuration of the double pendulum at the initial time is
shown below, where the tip position is required to have the coordinates
x=0.7, y=0.3.
</p>

<blockquote>
<img src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Examples/Elementary/DoublePendulumInitTip.png\">
</blockquote>

</html>"));
          end DoublePendulumInitTip;

          model ForceAndTorque "Demonstrate usage of ForceAndTorque element"
            extends Modelica.Icons.Example;
            inner World world(animateGravity = false) annotation(
              Placement(transformation(extent = {{-90, 30}, {-70, 50}}, rotation = 0)));
            Parts.BodyCylinder body(r = {1, 0, 0}) annotation(
              Placement(transformation(extent = {{0, 30}, {20, 50}}, rotation = 0)));
            Parts.Fixed fixed1(r = {0, -0.5, 0}, width = 0.03) annotation(
              Placement(transformation(extent = {{-90, -10}, {-70, 10}}, rotation = 0)));
            Parts.FixedRotation fixedRotation(n = {0, 0, 1}, angle = 30) annotation(
              Placement(transformation(extent = {{-8, -30}, {12, -10}}, rotation = 0)));
            Forces.ForceAndTorque forceAndTorque(Nm_to_m = 120, N_to_m = 1200, resolveInFrame = Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB.frame_resolve) annotation(
              Placement(transformation(extent = {{60, 50}, {40, 30}}, rotation = 0)));
            Joints.Revolute revolute2(n = {0, 1, 0}, phi(fixed = true), w(fixed = true)) annotation(
              Placement(transformation(origin = {-20, 20}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
            Modelica.Blocks.Sources.Constant torque[3](k = {-100, 100, 0}) annotation(
              Placement(transformation(origin = {40, -10}, extent = {{10, -10}, {-10, 10}}, rotation = 270)));
            Joints.Revolute revolute1(phi(fixed = true), w(fixed = true)) annotation(
              Placement(transformation(extent = {{-60, -10}, {-40, 10}}, rotation = 0)));
            Parts.Fixed fixed2(width = 0.03, r = {1.5, 0.25, 0}) annotation(
              Placement(transformation(extent = {{100, 30}, {80, 50}}, rotation = 0)));
            Modelica.Blocks.Sources.Constant force[3](k = {0, 1000, 0}) annotation(
              Placement(transformation(origin = {80, -10}, extent = {{10, -10}, {-10, 10}}, rotation = 270)));
          equation
            connect(revolute2.frame_b, body.frame_a) annotation(
              Line(points = {{-20, 30}, {-20, 40}, {0, 40}}, color = {95, 95, 95}, thickness = 0.5));
            connect(forceAndTorque.frame_b, body.frame_b) annotation(
              Line(points = {{40, 40}, {20, 40}}, color = {95, 95, 95}, thickness = 0.5));
            connect(fixed1.frame_b, revolute1.frame_a) annotation(
              Line(points = {{-70, 0}, {-60, 0}}, color = {95, 95, 95}, thickness = 0.5));
            connect(revolute1.frame_b, revolute2.frame_a) annotation(
              Line(points = {{-40, 0}, {-20, 0}, {-20, 10}}, color = {95, 95, 95}, thickness = 0.5));
            connect(fixed2.frame_b, forceAndTorque.frame_a) annotation(
              Line(points = {{80, 40}, {60, 40}}, color = {95, 95, 95}, thickness = 0.5));
            connect(fixedRotation.frame_a, fixed1.frame_b) annotation(
              Line(points = {{-8, -20}, {-65, -20}, {-65, 0}, {-70, 0}}, color = {95, 95, 95}, thickness = 0.5));
            connect(forceAndTorque.frame_resolve, fixedRotation.frame_b) annotation(
              Line(points = {{42, 30}, {42, 30}, {20, 30}, {20, -20}, {12, -20}}, color = {95, 95, 95}, pattern = LinePattern.Dot));
            connect(force.y, forceAndTorque.force) annotation(
              Line(points = {{80, 1}, {80, 10}, {58, 10}, {58, 28}}, color = {0, 0, 127}, smooth = Smooth.None));
            connect(torque.y, forceAndTorque.torque) annotation(
              Line(points = {{40, 1}, {40, 10}, {50, 10}, {50, 28}}, color = {0, 0, 127}, smooth = Smooth.None));
            annotation(
              Documentation(info = "<HTML>
<p>
In this example the usage of the general force element
\"<a href=\"modelica://Modelica.Mechanics.MultiBody.Forces.ForceAndTorque\">ForceAndTorque</a>\"
is shown. A \"ForceAndTorque\" element is connected
between a body and a fixed point in the world system. The force and torque
is defined by the \"Constant\" block. The two vectors are resolved in the
coordinate system defined by the \"fixedRotation\" component that is
fixed in the world system:
</p>
<p>
The animation view at time = 0 is shown in the figure below.
The yellow line is directed from frame_a to frame_b of the
forceAndTorque component. The green arrow characterizes the
force acting at the body whereas the green double arrow characterizes
the torque acting at the body. The lengths of the two vectors
are proportional to the lengths of the force and torque vectors
(constant scaling factors are defined as parameters in the
forceAndTorque component):
</p>
<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Examples/Elementary/ForceAndTorque.png\">
</html>"),
              experiment(StopTime = 1.01));
          end ForceAndTorque;

          model FreeBody "Free flying body attached by two springs to environment"
            extends Modelica.Icons.Example;
            parameter Boolean animation = true "= true, if animation shall be enabled";
            inner Modelica.Mechanics.MultiBody.World world annotation(
              Placement(transformation(extent = {{-60, 20}, {-40, 40}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Parts.FixedTranslation bar2(r = {0.8, 0, 0}, animation = false) annotation(
              Placement(transformation(extent = {{0, 20}, {20, 40}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Forces.Spring spring1(width = 0.1, coilWidth = 0.005, numberOfWindings = 5, c = 20, s_unstretched = 0) annotation(
              Placement(transformation(origin = {-20, 6}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
            Modelica.Mechanics.MultiBody.Parts.BodyShape body(m = 1, I_11 = 1, I_22 = 1, I_33 = 1, r = {0.4, 0, 0}, r_CM = {0.2, 0, 0}, width = 0.05, r_0(start = {0.2, -0.5, 0.1}, each fixed = true), v_0(each fixed = true), angles_fixed = true, w_0_fixed = true, angles_start = {0.174532925199433, 0.174532925199433, 0.174532925199433}) annotation(
              Placement(transformation(extent = {{0, -40}, {20, -20}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Forces.Spring spring2(c = 20, s_unstretched = 0, width = 0.1, coilWidth = 0.005, numberOfWindings = 5) annotation(
              Placement(transformation(origin = {40, 6}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          equation
            connect(bar2.frame_a, world.frame_b) annotation(
              Line(points = {{0, 30}, {-40, 30}}, color = {95, 95, 95}, thickness = 0.5));
            connect(spring1.frame_b, body.frame_a) annotation(
              Line(points = {{-20, -4}, {-20, -30}, {0, -30}}, color = {95, 95, 95}, thickness = 0.5));
            connect(bar2.frame_b, spring2.frame_a) annotation(
              Line(points = {{20, 30}, {40, 30}, {40, 16}}, color = {95, 95, 95}, thickness = 0.5));
            connect(spring1.frame_a, world.frame_b) annotation(
              Line(points = {{-20, 16}, {-20, 30}, {-40, 30}}, color = {95, 95, 95}, thickness = 0.5));
            connect(body.frame_b, spring2.frame_b) annotation(
              Line(points = {{20, -30}, {40, -30}, {40, -4}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            annotation(
              experiment(StopTime = 10),
              Documentation(info = "<html>
<p>
This example demonstrates:
</p>
<ul>
<li>The animation of spring and damper components</li>
<li>A body can be freely moving without any connection to a joint.
    In this case body coordinates are used automatically as
    states (whenever joints are present, it is first tried to
    use the generalized coordinates of the joints as states).</li>
<li>If a body is freely moving, the initial position and velocity of the body
    can be defined with the \"Initialization\" menu as shown with the
    body \"body1\" in the left part (click on \"Initialization\").</li>
</ul>

<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Examples/Elementary/FreeBody.png\"
ALT=\"model Examples.Elementary.FreeBody\">
</html>"));
          end FreeBody;

          model InitSpringConstant "Determine spring constant such that system is in steady state at given position"
            extends Modelica.Icons.Example;
            inner Modelica.Mechanics.MultiBody.World world(gravityType = Modelica.Mechanics.MultiBody.Types.GravityTypes.UniformGravity) annotation(
              Placement(transformation(extent = {{-80, 0}, {-60, 20}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Joints.Revolute rev(useAxisFlange = true, n = {0, 0, 1}, phi(fixed = true), w(fixed = true), a(fixed = true)) annotation(
              Placement(transformation(extent = {{-40, 0}, {-20, 20}}, rotation = 0)));
            Modelica.Mechanics.Rotational.Components.Damper damper(d = 0.1) annotation(
              Placement(transformation(extent = {{-40, 40}, {-20, 60}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Parts.BodyShape body(r = {1, 0, 0}, r_CM = {0.5, 0, 0}, m = 1) annotation(
              Placement(transformation(extent = {{0, 0}, {20, 20}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Parts.Fixed fixed(r = {1, 0.2, 0}, width = 0.02) annotation(
              Placement(transformation(origin = {50, 70}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
            Modelica.Mechanics.MultiBody.Forces.Spring spring(s_unstretched = 0.1, c(fixed = false, start = 100)) annotation(
              Placement(transformation(origin = {50, 30}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          equation
            connect(world.frame_b, rev.frame_a) annotation(
              Line(points = {{-60, 10}, {-40, 10}}, color = {95, 95, 95}, thickness = 0.5));
            connect(damper.flange_b, rev.axis) annotation(
              Line(points = {{-20, 50}, {-16, 50}, {-16, 26}, {-30, 26}, {-30, 20}}, color = {0, 0, 0}));
            connect(rev.support, damper.flange_a) annotation(
              Line(points = {{-36, 20}, {-36, 26}, {-48, 26}, {-48, 50}, {-40, 50}}, color = {0, 0, 0}));
            connect(rev.frame_b, body.frame_a) annotation(
              Line(points = {{-20, 10}, {0, 10}}, color = {95, 95, 95}, thickness = 0.5));
            connect(fixed.frame_b, spring.frame_a) annotation(
              Line(points = {{50, 60}, {50, 40}}, color = {95, 95, 95}, thickness = 0.5));
            connect(body.frame_b, spring.frame_b) annotation(
              Line(points = {{20, 10}, {50, 10}, {50, 20}}, color = {95, 95, 95}, thickness = 0.5));
            annotation(
              Documentation(info = "<html>
<p>
This example demonstrates a non-standard type of initialization
by calculating a spring constant such
that a simple pendulum is at a defined position in steady state.
</p>
<p>
The goal is that the pendulum should be in steady state
when the rotation angle of the pendulum is zero. The spring
constant of the spring shall be calculated during initialization
such that this goal is reached.
</p>
<p>
The pendulum has one degree of freedom, i.e., two states.
Therefore, two additional equations have to be provided
for initialization. However, parameter \"c\" of the spring
component is defined with attribute \"fixed = <b>false</b>\", i.e.,
the value of this parameter is computed during initialization.
Therefore, there is one additional equation required during
initialization. The 3 initial equations are the rotational
angle of the revolute joint and its first and second
derivative. The latter ones are zero, in order to initialize
in steady state. By setting the start values of phi, w, a to zero and
their fixed attributes to true, the required
3 initial equations are defined.
</p>
<p>
After translation, this model is initialized in steady-state.
The spring constant is computed as c = 49.05 N/m.
An animation of this simulation is shown in the figure below.
</p>

<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Examples/Elementary/InitSpringConstant.png\"
ALT=\"model Examples.Elementary.InitSpringConstant\">
</html>"),
              experiment(StopTime = 1.01));
          end InitSpringConstant;

          model LineForceWithTwoMasses "Demonstrate line force with two point masses using a JointUPS and alternatively a LineForceWithTwoMasses component"
            extends Modelica.Icons.Example;
            parameter Modelica.SIunits.Mass m = 1 "Mass of point masses";
            SI.Force rod_f_diff[3] = rod1.frame_b.f - rod3.frame_b.f "Difference of cut-forces in rod1 and rod3";
            SI.Force body_f_diff[3] = bodyBox1.frame_b.f - bodyBox2.frame_b.f "Difference of cut-forces in bodyBox1 and bodyBox2";
            inner Modelica.Mechanics.MultiBody.World world annotation(
              Placement(transformation(extent = {{-80, 60}, {-60, 80}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Joints.Revolute revolute1(phi(fixed = true), w(fixed = true)) annotation(
              Placement(transformation(extent = {{-20, 60}, {0, 80}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Parts.BodyBox bodyBox1(r = {0.7, 0, 0}) annotation(
              Placement(transformation(extent = {{20, 60}, {40, 80}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Parts.FixedTranslation rod1(r = {0, -0.9, 0}, width = 0.01, animation = false) annotation(
              Placement(transformation(origin = {-40, 54}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
            Modelica.Mechanics.MultiBody.Joints.Assemblies.JointUPS jointUPS(nAxis_ia = {0.7, 1.2, 0}, animation = true) annotation(
              Placement(transformation(extent = {{0, 50}, {20, 30}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Parts.Body body1(r_CM = 0.2 * jointUPS.eAxis_ia, cylinderDiameter = 0.05, animation = true, m = m, I_11 = 0, I_22 = 0, I_33 = 0) annotation(
              Placement(transformation(extent = {{-14, 14}, {-34, 34}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Parts.Body body2(r_CM = -0.2 * jointUPS.eAxis_ia, cylinderDiameter = 0.05, animation = true, m = m, I_11 = 0, I_22 = 0, I_33 = 0) annotation(
              Placement(transformation(extent = {{32, 14}, {52, 34}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Parts.FixedTranslation rod2(r = {0, 0.3, 0}, width = 0.01, animation = false) annotation(
              Placement(transformation(origin = {-40, 84}, extent = {{10, -10}, {-10, 10}}, rotation = 270)));
            Modelica.Mechanics.Translational.Components.Damper damper1(d = 3) annotation(
              Placement(transformation(extent = {{0, 24}, {20, 4}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Joints.Revolute revolute2(phi(fixed = true), w(fixed = true)) annotation(
              Placement(transformation(extent = {{-20, -40}, {0, -20}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Parts.BodyBox bodyBox2(r = {0.7, 0, 0}) annotation(
              Placement(transformation(extent = {{20, -40}, {40, -20}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Parts.FixedTranslation rod3(width = 0.01, r = {0, -0.9, 0.3}, animation = false) annotation(
              Placement(transformation(origin = {-40, -46}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
            Modelica.Mechanics.MultiBody.Parts.FixedTranslation rod4(width = 0.01, r = {0, 0.3, 0.3}, animation = false) annotation(
              Placement(transformation(origin = {-40, -16}, extent = {{10, -10}, {-10, 10}}, rotation = 270)));
            Modelica.Mechanics.Translational.Components.Damper damper2(d = 3) annotation(
              Placement(transformation(extent = {{0, -76}, {20, -96}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Forces.LineForceWithTwoMasses lineForceWithTwoMasses(L_a = 0.2, L_b = 0.2, cylinderLength_a = 0.2, cylinderLength_b = 1.2, massDiameterFaction = 2.2, m_a = m, m_b = m) annotation(
              Placement(transformation(extent = {{0, -50}, {20, -70}}, rotation = 0)));
          equation
            connect(jointUPS.bearing, damper1.flange_a) annotation(
              Line(points = {{6, 30}, {6, 20}, {0, 20}, {0, 14}}, color = {0, 191, 0}));
            connect(jointUPS.axis, damper1.flange_b) annotation(
              Line(points = {{14, 30}, {14, 20}, {20, 20}, {20, 14}}, color = {0, 191, 0}));
            connect(jointUPS.frame_ib, body2.frame_a) annotation(
              Line(points = {{18, 30}, {18, 24}, {32, 24}}, color = {95, 95, 95}, thickness = 0.5));
            connect(world.frame_b, rod2.frame_a) annotation(
              Line(points = {{-60, 70}, {-40, 70}, {-40, 74}}, color = {95, 95, 95}, thickness = 0.5));
            connect(world.frame_b, rod1.frame_a) annotation(
              Line(points = {{-60, 70}, {-40, 70}, {-40, 64}}, color = {95, 95, 95}, thickness = 0.5));
            connect(rod2.frame_b, revolute1.frame_a) annotation(
              Line(points = {{-40, 94}, {-40, 98}, {-28, 98}, {-28, 70}, {-20, 70}}, color = {95, 95, 95}, thickness = 0.5));
            connect(revolute1.frame_b, bodyBox1.frame_a) annotation(
              Line(points = {{0, 70}, {20, 70}}, color = {95, 95, 95}, thickness = 0.5));
            connect(bodyBox1.frame_b, jointUPS.frame_b) annotation(
              Line(points = {{40, 70}, {46, 70}, {46, 40}, {20, 40}}, color = {95, 95, 95}, thickness = 0.5));
            connect(body1.frame_a, jointUPS.frame_ia) annotation(
              Line(points = {{-14, 24}, {2, 24}, {2, 30}}, color = {95, 95, 95}, thickness = 0.5));
            connect(rod1.frame_b, jointUPS.frame_a) annotation(
              Line(points = {{-40, 44}, {-40, 40}, {0, 40}}, color = {95, 95, 95}, thickness = 0.5));
            connect(rod4.frame_b, revolute2.frame_a) annotation(
              Line(points = {{-40, -6}, {-40, -2}, {-28, -2}, {-28, -30}, {-20, -30}}, color = {95, 95, 95}, thickness = 0.5));
            connect(revolute2.frame_b, bodyBox2.frame_a) annotation(
              Line(points = {{0, -30}, {20, -30}}, color = {95, 95, 95}, thickness = 0.5));
            connect(world.frame_b, rod4.frame_a) annotation(
              Line(points = {{-60, 70}, {-52, 70}, {-52, -32}, {-40, -32}, {-40, -26}}, color = {95, 95, 95}, thickness = 0.5));
            connect(rod3.frame_a, rod4.frame_a) annotation(
              Line(points = {{-40, -36}, {-40, -26}}, color = {0, 0, 0}, thickness = 0.5));
            connect(lineForceWithTwoMasses.frame_a, rod3.frame_b) annotation(
              Line(points = {{0, -60}, {-40, -60}, {-40, -56}}, color = {95, 95, 95}, thickness = 0.5));
            connect(lineForceWithTwoMasses.frame_b, bodyBox2.frame_b) annotation(
              Line(points = {{20, -60}, {54, -60}, {54, -30}, {40, -30}}, color = {95, 95, 95}, thickness = 0.5));
            connect(lineForceWithTwoMasses.flange_b, damper2.flange_b) annotation(
              Line(points = {{16, -71}, {20, -71}, {20, -86}}, color = {0, 191, 0}));
            connect(lineForceWithTwoMasses.flange_a, damper2.flange_a) annotation(
              Line(points = {{4, -71}, {0, -71}, {0, -86}}, color = {0, 191, 0}));
            annotation(
              experiment(StopTime = 3),
              Documentation(info = "<html>
<p>
It is demonstrated how to implement line force components
that shall have mass properties. Two alternative implementations
are given:
</p>
<ul>
<li> With <a href=\"modelica://Modelica.Mechanics.MultiBody.Joints.Assemblies.JointUPS\">JointUPS</a>:<br>
     Modelica.Mechanics.MultiBody.Joints.Assemblies.JointUPS is an aggregation
     of a universal, a prismatic and a spherical joint that approximates
     a real force component, such as a hydraulic cylinder. At the two
     frames of the prismatic joint (frame_ia, frame_ib of jointUPS)
     two bodies are attached. The parameters are selected such that
     the center of masses of the two bodies are located on the line
     connecting frame_a and frame_b of the jointUPS component.
     Both bodies have the same mass and the inertia tensor is set to zero,
     i.e., the two bodies are treated as point masses.</li>
<li> With <a href=\"modelica://Modelica.Mechanics.MultiBody.Forces.LineForceWithTwoMasses\">LineForceWithTwoMasses</a>:<br>
     Modelica.Mechanics.MultiBody.Forces.LineForceWithTwoMasses is a line force component
     with the built-in property that two point masses are located
     on the line on which the line force is acting.
     The parameters are selected in such a way that the same
     system as with the jointUPS component is described.</li>
</ul>
<p>
In both cases, a linear 1-dimensional translational damper from the
Modelica.Mechanics.Translational library is used as
line force between the two attachment points. Simulate
this system and plot the differences of the cut forces at both sides
of the line force component (\"rod_f_diff\" and \"body_f_diff\").
Both vectors should be zero
(depending on the chosen relative tolerance of the integration,
the difference is in the order of 1.e-10 ... 1.e-15).
</p>
<p>
Note, that the implementation with the LineForceWithTwoMasses
component is simpler and more convenient.
An animation of this simulation is shown in the figure below.
The system on the left side in the front is the animation with
the LineForceWithTwoMasses component whereas the system on the right
side in the back is the animation with the JointUPS component.
</p>

<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Examples/Elementary/LineForceWithTwoMasses2.png\">

</html>"));
          end LineForceWithTwoMasses;

          model Pendulum "Simple pendulum with one revolute joint and one body"
            extends Modelica.Icons.Example;
            inner Modelica.Mechanics.MultiBody.World world(gravityType = Modelica.Mechanics.MultiBody.Types.GravityTypes.UniformGravity) annotation(
              Placement(transformation(extent = {{-60, 0}, {-40, 20}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Joints.Revolute rev(n = {0, 0, 1}, useAxisFlange = true, phi(fixed = true), w(fixed = true)) annotation(
              Placement(transformation(extent = {{-20, 0}, {0, 20}}, rotation = 0)));
            Modelica.Mechanics.Rotational.Components.Damper damper(d = 0.1) annotation(
              Placement(transformation(extent = {{-20, 40}, {0, 60}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Parts.Body body(m = 1.0, r_CM = {0.5, 0, 0}) annotation(
              Placement(transformation(extent = {{20, 0}, {40, 20}}, rotation = 0)));
          equation
            connect(world.frame_b, rev.frame_a) annotation(
              Line(points = {{-40, 10}, {-20, 10}}, color = {95, 95, 95}, thickness = 0.5));
            connect(damper.flange_b, rev.axis) annotation(
              Line(points = {{0, 50}, {4, 50}, {4, 26}, {-10, 26}, {-10, 20}}, color = {0, 0, 0}));
            connect(rev.support, damper.flange_a) annotation(
              Line(points = {{-16, 20}, {-16, 26}, {-28, 26}, {-28, 50}, {-20, 50}}, color = {0, 0, 0}));
            connect(body.frame_a, rev.frame_b) annotation(
              Line(points = {{20, 10}, {0, 10}}, color = {95, 95, 95}, thickness = 0.5));
            annotation(
              experiment(StopTime = 5),
              Documentation(info = "<html>
<p>
This simple model demonstrates that by just dragging components
default animation is defined that shows the structure of the
assembled system.

<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Examples/Elementary/Pendulum.png\"
ALT=\"model Examples.Elementary.Pendulum\">
</html>"));
          end Pendulum;

          model PendulumWithSpringDamper "Simple spring/damper/mass system"
            extends Modelica.Icons.Example;
            parameter Boolean animation = true "= true, if animation shall be enabled";
            inner Modelica.Mechanics.MultiBody.World world(axisLength = 0.6) annotation(
              Placement(transformation(extent = {{-80, 20}, {-60, 40}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Parts.Body body1(m = 1, animation = animation, I_11 = 1, I_22 = 1, I_33 = 1, r_CM = {0, 0, 0}, cylinderDiameter = 0.05, sphereDiameter = 0.2) annotation(
              Placement(transformation(origin = {72, 30}, extent = {{10, -10}, {-10, 10}}, rotation = 180)));
            Modelica.Mechanics.MultiBody.Parts.FixedTranslation bar1(animation = animation, r = {0.3, 0, 0}) annotation(
              Placement(transformation(extent = {{-46, 20}, {-26, 40}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Forces.Spring spring1(coilWidth = 0.01, numberOfWindings = 5, c = 20, s_unstretched = 0.2) annotation(
              Placement(transformation(extent = {{0, -46}, {20, -26}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Forces.Damper damper1(d = 1, length_a = 0.1, diameter_a = 0.08, animation = false) annotation(
              Placement(transformation(extent = {{0, -20}, {20, 0}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Joints.Revolute revolute(phi(fixed = true), w(fixed = true)) annotation(
              Placement(transformation(extent = {{-12, 20}, {8, 40}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Joints.Prismatic prismatic(boxWidth = 0.04, boxColor = {255, 65, 65}, s(fixed = true, start = 0.5), v(fixed = true)) annotation(
              Placement(transformation(extent = {{20, 20}, {40, 40}}, rotation = 0)));
          equation
            connect(world.frame_b, bar1.frame_a) annotation(
              Line(points = {{-60, 30}, {-46, 30}}, color = {95, 95, 95}, thickness = 0.5));
            connect(revolute.frame_a, bar1.frame_b) annotation(
              Line(points = {{-12, 30}, {-26, 30}}, color = {95, 95, 95}, thickness = 0.5));
            connect(prismatic.frame_a, revolute.frame_b) annotation(
              Line(points = {{20, 30}, {8, 30}}, color = {95, 95, 95}, thickness = 0.5));
            connect(damper1.frame_a, bar1.frame_b) annotation(
              Line(points = {{0, -10}, {-18, -10}, {-18, 30}, {-26, 30}}, color = {95, 95, 95}, thickness = 0.5));
            connect(damper1.frame_b, prismatic.frame_b) annotation(
              Line(points = {{20, -10}, {44, -10}, {44, 30}, {40, 30}}, color = {95, 95, 95}, thickness = 0.5));
            connect(spring1.frame_a, bar1.frame_b) annotation(
              Line(points = {{0, -36}, {-18, -36}, {-18, 30}, {-26, 30}}, color = {95, 95, 95}, thickness = 0.5));
            connect(spring1.frame_b, prismatic.frame_b) annotation(
              Line(points = {{20, -36}, {44, -36}, {44, 30}, {40, 30}}, color = {95, 95, 95}, thickness = 0.5));
            connect(body1.frame_a, prismatic.frame_b) annotation(
              Line(points = {{62, 30}, {40, 30}}, color = {95, 95, 95}, thickness = 0.5));
            annotation(
              experiment(StopTime = 10),
              Documentation(info = "<html>
<p>
A body is attached on a revolute and prismatic joint.
A 3-dim. spring and a 3-dim. damper are connected between the body
and a point fixed in the world frame:
</p>

<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Examples/Elementary/PendulumWithSpringDamper.png\"
ALT=\"model Examples.Elementary.PendulumWithSpringDamper\">
</html>"));
          end PendulumWithSpringDamper;

          model PointGravity "Two point masses in a point gravity field"
            extends Modelica.Icons.Example;
            inner Modelica.Mechanics.MultiBody.World world(mue = 1, gravitySphereDiameter = 0.1, gravityType = Modelica.Mechanics.MultiBody.Types.GravityTypes.PointGravity) annotation(
              Placement(transformation(extent = {{-20, -20}, {0, 0}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Parts.Body body1(m = 1, sphereDiameter = 0.1, I_11 = 0.1, I_22 = 0.1, I_33 = 0.1, r_0(start = {0, 0.6, 0}, each fixed = true), v_0(start = {1, 0, 0}, each fixed = true), angles_fixed = true, w_0_fixed = true, r_CM = {0, 0, 0}) annotation(
              Placement(transformation(extent = {{-20, 20}, {0, 40}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Parts.Body body2(m = 1, sphereDiameter = 0.1, I_11 = 0.1, I_22 = 0.1, I_33 = 0.1, r_0(start = {0.6, 0.6, 0}, each fixed = true), v_0(start = {0.6, 0, 0}, each fixed = true), angles_fixed = true, w_0_fixed = true, r_CM = {0, 0, 0}) annotation(
              Placement(transformation(extent = {{20, 20}, {40, 40}}, rotation = 0)));
            annotation(
              experiment(StopTime = 5),
              Documentation(info = "<HTML>
<p>
This model demonstrates a point gravity field. Two bodies
are placed in the gravity field. The initial positions and velocities of
these bodies are selected such that one body rotates on a circle and
the other body rotates on an ellipse around the center of the
point gravity field.
</p>

<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Examples/Elementary/PointGravity.png\"
ALT=\"model Examples.Elementary.PointGravity\">
</HTML>"));
          end PointGravity;

          model PointGravityWithPointMasses "Two point masses in a point gravity field (rotation of bodies is neglected)"
            extends Modelica.Icons.Example;
            inner Modelica.Mechanics.MultiBody.World world(mue = 1, gravitySphereDiameter = 0.1, gravityType = Modelica.Mechanics.MultiBody.Types.GravityTypes.PointGravity) annotation(
              Placement(transformation(extent = {{-80, -20}, {-60, 0}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Parts.PointMass body1(m = 1, sphereDiameter = 0.1, r_0(start = {0, 0.6, 0}, each fixed = true), v_0(start = {1, 0, 0}, each fixed = true)) annotation(
              Placement(transformation(extent = {{-20, 20}, {0, 40}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Parts.PointMass body2(m = 1, sphereDiameter = 0.1, r_0(start = {0.6, 0.6, 0}, each fixed = true), v_0(start = {0.6, 0, 0}, each fixed = true)) annotation(
              Placement(transformation(extent = {{20, 20}, {40, 40}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Parts.PointMass body3(m = 1, sphereDiameter = 0.1, r_0(start = {0, 0.8, 0}, each fixed = true), v_0(start = {0.6, 0, 0}, each fixed = true)) annotation(
              Placement(transformation(extent = {{-20, 60}, {0, 80}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Parts.PointMass body4(m = 1, sphereDiameter = 0.1, r_0(start = {0.3, 0.8, 0}, each fixed = true), v_0(start = {0.6, 0, 0}, each fixed = true)) annotation(
              Placement(transformation(extent = {{20, 60}, {40, 80}}, rotation = 0)));
            Forces.Spring spring(showMass = false, c = 10, fixedRotationAtFrame_b = true, fixedRotationAtFrame_a = true) annotation(
              Placement(transformation(extent = {{0, 60}, {20, 80}}, rotation = 0)));
          equation
            connect(spring.frame_a, body3.frame_a) annotation(
              Line(points = {{0, 70}, {-10, 70}}, color = {95, 95, 95}, thickness = 0.5));
            connect(spring.frame_b, body4.frame_a) annotation(
              Line(points = {{20, 70}, {30, 70}}, color = {95, 95, 95}, thickness = 0.5));
            annotation(
              experiment(StopTime = 2),
              Documentation(info = "<HTML>
<p>
This model demonstrates the usage of model Parts.PointMass in a
point gravity field. The PointMass model has the feature that
that rotation is not taken into account and can therefore also not be
calculated. This example demonstrates two cases where this does not matter:
If a PointMass is not connected (body1, body2), the orientation object in
these point masses is set to a unit rotation.
If a PointMass is connected by a line force element, such as
the used Forces.LineForceWithMass component, then the orientation object
is set to a unit rotation within the line force element.
These are the two cases where the rotation is automatically set to
a default value, when the physical system does not provide the equations.
</p>

<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Examples/Elementary/PointGravityWithPointMasses.png\">
</HTML>"));
          end PointGravityWithPointMasses;

          model PointGravityWithPointMasses2 "Rigidly connected point masses in a point gravity field"
            extends Modelica.Icons.Example;
            model PointMass = Modelica.Mechanics.MultiBody.Parts.PointMass(m = 1, sphereColor = {255, 0, 0}) "Point mass used at all places of this example";
            PointMass pointMass1(r_0(start = {3, 0, 0}, each fixed = true), v_0(start = {0, 0, -1}, each fixed = true)) annotation(
              Placement(transformation(extent = {{60, -10}, {80, 10}}, rotation = 0)));
            PointMass pointMass2 annotation(
              Placement(transformation(extent = {{-60, -10}, {-40, 10}}, rotation = 0)));
            PointMass pointMass3(r_0(start = {2, 1, 0}, each fixed = true), v_0(start = {0, 0, -1}, each fixed = true)) annotation(
              Placement(transformation(extent = {{60, 20}, {80, 40}}, rotation = 0)));
            PointMass pointMass4 annotation(
              Placement(transformation(extent = {{-50, -40}, {-30, -20}}, rotation = 0)));
            PointMass pointMass5 annotation(
              Placement(transformation(extent = {{0, 80}, {20, 100}}, rotation = 0)));
            PointMass pointMass6 annotation(
              Placement(transformation(extent = {{0, -100}, {20, -80}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Parts.FixedTranslation fixedTranslation(r = {1, 0, 0}) annotation(
              Placement(transformation(extent = {{20, -10}, {40, 10}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Parts.FixedTranslation fixedTranslation1(r = {-1, 0, 0}) annotation(
              Placement(transformation(extent = {{0, -10}, {-20, 10}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Parts.FixedTranslation fixedTranslation2(r = {0, 1, 0}) annotation(
              Placement(transformation(extent = {{20, 20}, {40, 40}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Parts.FixedTranslation fixedTranslation3(r = {0, -1, 0}) annotation(
              Placement(transformation(extent = {{0, -40}, {-20, -20}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Parts.FixedTranslation fixedTranslation4(r = {0, 0, 1}) annotation(
              Placement(transformation(origin = {10, 60}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
            Modelica.Mechanics.MultiBody.Parts.FixedTranslation fixedTranslation5(r = {0, 0, -1}) annotation(
              Placement(transformation(origin = {10, -60}, extent = {{10, -10}, {-10, 10}}, rotation = 90)));
            inner World world(gravitySphereDiameter = 0.1, gravityType = Modelica.Mechanics.MultiBody.Types.GravityTypes.PointGravity, mue = 5) annotation(
              Placement(transformation(extent = {{-80, 60}, {-60, 80}}, rotation = 0)));
            Joints.FreeMotion freeMotion annotation(
              Placement(transformation(extent = {{-40, 60}, {-20, 80}}, rotation = 0)));

            model SystemWithStandardBodies "For comparison purposes, an equivalent model with Bodies instead of PointMasses"
              model PointMass = Modelica.Mechanics.MultiBody.Parts.Body(m = 1, I_11 = 0, I_22 = 0, I_33 = 0) "Body used all places of the comparison model with zero inertia tensor";
              PointMass pointMass1(r_0(start = {3, 0, 0}, each fixed = true), v_0(start = {0, 0, -1}, each fixed = true), angles_fixed = true, w_0_fixed = true, r_CM = {0, 0, 0}) annotation(
                Placement(transformation(extent = {{40, -20}, {60, 0}}, rotation = 0)));
              PointMass pointMass2(r_CM = {0, 0, 0}) annotation(
                Placement(transformation(extent = {{-60, -20}, {-80, 0}}, rotation = 0)));
              PointMass pointMass3(r_CM = {0, 0, 0}) annotation(
                Placement(transformation(extent = {{40, 10}, {60, 30}}, rotation = 0)));
              PointMass pointMass4(r_CM = {0, 0, 0}) annotation(
                Placement(transformation(extent = {{-50, -50}, {-70, -30}}, rotation = 0)));
              PointMass pointMass5(r_CM = {0, 0, 0}) annotation(
                Placement(transformation(extent = {{0, 60}, {20, 80}}, rotation = 0)));
              PointMass pointMass6(r_CM = {0, 0, 0}) annotation(
                Placement(transformation(extent = {{2, -102}, {22, -82}}, rotation = 0)));
              Modelica.Mechanics.MultiBody.Parts.FixedTranslation fixedTranslation(r = {1, 0, 0}) annotation(
                Placement(transformation(extent = {{0, -20}, {20, 0}}, rotation = 0)));
              Modelica.Mechanics.MultiBody.Parts.FixedTranslation fixedTranslation1(r = {-1, 0, 0}) annotation(
                Placement(transformation(extent = {{-20, -20}, {-40, 0}}, rotation = 0)));
              Modelica.Mechanics.MultiBody.Parts.FixedTranslation fixedTranslation2(r = {0, 1, 0}) annotation(
                Placement(transformation(extent = {{0, 10}, {20, 30}}, rotation = 0)));
              Modelica.Mechanics.MultiBody.Parts.FixedTranslation fixedTranslation3(r = {0, -1, 0}) annotation(
                Placement(transformation(extent = {{-20, -50}, {-40, -30}}, rotation = 0)));
              Modelica.Mechanics.MultiBody.Parts.FixedTranslation fixedTranslation4(r = {0, 0, 1}) annotation(
                Placement(transformation(origin = {-10, 50}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
              Modelica.Mechanics.MultiBody.Parts.FixedTranslation fixedTranslation5(r = {0, 0, -1}) annotation(
                Placement(transformation(origin = {-10, -70}, extent = {{10, -10}, {-10, 10}}, rotation = 90)));
            equation
              connect(fixedTranslation1.frame_a, fixedTranslation.frame_a) annotation(
                Line(points = {{-20, -10}, {0, -10}}, color = {95, 95, 95}, thickness = 0.5));
              connect(fixedTranslation1.frame_a, fixedTranslation2.frame_a) annotation(
                Line(points = {{-20, -10}, {-10, -10}, {-10, 20}, {0, 20}}, color = {95, 95, 95}, thickness = 0.5));
              connect(fixedTranslation3.frame_a, fixedTranslation.frame_a) annotation(
                Line(points = {{-20, -40}, {-10, -40}, {-10, -10}, {0, -10}}, color = {95, 95, 95}, thickness = 0.5));
              connect(fixedTranslation1.frame_a, fixedTranslation4.frame_a) annotation(
                Line(points = {{-20, -10}, {-10, -10}, {-10, 40}}, color = {95, 95, 95}, thickness = 0.5));
              connect(fixedTranslation5.frame_a, fixedTranslation.frame_a) annotation(
                Line(points = {{-10, -60}, {-10, -10}, {0, -10}}, color = {95, 95, 95}, thickness = 0.5));
              connect(fixedTranslation2.frame_b, pointMass3.frame_a) annotation(
                Line(points = {{20, 20}, {40, 20}}, color = {95, 95, 95}, thickness = 0.5));
              connect(fixedTranslation3.frame_b, pointMass4.frame_a) annotation(
                Line(points = {{-40, -40}, {-50, -40}}, color = {95, 95, 95}, thickness = 0.5));
              connect(pointMass5.frame_a, fixedTranslation4.frame_b) annotation(
                Line(points = {{0, 70}, {-10, 70}, {-10, 60}}, color = {95, 95, 95}, thickness = 0.5));
              connect(fixedTranslation5.frame_b, pointMass6.frame_a) annotation(
                Line(points = {{-10, -80}, {-10, -92}, {2, -92}}, color = {95, 95, 95}, thickness = 0.5));
              connect(fixedTranslation.frame_b, pointMass1.frame_a) annotation(
                Line(points = {{20, -10}, {40, -10}}, color = {95, 95, 95}, thickness = 0.5));
              connect(fixedTranslation1.frame_b, pointMass2.frame_a) annotation(
                Line(points = {{-40, -10}, {-60, -10}}, color = {95, 95, 95}, thickness = 0.5));
              annotation(
                Documentation(info = "<html>
<p>
In order to compare the results of the \"PointMass\" example where
6 point masses are rigidly connected together, in this comparison model,
an equivalent system is setup, with the only difference that the
point masses are replaced by Bodies with zero inertia.
</p>
</html>"));
            end SystemWithStandardBodies;

            SystemWithStandardBodies referenceSystem annotation(
              Placement(transformation(extent = {{60, -60}, {80, -40}}, rotation = 0)));
          equation
            connect(fixedTranslation1.frame_a, fixedTranslation.frame_a) annotation(
              Line(points = {{0, 0}, {20, 0}}, color = {95, 95, 95}, thickness = 0.5));
            connect(fixedTranslation1.frame_a, fixedTranslation2.frame_a) annotation(
              Line(points = {{0, 0}, {10, 0}, {10, 30}, {20, 30}}, color = {95, 95, 95}, thickness = 0.5));
            connect(fixedTranslation3.frame_a, fixedTranslation.frame_a) annotation(
              Line(points = {{0, -30}, {10, -30}, {10, 0}, {20, 0}}, color = {95, 95, 95}, thickness = 0.5));
            connect(fixedTranslation1.frame_a, fixedTranslation4.frame_a) annotation(
              Line(points = {{0, 0}, {10, 0}, {10, 50}}, color = {95, 95, 95}, thickness = 0.5));
            connect(fixedTranslation5.frame_a, fixedTranslation.frame_a) annotation(
              Line(points = {{10, -50}, {10, 0}, {20, 0}}, color = {95, 95, 95}, thickness = 0.5));
            connect(fixedTranslation2.frame_b, pointMass3.frame_a) annotation(
              Line(points = {{40, 30}, {70, 30}}, color = {95, 95, 95}, thickness = 0.5));
            connect(fixedTranslation3.frame_b, pointMass4.frame_a) annotation(
              Line(points = {{-20, -30}, {-40, -30}}, color = {95, 95, 95}, thickness = 0.5));
            connect(pointMass5.frame_a, fixedTranslation4.frame_b) annotation(
              Line(points = {{10, 90}, {10, 70}}, color = {95, 95, 95}, thickness = 0.5));
            connect(fixedTranslation5.frame_b, pointMass6.frame_a) annotation(
              Line(points = {{10, -70}, {10, -90}}, color = {95, 95, 95}, thickness = 0.5));
            connect(fixedTranslation.frame_b, pointMass1.frame_a) annotation(
              Line(points = {{40, 0}, {70, 0}}, color = {95, 95, 95}, thickness = 0.5));
            connect(fixedTranslation1.frame_b, pointMass2.frame_a) annotation(
              Line(points = {{-20, 0}, {-50, 0}}, color = {95, 95, 95}, thickness = 0.5));
            connect(world.frame_b, freeMotion.frame_a) annotation(
              Line(points = {{-60, 70}, {-40, 70}}, color = {95, 95, 95}, thickness = 0.5));
            connect(freeMotion.frame_b, fixedTranslation1.frame_a) annotation(
              Line(points = {{-20, 70}, {-10, 70}, {-10, 20}, {10, 20}, {10, 0}, {0, 0}}, color = {95, 95, 95}, thickness = 0.5));
            annotation(
              experiment(StopTime = 3.0, Tolerance = 1e-006),
              Documentation(info = "<html>
<p>
This model demonstrates the usage of model Parts.PointMass in a
point gravity field. 6 point masses are connected rigidly together.
Translating such a model results in an error, because point masses do
not define an orientation object. The example demonstrates that in such
a case (when the orientation object is not defined by an object that
is connected to a point mass), a \"MultiBody.Joints.FreeMotion\" joint
has to be used, to define the the degrees of freedom of this structure.
</p>

<p>
In order to demonstrate that this approach is correct, in model
\"referenceSystem\", the same system is again provided, but this time
modeled with a generic body (Parts.Body) where the inertia tensor is
set to zero. In this case, no FreeMotion object is needed because every
body provides its absolute translational and rotational position and
velocity as potential states.
</p>

<p>
The two systems should move exactly in the same way. The system with the PointMasses
object visualizes the point masses in \"red\", whereas the \"referenceSystem\" shows
its bodies in \"blue\".

<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Examples/Elementary/PointGravityWithPointMasses2.png\">
</html>"));
          end PointGravityWithPointMasses2;

          model SpringDamperSystem "Simple spring/damper/mass system"
            extends Modelica.Icons.Example;
            parameter Boolean animation = true "= true, if animation shall be enabled";
            inner Modelica.Mechanics.MultiBody.World world annotation(
              Placement(transformation(extent = {{-80, 20}, {-60, 40}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Parts.Body body1(m = 1, animation = animation, r_CM = {0, -0.2, 0}, cylinderDiameter = 0.05, sphereDiameter = 0.15, I_11 = 0.1, I_22 = 0.1, I_33 = 0.1, r_0(start = {0.3, -0.2, 0}, each fixed = true), v_0(each fixed = true), angles_fixed = true, w_0_fixed = true, w_0_start(each displayUnit = "deg/s") = {0, 0, 0.03490658503988659}) annotation(
              Placement(transformation(origin = {-20, -50}, extent = {{-10, 10}, {10, -10}}, rotation = 270)));
            Modelica.Mechanics.MultiBody.Parts.FixedTranslation bar1(animation = animation, r = {0.3, 0, 0}) annotation(
              Placement(transformation(extent = {{-46, 20}, {-26, 40}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Parts.FixedTranslation bar2(animation = animation, r = {0.6, 0, 0}) annotation(
              Placement(transformation(extent = {{0, 20}, {20, 40}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Parts.Body body2(m = 1, animation = animation, cylinderDiameter = 0.05, sphereDiameter = 0.15, r_CM = {0, 0, 0}) annotation(
              Placement(transformation(origin = {50, -50}, extent = {{-10, 10}, {10, -10}}, rotation = 270)));
            Modelica.Mechanics.MultiBody.Joints.Prismatic p2(useAxisFlange = true, n = {0, -1, 0}, animation = animation, boxWidth = 0.05, stateSelect = StateSelect.always, v(fixed = true), s(fixed = true, start = 0.1)) annotation(
              Placement(transformation(origin = {50, -10}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
            Modelica.Mechanics.MultiBody.Forces.Spring spring2(c = 30, s_unstretched = 0.1, coilWidth = 0.01, width = 0.1) annotation(
              Placement(transformation(origin = {82, -10}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
            Modelica.Mechanics.MultiBody.Forces.Spring spring1(s_unstretched = 0.1, coilWidth = 0.01, c = 30, numberOfWindings = 10, width = 0.1) annotation(
              Placement(transformation(origin = {-6, -10}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
            Modelica.Mechanics.MultiBody.Forces.Damper damper1(d = 2) annotation(
              Placement(transformation(origin = {-30, -10}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          equation
            connect(world.frame_b, bar1.frame_a) annotation(
              Line(points = {{-60, 30}, {-46, 30}}, color = {95, 95, 95}, thickness = 0.5));
            connect(bar1.frame_b, bar2.frame_a) annotation(
              Line(points = {{-26, 30}, {0, 30}}, color = {95, 95, 95}, thickness = 0.5));
            connect(bar2.frame_b, p2.frame_a) annotation(
              Line(points = {{20, 30}, {50, 30}, {50, 0}}, color = {95, 95, 95}, thickness = 0.5));
            connect(p2.frame_b, body2.frame_a) annotation(
              Line(points = {{50, -20}, {50, -40}}, color = {95, 95, 95}, thickness = 0.5));
            connect(bar2.frame_b, spring2.frame_a) annotation(
              Line(points = {{20, 30}, {82, 30}, {82, 0}}, color = {95, 95, 95}, thickness = 0.5));
            connect(body2.frame_a, spring2.frame_b) annotation(
              Line(points = {{50, -40}, {82, -40}, {82, -20}}, color = {95, 95, 95}, thickness = 0.5));
            connect(damper1.frame_a, bar1.frame_b) annotation(
              Line(points = {{-30, 0}, {-30, 10}, {-20, 10}, {-20, 30}, {-26, 30}}, color = {95, 95, 95}, thickness = 0.5));
            connect(spring1.frame_a, bar1.frame_b) annotation(
              Line(points = {{-6, 0}, {-6, 10}, {-20, 10}, {-20, 30}, {-26, 30}}, color = {95, 95, 95}, thickness = 0.5));
            connect(damper1.frame_b, body1.frame_a) annotation(
              Line(points = {{-30, -20}, {-30, -28}, {-20, -28}, {-20, -40}}, color = {95, 95, 95}, thickness = 0.5));
            connect(spring1.frame_b, body1.frame_a) annotation(
              Line(points = {{-6, -20}, {-6, -28}, {-20, -28}, {-20, -40}}, color = {95, 95, 95}, thickness = 0.5));
            annotation(
              experiment(StopTime = 10),
              Documentation(info = "<html>
<p>
This example demonstrates:
</p>
<ul>
<li>The animation of spring and damper components</li>
<li>A body can be freely moving without any connection to a joint.
    In this case body coordinates are used automatically as
    states (whenever joints are present, it is first tried to
    use the generalized coordinates of the joints as states).</li>
<li>If a body is freely moving, the initial position and velocity of the body
    can be defined with the \"Initialization\" menu as shown with the
    body \"body1\" in the left part (click on \"Initialization\").</li>
</ul>

<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Examples/Elementary/SpringDamperSystem.png\"
ALT=\"model Examples.Elementary.SpringDamperSystem\">

</html>"));
          end SpringDamperSystem;

          model SpringMassSystem "Mass attached with a spring to the world frame"
            extends Modelica.Icons.Example;
            parameter Boolean animation = true "= true, if animation shall be enabled";
            inner Modelica.Mechanics.MultiBody.World world annotation(
              Placement(transformation(extent = {{-80, 20}, {-60, 40}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Joints.Prismatic p1(useAxisFlange = true, n = {0, -1, 0}, animation = animation, boxWidth = 0.05, s(fixed = true, start = 0.1), v(fixed = true)) annotation(
              Placement(transformation(origin = {-20, -10}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
            Modelica.Mechanics.Translational.Components.Spring spring1(c = 30, s_rel0 = 0.1) annotation(
              Placement(transformation(origin = {10, -10}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
            Modelica.Mechanics.MultiBody.Parts.Body body1(m = 1, sphereDiameter = 0.2, animation = animation, r_CM = {0, 0, 0}) annotation(
              Placement(transformation(origin = {-20, -50}, extent = {{-10, 10}, {10, -10}}, rotation = 270)));
            Modelica.Mechanics.MultiBody.Parts.FixedTranslation bar1(animation = animation, r = {0.3, 0, 0}) annotation(
              Placement(transformation(extent = {{-46, 20}, {-26, 40}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Parts.FixedTranslation bar2(animation = animation, r = {0.3, 0, 0}) annotation(
              Placement(transformation(extent = {{0, 20}, {20, 40}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Parts.Body body2(m = 1, sphereDiameter = 0.2, animation = animation, r_CM = {0, 0, 0}) annotation(
              Placement(transformation(origin = {50, -50}, extent = {{-10, 10}, {10, -10}}, rotation = 270)));
            Modelica.Mechanics.MultiBody.Joints.Prismatic p2(useAxisFlange = true, n = {0, -1, 0}, animation = animation, boxWidth = 0.05, stateSelect = StateSelect.always, s(fixed = true, start = 0.1), v(fixed = true)) annotation(
              Placement(transformation(origin = {50, -10}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
            Modelica.Mechanics.MultiBody.Forces.Spring spring2(c = 30, s_unstretched = 0.1, width = 0.1) annotation(
              Placement(transformation(origin = {90, -10}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          equation
            connect(body1.frame_a, p1.frame_b) annotation(
              Line(points = {{-20, -40}, {-20, -35}, {-20, -35}, {-20, -30}, {-20, -20}, {-20, -20}}, color = {95, 95, 95}, thickness = 0.5));
            connect(world.frame_b, bar1.frame_a) annotation(
              Line(points = {{-60, 30}, {-46, 30}}, color = {95, 95, 95}, thickness = 0.5));
            connect(bar1.frame_b, p1.frame_a) annotation(
              Line(points = {{-26, 30}, {-20, 30}, {-20, 0}}, color = {95, 95, 95}, thickness = 0.5));
            connect(spring1.flange_b, p1.axis) annotation(
              Line(points = {{10, -20}, {10, -30}, {-8, -30}, {-8, -18}, {-14, -18}}, color = {0, 191, 0}));
            connect(bar1.frame_b, bar2.frame_a) annotation(
              Line(points = {{-26, 30}, {0, 30}}, color = {95, 95, 95}, thickness = 0.5));
            connect(bar2.frame_b, p2.frame_a) annotation(
              Line(points = {{20, 30}, {50, 30}, {50, 0}}, color = {95, 95, 95}, thickness = 0.5));
            connect(p2.frame_b, body2.frame_a) annotation(
              Line(points = {{50, -20}, {50, -40}}, color = {95, 95, 95}, thickness = 0.5));
            connect(bar2.frame_b, spring2.frame_a) annotation(
              Line(points = {{20, 30}, {90, 30}, {90, 0}}, color = {95, 95, 95}, thickness = 0.5));
            connect(body2.frame_a, spring2.frame_b) annotation(
              Line(points = {{50, -40}, {90, -40}, {90, -20}}, color = {95, 95, 95}, thickness = 0.5));
            connect(spring1.flange_a, p1.support) annotation(
              Line(points = {{10, 0}, {-8, 0}, {-8, -6}, {-14, -6}}, color = {0, 127, 0}, smooth = Smooth.None));
            annotation(
              experiment(StopTime = 5),
              Documentation(info = "<html>
<p>
This example shows the two different ways how force laws
can be utilized:
</p>
<ul>
<li>In the left system a body is attached via a prismatic
    joint to the world frame. The prismatic joint has two
    1-dimensional translational flanges (called \"support\" and \"axis\")
    that allows to connect elements from the Modelica.Mechanics.Translational
    library between the support and the axis connector. The effect is
    that the force generated by the 1-dimensional elements acts as driving
    force in the axis of the prismatic joint. In the example a simple
    spring is used.<br>
    The advantage of this approach is that the many elements from the
    Translational library can be easily used here and that this implementation
    is usually more efficient as when using 3-dimensional springs.</li>
<li>In the right system the same model is defined. The difference is
    that a 3-dimensional spring from the Modelica.Mechanics.MultiBody.Forces library is used.
    This has the advantage to get a nice animation of the force component.</li>
</ul>

<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Examples/Elementary/SpringMassSystem.png\"
ALT=\"model Examples.Elementary.SpringMassSystem\">
</html>"));
          end SpringMassSystem;

          model SpringWithMass "Point mass hanging on a spring"
            extends Modelica.Icons.Example;
            inner Modelica.Mechanics.MultiBody.World world(animateGravity = false) annotation(
              Placement(transformation(extent = {{-40, 40}, {-20, 60}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Forces.Spring spring(s_unstretched = 0.2, m = 0.5, c = 40, width = 0.1, massDiameter = 0.07) annotation(
              Placement(transformation(origin = {10, 30}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
            Modelica.Mechanics.MultiBody.Parts.Body body(r_0(start = {0, -0.3, 0}, each fixed = true), v_0(each fixed = true), angles_fixed = true, w_0_fixed = true, r_CM = {0, 0, 0}, m = 1) annotation(
              Placement(transformation(origin = {10, -10}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          equation
            connect(world.frame_b, spring.frame_a) annotation(
              Line(points = {{-20, 50}, {10, 50}, {10, 40}}, color = {95, 95, 95}, thickness = 0.5));
            connect(body.frame_a, spring.frame_b) annotation(
              Line(points = {{10, 0}, {10, 10}, {10, 20}}, color = {95, 95, 95}, thickness = 0.5));
            annotation(
              experiment(StopTime = 5),
              Documentation(info = "<html>
<p>
This example shows that a force component may have a mass.
The 3-dimensional spring as used in this example, has an optional
point mass between the two points where the spring is attached.
In the animation, this point mass is represented by a small,
light blue, sphere.
</p>

<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Examples/Elementary/SpringWithMass.png\"
ALT=\"model Examples.Elementary.SpringWithMass\">
</html>"));
          end SpringWithMass;

          model ThreeSprings "3-dim. springs in series and parallel connection"
            extends Modelica.Icons.Example;
            parameter Boolean animation = true "= true, if animation shall be enabled";
            inner Modelica.Mechanics.MultiBody.World world(animateWorld = animation) annotation(
              Placement(transformation(extent = {{-60, 20}, {-40, 40}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Parts.Body body1(animation = animation, r_CM = {0, -0.2, 0}, m = 0.8, I_11 = 0.1, I_22 = 0.1, I_33 = 0.1, sphereDiameter = 0.2, r_0(start = {0.5, -0.3, 0}, each fixed = true), v_0(each fixed = true), angles_fixed = true, w_0_fixed = true) annotation(
              Placement(transformation(origin = {30, -70}, extent = {{-10, 10}, {10, -10}}, rotation = 270)));
            Modelica.Mechanics.MultiBody.Parts.FixedTranslation bar1(animation = animation, r = {0.3, 0, 0}) annotation(
              Placement(transformation(extent = {{-20, 20}, {0, 40}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Forces.Spring spring1(lineForce(r_rel_0(start = {-0.2, -0.2, 0.2})), s_unstretched = 0.1, width = 0.1, coilWidth = 0.005, numberOfWindings = 5, c = 20, animation = animation) annotation(
              Placement(transformation(origin = {30, 10}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
            Modelica.Mechanics.MultiBody.Parts.FixedTranslation bar2(animation = animation, r = {0, 0, 0.3}) annotation(
              Placement(transformation(origin = {-30, -10}, extent = {{10, -10}, {-10, 10}}, rotation = 90)));
            Modelica.Mechanics.MultiBody.Forces.Spring spring2(s_unstretched = 0.1, width = 0.1, coilWidth = 0.005, numberOfWindings = 5, c = 40, animation = animation) annotation(
              Placement(transformation(origin = {30, -38}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
            Modelica.Mechanics.MultiBody.Forces.Spring spring3(s_unstretched = 0.1, width = 0.1, coilWidth = 0.005, numberOfWindings = 5, c = 20, animation = animation, fixedRotationAtFrame_b = true) annotation(
              Placement(transformation(extent = {{-20, -42}, {0, -22}}, rotation = 0)));
          equation
            connect(world.frame_b, bar1.frame_a) annotation(
              Line(points = {{-40, 30}, {-20, 30}}, color = {95, 95, 95}, thickness = 0.5));
            connect(world.frame_b, bar2.frame_a) annotation(
              Line(points = {{-40, 30}, {-30, 30}, {-30, 0}}, color = {95, 95, 95}, thickness = 0.5));
            connect(bar1.frame_b, spring1.frame_a) annotation(
              Line(points = {{0, 30}, {30, 30}, {30, 20}}, color = {95, 95, 95}, thickness = 0.5));
            connect(bar2.frame_b, spring3.frame_a) annotation(
              Line(points = {{-30, -20}, {-30, -32}, {-20, -32}}, color = {95, 95, 95}, thickness = 0.5));
            connect(spring2.frame_b, body1.frame_a) annotation(
              Line(points = {{30, -48}, {30, -60}, {30, -60}}, color = {0, 0, 0}, thickness = 0.5));
            connect(spring3.frame_b, spring1.frame_b) annotation(
              Line(points = {{0, -32}, {30, -14}, {30, 0}}, color = {95, 95, 95}, thickness = 0.5));
            connect(spring2.frame_a, spring1.frame_b) annotation(
              Line(points = {{30, -28}, {30, -21}, {30, -21}, {30, -14}, {30, 0}, {30, 0}}, color = {95, 95, 95}, thickness = 0.5));
            annotation(
              experiment(StopTime = 10),
              Documentation(info = "<html>
<p>
This example demonstrates that <b>3-dimensional line force</b> elements
(here: Modelica.Mechanics.MultiBody.Forces.Spring elements) can be connected together
in <b>series</b> without having a body with mass at the
connection point (as usually required by multi-body programs).
This is advantageous since stiff systems can be avoided, say, due to
a stiff spring and a small mass at the connection point.
</p>

<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Examples/Elementary/ThreeSprings.png\"
ALT=\"model Examples.Elementary.ThreeSprings\">

<p>
For a more thorough explanation, see
<a href=\"modelica://Modelica.Mechanics.MultiBody.UsersGuide.Tutorial.ConnectionOfLineForces\">MultiBody.UsersGuide.Tutorial.ConnectionOfLineForces</a>.
</p>
</html>"));
          end ThreeSprings;

          model RollingWheel "Single wheel rolling on ground starting from an initial speed"
            extends Modelica.Icons.Example;
            Modelica.Mechanics.MultiBody.Parts.RollingWheel wheel1(wheelRadius = 0.3, wheelMass = 2, wheel_I_axis = 0.06, wheel_I_long = 0.12, hollowFraction = 0.6, x(start = 0.2), y(start = 0.2), der_angles(start = {0, 5, 1})) annotation(
              Placement(transformation(extent = {{-20, 0}, {0, 20}})));
            inner Modelica.Mechanics.MultiBody.World world(label2 = "z", n = {0, 0, -1}) annotation(
              Placement(transformation(extent = {{-60, 0}, {-40, 20}})));
            Modelica.Mechanics.MultiBody.Visualizers.Ground ground(length = 4) annotation(
              Placement(transformation(extent = {{-20, -40}, {0, -20}})));
            annotation(
              experiment(StopTime = 4),
              Documentation(info = "<html>

</html>"));
          end RollingWheel;

          model RollingWheelSetDriving "Rolling wheel set that is driven by torques driving the wheels"
            extends Modelica.Icons.Example;
            Modelica.Mechanics.MultiBody.Visualizers.Ground ground(length = 3, groundColor = {0, 255, 0}) annotation(
              Placement(transformation(extent = {{-20, -60}, {0, -40}})));
            inner Modelica.Mechanics.MultiBody.World world(label2 = "z", n = {0, 0, -1}) annotation(
              Placement(transformation(extent = {{-80, -60}, {-60, -40}})));
            Modelica.Mechanics.MultiBody.Parts.RollingWheelSet wheelSet(wheelRadius = 0.1, wheelMass = 0.5, wheel_I_axis = 0.01, wheel_I_long = 0.02, wheelDistance = 0.5, x(start = 0.1, fixed = true), y(start = 0.1, fixed = true), phi(fixed = true), theta1(fixed = true), theta2(fixed = true), der_theta1(fixed = true), der_theta2(fixed = true)) annotation(
              Placement(transformation(extent = {{-20, -20}, {0, 0}})));
            Modelica.Mechanics.MultiBody.Parts.Body body(m = 0.01, r_CM = {0, 0, 0}, animation = false) annotation(
              Placement(transformation(extent = {{40, 56}, {60, 76}})));
            Modelica.Mechanics.MultiBody.Parts.FixedTranslation fixedTranslation(r = {0.2, 0, 0}, animation = true, width = 0.04) annotation(
              Placement(transformation(extent = {{0, 56}, {20, 76}})));
            Modelica.Blocks.Sources.Sine sine1(freqHz = 1, amplitude = 2) annotation(
              Placement(transformation(extent = {{-80, 20}, {-60, 40}})));
            Modelica.Blocks.Sources.Sine sine2(freqHz = 1, amplitude = 2, phase = 1.5707963267949) annotation(
              Placement(transformation(extent = {{60, 20}, {40, 40}})));
            Modelica.Mechanics.Rotational.Sources.Torque2 torque1 annotation(
              Placement(transformation(extent = {{-40, 4}, {-20, 24}})));
            Modelica.Mechanics.Rotational.Sources.Torque2 torque2 annotation(
              Placement(transformation(extent = {{24, 4}, {4, 24}})));
            Modelica.Mechanics.MultiBody.Visualizers.FixedShape shape(final lengthDirection = {0, 1, 0}, final widthDirection = {1, 0, 0}, final shapeType = "pipe", final r_shape = {0, -wheelSet.wheelWidth, 0}, final length = 2 * wheelSet.wheelWidth, final width = 2 * wheelSet.wheelRadius, final height = 2 * wheelSet.wheelRadius, final color = {0, 128, 255}, final extra = 0.8) annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0, origin = {50, 90})));
          equation
            connect(fixedTranslation.frame_a, wheelSet.frameMiddle) annotation(
              Line(points = {{0, 66}, {-8, 66}, {-8, -6}, {-10, -6}, {-10, -10}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            connect(fixedTranslation.frame_b, body.frame_a) annotation(
              Line(points = {{20, 66}, {40, 66}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            connect(wheelSet.axis1, torque1.flange_a) annotation(
              Line(points = {{-20, 0}, {-40, 0}, {-40, 14}}, color = {0, 0, 0}, smooth = Smooth.None));
            connect(torque1.flange_b, wheelSet.support) annotation(
              Line(points = {{-20, 14}, {-10, 14}, {-10, -1.8}}, color = {0, 0, 0}, smooth = Smooth.None));
            connect(wheelSet.axis2, torque2.flange_a) annotation(
              Line(points = {{0, 0}, {24, 0}, {24, 14}}, color = {0, 0, 0}, smooth = Smooth.None));
            connect(wheelSet.support, torque2.flange_b) annotation(
              Line(points = {{-10, -1.8}, {-10, 14}, {4, 14}}, color = {0, 0, 0}, smooth = Smooth.None));
            connect(sine1.y, torque1.tau) annotation(
              Line(points = {{-59, 30}, {-30, 30}, {-30, 18}}, color = {0, 0, 127}, smooth = Smooth.None));
            connect(sine2.y, torque2.tau) annotation(
              Line(points = {{39, 30}, {14, 30}, {14, 18}}, color = {0, 0, 127}, smooth = Smooth.None));
            connect(shape.frame_a, fixedTranslation.frame_b) annotation(
              Line(points = {{40, 90}, {28, 90}, {28, 66}, {20, 66}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            annotation(
              experiment(StopTime = 3),
              Documentation(info = "<html>

</html>"));
          end RollingWheelSetDriving;

          model RollingWheelSetPulling "Rolling wheel set that is pulled by a force"
            extends Modelica.Icons.Example;
            Modelica.Mechanics.MultiBody.Forces.WorldForce force(animation = false) annotation(
              Placement(transformation(extent = {{-20, 60}, {0, 80}})));
            Modelica.Mechanics.MultiBody.Visualizers.Ground ground(length = 3) annotation(
              Placement(transformation(extent = {{-20, -40}, {0, -20}})));
            inner Modelica.Mechanics.MultiBody.World world(label2 = "z", n = {0, 0, -1}) annotation(
              Placement(transformation(extent = {{-60, 0}, {-40, 20}})));
            Modelica.Mechanics.MultiBody.Parts.RollingWheelSet wheelSet(wheelRadius = 0.1, wheelMass = 0.5, wheel_I_axis = 0.01, wheel_I_long = 0.02, wheelDistance = 0.5, x(start = 0.1, fixed = true), y(start = 0.1, fixed = true), phi(fixed = true), theta1(fixed = true), theta2(fixed = true), der_theta1(fixed = true), der_theta2(fixed = true)) annotation(
              Placement(transformation(extent = {{-20, 0}, {0, 20}})));
            Modelica.Mechanics.MultiBody.Parts.Body body(m = 0.01, r_CM = {0, 0, 0}, animation = false) annotation(
              Placement(transformation(extent = {{42, 20}, {62, 40}})));
            Modelica.Blocks.Sources.CombiTimeTable combiTimeTable(table = [0, 1, 0, 0; 1, 1, 0, 0; 2, 0, 2, 0; 3, 0, 2, 0]) annotation(
              Placement(transformation(extent = {{-80, 70}, {-60, 90}})));
            Modelica.Mechanics.MultiBody.Parts.FixedTranslation fixedTranslation(r = {0.2, 0, 0}, animation = true, width = 0.04) annotation(
              Placement(transformation(extent = {{0, 20}, {20, 40}})));
            Modelica.Mechanics.MultiBody.Visualizers.FixedShape shape(final lengthDirection = {0, 1, 0}, final widthDirection = {1, 0, 0}, final shapeType = "pipe", final r_shape = {0, -wheelSet.wheelWidth, 0}, final length = 2 * wheelSet.wheelWidth, final width = 2 * wheelSet.wheelRadius, final height = 2 * wheelSet.wheelRadius, final color = {0, 128, 255}, final extra = 0.8) annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0, origin = {52, 54})));
          equation
            connect(combiTimeTable.y, force.force) annotation(
              Line(points = {{-59, 80}, {-42, 80}, {-42, 70}, {-22, 70}}, color = {0, 0, 127}, smooth = Smooth.None));
            connect(fixedTranslation.frame_a, wheelSet.frameMiddle) annotation(
              Line(points = {{0, 30}, {-6, 30}, {-6, 14}, {-10, 14}, {-10, 10}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            connect(fixedTranslation.frame_b, body.frame_a) annotation(
              Line(points = {{20, 30}, {42, 30}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            connect(force.frame_b, fixedTranslation.frame_b) annotation(
              Line(points = {{0, 70}, {34, 70}, {34, 30}, {20, 30}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            connect(shape.frame_a, fixedTranslation.frame_b) annotation(
              Line(points = {{42, 54}, {34, 54}, {34, 30}, {20, 30}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            annotation(
              experiment(StopTime = 3),
              Documentation(info = "<html>

</html>"));
          end RollingWheelSetPulling;

          model HeatLosses "Demonstrate the modeling of heat losses"
            extends Modelica.Icons.Example;
            inner World world annotation(
              Placement(transformation(extent = {{-100, 40}, {-80, 60}}, rotation = 0)));
            Parts.Body body1(m = 1, r_CM = {0, -0.2, 0}, cylinderDiameter = 0.05, sphereDiameter = 0.15, I_11 = 0.1, I_22 = 0.1, I_33 = 0.1, r_0(start = {0.3, -0.2, 0}, each fixed = true), v_0(each fixed = true), angles_fixed = true, w_0_fixed = true, w_0_start(each displayUnit = "deg/s") = {0, 0, 0.034906585039887}) annotation(
              Placement(transformation(origin = {-40, -30}, extent = {{-10, 10}, {10, -10}}, rotation = 270)));
            Parts.FixedTranslation bar1(r = {0.3, 0, 0}) annotation(
              Placement(transformation(extent = {{-66, 40}, {-46, 60}}, rotation = 0)));
            Parts.FixedTranslation bar2(r = {0.3, 0, 0}) annotation(
              Placement(transformation(extent = {{-20, 40}, {0, 60}}, rotation = 0)));
            Forces.Spring spring1(s_unstretched = 0.1, coilWidth = 0.01, c = 30, numberOfWindings = 10, width = 0.1) annotation(
              Placement(transformation(origin = {-26, 10}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
            Forces.Damper damper1(d = 2, useHeatPort = true) annotation(
              Placement(transformation(origin = {-50, 10}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
            Forces.SpringDamperParallel springDamper(d = 2, c = 30, s_unstretched = 0.1, width = 0.1, coilWidth = 0.01, numberOfWindings = 10, useHeatPort = true) annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = -90, origin = {26, 10})));
            Parts.Body body2(m = 1, r_CM = {0, -0.2, 0}, cylinderDiameter = 0.05, sphereDiameter = 0.15, I_11 = 0.1, I_22 = 0.1, I_33 = 0.1, v_0(each fixed = true), angles_fixed = true, w_0_fixed = true, w_0_start(each displayUnit = "deg/s") = {0, 0, 0.034906585039887}, r_0(start = {0.6, -0.2, 0}, each fixed = true)) annotation(
              Placement(transformation(origin = {26, -30}, extent = {{-10, 10}, {10, -10}}, rotation = 270)));
            Parts.FixedTranslation bar3(r = {0.3, 0, 0}) annotation(
              Placement(transformation(extent = {{34, 40}, {54, 60}}, rotation = 0)));
            Forces.SpringDamperSeries springDamperSeries(d = 2, c = 30, s_unstretched = 0.1, useHeatPort = true) annotation(
              Placement(transformation(extent = {{-10, 10}, {10, -10}}, rotation = -90, origin = {80, 10})));
            Parts.Body body3(m = 1, r_CM = {0, -0.2, 0}, cylinderDiameter = 0.05, sphereDiameter = 0.15, I_11 = 0.1, I_22 = 0.1, I_33 = 0.1, v_0(each fixed = true), angles_fixed = true, w_0_fixed = true, w_0_start(each displayUnit = "deg/s") = {0, 0, 0.034906585039887}, r_0(start = {0.9, -0.2, 0}, each fixed = true)) annotation(
              Placement(transformation(origin = {80, -30}, extent = {{-10, 10}, {10, -10}}, rotation = 270)));
            Forces.Spring spring(s_unstretched = 0.2, width = 0.05, c = 30) annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = -90, origin = {60, 10})));
            Blocks.Sources.Constant const(k = 20) annotation(
              Placement(transformation(extent = {{-8, -100}, {12, -80}})));
            Thermal.HeatTransfer.Components.Convection convection annotation(
              Placement(transformation(extent = {{26, -52}, {46, -72}})));
            Thermal.HeatTransfer.Celsius.FixedTemperature TAmbient(T = 25) "Ambient temperature" annotation(
              Placement(transformation(extent = {{80, -72}, {60, -52}})));
          equation
            connect(world.frame_b, bar1.frame_a) annotation(
              Line(points = {{-80, 50}, {-66, 50}}, color = {95, 95, 95}, thickness = 0.5));
            connect(bar1.frame_b, bar2.frame_a) annotation(
              Line(points = {{-46, 50}, {-20, 50}}, color = {95, 95, 95}, thickness = 0.5));
            connect(damper1.frame_a, bar1.frame_b) annotation(
              Line(points = {{-50, 20}, {-50, 30}, {-40, 30}, {-40, 50}, {-46, 50}}, color = {95, 95, 95}, thickness = 0.5));
            connect(spring1.frame_a, bar1.frame_b) annotation(
              Line(points = {{-26, 20}, {-26, 30}, {-40, 30}, {-40, 50}, {-46, 50}}, color = {95, 95, 95}, thickness = 0.5));
            connect(damper1.frame_b, body1.frame_a) annotation(
              Line(points = {{-50, 0}, {-50, -8}, {-40, -8}, {-40, -20}}, color = {95, 95, 95}, thickness = 0.5));
            connect(spring1.frame_b, body1.frame_a) annotation(
              Line(points = {{-26, 0}, {-26, -8}, {-40, -8}, {-40, -20}}, color = {95, 95, 95}, thickness = 0.5));
            connect(bar2.frame_b, springDamper.frame_a) annotation(
              Line(points = {{0, 50}, {26, 50}, {26, 20}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            connect(springDamper.frame_b, body2.frame_a) annotation(
              Line(points = {{26, 0}, {26, -5}, {26, -5}, {26, -10}, {26, -20}, {26, -20}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            connect(bar3.frame_b, springDamperSeries.frame_a) annotation(
              Line(points = {{54, 50}, {80, 50}, {80, 20}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            connect(springDamperSeries.frame_b, body3.frame_a) annotation(
              Line(points = {{80, 0}, {80, -20}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            connect(bar3.frame_a, bar2.frame_b) annotation(
              Line(points = {{34, 50}, {0, 50}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            connect(bar3.frame_b, spring.frame_a) annotation(
              Line(points = {{54, 50}, {60, 50}, {60, 20}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            connect(spring.frame_b, body3.frame_a) annotation(
              Line(points = {{60, 0}, {60, -12}, {80, -12}, {80, -20}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            connect(const.y, convection.Gc) annotation(
              Line(points = {{13, -90}, {36, -90}, {36, -72}}, color = {0, 0, 127}, smooth = Smooth.None));
            connect(TAmbient.port, convection.fluid) annotation(
              Line(points = {{60, -62}, {46, -62}}, color = {191, 0, 0}, smooth = Smooth.None));
            connect(damper1.heatPort, convection.solid) annotation(
              Line(points = {{-60, 20}, {-60, -62}, {26, -62}}, color = {191, 0, 0}, smooth = Smooth.None));
            connect(springDamper.heatPort, convection.solid) annotation(
              Line(points = {{16, 20}, {4, 20}, {4, -62}, {26, -62}}, color = {191, 0, 0}, smooth = Smooth.None));
            connect(springDamperSeries.heatPort, convection.solid) annotation(
              Line(points = {{90, 20}, {96, 20}, {96, -46}, {4, -46}, {4, -62}, {26, -62}}, color = {191, 0, 0}, smooth = Smooth.None));
            annotation(
              experiment(StopTime = 3),
              Documentation(info = "<html>
<p>
This model demonstrates how to model the dissipated power of a multi-body
force element by enabling the heatPort of all components and connecting these heatPorts via
a convection element to the environment. The total heat flow generated by the
elements of this multi-body system and transported to the environment
is present in variable convection.fluid.
</p>
</html>"));
          end HeatLosses;

          model UserDefinedGravityField "Demonstrate the modeling of a user-defined gravity field"
            extends Modelica.Icons.Example;
            parameter Modelica.SIunits.Conversions.NonSIunits.Angle_deg geodeticLatitude = 0 "Geodetic latitude";
            parameter Modelica.SIunits.Position height = 20 "Height of pendulum attachment point over WGS84 earth ellipsoid";
            Modelica.SIunits.Acceleration gravity[3] = body.g_0 "Gravity acceleration at center of mass of body";
            inner Modelica.Mechanics.MultiBody.World world(gravityType = Modelica.Mechanics.MultiBody.Types.GravityTypes.NoGravity, redeclare function gravityAcceleration = Modelica.Mechanics.MultiBody.Examples.Elementary.Utilities.theoreticalNormalGravityWGS84(phi = geodeticLatitude), axisLength = 10, nominalLength = 10) annotation(
              Placement(transformation(extent = {{-80, -20}, {-60, 0}}, rotation = 0)));
            Joints.Revolute rev(n = {0, 0, 1}, useAxisFlange = true, phi(fixed = true), w(fixed = true)) annotation(
              Placement(transformation(extent = {{-14, 20}, {6, 40}}, rotation = 0)));
            Rotational.Components.Damper damper(d = 0.1) annotation(
              Placement(transformation(extent = {{-14, 60}, {6, 80}}, rotation = 0)));
            Parts.Body body(r_CM = {10, 0, 0}, m = 1000.0, sphereDiameter = 1) annotation(
              Placement(transformation(extent = {{26, 20}, {46, 40}}, rotation = 0)));
            Parts.FixedTranslation fixedTranslation(r = {0, height, 0}, width = 0.3) annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 90, origin = {-40, 8})));
          equation
            connect(damper.flange_b, rev.axis) annotation(
              Line(points = {{6, 70}, {10, 70}, {10, 46}, {-4, 46}, {-4, 40}}, color = {0, 0, 0}));
            connect(rev.support, damper.flange_a) annotation(
              Line(points = {{-10, 40}, {-10, 46}, {-22, 46}, {-22, 70}, {-14, 70}}, color = {0, 0, 0}));
            connect(body.frame_a, rev.frame_b) annotation(
              Line(points = {{26, 30}, {6, 30}}, color = {95, 95, 95}, thickness = 0.5));
            connect(world.frame_b, fixedTranslation.frame_a) annotation(
              Line(points = {{-60, -10}, {-40, -10}, {-40, -2}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            connect(fixedTranslation.frame_b, rev.frame_a) annotation(
              Line(points = {{-40, 18}, {-40, 30}, {-14, 30}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            annotation(
              experiment(StopTime = 10, Tolerance = 1e-008),
              Documentation(info = "<html>
<p>
This example demonstrates a user defined gravity field.
Function \"world.gravityAcceleration\" is redeclared to function
<a href=\"modelica://Modelica.Mechanics.MultiBody.Examples.Elementary.Utilities.theoreticalNormalGravityWGS84\">theoreticalNormalGravityWGS84</a>
that computes the theoretical gravity of the
<a href=\"http://earth-info.nga.mil/GandG/publications/tr8350.2/wgs84fin.pdf\">WGS84 ellipsoid earth model</a> at and close to
the earth ellipsoid surface. In the gravity field, a large, single pendulum is present. Via parameter \"geodeticLatitude\", the geodetic latitude on the earth can be defined, where the pendulum is present. The world frame is located at the WGS84 earth ellipsoid at this latitude. The result variable
\"gravity\" is the gravity vector at the center of mass of the pendulum mass.
Since the height of this mass is changing, the value of the gravity is also changing
(the difference is in the order of 0.00001).
</p>

<p>
The result of the simulation is slightly different at the equator (geodeticLatitude=0)
and at the poles (geodeticLatitude=90). For example, after 10 s of simulation time
the rotation angle of the pendulum, rev.phi, has the following values:
</p>

<table border=1 cellspacing=0 cellpadding=2>
<tr><td><b><i>latitude [deg]</i></b></td>
    <td><b><i>rev.phi [rad]</i></b></td></tr>
<tr><td> = 0</td>
    <td>= -2.39 rad</td></tr>

<tr><td>= 90</td>
    <td>= -2.42 rad</td></tr>
</table>
</html>"));
          end UserDefinedGravityField;

          model Surfaces "Demonstrate the visualization of a sine surface, as well as a torus and a wheel constructed from a surface"
            extends Modelica.Icons.Example;
            parameter Real x_min = -1 "Minimum value of x";
            parameter Real x_max = +1 "Maximum value of x";
            parameter Real y_min = -1 "Minimum value of y";
            parameter Real y_max = +1 "Maximum value of y";
            parameter Real z_min = 0 "Minimum value of z";
            parameter Real z_max = 1 "Maximum value of z";
            Real wz = time;
            Modelica.Mechanics.MultiBody.Visualizers.Advanced.Surface surface(redeclare function surfaceCharacteristic = Modelica.Mechanics.MultiBody.Examples.Elementary.Utilities.sineSurface(x_min = x_min, x_max = x_max, y_min = y_min, y_max = y_max, z_min = z_min, z_max = z_max, wz = wz), multiColoredSurface = false, nu = 50, nv = 50) annotation(
              Placement(transformation(extent = {{-72, -52}, {-52, -32}})));
            inner World world(axisLength = 1, n = {0, 0, -1}) annotation(
              Placement(transformation(extent = {{-80, 20}, {-60, 40}})));
            Visualizers.Torus torus annotation(
              Placement(transformation(extent = {{32, 20}, {52, 40}})));
            Joints.Prismatic prismatic(useAxisFlange = true, animation = false, v(fixed = true)) annotation(
              Placement(transformation(extent = {{-38, 20}, {-18, 40}})));
            Translational.Sources.Position position annotation(
              Placement(transformation(extent = {{-66, 70}, {-46, 90}})));
            Blocks.Sources.Sine sine(amplitude = 2, freqHz = 0.5) annotation(
              Placement(transformation(extent = {{-98, 70}, {-78, 90}})));
            Visualizers.Ground ground(groundColor = {215, 215, 215}, length = 4) annotation(
              Placement(transformation(extent = {{-72, -16}, {-52, 4}})));
            Parts.FixedTranslation fixedTranslation1(r = {0, -1.3, torus.ro + torus.ri}, animation = false) annotation(
              Placement(transformation(extent = {{-4, 20}, {16, 40}})));
            Parts.FixedTranslation fixedTranslation2(animation = false, r = {0, -1.6, wheel.rTire}) annotation(
              Placement(transformation(extent = {{-4, 50}, {16, 70}})));
            Visualizers.VoluminousWheel wheel annotation(
              Placement(transformation(extent = {{32, 50}, {52, 70}})));
            Visualizers.PipeWithScalarField pipeWithScalarField(rOuter = 0.3, length = 1, T_min = 0, T_max = 2, T = sin(Modelica.Constants.pi * pipeWithScalarField.xsi) * cos(Modelica.Constants.pi * time) .+ 1, n_colors = 32) annotation(
              Placement(transformation(extent = {{14, -30}, {34, -10}})));
            Parts.FixedTranslation fixedTranslation3(animation = false, r = {0, -2.2, 0}) annotation(
              Placement(transformation(extent = {{-20, -30}, {0, -10}})));
          equation
            connect(world.frame_b, prismatic.frame_a) annotation(
              Line(points = {{-60, 30}, {-38, 30}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            connect(position.flange, prismatic.axis) annotation(
              Line(points = {{-46, 80}, {-24, 80}, {-24, 36}, {-20, 36}}, color = {0, 127, 0}, smooth = Smooth.None));
            connect(sine.y, position.s_ref) annotation(
              Line(points = {{-77, 80}, {-68, 80}}, color = {0, 0, 127}, smooth = Smooth.None));
            connect(prismatic.frame_b, fixedTranslation1.frame_a) annotation(
              Line(points = {{-18, 30}, {-4, 30}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            connect(fixedTranslation1.frame_b, torus.frame_a) annotation(
              Line(points = {{16, 30}, {32, 30}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            connect(prismatic.frame_b, fixedTranslation2.frame_a) annotation(
              Line(points = {{-18, 30}, {-14, 30}, {-14, 60}, {-4, 60}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            connect(fixedTranslation2.frame_b, wheel.frame_a) annotation(
              Line(points = {{16, 60}, {32, 60}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            connect(world.frame_b, fixedTranslation3.frame_a) annotation(
              Line(points = {{-60, 30}, {-46, 30}, {-46, -20}, {-20, -20}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            connect(fixedTranslation3.frame_b, pipeWithScalarField.frame_a) annotation(
              Line(points = {{0, -20}, {14, -20}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            annotation(
              experiment(StopTime = 5),
              Documentation(info = "<html>
<p>
This example demonstrates the use of the
<a href=\"modelica://Modelica.Mechanics.MultiBody.Visualizers.Advanced.Surface\">Surface</a>
visualizer that visualizes a moving, parameterized surface.
The \"sine-wave\" surface is a direct application of the surface model.
Furthermore, the \"torus\" surface is an instance of
<a href=\"modelica://Modelica.Mechanics.MultiBody.Visualizers.Torus\">Torus</a>,
the \"wheel\" surface is an instance of
<a href=\"modelica://Modelica.Mechanics.MultiBody.Visualizers.VoluminousWheel\">VoluminousWheel</a>,
and the \"pipeWithScalarField surface is an instance of
<a href=\"modelica://Modelica.Mechanics.MultiBody.Visualizers.PipeWithScalarField\">PipeWithScalarField</a>.
All latter visual shapes are constructed with the surface model.
The following image shows a screen-shot of this example model:
</p>

<blockquote>
<img src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Examples/Elementary/Surfaces.png\">
</blockquote>

</html>"));
          end Surfaces;

          package Utilities "Utility models and functions used by MultiBody.Examples.Elementary"
            extends Modelica.Icons.UtilitiesPackage;

            function theoreticalNormalGravityWGS84 "WGS84 normal gravity over earth ellipsoid in negative y-direction"
              extends Modelica.Mechanics.MultiBody.Interfaces.partialGravityAcceleration;
              input Modelica.SIunits.Conversions.NonSIunits.Angle_deg phi "Geodetic latitude" annotation(
                Dialog);
            protected
              constant Modelica.SIunits.Position a = 6378137.0 "Semi-major axis of the earth ellipsoid";
              constant Modelica.SIunits.Position b = 6356752.3142 "Semi-minor axis of the earth ellipsoid";
              constant Modelica.SIunits.AngularAcceleration g_e = 9.7803253359 "Theoretical gravity acceleration at the equator";
              constant Modelica.SIunits.AngularAcceleration g_p = 9.8321849378 "Theoretical gravity acceleration at the poles";
              constant Real k = b / a * (g_p / g_e) - 1;
              constant Real e2 = 8.1819190842622e-2 ^ 2 "Square of the first ellipsoidal eccentricity";
              constant Real f = 1 / 298.257223563 "Ellipsoidal flattening";
              constant Modelica.SIunits.AngularVelocity w = 7292115e-11 "Angular velocity of earth";
              constant Real GM(unit = "m3/s2") = 3986004.418e8 "Earths Gravitational Constant";
              constant Real m(unit = "1") = w ^ 2 * a ^ 2 * b / GM;
              Real sinphi2(unit = "1");
              Modelica.SIunits.AngularAcceleration gn "Normal gravity at the earth ellipsoid";
              Modelica.SIunits.AngularAcceleration gh "Normal gravity at height h over the earth ellipsoid";
              Modelica.SIunits.Position h "Height over the WGS84 earth ellipsoid";
              Real ha(unit = "1") "h/a";
            algorithm
              h := r[2];
              sinphi2 := Modelica.Math.sin(Modelica.SIunits.Conversions.from_deg(phi)) ^ 2;
              gn := g_e * (1 + k * sinphi2) / sqrt(1 - e2 * sinphi2);
              ha := h / a;
              gh := gn * (1 - ha * (2 * (1 + f + m - 2 * f * sinphi2) + 3 * ha));
              gravity := {0, -gh, 0};
              annotation(
                Documentation(info = "<html>
<p>
Function that computes the theoretical gravity of the
<a href=\"http://earth-info.nga.mil/GandG/publications/tr8350.2/wgs84fin.pdf\">WGS84 ellipsoid earth model</a> at and close to the earth ellipsoid surface, for the
given \"geodeticLatitude\" and the given \"height=r[2]\" over the
ellipsoid surface.
</p>

</html>"));
            end theoreticalNormalGravityWGS84;

            function sineSurface "Function defining the characteristic of a moving sine in three dimensions"
              extends Modelica.Mechanics.MultiBody.Interfaces.partialSurfaceCharacteristic;
              input Real x_min "Minimum value of x";
              input Real x_max "Maximum value of x";
              input Real y_min "Minimum value of y";
              input Real y_max "Maximum value of y";
              input Real z_min "Minimum value of z";
              input Real z_max "Maximum value of z";
              input Real wz "Factor for angular frequency";
            protected
              Real aux_y;
              Real A = (z_max - z_min) / 2;
            algorithm
              for i in 1:nu loop
                aux_y := y_min + (y_max - y_min) * (i - 1) / (nu - 1);
                for j in 1:nv loop
                  X[i, j] := x_min + (x_max - x_min) * (j - 1) / (nv - 1);
                  Y[i, j] := aux_y;
                  Z[i, j] := A * sin(wz + 0.1 * j + 0.1 * i) + A;
                end for;
              end for;
              if multiColoredSurface then
                C := {{(Z[i, j] + 1) * 200, 255, 0} for j in 1:nv, i in 1:nu};
              end if;
            end sineSurface;
          end Utilities;
          annotation(
            Documentation(info = "<HTML>
<p>
This package contains elementary example models to demonstrate
the usage of the MultiBody library
</p>
<h4>Content</h4>
<table border=1 cellspacing=0 cellpadding=2>
  <tr><th><b><i>Model</i></b></th><th><b><i>Description</i></b></th></tr>
  <tr><td valign=\"top\"><a href=\"modelica://Modelica.Mechanics.MultiBody.Examples.Elementary.DoublePendulum\">DoublePendulum</a></td>
      <td valign=\"top\"> Simple double pendulum with two revolute joints and two bodies.<br>
      <img src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Examples/Elementary/DoublePendulumSmall.png\">
      </td>
  </tr>
  <tr><td valign=\"top\"><a href=\"modelica://Modelica.Mechanics.MultiBody.Examples.Elementary.ForceAndTorque\">ForceAndTorque</a></td>
      <td valign=\"top\"> Demonstrates usage of Forces.ForceAndTorque element.<br>
      <img src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Examples/Elementary/ForceAndTorque_small.png\">
      </td>
  </tr>
  <tr><td valign=\"top\"><a href=\"modelica://Modelica.Mechanics.MultiBody.Examples.Elementary.FreeBody\">FreeBody</a></td>
      <td valign=\"top\"> Free flying body attached by two springs to environment.<br>
      <img src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Examples/Elementary/FreeBody_small.png\">
      </td>
  </tr>
  <tr><td valign=\"top\"><a href=\"modelica://Modelica.Mechanics.MultiBody.Examples.Elementary.InitSpringConstant\">InitSpringConstant</a></td>
      <td valign=\"top\"> Determine spring constant such that system is in steady state
           at given position.<br>
      <img src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Examples/Elementary/InitSpringConstant_small.png\">
      </td>
  </tr>
  <tr><td valign=\"top\"><a href=\"modelica://Modelica.Mechanics.MultiBody.Examples.Elementary.LineForceWithTwoMasses\">LineForceWithTwoMasses</a></td>
      <td valign=\"top\"> Demonstrates a line force with two point masses using a
           Joints.Assemblies.JointUPS and alternatively a
           Forces.LineForceWithTwoMasses component.<br>
      <img src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Examples/Elementary/LineForceWithTwoMasses_small.png\">
      </td>
  </tr>
  <tr><td valign=\"top\"><a href=\"modelica://Modelica.Mechanics.MultiBody.Examples.Elementary.Pendulum\">Pendulum</a></td>
      <td valign=\"top\"> Simple pendulum with one revolute joint and one body. <br>
      <img src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Examples/Elementary/Pendulum_small.png\">
      </td>
  </tr>
  <tr><td valign=\"top\"><a href=\"modelica://Modelica.Mechanics.MultiBody.Examples.Elementary.PendulumWithSpringDamper\">PendulumWithSpringDamper</a></td>
      <td valign=\"top\"> Simple spring/damper/mass system <br>
      <img src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Examples/Elementary/PendulumWithSpringDamper_small.png\">
      </td>
  </tr>
  <tr><td valign=\"top\"><a href=\"modelica://Modelica.Mechanics.MultiBody.Examples.Elementary.PointGravity\">PointGravity</a></td>
      <td valign=\"top\"> Two bodies in a point gravity field <br>
      <img src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Examples/Elementary/PointGravity_small.png\">
      </td>
  </tr>
  <tr><td valign=\"top\"><a href=\"modelica://Modelica.Mechanics.MultiBody.Examples.Elementary.PointGravityWithPointMasses\">PointGravityWithPointMasses</a></td>
      <td valign=\"top\"> Two point masses in a point gravity field (rotation of bodies is neglected) <br>
      <img src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Examples/Elementary/PointGravityWithPointMasses_small.png\">
      </td>
  </tr>
  <tr><td valign=\"top\"><a href=\"modelica://Modelica.Mechanics.MultiBody.Examples.Elementary.PointGravityWithPointMasses2\">PointGravityWithPointMasses2</a></td>
      <td valign=\"top\"> Rigidly connected point masses in a point gravity field <br>
      <img src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Examples/Elementary/PointGravityWithPointMasses2_small.png\">
      </td>
  </tr>

  <tr><td valign=\"top\"><a href=\"modelica://Modelica.Mechanics.MultiBody.Examples.Elementary.RollingWheel\">RollingWheel</a></td>
      <td valign=\"top\"> Single wheel rolling on ground starting from an initial speed <br>
      <img src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Examples/Elementary/RollingWheel.png\">
      </td>
  </tr>

  <tr><td valign=\"top\"><a href=\"modelica://Modelica.Mechanics.MultiBody.Examples.Elementary.RollingWheelSetDriving\">RollingWheelSetDriving</a></td>
      <td valign=\"top\"> Rolling wheel set that is driven by torques driving the wheels <br>
      <img src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Examples/Elementary/RollingWheelSetDriving.png\">
      </td>
  </tr>

  <tr><td valign=\"top\"><a href=\"modelica://Modelica.Mechanics.MultiBody.Examples.Elementary.RollingWheelSetPulling\">RollingWheelSetPulling</a></td>
      <td valign=\"top\"> Rolling wheel set that is pulled by a force <br>
      <img src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Examples/Elementary/RollingWheelSetPulling.png\">
      </td>
  </tr>

  <tr><td valign=\"top\"><a href=\"modelica://Modelica.Mechanics.MultiBody.Examples.Elementary.SpringDamperSystem\">SpringDamperSystem</a></td>
      <td valign=\"top\"> Spring/damper system with a prismatic joint and
           attached on free flying body <br>
      <img src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Examples/Elementary/SpringDamperSystem_small.png\">
      </td>
  </tr>
  <tr><td valign=\"top\"><a href=\"modelica://Modelica.Mechanics.MultiBody.Examples.Elementary.SpringMassSystem\">SpringMassSystem</a></td>
      <td valign=\"top\"> Mass attached via a prismatic joint and a spring to the world frame <br>
      <img src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Examples/Elementary/SpringMassSystem_small.png\">
      </td>
  </tr>
  <tr><td valign=\"top\"><a href=\"modelica://Modelica.Mechanics.MultiBody.Examples.Elementary.SpringWithMass\">SpringWithMass</a></td>
      <td valign=\"top\"> Point mass hanging on a spring <br>
      <img src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Examples/Elementary/SpringWithMass_small.png\">
      </td>
  </tr>
  <tr><td valign=\"top\"><a href=\"modelica://Modelica.Mechanics.MultiBody.Examples.Elementary.ThreeSprings\">ThreeSprings</a></td>
      <td valign=\"top\"> 3-dimensional springs in series and parallel connection<br>
      <img src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Examples/Elementary/ThreeSprings_small.png\">
      </td>
  </tr>

  <tr><td valign=\"top\"><a href=\"modelica://Modelica.Mechanics.MultiBody.Examples.Elementary.HeatLosses\">HeatLosses</a></td>
      <td valign=\"top\"> Demonstrate the modeling of heat losses.  </td>
  </tr>

  <tr><td valign=\"top\"><a href=\"modelica://Modelica.Mechanics.MultiBody.Examples.Elementary.UserDefinedGravityField\">UserDefinedGravityField </a></td>
      <td valign=\"top\"> Demonstrate the modeling of a user-defined gravity field.  </td>
  </tr>

  <tr><td valign=\"top\"><a href=\"modelica://Modelica.Mechanics.MultiBody.Examples.Elementary.Surfaces\">Surfaces</a></td>
      <td valign=\"top\"> Demonstrate the visualization of a sine surface,
       as well as a torus and a wheel constructed from a surface <br>
      <img src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Examples/Elementary/Surfaces_small.png\">
      </td>
  </tr>

</table>
</HTML>"));
        end Elementary;

        package Loops "Examples with kinematic loops"
          extends Modelica.Icons.ExamplesPackage;

          model Engine1a "Model of one cylinder engine"
            extends Modelica.Icons.Example;
            Modelica.Mechanics.MultiBody.Parts.BodyCylinder Piston(diameter = 0.1, r = {0, -0.1, 0}) annotation(
              Placement(transformation(origin = {90.5, 66.5}, extent = {{-10.5, 30.5}, {10.5, -30.5}}, rotation = 270)));
            Modelica.Mechanics.MultiBody.Parts.BodyBox Rod(widthDirection = {1, 0, 0}, width = 0.02, height = 0.06, r = {0, -0.2, 0}, color = {0, 0, 200}) annotation(
              Placement(transformation(origin = {90, 5}, extent = {{10, -10}, {-10, 10}}, rotation = 90)));
            Modelica.Mechanics.MultiBody.Joints.Revolute B2(n = {1, 0, 0}, cylinderLength = 0.02, cylinderDiameter = 0.05) annotation(
              Placement(transformation(extent = {{80, 22}, {100, 42}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Joints.Revolute Bearing(useAxisFlange = true, n = {1, 0, 0}, cylinderLength = 0.02, cylinderDiameter = 0.05) annotation(
              Placement(transformation(extent = {{-10, -80}, {10, -100}}, rotation = 0)));
            inner Modelica.Mechanics.MultiBody.World world annotation(
              Placement(transformation(extent = {{-50, -100}, {-30, -80}}, rotation = 0)));
            Modelica.Mechanics.Rotational.Components.Inertia Inertia(stateSelect = StateSelect.always, phi(fixed = true, start = 0), w(fixed = true, start = 10), J = 1) annotation(
              Placement(transformation(extent = {{-28, -120}, {-8, -100}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Parts.BodyBox Crank4(height = 0.05, widthDirection = {1, 0, 0}, width = 0.02, r = {0, -0.1, 0}) annotation(
              Placement(transformation(origin = {115.5, -75}, extent = {{10, -10}, {-10, 10}}, rotation = 90)));
            Modelica.Mechanics.MultiBody.Parts.BodyCylinder Crank3(r = {0.1, 0, 0}, diameter = 0.03) annotation(
              Placement(transformation(extent = {{81.5, -71}, {101.5, -51}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Parts.BodyCylinder Crank1(diameter = 0.05, r = {0.1, 0, 0}) annotation(
              Placement(transformation(extent = {{24, -100}, {44, -80}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Parts.BodyBox Crank2(r = {0, 0.1, 0}, height = 0.05, widthDirection = {1, 0, 0}, width = 0.02) annotation(
              Placement(transformation(origin = {70, -76}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
            Joints.RevolutePlanarLoopConstraint B1(n = {1, 0, 0}, cylinderLength = 0.02, cylinderDiameter = 0.05) annotation(
              Placement(transformation(extent = {{80, -30}, {100, -10}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Parts.FixedTranslation Mid(r = {0.05, 0, 0}) annotation(
              Placement(transformation(extent = {{70, -53}, {90, -33}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Joints.Prismatic Cylinder(boxWidth = 0.02, n = {0, -1, 0}, s(start = 0.15)) annotation(
              Placement(transformation(origin = {90, 96}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
            Modelica.Mechanics.MultiBody.Parts.FixedTranslation cylPosition(animation = false, r = {0.15, 0.45, 0}) annotation(
              Placement(transformation(extent = {{-0.5, 100}, {19.5, 120}}, rotation = 0)));
          equation
            connect(world.frame_b, Bearing.frame_a) annotation(
              Line(points = {{-30, -90}, {-10, -90}}, color = {95, 95, 95}, thickness = 0.5));
            connect(Crank2.frame_a, Crank1.frame_b) annotation(
              Line(points = {{70, -86}, {70, -90}, {44, -90}}, color = {95, 95, 95}, thickness = 0.5));
            connect(Crank2.frame_b, Crank3.frame_a) annotation(
              Line(points = {{70, -66}, {70, -61}, {81.5, -61}}, color = {95, 95, 95}, thickness = 0.5));
            connect(Bearing.frame_b, Crank1.frame_a) annotation(
              Line(points = {{10, -90}, {24, -90}}, color = {95, 95, 95}, thickness = 0.5));
            connect(cylPosition.frame_b, Cylinder.frame_a) annotation(
              Line(points = {{19.5, 110}, {90, 110}, {90, 106}}, color = {95, 95, 95}, thickness = 0.5));
            connect(world.frame_b, cylPosition.frame_a) annotation(
              Line(points = {{-30, -90}, {-20, -90}, {-20, 110}, {-0.5, 110}}, color = {95, 95, 95}, thickness = 0.5));
            connect(Crank3.frame_b, Crank4.frame_a) annotation(
              Line(points = {{101.5, -61}, {115, -61}, {115, -65}, {115.5, -65}}, color = {95, 95, 95}, thickness = 0.5));
            connect(B1.frame_a, Mid.frame_b) annotation(
              Line(points = {{80, -20}, {70, -20}, {70, -32}, {98, -32}, {98, -43}, {90, -43}}, color = {95, 95, 95}, thickness = 0.5));
            connect(B1.frame_b, Rod.frame_b) annotation(
              Line(points = {{100, -20}, {112, -20}, {112, -9}, {90, -9}, {90, -5}}, color = {95, 95, 95}, thickness = 0.5));
            connect(Rod.frame_a, B2.frame_b) annotation(
              Line(points = {{90, 15}, {90, 21}, {110, 21}, {110, 32}, {100, 32}}, color = {95, 95, 95}, thickness = 0.5));
            connect(B2.frame_a, Piston.frame_b) annotation(
              Line(points = {{80, 32}, {70, 32}, {70, 46}, {90.5, 46}, {90.5, 56}}, color = {95, 95, 95}, thickness = 0.5));
            connect(Inertia.flange_b, Bearing.axis) annotation(
              Line(points = {{-8, -110}, {0, -110}, {0, -100}}, color = {0, 0, 0}));
            connect(Mid.frame_a, Crank2.frame_b) annotation(
              Line(points = {{70, -43}, {63, -43}, {63, -61}, {70, -61}, {70, -66}}, color = {95, 95, 95}, thickness = 0.5));
            connect(Cylinder.frame_b, Piston.frame_a) annotation(
              Line(points = {{90, 86}, {90, 77}, {90.5, 77}}, color = {95, 95, 95}, thickness = 0.5));
            annotation(
              Diagram(coordinateSystem(extent = {{-130.0, -130.0}, {130.0, 130.0}})),
              experiment(StopTime = 5),
              Documentation(info = "<html>
<p>
This is a model of the mechanical part of one cylinder of an engine.
The combustion is not modelled. The \"inertia\" component at the lower
left part is the output inertia of the engine driving the gearbox.
The angular velocity of the output inertia has a start value of 10 rad/s
in order to demonstrate the movement of the engine.
</p>
<p>
The engine is modeled solely by revolute and prismatic joints.
Since this results in a <b>planar</b> loop there is the well known
difficulty that the cut-forces perpendicular to the loop cannot be
uniquely computed, as well as the cut-torques within the plane.
This ambiguity is resolved by using the option <b>planarCutJoint</b>
in the <b>Advanced</b> menu of one revolute joint in every planar loop
(here: joint B1). This option sets the cut-force in direction of the
axis of rotation, as well as the cut-torques perpendicular to the axis
of rotation at this joint to zero and makes the problem mathematically
well-formed.
</p>
<p>
An animation of this example is shown in the figure below.
</p>

<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Examples/Loops/Engine.png\" ALT=\"model Examples.Loops.Engine\">
</html>"));
          end Engine1a;

          model Engine1b "Model of one cylinder engine with gas force and preparation for assembly joint JointRRP"
            extends Modelica.Icons.Example;
            extends Utilities.Engine1bBase(Inertia(w(start = 0)));
            Joints.RevolutePlanarLoopConstraint B2(n = {1, 0, 0}, cylinderLength = 0.02, cylinderDiameter = 0.05) annotation(
              Placement(transformation(extent = {{40, 20}, {60, 40}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Joints.Revolute B1(n = {1, 0, 0}, cylinderLength = 0.02, cylinderDiameter = 0.05) annotation(
              Placement(transformation(extent = {{40, -20}, {60, 0}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Joints.Prismatic Cylinder(useAxisFlange = true, boxWidth = 0.02, n = {0, -1, 0}) annotation(
              Placement(transformation(origin = {50, 97}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
            Parts.FixedTranslation Rod1(r = {0, 0.2, 0}, animation = false) annotation(
              Placement(transformation(origin = {70, 10}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
            Parts.FixedTranslation Rod3(r = {0, -0.1, 0}, animation = false) annotation(
              Placement(transformation(origin = {50, 58}, extent = {{10, -10}, {-10, 10}}, rotation = 90)));
          equation
            connect(B1.frame_b, Rod1.frame_a) annotation(
              Line(points = {{60, -10}, {70, -10}, {70, 0}}, color = {95, 95, 95}, thickness = 0.5));
            connect(Rod1.frame_b, B2.frame_b) annotation(
              Line(points = {{70, 20}, {70, 30}, {60, 30}}, color = {95, 95, 95}, thickness = 0.5));
            connect(Cylinder.frame_b, Rod3.frame_a) annotation(
              Line(points = {{50, 87}, {50, 68}}, color = {95, 95, 95}, thickness = 0.5));
            connect(B2.frame_a, Rod3.frame_b) annotation(
              Line(points = {{40, 30}, {30, 30}, {30, 44}, {50, 44}, {50, 48}}, color = {95, 95, 95}, thickness = 0.5));
            connect(cylPosition.frame_b, Cylinder.frame_a) annotation(
              Line(points = {{-20.5, 110}, {50, 110}, {50, 107}}, color = {95, 95, 95}, thickness = 0.5));
            connect(gasForce.flange_a, Cylinder.support) annotation(
              Line(points = {{119, 107}, {119, 112}, {70, 112}, {70, 101}, {56, 101}}, color = {0, 127, 0}));
            connect(Cylinder.axis, gasForce.flange_b) annotation(
              Line(points = {{56, 89}, {71, 89}, {71, 80}, {119, 80}, {119, 87}}, color = {0, 127, 0}));
            connect(Piston.frame_a, Rod3.frame_a) annotation(
              Line(points = {{120, 63}, {120, 75}, {50, 75}, {50, 68}}, color = {95, 95, 95}, thickness = 0.5));
            connect(B1.frame_b, Rod2.frame_a) annotation(
              Line(points = {{60, -10}, {120, -10}, {120, 0}}, color = {95, 95, 95}, thickness = 0.5));
            connect(Mid.frame_b, B1.frame_a) annotation(
              Line(points = {{50, -43}, {57, -43}, {57, -28}, {30, -28}, {30, -10}, {40, -10}}, color = {95, 95, 95}, thickness = 0.5));
            annotation(
              experiment(StopTime = 0.5),
              Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -120}, {150, 120}}), graphics = {Rectangle(extent = {{3, 117}, {88, -23}}, lineColor = {255, 0, 0}, lineThickness = 0.5), Text(extent = {{65, -30}, {145, -39}}, lineColor = {255, 0, 0}, lineThickness = 0.5, textString = "jointRRP in model"), Text(extent = {{66, -34}, {141, -52}}, lineColor = {255, 0, 0}, lineThickness = 0.5, textString = "Loops.Engine1b_analytic")}),
              Documentation(info = "<html>
<p>
This is a model of the mechanical part of one cylinder of an engine.
It is similar to
<a href=\"modelica://Modelica.Mechanics.MultiBody.Examples.Loops.Engine1a#diagram\">Loops.Engine1a</a>.
The difference is that a simple
model for the gas force in the cylinder is added and that the
model is restructured in such a way, that the central part of
the planar kinematic loop can be easily replaced by the
assembly joint \"Modelica.Mechanics.MultiBody.Joints.Assemblies.<b>JointRRP</b>\".
This exchange of the kinematic loop is shown in
<a href=\"modelica://Modelica.Mechanics.MultiBody.Examples.Loops.Engine1b_analytic#diagram\">Loops.Engine1b_analytic</a>.
The advantage of using JointRRP is, that the
non-linear algebraic equation of this loop is solved analytically, and
not numerically as in this model (Engine1b).
</p>
<p>
An animation of this example is shown in the figure below.
</p>

<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Examples/Loops/Engine.png\" ALT=\"model Examples.Loops.Engine\">
</html>"));
          end Engine1b;

          model Engine1b_analytic "Model of one cylinder engine with gas force and analytic loop handling"
            extends Modelica.Icons.Example;
            extends Utilities.Engine1bBase(Inertia(w(start = 0)));
            Joints.Assemblies.JointRRP jointRRP(n_a = {1, 0, 0}, n_b = {0, -1, 0}, animation = false, rRod1_ia = {0, 0.2, 0}, rRod2_ib = {0, -0.1, 0}) annotation(
              Placement(transformation(origin = {30, 54}, extent = {{-20, 20}, {20, -20}}, rotation = 90)));
          equation
            connect(Mid.frame_b, jointRRP.frame_a) annotation(
              Line(points = {{50, -43}, {58, -43}, {58, -30}, {30, -30}, {30, 34}}, color = {95, 95, 95}, thickness = 0.5));
            connect(jointRRP.frame_b, cylPosition.frame_b) annotation(
              Line(points = {{30, 74}, {30, 110}, {-20.5, 110}}, color = {95, 95, 95}, thickness = 0.5));
            connect(jointRRP.axis, gasForce.flange_b) annotation(
              Line(points = {{46, 74}, {46, 82}, {119, 82}, {119, 87}}, color = {0, 127, 0}));
            connect(jointRRP.bearing, gasForce.flange_a) annotation(
              Line(points = {{38, 74}, {38, 114}, {119, 114}, {119, 107}}, color = {0, 127, 0}));
            connect(jointRRP.frame_ib, Piston.frame_a) annotation(
              Line(points = {{50, 70}, {120, 70}, {120, 63}}, color = {95, 95, 95}, thickness = 0.5));
            connect(jointRRP.frame_ia, Rod2.frame_a) annotation(
              Line(points = {{50, 38}, {80, 38}, {80, -10}, {120, -10}, {120, 0}}, color = {95, 95, 95}, thickness = 0.5));
            annotation(
              experiment(StopTime = 0.5),
              Documentation(info = "<html>
<p>
This is the same model as
<a href=\"modelica://Modelica.Mechanics.MultiBody.Examples.Loops.Engine1b#diagram\">Loops.Engine1b</a>.
The only difference is that the central part of
the planar kinematic loop has been replaced by the
assembly joint \"Modelica.Mechanics.MultiBody.Joints.Assemblies.<b>JointRRP</b>\".
The advantage of using JointRRP is, that the
non-linear algebraic equation of this loop is solved analytically, and
not numerically as in
<a href=\"modelica://Modelica.Mechanics.MultiBody.Examples.Loops.Engine1b#diagram\">Loops.Engine1b</a>.
</p>
<p>
An animation of this example is shown in the figure below.
</p>

<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Examples/Loops/Engine.png\" ALT=\"model Examples.Loops.Engine\">
</html>"));
          end Engine1b_analytic;

          model EngineV6 "V6 engine with 6 cylinders, 6 planar loops and 1 degree-of-freedom"
            extends Modelica.Icons.Example;
            parameter Boolean animation = true "= true, if animation shall be enabled";
            output Modelica.SIunits.Conversions.NonSIunits.AngularVelocity_rpm engineSpeed_rpm = Modelica.SIunits.Conversions.to_rpm(load.w) "Engine speed";
            output Modelica.SIunits.Torque engineTorque = filter.u "Torque generated by engine";
            output Modelica.SIunits.Torque filteredEngineTorque = filter.y "Filtered torque generated by engine";
            Modelica.Mechanics.MultiBody.Joints.Revolute bearing(useAxisFlange = true, n = {1, 0, 0}, cylinderLength = 0.02, cylinderDiameter = 0.06, animation = animation) annotation(
              Placement(transformation(extent = {{-90, 20}, {-70, 0}}, rotation = 0)));
            inner Modelica.Mechanics.MultiBody.World world(animateWorld = false, animateGravity = false) annotation(
              Placement(transformation(extent = {{-120, 20}, {-100, 40}}, rotation = 0)));
            Utilities.Cylinder cylinder1(crankAngleOffset = Cv.from_deg(-30), cylinderInclination = Cv.from_deg(-30), animation = animation) annotation(
              Placement(transformation(extent = {{-50, 4}, {-30, 34}}, rotation = 0)));
            Utilities.Cylinder cylinder2(crankAngleOffset = Cv.from_deg(90), cylinderInclination = Cv.from_deg(30), animation = animation) annotation(
              Placement(transformation(extent = {{-20, 4}, {0, 34}}, rotation = 0)));
            Utilities.Cylinder cylinder3(cylinderInclination = Cv.from_deg(-30), animation = animation, crankAngleOffset = Cv.from_deg(210)) annotation(
              Placement(transformation(extent = {{10, 4}, {30, 34}}, rotation = 0)));
            Utilities.Cylinder cylinder4(cylinderInclination = Cv.from_deg(30), animation = animation, crankAngleOffset = Cv.from_deg(210)) annotation(
              Placement(transformation(extent = {{39, 4}, {59, 34}}, rotation = 0)));
            Utilities.Cylinder cylinder5(cylinderInclination = Cv.from_deg(-30), animation = animation, crankAngleOffset = Cv.from_deg(90)) annotation(
              Placement(transformation(extent = {{70, 4}, {90, 34}}, rotation = 0)));
            Utilities.Cylinder cylinder6(cylinderInclination = Cv.from_deg(30), animation = animation, crankAngleOffset = Cv.from_deg(-30)) annotation(
              Placement(transformation(extent = {{100, 4}, {120, 34}}, rotation = 0)));
            Modelica.Mechanics.Rotational.Components.Inertia load(phi(start = 0, fixed = true), w(start = 10, fixed = true), stateSelect = StateSelect.always, J = 1) annotation(
              Placement(transformation(extent = {{-44, -30}, {-24, -10}}, rotation = 0)));
            Modelica.Mechanics.Rotational.Sources.QuadraticSpeedDependentTorque load2(tau_nominal = -100, w_nominal = 200, useSupport = false) annotation(
              Placement(transformation(extent = {{6, -30}, {-14, -10}}, rotation = 0)));
            Rotational.Sensors.TorqueSensor torqueSensor annotation(
              Placement(transformation(extent = {{-72, -30}, {-52, -10}}, rotation = 0)));
            Blocks.Continuous.CriticalDamping filter(n = 2, initType = Modelica.Blocks.Types.Init.SteadyState, f = 5) annotation(
              Placement(transformation(extent = {{-54, -60}, {-34, -40}}, rotation = 0)));
          equation
            connect(bearing.frame_b, cylinder1.crank_a) annotation(
              Line(points = {{-70, 10}, {-50, 10}}, color = {95, 95, 95}, thickness = 0.5));
            connect(cylinder1.crank_b, cylinder2.crank_a) annotation(
              Line(points = {{-30, 10}, {-20, 10}}, color = {95, 95, 95}, thickness = 0.5));
            connect(cylinder2.crank_b, cylinder3.crank_a) annotation(
              Line(points = {{0, 10}, {10, 10}}, color = {95, 95, 95}, thickness = 0.5));
            connect(cylinder3.crank_b, cylinder4.crank_a) annotation(
              Line(points = {{30, 10}, {39, 10}}, color = {95, 95, 95}, thickness = 0.5));
            connect(cylinder4.crank_b, cylinder5.crank_a) annotation(
              Line(points = {{59, 10}, {70, 10}}, color = {95, 95, 95}, thickness = 0.5));
            connect(cylinder5.crank_b, cylinder6.crank_a) annotation(
              Line(points = {{90, 10}, {100, 10}}, color = {95, 95, 95}, thickness = 0.5));
            connect(cylinder5.cylinder_b, cylinder6.cylinder_a) annotation(
              Line(points = {{90, 32}, {100, 32}}, color = {95, 95, 95}, thickness = 0.5));
            connect(cylinder4.cylinder_b, cylinder5.cylinder_a) annotation(
              Line(points = {{59, 32}, {70, 32}}, color = {95, 95, 95}, thickness = 0.5));
            connect(cylinder4.cylinder_a, cylinder3.cylinder_b) annotation(
              Line(points = {{39, 32}, {30, 32}}, color = {95, 95, 95}, thickness = 0.5));
            connect(cylinder3.cylinder_a, cylinder2.cylinder_b) annotation(
              Line(points = {{10, 32}, {0, 32}}, color = {95, 95, 95}, thickness = 0.5));
            connect(cylinder2.cylinder_a, cylinder1.cylinder_b) annotation(
              Line(points = {{-20, 32}, {-30, 32}}, color = {95, 95, 95}, thickness = 0.5));
            connect(world.frame_b, cylinder1.cylinder_a) annotation(
              Line(points = {{-100, 30}, {-59, 30}, {-59, 32}, {-50, 32}}, color = {95, 95, 95}, thickness = 0.5));
            connect(world.frame_b, bearing.frame_a) annotation(
              Line(points = {{-100, 30}, {-96, 30}, {-96, 10}, {-90, 10}}, color = {95, 95, 95}, thickness = 0.5));
            connect(load2.flange, load.flange_b) annotation(
              Line(points = {{-14, -20}, {-24, -20}}, color = {0, 0, 0}));
            connect(torqueSensor.flange_b, load.flange_a) annotation(
              Line(points = {{-52, -20}, {-44, -20}}, color = {0, 0, 0}));
            connect(torqueSensor.tau, filter.u) annotation(
              Line(points = {{-70, -31}, {-70, -50}, {-56, -50}}, color = {0, 0, 127}));
            connect(torqueSensor.flange_a, bearing.axis) annotation(
              Line(points = {{-72, -20}, {-80, -20}, {-80, 0}}, color = {0, 0, 0}));
            annotation(
              Documentation(info = "<HTML>
<p>
This is a V6 engine with 6 cylinders. It is hierarchically built
up by using instances of one cylinder. For more details on the
modeling of one cylinder, see example
<a href=\"modelica://Modelica.Mechanics.MultiBody.Examples.Loops.Engine1b\">Engine1b</a>.
An animation of the engine is shown in the figure below.
</p>

<p>
<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Examples/Loops/EngineV6.png\" ALT=\"model Examples.Loops.EngineV6\">
</p>

<p>
Simulate for 3 s with about 50000 output intervals, and plot the variables <b>engineSpeed_rpm</b>,
<b>engineTorque</b>, and <b>filteredEngineTorque</b>. Note, the result file has
a size of about 300 Mbyte in this case. The default setting of StopTime = 1.01 s (with the default setting of the tool for the number of output points), in order that (automatic) regression testing does not have to cope with a large result file.
</p>
</html>"),
              experiment(StopTime = 1.01),
              Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{-140, -100}, {140, 100}})));
          end EngineV6;

          model EngineV6_analytic "V6 engine with 6 cylinders, 6 planar loops, 1 degree-of-freedom and analytic handling of kinematic loops"
            extends Modelica.Icons.Example;
            parameter Boolean animation = true "= true, if animation shall be enabled";
            output Modelica.SIunits.Conversions.NonSIunits.AngularVelocity_rpm engineSpeed_rpm = Modelica.SIunits.Conversions.to_rpm(load.w) "Engine speed";
            output Modelica.SIunits.Torque engineTorque = filter.u "Torque generated by engine";
            output Modelica.SIunits.Torque filteredEngineTorque = filter.y "Filtered torque generated by engine";
            inner Modelica.Mechanics.MultiBody.World world(animateWorld = false, animateGravity = false) annotation(
              Placement(transformation(extent = {{-80, -20}, {-60, 0}}, rotation = 0)));
            Utilities.EngineV6_analytic engine(redeclare model Cylinder = Modelica.Mechanics.MultiBody.Examples.Loops.Utilities.Cylinder_analytic_CAD) annotation(
              Placement(transformation(extent = {{-40, 0}, {0, 40}}, rotation = 0)));
            Modelica.Mechanics.Rotational.Components.Inertia load(phi(start = 0, fixed = true), w(start = 10, fixed = true), stateSelect = StateSelect.always, J = 1) annotation(
              Placement(transformation(extent = {{40, 10}, {60, 30}}, rotation = 0)));
            Modelica.Mechanics.Rotational.Sources.QuadraticSpeedDependentTorque load2(tau_nominal = -100, w_nominal = 200, useSupport = false) annotation(
              Placement(transformation(extent = {{90, 10}, {70, 30}}, rotation = 0)));
            Rotational.Sensors.TorqueSensor torqueSensor annotation(
              Placement(transformation(extent = {{12, 10}, {32, 30}}, rotation = 0)));
            Blocks.Continuous.CriticalDamping filter(n = 2, initType = Modelica.Blocks.Types.Init.SteadyState, f = 5) annotation(
              Placement(transformation(extent = {{30, -20}, {50, 0}}, rotation = 0)));
          equation
            connect(world.frame_b, engine.frame_a) annotation(
              Line(points = {{-60, -10}, {-20, -10}, {-20, -0.2}}, color = {95, 95, 95}, thickness = 0.5));
            connect(load2.flange, load.flange_b) annotation(
              Line(points = {{70, 20}, {60, 20}}, color = {0, 0, 0}));
            connect(torqueSensor.flange_a, engine.flange_b) annotation(
              Line(points = {{12, 20}, {2, 20}}, color = {0, 0, 0}));
            connect(torqueSensor.flange_b, load.flange_a) annotation(
              Line(points = {{32, 20}, {40, 20}}, color = {0, 0, 0}));
            connect(torqueSensor.tau, filter.u) annotation(
              Line(points = {{14, 9}, {14, -10}, {28, -10}}, color = {0, 0, 127}));
            annotation(
              Documentation(info = "<HTML>
<p>
This is a similar model as the example \"EngineV6\". However, the cylinders
have been built up with component Modelica.Mechanics.MultiBody.Joints.Assemblies.JointRRR that
solves the non-linear system of equations in an aggregation of 3 revolution
joints <b>analytically</b> and only one body is used that holds the total
mass of the crank shaft:
</p>

<p>
<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Examples/Loops/EngineV6_CAD_small.png\">
</p>

<p>
This model is about 20 times faster as the EngineV6 example and <b>no</b> linear or
non-linear system of equations occur. In contrast, the \"EngineV6\" example
leads to 6 systems of nonlinear equations (every system has dimension = 5, with
Evaluate=false and dimension=1 with Evaluate=true) and a linear system of equations
of about 40. This shows the power of the analytic loop handling.
</p>


<p>
Simulate for 3 s with about 50000 output intervals, and plot the variables <b>engineSpeed_rpm</b>,
<b>engineTorque</b>, and <b>filteredEngineTorque</b>. Note, the result file has
a size of about 240 Mbyte in this case. The default setting of StopTime = 1.01 s (with the default setting of the tool for the number of output points), in order that (automatic) regression testing does not have to cope with a large result file.
</p>

</html>"),
              experiment(StopTime = 1.01));
          end EngineV6_analytic;

          model Fourbar1 "One kinematic loop with four bars (with only revolute joints; 5 non-linear equations)"
            extends Modelica.Icons.Example;
            output SI.Angle j1_phi "angle of revolute joint j1";
            output SI.Position j2_s "distance of prismatic joint j2";
            output SI.AngularVelocity j1_w "axis speed of revolute joint j1";
            output SI.Velocity j2_v "axis velocity of prismatic joint j2";
            inner Modelica.Mechanics.MultiBody.World world annotation(
              Placement(transformation(extent = {{-100, -80}, {-80, -60}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Joints.Revolute j1(n = {1, 0, 0}, stateSelect = StateSelect.always, phi(fixed = true), w(displayUnit = "deg/s", start = 5.235987755982989, fixed = true)) annotation(
              Placement(transformation(extent = {{-54, -40}, {-34, -20}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Joints.Prismatic j2(n = {1, 0, 0}, s(start = -0.2), boxWidth = 0.05) annotation(
              Placement(transformation(extent = {{10, -80}, {30, -60}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Parts.BodyCylinder b1(r = {0, 0.5, 0.1}, diameter = 0.05) annotation(
              Placement(transformation(origin = {-30, 2}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
            Modelica.Mechanics.MultiBody.Parts.BodyCylinder b2(r = {0, 0.2, 0}, diameter = 0.05) annotation(
              Placement(transformation(origin = {50, -50}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
            Modelica.Mechanics.MultiBody.Parts.BodyCylinder b3(r = {-1, 0.3, 0.1}, diameter = 0.05) annotation(
              Placement(transformation(extent = {{38, 20}, {18, 40}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Joints.Revolute rev(n = {0, 1, 0}) annotation(
              Placement(transformation(origin = {50, -22}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
            Modelica.Mechanics.MultiBody.Joints.Revolute rev1 annotation(
              Placement(transformation(extent = {{60, 0}, {80, 20}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Joints.Revolute j3(n = {1, 0, 0}) annotation(
              Placement(transformation(extent = {{-60, 40}, {-40, 60}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Joints.Revolute j4(n = {0, 1, 0}) annotation(
              Placement(transformation(extent = {{-32, 60}, {-12, 80}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Joints.Revolute j5(n = {0, 0, 1}) annotation(
              Placement(transformation(extent = {{0, 70}, {20, 90}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Parts.FixedTranslation b0(animation = false, r = {1.2, 0, 0}) annotation(
              Placement(transformation(extent = {{-40, -80}, {-20, -60}}, rotation = 0)));
          equation
            connect(j2.frame_b, b2.frame_a) annotation(
              Line(points = {{30, -70}, {50, -70}, {50, -60}}, color = {95, 95, 95}, thickness = 0.5));
            connect(j1.frame_b, b1.frame_a) annotation(
              Line(points = {{-34, -30}, {-30, -30}, {-30, -8}}, color = {95, 95, 95}, thickness = 0.5));
            connect(rev.frame_a, b2.frame_b) annotation(
              Line(points = {{50, -32}, {50, -40}}, color = {95, 95, 95}, thickness = 0.5));
            connect(rev.frame_b, rev1.frame_a) annotation(
              Line(points = {{50, -12}, {50, 10}, {60, 10}}, color = {95, 95, 95}, thickness = 0.5));
            connect(rev1.frame_b, b3.frame_a) annotation(
              Line(points = {{80, 10}, {90, 10}, {90, 30}, {38, 30}}, color = {95, 95, 95}, thickness = 0.5));
            connect(world.frame_b, j1.frame_a) annotation(
              Line(points = {{-80, -70}, {-66, -70}, {-66, -30}, {-54, -30}}, color = {95, 95, 95}, thickness = 0.5));
            connect(b1.frame_b, j3.frame_a) annotation(
              Line(points = {{-30, 12}, {-30, 28}, {-72, 28}, {-72, 50}, {-60, 50}}, color = {95, 95, 95}, thickness = 0.5));
            connect(j3.frame_b, j4.frame_a) annotation(
              Line(points = {{-40, 50}, {-34, 50}, {-42, 70}, {-32, 70}}, color = {95, 95, 95}, thickness = 0.5));
            connect(j4.frame_b, j5.frame_a) annotation(
              Line(points = {{-12, 70}, {-5.55112e-16, 70}, {-5.55112e-16, 80}}, color = {95, 95, 95}, thickness = 0.5));
            connect(j5.frame_b, b3.frame_b) annotation(
              Line(points = {{20, 80}, {30, 80}, {30, 54}, {4, 54}, {4, 30}, {18, 30}}, color = {95, 95, 95}, thickness = 0.5));
            connect(b0.frame_a, world.frame_b) annotation(
              Line(points = {{-40, -70}, {-80, -70}}, color = {95, 95, 95}, thickness = 0.5));
            connect(b0.frame_b, j2.frame_a) annotation(
              Line(points = {{-20, -70}, {10, -70}}, color = {95, 95, 95}, thickness = 0.5));
            j1_phi = j1.phi;
            j2_s = j2.s;
            j1_w = j1.w;
            j2_v = j2.v;
            annotation(
              experiment(StopTime = 5),
              Documentation(info = "<html>
<p>
This is a simple kinematic loop consisting of 6 revolute joints, 1 prismatic joint
and 4 bars that is often used as basic constructing unit in mechanisms.
This example demonstrates that usually no particular knowledge
of the user is needed to handle kinematic loops.
Just connect the joints and bodies together according
to the real system. In particular <b>no</b> cut-joints or a spanning tree has
to be determined. In this case, the initial condition of the angular velocity
of revolute joint j1 is set to 300 deg/s in order to drive this loop.
</p>

<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Examples/Loops/Fourbar1.png\" ALT=\"model Examples.Loops.Fourbar1\">
</html>"));
          end Fourbar1;

          model Fourbar2 "One kinematic loop with four bars (with UniversalSpherical joint; 1 non-linear equation)"
            extends Modelica.Icons.Example;
            output SI.Angle j1_phi "angle of revolute joint j1";
            output SI.Position j2_s "distance of prismatic joint j2";
            output SI.AngularVelocity j1_w "axis speed of revolute joint j1";
            output SI.Velocity j2_v "axis velocity of prismatic joint j2";
            inner Modelica.Mechanics.MultiBody.World world annotation(
              Placement(transformation(extent = {{-80, -80}, {-60, -60}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Joints.Revolute j1(useAxisFlange = true, n = {1, 0, 0}, stateSelect = StateSelect.always, phi(fixed = true), w(displayUnit = "deg/s", start = 5.235987755982989, fixed = true)) annotation(
              Placement(transformation(extent = {{-54, -40}, {-34, -20}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Joints.Prismatic j2(n = {1, 0, 0}, boxWidth = 0.05, s(fixed = true, start = -0.2)) annotation(
              Placement(transformation(extent = {{12, -80}, {32, -60}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Parts.BodyCylinder b1(r = {0, 0.5, 0.1}, diameter = 0.05) annotation(
              Placement(transformation(origin = {-30, 0}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
            Modelica.Mechanics.MultiBody.Parts.BodyCylinder b2(r = {0, 0.2, 0}, diameter = 0.05) annotation(
              Placement(transformation(origin = {50, -50}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
            Modelica.Mechanics.MultiBody.Joints.UniversalSpherical universalSpherical(n1_a = {0, 1, 0}, computeRodLength = true, rRod_ia = {-1, 0.3, 0.1}) annotation(
              Placement(transformation(extent = {{0, 18}, {-20, 38}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Parts.FixedTranslation b3(r = {1.2, 0, 0}, animation = false) annotation(
              Placement(transformation(extent = {{-32, -80}, {-12, -60}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Visualizers.FixedFrame fixedFrame(color_x = {0, 0, 255}) annotation(
              Placement(transformation(origin = {-6, 70}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
          equation
            j1_phi = j1.phi;
            j2_s = j2.s;
            j1_w = j1.w;
            j2_v = j2.v;
            connect(j2.frame_b, b2.frame_a) annotation(
              Line(points = {{32, -70}, {50, -70}, {50, -60}}, color = {95, 95, 95}, thickness = 0.5));
            connect(j1.frame_b, b1.frame_a) annotation(
              Line(points = {{-34, -30}, {-30, -30}, {-30, -10}}, color = {95, 95, 95}, thickness = 0.5));
            connect(j1.frame_a, world.frame_b) annotation(
              Line(points = {{-54, -30}, {-60, -30}, {-60, -70}}, color = {95, 95, 95}, thickness = 0.5));
            connect(b1.frame_b, universalSpherical.frame_b) annotation(
              Line(points = {{-30, 10}, {-30, 28}, {-20, 28}}, color = {95, 95, 95}, thickness = 0.5));
            connect(universalSpherical.frame_a, b2.frame_b) annotation(
              Line(points = {{0, 28}, {50, 28}, {50, -40}}, color = {95, 95, 95}, thickness = 0.5));
            connect(b3.frame_a, world.frame_b) annotation(
              Line(points = {{-32, -70}, {-60, -70}}, color = {95, 95, 95}, thickness = 0.5));
            connect(b3.frame_b, j2.frame_a) annotation(
              Line(points = {{-12, -70}, {12, -70}}, color = {95, 95, 95}, thickness = 0.5));
            connect(fixedFrame.frame_a, universalSpherical.frame_ia) annotation(
              Line(points = {{-6, 60}, {-6, 49}, {-6, 49}, {-6, 38}}, color = {95, 95, 95}, thickness = 0.5));
            annotation(
              experiment(StopTime = 5),
              Documentation(info = "<html>
<p>
This is a second version of the \"four-bar\" mechanism, see figure:
</p>

<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Examples/Loops/Fourbar2.png\" ALT=\"model Examples.Loops.Fourbar2\">

<p>
In this case
the three revolute joints on the left top-side and the two revolute
joints on the right top side have been replaced by the joint <b>UniversalSpherical</b>
that is a rod connecting a spherical and a universal joint. This joint is defined
by <b>1 constraint</b> stating that the distance between the two spherical joints is
constant. Using this joint in a kinematic loop reduces the sizes of
non-linear algebraic equations. For this loop, only one non-linear
algebraic system of equations of order 1 remains.
</p>
<p>
At the UniversalSpherical joint an additional frame_ia fixed to the rod
is present where components can be attached to the connecting rod. In this
example just a coordinate system is attached to visualize frame_ia (coordinate
system on the right in blue color).
</p>
<p>
Another feature is that the length of the connecting rod can be
automatically calculated during <b>initialization</b>. In order to do this,
another initialization condition has to be given. In this example, the
initial value of the distance of the prismatic joint j2 has been fixed
(via the \"Initialization\" menu) and the rod length of joint
\"UniversalSpherical\" is computed during initialization since parameter
<b>computeLength</b> = <b>true</b> is set in the joint parameter
menu. The main advantage is that during initialization no non-linear
system of equation is solved and therefore initialization always works.
To be precise, the following trivial non-linear equation is actually solved
for rodLength:
</p>
<pre>
   rodLength*rodLength = f(angle of revolute joint, distance of prismatic joint)
</pre>
</html>"));
          end Fourbar2;

          model Fourbar_analytic "One kinematic loop with four bars (with JointSSP joint; analytic solution of non-linear algebraic loop)"
            extends Modelica.Icons.Example;
            output SI.Angle j1_phi "angle of revolute joint j1";
            output SI.Position j2_s "distance of prismatic joint j2";
            output SI.AngularVelocity j1_w "axis speed of revolute joint j1";
            output SI.Velocity j2_v "axis velocity of prismatic joint j2";
            inner Modelica.Mechanics.MultiBody.World world(animateGravity = false) annotation(
              Placement(transformation(extent = {{-80, -60}, {-60, -40}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Joints.Revolute j1(useAxisFlange = true, n = {1, 0, 0}, stateSelect = StateSelect.always, phi(fixed = true), w(displayUnit = "deg/s", start = 5.235987755982989, fixed = true)) annotation(
              Placement(transformation(extent = {{-54, -40}, {-34, -20}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Parts.BodyCylinder b1(r = {0, 0.5, 0.1}, diameter = 0.05) annotation(
              Placement(transformation(origin = {-30, -8}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
            Modelica.Mechanics.MultiBody.Parts.FixedTranslation b3(r = {1.2, 0, 0}, animation = false) annotation(
              Placement(transformation(extent = {{-32, -60}, {-12, -40}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Joints.Assemblies.JointSSP jointSSP(rod1Length = sqrt({-1, 0.3, 0.1} * {-1, 0.3, 0.1}), n_b = {1, 0, 0}, s_offset = -0.2, rRod2_ib = {0, 0.2, 0}, rod1Color = {0, 128, 255}, rod2Color = {0, 128, 255}, checkTotalPower = true) annotation(
              Placement(transformation(extent = {{-20, 0}, {20, 40}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Parts.BodyCylinder b2(r = {0, 0.2, 0}, diameter = 0.05, animation = false) annotation(
              Placement(transformation(origin = {50, 10}, extent = {{10, -10}, {-10, 10}}, rotation = 90)));
          equation
            j1_phi = j1.phi;
            j2_s = jointSSP.prismatic.distance;
            j1_w = j1.w;
            j2_v = der(jointSSP.prismatic.distance);
            connect(j1.frame_b, b1.frame_a) annotation(
              Line(points = {{-34, -30}, {-30, -30}, {-30, -18}}, color = {95, 95, 95}, thickness = 0.5));
            connect(j1.frame_a, world.frame_b) annotation(
              Line(points = {{-54, -30}, {-60, -30}, {-60, -50}}, color = {95, 95, 95}, thickness = 0.5));
            connect(b3.frame_a, world.frame_b) annotation(
              Line(points = {{-32, -50}, {-60, -50}}, color = {95, 95, 95}, thickness = 0.5));
            connect(b1.frame_b, jointSSP.frame_a) annotation(
              Line(points = {{-30, 2}, {-30, 20}, {-20, 20}}, color = {95, 95, 95}, thickness = 0.5));
            connect(b3.frame_b, jointSSP.frame_b) annotation(
              Line(points = {{-12, -50}, {30, -50}, {30, 20}, {20, 20}}, color = {95, 95, 95}, thickness = 0.5));
            connect(b2.frame_a, jointSSP.frame_ib) annotation(
              Line(points = {{50, 20}, {50, 48}, {16, 48}, {16, 40}}, color = {95, 95, 95}, thickness = 0.5));
            annotation(
              experiment(StopTime = 5),
              Documentation(info = "<html>
<p>
This is a third version of the \"four-bar\" mechanism, see figure:
</p>

<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Examples/Loops/Fourbar_analytic.png\" ALT=\"model Examples.Loops.Fourbar_analytic\">

<p>
In this case
the three revolute joints on the left top-side and the two revolute
joints on the right top side have been replaced by the assembly joint
<b>Joints.Assemblies.JointSSP</b>
which consists of two spherical joints and one prismatic joint.
Since JointSSP solves the non-linear constraint equation internally
analytically, no non-linear equation appears any more and a Modelica
translator, such as Dymola, can transform the system into state space
form without solving a system of equations. For more details, see
<a href=\"modelica://Modelica.Mechanics.MultiBody.UsersGuide.Tutorial.LoopStructures.AnalyticLoopHandling\">
MultiBody.UsersGuide.Tutorial.LoopStructures.AnalyticLoopHandling</a>.
</p>
</html>"));
          end Fourbar_analytic;

          model PlanarLoops_analytic "Mechanism with three planar kinematic loops and one degree-of-freedom with analytic loop handling (with JointRRR joints)"
            extends Modelica.Icons.Example;
            parameter SI.Length rh[3] = {0.5, 0, 0} "Position vector from 'lower left' revolute to 'lower right' revolute joint for all the 3 loops";
            parameter SI.Length rv[3] = {0, 0.5, 0} "Position vector from 'lower left' revolute to 'upper left' revolute joint in the first loop";
            parameter SI.Length r1b[3] = {0.1, 0.5, 0} "Position vector from 'lower right' revolute to 'upper right' revolute joint in the first loop";
            final parameter SI.Length r1a[3] = r1b + rh - rv "Position vector from 'upper left' revolute to 'upper right' revolute joint in the first loop";
            parameter SI.Length r2b[3] = {0.1, 0.6, 0} "Position vector from 'lower right' revolute to 'upper right' revolute joint in the second loop";
            final parameter SI.Length r2a[3] = r2b + rh - r1b "Position vector from 'upper left' revolute to 'upper right' revolute joint in the second loop";
            parameter SI.Length r3b[3] = {0, 0.55, 0} "Position vector from 'lower right' revolute to 'upper right' revolute joint in the third loop";
            final parameter SI.Length r3a[3] = r3b + rh - r2b "Position vector from 'upper left' revolute to 'upper right' revolute joint in the third loop";
            inner Modelica.Mechanics.MultiBody.World world annotation(
              Placement(transformation(extent = {{-94, -90}, {-74, -70}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Joints.Assemblies.JointRRR jointRRR1(rRod1_ia = r1a, rRod2_ib = r1b, checkTotalPower = true) annotation(
              Placement(transformation(origin = {-20, 0}, extent = {{-20, -20}, {20, 20}}, rotation = 270)));
            Modelica.Mechanics.MultiBody.Joints.Revolute rev(useAxisFlange = true, w(fixed = true)) annotation(
              Placement(transformation(origin = {-56, -10}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
            Modelica.Mechanics.MultiBody.Parts.FixedTranslation rod1(r = rv) annotation(
              Placement(transformation(origin = {-56, 30}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
            Modelica.Mechanics.MultiBody.Parts.FixedTranslation rod2(r = rh) annotation(
              Placement(transformation(extent = {{-50, -60}, {-30, -40}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Parts.Body body1(m = 1, cylinderColor = {155, 155, 155}, r_CM = jointRRR1.rRod1_ia / 2) annotation(
              Placement(transformation(origin = {6, 60}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
            Modelica.Mechanics.Rotational.Sources.Position position(useSupport = true) annotation(
              Placement(transformation(extent = {{-90, -20}, {-70, 0}}, rotation = 0)));
            Modelica.Blocks.Sources.Sine sine(amplitude = 0.7, freqHz = 1) annotation(
              Placement(transformation(origin = {-96, 30}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
            Modelica.Mechanics.MultiBody.Joints.Assemblies.JointRRR jointRRR2(rRod1_ia = r2a, rRod2_ib = r2b, checkTotalPower = true) annotation(
              Placement(transformation(origin = {30, 0}, extent = {{-20, -20}, {20, 20}}, rotation = 270)));
            Modelica.Mechanics.MultiBody.Parts.FixedTranslation rod3(r = rh) annotation(
              Placement(transformation(extent = {{0, -60}, {20, -40}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Parts.Body body2(m = 1, cylinderColor = {155, 155, 155}, r_CM = jointRRR2.rRod1_ia / 2) annotation(
              Placement(transformation(origin = {55, 61}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
            Modelica.Mechanics.MultiBody.Joints.Assemblies.JointRRR jointRRR3(rRod1_ia = r3a, rRod2_ib = r3b, checkTotalPower = true) annotation(
              Placement(transformation(origin = {80, 0}, extent = {{-20, -20}, {20, 20}}, rotation = 270)));
            Modelica.Mechanics.MultiBody.Parts.FixedTranslation rod4(r = rh) annotation(
              Placement(transformation(extent = {{40, -60}, {60, -40}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Parts.Body body3(m = 1, cylinderColor = {155, 155, 155}, r_CM = jointRRR3.rRod1_ia / 2) annotation(
              Placement(transformation(origin = {108, 62}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
            Parts.Mounting1D mounting1D annotation(
              Placement(transformation(extent = {{-100, -50}, {-80, -30}})));
          equation
            connect(world.frame_b, rev.frame_a) annotation(
              Line(points = {{-74, -80}, {-56, -80}, {-56, -20}}, color = {95, 95, 95}, thickness = 0.5));
            connect(rod1.frame_a, rev.frame_b) annotation(
              Line(points = {{-56, 20}, {-56, 0}}, color = {0, 0, 0}, thickness = 0.5));
            connect(rod1.frame_b, jointRRR1.frame_a) annotation(
              Line(points = {{-56, 40}, {-56, 50}, {-20, 50}, {-20, 20}}, color = {95, 95, 95}, thickness = 0.5));
            connect(rod2.frame_a, world.frame_b) annotation(
              Line(points = {{-50, -50}, {-56, -50}, {-56, -80}, {-74, -80}}, color = {95, 95, 95}, thickness = 0.5));
            connect(rod2.frame_b, jointRRR1.frame_b) annotation(
              Line(points = {{-30, -50}, {-20, -50}, {-20, -20}}, color = {95, 95, 95}, thickness = 0.5));
            connect(jointRRR1.frame_ia, body1.frame_a) annotation(
              Line(points = {{0, 16}, {6, 16}, {6, 50}}, color = {95, 95, 95}, thickness = 0.5));
            connect(rod3.frame_a, rod2.frame_b) annotation(
              Line(points = {{0, -50}, {-30, -50}}, color = {95, 95, 95}, thickness = 0.5));
            connect(rod3.frame_b, jointRRR2.frame_b) annotation(
              Line(points = {{20, -50}, {30, -50}, {30, -20}}, color = {95, 95, 95}, thickness = 0.5));
            connect(jointRRR2.frame_ia, body2.frame_a) annotation(
              Line(points = {{50, 16}, {55, 16}, {55, 51}}, color = {95, 95, 95}, thickness = 0.5));
            connect(jointRRR1.frame_im, jointRRR2.frame_a) annotation(
              Line(points = {{0, 0}, {12, 0}, {12, 25}, {30, 25}, {30, 20}}, color = {95, 95, 95}, thickness = 0.5));
            connect(rod3.frame_b, rod4.frame_a) annotation(
              Line(points = {{20, -50}, {40, -50}}, color = {95, 95, 95}, thickness = 0.5));
            connect(rod4.frame_b, jointRRR3.frame_b) annotation(
              Line(points = {{60, -50}, {80, -50}, {80, -20}}, color = {95, 95, 95}, thickness = 0.5));
            connect(jointRRR2.frame_im, jointRRR3.frame_a) annotation(
              Line(points = {{50, 0}, {60, 0}, {60, 26}, {80, 26}, {80, 20}}, color = {95, 95, 95}, thickness = 0.5));
            connect(jointRRR3.frame_ia, body3.frame_a) annotation(
              Line(points = {{100, 16}, {108, 16}, {108, 52}}, color = {95, 95, 95}, thickness = 0.5));
            connect(sine.y, position.phi_ref) annotation(
              Line(points = {{-96, 19}, {-96, -10}, {-92, -10}}, color = {0, 0, 127}));
            connect(mounting1D.flange_b, position.support) annotation(
              Line(points = {{-80, -40}, {-80, -20}}, color = {0, 0, 0}, smooth = Smooth.None));
            connect(mounting1D.frame_a, world.frame_b) annotation(
              Line(points = {{-90, -50}, {-90, -57}, {-70, -57}, {-70, -80}, {-74, -80}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            connect(position.flange, rev.axis) annotation(
              Line(points = {{-70, -10}, {-66, -10}}, color = {0, 0, 0}, smooth = Smooth.None));
            annotation(
              experiment(StopTime = 5),
              Documentation(info = "<html>
<p>
It is demonstrated how the Modelica.Mechanics.MultiBody.Joints.Assemblies.JointRRR joint can be
used to solve the non-linear equations of coupled planar loops analytically.
In the mechanism below no non-linear equation occurs any more from the tool
view, since these equations are solved analytically in the JointRRR joints.
For more details, see
<a href=\"modelica://Modelica.Mechanics.MultiBody.UsersGuide.Tutorial.LoopStructures.AnalyticLoopHandling\">
MultiBody.UsersGuide.Tutorial.LoopStructures.AnalyticLoopHandling</a>.
</p>

<p>
In the following figure the parameter vectors of this example are visualized in the
animation view.
</p>

<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Examples/Loops/PlanarLoops2.png\" ALT=\"model Examples.Loops.PlanarLoops2\">
</html>"));
          end PlanarLoops_analytic;

          package Utilities "Utility models for Examples.Loops"
            extends Modelica.Icons.UtilitiesPackage;

            model Cylinder "Cylinder with rod and crank of a combustion engine"
              parameter Boolean animation = true "= true, if animation shall be enabled";
              parameter SI.Length cylinderTopPosition = 0.42 "Length from crank shaft to end of cylinder.";
              parameter SI.Length pistonLength = 0.1 "Length of cylinder";
              parameter SI.Length rodLength = 0.2 "Length of rod";
              parameter SI.Length crankLength = 0.2 "Length of crank shaft in x direction";
              parameter SI.Length crankPinOffset = 0.1 "Offset of crank pin from center axis";
              parameter SI.Length crankPinLength = 0.1 "Offset of crank pin from center axis";
              parameter SI.Angle cylinderInclination = 0 "Inclination of cylinder";
              parameter SI.Angle crankAngleOffset = 0 "Offset for crank angle";
              parameter SI.Length cylinderLength = cylinderTopPosition - (pistonLength + rodLength - crankPinOffset) "Maximum length of cylinder volume";
              Modelica.Mechanics.MultiBody.Parts.BodyCylinder Piston(diameter = 0.1, r = {0, pistonLength, 0}, color = {180, 180, 180}, animation = animation) annotation(
                Placement(transformation(origin = {14.5, 69.5}, extent = {{10.5, -30.5}, {-10.5, 30.5}}, rotation = 270)));
              Modelica.Mechanics.MultiBody.Parts.BodyBox Rod(widthDirection = {1, 0, 0}, height = 0.06, color = {0, 0, 200}, width = 0.02, r_shape = {0, -0.02, 0}, r = {0, rodLength, 0}, animation = animation) annotation(
                Placement(transformation(origin = {14, 8}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
              Modelica.Mechanics.MultiBody.Joints.Revolute B2(n = {1, 0, 0}, cylinderLength = 0.02, animation = animation, cylinderDiameter = 0.055) annotation(
                Placement(transformation(extent = {{4, 25}, {24, 45}}, rotation = 0)));
              Modelica.Mechanics.MultiBody.Parts.BodyBox Crank4(height = 0.05, widthDirection = {1, 0, 0}, width = 0.02, r = {0, -crankPinOffset, 0}, animation = animation) annotation(
                Placement(transformation(origin = {40.5, -74}, extent = {{10, -10}, {-10, 10}}, rotation = 90)));
              Modelica.Mechanics.MultiBody.Parts.BodyCylinder Crank3(r_shape = {-0.01, 0, 0}, length = 0.12, diameter = 0.03, r = {crankPinLength, 0, 0}, color = {180, 180, 180}, animation = animation) annotation(
                Placement(transformation(extent = {{4.5, -60}, {24.5, -40}}, rotation = 0)));
              Modelica.Mechanics.MultiBody.Parts.BodyCylinder Crank1(diameter = 0.05, r_shape = {-0.01, 0, 0}, length = 0.12, r = {crankLength - crankPinLength, 0, 0}, color = {180, 180, 180}, animation = animation) annotation(
                Placement(transformation(extent = {{-50, -100}, {-30, -80}}, rotation = 0)));
              Modelica.Mechanics.MultiBody.Parts.BodyBox Crank2(height = 0.05, widthDirection = {1, 0, 0}, width = 0.02, r = {0, crankPinOffset, 0}, animation = animation) annotation(
                Placement(transformation(origin = {-10, -76}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
              Joints.RevolutePlanarLoopConstraint B1(n = {1, 0, 0}, cylinderLength = 0.02, animation = animation, cylinderDiameter = 0.055) annotation(
                Placement(transformation(extent = {{4, -27}, {24, -7}}, rotation = 0)));
              Modelica.Mechanics.MultiBody.Parts.FixedTranslation Mid(r = {crankPinLength / 2, 0, 0}, animation = false) annotation(
                Placement(transformation(extent = {{-6, -46}, {14, -26}}, rotation = 0)));
              Modelica.Mechanics.MultiBody.Joints.Prismatic Cylinder(useAxisFlange = true, s(start = -0.3), n = {0, -1, 0}, boxWidth = 0.02) annotation(
                Placement(transformation(origin = {14, 99}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
              Modelica.Mechanics.MultiBody.Parts.FixedTranslation Mounting(r = {crankLength, 0, 0}, animation = false) annotation(
                Placement(transformation(extent = {{0, 120}, {20, 140}}, rotation = 0)));
              Modelica.Mechanics.MultiBody.Parts.FixedRotation CylinderInclination(r = {crankLength - crankPinLength / 2, 0, 0}, n_y = {0, cos(cylinderInclination), sin(cylinderInclination)}, animation = false, rotationType = Modelica.Mechanics.MultiBody.Types.RotationTypes.TwoAxesVectors) annotation(
                Placement(transformation(extent = {{-60, 30}, {-40, 50}}, rotation = 0)));
              Modelica.Mechanics.MultiBody.Parts.FixedRotation CrankAngle1(n_y = {0, cos(crankAngleOffset), sin(crankAngleOffset)}, animation = false, rotationType = Modelica.Mechanics.MultiBody.Types.RotationTypes.TwoAxesVectors) annotation(
                Placement(transformation(extent = {{-90, -100}, {-70, -80}}, rotation = 0)));
              Modelica.Mechanics.MultiBody.Parts.FixedRotation CrankAngle2(n_y = {0, cos(-crankAngleOffset), sin(-crankAngleOffset)}, animation = false, rotationType = Modelica.Mechanics.MultiBody.Types.RotationTypes.TwoAxesVectors) annotation(
                Placement(transformation(extent = {{60, -100}, {80, -80}}, rotation = 0)));
              Modelica.Mechanics.MultiBody.Parts.FixedTranslation CylinderTop(r = {0, cylinderTopPosition, 0}, animation = false) annotation(
                Placement(transformation(origin = {-30, 71}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
              GasForce gasForce(L = cylinderLength, d = 0.1) annotation(
                Placement(transformation(origin = {50, 107}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
              Interfaces.Frame_a cylinder_a annotation(
                Placement(transformation(extent = {{-116, 114}, {-84, 146}}, rotation = 0)));
              Interfaces.Frame_a cylinder_b annotation(
                Placement(transformation(extent = {{84, 114}, {116, 146}}, rotation = 0)));
              Interfaces.Frame_a crank_a annotation(
                Placement(transformation(extent = {{-116, -106}, {-84, -74}}, rotation = 0)));
              Interfaces.Frame_a crank_b annotation(
                Placement(transformation(extent = {{84, -106}, {116, -74}}, rotation = 0)));
            equation
              connect(B1.frame_a, Mid.frame_b) annotation(
                Line(points = {{4, -17}, {-6, -17}, {-6, -29}, {22, -29}, {22, -36}, {14, -36}}, color = {95, 95, 95}, thickness = 0.5));
              connect(Rod.frame_a, B1.frame_b) annotation(
                Line(points = {{14, -2}, {14, -9}, {30, -9}, {30, -17}, {24, -17}}, color = {95, 95, 95}, thickness = 0.5));
              connect(Cylinder.frame_b, Piston.frame_b) annotation(
                Line(points = {{14, 89}, {14, 80}, {14.5, 80}}, color = {95, 95, 95}, thickness = 0.5));
              connect(Crank1.frame_a, CrankAngle1.frame_b) annotation(
                Line(points = {{-50, -90}, {-70, -90}}, color = {95, 95, 95}, thickness = 0.5));
              connect(B2.frame_a, Piston.frame_a) annotation(
                Line(points = {{4, 35}, {-6, 35}, {-6, 49}, {14.5, 49}, {14.5, 59}}, color = {95, 95, 95}, thickness = 0.5));
              connect(Rod.frame_b, B2.frame_b) annotation(
                Line(points = {{14, 18}, {14, 23}, {32, 23}, {32, 35}, {24, 35}}, color = {95, 95, 95}, thickness = 0.5));
              connect(Crank4.frame_b, CrankAngle2.frame_a) annotation(
                Line(points = {{40.5, -84}, {40.5, -90}, {60, -90}}, color = {95, 95, 95}, thickness = 0.5));
              connect(Cylinder.support, gasForce.flange_b) annotation(
                Line(points = {{20, 103}, {34, 103}, {34, 117}, {50, 117}}, color = {0, 191, 0}));
              connect(Cylinder.axis, gasForce.flange_a) annotation(
                Line(points = {{20, 91}, {50, 91}, {50, 97}}, color = {0, 191, 0}));
              connect(CylinderInclination.frame_b, CylinderTop.frame_a) annotation(
                Line(points = {{-40, 40}, {-30, 40}, {-30, 61}}));
              connect(Crank1.frame_b, Crank2.frame_a) annotation(
                Line(points = {{-30, -90}, {-10, -90}, {-10, -86}}, color = {95, 95, 95}, thickness = 0.5));
              connect(Crank3.frame_b, Crank4.frame_a) annotation(
                Line(points = {{24.5, -50}, {40.5, -50}, {40.5, -64}}, color = {95, 95, 95}, thickness = 0.5));
              connect(Crank3.frame_a, Crank2.frame_b) annotation(
                Line(points = {{4.5, -50}, {-10, -50}, {-10, -66}}, color = {95, 95, 95}, thickness = 0.5));
              connect(Crank2.frame_b, Mid.frame_a) annotation(
                Line(points = {{-10, -66}, {-10, -36}, {-6, -36}}, color = {95, 95, 95}, thickness = 0.5));
              connect(CylinderTop.frame_b, Cylinder.frame_a) annotation(
                Line(points = {{-30, 81}, {-30, 120}, {14, 120}, {14, 109}}, color = {95, 95, 95}, thickness = 0.5));
              connect(CylinderInclination.frame_a, cylinder_a) annotation(
                Line(points = {{-60, 40}, {-80, 40}, {-80, 130}, {-100, 130}}, color = {95, 95, 95}, thickness = 0.5));
              connect(Mounting.frame_a, cylinder_a) annotation(
                Line(points = {{0, 130}, {-100, 130}}, color = {95, 95, 95}, thickness = 0.5));
              connect(Mounting.frame_b, cylinder_b) annotation(
                Line(points = {{20, 130}, {100, 130}}, color = {95, 95, 95}, thickness = 0.5));
              connect(CrankAngle1.frame_a, crank_a) annotation(
                Line(points = {{-90, -90}, {-100, -90}}, color = {95, 95, 95}, thickness = 0.5));
              connect(CrankAngle2.frame_b, crank_b) annotation(
                Line(points = {{80, -90}, {100, -90}}, color = {95, 95, 95}, thickness = 0.5));
              annotation(
                Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -150}, {100, 150}}), graphics = {Polygon(points = {{-60, -50}, {-60, 100}, {60, 100}, {60, -52}, {100, -52}, {100, 150}, {-100, 150}, {-100, -50}, {-60, -50}}, lineColor = {0, 0, 0}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-58, 89}, {58, 13}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.VerticalCylinder, fillColor = {192, 192, 192}), Rectangle(extent = {{-60, 81}, {60, 75}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-60, 67}, {60, 61}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-60, 55}, {60, 49}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Polygon(points = {{-60, 11}, {-42, 23}, {38, 23}, {56, 11}, {-60, 11}}, pattern = LinePattern.None, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Ellipse(extent = {{-6, 41}, {2, 33}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-40, -129}, {40, -49}}, lineColor = {192, 192, 192}), Line(points = {{0, -90}, {26, -58}, {-2, 37}}, color = {0, 0, 0}, thickness = 1), Text(extent = {{-150, -125}, {150, -165}}, textString = "%name", lineColor = {0, 0, 255}), Line(points = {{-100, -90}, {100, -91}}, color = {0, 0, 0}, thickness = 0.5)}));
            end Cylinder;

            model GasForce "Simple gas force computation for combustion engine"
              extends Modelica.Mechanics.Translational.Interfaces.PartialCompliant;
              parameter SI.Length L "Length of cylinder";
              parameter SI.Diameter d "Diameter of cylinder";
              parameter SIunits.Volume k0 = 0.01 "Volume V = k0 + k1*(1-x), with x = 1 + s_rel/L";
              parameter SIunits.Volume k1 = 1 "Volume V = k0 + k1*(1-x), with x = 1 + s_rel/L";
              parameter SIunits.HeatCapacity k = 1 "Gas constant (p*V = k*T)";
              constant Real pi = Modelica.Constants.pi;
              // Only for compatibility reasons
              Real x "Normalized position of cylinder";
              Real y "Normalized relative movement (= -s_rel/L)";
              SI.Density dens;
              Modelica.SIunits.Conversions.NonSIunits.Pressure_bar press "cylinder pressure";
              SI.Volume V;
              SI.Temperature T;
              SI.Velocity v_rel;
            protected
              constant SI.Mass unitMass = 1;
              Modelica.SIunits.Pressure p;
            equation
              y = -s_rel / L;
              x = 1 + s_rel / L;
              v_rel = der(s_rel);
              press = p / 1e5;
              p = (if v_rel < 0 then if x < 0.987 then 177.4132 * x ^ 4 - 287.2189 * x ^ 3 + 151.8252 * x ^ 2 - 24.9973 * x + 2.4 else 2836360 * x ^ 4 - 10569296 * x ^ 3 + 14761814 * x ^ 2 - 9158505 * x + 2129670 else if x > 0.93 then (-3929704 * x ^ 4) + 14748765 * x ^ 3 - 20747000 * x ^ 2 + 12964477 * x - 3036495 else 145.930 * x ^ 4 - 131.707 * x ^ 3 + 17.3438 * x ^ 2 + 17.9272 * x + 2.4) * 1e5;
              f = -1.0E5 * press * pi * d ^ 2 / 4;
              V = k0 + k1 * (1 - x);
              dens = unitMass / V;
              p / 1e5 * V = k * T;
              annotation(
                Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-90, 50}, {90, -50}}, lineColor = {0, 0, 0}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-40, 50}, {-20, -50}}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Rectangle(extent = {{-90, 10}, {-40, -10}}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Polygon(points = {{60, 2}, {54, 2}, {0, 2}, {0, 10}, {-20, 0}, {0, -10}, {0, -4}, {60, -4}, {60, 2}}, lineColor = {255, 0, 0}, fillColor = {255, 0, 0}, fillPattern = FillPattern.Solid), Text(extent = {{-100, 120}, {100, 60}}, textString = "%name", lineColor = {0, 0, 255})}),
                Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-90, 50}, {90, -50}}, lineColor = {0, 0, 0}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-40, 50}, {-20, -50}}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Rectangle(extent = {{-90, 10}, {-40, -10}}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Polygon(points = {{60, 2}, {54, 2}, {0, 2}, {0, 10}, {-20, 0}, {0, -10}, {0, -4}, {60, -4}, {60, 2}}, lineColor = {255, 0, 0}, fillColor = {255, 0, 0}, fillPattern = FillPattern.Solid), Text(extent = {{-100, 120}, {100, 60}}, textString = "%name", lineColor = {0, 0, 255})}));
            end GasForce;

            model GasForce2 "Rough approximation of gas force in a cylinder"
              extends Modelica.Mechanics.Translational.Interfaces.PartialCompliant;
              parameter SI.Length L "Length of cylinder";
              parameter SI.Length d "diameter of cylinder";
              parameter SIunits.Volume k0 = 0.01 "Volume V = k0 + k1*(1-x), with x = 1 - s_rel/L";
              parameter SIunits.Volume k1 = 1 "Volume V = k0 + k1*(1-x), with x = 1 - s_rel/L";
              parameter SIunits.HeatCapacity k = 1 "Gas constant (p*V = k*T)";
              /*
                parameter Real k0=0.01;
                parameter Real k1=1;
                parameter Real k=1;
              */
              constant Real pi = Modelica.Constants.pi;
              Real x "Normalized position of cylinder (= 1 - s_rel/L)";
              SI.Density dens;
              Modelica.SIunits.AbsolutePressure press "Cylinder pressure";
              SI.Volume V;
              SI.Temperature T;
              SI.Velocity v_rel;
            protected
              Modelica.SIunits.SpecificHeatCapacity R_air = Modelica.Constants.R / 0.0289651159;
            equation
              x = 1 - s_rel / L;
              v_rel = der(s_rel);
              press = 1.0E5 * (if v_rel < 0 then if x < 0.987 then 177.4132 * x ^ 4 - 287.2189 * x ^ 3 + 151.8252 * x ^ 2 - 24.9973 * x + 2.4 else 2836360 * x ^ 4 - 10569296 * x ^ 3 + 14761814 * x ^ 2 - 9158505 * x + 2129670 else if x > 0.93 then (-3929704 * x ^ 4) + 14748765 * x ^ 3 - 20747000 * x ^ 2 + 12964477 * x - 3036495 else 145.930 * x ^ 4 - 131.707 * x ^ 3 + 17.3438 * x ^ 2 + 17.9272 * x + 2.4);
              f = -press * pi * d ^ 2 / 4;
              V = k0 + k1 * (1 - x);
              dens = press / (R_air * T);
              press * V = k * T;
              assert(s_rel >= (-1.e-12), "flange_b.s - flange_a.s (= " + String(s_rel, significantDigits = 14) + ") >= 0 required for GasForce component.\n" + "Most likely, the component has to be flipped.");
              assert(s_rel <= L + 1.e-12, " flange_b.s - flange_a.s (= " + String(s_rel, significantDigits = 14) + ") <= L (= " + String(L, significantDigits = 14) + ") required for GasForce component.\n" + "Most likely, parameter L is not correct.");
              annotation(
                Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-90, 50}, {90, -50}}, lineColor = {0, 0, 0}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Polygon(points = {{-13, 4}, {-16, 4}, {-65, 4}, {-65, 15}, {-90, 0}, {-65, -15}, {-65, -4}, {-13, -4}, {-13, 4}}, lineColor = {255, 0, 0}, fillColor = {255, 0, 0}, fillPattern = FillPattern.Solid), Text(extent = {{-135, 44}, {-99, 19}}, lineColor = {128, 128, 128}, textString = "a"), Text(extent = {{97, 40}, {133, 15}}, lineColor = {128, 128, 128}, textString = "b"), Polygon(points = {{12, 4}, {70, 4}, {65, 4}, {65, 15}, {90, 0}, {65, -15}, {65, -4}, {12, -4}, {12, 4}}, lineColor = {255, 0, 0}, fillColor = {255, 0, 0}, fillPattern = FillPattern.Solid), Text(extent = {{-150, 105}, {150, 65}}, textString = "%name", lineColor = {0, 0, 255})}),
                Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-90, 50}, {90, -50}}, lineColor = {0, 0, 0}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Polygon(points = {{12, 5}, {70, 5}, {65, 5}, {65, 16}, {90, 1}, {65, -14}, {65, -3}, {12, -3}, {12, 5}}, lineColor = {255, 0, 0}, fillColor = {255, 0, 0}, fillPattern = FillPattern.Solid), Polygon(points = {{-13, 5}, {-16, 5}, {-65, 5}, {-65, 16}, {-90, 1}, {-65, -14}, {-65, -3}, {-13, -3}, {-13, 5}}, lineColor = {255, 0, 0}, fillColor = {255, 0, 0}, fillPattern = FillPattern.Solid)}),
                Documentation(info = "<html>
<p>
The gas force in a cylinder is computed as function of the relative
distance of the two flanges. It is required that s_rel = flange_b.s - flange_a.s
is in the range
</p>
<pre>
    0 &le; s_rel &le; L
</pre>
<p>
where the parameter L is the length
of the cylinder. If this assumption is not fulfilled, an error occurs.
</p>
</html>"));
            end GasForce2;

            model CylinderBase "One cylinder with analytic handling of kinematic loop"
              parameter Boolean animation = true "= true, if animation shall be enabled";
              parameter SI.Length cylinderTopPosition = 0.42 "Length from crank shaft to end of cylinder.";
              parameter SI.Length crankLength = 0.14 "Length of crank shaft in x direction";
              parameter SI.Length crankPinOffset = 0.05 "Offset of crank pin from center axis";
              parameter SI.Length crankPinLength = 0.1 "Offset of crank pin from center axis";
              parameter Cv.NonSIunits.Angle_deg cylinderInclination = 0 "Inclination of cylinder";
              parameter Cv.NonSIunits.Angle_deg crankAngleOffset = 0 "Offset for crank angle";
              parameter SI.Length pistonLength = 0.1 "Length of cylinder" annotation(
                Dialog(group = "Piston"));
              parameter SI.Length pistonCenterOfMass = pistonLength / 2 "Distance from frame_a to center of mass of piston" annotation(
                Dialog(group = "Piston"));
              parameter SI.Mass pistonMass(min = 0) = 6 "Mass of piston" annotation(
                Dialog(group = "Piston"));
              parameter SI.Inertia pistonInertia_11(min = 0) = 0.0088 "Inertia 11 of piston with respect to center of mass frame, parallel to frame_a" annotation(
                Dialog(group = "Piston"));
              parameter SI.Inertia pistonInertia_22(min = 0) = 0.0076 "Inertia 22 of piston with respect to center of mass frame, parallel to frame_a" annotation(
                Dialog(group = "Piston"));
              parameter SI.Inertia pistonInertia_33(min = 0) = 0.0088 "Inertia 33 of piston with respect to center of mass frame, parallel to frame_a" annotation(
                Dialog(group = "Piston"));
              parameter SI.Length rodLength = 0.175 "Length of rod" annotation(
                Dialog(group = "Rod"));
              parameter SI.Length rodCenterOfMass = rodLength / 2 "Distance from frame_a to center of mass of piston" annotation(
                Dialog(group = "Rod"));
              parameter SI.Mass rodMass(min = 0) = 1 "Mass of rod" annotation(
                Dialog(group = "Rod"));
              parameter SI.Inertia rodInertia_11(min = 0) = 0.006 "Inertia 11 of rod with respect to center of mass frame, parallel to frame_a" annotation(
                Dialog(group = "Rod"));
              parameter SI.Inertia rodInertia_22(min = 0) = 0.0005 "Inertia 22 of rod with respect to center of mass frame, parallel to frame_a" annotation(
                Dialog(group = "Rod"));
              parameter SI.Inertia rodInertia_33(min = 0) = 0.006 "Inertia 33 of rod with respect to center of mass frame, parallel to frame_a" annotation(
                Dialog(group = "Rod"));
              final parameter SI.Length cylinderLength = cylinderTopPosition - (pistonLength + rodLength - crankPinOffset) "Maximum length of cylinder volume";
              Modelica.Mechanics.MultiBody.Parts.FixedTranslation Mid(animation = false, r = {crankLength - crankPinLength / 2, crankPinOffset, 0}) annotation(
                Placement(transformation(extent = {{-44, -30}, {-24, -10}}, rotation = 0)));
              Modelica.Mechanics.MultiBody.Parts.FixedTranslation Mounting(r = {crankLength, 0, 0}, animation = false) annotation(
                Placement(transformation(extent = {{-3, 90}, {17, 110}}, rotation = 0)));
              Modelica.Mechanics.MultiBody.Parts.FixedRotation CylinderInclination(r = {crankLength - crankPinLength / 2, 0, 0}, animation = false, rotationType = Modelica.Mechanics.MultiBody.Types.RotationTypes.RotationAxis, n = {1, 0, 0}, angle = cylinderInclination) annotation(
                Placement(transformation(extent = {{-44, 30}, {-24, 50}}, rotation = 0)));
              Modelica.Mechanics.MultiBody.Parts.FixedRotation CrankAngle(animation = false, rotationType = Modelica.Mechanics.MultiBody.Types.RotationTypes.RotationAxis, n = {1, 0, 0}, angle = crankAngleOffset) annotation(
                Placement(transformation(extent = {{-84, -80}, {-64, -60}}, rotation = 0)));
              Joints.Assemblies.JointRRP jointRRP(n_a = {1, 0, 0}, n_b = {0, -1, 0}, rRod1_ia = {0, rodLength, 0}, animation = false, rRod2_ib = -{0, pistonLength, 0}, s_offset = -cylinderTopPosition) annotation(
                Placement(transformation(origin = {0, 12}, extent = {{-20, 20}, {20, -20}}, rotation = 90)));
              Modelica.Mechanics.MultiBody.Parts.BodyShape Rod(animation = animation, r = {0, rodLength, 0}, r_CM = {0, rodLength / 2, 0}, shapeType = "modelica://Modelica/Resources/Data/Shapes/Engine/rod.dxf", lengthDirection = {1, 0, 0}, widthDirection = {0, 0, -1}, length = rodLength / 1.75, width = rodLength / 1.75, height = rodLength / 1.75, color = {155, 155, 155}, extra = 1, r_shape = {0, 0, 0}, animateSphere = false, m = rodMass, I_11 = rodInertia_11, I_22 = rodInertia_22, I_33 = rodInertia_33) annotation(
                Placement(transformation(origin = {49, 9}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
              Modelica.Mechanics.MultiBody.Parts.BodyShape Piston(animation = animation, r = {0, pistonLength, 0}, r_CM = {0, pistonLength / 2, 0}, shapeType = "modelica://Modelica/Resources/Data/Shapes/Engine/piston.dxf", length = 0.08, width = 0.08, height = 0.08, extra = 1, lengthDirection = {1, 0, 0}, widthDirection = {0, 0, -1}, color = {180, 180, 180}, animateSphere = false, m = pistonMass, I_11 = pistonInertia_11, I_22 = pistonInertia_22, I_33 = pistonInertia_33) annotation(
                Placement(transformation(origin = {50, 50}, extent = {{10, -10}, {-10, 10}}, rotation = 270)));
              GasForce gasForce(L = cylinderLength, d = 0.1) annotation(
                Placement(transformation(origin = {-1, 70}, extent = {{-10, -10}, {10, 10}}, rotation = 180)));
              Modelica.Mechanics.MultiBody.Parts.FixedTranslation Crank(animation = false, r = {crankLength, 0, 0}) annotation(
                Placement(transformation(extent = {{-10, -110}, {10, -90}}, rotation = 0)));
              Interfaces.Frame_a cylinder_a annotation(
                Placement(transformation(extent = {{-116, 84}, {-84, 116}}, rotation = 0)));
              Interfaces.Frame_a cylinder_b annotation(
                Placement(transformation(extent = {{84, 84}, {116, 116}}, rotation = 0)));
              Interfaces.Frame_a crank_a annotation(
                Placement(transformation(extent = {{-116, -116}, {-84, -84}}, rotation = 0)));
              Interfaces.Frame_a crank_b annotation(
                Placement(transformation(extent = {{84, -116}, {116, -84}}, rotation = 0)));
            equation
              connect(jointRRP.frame_ia, Rod.frame_a) annotation(
                Line(points = {{20, -4}, {49, -4}, {49, -1}}, color = {95, 95, 95}, thickness = 0.5));
              connect(Mid.frame_b, jointRRP.frame_a) annotation(
                Line(points = {{-24, -20}, {0, -20}, {0, -8}}, color = {95, 95, 95}, thickness = 0.5));
              connect(gasForce.flange_a, jointRRP.axis) annotation(
                Line(points = {{9, 70}, {16, 70}, {16, 32}}, color = {0, 191, 0}));
              connect(jointRRP.bearing, gasForce.flange_b) annotation(
                Line(points = {{8, 32}, {8, 52}, {-20, 52}, {-20, 70}, {-11, 70}}, color = {0, 191, 0}));
              connect(jointRRP.frame_ib, Piston.frame_b) annotation(
                Line(points = {{20, 28}, {30, 28}, {30, 70}, {50, 70}, {50, 60}}, color = {95, 95, 95}, thickness = 0.5));
              connect(jointRRP.frame_b, CylinderInclination.frame_b) annotation(
                Line(points = {{0, 32}, {1, 32}, {1, 40}, {-24, 40}}, color = {95, 95, 95}, thickness = 0.5));
              connect(CrankAngle.frame_b, Mid.frame_a) annotation(
                Line(points = {{-64, -70}, {-56, -70}, {-56, -20}, {-44, -20}}, color = {95, 95, 95}, thickness = 0.5));
              connect(cylinder_a, CylinderInclination.frame_a) annotation(
                Line(points = {{-100, 100}, {-70, 100}, {-70, 40}, {-44, 40}}, color = {95, 95, 95}, thickness = 0.5));
              connect(cylinder_a, Mounting.frame_a) annotation(
                Line(points = {{-100, 100}, {-3, 100}}, color = {95, 95, 95}, thickness = 0.5));
              connect(cylinder_b, Mounting.frame_b) annotation(
                Line(points = {{100, 100}, {17, 100}}, color = {95, 95, 95}, thickness = 0.5));
              connect(CrankAngle.frame_a, crank_a) annotation(
                Line(points = {{-84, -70}, {-89.5, -70}, {-89.5, -100}, {-100, -100}}, color = {95, 95, 95}, thickness = 0.5));
              connect(crank_a, Crank.frame_a) annotation(
                Line(points = {{-100, -100}, {-10, -100}}, color = {95, 95, 95}, thickness = 0.5));
              connect(Crank.frame_b, crank_b) annotation(
                Line(points = {{10, -100}, {100, -100}}, color = {95, 95, 95}, thickness = 0.5));
              annotation(
                Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{-60, -61}, {-60, 64}, {60, 64}, {60, -61}, {100, -61}, {100, 114}, {-100, 114}, {-100, -61}, {-60, -61}}, lineColor = {0, 0, 0}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-58, 63}, {58, -13}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.VerticalCylinder, fillColor = {192, 192, 192}), Rectangle(extent = {{-60, 55}, {60, 49}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-60, 41}, {60, 35}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-60, 29}, {60, 23}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Polygon(points = {{-57, -13}, {-39, -1}, {41, -1}, {59, -13}, {-57, -13}}, pattern = LinePattern.None, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Ellipse(extent = {{-6, 15}, {2, 7}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-41, -139}, {39, -59}}, lineColor = {192, 192, 192}), Line(points = {{-1, -99}, {25, -67}, {-2, 10}}, color = {0, 0, 0}, thickness = 1), Text(extent = {{-156, 178}, {158, 116}}, textString = "%name", lineColor = {0, 0, 255}), Line(points = {{-100, -99}, {100, -100}}, color = {0, 0, 0}, thickness = 0.5)}));
            end CylinderBase;

            model Cylinder_analytic_CAD "One cylinder with analytic handling of kinematic loop and CAD visualization"
              extends CylinderBase;
              Visualizers.FixedShape CrankShape(animation = animation, shapeType = "modelica://Modelica/Resources/Data/Shapes/Engine/crank.dxf", lengthDirection = {1, 0, 0}, extra = 1, widthDirection = {0, 1, 0}, length = crankPinOffset / 0.5, width = crankPinOffset / 0.5, height = crankPinOffset / 0.5, r_shape = {crankLength - crankPinLength / 2 - 0.002, 0, 0}) annotation(
                Placement(transformation(origin = {-10, -70}, extent = {{10, 10}, {-10, -10}}, rotation = 180)));
            equation
              connect(CrankShape.frame_a, CrankAngle.frame_b) annotation(
                Line(points = {{-20, -70}, {-64, -70}}, color = {95, 95, 95}, thickness = 0.5));
            end Cylinder_analytic_CAD;

            model EngineV6_analytic "V6 engine with analytic loop handling"
              parameter Boolean animation = true "= true, if animation shall be enabled";
              replaceable model Cylinder = Cylinder_analytic_CAD constrainedby CylinderBase "Cylinder type" annotation(
                 choices(choice(redeclare model Cylinder = Modelica.Mechanics.MultiBody.Examples.Loops.Utilities.Cylinder_analytic_CAD "Analytic loop handling + CAD animation"), choice(redeclare model Cylinder = Modelica.Mechanics.MultiBody.Examples.Loops.Utilities.Cylinder_analytic "Analytic loop handling + standard animation")));
              Cylinder cylinder1(crankAngleOffset = -30, cylinderInclination = -30, animation = animation) annotation(
                Placement(transformation(extent = {{-90, -10}, {-70, 10}}, rotation = 0)));
              Cylinder cylinder2(crankAngleOffset = 90, cylinderInclination = 30, animation = animation) annotation(
                Placement(transformation(extent = {{-60, -10}, {-40, 10}}, rotation = 0)));
              Cylinder cylinder3(cylinderInclination = -30, animation = animation, crankAngleOffset = 210) annotation(
                Placement(transformation(extent = {{-30, -10}, {-10, 10}}, rotation = 0)));
              Cylinder cylinder4(cylinderInclination = 30, animation = animation, crankAngleOffset = 210) annotation(
                Placement(transformation(extent = {{0, -10}, {20, 10}}, rotation = 0)));
              Cylinder cylinder5(cylinderInclination = -30, animation = animation, crankAngleOffset = 90) annotation(
                Placement(transformation(extent = {{30, -10}, {50, 10}}, rotation = 0)));
              Cylinder cylinder6(cylinderInclination = 30, animation = animation, crankAngleOffset = -30) annotation(
                Placement(transformation(extent = {{60, -10}, {80, 10}}, rotation = 0)));
              Joints.Revolute bearing(useAxisFlange = true, n = {1, 0, 0}, cylinderLength = 0.02, cylinderDiameter = 0.06, animation = true) annotation(
                Placement(transformation(extent = {{-90, -40}, {-70, -60}}, rotation = 0)));
              Parts.BodyShape crank(animation = false, r = {0, 0, 0}, r_CM = {6 * 0.1 / 2, 0, 0}, I_22 = 1.e-5, I_33 = 1.e-5, m = 6 * 30, I_11 = 0.1) annotation(
                Placement(transformation(extent = {{-50, -60}, {-30, -40}}, rotation = 0)));
              Modelica.Mechanics.Rotational.Interfaces.Flange_b flange_b annotation(
                Placement(transformation(extent = {{100, -10}, {120, 10}}, rotation = 0)));
              Interfaces.Frame_a frame_a annotation(
                Placement(transformation(origin = {0, -101}, extent = {{-16, -16}, {16, 16}}, rotation = 270)));
            equation
              connect(cylinder1.crank_b, cylinder2.crank_a) annotation(
                Line(points = {{-70, -10}, {-60, -10}}, color = {95, 95, 95}, thickness = 0.5));
              connect(cylinder2.cylinder_a, cylinder1.cylinder_b) annotation(
                Line(points = {{-60, 10}, {-70, 10}}, color = {95, 95, 95}, thickness = 0.5));
              connect(cylinder3.cylinder_a, cylinder2.cylinder_b) annotation(
                Line(points = {{-30, 10}, {-40, 10}}, color = {95, 95, 95}, thickness = 0.5));
              connect(cylinder3.crank_a, cylinder2.crank_b) annotation(
                Line(points = {{-30, -10}, {-40, -10}}, color = {95, 95, 95}, thickness = 0.5));
              connect(cylinder3.cylinder_b, cylinder4.cylinder_a) annotation(
                Line(points = {{-10, 10}, {0, 10}}, color = {95, 95, 95}, thickness = 0.5));
              connect(cylinder3.crank_b, cylinder4.crank_a) annotation(
                Line(points = {{-10, -10}, {0, -10}}, color = {95, 95, 95}, thickness = 0.5));
              connect(cylinder4.cylinder_b, cylinder5.cylinder_a) annotation(
                Line(points = {{20, 10}, {30, 10}}, color = {95, 95, 95}, thickness = 0.5));
              connect(cylinder4.crank_b, cylinder5.crank_a) annotation(
                Line(points = {{20, -10}, {30, -10}}, color = {95, 95, 95}, thickness = 0.5));
              connect(cylinder5.cylinder_b, cylinder6.cylinder_a) annotation(
                Line(points = {{50, 10}, {60, 10}}, color = {95, 95, 95}, thickness = 0.5));
              connect(cylinder5.crank_b, cylinder6.crank_a) annotation(
                Line(points = {{50, -10}, {60, -10}}, color = {95, 95, 95}, thickness = 0.5));
              connect(bearing.frame_b, crank.frame_a) annotation(
                Line(points = {{-70, -50}, {-50, -50}}, color = {95, 95, 95}, thickness = 0.5));
              connect(crank.frame_b, cylinder1.crank_a) annotation(
                Line(points = {{-30, -50}, {-24, -50}, {-24, -26}, {-94, -26}, {-94, -10}, {-90, -10}}, color = {95, 95, 95}, thickness = 0.5));
              connect(bearing.axis, flange_b) annotation(
                Line(points = {{-80, -60}, {-80, -66}, {90, -66}, {90, 0}, {110, 0}}, color = {0, 0, 0}));
              connect(frame_a, bearing.frame_a) annotation(
                Line(points = {{0, -101}, {0, -81}, {-98, -81}, {-98, -50}, {-90, -50}}, color = {95, 95, 95}, thickness = 0.5));
              connect(bearing.frame_a, cylinder1.cylinder_a) annotation(
                Line(points = {{-90, -50}, {-98, -50}, {-98, 10}, {-90, 10}}, color = {95, 95, 95}, thickness = 0.5));
              annotation(
                Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, 100}, {100, -100}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-150, 145}, {150, 105}}, lineColor = {0, 0, 255}, textString = "%name"), Bitmap(extent = {{-97, -39}, {99, 75}}, fileName = "modelica://Modelica/Resources/Images/Mechanics/MultiBody/Examples/Loops/EngineV6_CAD.png")}));
            end EngineV6_analytic;

            partial model Engine1bBase "Model of one cylinder engine with gas force"
              Modelica.Mechanics.MultiBody.Parts.BodyCylinder Piston(diameter = 0.1, r = {0, -0.1, 0}) annotation(
                Placement(transformation(origin = {120, 53}, extent = {{-10, 30}, {10, -30}}, rotation = 270)));
              Modelica.Mechanics.MultiBody.Parts.BodyBox Rod2(widthDirection = {1, 0, 0}, width = 0.02, height = 0.06, color = {0, 0, 200}, r = {0, 0.2, 0}) annotation(
                Placement(transformation(origin = {120, 10}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
              Modelica.Mechanics.MultiBody.Joints.Revolute Bearing(useAxisFlange = true, n = {1, 0, 0}, cylinderLength = 0.02, cylinderDiameter = 0.05) annotation(
                Placement(transformation(extent = {{-50, -80}, {-30, -100}}, rotation = 0)));
              inner Modelica.Mechanics.MultiBody.World world annotation(
                Placement(transformation(extent = {{-90, -100}, {-70, -80}}, rotation = 0)));
              Modelica.Mechanics.Rotational.Components.Inertia Inertia(stateSelect = StateSelect.always, J = 0.1, w(fixed = true), phi(fixed = true, start = 0.001, displayUnit = "rad")) annotation(
                Placement(transformation(extent = {{-68, -120}, {-48, -100}}, rotation = 0)));
              Modelica.Mechanics.MultiBody.Parts.BodyBox Crank4(height = 0.05, widthDirection = {1, 0, 0}, width = 0.02, r = {0, -0.1, 0}) annotation(
                Placement(transformation(origin = {75.5, -75}, extent = {{10, -10}, {-10, 10}}, rotation = 90)));
              Modelica.Mechanics.MultiBody.Parts.BodyCylinder Crank3(r = {0.1, 0, 0}, diameter = 0.03) annotation(
                Placement(transformation(extent = {{41.5, -71}, {61.5, -51}}, rotation = 0)));
              Modelica.Mechanics.MultiBody.Parts.BodyCylinder Crank1(diameter = 0.05, r = {0.1, 0, 0}) annotation(
                Placement(transformation(extent = {{-16, -100}, {4, -80}}, rotation = 0)));
              Modelica.Mechanics.MultiBody.Parts.BodyBox Crank2(height = 0.05, widthDirection = {1, 0, 0}, width = 0.02, r = {0, 0.1, 0}) annotation(
                Placement(transformation(origin = {30, -76}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
              Modelica.Mechanics.MultiBody.Parts.FixedTranslation Mid(r = {0.05, 0, 0}) annotation(
                Placement(transformation(extent = {{30, -53}, {50, -33}}, rotation = 0)));
              Modelica.Mechanics.MultiBody.Parts.FixedTranslation cylPosition(animation = false, r = {0.15, 0.55, 0}) annotation(
                Placement(transformation(extent = {{-40.5, 100}, {-20.5, 120}}, rotation = 0)));
              Utilities.GasForce2 gasForce(d = 0.1, L = 0.35) annotation(
                Placement(transformation(origin = {119, 97}, extent = {{10, -10}, {-10, 10}}, rotation = 90)));
            equation
              connect(world.frame_b, Bearing.frame_a) annotation(
                Line(points = {{-70, -90}, {-50, -90}}, color = {95, 95, 95}, thickness = 0.5));
              connect(Crank2.frame_a, Crank1.frame_b) annotation(
                Line(points = {{30, -86}, {30, -90}, {4, -90}}, color = {95, 95, 95}, thickness = 0.5));
              connect(Crank2.frame_b, Crank3.frame_a) annotation(
                Line(points = {{30, -66}, {30, -61}, {41.5, -61}}, color = {95, 95, 95}, thickness = 0.5));
              connect(Bearing.frame_b, Crank1.frame_a) annotation(
                Line(points = {{-30, -90}, {-16, -90}}, color = {95, 95, 95}, thickness = 0.5));
              connect(world.frame_b, cylPosition.frame_a) annotation(
                Line(points = {{-70, -90}, {-60, -90}, {-60, 110}, {-40.5, 110}}, color = {95, 95, 95}, thickness = 0.5));
              connect(Crank3.frame_b, Crank4.frame_a) annotation(
                Line(points = {{61.5, -61}, {75, -61}, {75, -65}, {75.5, -65}}, color = {95, 95, 95}, thickness = 0.5));
              connect(Inertia.flange_b, Bearing.axis) annotation(
                Line(points = {{-48, -110}, {-40, -110}, {-40, -100}}, color = {0, 0, 0}));
              connect(Mid.frame_a, Crank2.frame_b) annotation(
                Line(points = {{30, -43}, {23, -43}, {23, -61}, {30, -61}, {30, -66}}, color = {95, 95, 95}, thickness = 0.5));
              annotation(
                Documentation(info = "<html>
<p>
This is a model of the mechanical part of one cylinder of an engine.
The combustion is not modelled. The \"inertia\" component at the lower
left part is the output inertia of the engine driving the gearbox.
The angular velocity of the output inertia has a start value of 10 rad/s
in order to demonstrate the movement of the engine.
</p>
<p>
The engine is modeled solely by revolute and prismatic joints.
Since this results in a <b>planar</b> loop there is the well known
difficulty that the cut-forces perpendicular to the loop cannot be
uniquely computed, as well as the cut-torques within the plane.
This ambiguity is resolved by using the option <b>planarCutJoint</b>
in the <b>Advanced</b> menu of one revolute joint in every planar loop
(here: joint B1). This option sets the cut-force in direction of the
axis of rotation, as well as the cut-torques perpendicular to the axis
of rotation at this joint to zero and makes the problem mathematically
well-formed.
</p>
<p>
An animation of this example is shown in the figure below.
</p>

<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Examples/Loops/Engine.png\" ALT=\"model Examples.Loops.Engine\">
</html>"));
            end Engine1bBase;
          end Utilities;
          annotation(
            Documentation(info = "<html>
<p>
This package contains different examples to show how
mechanical systems with kinematic loops can be modeled.
</p>
<h4>Content</h4>
<table border=1 cellspacing=0 cellpadding=2>
  <tr><th><b><i>Model</i></b></th><th><b><i>Description</i></b></th></tr>
  <tr><td valign=\"top\"><a href=\"modelica://Modelica.Mechanics.MultiBody.Examples.Loops.Engine1a\">Engine1a</a><br>
             <a href=\"modelica://Modelica.Mechanics.MultiBody.Examples.Loops.Engine1b\">Engine1b</a><br>
             <a href=\"modelica://Modelica.Mechanics.MultiBody.Examples.Loops.Engine1b_analytic\">Engine1b_analytic</a></td>
      <td valign=\"top\"> Model of one cylinder engine (Engine1a: simple, without combustion; Engine1b: with combustion;
           Engine1b_analytic: same as Engine1b but analytic loop handling)<br>
      <img src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Examples/Loops/Engine.png\">
      </td>
  </tr>
  <tr><td valign=\"top\"><a href=\"modelica://Modelica.Mechanics.MultiBody.Examples.Loops.EngineV6\">EngineV6</a><br>
<a href=\"modelica://Modelica.Mechanics.MultiBody.Examples.Loops.EngineV6_analytic\">EngineV6_analytic</a></td>
      <td valign=\"top\"> V6 engine with 6 cylinders, 6 planar loops and 1 degree-of-freedom.
           Second version with analytic handling of kinematic loops and CAD data
           animation.<br>
      <img src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Examples/Loops/EngineV6_small.png\">
      <img src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Examples/Loops/EngineV6_CAD_smaller.png\">
      </td>
  </tr>
  <tr><td valign=\"top\"><a href=\"modelica://Modelica.Mechanics.MultiBody.Examples.Loops.Fourbar1\">Fourbar1</a></td>
      <td valign=\"top\"> One kinematic loop with four bars (with only revolute joints;
           5 non-linear equations)<br>
      <img src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Examples/Loops/Fourbar1_small.png\">
      </td>
  </tr>
  <tr><td valign=\"top\"><a href=\"modelica://Modelica.Mechanics.MultiBody.Examples.Loops.Fourbar2\">Fourbar2</a></td>
      <td valign=\"top\"> One kinematic loop with four bars (with UniversalSpherical
           joint; 1 non-linear equation) <br>
      <img src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Examples/Loops/Fourbar2_small.png\">
      </td>
  </tr>
  <tr><td valign=\"top\"><a href=\"modelica://Modelica.Mechanics.MultiBody.Examples.Loops.Fourbar_analytic\">Fourbar_analytic</a></td>
      <td valign=\"top\"> One kinematic loop with four bars (with JointSSP joint;
           analytic solution of non-linear algebraic loop)  <br>
      <img src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Examples/Loops/Fourbar_analytic_small.png\">
      </td>
  </tr>
  <tr><td valign=\"top\"><a href=\"modelica://Modelica.Mechanics.MultiBody.Examples.Loops.PlanarLoops_analytic\">PlanarLoops_analytic</a></td>
      <td valign=\"top\"> Mechanism with three planar kinematic loops and one
           degree-of-freedom with analytic loop handling
           (with JointRRR joints) <br>
      <img src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Examples/Loops/PlanarLoops_small.png\">
      </td>
  </tr>
</table>
</html>"));
        end Loops;

        package Rotational3DEffects "Demonstrates the usage of 1-dim. rotational elements with all 3-dim. effects included"
          extends Modelica.Icons.ExamplesPackage;

          model GyroscopicEffects "Demonstrates that a cylindrical body can be replaced by Rotor1D model"
            extends Modelica.Icons.Example;
            inner World world(driveTrainMechanics3D = true) annotation(
              Placement(transformation(extent = {{-100, 20}, {-80, 40}}, rotation = 0)));
            Joints.Spherical spherical1(angles_fixed = true, w_rel_a_fixed = true, z_rel_a_fixed = false) annotation(
              Placement(transformation(extent = {{-70, 20}, {-50, 40}}, rotation = 0)));
            Parts.BodyCylinder bodyCylinder1(r = {0.25, 0, 0}, diameter = 0.05) annotation(
              Placement(transformation(extent = {{-44, 20}, {-24, 40}}, rotation = 0)));
            Parts.FixedRotation fixedRotation1(angle = 45, n = {0, 1, 0}) annotation(
              Placement(transformation(extent = {{-16, 20}, {4, 40}}, rotation = 0)));
            Joints.Revolute revolute(n = {1, 0, 0}, a(fixed = false), phi(fixed = true), w(fixed = true, start = 10)) annotation(
              Placement(transformation(extent = {{14, 20}, {34, 40}}, rotation = 0)));
            Parts.FixedTranslation fixedTranslation(r = {-0.1, 0, 0}) annotation(
              Placement(transformation(extent = {{42, 20}, {62, 40}}, rotation = 0)));
            Parts.BodyCylinder bodyCylinder2(diameter = 0.1, r = {0.2, 0, 0}) annotation(
              Placement(transformation(extent = {{70, 20}, {90, 40}}, rotation = 0)));
            Parts.Fixed fixed annotation(
              Placement(transformation(extent = {{-100, -40}, {-80, -20}}, rotation = 0)));
            Joints.Spherical spherical2(angles_fixed = true, w_rel_a_fixed = true, z_rel_a_fixed = false) annotation(
              Placement(transformation(extent = {{-70, -40}, {-50, -20}}, rotation = 0)));
            Parts.BodyCylinder bodyCylinder3(r = {0.25, 0, 0}, diameter = 0.05, color = {0, 128, 0}) annotation(
              Placement(transformation(extent = {{-44, -40}, {-24, -20}}, rotation = 0)));
            Parts.FixedRotation fixedRotation2(n = {0, 1, 0}, angle = 45) annotation(
              Placement(transformation(extent = {{-16, -40}, {4, -20}}, rotation = 0)));
            Parts.FixedTranslation fixedTranslation1(r = {-0.1, 0, 0}) annotation(
              Placement(transformation(extent = {{24, -40}, {44, -20}}, rotation = 0)));
            Parts.BodyCylinder bodyCylinder4(diameter = 0.1, r = {0.2, 0, 0}, color = {0, 128, 0}) annotation(
              Placement(transformation(extent = {{56, -40}, {76, -20}}, rotation = 0)));
            Parts.Rotor1D rotor1D(J = bodyCylinder4.I[1, 1], n = {1, 0, 0}, a(fixed = false), phi(fixed = true), w(fixed = true, start = 10)) annotation(
              Placement(transformation(extent = {{28, -70}, {48, -50}}, rotation = 0)));
          equation
            connect(world.frame_b, spherical1.frame_a) annotation(
              Line(points = {{-80, 30}, {-70, 30}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            connect(spherical1.frame_b, bodyCylinder1.frame_a) annotation(
              Line(points = {{-50, 30}, {-44, 30}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            connect(bodyCylinder1.frame_b, fixedRotation1.frame_a) annotation(
              Line(points = {{-24, 30}, {-16, 30}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            connect(fixedRotation1.frame_b, revolute.frame_a) annotation(
              Line(points = {{4, 30}, {14, 30}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            connect(revolute.frame_b, fixedTranslation.frame_a) annotation(
              Line(points = {{34, 30}, {42, 30}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            connect(fixedTranslation.frame_b, bodyCylinder2.frame_a) annotation(
              Line(points = {{62, 30}, {70, 30}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            connect(spherical2.frame_b, bodyCylinder3.frame_a) annotation(
              Line(points = {{-50, -30}, {-44, -30}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            connect(bodyCylinder3.frame_b, fixedRotation2.frame_a) annotation(
              Line(points = {{-24, -30}, {-16, -30}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            connect(fixed.frame_b, spherical2.frame_a) annotation(
              Line(points = {{-80, -30}, {-70, -30}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            connect(fixedTranslation1.frame_b, bodyCylinder4.frame_a) annotation(
              Line(points = {{44, -30}, {56, -30}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            connect(fixedRotation2.frame_b, fixedTranslation1.frame_a) annotation(
              Line(points = {{4, -30}, {24, -30}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            connect(rotor1D.frame_a, fixedRotation2.frame_b) annotation(
              Line(points = {{38, -70}, {38, -76}, {10, -76}, {10, -30}, {4, -30}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            annotation(
              experiment(StopTime = 5, Tolerance = 1e-008),
              Documentation(info = "<html>
<p>
This example consists of a body that is attached to the world system
with a spherical joint. On this body, a \"rotor\", i.e., a body with rotational
symmetry is present. Two kinds of models are shown:
</p>

<ul>
<li> In the upper part of the diagram layer, only multi-body components are used.</li>
<li> In the lower part of the diagram layer, the same model is implemented,
     but by a different modeling of the cylindrical body:
     The cylindrical body is included, but it is rigidly attached to its mount.
     This part takes into account the movement of the center of mass and of
     the inertia tensor of the cylindrical body. Note, since the cylindrical body
     has rotational symmetry, its center of mass and its inertia tensor is independent
     of the angle of the inertia and can therefore be rigidly attached to its mount.
     Additionally, with a \"MultiBody.Parts.Rotor1D\" model, a primarily 1-dim.
     inertia is included that takes into account the additional effects when the
     cylindrical body is moving relatively to its mounts</li>
</ul>

<p>
The simulation reveals that both the kinematic movement and the reaction forces on the
environment (object \"world\" and \"fixed\" respectively) are identical for both models.
</p>

<p>
A typical usage scenario is to model a complete drive train of a vehicle, including
the automatic gearbox, with elements of the \"Mechanics.Rotational\" library, but using
the \"Rotor1D\" model instead of the \"Rotational.Components.Inertia\" component.
This drive train model can be mounted on a 3-dim. multi-body model of the vehicle.
Additionally, one rigid body has to be fixed to the vehicle that has the mass, center
of mass and inertia tensor of the complete drive train. Both models together, give
exactly the same effect, as if every part of the drive train would have been modelled
solely with mult-body components. One benefit of this modeling is that the simulation
is much faster.
</p>

</html>"));
          end GyroscopicEffects;

          model ActuatedDrive
            extends Modelica.Icons.Example;
            Parts.BodyShape bodyCylinder(r = {0.5, 0, 0}, m = 0, I_11 = 2, I_22 = 0, I_33 = 0, shapeType = "cylinder", width = 0.1, animateSphere = false, r_shape = {0.1, 0, 0}, r_CM = {0, 0, 0}) annotation(
              Placement(transformation(extent = {{8, 0}, {28, 20}}, rotation = 0)));
            Joints.Revolute revolute(n = {1, 0, 0}, a(fixed = false), phi(fixed = true), w(fixed = true)) annotation(
              Placement(transformation(extent = {{-26, 0}, {-6, 20}}, rotation = 0)));
            inner World world(g = 0, driveTrainMechanics3D = true) annotation(
              Placement(transformation(extent = {{-60, 0}, {-40, 20}}, rotation = 0)));
            Forces.Torque torque annotation(
              Placement(transformation(extent = {{8, 30}, {28, 50}}, rotation = 0)));
            Blocks.Sources.Sine sine[3](amplitude = {1, 0, 0}, freqHz = {1, 1, 1}) annotation(
              Placement(transformation(extent = {{-80, 60}, {-60, 80}}, rotation = 0)));
            inner Parts.Fixed fixed annotation(
              Placement(transformation(extent = {{-62, -90}, {-42, -70}}, rotation = 0)));
            Parts.Rotor1D rotor1D(J = 2, phi(fixed = true), w(fixed = true)) annotation(
              Placement(transformation(extent = {{0, -40}, {20, -20}}, rotation = 0)));
            Rotational.Sources.Torque torque1(useSupport = true) annotation(
              Placement(transformation(extent = {{-32, -40}, {-12, -20}}, rotation = 0)));
            Parts.Mounting1D mounting1D annotation(
              Placement(transformation(extent = {{-46, -60}, {-26, -40}}, rotation = 0)));
          equation
            connect(world.frame_b, revolute.frame_a) annotation(
              Line(points = {{-40, 10}, {-26, 10}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            connect(revolute.frame_b, bodyCylinder.frame_a) annotation(
              Line(points = {{-6, 10}, {8, 10}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            connect(torque.frame_b, bodyCylinder.frame_b) annotation(
              Line(points = {{28, 40}, {38, 40}, {38, 10}, {28, 10}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            connect(torque.frame_resolve, world.frame_b) annotation(
              Line(points = {{22, 50}, {22, 64}, {-30, 64}, {-30, 10}, {-40, 10}}, color = {95, 95, 95}, pattern = LinePattern.Dot, smooth = Smooth.None));
            connect(torque.frame_a, world.frame_b) annotation(
              Line(points = {{8, 40}, {-30, 40}, {-30, 10}, {-40, 10}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            connect(sine.y, torque.torque) annotation(
              Line(points = {{-59, 70}, {12, 70}, {12, 52}}, color = {0, 0, 127}, smooth = Smooth.None));
            connect(fixed.frame_b, rotor1D.frame_a) annotation(
              Line(points = {{-42, -80}, {10, -80}, {10, -40}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            connect(torque1.flange, rotor1D.flange_a) annotation(
              Line(points = {{-12, -30}, {0, -30}}, color = {0, 0, 0}, smooth = Smooth.None));
            connect(mounting1D.flange_b, torque1.support) annotation(
              Line(points = {{-26, -50}, {-22, -50}, {-22, -40}}, color = {0, 0, 0}, smooth = Smooth.None));
            connect(mounting1D.frame_a, fixed.frame_b) annotation(
              Line(points = {{-36, -60}, {-36, -80}, {-42, -80}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            connect(sine[1].y, torque1.tau) annotation(
              Line(points = {{-59, 70}, {-50, 70}, {-50, 40}, {-72, 40}, {-72, -30}, {-34, -30}}, color = {0, 0, 127}, smooth = Smooth.None));
            annotation(
              experiment(StopTime = 1.1));
          end ActuatedDrive;

          model MovingActuatedDrive
            extends Modelica.Icons.Example;
            Parts.BodyShape bodyCylinder(r = {0.5, 0, 0}, m = 0, I_11 = 2, I_22 = 0, I_33 = 0, shapeType = "cylinder", width = 0.1, animateSphere = false, r_shape = {0.1, 0, 0}, r_CM = {0, 0, 0}) annotation(
              Placement(transformation(extent = {{50, 10}, {70, 30}}, rotation = 0)));
            Joints.Revolute revolute(n = {1, 0, 0}, a(fixed = false), phi(fixed = true), w(fixed = true)) annotation(
              Placement(transformation(extent = {{16, 10}, {36, 30}}, rotation = 0)));
            inner World world(g = 0, driveTrainMechanics3D = true) annotation(
              Placement(transformation(extent = {{-84, 10}, {-64, 30}}, rotation = 0)));
            Forces.Torque torque annotation(
              Placement(transformation(extent = {{50, 40}, {70, 60}}, rotation = 0)));
            Blocks.Sources.Sine sine1[3](amplitude = {1, 0, 0}, freqHz = {1, 1, 1}) annotation(
              Placement(transformation(extent = {{16, 70}, {36, 90}}, rotation = 0)));
            Parts.Rotor1D rotor1D(J = 2, phi(fixed = true), w(fixed = true)) annotation(
              Placement(transformation(extent = {{50, -36}, {70, -16}}, rotation = 0)));
            Rotational.Sources.Torque torque1(useSupport = true) annotation(
              Placement(transformation(extent = {{18, -36}, {38, -16}}, rotation = 0)));
            Parts.Mounting1D mounting1D annotation(
              Placement(transformation(extent = {{4, -56}, {24, -36}}, rotation = 0)));
            Joints.Revolute r1(useAxisFlange = true, n = {0, 1, 0}) annotation(
              Placement(transformation(extent = {{-32, 10}, {-12, 30}}, rotation = 0)));
            Rotational.Sources.Position position1(useSupport = true, w(fixed = true)) annotation(
              Placement(transformation(extent = {{-46, 60}, {-26, 80}}, rotation = 0)));
            Blocks.Sources.Sine sine2(amplitude = 2, freqHz = 1) annotation(
              Placement(transformation(extent = {{-100, 60}, {-80, 80}}, rotation = 0)));
            Parts.Mounting1D mounting1D1 annotation(
              Placement(transformation(extent = {{-60, 34}, {-40, 54}}, rotation = 0)));
            Joints.Revolute r2(useAxisFlange = true, n = {0, 1, 0}) annotation(
              Placement(transformation(extent = {{-38, -80}, {-18, -60}}, rotation = 0)));
            Rotational.Sources.Position position2(useSupport = true, w(fixed = true)) annotation(
              Placement(transformation(extent = {{-52, -30}, {-32, -10}}, rotation = 0)));
            Parts.Mounting1D mounting1D2 annotation(
              Placement(transformation(extent = {{-66, -56}, {-46, -36}}, rotation = 0)));
            Parts.Fixed fixed annotation(
              Placement(transformation(extent = {{-86, -80}, {-66, -60}}, rotation = 0)));
          equation
            connect(revolute.frame_b, bodyCylinder.frame_a) annotation(
              Line(points = {{36, 20}, {50, 20}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            connect(torque.frame_b, bodyCylinder.frame_b) annotation(
              Line(points = {{70, 50}, {80, 50}, {80, 20}, {70, 20}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            connect(sine1.y, torque.torque) annotation(
              Line(points = {{37, 80}, {54, 80}, {54, 62}}, color = {0, 0, 127}, smooth = Smooth.None));
            connect(torque1.flange, rotor1D.flange_a) annotation(
              Line(points = {{38, -26}, {50, -26}}, color = {0, 0, 0}, smooth = Smooth.None));
            connect(mounting1D.flange_b, torque1.support) annotation(
              Line(points = {{24, -46}, {28, -46}, {28, -36}}, color = {0, 0, 0}, smooth = Smooth.None));
            connect(r1.frame_a, world.frame_b) annotation(
              Line(points = {{-32, 20}, {-64, 20}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            connect(position1.flange, r1.axis) annotation(
              Line(points = {{-26, 70}, {-22, 70}, {-22, 30}}, color = {0, 0, 0}, smooth = Smooth.None));
            connect(position1.support, mounting1D1.flange_b) annotation(
              Line(points = {{-36, 60}, {-36, 44}, {-40, 44}}, color = {0, 0, 0}, smooth = Smooth.None));
            connect(mounting1D1.frame_a, world.frame_b) annotation(
              Line(points = {{-50, 34}, {-50, 20}, {-64, 20}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            connect(sine2.y, position1.phi_ref) annotation(
              Line(points = {{-79, 70}, {-48, 70}}, color = {0, 0, 127}, smooth = Smooth.None));
            connect(r1.frame_b, revolute.frame_a) annotation(
              Line(points = {{-12, 20}, {16, 20}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            connect(torque.frame_resolve, revolute.frame_a) annotation(
              Line(points = {{64, 60}, {8, 60}, {8, 20}, {16, 20}}, color = {95, 95, 95}, pattern = LinePattern.Dot, smooth = Smooth.None));
            connect(torque.frame_a, revolute.frame_a) annotation(
              Line(points = {{50, 50}, {8, 50}, {8, 20}, {16, 20}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            connect(position2.flange, r2.axis) annotation(
              Line(points = {{-32, -20}, {-28, -20}, {-28, -60}}, color = {0, 0, 0}, smooth = Smooth.None));
            connect(position2.support, mounting1D2.flange_b) annotation(
              Line(points = {{-42, -30}, {-42, -46}, {-46, -46}}, color = {0, 0, 0}, smooth = Smooth.None));
            connect(fixed.frame_b, r2.frame_a) annotation(
              Line(points = {{-66, -70}, {-38, -70}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            connect(fixed.frame_b, mounting1D2.frame_a) annotation(
              Line(points = {{-66, -70}, {-56, -70}, {-56, -56}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            connect(sine2.y, position2.phi_ref) annotation(
              Line(points = {{-79, 70}, {-70, 70}, {-70, 46}, {-92, 46}, {-92, -20}, {-54, -20}}, color = {0, 0, 127}, smooth = Smooth.None));
            connect(r2.frame_b, rotor1D.frame_a) annotation(
              Line(points = {{-18, -70}, {60, -70}, {60, -36}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            connect(r2.frame_b, mounting1D.frame_a) annotation(
              Line(points = {{-18, -70}, {14, -70}, {14, -56}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            connect(sine1[1].y, torque1.tau) annotation(
              Line(points = {{37, 80}, {92, 80}, {92, -10}, {0, -10}, {0, -26}, {16, -26}}, color = {0, 0, 127}, smooth = Smooth.None));
            annotation(
              experiment(StopTime = 1.1),
              Documentation(info = "<html>
<p>
This model demonstrates how a moving drive train modelled with 3-dim. multi-body elements
(revolute, bodyCylinder) can alternatively be modeled with component rotor1D to speed up
simulation. The movement of the two systems is identical and also the cut-torques in the
various frames (such as: r1.frame_b.t and r2.frame_b.t).
</p>

<p>
The driving joints (r1, r2) with rotation axis {0,1,0} are modelled to be driven by a motor torque
along the {1,0,0} axis. Basically, this means that an idealized bevel gear is used to drive the
axes of the revolute joints.
</p>
</html>"));
          end MovingActuatedDrive;

          model GearConstraint
            extends Modelica.Icons.Example;
            Joints.GearConstraint gearConstraint(ratio = 10, phi_b(fixed = true), w_b(fixed = true)) annotation(
              Placement(transformation(extent = {{34, 40}, {54, 60}}, rotation = 0)));
            inner World world(driveTrainMechanics3D = true, g = 0) annotation(
              Placement(transformation(extent = {{-62, 10}, {-42, 30}}, rotation = 0)));
            Parts.BodyCylinder cyl1(diameter = 0.1, color = {0, 128, 0}, r = {0.4, 0, 0}) annotation(
              Placement(transformation(extent = {{2, 40}, {22, 60}}, rotation = 0)));
            Parts.BodyCylinder cyl2(r = {0.4, 0, 0}, diameter = 0.2) annotation(
              Placement(transformation(extent = {{70, 40}, {90, 60}}, rotation = 0)));
            Forces.Torque torque1 annotation(
              Placement(transformation(extent = {{-26, 40}, {-6, 60}}, rotation = 0)));
            Blocks.Sources.Sine sine[3](amplitude = {2, 0, 0}, freqHz = {1, 1, 1}) annotation(
              Placement(transformation(extent = {{-100, 60}, {-80, 80}}, rotation = 0)));
            Parts.Fixed fixed annotation(
              Placement(transformation(extent = {{-48, -90}, {-28, -70}}, rotation = 0)));
            Rotational.Components.Inertia inertia1(J = cyl1.I[1, 1], a(fixed = false), phi(fixed = true, start = 0), w(fixed = true, start = 0)) annotation(
              Placement(transformation(extent = {{-20, -40}, {0, -20}}, rotation = 0)));
            Rotational.Components.IdealGear idealGear(ratio = 10, useSupport = true) annotation(
              Placement(transformation(extent = {{12, -40}, {32, -20}}, rotation = 0)));
            Rotational.Components.Inertia inertia2(J = cyl2.I[1, 1]) annotation(
              Placement(transformation(extent = {{44, -40}, {64, -20}}, rotation = 0)));
            Rotational.Sources.Torque torque2(useSupport = true) annotation(
              Placement(transformation(extent = {{-48, -40}, {-28, -20}}, rotation = 0)));
            Parts.Mounting1D mounting1D annotation(
              Placement(transformation(extent = {{-20, -70}, {0, -50}}, rotation = 0)));
          equation
            connect(world.frame_b, gearConstraint.bearing) annotation(
              Line(points = {{-42, 20}, {44, 20}, {44, 40}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            connect(cyl1.frame_b, gearConstraint.frame_a) annotation(
              Line(points = {{22, 50}, {34, 50}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            connect(gearConstraint.frame_b, cyl2.frame_a) annotation(
              Line(points = {{54, 50}, {70, 50}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            connect(torque1.frame_b, cyl1.frame_a) annotation(
              Line(points = {{-6, 50}, {2, 50}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            connect(torque1.frame_a, world.frame_b) annotation(
              Line(points = {{-26, 50}, {-36, 50}, {-36, 20}, {-42, 20}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            connect(sine.y, torque1.torque) annotation(
              Line(points = {{-79, 70}, {-22, 70}, {-22, 62}}, color = {0, 0, 127}, smooth = Smooth.None));
            connect(inertia1.flange_b, idealGear.flange_a) annotation(
              Line(points = {{0, -30}, {12, -30}}, color = {0, 0, 0}, smooth = Smooth.None));
            connect(idealGear.flange_b, inertia2.flange_a) annotation(
              Line(points = {{32, -30}, {44, -30}}, color = {0, 0, 0}, smooth = Smooth.None));
            connect(torque2.flange, inertia1.flange_a) annotation(
              Line(points = {{-28, -30}, {-20, -30}}, color = {0, 0, 0}, smooth = Smooth.None));
            connect(sine[1].y, torque2.tau) annotation(
              Line(points = {{-79, 70}, {-72, 70}, {-72, -30}, {-50, -30}}, color = {0, 0, 127}, smooth = Smooth.None));
            connect(mounting1D.flange_b, idealGear.support) annotation(
              Line(points = {{0, -60}, {22, -60}, {22, -40}}, color = {0, 0, 0}, smooth = Smooth.None));
            connect(mounting1D.flange_b, torque2.support) annotation(
              Line(points = {{0, -60}, {4, -60}, {4, -48}, {-38, -48}, {-38, -40}}, color = {0, 0, 0}, smooth = Smooth.None));
            connect(fixed.frame_b, mounting1D.frame_a) annotation(
              Line(points = {{-28, -80}, {-10, -80}, {-10, -70}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            annotation(
              experiment(StopTime = 5));
          end GearConstraint;
          annotation(
            Documentation(info = "<html>
<p>
This library demonstrates the usage of elements of the Mechanics.Rotational library
by taking into account all 3-dim. effects. The reason for this type of modeling is
to speedup the simulation drastically. This is possible if moving bodies have
rotational symmetry. A typical application area are drive trains, driving joints of
a multi-body system.
</p>
</html>"));
        end Rotational3DEffects;

        package Constraints "Examples with constraint joints"
          extends Modelica.Icons.ExamplesPackage;

          model PrismaticConstraint "Body attached by one spring and two prismatic joints or constrained to environment"
            extends Modelica.Icons.Example;
            parameter Boolean animation = true "True, if animation shall be enabled";
            Joints.Prismatic jointPrismatic_x(stateSelect = StateSelect.never, n = {1, 0, 0}, s(fixed = true), v(fixed = true)) annotation(
              Placement(transformation(extent = {{80, -30}, {60, -10}})));
            Joints.Prismatic jointPrismatic_y(stateSelect = StateSelect.never, n = {0, 1, 0}, s(fixed = true), v(fixed = true)) annotation(
              Placement(transformation(extent = {{40, -30}, {20, -10}})));
            Joints.Constraints.Prismatic constraint(x_locked = false, y_locked = false) annotation(
              Placement(transformation(extent = {{60, 10}, {40, 30}})));
            Modelica.Mechanics.MultiBody.Sensors.RelativeSensor sensorConstraintRelative(resolveInFrame = Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB.frame_a, get_r_rel = true, get_a_rel = false, get_angles = true) annotation(
              Placement(transformation(extent = {{60, 60}, {40, 40}})));
            Modelica.Mechanics.MultiBody.Parts.BodyShape bodyOfJoint(m = 1, I_11 = 1, I_22 = 1, I_33 = 1, r = {0.4, 0, 0}, r_CM = {0.2, 0, 0}, width = 0.05, r_0(start = {0.2, -0.5, 0.1}, each fixed = false), v_0(each fixed = false), angles_fixed = false, w_0_fixed = false, angles_start = {0.17453292519943, 0.95993108859688, 1.1868238913561}, final color = {0, 0, 255}) annotation(
              Placement(transformation(extent = {{-10, 10}, {10, -10}}, rotation = 180, origin = {-10, -20})));
            Modelica.Mechanics.MultiBody.Parts.BodyShape bodyOfConstraint(I_11 = 1, I_22 = 1, I_33 = 1, width = 0.05, w_0_fixed = false, final color = {0, 128, 0}, r = bodyOfJoint.r, r_CM = bodyOfJoint.r_CM, m = bodyOfJoint.m, angles_fixed = false, angles_start = {0.17453292519943, 0.95993108859688, 1.1868238913561}) annotation(
              Placement(transformation(extent = {{-10, 10}, {10, -10}}, rotation = 180, origin = {-10, 20})));
            Modelica.Mechanics.MultiBody.Forces.Spring springOfJoint(c = 20, s_unstretched = 0, width = 0.1, coilWidth = 0.005, numberOfWindings = 5) annotation(
              Placement(transformation(origin = {-50, -20}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Forces.Spring springOfConstraint(width = 0.1, coilWidth = 0.005, c = springOfJoint.c, s_unstretched = springOfJoint.s_unstretched, numberOfWindings = springOfJoint.numberOfWindings) annotation(
              Placement(transformation(origin = {-50, 20}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
            inner Modelica.Mechanics.MultiBody.World world annotation(
              Placement(transformation(extent = {{-100, -100}, {-80, -80}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Parts.FixedRotation fixedRotation(r = {0.2, -0.3, 0.2}, rotationType = Modelica.Mechanics.MultiBody.Types.RotationTypes.PlanarRotationSequence, angles = {10, 55, 68}) annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 90, origin = {90, -50})));
            Modelica.Mechanics.MultiBody.Parts.FixedTranslation fixedTranslation(animation = false, r = {0.8, 0, 0.3}) annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 90, origin = {-70, -50})));
            Joints.FreeMotionScalarInit freeMotionScalarInit(use_r = true, use_v = true, v_rel_a_2(fixed = true, start = 0), v_rel_a_3(fixed = true, start = 0), use_w = true, w_rel_b_1(fixed = false), w_rel_b_2(fixed = false), w_rel_b_3(fixed = false), angle_d_3(fixed = false), r_rel_a_2(fixed = true, start = 0), r_rel_a_3(fixed = true, start = 0), angle_1(fixed = false)) annotation(
              Placement(transformation(extent = {{40, 60}, {20, 80}})));
          equation
            connect(fixedTranslation.frame_a, world.frame_b) annotation(
              Line(points = {{-70, -60}, {-70, -90}, {-80, -90}}, color = {95, 95, 95}, thickness = 0.5));
            connect(bodyOfConstraint.frame_b, springOfConstraint.frame_b) annotation(
              Line(points = {{-20, 20}, {-40, 20}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            connect(world.frame_b, fixedRotation.frame_a) annotation(
              Line(points = {{-80, -90}, {90, -90}, {90, -60}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            connect(fixedRotation.frame_b, constraint.frame_a) annotation(
              Line(points = {{90, -40}, {90, 20}, {60, 20}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            connect(constraint.frame_a, sensorConstraintRelative.frame_a) annotation(
              Line(points = {{60, 20}, {70, 20}, {70, 50}, {60, 50}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            connect(bodyOfJoint.frame_b, springOfJoint.frame_b) annotation(
              Line(points = {{-20, -20}, {-40, -20}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            connect(sensorConstraintRelative.frame_b, constraint.frame_b) annotation(
              Line(points = {{40, 50}, {30, 50}, {30, 20}, {40, 20}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            connect(fixedTranslation.frame_b, springOfJoint.frame_a) annotation(
              Line(points = {{-70, -40}, {-70, -20}, {-60, -20}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            connect(fixedTranslation.frame_b, springOfConstraint.frame_a) annotation(
              Line(points = {{-70, -40}, {-70, 20}, {-60, 20}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            connect(bodyOfConstraint.frame_a, constraint.frame_b) annotation(
              Line(points = {{0, 20}, {40, 20}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            connect(jointPrismatic_x.frame_b, jointPrismatic_y.frame_a) annotation(
              Line(points = {{60, -20}, {40, -20}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            connect(fixedRotation.frame_b, jointPrismatic_x.frame_a) annotation(
              Line(points = {{90, -40}, {90, -20}, {80, -20}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            connect(bodyOfJoint.frame_a, jointPrismatic_y.frame_b) annotation(
              Line(points = {{0, -20}, {20, -20}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            connect(freeMotionScalarInit.frame_a, fixedRotation.frame_b) annotation(
              Line(points = {{40, 70}, {90, 70}, {90, -40}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            connect(freeMotionScalarInit.frame_b, bodyOfConstraint.frame_a) annotation(
              Line(points = {{20, 70}, {10, 70}, {10, 20}, {0, 20}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            annotation(
              experiment(StopTime = 10),
              Documentation(info = "<html>
<p>This example demonstrates the functionality of <b>constraint</b> representing <b>prismatic joint</b>. Each of two bodies is at one of its end connected by spring to the world. The other end is also connected to the world either by two serial coupled prismatic joints or by appropriate constraint. Therefore, the body can only perform translation in two directions specified in the two joints depending on working forces.</p>
<p><b>Simulation results</b> </p>
<p>After simulating the model, see the animation of the multibody system and compare movement of body connected by joint (blue colored) with movement of that one connected by constraint (of green color). Additionally, the outputs from <code>sensorConstraintRelative</code> depict both position and angle deviations in the constraining element.</p>
</html>"));
          end PrismaticConstraint;

          model RevoluteConstraint "Body attached by one spring and revolute joint or constrained to environment"
            extends Modelica.Icons.Example;
            parameter Boolean animation = true "= true, if animation shall be enabled";
            Modelica.Mechanics.MultiBody.Joints.Revolute joint(stateSelect = StateSelect.never, n = {0, 1, 0}, phi(fixed = true), w(fixed = true)) annotation(
              Placement(transformation(extent = {{60, -30}, {40, -10}})));
            Joints.Constraints.Revolute constraint(n = joint.n) annotation(
              Placement(transformation(extent = {{60, 10}, {40, 30}})));
            Modelica.Mechanics.MultiBody.Sensors.RelativeSensor sensorConstraintRelative(resolveInFrame = Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB.frame_a, get_r_rel = true, get_a_rel = false, get_angles = true) annotation(
              Placement(transformation(extent = {{60, 60}, {40, 40}})));
            Modelica.Mechanics.MultiBody.Parts.BodyShape bodyOfJoint(m = 1, I_11 = 1, I_22 = 1, I_33 = 1, r = {0.4, 0, 0}, r_CM = {0.2, 0, 0}, width = 0.05, r_0(start = {0.2, -0.5, 0.1}, each fixed = false), v_0(each fixed = false), angles_fixed = false, w_0_fixed = false, angles_start = {0.17453292519943, 0.95993108859688, 1.1868238913561}, final color = {0, 0, 255}) annotation(
              Placement(transformation(extent = {{-10, 10}, {10, -10}}, rotation = 180, origin = {-10, -20})));
            Modelica.Mechanics.MultiBody.Parts.BodyShape bodyOfConstraint(I_11 = 1, I_22 = 1, I_33 = 1, width = 0.05, r_0(start = {0.2, -0.5, 0.1}, each fixed = false), v_0(each fixed = false), angles_fixed = false, w_0_fixed = false, final color = {0, 128, 0}, r = bodyOfJoint.r, r_CM = bodyOfJoint.r_CM, m = bodyOfJoint.m, angles_start = {0.17453292519943, 0.95993108859688, 1.1868238913561}) annotation(
              Placement(transformation(extent = {{-10, 10}, {10, -10}}, rotation = 180, origin = {-10, 20})));
            Modelica.Mechanics.MultiBody.Forces.Spring springOfJoint(c = 20, s_unstretched = 0, width = 0.1, coilWidth = 0.005, numberOfWindings = 5) annotation(
              Placement(transformation(origin = {-50, -20}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Forces.Spring springOfConstraint(width = 0.1, coilWidth = 0.005, c = springOfJoint.c, s_unstretched = springOfJoint.s_unstretched, numberOfWindings = springOfJoint.numberOfWindings) annotation(
              Placement(transformation(origin = {-50, 20}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
            inner Modelica.Mechanics.MultiBody.World world annotation(
              Placement(transformation(extent = {{-100, -100}, {-80, -80}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Parts.FixedRotation fixedRotation(r = {0.2, -0.3, 0.2}, rotationType = Modelica.Mechanics.MultiBody.Types.RotationTypes.PlanarRotationSequence, angles = {10, 55, 68}) annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 90, origin = {80, -50})));
            Modelica.Mechanics.MultiBody.Parts.FixedTranslation fixedTranslation(animation = false, r = {0.8, 0, 0.3}) annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 90, origin = {-70, -50})));
            Joints.FreeMotionScalarInit freeMotionScalarInit(use_angle = true, use_angle_d = true, angle_2(start = 0, fixed = true), angle_d_2(start = 0, fixed = true)) annotation(
              Placement(transformation(extent = {{40, 60}, {20, 80}})));
          equation
            connect(fixedTranslation.frame_a, world.frame_b) annotation(
              Line(points = {{-70, -60}, {-70, -90}, {-80, -90}}, color = {95, 95, 95}, thickness = 0.5));
            connect(bodyOfConstraint.frame_b, springOfConstraint.frame_b) annotation(
              Line(points = {{-20, 20}, {-40, 20}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            connect(world.frame_b, fixedRotation.frame_a) annotation(
              Line(points = {{-80, -90}, {80, -90}, {80, -60}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            connect(fixedRotation.frame_b, constraint.frame_a) annotation(
              Line(points = {{80, -40}, {80, 20}, {60, 20}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            connect(constraint.frame_a, sensorConstraintRelative.frame_a) annotation(
              Line(points = {{60, 20}, {80, 20}, {80, 50}, {60, 50}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            connect(bodyOfJoint.frame_b, springOfJoint.frame_b) annotation(
              Line(points = {{-20, -20}, {-40, -20}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            connect(joint.frame_b, bodyOfJoint.frame_a) annotation(
              Line(points = {{40, -20}, {0, -20}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            connect(sensorConstraintRelative.frame_b, constraint.frame_b) annotation(
              Line(points = {{40, 50}, {30, 50}, {30, 20}, {40, 20}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            connect(fixedTranslation.frame_b, springOfJoint.frame_a) annotation(
              Line(points = {{-70, -40}, {-70, -20}, {-60, -20}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            connect(fixedTranslation.frame_b, springOfConstraint.frame_a) annotation(
              Line(points = {{-70, -40}, {-70, 20}, {-60, 20}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            connect(bodyOfConstraint.frame_a, constraint.frame_b) annotation(
              Line(points = {{0, 20}, {40, 20}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            connect(joint.frame_a, fixedRotation.frame_b) annotation(
              Line(points = {{60, -20}, {80, -20}, {80, -40}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            connect(freeMotionScalarInit.frame_a, fixedRotation.frame_b) annotation(
              Line(points = {{40, 70}, {80, 70}, {80, -40}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            connect(bodyOfConstraint.frame_a, freeMotionScalarInit.frame_b) annotation(
              Line(points = {{0, 20}, {8, 20}, {8, 70}, {20, 70}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            annotation(
              experiment(StopTime = 10),
              Documentation(info = "<html>
<p>This example demonstrates the functionality of <b>constraint</b> representing <b>revolute joint</b>. Each of two bodies is at one of its end connected by spring to the world. The other end is also connected to the world either by revolute joint or by appropriate constraint. Therefore, the body can only perform rotation about the revolute axis depending on working forces.</p>
<p><b>Simulation results</b> </p>
<p>After simulating the model, see the animation of the multibody system and compare movement of body connected by joint (blue colored) with movement of that one connected by constraint (of green color). Additionally, the outputs from <code>sensorConstraintRelative</code> depict both position and angle deviations in the constraining element.</p>
</html>"));
          end RevoluteConstraint;

          model SphericalConstraint "Body attached by one spring and spherical joint or constrained to environment"
            extends Modelica.Icons.Example;
            parameter Boolean animation = true "= true, if animation shall be enabled";
            Joints.Spherical joint(angles_fixed = true, w_rel_a_fixed = true, enforceStates = true) annotation(
              Placement(transformation(extent = {{60, -30}, {40, -10}})));
            Joints.Constraints.Spherical constraint annotation(
              Placement(transformation(extent = {{60, 10}, {40, 30}})));
            Modelica.Mechanics.MultiBody.Sensors.RelativeSensor sensorConstraintRelative(resolveInFrame = Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB.frame_a, get_r_rel = true, get_a_rel = false, get_angles = true) annotation(
              Placement(transformation(extent = {{60, 60}, {40, 40}})));
            Modelica.Mechanics.MultiBody.Parts.BodyShape bodyOfJoint(m = 1, I_11 = 1, I_22 = 1, I_33 = 1, r = {0.4, 0, 0}, r_CM = {0.2, 0, 0}, width = 0.05, r_0(start = {0.2, -0.5, 0.1}, each fixed = false), v_0(each fixed = false), angles_fixed = false, w_0_fixed = false, angles_start = {0.17453292519943, 0.95993108859688, 1.1868238913561}, final color = {0, 0, 255}) annotation(
              Placement(transformation(extent = {{-10, 10}, {10, -10}}, rotation = 180, origin = {-10, -20})));
            Modelica.Mechanics.MultiBody.Parts.BodyShape bodyOfConstraint(I_11 = 1, I_22 = 1, I_33 = 1, width = 0.05, r_0(start = {0.2, -0.5, 0.1}, each fixed = false), v_0(each fixed = false), angles_fixed = false, w_0_fixed = false, final color = {0, 128, 0}, r = bodyOfJoint.r, r_CM = bodyOfJoint.r_CM, m = bodyOfJoint.m, angles_start = {0.17453292519943, 0.95993108859688, 1.1868238913561}) annotation(
              Placement(transformation(extent = {{-10, 10}, {10, -10}}, rotation = 180, origin = {-10, 20})));
            Modelica.Mechanics.MultiBody.Forces.Spring springOfJoint(c = 20, s_unstretched = 0, width = 0.1, coilWidth = 0.005, numberOfWindings = 5) annotation(
              Placement(transformation(origin = {-50, -20}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Forces.Spring springOfConstraint(width = 0.1, coilWidth = 0.005, c = springOfJoint.c, s_unstretched = springOfJoint.s_unstretched, numberOfWindings = springOfJoint.numberOfWindings) annotation(
              Placement(transformation(origin = {-50, 20}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
            inner Modelica.Mechanics.MultiBody.World world annotation(
              Placement(transformation(extent = {{-100, -100}, {-80, -80}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Parts.FixedRotation fixedRotation(r = {0.2, -0.3, 0.2}, rotationType = Modelica.Mechanics.MultiBody.Types.RotationTypes.PlanarRotationSequence, angles = {10, 55, 68}) annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 90, origin = {80, -50})));
            Modelica.Mechanics.MultiBody.Parts.FixedTranslation fixedTranslation(animation = false, r = {0.8, 0, 0.3}) annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 90, origin = {-70, -50})));
            Joints.FreeMotionScalarInit freeMotionScalarInit(use_angle = true, use_angle_d = true, angle_1(fixed = true), angle_2(fixed = true), angle_3(fixed = true), angle_d_1(fixed = true), angle_d_2(fixed = true), angle_d_3(fixed = true)) annotation(
              Placement(transformation(extent = {{38, 60}, {18, 80}})));
          equation
            connect(fixedTranslation.frame_a, world.frame_b) annotation(
              Line(points = {{-70, -60}, {-70, -90}, {-80, -90}}, color = {95, 95, 95}, thickness = 0.5));
            connect(bodyOfConstraint.frame_b, springOfConstraint.frame_b) annotation(
              Line(points = {{-20, 20}, {-40, 20}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            connect(world.frame_b, fixedRotation.frame_a) annotation(
              Line(points = {{-80, -90}, {80, -90}, {80, -60}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            connect(fixedRotation.frame_b, constraint.frame_a) annotation(
              Line(points = {{80, -40}, {80, 20}, {60, 20}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            connect(constraint.frame_a, sensorConstraintRelative.frame_a) annotation(
              Line(points = {{60, 20}, {80, 20}, {80, 50}, {60, 50}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            connect(bodyOfJoint.frame_b, springOfJoint.frame_b) annotation(
              Line(points = {{-20, -20}, {-40, -20}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            connect(joint.frame_b, bodyOfJoint.frame_a) annotation(
              Line(points = {{40, -20}, {0, -20}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            connect(sensorConstraintRelative.frame_b, constraint.frame_b) annotation(
              Line(points = {{40, 50}, {30, 50}, {30, 20}, {40, 20}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            connect(fixedTranslation.frame_b, springOfJoint.frame_a) annotation(
              Line(points = {{-70, -40}, {-70, -20}, {-60, -20}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            connect(fixedTranslation.frame_b, springOfConstraint.frame_a) annotation(
              Line(points = {{-70, -40}, {-70, 20}, {-60, 20}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            connect(bodyOfConstraint.frame_a, constraint.frame_b) annotation(
              Line(points = {{0, 20}, {40, 20}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            connect(joint.frame_a, fixedRotation.frame_b) annotation(
              Line(points = {{60, -20}, {80, -20}, {80, -40}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            connect(bodyOfConstraint.frame_a, freeMotionScalarInit.frame_b) annotation(
              Line(points = {{0, 20}, {8, 20}, {8, 70}, {18, 70}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            connect(freeMotionScalarInit.frame_a, fixedRotation.frame_b) annotation(
              Line(points = {{38, 70}, {80, 70}, {80, -40}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            annotation(
              experiment(StopTime = 10),
              Documentation(info = "<html>
<p>This example demonstrates the functionality of <b>constraint</b> representing <b>spherical joint</b>. Each of two bodies is at one of its end connected by spring to the world. The other end is also connected to the world either by spherical joint or by appropriate constraint. Therefore, the body can only perform spherical movement depending on working forces.</p>
<p><b>Simulation results</b> </p>
<p>After simulating the model, see the animation of the multibody system and compare movement of body connected by joint (blue colored) with movement of that one connected by constraint (of green color). Additionally, the outputs from <code>sensorConstraintRelative</code> depict position deviations in the constraining element.</p>
</html>"));
          end SphericalConstraint;

          model UniversalConstraint "Body attached by one spring and universal joint or constrained to environment"
            extends Modelica.Icons.Example;
            parameter Boolean animation = true "= true, if animation shall be enabled";
            Joints.Universal joint(n_a = {0, 0, 1}, n_b = {1, 0, 0}, stateSelect = StateSelect.always, phi_a(fixed = true), phi_b(fixed = true), w_a(fixed = true), w_b(fixed = true)) annotation(
              Placement(transformation(extent = {{60, -30}, {40, -10}})));
            Joints.Constraints.Universal constraint(n_a = joint.n_a, n_b = joint.n_b) annotation(
              Placement(transformation(extent = {{60, 10}, {40, 30}})));
            Modelica.Mechanics.MultiBody.Sensors.RelativeSensor sensorConstraintRelative(resolveInFrame = Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB.frame_a, get_r_rel = true, get_a_rel = false, get_angles = true, sequence = {3, 2, 1}) annotation(
              Placement(transformation(extent = {{60, 60}, {40, 40}})));
            Modelica.Mechanics.MultiBody.Parts.BodyShape bodyOfJoint(m = 1, I_11 = 1, I_22 = 1, I_33 = 1, r = {0.4, 0, 0}, r_CM = {0.2, 0, 0}, width = 0.05, r_0(start = {0.2, -0.5, 0.1}, each fixed = false), v_0(each fixed = false), angles_fixed = false, w_0_fixed = false, angles_start = {0.17453292519943, 0.95993108859688, 1.1868238913561}, final color = {0, 0, 255}) annotation(
              Placement(transformation(extent = {{-10, 10}, {10, -10}}, rotation = 180, origin = {10, -20})));
            Modelica.Mechanics.MultiBody.Parts.BodyShape bodyOfConstraint(I_11 = 1, I_22 = 1, I_33 = 1, width = 0.05, r_0(start = {0.2, -0.5, 0.1}, each fixed = false), v_0(each fixed = false), angles_fixed = false, w_0_fixed = false, final color = {0, 128, 0}, r = bodyOfJoint.r, r_CM = bodyOfJoint.r_CM, m = bodyOfJoint.m, angles_start = {0.17453292519943, 0.95993108859688, 1.1868238913561}) annotation(
              Placement(transformation(extent = {{-10, 10}, {10, -10}}, rotation = 180, origin = {10, 20})));
            Modelica.Mechanics.MultiBody.Forces.Spring springOfJoint(c = 20, s_unstretched = 0, width = 0.1, coilWidth = 0.005, numberOfWindings = 5) annotation(
              Placement(transformation(origin = {-50, -20}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Forces.Spring springOfConstraint(width = 0.1, coilWidth = 0.005, c = springOfJoint.c, s_unstretched = springOfJoint.s_unstretched, numberOfWindings = springOfJoint.numberOfWindings) annotation(
              Placement(transformation(origin = {-50, 20}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
            inner Modelica.Mechanics.MultiBody.World world annotation(
              Placement(transformation(extent = {{-100, -100}, {-80, -80}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Parts.FixedRotation fixedRotation(r = {0.2, -0.3, 0.2}, rotationType = Modelica.Mechanics.MultiBody.Types.RotationTypes.PlanarRotationSequence, angles = {10, 55, 68}) annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 90, origin = {80, -50})));
            Modelica.Mechanics.MultiBody.Parts.FixedTranslation fixedTranslation(animation = false, r = {0.8, 0, 0.3}) annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 90, origin = {-70, -50})));
            Parts.FixedTranslation fixedTranslationOfJoint(r = {0.1, 0.15, 0.2}) annotation(
              Placement(transformation(extent = {{-10, -30}, {-30, -10}})));
            Parts.FixedTranslation fixedTranslationOfConstraint(r = fixedTranslationOfJoint.r) annotation(
              Placement(transformation(extent = {{-10, 10}, {-30, 30}})));
            Joints.FreeMotionScalarInit freeMotionScalarInit(use_angle = true, use_angle_d = true, angle_1(fixed = true), angle_3(fixed = true), angle_d_1(fixed = true), angle_d_3(fixed = true), sequence_start = {1, 2, 3}) annotation(
              Placement(transformation(extent = {{60, 70}, {40, 90}})));
          equation
            connect(fixedTranslation.frame_a, world.frame_b) annotation(
              Line(points = {{-70, -60}, {-70, -90}, {-80, -90}}, color = {95, 95, 95}, thickness = 0.5));
            connect(world.frame_b, fixedRotation.frame_a) annotation(
              Line(points = {{-80, -90}, {80, -90}, {80, -60}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            connect(fixedRotation.frame_b, constraint.frame_a) annotation(
              Line(points = {{80, -40}, {80, 20}, {60, 20}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            connect(constraint.frame_a, sensorConstraintRelative.frame_a) annotation(
              Line(points = {{60, 20}, {80, 20}, {80, 50}, {60, 50}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            connect(joint.frame_b, bodyOfJoint.frame_a) annotation(
              Line(points = {{40, -20}, {20, -20}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            connect(sensorConstraintRelative.frame_b, constraint.frame_b) annotation(
              Line(points = {{40, 50}, {30, 50}, {30, 20}, {40, 20}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            connect(fixedTranslation.frame_b, springOfJoint.frame_a) annotation(
              Line(points = {{-70, -40}, {-70, -20}, {-60, -20}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            connect(fixedTranslation.frame_b, springOfConstraint.frame_a) annotation(
              Line(points = {{-70, -40}, {-70, 20}, {-60, 20}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            connect(bodyOfConstraint.frame_a, constraint.frame_b) annotation(
              Line(points = {{20, 20}, {40, 20}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            connect(joint.frame_a, fixedRotation.frame_b) annotation(
              Line(points = {{60, -20}, {80, -20}, {80, -40}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            connect(bodyOfJoint.frame_b, fixedTranslationOfJoint.frame_a) annotation(
              Line(points = {{0, -20}, {-10, -20}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            connect(bodyOfConstraint.frame_b, fixedTranslationOfConstraint.frame_a) annotation(
              Line(points = {{0, 20}, {-10, 20}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            connect(springOfJoint.frame_b, fixedTranslationOfJoint.frame_b) annotation(
              Line(points = {{-40, -20}, {-30, -20}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            connect(springOfConstraint.frame_b, fixedTranslationOfConstraint.frame_b) annotation(
              Line(points = {{-40, 20}, {-30, 20}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            connect(fixedRotation.frame_b, freeMotionScalarInit.frame_a) annotation(
              Line(points = {{80, -40}, {80, 80}, {60, 80}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            connect(bodyOfConstraint.frame_a, freeMotionScalarInit.frame_b) annotation(
              Line(points = {{20, 20}, {30, 20}, {30, 80}, {40, 80}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
            annotation(
              experiment(StopTime = 10),
              Documentation(info = "<html>
<p>This example demonstrates the functionality of <b>constraint</b> representing <b>universal joint</b>. Each of two bodies is at one of its end connected by spring to the world. The other end is also connected to the world either by universal joint or by appropriate constraint. Therefore, the body can only perform rotation about two revolute axes depending on working forces.</p>
<p><b>Simulation results</b> </p>
<p>After simulating the model, see the animation of the multibody system and compare movement of body connected by joint (blue colored) with movement of that one connected by constraint (of green color). Additionally, the outputs from <code>sensorConstraintRelative</code> depict position deviations in the constraining element.</p>
</html>"));
          end UniversalConstraint;
          annotation(
            Documentation(info = "<html>
<p>This package is a collection of simulatable models involving constraints in a multibody system.
  The examples mainly show comparison of constraints to the standard joints.</p>

</html>"));
        end Constraints;

        package Systems "Examples of complete system models including 3-dimensional mechanics"
          extends Modelica.Icons.ExamplesPackage;

          package RobotR3 "Library to demonstrate robot system models based on the Manutec r3 robot"
            model oneAxis "Model of one axis of robot (controller, motor, gearbox) with simple load"
              extends Modelica.Icons.Example;
              parameter SI.Mass mLoad(min = 0) = 15 "Mass of load";
              parameter Real kp = 5 "Gain of position controller of axis 2";
              parameter Real ks = 0.5 "Gain of speed controller of axis 2";
              parameter SI.Time Ts = 0.05 "Time constant of integrator of speed controller of axis 2";
              parameter Real startAngle(unit = "deg") = 0 "Start angle of axis 2";
              parameter Real endAngle(unit = "deg") = 120 "End angle of axis 2";
              parameter SI.Time swingTime = 0.5 "Additional time after reference motion is in rest before simulation is stopped";
              parameter SI.AngularVelocity refSpeedMax = 3 "Maximum reference speed";
              parameter SI.AngularAcceleration refAccMax = 10 "Maximum reference acceleration";
              Components.AxisType1 axis(w = 5500, ratio = 210, c = 8, cd = 0.01, Rv0 = 0.5, Rv1 = 0.1 / 130, kp = kp, ks = ks, Ts = Ts) annotation(
                Placement(transformation(extent = {{20, 0}, {40, 20}}, rotation = 0)));
              Modelica.Mechanics.Rotational.Components.Inertia load(J = 1.3 * mLoad) annotation(
                Placement(transformation(extent = {{54, 0}, {74, 20}}, rotation = 0)));
              Components.PathPlanning1 pathPlanning(swingTime = swingTime, angleBegDeg = startAngle, angleEndDeg = endAngle, speedMax = refSpeedMax, accMax = refAccMax) annotation(
                Placement(transformation(extent = {{-60, 0}, {-40, 20}}, rotation = 0)));
            protected
              Components.ControlBus controlBus annotation(
                Placement(transformation(extent = {{-32, 10}, {8, 50}}, rotation = 0)));
            equation
              connect(axis.flange, load.flange_a) annotation(
                Line(points = {{40, 10}, {54, 10}}, color = {128, 128, 128}, thickness = 0.5));
              connect(pathPlanning.controlBus, controlBus) annotation(
                Line(points = {{-40, 10}, {-15, 10}, {-15, 28}, {-12, 28}, {-12, 30}}, color = {255, 204, 51}, thickness = 0.5));
              connect(controlBus.axisControlBus1, axis.axisControlBus) annotation(
                Text(string = "%first", index = -1, extent = [-6, 3; -6, 3]),
                Line(points = {{-12, 30}, {-12, 29}, {-9, 29}, {-9, 10}, {20, 10}}, color = {255, 204, 51}, thickness = 0.5));
              annotation(
                Documentation(info = "<HTML>
<p>
With this model one axis of the r3 robot is checked.
The mechanical structure is replaced by a simple
load inertia.
</p>
</html>"),
                experiment(StopTime = 1.6),
                __Dymola_Commands(file = "modelica://Modelica/Resources/Scripts/Dymola/Mechanics/MultiBody/Examples/Systems/oneAxisPlot.mos" "Plot result"));
            end oneAxis;

            model fullRobot "6 degree of freedom robot with path planning, controllers, motors, brakes, gears and mechanics"
              extends Modelica.Icons.Example;
              parameter SI.Mass mLoad(min = 0) = 15 "Mass of load";
              parameter SI.Position rLoad[3] = {0.1, 0.25, 0.1} "Distance from last flange to load mass";
              parameter SI.Acceleration g = 9.81 "Gravity acceleration";
              parameter SI.Time refStartTime = 0 "Start time of reference motion";
              parameter SI.Time refSwingTime = 0.5 "Additional time after reference motion is in rest before simulation is stopped";
              parameter Real startAngle1(unit = "deg") = -60 "Start angle of axis 1" annotation(
                Dialog(tab = "Reference", group = "startAngles"));
              parameter Real startAngle2(unit = "deg") = 20 "Start angle of axis 2" annotation(
                Dialog(tab = "Reference", group = "startAngles"));
              parameter Real startAngle3(unit = "deg") = 90 "Start angle of axis 3" annotation(
                Dialog(tab = "Reference", group = "startAngles"));
              parameter Real startAngle4(unit = "deg") = 0 "Start angle of axis 4" annotation(
                Dialog(tab = "Reference", group = "startAngles"));
              parameter Real startAngle5(unit = "deg") = -110 "Start angle of axis 5" annotation(
                Dialog(tab = "Reference", group = "startAngles"));
              parameter Real startAngle6(unit = "deg") = 0 "Start angle of axis 6" annotation(
                Dialog(tab = "Reference", group = "startAngles"));
              parameter Real endAngle1(unit = "deg") = 60 "End angle of axis 1" annotation(
                Dialog(tab = "Reference", group = "endAngles"));
              parameter Real endAngle2(unit = "deg") = -70 "End angle of axis 2" annotation(
                Dialog(tab = "Reference", group = "endAngles"));
              parameter Real endAngle3(unit = "deg") = -35 "End angle of axis 3" annotation(
                Dialog(tab = "Reference", group = "endAngles"));
              parameter Real endAngle4(unit = "deg") = 45 "End angle of axis 4" annotation(
                Dialog(tab = "Reference", group = "endAngles"));
              parameter Real endAngle5(unit = "deg") = 110 "End angle of axis 5" annotation(
                Dialog(tab = "Reference", group = "endAngles"));
              parameter Real endAngle6(unit = "deg") = 45 "End angle of axis 6" annotation(
                Dialog(tab = "Reference", group = "endAngles"));
              parameter SI.AngularVelocity refSpeedMax[6] = {3, 1.5, 5, 3.1, 3.1, 4.1} "Maximum reference speeds of all joints" annotation(
                Dialog(tab = "Reference", group = "Limits"));
              parameter SI.AngularAcceleration refAccMax[6] = {15, 15, 15, 60, 60, 60} "Maximum reference accelerations of all joints" annotation(
                Dialog(tab = "Reference", group = "Limits"));
              parameter Real kp1 = 5 "Gain of position controller" annotation(
                Dialog(tab = "Controller", group = "Axis 1"));
              parameter Real ks1 = 0.5 "Gain of speed controller" annotation(
                Dialog(tab = "Controller", group = "Axis 1"));
              parameter SI.Time Ts1 = 0.05 "Time constant of integrator of speed controller" annotation(
                Dialog(tab = "Controller", group = "Axis 1"));
              parameter Real kp2 = 5 "Gain of position controller" annotation(
                Dialog(tab = "Controller", group = "Axis 2"));
              parameter Real ks2 = 0.5 "Gain of speed controller" annotation(
                Dialog(tab = "Controller", group = "Axis 2"));
              parameter SI.Time Ts2 = 0.05 "Time constant of integrator of speed controller" annotation(
                Dialog(tab = "Controller", group = "Axis 2"));
              parameter Real kp3 = 5 "Gain of position controller" annotation(
                Dialog(tab = "Controller", group = "Axis 3"));
              parameter Real ks3 = 0.5 "Gain of speed controller" annotation(
                Dialog(tab = "Controller", group = "Axis 3"));
              parameter SI.Time Ts3 = 0.05 "Time constant of integrator of speed controller" annotation(
                Dialog(tab = "Controller", group = "Axis 3"));
              parameter Real kp4 = 5 "Gain of position controller" annotation(
                Dialog(tab = "Controller", group = "Axis 4"));
              parameter Real ks4 = 0.5 "Gain of speed controller" annotation(
                Dialog(tab = "Controller", group = "Axis 4"));
              parameter SI.Time Ts4 = 0.05 "Time constant of integrator of speed controller" annotation(
                Dialog(tab = "Controller", group = "Axis 4"));
              parameter Real kp5 = 5 "Gain of position controller" annotation(
                Dialog(tab = "Controller", group = "Axis 5"));
              parameter Real ks5 = 0.5 "Gain of speed controller" annotation(
                Dialog(tab = "Controller", group = "Axis 5"));
              parameter SI.Time Ts5 = 0.05 "Time constant of integrator of speed controller" annotation(
                Dialog(tab = "Controller", group = "Axis 5"));
              parameter Real kp6 = 5 "Gain of position controller" annotation(
                Dialog(tab = "Controller", group = "Axis 6"));
              parameter Real ks6 = 0.5 "Gain of speed controller" annotation(
                Dialog(tab = "Controller", group = "Axis 6"));
              parameter SI.Time Ts6 = 0.05 "Time constant of integrator of speed controller" annotation(
                Dialog(tab = "Controller", group = "Axis 6"));
              Components.MechanicalStructure mechanics(mLoad = mLoad, rLoad = rLoad, g = g) annotation(
                Placement(transformation(extent = {{35, -35}, {95, 25}}, rotation = 0)));
              Modelica.Mechanics.MultiBody.Examples.Systems.RobotR3.Components.PathPlanning6 pathPlanning(naxis = 6, angleBegDeg = {startAngle1, startAngle2, startAngle3, startAngle4, startAngle5, startAngle6}, angleEndDeg = {endAngle1, endAngle2, endAngle3, endAngle4, endAngle5, endAngle6}, speedMax = refSpeedMax, accMax = refAccMax, startTime = refStartTime, swingTime = refSwingTime) annotation(
                Placement(transformation(extent = {{-5, 50}, {-25, 70}}, rotation = 0)));
              RobotR3.Components.AxisType1 axis1(w = 4590, ratio = -105, c = 43, cd = 0.005, Rv0 = 0.4, Rv1 = 0.13 / 160, kp = kp1, ks = ks1, Ts = Ts1) annotation(
                Placement(transformation(extent = {{-25, -75}, {-5, -55}}, rotation = 0)));
              RobotR3.Components.AxisType1 axis2(w = 5500, ratio = 210, c = 8, cd = 0.01, Rv1 = 0.1 / 130, Rv0 = 0.5, kp = kp2, ks = ks2, Ts = Ts2) annotation(
                Placement(transformation(extent = {{-25, -55}, {-5, -35}}, rotation = 0)));
              RobotR3.Components.AxisType1 axis3(w = 5500, ratio = 60, c = 58, cd = 0.04, Rv0 = 0.7, Rv1 = 0.2 / 130, kp = kp3, ks = ks3, Ts = Ts3) annotation(
                Placement(transformation(extent = {{-25, -35}, {-5, -15}}, rotation = 0)));
              RobotR3.Components.AxisType2 axis4(k = 0.2365, w = 6250, D = 0.55, J = 1.6e-4, ratio = -99, Rv0 = 21.8, Rv1 = 9.8, peak = 26.7 / 21.8, kp = kp4, ks = ks4, Ts = Ts4) annotation(
                Placement(transformation(extent = {{-25, -15}, {-5, 5}}, rotation = 0)));
              RobotR3.Components.AxisType2 axis5(k = 0.2608, w = 6250, D = 0.55, J = 1.8e-4, ratio = 79.2, Rv0 = 30.1, Rv1 = 0.03, peak = 39.6 / 30.1, kp = kp5, ks = ks5, Ts = Ts5) annotation(
                Placement(transformation(extent = {{-25, 5}, {-5, 25}}, rotation = 0)));
              RobotR3.Components.AxisType2 axis6(k = 0.0842, w = 7400, D = 0.27, J = 4.3e-5, ratio = -99, Rv0 = 10.9, Rv1 = 3.92, peak = 16.8 / 10.9, kp = kp6, ks = ks6, Ts = Ts6) annotation(
                Placement(transformation(extent = {{-25, 25}, {-5, 45}}, rotation = 0)));
            protected
              Components.ControlBus controlBus annotation(
                Placement(transformation(origin = {-80, -10}, extent = {{-20, -20}, {20, 20}}, rotation = 90)));
            equation
              connect(axis2.flange, mechanics.axis2) annotation(
                Line(points = {{-5, -45}, {25, -45}, {25, -21.5}, {33.5, -21.5}}, color = {0, 0, 0}));
              connect(axis1.flange, mechanics.axis1) annotation(
                Line(points = {{-5, -65}, {30, -65}, {30, -30.5}, {33.5, -30.5}}, color = {0, 0, 0}));
              connect(axis3.flange, mechanics.axis3) annotation(
                Line(points = {{-5, -25}, {15, -25}, {15, -12.5}, {33.5, -12.5}}, color = {0, 0, 0}));
              connect(axis4.flange, mechanics.axis4) annotation(
                Line(points = {{-5, -5}, {15, -5}, {15, -3.5}, {33.5, -3.5}}, color = {0, 0, 0}));
              connect(axis5.flange, mechanics.axis5) annotation(
                Line(points = {{-5, 15}, {10, 15}, {10, 5.5}, {33.5, 5.5}}, color = {0, 0, 0}));
              connect(axis6.flange, mechanics.axis6) annotation(
                Line(points = {{-5, 35}, {20, 35}, {20, 14.5}, {33.5, 14.5}}, color = {0, 0, 0}));
              connect(controlBus, pathPlanning.controlBus) annotation(
                Line(points = {{-80, -10}, {-80, 60}, {-25, 60}}, color = {255, 204, 51}, thickness = 0.5));
              connect(controlBus.axisControlBus1, axis1.axisControlBus) annotation(
                Text(string = "%first", index = -1, extent = [-6, 3; -6, 3]),
                Line(points = {{-80, -10}, {-80, -14.5}, {-79, -14.5}, {-79, -17}, {-65, -17}, {-65, -65}, {-25, -65}}, color = {255, 204, 51}, thickness = 0.5));
              connect(controlBus.axisControlBus2, axis2.axisControlBus) annotation(
                Text(string = "%first", index = -1, extent = [-6, 3; -6, 3]),
                Line(points = {{-80, -10}, {-79, -10}, {-79, -15}, {-62.5, -15}, {-62.5, -45}, {-25, -45}}, color = {255, 204, 51}, thickness = 0.5));
              connect(controlBus.axisControlBus3, axis3.axisControlBus) annotation(
                Text(string = "%first", index = -1, extent = [-6, 3; -6, 3]),
                Line(points = {{-80, -10}, {-77, -10}, {-77, -12.5}, {-61, -12.5}, {-61, -25}, {-25, -25}}, color = {255, 204, 51}, thickness = 0.5));
              connect(controlBus.axisControlBus4, axis4.axisControlBus) annotation(
                Text(string = "%first", index = -1, extent = [-6, 3; -6, 3]),
                Line(points = {{-80, -10}, {-60.5, -10}, {-60.5, -5}, {-25, -5}}, color = {255, 204, 51}, thickness = 0.5));
              connect(controlBus.axisControlBus5, axis5.axisControlBus) annotation(
                Text(string = "%first", index = -1, extent = [-6, 3; -6, 3]),
                Line(points = {{-80, -10}, {-77, -10}, {-77, -7}, {-63, -7}, {-63, 15}, {-25, 15}}, color = {255, 204, 51}, thickness = 0.5));
              connect(controlBus.axisControlBus6, axis6.axisControlBus) annotation(
                Text(string = "%first", index = -1, extent = [-6, 3; -6, 3]),
                Line(points = {{-80, -10}, {-79, -10}, {-79, -5}, {-65, -5}, {-65, 35}, {-25, 35}}, color = {255, 204, 51}, thickness = 0.5));
              annotation(
                experiment(StopTime = 2),
                __Dymola_Commands(file = "modelica://Modelica/Resources/Scripts/Dymola/Mechanics/MultiBody/Examples/Systems/Run.mos" "Simulate", file = "modelica://Modelica/Resources/Scripts/Dymola/Mechanics/MultiBody/Examples/Systems/fullRobotPlot.mos" "Plot result of axis 3 + animate"),
                Documentation(info = "<HTML>
<p>
This is a detailed model of the robot. For animation CAD data
is used. Translate and simulate with the default settings
(default simulation time = 3 s). Use command script \"modelica://Modelica/Resources/Scripts/Dymola/Mechanics/MultiBody/Examples/Systems/fullRobotPlot.mos\"
to plot variables.
</p>

<p>
<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Examples/Systems/r3_fullRobot.png\" ALT=\"model Examples.Loops.Systems.RobotR3.fullRobot\">
</p>
</HTML>"));
            end fullRobot;

            extends Modelica.Icons.ExamplesPackage;

            package Components "Library of components of the robot"
              extends Modelica.Icons.Package;

              expandable connector AxisControlBus "Data bus for one robot axis"
                extends Modelica.Icons.SignalSubBus;
                Boolean motion_ref "= true, if reference motion is not in rest" annotation(
                  HideResult = false);
                SI.Angle angle_ref "Reference angle of axis flange" annotation(
                  HideResult = false);
                SI.Angle angle "Angle of axis flange" annotation(
                  HideResult = false);
                SI.AngularVelocity speed_ref "Reference speed of axis flange" annotation(
                  HideResult = false);
                SI.AngularVelocity speed "Speed of axis flange" annotation(
                  HideResult = false);
                SI.AngularAcceleration acceleration_ref "Reference acceleration of axis flange" annotation(
                  HideResult = false);
                SI.AngularAcceleration acceleration "Acceleration of axis flange" annotation(
                  HideResult = false);
                SI.Current current_ref "Reference current of motor" annotation(
                  HideResult = false);
                SI.Current current "Current of motor" annotation(
                  HideResult = false);
                SI.Angle motorAngle "Angle of motor flange" annotation(
                  HideResult = false);
                SI.AngularVelocity motorSpeed "Speed of motor flange" annotation(
                  HideResult = false);
                annotation(
                  defaultComponentPrefixes = "protected",
                  Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-20, 2}, {22, -2}}, lineColor = {255, 204, 51}, lineThickness = 0.5)}),
                  Documentation(info = "<html>
<p>
Signal bus that is used to communicate all signals for <b>one</b> axis.
This is an expandable connector which has a \"default\" set of
signals. Note, the input/output causalities of the signals are
determined from the connections to this bus.
</p>

</html>"));
              end AxisControlBus;

              expandable connector ControlBus "Data bus for all axes of robot"
                extends Modelica.Icons.SignalBus;
                Modelica.Mechanics.MultiBody.Examples.Systems.RobotR3.Components.AxisControlBus axisControlBus1 "Bus of axis 1";
                Modelica.Mechanics.MultiBody.Examples.Systems.RobotR3.Components.AxisControlBus axisControlBus2 "Bus of axis 2";
                Modelica.Mechanics.MultiBody.Examples.Systems.RobotR3.Components.AxisControlBus axisControlBus3 "Bus of axis 3";
                Modelica.Mechanics.MultiBody.Examples.Systems.RobotR3.Components.AxisControlBus axisControlBus4 "Bus of axis 4";
                Modelica.Mechanics.MultiBody.Examples.Systems.RobotR3.Components.AxisControlBus axisControlBus5 "Bus of axis 5";
                Modelica.Mechanics.MultiBody.Examples.Systems.RobotR3.Components.AxisControlBus axisControlBus6 "Bus of axis 6";
                annotation(
                  Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-20, 2}, {22, -2}}, lineColor = {255, 204, 51}, lineThickness = 0.5)}),
                  Documentation(info = "<html>
<p>
Signal bus that is used to communicate <b>all signals</b> of the robot.
This is an expandable connector which has a \"default\" set of
signals. Note, the input/output causalities of the signals are
determined from the connections to this bus.
</p>
</html>"));
              end ControlBus;

              model PathPlanning1 "Generate reference angles for fastest kinematic movement"
                parameter Real angleBegDeg(unit = "deg") = 0 "Start angle";
                parameter Real angleEndDeg(unit = "deg") = 1 "End angle";
                parameter SI.AngularVelocity speedMax = 3 "Maximum axis speed";
                parameter SI.AngularAcceleration accMax = 2.5 "Maximum axis acceleration";
                parameter SI.Time startTime = 0 "Start time of movement";
                parameter SI.Time swingTime = 0.5 "Additional time after reference motion is in rest before simulation is stopped";
                final parameter SI.Angle angleBeg = Cv.from_deg(angleBegDeg) "Start angles";
                final parameter SI.Angle angleEnd = Cv.from_deg(angleEndDeg) "End angles";
                ControlBus controlBus annotation(
                  Placement(transformation(origin = {100, 0}, extent = {{-20, -20}, {20, 20}}, rotation = 270)));
                Modelica.Blocks.Sources.KinematicPTP2 path(q_end = {angleEnd}, qd_max = {speedMax}, qdd_max = {accMax}, startTime = startTime, q_begin = {angleBeg}) annotation(
                  Placement(transformation(extent = {{-50, -10}, {-30, 10}}, rotation = 0)));
                PathToAxisControlBus pathToAxis1(final nAxis = 1, final axisUsed = 1) annotation(
                  Placement(transformation(extent = {{0, -10}, {20, 10}}, rotation = 0)));
                Blocks.Logical.TerminateSimulation terminateSimulation(condition = time >= path.endTime + swingTime) annotation(
                  Placement(transformation(extent = {{-50, -30}, {30, -24}}, rotation = 0)));
              equation
                connect(path.q, pathToAxis1.q) annotation(
                  Line(points = {{-29, 8}, {-2, 8}}, color = {0, 0, 127}));
                connect(path.qd, pathToAxis1.qd) annotation(
                  Line(points = {{-29, 3}, {-2, 3}}, color = {0, 0, 127}));
                connect(path.qdd, pathToAxis1.qdd) annotation(
                  Line(points = {{-29, -3}, {-2, -3}}, color = {0, 0, 127}));
                connect(path.moving, pathToAxis1.moving) annotation(
                  Line(points = {{-29, -8}, {-2, -8}}, color = {255, 0, 255}));
                connect(pathToAxis1.axisControlBus, controlBus.axisControlBus1) annotation(
                  Text(string = "%second", index = 1, extent = [6, 3; 6, 3]),
                  Line(points = {{20, 0}, {100, 0}}, color = {255, 204, 51}, thickness = 0.5));
                annotation(
                  Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, 100}, {100, -100}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-150, 150}, {150, 110}}, textString = "%name", lineColor = {0, 0, 255}), Polygon(points = {{-80, 90}, {-88, 68}, {-72, 68}, {-80, 88}, {-80, 90}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-80, 78}, {-80, -82}}, color = {192, 192, 192}), Line(points = {{-90, 0}, {82, 0}}, color = {192, 192, 192}), Polygon(points = {{90, 0}, {68, 8}, {68, -8}, {90, 0}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Text(extent = {{-42, 55}, {29, 12}}, lineColor = {192, 192, 192}, textString = "w"), Line(points = {{-80, 0}, {-41, 69}, {26, 69}, {58, 0}}, color = {0, 0, 0}), Text(extent = {{-73, -44}, {82, -69}}, lineColor = {0, 0, 0}, textString = "1 axis")}),
                  Documentation(info = "<html>
<p>
Given
</p>
<ul>
<li> start and end angle of an axis</li>
<li> maximum speed of the axis </li>
<li> maximum acceleration of the axis </li>
</ul>

<p>
this component computes the fastest movement under the
given constraints. This means, that:
</p>

<ol>
<li> The axis accelerates with the maximum acceleration
     until the maximum speed is reached.</li>
<li> Drives with the maximum speed as long as possible.</li>
<li> Decelerates with the negative of the maximum acceleration
     until rest.</li>
</ol>

<p>
The acceleration, constant velocity and deceleration
phase are determined in such a way that the movement
starts form the start angles and ends at the end angles.
The output of this block are the computed angles, angular velocities
and angular acceleration and this information is stored as reference
motion on the controlBus of the r3 robot.
</p>

</html>"));
              end PathPlanning1;

              model PathPlanning6 "Generate reference angles for fastest kinematic movement"
                parameter Integer naxis = 6 "number of driven axis";
                parameter Real angleBegDeg[naxis](each unit = "deg") = zeros(naxis) "Start angles";
                parameter Real angleEndDeg[naxis](each unit = "deg") = ones(naxis) "End angles";
                parameter SI.AngularVelocity speedMax[naxis] = fill(3, naxis) "Maximum axis speed";
                parameter SI.AngularAcceleration accMax[naxis] = fill(2.5, naxis) "Maximum axis acceleration";
                parameter SI.Time startTime = 0 "Start time of movement";
                parameter SI.Time swingTime = 0.5 "Additional time after reference motion is in rest before simulation is stopped";
                final parameter SI.Angle angleBeg[:] = Cv.from_deg(angleBegDeg) "Start angles";
                final parameter SI.Angle angleEnd[:] = Cv.from_deg(angleEndDeg) "End angles";
                ControlBus controlBus annotation(
                  Placement(transformation(origin = {100, 0}, extent = {{-20, -20}, {20, 20}}, rotation = 270)));
                Modelica.Blocks.Sources.KinematicPTP2 path(q_end = angleEnd, qd_max = speedMax, qdd_max = accMax, startTime = startTime, q_begin = angleBeg) annotation(
                  Placement(transformation(extent = {{-90, -80}, {-70, -60}}, rotation = 0)));
                PathToAxisControlBus pathToAxis1(nAxis = naxis, axisUsed = 1) annotation(
                  Placement(transformation(extent = {{-10, 70}, {10, 90}}, rotation = 0)));
                PathToAxisControlBus pathToAxis2(nAxis = naxis, axisUsed = 2) annotation(
                  Placement(transformation(extent = {{-10, 40}, {10, 60}}, rotation = 0)));
                PathToAxisControlBus pathToAxis3(nAxis = naxis, axisUsed = 3) annotation(
                  Placement(transformation(extent = {{-10, 10}, {10, 30}}, rotation = 0)));
                PathToAxisControlBus pathToAxis4(nAxis = naxis, axisUsed = 4) annotation(
                  Placement(transformation(extent = {{-10, -20}, {10, 0}}, rotation = 0)));
                PathToAxisControlBus pathToAxis5(nAxis = naxis, axisUsed = 5) annotation(
                  Placement(transformation(extent = {{-10, -50}, {10, -30}}, rotation = 0)));
                PathToAxisControlBus pathToAxis6(nAxis = naxis, axisUsed = 6) annotation(
                  Placement(transformation(extent = {{-10, -80}, {10, -60}}, rotation = 0)));
                Blocks.Logical.TerminateSimulation terminateSimulation(condition = time >= path.endTime + swingTime) annotation(
                  Placement(transformation(extent = {{-50, -100}, {30, -94}}, rotation = 0)));
              equation
                connect(path.q, pathToAxis1.q) annotation(
                  Line(points = {{-69, -62}, {-60, -62}, {-60, 88}, {-12, 88}}, color = {0, 0, 127}));
                connect(path.qd, pathToAxis1.qd) annotation(
                  Line(points = {{-69, -67}, {-59, -67}, {-59, 83}, {-12, 83}}, color = {0, 0, 127}));
                connect(path.qdd, pathToAxis1.qdd) annotation(
                  Line(points = {{-69, -73}, {-58, -73}, {-58, 77}, {-12, 77}}, color = {0, 0, 127}));
                connect(path.moving, pathToAxis1.moving) annotation(
                  Line(points = {{-69, -78}, {-57, -78}, {-57, 72}, {-12, 72}}, color = {255, 0, 255}));
                connect(path.q, pathToAxis2.q) annotation(
                  Line(points = {{-69, -62}, {-60, -62}, {-60, 58}, {-12, 58}}, color = {0, 0, 127}));
                connect(path.qd, pathToAxis2.qd) annotation(
                  Line(points = {{-69, -67}, {-59, -67}, {-59, 53}, {-12, 53}}, color = {0, 0, 127}));
                connect(path.qdd, pathToAxis2.qdd) annotation(
                  Line(points = {{-69, -73}, {-58, -73}, {-58, 47}, {-12, 47}}, color = {0, 0, 127}));
                connect(path.moving, pathToAxis2.moving) annotation(
                  Line(points = {{-69, -78}, {-57, -78}, {-57, 42}, {-12, 42}}, color = {255, 0, 255}));
                connect(path.q, pathToAxis3.q) annotation(
                  Line(points = {{-69, -62}, {-60, -62}, {-60, 28}, {-12, 28}}, color = {0, 0, 127}));
                connect(path.qd, pathToAxis3.qd) annotation(
                  Line(points = {{-69, -67}, {-59, -67}, {-59, 23}, {-12, 23}}, color = {0, 0, 127}));
                connect(path.qdd, pathToAxis3.qdd) annotation(
                  Line(points = {{-69, -73}, {-58, -73}, {-58, 17}, {-12, 17}}, color = {0, 0, 127}));
                connect(path.moving, pathToAxis3.moving) annotation(
                  Line(points = {{-69, -78}, {-57, -78}, {-57, 12}, {-12, 12}}, color = {255, 0, 255}));
                connect(path.q, pathToAxis4.q) annotation(
                  Line(points = {{-69, -62}, {-60, -62}, {-60, -2}, {-12, -2}}, color = {0, 0, 127}));
                connect(path.qd, pathToAxis4.qd) annotation(
                  Line(points = {{-69, -67}, {-59, -67}, {-59, -7}, {-12, -7}}, color = {0, 0, 127}));
                connect(path.qdd, pathToAxis4.qdd) annotation(
                  Line(points = {{-69, -73}, {-58, -73}, {-58, -13}, {-12, -13}}, color = {0, 0, 127}));
                connect(path.moving, pathToAxis4.moving) annotation(
                  Line(points = {{-69, -78}, {-57, -78}, {-57, -18}, {-12, -18}}, color = {255, 0, 255}));
                connect(path.q, pathToAxis5.q) annotation(
                  Line(points = {{-69, -62}, {-60, -62}, {-60, -32}, {-12, -32}}, color = {0, 0, 127}));
                connect(path.qd, pathToAxis5.qd) annotation(
                  Line(points = {{-69, -67}, {-59, -67}, {-59, -37}, {-12, -37}}, color = {0, 0, 127}));
                connect(path.qdd, pathToAxis5.qdd) annotation(
                  Line(points = {{-69, -73}, {-58, -73}, {-58, -43}, {-12, -43}}, color = {0, 0, 127}));
                connect(path.moving, pathToAxis5.moving) annotation(
                  Line(points = {{-69, -78}, {-57, -78}, {-57, -48}, {-12, -48}}, color = {255, 0, 255}));
                connect(path.q, pathToAxis6.q) annotation(
                  Line(points = {{-69, -62}, {-12, -62}}, color = {0, 0, 127}));
                connect(path.qd, pathToAxis6.qd) annotation(
                  Line(points = {{-69, -67}, {-12, -67}}, color = {0, 0, 127}));
                connect(path.qdd, pathToAxis6.qdd) annotation(
                  Line(points = {{-69, -73}, {-12, -73}}, color = {0, 0, 127}));
                connect(path.moving, pathToAxis6.moving) annotation(
                  Line(points = {{-69, -78}, {-12, -78}}, color = {255, 0, 255}));
                connect(pathToAxis1.axisControlBus, controlBus.axisControlBus1) annotation(
                  Text(string = "%second", index = 1, extent = [6, 3; 6, 3]),
                  Line(points = {{10, 80}, {80, 80}, {80, 7}, {98, 7}}, color = {255, 204, 51}, thickness = 0.5));
                connect(pathToAxis2.axisControlBus, controlBus.axisControlBus2) annotation(
                  Text(string = "%second", index = 1, extent = [6, 3; 6, 3]),
                  Line(points = {{10, 50}, {77, 50}, {77, 5}, {97, 5}}, color = {255, 204, 51}, thickness = 0.5));
                connect(pathToAxis3.axisControlBus, controlBus.axisControlBus3) annotation(
                  Text(string = "%second", index = 1, extent = [6, 3; 6, 3]),
                  Line(points = {{10, 20}, {75, 20}, {75, 3}, {96, 3}}, color = {255, 204, 51}, thickness = 0.5));
                connect(pathToAxis4.axisControlBus, controlBus.axisControlBus4) annotation(
                  Text(string = "%second", index = 1, extent = [6, 3; 6, 3]),
                  Line(points = {{10, -10}, {73, -10}, {73, 0}, {100, 0}}, color = {255, 204, 51}, thickness = 0.5));
                connect(pathToAxis5.axisControlBus, controlBus.axisControlBus5) annotation(
                  Text(string = "%second", index = 1, extent = [6, 3; 6, 3]),
                  Line(points = {{10, -40}, {75, -40}, {75, -3}, {100, -3}, {100, 0}}, color = {255, 204, 51}, thickness = 0.5));
                connect(pathToAxis6.axisControlBus, controlBus.axisControlBus6) annotation(
                  Text(string = "%second", index = 1, extent = [6, 3; 6, 3]),
                  Line(points = {{10, -70}, {78, -70}, {78, -6}, {98, -6}}, color = {255, 204, 51}, thickness = 0.5));
                annotation(
                  Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, 100}, {100, -100}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-150, 150}, {150, 110}}, textString = "%name", lineColor = {0, 0, 255}), Polygon(points = {{-80, 90}, {-88, 68}, {-72, 68}, {-80, 88}, {-80, 90}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-80, 78}, {-80, -82}}, color = {192, 192, 192}), Line(points = {{-90, 0}, {82, 0}}, color = {192, 192, 192}), Polygon(points = {{90, 0}, {68, 8}, {68, -8}, {90, 0}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Text(extent = {{-42, 55}, {29, 12}}, lineColor = {192, 192, 192}, textString = "w"), Line(points = {{-80, 0}, {-41, 69}, {26, 69}, {58, 0}}, color = {0, 0, 0}), Text(extent = {{-70, -43}, {85, -68}}, lineColor = {0, 0, 0}, textString = "6 axes")}),
                  Documentation(info = "<html>
<p>
Given
</p>
<ul>
<li> start and end angles of every axis</li>
<li> maximum speed of every axis </li>
<li> maximum acceleration of every axis </li>
</ul>

<p>
this component computes the fastest movement under the
given constraints. This means, that:
</p>

<ol>
<li> Every axis accelerates with the maximum acceleration
     until the maximum speed is reached.</li>
<li> Drives with the maximum speed as long as possible.</li>
<li> Decelerates with the negative of the maximum acceleration
     until rest.</li>
</ol>

<p>
The acceleration, constant velocity and deceleration
phase are determined in such a way that the movement
starts form the start angles and ends at the end angles.
The output of this block are the computed angles, angular velocities
and angular acceleration and this information is stored as reference
motion on the controlBus of the r3 robot.
</p>

</html>"));
              end PathPlanning6;

              model PathToAxisControlBus "Map path planning to one axis control bus"
                extends Modelica.Blocks.Icons.Block;
                parameter Integer nAxis = 6 "Number of driven axis";
                parameter Integer axisUsed = 1 "Map path planning of axisUsed to axisControlBus";
                Blocks.Interfaces.RealInput q[nAxis] annotation(
                  Placement(transformation(extent = {{-140, 60}, {-100, 100}}, rotation = 0)));
                Blocks.Interfaces.RealInput qd[nAxis] annotation(
                  Placement(transformation(extent = {{-140, 10}, {-100, 50}}, rotation = 0)));
                Blocks.Interfaces.RealInput qdd[nAxis] annotation(
                  Placement(transformation(extent = {{-140, -50}, {-100, -10}}, rotation = 0)));
                AxisControlBus axisControlBus annotation(
                  Placement(transformation(origin = {100, 0}, extent = {{-20, -20}, {20, 20}}, rotation = 270)));
                Blocks.Routing.RealPassThrough q_axisUsed annotation(
                  Placement(transformation(extent = {{-40, 50}, {-20, 70}}, rotation = 0)));
                Blocks.Routing.RealPassThrough qd_axisUsed annotation(
                  Placement(transformation(extent = {{-40, 10}, {-20, 30}}, rotation = 0)));
                Blocks.Routing.RealPassThrough qdd_axisUsed annotation(
                  Placement(transformation(extent = {{-40, -30}, {-20, -10}}, rotation = 0)));
                Blocks.Interfaces.BooleanInput moving[nAxis] annotation(
                  Placement(transformation(extent = {{-140, -100}, {-100, -60}}, rotation = 0)));
                Blocks.Routing.BooleanPassThrough motion_ref_axisUsed annotation(
                  Placement(transformation(extent = {{-40, -70}, {-20, -50}}, rotation = 0)));
              equation
                connect(q_axisUsed.u, q[axisUsed]) annotation(
                  Line(points = {{-42, 60}, {-60, 60}, {-60, 80}, {-120, 80}}, color = {0, 0, 127}));
                connect(qd_axisUsed.u, qd[axisUsed]) annotation(
                  Line(points = {{-42, 20}, {-80, 20}, {-80, 30}, {-120, 30}}, color = {0, 0, 127}));
                connect(qdd_axisUsed.u, qdd[axisUsed]) annotation(
                  Line(points = {{-42, -20}, {-80, -20}, {-80, -30}, {-120, -30}}, color = {0, 0, 127}));
                connect(motion_ref_axisUsed.u, moving[axisUsed]) annotation(
                  Line(points = {{-42, -60}, {-60, -60}, {-60, -80}, {-120, -80}}, color = {255, 0, 255}));
                connect(motion_ref_axisUsed.y, axisControlBus.motion_ref) annotation(
                  Text(string = "%second", index = 1, extent = [6, 3; 6, 3]),
                  Line(points = {{-19, -60}, {44, -60}, {44, -8}, {102, -8}, {102, 0}, {100, 0}}, color = {255, 0, 255}));
                connect(qdd_axisUsed.y, axisControlBus.acceleration_ref) annotation(
                  Text(string = "%second", index = 1, extent = [6, 3; 6, 3]),
                  Line(points = {{-19, -20}, {40, -20}, {40, -4}, {98, -4}, {98, 0}, {100, 0}}, color = {0, 0, 127}));
                connect(qd_axisUsed.y, axisControlBus.speed_ref) annotation(
                  Text(string = "%second", index = 1, extent = [6, 3; 6, 3]),
                  Line(points = {{-19, 20}, {20, 20}, {20, 0}, {100, 0}}, color = {0, 0, 127}));
                connect(q_axisUsed.y, axisControlBus.angle_ref) annotation(
                  Text(string = "%second", index = 1, extent = [6, 3; 6, 3]),
                  Line(points = {{-19, 60}, {40, 60}, {40, 4}, {96, 4}}, color = {0, 0, 127}));
                annotation(
                  defaultComponentName = "pathToAxis1",
                  Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}, initialScale = 0.1), graphics = {Text(extent = {{-100, 98}, {-24, 68}}, lineColor = {0, 0, 0}, textString = "q"), Text(extent = {{-94, 46}, {-18, 16}}, lineColor = {0, 0, 0}, textString = "qd"), Text(extent = {{-96, -16}, {-20, -46}}, lineColor = {0, 0, 0}, textString = "qdd"), Text(extent = {{-2, 20}, {80, -18}}, lineColor = {0, 0, 0}, textString = "%axisUsed"), Text(extent = {{2, 52}, {76, 28}}, lineColor = {0, 0, 0}, textString = "axis"), Text(extent = {{-94, -70}, {32, -96}}, lineColor = {0, 0, 0}, textString = "moving")}));
              end PathToAxisControlBus;

              model GearType1 "Motor inertia and gearbox model for r3 joints 1,2,3"
                extends Modelica.Mechanics.Rotational.Interfaces.PartialTwoFlanges;
                parameter Real i = -105 "gear ratio";
                parameter Real c(unit = "N.m/rad") = 43 "Spring constant";
                parameter Real d(unit = "N.m.s/rad") = 0.005 "Damper constant";
                parameter SI.Torque Rv0 = 0.4 "Viscous friction torque at zero velocity";
                parameter Real Rv1(unit = "N.m.s/rad") = 0.13 / 160 "Viscous friction coefficient (R=Rv0+Rv1*abs(qd))";
                parameter Real peak = 1 "Maximum static friction torque is peak*Rv0 (peak >= 1)";
                SI.AngularAcceleration a_rel = der(spring.w_rel) "Relative angular acceleration of spring";
                constant SI.AngularVelocity unitAngularVelocity = 1;
                constant SI.Torque unitTorque = 1;
                Modelica.Mechanics.Rotational.Components.IdealGear gear(ratio = i, useSupport = false) annotation(
                  Placement(transformation(extent = {{50, -10}, {70, 10}}, rotation = 0)));
                Modelica.Mechanics.Rotational.Components.SpringDamper spring(c = c, d = d) annotation(
                  Placement(transformation(extent = {{0, -10}, {20, 10}}, rotation = 0)));
                Modelica.Mechanics.Rotational.Components.BearingFriction bearingFriction(tau_pos = [0, Rv0 / unitTorque; 1, (Rv0 + Rv1 * unitAngularVelocity) / unitTorque], useSupport = false) annotation(
                  Placement(transformation(extent = {{-60, -10}, {-40, 10}}, rotation = 0)));
              equation
                connect(spring.flange_b, gear.flange_a) annotation(
                  Line(points = {{20, 0}, {50, 0}}, color = {128, 128, 128}, thickness = 0.5));
                connect(bearingFriction.flange_b, spring.flange_a) annotation(
                  Line(points = {{-40, 0}, {0, 0}}, color = {128, 128, 128}, thickness = 0.5));
                connect(gear.flange_b, flange_b) annotation(
                  Line(points = {{70, 0}, {100, 0}}, color = {128, 128, 128}, thickness = 0.5));
                connect(bearingFriction.flange_a, flange_a) annotation(
                  Line(points = {{-60, 0}, {-100, 0}}, color = {128, 128, 128}, thickness = 0.5));
              initial equation
                spring.w_rel = 0;
                a_rel = 0;
                annotation(
                  Documentation(info = "<html>
<p>
Models the gearbox used in the first three joints with all its effects,
like elasticity and friction.
Coulomb friction is approximated by a friction element acting
at the \"motor\"-side. In reality, bearing friction should be
also incorporated at the driven side of the gearbox. However,
this would require considerable more effort for the measurement
of the friction parameters.
Default values for all parameters are given for joint 1.
Model relativeStates is used to define the relative angle
and relative angular velocity across the spring (=gear elasticity)
as state variables. The reason is, that a default initial
value of zero of these states makes always sense.
If the absolute angle and the absolute angular velocity of model
Jmotor would be used as states, and the load angle (= joint angle of
robot) is NOT zero, one has always to ensure that the initial values
of the motor angle and of the joint angle are modified correspondingly.
Otherwise, the spring has an unrealistic deflection at initial time.
Since relative quantities are used as state variables, this simplifies
the definition of initial values considerably.
</p>
</html>"),
                  Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(lineColor = {64, 64, 64}, fillColor = {255, 255, 255}, fillPattern = FillPattern.HorizontalCylinder, extent = {{-90.0, -10.0}, {-60.0, 10.0}}), Polygon(fillColor = {192, 192, 192}, fillPattern = FillPattern.HorizontalCylinder, points = {{-60.0, 10.0}, {-60.0, 20.0}, {-40.0, 40.0}, {-40.0, -40.0}, {-60.0, -20.0}, {-60.0, 10.0}}), Rectangle(lineColor = {64, 64, 64}, fillColor = {255, 255, 255}, fillPattern = FillPattern.HorizontalCylinder, extent = {{-40.0, -60.0}, {40.0, 60.0}}, radius = 10.0), Polygon(fillColor = {192, 192, 192}, fillPattern = FillPattern.HorizontalCylinder, points = {{60.0, 20.0}, {40.0, 40.0}, {40.0, -40.0}, {60.0, -20.0}, {60.0, 20.0}}), Rectangle(lineColor = {64, 64, 64}, fillColor = {255, 255, 255}, fillPattern = FillPattern.HorizontalCylinder, extent = {{60.0, -10.0}, {90.0, 10.0}}), Polygon(fillColor = {64, 64, 64}, fillPattern = FillPattern.Solid, points = {{-60.0, -90.0}, {-50.0, -90.0}, {-20.0, -30.0}, {20.0, -30.0}, {48.0, -90.0}, {60.0, -90.0}, {60.0, -100.0}, {-60.0, -100.0}, {-60.0, -90.0}}), Text(extent = {{0, 128}, {0, 68}}, textString = "%name", lineColor = {0, 0, 255}), Text(extent = {{-36, 40}, {36, -30}}, textString = "1", lineColor = {0, 0, 255})}),
                  Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{72, 30}, {130, 22}}, lineColor = {0, 0, 0}, textString = "flange of joint axis"), Text(extent = {{-128, 26}, {-70, 18}}, lineColor = {0, 0, 0}, textString = "flange of motor axis")}));
              end GearType1;

              model GearType2 "Motor inertia and gearbox model for r3 joints 4,5,6"
                extends Modelica.Mechanics.Rotational.Interfaces.PartialTwoFlanges;
                parameter Real i = -99 "Gear ratio";
                parameter SI.Torque Rv0 = 21.8 "Viscous friction torque at zero velocity";
                parameter Real Rv1 = 9.8 "Viscous friction coefficient in [Nms/rad] (R=Rv0+Rv1*abs(qd))";
                parameter Real peak = 26.7 / 21.8 "Maximum static friction torque is peak*Rv0 (peak >= 1)";
                constant SI.AngularVelocity unitAngularVelocity = 1;
                constant SI.Torque unitTorque = 1;
                Modelica.Mechanics.Rotational.Components.IdealGear gear(ratio = i, useSupport = false) annotation(
                  Placement(transformation(extent = {{-28, -10}, {-8, 10}}, rotation = 0)));
                Modelica.Mechanics.Rotational.Components.BearingFriction bearingFriction(tau_pos = [0, Rv0 / unitTorque; 1, (Rv0 + Rv1 * unitAngularVelocity) / unitTorque], peak = peak, useSupport = false) annotation(
                  Placement(transformation(extent = {{30, -10}, {50, 10}}, rotation = 0)));
              equation
                connect(gear.flange_b, bearingFriction.flange_a) annotation(
                  Line(points = {{-8, 0}, {30, 0}}, color = {128, 128, 128}, thickness = 0.5));
                connect(bearingFriction.flange_b, flange_b) annotation(
                  Line(points = {{50, 0}, {100, 0}}, color = {128, 128, 128}, thickness = 0.5));
                connect(gear.flange_a, flange_a) annotation(
                  Line(points = {{-28, 0}, {-100, 0}}, color = {128, 128, 128}, thickness = 0.5));
                annotation(
                  Documentation(info = "<html>
<p>
The elasticity and damping in the gearboxes of the outermost
three joints of the robot is neglected.
Default values for all parameters are given for joint 4.
</p>
</html>"),
                  Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(lineColor = {64, 64, 64}, fillColor = {255, 255, 255}, fillPattern = FillPattern.HorizontalCylinder, extent = {{-90.0, -10.0}, {-60.0, 10.0}}), Polygon(fillColor = {192, 192, 192}, fillPattern = FillPattern.HorizontalCylinder, points = {{-60.0, 10.0}, {-60.0, 20.0}, {-40.0, 40.0}, {-40.0, -40.0}, {-60.0, -20.0}, {-60.0, 10.0}}), Rectangle(lineColor = {64, 64, 64}, fillColor = {255, 255, 255}, fillPattern = FillPattern.HorizontalCylinder, extent = {{-40.0, -60.0}, {40.0, 60.0}}, radius = 10.0), Polygon(fillColor = {192, 192, 192}, fillPattern = FillPattern.HorizontalCylinder, points = {{60.0, 20.0}, {40.0, 40.0}, {40.0, -40.0}, {60.0, -20.0}, {60.0, 20.0}}), Rectangle(lineColor = {64, 64, 64}, fillColor = {255, 255, 255}, fillPattern = FillPattern.HorizontalCylinder, extent = {{60.0, -10.0}, {90.0, 10.0}}), Polygon(fillColor = {64, 64, 64}, fillPattern = FillPattern.Solid, points = {{-60.0, -90.0}, {-50.0, -90.0}, {-20.0, -30.0}, {20.0, -30.0}, {48.0, -90.0}, {60.0, -90.0}, {60.0, -100.0}, {-60.0, -100.0}, {-60.0, -90.0}}), Text(extent = {{0, 128}, {0, 68}}, textString = "%name", lineColor = {0, 0, 255}), Text(extent = {{-36, 40}, {38, -30}}, textString = "2", lineColor = {0, 0, 255})}));
              end GearType2;

              model Motor "Motor model including current controller of r3 motors"
                extends Modelica.Mechanics.MultiBody.Icons.MotorIcon;
                parameter SI.Inertia J(min = 0) = 0.0013 "Moment of inertia of motor";
                parameter Real k = 1.1616 "Gain of motor";
                parameter Real w = 4590 "Time constant of motor";
                parameter Real D = 0.6 "Damping constant of motor";
                parameter SI.AngularVelocity w_max = 315 "Maximum speed of motor";
                parameter SI.Current i_max = 9 "Maximum current of motor";
                Modelica.Mechanics.Rotational.Interfaces.Flange_b flange_motor annotation(
                  Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
                Modelica.Electrical.Analog.Sources.SignalVoltage Vs annotation(
                  Placement(transformation(origin = {-90, 0}, extent = {{-10, 10}, {10, -10}}, rotation = 270)));
                Modelica.Electrical.Analog.Ideal.IdealOpAmp diff annotation(
                  Placement(transformation(extent = {{-64, 15}, {-44, 35}}, rotation = 0)));
                Modelica.Electrical.Analog.Ideal.IdealOpAmp power annotation(
                  Placement(transformation(extent = {{16, 15}, {36, 35}}, rotation = 0)));
                Electrical.Analog.Basic.EMF emf(k = k, useSupport = false) annotation(
                  Placement(transformation(extent = {{46, -10}, {66, 10}}, rotation = 0)));
                Modelica.Electrical.Analog.Basic.Inductor La(L = 250 / (2 * D * w)) annotation(
                  Placement(transformation(origin = {56, 30}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
                Modelica.Electrical.Analog.Basic.Resistor Ra(R = 250) annotation(
                  Placement(transformation(origin = {56, 60}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
                Modelica.Electrical.Analog.Basic.Resistor Rd2(R = 100) annotation(
                  Placement(transformation(extent = {{-86, 22}, {-71, 38}}, rotation = 0)));
                Modelica.Electrical.Analog.Basic.Capacitor C(C = 0.004 * D / w) annotation(
                  Placement(transformation(extent = {{-14, 36}, {6, 56}}, rotation = 0)));
                Modelica.Electrical.Analog.Ideal.IdealOpAmp OpI annotation(
                  Placement(transformation(extent = {{-14, 10}, {6, 30}}, rotation = 0)));
                Modelica.Electrical.Analog.Basic.Resistor Rd1(R = 100) annotation(
                  Placement(transformation(extent = {{-63, 37}, {-48, 53}}, rotation = 0)));
                Modelica.Electrical.Analog.Basic.Resistor Ri(R = 10) annotation(
                  Placement(transformation(extent = {{-37, 17}, {-22, 33}}, rotation = 0)));
                Modelica.Electrical.Analog.Basic.Resistor Rp1(R = 200) annotation(
                  Placement(transformation(extent = {{17, 38}, {32, 54}}, rotation = 0)));
                Modelica.Electrical.Analog.Basic.Resistor Rp2(R = 50) annotation(
                  Placement(transformation(origin = {11, 72}, extent = {{-8, -7}, {8, 7}}, rotation = 90)));
                Modelica.Electrical.Analog.Basic.Resistor Rd4(R = 100) annotation(
                  Placement(transformation(extent = {{-55, -15}, {-40, 1}}, rotation = 0)));
                Modelica.Electrical.Analog.Sources.SignalVoltage hall2 annotation(
                  Placement(transformation(origin = {-70, -50}, extent = {{-10, 10}, {10, -10}}, rotation = 90)));
                Modelica.Electrical.Analog.Basic.Resistor Rd3(R = 100) annotation(
                  Placement(transformation(origin = {-70, -22}, extent = {{-8, -7}, {8, 7}}, rotation = 90)));
                Modelica.Electrical.Analog.Basic.Ground g1 annotation(
                  Placement(transformation(extent = {{-100, -37}, {-80, -17}}, rotation = 0)));
                Modelica.Electrical.Analog.Basic.Ground g2 annotation(
                  Placement(transformation(extent = {{-80, -91}, {-60, -71}}, rotation = 0)));
                Modelica.Electrical.Analog.Basic.Ground g3 annotation(
                  Placement(transformation(extent = {{-34, -27}, {-14, -7}}, rotation = 0)));
                Modelica.Electrical.Analog.Sensors.CurrentSensor hall1 annotation(
                  Placement(transformation(origin = {16, -50}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
                Modelica.Electrical.Analog.Basic.Ground g4 annotation(
                  Placement(transformation(extent = {{6, -84}, {26, -64}}, rotation = 0)));
                Modelica.Electrical.Analog.Basic.Ground g5 annotation(
                  Placement(transformation(origin = {11, 93}, extent = {{-10, -10}, {10, 10}}, rotation = 180)));
                Modelica.Mechanics.Rotational.Sensors.AngleSensor phi annotation(
                  Placement(transformation(origin = {76, -40}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
                Modelica.Mechanics.Rotational.Sensors.SpeedSensor speed annotation(
                  Placement(transformation(origin = {55, -40}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
                Modelica.Mechanics.Rotational.Components.Inertia Jmotor(J = J) annotation(
                  Placement(transformation(extent = {{70, -10}, {90, 10}}, rotation = 0)));
                Modelica.Mechanics.MultiBody.Examples.Systems.RobotR3.Components.AxisControlBus axisControlBus annotation(
                  Placement(transformation(extent = {{60, -120}, {100, -80}}, rotation = 0)));
                Blocks.Math.Gain convert1(k(unit = "V/A") = 1) annotation(
                  Placement(transformation(extent = {{-30, -56}, {-42, -44}}, rotation = 0)));
                Blocks.Math.Gain convert2(k(unit = "V/A") = 1) annotation(
                  Placement(transformation(extent = {{-30, -101}, {-42, -89}}, rotation = 0)));
              initial equation
// initialize motor in steady state
                der(C.v) = 0;
                der(La.i) = 0;
              equation
                connect(La.n, emf.p) annotation(
                  Line(points = {{56, 20}, {56, 15}, {56, 10}}));
                connect(Ra.n, La.p) annotation(
                  Line(points = {{56, 50}, {56, 40}}));
                connect(Rd2.n, diff.n1) annotation(
                  Line(points = {{-71, 30}, {-64, 30}}));
                connect(C.n, OpI.p2) annotation(
                  Line(points = {{6, 46}, {6, 20}}));
                connect(OpI.p2, power.p1) annotation(
                  Line(points = {{6, 20}, {16, 20}}));
                connect(Vs.p, Rd2.p) annotation(
                  Line(points = {{-90, 10}, {-90, 30}, {-86, 30}}));
                connect(diff.n1, Rd1.p) annotation(
                  Line(points = {{-64, 30}, {-68, 30}, {-68, 45}, {-63, 45}}));
                connect(Rd1.n, diff.p2) annotation(
                  Line(points = {{-48, 45}, {-44, 45}, {-44, 25}}));
                connect(diff.p2, Ri.p) annotation(
                  Line(points = {{-44, 25}, {-37, 25}}));
                connect(Ri.n, OpI.n1) annotation(
                  Line(points = {{-22, 25}, {-14, 25}}));
                connect(OpI.n1, C.p) annotation(
                  Line(points = {{-14, 25}, {-14, 46}}));
                connect(power.n1, Rp1.p) annotation(
                  Line(points = {{16, 30}, {11, 30}, {11, 46}, {17, 46}}));
                connect(power.p2, Rp1.n) annotation(
                  Line(points = {{36, 25}, {36, 46}, {32, 46}}));
                connect(Rp1.p, Rp2.p) annotation(
                  Line(points = {{17, 46}, {11, 46}, {11, 64}}));
                connect(power.p2, Ra.p) annotation(
                  Line(points = {{36, 25}, {42, 25}, {42, 80}, {56, 80}, {56, 70}}));
                connect(Rd3.p, hall2.p) annotation(
                  Line(points = {{-70, -30}, {-70, -60}}));
                connect(Rd3.n, diff.p1) annotation(
                  Line(points = {{-70, -14}, {-70, 20}, {-64, 20}}));
                connect(Rd3.n, Rd4.p) annotation(
                  Line(points = {{-70, -14}, {-70, -7}, {-55, -7}}));
                connect(Vs.n, g1.p) annotation(
                  Line(points = {{-90, -10}, {-90, -17}}));
                connect(g2.p, hall2.n) annotation(
                  Line(points = {{-70, -71}, {-70, -40}}));
                connect(Rd4.n, g3.p) annotation(
                  Line(points = {{-40, -7}, {-24, -7}}));
                connect(g3.p, OpI.p1) annotation(
                  Line(points = {{-24, -7}, {-24, 15}, {-14, 15}}));
                connect(g5.p, Rp2.n) annotation(
                  Line(points = {{11, 83}, {11, 81.5}, {11, 80}}));
                connect(emf.n, hall1.p) annotation(
                  Line(points = {{56, -10}, {56, -24}, {16, -24}, {16, -40}}));
                connect(hall1.n, g4.p) annotation(
                  Line(points = {{16, -60}, {16, -62}, {16, -64}}));
                connect(emf.flange, phi.flange) annotation(
                  Line(points = {{66, 0}, {66, -30}, {76, -30}}, pattern = LinePattern.Dot));
                connect(emf.flange, speed.flange) annotation(
                  Line(points = {{66, 0}, {66, -30}, {55, -30}}, pattern = LinePattern.Dot));
                connect(OpI.n2, power.n2) annotation(
                  Line(points = {{-4, 10}, {-4, 4}, {26, 4}, {26, 15}}));
                connect(OpI.p1, OpI.n2) annotation(
                  Line(points = {{-14, 15}, {-14, 10}, {-4, 10}}));
                connect(OpI.p1, diff.n2) annotation(
                  Line(points = {{-14, 15}, {-54, 15}}));
                connect(Jmotor.flange_b, flange_motor) annotation(
                  Line(points = {{90, 0}, {100, 0}}, color = {128, 128, 128}, thickness = 0.5));
                connect(phi.phi, axisControlBus.motorAngle) annotation(
                  Line(points = {{76, -51}, {76, -100}, {80, -100}}, color = {0, 0, 127}));
                connect(speed.w, axisControlBus.motorSpeed) annotation(
                  Line(points = {{55, -51}, {55, -95}, {80, -95}, {80, -100}}, color = {0, 0, 127}));
                connect(hall1.i, axisControlBus.current) annotation(
                  Line(points = {{6, -50}, {-10, -50}, {-10, -95}, {80, -95}, {80, -100}}, color = {0, 0, 127}));
                connect(hall1.i, convert1.u) annotation(
                  Line(points = {{6, -50}, {-28.8, -50}}, color = {0, 0, 127}));
                connect(convert1.y, hall2.v) annotation(
                  Line(points = {{-42.6, -50}, {-63, -50}}, color = {0, 0, 127}));
                connect(convert2.u, axisControlBus.current_ref) annotation(
                  Line(points = {{-28.8, -95}, {80, -95}, {80, -100}}, color = {0, 0, 127}));
                connect(convert2.y, Vs.v) annotation(
                  Line(points = {{-42.6, -95}, {-108, -95}, {-108, 0}, {-97, 0}}, color = {0, 0, 127}));
                connect(emf.flange, Jmotor.flange_a) annotation(
                  Line(points = {{66, 0}, {70, 0}}, color = {0, 0, 0}, smooth = Smooth.None));
                annotation(
                  Documentation(info = "<html>
<p>
Default values are given for the motor of joint 1.
The input of the motor is the desired current
(the actual current is proportional to the torque
produced by the motor).
</p>
</html>"),
                  Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{0, 120}, {0, 60}}, textString = "%name", lineColor = {0, 0, 255}), Line(points = {{80, -102}, {80, -10}}, color = {255, 204, 51}, thickness = 0.5)}));
              end Motor;

              model Controller "P-PI cascade controller for one axis"
                parameter Real kp = 10 "Gain of position controller";
                parameter Real ks = 1 "Gain of speed controller";
                parameter SI.Time Ts = 0.01 "Time constant of integrator of speed controller";
                parameter Real ratio = 1 "Gear ratio of gearbox";
                Modelica.Blocks.Math.Gain gain1(k = ratio) annotation(
                  Placement(transformation(extent = {{-70, 0}, {-50, 20}}, rotation = 0)));
                Modelica.Blocks.Continuous.PI PI(k = ks, T = Ts) annotation(
                  Placement(transformation(extent = {{60, 0}, {80, 20}}, rotation = 0)));
                Modelica.Blocks.Math.Feedback feedback1 annotation(
                  Placement(transformation(extent = {{-46, 0}, {-26, 20}}, rotation = 0)));
                Modelica.Blocks.Math.Gain P(k = kp) annotation(
                  Placement(transformation(extent = {{-16, 0}, {4, 20}}, rotation = 0)));
                Modelica.Blocks.Math.Add3 add3(k3 = -1) annotation(
                  Placement(transformation(extent = {{20, 0}, {40, 20}}, rotation = 0)));
                Modelica.Blocks.Math.Gain gain2(k = ratio) annotation(
                  Placement(transformation(extent = {{-60, 40}, {-40, 60}}, rotation = 0)));
                Modelica.Mechanics.MultiBody.Examples.Systems.RobotR3.Components.AxisControlBus axisControlBus annotation(
                  Placement(transformation(extent = {{-20, -120}, {20, -80}}, rotation = 0)));
              equation
                connect(gain1.y, feedback1.u1) annotation(
                  Line(points = {{-49, 10}, {-44, 10}}, color = {0, 0, 127}));
                connect(feedback1.y, P.u) annotation(
                  Line(points = {{-27, 10}, {-18, 10}}, color = {0, 0, 127}));
                connect(P.y, add3.u2) annotation(
                  Line(points = {{5, 10}, {18, 10}}, color = {0, 0, 127}));
                connect(gain2.y, add3.u1) annotation(
                  Line(points = {{-39, 50}, {10, 50}, {10, 18}, {18, 18}}, color = {0, 0, 127}));
                connect(add3.y, PI.u) annotation(
                  Line(points = {{41, 10}, {58, 10}}, color = {0, 0, 127}));
                connect(gain2.u, axisControlBus.speed_ref) annotation(
                  Line(points = {{-62, 50}, {-90, 50}, {-90, -100}, {0, -100}}, color = {0, 0, 127}));
                connect(gain1.u, axisControlBus.angle_ref) annotation(
                  Line(points = {{-72, 10}, {-80, 10}, {-80, -100}, {0, -100}}, color = {0, 0, 127}));
                connect(feedback1.u2, axisControlBus.motorAngle) annotation(
                  Line(points = {{-36, 2}, {-36, -100}, {0, -100}}, color = {0, 0, 127}));
                connect(add3.u3, axisControlBus.motorSpeed) annotation(
                  Line(points = {{18, 2}, {0, 2}, {0, -100}}, color = {0, 0, 127}));
                connect(PI.y, axisControlBus.current_ref) annotation(
                  Line(points = {{81, 10}, {90, 10}, {90, -100}, {0, -100}}, color = {0, 0, 127}));
                annotation(
                  Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, -100}, {100, 100}}, lineColor = {0, 0, 0}, pattern = LinePattern.Solid, lineThickness = 0.25, fillColor = {235, 235, 235}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-30, 54}, {30, 24}}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Polygon(points = {{-30, 40}, {-60, 50}, {-60, 30}, {-30, 40}}, lineColor = {0, 0, 255}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid), Line(points = {{-31, -41}, {-78, -41}, {-78, 39}, {-30, 39}}, color = {0, 0, 255}), Rectangle(extent = {{-30, -26}, {30, -56}}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Polygon(points = {{60, -32}, {30, -42}, {60, -52}, {60, -32}}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Line(points = {{30, 39}, {76, 39}, {76, -41}, {30, -41}}, color = {0, 0, 255}), Text(extent = {{-100, 150}, {100, 110}}, textString = "%name", lineColor = {0, 0, 255})}),
                  Documentation(info = "<html>
<p>
This controller has an inner PI-controller to control the motor speed,
and an outer P-controller to control the motor position of one axis.
The reference signals are with respect to the gear-output, and the
gear ratio is used in the controller to determine the motor
reference signals. All signals are communicated via the
\"axisControlBus\".
</p>
</html>"));
              end Controller;

              model AxisType1 "Axis model of the r3 joints 1,2,3"
                extends AxisType2(redeclare GearType1 gear(c = c, d = cd));
                parameter Real c(unit = "N.m/rad") = 43 "Spring constant" annotation(
                  Dialog(group = "Gear"));
                parameter Real cd(unit = "N.m.s/rad") = 0.005 "Damper constant" annotation(
                  Dialog(group = "Gear"));
              end AxisType1;

              model AxisType2 "Axis model of the r3 joints 4,5,6"
                parameter Real kp = 10 "Gain of position controller" annotation(
                  Dialog(group = "Controller"));
                parameter Real ks = 1 "Gain of speed controller" annotation(
                  Dialog(group = "Controller"));
                parameter SI.Time Ts = 0.01 "Time constant of integrator of speed controller" annotation(
                  Dialog(group = "Controller"));
                parameter Real k = 1.1616 "Gain of motor" annotation(
                  Dialog(group = "Motor"));
                parameter Real w = 4590 "Time constant of motor" annotation(
                  Dialog(group = "Motor"));
                parameter Real D = 0.6 "Damping constant of motor" annotation(
                  Dialog(group = "Motor"));
                parameter SI.Inertia J(min = 0) = 0.0013 "Moment of inertia of motor" annotation(
                  Dialog(group = "Motor"));
                parameter Real ratio = -105 "Gear ratio" annotation(
                  Dialog(group = "Gear"));
                parameter SI.Torque Rv0 = 0.4 "Viscous friction torque at zero velocity in [Nm]" annotation(
                  Dialog(group = "Gear"));
                parameter Real Rv1(unit = "N.m.s/rad") = 0.13 / 160 "Viscous friction coefficient in [Nms/rad]" annotation(
                  Dialog(group = "Gear"));
                parameter Real peak = 1 "Maximum static friction torque is peak*Rv0 (peak >= 1)" annotation(
                  Dialog(group = "Gear"));
                Modelica.Mechanics.Rotational.Interfaces.Flange_b flange annotation(
                  Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
                replaceable GearType2 gear(Rv0 = Rv0, Rv1 = Rv1, peak = peak, i = ratio) annotation(
                  Placement(transformation(extent = {{0, -10}, {20, 10}}, rotation = 0)));
                Motor motor(J = J, k = k, w = w, D = D) annotation(
                  Placement(transformation(extent = {{-30, -10}, {-10, 10}}, rotation = 0)));
                RobotR3.Components.Controller controller(kp = kp, ks = ks, Ts = Ts, ratio = ratio) annotation(
                  Placement(transformation(extent = {{-70, -10}, {-50, 10}}, rotation = 0)));
                Modelica.Mechanics.MultiBody.Examples.Systems.RobotR3.Components.AxisControlBus axisControlBus annotation(
                  Placement(transformation(origin = {-100, 0}, extent = {{-20, -20}, {20, 20}}, rotation = 270)));
                Modelica.Mechanics.Rotational.Sensors.AngleSensor angleSensor annotation(
                  Placement(transformation(extent = {{30, 60}, {50, 80}}, rotation = 0)));
                Modelica.Mechanics.Rotational.Sensors.SpeedSensor speedSensor annotation(
                  Placement(transformation(origin = {40, 50}, extent = {{10, 10}, {-10, -10}}, rotation = 180)));
                Modelica.Mechanics.Rotational.Sensors.AccSensor accSensor annotation(
                  Placement(transformation(extent = {{30, 20}, {50, 40}}, rotation = 0)));
                Modelica.Mechanics.Rotational.Components.InitializeFlange initializeFlange(stateSelect = StateSelect.prefer) annotation(
                  Placement(transformation(extent = {{-40, -60}, {-20, -40}}, rotation = 0)));
                Blocks.Sources.Constant const(k = 0) annotation(
                  Placement(transformation(extent = {{-65, -65}, {-55, -55}}, rotation = 0)));
              equation
                connect(gear.flange_b, flange) annotation(
                  Line(points = {{20, 0}, {100, 0}}, color = {0, 0, 0}));
                connect(gear.flange_b, angleSensor.flange) annotation(
                  Line(points = {{20, 0}, {20, 70}, {30, 70}}, color = {0, 0, 0}));
                connect(gear.flange_b, speedSensor.flange) annotation(
                  Line(points = {{20, 0}, {24, 0}, {24, 50}, {30, 50}}, color = {0, 0, 0}));
                connect(motor.flange_motor, gear.flange_a) annotation(
                  Line(points = {{-10, 0}, {0, 0}}, color = {0, 0, 0}));
                connect(gear.flange_b, accSensor.flange) annotation(
                  Line(points = {{20, 0}, {28, 0}, {28, 30}, {30, 30}}, color = {0, 0, 0}));
                connect(controller.axisControlBus, axisControlBus) annotation(
                  Line(points = {{-60, -10}, {-60, -20}, {-95, -20}, {-95, -4}, {-100, -4}, {-100, 0}}, color = {255, 204, 51}, thickness = 0.5));
                connect(motor.axisControlBus, axisControlBus) annotation(
                  Line(points = {{-12, -10}, {-12, -20}, {-95, -20}, {-95, -5}, {-100, -5}, {-100, 0}}, color = {255, 204, 51}, thickness = 0.5));
                connect(angleSensor.phi, axisControlBus.angle) annotation(
                  Text(string = "%second", index = 1, extent = [6, 3; 6, 3]),
                  Line(points = {{51, 70}, {70, 70}, {70, 84}, {-98, 84}, {-98, 9}, {-100, 9}, {-100, 0}}, color = {0, 0, 127}));
                connect(speedSensor.w, axisControlBus.speed) annotation(
                  Text(string = "%second", index = 1, extent = [6, 3; 6, 3]),
                  Line(points = {{51, 50}, {74, 50}, {74, 87}, {-100, 87}, {-100, 0}}, color = {0, 0, 127}));
                connect(accSensor.a, axisControlBus.acceleration) annotation(
                  Text(string = "%second", index = 1, extent = [6, 3; 6, 3]),
                  Line(points = {{51, 30}, {77, 30}, {77, 90}, {-102, 90}, {-102, 0}, {-100, 0}}, color = {0, 0, 127}));
                connect(axisControlBus.angle_ref, initializeFlange.phi_start) annotation(
                  Text(string = "%first", index = -1, extent = [-6, 3; -6, 3]),
                  Line(points = {{-100, 0}, {-100, -7}, {-97, -7}, {-97, -42}, {-42, -42}}, color = {0, 0, 0}));
                connect(axisControlBus.speed_ref, initializeFlange.w_start) annotation(
                  Text(string = "%first", index = -1, extent = [-6, 3; -6, 3]),
                  Line(points = {{-100, 0}, {-99, 0}, {-99, -50}, {-42, -50}}, color = {0, 0, 127}));
                connect(initializeFlange.flange, flange) annotation(
                  Line(points = {{-20, -50}, {80, -50}, {80, 0}, {100, 0}}, color = {0, 0, 0}));
                connect(const.y, initializeFlange.a_start) annotation(
                  Line(points = {{-54.5, -60}, {-48, -60}, {-48, -58}, {-42, -58}}, color = {0, 0, 127}));
                annotation(
                  Documentation(info = "<HTML>
<p>
The axis model consists of the <b>controller</b>, the <b>motor</b> including current
controller and the <b>gearbox</b> including gear elasticity and bearing friction.
The only difference to the axis model of joints 4,5,6 (= model axisType2) is
that elasticity and damping in the gear boxes are not neglected.
</p>
<p>
The input signals of this component are the desired angle and desired angular
velocity of the joint. The reference signals have to be \"smooth\" (position
has to be differentiable at least 2 times). Otherwise, the gear elasticity
leads to significant oscillations.
</p>
<p>
Default values of the parameters are given for the axis of joint 1.
</p>
</html>"),
                  Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, 50}, {100, -50}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {160, 160, 164}), Text(extent = {{-150, 57}, {150, 97}}, textString = "%name", lineColor = {0, 0, 255})}));
              end AxisType2;

              model MechanicalStructure "Model of the mechanical part of the r3 robot (without animation)"
                import Modelica.SIunits.Conversions.to_unit1;
                parameter Boolean animation = true "= true, if animation shall be enabled";
                parameter SI.Mass mLoad(min = 0) = 15 "Mass of load";
                parameter SI.Position rLoad[3] = {0, 0.25, 0} "Distance from last flange to load mass>";
                parameter SI.Acceleration g = 9.81 "Gravity acceleration";
                SI.Angle q[6] "Joint angles";
                SI.AngularVelocity qd[6] "Joint speeds";
                SI.AngularAcceleration qdd[6] "Joint accelerations";
                SI.Torque tau[6] "Joint driving torques";
                //r0={0,0.351,0},
                Modelica.Mechanics.Rotational.Interfaces.Flange_a axis1 annotation(
                  Placement(transformation(extent = {{-220, -180}, {-200, -160}}, rotation = 0)));
                Modelica.Mechanics.Rotational.Interfaces.Flange_a axis2 annotation(
                  Placement(transformation(extent = {{-220, -120}, {-200, -100}}, rotation = 0)));
                Modelica.Mechanics.Rotational.Interfaces.Flange_a axis3 annotation(
                  Placement(transformation(extent = {{-220, -60}, {-200, -40}}, rotation = 0)));
                Modelica.Mechanics.Rotational.Interfaces.Flange_a axis4 annotation(
                  Placement(transformation(extent = {{-220, 0}, {-200, 20}}, rotation = 0)));
                Modelica.Mechanics.Rotational.Interfaces.Flange_a axis5 annotation(
                  Placement(transformation(extent = {{-220, 60}, {-200, 80}}, rotation = 0)));
                Modelica.Mechanics.Rotational.Interfaces.Flange_a axis6 annotation(
                  Placement(transformation(extent = {{-220, 120}, {-200, 140}}, rotation = 0)));
                inner Modelica.Mechanics.MultiBody.World world(g = g * Modelica.Math.Vectors.length({0, -1, 0}), n = {0, -1, 0}, animateWorld = false, animateGravity = false, enableAnimation = animation) annotation(
                  Placement(transformation(extent = {{-100, -200}, {-80, -180}}, rotation = 0)));
                Modelica.Mechanics.MultiBody.Joints.Revolute r1(n = {0, 1, 0}, useAxisFlange = true, animation = animation) annotation(
                  Placement(transformation(origin = {-70, -160}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
                Modelica.Mechanics.MultiBody.Joints.Revolute r2(n = {1, 0, 0}, useAxisFlange = true, animation = animation) annotation(
                  Placement(transformation(extent = {{-50, -110}, {-30, -90}}, rotation = 0)));
                Modelica.Mechanics.MultiBody.Joints.Revolute r3(n = {1, 0, 0}, useAxisFlange = true, animation = animation) annotation(
                  Placement(transformation(origin = {-50, -36}, extent = {{-10, -10}, {10, 10}}, rotation = 180)));
                Modelica.Mechanics.MultiBody.Joints.Revolute r4(n = {0, 1, 0}, useAxisFlange = true, animation = animation) annotation(
                  Placement(transformation(origin = {-70, 10}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
                Modelica.Mechanics.MultiBody.Joints.Revolute r5(n = {1, 0, 0}, useAxisFlange = true, animation = animation) annotation(
                  Placement(transformation(extent = {{-60, 70}, {-40, 90}}, rotation = 0)));
                Modelica.Mechanics.MultiBody.Joints.Revolute r6(n = {0, 1, 0}, useAxisFlange = true, animation = animation) annotation(
                  Placement(transformation(origin = {-60, 130}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
                Modelica.Mechanics.MultiBody.Parts.BodyShape b0(r = {0, 0.351, 0}, shapeType = "modelica://Modelica/Resources/Data/Shapes/RobotR3/b0.dxf", r_shape = {0, 0, 0}, lengthDirection = {1, 0, 0}, widthDirection = {0, 1, 0}, length = 0.225, width = 0.3, height = 0.3, color = {0, 0, 255}, animation = animation, animateSphere = false, r_CM = {0, 0, 0}, m = 1) annotation(
                  Placement(transformation(origin = {-30, -170}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
                Modelica.Mechanics.MultiBody.Parts.BodyShape b1(r = {0, 0.324, 0.3}, I_22 = 1.16, shapeType = "modelica://Modelica/Resources/Data/Shapes/RobotR3/b1.dxf", lengthDirection = {1, 0, 0}, widthDirection = {0, 1, 0}, length = 0.25, width = 0.15, height = 0.2, animation = animation, animateSphere = false, color = {255, 0, 0}, r_CM = {0, 0, 0}, m = 1) annotation(
                  Placement(transformation(origin = {-70, -118}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
                Modelica.Mechanics.MultiBody.Parts.BodyShape b2(r = {0, 0.65, 0}, r_CM = {0.172, 0.205, 0}, m = 56.5, I_11 = 2.58, I_22 = 0.64, I_33 = 2.73, I_21 = -0.46, shapeType = "modelica://Modelica/Resources/Data/Shapes/RobotR3/b2.dxf", r_shape = {0, 0, 0}, lengthDirection = {1, 0, 0}, widthDirection = {0, 1, 0}, length = 0.5, width = 0.2, height = 0.15, animation = animation, animateSphere = false, color = {255, 178, 0}) annotation(
                  Placement(transformation(origin = {-16, -70}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
                Modelica.Mechanics.MultiBody.Parts.BodyShape b3(r = {0, 0.414, -0.155}, r_CM = {0.064, -0.034, 0}, m = 26.4, I_11 = 0.279, I_22 = 0.245, I_33 = 0.413, I_21 = -0.070, shapeType = "modelica://Modelica/Resources/Data/Shapes/RobotR3/b3.dxf", r_shape = {0, 0, 0}, lengthDirection = {1, 0, 0}, widthDirection = {0, 1, 0}, length = 0.15, width = 0.15, height = 0.15, animation = animation, animateSphere = false, color = {255, 0, 0}) annotation(
                  Placement(transformation(origin = {-86, -22}, extent = {{-10, 10}, {10, -10}}, rotation = 90)));
                Modelica.Mechanics.MultiBody.Parts.BodyShape b4(r = {0, 0.186, 0}, r_CM = {0, 0, 0}, m = 28.7, I_11 = 1.67, I_22 = 0.081, I_33 = 1.67, shapeType = "modelica://Modelica/Resources/Data/Shapes/RobotR3/b4.dxf", r_shape = {0, 0, 0}, lengthDirection = {1, 0, 0}, widthDirection = {0, 1, 0}, length = 0.73, width = 0.1, height = 0.1, animation = animation, animateSphere = false, color = {255, 178, 0}) annotation(
                  Placement(transformation(origin = {-70, 50}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
                Modelica.Mechanics.MultiBody.Parts.BodyShape b5(r = {0, 0.125, 0}, r_CM = {0, 0, 0}, m = 5.2, I_11 = 1.25, I_22 = 0.81, I_33 = 1.53, shapeType = "modelica://Modelica/Resources/Data/Shapes/RobotR3/b5.dxf", r_shape = {0, 0, 0}, lengthDirection = {1, 0, 0}, widthDirection = {0, 1, 0}, length = 0.225, width = 0.075, height = 0.1, animation = animation, animateSphere = false, color = {0, 0, 255}) annotation(
                  Placement(transformation(origin = {-20, 98}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
                Modelica.Mechanics.MultiBody.Parts.BodyShape b6(r = {0, 0, 0}, r_CM = {0.05, 0.05, 0.05}, m = 0.5, shapeType = "modelica://Modelica/Resources/Data/Shapes/RobotR3/b6.dxf", r_shape = {0, 0, 0}, lengthDirection = {1, 0, 0}, widthDirection = {0, 1, 0}, animation = animation, animateSphere = false, color = {0, 0, 255}) annotation(
                  Placement(transformation(origin = {-60, 160}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
                Modelica.Mechanics.MultiBody.Parts.BodyShape load(r = {0, 0, 0}, r_CM = rLoad, m = mLoad, r_shape = {0, 0, 0}, widthDirection = {1, 0, 0}, width = 0.05, height = 0.05, color = {255, 0, 0}, lengthDirection = to_unit1(rLoad), length = Modelica.Math.Vectors.length(rLoad), animation = animation) annotation(
                  Placement(transformation(origin = {-60, 188}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
              equation
                connect(r6.frame_b, b6.frame_a) annotation(
                  Line(points = {{-60, 140}, {-60, 150}}, color = {95, 95, 95}, thickness = 0.5));
                q = {r1.phi, r2.phi, r3.phi, r4.phi, r5.phi, r6.phi};
                qd = der(q);
                qdd = der(qd);
                tau = {r1.tau, r2.tau, r3.tau, r4.tau, r5.tau, r6.tau};
                connect(load.frame_a, b6.frame_b) annotation(
                  Line(points = {{-60, 178}, {-60, 170}}, color = {95, 95, 95}, thickness = 0.5));
                connect(world.frame_b, b0.frame_a) annotation(
                  Line(points = {{-80, -190}, {-30, -190}, {-30, -180}}, color = {95, 95, 95}, thickness = 0.5));
                connect(b0.frame_b, r1.frame_a) annotation(
                  Line(points = {{-30, -160}, {-30, -146}, {-48, -146}, {-48, -180}, {-70, -180}, {-70, -170}}, color = {95, 95, 95}, thickness = 0.5));
                connect(b1.frame_b, r2.frame_a) annotation(
                  Line(points = {{-70, -108}, {-70, -100}, {-50, -100}}, color = {95, 95, 95}, thickness = 0.5));
                connect(r1.frame_b, b1.frame_a) annotation(
                  Line(points = {{-70, -150}, {-70, -128}}, color = {95, 95, 95}, thickness = 0.5));
                connect(r2.frame_b, b2.frame_a) annotation(
                  Line(points = {{-30, -100}, {-16, -100}, {-16, -80}}, color = {95, 95, 95}, thickness = 0.5));
                connect(b2.frame_b, r3.frame_a) annotation(
                  Line(points = {{-16, -60}, {-16, -36}, {-40, -36}}, color = {95, 95, 95}, thickness = 0.5));
                connect(r2.axis, axis2) annotation(
                  Line(points = {{-40, -90}, {-42, -90}, {-42, -80}, {-160, -80}, {-160, -110}, {-210, -110}}, color = {0, 0, 0}));
                connect(r1.axis, axis1) annotation(
                  Line(points = {{-80, -160}, {-160, -160}, {-160, -170}, {-210, -170}}, color = {0, 0, 0}));
                connect(r3.frame_b, b3.frame_a) annotation(
                  Line(points = {{-60, -36}, {-88, -36}, {-86, -32}}, color = {95, 95, 95}, thickness = 0.5));
                connect(b3.frame_b, r4.frame_a) annotation(
                  Line(points = {{-86, -12}, {-86, -8}, {-70, -8}, {-70, 0}}, color = {95, 95, 95}, thickness = 0.5));
                connect(r3.axis, axis3) annotation(
                  Line(points = {{-50, -46}, {-50, -50}, {-210, -50}}, color = {0, 0, 0}));
                connect(r4.axis, axis4) annotation(
                  Line(points = {{-80, 10}, {-210, 10}}, color = {0, 0, 0}));
                connect(r4.frame_b, b4.frame_a) annotation(
                  Line(points = {{-70, 20}, {-70, 40}}, color = {95, 95, 95}, thickness = 0.5));
                connect(b4.frame_b, r5.frame_a) annotation(
                  Line(points = {{-70, 60}, {-70, 80}, {-60, 80}}, color = {95, 95, 95}, thickness = 0.5));
                connect(r5.axis, axis5) annotation(
                  Line(points = {{-50, 90}, {-50, 94}, {-160, 94}, {-160, 70}, {-210, 70}}, color = {0, 0, 0}));
                connect(r5.frame_b, b5.frame_a) annotation(
                  Line(points = {{-40, 80}, {-20, 80}, {-20, 88}}, color = {95, 95, 95}, thickness = 0.5));
                connect(b5.frame_b, r6.frame_a) annotation(
                  Line(points = {{-20, 108}, {-20, 116}, {-60, 116}, {-60, 120}}, color = {95, 95, 95}, thickness = 0.5));
                connect(r6.axis, axis6) annotation(
                  Line(points = {{-70, 130}, {-210, 130}}, color = {0, 0, 0}));
                annotation(
                  Documentation(info = "<HTML>
<p>
This model contains the mechanical components of the r3 robot
(multibody system).
</p>
</html>"),
                  Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-200, -200}, {200, 200}}), graphics = {Rectangle(extent = {{-200, 200}, {200, -200}}, lineColor = {0, 0, 0}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Text(extent = {{-200, 250}, {200, 210}}, textString = "%name", lineColor = {0, 0, 255}), Text(extent = {{-200, -150}, {-140, -190}}, textString = "1", lineColor = {0, 0, 255}), Text(extent = {{-200, -30}, {-140, -70}}, textString = "3", lineColor = {0, 0, 255}), Text(extent = {{-200, -90}, {-140, -130}}, textString = "2", lineColor = {0, 0, 255}), Text(extent = {{-200, 90}, {-140, 50}}, textString = "5", lineColor = {0, 0, 255}), Text(extent = {{-200, 28}, {-140, -12}}, textString = "4", lineColor = {0, 0, 255}), Text(extent = {{-198, 150}, {-138, 110}}, textString = "6", lineColor = {0, 0, 255}), Bitmap(extent = {{-130, -195}, {195, 195}}, fileName = "modelica://Modelica/Resources/Images/Mechanics/MultiBody/Examples/Systems/robot_kr15.png")}),
                  Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-200, -200}, {200, 200}})));
              end MechanicalStructure;
              annotation(
                Documentation(info = "<html>
<p>
This library contains the different components
of the r3 robot. Usually, there is no need to
use this library directly.
</p>
</html>"));
            end Components;
            annotation(
              Documentation(info = "<HTML>
<p>
This package contains models of the robot r3 of the company Manutec.
These models are used to demonstrate in which way complex
robot models might be built up by testing first the component
models individually before composing them together.
Furthermore, it is shown how CAD data can be used
for animation.
</p>

<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Examples/Systems/robot_kr15.png\"
ALT=\"model Examples.Systems.RobotR3\">

<p>
The following models are available:
</p>
<pre>
   <b>oneAxis</b>   Test one axis (controller, motor, gearbox).
   <b>fullRobot</b> Test complete robot model.
</pre>
<p>
The r3 robot is no longer manufactured. In fact the company
Manutec does no longer exist.
The parameters of this robot have been determined by measurements
in the laboratory of DLR. The measurement procedure is described in:
</p>
<pre>
   Tuerk S. (1990): Zur Modellierung der Dynamik von Robotern mit
       rotatorischen Gelenken. Fortschrittberichte VDI, Reihe 8, Nr. 211,
       VDI-Verlag 1990.
</pre>
<p>
The robot model is described in detail in
</p>
<pre>
   Otter M. (1995): Objektorientierte Modellierung mechatronischer
       Systeme am Beispiel geregelter Roboter. Dissertation,
       Fortschrittberichte VDI, Reihe 20, Nr. 147, VDI-Verlag 1995.
       This report can be downloaded as compressed postscript file
       from: <a href=\"http://www.robotic.dlr.de/Martin.Otter\">http://www.robotic.dlr.de/Martin.Otter</a>.
</pre>
<p>
The path planning is performed in a simple way by using essentially
the Modelica.Mechanics.Rotational.KinematicPTP block. A user defines
a path by start and end angle of every axis. A path is planned such
that all axes are moving as fast as possible under the given
restrictions of maximum joint speeds and maximum joint accelerations.
The actual r3 robot from Manutec had a different path planning strategy.
Todays path planning algorithms from robot companies are much
more involved.
</p>
<p>
In order to get a nice animation, CAD data from a KUKA robot
is used, since CAD data of the original r3 robot was not available.
The KUKA CAD data was derived from public data of KUKA available at:
<a href=\"http://www.kuka-roboter.de/english/produkte/cad/low_payloads.html\">
http://www.kuka-roboter.de/english/produkte/cad/low_payloads.html</a>.
Since dimensions of the corresponding KUKA robot are similar but not
identical to the r3 robot, the data of the r3 robot (such as arm lengths) have been modified, such that it matches the CAD data.
</p>
<p>
In this model, a simplified P-PI cascade controller for every
axes is used. The parameters have been manually adjusted by
simulations. The original r3 controllers are more complicated.
The reason to use simplified controllers is to have a simpler demo.
</p>
</html>"));
          end RobotR3;
          annotation(
            Documentation(info = "<html>
<p>
This package contains complete <b>system models</b> where components
from different domains are used, including 3-dimensional mechanics.
</p>
<h4>Content</h4>
<table border=1 cellspacing=0 cellpadding=2>
  <tr><th><b><i>Model</i></b></th><th><b><i>Description</i></b></th></tr>
  <tr><td valign=\"top\"><a href=\"modelica://Modelica.Mechanics.MultiBody.Examples.Systems.RobotR3\">RobotR3</a><br>
<a href=\"modelica://Modelica.Mechanics.MultiBody.Examples.Systems.RobotR3.oneAxis\">RobotR3.oneAxis</a><br>
<a href=\"modelica://Modelica.Mechanics.MultiBody.Examples.Systems.RobotR3.fullRobot\">RobotR3.fullRobot</a></td>
      <td valign=\"top\"> 6 degree of freedom robot with path planning,
           controllers, motors, brakes, gears and mechanics.
           \"oneAxis\" models only one drive train. \"fullRobot\" is
           the complete, detailed robot model.<br>
      <img src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Examples/Systems/r3_fullRobot_small.png\">
      </td>
  </tr>
</table>
</html>"));
        end Systems;
        annotation(
          Documentation(info = "<html>
<p>
This package contains example models to demonstrate the usage of the
MultiBody package. Open the models and
simulate them according to the provided description in the models.
</p>

</html>"));
      end Examples;

      package Forces "Components that exert forces and/or torques between frames"
        extends Modelica.Icons.SourcesPackage;

        model WorldForce "External force acting at frame_b, defined by 3 input signals and resolved in frame world, frame_b or frame_resolve"
          extends Interfaces.PartialOneFrame_b;
          Interfaces.Frame_resolve frame_resolve if resolveInFrame == Modelica.Mechanics.MultiBody.Types.ResolveInFrameB.frame_resolve "The input signals are optionally resolved in this frame" annotation(
            Placement(transformation(origin = {0, -100}, extent = {{-16, -16}, {16, 16}}, rotation = 270)));
          Modelica.Blocks.Interfaces.RealInput force[3](each final quantity = "Force", each final unit = "N") "x-, y-, z-coordinates of force resolved in frame defined by resolveInFrame" annotation(
            Placement(transformation(extent = {{-140, -20}, {-100, 20}}, rotation = 0)));
          parameter Boolean animation = true "= true, if animation shall be enabled";
          parameter Modelica.Mechanics.MultiBody.Types.ResolveInFrameB resolveInFrame = Modelica.Mechanics.MultiBody.Types.ResolveInFrameB.world "Frame in which input force is resolved (1: world, 2: frame_b, 3: frame_resolve)";
          parameter Real N_to_m(unit = "N/m") = world.defaultN_to_m "Force arrow scaling (length = force/N_to_m)" annotation(
            Dialog(group = "if animation = true", enable = animation));
          input SI.Diameter diameter = world.defaultArrowDiameter "Diameter of force arrow" annotation(
            Dialog(group = "if animation = true", enable = animation));
          input Types.Color color = Modelica.Mechanics.MultiBody.Types.Defaults.ForceColor "Color of arrow" annotation(
            Dialog(colorSelector = true, group = "if animation = true", enable = animation));
          input Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(
            Dialog(group = "if animation = true", enable = animation));
        protected
          SI.Position f_in_m[3] = frame_b.f / N_to_m "Force mapped from N to m for animation";
          Visualizers.Advanced.Arrow arrow(diameter = diameter, color = color, specularCoefficient = specularCoefficient, R = frame_b.R, r = frame_b.r_0, r_tail = f_in_m, r_head = -f_in_m) if world.enableAnimation and animation;
        public
          Internal.BasicWorldForce basicWorldForce(resolveInFrame = resolveInFrame) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}})));
        protected
          Interfaces.ZeroPosition zeroPosition if not resolveInFrame == Modelica.Mechanics.MultiBody.Types.ResolveInFrameB.frame_resolve annotation(
            Placement(transformation(extent = {{20, -40}, {40, -20}})));
        equation
          connect(basicWorldForce.frame_b, frame_b) annotation(
            Line(points = {{10, 0}, {100, 0}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
          connect(basicWorldForce.force, force) annotation(
            Line(points = {{-12, 0}, {-120, 0}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(basicWorldForce.frame_resolve, frame_resolve) annotation(
            Line(points = {{0, -10}, {0, -100}}, color = {95, 95, 95}, pattern = LinePattern.Dot, smooth = Smooth.None));
          connect(zeroPosition.frame_resolve, basicWorldForce.frame_resolve) annotation(
            Line(points = {{20, -30}, {0, -30}, {0, -10}}, color = {95, 95, 95}, pattern = LinePattern.Dot, smooth = Smooth.None));
          annotation(
            defaultComponentName = "force",
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-100, -46}, {100, -76}}, lineColor = {192, 192, 192}, textString = "resolve"), Polygon(points = {{-100, 10}, {50, 10}, {50, 31}, {94, 0}, {50, -31}, {50, -10}, {-100, -10}, {-100, 10}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Text(extent = {{-150, 85}, {150, 45}}, textString = "%name", lineColor = {0, 0, 255}), Line(points = {{0, -10}, {0, -95}}, color = {95, 95, 95}, pattern = LinePattern.Dot)}),
            Documentation(info = "<HTML>

<p>
The <b>3</b> signals of the <b>force</b> connector are interpreted
as the x-, y- and z-coordinates of a <b>force</b> acting at the frame
connector to which frame_b of this component is attached.
Via parameter <b>resolveInFrame</b> it is defined, in which frame these
coordinates shall be resolved:
</p>

<table border=1 cellspacing=0 cellpadding=2>
<tr><th><b>Types.ResolveInFrameB.</b></th><th><b>Meaning</b></th></tr>
<tr><td valign=\"top\">world</td>
    <td valign=\"top\">Resolve input force in world frame (= default)</td></tr>

<tr><td valign=\"top\">frame_b</td>
    <td valign=\"top\">Resolve input force in frame_b</td></tr>

<tr><td valign=\"top\">frame_resolve</td>
    <td valign=\"top\">Resolve input force in frame_resolve (frame_resolve must be connected)</td></tr>
</table>

<p>
If resolveInFrame = Types.ResolveInFrameB.frame_resolve, the force coordinates
are with respect to the frame, that is connected to <b>frame_resolve</b>.
</p>

<p>
If force={100,0,0}, and for all parameters the default setting is used,
then the interpretation is that a force of 100 N is acting along the positive
x-axis of frame_b.
</p>

<p>
Note, the cut-torque in frame_b (frame_b.t) is always set to zero.
Conceptually, a force and torque acts on the world frame in such a way that
the force and torque balance between world.frame_b and frame_b is fulfilled.
For efficiency reasons, this reaction torque is, however, not computed.
</p>

<p>
This force component is by default visualized as an arrow
acting at the connector to which it is connected. The diameter
and color of the arrow can be defined via
variables <b>diameter</b> and <b>color</b>. The arrow
points in the direction defined by the
force signal. The length of the arrow is proportional
to the length of the force vector using parameter
<b>N_to_m</b> as scaling factor. For example, if N_to_m = 100 N/m,
then a force of 350 N is displayed as an arrow of length 3.5 m.
</p>
<p>
An example how to use this model is given in the
following figure:
</p>

<p>
<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Forces/WorldForce1.png\">
</p>

<p>
This leads to the following animation
</p>

<p>
<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Forces/WorldForce2.png\">
</p>

</html>"));
        end WorldForce;

        model WorldTorque "External torque acting at frame_b, defined by 3 input signals and resolved in frame world, frame_b or frame_resolve"
          extends Interfaces.PartialOneFrame_b;
          Interfaces.Frame_resolve frame_resolve if resolveInFrame == Modelica.Mechanics.MultiBody.Types.ResolveInFrameB.frame_resolve "The input signals are optionally resolved in this frame" annotation(
            Placement(transformation(origin = {0, 100}, extent = {{16, -16}, {-16, 16}}, rotation = 270)));
          Modelica.Blocks.Interfaces.RealInput torque[3](each final quantity = "Torque", each final unit = "N.m") "x-, y-, z-coordinates of torque resolved in frame defined by resolveInFrame" annotation(
            Placement(transformation(extent = {{-140, -20}, {-100, 20}}, rotation = 0)));
          parameter Boolean animation = true "= true, if animation shall be enabled";
          parameter Modelica.Mechanics.MultiBody.Types.ResolveInFrameB resolveInFrame = Modelica.Mechanics.MultiBody.Types.ResolveInFrameB.world "Frame in which input torque is resolved (1: world, 2: frame_b, 3: frame_resolve)";
          parameter Real Nm_to_m(unit = "N.m/m") = world.defaultNm_to_m "Torque arrow scaling (length = torque/Nm_to_m)" annotation(
            Dialog(group = "if animation = true", enable = animation));
          input SI.Diameter diameter = world.defaultArrowDiameter "Diameter of torque arrow" annotation(
            Dialog(group = "if animation = true", enable = animation));
          input Types.Color color = Modelica.Mechanics.MultiBody.Types.Defaults.TorqueColor "Color of arrow" annotation(
            Dialog(colorSelector = true, group = "if animation = true", enable = animation));
          input Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(
            Dialog(group = "if animation = true", enable = animation));
        protected
          SI.Position t_in_m[3] = frame_b.t / Nm_to_m "Torque mapped from Nm to m for animation";
          Visualizers.Advanced.DoubleArrow arrow(diameter = diameter, color = color, specularCoefficient = specularCoefficient, R = frame_b.R, r = frame_b.r_0, r_tail = t_in_m, r_head = -t_in_m) if world.enableAnimation and animation;
        public
          Internal.BasicWorldTorque basicWorldTorque(resolveInFrame = resolveInFrame) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}})));
        protected
          Interfaces.ZeroPosition zeroPosition if not resolveInFrame == Modelica.Mechanics.MultiBody.Types.ResolveInFrameB.frame_resolve annotation(
            Placement(transformation(extent = {{20, 10}, {40, 30}})));
        equation
          connect(basicWorldTorque.frame_b, frame_b) annotation(
            Line(points = {{10, 0}, {100, 0}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
          connect(basicWorldTorque.torque, torque) annotation(
            Line(points = {{-12, 0}, {-120, 0}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(frame_resolve, basicWorldTorque.frame_resolve) annotation(
            Line(points = {{0, 100}, {0, 10}}, color = {95, 95, 95}, pattern = LinePattern.Dot, smooth = Smooth.None));
          connect(zeroPosition.frame_resolve, basicWorldTorque.frame_resolve) annotation(
            Line(points = {{20, 20}, {0, 20}, {0, 10}}, color = {95, 95, 95}, pattern = LinePattern.Dot, smooth = Smooth.None));
          annotation(
            defaultComponentName = "torque",
            Documentation(info = "<HTML>

<p>
The <b>3</b> signals of the <b>torque</b> connector are interpreted
as the x-, y- and z-coordinates of a <b>torque</b> acting at the frame
connector to which frame_b of this component is attached.
Via parameter <b>resolveInFrame</b> it is defined, in which frame these
coordinates shall be resolved:
</p>

<table border=1 cellspacing=0 cellpadding=2>
<tr><th><b>Types.ResolveInFrameB.</b></th><th><b>Meaning</b></th></tr>
<tr><td valign=\"top\">world</td>
    <td valign=\"top\">Resolve input torque in world frame (= default)</td></tr>

<tr><td valign=\"top\">frame_b</td>
    <td valign=\"top\">Resolve input torque in frame_b</td></tr>

<tr><td valign=\"top\">frame_resolve</td>
    <td valign=\"top\">Resolve input torque in frame_resolve (frame_resolve must be connected)</td></tr>
</table>

<p>
If resolveInFrame = Types.ResolveInFrameB.frame_resolve, the torque coordinates
are with respect to the frame, that is connected to <b>frame_resolve</b>.
</p>

<p>
If torque={100,0,0}, and for all parameters the default setting is used,
then the interpretation is that a torque of 100 N is acting along the positive
x-axis of frame_b.
</p>

<p>
Note, the cut-force in frame_b (frame_b.f) is always set to zero.
Conceptually, a force and torque acts on the world frame in such a way that
the force and torque balance between world.frame_b and frame_b is fulfilled.
For efficiency reasons, this reaction torque is, however, not computed.
</p>

<p>
This torque component is by default visualized as a <b>double arrow</b>
acting at the connector to which it is connected. The diameter
and color of the arrow can be defined via
variables <b>diameter</b> and <b>color</b>. The double arrow points
in the direction defined by the
torque vector. The length of the double arrow is proportional
to the length of the torque vector using parameter
<b>Nm_to_m</b> as scaling factor. For example, if Nm_to_m = 100 Nm/m,
then a torque of 350 Nm is displayed as an arrow of length 3.5 m.
</p>
<p>
An example how to use this model is given in the
following figure:
</p>

<p>
<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Forces/WorldTorque1.png\">
</p>

<p>
This leads to the following animation
</p>

<p>
<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Forces/WorldTorque2.png\">
</p>

</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-61, 64}, {46, 27}}, lineColor = {192, 192, 192}, textString = "resolve"), Text(extent = {{-150, -40}, {150, -80}}, textString = "%name", lineColor = {0, 0, 255}), Line(points = {{0, 95}, {0, 82}}, color = {95, 95, 95}, pattern = LinePattern.Dot), Line(points = {{-100, 0}, {-94, 13}, {-86, 28}, {-74, 48}, {-65, 60}, {-52, 72}, {-35, 81}, {-22, 84}, {-8, 84}, {7, 80}, {19, 73}, {32, 65}, {44, 55}, {52, 47}, {58, 40}}, color = {0, 0, 0}, thickness = 0.5), Polygon(points = {{94, 10}, {75, 59}, {41, 24}, {94, 10}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid)}));
        end WorldTorque;

        model WorldForceAndTorque "External force and torque acting at frame_b, defined by 3+3 input signals and resolved in frame world, frame_b or in frame_resolve"
          import Modelica.Mechanics.MultiBody.Types;
          extends Interfaces.PartialOneFrame_b;
          Interfaces.Frame_resolve frame_resolve if resolveInFrame == Modelica.Mechanics.MultiBody.Types.ResolveInFrameB.frame_resolve "The input signals are optionally resolved in this frame" annotation(
            Placement(transformation(origin = {0, 100}, extent = {{16, -16}, {-16, 16}}, rotation = 270)));
          Blocks.Interfaces.RealInput force[3](each final quantity = "Force", each final unit = "N") "x-, y-, z-coordinates of force resolved in frame defined by resolveInFrame" annotation(
            Placement(transformation(extent = {{-140, -80}, {-100, -40}}, rotation = 0)));
          Blocks.Interfaces.RealInput torque[3](each final quantity = "Torque", each final unit = "N.m") "x-, y-, z-coordinates of torque resolved in frame defined by resolveInFrame" annotation(
            Placement(transformation(extent = {{-140, 40}, {-100, 80}}, rotation = 0)));
          parameter Boolean animation = true "= true, if animation shall be enabled";
          parameter Modelica.Mechanics.MultiBody.Types.ResolveInFrameB resolveInFrame = Modelica.Mechanics.MultiBody.Types.ResolveInFrameB.world "Frame in which input force and torque are resolved (1: world, 2: frame_b, 3: frame_resolve)";
          parameter Real N_to_m(unit = "N/m") = world.defaultN_to_m "Force arrow scaling (length = force/N_to_m)" annotation(
            Dialog(group = "if animation = true", enable = animation));
          parameter Real Nm_to_m(unit = "N.m/m") = world.defaultNm_to_m "Torque arrow scaling (length = torque/Nm_to_m)" annotation(
            Dialog(group = "if animation = true", enable = animation));
          input SI.Diameter forceDiameter = world.defaultArrowDiameter "Diameter of force arrow" annotation(
            Dialog(group = "if animation = true", enable = animation));
          input SI.Diameter torqueDiameter = forceDiameter "Diameter of torque arrow" annotation(
            Dialog(group = "if animation = true", enable = animation));
          input Types.Color forceColor = Modelica.Mechanics.MultiBody.Types.Defaults.ForceColor "Color of force arrow" annotation(
            Dialog(colorSelector = true, group = "if animation = true", enable = animation));
          input Types.Color torqueColor = Modelica.Mechanics.MultiBody.Types.Defaults.TorqueColor "Color of torque arrow" annotation(
            Dialog(colorSelector = true, group = "if animation = true", enable = animation));
          input Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(
            Dialog(group = "if animation = true", enable = animation));
        protected
          SI.Position f_in_m[3] = frame_b.f / N_to_m "Force mapped from N to m for animation";
          SI.Position t_in_m[3] = frame_b.t / Nm_to_m "Torque mapped from Nm to m for animation";
          Visualizers.Advanced.Arrow forceArrow(diameter = forceDiameter, color = forceColor, specularCoefficient = specularCoefficient, R = frame_b.R, r = frame_b.r_0, r_tail = f_in_m, r_head = -f_in_m) if world.enableAnimation and animation;
          Visualizers.Advanced.DoubleArrow torqueArrow(diameter = torqueDiameter, color = torqueColor, specularCoefficient = specularCoefficient, R = frame_b.R, r = frame_b.r_0, r_tail = t_in_m, r_head = -t_in_m) if world.enableAnimation and animation;
        public
          Internal.BasicWorldForce basicWorldForce(resolveInFrame = resolveInFrame) annotation(
            Placement(transformation(extent = {{18, -50}, {38, -70}})));
          Internal.BasicWorldTorque basicWorldTorque(resolveInFrame = resolveInFrame) annotation(
            Placement(transformation(extent = {{-10, 50}, {10, 70}})));
        protected
          Interfaces.ZeroPosition zeroPosition if not resolveInFrame == Modelica.Mechanics.MultiBody.Types.ResolveInFrameB.frame_resolve annotation(
            Placement(transformation(extent = {{58, 70}, {78, 90}})));
        equation
          connect(basicWorldForce.frame_b, frame_b) annotation(
            Line(points = {{38, -60}, {60, -60}, {60, 0}, {100, 0}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
          connect(basicWorldForce.force, force) annotation(
            Line(points = {{16, -60}, {-120, -60}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(basicWorldTorque.frame_b, frame_b) annotation(
            Line(points = {{10, 60}, {60, 60}, {60, 0}, {100, 0}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
          connect(basicWorldTorque.torque, torque) annotation(
            Line(points = {{-12, 60}, {-120, 60}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(basicWorldForce.frame_resolve, frame_resolve) annotation(
            Line(points = {{28, -50}, {28, 80}, {0, 80}, {0, 100}}, color = {95, 95, 95}, pattern = LinePattern.Dot, smooth = Smooth.None));
          connect(basicWorldTorque.frame_resolve, frame_resolve) annotation(
            Line(points = {{0, 70}, {0, 100}}, color = {95, 95, 95}, pattern = LinePattern.Dot, smooth = Smooth.None));
          connect(zeroPosition.frame_resolve, basicWorldTorque.frame_resolve) annotation(
            Line(points = {{58, 80}, {0, 80}, {0, 70}}, color = {95, 95, 95}, pattern = LinePattern.Dot, smooth = Smooth.None));
          connect(zeroPosition.frame_resolve, basicWorldForce.frame_resolve) annotation(
            Line(points = {{58, 80}, {40, 80}, {40, -40}, {28, -40}, {28, -50}}, color = {95, 95, 95}, pattern = LinePattern.Dot, smooth = Smooth.None));
          annotation(
            defaultComponentName = "forceAndTorque",
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-63, 56}, {44, 19}}, lineColor = {192, 192, 192}, textString = "resolve"), Text(extent = {{-150, -75}, {150, -115}}, textString = "%name", lineColor = {0, 0, 255}), Line(points = {{-100, 60}, {-86, 68}, {-80, 72}, {-70, 78}, {-64, 82}, {-46, 86}, {-34, 88}, {-16, 88}, {-2, 86}, {12, 80}, {24, 74}, {34, 68}, {46, 58}, {52, 54}, {58, 48}}, color = {0, 0, 0}, thickness = 0.5), Polygon(points = {{89, 17}, {64, 76}, {30, 41}, {89, 17}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Line(points = {{0, 95}, {0, -26}}, color = {95, 95, 95}, pattern = LinePattern.Dot), Line(points = {{0, 0}, {96, 0}}, color = {95, 95, 95}, pattern = LinePattern.Dot), Polygon(points = {{-104, -48}, {54, 0}, {46, 20}, {96, 0}, {66, -42}, {60, -22}, {-96, -72}, {-104, -48}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid)}),
            Documentation(info = "<HTML>
<p>
The <b>3</b> signals of the <b>force</b> and <b>torque</b>
connector are interpreted
as the x-, y- and z-coordinates of a <b>force</b> and
<b>torque</b> acting at the frame
connector to which frame_b of this component is attached.
Via parameter <b>resolveInFrame</b> it is defined, in which frame these
coordinates shall be resolved:
</p>

<table border=1 cellspacing=0 cellpadding=2>
<tr><th><b>Types.ResolveInFrameB.</b></th><th><b>Meaning</b></th></tr>
<tr><td valign=\"top\">world</td>
    <td valign=\"top\">Resolve input force and torque in world frame (= default)</td></tr>

<tr><td valign=\"top\">frame_b</td>
    <td valign=\"top\">Resolve input force and torque in frame_b</td></tr>

<tr><td valign=\"top\">frame_resolve</td>
    <td valign=\"top\">Resolve input force and torque in frame_resolve
                    (frame_resolve must be connected)</td></tr>
</table>

<p>
If resolveInFrame = Types.ResolveInFrameB.frame_resolve, the force and
torque coordinates
are with respect to the frame, that is connected to <b>frame_resolve</b>.
</p>

<p>
If force={100,0,0}, and for all parameters the default setting is used,
then the interpretation is that a force of 100 N is acting along the positive
x-axis of frame_b.
</p>

<p>
Conceptually, a force and torque acts on the world frame in such a way that
the force and torque balance between world.frame_b and frame_b is fulfilled.
For efficiency reasons, this reaction torque is, however, not computed.
</p>

<p>
The force and torque are by default visualized as an arrow (force)
and as a double arrow (torque) acting at the connector to which
they are connected. The diameters
and colors of the arrows can be defined via
variables <b>forceDiameter</b>, <b>torqueDiameter</b>,
<b>forceColor</b> and <b>torqueColor</b>. The arrows
point in the directions defined by the
force and torque vectors. The lengths of the arrows are proportional
to the length of the force and torque vectors, respectively, using parameters
<b>N_to_m</b> and <b>Nm_to_m</b> as scaling factors. For example, if N_to_m = 100 N/m,
then a force of 350 N is displayed as an arrow of length 3.5 m.
</p>
<p>
An example how to use this model is given in the
following figure:
</p>

<p>
<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Forces/WorldForceAndTorque1.png\">
</p>

<p>
This leads to the following animation
</p>

<p>
<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Forces/WorldForceAndTorque2.png\">
</p>

</html>"));
        end WorldForceAndTorque;

        model Force "Force acting between two frames, defined by 3 input signals and resolved in frame world, frame_a, frame_b or frame_resolve"
          import Modelica.SIunits.Conversions.to_unit1;
          extends Modelica.Mechanics.MultiBody.Interfaces.PartialTwoFrames;
          Interfaces.Frame_resolve frame_resolve if resolveInFrame == Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB.frame_resolve "The input signals are optionally resolved in this frame" annotation(
            Placement(transformation(origin = {40, 100}, extent = {{-16, -16}, {16, 16}}, rotation = 90)));
          Modelica.Blocks.Interfaces.RealInput force[3](each final quantity = "Force", each final unit = "N") "x-, y-, z-coordinates of force resolved in frame defined by resolveInFrame" annotation(
            Placement(transformation(origin = {-60, 120}, extent = {{-20, -20}, {20, 20}}, rotation = 270)));
          parameter Boolean animation = true "= true, if animation shall be enabled";
          parameter Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB resolveInFrame = Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB.frame_b "Frame in which input force is resolved (1: world, 2: frame_a, 3: frame_b, 4: frame_resolve)";
          parameter Real N_to_m(unit = "N/m") = world.defaultN_to_m "Force arrow scaling (length = force/N_to_m)" annotation(
            Dialog(group = "if animation = true", enable = animation));
          input SI.Diameter forceDiameter = world.defaultArrowDiameter "Diameter of force arrow" annotation(
            Dialog(group = "if animation = true", enable = animation));
          input SI.Diameter connectionLineDiameter = forceDiameter "Diameter of line connecting frame_a and frame_b" annotation(
            Dialog(group = "if animation = true", enable = animation));
          input Types.Color forceColor = Modelica.Mechanics.MultiBody.Types.Defaults.ForceColor "Color of force arrow" annotation(
            Dialog(colorSelector = true, group = "if animation = true", enable = animation));
          input Types.Color connectionLineColor = Modelica.Mechanics.MultiBody.Types.Defaults.SensorColor "Color of line connecting frame_a and frame_b" annotation(
            Dialog(colorSelector = true, group = "if animation = true", enable = animation));
          input Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(
            Dialog(group = "if animation = true", enable = animation));
        protected
          SI.Position f_in_m[3] = frame_b.f / N_to_m "Force mapped from N to m for animation";
          Visualizers.Advanced.Arrow forceArrow(diameter = forceDiameter, color = forceColor, specularCoefficient = specularCoefficient, R = frame_b.R, r = frame_b.r_0, r_tail = f_in_m, r_head = -f_in_m) if world.enableAnimation and animation;
          Visualizers.Advanced.Shape connectionLine(shapeType = "cylinder", lengthDirection = to_unit1(basicForce.r_0), widthDirection = {0, 1, 0}, length = Modelica.Math.Vectors.length(basicForce.r_0), width = connectionLineDiameter, height = connectionLineDiameter, color = connectionLineColor, specularCoefficient = specularCoefficient, r = frame_a.r_0) if world.enableAnimation and animation;
        public
          MultiBody.Forces.Internal.BasicForce basicForce(resolveInFrame = resolveInFrame) annotation(
            Placement(transformation(extent = {{0, -10}, {20, 10}})));
        protected
          MultiBody.Interfaces.ZeroPosition zeroPosition if not resolveInFrame == Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB.frame_resolve annotation(
            Placement(transformation(extent = {{40, 10}, {60, 30}})));
        equation
          connect(basicForce.frame_a, frame_a) annotation(
            Line(points = {{0, 0}, {-100, 0}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
          connect(basicForce.frame_b, frame_b) annotation(
            Line(points = {{20, 0}, {100, 0}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
          connect(force, basicForce.force) annotation(
            Line(points = {{-60, 120}, {-60, 40}, {4, 40}, {4, 12}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(basicForce.frame_resolve, frame_resolve) annotation(
            Line(points = {{14, 10}, {14, 40}, {40, 40}, {40, 100}}, color = {95, 95, 95}, pattern = LinePattern.Dot, smooth = Smooth.None));
          connect(zeroPosition.frame_resolve, basicForce.frame_resolve) annotation(
            Line(points = {{40, 20}, {27, 20}, {27, 10}, {14, 10}}, color = {95, 95, 95}, pattern = LinePattern.Dot, smooth = Smooth.None));
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-98, 99}, {99, -98}}, lineColor = {255, 255, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-92, 61}, {87, 35}}, lineColor = {192, 192, 192}, textString = "resolve"), Text(extent = {{-150, -55}, {150, -95}}, textString = "%name", lineColor = {0, 0, 255}), Line(points = {{40, 100}, {40, 0}}, color = {95, 95, 95}, pattern = LinePattern.Dot), Polygon(points = {{-94, 0}, {-64, 11}, {-64, -10}, {-94, 0}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Line(points = {{-60, 100}, {40, 100}}, color = {95, 95, 95}, pattern = LinePattern.Dot), Polygon(points = {{94, 0}, {65, 12}, {65, -11}, {94, 0}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Line(points = {{-64, 0}, {-20, 0}}, color = {0, 0, 0}), Line(points = {{20, 0}, {65, 0}}, color = {0, 0, 0})}),
            Documentation(info = "<html>
<p>
The <b>3</b> signals of the <b>force</b> connector are interpreted
as the x-, y- and z-coordinates of a <b>force</b> acting at the frame
connector to which frame_b of this component is attached.
Via parameter <b>resolveInFrame</b> it is defined, in which frame these
coordinates shall be resolved:
</p>

<table border=1 cellspacing=0 cellpadding=2>
<tr><th><b>Types.ResolveInFrameAB.</b></th><th><b>Meaning</b></th></tr>
<tr><td valign=\"top\">world</td>
    <td valign=\"top\">Resolve input force in world frame</td></tr>

<tr><td valign=\"top\">frame_a</td>
    <td valign=\"top\">Resolve input force in frame_a</td></tr>

<tr><td valign=\"top\">frame_b</td>
    <td valign=\"top\">Resolve input force in frame_b (= default)</td></tr>

<tr><td valign=\"top\">frame_resolve</td>
    <td valign=\"top\">Resolve input force in frame_resolve (frame_resolve must be connected)</td></tr>
</table>

<p>
If resolveInFrame = ResolveInFrameAB.frame_resolve, the force coordinates
are with respect to the frame, that is connected to <b>frame_resolve</b>.
</p>

<p>
If force={100,0,0}, and for all parameters the default setting is used,
then the interpretation is that a force of 100 N is acting along the positive
x-axis of frame_b.
</p>

<p>
Note, the cut-torque in frame_b (frame_b.t) is always set to zero.
Additionally, a force and torque acts on frame_a in such a way that
the force and torque balance between frame_a and frame_b is fulfilled.
</p>

<p>
An example how to use this model is given in the
following figure:
</p>

<p>
<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Forces/Force1.png\">
</p>

<p>
This leads to the following animation (the yellow cylinder
characterizes the line between frame_a and frame_b of the
Force component, i.e., the force acts with negative sign
also on the opposite side of this cylinder, but for
clarity this is not shown in the animation):
</p>

<p>
<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Forces/Force2.png\">
</p>

</html>"));
        end Force;

        model Torque "Torque acting between two frames, defined by 3 input signals and resolved in frame world, frame_a, frame_b or frame_resolve"
          import Modelica.SIunits.Conversions.to_unit1;
          extends Modelica.Mechanics.MultiBody.Interfaces.PartialTwoFrames;
          Interfaces.Frame_resolve frame_resolve if resolveInFrame == Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB.frame_resolve "The input signals are optionally resolved in this frame" annotation(
            Placement(transformation(origin = {40, 100}, extent = {{-16, -16}, {16, 16}}, rotation = 90)));
          Modelica.Blocks.Interfaces.RealInput torque[3](each final quantity = "Torque", each final unit = "N.m") "x-, y-, z-coordinates of torque resolved in frame defined by resolveInFrame" annotation(
            Placement(transformation(origin = {-60, 120}, extent = {{-20, -20}, {20, 20}}, rotation = 270)));
          parameter Boolean animation = true "= true, if animation shall be enabled";
          parameter Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB resolveInFrame = Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB.frame_b "Frame in which input force is resolved (1: world, 2: frame_a, 3: frame_b, 4: frame_resolve)";
          parameter Real Nm_to_m(unit = "N.m/m") = world.defaultNm_to_m "Torque arrow scaling (length = torque/Nm_to_m)" annotation(
            Dialog(group = "if animation = true", enable = animation));
          input SI.Diameter torqueDiameter = world.defaultArrowDiameter "Diameter of torque arrow" annotation(
            Dialog(group = "if animation = true", enable = animation));
          input SI.Diameter connectionLineDiameter = torqueDiameter "Diameter of line connecting frame_a and frame_b" annotation(
            Dialog(group = "if animation = true", enable = animation));
          input Types.Color torqueColor = Modelica.Mechanics.MultiBody.Types.Defaults.TorqueColor "Color of torque arrow" annotation(
            Dialog(colorSelector = true, group = "if animation = true", enable = animation));
          input Types.Color connectionLineColor = Modelica.Mechanics.MultiBody.Types.Defaults.SensorColor "Color of line connecting frame_a and frame_b" annotation(
            Dialog(colorSelector = true, group = "if animation = true", enable = animation));
          input Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(
            Dialog(group = "if animation = true", enable = animation));
        protected
          SI.Position t_in_m[3] = frame_b.t / Nm_to_m "Torque mapped from Nm to m for animation";
          Visualizers.Advanced.DoubleArrow torqueArrow(diameter = torqueDiameter, color = torqueColor, specularCoefficient = specularCoefficient, R = frame_b.R, r = frame_b.r_0, r_tail = t_in_m, r_head = -t_in_m) if world.enableAnimation and animation;
          Visualizers.Advanced.Shape connectionLine(shapeType = "cylinder", lengthDirection = to_unit1(basicTorque.r_0), widthDirection = {0, 1, 0}, length = Modelica.Math.Vectors.length(basicTorque.r_0), width = connectionLineDiameter, height = connectionLineDiameter, color = connectionLineColor, specularCoefficient = specularCoefficient, r = frame_a.r_0) if world.enableAnimation and animation;
        public
          Internal.BasicTorque basicTorque(resolveInFrame = resolveInFrame) annotation(
            Placement(transformation(extent = {{-8, -10}, {12, 10}})));
        protected
          Interfaces.ZeroPosition zeroPosition if not resolveInFrame == Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB.frame_resolve annotation(
            Placement(transformation(extent = {{34, 10}, {54, 30}})));
        equation
          connect(basicTorque.frame_a, frame_a) annotation(
            Line(points = {{-8, 0}, {-100, 0}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
          connect(basicTorque.frame_b, frame_b) annotation(
            Line(points = {{12, 0}, {100, 0}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
          connect(basicTorque.torque, torque) annotation(
            Line(points = {{-4, 12}, {-4, 60}, {-60, 60}, {-60, 120}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(basicTorque.frame_resolve, frame_resolve) annotation(
            Line(points = {{6, 10}, {6, 60}, {40, 60}, {40, 100}}, color = {95, 95, 95}, pattern = LinePattern.Dot, smooth = Smooth.None));
          connect(zeroPosition.frame_resolve, basicTorque.frame_resolve) annotation(
            Line(points = {{34, 20}, {20, 20}, {20, 10}, {6, 10}}, color = {95, 95, 95}, pattern = LinePattern.Dot, smooth = Smooth.None));
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-98, 99}, {99, -98}}, lineColor = {255, 255, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-59, 55}, {72, 30}}, lineColor = {192, 192, 192}, textString = "resolve"), Text(extent = {{-150, -30}, {150, -70}}, textString = "%name", lineColor = {0, 0, 255}), Polygon(points = {{100, 20}, {84, 52}, {69, 39}, {100, 20}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Line(points = {{40, 100}, {76, 46}}, color = {95, 95, 95}, pattern = LinePattern.Dot), Polygon(points = {{-99, 20}, {-86, 53}, {-70, 42}, {-99, 20}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Line(points = {{-60, 100}, {40, 100}}, color = {95, 95, 95}, pattern = LinePattern.Dot), Line(points = {{-79, 47}, {-70, 61}, {-59, 72}, {-45, 81}, {-32, 84}, {-20, 85}}, color = {0, 0, 0}), Line(points = {{77, 45}, {66, 60}, {55, 69}, {49, 74}, {41, 80}, {31, 84}, {20, 85}}, color = {0, 0, 0})}),
            Documentation(info = "<HTML>
<p>
The <b>3</b> signals of the <b>torque</b> connector are interpreted
as the x-, y- and z-coordinates of a <b>torque</b> acting at the frame
connector to which frame_b of this component is attached.
Via parameter <b>resolveInFrame</b> it is defined, in which frame these
coordinates shall be resolved:
</p>

<table border=1 cellspacing=0 cellpadding=2>
<tr><th><b>Types.ResolveInFrameAB.</b></th><th><b>Meaning</b></th></tr>
<tr><td valign=\"top\">world</td>
    <td valign=\"top\">Resolve input torque in world frame</td></tr>

<tr><td valign=\"top\">frame_a</td>
    <td valign=\"top\">Resolve input torque in frame_a</td></tr>

<tr><td valign=\"top\">frame_b</td>
    <td valign=\"top\">Resolve input torque in frame_b (= default)</td></tr>

<tr><td valign=\"top\">frame_resolve</td>
    <td valign=\"top\">Resolve input torque in frame_resolve (frame_resolve must be connected)</td></tr>
</table>

<p>
If resolveInFrame = ResolveInFrameAB.frame_resolve, the torque coordinates
are with respect to the frame, that is connected to <b>frame_resolve</b>.
</p>

<p>
If torque={100,0,0}, and for all parameters the default setting is used,
then the interpretation is that a torque of 100 N.m is acting along the positive
x-axis of frame_b.
</p>

<p>
Note, the cut-forces in frame_a and frame_b (frame_a.f, frame_b.f) are
always set to zero and the cut-torque at frame_a (frame_a.t) is the same
as the cut-torque at frame_b (frame_b.t) but with opposite sign.
</p>

<p>
An example how to use this model is given in the
following figure:
</p>

<p>
<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Forces/Torque1.png\">
</p>

<p>
This leads to the following animation (the yellow cylinder
characterizes the line between frame_a and frame_b of the
Torque component, i.e., the torque acts with negative sign
also on the opposite side of this cylinder, but for
clarity this is not shown in the animation):
</p>

<p>
<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Forces/Torque2.png\">
</p>

</html>"));
        end Torque;

        model ForceAndTorque "Force and torque acting between two frames, defined by 3+3 input signals and resolved in frame world, frame_a, frame_b or frame_resolve"
          import Modelica.Mechanics.MultiBody.Types;
          import Modelica.SIunits.Conversions.to_unit1;
          extends Modelica.Mechanics.MultiBody.Interfaces.PartialTwoFrames;
          Blocks.Interfaces.RealInput force[3](each final quantity = "Force", each final unit = "N") "x-, y-, z-coordinates of force resolved in frame defined by resolveInFrame" annotation(
            Placement(transformation(origin = {-80, 120}, extent = {{-20, -20}, {20, 20}}, rotation = 270)));
          Blocks.Interfaces.RealInput torque[3](each final quantity = "Torque", each final unit = "N.m") "x-, y-, z-coordinates of torque resolved in frame defined by resolveInFrame" annotation(
            Placement(transformation(origin = {0, 120}, extent = {{-20, -20}, {20, 20}}, rotation = 270)));
          Interfaces.Frame_resolve frame_resolve if resolveInFrame == Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB.frame_resolve "The input signals are optionally resolved in this frame" annotation(
            Placement(transformation(origin = {80, 100}, extent = {{-16, -16}, {16, 16}}, rotation = 90)));
          parameter Boolean animation = true "= true, if animation shall be enabled";
          parameter Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB resolveInFrame = Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB.frame_b "Frame in which input force and torque are resolved (1: world, 2: frame_a, 3: frame_b, 4: frame_resolve)";
          parameter Real N_to_m(unit = "N/m") = world.defaultN_to_m "Force arrow scaling (length = force/N_to_m)" annotation(
            Dialog(group = "if animation = true", enable = animation));
          parameter Real Nm_to_m(unit = "N.m/m") = world.defaultNm_to_m "Torque arrow scaling (length = torque/Nm_to_m)" annotation(
            Dialog(group = "if animation = true", enable = animation));
          input SI.Diameter forceDiameter = world.defaultArrowDiameter "Diameter of force arrow" annotation(
            Dialog(group = "if animation = true", enable = animation));
          input SI.Diameter torqueDiameter = forceDiameter "Diameter of torque arrow" annotation(
            Dialog(group = "if animation = true", enable = animation));
          input SI.Diameter connectionLineDiameter = forceDiameter "Diameter of line connecting frame_a and frame_b" annotation(
            Dialog(group = "if animation = true", enable = animation));
          input Types.Color forceColor = Modelica.Mechanics.MultiBody.Types.Defaults.ForceColor "Color of force arrow" annotation(
            Dialog(colorSelector = true, group = "if animation = true", enable = animation));
          input Types.Color torqueColor = Modelica.Mechanics.MultiBody.Types.Defaults.TorqueColor "Color of torque arrow" annotation(
            Dialog(colorSelector = true, group = "if animation = true", enable = animation));
          input Types.Color connectionLineColor = Modelica.Mechanics.MultiBody.Types.Defaults.SensorColor "Color of line connecting frame_a and frame_b" annotation(
            Dialog(colorSelector = true, group = "if animation = true", enable = animation));
          input Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(
            Dialog(group = "if animation = true", enable = animation));
        protected
          SI.Position f_in_m[3] = frame_b.f / N_to_m "Force mapped from N to m for animation";
          SI.Position t_in_m[3] = frame_b.t / Nm_to_m "Torque mapped from Nm to m for animation";
          Visualizers.Advanced.Arrow forceArrow(diameter = forceDiameter, color = forceColor, specularCoefficient = specularCoefficient, R = frame_b.R, r = frame_b.r_0, r_tail = f_in_m, r_head = -f_in_m) if world.enableAnimation and animation;
          Visualizers.Advanced.DoubleArrow torqueArrow(diameter = torqueDiameter, color = torqueColor, specularCoefficient = specularCoefficient, R = frame_b.R, r = frame_b.r_0, r_tail = t_in_m, r_head = -t_in_m) if world.enableAnimation and animation;
          Visualizers.Advanced.Shape connectionLine(shapeType = "cylinder", lengthDirection = to_unit1(basicForce.r_0), widthDirection = {0, 1, 0}, length = Modelica.Math.Vectors.length(basicForce.r_0), width = connectionLineDiameter, height = connectionLineDiameter, color = connectionLineColor, specularCoefficient = specularCoefficient, r = frame_a.r_0) if world.enableAnimation and animation;
        public
          Internal.BasicForce basicForce(resolveInFrame = resolveInFrame) annotation(
            Placement(transformation(extent = {{-84, -10}, {-64, 10}})));
          Internal.BasicTorque basicTorque(resolveInFrame = resolveInFrame) annotation(
            Placement(transformation(extent = {{-4, 10}, {16, 30}})));
        protected
          Interfaces.ZeroPosition zeroPosition if not resolveInFrame == Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB.frame_resolve annotation(
            Placement(transformation(extent = {{20, 30}, {40, 50}})));
        equation
          connect(basicForce.frame_a, frame_a) annotation(
            Line(points = {{-84, 0}, {-100, 0}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
          connect(basicForce.frame_b, frame_b) annotation(
            Line(points = {{-64, 0}, {100, 0}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
          connect(basicTorque.frame_b, frame_b) annotation(
            Line(points = {{16, 20}, {68, 20}, {68, 0}, {100, 0}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
          connect(basicTorque.frame_a, frame_a) annotation(
            Line(points = {{-4, 20}, {-90, 20}, {-90, 0}, {-100, 0}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
          connect(basicForce.force, force) annotation(
            Line(points = {{-80, 12}, {-80, 120}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(basicTorque.torque, torque) annotation(
            Line(points = {{0, 32}, {0, 120}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(basicTorque.frame_resolve, frame_resolve) annotation(
            Line(points = {{10, 30}, {10, 80}, {80, 80}, {80, 100}}, color = {95, 95, 95}, pattern = LinePattern.Dot, smooth = Smooth.None));
          connect(basicForce.frame_resolve, frame_resolve) annotation(
            Line(points = {{-70, 10}, {-70, 80}, {80, 80}, {80, 100}}, color = {95, 95, 95}, pattern = LinePattern.Dot, smooth = Smooth.None));
          connect(zeroPosition.frame_resolve, basicTorque.frame_resolve) annotation(
            Line(points = {{20, 40}, {10, 40}, {10, 30}}, color = {95, 95, 95}, pattern = LinePattern.Dot, smooth = Smooth.None));
          connect(zeroPosition.frame_resolve, basicForce.frame_resolve) annotation(
            Line(points = {{20, 40}, {-70, 40}, {-70, 10}}, color = {95, 95, 95}, pattern = LinePattern.Dot, smooth = Smooth.None));
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-98, 99}, {99, -98}}, lineColor = {255, 255, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-59, 55}, {72, 30}}, lineColor = {192, 192, 192}, textString = "resolve"), Text(extent = {{-150, -55}, {150, -95}}, textString = "%name", lineColor = {0, 0, 255}), Polygon(points = {{100, 21}, {84, 55}, {69, 39}, {100, 21}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Line(points = {{80, 100}, {80, 0}}, color = {95, 95, 95}, pattern = LinePattern.Dot), Polygon(points = {{-95, 1}, {-64, 11}, {-64, -10}, {-95, 1}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Polygon(points = {{-100, 20}, {-86, 53}, {-70, 42}, {-100, 20}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Line(points = {{-80, 100}, {80, 100}}, color = {95, 95, 95}, pattern = LinePattern.Dot), Polygon(points = {{94, 0}, {65, 12}, {65, -11}, {94, 0}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Line(points = {{-64, 0}, {-20, 0}}, color = {0, 0, 0}), Line(points = {{20, 0}, {65, 0}}, color = {0, 0, 0}), Line(points = {{-79, 47}, {-70, 61}, {-59, 72}, {-45, 81}, {-32, 84}, {-20, 85}}, color = {0, 0, 0}), Line(points = {{76, 47}, {66, 60}, {55, 69}, {49, 74}, {41, 80}, {31, 84}, {20, 85}}, color = {0, 0, 0}), Text(extent = {{-144, 124}, {-106, 102}}, lineColor = {0, 0, 0}, textString = "f"), Text(extent = {{20, 124}, {58, 102}}, lineColor = {0, 0, 0}, textString = "t")}),
            Documentation(info = "<HTML>
<p>
The <b>3</b> signals of the <b>force</b> connector and the
<b>3</b> signals of the <b>torque</b> connector
are interpreted
as the x-, y- and z-coordinates of a <b>force</b> and of a
<b>torque</b> acting at the frame
connector to which frame_b of this component is attached.
Via parameter <b>resolveInFrame</b> it is defined, in which frame these
coordinates shall be resolved:
</p>

<table border=1 cellspacing=0 cellpadding=2>
<tr><th><b>Types.ResolveInFrameAB.</b></th><th><b>Meaning</b></th></tr>
<tr><td valign=\"top\">world</td>
    <td valign=\"top\">Resolve input force/torque in world frame</td></tr>

<tr><td valign=\"top\">frame_a</td>
    <td valign=\"top\">Resolve input force/torque in frame_a</td></tr>

<tr><td valign=\"top\">frame_b</td>
    <td valign=\"top\">Resolve input force/torque in frame_b (= default)</td></tr>

<tr><td valign=\"top\">frame_resolve</td>
    <td valign=\"top\">Resolve input force/torque in frame_resolve (frame_resolve must be connected)</td></tr>
</table>

<p>
If resolveInFrame = ResolveInFrameAB.frame_resolve, the force and torque coordinates
are with respect to the frame, that is connected to <b>frame_resolve</b>.
</p>

<p>
If force={100,0,0}, and for all parameters the default setting is used,
then the interpretation is that a force of 100 N is acting along the positive
x-axis of frame_b.
</p>

<p>
Note, a force and torque acts on frame_a in such a way that
the force and torque balance between frame_a and frame_b is fulfilled.
</p>

<p>
An example how to use this model is given in the
following figure:
</p>

<p>
<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Forces/ForceAndTorque1.png\">
</p>

<p>
This leads to the following animation (the yellow cylinder
characterizes the line between frame_a and frame_b of the
ForceAndTorque component, i.e., the force and torque acts with
negative sign
also on the opposite side of this cylinder, but for
clarity this is not shown in the animation):
</p>

<p>
<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Forces/ForceAndTorque2.png\">
</p>

</html>"));
        end ForceAndTorque;

        model LineForceWithMass "General line force component with an optional point mass on the connection line"
          import Modelica.Mechanics.MultiBody.Types;
          extends Interfaces.PartialTwoFrames;
          Modelica.Mechanics.Translational.Interfaces.Flange_a flange_b "1-dim. translational flange (connect force of Translational library between flange_a and flange_b)" annotation(
            Placement(transformation(origin = {60, 100}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
          Modelica.Mechanics.Translational.Interfaces.Flange_b flange_a "1-dim. translational flange (connect force of Translational library between flange_a and flange_b)" annotation(
            Placement(transformation(origin = {-60, 100}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
          parameter Boolean animateLine = true "= true, if a line shape between frame_a and frame_b shall be visualized";
          parameter Boolean animateMass = true "= true, if point mass shall be visualized as sphere provided m > 0";
          parameter SI.Mass m(min = 0) = 0 "Mass of point mass on the connection line between the origin of frame_a and the origin of frame_b";
          parameter Real lengthFraction(unit = "1", min = 0, max = 1) = 0.5 "Location of point mass with respect to frame_a as a fraction of the distance from frame_a to frame_b";
          input Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(
            Dialog(tab = "Animation"));
          parameter Types.ShapeType lineShapeType = "cylinder" "Type of shape visualizing the line from frame_a to frame_b" annotation(
            Dialog(tab = "Animation", group = "if animateLine = true", enable = animateLine));
          input SI.Length lineShapeWidth = world.defaultArrowDiameter "Width of shape" annotation(
            Dialog(tab = "Animation", group = "if animateLine = true", enable = animateLine));
          input SI.Length lineShapeHeight = lineShapeWidth "Height of shape" annotation(
            Dialog(tab = "Animation", group = "if animateLine = true", enable = animateLine));
          parameter Types.ShapeExtra lineShapeExtra = 0.0 "Extra parameter for shape" annotation(
            Dialog(tab = "Animation", group = "if animateLine = true", enable = animateLine));
          input Types.Color lineShapeColor = Modelica.Mechanics.MultiBody.Types.Defaults.SensorColor "Color of line shape" annotation(
            Dialog(colorSelector = true, tab = "Animation", group = "if animateLine = true", enable = animateLine));
          input Real massDiameter = world.defaultBodyDiameter "Diameter of point mass sphere" annotation(
            Dialog(tab = "Animation", group = "if animateMass = true", enable = animateMass));
          input Types.Color massColor = Modelica.Mechanics.MultiBody.Types.Defaults.BodyColor "Color of point mass" annotation(
            Dialog(colorSelector = true, tab = "Animation", group = "if animateMass = true", enable = animateMass));
          parameter SI.Position s_small = 1.E-10 "Prevent zero-division if distance between frame_a and frame_b is zero" annotation(
            Dialog(tab = "Advanced"));
          parameter Boolean fixedRotationAtFrame_a = false "=true, if rotation frame_a.R is fixed (to directly connect line forces)" annotation(
            Evaluate = true,
            choices(checkBox = true),
            Dialog(tab = "Advanced", group = "If enabled, can give wrong results, see MultiBody.UsersGuide.Tutorial.ConnectionOfLineForces"));
          parameter Boolean fixedRotationAtFrame_b = false "=true, if rotation frame_b.R is fixed (to directly connect line forces)" annotation(
            Evaluate = true,
            choices(checkBox = true),
            Dialog(tab = "Advanced", group = "If enabled, can give wrong results, see MultiBody.UsersGuide.Tutorial.ConnectionOfLineForces"));
          SI.Distance length "Distance between the origin of frame_a and the origin of frame_b";
          SI.Position r_rel_0[3] "Position vector from frame_a to frame_b resolved in world frame";
          Real e_rel_0[3](each final unit = "1") "Unit vector in direction from frame_a to frame_b, resolved in world frame";
        protected
          SI.Force fa "Force from flange_a";
          SI.Force fb "Force from flange_b";
          SI.Position r_CM_0[3](each stateSelect = StateSelect.avoid) "Position vector from world frame to point mass, resolved in world frame";
          SI.Velocity v_CM_0[3](each stateSelect = StateSelect.avoid) "First derivative of r_CM_0";
          SI.Acceleration ag_CM_0[3] "der(v_CM_0) - gravityAcceleration";
          Visualizers.Advanced.Shape lineShape(shapeType = lineShapeType, color = lineShapeColor, specularCoefficient = specularCoefficient, length = length, width = lineShapeWidth, height = lineShapeHeight, lengthDirection = e_rel_0, widthDirection = Frames.resolve1(frame_a.R, {0, 1, 0}), extra = lineShapeExtra, r = frame_a.r_0) if world.enableAnimation and animateLine;
          Visualizers.Advanced.Shape massShape(shapeType = "sphere", color = massColor, specularCoefficient = specularCoefficient, length = massDiameter, width = massDiameter, height = massDiameter, lengthDirection = e_rel_0, widthDirection = {0, 1, 0}, r_shape = e_rel_0 * (length * lengthFraction - massDiameter / 2), r = frame_a.r_0) if world.enableAnimation and animateMass and m > 0;
        equation
          assert(noEvent(length > s_small), "
The distance between the origin of frame_a and the origin of frame_b
of a LineForceWithMass component became smaller as parameter s_small
(= a small number, defined in the \"Advanced\" menu). The distance is
set to s_small, although it is smaller, to avoid a division by zero
when computing the direction of the line force. Possible reasons
for this situation:
- At initial time the distance may already be zero: Change the initial
  positions of the bodies connected by this element.
- Hardware stops are not modeled or are modeled not stiff enough.
  Include stops, e.g., stiff springs, or increase the stiffness
  if already present.
- Another error in your model may lead to unrealistically large forces
  and torques that would in reality destroy the stops.
- The flange_b connector might be defined by a pre-defined motion,
  e.g., with Modelica.Mechanics.Translational.Position and the
  predefined flange_b.s is zero or negative.
          ");
// Determine relative position vector between the two frames
          r_rel_0 = frame_b.r_0 - frame_a.r_0;
          length = Modelica.Math.Vectors.length(r_rel_0);
          flange_a.s = 0;
          flange_b.s = length;
          e_rel_0 = r_rel_0 / Frames.Internal.maxWithoutEvent(length, s_small);
// Determine translational flange forces
          if cardinality(flange_a) > 0 and cardinality(flange_b) > 0 then
            fa = flange_a.f;
            fb = flange_b.f;
          elseif cardinality(flange_a) > 0 and cardinality(flange_b) == 0 then
            fa = flange_a.f;
            fb = -fa;
          elseif cardinality(flange_a) == 0 and cardinality(flange_b) > 0 then
            fa = -fb;
            fb = flange_b.f;
          else
            fa = 0;
            fb = 0;
          end if;
/* Force and torque balance of point mass
     - Kinematics for center of mass CM of point mass including gravity
       r_CM_0 = frame_a.r0 + r_rel_CM_0;
       v_CM_0 = der(r_CM_0);
       ag_CM_0 = der(v_CM_0) - world.gravityAcceleration(r_CM_0);
     - Power balance for the connection line
       (f1=force on frame_a side, f2=force on frame_b side, h=lengthFraction)
       0 = f1*va - m*ag_CM*(va+(vb-va)*h) + f2*vb
         = (f1 - m*ag_CM*(1-h))*va + (f2 - m*ag_CM*h)*vb
       since va and vb are completely independent from other
       the parenthesis must vanish:
         f1 := m*ag_CM*(1-h)
         f2 := m*ag_CM*h
     - Force balance on frame_a and frame_b finally results in
         0 = frame_a.f + e_rel_a*fa - f1_a
         0 = frame_b.f + e_rel_b*fb - f2_b
       and therefore
         frame_a.f = -e_rel_a*fa + m*ag_CM_a*(1-h)
         frame_b.f = -e_rel_b*fb + m*ag_CM_b*h
  */
          if m > 0 then
            r_CM_0 = frame_a.r_0 + r_rel_0 * lengthFraction;
            v_CM_0 = der(r_CM_0);
            ag_CM_0 = der(v_CM_0) - world.gravityAcceleration(r_CM_0);
            frame_a.f = Frames.resolve2(frame_a.R, m * (1 - lengthFraction) * ag_CM_0 - e_rel_0 * fa);
            frame_b.f = Frames.resolve2(frame_b.R, m * lengthFraction * ag_CM_0 - e_rel_0 * fb);
          else
            r_CM_0 = zeros(3);
            v_CM_0 = zeros(3);
            ag_CM_0 = zeros(3);
            frame_a.f = -Frames.resolve2(frame_a.R, e_rel_0 * fa);
            frame_b.f = -Frames.resolve2(frame_b.R, e_rel_0 * fb);
          end if;
// Provide appropriate equations, if direct connections of line forces
          if fixedRotationAtFrame_a then
            Connections.root(frame_a.R);
            frame_a.R = Frames.nullRotation();
          else
            frame_a.t = zeros(3);
          end if;
          if fixedRotationAtFrame_b then
            Connections.root(frame_b.R);
            frame_b.R = Frames.nullRotation();
          else
            frame_b.t = zeros(3);
          end if;
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-95, -40}, {-15, 40}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.Sphere, fillColor = {192, 192, 192}), Ellipse(extent = {{-85, -30}, {-25, 30}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Ellipse(extent = {{15, -40}, {95, 40}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.Sphere, fillColor = {192, 192, 192}), Ellipse(extent = {{23, -30}, {83, 29}}, lineColor = {128, 128, 128}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-150, -55}, {150, -95}}, textString = "%name", lineColor = {0, 0, 255}), Rectangle(extent = {{-40, 41}, {44, -40}}, lineColor = {255, 255, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-70, 15}, {-41, -13}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.Sphere, fillColor = {192, 192, 192}), Ellipse(extent = {{40, 14}, {69, -14}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.Sphere, fillColor = {192, 192, 192}), Line(points = {{-56, 0}, {-56, 23}, {-30, 23}, {-30, 70}, {-60, 70}, {-60, 101}}, color = {0, 0, 0}), Line(points = {{55, -1}, {55, 20}, {30, 20}, {30, 70}, {60, 70}, {60, 100}}, color = {0, 0, 0}), Line(points = {{-56, 0}, {55, -1}}, color = {0, 0, 0}, pattern = LinePattern.Dot), Ellipse(extent = {{-8, 8}, {8, -8}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Ellipse(visible = fixedRotationAtFrame_a, extent = {{-70, 30}, {-130, -30}}, lineColor = {255, 0, 0}), Text(visible = fixedRotationAtFrame_a, extent = {{-62, 50}, {-140, 30}}, lineColor = {255, 0, 0}, textString = "R=0"), Ellipse(visible = fixedRotationAtFrame_b, extent = {{70, 30}, {130, -30}}, lineColor = {255, 0, 0}), Text(visible = fixedRotationAtFrame_b, extent = {{62, 50}, {140, 30}}, lineColor = {255, 0, 0}, textString = "R=0")}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-60, 80}, {46, 80}}, color = {0, 0, 255}), Polygon(points = {{60, 80}, {45, 86}, {45, 74}, {60, 80}}, lineColor = {0, 0, 255}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-50, 93}, {32, 78}}, textString = "length", lineColor = {0, 0, 255}), Ellipse(extent = {{-100, -40}, {-20, 40}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.Sphere, fillColor = {192, 192, 192}), Ellipse(extent = {{-90, -30}, {-30, 30}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Ellipse(extent = {{20, -40}, {100, 40}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.Sphere, fillColor = {192, 192, 192}), Ellipse(extent = {{31, -29}, {91, 30}}, lineColor = {128, 128, 128}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-50, 39}, {50, -41}}, lineColor = {255, 255, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-74, 15}, {-45, -13}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.Sphere, fillColor = {192, 192, 192}), Ellipse(extent = {{45, 15}, {74, -13}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.Sphere, fillColor = {192, 192, 192}), Line(points = {{-60, 0}, {-60, 24}, {-40, 24}, {-40, 60}, {-60, 60}, {-60, 100}}, color = {0, 0, 0}), Line(points = {{60, 1}, {60, 21}, {40, 21}, {40, 60}, {60, 60}, {60, 100}}, color = {0, 0, 0}), Line(points = {{-60, 0}, {60, 0}}, color = {0, 0, 0}, pattern = LinePattern.Dot), Ellipse(extent = {{-8, 8}, {8, -8}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Line(points = {{-60, 0}, {-31, 0}}, color = {0, 0, 255}), Polygon(points = {{-19, 0}, {-31, 3}, {-31, -3}, {-19, 0}}, lineColor = {0, 0, 255}), Line(points = {{-60, 16}, {0, 16}}, color = {0, 0, 255}), Line(points = {{0, 0}, {0, 20}}, color = {0, 0, 255}), Text(extent = {{-49, -11}, {-8, -21}}, lineColor = {0, 0, 0}, textString = "e_rel_0"), Polygon(points = {{0, 16}, {-12, 19}, {-12, 13}, {0, 16}}, lineColor = {0, 0, 255}), Text(extent = {{-50, 35}, {51, 26}}, lineColor = {0, 0, 0}, textString = "length*lengthFraction"), Line(points = {{-17, 26}, {-26, 16}}, pattern = LinePattern.Dot, color = {0, 0, 255}), Line(points = {{-31, -13}, {-40, 0}}, pattern = LinePattern.Dot, color = {0, 0, 255})}),
            Documentation(info = "<html>
<p>
This component is used to exert a <b>line force</b>
between the origin of frame_a and the origin of frame_b
by attaching components of the <b>1-dimensional translational</b>
mechanical library of Modelica (Modelica.Mechanics.Translational)
between the two flange connectors <b>flange_a</b> and
<b>flange_b</b>. Optionally, there is a <b>point mass</b> on the line
connecting the origin of frame_a and the origin of frame_b.
This point mass approximates the <b>mass</b> of the <b>force element</b>.
The distance of the point mass from frame_a as a fraction of the
distance between frame_a and frame_b is defined via
parameter <b>lengthFraction</b> (default is 0.5, i.e., the point
mass is in the middle of the line).
</p>
<p>
In the translational library there is the implicit assumption that
forces of components that have only one flange connector act with
opposite sign on the bearings of the component. This assumption
is also used in the LineForceWithMass component: If a connection
is present to only one of the flange connectors, then the force
in this flange connector acts implicitly with opposite sign also
in the other flange connector.
</p>
</html>"));
        end LineForceWithMass;

        model LineForceWithTwoMasses "General line force component with two optional point masses on the connection line"
          import Modelica.Mechanics.MultiBody.Types;
          extends Interfaces.PartialTwoFrames;
          Modelica.Mechanics.Translational.Interfaces.Flange_a flange_b "1-dim. translational flange (connect force of Translational library between flange_a and flange_b)" annotation(
            Placement(transformation(origin = {60, 110}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
          Modelica.Mechanics.Translational.Interfaces.Flange_b flange_a "1-dim. translational flange (connect force of Translational library between flange_a and flange_b)" annotation(
            Placement(transformation(origin = {-60, 110}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
          parameter Boolean animate = true "= true, if animation shall be enabled";
          parameter Boolean animateMasses = true "= true, if point masses shall be visualized provided animate=true and m_a, m_b > 0";
          parameter SI.Mass m_a(min = 0) = 0 "Mass of point mass a on the connection line between the origin of frame_a and the origin of frame_b";
          parameter SI.Mass m_b(min = 0) = 0 "Mass of point mass b on the connection line between the origin of frame_a and the origin of frame_b";
          parameter SI.Position L_a = 0 "Distance between point mass a and frame_a (positive, if in direction of frame_b)";
          parameter SI.Position L_b = L_a "Distance between point mass b and frame_b (positive, if in direction of frame_a)";
          input SI.Diameter cylinderDiameter_a = world.defaultForceWidth "Diameter of cylinder at frame_a" annotation(
            Dialog(tab = "Animation", group = "Cylinder at frame_a if animation = true", enable = animate));
          parameter SI.Length cylinderLength_a = 2 * L_a "Length of cylinder at frame_a" annotation(
            Dialog(tab = "Animation", group = "Cylinder at frame_a if animation = true", enable = animate));
          input Types.Color color_a = {155, 155, 155} "Color of cylinder at frame_a" annotation(
            Dialog(colorSelector = true, tab = "Animation", group = "Cylinder at frame_a if animation = true", enable = animate));
          input Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(
            Dialog(tab = "Animation", group = "Cylinder at frame_a if animation = true", enable = animate));
          input Real diameterFraction = 0.8 "Diameter of cylinder at frame_b with respect to diameter of cylinder at frame_a" annotation(
            Dialog(tab = "Animation", group = "Cylinder at frame_b if animation = true", enable = animate));
          parameter SI.Length cylinderLength_b = 2 * L_b "Length of cylinder at frame_b" annotation(
            Dialog(tab = "Animation", group = "Cylinder at frame_b if animation = true", enable = animate));
          input Types.Color color_b = {100, 100, 100} "Color of cylinder at frame_b" annotation(
            Dialog(colorSelector = true, tab = "Animation", group = "Cylinder at frame_b if animation = true", enable = animate));
          input Real massDiameterFaction = 1.7 "Diameter of point mass spheres with respect to cylinderDiameter_a" annotation(
            Dialog(tab = "Animation", group = "if animation = true and animateMasses = true", enable = animate and animateMasses));
          input Types.Color massColor = Modelica.Mechanics.MultiBody.Types.Defaults.BodyColor "Color of point masses" annotation(
            Dialog(colorSelector = true, tab = "Animation", group = "if animation = true and animateMasses = true", enable = animate and animateMasses));
          parameter SI.Position s_small = 1.E-10 "Prevent zero-division if distance between frame_a and frame_b is zero" annotation(
            Dialog(tab = "Advanced"));
          parameter Boolean fixedRotationAtFrame_a = false "=true, if rotation frame_a.R is fixed (to directly connect line forces)" annotation(
            Evaluate = true,
            choices(checkBox = true),
            Dialog(tab = "Advanced", group = "If enabled, can give wrong results, see MultiBody.UsersGuide.Tutorial.ConnectionOfLineForces"));
          parameter Boolean fixedRotationAtFrame_b = false "=true, if rotation frame_b.R is fixed (to directly connect line forces)" annotation(
            Evaluate = true,
            choices(checkBox = true),
            Dialog(tab = "Advanced", group = "If enabled, can give wrong results, see MultiBody.UsersGuide.Tutorial.ConnectionOfLineForces"));
          SI.Distance length "Distance between the origin of frame_a and the origin of frame_b";
          SI.Position r_rel_0[3] "Position vector from frame_a to frame_b resolved in world frame";
          Real e_rel_0[3](each final unit = "1") "Unit vector in direction from frame_a to frame_b, resolved in world frame";
        protected
          SI.Force fa "Force from flange_a";
          SI.Force fb "Force from flange_b";
          SI.Position r_CM1_0[3](each stateSelect = StateSelect.avoid) "Position vector from world frame to point mass 1, resolved in world frame";
          SI.Position r_CM2_0[3](each stateSelect = StateSelect.avoid) "Position vector from world frame to point mass 2, resolved in world frame";
          SI.Velocity v_CM1_0[3](each stateSelect = StateSelect.avoid) "der(r_CM_1_0) - velocity of point mass 1";
          SI.Velocity v_CM2_0[3](each stateSelect = StateSelect.avoid) "der(r_CM_2_0) - velocity of point mass 2";
          SI.Acceleration ag_CM1_0[3] "der(v_CM1_0) - gravityAcceleration(r_CM1_0)";
          SI.Acceleration ag_CM2_0[3] "der(v_CM2_0) - gravityAcceleration(r_CM2_0)";
          SI.Force aux1_0[3] "Auxiliary force 1";
          SI.Force aux2_0[3] "Auxiliary force 2";
          input SI.Length cylinderDiameter_b = cylinderDiameter_a * diameterFraction;
          input SI.Length massDiameter = cylinderDiameter_a * massDiameterFaction;
          parameter Boolean animateMasses2 = world.enableAnimation and animate and animateMasses and m_a > 0 and m_b > 0;
          Visualizers.Advanced.Shape cylinder_a(shapeType = "cylinder", color = color_a, specularCoefficient = specularCoefficient, length = cylinderLength_a, width = cylinderDiameter_a, height = cylinderDiameter_a, lengthDirection = e_rel_0, widthDirection = {0, 1, 0}, r = frame_a.r_0) if world.enableAnimation and animate;
          Visualizers.Advanced.Shape cylinder_b(shapeType = "cylinder", color = color_b, specularCoefficient = specularCoefficient, length = cylinderLength_b, width = cylinderDiameter_b, height = cylinderDiameter_b, lengthDirection = -e_rel_0, widthDirection = {0, 1, 0}, r = frame_b.r_0) if world.enableAnimation and animate;
          Visualizers.Advanced.Shape sphere_a(shapeType = "sphere", color = massColor, specularCoefficient = specularCoefficient, length = massDiameter, width = massDiameter, height = massDiameter, lengthDirection = e_rel_0, widthDirection = {0, 1, 0}, r_shape = e_rel_0 * (L_a - massDiameter / 2), r = frame_a.r_0) if animateMasses2;
          Visualizers.Advanced.Shape sphere_b(shapeType = "sphere", color = massColor, specularCoefficient = specularCoefficient, length = massDiameter, width = massDiameter, height = massDiameter, lengthDirection = -e_rel_0, widthDirection = {0, 1, 0}, r_shape = -e_rel_0 * (L_b - massDiameter / 2), r = frame_b.r_0) if animateMasses2;
        equation
          assert(noEvent(length > s_small), "
The distance between the origin of frame_a and the origin of frame_b
of a LineForceWithTwoMasses component became smaller as parameter s_small
(= a small number, defined in the \"Advanced\" menu). The distance is
set to s_small, although it is smaller, to avoid a division by zero
when computing the direction of the line force. Possible reasons
for this situation:
- At initial time the distance may already be zero: Change the initial
  positions of the bodies connected by this element.
- Hardware stops are not modeled or are modeled not stiff enough.
  Include stops, e.g., stiff springs, or increase the stiffness
  if already present.
- Another error in your model may lead to unrealistically large forces
  and torques that would in reality destroy the stops.
- The flange_b connector might be defined by a pre-defined motion,
  e.g., with Modelica.Mechanics.Translational.Position and the
  predefined flange_b.s is zero or negative.
          ");
// Determine relative position vector between the two frames
          r_rel_0 = frame_b.r_0 - frame_a.r_0;
          length = Modelica.Math.Vectors.length(r_rel_0);
          flange_a.s = 0;
          flange_b.s = length;
          e_rel_0 = r_rel_0 / Frames.Internal.maxWithoutEvent(length, s_small);
// Determine translational flange forces
          if cardinality(flange_a) > 0 and cardinality(flange_b) > 0 then
            fa = flange_a.f;
            fb = flange_b.f;
          elseif cardinality(flange_a) > 0 and cardinality(flange_b) == 0 then
            fa = flange_a.f;
            fb = -fa;
          elseif cardinality(flange_a) == 0 and cardinality(flange_b) > 0 then
            fa = -fb;
            fb = flange_b.f;
          else
            fa = 0;
            fb = 0;
          end if;
/* Force and torque balance of the two point masses
     - Kinematics for center of masses CM1, CM2 of point masses including gravity
       (L = length, va = der(frame_a.r_0), vb = der(frame_b.r_0))
       r_CM1_0 = frame_a.r_0 + e_rel_0*L_a;
       r_CM2_0 = frame_b.r_0 - e_rel_0*L_b;
       v_CM1_0 = der(r_CM1_0);
       v_CM2_0 = der(r_CM2_0);
       ag_CM1_0 = der(v_CM1_0) - world.gravityAcceleration(r_CM1_0);
       ag_CM2_0 = der(v_CM2_0) - world.gravityAcceleration(r_CM2_0);
       der(e_rel_0) = der(r_rel_0/sqrt(r_rel_0*r_rel_0))
                    = 1/L*(I - e_rel_0*e_rel_0')*der(r_rel_0)
                    = 1/L*(I - e_rel_0*e_rel_0')*(vb - va)
       v_CM1_0 = va + L_a/L*(I - e_rel_0*e_rel_0')*(vb - va)
       v_CM2_0 = vb - L_b/L*(I - e_rel_0*e_rel_0')*(vb - va)
     - Power balance for the connection line
       (f1=force on frame_a side, f2=force on frame_b side)
       0 = f1*va - m_a*ag_CM1*v_CM1 + f2*vb - m_b*ag_CM2*v_CM2
         = f1*va - m_a*ag_CM1*(va + L_a/L*(I - e_rel*e_rel')*(vb - va)) +
           f2*vb - m_b*ag_CM2*(vb - L_b/L*(I - e_rel*e_rel')*(vb - va))
         = (f1 - m_a*ag_CM1*(I - L_a/L*(I - e_rel*e_rel'))
               - m_b*ag_CM2*(L_b/L*(I - e_rel*e_rel')))*va +
           (f2 - m_b*ag_CM2*(I - L_b/L*(I - e_rel_0*e_rel_0'))
               - m_a*ag_CM1*(L_a/L*(I - e_rel*e_rel')))*vb
         = va*(f1 - m_a*ag_CM1 +
               (m_a*ag_CM1*L_a/L - m_b*ag_CM2*L_b/L)*(I - e_rel*e_rel')) +
           vb*(f2 - m_b*ag_CM2 +
               (m_b*ag_CM2*L_b/L - m_a*ag_CM1*L_a/L)*(I - e_rel*e_rel'))
       since va and vb are completely independent from other
       the parenthesis must vanish:
         f1 := m_a*ag_CM1 - (m_a*ag_CM1*L_a/L - m_b*ag_CM2*L_b/L)*(I - e_rel*e_rel')
         f2 := m_b*ag_CM2 + (m_a*ag_CM1*L_a/L - m_b*ag_CM2*L_b/L)*(I - e_rel*e_rel')
       or
         aux1 := ag_CM1*(m_a*L_a/L) - ag_CM2*(m_b*L_b/L);
         aux2 := aux1 - (aux1'*e_rel)*e_rel
         f1 := m_a*ag_CM1 - aux2
         f2 := m_b*ag_CM2 + aux2
     - Force balance on frame_a and frame_b finally results in
         0 = frame_a.f + e_rel_a*fa - f1_a
         0 = frame_b.f + e_rel_b*fb - f2_b
       and therefore
         frame_a.f = -e_rel_a*fa + m_a*ag_CM1 - aux2
         frame_b.f = -e_rel_b*fb + m_b*ag_CM2 + aux2
  */
          if m_a > 0 or m_b > 0 then
            r_CM1_0 = frame_a.r_0 + e_rel_0 * L_a;
            r_CM2_0 = frame_b.r_0 - e_rel_0 * L_b;
            v_CM1_0 = der(r_CM1_0);
            v_CM2_0 = der(r_CM2_0);
            ag_CM1_0 = der(v_CM1_0) - world.gravityAcceleration(r_CM1_0);
            ag_CM2_0 = der(v_CM2_0) - world.gravityAcceleration(r_CM2_0);
            aux1_0 = ag_CM1_0 * (m_a * L_a / length) - ag_CM2_0 * (m_b * L_b / length);
            aux2_0 = aux1_0 - aux1_0 * e_rel_0 * e_rel_0;
            frame_a.f = Frames.resolve2(frame_a.R, m_a * ag_CM1_0 - aux2_0 - e_rel_0 * fa);
            frame_b.f = Frames.resolve2(frame_b.R, m_b * ag_CM2_0 + aux2_0 - e_rel_0 * fb);
          else
            r_CM1_0 = zeros(3);
            r_CM2_0 = zeros(3);
            v_CM1_0 = zeros(3);
            v_CM2_0 = zeros(3);
            ag_CM1_0 = zeros(3);
            ag_CM2_0 = zeros(3);
            aux1_0 = zeros(3);
            aux2_0 = zeros(3);
            frame_a.f = -Frames.resolve2(frame_a.R, e_rel_0 * fa);
            frame_b.f = -Frames.resolve2(frame_b.R, e_rel_0 * fb);
          end if;
// Provide appropriate equations, if direct connections of line forces
          if fixedRotationAtFrame_a then
            Connections.root(frame_a.R);
            frame_a.R = Frames.nullRotation();
          else
            frame_a.t = zeros(3);
          end if;
          if fixedRotationAtFrame_b then
            Connections.root(frame_b.R);
            frame_b.R = Frames.nullRotation();
          else
            frame_b.t = zeros(3);
          end if;
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-100, -40}, {-20, 40}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.Sphere, fillColor = {192, 192, 192}), Ellipse(extent = {{-90, -30}, {-30, 30}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Ellipse(extent = {{20, -40}, {100, 40}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.Sphere, fillColor = {192, 192, 192}), Ellipse(extent = {{31, -29}, {91, 30}}, lineColor = {128, 128, 128}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-150, -55}, {150, -95}}, textString = "%name", lineColor = {0, 0, 255}), Rectangle(extent = {{-52, 40}, {48, -40}}, lineColor = {255, 255, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-74, 15}, {-45, -13}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.Sphere, fillColor = {192, 192, 192}), Ellipse(extent = {{45, 14}, {74, -14}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.Sphere, fillColor = {192, 192, 192}), Line(points = {{-60, 0}, {-60, 23}, {-30, 23}, {-30, 70}, {-60, 70}, {-60, 101}}, color = {0, 0, 0}), Line(points = {{60, 0}, {60, 20}, {30, 20}, {30, 70}, {60, 70}, {60, 100}}, color = {0, 0, 0}), Line(points = {{-23, 0}, {25, 0}}, color = {0, 0, 0}, pattern = LinePattern.Dot), Ellipse(extent = {{23, 8}, {39, -8}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-39, 8}, {-23, -8}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Line(points = {{-60, 0}, {-29, 0}}, color = {0, 0, 0}), Line(points = {{29, 0}, {60, 0}}, color = {0, 0, 0}), Ellipse(visible = fixedRotationAtFrame_a, extent = {{-70, 30}, {-130, -30}}, lineColor = {255, 0, 0}), Text(visible = fixedRotationAtFrame_a, extent = {{-62, 50}, {-140, 30}}, lineColor = {255, 0, 0}, textString = "R=0"), Ellipse(visible = fixedRotationAtFrame_b, extent = {{70, 30}, {130, -30}}, lineColor = {255, 0, 0}), Text(visible = fixedRotationAtFrame_b, extent = {{62, 50}, {140, 30}}, lineColor = {255, 0, 0}, textString = "R=0")}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-60, 80}, {46, 80}}, color = {0, 0, 255}), Polygon(points = {{60, 80}, {45, 86}, {45, 74}, {60, 80}}, lineColor = {0, 0, 255}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-42, 91}, {30, 79}}, textString = "length", lineColor = {0, 0, 255}), Ellipse(extent = {{-100, -40}, {-20, 40}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.Sphere, fillColor = {192, 192, 192}), Ellipse(extent = {{-90, -30}, {-30, 30}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Ellipse(extent = {{20, -40}, {100, 40}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.Sphere, fillColor = {192, 192, 192}), Ellipse(extent = {{31, -29}, {91, 30}}, lineColor = {128, 128, 128}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-49, 39}, {51, -41}}, lineColor = {255, 255, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-74, 15}, {-45, -13}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.Sphere, fillColor = {192, 192, 192}), Ellipse(extent = {{45, 15}, {74, -13}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.Sphere, fillColor = {192, 192, 192}), Line(points = {{-60, 0}, {-60, 24}, {-40, 24}, {-40, 60}, {-60, 60}, {-60, 110}}, color = {0, 0, 0}), Line(points = {{60, 1}, {60, 21}, {40, 21}, {40, 60}, {60, 60}, {60, 110}}, color = {0, 0, 0}), Line(points = {{-60, 0}, {60, 0}}, color = {0, 0, 0}, pattern = LinePattern.Dot), Ellipse(extent = {{20, 8}, {36, -8}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Line(points = {{-18, -18}, {11, -18}}, color = {0, 0, 255}), Polygon(points = {{23, -18}, {11, -15}, {11, -21}, {23, -18}}, lineColor = {0, 0, 255}), Line(points = {{-60, 16}, {-37, 16}}, color = {0, 0, 255}), Line(points = {{-25, 0}, {-25, 20}}, color = {0, 0, 255}), Text(extent = {{-38, -20}, {33, -35}}, lineColor = {0, 0, 0}, textString = "e_rel_0"), Polygon(points = {{-25, 16}, {-37, 19}, {-37, 13}, {-25, 16}}, lineColor = {0, 0, 255}), Text(extent = {{-39, 31}, {-22, 21}}, lineColor = {0, 0, 0}, textString = "L_a"), Ellipse(extent = {{-33, 7}, {-17, -9}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Line(points = {{29, 3}, {29, 22}}, color = {0, 0, 255}), Line(points = {{29, 16}, {60, 16}}, color = {0, 0, 255}), Polygon(points = {{29, 16}, {41, 19}, {41, 13}, {29, 16}}, lineColor = {0, 0, 255}), Text(extent = {{15, 36}, {32, 26}}, lineColor = {0, 0, 0}, textString = "L_b"), Line(points = {{37, 18}, {30, 27}}, pattern = LinePattern.Dot, color = {0, 0, 255})}),
            Documentation(info = "<html>
<p>
This component is used to exert a <b>line force</b>
between the origin of frame_a and the origin of frame_b
by attaching components of the <b>1-dimensional translational</b>
mechanical library of Modelica (Modelica.Mechanics.Translational)
between the two flange connectors <b>flange_a</b> and
<b>flange_b</b>. Optionally, there are <b>two point masses</b> on the line
connecting the origin of frame_a and the origin of frame_b.
These point masses approximate the <b>masses</b> of the <b>force element</b>.
The locations of the two point masses are defined by their
(fixed) distances of L_a relative to frame_a and of L_b relative
to frame_b, respectively.
</p>
<p>
In example
<a href=\"modelica://Modelica.Mechanics.MultiBody.Examples.Elementary.LineForceWithTwoMasses\">
MultiBody.Examples.Elementary.LineForceWithTwoMasses</a> the usage of this
line force element is shown and is compared with an alternative
implementation using a
<a href=\"modelica://Modelica.Mechanics.MultiBody.Joints.Assemblies.JointUPS\">
MultiBody.Joints.Assemblies.JointUPS</a> component.
The composition diagram of this example
is displayed in the figure below.
</p>

<p>
<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Examples/Elementary/LineForceWithTwoMasses1.png\">
</p>

<p>
The animation view at time = 0 is shown in the next figure.
The system on the left side in the front is the animation with
the LineForceWithTwoMasses component whereas the system on the right
side in the back is the animation with the JointUPS component.
Both implementations yield the same result. However, the implementation
with the LineForceWithTwoMasses component is simpler.
</p>

<p>
<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Examples/Elementary/LineForceWithTwoMasses2.png\">
</p>

<p>
In the translational library there is the implicit assumption that
forces of components that have only one flange connector act with
opposite sign on the bearings of the component. This assumption
is also used in the LineForceWithTwoMasses component: If a connection
is present to only one of the flange connectors, then the force
in this flange connector acts implicitly with opposite sign also
in the other flange connector.
</p>
</html>"));
        end LineForceWithTwoMasses;

        model Spring "Linear translational spring with optional mass"
          import Modelica.Mechanics.MultiBody.Types;
          extends Interfaces.PartialTwoFrames;
          parameter Boolean animation = true "= true, if animation shall be enabled";
          parameter Boolean showMass = true "= true, if point mass shall be visualized as sphere if animation=true and m>0";
          parameter SI.TranslationalSpringConstant c(final min = 0) "Spring constant";
          parameter SI.Length s_unstretched = 0 "Unstretched spring length";
          parameter SI.Mass m(min = 0) = 0 "Spring mass located on the connection line between the origin of frame_a and the origin of frame_b";
          parameter Real lengthFraction(min = 0, max = 1) = 0.5 "Location of spring mass with respect to frame_a as a fraction of the distance from frame_a to frame_b (=0: at frame_a; =1: at frame_b)";
          input SI.Distance width = world.defaultForceWidth "Width of spring" annotation(
            Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          input SI.Distance coilWidth = width / 10 "Width of spring coil" annotation(
            Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter Integer numberOfWindings = 5 "Number of spring windings" annotation(
            Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          input Types.Color color = Modelica.Mechanics.MultiBody.Types.Defaults.SpringColor "Color of spring" annotation(
            Dialog(colorSelector = true, tab = "Animation", group = "if animation = true", enable = animation));
          input Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(
            Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          input SIunits.Diameter massDiameter = max(0, (width - 2 * coilWidth) * 0.9) "Diameter of mass point sphere" annotation(
            Dialog(tab = "Animation", group = "if animation = true and showMass = true", enable = animation and showMass));
          input Types.Color massColor = Modelica.Mechanics.MultiBody.Types.Defaults.BodyColor "Color of mass point" annotation(
            Dialog(colorSelector = true, tab = "Animation", group = "if animation = true and showMass = true", enable = animation and showMass));
          parameter Boolean fixedRotationAtFrame_a = false "=true, if rotation frame_a.R is fixed (to directly connect line forces)" annotation(
            Evaluate = true,
            choices(checkBox = true),
            Dialog(tab = "Advanced", group = "If enabled, can give wrong results, see MultiBody.UsersGuide.Tutorial.ConnectionOfLineForces"));
          parameter Boolean fixedRotationAtFrame_b = false "=true, if rotation frame_b.R is fixed (to directly connect line forces)" annotation(
            Evaluate = true,
            choices(checkBox = true),
            Dialog(tab = "Advanced", group = "If enabled, can give wrong results, see MultiBody.UsersGuide.Tutorial.ConnectionOfLineForces"));
          Forces.LineForceWithMass lineForce(animateLine = animation, animateMass = showMass, m = m, lengthFraction = lengthFraction, lineShapeType = "spring", lineShapeHeight = coilWidth * 2, lineShapeWidth = width, lineShapeExtra = numberOfWindings, lineShapeColor = color, specularCoefficient = specularCoefficient, massDiameter = massDiameter, massColor = massColor, fixedRotationAtFrame_a = fixedRotationAtFrame_a, fixedRotationAtFrame_b = fixedRotationAtFrame_b) annotation(
            Placement(transformation(extent = {{-20, -20}, {20, 20}}, rotation = 0)));
          Modelica.Mechanics.Translational.Components.Spring spring(s_rel0 = s_unstretched, c = c) annotation(
            Placement(transformation(extent = {{-8, 40}, {12, 60}}, rotation = 0)));
        equation
          connect(lineForce.frame_a, frame_a) annotation(
            Line(points = {{-20, 0}, {-100, 0}}, color = {95, 95, 95}, thickness = 0.5));
          connect(lineForce.frame_b, frame_b) annotation(
            Line(points = {{20, 0}, {100, 0}}, color = {95, 95, 95}, thickness = 0.5));
          connect(spring.flange_b, lineForce.flange_b) annotation(
            Line(points = {{12, 50}, {12, 20}}, color = {0, 191, 0}));
          connect(spring.flange_a, lineForce.flange_a) annotation(
            Line(points = {{-8, 50}, {-12, 50}, {-12, 20}}, color = {0, 191, 0}));
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-100, 0}, {-58, 0}, {-43, -30}, {-13, 30}, {17, -30}, {47, 30}, {62, 0}, {100, 0}}, color = {0, 0, 0}, pattern = LinePattern.Solid, thickness = 0.25, arrow = {Arrow.None, Arrow.None}), Text(extent = {{-150, 56}, {150, 96}}, textString = "%name", lineColor = {0, 0, 255}), Text(extent = {{-150, -80}, {150, -50}}, lineColor = {0, 0, 0}, textString = "c=%c"), Ellipse(extent = {{-8, 8}, {8, -8}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Ellipse(visible = fixedRotationAtFrame_a, extent = {{-70, 30}, {-130, -30}}, lineColor = {255, 0, 0}), Text(visible = fixedRotationAtFrame_a, extent = {{-62, 50}, {-140, 30}}, lineColor = {255, 0, 0}, textString = "R=0"), Ellipse(visible = fixedRotationAtFrame_b, extent = {{70, 30}, {130, -30}}, lineColor = {255, 0, 0}), Text(visible = fixedRotationAtFrame_b, extent = {{62, 50}, {140, 30}}, lineColor = {255, 0, 0}, textString = "R=0")}),
            Documentation(info = "<HTML>
<p>
<b>Linear spring</b> acting as line force between frame_a and frame_b.
A <b>force f</b> is exerted on the origin of frame_b and with opposite sign
on the origin of frame_a along the line from the origin of frame_a to the origin
of frame_b according to the equation:
</p>
<pre>
   f = c*(s - s_unstretched);
</pre>
<p>
where \"c\" and \"s_unstretched\" are parameters and \"s\" is the
distance between the origin of frame_a and the origin of frame_b.
</p>
<p>
Optionally, the mass of the spring is taken into account by a
point mass located on the line between frame_a and frame_b
(default: middle of the line). If the spring mass is zero, the
additional equations to handle the mass are removed.
</p>
<p>
In the following figure a typical animation of the
spring is shown. The blue sphere in the middle of the
spring characterizes the location of the point mass.
</p>

<p>
<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Examples/Elementary/SpringWithMass.png\"
ALT=\"model Examples.Elementary.SpringWithMass\">
</p>

</HTML>"));
        end Spring;

        model Damper "Linear (velocity dependent) damper"
          import Modelica.Mechanics.MultiBody.Types;
          parameter Boolean animation = true "= true, if animation shall be enabled";
          parameter SI.TranslationalDampingConstant d(final min = 0, start = 0) "Damping constant";
          parameter SI.Distance length_a = world.defaultForceLength "Length of cylinder at frame_a side" annotation(
            Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          input SIunits.Diameter diameter_a = world.defaultForceWidth "Diameter of cylinder at frame_a side" annotation(
            Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          input SIunits.Diameter diameter_b = 0.6 * diameter_a "Diameter of cylinder at frame_b side" annotation(
            Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          input Types.Color color_a = {100, 100, 100} "Color at frame_a" annotation(
            Dialog(colorSelector = true, tab = "Animation", group = "if animation = true", enable = animation));
          input Types.Color color_b = {155, 155, 155} "Color at frame_b" annotation(
            Dialog(colorSelector = true, tab = "Animation", group = "if animation = true", enable = animation));
          input Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(
            Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          extends Interfaces.PartialLineForce;
          extends Modelica.Thermal.HeatTransfer.Interfaces.PartialElementaryConditionalHeatPort(final T = 293.15);
        protected
          SI.Position r0_b[3] = e_a * noEvent(min(length_a, s));
          Visualizers.Advanced.Shape shape_a(shapeType = "cylinder", color = color_a, specularCoefficient = specularCoefficient, length = noEvent(min(length_a, s)), width = diameter_a, height = diameter_a, lengthDirection = e_a, widthDirection = {0, 1, 0}, r = frame_a.r_0, R = frame_a.R) if world.enableAnimation and animation;
          Visualizers.Advanced.Shape shape_b(shapeType = "cylinder", color = color_b, specularCoefficient = specularCoefficient, length = noEvent(max(s - length_a, 0)), width = diameter_b, height = diameter_b, lengthDirection = e_a, widthDirection = {0, 1, 0}, r_shape = r0_b, r = frame_a.r_0, R = frame_a.R) if world.enableAnimation and animation;
        equation
          f = d * der(s);
          lossPower = f * der(s);
          annotation(
            Documentation(info = "<HTML>
<p>
<b>Linear damper</b> acting as line force between frame_a and frame_b.
A <b>force f</b> is exerted on the origin of frame_b and with opposite sign
on the origin of frame_a along the line from the origin of frame_a to the origin
of frame_b according to the equation:
</p>
<pre>
   f = d*<b>der</b>(s);
</pre>
<p>
where \"d\" is a parameter, \"s\" is the
distance between the origin of frame_a and the origin of frame_b
and der(s) is the time derivative of \"s\".
</p>
<p>
In the following figure a typical animation is shown
where a mass is hanging on a damper.
</p>

<p>
<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Forces/Damper.png\">
</p>

</HTML>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-101, 0}, {-60, 0}}, color = {0, 0, 0}), Line(points = {{-60, -30}, {-60, 30}}, color = {0, 0, 0}), Line(points = {{-60, -30}, {60, -30}}, color = {0, 0, 0}), Line(points = {{-60, 30}, {60, 30}}, color = {0, 0, 0}), Rectangle(extent = {{-60, 30}, {30, -30}}, lineColor = {0, 0, 0}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{30, 0}, {100, 0}}, color = {0, 0, 0}), Text(extent = {{-150, 50}, {150, 90}}, textString = "%name", lineColor = {0, 0, 255}), Text(extent = {{-150, -75}, {150, -45}}, lineColor = {0, 0, 0}, textString = "d=%d"), Line(visible = useHeatPort, points = {{-100, -99}, {-100, -25}, {-10, -25}}, color = {191, 0, 0}, pattern = LinePattern.Dot, smooth = Smooth.None)}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-100, 0}, {-60, 0}}, color = {0, 0, 0}), Line(points = {{-60, -30}, {-60, 30}}, color = {0, 0, 0}), Line(points = {{-60, -30}, {60, -30}}, color = {0, 0, 0}), Line(points = {{-60, 30}, {60, 30}}, color = {0, 0, 0}), Rectangle(extent = {{-60, 30}, {30, -30}}, lineColor = {0, 0, 0}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{30, 0}, {100, 0}}, color = {0, 0, 0}), Line(points = {{-50, 60}, {50, 60}}, color = {128, 128, 128}), Text(extent = {{-40, 64}, {23, 77}}, lineColor = {128, 128, 128}, textString = "der(s)"), Polygon(points = {{64, 60}, {42, 68}, {42, 52}, {62, 60}, {64, 60}}, lineColor = {128, 128, 128}, fillColor = {160, 160, 164}, fillPattern = FillPattern.Solid)}));
        end Damper;

        model SpringDamperParallel "Linear spring and linear damper in parallel"
          import Modelica.Mechanics.MultiBody.Types;
          parameter Boolean animation = true "= true, if animation shall be enabled";
          parameter SI.TranslationalSpringConstant c(final min = 0) "Spring constant";
          parameter SI.Length s_unstretched = 0 "Unstretched spring length";
          parameter SI.TranslationalDampingConstant d(final min = 0) = 0 "Damping constant";
          input SI.Distance width = world.defaultForceWidth "Width of spring" annotation(
            Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          input SI.Distance coilWidth = width / 10 "Width of spring coil" annotation(
            Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter Integer numberOfWindings = 5 "Number of spring windings" annotation(
            Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          input Types.Color color = Modelica.Mechanics.MultiBody.Types.Defaults.SpringColor "Color of spring" annotation(
            Dialog(colorSelector = true, tab = "Animation", group = "if animation = true", enable = animation));
          input Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(
            Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          extends Interfaces.PartialLineForce;
          extends Modelica.Thermal.HeatTransfer.Interfaces.PartialElementaryConditionalHeatPort(final T = 293.15);
        protected
          Modelica.SIunits.Force f_d "Damping force";
          Visualizers.Advanced.Shape shape(shapeType = "spring", color = color, length = s, width = width, height = coilWidth * 2, lengthDirection = e_a, widthDirection = {0, 1, 0}, extra = numberOfWindings, r = frame_a.r_0, R = frame_a.R) if world.enableAnimation and animation;
        equation
          f_d = d * der(s);
          f = c * (s - s_unstretched) + f_d;
          lossPower = f_d * der(s);
          annotation(
            Documentation(info = "<HTML>
<p>
<b>Linear spring</b> and <b>linear damper</b>
in parallel acting as line force between frame_a and frame_b.
A <b>force f</b> is exerted on the origin of frame_b and with opposite sign
on the origin of frame_a along the line from the origin of frame_a to the origin
of frame_b according to the equation:
</p>
<pre>
   f = c*(s - s_unstretched) + d*<b>der</b>(s);
</pre>
<p>
where \"c\", \"s_unstretched\" and \"d\" are parameters, \"s\" is the
distance between the origin of frame_a and the origin of frame_b
and der(s) is the time derivative of s.
</p>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-150, -150}, {150, -110}}, textString = "%name", lineColor = {0, 0, 255}), Line(points = {{-80, 40}, {-60, 40}, {-45, 10}, {-15, 70}, {15, 10}, {45, 70}, {60, 40}, {80, 40}}, color = {0, 0, 0}), Line(points = {{-80, 40}, {-80, -70}}, color = {0, 0, 0}), Line(points = {{-80, -70}, {-52, -70}}, color = {0, 0, 0}), Rectangle(extent = {{-52, -40}, {38, -100}}, lineColor = {0, 0, 0}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-52, -40}, {68, -40}}, color = {0, 0, 0}), Line(points = {{-52, -100}, {68, -100}}, color = {0, 0, 0}), Line(points = {{38, -70}, {80, -70}}, color = {0, 0, 0}), Line(points = {{80, 40}, {80, -70}}, color = {0, 0, 0}), Line(points = {{-100, 0}, {-80, 0}}, color = {0, 0, 0}), Line(points = {{80, 0}, {100, 0}}, color = {0, 0, 0}), Text(extent = {{-150, 70}, {150, 100}}, lineColor = {0, 0, 0}, textString = "d=%d"), Line(visible = useHeatPort, points = {{-100, -101}, {-100, -80}, {-6, -80}}, color = {191, 0, 0}, pattern = LinePattern.Dot, smooth = Smooth.None), Text(extent = {{-150, 105}, {150, 135}}, lineColor = {0, 0, 0}, textString = "c=%c")}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-80, 32}, {-58, 32}, {-43, 2}, {-13, 62}, {17, 2}, {47, 62}, {62, 32}, {80, 32}}, color = {0, 0, 0}, thickness = 0.5), Line(points = {{-68, 32}, {-68, 97}}, color = {128, 128, 128}), Line(points = {{72, 32}, {72, 97}}, color = {128, 128, 128}), Line(points = {{-68, 92}, {72, 92}}, color = {128, 128, 128}), Polygon(points = {{62, 95}, {72, 92}, {62, 89}, {62, 95}}, lineColor = {128, 128, 128}, fillColor = {128, 128, 128}, fillPattern = FillPattern.Solid), Text(extent = {{-20, 72}, {20, 97}}, lineColor = {0, 0, 255}, textString = "s"), Rectangle(extent = {{-52, -20}, {38, -80}}, lineColor = {0, 0, 0}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-52, -80}, {68, -80}}, color = {0, 0, 0}), Line(points = {{-52, -20}, {68, -20}}, color = {0, 0, 0}), Line(points = {{38, -50}, {80, -50}}, color = {0, 0, 0}), Line(points = {{-80, -50}, {-52, -50}}, color = {0, 0, 0}), Line(points = {{-80, 32}, {-80, -50}}, color = {0, 0, 0}), Line(points = {{80, 32}, {80, -50}}, color = {0, 0, 0}), Line(points = {{-100, 0}, {-80, 0}}, color = {0, 0, 0}), Line(points = {{100, 0}, {80, 0}}, color = {0, 0, 0})}));
        end SpringDamperParallel;

        model SpringDamperSeries "Linear spring and linear damper in series connection"
          parameter SI.TranslationalSpringConstant c(final min = 0) "Spring constant";
          parameter SI.Length s_unstretched = 0 "Unstretched spring length";
          parameter SI.TranslationalDampingConstant d(final min = 0) = 0 "Damping constant";
          parameter SI.Length s_damper_start = 0 "Initial length of damper";
          SI.Position s_damper(start = s_damper_start, fixed = true) "Actual length of damper (frame_a - damper - spring - frame_b)";
          extends Interfaces.PartialLineForce;
          extends Modelica.Thermal.HeatTransfer.Interfaces.PartialElementaryConditionalHeatPort(final T = 293.15);
        equation
          f = c * (s - s_unstretched - s_damper);
          d * der(s_damper) = f;
          lossPower = f * der(s_damper);
          annotation(
            Documentation(info = "<HTML>
<p>
<b>Linear spring</b> and <b>linear damper</b> in series connection
acting as line force between frame_a and frame_b:
</p>
<pre>
  frame_a --> damper ----> spring --> frame_b
          |              |
          |-- s_damper --|  (s_damper is the state variable of this system)
</pre>
<p>
A <b>force f</b> is exerted on the origin of frame_b and with opposite sign
on the origin of frame_a along the line from the origin of frame_a to the origin
of frame_b according to the equations:
</p>
<pre>
   f = c*(s - s_unstretched - s_damper);
   f = d*der(s_damper);
</pre>
<p>
where \"c\", \"s_unstretched\" and \"d\" are parameters, \"s\" is the
distance between the origin of frame_a and the origin of frame_b.
\"s_damper\" is the length of the damper (= an internal state of this
force element) and der(s_damper) is the time derivative of s_damper.
</p>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-100, 0}, {-15, 0}}, color = {0, 0, 0}), Line(points = {{-60, -30}, {-15, -30}}, color = {0, 0, 0}), Line(points = {{-60, 30}, {-15, 30}}, color = {0, 0, 0}), Rectangle(extent = {{-60, 30}, {-30, -30}}, lineColor = {0, 0, 0}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-15, 0}, {-5, 0}, {5, -30}, {25, 30}, {45, -30}, {65, 30}, {75, 0}, {100, 0}}, color = {0, 0, 0}), Text(extent = {{-150, 50}, {150, 90}}, textString = "%name", lineColor = {0, 0, 255}), Text(extent = {{-150, -65}, {150, -35}}, lineColor = {0, 0, 0}, textString = "c=%c"), Line(visible = useHeatPort, points = {{-100, -99}, {-100, -24}, {-45, -24}}, color = {191, 0, 0}, smooth = Smooth.None, pattern = LinePattern.Dot), Text(extent = {{-150, -100}, {150, -70}}, lineColor = {0, 0, 0}, textString = "d=%d")}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-100, 0}, {-15, 0}}, color = {0, 0, 0}), Line(points = {{-60, -30}, {-15, -30}}, color = {0, 0, 0}), Line(points = {{-60, 30}, {-15, 30}}, color = {0, 0, 0}), Rectangle(extent = {{-60, 30}, {-30, -30}}, lineColor = {0, 0, 0}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-15, 0}, {-5, 0}, {5, -30}, {25, 30}, {45, -30}, {65, 30}, {75, 0}, {99, 0}}, color = {0, 0, 0}), Line(points = {{-75, 0}, {-75, 85}}, color = {160, 160, 164}), Line(points = {{-10, 0}, {-10, 65}}, color = {160, 160, 164}), Line(points = {{80, 0}, {80, 85}}, color = {160, 160, 164}), Line(points = {{-75, 80}, {80, 80}}, color = {160, 160, 164}), Line(points = {{-75, 60}, {-10, 60}}, color = {160, 160, 164}), Polygon(points = {{-10, 60}, {-20, 65}, {-20, 55}, {-10, 60}}, lineColor = {160, 160, 164}, fillColor = {160, 160, 164}, fillPattern = FillPattern.Solid), Polygon(points = {{80, 80}, {70, 85}, {70, 75}, {80, 80}}, lineColor = {160, 160, 164}, fillColor = {160, 160, 164}, fillPattern = FillPattern.Solid), Text(extent = {{-72, 63}, {-11, 78}}, lineColor = {160, 160, 164}, textString = "s_damper"), Text(extent = {{0, 80}, {20, 100}}, lineColor = {160, 160, 164}, textString = "s")}));
        end SpringDamperSeries;

        package Internal "Internal package, should not be used by user"
          extends Modelica.Icons.InternalPackage;

          model BasicForce "Force acting between two frames, defined by 3 input signals"
            extends Modelica.Mechanics.MultiBody.Interfaces.PartialTwoFrames;
            import Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB;
            Interfaces.Frame_resolve frame_resolve "The input signals are optionally resolved in this frame" annotation(
              Placement(transformation(origin = {40, 100}, extent = {{-16, -16}, {16, 16}}, rotation = 90)));
            Modelica.Blocks.Interfaces.RealInput force[3](each final quantity = "Force", each final unit = "N") "x-, y-, z-coordinates of force resolved in frame defined by resolveInFrame" annotation(
              Placement(transformation(origin = {-60, 120}, extent = {{-20, -20}, {20, 20}}, rotation = 270)));
            parameter Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB resolveInFrame = Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB.frame_b "Frame in which force is resolved (1: world, 2: frame_a, 3: frame_b, 4: frame_resolve)";
            Modelica.SIunits.Position r_0[3] "Position vector from origin of frame_a to origin of frame_b resolved in world frame";
            Modelica.SIunits.Force f_b_0[3] "frame_b.f resolved in world frame";
          equation
            assert(cardinality(frame_resolve) > 0, "Connector frame_resolve must be connected at least once and frame_resolve.r_0/.R must be set");
            frame_resolve.f = zeros(3);
            frame_resolve.t = zeros(3);
            if resolveInFrame == ResolveInFrameAB.frame_a then
              f_b_0 = -Frames.resolve1(frame_a.R, force);
              frame_b.f = Frames.resolve2(frame_b.R, f_b_0);
            elseif resolveInFrame == ResolveInFrameAB.frame_b then
              f_b_0 = -Frames.resolve1(frame_b.R, force);
              frame_b.f = -force;
            elseif resolveInFrame == ResolveInFrameAB.world then
              f_b_0 = -force;
              frame_b.f = Frames.resolve2(frame_b.R, f_b_0);
            elseif resolveInFrame == ResolveInFrameAB.frame_resolve then
              f_b_0 = -Frames.resolve1(frame_resolve.R, force);
              frame_b.f = Frames.resolve2(frame_b.R, f_b_0);
            else
              assert(false, "Wrong value for parameter resolveInFrame");
              f_b_0 = zeros(3);
              frame_b.f = zeros(3);
            end if;
            frame_b.t = zeros(3);
// Force and torque balance
            r_0 = frame_b.r_0 - frame_a.r_0;
            zeros(3) = frame_a.f + Frames.resolve2(frame_a.R, f_b_0);
            zeros(3) = frame_a.t + Frames.resolve2(frame_a.R, cross(r_0, f_b_0));
            annotation(
              Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-98, 99}, {99, -98}}, lineColor = {255, 255, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-92, 61}, {87, 35}}, lineColor = {192, 192, 192}, textString = "resolve"), Text(extent = {{-136, -52}, {149, -113}}, textString = "%name", lineColor = {0, 0, 255}), Line(points = {{40, 100}, {40, 0}}, color = {95, 95, 95}, pattern = LinePattern.Dot), Polygon(points = {{-94, 0}, {-64, 11}, {-64, -10}, {-94, 0}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Line(points = {{-60, 100}, {40, 100}}, color = {95, 95, 95}, pattern = LinePattern.Dot), Polygon(points = {{94, 0}, {65, 12}, {65, -11}, {94, 0}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Line(points = {{-64, 0}, {-20, 0}}, color = {0, 0, 0}), Line(points = {{20, 0}, {65, 0}}, color = {0, 0, 0})}),
              Documentation(info = "<HTML>
<p>
The <b>3</b> signals of the <b>force</b> connector are interpreted
as the x-, y- and z-coordinates of a <b>force</b> acting at the frame
connector to which frame_b of this component is attached.
Via parameter <b>resolveInFrame</b> it is defined, in which frame these
coordinates shall be resolved:
</p>

<table border=1 cellspacing=0 cellpadding=2>
<tr><th><b>Types.ResolveInFrameAB.</b></th><th><b>Meaning</b></th></tr>
<tr><td valign=\"top\">world</td>
    <td valign=\"top\">Resolve input force in world frame</td></tr>

<tr><td valign=\"top\">frame_a</td>
    <td valign=\"top\">Resolve input force in frame_a</td></tr>

<tr><td valign=\"top\">frame_b</td>
    <td valign=\"top\">Resolve input force in frame_b (= default)</td></tr>

<tr><td valign=\"top\">frame_resolve</td>
    <td valign=\"top\">Resolve input force in frame_resolve (frame_resolve must be connected)</td></tr>
</table>

<p>
If resolveInFrame = ResolveInFrameAB.frame_resolve, the force coordinates
are with respect to the frame, that is connected to <b>frame_resolve</b>.
</p>

<p>
If resolveInFrame is not ResolveInFrameAB.frame_resolve, then the position
vector and the orientation object of frame_resolve must be set to constant
values from the outside in order that the model remains balanced
(these constant values are ignored).
</p>

</html>"));
          end BasicForce;

          model BasicTorque "Torque acting between two frames, defined by 3 input signals"
            import Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB;
            extends Modelica.Mechanics.MultiBody.Interfaces.PartialTwoFrames;
            Interfaces.Frame_resolve frame_resolve "The input signals are optionally resolved in this frame" annotation(
              Placement(transformation(origin = {40, 100}, extent = {{-16, -16}, {16, 16}}, rotation = 90)));
            Modelica.Blocks.Interfaces.RealInput torque[3](each final quantity = "Torque", each final unit = "N.m") "x-, y-, z-coordinates of torque resolved in frame defined by resolveInFrame" annotation(
              Placement(transformation(origin = {-60, 120}, extent = {{-20, -20}, {20, 20}}, rotation = 270)));
            parameter Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB resolveInFrame = Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB.frame_b "Frame in which torque is resolved (1: world, 2: frame_a, 3: frame_b, 4: frame_resolve)";
            SI.Position r_0[3] "Position vector from origin of frame_a to origin of frame_b resolved in world frame";
            SI.Torque t_b_0[3] "frame_b.t resolved in world frame";
          equation
            assert(cardinality(frame_resolve) > 0, "Connector frame_resolve must be connected at least once and frame_resolve.r_0/.R must be set");
            frame_resolve.f = zeros(3);
            frame_resolve.t = zeros(3);
            r_0 = frame_b.r_0 - frame_a.r_0;
            frame_a.f = zeros(3);
            frame_b.f = zeros(3);
            if resolveInFrame == ResolveInFrameAB.frame_a then
              t_b_0 = -Frames.resolve1(frame_a.R, torque);
              frame_b.t = Frames.resolve2(frame_b.R, t_b_0);
            elseif resolveInFrame == ResolveInFrameAB.frame_b then
              t_b_0 = -Frames.resolve1(frame_b.R, torque);
              frame_b.t = -torque;
            elseif resolveInFrame == ResolveInFrameAB.world then
              t_b_0 = -torque;
              frame_b.t = Frames.resolve2(frame_b.R, t_b_0);
            elseif resolveInFrame == ResolveInFrameAB.frame_resolve then
              t_b_0 = -Frames.resolve1(frame_resolve.R, torque);
              frame_b.t = Frames.resolve2(frame_b.R, t_b_0);
            else
              assert(false, "Wrong value for parameter resolveInFrame");
              t_b_0 = zeros(3);
              frame_b.t = zeros(3);
            end if;
// torque balance
            zeros(3) = frame_a.t + Frames.resolve2(frame_a.R, t_b_0);
            annotation(
              Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-98, 99}, {99, -98}}, lineColor = {255, 255, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-59, 55}, {72, 30}}, lineColor = {192, 192, 192}, textString = "resolve"), Text(extent = {{-139, -27}, {146, -88}}, textString = "%name", lineColor = {0, 0, 255}), Polygon(points = {{100, 20}, {84, 52}, {69, 39}, {100, 20}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Line(points = {{40, 100}, {76, 46}}, color = {95, 95, 95}, pattern = LinePattern.Dot), Polygon(points = {{-99, 20}, {-86, 53}, {-70, 42}, {-99, 20}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Line(points = {{-60, 100}, {40, 100}}, color = {95, 95, 95}, pattern = LinePattern.Dot), Line(points = {{-79, 47}, {-70, 61}, {-59, 72}, {-45, 81}, {-32, 84}, {-20, 85}}, color = {0, 0, 0}), Line(points = {{77, 45}, {66, 60}, {55, 69}, {49, 74}, {41, 80}, {31, 84}, {20, 85}}, color = {0, 0, 0})}),
              Documentation(info = "<HTML>
<p>
The <b>3</b> signals of the <b>torque</b> connector are interpreted
as the x-, y- and z-coordinates of a <b>torque</b> acting at the frame
connector to which frame_b of this component is attached.
Via parameter <b>resolveInFrame</b> it is defined, in which frame these
coordinates shall be resolved:
</p>

<table border=1 cellspacing=0 cellpadding=2>
<tr><th><b>Types.ResolveInFrameAB.</b></th><th><b>Meaning</b></th></tr>
<tr><td valign=\"top\">world</td>
    <td valign=\"top\">Resolve input torque in world frame</td></tr>

<tr><td valign=\"top\">frame_a</td>
    <td valign=\"top\">Resolve input torque in frame_a</td></tr>

<tr><td valign=\"top\">frame_b</td>
    <td valign=\"top\">Resolve input torque in frame_b (= default)</td></tr>

<tr><td valign=\"top\">frame_resolve</td>
    <td valign=\"top\">Resolve input torque in frame_resolve (frame_resolve must be connected)</td></tr>
</table>

<p>
If resolveInFrame = ResolveInFrameAB.frame_resolve, the torque coordinates
are with respect to the frame, that is connected to <b>frame_resolve</b>.
</p>

<p>
If resolveInFrame is not ResolveInFrameAB.frame_resolve, then the position
vector and the orientation object of frame_resolve must be set to constant
values from the outside in order that the model remains balanced
(these constant values are ignored).
</p>
</html>"));
          end BasicTorque;

          model BasicWorldForce "External force acting at frame_b, defined by 3 input signals"
            import Modelica.Mechanics.MultiBody.Types.ResolveInFrameB;
            extends Interfaces.PartialOneFrame_b;
            Interfaces.Frame_resolve frame_resolve "The input signals are optionally resolved in this frame" annotation(
              Placement(transformation(origin = {0, -100}, extent = {{-16, -16}, {16, 16}}, rotation = 270)));
            Modelica.Blocks.Interfaces.RealInput force[3](each final quantity = "Force", each final unit = "N") "x-, y-, z-coordinates of force resolved in frame defined by resolveInFrame" annotation(
              Placement(transformation(extent = {{-140, -20}, {-100, 20}}, rotation = 0)));
            parameter Modelica.Mechanics.MultiBody.Types.ResolveInFrameB resolveInFrame = Modelica.Mechanics.MultiBody.Types.ResolveInFrameB.world "Frame in which force is resolved (1: world, 2: frame_b, 3: frame_resolve)";
          equation
            assert(cardinality(frame_resolve) > 0, "Connector frame_resolve must be connected at least once and frame_resolve.r_0/.R must be set");
            frame_resolve.f = zeros(3);
            frame_resolve.t = zeros(3);
            if resolveInFrame == ResolveInFrameB.world then
              frame_b.f = -Frames.resolve2(frame_b.R, force);
            elseif resolveInFrame == ResolveInFrameB.frame_b then
              frame_b.f = -force;
            elseif resolveInFrame == ResolveInFrameB.frame_resolve then
              frame_b.f = -Frames.resolveRelative(force, frame_resolve.R, frame_b.R);
            else
              assert(false, "Wrong value for parameter resolveInFrame");
              frame_b.f = zeros(3);
            end if;
            frame_b.t = zeros(3);
            annotation(
              Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{-100, 10}, {50, 10}, {50, 31}, {97, 0}, {50, -31}, {50, -10}, {-100, -10}, {-100, 10}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Line(points = {{0, -10}, {0, -97}}, color = {95, 95, 95}, pattern = LinePattern.Dot)}),
              Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-89, -46}, {91, -76}}, lineColor = {192, 192, 192}, textString = "resolve"), Polygon(points = {{-100, 10}, {50, 10}, {50, 31}, {94, 0}, {50, -31}, {50, -10}, {-100, -10}, {-100, 10}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Text(extent = {{-149, 103}, {136, 42}}, textString = "%name", lineColor = {0, 0, 255}), Line(points = {{0, -10}, {0, -95}}, color = {95, 95, 95}, pattern = LinePattern.Dot)}),
              Documentation(info = "<HTML>
<p>
The 3 signals of the <b>force</b> connector are interpreted
as the x-, y- and z-coordinates of a <b>force</b> acting at the frame
connector to which this component is attached.
Via parameter <b>resolveInFrame</b> it is defined, in which frame these
coordinates shall be resolved:
</p>

<table border=1 cellspacing=0 cellpadding=2>
<tr><th><b>Types.ResolveInFrameB.</b></th><th><b>Meaning</b></th></tr>
<tr><td valign=\"top\">world</td>
    <td valign=\"top\">Resolve input force in world frame (= default)</td></tr>

<tr><td valign=\"top\">frame_b</td>
    <td valign=\"top\">Resolve input force in frame_b</td></tr>

<tr><td valign=\"top\">frame_resolve</td>
    <td valign=\"top\">Resolve input force in frame_resolve (frame_resolve must be connected)</td></tr>
</table>

<p>
If resolveInFrame = Types.ResolveInFrameB.frame_resolve, the force coordinates
are with respect to the frame, that is connected to <b>frame_resolve</b>.
</p>

<p>
If resolveInFrame is not Types.ResolveInFrameB.frame_resolve, then the position
vector and the orientation object of frame_resolve must be set to constant
values from the outside in order that the model remains balanced
(these constant values are ignored).
</p>

</html>"));
          end BasicWorldForce;

          model BasicWorldTorque "External torque acting at frame_b, defined by 3 input signals"
            import Modelica.Mechanics.MultiBody.Types.ResolveInFrameB;
            extends Interfaces.PartialOneFrame_b;
            Interfaces.Frame_resolve frame_resolve "The input signals are optionally resolved in this frame" annotation(
              Placement(transformation(origin = {0, 100}, extent = {{16, -16}, {-16, 16}}, rotation = 270)));
            Modelica.Blocks.Interfaces.RealInput torque[3](each final quantity = "Torque", each final unit = "N.m") "x-, y-, z-coordinates of torque resolved in frame defined by resolveInFrame" annotation(
              Placement(transformation(extent = {{-140, -20}, {-100, 20}}, rotation = 0)));
            parameter Modelica.Mechanics.MultiBody.Types.ResolveInFrameB resolveInFrame = Modelica.Mechanics.MultiBody.Types.ResolveInFrameB.world "Frame in which torque is resolved (1: world, 2: frame_b, 3: frame_resolve)";
          equation
            assert(cardinality(frame_resolve) > 0, "Connector frame_resolve must be connected at least once and frame_resolve.r_0/.R must be set");
            frame_resolve.f = zeros(3);
            frame_resolve.t = zeros(3);
            if resolveInFrame == ResolveInFrameB.world then
              frame_b.t = -Frames.resolve2(frame_b.R, torque);
            elseif resolveInFrame == ResolveInFrameB.frame_b then
              frame_b.t = -torque;
            elseif resolveInFrame == ResolveInFrameB.frame_resolve then
              frame_b.t = -Frames.resolveRelative(torque, frame_resolve.R, frame_b.R);
            else
              assert(false, "Wrong value for parameter resolveInFrame");
              frame_b.t = zeros(3);
            end if;
            frame_b.f = zeros(3);
            annotation(
              Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{0, 97}, {0, 82}}, color = {95, 95, 95}, pattern = LinePattern.Dot), Line(points = {{-100, 0}, {-94, 13}, {-86, 28}, {-74, 48}, {-65, 60}, {-52, 72}, {-35, 81}, {-22, 84}, {-8, 84}, {7, 80}, {19, 73}, {32, 65}, {44, 55}, {52, 47}, {58, 40}}, color = {0, 0, 0}, thickness = 0.5), Polygon(points = {{97, 6}, {75, 59}, {41, 24}, {97, 6}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid)}),
              Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-61, 64}, {46, 27}}, lineColor = {192, 192, 192}, textString = "resolve"), Text(extent = {{-145, -28}, {140, -89}}, textString = "%name", lineColor = {0, 0, 255}), Line(points = {{0, 95}, {0, 82}}, color = {95, 95, 95}, pattern = LinePattern.Dot), Line(points = {{-100, 0}, {-94, 13}, {-86, 28}, {-74, 48}, {-65, 60}, {-52, 72}, {-35, 81}, {-22, 84}, {-8, 84}, {7, 80}, {19, 73}, {32, 65}, {44, 55}, {52, 47}, {58, 40}}, color = {0, 0, 0}, thickness = 0.5), Polygon(points = {{94, 10}, {75, 59}, {41, 24}, {94, 10}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid)}),
              Documentation(info = "<HTML>
<p>
The 3 signals of the <b>torque</b> connector are interpreted
as the x-, y- and z-coordinates of a <b>torque</b> acting at the frame
connector to which this component is attached.
Via parameter <b>resolveInFrame</b> it is defined, in which frame these
coordinates shall be resolved:
</p>

<table border=1 cellspacing=0 cellpadding=2>
<tr><th><b>Types.ResolveInFrameB.</b></th><th><b>Meaning</b></th></tr>
<tr><td valign=\"top\">world</td>
    <td valign=\"top\">Resolve input torque in world frame (= default)</td></tr>

<tr><td valign=\"top\">frame_b</td>
    <td valign=\"top\">Resolve input torque in frame_b</td></tr>

<tr><td valign=\"top\">frame_resolve</td>
    <td valign=\"top\">Resolve input torque in frame_resolve (frame_resolve must be connected)</td></tr>
</table>

<p>
If resolveInFrame = Types.ResolveInFrameB.frame_resolve, the torque coordinates
are with respect to the frame, that is connected to <b>frame_resolve</b>.
</p>

<p>
If resolveInFrame is not Types.ResolveInFrameB.frame_resolve, then the position
vector and the orientation object of frame_resolve must be set to constant
values from the outside in order that the model remains balanced
(these constant values are ignored).
</p>

</html>"));
          end BasicWorldTorque;

          function standardGravityAcceleration "Standard gravity fields (no/parallel/point field)"
            extends Modelica.Icons.Function;
            extends Modelica.Mechanics.MultiBody.Interfaces.partialGravityAcceleration;
            import Modelica.Mechanics.MultiBody.Types.GravityTypes;
            input GravityTypes gravityType "Type of gravity field" annotation(
              Dialog);
            input Modelica.SIunits.Acceleration g[3] "Constant gravity acceleration, resolved in world frame, if gravityType=UniformGravity" annotation(
              Dialog);
            input Real mue(unit = "m3/s2") "Field constant of point gravity field, if gravityType=PointGravity" annotation(
              Dialog);
          algorithm
            gravity := if gravityType == GravityTypes.UniformGravity then g else if gravityType == GravityTypes.PointGravity then -mue / (r * r) * (r / Modelica.Math.Vectors.length(r)) else zeros(3);
            annotation(
              Inline = true,
              Documentation(info = "<html>
<p>
This function defines the standard gravity fields for the World object.
</p>

<table border=1 cellspacing=0 cellpadding=2>
<tr><td><b><i>gravityType</i></b></td>
    <td><b><i>gravity [m/s2]</i></b></td>
    <td><b><i>description</i></b></td></tr>
<tr><td>Types.GravityType.NoGravity</td>
    <td>= {0,0,0}</td>
    <td>No gravity</td></tr>

<tr><td>Types.GravityType.UniformGravity</td>
    <td>= g</td>
    <td> Constant parallel gravity field</td></tr>

<tr><td>Types.GravityType.PointGravity</td>
    <td>= -(mue/(r*r))*r/|r|</td>
    <td> Point gravity field with spherical mass</td></tr>
</table>

</html>"));
          end standardGravityAcceleration;
        end Internal;
        annotation(
          Documentation(info = "<HTML>
<p>
This package contains components that exert forces and torques
between two frame connectors, e.g., between two parts.
</p>
<h4>Content</h4>
<table border=1 cellspacing=0 cellpadding=2>
  <tr><th><b><i>Model</i></b></th><th><b><i>Description</i></b></th></tr>
  <tr><td valign=\"top\"><a href=\"modelica://Modelica.Mechanics.MultiBody.Forces.WorldForce\">WorldForce</a></td>
      <td valign=\"top\"> External force acting at the frame to which this component
           is connected and defined by 3 input signals,
           that are interpreted as one vector resolved in frame world, frame_b or frame_resolve. <br>
           <IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Forces/ArrowForce.png\"></td>
  </tr>
  <tr><td valign=\"top\"><a href=\"modelica://Modelica.Mechanics.MultiBody.Forces.WorldTorque\">WorldTorque</a></td>
      <td valign=\"top\"> External torque acting at the frame to which this component
           is connected and defined by 3 input signals,
           that are interpreted as one vector resolved in frame world, frame_b or frame_resolve. <br>
           <IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Forces/ArrowTorque.png\"></td>
  </tr>
  <tr><td valign=\"top\"><a href=\"modelica://Modelica.Mechanics.MultiBody.Forces.WorldForceAndTorque\">WorldForceAndTorque</a></td>
      <td valign=\"top\"> External force and external torque acting at the frame
           to which this component
           is connected and defined by 3+3 input signals,
           that are interpreted as a force and as a torque vector
           resolved in frame world, frame_b or frame_resolve. <br>
           <IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Forces/ArrowForce.png\"><br>
           <IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Forces/ArrowTorque.png\">
      </td>
  </tr>
  <tr><td valign=\"top\"><a href=\"modelica://Modelica.Mechanics.MultiBody.Forces.Force\">Force</a></td>
      <td valign=\"top\"> Force acting between two frames defined by 3 input signals
           resolved in frame world, frame_a, frame_b or in frame_resolve. <br>
           <IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Forces/ArrowForce2.png\"></td>
  </tr>
  <tr><td valign=\"top\"><a href=\"modelica://Modelica.Mechanics.MultiBody.Forces.Torque\">Torque</a></td>
      <td valign=\"top\"> Torque acting between two frames defined by 3 input signals
           resolved in frame world, frame_a, frame_b or in frame_resolve. <br>
           <IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Forces/ArrowTorque2.png\"></td>
  </tr>
  <tr><td valign=\"top\"><a href=\"modelica://Modelica.Mechanics.MultiBody.Forces.ForceAndTorque\">ForceAndTorque</a></td>
      <td valign=\"top\"> Force and torque acting between two frames defined by 3+3 input signals
           resolved in frame world, frame_a, frame_b or in frame_resolve. <br>
           <IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Forces/ArrowForce2.png\"><br>
           <IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Forces/ArrowTorque2.png\"></td>
  </tr>
  <tr><td valign=\"top\"><a href=\"modelica://Modelica.Mechanics.MultiBody.Forces.LineForceWithMass\">LineForceWithMass</a></td>
      <td valign=\"top\">  General line force component with an optional point mass
            on the connection line. The force law can be defined by a
            component of Modelica.Mechanics.Translational<br>
           <IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Forces/LineForceWithMass.png\"></td>
  </tr>
  <tr><td valign=\"top\"><a href=\"modelica://Modelica.Mechanics.MultiBody.Forces.LineForceWithTwoMasses\">LineForceWithTwoMasses</a></td>
      <td valign=\"top\">  General line force component with two optional point masses
            on the connection line. The force law can be defined by a
            component of Modelica.Mechanics.Translational<br>
           <IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Forces/LineForceWithTwoMasses.png\"></td>
  </tr>
  <tr><td valign=\"top\"><a href=\"modelica://Modelica.Mechanics.MultiBody.Forces.Spring\">Spring</a></td>
      <td valign=\"top\"> Linear translational spring with optional mass <br>
           <IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Forces/Spring2.png\"></td>
  </tr>
  <tr><td valign=\"top\"><a href=\"modelica://Modelica.Mechanics.MultiBody.Forces.Damper\">Damper</a></td>
      <td valign=\"top\"> Linear (velocity dependent) damper <br>
           <IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Forces/Damper2.png\"></td>
  </tr>
  <tr><td valign=\"top\"><a href=\"modelica://Modelica.Mechanics.MultiBody.Forces.SpringDamperParallel\">SpringDamperParallel</a></td>
      <td valign=\"top\"> Linear spring and damper in parallel connection </td>
  </tr>
  <tr><td valign=\"top\"><a href=\"modelica://Modelica.Mechanics.MultiBody.Forces.SpringDamperSeries\">SpringDamperSeries</a></td>
      <td valign=\"top\"> Linear spring and damper in series connection </td>
  </tr>
</table>
</HTML>"));
      end Forces;

      package Frames "Functions to transform rotational frame quantities"
        extends Modelica.Icons.Package;

        record Orientation "Orientation object defining rotation from a frame 1 into a frame 2"
          extends Modelica.Icons.Record;
          Real T[3, 3] "Transformation matrix from world frame to local frame";
          SI.AngularVelocity w[3] "Absolute angular velocity of local frame, resolved in local frame";

          encapsulated function equalityConstraint "Return the constraint residues to express that two frames have the same orientation"
            import Modelica;
            import Modelica.Mechanics.MultiBody.Frames;
            extends Modelica.Icons.Function;
            input Frames.Orientation R1 "Orientation object to rotate frame 0 into frame 1";
            input Frames.Orientation R2 "Orientation object to rotate frame 0 into frame 2";
            output Real residue[3] "The rotation angles around x-, y-, and z-axis of frame 1 to rotate frame 1 into frame 2 for a small rotation (should be zero)";
          algorithm
            residue := {Modelica.Math.atan2(cross(R1.T[1, :], R1.T[2, :]) * R2.T[2, :], R1.T[1, :] * R2.T[1, :]), Modelica.Math.atan2(-cross(R1.T[1, :], R1.T[2, :]) * R2.T[1, :], R1.T[2, :] * R2.T[2, :]), Modelica.Math.atan2(R1.T[2, :] * R2.T[1, :], R1.T[3, :] * R2.T[3, :])};
            annotation(
              Inline = true);
          end equalityConstraint;
          annotation(
            Documentation(info = "<html>
<p>
This object describes the <b>rotation</b> from a <b>frame 1</b> into a <b>frame 2</b>.
An instance of this type should never be directly accessed but
only with the access functions provided
in package Modelica.Mechanics.MultiBody.Frames. As a consequence, it is not necessary to know
the internal representation of this object as described in the next paragraphs.
</p>
<p>
\"Orientation\" is defined to be a record consisting of two
elements: \"Real T[3,3]\", the transformation matrix to rotate frame 1
into frame 2 and \"Real w[3]\", the angular velocity of frame 2 with
respect to frame 1, resolved in frame 2. Element \"T\"
has the following interpretation:
</p>
<pre>
   Orientation R;
   <b>R.T</b> = [<b>e</b><sub>x</sub>, <b>e</b><sub>y</sub>, <b>e</b><sub>z</sub>];
       e.g., <b>R.T</b> = [1,0,0; 0,1,0; 0,0,1]
</pre>
<p>
where <b>e</b><sub>x</sub>,<b>e</b><sub>y</sub>,<b>e</b><sub>z</sub>
are unit vectors in the direction of the x-axis, y-axis, and z-axis
of frame 1, resolved in frame 2, respectively. Therefore, if <b>v</b><sub>1</sub>
is vector <b>v</b> resolved in frame 1 and <b>v</b><sub>2</sub> is
vector <b>v</b> resolved in frame 2, the following relationship holds:
</p>
<pre>
    <b>v</b><sub>2</sub> = <b>R.T</b> * <b>v</b><sub>1</sub>
</pre>
<p>
The <b>inverse</b> orientation
<b>R_inv.T</b> = <b>R.T</b><sup>T</sup> describes the rotation
from frame 2 into frame 1.
</p>
<p>
Since the orientation is described by 9 variables, there are
6 constraints between these variables. These constraints
are defined in function <b>Frames.orientationConstraint</b>.
</p>
<p>
R.w is the angular velocity of frame 2 with respect to frame 1, resolved
in frame 2. Formally, R.w is defined as:<br>
<b>skew</b>(R.w) = R.T*<b>der</b>(transpose(R.T))
with
</p>
<pre>
             |   0   -w[3]  w[2] |
   <b>skew</b>(w) = |  w[3]   0   -w[1] |
             | -w[2]  w[1]     0 |
</pre>
</html>"));
        end Orientation;

        function orientationConstraint "Return residues of orientation constraints (shall be zero)"
          extends Modelica.Icons.Function;
          input Orientation R "Orientation object to rotate frame 1 into frame 2";
          output Real residue[6] "Residues of constraints between elements of orientation object (shall be zero)";
        algorithm
          residue := {R.T[:, 1] * R.T[:, 1] - 1, R.T[:, 2] * R.T[:, 2] - 1, R.T[:, 3] * R.T[:, 3] - 1, R.T[:, 1] * R.T[:, 2], R.T[:, 1] * R.T[:, 3], R.T[:, 2] * R.T[:, 3]};
          annotation(
            Inline = true);
        end orientationConstraint;

        function angularVelocity1 "Return angular velocity resolved in frame 1 from orientation object"
          extends Modelica.Icons.Function;
          input Orientation R "Orientation object to rotate frame 1 into frame 2";
          output Modelica.SIunits.AngularVelocity w[3] "Angular velocity of frame 2 with respect to frame 1 resolved in frame 1";
        algorithm
          w := resolve1(R, R.w);
          annotation(
            Inline = true);
        end angularVelocity1;

        function angularVelocity2 "Return angular velocity resolved in frame 2 from orientation object"
          extends Modelica.Icons.Function;
          input Orientation R "Orientation object to rotate frame 1 into frame 2";
          output Modelica.SIunits.AngularVelocity w[3] "Angular velocity of frame 2 with respect to frame 1 resolved in frame 2";
        algorithm
          w := R.w;
          annotation(
            Inline = true);
        end angularVelocity2;

        function resolve1 "Transform vector from frame 2 to frame 1"
          extends Modelica.Icons.Function;
          input Orientation R "Orientation object to rotate frame 1 into frame 2";
          input Real v2[3] "Vector in frame 2";
          output Real v1[3] "Vector in frame 1";
        algorithm
          v1 := transpose(R.T) * v2;
          annotation(
            derivative(noDerivative = R) = Internal.resolve1_der,
            InlineAfterIndexReduction = true);
        end resolve1;

        function resolve2 "Transform vector from frame 1 to frame 2"
          extends Modelica.Icons.Function;
          input Orientation R "Orientation object to rotate frame 1 into frame 2";
          input Real v1[3] "Vector in frame 1";
          output Real v2[3] "Vector in frame 2";
        algorithm
          v2 := R.T * v1;
          annotation(
            derivative(noDerivative = R) = Internal.resolve2_der,
            InlineAfterIndexReduction = true);
        end resolve2;

        function resolveRelative "Transform vector from frame 1 to frame 2 using absolute orientation objects of frame 1 and of frame 2"
          extends Modelica.Icons.Function;
          input Real v1[3] "Vector in frame 1";
          input Orientation R1 "Orientation object to rotate frame 0 into frame 1";
          input Orientation R2 "Orientation object to rotate frame 0 into frame 2";
          output Real v2[3] "Vector in frame 2";
        algorithm
          v2 := resolve2(R2, resolve1(R1, v1));
          annotation(
            derivative(noDerivative = R1, noDerivative = R2) = Internal.resolveRelative_der,
            InlineAfterIndexReduction = true);
        end resolveRelative;

        function resolveDyade1 "Transform second order tensor from frame 2 to frame 1"
          extends Modelica.Icons.Function;
          input Orientation R "Orientation object to rotate frame 1 into frame 2";
          input Real D2[3, 3] "Second order tensor resolved in frame 2";
          output Real D1[3, 3] "Second order tensor resolved in frame 1";
        algorithm
          D1 := transpose(R.T) * D2 * R.T;
          annotation(
            Inline = true);
        end resolveDyade1;

        function resolveDyade2 "Transform second order tensor from frame 1 to frame 2"
          extends Modelica.Icons.Function;
          input Orientation R "Orientation object to rotate frame 1 into frame 2";
          input Real D1[3, 3] "Second order tensor resolved in frame 1";
          output Real D2[3, 3] "Second order tensor resolved in frame 2";
        algorithm
          D2 := R.T * D1 * transpose(R.T);
          annotation(
            Inline = true);
        end resolveDyade2;

        function nullRotation "Return orientation object that does not rotate a frame"
          extends Modelica.Icons.Function;
          output Orientation R "Orientation object such that frame 1 and frame 2 are identical";
        algorithm
          R := Orientation(T = identity(3), w = zeros(3));
          annotation(
            Inline = true);
        end nullRotation;

        function inverseRotation "Return inverse orientation object"
          extends Modelica.Icons.Function;
          input Orientation R "Orientation object to rotate frame 1 into frame 2";
          output Orientation R_inv "Orientation object to rotate frame 2 into frame 1";
        algorithm
          R_inv := Orientation(T = transpose(R.T), w = -resolve1(R, R.w));
          annotation(
            Inline = true);
        end inverseRotation;

        function relativeRotation "Return relative orientation object"
          extends Modelica.Icons.Function;
          input Orientation R1 "Orientation object to rotate frame 0 into frame 1";
          input Orientation R2 "Orientation object to rotate frame 0 into frame 2";
          output Orientation R_rel "Orientation object to rotate frame 1 into frame 2";
        algorithm
          R_rel := Orientation(T = R2.T * transpose(R1.T), w = R2.w - resolve2(R2, resolve1(R1, R1.w)));
          annotation(
            Inline = true);
        end relativeRotation;

        function absoluteRotation "Return absolute orientation object from another absolute and a relative orientation object"
          extends Modelica.Icons.Function;
          input Orientation R1 "Orientation object to rotate frame 0 into frame 1";
          input Orientation R_rel "Orientation object to rotate frame 1 into frame 2";
          output Orientation R2 "Orientation object to rotate frame 0 into frame 2";
        algorithm
          R2 := Orientation(T = R_rel.T * R1.T, w = resolve2(R_rel, R1.w) + R_rel.w);
          annotation(
            Inline = true);
        end absoluteRotation;

        function planarRotation "Return orientation object of a planar rotation"
          import Modelica.Math;
          extends Modelica.Icons.Function;
          input Real e[3](each final unit = "1") "Normalized axis of rotation (must have length=1)";
          input Modelica.SIunits.Angle angle "Rotation angle to rotate frame 1 into frame 2 along axis e";
          input Modelica.SIunits.AngularVelocity der_angle "= der(angle)";
          output Orientation R "Orientation object to rotate frame 1 into frame 2";
        algorithm
          R := Orientation(T = [e] * transpose([e]) + (identity(3) - [e] * transpose([e])) * Math.cos(angle) - skew(e) * Math.sin(angle), w = e * der_angle);
          annotation(
            Inline = true);
        end planarRotation;

        function planarRotationAngle "Return angle of a planar rotation, given the rotation axis and the representations of a vector in frame 1 and frame 2"
          extends Modelica.Icons.Function;
          input Real e[3](each final unit = "1") "Normalized axis of rotation to rotate frame 1 around e into frame 2 (must have length=1)";
          input Real v1[3] "A vector v resolved in frame 1 (shall not be parallel to e)";
          input Real v2[3] "Vector v resolved in frame 2, i.e., v2 = resolve2(planarRotation(e,angle),v1)";
          output Modelica.SIunits.Angle angle "Rotation angle to rotate frame 1 into frame 2 along axis e in the range: -pi <= angle <= pi";
        algorithm
/* Vector v is resolved in frame 1 and frame 2 according to:
        (1)  v2 = (e*transpose(e) + (identity(3) - e*transpose(e))*cos(angle) - skew(e)*sin(angle))*v1;
                = e*(e*v1) + (v1 - e*(e*v1))*cos(angle) - cross(e,v1)*sin(angle)
       Equation (1) is multiplied with "v1" resulting in (note: e*e = 1)
            v1*v2 = (e*v1)*(e*v2) + (v1*v1 - (e*v1)*(e*v1))*cos(angle)
       and therefore:
        (2) cos(angle) = ( v1*v2 - (e*v1)*(e*v2)) / (v1*v1 - (e*v1)*(e*v1))
       Similarly, equation (1) is multiplied with cross(e,v1), i.e., a
       a vector that is orthogonal to e and to v1:
              cross(e,v1)*v2 = - cross(e,v1)*cross(e,v1)*sin(angle)
       and therefore:
          (3) sin(angle) = -cross(e,v1)*v2/(cross(e,v1)*cross(e,v1));
       We have e*e=1; Therefore:
          (4) v1*v1 - (e*v1)*(e*v1) = |v1|^2 - (|v1|*cos(e,v1))^2
       and
          (5) cross(e,v1)*cross(e,v1) = (|v1|*sin(e,v1))^2
                                      = |v1|^2*(1 - cos(e,v1)^2)
                                      = |v1|^2 - (|v1|*cos(e,v1))^2
       The denominators of (2) and (3) are identical, according to (4) and (5).
       Furthermore, the denominators are always positive according to (5).
       Therefore, in the equation "angle = atan2(sin(angle), cos(angle))" the
       denominators of sin(angle) and cos(angle) can be removed,
       resulting in:
          angle = atan2(-cross(e,v1)*v2, v1*v2 - (e*v1)*(e*v2));
    */
          angle := Modelica.Math.atan2(-cross(e, v1) * v2, v1 * v2 - e * v1 * (e * v2));
          annotation(
            Inline = true,
            Documentation(info = "<HTML>
<p>
A call to this function of the form
</p>
<pre>
    Real[3]                e, v1, v2;
    Modelica.SIunits.Angle angle;
  <b>equation</b>
    angle = <b>planarRotationAngle</b>(e, v1, v2);
</pre>
<p>
computes the rotation angle \"<b>angle</b>\" of a planar
rotation along unit vector <b>e</b>, rotating frame 1 into frame 2, given
the coordinate representations of a vector \"v\" in frame 1 (<b>v1</b>)
and in frame 2 (<b>v2</b>). Therefore, the result of this function
fulfills the following equation:
</p>
<pre>
    v2 = <b>resolve2</b>(<b>planarRotation</b>(e,angle), v1)
</pre>
<p>
The rotation angle is returned in the range
</p>
<pre>
    -<font face=\"Symbol\">p</font> &lt;= angle &lt;= <font face=\"Symbol\">p</font>
</pre>
<p>
This function makes the following assumptions on the input arguments
</p>
<ul>
<li> Vector <b>e</b> has length 1, i.e., length(e) = 1</li>
<li> Vector \"v\" is not parallel to <b>e</b>, i.e.,
     length(cross(e,v1)) &ne; 0</li>
</ul>
<p>
The function does not check the above assumptions. If these
assumptions are violated, a wrong result will be returned
and/or a division by zero will occur.
</p>
</HTML>"));
        end planarRotationAngle;

        function axisRotation "Return rotation object to rotate around an angle along one frame axis"
          extends Modelica.Icons.Function;
          input Integer axis(min = 1, max = 3) "Rotate around 'axis' of frame 1";
          input Modelica.SIunits.Angle angle "Rotation angle to rotate frame 1 into frame 2 along 'axis' of frame 1";
          input Modelica.SIunits.AngularVelocity der_angle "= der(angle)";
          output Orientation R "Orientation object to rotate frame 1 into frame 2";
        algorithm
          R := Orientation(T = if axis == 1 then [1, 0, 0; 0, cos(angle), sin(angle); 0, -sin(angle), cos(angle)] else if axis == 2 then [cos(angle), 0, -sin(angle); 0, 1, 0; sin(angle), 0, cos(angle)] else [cos(angle), sin(angle), 0; -sin(angle), cos(angle), 0; 0, 0, 1], w = if axis == 1 then {der_angle, 0, 0} else if axis == 2 then {0, der_angle, 0} else {0, 0, der_angle});
          annotation(
            Inline = true);
        end axisRotation;

        function axesRotations "Return fixed rotation object to rotate in sequence around fixed angles along 3 axes"
          import TM = Modelica.Mechanics.MultiBody.Frames.TransformationMatrices;
          extends Modelica.Icons.Function;
          input Integer sequence[3](min = {1, 1, 1}, max = {3, 3, 3}) = {1, 2, 3} "Sequence of rotations from frame 1 to frame 2 along axis sequence[i]";
          input Modelica.SIunits.Angle angles[3] "Rotation angles around the axes defined in 'sequence'";
          input Modelica.SIunits.AngularVelocity der_angles[3] "= der(angles)";
          output Orientation R "Orientation object to rotate frame 1 into frame 2";
        algorithm
/*
  R := absoluteRotation(absoluteRotation(axisRotation(sequence[1], angles[1],
    der_angles[1]), axisRotation(sequence[2], angles[2], der_angles[2])),
    axisRotation(sequence[3], angles[3], der_angles[3]));
*/
          R := Orientation(T = TM.axisRotation(sequence[3], angles[3]) * TM.axisRotation(sequence[2], angles[2]) * TM.axisRotation(sequence[1], angles[1]), w = Frames.axis(sequence[3]) * der_angles[3] + TM.resolve2(TM.axisRotation(sequence[3], angles[3]), Frames.axis(sequence[2]) * der_angles[2]) + TM.resolve2(TM.axisRotation(sequence[3], angles[3]) * TM.axisRotation(sequence[2], angles[2]), Frames.axis(sequence[1]) * der_angles[1]));
          annotation(
            Inline = true);
        end axesRotations;

        function axesRotationsAngles "Return the 3 angles to rotate in sequence around 3 axes to construct the given orientation object"
          extends Modelica.Icons.Function;
          input Orientation R "Orientation object to rotate frame 1 into frame 2";
          input Integer sequence[3](min = {1, 1, 1}, max = {3, 3, 3}) = {1, 2, 3} "Sequence of rotations from frame 1 to frame 2 along axis sequence[i]";
          input SI.Angle guessAngle1 = 0 "Select angles[1] such that |angles[1] - guessAngle1| is a minimum";
          output SI.Angle angles[3] "Rotation angles around the axes defined in 'sequence' such that R=Frames.axesRotation(sequence,angles); -pi < angles[i] <= pi";
        protected
          Real e1_1[3](each final unit = "1") "First rotation axis, resolved in frame 1";
          Real e2_1a[3](each final unit = "1") "Second rotation axis, resolved in frame 1a";
          Real e3_1[3](each final unit = "1") "Third rotation axis, resolved in frame 1";
          Real e3_2[3](each final unit = "1") "Third rotation axis, resolved in frame 2";
          Real A "Coefficient A in the equation A*cos(angles[1])+B*sin(angles[1]) = 0";
          Real B "Coefficient B in the equation A*cos(angles[1])+B*sin(angles[1]) = 0";
          SI.Angle angle_1a "Solution 1 for angles[1]";
          SI.Angle angle_1b "Solution 2 for angles[1]";
          TransformationMatrices.Orientation T_1a "Orientation object to rotate frame 1 into frame 1a";
        algorithm
/* The rotation object R is constructed by:
     (1) Rotating frame 1 along axis e1 (= axis sequence[1]) with angles[1]
         arriving at frame 1a.
     (2) Rotating frame 1a along axis e2 (= axis sequence[2]) with angles[2]
         arriving at frame 1b.
     (3) Rotating frame 1b along axis e3 (= axis sequence[3]) with angles[3]
         arriving at frame 2.
     The goal is to determine angles[1:3]. This is performed in the following way:
     1. e2 and e3 are perpendicular to each other, i.e., e2*e3 = 0;
        Both vectors are resolved in frame 1 (T_ij is transformation matrix
        from frame j to frame i; e1_1*e2_1a = 0, since the vectors are
        perpendicular to each other):
           e3_1 = T_12*e3_2
                = R[sequence[3],:];
           e2_1 = T_11a*e2_1a
                = ( e1_1*transpose(e1_1) + (identity(3) - e1_1*transpose(e1_1))*cos(angles[1])
                    + skew(e1_1)*sin(angles[1]) )*e2_1a
                = e2_1a*cos(angles[1]) + cross(e1_1, e2_1a)*sin(angles[1]);
        From this follows finally an equation for angles[1]
           e2_1*e3_1 = 0
                     = (e2_1a*cos(angles[1]) + cross(e1_1, e2_1a)*sin(angles[1]))*e3_1
                     = (e2_1a*e3_1)*cos(angles[1]) + cross(e1_1, e2_1a)*e3_1*sin(angles[1])
                     = A*cos(angles[1]) + B*sin(angles[1])
                       with A = e2_1a*e3_1, B = cross(e1_1, e2_1a)*e3_1
        This equation has two solutions in the range -pi < angles[1] <= pi:
           sin(angles[1]) =  k*A/sqrt(A*A + B*B)
           cos(angles[1]) = -k*B/sqrt(A*A + B*B)
                        k = +/-1
           tan(angles[1]) = k*A/(-k*B)
        that is:
           angles[1] = atan2(k*A, -k*B)
        If A and B are both zero at the same time, there is a singular configuration
        resulting in an infinite number of solutions for angles[1] (every value
        is possible).
     2. angles[2] is determined with function Frames.planarRotationAngle.
        This function requires to provide e_3 in frame 1a and in frame 1b:
          e3_1a = Frames.resolve2(planarRotation(e1_1,angles[1]), e3_1);
          e3_1b = e3_2
     3. angles[3] is determined with function Frames.planarRotationAngle.
        This function requires to provide e_2 in frame 1b and in frame 2:
          e2_1b = e2_1a
          e2_2  = Frames.resolve2( R, Frames.resolve1(planarRotation(e1_1,angles[1]), e2_1a));
  */
          assert(sequence[1] <> sequence[2] and sequence[2] <> sequence[3], "input argument 'sequence[1:3]' is not valid");
          e1_1 := if sequence[1] == 1 then {1, 0, 0} else if sequence[1] == 2 then {0, 1, 0} else {0, 0, 1};
          e2_1a := if sequence[2] == 1 then {1, 0, 0} else if sequence[2] == 2 then {0, 1, 0} else {0, 0, 1};
          e3_1 := R.T[sequence[3], :];
          e3_2 := if sequence[3] == 1 then {1, 0, 0} else if sequence[3] == 2 then {0, 1, 0} else {0, 0, 1};
          A := e2_1a * e3_1;
          B := cross(e1_1, e2_1a) * e3_1;
          if abs(A) <= 1.e-12 and abs(B) <= 1.e-12 then
            angles[1] := guessAngle1;
          else
            angle_1a := Modelica.Math.atan2(A, -B);
            angle_1b := Modelica.Math.atan2(-A, B);
            angles[1] := if abs(angle_1a - guessAngle1) <= abs(angle_1b - guessAngle1) then angle_1a else angle_1b;
          end if;
          T_1a := TransformationMatrices.planarRotation(e1_1, angles[1]);
          angles[2] := planarRotationAngle(e2_1a, TransformationMatrices.resolve2(T_1a, e3_1), e3_2);
          angles[3] := planarRotationAngle(e3_2, e2_1a, TransformationMatrices.resolve2(R.T, TransformationMatrices.resolve1(T_1a, e2_1a)));
          annotation(
            Documentation(info = "<HTML>
<p>
A call to this function of the form
</p>
<pre>
    Frames.Orientation     R;
    <b>parameter</b> Integer      sequence[3] = {1,2,3};
    Modelica.SIunits.Angle angles[3];
  <b>equation</b>
    angle = <b>axesRotationAngles</b>(R, sequence);
</pre>
<p>
computes the rotation angles \"<b>angles</b>[1:3]\" to rotate frame 1
into frame 2 along axes <b>sequence</b>[1:3], given the orientation
object <b>R</b> from frame 1 to frame 2. Therefore, the result of
this function fulfills the following equation:
</p>
<pre>
    R = <b>axesRotation</b>(sequence, angles)
</pre>
<p>
The rotation angles are returned in the range
</p>
<pre>
    -<font face=\"Symbol\">p</font> &lt;= angles[i] &lt;= <font face=\"Symbol\">p</font>
</pre>
<p>
There are <b>two solutions</b> for \"angles[1]\" in this range.
Via the third argument <b>guessAngle1</b> (default = 0) the
returned solution is selected such that |angles[1] - guessAngle1| is
minimal. The orientation object R may be in a singular configuration, i.e.,
there is an infinite number of angle values leading to the same R. The returned solution is
selected by setting angles[1] = guessAngle1. Then angles[2]
and angles[3] can be uniquely determined in the above range.
</p>
<p>
Note, that input argument <b>sequence</b> has the restriction that
only values 1,2,3 can be used and that sequence[1] &ne; sequence[2]
and sequence[2] &ne; sequence[3]. Often used values are:
</p>
<pre>
  sequence = <b>{1,2,3}</b>  // Cardan angle sequence
           = <b>{3,1,3}</b>  // Euler angle sequence
           = <b>{3,2,1}</b>  // Tait-Bryan angle sequence
</pre>
</HTML>"));
        end axesRotationsAngles;

        function smallRotation "Return rotation angles valid for a small rotation and optionally residues that should be zero"
          extends Modelica.Icons.Function;
          input Orientation R "Orientation object to rotate frame 1 into frame 2";
          input Boolean withResidues = false "= false/true, if 'angles'/'angles and residues' are returned in phi";
          output Modelica.SIunits.Angle phi[if withResidues then 6 else 3] "The rotation angles around x-, y-, and z-axis of frame 1 to rotate frame 1 into frame 2 for a small rotation + optionally 3 residues that should be zero";
        algorithm
/* Planar rotation:
       Trel = [e]*transpose([e]) + (identity(3) - [e]*transpose([e]))*cos(angle) - skew(e)*sin(angle)
            = identity(3) - skew(e)*angle, for small angles
            = identity(3) - skew(e*angle)
               define phi = e*angle, then
       Trel = [1,      phi3,   -phi2;
               -phi3,     1,    phi1;
                phi2, -phi1,       1 ];
  */
          phi := if withResidues then {R.T[2, 3], -R.T[1, 3], R.T[1, 2], R.T[1, 1] - 1, R.T[2, 2] - 1, R.T[1, 1] * R.T[2, 2] - R.T[2, 1] * R.T[1, 2] - 1} else {R.T[2, 3], -R.T[1, 3], R.T[1, 2]};
          annotation(
            Inline = true);
        end smallRotation;

        function from_nxy "Return fixed orientation object from n_x and n_y vectors"
          extends Modelica.Icons.Function;
          input Real n_x[3](each final unit = "1") "Vector in direction of x-axis of frame 2, resolved in frame 1";
          input Real n_y[3](each final unit = "1") "Vector in direction of y-axis of frame 2, resolved in frame 1";
          output Orientation R "Orientation object to rotate frame 1 into frame 2";
        protected
          Real abs_n_x = sqrt(n_x * n_x);
          Real e_x[3](each final unit = "1") = if abs_n_x < 1.e-10 then {1, 0, 0} else n_x / abs_n_x;
          Real n_z_aux[3](each final unit = "1") = cross(e_x, n_y);
          Real n_y_aux[3](each final unit = "1") = if n_z_aux * n_z_aux > 1.0e-6 then n_y else if abs(e_x[1]) > 1.0e-6 then {0, 1, 0} else {1, 0, 0};
          Real e_z_aux[3](each final unit = "1") = cross(e_x, n_y_aux);
          Real e_z[3](each final unit = "1") = e_z_aux / sqrt(e_z_aux * e_z_aux);
        algorithm
          R := Orientation(T = {e_x, cross(e_z, e_x), e_z}, w = zeros(3));
          annotation(
            Documentation(info = "<html>
<p>
It is assumed that the two input vectors n_x and n_y are
resolved in frame 1 and are directed along the x and y axis
of frame 2 (i.e., n_x and n_y are orthogonal to each other)
The function returns the orientation object R to rotate from
frame 1 to frame 2.
</p>
<p>
The function is robust in the sense that it returns always
an orientation object R, even if n_y is not orthogonal to n_x.
This is performed in the following way:
</p>
<p>
If n_x and n_y are not orthogonal to each other, first a unit
vector e_y is determined that is orthogonal to n_x and is lying
in the plane spanned by n_x and n_y. If n_x and n_y are parallel
or nearly parallel to each other, a vector e_y is selected
arbitrarily such that e_x and e_y are orthogonal to each other.
</p>
</html>"));
        end from_nxy;

        function from_nxz "Return fixed orientation object from n_x and n_z vectors"
          extends Modelica.Icons.Function;
          input Real n_x[3](each final unit = "1") "Vector in direction of x-axis of frame 2, resolved in frame 1";
          input Real n_z[3](each final unit = "1") "Vector in direction of z-axis of frame 2, resolved in frame 1";
          output Orientation R "Orientation object to rotate frame 1 into frame 2";
        protected
          Real abs_n_x = sqrt(n_x * n_x);
          Real e_x[3](each final unit = "1") = if abs_n_x < 1.e-10 then {1, 0, 0} else n_x / abs_n_x;
          Real n_y_aux[3](each final unit = "1") = cross(n_z, e_x);
          Real n_z_aux[3](each final unit = "1") = if n_y_aux * n_y_aux > 1.0e-6 then n_z else if abs(e_x[1]) > 1.0e-6 then {0, 0, 1} else {1, 0, 0};
          Real e_y_aux[3](each final unit = "1") = cross(n_z_aux, e_x);
          Real e_y[3](each final unit = "1") = e_y_aux / sqrt(e_y_aux * e_y_aux);
        algorithm
          R := Orientation(T = {e_x, e_y, cross(e_x, e_y)}, w = zeros(3));
          annotation(
            Documentation(info = "<html>
<p>
It is assumed that the two input vectors n_x and n_z are
resolved in frame 1 and are directed along the x and z axis
of frame 2 (i.e., n_x and n_z are orthogonal to each other)
The function returns the orientation object R to rotate from
frame 1 to frame 2.
</p>
<p>
The function is robust in the sense that it returns always
an orientation object R, even if n_z is not orthogonal to n_x.
This is performed in the following way:
</p>
<p>
If n_x and n_z are not orthogonal to each other, first a unit
vector e_z is determined that is orthogonal to n_x and is lying
in the plane spanned by n_x and n_z. If n_x and n_z are parallel
or nearly parallel to each other, a vector e_z is selected
arbitrarily such that n_x and e_z are orthogonal to each other.
</p>
</html>"));
        end from_nxz;

        function from_T "Return orientation object R from transformation matrix T"
          extends Modelica.Icons.Function;
          input Real T[3, 3] "Transformation matrix to transform vector from frame 1 to frame 2 (v2=T*v1)";
          input Modelica.SIunits.AngularVelocity w[3] "Angular velocity from frame 2 with respect to frame 1, resolved in frame 2 (skew(w)=T*der(transpose(T)))";
          output Orientation R "Orientation object to rotate frame 1 into frame 2";
        algorithm
          R := Orientation(T = T, w = w);
          annotation(
            Inline = true);
        end from_T;

        function from_T2 "Return orientation object R from transformation matrix T and its derivative der(T)"
          extends Modelica.Icons.Function;
          input Real T[3, 3] "Transformation matrix to transform vector from frame 1 to frame 2 (v2=T*v1)";
          input Real der_T[3, 3] "= der(T)";
          output Orientation R "Orientation object to rotate frame 1 into frame 2";
        algorithm
          R := Orientation(T = T, w = {T[3, :] * der_T[2, :], -T[3, :] * der_T[1, :], T[2, :] * der_T[1, :]});
          annotation(
            Inline = true,
            Documentation(info = "<html>
<p>
Computes the orientation object from a transformation matrix T and
the derivative der(T) of the transformation matrix.
Usually, it is more efficient to use function \"from_T\" instead, where
the angular velocity has to be given as input argument. Only if this
is not possible or too difficult to compute, use function from_T2(..).
</p>
</html>"));
        end from_T2;

        function from_T_inv "Return orientation object R from inverse transformation matrix T_inv"
          extends Modelica.Icons.Function;
          input Real T_inv[3, 3] "Inverse transformation matrix to transform vector from frame 2 to frame 1 (v1=T_inv*v2)";
          input Modelica.SIunits.AngularVelocity w[3] "Angular velocity from frame 1 with respect to frame 2, resolved in frame 1 (skew(w)=T_inv*der(transpose(T_inv)))";
          output Orientation R "Orientation object to rotate frame 1 into frame 2";
        algorithm
          R := Orientation(T = transpose(T_inv), w = -w);
          annotation(
            Inline = true);
        end from_T_inv;

        function from_Q "Return orientation object R from quaternion orientation object Q"
          extends Modelica.Icons.Function;
          input Quaternions.Orientation Q "Quaternions orientation object to rotate frame 1 into frame 2";
          input Modelica.SIunits.AngularVelocity w[3] "Angular velocity from frame 2 with respect to frame 1, resolved in frame 2";
          output Orientation R "Orientation object to rotate frame 1 into frame 2";
        algorithm
/*
  T := (2*Q[4]*Q[4] - 1)*identity(3) + 2*([Q[1:3]]*transpose([Q[1:3]]) - Q[4]*
    skew(Q[1:3]));
*/
          R := Orientation([2 * (Q[1] * Q[1] + Q[4] * Q[4]) - 1, 2 * (Q[1] * Q[2] + Q[3] * Q[4]), 2 * (Q[1] * Q[3] - Q[2] * Q[4]); 2 * (Q[2] * Q[1] - Q[3] * Q[4]), 2 * (Q[2] * Q[2] + Q[4] * Q[4]) - 1, 2 * (Q[2] * Q[3] + Q[1] * Q[4]); 2 * (Q[3] * Q[1] + Q[2] * Q[4]), 2 * (Q[3] * Q[2] - Q[1] * Q[4]), 2 * (Q[3] * Q[3] + Q[4] * Q[4]) - 1], w = w);
          annotation(
            Inline = true);
        end from_Q;

        function to_T "Return transformation matrix T from orientation object R"
          extends Modelica.Icons.Function;
          input Orientation R "Orientation object to rotate frame 1 into frame 2";
          output Real T[3, 3] "Transformation matrix to transform vector from frame 1 to frame 2 (v2=T*v1)";
        algorithm
          T := R.T;
          annotation(
            Inline = true);
        end to_T;

        function to_T_inv "Return inverse transformation matrix T_inv from orientation object R"
          extends Modelica.Icons.Function;
          input Orientation R "Orientation object to rotate frame 1 into frame 2";
          output Real T_inv[3, 3] "Inverse transformation matrix to transform vector from frame 2 into frame 1 (v1=T_inv*v2)";
        algorithm
          T_inv := transpose(R.T);
          annotation(
            Inline = true);
        end to_T_inv;

        function to_Q "Return quaternion orientation object Q from orientation object R"
          extends Modelica.Icons.Function;
          input Orientation R "Orientation object to rotate frame 1 into frame 2";
          input Quaternions.Orientation Q_guess = Quaternions.nullRotation() "Guess value for output Q (there are 2 solutions; the one closer to Q_guess is used";
          output Quaternions.Orientation Q "Quaternions orientation object to rotate frame 1 into frame 2";
        algorithm
          Q := Quaternions.from_T(R.T, Q_guess);
          annotation(
            Inline = true);
        end to_Q;

        function to_vector "Map rotation object into vector"
          extends Modelica.Icons.Function;
          input Orientation R "Orientation object to rotate frame 1 into frame 2";
          output Real vec[9] "Elements of R in one vector";
        algorithm
          vec := {R.T[1, 1], R.T[2, 1], R.T[3, 1], R.T[1, 2], R.T[2, 2], R.T[3, 2], R.T[1, 3], R.T[2, 3], R.T[3, 3]};
          annotation(
            Inline = true);
        end to_vector;

        function to_exy "Map rotation object into e_x and e_y vectors of frame 2, resolved in frame 1"
          extends Modelica.Icons.Function;
          input Orientation R "Orientation object to rotate frame 1 into frame 2";
          output Real exy[3, 2] "= [e_x, e_y] where e_x and e_y are axes unit vectors of frame 2, resolved in frame 1";
        algorithm
          exy := [R.T[1, :], R.T[2, :]];
          annotation(
            Inline = true);
        end to_exy;

        function axis "Return unit vector for x-, y-, or z-axis"
          extends Modelica.Icons.Function;
          input Integer axis(min = 1, max = 3) "Axis vector to be returned";
          output Real e[3](each final unit = "1") "Unit axis vector";
        algorithm
          e := if axis == 1 then {1, 0, 0} else if axis == 2 then {0, 1, 0} else {0, 0, 1};
          annotation(
            Inline = true);
        end axis;

        package Quaternions "Functions to transform rotational frame quantities based on quaternions (also called Euler parameters)"
          extends Modelica.Icons.Package;

          type Orientation "Orientation type defining rotation from a frame 1 into a frame 2 with quaternions {p1,p2,p3,p0}"
            extends Internal.QuaternionBase;

            encapsulated function equalityConstraint "Return the constraint residues to express that two frames have the same quaternion orientation"
              import Modelica;
              import Modelica.Mechanics.MultiBody.Frames.Quaternions;
              extends Modelica.Icons.Function;
              input Quaternions.Orientation Q1 "Quaternions orientation object to rotate frame 0 into frame 1";
              input Quaternions.Orientation Q2 "Quaternions orientation object to rotate frame 0 into frame 2";
              output Real residue[3] "The half of the rotation angles around x-, y-, and z-axis of frame 1 to rotate frame 1 into frame 2 for a small rotation (shall be zero)";
            algorithm
              residue := [Q1[4], Q1[3], -Q1[2], -Q1[1]; -Q1[3], Q1[4], Q1[1], -Q1[2]; Q1[2], -Q1[1], Q1[4], -Q1[3]] * Q2;
              annotation(
                Inline = true);
            end equalityConstraint;
            annotation(
              Documentation(info = "<html>
<p>
This type describes the <b>rotation</b> to rotate a frame 1 into
a frame 2 using quaternions (also called <b>Euler parameters</b>)
according to the following definition:
</p>
<pre>
   Quaternions.Orientation Q;
   Real  n[3];
   Real  phi(unit=\"rad\");
   Q = [ n*sin(phi/2)
           cos(phi/2) ]
</pre>
<p>
where \"n\" is the <b>axis of rotation</b> to rotate frame 1 into
frame 2 and \"phi\" is the <b>rotation angle</b> for this rotation.
Vector \"n\" is either resolved in frame 1 or in frame 2
(the result is the same since the coordinates of \"n\" with respect to
frame 1 are identical to its coordinates with respect to frame 2).
</p>
<p>
The term \"quaternions\" is preferred over the historically
more reasonable \"Euler parameters\" in order to not get
confused with Modelica \"parameters\".
</p>
</html>"));
          end Orientation;

          type der_Orientation = Real[4](each unit = "1/s") "First time derivative of Quaternions.Orientation";

          function orientationConstraint "Return residues of orientation constraints (shall be zero)"
            extends Modelica.Icons.Function;
            input Quaternions.Orientation Q "Quaternions orientation object to rotate frame 1 into frame 2";
            output Real residue[1] "Residue constraint (shall be zero)";
          algorithm
            residue := {Q * Q - 1};
            annotation(
              Inline = true);
          end orientationConstraint;

          function angularVelocity1 "Compute angular velocity resolved in frame 1 from quaternion orientation object and its derivative"
            extends Modelica.Icons.Function;
            input Quaternions.Orientation Q "Quaternions orientation object to rotate frame 1 into frame 2";
            input der_Orientation der_Q "Derivative of Q";
            output Modelica.SIunits.AngularVelocity w[3] "Angular velocity resolved in frame 1";
          algorithm
            w := 2 * ([Q[4], -Q[3], Q[2], -Q[1]; Q[3], Q[4], -Q[1], -Q[2]; -Q[2], Q[1], Q[4], -Q[3]] * der_Q);
            annotation(
              Inline = true);
          end angularVelocity1;

          function angularVelocity2 "Compute angular velocity resolved in frame 2 from quaternions orientation object and its derivative"
            extends Modelica.Icons.Function;
            input Quaternions.Orientation Q "Quaternions orientation object to rotate frame 1 into frame 2";
            input der_Orientation der_Q "Derivative of Q";
            output Modelica.SIunits.AngularVelocity w[3] "Angular velocity of frame 2 with respect to frame 1 resolved in frame 2";
          algorithm
            w := 2 * ([Q[4], Q[3], -Q[2], -Q[1]; -Q[3], Q[4], Q[1], -Q[2]; Q[2], -Q[1], Q[4], -Q[3]] * der_Q);
            annotation(
              Inline = true);
          end angularVelocity2;

          function resolve1 "Transform vector from frame 2 to frame 1"
            extends Modelica.Icons.Function;
            input Quaternions.Orientation Q "Quaternions orientation object to rotate frame 1 into frame 2";
            input Real v2[3] "Vector in frame 2";
            output Real v1[3] "Vector in frame 1";
          algorithm
            v1 := 2 * ((Q[4] * Q[4] - 0.5) * v2 + Q[1:3] * v2 * Q[1:3] + Q[4] * cross(Q[1:3], v2));
            annotation(
              Inline = true);
          end resolve1;

          function resolve2 "Transform vector from frame 1 to frame 2"
            extends Modelica.Icons.Function;
            input Quaternions.Orientation Q "Quaternions orientation object to rotate frame 1 into frame 2";
            input Real v1[3] "Vector in frame 1";
            output Real v2[3] "Vector in frame 2";
          algorithm
            v2 := 2 * ((Q[4] * Q[4] - 0.5) * v1 + Q[1:3] * v1 * Q[1:3] - Q[4] * cross(Q[1:3], v1));
            annotation(
              Inline = true);
          end resolve2;

          function multipleResolve1 "Transform several vectors from frame 2 to frame 1"
            extends Modelica.Icons.Function;
            input Quaternions.Orientation Q "Quaternions orientation object to rotate frame 1 into frame 2";
            input Real v2[3, :] "Vectors in frame 2";
            output Real v1[3, size(v2, 2)] "Vectors in frame 1";
          algorithm
            v1 := ((2 * Q[4] * Q[4] - 1) * identity(3) + 2 * ([Q[1:3]] * transpose([Q[1:3]]) + Q[4] * skew(Q[1:3]))) * v2;
            annotation(
              Inline = true);
          end multipleResolve1;

          function multipleResolve2 "Transform several vectors from frame 1 to frame 2"
            extends Modelica.Icons.Function;
            input Quaternions.Orientation Q "Quaternions orientation object to rotate frame 1 into frame 2";
            input Real v1[3, :] "Vectors in frame 1";
            output Real v2[3, size(v1, 2)] "Vectors in frame 2";
          algorithm
            v2 := ((2 * Q[4] * Q[4] - 1) * identity(3) + 2 * ([Q[1:3]] * transpose([Q[1:3]]) - Q[4] * skew(Q[1:3]))) * v1;
            annotation(
              Inline = true);
          end multipleResolve2;

          function nullRotation "Return quaternion orientation object that does not rotate a frame"
            extends Modelica.Icons.Function;
            output Quaternions.Orientation Q "Quaternions orientation object to rotate frame 1 into frame 2";
          algorithm
            Q := {0, 0, 0, 1};
            annotation(
              Inline = true);
          end nullRotation;

          function inverseRotation "Return inverse quaternions orientation object"
            extends Modelica.Icons.Function;
            input Quaternions.Orientation Q "Quaternions orientation object to rotate frame 1 into frame 2";
            output Quaternions.Orientation Q_inv "Quaternions orientation object to rotate frame 2 into frame 1";
          algorithm
            Q_inv := {-Q[1], -Q[2], -Q[3], Q[4]};
            annotation(
              Inline = true);
          end inverseRotation;

          function relativeRotation "Return relative quaternions orientation object"
            extends Modelica.Icons.Function;
            input Quaternions.Orientation Q1 "Quaternions orientation object to rotate frame 0 into frame 1";
            input Quaternions.Orientation Q2 "Quaternions orientation object to rotate frame 0 into frame 2";
            output Quaternions.Orientation Q_rel "Quaternions orientation object to rotate frame 1 into frame 2";
          algorithm
            Q_rel := [Q1[4], Q1[3], -Q1[2], -Q1[1]; -Q1[3], Q1[4], Q1[1], -Q1[2]; Q1[2], -Q1[1], Q1[4], -Q1[3]; Q1[1], Q1[2], Q1[3], Q1[4]] * Q2;
            annotation(
              Inline = true);
          end relativeRotation;

          function absoluteRotation "Return absolute quaternions orientation object from another absolute and a relative quaternions orientation object"
            extends Modelica.Icons.Function;
            input Quaternions.Orientation Q1 "Quaternions orientation object to rotate frame 0 into frame 1";
            input Quaternions.Orientation Q_rel "Quaternions orientation object to rotate frame 1 into frame 2";
            output Quaternions.Orientation Q2 "Quaternions orientation object to rotate frame 0 into frame 2";
          algorithm
            Q2 := [Q_rel[4], Q_rel[3], -Q_rel[2], Q_rel[1]; -Q_rel[3], Q_rel[4], Q_rel[1], Q_rel[2]; Q_rel[2], -Q_rel[1], Q_rel[4], Q_rel[3]; -Q_rel[1], -Q_rel[2], -Q_rel[3], Q_rel[4]] * Q1;
            annotation(
              Inline = true);
          end absoluteRotation;

          function planarRotation "Return quaternion orientation object of a planar rotation"
            import Modelica.Math;
            extends Modelica.Icons.Function;
            input Real e[3](each final unit = "1") "Normalized axis of rotation (must have length=1)";
            input Modelica.SIunits.Angle angle "Rotation angle to rotate frame 1 into frame 2 along axis e";
            output Quaternions.Orientation Q "Quaternions orientation object to rotate frame 1 into frame 2 along axis e";
          algorithm
            Q := vector([e * Math.sin(angle / 2); Math.cos(angle / 2)]);
            annotation(
              Inline = true);
          end planarRotation;

          function smallRotation "Return rotation angles valid for a small rotation"
            extends Modelica.Icons.Function;
            input Quaternions.Orientation Q "Quaternions orientation object to rotate frame 1 into frame 2";
            output Modelica.SIunits.Angle phi[3] "The rotation angles around x-, y-, and z-axis of frame 1 to rotate frame 1 into frame 2 for a small relative rotation";
          algorithm
            phi := 2 * {Q[1], Q[2], Q[3]};
            annotation(
              Inline = true);
          end smallRotation;

          function from_T "Return quaternion orientation object Q from transformation matrix T"
            extends Modelica.Icons.Function;
            input Real T[3, 3] "Transformation matrix to transform vector from frame 1 to frame 2 (v2=T*v1)";
            input Quaternions.Orientation Q_guess = nullRotation() "Guess value for Q (there are 2 solutions; the one close to Q_guess is used";
            output Quaternions.Orientation Q "Quaternions orientation object to rotate frame 1 into frame 2 (Q and -Q have same transformation matrix)";
          protected
            Real paux;
            Real paux4;
            Real c1;
            Real c2;
            Real c3;
            Real c4;
            constant Real p4limit = 0.1;
            constant Real c4limit = 4 * p4limit * p4limit;
          algorithm
/*
   Note, for quaternions, Q and -Q have the same transformation matrix.
   Calculation of quaternions from transformation matrix T:
   It is guaranteed that c1>=0, c2>=0, c3>=0, c4>=0 and
   that not all of them can be zero at the same time
   (e.g., if 3 of them are zero, the 4th variable is 1).
   Since the sqrt(..) has to be performed on one of these variables,
   it is applied on a variable which is far enough from zero.
   This guarantees that the sqrt(..) is never taken near zero
   and therefore the derivative of sqrt(..) can never be infinity.
   There is an ambiguity for quaternions, since Q and -Q
   lead to the same transformation matrix. The ambiguity
   is resolved here by selecting the Q that is closer to
   the input argument Q_guess.
*/
            c1 := 1 + T[1, 1] - T[2, 2] - T[3, 3];
            c2 := 1 + T[2, 2] - T[1, 1] - T[3, 3];
            c3 := 1 + T[3, 3] - T[1, 1] - T[2, 2];
            c4 := 1 + T[1, 1] + T[2, 2] + T[3, 3];
            if c4 > c4limit or c4 > c1 and c4 > c2 and c4 > c3 then
              paux := sqrt(c4) / 2;
              paux4 := 4 * paux;
              Q := {(T[2, 3] - T[3, 2]) / paux4, (T[3, 1] - T[1, 3]) / paux4, (T[1, 2] - T[2, 1]) / paux4, paux};
            elseif c1 > c2 and c1 > c3 and c1 > c4 then
              paux := sqrt(c1) / 2;
              paux4 := 4 * paux;
              Q := {paux, (T[1, 2] + T[2, 1]) / paux4, (T[1, 3] + T[3, 1]) / paux4, (T[2, 3] - T[3, 2]) / paux4};
            elseif c2 > c1 and c2 > c3 and c2 > c4 then
              paux := sqrt(c2) / 2;
              paux4 := 4 * paux;
              Q := {(T[1, 2] + T[2, 1]) / paux4, paux, (T[2, 3] + T[3, 2]) / paux4, (T[3, 1] - T[1, 3]) / paux4};
            else
              paux := sqrt(c3) / 2;
              paux4 := 4 * paux;
              Q := {(T[1, 3] + T[3, 1]) / paux4, (T[2, 3] + T[3, 2]) / paux4, paux, (T[1, 2] - T[2, 1]) / paux4};
            end if;
            if Q * Q_guess < 0 then
              Q := -Q;
            end if;
          end from_T;

          function from_T_inv "Return quaternion orientation object Q from inverse transformation matrix T_inv"
            extends Modelica.Icons.Function;
            input Real T_inv[3, 3] "Inverse transformation matrix to transform vector from frame 2 to frame 1 (v1=T_inv*v2)";
            input Quaternions.Orientation Q_guess = nullRotation() "Guess value for output Q (there are 2 solutions; the one closer to Q_guess is used";
            output Quaternions.Orientation Q "Quaternions orientation object to rotate frame 1 into frame 2 (Q and -Q have same transformation matrix)";
          algorithm
            Q := from_T(transpose(T_inv), Q_guess);
            annotation(
              Inline = true);
          end from_T_inv;

          function to_T "Return transformation matrix T from quaternion orientation object Q"
            extends Modelica.Icons.Function;
            input Quaternions.Orientation Q "Quaternions orientation object to rotate frame 1 into frame 2";
            output Real T[3, 3] "Transformation matrix to transform vector from frame 1 to frame 2 (v2=T*v1)";
          algorithm
/*
  T := (2*Q[4]*Q[4] - 1)*identity(3) + 2*([Q[1:3]]*transpose([Q[1:3]]) - Q[4]*
    skew(Q[1:3]));
*/
            T := [2 * (Q[1] * Q[1] + Q[4] * Q[4]) - 1, 2 * (Q[1] * Q[2] + Q[3] * Q[4]), 2 * (Q[1] * Q[3] - Q[2] * Q[4]); 2 * (Q[2] * Q[1] - Q[3] * Q[4]), 2 * (Q[2] * Q[2] + Q[4] * Q[4]) - 1, 2 * (Q[2] * Q[3] + Q[1] * Q[4]); 2 * (Q[3] * Q[1] + Q[2] * Q[4]), 2 * (Q[3] * Q[2] - Q[1] * Q[4]), 2 * (Q[3] * Q[3] + Q[4] * Q[4]) - 1];
            annotation(
              Inline = true);
          end to_T;

          function to_T_inv "Return inverse transformation matrix T_inv from quaternion orientation object Q"
            extends Modelica.Icons.Function;
            input Quaternions.Orientation Q "Quaternions orientation object to rotate frame 1 into frame 2";
            output Real T_inv[3, 3] "Transformation matrix to transform vector from frame 2 to frame 1 (v1=T*v2)";
          algorithm
/*
  T_inv := (2*Q[4]*Q[4] - 1)*identity(3) + 2*([Q[1:3]]*transpose([Q[1:3]]) + Q[
    4]*skew(Q[1:3]));
*/
            T_inv := [2 * (Q[1] * Q[1] + Q[4] * Q[4]) - 1, 2 * (Q[2] * Q[1] - Q[3] * Q[4]), 2 * (Q[3] * Q[1] + Q[2] * Q[4]); 2 * (Q[1] * Q[2] + Q[3] * Q[4]), 2 * (Q[2] * Q[2] + Q[4] * Q[4]) - 1, 2 * (Q[3] * Q[2] - Q[1] * Q[4]); 2 * (Q[1] * Q[3] - Q[2] * Q[4]), 2 * (Q[2] * Q[3] + Q[1] * Q[4]), 2 * (Q[3] * Q[3] + Q[4] * Q[4]) - 1];
            annotation(
              Inline = true);
          end to_T_inv;
          annotation(
            Documentation(info = "<HTML>
<p>
Package <b>Frames.Quaternions</b> contains type definitions and
functions to transform rotational frame quantities with quaternions.
Functions of this package are currently only utilized in
MultiBody.Parts.Body components, when quaternions shall be used
as parts of the body states.
Some functions are also used in a new Modelica package for
B-Spline interpolation that is able to interpolate paths consisting of
position vectors and orientation objects.
</p>
<h4>Content</h4>
<p>In the table below an example is given for every function definition.
The used variables have the following declaration:
</p>
<pre>
   Quaternions.Orientation Q, Q1, Q2, Q_rel, Q_inv;
   Real[3,3]   T, T_inv;
   Real[3]     v1, v2, w1, w2, n_x, n_y, n_z, res_ori, phi;
   Real[6]     res_equal;
   Real        L, angle;
</pre>
<table border=1 cellspacing=0 cellpadding=2>
  <tr><th><b><i>Function/type</i></b></th><th><b><i>Description</i></b></th></tr>
  <tr><td valign=\"top\"><b>Orientation Q;</b></td>
      <td valign=\"top\">New type defining a quaternion object that describes<br>
          the rotation of frame 1 into frame 2.
      </td>
  </tr>
  <tr><td valign=\"top\"><b>der_Orientation</b> der_Q;</td>
      <td valign=\"top\">New type defining the first time derivative
         of Frames.Quaternions.Orientation.
      </td>
  </tr>
  <tr><td valign=\"top\">res_ori = <b>orientationConstraint</b>(Q);</td>
      <td valign=\"top\">Return the constraints between the variables of a quaternion object<br>
      (shall be zero).</td>
  </tr>
  <tr><td valign=\"top\">w1 = <b>angularVelocity1</b>(Q, der_Q);</td>
      <td valign=\"top\">Return angular velocity resolved in frame 1 from
          quaternion object Q<br> and its derivative der_Q.
     </td>
  </tr>
  <tr><td valign=\"top\">w2 = <b>angularVelocity2</b>(Q, der_Q);</td>
      <td valign=\"top\">Return angular velocity resolved in frame 2 from
          quaternion object Q<br> and its derivative der_Q.
     </td>
  </tr>
  <tr><td valign=\"top\">v1 = <b>resolve1</b>(Q,v2);</td>
      <td valign=\"top\">Transform vector v2 from frame 2 to frame 1.
      </td>
  </tr>
  <tr><td valign=\"top\">v2 = <b>resolve2</b>(Q,v1);</td>
      <td valign=\"top\">Transform vector v1 from frame 1 to frame 2.
     </td>
  </tr>
  <tr><td valign=\"top\">[v1,w1] = <b>multipleResolve1</b>(Q, [v2,w2]);</td>
      <td valign=\"top\">Transform several vectors from frame 2 to frame 1.
      </td>
  </tr>
  <tr><td valign=\"top\">[v2,w2] = <b>multipleResolve2</b>(Q, [v1,w1]);</td>
      <td valign=\"top\">Transform several vectors from frame 1 to frame 2.
      </td>
  </tr>
  <tr><td valign=\"top\">Q = <b>nullRotation</b>()</td>
      <td valign=\"top\">Return quaternion object R that does not rotate a frame.
  </tr>
  <tr><td valign=\"top\">Q_inv = <b>inverseRotation</b>(Q);</td>
      <td valign=\"top\">Return inverse quaternion object.
      </td>
  </tr>
  <tr><td valign=\"top\">Q_rel = <b>relativeRotation</b>(Q1,Q2);</td>
      <td valign=\"top\">Return relative quaternion object from two absolute
          quaternion objects.
      </td>
  </tr>
  <tr><td valign=\"top\">Q2 = <b>absoluteRotation</b>(Q1,Q_rel);</td>
      <td valign=\"top\">Return absolute quaternion object from another
          absolute<br> and a relative quaternion object.
      </td>
  </tr>
  <tr><td valign=\"top\">Q = <b>planarRotation</b>(e, angle);</td>
      <td valign=\"top\">Return quaternion object of a planar rotation.
      </td>
  </tr>
  <tr><td valign=\"top\">phi = <b>smallRotation</b>(Q);</td>
      <td valign=\"top\">Return rotation angles phi valid for a small rotation.
      </td>
  </tr>
  <tr><td valign=\"top\">Q = <b>from_T</b>(T);</td>
      <td valign=\"top\">Return quaternion object Q from transformation matrix T.
      </td>
  </tr>
  <tr><td valign=\"top\">Q = <b>from_T_inv</b>(T_inv);</td>
      <td valign=\"top\">Return quaternion object Q from inverse transformation matrix T_inv.
      </td>
  </tr>
  <tr><td valign=\"top\">T = <b>to_T</b>(Q);</td>
      <td valign=\"top\">Return transformation matrix T from quaternion object Q.
  </tr>
  <tr><td valign=\"top\">T_inv = <b>to_T_inv</b>(Q);</td>
      <td valign=\"top\">Return inverse transformation matrix T_inv from quaternion object Q.
      </td>
  </tr>
</table>
</HTML>"));
        end Quaternions;

        package TransformationMatrices "Functions for transformation matrices"
          extends Modelica.Icons.Package;

          type Orientation "Orientation type defining rotation from a frame 1 into a frame 2 with a transformation matrix"
            extends Internal.TransformationMatrix;

            encapsulated function equalityConstraint "Return the constraint residues to express that two frames have the same orientation"
              import Modelica;
              import Modelica.Mechanics.MultiBody.Frames.TransformationMatrices;
              extends Modelica.Icons.Function;
              input TransformationMatrices.Orientation T1 "Orientation object to rotate frame 0 into frame 1";
              input TransformationMatrices.Orientation T2 "Orientation object to rotate frame 0 into frame 2";
              output Real residue[3] "The rotation angles around x-, y-, and z-axis of frame 1 to rotate frame 1 into frame 2 for a small rotation (should be zero)";
            algorithm
              residue := {cross(T1[1, :], T1[2, :]) * T2[2, :], -cross(T1[1, :], T1[2, :]) * T2[1, :], T1[2, :] * T2[1, :]};
              annotation(
                Inline = true);
            end equalityConstraint;
            annotation(
              Documentation(info = "<html>
<p>
This type describes the <b>rotation</b> from a <b>frame 1</b> into a <b>frame 2</b>.
An instance <b>R</b> of type <b>Orientation</b> has the following interpretation:
</p>
<pre>
   <b>T</b> = [<b>e</b><sub>x</sub>, <b>e</b><sub>y</sub>, <b>e</b><sub>z</sub>];
       e.g., <b>T</b> = [1,0,0; 0,1,0; 0,0,1]
</pre>
<p>
where <b>e</b><sub>x</sub>,<b>e</b><sub>y</sub>,<b>e</b><sub>z</sub>
are unit vectors in the direction of the x-axis, y-axis, and z-axis
of frame 1, resolved in frame 2, respectively. Therefore, if <b>v</b><sub>1</sub>
is vector <b>v</b> resolved in frame 1 and <b>v</b><sub>2</sub> is
vector <b>v</b> resolved in frame 2, the following relationship holds:
</p>
<pre>
    <b>v</b><sub>2</sub> = <b>T</b> * <b>v</b><sub>1</sub>
</pre>
<p>
The <b>inverse</b> orientation
<b>T_inv</b> = <b>T</b><sup>T</sup> describes the rotation
from frame 2 into frame 1.
</p>
<p>
Since the orientation is described by 9 variables, there are
6 constraints between these variables. These constraints
are defined in function <b>TransformationMatrices.orientationConstraint</b>.
</p>
<p>
Note, that in the MultiBody library the rotation object is
never directly accessed but only with the access functions provided
in package TransformationMatrices. As a consequence, other implementations of
Rotation can be defined by adapting this package correspondingly.
</p>
</html>"));
          end Orientation;

          type der_Orientation = Real[3, 3](each unit = "1/s") "New type defining the first time derivative of Orientation";

          function orientationConstraint "Return residues of orientation constraints (shall be zero)"
            extends Modelica.Icons.Function;
            input TransformationMatrices.Orientation T "Orientation object to rotate frame 1 into frame 2";
            output Real residue[6] "Residues of constraints between elements of orientation object (shall be zero)";
          algorithm
            residue := {T[:, 1] * T[:, 1] - 1, T[:, 2] * T[:, 2] - 1, T[:, 3] * T[:, 3] - 1, T[:, 1] * T[:, 2], T[:, 1] * T[:, 3], T[:, 2] * T[:, 3]};
            annotation(
              Inline = true);
          end orientationConstraint;

          function angularVelocity1 "Return angular velocity resolved in frame 1 from orientation object and its derivative"
            extends Modelica.Icons.Function;
            input TransformationMatrices.Orientation T "Orientation object to rotate frame 1 into frame 2";
            input der_Orientation der_T "Derivative of T";
            output Modelica.SIunits.AngularVelocity w[3] "Angular velocity of frame 2 with respect to frame 1 resolved in frame 1";
          algorithm
/* The angular velocity w of frame 2 with respect to frame 1 resolved in frame 1,
     is defined as:
        w = vec( der(transpose(T))*T );
     where
                   |   0 -w3  w2 |
         skew(w) = |  w3   0 -w1 | and w = vec(skew(w))
                   | -w2  w1   0 |
     i.e.
         W = der(transpose(T))*T)
         w = {W(3,2), -W(3,1), W(2,1)}
     Therefore, only 3 values of W need to be computed:
             | der(T[:,1]) |
         W = | der(T[:,2]) | * | T[:,1], T[:,2], T[:,3] |
             | der(T[:,3]) |
             |  W(3,2) |   |  der(T[:,3])*T[:,2] |
         w = | -W(3,1) | = | -der(T[:,3])*T[:,1] |
             |  W(2,1) |   |  der(T[:,2])*T[:,1] |
  */
            w := {der_T[:, 3] * T[:, 2], -der_T[:, 3] * T[:, 1], der_T[:, 2] * T[:, 1]};
            annotation(
              Inline = true);
          end angularVelocity1;

          function angularVelocity2 "Return angular velocity resolved in frame 2 from orientation object and its derivative"
            extends Modelica.Icons.Function;
            input TransformationMatrices.Orientation T "Orientation object to rotate frame 1 into frame 2";
            input der_Orientation der_T "Derivative of T";
            output Modelica.SIunits.AngularVelocity w[3] "Angular velocity of frame 2 with respect to frame 1 resolved in frame 2";
          algorithm
/* The angular velocity w of frame 2 with respect to frame 1 resolved in frame 2,
     is defined as:
        w = vec(T*der(transpose(T)));
     where
                   |   0 -w3  w2 |
         skew(w) = |  w3   0 -w1 | and w = vec(skew(w))
                   | -w2  w1   0 |
     i.e.
         W = T*der(transpose(T))
         w = {W(3,2), -W(3,1), W(2,1)}
     Therefore, only 3 values of W need to be computed:
             | T[1,:] |
         W = | T[2,:] | * | der(T[1,:]), der(T[2,:]), der(T[3,:]) |
             | T[3,:] |
             |  W(3,2) |   |  T[3,:]*der(T[2,:]) |
         w = | -W(3,1) | = | -T[3,:]*der(T[1,:]) |
             |  W(2,1) |   |  T[2,:]*der(T[1,:]) |
  */
            w := {T[3, :] * der_T[2, :], -T[3, :] * der_T[1, :], T[2, :] * der_T[1, :]};
            annotation(
              Inline = true);
          end angularVelocity2;

          function resolve1 "Transform vector from frame 2 to frame 1"
            extends Modelica.Icons.Function;
            input TransformationMatrices.Orientation T "Orientation object to rotate frame 1 into frame 2";
            input Real v2[3] "Vector in frame 2";
            output Real v1[3] "Vector in frame 1";
          algorithm
            v1 := transpose(T) * v2;
            annotation(
              Inline = true);
          end resolve1;

          function resolve2 "Transform vector from frame 1 to frame 2"
            extends Modelica.Icons.Function;
            input TransformationMatrices.Orientation T "Orientation object to rotate frame 1 into frame 2";
            input Real v1[3] "Vector in frame 1";
            output Real v2[3] "Vector in frame 2";
          algorithm
            v2 := T * v1;
            annotation(
              Inline = true);
          end resolve2;

          function multipleResolve1 "Transform several vectors from frame 2 to frame 1"
            extends Modelica.Icons.Function;
            input TransformationMatrices.Orientation T "Orientation object to rotate frame 1 into frame 2";
            input Real v2[3, :] "Vectors in frame 2";
            output Real v1[3, size(v2, 2)] "Vectors in frame 1";
          algorithm
            v1 := transpose(T) * v2;
            annotation(
              Inline = true);
          end multipleResolve1;

          function multipleResolve2 "Transform several vectors from frame 1 to frame 2"
            extends Modelica.Icons.Function;
            input TransformationMatrices.Orientation T "Orientation object to rotate frame 1 into frame 2";
            input Real v1[3, :] "Vectors in frame 1";
            output Real v2[3, size(v1, 2)] "Vectors in frame 2";
          algorithm
            v2 := T * v1;
            annotation(
              Inline = true);
          end multipleResolve2;

          function resolveDyade1 "Transform second order tensor from frame 2 to frame 1"
            extends Modelica.Icons.Function;
            input TransformationMatrices.Orientation T "Orientation object to rotate frame 1 into frame 2";
            input Real D2[3, 3] "Second order tensor resolved in frame 2";
            output Real D1[3, 3] "Second order tensor resolved in frame 1";
          algorithm
            D1 := transpose(T) * D2 * T;
            annotation(
              Inline = true);
          end resolveDyade1;

          function resolveDyade2 "Transform second order tensor from frame 1 to frame 2"
            extends Modelica.Icons.Function;
            input TransformationMatrices.Orientation T "Orientation object to rotate frame 1 into frame 2";
            input Real D1[3, 3] "Second order tensor resolved in frame 1";
            output Real D2[3, 3] "Second order tensor resolved in frame 2";
          algorithm
            D2 := T * D1 * transpose(T);
            annotation(
              Inline = true);
          end resolveDyade2;

          function nullRotation "Return orientation object that does not rotate a frame"
            extends Modelica.Icons.Function;
            output TransformationMatrices.Orientation T "Orientation object such that frame 1 and frame 2 are identical";
          algorithm
            T := identity(3);
            annotation(
              Inline = true);
          end nullRotation;

          function inverseRotation "Return inverse orientation object"
            extends Modelica.Icons.Function;
            input TransformationMatrices.Orientation T "Orientation object to rotate frame 1 into frame 2";
            output TransformationMatrices.Orientation T_inv "Orientation object to rotate frame 2 into frame 1";
          algorithm
            T_inv := transpose(T);
            annotation(
              Inline = true);
          end inverseRotation;

          function relativeRotation "Return relative orientation object"
            extends Modelica.Icons.Function;
            input TransformationMatrices.Orientation T1 "Orientation object to rotate frame 0 into frame 1";
            input TransformationMatrices.Orientation T2 "Orientation object to rotate frame 0 into frame 2";
            output TransformationMatrices.Orientation T_rel "Orientation object to rotate frame 1 into frame 2";
          algorithm
            T_rel := T2 * transpose(T1);
            annotation(
              Inline = true);
          end relativeRotation;

          function absoluteRotation "Return absolute orientation object from another absolute and a relative orientation object"
            extends Modelica.Icons.Function;
            input TransformationMatrices.Orientation T1 "Orientation object to rotate frame 0 into frame 1";
            input TransformationMatrices.Orientation T_rel "Orientation object to rotate frame 1 into frame 2";
            output TransformationMatrices.Orientation T2 "Orientation object to rotate frame 0 into frame 2";
          algorithm
            T2 := T_rel * T1;
            annotation(
              Inline = true);
          end absoluteRotation;

          function planarRotation "Return orientation object of a planar rotation"
            import Modelica.Math;
            extends Modelica.Icons.Function;
            input Real e[3](each final unit = "1") "Normalized axis of rotation (must have length=1)";
            input Modelica.SIunits.Angle angle "Rotation angle to rotate frame 1 into frame 2 along axis e";
            output TransformationMatrices.Orientation T "Orientation object to rotate frame 1 into frame 2";
          algorithm
            T := [e] * transpose([e]) + (identity(3) - [e] * transpose([e])) * Math.cos(angle) - skew(e) * Math.sin(angle);
            annotation(
              Inline = true);
          end planarRotation;

          function planarRotationAngle "Return angle of a planar rotation, given the rotation axis and the representations of a vector in frame 1 and frame 2"
            extends Modelica.Icons.Function;
            input Real e[3](each final unit = "1") "Normalized axis of rotation to rotate frame 1 around e into frame 2 (must have length=1)";
            input Real v1[3] "A vector v resolved in frame 1 (shall not be parallel to e)";
            input Real v2[3] "Vector v resolved in frame 2, i.e., v2 = resolve2(planarRotation(e,angle),v1)";
            output Modelica.SIunits.Angle angle "Rotation angle to rotate frame 1 into frame 2 along axis e in the range: -pi <= angle <= pi";
          algorithm
/* Vector v is resolved in frame 1 and frame 2 according to:
        (1)  v2 = (e*transpose(e) + (identity(3) - e*transpose(e))*cos(angle) - skew(e)*sin(angle))*v1;
                = e*(e*v1) + (v1 - e*(e*v1))*cos(angle) - cross(e,v1)*sin(angle)
       Equation (1) is multiplied with "v1" resulting in (note: e*e = 1)
            v1*v2 = (e*v1)*(e*v2) + (v1*v1 - (e*v1)*(e*v1))*cos(angle)
       and therefore:
        (2) cos(angle) = ( v1*v2 - (e*v1)*(e*v2)) / (v1*v1 - (e*v1)*(e*v1))
       Similarly, equation (1) is multiplied with cross(e,v1), i.e., a
       a vector that is orthogonal to e and to v1:
              cross(e,v1)*v2 = - cross(e,v1)*cross(e,v1)*sin(angle)
       and therefore:
          (3) sin(angle) = -cross(e,v1)*v2/(cross(e,v1)*cross(e,v1));
       We have e*e=1; Therefore:
          (4) v1*v1 - (e*v1)*(e*v1) = |v1|^2 - (|v1|*cos(e,v1))^2
       and
          (5) cross(e,v1)*cross(e,v1) = (|v1|*sin(e,v1))^2
                                      = |v1|^2*(1 - cos(e,v1)^2)
                                      = |v1|^2 - (|v1|*cos(e,v1))^2
       The denominators of (2) and (3) are identical, according to (4) and (5).
       Furthermore, the denominators are always positive according to (5).
       Therefore, in the equation "angle = atan2(sin(angle), cos(angle))" the
       denominators of sin(angle) and cos(angle) can be removed,
       resulting in:
          angle = atan2(-cross(e,v1)*v2, v1*v2 - (e*v1)*(e*v2));
    */
            angle := Modelica.Math.atan2(-cross(e, v1) * v2, v1 * v2 - e * v1 * (e * v2));
            annotation(
              Inline = true,
              Documentation(info = "<HTML>
<p>
A call to this function of the form
</p>
<pre>
    Real[3]                e, v1, v2;
    Modelica.SIunits.Angle angle;
  <b>equation</b>
    angle = <b>planarRotationAngle</b>(e, v1, v2);
</pre>
<p>
computes the rotation angle \"<b>angle</b>\" of a planar
rotation along unit vector <b>e</b>, rotating frame 1 into frame 2, given
the coordinate representations of a vector \"v\" in frame 1 (<b>v1</b>)
and in frame 2 (<b>v2</b>). Therefore, the result of this function
fulfills the following equation:
</p>
<pre>
    v2 = <b>resolve2</b>(<b>planarRotation</b>(e,angle), v1)
</pre>
<p>
The rotation angle is returned in the range
</p>
<pre>
    -<font face=\"Symbol\">p</font> &lt;= angle &lt;= <font face=\"Symbol\">p</font>
</pre>
<p>
This function makes the following assumptions on the input arguments
</p>
<ul>
<li> Vector <b>e</b> has length 1, i.e., length(e) = 1</li>
<li> Vector \"v\" is not parallel to <b>e</b>, i.e.,
     length(cross(e,v1)) &ne; 0</li>
</ul>
<p>
The function does not check the above assumptions. If these
assumptions are violated, a wrong result will be returned
and/or a division by zero will occur.
</p>
</HTML>"));
          end planarRotationAngle;

          function axisRotation "Return rotation object to rotate around one frame axis"
            extends Modelica.Icons.Function;
            input Integer axis(min = 1, max = 3) "Rotate around 'axis' of frame 1";
            input Modelica.SIunits.Angle angle "Rotation angle to rotate frame 1 into frame 2 along 'axis' of frame 1";
            output TransformationMatrices.Orientation T "Orientation object to rotate frame 1 into frame 2";
          algorithm
            T := if axis == 1 then [1, 0, 0; 0, cos(angle), sin(angle); 0, -sin(angle), cos(angle)] else if axis == 2 then [cos(angle), 0, -sin(angle); 0, 1, 0; sin(angle), 0, cos(angle)] else [cos(angle), sin(angle), 0; -sin(angle), cos(angle), 0; 0, 0, 1];
            annotation(
              Inline = true);
          end axisRotation;

          function axesRotations "Return rotation object to rotate in sequence around 3 axes"
            extends Modelica.Icons.Function;
            input Integer sequence[3](min = {1, 1, 1}, max = {3, 3, 3}) = {1, 2, 3} "Sequence of rotations from frame 1 to frame 2 along axis sequence[i]";
            input Modelica.SIunits.Angle angles[3] = {0, 0, 0} "Rotation angles around the axes defined in 'sequence'";
            output TransformationMatrices.Orientation T "Orientation object to rotate frame 1 into frame 2";
          algorithm
            T := absoluteRotation(absoluteRotation(axisRotation(sequence[1], angles[1]), axisRotation(sequence[2], angles[2])), axisRotation(sequence[3], angles[3]));
            annotation(
              Inline = true);
          end axesRotations;

          function axesRotationsAngles "Return the 3 angles to rotate in sequence around 3 axes to construct the given orientation object"
            extends Modelica.Icons.Function;
            input TransformationMatrices.Orientation T "Orientation object to rotate frame 1 into frame 2";
            input Integer sequence[3](min = {1, 1, 1}, max = {3, 3, 3}) = {1, 2, 3} "Sequence of rotations from frame 1 to frame 2 along axis sequence[i]";
            input SI.Angle guessAngle1 = 0 "Select angles[1] such that |angles[1] - guessAngle1| is a minimum";
            output SI.Angle angles[3] "Rotation angles around the axes defined in 'sequence' such that T=TransformationMatrices.axesRotation(sequence,angles); -pi < angles[i] <= pi";
          protected
            Real e1_1[3](each final unit = "1") "First rotation axis, resolved in frame 1";
            Real e2_1a[3](each final unit = "1") "Second rotation axis, resolved in frame 1a";
            Real e3_1[3](each final unit = "1") "Third rotation axis, resolved in frame 1";
            Real e3_2[3](each final unit = "1") "Third rotation axis, resolved in frame 2";
            Real A "Coefficient A in the equation A*cos(angles[1])+B*sin(angles[1]) = 0";
            Real B "Coefficient B in the equation A*cos(angles[1])+B*sin(angles[1]) = 0";
            SI.Angle angle_1a "Solution 1 for angles[1]";
            SI.Angle angle_1b "Solution 2 for angles[1]";
            TransformationMatrices.Orientation T_1a "Orientation object to rotate frame 1 into frame 1a";
          algorithm
/* The rotation object T is constructed by:
     (1) Rotating frame 1 along axis e1 (= axis sequence[1]) with angles[1]
         arriving at frame 1a.
     (2) Rotating frame 1a along axis e2 (= axis sequence[2]) with angles[2]
         arriving at frame 1b.
     (3) Rotating frame 1b along axis e3 (= axis sequence[3]) with angles[3]
         arriving at frame 2.
     The goal is to determine angles[1:3]. This is performed in the following way:
     1. e2 and e3 are perpendicular to each other, i.e., e2*e3 = 0;
        Both vectors are resolved in frame 1 (T_ij is transformation matrix
        from frame j to frame i; e1_1*e2_1a = 0, since the vectors are
        perpendicular to each other):
           e3_1 = T_12*e3_2
                = T[sequence[3],:];
           e2_1 = T_11a*e2_1a
                = ( e1_1*transpose(e1_1) + (identity(3) - e1_1*transpose(e1_1))*cos(angles[1])
                    + skew(e1_1)*sin(angles[1]) )*e2_1a
                = e2_1a*cos(angles[1]) + cross(e1_1, e2_1a)*sin(angles[1]);
        From this follows finally an equation for angles[1]
           e2_1*e3_1 = 0
                     = (e2_1a*cos(angles[1]) + cross(e1_1, e2_1a)*sin(angles[1]))*e3_1
                     = (e2_1a*e3_1)*cos(angles[1]) + cross(e1_1, e2_1a)*e3_1*sin(angles[1])
                     = A*cos(angles[1]) + B*sin(angles[1])
                       with A = e2_1a*e3_1, B = cross(e1_1, e2_1a)*e3_1
        This equation has two solutions in the range -pi < angles[1] <= pi:
           sin(angles[1]) =  k*A/sqrt(A*A + B*B)
           cos(angles[1]) = -k*B/sqrt(A*A + B*B)
                        k = +/-1
           tan(angles[1]) = k*A/(-k*B)
        that is:
           angles[1] = atan2(k*A, -k*B)
        If A and B are both zero at the same time, there is a singular configuration
        resulting in an infinite number of solutions for angles[1] (every value
        is possible).
     2. angles[2] is determined with function TransformationMatrices.planarRotationAngle.
        This function requires to provide e_3 in frame 1a and in frame 1b:
          e3_1a = TransformationMatrices.resolve2(planarRotation(e1_1,angles[1]), e3_1);
          e3_1b = e3_2
     3. angles[3] is determined with function TransformationMatrices.planarRotationAngle.
        This function requires to provide e_2 in frame 1b and in frame 2:
          e2_1b = e2_1a
          e2_2  = TransformationMatrices.resolve2( T, TransformationMatrices.resolve1(planarRotation(e1_1,angles[1]), e2_1a));
  */
            assert(sequence[1] <> sequence[2] and sequence[2] <> sequence[3], "input argument 'sequence[1:3]' is not valid");
            e1_1 := if sequence[1] == 1 then {1, 0, 0} else if sequence[1] == 2 then {0, 1, 0} else {0, 0, 1};
            e2_1a := if sequence[2] == 1 then {1, 0, 0} else if sequence[2] == 2 then {0, 1, 0} else {0, 0, 1};
            e3_1 := T[sequence[3], :];
            e3_2 := if sequence[3] == 1 then {1, 0, 0} else if sequence[3] == 2 then {0, 1, 0} else {0, 0, 1};
            A := e2_1a * e3_1;
            B := cross(e1_1, e2_1a) * e3_1;
            if abs(A) <= 1.e-12 and abs(B) <= 1.e-12 then
              angles[1] := guessAngle1;
            else
              angle_1a := Modelica.Math.atan2(A, -B);
              angle_1b := Modelica.Math.atan2(-A, B);
              angles[1] := if abs(angle_1a - guessAngle1) <= abs(angle_1b - guessAngle1) then angle_1a else angle_1b;
            end if;
            T_1a := planarRotation(e1_1, angles[1]);
            angles[2] := TransformationMatrices.planarRotationAngle(e2_1a, TransformationMatrices.resolve2(T_1a, e3_1), e3_2);
            angles[3] := TransformationMatrices.planarRotationAngle(e3_2, e2_1a, TransformationMatrices.resolve2(T, TransformationMatrices.resolve1(T_1a, e2_1a)));
            annotation(
              Documentation(info = "<HTML>
<p>
A call to this function of the form
</p>
<pre>
    TransformationMatrices.Orientation     T;
    <b>parameter</b> Integer      sequence[3] = {1,2,3};
    Modelica.SIunits.Angle angles[3];
  <b>equation</b>
    angle = <b>axesRotationAngles</b>(T, sequence);
</pre>
<p>
computes the rotation angles \"<b>angles</b>[1:3]\" to rotate frame 1
into frame 2 along axes <b>sequence</b>[1:3], given the orientation
object <b>T</b> from frame 1 to frame 2. Therefore, the result of
this function fulfills the following equation:
</p>
<pre>
    T = <b>axesRotation</b>(sequence, angles)
</pre>
<p>
The rotation angles are returned in the range
</p>
<pre>
    -<font face=\"Symbol\">p</font> &lt;= angles[i] &lt;= <font face=\"Symbol\">p</font>
</pre>
<p>
There are <b>two solutions</b> for \"angles[1]\" in this range.
Via the third argument <b>guessAngle1</b> (default = 0) the
returned solution is selected such that |angles[1] - guessAngle1| is
minimal. The orientation object T may be in a singular configuration, i.e.,
there is an infinite number of angle values leading to the same T. The returned solution is
selected by setting angles[1] = guessAngle1. Then angles[2]
and angles[3] can be uniquely determined in the above range.
</p>
<p>
Note, that input argument <b>sequence</b> has the restriction that
only values 1,2,3 can be used and that sequence[1] &ne; sequence[2]
and sequence[2] &ne; sequence[3]. Often used values are:
</p>
<pre>
  sequence = <b>{1,2,3}</b>  // Cardan angle sequence
           = <b>{3,1,3}</b>  // Euler angle sequence
           = <b>{3,2,1}</b>  // Tait-Bryan angle sequence
</pre>
</HTML>"));
          end axesRotationsAngles;

          function smallRotation "Return rotation angles valid for a small rotation and optionally residues that should be zero"
            extends Modelica.Icons.Function;
            input TransformationMatrices.Orientation T "Orientation object to rotate frame 1 into frame 2";
            input Boolean withResidues = false "= false/true, if 'angles'/'angles and residues' are returned in phi";
            output Modelica.SIunits.Angle phi[if withResidues then 6 else 3] "The rotation angles around x-, y-, and z-axis of frame 1 to rotate frame 1 into frame 2 for a small rotation + optionally 3 residues that should be zero";
          algorithm
/* Planar rotation:
       Trel = [e]*transpose([e]) + (identity(3) - [e]*transpose([e]))*cos(angle) - skew(e)*sin(angle)
            = identity(3) - skew(e)*angle, for small angles
            = identity(3) - skew(e*angle)
               define phi = e*angle, then
       Trel = [1,      phi3,   -phi2;
               -phi3,     1,    phi1;
                phi2, -phi1,       1 ];
  */
            phi := if withResidues then {T[2, 3], -T[1, 3], T[1, 2], T[1, 1] - 1, T[2, 2] - 1, T[1, 1] * T[2, 2] - T[2, 1] * T[1, 2] - 1} else {T[2, 3], -T[1, 3], T[1, 2]};
            annotation(
              Inline = true);
          end smallRotation;

          function from_nxy "Return orientation object from n_x and n_y vectors"
            extends Modelica.Icons.Function;
            input Real n_x[3](each final unit = "1") "Vector in direction of x-axis of frame 2, resolved in frame 1";
            input Real n_y[3](each final unit = "1") "Vector in direction of y-axis of frame 2, resolved in frame 1";
            output TransformationMatrices.Orientation T "Orientation object to rotate frame 1 into frame 2";
          protected
            Real abs_n_x = sqrt(n_x * n_x);
            Real e_x[3](each final unit = "1") = if abs_n_x < 1.e-10 then {1, 0, 0} else n_x / abs_n_x;
            Real n_z_aux[3](each final unit = "1") = cross(e_x, n_y);
            Real n_y_aux[3](each final unit = "1") = if n_z_aux * n_z_aux > 1.0e-6 then n_y else if abs(e_x[1]) > 1.0e-6 then {0, 1, 0} else {1, 0, 0};
            Real e_z_aux[3](each final unit = "1") = cross(e_x, n_y_aux);
            Real e_z[3](each final unit = "1") = e_z_aux / sqrt(e_z_aux * e_z_aux);
          algorithm
            T := {e_x, cross(e_z, e_x), e_z};
            annotation(
              Documentation(info = "<html>
<p>
It is assumed that the two input vectors n_x and n_y are
resolved in frame 1 and are directed along the x and y axis
of frame 2 (i.e., n_x and n_y are orthogonal to each other)
The function returns the orientation object T to rotate from
frame 1 to frame 2.
</p>
<p>
The function is robust in the sense that it returns always
an orientation object T, even if n_y is not orthogonal to n_x.
This is performed in the following way:
</p>
<p>
If n_x and n_y are not orthogonal to each other, first a unit
vector e_y is determined that is orthogonal to n_x and is lying
in the plane spanned by n_x and n_y. If n_x and n_y are parallel
or nearly parallel to each other, a vector e_y is selected
arbitrarily such that e_x and e_y are orthogonal to each other.
</p>
</html>"));
          end from_nxy;

          function from_nxz "Return orientation object from n_x and n_z vectors"
            extends Modelica.Icons.Function;
            input Real n_x[3](each final unit = "1") "Vector in direction of x-axis of frame 2, resolved in frame 1";
            input Real n_z[3](each final unit = "1") "Vector in direction of z-axis of frame 2, resolved in frame 1";
            output TransformationMatrices.Orientation T "Orientation object to rotate frame 1 into frame 2";
          protected
            Real abs_n_x = sqrt(n_x * n_x);
            Real e_x[3](each final unit = "1") = if abs_n_x < 1.e-10 then {1, 0, 0} else n_x / abs_n_x;
            Real n_y_aux[3](each final unit = "1") = cross(n_z, e_x);
            Real n_z_aux[3](each final unit = "1") = if n_y_aux * n_y_aux > 1.0e-6 then n_z else if abs(e_x[1]) > 1.0e-6 then {0, 0, 1} else {1, 0, 0};
            Real e_y_aux[3](each final unit = "1") = cross(n_z_aux, e_x);
            Real e_y[3](each final unit = "1") = e_y_aux / sqrt(e_y_aux * e_y_aux);
          algorithm
            T := {e_x, e_y, cross(e_x, e_y)};
            annotation(
              Documentation(info = "<html>
<p>
It is assumed that the two input vectors n_x and n_z are
resolved in frame 1 and are directed along the x and z axis
of frame 2 (i.e., n_x and n_z are orthogonal to each other)
The function returns the orientation object T to rotate from
frame 1 to frame 2.
</p>
<p>
The function is robust in the sense that it returns always
an orientation object T, even if n_z is not orthogonal to n_x.
This is performed in the following way:
</p>
<p>
If n_x and n_z are not orthogonal to each other, first a unit
vector e_z is determined that is orthogonal to n_x and is lying
in the plane spanned by n_x and n_z. If n_x and n_z are parallel
or nearly parallel to each other, a vector e_z is selected
arbitrarily such that n_x and e_z are orthogonal to each other.
</p>
</html>"));
          end from_nxz;

          function from_T "Return orientation object R from transformation matrix T"
            extends Modelica.Icons.Function;
            input Real T[3, 3] "Transformation matrix to transform vector from frame 1 to frame 2 (v2=T*v1)";
            output TransformationMatrices.Orientation R "Orientation object to rotate frame 1 into frame 2";
          algorithm
            R := T;
            annotation(
              Inline = true);
          end from_T;

          function from_T_inv "Return orientation object R from inverse transformation matrix T_inv"
            extends Modelica.Icons.Function;
            input Real T_inv[3, 3] "Inverse transformation matrix to transform vector from frame 2 to frame 1 (v1=T_inv*v2)";
            output TransformationMatrices.Orientation R "Orientation object to rotate frame 1 into frame 2";
          algorithm
            R := transpose(T_inv);
            annotation(
              Inline = true);
          end from_T_inv;

          function from_Q "Return orientation object T from quaternion orientation object Q"
            extends Modelica.Icons.Function;
            input Quaternions.Orientation Q "Quaternions orientation object to rotate frame 1 into frame 2";
            output TransformationMatrices.Orientation T "Orientation object to rotate frame 1 into frame 2";
          algorithm
/*
  T := (2*Q[4]*Q[4] - 1)*identity(3) + 2*([Q[1:3]]*transpose([Q[1:3]]) - Q[4]*
    skew(Q[1:3]));
*/
            T := [2 * (Q[1] * Q[1] + Q[4] * Q[4]) - 1, 2 * (Q[1] * Q[2] + Q[3] * Q[4]), 2 * (Q[1] * Q[3] - Q[2] * Q[4]); 2 * (Q[2] * Q[1] - Q[3] * Q[4]), 2 * (Q[2] * Q[2] + Q[4] * Q[4]) - 1, 2 * (Q[2] * Q[3] + Q[1] * Q[4]); 2 * (Q[3] * Q[1] + Q[2] * Q[4]), 2 * (Q[3] * Q[2] - Q[1] * Q[4]), 2 * (Q[3] * Q[3] + Q[4] * Q[4]) - 1];
            annotation(
              Inline = true);
          end from_Q;

          function to_T "Return transformation matrix T from orientation object R"
            extends Modelica.Icons.Function;
            input TransformationMatrices.Orientation R "Orientation object to rotate frame 1 into frame 2";
            output Real T[3, 3] "Transformation matrix to transform vector from frame 1 to frame 2 (v2=T*v1)";
          algorithm
            T := R;
            annotation(
              Inline = true);
          end to_T;

          function to_T_inv "Return inverse transformation matrix T_inv from orientation object R"
            extends Modelica.Icons.Function;
            input TransformationMatrices.Orientation R "Orientation object to rotate frame 1 into frame 2";
            output Real T_inv[3, 3] "Inverse transformation matrix to transform vector from frame 2 into frame 1 (v1=T_inv*v2)";
          algorithm
            T_inv := transpose(R);
            annotation(
              Inline = true);
          end to_T_inv;

          function to_Q "Return quaternion orientation object Q from orientation object T"
            extends Modelica.Icons.Function;
            input TransformationMatrices.Orientation T "Orientation object to rotate frame 1 into frame 2";
            input Quaternions.Orientation Q_guess = Quaternions.nullRotation() "Guess value for output Q (there are 2 solutions; the one closer to Q_guess is used";
            output Quaternions.Orientation Q "Quaternions orientation object to rotate frame 1 into frame 2";
          algorithm
            Q := Quaternions.from_T(T, Q_guess);
            annotation(
              Inline = true);
          end to_Q;

          function to_vector "Map rotation object into vector"
            extends Modelica.Icons.Function;
            input TransformationMatrices.Orientation T "Orientation object to rotate frame 1 into frame 2";
            output Real vec[9] "Elements of T in one vector";
          algorithm
            vec := {T[1, 1], T[2, 1], T[3, 1], T[1, 2], T[2, 2], T[3, 2], T[1, 3], T[2, 3], T[3, 3]};
            annotation(
              Inline = true);
          end to_vector;

          function to_exy "Map rotation object into e_x and e_y vectors of frame 2, resolved in frame 1"
            extends Modelica.Icons.Function;
            input TransformationMatrices.Orientation T "Orientation object to rotate frame 1 into frame 2";
            output Real exy[3, 2] "= [e_x, e_y] where e_x and e_y are axes unit vectors of frame 2, resolved in frame 1";
          algorithm
            exy := [T[1, :], T[2, :]];
            annotation(
              Inline = true);
          end to_exy;
          annotation(
            Documentation(info = "<HTML>
<p>
Package <b>Frames.TransformationMatrices</b> contains type definitions and
functions to transform rotational frame quantities using
transformation matrices.
</p>
<h4>Content</h4>
<p>In the table below an example is given for every function definition.
The used variables have the following declaration:
</p>
<pre>
   Orientation T, T1, T2, T_rel, T_inv;
   Real[3]     v1, v2, w1, w2, n_x, n_y, n_z, e, e_x, res_ori, phi;
   Real[6]     res_equal;
   Real        L, angle;
</pre>
<table border=1 cellspacing=0 cellpadding=2>
  <tr><th><b><i>Function/type</i></b></th><th><b><i>Description</i></b></th></tr>
  <tr><td valign=\"top\"><b>Orientation T;</b></td>
      <td valign=\"top\">New type defining an orientation object that describes<br>
          the rotation of frame 1 into frame 2.
      </td>
  </tr>
  <tr><td valign=\"top\"><b>der_Orientation</b> der_T;</td>
      <td valign=\"top\">New type defining the first time derivative
         of Frames.Orientation.
      </td>
  </tr>
  <tr><td valign=\"top\">res_ori = <b>orientationConstraint</b>(T);</td>
      <td valign=\"top\">Return the constraints between the variables of an orientation object<br>
      (shall be zero).</td>
  </tr>
  <tr><td valign=\"top\">w1 = <b>angularVelocity1</b>(T, der_T);</td>
      <td valign=\"top\">Return angular velocity resolved in frame 1 from
          orientation object T<br> and its derivative der_T.
     </td>
  </tr>
  <tr><td valign=\"top\">w2 = <b>angularVelocity2</b>(T, der_T);</td>
      <td valign=\"top\">Return angular velocity resolved in frame 2 from
          orientation object T<br> and its derivative der_T.
     </td>
  </tr>
  <tr><td valign=\"top\">v1 = <b>resolve1</b>(T,v2);</td>
      <td valign=\"top\">Transform vector v2 from frame 2 to frame 1.
      </td>
  </tr>
  <tr><td valign=\"top\">v2 = <b>resolve2</b>(T,v1);</td>
      <td valign=\"top\">Transform vector v1 from frame 1 to frame 2.
     </td>
  </tr>
  <tr><td valign=\"top\">[v1,w1] = <b>multipleResolve1</b>(T, [v2,w2]);</td>
      <td valign=\"top\">Transform several vectors from frame 2 to frame 1.
      </td>
  </tr>
  <tr><td valign=\"top\">[v2,w2] = <b>multipleResolve2</b>(T, [v1,w1]);</td>
      <td valign=\"top\">Transform several vectors from frame 1 to frame 2.
      </td>
  </tr>
  <tr><td valign=\"top\">D1 = <b>resolveDyade1</b>(T,D2);</td>
      <td valign=\"top\">Transform second order tensor D2 from frame 2 to frame 1.
      </td>
  </tr>
  <tr><td valign=\"top\">D2 = <b>resolveDyade2</b>(T,D1);</td>
      <td valign=\"top\">Transform second order tensor D1 from frame 1 to frame 2.
     </td>
  </tr>
  <tr><td valign=\"top\">T= <b>nullRotation</b>()</td>
      <td valign=\"top\">Return orientation object T that does not rotate a frame.
  </tr>
  <tr><td valign=\"top\">T_inv = <b>inverseRotation</b>(T);</td>
      <td valign=\"top\">Return inverse orientation object.
      </td>
  </tr>
  <tr><td valign=\"top\">T_rel = <b>relativeRotation</b>(T1,T2);</td>
      <td valign=\"top\">Return relative orientation object from two absolute
          orientation objects.
      </td>
  </tr>
  <tr><td valign=\"top\">T2 = <b>absoluteRotation</b>(T1,T_rel);</td>
      <td valign=\"top\">Return absolute orientation object from another
          absolute<br> and a relative orientation object.
      </td>
  </tr>
  <tr><td valign=\"top\">T = <b>planarRotation</b>(e, angle);</td>
      <td valign=\"top\">Return orientation object of a planar rotation.
      </td>
  </tr>
  <tr><td valign=\"top\">angle = <b>planarRotationAngle</b>(e, v1, v2);</td>
      <td valign=\"top\">Return angle of a planar rotation, given the rotation axis<br>
        and the representations of a vector in frame 1 and frame 2.
      </td>
  </tr>
  <tr><td valign=\"top\">T = <b>axisRotation</b>(i, angle);</td>
      <td valign=\"top\">Return orientation object T for rotation around axis i of frame 1.
      </td>
  </tr>
  <tr><td valign=\"top\">T = <b>axesRotations</b>(sequence, angles);</td>
      <td valign=\"top\">Return rotation object to rotate in sequence around 3 axes. Example:<br>
          T = axesRotations({1,2,3},{90,45,-90});
      </td>
  </tr>
  <tr><td valign=\"top\">angles = <b>axesRotationsAngles</b>(T, sequence);</td>
      <td valign=\"top\">Return the 3 angles to rotate in sequence around 3 axes to<br>
          construct the given orientation object.
      </td>
  </tr>
  <tr><td valign=\"top\">phi = <b>smallRotation</b>(T);</td>
      <td valign=\"top\">Return rotation angles phi valid for a small rotation.
      </td>
  </tr>
  <tr><td valign=\"top\">T = <b>from_nxy</b>(n_x, n_y);</td>
      <td valign=\"top\">Return orientation object from n_x and n_y vectors.
      </td>
  </tr>
  <tr><td valign=\"top\">T = <b>from_nxz</b>(n_x, n_z);</td>
      <td valign=\"top\">Return orientation object from n_x and n_z vectors.
      </td>
  </tr>
  <tr><td valign=\"top\">R = <b>from_T</b>(T);</td>
      <td valign=\"top\">Return orientation object R from transformation matrix T.
      </td>
  </tr>
  <tr><td valign=\"top\">R = <b>from_T_inv</b>(T_inv);</td>
      <td valign=\"top\">Return orientation object R from inverse transformation matrix T_inv.
      </td>
  </tr>
  <tr><td valign=\"top\">T = <b>from_Q</b>(Q);</td>
      <td valign=\"top\">Return orientation object T from quaternion orientation object Q.
      </td>
  </tr>
  <tr><td valign=\"top\">T = <b>to_T</b>(R);</td>
      <td valign=\"top\">Return transformation matrix T from orientation object R.
  </tr>
  <tr><td valign=\"top\">T_inv = <b>to_T_inv</b>(R);</td>
      <td valign=\"top\">Return inverse transformation matrix T_inv from orientation object R.
      </td>
  </tr>
  <tr><td valign=\"top\">Q = <b>to_Q</b>(T);</td>
      <td valign=\"top\">Return quaternion orientation object Q from orientation object T.
      </td>
  </tr>
  <tr><td valign=\"top\">exy = <b>to_exy</b>(T);</td>
      <td valign=\"top\">Return [e_x, e_y] matrix of an orientation object T, <br>
          with e_x and e_y vectors of frame 2, resolved in frame 1.
  </tr>
</table>
</HTML>"));
        end TransformationMatrices;

        package Internal "Internal definitions that may be removed or changed (do not use)"
          extends Modelica.Icons.InternalPackage;
          type TransformationMatrix = Real[3, 3];
          type QuaternionBase = Real[4];

          function maxWithoutEvent "Maximum of the input arguments, without event and function can be differentiated"
            extends Modelica.Icons.Function;
            input Real u1;
            input Real u2;
            output Real y;
          algorithm
            y := if u1 > u2 then u1 else u2;
            annotation(
              Inline = false,
              derivative = maxWithoutEvent_d,
              Documentation(info = "<html>
<p>
Function <b>maxWithoutEvent</b> returns the maximum of its two
input arguments. This functions is used instead of the Modelica
built-in function \"max\" or an if-statement with \"noEvent(..)\",
in order that the function can be differentiated by providing
the first and second derivatives with additional functions.
Note, from a strict mathematical point of view the derivatives
will be wrong, since a Dirac impulses would occur in the
derivatives. For the special cases as used in the MultiBody
library, this is irrelevant and therefore the usage of the function is correct.
</p>
</html>"));
          end maxWithoutEvent;

          function maxWithoutEvent_d "First derivative of function maxWithoutEvent(..)"
            extends Modelica.Icons.Function;
            input Real u1;
            input Real u2;
            input Real u1_d;
            input Real u2_d;
            output Real y_d;
          algorithm
            y_d := if u1 > u2 then u1_d else u2_d;
            annotation(
              Inline = false,
              derivative(order = 2) = maxWithoutEvent_dd);
          end maxWithoutEvent_d;

          function maxWithoutEvent_dd "First derivative of function maxWithoutEvent_d(..)"
            extends Modelica.Icons.Function;
            input Real u1;
            input Real u2;
            input Real u1_d;
            input Real u2_d;
            input Real u1_dd;
            input Real u2_dd;
            output Real y_dd;
          algorithm
            y_dd := if u1 > u2 then u1_dd else u2_dd;
            annotation(
              Inline = true);
          end maxWithoutEvent_dd;

          function resolve1_der "Derivative of function Frames.resolve1(..)"
            import Modelica.Mechanics.MultiBody.Frames;
            extends Modelica.Icons.Function;
            input Orientation R "Orientation object to rotate frame 1 into frame 2";
            input Real v2[3] "Vector resolved in frame 2";
            input Real v2_der[3] "= der(v2)";
            output Real v1_der[3] "Derivative of vector v resolved in frame 1";
          algorithm
            v1_der := Frames.resolve1(R, v2_der + cross(R.w, v2));
            annotation(
              Inline = true);
          end resolve1_der;

          function resolve2_der "Derivative of function Frames.resolve2(..)"
            import Modelica.Mechanics.MultiBody.Frames;
            extends Modelica.Icons.Function;
            input Orientation R "Orientation object to rotate frame 1 into frame 2";
            input Real v1[3] "Vector resolved in frame 1";
            input Real v1_der[3] "= der(v1)";
            output Real v2_der[3] "Derivative of vector v resolved in frame 2";
          algorithm
            v2_der := Frames.resolve2(R, v1_der) - cross(R.w, Frames.resolve2(R, v1));
            annotation(
              Inline = true);
          end resolve2_der;

          function resolveRelative_der "Derivative of function Frames.resolveRelative(..)"
            import Modelica.Mechanics.MultiBody.Frames;
            extends Modelica.Icons.Function;
            input Real v1[3] "Vector in frame 1";
            input Orientation R1 "Orientation object to rotate frame 0 into frame 1";
            input Orientation R2 "Orientation object to rotate frame 0 into frame 2";
            input Real v1_der[3] "= der(v1)";
            output Real v2_der[3] "Derivative of vector v resolved in frame 2";
          algorithm
            v2_der := Frames.resolveRelative(v1_der + cross(R1.w, v1), R1, R2) - cross(R2.w, Frames.resolveRelative(v1, R1, R2));
/* skew(w) = T*der(T'), -skew(w) = der(T)*T'

     v2 = T2*(T1'*v1)
     der(v2) = der(T2)*T1'*v1 + T2*der(T1')*v1 + T2*T1'*der(v1)
             = der(T2)*T2'*T2*T1'*v1 + T2*T1'*T1*der(T1')*v1 + T2*T1'*der(v1)
             = -w2 x (T2*T1'*v1) + T2*T1'*(w1 x v1) + T2*T1'*der(v1)
             = T2*T1'*(der(v1) + w1 x v1) - w2 x (T2*T1'*v1)
  */
            annotation(
              Inline = true);
          end resolveRelative_der;
        end Internal;
        annotation(
          Documentation(info = "<HTML>
<p>
Package <b>Frames</b> contains type definitions and
functions to transform rotational frame quantities. The basic idea is to
hide the actual definition of an <b>orientation</b> in this package
by providing essentially type <b>Orientation</b> together with
<b>functions</b> operating on instances of this type.
</p>
<h4>Content</h4>
<p>In the table below an example is given for every function definition.
The used variables have the following declaration:
</p>
<pre>
   Frames.Orientation R, R1, R2, R_rel, R_inv;
   Real[3,3]   T, T_inv;
   Real[3]     v1, v2, w1, w2, n_x, n_y, n_z, e, e_x, res_ori, phi;
   Real[6]     res_equal;
   Real        L, angle;
</pre>
<table border=1 cellspacing=0 cellpadding=2>
  <tr><th><b><i>Function/type</i></b></th><th><b><i>Description</i></b></th></tr>
  <tr><td valign=\"top\"><b>Orientation R;</b></td>
      <td valign=\"top\">New type defining an orientation object that describes<br>
          the rotation of frame 1 into frame 2.
      </td>
  </tr>
  <tr><td valign=\"top\">res_ori = <b>orientationConstraint</b>(R);</td>
      <td valign=\"top\">Return the constraints between the variables of an orientation object<br>
      (shall be zero).</td>
  </tr>
  <tr><td valign=\"top\">w1 = <b>angularVelocity1</b>(R);</td>
      <td valign=\"top\">Return angular velocity resolved in frame 1 from
          orientation object R.
     </td>
  </tr>
  <tr><td valign=\"top\">w2 = <b>angularVelocity2</b>(R);</td>
      <td valign=\"top\">Return angular velocity resolved in frame 2 from
          orientation object R.
     </td>
  </tr>
  <tr><td valign=\"top\">v1 = <b>resolve1</b>(R,v2);</td>
      <td valign=\"top\">Transform vector v2 from frame 2 to frame 1.
      </td>
  </tr>
  <tr><td valign=\"top\">v2 = <b>resolve2</b>(R,v1);</td>
      <td valign=\"top\">Transform vector v1 from frame 1 to frame 2.
     </td>
  </tr>
  <tr><td valign=\"top\">v2 = <b>resolveRelative</b>(v1,R1,R2);</td>
      <td valign=\"top\">Transform vector v1 from frame 1 to frame 2
          using absolute orientation objects R1 of frame 1 and R2 of frame 2.
      </td>
  </tr>
  <tr><td valign=\"top\">D1 = <b>resolveDyade1</b>(R,D2);</td>
      <td valign=\"top\">Transform second order tensor D2 from frame 2 to frame 1.
      </td>
  </tr>
  <tr><td valign=\"top\">D2 = <b>resolveDyade2</b>(R,D1);</td>
      <td valign=\"top\">Transform second order tensor D1 from frame 1 to frame 2.
     </td>
  </tr>
  <tr><td valign=\"top\">R = <b>nullRotation</b>()</td>
      <td valign=\"top\">Return orientation object R that does not rotate a frame.
  </tr>
  <tr><td valign=\"top\">R_inv = <b>inverseRotation</b>(R);</td>
      <td valign=\"top\">Return inverse orientation object.
      </td>
  </tr>
  <tr><td valign=\"top\">R_rel = <b>relativeRotation</b>(R1,R2);</td>
      <td valign=\"top\">Return relative orientation object from two absolute
          orientation objects.
      </td>
  </tr>
  <tr><td valign=\"top\">R2 = <b>absoluteRotation</b>(R1,R_rel);</td>
      <td valign=\"top\">Return absolute orientation object from another
          absolute<br> and a relative orientation object.
      </td>
  </tr>
  <tr><td valign=\"top\">R = <b>planarRotation</b>(e, angle, der_angle);</td>
      <td valign=\"top\">Return orientation object of a planar rotation.
      </td>
  </tr>
  <tr><td valign=\"top\">angle = <b>planarRotationAngle</b>(e, v1, v2);</td>
      <td valign=\"top\">Return angle of a planar rotation, given the rotation axis<br>
        and the representations of a vector in frame 1 and frame 2.
      </td>
  </tr>
  <tr><td valign=\"top\">R = <b>axisRotation</b>(axis, angle, der_angle);</td>
      <td valign=\"top\">Return orientation object R to rotate around angle along axis of frame 1.
      </td>
  </tr>
  <tr><td valign=\"top\">R = <b>axesRotations</b>(sequence, angles, der_angles);</td>
      <td valign=\"top\">Return rotation object to rotate in sequence around 3 axes. Example:<br>
          R = axesRotations({1,2,3},{pi/2,pi/4,-pi}, zeros(3));
      </td>
  </tr>
  <tr><td valign=\"top\">angles = <b>axesRotationsAngles</b>(R, sequence);</td>
      <td valign=\"top\">Return the 3 angles to rotate in sequence around 3 axes to<br>
          construct the given orientation object.
      </td>
  </tr>
  <tr><td valign=\"top\">phi = <b>smallRotation</b>(R);</td>
      <td valign=\"top\">Return rotation angles phi valid for a small rotation R.
      </td>
  </tr>
  <tr><td valign=\"top\">R = <b>from_nxy</b>(n_x, n_y);</td>
      <td valign=\"top\">Return orientation object from n_x and n_y vectors.
      </td>
  </tr>
  <tr><td valign=\"top\">R = <b>from_nxz</b>(n_x, n_z);</td>
      <td valign=\"top\">Return orientation object from n_x and n_z vectors.
      </td>
  </tr>
  <tr><td valign=\"top\">R = <b>from_T</b>(T,w);</td>
      <td valign=\"top\">Return orientation object R from transformation matrix T and
          its angular velocity w.
      </td>
  </tr>
  <tr><td valign=\"top\">R = <b>from_T2</b>(T,der(T));</td>
      <td valign=\"top\">Return orientation object R from transformation matrix T and
          its derivative der(T).
      </td>
  </tr>
  <tr><td valign=\"top\">R = <b>from_T_inv</b>(T_inv,w);</td>
      <td valign=\"top\">Return orientation object R from inverse transformation matrix T_inv and
          its angular velocity w.
      </td>
  </tr>
  <tr><td valign=\"top\">R = <b>from_Q</b>(Q,w);</td>
      <td valign=\"top\">Return orientation object R from quaternion orientation object Q
          and its angular velocity w.
      </td>
  </tr>
  <tr><td valign=\"top\">T = <b>to_T</b>(R);</td>
      <td valign=\"top\">Return transformation matrix T from orientation object R.
  </tr>
  <tr><td valign=\"top\">T_inv = <b>to_T_inv</b>(R);</td>
      <td valign=\"top\">Return inverse transformation matrix T_inv from orientation object R.
      </td>
  </tr>
  <tr><td valign=\"top\">Q = <b>to_Q</b>(R);</td>
      <td valign=\"top\">Return quaternion orientation object Q from orientation object R.
      </td>
  </tr>
  <tr><td valign=\"top\">exy = <b>to_exy</b>(R);</td>
      <td valign=\"top\">Return [e_x, e_y] matrix of an orientation object R, <br>
          with e_x and e_y vectors of frame 2, resolved in frame 1.
  </tr>
  <tr><td valign=\"top\">L = <b>length</b>(n_x);</td>
      <td valign=\"top\">Return length L of a vector n_x.
      </td>
  </tr>
  <tr><td valign=\"top\">e_x = <b>normalize</b>(n_x);</td>
      <td valign=\"top\">Return normalized vector e_x of n_x such that length of e_x is one.
      </td>
  </tr>
  <tr><td valign=\"top\">e = <b>axis</b>(i);</td>
      <td valign=\"top\">Return unit vector e directed along axis i
      </td>
  </tr>
  <tr><td valign=\"top\"><a href=\"modelica://Modelica.Mechanics.MultiBody.Frames.Quaternions\">Quaternions</a></td>
      <td valign=\"top\"><b>Package</b> with functions to transform rotational frame quantities based
          on quaternions (also called Euler parameters).
      </td>
  </tr>
  <tr><td valign=\"top\"><a href=\"modelica://Modelica.Mechanics.MultiBody.Frames.TransformationMatrices\">TransformationMatrices</a></td>
      <td valign=\"top\"><b>Package</b> with functions to transform rotational frame quantities based
          on transformation matrices.
      </td>
  </tr>
</table>
</HTML>"),
          Icon(graphics = {Line(points = {{-2, -18}, {80, -60}}, color = {95, 95, 95}), Line(points = {{-2, -18}, {-2, 80}}, color = {95, 95, 95}), Line(points = {{-78, -56}, {-2, -18}}, color = {95, 95, 95})}));
      end Frames;

      package Interfaces "Connectors and partial models for 3-dim. mechanical components"
        extends Modelica.Icons.InterfacesPackage;

        connector Frame "Coordinate system fixed to the component with one cut-force and cut-torque (no icon)"
          SI.Position r_0[3] "Position vector from world frame to the connector frame origin, resolved in world frame";
          Frames.Orientation R "Orientation object to rotate the world frame into the connector frame";
          flow SI.Force f[3] "Cut-force resolved in connector frame" annotation(
            unassignedMessage = "All Forces cannot be uniquely calculated.
The reason could be that the mechanism contains
a planar loop or that joints constrain the
same motion. For planar loops, use for one
revolute joint per loop the joint
Joints.RevolutePlanarLoopConstraint instead of
Joints.Revolute.");
          flow SI.Torque t[3] "Cut-torque resolved in connector frame";
          annotation(
            Documentation(info = "<html>
<p>
Basic definition of a coordinate system that is fixed to a mechanical
component. In the origin of the coordinate system the cut-force
and the cut-torque is acting. This component has no icon definition
and is only used by inheritance from frame connectors to define
different icons.
</p>
</html>"));
        end Frame;

        connector Frame_a "Coordinate system fixed to the component with one cut-force and cut-torque (filled rectangular icon)"
          extends Frame;
          annotation(
            defaultComponentName = "frame_a",
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}, initialScale = 0.16), graphics = {Rectangle(extent = {{-10, 10}, {10, -10}}, lineColor = {95, 95, 95}, lineThickness = 0.5), Rectangle(extent = {{-30, 100}, {30, -100}}, lineColor = {0, 0, 0}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid)}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}, initialScale = 0.16), graphics = {Text(extent = {{-140, -50}, {140, -88}}, lineColor = {0, 0, 0}, textString = "%name"), Rectangle(extent = {{-12, 40}, {12, -40}}, lineColor = {0, 0, 0}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid)}),
            Documentation(info = "<html>
<p>
Basic definition of a coordinate system that is fixed to a mechanical
component. In the origin of the coordinate system the cut-force
and the cut-torque is acting.
This component has a filled rectangular icon.
</p>
</html>"));
        end Frame_a;

        connector Frame_b "Coordinate system fixed to the component with one cut-force and cut-torque (non-filled rectangular icon)"
          extends Frame;
          annotation(
            defaultComponentName = "frame_b",
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}, initialScale = 0.16), graphics = {Rectangle(extent = {{-10, 10}, {10, -10}}, lineColor = {95, 95, 95}, lineThickness = 0.5), Rectangle(extent = {{-30, 100}, {30, -100}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid)}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}, initialScale = 0.16), graphics = {Text(extent = {{-140, -50}, {140, -88}}, lineColor = {0, 0, 0}, textString = "%name"), Rectangle(extent = {{-12, 40}, {12, -40}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid)}),
            Documentation(info = "<html>
<p>
Basic definition of a coordinate system that is fixed to a mechanical
component. In the origin of the coordinate system the cut-force
and the cut-torque is acting. This component has a non-filled rectangular icon.
</p>
</html>"));
        end Frame_b;

        connector Frame_resolve "Coordinate system fixed to the component used to express in which
        coordinate system a vector is resolved (non-filled rectangular icon)"
          extends Frame;
          annotation(
            defaultComponentName = "frame_resolve",
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}, initialScale = 0.16), graphics = {Rectangle(extent = {{-10, 10}, {10, -10}}, lineColor = {95, 95, 95}, pattern = LinePattern.Dot), Rectangle(extent = {{-30, 100}, {30, -100}}, lineColor = {95, 95, 95}, pattern = LinePattern.Dot, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid)}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}, initialScale = 0.16), graphics = {Text(extent = {{-140, -50}, {140, -88}}, lineColor = {0, 0, 0}, textString = "%name"), Rectangle(extent = {{-12, 40}, {12, -40}}, lineColor = {95, 95, 95}, pattern = LinePattern.Dot, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid)}),
            Documentation(info = "<html>
<p>
Basic definition of a coordinate system that is fixed to a mechanical
component. In the origin of the coordinate system the cut-force
and the cut-torque is acting. This coordinate system is used to
express in which coordinate system a vector is resolved.
A component that uses a Frame_resolve connector has to set the
cut-force and cut-torque of this frame to zero. When connecting
from a Frame_resolve connector to another frame connector,
by default the connecting line has line style \"dotted\".
This component has a non-filled rectangular icon.
</p>
</html>"));
        end Frame_resolve;

        connector FlangeWithBearing "Connector consisting of 1-dim. rotational flange and its bearing frame"
          parameter Boolean includeBearingConnector = false "= true, if bearing frame connector is present, otherwise not present";
          Modelica.Mechanics.Rotational.Interfaces.Flange_a flange "1-dim. rotational flange";
          Modelica.Mechanics.MultiBody.Interfaces.Frame bearingFrame if includeBearingConnector "3-dim. frame in which the 1-dim. shaft is mounted";
          annotation(
            defaultComponentName = "flange",
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-20, 1}, {20, -1}}, lineColor = {135, 135, 135}, lineThickness = 0.5), Rectangle(extent = {{-100, 100}, {100, -100}}, lineColor = {255, 255, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-100, 25}, {100, -25}}, lineColor = {64, 64, 64}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192, 192, 192}), Line(points = {{-80, 60}, {80, 60}}, color = {0, 0, 0}), Line(points = {{-80, -60}, {80, -60}}, color = {0, 0, 0}), Line(points = {{0, 100}, {0, 60}}, color = {0, 0, 0}), Line(points = {{0, -60}, {0, -100}}, color = {0, 0, 0}), Rectangle(extent = {{-100, 100}, {100, -100}}, lineColor = {135, 135, 135}), Rectangle(extent = {{-100, 25}, {100, -25}}, lineColor = {64, 64, 64})}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-50, -40}, {50, -40}}, color = {0, 0, 0}, thickness = 0.5), Line(points = {{-50, 40}, {50, 40}}, color = {0, 0, 0}, thickness = 0.5), Text(extent = {{-158, -66}, {158, -124}}, lineColor = {0, 0, 0}, lineThickness = 0.5, textString = "%name"), Rectangle(extent = {{-60, 60}, {60, -60}}, lineColor = {255, 255, 255}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-60, 15}, {60, -15}}, lineColor = {64, 64, 64}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192, 192, 192}), Line(points = {{0, 60}, {0, 40}}, color = {0, 0, 0}), Line(points = {{0, -40}, {0, -60}}, color = {0, 0, 0}), Line(points = {{-50, 40}, {50, 40}}, color = {0, 0, 0}), Line(points = {{-50, -40}, {50, -40}}, color = {0, 0, 0}), Rectangle(extent = {{-60, 60}, {60, -60}}, lineColor = {135, 135, 135}), Rectangle(extent = {{-60, 15}, {60, -15}}, lineColor = {64, 64, 64})}),
            Documentation(info = "<html>
<p>
This hierarchical connector models a 1-dim. rotational flange
connector and its optional bearing defined by a 3-dim. frame connector.
If a connection to the subconnectors should be clearly visible,
connect first an  instance of
<a href=\"modelica://Modelica.Mechanics.MultiBody.Interfaces.FlangeWithBearingAdaptor\">FlangeWithBearingAdaptor</a>
to the FlangeWithBearing connector.
</p>
</html>"));
        end FlangeWithBearing;

        model FlangeWithBearingAdaptor "Adaptor to allow direct connections to the sub-connectors of FlangeWithBearing"
          parameter Boolean includeBearingConnector = false "= true, if bearing frame connector is present, otherwise not present";
          Modelica.Mechanics.MultiBody.Interfaces.FlangeWithBearing flangeAndFrame(includeBearingConnector = includeBearingConnector) "Compound connector consisting of 1-dim. rotational flange and 3-dim. frame mounting" annotation(
            Placement(transformation(extent = {{-130, -30}, {-70, 30}}, rotation = 0)));
          Modelica.Mechanics.Rotational.Interfaces.Flange_b flange "1-dim. rotational flange" annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0)));
          Frame_a frame if includeBearingConnector "3-dim. frame in which the 1-dim. shaft is mounted" annotation(
            Placement(transformation(origin = {0, -100}, extent = {{-16, -16}, {16, 16}}, rotation = 90)));
        equation
          connect(flange, flangeAndFrame.flange) annotation(
            Line(points = {{0, 0}, {-100, 0}}, color = {0, 0, 0}));
          connect(frame, flangeAndFrame.bearingFrame) annotation(
            Line(points = {{0, -100}, {0, -40}, {-100, -40}, {-100, 0}}, color = {0, 0, 0}, thickness = 0.5));
          annotation(
            defaultComponentName = "adaptor",
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, 30}, {20, -100}}, lineColor = {255, 255, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{-100, -10}, {-100, -40}, {0, -40}, {0, -100}}, color = {0, 0, 0}, thickness = 0.5), Line(points = {{-90, 0}, {0, 0}}, color = {0, 0, 0}), Text(extent = {{-216, 88}, {86, 36}}, lineColor = {0, 0, 255}, textString = "%name")}),
            Documentation(info = "<html>
<p>
Adaptor object to make a more visible connection to the flange and frame
subconnectors of a
<a href=\"modelica://Modelica.Mechanics.MultiBody.Interfaces.FlangeWithBearing\">FlangeWithBearing</a>
connector.
</p>
</html>"));
        end FlangeWithBearingAdaptor;

        partial model PartialTwoFrames "Base model for components providing two frame connectors + outer world + assert to guarantee that the component is connected"
          Interfaces.Frame_a frame_a "Coordinate system fixed to the component with one cut-force and cut-torque" annotation(
            Placement(transformation(extent = {{-116, -16}, {-84, 16}}, rotation = 0)));
          Interfaces.Frame_b frame_b "Coordinate system fixed to the component with one cut-force and cut-torque" annotation(
            Placement(transformation(extent = {{84, -16}, {116, 16}}, rotation = 0)));
        protected
          outer Modelica.Mechanics.MultiBody.World world;
        equation
          assert(cardinality(frame_a) > 0, "Connector frame_a of component is not connected");
          assert(cardinality(frame_b) > 0, "Connector frame_b of component is not connected");
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-136, -25}, {-100, -50}}, lineColor = {128, 128, 128}, textString = "a"), Text(extent = {{100, -25}, {136, -50}}, lineColor = {128, 128, 128}, textString = "b")}),
            Documentation(info = "<HTML>
<p>
This partial model provides two frame connectors, access to the world
object and an assert to check that both frame connectors are connected.
Therefore, inherit from this partial model if the two frame connectors are
needed and if the two frame connectors should be connected for a correct model.
</p>
</HTML>"));
        end PartialTwoFrames;

        partial model PartialTwoFramesDoubleSize "Base model for components providing two frame connectors + outer world + assert to guarantee that the component is connected (default icon size is factor 2 larger as usual)"
          Interfaces.Frame_a frame_a "Coordinate system fixed to the component with one cut-force and cut-torque" annotation(
            Placement(transformation(extent = {{-108, -8}, {-92, 8}})));
          Interfaces.Frame_b frame_b "Coordinate system fixed to the component with one cut-force and cut-torque" annotation(
            Placement(transformation(extent = {{92, -8}, {108, 8}})));
        protected
          outer Modelica.Mechanics.MultiBody.World world;
        equation
          assert(cardinality(frame_a) > 0, "Connector frame_a of component is not connected");
          assert(cardinality(frame_b) > 0, "Connector frame_b of component is not connected");
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}, initialScale = 0.2), graphics = {Text(extent = {{-117, -13}, {-106, -23}}, lineColor = {128, 128, 128}, textString = "a"), Text(extent = {{110, -15}, {122, -25}}, lineColor = {128, 128, 128}, textString = "b")}),
            Documentation(info = "<HTML>
<p>
This partial model provides two frame connectors, access to the world
object and an assert to check that both frame connectors are connected.
Therefore, inherit from this partial model if the two frame connectors are
needed and if the two frame connectors should be connected for a correct model.
</p>
<p>
When dragging \"PartialTwoFrames\", the default size is a factor of two
larger as usual. This partial model is used by the Joint.Assemblies
joint aggregation models.
</p>
</HTML>"));
        end PartialTwoFramesDoubleSize;

        partial model PartialOneFrame_a "Base model for components providing one frame_a connector + outer world + assert to guarantee that the component is connected"
          Interfaces.Frame_a frame_a "Coordinate system fixed to the component with one cut-force and cut-torque" annotation(
            Placement(transformation(extent = {{-116, -16}, {-84, 16}}, rotation = 0)));
        protected
          outer Modelica.Mechanics.MultiBody.World world;
        equation
          assert(cardinality(frame_a) > 0, "Connector frame_a of component is not connected");
          annotation(
            Documentation(info = "<html>
<p>
This partial model provides one frame_a connector, access to the world
object and an assert to check that the frame_a connector is connected.
Therefore, inherit from this partial model if the frame_a connector is
needed and if this connector should be connected for a correct model.
</p>
</html>"));
        end PartialOneFrame_a;

        partial model PartialOneFrame_b "Base model for components providing one frame_b connector + outer world + assert to guarantee that the component is connected"
          Interfaces.Frame_b frame_b "Coordinate system fixed to the component with one cut-force and cut-torque" annotation(
            Placement(transformation(extent = {{84, -16}, {116, 16}}, rotation = 0)));
        protected
          outer Modelica.Mechanics.MultiBody.World world;
        equation
          assert(cardinality(frame_b) > 0, "Connector frame_b of component is not connected");
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{94, -20}, {130, -45}}, lineColor = {128, 128, 128}, textString = "b")}),
            Documentation(info = "<HTML>
<p>
This partial model provides one frame_b connector, access to the world
object and an assert to check that the frame_b connector is connected.
Therefore, inherit from this partial model if the frame_b connector is
needed and if this connector should be connected for a correct model.
</p>
</HTML>"));
        end PartialOneFrame_b;

        partial model PartialElementaryJoint "Base model for elementary joints (has two frames + outer world + assert to guarantee that the joint is connected)"
          Interfaces.Frame_a frame_a "Coordinate system fixed to the joint with one cut-force and cut-torque" annotation(
            Placement(transformation(extent = {{-116, -16}, {-84, 16}}, rotation = 0)));
          Interfaces.Frame_b frame_b "Coordinate system fixed to the joint with one cut-force and cut-torque" annotation(
            Placement(transformation(extent = {{84, -16}, {116, 16}}, rotation = 0)));
        protected
          outer Modelica.Mechanics.MultiBody.World world;
        equation
          Connections.branch(frame_a.R, frame_b.R);
          assert(cardinality(frame_a) > 0, "Connector frame_a of joint object is not connected");
          assert(cardinality(frame_b) > 0, "Connector frame_b of joint object is not connected");
          annotation(
            Documentation(info = "<HTML>
<p>
All <b>elementary joints</b> should inherit from this base model, i.e.,
joints that are directly defined by equations, provided they compute
either the rotation object of frame_b from the rotation object of frame_a
and from relative quantities (or vice versa), or there is a constraint
equation between the rotation objects of the two frames.
In other cases, a joint object should inherit from
<b>Interfaces.PartialTwoFrames</b> (e.g., joint Spherical, because there
is no constraint between the rotation objects of frame_a and frame_b
or joint Cylindrical because it is not an elementary joint).
</p>
<p>
This partial model provides two frame connectors, a \"Connections.branch\"
between frame_a and frame_b, access to the world
object and an assert to check that both frame connectors are connected.
</p>
</html>"));
        end PartialElementaryJoint;

        partial model PartialForce "Base model for force elements (provide frame_b.f and frame_b.t in subclasses)"
          Interfaces.Frame_a frame_a "Coordinate system fixed to the joint with one cut-force and cut-torque" annotation(
            Placement(transformation(extent = {{-116, -16}, {-84, 16}}, rotation = 0)));
          Interfaces.Frame_b frame_b "Coordinate system fixed to the joint with one cut-force and cut-torque" annotation(
            Placement(transformation(extent = {{84, -16}, {116, 16}}, rotation = 0)));
          SI.Position r_rel_b[3] "Position vector from origin of frame_a to origin of frame_b, resolved in frame_b";
        protected
          outer Modelica.Mechanics.MultiBody.World world;
        equation
          assert(cardinality(frame_a) > 0, "Connector frame_a of force object is not connected");
          assert(cardinality(frame_b) > 0, "Connector frame_b of force object is not connected");
/* Determine relative position vector
     between frame_a and frame_b
  */
          r_rel_b = Frames.resolve2(frame_b.R, frame_b.r_0 - frame_a.r_0);
/* Force and torque balance between frame_a and frame_b */
          zeros(3) = frame_a.f + Frames.resolveRelative(frame_b.f, frame_b.R, frame_a.R);
          zeros(3) = frame_a.t + Frames.resolveRelative(frame_b.t + cross(r_rel_b, frame_b.f), frame_b.R, frame_a.R);
          annotation(
            Documentation(info = "<HTML>
<p>
All <b>3-dimensional force</b> and <b>torque elements</b>
should be based on this superclass.
This model defines frame_a and frame_b, computes the relative
translation and rotation between the two frames and calculates
the cut-force and cut-torque at frame_a by a force and torque
balance from the cut-force and cut-torque at frame_b.
As a result, in a subclass, only the relationship between
the cut-force and cut-torque at frame_b has to be defined as
a function of the following relative quantities:
</p>
<pre>
  r_rel_b[3]: Position vector from origin of frame_a to origin
              of frame_b, resolved in frame_b
  R_rel     : Relative orientation object to rotate from frame_a to frame_b
</pre>
<p>
Assume that force f = {100,0,0} should be applied on the body
to which this force element is attached at frame_b, then
the definition should be:
</p>
<pre>
   <b>model</b> Constant_x_Force
      extends Modelica.Mechanics.MultiBody.Interfaces.PartialForce;
   <b>equation</b>
      frame_b.f = {-100, 0, 0};
      frame_b.t = zeros(3);
   <b>end</b> Constant_x_Force;
</pre>
<p>
Note, that frame_b.f and frame_b.t are flow variables and therefore
the negative value of frame_b.f and frame_b.t is acting at the part
to which this force element is connected.
</p>
</HTML>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-136, 42}, {-100, 17}}, lineColor = {128, 128, 128}, textString = "a"), Text(extent = {{102, 44}, {138, 19}}, lineColor = {128, 128, 128}, textString = "b")}));
        end PartialForce;

        partial model PartialLineForce "Base model for line force elements"
          parameter SI.Position s_small = 1.E-6 "Prevent zero-division if relative distance s=0" annotation(
            Dialog(tab = "Advanced"));
          parameter Boolean fixedRotationAtFrame_a = false "=true, if rotation frame_a.R is fixed (to directly connect line forces)" annotation(
            Evaluate = true,
            choices(checkBox = true),
            Dialog(tab = "Advanced", group = "If enabled, can give wrong results, see MultiBody.UsersGuide.Tutorial.ConnectionOfLineForces"));
          parameter Boolean fixedRotationAtFrame_b = false "=true, if rotation frame_b.R is fixed (to directly connect line forces)" annotation(
            Evaluate = true,
            choices(checkBox = true),
            Dialog(tab = "Advanced", group = "If enabled, can give wrong results, see MultiBody.UsersGuide.Tutorial.ConnectionOfLineForces"));
          Interfaces.Frame_a frame_a "Coordinate system fixed to the force element with one cut-force and cut-torque" annotation(
            Placement(transformation(extent = {{-116, -16}, {-84, 16}}, rotation = 0)));
          Interfaces.Frame_b frame_b "Coordinate system fixed to the force element with one cut-force and cut-torque" annotation(
            Placement(transformation(extent = {{84, -16}, {116, 16}}, rotation = 0)));
          SI.Force f "Line force acting on frame_a and on frame_b (positive, if acting on frame_b and directed from frame_a to frame_b)";
          SI.Position s "(Guarded) distance between the origin of frame_a and the origin of frame_b (>= s_small))";
          Real e_a[3](each final unit = "1") "Unit vector on the line connecting the origin of frame_a with the origin of frame_b resolved in frame_a (directed from frame_a to frame_b)";
          Modelica.SIunits.Position r_rel_a[3] "Position vector from origin of frame_a to origin of frame_b, resolved in frame_a";
        protected
          outer Modelica.Mechanics.MultiBody.World world;
        equation
          assert(cardinality(frame_a) > 0, "Connector frame_a of line force object is not connected");
          assert(cardinality(frame_b) > 0, "Connector frame_b of line force object is not connected");
// Determine distance s and n_a
          r_rel_a = Frames.resolve2(frame_a.R, frame_b.r_0 - frame_a.r_0);
          s = noEvent(max(Modelica.Math.Vectors.length(r_rel_a), s_small));
          e_a = r_rel_a / s;
/* Determine forces and torques at frame_a and frame_b */
          frame_a.f = -e_a * f;
          frame_b.f = -Frames.resolve2(Frames.relativeRotation(frame_a.R, frame_b.R), frame_a.f);
// Additional equations, if direct connections of line forces
          if fixedRotationAtFrame_a then
            Connections.root(frame_a.R);
            frame_a.R = Frames.nullRotation();
          else
            frame_a.t = zeros(3);
          end if;
          if fixedRotationAtFrame_b then
            Connections.root(frame_b.R);
            frame_b.R = Frames.nullRotation();
          else
            frame_b.t = zeros(3);
          end if;
          annotation(
            Documentation(info = "<HTML>
<p>
All <b>line force</b> elements should be based on this base model.
This model defines frame_a and frame_b, computes the relative
distance <b>s</b> and provides the force and torque
balance of the cut-forces and cut-torques at frame_a and
frame_b, respectively. In sub-models, only the line force <b>f</b>,
acting at frame_b on the line from frame_a to frame_b, as a function
of the relative distance <b>s</b> and its derivative <b>der</b>(<b>s</b>)
has to be defined. Example:
</p>
<pre>
   <b>model</b> Spring
      <b>parameter</b> Real c \"spring constant\",
      <b>parameter</b> Real s_unstretched \"unstretched spring length\";
      <b>extends</b> Modelica.Mechanics.MultiBody.Interfaces.PartialLineForce;
   <b>equation</b>
      f = c*(s-s_unstretched);
   <b>end</b> Spring;
</pre>
</HTML>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-136, -44}, {-100, -19}}, lineColor = {128, 128, 128}, textString = "a"), Text(extent = {{100, -42}, {136, -17}}, lineColor = {128, 128, 128}, textString = "b"), Ellipse(visible = fixedRotationAtFrame_a, extent = {{-70, 30}, {-130, -30}}, lineColor = {255, 0, 0}), Text(visible = fixedRotationAtFrame_a, extent = {{-62, 50}, {-140, 30}}, lineColor = {255, 0, 0}, textString = "R=0"), Ellipse(visible = fixedRotationAtFrame_b, extent = {{70, 30}, {130, -30}}, lineColor = {255, 0, 0}), Text(visible = fixedRotationAtFrame_b, extent = {{62, 50}, {140, 30}}, lineColor = {255, 0, 0}, textString = "R=0")}));
        end PartialLineForce;

        partial model PartialAbsoluteSensor "Base model to measure an absolute frame variable"
          extends Modelica.Icons.RotationalSensor;
          parameter Integer n_out = 1 "Number of output signals";
          Interfaces.Frame_a frame_a "Coordinate system from which absolute quantities are provided as output signals" annotation(
            Placement(transformation(extent = {{-116, -16}, {-84, 16}}, rotation = 0)));
          Modelica.Blocks.Interfaces.RealOutput y[n_out] "Measured data as signal vector" annotation(
            Placement(transformation(extent = {{100, -10}, {120, 10}}, rotation = 0)));
        protected
          outer Modelica.Mechanics.MultiBody.World world;
        equation
          assert(cardinality(frame_a) > 0, "Connector frame_a of absolute sensor object is not connected");
          annotation(
            Documentation(info = "<html>
<p>
This is the base class of a 3-dim. mechanics component with one frame and one
output port in order to measure an absolute quantity in the frame connector
and to provide the measured signal as output for further processing
with the blocks of package Modelica.Blocks.
</p>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-70, 0}, {-100, 0}}, color = {0, 0, 0}), Line(points = {{70, 0}, {100, 0}}, color = {0, 0, 255}), Text(extent = {{-132, -125}, {131, -79}}, textString = "%name", lineColor = {0, 0, 255})}));
        end PartialAbsoluteSensor;

        partial model PartialRelativeSensor "Base model to measure a relative variable between two frames"
          extends Modelica.Icons.RotationalSensor;
          parameter Integer n_out = 1 "Number of output signals";
          Interfaces.Frame_a frame_a "Coordinate system a" annotation(
            Placement(transformation(extent = {{-116, -16}, {-84, 16}}, rotation = 0)));
          Interfaces.Frame_b frame_b "Coordinate system b" annotation(
            Placement(transformation(extent = {{84, -16}, {116, 16}}, rotation = 0)));
          Modelica.Blocks.Interfaces.RealOutput y[n_out] "Measured data as signal vector" annotation(
            Placement(transformation(origin = {0, -110}, extent = {{10, -10}, {-10, 10}}, rotation = 90)));
        protected
          outer Modelica.Mechanics.MultiBody.World world;
        equation
          assert(cardinality(frame_a) > 0, "Connector frame_a of relative sensor object is not connected");
          assert(cardinality(frame_b) > 0, "Connector frame_b of relative sensor object is not connected");
          annotation(
            Documentation(info = "<html>
<p>
This is a base class for 3-dim. mechanical components with two frames
and one output port in order to measure relative quantities
between the two frames or the cut-forces/torques in the frame and
to provide the measured signals as output for further processing
with the blocks of package Modelica.Blocks.
</p>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-70, 0}, {-101, 0}}, color = {0, 0, 0}), Line(points = {{70, 0}, {100, 0}}, color = {0, 0, 0}), Line(points = {{0, -100}, {0, -70}}, color = {0, 0, 127}), Text(extent = {{-132, 76}, {129, 124}}, textString = "%name", lineColor = {0, 0, 255}), Text(extent = {{-118, 52}, {-82, 27}}, lineColor = {128, 128, 128}, textString = "a"), Text(extent = {{85, 53}, {121, 28}}, lineColor = {128, 128, 128}, textString = "b")}));
        end PartialRelativeSensor;

        partial model PartialVisualizer "Base model for visualizers (has a frame_a on the left side + outer world + assert to guarantee that the component is connected)"
          Interfaces.Frame_a frame_a "Coordinate system in which visualization data is resolved" annotation(
            Placement(transformation(extent = {{-116, -16}, {-84, 16}}, rotation = 0)));
        protected
          outer Modelica.Mechanics.MultiBody.World world;
        equation
          assert(cardinality(frame_a) > 0, "Connector frame_a of visualizer object is not connected");
          annotation(
            Documentation(info = "<html>
<p>
This partial model provides one frame_a connector, access to the world
object and an assert to check that the frame_a connector is connected.
It is used by inheritance from all visualizer objects.
</p>
</html>"));
        end PartialVisualizer;

        model ZeroPosition "Set absolute position vector of frame_resolve to a zero vector and the orientation object to a null rotation"
          extends Modelica.Blocks.Icons.Block;
          Interfaces.Frame_resolve frame_resolve annotation(
            Placement(transformation(extent = {{-116, -16}, {-84, 16}})));
        equation
          Connections.root(frame_resolve.R);
          frame_resolve.R = Modelica.Mechanics.MultiBody.Frames.nullRotation();
          frame_resolve.r_0 = zeros(3);
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-74, 24}, {80, -20}}, lineColor = {0, 0, 0}, textString = "r = 0")}));
        end ZeroPosition;

        partial function partialGravityAcceleration
          extends Modelica.Icons.Function;
          input Modelica.SIunits.Position r[3] "Position vector from world frame to actual point, resolved in world frame";
          output Modelica.SIunits.Acceleration gravity[3] "Gravity acceleration at position r, resolved in world frame";
          annotation(
            Documentation(info = "<html>
<p>
This partial function defines the interface to the gravity function
used in the World object. All gravity field functions must inherit
from this function. The input to the function is the absolute position
vector of a point in the gravity field, whereas the output is the
gravity acceleration at this point, resolved in the world frame.
</p>
</html>"));
        end partialGravityAcceleration;

        partial function partialSurfaceCharacteristic
          extends Modelica.Icons.Function;
          input Integer nu "Number of points in u-Dimension";
          input Integer nv "Number of points in v-Dimension";
          input Boolean multiColoredSurface = false "= true: Color is defined for each surface point";
          output Modelica.SIunits.Position X[nu, nv] "[nu,nv] positions of points in x-Direction resolved in surface frame";
          output Modelica.SIunits.Position Y[nu, nv] "[nu,nv] positions of points in y-Direction resolved in surface frame";
          output Modelica.SIunits.Position Z[nu, nv] "[nu,nv] positions of points in z-Direction resolved in surface frame";
          output Real C[if multiColoredSurface then nu else 0, if multiColoredSurface then nv else 0, 3] "[nu,nv,3] Color array, defining the color for each surface point";
        end partialSurfaceCharacteristic;

        partial function partialColorMap "Interface for a function returning a color map"
          extends Modelica.Icons.Function;
          input Integer n_colors = 64 "Number of colors in the color map";
          output Real colorMap[n_colors, 3] "Color map to map a scalar to a color";
          annotation(
            Documentation(info = "<html>
<p>
This partial function defines the interface of a function that returns
a color map. Predefined color map functions are defined in package
<a href=\"modelica://Modelica.Mechanics.MultiBody.Visualizers.Colors.ColorMaps\">Modelica.Mechanics.MultiBody.Visualizers.Colors.ColorMaps</a>.
</p>
</html>"));
        end partialColorMap;
        annotation(
          Documentation(info = "<html>
<p>
This package contains connectors and partial models (i.e., models
that are only used to build other models) of the MultiBody library.
</p>
</html>"));
      end Interfaces;

      package Joints "Components that constrain the motion between two frames"
        extends Modelica.Icons.Package;

        model Prismatic "Prismatic joint (1 translational degree-of-freedom, 2 potential states, optional axis flange)"
          extends Modelica.Mechanics.MultiBody.Interfaces.PartialElementaryJoint;
          Modelica.Mechanics.Translational.Interfaces.Flange_a axis if useAxisFlange "1-dim. translational flange that drives the joint" annotation(
            Placement(transformation(extent = {{90, 50}, {70, 70}}, rotation = 0)));
          Modelica.Mechanics.Translational.Interfaces.Flange_b support if useAxisFlange "1-dim. translational flange of the drive drive support (assumed to be fixed in the world frame, NOT in the joint)" annotation(
            Placement(transformation(extent = {{-30, 50}, {-50, 70}}, rotation = 0)));
          parameter Boolean useAxisFlange = false "= true, if axis flange is enabled" annotation(
            Evaluate = true,
            HideResult = true,
            choices(checkBox = true));
          parameter Boolean animation = true "= true, if animation shall be enabled";
          parameter Modelica.Mechanics.MultiBody.Types.Axis n = {1, 0, 0} "Axis of translation resolved in frame_a (= same as in frame_b)" annotation(
            Evaluate = true);
          constant SI.Position s_offset = 0 "Relative distance offset (distance between frame_a and frame_b = s_offset + s)" annotation(
            Evaluate = false);
          parameter Types.Axis boxWidthDirection = {0, 1, 0} "Vector in width direction of box, resolved in frame_a" annotation(
            Evaluate = true,
            Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter SI.Distance boxWidth = world.defaultJointWidth "Width of prismatic joint box" annotation(
            Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter SI.Distance boxHeight = boxWidth "Height of prismatic joint box" annotation(
            Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          input Types.Color boxColor = Modelica.Mechanics.MultiBody.Types.Defaults.JointColor "Color of prismatic joint box" annotation(
            Dialog(colorSelector = true, tab = "Animation", group = "if animation = true", enable = animation));
          input Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(
            Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter StateSelect stateSelect = StateSelect.prefer "Priority to use distance s and v=der(s) as states" annotation(
            Dialog(tab = "Advanced"));
          final parameter Real e[3](each final unit = "1") = Modelica.Math.Vectors.normalizeWithAssert(n) "Unit vector in direction of prismatic axis n";
          SI.Position s(start = 0, final stateSelect = stateSelect) "Relative distance between frame_a and frame_b" annotation(
            unassignedMessage = "
The relative distance s of a prismatic joint cannot be determined.
Possible reasons:
- A non-zero mass might be missing on either side of the parts
  connected to the prismatic joint.
- Too many StateSelect.always are defined and the model
  has less degrees of freedom as specified with this setting
  (remove all StateSelect.always settings).
            ");
          SI.Velocity v(start = 0, final stateSelect = stateSelect) "First derivative of s (relative velocity)";
          SI.Acceleration a(start = 0) "Second derivative of s (relative acceleration)";
          SI.Force f "Actuation force in direction of joint axis";
        protected
          Visualizers.Advanced.Shape box(shapeType = "box", color = boxColor, specularCoefficient = specularCoefficient, length = if noEvent(abs(s + s_offset) > 1.e-6) then s + s_offset else 1.e-6, width = boxWidth, height = boxHeight, lengthDirection = e, widthDirection = boxWidthDirection, r = frame_a.r_0, R = frame_a.R) if world.enableAnimation and animation;
          Translational.Components.Fixed fixed annotation(
            Placement(transformation(extent = {{-50, 30}, {-30, 50}})));
          Translational.Interfaces.InternalSupport internalAxis(f = f) annotation(
            Placement(transformation(extent = {{70, 50}, {90, 30}})));
          Translational.Sources.ConstantForce constantForce(f_constant = 0) if not useAxisFlange annotation(
            Placement(transformation(extent = {{40, 30}, {60, 50}})));
        equation
          v = der(s);
          a = der(v);
// relationships between kinematic quantities of frame_a and of frame_b
          frame_b.r_0 = frame_a.r_0 + Frames.resolve1(frame_a.R, e * (s_offset + s));
          frame_b.R = frame_a.R;
// Force and torque balance
          zeros(3) = frame_a.f + frame_b.f;
          zeros(3) = frame_a.t + frame_b.t + cross(e * (s_offset + s), frame_b.f);
// d'Alemberts principle
          f = -e * frame_b.f;
// Connection to internal connectors
          s = internalAxis.s;
          connect(fixed.flange, support) annotation(
            Line(points = {{-40, 40}, {-40, 60}}, color = {0, 127, 0}, smooth = Smooth.None));
          connect(internalAxis.flange, axis) annotation(
            Line(points = {{80, 40}, {80, 60}}, color = {0, 127, 0}, smooth = Smooth.None));
          connect(constantForce.flange, internalAxis.flange) annotation(
            Line(points = {{60, 40}, {80, 40}}, color = {0, 127, 0}, smooth = Smooth.None));
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, -50}, {-30, 41}}, pattern = LinePattern.None, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Rectangle(extent = {{-100, 40}, {-30, 50}}, pattern = LinePattern.None, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Rectangle(extent = {{-30, -30}, {100, 20}}, pattern = LinePattern.None, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Rectangle(extent = {{-30, 20}, {100, 30}}, pattern = LinePattern.None, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Line(points = {{-30, -50}, {-30, 50}}, color = {0, 0, 0}), Line(points = {{100, -30}, {100, 21}}, color = {0, 0, 0}), Text(extent = {{60, 12}, {96, -13}}, lineColor = {128, 128, 128}, textString = "b"), Text(extent = {{-95, 13}, {-60, -9}}, lineColor = {128, 128, 128}, textString = "a"), Text(visible = useAxisFlange, extent = {{-150, -135}, {150, -95}}, textString = "%name", lineColor = {0, 0, 255}), Text(extent = {{-150, -90}, {150, -60}}, lineColor = {0, 0, 0}, textString = "n=%n"), Rectangle(visible = useAxisFlange, extent = {{90, 30}, {100, 70}}, pattern = LinePattern.None, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Text(visible = not useAxisFlange, extent = {{-150, 60}, {150, 100}}, textString = "%name", lineColor = {0, 0, 255})}),
            Documentation(info = "<HTML>
<p>
Joint where frame_b is translated along axis n which is fixed in frame_a.
The two frames coincide when the relative distance \"s = 0\".
</p>

<p>
Optionally, two additional 1-dimensional mechanical flanges
(flange \"axis\" represents the driving flange and
flange \"support\" represents the bearing) can be enabled via
parameter <b>useAxisFlange</b>. The enabled axis flange can be
driven with elements of the
<a href=\"modelica://Modelica.Mechanics.Translational\">Modelica.Mechanics.Translational</a>
library.

</p>

<p>
In the \"Advanced\" menu it can be defined via parameter <b>stateSelect</b>
that the relative distance \"s\" and its derivative shall be definitely
used as states by setting stateSelect=StateSelect.always.
Default is StateSelect.prefer to use the relative distance and its
derivative as preferred states. The states are usually selected automatically.
In certain situations, especially when closed kinematic loops are present,
it might be slightly more efficient, when using the StateSelect.always setting.
</p>

<p>
In the following figure the animation of a prismatic
joint is shown. The light blue coordinate system is
frame_a and the dark blue coordinate system is
frame_b of the joint. The black arrow is parameter
vector \"n\" defining the translation axis
(here: n = {1,1,0}).
</p>

<p>
<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Joints/Prismatic.png\">
</p>

</html>"));
        end Prismatic;

        model Revolute "Revolute joint (1 rotational degree-of-freedom, 2 potential states, optional axis flange)"
          Modelica.Mechanics.Rotational.Interfaces.Flange_a axis if useAxisFlange "1-dim. rotational flange that drives the joint" annotation(
            Placement(transformation(extent = {{10, 90}, {-10, 110}}, rotation = 0)));
          Modelica.Mechanics.Rotational.Interfaces.Flange_b support if useAxisFlange "1-dim. rotational flange of the drive support (assumed to be fixed in the world frame, NOT in the joint)" annotation(
            Placement(transformation(extent = {{-70, 90}, {-50, 110}}, rotation = 0)));
          Modelica.Mechanics.MultiBody.Interfaces.Frame_a frame_a "Coordinate system fixed to the joint with one cut-force and cut-torque" annotation(
            Placement(transformation(extent = {{-116, -16}, {-84, 16}}, rotation = 0)));
          Modelica.Mechanics.MultiBody.Interfaces.Frame_b frame_b "Coordinate system fixed to the joint with one cut-force and cut-torque" annotation(
            Placement(transformation(extent = {{84, -16}, {116, 16}}, rotation = 0)));
          parameter Boolean useAxisFlange = false "= true, if axis flange is enabled" annotation(
            Evaluate = true,
            HideResult = true,
            choices(checkBox = true));
          parameter Boolean animation = true "= true, if animation shall be enabled (show axis as cylinder)";
          parameter Modelica.Mechanics.MultiBody.Types.Axis n = {0, 0, 1} "Axis of rotation resolved in frame_a (= same as in frame_b)" annotation(
            Evaluate = true);
          constant SI.Angle phi_offset = 0 "Relative angle offset (angle = phi_offset + phi)";
          parameter SI.Distance cylinderLength = world.defaultJointLength "Length of cylinder representing the joint axis" annotation(
            Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter SI.Distance cylinderDiameter = world.defaultJointWidth "Diameter of cylinder representing the joint axis" annotation(
            Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          input Modelica.Mechanics.MultiBody.Types.Color cylinderColor = Modelica.Mechanics.MultiBody.Types.Defaults.JointColor "Color of cylinder representing the joint axis" annotation(
            Dialog(colorSelector = true, tab = "Animation", group = "if animation = true", enable = animation));
          input Modelica.Mechanics.MultiBody.Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(
            Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter StateSelect stateSelect = StateSelect.prefer "Priority to use joint angle phi and w=der(phi) as states" annotation(
            Dialog(tab = "Advanced"));
          SI.Angle phi(start = 0, final stateSelect = stateSelect) "Relative rotation angle from frame_a to frame_b" annotation(
            unassignedMessage = "
The rotation angle phi of a revolute joint cannot be determined.
Possible reasons:
- A non-zero mass might be missing on either side of the parts
  connected to the revolute joint.
- Too many StateSelect.always are defined and the model
  has less degrees of freedom as specified with this setting
  (remove all StateSelect.always settings).
            ");
          SI.AngularVelocity w(start = 0, stateSelect = stateSelect) "First derivative of angle phi (relative angular velocity)";
          SI.AngularAcceleration a(start = 0) "Second derivative of angle phi (relative angular acceleration)";
          SI.Torque tau "Driving torque in direction of axis of rotation";
          SI.Angle angle "= phi_offset + phi";
        protected
          outer Modelica.Mechanics.MultiBody.World world;
          parameter Real e[3](each final unit = "1") = Modelica.Math.Vectors.normalizeWithAssert(n) "Unit vector in direction of rotation axis, resolved in frame_a (= same as in frame_b)";
          Frames.Orientation R_rel "Relative orientation object from frame_a to frame_b or from frame_b to frame_a";
          Visualizers.Advanced.Shape cylinder(shapeType = "cylinder", color = cylinderColor, specularCoefficient = specularCoefficient, length = cylinderLength, width = cylinderDiameter, height = cylinderDiameter, lengthDirection = e, widthDirection = {0, 1, 0}, r_shape = -e * (cylinderLength / 2), r = frame_a.r_0, R = frame_a.R) if world.enableAnimation and animation;
        protected
          Modelica.Mechanics.Rotational.Components.Fixed fixed "support flange is fixed to ground" annotation(
            Placement(transformation(extent = {{-70, 70}, {-50, 90}})));
          Rotational.Interfaces.InternalSupport internalAxis(tau = tau) annotation(
            Placement(transformation(extent = {{-10, 90}, {10, 70}})));
          Rotational.Sources.ConstantTorque constantTorque(tau_constant = 0) if not useAxisFlange annotation(
            Placement(transformation(extent = {{40, 70}, {20, 90}})));
        equation
          Connections.branch(frame_a.R, frame_b.R);
          assert(cardinality(frame_a) > 0, "Connector frame_a of revolute joint is not connected");
          assert(cardinality(frame_b) > 0, "Connector frame_b of revolute joint is not connected");
          angle = phi_offset + phi;
          w = der(phi);
          a = der(w);
// relationships between quantities of frame_a and of frame_b
          frame_b.r_0 = frame_a.r_0;
          if Connections.rooted(frame_a.R) then
            R_rel = Frames.planarRotation(e, phi_offset + phi, w);
            frame_b.R = Frames.absoluteRotation(frame_a.R, R_rel);
            frame_a.f = -Frames.resolve1(R_rel, frame_b.f);
            frame_a.t = -Frames.resolve1(R_rel, frame_b.t);
          else
            R_rel = Frames.planarRotation(-e, phi_offset + phi, w);
            frame_a.R = Frames.absoluteRotation(frame_b.R, R_rel);
            frame_b.f = -Frames.resolve1(R_rel, frame_a.f);
            frame_b.t = -Frames.resolve1(R_rel, frame_a.t);
          end if;
// d'Alemberts principle
          tau = -frame_b.t * e;
// Connection to internal connectors
          phi = internalAxis.phi;
          connect(fixed.flange, support) annotation(
            Line(points = {{-60, 80}, {-60, 100}}, color = {0, 0, 0}, smooth = Smooth.None));
          connect(internalAxis.flange, axis) annotation(
            Line(points = {{0, 80}, {0, 100}}, color = {0, 0, 0}, smooth = Smooth.None));
          connect(constantTorque.flange, internalAxis.flange) annotation(
            Line(points = {{20, 80}, {0, 80}}, color = {0, 0, 0}, smooth = Smooth.None));
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, -60}, {-30, 60}}, lineColor = {64, 64, 64}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {255, 255, 255}, radius = 10), Rectangle(extent = {{30, -60}, {100, 60}}, lineColor = {64, 64, 64}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {255, 255, 255}, radius = 10), Rectangle(extent = {{-100, 60}, {-30, -60}}, lineColor = {64, 64, 64}, radius = 10), Rectangle(extent = {{30, 60}, {100, -60}}, lineColor = {64, 64, 64}, radius = 10), Text(extent = {{-90, 14}, {-54, -11}}, lineColor = {128, 128, 128}, textString = "a"), Text(extent = {{51, 11}, {87, -14}}, lineColor = {128, 128, 128}, textString = "b"), Line(visible = useAxisFlange, points = {{-20, 80}, {-20, 60}}, color = {0, 0, 0}), Line(visible = useAxisFlange, points = {{20, 80}, {20, 60}}, color = {0, 0, 0}), Rectangle(visible = useAxisFlange, extent = {{-10, 100}, {10, 50}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.VerticalCylinder, fillColor = {192, 192, 192}), Polygon(visible = useAxisFlange, points = {{-10, 30}, {10, 30}, {30, 50}, {-30, 50}, {-10, 30}}, lineColor = {64, 64, 64}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-30, 11}, {30, -10}}, lineColor = {64, 64, 64}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Polygon(visible = useAxisFlange, points = {{10, 30}, {30, 50}, {30, -50}, {10, -30}, {10, 30}}, lineColor = {64, 64, 64}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Text(extent = {{-150, -110}, {150, -80}}, lineColor = {0, 0, 0}, textString = "n=%n"), Text(visible = useAxisFlange, extent = {{-150, -155}, {150, -115}}, textString = "%name", lineColor = {0, 0, 255}), Line(visible = useAxisFlange, points = {{-20, 70}, {-60, 70}, {-60, 60}}, color = {0, 0, 0}, smooth = Smooth.None), Line(visible = useAxisFlange, points = {{20, 70}, {50, 70}, {50, 60}}, color = {0, 0, 0}, smooth = Smooth.None), Line(visible = useAxisFlange, points = {{-90, 100}, {-30, 100}}, color = {0, 0, 0}), Line(visible = useAxisFlange, points = {{-30, 100}, {-50, 80}}, color = {0, 0, 0}), Line(visible = useAxisFlange, points = {{-49, 100}, {-70, 80}}, color = {0, 0, 0}), Line(visible = useAxisFlange, points = {{-70, 100}, {-90, 80}}, color = {0, 0, 0}), Text(visible = not useAxisFlange, extent = {{-150, 70}, {150, 110}}, textString = "%name", lineColor = {0, 0, 255})}),
            Documentation(info = "<html>

<p>
Joint where frame_b rotates around axis n which is fixed in frame_a.
The two frames coincide when the rotation angle \"phi = 0\".
</p>

<p>
Optionally, two additional 1-dimensional mechanical flanges
(flange \"axis\" represents the driving flange and
flange \"support\" represents the bearing) can be enabled via
parameter <b>useAxisFlange</b>. The enabled axis flange can be
driven with elements of the
<a href=\"modelica://Modelica.Mechanics.Rotational\">Modelica.Mechanics.Rotational</a>
library.

</p>

<p>
In the \"Advanced\" menu it can be defined via parameter <b>stateSelect</b>
that the rotation angle \"phi\" and its derivative shall be definitely
used as states by setting stateSelect=StateSelect.always.
Default is StateSelect.prefer to use the joint angle and its
derivative as preferred states. The states are usually selected automatically.
In certain situations, especially when closed kinematic loops are present,
it might be slightly more efficient, when using the StateSelect.always setting.
</p>
<p>
If a <b>planar loop</b> is present, e.g., consisting of 4 revolute joints
where the joint axes are all parallel to each other, then there is no
longer a unique mathematical solution and the symbolic algorithms will
fail. Usually, an error message will be printed pointing out this
situation. In this case, one revolute joint of the loop has to be replaced
by a Joints.RevolutePlanarLoopConstraint joint. The
effect is that from the 5 constraints of a usual revolute joint,
3 constraints are removed and replaced by appropriate known
variables (e.g., the force in the direction of the axis of rotation is
treated as known with value equal to zero; for standard revolute joints,
this force is an unknown quantity).
</p>

<p>
In the following figure the animation of a revolute
joint is shown. The light blue coordinate system is
frame_a and the dark blue coordinate system is
frame_b of the joint. The black arrow is parameter
vector \"n\" defining the translation axis
(here: n = {0,0,1}, phi.start = 45<sup>o</sup>).
</p>

<p>
<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Joints/Revolute.png\">
</p>

</html>"));
        end Revolute;

        model RevolutePlanarLoopConstraint "Revolute joint that is described by 2 positional constraints for usage in a planar loop (the ambiguous cut-force perpendicular to the loop and the ambiguous cut-torques are set arbitrarily to zero)"
          import T = Modelica.Mechanics.MultiBody.Frames.TransformationMatrices;
          import Modelica.Mechanics.MultiBody.Types;
          Interfaces.Frame_a frame_a "Coordinate system fixed to the joint with one cut-force and cut-torque" annotation(
            Placement(transformation(extent = {{-116, -16}, {-84, 16}}, rotation = 0)));
          Interfaces.Frame_b frame_b "Coordinate system fixed to the joint with one cut-force and cut-torque" annotation(
            Placement(transformation(extent = {{84, -16}, {116, 16}}, rotation = 0)));
          parameter Boolean animation = true "= true, if animation shall be enabled (show axis as cylinder)";
          parameter Modelica.Mechanics.MultiBody.Types.Axis n = {0, 0, 1} "Axis of rotation resolved in frame_a (= same as in frame_b)" annotation(
            Evaluate = true);
          parameter SI.Distance cylinderLength = world.defaultJointLength "Length of cylinder representing the joint axis" annotation(
            Dialog(group = "if animation = true", enable = animation));
          parameter SI.Distance cylinderDiameter = world.defaultJointWidth "Diameter of cylinder representing the joint axis" annotation(
            Dialog(group = "if animation = true", enable = animation));
          input Types.Color cylinderColor = Modelica.Mechanics.MultiBody.Types.Defaults.JointColor "Color of cylinder representing the joint axis" annotation(
            Dialog(colorSelector = true, group = "if animation = true", enable = animation));
          input Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(
            Dialog(group = "if animation = true", enable = animation));
        protected
          outer Modelica.Mechanics.MultiBody.World world;
          parameter Real e[3](each final unit = "1") = Modelica.Math.Vectors.normalizeWithAssert(n) "Unit vector in direction of rotation axis, resolved in frame_a (= same as in frame_b)";
          parameter Real nnx_a[3](each final unit = "1") = if abs(e[1]) > 0.1 then {0, 1, 0} else if abs(e[2]) > 0.1 then {0, 0, 1} else {1, 0, 0} "Arbitrary vector that is not aligned with rotation axis n" annotation(
            Evaluate = true);
          parameter Real ey_a[3](each final unit = "1") = Modelica.Math.Vectors.normalizeWithAssert(cross(e, nnx_a)) "Unit vector orthogonal to axis n of revolute joint, resolved in frame_a" annotation(
            Evaluate = true);
          parameter Real ex_a[3](each final unit = "1") = cross(ey_a, e) "Unit vector orthogonal to axis n of revolute joint and to ey_a, resolved in frame_a" annotation(
            Evaluate = true);
          Real ey_b[3](each final unit = "1") "ey_a, resolved in frame_b";
          Real ex_b[3](each final unit = "1") "ex_a, resolved in frame_b";
          Frames.Orientation R_rel "Dummy or relative orientation object from frame_a to frame_b";
          Modelica.SIunits.Position r_rel_a[3] "Position vector from origin of frame_a to origin of frame_b, resolved in frame_a";
          SI.Force f_c[2] "Dummy or constraint forces in direction of ex_a, ey_a";
          Visualizers.Advanced.Shape cylinder(shapeType = "cylinder", color = cylinderColor, specularCoefficient = specularCoefficient, length = cylinderLength, width = cylinderDiameter, height = cylinderDiameter, lengthDirection = e, widthDirection = {0, 1, 0}, r_shape = -e * (cylinderLength / 2), r = frame_a.r_0, R = frame_a.R) if world.enableAnimation and animation;
        equation
          assert(cardinality(frame_a) > 0, "Connector frame_a of revolute joint is not connected");
          assert(cardinality(frame_b) > 0, "Connector frame_b of revolute joint is not connected");
// Determine relative position vector resolved in frame_a
          R_rel = Frames.relativeRotation(frame_a.R, frame_b.R);
          r_rel_a = Frames.resolve2(frame_a.R, frame_b.r_0 - frame_a.r_0);
// r_rel_a = T.resolve1(R_rel.T, T.resolve2(frame_b.R.T, frame_b.r_0 - frame_a.r_0));
// Constraint equations
          0 = ex_a * r_rel_a;
          0 = ey_a * r_rel_a;
/* Transform forces and torques
     (the torques are assumed to be zero by the assumption
      of a planar joint)
  */
          frame_a.t = zeros(3);
          frame_b.t = zeros(3);
          frame_a.f = [ex_a, ey_a] * f_c;
          frame_b.f = -Frames.resolve2(R_rel, frame_a.f);
// check that revolute joint is used in planar loop
          ex_b = Frames.resolve2(R_rel, ex_a);
          ey_b = Frames.resolve2(R_rel, ey_a);
          assert(noEvent(abs(e * r_rel_a) <= 1.e-10 and abs(e * ex_b) <= 1.e-10 and abs(e * ey_b) <= 1.e-10), "
The MultiBody.Joints.RevolutePlanarLoopConstraint joint is used as cut-joint of a
planar loop. However, the revolute joint is not part of a planar loop where the
axis of the revolute joint (parameter n) is orthogonal to the possible movements.
Either use instead joint MultiBody.Joints.Revolute or correct the
definition of the axes vectors n in the revolute joints of the planar loop.
          ");
          annotation(
            defaultComponentName = "revolute",
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-150, 70}, {150, 100}}, lineColor = {0, 0, 0}, textString = "n=%n"), Text(extent = {{-150, -110}, {150, -70}}, textString = "%name", lineColor = {0, 0, 255}), Rectangle(extent = {{-20, 10}, {20, -10}}, lineColor = {64, 64, 64}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-100, -60}, {-20, 60}}, lineColor = {64, 64, 64}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {255, 255, 255}, radius = 10), Rectangle(extent = {{20, -60}, {100, 60}}, lineColor = {64, 64, 64}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {255, 255, 255}, radius = 10), Rectangle(extent = {{-100, 60}, {-20, -60}}, lineColor = {64, 64, 64}, radius = 10), Rectangle(extent = {{20, 60}, {100, -60}}, lineColor = {64, 64, 64}, radius = 10), Text(extent = {{-90, 14}, {-54, -11}}, lineColor = {128, 128, 128}, textString = "a"), Text(extent = {{51, 11}, {87, -14}}, lineColor = {128, 128, 128}, textString = "b"), Line(points = {{-91, -76}, {-33, 15}, {30, -49}, {87, 61}}, color = {255, 0, 0}, thickness = 0.5, smooth = Smooth.None)}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, -60}, {-20, 60}}, lineColor = {64, 64, 64}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {255, 255, 255}, radius = 10), Rectangle(extent = {{-100, -60}, {-20, 60}}, lineColor = {64, 64, 64}, fillPattern = FillPattern.None, radius = 10), Rectangle(extent = {{-20, 10}, {20, -10}}, lineColor = {64, 64, 64}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Rectangle(extent = {{20, -60}, {100, 60}}, lineColor = {64, 64, 64}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {255, 255, 255}, radius = 10), Rectangle(extent = {{20, -60}, {100, 60}}, lineColor = {64, 64, 64}, fillPattern = FillPattern.None, radius = 10)}),
            Documentation(info = "<HTML>
<p>
Joint where frame_b rotates around axis n which is fixed in frame_a and
where this joint is used in a planar loop providing 2 constraint equations
on position level.
</p>

<p>
If a <b>planar loop</b> is present, e.g., consisting of 4 revolute joints
where the joint axes are all parallel to each other, then there is no
unique mathematical solution if all revolute joints are modelled with
Joints.Revolute and the symbolic algorithms will
fail. The reason is that, e.g., the cut-forces in the revolute joints perpendicular
to the planar loop are not uniquely defined when 3-dim. descriptions of revolute
joints are used. Usually, an error message will be printed pointing out this
situation. In this case, <b>one</b> revolute joint in the loop has to be replaced by
model Joints.RevolutePlanarLoopCutJoint. The
effect is that from the 5 constraints of a 3-dim. revolute joint,
3 constraints are removed and replaced by appropriate known
variables (e.g., the force in the direction of the axis of rotation is
treated as known with value equal to zero; for standard revolute joints,
this force is an unknown quantity).
</p>

</html>"));
        end RevolutePlanarLoopConstraint;

        model Cylindrical "Cylindrical joint (2 degrees-of-freedom, 4 potential states)"
          extends Modelica.Mechanics.MultiBody.Interfaces.PartialTwoFrames;
          parameter Boolean animation = true "= true, if animation shall be enabled (show cylinder)";
          parameter Modelica.Mechanics.MultiBody.Types.Axis n = {1, 0, 0} "Cylinder axis resolved in frame_a (= same as in frame_b)" annotation(
            Evaluate = true);
          parameter SI.Distance cylinderDiameter = world.defaultJointWidth "Diameter of cylinder" annotation(
            Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          input Types.Color cylinderColor = Modelica.Mechanics.MultiBody.Types.Defaults.JointColor "Color of cylinder" annotation(
            Dialog(colorSelector = true, tab = "Animation", group = "if animation = true", enable = animation));
          input Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(
            Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter StateSelect stateSelect = StateSelect.prefer "Priority to use joint coordinates (phi, s, w, v) as states" annotation(
            Dialog(tab = "Advanced"));
          Prismatic prismatic(n = n, animation = false, stateSelect = StateSelect.never) annotation(
            Placement(transformation(extent = {{-70, -25}, {-15, 25}}, rotation = 0)));
          Revolute revolute(n = n, animation = false, stateSelect = StateSelect.never) annotation(
            Placement(transformation(extent = {{10, -25}, {65, 25}}, rotation = 0)));
          SI.Position s(start = 0, stateSelect = stateSelect) "Relative distance between frame_a and frame_b";
          SI.Angle phi(start = 0, stateSelect = stateSelect) "Relative rotation angle from frame_a to frame_b";
          SI.Velocity v(start = 0, stateSelect = stateSelect) "First derivative of s (relative velocity)";
          SI.AngularVelocity w(start = 0, stateSelect = stateSelect) "First derivative of angle phi (relative angular velocity)";
          SI.Acceleration a(start = 0) "Second derivative of s (relative acceleration)";
          SI.AngularAcceleration wd(start = 0) "Second derivative of angle phi (relative angular acceleration)";
        protected
          Visualizers.Advanced.Shape cylinder(shapeType = "cylinder", color = cylinderColor, specularCoefficient = specularCoefficient, length = prismatic.s, width = cylinderDiameter, height = cylinderDiameter, lengthDirection = prismatic.n, widthDirection = {0, 1, 0}, r = frame_a.r_0, R = frame_a.R) if world.enableAnimation and animation annotation(
            Placement(transformation(extent = {{-20, 40}, {0, 60}}, rotation = 0)));
        equation
          phi = revolute.phi;
          w = der(phi);
          wd = der(w);
          s = prismatic.s;
          v = der(s);
          a = der(v);
          connect(frame_a, prismatic.frame_a) annotation(
            Line(points = {{-100, 0}, {-70, 0}}, color = {95, 95, 95}, thickness = 0.5));
          connect(prismatic.frame_b, revolute.frame_a) annotation(
            Line(points = {{-15, 0}, {10, 0}}, color = {95, 95, 95}, thickness = 0.5));
          connect(revolute.frame_b, frame_b) annotation(
            Line(points = {{65, 0}, {100, 0}}, color = {95, 95, 95}, thickness = 0.5));
          annotation(
            Documentation(info = "<HTML>
<p>
Joint where frame_b rotates around and translates along axis n
which is fixed in frame_a. The two frames coincide when
\"phi=revolute.phi=0\" and \"s=prismatic.s=0\". This joint
has the following potential states;
<ul>
<li> The relative angle phi [rad] around axis n, </li>
<li> the relative distance s [m] along axis n, </li>
<li> the relative angular velocity w [rad/s] (= der(phi))
     and </li>
<li> the relative velocity v [m/s] (= der(s)).</li>
</ul>
<p>
They are used as candidates for automatic selection of states
from the tool. This may be enforced by setting \"stateSelect=StateSelect.<b>always</b>\"
in the <b>Advanced</b> menu. The states are usually selected automatically.
In certain situations, especially when closed kinematic loops are present,
it might be slightly more efficient, when using the \"StateSelect.always\" setting.
</p>
<p>
In the following figure the animation of a cylindrical
joint is shown. The light blue coordinate system is
frame_a and the dark blue coordinate system is
frame_b of the joint. The black arrow is parameter
vector \"n\" defining the cylinder axis
(here: n = {0,0,1}).
</p>

<p>
<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Joints/Cylindrical.png\">
</p>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-30, -30}, {100, 30}}, lineColor = {64, 64, 64}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {255, 255, 255}, radius = 10), Rectangle(extent = {{-30, -30}, {100, 30}}, lineColor = {64, 64, 64}, fillPattern = FillPattern.None, radius = 10), Rectangle(extent = {{-100, -50}, {0, 50}}, lineColor = {64, 64, 64}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {255, 255, 255}, radius = 10), Rectangle(extent = {{-100, -50}, {0, 50}}, lineColor = {64, 64, 64}, fillPattern = FillPattern.None, radius = 10), Text(extent = {{-150, 100}, {150, 60}}, textString = "%name", lineColor = {0, 0, 255}), Text(extent = {{-150, -65}, {150, -95}}, lineColor = {0, 0, 0}, textString = "n=%n")}));
        end Cylindrical;

        model Universal "Universal joint (2 degrees-of-freedom, 4 potential states)"
          extends Modelica.Mechanics.MultiBody.Interfaces.PartialTwoFrames;
          parameter Boolean animation = true "= true, if animation shall be enabled";
          parameter Modelica.Mechanics.MultiBody.Types.Axis n_a = {1, 0, 0} "Axis of revolute joint 1 resolved in frame_a" annotation(
            Evaluate = true);
          parameter Modelica.Mechanics.MultiBody.Types.Axis n_b = {0, 1, 0} "Axis of revolute joint 2 resolved in frame_b" annotation(
            Evaluate = true);
          parameter SI.Distance cylinderLength = world.defaultJointLength "Length of cylinders representing the joint axes" annotation(
            Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter SI.Distance cylinderDiameter = world.defaultJointWidth "Diameter of cylinders representing the joint axes" annotation(
            Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          input Types.Color cylinderColor = Modelica.Mechanics.MultiBody.Types.Defaults.JointColor "Color of cylinders representing the joint axes" annotation(
            Dialog(colorSelector = true, tab = "Animation", group = "if animation = true", enable = animation));
          input Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(
            Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter StateSelect stateSelect = StateSelect.prefer "Priority to use joint coordinates (phi_a, phi_b, w_a, w_b) as states" annotation(
            Dialog(tab = "Advanced"));
          Modelica.Mechanics.MultiBody.Joints.Revolute revolute_a(n = n_a, stateSelect = StateSelect.never, cylinderDiameter = cylinderDiameter, cylinderLength = cylinderLength, cylinderColor = cylinderColor, specularCoefficient = specularCoefficient, animation = animation) annotation(
            Placement(transformation(extent = {{-60, -25}, {-10, 25}}, rotation = 0)));
          Modelica.Mechanics.MultiBody.Joints.Revolute revolute_b(n = n_b, stateSelect = StateSelect.never, animation = animation, cylinderDiameter = cylinderDiameter, cylinderLength = cylinderLength, cylinderColor = cylinderColor, specularCoefficient = specularCoefficient) annotation(
            Placement(transformation(origin = {35, 45}, extent = {{-25, -25}, {25, 25}}, rotation = 90)));
          SI.Angle phi_a(start = 0, stateSelect = stateSelect) "Relative rotation angle from frame_a to intermediate frame";
          SI.Angle phi_b(start = 0, stateSelect = stateSelect) "Relative rotation angle from intermediate frame to frame_b";
          SI.AngularVelocity w_a(start = 0, stateSelect = stateSelect) "First derivative of angle phi_a (relative angular velocity a)";
          SI.AngularVelocity w_b(start = 0, stateSelect = stateSelect) "First derivative of angle phi_b (relative angular velocity b)";
          SI.AngularAcceleration a_a(start = 0) "Second derivative of angle phi_a (relative angular acceleration a)";
          SI.AngularAcceleration a_b(start = 0) "Second derivative of angle phi_b (relative angular acceleration b)";
        equation
          phi_a = revolute_a.phi;
          phi_b = revolute_b.phi;
          w_a = der(phi_a);
          w_b = der(phi_b);
          a_a = der(w_a);
          a_b = der(w_b);
          connect(frame_a, revolute_a.frame_a) annotation(
            Line(points = {{-100, 0}, {-60, 0}}, color = {95, 95, 95}, thickness = 0.5));
          connect(revolute_b.frame_b, frame_b) annotation(
            Line(points = {{35, 70}, {35, 90}, {70, 90}, {70, 0}, {100, 0}}, color = {95, 95, 95}, thickness = 0.5));
          connect(revolute_a.frame_b, revolute_b.frame_a) annotation(
            Line(points = {{-10, 0}, {35, 0}, {35, 20}}, color = {95, 95, 95}, thickness = 0.5));
          annotation(
            Documentation(info = "<HTML>
<p>
Joint where frame_a rotates around axis n_a which is fixed in frame_a
and frame_b rotates around axis n_b which is fixed in frame_b.
The two frames coincide when
\"revolute_a.phi=0\" and \"revolute_b.phi=0\". This joint
has the following potential states;
<ul>
<li> The relative angle phi_a = revolute_a.phi [rad] around axis n_a, </li>
<li> the relative angle phi_b = revolute_b.phi [rad] around axis n_b, </li>
<li> the relative angular velocity w_a (= der(phi_a))  and </li>
<li> the relative angular velocity w_b (= der(phi_b)).</li>
</ul>
<p>
They are used as candidates for automatic selection of states
from the tool. This may be enforced by setting \"stateSelect=StateSelect.<b>always</b>\"
in the <b>Advanced</b> menu. The states are usually selected automatically.
In certain situations, especially when closed kinematic loops are present,
it might be slightly more efficient, when using the \"StateSelect.always\" setting.
</p>

<p>
In the following figure the animation of a universal
joint is shown. The light blue coordinate system is
frame_a and the dark blue coordinate system is
frame_b of the joint
(here: n_a = {0,0,1}, n_b = {0,1,0}, phi_a.start = 90<sup>o</sup>,
phi_b.start = 45<sup>o</sup>).
</p>

<p>
<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Joints/Universal.png\">
</p>
</HTML>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, 15}, {-65, -15}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {235, 235, 235}), Ellipse(extent = {{-80, -80}, {80, 80}}, lineColor = {160, 160, 164}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-60, -60}, {60, 60}}, lineColor = {160, 160, 164}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-150, -80}, {150, -120}}, textString = "%name", lineColor = {0, 0, 255}), Rectangle(extent = {{12, 82}, {80, -82}}, lineColor = {255, 255, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Rectangle(extent = {{56, 15}, {100, -15}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {235, 235, 235}), Line(points = {{12, 78}, {12, -78}}, color = {0, 0, 0}, thickness = 0.5), Ellipse(extent = {{-52, -40}, {80, 40}}, lineColor = {160, 160, 164}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-32, -20}, {60, 26}}, lineColor = {160, 160, 164}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Polygon(points = {{-22, -54}, {-60, 0}, {-22, 50}, {40, 52}, {-22, -54}}, pattern = LinePattern.None, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Line(points = {{12, 78}, {12, -20}}, color = {0, 0, 0}, thickness = 0.5), Line(points = {{32, 38}, {-12, -36}}, color = {0, 0, 0}, thickness = 0.5)}));
        end Universal;

        model Planar "Planar joint (3 degrees-of-freedom, 6 potential states)"
          extends Modelica.Mechanics.MultiBody.Interfaces.PartialTwoFrames;
          parameter Boolean animation = true "= true, if animation shall be enabled";
          parameter Modelica.Mechanics.MultiBody.Types.Axis n = {0, 0, 1} "Axis orthogonal to unconstrained plane, resolved in frame_a (= same as in frame_b)" annotation(
            Evaluate = true);
          parameter Modelica.Mechanics.MultiBody.Types.Axis n_x = {1, 0, 0} "Vector in direction of x-axis of plane, resolved in frame_a (n_x shall be orthogonal to n)" annotation(
            Evaluate = true);
          parameter SI.Distance cylinderLength = world.defaultJointLength "Length of revolute cylinder" annotation(
            Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter SI.Distance cylinderDiameter = world.defaultJointWidth "Diameter of revolute cylinder" annotation(
            Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          input Types.Color cylinderColor = Modelica.Mechanics.MultiBody.Types.Defaults.JointColor "Color of revolute cylinder" annotation(
            Dialog(colorSelector = true, tab = "Animation", group = "if animation = true", enable = animation));
          parameter SI.Distance boxWidth = 0.3 * cylinderDiameter "Width of prismatic joint boxes" annotation(
            Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter SI.Distance boxHeight = boxWidth "Height of prismatic joint boxes" annotation(
            Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          input Types.Color boxColor = Modelica.Mechanics.MultiBody.Types.Defaults.JointColor "Color of prismatic joint boxes" annotation(
            Dialog(colorSelector = true, tab = "Animation", group = "if animation = true", enable = animation));
          parameter StateSelect stateSelect = StateSelect.prefer "Priority to use joint coordinates (s_x, s_y, phi, v_x, v_y, w) as states" annotation(
            Dialog(tab = "Advanced"));
          Prismatic prismatic_x(stateSelect = StateSelect.never, n = cross(cross(n, n_x), n), animation = false) annotation(
            Placement(transformation(extent = {{-69, -20}, {-29, 20}}, rotation = 0)));
          Prismatic prismatic_y(stateSelect = StateSelect.never, n = cross(n, n_x), animation = false) annotation(
            Placement(transformation(origin = {0, 50}, extent = {{-20, -20}, {20, 20}}, rotation = 90)));
          Revolute revolute(stateSelect = StateSelect.never, n = n, animation = false) annotation(
            Placement(transformation(extent = {{41, -20}, {81, 20}}, rotation = 0)));
          SI.Position s_x(start = 0, stateSelect = stateSelect) "Relative distance along first prismatic joint starting at frame_a";
          SI.Position s_y(start = 0, stateSelect = stateSelect) "Relative distance along second prismatic joint starting at first prismatic joint";
          SI.Angle phi(start = 0, stateSelect = stateSelect) "Relative rotation angle from frame_a to frame_b";
          SI.Velocity v_x(start = 0, stateSelect = stateSelect) "First derivative of s_x (relative velocity in s_x direction)";
          SI.Velocity v_y(start = 0, stateSelect = stateSelect) "First derivative of s_y (relative velocity in s_y direction)";
          SI.AngularVelocity w(start = 0, stateSelect = stateSelect) "First derivative of angle phi (relative angular velocity)";
          SI.Acceleration a_x(start = 0) "Second derivative of s_x (relative acceleration in s_x direction)";
          SI.Acceleration a_y(start = 0) "Second derivative of s_y (relative acceleration in s_y direction)";
          SI.AngularAcceleration wd(start = 0) "Second derivative of angle phi (relative angular acceleration)";
        protected
          parameter Integer ndim = if world.enableAnimation and animation then 1 else 0;
          parameter Real e[3](each final unit = "1") = Modelica.Math.Vectors.normalize(n);
        protected
          Visualizers.Advanced.Shape box_x[ndim](each shapeType = "box", each color = boxColor, each length = prismatic_x.s, each width = boxWidth, each height = boxWidth, each lengthDirection = prismatic_x.e, each widthDirection = {0, 1, 0}, each r = frame_a.r_0, each R = frame_a.R) annotation(
            Placement(transformation(extent = {{-80, 30}, {-60, 50}}, rotation = 0)));
          Visualizers.Advanced.Shape box_y[ndim](each shapeType = "box", each color = boxColor, each length = prismatic_y.s, each width = boxWidth, each height = boxWidth, each lengthDirection = prismatic_y.e, each widthDirection = {1, 0, 0}, each r = prismatic_y.frame_a.r_0, each R = prismatic_y.frame_a.R) annotation(
            Placement(transformation(extent = {{-46, 69}, {-26, 89}}, rotation = 0)));
          Visualizers.Advanced.Shape cylinder[ndim](each shapeType = "cylinder", each color = cylinderColor, each length = cylinderLength, each width = cylinderDiameter, each height = cylinderDiameter, each lengthDirection = n, each widthDirection = {0, 1, 0}, each r_shape = -e * (cylinderLength / 2), each r = revolute.frame_b.r_0, each R = revolute.frame_b.R) annotation(
            Placement(transformation(extent = {{50, 30}, {70, 50}}, rotation = 0)));
        equation
          s_x = prismatic_x.s;
          s_y = prismatic_y.s;
          phi = revolute.phi;
          v_x = der(s_x);
          v_y = der(s_y);
          w = der(phi);
          a_x = der(v_x);
          a_y = der(v_y);
          wd = der(w);
          connect(frame_a, prismatic_x.frame_a) annotation(
            Line(points = {{-100, 4.44089e-16}, {-84, 4.44089e-16}, {-84, 0}, {-69, 0}}, color = {95, 95, 95}, thickness = 0.5));
          connect(prismatic_x.frame_b, prismatic_y.frame_a) annotation(
            Line(points = {{-29, 0}, {0, 0}, {0, 30}}, color = {95, 95, 95}, thickness = 0.5));
          connect(prismatic_y.frame_b, revolute.frame_a) annotation(
            Line(points = {{0, 70}, {0, 80}, {30, 80}, {30, 0}, {41, 0}}, color = {95, 95, 95}, thickness = 0.5));
          connect(revolute.frame_b, frame_b) annotation(
            Line(points = {{81, 0}, {92, 0}, {92, 4.44089e-16}, {100, 4.44089e-16}}, color = {95, 95, 95}, thickness = 0.5));
          annotation(
            Documentation(info = "<HTML>
<p>
Joint where frame_b can move in a plane and can rotate around an
axis orthogonal to the plane. The plane is defined by
vector n which is perpendicular to the plane and by vector n_x,
which points in the direction of the x-axis of the plane.
frame_a and frame_b coincide when s_x=prismatic_x.s=0,
s_y=prismatic_y.s=0 and phi=revolute.phi=0. This joint has the following
potential states:
<ul>
<li> the relative distance s_x = prismatic_x.s [m] along axis n_x, </li>
<li> the relative distance s_y = prismatic_y.s [m] along axis n_y = cross(n,n_x), </li>
<li> the relative angle phi = revolute.phi [rad] around axis n, </li>
<li> the relative velocity v_x (= der(s_x)).</li>
<li> the relative velocity v_y (= der(s_y)).</li>
<li> the relative angular velocity w (= der(phi))</li>
</ul>
<p>
They are used as candidates for automatic selection of states
from the tool. This may be enforced by setting \"stateSelect=StateSelect.<b>always</b>\"
in the <b>Advanced</b> menu. The states are usually selected automatically.
In certain situations, especially when closed kinematic loops are present,
it might be slightly more efficient, when using the \"StateSelect.always\" setting.
</p>
<p>
In the following figure the animation of a planar
joint is shown. The light blue coordinate system is
frame_a and the dark blue coordinate system is
frame_b of the joint. The black arrows are parameter
vectors \"n\" and \"n_x\"
(here: n = {0,1,0}, n_x = {0,0,1}, s_x.start = 0.5,
s_y.start = 0.5, phi.start = 45<sup>o</sup>).
</p>

<p>
<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Joints/Planar.png\">
</p>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-30, -60}, {-10, 60}}, lineColor = {0, 0, 0}, pattern = LinePattern.None, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Rectangle(extent = {{10, -60}, {30, 60}}, lineColor = {0, 0, 0}, pattern = LinePattern.None, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-100, -10}, {-30, 10}}, lineColor = {0, 0, 0}, pattern = LinePattern.None, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Rectangle(extent = {{100, -10}, {30, 10}}, lineColor = {0, 0, 0}, pattern = LinePattern.None, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Text(extent = {{-150, -75}, {150, -105}}, lineColor = {0, 0, 0}, textString = "n=%n"), Text(extent = {{-150, 110}, {150, 70}}, textString = "%name", lineColor = {0, 0, 255})}));
        end Planar;

        model Spherical "Spherical joint (3 constraints and no potential states, or 3 degrees-of-freedom and 3 states)"
          import Modelica.Mechanics.MultiBody.Frames;
          extends Modelica.Mechanics.MultiBody.Interfaces.PartialTwoFrames;
          parameter Boolean animation = true "= true, if animation shall be enabled (show sphere)";
          parameter SI.Distance sphereDiameter = world.defaultJointLength "Diameter of sphere representing the spherical joint" annotation(
            Dialog(group = "if animation = true", enable = animation));
          input Types.Color sphereColor = Modelica.Mechanics.MultiBody.Types.Defaults.JointColor "Color of sphere representing the spherical joint" annotation(
            Dialog(colorSelector = true, group = "if animation = true", enable = animation));
          input Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(
            Dialog(group = "if animation = true", enable = animation));
          parameter Boolean angles_fixed = false "= true, if angles_start are used as initial values, else as guess values" annotation(
            Evaluate = true,
            choices(checkBox = true),
            Dialog(tab = "Initialization"));
          parameter SI.Angle angles_start[3] = {0, 0, 0} "Initial values of angles to rotate frame_a around 'sequence_start' axes into frame_b" annotation(
            Dialog(tab = "Initialization"));
          parameter Types.RotationSequence sequence_start = {1, 2, 3} "Sequence of rotations to rotate frame_a into frame_b at initial time" annotation(
            Evaluate = true,
            Dialog(tab = "Initialization"));
          parameter Boolean w_rel_a_fixed = false "= true, if w_rel_a_start are used as initial values, else as guess values" annotation(
            Evaluate = true,
            choices(checkBox = true),
            Dialog(tab = "Initialization"));
          parameter SI.AngularVelocity w_rel_a_start[3] = {0, 0, 0} "Initial values of angular velocity of frame_b with respect to frame_a, resolved in frame_a" annotation(
            Dialog(tab = "Initialization"));
          parameter Boolean z_rel_a_fixed = false "= true, if z_rel_a_start are used as initial values, else as guess values" annotation(
            Evaluate = true,
            choices(checkBox = true),
            Dialog(tab = "Initialization"));
          parameter SI.AngularAcceleration z_rel_a_start[3] = {0, 0, 0} "Initial values of angular acceleration z_rel_a = der(w_rel_a)" annotation(
            Dialog(tab = "Initialization"));
          parameter Boolean enforceStates = false "= true, if relative variables of spherical joint shall be used as states (StateSelect.always)" annotation(
            Dialog(tab = "Advanced"));
          parameter Boolean useQuaternions = true "= true, if quaternions shall be used as states otherwise use 3 angles as states (provided enforceStates=true)" annotation(
            Dialog(tab = "Advanced", enable = enforceStates));
          parameter Types.RotationSequence sequence_angleStates = {1, 2, 3} "Sequence of rotations to rotate frame_a into frame_b around the 3 angles used as states" annotation(
            Evaluate = true,
            Dialog(tab = "Advanced", enable = enforceStates and not useQuaternions));
          final parameter Frames.Orientation R_rel_start = Frames.axesRotations(sequence_start, angles_start, zeros(3)) "Orientation object from frame_a to frame_b at initial time";
        protected
          Visualizers.Advanced.Shape sphere(shapeType = "sphere", color = sphereColor, specularCoefficient = specularCoefficient, length = sphereDiameter, width = sphereDiameter, height = sphereDiameter, lengthDirection = {1, 0, 0}, widthDirection = {0, 1, 0}, r_shape = {-0.5, 0, 0} * sphereDiameter, r = frame_a.r_0, R = frame_a.R) if world.enableAnimation and animation;
          // Declarations for quaternions (dummies, if quaternions are not used)
          parameter Frames.Quaternions.Orientation Q_start = Modelica.Mechanics.MultiBody.Frames.to_Q(R_rel_start) "Quaternion orientation object from frame_a to frame_b at initial time";
          Frames.Quaternions.Orientation Q(start = Q_start, each stateSelect = if enforceStates and useQuaternions then StateSelect.prefer else StateSelect.never) "Quaternion orientation object from frame_a to frame_b (dummy value, if quaternions are not used as states)";
          // Declaration for 3 angles
          parameter SI.Angle phi_start[3] = if sequence_start[1] == sequence_angleStates[1] and sequence_start[2] == sequence_angleStates[2] and sequence_start[3] == sequence_angleStates[3] then angles_start else Frames.axesRotationsAngles(R_rel_start, sequence_angleStates) "Potential angle states at initial time";
          SI.Angle phi[3](start = phi_start, each stateSelect = if enforceStates and not useQuaternions then StateSelect.always else StateSelect.never) "Dummy or 3 angles to rotate frame_a into frame_b";
          SI.AngularVelocity phi_d[3](each stateSelect = if enforceStates and not useQuaternions then StateSelect.always else StateSelect.never) "= der(phi)";
          SI.AngularAcceleration phi_dd[3] "= der(phi_d)";
          // Other declarations
          SI.AngularVelocity w_rel[3](start = Frames.resolve2(R_rel_start, w_rel_a_start), fixed = fill(w_rel_a_fixed, 3), each stateSelect = if enforceStates and useQuaternions then StateSelect.always else StateSelect.never) "Dummy or relative angular velocity of frame_b with respect to frame_a, resolved in frame_b";
          Frames.Orientation R_rel "Dummy or relative orientation object to rotate from frame_a to frame_b";
          Frames.Orientation R_rel_inv "Dummy or relative orientation object to rotate from frame_b to frame_a";
        initial equation
          if angles_fixed then
            if not enforceStates then
// no states defined in spherical object
              zeros(3) = Frames.Orientation.equalityConstraint(Frames.absoluteRotation(frame_a.R, R_rel_start), frame_b.R);
            elseif useQuaternions then
// Quaternions Q are used as states
              zeros(3) = Frames.Quaternions.Orientation.equalityConstraint(Q, Q_start);
            else
// The 3 angles 'phi' are used as states
              phi = phi_start;
            end if;
          end if;
          if z_rel_a_fixed then
// Initialize acceleration variables
            der(w_rel) = Frames.resolve2(R_rel_start, z_rel_a_start);
          end if;
        equation
// torque balance
          zeros(3) = frame_a.t;
          zeros(3) = frame_b.t;
          if enforceStates then
            Connections.branch(frame_a.R, frame_b.R);
            frame_b.r_0 = frame_a.r_0;
            if Connections.rooted(frame_a.R) then
              R_rel_inv = Frames.nullRotation();
              frame_b.R = Frames.absoluteRotation(frame_a.R, R_rel);
              zeros(3) = frame_a.f + Frames.resolve1(R_rel, frame_b.f);
            else
              R_rel_inv = Frames.inverseRotation(R_rel);
              frame_a.R = Frames.absoluteRotation(frame_b.R, R_rel_inv);
              zeros(3) = frame_b.f + Frames.resolve2(R_rel, frame_a.f);
            end if;
// Compute relative orientation object
            if useQuaternions then
// Use Quaternions as states (with dynamic state selection)
              {0} = Frames.Quaternions.orientationConstraint(Q);
              w_rel = Frames.Quaternions.angularVelocity2(Q, der(Q));
              R_rel = Frames.from_Q(Q, w_rel);
// Dummies
              phi = zeros(3);
              phi_d = zeros(3);
              phi_dd = zeros(3);
            else
// Use angles as states
              phi_d = der(phi);
              phi_dd = der(phi_d);
              R_rel = Frames.axesRotations(sequence_angleStates, phi, phi_d);
              w_rel = Frames.angularVelocity2(R_rel);
// Dummies
              Q = zeros(4);
            end if;
          else
// Spherical joint does not have states
            frame_b.r_0 = frame_a.r_0;
//frame_b.r_0 = transpose(frame_b.R.T)*(frame_b.R.T*(transpose(frame_a.R.T)*(frame_a.R.T*frame_a.r_0)));
            zeros(3) = frame_a.f + Frames.resolveRelative(frame_b.f, frame_b.R, frame_a.R);
            if w_rel_a_fixed or z_rel_a_fixed then
              w_rel = Frames.angularVelocity2(frame_b.R) - Frames.resolve2(frame_b.R, Frames.angularVelocity1(frame_a.R));
            else
              w_rel = zeros(3);
            end if;
// Dummies
            R_rel = Frames.nullRotation();
            R_rel_inv = Frames.nullRotation();
            Q = zeros(4);
            phi = zeros(3);
            phi_d = zeros(3);
            phi_dd = zeros(3);
          end if;
          annotation(
            Documentation(info = "<html>
<p>
Joint with <b>3 constraints</b> that define that the origin of
frame_a and the origin of frame_b coincide. By default this joint
defines only the 3 constraints without any potential states.
If parameter <b>enforceStates</b> is set to <b>true</b>
in the \"Advanced\" menu, three states are introduced.
Depending on parameter <b>useQuaternions</b> these are either
quaternions and the relative angular velocity or 3 angles
and the angle derivatives. In the latter case the orientation
of frame_b is computed by rotating frame_a along the axes defined
in parameter vector \"sequence_angleStates\" (default = {1,2,3}, i.e.,
the Cardan angle sequence) around the angles used as states.
For example, the default is to rotate the x-axis of frame_a
around angles[1], the new y-axis around angles[2] and the new z-axis
around angles[3], arriving at frame_b. If angles are used
as states there is the slight disadvantage that
a singular configuration is present leading to a division by zero.
</p>
<p>
If this joint is used in a <b>chain</b> structure, a Modelica translator
has to select orientation coordinates of a body as states, if the
default setting is used. It is usually better to use relative coordinates
in the spherical joint as states, and therefore in this situation
parameter enforceStates might be set to <b>true</b>.
</p>
<p>
If this joint is used in a <b>loop</b> structure, the default
setting results in a <b>cut-joint</b> that
breaks the loop in independent kinematic pieces, hold together
by the constraints of this joint. As a result, a Modelica translator
will first try to select 3 generalized coordinates in the joints of
the remaining parts of the loop and their first derivative as states
and if this is not possible, e.g., because there are only spherical
joints in the loop, will select coordinates from a body of the loop
as states.
</p>
<p>
In the following figure the animation of a spherical
joint is shown. The light blue coordinate system is
frame_a and the dark blue coordinate system is
frame_b of the joint.
(here: angles_start = {45, 45, 45}<sup>o</sup>).
</p>

<p>
<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Joints/Spherical.png\">
</p>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-70, -70}, {70, 70}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.Sphere, fillColor = {192, 192, 192}), Ellipse(extent = {{-49, -50}, {51, 50}}, lineColor = {128, 128, 128}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Rectangle(extent = {{30, 70}, {71, -68}}, lineColor = {255, 255, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-100, 10}, {-68, -10}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192, 192, 192}), Rectangle(extent = {{23, 10}, {100, -10}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192, 192, 192}), Ellipse(extent = {{-24, 25}, {26, -25}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.Sphere, fillColor = {160, 160, 164}), Text(extent = {{-150, -115}, {150, -75}}, textString = "%name", lineColor = {0, 0, 255})}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-70, -70}, {70, 70}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.Sphere, fillColor = {192, 192, 192}), Ellipse(extent = {{-49, -50}, {51, 50}}, lineColor = {128, 128, 128}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Rectangle(extent = {{30, 70}, {71, -68}}, lineColor = {255, 255, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-100, 10}, {-68, -10}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192, 192, 192}), Rectangle(extent = {{23, 10}, {100, -10}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192, 192, 192}), Ellipse(extent = {{-24, 25}, {26, -25}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.Sphere, fillColor = {160, 160, 164})}));
        end Spherical;

        model FreeMotion "Free motion joint (6 degrees-of-freedom, 12 potential states)"
          extends Modelica.Mechanics.MultiBody.Interfaces.PartialTwoFrames;
          parameter Boolean animation = true "= true, if animation shall be enabled (show arrow from frame_a to frame_b)";
          SI.Position r_rel_a[3](start = {0, 0, 0}, each stateSelect = if enforceStates then StateSelect.always else StateSelect.prefer) "Position vector from origin of frame_a to origin of frame_b, resolved in frame_a" annotation(
            Dialog(group = "Initialization", showStartAttribute = true));
          SI.Velocity v_rel_a[3](start = {0, 0, 0}, each stateSelect = if enforceStates then StateSelect.always else StateSelect.prefer) "= der(r_rel_a), i.e., velocity of origin of frame_b with respect to origin of frame_a, resolved in frame_a" annotation(
            Dialog(group = "Initialization", showStartAttribute = true));
          SI.Acceleration a_rel_a[3](start = {0, 0, 0}) "= der(v_rel_a)" annotation(
            Dialog(group = "Initialization", showStartAttribute = true));
          parameter Boolean angles_fixed = false "= true, if angles_start are used as initial values, else as guess values" annotation(
            Evaluate = true,
            choices(checkBox = true),
            Dialog(group = "Initialization"));
          parameter SI.Angle angles_start[3] = {0, 0, 0} "Initial values of angles to rotate frame_a around 'sequence_start' axes into frame_b" annotation(
            Dialog(group = "Initialization"));
          parameter Types.RotationSequence sequence_start = {1, 2, 3} "Sequence of rotations to rotate frame_a into frame_b at initial time" annotation(
            Evaluate = true,
            Dialog(group = "Initialization"));
          parameter Boolean w_rel_a_fixed = false "= true, if w_rel_a_start are used as initial values, else as guess values" annotation(
            Evaluate = true,
            choices(checkBox = true),
            Dialog(group = "Initialization"));
          parameter SI.AngularVelocity w_rel_a_start[3] = {0, 0, 0} "Initial values of angular velocity of frame_b with respect to frame_a, resolved in frame_a" annotation(
            Dialog(group = "Initialization"));
          parameter Boolean z_rel_a_fixed = false "= true, if z_rel_a_start are used as initial values, else as guess values" annotation(
            Evaluate = true,
            choices(checkBox = true),
            Dialog(group = "Initialization"));
          parameter SI.AngularAcceleration z_rel_a_start[3] = {0, 0, 0} "Initial values of angular acceleration z_rel_a = der(w_rel_a)" annotation(
            Dialog(group = "Initialization"));
          parameter SI.Length arrowDiameter = world.defaultArrowDiameter "Diameter of arrow from frame_a to frame_b" annotation(
            Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          input Types.Color arrowColor = Modelica.Mechanics.MultiBody.Types.Defaults.SensorColor "Color of arrow" annotation(
            Dialog(colorSelector = true, tab = "Animation", group = "if animation = true", enable = animation));
          input Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(
            Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter Boolean enforceStates = true "= true, if relative variables between frame_a and frame_b shall be used as states" annotation(
            Dialog(tab = "Advanced"));
          parameter Boolean useQuaternions = true "= true, if quaternions shall be used as states otherwise use 3 angles as states" annotation(
            Dialog(tab = "Advanced"));
          parameter Types.RotationSequence sequence_angleStates = {1, 2, 3} "Sequence of rotations to rotate frame_a into frame_b around the 3 angles used as states" annotation(
            Evaluate = true,
            Dialog(tab = "Advanced", enable = not useQuaternions));
          final parameter Frames.Orientation R_rel_start = Modelica.Mechanics.MultiBody.Frames.axesRotations(sequence_start, angles_start, zeros(3)) "Orientation object from frame_a to frame_b at initial time";
        protected
          Visualizers.Advanced.Arrow arrow(r_head = r_rel_a, diameter = arrowDiameter, color = arrowColor, specularCoefficient = specularCoefficient, r = frame_a.r_0, R = frame_a.R) if world.enableAnimation and animation;
          // Declarations for quaternions (dummies, if quaternions are not used)
          parameter Frames.Quaternions.Orientation Q_start = Frames.to_Q(R_rel_start) "Quaternion orientation object from frame_a to frame_b at initial time";
          Frames.Quaternions.Orientation Q(start = Q_start, each stateSelect = if enforceStates then if useQuaternions then StateSelect.prefer else StateSelect.never else StateSelect.default) "Quaternion orientation object from frame_a to frame_b (dummy value, if quaternions are not used as states)";
          // Declaration for 3 angles
          parameter SI.Angle phi_start[3] = if sequence_start[1] == sequence_angleStates[1] and sequence_start[2] == sequence_angleStates[2] and sequence_start[3] == sequence_angleStates[3] then angles_start else Frames.axesRotationsAngles(R_rel_start, sequence_angleStates) "Potential angle states at initial time";
          SI.Angle phi[3](start = phi_start, each stateSelect = if enforceStates then if useQuaternions then StateSelect.never else StateSelect.always else StateSelect.prefer) "Dummy or 3 angles to rotate frame_a into frame_b";
          SI.AngularVelocity phi_d[3](each stateSelect = if enforceStates then if useQuaternions then StateSelect.never else StateSelect.always else StateSelect.prefer) "= der(phi)";
          SI.AngularAcceleration phi_dd[3] "= der(phi_d)";
          // Other declarations
          SI.AngularVelocity w_rel_b[3](start = Frames.resolve2(R_rel_start, w_rel_a_start), fixed = fill(w_rel_a_fixed, 3), each stateSelect = if enforceStates then if useQuaternions then StateSelect.always else StateSelect.avoid else StateSelect.prefer) "Dummy or relative angular velocity of frame_b with respect to frame_a, resolved in frame_b";
          Frames.Orientation R_rel "Dummy or relative orientation object to rotate from frame_a to frame_b";
          Frames.Orientation R_rel_inv "Dummy or relative orientation object to rotate from frame_b to frame_a";
        initial equation
          if angles_fixed then
// Initialize positional variables
            if not enforceStates then
// no states defined
              zeros(3) = Frames.Orientation.equalityConstraint(Frames.absoluteRotation(frame_a.R, R_rel_start), frame_b.R);
            elseif useQuaternions then
// Quaternions Q are used as states
              zeros(3) = Frames.Quaternions.Orientation.equalityConstraint(Q, Q_start);
            else
// The 3 angles 'phi' are used as states
              phi = phi_start;
            end if;
          end if;
          if z_rel_a_fixed then
// Initialize acceleration variables
            der(w_rel_b) = Frames.resolve2(R_rel_start, z_rel_a_start);
          end if;
        equation
// Kinematic differential equations for translational motion
          der(r_rel_a) = v_rel_a;
          der(v_rel_a) = a_rel_a;
// Kinematic relationships
          frame_b.r_0 = frame_a.r_0 + Frames.resolve1(frame_a.R, r_rel_a);
// Cut-forces and cut-torques are zero
          frame_a.f = zeros(3);
          frame_a.t = zeros(3);
          frame_b.f = zeros(3);
          frame_b.t = zeros(3);
          if enforceStates then
            Connections.branch(frame_a.R, frame_b.R);
            if Connections.rooted(frame_a.R) then
              R_rel_inv = Frames.nullRotation();
              frame_b.R = Frames.absoluteRotation(frame_a.R, R_rel);
            else
              R_rel_inv = Frames.inverseRotation(R_rel);
              frame_a.R = Frames.absoluteRotation(frame_b.R, R_rel_inv);
            end if;
// Compute relative orientation object
            if useQuaternions then
// Use Quaternions as states (with dynamic state selection)
              {0} = Frames.Quaternions.orientationConstraint(Q);
              w_rel_b = Frames.Quaternions.angularVelocity2(Q, der(Q));
              R_rel = Frames.from_Q(Q, w_rel_b);
// Dummies
              phi = zeros(3);
              phi_d = zeros(3);
              phi_dd = zeros(3);
            else
// Use angles as states
              phi_d = der(phi);
              phi_dd = der(phi_d);
              R_rel = Frames.axesRotations(sequence_angleStates, phi, phi_d);
              w_rel_b = Frames.angularVelocity2(R_rel);
// Dummies
              Q = zeros(4);
            end if;
          else
// Free motion joint does not have states
            if w_rel_a_fixed or z_rel_a_fixed then
              w_rel_b = Frames.angularVelocity2(frame_b.R) - Frames.resolve2(frame_b.R, Frames.angularVelocity1(frame_a.R));
            else
// dummy
              w_rel_b = zeros(3);
            end if;
// Dummies
            R_rel = Frames.nullRotation();
            R_rel_inv = Frames.nullRotation();
            Q = zeros(4);
            phi = zeros(3);
            phi_d = zeros(3);
            phi_dd = zeros(3);
          end if;
          annotation(
            Documentation(info = "<HTML>
<p>
Joint which does not constrain the motion between frame_a and frame_b.
Such a joint is only meaningful if the <b>relative</b> distance and orientation
between frame_a and frame_b, and their derivatives, shall be used
as <b>states</b>.
</p>
<p>
Note, that <b>bodies</b> such as Parts.Body, Parts.BodyShape,
have potential states describing the distance
and orientation, and their derivatives, between the <b>world frame</b> and
a <b>body fixed frame</b>.
Therefore, if these potential state variables are suited,
a FreeMotion joint is not needed.
</p>
<p>
The states of the FreeMotion object are:
</p>
<ul>
<li> The <b>relative position vector</b> r_rel_a from the origin of
     frame_a to the origin of frame_b, resolved in
     frame_a and the <b>relative velocity</b> v_rel_a of the origin of
     frame_b with respect to the origin of frame_a, resolved in frame_a
     (= der(r_rel_a)).
</li>
<li> If parameter <b>useQuaternions</b> in the \"Advanced\" menu
     is <b>true</b> (this is the default), then <b>4 quaternions</b>
     are states. Additionally, the coordinates of the
     relative angular velocity vector are 3 potential states.<br>
     If <b>useQuaternions</b> in the \"Advanced\" menu
     is <b>false</b>, then <b>3 angles</b> and the derivatives of
     these angles are potential states. The orientation of frame_b
     is computed by rotating frame_a along the axes defined
     in parameter vector \"sequence_angleStates\" (default = {1,2,3}, i.e.,
     the Cardan angle sequence) around the angles used as states.
     For example, the default is to rotate the x-axis of frame_a
     around angles[1], the new y-axis around angles[2] and the new z-axis
     around angles[3], arriving at frame_b.
 </li>
</ul>
<p>
The quaternions have the slight disadvantage that there is a
non-linear constraint equation between the 4 quaternions.
Therefore, at least one non-linear equation has to be solved
during simulation. A tool might, however, analytically solve this
simple constraint equation. Using the 3 angles as states has the
disadvantage that there is a singular configuration in which a
division by zero will occur. If it is possible to determine in advance
for an application class that this singular configuration is outside
of the operating region, the 3 angles might be used as
states by setting <b>useQuaternions</b> = <b>false</b>.
</p>
<p>
In text books about 3-dimensional mechanics often 3 angles and the
angular velocity are used as states. This is not the case here, since
3 angles and their derivatives are used as states
(if useQuaternions = false). The reason
is that for real-time simulation the discretization formula of the
integrator might be \"inlined\" and solved together with the model equations.
By appropriate symbolic transformation the performance is
drastically increased if angles and their
derivatives are used as states, instead of angles and the angular
velocity.
</p>
<p>
If parameter
<b>enforceStates</b> is set to <b>true</b> (= the default)
in the \"Advanced\" menu,
then FreeMotion variables are forced to be used as states according
to the setting of parameters \"useQuaternions\" and
\"sequence_angleStates\".
</p>
<p>
In the following figure the animation of a FreeMotion
joint is shown. The light blue coordinate system is
frame_a and the dark blue coordinate system is
frame_b of the joint.
(here: r_rel_a_start = {0.5, 0, 0.5}, angles_start = {45, 45, 45}<sup>o</sup>).
</p>

<p>
<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Joints/FreeMotion.png\">
</p>

</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-86, 31}, {-74, 61}, {-49, 83}, {-17, 92}, {19, 88}, {40, 69}, {59, 48}}, color = {160, 160, 164}, thickness = 0.5, smooth = Smooth.Bezier), Polygon(points = {{90, 0}, {50, 20}, {50, -20}, {90, 0}}, lineColor = {0, 0, 0}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Polygon(points = {{69, 58}, {49, 40}, {77, 28}, {69, 58}}, lineColor = {0, 0, 0}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Text(extent = {{150, -35}, {-150, -75}}, lineColor = {0, 0, 255}, textString = "%name"), Rectangle(extent = {{-70, -5}, {-90, 5}}, lineColor = {0, 0, 0}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Rectangle(extent = {{50, -5}, {30, 5}}, lineColor = {0, 0, 0}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Rectangle(extent = {{11, -5}, {-9, 5}}, lineColor = {0, 0, 0}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-30, -5}, {-50, 5}}, lineColor = {0, 0, 0}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid)}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-86, 31}, {-74, 61}, {-49, 83}, {-17, 92}, {19, 88}, {40, 69}, {59, 48}}, color = {160, 160, 164}, thickness = 0.5, smooth = Smooth.Bezier), Polygon(points = {{90, 0}, {50, 20}, {50, -20}, {90, 0}}, lineColor = {0, 0, 0}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Polygon(points = {{69, 58}, {49, 40}, {77, 28}, {69, 58}}, lineColor = {0, 0, 0}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Rectangle(extent = {{50, -5}, {30, 5}}, lineColor = {0, 0, 0}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Rectangle(extent = {{11, -5}, {-9, 5}}, lineColor = {0, 0, 0}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-30, -5}, {-50, 5}}, lineColor = {0, 0, 0}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-70, -5}, {-90, 5}}, lineColor = {0, 0, 0}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid)}));
        end FreeMotion;

        model FreeMotionScalarInit "Free motion joint with scalar initialization and state selection (6 degrees-of-freedom, 12 potential states)"
          extends Modelica.Mechanics.MultiBody.Interfaces.PartialTwoFrames;
          parameter Boolean animation = true "= true, if animation shall be enabled (show arrow from frame_a to frame_b)" annotation(
            Dialog(enable = use_r));
          parameter Boolean use_r = false "= true, if r_rel_a shall be used" annotation(
            Evaluate = true,
            HideResult = true,
            Dialog(tab = "Translational Initialization", group = "Position vector r_rel_a from origin of frame_a to origin of frame_b, resolved in frame_a"));
          Modelica.Blocks.Interfaces.RealOutput r_rel_a_1(final quantity = "Length", final unit = "m", start = 0, final stateSelect = r_rel_a_1_stateSelect) if use_r "Relative distance r_rel_a[1]" annotation(
            Dialog(enable = use_r, tab = "Translational Initialization", group = "Position vector r_rel_a from origin of frame_a to origin of frame_b, resolved in frame_a", showStartAttribute = true));
          Modelica.Blocks.Interfaces.RealOutput r_rel_a_2(final quantity = "Length", final unit = "m", start = 0, final stateSelect = r_rel_a_2_stateSelect) if use_r "Relative distance r_rel_a[2]" annotation(
            Dialog(enable = use_r, tab = "Translational Initialization", group = "Position vector r_rel_a from origin of frame_a to origin of frame_b, resolved in frame_a", showStartAttribute = true));
          Modelica.Blocks.Interfaces.RealOutput r_rel_a_3(final quantity = "Length", final unit = "m", start = 0, final stateSelect = r_rel_a_3_stateSelect) if use_r "Relative distance r_rel_a[3]" annotation(
            Dialog(enable = use_r, tab = "Translational Initialization", group = "Position vector r_rel_a from origin of frame_a to origin of frame_b, resolved in frame_a", showStartAttribute = true));
          parameter StateSelect r_rel_a_1_stateSelect = StateSelect.never "StateSelect of r_rel_a[1]" annotation(
            HideResult = true,
            Dialog(enable = use_r, tab = "Translational Initialization", group = "Position vector r_rel_a from origin of frame_a to origin of frame_b, resolved in frame_a"));
          parameter StateSelect r_rel_a_2_stateSelect = StateSelect.never "StateSelect of r_rel_a[2]" annotation(
            HideResult = true,
            Dialog(enable = use_r, tab = "Translational Initialization", group = "Position vector r_rel_a from origin of frame_a to origin of frame_b, resolved in frame_a"));
          parameter StateSelect r_rel_a_3_stateSelect = StateSelect.never "StateSelect of r_rel_a[3]" annotation(
            HideResult = true,
            Dialog(enable = use_r, tab = "Translational Initialization", group = "Position vector r_rel_a from origin of frame_a to origin of frame_b, resolved in frame_a"));
          parameter Boolean use_v = false "= true, if v_rel_a shall be used" annotation(
            Evaluate = true,
            HideResult = true,
            Dialog(enable = use_r, tab = "Translational Initialization", group = "Velocity vector v_rel_a = der(r_rel_a)"));
          Modelica.Blocks.Interfaces.RealOutput v_rel_a_1(final quantity = "Velocity", final unit = "m/s", start = 0, final stateSelect = v_rel_a_1_stateSelect) if use_r and use_v "Relative velocity v_rel_a[1]" annotation(
            Dialog(enable = use_r and use_v, tab = "Translational Initialization", group = "Velocity vector v_rel_a = der(r_rel_a)", showStartAttribute = true));
          Modelica.Blocks.Interfaces.RealOutput v_rel_a_2(final quantity = "Velocity", final unit = "m/s", start = 0, final stateSelect = v_rel_a_2_stateSelect) if use_r and use_v "Relative velocity v_rel_a[2]" annotation(
            Dialog(enable = use_r and use_v, tab = "Translational Initialization", group = "Velocity vector v_rel_a = der(r_rel_a)", showStartAttribute = true));
          Modelica.Blocks.Interfaces.RealOutput v_rel_a_3(final quantity = "Velocity", final unit = "m/s", start = 0, final stateSelect = v_rel_a_3_stateSelect) if use_r and use_v "Relative velocity v_rel_a[3]" annotation(
            Dialog(enable = use_r and use_v, tab = "Translational Initialization", group = "Velocity vector v_rel_a = der(r_rel_a)", showStartAttribute = true));
          parameter StateSelect v_rel_a_1_stateSelect = StateSelect.never "StateSelect of v_rel_a[1]" annotation(
            HideResult = true,
            Dialog(enable = use_r and use_v, tab = "Translational Initialization", group = "Velocity vector v_rel_a = der(r_rel_a)"));
          parameter StateSelect v_rel_a_2_stateSelect = StateSelect.never "StateSelect of v_rel_a[2]" annotation(
            HideResult = true,
            Dialog(enable = use_r and use_v, tab = "Translational Initialization", group = "Velocity vector v_rel_a = der(r_rel_a)"));
          parameter StateSelect v_rel_a_3_stateSelect = StateSelect.never "StateSelect of v_rel_a[3]" annotation(
            HideResult = true,
            Dialog(enable = use_r and use_v, tab = "Translational Initialization", group = "Velocity vector v_rel_a = der(r_rel_a)"));
          parameter Boolean use_a = false "= true, if a_rel_a shall be used" annotation(
            Evaluate = true,
            HideResult = true,
            Dialog(enable = use_r and use_v, tab = "Translational Initialization", group = "Acceleration vector a_rel_a = der(v_rel_a)"));
          Modelica.Blocks.Interfaces.RealOutput a_rel_a_1(final quantity = "Acceleration", final unit = "m/s2", start = 0) if use_r and use_v and use_a "Relative acceleration a_rel_a[1]" annotation(
            Dialog(enable = use_r and use_v and use_a, tab = "Translational Initialization", group = "Acceleration vector a_rel_a = der(v_rel_a)", showStartAttribute = true));
          Modelica.Blocks.Interfaces.RealOutput a_rel_a_2(final quantity = "Acceleration", final unit = "m/s2", start = 0) if use_r and use_v and use_a "Relative acceleration a_rel_a[2]" annotation(
            Dialog(enable = use_r and use_v and use_a, tab = "Translational Initialization", group = "Acceleration vector a_rel_a = der(v_rel_a)", showStartAttribute = true));
          Modelica.Blocks.Interfaces.RealOutput a_rel_a_3(final quantity = "Acceleration", final unit = "m/s2", start = 0) if use_r and use_v and use_a "Relative acceleration a_rel_a[3]" annotation(
            Dialog(enable = use_r and use_v and use_a, tab = "Translational Initialization", group = "Acceleration vector a_rel_a = der(v_rel_a)", showStartAttribute = true));
          parameter Boolean use_angle = false "= true, if angle shall be used" annotation(
            Evaluate = true,
            HideResult = true,
            Dialog(tab = "Angle Initialization", group = "Angles to rotate frame_a to frame_b along sequence_start"));
          parameter Types.RotationSequence sequence_start = {1, 2, 3} "Sequence of angle rotations" annotation(
            Evaluate = true,
            Dialog(enable = use_angle, tab = "Angle Initialization", group = "Angles to rotate frame_a to frame_b along sequence_start"));
          Modelica.Blocks.Interfaces.RealOutput angle_1(final quantity = "Angle", final unit = "rad", start = 0, stateSelect = angle_1_stateSelect) if use_angle "First rotation angle or dummy" annotation(
            Dialog(enable = use_angle, tab = "Angle Initialization", group = "Angles to rotate frame_a to frame_b along sequence_start", showStartAttribute = true));
          Modelica.Blocks.Interfaces.RealOutput angle_2(final quantity = "Angle", final unit = "rad", start = 0, stateSelect = angle_2_stateSelect) if use_angle "Second rotation angle or dummy" annotation(
            Dialog(enable = use_angle, tab = "Angle Initialization", group = "Angles to rotate frame_a to frame_b along sequence_start", showStartAttribute = true));
          Modelica.Blocks.Interfaces.RealOutput angle_3(final quantity = "Angle", final unit = "rad", start = 0, stateSelect = angle_3_stateSelect) if use_angle "Third rotation angle or dummy" annotation(
            Dialog(enable = use_angle, tab = "Angle Initialization", group = "Angles to rotate frame_a to frame_b along sequence_start", showStartAttribute = true));
          parameter StateSelect angle_1_stateSelect = StateSelect.never "StateSelect of angle_1" annotation(
            HideResult = true,
            Dialog(enable = use_angle, tab = "Angle Initialization", group = "Angles to rotate frame_a to frame_b along sequence_start"));
          parameter StateSelect angle_2_stateSelect = StateSelect.never "StateSelect of angle_2" annotation(
            HideResult = true,
            Dialog(enable = use_angle, tab = "Angle Initialization", group = "Angles to rotate frame_a to frame_b along sequence_start"));
          parameter StateSelect angle_3_stateSelect = StateSelect.never "StateSelect of angle_3" annotation(
            HideResult = true,
            Dialog(enable = use_angle, tab = "Angle Initialization", group = "Angles to rotate frame_a to frame_b along sequence_start"));
          parameter Boolean use_angle_d = false "= true, if angle_d shall be used" annotation(
            Evaluate = true,
            HideResult = true,
            Dialog(enable = use_angle, tab = "Angle Initialization", group = "angle_d = der(angle)"));
          Modelica.Blocks.Interfaces.RealOutput angle_d_1(final quantity = "AngularVelocity", final unit = "rad/s", start = 0, final stateSelect = angle_d_1_stateSelect) if use_angle and use_angle_d "= der(angle_1)" annotation(
            Dialog(enable = use_angle and use_angle_d, tab = "Angle Initialization", group = "angle_d = der(angle)", showStartAttribute = true));
          Modelica.Blocks.Interfaces.RealOutput angle_d_2(final quantity = "AngularVelocity", final unit = "rad/s", start = 0, final stateSelect = angle_d_2_stateSelect) if use_angle and use_angle_d "= der(angle_2)" annotation(
            Dialog(enable = use_angle and use_angle_d, tab = "Angle Initialization", group = "angle_d = der(angle)", showStartAttribute = true));
          Modelica.Blocks.Interfaces.RealOutput angle_d_3(final quantity = "AngularVelocity", final unit = "rad/s", start = 0, final stateSelect = angle_d_3_stateSelect) if use_angle and use_angle_d "= der(angle_3)" annotation(
            Dialog(enable = use_angle and use_angle_d, tab = "Angle Initialization", group = "angle_d = der(angle)", showStartAttribute = true));
          parameter StateSelect angle_d_1_stateSelect = StateSelect.never "StateSelect of angle_d_1" annotation(
            HideResult = true,
            Dialog(enable = use_angle and use_angle_d, tab = "Angle Initialization", group = "angle_d = der(angle)"));
          parameter StateSelect angle_d_2_stateSelect = StateSelect.never "StateSelect of angle_d_2" annotation(
            HideResult = true,
            Dialog(enable = use_angle and use_angle_d, tab = "Angle Initialization", group = "angle_d = der(angle)"));
          parameter StateSelect angle_d_3_stateSelect = StateSelect.never "StateSelect of angle_d_3" annotation(
            HideResult = true,
            Dialog(enable = use_angle and use_angle_d, tab = "Angle Initialization", group = "angle_d = der(angle)"));
          parameter Boolean use_angle_dd = false "= true, if angle_dd shall be used" annotation(
            Evaluate = true,
            HideResult = true,
            Dialog(enable = use_angle and use_angle_d, tab = "Angle Initialization", group = "angle_dd = der(angle_d)"));
          Modelica.Blocks.Interfaces.RealOutput angle_dd_1(final quantity = "AngularAcceleration", final unit = "rad/s2", start = 0) if use_angle and use_angle_d and use_angle_dd "= der(angle_d_1)" annotation(
            Dialog(enable = use_angle and use_angle_d and use_angle_dd, tab = "Angle Initialization", group = "angle_dd = der(angle_d)", showStartAttribute = true));
          Modelica.Blocks.Interfaces.RealOutput angle_dd_2(final quantity = "AngularAcceleration", final unit = "rad/s2", start = 0) if use_angle and use_angle_d and use_angle_dd "= der(angle_d_2)" annotation(
            Dialog(enable = use_angle and use_angle_d and use_angle_dd, tab = "Angle Initialization", group = "angle_dd = der(angle_d)", showStartAttribute = true));
          Modelica.Blocks.Interfaces.RealOutput angle_dd_3(final quantity = "AngularAcceleration", final unit = "rad/s2", start = 0) if use_angle and use_angle_d and use_angle_dd "= der(angle_d_3)" annotation(
            Dialog(enable = use_angle and use_angle_d and use_angle_dd, tab = "Angle Initialization", group = "angle_dd = der(angle_d)", showStartAttribute = true));
          parameter Boolean use_w = false "= true, if w_rel_b shall be used" annotation(
            Evaluate = true,
            HideResult = true,
            Dialog(tab = "Angular Velocity Initialization", group = "Angular velocity w_rel_b of frame_b with respect to frame_a, resolved in frame_b"));
          Modelica.Blocks.Interfaces.RealOutput w_rel_b_1(final quantity = "AngularVelocity", final unit = "rad/s", start = 0, stateSelect = w_rel_b_1_stateSelect) if use_w "Relative angular velocity w_rel_b[1]" annotation(
            Dialog(enable = use_w, tab = "Angular Velocity Initialization", group = "Angular velocity w_rel_b of frame_b with respect to frame_a, resolved in frame_b", showStartAttribute = true));
          Modelica.Blocks.Interfaces.RealOutput w_rel_b_2(final quantity = "AngularVelocity", final unit = "rad/s", start = 0, stateSelect = w_rel_b_2_stateSelect) if use_w "Relative angular velocity w_rel_b[2]" annotation(
            Dialog(enable = use_w, tab = "Angular Velocity Initialization", group = "Angular velocity w_rel_b of frame_b with respect to frame_a, resolved in frame_b", showStartAttribute = true));
          Modelica.Blocks.Interfaces.RealOutput w_rel_b_3(final quantity = "AngularVelocity", final unit = "rad/s", start = 0, stateSelect = w_rel_b_3_stateSelect) if use_w "Relative angular velocity w_rel_b[3]" annotation(
            Dialog(enable = use_w, tab = "Angular Velocity Initialization", group = "Angular velocity w_rel_b of frame_b with respect to frame_a, resolved in frame_b", showStartAttribute = true));
          parameter StateSelect w_rel_b_1_stateSelect = StateSelect.never "StateSelect of w_rel_b[1]" annotation(
            HideResult = true,
            Dialog(enable = use_w, tab = "Angular Velocity Initialization", group = "Angular velocity w_rel_b of frame_b with respect to frame_a, resolved in frame_b"));
          parameter StateSelect w_rel_b_2_stateSelect = StateSelect.never "StateSelect of w_rel_b[2]" annotation(
            HideResult = true,
            Dialog(enable = use_w, tab = "Angular Velocity Initialization", group = "Angular velocity w_rel_b of frame_b with respect to frame_a, resolved in frame_b"));
          parameter StateSelect w_rel_b_3_stateSelect = StateSelect.never "StateSelect of w_rel_b[3]" annotation(
            HideResult = true,
            Dialog(enable = use_w, tab = "Angular Velocity Initialization", group = "Angular velocity w_rel_b of frame_b with respect to frame_a, resolved in frame_b"));
          parameter Boolean use_z = false "= true, if z_rel_b shall be used" annotation(
            Evaluate = true,
            HideResult = true,
            Dialog(enable = use_w, tab = "Angular Velocity Initialization", group = "Angular acceleration z_rel_b = der(w_rel_b)"));
          Modelica.Blocks.Interfaces.RealOutput z_rel_b_1(final quantity = "AngularAcceleration", final unit = "rad/s2", start = 0) if use_w and use_z "Relative angular acceleration z_rel_b[1]" annotation(
            Dialog(enable = use_w and use_z, tab = "Angular Velocity Initialization", group = "Angular acceleration z_rel_b = der(w_rel_b)", showStartAttribute = true));
          Modelica.Blocks.Interfaces.RealOutput z_rel_b_2(final quantity = "AngularAcceleration", final unit = "rad/s2", start = 0) if use_w and use_z "Relative angular acceleration z_rel_b[2]" annotation(
            Dialog(enable = use_w and use_z, tab = "Angular Velocity Initialization", group = "Angular acceleration z_rel_b = der(w_rel_b)", showStartAttribute = true));
          Modelica.Blocks.Interfaces.RealOutput z_rel_b_3(final quantity = "AngularAcceleration", final unit = "rad/s2", start = 0) if use_w and use_z "Relative angular acceleration z_rel_b[3]" annotation(
            Dialog(enable = use_w and use_z, tab = "Angular Velocity Initialization", group = "Angular acceleration z_rel_b = der(w_rel_b)", showStartAttribute = true));
          parameter SI.Length arrowDiameter = world.defaultArrowDiameter "Diameter of arrow from frame_a to frame_b" annotation(
            Dialog(tab = "Animation", group = "if animation = true", enable = animation and use_r));
          input Types.Color arrowColor = Modelica.Mechanics.MultiBody.Types.Defaults.SensorColor "Color of arrow" annotation(
            Dialog(colorSelector = true, tab = "Animation", group = "if animation = true", enable = animation and use_r));
          input Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(
            Dialog(tab = "Animation", group = "if animation = true", enable = animation and use_r));
        protected
          Modelica.Mechanics.MultiBody.Joints.Internal.InitPosition initPosition(r_a_0 = frame_a.r_0, r_b_0 = frame_b.r_0, R_a = frame_a.R) if use_r annotation(
            Placement(transformation(extent = {{-20, 60}, {0, 80}})));
          Modelica.Mechanics.MultiBody.Joints.Internal.InitAngle initAngle(sequence_start = sequence_start) if use_angle annotation(
            Placement(transformation(extent = {{-60, -10}, {-40, 10}})));
          Modelica.Mechanics.MultiBody.Joints.Internal.InitAngularVelocity initAngularVelocity(R_a = frame_a.R, R_b = frame_b.R) if use_w annotation(
            Placement(transformation(extent = {{-20, 20}, {0, 40}})));
          Modelica.Blocks.Continuous.Der derv[3] if use_r and use_v annotation(
            Placement(transformation(extent = {{20, 60}, {40, 80}})));
          Modelica.Blocks.Continuous.Der dera[3] if use_r and use_v and use_a annotation(
            Placement(transformation(extent = {{60, 60}, {80, 80}})));
          Modelica.Blocks.Continuous.Der derd[3] if use_angle and use_angle_d annotation(
            Placement(transformation(extent = {{-20, -30}, {0, -10}})));
          Modelica.Blocks.Continuous.Der derdd[3] if use_angle and use_angle_d and use_angle_dd annotation(
            Placement(transformation(extent = {{20, -30}, {40, -10}})));
          Modelica.Blocks.Continuous.Der derz[3] if use_w and use_z annotation(
            Placement(transformation(extent = {{20, 20}, {40, 40}})));
          Modelica.Mechanics.MultiBody.Sensors.Internal.ZeroForceAndTorque zeroForceAndTorque1 annotation(
            Placement(transformation(extent = {{-80, -50}, {-60, -30}})));
          Modelica.Mechanics.MultiBody.Sensors.Internal.ZeroForceAndTorque zeroForceAndTorque2 annotation(
            Placement(transformation(extent = {{80, -50}, {60, -30}})));
          Modelica.Mechanics.MultiBody.Visualizers.SignalArrow arrow(diameter = arrowDiameter, color = arrowColor, specularCoefficient = specularCoefficient) if world.enableAnimation and animation and use_r annotation(
            Placement(transformation(extent = {{-80, 60}, {-60, 80}})));
        equation
// r_rel_a
          connect(initPosition.r_rel_a[1], r_rel_a_1);
          connect(initPosition.r_rel_a[2], r_rel_a_2);
          connect(initPosition.r_rel_a[3], r_rel_a_3);
// v_rel_a
          connect(derv[1].y, v_rel_a_1);
          connect(derv[2].y, v_rel_a_2);
          connect(derv[3].y, v_rel_a_3);
// a_rel_a
          connect(dera[1].y, a_rel_a_1);
          connect(dera[2].y, a_rel_a_2);
          connect(dera[3].y, a_rel_a_3);
// angle
          connect(initAngle.angle[1], angle_1);
          connect(initAngle.angle[2], angle_2);
          connect(initAngle.angle[3], angle_3);
// angle_d
          connect(derd[1].y, angle_d_1);
          connect(derd[2].y, angle_d_2);
          connect(derd[3].y, angle_d_3);
// angle_dd
          connect(derdd[1].y, angle_dd_1);
          connect(derdd[2].y, angle_dd_2);
          connect(derdd[3].y, angle_dd_3);
// w_rel_b
          connect(initAngularVelocity.w_rel_b[1], w_rel_b_1);
          connect(initAngularVelocity.w_rel_b[2], w_rel_b_2);
          connect(initAngularVelocity.w_rel_b[3], w_rel_b_3);
// z_rel_b
          connect(derz[1].y, z_rel_b_1);
          connect(derz[2].y, z_rel_b_2);
          connect(derz[3].y, z_rel_b_3);
          connect(initPosition.r_rel_a, derv.u) annotation(
            Line(points = {{1, 70}, {18, 70}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(derv.y, dera.u) annotation(
            Line(points = {{41, 70}, {58, 70}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(initAngle.frame_a, frame_a) annotation(
            Line(points = {{-60, 0}, {-100, 0}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
          connect(initAngle.frame_b, frame_b) annotation(
            Line(points = {{-40, 0}, {100, 0}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
          connect(initAngle.angle, derd.u) annotation(
            Line(points = {{-50, -11}, {-50, -20}, {-22, -20}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(derd.y, derdd.u) annotation(
            Line(points = {{1, -20}, {18, -20}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(zeroForceAndTorque1.frame_a, frame_a) annotation(
            Line(points = {{-80, -40}, {-88, -40}, {-88, 0}, {-100, 0}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
          connect(zeroForceAndTorque2.frame_a, frame_b) annotation(
            Line(points = {{80, -40}, {90, -40}, {90, 0}, {100, 0}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
          connect(initAngularVelocity.w_rel_b, derz.u) annotation(
            Line(points = {{1, 30}, {18, 30}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(frame_a, arrow.frame_a) annotation(
            Line(points = {{-100, 0}, {-88, 0}, {-88, 70}, {-80, 70}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
          connect(initPosition.r_rel_a, arrow.r_head) annotation(
            Line(points = {{1, 70}, {10, 70}, {10, 52}, {-70, 52}, {-70, 58}}, color = {0, 0, 127}, smooth = Smooth.None));
          annotation(
            Documentation(info = "<HTML>
<p>
Joint which does not constrain the motion between frame_a and frame_b.
Such a joint is meaningful if the <b>relative</b> distance and orientation
between frame_a and frame_b, and their derivatives, shall be used
as <b>states</b> or shall be used for non-standard
<b>initialization</b>. This joint allows to <b>initialize</b>
every <b>scalar</b> element of the relative quantities, as well
as to define <b>StateSelect</b> attributes for every
<b>scalar</b> element separately.
</p>

<p>
In the following figure the animation of a FreeMotionScalarInit
joint is shown. The light blue coordinate system is
frame_a and the dark blue coordinate system is
frame_b of the joint.
(here: r_rel_a_1(start = 0.5), r_rel_a_2(start = 0), r_rel_a_3(start = 0.5),
       angle_1(start = 45<sup>o</sup>), angle_2(start = 45<sup>o</sup>), angle_3(start = 45<sup>o</sup>)).
</p>

<p>
<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Joints/FreeMotion.png\">
</p>

<p>
A example to use this joint for the initialization of a planar double pendulum by providing
its tip position, is shown in
<a href=\"modelica://Modelica.Mechanics.MultiBody.Examples.Elementary.DoublePendulumInitTip\">Examples.Elementary.DoublePendulumInitTip</a>.
</p>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-86, 31}, {-74, 61}, {-49, 83}, {-17, 92}, {19, 88}, {40, 69}, {59, 48}}, color = {160, 160, 164}, thickness = 0.5, smooth = Smooth.Bezier), Polygon(points = {{90, 0}, {50, 20}, {50, -20}, {90, 0}}, lineColor = {0, 0, 0}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Polygon(points = {{69, 58}, {49, 40}, {77, 28}, {69, 58}}, lineColor = {0, 0, 0}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Text(extent = {{150, -44}, {-150, -84}}, lineColor = {0, 0, 255}, textString = "%name"), Rectangle(extent = {{-70, -5}, {-90, 5}}, lineColor = {0, 0, 0}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Rectangle(extent = {{50, -5}, {30, 5}}, lineColor = {0, 0, 0}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Rectangle(extent = {{11, -5}, {-9, 5}}, lineColor = {0, 0, 0}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-30, -5}, {-50, 5}}, lineColor = {0, 0, 0}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid)}));
        end FreeMotionScalarInit;

        model SphericalSpherical "Spherical - spherical joint aggregation (1 constraint, no potential states) with an optional point mass in the middle"
          import Modelica.Mechanics.MultiBody.Types;
          extends Interfaces.PartialTwoFrames;
          parameter Boolean animation = true "= true, if animation shall be enabled";
          parameter Boolean showMass = true "= true, if mass shall be shown (provided animation = true and m > 0)";
          parameter Boolean computeRodLength = false "= true, if rodLength shall be computed during initialization (see info)";
          parameter SI.Length rodLength(min = Modelica.Constants.eps, fixed = not computeRodLength, start = 1) "Distance between the origins of frame_a and frame_b (if computeRodLength=true, guess value)";
          parameter SI.Mass m(min = 0) = 0 "Mass of rod (= point mass located in middle of rod)";
          parameter SI.Diameter sphereDiameter = world.defaultJointLength "Diameter of spheres representing the spherical joints" annotation(
            Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          input Types.Color sphereColor = Modelica.Mechanics.MultiBody.Types.Defaults.JointColor "Color of spheres representing the spherical joints" annotation(
            Dialog(colorSelector = true, tab = "Animation", group = "if animation = true", enable = animation));
          parameter SI.Diameter rodDiameter = sphereDiameter / Types.Defaults.JointRodDiameterFraction "Diameter of rod connecting the two spherical joint" annotation(
            Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          input Types.Color rodColor = Modelica.Mechanics.MultiBody.Types.Defaults.RodColor "Color of rod connecting the two spherical joints" annotation(
            Dialog(colorSelector = true, tab = "Animation", group = "if animation = true", enable = animation));
          parameter SI.Diameter massDiameter = sphereDiameter "Diameter of sphere representing the mass point" annotation(
            Dialog(tab = "Animation", group = "if animation = true and showMass = true and m > 0", enable = animation and showMass and m > 0));
          input Types.Color massColor = Modelica.Mechanics.MultiBody.Types.Defaults.BodyColor "Color of sphere representing the mass point" annotation(
            Dialog(colorSelector = true, tab = "Animation", group = "if animation = true and showMass = true and m > 0", enable = animation and showMass and m > 0));
          input Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(
            Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter Boolean kinematicConstraint = true "= false, if no constraint shall be defined, due to analytically solving a kinematic loop (\"false\" should not be used by user, but only by MultiBody.Joints.Assemblies joints)" annotation(
            Dialog(tab = "Advanced"));
          Real constraintResidue = rRod_0 * rRod_0 - rodLength * rodLength "Constraint equation of joint in residue form: Either length constraint (= default) or equation to compute rod force (for analytic solution of loops in combination with Internal.RevoluteWithLengthConstraint/PrismaticWithLengthConstraint)" annotation(
            Dialog(tab = "Advanced", enable = not kinematicConstraint));
          parameter Boolean checkTotalPower = false "= true, if total power flowing into this component shall be determined (must be zero)" annotation(
            Dialog(tab = "Advanced"));
          SI.Force f_rod "Constraint force in direction of the rod (positive on frame_a, when directed from frame_a to frame_b)";
          SI.Position rRod_0[3] "Position vector from frame_a to frame_b resolved in world frame";
          SI.Position rRod_a[3] "Position vector from frame_a to frame_b resolved in frame_a";
          Real eRod_a[3](each final unit = "1") "Unit vector in direction from frame_a to frame_b, resolved in frame_a";
          SI.Position r_CM_0[3] "Dummy if m==0, or position vector from world frame to mid-point of rod, resolved in world frame";
          SI.Velocity v_CM_0[3] "First derivative of r_CM_0";
          SI.Force f_CM_a[3] "Dummy if m==0, or inertial force acting at mid-point of rod due to mass oint acceleration, resolved in frame_a";
          SI.Force f_CM_e[3] "Dummy if m==0, or projection of f_CM_a onto eRod_a, resolved in frame_a";
          SI.Force f_b_a1[3] "Force acting at frame_b, but without force in rod, resolved in frame_a";
          SI.Power totalPower "Total power flowing into this element, if checkTotalPower=true (otherwise dummy)";
        protected
          Visualizers.Advanced.Shape shape_rod(shapeType = "cylinder", color = rodColor, specularCoefficient = specularCoefficient, length = rodLength, width = rodDiameter, height = rodDiameter, lengthDirection = eRod_a, widthDirection = {0, 1, 0}, r = frame_a.r_0, R = frame_a.R) if world.enableAnimation and animation;
          Visualizers.Advanced.Shape shape_a(shapeType = "sphere", color = sphereColor, specularCoefficient = specularCoefficient, length = sphereDiameter, width = sphereDiameter, height = sphereDiameter, lengthDirection = eRod_a, widthDirection = {0, 1, 0}, r_shape = -eRod_a * (sphereDiameter / 2), r = frame_a.r_0, R = frame_a.R) if world.enableAnimation and animation;
          Visualizers.Advanced.Shape shape_b(shapeType = "sphere", color = sphereColor, specularCoefficient = specularCoefficient, length = sphereDiameter, width = sphereDiameter, height = sphereDiameter, lengthDirection = eRod_a, widthDirection = {0, 1, 0}, r_shape = eRod_a * (rodLength - sphereDiameter / 2), r = frame_a.r_0, R = frame_a.R) if world.enableAnimation and animation;
          Visualizers.Advanced.Shape shape_mass(shapeType = "sphere", color = massColor, specularCoefficient = specularCoefficient, length = massDiameter, width = massDiameter, height = massDiameter, lengthDirection = eRod_a, widthDirection = {0, 1, 0}, r_shape = eRod_a * (rodLength / 2 - sphereDiameter / 2), r = frame_a.r_0, R = frame_a.R) if world.enableAnimation and animation and showMass and m > 0;
        equation
// Determine relative position vector between the two frames
          if kinematicConstraint then
            rRod_0 = transpose(frame_b.R.T) * (frame_b.R.T * frame_b.r_0) - transpose(frame_a.R.T) * (frame_a.R.T * frame_a.r_0);
          else
            rRod_0 = frame_b.r_0 - frame_a.r_0;
          end if;
//rRod_0 = frame_b.r_0 - frame_a.r_0;
          rRod_a = Frames.resolve2(frame_a.R, rRod_0);
          eRod_a = rRod_a / rodLength;
// Constraint equation
          constraintResidue = 0;
// Cut-torques at frame_a and frame_b
          frame_a.t = zeros(3);
          frame_b.t = zeros(3);
/* Force and torque balance of rod
     - Kinematics for center of mass CM of mass point
       r_CM_0 = frame_a.r_0 + rRod_0/2;
       v_CM_0 = der(r_CM_0);
       a_CM_a = resolve2(frame_a.R, der(v_CM_0) - world.gravityAcceleration(r_CM_0));
     - Inertial and gravity force in direction (f_CM_e) and orthogonal (f_CM_n) to rod
       f_CM_a = m*a_CM_a
       f_CM_e = f_CM_a*eRod_a;           // in direction of rod
       f_CM_n = rodLength(f_CM_a - f_CM_e);  // orthogonal to rod
     - Force balance in direction of rod
       f_CM_e = fa_rod_e + fb_rod_e;
     - Force balance orthogonal to rod
       f_CM_n = fa_rod_n + fb_rod_n;
     - Torque balance with respect to frame_a
       0 = (-f_CM_n)*rodLength/2 + fb_rod_n*rodLength
     The result is:
     fb_rod_n = f_CM_n/2;
     fa_rod_n = fb_rod_n;
     fb_rod_e = f_CM_e - fa_rod_e;
     fa_rod_e is the unknown computed from loop
  */
// f_b_a1 is needed in aggregation joints to solve kinematic loops analytically
          if m > 0 then
            r_CM_0 = frame_a.r_0 + rRod_0 / 2;
            v_CM_0 = der(r_CM_0);
            f_CM_a = m * Frames.resolve2(frame_a.R, der(v_CM_0) - world.gravityAcceleration(r_CM_0));
            f_CM_e = f_CM_a * eRod_a * eRod_a;
            frame_a.f = (f_CM_a - f_CM_e) / 2 + f_rod * eRod_a;
            f_b_a1 = (f_CM_a + f_CM_e) / 2;
            frame_b.f = Frames.resolveRelative(f_b_a1 - f_rod * eRod_a, frame_a.R, frame_b.R);
          else
            r_CM_0 = zeros(3);
            v_CM_0 = zeros(3);
            f_CM_a = zeros(3);
            f_CM_e = zeros(3);
            f_b_a1 = zeros(3);
            frame_a.f = f_rod * eRod_a;
            frame_b.f = -Frames.resolveRelative(frame_a.f, frame_a.R, frame_b.R);
          end if;
          if checkTotalPower then
            totalPower = frame_a.f * Frames.resolve2(frame_a.R, der(frame_a.r_0)) + frame_b.f * Frames.resolve2(frame_b.R, der(frame_b.r_0)) + (-m) * (der(v_CM_0) - world.gravityAcceleration(r_CM_0)) * v_CM_0 + frame_a.t * Frames.angularVelocity2(frame_a.R) + frame_b.t * Frames.angularVelocity2(frame_b.R);
          else
            totalPower = 0;
          end if;
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-95, -40}, {-15, 40}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.Sphere, fillColor = {192, 192, 192}), Ellipse(extent = {{-84, -30}, {-24, 30}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Ellipse(extent = {{15, -40}, {95, 40}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.Sphere, fillColor = {192, 192, 192}), Ellipse(extent = {{25, -29}, {85, 30}}, lineColor = {128, 128, 128}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-150, 90}, {150, 50}}, textString = "%name", lineColor = {0, 0, 255}), Rectangle(extent = {{-40, 40}, {41, -41}}, lineColor = {255, 255, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-51, 6}, {48, -4}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192, 192, 192}), Ellipse(extent = {{-68, 15}, {-39, -13}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.Sphere, fillColor = {192, 192, 192}), Ellipse(extent = {{39, 14}, {68, -14}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.Sphere, fillColor = {192, 192, 192}), Text(extent = {{-150, -60}, {150, -90}}, lineColor = {0, 0, 0}, textString = "%rodLength")}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-98, -40}, {-18, 40}}, lineColor = {0, 0, 0}, fillColor = {160, 160, 164}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-88, -30}, {-28, 30}}, lineColor = {160, 160, 164}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Ellipse(extent = {{18, -40}, {98, 40}}, lineColor = {160, 160, 164}, fillColor = {160, 160, 164}, fillPattern = FillPattern.Solid), Ellipse(extent = {{29, -30}, {89, 29}}, lineColor = {192, 192, 192}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{-56, -60}, {46, -60}}, color = {0, 0, 255}), Polygon(points = {{56, -60}, {41, -54}, {41, -66}, {56, -60}}, lineColor = {0, 0, 255}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-37, -63}, {33, -79}}, textString = "rodLength", lineColor = {0, 0, 255}), Rectangle(extent = {{-40, 41}, {40, -40}}, lineColor = {255, 255, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-51, 6}, {48, -4}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192, 192, 192}), Ellipse(extent = {{-71, 15}, {-42, -13}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Ellipse(extent = {{42, 14}, {71, -14}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Line(points = {{-56, -71}, {-56, 1}}, color = {0, 0, 255}), Line(points = {{56, -72}, {56, 0}}, color = {0, 0, 255}), Polygon(points = {{11, 1}, {-1, 4}, {-1, -2}, {11, 1}}, lineColor = {0, 0, 255}), Line(points = {{-56, 1}, {-1, 1}}, color = {0, 0, 255}), Text(extent = {{-32, -4}, {4, -29}}, lineColor = {0, 0, 0}, textString = "eRod_a")}),
            Documentation(info = "<html>
<p>
Joint that has a spherical joint on each of its two ends.
The rod connecting the two spherical joints is approximated by a
point mass that is located in the middle of the rod. When the mass
is set to zero (default), special code for a massless body is generated.
In the following default animation figure, the two spherical joints are
represented by two red spheres, the connecting rod by a grey cylinder
and the point mass in the middle of the rod by a light blue sphere:
</p>

<p>
<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Joints/SphericalSpherical.png\" ALT=\"model Joints.SphericalSpherical\">
</p>

<p>
This joint introduces <b>one constraint</b> defining that the distance between
the origin of frame_a and the origin of frame_b is constant (= rodLength).
It is highly recommended to use this joint in loops
whenever possible, because this enhances the efficiency
considerably due to smaller systems of non-linear algebraic
equations.
</p>
<p>
It is sometimes desirable to <b>compute</b> the <b>rodLength</b>
of the connecting rod during initialization. For this, parameter
<b>computeLength</b> has to be set to <b>true</b> and instead <b>one</b> other,
easier to determine, position variable in the same loop
needs to have a fixed attribute of <b>true</b>. For example,
if a loop consists of one Revolute joint, one Prismatic joint and
a SphericalSpherical joint, one may fix the start values of the revolute
joint angle and of the relative distance of the prismatic joint
in order to compute the rodLength of the rod.
</p>
<p>
It is not possible to connect other components, such as a body with mass
properties or a special visual shape object to the rod connecting
the two spherical joints. If this is needed, use instead joint Joints.<b>UniversalSpherical</b>
that has this property.
</p>
</html>"));
        end SphericalSpherical;

        model UniversalSpherical "Universal - spherical joint aggregation (1 constraint, no potential states)"
          import Modelica.Mechanics.MultiBody.Types;
          extends Interfaces.PartialTwoFrames;
          Interfaces.Frame_a frame_ia "Coordinate system at the origin of frame_a, fixed at the rod connecting the universal with the spherical joint" annotation(
            Placement(transformation(origin = {-40, 100}, extent = {{-16, -16}, {16, 16}}, rotation = 270)));
          parameter Boolean animation = true "= true, if animation shall be enabled";
          parameter Boolean showUniversalAxes = true "= true, if universal joint shall be visualized with two cylinders, otherwise with a sphere (provided animation=true)";
          parameter Boolean computeRodLength = false "= true, if distance between frame_a and frame_b shall be computed during initialization (see info)";
          parameter Modelica.Mechanics.MultiBody.Types.Axis n1_a = {0, 0, 1} "Axis 1 of universal joint resolved in frame_a (axis 2 is orthogonal to axis 1 and to rod)" annotation(
            Evaluate = true);
          parameter SI.Position rRod_ia[3] = {1, 0, 0} "Vector from origin of frame_a to origin of frame_b, resolved in frame_ia (if computeRodLength=true, rRod_ia is only an axis vector along the connecting rod)" annotation(
            Evaluate = true);
          parameter SI.Diameter sphereDiameter = world.defaultJointLength "Diameter of spheres representing the universal and the spherical joint" annotation(
            Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          input Types.Color sphereColor = Modelica.Mechanics.MultiBody.Types.Defaults.JointColor "Color of spheres representing the universal and the spherical joint" annotation(
            Dialog(colorSelector = true, tab = "Animation", group = "if animation = true", enable = animation));
          parameter Types.ShapeType rodShapeType = "cylinder" "Shape type of rod connecting the universal and the spherical joint" annotation(
            Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter SI.Distance rodWidth = sphereDiameter / Types.Defaults.JointRodDiameterFraction "Width of rod shape in direction of axis 2 of universal joint." annotation(
            Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter SI.Distance rodHeight = rodWidth "Height of rod shape in direction that is orthogonal to rod and to axis 2" annotation(
            Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter Types.ShapeExtra rodExtra = 0.0 "Additional parameter depending on rodShapeType" annotation(
            Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          input Types.Color rodColor = Modelica.Mechanics.MultiBody.Types.Defaults.RodColor "Color of rod shape connecting the universal and the spherical joints" annotation(
            Dialog(colorSelector = true, tab = "Animation", group = "if animation = true", enable = animation));
          parameter SI.Distance cylinderLength = world.defaultJointLength "Length of cylinders representing the two universal joint axes" annotation(
            Dialog(tab = "Animation", group = "if animation = true and showUniversalAxes", enable = animation and showUniversalAxes));
          parameter SI.Distance cylinderDiameter = world.defaultJointWidth "Diameter of cylinders representing the two universal joint axes" annotation(
            Dialog(tab = "Animation", group = "if animation = true and showUniversalAxes", enable = animation and showUniversalAxes));
          input Types.Color cylinderColor = Modelica.Mechanics.MultiBody.Types.Defaults.JointColor "Color of cylinders representing the two universal joint axes" annotation(
            Dialog(colorSelector = true, tab = "Animation", group = "if animation = true and showUniversalAxes", enable = animation and showUniversalAxes));
          input Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(
            Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter Boolean kinematicConstraint = true "= false, if no constraint shall be defined, due to analytically solving a kinematic loop" annotation(
            Dialog(tab = "Advanced"));
          Real constraintResidue = rRod_0 * rRod_0 - rodLength * rodLength "Constraint equation of joint in residue form: Either length constraint (= default) or equation to compute rod force (for analytic solution of loops in combination with Internal.RevoluteWithLengthConstraint/PrismaticWithLengthConstraint)" annotation(
            Dialog(tab = "Advanced", enable = not kinematicConstraint));
          parameter Boolean checkTotalPower = false "= true, if total power flowing into this component shall be determined (must be zero)" annotation(
            Dialog(tab = "Advanced"));
          SI.Force f_rod "Constraint force in direction of the rod (positive, if rod is pressed)";
          final parameter SI.Distance rodLength(fixed = false, start = Modelica.Math.Vectors.length(rRod_ia)) "Length of rod (distance between origin of frame_a and origin of frame_b)";
          final parameter Real eRod_ia[3](each final unit = "1") = Modelica.Math.Vectors.normalizeWithAssert(rRod_ia) "Unit vector from origin of frame_a to origin of frame_b, resolved in frame_ia";
          final parameter Real e2_ia[3](each final unit = "1") = Modelica.Math.Vectors.normalize(cross(n1_a, eRod_ia)) "Unit vector in direction of axis 2 of universal joint, resolved in frame_ia (orthogonal to n1_a and eRod_ia; note: frame_ia is parallel to frame_a when the universal joint angles are zero)";
          final parameter Real e3_ia[3](each final unit = "1") = cross(eRod_ia, e2_ia) "Unit vector perpendicular to eRod_ia and e2_ia, resolved in frame_ia";
          SI.Power totalPower "Total power flowing into this element, if checkTotalPower=true (otherwise dummy)";
          SI.Force f_b_a1[3] "frame_b.f without f_rod part, resolved in frame_a (needed for analytic loop handling)";
          Real eRod_a[3](each final unit = "1") "Unit vector in direction of rRod_a, resolved in frame_a (needed for analytic loop handling)";
          SI.Position rRod_0[3](start = rRod_ia) "Position vector from origin of frame_a to origin of frame_b resolved in world frame";
          SI.Position rRod_a[3](start = rRod_ia) "Position vector from origin of frame_a to origin of frame_b resolved in frame_a";
        protected
          SI.Force f_b_a[3] "frame_b.f resolved in frame_a";
          SI.Force f_ia_a[3] "frame_ia.f resolved in frame_a";
          SI.Torque t_ia_a[3] "frame_ia.t resolved in frame_a";
          Real n2_a[3](each final unit = "1") "Vector in direction of axis 2 of the universal joint (e2_ia), resolved in frame_a";
          Real length2_n2_a(start = 1, unit = "1") "Square of length of vector n2_a";
          Real length_n2_a(unit = "1") "Length of vector n2_a";
          Real e2_a[3](each final unit = "1") "Unit vector in direction of axis 2 of the universal joint (e2_ia), resolved in frame_a";
          Real e3_a[3](each final unit = "1") "Unit vector perpendicular to eRod_ia and e2_a, resolved in frame_a";
          Real der_rRod_a_L[3](each unit = "1/s") "= der(rRod_a)/rodLength";
          SI.AngularVelocity w_rel_ia1[3];
          Frames.Orientation R_rel_ia1;
          Frames.Orientation R_rel_ia2;
          // Real T_rel_ia[3, 3];
          Frames.Orientation R_rel_ia "Rotation from frame_a to frame_ia";
          Visualizers.Advanced.Shape rodShape(shapeType = rodShapeType, color = rodColor, specularCoefficient = specularCoefficient, length = rodLength, width = rodWidth, height = rodHeight, lengthDirection = eRod_ia, widthDirection = e2_ia, r = frame_ia.r_0, R = frame_ia.R) if world.enableAnimation and animation;
          Visualizers.Advanced.Shape sphericalShape_b(shapeType = "sphere", color = sphereColor, specularCoefficient = specularCoefficient, length = sphereDiameter, width = sphereDiameter, height = sphereDiameter, lengthDirection = {1, 0, 0}, widthDirection = {0, 1, 0}, r_shape = {-0.5, 0, 0} * sphereDiameter, r = frame_b.r_0, R = frame_b.R) if world.enableAnimation and animation;
          Visualizers.Advanced.Shape sphericalShape_a(shapeType = "sphere", color = sphereColor, specularCoefficient = specularCoefficient, length = sphereDiameter, width = sphereDiameter, height = sphereDiameter, lengthDirection = {1, 0, 0}, widthDirection = {0, 1, 0}, r_shape = {-0.5, 0, 0} * sphereDiameter, r = frame_a.r_0, R = frame_a.R) if world.enableAnimation and animation and not showUniversalAxes;
          Visualizers.Advanced.Shape universalShape1(shapeType = "cylinder", color = cylinderColor, specularCoefficient = specularCoefficient, length = cylinderLength, width = cylinderDiameter, height = cylinderDiameter, lengthDirection = n1_a, widthDirection = {0, 1, 0}, r_shape = -n1_a * (cylinderLength / 2), r = frame_a.r_0, R = frame_a.R) if world.enableAnimation and animation and showUniversalAxes;
          Visualizers.Advanced.Shape universalShape2(shapeType = "cylinder", color = cylinderColor, specularCoefficient = specularCoefficient, length = cylinderLength, width = cylinderDiameter, height = cylinderDiameter, lengthDirection = e2_ia, widthDirection = {0, 1, 0}, r_shape = -e2_ia * (cylinderLength / 2), r = frame_ia.r_0, R = frame_ia.R) if world.enableAnimation and animation and showUniversalAxes;
        initial equation
          if not computeRodLength then
            rodLength = Modelica.Math.Vectors.length(rRod_ia);
          end if;
        equation
          Connections.branch(frame_a.R, frame_ia.R);
          if kinematicConstraint then
            rRod_0 = transpose(frame_b.R.T) * (frame_b.R.T * frame_b.r_0) - transpose(frame_a.R.T) * (frame_a.R.T * frame_a.r_0);
          else
            rRod_0 = frame_b.r_0 - frame_a.r_0;
          end if;
//rRod_0 = frame_b.r_0 - frame_a.r_0;
          rRod_a = Frames.resolve2(frame_a.R, rRod_0);
// Constraint equation
          constraintResidue = 0;
/* Determine relative Rotation R_rel_ia from frame_a to frame_ia
     and absolute rotation of frame_a.R.
  */
          eRod_a = rRod_a / rodLength;
          n2_a = cross(n1_a, eRod_a);
          length2_n2_a = n2_a * n2_a;
          assert(length2_n2_a > 1.e-10, "
A Modelica.Mechanics.MultiBody.Joints.UniversalSpherical joint (consisting of
a universal joint and a spherical joint connected together
by a rigid rod) is in the singular configuration of the
universal joint. This means that axis 1 of the universal
joint defined via parameter \"n1_a\" is parallel to vector
\"rRod_ia\" that is directed from the origin of frame_a to the
origin of frame_b.
   You may try to use another \"n1_a\" vector. If this fails,
use instead Modelica.Mechanics.MultiBody.Joints.SphericalSpherical, if this is
possible, because this joint aggregation does not have a
singular configuration.
          ");
          length_n2_a = sqrt(length2_n2_a);
          e2_a = n2_a / length_n2_a;
          e3_a = cross(eRod_a, e2_a);
/* The statements below are an efficient implementation of the
   original equations:
     T_rel_ia = [eRod_ia, e2_ia, e3_ia]*transpose([eRod_a, e2_a, e3_a]);
     R_rel_ia = Frames.from_T(T_rel_ia,
                   Frames.TransformationMatrices.angularVelocity2(T_rel_ia, der(T_rel_ia)));
   To perform this, the rotation is split into two parts:
     R_rel_ia : Rotation object from frame_a to frame_ia
     R_rel_ia1: Rotation object from frame_a to frame_ia1
                (frame that is fixed in frame_ia such that x-axis
                is along the rod axis)
                T = transpose([eRod_a, e2_a, e3_a]; w = w_rel_ia1
     R_rel_ia2: Fixed rotation object from frame_ia1 to frame_ia
                T = [eRod_ia, e2_ia, e3_ia]; w = zeros(3)

   The difficult part is to compute w_rel_ia1:
      w_rel_ia1 = [  e3_a*der(e2_a);
                    -e3_a*der(eRod_a);
                     e2_a*der(eRod_a)]
   der(eRod_a) is directly given, since eRod_a is a function
   of translational quantities only.
      der(eRod_a) = (der(rRod_a) - eRod_a*(eRod_a*der(rRod_a)))/rodLength
      der(n2_a)   = cross(n1_a, der(eRod_a))
      der(e2_a)   = (der(n2_a) - e2_a*(e2_a*der(n2_a)))/length_n2_a
   Inserting these equations in w_rel_ia1 results in:
      e3_a*der(eRod_a) = e3_a*der(rRod_a)/rodLength       // e3_a*eRod_a = 0
      e2_a*der(eRod_a) = e2_a*der(rRod_a)/rodLength       // e2_a*eRod_a = 0
      e3_a*der(e2_a)   = e3_a*der(n2_a)/lenght_n2_a       // e3_a*e2_a = 0
                       = e3_a*cross(n1_a, der(eRod_a))/length_n2_a
                       = e3_a*cross(n1_a, der(rRod_a) - eRod_a*(eRod_a*der(rRod_a)))/(length_n2_a*rodLength)
                       = e3_a*cross(n1_a, der(rRod_a))/(length_n2_a*rodLength)
   Furthermore, we have:
     rRod_a            = resolve2(frame_a.R, rRod_0);
     der(rRod_a)       = resolve2(frame_a.R, der(rRod_0)) - cross(frame_a.R.w, rRod_a));
*/
          der_rRod_a_L = (Frames.resolve2(frame_a.R, der(rRod_0)) - cross(frame_a.R.w, rRod_a)) / rodLength;
          w_rel_ia1 = {e3_a * cross(n1_a, der_rRod_a_L) / length_n2_a, -e3_a * der_rRod_a_L, e2_a * der_rRod_a_L};
          R_rel_ia1 = Frames.from_T(transpose([eRod_a, e2_a, e3_a]), w_rel_ia1);
          R_rel_ia2 = Frames.from_T([eRod_ia, e2_ia, e3_ia], zeros(3));
          R_rel_ia = Frames.absoluteRotation(R_rel_ia1, R_rel_ia2);
/*
  T_rel_ia = [eRod_ia, e2_ia, e3_ia]*transpose([eRod_a, e2_a, e3_a]);
  R_rel_ia = Frames.from_T(T_rel_ia,
    Frames.TransformationMatrices.angularVelocity2(T_rel_ia, der(T_rel_ia)));
*/
// Compute kinematic quantities of frame_ia
          frame_ia.r_0 = frame_a.r_0;
          frame_ia.R = Frames.absoluteRotation(frame_a.R, R_rel_ia);
/* In the following formulas f_a, f_b, f_ia, t_a, t_b, t_ia are
     the forces and torques at frame_a, frame_b, frame_ia, respectively,
     resolved in frame_a. e_x, e_y, e_z are the unit vectors resolved in frame_a.
     Torque balance at the rod around the origin of frame_a:
       0 = t_a + t_ia + cross(rRod_a, f_b)
     with
         rRod_a = rodLength*e_x
         f_b     = -f_rod*e_x + f_b[2]*e_y + f_b[3]*e_z
     follows:
       0 = t_a + t_ia + rodLength*(f_b[2]*e_z - f_b[3]*e_y)
     The projection of t_a with respect to universal joint axes vanishes:
       n1_a*t_a = 0
       e_y*t_a = 0
     Therefore:
        0 = n1_a*t_ia + rodLength*f_b[2]*(n1_a*e_z)
        0 = e_y*t_ia - rodLength*f_b[3]
     or
        f_b = -f_rod*e_x - e_y*(n1_a*t_ia)/(rodLength*(n1_a*e_z)) + e_z*(e_y*t_ia)/rodLength
     Force balance:
        0 = f_a + f_b + f_ia
  */
          f_ia_a = Frames.resolve1(R_rel_ia, frame_ia.f);
          t_ia_a = Frames.resolve1(R_rel_ia, frame_ia.t);
// f_b_a1 is needed in aggregation joints to solve kinematic loops analytically
          f_b_a1 = (-e2_a * (n1_a * t_ia_a / (rodLength * (n1_a * e3_a)))) + e3_a * (e2_a * t_ia_a / rodLength);
          f_b_a = (-f_rod * eRod_a) + f_b_a1;
          frame_b.f = Frames.resolveRelative(f_b_a, frame_a.R, frame_b.R);
          frame_b.t = zeros(3);
          zeros(3) = frame_a.f + f_b_a + f_ia_a;
          zeros(3) = frame_a.t + t_ia_a + cross(rRod_a, f_b_a);
// Measure power for test purposes
          if checkTotalPower then
            totalPower = frame_a.f * Frames.resolve2(frame_a.R, der(frame_a.r_0)) + frame_b.f * Frames.resolve2(frame_b.R, der(frame_b.r_0)) + frame_ia.f * Frames.resolve2(frame_ia.R, der(frame_ia.r_0)) + frame_a.t * Frames.angularVelocity2(frame_a.R) + frame_b.t * Frames.angularVelocity2(frame_b.R) + frame_ia.t * Frames.angularVelocity2(frame_ia.R);
          else
            totalPower = 0;
          end if;
          annotation(
            Documentation(info = "<html>
<p>
This component consists of a <b>universal joint</b> at frame_a and
a <b>spherical joint</b> at frame_b that are connected together with
a <b>rigid rod</b>, see default animation figure (the arrows are not
part of the default animation):
</p>

<p>
<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Joints/UniversalSpherical.png\" ALT=\"model Joints.UniversalSpherical\">
</p>

<p>
This joint aggregation has no mass and no inertia and introduces the constraint
that the distance between the origin of frame_a and the origin of frame_b is constant
(= Frames.length(rRod_ia)). The universal joint is defined in the following way:
</p>

<ul>
<li> The rotation <b>axis</b> of revolute joint <b>1</b> is along parameter
     vector n1_a which is fixed in frame_a.</li>
<li> The rotation <b>axis</b> of revolute joint <b>2</b> is perpendicular to
     axis 1 and to the line connecting the universal and the spherical joint.</li>
</ul>
<p>
The definition of axis 2 of the universal joint is performed according
to the most often occurring case. In a future release, axis 2 might
be explicitly definable via a parameter. However, the treatment is much more
complicated and the number of operations is considerably higher,
if axis 2 is not orthogonal to axis 1 and to the connecting rod.
</p>
<p>
Note, there is a <b>singularity</b> when axis 1 and the connecting rod are parallel
to other. Therefore, if possible n1_a should be selected in such a way that it
is perpendicular to rRod_ia in the initial configuration (i.e., the
distance to the singularity is as large as possible).
</p>
<p>
An additional <b>frame_ia</b> is present. It is <b>fixed</b> in the connecting
<b>rod</b> at the origin of <b>frame_a</b>. The placement of frame_ia on the rod
is implicitly defined by the universal joint (frame_a and frame_ia coincide
when the angles of the two revolute joints of the universal joint are zero)
and by parameter vector <b>rRod_ia</b>, the position vector
from the origin of frame_a to the origin of frame_b, resolved in frame_<b>ia</b>.
</p>
<p>
The easiest way to define the parameters of this joint is by moving the
MultiBody system in a <b>reference configuration</b> where <b>all frames</b>
of all components are <b>parallel</b> to other (alternatively,
at least frame_a and frame_ia of the UniversalSpherical joint
should be parallel to other when defining an instance of this
component). Since frame_a and frame_ia are parallel to other,
vector <b>rRod_ia</b> from frame_a to frame_b resolved in frame_<b>ia</b> can be resolved
in frame_<b>a</b> (or the <b>world frame</b>, if all frames are parallel to other).
</p>
<p>
This joint aggregation can be used in cases where
in reality a rod with spherical joints at end are present.
Such a system has an additional degree of freedom to rotate
the rod along its axis. In practice this rotation is usually
of no interest and is mathematically removed by replacing one
of the spherical joints by a universal joint. Still, in most
cases the Joints.SphericalSpherical joint aggregation can be used instead
of the UniversalSpherical joint
since the rod is animated and its mass properties are approximated by
a point mass in the middle of the rod. The SphericalSpherical joint
has the advantage that it does not have a singular configuration.
</p>
<p>
In the public interface of the UniversalSpherical joint, the following
(final) <b>parameters</b> are provided:
</p>
<pre>
  <b>parameter</b> Real rodLength(unit=\"m\")  \"Length of rod\";
  <b>parameter</b> Real eRod_ia[3] \"Unit vector along rod, resolved in frame_ia\";
  <b>parameter</b> Real e2_ia  [3] \"Unit vector along axis 2, resolved in frame_ia\";
</pre>
<p>
This allows a more convenient definition of data which is related to the rod.
For example, if a box shall be connected at frame_ia directing from
the origin of frame_a to the middle of the rod, this might be defined as:
</p>
<pre>
    Modelica.Mechanics.MultiBody.Joints.UniversalSpherical jointUS(rRod_ia={1.2, 1, 0.2});
    Modelica.Mechanics.MultiBody.Visualizers.FixedShape    shape(shapeType       = \"box\",
                                              lengthDirection = jointUS.eRod_ia,
                                              widthDirection  = jointUS.e2_ia,
                                              length          = jointUS.rodLength/2,
                                              width           = jointUS.rodLength/10);
  <b>equation</b>
    <b>connect</b>(jointUS.frame_ia, shape.frame_a);
</pre>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-150, -50}, {150, -90}}, lineColor = {0, 0, 255}, textString = "%name"), Ellipse(extent = {{-100, -40}, {-19, 40}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.Sphere, fillColor = {192, 192, 192}), Ellipse(extent = {{-90, -30}, {-29, 29}}, lineColor = {160, 160, 164}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-60, 41}, {-9, -44}}, lineColor = {255, 255, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{-60, 40}, {-60, -40}}, color = {0, 0, 0}, thickness = 0.5), Ellipse(extent = {{-83, -17}, {-34, 21}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.Sphere, fillColor = {192, 192, 192}), Ellipse(extent = {{-74, -12}, {-40, 15}}, lineColor = {160, 160, 164}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Polygon(points = {{-72, -20}, {-89, 3}, {-69, 25}, {-45, 27}, {-72, -20}}, pattern = LinePattern.None, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Line(points = {{-60, 40}, {-60, -10}}, color = {0, 0, 0}, thickness = 0.5), Line(points = {{-49, 20}, {-69, -15}}, color = {0, 0, 0}, thickness = 0.5), Ellipse(extent = {{44, 14}, {73, -14}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Ellipse(extent = {{20, -40}, {100, 40}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.Sphere, fillColor = {192, 192, 192}), Ellipse(extent = {{30, -30}, {90, 30}}, lineColor = {192, 192, 192}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-22, 45}, {40, -43}}, lineColor = {255, 255, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Ellipse(extent = {{46, 14}, {75, -14}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.Sphere, fillColor = {192, 192, 192}), Rectangle(extent = {{-36, -8}, {48, 8}}, lineColor = {0, 0, 0}, pattern = LinePattern.None, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192, 192, 192}), Text(extent = {{-105, 118}, {-67, 86}}, lineColor = {128, 128, 128}, textString = "ia"), Text(extent = {{-24, 95}, {167, 65}}, lineColor = {0, 0, 0}, textString = "%rRod_ia"), Line(points = {{-40, 101}, {-40, 60}, {-60, 1}}, color = {128, 128, 128}, thickness = 0.5)}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-60, -70}, {46, -70}}, color = {0, 0, 255}), Polygon(points = {{60, -70}, {45, -64}, {45, -76}, {60, -70}}, lineColor = {0, 0, 255}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-56, -71}, {56, -90}}, textString = "rRod", lineColor = {0, 0, 255}), Ellipse(extent = {{-100, -40}, {-19, 40}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.Sphere, fillColor = {192, 192, 192}), Ellipse(extent = {{-90, -30}, {-29, 29}}, lineColor = {160, 160, 164}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-60, 41}, {-19, -41}}, lineColor = {255, 255, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{-60, 40}, {-60, -40}}, color = {0, 0, 0}, thickness = 0.5), Ellipse(extent = {{-83, -17}, {-34, 21}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.Sphere, fillColor = {192, 192, 192}), Ellipse(extent = {{-74, -12}, {-40, 15}}, lineColor = {160, 160, 164}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Polygon(points = {{-72, -20}, {-89, 3}, {-69, 25}, {-45, 27}, {-72, -20}}, pattern = LinePattern.None, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Line(points = {{-60, 40}, {-60, -10}}, color = {0, 0, 0}, thickness = 0.5), Line(points = {{-49, 20}, {-69, -15}}, color = {0, 0, 0}, thickness = 0.5), Polygon(points = {{7, -1}, {-5, 2}, {-5, -4}, {7, -1}}, lineColor = {0, 0, 255}), Line(points = {{-50, 19}, {-30, 57}}, color = {0, 0, 255}), Text(extent = {{-34, 78}, {8, 62}}, lineColor = {0, 0, 0}, textString = "e2"), Polygon(points = {{-25, 64}, {-33, 56}, {-27, 53}, {-25, 64}}, lineColor = {0, 0, 255}), Line(points = {{-60, 41}, {-60, 65}}, color = {0, 0, 255}), Polygon(points = {{-60, 75}, {-64, 63}, {-56, 63}, {-60, 75}}, lineColor = {0, 0, 255}), Text(extent = {{-93, 82}, {-64, 62}}, lineColor = {0, 0, 0}, textString = "n1"), Line(points = {{-60, -40}, {-60, -72}}, color = {0, 0, 255}), Ellipse(extent = {{20, -40}, {100, 40}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.Sphere, fillColor = {192, 192, 192}), Ellipse(extent = {{30, -30}, {90, 30}}, lineColor = {192, 192, 192}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-22, 45}, {40, -43}}, lineColor = {255, 255, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Ellipse(extent = {{45, 14}, {74, -14}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-36, -8}, {48, 8}}, lineColor = {0, 0, 0}, pattern = LinePattern.None, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192, 192, 192}), Text(extent = {{-31, -7}, {0, -28}}, lineColor = {0, 0, 0}, textString = "eRod"), Line(points = {{-60, 0}, {-5, 0}}, color = {0, 0, 255}), Polygon(points = {{7, 0}, {-5, 3}, {-5, -3}, {7, 0}}, lineColor = {0, 0, 255}), Line(points = {{60, -1}, {60, -72}}, color = {0, 0, 255}), Line(points = {{-40, 100}, {-40, 70}, {-60, 0}}, color = {128, 128, 128}, thickness = 0.5), Text(extent = {{-23, 30}, {26, 10}}, textString = " eRod*e2 = 0;  n1*e2 = 0", lineColor = {0, 0, 255})}));
        end UniversalSpherical;

        model GearConstraint "Ideal 3-dim. gearbox (arbitrary shaft directions)"
          extends Modelica.Mechanics.MultiBody.Interfaces.PartialTwoFrames;
          Interfaces.Frame_a bearing "Coordinate system fixed in the bearing" annotation(
            Placement(transformation(origin = {0, -100}, extent = {{-16, -16}, {16, 16}}, rotation = 90)));
          parameter Real ratio(start = 2) "Gear speed ratio";
          parameter Modelica.Mechanics.MultiBody.Types.Axis n_a = {1, 0, 0} "Axis of rotation of shaft a (same coordinates in frame_a, frame_b, bearing)";
          parameter Modelica.Mechanics.MultiBody.Types.Axis n_b = {1, 0, 0} "Axis of rotation of shaft b (same coordinates in frame_a, frame_b, bearing)";
          parameter Modelica.SIunits.Position r_a[3] = {0, 0, 0} "Vector from frame bearing to frame_a resolved in bearing";
          parameter Modelica.SIunits.Position r_b[3] = {0, 0, 0} "Vector from frame bearing to frame_b resolved in bearing";
          parameter StateSelect stateSelect = StateSelect.default "Priority to use joint coordinates (phi_a, phi_b, w_a, w_b) as states" annotation(
            Dialog(tab = "Advanced"));
          SI.Angle phi_b(start = 0, stateSelect = stateSelect) "Relative rotation angle of revolute joint at frame_b";
          SI.AngularVelocity w_b(start = 0, stateSelect = stateSelect) "First derivative of angle phi_b (relative angular velocity b)";
          SI.AngularAcceleration a_b(start = 0) "Second derivative of angle phi_b (relative angular acceleration b)";
          Modelica.Mechanics.MultiBody.Joints.Revolute actuatedRevolute_a(useAxisFlange = true, n = n_a, animation = false) annotation(
            Placement(transformation(extent = {{-40, -10}, {-60, 10}}, rotation = 0)));
          Modelica.Mechanics.MultiBody.Joints.Revolute actuatedRevolute_b(useAxisFlange = true, n = n_b, animation = false) annotation(
            Placement(transformation(extent = {{40, -10}, {60, 10}}, rotation = 0)));
          Modelica.Mechanics.Rotational.Components.IdealGear idealGear(ratio = ratio) annotation(
            Placement(transformation(extent = {{-10, 30}, {10, 50}}, rotation = 0)));
          Modelica.Mechanics.MultiBody.Parts.FixedTranslation fixedTranslation1(animation = false, r = r_b) annotation(
            Placement(transformation(extent = {{10, -10}, {30, 10}}, rotation = 0)));
          Modelica.Mechanics.MultiBody.Parts.FixedTranslation fixedTranslation2(animation = false, r = r_a) annotation(
            Placement(transformation(origin = {-20, 0}, extent = {{-10, -10}, {10, 10}}, rotation = 180)));
        equation
          assert(cardinality(bearing) > 0, "Connector bearing of component is not connected");
          phi_b = actuatedRevolute_b.phi;
          w_b = der(phi_b);
          a_b = der(w_b);
          connect(actuatedRevolute_a.axis, idealGear.flange_a) annotation(
            Line(points = {{-50, 10}, {-50, 40}, {-10, 40}}, color = {0, 0, 0}));
          connect(idealGear.flange_b, actuatedRevolute_b.axis) annotation(
            Line(points = {{10, 40}, {50, 40}, {50, 10}}, color = {0, 0, 0}));
          connect(actuatedRevolute_a.frame_a, fixedTranslation2.frame_b) annotation(
            Line(points = {{-40, 0}, {-35, 0}, {-30, 0}}, color = {95, 95, 95}, thickness = 0.5));
          connect(fixedTranslation2.frame_a, bearing) annotation(
            Line(points = {{-10, 0}, {-4, 0}, {0, 0}, {0, -100}}, color = {95, 95, 95}, thickness = 0.5));
          connect(fixedTranslation1.frame_a, bearing) annotation(
            Line(points = {{10, 0}, {0, 0}, {0, -100}}, color = {95, 95, 95}, thickness = 0.5));
          connect(fixedTranslation1.frame_b, actuatedRevolute_b.frame_a) annotation(
            Line(points = {{30, 0}, {40, 0}}, color = {95, 95, 95}, thickness = 0.5));
          connect(frame_a, actuatedRevolute_a.frame_b) annotation(
            Line(points = {{-100, 4.44089e-16}, {-80, 4.44089e-16}, {-80, 0}, {-60, 0}}, color = {95, 95, 95}, thickness = 0.5));
          connect(actuatedRevolute_b.frame_b, frame_b) annotation(
            Line(points = {{60, 0}, {80, 0}, {80, 4.44089e-16}, {100, 4.44089e-16}}, color = {95, 95, 95}, thickness = 0.5));
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(origin = {0, -20}, lineColor = {0, 0, 255}, extent = {{-150, 135}, {150, 175}}, textString = "%name"), Text(origin = {0, 12}, extent = {{-150, -94}, {150, -64}}, textString = "%ratio"), Rectangle(origin = {-35, 60}, fillColor = {255, 255, 255}, fillPattern = FillPattern.HorizontalCylinder, extent = {{-15, -40}, {15, 40}}), Rectangle(origin = {-35, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.HorizontalCylinder, extent = {{-15, -21}, {15, 21}}), Line(points = {{-80, 20}, {-60, 20}}), Line(points = {{-80, -20}, {-60, -20}}), Line(points = {{-70, -20}, {-70, -86}}), Line(points = {{0, 40}, {0, -100}}), Line(points = {{-10, 40}, {10, 40}}), Line(points = {{-10, 80}, {10, 80}}), Line(points = {{60, -20}, {80, -20}}), Line(points = {{60, 20}, {80, 20}}), Line(points = {{70, -20}, {70, -86}}), Rectangle(origin = {-75, 0}, lineColor = {64, 64, 64}, fillColor = {191, 191, 191}, fillPattern = FillPattern.HorizontalCylinder, extent = {{-25, -10}, {25, 10}}), Rectangle(origin = {75, 0}, lineColor = {64, 64, 64}, fillColor = {191, 191, 191}, fillPattern = FillPattern.HorizontalCylinder, extent = {{-25, -10}, {25, 10}}), Rectangle(origin = {-35, -19}, fillColor = {153, 153, 153}, fillPattern = FillPattern.Solid, extent = {{-15, -2}, {15, 2}}), Rectangle(origin = {-35, -8}, fillColor = {204, 204, 204}, fillPattern = FillPattern.Solid, extent = {{-15, -3}, {15, 3}}), Rectangle(origin = {-35, 19}, fillColor = {204, 204, 204}, fillPattern = FillPattern.Solid, extent = {{-15, -2}, {15, 2}}), Rectangle(origin = {-35, 8}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, extent = {{-15, -3}, {15, 3}}), Rectangle(origin = {0, 60}, lineColor = {64, 64, 64}, fillColor = {191, 191, 191}, fillPattern = FillPattern.HorizontalCylinder, extent = {{-20, -10}, {20, 10}}), Rectangle(origin = {-35, 98}, fillColor = {153, 153, 153}, fillPattern = FillPattern.Solid, extent = {{-15, -2}, {15, 2}}), Rectangle(origin = {-35, 87}, fillColor = {204, 204, 204}, fillPattern = FillPattern.Solid, extent = {{-15, -3}, {15, 3}}), Rectangle(origin = {-35, 50}, fillColor = {204, 204, 204}, fillPattern = FillPattern.Solid, extent = {{-15, -4}, {15, 4}}), Rectangle(origin = {-35, 22}, fillColor = {102, 102, 102}, fillPattern = FillPattern.Solid, extent = {{-15, -2}, {15, 2}}), Rectangle(origin = {-35, 33}, fillColor = {153, 153, 153}, fillPattern = FillPattern.Solid, extent = {{-15, -3}, {15, 3}}), Rectangle(origin = {-35, 70}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, extent = {{-15, -4}, {15, 4}}), Rectangle(origin = {35, 60}, fillColor = {255, 255, 255}, fillPattern = FillPattern.HorizontalCylinder, extent = {{-15, -21}, {15, 21}}), Rectangle(origin = {35, 41}, fillColor = {153, 153, 153}, fillPattern = FillPattern.Solid, extent = {{-15, -2}, {15, 2}}), Rectangle(origin = {35, 52}, fillColor = {204, 204, 204}, fillPattern = FillPattern.Solid, extent = {{-15, -3}, {15, 3}}), Rectangle(origin = {35, 79}, fillColor = {204, 204, 204}, fillPattern = FillPattern.Solid, extent = {{-15, -2}, {15, 2}}), Rectangle(origin = {35, 68}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, extent = {{-15, -3}, {15, 3}}), Rectangle(origin = {35, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.HorizontalCylinder, extent = {{-15, -40}, {15, 40}}), Rectangle(origin = {35, 38}, fillColor = {153, 153, 153}, fillPattern = FillPattern.Solid, extent = {{-15, -2}, {15, 2}}), Rectangle(origin = {35, 27}, fillColor = {204, 204, 204}, fillPattern = FillPattern.Solid, extent = {{-15, -3}, {15, 3}}), Rectangle(origin = {35, -10}, fillColor = {204, 204, 204}, fillPattern = FillPattern.Solid, extent = {{-15, -4}, {15, 4}}), Rectangle(origin = {35, -27}, fillColor = {153, 153, 153}, fillPattern = FillPattern.Solid, extent = {{-15, -3}, {15, 3}}), Rectangle(origin = {35, 10}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, extent = {{-15, -4}, {15, 4}}), Rectangle(origin = {-35, 40}, fillColor = {255, 255, 255}, extent = {{-15, -61}, {15, 60}}), Rectangle(origin = {35, 21}, fillColor = {255, 255, 255}, extent = {{-15, -61}, {15, 60}}), Line(points = {{70, -86}, {-70, -86}})}),
            Documentation(info = "<html>
<p>This ideal massless joint provides a gear constraint between
frames <code>frame_a</code> and <code>frame_b</code>. The axes of rotation
of <code>frame_a</code> and <code>frame_b</code> may be arbitrary.</p>
<p><b>Reference</b><br>
<span style=\"font-variant:small-caps\">Schweiger</span>, Christian ;
<span style=\"font-variant:small-caps\">Otter</span>, Martin:
<a href=\"https://www.modelica.org/events/Conference2003/papers/h06_Schweiger_powertrains_v5.pdf\">Modelling
3D Mechanical Effects of 1-dim. Powertrains</a>. In: <i>Proceedings of the 3rd International
Modelica Conference</i>. Link&ouml;ping : The Modelica Association and Link&ouml;ping University,
November 3-4, 2003, pp. 149-158</p>
</html>"));
        end GearConstraint;

        model RollingWheel "Joint (no mass, no inertia) that describes an ideal rolling wheel (rolling on the plane z=0)"
          import Modelica.Mechanics.MultiBody.Frames;
          Modelica.Mechanics.MultiBody.Interfaces.Frame_a frame_a "Frame fixed in wheel center point. x-Axis: upwards, y-axis: along wheel axis" annotation(
            Placement(transformation(extent = {{-16, -16}, {16, 16}})));
          parameter SI.Radius wheelRadius "Wheel radius";
          parameter StateSelect stateSelect = StateSelect.always "Priority to use generalized coordinates as states" annotation(
            HideResult = true,
            Evaluate = true);
          SI.Position x(start = 0, stateSelect = stateSelect) "x-coordinate of wheel axis";
          SI.Position y(start = 0, stateSelect = stateSelect) "y-coordinate of wheel axis";
          SI.Position z;
          SI.Angle angles[3](start = {0, 0, 0}, each stateSelect = stateSelect) "Angles to rotate world-frame in to frame_a around z-, y-, x-axis" annotation(
            Dialog(group = "Initialization", showStartAttribute = true));
          SI.AngularVelocity der_angles[3](start = {0, 0, 0}, each stateSelect = stateSelect) "Derivative of angles" annotation(
            Dialog(group = "Initialization", showStartAttribute = true));
          SI.Position r_road_0[3] "Position vector from world frame to contact point on road, resolved in world frame";
          // Contact force
          SI.Force f_wheel_0[3] "Contact force acting on wheel, resolved in world frame";
          SI.Force f_n "Contact force acting on wheel in normal direction";
          SI.Force f_lat "Contact force acting on wheel in lateral direction";
          SI.Force f_long "Contact force acting on wheel in longitudinal direction";
          SI.Position err "|r_road_0 - frame_a.r_0| - wheelRadius (must be zero; used for checking)";
        protected
          Real e_axis_0[3] "Unit vector along wheel axis, resolved in world frame";
          SI.Position delta_0[3](start = {0, 0, -wheelRadius}) "Distance vector from wheel center to contact point";
          // Coordinate system at contact point
          Real e_n_0[3] "Unit vector in normal direction of road at contact point, resolved in world frame";
          Real e_lat_0[3] "Unit vector in lateral direction of wheel at contact point, resolved in world frame";
          Real e_long_0[3] "Unit vector in longitudinal direction of wheel at contact point, resolved in world frame";
          // Road description
          SI.Position s "Road surface parameter 1";
          SI.Position w "Road surface parameter 2";
          Real e_s_0[3] "Road heading at (s,w), resolved in world frame (unit vector)";
          // Slip velocities
          SI.Velocity v_0[3] "Velocity of wheel center, resolved in world frame";
          SI.AngularVelocity w_0[3] "Angular velocity of wheel, resolved in world frame";
          SI.Velocity vContact_0[3] "Velocity of wheel contact point, resolved in world frame";
          // Utility vectors
          Real aux[3];
        equation
// frame_a.R is computed from generalized coordinates
          Connections.root(frame_a.R);
          frame_a.r_0 = {x, y, z};
          der_angles = der(angles);
          frame_a.R = Frames.axesRotations({3, 2, 1}, angles, der_angles);
// Road description
          r_road_0 = {s, w, 0};
          e_n_0 = {0, 0, 1};
          e_s_0 = {1, 0, 0};
// Coordinate system at contact point (e_long_0, e_lat_0, e_n_0)
          e_axis_0 = Frames.resolve1(frame_a.R, {0, 1, 0});
          aux = cross(e_n_0, e_axis_0);
          e_long_0 = aux / Modelica.Math.Vectors.length(aux);
          e_lat_0 = cross(e_long_0, e_n_0);
// Determine point on road where the wheel is in contact with the road
          delta_0 = r_road_0 - frame_a.r_0;
          0 = delta_0 * e_axis_0;
          0 = delta_0 * e_long_0;
// One holonomic positional constraint equation (no penetration in to the ground)
          0 = wheelRadius - delta_0 * cross(e_long_0, e_axis_0);
// only for testing
          err = Modelica.Math.Vectors.length(delta_0) - wheelRadius;
// Slip velocities
          v_0 = der(frame_a.r_0);
          w_0 = Frames.angularVelocity1(frame_a.R);
          vContact_0 = v_0 + cross(w_0, delta_0);
// Two non-holonomic constraint equations on velocity level (ideal rolling, no slippage)
          0 = vContact_0 * e_long_0;
          0 = vContact_0 * e_lat_0;
// Contact force
          f_wheel_0 = f_n * e_n_0 + f_lat * e_lat_0 + f_long * e_long_0;
// Force and torque balance at the wheel center
          zeros(3) = frame_a.f + Frames.resolve2(frame_a.R, f_wheel_0);
          zeros(3) = frame_a.t + Frames.resolve2(frame_a.R, cross(delta_0, f_wheel_0));
// Guard against singularity
          assert(abs(e_n_0 * e_axis_0) < 0.99, "Wheel lays nearly on the ground (which is a singularity)");
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, -80}, {100, -100}}, lineColor = {0, 0, 0}, fillColor = {175, 175, 175}, fillPattern = FillPattern.Solid), Text(extent = {{-154, 124}, {146, 84}}, lineColor = {0, 0, 255}, textString = "%name"), Ellipse(extent = {{-80, 80}, {80, -80}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid)}));
        end RollingWheel;

        model RollingWheelSet "Joint (no mass, no inertia) that describes an ideal rolling wheel set (two ideal rolling wheels connected together by an axis)"
          Modelica.Mechanics.MultiBody.Interfaces.Frame_a frameMiddle "Frame fixed in middle of axis connecting both wheels (y-axis: along wheel axis, z-Axis: upwards)" annotation(
            Placement(transformation(extent = {{-16, 16}, {16, -16}}), iconTransformation(extent = {{-16, -16}, {16, 16}})));
          parameter Boolean animation = true "= true, if animation of wheel set shall be enabled";
          parameter SI.Radius wheelRadius "Radius of one wheel";
          parameter SI.Distance wheelDistance "Distance between the two wheels";
          parameter StateSelect stateSelect = StateSelect.default "Priority to use the generalized coordinates as states";
          Modelica.SIunits.Position x(start = 0, stateSelect = stateSelect) "x coordinate for center between wheels";
          Modelica.SIunits.Position y(start = 0, stateSelect = stateSelect) "y coordinate for center between wheels";
          Modelica.SIunits.Angle phi(start = 0, stateSelect = stateSelect) "Orientation angle of wheel axis along z-axis";
          Modelica.SIunits.Angle theta1(start = 0, stateSelect = stateSelect) "Angle of wheel 1";
          Modelica.SIunits.Angle theta2(start = 0, stateSelect = stateSelect) "Angle of wheel 2";
          Modelica.SIunits.AngularVelocity der_theta1(start = 0, stateSelect = stateSelect) "Derivative of theta 1";
          Modelica.SIunits.AngularVelocity der_theta2(start = 0, stateSelect = stateSelect) "Derivative of theta 2";
          Modelica.Mechanics.MultiBody.Interfaces.Frame_a frame1 "Frame fixed in center point of left wheel (y-axis: along wheel axis, z-Axis: upwards)" annotation(
            Placement(transformation(extent = {{-96, 16}, {-64, -16}}), iconTransformation(extent = {{-96, 16}, {-64, -16}})));
          Modelica.Mechanics.MultiBody.Interfaces.Frame_b frame2 "Frame fixed in center point of right wheel (y-axis: along wheel axis, z-Axis: upwards)" annotation(
            Placement(transformation(extent = {{64, 16}, {96, -16}})));
          Modelica.Mechanics.MultiBody.Parts.Fixed fixed(r = {0, 0, wheelRadius}, animation = animation) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 90, origin = {0, -90})));
          Modelica.Mechanics.MultiBody.Parts.FixedTranslation rod1(r = {0, wheelDistance / 2, 0}, animation = animation) annotation(
            Placement(transformation(extent = {{-8, -10}, {-28, 10}})));
          Modelica.Mechanics.MultiBody.Joints.Prismatic prismatic1(animation = animation) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 90, origin = {0, -66})));
          Modelica.Mechanics.MultiBody.Joints.Prismatic prismatic2(n = {0, 1, 0}, animation = animation) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 180, origin = {-24, -50})));
          Modelica.Mechanics.MultiBody.Joints.Revolute revolute(animation = animation) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 90, origin = {0, -22})));
          Modelica.Mechanics.MultiBody.Parts.FixedTranslation rod2(r = {0, -wheelDistance / 2, 0}, animation = animation) annotation(
            Placement(transformation(extent = {{12, -10}, {32, 10}})));
          Modelica.Mechanics.MultiBody.Joints.Revolute revolute1(n = {0, 1, 0}, useAxisFlange = true, animation = animation) annotation(
            Placement(transformation(extent = {{-34, -10}, {-54, 10}})));
          Modelica.Mechanics.MultiBody.Joints.Revolute revolute2(n = {0, 1, 0}, useAxisFlange = true, animation = animation) annotation(
            Placement(transformation(extent = {{40, -10}, {60, 10}})));
          Modelica.Mechanics.MultiBody.Joints.Internal.RollingConstraintVerticalWheel rolling1(radius = wheelRadius) annotation(
            Placement(transformation(extent = {{-70, -60}, {-50, -40}})));
          Modelica.Mechanics.MultiBody.Joints.Internal.RollingConstraintVerticalWheel rolling2(radius = wheelRadius, lateralSlidingConstraint = false) annotation(
            Placement(transformation(extent = {{54, -60}, {74, -40}})));
          Modelica.Mechanics.Rotational.Interfaces.Flange_a axis1 "1-dim. rotational flange that drives the joint" annotation(
            Placement(transformation(extent = {{-110, 90}, {-90, 110}})));
          Modelica.Mechanics.Rotational.Interfaces.Flange_a axis2 "1-dim. rotational flange that drives the joint" annotation(
            Placement(transformation(extent = {{90, 90}, {110, 110}})));
          Modelica.Mechanics.MultiBody.Parts.Mounting1D mounting1D annotation(
            Placement(transformation(extent = {{-10, 38}, {10, 58}})));
          Modelica.Mechanics.Rotational.Interfaces.Flange_b support "Support of 1D axes" annotation(
            Placement(transformation(extent = {{-10, 70}, {10, 90}}), iconTransformation(extent = {{-10, 70}, {10, 90}})));
        equation
          prismatic1.s = x;
          prismatic2.s = y;
          revolute.phi = phi;
          revolute1.phi = theta1;
          revolute2.phi = theta2;
          der_theta1 = der(theta1);
          der_theta2 = der(theta2);
          connect(revolute.frame_b, frameMiddle) annotation(
            Line(points = {{0, -12}, {0, 0}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
          connect(rod1.frame_a, frameMiddle) annotation(
            Line(points = {{-8, 0}, {0, 0}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
          connect(rod2.frame_a, frameMiddle) annotation(
            Line(points = {{12, 0}, {0, 0}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
          connect(rod1.frame_b, revolute1.frame_a) annotation(
            Line(points = {{-28, 0}, {-34, 0}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
          connect(revolute1.frame_b, frame1) annotation(
            Line(points = {{-54, 0}, {-80, 0}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
          connect(revolute2.frame_a, rod2.frame_b) annotation(
            Line(points = {{40, 0}, {32, 0}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
          connect(revolute2.frame_b, frame2) annotation(
            Line(points = {{60, 0}, {80, 0}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
          connect(prismatic1.frame_a, fixed.frame_b) annotation(
            Line(points = {{0, -76}, {0, -80}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
          connect(prismatic1.frame_b, prismatic2.frame_a) annotation(
            Line(points = {{0, -56}, {0, -50}, {-14, -50}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
          connect(prismatic2.frame_b, revolute.frame_a) annotation(
            Line(points = {{-34, -50}, {-40, -50}, {-40, -36}, {0, -36}, {0, -32}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
          connect(rolling1.frame_a, revolute1.frame_b) annotation(
            Line(points = {{-60, -48}, {-60, 0}, {-54, 0}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
          connect(rolling2.frame_a, revolute2.frame_b) annotation(
            Line(points = {{64, -48}, {64, 0}, {60, 0}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
          connect(revolute1.axis, axis1) annotation(
            Line(points = {{-44, 10}, {-44, 100}, {-100, 100}}, color = {0, 0, 0}, smooth = Smooth.None));
          connect(revolute2.axis, axis2) annotation(
            Line(points = {{50, 10}, {50, 100}, {100, 100}}, color = {0, 0, 0}, smooth = Smooth.None));
          connect(frameMiddle, mounting1D.frame_a) annotation(
            Line(points = {{0, 0}, {0, 38}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
          connect(mounting1D.flange_b, support) annotation(
            Line(points = {{10, 48}, {16, 48}, {16, 80}, {0, 80}}, color = {0, 0, 0}, smooth = Smooth.None));
          annotation(
            defaultComponentName = "wheelSet",
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, -80}, {100, -100}}, lineColor = {0, 0, 0}, fillColor = {175, 175, 175}, fillPattern = FillPattern.Solid), Text(extent = {{-146, -98}, {154, -138}}, textString = "%name", lineColor = {0, 0, 255}), Ellipse(extent = {{42, 80}, {118, -80}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-62, 2}, {64, -6}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-118, 80}, {-42, -80}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{86, 24}, {64, 24}, {64, 10}, {56, 10}}, color = {0, 0, 0}, smooth = Smooth.None), Line(points = {{86, -24}, {64, -24}, {64, -12}, {56, -12}}, color = {0, 0, 0}, smooth = Smooth.None), Line(points = {{-96, 100}, {-80, 100}, {-80, 4}}, color = {0, 0, 0}, smooth = Smooth.None), Line(points = {{100, 100}, {80, 100}, {80, -2}}, color = {0, 0, 0}, smooth = Smooth.None), Line(points = {{0, 72}, {0, 40}, {-20, 40}, {-20, 2}}, color = {0, 0, 0}, smooth = Smooth.None), Line(points = {{0, 40}, {20, 40}, {20, 2}}, color = {0, 0, 0}, smooth = Smooth.None)}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-68, 24}, {-68, 52}}, color = {0, 0, 255}, smooth = Smooth.None), Polygon(points = {{-68, 70}, {-74, 52}, {-62, 52}, {-68, 70}}, lineColor = {0, 0, 255}, smooth = Smooth.None, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-56, 62}, {-38, 50}}, lineColor = {0, 0, 255}, textString = "x"), Line(points = {{-62, 30}, {-94, 30}}, color = {0, 0, 255}, smooth = Smooth.None), Polygon(points = {{-90, 36}, {-90, 24}, {-108, 30}, {-90, 36}}, lineColor = {0, 0, 255}, smooth = Smooth.None, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-114, 50}, {-96, 38}}, lineColor = {0, 0, 255}, textString = "y")}));
        end RollingWheelSet;

        package Assemblies "Components that aggregate several joints for analytic loop handling"
          extends Modelica.Icons.Package;

          model JointUPS "Universal - prismatic - spherical joint aggregation (no constraints, no potential states)"
            import Modelica.Mechanics.MultiBody.Types;
            extends Interfaces.PartialTwoFramesDoubleSize;
            Modelica.Mechanics.MultiBody.Interfaces.Frame_a frame_ia "Coordinate system at origin of frame_a fixed at prismatic joint" annotation(
              Placement(transformation(origin = {-80, 100}, extent = {{-8, -8}, {8, 8}}, rotation = 270)));
            Modelica.Mechanics.MultiBody.Interfaces.Frame_b frame_ib "Coordinate system at origin of frame_b fixed at prismatic joint" annotation(
              Placement(transformation(origin = {80, 100}, extent = {{-8, 8}, {8, -8}}, rotation = 270)));
            Modelica.Mechanics.Translational.Interfaces.Flange_a axis "1-dim. translational flange that drives the prismatic joint" annotation(
              Placement(transformation(extent = {{45, 95}, {35, 105}}, rotation = 0)));
            Modelica.Mechanics.Translational.Interfaces.Flange_b bearing "1-dim. translational flange of the drive bearing of the prismatic joint" annotation(
              Placement(transformation(extent = {{-35, 95}, {-45, 105}}, rotation = 0)));
            parameter Boolean animation = true "= true, if animation shall be enabled";
            parameter Boolean showUniversalAxes = true "= true, if universal joint shall be visualized with two cylinders, otherwise with a sphere (provided animation=true)";
            parameter Modelica.Mechanics.MultiBody.Types.Axis n1_a = {0, 0, 1} "Axis 1 of universal joint resolved in frame_a (axis 2 is orthogonal to axis 1 and to line from universal to spherical joint)" annotation(
              Evaluate = true);
            parameter SI.Position nAxis_ia[3] = {1, 0, 0} "Axis vector along line from origin of frame_a to origin of frame_b, resolved in frame_ia" annotation(
              Evaluate = true);
            parameter SI.Position s_offset = 0 "Relative distance offset (distance between frame_a and frame_b = s(t) + s_offset)";
            parameter SI.Diameter sphereDiameter = world.defaultJointLength "Diameter of spheres representing the spherical joints" annotation(
              Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            input Types.Color sphereColor = Modelica.Mechanics.MultiBody.Types.Defaults.JointColor "Color of spheres representing the spherical joints" annotation(
              Dialog(colorSelector = true, tab = "Animation", group = "if animation = true", enable = animation));
            parameter SI.Diameter axisDiameter = sphereDiameter / Types.Defaults.JointRodDiameterFraction "Diameter of cylinder on the connecting line from frame_a to frame_b" annotation(
              Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            input Types.Color axisColor = Modelica.Mechanics.MultiBody.Types.Defaults.SensorColor "Color of cylinder on the connecting line from frame_a to frame_b" annotation(
              Dialog(colorSelector = true, tab = "Animation", group = "if animation = true", enable = animation));
            input Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(
              Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            parameter SI.Distance cylinderLength = world.defaultJointLength "Length of cylinders representing the two universal joint axes" annotation(
              Dialog(tab = "Animation", group = "if animation = true and showUniversalAxes", enable = animation and showUniversalAxes));
            parameter SI.Distance cylinderDiameter = world.defaultJointWidth "Diameter of cylinders representing the two universal joint axes" annotation(
              Dialog(tab = "Animation", group = "if animation = true and showUniversalAxes", enable = animation and showUniversalAxes));
            input Types.Color cylinderColor = Modelica.Mechanics.MultiBody.Types.Defaults.JointColor "Color of cylinders representing the two universal joint axes" annotation(
              Dialog(colorSelector = true, tab = "Animation", group = "if animation = true and showUniversalAxes", enable = animation and showUniversalAxes));
            parameter Boolean checkTotalPower = false "= true, if total power flowing into this component shall be determined (must be zero)" annotation(
              Dialog(tab = "Advanced"));
            final parameter Real eAxis_ia[3](each final unit = "1") = Modelica.Math.Vectors.normalizeWithAssert(nAxis_ia) "Unit vector from origin of frame_a to origin of frame_b, resolved in frame_ia";
            final parameter Real e2_ia[3](each final unit = "1") = Modelica.Math.Vectors.normalizeWithAssert(cross(n1_a, eAxis_ia)) "Unit vector in direction of second rotation axis of universal joint, resolved in frame_ia";
            final parameter Real e3_ia[3](each final unit = "1") = cross(eAxis_ia, e2_ia) "Unit vector perpendicular to eAxis_ia and e2_ia, resolved in frame_ia";
            SI.Position s "Relative distance between frame_a and frame_b along axis nAxis = s + s_offset";
            SI.Force f "= axis.f (driving force in the axis; = -bearing.f)";
            SI.Length axisLength "Distance between frame_a and frame_b";
            SI.Power totalPower "Total power flowing into this element, if checkTotalPower=true (otherwise dummy)";
          protected
            SI.Force f_c_a[3] "frame_ia.f resolved in frame_a";
            SI.Torque t_cd_a[3] "frame_ia.t + frame_ib.t resolved in frame_a";
            SI.Force f_bd_a[3] "frame_b.f + frame_ib.f resolved in frame_a";
            SI.Position rAxis_0[3] "Position vector from origin of frame_a to origin of frame_b resolved in world frame";
            SI.Position rAxis_a[3] "Position vector from origin of frame_a to origin of frame_b resolved in frame_a";
            Real eAxis_a[3](each final unit = "1") "Unit vector in direction of rAxis_a, resolved in frame_a";
            Real e2_a[3](each final unit = "1") "Unit vector in direction of second rotation axis of universal joint, resolved in frame_a";
            Real e3_a[3](each final unit = "1") "Unit vector perpendicular to eAxis_a and e2_a, resolved in frame_a";
            Real n2_a[3](each final unit = "1") "Vector in direction of second rotation axis of universal joint, resolved in frame_a";
            Real length2_n2_a(unit = "1") "Square of length of vector n2_a";
            Real length_n2_a(unit = "1") "Length of vector n2_a";
            Real der_rAxis_a_L[3](each unit = "1/s") "= der(rAxis_a)/axisLength";
            SI.AngularVelocity w_rel_ia1[3];
            Frames.Orientation R_ia1_a;
            Frames.Orientation R_ia2_a;
            Frames.Orientation R_ia_a "Rotation from frame_a to frame_ia";
            // Real T_ia_a[3, 3] "Transformation matrix from frame_a to frame_ia";
            Visualizers.Advanced.Shape axisCylinder(shapeType = "cylinder", color = axisColor, specularCoefficient = specularCoefficient, length = axisLength, width = axisDiameter, height = axisDiameter, lengthDirection = eAxis_ia, widthDirection = e2_ia, r = frame_ia.r_0, R = frame_ia.R) if world.enableAnimation and animation;
            Visualizers.Advanced.Shape sphericalShape_b(shapeType = "sphere", color = sphereColor, specularCoefficient = specularCoefficient, length = sphereDiameter, width = sphereDiameter, height = sphereDiameter, lengthDirection = {1, 0, 0}, widthDirection = {0, 1, 0}, r_shape = {-0.5, 0, 0} * sphereDiameter, r = frame_b.r_0, R = frame_b.R) if world.enableAnimation and animation;
            Visualizers.Advanced.Shape sphericalShape_a(shapeType = "sphere", color = sphereColor, specularCoefficient = specularCoefficient, length = sphereDiameter, width = sphereDiameter, height = sphereDiameter, lengthDirection = {1, 0, 0}, widthDirection = {0, 1, 0}, r_shape = {-0.5, 0, 0} * sphereDiameter, r = frame_a.r_0, R = frame_a.R) if world.enableAnimation and animation;
            Visualizers.Advanced.Shape universalShape1(shapeType = "cylinder", color = cylinderColor, specularCoefficient = specularCoefficient, length = cylinderLength, width = cylinderDiameter, height = cylinderDiameter, lengthDirection = n1_a, widthDirection = {0, 1, 0}, r_shape = -n1_a * (cylinderLength / 2), r = frame_a.r_0, R = frame_a.R) if world.enableAnimation and animation and showUniversalAxes;
            Visualizers.Advanced.Shape universalShape2(shapeType = "cylinder", color = cylinderColor, specularCoefficient = specularCoefficient, length = cylinderLength, width = cylinderDiameter, height = cylinderDiameter, lengthDirection = e2_ia, widthDirection = {0, 1, 0}, r_shape = -e2_ia * (cylinderLength / 2), r = frame_ia.r_0, R = frame_ia.R) if world.enableAnimation and animation and showUniversalAxes;
          equation
            Connections.branch(frame_a.R, frame_ia.R);
            Connections.branch(frame_ia.R, frame_ib.R);
// Translational flanges
            axisLength = s + s_offset;
            bearing.s = 0;
            axis.s = s;
            axis.f = f;
// Position vector rAxis from frame_a to frame_b
            rAxis_0 = frame_b.r_0 - frame_a.r_0;
            rAxis_a = Frames.resolve2(frame_a.R, rAxis_0);
/* Determine relative Rotation R_rel_c from frame_a to frame_ia
     and absolute rotation of frame_a.R.
  */
            axisLength = sqrt(rAxis_0 * rAxis_0);
            assert(axisLength > 1.0e-15, "
Distance between frame_a and frame_b of a JointUPS joint
became zero. This is not allowed. If this occurs during
initialization, the initial conditions are probably wrong.");
            eAxis_a = rAxis_a / axisLength;
            n2_a = cross(n1_a, eAxis_a);
            length2_n2_a = n2_a * n2_a;
            assert(noEvent(length2_n2_a > 1.e-10), "
A Modelica.Mechanics.MultiBody.Joints.Assemblies.JointUPS joint (consisting of
a universal, prismatic and spherical joint) is in the singular
configuration of the universal joint. This means that axis 1 of
the universal joint defined via parameter \"n1_a\" is parallel to vector
\"eAxis_ia\" that is directed from the origin of frame_a to the
origin of frame_b. You may try to use another \"n1_a\" vector.
            ");
            length_n2_a = sqrt(length2_n2_a);
            e2_a = n2_a / length_n2_a;
            e3_a = cross(eAxis_a, e2_a);
/* The statements below are an efficient implementation of the
     original equations:
       T_ia_a = [eAxis_ia, e2_ia, e3_ia]*transpose([eAxis_a, e2_a, e3_a]);
       R_ia_a = Frames.from_T(T_ia_a,
                     Frames.TransformationMatrices.angularVelocity2(T_ia_a, der(T_ia_a)));
   To perform this, the rotation is split into two parts:
     R_ia_a : Rotation object from frame_a to frame_ia
     R_ia1_a: Rotation object from frame_a to frame_ia1
                (frame that is fixed in frame_ia such that x-axis
                is along the rod axis)
                T = transpose([eAxis_a, e2_a, e3_a]; w = w_rel_ia1
     R_ia2_a: Fixed rotation object from frame_ia1 to frame_ia
                T = [eAxis_a, e2_ia, e3_ia]; w = zeros(3)

   The difficult part is to compute w_rel_ia1:
      w_rel_ia1 = [  e3_a*der(e2_a);
                    -e3_a*der(eAxis_a);
                     e2_a*der(eAxis_a)]
   der(eAxis_a) is directly given, since eAxis_a is a function
   of translational quantities only.
      der(eAxis_a) = (der(rAxis_a) - eAxis_a*(eAxis_a*der(rAxis_a)))/axisLength
      der(n2_a)    = cross(n1_a, der(eAxis_a))
      der(e2_a)    = (der(n2_a) - e2_a*(e2_a*der(n2_a)))/length_n2_a
   Inserting these equations in w_rel_ia1 results in:
      e3_a*der(eAxis_a) = e3_a*der(rAxis_a)/axisLength       // e3_a*eAxis_a = 0
      e2_a*der(eAxis_a) = e2_a*der(rAxis_a)/axisLength       // e2_a*eAxis_a = 0
      e3_a*der(e2_a)    = e3_a*der(n2_a)/lenght_n2_a       // e3_a*e2_a = 0
                        = e3_a*cross(n1_a, der(eAxis_a))/length_n2_a
                        = e3_a*cross(n1_a, der(rAxis_a) - eAxis_a*(eAxis_a*der(rAxis_a)))/(length_n2_a*axisLength)
                        = e3_a*cross(n1_a, der(rAxis_a))/(length_n2_a*axisLength)
   Furthermore, we have:
     rAxis_a      = resolve2(frame_a.R, rAxis_0);
     der(rAxis_a) = resolve2(frame_a.R, der(rAxis_0)) - cross(frame_a.R.w, rAxis_a));
*/
            der_rAxis_a_L = (Frames.resolve2(frame_a.R, der(rAxis_0)) - cross(frame_a.R.w, rAxis_a)) / axisLength;
            w_rel_ia1 = {e3_a * cross(n1_a, der_rAxis_a_L) / length_n2_a, -e3_a * der_rAxis_a_L, e2_a * der_rAxis_a_L};
            R_ia1_a = Frames.from_T(transpose([eAxis_a, e2_a, e3_a]), w_rel_ia1);
            R_ia2_a = Frames.from_T([eAxis_ia, e2_ia, e3_ia], zeros(3));
            R_ia_a = Frames.absoluteRotation(R_ia1_a, R_ia2_a);
/*
  T_ia_a = [eAxis_ia, e2_ia, e3_ia]*transpose([eAxis_a, e2_a, e3_a]);
  R_ia_a = Frames.from_T(T_ia_a, Frames.TransformationMatrices.angularVelocity2
    (T_ia_a, der(T_ia_a)));
*/
// Compute kinematic quantities of frame_ia and frame_ib
            frame_ia.r_0 = frame_a.r_0;
            frame_ib.r_0 = frame_b.r_0;
            frame_ia.R = Frames.absoluteRotation(frame_a.R, R_ia_a);
            frame_ib.R = frame_ia.R;
/* In the following formulas f_a, f_b, f_ia, f_ib, t_a, t_b, t_ia, t_ib are
     the forces and torques at frame_a, frame_b, frame_ia, frame_ib respectively,
     resolved in frame_a. eAxis, e2, e3 are the unit vectors resolved in frame_a.
     Torque balance at the rod around the origin of frame_a:
       0 = t_a + t_ia + t_ib + cross(rAxis, (f_b+f_ib))
     with
         rAxis = axisLength*eAxis
         f_bd  = f_b + f_ib
         f_bd  = f*eAxis + f_bd[2]*e2 + f_bd[3]*e3
     follows:
         0 = t_a + t_ia + axisLength*(f_bd[2]*e_z - f_bd[3]*e_y)
     The projection of t_a with respect to universal joint axes vanishes:
       e1*t_a = 0
       e2*t_a = 0
     Therefore:
        0 = e1*(t_ia + t_ib) + axisLength*f_bd[2]*(e1*e3)
        0 = e2*(t_ia + t_ib) - axisLength*f_bd[3]
     or
        f_bd = f*eAxis - e2*(e1*(t_ia+t_ib))/(axisLength*(e1*e3)) +
                e3*(e2*(t_ia+t_ib))/axisLength
     Force balance:
        0 = f_a + f_bd + f_ia
  */
            f_c_a = Frames.resolve1(R_ia_a, frame_ia.f);
            t_cd_a = Frames.resolve1(R_ia_a, frame_ia.t + frame_ib.t);
            f_bd_a = (-eAxis_a * f) - e2_a * (n1_a * t_cd_a / (axisLength * (n1_a * e3_a))) + e3_a * (e2_a * t_cd_a / axisLength);
            zeros(3) = frame_b.f + Frames.resolveRelative(frame_ib.f, frame_ib.R, frame_b.R) - Frames.resolveRelative(f_bd_a, frame_a.R, frame_b.R);
            zeros(3) = frame_b.t;
            zeros(3) = frame_a.f + f_c_a + f_bd_a;
            zeros(3) = frame_a.t + t_cd_a + cross(rAxis_a, f_bd_a);
// Measure power for test purposes
            if checkTotalPower then
              totalPower = frame_a.f * Frames.resolve2(frame_a.R, der(frame_a.r_0)) + frame_b.f * Frames.resolve2(frame_b.R, der(frame_b.r_0)) + frame_ia.f * Frames.resolve2(frame_ia.R, der(frame_ia.r_0)) + frame_ib.f * Frames.resolve2(frame_ib.R, der(frame_ib.r_0)) + frame_a.t * Frames.angularVelocity2(frame_a.R) + frame_b.t * Frames.angularVelocity2(frame_b.R) + frame_ia.t * Frames.angularVelocity2(frame_ia.R) + frame_ib.t * Frames.angularVelocity2(frame_ib.R) + axis.f * der(axis.s) + bearing.f * der(bearing.s);
            else
              totalPower = 0;
            end if;
            annotation(
              Documentation(info = "<html>
<p>
This component consists of a <b>universal</b> joint at frame_a,
a <b>spherical</b> joint at frame_b and a <b>prismatic</b> joint along the
line connecting the origin of frame_a and the origin of frame_b,
see the default animation in the following figure (the axes vectors
are not part of the default animation):
</p>

<p>
<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Joints/JointUPS.png\" ALT=\"model Joints.Assemblies.JointUPS\">
</p>

<p>
This joint aggregation has no mass and no inertia and
introduces neither constraints nor potential state variables.
It is especially useful to build up more complicated force elements
where the mass and/or inertia of the force element shall be taken
into account.
</p>
<p>
The universal joint is defined in the following way:
</p>
<ul>
<li> The rotation <b>axis</b> of revolute joint <b>1</b> is along parameter
     vector n1_a which is fixed in frame_a.</li>
<li> The rotation <b>axis</b> of revolute joint <b>2</b> is perpendicular to
     axis 1 and to the line connecting the universal and the spherical joint.</li>
</ul>
<p>
The definition of axis 2 of the universal joint is performed according
to the most often occurring case. In a future release, axis 2 might
be explicitly definable via a parameter. However, the treatment is much more
complicated and the number of operations is considerably higher,
if axis 2 is not orthogonal to axis 1 and to the connecting rod.
</p>
<p>
Note, there is a <b>singularity</b> when axis 1 and the connecting line are parallel
to each other. Therefore, if possible n1_a should be selected in such a way that it
is perpendicular to nAxis_ia in the initial configuration (i.e., the
distance to the singularity is as large as possible).
</p>
<p>
An additional <b>frame_ia</b> is present. It is <b>fixed</b> on the line
connecting the universal and the spherical joint at the
origin of <b>frame_a</b>. The placement of frame_ia on this line
is implicitly defined by the universal joint (frame_a and frame_ia coincide
when the angles of the two revolute joints of the universal joint are zero)
and by parameter vector <b>nAxis_ia</b>, an axis vector directed
along the line from the origin of frame_a to the spherical joint,
resolved in frame_<b>ia</b>.
</p>
<p>
An additional <b>frame_ib</b> is present. It is <b>fixed</b> in the line
connecting the prismatic and the spherical joint at the
origin of <b>frame_b</b>.
It is always parallel to <b>frame_ia</b>.
</p>
<p>
Note, this joint aggregation can be used in cases where
in reality a rod with spherical joints at each end are present.
Such a system has an additional degree of freedom to rotate
the rod along its axis. In practice this rotation is usually
of no interest and is mathematically removed by replacing one
of the spherical joints by a universal joint.
</p>
<p>
The easiest way to define the parameters of this joint is by moving the
MultiBody system in a <b>reference configuration</b> where <b>all frames</b>
of all components are <b>parallel</b> to each other (alternatively,
at least frame_a, frame_ia and frame_ib of the JointUSP joint
should be parallel to each other when defining an instance of this
component).
</p>
</html>"),
              Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}, initialScale = 0.2), graphics = {Text(extent = {{-140, -50}, {140, -75}}, lineColor = {0, 0, 255}, textString = "%name"), Ellipse(extent = {{-100, -40}, {-19, 40}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.Sphere, fillColor = {192, 192, 192}), Ellipse(extent = {{-90, -30}, {-29, 29}}, lineColor = {160, 160, 164}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-60, 41}, {-9, -44}}, lineColor = {255, 255, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{-60, 40}, {-60, -40}}, color = {0, 0, 0}, thickness = 0.5), Ellipse(extent = {{-83, -17}, {-34, 21}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.Sphere, fillColor = {192, 192, 192}), Ellipse(extent = {{-74, -12}, {-40, 15}}, lineColor = {160, 160, 164}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Polygon(points = {{-72, -20}, {-89, 3}, {-69, 25}, {-45, 27}, {-72, -20}}, pattern = LinePattern.None, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Line(points = {{-60, 40}, {-60, -10}}, color = {0, 0, 0}, thickness = 0.5), Line(points = {{-49, 20}, {-69, -15}}, color = {0, 0, 0}, thickness = 0.5), Ellipse(extent = {{44, 14}, {73, -14}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Ellipse(extent = {{20, -40}, {100, 40}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.Sphere, fillColor = {192, 192, 192}), Ellipse(extent = {{30, -30}, {90, 30}}, lineColor = {192, 192, 192}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-22, 45}, {40, -43}}, lineColor = {255, 255, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Ellipse(extent = {{45, 14}, {74, -14}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.Sphere, fillColor = {192, 192, 192}), Text(extent = {{-98, 84}, {-60, 65}}, lineColor = {128, 128, 128}, textString = "ia"), Line(points = {{-40, 0}, {-40, 90}, {-80, 90}, {-80, 97}}, color = {95, 95, 95}, thickness = 0.5), Text(extent = {{61, 86}, {109, 64}}, lineColor = {128, 128, 128}, textString = "ib"), Rectangle(extent = {{-35, -13}, {-6, 14}}, pattern = LinePattern.None, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Rectangle(extent = {{-35, 14}, {-6, 18}}, pattern = LinePattern.None, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Rectangle(extent = {{-6, -7}, {46, 6}}, pattern = LinePattern.None, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Rectangle(extent = {{-6, 6}, {46, 10}}, pattern = LinePattern.None, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Line(points = {{-6, -13}, {-6, 18}}, color = {0, 0, 0}), Line(points = {{60, -1}, {60, 90}, {80, 90}, {80, 97}}, color = {95, 95, 95}, thickness = 0.5), Line(points = {{60, 90}, {40, 90}, {40, 95}}, color = {95, 95, 95}, thickness = 0.5), Line(points = {{-30, 70}, {10, 70}}, color = {0, 0, 0}), Polygon(points = {{30, 70}, {10, 76}, {10, 63}, {30, 70}}, lineColor = {128, 128, 128}, fillColor = {128, 128, 128}, fillPattern = FillPattern.Solid), Line(points = {{-40, 90}, {-40, 90}, {-40, 95}}, color = {95, 95, 95}, thickness = 0.5)}),
              Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}, initialScale = 0.2), graphics = {Line(points = {{-60, -70}, {46, -70}}, color = {0, 0, 255}), Polygon(points = {{60, -70}, {45, -64}, {45, -76}, {60, -70}}, lineColor = {0, 0, 255}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-62, -73}, {65, -90}}, textString = "rAxis", lineColor = {0, 0, 255}), Ellipse(extent = {{-100, -40}, {-19, 40}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.Sphere, fillColor = {192, 192, 192}), Ellipse(extent = {{-90, -30}, {-29, 29}}, lineColor = {160, 160, 164}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-60, 41}, {-19, -41}}, lineColor = {255, 255, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{-60, 40}, {-60, -40}}, color = {0, 0, 0}, thickness = 0.5), Ellipse(extent = {{-83, -17}, {-34, 21}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.Sphere, fillColor = {192, 192, 192}), Ellipse(extent = {{-74, -12}, {-40, 15}}, lineColor = {160, 160, 164}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Polygon(points = {{-72, -20}, {-89, 3}, {-69, 25}, {-45, 27}, {-72, -20}}, pattern = LinePattern.None, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Line(points = {{-60, 40}, {-60, -10}}, color = {0, 0, 0}, thickness = 0.5), Line(points = {{-49, 20}, {-69, -15}}, color = {0, 0, 0}, thickness = 0.5), Line(points = {{-40, 0}, {-40, 90}, {-80, 90}, {-80, 99}}, color = {95, 95, 95}, thickness = 0.5), Polygon(points = {{7, -1}, {-5, 2}, {-5, -4}, {7, -1}}, lineColor = {0, 0, 255}), Line(points = {{-50, 19}, {-30, 57}}, color = {0, 0, 255}), Text(extent = {{-24, 74}, {7, 53}}, lineColor = {0, 0, 0}, textString = "e2"), Polygon(points = {{-25, 64}, {-33, 56}, {-27, 53}, {-25, 64}}, lineColor = {0, 0, 255}), Line(points = {{-60, 41}, {-60, 65}}, color = {0, 0, 255}), Polygon(points = {{-60, 75}, {-64, 63}, {-56, 63}, {-60, 75}}, lineColor = {0, 0, 255}), Text(extent = {{-96, 82}, {-65, 61}}, lineColor = {0, 0, 0}, textString = "n1"), Line(points = {{-60, -40}, {-60, -72}}, color = {0, 0, 255}), Ellipse(extent = {{20, -40}, {100, 40}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.Sphere, fillColor = {192, 192, 192}), Ellipse(extent = {{30, -30}, {90, 30}}, lineColor = {192, 192, 192}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-22, 45}, {40, -43}}, lineColor = {255, 255, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Ellipse(extent = {{45, 14}, {74, -14}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.Sphere, fillColor = {128, 128, 128}), Line(points = {{60, 0}, {60, -74}}, color = {0, 0, 255}), Rectangle(extent = {{-35, 14}, {-6, 18}}, pattern = LinePattern.None, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Rectangle(extent = {{-35, -13}, {-6, 14}}, pattern = LinePattern.None, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Rectangle(extent = {{-6, 6}, {46, 10}}, pattern = LinePattern.None, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Rectangle(extent = {{-6, -7}, {46, 6}}, pattern = LinePattern.None, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Line(points = {{-6, -13}, {-6, 18}}, color = {0, 0, 0}), Text(extent = {{-40, -2}, {-1, -16}}, lineColor = {0, 0, 0}, textString = "nAxis"), Line(points = {{-61, 1}, {-2, 1}}, color = {0, 0, 255}), Polygon(points = {{10, 1}, {-2, 4}, {-2, -2}, {10, 1}}, lineColor = {0, 0, 255}), Line(points = {{60, -1}, {60, 90}, {80, 90}, {80, 99}}, color = {95, 95, 95}, thickness = 0.5), Text(extent = {{-24, 117}, {-9, 102}}, textString = "f", lineColor = {0, 0, 255}), Polygon(points = {{-26, 103}, {-36, 100}, {-26, 97}, {-26, 103}}, lineColor = {0, 0, 255}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid), Polygon(points = {{26, 103}, {36, 100}, {26, 97}, {26, 103}}, lineColor = {0, 0, 255}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid), Line(points = {{14, 100}, {36, 100}}, color = {0, 0, 255}), Text(extent = {{12, 116}, {27, 101}}, textString = "f", lineColor = {0, 0, 255}), Polygon(points = {{30, 93}, {40, 90}, {30, 87}, {30, 93}}, lineColor = {128, 128, 128}, fillColor = {128, 128, 128}, fillPattern = FillPattern.Solid), Line(points = {{-40, 90}, {40, 90}}, color = {128, 128, 128}), Line(points = {{-25, 100}, {-10, 100}}, color = {0, 0, 255}), Text(extent = {{-18, 90}, {19, 77}}, lineColor = {128, 128, 128}, textString = "s"), Line(points = {{60, 90}, {40, 90}, {40, 98}}, color = {95, 95, 95}, thickness = 0.5), Line(points = {{-40, 90}, {-40, 96}, {-40, 98}}, color = {135, 135, 135}, thickness = 0.5)}));
          end JointUPS;

          model JointUSR "Universal - spherical - revolute joint aggregation (no constraints, no potential states)"
            import Modelica.Mechanics.MultiBody.Types;
            extends Interfaces.PartialTwoFramesDoubleSize;
            Modelica.Mechanics.MultiBody.Interfaces.Frame_a frame_ia "Coordinate system at origin of frame_a fixed at connecting rod of universal and spherical joint" annotation(
              Placement(transformation(origin = {-80, 100}, extent = {{-8, -8}, {8, 8}}, rotation = 90)));
            Modelica.Mechanics.MultiBody.Interfaces.Frame_b frame_ib "Coordinate system at origin of frame_b fixed at connecting rod of spherical and revolute joint" annotation(
              Placement(transformation(origin = {80, 100}, extent = {{-8, 8}, {8, -8}}, rotation = 270)));
            Modelica.Mechanics.MultiBody.Interfaces.Frame_b frame_im "Coordinate system at origin of spherical joint fixed at connecting rod of spherical and revolute joint" annotation(
              Placement(transformation(origin = {0, 100}, extent = {{8, -8}, {-8, 8}}, rotation = 270)));
            Modelica.Mechanics.Rotational.Interfaces.Flange_a axis "1-dim. rotational flange that drives the revolute joint" annotation(
              Placement(transformation(extent = {{105, 85}, {95, 75}}, rotation = 0)));
            Modelica.Mechanics.Rotational.Interfaces.Flange_b bearing "1-dim. rotational flange of the drive bearing of the revolute joint" annotation(
              Placement(transformation(extent = {{95, 45}, {105, 35}}, rotation = 0)));
            parameter Boolean animation = true "= true, if animation shall be enabled";
            parameter Boolean showUniversalAxes = true "= true, if universal joint shall be visualized with two cylinders, otherwise with a sphere (provided animation=true)";
            parameter Modelica.Mechanics.MultiBody.Types.Axis n1_a = {0, 0, 1} "Axis 1 of universal joint fixed and resolved in frame_a (axis 2 is orthogonal to axis 1 and to rod 1)" annotation(
              Evaluate = true);
            parameter Modelica.Mechanics.MultiBody.Types.Axis n_b = {0, 0, 1} "Axis of revolute joint fixed and resolved in frame_b" annotation(
              Evaluate = true);
            parameter SI.Position rRod1_ia[3] = {1, 0, 0} "Vector from origin of frame_a to spherical joint, resolved in frame_ia" annotation(
              Evaluate = true);
            parameter SI.Position rRod2_ib[3] = {-1, 0, 0} "Vector from origin of frame_ib to spherical joint, resolved in frame_ib";
            parameter Cv.NonSIunits.Angle_deg phi_offset = 0 "Relative angle offset of revolute joint (angle = phi(t) + from_deg(phi_offset))";
            parameter Cv.NonSIunits.Angle_deg phi_guess = 0 "Select the configuration such that at initial time |phi(t0) - from_deg(phi_guess)| is minimal";
            parameter SI.Diameter sphereDiameter = world.defaultJointLength "Diameter of the spheres representing the universal and the spherical joint" annotation(
              Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            input Types.Color sphereColor = Modelica.Mechanics.MultiBody.Types.Defaults.JointColor "Color of the spheres representing the universal and the spherical joint" annotation(
              Dialog(colorSelector = true, tab = "Animation", group = "if animation = true", enable = animation));
            parameter SI.Diameter rod1Diameter = sphereDiameter / Types.Defaults.JointRodDiameterFraction "Diameter of rod 1 connecting the universal and the spherical joint" annotation(
              Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            input Types.Color rod1Color = Modelica.Mechanics.MultiBody.Types.Defaults.RodColor "Color of rod 1 connecting the universal and the spherical joint" annotation(
              Dialog(colorSelector = true, tab = "Animation", group = "if animation = true", enable = animation));
            parameter SI.Diameter rod2Diameter = rod1Diameter "Diameter of rod 2 connecting the revolute and the spherical joint" annotation(
              Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            input Types.Color rod2Color = rod1Color "Color of rod 2 connecting the revolute and the spherical joint" annotation(
              Dialog(colorSelector = true, tab = "Animation", group = "if animation = true", enable = animation));
            parameter SI.Diameter revoluteDiameter = world.defaultJointWidth "Diameter of cylinder representing the revolute joint" annotation(
              Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            parameter SI.Distance revoluteLength = world.defaultJointLength "Length of cylinder representing the revolute joint" annotation(
              Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            input Types.Color revoluteColor = Modelica.Mechanics.MultiBody.Types.Defaults.JointColor "Color of cylinder representing the revolute joint" annotation(
              Dialog(colorSelector = true, tab = "Animation", group = "if animation = true", enable = animation));
            input Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(
              Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            parameter SI.Distance cylinderLength = world.defaultJointLength "Length of cylinders representing the two universal joint axes" annotation(
              Dialog(tab = "Animation", group = "if animation = true and showUniversalAxes", enable = animation and showUniversalAxes));
            parameter SI.Distance cylinderDiameter = world.defaultJointWidth "Diameter of cylinders representing the two universal joint axes" annotation(
              Dialog(tab = "Animation", group = "if animation = true and showUniversalAxes", enable = animation and showUniversalAxes));
            input Types.Color cylinderColor = Modelica.Mechanics.MultiBody.Types.Defaults.JointColor "Color of cylinders representing the two universal joint axes" annotation(
              Dialog(colorSelector = true, tab = "Animation", group = "if animation = true and showUniversalAxes", enable = animation and showUniversalAxes));
            parameter Boolean checkTotalPower = false "= true, if total power flowing into this component shall be determined (must be zero)" annotation(
              Dialog(tab = "Advanced"));
            final parameter Real eRod1_ia[3](each final unit = "1") = rod1.eRod_ia "Unit vector from origin of frame_a to origin of spherical joint, resolved in frame_ia";
            final parameter Real e2_ia[3](each final unit = "1") = rod1.e2_ia "Unit vector in direction of axis 2 of universal joint, resolved in frame_ia";
            final parameter SI.Distance rod1Length = rod1.rodLength "Length of rod 1 (= distance between universal and spherical joint)";
            SI.Power totalPower "Total power flowing into this element, if checkTotalPower=true (otherwise dummy)";
            SI.Position aux "Denominator used to compute force in rod connecting universal and spherical joint";
            SI.Force f_rod "Constraint force in direction of the rod (positive, if rod is pressed)";
            Modelica.Mechanics.MultiBody.Joints.Internal.RevoluteWithLengthConstraint revolute(animation = animation, lengthConstraint = rod1Length, n = n_b, phi_offset = phi_offset, phi_guess = phi_guess, cylinderDiameter = revoluteDiameter, cylinderLength = revoluteLength, cylinderColor = revoluteColor, specularCoefficient = specularCoefficient) annotation(
              Placement(transformation(extent = {{75, -20}, {35, 20}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Joints.UniversalSpherical rod1(animation = animation, showUniversalAxes = showUniversalAxes, rRod_ia = rRod1_ia, n1_a = n1_a, sphereDiameter = sphereDiameter, sphereColor = sphereColor, rodWidth = rod1Diameter, rodHeight = rod1Diameter, rodColor = rod1Color, cylinderLength = cylinderLength, cylinderDiameter = cylinderDiameter, cylinderColor = cylinderColor, specularCoefficient = specularCoefficient, kinematicConstraint = false, constraintResidue = rod1.f_rod - f_rod) annotation(
              Placement(transformation(extent = {{-92, -20}, {-52, 20}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Parts.FixedTranslation rod2(animation = animation, width = rod2Diameter, height = rod2Diameter, color = rod2Color, specularCoefficient = specularCoefficient, r = rRod2_ib) annotation(
              Placement(transformation(extent = {{15, -20}, {-25, 20}}, rotation = 0)));
            Sensors.RelativePosition relativePosition(resolveInFrame = Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB.frame_a) annotation(
              Placement(transformation(extent = {{60, -70}, {40, -90}}, rotation = 0)));
            Modelica.Blocks.Sources.Constant position_b[3](k = rRod2_ib) annotation(
              Placement(transformation(extent = {{-20, -50}, {0, -30}}, rotation = 0)));
          equation
// Connections.root(frame_ib.R);
/* Compute the unknown force in the rod of the rod1 joint
     by a torque balance at the revolute joint:
       0 = revolute.frame_b.t + frame_ib.t + frame_im.t + cross(rRod2_ib, frame_im.f)
           + cross(r_ib, -rod1.f_b_a1)
           + cross(r_ib, Frames.resolve2(rod1.R_rel, rod1.f_rod*rod1.eRod1_ia))
     The condition is that the projection of the torque in the revolute
     joint along the axis of the revolute joint is equal to the driving
     axis torque in the flange:
       -revolute.tau = revolute.e*frame_b.t
     Therefore, we have
        tau = e*(frame_ib.t  + frame_im.t + cross(rRod2_ib, frame_im.f)
              + cross(rRod2_ib, -rod1.f_b_a1))
              + e*cross(rRod2_ib, Frames.resolve2(rod1.R_rel, rod1.f_rod*rod1.eRod_a))
            = e*(frame_ib.t + frame_im.t + cross(rRod2_ib, frame_im.f)
              + cross(rRod2_ib, -rod.f_b_a1))
              + rod1.f_rod*e*cross(rRod2_ib, Frames.resolve2(rod1.R_rel, rod1.eRod_a))
     Solving this equation for f_rod results in
       f_rod = (-tau - e*(frame_ib.t + frame_im.t + cross(rRod2_ib, frame_im.f)
               + cross(rRod2_ib, -rod1.f_b_a1)))
               / (cross(e,rRod2_ib)*Frames.resolve2(rod1.R_rel, rod1.eRod_a))
     Additionally, a guard against division by zero is introduced

     f_rod is passed to component JointsUSR.rod1 via variable "constraintResidue" in the Advanced menu
  */
            aux = cross(revolute.e, rRod2_ib) * Frames.resolveRelative(rod1.eRod_a, rod1.frame_a.R, rod1.frame_b.R);
            f_rod = ((-revolute.tau) - revolute.e * (frame_ib.t + frame_im.t + cross(rRod2_ib, frame_im.f) - cross(rRod2_ib, Frames.resolveRelative(rod1.f_b_a1, rod1.frame_a.R, rod1.frame_b.R)))) / noEvent(if abs(aux) < 1.e-10 then 1.e-10 else aux);
// Measure power for test purposes
            if checkTotalPower then
              totalPower = frame_a.f * Frames.resolve2(frame_a.R, der(frame_a.r_0)) + frame_b.f * Frames.resolve2(frame_b.R, der(frame_b.r_0)) + frame_ia.f * Frames.resolve2(frame_ia.R, der(frame_ia.r_0)) + frame_ib.f * Frames.resolve2(frame_ib.R, der(frame_ib.r_0)) + frame_im.f * Frames.resolve2(frame_im.R, der(frame_im.r_0)) + frame_a.t * Frames.angularVelocity2(frame_a.R) + frame_b.t * Frames.angularVelocity2(frame_b.R) + frame_ia.t * Frames.angularVelocity2(frame_ia.R) + frame_ib.t * Frames.angularVelocity2(frame_ib.R) + frame_im.t * Frames.angularVelocity2(frame_im.R) + axis.tau * der(axis.phi) + bearing.tau * der(bearing.phi);
            else
              totalPower = 0;
            end if;
            connect(revolute.frame_b, rod2.frame_a) annotation(
              Line(points = {{35, 0}, {15, 0}}, color = {95, 95, 95}, thickness = 0.5));
            connect(rod2.frame_b, rod1.frame_b) annotation(
              Line(points = {{-25, 0}, {-52, 0}}, color = {95, 95, 95}, thickness = 0.5));
            connect(revolute.frame_a, frame_b) annotation(
              Line(points = {{75, 0}, {100, 0}}, color = {95, 95, 95}, thickness = 0.5));
            connect(rod2.frame_a, frame_ib) annotation(
              Line(points = {{15, 0}, {26, 0}, {26, 70}, {80, 70}, {80, 100}}, color = {95, 95, 95}, thickness = 0.5));
            connect(rod1.frame_a, frame_a) annotation(
              Line(points = {{-92, 0}, {-100, 0}}, color = {95, 95, 95}, thickness = 0.5));
            connect(relativePosition.frame_b, frame_a) annotation(
              Line(points = {{40, -80}, {-96, -80}, {-96, 0}, {-100, 0}}, color = {95, 95, 95}, pattern = LinePattern.Dot));
            connect(relativePosition.frame_a, frame_b) annotation(
              Line(points = {{60, -80}, {96, -80}, {96, 0}, {100, 0}}, color = {95, 95, 95}, pattern = LinePattern.Dot));
            connect(position_b.y, revolute.position_b) annotation(
              Line(points = {{1, -40}, {20, -40}, {20, -12}, {31, -12}}, color = {0, 0, 127}));
            connect(rod2.frame_b, frame_im) annotation(
              Line(points = {{-25, 0}, {-40, 0}, {-40, 80}, {0, 80}, {0, 100}}, color = {95, 95, 95}, thickness = 0.5));
            connect(rod1.frame_ia, frame_ia) annotation(
              Line(points = {{-80, 20}, {-80, 100}}, color = {95, 95, 95}, thickness = 0.5));
            connect(revolute.axis, axis) annotation(
              Line(points = {{55, 20}, {55, 60}, {90, 60}, {90, 80}, {100, 80}}, color = {0, 0, 0}));
            connect(relativePosition.r_rel, revolute.position_a) annotation(
              Line(points = {{50, -69}, {50, -40}, {90, -40}, {90, -12}, {79, -12}}, color = {0, 0, 127}, smooth = Smooth.None));
            connect(revolute.bearing, bearing) annotation(
              Line(points = {{67, 20}, {67, 40}, {100, 40}}, color = {0, 0, 0}, smooth = Smooth.None));
            annotation(
              Documentation(info = "<html>
<p>
This component consists of a <b>universal</b> joint at frame_a, a <b>revolute</b>
joint at frame_b and a <b>spherical</b> joint which is connected via <b>rod1</b>
to the universal and via <b>rod2</b> to the revolute joint, see the default
animation in the following figure (the axes vectors are not part of the
default animation):
</p>

<p>
<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Joints/JointUSR.png\" ALT=\"model Joints.Assemblies.JointUSR\">
</p>

<p>
This joint aggregation has no mass and no inertia and
introduces neither constraints nor potential state variables.
It should be used in kinematic loops whenever possible since
the non-linear system of equations introduced by this joint aggregation
is solved <b>analytically</b> (i.e., a solution is always computed, if a
unique solution exists).
</p>
<p>
The universal joint is defined in the following way:
</p>
<ul>
<li> The rotation <b>axis</b> of revolute joint <b>1</b> is along parameter
     vector n1_a which is fixed in frame_a.</li>
<li> The rotation <b>axis</b> of revolute joint <b>2</b> is perpendicular to
     axis 1 and to the line connecting the universal and the spherical joint
     (= rod 1).</li>
</ul>
<p>
The definition of axis 2 of the universal joint is performed according
to the most often occurring case. In a future release, axis 2 might
be explicitly definable via a parameter. However, the treatment is much more
complicated and the number of operations is considerably higher,
if axis 2 is not orthogonal to axis 1 and to the connecting rod.
</p>
<p>
Note, there is a <b>singularity</b> when axis 1 and the connecting rod are parallel
to each other. Therefore, if possible n1_a should be selected in such a way that it
is perpendicular to rRod1_ia in the initial configuration (i.e., the
distance to the singularity is as large as possible).
</p>
<p>
The rest of this joint aggregation is defined by the following parameters:
</p>
<ul>
<li> The position of the spherical joint with respect to the universal
     joint is defined by vector <b>rRod1_ia</b>. This vector is directed from
     frame_a to the spherical joint and is resolved in frame_ia
     (it is most simple to select frame_ia such that it is parallel to
     frame_a in the reference or initial configuration).</li>
<li> The position of the spherical joint with respect to the revolute
     joint is defined by vector <b>rRod2_ib</b>. This vector is directed from
     the inner frame of the revolute joint (frame_ib or revolute.frame_a)
     to the spherical joint and is resolved in frame_ib (note, that frame_ib
     and frame_b are parallel to each other).</li>
<li> The axis of rotation of the revolute joint is defined by axis
     vector <b>n_b</b>. It is fixed and resolved in frame_b.</li>
<li> When specifying this joint aggregation with the definitions above, <b>two</b>
     different <b>configurations</b> are possible. Via parameter <b>phi_guess</b>
     a guess value for revolute.phi(t0) at the initial time t0 is given. The configuration is selected that is closest to phi_guess (|revolute.phi - phi_guess| is minimal).</li>
</ul>
<p>
An additional <b>frame_ia</b> is present. It is <b>fixed</b> in the rod
connecting the universal and the spherical joint at the
origin of <b>frame_a</b>. The placement of frame_ia on the rod
is implicitly defined by the universal joint (frame_a and frame_ia coincide
when the angles of the two revolute joints of the universal joint are zero)
and by parameter vector <b>rRod1_ia</b>, the position vector
from the origin of frame_a to the spherical joint, resolved in frame_<b>ia</b>.
</p>
<p>
An additional <b>frame_ib</b> is present. It is <b>fixed</b> in the rod
connecting the revolute and the spherical joint at the side of the revolute
joint that is connected to this rod (= rod2.frame_a = revolute.frame_a).
</p>
<p>
An additional <b>frame_im</b> is present. It is <b>fixed</b> in the rod
connecting the revolute and the spherical joint at the side of the spherical
joint that is connected to this rod (= rod2.frame_b).
It is always parallel to <b>frame_ib</b>.
</p>
<p>
The easiest way to define the parameters of this joint is by moving the
MultiBody system in a <b>reference configuration</b> where <b>all frames</b>
of all components are <b>parallel</b> to each other (alternatively,
at least frame_a and frame_ia of the JointUSR joint
should be parallel to each other when defining an instance of this
component).
</p>
<p>
In the public interface of the JointUSR joint, the following
(final) <b>parameters</b> are provided:
</p>
<pre>
  <b>parameter</b> Real rod1Length(unit=\"m\")  \"Length of rod 1\";
  <b>parameter</b> Real eRod1_ia[3] \"Unit vector along rod 1, resolved in frame_ia\";
  <b>parameter</b> Real e2_ia  [3]  \"Unit vector along axis 2, resolved in frame_ia\";
</pre>
<p>
This allows a more convenient definition of data which is related to rod 1.
For example, if a box shall be connected at frame_ia directing from
the origin of frame_a to the middle of rod 1, this might be defined as:
</p>
<pre>
    Modelica.Mechanics.MultiBody.Joints.Assemblies.JointUSP jointUSR(rRod1_ia={1.2, 1, 0.2});
    Modelica.Mechanics.MultiBody.Visualizers.FixedShape     shape(shapeType       = \"box\",
                                               lengthDirection = jointUSR.eRod1_ia,
                                               widthDirection  = jointUSR.e2_ia,
                                               length          = jointUSR.rod1Length/2,
                                               width           = jointUSR.rod1Length/10);
  <b>equation</b>
    <b>connect</b>(jointUSP.frame_ia, shape.frame_a);
</pre>
</html>"),
              Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}, initialScale = 0.2), graphics = {Text(extent = {{-140, -41}, {140, -66}}, lineColor = {0, 0, 255}, textString = "%name"), Ellipse(extent = {{-100, -30}, {-40, 30}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.Sphere, fillColor = {192, 192, 192}), Ellipse(extent = {{-93, -22}, {-48, 23}}, lineColor = {160, 160, 164}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-70, 40}, {-39, -33}}, lineColor = {255, 255, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{-70, 28}, {-70, -30}}, color = {0, 0, 0}, thickness = 0.5), Ellipse(extent = {{-89, -18}, {-48, 18}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.Sphere, fillColor = {192, 192, 192}), Ellipse(extent = {{-84, -12}, {-53, 13}}, lineColor = {160, 160, 164}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Polygon(points = {{-81, -17}, {-92, -1}, {-83, 16}, {-57, 24}, {-81, -17}}, pattern = LinePattern.None, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Line(points = {{-70, 30}, {-70, -10}}, color = {0, 0, 0}, thickness = 0.5), Line(points = {{-61, 16}, {-79, -15}}, color = {0, 0, 0}, thickness = 0.5), Line(points = {{-50, 0}, {-50, 80}, {-80, 80}, {-80, 100}}, color = {95, 95, 95}, thickness = 0.5), Ellipse(extent = {{-40, -30}, {20, 30}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.Sphere, fillColor = {192, 192, 192}), Ellipse(extent = {{-33, -22}, {12, 23}}, lineColor = {192, 192, 192}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-44, 31}, {-14, -30}}, lineColor = {255, 255, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-23, 10}, {-3, -10}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Rectangle(extent = {{19, 6}, {61, -6}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192, 192, 192}), Rectangle(extent = {{-50, 5}, {-21, -5}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192, 192, 192}), Rectangle(extent = {{60, -30}, {76, 30}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192, 192, 192}), Rectangle(extent = {{85, -30}, {100, 30}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192, 192, 192}), Rectangle(extent = {{76, 10}, {85, -10}}, lineColor = {0, 0, 0}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Rectangle(extent = {{60, 30}, {76, -30}}, lineColor = {0, 0, 0}), Rectangle(extent = {{85, 30}, {100, -30}}, lineColor = {0, 0, 0}), Text(extent = {{40, 109}, {77, 91}}, lineColor = {128, 128, 128}, textString = "ib"), Text(extent = {{-124, 109}, {-95, 92}}, lineColor = {128, 128, 128}, textString = "ia"), Line(points = {{60, 30}, {60, 80}, {80, 80}, {80, 100}}, color = {95, 95, 95}, thickness = 0.5), Text(extent = {{-43, 108}, {-10, 92}}, lineColor = {128, 128, 128}, textString = "im"), Line(points = {{19, 6}, {19, 80}, {0, 80}, {0, 100}}, color = {95, 95, 95}, thickness = 0.5), Line(points = {{80, 80}, {101, 80}}, color = {128, 128, 128}, thickness = 0.5), Line(points = {{90, 30}, {90, 40}, {95, 40}}, color = {95, 95, 95}, thickness = 0.5)}));
          end JointUSR;

          model JointUSP "Universal - spherical - prismatic joint aggregation (no constraints, no potential states)"
            import Modelica.Mechanics.MultiBody.Types;
            extends Interfaces.PartialTwoFramesDoubleSize;
            Modelica.Mechanics.MultiBody.Interfaces.Frame_a frame_ia "Coordinate system at origin of frame_a fixed at connecting rod of universal and spherical joint" annotation(
              Placement(transformation(origin = {-80, 100}, extent = {{-8, -8}, {8, 8}}, rotation = 90)));
            Modelica.Mechanics.MultiBody.Interfaces.Frame_b frame_ib "Coordinate system at origin of frame_b fixed at connecting rod of spherical and prismatic joint" annotation(
              Placement(transformation(origin = {80, 100}, extent = {{-8, 8}, {8, -8}}, rotation = 270)));
            Modelica.Mechanics.MultiBody.Interfaces.Frame_b frame_im "Coordinate system at origin of spherical joint fixed at connecting rod of spherical and prismatic joint" annotation(
              Placement(transformation(origin = {0, 100}, extent = {{8, -8}, {-8, 8}}, rotation = 270)));
            Modelica.Mechanics.Translational.Interfaces.Flange_a axis "1-dim. translational flange that drives the prismatic joint" annotation(
              Placement(transformation(extent = {{95, 75}, {105, 85}}, rotation = 0)));
            Modelica.Mechanics.Translational.Interfaces.Flange_b bearing "1-dim. translational flange of the drive bearing of the prismatic joint" annotation(
              Placement(transformation(extent = {{105, 35}, {95, 45}}, rotation = 0)));
            parameter Boolean animation = true "= true, if animation shall be enabled";
            parameter Boolean showUniversalAxes = true "= true, if universal joint shall be visualized with two cylinders, otherwise with a sphere (provided animation=true)";
            parameter Modelica.Mechanics.MultiBody.Types.Axis n1_a = {0, 0, 1} "Axis 1 of universal joint fixed and resolved in frame_a (axis 2 is orthogonal to axis 1 and to rod 1)" annotation(
              Evaluate = true);
            parameter Modelica.Mechanics.MultiBody.Types.Axis n_b = {-1, 0, 0} "Axis of prismatic joint fixed and resolved in frame_b" annotation(
              Evaluate = true);
            parameter SI.Position rRod1_ia[3] = {1, 0, 0} "Vector from origin of frame_a to spherical joint, resolved in frame_ia" annotation(
              Evaluate = true);
            parameter SI.Position rRod2_ib[3] = {-1, 0, 0} "Vector from origin of frame_ib to spherical joint, resolved in frame_ib (frame_ib is parallel to frame_b)" annotation(
              Evaluate = true);
            parameter SI.Position s_offset = 0 "Relative distance offset of prismatic joint (distance between the prismatic joint frames = s(t) + s_offset)";
            parameter SI.Position s_guess = 0 "Select the configuration such that at initial time |s(t0)-s_guess| is minimal";
            parameter SI.Diameter sphereDiameter = world.defaultJointLength "Diameter of the spheres representing the universal and the spherical joint" annotation(
              Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            input Types.Color sphereColor = Modelica.Mechanics.MultiBody.Types.Defaults.JointColor "Color of the spheres representing the universal and the spherical joint" annotation(
              Dialog(colorSelector = true, tab = "Animation", group = "if animation = true", enable = animation));
            parameter SI.Diameter rod1Diameter = sphereDiameter / Types.Defaults.JointRodDiameterFraction "Diameter of rod 1 connecting the universal and the spherical joint" annotation(
              Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            input Types.Color rod1Color = Modelica.Mechanics.MultiBody.Types.Defaults.RodColor "Color of rod 1 connecting the universal and the spherical joint" annotation(
              Dialog(colorSelector = true, tab = "Animation", group = "if animation = true", enable = animation));
            parameter SI.Diameter rod2Diameter = rod1Diameter "Diameter of rod 2 connecting the prismatic and the spherical joint" annotation(
              Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            input Types.Color rod2Color = rod1Color "Color of rod 2 connecting the prismatic and the spherical joint" annotation(
              Dialog(colorSelector = true, tab = "Animation", group = "if animation = true", enable = animation));
            parameter Types.Axis boxWidthDirection = {0, 1, 0} "Vector in width direction of prismatic joint, resolved in frame_b" annotation(
              Evaluate = true,
              Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            parameter SI.Distance boxWidth = world.defaultJointWidth "Width of prismatic joint box" annotation(
              Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            parameter SI.Distance boxHeight = boxWidth "Height of prismatic joint box" annotation(
              Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            input Types.Color boxColor = sphereColor "Color of prismatic joint box" annotation(
              Dialog(colorSelector = true, tab = "Animation", group = "if animation = true", enable = animation));
            input Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(
              Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            parameter SI.Distance cylinderLength = world.defaultJointLength "Length of cylinders representing the two universal joint axes" annotation(
              Dialog(tab = "Animation", group = "if animation = true and showUniversalAxes", enable = animation and showUniversalAxes));
            parameter SI.Distance cylinderDiameter = world.defaultJointWidth "Diameter of cylinders representing the two universal joint axes" annotation(
              Dialog(tab = "Animation", group = "if animation = true and showUniversalAxes", enable = animation and showUniversalAxes));
            input Types.Color cylinderColor = Modelica.Mechanics.MultiBody.Types.Defaults.JointColor "Color of cylinders representing the two universal joint axes" annotation(
              Dialog(colorSelector = true, tab = "Animation", group = "if animation = true and showUniversalAxes", enable = animation and showUniversalAxes));
            parameter Boolean checkTotalPower = false "= true, if total power flowing into this component shall be determined (must be zero)" annotation(
              Dialog(tab = "Advanced"));
            final parameter Real eRod1_ia[3](each final unit = "1") = rod1.eRod_ia "Unit vector from origin of frame_a to origin of spherical joint, resolved in frame_ia";
            final parameter Real e2_ia[3](each final unit = "1") = rod1.e2_ia "Unit vector in direction of axis 2 of universal joint, resolved in frame_ia";
            final parameter SI.Distance rod1Length = rod1.rodLength "Length of rod 1 (= distance between universal and spherical joint)";
            SI.Force f_rod "Constraint force in direction of the rod (positive, if rod is pressed)";
            SI.Power totalPower "Total power flowing into this element, if checkTotalPower=true (otherwise dummy)";
            Modelica.Mechanics.MultiBody.Joints.Internal.PrismaticWithLengthConstraint prismatic(animation = animation, length = rod1.rodLength, n = n_b, s_offset = s_offset, s_guess = s_guess, boxWidthDirection = boxWidthDirection, boxWidth = boxWidth, boxHeight = boxHeight, boxColor = boxColor, specularCoefficient = specularCoefficient) annotation(
              Placement(transformation(extent = {{76, -20}, {36, 20}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Joints.UniversalSpherical rod1(animation = animation, showUniversalAxes = showUniversalAxes, rRod_ia = rRod1_ia, n1_a = n1_a, sphereDiameter = sphereDiameter, sphereColor = sphereColor, rodWidth = rod1Diameter, rodHeight = rod1Diameter, rodColor = rod1Color, specularCoefficient = specularCoefficient, cylinderLength = cylinderLength, cylinderDiameter = cylinderDiameter, cylinderColor = cylinderColor, kinematicConstraint = false, constraintResidue = rod1.f_rod - f_rod) annotation(
              Placement(transformation(extent = {{-92, -20}, {-52, 20}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Parts.FixedTranslation rod2(animation = animation, r = rRod2_ib, width = rod2Diameter, height = rod2Diameter, specularCoefficient = specularCoefficient, color = rod2Color) annotation(
              Placement(transformation(extent = {{0, 20}, {-40, -20}}, rotation = 0)));
            Sensors.RelativePosition relativePosition(resolveInFrame = Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB.frame_a) annotation(
              Placement(transformation(extent = {{50, -70}, {30, -90}}, rotation = 0)));
            Modelica.Blocks.Sources.Constant position_b[3](k = rRod2_ib) annotation(
              Placement(transformation(extent = {{-20, -60}, {0, -40}}, rotation = 0)));
          protected
            Real aux "Denominator used to compute force in rod connecting universal and spherical joint";
          equation
/* Compute the unknown force in rod1 connecting the universal and
     the spherical joint by a force balance at the prismatic joint
        0 = -prismatic.frame_b.f + frame_ib.f + frame_im.f - rod1.frame_b.f
     The force at rod1.frame_b is split into two parts:
        rod1.frame_b.f = Frames.resolve2(rod1.R_rel, rod1.f_b_a1 - rod1.f_rod*rod1.eRod_a)
     where rod1.f_rod is the unknown force in rod1.
     The condition is that the projection of the force in the prismatic
     joint along the axis of its translation axis is equal to the driving
     axis force in the flange:
       -prismatic.f = prismatic.e*prismatic.frame_b.f
     Therefore, we have with e=prismatic.e and f=prismatic.f
       -f = e*(frame_ib.f + frame_im.f
               - Frames.resolve2(rod1.R_rel, rod1.f_b_a1 - rod1.f_rod*rod1.eRod_a))
          = e*(frame_ib.f + frame_im.f - Frames.resolve2(rod1.R_rel, rod1.f_b_a1)
              + rod1.f_rod*Frames.resolve2(rod1.R_rel, rod1.eRod_a))
     Solving this equation for f_rod results in
       rod1.f_rod = -(f+e*(frame_ib.f + frame_im.f - Frames.resolve2(rod1.R_rel, rod1.f_b_a1)))
                   /(e*Frames.resolve2(rod1.R_rel, rod1.eRod_a))
     Additionally, a guard against division by zero is introduced
  */
            aux = prismatic.e * Frames.resolveRelative(rod1.eRod_a, rod1.frame_a.R, rod1.frame_b.R);
            f_rod = ((-prismatic.f) - prismatic.e * (frame_ib.f + frame_im.f - Frames.resolveRelative(rod1.f_b_a1, rod1.frame_a.R, rod1.frame_b.R))) / noEvent(if abs(aux) < 1.e-10 then 1.e-10 else aux);
// Measure power for test purposes
            if checkTotalPower then
              totalPower = frame_a.f * Frames.resolve2(frame_a.R, der(frame_a.r_0)) + frame_b.f * Frames.resolve2(frame_b.R, der(frame_b.r_0)) + frame_ia.f * Frames.resolve2(frame_ia.R, der(frame_ia.r_0)) + frame_ib.f * Frames.resolve2(frame_ib.R, der(frame_ib.r_0)) + frame_im.f * Frames.resolve2(frame_im.R, der(frame_im.r_0)) + frame_a.t * Frames.angularVelocity2(frame_a.R) + frame_b.t * Frames.angularVelocity2(frame_b.R) + frame_ia.t * Frames.angularVelocity2(frame_ia.R) + frame_ib.t * Frames.angularVelocity2(frame_ib.R) + frame_im.t * Frames.angularVelocity2(frame_im.R) + axis.f * der(axis.s) + bearing.f * der(bearing.s);
            else
              totalPower = 0;
            end if;
            connect(prismatic.frame_b, rod2.frame_a) annotation(
              Line(points = {{36, 0}, {0, 0}}, color = {95, 95, 95}, thickness = 0.5));
            connect(rod2.frame_b, rod1.frame_b) annotation(
              Line(points = {{-40, 0}, {-52, 0}}, color = {0, 0, 0}, thickness = 0.5));
            connect(prismatic.frame_a, frame_b) annotation(
              Line(points = {{76, 0}, {100, 0}}, color = {95, 95, 95}, thickness = 0.5));
            connect(rod2.frame_a, frame_ib) annotation(
              Line(points = {{0, 0}, {7, 0}, {7, 70}, {80, 70}, {80, 100}}, color = {95, 95, 95}, thickness = 0.5));
            connect(rod1.frame_a, frame_a) annotation(
              Line(points = {{-92, 0}, {-100, 0}}, color = {95, 95, 95}, thickness = 0.5));
            connect(relativePosition.frame_b, frame_a) annotation(
              Line(points = {{30, -80}, {-97, -80}, {-97, 0}, {-100, 0}}, color = {95, 95, 95}, pattern = LinePattern.Dot));
            connect(relativePosition.frame_a, frame_b) annotation(
              Line(points = {{50, -80}, {95, -80}, {95, 0}, {100, 0}}, color = {95, 95, 95}, pattern = LinePattern.Dot));
            connect(rod2.frame_b, frame_im) annotation(
              Line(points = {{-40, 0}, {-46, 0}, {-46, 80}, {0, 80}, {0, 100}}, color = {95, 95, 95}, thickness = 0.5));
            connect(rod1.frame_ia, frame_ia) annotation(
              Line(points = {{-80, 20}, {-80, 100}}, color = {95, 95, 95}, thickness = 0.5));
            connect(position_b.y, prismatic.position_b) annotation(
              Line(points = {{1, -50}, {10, -50}, {10, -12}, {32, -12}}, color = {0, 0, 127}));
            connect(prismatic.axis, axis) annotation(
              Line(points = {{40, 14}, {40, 56}, {90, 56}, {90, 80}, {100, 80}}, color = {0, 191, 0}));
            connect(prismatic.bearing, bearing) annotation(
              Line(points = {{64, 14}, {64, 40}, {100, 40}}, color = {0, 191, 0}));
            connect(relativePosition.r_rel, prismatic.position_a) annotation(
              Line(points = {{40, -69}, {40, -50}, {90, -50}, {90, -12}, {80, -12}}, color = {0, 0, 127}, smooth = Smooth.None));
            annotation(
              Documentation(info = "<html>
<p>
This component consists of a <b>universal</b> joint at frame_a, a <b>prismatic</b>
joint at frame_b and a <b>spherical</b> joint which is connected via <b>rod1</b>
to the universal and via <b>rod2</b> to the prismatic joint, see the default
animation in the following figure (the axes vectors are not part of the
default animation):
</p>

<p>
<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Joints/JointUSP.png\" ALT=\"model Joints.Assemblies.JointUSP\">
</p>

<p>
This joint aggregation has no mass and no inertia and
introduces neither constraints nor potential state variables.
It should be used in kinematic loops whenever possible since
the non-linear system of equations introduced by this joint aggregation
is solved <b>analytically</b> (i.e., a solution is always computed, if a
unique solution exists).
</p>
<p>
The universal joint is defined in the following way:
</p>
<ul>
<li> The rotation <b>axis</b> of revolute joint <b>1</b> is along parameter
     vector n1_a which is fixed in frame_a.</li>
<li> The rotation <b>axis</b> of revolute joint <b>2</b> is perpendicular to
     axis 1 and to the line connecting the universal and the spherical joint
     (= rod 1).</li>
</ul>
<p>
The definition of axis 2 of the universal joint is performed according
to the most often occurring case. In a future release, axis 2 might
be explicitly definable via a parameter. However, the treatment is much more
complicated and the number of operations is considerably higher,
if axis 2 is not orthogonal to axis 1 and to the connecting rod.
</p>
<p>
Note, there is a <b>singularity</b> when axis 1 and the connecting rod are parallel
to each other. Therefore, if possible n1_a should be selected in such a way that it
is perpendicular to rRod1_ia in the initial configuration (i.e., the
distance to the singularity is as large as possible).
</p>
<p>
The rest of this joint aggregation is defined by the following parameters:
</p>
<ul>
<li> The position of the spherical joint with respect to the universal
     joint is defined by vector <b>rRod1_ia</b>. This vector is directed from
     frame_a to the spherical joint and is resolved in frame_ia
     (it is most simple to select frame_ia such that it is parallel to
     frame_a in the reference or initial configuration).</li>
<li> The position of the spherical joint with respect to the prismatic
     joint is defined by vector <b>rRod2_ib</b>. This vector is directed from
     the inner frame of the prismatic joint (frame_ib or prismatic.frame_a)
     to the spherical joint and is resolved in frame_ib (note, that frame_ib
     and frame_b are parallel to each other).</li>
<li> The axis of translation of the prismatic joint is defined by axis
     vector <b>n_b</b>. It is fixed and resolved in frame_b.</li>
<li> The two frames of the prismatic joint, i.e., frame_b and frame_ib,
     are parallel to each other.
     The distance between the origins of these two frames along axis n_b
     is equal to \"prismatic.s(t) + s_offset\", where \"prismatic.s(t)\" is
     a time varying variable and \"s_offset\" is a fixed, constant offset
     parameter.</li>
<li> When specifying this joint aggregation with the definitions above, <b>two</b>
     different <b>configurations</b> are possible. Via parameter <b>s_guess</b>
     a guess value for prismatic.s(t0) at the initial time t0 is given. The configuration
     is selected that is closest to s_guess (|prismatic.s - s_guess| is minimal).</li>
</ul>
<p>
An additional <b>frame_ia</b> is present. It is <b>fixed</b> in the rod
connecting the universal and the spherical joint at the
origin of <b>frame_a</b>. The placement of frame_ia on the rod
is implicitly defined by the universal joint (frame_a and frame_ia coincide
when the angles of the two revolute joints of the universal joint are zero)
and by parameter vector <b>rRod1_ia</b>, the position vector
from the origin of frame_a to the spherical joint, resolved in frame_<b>ia</b>.
</p>
<p>
An additional <b>frame_ib</b> is present. It is <b>fixed</b> in the rod
connecting the prismatic and the spherical joint at the side of the prismatic
joint that is connected to this rod (= rod2.frame_a = prismatic.frame_a).
It is always parallel to <b>frame_b</b>.
</p>
<p>
An additional <b>frame_im</b> is present. It is <b>fixed</b> in the rod
connecting the prismatic and the spherical joint at the side of the spherical
joint that is connected to this rod (= rod2.frame_b).
It is always parallel to <b>frame_b</b>.
</p>
<p>
The easiest way to define the parameters of this joint is by moving the
MultiBody system in a <b>reference configuration</b> where <b>all frames</b>
of all components are <b>parallel</b> to each other (alternatively,
at least frame_a and frame_ia of the JointUSP joint
should be parallel to each other when defining an instance of this
component).
</p>
<p>
In the public interface of the JointUSP joint, the following
(final) <b>parameters</b> are provided:
</p>
<pre>
  <b>parameter</b> Real rod1Length(unit=\"m\")  \"Length of rod 1\";
  <b>parameter</b> Real eRod1_ia[3] \"Unit vector along rod 1, resolved in frame_ia\";
  <b>parameter</b> Real e2_ia  [3]  \"Unit vector along axis 2, resolved in frame_ia\";
</pre>
<p>
This allows a more convenient definition of data which is related to rod 1.
For example, if a box shall be connected at frame_ia directing from
the origin of frame_a to the middle of rod 1, this might be defined as:
</p>
<pre>
    Modelica.Mechanics.MultiBody.Joints.Assemblies.JointUSP jointUSP(rRod1_ia={1.2, 1, 0.2});
    Modelica.Mechanics.MultiBody.Visualizers.FixedShape     shape(shapeType       = \"box\",
                                               lengthDirection = jointUSP.eRod1_ia,
                                               widthDirection  = jointUSP.e2_ia,
                                               length          = jointUSP.rod1Length/2,
                                               width           = jointUSP.rod1Length/10);
  <b>equation</b>
    <b>connect</b>(jointUSP.frame_ia, shape.frame_a);
</pre>
</html>"),
              Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}, initialScale = 0.2), graphics = {Rectangle(extent = {{50, 20}, {80, -20}}, lineColor = {0, 0, 0}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Rectangle(extent = {{80, 30}, {100, -30}}, lineColor = {0, 0, 0}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Text(extent = {{-140, -45}, {140, -70}}, lineColor = {0, 0, 255}, textString = "%name"), Ellipse(extent = {{-100, -30}, {-40, 30}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.Sphere, fillColor = {192, 192, 192}), Ellipse(extent = {{-93, -22}, {-48, 23}}, lineColor = {160, 160, 164}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-70, 40}, {-39, -33}}, lineColor = {255, 255, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{-70, 28}, {-70, -30}}, color = {0, 0, 0}, thickness = 0.5), Ellipse(extent = {{-89, -18}, {-48, 18}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.Sphere, fillColor = {192, 192, 192}), Ellipse(extent = {{-84, -12}, {-53, 13}}, lineColor = {160, 160, 164}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Polygon(points = {{-81, -17}, {-92, -1}, {-83, 16}, {-57, 24}, {-81, -17}}, pattern = LinePattern.None, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Line(points = {{-70, 30}, {-70, -10}}, color = {0, 0, 0}, thickness = 0.5), Line(points = {{-61, 16}, {-79, -15}}, color = {0, 0, 0}, thickness = 0.5), Line(points = {{-50, 0}, {-50, 80}, {-80, 80}, {-80, 100}}, color = {95, 95, 95}, thickness = 0.5), Ellipse(extent = {{-40, -30}, {20, 30}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.Sphere, fillColor = {192, 192, 192}), Ellipse(extent = {{-33, -22}, {12, 23}}, lineColor = {192, 192, 192}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-44, 31}, {-14, -30}}, lineColor = {255, 255, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-23, 10}, {-3, -10}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Rectangle(extent = {{19, 6}, {50, -6}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192, 192, 192}), Rectangle(extent = {{-50, 5}, {-21, -5}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192, 192, 192}), Text(extent = {{37, 109}, {68, 90}}, lineColor = {128, 128, 128}, textString = "ib"), Text(extent = {{-124, 110}, {-93, 90}}, lineColor = {128, 128, 128}, textString = "ia"), Line(points = {{50, 6}, {50, 80}, {80, 80}, {80, 100}}, color = {95, 95, 95}, thickness = 0.5), Text(extent = {{-44, 111}, {-8, 91}}, lineColor = {128, 128, 128}, textString = "im"), Line(points = {{19, 6}, {19, 80}, {0, 80}, {0, 100}}, color = {95, 95, 95}, thickness = 0.5), Rectangle(extent = {{80, 24}, {100, 30}}, pattern = LinePattern.None, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Rectangle(extent = {{50, 14}, {80, 20}}, pattern = LinePattern.None, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Line(points = {{95, 80}, {79, 80}}, color = {135, 135, 135}, thickness = 0.5), Line(points = {{95, 40}, {90, 40}, {90, 30}}, color = {135, 135, 135}, thickness = 0.5)}),
              Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}, initialScale = 0.2), graphics = {Line(points = {{-78, 30}, {-50, 30}}, color = {128, 128, 128}, arrow = {Arrow.None, Arrow.Filled}), Text(extent = {{-76, 39}, {-49, 32}}, lineColor = {128, 128, 128}, textString = "rRod1_ia"), Text(extent = {{-27, 40}, {0, 33}}, lineColor = {128, 128, 128}, textString = "rRod2_ib"), Line(points = {{3, 30}, {-43, 30}}, color = {128, 128, 128}, arrow = {Arrow.None, Arrow.Filled})}));
          end JointUSP;

          model JointSSR "Spherical - spherical - revolute joint aggregation with mass (no constraints, no potential states)"
            import Modelica.Mechanics.MultiBody.Types;
            extends Interfaces.PartialTwoFramesDoubleSize;
            Modelica.Mechanics.MultiBody.Interfaces.Frame_b frame_ib "Coordinate system at origin of frame_b fixed at connecting rod of spherical and revolute joint" annotation(
              Placement(transformation(origin = {80, 100}, extent = {{-8, 8}, {8, -8}}, rotation = 270)));
            Modelica.Mechanics.MultiBody.Interfaces.Frame_b frame_im "Coordinate system at origin of spherical joint in the middle fixed at connecting rod of spherical and revolute joint" annotation(
              Placement(transformation(origin = {0, 100}, extent = {{8, -8}, {-8, 8}}, rotation = 270)));
            Modelica.Mechanics.Rotational.Interfaces.Flange_a axis "1-dim. rotational flange that drives the revolute joint" annotation(
              Placement(transformation(extent = {{105, 85}, {95, 75}}, rotation = 0)));
            Modelica.Mechanics.Rotational.Interfaces.Flange_b bearing "1-dim. rotational flange of the drive bearing of the revolute joint" annotation(
              Placement(transformation(extent = {{95, 45}, {105, 35}}, rotation = 0)));
            parameter Boolean animation = true "= true, if animation shall be enabled";
            parameter Boolean showMass = true "= true, if point mass on rod 1 shall be shown (provided animation = true and rod1Mass > 0)";
            parameter SI.Length rod1Length(min = Modelica.Constants.eps, start = 1) "Distance between the origins of the two spherical joints";
            parameter SI.Mass rod1Mass(min = 0) = 0 "Mass of rod 1 (= point mass located in middle of rod connecting the two spherical joints)";
            parameter Modelica.Mechanics.MultiBody.Types.Axis n_b = {0, 0, 1} "Axis of revolute joint fixed and resolved in frame_b";
            parameter SI.Position rRod2_ib[3] = {1, 0, 0} "Vector from origin of frame_ib to spherical joint in the middle, resolved in frame_ib";
            parameter Cv.NonSIunits.Angle_deg phi_offset = 0 "Relative angle offset of revolute joint (angle = phi(t) + from_deg(phi_offset))";
            parameter Cv.NonSIunits.Angle_deg phi_guess = 0 "Select the configuration such that at initial time |phi(t0) - from_deg(phi_guess)| is minimal";
            parameter SI.Diameter sphereDiameter = world.defaultJointLength "Diameter of the spheres representing the two spherical joints" annotation(
              Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            input Types.Color sphereColor = Modelica.Mechanics.MultiBody.Types.Defaults.JointColor "Color of the spheres representing the two spherical joints" annotation(
              Dialog(colorSelector = true, tab = "Animation", group = "if animation = true", enable = animation));
            parameter SI.Diameter rod1Diameter = sphereDiameter / Types.Defaults.JointRodDiameterFraction "Diameter of rod 1 connecting the two spherical joints" annotation(
              Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            input Types.Color rod1Color = Modelica.Mechanics.MultiBody.Types.Defaults.RodColor "Color of rod 1 connecting the two spherical joint" annotation(
              Dialog(colorSelector = true, tab = "Animation", group = "if animation = true", enable = animation));
            parameter SI.Diameter rod2Diameter = rod1Diameter "Diameter of rod 2 connecting the revolute joint and spherical joint 2" annotation(
              Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            input Types.Color rod2Color = rod1Color "Color of rod 2 connecting the revolute joint and spherical joint 2" annotation(
              Dialog(colorSelector = true, tab = "Animation", group = "if animation = true", enable = animation));
            parameter SI.Diameter revoluteDiameter = world.defaultJointWidth "Diameter of cylinder representing the revolute joint" annotation(
              Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            parameter SI.Distance revoluteLength = world.defaultJointLength "Length of cylinder representing the revolute joint" annotation(
              Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            input Types.Color revoluteColor = Modelica.Mechanics.MultiBody.Types.Defaults.JointColor "Color of cylinder representing the revolute joint" annotation(
              Dialog(colorSelector = true, tab = "Animation", group = "if animation = true", enable = animation));
            input Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(
              Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            parameter Boolean checkTotalPower = false "= true, if total power flowing into this component shall be determined (must be zero)" annotation(
              Dialog(tab = "Advanced"));
            SI.Position aux "Denominator used to compute force in rod connecting universal and spherical joint";
            SI.Force f_rod "Constraint force in direction of the rod (positive, if rod is pressed)";
            SI.Power totalPower "Total power flowing into this element, if checkTotalPower=true (otherwise dummy)";
            Modelica.Mechanics.MultiBody.Joints.Internal.RevoluteWithLengthConstraint revolute(animation = animation, lengthConstraint = rod1Length, n = n_b, phi_offset = phi_offset, phi_guess = phi_guess, cylinderDiameter = revoluteDiameter, cylinderLength = revoluteLength, cylinderColor = revoluteColor, specularCoefficient = specularCoefficient) annotation(
              Placement(transformation(extent = {{75, -20}, {35, 20}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Joints.SphericalSpherical rod1(animation = animation, showMass = showMass, m = rod1Mass, rodLength = rod1Length, rodDiameter = rod1Diameter, sphereDiameter = sphereDiameter, rodColor = rod1Color, specularCoefficient = specularCoefficient, kinematicConstraint = false, sphereColor = sphereColor, constraintResidue = rod1.f_rod - f_rod) annotation(
              Placement(transformation(extent = {{-89, -20}, {-49, 20}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Parts.FixedTranslation rod2(animation = animation, width = rod2Diameter, height = rod2Diameter, color = rod2Color, specularCoefficient = specularCoefficient, r = rRod2_ib) annotation(
              Placement(transformation(extent = {{15, -20}, {-25, 20}}, rotation = 0)));
            Sensors.RelativePosition relativePosition(resolveInFrame = Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB.frame_a) annotation(
              Placement(transformation(extent = {{60, -70}, {40, -90}}, rotation = 0)));
            Modelica.Blocks.Sources.Constant position_b[3](k = rRod2_ib) annotation(
              Placement(transformation(extent = {{-20, -50}, {0, -30}}, rotation = 0)));
          equation
/* Compute the unknown force in the rod of the rod1 joint
     by a torque balance at the revolute joint:
       0 = frame_b.t + frame_ib.t + frame_im.t + cross(rRod2_ib, frame_im.f)
           + cross(rRod2_ib, -rod1.f_b_a1)
           + cross(rRod2_ib, Frames.resolve2(rod1.R_rel, rod1.f_rod*rod1.eRod_a))
     The condition is that the projection of the torque in the revolute
     joint along the axis of the revolute joint is equal to the driving
     axis torque in the flange:
       -revolute.tau = revolute.e*frame_b.t
     Therefore, we have with e=revolute.e and tau=revolute.tau
        tau = e*(frame_ib.t  + frame_im.t + cross(rRod2_ib, frame_im.f)
              + cross(rRod2_ib, -rod1.f_b_a1))
              + e*cross(rRod2_ib, Frames.resolve2(rod1.R_rel, rod1.f_rod*rod1.eRod_a))
            = e*(frame_ib.t + frame_im.t + cross(rRod2_ib, frame_im.f)
              + cross(rRod2_ib, -rod.f_b_a1))
              + rod1.f_rod*e*cross(rRod2_ib, Frames.resolve2(rod1.R_rel, rod1.eRod_a))
     Solving this equation for f_rod results in
       rod1.f_rod = (tau - e*(frame_ib.t + frame_im.t + cross(rRod2_ib, frame_im.f)
                   + cross(rRod2_ib, -rod1.f_b_a1)))
                   / (cross(e,rRod2_ib)*Frames.resolve2(rod1.R_rel, rod1.eRod_a))
     Additionally, a guard against division by zero is introduced
  */
            aux = cross(revolute.e, rRod2_ib) * Frames.resolveRelative(rod1.eRod_a, rod1.frame_a.R, rod1.frame_b.R);
            f_rod = ((-revolute.tau) - revolute.e * (frame_ib.t + frame_im.t + cross(rRod2_ib, frame_im.f) - cross(rRod2_ib, Frames.resolveRelative(rod1.f_b_a1, rod1.frame_a.R, rod1.frame_b.R)))) / noEvent(if abs(aux) < 1.e-10 then 1.e-10 else aux);
// Measure power for test purposes
            if checkTotalPower then
              totalPower = frame_a.f * Frames.resolve2(frame_a.R, der(frame_a.r_0)) + frame_b.f * Frames.resolve2(frame_b.R, der(frame_b.r_0)) + frame_ib.f * Frames.resolve2(frame_ib.R, der(frame_ib.r_0)) + frame_im.f * Frames.resolve2(frame_im.R, der(frame_im.r_0)) + frame_a.t * Frames.angularVelocity2(frame_a.R) + frame_b.t * Frames.angularVelocity2(frame_b.R) + frame_ib.t * Frames.angularVelocity2(frame_ib.R) + frame_im.t * Frames.angularVelocity2(frame_im.R) + axis.tau * der(axis.phi) + bearing.tau * der(bearing.phi) + (-rod1Mass) * (der(rod1.v_CM_0) - world.gravityAcceleration(rod1.r_CM_0)) * rod1.v_CM_0;
            else
              totalPower = 0;
            end if;
            connect(revolute.frame_b, rod2.frame_a) annotation(
              Line(points = {{35, 0}, {15, 0}}, color = {95, 95, 95}, thickness = 0.5));
            connect(rod2.frame_b, rod1.frame_b) annotation(
              Line(points = {{-25, 0}, {-49, 0}}, color = {95, 95, 95}, thickness = 0.5));
            connect(revolute.frame_a, frame_b) annotation(
              Line(points = {{75, 0}, {100, 0}}, color = {95, 95, 95}, thickness = 0.5));
            connect(rod2.frame_a, frame_ib) annotation(
              Line(points = {{15, 0}, {26, 0}, {26, 70}, {80, 70}, {80, 100}}, color = {95, 95, 95}, thickness = 0.5));
            connect(rod1.frame_a, frame_a) annotation(
              Line(points = {{-89, 0}, {-100, 0}}, color = {95, 95, 95}, thickness = 0.5));
            connect(relativePosition.frame_b, frame_a) annotation(
              Line(points = {{40, -80}, {-95, -80}, {-95, 0}, {-100, 0}}, color = {95, 95, 95}, pattern = LinePattern.Dot));
            connect(relativePosition.frame_a, frame_b) annotation(
              Line(points = {{60, -80}, {96, -80}, {96, 0}, {100, 0}}, color = {95, 95, 95}, pattern = LinePattern.Dot));
            connect(position_b.y, revolute.position_b) annotation(
              Line(points = {{1, -40}, {20, -40}, {20, -12}, {31, -12}}, color = {0, 0, 127}));
            connect(revolute.axis, axis) annotation(
              Line(points = {{55, 20}, {55, 60}, {90, 60}, {90, 80}, {100, 80}}, color = {0, 0, 0}));
            connect(rod2.frame_b, frame_im) annotation(
              Line(points = {{-25, 0}, {-35, 0}, {-35, 60}, {0, 60}, {0, 100}}, color = {95, 95, 95}, thickness = 0.5));
            connect(relativePosition.r_rel, revolute.position_a) annotation(
              Line(points = {{50, -69}, {50, -50}, {90, -50}, {90, -12}, {79, -12}}, color = {0, 0, 127}, smooth = Smooth.None));
            connect(revolute.bearing, bearing) annotation(
              Line(points = {{67, 20}, {67, 40}, {100, 40}}, color = {0, 0, 0}, smooth = Smooth.None));
            annotation(
              Documentation(info = "<html>
<p>
This component consists of a <b>spherical</b> joint 1 at frame_a, a <b>revolute</b>
joint at frame_b and a <b>spherical</b> joint 2 which is connected via rod 1
to the spherical joint 1 and via rod 2 to the revolute joint, see the default
animation in the following figure (the axes vectors are not part of the
default animation):
</p>

<p>
<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Joints/JointSSR.png\" ALT=\"model Joints.Assemblies.JointSSR\">
</p>

<p>
Besides an optional point mass in the middle of rod 1,
this joint aggregation has no mass and no inertia,
and introduces neither constraints nor potential state variables.
It should be used in kinematic loops whenever possible since
the non-linear system of equations introduced by this joint aggregation
is solved <b>analytically</b> (i.e., a solution is always computed, if a
unique solution exists).
</p>
<p>
An additional <b>frame_ib</b> is present. It is <b>fixed</b> in rod 2
connecting the revolute and the spherical joint at the side of the revolute
joint that is connected to this rod (= rod2.frame_a = revolute.frame_a).
</p>
<p>
An additional <b>frame_im</b> is present. It is <b>fixed</b> in rod 2
connecting the revolute and the spherical joint at the side of spherical
joint 2 that is connected to this rod (= rod2.frame_b).
It is always parallel to <b>frame_ib</b>.
</p>
<p>
The easiest way to define the parameters of this joint is by moving the
MultiBody system in a <b>reference configuration</b> where <b>all frames</b>
of all components are <b>parallel</b> to each other (alternatively,
at least frame_b and frame_ib of the JointSSR joint
should be parallel to each other when defining an instance of this
component).
</p>
</html>"),
              Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}, initialScale = 0.2), graphics = {Text(extent = {{-141, -41}, {139, -66}}, lineColor = {0, 0, 255}, textString = "%name"), Ellipse(extent = {{-100, -30}, {-40, 30}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.Sphere, fillColor = {192, 192, 192}), Ellipse(extent = {{-93, -22}, {-48, 23}}, lineColor = {160, 160, 164}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-63, 33}, {-39, -33}}, lineColor = {255, 255, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-40, -30}, {20, 30}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.Sphere, fillColor = {192, 192, 192}), Ellipse(extent = {{-33, -22}, {12, 23}}, lineColor = {192, 192, 192}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-44, 31}, {-19, -30}}, lineColor = {255, 255, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-23, 10}, {-3, -10}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Rectangle(extent = {{19, 6}, {61, -6}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192, 192, 192}), Rectangle(extent = {{60, -30}, {76, 30}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192, 192, 192}), Rectangle(extent = {{85, -30}, {100, 30}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192, 192, 192}), Rectangle(extent = {{76, 10}, {85, -10}}, lineColor = {0, 0, 0}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Rectangle(extent = {{60, 30}, {76, -30}}, lineColor = {0, 0, 0}), Rectangle(extent = {{85, 30}, {100, -30}}, lineColor = {0, 0, 0}), Text(extent = {{88, 112}, {127, 92}}, lineColor = {128, 128, 128}, textString = "ib"), Ellipse(extent = {{-80, 11}, {-60, -9}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-62, 6}, {-21, -5}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192, 192, 192}), Line(points = {{80, 80}, {100, 80}}, color = {95, 95, 95}, thickness = 0.5), Line(points = {{19, 6}, {19, 80}, {0, 80}, {0, 100}}, color = {95, 95, 95}, thickness = 0.5), Text(extent = {{-47, 111}, {-8, 92}}, lineColor = {128, 128, 128}, textString = "im"), Line(points = {{68, 30}, {68, 80}, {80, 80}, {80, 98}}, color = {95, 95, 95}, thickness = 0.5), Line(points = {{90, 30}, {90, 40}, {95, 40}}, color = {95, 95, 95}, thickness = 0.5)}));
          end JointSSR;

          model JointSSP "Spherical - spherical - prismatic joint aggregation with mass (no constraints, no potential states)"
            import Modelica.Mechanics.MultiBody.Types;
            extends Interfaces.PartialTwoFramesDoubleSize;
            Modelica.Mechanics.MultiBody.Interfaces.Frame_b frame_ib "Coordinate system at origin of frame_b fixed at connecting rod of spherical and prismatic joint" annotation(
              Placement(transformation(origin = {80, 100}, extent = {{-8, 8}, {8, -8}}, rotation = 270)));
            Modelica.Mechanics.MultiBody.Interfaces.Frame_b frame_im "Coordinate system at origin of spherical joint in the middle fixed at connecting rod of spherical and prismatic joint" annotation(
              Placement(transformation(origin = {0, 100}, extent = {{8, -8}, {-8, 8}}, rotation = 270)));
            Modelica.Mechanics.Translational.Interfaces.Flange_a axis "1-dim. translational flange that drives the prismatic joint" annotation(
              Placement(transformation(extent = {{95, 75}, {105, 85}}, rotation = 0)));
            Modelica.Mechanics.Translational.Interfaces.Flange_b bearing "1-dim. translational flange of the drive bearing of the prismatic joint" annotation(
              Placement(transformation(extent = {{105, 35}, {95, 45}}, rotation = 0)));
            parameter Boolean animation = true "= true, if animation shall be enabled";
            parameter Boolean showMass = true "= true, if point mass on rod 1 shall be shown (provided animation = true and rod1Mass > 0)";
            parameter SI.Length rod1Length(min = Modelica.Constants.eps, start = 1) "Distance between the origins of the two spherical joints";
            parameter SI.Mass rod1Mass(min = 0) = 0 "Mass of rod 1 (= point mass located in middle of rod connecting the two spherical joints)";
            parameter Modelica.Mechanics.MultiBody.Types.Axis n_b = {0, 0, 1} "Axis of prismatic joint fixed and resolved in frame_b";
            parameter SI.Position rRod2_ib[3] = {1, 0, 0} "Vector from origin of frame_ib to spherical joint in the middle, resolved in frame_ib";
            parameter SI.Position s_offset = 0 "Relative distance offset of prismatic joint (distance between frame_b and frame_ib = s(t) + s_offset)";
            parameter SI.Position s_guess = 0 "Select the configuration such that at initial time |s(t0)-s_guess| is minimal";
            parameter SI.Diameter sphereDiameter = world.defaultJointLength "Diameter of the spheres representing the two spherical joints" annotation(
              Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            input Types.Color sphereColor = Modelica.Mechanics.MultiBody.Types.Defaults.JointColor "Color of the spheres representing the two spherical joints" annotation(
              Dialog(colorSelector = true, tab = "Animation", group = "if animation = true", enable = animation));
            parameter SI.Diameter rod1Diameter = sphereDiameter / Types.Defaults.JointRodDiameterFraction "Diameter of rod 1 connecting the two spherical joints" annotation(
              Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            input Types.Color rod1Color = Modelica.Mechanics.MultiBody.Types.Defaults.RodColor "Color of rod 1 connecting the two spherical joint" annotation(
              Dialog(colorSelector = true, tab = "Animation", group = "if animation = true", enable = animation));
            parameter SI.Diameter rod2Diameter = rod1Diameter "Diameter of rod 2 connecting the revolute joint and spherical joint 2" annotation(
              Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            input Types.Color rod2Color = rod1Color "Color of rod 2 connecting the revolute joint and spherical joint 2" annotation(
              Dialog(colorSelector = true, tab = "Animation", group = "if animation = true", enable = animation));
            parameter Types.Axis boxWidthDirection = {0, 1, 0} "Vector in width direction of prismatic joint box, resolved in frame_b" annotation(
              Evaluate = true,
              Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            parameter SI.Distance boxWidth = world.defaultJointWidth "Width of prismatic joint box" annotation(
              Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            parameter SI.Distance boxHeight = boxWidth "Height of prismatic joint box" annotation(
              Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            input Types.Color boxColor = Modelica.Mechanics.MultiBody.Types.Defaults.JointColor "Color of prismatic joint box" annotation(
              Dialog(colorSelector = true, tab = "Animation", group = "if animation = true", enable = animation));
            input Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(
              Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            parameter Boolean checkTotalPower = false "= true, if total power flowing into this component shall be determined (must be zero)" annotation(
              Dialog(tab = "Advanced"));
            Real aux "Denominator used to compute force in rod connecting universal and spherical joint";
            SI.Force f_rod "Constraint force in direction of the rod (positive, if rod is pressed)";
            SI.Power totalPower "Total power flowing into this element, if checkTotalPower=true (otherwise dummy)";
            Modelica.Mechanics.MultiBody.Joints.Internal.PrismaticWithLengthConstraint prismatic(animation = animation, length = rod1Length, n = n_b, s_offset = s_offset, s_guess = s_guess, boxWidthDirection = boxWidthDirection, boxWidth = boxWidth, boxHeight = boxHeight, specularCoefficient = specularCoefficient, boxColor = boxColor) annotation(
              Placement(transformation(extent = {{75, -20}, {35, 20}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Joints.SphericalSpherical rod1(animation = animation, showMass = showMass, m = rod1Mass, rodLength = rod1Length, rodDiameter = rod1Diameter, sphereDiameter = sphereDiameter, rodColor = rod1Color, kinematicConstraint = false, specularCoefficient = specularCoefficient, sphereColor = sphereColor, constraintResidue = rod1.f_rod - f_rod) annotation(
              Placement(transformation(extent = {{-89, -20}, {-49, 20}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Parts.FixedTranslation rod2(animation = animation, width = rod2Diameter, height = rod2Diameter, specularCoefficient = specularCoefficient, color = rod2Color, r = rRod2_ib) annotation(
              Placement(transformation(extent = {{15, -20}, {-25, 20}}, rotation = 0)));
            Sensors.RelativePosition relativePosition(resolveInFrame = Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB.frame_a) annotation(
              Placement(transformation(extent = {{60, -70}, {40, -90}}, rotation = 0)));
            Modelica.Blocks.Sources.Constant position_b[3](k = rRod2_ib) annotation(
              Placement(transformation(extent = {{-20, -50}, {0, -30}}, rotation = 0)));
          equation
/* Compute the unknown force in the rod of the rod1 joint
     by a force balance:
       0 = frame_b.f + frame_ib.f + frame_im.f +
           Frames.resolve2(rod1.R_rel, rod1.f_rod*rod1.eRod_a)
     The condition is that the projection of the force in the prismatic
     joint along the axis of the prismatic joint is equal to the driving
     axis force in the flange:
       -prismatic.f = prismatic.e*frame_b.f
     Therefore, we have with e=prismatic.e and f=prismatic.f
        f = e*(frame_ib.f + frame_im.f +
               Frames.resolve2(rod1.R_rel, rod1.f_rod*rod1.eRod_a))
          = e*(frame_ib.f + frame_im.f +
               rod1.f_rod*Frames.resolve2(rod1.R_rel, rod1.eRod_a))
     Solving this equation for f_rod results in
       rod1.f_rod = (f - e*(frame_ib.f + frame_im.f))
                    / (e*Frames.resolve2(rod1.R_rel, rod1.eRod_a))
     Additionally, a guard against division by zero is introduced
  */
            aux = prismatic.e * Frames.resolveRelative(rod1.eRod_a, rod1.frame_a.R, rod1.frame_b.R);
            f_rod = ((-prismatic.f) - prismatic.e * (frame_ib.f + frame_im.f)) / noEvent(if abs(aux) < 1.e-10 then 1.e-10 else aux);
// Measure power for test purposes
            if checkTotalPower then
              totalPower = frame_a.f * Frames.resolve2(frame_a.R, der(frame_a.r_0)) + frame_b.f * Frames.resolve2(frame_b.R, der(frame_b.r_0)) + frame_ib.f * Frames.resolve2(frame_ib.R, der(frame_ib.r_0)) + frame_im.f * Frames.resolve2(frame_im.R, der(frame_im.r_0)) + frame_a.t * Frames.angularVelocity2(frame_a.R) + frame_b.t * Frames.angularVelocity2(frame_b.R) + frame_ib.t * Frames.angularVelocity2(frame_ib.R) + frame_im.t * Frames.angularVelocity2(frame_im.R) + axis.f * der(axis.s) + bearing.f * der(bearing.s) + (-rod1Mass) * (der(rod1.v_CM_0) - world.gravityAcceleration(rod1.r_CM_0)) * rod1.v_CM_0;
            else
              totalPower = 0;
            end if;
            connect(prismatic.frame_b, rod2.frame_a) annotation(
              Line(points = {{35, 0}, {15, 0}}, color = {95, 95, 95}, thickness = 0.5));
            connect(rod2.frame_b, rod1.frame_b) annotation(
              Line(points = {{-25, 0}, {-49, 0}}, color = {95, 95, 95}, thickness = 0.5));
            connect(prismatic.frame_a, frame_b) annotation(
              Line(points = {{75, 0}, {100, 0}}, color = {95, 95, 95}, thickness = 0.5));
            connect(rod2.frame_a, frame_ib) annotation(
              Line(points = {{15, 0}, {26, 0}, {26, 70}, {80, 70}, {80, 100}}, color = {95, 95, 95}, thickness = 0.5));
            connect(rod1.frame_a, frame_a) annotation(
              Line(points = {{-89, 0}, {-100, 0}}, color = {95, 95, 95}, thickness = 0.5));
            connect(relativePosition.frame_b, frame_a) annotation(
              Line(points = {{40, -80}, {-95, -80}, {-95, 0}, {-100, 0}}, color = {95, 95, 95}, pattern = LinePattern.Dot));
            connect(relativePosition.frame_a, frame_b) annotation(
              Line(points = {{60, -80}, {96, -80}, {96, 0}, {100, 0}}, color = {95, 95, 95}, pattern = LinePattern.Dot));
            connect(position_b.y, prismatic.position_b) annotation(
              Line(points = {{1, -40}, {20, -40}, {20, -12}, {31, -12}}, color = {0, 0, 127}));
            connect(prismatic.axis, axis) annotation(
              Line(points = {{39, 14}, {40, 14}, {40, 60}, {90, 60}, {90, 80}, {100, 80}}, color = {0, 0, 0}));
            connect(prismatic.bearing, bearing) annotation(
              Line(points = {{63, 14}, {63, 40}, {100, 40}}, color = {0, 0, 0}));
            connect(rod2.frame_b, frame_im) annotation(
              Line(points = {{-25, 0}, {-35, 0}, {-35, 60}, {0, 60}, {0, 100}}, color = {95, 95, 95}, thickness = 0.5));
            connect(relativePosition.r_rel, prismatic.position_a) annotation(
              Line(points = {{50, -69}, {50, -50}, {90, -50}, {90, -12}, {79, -12}}, color = {0, 0, 127}, smooth = Smooth.None));
            annotation(
              Documentation(info = "<html>
<p>
This component consists of a <b>spherical</b> joint 1 at frame_a, a <b>prismatic</b>
joint at frame_b and a <b>spherical</b> joint 2 which is connected via rod 1
to the spherical joint 1 and via rod 2 to the prismatic joint, see the default
animation in the following figure (the axes vectors are not part of the
default animation):
</p>

<p>
<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Joints/JointSSP.png\" ALT=\"model Joints.Assemblies.JointSSP\">
</p>

<p>
Besides an optional point mass in the middle of rod 1,
this joint aggregation has no mass and no inertia,
and introduces neither constraints nor potential state variables.
It should be used in kinematic loops whenever possible since
the non-linear system of equations introduced by this joint aggregation
is solved <b>analytically</b> (i.e., a solution is always computed, if a
unique solution exists).
</p>
<p>
An additional <b>frame_ib</b> is present. It is <b>fixed</b> in rod 2
connecting the prismatic and the spherical joint at the side of the prismatic
joint that is connected to this rod (= rod2.frame_a = prismatic.frame_a).
</p>
<p>
An additional <b>frame_im</b> is present. It is <b>fixed</b> in rod 2
connecting the prismatic and the spherical joint at the side of spherical
joint 2 that is connected to this rod (= rod2.frame_b).
It is always parallel to <b>frame_ib</b>.
</p>
<p>
The easiest way to define the parameters of this joint is by moving the
MultiBody system in a <b>reference configuration</b> where <b>all frames</b>
of all components are <b>parallel</b> to each other (alternatively,
at least frame_b and frame_ib of the JointSSP joint
should be parallel to each other when defining an instance of this
component).
</p>
</html>"),
              Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}, initialScale = 0.2), graphics = {Text(extent = {{-140, -40}, {140, -65}}, lineColor = {0, 0, 255}, textString = "%name"), Ellipse(extent = {{-100, -30}, {-40, 30}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.Sphere, fillColor = {192, 192, 192}), Ellipse(extent = {{-93, -22}, {-48, 23}}, lineColor = {160, 160, 164}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-63, 33}, {-39, -33}}, lineColor = {255, 255, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-40, -30}, {20, 30}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.Sphere, fillColor = {192, 192, 192}), Ellipse(extent = {{-33, -22}, {12, 23}}, lineColor = {192, 192, 192}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-44, 31}, {-19, -30}}, lineColor = {255, 255, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-23, 10}, {-3, -10}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Rectangle(extent = {{19, 6}, {61, -6}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192, 192, 192}), Text(extent = {{89, 115}, {132, 92}}, lineColor = {128, 128, 128}, textString = "ib"), Ellipse(extent = {{-80, 11}, {-60, -9}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-62, 6}, {-21, -5}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192, 192, 192}), Line(points = {{19, 6}, {19, 80}, {0, 80}, {0, 100}}, color = {95, 95, 95}, thickness = 0.5), Text(extent = {{-49, 114}, {-11, 92}}, lineColor = {128, 128, 128}, textString = "im"), Rectangle(extent = {{50, 20}, {80, -20}}, lineColor = {0, 0, 0}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Rectangle(extent = {{80, 30}, {100, -30}}, lineColor = {0, 0, 0}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Rectangle(extent = {{50, 14}, {80, 20}}, pattern = LinePattern.None, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Rectangle(extent = {{80, 24}, {100, 30}}, pattern = LinePattern.None, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Line(points = {{50, 6}, {50, 80}, {80, 80}, {80, 100}}, color = {95, 95, 95}, thickness = 0.5), Line(points = {{101, 80}, {80, 80}}, color = {95, 95, 95}, thickness = 0.5), Line(points = {{99, 40}, {90, 40}, {90, 30}}, color = {95, 95, 95}, thickness = 0.5)}));
          end JointSSP;

          model JointRRR "Planar revolute - revolute - revolute joint aggregation (no constraints, no potential states)"
            import Modelica.Mechanics.MultiBody.Types;
            import Modelica.SIunits.Conversions.to_unit1;
            extends Interfaces.PartialTwoFramesDoubleSize;
            Modelica.Mechanics.MultiBody.Interfaces.Frame_a frame_ia "Coordinate system at origin of frame_a fixed at connecting rod of left and middle revolute joint" annotation(
              Placement(transformation(origin = {-80, 100}, extent = {{-8, -8}, {8, 8}}, rotation = 90)));
            Modelica.Mechanics.MultiBody.Interfaces.Frame_b frame_ib "Coordinate system at origin of frame_b fixed at connecting rod of middle and right revolute joint" annotation(
              Placement(transformation(origin = {80, 100}, extent = {{-8, 8}, {8, -8}}, rotation = 270)));
            Modelica.Mechanics.MultiBody.Interfaces.Frame_b frame_im "Coordinate system at origin of revolute joint in the middle fixed at connecting rod of middle and right revolute joint" annotation(
              Placement(transformation(origin = {0, 100}, extent = {{8, -8}, {-8, 8}}, rotation = 270)));
            Modelica.Mechanics.Rotational.Interfaces.Flange_a axis "1-dim. rotational flange that drives the right revolute joint at frame_b" annotation(
              Placement(transformation(extent = {{105, 85}, {95, 75}}, rotation = 0)));
            Modelica.Mechanics.Rotational.Interfaces.Flange_b bearing "1-dim. rotational flange of the drive bearing of the right revolute joint at frame_b" annotation(
              Placement(transformation(extent = {{95, 45}, {105, 35}}, rotation = 0)));
            parameter Boolean animation = true "= true, if animation shall be enabled";
            parameter Modelica.Mechanics.MultiBody.Types.Axis n_a = {0, 0, 1} "Axes of revolute joints resolved in frame_a (all axes are parallel to each other)" annotation(
              Evaluate = true);
            final parameter Real n_b[3](each final unit = "1", each fixed = false, start = {0, 0, 1}) "Axis of revolute joint fixed and resolved in frame_b" annotation(
              Evaluate = true);
            parameter SI.Position rRod1_ia[3] = {1, 0, 0} "Vector from origin of frame_a to revolute joint in the middle, resolved in frame_ia" annotation(
              Evaluate = true);
            parameter SI.Position rRod2_ib[3] = {-1, 0, 0} "Vector from origin of frame_ib to revolute joint in the middle, resolved in frame_ib";
            parameter Cv.NonSIunits.Angle_deg phi_offset = 0 "Relative angle offset of revolute joint at frame_b (angle = phi(t) + from_deg(phi_offset))";
            parameter Cv.NonSIunits.Angle_deg phi_guess = 0 "Select the configuration such that at initial time |phi(t0) - from_deg(phi_guess)| is minimal";
            parameter SI.Distance cylinderLength = world.defaultJointLength "Length of cylinders representing the revolute joints" annotation(
              Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            parameter SI.Distance cylinderDiameter = world.defaultJointWidth "Diameter of cylinders representing the revolute joints" annotation(
              Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            input Types.Color cylinderColor = Modelica.Mechanics.MultiBody.Types.Defaults.JointColor "Color of cylinders representing the revolute joints" annotation(
              Dialog(colorSelector = true, tab = "Animation", group = "if animation = true", enable = animation));
            parameter SI.Diameter rodDiameter = 1.1 * cylinderDiameter "Diameter of the two rods connecting the revolute joints" annotation(
              Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            input Types.Color rodColor = Modelica.Mechanics.MultiBody.Types.Defaults.RodColor "Color of the two rods connecting the revolute joint" annotation(
              Dialog(colorSelector = true, tab = "Animation", group = "if animation = true", enable = animation));
            input Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(
              Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            parameter Boolean checkTotalPower = false "= true, if total power flowing into this component shall be determined (must be zero)" annotation(
              Dialog(tab = "Advanced"));
            final parameter Real e_a[3](each final unit = "1") = Modelica.Math.Vectors.normalizeWithAssert(n_a) "Unit vector along axes of rotations, resolved in frame_a";
            final parameter Real e_ia[3](each final unit = "1") = jointUSR.e2_ia "Unit vector along axes of rotations, resolved in frame_ia";
            final parameter Real e_b[3](each final unit = "1") = jointUSR.revolute.e "Unit vector along axes of rotations, resolved in frame_b, frame_ib and frame_im";
            SI.Power totalPower = jointUSR.totalPower "Total power flowing into this element, if checkTotalPower=true (otherwise dummy)";
            JointUSR jointUSR(animation = false, n1_a = n_a, n_b = n_b, phi_offset = phi_offset, rRod2_ib = rRod2_ib, showUniversalAxes = false, rRod1_ia = rRod1_ia, checkTotalPower = checkTotalPower, phi_guess = phi_guess) annotation(
              Placement(transformation(extent = {{-30, -20}, {10, 20}}, rotation = 0)));
          protected
            Visualizers.Advanced.Shape shape_rev1(shapeType = "cylinder", color = cylinderColor, specularCoefficient = specularCoefficient, length = cylinderLength, width = cylinderDiameter, height = cylinderDiameter, lengthDirection = e_a, widthDirection = {0, 1, 0}, r_shape = -e_a * (cylinderLength / 2), r = frame_a.r_0, R = frame_a.R) if world.enableAnimation and animation;
            Visualizers.Advanced.Shape shape_rev2(shapeType = "cylinder", color = cylinderColor, specularCoefficient = specularCoefficient, length = cylinderLength, width = cylinderDiameter, height = cylinderDiameter, lengthDirection = e_b, widthDirection = {0, 1, 0}, r_shape = -e_b * (cylinderLength / 2), r = frame_im.r_0, R = frame_im.R) if world.enableAnimation and animation;
            Visualizers.Advanced.Shape shape_rev3(shapeType = "cylinder", color = cylinderColor, specularCoefficient = specularCoefficient, length = cylinderLength, width = cylinderDiameter, height = cylinderDiameter, lengthDirection = e_b, widthDirection = {0, 1, 0}, r_shape = -e_b * (cylinderLength / 2), r = frame_b.r_0, R = frame_b.R) if world.enableAnimation and animation;
            Visualizers.Advanced.Shape shape_rod1(shapeType = "cylinder", color = rodColor, specularCoefficient = specularCoefficient, length = Modelica.Math.Vectors.length(rRod1_ia), width = rodDiameter, height = rodDiameter, lengthDirection = to_unit1(rRod1_ia), widthDirection = e_ia, r = frame_ia.r_0, R = frame_ia.R) if world.enableAnimation and animation;
            Visualizers.Advanced.Shape shape_rod2(shapeType = "cylinder", color = rodColor, specularCoefficient = specularCoefficient, length = Modelica.Math.Vectors.length(rRod2_ib), width = rodDiameter, height = rodDiameter, lengthDirection = to_unit1(rRod2_ib), widthDirection = e_b, r = frame_ib.r_0, R = frame_ib.R) if world.enableAnimation and animation;
          initial equation
            n_b = Frames.resolve2(frame_b.R, Frames.resolve1(frame_a.R, n_a));
          equation
            connect(jointUSR.frame_a, frame_a) annotation(
              Line(points = {{-30, 0}, {-100, 0}}, color = {95, 95, 95}, thickness = 0.5));
            connect(jointUSR.frame_b, frame_b) annotation(
              Line(points = {{10, 0}, {100, 0}}, color = {95, 95, 95}, thickness = 0.5));
            connect(jointUSR.frame_ia, frame_ia) annotation(
              Line(points = {{-26, 20}, {-26, 70}, {-80, 70}, {-80, 100}}, color = {95, 95, 95}, thickness = 0.5));
            connect(jointUSR.frame_im, frame_im) annotation(
              Line(points = {{-10, 20}, {-10, 70}, {0, 70}, {0, 100}}, color = {95, 95, 95}, thickness = 0.5));
            connect(jointUSR.frame_ib, frame_ib) annotation(
              Line(points = {{6, 20}, {6, 50}, {80, 50}, {80, 100}}, color = {95, 95, 95}, thickness = 0.5));
            connect(jointUSR.axis, axis) annotation(
              Line(points = {{10, 16}, {86, 16}, {86, 80}, {100, 80}}, color = {0, 0, 0}));
            connect(jointUSR.bearing, bearing) annotation(
              Line(points = {{10, 8}, {94, 8}, {94, 40}, {100, 40}}, color = {0, 0, 0}));
            annotation(
              Documentation(info = "<html>
<p>
This component consists of <b>3 revolute</b> joints with parallel
axes of rotation that are connected together by two rods, see the default
animation in the following figure (the axes vectors are not part of the
default animation):
</p>

<p>
<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Joints/JointRRR.png\" ALT=\"model Joints.Assemblies.JointRRR\">
</p>

<p>
This joint aggregation introduces neither constraints nor state variables and
should therefore be used in kinematic loops whenever possible to
avoid non-linear systems of equations. It is only meaningful to
use this component in <b>planar loops</b>. Basically, the position
and orientation of the 3 revolute joints as well as of frame_ia, frame_ib, and
frame_im are calculated by solving analytically a non-linear equation,
given the position and orientation at frame_a and at frame_b.
</p>
<p>
Connector <b>frame_a</b> is the \"left\" side of the first revolute joint
whereas <b>frame_ia</b> is the \"right side of this revolute joint, fixed in rod 1.
Connector <b>frame_b</b> is the \"right\" side of the third revolute joint
whereas <b>frame_ib</b> is the \"left\" side of this revolute joint, fixed in rod 2.
Finally, connector <b>frame_im</b> is the connector at the \"right\" side
of the revolute joint in the middle, fixed in rod 2.
</p>
<p>
The easiest way to define the parameters of this joint is by moving the
MultiBody system in a <b>reference configuration</b> where <b>all frames</b>
of all components are <b>parallel</b> to each other (alternatively,
at least frame_a, frame_ia, frame_im, frame_ib, frame_b of the JointRRR joint
should be parallel to each other when defining an instance of this
component).
</p>
<p>
Basically, the JointRRR model consists internally of a universal -
spherical - revolute joint aggregation (= JointUSR). In a planar
loop this will behave as if 3 revolute joints with parallel axes
are connected by rigid rods.
</p>
</html>"),
              Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}, initialScale = 0.2), graphics = {Rectangle(extent = {{-90, 90}, {90, -90}}, lineColor = {255, 255, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-140, -55}, {140, -80}}, lineColor = {0, 0, 255}, textString = "%name"), Text(extent = {{36, 114}, {71, 92}}, lineColor = {128, 128, 128}, textString = "ib"), Text(extent = {{-126, 115}, {-87, 90}}, lineColor = {128, 128, 128}, textString = "ia"), Ellipse(extent = {{-100, 25}, {-50, -25}}, lineColor = {0, 0, 0}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-85, 10}, {-65, -10}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Ellipse(extent = {{50, 25}, {100, -25}}, lineColor = {0, 0, 0}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Ellipse(extent = {{65, 10}, {85, -10}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-26, 80}, {24, 30}}, lineColor = {0, 0, 0}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-10, 66}, {10, 46}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Polygon(points = {{-71, 9}, {-24, 45}, {-19, 39}, {-66, 3}, {-71, 9}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Polygon(points = {{54, 12}, {5, 47}, {10, 52}, {59, 18}, {54, 12}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Polygon(points = {{100, -4}, {83, -4}, {84, 3}, {100, 3}, {100, -4}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Line(points = {{80, 24}, {80, 80}, {80, 80}, {80, 100}}, color = {95, 95, 95}, thickness = 0.5), Text(extent = {{-128, -29}, {136, -47}}, lineColor = {0, 0, 0}, textString = "n_a=%n_a"), Line(points = {{0, 57}, {0, 86}, {0, 86}, {0, 100}}, color = {95, 95, 95}, thickness = 0.5), Text(extent = {{-46, 114}, {-7, 91}}, lineColor = {128, 128, 128}, textString = "im"), Line(points = {{-80, 100}, {-80, 8}}, color = {95, 95, 95}, thickness = 0.5), Line(points = {{80, 80}, {101, 80}}, color = {95, 95, 95}, thickness = 0.5), Line(points = {{100, 40}, {93, 40}, {93, 3}}, color = {95, 95, 95}, thickness = 0.5)}));
          end JointRRR;

          model JointRRP "Planar revolute - revolute - prismatic joint aggregation (no constraints, no potential states)"
            import Modelica.Mechanics.MultiBody.Types;
            import Modelica.SIunits.Conversions.to_unit1;
            extends Interfaces.PartialTwoFramesDoubleSize;
            Modelica.Mechanics.MultiBody.Interfaces.Frame_a frame_ia "Coordinate system at origin of frame_a fixed at connecting rod of revolute joints" annotation(
              Placement(transformation(origin = {-80, 100}, extent = {{-8, -8}, {8, 8}}, rotation = 90)));
            Modelica.Mechanics.MultiBody.Interfaces.Frame_b frame_ib "Coordinate system at origin of frame_b fixed at connecting rod of revolute and prismatic joint" annotation(
              Placement(transformation(origin = {80, 100}, extent = {{-8, 8}, {8, -8}}, rotation = 270)));
            Modelica.Mechanics.MultiBody.Interfaces.Frame_b frame_im "Coordinate system at origin of revolute joint in the middle fixed at connecting rod of revolute and prismatic joint" annotation(
              Placement(transformation(origin = {0, 100}, extent = {{8, -8}, {-8, 8}}, rotation = 270)));
            Modelica.Mechanics.Translational.Interfaces.Flange_a axis "1-dim. translational flange that drives the prismatic joint" annotation(
              Placement(transformation(extent = {{95, 75}, {105, 85}}, rotation = 0)));
            Modelica.Mechanics.Translational.Interfaces.Flange_b bearing "1-dim. translational flange of the drive bearing of the prismatic joint" annotation(
              Placement(transformation(extent = {{105, 35}, {95, 45}}, rotation = 0)));
            parameter Boolean animation = true "= true, if animation shall be enabled";
            parameter Modelica.Mechanics.MultiBody.Types.Axis n_a = {0, 0, 1} "Axes of the two revolute joints resolved in frame_a (both axes are parallel to each other)" annotation(
              Evaluate = true);
            parameter Modelica.Mechanics.MultiBody.Types.Axis n_b = {-1, 0, 0} "Axis of prismatic joint fixed and resolved in frame_b (must be orthogonal to revolute joint axes)" annotation(
              Evaluate = true);
            parameter SI.Position rRod1_ia[3] = {1, 0, 0} "Vector from origin of frame_a to revolute joint in the middle, resolved in frame_ia" annotation(
              Evaluate = true);
            parameter SI.Position rRod2_ib[3] = {-1, 0, 0} "Vector from origin of frame_ib to revolute joint in the middle, resolved in frame_ib (frame_ib is parallel to frame_b)";
            parameter SI.Position s_offset = 0 "Relative distance offset of prismatic joint (distance between the prismatic joint frames = s(t) + s_offset)";
            parameter SI.Position s_guess = 0 "Select the configuration such that at initial time |s(t0)-s_guess| is minimal";
            parameter SI.Distance cylinderLength = world.defaultJointLength "Length of cylinders representing the revolute joints" annotation(
              Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            parameter SI.Distance cylinderDiameter = world.defaultJointWidth "Diameter of cylinders representing the revolute joints" annotation(
              Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            input Types.Color cylinderColor = Modelica.Mechanics.MultiBody.Types.Defaults.JointColor "Color of cylinders representing the revolute joints" annotation(
              Dialog(colorSelector = true, tab = "Animation", group = "if animation = true", enable = animation));
            parameter Types.Axis boxWidthDirection = {0, 1, 0} "Vector in width direction of prismatic joint, resolved in frame_b" annotation(
              Evaluate = true,
              Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            parameter SI.Distance boxWidth = world.defaultJointWidth "Width of prismatic joint box" annotation(
              Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            parameter SI.Distance boxHeight = boxWidth "Height of prismatic joint box" annotation(
              Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            input Types.Color boxColor = cylinderColor "Color of prismatic joint box" annotation(
              Dialog(colorSelector = true, tab = "Animation", group = "if animation = true", enable = animation));
            parameter SI.Diameter rodDiameter = 1.1 * cylinderDiameter "Diameter of the two rods connecting the joints" annotation(
              Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            input Types.Color rodColor = Modelica.Mechanics.MultiBody.Types.Defaults.RodColor "Color of the two rods connecting the joints" annotation(
              Dialog(colorSelector = true, tab = "Animation", group = "if animation = true", enable = animation));
            input Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(
              Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            parameter Boolean checkTotalPower = false "= true, if total power flowing into this component shall be determined (must be zero)" annotation(
              Dialog(tab = "Advanced"));
            final parameter Real e_a[3](each final unit = "1") = Modelica.Math.Vectors.normalizeWithAssert(n_a) "Unit vector along axes of rotations, resolved in frame_a";
            final parameter Real e_ia[3](each final unit = "1") = jointUSP.e2_ia "Unit vector along axes of rotations, resolved in frame_ia";
            final parameter Real e_im[3](each final unit = "1", each fixed = false) "Unit vector along axes of rotations, resolved in frame_im";
            final parameter Real e_b[3](each final unit = "1") = jointUSP.prismatic.e "Unit vector along axes of translation of the prismatic joint, resolved in frame_b and frame_ib";
            SI.Power totalPower = jointUSP.totalPower "Total power flowing into this element, if checkTotalPower=true (otherwise dummy)";
            JointUSP jointUSP(animation = false, showUniversalAxes = false, n1_a = n_a, n_b = n_b, s_offset = s_offset, s_guess = s_guess, rRod1_ia = rRod1_ia, rRod2_ib = rRod2_ib, checkTotalPower = checkTotalPower) annotation(
              Placement(transformation(extent = {{-30, -20}, {10, 20}}, rotation = 0)));
          protected
            Visualizers.Advanced.Shape shape_rev1(shapeType = "cylinder", color = cylinderColor, specularCoefficient = specularCoefficient, length = cylinderLength, width = cylinderDiameter, height = cylinderDiameter, lengthDirection = e_a, widthDirection = {0, 1, 0}, r_shape = -e_a * (cylinderLength / 2), r = frame_a.r_0, R = frame_a.R) if world.enableAnimation and animation;
            Visualizers.Advanced.Shape shape_rev2(shapeType = "cylinder", color = cylinderColor, specularCoefficient = specularCoefficient, length = cylinderLength, width = cylinderDiameter, height = cylinderDiameter, lengthDirection = e_im, widthDirection = {0, 1, 0}, r_shape = -e_im * (cylinderLength / 2), r = frame_im.r_0, R = frame_im.R) if world.enableAnimation and animation;
            Visualizers.Advanced.Shape shape_prism(shapeType = "box", color = boxColor, specularCoefficient = specularCoefficient, length = jointUSP.prismatic.distance, width = boxWidth, height = boxHeight, lengthDirection = e_b, widthDirection = e_im, r = frame_b.r_0, R = frame_b.R) if world.enableAnimation and animation;
            Visualizers.Advanced.Shape shape_rod1(shapeType = "cylinder", color = rodColor, specularCoefficient = specularCoefficient, length = Modelica.Math.Vectors.length(rRod1_ia), width = rodDiameter, height = rodDiameter, lengthDirection = to_unit1(rRod1_ia), widthDirection = e_ia, r = frame_ia.r_0, R = frame_ia.R) if world.enableAnimation and animation;
            Visualizers.Advanced.Shape shape_rod2(shapeType = "cylinder", color = rodColor, specularCoefficient = specularCoefficient, length = Modelica.Math.Vectors.length(rRod2_ib), width = rodDiameter, height = rodDiameter, lengthDirection = to_unit1(rRod2_ib), widthDirection = e_b, r = frame_ib.r_0, R = frame_ib.R) if world.enableAnimation and animation;
          initial equation
            e_im = Frames.resolve2(frame_im.R, Frames.resolve1(frame_a.R, e_a));
          equation
            connect(jointUSP.frame_a, frame_a) annotation(
              Line(points = {{-30, 0}, {-100, 0}}, color = {95, 95, 95}, thickness = 0.5));
            connect(jointUSP.frame_b, frame_b) annotation(
              Line(points = {{10, 0}, {100, 0}}, color = {95, 95, 95}, thickness = 0.5));
            connect(jointUSP.frame_ia, frame_ia) annotation(
              Line(points = {{-26, 20}, {-26, 70}, {-80, 70}, {-80, 100}}, color = {95, 95, 95}, thickness = 0.5));
            connect(jointUSP.frame_im, frame_im) annotation(
              Line(points = {{-10, 20}, {-10, 70}, {0, 70}, {0, 100}}, color = {95, 95, 95}, thickness = 0.5));
            connect(jointUSP.frame_ib, frame_ib) annotation(
              Line(points = {{6, 20}, {6, 50}, {80, 50}, {80, 100}}, color = {95, 95, 95}, thickness = 0.5));
            connect(jointUSP.axis, axis) annotation(
              Line(points = {{10, 16}, {86, 16}, {86, 80}, {100, 80}}, color = {0, 0, 0}));
            connect(jointUSP.bearing, bearing) annotation(
              Line(points = {{10, 8}, {94, 8}, {94, 40}, {100, 40}}, color = {0, 0, 0}));
            annotation(
              Documentation(info = "<html>
<p>
This component consists of <b>2 revolute</b> joints with parallel
axes of rotation that and a <b>prismatic</b> joint with a translational
axis that is orthogonal to the revolute joint axes, see the default
animation in the following figure (the axes vectors are not part of the
default animation):
</p>

<p>
<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Joints/JointRRP.png\" ALT=\"model Joints.Assemblies.JointRRP\">
</p>

<p>
This joint aggregation introduces neither constraints nor state variables and
should therefore be used in kinematic loops whenever possible to
avoid non-linear systems of equations. It is only meaningful to
use this component in <b>planar loops</b>. Basically, the position
and orientation of the 3 joints as well as of frame_ia, frame_ib, and
frame_im are calculated by solving analytically a non-linear equation,
given the position and orientation at frame_a and at frame_b.
</p>
<p>
Connector <b>frame_a</b> is the \"left\" side of the first revolute joint
whereas <b>frame_ia</b> is the \"right side of this revolute joint, fixed in rod 1.
Connector <b>frame_b</b> is the \"right\" side of the prismatic joint
whereas <b>frame_ib</b> is the \"left\" side of this prismatic joint, fixed in rod 2.
Finally, connector <b>frame_im</b> is the connector at the \"right\" side
of the revolute joint in the middle, fixed in rod 2. The frames
frame_b, frame_ib, frame_im are always parallel to each other.
</p>
<p>
The easiest way to define the parameters of this joint is by moving the
MultiBody system in a <b>reference configuration</b> where <b>all frames</b>
of all components are <b>parallel</b> to each other (alternatively,
at least frame_a, frame_ia, frame_im, frame_ib, frame_b of the JointRRP joint
should be parallel to each other when defining an instance of this
component).
</p>
<p>
Basically, the JointRRP model consists internally of a universal -
spherical - prismatic joint aggregation (= JointUSP). In a planar
loop this will behave as if 2 revolute joints with parallel axes
and 1 prismatic joint are connected by rigid rods.
</p>
</html>"),
              Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}, initialScale = 0.2), graphics = {Rectangle(extent = {{-90, 90}, {90, -90}}, lineColor = {255, 255, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-139, -53}, {141, -78}}, lineColor = {0, 0, 255}, textString = "%name"), Text(extent = {{26, 124}, {68, 93}}, lineColor = {128, 128, 128}, textString = "ib"), Text(extent = {{-134, 128}, {-94, 94}}, lineColor = {128, 128, 128}, textString = "ia"), Ellipse(extent = {{-100, 25}, {-50, -25}}, lineColor = {0, 0, 0}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-85, 10}, {-65, -10}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-26, 80}, {24, 30}}, lineColor = {0, 0, 0}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-10, 66}, {10, 46}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Polygon(points = {{-71, 9}, {-24, 45}, {-19, 39}, {-66, 3}, {-71, 9}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Polygon(points = {{54, 5}, {5, 47}, {8, 53}, {58, 11}, {54, 5}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Text(extent = {{-128, -29}, {139, -47}}, lineColor = {0, 0, 0}, textString = "n_a=%n_a"), Line(points = {{0, 57}, {0, 86}, {0, 86}, {0, 100}}, color = {95, 95, 95}, thickness = 0.5), Text(extent = {{-55, 126}, {-15, 92}}, lineColor = {128, 128, 128}, textString = "im"), Line(points = {{-80, 100}, {-80, 8}}, color = {95, 95, 95}, thickness = 0.5), Line(points = {{80, 80}, {101, 80}}, color = {95, 95, 95}, thickness = 0.5), Line(points = {{100, 40}, {93, 40}, {93, 3}}, color = {95, 95, 95}, thickness = 0.5), Rectangle(extent = {{80, 15}, {100, 21}}, pattern = LinePattern.None, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Rectangle(extent = {{53, 5}, {80, 11}}, pattern = LinePattern.None, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Rectangle(extent = {{53, 5}, {80, -15}}, lineColor = {0, 0, 0}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Rectangle(extent = {{80, 15}, {100, -21}}, lineColor = {0, 0, 0}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{80, 100}, {80, 80}, {57, 11}}, color = {95, 95, 95}, thickness = 0.5)}));
          end JointRRP;
          annotation(
            Documentation(info = "<HTML>
<p>
The joints in this package are mainly designed to be used
in <b>kinematic loop</b> structures. Every component consists of
<b>3 elementary joints</b>. These joints are combined in such a
way that the kinematics of the 3 joints between frame_a and
frame_b are computed from the movement of frame_a and frame_b,
i.e., there are <b>no constraints</b> between frame_a and frame_b.
This requires to solve a <b>non-linear system of equations</b> which
is performed <b>analytically</b> (i.e., when a mathematical
solution exists, it is computed efficiently and reliably).
A detailed description how to use these joints is provided in
<a href=\"modelica://Modelica.Mechanics.MultiBody.UsersGuide.Tutorial.LoopStructures.AnalyticLoopHandling\">MultiBody.UsersGuide.Tutorial.LoopStructures.AnalyticLoopHandling</a>.
</p>
<p>
The assembly joints in this package are named <b>JointXYZ</b> where
<b>XYZ</b> are the first letters of the elementary joints used in the
component, in particular:
</p>
<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"><b>P</b></td><td valign=\"top\">Prismatic joint</td></tr>
  <tr><td valign=\"top\"><b>R</b></td><td valign=\"top\">Revolute joint</td></tr>
  <tr><td valign=\"top\"><b>S</b></td><td valign=\"top\">Spherical joint</td></tr>
  <tr><td valign=\"top\"><b>U</b></td><td valign=\"top\">Universal joint</td></tr>
</table>
<p>
For example, JointUSR is an assembly joint consisting
of a universal, a spherical and a revolute joint.
</p>
<p> This package contains the following models:
</p>
<h4>Content</h4>
<table border=1 cellspacing=0 cellpadding=2>
  <tr><th><b><i>Model</i></b></th><th><b><i>Description</i></b></th></tr>
  <tr><td valign=\"top\"><a href=\"modelica://Modelica.Mechanics.MultiBody.Joints.Assemblies.JointUPS\">JointUPS</a></td>
      <td valign=\"top\"> Universal - prismatic - spherical joint aggregation<br>
     <img src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Joints/JointUPS.png\">
      </td>
  </tr>
  <tr><td valign=\"top\"><a href=\"modelica://Modelica.Mechanics.MultiBody.Joints.Assemblies.JointUSR\">JointUSR</a></td>
      <td valign=\"top\"> Universal - spherical - revolute joint aggregation<br>
     <img src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Joints/JointUSR.png\">
      </td>
  </tr>
  <tr><td valign=\"top\"><a href=\"modelica://Modelica.Mechanics.MultiBody.Joints.Assemblies.JointUSP\">JointUSP</a></td>
      <td valign=\"top\"> Universal - spherical - prismatic joint aggregation<br>
     <img src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Joints/JointUSP.png\">
      </td>
  </tr>
  <tr><td valign=\"top\"><a href=\"modelica://Modelica.Mechanics.MultiBody.Joints.Assemblies.JointSSR\">JointSSR</a></td>
      <td valign=\"top\"> Spherical - spherical - revolute joint aggregation
           with an optional mass point at the rod connecting
           the two spherical joints<br>
     <img src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Joints/JointSSR.png\">
      </td>
  </tr>
  <tr><td valign=\"top\"><a href=\"modelica://Modelica.Mechanics.MultiBody.Joints.Assemblies.JointSSP\">JointSSP</a></td>
      <td valign=\"top\"> Spherical - spherical - prismatic joint aggregation
           with an optional mass point at the rod connecting
           the two spherical joints<br>
     <img src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Joints/JointSSP.png\">
      </td>
  </tr>
  <tr><td valign=\"top\"><a href=\"modelica://Modelica.Mechanics.MultiBody.Joints.Assemblies.JointRRR\">JointRRR</a></td>
      <td valign=\"top\"> Revolute - revolute - revolute joint aggregation for planar loops<br>
     <img src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Joints/JointRRR.png\">
      </td>
  </tr>
  <tr><td valign=\"top\"><a href=\"modelica://Modelica.Mechanics.MultiBody.Joints.Assemblies.JointRRP\">JointRRP</a></td>
      <td valign=\"top\"> Revolute - revolute - prismatic joint aggregation for planar loops<br>
     <img src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Joints/JointRRP.png\">
      </td>
  </tr>
</table>
<p>
Note, no component of this package has potential states, since the
components are designed in such a way that the generalized coordinates
of the used elementary joints are computed from the frame_a and frame_b
coordinates. Still, it is possible to use the components in a
tree structure. In this case states are selected from bodies that are
connected to the frame_a or frame_b side of the component.
In most cases this gives a less efficient solution, as if elementary
joints of package Modelica.Mechanics.MultiBody.Joints would be used directly.
</p>
<p>
The analytic handling of kinematic loops by using joint aggregations
with 6 degrees of freedom as provided in this package, is a <b>new</b>
methodology. It is based on a more general method for solving
non-linear equations of kinematic loops developed by Woernle and
Hiller. An automatic application of this more general method
is difficult, and a manual application is only suited for
specialists in this field. The method introduced here is a
compromise: It can be quite easily applied by an end user, but
for a smaller class of kinematic loops. The method of the \"characteristic
pair of joints\" from Woernle and Hiller is described in:
</p>
<dl>
<dt>Woernle C.:</dt>
<dd><b>Ein systematisches Verfahren zur Aufstellung der geometrischen
    Schliessbedingungen in kinematischen Schleifen mit Anwendung
    bei der R&uuml;ckw&auml;rtstransformation f&uuml;r
    Industrieroboter.</b><br>
    Fortschritt-Berichte VDI, Reihe 18, Nr. 59, Duesseldorf: VDI-Verlag 1988,
    ISBN 3-18-145918-6.<br>&nbsp;</dd>
<dt>Hiller M., and Woernle C.:</dt>
<dd><b>A Systematic Approach for Solving the Inverse Kinematic
    Problem of Robot Manipulators</b>.<br>
    Proceedings 7th World Congress Th. Mach. Mech., Sevilla 1987. </dd>
</dl>
</HTML>"));
        end Assemblies;

        package Constraints "Components that define joints by constraints"
          extends Modelica.Icons.Package;

          model Prismatic "Prismatic cut-joint and translational directions may be constrained or released"
            extends Modelica.Mechanics.MultiBody.Interfaces.PartialTwoFrames;
            import Cv = Modelica.SIunits.Conversions;
            parameter Boolean x_locked = true "= true: constraint force in x-direction, resolved in frame_a" annotation(
              Dialog(group = "Constraints"),
              choices(checkBox = true));
            parameter Boolean y_locked = true "= true: constraint force in y-direction, resolved in frame_a" annotation(
              Dialog(group = "Constraints"),
              choices(checkBox = true));
            parameter Boolean z_locked = true "= true: constraint force in z-direction, resolved in frame_a" annotation(
              Dialog(group = "Constraints"),
              choices(checkBox = true));
            parameter Boolean animation = true "= true, if animation shall be enabled (show sphere)";
            parameter SI.Distance sphereDiameter = world.defaultJointLength / 3 "Diameter of sphere representing the spherical joint" annotation(
              Dialog(group = "if animation = true", enable = animation));
            input Types.Color sphereColor = Types.Defaults.JointColor "Color of sphere representing the spherical joint" annotation(
              Dialog(colorSelector = true, group = "if animation = true", enable = animation));
            input Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(
              Dialog(group = "if animation = true", enable = animation));
          protected
            Frames.Orientation R_rel "Dummy or relative orientation object from frame_a to frame_b";
            Modelica.SIunits.Position r_rel_a[3] "Position vector from origin of frame_a to origin of frame_b, resolved in frame_a";
            Modelica.SIunits.InstantaneousPower P;
          public
            Visualizers.Advanced.Shape sphere(shapeType = "sphere", color = sphereColor, specularCoefficient = specularCoefficient, length = sphereDiameter, width = sphereDiameter, height = sphereDiameter, lengthDirection = {1, 0, 0}, widthDirection = {0, 1, 0}, r_shape = {-0.5, 0, 0} * sphereDiameter, r = frame_a.r_0, R = frame_a.R) if world.enableAnimation and animation;
          equation
// Determine relative position vector resolved in frame_a
            R_rel = Frames.relativeRotation(frame_a.R, frame_b.R);
            r_rel_a = Frames.resolve2(frame_a.R, frame_b.r_0 - frame_a.r_0);
// Constraint equations concerning rotations
            ones(3) = {R_rel.T[1, 1], R_rel.T[2, 2], R_rel.T[3, 3]};
// Constraint equations concerning translations
            if x_locked and y_locked and z_locked then
              r_rel_a = zeros(3);
            elseif x_locked and y_locked and not z_locked then
              r_rel_a[1] = 0;
              r_rel_a[2] = 0;
              frame_a.f[3] = 0;
            elseif x_locked and not y_locked and z_locked then
              r_rel_a[1] = 0;
              r_rel_a[3] = 0;
              frame_a.f[2] = 0;
            elseif x_locked and not y_locked and not z_locked then
              r_rel_a[1] = 0;
              frame_a.f[2] = 0;
              frame_a.f[3] = 0;
            elseif not x_locked and y_locked and z_locked then
              r_rel_a[2] = 0;
              r_rel_a[3] = 0;
              frame_a.f[1] = 0;
            elseif not x_locked and y_locked and not z_locked then
              r_rel_a[2] = 0;
              frame_a.f[1] = 0;
              frame_a.f[3] = 0;
            elseif not x_locked and not y_locked and z_locked then
              r_rel_a[3] = 0;
              frame_a.f[1] = 0;
              frame_a.f[2] = 0;
            else
              frame_a.f = zeros(3);
            end if;
            zeros(3) = frame_a.t + Frames.resolve1(R_rel, frame_b.t) + cross(r_rel_a, Frames.resolve1(R_rel, frame_b.f));
            zeros(3) = Frames.resolve1(R_rel, frame_b.f) + frame_a.f;
            P = frame_a.t * Frames.angularVelocity2(frame_a.R) + frame_b.t * Frames.angularVelocity2(frame_b.R) + frame_b.f * Frames.resolve2(frame_b.R, der(frame_b.r_0)) + frame_a.f * Frames.resolve2(frame_a.R, der(frame_a.r_0));
            annotation(
              defaultComponentName = "constraint",
              Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-100, -70}, {100, -100}}, lineColor = {95, 95, 95}, textString = "lock: x", visible = x_locked and not y_locked and not z_locked), Text(extent = {{-100, -70}, {100, -100}}, lineColor = {95, 95, 95}, textString = "lock: y", visible = not x_locked and y_locked and not z_locked), Text(extent = {{-100, -70}, {100, -100}}, lineColor = {95, 95, 95}, textString = "lock: z", visible = not x_locked and not y_locked and z_locked), Text(extent = {{-100, -70}, {100, -100}}, lineColor = {95, 95, 95}, textString = "lock: x, y", visible = x_locked and y_locked and not z_locked), Text(extent = {{-100, -70}, {100, -100}}, lineColor = {95, 95, 95}, textString = "lock: x, z", visible = x_locked and not y_locked and z_locked), Text(extent = {{-100, -70}, {100, -100}}, lineColor = {95, 95, 95}, textString = "lock: y, z", visible = not x_locked and y_locked and z_locked), Text(extent = {{-100, -70}, {100, -100}}, lineColor = {95, 95, 95}, textString = "lock: x, y, z", visible = x_locked and y_locked and z_locked), Rectangle(extent = {{-100, 46}, {-30, 56}}, pattern = LinePattern.None, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Rectangle(extent = {{-100, -44}, {-30, 47}}, pattern = LinePattern.None, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Rectangle(extent = {{-30, 24}, {100, 34}}, pattern = LinePattern.None, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Rectangle(extent = {{-30, -26}, {100, 24}}, pattern = LinePattern.None, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Line(points = {{100, -26}, {100, 25}}, color = {0, 0, 0}), Line(points = {{-30, -44}, {-30, 56}}, color = {0, 0, 0}), Text(extent = {{-150, 120}, {150, 80}}, lineColor = {0, 0, 255}, textString = "%name"), Line(points = {{-81, -66}, {-23, 25}, {40, -39}, {97, 71}}, color = {255, 0, 0}, thickness = 0.5, smooth = Smooth.None)}),
              Documentation(info = "<html>
<p>This model does not use explicit variables e.g. state variables in order to describe the relative motion of frame_b with respect to frame_a, but defines kinematic constraints between the frame_a and frame_b. The forces and torques at both frames are then evaluated in such a way that the constraints are satisfied.  Sometimes this type of formulation is also called an implicit joint in literature.</p>
<p>As a consequence of the formulation the relative kinematics between frame_a and frame_b cannot be initialized.</p>
<p>In particular in complex multibody systems with closed loops this may help to simplify the system of non-linear equations. Please compare the translation log using the classical joint formulation and the alternative formulation used here in order to check whether this fact applies to the particular system under consideration.</p>
<p>In systems without closed loops the use of this implicit joint does not make sense or may even be disadvantageous.</p>
<p>See the subpackage <a href=\"Modelica://Modelica.Mechanics.MultiBody.Examples.Constraints\">Examples.Constraints</a> for testing the joint. </p>
</html>"));
          end Prismatic;

          model Revolute "Revolute cut-joint and translational directions may be constrained or released"
            extends Modelica.Mechanics.MultiBody.Interfaces.PartialTwoFrames;
            parameter Boolean x_locked = true "= true: constraint force in x-direction, resolved in frame_a" annotation(
              Dialog(group = "Constraints in translational motion"),
              choices(checkBox = true));
            parameter Boolean y_locked = true "= true: constraint force in y-direction, resolved in frame_a" annotation(
              Dialog(group = "Constraints in translational motion"),
              choices(checkBox = true));
            parameter Boolean z_locked = true "= true: constraint force in z-direction, resolved in frame_a" annotation(
              Dialog(group = "Constraints in translational motion"),
              choices(checkBox = true));
            parameter Boolean animation = true "= true, if animation shall be enabled (show sphere)";
            parameter Types.Axis n = {0, 1, 0} "Axis of rotation resolved in frame_a (= same as in frame_b)" annotation(
              Evaluate = true);
            parameter SI.Distance sphereDiameter = world.defaultJointLength / 3 "Diameter of sphere representing the spherical joint" annotation(
              Dialog(group = "if animation = true", enable = animation));
            input Types.Color sphereColor = Types.Defaults.JointColor "Color of sphere representing the spherical joint" annotation(
              Dialog(colorSelector = true, group = "if animation = true", enable = animation));
            input Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(
              Dialog(group = "if animation = true", enable = animation));
          protected
            Frames.Orientation R_rel "Dummy or relative orientation object from frame_a to frame_b";
            Modelica.SIunits.Position r_rel_a[3] "Position vector from origin of frame_a to origin of frame_b, resolved in frame_a";
            Modelica.SIunits.InstantaneousPower P;
            parameter Real e[3](each final unit = "1") = Modelica.Math.Vectors.normalizeWithAssert(n) "Unit vector in direction of rotation axis, resolved in frame_a (= same as in frame_b)";
            parameter Real nnx_a[3](each final unit = "1") = if abs(e[1]) > 0.1 then {0, 1, 0} else if abs(e[2]) > 0.1 then {0, 0, 1} else {1, 0, 0} "Arbitrary vector that is not aligned with rotation axis n" annotation(
              Evaluate = true);
            parameter Real ey_a[3](each final unit = "1") = Modelica.Math.Vectors.normalizeWithAssert(cross(e, nnx_a)) "Unit vector orthogonal to axis n of revolute joint, resolved in frame_a" annotation(
              Evaluate = true);
            parameter Real ex_a[3](each final unit = "1") = cross(ey_a, e) "Unit vector orthogonal to axis n of revolute joint and to ey_a, resolved in frame_a";
          public
            Visualizers.Advanced.Shape sphere(shapeType = "sphere", color = sphereColor, specularCoefficient = specularCoefficient, length = sphereDiameter, width = sphereDiameter, height = sphereDiameter, lengthDirection = {1, 0, 0}, widthDirection = {0, 1, 0}, r_shape = {-0.5, 0, 0} * sphereDiameter, r = frame_a.r_0, R = frame_a.R) if world.enableAnimation and animation;
          equation
// Determine relative position vector resolved in frame_a
            R_rel = Frames.relativeRotation(frame_a.R, frame_b.R);
            r_rel_a = Frames.resolve2(frame_a.R, frame_b.r_0 - frame_a.r_0);
// Constraint equations concerning translations
            if x_locked and y_locked and z_locked then
              r_rel_a = zeros(3);
            elseif x_locked and y_locked and not z_locked then
              r_rel_a[1] = 0;
              r_rel_a[2] = 0;
              frame_a.f[3] = 0;
            elseif x_locked and not y_locked and z_locked then
              r_rel_a[1] = 0;
              r_rel_a[3] = 0;
              frame_a.f[2] = 0;
            elseif x_locked and not y_locked and not z_locked then
              r_rel_a[1] = 0;
              frame_a.f[2] = 0;
              frame_a.f[3] = 0;
            elseif not x_locked and y_locked and z_locked then
              r_rel_a[2] = 0;
              r_rel_a[3] = 0;
              frame_a.f[1] = 0;
            elseif not x_locked and y_locked and not z_locked then
              r_rel_a[2] = 0;
              frame_a.f[1] = 0;
              frame_a.f[3] = 0;
            elseif not x_locked and not y_locked and z_locked then
              r_rel_a[3] = 0;
              frame_a.f[1] = 0;
              frame_a.f[2] = 0;
            else
              frame_a.f = zeros(3);
            end if;
// Constraint equations concerning rotations
            0 = ex_a * R_rel.T * e;
            0 = ey_a * R_rel.T * e;
            frame_a.t * n = 0;
            zeros(3) = frame_a.f + Frames.resolve1(R_rel, frame_b.f);
            zeros(3) = frame_a.t + Frames.resolve1(R_rel, frame_b.t) - cross(r_rel_a, frame_a.f);
            P = frame_a.t * Frames.angularVelocity2(frame_a.R) + frame_b.t * Frames.angularVelocity2(frame_b.R) + Frames.resolve1(frame_b.R, frame_b.f) * der(frame_b.r_0) + Frames.resolve1(frame_a.R, frame_a.f) * der(frame_a.r_0);
            annotation(
              defaultComponentName = "constraint",
              Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-63, -63}, {53, -93}}, lineColor = {0, 0, 0}, textString = "n=%n", fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-100, -60}, {-30, 60}}, lineColor = {64, 64, 64}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {255, 255, 255}, radius = 10), Rectangle(extent = {{30, -60}, {100, 60}}, lineColor = {64, 64, 64}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {255, 255, 255}, radius = 10), Rectangle(extent = {{-100, 60}, {-30, -60}}, lineColor = {64, 64, 64}, radius = 10), Rectangle(extent = {{30, 60}, {100, -60}}, lineColor = {64, 64, 64}, radius = 10), Text(extent = {{-90, 14}, {-54, -11}}, lineColor = {128, 128, 128}, textString = "a"), Text(extent = {{51, 11}, {87, -14}}, lineColor = {128, 128, 128}, textString = "b"), Rectangle(extent = {{-30, 11}, {30, -10}}, lineColor = {64, 64, 64}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-81, -66}, {-23, 25}, {40, -39}, {97, 71}}, color = {255, 0, 0}, thickness = 0.5, smooth = Smooth.None), Text(extent = {{-49, 82}, {45, 59}}, textString = "constraint"), Text(extent = {{-150, 120}, {150, 80}}, lineColor = {0, 0, 255}, textString = "%name")}),
              Documentation(info = "<html>
<p>This model does not use explicit variables e.g. state variables in order to describe the relative motion of frame_b with respect to frame_a, but defines kinematic constraints between the frame_a and frame_b. The forces and torques at both frames are then evaluated in such a way that the constraints are satisfied. Sometimes this type of formulation is also called an implicit joint in literature.</p>
<p>As a consequence of the formulation the relative kinematics between frame_a and frame_b cannot be initialized.</p>
<p>In particular in complex multibody systems with closed loops this may help to simplify the system of non-linear equations. Please compare the translation log using the classical joint formulation and the alternative formulation used here in order to check whether this fact applies to the particular system under consideration.</p>
<p>In systems without closed loops the use of this implicit joint does not make sense or may even be disadvantageous.</p>
<p>See the subpackage <a href=\"Modelica://Modelica.Mechanics.MultiBody.Examples.Constraints\">Examples.Constraints</a> for testing the joint. </p>
</html>"));
          end Revolute;

          model Spherical "Spherical cut joint and translational directions may be constrained or released"
            extends Modelica.Mechanics.MultiBody.Interfaces.PartialTwoFrames;
            import MBS = Modelica.Mechanics.MultiBody;
            parameter Boolean x_locked = true "= true: constraint force in x-direction, resolved in frame_a" annotation(
              Dialog(group = "Constraints"),
              choices(checkBox = true));
            parameter Boolean y_locked = true "= true: constraint force in y-direction, resolved in frame_a" annotation(
              Dialog(group = "Constraints"),
              choices(checkBox = true));
            parameter Boolean z_locked = true "= true: constraint force in z-direction, resolved in frame_a" annotation(
              Dialog(group = "Constraints"),
              choices(checkBox = true));
            parameter Boolean animation = true "= true, if animation shall be enabled (show sphere)" annotation(
              Dialog(group = "Animation"));
            parameter Modelica.SIunits.Distance sphereDiameter = world.defaultJointLength / 3 "Diameter of sphere representing the spherical joint" annotation(
              Dialog(group = "Animation", enable = animation));
            input MBS.Types.Color sphereColor = MBS.Types.Defaults.JointColor "Color of sphere representing the spherical joint" annotation(
              Dialog(colorSelector = true, group = "Animation", enable = animation));
            input MBS.Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(
              Dialog(group = "Animation", enable = animation));
            Modelica.Mechanics.MultiBody.Visualizers.Advanced.Shape sphere(shapeType = "sphere", color = sphereColor, specularCoefficient = specularCoefficient, length = sphereDiameter, width = sphereDiameter, height = sphereDiameter, lengthDirection = {1, 0, 0}, widthDirection = {0, 1, 0}, r_shape = {-0.5, 0, 0} * sphereDiameter, r = frame_a.r_0, R = frame_a.R) if world.enableAnimation and animation;
          protected
            MBS.Frames.Orientation R_rel "Dummy or relative orientation object from frame_a to frame_b";
            Modelica.SIunits.Position r_rel_a[3] "Position vector from origin of frame_a to origin of frame_b, resolved in frame_a";
            Modelica.SIunits.InstantaneousPower P;
          equation
// Determine relative position vector resolved in frame_a
            R_rel = MBS.Frames.relativeRotation(frame_a.R, frame_b.R);
            r_rel_a = MBS.Frames.resolve2(frame_a.R, frame_b.r_0 - frame_a.r_0);
// Constraint equations concerning translation
            if x_locked and y_locked and z_locked then
              r_rel_a = zeros(3);
            elseif x_locked and y_locked and not z_locked then
              r_rel_a[1] = 0;
              r_rel_a[2] = 0;
              frame_a.f[3] = 0;
            elseif x_locked and not y_locked and z_locked then
              r_rel_a[1] = 0;
              r_rel_a[3] = 0;
              frame_a.f[2] = 0;
            elseif x_locked and not y_locked and not z_locked then
              r_rel_a[1] = 0;
              frame_a.f[2] = 0;
              frame_a.f[3] = 0;
            elseif not x_locked and y_locked and z_locked then
              r_rel_a[2] = 0;
              r_rel_a[3] = 0;
              frame_a.f[1] = 0;
            elseif not x_locked and y_locked and not z_locked then
              r_rel_a[2] = 0;
              frame_a.f[1] = 0;
              frame_a.f[3] = 0;
            elseif not x_locked and not y_locked and z_locked then
              r_rel_a[3] = 0;
              frame_a.f[1] = 0;
              frame_a.f[2] = 0;
            else
              frame_a.f = zeros(3);
            end if;
//frame_a.t = zeros(3);
            frame_b.t = zeros(3);
            frame_b.f = -MBS.Frames.resolve2(R_rel, frame_a.f);
            zeros(3) = frame_a.t + MBS.Frames.resolve1(R_rel, frame_b.t) - cross(r_rel_a, frame_a.f);
            P = frame_a.t * MBS.Frames.angularVelocity2(frame_a.R) + frame_b.t * MBS.Frames.angularVelocity2(frame_b.R) + MBS.Frames.resolve1(frame_b.R, frame_b.f) * der(frame_b.r_0) + MBS.Frames.resolve1(frame_a.R, frame_a.f) * der(frame_a.r_0);
            annotation(
              defaultComponentName = "constraint",
              Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-150, 120}, {150, 80}}, lineColor = {0, 0, 255}, textString = "%name"), Text(extent = {{-100, -70}, {100, -100}}, lineColor = {95, 95, 95}, textString = "lock: x", visible = x_locked and not y_locked and not z_locked), Text(extent = {{-100, -70}, {100, -100}}, lineColor = {95, 95, 95}, textString = "lock: y", visible = not x_locked and y_locked and not z_locked), Text(extent = {{-100, -70}, {100, -100}}, lineColor = {95, 95, 95}, textString = "lock: z", visible = not x_locked and not y_locked and z_locked), Text(extent = {{-100, -70}, {100, -100}}, lineColor = {95, 95, 95}, textString = "lock: x, y", visible = x_locked and y_locked and not z_locked), Text(extent = {{-100, -70}, {100, -100}}, lineColor = {95, 95, 95}, textString = "lock: x, z", visible = x_locked and not y_locked and z_locked), Text(extent = {{-100, -70}, {100, -100}}, lineColor = {95, 95, 95}, textString = "lock: y, z", visible = not x_locked and y_locked and z_locked), Text(extent = {{-100, -70}, {100, -100}}, lineColor = {95, 95, 95}, textString = "lock: x, y, z", visible = x_locked and y_locked and z_locked), Ellipse(extent = {{-66, -70}, {74, 70}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.Sphere, fillColor = {192, 192, 192}), Ellipse(extent = {{-45, -50}, {55, 50}}, lineColor = {128, 128, 128}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Rectangle(extent = {{34, 70}, {75, -68}}, lineColor = {255, 255, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-96, 10}, {-64, -10}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192, 192, 192}), Rectangle(extent = {{27, 10}, {104, -10}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192, 192, 192}), Ellipse(extent = {{-20, 25}, {30, -25}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.Sphere, fillColor = {160, 160, 164}), Line(points = {{-81, -66}, {-23, 25}, {40, -39}, {97, 71}}, color = {255, 0, 0}, thickness = 0.5, smooth = Smooth.None)}),
              Documentation(info = "<html>
<p>This model does not use explicit variables e.g. state variables in order to describe the relative motion of frame_b with to respect to frame_a, but defines kinematic constraints between the frame_a and frame_b. The forces and torques at both frames are then evaluated in such a way that the constraints are satisfied. Sometimes this type of formulation is also called an implicit joint in literature.</p>
<p>As a consequence of the formulation the relative kinematics between frame_a and frame_b cannot be initialized.</p>
<p>In particular in complex multibody systems with closed loops this may help to simplify the system of non-linear equations. Please compare the translation log using the classical joint formulation and the alternative formulation used here in order to check whether this fact applies to the particular system under consideration.</p>
<p>In systems without closed loops the use of this implicit joint does not make sense or may even be disadvantageous.</p>
<p>See the subpackage <a href=\"Modelica://Modelica.Mechanics.MultiBody.Examples.Constraints\">Examples.Constraints</a> for testing the joint. </p>
</html>"));
          end Spherical;

          model Universal "Universal cut-joint and translational directions may be constrained or released"
            extends Modelica.Mechanics.MultiBody.Interfaces.PartialTwoFrames;
            import MBS = Modelica.Mechanics.MultiBody;
            parameter MBS.Types.Axis n_a = {1, 0, 0} "Axis of revolute joint 1 resolved in frame_a" annotation(
              Evaluate = true);
            parameter MBS.Types.Axis n_b = {0, 1, 0} "Axis of revolute joint 2 resolved in frame_b" annotation(
              Evaluate = true);
            parameter Boolean x_locked = true "= true: constraint force in x-direction, resolved in frame_a" annotation(
              Dialog(group = "Constraints in translational motion"),
              choices(checkBox = true));
            parameter Boolean y_locked = true "= true: constraint force in y-direction, resolved in frame_a" annotation(
              Dialog(group = "Constraints in translational motion"),
              choices(checkBox = true));
            parameter Boolean z_locked = true "= true: constraint force in z-direction, resolved in frame_a" annotation(
              Dialog(group = "Constraints in translational motion"),
              choices(checkBox = true));
            parameter Boolean animation = true "= true, if animation shall be enabled (show sphere)" annotation(
              Dialog(group = "Animation"));
            parameter SI.Distance sphereDiameter = world.defaultJointLength / 3 "Diameter of sphere representing the spherical joint" annotation(
              Dialog(group = "Animation", enable = animation));
            input MBS.Types.Color sphereColor = MBS.Types.Defaults.JointColor "Color of sphere representing the spherical joint" annotation(
              Dialog(colorSelector = true, group = "Animation", enable = animation));
            input MBS.Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(
              Dialog(group = "Animation", enable = animation));
          protected
            MBS.Frames.Orientation R_rel "Dummy or relative orientation object from frame_a to frame_b";
            Real w_rel[3];
            Modelica.SIunits.Position r_rel_a[3] "Position vector from origin of frame_a to origin of frame_b, resolved in frame_a";
            Modelica.SIunits.InstantaneousPower P;
          equation
// Determine relative position vector resolved in frame_a
            R_rel = MBS.Frames.relativeRotation(frame_a.R, frame_b.R);
            w_rel = MBS.Frames.angularVelocity1(R_rel);
            r_rel_a = MBS.Frames.resolve2(frame_a.R, frame_b.r_0 - frame_a.r_0);
// Constraint equations concerning translations
            if x_locked and y_locked and z_locked then
              r_rel_a = zeros(3);
            elseif x_locked and y_locked and not z_locked then
              r_rel_a[1] = 0;
              r_rel_a[2] = 0;
              frame_a.f[3] = 0;
            elseif x_locked and not y_locked and z_locked then
              r_rel_a[1] = 0;
              r_rel_a[3] = 0;
              frame_a.f[2] = 0;
            elseif x_locked and not y_locked and not z_locked then
              r_rel_a[1] = 0;
              frame_a.f[2] = 0;
              frame_a.f[3] = 0;
            elseif not x_locked and y_locked and z_locked then
              r_rel_a[2] = 0;
              r_rel_a[3] = 0;
              frame_a.f[1] = 0;
            elseif not x_locked and y_locked and not z_locked then
              r_rel_a[2] = 0;
              frame_a.f[1] = 0;
              frame_a.f[3] = 0;
            elseif not x_locked and not y_locked and z_locked then
              r_rel_a[3] = 0;
              frame_a.f[1] = 0;
              frame_a.f[2] = 0;
            else
              frame_a.f = zeros(3);
            end if;
// Constraint equations concerning rotations
            frame_a.t * n_a = 0;
            frame_b.t * n_b = 0;
            n_b * R_rel.T * n_a = 0;
            assert(abs(n_a * n_b) < Modelica.Constants.eps, "The two axes that constitute the Constraints.Universal joint must be different");
            zeros(3) = frame_a.f + MBS.Frames.resolve1(R_rel, frame_b.f);
            zeros(3) = frame_a.t + MBS.Frames.resolve1(R_rel, frame_b.t) - cross(r_rel_a, frame_a.f);
            P = frame_a.t * MBS.Frames.angularVelocity2(frame_a.R) + frame_b.t * MBS.Frames.angularVelocity2(frame_b.R) + MBS.Frames.resolve1(frame_b.R, frame_b.f) * der(frame_b.r_0) + MBS.Frames.resolve1(frame_a.R, frame_a.f) * der(frame_a.r_0);
            annotation(
              defaultComponentName = "constraint",
              Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-100, -70}, {100, -100}}, lineColor = {95, 95, 95}, textString = "lock: x", visible = x_locked and not y_locked and not z_locked), Text(extent = {{-100, -70}, {100, -100}}, lineColor = {95, 95, 95}, textString = "lock: y", visible = not x_locked and y_locked and not z_locked), Text(extent = {{-100, -70}, {100, -100}}, lineColor = {95, 95, 95}, textString = "lock: z", visible = not x_locked and not y_locked and z_locked), Text(extent = {{-100, -70}, {100, -100}}, lineColor = {95, 95, 95}, textString = "lock: x, y", visible = x_locked and y_locked and not z_locked), Text(extent = {{-100, -70}, {100, -100}}, lineColor = {95, 95, 95}, textString = "lock: x, z", visible = x_locked and not y_locked and z_locked), Text(extent = {{-100, -70}, {100, -100}}, lineColor = {95, 95, 95}, textString = "lock: y, z", visible = not x_locked and y_locked and z_locked), Text(extent = {{-100, -76}, {100, -106}}, lineColor = {95, 95, 95}, textString = "lock: x, y, z", visible = x_locked and y_locked and z_locked), Rectangle(extent = {{-96, 15}, {-61, -15}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {235, 235, 235}), Ellipse(extent = {{-76, -80}, {84, 80}}, lineColor = {160, 160, 164}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-56, -60}, {64, 60}}, lineColor = {160, 160, 164}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Rectangle(extent = {{16, 82}, {84, -82}}, lineColor = {255, 255, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Rectangle(extent = {{60, 15}, {104, -15}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {235, 235, 235}), Line(points = {{16, 78}, {16, -78}}, color = {0, 0, 0}, thickness = 0.5), Ellipse(extent = {{-48, -40}, {84, 40}}, lineColor = {160, 160, 164}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-28, -20}, {64, 26}}, lineColor = {160, 160, 164}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Polygon(points = {{-18, -54}, {-56, 0}, {-18, 50}, {44, 52}, {-18, -54}}, pattern = LinePattern.None, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Line(points = {{16, 78}, {16, -20}}, color = {0, 0, 0}, thickness = 0.5), Line(points = {{36, 38}, {-8, -36}}, color = {0, 0, 0}, thickness = 0.5), Text(extent = {{-150, 120}, {150, 80}}, lineColor = {0, 0, 255}, textString = "%name"), Line(points = {{-81, -66}, {-23, 25}, {40, -39}, {97, 71}}, color = {255, 0, 0}, thickness = 0.5, smooth = Smooth.None)}),
              Documentation(info = "<html>
<p>This model does not use explicit variables e.g. state variables in order to describe the relative motion of frame_b with respect to frame_a, but defines kinematic constraints between the frame_a and frame_b. The forces and torques at both frames are then evaluated in such a way that the constraints are satisfied. Sometimes this type of formulation is also called an implicit joint in literature.</p>
<p>As a consequence of the formulation the relative kinematics between frame_a and frame_b cannot be initialized.</p>
<p>In particular in complex multibody systems with closed loops this may help to simplify the system of non-linear equations. Please compare the translation log using the classical joint formulation and the alternative formulation used here in order to check whether this fact applies to the particular system under consideration.</p>
<p>In systems without closed loops the use of this implicit joint does not make sense or may even be disadvantageous.</p>
<p>See the subpackage <a href=\"Modelica://Modelica.Mechanics.MultiBody.Examples.Constraints\">Examples.Constraints</a> for testing the joint. </p>
</html>"));
          end Universal;
          annotation(
            Documentation(info = "<html>
<p>
This package contains <b>constraint components</b>, that is, idealized, massless elements that
constrain the motion between frames by means of kinematic constraints. The constraint
elements are especially aimed to be used for multibody models which contain <b>kinematic loops</b>.
Usually, kinematic loops are automatically handled. However, the performance might be improved
by either solving certain kinds of loops analytically with the help of the components of
subpackage  <a href=\"Modelica://Modelica.Mechanics.MultiBody.Joints.Assemblies\">Assemblies</a>, or
by providing numerically better loop constraint formulations with the help of the components
of this subpackage.
</p>
</html>"));
        end Constraints;

        package Internal "Components used for analytic solution of kinematic loops (use only if you know what you are doing)"
          extends Modelica.Icons.InternalPackage;

          model RevoluteWithLengthConstraint "Revolute joint where the rotation angle is computed from a length constraint (1 degree-of-freedom, no potential state)"
            extends Modelica.Mechanics.MultiBody.Interfaces.PartialTwoFrames;
            Modelica.Mechanics.Rotational.Interfaces.Flange_a axis "1-dim. rotational flange that drives the joint" annotation(
              Placement(transformation(extent = {{10, 90}, {-10, 110}}, rotation = 0)));
            Modelica.Mechanics.Rotational.Interfaces.Flange_b bearing "1-dim. rotational flange of the drive bearing" annotation(
              Placement(transformation(extent = {{-50, 90}, {-70, 110}}, rotation = 0)));
            Modelica.Blocks.Interfaces.RealInput position_a[3](each final quantity = "Length", each final unit = "m") "Position vector from frame_a to frame_a side of length constraint, resolved in frame_a of revolute joint" annotation(
              Placement(transformation(extent = {{-140, -80}, {-100, -40}}, rotation = 0)));
            Modelica.Blocks.Interfaces.RealInput position_b[3](each final quantity = "Length", each final unit = "m") "Position vector from frame_b to frame_b side of length constraint, resolved in frame_b of revolute joint" annotation(
              Placement(transformation(extent = {{140, -80}, {100, -40}}, rotation = 0)));
            parameter Boolean animation = true "= true, if animation shall be enabled";
            parameter SI.Position lengthConstraint(start = 1) "Fixed length of length constraint";
            parameter Modelica.Mechanics.MultiBody.Types.Axis n = {0, 0, 1} "Axis of rotation resolved in frame_a (= same as in frame_b)" annotation(
              Evaluate = true);
            parameter Cv.NonSIunits.Angle_deg phi_offset = 0 "Relative angle offset (angle = phi + from_deg(phi_offset))";
            parameter Cv.NonSIunits.Angle_deg phi_guess = 0 "Select the configuration such that at initial time |phi - from_deg(phi_guess)| is minimal";
            parameter SI.Distance cylinderLength = world.defaultJointLength "Length of cylinder representing the joint axis" annotation(
              Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            parameter SI.Distance cylinderDiameter = world.defaultJointWidth "Diameter of cylinder representing the joint axis" annotation(
              Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            input Types.Color cylinderColor = Modelica.Mechanics.MultiBody.Types.Defaults.JointColor "Color of cylinder representing the joint axis" annotation(
              Dialog(colorSelector = true, tab = "Animation", group = "if animation = true", enable = animation));
            input Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(
              Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            final parameter Boolean positiveBranch(fixed = false) "Based on phi_guess, selection of one of the two solutions of the non-linear constraint equation";
            final parameter Real e[3](each final unit = "1") = Modelica.Math.Vectors.normalizeWithAssert(n) "Unit vector in direction of rotation axis, resolved in frame_a";
            SI.Angle phi "Rotation angle of revolute joint";
            Frames.Orientation R_rel "Relative orientation object from frame_a to frame_b";
            SI.Angle angle "= phi + from_deg(phi_offset) (relative rotation angle between frame_a and frame_b)";
            SI.Torque tau "= axis.tau (driving torque in the axis)";
          protected
            SI.Position r_a[3] = position_a "Position vector from frame_a to frame_a side of length constraint, resolved in frame_a of revolute joint";
            SI.Position r_b[3] = position_b "Position vector from frame_b to frame_b side of length constraint, resolved in frame_b of revolute joint";
            Real e_r_a "Projection of r_a on e";
            Real e_r_b "Projection of r_b on e";
            Real A "Coefficient A of equation: A*cos(phi) + B*sin(phi) + C = 0";
            Real B "Coefficient B of equation: A*cos(phi) + B*sin(phi) + C = 0";
            Real C "Coefficient C of equation: A*cos(phi) + B*sin(phi) + C = 0";
            Real k1 "Constant of quadratic equation";
            Real k2 "Constant of quadratic equation";
            Real k1a(start = 1);
            Real k1b;
            Real kcos_angle "= k1*cos(angle)";
            Real ksin_angle "= k1*sin(angle)";
            Visualizers.Advanced.Shape cylinder(shapeType = "cylinder", color = cylinderColor, specularCoefficient = specularCoefficient, length = cylinderLength, width = cylinderDiameter, height = cylinderDiameter, lengthDirection = e, widthDirection = {0, 1, 0}, r_shape = -e * (cylinderLength / 2), r = frame_a.r_0, R = frame_a.R) if world.enableAnimation and animation;

            function selectBranch "Determine branch which is closest to initial angle=0"
              extends Modelica.Icons.Function;
              input SI.Length L "Length of length constraint";
              input Real e[3](each final unit = "1") "Unit vector along axis of rotation, resolved in frame_a (= same in frame_b)";
              input SI.Angle angle_guess "Select the configuration such that at initial time |angle-angle_guess| is minimal (angle=0: frame_a and frame_b coincide)";
              input SI.Position r_a[3] "Position vector from frame_a to frame_a side of length constraint, resolved in frame_a of revolute joint";
              input SI.Position r_b[3] "Position vector from frame_b to frame_b side of length constraint, resolved in frame_b of revolute joint";
              output Boolean positiveBranch "Branch of the initial solution";
            protected
              Real e_r_a "Projection of r_a on e";
              Real e_r_b "Projection of r_b on e";
              Real A "Coefficient A of equation: A*cos(phi) + B*sin(phi) + C = 0";
              Real B "Coefficient B of equation: A*cos(phi) + B*sin(phi) + C = 0";
              Real C "Coefficient C of equation: A*cos(phi) + B*sin(phi) + C = 0";
              Real k1 "Constant of quadratic equation";
              Real k2 "Constant of quadratic equation";
              Real k1a;
              Real k1b;
              Real kcos1 "k1*cos(angle1)";
              Real ksin1 "k1*sin(angle1)";
              Real kcos2 "k2*cos(angle2)";
              Real ksin2 "k2*sin(angle2)";
              SI.Angle angle1 "solution 1 of nonlinear equation";
              SI.Angle angle2 "solution 2 of nonlinear equation";
            algorithm
/* The position vector r_rel from frame_a to frame_b of the length constraint
       element, resolved in frame_b of the revolute joint is given by
       (T_rel is the planar transformation matrix from frame_a to frame_b of
        the revolute joint):
          r_rel = r_b - T_rel*r_a
       The length constraint can therefore be formulated as:
          r_rel*r_rel = L*L
       with
          (r_b - T_rel*r_a)*(r_b - T_rel*r_a)
             = r_b*r_b - 2*r_b*T_rel*r_a + r_a*transpose(T_rel)*T_rel*r_a
             = r_b*r_b + r_a*r_a - 2*r_b*T_rel*r_a
       follows
          (1) 0 = r_a*r_a + r_b*r_b - 2*r_b*T_rel*r_a - L*L
       The vectors r_a, r_b and parameter L are NOT a function of
       the angle of the revolute joint. Since T_rel = T_rel(angle) is a function
       of the unknown angle of the revolute joint, this is a non-linear
       equation in this angle.
          T_rel = [e]*transpose([e]) + (identity(3) - [e]*transpose([e]))*cos(angle)
                  - skew(e)*sin(angle);
       with
          r_b*T_rel*r_a
             = r_b*(e*(e*r_a) + (r_a - e*(e*r_a))*cos(angle) - cross(e,r_a)*sin(angle)
             = (e*r_b)*(e*r_a) + (r_b*r_a - (e*r_b)*(e*r_a))*cos(angle) - r_b*cross(e,r_a)*sin(angle)
       follows for the constraint equation (1)
          (2) 0 = r_a*r_a + r_b*r_b - L*L
                  - 2*(e*r_b)*(e*r_a)
                  - 2*(r_b*r_a - (e*r_b)*(e*r_a))*cos(angle)
                  + 2*r_b*cross(e,r_a)*sin(angle)
       or
          (3) A*cos(angle) + B*sin(angle) + C = 0
       with
              A = -2*(r_b*r_a - (e*r_b)*(e*r_a))
              B = 2*r_b*cross(e,r_a)
              C = r_a*r_a + r_b*r_b - L*L - 2*(e*r_b)*(e*r_a)
       Equation (3) is solved by computing sin(angle) and cos(angle)
       independently from each other. This allows to compute
       angle in the range: -180 deg <= angle <= 180 deg
    */
              e_r_a := e * r_a;
              e_r_b := e * r_b;
              A := -2 * (r_b * r_a - e_r_b * e_r_a);
              B := 2 * r_b * cross(e, r_a);
              C := r_a * r_a + r_b * r_b - L * L - 2 * e_r_b * e_r_a;
              k1 := A * A + B * B;
              k1a := k1 - C * C;
              assert(k1a > 1.e-10, "
Singular position of loop (either no or two analytic solutions;
the mechanism has lost one-degree-of freedom in this position).
Try first to use another Modelica.Mechanics.MultiBody.Joints.Assemblies.JointXXX component.
In most cases it is best that the joints outside of the JointXXX
component are revolute and NOT prismatic joints. If this also
lead to singular positions, it could be that this kinematic loop
cannot be solved analytically. In this case you have to build
up the loop with basic joints (NO aggregation JointXXX components)
and rely on dynamic state selection, i.e., during simulation
the states will be dynamically selected in such a way that in no
position a degree of freedom is lost.
              ");
              k1b := max(k1a, 1.0e-12);
              k2 := sqrt(k1b);
              kcos1 := (-A * C) + B * k2;
              ksin1 := (-B * C) - A * k2;
              angle1 := atan2(ksin1, kcos1);
              kcos2 := (-A * C) - B * k2;
              ksin2 := (-B * C) + A * k2;
              angle2 := atan2(ksin2, kcos2);
              if abs(angle1 - angle_guess) <= abs(angle2 - angle_guess) then
                positiveBranch := true;
              else
                positiveBranch := false;
              end if;
            end selectBranch;
          initial equation
            positiveBranch = selectBranch(lengthConstraint, e, Cv.from_deg(phi_offset + phi_guess), r_a, r_b);
          equation
            Connections.branch(frame_a.R, frame_b.R);
            axis.tau = tau;
            axis.phi = phi;
            bearing.phi = 0;
            angle = Cv.from_deg(phi_offset) + phi;
// transform kinematic quantities from frame_a to frame_b
            frame_b.r_0 = frame_a.r_0;
            R_rel = Frames.planarRotation(e, angle, der(angle));
            frame_b.R = Frames.absoluteRotation(frame_a.R, R_rel);
// Force and torque balance
            zeros(3) = frame_a.f + Frames.resolve1(R_rel, frame_b.f);
            zeros(3) = frame_a.t + Frames.resolve1(R_rel, frame_b.t);
// Compute rotation angle (details, see function "selectBranch")
            e_r_a = e * r_a;
            e_r_b = e * r_b;
            A = -2 * (r_b * r_a - e_r_b * e_r_a);
            B = 2 * r_b * cross(e, r_a);
            C = r_a * r_a + r_b * r_b - lengthConstraint * lengthConstraint - 2 * e_r_b * e_r_a;
            k1 = A * A + B * B;
            k1a = k1 - C * C;
            assert(k1a > 1.e-10, "
Singular position of loop (either no or two analytic solutions;
the mechanism has lost one-degree-of freedom in this position).
Try first to use another Modelica.Mechanics.MultiBody.Joints.Assemblies.JointXXX component.
In most cases it is best that the joints outside of the JointXXX
component are revolute and NOT prismatic joints. If this also
lead to singular positions, it could be that this kinematic loop
cannot be solved analytically. In this case you have to build
up the loop with basic joints (NO aggregation JointXXX components)
and rely on dynamic state selection, i.e., during simulation
the states will be dynamically selected in such a way that in no
position a degree of freedom is lost.
            ");
            k1b = Frames.Internal.maxWithoutEvent(k1a, 1.0e-12);
            k2 = sqrt(k1b);
            kcos_angle = (-A * C) + (if positiveBranch then B else -B) * k2;
            ksin_angle = (-B * C) + (if positiveBranch then -A else A) * k2;
            angle = Modelica.Math.atan2(ksin_angle, kcos_angle);
            annotation(
              Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-30, 10}, {10, -10}}, lineColor = {64, 64, 64}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-100, -60}, {-30, 60}}, lineColor = {64, 64, 64}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {255, 255, 255}, radius = 10), Rectangle(extent = {{30, -60}, {100, 60}}, lineColor = {64, 64, 64}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {255, 255, 255}, radius = 10), Text(extent = {{-139, -168}, {137, -111}}, textString = "%name", lineColor = {0, 0, 255}), Rectangle(extent = {{-100, 60}, {-30, -60}}, lineColor = {64, 64, 64}, radius = 10), Rectangle(extent = {{30, 60}, {100, -60}}, lineColor = {64, 64, 64}, radius = 10), Text(extent = {{-142, -108}, {147, -69}}, lineColor = {0, 0, 0}, textString = "n=%n"), Line(points = {{-60, 60}, {-60, 90}}, color = {0, 0, 0}), Line(points = {{-20, 70}, {-60, 70}}, color = {0, 0, 0}), Line(points = {{-20, 80}, {-20, 60}}, color = {0, 0, 0}), Line(points = {{20, 80}, {20, 60}}, color = {0, 0, 0}), Line(points = {{20, 70}, {41, 70}}, color = {0, 0, 0}), Polygon(points = {{-9, 30}, {10, 30}, {30, 50}, {-29, 50}, {-9, 30}}, lineColor = {64, 64, 64}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Polygon(points = {{10, 30}, {30, 50}, {30, -51}, {10, -31}, {10, 30}}, lineColor = {64, 64, 64}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-10, 90}, {10, 50}}, lineColor = {64, 64, 64}, fillPattern = FillPattern.VerticalCylinder, fillColor = {192, 192, 192})}),
              Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, -60}, {-30, 60}}, lineColor = {64, 64, 64}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {255, 255, 255}, radius = 10), Rectangle(extent = {{-100, -60}, {-30, 60}}, lineColor = {64, 64, 64}, fillPattern = FillPattern.None, radius = 10), Rectangle(extent = {{-30, 10}, {10, -10}}, lineColor = {64, 64, 64}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Rectangle(extent = {{30, -60}, {100, 60}}, lineColor = {64, 64, 64}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {255, 255, 255}, radius = 10), Rectangle(extent = {{30, -60}, {100, 60}}, lineColor = {64, 64, 64}, fillPattern = FillPattern.None, radius = 10), Line(points = {{-60, 60}, {-60, 96}}, color = {0, 0, 0}), Line(points = {{-20, 70}, {-60, 70}}, color = {0, 0, 0}), Line(points = {{-20, 80}, {-20, 60}}, color = {0, 0, 0}), Line(points = {{20, 80}, {20, 60}}, color = {0, 0, 0}), Line(points = {{20, 70}, {41, 70}}, color = {0, 0, 0}), Polygon(points = {{-9, 30}, {10, 30}, {30, 50}, {-29, 50}, {-9, 30}}, lineColor = {64, 64, 64}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Polygon(points = {{10, 30}, {30, 50}, {30, -51}, {10, -31}, {10, 30}}, lineColor = {64, 64, 64}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-10, 50}, {10, 100}}, lineColor = {64, 64, 64}, fillPattern = FillPattern.VerticalCylinder, fillColor = {192, 192, 192})}),
              Documentation(info = "<html>
<p>
Joint where frame_b rotates around axis n which is fixed in frame_a.
The two frames coincide when \"phi + phi_offset = 0\", where
\"phi_offset\" is a parameter with a zero default
and \"phi\" is the rotation angle.
</p>
<p>
This variant of the revolute joint is designed to work together
with a length constraint in a kinematic loop. This means that the
angle of the revolute joint, phi, is computed such that the
length constraint is fulfilled.
</p>
<p>
<b>Usually, this joint should not be used by a user of the MultiBody
library. It is only provided to built-up the Modelica.Mechanics.MultiBody.Joints.Assemblies.JointXYZ
joints.</b>
</p>

<p>
In releases before version 3.0 of the Modelica Standard Library, it was possible
to activate the torque projection equation (= cut-torque projected to the rotation
axis must be identical to the drive torque of flange axis) via parameter
<b>axisTorqueBalance</b>. This is no longer possible, since otherwise this
model would not be \"balanced\" (= same number of unknowns as equations).
Instead, when using this model in version 3.0 and later versions,
the force in the length constraint component (Joints.SphericalSpherical or
Joints.UniversalSpherical) must be calculated such that the driving torque
in direction of the rotation
axis is (RC shall be the name of the instance of RevoluteWithLenghtConstraint):
</p>
<pre>
    0 = RC.axis.tau + RC.e*RC.frame_b.t;
</pre>
<p>
If this equation is used, usually the force in the length constraint
and the second derivative of the revolute angle will be part of a linear
algebraic system of equations. In some cases it is possible to solve
this system of equations locally, i.e., provide the rod force directly
as function of the revolute constraint torque. In any case, this projection
equation or an equivalent one has to be provided via variable \"constraintResidue\" in the \"Advanced\"
menu of \"Joints.SphericalSpherical\" or \"Joints.UniversalSpherical\".
</p>

</html>"));
          end RevoluteWithLengthConstraint;

          model PrismaticWithLengthConstraint "Prismatic joint where the translational distance is computed from a length constraint (1 degree-of-freedom, no potential state)"
            extends Modelica.Mechanics.MultiBody.Interfaces.PartialTwoFrames;
            Modelica.Mechanics.Translational.Interfaces.Flange_a axis "1-dim. translational flange that drives the joint" annotation(
              Placement(transformation(extent = {{70, 80}, {90, 60}}, rotation = 0)));
            Modelica.Mechanics.Translational.Interfaces.Flange_b bearing "1-dim. translational flange of the drive bearing" annotation(
              Placement(transformation(extent = {{-30, 80}, {-50, 60}}, rotation = 0)));
            Modelica.Blocks.Interfaces.RealInput position_a[3](each final quantity = "Length", each final unit = "m") "Position vector from frame_a to frame_a side of length constraint, resolved in frame_a of prismatic joint" annotation(
              Placement(transformation(extent = {{-140, -80}, {-100, -40}}, rotation = 0)));
            Modelica.Blocks.Interfaces.RealInput position_b[3](each final quantity = "Length", each final unit = "m") "Position vector from frame_b to frame_b side of length constraint, resolved in frame_b of prismatic joint" annotation(
              Placement(transformation(extent = {{140, -80}, {100, -40}}, rotation = 0)));
            parameter Boolean animation = true "= true, if animation shall be enabled";
            parameter SI.Position length(start = 1) "Fixed length of length constraint";
            parameter Modelica.Mechanics.MultiBody.Types.Axis n = {1, 0, 0} "Axis of translation resolved in frame_a (= same as in frame_b)" annotation(
              Evaluate = true);
            parameter SI.Position s_offset = 0 "Relative distance offset (distance between frame_a and frame_b = s(t) + s_offset)";
            parameter SI.Position s_guess = 0 "Select the configuration such that at initial time |s(t0)-s_guess| is minimal";
            parameter Types.Axis boxWidthDirection = {0, 1, 0} "Vector in width direction of box, resolved in frame_a" annotation(
              Evaluate = true,
              Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            parameter SI.Distance boxWidth = world.defaultJointWidth "Width of prismatic joint box" annotation(
              Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            parameter SI.Distance boxHeight = boxWidth "Height of prismatic joint box" annotation(
              Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            input Types.Color boxColor = Modelica.Mechanics.MultiBody.Types.Defaults.JointColor "Color of prismatic joint box" annotation(
              Dialog(colorSelector = true, tab = "Animation", group = "if animation = true", enable = animation));
            input Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(
              Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            final parameter Boolean positiveBranch(fixed = false) "Selection of one of the two solutions of the non-linear constraint equation";
            final parameter Real e[3](each final unit = "1") = Modelica.Math.Vectors.normalizeWithAssert(n) "Unit vector in direction of translation axis, resolved in frame_a";
            SI.Position s "Relative distance between frame_a and frame_b along axis n = s + s_offset)";
            SI.Position distance "Relative distance between frame_a and frame_b along axis n";
            SI.Position r_rel_a[3] "Position vector from frame_a to frame_b resolved in frame_a";
            SI.Force f "= axis.f (driving force in the axis)";
          protected
            SI.Position r_a[3] = position_a "Position vector from frame_a to frame_a side of length constraint, resolved in frame_a of prismatic joint";
            SI.Position r_b[3] = position_b "Position vector from frame_b to frame_b side of length constraint, resolved in frame_b of prismatic joint";
            Modelica.SIunits.Position rbra[3] "= rb - ra";
            Real B "Coefficient B of equation: s*s + B*s + C = 0";
            Real C "Coefficient C of equation: s*s + B*s + C = 0";
            Real k1 "Constant of quadratic equation solution";
            Real k2 "Constant of quadratic equation solution";
            Real k1a(start = 1);
            Real k1b;
            Visualizers.Advanced.Shape box(shapeType = "box", color = boxColor, specularCoefficient = specularCoefficient, length = if noEvent(abs(s + s_offset) > 1.e-6) then s + s_offset else 1.e-6, width = boxWidth, height = boxHeight, lengthDirection = e, widthDirection = boxWidthDirection, r = frame_a.r_0, R = frame_a.R) if world.enableAnimation and animation;

            function selectBranch "Determine branch which is closest to initial angle=0"
              extends Modelica.Icons.Function;
              input SI.Length L "Length of length constraint";
              input Real e[3](each final unit = "1") "Unit vector along axis of translation, resolved in frame_a (= same in frame_b)";
              input SI.Position d_guess "Select the configuration such that at initial time |d-d_guess| is minimal (d: distance between origin of frame_a and origin of frame_b)";
              input SI.Position r_a[3] "Position vector from frame_a to frame_a side of length constraint, resolved in frame_a of prismatic joint";
              input SI.Position r_b[3] "Position vector from frame_b to frame_b side of length constraint, resolved in frame_b of prismatic joint";
              output Boolean positiveBranch "Branch of the initial solution";
            protected
              Modelica.SIunits.Position rbra[3] "= rb - ra";
              Real B "Coefficient B of equation: d*d + B*d + C = 0";
              Real C "Coefficient C of equation: d*d + B*d + C = 0";
              Real k1 "Constant of quadratic equation solution";
              Real k2 "Constant of quadratic equation solution";
              Real k1a;
              Real k1b;
              Real d1 "solution 1 of quadratic equation";
              Real d2 "solution 2 of quadratic equation";
            algorithm
/* The position vector r_rel from frame_a to frame_b of the length constraint
       element, resolved in frame_b of the prismatic joint (frame_a and frame_b
       of the prismatic joint are parallel to each other) is given by:
          r_rel = d*e + r_b - r_a
       The length constraint can therefore be formulated as:
          r_rel*r_rel = L*L
       with
          (d*e + r_b - r_a)*(d*e + r_b - r_a)
                   = d*d + 2*d*e*(r_b - r_a) + (r_b - r_a)*(r_b - r_a)
       follows
          (1)  0 = d*d + d*2*e*(r_b - r_a) + (r_b - r_a)*(r_b - r_a) - L*L
       The vectors r_a, r_b and parameter L are NOT a function of
       the distance d of the prismatic joint. Therefore, (1) is a quadratic
       equation in the single unknown "d":
          (2) d*d + B*d + C = 0
              with   B = 2*e*(r_b - r_a)
                     C = (r_b - r_a)*(r_b - r_a) - L*L
       The solution is
          (3) d = - B/2 +/- sqrt(B*B/4 - C)
    */
              rbra := r_b - r_a;
              B := 2 * (e * rbra);
              C := rbra * rbra - L * L;
              k1 := B / 2;
              k1a := k1 * k1 - C;
              assert(noEvent(k1a > 1.e-10), "
Singular position of loop (either no or two analytic solutions;
the mechanism has lost one-degree-of freedom in this position).
Try first to use another Modelica.Mechanics.MultiBody.Joints.Assemblies.JointXXX component.
If this also lead to singular positions, it could be that this
kinematic loop cannot be solved analytically with a fixed state
selection. In this case you have to build up the loop with
basic joints (NO aggregation JointXXX components) and rely on
dynamic state selection, i.e., during simulation the states will
be dynamically selected in such a way that in no position a
degree of freedom is lost.
              ");
              k1b := max(k1a, 1.0e-12);
              k2 := sqrt(k1b);
              d1 := (-k1) + k2;
              d2 := (-k1) - k2;
              if abs(d1 - d_guess) <= abs(d2 - d_guess) then
                positiveBranch := true;
              else
                positiveBranch := false;
              end if;
            end selectBranch;
          initial equation
            positiveBranch = selectBranch(length, e, s_offset + s_guess, r_a, r_b);
          equation
            Connections.branch(frame_a.R, frame_b.R);
            axis.f = f;
            axis.s = s;
            bearing.s = 0;
            distance = s_offset + s;
// relationships of frame_a and frame_b quantities
            r_rel_a = e * distance;
            frame_b.r_0 = frame_a.r_0 + Frames.resolve1(frame_a.R, r_rel_a);
            frame_b.R = frame_a.R;
            zeros(3) = frame_a.f + frame_b.f;
            zeros(3) = frame_a.t + frame_b.t + cross(r_rel_a, frame_b.f);
// Compute translational distance (details, see function "selectBranch")
            rbra = r_b - r_a;
            B = 2 * (e * rbra);
            C = rbra * rbra - length * length;
            k1 = B / 2;
            k1a = k1 * k1 - C;
            assert(noEvent(k1a > 1.e-10), "
Singular position of loop (either no or two analytic solutions;
the mechanism has lost one-degree-of freedom in this position).
Try first to use another Modelica.Mechanics.MultiBody.Joints.Assemblies.JointXXX component.
If this also lead to singular positions, it could be that this
kinematic loop cannot be solved analytically with a fixed state
selection. In this case you have to build up the loop with
basic joints (NO aggregation JointXXX components) and rely on
dynamic state selection, i.e., during simulation the states will
be dynamically selected in such a way that in no position a
degree of freedom is lost.
            ");
            k1b = Frames.Internal.maxWithoutEvent(k1a, 1.0e-12);
            k2 = sqrt(k1b);
            distance = (-k1) + (if positiveBranch then k2 else -k2);
            annotation(
              Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-30, -40}, {100, 30}}, pattern = LinePattern.None, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Rectangle(extent = {{-30, 40}, {100, -40}}, lineColor = {0, 0, 0}), Rectangle(extent = {{-100, -60}, {-30, 50}}, pattern = LinePattern.None, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Rectangle(extent = {{-100, 50}, {-30, 60}}, pattern = LinePattern.None, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Rectangle(extent = {{-30, 30}, {100, 40}}, pattern = LinePattern.None, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Text(extent = {{-136, -170}, {140, -113}}, textString = "%name", lineColor = {0, 0, 255}), Rectangle(extent = {{-100, 60}, {-30, -60}}, lineColor = {0, 0, 0}), Line(points = {{100, -40}, {100, -60}}, color = {0, 0, 255}), Rectangle(extent = {{100, 40}, {90, 80}}, lineColor = {0, 0, 0}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Text(extent = {{-136, -116}, {153, -77}}, lineColor = {0, 0, 0}, textString = "n=%n")}),
              Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-30, -50}, {-30, 50}}, color = {0, 0, 0}), Line(points = {{0, -67}, {90, -67}}, color = {128, 128, 128}), Text(extent = {{31, -68}, {68, -81}}, lineColor = {128, 128, 128}, textString = "s"), Line(points = {{-100, -67}, {0, -67}}, color = {128, 128, 128}), Polygon(points = {{-39, -64}, {-29, -67}, {-39, -70}, {-39, -64}}, lineColor = {128, 128, 128}, fillColor = {128, 128, 128}, fillPattern = FillPattern.Solid), Text(extent = {{-77, -70}, {-43, -85}}, lineColor = {128, 128, 128}, textString = "s_offset"), Line(points = {{-100, -71}, {-100, -51}}, color = {128, 128, 128}), Line(points = {{-30, -73}, {-30, -33}}, color = {128, 128, 128}), Line(points = {{100, -70}, {100, -30}}, color = {128, 128, 128}), Polygon(points = {{90, -64}, {100, -67}, {90, -70}, {90, -64}}, lineColor = {128, 128, 128}, fillColor = {128, 128, 128}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-100, 50}, {-30, 60}}, pattern = LinePattern.None, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Rectangle(extent = {{-100, -60}, {-30, 50}}, pattern = LinePattern.None, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Rectangle(extent = {{-30, 40}, {100, -40}}, lineColor = {0, 0, 0}), Rectangle(extent = {{-30, -40}, {100, 30}}, pattern = LinePattern.None, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Rectangle(extent = {{-30, 30}, {100, 40}}, pattern = LinePattern.None, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Rectangle(extent = {{-100, 60}, {-30, -60}}, lineColor = {0, 0, 0}), Line(points = {{100, -40}, {100, -60}}, color = {0, 0, 255}), Text(extent = {{42, 91}, {57, 76}}, textString = "f", lineColor = {0, 0, 255}), Line(points = {{40, 75}, {70, 75}}, color = {0, 0, 255}), Polygon(points = {{-21, 78}, {-31, 75}, {-21, 72}, {-21, 78}}, lineColor = {0, 0, 255}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid), Line(points = {{-8, 75}, {-31, 75}}, color = {0, 0, 255}), Text(extent = {{-21, 90}, {-6, 75}}, textString = "f", lineColor = {0, 0, 255}), Polygon(points = {{60, 78}, {70, 75}, {60, 72}, {60, 78}}, lineColor = {0, 0, 255}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid), Line(points = {{-30, 64}, {70, 64}}, color = {128, 128, 128}), Polygon(points = {{60, 67}, {70, 64}, {60, 61}, {60, 67}}, lineColor = {128, 128, 128}, fillColor = {128, 128, 128}, fillPattern = FillPattern.Solid), Text(extent = {{0, 63}, {37, 50}}, lineColor = {128, 128, 128}, textString = "s"), Rectangle(extent = {{100, 40}, {90, 80}}, lineColor = {0, 0, 0}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid)}),
              Documentation(info = "<HTML>
<p>
Joint where frame_b is translated along axis n which is fixed in frame_a.
The two frames coincide when \"s + s_offset = 0\", where
\"s_offset\" is a parameter with a zero default
and \"s\" is the relative distance.
</p>
<p>
This variant of the prismatic joint is designed to work together
with a length constraint in a kinematic loop. This means that the
relative distance \"s\" of the joint is computed such that the
length constraint is fulfilled.
</p>
<p>
<b>Usually, this joint should not be used by a user of the MultiBody
library. It is only provided to built-up the Modelica.Mechanics.MultiBody.Joints.Assemblies.JointXYZ
joints.</b>
</p>

<p>
In releases before version 3.0 of the Modelica Standard Library, it was possible
to activate the force projection equation (= cut-force projected to the translation
axis must be identical to the driving force of flange axis) via parameter
<b>axisForceBalance</b>. This is no longer possible, since otherwise this
model would not be \"balanced\" (= same number of unknowns as equations).
Instead, when using this model in version 3.0 and later versions,
the force in the length constraint component (Joints.SphericalSpherical or
Joints.UniversalSpherical) must be calculated such that the driving force
in direction of the translation
axis is (RC shall be the name of the instance of PrismaticWithLenghtConstraint):
</p>
<pre>
    0 = RC.axis.f + RC.e*RC.frame_b.f;
</pre>
<p>
If this equation is used, usually the force in the length constraint
and the second derivative of the prismatic distance will be part of a linear
algebraic system of equations. In some cases it is possible to solve
this system of equations locally, i.e., provide the rod force directly
as function of the prismatic constraint force. In any case, this projection
equation or an equivalent one has to be provided via variable \"constraintResidue\" in the \"Advanced\"
menu of \"Joints.SphericalSpherical\" or \"Joints.UniversalSpherical\".
</p>

</html>"));
          end PrismaticWithLengthConstraint;

          model RollingConstraintVerticalWheel "Rolling constraint for wheel that is always perpendicular to x-y plane"
            import Modelica.Mechanics.MultiBody.Frames;
            Modelica.Mechanics.MultiBody.Interfaces.Frame_a frame_a "Frame fixed in wheel center point. x-Axis: upwards, y-axis: along wheel axis" annotation(
              Placement(transformation(extent = {{-16, 4}, {16, 36}}), iconTransformation(extent = {{-16, 4}, {16, 36}})));
            parameter SI.Radius radius "Wheel radius";
            parameter Boolean lateralSlidingConstraint = true "= true, if lateral sliding constraint taken into account, = false if lateral force = 0 (needed to avoid overconstraining if two ideal rolling wheels are connect on one axis)" annotation(
              choices(checkBox = true),
              HideResult = true,
              Evaluate = true);
            // Contact force
            SI.Force f_wheel_0[3] "Contact force acting on wheel, resolved in world frame";
            SI.Force f_lat "Contact force acting on wheel in lateral direction";
            SI.Force f_long "Contact force acting on wheel in longitudinal direction";
          protected
            Real e_axis_0[3] "Unit vector along wheel axis, resolved in world frame";
            SI.Position rContact_0[3] "Distance vector from wheel center to contact point, resolved in world frame";
            // Coordinate system at contact point
            Real e_n_0[3] "Unit vector in normal direction of road at contact point, resolved in world frame";
            Real e_lat_0[3] "Unit vector in lateral direction of wheel at contact point, resolved in world frame";
            Real e_long_0[3] "Unit vector in longitudinal direction of wheel at contact point, resolved in world frame";
            // Slip velocities
            SI.Velocity v_0[3] "Velocity of wheel center, resolved in world frame";
            SI.AngularVelocity w_0[3] "Angular velocity of wheel, resolved in world frame";
            SI.Velocity vContact_0[3] "Velocity of wheel contact point, resolved in world frame";
            // Utility vectors
            Real aux[3];
          equation
// Coordinate system at contact point (e_long_0, e_lat_0, e_n_0)
            e_n_0 = {0, 0, 1};
            e_axis_0 = Frames.resolve1(frame_a.R, {0, 1, 0});
            aux = cross(e_n_0, e_axis_0);
            e_long_0 = aux / Modelica.Math.Vectors.length(aux);
            e_lat_0 = cross(e_long_0, e_n_0);
// Slip velocities
            rContact_0 = {0, 0, -radius};
            v_0 = der(frame_a.r_0);
            w_0 = Frames.angularVelocity1(frame_a.R);
            vContact_0 = v_0 + cross(w_0, rContact_0);
// Two non-holonomic constraint equations on velocity level (ideal rolling, no slippage)
            0 = vContact_0 * e_long_0;
            if lateralSlidingConstraint then
              0 = vContact_0 * e_lat_0;
              f_wheel_0 = f_lat * e_lat_0 + f_long * e_long_0;
            else
              0 = f_lat;
              f_wheel_0 = f_long * e_long_0;
            end if;
// Force and torque balance at the wheel center
            zeros(3) = frame_a.f + Frames.resolve2(frame_a.R, f_wheel_0);
            zeros(3) = frame_a.t + Frames.resolve2(frame_a.R, cross(rContact_0, f_wheel_0));
            annotation(
              Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, -60}, {100, -80}}, lineColor = {0, 0, 0}, fillColor = {175, 175, 175}, fillPattern = FillPattern.Solid), Text(extent = {{-148, -86}, {152, -126}}, lineColor = {0, 0, 255}, textString = "%name"), Line(points = {{0, -60}, {0, 4}}, color = {0, 0, 0}, smooth = Smooth.None, pattern = LinePattern.Dot), Line(visible = lateralSlidingConstraint, points = {{-98, -30}, {-16, -30}}, color = {0, 0, 0}, smooth = Smooth.None), Polygon(visible = lateralSlidingConstraint, points = {{-40, -16}, {-40, -42}, {-6, -30}, {-40, -16}}, lineColor = {0, 0, 0}, smooth = Smooth.None, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid)}));
          end RollingConstraintVerticalWheel;

          model InitPosition "Internal model to initialize r_rel_a for Joints.FreeMotionScalarInit"
            extends Modelica.Blocks.Icons.Block;
            import SI = Modelica.SIunits;
            import Modelica.Mechanics.MultiBody.Frames;
            input SI.Position r_a_0[3] annotation(
              HideResult = true,
              Dialog);
            input SI.Position r_b_0[3] annotation(
              HideResult = true,
              Dialog);
            input Frames.Orientation R_a annotation(
              HideResult = true,
              Dialog);
            Modelica.Blocks.Interfaces.RealOutput r_rel_a[3](each final quantity = "Length", each final unit = "m") annotation(
              Placement(transformation(extent = {{100, -10}, {120, 10}})));
          equation
            r_b_0 = r_a_0 + Frames.resolve1(R_a, {r_rel_a[1], r_rel_a[2], r_rel_a[3]});
            annotation(
              Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-88, 16}, {82, -12}}, lineColor = {0, 0, 0}, textString = "r_rel_a")}));
          end InitPosition;

          model InitAngle "Internal model to initialize the angels for Joints.FreeMotionScalarInit"
            extends Modelica.Blocks.Icons.Block;
            import SI = Modelica.SIunits;
            import Modelica.Mechanics.MultiBody.Frames;
            parameter Modelica.Mechanics.MultiBody.Types.RotationSequence sequence_start = {1, 2, 3} "Sequence of angle rotations";
            Interfaces.Frame_a frame_a annotation(
              Placement(transformation(extent = {{-116, -16}, {-84, 16}})));
            Interfaces.Frame_b frame_b annotation(
              Placement(transformation(extent = {{84, -16}, {116, 16}})));
            Frames.Orientation R_rel "Relative orientation object to rotate from frame_a to frame_b" annotation(
              HideResult = true);
            Frames.Orientation R_rel_inv "Relative orientation object to rotate from frame_b to frame_a" annotation(
              HideResult = true);
            Blocks.Interfaces.RealOutput angle[3](each final quantity = "Angle", each final unit = "rad") annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = -90, origin = {0, -110})));
          equation
            Connections.branch(frame_a.R, frame_b.R);
            R_rel = Frames.axesRotations(sequence_start, {angle[1], angle[2], angle[3]}, {der(angle[1]), der(angle[2]), der(angle[3])});
            if Connections.rooted(frame_a.R) then
              R_rel_inv = Frames.nullRotation();
              frame_b.R = Frames.absoluteRotation(frame_a.R, R_rel);
            else
              R_rel_inv = Frames.inverseRotation(R_rel);
              frame_a.R = Frames.absoluteRotation(frame_b.R, R_rel_inv);
            end if;
            frame_a.f = zeros(3);
            frame_a.t = zeros(3);
            frame_b.f = zeros(3);
            frame_b.t = zeros(3);
            annotation(
              Icon(graphics = {Text(extent = {{-84, -58}, {86, -86}}, lineColor = {0, 0, 0}, textString = "angle")}));
          end InitAngle;

          model InitAngularVelocity "Internal model to initialize w_rel_b for Joints.FreeMotionScalarInit"
            extends Modelica.Blocks.Icons.Block;
            import SI = Modelica.SIunits;
            import Modelica.Mechanics.MultiBody.Frames;
            input Frames.Orientation R_a annotation(
              HideResult = true,
              Dialog);
            input Frames.Orientation R_b annotation(
              HideResult = true,
              Dialog);
            Modelica.Blocks.Interfaces.RealOutput w_rel_b[3](each final quantity = "AngularVelocity", each final unit = "rad/s") annotation(
              Placement(transformation(extent = {{100, -10}, {120, 10}})));
          equation
            Frames.angularVelocity2(R_b) = Frames.resolve2(R_b, Frames.angularVelocity1(R_a)) + w_rel_b;
            annotation(
              Icon(graphics = {Text(extent = {{-86, 16}, {84, -12}}, lineColor = {0, 0, 0}, textString = "w_rel_b")}));
          end InitAngularVelocity;
          annotation(
            Documentation(info = "<HTML>
<p>
The models in this package should not be used by the user.
They are designed to build up other models in the MultiBody library
and some of them cannot be used in an arbitrary way and require
particular knowledge how to set the options in the parameter menu.
Don't use the models of this package.
</p>
</HTML>"));
        end Internal;
        annotation(
          Documentation(info = "<HTML>
<p>
This package contains <b>joint components</b>,
that is, idealized, massless elements that constrain
the motion between frames. In subpackage <b>Assemblies</b>
aggregation joint components are provided to handle
kinematic loops analytically (this means that non-linear systems
of equations occurring in these joint aggregations are analytically
solved, i.e., robustly and efficiently).
</p>
<h4>Content</h4>
<table border=1 cellspacing=0 cellpadding=2>
  <tr><th><b><i>Model</i></b></th><th><b><i>Description</i></b></th></tr>
  <tr><td valign=\"top\"><a href=\"modelica://Modelica.Mechanics.MultiBody.Joints.Prismatic\">Prismatic</a>
      <td valign=\"top\">Prismatic joint and actuated prismatic joint
          (1 translational degree-of-freedom, 2 potential states)<br>
      <IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Joints/Prismatic.png\">
      </td>
  </tr>
  <tr><td valign=\"top\"><a href=\"modelica://Modelica.Mechanics.MultiBody.Joints.Revolute\">Revolute</a>
 </td>
      <td valign=\"top\">Revolute and actuated revolute joint
          (1 rotational degree-of-freedom, 2 potential states)<br>
      <IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Joints/Revolute.png\">
      </td>
  </tr>
  <tr><td valign=\"top\"><a href=\"modelica://Modelica.Mechanics.MultiBody.Joints.Cylindrical\">Cylindrical</a></td>
      <td valign=\"top\">Cylindrical joint (2 degrees-of-freedom, 4 potential states)<br>
      <IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Joints/Cylindrical.png\">
      </td>
  </tr>
  <tr><td valign=\"top\"><a href=\"modelica://Modelica.Mechanics.MultiBody.Joints.Universal\">Universal</a></td>
      <td valign=\"top\">Universal joint (2 degrees-of-freedom, 4 potential states)<br>
      <IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Joints/Universal.png\">
      </td>
  </tr>
  <tr><td valign=\"top\"><a href=\"modelica://Modelica.Mechanics.MultiBody.Joints.Planar\">Planar</a></td>
      <td valign=\"top\">Planar joint (3 degrees-of-freedom, 6 potential states)<br>
      <IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Joints/Planar.png\">
      </td>
  </tr>
  <tr><td valign=\"top\"><a href=\"modelica://Modelica.Mechanics.MultiBody.Joints.Spherical\">Spherical</a></td>
      <td valign=\"top\">Spherical joint (3 constraints and no potential states, or 3 degrees-of-freedom and 3 states)<br>
      <IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Joints/Spherical.png\">
      </td>
  </tr>
  <tr><td valign=\"top\"><a href=\"modelica://Modelica.Mechanics.MultiBody.Joints.FreeMotion\">FreeMotion</a></td>
      <td valign=\"top\">Free motion joint (6 degrees-of-freedom, 12 potential states)<br>
      <IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Joints/FreeMotion.png\">
      </td>
  </tr>
  <tr><td valign=\"top\"><a href=\"modelica://Modelica.Mechanics.MultiBody.Joints.SphericalSpherical\">SphericalSpherical</a></td>
      <td valign=\"top\">Spherical - spherical joint aggregation (1 constraint,
          no potential states) with an optional point mass in the middle<br>
      <IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Joints/SphericalSpherical.png\">
      </td>
  </tr>
  <tr><td valign=\"top\"><a href=\"modelica://Modelica.Mechanics.MultiBody.Joints.UniversalSpherical\">UniversalSpherical</a></td>
      <td valign=\"top\">Universal - spherical joint aggregation (1 constraint, no potential states)<br>
      <IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Joints/UniversalSpherical.png\">
      </td>
  </tr>
  <tr><td valign=\"top\"><a href=\"modelica://Modelica.Mechanics.MultiBody.Joints.GearConstraint\">GearConstraint</a></td>
      <td valign=\"top\">Ideal 3-dim. gearbox (arbitrary shaft directions)
      </td>
  </tr>
  <tr><td valign=\"top\"><a href=\"modelica://Modelica.Mechanics.MultiBody.Joints.Assemblies\">MultiBody.Joints.Assemblies</a></td>
      <td valign=\"top\"><b>Package</b> of joint aggregations for analytic loop handling.
      </td>
  </tr>
  <tr><td valign=\"top\"><a href=\"modelica://Modelica.Mechanics.MultiBody.Joints.Constraints\">MultiBody.Joints.Constraints</a></td>
      <td valign=\"top\"><b>Package</b> of components that define joints by constraints
      </td>
  </tr>
</table>
</HTML>"),
          Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{6, 6}, {28, -2}, {54, 80}, {32, 86}, {6, 6}}, lineColor = {95, 95, 95}, fillPattern = FillPattern.Sphere, smooth = Smooth.None, fillColor = {255, 255, 255}), Polygon(points = {{-12, -18}, {0, -36}, {-70, -84}, {-82, -66}, {-12, -18}}, lineColor = {95, 95, 95}, fillPattern = FillPattern.Sphere, smooth = Smooth.None, fillColor = {255, 255, 255}), Ellipse(extent = {{-12, 8}, {34, -38}}, lineColor = {95, 95, 95}, fillPattern = FillPattern.Sphere, fillColor = {95, 95, 95})}));
      end Joints;

      package Parts "Rigid components such as bodies with mass and inertia and massless rods"
        extends Modelica.Icons.Package;

        model Fixed "Frame fixed in the world frame at a given position"
          import Modelica.Mechanics.MultiBody.Types;
          import Modelica.SIunits.Conversions.to_unit1;
          Interfaces.Frame_b frame_b "Coordinate system fixed in the world frame" annotation(
            Placement(transformation(extent = {{84, -16}, {116, 16}}, rotation = 0)));
          parameter Boolean animation = true "= true, if animation shall be enabled";
          parameter SI.Position r[3] = {0, 0, 0} "Position vector from world frame to frame_b, resolved in world frame";
          parameter Types.ShapeType shapeType = "cylinder" "Type of shape" annotation(
            Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter SI.Position r_shape[3] = {0, 0, 0} "Vector from world frame to shape origin, resolved in world frame" annotation(
            Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter Types.Axis lengthDirection = to_unit1(r - r_shape) "Vector in length direction of shape, resolved in world frame" annotation(
            Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter Types.Axis widthDirection = {0, 1, 0} "Vector in width direction of shape, resolved in world frame" annotation(
            Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter SI.Length length = Modelica.Math.Vectors.length(r - r_shape) "Length of shape" annotation(
            Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter SI.Distance width = length / world.defaultWidthFraction "Width of shape" annotation(
            Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter SI.Distance height = width "Height of shape" annotation(
            Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter Types.ShapeExtra extra = 0.0 "Additional parameter for cone, pipe etc. (see docu of Visualizers.Advanced.Shape)" annotation(
            Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          input Types.Color color = Modelica.Mechanics.MultiBody.Types.Defaults.RodColor "Color of shape" annotation(
            Dialog(colorSelector = true, tab = "Animation", group = "if animation = true", enable = animation));
          input Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(
            Dialog(tab = "Animation", group = "if animation = true", enable = animation));
        protected
          outer Modelica.Mechanics.MultiBody.World world;
          Visualizers.Advanced.Shape shape(shapeType = shapeType, color = color, specularCoefficient = specularCoefficient, length = length, width = width, height = height, lengthDirection = lengthDirection, widthDirection = widthDirection, extra = extra, r_shape = r_shape, r = zeros(3), R = Frames.nullRotation()) if world.enableAnimation and animation;
        equation
          Connections.root(frame_b.R);
          frame_b.r_0 = r;
          frame_b.R = Frames.nullRotation();
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-90, 90}, {90, -90}}, lineColor = {255, 255, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Text(extent = {{150, 145}, {-150, 105}}, textString = "%name", lineColor = {0, 0, 255}), Line(points = {{0, 100}, {0, -100}}, color = {0, 0, 0}), Line(points = {{0, -80}, {-100, -20}}, color = {0, 0, 0}), Line(points = {{0, -40}, {-100, 20}}, color = {0, 0, 0}), Line(points = {{0, 0}, {-100, 60}}, color = {0, 0, 0}), Line(points = {{0, 40}, {-100, 100}}, color = {0, 0, 0}), Line(points = {{0, 0}, {100, 0}}, color = {0, 0, 0}), Text(extent = {{-150, -105}, {150, -135}}, lineColor = {0, 0, 0}, textString = "r=%r")}),
            Documentation(info = "<html>
<p>
Element consisting of a frame (frame_b) that is fixed in the world
frame at a given position defined by parameter vector <b>r</b>
(vector from origin of world frame to frame_b, resolved in the
world frame).
</p>
<p>
By default, this component is visualized by a cylinder connecting the
world frame and frame_b of this components, as shown in the figure below.
Note, that the visualized world frame on the left side and
Fixed.frame_b on the right side are not part of the
component animation and that the animation may be switched off via parameter
animation = <b>false</b>.
</p>
<p>
<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Fixed.png\" ALT=\"Parts.Fixed\">
</p>

</html>"),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{0, 100}, {0, -100}}, color = {0, 0, 0}), Line(points = {{0, -80}, {-100, -20}}, color = {0, 0, 0}), Line(points = {{0, -40}, {-100, 20}}, color = {0, 0, 0}), Line(points = {{0, 0}, {-100, 60}}, color = {0, 0, 0}), Line(points = {{0, 40}, {-100, 100}}, color = {0, 0, 0}), Line(points = {{0, 0}, {100, 0}}, color = {0, 0, 0})}));
        end Fixed;

        model FixedTranslation "Fixed translation of frame_b with respect to frame_a"
          import Modelica.Mechanics.MultiBody.Types;
          import Modelica.SIunits.Conversions.to_unit1;
          Interfaces.Frame_a frame_a "Coordinate system fixed to the component with one cut-force and cut-torque" annotation(
            Placement(transformation(extent = {{-116, -16}, {-84, 16}}, rotation = 0)));
          Interfaces.Frame_b frame_b "Coordinate system fixed to the component with one cut-force and cut-torque" annotation(
            Placement(transformation(extent = {{84, -16}, {116, 16}}, rotation = 0)));
          parameter Boolean animation = true "= true, if animation shall be enabled";
          parameter SI.Position r[3](start = {0, 0, 0}) "Vector from frame_a to frame_b resolved in frame_a";
          parameter Types.ShapeType shapeType = "cylinder" "Type of shape" annotation(
            Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter SI.Position r_shape[3] = {0, 0, 0} "Vector from frame_a to shape origin, resolved in frame_a" annotation(
            Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter Types.Axis lengthDirection = to_unit1(r - r_shape) "Vector in length direction of shape, resolved in frame_a" annotation(
            Evaluate = true,
            Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter Types.Axis widthDirection = {0, 1, 0} "Vector in width direction of shape, resolved in frame_a" annotation(
            Evaluate = true,
            Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter SI.Length length = Modelica.Math.Vectors.length(r - r_shape) "Length of shape" annotation(
            Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter SI.Distance width = length / world.defaultWidthFraction "Width of shape" annotation(
            Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter SI.Distance height = width "Height of shape" annotation(
            Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter Types.ShapeExtra extra = 0.0 "Additional parameter depending on shapeType (see docu of Visualizers.Advanced.Shape)" annotation(
            Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          input Types.Color color = Modelica.Mechanics.MultiBody.Types.Defaults.RodColor "Color of shape" annotation(
            Dialog(colorSelector = true, tab = "Animation", group = "if animation = true", enable = animation));
          input Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(
            Dialog(tab = "Animation", group = "if animation = true", enable = animation));
        protected
          outer Modelica.Mechanics.MultiBody.World world;
          Visualizers.Advanced.Shape shape(shapeType = shapeType, color = color, specularCoefficient = specularCoefficient, r_shape = r_shape, lengthDirection = lengthDirection, widthDirection = widthDirection, length = length, width = width, height = height, extra = extra, r = frame_a.r_0, R = frame_a.R) if world.enableAnimation and animation;
        equation
          Connections.branch(frame_a.R, frame_b.R);
          assert(cardinality(frame_a) > 0 or cardinality(frame_b) > 0, "Neither connector frame_a nor frame_b of FixedTranslation object is connected");
          frame_b.r_0 = frame_a.r_0 + Frames.resolve1(frame_a.R, r);
          frame_b.R = frame_a.R;
/* Force and torque balance */
          zeros(3) = frame_a.f + frame_b.f;
          zeros(3) = frame_a.t + frame_b.t + cross(r, frame_b.f);
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-99, 5}, {101, -5}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Text(extent = {{-150, 85}, {150, 45}}, textString = "%name", lineColor = {0, 0, 255}), Text(extent = {{150, -50}, {-150, -20}}, lineColor = {0, 0, 0}, textString = "r=%r"), Text(extent = {{-89, 38}, {-53, 13}}, lineColor = {128, 128, 128}, textString = "a"), Text(extent = {{57, 39}, {93, 14}}, lineColor = {128, 128, 128}, textString = "b")}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, 5}, {100, -5}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Line(points = {{-95, 20}, {-58, 20}}, color = {128, 128, 128}, arrow = {Arrow.None, Arrow.Filled}), Line(points = {{-94, 18}, {-94, 50}}, color = {128, 128, 128}, arrow = {Arrow.None, Arrow.Filled}), Text(extent = {{-72, 35}, {-58, 24}}, lineColor = {128, 128, 128}, textString = "x"), Text(extent = {{-113, 57}, {-98, 45}}, lineColor = {128, 128, 128}, textString = "y"), Line(points = {{-100, -4}, {-100, -69}}, color = {128, 128, 128}), Line(points = {{-100, -63}, {90, -63}}, color = {128, 128, 128}), Text(extent = {{-22, -39}, {16, -63}}, lineColor = {128, 128, 128}, textString = "r"), Polygon(points = {{88, -59}, {88, -68}, {100, -63}, {88, -59}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Line(points = {{100, -3}, {100, -68}}, color = {128, 128, 128}), Line(points = {{69, 20}, {106, 20}}, color = {128, 128, 128}, arrow = {Arrow.None, Arrow.Filled}), Line(points = {{70, 18}, {70, 50}}, color = {128, 128, 128}, arrow = {Arrow.None, Arrow.Filled}), Text(extent = {{92, 35}, {106, 24}}, lineColor = {128, 128, 128}, textString = "x"), Text(extent = {{51, 57}, {66, 45}}, lineColor = {128, 128, 128}, textString = "y")}),
            Documentation(info = "<HTML>
<p>
Component for a <b>fixed translation</b> of frame_b with respect
to frame_a, i.e., the relationship between connectors frame_a and frame_b
remains constant and frame_a is always <b>parallel</b> to frame_b.
</p>
<p>
By default, this component is visualized by a cylinder connecting
frame_a and frame_b, as shown in the figure below. Note, that the
two visualized frames are not part of the component animation and that
the animation may be switched off via parameter animation = <b>false</b>.
</p>

<p>
<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/FixedTranslation.png\" ALT=\"Parts.FixedTranslation\">
</p>
</HTML>"));
        end FixedTranslation;

        model FixedRotation "Fixed translation followed by a fixed rotation of frame_b with respect to frame_a"
          import Modelica.Mechanics.MultiBody.Frames;
          import Modelica.SIunits.Conversions.to_unit1;
          Interfaces.Frame_a frame_a "Coordinate system fixed to the component with one cut-force and cut-torque" annotation(
            Placement(transformation(extent = {{-116, -16}, {-84, 16}}, rotation = 0)));
          Interfaces.Frame_b frame_b "Coordinate system fixed to the component with one cut-force and cut-torque" annotation(
            Placement(transformation(extent = {{84, -16}, {116, 16}}, rotation = 0)));
          parameter Boolean animation = true "= true, if animation shall be enabled";
          parameter SI.Position r[3] = {0, 0, 0} "Vector from frame_a to frame_b resolved in frame_a";
          parameter Modelica.Mechanics.MultiBody.Types.RotationTypes rotationType = Modelica.Mechanics.MultiBody.Types.RotationTypes.RotationAxis "Type of rotation description" annotation(
            Evaluate = true);
          parameter Modelica.Mechanics.MultiBody.Types.Axis n = {1, 0, 0} "Axis of rotation in frame_a (= same as in frame_b)" annotation(
            Evaluate = true,
            Dialog(group = "if rotationType = RotationAxis", enable = rotationType == Modelica.Mechanics.MultiBody.Types.RotationTypes.RotationAxis));
          parameter Cv.NonSIunits.Angle_deg angle = 0 "Angle to rotate frame_a around axis n into frame_b" annotation(
            Dialog(group = "if rotationType = RotationAxis", enable = rotationType == Modelica.Mechanics.MultiBody.Types.RotationTypes.RotationAxis));
          parameter Modelica.Mechanics.MultiBody.Types.Axis n_x = {1, 0, 0} "Vector along x-axis of frame_b resolved in frame_a" annotation(
            Evaluate = true,
            Dialog(group = "if rotationType = TwoAxesVectors", enable = rotationType == Modelica.Mechanics.MultiBody.Types.RotationTypes.TwoAxesVectors));
          parameter Modelica.Mechanics.MultiBody.Types.Axis n_y = {0, 1, 0} "Vector along y-axis of frame_b resolved in frame_a" annotation(
            Evaluate = true,
            Dialog(group = "if rotationType = TwoAxesVectors", enable = rotationType == Modelica.Mechanics.MultiBody.Types.RotationTypes.TwoAxesVectors));
          parameter Modelica.Mechanics.MultiBody.Types.RotationSequence sequence(min = {1, 1, 1}, max = {3, 3, 3}) = {1, 2, 3} "Sequence of rotations" annotation(
            Evaluate = true,
            Dialog(group = "if rotationType = PlanarRotationSequence", enable = rotationType == Modelica.Mechanics.MultiBody.Types.RotationTypes.PlanarRotationSequence));
          parameter Cv.NonSIunits.Angle_deg angles[3] = {0, 0, 0} "Rotation angles around the axes defined in 'sequence'" annotation(
            Dialog(group = "if rotationType = PlanarRotationSequence", enable = rotationType == Modelica.Mechanics.MultiBody.Types.RotationTypes.PlanarRotationSequence));
          parameter Modelica.Mechanics.MultiBody.Types.ShapeType shapeType = "cylinder" "Type of shape" annotation(
            Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter SI.Position r_shape[3] = {0, 0, 0} "Vector from frame_a to shape origin, resolved in frame_a" annotation(
            Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter Modelica.Mechanics.MultiBody.Types.Axis lengthDirection = to_unit1(r - r_shape) "Vector in length direction of shape, resolved in frame_a" annotation(
            Evaluate = true,
            Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter Modelica.Mechanics.MultiBody.Types.Axis widthDirection = {0, 1, 0} "Vector in width direction of shape, resolved in frame_a" annotation(
            Evaluate = true,
            Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter SI.Length length = Modelica.Math.Vectors.length(r - r_shape) "Length of shape" annotation(
            Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter SI.Distance width = length / world.defaultWidthFraction "Width of shape" annotation(
            Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter SI.Distance height = width "Height of shape" annotation(
            Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter Modelica.Mechanics.MultiBody.Types.ShapeExtra extra = 0.0 "Additional parameter depending on shapeType (see docu of Visualizers.Advanced.Shape)" annotation(
            Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          /*
            parameter Boolean checkTotalPower=false
              "= true, if total power flowing into this component shall be determined (must be zero)"
              annotation (Dialog(tab="Advanced"));
          */
          input Modelica.Mechanics.MultiBody.Types.Color color = Modelica.Mechanics.MultiBody.Types.Defaults.RodColor "Color of shape" annotation(
            Dialog(colorSelector = true, tab = "Animation", group = "if animation = true", enable = animation));
          input Modelica.Mechanics.MultiBody.Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(
            Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          final parameter Frames.Orientation R_rel = if rotationType == Types.RotationTypes.RotationAxis then Frames.planarRotation(Modelica.Math.Vectors.normalizeWithAssert(n), Cv.from_deg(angle), 0) else if rotationType == Types.RotationTypes.TwoAxesVectors then Frames.from_nxy(n_x, n_y) else Frames.axesRotations(sequence, Cv.from_deg(angles), zeros(3)) "Fixed rotation object from frame_a to frame_b";
          /*
            SI.Power totalPower
              "Total power flowing into this element, if checkTotalPower=true (otherwise dummy)";
          */
        protected
          outer Modelica.Mechanics.MultiBody.World world;
          /*
            parameter Frames.Orientation R_rel_inv=
                Frames.inverseRotation(R_rel)
          */
          parameter Frames.Orientation R_rel_inv = Frames.from_T(transpose(R_rel.T), zeros(3)) "Inverse of R_rel (rotate from frame_b to frame_a)";
          Modelica.Mechanics.MultiBody.Visualizers.Advanced.Shape shape(shapeType = shapeType, color = color, specularCoefficient = specularCoefficient, r_shape = r_shape, lengthDirection = lengthDirection, widthDirection = widthDirection, length = length, width = width, height = height, extra = extra, r = frame_a.r_0, R = frame_a.R) if world.enableAnimation and animation;
        equation
          Connections.branch(frame_a.R, frame_b.R);
          assert(cardinality(frame_a) > 0 or cardinality(frame_b) > 0, "Neither connector frame_a nor frame_b of FixedRotation object is connected");
/* Relationships between quantities of frame_a and frame_b */
          frame_b.r_0 = frame_a.r_0 + Frames.resolve1(frame_a.R, r);
          if Connections.rooted(frame_a.R) then
            frame_b.R = Frames.absoluteRotation(frame_a.R, R_rel);
            zeros(3) = frame_a.f + Frames.resolve1(R_rel, frame_b.f);
            zeros(3) = frame_a.t + Frames.resolve1(R_rel, frame_b.t) - cross(r, frame_a.f);
          else
            frame_a.R = Frames.absoluteRotation(frame_b.R, R_rel_inv);
            zeros(3) = frame_b.f + Frames.resolve1(R_rel_inv, frame_a.f);
            zeros(3) = frame_b.t + Frames.resolve1(R_rel_inv, frame_a.t) + cross(Frames.resolve1(R_rel_inv, r), frame_b.f);
          end if;
/*
  if checkTotalPower then
    totalPower = frame_a.f*Frames.resolve2(frame_a.R, der(frame_a.r_0)) +
                 frame_b.f*Frames.resolve2(frame_b.R, der(frame_b.r_0)) +
                 frame_a.t*Frames.angularVelocity2(frame_a.R) +
                 frame_b.t*Frames.angularVelocity2(frame_b.R);
  else
    totalPower = 0;
  end if;
*/
          annotation(
            Documentation(info = "<HTML>
<p>
Component for a <b>fixed translation</b> and <b>fixed rotation</b> of frame_b with respect
to frame_a, i.e., the relationship between connectors frame_a and frame_b
remains constant. There are several possibilities to define the
orientation of frame_b with respect to frame_a:
</p>
<ul>
<li><b>Planar rotation</b> along axis 'n' (that is fixed and resolved
    in frame_a) with a fixed angle 'angle'.</li>
<li><b>Vectors n_x</b> and <b>n_y</b> that are directed along the corresponding axes
    direction of frame_b and are resolved in frame_a (if n_y is not
    orthogonal to n_x, the y-axis of frame_b is selected such that it is
    orthogonal to n_x and in the plane of n_x and n_y).</li>
<li><b>Sequence</b> of <b>three planar axes rotations</b>.
    For example, \"sequence = {1,2,3}\" and \"angles = {90, 45, -90}\"
    means to rotate frame_a around the x axis with 90 degrees, around the new
    y axis with 45 degrees and around the new z axis around -90 degrees to
    arrive at frame_b. Note, that sequence={1,2,3}
    is the Cardan angle sequence and sequence = {3,1,3} is the Euler angle
    sequence.</li>
</ul>
<p>
By default, this component is visualized by a cylinder connecting
frame_a and frame_b, as shown in the figure below. In this figure
frame_b is rotated along the z-axis of frame_a with 60 degree. Note, that the
two visualized frames are not part of the component animation and that
the animation may be switched off via parameter animation = <b>false</b>.
</p>

<p>
<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/FixedRotation.png\" ALT=\"Parts.FixedRotation\">
</p>
</HTML>", revisions = "<HTML><p><b>Release Notes:</b></p>
<ul>
  <li><i>July 28, 2003</i><br>
         Bug fixed: if rotationType = PlanarRotationSequence, then 'angles'
         was used with unit [rad] instead of [deg].</li>
</ul>
</HTML>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-150, 80}, {150, 120}}, textString = "%name", lineColor = {0, 0, 255}), Rectangle(extent = {{-100, 5}, {100, -4}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Line(points = {{80, 20}, {129, 50}}, color = {0, 0, 0}), Line(points = {{80, 20}, {57, 59}}, color = {0, 0, 0}), Polygon(points = {{144, 60}, {117, 59}, {132, 37}, {144, 60}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Polygon(points = {{43, 80}, {46, 50}, {68, 65}, {43, 80}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Text(extent = {{-150, -50}, {150, -80}}, lineColor = {0, 0, 0}, textString = "r=%r"), Text(extent = {{-117, 51}, {-81, 26}}, lineColor = {128, 128, 128}, textString = "a"), Text(extent = {{84, -24}, {120, -49}}, lineColor = {128, 128, 128}, textString = "b")}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-100, -1}, {-100, -66}}, color = {128, 128, 128}), Line(points = {{100, 0}, {100, -65}}, color = {128, 128, 128}), Line(points = {{-100, -60}, {89, -60}}, color = {128, 128, 128}), Text(extent = {{-22, -36}, {16, -60}}, lineColor = {128, 128, 128}, textString = "r"), Rectangle(extent = {{-100, 5}, {100, -5}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Line(points = {{69, 29}, {97, 45}}, color = {128, 128, 128}, arrow = {Arrow.None, Arrow.Filled}), Line(points = {{70, 27}, {55, 54}}, color = {128, 128, 128}, arrow = {Arrow.None, Arrow.Filled}), Text(extent = {{95, 42}, {109, 31}}, lineColor = {128, 128, 128}, textString = "x"), Text(extent = {{42, 70}, {57, 58}}, lineColor = {128, 128, 128}, textString = "y"), Line(points = {{-95, 22}, {-58, 22}}, color = {128, 128, 128}, arrow = {Arrow.None, Arrow.Filled}), Line(points = {{-94, 20}, {-94, 52}}, color = {128, 128, 128}, arrow = {Arrow.None, Arrow.Filled}), Text(extent = {{-72, 37}, {-58, 26}}, lineColor = {128, 128, 128}, textString = "x"), Text(extent = {{-113, 59}, {-98, 47}}, lineColor = {128, 128, 128}, textString = "y"), Polygon(points = {{88, -56}, {88, -65}, {100, -60}, {88, -56}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid)}));
        end FixedRotation;

        model Body "Rigid body with mass, inertia tensor and one frame connector (12 potential states)"
          import Modelica.Mechanics.MultiBody.Types;
          import Modelica.Mechanics.MultiBody.Frames;
          import Modelica.SIunits.Conversions.to_unit1;
          Modelica.Mechanics.MultiBody.Interfaces.Frame_a frame_a "Coordinate system fixed at body" annotation(
            Placement(transformation(extent = {{-116, -16}, {-84, 16}}, rotation = 0)));
          parameter Boolean animation = true "= true, if animation shall be enabled (show cylinder and sphere)";
          parameter SI.Position r_CM[3](start = {0, 0, 0}) "Vector from frame_a to center of mass, resolved in frame_a";
          parameter SI.Mass m(min = 0, start = 1) "Mass of rigid body";
          parameter SI.Inertia I_11(min = 0) = 0.001 "(1,1) element of inertia tensor" annotation(
            Dialog(group = "Inertia tensor (resolved in center of mass, parallel to frame_a)"));
          parameter SI.Inertia I_22(min = 0) = 0.001 "(2,2) element of inertia tensor" annotation(
            Dialog(group = "Inertia tensor (resolved in center of mass, parallel to frame_a)"));
          parameter SI.Inertia I_33(min = 0) = 0.001 "(3,3) element of inertia tensor" annotation(
            Dialog(group = "Inertia tensor (resolved in center of mass, parallel to frame_a)"));
          parameter SI.Inertia I_21(min = -C.inf) = 0 "(2,1) element of inertia tensor" annotation(
            Dialog(group = "Inertia tensor (resolved in center of mass, parallel to frame_a)"));
          parameter SI.Inertia I_31(min = -C.inf) = 0 "(3,1) element of inertia tensor" annotation(
            Dialog(group = "Inertia tensor (resolved in center of mass, parallel to frame_a)"));
          parameter SI.Inertia I_32(min = -C.inf) = 0 "(3,2) element of inertia tensor" annotation(
            Dialog(group = "Inertia tensor (resolved in center of mass, parallel to frame_a)"));
          SI.Position r_0[3](start = {0, 0, 0}, each stateSelect = if enforceStates then StateSelect.always else StateSelect.avoid) "Position vector from origin of world frame to origin of frame_a" annotation(
            Dialog(tab = "Initialization", showStartAttribute = true));
          SI.Velocity v_0[3](start = {0, 0, 0}, each stateSelect = if enforceStates then StateSelect.always else StateSelect.avoid) "Absolute velocity of frame_a, resolved in world frame (= der(r_0))" annotation(
            Dialog(tab = "Initialization", showStartAttribute = true));
          SI.Acceleration a_0[3](start = {0, 0, 0}) "Absolute acceleration of frame_a resolved in world frame (= der(v_0))" annotation(
            Dialog(tab = "Initialization", showStartAttribute = true));
          parameter Boolean angles_fixed = false "= true, if angles_start are used as initial values, else as guess values" annotation(
            Evaluate = true,
            choices(checkBox = true),
            Dialog(tab = "Initialization"));
          parameter SI.Angle angles_start[3] = {0, 0, 0} "Initial values of angles to rotate frame_a around 'sequence_start' axes into frame_b" annotation(
            Dialog(tab = "Initialization"));
          parameter Types.RotationSequence sequence_start = {1, 2, 3} "Sequence of rotations to rotate frame_a into frame_b at initial time" annotation(
            Evaluate = true,
            Dialog(tab = "Initialization"));
          parameter Boolean w_0_fixed = false "= true, if w_0_start are used as initial values, else as guess values" annotation(
            Evaluate = true,
            choices(checkBox = true),
            Dialog(tab = "Initialization"));
          parameter SI.AngularVelocity w_0_start[3] = {0, 0, 0} "Initial or guess values of angular velocity of frame_a resolved in world frame" annotation(
            Dialog(tab = "Initialization"));
          parameter Boolean z_0_fixed = false "= true, if z_0_start are used as initial values, else as guess values" annotation(
            Evaluate = true,
            choices(checkBox = true),
            Dialog(tab = "Initialization"));
          parameter SI.AngularAcceleration z_0_start[3] = {0, 0, 0} "Initial values of angular acceleration z_0 = der(w_0)" annotation(
            Dialog(tab = "Initialization"));
          parameter SI.Diameter sphereDiameter = world.defaultBodyDiameter "Diameter of sphere" annotation(
            Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          input Types.Color sphereColor = Modelica.Mechanics.MultiBody.Types.Defaults.BodyColor "Color of sphere" annotation(
            Dialog(colorSelector = true, tab = "Animation", group = "if animation = true", enable = animation));
          parameter SI.Diameter cylinderDiameter = sphereDiameter / Types.Defaults.BodyCylinderDiameterFraction "Diameter of cylinder" annotation(
            Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          input Types.Color cylinderColor = sphereColor "Color of cylinder" annotation(
            Dialog(colorSelector = true, tab = "Animation", group = "if animation = true", enable = animation));
          input Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(
            Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter Boolean enforceStates = false "= true, if absolute variables of body object shall be used as states (StateSelect.always)" annotation(
            Evaluate = true,
            Dialog(tab = "Advanced"));
          parameter Boolean useQuaternions = true "= true, if quaternions shall be used as potential states otherwise use 3 angles as potential states" annotation(
            Evaluate = true,
            Dialog(tab = "Advanced"));
          parameter Types.RotationSequence sequence_angleStates = {1, 2, 3} "Sequence of rotations to rotate world frame into frame_a around the 3 angles used as potential states" annotation(
            Evaluate = true,
            Dialog(tab = "Advanced", enable = not useQuaternions));
          final parameter SI.Inertia I[3, 3] = [I_11, I_21, I_31; I_21, I_22, I_32; I_31, I_32, I_33] "inertia tensor";
          final parameter Frames.Orientation R_start = Modelica.Mechanics.MultiBody.Frames.axesRotations(sequence_start, angles_start, zeros(3)) "Orientation object from world frame to frame_a at initial time";
          final parameter SI.AngularAcceleration z_a_start[3] = Frames.resolve2(R_start, z_0_start) "Initial values of angular acceleration z_a = der(w_a), i.e., time derivative of angular velocity resolved in frame_a";
          SI.AngularVelocity w_a[3](start = Frames.resolve2(R_start, w_0_start), fixed = fill(w_0_fixed, 3), each stateSelect = if enforceStates then if useQuaternions then StateSelect.always else StateSelect.never else StateSelect.avoid) "Absolute angular velocity of frame_a resolved in frame_a";
          SI.AngularAcceleration z_a[3](start = Frames.resolve2(R_start, z_0_start), fixed = fill(z_0_fixed, 3)) "Absolute angular acceleration of frame_a resolved in frame_a";
          SI.Acceleration g_0[3] "Gravity acceleration resolved in world frame";
        protected
          outer Modelica.Mechanics.MultiBody.World world;
          // Declarations for quaternions (dummies, if quaternions are not used)
          parameter Frames.Quaternions.Orientation Q_start = Frames.to_Q(R_start) "Quaternion orientation object from world frame to frame_a at initial time";
          Frames.Quaternions.Orientation Q(start = Q_start, each stateSelect = if enforceStates then if useQuaternions then StateSelect.prefer else StateSelect.never else StateSelect.avoid) "Quaternion orientation object from world frame to frame_a (dummy value, if quaternions are not used as states)";
          // Declaration for 3 angles
          parameter SI.Angle phi_start[3] = if sequence_start[1] == sequence_angleStates[1] and sequence_start[2] == sequence_angleStates[2] and sequence_start[3] == sequence_angleStates[3] then angles_start else Frames.axesRotationsAngles(R_start, sequence_angleStates) "Potential angle states at initial time";
          SI.Angle phi[3](start = phi_start, each stateSelect = if enforceStates then if useQuaternions then StateSelect.never else StateSelect.always else StateSelect.avoid) "Dummy or 3 angles to rotate world frame into frame_a of body";
          SI.AngularVelocity phi_d[3](each stateSelect = if enforceStates then if useQuaternions then StateSelect.never else StateSelect.always else StateSelect.avoid) "= der(phi)";
          SI.AngularAcceleration phi_dd[3] "= der(phi_d)";
          // Declarations for animation
          Visualizers.Advanced.Shape cylinder(shapeType = "cylinder", color = cylinderColor, specularCoefficient = specularCoefficient, length = if Modelica.Math.Vectors.length(r_CM) > sphereDiameter / 2 then Modelica.Math.Vectors.length(r_CM) - (if cylinderDiameter > 1.1 * sphereDiameter then sphereDiameter / 2 else 0) else 0, width = cylinderDiameter, height = cylinderDiameter, lengthDirection = to_unit1(r_CM), widthDirection = {0, 1, 0}, r = frame_a.r_0, R = frame_a.R) if world.enableAnimation and animation;
          Visualizers.Advanced.Shape sphere(shapeType = "sphere", color = sphereColor, specularCoefficient = specularCoefficient, length = sphereDiameter, width = sphereDiameter, height = sphereDiameter, lengthDirection = {1, 0, 0}, widthDirection = {0, 1, 0}, r_shape = r_CM - {1, 0, 0} * sphereDiameter / 2, r = frame_a.r_0, R = frame_a.R) if world.enableAnimation and animation and sphereDiameter > 0;
        initial equation
          if angles_fixed then
// Initialize positional variables
            if not Connections.isRoot(frame_a.R) then
// frame_a.R is computed somewhere else
              zeros(3) = Frames.Orientation.equalityConstraint(frame_a.R, R_start);
            elseif useQuaternions then
// frame_a.R is computed from quaternions Q
              zeros(3) = Frames.Quaternions.Orientation.equalityConstraint(Q, Q_start);
            else
// frame_a.R is computed from the 3 angles 'phi'
              phi = phi_start;
            end if;
          end if;
        equation
          if enforceStates then
            Connections.root(frame_a.R);
          else
            Connections.potentialRoot(frame_a.R);
          end if;
          r_0 = frame_a.r_0;
          if not Connections.isRoot(frame_a.R) then
// Body does not have states
// Dummies
            Q = {0, 0, 0, 1};
            phi = zeros(3);
            phi_d = zeros(3);
            phi_dd = zeros(3);
          elseif useQuaternions then
// Use Quaternions as states (with dynamic state selection)
            frame_a.R = Frames.from_Q(Q, Frames.Quaternions.angularVelocity2(Q, der(Q)));
            {0} = Frames.Quaternions.orientationConstraint(Q);
// Dummies
            phi = zeros(3);
            phi_d = zeros(3);
            phi_dd = zeros(3);
          else
// Use Cardan angles as states
            phi_d = der(phi);
            phi_dd = der(phi_d);
            frame_a.R = Frames.axesRotations(sequence_angleStates, phi, phi_d);
// Dummies
            Q = {0, 0, 0, 1};
          end if;
// gravity acceleration at center of mass resolved in world frame
          g_0 = world.gravityAcceleration(frame_a.r_0 + Frames.resolve1(frame_a.R, r_CM));
// translational kinematic differential equations
          v_0 = der(frame_a.r_0);
          a_0 = der(v_0);
// rotational kinematic differential equations
          w_a = Frames.angularVelocity2(frame_a.R);
          z_a = der(w_a);
/* Newton/Euler equations with respect to center of mass
            a_CM = a_a + cross(z_a, r_CM) + cross(w_a, cross(w_a, r_CM));
            f_CM = m*(a_CM - g_a);
            t_CM = I*z_a + cross(w_a, I*w_a);
       frame_a.f = f_CM
       frame_a.t = t_CM + cross(r_CM, f_CM);
    Inserting the first three equations in the last two results in:
  */
          frame_a.f = m * (Frames.resolve2(frame_a.R, a_0 - g_0) + cross(z_a, r_CM) + cross(w_a, cross(w_a, r_CM)));
          frame_a.t = I * z_a + cross(w_a, I * w_a) + cross(r_CM, frame_a.f);
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, 30}, {-3, -31}}, lineColor = {0, 24, 48}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {0, 127, 255}, radius = 10), Text(extent = {{150, -100}, {-150, -70}}, lineColor = {0, 0, 0}, textString = "m=%m"), Text(extent = {{-150, 110}, {150, 70}}, textString = "%name", lineColor = {0, 0, 255}), Ellipse(extent = {{-20, 60}, {100, -60}}, lineColor = {0, 24, 48}, fillPattern = FillPattern.Sphere, fillColor = {0, 127, 255})}),
            Documentation(info = "<HTML>
<p>
<b>Rigid body</b> with mass and inertia tensor.
All parameter vectors have to be resolved in frame_a.
The <b>inertia tensor</b> has to be defined with respect to a
coordinate system that is parallel to frame_a with the
origin at the center of mass of the body.
</p>
<p>
By default, this component is visualized by a <b>cylinder</b> located
between frame_a and the center of mass and by a <b>sphere</b> that has
its center at the center of mass. If the cylinder length is smaller as
the radius of the sphere, e.g., since frame_a is located at the
center of mass, the cylinder is not displayed. Note, that
the animation may be switched off via parameter animation = <b>false</b>.
</p>
<p>
<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Body.png\" ALT=\"Parts.Body\">
</p>

<p>
<b>States of Body Components</b>
</p>
<p>
Every body has potential states. If possible a tool will select
the states of joints and not the states of bodies because this is
usually the most efficient choice. In this case the position, orientation,
velocity and angular velocity of frame_a of the body will be computed
by the component that is connected to frame_a. However, if a body is moving
freely in space, variables of the body have to be used as states. The potential
states of the body are:
</p>
<ul>
<li> The <b>position vector</b> frame_a.r_0 from the origin of the
     world frame to the origin of frame_a of the body, resolved in
     the world frame and the <b>absolute velocity</b> v_0 of the origin of
     frame_a, resolved in the world frame (= der(frame_a.r_0)).
</li>
<li> If parameter <b>useQuaternions</b> in the \"Advanced\" menu
     is <b>true</b> (this is the default), then <b>4 quaternions</b>
     are potential states. Additionally, the coordinates of the
     absolute angular velocity vector of the
     body are 3 potential states.<br>
     If <b>useQuaternions</b> in the \"Advanced\" menu
     is <b>false</b>, then <b>3 angles</b> and the derivatives of
     these angles are potential states. The orientation of frame_a
     is computed by rotating the world frame along the axes defined
     in parameter vector \"sequence_angleStates\" (default = {1,2,3}, i.e.,
     the Cardan angle sequence) around the angles used as potential states.
     For example, the default is to rotate the x-axis of the world frame
     around angles[1], the new y-axis around angles[2] and the new z-axis
     around angles[3], arriving at frame_a.
 </li>
</ul>
<p>
The quaternions have the slight disadvantage that there is a
non-linear constraint equation between the 4 quaternions.
Therefore, at least one non-linear equation has to be solved
during simulation. A tool might, however, analytically solve this
simple constraint equation. Using the 3 angles as states has the
disadvantage that there is a singular configuration in which a
division by zero will occur. If it is possible to determine in advance
for an application class that this singular configuration is outside
of the operating region, the 3 angles might be used as potential
states by setting <b>useQuaternions</b> = <b>false</b>.
</p>
<p>
In text books about 3-dimensional mechanics often 3 angles and the
angular velocity are used as states. This is not the case here, since
3 angles and their derivatives are used as potential states
(if useQuaternions = false). The reason
is that for real-time simulation the discretization formula of the
integrator might be \"inlined\" and solved together with the body equations.
By appropriate symbolic transformation the performance is
drastically increased if angles and their
derivatives are used as states, instead of angles and the angular
velocity.
</p>
<p>
Whether or not variables of the body are used as states is usually
automatically selected by the Modelica translator. If parameter
<b>enforceStates</b> is set to <b>true</b> in the \"Advanced\" menu,
then body variables are forced to be used as states according
to the setting of parameters \"useQuaternions\" and
\"sequence_angleStates\".
</p>
</HTML>"));
        end Body;

        model BodyShape "Rigid body with mass, inertia tensor, different shapes for animation, and two frame connectors (12 potential states)"
          import Modelica.Mechanics.MultiBody.Types;
          import Modelica.SIunits.Conversions.to_unit1;
          Interfaces.Frame_a frame_a "Coordinate system fixed to the component with one cut-force and cut-torque" annotation(
            Placement(transformation(extent = {{-116, -16}, {-84, 16}}, rotation = 0)));
          Interfaces.Frame_b frame_b "Coordinate system fixed to the component with one cut-force and cut-torque" annotation(
            Placement(transformation(extent = {{84, -16}, {116, 16}}, rotation = 0)));
          parameter Boolean animation = true "= true, if animation shall be enabled (show shape between frame_a and frame_b and optionally a sphere at the center of mass)";
          parameter Boolean animateSphere = true "= true, if mass shall be animated as sphere provided animation=true";
          parameter SI.Position r[3](start = {0, 0, 0}) "Vector from frame_a to frame_b resolved in frame_a";
          parameter SI.Position r_CM[3](start = {0, 0, 0}) "Vector from frame_a to center of mass, resolved in frame_a";
          parameter SI.Mass m(min = 0, start = 1) "Mass of rigid body";
          parameter SI.Inertia I_11(min = 0) = 0.001 "(1,1) element of inertia tensor" annotation(
            Dialog(group = "Inertia tensor (resolved in center of mass, parallel to frame_a)"));
          parameter SI.Inertia I_22(min = 0) = 0.001 "(2,2) element of inertia tensor" annotation(
            Dialog(group = "Inertia tensor (resolved in center of mass, parallel to frame_a)"));
          parameter SI.Inertia I_33(min = 0) = 0.001 "(3,3) element of inertia tensor" annotation(
            Dialog(group = "Inertia tensor (resolved in center of mass, parallel to frame_a)"));
          parameter SI.Inertia I_21(min = -C.inf) = 0 "(2,1) element of inertia tensor" annotation(
            Dialog(group = "Inertia tensor (resolved in center of mass, parallel to frame_a)"));
          parameter SI.Inertia I_31(min = -C.inf) = 0 "(3,1) element of inertia tensor" annotation(
            Dialog(group = "Inertia tensor (resolved in center of mass, parallel to frame_a)"));
          parameter SI.Inertia I_32(min = -C.inf) = 0 "(3,2) element of inertia tensor" annotation(
            Dialog(group = "Inertia tensor (resolved in center of mass, parallel to frame_a)"));
          SI.Position r_0[3](start = {0, 0, 0}, each stateSelect = if enforceStates then StateSelect.always else StateSelect.avoid) "Position vector from origin of world frame to origin of frame_a" annotation(
            Dialog(tab = "Initialization", showStartAttribute = true));
          SI.Velocity v_0[3](start = {0, 0, 0}, each stateSelect = if enforceStates then StateSelect.always else StateSelect.avoid) "Absolute velocity of frame_a, resolved in world frame (= der(r_0))" annotation(
            Dialog(tab = "Initialization", showStartAttribute = true));
          SI.Acceleration a_0[3](start = {0, 0, 0}) "Absolute acceleration of frame_a resolved in world frame (= der(v_0))" annotation(
            Dialog(tab = "Initialization", showStartAttribute = true));
          parameter Boolean angles_fixed = false "= true, if angles_start are used as initial values, else as guess values" annotation(
            Evaluate = true,
            choices(checkBox = true),
            Dialog(tab = "Initialization"));
          parameter SI.Angle angles_start[3] = {0, 0, 0} "Initial values of angles to rotate frame_a around 'sequence_start' axes into frame_b" annotation(
            Dialog(tab = "Initialization"));
          parameter Types.RotationSequence sequence_start = {1, 2, 3} "Sequence of rotations to rotate frame_a into frame_b at initial time" annotation(
            Evaluate = true,
            Dialog(tab = "Initialization"));
          parameter Boolean w_0_fixed = false "= true, if w_0_start are used as initial values, else as guess values" annotation(
            Evaluate = true,
            choices(checkBox = true),
            Dialog(tab = "Initialization"));
          parameter SI.AngularVelocity w_0_start[3] = {0, 0, 0} "Initial or guess values of angular velocity of frame_a resolved in world frame" annotation(
            Dialog(tab = "Initialization"));
          parameter Boolean z_0_fixed = false "= true, if z_0_start are used as initial values, else as guess values" annotation(
            Evaluate = true,
            choices(checkBox = true),
            Dialog(tab = "Initialization"));
          parameter SI.AngularAcceleration z_0_start[3] = {0, 0, 0} "Initial values of angular acceleration z_0 = der(w_0)" annotation(
            Dialog(tab = "Initialization"));
          parameter Types.ShapeType shapeType = "cylinder" "Type of shape" annotation(
            Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter SI.Position r_shape[3] = {0, 0, 0} "Vector from frame_a to shape origin, resolved in frame_a" annotation(
            Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter Types.Axis lengthDirection = to_unit1(r - r_shape) "Vector in length direction of shape, resolved in frame_a" annotation(
            Evaluate = true,
            Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter Types.Axis widthDirection = {0, 1, 0} "Vector in width direction of shape, resolved in frame_a" annotation(
            Evaluate = true,
            Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter SI.Length length = Modelica.Math.Vectors.length(r - r_shape) "Length of shape" annotation(
            Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter SI.Distance width = length / world.defaultWidthFraction "Width of shape" annotation(
            Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter SI.Distance height = width "Height of shape" annotation(
            Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter Types.ShapeExtra extra = 0.0 "Additional parameter depending on shapeType (see docu of Visualizers.Advanced.Shape)" annotation(
            Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          input Types.Color color = Modelica.Mechanics.MultiBody.Types.Defaults.BodyColor "Color of shape" annotation(
            Dialog(colorSelector = true, tab = "Animation", group = "if animation = true", enable = animation));
          parameter SI.Diameter sphereDiameter = 2 * width "Diameter of sphere" annotation(
            Dialog(tab = "Animation", group = "if animation = true and animateSphere = true", enable = animation and animateSphere));
          input Types.Color sphereColor = color "Color of sphere of mass" annotation(
            Dialog(colorSelector = true, tab = "Animation", group = "if animation = true and animateSphere = true", enable = animation and animateSphere));
          input Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(
            Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter Boolean enforceStates = false "= true, if absolute variables of body object shall be used as states (StateSelect.always)" annotation(
            Dialog(tab = "Advanced"));
          parameter Boolean useQuaternions = true "= true, if quaternions shall be used as potential states otherwise use 3 angles as potential states" annotation(
            Dialog(tab = "Advanced"));
          parameter Types.RotationSequence sequence_angleStates = {1, 2, 3} "Sequence of rotations to rotate world frame into frame_a around the 3 angles used as potential states" annotation(
            Evaluate = true,
            Dialog(tab = "Advanced", enable = not useQuaternions));
          FixedTranslation frameTranslation(r = r, animation = false) annotation(
            Placement(transformation(extent = {{-20, -20}, {20, 20}}, rotation = 0)));
          Body body(r_CM = r_CM, m = m, I_11 = I_11, I_22 = I_22, I_33 = I_33, I_21 = I_21, I_31 = I_31, I_32 = I_32, animation = false, sequence_start = sequence_start, angles_fixed = angles_fixed, angles_start = angles_start, w_0_fixed = w_0_fixed, w_0_start = w_0_start, z_0_fixed = z_0_fixed, z_0_start = z_0_start, useQuaternions = useQuaternions, sequence_angleStates = sequence_angleStates, enforceStates = false) annotation(
            Placement(transformation(extent = {{-27.3333, -70.3333}, {13, -30}}, rotation = 0)));
        protected
          outer Modelica.Mechanics.MultiBody.World world;
          Visualizers.Advanced.Shape shape1(shapeType = shapeType, color = color, specularCoefficient = specularCoefficient, length = length, width = width, height = height, lengthDirection = lengthDirection, widthDirection = widthDirection, r_shape = r_shape, extra = extra, r = frame_a.r_0, R = frame_a.R) if world.enableAnimation and animation;
          Visualizers.Advanced.Shape shape2(shapeType = "sphere", color = sphereColor, specularCoefficient = specularCoefficient, length = sphereDiameter, width = sphereDiameter, height = sphereDiameter, lengthDirection = {1, 0, 0}, widthDirection = {0, 1, 0}, r_shape = r_CM - {1, 0, 0} * sphereDiameter / 2, r = frame_a.r_0, R = frame_a.R) if world.enableAnimation and animation and animateSphere;
        equation
          r_0 = frame_a.r_0;
          v_0 = der(r_0);
          a_0 = der(v_0);
          connect(frame_a, frameTranslation.frame_a) annotation(
            Line(points = {{-100, 0}, {-20, 0}}, color = {95, 95, 95}, thickness = 0.5));
          connect(frame_b, frameTranslation.frame_b) annotation(
            Line(points = {{100, 0}, {20, 0}}, color = {95, 95, 95}, thickness = 0.5));
          connect(frame_a, body.frame_a) annotation(
            Line(points = {{-100, 0}, {-60, 0}, {-60, -50.1666}, {-27.3333, -50.1666}}, color = {95, 95, 95}, thickness = 0.5));
          annotation(
            Documentation(info = "<HTML>
<p>
<b>Rigid body</b> with mass and inertia tensor and <b>two frame connectors</b>.
All parameter vectors have to be resolved in frame_a.
The <b>inertia tensor</b> has to be defined with respect to a
coordinate system that is parallel to frame_a with the
origin at the center of mass of the body. The coordinate system <b>frame_b</b>
is always parallel to <b>frame_a</b>.
</p>
<p>
By default, this component is visualized by any <b>shape</b> that can be
defined with Modelica.Mechanics.MultiBody.Visualizers.FixedShape. This shape is placed
between frame_a and frame_b (default: length(shape) = Frames.length(r)).
Additionally a <b>sphere</b> may be visualized that has
its center at the center of mass.
Note, that
the animation may be switched off via parameter animation = <b>false</b>.
</p>
<p>
<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/BodyShape.png\" ALT=\"Parts.BodyShape\">
</p>

<p>
The following shapes can be defined via parameter <b>shapeType</b>,
e.g., shapeType=\"cone\":
</p>
<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/FixedShape.png\" ALT=\"Visualizers.FixedShape\">
<p>
A BodyShape component has potential states. For details of these
states and of the \"Advanced\" menu parameters, see model
<a href=\"modelica://Modelica.Mechanics.MultiBody.Parts.Body\">MultiBody.Parts.Body</a>.
</p>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-150, 110}, {150, 70}}, textString = "%name", lineColor = {0, 0, 255}), Text(extent = {{-150, -100}, {150, -70}}, lineColor = {0, 0, 0}, textString = "r=%r"), Rectangle(extent = {{-100, 31}, {101, -30}}, lineColor = {0, 24, 48}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {0, 127, 255}, radius = 10), Ellipse(extent = {{-60, 60}, {60, -60}}, lineColor = {0, 24, 48}, fillPattern = FillPattern.Sphere, fillColor = {0, 127, 255}), Text(extent = {{-50, 24}, {55, -27}}, lineColor = {0, 0, 0}, textString = "%m"), Text(extent = {{55, 12}, {91, -13}}, lineColor = {0, 0, 0}, textString = "b"), Text(extent = {{-92, 13}, {-56, -12}}, lineColor = {0, 0, 0}, textString = "a")}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-100, 9}, {-100, 43}}, color = {128, 128, 128}), Line(points = {{100, 0}, {100, 44}}, color = {128, 128, 128}), Line(points = {{-100, 40}, {90, 40}}, color = {135, 135, 135}), Polygon(points = {{90, 44}, {90, 36}, {100, 40}, {90, 44}}, lineColor = {128, 128, 128}, fillColor = {128, 128, 128}, fillPattern = FillPattern.Solid), Text(extent = {{-22, 68}, {20, 40}}, lineColor = {128, 128, 128}, textString = "r"), Line(points = {{-100, -10}, {-100, -90}}, color = {128, 128, 128}), Line(points = {{-100, -84}, {-10, -84}}, color = {128, 128, 128}), Polygon(points = {{-10, -80}, {-10, -88}, {0, -84}, {-10, -80}}, lineColor = {128, 128, 128}, fillColor = {128, 128, 128}, fillPattern = FillPattern.Solid), Text(extent = {{-82, -66}, {-56, -84}}, lineColor = {128, 128, 128}, textString = "r_CM"), Line(points = {{0, -46}, {0, -90}}, color = {128, 128, 128})}));
        end BodyShape;

        model BodyBox "Rigid body with box shape. Mass and animation properties are computed from box data and density (12 potential states)"
          import Modelica.Mechanics.MultiBody.Types;
          import Modelica.Math.Vectors.normalizeWithAssert;
          import Modelica.SIunits.Conversions.to_unit1;
          Interfaces.Frame_a frame_a "Coordinate system fixed to the component with one cut-force and cut-torque" annotation(
            Placement(transformation(extent = {{-116, -16}, {-84, 16}}, rotation = 0)));
          Interfaces.Frame_b frame_b "Coordinate system fixed to the component with one cut-force and cut-torque" annotation(
            Placement(transformation(extent = {{84, -16}, {116, 16}}, rotation = 0)));
          parameter Boolean animation = true "= true, if animation shall be enabled (show box between frame_a and frame_b)";
          parameter SI.Position r[3](start = {0.1, 0, 0}) "Vector from frame_a to frame_b resolved in frame_a";
          parameter SI.Position r_shape[3] = {0, 0, 0} "Vector from frame_a to box origin, resolved in frame_a";
          parameter Modelica.Mechanics.MultiBody.Types.Axis lengthDirection = to_unit1(r - r_shape) "Vector in length direction of box, resolved in frame_a" annotation(
            Evaluate = true);
          parameter Modelica.Mechanics.MultiBody.Types.Axis widthDirection = {0, 1, 0} "Vector in width direction of box, resolved in frame_a" annotation(
            Evaluate = true);
          parameter SI.Length length = Modelica.Math.Vectors.length(r - r_shape) "Length of box";
          parameter SI.Distance width = length / world.defaultWidthFraction "Width of box";
          parameter SI.Distance height = width "Height of box";
          parameter SI.Distance innerWidth = 0 "Width of inner box surface (0 <= innerWidth <= width)";
          parameter SI.Distance innerHeight = innerWidth "Height of inner box surface (0 <= innerHeight <= height)";
          parameter SI.Density density = 7700 "Density of cylinder (e.g., steel: 7700 .. 7900, wood : 400 .. 800)";
          input Modelica.Mechanics.MultiBody.Types.Color color = Modelica.Mechanics.MultiBody.Types.Defaults.BodyColor "Color of box" annotation(
            Dialog(colorSelector = true, enable = animation));
          input Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(
            Dialog(enable = animation));
          SI.Position r_0[3](start = {0, 0, 0}, each stateSelect = if enforceStates then StateSelect.always else StateSelect.avoid) "Position vector from origin of world frame to origin of frame_a" annotation(
            Dialog(tab = "Initialization", showStartAttribute = true));
          SI.Velocity v_0[3](start = {0, 0, 0}, each stateSelect = if enforceStates then StateSelect.always else StateSelect.avoid) "Absolute velocity of frame_a, resolved in world frame (= der(r_0))" annotation(
            Dialog(tab = "Initialization", showStartAttribute = true));
          SI.Acceleration a_0[3](start = {0, 0, 0}) "Absolute acceleration of frame_a resolved in world frame (= der(v_0))" annotation(
            Dialog(tab = "Initialization", showStartAttribute = true));
          parameter Boolean angles_fixed = false "= true, if angles_start are used as initial values, else as guess values" annotation(
            Evaluate = true,
            choices(checkBox = true),
            Dialog(tab = "Initialization"));
          parameter SI.Angle angles_start[3] = {0, 0, 0} "Initial values of angles to rotate frame_a around 'sequence_start' axes into frame_b" annotation(
            Dialog(tab = "Initialization"));
          parameter Types.RotationSequence sequence_start = {1, 2, 3} "Sequence of rotations to rotate frame_a into frame_b at initial time" annotation(
            Evaluate = true,
            Dialog(tab = "Initialization"));
          parameter Boolean w_0_fixed = false "= true, if w_0_start are used as initial values, else as guess values" annotation(
            Evaluate = true,
            choices(checkBox = true),
            Dialog(tab = "Initialization"));
          parameter SI.AngularVelocity w_0_start[3] = {0, 0, 0} "Initial or guess values of angular velocity of frame_a resolved in world frame" annotation(
            Dialog(tab = "Initialization"));
          parameter Boolean z_0_fixed = false "= true, if z_0_start are used as initial values, else as guess values" annotation(
            Evaluate = true,
            choices(checkBox = true),
            Dialog(tab = "Initialization"));
          parameter SI.AngularAcceleration z_0_start[3] = {0, 0, 0} "Initial values of angular acceleration z_0 = der(w_0)" annotation(
            Dialog(tab = "Initialization"));
          parameter Boolean enforceStates = false "= true, if absolute variables of body object shall be used as states (StateSelect.always)" annotation(
            Dialog(tab = "Advanced"));
          parameter Boolean useQuaternions = true "= true, if quaternions shall be used as potential states otherwise use 3 angles as potential states" annotation(
            Dialog(tab = "Advanced"));
          parameter Types.RotationSequence sequence_angleStates = {1, 2, 3} "Sequence of rotations to rotate world frame into frame_a around the 3 angles used as potential states" annotation(
            Evaluate = true,
            Dialog(tab = "Advanced", enable = not useQuaternions));
          final parameter SI.Mass mo(min = 0) = density * length * width * height "Mass of box without hole";
          final parameter SI.Mass mi(min = 0) = density * length * innerWidth * innerHeight "Mass of hole of box";
          final parameter SI.Mass m(min = 0) = mo - mi "Mass of box";
          final parameter Frames.Orientation R = Frames.from_nxy(r, widthDirection) "Orientation object from frame_a to coordinates system spanned by r and widthDirection";
          final parameter SI.Position r_CM[3] = r_shape + normalizeWithAssert(lengthDirection) * length / 2 "Position vector from origin of frame_a to center of mass, resolved in frame_a";
          final parameter SI.Inertia I[3, 3] = Frames.resolveDyade1(R, diagonal({mo * (width * width + height * height) - mi * (innerWidth * innerWidth + innerHeight * innerHeight), mo * (length * length + height * height) - mi * (length * length + innerHeight * innerHeight), mo * (length * length + width * width) - mi * (length * length + innerWidth * innerWidth)} / 12)) "Inertia tensor of body box with respect to center of mass, parallel to frame_a";
          Body body(animation = false, r_CM = r_CM, m = m, I_11 = I[1, 1], I_22 = I[2, 2], I_33 = I[3, 3], I_21 = I[2, 1], I_31 = I[3, 1], I_32 = I[3, 2], sequence_start = sequence_start, angles_fixed = angles_fixed, angles_start = angles_start, w_0_fixed = w_0_fixed, w_0_start = w_0_start, z_0_fixed = z_0_fixed, z_0_start = z_0_start, useQuaternions = useQuaternions, sequence_angleStates = sequence_angleStates, enforceStates = false) annotation(
            Placement(transformation(extent = {{-30, -80}, {10, -40}}, rotation = 0)));
          FixedTranslation frameTranslation(r = r, animation = animation, shapeType = "box", r_shape = r_shape, lengthDirection = lengthDirection, widthDirection = widthDirection, length = length, width = width, height = height, color = color, specularCoefficient = specularCoefficient) annotation(
            Placement(transformation(extent = {{-30, -20}, {10, 20}}, rotation = 0)));
        protected
          outer Modelica.Mechanics.MultiBody.World world;
        equation
          r_0 = frame_a.r_0;
          v_0 = der(r_0);
          a_0 = der(v_0);
          assert(innerWidth <= width, "parameter innerWidth is greater than parameter width");
          assert(innerHeight <= height, "parameter innerHeight is greater than parameter height");
          connect(frameTranslation.frame_a, frame_a) annotation(
            Line(points = {{-30, 0}, {-100, 0}}, color = {95, 95, 95}, thickness = 0.5));
          connect(frameTranslation.frame_b, frame_b) annotation(
            Line(points = {{10, 0}, {100, 0}}, color = {95, 95, 95}, thickness = 0.5));
          connect(frame_a, body.frame_a) annotation(
            Line(points = {{-100, 0}, {-70, 0}, {-70, -60}, {-30, -60}}, color = {95, 95, 95}, thickness = 0.5));
          annotation(
            Documentation(info = "<HTML>
<p>
<b>Rigid body</b> with <b>box</b> shape.
The mass properties of the body (mass, center of mass,
inertia tensor) are computed
from the box data. Optionally, the box may be hollow.
The (outer) box shape is by default used in the animation.
The hollow part is not shown in the animation.
The two connector frames <b>frame_a</b> and <b>frame_b</b>
are always parallel to each other. Example of component
animation (note, that
the animation may be switched off via parameter animation = <b>false</b>):
</p>

<p>
<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/BodyBox.png\" ALT=\"Parts.BodyBox\">
</p>

<p>
A BodyBox component has potential states. For details of these
states and of the \"Advanced\" menu parameters, see model
<a href=\"modelica://Modelica.Mechanics.MultiBody.Parts.Body\">MultiBody.Parts.Body</a>.</p>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{100, 40}, {100, -30}, {90, -40}, {90, 30}, {100, 40}}, lineColor = {0, 95, 191}, fillColor = {0, 95, 191}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-100, 30}, {90, -40}}, lineColor = {0, 127, 255}, pattern = LinePattern.None, fillColor = {0, 127, 255}, fillPattern = FillPattern.Solid), Polygon(points = {{-100, 30}, {-90, 40}, {100, 40}, {90, 30}, {-100, 30}}, lineColor = {0, 95, 191}, fillColor = {0, 95, 191}, fillPattern = FillPattern.Solid), Text(extent = {{-150, 90}, {150, 50}}, textString = "%name", lineColor = {0, 0, 255}), Text(extent = {{150, -80}, {-150, -50}}, lineColor = {0, 0, 0}, textString = "r=%r"), Text(extent = {{52, 8}, {88, -17}}, lineColor = {0, 0, 0}, textString = "b"), Text(extent = {{-87, 12}, {-51, -13}}, lineColor = {0, 0, 0}, textString = "a")}));
        end BodyBox;

        model BodyCylinder "Rigid body with cylinder shape. Mass and animation properties are computed from cylinder data and density (12 potential states)"
          import NonSI = Modelica.SIunits.Conversions.NonSIunits;
          import Modelica.Mechanics.MultiBody.Types;
          import Modelica.Math.Vectors.normalizeWithAssert;
          import Modelica.SIunits.Conversions.to_unit1;
          Interfaces.Frame_a frame_a "Coordinate system fixed to the component with one cut-force and cut-torque" annotation(
            Placement(transformation(extent = {{-116, -16}, {-84, 16}}, rotation = 0)));
          Interfaces.Frame_b frame_b "Coordinate system fixed to the component with one cut-force and cut-torque" annotation(
            Placement(transformation(extent = {{84, -16}, {116, 16}}, rotation = 0)));
          parameter Boolean animation = true "= true, if animation shall be enabled (show cylinder between frame_a and frame_b)";
          parameter SI.Position r[3](start = {0.1, 0, 0}) "Vector from frame_a to frame_b, resolved in frame_a";
          parameter SI.Position r_shape[3] = {0, 0, 0} "Vector from frame_a to cylinder origin, resolved in frame_a";
          parameter Modelica.Mechanics.MultiBody.Types.Axis lengthDirection = to_unit1(r - r_shape) "Vector in length direction of cylinder, resolved in frame_a" annotation(
            Evaluate = true);
          parameter SI.Length length = Modelica.Math.Vectors.length(r - r_shape) "Length of cylinder";
          parameter SI.Distance diameter = length / world.defaultWidthFraction "Diameter of cylinder";
          parameter SI.Distance innerDiameter = 0 "Inner diameter of cylinder (0 <= innerDiameter <= Diameter)";
          parameter SI.Density density = 7700 "Density of cylinder (e.g., steel: 7700 .. 7900, wood : 400 .. 800)";
          input Modelica.Mechanics.MultiBody.Types.Color color = Modelica.Mechanics.MultiBody.Types.Defaults.BodyColor "Color of cylinder" annotation(
            Dialog(colorSelector = true, enable = animation));
          input Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(
            Dialog(enable = animation));
          SI.Position r_0[3](start = {0, 0, 0}, each stateSelect = if enforceStates then StateSelect.always else StateSelect.avoid) "Position vector from origin of world frame to origin of frame_a" annotation(
            Dialog(tab = "Initialization", showStartAttribute = true));
          SI.Velocity v_0[3](start = {0, 0, 0}, each stateSelect = if enforceStates then StateSelect.always else StateSelect.avoid) "Absolute velocity of frame_a, resolved in world frame (= der(r_0))" annotation(
            Dialog(tab = "Initialization", showStartAttribute = true));
          SI.Acceleration a_0[3](start = {0, 0, 0}) "Absolute acceleration of frame_a resolved in world frame (= der(v_0))" annotation(
            Dialog(tab = "Initialization", showStartAttribute = true));
          parameter Boolean angles_fixed = false "= true, if angles_start are used as initial values, else as guess values" annotation(
            Evaluate = true,
            choices(checkBox = true),
            Dialog(tab = "Initialization"));
          parameter SI.Angle angles_start[3] = {0, 0, 0} "Initial values of angles to rotate frame_a around 'sequence_start' axes into frame_b" annotation(
            Dialog(tab = "Initialization"));
          parameter Types.RotationSequence sequence_start = {1, 2, 3} "Sequence of rotations to rotate frame_a into frame_b at initial time" annotation(
            Evaluate = true,
            Dialog(tab = "Initialization"));
          parameter Boolean w_0_fixed = false "= true, if w_0_start are used as initial values, else as guess values" annotation(
            Evaluate = true,
            choices(checkBox = true),
            Dialog(tab = "Initialization"));
          parameter SI.AngularVelocity w_0_start[3] = {0, 0, 0} "Initial or guess values of angular velocity of frame_a resolved in world frame" annotation(
            Dialog(tab = "Initialization"));
          parameter Boolean z_0_fixed = false "= true, if z_0_start are used as initial values, else as guess values" annotation(
            Evaluate = true,
            choices(checkBox = true),
            Dialog(tab = "Initialization"));
          parameter SI.AngularAcceleration z_0_start[3] = {0, 0, 0} "Initial values of angular acceleration z_0 = der(w_0)" annotation(
            Dialog(tab = "Initialization"));
          parameter Boolean enforceStates = false "= true, if absolute variables of body object shall be used as states (StateSelect.always)" annotation(
            Dialog(tab = "Advanced"));
          parameter Boolean useQuaternions = true "= true, if quaternions shall be used as potential states otherwise use 3 angles as potential states" annotation(
            Dialog(tab = "Advanced"));
          parameter Types.RotationSequence sequence_angleStates = {1, 2, 3} "Sequence of rotations to rotate world frame into frame_a around the 3 angles used as potential states" annotation(
            Evaluate = true,
            Dialog(tab = "Advanced", enable = not useQuaternions));
          constant Real pi = Modelica.Constants.pi;
          final parameter SI.Distance radius = diameter / 2 "Radius of cylinder";
          final parameter SI.Distance innerRadius = innerDiameter / 2 "Inner-Radius of cylinder";
          final parameter SI.Mass mo(min = 0) = density * pi * length * radius * radius "Mass of cylinder without hole";
          final parameter SI.Mass mi(min = 0) = density * pi * length * innerRadius * innerRadius "Mass of hole of cylinder";
          final parameter SI.Inertia I22 = (mo * (length * length + 3 * radius * radius) - mi * (length * length + 3 * innerRadius * innerRadius)) / 12 "Inertia with respect to axis through center of mass, perpendicular to cylinder axis";
          final parameter SI.Mass m(min = 0) = mo - mi "Mass of cylinder";
          final parameter Frames.Orientation R = Frames.from_nxy(r, {0, 1, 0}) "Orientation object from frame_a to frame spanned by cylinder axis and axis perpendicular to cylinder axis";
          final parameter SI.Position r_CM[3] = r_shape + normalizeWithAssert(lengthDirection) * length / 2 "Position vector from frame_a to center of mass, resolved in frame_a";
          final parameter SI.Inertia I[3, 3] = Frames.resolveDyade1(R, diagonal({(mo * radius * radius - mi * innerRadius * innerRadius) / 2, I22, I22})) "Inertia tensor of cylinder with respect to center of mass, resolved in frame parallel to frame_a";
          Body body(r_CM = r_CM, m = m, I_11 = I[1, 1], I_22 = I[2, 2], I_33 = I[3, 3], I_21 = I[2, 1], I_31 = I[3, 1], I_32 = I[3, 2], animation = false, sequence_start = sequence_start, angles_fixed = angles_fixed, angles_start = angles_start, w_0_fixed = w_0_fixed, w_0_start = w_0_start, z_0_fixed = z_0_fixed, z_0_start = z_0_start, useQuaternions = useQuaternions, sequence_angleStates = sequence_angleStates, enforceStates = false) annotation(
            Placement(transformation(extent = {{-30, -80}, {10, -40}}, rotation = 0)));
          FixedTranslation frameTranslation(r = r, animation = animation, shapeType = "pipecylinder", r_shape = r_shape, lengthDirection = lengthDirection, length = length, width = diameter, height = diameter, extra = innerDiameter / diameter, color = color, specularCoefficient = specularCoefficient, widthDirection = {0, 1, 0}) annotation(
            Placement(transformation(extent = {{-30, -20}, {10, 20}}, rotation = 0)));
        protected
          outer Modelica.Mechanics.MultiBody.World world;
        equation
          r_0 = frame_a.r_0;
          v_0 = der(r_0);
          a_0 = der(v_0);
          assert(innerDiameter < diameter, "parameter innerDiameter is greater than parameter diameter");
          connect(frameTranslation.frame_a, frame_a) annotation(
            Line(points = {{-30, 0}, {-100, 0}}, color = {95, 95, 95}, thickness = 0.5));
          connect(frameTranslation.frame_b, frame_b) annotation(
            Line(points = {{10, 0}, {100, 0}}, color = {95, 95, 95}, thickness = 0.5));
          connect(frame_a, body.frame_a) annotation(
            Line(points = {{-100, 0}, {-70, 0}, {-70, -60}, {-30, -60}}, color = {95, 95, 95}, thickness = 0.5));
          annotation(
            Documentation(info = "<HTML>
<p>
<b>Rigid body</b> with <b>cylinder</b> shape.
The mass properties of the body (mass, center of mass,
inertia tensor) are computed
from the cylinder data. Optionally, the cylinder may be hollow.
The cylinder shape is by default used in the animation.
The two connector frames <b>frame_a</b> and <b>frame_b</b>
are always parallel to each other. Example of component
animation (note, that
the animation may be switched off via parameter animation = <b>false</b>):
</p>

<p>
<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/BodyCylinder.png\" ALT=\"Parts.BodyCylinder\">
</p>

<p>
A BodyCylinder component has potential states. For details of these
states and of the \"Advanced\" menu parameters, see model
<a href=\"modelica://Modelica.Mechanics.MultiBody.Parts.Body\">MultiBody.Parts.Body</a>.</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-150, 90}, {150, 50}}, textString = "%name", lineColor = {0, 0, 255}), Text(extent = {{150, -80}, {-150, -50}}, lineColor = {0, 0, 0}, textString = "r=%r"), Rectangle(extent = {{-100, 40}, {100, -40}}, lineColor = {0, 24, 48}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {0, 127, 255}, radius = 10), Text(extent = {{-87, 13}, {-51, -12}}, lineColor = {0, 0, 0}, textString = "a"), Text(extent = {{51, 12}, {87, -13}}, lineColor = {0, 0, 0}, textString = "b")}));
        end BodyCylinder;

        model PointMass "Rigid body where body rotation and inertia tensor is neglected (6 potential states)"
          import Modelica.Mechanics.MultiBody.Types;
          Interfaces.Frame_a frame_a "Coordinate system fixed at center of mass point" annotation(
            Placement(transformation(extent = {{-16, -16}, {16, 16}}, rotation = 0)));
          parameter Boolean animation = true "= true, if animation shall be enabled (show sphere)";
          parameter SI.Mass m(min = 0) "Mass of mass point";
          input SI.Diameter sphereDiameter = world.defaultBodyDiameter "Diameter of sphere" annotation(
            Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          input Types.Color sphereColor = Modelica.Mechanics.MultiBody.Types.Defaults.BodyColor "Color of sphere" annotation(
            Dialog(colorSelector = true, tab = "Animation", group = "if animation = true", enable = animation));
          input Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(
            Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter StateSelect stateSelect = StateSelect.avoid "Priority to use frame_a.r_0, v_0 (= der(frame_a.r_0)) as states" annotation(
            Dialog(tab = "Advanced"));
          SI.Position r_0[3](start = {0, 0, 0}, each stateSelect = stateSelect) "Position vector from origin of world frame to origin of frame_a, resolved in world frame" annotation(
            Dialog(group = "Initialization", showStartAttribute = true));
          SI.Velocity v_0[3](start = {0, 0, 0}, each stateSelect = stateSelect) "Absolute velocity of frame_a, resolved in world frame (= der(r_0))" annotation(
            Dialog(group = "Initialization", showStartAttribute = true));
          SI.Acceleration a_0[3](start = {0, 0, 0}) "Absolute acceleration of frame_a resolved in world frame (= der(v_0))" annotation(
            Dialog(group = "Initialization", showStartAttribute = true));
        protected
          outer Modelica.Mechanics.MultiBody.World world;
          // Declarations for animation
          Visualizers.Advanced.Shape sphere(shapeType = "sphere", color = sphereColor, specularCoefficient = specularCoefficient, length = sphereDiameter, width = sphereDiameter, height = sphereDiameter, lengthDirection = {1, 0, 0}, widthDirection = {0, 1, 0}, r_shape = -{1, 0, 0} * sphereDiameter / 2, r = frame_a.r_0, R = frame_a.R) if world.enableAnimation and animation;
        equation
// If any possible, do not use the connector as root
          Connections.potentialRoot(frame_a.R, 10000);
          if Connections.isRoot(frame_a.R) then
            assert(cardinality(frame_a) == 0, "
A Modelica.Mechanics.MultiBody.Parts.PointMass model is connected in
a way, so that no equations are present to compute frame_a.R
(the orientation object in the connector). Setting frame_a.R to
an arbitrary value in the PointMass model, might lead to a wrong
overall model, depending on how the PointMass model is used.
   You can avoid this message, by providing equations that
compute the orientation object, e.g., by using the
Modelica.Mechanics.MultiBody.Joints.FreeMotion joint.
   If a PointMass model is not connected at all, the
orientation object is set to a unit rotation. But this is
the only case where this is done.
            ");
            frame_a.R = Frames.nullRotation();
          else
            frame_a.t = zeros(3);
          end if;
// Newton equation: f = m*(a-g)
          r_0 = frame_a.r_0;
          v_0 = der(r_0);
          a_0 = der(v_0);
          frame_a.f = m * Frames.resolve2(frame_a.R, a_0 - world.gravityAcceleration(r_0));
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{150, -90}, {-150, -60}}, lineColor = {0, 0, 0}, textString = "m=%m"), Text(extent = {{-150, 100}, {150, 60}}, textString = "%name", lineColor = {0, 0, 255}), Ellipse(extent = {{-50, 50}, {50, -50}}, lineColor = {0, 24, 48}, fillPattern = FillPattern.Sphere, fillColor = {0, 127, 255})}),
            Documentation(info = "<HTML>
<p>
<b>Rigid body</b> where the inertia tensor is neglected.
This body is
solely defined by its mass.
By default, this component is visualized by a <b>sphere</b> that has
its center at frame_a. Note, that
the animation may be switched off via parameter animation = <b>false</b>.
</p>

<p>
Every PointMass has potential states. If possible a tool will select
the states of joints and not the states of PointMass because this is
usually the most efficient choice. In this case the position and
velocity of frame_a of the body will be computed
by the component that is connected to frame_a. However, if a PointMass is moving
freely in space, variables of the PointMass have to be used as states. The potential
states are: The <b>position vector</b> frame_a.r_0 from the origin of the
world frame to the origin of frame_a of the body, resolved in
the world frame and the <b>absolute velocity</b> v_0 of the origin of
frame_a, resolved in the world frame (= der(frame_a.r_0)).
</p>

<p>
Whether or not variables of the body are used as states is usually
automatically selected by the Modelica translator. If parameter
<b>enforceStates</b> is set to <b>true</b> in the \"Advanced\" menu,
then PointMass variables frame_a.r_0 and der(frame_a.r_0)
are forced to be used as states.
</p>
</HTML>"));
        end PointMass;

        model Mounting1D "Propagate 1-dim. support torque to 3-dim. system (provided world.driveTrainMechanics3D=true)"
          parameter Modelica.SIunits.Angle phi0 = 0 "Fixed offset angle of housing";
          parameter Modelica.Mechanics.MultiBody.Types.Axis n = {1, 0, 0} "Axis of rotation = axis of support torque (resolved in frame_a)" annotation(
            Evaluate = true);
          Modelica.Mechanics.Rotational.Interfaces.Flange_b flange_b "(right) flange fixed in housing" annotation(
            Placement(transformation(extent = {{110, 10}, {90, -10}}, rotation = 0)));
          Modelica.Mechanics.MultiBody.Interfaces.Frame_a frame_a if world.driveTrainMechanics3D "Frame in which housing is fixed (connector is removed, if world.driveTrainMechanics3D=false)" annotation(
            Placement(transformation(origin = {0, -100}, extent = {{-20, -20}, {20, 20}}, rotation = 90)));
        protected
          outer Modelica.Mechanics.MultiBody.World world;

          encapsulated model Housing
            import Modelica;
            input Modelica.SIunits.Torque t[3];
            Modelica.Mechanics.MultiBody.Interfaces.Frame_a frame_a annotation(
              Placement(transformation(extent = {{-116, -16}, {-84, 16}}, rotation = 0)));
          equation
            frame_a.f = zeros(3);
            frame_a.t = t;
            annotation(
              Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, 100}, {100, -100}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-150, 110}, {150, 150}}, lineColor = {0, 0, 255}, textString = "%name")}));
          end Housing;

          Housing housing(t = -n * flange_b.tau) if world.driveTrainMechanics3D annotation(
            Placement(transformation(extent = {{20, -60}, {40, -40}}, rotation = 0)));
        equation
          flange_b.phi = phi0;
          connect(housing.frame_a, frame_a) annotation(
            Line(points = {{20, -50}, {0, -50}, {0, -100}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-80, -60}, {80, -100}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-150, 60}, {150, 20}}, lineColor = {0, 0, 255}, textString = "%name"), Line(points = {{80, -60}, {40, -100}}, color = {0, 0, 0}), Line(points = {{40, -60}, {0, -100}}, color = {0, 0, 0}), Line(points = {{0, -60}, {-40, -100}}, color = {0, 0, 0}), Line(points = {{-40, -60}, {-80, -100}}, color = {0, 0, 0}), Line(points = {{0, -60}, {0, 0}}, color = {0, 0, 0}), Line(points = {{0, 0}, {90, 0}}, color = {0, 0, 0})}),
            Documentation(info = "<html>
<p>This component is used to acquire support torques from a 1-dim.-rotational
mechanical system (e.g., components from Modelica.Mechanics.Rotational)
and to propagate them to a carrier body.</p>
<p>The 1-dim. support torque at <code>flange_b</code> is transformed into 3-dim. space under
consideration of the rotation axis, parameter <code>n</code>, which has to be given in the
local coordinate system of <code>frame_a</code>.</p>
<p>All components of a 1-dim.-rotational mechanical system that are connected to <b>a</b> common
<b>Mounting1D</b> element need to have the same axis of rotation
along parameter vector <code>n</code>. This means that, e.g., bevel
gears where the axis of rotation of <code>flange_a</code> and
<code>flange_b</code> are different cannot be described properly by
connecting to the <b>Mounting1D</b> component. In this case, a combination of several
<b>Mounting1D</b> components or the component <b>BevelGear1D</b> should be used.</p>
<p><b>Reference</b><br>
<span style=\"font-variant:small-caps\">Schweiger</span>, Christian ;
<span style=\"font-variant:small-caps\">Otter</span>, Martin:
<a href=\"https://www.modelica.org/events/Conference2003/papers/h06_Schweiger_powertrains_v5.pdf\">Modelling
3D Mechanical Effects of 1-dim. Powertrains</a>. In: <i>Proceedings of the 3rd International
Modelica Conference</i>. Link&ouml;ping : The Modelica Association and Link&ouml;ping University,
November 3-4, 2003, pp. 149-158</p>
</html>"));
        end Mounting1D;

        model Rotor1D "1D inertia attachable on 3-dim. bodies (3D dynamic effects are taken into account if world.driveTrainMechanics3D=true)"
          parameter Boolean animation = true "= true, if animation shall be enabled (show rotor as cylinder)";
          parameter SI.Inertia J(min = 0, start = 1) "Moment of inertia of rotor around its axis of rotation";
          parameter Modelica.Mechanics.MultiBody.Types.Axis n = {1, 0, 0} "Axis of rotation resolved in frame_a" annotation(
            Evaluate = true);
          parameter SI.Position r_center[3] = zeros(3) "Position vector from origin of frame_a to center of cylinder" annotation(
            Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter SI.Distance cylinderLength = 2 * world.defaultJointLength "Length of cylinder representing the rotor" annotation(
            Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter SI.Distance cylinderDiameter = 2 * world.defaultJointWidth "Diameter of cylinder representing the rotor" annotation(
            Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          input Modelica.Mechanics.MultiBody.Types.Color cylinderColor = Modelica.Mechanics.MultiBody.Types.Defaults.RodColor "Color of cylinder representing the rotor" annotation(
            Dialog(colorSelector = true, tab = "Animation", group = "if animation = true", enable = animation));
          input Modelica.Mechanics.MultiBody.Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(
            Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter StateSelect stateSelect = StateSelect.default "Priority to use rotor angle (phi) and rotor speed (w) as states" annotation(
            Dialog(tab = "Advanced"));
          parameter Boolean exact = true "= true, if exact calculations; false if influence of bearing on rotor acceleration is neglected to avoid an algebraic loop" annotation(
            Dialog(tab = "Advanced"));
          SI.Angle phi(start = 0, stateSelect = stateSelect) "Rotation angle of rotor with respect to frame_a (= flange_a.phi = flange_b.phi)";
          SI.AngularVelocity w(start = 0, stateSelect = stateSelect) "Angular velocity of rotor with respect to frame_a";
          SI.AngularAcceleration a(start = 0) "Angular acceleration of rotor with respect to frame_a";
          Modelica.Mechanics.Rotational.Interfaces.Flange_a flange_a "(left) driving flange (flange axis directed INTO cut plane)" annotation(
            Placement(transformation(extent = {{-110, -10}, {-90, 10}}, rotation = 0)));
          Modelica.Mechanics.Rotational.Interfaces.Flange_b flange_b "(right) driven flange (flange axis directed OUT OF cut plane)" annotation(
            Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
          Modelica.Mechanics.MultiBody.Interfaces.Frame_a frame_a if world.driveTrainMechanics3D "Frame in which rotor housing is fixed (connector is removed, if world.driveTrainMechanics3D=false)" annotation(
            Placement(transformation(origin = {0, -100}, extent = {{-20, -20}, {20, 20}}, rotation = 90)));

          encapsulated model RotorWith3DEffects "1D inertia attachable on 3-dim. bodies (3D dynamic effects are taken into account)"
            import Modelica;
            import Modelica.Mechanics.MultiBody.Frames;
            import Modelica.Mechanics.MultiBody.Types;
            import SI = Modelica.SIunits;
            import Cv = Modelica.SIunits.Conversions;
            parameter Boolean animation = true "= true, if animation shall be enabled (show rotor as cylinder)";
            parameter SI.Inertia J(min = 0) = 1 "Moment of inertia of rotor around its axis of rotation";
            parameter Types.Axis n = {1, 0, 0} "Axis of rotation resolved in frame_a";
            parameter SI.Position r_center[3] = zeros(3) "Position vector from origin of frame_a to center of cylinder" annotation(
              Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            parameter SI.Distance cylinderLength = 2 * world.defaultJointLength "Length of cylinder representing the rotor" annotation(
              Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            parameter SI.Distance cylinderDiameter = 2 * world.defaultJointWidth "Diameter of cylinder representing the rotor" annotation(
              Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            input Types.Color cylinderColor = Modelica.Mechanics.MultiBody.Types.Defaults.RodColor "Color of cylinder representing the rotor" annotation(
              Dialog(colorSelector = true, tab = "Animation", group = "if animation = true", enable = animation));
            input Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(
              Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            parameter StateSelect stateSelect = StateSelect.default "Priority to use rotor angle (phi) and rotor speed (w) as states" annotation(
              Dialog(tab = "Advanced"));
            parameter Boolean exact = true "= true, if exact calculations; false if influence of bearing on rotor acceleration is neglected to avoid an algebraic loop" annotation(
              Dialog(tab = "Advanced"));
            SI.AngularVelocity w_a[3] "Angular velocity of frame_a, resolved in frame_a";
            SI.Angle phi(start = 0, final stateSelect = stateSelect) "Rotation angle of rotor with respect to frame_a (= flange_a.phi = flange_b.phi)" annotation(
              Dialog(showStartAttribute = true));
            SI.AngularVelocity w(start = 0, stateSelect = stateSelect) "Angular velocity of rotor with respect to frame_a" annotation(
              Dialog(showStartAttribute = true));
            SI.AngularAcceleration a(start = 0) "Angular acceleration of rotor with respect to frame_a" annotation(
              Dialog(showStartAttribute = true));
            Modelica.Mechanics.Rotational.Interfaces.Flange_a flange_a "(left) driving flange (flange axis directed INTO cut plane)" annotation(
              Placement(transformation(extent = {{-110, -10}, {-90, 10}}, rotation = 0)));
            Modelica.Mechanics.Rotational.Interfaces.Flange_b flange_b "(right) driven flange (flange axis directed OUT OF cut plane)" annotation(
              Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Interfaces.Frame_a frame_a "Frame in which rotor housing is fixed" annotation(
              Placement(transformation(origin = {0, -100}, extent = {{-20, -20}, {20, 20}}, rotation = 90)));
          protected
            outer Modelica.Mechanics.MultiBody.World world;
            parameter Real e[3](each final unit = "1") = Modelica.Math.Vectors.normalizeWithAssert(n) "Unit vector in direction of rotor axis, resolved in frame_a";
            parameter SI.Inertia nJ[3] = J * e;
            Modelica.Mechanics.MultiBody.Visualizers.Advanced.Shape cylinder(shapeType = "cylinder", color = cylinderColor, specularCoefficient = specularCoefficient, length = cylinderLength, width = cylinderDiameter, height = cylinderDiameter, lengthDirection = n, widthDirection = {0, 1, 0}, extra = 1, r_shape = r_center - e * (cylinderLength / 2), r = frame_a.r_0, R = Frames.absoluteRotation(frame_a.R, Frames.planarRotation(e, phi, 0))) if world.enableAnimation and animation;
          equation
            phi = flange_a.phi;
            phi = flange_b.phi;
            w = der(phi);
            a = der(w);
            w_a = Modelica.Mechanics.MultiBody.Frames.angularVelocity2(frame_a.R);
            if exact then
              J * a = flange_a.tau + flange_b.tau - nJ * der(w_a);
            else
              J * a = flange_a.tau + flange_b.tau;
            end if;
/* Reaction torque:
        t = n*(J*a - flange_a.tau - flange_b.tau) + cross(w_a, nJ*w)

     Since
        J*a = flange_a.tau + flange_b.tau - nJ*der(w_a);

     the reaction torque can be simplified to
        t = n*(- nJ*der(w_a)) + cross(w_a, nJ*w)

  */
            frame_a.f = zeros(3);
            frame_a.t = cross(w_a, nJ * w) - e * (nJ * der(w_a));
            annotation(
              Documentation(info = "<html>
<p>
This component is used to model the gyroscopic torques exerted by a 1-dim.
inertia (so called <i>rotor</i>) on its 3-dim. carrier body. Gyroscopic torques
appear, if the vector of the carrier body's angular velocity is not parallel
to the vector of the rotor's axis. The axis of rotation of the rotor is defined by
the parameter <code>n</code>, which has to be given in the local coordinate system
of <code>frame_a</code>. The default animation of this component is
shown in the figure below.</p>

<p>
      <IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Parts/Rotor1D.png\" ALT=\"model Parts.Rotor1D\">
</p>

<p>This component is a replacement for
<a href=\"modelica://Modelica.Mechanics.Rotational.Components.Inertia\">Modelica.Mechanics.Rotational.Components.Inertia</a>
for the case, that a 1-dim.-rotational mechanical system should be attached with a 3-dim.
carrier body.</p>
<p>The Boolean parameter <code>exact</code> was introduced due to performance
reasons. If <code>exact</code> is set to <b>false</b>, the influence of the carrier body
motion on the angular velocity of the rotor is neglected. This influence is usually
negligible if the 1-dim.-rotational mechanical system accelerates much faster as the base body (this is,
e.g., the case in vehicle powertrains). The essential advantage is
that an algebraic loop is removed since then there is only an
action on acceleration level from the powertrain to the base body
but not vice versa.</p>
<p><b>Reference</b><br>
<span style=\"font-variant:small-caps\">Schweiger</span>, Christian ;
<span style=\"font-variant:small-caps\">Otter</span>, Martin:
<a href=\"https://www.modelica.org/events/Conference2003/papers/h06_Schweiger_powertrains_v5.pdf\">Modelling
3D Mechanical Effects of 1-dim. Powertrains</a>. In: <i>Proceedings of the 3rd International
Modelica Conference</i>. Link&ouml;ping : The Modelica Association and Link&ouml;ping University,
November 3-4, 2003, pp. 149-158</p>
</html>"),
              Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-80, -25}, {-60, -25}}, color = {0, 0, 0}), Line(points = {{60, -25}, {80, -25}}, color = {0, 0, 0}), Line(points = {{-70, -25}, {-70, -70}}, color = {0, 0, 0}), Line(points = {{70, -25}, {70, -70}}, color = {0, 0, 0}), Line(points = {{-80, 25}, {-60, 25}}, color = {0, 0, 0}), Line(points = {{60, 25}, {80, 25}}, color = {0, 0, 0}), Line(points = {{-70, 45}, {-70, 25}}, color = {0, 0, 0}), Line(points = {{70, 45}, {70, 25}}, color = {0, 0, 0}), Line(points = {{-70, -70}, {70, -70}}, color = {0, 0, 0}), Rectangle(extent = {{-50, 50}, {50, -50}}, lineColor = {64, 64, 64}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {255, 255, 255}, radius = 10), Rectangle(extent = {{-50, 50}, {50, -50}}, lineColor = {64, 64, 64}, fillPattern = FillPattern.None, radius = 10), Rectangle(extent = {{-100, 10}, {-50, -10}}, lineColor = {64, 64, 64}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192, 192, 192}), Rectangle(extent = {{50, 10}, {100, -10}}, lineColor = {64, 64, 64}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192, 192, 192}), Text(extent = {{-148, 112}, {152, 72}}, textString = "%name=%J", lineColor = {0, 0, 255}), Line(points = {{0, -70}, {0, -100}}, color = {0, 0, 0})}));
          end RotorWith3DEffects;
        protected
          outer Modelica.Mechanics.MultiBody.World world;
          parameter Real e[3](each final unit = "1") = Modelica.Math.Vectors.normalizeWithAssert(n) "Unit vector in direction of rotor axis, resolved in frame_a";
          Modelica.Mechanics.MultiBody.Visualizers.Advanced.Shape cylinder(shapeType = "cylinder", color = cylinderColor, specularCoefficient = specularCoefficient, length = cylinderLength, width = cylinderDiameter, height = cylinderDiameter, lengthDirection = n, widthDirection = {0, 1, 0}, extra = 1, r_shape = r_center - e * (cylinderLength / 2), r = zeros(3), R = Modelica.Mechanics.MultiBody.Frames.planarRotation(e, phi, 0)) if world.enableAnimation and animation and not world.driveTrainMechanics3D;
          Modelica.Mechanics.Rotational.Components.Inertia inertia(J = J, stateSelect = StateSelect.never) if not world.driveTrainMechanics3D annotation(
            Placement(transformation(extent = {{-20, -20}, {20, 20}}, rotation = 0)));
          RotorWith3DEffects rotorWith3DEffects(animation = animation, J = J, n = n, r_center = r_center, cylinderLength = cylinderLength, cylinderDiameter = cylinderDiameter, cylinderColor = cylinderColor, specularCoefficient = specularCoefficient, exact = exact, stateSelect = StateSelect.never) if world.driveTrainMechanics3D annotation(
            Placement(transformation(extent = {{-20, -80}, {20, -40}}, rotation = 0)));
        equation
          phi = flange_a.phi;
          w = der(phi);
          a = der(w);
          connect(inertia.flange_a, flange_a) annotation(
            Line(points = {{-20, 0}, {-100, 0}}, color = {0, 0, 0}, smooth = Smooth.None));
          connect(inertia.flange_b, flange_b) annotation(
            Line(points = {{20, 0}, {100, 0}}, color = {0, 0, 0}, smooth = Smooth.None));
          connect(rotorWith3DEffects.flange_b, flange_b) annotation(
            Line(points = {{20, -60}, {60, -60}, {60, 0}, {100, 0}}, color = {0, 0, 0}, smooth = Smooth.None));
          connect(rotorWith3DEffects.flange_a, flange_a) annotation(
            Line(points = {{-20, -60}, {-60, -60}, {-60, 0}, {-100, 0}}, color = {0, 0, 0}, smooth = Smooth.None));
          connect(rotorWith3DEffects.frame_a, frame_a) annotation(
            Line(points = {{0, -80}, {0, -100}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
          annotation(
            Documentation(info = "<html>
<p>This component is used to model the gyroscopic torques exerted by a 1-dim.
inertia (so called <i>rotor</i>) on its 3-dim. carrier body. Gyroscopic torques
appear, if the vector of the carrier body's angular velocity is not parallel
to the vector of the rotor's axis. The axis of rotation of the rotor is defined by
the parameter <code>n</code>, which has to be given in the local coordinate system
of <code>frame_a</code>. The default animation of this component is
shown in the figure below.</p>
<p>
      <IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Parts/Rotor1D.png\" ALT=\"model Parts.Rotor1D\">
</p>
<p>This component is a replacement for
<a href=\"modelica://Modelica.Mechanics.Rotational.Components.Inertia\">Modelica.Mechanics.Rotational.Components.Inertia</a>
for the case, that a 1-dim.-rotational mechanical system should be attached with a 3-dim.
carrier body.</p>
<p>The Boolean parameter <code>exact</code> was introduced due to performance
reasons. If <code>exact</code> is set to <b>false</b>, the influence of the carrier body
motion on the angular velocity of the rotor is neglected. This influence is usually
negligible if the 1-dim.-rotational mechanical system accelerates much faster as the base body (this is,
e.g., the case in vehicle powertrains). The essential advantage is
that an algebraic loop is removed since then there is only an
action on acceleration level from the powertrain to the base body
but not vice versa.</p>
<p><b>Reference</b><br>
<span style=\"font-variant:small-caps\">Schweiger</span>, Christian ;
<span style=\"font-variant:small-caps\">Otter</span>, Martin:
<a href=\"https://www.modelica.org/events/Conference2003/papers/h06_Schweiger_powertrains_v5.pdf\">Modelling
3D Mechanical Effects of 1-dim. Powertrains</a>. In: <i>Proceedings of the 3rd International
Modelica Conference</i>. Link&ouml;ping : The Modelica Association and Link&ouml;ping University,
November 3-4, 2003, pp. 149-158</p>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-80, -25}, {-60, -25}}, color = {0, 0, 0}), Line(points = {{60, -25}, {80, -25}}, color = {0, 0, 0}), Line(points = {{-70, -25}, {-70, -70}}, color = {0, 0, 0}), Line(points = {{70, -25}, {70, -70}}, color = {0, 0, 0}), Line(points = {{-80, 25}, {-60, 25}}, color = {0, 0, 0}), Line(points = {{60, 25}, {80, 25}}, color = {0, 0, 0}), Line(points = {{-70, 45}, {-70, 25}}, color = {0, 0, 0}), Line(points = {{70, 45}, {70, 25}}, color = {0, 0, 0}), Line(points = {{-70, -70}, {70, -70}}, color = {0, 0, 0}), Rectangle(extent = {{-50, 50}, {50, -50}}, lineColor = {64, 64, 64}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {255, 255, 255}, radius = 10), Rectangle(extent = {{-50, 50}, {50, -50}}, lineColor = {64, 64, 64}, fillPattern = FillPattern.None, radius = 10), Rectangle(extent = {{-100, 10}, {-50, -10}}, lineColor = {64, 64, 64}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192, 192, 192}), Rectangle(extent = {{50, 10}, {100, -10}}, lineColor = {64, 64, 64}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192, 192, 192}), Text(extent = {{-150, 125}, {150, 85}}, lineColor = {0, 0, 255}, textString = "%name"), Line(points = {{0, -70}, {0, -100}}, color = {0, 0, 0}), Text(extent = {{-150, 80}, {150, 50}}, lineColor = {0, 0, 0}, textString = "%J")}));
        end Rotor1D;

        model BevelGear1D "1D gearbox with arbitrary shaft directions and 3-dim. bearing frame (3D dynamic effects are taken into account provided world.driveTrainMechanics3D=true)"
          extends Modelica.Mechanics.Rotational.Interfaces.PartialTwoFlanges;
          parameter Real ratio(start = 1) "Gear speed ratio";
          parameter Modelica.Mechanics.MultiBody.Types.Axis n_a = {1, 0, 0} "Axis of rotation of flange_a, resolved in frame_a" annotation(
            Evaluate = true);
          parameter Modelica.Mechanics.MultiBody.Types.Axis n_b = {1, 0, 0} "Axis of rotation of flange_b, resolved in frame_a" annotation(
            Evaluate = true);
          Modelica.Mechanics.MultiBody.Interfaces.Frame_a frame_a if world.driveTrainMechanics3D "Bearing frame" annotation(
            Placement(transformation(origin = {0, -100}, extent = {{-20, -20}, {20, 20}}, rotation = 90)));
        protected
          outer World world;
          parameter Real e_a[3](each final unit = "1") = Modelica.Math.Vectors.normalizeWithAssert(n_a) "Unit vector in direction of flange_a rotation axis";
          parameter Real e_b[3](each final unit = "1") = Modelica.Math.Vectors.normalizeWithAssert(n_b) "Unit vector in direction of flange_b rotation axis";

          encapsulated model Housing
            import Modelica;
            input Modelica.SIunits.Torque t[3];
            Modelica.Mechanics.MultiBody.Interfaces.Frame_a frame_a annotation(
              Placement(transformation(extent = {{-116, -16}, {-84, 16}}, rotation = 0)));
          equation
            frame_a.f = zeros(3);
            frame_a.t = t;
            annotation(
              Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, 100}, {100, -100}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-150, 110}, {150, 150}}, lineColor = {0, 0, 255}, textString = "%name")}));
          end Housing;

          Housing housing(t = (-flange_a.tau * e_a) - flange_b.tau * e_b) if world.driveTrainMechanics3D annotation(
            Placement(transformation(extent = {{20, -60}, {40, -40}}, rotation = 0)));
        equation
          flange_a.phi = ratio * flange_b.phi;
          0 = ratio * flange_a.tau + flange_b.tau;
          connect(housing.frame_a, frame_a) annotation(
            Line(points = {{20, -50}, {0, -50}, {0, -100}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}, initialScale = 0.1), graphics = {Text(origin = {0, -40}, lineColor = {0, 0, 255}, extent = {{-150, 150}, {150, 180}}, textString = "%name=%ratio"), Text(origin = {0, -10}, extent = {{-150, -66}, {150, -36}}, textString = "%name=%ratio"), Rectangle(origin = {-35, 60}, fillColor = {255, 255, 255}, fillPattern = FillPattern.HorizontalCylinder, extent = {{-15, -40}, {15, 40}}), Rectangle(origin = {-35, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.HorizontalCylinder, extent = {{-15, -21}, {15, 21}}), Line(points = {{-80, 20}, {-60, 20}}), Line(points = {{-80, -20}, {-60, -20}}), Line(points = {{-70, -20}, {-70, -86}}), Line(points = {{0, 40}, {0, -100}}), Line(points = {{-10, 40}, {10, 40}}), Line(points = {{-10, 80}, {10, 80}}), Line(points = {{60, -20}, {80, -20}}), Line(points = {{60, 20}, {80, 20}}), Line(points = {{70, -20}, {70, -86}}), Line(points = {{70, -86}, {-70, -86}}), Rectangle(origin = {-75, 0}, lineColor = {64, 64, 64}, fillColor = {191, 191, 191}, fillPattern = FillPattern.HorizontalCylinder, extent = {{-25, -10}, {25, 10}}), Rectangle(origin = {75, 0}, lineColor = {64, 64, 64}, fillColor = {191, 191, 191}, fillPattern = FillPattern.HorizontalCylinder, extent = {{-25, -10}, {25, 10}}), Rectangle(origin = {-35, -19}, fillColor = {153, 153, 153}, fillPattern = FillPattern.Solid, extent = {{-15, -2}, {15, 2}}), Rectangle(origin = {-35, -8}, fillColor = {204, 204, 204}, fillPattern = FillPattern.Solid, extent = {{-15, -3}, {15, 3}}), Rectangle(origin = {-35, 19}, fillColor = {204, 204, 204}, fillPattern = FillPattern.Solid, extent = {{-15, -2}, {15, 2}}), Rectangle(origin = {-35, 8}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, extent = {{-15, -3}, {15, 3}}), Rectangle(origin = {0, 60}, lineColor = {64, 64, 64}, fillColor = {191, 191, 191}, fillPattern = FillPattern.HorizontalCylinder, extent = {{-20, -10}, {20, 10}}), Rectangle(origin = {-35, 98}, fillColor = {153, 153, 153}, fillPattern = FillPattern.Solid, extent = {{-15, -2}, {15, 2}}), Rectangle(origin = {-35, 87}, fillColor = {204, 204, 204}, fillPattern = FillPattern.Solid, extent = {{-15, -3}, {15, 3}}), Rectangle(origin = {-35, 50}, fillColor = {204, 204, 204}, fillPattern = FillPattern.Solid, extent = {{-15, -4}, {15, 4}}), Rectangle(origin = {-35, 22}, fillColor = {102, 102, 102}, fillPattern = FillPattern.Solid, extent = {{-15, -2}, {15, 2}}), Rectangle(origin = {-35, 33}, fillColor = {153, 153, 153}, fillPattern = FillPattern.Solid, extent = {{-15, -3}, {15, 3}}), Rectangle(origin = {-35, 70}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, extent = {{-15, -4}, {15, 4}}), Rectangle(origin = {35, 60}, fillColor = {255, 255, 255}, fillPattern = FillPattern.HorizontalCylinder, extent = {{-15, -21}, {15, 21}}), Rectangle(origin = {35, 41}, fillColor = {153, 153, 153}, fillPattern = FillPattern.Solid, extent = {{-15, -2}, {15, 2}}), Rectangle(origin = {35, 52}, fillColor = {204, 204, 204}, fillPattern = FillPattern.Solid, extent = {{-15, -3}, {15, 3}}), Rectangle(origin = {35, 79}, fillColor = {204, 204, 204}, fillPattern = FillPattern.Solid, extent = {{-15, -2}, {15, 2}}), Rectangle(origin = {35, 68}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, extent = {{-15, -3}, {15, 3}}), Rectangle(origin = {35, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.HorizontalCylinder, extent = {{-15, -40}, {15, 40}}), Rectangle(origin = {35, 38}, fillColor = {153, 153, 153}, fillPattern = FillPattern.Solid, extent = {{-15, -2}, {15, 2}}), Rectangle(origin = {35, 27}, fillColor = {204, 204, 204}, fillPattern = FillPattern.Solid, extent = {{-15, -3}, {15, 3}}), Rectangle(origin = {35, -10}, fillColor = {204, 204, 204}, fillPattern = FillPattern.Solid, extent = {{-15, -4}, {15, 4}}), Rectangle(origin = {35, -38}, fillColor = {102, 102, 102}, fillPattern = FillPattern.Solid, extent = {{-15, -2}, {15, 2}}), Rectangle(origin = {35, -27}, fillColor = {153, 153, 153}, fillPattern = FillPattern.Solid, extent = {{-15, -3}, {15, 3}}), Rectangle(origin = {35, 10}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, extent = {{-15, -4}, {15, 4}}), Rectangle(origin = {-35, 40}, fillColor = {255, 255, 255}, extent = {{-15, -61}, {15, 60}}), Rectangle(origin = {35, 21}, fillColor = {255, 255, 255}, extent = {{-15, -61}, {15, 60}})}),
            Documentation(info = "<html>
<p>This component is used to model a 1-dim. gearbox
with non-parallel axes (defined by parameters <code>n_a</code>, <code>n_b</code>).
A 3-dim. <code>bearing</code> frame is necessary to reflect the
correct support torque, as the axes of rotation of <code>flange_a</code> and
<code>flange_b</code> and the direction of the support torque vector
are different in general.</p>
<p>Note: The name BevelGear1D is kept only for simplicity. Regardless,
this component could be used to model any kind of gearbox with non-parallel axes.</p>
<p><b>Reference</b><br>
<span style=\"font-variant:small-caps\">Schweiger</span>, Christian ;
<span style=\"font-variant:small-caps\">Otter</span>, Martin:
<a href=\"https://www.modelica.org/events/Conference2003/papers/h06_Schweiger_powertrains_v5.pdf\">Modelling
3D Mechanical Effects of 1-dim. Powertrains</a>. In: <i>Proceedings of the 3rd International
Modelica Conference</i>. Link&ouml;ping : The Modelica Association and Link&ouml;ping University,
November 3-4, 2003, pp. 149-158</p>
</html>"));
        end BevelGear1D;

        model RollingWheel "Ideal rolling wheel on flat surface z=0 (5 positional, 3 velocity degrees of freedom)"
          parameter Boolean animation = true "= true, if animation of wheel shall be enabled";
          parameter SI.Radius wheelRadius "Radius of wheel";
          parameter SI.Mass wheelMass "Mass of wheel";
          parameter SI.Inertia wheel_I_axis "Inertia along the wheel axis";
          parameter SI.Inertia wheel_I_long "Inertia perpendicular to the wheel axis";
          parameter StateSelect stateSelect = StateSelect.always "Priority to use generalized coordinates as states" annotation(
            HideResult = true,
            Evaluate = true);
          SI.Position x(start = 0, fixed = true, stateSelect = stateSelect) "x-coordinate of wheel axis";
          SI.Position y(start = 0, fixed = true, stateSelect = stateSelect) "y-coordinate of wheel axis";
          SI.Angle angles[3](start = {0, 0, 0}, each fixed = true, each stateSelect = stateSelect) "Angles to rotate world-frame in to frame_a around z-, y-, x-axis" annotation(
            Dialog(group = "Initialization", showStartAttribute = true));
          SI.AngularVelocity der_angles[3](start = {0, 0, 0}, each fixed = true, each stateSelect = stateSelect) "Derivative of angles" annotation(
            Dialog(group = "Initialization", showStartAttribute = true));
          parameter SI.Distance width = 0.035 "Width of wheel" annotation(
            Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter Real hollowFraction = 0.8 "1.0: Completely hollow, 0.0: rigid cylinder" annotation(
            Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter Modelica.Mechanics.MultiBody.Types.Color wheelColor = {30, 30, 30} "Color of wheel" annotation(
            Dialog(colorSelector = true, tab = "Animation", group = "if animation = true", enable = animation));
          Modelica.Mechanics.MultiBody.Parts.Body body(final r_CM = {0, 0, 0}, final m = wheelMass, final I_11 = wheel_I_long, final I_22 = wheel_I_axis, final I_33 = wheel_I_long, final I_21 = 0, final I_31 = 0, final I_32 = 0, animation = false) annotation(
            Placement(transformation(extent = {{20, -10}, {40, 10}})));
          Modelica.Mechanics.MultiBody.Interfaces.Frame_a frame_a "Frame fixed in wheel center point (y-axis: along wheel axis, z-axis: upwards)" annotation(
            Placement(transformation(extent = {{-16, -16}, {16, 16}})));
          Modelica.Mechanics.MultiBody.Joints.RollingWheel rollingWheel(wheelRadius = wheelRadius, stateSelect = StateSelect.avoid) annotation(
            Placement(transformation(extent = {{-60, -60}, {-40, -40}})));
          Modelica.Mechanics.MultiBody.Visualizers.FixedShape fixedShape(final animation = animation, final r_shape = {0, -width, 0}, final lengthDirection = {0, 1, 0}, final widthDirection = {1, 0, 0}, final length = 2 * width, final width = 2 * wheelRadius, final height = 2 * wheelRadius, final color = wheelColor, final extra = hollowFraction, final shapeType = "pipe") if animation annotation(
            Placement(transformation(extent = {{20, 20}, {40, 40}})));
        equation
          rollingWheel.x = x;
          rollingWheel.y = y;
          rollingWheel.angles = angles;
          rollingWheel.der_angles = der_angles;
          connect(body.frame_a, frame_a) annotation(
            Line(points = {{20, 0}, {0, 0}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
          connect(rollingWheel.frame_a, frame_a) annotation(
            Line(points = {{-50, -50}, {-25, -50}, {-25, 0}, {0, 0}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
          connect(fixedShape.frame_a, frame_a) annotation(
            Line(points = {{20, 30}, {0, 30}, {0, 0}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
          annotation(
            defaultComponentName = "wheel",
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, -80}, {100, -100}}, lineColor = {0, 0, 0}, fillColor = {175, 175, 175}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-80, 80}, {80, -80}}, lineColor = {0, 0, 0}, fillColor = {215, 215, 215}, fillPattern = FillPattern.Solid), Text(extent = {{-150, 125}, {150, 85}}, textString = "%name", lineColor = {0, 0, 255})}));
        end RollingWheel;

        model RollingWheelSet "Ideal rolling wheel set consisting of two ideal rolling wheels connected together by an axis"
          Modelica.Mechanics.MultiBody.Interfaces.Frame_a frameMiddle "Frame fixed in middle of axis connecting both wheels (y-axis: along wheel axis, z-axis: upwards)" annotation(
            Placement(transformation(extent = {{-16, 16}, {16, -16}}), iconTransformation(extent = {{-16, -16}, {16, 16}})));
          parameter Boolean animation = true "= true, if animation of wheel set shall be enabled";
          parameter SI.Radius wheelRadius "Radius of one wheel";
          parameter SI.Mass wheelMass "Mass of one wheel";
          parameter SI.Inertia wheel_I_axis "Inertia along one wheel axis";
          parameter SI.Inertia wheel_I_long "Inertia perpendicular to one wheel axis";
          parameter SI.Distance wheelDistance "Distance between the two wheels";
          parameter StateSelect stateSelect = StateSelect.always "Priority to use the generalized coordinates as states";
          Modelica.SIunits.Position x(start = 0, fixed = true, stateSelect = stateSelect) "x coordinate of center between wheels";
          Modelica.SIunits.Position y(start = 0, fixed = true, stateSelect = stateSelect) "y coordinate of center between wheels";
          Modelica.SIunits.Angle phi(start = 0, fixed = true, stateSelect = stateSelect) "Orientation angle of wheel axis along z-axis";
          Modelica.SIunits.Angle theta1(start = 0, fixed = true, stateSelect = stateSelect) "Angle of wheel 1";
          Modelica.SIunits.Angle theta2(start = 0, fixed = true, stateSelect = stateSelect) "Angle of wheel 2";
          Modelica.SIunits.AngularVelocity der_theta1(start = 0, fixed = true, stateSelect = stateSelect) "Derivative of theta 1";
          Modelica.SIunits.AngularVelocity der_theta2(start = 0, fixed = true, stateSelect = stateSelect) "Derivative of theta 2";
          parameter SI.Distance wheelWidth = 0.01 "Width of one wheel" annotation(
            Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter Real hollowFraction = 0.8 "1.0: Completely hollow, 0.0: rigid cylinder" annotation(
            Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter Modelica.Mechanics.MultiBody.Types.Color wheelColor = {30, 30, 30} "Color of wheels" annotation(
            Dialog(colorSelector = true, tab = "Animation", group = "if animation = true", enable = animation));
          Modelica.Mechanics.MultiBody.Interfaces.Frame_a frame1 "Frame fixed in center point of left wheel (y-axis: along wheel axis, z-axis: upwards)" annotation(
            Placement(transformation(extent = {{-96, 16}, {-64, -16}}), iconTransformation(extent = {{-96, 16}, {-64, -16}})));
          Modelica.Mechanics.MultiBody.Interfaces.Frame_b frame2 "Frame fixed in center point of right wheel (y-axis: along wheel axis, z-axis: upwards)" annotation(
            Placement(transformation(extent = {{64, 16}, {96, -16}})));
          Modelica.Mechanics.MultiBody.Parts.Body body2(final r_CM = {0, 0, 0}, final I_21 = 0, final I_31 = 0, final I_32 = 0, animation = false, final m = wheelMass, final I_11 = wheel_I_long, final I_22 = wheel_I_axis, final I_33 = wheel_I_long) annotation(
            Placement(transformation(extent = {{10, -10}, {-10, 10}}, rotation = -90, origin = {60, 30})));
          Modelica.Mechanics.MultiBody.Visualizers.FixedShape shape2(final animation = animation, final lengthDirection = {0, 1, 0}, final widthDirection = {1, 0, 0}, final color = wheelColor, final extra = hollowFraction, final shapeType = "pipe", final r_shape = {0, -wheelWidth, 0}, final length = 2 * wheelWidth, final width = 2 * wheelRadius, final height = 2 * wheelRadius) if animation annotation(
            Placement(transformation(extent = {{10, -10}, {-10, 10}}, rotation = 90, origin = {60, -38})));
          Modelica.Mechanics.MultiBody.Parts.Body body1(final r_CM = {0, 0, 0}, final I_21 = 0, final I_31 = 0, final I_32 = 0, animation = false, final m = wheelMass, final I_11 = wheel_I_long, final I_22 = wheel_I_axis, final I_33 = wheel_I_long) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 90, origin = {-60, 30})));
          Modelica.Mechanics.MultiBody.Visualizers.FixedShape shape1(final animation = animation, final lengthDirection = {0, 1, 0}, final widthDirection = {1, 0, 0}, final color = wheelColor, final extra = hollowFraction, final shapeType = "pipe", final r_shape = {0, -wheelWidth, 0}, final length = 2 * wheelWidth, final width = 2 * wheelRadius, final height = 2 * wheelRadius) if animation annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = -90, origin = {-60, -40})));
          Modelica.Mechanics.Rotational.Interfaces.Flange_a axis1 "1-dim. rotational flange that drives the left wheel" annotation(
            Placement(transformation(extent = {{-110, 90}, {-90, 110}})));
          Modelica.Mechanics.Rotational.Interfaces.Flange_a axis2 "1-dim. rotational flange that drives the right wheel" annotation(
            Placement(transformation(extent = {{90, 90}, {110, 110}})));
          Modelica.Mechanics.MultiBody.Joints.RollingWheelSet wheelSetJoint(animation = false, wheelRadius = wheelRadius, wheelDistance = wheelDistance, stateSelect = StateSelect.default, x(fixed = false), y(fixed = false), phi(fixed = false), theta1(fixed = false), theta2(fixed = false), der_theta1(fixed = false), der_theta2(fixed = false)) annotation(
            Placement(transformation(extent = {{-10, -42}, {10, -22}})));
          Modelica.Mechanics.Rotational.Interfaces.Flange_b support "Support of 1D axes" annotation(
            Placement(transformation(extent = {{-10, 70}, {10, 90}}), iconTransformation(extent = {{-10, 72}, {10, 92}})));
        equation
          wheelSetJoint.x = x;
          wheelSetJoint.y = y;
          wheelSetJoint.phi = phi;
          wheelSetJoint.theta1 = theta1;
          wheelSetJoint.theta2 = theta2;
          der_theta1 = der(theta1);
          der_theta2 = der(theta2);
          connect(body2.frame_a, frame2) annotation(
            Line(points = {{60, 20}, {60, 0}, {80, 0}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
          connect(body1.frame_a, frame1) annotation(
            Line(points = {{-60, 20}, {-60, 0}, {-80, 0}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
          connect(shape1.frame_a, frame1) annotation(
            Line(points = {{-60, -30}, {-60, 0}, {-80, 0}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
          connect(shape2.frame_a, frame2) annotation(
            Line(points = {{60, -28}, {60, 0}, {80, 0}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
          connect(wheelSetJoint.frame2, frame2) annotation(
            Line(points = {{8, -32}, {40, -32}, {40, 0}, {80, 0}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
          connect(wheelSetJoint.frame1, frame1) annotation(
            Line(points = {{-8, -32}, {-40, -32}, {-40, 0}, {-80, 0}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
          connect(wheelSetJoint.axis1, axis1) annotation(
            Line(points = {{-10, -22}, {-20, -22}, {-20, 50}, {-80, 50}, {-80, 100}, {-100, 100}}, color = {0, 0, 0}, smooth = Smooth.None));
          connect(wheelSetJoint.axis2, axis2) annotation(
            Line(points = {{10, -22}, {24, -22}, {24, 50}, {80, 50}, {80, 100}, {100, 100}}, color = {0, 0, 0}, smooth = Smooth.None));
          connect(wheelSetJoint.support, support) annotation(
            Line(points = {{0, -24}, {0, -14}, {16, -14}, {16, 58}, {0, 58}, {0, 80}}, color = {0, 0, 0}, smooth = Smooth.None));
          connect(wheelSetJoint.frameMiddle, frameMiddle) annotation(
            Line(points = {{0, -32}, {-4, -32}, {-4, 0}, {0, 0}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
          annotation(
            defaultComponentName = "wheelSet",
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, -80}, {100, -100}}, lineColor = {0, 0, 0}, fillColor = {175, 175, 175}, fillPattern = FillPattern.Solid), Text(extent = {{-150, -105}, {150, -145}}, textString = "%name", lineColor = {0, 0, 255}), Ellipse(extent = {{42, 80}, {118, -80}}, lineColor = {0, 0, 0}, fillColor = {215, 215, 215}, fillPattern = FillPattern.Sphere), Rectangle(extent = {{-62, 2}, {64, -6}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-118, 80}, {-42, -80}}, lineColor = {0, 0, 0}, fillColor = {215, 215, 215}, fillPattern = FillPattern.Sphere), Line(points = {{86, 24}, {64, 24}, {64, 10}, {56, 10}}, color = {0, 0, 0}, smooth = Smooth.None), Line(points = {{86, -24}, {64, -24}, {64, -12}, {56, -12}}, color = {0, 0, 0}, smooth = Smooth.None), Line(points = {{-96, 100}, {-80, 100}, {-80, 4}}, color = {0, 0, 0}, smooth = Smooth.None), Line(points = {{100, 100}, {80, 100}, {80, -2}}, color = {0, 0, 0}, smooth = Smooth.None), Line(points = {{0, 72}, {0, 40}, {-20, 40}, {-20, 2}}, color = {0, 0, 0}, smooth = Smooth.None), Line(points = {{0, 40}, {20, 40}, {20, 2}}, color = {0, 0, 0}, smooth = Smooth.None)}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{0, -106}, {0, -78}}, color = {0, 0, 255}, smooth = Smooth.None), Polygon(points = {{0, -60}, {-6, -78}, {6, -78}, {0, -60}}, lineColor = {0, 0, 255}, smooth = Smooth.None, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Text(extent = {{12, -68}, {30, -80}}, lineColor = {0, 0, 255}, textString = "x"), Line(points = {{6, -100}, {-26, -100}}, color = {0, 0, 255}, smooth = Smooth.None), Polygon(points = {{-22, -94}, {-22, -106}, {-40, -100}, {-22, -94}}, lineColor = {0, 0, 255}, smooth = Smooth.None, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-46, -80}, {-28, -92}}, lineColor = {0, 0, 255}, textString = "y")}),
            Documentation(info = "<html>
<p>
Two wheels are connected by an axis and can rotate around this axis.
The wheels are rolling on the x-y plane. The coordinate system attached
to the center of the wheel axis (frameMiddle) is constrained so that it
is always parallel to the x-y plane. If all generalized coordinates are zero,
frameMiddle is parallel to the world frame.
</p>
</html>"));
        end RollingWheelSet;
        annotation(
          Documentation(info = "<HTML>
<p>
Package <b>Parts</b> contains <b>rigid components</b> of a
multi-body system. These components may be used to build up
more complicated structures. For example, a part may be built up of
a \"Body\" and of several \"FixedTranslation\" components.
</p>
<h4>Content</h4>
<table border=1 cellspacing=0 cellpadding=2>
  <tr><th><b><i>Model</i></b></th><th><b><i>Description</i></b></th></tr>
  <tr><td valign=\"top\"><a href=\"modelica://Modelica.Mechanics.MultiBody.Parts.Fixed\">Fixed</a></td>
      <td valign=\"top\">Frame fixed in world frame at a given position.
          It is visualized with a shape, see <b>shapeType</b> below
         (the frames on the two
          sides do not belong to the component):<br>&nbsp;<br>
      <IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Fixed.png\" ALT=\"model Parts.Fixed\">
      </td>
  </tr>
  <tr><td valign=\"top\"><a href=\"modelica://Modelica.Mechanics.MultiBody.Parts.FixedTranslation\">FixedTranslation</a></td>
      <td valign=\"top\">Fixed translation of frame_b with respect to frame_a.
          It is visualized with a shape, see <b>shapeType</b> below
          (the frames on the two sides do not belong to the component):<br>&nbsp;<br>
      <IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/FixedTranslation.png\" ALT=\"model Parts.FixedTranslation\">
      </td>
  </tr>
  <tr><td valign=\"top\"><a href=\"modelica://Modelica.Mechanics.MultiBody.Parts.FixedRotation\">FixedRotation</a></td>
      <td valign=\"top\">Fixed translation and fixed rotation of frame_b with respect to frame_a
          It is visualized with a shape, see <b>shapeType</b>  below
          (the frames on the two sides do not belong to the component):<br>&nbsp;<br>
      <IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/FixedRotation.png\" ALT=\"model Parts.FixedRotation\">
      </td>
  </tr>
  <tr><td valign=\"top\"><a href=\"modelica://Modelica.Mechanics.MultiBody.Parts.Body\">Body</a></td>
      <td valign=\"top\">Rigid body with mass, inertia tensor and one frame connector.
          It is visualized with a cylinder and a sphere at the
          center of mass:<br>&nbsp;<br>
      <IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Body.png\" ALT=\"model Parts.Body\">
      </td>
  </tr>
  <tr><td valign=\"top\"><a href=\"modelica://Modelica.Mechanics.MultiBody.Parts.BodyShape\">BodyShape</a></td>
      <td valign=\"top\">Rigid body with mass, inertia tensor, different shapes
          (see <b>shapeType</b> below)
          for animation, and two frame connectors:<br>&nbsp;<br>
      <IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/BodyShape.png\" ALT=\"model Parts.BodyShape\">
      </td>
  </tr>
  <tr><td valign=\"top\"><a href=\"modelica://Modelica.Mechanics.MultiBody.Parts.Fixed\">Fixed BodyBox</a></td>
      <td valign=\"top\">Rigid body with box shape (mass and animation properties are computed
          from box data and from density):<br>&nbsp;<br>
      <IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/BodyBox.png\" ALT=\"model Parts.BodyBox\">
      </td>
  </tr>
  <tr><td valign=\"top\"><a href=\"modelica://Modelica.Mechanics.MultiBody.Parts.BodyCylinder\">BodyCylinder</a></td>
      <td valign=\"top\">Rigid body with cylinder shape (mass and animation properties
          are computed from cylinder data and from density):<br>&nbsp;<br>
      <IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/BodyCylinder.png\" ALT=\"model Parts.BodyCylinder\">
      </td>
  </tr>
  <tr><td valign=\"top\"><a href=\"modelica://Modelica.Mechanics.MultiBody.Parts.PointMass\">PointMass</a></td>
      <td valign=\"top\">Rigid body where inertia tensor and rotation is neglected:<br>&nbsp;<br>
      <IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Parts/PointMass.png\" ALT=\"model Parts.PointMass\">
      </td>
  </tr>
  <tr><td valign=\"top\"><a href=\"modelica://Modelica.Mechanics.MultiBody.Parts.Mounting1D\">Mounting1D</a></td>
      <td valign=\"top\"> Propagate 1-dim. support torque to 3-dim. system
      </td>
  </tr>
  <tr><td valign=\"top\"><a href=\"modelica://Modelica.Mechanics.MultiBody.Parts.Rotor1D\">Rotor1D</a></td>
      <td valign=\"top\">1D inertia attachable on 3-dim. bodies (without neglecting dynamic effects)<br>
      <IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Parts/Rotor1D.png\" ALT=\"model Parts.Rotor1D\">
      </td>
  </tr>
  <tr><td valign=\"top\"><a href=\"modelica://Modelica.Mechanics.MultiBody.Parts.BevelGear1D\">BevelGear1D</a></td>
      <td valign=\"top\">1D gearbox with arbitrary shaft directions (3D bearing frame)
      </td>
  </tr>
</table>
<p>
Components <b>Fixed</b>, <b>FixedTranslation</b>, <b>FixedRotation</b>
and <b>BodyShape</b> are visualized according to parameter
<b>shapeType</b>, that may have the following values (e.g., shapeType = \"box\"): <br>&nbsp;<br>
</p>
<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/FixedShape.png\" ALT=\"model Visualizers.FixedShape\">
<p>
All the details of the visualization shape parameters are
given in
<a href=\"modelica://Modelica.Mechanics.MultiBody.Visualizers.FixedShape\">Visualizers.FixedShape</a>
</p>
<p>
Colors in all animation parts are defined via parameter <b>color</b>.
This is an Integer vector with 3 elements, {r, g, b}, and specifies the
color of the shape. {r,g,b} are the \"red\", \"green\" and \"blue\" color parts,
given in the ranges 0 .. 255, respectively. The predefined type
<b>MultiBody.Types.Color</b> contains a menu
definition of the colors used in the MultiBody library
(this will be replaced by a color editor).
</p>
</html>"),
          Icon(graphics = {Rectangle(extent = {{-80, 28}, {2, -16}}, lineColor = {95, 95, 95}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {215, 215, 215}, radius = 10), Ellipse(extent = {{-8, 52}, {86, -42}}, lineColor = {95, 95, 95}, fillPattern = FillPattern.Sphere, fillColor = {215, 215, 215})}));
      end Parts;

      package Sensors "Sensors to measure variables"
        extends Modelica.Icons.SensorsPackage;

        model AbsoluteSensor "Measure absolute kinematic quantities of frame connector"
          Blocks.Interfaces.RealOutput r[3](each final quantity = "Length", each final unit = "m") if get_r "Absolute position vector frame_a.r_0 resolved in frame defined by resolveInFrame" annotation(
            Placement(transformation(origin = {-100, -110}, extent = {{10, -10}, {-10, 10}}, rotation = 90)));
          Blocks.Interfaces.RealOutput v[3](each final quantity = "Velocity", each final unit = "m/s") if get_v "Absolute velocity vector" annotation(
            Placement(transformation(origin = {-60, -110}, extent = {{10, -10}, {-10, 10}}, rotation = 90)));
          Blocks.Interfaces.RealOutput a[3](each final quantity = "Acceleration", each final unit = "m/s2") if get_a "Absolute acceleration vector" annotation(
            Placement(transformation(origin = {-20, -110}, extent = {{10, -10}, {-10, 10}}, rotation = 90)));
          Blocks.Interfaces.RealOutput angles[3](each final quantity = "Angle", each final unit = "rad", each displayUnit = "deg") if get_angles "Angles to rotate world frame into frame_a via 'sequence'" annotation(
            Placement(transformation(origin = {20, -110}, extent = {{10, -10}, {-10, 10}}, rotation = 90)));
          Blocks.Interfaces.RealOutput w[3](each final quantity = "AngularVelocity", each final unit = "1/s") if get_w "Absolute angular velocity vector" annotation(
            Placement(transformation(origin = {60, -110}, extent = {{10, -10}, {-10, 10}}, rotation = 90)));
          Blocks.Interfaces.RealOutput z[3](each final quantity = "AngularAcceleration", each final unit = "1/s2") if get_z "Absolute angular acceleration vector" annotation(
            Placement(transformation(origin = {100, -110}, extent = {{10, -10}, {-10, 10}}, rotation = 90)));
          extends Modelica.Mechanics.MultiBody.Sensors.Internal.PartialAbsoluteSensor;
          Interfaces.Frame_resolve frame_resolve if resolveInFrame == Modelica.Mechanics.MultiBody.Types.ResolveInFrameA.frame_resolve "If resolveInFrame = Types.ResolveInFrameA.frame_resolve, the output signals are resolved in this frame" annotation(
            Placement(transformation(extent = {{84, -16}, {116, 16}}), iconTransformation(extent = {{84, -16}, {116, 16}})));
          parameter Boolean animation = true "= true, if animation shall be enabled (show arrow)";
          parameter Modelica.Mechanics.MultiBody.Types.ResolveInFrameA resolveInFrame = Modelica.Mechanics.MultiBody.Types.ResolveInFrameA.frame_a "Frame in which vectors are resolved (world, frame_a, or frame_resolve)";
          parameter Boolean get_r = false "= true, to measure the absolute position vector of the origin of frame_a" annotation(
            HideResult = true,
            choices(checkBox = true));
          parameter Boolean get_v = false "= true, to measure the absolute velocity of the origin of frame_a" annotation(
            HideResult = true,
            choices(checkBox = true));
          parameter Boolean get_a = false "= true, to measure the absolute acceleration of the origin of frame_a" annotation(
            HideResult = true,
            choices(checkBox = true));
          parameter Boolean get_w = false "= true, to measure the absolute angular velocity of frame_a" annotation(
            HideResult = true,
            choices(checkBox = true));
          parameter Boolean get_z = false "= true, to measure the absolute angular acceleration of frame_a" annotation(
            HideResult = true,
            choices(checkBox = true));
          parameter Boolean get_angles = false "= true, to measure the 3 rotation angles" annotation(
            HideResult = true,
            choices(checkBox = true),
            Dialog(group = "3 angles to rotate the world frame into frame_a along the axes defined in \"sequence\""));
          parameter Types.RotationSequence sequence(min = {1, 1, 1}, max = {3, 3, 3}) = {1, 2, 3} "If get_angles=true: Angles are returned to rotate world frame around axes sequence[1], sequence[2] and finally sequence[3] into frame_a" annotation(
            HideResult = true,
            Evaluate = true,
            Dialog(group = "3 angles to rotate the world frame into frame_a along the axes defined in \"sequence\"", enable = get_angles));
          parameter SI.Angle guessAngle1 = 0 "If get_angles=true: Select angles[1] such that abs(angles[1] - guessAngle1) is a minimum" annotation(
            HideResult = true,
            Dialog(group = "3 angles to rotate the world frame into frame_a along the axes defined in \"sequence\"", enable = get_angles));
          input SI.Diameter arrowDiameter = world.defaultArrowDiameter "Diameter of absolute arrow from world frame to frame_a" annotation(
            Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          input Types.Color arrowColor = Modelica.Mechanics.MultiBody.Types.Defaults.SensorColor "Color of absolute arrow from world frame to frame_b" annotation(
            Dialog(colorSelector = true, tab = "Animation", group = "if animation = true", enable = animation));
          input Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(
            Dialog(tab = "Animation", group = "if animation = true", enable = animation));
        protected
          AbsolutePosition position(resolveInFrame = resolveInFrame) if get_r annotation(
            Placement(transformation(extent = {{10, 10}, {-10, 30}}, rotation = 90, origin = {-80, -60})));
        protected
          AbsoluteVelocity velocity(resolveInFrame = resolveInFrame) if get_v annotation(
            Placement(transformation(extent = {{10, -10}, {-10, 10}}, rotation = 90, origin = {-60, -60})));
          Modelica.Mechanics.MultiBody.Sensors.AbsoluteAngles absoluteAngles(sequence = sequence, guessAngle1 = guessAngle1) if get_angles annotation(
            Placement(transformation(extent = {{-10, 10}, {10, -10}}, rotation = -90, origin = {20, -60})));
          AbsoluteAngularVelocity angularVelocity(resolveInFrame = resolveInFrame) if get_w annotation(
            Placement(transformation(extent = {{-10, -10}, {10, -30}}, rotation = -90, origin = {70, -60})));
        protected
          Blocks.Continuous.Der der1[3] if get_a annotation(
            Placement(transformation(extent = {{-6, -6}, {6, 6}}, rotation = -90, origin = {-20, -34})));
        protected
          Blocks.Continuous.Der der2[3] if get_z annotation(
            Placement(transformation(extent = {{-6, -6}, {6, 6}}, rotation = -90, origin = {100, -46})));
        protected
          Modelica.Mechanics.MultiBody.Sensors.TransformAbsoluteVector transformVector_a(frame_r_in = Modelica.Mechanics.MultiBody.Types.ResolveInFrameA.world, frame_r_out = resolveInFrame) if get_a annotation(
            Placement(transformation(extent = {{-10, -70}, {-30, -50}})));
          Modelica.Mechanics.MultiBody.Sensors.TransformAbsoluteVector transformVector_z(frame_r_in = Modelica.Mechanics.MultiBody.Types.ResolveInFrameA.world, frame_r_out = resolveInFrame) if get_z annotation(
            Placement(transformation(extent = {{90, -80}, {110, -60}})));
        protected
          outer Modelica.Mechanics.MultiBody.World world;
          Modelica.Mechanics.MultiBody.Visualizers.Advanced.Arrow arrow(r_head = frame_a.r_0, diameter = arrowDiameter, color = arrowColor, specularCoefficient) if world.enableAnimation and animation;
        protected
          AbsoluteVelocity absoluteVelocity(resolveInFrame = Modelica.Mechanics.MultiBody.Types.ResolveInFrameA.world) if get_a annotation(
            Placement(transformation(extent = {{10, -10}, {-10, 10}}, rotation = 90, origin = {-20, -12})));
          AbsoluteAngularVelocity absoluteAngularVelocity(resolveInFrame = Modelica.Mechanics.MultiBody.Types.ResolveInFrameA.world) if get_z annotation(
            Placement(transformation(extent = {{40, -10}, {60, 10}}, rotation = 0)));
          Internal.ZeroForceAndTorque zeroForce1 annotation(
            Placement(transformation(extent = {{-60, 20}, {-40, 40}})));
          Internal.ZeroForceAndTorque zeroForce2 if resolveInFrame == Modelica.Mechanics.MultiBody.Types.ResolveInFrameA.frame_resolve annotation(
            Placement(transformation(extent = {{80, 20}, {60, 40}})));
        equation
          connect(zeroForce1.frame_a, frame_a) annotation(
            Line(points = {{-60, 30}, {-80, 30}, {-80, 0}, {-100, 0}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
          connect(absoluteAngles.angles, angles) annotation(
            Line(points = {{20, -71}, {20, -90}, {20, -90}, {20, -110}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(angularVelocity.frame_a, frame_a) annotation(
            Line(points = {{50, -50}, {50, -40}, {20, -40}, {20, 0}, {-100, 0}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
          connect(angularVelocity.w, w) annotation(
            Line(points = {{50, -71}, {50, -94}, {60, -94}, {60, -110}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(frame_resolve, position.frame_resolve) annotation(
            Line(points = {{100, 0}, {114, 0}, {114, -90}, {-80, -90}, {-80, -60}, {-89.9, -60}}, color = {95, 95, 95}, pattern = LinePattern.Dot, smooth = Smooth.None));
          connect(frame_resolve, zeroForce2.frame_a) annotation(
            Line(points = {{100, 0}, {90, 0}, {90, 30}, {80, 30}}, color = {95, 95, 95}, pattern = LinePattern.Dot, smooth = Smooth.None));
          connect(angularVelocity.frame_resolve, frame_resolve) annotation(
            Line(points = {{60.1, -60}, {66, -60}, {66, -90}, {114, -90}, {114, 0}, {100, 0}}, color = {95, 95, 95}, pattern = LinePattern.Dot, smooth = Smooth.None));
          connect(transformVector_a.frame_a, frame_a) annotation(
            Line(points = {{-10, -60}, {0, -60}, {0, -40}, {20, -40}, {20, 0}, {-100, 0}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
          connect(transformVector_a.frame_resolve, frame_resolve) annotation(
            Line(points = {{-30, -59.9}, {-40, -59.9}, {-40, -90}, {114, -90}, {114, 0}, {100, 0}}, color = {95, 95, 95}, pattern = LinePattern.Dot, smooth = Smooth.None));
          connect(transformVector_a.r_out, a) annotation(
            Line(points = {{-20, -71}, {-20, -110}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(der2.y, transformVector_z.r_in) annotation(
            Line(points = {{100, -52.6}, {100, -58}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(transformVector_z.r_out, z) annotation(
            Line(points = {{100, -81}, {100, -110}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(transformVector_z.frame_a, frame_a) annotation(
            Line(points = {{90, -70}, {80, -70}, {80, -40}, {20, -40}, {20, 0}, {-100, 0}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
          connect(transformVector_z.frame_resolve, frame_resolve) annotation(
            Line(points = {{110, -69.9}, {110, -70}, {114, -70}, {114, 0}, {100, 0}}, color = {95, 95, 95}, pattern = LinePattern.Dot, smooth = Smooth.None));
          connect(frame_a, position.frame_a) annotation(
            Line(points = {{-100, 0}, {-80, 0}, {-80, -40}, {-100, -40}, {-100, -50}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
          connect(absoluteAngles.frame_a, frame_a) annotation(
            Line(points = {{20, -50}, {20, 0}, {-100, 0}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
          connect(position.r, r) annotation(
            Line(points = {{-100, -71}, {-100, -110}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(velocity.frame_a, frame_a) annotation(
            Line(points = {{-60, -50}, {-60, -40}, {-80, -40}, {-80, 0}, {-100, 0}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
          connect(velocity.frame_resolve, frame_resolve) annotation(
            Line(points = {{-50, -60}, {-40, -60}, {-40, -90}, {114, -90}, {114, 0}, {100, 0}}, color = {95, 95, 95}, pattern = LinePattern.Dot, smooth = Smooth.None));
          connect(velocity.v, v) annotation(
            Line(points = {{-60, -71}, {-60, -110}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(der1.y, transformVector_a.r_in) annotation(
            Line(points = {{-20, -40.6}, {-20, -48}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(absoluteVelocity.v, der1.u) annotation(
            Line(points = {{-20, -23}, {-20, -26.8}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(absoluteVelocity.frame_a, frame_a) annotation(
            Line(points = {{-20, -2}, {-20, 0}, {-100, 0}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
          connect(absoluteAngularVelocity.frame_a, frame_a) annotation(
            Line(points = {{40, 0}, {-100, 0}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
          connect(absoluteAngularVelocity.w, der2.u) annotation(
            Line(points = {{61, 0}, {80, 0}, {80, -30}, {100, -30}, {100, -38.8}}, color = {0, 0, 127}, smooth = Smooth.None));
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(visible = get_r, points = {{-84, 0}, {-84, -60}, {-100, -60}, {-100, -100}}, color = {0, 0, 127}, smooth = Smooth.None), Line(visible = get_a, points = {{-20, -67}, {-20, -100}}, color = {0, 0, 127}, smooth = Smooth.None), Line(visible = get_w, points = {{60, -36}, {60, -100}}, color = {0, 0, 127}, smooth = Smooth.None), Line(visible = get_z, points = {{86, 0}, {86, -60}, {100, -60}, {100, -100}}, color = {0, 0, 127}, smooth = Smooth.None), Line(visible = get_v, points = {{-60, -36}, {-60, -100}}, color = {0, 0, 127}, smooth = Smooth.None), Line(visible = get_angles, points = {{20, -67}, {20, -100}}, color = {0, 0, 127}, smooth = Smooth.None), Line(points = {{95, 0}, {95, 0}, {70, 0}}, color = {0, 0, 0}, pattern = LinePattern.Dot, smooth = Smooth.None), Text(extent = {{-132, 76}, {129, 124}}, textString = "%name", lineColor = {0, 0, 255}), Text(visible = get_r, extent = {{-130, -74}, {-95, -90}}, lineColor = {0, 0, 0}, textString = "r"), Text(visible = get_v, extent = {{-95, -74}, {-60, -90}}, lineColor = {0, 0, 0}, textString = "v"), Text(visible = get_a, extent = {{-55, -74}, {-20, -90}}, lineColor = {0, 0, 0}, textString = "a"), Text(visible = get_angles, extent = {{-71, -35}, {96, -54}}, lineColor = {0, 0, 0}, textString = "angles"), Text(visible = get_w, extent = {{59, -68}, {105, -85}}, lineColor = {0, 0, 0}, textString = "w"), Text(visible = get_z, extent = {{107, -68}, {153, -84}}, lineColor = {0, 0, 0}, textString = "z"), Text(extent = {{60, 52}, {191, 27}}, lineColor = {95, 95, 95}, textString = "resolve")}),
            Documentation(info = "<html>
<p>
Absolute kinematic quantities of frame_a are
determined and provided at the conditional output signal connectors.
For example, if parameter \"get_r = <b>true</b>\", the connector
\"r\" is enabled and contains the absolute vector from the world frame
to the origin of frame_a. The following quantities can be provided
as output signals:
</p>

<ol>
<li> Absolute position vector (= r) </li>
<li> Absolute velocity vector (= v)</li>
<li> Absolute acceleration vector (= a)</li>
<li> Three angles to rotate world frame into frame_a (= angles)</li>
<li> Absolute angular velocity vector (= w)</li>
<li> Absolute angular acceleration vector (= z)</li>
</ol>

<p>
Via parameter <b>resolveInFrame</b> it is defined, in which frame
a vector is resolved:
</p>

<table border=1 cellspacing=0 cellpadding=2>
<tr><th><b>resolveInFrame =<br>Types.ResolveInFrameA.</b></th><th><b>Meaning</b></th></tr>
<tr><td valign=\"top\">world</td>
    <td valign=\"top\">Resolve vectors in world frame</td></tr>

<tr><td valign=\"top\">frame_a</td>
    <td valign=\"top\">Resolve vectors in frame_a</td></tr>

<tr><td valign=\"top\">frame_resolve</td>
    <td valign=\"top\">Resolve vectors in frame_resolve</td></tr>
</table>

<p>
If resolveInFrame = Types.ResolveInFrameA.frame_resolve, the conditional connector
\"frame_resolve\" is enabled and the vectors are resolved in the frame, to
which frame_resolve is connected. Note, if this connector is enabled, it must
be connected.
</p>

<p>
In the following figure the animation of an AbsoluteSensor
component is shown. The light blue coordinate system is
frame_a and the yellow arrow is the animated sensor.
</p>

<p>
<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Sensors/AbsoluteSensor.png\">
</p>

<p>
Velocity, acceleration, angular velocity and angular acceleration are
determined by differentiating them in the world frame and then transforming
them in to the frame defined by <b>resolveInFrame</b>.
</p>
<p>
For example, if resolveInFrame = <b>Types.ResolveInFrameA.frame_a</b>, then
</p>
<pre>
   v0 = <b>der</b>(frame_a.r0);
   v  = resolve2(frame_a.R, v0);
</pre>
<p>
is returned, i.e., the derivative of the absolute distance from the
world frame to the origin of frame_a, resolved in frame_a.
</p>

<p>
The cut-force and the cut-torque in frame_resolve are
always zero, whether frame_resolve is connected or not.
</p>

<p>
If <b>get_angles</b> = <b>true</b>, the 3 angles to rotate the world
frame into frame_a along the axes defined by parameter <b>sequence</b>
are returned. For example, if sequence = {3,1,2} then the world frame is
rotated around angles[1] along the z-axis, afterwards it is rotated
around angles[2] along the x-axis, and finally it is rotated around
angles[3] along the y-axis and is then identical to frame_a.
The 3 angles are returned in the range
</p>
<pre>
    -<font face=\"Symbol\">p</font> &lt;= angles[i] &lt;= <font face=\"Symbol\">p</font>
</pre>
<p>
There are <b>two solutions</b> for \"angles[1]\" in this range.
Via parameter <b>guessAngle1</b> (default = 0) the
returned solution is selected such that |angles[1] - guessAngle1| is
minimal. The absolute transformation matrix of frame_a
may be in a singular configuration with respect to \"sequence\", i.e.,
there is an infinite number of angle values leading to the same absolute
transformation matrix. In this case, the returned solution is
selected by setting angles[1] = guessAngle1. Then angles[2]
and angles[3] can be uniquely determined in the above range.
</p>
<p>
The parameter <b>sequence</b> has the restriction that
only values 1,2,3 can be used and that sequence[1] &ne; sequence[2]
and sequence[2] &ne; sequence[3]. Often used values are:
</p>
<pre>
  sequence = <b>{1,2,3}</b>  // Cardan or Tait-Bryan angle sequence
           = <b>{3,1,3}</b>  // Euler angle sequence
           = <b>{3,2,1}</b>
</pre>
</html>"));
        end AbsoluteSensor;

        model RelativeSensor "Measure relative kinematic quantities between two frame connectors"
          extends Modelica.Mechanics.MultiBody.Sensors.Internal.PartialRelativeSensor;
          Interfaces.Frame_resolve frame_resolve if resolveInFrame == Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB.frame_resolve or resolveInFrameAfterDifferentiation == Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB.frame_resolve "If resolveInFrame = Types.ResolveInFrameAB.frame_resolve, the output signals are resolved in this frame" annotation(
            Placement(transformation(extent = {{84, 64}, {116, 96}}), iconTransformation(extent = {{84, 64}, {116, 96}})));
          parameter Boolean animation = true "= true, if animation shall be enabled (show arrow)";
          parameter Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB resolveInFrame = Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB.frame_a "Frame in which vectors are resolved before differentiation (world, frame_a, frame_b, or frame_resolve)";
          parameter Boolean get_r_rel = false "= true, to measure the relative position vector from the origin of frame_a to frame_b" annotation(
            HideResult = true,
            choices(checkBox = true));
          parameter Boolean get_v_rel = false "= true, to measure the relative velocity of the origin of frame_b with respect to frame_a" annotation(
            HideResult = true,
            choices(checkBox = true));
          parameter Boolean get_a_rel = false "= true, to measure the relative acceleration of the origin of frame_b with respect to frame_a" annotation(
            HideResult = true,
            choices(checkBox = true));
          parameter Boolean get_w_rel = false "= true, to measure the relative angular velocity of frame_b with respect to frame_a" annotation(
            HideResult = true,
            choices(checkBox = true));
          parameter Boolean get_z_rel = false "= true, to measure the relative angular acceleration of frame_b with respect to frame_a" annotation(
            HideResult = true,
            choices(checkBox = true));
          parameter Boolean get_angles = false "= true, to measure the 3 rotation angles" annotation(
            HideResult = true,
            choices(checkBox = true),
            Dialog(group = "3 angles to rotate frame_a into frame_b along the axes defined in \"sequence\""));
          parameter Types.RotationSequence sequence(min = {1, 1, 1}, max = {3, 3, 3}) = {1, 2, 3} "If get_angles=true: Angles are returned to rotate frame_a around axes sequence[1], sequence[2] and finally sequence[3] into frame_b" annotation(
            HideResult = true,
            Evaluate = true,
            Dialog(group = "3 angles to rotate frame_a into frame_b along the axes defined in \"sequence\"", enable = get_angles));
          parameter SI.Angle guessAngle1 = 0 "If get_angles=true: Select angles[1] such that abs(angles[1] - guessAngle1) is a minimum" annotation(
            HideResult = true,
            Dialog(group = "3 angles to rotate frame_a into frame_b along the axes defined in \"sequence\"", enable = get_angles));
          input SI.Diameter arrowDiameter = world.defaultArrowDiameter "Diameter of relative arrow from frame_a to frame_b" annotation(
            Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          input Types.Color arrowColor = Modelica.Mechanics.MultiBody.Types.Defaults.SensorColor "Color of relative arrow from frame_a to frame_b" annotation(
            Dialog(colorSelector = true, tab = "Animation", group = "if animation = true", enable = animation));
          input Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(
            Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB resolveInFrameAfterDifferentiation = resolveInFrame "Frame in which vectors are resolved after differentiation (world, frame_a, frame_b, or frame_resolve)" annotation(
            Dialog(tab = "Advanced", group = "if get_v_rel or get_a_rel or get_z_rel", enable = get_v_rel or get_a_rel or get_z_rel));
          Blocks.Interfaces.RealOutput r_rel[3](each final quantity = "Length", each final unit = "m") if get_r_rel "Relative position vector frame_b.r_0 - frame_a.r_0 resolved in frame defined by resolveInFrame" annotation(
            Placement(transformation(origin = {-100, -110}, extent = {{10, -10}, {-10, 10}}, rotation = 90)));
          Blocks.Interfaces.RealOutput v_rel[3](each final quantity = "Velocity", each final unit = "m/s") if get_v_rel "Relative velocity vector" annotation(
            Placement(transformation(origin = {-60, -110}, extent = {{10, -10}, {-10, 10}}, rotation = 90)));
          Blocks.Interfaces.RealOutput a_rel[3](each final quantity = "Acceleration", each final unit = "m/s2") if get_a_rel "Relative acceleration vector" annotation(
            Placement(transformation(origin = {-20, -110}, extent = {{10, -10}, {-10, 10}}, rotation = 90)));
          Blocks.Interfaces.RealOutput angles[3](each final quantity = "Angle", each final unit = "rad", each displayUnit = "deg") if get_angles "Angles to rotate frame_a into frame_b via 'sequence'" annotation(
            Placement(transformation(origin = {20, -110}, extent = {{10, -10}, {-10, 10}}, rotation = 90)));
          Blocks.Interfaces.RealOutput w_rel[3](each final quantity = "AngularVelocity", each final unit = "1/s") if get_w_rel "Relative angular velocity vector" annotation(
            Placement(transformation(origin = {60, -110}, extent = {{10, -10}, {-10, 10}}, rotation = 90)));
          Blocks.Interfaces.RealOutput z_rel[3](each final quantity = "AngularAcceleration", each final unit = "1/s2") if get_z_rel "Relative angular acceleration vector" annotation(
            Placement(transformation(origin = {100, -110}, extent = {{10, -10}, {-10, 10}}, rotation = 90)));
        protected
          RelativePosition relativePosition(resolveInFrame = resolveInFrame) if get_r_rel or get_v_rel or get_a_rel annotation(
            Placement(transformation(extent = {{-80, -10}, {-60, 10}})));
        protected
          Blocks.Continuous.Der der1[3] if get_v_rel or get_a_rel annotation(
            Placement(transformation(extent = {{-10, -10}, {0, 0}}, rotation = -90, origin = {-55, -30})));
          Blocks.Continuous.Der der2[3] if get_a_rel annotation(
            Placement(transformation(extent = {{0, 0}, {10, 10}}, rotation = -90, origin = {-25, -40})));
          Modelica.Mechanics.MultiBody.Sensors.RelativeAngles relativeAngles(sequence = sequence, guessAngle1 = guessAngle1) if get_angles annotation(
            Placement(transformation(extent = {{10, -25}, {30, -5}})));
          RelativeAngularVelocity relativeAngularVelocity(resolveInFrame = resolveInFrame) if get_w_rel or get_z_rel annotation(
            Placement(transformation(extent = {{50, -40}, {70, -20}})));
        protected
          Blocks.Continuous.Der der3[3] if get_z_rel annotation(
            Placement(transformation(extent = {{-10, -10}, {0, 0}}, rotation = -90, origin = {105, -60})));
          Internal.ZeroForceAndTorque zeroForce1 annotation(
            Placement(transformation(extent = {{-81, 40}, {-61, 60}})));
          Internal.ZeroForceAndTorque zeroForce2 annotation(
            Placement(transformation(extent = {{70, 20}, {50, 40}})));
          Internal.ZeroForceAndTorque zeroForce3 if resolveInFrame == Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB.frame_resolve annotation(
            Placement(transformation(extent = {{70, 50}, {50, 70}})));
        protected
          Modelica.Mechanics.MultiBody.Sensors.TransformRelativeVector transformVector_v_rel(frame_r_in = resolveInFrame, frame_r_out = resolveInFrameAfterDifferentiation) if get_v_rel annotation(
            Placement(transformation(extent = {{-70, -64}, {-50, -44}})));
          Modelica.Mechanics.MultiBody.Sensors.TransformRelativeVector transformVector_a_rel(frame_r_in = resolveInFrame, frame_r_out = resolveInFrameAfterDifferentiation) if get_a_rel annotation(
            Placement(transformation(extent = {{-30, -80}, {-10, -60}})));
          Modelica.Mechanics.MultiBody.Sensors.TransformRelativeVector transformVector_z_rel(frame_r_in = resolveInFrame, frame_r_out = resolveInFrameAfterDifferentiation) if get_z_rel annotation(
            Placement(transformation(extent = {{90, -95}, {110, -75}})));
        protected
          outer Modelica.Mechanics.MultiBody.World world;
          Modelica.Mechanics.MultiBody.Visualizers.Advanced.Arrow arrow(r = frame_a.r_0, r_head = frame_b.r_0 - frame_a.r_0, diameter = arrowDiameter, color = arrowColor, specularCoefficient) if world.enableAnimation and animation;
        equation
          connect(relativePosition.frame_a, frame_a) annotation(
            Line(points = {{-80, 0}, {-100, 0}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
          connect(relativePosition.frame_b, frame_b) annotation(
            Line(points = {{-60, 0}, {100, 0}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
          connect(relativePosition.r_rel, r_rel) annotation(
            Line(points = {{-70, -11}, {-70, -15}, {-80, -15}, {-80, -80}, {-100, -80}, {-100, -110}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(zeroForce1.frame_a, frame_a) annotation(
            Line(points = {{-81, 50}, {-90, 50}, {-90, 0}, {-100, 0}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
          connect(zeroForce2.frame_a, frame_b) annotation(
            Line(points = {{70, 30}, {90, 30}, {90, 0}, {100, 0}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
          connect(relativePosition.r_rel, der1.u) annotation(
            Line(points = {{-70, -11}, {-70, -15}, {-60, -15}, {-60, -19}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(der2.u, der1.y) annotation(
            Line(points = {{-20, -39}, {-20, -35}, {-60, -35}, {-60, -30.5}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(relativeAngles.frame_a, frame_a) annotation(
            Line(points = {{10, -15}, {0, -15}, {0, 30}, {-90, 30}, {-90, 0}, {-100, 0}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
          connect(relativeAngles.frame_b, frame_b) annotation(
            Line(points = {{30, -15}, {40, -15}, {40, 0}, {100, 0}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
          connect(relativeAngles.angles, angles) annotation(
            Line(points = {{20, -26}, {20, -110}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(relativeAngularVelocity.frame_b, frame_b) annotation(
            Line(points = {{70, -30}, {80, -30}, {80, 0}, {100, 0}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
          connect(relativeAngularVelocity.frame_a, frame_a) annotation(
            Line(points = {{50, -30}, {0, -30}, {0, 30}, {-90, 30}, {-90, 0}, {-100, 0}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
          connect(relativeAngularVelocity.w_rel, w_rel) annotation(
            Line(points = {{60, -41}, {60, -110}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(relativeAngularVelocity.w_rel, der3.u) annotation(
            Line(points = {{60, -41}, {60, -43}, {100, -43}, {100, -49}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(der1.y, transformVector_v_rel.r_in) annotation(
            Line(points = {{-60, -30.5}, {-60, -42}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(transformVector_v_rel.r_out, v_rel) annotation(
            Line(points = {{-60, -65}, {-60, -110}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(transformVector_v_rel.frame_a, frame_a) annotation(
            Line(points = {{-70, -54}, {-90, -54}, {-90, 0}, {-100, 0}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
          connect(transformVector_v_rel.frame_b, frame_b) annotation(
            Line(points = {{-50, -54}, {-36, -54}, {-36, 0}, {100, 0}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
          connect(transformVector_v_rel.frame_resolve, frame_resolve) annotation(
            Line(points = {{-50, -45.9}, {-47, -45.9}, {-47, -47}, {-42, -47}, {-42, 80}, {100, 80}}, color = {95, 95, 95}, pattern = LinePattern.Dot, smooth = Smooth.None));
          connect(frame_resolve, relativePosition.frame_resolve) annotation(
            Line(points = {{100, 80}, {-50, 80}, {-50, 8.1}, {-60, 8.1}}, color = {95, 95, 95}, pattern = LinePattern.Dot, smooth = Smooth.None));
          connect(frame_resolve, zeroForce3.frame_a) annotation(
            Line(points = {{100, 80}, {80, 80}, {80, 60}, {70, 60}}, color = {95, 95, 95}, pattern = LinePattern.Dot, smooth = Smooth.None));
          connect(relativeAngularVelocity.frame_resolve, frame_resolve) annotation(
            Line(points = {{70, -21.9}, {77, -21.9}, {77, 80}, {100, 80}}, color = {95, 95, 95}, pattern = LinePattern.Dot, smooth = Smooth.None));
          connect(der2.y, transformVector_a_rel.r_in) annotation(
            Line(points = {{-20, -50.5}, {-20, -58}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(transformVector_a_rel.frame_a, frame_a) annotation(
            Line(points = {{-30, -70}, {-90, -70}, {-90, 0}, {-100, 0}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
          connect(transformVector_a_rel.frame_b, frame_b) annotation(
            Line(points = {{-10, -70}, {6, -70}, {6, 0}, {100, 0}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
          connect(transformVector_a_rel.frame_resolve, frame_resolve) annotation(
            Line(points = {{-10, -61.9}, {-10, 80}, {100, 80}}, color = {95, 95, 95}, pattern = LinePattern.Dot, smooth = Smooth.None));
          connect(transformVector_a_rel.r_out, a_rel) annotation(
            Line(points = {{-20, -81}, {-20, -110}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(der3.y, transformVector_z_rel.r_in) annotation(
            Line(points = {{100, -60.5}, {100, -73}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(transformVector_z_rel.r_out, z_rel) annotation(
            Line(points = {{100, -96}, {100, -110}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(transformVector_z_rel.frame_a, frame_a) annotation(
            Line(points = {{90, -85}, {-90, -85}, {-90, 0}, {-100, 0}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
          connect(transformVector_z_rel.frame_b, frame_b) annotation(
            Line(points = {{110, -85}, {119, -85}, {119, 0}, {100, 0}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
          connect(transformVector_z_rel.frame_resolve, frame_resolve) annotation(
            Line(points = {{110, -76.9}, {116, -76.9}, {116, 80}, {100, 80}}, color = {95, 95, 95}, pattern = LinePattern.Dot, smooth = Smooth.None));
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(visible = get_r_rel, points = {{-84, 0}, {-84, -60}, {-100, -60}, {-100, -100}}, color = {0, 0, 127}, smooth = Smooth.None), Line(visible = get_a_rel, points = {{-20, -67}, {-20, -100}}, color = {0, 0, 127}, smooth = Smooth.None), Line(visible = get_w_rel, points = {{60, -36}, {60, -100}}, color = {0, 0, 127}, smooth = Smooth.None), Line(visible = get_z_rel, points = {{86, 0}, {86, -60}, {100, -60}, {100, -100}}, color = {0, 0, 127}, smooth = Smooth.None), Line(visible = get_v_rel, points = {{-60, -36}, {-60, -100}}, color = {0, 0, 127}, smooth = Smooth.None), Line(visible = get_angles, points = {{20, -67}, {20, -100}}, color = {0, 0, 127}, smooth = Smooth.None), Text(extent = {{-132, 90}, {129, 138}}, textString = "%name", lineColor = {0, 0, 255}), Text(visible = get_r_rel, extent = {{-130, -74}, {-95, -90}}, lineColor = {0, 0, 0}, textString = "r"), Text(visible = get_v_rel, extent = {{-95, -74}, {-60, -90}}, lineColor = {0, 0, 0}, textString = "v"), Text(visible = get_a_rel, extent = {{-55, -74}, {-20, -90}}, lineColor = {0, 0, 0}, textString = "a"), Text(visible = get_angles, extent = {{-71, -35}, {96, -54}}, lineColor = {0, 0, 0}, textString = "angles"), Text(visible = get_w_rel, extent = {{63, -73}, {103, -90}}, lineColor = {0, 0, 0}, textString = "w"), Text(visible = get_z_rel, extent = {{103, -71}, {149, -87}}, lineColor = {0, 0, 0}, textString = "z")}),
            Documentation(info = "<html>
<p>
Relative kinematic quantities between frame_a and frame_b are
determined and provided at the conditional output signal connectors.
For example, if parameter \"get_r_rel = <b>true</b>\", the connector
\"r_rel\" is enabled and contains the relative vector from
frame_a to frame_b. The following quantities can be provided
as output signals:
</p>

<ol>
<li> Relative position vector (= r_rel) </li>
<li> Relative velocity vector (= v_rel)</li>
<li> Relative acceleration vector (= a_rel)</li>
<li> Three angles to rotate frame_a into frame_b (= angles)</li>
<li> Relative angular velocity vector (= w_rel)</li>
<li> Relative angular acceleration vector (= z_rel)</li>
</ol>

<p>
Via parameter <b>resolveInFrame</b> it is defined, in which frame
a vector is resolved (before differentiation):
</p>

<table border=1 cellspacing=0 cellpadding=2>
<tr><th><b>resolveInFrame =<br>Types.ResolveInFrameAB.</b></th><th><b>Meaning</b></th></tr>
<tr><td valign=\"top\">world</td>
    <td valign=\"top\">Resolve vectors in world frame</td></tr>

<tr><td valign=\"top\">frame_a</td>
    <td valign=\"top\">Resolve vectors in frame_a</td></tr>

<tr><td valign=\"top\">frame_b</td>
    <td valign=\"top\">Resolve vectors in frame_b</td></tr>

<tr><td valign=\"top\">frame_resolve</td>
    <td valign=\"top\">Resolve vectors in frame_resolve</td></tr>
</table>

<p>
If resolveInFrame = Types.ResolveInFrameAB.frame_resolve, the conditional connector
\"frame_resolve\" is enabled and the vectors are resolved in the frame, to
which frame_resolve is connected. Note, if this connector is enabled, it must
be connected.
</p>

<p>
In the following figure the animation of a RelativeSensor
component is shown. The light blue coordinate system is
frame_a, the dark blue coordinate system is frame_b, and
the yellow arrow is the animated sensor.
</p>

<p>
<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Sensors/RelativeSensor.png\">
</p>

<p>
Note, derivatives
of relative kinematic quantities are always performed with
respect to the frame, in which the vector to be differentiated
is resolved. After differentiation, it is possible via parameter
<b>resolveInFrameAfterDifferentiation</b> (in the \"Advanced\" menu)
to resolve the differentiated
vector in another frame.
</p>
<p>
For example, if resolveInFrame = <b>Types.ResolveInFrameAB.frame_b</b>, then
</p>

<pre>
   r_rel = resolve2(frame_b.R, frame_b.r_0 - frame_a.r0);
   v_rel = <b>der</b>(r_rel);
</pre>

<p>
is returned (r_rel = resolve2(frame_b.R, frame_b.r_0 - frame_a.r0)), i.e.,
the derivative of the relative distance from frame_a to frame_b,
resolved in frame_b. If
<b>resolveInFrameAfterDifferentiation</b> = Types.ResolveInFrameAB.world, then
v_rel is additionally transformed to:
</p>

<pre>
   v_rel = resolve1(frame_b.R, <b>der</b>(r_rel))
</pre>

<p>
The cut-force and the cut-torque in frame_resolve are
always zero, whether frame_resolve is connected or not.
</p>

<p>
If <b>get_angles</b> = <b>true</b>, the 3 angles to rotate frame_a
into frame_b along the axes defined by parameter <b>sequence</b>
are returned. For example, if sequence = {3,1,2} then frame_a is
rotated around angles[1] along the z-axis, afterwards it is rotated
around angles[2] along the x-axis, and finally it is rotated around
angles[3] along the y-axis and is then identical to frame_b.
The 3 angles are returned in the range
</p>
<pre>
    -<font face=\"Symbol\">p</font> &lt;= angles[i] &lt;= <font face=\"Symbol\">p</font>
</pre>
<p>
There are <b>two solutions</b> for \"angles[1]\" in this range.
Via parameter <b>guessAngle1</b> (default = 0) the
returned solution is selected such that |angles[1] - guessAngle1| is
minimal. The relative transformation matrix between frame_a and
frame_b may be in a singular configuration with respect to \"sequence\", i.e.,
there is an infinite number of angle values leading to the same relative
transformation matrix. In this case, the returned solution is
selected by setting angles[1] = guessAngle1. Then angles[2]
and angles[3] can be uniquely determined in the above range.
</p>
<p>
The parameter <b>sequence</b> has the restriction that
only values 1,2,3 can be used and that sequence[1] &ne; sequence[2]
and sequence[2] &ne; sequence[3]. Often used values are:
</p>
<pre>
  sequence = <b>{1,2,3}</b>  // Cardan or Tait-Bryan angle sequence
           = <b>{3,1,3}</b>  // Euler angle sequence
           = <b>{3,2,1}</b>
</pre>
</html>"));
        end RelativeSensor;

        model AbsolutePosition "Measure absolute position vector of the origin of a frame connector"
          extends Internal.PartialAbsoluteSensor;
          Blocks.Interfaces.RealOutput r[3](each final quantity = "Length", each final unit = "m") "Absolute position vector resolved in frame defined by resolveInFrame" annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0, origin = {110, 0})));
          Modelica.Mechanics.MultiBody.Interfaces.Frame_resolve frame_resolve if resolveInFrame == Modelica.Mechanics.MultiBody.Types.ResolveInFrameA.frame_resolve "Coordinate system in which output vector r is optionally resolved" annotation(
            Placement(transformation(extent = {{-16, -16}, {16, 16}}, rotation = -90, origin = {0, -100}), iconTransformation(extent = {{-16, -16}, {16, 16}}, rotation = -90, origin = {0, -101})));
          parameter Modelica.Mechanics.MultiBody.Types.ResolveInFrameA resolveInFrame = Modelica.Mechanics.MultiBody.Types.ResolveInFrameA.frame_a "Frame in which output vector r shall be resolved (world, frame_a, or frame_resolve)";
        protected
          Internal.BasicAbsolutePosition position(resolveInFrame = resolveInFrame) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}})));
          Modelica.Mechanics.MultiBody.Interfaces.ZeroPosition zeroPosition if not resolveInFrame == Modelica.Mechanics.MultiBody.Types.ResolveInFrameA.frame_resolve annotation(
            Placement(transformation(extent = {{20, -40}, {40, -20}})));
        equation
          connect(position.frame_resolve, frame_resolve) annotation(
            Line(points = {{0, -10}, {0, -100}}, color = {95, 95, 95}, pattern = LinePattern.Dot, smooth = Smooth.None));
          connect(zeroPosition.frame_resolve, position.frame_resolve) annotation(
            Line(points = {{20, -30}, {0, -30}, {0, -10}}, color = {95, 95, 95}, pattern = LinePattern.Dot, smooth = Smooth.None));
          connect(position.r, r) annotation(
            Line(points = {{11, 0}, {110, 0}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(position.frame_a, frame_a) annotation(
            Line(points = {{-10, 0}, {-100, 0}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{70, 0}, {100, 0}}, color = {0, 0, 127}, smooth = Smooth.None), Text(extent = {{-127, 95}, {134, 143}}, textString = "%name", lineColor = {0, 0, 255}), Text(extent = {{62, 46}, {146, 16}}, lineColor = {0, 0, 0}, textString = "r"), Text(extent = {{15, -67}, {146, -92}}, lineColor = {95, 95, 95}, textString = "resolve"), Line(points = {{0, -96}, {0, -96}, {0, -70}, {0, -70}}, color = {0, 0, 0}, smooth = Smooth.None, pattern = LinePattern.Dot)}),
            Documentation(info = "<html>
<p>
The absolute position vector of the origin of frame_a is
determined and provided at the output signal connector <b>r</b>.
</p>

<p>
Via parameter <b>resolveInFrame</b> it is defined, in which frame
the position vector is resolved:
</p>

<table border=1 cellspacing=0 cellpadding=2>
<tr><th><b>resolveInFrame =<br>Types.ResolveInFrameA.</b></th><th><b>Meaning</b></th></tr>
<tr><td valign=\"top\">world</td>
    <td valign=\"top\">Resolve vector in world frame</td></tr>

<tr><td valign=\"top\">frame_a</td>
    <td valign=\"top\">Resolve vector in frame_a</td></tr>

<tr><td valign=\"top\">frame_resolve</td>
    <td valign=\"top\">Resolve vector in frame_resolve</td></tr>
</table>

<p>
If resolveInFrame = Types.ResolveInFrameA.frame_resolve, the conditional connector
\"frame_resolve\" is enabled and r is resolved in the frame, to
which frame_resolve is connected. Note, if this connector is enabled, it must
be connected.
</p>

<p>
Example: If resolveInFrame = Types.ResolveInFrameA.frame_a, the output vector is
computed as:
</p>

<pre>
    r = MultiBody.Frames.resolve2(frame_a.R, frame_b.r_0);
</pre>
</html>"));
        end AbsolutePosition;

        model AbsoluteVelocity "Measure absolute velocity vector of origin of frame connector"
          extends Internal.PartialAbsoluteSensor;
          Blocks.Interfaces.RealOutput v[3](each final quantity = "Velocity", each final unit = "m/s") "Absolute velocity vector resolved in frame defined by resolveInFrame" annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0, origin = {110, 0})));
          Modelica.Mechanics.MultiBody.Interfaces.Frame_resolve frame_resolve if resolveInFrame == Modelica.Mechanics.MultiBody.Types.ResolveInFrameA.frame_resolve "Coordinate system in which output vector v is optionally resolved" annotation(
            Placement(transformation(extent = {{-16, -16}, {16, 16}}, rotation = -90, origin = {0, -100}), iconTransformation(extent = {{-16, -16}, {16, 16}}, rotation = -90, origin = {0, -100})));
          parameter Modelica.Mechanics.MultiBody.Types.ResolveInFrameA resolveInFrame = Modelica.Mechanics.MultiBody.Types.ResolveInFrameA.frame_a "Frame in which output vector v shall be resolved (world, frame_a, or frame_resolve)";
        protected
          Internal.BasicAbsolutePosition position(resolveInFrame = Modelica.Mechanics.MultiBody.Types.ResolveInFrameA.world) annotation(
            Placement(transformation(extent = {{-60, -10}, {-40, 10}})));
          Blocks.Continuous.Der der1[3] annotation(
            Placement(transformation(extent = {{-20, -20}, {0, 0}}, rotation = 0, origin = {10, 10})));
          Modelica.Mechanics.MultiBody.Sensors.TransformAbsoluteVector tansformAbsoluteVector(frame_r_in = Modelica.Mechanics.MultiBody.Types.ResolveInFrameA.world, frame_r_out = resolveInFrame) annotation(
            Placement(transformation(extent = {{10, -10}, {-10, 10}}, rotation = 90, origin = {50, 0})));
          Modelica.Mechanics.MultiBody.Interfaces.ZeroPosition zeroPosition annotation(
            Placement(transformation(extent = {{-60, -60}, {-80, -40}})));
          Modelica.Mechanics.MultiBody.Interfaces.ZeroPosition zeroPosition1 if not resolveInFrame == Modelica.Mechanics.MultiBody.Types.ResolveInFrameA.frame_resolve annotation(
            Placement(transformation(extent = {{60, -60}, {80, -40}})));
        equation
          connect(position.r, der1.u) annotation(
            Line(points = {{-39, 0}, {-12, 0}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(position.frame_a, frame_a) annotation(
            Line(points = {{-60, 0}, {-80, 0}, {-80, 4.44089e-16}, {-100, 4.44089e-16}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
          connect(der1.y, tansformAbsoluteVector.r_in) annotation(
            Line(points = {{11, 0}, {38, 0}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(tansformAbsoluteVector.r_out, v) annotation(
            Line(points = {{61, 0}, {110, 0}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(zeroPosition.frame_resolve, position.frame_resolve) annotation(
            Line(points = {{-60, -50}, {-50, -50}, {-50, -10}}, color = {95, 95, 95}, pattern = LinePattern.Dot, smooth = Smooth.None));
          connect(tansformAbsoluteVector.frame_a, frame_a) annotation(
            Line(points = {{50, 10}, {50, 20}, {-70, 20}, {-70, 4.44089e-16}, {-100, 4.44089e-16}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
          connect(tansformAbsoluteVector.frame_resolve, zeroPosition1.frame_resolve) annotation(
            Line(points = {{49.9, -10}, {50, -10}, {50, -50}, {60, -50}}, color = {95, 95, 95}, pattern = LinePattern.Dot, smooth = Smooth.None));
          connect(tansformAbsoluteVector.frame_resolve, frame_resolve) annotation(
            Line(points = {{49.9, -10}, {50, -10}, {50, -50}, {0, -50}, {0, -100}}, color = {95, 95, 95}, pattern = LinePattern.Dot, smooth = Smooth.None));
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{70, 0}, {100, 0}}, color = {0, 0, 127}, smooth = Smooth.None), Text(extent = {{-130, 72}, {131, 120}}, textString = "%name", lineColor = {0, 0, 255}), Text(extent = {{58, 48}, {142, 18}}, lineColor = {0, 0, 0}, textString = "v"), Text(extent = {{15, -67}, {146, -92}}, lineColor = {95, 95, 95}, textString = "resolve"), Line(points = {{0, -70}, {0, -95}}, color = {95, 95, 95}, smooth = Smooth.None, pattern = LinePattern.Dot)}),
            Documentation(info = "<html>
<p>
The absolute velocity vector of the origin of frame_a is
determined and provided at the output signal connector <b>v</b>.
</p>

<p>
Via parameter <b>resolveInFrame</b> it is defined, in which frame
the velocity vector is resolved:
</p>

<table border=1 cellspacing=0 cellpadding=2>
<tr><th><b>resolveInFrame =<br>Types.ResolveInFrameA.</b></th><th><b>Meaning</b></th></tr>
<tr><td valign=\"top\">world</td>
    <td valign=\"top\">Resolve vector in world frame</td></tr>

<tr><td valign=\"top\">frame_a</td>
    <td valign=\"top\">Resolve vector in frame_a</td></tr>

<tr><td valign=\"top\">frame_resolve</td>
    <td valign=\"top\">Resolve vector in frame_resolve</td></tr>
</table>

<p>
If resolveInFrame = Types.ResolveInFrameA.frame_resolve, the conditional connector
\"frame_resolve\" is enabled and v is resolved in the frame, to
which frame_resolve is connected. Note, if this connector is enabled, it must
be connected.
</p>

<p>
Example: If resolveInFrame = Types.ResolveInFrameA.frame_a, the output vector is
computed as:
</p>

<pre>
    v0 = der(frame_a.r_0);
    v  = MultiBody.Frames.resolve2(frame_a.R, v0);
</pre>

</html>"));
        end AbsoluteVelocity;

        model AbsoluteAngles "Measure absolute angles between frame connector and the world frame"
          extends Modelica.Icons.RotationalSensor;
          Modelica.Mechanics.MultiBody.Interfaces.Frame_a frame_a "Coordinate system a from which the angles shall be determined" annotation(
            Placement(transformation(extent = {{-116, -16}, {-84, 16}}, rotation = 0)));
          Modelica.Blocks.Interfaces.RealOutput angles[3](each final quantity = "Angle", each final unit = "rad", each displayUnit = "deg") "Angles to rotate world frame into frame_a via 'sequence'" annotation(
            Placement(transformation(origin = {110, 0}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
          parameter MultiBody.Types.RotationSequence sequence(min = {1, 1, 1}, max = {3, 3, 3}) = {1, 2, 3} "Angles are returned to rotate world frame around axes sequence[1], sequence[2] and finally sequence[3] into frame_a" annotation(
            Evaluate = true);
          parameter SI.Angle guessAngle1 = 0 "Select angles[1] such that abs(angles[1] - guessAngle1) is a minimum";
        equation
          frame_a.f = zeros(3);
          frame_a.t = zeros(3);
          angles = MultiBody.Frames.axesRotationsAngles(frame_a.R, sequence, guessAngle1);
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-108, 43}, {-72, 18}}, lineColor = {128, 128, 128}, textString = "a"), Line(points = {{-70, 0}, {-96, 0}, {-96, 0}}, color = {0, 0, 0}, smooth = Smooth.None), Text(extent = {{-132, 76}, {129, 124}}, textString = "%name", lineColor = {0, 0, 255}), Line(points = {{70, 0}, {100, 0}}, color = {0, 0, 127}, smooth = Smooth.None), Text(extent = {{62, -22}, {172, -44}}, lineColor = {0, 0, 0}, textString = "angles")}),
            Documentation(info = "<html>
<p>
This model determines the 3 angles to rotate the world frame
into frame_a along the axes defined by parameter <b>sequence</b>.
For example, if sequence = {3,1,2} then the world frame is
rotated around angles[1] along the z-axis, afterwards it is rotated
around angles[2] along the x-axis, and finally it is rotated around
angles[3] along the y-axis and is then identical to frame_a.
The 3 angles are returned in the range
</p>
<pre>
    -<font face=\"Symbol\">p</font> &lt;= angles[i] &lt;= <font face=\"Symbol\">p</font>
</pre>
<p>
There are <b>two solutions</b> for \"angles[1]\" in this range.
Via parameter <b>guessAngle1</b> (default = 0) the
returned solution is selected such that |angles[1] - guessAngle1| is
minimal. The transformation matrix between the world frame and
frame_a may be in a singular configuration with respect to \"sequence\", i.e.,
there is an infinite number of angle values leading to the same relative
transformation matrix. In this case, the returned solution is
selected by setting angles[1] = guessAngle1. Then angles[2]
and angles[3] can be uniquely determined in the above range.
</p>
<p>
The parameter <b>sequence</b> has the restriction that
only values 1,2,3 can be used and that sequence[1] &ne; sequence[2]
and sequence[2] &ne; sequence[3]. Often used values are:
</p>
<pre>
  sequence = <b>{1,2,3}</b>  // Cardan or Tait-Bryan angle sequence
           = <b>{3,1,3}</b>  // Euler angle sequence
           = <b>{3,2,1}</b>
</pre>
</html>"));
        end AbsoluteAngles;

        model AbsoluteAngularVelocity "Measure absolute angular velocity of frame connector"
          extends Internal.PartialAbsoluteSensor;
          Blocks.Interfaces.RealOutput w[3](each final quantity = "AngularVelocity", each final unit = "rad/s") "Absolute angular velocity vector of frame_a with respect to world frame, resolved in frame defined by resolveInFrame" annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0, origin = {110, 0})));
          Modelica.Mechanics.MultiBody.Interfaces.Frame_resolve frame_resolve if resolveInFrame == Modelica.Mechanics.MultiBody.Types.ResolveInFrameA.frame_resolve "Coordinate system in which w is optionally resolved" annotation(
            Placement(transformation(extent = {{-16, -16}, {16, 16}}, rotation = -90, origin = {0, -100}), iconTransformation(extent = {{-16, -16}, {16, 16}}, rotation = -90, origin = {0, -101})));
          parameter Modelica.Mechanics.MultiBody.Types.ResolveInFrameA resolveInFrame = Modelica.Mechanics.MultiBody.Types.ResolveInFrameA.frame_a "Frame in which output vector w shall be resolved (world, frame_a, or frame_resolve)";
        protected
          Internal.BasicAbsoluteAngularVelocity angularVelocity(resolveInFrame = resolveInFrame) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}})));
          Modelica.Mechanics.MultiBody.Interfaces.ZeroPosition zeroPosition if not resolveInFrame == Modelica.Mechanics.MultiBody.Types.ResolveInFrameA.frame_resolve annotation(
            Placement(transformation(extent = {{40, -40}, {60, -20}})));
        equation
          connect(angularVelocity.frame_resolve, frame_resolve) annotation(
            Line(points = {{0, -10}, {0, -100}}, color = {95, 95, 95}, pattern = LinePattern.Dot, smooth = Smooth.None));
          connect(zeroPosition.frame_resolve, angularVelocity.frame_resolve) annotation(
            Line(points = {{40, -30}, {0, -30}, {0, -10}}, color = {95, 95, 95}, pattern = LinePattern.Dot, smooth = Smooth.None));
          connect(angularVelocity.w, w) annotation(
            Line(points = {{11, 0}, {110, 0}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(angularVelocity.frame_a, frame_a) annotation(
            Line(points = {{-10, 0}, {-100, 0}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{70, 0}, {100, 0}}, color = {0, 0, 127}, smooth = Smooth.None), Text(extent = {{-127, 77}, {134, 125}}, textString = "%name", lineColor = {0, 0, 255}), Text(extent = {{66, 48}, {150, 18}}, lineColor = {0, 0, 0}, textString = "w"), Line(points = {{0, -70}, {0, -96}}, color = {95, 95, 95}, smooth = Smooth.None, pattern = LinePattern.Dot), Text(extent = {{15, -67}, {146, -92}}, lineColor = {95, 95, 95}, textString = "resolve")}),
            Documentation(info = "<html>
<p>
The absolute angular velocity of frame_a with respect to the
world frame is
determined and provided at the output signal connector <b>w</b>.
</p>

<p>
Via parameter <b>resolveInFrame</b> it is defined, in which frame
the angular velocity is resolved:
</p>

<table border=1 cellspacing=0 cellpadding=2>
<tr><th><b>resolveInFrame =<br>Types.ResolveInFrameAB.</b></th><th><b>Meaning</b></th></tr>
<tr><td valign=\"top\">world</td>
    <td valign=\"top\">Resolve vector in world frame</td></tr>

<tr><td valign=\"top\">frame_a</td>
    <td valign=\"top\">Resolve vector in frame_a</td></tr>

<tr><td valign=\"top\">frame_resolve</td>
    <td valign=\"top\">Resolve vector in frame_resolve</td></tr>
</table>

<p>
If resolveInFrame = Types.ResolveInFrameA.frame_resolve, the conditional connector
\"frame_resolve\" is enabled and w is resolved in the frame, to
which frame_resolve is connected. Note, if this connector is enabled, it must
be connected.
</p>

<p>
Example: If resolveInFrame = Types.ResolveInFrameA.frame_a, the output vector is
computed as:
</p>

<pre>
    w = MultiBody.Frames.angularVelocity2(frame_a.R);
</pre>

</html>"));
        end AbsoluteAngularVelocity;

        model RelativePosition "Measure relative position vector between the origins of two frame connectors"
          extends Internal.PartialRelativeSensor;
          Blocks.Interfaces.RealOutput r_rel[3] "Relative position vector resolved in frame defined by resolveInFrame" annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = -90, origin = {0, -110})));
          Modelica.Mechanics.MultiBody.Interfaces.Frame_resolve frame_resolve if resolveInFrame == Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB.frame_resolve "Coordinate system in which r_rel is optionally resolved" annotation(
            Placement(transformation(extent = {{84, 64}, {116, 96}}), iconTransformation(extent = {{84, 65}, {116, 97}})));
          parameter Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB resolveInFrame = Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB.frame_a "Frame in which output vector r_rel shall be resolved (world, frame_a, frame_b, or frame_resolve)";
        protected
          Internal.BasicRelativePosition relativePosition(resolveInFrame = resolveInFrame) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}})));
          Modelica.Mechanics.MultiBody.Interfaces.ZeroPosition zeroPosition if not resolveInFrame == Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB.frame_resolve annotation(
            Placement(transformation(extent = {{52, 20}, {72, 40}})));
        equation
          connect(relativePosition.frame_a, frame_a) annotation(
            Line(points = {{-10, 0}, {-100, 0}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
          connect(relativePosition.frame_b, frame_b) annotation(
            Line(points = {{10, 0}, {100, 0}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
          connect(relativePosition.frame_resolve, frame_resolve) annotation(
            Line(points = {{10, 8.1}, {26, 8.1}, {26, 8}, {36, 8}, {36, 80}, {100, 80}}, color = {95, 95, 95}, pattern = LinePattern.Dot, smooth = Smooth.None));
          connect(zeroPosition.frame_resolve, relativePosition.frame_resolve) annotation(
            Line(points = {{52, 30}, {36, 30}, {36, 8.1}, {10, 8.1}}, color = {95, 95, 95}, pattern = LinePattern.Dot, smooth = Smooth.None));
          connect(relativePosition.r_rel, r_rel) annotation(
            Line(points = {{0, -11}, {0, -110}}, color = {0, 0, 127}, smooth = Smooth.None));
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{0, -70}, {0, -100}}, color = {0, 0, 127}, smooth = Smooth.None), Text(extent = {{-127, 95}, {134, 143}}, textString = "%name", lineColor = {0, 0, 255}), Text(extent = {{18, -80}, {102, -110}}, lineColor = {0, 0, 0}, textString = "r_rel")}),
            Documentation(info = "<html>
<p>
The relative position vector between the origins of frame_a and frame_b are
determined and provided at the output signal connector <b>r_rel</b>.
</p>

<p>
Via parameter <b>resolveInFrame</b> it is defined, in which frame
the position vector is resolved:
</p>

<table border=1 cellspacing=0 cellpadding=2>
<tr><th><b>resolveInFrame =<br>Types.ResolveInFrameAB.</b></th><th><b>Meaning</b></th></tr>
<tr><td valign=\"top\">world</td>
    <td valign=\"top\">Resolve vector in world frame</td></tr>

<tr><td valign=\"top\">frame_a</td>
    <td valign=\"top\">Resolve vector in frame_a</td></tr>

<tr><td valign=\"top\">frame_b</td>
    <td valign=\"top\">Resolve vector in frame_b</td></tr>

<tr><td valign=\"top\">frame_resolve</td>
    <td valign=\"top\">Resolve vector in frame_resolve</td></tr>
</table>

<p>
If resolveInFrame = Types.ResolveInFrameAB.frame_resolve, the conditional connector
\"frame_resolve\" is enabled and r_rel is resolved in the frame, to
which frame_resolve is connected. Note, if this connector is enabled, it must
be connected.
</p>

<p>
Example: If resolveInFrame = Types.ResolveInFrameAB.frame_a, the output vector is
computed as:
</p>

<pre>
    r_rel = MultiBody.Frames.resolve2(frame_a.R, frame_b.r_0 - frame_a.r_0);
</pre>
</html>"));
        end RelativePosition;

        model RelativeVelocity "Measure relative velocity vector between the origins of two frame connectors"
          extends Internal.PartialRelativeSensor;
          Modelica.Blocks.Interfaces.RealOutput v_rel[3](each final quantity = "Velocity", each final unit = "m/s") "Relative velocity vector resolved in frame defined by resolveInFrame" annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = -90, origin = {0, -110})));
          Modelica.Mechanics.MultiBody.Interfaces.Frame_resolve frame_resolve if resolveInFrame == Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB.frame_resolve "Coordinate system in which v_rel is optionally resolved" annotation(
            Placement(transformation(extent = {{84, 64}, {116, 96}}), iconTransformation(extent = {{84, 65}, {116, 97}})));
          parameter Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB resolveInFrame = Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB.frame_a "Frame in which output vector v_rel shall be resolved (world, frame_a, frame_b, or frame_resolve)";
        protected
          RelativePosition relativePosition(resolveInFrame = Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB.frame_a) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}})));
          Modelica.Mechanics.MultiBody.Interfaces.ZeroPosition zeroPosition if not resolveInFrame == Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB.frame_resolve annotation(
            Placement(transformation(extent = {{50, -60}, {70, -40}})));
          Modelica.Blocks.Continuous.Der der_r_rel[3] annotation(
            Placement(transformation(extent = {{-20, -20}, {0, 0}}, rotation = -90, origin = {10, -40})));
          Modelica.Mechanics.MultiBody.Sensors.TransformRelativeVector tansformRelativeVector(frame_r_in = Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB.frame_a, frame_r_out = resolveInFrame) annotation(
            Placement(transformation(extent = {{-10, -80}, {10, -60}})));
        equation
          connect(relativePosition.frame_a, frame_a) annotation(
            Line(points = {{-10, 0}, {-100, 0}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
          connect(relativePosition.frame_b, frame_b) annotation(
            Line(points = {{10, 0}, {100, 0}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
          connect(relativePosition.r_rel, der_r_rel.u) annotation(
            Line(points = {{0, -11}, {0, -18}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(der_r_rel.y, tansformRelativeVector.r_in) annotation(
            Line(points = {{0, -41}, {0, -58}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(tansformRelativeVector.r_out, v_rel) annotation(
            Line(points = {{0, -81}, {0, -110}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(tansformRelativeVector.frame_a, frame_a) annotation(
            Line(points = {{-10, -70}, {-70, -70}, {-70, 0}, {-100, 0}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
          connect(tansformRelativeVector.frame_b, frame_b) annotation(
            Line(points = {{10, -70}, {80, -70}, {80, 0}, {100, 0}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
          connect(tansformRelativeVector.frame_resolve, frame_resolve) annotation(
            Line(points = {{10, -61.9}, {35, -61.9}, {35, 80}, {100, 80}}, color = {95, 95, 95}, pattern = LinePattern.Dot, smooth = Smooth.None));
          connect(zeroPosition.frame_resolve, tansformRelativeVector.frame_resolve) annotation(
            Line(points = {{50, -50}, {35, -50}, {35, -61.9}, {10, -61.9}}, color = {95, 95, 95}, pattern = LinePattern.Dot, smooth = Smooth.None));
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{0, -70}, {0, -100}}, color = {0, 0, 127}, smooth = Smooth.None), Text(extent = {{-127, 95}, {134, 143}}, textString = "%name", lineColor = {0, 0, 255}), Text(extent = {{18, -80}, {102, -110}}, lineColor = {0, 0, 0}, textString = "v_rel")}),
            Documentation(info = "<html>
<p>
The relative velocity vector between the origins of frame_a and of frame_b are
determined and provided at the output signal connector <b>v_rel</b>.
This vector is defined as:
</p>

<pre>
    r_rel = MultiBody.Frames.resolve2(frame_a.R, frame_b.r_0 - frame_a.r_0);
    v_rel = <b>der</b>(r_rel);
</pre>

<p>
Via parameter <b>resolveInFrame</b> it is defined, in which frame
the velocity vector is resolved:
</p>

<table border=1 cellspacing=0 cellpadding=2>
<tr><th><b>resolveInFrame =<br>Types.ResolveInFrameAB.</b></th><th><b>Meaning</b></th></tr>
<tr><td valign=\"top\">world</td>
    <td valign=\"top\">Resolve vector in world frame</td></tr>

<tr><td valign=\"top\">frame_a</td>
    <td valign=\"top\">Resolve vector in frame_a</td></tr>

<tr><td valign=\"top\">frame_b</td>
    <td valign=\"top\">Resolve vector in frame_b</td></tr>

<tr><td valign=\"top\">frame_resolve</td>
    <td valign=\"top\">Resolve vector in frame_resolve</td></tr>
</table>

<p>
If resolveInFrame = Types.ResolveInFrameAB.frame_resolve, the conditional connector
\"frame_resolve\" is enabled and v_rel is resolved in the frame, to
which frame_resolve is connected. Note, if this connector is enabled, it must
be connected.
</p>

<p>
Example: If resolveInFrame = Types.ResolveInFrameAB.frame_b, the output vector is
computed as:
</p>

<pre>
    r_rel   = MultiBody.Frames.resolve2(frame_a.R, frame_b.r_0 - frame_a.r_0);
    v_rel_a = <b>der</b>(r_rel);
    v_rel   = MultiBody.Frames.resolveRelative(frame_a.R, frame_b.R, v_rel_a);
</pre>

</html>"));
        end RelativeVelocity;

        model RelativeAngles "Measure relative angles between two frame connectors"
          extends Modelica.Icons.RotationalSensor;
          Modelica.Mechanics.MultiBody.Interfaces.Frame_a frame_a "Coordinate system a" annotation(
            Placement(transformation(extent = {{-116, -16}, {-84, 16}}, rotation = 0)));
          Modelica.Mechanics.MultiBody.Interfaces.Frame_b frame_b "Coordinate system b" annotation(
            Placement(transformation(extent = {{84, -16}, {116, 16}}, rotation = 0)));
          Modelica.Blocks.Interfaces.RealOutput angles[3](each final quantity = "Angle", each final unit = "rad", each displayUnit = "deg") "Angles to rotate frame_a into frame_b via 'sequence'" annotation(
            Placement(transformation(origin = {0, -110}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          parameter MultiBody.Types.RotationSequence sequence(min = {1, 1, 1}, max = {3, 3, 3}) = {1, 2, 3} "Angles are returned to rotate frame_a around axes sequence[1], sequence[2] and finally sequence[3] into frame_b" annotation(
            Evaluate = true);
          parameter SI.Angle guessAngle1 = 0 "Select angles[1] such that abs(angles[1] - guessAngle1) is a minimum";
          Modelica.Mechanics.MultiBody.Frames.Orientation R_rel "Relative orientation object from frame_a to frame_b";
        equation
          frame_a.f = zeros(3);
          frame_a.t = zeros(3);
          frame_b.f = zeros(3);
          frame_b.t = zeros(3);
          R_rel = Modelica.Mechanics.MultiBody.Frames.relativeRotation(frame_a.R, frame_b.R);
          angles = MultiBody.Frames.axesRotationsAngles(R_rel, sequence, guessAngle1);
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-108, 43}, {-72, 18}}, lineColor = {128, 128, 128}, textString = "a"), Text(extent = {{72, 41}, {108, 16}}, lineColor = {128, 128, 128}, textString = "b"), Line(points = {{-70, 0}, {-96, 0}, {-96, 0}}, color = {0, 0, 0}, smooth = Smooth.None), Line(points = {{96, 0}, {70, 0}, {70, 0}}, color = {0, 0, 0}, smooth = Smooth.None), Text(extent = {{-132, 90}, {129, 138}}, textString = "%name", lineColor = {0, 0, 255}), Line(points = {{0, -70}, {0, -100}}, color = {0, 0, 127}, smooth = Smooth.None), Text(extent = {{0, -76}, {110, -98}}, lineColor = {0, 0, 0}, textString = "angles")}),
            Documentation(info = "<html>
<p>
This model determines the 3 angles to rotate frame_a
into frame_b along the axes defined by parameter <b>sequence</b>.
For example, if sequence = {3,1,2} then frame_a is
rotated around angles[1] along the z-axis, afterwards it is rotated
around angles[2] along the x-axis, and finally it is rotated around
angles[3] along the y-axis and is then identical to frame_b.
The 3 angles are returned in the range
</p>
<pre>
    -<font face=\"Symbol\">p</font> &lt;= angles[i] &lt;= <font face=\"Symbol\">p</font>
</pre>
<p>
There are <b>two solutions</b> for \"angles[1]\" in this range.
Via parameter <b>guessAngle1</b> (default = 0) the
returned solution is selected such that |angles[1] - guessAngle1| is
minimal. The relative transformation matrix between frame_a and
frame_b may be in a singular configuration with respect to \"sequence\", i.e.,
there is an infinite number of angle values leading to the same relative
transformation matrix. In this case, the returned solution is
selected by setting angles[1] = guessAngle1. Then angles[2]
and angles[3] can be uniquely determined in the above range.
</p>
<p>
The parameter <b>sequence</b> has the restriction that
only values 1,2,3 can be used and that sequence[1] &ne; sequence[2]
and sequence[2] &ne; sequence[3]. Often used values are:
</p>
<pre>
  sequence = <b>{1,2,3}</b>  // Cardan or Tait-Bryan angle sequence
           = <b>{3,1,3}</b>  // Euler angle sequence
           = <b>{3,2,1}</b>
</pre>
</html>"));
        end RelativeAngles;

        model RelativeAngularVelocity "Measure relative angular velocity between two frame connectors"
          extends Internal.PartialRelativeSensor;
          Blocks.Interfaces.RealOutput w_rel[3](each final quantity = "AngularVelocity", each final unit = "rad/s") "Relative angular velocity vector between frame_a and frame_b resolved in frame defined by resolveInFrame" annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = -90, origin = {0, -110})));
          Modelica.Mechanics.MultiBody.Interfaces.Frame_resolve frame_resolve if resolveInFrame == Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB.frame_resolve "Coordinate system in which w_rel is optionally resolved" annotation(
            Placement(transformation(extent = {{84, 64}, {116, 96}}), iconTransformation(extent = {{84, 65}, {116, 97}})));
          parameter Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB resolveInFrame = Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB.frame_a "Frame in which output vector w_rel shall be resolved (world, frame_a, frame_b, or frame_resolve)";
        protected
          Internal.BasicRelativeAngularVelocity relativeAngularVelocity(resolveInFrame = resolveInFrame) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}})));
          Modelica.Mechanics.MultiBody.Interfaces.ZeroPosition zeroPosition if not resolveInFrame == Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB.frame_resolve annotation(
            Placement(transformation(extent = {{52, 20}, {72, 40}})));
        equation
          connect(relativeAngularVelocity.frame_a, frame_a) annotation(
            Line(points = {{-10, 0}, {-100, 0}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
          connect(relativeAngularVelocity.frame_b, frame_b) annotation(
            Line(points = {{10, 0}, {100, 0}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
          connect(relativeAngularVelocity.frame_resolve, frame_resolve) annotation(
            Line(points = {{10, 8.1}, {26, 8.1}, {26, 8}, {34, 8}, {34, 80}, {100, 80}}, color = {95, 95, 95}, pattern = LinePattern.Dot, smooth = Smooth.None));
          connect(zeroPosition.frame_resolve, relativeAngularVelocity.frame_resolve) annotation(
            Line(points = {{52, 30}, {34, 30}, {34, 8.1}, {10, 8.1}}, color = {95, 95, 95}, pattern = LinePattern.Dot, smooth = Smooth.None));
          connect(relativeAngularVelocity.w_rel, w_rel) annotation(
            Line(points = {{0, -11}, {0, -110}}, color = {0, 0, 127}, smooth = Smooth.None));
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{0, -70}, {0, -100}}, color = {0, 0, 127}, smooth = Smooth.None), Text(extent = {{-127, 95}, {134, 143}}, textString = "%name", lineColor = {0, 0, 255}), Text(extent = {{14, -78}, {98, -108}}, lineColor = {0, 0, 0}, textString = "w_rel")}),
            Documentation(info = "<html>
<p>
The relative angular velocity between frame_a and frame_b is
determined and provided at the output signal connector <b>w_rel</b>.
</p>

<p>
Via parameter <b>resolveInFrame</b> it is defined, in which frame
the angular velocity is resolved:
</p>

<table border=1 cellspacing=0 cellpadding=2>
<tr><th><b>resolveInFrame =<br>Types.ResolveInFrameAB.</b></th><th><b>Meaning</b></th></tr>
<tr><td valign=\"top\">world</td>
    <td valign=\"top\">Resolve vector in world frame</td></tr>

<tr><td valign=\"top\">frame_a</td>
    <td valign=\"top\">Resolve vector in frame_a</td></tr>

<tr><td valign=\"top\">frame_b</td>
    <td valign=\"top\">Resolve vector in frame_b</td></tr>

<tr><td valign=\"top\">frame_resolve</td>
    <td valign=\"top\">Resolve vector in frame_resolve</td></tr>
</table>

<p>
If resolveInFrame = Types.ResolveInFrameAB.frame_resolve, the conditional connector
\"frame_resolve\" is enabled and w_rel is resolved in the frame, to
which frame_resolve is connected. Note, if this connector is enabled, it must
be connected.
</p>

<p>
Example: If resolveInFrame = Types.ResolveInFrameAB.frame_a, the output vector is
computed as:
</p>

<pre>
    // Relative orientation object from frame_a to frame_b
    R_rel = MultiBody.Frames.relativeRotation(frame_a.R, frame_b.R);

    // Angular velocity resolved in frame_a
    w_rel = MultiBody.Frames.angularVelocity1(R_rel);
</pre>

</html>"));
        end RelativeAngularVelocity;

        model Distance "Measure the distance between the origins of two frame connectors"
          import Modelica.Mechanics.MultiBody.Frames;
          import Modelica.Mechanics.MultiBody.Types;
          extends Interfaces.PartialTwoFrames;
          extends Modelica.Icons.TranslationalSensor;
          Modelica.Blocks.Interfaces.RealOutput distance "Distance between the origin of frame_a and the origin of frame_b" annotation(
            Placement(transformation(origin = {0, -110}, extent = {{10, -10}, {-10, 10}}, rotation = 90)));
          parameter Boolean animation = true "= true, if animation shall be enabled (show arrow)";
          input SI.Diameter arrowDiameter = world.defaultArrowDiameter "Diameter of relative arrow from frame_a to frame_b" annotation(
            Dialog(group = "if animation = true", enable = animation));
          input Types.Color arrowColor = Modelica.Mechanics.MultiBody.Types.Defaults.SensorColor "Color of relative arrow from frame_a to frame_b" annotation(
            Dialog(colorSelector = true, group = "if animation = true", enable = animation));
          input Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(
            Dialog(group = "if animation = true", enable = animation));
          input SI.Position s_small(min = sqrt(Modelica.Constants.small)) = 1.E-10 "Prevent zero-division if distance between frame_a and frame_b is zero" annotation(
            Dialog(tab = "Advanced"));
        protected
          Modelica.Mechanics.MultiBody.Visualizers.Advanced.Arrow arrow(r = frame_a.r_0, r_head = frame_b.r_0 - frame_a.r_0, diameter = arrowDiameter, color = arrowColor, specularCoefficient = specularCoefficient) if world.enableAnimation and animation;
        protected
          SI.Position r_rel_0[3] = frame_b.r_0 - frame_a.r_0 "Position vector from frame_a to frame_b resolved in world frame";
          SI.Area L2 = r_rel_0 * r_rel_0;
          SI.Area s_small2 = s_small ^ 2;
        equation
          frame_a.f = zeros(3);
          frame_b.f = zeros(3);
          frame_a.t = zeros(3);
          frame_b.t = zeros(3);
          distance = smooth(1, if noEvent(L2 > s_small2) then sqrt(L2) else L2 / (2 * s_small) * (3 - L2 / s_small2));
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{0, -60}, {0, -100}}, color = {0, 0, 255}), Line(points = {{-70, 0}, {-101, 0}}, color = {0, 0, 0}), Line(points = {{70, 0}, {100, 0}}, color = {0, 0, 0}), Text(extent = {{-128, 30}, {133, 78}}, textString = "%name", lineColor = {0, 0, 255})}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-70, 0}, {-101, 0}}, color = {0, 0, 0}), Line(points = {{70, 0}, {100, 0}}, color = {0, 0, 0}), Line(points = {{0, -60}, {0, -100}}, color = {0, 0, 255}), Text(extent = {{-22, 70}, {20, 46}}, textString = "s", lineColor = {0, 0, 255}), Line(points = {{-98, 40}, {88, 40}}, color = {0, 0, 255}), Polygon(points = {{102, 40}, {87, 46}, {87, 34}, {102, 40}}, lineColor = {0, 0, 255}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid)}),
            Documentation(info = "<HTML>
<p>
The <b>distance</b> between the origins of frame_a
and of frame_b are determined and provided at the
output signal connector <b>distance</b>. This
distance is always positive. <b>Derivatives</b> of this
signal can be easily obtained by connecting the
block
<a href=\"modelica://Modelica.Blocks.Continuous.Der\">Modelica.Blocks.Continuous.Der</a>
to \"distance\" (this block performs analytic differentiation
of the input signal using the der(..) operator).
</p>
<p>
In the following figure the animation of a Distance
sensor is shown. The light blue coordinate system is
frame_a, the dark blue coordinate system is frame_b, and
the yellow arrow is the animated sensor.
</p>

<p>
<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Sensors/Distance.png\">
</p>

<p>
If the distance is smaller as parameter <b>s_small</b> (in the \"advanced\" menu),
it is approximated such that its derivative is
finite for zero distance. Without such an approximation, the derivative would
be infinite and a division by zero would occur. The approximation is performed
in the following way: If distance > s_small, it is computed as sqrt(r*r) where
r is the position vector from the origin of frame_a to the origin of frame_b.
If the distance becomes smaller as s_small, the \"sqrt()\" function is approximated
by a second order polynomial, such that the function value and its first derivative
are identical for sqrt() and the polynomial at s_small. Furthermore, the polynomial
passes through zero. The effect is, that the distance function is continuous and
differentiable everywhere. The derivative at zero distance is 3/(2*s_small).
</p>
</HTML>"));
        end Distance;

        model CutForce "Measure cut force vector"
          Modelica.Blocks.Interfaces.RealOutput force[3](each final quantity = "Force", each final unit = "N") "Cut force resolved in frame defined by resolveInFrame" annotation(
            Placement(transformation(origin = {-80, -110}, extent = {{10, -10}, {-10, 10}}, rotation = 90)));
          parameter Boolean animation = true "= true, if animation shall be enabled (show arrow)";
          parameter Boolean positiveSign = true "= true, if force with positive sign is returned (= frame_a.f), otherwise with negative sign (= frame_b.f)";
          input Real N_to_m(unit = "N/m") = 1000 "Force arrow scaling (length = force/N_to_m)" annotation(
            Dialog(group = "if animation = true", enable = animation));
          input SI.Diameter forceDiameter = world.defaultArrowDiameter "Diameter of force arrow" annotation(
            Dialog(group = "if animation = true", enable = animation));
          input Types.Color forceColor = Modelica.Mechanics.MultiBody.Types.Defaults.ForceColor "Color of force arrow" annotation(
            Dialog(colorSelector = true, group = "if animation = true", enable = animation));
          input Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(
            Dialog(group = "if animation = true", enable = animation));
          extends Modelica.Mechanics.MultiBody.Sensors.Internal.PartialCutForceSensor;
        protected
          SI.Position f_in_m[3] = frame_a.f * (if positiveSign then +1 else -1) / N_to_m "Force mapped from N to m for animation";
          Visualizers.Advanced.Arrow forceArrow(diameter = forceDiameter, color = forceColor, specularCoefficient = specularCoefficient, R = frame_b.R, r = frame_b.r_0, r_tail = f_in_m, r_head = -f_in_m) if world.enableAnimation and animation;
          Internal.BasicCutForce cutForce(resolveInFrame = resolveInFrame, positiveSign = positiveSign) annotation(
            Placement(transformation(extent = {{-50, -10}, {-30, 10}})));
          Modelica.Mechanics.MultiBody.Interfaces.ZeroPosition zeroPosition if not resolveInFrame == Modelica.Mechanics.MultiBody.Types.ResolveInFrameA.frame_resolve annotation(
            Placement(transformation(extent = {{0, -40}, {20, -20}})));
        equation
          connect(cutForce.frame_a, frame_a) annotation(
            Line(points = {{-50, 0}, {-100, 0}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
          connect(cutForce.frame_b, frame_b) annotation(
            Line(points = {{-30, 0}, {100, 0}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
          connect(cutForce.frame_resolve, frame_resolve) annotation(
            Line(points = {{-32, -10}, {-32, -60}, {80, -60}, {80, -100}}, color = {95, 95, 95}, pattern = LinePattern.Dot, smooth = Smooth.None));
          connect(cutForce.force, force) annotation(
            Line(points = {{-48, -11}, {-48, -60}, {-80, -60}, {-80, -110}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(zeroPosition.frame_resolve, cutForce.frame_resolve) annotation(
            Line(points = {{0, -30}, {-32, -30}, {-32, -10}}, color = {95, 95, 95}, pattern = LinePattern.Dot, smooth = Smooth.None));
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-190, -70}, {-74, -96}}, lineColor = {0, 0, 0}, textString = "force"), Line(points = {{-80, -100}, {-80, 0}}, color = {0, 0, 127})}),
            Documentation(info = "<HTML>
<p>
The cut-force acting between the two frames to which this
model is connected, is determined and provided at the output signal connector
<b>force</b> (= frame_a.f). If parameter <b>positiveSign</b> =
<b>false</b>, the negative cut-force is provided (= frame_b.f).

<p>
Via parameter <b>resolveInFrame</b> it is defined, in which frame
the force vector is resolved:
</p>

<table border=1 cellspacing=0 cellpadding=2>
<tr><th><b>resolveInFrame =<br>Types.ResolveInFrameAB.</b></th><th><b>Meaning</b></th></tr>
<tr><td valign=\"top\">world</td>
    <td valign=\"top\">Resolve vector in world frame</td></tr>

<tr><td valign=\"top\">frame_a</td>
    <td valign=\"top\">Resolve vector in frame_a</td></tr>

<tr><td valign=\"top\">frame_b</td>
    <td valign=\"top\">Resolve vector in frame_b</td></tr>

<tr><td valign=\"top\">frame_resolve</td>
    <td valign=\"top\">Resolve vector in frame_resolve</td></tr>
</table>

<p>
If resolveInFrame = Types.ResolveInFrameAB.frame_resolve, the conditional connector
\"frame_resolve\" is enabled and output force is resolved in the frame, to
which frame_resolve is connected. Note, if this connector is enabled, it must
be connected.
</p>

<p>
In the following figure the animation of a CutForce
sensor is shown. The dark blue coordinate system is frame_b,
and the green arrow is the cut force acting at frame_b and
with negative sign at frame_a.
</p>

<p>
<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Sensors/CutForce.png\">
</p>
</HTML>"));
        end CutForce;

        model CutTorque "Measure cut torque vector"
          Modelica.Blocks.Interfaces.RealOutput torque[3] "Cut torque resolved in frame defined by resolveInFrame" annotation(
            Placement(transformation(origin = {-80, -110}, extent = {{10, -10}, {-10, 10}}, rotation = 90)));
          parameter Boolean animation = true "= true, if animation shall be enabled (show arrow)";
          parameter Boolean positiveSign = true "= true, if torque with positive sign is returned (= frame_a.t), otherwise with negative sign (= frame_b.t)";
          input Real Nm_to_m(unit = "N.m/m") = 1000 "Torque arrow scaling (length = torque/Nm_to_m)" annotation(
            Dialog(group = "if animation = true", enable = animation));
          input SI.Diameter torqueDiameter = world.defaultArrowDiameter "Diameter of torque arrow" annotation(
            Dialog(group = "if animation = true", enable = animation));
          input Types.Color torqueColor = Modelica.Mechanics.MultiBody.Types.Defaults.TorqueColor "Color of torque arrow" annotation(
            Dialog(colorSelector = true, group = "if animation = true", enable = animation));
          input Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(
            Dialog(group = "if animation = true", enable = animation));
          extends Modelica.Mechanics.MultiBody.Sensors.Internal.PartialCutForceSensor;
        protected
          SI.Position t_in_m[3] = frame_a.t * (if positiveSign then +1 else -1) / Nm_to_m "Torque mapped from Nm to m for animation";
          Visualizers.Advanced.DoubleArrow torqueArrow(diameter = torqueDiameter, color = torqueColor, specularCoefficient = specularCoefficient, R = frame_b.R, r = frame_b.r_0, r_tail = t_in_m, r_head = -t_in_m) if world.enableAnimation and animation;
          Internal.BasicCutTorque cutTorque(resolveInFrame = resolveInFrame, positiveSign = positiveSign) annotation(
            Placement(transformation(extent = {{-62, -10}, {-42, 10}})));
          Modelica.Mechanics.MultiBody.Interfaces.ZeroPosition zeroPosition if not resolveInFrame == Modelica.Mechanics.MultiBody.Types.ResolveInFrameA.frame_resolve annotation(
            Placement(transformation(extent = {{-20, -40}, {0, -20}})));
        equation
          connect(cutTorque.frame_a, frame_a) annotation(
            Line(points = {{-62, 0}, {-100, 0}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
          connect(cutTorque.frame_b, frame_b) annotation(
            Line(points = {{-42, 0}, {100, 0}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
          connect(cutTorque.torque, torque) annotation(
            Line(points = {{-60, -11}, {-60, -80}, {-80, -80}, {-80, -110}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(cutTorque.frame_resolve, frame_resolve) annotation(
            Line(points = {{-44, -10}, {-44, -74}, {80, -74}, {80, -100}}, color = {95, 95, 95}, pattern = LinePattern.Dot, smooth = Smooth.None));
          connect(zeroPosition.frame_resolve, cutTorque.frame_resolve) annotation(
            Line(points = {{-20, -30}, {-44, -30}, {-44, -10}}, color = {95, 95, 95}, pattern = LinePattern.Dot, smooth = Smooth.None));
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-180, -72}, {-64, -98}}, lineColor = {0, 0, 0}, textString = "torque"), Line(points = {{-80, -100}, {-80, 0}}, color = {0, 0, 127})}),
            Documentation(info = "<HTML>
<p>
The cut-torque acting between the two frames to which this
model is connected, is determined and provided at the output signal connector
<b>torque</b> (= frame_a.t). If parameter <b>positiveSign</b> =
<b>false</b>, the negative cut-torque is provided (= frame_b.t).
</p>

<p>
Via parameter <b>resolveInFrame</b> it is defined, in which frame
the torque vector is resolved:
</p>

<table border=1 cellspacing=0 cellpadding=2>
<tr><th><b>resolveInFrame =<br>Types.ResolveInFrameAB.</b></th><th><b>Meaning</b></th></tr>
<tr><td valign=\"top\">world</td>
    <td valign=\"top\">Resolve vector in world frame</td></tr>

<tr><td valign=\"top\">frame_a</td>
    <td valign=\"top\">Resolve vector in frame_a</td></tr>

<tr><td valign=\"top\">frame_b</td>
    <td valign=\"top\">Resolve vector in frame_b</td></tr>

<tr><td valign=\"top\">frame_resolve</td>
    <td valign=\"top\">Resolve vector in frame_resolve</td></tr>
</table>

<p>
If resolveInFrame = Types.ResolveInFrameAB.frame_resolve, the conditional connector
\"frame_resolve\" is enabled and output torque is resolved in the frame, to
which frame_resolve is connected. Note, if this connector is enabled, it must
be connected.
</p>

<p>
In the following figure the animation of a CutTorque
sensor is shown. The dark blue coordinate system is frame_b,
and the green arrow is the cut torque acting at frame_b and
with negative sign at frame_a.
</p>

<p>
<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Sensors/CutTorque.png\">
</p>
</HTML>"));
        end CutTorque;

        model CutForceAndTorque "Measure cut force and cut torque vector"
          import Modelica.Mechanics.MultiBody.Types;
          Modelica.Blocks.Interfaces.RealOutput force[3](each final quantity = "Force", each final unit = "N") "Cut force resolved in frame defined by resolveInFrame" annotation(
            Placement(transformation(origin = {-80, -110}, extent = {{10, -10}, {-10, 10}}, rotation = 90)));
          Modelica.Blocks.Interfaces.RealOutput torque[3] "Cut torque resolved in frame defined by resolveInFrame" annotation(
            Placement(transformation(origin = {0, -110}, extent = {{10, -10}, {-10, 10}}, rotation = 90)));
          parameter Boolean animation = true "= true, if animation shall be enabled (show force and torque arrow)";
          parameter Boolean positiveSign = true "= true, if force and torque with positive sign is returned (= frame_a.f/.t), otherwise with negative sign (= frame_b.f/.t)";
          input Real N_to_m(unit = "N/m") = 1000 "Force arrow scaling (length = force/N_to_m)" annotation(
            Dialog(group = "if animation = true", enable = animation));
          input Real Nm_to_m(unit = "N.m/m") = 1000 "Torque arrow scaling (length = torque/Nm_to_m)" annotation(
            Dialog(group = "if animation = true", enable = animation));
          input SI.Diameter forceDiameter = world.defaultArrowDiameter "Diameter of force arrow" annotation(
            Dialog(group = "if animation = true", enable = animation));
          input SI.Diameter torqueDiameter = forceDiameter "Diameter of torque arrow" annotation(
            Dialog(group = "if animation = true", enable = animation));
          input Types.Color forceColor = Modelica.Mechanics.MultiBody.Types.Defaults.ForceColor "Color of force arrow" annotation(
            Dialog(colorSelector = true, group = "if animation = true", enable = animation));
          input Types.Color torqueColor = Modelica.Mechanics.MultiBody.Types.Defaults.TorqueColor "Color of torque arrow" annotation(
            Dialog(colorSelector = true, group = "if animation = true", enable = animation));
          input Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(
            Dialog(group = "if animation = true", enable = animation));
          extends Modelica.Mechanics.MultiBody.Sensors.Internal.PartialCutForceSensor;
        protected
          parameter Integer csign = if positiveSign then +1 else -1;
          SI.Position f_in_m[3] = frame_a.f * csign / N_to_m "Force mapped from N to m for animation";
          SI.Position t_in_m[3] = frame_a.t * csign / Nm_to_m "Torque mapped from Nm to m for animation";
          Visualizers.Advanced.Arrow forceArrow(diameter = forceDiameter, color = forceColor, specularCoefficient = specularCoefficient, R = frame_b.R, r = frame_b.r_0, r_tail = f_in_m, r_head = -f_in_m) if world.enableAnimation and animation;
          Visualizers.Advanced.DoubleArrow torqueArrow(diameter = torqueDiameter, color = torqueColor, specularCoefficient = specularCoefficient, R = frame_b.R, r = frame_b.r_0, r_tail = t_in_m, r_head = -t_in_m) if world.enableAnimation and animation;
          Internal.BasicCutForce cutForce(resolveInFrame = resolveInFrame, positiveSign = positiveSign) annotation(
            Placement(transformation(extent = {{-60, -10}, {-40, 10}})));
          Internal.BasicCutTorque cutTorque(resolveInFrame = resolveInFrame, positiveSign = positiveSign) annotation(
            Placement(transformation(extent = {{-2, -10}, {18, 10}})));
          Modelica.Mechanics.MultiBody.Interfaces.ZeroPosition zeroPosition if not resolveInFrame == Modelica.Mechanics.MultiBody.Types.ResolveInFrameA.frame_resolve annotation(
            Placement(transformation(extent = {{60, 30}, {80, 50}})));
        equation
          connect(cutForce.frame_a, frame_a) annotation(
            Line(points = {{-60, 0}, {-100, 0}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
          connect(cutForce.frame_b, cutTorque.frame_a) annotation(
            Line(points = {{-40, 0}, {-2, 0}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
          connect(cutTorque.frame_b, frame_b) annotation(
            Line(points = {{18, 0}, {100, 0}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
          connect(cutForce.force, force) annotation(
            Line(points = {{-58, -11}, {-58, -20}, {-80, -20}, {-80, -110}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(cutTorque.torque, torque) annotation(
            Line(points = {{0, -11}, {0, -110}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(zeroPosition.frame_resolve, cutTorque.frame_resolve) annotation(
            Line(points = {{60, 40}, {32, 40}, {32, -20}, {16, -20}, {16, -10}}, color = {95, 95, 95}, pattern = LinePattern.Dot, smooth = Smooth.None));
          connect(zeroPosition.frame_resolve, cutForce.frame_resolve) annotation(
            Line(points = {{60, 40}, {-26, 40}, {-26, -20}, {-42, -20}, {-42, -10}}, color = {95, 95, 95}, pattern = LinePattern.Dot, smooth = Smooth.None));
          connect(cutForce.frame_resolve, frame_resolve) annotation(
            Line(points = {{-42, -10}, {-42, -70}, {80, -70}, {80, -100}}, color = {95, 95, 95}, pattern = LinePattern.Dot, smooth = Smooth.None));
          connect(cutTorque.frame_resolve, frame_resolve) annotation(
            Line(points = {{16, -10}, {16, -70}, {80, -70}, {80, -100}}, color = {95, 95, 95}, pattern = LinePattern.Dot, smooth = Smooth.None));
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-80, -100}, {-80, 0}}, color = {0, 0, 127}), Line(points = {{0, -100}, {0, -70}}, color = {0, 0, 127}), Text(extent = {{-188, -70}, {-72, -96}}, lineColor = {0, 0, 0}, textString = "force"), Text(extent = {{-56, -70}, {60, -96}}, lineColor = {0, 0, 0}, textString = "torque")}),
            Documentation(info = "<HTML>
<p>
The cut-force and cut-torque acting between the two frames to which this
model is connected, are determined and provided at the output signal connectors
<b>force</b> (= frame_a.f) and <b>torque</b> (= frame_a.t).
If parameter <b>positiveSign</b> =
<b>false</b>, the negative cut-force and cut-torque is provided
(= frame_b.f, frame_b.t).

<p>
Via parameter <b>resolveInFrame</b> it is defined, in which frame
the two vectors are resolved:
</p>

<table border=1 cellspacing=0 cellpadding=2>
<tr><th><b>resolveInFrame =<br>Types.ResolveInFrameAB.</b></th><th><b>Meaning</b></th></tr>
<tr><td valign=\"top\">world</td>
    <td valign=\"top\">Resolve vectors in world frame</td></tr>

<tr><td valign=\"top\">frame_a</td>
    <td valign=\"top\">Resolve vectors in frame_a</td></tr>

<tr><td valign=\"top\">frame_b</td>
    <td valign=\"top\">Resolve vectors in frame_b</td></tr>

<tr><td valign=\"top\">frame_resolve</td>
    <td valign=\"top\">Resolve vectors in frame_resolve</td></tr>
</table>

<p>
If resolveInFrame = Types.ResolveInFrameAB.frame_resolve, the conditional connector
\"frame_resolve\" is enabled and the output vectors force and torque are resolved in the frame, to
which frame_resolve is connected. Note, if this connector is enabled, it must
be connected.
</p>

<p>
In the following figure the animation of a CutForceAndTorque
sensor is shown. The dark blue coordinate system is frame_b,
and the green arrows are the cut force and the cut torque,
respectively, acting at frame_b and
with negative sign at frame_a.
</p>

<p>
<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Sensors/CutForceAndTorque.png\">
</p>
</HTML>"));
        end CutForceAndTorque;

        model Power "Measure power flowing from frame_a to frame_b"
          extends Modelica.Icons.RotationalSensor;
          extends Modelica.Mechanics.MultiBody.Interfaces.PartialTwoFrames;
          Modelica.Blocks.Interfaces.RealOutput power(quantity = "Power", unit = "W") "Power at frame_a as output signal" annotation(
            Placement(transformation(origin = {-80, -110}, extent = {{10, -10}, {-10, 10}}, rotation = 90)));
        equation
          Connections.branch(frame_a.R, frame_b.R);
          frame_a.r_0 = frame_b.r_0;
          frame_a.R = frame_b.R;
          zeros(3) = frame_a.f + frame_b.f;
          zeros(3) = frame_a.t + frame_b.t;
          power = frame_a.f * Frames.resolve2(frame_a.R, der(frame_a.r_0)) + frame_a.t * Frames.angularVelocity2(frame_a.R);
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-70, 0}, {-101, 0}}, color = {0, 0, 0}), Line(points = {{70, 0}, {100, 0}}, color = {0, 0, 0}), Line(points = {{-80, 0}, {-80, -100}}, color = {0, 0, 127}), Text(extent = {{-60, -92}, {16, -114}}, lineColor = {0, 0, 0}, textString = "power"), Text(extent = {{-128, 126}, {126, 68}}, textString = "%name", lineColor = {0, 0, 255})}),
            Documentation(info = "<HTML>
<p>
This component provides the power flowing from frame_a to frame_b
as output signal <b>power</b>.
</p>
</HTML>"));
        end Power;

        model TransformAbsoluteVector "Transform absolute vector in to another frame"
          extends Modelica.Icons.RotationalSensor;
          Modelica.Mechanics.MultiBody.Interfaces.Frame_a frame_a "Coordinate system from which absolute kinematic quantities are measured" annotation(
            Placement(transformation(extent = {{-116, -16}, {-84, 16}}, rotation = 0)));
          Modelica.Mechanics.MultiBody.Interfaces.Frame_resolve frame_resolve if frame_r_in == Modelica.Mechanics.MultiBody.Types.ResolveInFrameA.frame_resolve or frame_r_out == Modelica.Mechanics.MultiBody.Types.ResolveInFrameA.frame_resolve "Coordinate system in which r_in or r_out is optionally resolved" annotation(
            Placement(transformation(extent = {{84, -16}, {116, 16}}), iconTransformation(extent = {{84, -15}, {116, 17}})));
          Blocks.Interfaces.RealInput r_in[3] "Input vector resolved in frame defined by frame_r_in" annotation(
            Placement(transformation(extent = {{-20, -20}, {20, 20}}, rotation = -90, origin = {0, 120})));
          Blocks.Interfaces.RealOutput r_out[3] "Input vector r_in resolved in frame defined by frame_r_out" annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = -90, origin = {0, -110})));
          parameter Modelica.Mechanics.MultiBody.Types.ResolveInFrameA frame_r_in = Modelica.Mechanics.MultiBody.Types.ResolveInFrameA.frame_a "Frame in which vector r_in is resolved (world, frame_a, or frame_resolve)";
          parameter Modelica.Mechanics.MultiBody.Types.ResolveInFrameA frame_r_out = frame_r_in "Frame in which vector r_in shall be resolved and provided as r_out (world, frame_a, or frame_resolve)";
        protected
          Internal.BasicTransformAbsoluteVector basicTransformVector(frame_r_in = frame_r_in, frame_r_out = frame_r_out) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}})));
          Modelica.Mechanics.MultiBody.Interfaces.ZeroPosition zeroPosition if not (frame_r_in == Modelica.Mechanics.MultiBody.Types.ResolveInFrameA.frame_resolve or frame_r_out == Modelica.Mechanics.MultiBody.Types.ResolveInFrameA.frame_resolve) annotation(
            Placement(transformation(extent = {{40, 18}, {60, 38}})));
        equation
          connect(basicTransformVector.frame_a, frame_a) annotation(
            Line(points = {{-10, 0}, {-100, 0}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
          connect(basicTransformVector.frame_resolve, frame_resolve) annotation(
            Line(points = {{10, 0}, {100, 0}}, color = {95, 95, 95}, pattern = LinePattern.Dot, smooth = Smooth.None));
          connect(zeroPosition.frame_resolve, basicTransformVector.frame_resolve) annotation(
            Line(points = {{40, 28}, {32, 28}, {32, 0}, {10, 0}}, color = {95, 95, 95}, pattern = LinePattern.Dot, smooth = Smooth.None));
          connect(basicTransformVector.r_out, r_out) annotation(
            Line(points = {{0, -11}, {0, -110}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(basicTransformVector.r_in, r_in) annotation(
            Line(points = {{0, 12}, {0, 120}}, color = {0, 0, 127}, smooth = Smooth.None));
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{0, -70}, {0, -100}}, color = {0, 0, 127}, smooth = Smooth.None), Line(points = {{0, 100}, {0, 70}}, color = {0, 0, 127}, smooth = Smooth.None), Text(extent = {{-104, 124}, {-18, 96}}, lineColor = {0, 0, 0}, textString = "r_in"), Text(extent = {{-124, -76}, {2, -104}}, lineColor = {0, 0, 0}, textString = "r_out"), Line(points = {{95, 0}, {95, 0}, {70, 0}, {70, 0}}, color = {0, 0, 0}, smooth = Smooth.None, pattern = LinePattern.Dot), Text(extent = {{58, 47}, {189, 22}}, lineColor = {95, 95, 95}, textString = "resolve"), Line(points = {{-70, 0}, {-96, 0}, {-96, 0}}, color = {0, 0, 0}, smooth = Smooth.None), Text(extent = {{-116, 45}, {-80, 20}}, lineColor = {95, 95, 95}, textString = "a")}),
            Documentation(info = "<html>
<p>
The input vector \"Real r_in[3]\" is assumed to be an absolute kinematic quantity
of frame_a that is defined to be resolved in the frame defined
with parameter \"frame_r_in\". This model resolves vector r_in in the
coordinate system defined with parameter \"frame_r_out\" and returns the
transformed output vector as \"Real r_out[3]\";
</p>
</html>"));
        end TransformAbsoluteVector;

        model TransformRelativeVector "Transform relative vector in to another frame"
          extends Internal.PartialRelativeSensor;
          Modelica.Mechanics.MultiBody.Interfaces.Frame_resolve frame_resolve if frame_r_in == Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB.frame_resolve or frame_r_out == Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB.frame_resolve "Coordinate system in which r_in or r_out is optionally resolved" annotation(
            Placement(transformation(extent = {{84, 64}, {116, 96}}), iconTransformation(extent = {{84, 65}, {116, 97}})));
          Blocks.Interfaces.RealInput r_in[3] "Input vector resolved in frame defined by frame_r_in" annotation(
            Placement(transformation(extent = {{-20, -20}, {20, 20}}, rotation = -90, origin = {0, 120})));
          Blocks.Interfaces.RealOutput r_out[3] "Input vector r_in resolved in frame defined by frame_r_out" annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = -90, origin = {0, -110})));
          parameter Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB frame_r_in = Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB.frame_a "Frame in which vector r_in is resolved (world, frame_a, frame_b, or frame_resolve)";
          parameter Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB frame_r_out = frame_r_in "Frame in which vector r_in shall be resolved and provided as r_out (world, frame_a, frame_b, or frame_resolve)";
        protected
          Modelica.Mechanics.MultiBody.Sensors.Internal.BasicTransformRelativeVector basicTransformVector(frame_r_in = frame_r_in, frame_r_out = frame_r_out) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}})));
          Modelica.Mechanics.MultiBody.Interfaces.ZeroPosition zeroPosition if not (frame_r_in == Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB.frame_resolve or frame_r_out == Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB.frame_resolve) annotation(
            Placement(transformation(extent = {{48, 10}, {68, 30}})));
        equation
          connect(basicTransformVector.frame_a, frame_a) annotation(
            Line(points = {{-10, 0}, {-100, 0}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
          connect(basicTransformVector.frame_b, frame_b) annotation(
            Line(points = {{10, 0}, {100, 0}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
          connect(basicTransformVector.frame_resolve, frame_resolve) annotation(
            Line(points = {{10, 8.1}, {20, 8.1}, {20, 8}, {32, 8}, {32, 80}, {100, 80}}, color = {95, 95, 95}, pattern = LinePattern.Dot, smooth = Smooth.None));
          connect(zeroPosition.frame_resolve, basicTransformVector.frame_resolve) annotation(
            Line(points = {{48, 20}, {32, 20}, {32, 8.1}, {10, 8.1}}, color = {95, 95, 95}, pattern = LinePattern.Dot, smooth = Smooth.None));
          connect(basicTransformVector.r_out, r_out) annotation(
            Line(points = {{0, -11}, {0, -110}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(basicTransformVector.r_in, r_in) annotation(
            Line(points = {{0, 12}, {0, 120}}, color = {0, 0, 127}, smooth = Smooth.None));
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{0, -70}, {0, -100}}, color = {0, 0, 127}, smooth = Smooth.None), Line(points = {{0, 100}, {0, 70}}, color = {0, 0, 127}, smooth = Smooth.None), Text(extent = {{-104, 124}, {-18, 96}}, lineColor = {0, 0, 0}, textString = "r_in"), Text(extent = {{-124, -76}, {2, -104}}, lineColor = {0, 0, 0}, textString = "r_out")}),
            Documentation(info = "<html>
<p>
The input vector \"Real r_in[3]\" is assumed to be a relative kinematic quantity
between frame_a and frame_b
that is defined to be resolved in the frame defined
with parameter \"frame_r_in\". This model resolves vector r_in in the
coordinate system defined with parameter \"frame_r_out\" and returns the
transformed output vector as \"Real r_out[3]\";
</p>
</html>"));
        end TransformRelativeVector;

        package Internal "Internal package, should not be used by user"
          extends Modelica.Icons.InternalPackage;

          partial model PartialAbsoluteSensor "Partial absolute sensor model for sensors defined by components"
            extends Modelica.Icons.RotationalSensor;
            Modelica.Mechanics.MultiBody.Interfaces.Frame_a frame_a "Coordinate system at which the kinematic quantities are measured" annotation(
              Placement(transformation(extent = {{-116, -16}, {-84, 16}}, rotation = 0)));
          equation
            assert(cardinality(frame_a) > 0, "Connector frame_a must be connected at least once");
            annotation(
              Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-108, 43}, {-72, 18}}, lineColor = {128, 128, 128}, textString = "a"), Line(points = {{-70, 0}, {-96, 0}, {-96, 0}}, color = {0, 0, 0}, smooth = Smooth.None)}));
          end PartialAbsoluteSensor;

          model PartialAbsoluteBaseSensor "Partial absolute sensor models for sensors defined by equations (frame_resolve must be connected exactly once)"
            extends Modelica.Icons.RotationalSensor;
            Modelica.Mechanics.MultiBody.Interfaces.Frame_a frame_a "Coordinate system from which kinematic quantities are measured" annotation(
              Placement(transformation(extent = {{-116, -16}, {-84, 16}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Interfaces.Frame_resolve frame_resolve "Coordinate system in which vector is optionally resolved" annotation(
              Placement(transformation(extent = {{-16, -16}, {16, 16}}, rotation = -90, origin = {10, -100}), iconTransformation(extent = {{-16, -16}, {16, 16}}, rotation = -90, origin = {0, -100})));
          equation
            assert(cardinality(frame_a) > 0, "Connector frame_a must be connected at least once");
            assert(cardinality(frame_resolve) == 1, "Connector frame_resolve must be connected exactly once");
            frame_a.f = zeros(3);
            frame_a.t = zeros(3);
            frame_resolve.f = zeros(3);
            frame_resolve.t = zeros(3);
            annotation(
              Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-108, 43}, {-72, 18}}, lineColor = {95, 95, 95}, textString = "a"), Line(points = {{-70, 0}, {-96, 0}, {-96, 0}}, color = {0, 0, 0}, smooth = Smooth.None), Line(points = {{0, 15}, {0, -15}}, color = {0, 0, 127}, smooth = Smooth.None, origin = {85, 0}, rotation = 90), Line(points = {{0, -95}, {0, -95}, {0, -70}, {0, -70}}, color = {0, 0, 0}, smooth = Smooth.None, pattern = LinePattern.Dot), Text(extent = {{0, -75}, {131, -100}}, lineColor = {95, 95, 95}, textString = "resolve")}));
          end PartialAbsoluteBaseSensor;

          partial model PartialRelativeSensor "Partial relative sensor model for sensors defined by components"
            extends Modelica.Icons.RotationalSensor;
            Modelica.Mechanics.MultiBody.Interfaces.Frame_a frame_a "Coordinate system a" annotation(
              Placement(transformation(extent = {{-116, -16}, {-84, 16}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Interfaces.Frame_b frame_b "Coordinate system b" annotation(
              Placement(transformation(extent = {{84, -16}, {116, 16}}, rotation = 0)));
          equation
            assert(cardinality(frame_a) > 0, "Connector frame_a must be connected at least once");
            assert(cardinality(frame_b) > 0, "Connector frame_b must be connected at least once");
            annotation(
              Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-108, 43}, {-72, 18}}, lineColor = {128, 128, 128}, textString = "a"), Text(extent = {{72, 41}, {108, 16}}, lineColor = {128, 128, 128}, textString = "b"), Line(points = {{-70, 0}, {-96, 0}, {-96, 0}}, color = {0, 0, 0}, smooth = Smooth.None), Line(points = {{96, 0}, {70, 0}, {70, 0}}, color = {0, 0, 0}, smooth = Smooth.None), Line(points = {{60, 36}, {60, 36}, {60, 80}, {95, 80}}, color = {0, 0, 0}, smooth = Smooth.None, pattern = LinePattern.Dot)}));
          end PartialRelativeSensor;

          model PartialRelativeBaseSensor "Partial relative sensor models for sensors defined by equations (frame_resolve must be connected exactly once)"
            extends Modelica.Icons.RotationalSensor;
            Modelica.Mechanics.MultiBody.Interfaces.Frame_a frame_a "Coordinate system a (measurement is between frame_a and frame_b)" annotation(
              Placement(transformation(extent = {{-116, -16}, {-84, 16}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Interfaces.Frame_b frame_b "Coordinate system b (measurement is between frame_a and frame_b)" annotation(
              Placement(transformation(extent = {{84, -16}, {116, 16}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Interfaces.Frame_resolve frame_resolve "Coordinate system in which vector is optionally resolved" annotation(
              Placement(transformation(extent = {{84, 64}, {116, 96}}), iconTransformation(extent = {{84, 65}, {116, 97}})));
          equation
            assert(cardinality(frame_a) > 0, "Connector frame_a must be connected at least once");
            assert(cardinality(frame_b) > 0, "Connector frame_b must be connected at least once");
            assert(cardinality(frame_resolve) == 1, "Connector frame_resolve must be connected exactly once");
            frame_a.f = zeros(3);
            frame_a.t = zeros(3);
            frame_b.f = zeros(3);
            frame_b.t = zeros(3);
            frame_resolve.f = zeros(3);
            frame_resolve.t = zeros(3);
            annotation(
              Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-108, 43}, {-72, 18}}, lineColor = {128, 128, 128}, textString = "a"), Text(extent = {{72, 41}, {108, 16}}, lineColor = {128, 128, 128}, textString = "b"), Line(points = {{-70, 0}, {-96, 0}, {-96, 0}}, color = {0, 0, 0}, smooth = Smooth.None), Line(points = {{96, 0}, {70, 0}, {70, 0}}, color = {0, 0, 0}, smooth = Smooth.None), Line(points = {{0, -70}, {0, -100}}, color = {0, 0, 127}, smooth = Smooth.None), Line(points = {{60, 36}, {60, 36}, {60, 80}, {95, 80}}, color = {0, 0, 0}, smooth = Smooth.None, pattern = LinePattern.Dot)}));
          end PartialRelativeBaseSensor;

          model BasicAbsolutePosition "Measure absolute position vector (same as Sensors.AbsolutePosition, but frame_resolve is not conditional and must be connected)"
            import Modelica.Mechanics.MultiBody.Types.ResolveInFrameA;
            extends Modelica.Mechanics.MultiBody.Sensors.Internal.PartialAbsoluteBaseSensor;
            Modelica.Blocks.Interfaces.RealOutput r[3](each final quantity = "Length", each final unit = "m") "Absolute position vector frame_a.r_0 resolved in frame defined by resolveInFrame" annotation(
              Placement(transformation(origin = {110, 0}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
            parameter Modelica.Mechanics.MultiBody.Types.ResolveInFrameA resolveInFrame = Modelica.Mechanics.MultiBody.Types.ResolveInFrameA.frame_a "Frame in which output vector r is resolved (world, frame_a, or frame_resolve)";
          equation
            if resolveInFrame == ResolveInFrameA.world then
              r = frame_a.r_0;
            elseif resolveInFrame == ResolveInFrameA.frame_a then
              r = Frames.resolve2(frame_a.R, frame_a.r_0);
            elseif resolveInFrame == ResolveInFrameA.frame_resolve then
              r = Frames.resolve2(frame_resolve.R, frame_a.r_0);
            else
              assert(false, "Wrong value for parameter resolveInFrame");
              r = zeros(3);
            end if;
            annotation(
              Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{61, 47}, {145, 17}}, lineColor = {0, 0, 0}, textString = "r"), Text(extent = {{-127, 75}, {134, 123}}, textString = "%name", lineColor = {0, 0, 255})}));
          end BasicAbsolutePosition;

          model BasicAbsoluteAngularVelocity "Measure absolute angular velocity"
            import Modelica.Mechanics.MultiBody.Frames;
            import Modelica.Mechanics.MultiBody.Types.ResolveInFrameA;
            extends Modelica.Mechanics.MultiBody.Sensors.Internal.PartialAbsoluteBaseSensor;
            Modelica.Blocks.Interfaces.RealOutput w[3](each final quantity = "AngularVelocity", each final unit = "rad/s") "Absolute angular velocity vector" annotation(
              Placement(transformation(origin = {110, 0}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
            parameter Modelica.Mechanics.MultiBody.Types.ResolveInFrameA resolveInFrame = Modelica.Mechanics.MultiBody.Types.ResolveInFrameA.frame_a "Frame in which output vector w is resolved (world, frame_a, or frame_resolve)";
          equation
            if resolveInFrame == ResolveInFrameA.world then
              w = Frames.angularVelocity1(frame_a.R);
            elseif resolveInFrame == ResolveInFrameA.frame_a then
              w = Frames.angularVelocity2(frame_a.R);
            elseif resolveInFrame == ResolveInFrameA.frame_resolve then
              w = Frames.resolveRelative(Frames.angularVelocity1(frame_a.R), frame_a.R, frame_resolve.R);
            else
              assert(false, "Wrong value for parameter resolveInFrame");
              w = zeros(3);
            end if;
            annotation(
              Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{62, 48}, {146, 18}}, lineColor = {0, 0, 0}, textString = "w"), Text(extent = {{-130, 76}, {131, 124}}, textString = "%name", lineColor = {0, 0, 255})}));
          end BasicAbsoluteAngularVelocity;

          model BasicRelativePosition "Measure relative position vector (same as Sensors.RelativePosition, but frame_resolve is not conditional and must be connected)"
            import Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB;
            extends Modelica.Mechanics.MultiBody.Sensors.Internal.PartialRelativeBaseSensor;
            Modelica.Blocks.Interfaces.RealOutput r_rel[3](each final quantity = "Length", each final unit = "m") "Relative position vector frame_b.r_0 - frame_a.r_0 resolved in frame defined by resolveInFrame" annotation(
              Placement(transformation(origin = {0, -110}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
            parameter Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB resolveInFrame = Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB.frame_a "Frame in which output vector r_rel is resolved (world, frame_a, frame_b, or frame_resolve)";
          equation
            if resolveInFrame == ResolveInFrameAB.frame_a then
              r_rel = Frames.resolve2(frame_a.R, frame_b.r_0 - frame_a.r_0);
            elseif resolveInFrame == ResolveInFrameAB.frame_b then
              r_rel = Frames.resolve2(frame_b.R, frame_b.r_0 - frame_a.r_0);
            elseif resolveInFrame == ResolveInFrameAB.world then
              r_rel = frame_b.r_0 - frame_a.r_0;
            elseif resolveInFrame == ResolveInFrameAB.frame_resolve then
              r_rel = Frames.resolve2(frame_resolve.R, frame_b.r_0 - frame_a.r_0);
            else
              assert(false, "Wrong value for parameter resolveInFrame");
              r_rel = zeros(3);
            end if;
            annotation(
              Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{12, -76}, {96, -106}}, lineColor = {0, 0, 0}, textString = "r_rel"), Text(extent = {{-127, 95}, {134, 143}}, textString = "%name", lineColor = {0, 0, 255})}));
          end BasicRelativePosition;

          model BasicRelativeAngularVelocity "Measure relative angular velocity"
            import Modelica.Mechanics.MultiBody.Frames;
            import Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB;
            extends Modelica.Mechanics.MultiBody.Sensors.Internal.PartialRelativeBaseSensor;
            Modelica.Blocks.Interfaces.RealOutput w_rel[3](each final quantity = "AngularVelocity", each final unit = "rad/s") "Relative angular velocity vector" annotation(
              Placement(transformation(origin = {0, -110}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
            parameter Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB resolveInFrame = Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB.frame_a "Frame in which output vector w_rel is resolved (world, frame_a, frame_b, or frame_resolve)";
          protected
            Modelica.Mechanics.MultiBody.Frames.Orientation R_rel "Relative orientation object from frame_a to frame_b";
          equation
            R_rel = Frames.relativeRotation(frame_a.R, frame_b.R);
            if resolveInFrame == ResolveInFrameAB.frame_a then
              w_rel = Frames.angularVelocity1(R_rel);
            elseif resolveInFrame == ResolveInFrameAB.frame_b then
              w_rel = Frames.angularVelocity2(R_rel);
            elseif resolveInFrame == ResolveInFrameAB.world then
              w_rel = Frames.resolve1(frame_a.R, Frames.angularVelocity1(R_rel));
            elseif resolveInFrame == ResolveInFrameAB.frame_resolve then
              w_rel = Frames.resolveRelative(Frames.angularVelocity1(R_rel), frame_a.R, frame_resolve.R);
            else
              assert(false, "Wrong value for parameter resolveInFrame");
              w_rel = zeros(3);
            end if;
            annotation(
              Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{12, -76}, {96, -106}}, lineColor = {0, 0, 0}, textString = "w_rel"), Text(extent = {{-132, 90}, {129, 138}}, textString = "%name", lineColor = {0, 0, 255})}));
          end BasicRelativeAngularVelocity;

          model BasicTransformAbsoluteVector "Transform absolute vector in to another frame"
            import Modelica.Mechanics.MultiBody.Frames;
            import Modelica.Mechanics.MultiBody.Types.ResolveInFrameA;
            extends Modelica.Icons.RotationalSensor;
            parameter Modelica.Mechanics.MultiBody.Types.ResolveInFrameA frame_r_in = Modelica.Mechanics.MultiBody.Types.ResolveInFrameA.frame_a "Frame in which vector r_in is resolved (world, frame_a, or frame_resolve)";
            parameter Modelica.Mechanics.MultiBody.Types.ResolveInFrameA frame_r_out = frame_r_in "Frame in which vector r_out (= r_in in other frame) is resolved (world, frame_a, or frame_resolve)";
            Modelica.Mechanics.MultiBody.Interfaces.Frame_a frame_a "Coordinate system from which absolute kinematic quantities are measured" annotation(
              Placement(transformation(extent = {{-116, -16}, {-84, 16}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Interfaces.Frame_resolve frame_resolve "Coordinate system in which vector is optionally resolved" annotation(
              Placement(transformation(extent = {{-16, -16}, {16, 16}}, rotation = 0, origin = {100, 0}), iconTransformation(extent = {{-16, -16}, {16, 16}}, rotation = 0, origin = {100, 0})));
            Blocks.Interfaces.RealInput r_in[3] "Input vector resolved in frame defined by frame_r_in" annotation(
              Placement(transformation(extent = {{-20, -20}, {20, 20}}, rotation = -90, origin = {0, 120})));
            Blocks.Interfaces.RealOutput r_out[3] "Input vector r_in resolved in frame defined by frame_r_out" annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = -90, origin = {0, -110})));
          protected
            Modelica.Mechanics.MultiBody.Frames.Orientation R1 "Orientation object from world frame to frame in which r_in is resolved";
          equation
            assert(cardinality(frame_a) > 0, "Connector frame_a must be connected at least once");
            assert(cardinality(frame_resolve) == 1, "Connector frame_resolve must be connected exactly once");
            frame_a.f = zeros(3);
            frame_a.t = zeros(3);
            frame_resolve.f = zeros(3);
            frame_resolve.t = zeros(3);
            if frame_r_out == frame_r_in then
              r_out = r_in;
              R1 = Frames.nullRotation();
            else
              if frame_r_in == ResolveInFrameA.world then
                R1 = Frames.nullRotation();
              elseif frame_r_in == ResolveInFrameA.frame_a then
                R1 = frame_a.R;
              elseif frame_r_in == ResolveInFrameA.frame_resolve then
                R1 = frame_resolve.R;
              else
                assert(false, "Wrong value for parameter frame_r_in");
                R1 = Frames.nullRotation();
              end if;
              if frame_r_out == ResolveInFrameA.world then
                r_out = Frames.resolve1(R1, r_in);
              elseif frame_r_out == ResolveInFrameA.frame_a then
                r_out = Frames.resolveRelative(r_in, R1, frame_a.R);
              elseif frame_r_out == ResolveInFrameA.frame_resolve then
                r_out = Frames.resolveRelative(r_in, R1, frame_resolve.R);
              else
                assert(false, "Wrong value for parameter frame_r_out");
                r_out = zeros(3);
              end if;
            end if;
            annotation(
              Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-128, -84}, {-2, -112}}, lineColor = {0, 0, 0}, textString = "r_out"), Text(extent = {{-108, 137}, {-22, 109}}, lineColor = {0, 0, 0}, textString = "r_in"), Line(points = {{0, 100}, {0, 70}}, color = {0, 0, 127}, smooth = Smooth.None), Line(points = {{0, -70}, {0, -100}}, color = {0, 0, 127}, smooth = Smooth.None), Text(extent = {{58, 47}, {189, 22}}, lineColor = {95, 95, 95}, textString = "resolve"), Text(extent = {{-116, 45}, {-80, 20}}, lineColor = {95, 95, 95}, textString = "a"), Line(points = {{-70, 0}, {-96, 0}, {-96, 0}}, color = {0, 0, 0}, smooth = Smooth.None), Line(points = {{95, 0}, {95, 0}, {70, 0}, {70, 0}}, color = {0, 0, 0}, smooth = Smooth.None, pattern = LinePattern.Dot)}));
          end BasicTransformAbsoluteVector;

          model BasicTransformRelativeVector "Transform relative vector in to another frame"
            import Modelica.Mechanics.MultiBody.Frames;
            import Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB;
            extends Modelica.Mechanics.MultiBody.Sensors.Internal.PartialRelativeBaseSensor;
            parameter Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB frame_r_in = Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB.frame_a "Frame in which vector r_in is resolved (world, frame_a, frame_b, or frame_resolve)";
            parameter Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB frame_r_out = frame_r_in "Frame in which vector r_out (= r_in in other frame) is resolved (world, frame_a, frame_b, or frame_resolve)";
            Blocks.Interfaces.RealInput r_in[3] "Input vector resolved in frame defined by frame_r_in" annotation(
              Placement(transformation(extent = {{-20, -20}, {20, 20}}, rotation = -90, origin = {0, 120})));
            Blocks.Interfaces.RealOutput r_out[3] "Input vector r_in resolved in frame defined by frame_r_out" annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = -90, origin = {0, -110})));
          protected
            Modelica.Mechanics.MultiBody.Frames.Orientation R1 "Orientation object from world frame to frame in which r_in is resolved";
          equation
            if frame_r_out == frame_r_in then
              r_out = r_in;
              R1 = Frames.nullRotation();
            else
              if frame_r_in == ResolveInFrameAB.world then
                R1 = Frames.nullRotation();
              elseif frame_r_in == ResolveInFrameAB.frame_a then
                R1 = frame_a.R;
              elseif frame_r_in == ResolveInFrameAB.frame_b then
                R1 = frame_b.R;
              else
                R1 = frame_resolve.R;
              end if;
              if frame_r_out == ResolveInFrameAB.world then
                r_out = Frames.resolve1(R1, r_in);
              elseif frame_r_out == ResolveInFrameAB.frame_a then
                r_out = Frames.resolveRelative(r_in, R1, frame_a.R);
              elseif frame_r_out == ResolveInFrameAB.frame_b then
                r_out = Frames.resolveRelative(r_in, R1, frame_b.R);
              else
                r_out = Frames.resolveRelative(r_in, R1, frame_resolve.R);
              end if;
            end if;
            annotation(
              Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-128, -92}, {-2, -120}}, lineColor = {0, 0, 0}, textString = "r_out"), Text(extent = {{-108, 144}, {-22, 116}}, lineColor = {0, 0, 0}, textString = "r_in"), Line(points = {{0, 100}, {0, 70}}, color = {0, 0, 127}, smooth = Smooth.None)}));
          end BasicTransformRelativeVector;

          model ZeroForceAndTorque "Set force and torque to zero"
            extends Modelica.Blocks.Icons.Block;
            Interfaces.Frame_a frame_a annotation(
              Placement(transformation(extent = {{-116, -16}, {-84, 16}})));
          equation
            frame_a.f = zeros(3);
            frame_a.t = zeros(3);
            annotation(
              Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-74, 24}, {80, -20}}, lineColor = {0, 0, 0}, textString = "f = t = 0")}));
          end ZeroForceAndTorque;

          partial model PartialCutForceSensor "Base model to measure the cut force and/or torque between two frames, defined by components"
            extends Modelica.Icons.RotationalSensor;
            Modelica.Mechanics.MultiBody.Interfaces.Frame_a frame_a "Coordinate system a" annotation(
              Placement(transformation(extent = {{-116, -16}, {-84, 16}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Interfaces.Frame_b frame_b "Coordinate system b" annotation(
              Placement(transformation(extent = {{84, -16}, {116, 16}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Interfaces.Frame_resolve frame_resolve if resolveInFrame == Modelica.Mechanics.MultiBody.Types.ResolveInFrameA.frame_resolve "Output vectors are optionally resolved in this frame (cut-force/-torque are set to zero)" annotation(
              Placement(transformation(origin = {80, -100}, extent = {{-16, -16}, {16, 16}}, rotation = 270)));
            parameter Modelica.Mechanics.MultiBody.Types.ResolveInFrameA resolveInFrame = Modelica.Mechanics.MultiBody.Types.ResolveInFrameA.frame_a "Frame in which output vector(s) is/are resolved (world, frame_a, or frame_resolve)";
          protected
            outer Modelica.Mechanics.MultiBody.World world;
          equation
            assert(cardinality(frame_a) > 0, "Connector frame_a of cut-force/-torque sensor object is not connected");
            assert(cardinality(frame_b) > 0, "Connector frame_b of cut-force/-torque sensor object is not connected");
            annotation(
              Documentation(info = "<html>
<p>
This is a base class for 3-dim. mechanical components with two frames
and one output port in order to measure the cut-force and/or
cut-torque acting between the two frames and
to provide the measured signals as output for further processing
with the blocks of package Modelica.Blocks.
</p>
</html>"),
              Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-70, 0}, {-101, 0}}, color = {0, 0, 0}), Line(points = {{70, 0}, {100, 0}}, color = {0, 0, 0}), Text(extent = {{-132, 76}, {129, 124}}, textString = "%name", lineColor = {0, 0, 255}), Text(extent = {{-118, 55}, {-82, 30}}, lineColor = {128, 128, 128}, textString = "a"), Text(extent = {{83, 55}, {119, 30}}, lineColor = {128, 128, 128}, textString = "b"), Text(extent = {{70, -66}, {201, -91}}, lineColor = {95, 95, 95}, textString = "resolve"), Line(points = {{80, 0}, {80, -100}}, color = {95, 95, 95}, pattern = LinePattern.Dot)}));
          end PartialCutForceSensor;

          partial model PartialCutForceBaseSensor "Base model to measure the cut force and/or torque between two frames, defined by equations (frame_resolve must be connected exactly once)"
            extends Modelica.Icons.RotationalSensor;
            Modelica.Mechanics.MultiBody.Interfaces.Frame_a frame_a "Coordinate system a" annotation(
              Placement(transformation(extent = {{-116, -16}, {-84, 16}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Interfaces.Frame_b frame_b "Coordinate system b" annotation(
              Placement(transformation(extent = {{84, -16}, {116, 16}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Interfaces.Frame_resolve frame_resolve "The output vector is optionally resolved in this frame (cut-force/-torque are set to zero)" annotation(
              Placement(transformation(origin = {80, -100}, extent = {{-16, -16}, {16, 16}}, rotation = 270)));
            parameter Modelica.Mechanics.MultiBody.Types.ResolveInFrameA resolveInFrame = Modelica.Mechanics.MultiBody.Types.ResolveInFrameA.frame_a "Frame in which output vector is resolved (world, frame_a, or frame_resolve)";
          protected
            outer Modelica.Mechanics.MultiBody.World world;
          equation
            Connections.branch(frame_a.R, frame_b.R);
            assert(cardinality(frame_a) > 0, "Connector frame_a of cut-force/-torque sensor object is not connected");
            assert(cardinality(frame_b) > 0, "Connector frame_b of cut-force/-torque sensor object is not connected");
// frame_a and frame_b are identical
            frame_a.r_0 = frame_b.r_0;
            frame_a.R = frame_b.R;
// force and torque balance
            zeros(3) = frame_a.f + frame_b.f;
            zeros(3) = frame_a.t + frame_b.t;
            frame_resolve.f = zeros(3);
            frame_resolve.t = zeros(3);
            annotation(
              Documentation(info = "<html>
<p>
This is a base class for 3-dim. mechanical components with two frames
and one output port in order to measure the cut-force and/or
cut-torque acting between the two frames and
to provide the measured signals as output for further processing
with the blocks of package Modelica.Blocks.
</p>
</html>"),
              Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-70, 0}, {-101, 0}}, color = {0, 0, 0}), Line(points = {{70, 0}, {100, 0}}, color = {0, 0, 0}), Text(extent = {{-132, 76}, {129, 124}}, textString = "%name", lineColor = {0, 0, 255}), Text(extent = {{-118, 55}, {-82, 30}}, lineColor = {128, 128, 128}, textString = "a"), Text(extent = {{83, 55}, {119, 30}}, lineColor = {128, 128, 128}, textString = "b"), Text(extent = {{70, -66}, {201, -91}}, lineColor = {95, 95, 95}, textString = "resolve"), Line(points = {{80, 0}, {80, -100}}, color = {95, 95, 95}, pattern = LinePattern.Dot)}));
          end PartialCutForceBaseSensor;

          model BasicCutForce "Measure cut force vector (frame_resolve must be connected)"
            import Modelica.Mechanics.MultiBody.Types.ResolveInFrameA;
            import Modelica.Mechanics.MultiBody.Frames;
            extends Modelica.Mechanics.MultiBody.Sensors.Internal.PartialCutForceBaseSensor;
            Modelica.Blocks.Interfaces.RealOutput force[3](each final quantity = "Force", each final unit = "N") "Cut force resolved in frame defined by resolveInFrame" annotation(
              Placement(transformation(origin = {-80, -110}, extent = {{10, -10}, {-10, 10}}, rotation = 90)));
            parameter Boolean positiveSign = true "= true, if force with positive sign is returned (= frame_a.f), otherwise with negative sign (= frame_b.f)";
          protected
            parameter Integer csign = if positiveSign then +1 else -1;
          equation
            if resolveInFrame == ResolveInFrameA.world then
              force = Frames.resolve1(frame_a.R, frame_a.f) * csign;
            elseif resolveInFrame == ResolveInFrameA.frame_a then
              force = frame_a.f * csign;
            elseif resolveInFrame == ResolveInFrameA.frame_resolve then
              force = Frames.resolveRelative(frame_a.f, frame_a.R, frame_resolve.R) * csign;
            else
              assert(false, "Wrong value for parameter resolveInFrame");
              force = zeros(3);
            end if;
            annotation(
              Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-190, -70}, {-74, -96}}, lineColor = {0, 0, 0}, textString = "force"), Line(points = {{-80, -100}, {-80, 0}}, color = {0, 0, 127})}),
              Documentation(info = "<HTML>

</HTML>"));
          end BasicCutForce;

          model BasicCutTorque "Measure cut torque vector (frame_resolve must be connected)"
            import Modelica.Mechanics.MultiBody.Types.ResolveInFrameA;
            import Modelica.Mechanics.MultiBody.Frames;
            extends Modelica.Mechanics.MultiBody.Sensors.Internal.PartialCutForceBaseSensor;
            Modelica.Blocks.Interfaces.RealOutput torque[3](each final quantity = "Torque", each final unit = "N.m") "Cut torque resolved in frame defined by resolveInFrame" annotation(
              Placement(transformation(origin = {-80, -110}, extent = {{10, -10}, {-10, 10}}, rotation = 90)));
            parameter Boolean positiveSign = true "= true, if torque with positive sign is returned (= frame_a.t), otherwise with negative sign (= frame_b.t)";
          protected
            parameter Integer csign = if positiveSign then +1 else -1;
          equation
            if resolveInFrame == ResolveInFrameA.world then
              torque = Frames.resolve1(frame_a.R, frame_a.t) * csign;
            elseif resolveInFrame == ResolveInFrameA.frame_a then
              torque = frame_a.t * csign;
            elseif resolveInFrame == ResolveInFrameA.frame_resolve then
              torque = Frames.resolveRelative(frame_a.t, frame_a.R, frame_resolve.R) * csign;
            else
              assert(false, "Wrong value for parameter resolveInFrame");
              torque = zeros(3);
            end if;
            annotation(
              Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-190, -70}, {-74, -96}}, lineColor = {0, 0, 0}, textString = "torque"), Line(points = {{-80, -100}, {-80, 0}}, color = {0, 0, 127})}));
          end BasicCutTorque;
        end Internal;

        model TansformAbsoluteVector "Obsolete model will be removed in future versions, use TransformAbsoluteVector instead!"
          extends TransformAbsoluteVector;
          // extends Modelica.Icons.ObsoleteModel;
        end TansformAbsoluteVector;

        model TansformRelativeVector "Obsolete model will be removed in future versions, use TransformRelativeVector instead!"
          extends TransformRelativeVector;
          // extends Modelica.Icons.ObsoleteModel;
        end TansformRelativeVector;
        annotation(
          Documentation(info = "<html>
<p>
Package <b>Sensors</b> contains <b>ideal measurement</b>
components to determine absolute and relative kinematic
quantities, as well as cut-forces, cut-torques and power. All
measured quantities can be provided in every desired
coordinate system.
</p>
</html>"));
      end Sensors;

      package Visualizers "3-dimensional visual objects used for animation"
        extends Modelica.Icons.Package;

        model FixedShape "Visualizing an elementary shape with dynamically varying shape attributes (has one frame connector)"
          import Modelica.Mechanics.MultiBody.Types;
          extends Modelica.Mechanics.MultiBody.Interfaces.PartialVisualizer;
          parameter Boolean animation = true "= true, if animation shall be enabled";
          parameter Types.ShapeType shapeType = "box" "Type of shape" annotation(
            Dialog(group = "if animation = true", enable = animation));
          input SI.Position r_shape[3] = {0, 0, 0} "Vector from frame_a to shape origin, resolved in frame_a" annotation(
            Dialog(group = "if animation = true", enable = animation));
          input Types.Axis lengthDirection = {1, 0, 0} "Vector in length direction of shape, resolved in frame_a" annotation(
            Evaluate = true,
            Dialog(group = "if animation = true", enable = animation));
          input Types.Axis widthDirection = {0, 1, 0} "Vector in width direction of shape, resolved in frame_a" annotation(
            Evaluate = true,
            Dialog(group = "if animation = true", enable = animation));
          input SI.Distance length(start = 1) "Length of shape" annotation(
            Dialog(group = "if animation = true", enable = animation));
          input SI.Distance width(start = 0.1) "Width of shape" annotation(
            Dialog(group = "if animation = true", enable = animation));
          input SI.Distance height(start = 0.1) "Height of shape" annotation(
            Dialog(group = "if animation = true", enable = animation));
          input Modelica.Mechanics.MultiBody.Types.Color color = {0, 128, 255} "Color of shape" annotation(
            Dialog(colorSelector = true, group = "if animation = true", enable = animation));
          input Types.ShapeExtra extra = 0.0 "Additional data for cylinder, cone, pipe, gearwheel and spring" annotation(
            Dialog(group = "if animation = true", enable = animation));
          input Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(
            Dialog(group = "if animation = true", enable = animation));
        protected
          Advanced.Shape vis(shapeType = shapeType, r_shape = r_shape, lengthDirection = lengthDirection, widthDirection = widthDirection, length = length, width = width, height = height, color = color, extra = extra, specularCoefficient = specularCoefficient, r = frame_a.r_0, R = frame_a.R) if world.enableAnimation and animation;
        equation
// No forces and torques
          frame_a.f = zeros(3);
          frame_a.t = zeros(3);
          annotation(
            Documentation(info = "<HTML>
<p>
Model <b>FixedShape</b> defines a visual shape that is
shown at the location of its frame_a.
All describing data such as size and color can vary dynamically by
providing appropriate expressions in the input fields of the
parameter menu. The only exception is parameter shapeType
that cannot be changed during simulation.
The following shapes are currently supported via
parameter <b>shapeType</b> (e.g., shapeType=\"box\"):<br>&nbsp;
</p>

<p>
<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Shape.png\" ALT=\"model Visualizers.FixedShape\">
</p>

<p>
The dark blue arrows in the figure above are directed along
variable <b>lengthDirection</b>. The light blue arrows are directed
along variable <b>widthDirection</b>. The <b>coordinate systems</b>
in the figure represent frame_a of the FixedShape component.
</p>

<p>
Additionally, <b>external shapes</b> can be specified as (not all options might be supported by all tools):
</p>

<ul>
<li> <b>\"1\", \"2\", ...</b><br>
     define external shapes specified in DXF format in files \"1.dxf\", \"2.dxf\", ...
     The DXF-files must be found either in the current directory or in the directory where
     the Shape instance is stored that references the DXF file.
     This (very limited) option should not be used for new models. Example:<br>
    shapeType=\"1\".<br></li>

<li> \"<b>modelica:</b>//&lt;Modelica-name&gt;/&lt;relative-path-file-name&gt;\"<br>
     characterizes the file that is stored under the location of the
     &lt;Modelica-name&gt; library path with the given relative file name.
     Example:<br> shapeType = \"modelica://Modelica/Resources/Data/Shapes/Engine/piston.dxf\".<br></li>

<li> \"<b>file:</b>//&lt;absolute-file-name&gt;\"<br>
     characterizes an absolute file name in the file system. Example:<br>
     shapeType=\"file://C:/users/myname/shapes/piston.dxf\".</li>
</ul>

<p>
The supported file formats are tool dependent. Most tools support
at least DXF-files (a tool might support 3-dim. Face of the DXF format only),
but may support other format as well (such as stl, obj, 3ds).
Since visualization files contain color and other data, the corresponding
information in the model is usually ignored.
</p>

<p>
The sizes of any of the above components are specified by the
<b>length</b>, <b>width</b> and <b>height</b> variables.
Via variable <b>extra</b> additional data can be defined:
</p>
<table border=1 cellspacing=0 cellpadding=2>
<tr><th><b>shapeType</b></th><th>Meaning of parameter <b>extra</b></th></tr>
<tr>
  <td valign=\"top\">\"cylinder\"</td>
  <td valign=\"top\">if extra &gt; 0, a black line is included in the
      cylinder to show the rotation of it.</td>
</tr>
<tr>
  <td valign=\"top\">\"cone\"</td>
  <td valign=\"top\">extra = diameter-left-side / diameter-right-side, i.e.,<br>
      extra = 1: cylinder<br>
      extra = 0: \"real\" cone.</td>
</tr>
<tr>
  <td valign=\"top\">\"pipe\"</td>
  <td valign=\"top\">extra = outer-diameter / inner-diameter, i.e, <br>
      extra = 1: cylinder that is completely hollow<br>
      extra = 0: cylinder without a hole.</td>
</tr>
<tr>
  <td valign=\"top\">\"gearwheel\"</td>
  <td valign=\"top\">extra is the number of teeth of the (external) gear.
If extra &lt; 0, an internal gear is visualized with |extra| teeth.
The axis of the gearwheel is along \"lengthDirection\", and usually:
width = height = 2*radiusOfGearWheel.</td>
</tr>
<tr>
  <td valign=\"top\">\"spring\"</td>
  <td valign=\"top\">extra is the number of windings of the spring.
      Additionally, \"height\" is <b>not</b> the \"height\" but
      2*coil-width.</td>
</tr>
<tr>
  <td valign=\"top\">external shape</td>
  <td valign=\"top\">extra = 0: Visualization from file is not scaled.<br>
                     extra = 1: Visualization from file is scaled with \"length\", \"width\" and height\"
                                of the shape</td>
</tr>
</table>
<p>
Parameter <b>color</b> is a vector with 3 elements,
{r, g, b}, and specifies the color of the shape.
{r,g,b} are the \"red\", \"green\" and \"blue\" color parts.
Note, r g, b are given as Integer[3] in the ranges 0 .. 255,
respectively. The predefined type
<a href=\"modelica://Modelica.Mechanics.MultiBody.Types.Color\">MultiBody.Types.Color</a> contains a menu
definition of the colors used in the MultiBody library together with a color editor.
</p>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{-100, 33}, {-100, -61}, {-10, -41}, {50, -61}, {50, 33}, {-10, 11}, {-100, 33}}, lineColor = {0, 127, 255}, fillColor = {0, 127, 255}, fillPattern = FillPattern.Solid), Polygon(points = {{-98, 34}, {-64, 46}, {0, 30}, {74, 56}, {50, 32}, {-10, 12}, {-98, 34}}, lineColor = {255, 255, 255}, fillColor = {160, 160, 164}, fillPattern = FillPattern.Solid), Polygon(points = {{72, 54}, {50, 32}, {50, -60}, {72, -32}, {72, 54}}, lineColor = {255, 255, 255}, fillColor = {160, 160, 164}, fillPattern = FillPattern.Solid), Text(extent = {{-150, -65}, {150, -95}}, lineColor = {0, 0, 0}, textString = "%shapeType"), Text(extent = {{-150, 100}, {150, 60}}, textString = "%name", lineColor = {0, 0, 255})}));
        end FixedShape;

        model FixedShape2 "Visualizing an elementary shape with dynamically varying shape attributes (has two frame connectors)"
          import Modelica.Mechanics.MultiBody.Frames;
          import Modelica.Mechanics.MultiBody.Types;
          import Modelica.SIunits.Conversions.to_unit1;
          Interfaces.Frame_a frame_a "Coordinate system a (all shape definition vectors are resolved in this frame)" annotation(
            Placement(transformation(extent = {{-116, -16}, {-84, 16}}, rotation = 0)));
          Interfaces.Frame_b frame_b "Coordinate system b" annotation(
            Placement(transformation(extent = {{84, -16}, {116, 16}}, rotation = 0)));
          parameter Boolean animation = true "= true, if animation shall be enabled";
          parameter Types.ShapeType shapeType = "box" "Type of shape" annotation(
            Dialog(group = "if animation = true", enable = animation));
          input SI.Position r[3] = {1, 0, 0} "Vector from frame_a to frame_b resolved in frame_a";
          input SI.Position r_shape[3] = {0, 0, 0} "Vector from frame_a to shape origin, resolved in frame_a" annotation(
            Dialog(group = "if animation = true", enable = animation));
          input Types.Axis lengthDirection = to_unit1(r - r_shape) "Vector in length direction of shape, resolved in frame_a" annotation(
            Evaluate = true,
            Dialog(group = "if animation = true", enable = animation));
          input Types.Axis widthDirection = {0, 1, 0} "Vector in width direction of shape, resolved in frame_a" annotation(
            Evaluate = true,
            Dialog(group = "if animation = true", enable = animation));
          input SI.Length length = Modelica.Math.Vectors.length(r - r_shape) "Length of shape" annotation(
            Dialog(group = "if animation = true", enable = animation));
          input SI.Distance width = 0.1 "Width of shape" annotation(
            Dialog(group = "if animation = true", enable = animation));
          input SI.Distance height = width "Height of shape" annotation(
            Dialog(group = "if animation = true", enable = animation));
          input Types.ShapeExtra extra = 0.0 "Additional data for cylinder, cone, pipe, gearwheel and spring" annotation(
            Dialog(group = "if animation = true", enable = animation));
          input Types.Color color = {0, 128, 255} "Color of shape" annotation(
            Dialog(colorSelector = true, group = "if animation = true", enable = animation));
          input Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(
            Dialog(group = "if animation = true", enable = animation));
        protected
          outer MultiBody.World world;
          Advanced.Shape shape(shapeType = shapeType, r_shape = r_shape, lengthDirection = lengthDirection, widthDirection = widthDirection, length = length, width = width, height = height, extra = extra, color = color, specularCoefficient = specularCoefficient, r = frame_a.r_0, R = frame_a.R) if world.enableAnimation and animation;
        equation
          Connections.branch(frame_a.R, frame_b.R);
          assert(cardinality(frame_a) > 0 or cardinality(frame_b) > 0, "Neither connector frame_a nor frame_b of
MultiBody.Visualizers.FixedShape2 object is connected");
          frame_b.r_0 = frame_a.r_0 + Frames.resolve1(frame_a.R, r);
          frame_b.R = frame_a.R;
/* Force and torque balance */
          zeros(3) = frame_a.f + frame_b.f;
          zeros(3) = frame_a.t + frame_b.t + cross(r, frame_b.f);
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-150, 115}, {150, 75}}, textString = "%name", lineColor = {0, 0, 255}), Text(extent = {{150, -90}, {-150, -60}}, lineColor = {0, 0, 0}, textString = "r=%r"), Polygon(points = {{-100, 50}, {-100, -44}, {-10, -24}, {79, -43}, {80, 49}, {-10, 28}, {-100, 50}}, lineColor = {0, 127, 255}, fillColor = {0, 127, 255}, fillPattern = FillPattern.Solid), Polygon(points = {{-101, 50}, {-65, 62}, {-1, 46}, {100, 70}, {79, 49}, {-11, 28}, {-101, 50}}, lineColor = {255, 255, 255}, fillColor = {160, 160, 164}, fillPattern = FillPattern.Solid), Text(extent = {{-86, 15}, {-50, -10}}, lineColor = {0, 0, 0}, textString = "a"), Text(extent = {{37, 15}, {73, -10}}, lineColor = {0, 0, 0}, textString = "b"), Polygon(points = {{100, 70}, {78, 48}, {78, -44}, {100, -16}, {100, 70}}, lineColor = {255, 255, 255}, fillColor = {160, 160, 164}, fillPattern = FillPattern.Solid)}),
            Documentation(info = "<HTML>
<p>
Model <b>FixedShape2</b> defines a visual shape that is
shown at the location of its frame_a. This model is identical
to <b>FixedShape</b> with the only difference that an
additional frame_b is present which is parallel to frame_a.
This makes it more convenient to connect several visual
shapes together when building up more complex visual
objects. All describing data such as size and color can vary dynamically by
providing appropriate expressions in the input fields of the
parameter menu. The only exception is parameter shapeType
that cannot be changed during simulation.
The following shapes are currently supported via
parameter <b>shapeType</b> (e.g., shapeType=\"box\"):<br>&nbsp;
</p>

<p>
<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Shape.png\" ALT=\"model Visualizers.FixedShape2\">
</p>

<p>&nbsp;<br>
The dark blue arrows in the figure above are directed along
variable <b>lengthDirection</b>. The light blue arrows are directed
along variable <b>widthDirection</b>. The <b>coordinate systems</b>
in the figure represent frame_a of the FixedShape component.
</p>

<p>
Additionally, <b>external shapes</b> can be specified as (not all options might be supported by all tools):
</p>

<ul>
<li> <b>\"1\", \"2\", ...</b><br>
     define external shapes specified in DXF format in files \"1.dxf\", \"2.dxf\", ...
     The DXF-files must be found either in the current directory or in the directory where
     the Shape instance is stored that references the DXF file.
     This (very limited) option should not be used for new models. Example:<br>
    shapeType=\"1\".<br></li>

<li> \"<b>modelica:</b>//&lt;Modelica-name&gt;/&lt;relative-path-file-name&gt;\"<br>
     characterizes the file that is stored under the location of the
     &lt;Modelica-name&gt; library path with the given relative file name.
     Example:<br> shapeType = \"modelica://Modelica/Resources/Data/Shapes/Engine/piston.dxf\".<br></li>

<li> \"<b>file:</b>//&lt;absolute-file-name&gt;\"<br>
     characterizes an absolute file name in the file system. Example:<br>
     shapeType=\"file://C:/users/myname/shapes/piston.dxf\".</li>
</ul>

<p>
The supported file formats are tool dependent. Most tools support
at least DXF-files (a tool might support 3-dim. Face of the DXF format only),
but may support other format as well (such as stl, obj, 3ds).
Since visualization files contain color and other data, the corresponding
information in the model is usually ignored.
</p>

<p>
The sizes of any of the above components are specified by the
<b>length</b>, <b>width</b> and <b>height</b> variables.
Via variable <b>extra</b> additional data can be defined:
</p>
<table border=1 cellspacing=0 cellpadding=2>
<tr><th><b>shapeType</b></th><th>Meaning of parameter <b>extra</b></th></tr>
<tr>
  <td valign=\"top\">\"cylinder\"</td>
  <td valign=\"top\">if extra &gt; 0, a black line is included in the
      cylinder to show the rotation of it.</td>
</tr>
<tr>
  <td valign=\"top\">\"cone\"</td>
  <td valign=\"top\">extra = diameter-left-side / diameter-right-side, i.e.,<br>
      extra = 1: cylinder<br>
      extra = 0: \"real\" cone.</td>
</tr>
<tr>
  <td valign=\"top\">\"pipe\"</td>
  <td valign=\"top\">extra = outer-diameter / inner-diameter, i.e, <br>
      extra = 1: cylinder that is completely hollow<br>
      extra = 0: cylinder without a hole.</td>
</tr>
<tr>
  <td valign=\"top\">\"gearwheel\"</td>
  <td valign=\"top\">extra is the number of teeth of the (external) gear.
If extra &lt; 0, an internal gear is visualized with |extra| teeth.
The axis of the gearwheel is along \"lengthDirection\", and usually:
width = height = 2*radiusOfGearWheel.</td>
</tr>
<tr>
  <td valign=\"top\">\"spring\"</td>
  <td valign=\"top\">extra is the number of windings of the spring.
      Additionally, \"height\" is <b>not</b> the \"height\" but
      2*coil-width.</td>
</tr>
<tr>
  <td valign=\"top\">external shape</td>
  <td valign=\"top\">extra = 0: Visualization from file is not scaled.<br>
                     extra = 1: Visualization from file is scaled with \"length\", \"width\" and height\"
                                of the shape</td>
</tr>
</table>
<p>
Parameter <b>color</b> is a vector with 3 elements,
{r, g, b}, and specifies the color of the shape.
{r,g,b} are the \"red\", \"green\" and \"blue\" color parts.
Note, r g, b are given as Integer[3] in the ranges 0 .. 255,
respectively. The predefined type
<a href=\"modelica://Modelica.Mechanics.MultiBody.Types.Color\">MultiBody.Types.Color</a> contains a menu
definition of the colors used in the MultiBody library together with a color editor.
</p>
<p>
In the following figure the relationships between
frame_a and frame_b are shown. The origin of frame_b
with respect to frame_a is specified via parameter
vector <b>r</b>.
</p>

<p>
<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/FixedTranslation.png\" ALT=\"Parts.FixedTranslation\">
</p>
</html>"));
        end FixedShape2;

        model FixedFrame "Visualizing a coordinate system including axes labels (visualization data may vary dynamically)"
          import Modelica.Mechanics.MultiBody.Types;
          extends Modelica.Mechanics.MultiBody.Interfaces.PartialVisualizer;
          parameter Boolean animation = true "= true, if animation shall be enabled";
          parameter Boolean showLabels = true "= true, if labels shall be shown" annotation(
            Dialog(group = "if animation = true", enable = animation));
          input SI.Distance length = 0.5 "Length of axes arrows" annotation(
            Dialog(group = "if animation = true", enable = animation));
          input SI.Distance diameter = length / world.defaultFrameDiameterFraction "Diameter of axes arrows" annotation(
            Dialog(group = "if animation = true", enable = animation));
          input Types.Color color_x = Modelica.Mechanics.MultiBody.Types.Defaults.FrameColor "Color of x-arrow" annotation(
            Dialog(colorSelector = true, group = "if animation = true", enable = animation));
          input Types.Color color_y = color_x "Color of y-arrow" annotation(
            Dialog(colorSelector = true, group = "if animation = true", enable = animation));
          input Types.Color color_z = color_x "Color of z-arrow" annotation(
            Dialog(colorSelector = true, group = "if animation = true", enable = animation));
          input Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(
            Dialog(group = "if animation = true", enable = animation));
        protected
          parameter Boolean animation2 = world.enableAnimation and animation;
          parameter Boolean showLabels2 = world.enableAnimation and animation and showLabels;
          // Parameters to define axes
          SI.Length headLength = min(length, diameter * Types.Defaults.FrameHeadLengthFraction);
          SI.Length headWidth = diameter * Types.Defaults.FrameHeadWidthFraction;
          SI.Length lineLength = max(0, length - headLength);
          SI.Length lineWidth = diameter;
          // Parameters to define axes labels
          SI.Length scaledLabel = Modelica.Mechanics.MultiBody.Types.Defaults.FrameLabelHeightFraction * diameter;
          SI.Length labelStart = 1.05 * length;
          // x-axis
          Visualizers.Advanced.Shape x_arrowLine(shapeType = "cylinder", length = lineLength, width = lineWidth, height = lineWidth, lengthDirection = {1, 0, 0}, widthDirection = {0, 1, 0}, color = color_x, specularCoefficient = specularCoefficient, r = frame_a.r_0, R = frame_a.R) if animation2;
          Visualizers.Advanced.Shape x_arrowHead(shapeType = "cone", length = headLength, width = headWidth, height = headWidth, lengthDirection = {1, 0, 0}, widthDirection = {0, 1, 0}, color = color_x, specularCoefficient = specularCoefficient, r = frame_a.r_0 + Frames.resolve1(frame_a.R, {lineLength, 0, 0}), R = frame_a.R) if animation2;
          Visualizers.Internal.Lines x_label(lines = scaledLabel * {[0, 0; 1, 1], [0, 1; 1, 0]}, diameter = diameter, color = color_x, specularCoefficient = specularCoefficient, r_lines = {labelStart, 0, 0}, n_x = {1, 0, 0}, n_y = {0, 1, 0}, r = frame_a.r_0, R = frame_a.R) if showLabels2;
          // y-axis
          Visualizers.Advanced.Shape y_arrowLine(shapeType = "cylinder", length = lineLength, width = lineWidth, height = lineWidth, lengthDirection = {0, 1, 0}, widthDirection = {1, 0, 0}, color = color_y, specularCoefficient = specularCoefficient, r = frame_a.r_0, R = frame_a.R) if animation2;
          Visualizers.Advanced.Shape y_arrowHead(shapeType = "cone", length = headLength, width = headWidth, height = headWidth, lengthDirection = {0, 1, 0}, widthDirection = {1, 0, 0}, color = color_y, specularCoefficient = specularCoefficient, r = frame_a.r_0 + Frames.resolve1(frame_a.R, {0, lineLength, 0}), R = frame_a.R) if animation2;
          Visualizers.Internal.Lines y_label(lines = scaledLabel * {[0, 0; 1, 1.5], [0, 1.5; 0.5, 0.75]}, diameter = diameter, color = color_y, specularCoefficient = specularCoefficient, r_lines = {0, labelStart, 0}, n_x = {0, 1, 0}, n_y = {-1, 0, 0}, r = frame_a.r_0, R = frame_a.R) if showLabels2;
          // z-axis
          Visualizers.Advanced.Shape z_arrowLine(shapeType = "cylinder", length = lineLength, width = lineWidth, height = lineWidth, lengthDirection = {0, 0, 1}, widthDirection = {0, 1, 0}, color = color_z, specularCoefficient = specularCoefficient, r = frame_a.r_0, R = frame_a.R) if animation2;
          Visualizers.Advanced.Shape z_arrowHead(shapeType = "cone", length = headLength, width = headWidth, height = headWidth, lengthDirection = {0, 0, 1}, widthDirection = {0, 1, 0}, color = color_z, specularCoefficient = specularCoefficient, r = frame_a.r_0 + Frames.resolve1(frame_a.R, {0, 0, lineLength}), R = frame_a.R) if animation2;
          Visualizers.Internal.Lines z_label(lines = scaledLabel * {[0, 0; 1, 0], [0, 1; 1, 1], [0, 1; 1, 0]}, diameter = diameter, color = color_z, specularCoefficient = specularCoefficient, r_lines = {0, 0, labelStart}, n_x = {0, 0, 1}, n_y = {0, 1, 0}, r = frame_a.r_0, R = frame_a.R) if showLabels2;
        equation
          frame_a.f = zeros(3);
          frame_a.t = zeros(3);
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, 100}, {100, -100}}, lineColor = {0, 127, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Polygon(points = {{-2, 92}, {-14, 52}, {12, 52}, {-2, 92}, {-2, 92}}, lineColor = {0, 191, 0}, fillColor = {0, 191, 0}, fillPattern = FillPattern.Solid), Line(points = {{-2, -18}, {-2, 52}}, color = {0, 191, 0}, thickness = 0.5), Text(extent = {{16, 93}, {67, 43}}, lineColor = {0, 191, 0}, textString = "y"), Text(extent = {{43, 11}, {92, -38}}, lineColor = {0, 0, 0}, textString = "x"), Polygon(points = {{98, -70}, {74, -44}, {64, -60}, {98, -70}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Line(points = {{-2, -18}, {72, -54}}, color = {0, 0, 0}, thickness = 0.5), Line(points = {{-72, -54}, {-2, -18}}, thickness = 0.5, color = {0, 0, 255}), Polygon(points = {{-98, -68}, {-66, -60}, {-78, -46}, {-98, -68}}, lineColor = {0, 0, 255}), Text(extent = {{-87, 13}, {-38, -36}}, textString = "z", lineColor = {0, 0, 255}), Text(extent = {{-150, 145}, {150, 105}}, textString = "%name", lineColor = {0, 0, 255})}),
            Documentation(info = "<HTML>
<p>
Model <b>FixedFrame</b> visualizes the three axes of
its coordinate system <b>frame_a</b> together with appropriate axes
labels. A typical example is shown in the following figure:
<br>&nbsp;
</p>

<p>
<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/FixedFrame.png\" ALT=\"model Visualizers.FixedFrame\">
</p>

<p>
The sizes of the axes, the axes colors and the specular coefficient
(= reflection factor for
ambient light) can vary dynamically by
providing appropriate expressions in the input fields of the
parameter menu.
</p>
</HTML>"));
        end FixedFrame;

        model FixedArrow "Visualizing an arrow with dynamically varying size in frame_a"
          import Modelica.Mechanics.MultiBody.Types;
          extends Modelica.Mechanics.MultiBody.Interfaces.PartialVisualizer;
          parameter Boolean animation = true "= true, if animation shall be enabled";
          input SI.Position r_tail[3] = {0, 0, 0} "Vector from frame_a to arrow tail, resolved in frame_a" annotation(
            Dialog(group = "if animation = true", enable = animation));
          input Types.Axis n = {1, 0, 0} "Vector in arrow direction, resolved in frame_a" annotation(
            Dialog(group = "if animation = true", enable = animation));
          input SI.Length length = 0.1 "Length of complete arrow" annotation(
            Dialog(group = "if animation = true", enable = animation));
          input SI.Diameter diameter = world.defaultArrowDiameter "Diameter of arrow line" annotation(
            Dialog(group = "if animation = true", enable = animation));
          input Types.Color color = {0, 0, 255} "Color of arrow" annotation(
            Dialog(colorSelector = true, group = "if animation = true", enable = animation));
          input Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(
            Dialog(group = "if animation = true", enable = animation));
        protected
          SI.Length headLength = min(length, diameter * Types.Defaults.ArrowHeadLengthFraction);
          SI.Length headWidth = diameter * Types.Defaults.ArrowHeadWidthFraction;
          SI.Length lineLength = max(0, length - headLength);
          Visualizers.Advanced.Shape arrowLine(shapeType = "cylinder", length = lineLength, width = diameter, height = diameter, lengthDirection = n, widthDirection = {0, 1, 0}, color = color, specularCoefficient = specularCoefficient, r_shape = r_tail, r = frame_a.r_0, R = frame_a.R) if world.enableAnimation and animation;
          Visualizers.Advanced.Shape arrowHead(shapeType = "cone", length = headLength, width = headWidth, height = headWidth, lengthDirection = n, widthDirection = {0, 1, 0}, color = color, specularCoefficient = specularCoefficient, r_shape = r_tail + Modelica.Math.Vectors.normalize(n) * lineLength, r = frame_a.r_0, R = frame_a.R) if world.enableAnimation and animation;
        equation
          frame_a.f = zeros(3);
          frame_a.t = zeros(3);
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, 28}, {20, -30}}, lineColor = {128, 128, 128}, fillColor = {128, 128, 128}, fillPattern = FillPattern.Solid), Polygon(points = {{20, 60}, {100, 0}, {20, -60}, {20, 60}}, lineColor = {128, 128, 128}, fillColor = {128, 128, 128}, fillPattern = FillPattern.Solid), Text(extent = {{-150, 105}, {150, 65}}, textString = "%name", lineColor = {0, 0, 255}), Text(extent = {{-150, -75}, {150, -105}}, lineColor = {0, 0, 0}, textString = "%length")}),
            Documentation(info = "<html>
<p>
Model <b>FixedArrow</b> defines an arrow that is
shown at the location of its frame_a.
<br>&nbsp;
</p>

<p>
<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Visualizers/Arrow.png\" ALT=\"model Visualizers.FixedArrow\">
</p>

<p>
The direction of the arrow specified with vector
<b>n</b> is with respect to frame_a, i.e., the local frame to which the
arrow component is attached. The direction and length of the arrow, its diameter
and color can vary dynamically by
providing appropriate expressions in the input fields of the
parameter menu.
</p>
</html>"));
        end FixedArrow;

        model SignalArrow "Visualizing an arrow with dynamically varying size in frame_a based on input signal"
          import Modelica.Mechanics.MultiBody.Types;
          extends Modelica.Mechanics.MultiBody.Interfaces.PartialVisualizer;
          parameter Boolean animation = true "= true, if animation shall be enabled";
          input SI.Position r_tail[3] = {0, 0, 0} "Vector from frame_a to arrow tail, resolved in frame_a" annotation(
            Dialog(group = "if animation = true", enable = animation));
          input SI.Diameter diameter = world.defaultArrowDiameter "Diameter of arrow line" annotation(
            Dialog(group = "if animation = true", enable = animation));
          input Modelica.Mechanics.MultiBody.Types.Color color = {0, 0, 255} "Color of arrow" annotation(
            Dialog(colorSelector = true, group = "if animation = true", enable = animation));
          input Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(
            Dialog(group = "if animation = true", enable = animation));
          Modelica.Blocks.Interfaces.RealInput r_head[3](each final quantity = "Length", each final unit = "m") "Position vector from origin of frame_a to head of arrow, resolved in frame_a" annotation(
            Placement(transformation(origin = {0, -120}, extent = {{-20, -20}, {20, 20}}, rotation = 90)));
        protected
          Visualizers.Advanced.Arrow arrow(R = frame_a.R, r = frame_a.r_0, r_tail = r_tail, r_head = r_head, diameter = diameter, color = color, specularCoefficient = specularCoefficient) if world.enableAnimation and animation;
        equation
          frame_a.f = zeros(3);
          frame_a.t = zeros(3);
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, 28}, {20, -30}}, lineColor = {128, 128, 128}, fillColor = {128, 128, 128}, fillPattern = FillPattern.Solid), Polygon(points = {{20, 60}, {100, 0}, {20, -60}, {20, 60}}, lineColor = {128, 128, 128}, fillColor = {128, 128, 128}, fillPattern = FillPattern.Solid), Text(extent = {{-150, 105}, {150, 65}}, textString = "%name", lineColor = {0, 0, 255}), Line(points = {{0, -102}, {0, -30}}, color = {0, 0, 255})}),
            Documentation(info = "<HTML>
<p>
Model <b>SignalArrow</b> defines an arrow that is dynamically visualized
at the location where its frame_a is attached. The
position vector from the tail to the head of the arrow,
resolved in frame_a, is defined via the signal vector of
the connector <b>r_head</b> (Real r_head[3]):<br>&nbsp;
</p>

<p>
<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Visualizers/Arrow.png\" ALT=\"model Visualizers.SignalArrow\">
</p>
<p>
The tail of the arrow is defined with parameter <b>r_tail</b>
with respect to frame_a (vector from the origin of frame_a to the arrow tail).
</p>
</HTML>"));
        end SignalArrow;

        model Ground "Visualizing the ground (box in z=0)"
          parameter Boolean animation = true "= true, if animation of ground shall be enabled";
          parameter Modelica.SIunits.Position length = 10 "Length and width of box (center is at x=y=0)" annotation(
            Dialog(enable = animation));
          parameter Modelica.SIunits.Position height = 0.02 "Height of box (upper surface is at z=0, lower surface is at z=-height)" annotation(
            Dialog(enable = animation));
          parameter Modelica.Mechanics.MultiBody.Types.Color groundColor = {0, 255, 0} "Color of box" annotation(
            Dialog(colorSelector = true, enable = animation));
          Modelica.Mechanics.MultiBody.Visualizers.FixedShape ground(lengthDirection = {1, 0, 0}, widthDirection = {0, 1, 0}, animation = animation, r_shape = {-length / 2, 0, -height}, length = length, height = height, color = groundColor, width = length) annotation(
            Placement(transformation(extent = {{-20, 0}, {0, 20}})));
          Modelica.Mechanics.MultiBody.Parts.Fixed fixed annotation(
            Placement(transformation(extent = {{-60, 0}, {-40, 20}})));
        equation
          connect(fixed.frame_b, ground.frame_a) annotation(
            Line(points = {{-40, 10}, {-20, 10}}, color = {95, 95, 95}, thickness = 0.5, smooth = Smooth.None));
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{-20, 60}, {-100, -32}, {20, -92}, {100, 8}, {-20, 60}}, lineColor = {0, 0, 0}, smooth = Smooth.None, fillColor = {0, 255, 0}, fillPattern = FillPattern.Solid), Text(extent = {{2, 82}, {44, 54}}, lineColor = {0, 0, 0}, textString = "z"), Polygon(points = {{100, 8}, {100, -4}, {20, -104}, {20, -92}, {100, 8}}, lineColor = {0, 0, 0}, smooth = Smooth.None, fillColor = {0, 255, 0}, fillPattern = FillPattern.Solid), Polygon(points = {{-100, -32}, {-100, -44}, {20, -104}, {20, -92}, {-100, -32}}, lineColor = {0, 0, 0}, smooth = Smooth.None, fillColor = {0, 255, 0}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-11, -7}, {-1, -12}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{-6, -10}, {-6, 88}}, color = {0, 0, 0}, smooth = Smooth.None), Polygon(points = {{-6, 102}, {-14, 72}, {2, 72}, {-6, 102}}, lineColor = {0, 0, 0}, smooth = Smooth.None, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Text(extent = {{-150, -105}, {150, -145}}, textString = "%name", lineColor = {0, 0, 255})}),
            Documentation(info = "<html>
<p>
This shape visualizes the x-y plane by a box
</p>

<blockquote>
<img src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Visualizers/Ground.png\">
</blockquote>
</html>"));
        end Ground;

        model Torus "Visualizing a torus"
          extends Modelica.Mechanics.MultiBody.Interfaces.PartialVisualizer;
          parameter Boolean animation = true "= true, if animation shall be enabled";
          parameter Modelica.SIunits.Radius ri = 0.5 "Inner radius of torus" annotation(
            Dialog(enable = animation));
          parameter Modelica.SIunits.Radius ro = 0.1 "Outer radius of torus (=width/2)" annotation(
            Dialog(enable = animation));
          parameter Modelica.SIunits.Angle opening = 0 "Opening angle of torus" annotation(
            Dialog(enable = animation));
          parameter Modelica.SIunits.Angle startAngle = -3.1415926535898 "Start angle of torus slice" annotation(
            Dialog(enable = animation));
          parameter Modelica.SIunits.Angle stopAngle = 3.1415926535898 "End angle of torus slice" annotation(
            Dialog(enable = animation));
          parameter Boolean wireframe = false "= true: 3D model will be displayed without faces" annotation(
            Dialog(enable = animation, group = "Material properties"),
            choices(checkBox = true));
          input Modelica.Mechanics.MultiBody.Types.RealColor color = {0, 128, 255} "Color of surface" annotation(
            Dialog(enable = animation and not multiColoredSurface, colorSelector = true, group = "Material properties"));
          input Types.SpecularCoefficient specularCoefficient = 0.7 "Reflection of ambient light (= 0: light is completely absorbed)" annotation(
            Dialog(enable = animation, group = "Material properties"));
          input Real transparency = 0 "Transparency of shape: 0 (= opaque) ... 1 (= fully transparent)" annotation(
            Dialog(enable = animation, group = "Material properties"));
          parameter Integer n_ri = 40 "Number of points along ri" annotation(
            Dialog(enable = animation, tab = "Discretization"));
          parameter Integer n_ro = 20 "Number of points along ro" annotation(
            Dialog(enable = animation, tab = "Discretization"));
        protected
          Advanced.Surface surface(redeclare function surfaceCharacteristic = Modelica.Mechanics.MultiBody.Visualizers.Advanced.SurfaceCharacteristics.torus(ri = ri, ro = ro, opening = opening, startAngle = startAngle, stopAngle = stopAngle), nu = n_ri, nv = n_ro, multiColoredSurface = false, wireframe = wireframe, color = color, specularCoefficient = specularCoefficient, transparency = transparency, R = frame_a.R, r_0 = frame_a.r_0) if world.enableAnimation and animation annotation(
            Placement(transformation(extent = {{-20, -10}, {0, 10}})));
        equation
// No forces and torques
          frame_a.f = zeros(3);
          frame_a.t = zeros(3);
          annotation(
            Icon(graphics = {Bitmap(extent = {{-98, -98}, {98, 98}}, fileName = "modelica://Modelica/Resources/Images/Mechanics/MultiBody/Visualizers/TorusIcon.png"), Text(extent = {{-150, 100}, {150, 140}}, lineColor = {0, 0, 255}, textString = "%name")}),
            Documentation(info = "<html>
<p>
Model <b>Torus</b> visualizes a torus. The center of the torus is located at
connector frame_a (visualized by the red coordinate system in the figure below).
The left image below shows a torus with ri=0.5 m and ro = 0.2 m.
The right images below shows the torus with the additional parameter
settings:
</p>
<pre>
  opening    =   45 degree
  startAngle = -135 degree
  stopAngle  =  135 degree
</pre>

<blockquote>
<img src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Visualizers/Torus.png\">
</blockquote>

<p>
In the advanced menu the discretization of the surface visualization can be defined by
the number of points of the inner radius of the torus (n_ri) and by
the number of points of the outer radius of the torus (n_ro).
In case the torus is closed (that is, opening = 0 degree),
the actual number of points is one less (that is n_ri-1, n_ro-1), because the first and
the last point of the parametrization coincide in this case.
</p>
</html>", revisions = "<html>
  <ul>
  <li> July 2010 by Martin Otter<br>
       Adapted to the new Surface model.</li>
  <li> July 2005 by Dirk Zimmer (practical training at DLR)<br>
       First version to visualize a multi-level tyre wheel model.</li>
  </ul>
</html>"));
        end Torus;

        model VoluminousWheel "Visualizing a voluminous wheel"
          extends Modelica.Mechanics.MultiBody.Interfaces.PartialVisualizer;
          parameter Boolean animation = true "= true, if animation shall be enabled";
          parameter SI.Radius rTire = 0.25 "Radius of the tire";
          parameter SI.Radius rRim = 0.14 "Radius of the rim";
          parameter SI.Radius width = 0.25 "Width of the tire";
          parameter SI.Radius rCurvature = 0.30 "Radius of the curvature of the tire";
          parameter Modelica.Mechanics.MultiBody.Types.RealColor color = {64, 64, 64} "Color of tire" annotation(
            Dialog(enable = animation and not multiColoredSurface, colorSelector = true, group = "Material properties"));
          parameter Types.SpecularCoefficient specularCoefficient = 0.5 "Reflection of ambient light (= 0: light is completely absorbed)" annotation(
            Dialog(enable = animation, group = "Material properties"));
          parameter Integer n_rTire = 40 "Number of points along rTire" annotation(
            Dialog(enable = animation, tab = "Discretization"));
          parameter Integer n_rCurvature = 20 "Number of points along rCurvature" annotation(
            Dialog(enable = animation, tab = "Discretization"));
        protected
          parameter SI.Radius rw = width / 2;
          parameter SI.Radius rCurvature2 = if rCurvature > rw then rCurvature else rw;
          parameter SI.Radius h = sqrt(1 - rw / rCurvature2 * (rw / rCurvature2)) * rCurvature2;
          parameter SI.Radius ri = rTire - rCurvature2;
          parameter SI.Radius rRim2 = if rRim < 0 then 0 else if rRim > ri + h then ri + h else rRim;
          Visualizers.Advanced.Shape pipe(shapeType = "pipe", color = color, length = width, width = 2 * (ri + h), height = 2 * (ri + h), lengthDirection = {0, 1, 0}, widthDirection = {0, 0, 1}, extra = rRim2 / (ri + h), r = frame_a.r_0, r_shape = -{0, 1, 0} * (width / 2), R = frame_a.R, specularCoefficient = specularCoefficient) if world.enableAnimation and animation annotation(
            Placement(transformation(extent = {{-20, -10}, {0, 10}})));
          Visualizers.Advanced.Surface torus(redeclare function surfaceCharacteristic = Modelica.Mechanics.MultiBody.Visualizers.Advanced.SurfaceCharacteristics.torus(ri = ri, ro = rCurvature2, opening = Modelica.Constants.pi - Modelica.Math.asin(rw / rCurvature2)), nu = n_rTire, nv = n_rCurvature, multiColoredSurface = false, wireframe = false, color = color, specularCoefficient = specularCoefficient, transparency = 0, R = frame_a.R, r_0 = frame_a.r_0) if world.enableAnimation and animation annotation(
            Placement(transformation(extent = {{-50, -10}, {-30, 10}})));
        equation
// No forces and torques
          frame_a.f = zeros(3);
          frame_a.t = zeros(3);
          annotation(
            Icon(graphics = {Text(extent = {{-150, 100}, {150, 140}}, lineColor = {0, 0, 255}, textString = "%name"), Bitmap(extent = {{-96, -96}, {102, 96}}, fileName = "modelica://Modelica/Resources/Images/Mechanics/MultiBody/Visualizers/VoluminousWheelIcon.png"), Rectangle(extent = {{-96, 8}, {-18, -8}}, lineColor = {95, 95, 95}, fillColor = {215, 215, 215}, fillPattern = FillPattern.HorizontalCylinder)}),
            Documentation(info = "<html>
<p>
Model <b>VoluminousWheel</b> provides a simple visualization of a tire using
a torus and a pipe shape object. The center of the wheel is located at
connector frame_a (visualized by the red coordinate system in the figure below).
</p>

<blockquote>
<img src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Visualizers/VoluminousWheel.png\">
</blockquote>
</html>", revisions = "<html>
  <ul>
  <li> July 2010 by Martin Otter<br>
       Adapted to the new Surface model.</li>
  <li> July 2005 by Dirk Zimmer (practical training at DLR)<br>
       First version to visualize a multi-level tyre wheel model.</li>
  </ul>
</html>"));
        end VoluminousWheel;

        model PipeWithScalarField "Visualizing a pipe with scalar field quantities along the pipe axis"
          extends Modelica.Mechanics.MultiBody.Interfaces.PartialVisualizer;
          parameter Boolean animation = true "= true, if animation shall be enabled";
          parameter Modelica.SIunits.Radius rOuter "Outer radius of pipe" annotation(
            Dialog(enable = animation));
          parameter Modelica.SIunits.Length length "Length of pipe" annotation(
            Dialog(enable = animation));
          parameter Real xsi[:](each min = 0, each max = 1) = Modelica.Math.Vectors.relNodePositions(12) "[:] Relative position along the pipe with x[1] = 0, x[end] = 1" annotation(
            Dialog(enable = animation));
          input Real T[size(xsi, 1)] "[:] Scalar values at position xsi*length (will be visualized by color)" annotation(
            Dialog(enable = animation));
          parameter Real T_min "Minimum value of T that corresponds to colorMap[1,:]" annotation(
            Dialog(enable = animation));
          parameter Real T_max "Maximum value of T that corresponds to colorMap[end,:]" annotation(
            Dialog(enable = animation));
          replaceable function colorMap = Modelica.Mechanics.MultiBody.Visualizers.Colors.ColorMaps.jet constrainedby Modelica.Mechanics.MultiBody.Interfaces.partialColorMap "Function defining the color map" annotation(
             choicesAllMatching = true,
             Dialog(enable = animation, group = "Color coding"));
          parameter Integer n_colors = 64 "Number of colors in the colorMap" annotation(
            Dialog(enable = animation, group = "Color coding"));
          parameter Types.SpecularCoefficient specularCoefficient = 0.7 "Reflection of ambient light (= 0: light is completely absorbed)" annotation(
            Dialog(enable = animation, group = "Color coding"));
          parameter Real transparency = 0 "Transparency of shape: 0 (= opaque) ... 1 (= fully transparent)" annotation(
            Dialog(enable = animation, group = "Color coding"));
          parameter Integer n_rOuter = 30 "Number of points along outer radius" annotation(
            Dialog(enable = animation, tab = "Discretization"));
          parameter Integer n_length = 20 "Number of points along length" annotation(
            Dialog(enable = animation, tab = "Discretization"));
        protected
          Advanced.PipeWithScalarField pipe(redeclare function colorMap = colorMap, rOuter = rOuter, length = length, xsi = xsi, T = T, T_min = T_min, T_max = T_max, n_colors = n_colors, n_rOuter = n_rOuter, n_length = n_length, specularCoefficient = specularCoefficient, transparency = transparency, R = frame_a.R, r_0 = frame_a.r_0) if world.enableAnimation and animation annotation(
            Placement(transformation(extent = {{-20, -10}, {0, 10}})));
        equation
// No forces and torques
          frame_a.f = zeros(3);
          frame_a.t = zeros(3);
          annotation(
            Icon(graphics = {Text(extent = {{-150, 50}, {150, 90}}, lineColor = {0, 0, 255}, textString = "%name"), Bitmap(extent = {{-100, -62}, {98, 58}}, fileName = "modelica://Modelica/Resources/Images/Mechanics/MultiBody/Visualizers/PipeWithScalarFieldIcon.png")}),
            Documentation(info = "<html>
<p>
Model <b>PipeWithScalarField</b> visualizes a pipe and a scalar
field along the pipe axis. The latter is shown by mapping the scalar
field to color values with a color map and utilizing this color
at the perimeter associated with the corresponding axis location.
Typically the scalar field value is a temperature, but might
be also another quantity.
Predefined color maps are available from
<a href=\"modelica://Modelica.Mechanics.MultiBody.Visualizers.Colors.ColorMaps\">MultiBody.Visualizers.Colors.ColorMaps</a>
and can be selected via parameter \"colorMap\".
A color map with the corresponding scalar field values can be exported
as vector-graphics in svg-format with function
<a href=\"modelica://Modelica.Mechanics.MultiBody.Visualizers.Colors.colorMapToSvg\">MultiBody.Visualizers.Colors.colorMapToSvg</a>.
Connector frame_a of this component is located in the center of the
circle at the left side of the pipe and the pipe axis is oriented
along the x-axis of frame_a, see figure below in which frame_a is visualized
with a coordinate system:
</p>

<blockquote>
<img src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Visualizers/PipeWithScalarField.png\">
</blockquote>

<p>
The color coding is shown in the next figure. It was generated with
<a href=\"modelica://Modelica.Mechanics.MultiBody.Visualizers.Colors.colorMapToSvg\">MultiBody.Visualizers.Colors.colorMapToSvg</a>
using the following call:
</p>

<blockquote>
<pre>
colorMapToSvg(Modelica.Mechanics.MultiBody.Visualizers.Colors.ColorMaps.jet(),
              height=50, nScalars=6, T_max=100, caption=\"Temperature in C\");
</pre>
</blockquote>

<blockquote>
<img src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Visualizers/PipeWithScalarField-ColorMap.png\">
</blockquote>
</html>", revisions = "<html>
  <ul>
  <li> July 2010 by Martin Otter<br>
       Adapted to the new Surface model.</li>
  <li> July 2005 by Dirk Zimmer (practical training at DLR)<br>
       First version to visualize a multi-level tyre wheel model.</li>
  </ul>
</html>"));
        end PipeWithScalarField;

        package Colors "Library of functions operating on color"
          extends Modelica.Icons.Package;

          package ColorMaps "Library of functions returning color maps"
            extends Modelica.Icons.Package;

            function jet "Returns the \"jet\" color map"
              extends Modelica.Mechanics.MultiBody.Interfaces.partialColorMap;
            protected
              Real a = ceil(n_colors / 4);
              Real d = 1 / a;
              Real b = ceil(a / 2);
              Real c = floor(a / 2);
              Real v1[:] = {1 - (b - i) * d for i in 1:b};
              Real v2[:] = 0 + d:d:1;
              Real v3[:] = 1 - d:(-d):0;
              Real v4[:] = {0.5 + (c - i) * d for i in 1:c};
              Real cm[integer(ceil(n_colors / 4)) * 4, 3];
            algorithm
              cm := 255 * [zeros(size(v1, 1)), zeros(size(v1, 1)), v1; zeros(size(v2, 1)), v2, fill(1., size(v2, 1)); v2, fill(1, size(v2, 1)), v3; fill(1, size(v3, 1)), v3, fill(0., size(v3, 1)); v4, fill(0, size(v4, 1)), fill(0., size(v4, 1))];
              colorMap := cm[1:n_colors, :];
              annotation(
                Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
ColorMaps.<b>jet</b>();
ColorMaps.<b>jet</b>(n_colors=64);
</pre></blockquote>
<h4>Description</h4>
<p>
This function returns the color map \"jet.\" A color map
is a Real[:,3] array where every row represents a color.
With the optional argument \"n_colors\" the number of rows
of the returned array can be defined. The default value is
\"n_colors=64\" (it is usually best if n_colors is a multiple of 4).
Image of the \"jet\" color map:
</p>

<blockquote>
<img src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Visualizers/Colors/jet.png\">
</blockquote>

<h4>See also</h4>
<a href=\"modelica://Modelica.Mechanics.MultiBody.Visualizers.Colors.ColorMaps\">ColorMaps</a>,
<a href=\"modelica://Modelica.Mechanics.MultiBody.Visualizers.Colors.colorMapToSvg\">colorMapToSvg</a>,
<a href=\"modelica://Modelica.Mechanics.MultiBody.Visualizers.Colors.scalarToColor\">scalarToColor</a>.
</html>"));
            end jet;

            function hot "Returns the \"hot\" color map"
              extends Modelica.Mechanics.MultiBody.Interfaces.partialColorMap;
            protected
              Real a = ceil(n_colors / 3);
              Real d = 1 / a;
              Real v1[:] = 0 + d:d:1;
              Real cm[integer(ceil(n_colors / 3)) * 3, 3];
            algorithm
              cm := 255 * [v1, zeros(size(v1, 1)), zeros(size(v1, 1)); fill(1., size(v1, 1)), v1, zeros(size(v1, 1)); fill(1., size(v1, 1)), fill(1., size(v1, 1)), v1];
              colorMap := cm[1:n_colors, :];
              annotation(
                Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
ColorMaps.<b>hot</b>();
ColorMaps.<b>hot</b>(n_colors=64);
</pre></blockquote>
<h4>Description</h4>
<p>
This function returns the color map \"hot.\" A color map
is a Real[:,3] array where every row represents a color.
With the optional argument \"n_colors\" the number of rows
of the returned array can be defined. The default value is
\"n_colors=64\" (it is usually best if n_colors is a multiple of 4).
Image of the \"hot\" color map:
</p>

<blockquote>
<img src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Visualizers/Colors/hot.png\">
</blockquote>

<h4>See also</h4>
<a href=\"modelica://Modelica.Mechanics.MultiBody.Visualizers.Colors.ColorMaps\">ColorMaps</a>,
<a href=\"modelica://Modelica.Mechanics.MultiBody.Visualizers.Colors.colorMapToSvg\">colorMapToSvg</a>,
<a href=\"modelica://Modelica.Mechanics.MultiBody.Visualizers.Colors.scalarToColor\">scalarToColor</a>.
</html>"));
            end hot;

            function gray "Returns the \"gray\" color map"
              extends Modelica.Mechanics.MultiBody.Interfaces.partialColorMap;
            algorithm
              if n_colors > 1 then
                colorMap := 255 * [linspace(0, 1., n_colors), linspace(0, 1., n_colors), linspace(0, 1., n_colors)];
              else
                colorMap := [0, 0, 0];
              end if;
              annotation(
                Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
ColorMaps.<b>gray</b>();
ColorMaps.<b>gray</b>(n_colors=64);
</pre></blockquote>
<h4>Description</h4>
<p>
This function returns the color map \"gray.\" A color map
is a Real[:,3] array where every row represents a color.
With the optional argument \"n_colors\" the number of rows
of the returned array can be defined. The default value is
\"n_colors=64\" (it is usually best if n_colors is a multiple of 4).
Image of the \"gray\" color map:
</p>

<blockquote>
<img src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Visualizers/Colors/gray.png\">
</blockquote>

<h4>See also</h4>
<a href=\"modelica://Modelica.Mechanics.MultiBody.Visualizers.Colors.ColorMaps\">ColorMaps</a>,
<a href=\"modelica://Modelica.Mechanics.MultiBody.Visualizers.Colors.colorMapToSvg\">colorMapToSvg</a>,
<a href=\"modelica://Modelica.Mechanics.MultiBody.Visualizers.Colors.scalarToColor\">scalarToColor</a>.
</html>"));
            end gray;

            function spring "Returns the \"spring\" color map"
              extends Modelica.Mechanics.MultiBody.Interfaces.partialColorMap;
            algorithm
              if n_colors > 1 then
                colorMap := 255 * [fill(1, n_colors), linspace(0, 1., n_colors), linspace(1, 0, n_colors)];
              else
                colorMap := 255 * [1, 0, 1];
              end if;
              annotation(
                Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
ColorMaps.<b>spring</b>();
ColorMaps.<b>spring</b>(n_colors=64);
</pre></blockquote>
<h4>Description</h4>
<p>
This function returns the color map \"spring.\" A color map
is a Real[:,3] array where every row represents a color.
With the optional argument \"n_colors\" the number of rows
of the returned array can be defined. The default value is
\"n_colors=64\" (it is usually best if n_colors is a multiple of 4).
Image of the \"spring\" color map:
</p>

<blockquote>
<img src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Visualizers/Colors/spring.png\">
</blockquote>

<h4>See also</h4>
<a href=\"modelica://Modelica.Mechanics.MultiBody.Visualizers.Colors.ColorMaps\">ColorMaps</a>,
<a href=\"modelica://Modelica.Mechanics.MultiBody.Visualizers.Colors.colorMapToSvg\">colorMapToSvg</a>,
<a href=\"modelica://Modelica.Mechanics.MultiBody.Visualizers.Colors.scalarToColor\">scalarToColor</a>.
</html>"));
            end spring;

            function summer "Returns the \"summer\" color map"
              extends Modelica.Mechanics.MultiBody.Interfaces.partialColorMap;
            algorithm
              if n_colors > 1 then
                colorMap := 255 * [linspace(0, 1., n_colors), linspace(0.5, 1., n_colors), fill(0.4, n_colors)];
              else
                colorMap := 255 * [0, 0.5, 0.4];
              end if;
              annotation(
                Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
ColorMaps.<b>summer</b>();
ColorMaps.<b>summer</b>(n_colors=64);
</pre></blockquote>
<h4>Description</h4>
<p>
This function returns the color map \"summer.\" A color map
is a Real[:,3] array where every row represents a color.
With the optional argument \"n_colors\" the number of rows
of the returned array can be defined. The default value is
\"n_colors=64\" (it is usually best if n_colors is a multiple of 4).
Image of the \"summer\" color map:
</p>

<blockquote>
<img src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Visualizers/Colors/summer.png\">
</blockquote>

<h4>See also</h4>
<a href=\"modelica://Modelica.Mechanics.MultiBody.Visualizers.Colors.ColorMaps\">ColorMaps</a>,
<a href=\"modelica://Modelica.Mechanics.MultiBody.Visualizers.Colors.colorMapToSvg\">colorMapToSvg</a>,
<a href=\"modelica://Modelica.Mechanics.MultiBody.Visualizers.Colors.scalarToColor\">scalarToColor</a>.
</html>"));
            end summer;

            function autumn "Returns the \"autumn\" color map"
              extends Modelica.Mechanics.MultiBody.Interfaces.partialColorMap;
            algorithm
              if n_colors > 1 then
                colorMap := 255 * [fill(1, n_colors), linspace(0, 1., n_colors), zeros(n_colors)];
              else
                colorMap := 255 * [1, 0, 0];
              end if;
              annotation(
                Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
ColorMaps.<b>autumn</b>();
ColorMaps.<b>autumn</b>(n_colors=64);
</pre></blockquote>
<h4>Description</h4>
<p>
This function returns the color map \"autumn.\" A color map
is a Real[:,3] array where every row represents a color.
With the optional argument \"n_colors\" the number of rows
of the returned array can be defined. The default value is
\"n_colors=64\" (it is usually best if n_colors is a multiple of 4).
Image of the \"autumn\" color map:
</p>

<blockquote>
<img src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Visualizers/Colors/autumn.png\">
</blockquote>

<h4>See also</h4>
<a href=\"modelica://Modelica.Mechanics.MultiBody.Visualizers.Colors.ColorMaps\">ColorMaps</a>,
<a href=\"modelica://Modelica.Mechanics.MultiBody.Visualizers.Colors.colorMapToSvg\">colorMapToSvg</a>,
<a href=\"modelica://Modelica.Mechanics.MultiBody.Visualizers.Colors.scalarToColor\">scalarToColor</a>.
</html>"));
            end autumn;

            function winter "Returns the \"winter\" color map"
              extends Modelica.Mechanics.MultiBody.Interfaces.partialColorMap;
            algorithm
              if n_colors > 1 then
                colorMap := 255 * [zeros(n_colors), linspace(0, 1, n_colors), linspace(1, 0.5, n_colors)];
              else
                colorMap := [0, 0, 255];
              end if;
              annotation(
                Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
ColorMaps.<b>winter</b>();
ColorMaps.<b>winter</b>(n_colors=64);
</pre></blockquote>
<h4>Description</h4>
<p>
This function returns the color map \"winter.\" A color map
is a Real[:,3] array where every row represents a color.
With the optional argument \"n_colors\" the number of rows
of the returned array can be defined. The default value is
\"n_colors=64\" (it is usually best if n_colors is a multiple of 4).
Image of the \"winter\" color map:
</p>

<blockquote>
<img src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Visualizers/Colors/winter.png\">
</blockquote>

<h4>See also</h4>
<a href=\"modelica://Modelica.Mechanics.MultiBody.Visualizers.Colors.ColorMaps\">ColorMaps</a>,
<a href=\"modelica://Modelica.Mechanics.MultiBody.Visualizers.Colors.colorMapToSvg\">colorMapToSvg</a>,
<a href=\"modelica://Modelica.Mechanics.MultiBody.Visualizers.Colors.scalarToColor\">scalarToColor</a>.
</html>"));
            end winter;
            annotation(
              Documentation(info = "<html>
<p>
This package contains functions that return color maps.
A color map is a Real[:,3] array where every row represents a color.
Currently the following color maps are returned from the
respective function:
</p>

<blockquote>
<img src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Visualizers/Colors/ColorMaps.png\">
</blockquote>
</html>"));
          end ColorMaps;

          function colorMapToSvg "Save a color map on file in svg (scalable vector graphics) format"
            extends Modelica.Icons.Function;
            encapsulated type HeaderType = enumeration(svgBeginAndEnd, svgBegin, svgEnd, noHeader);
            import Modelica.Utilities.Streams.print;
            input Real colorMap[:, 3] "Color map to be stored in svg format" annotation(
              choices(choice = Modelica.Mechanics.MultiBody.Visualizers.Colors.ColorMaps.jet(), choice = Modelica.Mechanics.MultiBody.Visualizers.Colors.ColorMaps.hot(), choice = Modelica.Mechanics.MultiBody.Visualizers.Colors.ColorMaps.gray(), choice = Modelica.Mechanics.MultiBody.Visualizers.Colors.ColorMaps.spring(), choice = Modelica.Mechanics.MultiBody.Visualizers.Colors.ColorMaps.summer(), choice = Modelica.Mechanics.MultiBody.Visualizers.Colors.ColorMaps.autumn(), choice = Modelica.Mechanics.MultiBody.Visualizers.Colors.ColorMaps.winter()));
            input String fileName = "colorMap.svg" "File where the svg representation shall be stored";
            input Real width(unit = "mm") = 10 "Width in svg-figure";
            input Real height(unit = "mm") = 100 "Height in svg-figure";
            input Real x(unit = "mm") = 20 "X-Coordinate of left upper corner";
            input Real y(unit = "mm") = 10 "Y-Coordinate of left upper corner";
            input Real T_min = 0 "Value of scalar corresponding to colorMap[1,:]";
            input Real T_max = 100 "Value of scalar corresponding to colorMap[end,:]";
            input Integer nScalars = 11 "Number of scalars to be displayed at the right side";
            input String format = ".3g" "Format of the numbers";
            input Real fontSize = 11 "Font size in [pt]";
            input Real textWidth(unit = "mm") = 8 "Numbers are right justified starting at x+width+textWidth";
            input String caption = "" "Caption above the map";
            input HeaderType headerType = Colors.colorMapToSvg.HeaderType.svgBeginAndEnd "Type of header";
          protected
            Integer nc = size(colorMap, 1);
            Real dy = height / nc;
            Real yy = y - dy;
            String strWidth = String(width);
            String strHeight = String(dy);
            Real T;
            Integer ni;
            constant Real ptToMm = 127 / 360 "1 Point = ptToMm mm";
            Real fontHeight(unit = "mm") = fontSize * ptToMm;
            Real xx = x + width + textWidth;
            String strXX = String(xx);
            Real xHeading = x + width / 2;
            Real yHeading = y - 1.2 * fontHeight;
          algorithm
            if headerType == HeaderType.svgBeginAndEnd or headerType == HeaderType.svgBegin then
              Modelica.Utilities.Files.remove(fileName);
              print("... generating svg-file: " + Modelica.Utilities.Files.fullPathName(fileName));
            end if;
            if caption <> "" then
              print("... " + caption);
            end if;
            if headerType == HeaderType.svgBeginAndEnd or headerType == HeaderType.svgBegin then
              print("<svg xmlns=\"http://www.w3.org/2000/svg\">", fileName);
            end if;
            print("  <g>", fileName);
// Print colors
            for i in nc:(-1):1 loop
              yy := yy + dy;
              print("    <rect x=\"" + String(x) + "mm\" y=\"" + String(yy) + "mm\" width=\"" + strWidth + "mm\" height=\"" + strHeight + "mm\" style=\"fill:rgb(" + String(integer(colorMap[i, 1])) + "," + String(integer(colorMap[i, 2])) + "," + String(integer(colorMap[i, 3])) + ");stroke:none\"/>", fileName);
            end for;
// print:  <rect x="XXmm" y="XXmm" width="YYmm" height="ZZmm" style="fill:rgb(100,128,255);stroke:none"/>
// Print numbers
            ni := if nScalars == 1 then 2 else if nScalars < 1 then 0 else nScalars;
            dy := height / (ni - 1);
            yy := y - dy + 0.3 * fontHeight;
            for i in ni:(-1):1 loop
              yy := yy + dy;
              T := T_min + (T_max - T_min) * (i - 1) / (ni - 1);
              print("    <text x=\"" + strXX + "mm\" y=\"" + String(yy) + "mm\" font-family=\"Fixedsys\" font-size=\"" + String(fontSize) + "pt\" text-anchor=\"end\">" + String(T, format = format) + "</text>", fileName);
            end for;
// print: <text x="22mm" y="12mm" font-family="Arial,sans-serif" font-size="11pt">1.2345</text>
            if caption <> "" then
              print("   <text x=\"" + String(xHeading) + "mm\" y=\"" + String(yHeading) + "mm\" font-family=\"Fixedsys\" font-size=\"" + String(fontSize) + "pt\" text-anchor=\"middle\">" + caption + "</text>", fileName);
            end if;
            print("  </g>", fileName);
            if headerType == HeaderType.svgBeginAndEnd or headerType == HeaderType.svgEnd then
              print("</svg>", fileName);
            end if;
            annotation(
              Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
Colors.<b>colorMapToSvg</b>(colorMap);
Colors.<b>colorMapToSvg</b>(colorMap,
                     fileName   = \"colorMap.svg\",
                     width      =  10,  // [mm]
                     height     = 100,  // [mm]
                     x          =  20,  // [mm]
                     y          =  10,  // [mm]
                     T_min      =   0,
                     T_max      = 100,
                     nScalars   =  11,
                     format     = \".3g\",
                     fontSize   =  11,  // [pt]
                     textWidth  =   8,  // [mm]
                     caption    = \"\",
                     headerType = Colors.colorMapToSvg.Header.svgBeginAndEnd)
                                                           // svgBegin
                                                           // svgEnd
                                                           // svgNoHeader
</pre></blockquote>
<h4>Description</h4>
<p>
This function saves the color map \"Real colorMap[:,3]\" on file \"fileName\"
in svg format. The color map has a width of \"width\" and a height of \"height\" and
the upper left corner is placed at coordinates \"(x,y)\".
Over the color map, a caption \"caption\" is placed.
On the right side of the color map, a set of scalar field values T is
displayed where \"T_min\" is placed at colorMap[1,:],
\"T_max\" is placed at colorMap[end,:] and \"nScalars\" values between
\"T_min\" and \"T_max\" (including T_min and T_max) are shown.
The printing format of the numbers is defined with \"format\", see definition below.
With argument \"headerType\" it is defined whether \"svg\" begin and end
lines are printed. If the \"begin\" svg tag shall be printed, file \"fileName\"
is deleted, if it already exists. Otherwise, all output is appended to the
file \"fileName\".
</p>

<p>
A \"svg\" file can be displayed by a web browser, such as
<a href=\"http://www.mozilla.org/firefox\">Firefox</a> by dragging the
file in the browser window.
Alternatively, a svg-file can be loaded in a graphics program,
such as the free <a href=\"http://inkscape.org\">Inkscape</a>,
can be manipulated and can be exported in other graphic formats.
The following image was generated with a call to \"colorMapToSvg\", the
generated file was loaded in Inkscape and exported in \"png\" format:
</p>

<blockquote>
<img src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Visualizers/Colors/jet.png\">
</blockquote>

<p>
The \"format\" argument defines the string formatting according to
ANSI-C without \"%\" and \"*\" character<br>
(e.g., \".6g\", \"14.5e\", \"+6f\"). In particular:
</p>

<p>
format = \"[&lt;flags&gt;] [&lt;width&gt;] [.&lt;precision&gt;] &lt;conversion&gt;\"
with
</p>

<blockquote>
<table>
<tr>
  <td>&lt;flags&gt;</td>
  <td> zero, one or more of<br>
       \"-\": left adjustment of the converted number<br>
       \"+\": number will always be printed with a sign<br>
       \"0\": padding to the field width with leading zeros</td></tr>
<tr>
  <td>&lt;width&gt;</td>
  <td> Minimum field width. The converted number will be printed in a field at<br>
       least this wide and wider if necessary. If the converted number has <br>
       fewer characters it will be padded on the left (or the right depending<br>
       on &lt;flags&gt;) with blanks or 0 (depending on &lt;flags&gt;).</td></tr>
<tr>
  <td>&lt;precision&gt;</td>
  <td> The number of digits to be printed after the decimal point for <br>
       e, E, or f conversions, or the number of significant digits for <br>
       g or G conversions.</td></tr>
<tr>
  <td> &lt;conversion&gt;</td>
  <td> = \"e\": Exponential notation using a  lower case e<br>
       = \"E\": Exponential notation using an upper case E<br>
       = \"f\": Fixed point notation<br>
       = \"g\": Either \"e\" or \"f\"<br>
       = \"G\": Same as \"g\", but with upper case E</td></tr>
</table>
</blockquote>
</html>"));
          end colorMapToSvg;

          function scalarToColor "Map a scalar to a color using a color map"
            extends Modelica.Icons.Function;
            input Real T "Scalar value" annotation(
              Dialog);
            input Real T_min "T <= T_min is mapped to colorMap[1,:]" annotation(
              Dialog);
            input Real T_max "T >= T_max is mapped to colorMap[end,:]" annotation(
              Dialog);
            input Real colorMap[:, 3] "Color map" annotation(
              Dialog);
            output Real color[3] "Color of scalar value T";
          algorithm
/* old version, that could give an error
  color :=colorMap[integer((size(colorMap, 1) - 1)/(T_max - T_min)*
                            min((max(T,T_min) - T_min), T_max) + 1), :];
  */
            color := colorMap[1 + integer((size(colorMap, 1) - 1) * (max(T_min, min(T, T_max)) - T_min) / (T_max - T_min)), :];
            annotation(
              Inline = true,
              Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
//Real T, T_min, T_max, colorMap[:,3];
Colors.<b>scalarToColor</b>(T, T_min, T_max, colorMap);
</pre></blockquote>
<h4>Description</h4>
<p>
This function returns an rgb color Real[3] that corresponds to the value of \"T\".
The color is selected from the colorMap by interpolation so that
\"T_min\" corresponds to \"colorMap[1,:]\" and
\"T_max\" corresponds to \"colorMap[end,:]\".
</p>

<h4>See also</h4>
<a href=\"modelica://Modelica.Mechanics.MultiBody.Visualizers.Colors.ColorMaps\">ColorMaps</a>,
<a href=\"modelica://Modelica.Mechanics.MultiBody.Visualizers.Colors.colorMapToSvg\">colorMapToSvg</a>,
<a href=\"modelica://Modelica.Mechanics.MultiBody.Visualizers.PipeWithScalarField\">PipeWithScalarField</a>.

</html>"));
          end scalarToColor;
          annotation(
            Documentation(info = "<html>
<p>
This package contains functions to operate on colors.
Note, a color is represented as a Real array with 3 elements where
the elements are the red, green, blue values of the RGB color model.
Every element must be in the range 0..255.
The type of a color is Real and not Integer in order that a color
can be used with less problems in a model, since in a model an Integer
type could only be used in a when-clause. Typical declaration of a color value:
</p>

<blockquote>
<pre>
  Real color[3](each min=0, each max=255);
</pre>
</blockquote>

<p>
This definition is also available as type
<a href=\"modelica://Modelica.Mechanics.MultiBody.Types.RealColor\">Modelica.Mechanics.MultiBody.Types.RealColor</a>.
</p>
</html>"));
        end Colors;

        package Advanced "Visualizers that require basic knowledge about Modelica in order to use them"
          extends Modelica.Icons.Package;

          model Arrow "Visualizing an arrow with variable size; all data have to be set as modifiers (see info layer)"
            import Modelica.Mechanics.MultiBody.Types;
            import Modelica.Mechanics.MultiBody.Frames;
            import T = Modelica.Mechanics.MultiBody.Frames.TransformationMatrices;
            import Modelica.SIunits.Conversions.to_unit1;
            input Frames.Orientation R = Frames.nullRotation() "Orientation object to rotate the world frame into the arrow frame" annotation(
              Dialog);
            input SI.Position r[3] = {0, 0, 0} "Position vector from origin of world frame to origin of arrow frame, resolved in world frame" annotation(
              Dialog);
            input SI.Position r_tail[3] = {0, 0, 0} "Position vector from origin of arrow frame to arrow tail, resolved in arrow frame" annotation(
              Dialog);
            input SI.Position r_head[3] = {0, 0, 0} "Position vector from arrow tail to the head of the arrow, resolved in arrow frame" annotation(
              Dialog);
            input SI.Diameter diameter = world.defaultArrowDiameter "Diameter of arrow line" annotation(
              Dialog);
            input Modelica.Mechanics.MultiBody.Types.Color color = Modelica.Mechanics.MultiBody.Types.Defaults.ArrowColor "Color of arrow" annotation(
              Dialog(colorSelector = true));
            input Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Material property describing the reflecting of ambient light (= 0 means, that light is completely absorbed)" annotation(
              Dialog);
          protected
            outer Modelica.Mechanics.MultiBody.World world;
            SI.Length length = Modelica.Math.Vectors.length(r_head) "Length of arrow";
            Real e_x[3](each final unit = "1", start = {1, 0, 0}) = noEvent(if length < 1.e-10 then {1, 0, 0} else r_head / length);
            Real rxvisobj[3](each final unit = "1") = transpose(R.T) * e_x "X-axis unit vector of shape, resolved in world frame" annotation(
              HideResult = true);
            SI.Position rvisobj[3] = r + T.resolve1(R.T, r_tail) "Position vector from world frame to shape frame, resolved in world frame" annotation(
              HideResult = true);
            SI.Length arrowLength = noEvent(max(0, length - diameter * Types.Defaults.ArrowHeadLengthFraction)) annotation(
              HideResult = true);
            Visualizers.Advanced.Shape arrowLine(length = arrowLength, width = diameter, height = diameter, lengthDirection = to_unit1(r_head), widthDirection = {0, 1, 0}, shapeType = "cylinder", color = color, specularCoefficient = specularCoefficient, r_shape = r_tail, r = r, R = R) if world.enableAnimation;
            Visualizers.Advanced.Shape arrowHead(length = noEvent(max(0, min(length, diameter * Types.Defaults.ArrowHeadLengthFraction))), width = noEvent(max(0, diameter * MultiBody.Types.Defaults.ArrowHeadWidthFraction)), height = noEvent(max(0, diameter * MultiBody.Types.Defaults.ArrowHeadWidthFraction)), lengthDirection = to_unit1(r_head), widthDirection = {0, 1, 0}, shapeType = "cone", color = color, specularCoefficient = specularCoefficient, r = rvisobj + rxvisobj * arrowLength, R = R) if world.enableAnimation;
            annotation(
              Documentation(info = "<HTML>
<p>
Model <b>Arrow</b> defines an arrow that is dynamically
visualized at the defined location (see variables below).
</p>

<p>
<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Visualizers/Arrow.png\" ALT=\"model Visualizers.Advanced.Arrow\">
</p>

<p>
The variables under heading <b>Parameters</b> below
are declared as (time varying) <b>input</b> variables.
If the default equation is not appropriate, a corresponding
modifier equation has to be provided in the
model where an <b>Arrow</b> instance is used, e.g., in the form
</p>
<pre>
    Visualizers.Advanced.Arrow arrow(diameter = sin(time));
</pre>

<p>
Variable <b>color</b> is an Integer vector with 3 elements,
{r, g, b}, and specifies the color of the shape.
{r,g,b} are the \"red\", \"green\" and \"blue\" color parts.
Note, r g, b are given in the range 0 .. 255.
The predefined type <b>MultiBody.Types.Color</b> contains
a menu definition of the colors used in the MultiBody
library (will be replaced by a color editor).
</p>
</HTML>"),
              Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, 28}, {20, -30}}, lineColor = {128, 128, 128}, fillColor = {128, 128, 128}, fillPattern = FillPattern.Solid), Polygon(points = {{20, 60}, {100, 0}, {20, -60}, {20, 60}}, lineColor = {128, 128, 128}, fillColor = {128, 128, 128}, fillPattern = FillPattern.Solid), Text(extent = {{-150, 105}, {150, 65}}, textString = "%name", lineColor = {0, 0, 255})}));
          end Arrow;

          model DoubleArrow "Visualizing a double arrow with variable size; all data have to be set as modifiers (see info layer)"
            import Modelica.Mechanics.MultiBody.Types;
            import Modelica.Mechanics.MultiBody.Frames;
            import T = Modelica.Mechanics.MultiBody.Frames.TransformationMatrices;
            import Modelica.SIunits.Conversions.to_unit1;
            input Frames.Orientation R = Frames.nullRotation() "Orientation object to rotate the world frame into the arrow frame" annotation(
              Dialog);
            input SI.Position r[3] = {0, 0, 0} "Position vector from origin of world frame to origin of arrow frame, resolved in world frame" annotation(
              Dialog);
            input SI.Position r_tail[3] = {0, 0, 0} "Position vector from origin of arrow frame to double arrow tail, resolved in arrow frame" annotation(
              Dialog);
            input SI.Position r_head[3] = {0, 0, 0} "Position vector from double arrow tail to the head of the double arrow, resolved in arrow frame" annotation(
              Dialog);
            input SI.Diameter diameter = world.defaultArrowDiameter "Diameter of arrow line" annotation(
              Dialog);
            input Modelica.Mechanics.MultiBody.Types.Color color = Modelica.Mechanics.MultiBody.Types.Defaults.ArrowColor "Color of double arrow" annotation(
              Dialog(colorSelector = true));
            input Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Material property describing the reflecting of ambient light (= 0 means, that light is completely absorbed)" annotation(
              Dialog);
          protected
            outer Modelica.Mechanics.MultiBody.World world;
            SI.Length length = Modelica.Math.Vectors.length(r_head) "Length of arrow";
            Real e_x[3](each final unit = "1", start = {1, 0, 0}) = noEvent(if length < 1.e-10 then {1, 0, 0} else r_head / length);
            Real rxvisobj[3](each final unit = "1") = transpose(R.T) * e_x "X-axis unit vector of shape, resolved in world frame" annotation(
              HideResult = true);
            SI.Position rvisobj[3] = r + T.resolve1(R.T, r_tail) "Position vector from world frame to shape frame, resolved in world frame" annotation(
              HideResult = true);
            SI.Length headLength = noEvent(max(0, min(length, diameter * MultiBody.Types.Defaults.ArrowHeadLengthFraction)));
            SI.Length headWidth = noEvent(max(0, diameter * MultiBody.Types.Defaults.ArrowHeadWidthFraction));
            SI.Length arrowLength = noEvent(max(0, length - 1.5 * diameter * MultiBody.Types.Defaults.ArrowHeadLengthFraction));
            Visualizers.Advanced.Shape arrowLine(length = arrowLength, width = diameter, height = diameter, lengthDirection = to_unit1(r_head), widthDirection = {0, 1, 0}, shapeType = "cylinder", color = color, specularCoefficient = specularCoefficient, r_shape = r_tail, r = r, R = R) if world.enableAnimation;
            Visualizers.Advanced.Shape arrowHead1(length = headLength, width = headWidth, height = headWidth, lengthDirection = to_unit1(r_head), widthDirection = {0, 1, 0}, shapeType = "cone", color = color, specularCoefficient = specularCoefficient, r = rvisobj + rxvisobj * arrowLength, R = R) if world.enableAnimation;
            Visualizers.Advanced.Shape arrowHead2(length = headLength, width = headWidth, height = headWidth, lengthDirection = to_unit1(r_head), widthDirection = {0, 1, 0}, shapeType = "cone", color = color, specularCoefficient = specularCoefficient, r = rvisobj + rxvisobj * (arrowLength + 0.5 * headLength), R = R) if world.enableAnimation;
            annotation(
              Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, 28}, {0, -28}}, lineColor = {128, 128, 128}, fillColor = {128, 128, 128}, fillPattern = FillPattern.Solid), Polygon(points = {{40, 60}, {100, 0}, {40, -60}, {40, 60}}, lineColor = {128, 128, 128}, fillColor = {128, 128, 128}, fillPattern = FillPattern.Solid), Text(extent = {{-150, 105}, {150, 65}}, textString = "%name", lineColor = {0, 0, 255}), Polygon(points = {{0, 60}, {60, 0}, {0, -60}, {0, 60}}, lineColor = {128, 128, 128}, fillColor = {128, 128, 128}, fillPattern = FillPattern.Solid)}),
              Documentation(info = "<HTML>
<p>
Model <b>DoubleArrow</b> defines a double arrow that is dynamically
visualized at the defined location (see variables below).
</p>

<p>
<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Visualizers/DoubleArrow.png\" ALT=\"model Visualizers.Advanced.DoubleArrow\">
</p>

<p>
The variables under heading <b>Parameters</b> below
are declared as (time varying) <b>input</b> variables.
If the default equation is not appropriate, a corresponding
modifier equation has to be provided in the
model where an <b>Arrow</b> instance is used, e.g., in the form
</p>
<pre>
    Visualizers.Advanced.DoubleArrow doubleArrow(diameter = sin(time));
</pre>
<p>
Variable <b>color</b> is an Integer vector with 3 elements,
{r, g, b}, and specifies the color of the shape.
{r,g,b} are the \"red\", \"green\" and \"blue\" color parts.
Note, r g, b are given in the range 0 .. 255.
The predefined type <b>MultiBody.Types.Color</b> contains
a menu definition of the colors used in the MultiBody
library (will be replaced by a color editor).
</p>
</HTML>"));
          end DoubleArrow;

          model Shape "Visualizing an elementary object with variable size; all data have to be set as modifiers (see info layer)"
            extends ModelicaServices.Animation.Shape;
            extends Modelica.Utilities.Internal.PartialModelicaServices.Animation.PartialShape;
            annotation(
              Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, -100}, {80, 60}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Polygon(points = {{-100, 60}, {-80, 100}, {100, 100}, {80, 60}, {-100, 60}}, lineColor = {0, 0, 255}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Polygon(points = {{100, 100}, {100, -60}, {80, -100}, {80, 60}, {100, 100}}, lineColor = {0, 0, 255}, fillColor = {160, 160, 164}, fillPattern = FillPattern.Solid), Text(extent = {{-100, -54}, {80, 8}}, lineColor = {0, 0, 0}, textString = "%shapeType"), Text(extent = {{-150, 150}, {150, 110}}, lineColor = {0, 0, 255}, textString = "%name")}),
              Documentation(info = "<HTML>
<p>
Model <b>Shape</b> defines a visual shape that is
shown at the location of its reference coordinate system, called
'object frame' below. All describing variables such
as size and color can vary dynamically (with the only exception
of parameter shapeType). The default equations in the
declarations should be modified by providing appropriate modifier
equations. Model <b>Shape</b> is usually used as a basic building block to
implement simpler to use graphical components.
</p>
<p>
The following shapes are supported via
parameter <b>shapeType</b> (e.g., shapeType=\"box\"):<br>&nbsp;
</p>

<p>
<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Shape.png\" ALT=\"model Visualizers.FixedShape\">
</p>

<p>&nbsp;<br>
The dark blue arrows in the figure above are directed along
variable <b>lengthDirection</b>. The light blue arrows are directed
along variable <b>widthDirection</b>. The <b>coordinate systems</b>
in the figure represent frame_a of the Shape component.
</p>

<p>
Additionally, <b>external shapes</b> can be specified as (not all options might be supported by all tools):
</p>

<ul>
<li> <b>\"1\", \"2\", ...</b><br>
     define external shapes specified in DXF format in files \"1.dxf\", \"2.dxf\", ...
     The DXF-files must be found either in the current directory or in the directory where
     the Shape instance is stored that references the DXF file.
     This (very limited) option should not be used for new models. Example:<br>
    shapeType=\"1\".<br></li>

<li> \"<b>modelica:</b>//&lt;Modelica-name&gt;/&lt;relative-path-file-name&gt;\"<br>
     characterizes the file that is stored under the location of the
     &lt;Modelica-name&gt; library path with the given relative file name.
     Example:<br> shapeType = \"modelica://Modelica/Resources/Data/Shapes/Engine/piston.dxf\".<br></li>

<li> \"<b>file:</b>//&lt;absolute-file-name&gt;\"<br>
     characterizes an absolute file name in the file system. Example:<br>
     shapeType=\"file://C:/users/myname/shapes/piston.dxf\".</li>
</ul>

<p>
The supported file formats are tool dependent. Most tools support
at least DXF-files (a tool might support 3-dim. Face of the DXF format only),
but may support other format as well (such as stl, obj, 3ds).
Since visualization files contain color and other data, the corresponding
information in the model is usually ignored.
</p>

<p>
The sizes of any of the above components are specified by the
<b>length</b>, <b>width</b> and <b>height</b> variables.
Via variable <b>extra</b> additional data can be defined:
</p>
<table border=1 cellspacing=0 cellpadding=2>
<tr><th><b>shapeType</b></th><th>Meaning of parameter <b>extra</b></th></tr>
<tr>
  <td valign=\"top\">\"cylinder\"</td>
  <td valign=\"top\">if extra &gt; 0, a black line is included in the
      cylinder to show the rotation of it.</td>
</tr>
<tr>
  <td valign=\"top\">\"cone\"</td>
  <td valign=\"top\">extra = diameter-left-side / diameter-right-side, i.e.,<br>
      extra = 1: cylinder<br>
      extra = 0: \"real\" cone.</td>
</tr>
<tr>
  <td valign=\"top\">\"pipe\"</td>
  <td valign=\"top\">extra = outer-diameter / inner-diameter, i.e, <br>
      extra = 1: cylinder that is completely hollow<br>
      extra = 0: cylinder without a hole.</td>
</tr>
<tr>
  <td valign=\"top\">\"gearwheel\"</td>
  <td valign=\"top\">extra is the number of teeth of the (external) gear.
If extra &lt; 0, an internal gear is visualized with |extra| teeth.
The axis of the gearwheel is along \"lengthDirection\", and usually:
width = height = 2*radiusOfGearWheel.</td>
</tr>
<tr>
  <td valign=\"top\">\"spring\"</td>
  <td valign=\"top\">extra is the number of windings of the spring.
      Additionally, \"height\" is <b>not</b> the \"height\" but
      2*coil-width.</td>
</tr>
<tr>
  <td valign=\"top\">external shape</td>
  <td valign=\"top\">extra = 0: Visualization from file is not scaled.<br>
                   extra = 1: Visualization from file is scaled with \"length\", \"width\" and height\"
                              of the shape</td>
</tr>
</table>
<p>
Parameter <b>color</b> is a vector with 3 elements,
{r, g, b}, and specifies the color of the shape.
{r,g,b} are the \"red\", \"green\" and \"blue\" color parts.
Note, r g, b are given as Integer[3] in the ranges 0 .. 255,
respectively. The predefined type
<a href=\"modelica://Modelica.Mechanics.MultiBody.Types.Color\">MultiBody.Types.Color</a> contains a menu
definition of the colors used in the MultiBody library together with a color editor.
</p>

<p>
The variables under heading <b>Parameters</b> below
are declared as (time varying) <b>input</b> variables.
If the default equation is not appropriate, a corresponding
modifier equation has to be provided in the
model where a <b>Shape</b> instance is used, e.g., in the form
</p>
<pre>
    Visualizers.Advanced.Shape shape(length = sin(time));
</pre>
</html>"));
          end Shape;

          model Surface "Visualizing a moveable, parameterized surface; the surface characteristic is provided by a function"
            extends Modelica.Mechanics.MultiBody.Icons.Surface;
            extends Modelica.Utilities.Internal.PartialModelicaServices.Animation.PartialSurface;
            extends ModelicaServices.Animation.Surface;
            annotation(
              Icon(graphics = {Polygon(points = {{-102, 40}, {-98, 92}, {28, -8}, {96, 146}, {104, -118}, {-18, -34}, {-52, -130}, {-102, 40}}, lineColor = {0, 0, 255}, smooth = Smooth.Bezier, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-150, 140}, {150, 100}}, lineColor = {0, 0, 255}, textString = "%name")}),
              Documentation(info = "<html>
<p>
Model <b>Surface</b> defines a moveable, parametrized surface in 3-dim. space
that is used for animation. This object is specified by:
</p>

<ul>
<li> The surface frame (orientation object \"R\" and origin \"r_0\")
     in which the data is specified.</li>
<li> A set of two parameters, one in u- and one in v-direction,
     that defines the control points. </li>
<li> A time-varying position of each control point with respect to
     the surface frame.</li>
</ul>

<p>
The parameter values (u,v) are given by the ordinal numbers of the
corresponding control point in u- or in v-direction, respectively.
The surface is then defined by the replaceable function \"surfaceCharacteristic\" with the
interface <a href=\"modelica://Modelica.Mechanics.MultiBody.Interfaces.partialSurfaceCharacteristic\">partialSurfaceCharacteristic</a>
that returns the x-, y-, z- coordinate of every control point in form of 3 arrays X, Y, Z, and an optional color array C, if every control point shall have a different color:
</p>

<pre>
  Real X[nu,nv], Y[nu,nv], Z[nu,nv], C[nu,nv,3];
</pre>

<p>
An example of a parameterized surface with color coding is shown in the next figure:
</p>

<blockquote>
<img src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Visualizers/Surface.png\">
</blockquote>

<p>
Models <a href=\"modelica://Modelica.Mechanics.MultiBody.Visualizers.Torus\">Torus</a>,
<a href=\"modelica://Modelica.Mechanics.MultiBody.Visualizers.VoluminousWheel\">VoluminousWheel</a>,
<a href=\"modelica://Modelica.Mechanics.MultiBody.Visualizers.PipeWithScalarField\">PipeWithScalarField</a>,
demonstrate how new visualizer objects can be constructed with the Surface model.<br>
The direct usage of the Surface model, as well as of the Torus and the VoluminousWheel models, are demonstrated with example
<a href=\"modelica://Modelica.Mechanics.MultiBody.Examples.Elementary.Surfaces\">Examples.Elementary.Surfaces</a>.
</p>
</html>"));
          end Surface;

          model PipeWithScalarField "Visualizing a pipe with a scalar field"
            input Frames.Orientation R = Frames.nullRotation() "Orientation object to rotate the world frame into the surface frame" annotation(
              Dialog(group = "Surface frame"));
            input Modelica.SIunits.Position r_0[3] = {0, 0, 0} "Position vector from origin of world frame to origin of surface frame, resolved in world frame" annotation(
              Dialog(group = "Surface frame"));
            parameter Modelica.SIunits.Radius rOuter "Outer radius of pipe" annotation(
              Dialog(group = "Surface properties"));
            parameter Modelica.SIunits.Length length "Length of pipe" annotation(
              Dialog(group = "Surface properties"));
            parameter Integer n_rOuter = 30 "Number of points along outer radius" annotation(
              Dialog(group = "Surface properties"));
            parameter Integer n_length = 10 "Number of points along length" annotation(
              Dialog(group = "Surface properties"));
            parameter Types.SpecularCoefficient specularCoefficient = 0.7 "Reflection of ambient light (= 0: light is completely absorbed)" annotation(
              Dialog(group = "Surface properties"));
            parameter Real transparency = 0 "Transparency of shape: 0 (= opaque) ... 1 (= fully transparent)" annotation(
              Dialog(group = "Surface properties"));
            parameter Real xsi[:](each min = 0, each max = 1) = Modelica.Math.Vectors.relNodePositions(12) "[:] Relative position along the pipe with x[1] = 0, x[end] = 1" annotation(
              Dialog(group = "Color coding"));
            input Real T[size(xsi, 1)] "[:] Scalar values at position xsi*length (will be visualized by color)" annotation(
              Dialog(group = "Color coding"));
            parameter Real T_min "Minimum value of T that corresponds to colorMap[1,:]" annotation(
              Dialog(group = "Color coding"));
            parameter Real T_max "Maximum value of T that corresponds to colorMap[end,:]" annotation(
              Dialog(group = "Color coding"));
            parameter Integer n_colors = 64 "Number of colors in the colorMap" annotation(
              Dialog(group = "Color coding"));
            replaceable function colorMap = Modelica.Mechanics.MultiBody.Visualizers.Colors.ColorMaps.jet constrainedby Modelica.Mechanics.MultiBody.Interfaces.partialColorMap "Function defining the color map" annotation(
               choicesAllMatching = true,
               Dialog(group = "Color coding"));
          protected
            parameter Real colorMapData[n_colors, 3] = colorMap(n_colors) annotation(
              HideResult = true);
            Surface surface(R = R, r_0 = r_0, nu = n_length, nv = n_rOuter, wireframe = false, multiColoredSurface = true, specularCoefficient = specularCoefficient, transparency = transparency, redeclare function surfaceCharacteristic = Modelica.Mechanics.MultiBody.Visualizers.Advanced.SurfaceCharacteristics.pipeWithScalarField(rOuter = rOuter, length = length, xsi = xsi, T = T, T_min = T_min, T_max = T_max, colorMap = colorMapData)) annotation(
              Placement(transformation(extent = {{-20, 2}, {0, 22}})));
            annotation(
              Icon(graphics = {Bitmap(extent = {{-99, -60}, {99, 60}}, fileName = "modelica://Modelica/Resources/Images/Mechanics/MultiBody/Visualizers/PipeWithScalarFieldIcon.png"), Text(extent = {{-150, 54}, {150, 94}}, lineColor = {0, 0, 255}, textString = "%name")}),
              Documentation(info = "<html>
<p>
Model <b>PipeWithScalarField</b> visualizes a pipe and a scalar
field along the pipe axis. The latter is shown by mapping scalar
field to color values with a color map and utilizing this color
at the perimeter associated with the corresponding axis location.
Typically the scalar field value is a temperature, but might
be also another quantity.
Predefined color maps are available from
<a href=\"modelica://Modelica.Mechanics.MultiBody.Visualizers.Colors.ColorMaps\">MultiBody.Visualizers.Colors.ColorMaps</a>
and can be selected via parameter \"colorMap\".
A color map with the corresponding scalar field values can be exported
as vector-graphics in svg-format with function
<a href=\"modelica://Modelica.Mechanics.MultiBody.Visualizers.Colors.colorMapToSvg\">MultiBody.Visualizers.Colors.colorMapToSvg</a>.
The position and orientation of the center of the
circle at the left end of the pipe is defined via parameters
\"r_0\" and \"R\", respectively. The pipe axis is oriented along
the x-axis of the local coordinate system described by \"R\",
see figure below:
</p>

<blockquote>
<img src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Visualizers/PipeWithScalarField.png\">
</blockquote>

<p>
The color coding is shown in the next figure. It was generated with
<a href=\"modelica://Modelica.Mechanics.MultiBody.Visualizers.Colors.colorMapToSvg\">MultiBody.Visualizers.Colors.colorMapToSvg</a>
using the following call:
</p>

<blockquote>
<pre>
colorMapToSvg(Modelica.Mechanics.MultiBody.Visualizers.Colors.ColorMaps.jet(),
              height=50, nScalars=6, T_max=100, caption=\"Temperature in C\");
</pre>
</blockquote>

<blockquote>
<img src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Visualizers/PipeWithScalarField-ColorMap.png\">
</blockquote>
</html>"));
          end PipeWithScalarField;

          package SurfaceCharacteristics "Functions returning surface descriptions"
            extends Modelica.Icons.Package;

            function torus "Function defining the surface characteristic of a torus"
              extends Modelica.Mechanics.MultiBody.Interfaces.partialSurfaceCharacteristic(final multiColoredSurface = false);
              input Modelica.SIunits.Radius ri = 1 "Inner radius of torus" annotation(
                Dialog);
              input Modelica.SIunits.Radius ro = 0.2 "Outer radius of torus (=width/2)" annotation(
                Dialog);
              input Modelica.SIunits.Angle opening = 0 "Opening angle of torus" annotation(
                Dialog);
              input Modelica.SIunits.Angle startAngle = -Modelica.Constants.pi "Start angle of torus slice" annotation(
                Dialog);
              input Modelica.SIunits.Angle stopAngle = Modelica.Constants.pi "End angle of torus slice" annotation(
                Dialog);
            protected
              Modelica.SIunits.Angle alpha;
              Modelica.SIunits.Angle beta;
              Modelica.SIunits.Angle phi_start;
              Modelica.SIunits.Angle phi_stop;
            algorithm
              phi_start := (-Modelica.Constants.pi) + opening;
              phi_stop := Modelica.Constants.pi - opening;
              for i in 1:nu loop
                alpha := startAngle + (stopAngle - startAngle) * (i - 1) / (nu - 1);
                for j in 1:nv loop
                  beta := phi_start + (phi_stop - phi_start) * (j - 1) / (nv - 1);
                  X[i, j] := (ri + ro * cos(beta)) * sin(alpha);
                  Y[i, j] := ro * sin(beta);
                  Z[i, j] := (ri + ro * cos(beta)) * cos(alpha);
                end for;
              end for;
              annotation(
                Documentation(info = "<html>
<p>
Function <b>torus</b> computes the X,Y,Z arrays to visualize a torus
with model <a href=\"modelica://Modelica.Mechanics.MultiBody.Visualizers.Torus\">Torus</a>.
The left image below shows a torus with ri=0.5 m and ro = 0.2 m.
The right images below shows the torus with the additional parameter
settings:
</p>
<pre>
  opening    =   45 degree
  startAngle = -135 degree
  stopAngle  =  135 degree
</pre>

<blockquote>
<img src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Visualizers/Torus.png\">
</blockquote>
</html>"));
            end torus;

            function pipeWithScalarField "Function defining the surface characteristic of a pipe where a scalar field value is displayed with color along the pipe axis"
              extends Modelica.Mechanics.MultiBody.Interfaces.partialSurfaceCharacteristic(final multiColoredSurface = true);
              input Modelica.SIunits.Radius rOuter "Outer radius of cylinder" annotation(
                Dialog);
              input Modelica.SIunits.Length length "Length of cylinder" annotation(
                Dialog);
              input Modelica.SIunits.Position xsi[:] "Relative position along the pipe with x[1] = 0, x[end] = 1" annotation(
                Dialog);
              input Real T[size(xsi, 1)] "Scalar field value at position xsi*length" annotation(
                Dialog);
              input Real T_min "T <= T_min is mapped to colorMap[1,:]" annotation(
                Dialog);
              input Real T_max "T >= T_max is mapped to colorMap[end,:]" annotation(
                Dialog);
              input Real colorMap[:, 3] "Color map to map scalar T to a corresponding color" annotation(
                Dialog);
            protected
              Real beta;
              Real xsi_i;
              Real Ti;
              Real Ci[3];
              Integer k;
            algorithm
              k := 1;
              for i in 1:nu loop
                xsi_i := (i - 1) / (nu - 1);
                (Ti, k) := Modelica.Math.Vectors.interpolate(xsi, T, xsi_i, k);
                Ci := Modelica.Mechanics.MultiBody.Visualizers.Colors.scalarToColor(Ti, T_min, T_max, colorMap);
                for j in 1:nv loop
                  beta := 2 * Modelica.Constants.pi * (j - 1) / (nv - 1);
                  X[i, j] := length * xsi_i;
                  Y[i, j] := rOuter * sin(beta);
                  Z[i, j] := rOuter * cos(beta);
                  C[i, j, :] := Ci;
                end for;
              end for;
// Compute actual xsi-position along cylinder axis
// Interpolate in xsi and T to determine the corresponding value of Ti(xsi_i)
// Map the scalar field value Ti to a color value
// Determine outputs
              annotation(
                Documentation(info = "<html>
<p>
Function <b>pipeWithScalarField</b> computes the X,Y,Z,C arrays in order to
visualize a pipe and a scalar field along the pipe axis with model
<a href=\"modelica://Modelica.Mechanics.MultiBody.Visualizers.Advanced.PipeWithScalarField\">PipeWithScalarField</a>.
The latter is shown by mapping scalar
field to color values with a color map and utilizing this color
at the perimeter associated with the corresponding axis location.
Typically the scalar field value is a temperature, but might
be also another quantity.
Predefined color maps are available from
<a href=\"modelica://Modelica.Mechanics.MultiBody.Visualizers.Colors.ColorMaps\">MultiBody.Visualizers.Colors.ColorMaps</a>
and can be selected via input argument \"colorMap\".
A color map with the corresponding scalar field values can be exported
as vector-graphics in svg-format with function
<a href=\"modelica://Modelica.Mechanics.MultiBody.Visualizers.Colors.colorMapToSvg\">MultiBody.Visualizers.Colors.colorMapToSvg</a>.
An example is shown in the next figure:
</p>

<blockquote>
<img src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Visualizers/PipeWithScalarField.png\">
</blockquote>

<p>
The color coding is shown in the next figure. It was generated with
<a href=\"modelica://Modelica.Mechanics.MultiBody.Visualizers.Colors.colorMapToSvg\">Mechanics.MultiBody.Visualizers.Colors.colorMapToSvg</a>
using the following call:
</p>

<blockquote>
<pre>
colorMapToSvg(Modelica.Mechanics.MultiBody.Visualizers.Colors.ColorMaps.jet(),
              height=50, nScalars=6, T_max=100, heading=\"Temperature in C\");
</pre>
</blockquote>

<blockquote>
<img src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Visualizers/PipeWithScalarField-ColorMap.png\">
</blockquote>
</html>"));
            end pipeWithScalarField;
            annotation(
              Documentation(info = "<html>
<p>
This package contains functions that are used to define
parameterized surfaces for use with the
<a href=\"modelica://Modelica.Mechanics.MultiBody.Visualizers.Advanced.Surface\">Surface</a>
model.
</p>
</html>"));
          end SurfaceCharacteristics;
          annotation(
            Documentation(info = "<HTML>
<p>
Package <b>Visualizers.Advanced</b> contains components to visualize
3-dimensional shapes with dynamical sizes. None of the components
has a frame connector. The position and orientation is set via
modifiers. Basic knowledge of Modelica
is needed in order to utilize the components of this package.
These components have also to be used for models,
where the forces and torques in the frame connector are set via
equations (in this case, the models of the Visualizers package cannot be used,
since they all have frame connectors).
</p>
<h4>Content</h4>
<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"><a href=\"modelica://Modelica.Mechanics.MultiBody.Visualizers.Advanced.Arrow\">Arrow</a></td>
      <td valign=\"top\">Visualizing an arrow where all parts of the arrow can vary dynamically:<br>
      <IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Visualizers/Arrow.png\" ALT=\"model Visualizers.Advanced.Arrow\">
      </td>
  </tr>
  <tr><td valign=\"top\"><a href=\"modelica://Modelica.Mechanics.MultiBody.Visualizers.Advanced.DoubleArrow\">DoubleArrow</a></td>
      <td valign=\"top\">Visualizing a double arrow where all parts of the arrow can vary dynamically:<br>
      <IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Visualizers/DoubleArrow.png\" ALT=\"model Visualizers.Advanced.DoubleArrow\">
      </td>
  </tr>
  <tr><td valign=\"top\"><a href=\"modelica://Modelica.Mechanics.MultiBody.Visualizers.Advanced.Shape\">Shape</a></td>
      <td valign=\"top\">Visualizing an elementary object with variable size.
      The following shape types are supported:<br>&nbsp;<br>
      <IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/FixedShape.png\" ALT=\"model Visualizers.Advanced.Shape\">
      </td>
  </tr>

  <tr><td valign=\"top\"><a href=\"modelica://Modelica.Mechanics.MultiBody.Visualizers.Advanced.Surface\">Surface</a></td>
      <td valign=\"top\">Visualizing a moveable parameterized surface:<br>
      <IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Visualizers/Surface_small.png\">
      </td>
  </tr>

  <tr><td valign=\"top\"><a href=\"modelica://Modelica.Mechanics.MultiBody.Visualizers.Advanced.PipeWithScalarField\">PipeWithScalarField</a></td>
      <td valign=\"top\">Visualizing a pipe with a scalar field represented by a color coding:<br>
      <IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Visualizers/PipeWithScalarFieldIcon.png\">
      </td>
  </tr>
</table>
</HTML>"));
        end Advanced;

        package Internal "Visualizers that will be replaced by improved versions in the future (do not use them)"
          extends Modelica.Icons.InternalPackage;

          model FixedLines "Visualizing a set of lines as cylinders (e.g., used to display characters)"
            import Modelica.Mechanics.MultiBody;
            import Modelica.Mechanics.MultiBody.Types;
            extends Modelica.Mechanics.MultiBody.Interfaces.PartialVisualizer;
            parameter Boolean animation = true "= true, if animation shall be enabled";
            input Real scale(min = 0) = 1 "The 'lines' are visualized 'scale' times bigger" annotation(
              Dialog(group = "if animation = true", enable = animation));
            input SI.Position lines[:, 2, 2] = {[0, 0; 1, 1], [0, 1; 1, 0]} "List of start and end points of cylinders resolved along n_x and n_y" annotation(
              Dialog(group = "if animation = true", enable = animation));
            input SI.Distance diameter(min = 0) = 0.05 "Diameter of the cylinders defined by lines" annotation(
              Dialog(group = "if animation = true", enable = animation));
            input SI.Position r_lines[3] = {0, 0, 0} "Position vector from origin of frame_a to the origin of the 'lines' frame, resolved in frame_a" annotation(
              Dialog(group = "if animation = true", enable = animation));
            input Real n_x[3](each final unit = "1") = {1, 0, 0} "Vector in direction of x-axis of 'lines' frame, resolved in frame_a." annotation(
              Dialog(group = "if animation = true", enable = animation));
            input Real n_y[3](each final unit = "1") = {0, 1, 0} "Vector in direction of y-axis of 'lines' frame, resolved in frame_a." annotation(
              Dialog(group = "if animation = true", enable = animation));
            input MultiBody.Types.Color color = {0, 128, 255} "Color of cylinders" annotation(
              Dialog(colorSelector = true, group = "if animation = true", enable = animation));
            input Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(
              Dialog(group = "if animation = true", enable = animation));
          protected
            Lines x_label(lines = scale * lines, diameter = scale * diameter, color = color, specularCoefficient = specularCoefficient, r_lines = r_lines, n_x = n_x, n_y = n_y, r = frame_a.r_0, R = frame_a.R) if world.enableAnimation and animation;
          equation
            frame_a.f = zeros(3);
            frame_a.t = zeros(3);
            annotation(
              Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, 100}, {100, -100}}, lineColor = {128, 128, 128}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-150, 145}, {150, 105}}, textString = "%name", lineColor = {0, 0, 255}), Polygon(points = {{-82, -24}, {-20, 46}, {-10, 38}, {-72, -32}, {-82, -24}}, lineColor = {0, 127, 255}, fillColor = {0, 127, 255}, fillPattern = FillPattern.Solid), Polygon(points = {{-24, -34}, {-82, 40}, {-72, 46}, {-14, -26}, {-24, -34}}, lineColor = {0, 127, 255}, fillColor = {0, 127, 255}, fillPattern = FillPattern.Solid), Polygon(points = {{42, -18}, {10, 40}, {20, 48}, {50, -6}, {42, -18}}, lineColor = {0, 127, 255}, fillColor = {0, 127, 255}, fillPattern = FillPattern.Solid), Polygon(points = {{10, -68}, {84, 48}, {96, 42}, {24, -72}, {10, -68}}, lineColor = {0, 127, 255}, fillColor = {0, 127, 255}, fillPattern = FillPattern.Solid)}),
              Documentation(info = "<HTML>
<p>
With model <b>FixedLines</b> a set of lines is defined
that are located relatively to frame_a. Every line
is represented by a cylinder. This allows to define simple shaped
3-dimensional characters. An example is shown in the
following figure:<br>&nbsp;
</p>
<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/FixedLines.png\" ALT=\"model Visualizers.FixedLines\">
<p>&nbsp;<br>
The two letters \"x\" and \"y\" are constructed with 4 lines
by providing the following data for parameter <b>lines</b>
</p>
<pre>
   lines = {[0, 0; 1, 1],[0, 1; 1, 0],[1.5, -0.5; 2.5, 1],[1.5, 1; 2, 0.25]}
</pre>
<p>
Via parameter vectors <b>n_x</b> and <b>n_y</b> a two-dimensional
coordinate system is defined. The points defined with parameter
<b>lines</b> are with respect to this coordinate system. For example
\"[0, 0; 1, 1]\" defines a line that starts at {0,0} and ends at {1,1}.
The diameter and color of all line cylinders are identical.
</p>
</HTML>"));
          end FixedLines;

          model Lines "Visualizing a set of lines as cylinders with variable size, e.g., used to display characters (no Frame connector)"
            import Modelica.Mechanics.MultiBody;
            import Modelica.Mechanics.MultiBody.Types;
            import Modelica.Mechanics.MultiBody.Frames;
            import T = Modelica.Mechanics.MultiBody.Frames.TransformationMatrices;
            input Modelica.Mechanics.MultiBody.Frames.Orientation R = Frames.nullRotation() "Orientation object to rotate the world frame into the object frame" annotation(
              Dialog);
            input SI.Position r[3] = {0, 0, 0} "Position vector from origin of world frame to origin of object frame, resolved in world frame" annotation(
              Dialog);
            input SI.Position r_lines[3] = {0, 0, 0} "Position vector from origin of object frame to the origin of 'lines' frame, resolved in object frame" annotation(
              Dialog);
            input Real n_x[3](each final unit = "1") = {1, 0, 0} "Vector in direction of x-axis of 'lines' frame, resolved in object frame" annotation(
              Dialog);
            input Real n_y[3](each final unit = "1") = {0, 1, 0} "Vector in direction of y-axis of 'lines' frame, resolved in object frame" annotation(
              Dialog);
            input SI.Position lines[:, 2, 2] = zeros(0, 2, 2) "List of start and end points of cylinders resolved in an x-y frame defined by n_x, n_y, e.g., {[0,0;1,1], [0,1;1,0], [2,0; 3,1]}" annotation(
              Dialog);
            input SI.Length diameter(min = 0) = 0.05 "Diameter of the cylinders defined by lines" annotation(
              Dialog);
            input Modelica.Mechanics.MultiBody.Types.Color color = {0, 128, 255} "Color of cylinders" annotation(
              Dialog(colorSelector = true));
            input Types.SpecularCoefficient specularCoefficient = 0.7 "Reflection of ambient light (= 0: light is completely absorbed)" annotation(
              Dialog);
          protected
            parameter Integer n = size(lines, 1) "Number of cylinders";
            T.Orientation R_rel = T.from_nxy(n_x, n_y);
            T.Orientation R_lines = T.absoluteRotation(R.T, R_rel);
            Modelica.SIunits.Position r_abs[3] = r + T.resolve1(R.T, r_lines);
            Modelica.Mechanics.MultiBody.Visualizers.Advanced.Shape cylinders[n](each shapeType = "cylinder", lengthDirection = {T.resolve1(R_rel, vector([lines[i, 2, :] - lines[i, 1, :]; 0])) for i in 1:n}, length = {Modelica.Math.Vectors.length(lines[i, 2, :] - lines[i, 1, :]) for i in 1:n}, r = {r_abs + T.resolve1(R_lines, vector([lines[i, 1, :]; 0])) for i in 1:n}, each width = diameter, each height = diameter, each widthDirection = {0, 1, 0}, each color = color, each R = R, each specularCoefficient = specularCoefficient);
            annotation(
              Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, 100}, {100, -100}}, lineColor = {128, 128, 128}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Polygon(points = {{-24, -34}, {-82, 40}, {-72, 46}, {-14, -26}, {-24, -34}}, lineColor = {0, 127, 255}, fillColor = {0, 127, 255}, fillPattern = FillPattern.Solid), Polygon(points = {{-82, -24}, {-20, 46}, {-10, 38}, {-72, -32}, {-82, -24}}, lineColor = {0, 127, 255}, fillColor = {0, 127, 255}, fillPattern = FillPattern.Solid), Polygon(points = {{42, -18}, {10, 40}, {20, 48}, {50, -6}, {42, -18}}, lineColor = {0, 127, 255}, fillColor = {0, 127, 255}, fillPattern = FillPattern.Solid), Polygon(points = {{10, -68}, {84, 48}, {96, 42}, {24, -72}, {10, -68}}, lineColor = {0, 127, 255}, fillColor = {0, 127, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-150, 145}, {150, 105}}, textString = "%name", lineColor = {0, 0, 255})}),
              Documentation(info = "<HTML>
<p>
With model <b>Lines</b> a set of dynamic lines is defined
that are located relatively to frame_a. Every line
is represented by a cylinder. This allows, e.g., to define simple shaped
3-dimensional characters. Note, if the lines are fixed relatively to frame_a,
it is more convenient to use model <b>Visualizers.FixedLines</b>.
An example for dynamic lines is shown in the following figure:<br>&nbsp;
</p>
<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/FixedLines.png\" ALT=\"model Visualizers.FixedLines\">
<p>&nbsp;<br>
The two letters \"x\" and \"y\" are constructed with 4 lines
by providing the following data for input variable <b>lines</b>
</p>
<pre>
   lines = {[0, 0; 1, 1],[0, 1; 1, 0],[1.5, -0.5; 2.5, 1],[1.5, 1; 2, 0.25]}
</pre>
<p>
Via vectors <b>n_x</b> and <b>n_y</b> a two-dimensional
coordinate system is defined. The points defined with variable
<b>lines</b> are with respect to this coordinate system. For example
\"[0, 0; 1, 1]\" defines a line that starts at {0,0} and ends at {1,1}.
The diameter and color of all line cylinders are identical
and are defined by parameters.
</p>

</html>"));
          end Lines;
          annotation(
            Documentation(info = "<html>
<p>
This package contains components to construct 3-dim. fonts
with \"cylinder\" elements for the animation window.
This is just a temporary hack until 3-dim. fonts are supported in
Modelica tools. The components are used to construct the \"x\", \"y\",
\"z\" labels of coordinates systems in the animation.
</p>
</html>"));
        end Internal;
        annotation(
          Documentation(info = "<HTML>
<p>
Package <b>Visualizers</b> contains components to visualize
3-dimensional shapes. These components are the basis for the
animation features of the MultiBody library.
</p>
<h4>Content</h4>
<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"><a href=\"modelica://Modelica.Mechanics.MultiBody.Visualizers.FixedShape\">FixedShape</a><br>
             <a href=\"modelica://Modelica.Mechanics.MultiBody.Visualizers.FixedShape2\">FixedShape2</a></td>
      <td valign=\"top\">Visualizing an elementary shape with dynamically varying shape attributes.
      FixedShape has one connector frame_a, whereas FixedShape2 has additionally
          a frame_b for easier connection to further visual objects.
          The following shape types are supported:<br>&nbsp;<br>
      <IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/FixedShape.png\" ALT=\"model Visualizers.FixedShape\">
      </td>
  </tr>
  <tr><td valign=\"top\"><a href=\"modelica://Modelica.Mechanics.MultiBody.Visualizers.FixedFrame\">FixedFrame</a></td>
      <td valign=\"top\">Visualizing a coordinate system including axes labels with fixed sizes:<br>
      <IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/FixedFrame2.png\"
       ALT=\"model Visualizers.FixedFrame\">
      </td>
  </tr>
  <tr><td valign=\"top\"><a href=\"modelica://Modelica.Mechanics.MultiBody.Visualizers.FixedArrow\">FixedArrow</a>,<br>
<a href=\"modelica://Modelica.Mechanics.MultiBody.Visualizers.SignalArrow\">SignalArrow</a></td>
      <td valign=\"top\">Visualizing an arrow. Model \"FixedArrow\" provides
      a fixed sized arrow, model \"SignalArrow\" provides
      an arrow with dynamically varying length that is defined
      by an input signal vector:<br>
      <IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Visualizers/Arrow.png\">
      </td>
  </tr>

  <tr><td valign=\"top\"><a href=\"modelica://Modelica.Mechanics.MultiBody.Visualizers.Ground\">Ground</a></td>
      <td valign=\"top\">Visualizing the x-y plane by a box:<br>
      <IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Visualizers/GroundSmall.png\">
      </td>
  </tr>

  <tr><td valign=\"top\"><a href=\"modelica://Modelica.Mechanics.MultiBody.Visualizers.Torus\">Torus</a></td>
      <td valign=\"top\">Visualizing a torus:<br>
      <IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Visualizers/TorusIcon.png\">
      </td>
  </tr>

  <tr><td valign=\"top\"><a href=\"modelica://Modelica.Mechanics.MultiBody.Visualizers.VoluminousWheel\">VoluminousWheel</a></td>
      <td valign=\"top\">Visualizing a wheel:<br>
      <IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Visualizers/VoluminousWheelIcon.png\">
      </td>
  </tr>

  <tr><td valign=\"top\"><a href=\"modelica://Modelica.Mechanics.MultiBody.Visualizers.PipeWithScalarField\">PipeWithScalarField</a></td>
      <td valign=\"top\">Visualizing a pipe with a scalar field represented by a color coding:<br>
      <IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Visualizers/PipeWithScalarFieldIcon.png\">
      </td>
  </tr>

<tr><td valign=\"top\"><a href=\"modelica://Modelica.Mechanics.MultiBody.Visualizers.Advanced\">Advanced</a></td>
      <td valign=\"top\"> <b>Package</b> that contains components to visualize
          3-dimensional shapes where all parts of the shape
          can vary dynamically. Basic knowledge of Modelica is
          needed in order to utilize the components of this package.
      </td>
  </tr>
</table>
<p>
The colors of the visualization components are declared with
the predefined type <b>MultiBody.Types.Color</b>.
This is a vector with 3 elements,
{r, g, b}, and specifies the color of the shape.
{r,g,b} are the \"red\", \"green\" and \"blue\" color parts.
Note, r g, b are given as Integer[3] in the ranges 0 .. 255,
respectively.
</p>
</HTML>"),
          Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-80, 26}, {26, -70}}, lineColor = {95, 95, 95}, fillPattern = FillPattern.Solid, fillColor = {245, 245, 245}), Polygon(points = {{-80, 26}, {-16, 70}, {80, 70}, {26, 26}, {-80, 26}}, lineColor = {95, 95, 95}, smooth = Smooth.None, fillColor = {245, 245, 245}, fillPattern = FillPattern.Solid), Polygon(points = {{80, 70}, {26, 26}, {26, -70}, {80, -32}, {80, 70}}, lineColor = {95, 95, 95}, smooth = Smooth.None, fillColor = {245, 245, 245}, fillPattern = FillPattern.Solid)}));
      end Visualizers;

      package Types "Constants and types with choices, especially to build menus"
        extends Modelica.Icons.TypesPackage;
        type Axis = Modelica.Icons.TypeReal[3](each final unit = "1") "Axis vector with choices for menus" annotation(
          preferredView = "text",
          Evaluate = true,
          choices(choice = {1, 0, 0} "{1,0,0} \"x axis\"", choice = {0, 1, 0} "{0,1,0} \"y axis\"", choice = {0, 0, 1} "{0,0,1} \"z axis\"", choice = {-1, 0, 0} "{-1,0,0} \"negative x axis\"", choice = {0, -1, 0} "{0,-1,0} \"negative y axis\"", choice = {0, 0, -1} "{0,0,-1} \"negative z axis\""),
          Documentation(info = "<html>
</html>"));
        type AxisLabel = Modelica.Icons.TypeString "Label of axis with choices for menus" annotation(
          preferredView = "text",
          choices(choice = "x" "x", choice = "y" "y", choice = "z" "z"));
        type RotationSequence = Modelica.Icons.TypeInteger[3](min = {1, 1, 1}, max = {3, 3, 3}) "Sequence of planar frame rotations with choices for menus" annotation(
          preferredView = "text",
          Evaluate = true,
          choices(choice = {1, 2, 3} "{1,2,3} \"Cardan/Tait-Bryan angles\"", choice = {3, 1, 3} "{3,1,3} \"Euler angles\"", choice = {3, 2, 1} "{3,2,1}"));
        type Color = Modelica.Icons.TypeInteger[3](each min = 0, each max = 255) "RGB representation of color" annotation(
          Dialog(colorSelector = true),
          choices(choice = {0, 0, 0} "{0,0,0}       \"black\"", choice = {155, 0, 0} "{155,0,0}     \"dark red\"", choice = {255, 0, 0} "{255,0,0 }    \"red\"", choice = {255, 65, 65} "{255,65,65}   \"light red\"", choice = {0, 128, 0} "{0,128,0}     \"dark green\"", choice = {0, 180, 0} "{0,180,0}     \"green\"", choice = {0, 230, 0} "{0,230,0}     \"light green\"", choice = {0, 0, 200} "{0,0,200}     \"dark blue\"", choice = {0, 0, 255} "{0,0,255}     \"blue\"", choice = {0, 128, 255} "{0,128,255}   \"light blue\"", choice = {255, 255, 0} "{255,255,0}   \"yellow\"", choice = {255, 0, 255} "{255,0,255}   \"pink\"", choice = {100, 100, 100} "{100,100,100} \"dark grey\"", choice = {155, 155, 155} "{155,155,155} \"grey\"", choice = {255, 255, 255} "{255,255,255} \"white\""),
          Documentation(info = "<html>
<p>
Type <b>Color</b> is an Integer vector with 3 elements,
{r, g, b}, and specifies the color of a shape.
{r,g,b} are the \"red\", \"green\" and \"blue\" color parts.
Note, r g, b are given in the range 0 .. 255.
</p>
</html>"));
        type RealColor = Modelica.Icons.TypeReal[3](each min = 0, each max = 255) "RGB representation of color as Real type" annotation(
          Dialog(colorSelector = true),
          choices(choice = {0, 0, 0} "{0,0,0}       \"black\"", choice = {155, 0, 0} "{155,0,0}     \"dark red\"", choice = {255, 0, 0} "{255,0,0 }    \"red\"", choice = {255, 65, 65} "{255,65,65}   \"light red\"", choice = {0, 128, 0} "{0,128,0}     \"dark green\"", choice = {0, 180, 0} "{0,180,0}     \"green\"", choice = {0, 230, 0} "{0,230,0}     \"light green\"", choice = {0, 0, 200} "{0,0,200}     \"dark blue\"", choice = {0, 0, 255} "{0,0,255}     \"blue\"", choice = {0, 128, 255} "{0,128,255}   \"light blue\"", choice = {255, 255, 0} "{255,255,0}   \"yellow\"", choice = {255, 0, 255} "{255,0,255}   \"pink\"", choice = {100, 100, 100} "{100,100,100} \"dark grey\"", choice = {155, 155, 155} "{155,155,155} \"grey\"", choice = {255, 255, 255} "{255,255,255} \"white\""),
          Documentation(info = "<html>
<p>
Type <b>RealColor</b> is a Real vector with 3 elements,
{r, g, b}, and specifies the color of a shape.
{r,g,b} are the \"red\", \"green\" and \"blue\" color parts.
Note, r g, b are given in the range 0 .. 255.
</p>
</html>"));
        type SpecularCoefficient = Modelica.Icons.TypeReal(min = 0) "Reflection of ambient light (= 0: light is completely absorbed)" annotation(
          choices(choice = 0 "\"0.0 (dull)\"", choice = 0.7 "\"0.7 (medium)\"", choice = 1 "\"1.0 (glossy)\""),
          Documentation(info = "<html>
<p>
Type <b>SpecularCoefficient</b> defines the reflection of
ambient light on shape surfaces. If value = 0, the light
is completely absorbed. Often, 0.7 is a reasonable value.
It might be that from some viewing directions, a body is no
longer visible, if the SpecularCoefficient value is too high.
In the following image, the different values of SpecularCoefficient
are shown for a cylinder:
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Visualizers/SpecularCoefficient.png\" />
</p>
</html>"));
        type ShapeType = Modelica.Icons.TypeString "Type of shape (box, sphere, cylinder, pipecylinder, cone, pipe, beam, gearwheel, spring, <external shape>)" annotation(
          choices(choice = "box" "\"box\"", choice = "sphere" "\"sphere\"", choice = "cylinder" "\"cylinder\"", choice = "pipecylinder" "\"pipecylinder\"", choice = "cone" "\"cone\"", choice = "pipe" "\"pipe\"", choice = "beam" "\"beam\"", choice = "gearwheel" "\"gearwheel\"", choice = "spring" "\"spring\"", choice = "modelica://PackageName/PathName.dxf"),
          Documentation(info = "<html>
<p>
Type <b>ShapeType</b> is used to define the shape of the
visual object as parameter String. Usually, \"shapeType\" is used
as instance name. The following
values for shapeType are possible, e.g., shapeType=\"box\":
</p>

<p>
<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/Shape.png\" ALT=\"model Visualizers.FixedShape\">
</p>

<p>
The dark blue arrows in the figure above are directed along
variable <b>lengthDirection</b>. The light blue arrows are directed
along variable <b>widthDirection</b>. The <b>coordinate systems</b>
in the figure represent frame_a of the Shape component.
</p>

<p>
Additionally, external shapes can be specified as (not all options might be supported by all tools):
</p>

<ul>
<li> <b>\"1\", \"2\", ...</b><br>
     define external shapes specified in DXF format in files \"1.dxf\", \"2.dxf\", ...
     The DXF-files must be found either in the current directory or in the directory where
     the Shape instance is stored that references the DXF file.
     This (very limited) option should not be used for new models. Example:<br>
    shapeType=\"1\".<br></li>

<li> \"<b>modelica:</b>//&lt;Modelica-name&gt;/&lt;relative-path-file-name&gt;\"<br>
     characterizes the file that is stored under the location of the
     &lt;Modelica-name&gt; library path with the given relative file name.
     Example:<br> shapeType = \"modelica://Modelica/Resources/Data/Shapes/Engine/piston.dxf\".<br></li>

<li> \"<b>file:</b>//&lt;absolute-file-name&gt;\"<br>
     characterizes an absolute file name in the file system. Example:<br>
     shapeType=\"file://C:/users/myname/shapes/piston.dxf\".</li>
</ul>

<p>
The supported file formats are tool dependent. Most tools support
at least DXF-files (a tool might support 3-dim. Face of the DXF format only),
but may support other format as well (such as stl, obj, 3ds).
Since visualization files contain color and other data, the corresponding
information in the model is usually ignored.
</p>
</html>"));
        type ShapeExtra = Modelica.Icons.TypeReal "Type of the additional data that can be defined for an elementary ShapeType" annotation(
          Documentation(info = "<html>
<p>
This type is used in shapes of visual objects to define
extra data depending on the shape type. Usually, input
variable <b>extra</b> is used as instance name:
</p>

<table border=1 cellspacing=0 cellpadding=2>
<tr><th><b>shapeType</b></th><th>Meaning of parameter <b>extra</b></th></tr>
<tr>
  <td valign=\"top\">\"cylinder\"</td>
  <td valign=\"top\">if extra &gt; 0, a black line is included in the
      cylinder to show the rotation of it.</td>
</tr>
<tr>
  <td valign=\"top\">\"cone\"</td>
  <td valign=\"top\">extra = diameter-left-side / diameter-right-side, i.e.,<br>
      extra = 1: cylinder<br>
      extra = 0: \"real\" cone.</td>
</tr>
<tr>
  <td valign=\"top\">\"pipe\"</td>
  <td valign=\"top\">extra = outer-diameter / inner-diameter, i.e, <br>
      extra = 1: cylinder that is completely hollow<br>
      extra = 0: cylinder without a hole.</td>
</tr>
<tr>
  <td valign=\"top\">\"gearwheel\"</td>
  <td valign=\"top\">extra is the number of teeth of the (external) gear.
If extra &lt; 0, an internal gear is visualized with |extra| teeth.
The axis of the gearwheel is along \"lengthDirection\", and usually:
width = height = 2*radiusOfGearWheel.</td>
</tr>
<tr>
  <td valign=\"top\">\"spring\"</td>
  <td valign=\"top\">extra is the number of windings of the spring.
      Additionally, \"height\" is <b>not</b> the \"height\" but
      2*coil-width.</td>
</tr>
<tr>
  <td valign=\"top\">external shape</td>
  <td valign=\"top\">extra = 0: Visualization from file is not scaled.<br>
                     extra = 1: Visualization from file is scaled with \"length\", \"width\" and height\"
                                of the shape</td>
</tr>

</table>

</html>"));
        type ResolveInFrameA = enumeration(world "Resolve in world frame", frame_a "Resolve in frame_a", frame_resolve "Resolve in frame_resolve (frame_resolve must be connected)") "Enumeration to define the frame in which an absolute vector is resolved (world, frame_a, frame_resolve)" annotation(
          Documentation(info = "<html>
<table border=1 cellspacing=0 cellpadding=2>
<tr><th><b>Types.ResolveInFrameA.</b></th><th><b>Meaning</b></th></tr>
<tr><td valign=\"top\">world</td>
    <td valign=\"top\">Resolve vector in world frame</td></tr>

<tr><td valign=\"top\">frame_a</td>
    <td valign=\"top\">Resolve vector in frame_a</td></tr>

<tr><td valign=\"top\">frame_resolve</td>
    <td valign=\"top\">Resolve vector in frame_resolve (frame_resolve must be connected)</td></tr>
</table>
</html>"));
        type ResolveInFrameB = enumeration(world "Resolve in world frame", frame_b "Resolve in frame_b", frame_resolve "Resolve in frame_resolve (frame_resolve must be connected)") "Enumeration to define the frame in which an absolute vector is resolved (world, frame_b, frame_resolve)" annotation(
          Documentation(info = "<html>
<table border=1 cellspacing=0 cellpadding=2>
<tr><th><b>Types.ResolveInFrameB.</b></th><th><b>Meaning</b></th></tr>
<tr><td valign=\"top\">world</td>
    <td valign=\"top\">Resolve vector in world frame</td></tr>

<tr><td valign=\"top\">frame_b</td>
    <td valign=\"top\">Resolve vector in frame_b</td></tr>

<tr><td valign=\"top\">frame_resolve</td>
    <td valign=\"top\">Resolve vector in frame_resolve (frame_resolve must be connected)</td></tr>
</table>
</html>"));
        type ResolveInFrameAB = enumeration(world "Resolve in world frame", frame_a "Resolve in frame_a", frame_b "Resolve in frame_b", frame_resolve "Resolve in frame_resolve (frame_resolve must be connected)") "Enumeration to define the frame in which a relative vector is resolved (world, frame_a, frame_b, frame_resolve)" annotation(
          Documentation(info = "<html>
<table border=1 cellspacing=0 cellpadding=2>
<tr><th><b>Types.ResolveInFrameAB.</b></th><th><b>Meaning</b></th></tr>
<tr><td valign=\"top\">world</td>
    <td valign=\"top\">Resolve vector in world frame</td></tr>

<tr><td valign=\"top\">frame_a</td>
    <td valign=\"top\">Resolve vector in frame_a</td></tr>

<tr><td valign=\"top\">frame_b</td>
    <td valign=\"top\">Resolve vector in frame_b</td></tr>

<tr><td valign=\"top\">frame_resolve</td>
    <td valign=\"top\">Resolve vector in frame_resolve (frame_resolve must be connected)</td></tr>
</table>
</html>"));
        type RotationTypes = enumeration(RotationAxis "Rotating frame_a around an angle with a fixed axis", TwoAxesVectors "Resolve two vectors of frame_b in frame_a", PlanarRotationSequence "Planar rotation sequence") "Enumeration defining in which way the fixed orientation of frame_b with respect to frame_a is specified" annotation(
          Documentation(Evaluate = true, info = "<html>
<table border=1 cellspacing=0 cellpadding=2>
<tr><th><b>Types.RotationTypes.</b></th><th><b>Meaning</b></th></tr>
<tr><td valign=\"top\">RotationAxis</td>
    <td valign=\"top\">frame_b is defined by rotating the coordinate system along
        an axis fixed in frame_a and with a fixed angle.</td></tr>

<tr><td valign=\"top\">TwoAxesVectors</td>
    <td valign=\"top\">frame_b is defined by resolving two vectors of frame_b in frame_a.</td></tr>

<tr><td valign=\"top\">PlanarRotationSequence</td>
    <td valign=\"top\">frame_b is defined by rotating the coordinate system along
        3 consecutive axes vectors with fixed rotation angles
        (e.g., Cardan or Euler angle sequence rotation).</td></tr>
</table>
</html>"));
        type GravityTypes = enumeration(NoGravity "No gravity field", UniformGravity "Uniform gravity field", PointGravity "Point gravity field") "Enumeration defining the type of the gravity field" annotation(
          Documentation(info = "<html>
<table border=1 cellspacing=0 cellpadding=2>
<tr><th><b>Types.GravityTypes.</b></th><th><b>Meaning</b></th></tr>
<tr><td valign=\"top\">NoGravity</td>
    <td valign=\"top\">No gravity field</td></tr>

<tr><td valign=\"top\">UniformGravity</td>
    <td valign=\"top\">Gravity field is described by a vector of constant gravity acceleration</td></tr>

<tr><td valign=\"top\">PointGravity</td>
    <td valign=\"top\">Central gravity field. The gravity acceleration vector is directed to
        the field center and the gravity is proportional to 1/r^2, where
        r is the distance to the field center.</td></tr>
</table>
</html>"));
        type Init = enumeration(Free, PositionVelocity, SteadyState, Position, Velocity, VelocityAcceleration, PositionVelocityAcceleration);

        package Defaults "Default settings of the MultiBody library via constants"
          extends Modelica.Icons.Package;
          // Color defaults
          constant Types.Color BodyColor = {0, 128, 255} "Default color for body shapes that have mass (light blue)";
          constant Types.Color RodColor = {155, 155, 155} "Default color for massless rod shapes (grey)";
          constant Types.Color JointColor = {255, 0, 0} "Default color for elementary joints (red)";
          constant Types.Color ForceColor = {0, 128, 0} "Default color for force arrow (dark green)";
          constant Types.Color TorqueColor = {0, 128, 0} "Default color for torque arrow (dark green)";
          constant Types.Color SpringColor = {0, 0, 255} "Default color for a spring (blue)";
          constant Types.Color SensorColor = {255, 255, 0} "Default color for sensors (yellow)";
          constant Types.Color FrameColor = {0, 0, 0} "Default color for frame axes and labels (black)";
          constant Types.Color ArrowColor = {0, 0, 255} "Default color for arrows and double arrows (blue)";
          // Arrow and frame defaults
          constant Real FrameHeadLengthFraction = 5.0 "Frame arrow head length / arrow diameter";
          constant Real FrameHeadWidthFraction = 3.0 "Frame arrow head width / arrow diameter";
          constant Real FrameLabelHeightFraction = 3.0 "Height of frame label / arrow diameter";
          constant Real ArrowHeadLengthFraction = 4.0 "Arrow head length / arrow diameter";
          constant Real ArrowHeadWidthFraction = 3.0 "Arrow head width / arrow diameter";
          // Other defaults
          constant SI.Diameter BodyCylinderDiameterFraction = 3 "Default for body cylinder diameter as a fraction of body sphere diameter";
          constant Real JointRodDiameterFraction = 2 "Default for rod diameter as a fraction of joint sphere diameter attached to rod";
          /*
            constant Real N_to_m(unit="N/m") = 1000
              "Default force arrow scaling (length = force/N_to_m_default)";
            constant Real Nm_to_m(unit="N.m/m") = 1000
              "Default torque arrow scaling (length = torque/Nm_to_m_default)";
          */
          annotation(
            Documentation(info = "<html>
<p>
This package contains constants used as default setting
in the MultiBody library.
</p>
</html>"));
        end Defaults;
        annotation(
          Documentation(info = "<HTML>
<p>
In this package <b>types</b> and <b>constants</b> are defined that are used in the
MultiBody library. The types have additional annotation choices
definitions that define the menus to be built up in the graphical
user interface when the type is used as parameter in a declaration.
</p>
</HTML>"));
      end Types;

      package Icons "Icons for MultiBody package"
        extends Modelica.Icons.IconsPackage;

        model Surface "Surface icon"
          annotation(
            Icon(graphics = {Polygon(points = {{-102, 40}, {-98, 92}, {28, -8}, {96, 146}, {104, -118}, {-18, -34}, {-52, -130}, {-102, 40}}, lineColor = {0, 0, 255}, smooth = Smooth.Bezier, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-150, 140}, {150, 100}}, lineColor = {0, 0, 255}, textString = "%name")}),
            Documentation(info = "<html>
<p>
Model <b>Surface</b> defines a moveable, parametrized surface in 3-dim. space
that is used for animation. This object is specified by:
</p>

<ul>
<li> The surface frame (orientation object \"R\" and origin \"r_0\")
     in which the data is specified.</li>
<li> A set of two parameters, one in u- and one in v-direction,
     that defines the control points. </li>
<li> A time-varying position of each control point with respect to
     the surface frame.</li>
</ul>

<p>
The parameter values (u,v) are given by the ordinal numbers of the
corresponding control point in u- or in v-direction, respectively.
The surface is then defined by the replaceable function \"surfaceCharacteristic\" with the
interface <a href=\"modelica://Modelica.Mechanics.MultiBody.Interfaces.partialSurfaceCharacteristic\">partialSurfaceCharacteristic</a>
that returns the x-, y-, z- coordinate of every control point in form of 3 positional
arrays and an optional color array, if every control point shall have a different
color:
</p>

<pre>
  Real X[nu,nv], Y[nu,nv], Z[nu,nv], colorArray[nu,nv];
</pre>

<p>
Models <a href=\"modelica://Modelica.Mechanics.MultiBody.Visualizers.Torus\">Torus</a>
and <a href=\"modelica://Modelica.Mechanics.MultiBody.Visualizers.VoluminousWheel\">VoluminousWheel</a>
demonstrate how new visualizer objects can be constructed with the Surface model.<br>
The direct usage of the Surface model, as well as of the Torus and the VoluminousWheel models, are demonstrated with example
<a href=\"modelica://Modelica.Mechanics.MultiBody.Examples.Elementary.Surfaces\">Examples.Elementary.Surfaces</a>.
</p>
</html>"));
        end Surface;

        partial class MotorIcon "Icon of an electrical motor."
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(lineColor = {82, 0, 2}, fillColor = {252, 37, 57}, fillPattern = FillPattern.HorizontalCylinder, extent = {{-100.0, -50.0}, {30.0, 50.0}}, radius = 10.0), Polygon(fillColor = {64, 64, 64}, fillPattern = FillPattern.Solid, points = {{-100.0, -90.0}, {-90.0, -90.0}, {-60.0, -20.0}, {-10.0, -20.0}, {20.0, -90.0}, {30.0, -90.0}, {30.0, -100.0}, {-100.0, -100.0}, {-100.0, -90.0}}), Rectangle(lineColor = {64, 64, 64}, fillColor = {255, 255, 255}, fillPattern = FillPattern.HorizontalCylinder, extent = {{30.0, -10.0}, {90.0, 10.0}})}),
            Documentation(info = "<html>
<p>
Icon of an <b>electrical motor</b> model.
</p>
</html>"));
        end MotorIcon;
      end Icons;
      annotation(
        Documentation(info = "<HTML>
<p>
Library <b>MultiBody</b> is a <b>free</b> Modelica package providing
3-dimensional mechanical components to model in a convenient way
<b>mechanical systems</b>, such as robots, mechanisms, vehicles.
Typical animations generated with this library are shown
in the next figure:
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Mechanics/MultiBody/MultiBody.png\">
</p>

<p>
For an introduction, have especially a look at:
</p>
<ul>
<li> <a href=\"modelica://Modelica.Mechanics.MultiBody.UsersGuide\">MultiBody.UsersGuide</a>
     discusses the most important aspects how to use this library.</li>
<li> <a href=\"modelica://Modelica.Mechanics.MultiBody.Examples\">MultiBody.Examples</a>
     contains examples that demonstrate the usage of this library.</li>
</ul>

<p>
Copyright &copy; 1998-2015, Modelica Association and DLR.
</p>
<p>
<i>This Modelica package is <u>free</u> software and the use is completely at <u>your own risk</u>; it can be redistributed and/or modified under the terms of the Modelica License 2. For license conditions (including the disclaimer of warranty) see <a href=\"modelica://Modelica.UsersGuide.ModelicaLicense2\">Modelica.UsersGuide.ModelicaLicense2</a> or visit <a href=\"https://www.modelica.org/licenses/ModelicaLicense2\"> https://www.modelica.org/licenses/ModelicaLicense2</a>.</i>
</p>
</HTML>"),
        Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{-58, 76}, {6, 76}, {-26, 50}, {-58, 76}}, lineColor = {95, 95, 95}, smooth = Smooth.None, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{-26, 50}, {28, -50}}, color = {0, 0, 0}, smooth = Smooth.None), Ellipse(extent = {{-4, -14}, {60, -78}}, lineColor = {135, 135, 135}, fillPattern = FillPattern.Sphere, fillColor = {255, 255, 255})}));
    end MultiBody;

    package Rotational "Library to model 1-dimensional, rotational mechanical systems"
      extends Modelica.Icons.Package;
      import SI = Modelica.SIunits;

      package UsersGuide "User's Guide of Rotational Library"
        extends Modelica.Icons.Information;

        class Overview "Overview"
          extends Modelica.Icons.Information;
          annotation(
            DocumentationClass = true,
            Documentation(info = "<HTML>

<p>
This package contains components to model <b>1-dimensional rotational
mechanical</b> systems, including different types of gearboxes,
shafts with inertia, external torques, spring/damper elements,
frictional elements, backlash, elements to measure angle, angular velocity,
angular acceleration and the cut-torque of a flange. In sublibrary
<b>Examples</b> several examples are present to demonstrate the usage of
the elements. Just open the corresponding example model and simulate
the model according to the provided description.
</p>
<p>
A unique feature of this library is the <b>component-oriented</b>
modeling of <b>Coulomb friction</b> elements, such as friction in bearings,
clutches, brakes, and gear efficiency. Even (dynamically) coupled
friction elements, e.g., as in automatic gearboxes, can be handled
<b>without</b> introducing stiffness which leads to fast simulations.
The underlying theory is new and is based on the solution of mixed
continuous/discrete systems of equations, i.e., equations where the
<b>unknowns</b> are of type <b>Real</b>, <b>Integer</b> or <b>Boolean</b>.
Provided appropriate numerical algorithms for the solution of such types of
systems are available in the simulation tool, the simulation of
(dynamically) coupled friction elements of this library is
<b>efficient</b> and <b>reliable</b>.
</p>

<p>
<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/Rotational/drive1.png\" ALT=\"drive1\">
</p>

<p>
A simple example of the usage of this library is given in the
figure above. This drive consists of a shaft with inertia J1=0.2 which
is connected via an ideal gearbox with gear ratio=5 to a second shaft
with inertia J2=5. The left shaft is driven via an external,
sinusoidal torque.
The <b>filled</b> and <b>non-filled grey squares</b> at the left and
right side of a component represent <b>mechanical flanges</b>.
Drawing a line between such squares means that the corresponding
flanges are <b>rigidly attached</b> to each other.
By convention in this library, the connector characterized as a
<b>filled</b> grey square is called <b>flange_a</b> and placed at the
left side of the component in the \"design view\" and the connector
characterized as a <b>non-filled</b> grey square is called <b>flange_b</b>
and placed at the right side of the component in the \"design view\".
The two connectors are completely <b>identical</b>, with the only
exception that the graphical layout is a little bit different in order
to distinguish them for easier access of the connector variables.
For example, <code>J1.flange_a.tau</code> is the cut-torque in the connector
<code>flange_a</code> of component <code>J1</code>.
</p>
<p>
The components of this
library can be <b>connected</b> together in an <b>arbitrary</b> way. E.g., it is
possible to connect two springs or two shafts with inertia directly
together, see figure below.
</p>

<p>
<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/Rotational/driveConnections1.png\" ALT=\"driveConnections1\"><br>
<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/Rotational/driveConnections2.png\" ALT=\"driveConnections2\"><br>
</p>

</HTML>"));
        end Overview;

        class FlangeConnectors "Flange Connectors"
          extends Modelica.Icons.Information;
          annotation(
            DocumentationClass = true,
            Documentation(info = "<HTML>
<p>
A flange is described by the connector class
Interfaces.<b>Flange_a</b>
or Interfaces.<b>Flange_b</b>. As already noted, the two connector
classes are completely identical. There is only a difference in the icons,
in order to easier identify a flange variable in a diagram.
Both connector classes contain the following variables:
</p>
<pre>
   Modelica.SIunits.Angle       phi  \"Absolute rotation angle of flange\";
   <b>flow</b> Modelica.SIunits.Torque tau  \"Cut-torque in the flange\";
</pre>
<p>
If needed, the angular velocity <code>w</code> and the
angular acceleration <code>a</code> of a flange connector can be
determined by differentiation of the flange angle <code>phi</code>:
</p>
<pre>
     w = <b>der</b>(phi);    a = <b>der</b>(w);
</pre>
</HTML>"));
        end FlangeConnectors;

        class SupportTorques "Support Torques"
          extends Modelica.Icons.Information;
          annotation(
            DocumentationClass = true,
            Documentation(info = "<HTML>

<p>The following figure shows examples of components equipped with
a support flange (framed flange in the lower center), which can be used
to fix components on the ground or on other rotating elements or to combine
them with force elements. Via Boolean parameter <b>useSupport</b>, the
support torque is enabled or disabled. If it is enabled, it must be connected.
If it is disabled, it must not be connected.
Enabled support flanges offer, e.g., the possibility to model gearboxes mounted on
the ground via spring-damper-systems (cf. example
<a href=\"modelica://Modelica.Mechanics.Rotational.Examples.ElasticBearing\">ElasticBearing</a>).
</p>

<p>
<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/Rotational/bearing.png\" ALT=\"bearing\">
</p>

<p>
Depending on the setting of <b>useSupport</b>, the icon of the corresponding
component is changing, to either show the support flange or a ground mounting.
For example, the two implementations in the following figure give
identical results.
</p>

<p>
<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/Rotational/bearing2.png\" ALT=\"bearing2\">
</p>

</HTML>"));
        end SupportTorques;

        class SignConventions "Sign Conventions"
          extends Modelica.Icons.Information;
          annotation(
            DocumentationClass = true,
            Documentation(info = "<HTML>

<p>
The variables of a component of this library can be accessed in the
usual way. However, since most of these variables are basically elements
of <b>vectors</b>, i.e., have a direction, the question arises how the
signs of variables shall be interpreted. The basic idea is explained
at hand of the following figure:
</p>

<p>
<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/Rotational/drive2.png\" ALT=\"drive2\">
</p>

<p>
In the figure, three identical drive trains are shown. The only
difference is that the gear of the middle drive train and the
gear as well as the right inertia of the lower drive train
are horizontally flipped with regards to the upper drive train.
The signs of variables are now interpreted in the following way:
Due to the 1-dimensional nature of the model, all components are
basically connected together along one line (more complicated
cases are discussed below). First, one has to define
a <b>positive</b> direction of this line, called <b>axis of rotation</b>.
In the top part of the figure this is characterized by an arrow
defined as <code>axis of rotation</code>. The simple rule is now:
If a variable of a component is positive and can be interpreted as
the element of a vector (e.g., torque or angular velocity vector), the
corresponding vector is directed into the positive direction
of the axis of rotation. In the following figure, the right-most
inertias of the figure above are displayed with the positive
vector direction displayed according to this rule:
</p>

<p>
<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/Rotational/drive3.png\" ALT=\"drive3\">
</p>
<p>
The cut-torques <code>J2.flange_a.tau, J4.flange_a.tau, J6.flange_b.tau</code>
of the right inertias are all identical and are directed into the
direction of rotation if the values are positive. Similarly,
the angular velocities <code>J2.w, J4.w, J6.w</code> of the right inertias
are all identical and are also directed into the
direction of rotation if the values are positive. Some special
cases are shown in the next figure:
</p>

<p>
<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/Rotational/drive4.png\" ALT=\"drive4\">
</p>

<p>
In the upper part of the figure, two variants of the connection of an
external torque and an inertia are shown. In both cases, a positive
signal input into the torque component accelerates the inertias
<code>inertia1, inertia2</code> into the positive axis of rotation,
i.e., the angular accelerations <code>inertia1.a, inertia2.a</code>
are positive and are directed along the \"axis of rotation\" arrow.
In the lower part of the figure the connection of inertias with
a planetary gear is shown. Note, that the three flanges of the
planetary gearbox are located along the axis of rotation and that
the axis direction determines the positive rotation along these
flanges. As a result, the positive rotation for <code>inertia4, inertia6</code>
is as indicated with the additional grey arrows.
</p>
</HTML>"));
        end SignConventions;

        class UserDefinedComponents "User Defined Components"
          extends Modelica.Icons.Information;
          annotation(
            DocumentationClass = true,
            Documentation(info = "<HTML>
<p>
In this section some hints are given to define your own
1-dimensional rotational components which are compatible with the
elements of this package.
It is convenient to define a new
component by inheritance from one of the following base classes,
which are defined in sublibrary Interfaces:
</p>
<table BORDER=1 CELLSPACING=0 CELLPADDING=2>
<tr><th>Name</th><th>Description</th></tr>
<tr>
  <td valign=\"top\"><a href=\"modelica://Modelica.Mechanics.Rotational.Interfaces.PartialCompliant\">PartialCompliant</a>
  </td>
  <td valign=\"top\">Compliant connection of two rotational 1-dim. flanges
                   (used for force laws such as a spring or a damper).</td>
</tr>

<tr>
  <td valign=\"top\"><a href=\"modelica://Modelica.Mechanics.Rotational.Interfaces.PartialCompliantWithRelativeStates\">PartialCompliantWithRelativeStates</a>
  </td>
  <td valign=\"top\"> Same as \"PartialCompliant\", but relative angle and relative speed are
                    defined as preferred states. Use this partial model if the force law
                    needs anyway the relative speed. The advantage is that it is usually better
                    to use relative angles between drive train components
                    as states, especially, if the angle is not limited (e.g., as for drive trains
                    in vehicles).
</td>
</tr>

<tr>
  <td valign=\"top\"><a href=\"modelica://Modelica.Mechanics.Rotational.Interfaces.PartialElementaryTwoFlangesAndSupport2\">PartialElementaryTwoFlangesAndSupport2</a>
</td>
  <td valign=\"top\"> Partial model for a 1-dim. rotational gear consisting of the flange of
                    an input shaft, the flange of an output shaft and the support.
  </td>
</tr>

<tr>
  <td valign=\"top\"><a href=\"modelica://Modelica.Mechanics.Rotational.Interfaces.PartialTorque\">PartialTorque</a>
</td>
  <td valign=\"top\"> Partial model of a torque acting at the flange (accelerates the flange).
  </td>
</tr>

<tr>
  <td valign=\"top\"><a href=\"modelica://Modelica.Mechanics.Rotational.Interfaces.PartialTwoFlanges\">PartialTwoFlanges</a>
</td>
  <td valign=\"top\">General connection of two rotational 1-dim. flanges.
  </td>
</tr>

<tr>
  <td valign=\"top\"><a href=\"modelica://Modelica.Mechanics.Rotational.Interfaces.PartialAbsoluteSensor\">PartialAbsoluteSensor</a>
</td>
  <td valign=\"top\">Measure absolute flange variables.
  </td>
</tr>

<tr>
  <td valign=\"top\"><a href=\"modelica://Modelica.Mechanics.Rotational.Interfaces.PartialRelativeSensor\">PartialRelativeSensor</a>
</td>
  <td valign=\"top\">Measure relative flange variables.
  </td>
</tr>
</table>

<p>
The difference between these base classes are the auxiliary
variables defined in the model and the relations between
the flange variables already defined in the base class.
For example, in model <b>PartialCompliant</b> there is no
support flange, whereas in model
<b>PartialElementaryTwoFlangesAndSupport2</b>
there is a support flange.
</p>
<p>
The equations of a mechanical component are vector equations, i.e.,
they need to be expressed in a common coordinate system.
Therefore, for a component a <b>local axis of rotation</b> has to be
defined. All vector quantities, such as cut-torques or angular
velocities have to be expressed according to this definition.
Examples for such a definition are given in the following figure
for an inertia component and a planetary gearbox:
</p>

<p>
<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/Rotational/driveAxis.png\" ALT=\"driveAxis\">
</p>

<p>
As can be seen, all vectors are directed into the direction
of the rotation axis. The angles in the flanges are defined
correspondingly. For example, the angle <code>sun.phi</code> in the
flange of the sun wheel of the planetary gearbox is positive,
if rotated in mathematical positive direction (= counter clock
wise) along the axis of rotation.
</p>
<p>
On first view, one may assume that the selected local
coordinate system has an influence on the usage of the
component. But this is not the case, as shown in the next figure:
</p>

<p>
<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/Rotational/inertias.png\" ALT=\"inertias\">
</p>

<p>
In the figure the <b>local</b> axes of rotation of the components
are shown. The connection of two inertias in the left and in the
right part of the figure are completely equivalent, i.e., the right
part is just a different drawing of the left part. This is due to the
fact, that by a connection, the two local coordinate systems are
made identical and the (automatically) generated connection equations
(= angles are identical, cut-torques sum-up to zero) are also
expressed in this common coordinate system. Therefore, even if in
the left figure it seems to be that the angular velocity vector of
<code>J2</code> goes from right to left, in reality it goes from
left to right as shown in the right part of the figure, where the
local coordinate systems are drawn such that they are aligned.
Note, that the simple rule stated in section 4 (Sign conventions)
also determines that
the angular velocity of <code>J2</code> in the left part of the
figure is directed from left to right.
</p>
<p>
To summarize, the local coordinate system selected for a component
is just necessary, in order that the equations of this component
are expressed correctly. The selection of the coordinate system
is arbitrary and has no influence on the usage of the component.
Especially, the actual direction of, e.g., a cut-torque is most
easily determined by the rule of section 4. A more strict determination
by aligning coordinate systems and then using the vector direction
of the local coordinate systems, often requires a re-drawing of the
diagram and is therefore less convenient to use.
</p>
</HTML>"));
        end UserDefinedComponents;

        class RequirementsForSimulationTool "Requirements for Simulation Tools"
          extends Modelica.Icons.Information;
          annotation(
            DocumentationClass = true,
            Documentation(info = "<HTML>

<p>
This library is designed in a fully object oriented way in order that
components can be connected together in every meaningful combination
(e.g., direct connection of two springs or two inertias).
As a consequence, most models lead to a system of
differential-algebraic equations of <b>index 3</b> (= constraint
equations have to be differentiated twice in order to arrive at
a state space representation) and the Modelica translator or
the simulator has to cope with this system representation.
According to our present knowledge, this requires that the
Modelica translator is able to symbolically differentiate equations
(otherwise it is e.g., not possible to provide consistent initial
conditions; even if consistent initial conditions are present, most
numerical DAE integrators can cope at most with index 2 DAEs).
</p>
<p>
The elements of this library can be connected together in an
arbitrary way. However, difficulties may occur, if the elements which can <b>lock</b> the
<b>relative motion</b> between two flanges are connected <b>rigidly</b>
together such that essentially the <b>same relative motion</b> can be locked.
The reason is
that the cut-torque in the locked phase is not uniquely defined if the
elements are locked at the same time instant (i.e., there does not exist a
unique solution) and some simulation systems may not be
able to handle this situation, since this leads to a singularity during
simulation. Currently, this type of problem can occur with the
Coulomb friction elements <b>BearingFriction, Clutch, Brake, LossyGear</b> when
the elements become stuck:
</p>
<p>
<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/Rotational/driveConnections3.png\" ALT=\"driveConnections3\">
</p>
<p>
In the figure above two typical situations are shown: In the upper part of
the figure, the series connection of rigidly attached BearingFriction and
Clutch components are shown. This does not hurt, because the BearingFriction
element can lock the relative motion between the element and the housing,
whereas the clutch element can lock the relative motion between the two
connected flanges. Contrary, the drive train in the lower part of the figure
may give rise to simulation problems, because the BearingFriction element
and the Brake element can lock the relative motion between a flange and
the housing and these flanges are rigidly connected together, i.e.,
essentially the same relative motion can be locked. These difficulties
may be solved by either introducing a compliance between these flanges
or by combining the BearingFriction and Brake element into
one component and resolving the ambiguity of the frictional torque in the
stuck mode. A tool may handle this situation also <b>automatically</b>,
by picking one solution of the infinitely many, e.g., the one where
the difference to the value of the previous time instant is as small
as possible.
</p>
</HTML>"));
        end RequirementsForSimulationTool;

        class StateSelection "State Selection"
          extends Modelica.Icons.Information;
          annotation(
            Documentation(info = "<html>
<p>
Only a few components of the Rotational library use the der(..) operator
and are therefore candidates to have states. Most important, component <a href=\"modelica://Modelica.Mechanics.Rotational.Components.Inertia\">Inertia</a>
defines the absolute rotation angle and the absolute angular velocity of this
component as candidate for states. In the \"Advanced\" menu the built-in StateSelect
enumeration can be set to define the priority to use these variables as states.
Without further action, in most cases a tool will select these variables as states.
</p>

<p>
For positioning drive trains where the goal is to position a load (e.g. the drive
train of a robot, or of an elevator), the absolute angles of the components are bounded,
and the issue discussed below is not present.
</p>

<p>
For drive trains where the goal is to control the velocity of a load (e.g. the drive
train of a vehicle or the crank angle of an engine),
the absolute angles of the components are quickly increasing
during operation. This is critical, because then the step size control of time
integrators might then no longer work appropriately:
</p>

<p>
Integrators with step size control adjust their time step size automatically
to meet user defined error bounds (\"tolerances\").
Typically the local error estimate EST_i is compared with a mixed bound for absolute and relative errors.
</p>

<pre>
   EST_i &le; abstol_i + reltol_i*|x_i|
</pre>

<p>
Here, abstol_i and reltol_i denote the bounds for the absolute and relative error of state variable x_i, respectively. This mixed error bound is used since it is more robust than a pure relative error based error bound if the nominal value x_i  is (very) close to 0.
In a Modelica simulation model, typically the same relative tolerance reltol is used for all
states and the absolute tolerances are computed using the relative tolerance and the
nominal values of the states:
</p>

<pre>
   reltol_i = reltol
   abstol_i = reltol*x_i(nominal)*0.01
</pre>

<p>
This error control fails if the state variable x_i grows without bounds (such as for a
drive train or the crank angle of a vehicle), since then the allowed error
also grows without bounds. The effect is that the error control on this variable is practically
switched off. The correct way to handle this would be to set reltol_i = 0 on such a state
variable and only use an absolute tolerance for the step size control.
</p>

<p>
Currently, in Modelica there is no possibility to provide this information.
In order to reduce this effect, it is advisable to not use absolute angles, but
relative angles as states. A user can define relative variables as states
explicitly with component
<a href=\"modelica://Modelica.Mechanics.Rotational.Components.RelativeStates\">RelativeStates</a>.
Furthermore, all compliant components, such as
<a href=\"modelica://Modelica.Mechanics.Rotational.Components.SpringDamper\">SpringDamper</a> are
defining the relative angle and the relative angular velocity as preferred states.
Therefore, a tool will select in most cases relative angles as states.
</p>

<p>
The relative angles of compliant components are usually small. For example, the
deformation of a typical elastic component is in the order of 1e-4 rad.
Without further action, the error control would not work properly on variables
that are so small (so often switching the error control off). The remedy is to define
explicitly a nominal value on the relative angle. This definition is provided in the
\"Advanced\" menu of the compliant components with parameter \"phi_nominal\".
The default value is 1e-4 rad, to be in the order of a compliant deformation of a
drive. For some components, like
a <a href=\"modelica://Modelica.Mechanics.Rotational.Components.Clutch\">Clutch</a>
this might be too small and a value of phi_nominal=1 might be more appropriate
(a value of phi_nominal = 1e-4 does not hurt, but just makes the error control
unnecessarily stringent).
</p>
</html>"));
        end StateSelection;

        class Contact "Contact"
          extends Modelica.Icons.Contact;
          annotation(
            Documentation(info = "<html>
<dl>
<dt><b>Library Officer</b>
<dd><a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a> <br>
    Deutsches Zentrum f&uuml;r Luft und Raumfahrt e.V. (DLR)<br>
    Institut f&uuml;r Robotik und Mechatronik (DLR-RM)<br>
    Abteilung Systemdynamik und Regelungstechnik<br>
    Postfach 1116<br>
    D-82230 Wessling<br>
    Germany<br>
    email: <A HREF=\"mailto:Martin.Otter@dlr.de\">Martin.Otter@dlr.de</A><br><br>
</dl>

<p>
<b>Contributors to this library:</b>
</p>

<ul>
<li> <a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a> (DLR-RM)</li>
<li> Christian Schweiger (DLR-RM, until 2006).</li>
<li> <a href=\"http://www.haumer.at/\">Anton Haumer</a><br>
     Technical Consulting &amp; Electrical Engineering<br>
     A-3423 St.Andrae-Woerdern, Austria<br>
     email: <a href=\"mailto:a.haumer@haumer.at\">a.haumer@haumer.at</a></li>
</ul>
</html>"));
        end Contact;
        annotation(
          DocumentationClass = true,
          Documentation(info = "<HTML>
<p>
Library <b>Rotational</b> is a <b>free</b> Modelica package providing
1-dimensional, rotational mechanical components to model in a convenient way
drive trains with frictional losses.
</p>

</HTML>"));
      end UsersGuide;

      package Examples "Demonstration examples of the components of this package"
        extends Modelica.Icons.ExamplesPackage;

        model First "First example: simple drive train"
          extends Modelica.Icons.Example;
          parameter Modelica.SIunits.Torque amplitude = 10 "Amplitude of driving torque";
          parameter SI.Frequency freqHz = 5 "Frequency of driving torque";
          parameter SI.Inertia Jmotor(min = 0) = 0.1 "Motor inertia";
          parameter SI.Inertia Jload(min = 0) = 2 "Load inertia";
          parameter Real ratio = 10 "Gear ratio";
          parameter Real damping = 10 "Damping in bearing of gear";
          Rotational.Components.Fixed fixed annotation(
            Placement(transformation(extent = {{38, -48}, {54, -32}}, rotation = 0)));
          Rotational.Sources.Torque torque(useSupport = true) annotation(
            Placement(transformation(extent = {{-68, -8}, {-52, 8}}, rotation = 0)));
          Rotational.Components.Inertia inertia1(J = Jmotor) annotation(
            Placement(transformation(extent = {{-38, -8}, {-22, 8}}, rotation = 0)));
          Rotational.Components.IdealGear idealGear(ratio = ratio, useSupport = true) annotation(
            Placement(transformation(extent = {{-8, -8}, {8, 8}}, rotation = 0)));
          Rotational.Components.Inertia inertia2(J = 2, phi(fixed = true, start = 0), w(fixed = true, start = 0)) annotation(
            Placement(transformation(extent = {{22, -8}, {38, 8}}, rotation = 0)));
          Rotational.Components.Spring spring(c = 1.e4, phi_rel(fixed = true)) annotation(
            Placement(transformation(extent = {{52, -8}, {68, 8}}, rotation = 0)));
          Rotational.Components.Inertia inertia3(J = Jload, w(fixed = true, start = 0)) annotation(
            Placement(transformation(extent = {{82, -8}, {98, 8}}, rotation = 0)));
          Rotational.Components.Damper damper(d = damping) annotation(
            Placement(transformation(origin = {46, -22}, extent = {{-8, -8}, {8, 8}}, rotation = 270)));
          Modelica.Blocks.Sources.Sine sine(amplitude = amplitude, freqHz = freqHz) annotation(
            Placement(transformation(extent = {{-98, -8}, {-82, 8}}, rotation = 0)));
        equation
          connect(inertia1.flange_b, idealGear.flange_a) annotation(
            Line(points = {{-22, 0}, {-8, 0}}, color = {0, 0, 0}));
          connect(idealGear.flange_b, inertia2.flange_a) annotation(
            Line(points = {{8, 0}, {22, 0}}, color = {0, 0, 0}));
          connect(inertia2.flange_b, spring.flange_a) annotation(
            Line(points = {{38, 0}, {52, 0}}, color = {0, 0, 0}));
          connect(spring.flange_b, inertia3.flange_a) annotation(
            Line(points = {{68, 0}, {82, 0}}, color = {0, 0, 0}));
          connect(damper.flange_a, inertia2.flange_b) annotation(
            Line(points = {{46, -14}, {46, 0}, {38, 0}}, color = {0, 0, 0}));
          connect(damper.flange_b, fixed.flange) annotation(
            Line(points = {{46, -30}, {46, -40}}, color = {0, 0, 0}));
          connect(sine.y, torque.tau) annotation(
            Line(points = {{-81.2, 0}, {-69.6, 0}}, color = {0, 0, 127}));
          connect(torque.support, fixed.flange) annotation(
            Line(points = {{-60, -8}, {-60, -40}, {46, -40}}, color = {0, 0, 0}));
          connect(idealGear.support, fixed.flange) annotation(
            Line(points = {{0, -8}, {0, -40}, {46, -40}}, color = {0, 0, 0}));
          connect(torque.flange, inertia1.flange_a) annotation(
            Line(points = {{-52, 0}, {-38, 0}}, color = {0, 0, 0}, smooth = Smooth.None));
          annotation(
            Documentation(info = "<html>
<p>The drive train consists of a motor inertia which is driven by
a sine-wave motor torque. Via a gearbox the rotational energy is
transmitted to a load inertia. Elasticity in the gearbox is modeled
by a spring element. A linear damper is used to model the
damping in the gearbox bearing.</p>
<p>Note, that a force component (like the damper of this example)
which is acting between a shaft and the housing has to be fixed
in the housing on one side via component Fixed.</p>
<p>Simulate for 1 second and plot the following variables:<br>
   angular velocities of inertias inertia2 and 3: inertia2.w, inertia3.w</p>

</html>"),
            experiment(StopTime = 1.0, Interval = 0.001));
        end First;

        model FirstGrounded "First example: simple drive train with grounded elements"
          extends Modelica.Icons.Example;
          parameter Modelica.SIunits.Torque amplitude = 10 "Amplitude of driving torque";
          parameter SI.Frequency freqHz = 5 "Frequency of driving torque";
          parameter SI.Inertia Jmotor(min = 0) = 0.1 "Motor inertia";
          parameter SI.Inertia Jload(min = 0) = 2 "Load inertia";
          parameter Real ratio = 10 "Gear ratio";
          parameter Real damping = 10 "Damping in bearing of gear";
          Rotational.Components.Fixed fixed annotation(
            Placement(transformation(extent = {{38, -48}, {54, -32}}, rotation = 0)));
          Rotational.Sources.Torque torque(useSupport = false) annotation(
            Placement(transformation(extent = {{-68, -8}, {-52, 8}}, rotation = 0)));
          Rotational.Components.Inertia inertia1(J = Jmotor) annotation(
            Placement(transformation(extent = {{-38, -8}, {-22, 8}}, rotation = 0)));
          Rotational.Components.IdealGear idealGear(ratio = ratio, useSupport = false) annotation(
            Placement(transformation(extent = {{-8, -8}, {8, 8}}, rotation = 0)));
          Rotational.Components.Inertia inertia2(J = 2, phi(fixed = true, start = 0), w(fixed = true, start = 0)) annotation(
            Placement(transformation(extent = {{22, -8}, {38, 8}}, rotation = 0)));
          Rotational.Components.Spring spring(c = 1.e4, phi_rel(fixed = true)) annotation(
            Placement(transformation(extent = {{52, -8}, {68, 8}}, rotation = 0)));
          Rotational.Components.Inertia inertia3(J = Jload, w(fixed = true, start = 0)) annotation(
            Placement(transformation(extent = {{82, -8}, {98, 8}}, rotation = 0)));
          Rotational.Components.Damper damper(d = damping) annotation(
            Placement(transformation(origin = {46, -22}, extent = {{-8, -8}, {8, 8}}, rotation = 270)));
          Modelica.Blocks.Sources.Sine sine(amplitude = amplitude, freqHz = freqHz) annotation(
            Placement(transformation(extent = {{-98, -8}, {-82, 8}}, rotation = 0)));
        equation
          connect(inertia1.flange_b, idealGear.flange_a) annotation(
            Line(points = {{-22, 0}, {-8, 0}}, color = {0, 0, 0}));
          connect(idealGear.flange_b, inertia2.flange_a) annotation(
            Line(points = {{8, 0}, {22, 0}}, color = {0, 0, 0}));
          connect(inertia2.flange_b, spring.flange_a) annotation(
            Line(points = {{38, 0}, {52, 0}}, color = {0, 0, 0}));
          connect(spring.flange_b, inertia3.flange_a) annotation(
            Line(points = {{68, 0}, {82, 0}}, color = {0, 0, 0}));
          connect(damper.flange_a, inertia2.flange_b) annotation(
            Line(points = {{46, -14}, {46, 0}, {38, 0}}, color = {0, 0, 0}));
          connect(damper.flange_b, fixed.flange) annotation(
            Line(points = {{46, -30}, {46, -40}}, color = {0, 0, 0}));
          connect(sine.y, torque.tau) annotation(
            Line(points = {{-81.2, 0}, {-69.6, 0}}, color = {0, 0, 127}));
          connect(torque.flange, inertia1.flange_a) annotation(
            Line(points = {{-52, 0}, {-38, 0}}, color = {0, 0, 0}, smooth = Smooth.None));
          annotation(
            Documentation(info = "<html>
<p>The drive train consists of a motor inertia which is driven by
a sine-wave motor torque. Via a gearbox the rotational energy is
transmitted to a load inertia. Elasticity in the gearbox is modeled
by a spring element. A linear damper is used to model the
damping in the gearbox bearing.</p>
<p>Note, that a force component (like the damper of this example)
which is acting between a shaft and the housing has to be fixed
in the housing on one side via component Fixed.</p>
<p>Simulate for 1 second and plot the following variables:<br>
   angular velocities of inertias inertia2 and 3: inertia2.w, inertia3.w</p>

</HTML>"),
            experiment(StopTime = 1.0, Interval = 0.001));
        end FirstGrounded;

        model Friction "Drive train with clutch and brake"
          import Modelica.Constants.pi;
          extends Modelica.Icons.Example;
          parameter SI.Time startTime = 0.5 "Start time of step";
          output SI.Torque tMotor = torque.tau "Driving torque of inertia3";
          output SI.Torque tClutch = clutch.tau "Friction torque of clutch";
          output SI.Torque tBrake = brake.tau "Friction torque of brake";
          output SI.Torque tSpring = spring.tau "Spring torque";
          Rotational.Sources.Torque torque(useSupport = true) annotation(
            Placement(transformation(extent = {{-90, -10}, {-70, 10}}, rotation = 0)));
          Rotational.Components.Inertia inertia3(J = 1, phi(start = 0, fixed = true, displayUnit = "deg"), w(start = 100, fixed = true, displayUnit = "rad/s")) annotation(
            Placement(transformation(extent = {{-60, -10}, {-40, 10}}, rotation = 0)));
          Rotational.Components.Clutch clutch(fn_max = 160) annotation(
            Placement(transformation(extent = {{-30, -10}, {-10, 10}}, rotation = 0)));
          Rotational.Components.Inertia inertia2(J = 0.05, phi(start = 0, fixed = true), w(start = 90, fixed = true)) annotation(
            Placement(transformation(extent = {{0, -10}, {20, 10}}, rotation = 0)));
          Rotational.Components.SpringDamper spring(c = 160, d = 1) annotation(
            Placement(transformation(extent = {{30, -10}, {50, 10}}, rotation = 0)));
          Rotational.Components.Inertia inertia1(J = 1, phi(start = 0, fixed = true), w(start = 90, fixed = true)) annotation(
            Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
          Rotational.Components.Brake brake(fn_max = 1600, useSupport = true) annotation(
            Placement(transformation(extent = {{60, -10}, {80, 10}}, rotation = 0)));
          Modelica.Blocks.Sources.Constant const(k = 1) annotation(
            Placement(transformation(origin = {-25, 35}, extent = {{-5, -5}, {15, 15}}, rotation = 270)));
          Modelica.Blocks.Sources.Step step(startTime = startTime) annotation(
            Placement(transformation(origin = {65, 35}, extent = {{-5, -5}, {15, 15}}, rotation = 270)));
          Modelica.Blocks.Sources.Step step2(height = -1, offset = 1, startTime = startTime) annotation(
            Placement(transformation(extent = {{-160, -30}, {-140, -10}}, rotation = 0)));
          Modelica.Blocks.Sources.Sine sine(amplitude = 200, freqHz = 50 / pi) annotation(
            Placement(transformation(extent = {{-160, 10}, {-140, 30}}, rotation = 0)));
          Modelica.Blocks.Math.Product product annotation(
            Placement(transformation(extent = {{-120, -10}, {-100, 10}}, rotation = 0)));
          Rotational.Components.Fixed fixed annotation(
            Placement(transformation(extent = {{-10, -30}, {10, -10}}, rotation = 0)));
        equation
          connect(torque.flange, inertia3.flange_a) annotation(
            Line(points = {{-70, 0}, {-70, 0}, {-60, 0}}, color = {0, 0, 0}));
          connect(inertia3.flange_b, clutch.flange_a) annotation(
            Line(points = {{-40, 0}, {-30, 0}}, color = {0, 0, 0}));
          connect(clutch.flange_b, inertia2.flange_a) annotation(
            Line(points = {{-10, 0}, {0, 0}}, color = {0, 0, 0}));
          connect(inertia2.flange_b, spring.flange_a) annotation(
            Line(points = {{20, 0}, {30, 0}}, color = {0, 0, 0}));
          connect(spring.flange_b, brake.flange_a) annotation(
            Line(points = {{50, 0}, {60, 0}}, color = {0, 0, 0}));
          connect(brake.flange_b, inertia1.flange_a) annotation(
            Line(points = {{80, 0}, {80, 0}, {90, 0}}, color = {0, 0, 0}));
          connect(sine.y, product.u1) annotation(
            Line(points = {{-139, 20}, {-130, 20}, {-130, 6}, {-122, 6}}, color = {0, 0, 127}));
          connect(step2.y, product.u2) annotation(
            Line(points = {{-139, -20}, {-130, -20}, {-130, -6}, {-126, -6}, {-122, -6}}, color = {0, 0, 127}));
          connect(product.y, torque.tau) annotation(
            Line(points = {{-99, 0}, {-99, 0}, {-92, 0}}, color = {0, 0, 127}));
          connect(const.y, clutch.f_normalized) annotation(
            Line(points = {{-20, 19}, {-20, 12.75}, {-20, 11}}, color = {0, 0, 127}));
          connect(step.y, brake.f_normalized) annotation(
            Line(points = {{70, 19}, {70, 16}, {70, 11}}, color = {0, 0, 127}));
          connect(torque.support, fixed.flange) annotation(
            Line(points = {{-80, -10}, {-80, -20}, {0, -20}}, color = {0, 0, 0}));
          connect(brake.support, fixed.flange) annotation(
            Line(points = {{70, -10}, {70, -20}, {0, -20}}, color = {0, 0, 0}));
          annotation(
            Documentation(info = "<html>
<p>This drive train contains a frictional <b>clutch</b> and a <b>brake</b>.
Simulate the system for 1 second using the following initial
values (defined already in the model):</p>
<pre>   inertia1.w =  90 (or brake.w)
   inertia2.w =  90
   inertia3.w = 100
</pre>
<p>Plot the output signals</p>
<pre>   tMotor      Torque of motor
   tClutch     Torque in clutch
   tBrake      Torque in brake
   tSpring     Torque in spring
</pre>
<p>as well as the absolute angular velocities of the three inertia components
(inertia1.w, inertia2.w, inertia3.w).</p>

</HTML>"),
            experiment(StopTime = 3.0, Interval = 0.001),
            Diagram(coordinateSystem(extent = {{-170, -100}, {120, 100}})));
        end Friction;

        model CoupledClutches "Drive train with 3 dynamically coupled clutches"
          extends Modelica.Icons.Example;
          parameter SI.Frequency freqHz = 0.2 "Frequency of sine function to invoke clutch1";
          parameter SI.Time T2 = 0.4 "Time when clutch2 is invoked";
          parameter SI.Time T3 = 0.9 "Time when clutch3 is invoked";
          Rotational.Components.Inertia J1(J = 1, phi(fixed = true, start = 0), w(start = 10, fixed = true)) annotation(
            Placement(transformation(extent = {{-70, -10}, {-50, 10}}, rotation = 0)));
          Rotational.Sources.Torque torque(useSupport = true) annotation(
            Placement(transformation(extent = {{-100, -10}, {-80, 10}}, rotation = 0)));
          Rotational.Components.Clutch clutch1(peak = 1.1, fn_max = 20) annotation(
            Placement(transformation(extent = {{-40, -10}, {-20, 10}}, rotation = 0)));
          Modelica.Blocks.Sources.Sine sin1(amplitude = 10, freqHz = 5) annotation(
            Placement(transformation(extent = {{-130, -10}, {-110, 10}}, rotation = 0)));
          Modelica.Blocks.Sources.Step step1(startTime = T2) annotation(
            Placement(transformation(origin = {25, 35}, extent = {{-5, -5}, {15, 15}}, rotation = 270)));
          Rotational.Components.Inertia J2(J = 1, phi(fixed = true, start = 0), w(fixed = true, start = 0)) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0)));
          Rotational.Components.Clutch clutch2(peak = 1.1, fn_max = 20) annotation(
            Placement(transformation(extent = {{20, -10}, {40, 10}}, rotation = 0)));
          Rotational.Components.Inertia J3(J = 1, phi(fixed = true, start = 0), w(fixed = true, start = 0)) annotation(
            Placement(transformation(extent = {{50, -10}, {70, 10}}, rotation = 0)));
          Rotational.Components.Clutch clutch3(peak = 1.1, fn_max = 20) annotation(
            Placement(transformation(extent = {{80, -10}, {100, 10}}, rotation = 0)));
          Rotational.Components.Inertia J4(J = 1, phi(fixed = true, start = 0), w(fixed = true, start = 0)) annotation(
            Placement(transformation(extent = {{110, -10}, {130, 10}}, rotation = 0)));
          Modelica.Blocks.Sources.Sine sin2(amplitude = 1, freqHz = freqHz, phase = 1.57) annotation(
            Placement(transformation(origin = {-35, 35}, extent = {{-5, -5}, {15, 15}}, rotation = 270)));
          Modelica.Blocks.Sources.Step step2(startTime = T3) annotation(
            Placement(transformation(origin = {85, 35}, extent = {{-5, -5}, {15, 15}}, rotation = 270)));
          Rotational.Components.Fixed fixed annotation(
            Placement(transformation(extent = {{-100, -30}, {-80, -10}}, rotation = 0)));
        equation
          connect(torque.flange, J1.flange_a) annotation(
            Line(points = {{-80, 0}, {-70, 0}}, color = {0, 0, 0}));
          connect(J1.flange_b, clutch1.flange_a) annotation(
            Line(points = {{-50, 0}, {-40, 0}}, color = {0, 0, 0}));
          connect(clutch1.flange_b, J2.flange_a) annotation(
            Line(points = {{-20, 0}, {-10, 0}}, color = {0, 0, 0}));
          connect(J2.flange_b, clutch2.flange_a) annotation(
            Line(points = {{10, 0}, {10, 0}, {20, 0}}, color = {0, 0, 0}));
          connect(clutch2.flange_b, J3.flange_a) annotation(
            Line(points = {{40, 0}, {50, 0}}, color = {0, 0, 0}));
          connect(J3.flange_b, clutch3.flange_a) annotation(
            Line(points = {{70, 0}, {80, 0}}, color = {0, 0, 0}));
          connect(clutch3.flange_b, J4.flange_a) annotation(
            Line(points = {{100, 0}, {110, 0}}, color = {0, 0, 0}));
          connect(sin1.y, torque.tau) annotation(
            Line(points = {{-109, 0}, {-102, 0}}, color = {0, 0, 127}));
          connect(sin2.y, clutch1.f_normalized) annotation(
            Line(points = {{-30, 19}, {-30, 19}, {-30, 11}}, color = {0, 0, 127}));
          connect(step1.y, clutch2.f_normalized) annotation(
            Line(points = {{30, 19}, {30, 19}, {30, 10}, {30, 11}}, color = {0, 0, 127}));
          connect(step2.y, clutch3.f_normalized) annotation(
            Line(points = {{90, 19}, {90, 19}, {90, 11}}, color = {0, 0, 127}));
          connect(fixed.flange, torque.support) annotation(
            Line(points = {{-90, -20}, {-90, -11}, {-90, -10}}, color = {0, 0, 0}));
          annotation(
            Documentation(info = "<html>
<p>This example demonstrates how variable structure
drive trains are handled. The drive train consists
of 4 inertias and 3 clutches, where the clutches
are controlled by input signals. The system has
2^3=8 different configurations and 3^3 = 27
different states (every clutch may be in forward
sliding, backward sliding or locked mode when the
relative angular velocity is zero). By invoking the
clutches at different time instances, the switching
of the configurations can be studied.</p>
<p>Simulate the system for 1.2 seconds with the
following initial values:<br>
J1.w = 10.</p>
<p>Plot the following variables:<br>
angular velocities of inertias (J1.w, J2.w, J3.w,
J4.w), frictional torques of clutches (clutchX.tau),
frictional mode of clutches (clutchX.mode) where
mode = -1/0/+1 means backward sliding,
locked, forward sliding.</p>

</HTML>"),
            __Dymola_Commands(file = "modelica://Modelica/Resources/Scripts/Dymola/Mechanics/Rotational/CoupledClutches.mos" "Simulate and Plot"),
            experiment(StopTime = 1.5, Interval = 0.001),
            Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{-140, -100}, {140, 100}})));
        end CoupledClutches;

        model LossyGearDemo1 "Example to show that gear efficiency may lead to stuck motion"
          extends Modelica.Icons.Example;
          SI.Power PowerLoss = gear.flange_a.tau * der(gear.flange_a.phi) + gear.flange_b.tau * der(gear.flange_b.phi) "Power lost in the gear";
          Rotational.Components.LossyGear gear(ratio = 2, lossTable = [0, 0.5, 0.5, 0, 0], useSupport = true) annotation(
            Placement(transformation(extent = {{-10, 0}, {10, 20}}, rotation = 0)));
          Rotational.Components.Inertia Inertia1(J = 1) annotation(
            Placement(transformation(extent = {{-40, 0}, {-20, 20}}, rotation = 0)));
          Rotational.Components.Inertia Inertia2(J = 1.5, phi(fixed = true, start = 0), w(fixed = true, start = 0)) annotation(
            Placement(transformation(extent = {{20, 0}, {40, 20}}, rotation = 0)));
          Rotational.Sources.Torque torque1(useSupport = true) annotation(
            Placement(transformation(extent = {{-70, 0}, {-50, 20}}, rotation = 0)));
          Rotational.Sources.Torque torque2(useSupport = true) annotation(
            Placement(transformation(extent = {{70, 0}, {50, 20}}, rotation = 0)));
          Modelica.Blocks.Sources.Sine DriveSine(amplitude = 10, freqHz = 1) annotation(
            Placement(transformation(extent = {{-100, 0}, {-80, 20}}, rotation = 0)));
          Modelica.Blocks.Sources.Ramp load(height = 5, duration = 2, offset = -10) annotation(
            Placement(transformation(extent = {{100, 0}, {80, 20}}, rotation = 0)));
          Rotational.Components.Fixed fixed annotation(
            Placement(transformation(extent = {{-10, -30}, {10, -10}}, rotation = 0)));
        equation
          connect(Inertia1.flange_b, gear.flange_a) annotation(
            Line(points = {{-20, 10}, {-10, 10}}, color = {0, 0, 0}));
          connect(gear.flange_b, Inertia2.flange_a) annotation(
            Line(points = {{10, 10}, {20, 10}}, color = {0, 0, 0}));
          connect(torque1.flange, Inertia1.flange_a) annotation(
            Line(points = {{-50, 10}, {-40, 10}}, color = {0, 0, 0}));
          connect(torque2.flange, Inertia2.flange_b) annotation(
            Line(points = {{50, 10}, {40, 10}}, color = {0, 0, 0}));
          connect(DriveSine.y, torque1.tau) annotation(
            Line(points = {{-79, 10}, {-72, 10}}, color = {0, 0, 127}));
          connect(load.y, torque2.tau) annotation(
            Line(points = {{79, 10}, {72, 10}}, color = {0, 0, 127}));
          connect(fixed.flange, gear.support) annotation(
            Line(points = {{0, -20}, {0, 0}}, color = {0, 0, 0}));
          connect(fixed.flange, torque1.support) annotation(
            Line(points = {{0, -20}, {-60, -20}, {-60, 0}}, color = {0, 0, 0}));
          connect(fixed.flange, torque2.support) annotation(
            Line(points = {{0, -20}, {60, -20}, {60, 0}}, color = {0, 0, 0}));
          annotation(
            Documentation(info = "<html>
<p>
This model contains two inertias which are connected by an ideal
gear where the friction between the teeth of the gear is modeled in
a physical meaningful way (friction may lead to stuck mode which
locks the motion of the gear). The friction is defined by an
efficiency factor (= 0.5) for forward and backward driving condition leading
to a torque dependent friction loss. Simulate for about 0.5 seconds.
The friction in the gear will take all modes
(forward and backward rolling, as well as stuck).
</p>
<p>
You may plot:
</p>
<pre>
Inertia1.w,
Inertia2.w : angular velocities of inertias
powerLoss  : power lost in the gear
gear.mode  :  1 = forward rolling
              0 = stuck (w=0)
             -1 = backward rolling
</pre>
</html>"),
            experiment(StopTime = 0.5, Interval = 0.001));
        end LossyGearDemo1;

        model LossyGearDemo2 "Example to show combination of LossyGear and BearingFriction"
          extends Modelica.Icons.Example;
          SI.Power PowerLoss = gear.flange_a.tau * der(gear.flange_a.phi) + gear.flange_b.tau * der(gear.flange_b.phi) "Power lost in the gear";
          Rotational.Components.LossyGear gear(ratio = 2, lossTable = [0, 0.5, 0.5, 0, 0], useSupport = true) annotation(
            Placement(transformation(extent = {{-20, 0}, {0, 20}}, rotation = 0)));
          Rotational.Components.Inertia Inertia1(J = 1) annotation(
            Placement(transformation(extent = {{-50, 0}, {-30, 20}}, rotation = 0)));
          Rotational.Components.Inertia Inertia2(J = 1.5, phi(fixed = true, start = 0), w(fixed = true, start = 0)) annotation(
            Placement(transformation(extent = {{10, 0}, {30, 20}}, rotation = 0)));
          Rotational.Sources.Torque torque1(useSupport = true) annotation(
            Placement(transformation(extent = {{-110, 0}, {-90, 20}}, rotation = 0)));
          Rotational.Sources.Torque torque2(useSupport = true) annotation(
            Placement(transformation(extent = {{60, 0}, {40, 20}}, rotation = 0)));
          Modelica.Blocks.Sources.Sine DriveSine(amplitude = 10, freqHz = 1) annotation(
            Placement(transformation(extent = {{-140, 0}, {-120, 20}}, rotation = 0)));
          Modelica.Blocks.Sources.Ramp load(height = 5, duration = 2, offset = -10) annotation(
            Placement(transformation(extent = {{90, 0}, {70, 20}}, rotation = 0)));
          Rotational.Components.BearingFriction bearingFriction(tau_pos = [0, 0.5; 1, 1], useSupport = true) annotation(
            Placement(transformation(extent = {{-80, 0}, {-60, 20}}, rotation = 0)));
          Rotational.Components.Fixed fixed annotation(
            Placement(transformation(extent = {{-20, -30}, {0, -10}}, rotation = 0)));
        equation
          connect(torque2.flange, Inertia2.flange_b) annotation(
            Line(points = {{40, 10}, {30, 10}}, color = {0, 0, 0}));
          connect(Inertia2.flange_a, gear.flange_b) annotation(
            Line(points = {{10, 10}, {0, 10}}, color = {0, 0, 0}));
          connect(gear.flange_a, Inertia1.flange_b) annotation(
            Line(points = {{-20, 10}, {-30, 10}}, color = {0, 0, 0}));
          connect(Inertia1.flange_a, bearingFriction.flange_b) annotation(
            Line(points = {{-50, 10}, {-60, 10}}, color = {0, 0, 0}));
          connect(bearingFriction.flange_a, torque1.flange) annotation(
            Line(points = {{-80, 10}, {-80, 10}, {-90, 10}}, color = {0, 0, 0}));
          connect(DriveSine.y, torque1.tau) annotation(
            Line(points = {{-119, 10}, {-119, 10}, {-112, 10}}, color = {0, 0, 127}));
          connect(load.y, torque2.tau) annotation(
            Line(points = {{69, 10}, {62, 10}}, color = {0, 0, 127}));
          connect(gear.support, fixed.flange) annotation(
            Line(points = {{-10, 0}, {-10, -20}}, color = {0, 0, 0}));
          connect(fixed.flange, torque2.support) annotation(
            Line(points = {{-10, -20}, {50, -20}, {50, 0}}, color = {0, 0, 0}));
          connect(fixed.flange, bearingFriction.support) annotation(
            Line(points = {{-10, -20}, {-70, -20}, {-70, 0}}, color = {0, 0, 0}));
          connect(torque1.support, fixed.flange) annotation(
            Line(points = {{-100, 0}, {-100, -20}, {-10, -20}}, color = {0, 0, 0}));
          annotation(
            Diagram(coordinateSystem(extent = {{-150, -100}, {100, 100}})),
            Documentation(info = "<html>
<p>
This model contains bearing friction and gear friction (= efficiency).
If both friction models are stuck, there is no unique solution.
Still a reliable Modelica simulator, such as Dymola, should
be able to handle this situation.
</p>
<p>
Simulate for about 0.5 seconds. The friction elements are
in all modes (forward and backward rolling, as well as stuck).
</p>
<p>
You may plot:
</p>
<pre>
Inertia1.w,
Inertia2.w          : angular velocities of inertias
powerLoss           : power lost in the gear
bearingFriction.mode:  1 = forward rolling
                       0 = stuck (w=0)
                      -1 = backward rolling
gear.mode           :  1 = forward rolling
                       0 = stuck (w=0)
                      -1 = backward rolling
</pre>
<p>Note: This combination of LossyGear and BearingFriction is not recommended to use,
as component LossyGear includes the functionality of component BearingFriction
(only <i>peak</i> not supported).</p>
</html>"),
            experiment(StopTime = 0.5, Interval = 0.001));
        end LossyGearDemo2;

        model LossyGearDemo3 "Example that failed in the previous version of the LossyGear version"
          extends Modelica.Icons.Example;
          Modelica.Mechanics.Rotational.Components.LossyGear gear(ratio = 1, lossTable = [0, 0.25, 0.25, 0.625, 2.5], useSupport = false) annotation(
            Placement(transformation(extent = {{-10, 0}, {10, 20}}, rotation = 0)));
          Modelica.Mechanics.Rotational.Components.Inertia Inertia1(w(start = 10), J = 0.001) annotation(
            Placement(transformation(extent = {{-40, 0}, {-20, 20}}, rotation = 0)));
          Modelica.Mechanics.Rotational.Sources.Torque torque1(useSupport = false) annotation(
            Placement(transformation(extent = {{-68, 0}, {-48, 20}}, rotation = 0)));
          Modelica.Mechanics.Rotational.Sources.Torque torque2(useSupport = false) annotation(
            Placement(transformation(extent = {{68, 0}, {48, 20}}, rotation = 0)));
          Modelica.Blocks.Sources.Step step(height = 0) annotation(
            Placement(transformation(extent = {{-100, 0}, {-80, 20}}, rotation = 0)));
          Modelica.Blocks.Sources.Step step1(startTime = 0.5, height = 1, offset = 0) annotation(
            Placement(transformation(origin = {90, 10}, extent = {{-10, -10}, {10, 10}}, rotation = 180)));
          Modelica.Mechanics.Rotational.Components.Inertia Inertia2(J = 0.001, phi(fixed = true, start = 0), w(start = 10, fixed = true)) annotation(
            Placement(transformation(extent = {{20, 0}, {40, 20}}, rotation = 0)));
        equation
          connect(Inertia1.flange_b, gear.flange_a) annotation(
            Line(points = {{-20, 10}, {-16, 10}, {-10, 10}}, color = {0, 0, 0}));
          connect(torque1.flange, Inertia1.flange_a) annotation(
            Line(points = {{-48, 10}, {-40, 10}}, color = {0, 0, 0}));
          connect(step.y, torque1.tau) annotation(
            Line(points = {{-79, 10}, {-70, 10}}, color = {0, 0, 127}));
          connect(gear.flange_b, Inertia2.flange_a) annotation(
            Line(points = {{10, 10}, {20, 10}}, color = {0, 0, 0}));
          connect(Inertia2.flange_b, torque2.flange) annotation(
            Line(points = {{40, 10}, {48, 10}}, color = {0, 0, 0}));
          connect(step1.y, torque2.tau) annotation(
            Line(points = {{79, 10}, {74.5, 10}, {74.5, 10}, {70, 10}}, color = {0, 0, 127}, smooth = Smooth.None));
          annotation(
            Diagram(coordinateSystem(extent = {{-110, -100}, {110, 100}})),
            Documentation(info = "<html>
<p>
This example demonstrates a situation where the driving side of the
LossyGear model is not obvious.
The version of LossyGear up to version 3.1 of package Modelica failed in this case
(no convergence of the event iteration).
</p>
</html>"),
            experiment(StopTime = 1.0, Interval = 0.001));
        end LossyGearDemo3;

        model ElasticBearing "Example to show possible usage of support flange"
          extends Modelica.Icons.Example;
          Rotational.Components.Inertia shaft(phi(fixed = true, start = 0), J = 1, w(fixed = true, start = 0)) annotation(
            Placement(transformation(extent = {{-20, 40}, {0, 60}}, rotation = 0)));
          Rotational.Components.Inertia load(J = 50, w(fixed = true, start = 0)) annotation(
            Placement(transformation(extent = {{70, 40}, {90, 60}}, rotation = 0)));
          Rotational.Components.Spring spring(c = 1e3, phi_rel(fixed = true)) annotation(
            Placement(transformation(extent = {{40, 40}, {60, 60}}, rotation = 0)));
          Rotational.Components.Fixed fixed annotation(
            Placement(transformation(extent = {{10, -70}, {30, -50}}, rotation = 0)));
          Rotational.Components.SpringDamper springDamper(c = 1e5, d = 5, phi_rel(fixed = true), w_rel(fixed = true, nominal = 1e-4)) annotation(
            Placement(transformation(origin = {20, -36}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
          Rotational.Sources.Torque torque(useSupport = true) annotation(
            Placement(transformation(extent = {{-50, 40}, {-30, 60}}, rotation = 0)));
          Modelica.Blocks.Sources.Ramp ramp(duration = 5, height = 100) annotation(
            Placement(transformation(extent = {{-90, 40}, {-70, 60}}, rotation = 0)));
          Rotational.Components.IdealGear idealGear(ratio = 3, useSupport = true) annotation(
            Placement(transformation(extent = {{10, 40}, {30, 60}}, rotation = 0)));
          Rotational.Components.Inertia housing(J = 5) annotation(
            Placement(transformation(origin = {20, 22}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
          Sensors.MultiSensor multiSensor annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 90, origin = {20, -6})));
        equation
          connect(torque.flange, shaft.flange_a) annotation(
            Line(points = {{-30, 50}, {-20, 50}}, color = {0, 0, 0}));
          connect(spring.flange_b, load.flange_a) annotation(
            Line(points = {{60, 50}, {70, 50}}, color = {0, 0, 0}));
          connect(springDamper.flange_a, fixed.flange) annotation(
            Line(points = {{20, -46}, {20, -46}, {20, -60}}, color = {0, 0, 0}));
          connect(shaft.flange_b, idealGear.flange_a) annotation(
            Line(points = {{0, 50}, {10, 50}}, color = {0, 0, 0}));
          connect(idealGear.flange_b, spring.flange_a) annotation(
            Line(points = {{30, 50}, {40, 50}}, color = {0, 0, 0}));
          connect(idealGear.support, housing.flange_b) annotation(
            Line(points = {{20, 40}, {20, 32}}, color = {0, 0, 0}));
          connect(ramp.y, torque.tau) annotation(
            Line(points = {{-69, 50}, {-69, 50}, {-52, 50}}, color = {0, 0, 127}));
          connect(fixed.flange, torque.support) annotation(
            Line(points = {{20, -60}, {-40, -60}, {-40, 40}}, color = {0, 0, 0}));
          connect(housing.flange_a, multiSensor.flange_b) annotation(
            Line(points = {{20, 12}, {20, 4}}, color = {0, 0, 0}, smooth = Smooth.None));
          connect(multiSensor.flange_a, springDamper.flange_b) annotation(
            Line(points = {{20, -16}, {20, -26}}, color = {0, 0, 0}, smooth = Smooth.None));
          annotation(
            Documentation(info = "<html>
<p>
This model demonstrates the usage of the bearing flange.
The gearbox is not connected rigidly to the ground, but by
a spring-damper-system. This allows examination of the gearbox
housing dynamics.</p>
<p>
Simulate for about 10 seconds and plot the angular velocities of the inertias <code>housing.w</code>,
<code>shaft.w</code> and <code>load.w</code>.</p>
</html>"),
            experiment(StopTime = 10, Interval = 0.01));
        end ElasticBearing;

        model Backlash "Example to demonstrate backlash"
          extends Modelica.Icons.Example;
          Rotational.Components.Fixed fixed1 annotation(
            Placement(transformation(extent = {{-50, 50}, {-30, 70}})));
          Rotational.Components.SpringDamper springDamper(c = 20E3, d = 50, phi_nominal = 1) annotation(
            Placement(transformation(extent = {{-20, 50}, {0, 70}})));
          Rotational.Components.Inertia inertia1(J = 5, w(fixed = true, start = 0), phi(fixed = true, displayUnit = "deg", start = 1.570796326794897)) annotation(
            Placement(transformation(extent = {{20, 50}, {40, 70}})));
          Rotational.Components.Fixed fixed2 annotation(
            Placement(transformation(extent = {{-50, -50}, {-30, -30}})));
          Rotational.Components.ElastoBacklash elastoBacklash(c = 20E3, d = 50, b(displayUnit = "deg") = 0.7853981633974483, phi_nominal = 1) annotation(
            Placement(transformation(extent = {{-20, -50}, {0, -30}})));
          Rotational.Components.Inertia inertia2(J = 5, w(fixed = true, start = 0), phi(fixed = true, start = 1.570796326794897, displayUnit = "deg")) annotation(
            Placement(transformation(extent = {{20, -50}, {40, -30}})));
        equation
          connect(springDamper.flange_b, inertia1.flange_a) annotation(
            Line(points = {{0, 60}, {20, 60}}, color = {0, 0, 0}, smooth = Smooth.None));
          connect(elastoBacklash.flange_b, inertia2.flange_a) annotation(
            Line(points = {{0, -40}, {20, -40}}, color = {0, 0, 0}, smooth = Smooth.None));
          connect(fixed1.flange, springDamper.flange_a) annotation(
            Line(points = {{-40, 60}, {-20, 60}}, color = {0, 0, 0}, smooth = Smooth.None));
          connect(fixed2.flange, elastoBacklash.flange_a) annotation(
            Line(points = {{-40, -40}, {-20, -40}}, color = {0, 0, 0}, smooth = Smooth.None));
          annotation(
            Documentation(info = "<html>
<p>
This model demonstrates the effect of a backlash on eigenfrequency, and
also that the damping torque does not lead to unphysical pulling torques
(since the ElastoBacklash model takes care of it).
</p>
</html>"),
            experiment(StopTime = 1.0, Interval = 0.001));
        end Backlash;

        model RollingWheel "Demonstrate coupling Rotational - Translational"
          extends Modelica.Icons.Example;
          Rotational.Components.IdealRollingWheel idealRollingWheel(radius = 1) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}})));
          Rotational.Components.Inertia inertia(J = 1, phi(fixed = true, start = 0), w(fixed = true, start = 0)) annotation(
            Placement(transformation(extent = {{-40, -10}, {-20, 10}})));
          Rotational.Sources.TorqueStep torqueStep(stepTorque = 10, offsetTorque = 0, startTime = 0.1, useSupport = false) annotation(
            Placement(transformation(extent = {{-70, -10}, {-50, 10}})));
          Translational.Components.Mass mass(L = 0, m = 1) annotation(
            Placement(transformation(extent = {{20, -10}, {40, 10}})));
          Translational.Sources.QuadraticSpeedDependentForce quadraticSpeedDependentForce(f_nominal = -10, ForceDirection = false, v_nominal = 5) annotation(
            Placement(transformation(extent = {{72, -10}, {52, 10}})));
        equation
          connect(torqueStep.flange, inertia.flange_a) annotation(
            Line(points = {{-50, 0}, {-40, 0}}, color = {0, 0, 0}, smooth = Smooth.None));
          connect(inertia.flange_b, idealRollingWheel.flangeR) annotation(
            Line(points = {{-20, 0}, {-10, 0}}, color = {0, 0, 0}, smooth = Smooth.None));
          connect(idealRollingWheel.flangeT, mass.flange_a) annotation(
            Line(points = {{10, 0}, {20, 0}}, color = {0, 127, 0}, smooth = Smooth.None));
          connect(quadraticSpeedDependentForce.flange, mass.flange_b) annotation(
            Line(points = {{52, 0}, {40, 0}}, color = {0, 127, 0}, smooth = Smooth.None));
          annotation(
            Documentation(info = "<html>
<p>
This model demonstrates the coupling between rotational and translational components:<br>
A torque (step) accelerates both the inertia (of the wheel) and the mass (of the vehicle).<br>
Du to a speed dependent force (like driving resistance), we find an equilibrium at 5 m/s after approx. 5 s.
</p>
</html>"),
            experiment(StopTime = 5.0, Interval = 0.001));
        end RollingWheel;

        model HeatLosses "Demonstrate the modeling of heat losses"
          extends Modelica.Icons.Example;
          Blocks.Sources.Sine sine(freqHz = 5, amplitude = 20) annotation(
            Placement(transformation(extent = {{-100, 20}, {-80, 40}})));
          Sources.Torque torque annotation(
            Placement(transformation(extent = {{-70, 20}, {-50, 40}})));
          Components.Inertia inertia1(J = 2, phi(fixed = true, start = 0), w(fixed = true, start = 0)) annotation(
            Placement(transformation(extent = {{-40, 20}, {-20, 40}})));
          Components.Damper damper(useHeatPort = true, d = 10) annotation(
            Placement(transformation(extent = {{10, -10}, {-10, 10}}, rotation = -90, origin = {-20, 10})));
          Components.Fixed fixed annotation(
            Placement(transformation(extent = {{-30, -20}, {-10, 0}})));
          Thermal.HeatTransfer.Components.Convection convection annotation(
            Placement(transformation(extent = {{20, -20}, {40, -40}})));
          Thermal.HeatTransfer.Celsius.FixedTemperature TAmbient(T = 25) "Ambient temperature" annotation(
            Placement(transformation(extent = {{68, -40}, {48, -20}})));
          Blocks.Sources.Constant const(k = 20) annotation(
            Placement(transformation(extent = {{0, -70}, {20, -50}})));
          Components.SpringDamper springDamper(c = 1e4, d = 20, useHeatPort = true) annotation(
            Placement(transformation(extent = {{-10, 20}, {10, 40}})));
          Components.Inertia inertia2(J = 2, phi(fixed = true, start = 0), w(fixed = true, start = 0)) annotation(
            Placement(transformation(extent = {{20, 20}, {40, 40}})));
          Components.ElastoBacklash elastoBacklash(c = 1e5, d = 100, useHeatPort = true, b(displayUnit = "rad") = 0.001) annotation(
            Placement(transformation(extent = {{50, 20}, {70, 40}})));
          Components.Inertia inertia3(J = 2, phi(fixed = true, start = 0), w(fixed = true, start = 0)) annotation(
            Placement(transformation(extent = {{80, 20}, {100, 40}})));
          Components.BearingFriction bearingFriction(useHeatPort = true) annotation(
            Placement(transformation(extent = {{110, 20}, {130, 40}})));
          Components.Spring spring3(c = 1e4) annotation(
            Placement(transformation(extent = {{-70, 70}, {-50, 90}})));
          Components.Inertia inertia4(J = 2, phi(fixed = true, start = 0), w(fixed = true, start = 0)) annotation(
            Placement(transformation(extent = {{-40, 70}, {-20, 90}})));
          Components.LossyGear lossyGear(ratio = 2, lossTable = [0, 0.8, 0.8, 1, 1; 1, 0.7, 0.7, 2, 2], useHeatPort = true) annotation(
            Placement(transformation(extent = {{-10, 70}, {10, 90}})));
          Components.Clutch clutch(useHeatPort = true, fn_max = 10, phi_rel(fixed = true), w_rel(fixed = true)) annotation(
            Placement(transformation(extent = {{20, 70}, {40, 90}})));
          Components.Inertia inertia5(J = 2) annotation(
            Placement(transformation(extent = {{50, 70}, {70, 90}})));
          Blocks.Sources.Sine sine2(freqHz = 0.2, amplitude = 1) annotation(
            Placement(transformation(extent = {{0, 110}, {20, 130}})));
          Components.Inertia inertia6(J = 2) annotation(
            Placement(transformation(extent = {{110, 70}, {130, 90}})));
          Components.OneWayClutch oneWayClutch(phi_rel(fixed = true), w_rel(fixed = true), startForward(fixed = true), stuck(fixed = true), fn_max = 1, useHeatPort = true) annotation(
            Placement(transformation(extent = {{80, 70}, {100, 90}})));
          Components.Brake brake(fn_max = 2, useHeatPort = true) annotation(
            Placement(transformation(extent = {{140, 70}, {160, 90}})));
        equation
          connect(sine.y, torque.tau) annotation(
            Line(points = {{-79, 30}, {-72, 30}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(torque.flange, inertia1.flange_a) annotation(
            Line(points = {{-50, 30}, {-40, 30}}, color = {0, 0, 0}, smooth = Smooth.None));
          connect(inertia1.flange_b, damper.flange_b) annotation(
            Line(points = {{-20, 30}, {-20, 25}, {-20, 20}, {-20, 20}}, color = {0, 0, 0}, smooth = Smooth.None));
          connect(damper.flange_a, fixed.flange) annotation(
            Line(points = {{-20, 0}, {-20, -5}, {-20, -5}, {-20, -10}}, color = {0, 0, 0}, smooth = Smooth.None));
          connect(damper.heatPort, convection.solid) annotation(
            Line(points = {{-30, 0}, {-30, -30}, {20, -30}}, color = {191, 0, 0}, smooth = Smooth.None));
          connect(TAmbient.port, convection.fluid) annotation(
            Line(points = {{48, -30}, {40, -30}}, color = {191, 0, 0}, smooth = Smooth.None));
          connect(const.y, convection.Gc) annotation(
            Line(points = {{21, -60}, {30, -60}, {30, -40}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(inertia1.flange_b, springDamper.flange_a) annotation(
            Line(points = {{-20, 30}, {-10, 30}}, color = {0, 0, 0}, smooth = Smooth.None));
          connect(springDamper.heatPort, convection.solid) annotation(
            Line(points = {{-10, 20}, {-10, -30}, {20, -30}}, color = {191, 0, 0}, smooth = Smooth.None));
          connect(springDamper.flange_b, inertia2.flange_a) annotation(
            Line(points = {{10, 30}, {20, 30}}, color = {0, 0, 0}, smooth = Smooth.None));
          connect(elastoBacklash.flange_a, inertia2.flange_b) annotation(
            Line(points = {{50, 30}, {40, 30}}, color = {0, 0, 0}, smooth = Smooth.None));
          connect(elastoBacklash.heatPort, convection.solid) annotation(
            Line(points = {{50, 20}, {50, 0}, {-10, 0}, {-10, -30}, {20, -30}}, color = {191, 0, 0}, smooth = Smooth.None));
          connect(elastoBacklash.flange_b, inertia3.flange_a) annotation(
            Line(points = {{70, 30}, {80, 30}}, color = {0, 0, 0}, smooth = Smooth.None));
          connect(inertia3.flange_b, bearingFriction.flange_a) annotation(
            Line(points = {{100, 30}, {110, 30}}, color = {0, 0, 0}, smooth = Smooth.None));
          connect(convection.solid, bearingFriction.heatPort) annotation(
            Line(points = {{20, -30}, {-10, -30}, {-10, 0}, {110, 0}, {110, 20}}, color = {191, 0, 0}, smooth = Smooth.None));
          connect(spring3.flange_b, inertia4.flange_a) annotation(
            Line(points = {{-50, 80}, {-40, 80}}, color = {0, 0, 0}, smooth = Smooth.None));
          connect(bearingFriction.flange_b, spring3.flange_a) annotation(
            Line(points = {{130, 30}, {130, 48}, {-70, 48}, {-70, 80}}, color = {0, 0, 0}, smooth = Smooth.None));
          connect(inertia4.flange_b, lossyGear.flange_a) annotation(
            Line(points = {{-20, 80}, {-10, 80}}, color = {0, 0, 0}, smooth = Smooth.None));
          connect(lossyGear.heatPort, convection.solid) annotation(
            Line(points = {{-10, 70}, {-10, 60}, {140, 60}, {140, 0}, {-10, 0}, {-10, -30}, {20, -30}}, color = {191, 0, 0}, smooth = Smooth.None));
          connect(lossyGear.flange_b, clutch.flange_a) annotation(
            Line(points = {{10, 80}, {20, 80}}, color = {0, 0, 0}, smooth = Smooth.None));
          connect(clutch.heatPort, convection.solid) annotation(
            Line(points = {{20, 70}, {20, 60}, {140, 60}, {140, 0}, {-10, 0}, {-10, -30}, {20, -30}}, color = {191, 0, 0}, smooth = Smooth.None));
          connect(clutch.flange_b, inertia5.flange_a) annotation(
            Line(points = {{40, 80}, {50, 80}}, color = {0, 0, 0}, smooth = Smooth.None));
          connect(sine2.y, clutch.f_normalized) annotation(
            Line(points = {{21, 120}, {30, 120}, {30, 91}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(inertia5.flange_b, oneWayClutch.flange_a) annotation(
            Line(points = {{70, 80}, {80, 80}}, color = {0, 0, 0}, smooth = Smooth.None));
          connect(oneWayClutch.flange_b, inertia6.flange_a) annotation(
            Line(points = {{100, 80}, {110, 80}}, color = {0, 0, 0}, smooth = Smooth.None));
          connect(sine2.y, oneWayClutch.f_normalized) annotation(
            Line(points = {{21, 120}, {90, 120}, {90, 91}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(inertia6.flange_b, brake.flange_a) annotation(
            Line(points = {{130, 80}, {140, 80}}, color = {0, 0, 0}, smooth = Smooth.None));
          connect(sine2.y, brake.f_normalized) annotation(
            Line(points = {{21, 120}, {150, 120}, {150, 91}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(oneWayClutch.heatPort, convection.solid) annotation(
            Line(points = {{80, 70}, {80, 60}, {140, 60}, {140, 0}, {-10, 0}, {-10, -30}, {20, -30}}, color = {191, 0, 0}, smooth = Smooth.None));
          connect(brake.heatPort, convection.solid) annotation(
            Line(points = {{140, 70}, {140, 0}, {-10, 0}, {-10, -30}, {20, -30}}, color = {191, 0, 0}, smooth = Smooth.None));
          annotation(
            Documentation(info = "<html>
<p>
This model demonstrates how to model the dissipated power of a drive train,
by enabling the heatPort of all components and connecting these heatPorts via
a convection element to the environment. The total heat flow generated by the
elements of the drive train and transported to the environment
is present in variable convection.fluid.
</p>
</html>"),
            experiment(StopTime = 1.0, Interval = 0.0001),
            Diagram(coordinateSystem(extent = {{-120, -100}, {180, 140}})));
        end HeatLosses;

        model SimpleGearShift "Simple Gearshift"
          extends Modelica.Icons.Example;
          output Modelica.SIunits.AngularVelocity wEngine = engine.w "Speed of engine";
          output Modelica.SIunits.AngularVelocity wLoad = load.w "Speed of load";
          output Real gearRatio = wLoad / max(wEngine, 1E-6) "gear ratio load/engine";
          Modelica.Mechanics.Rotational.Sources.TorqueStep torqueStep(offsetTorque = 0, startTime = 0.5, stepTorque = 20) annotation(
            Placement(transformation(extent = {{-80, -20}, {-60, 0}})));
          Modelica.Mechanics.Rotational.Components.Inertia engine(J = 1) annotation(
            Placement(transformation(extent = {{-50, -20}, {-30, 0}})));
          Modelica.Mechanics.Rotational.Components.IdealPlanetary idealPlanetary(ratio = 75 / 50) annotation(
            Placement(transformation(extent = {{-10, 0}, {10, -20}})));
          Modelica.Mechanics.Rotational.Components.Inertia load(J = 10, phi(fixed = true, start = 0), w(fixed = true, start = 0)) annotation(
            Placement(transformation(extent = {{30, -20}, {50, 0}})));
          Modelica.Mechanics.Rotational.Sources.QuadraticSpeedDependentTorque quadraticSpeedDependentTorque(w_nominal(displayUnit = "rpm") = 10.471975511966, tau_nominal = -20) annotation(
            Placement(transformation(extent = {{80, -20}, {60, 0}})));
          Modelica.Mechanics.Rotational.Components.Clutch clutch(cgeo = 2, fn_max = 100, phi_rel(fixed = true), w_rel(fixed = true)) annotation(
            Placement(transformation(extent = {{-10, 10}, {10, 30}})));
          Modelica.Mechanics.Rotational.Components.Brake brake(cgeo = 2, fn_max = 100) annotation(
            Placement(transformation(extent = {{20, 10}, {40, 30}})));
          Modelica.Blocks.Math.Feedback feedback annotation(
            Placement(transformation(extent = {{-10, 50}, {10, 70}})));
          Modelica.Blocks.Sources.Constant const1(k = 1) annotation(
            Placement(transformation(extent = {{-40, 50}, {-20, 70}})));
          Modelica.Blocks.Sources.Ramp ramp(height = 1, offset = 0, startTime = 2, duration = 0.1) annotation(
            Placement(transformation(extent = {{-60, 30}, {-40, 50}})));
        equation
          connect(torqueStep.flange, engine.flange_a) annotation(
            Line(points = {{-60, -10}, {-50, -10}}, color = {0, 0, 0}, smooth = Smooth.None));
          connect(quadraticSpeedDependentTorque.flange, load.flange_b) annotation(
            Line(points = {{60, -10}, {50, -10}}, color = {0, 0, 0}, smooth = Smooth.None));
          connect(feedback.y, brake.f_normalized) annotation(
            Line(points = {{9, 60}, {30, 60}, {30, 31}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(engine.flange_b, idealPlanetary.sun) annotation(
            Line(points = {{-30, -10}, {-10, -10}}, color = {0, 0, 0}, smooth = Smooth.None));
          connect(idealPlanetary.sun, clutch.flange_a) annotation(
            Line(points = {{-10, -10}, {-10, 20}}, color = {0, 0, 0}, smooth = Smooth.None));
          connect(idealPlanetary.ring, clutch.flange_b) annotation(
            Line(points = {{10, -10}, {10, 20}}, color = {0, 0, 0}, smooth = Smooth.None));
          connect(idealPlanetary.ring, brake.flange_a) annotation(
            Line(points = {{10, -10}, {20, -10}, {20, 20}}, color = {0, 0, 0}, smooth = Smooth.None));
          connect(idealPlanetary.carrier, load.flange_a) annotation(
            Line(points = {{-10, -14}, {-20, -14}, {-20, -30}, {30, -30}, {30, -10}}, color = {0, 0, 0}, smooth = Smooth.None));
          connect(const1.y, feedback.u1) annotation(
            Line(points = {{-19, 60}, {-8, 60}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(feedback.u2, clutch.f_normalized) annotation(
            Line(points = {{0, 52}, {0, 31}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(ramp.y, clutch.f_normalized) annotation(
            Line(points = {{-39, 40}, {0, 40}, {0, 31}}, color = {0, 0, 127}, smooth = Smooth.None));
          annotation(
            experiment(StopTime = 5, Interval = 0.01),
            Documentation(info = "<html>
<p>This model shows how an automatic gear shift is built up from a planetary gear, a brake and a clutch. </p>
<ul>
<li>In the beginning, the clutch is free and the brake fixes the ring of the planetary. Thus we obtain a gearRatio = 1/(1 + planetary.ratio).</li>
<li>At time = 2 s, the brake frees the ring of the planetary, whereas the clutch blocks the ring and the sun. Thus we obtain a gearRatio = 1.</li>
</ul>
</html>"));
        end SimpleGearShift;

        model GenerationOfFMUs "Example to demonstrate variants to generate FMUs (Functional Mockup Units)"
          extends Modelica.Icons.Example;
          Blocks.Sources.Sine sine1(freqHz = 2, amplitude = 10) annotation(
            Placement(transformation(extent = {{-60, 40}, {-40, 60}})));
          Utilities.DirectInertia directInertia(J = 1.1) annotation(
            Placement(transformation(extent = {{-20, 40}, {0, 60}})));
          Utilities.InverseInertia inverseInertia(J = 2.2) annotation(
            Placement(transformation(extent = {{20, 40}, {40, 60}})));
          Utilities.SpringDamper springDamper(c = 1e4, d = 100) annotation(
            Placement(transformation(extent = {{10, -20}, {30, 0}})));
          Blocks.Sources.Sine sine2(freqHz = 2, amplitude = 10) annotation(
            Placement(transformation(extent = {{-100, -20}, {-80, 0}})));
          Components.Inertia inertia2a(J = 1.1, phi(fixed = true, start = 0), w(fixed = true, start = 0)) annotation(
            Placement(transformation(extent = {{-40, -20}, {-20, 0}})));
          Sources.Torque torque2 annotation(
            Placement(transformation(extent = {{-68, -20}, {-48, 0}})));
          Components.TorqueToAngleAdaptor torqueToAngle2a(use_a = false) annotation(
            Placement(transformation(extent = {{-20, -20}, {0, 0}})));
          Components.Inertia inertia2b(phi(fixed = true, start = 0), w(fixed = true, start = 0), J = 2.2) annotation(
            Placement(transformation(extent = {{60, -20}, {80, 0}})));
          Components.TorqueToAngleAdaptor torqueToAngle2b(use_a = false) annotation(
            Placement(transformation(extent = {{60, -20}, {40, 0}})));
          Utilities.Spring spring(c = 1e4) annotation(
            Placement(transformation(extent = {{10, -80}, {30, -60}})));
          Blocks.Sources.Sine sine3(freqHz = 2, amplitude = 10) annotation(
            Placement(transformation(extent = {{-100, -80}, {-80, -60}})));
          Components.Inertia inertia3a(J = 1.1, phi(fixed = true, start = 0), w(fixed = true, start = 0)) annotation(
            Placement(transformation(extent = {{-40, -80}, {-20, -60}})));
          Sources.Torque torque3 annotation(
            Placement(transformation(extent = {{-68, -80}, {-48, -60}})));
          Components.TorqueToAngleAdaptor torqueToAngle3a(use_a = false, use_w = false) annotation(
            Placement(transformation(extent = {{-20, -80}, {0, -60}})));
          Components.Inertia inertia3b(phi(fixed = true, start = 0), w(fixed = true, start = 0), J = 2.2) annotation(
            Placement(transformation(extent = {{60, -80}, {80, -60}})));
          Components.TorqueToAngleAdaptor torqueToAngle3b(use_a = false, use_w = false) annotation(
            Placement(transformation(extent = {{60, -80}, {40, -60}})));
        equation
          connect(sine1.y, directInertia.tauDrive) annotation(
            Line(points = {{-39, 50}, {-22, 50}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(directInertia.phi, inverseInertia.phi) annotation(
            Line(points = {{1, 58}, {18, 58}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(directInertia.w, inverseInertia.w) annotation(
            Line(points = {{1, 53}, {18, 53}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(directInertia.a, inverseInertia.a) annotation(
            Line(points = {{1, 47}, {18, 47}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(inverseInertia.tau, directInertia.tau) annotation(
            Line(points = {{19, 42}, {2, 42}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(torque2.flange, inertia2a.flange_a) annotation(
            Line(points = {{-48, -10}, {-40, -10}}, color = {0, 0, 0}, smooth = Smooth.None));
          connect(sine2.y, torque2.tau) annotation(
            Line(points = {{-79, -10}, {-70, -10}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(inertia2a.flange_b, torqueToAngle2a.flange) annotation(
            Line(points = {{-20, -10}, {-12, -10}}, color = {0, 0, 0}, smooth = Smooth.None));
          connect(torqueToAngle2a.phi, springDamper.phi1) annotation(
            Line(points = {{-7, -2}, {8, -2}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(torqueToAngle2a.w, springDamper.w1) annotation(
            Line(points = {{-7, -7}, {8, -7}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(springDamper.tau1, torqueToAngle2a.tau) annotation(
            Line(points = {{9, -18}, {-6, -18}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(torqueToAngle2b.phi, springDamper.phi2) annotation(
            Line(points = {{47, -2}, {32, -2}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(torqueToAngle2b.w, springDamper.w2) annotation(
            Line(points = {{47, -7}, {32, -7}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(springDamper.tau2, torqueToAngle2b.tau) annotation(
            Line(points = {{31, -18}, {46, -18}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(inertia2b.flange_a, torqueToAngle2b.flange) annotation(
            Line(points = {{60, -10}, {52, -10}}, color = {0, 0, 0}, smooth = Smooth.None));
          connect(torque3.flange, inertia3a.flange_a) annotation(
            Line(points = {{-48, -70}, {-40, -70}}, color = {0, 0, 0}, smooth = Smooth.None));
          connect(sine3.y, torque3.tau) annotation(
            Line(points = {{-79, -70}, {-70, -70}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(inertia3a.flange_b, torqueToAngle3a.flange) annotation(
            Line(points = {{-20, -70}, {-12, -70}}, color = {0, 0, 0}, smooth = Smooth.None));
          connect(torqueToAngle3a.phi, spring.phi1) annotation(
            Line(points = {{-7, -62}, {8, -62}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(spring.tau1, torqueToAngle3a.tau) annotation(
            Line(points = {{9, -78}, {-6, -78}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(torqueToAngle3b.phi, spring.phi2) annotation(
            Line(points = {{47, -62}, {32, -62}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(spring.tau2, torqueToAngle3b.tau) annotation(
            Line(points = {{31, -78}, {46, -78}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(inertia3b.flange_a, torqueToAngle3b.flange) annotation(
            Line(points = {{60, -70}, {52, -70}}, color = {0, 0, 0}, smooth = Smooth.None));
          annotation(
            Documentation(info = "<html>
<p>
This example demonstrates how to generate an input/output block (e.g. in form of an
FMU - <a href=\"https://www.fmi-standard.org\">Functional Mockup Unit</a>) from various Rotational components.
The goal is to export such an input/output block from Modelica and import
it in another modeling environment. The essential issue is that before
exporting it must be known in which way the component is utilized in the
target environment. Depending on the target usage, different flange variables
need to be in the interface with either input or output causality.
Note, this example model can be used to test the FMU export/import of a Modelica tool.
Just export the components marked in the icons as \"toFMU\" as FMUs and import
them back. The models should then still work and give the same results as a
pure Modelica model.
</p>

<p>
<b>Connecting two inertias</b><br>
The upper part (DirectInertia, InverseInertia)
demonstrates how to export two inertias and connect them
together in a target system. This requires that one of the inertias
(here: DirectInertia)
is defined to have states and the angle, angular velocity and angular
acceleration are provided in the interface.
The other inertia (here: InverseInertia) is moved according
to the provided input angle, angular velocity and angular acceleration.
</p>

<p>
<b>Connecting a force element that needs angles and angular velocities</b><br>
The middle part (SpringDamper) demonstrates how to export a force element
that needs both angles and angular velocities for its force law and connect this
force law in a target system between two inertias.
</p>

<p>
<b>Connecting a force element that needs only angles</b><br>
The lower part (Spring) demonstrates how to export a force element
that needs only angles for its force law and connect this
force law in a target system between two inertias.
</p>
</html>"));
        end GenerationOfFMUs;

        package Utilities "Utility components used by the example models"
          extends Modelica.Icons.UtilitiesPackage;

          model DirectInertia "Input/output block of a direct inertia model"
            extends Modelica.Blocks.Icons.Block;
            parameter Modelica.SIunits.Inertia J(min = 0) = 1 "Moment of inertia";
            Modelica.Mechanics.Rotational.Components.Inertia inertia(phi(start = 0, fixed = true), w(start = 0, fixed = true), J = J) annotation(
              Placement(transformation(extent = {{-20, -10}, {0, 10}})));
            Modelica.Mechanics.Rotational.Sources.Torque torque annotation(
              Placement(transformation(extent = {{-50, -10}, {-30, 10}})));
            Modelica.Blocks.Interfaces.RealInput tauDrive(unit = "N.m") "Accelerating torque acting at flange (= -flange.tau)" annotation(
              Placement(transformation(extent = {{-140, -20}, {-100, 20}})));
            Components.TorqueToAngleAdaptor torqueToAngle annotation(
              Placement(transformation(extent = {{4, -10}, {24, 10}})));
            Modelica.Blocks.Interfaces.RealOutput phi(unit = "rad") "Inertia moves with angle phi due to torque tau" annotation(
              Placement(transformation(extent = {{100, 70}, {120, 90}})));
            Modelica.Blocks.Interfaces.RealOutput w(unit = "rad/s") "Inertia moves with speed w due to torque tau" annotation(
              Placement(transformation(extent = {{100, 20}, {120, 40}})));
            Modelica.Blocks.Interfaces.RealOutput a(unit = "rad/s2") "Inertia moves with angular acceleration a due to torque tau" annotation(
              Placement(transformation(extent = {{100, -40}, {120, -20}})));
            Modelica.Blocks.Interfaces.RealInput tau(unit = "N.m") "Torque to drive the inertia" annotation(
              Placement(transformation(extent = {{140, -100}, {100, -60}})));
          equation
            connect(torque.flange, inertia.flange_a) annotation(
              Line(points = {{-30, 0}, {-20, 0}}, color = {0, 0, 0}, smooth = Smooth.None));
            connect(torque.tau, tauDrive) annotation(
              Line(points = {{-52, 0}, {-120, 0}}, color = {0, 0, 127}, smooth = Smooth.None));
            connect(inertia.flange_b, torqueToAngle.flange) annotation(
              Line(points = {{0, 0}, {12, 0}}, color = {0, 0, 0}, smooth = Smooth.None));
            connect(torqueToAngle.phi, phi) annotation(
              Line(points = {{17, 8}, {60, 8}, {60, 80}, {110, 80}}, color = {0, 0, 127}, smooth = Smooth.None));
            connect(torqueToAngle.w, w) annotation(
              Line(points = {{17, 3}, {66, 3}, {66, 30}, {110, 30}}, color = {0, 0, 127}, smooth = Smooth.None));
            connect(torqueToAngle.tau, tau) annotation(
              Line(points = {{18, -8}, {60, -8}, {60, -80}, {120, -80}}, color = {0, 0, 127}, smooth = Smooth.None));
            connect(torqueToAngle.a, a) annotation(
              Line(points = {{17, -3}, {66, -3}, {66, -30}, {110, -30}}, color = {0, 0, 127}, smooth = Smooth.None));
            annotation(
              Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-84, -58}, {24, -90}}, lineColor = {135, 135, 135}, fillColor = {235, 245, 255}, fillPattern = FillPattern.Solid, textString = "to FMU"), Text(extent = {{8, 96}, {92, 66}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {245, 245, 245}, textString = "phi", horizontalAlignment = TextAlignment.Right), Text(extent = {{10, 46}, {94, 16}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {245, 245, 245}, horizontalAlignment = TextAlignment.Right, textString = "w"), Text(extent = {{6, -10}, {90, -40}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {245, 245, 245}, horizontalAlignment = TextAlignment.Right, textString = "a"), Text(extent = {{-150, -110}, {150, -140}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {245, 245, 245}, textString = "J=%J"), Bitmap(extent = {{-96, -42}, {64, 54}}, fileName = "modelica://Modelica/Resources/Images/Mechanics/Rotational/DirectInertia.png"), Text(extent = {{10, -60}, {94, -90}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {245, 245, 245}, horizontalAlignment = TextAlignment.Right, textString = "tau")}));
          end DirectInertia;

          model InverseInertia "Input/output block of an inverse inertia model"
            extends Modelica.Blocks.Icons.Block;
            parameter Modelica.SIunits.Inertia J(min = 0) = 1 "Moment of inertia";
            Modelica.Mechanics.Rotational.Components.Inertia inertia(J = J, phi(start = 0, fixed = false), w(start = 0, fixed = false)) annotation(
              Placement(transformation(extent = {{-12, -10}, {8, 10}})));
            Components.AngleToTorqueAdaptor angleToTorque annotation(
              Placement(transformation(extent = {{-36, -10}, {-16, 10}})));
            Modelica.Blocks.Interfaces.RealInput phi(unit = "rad") "Angle to drive the inertia" annotation(
              Placement(transformation(extent = {{-140, 60}, {-100, 100}})));
            Modelica.Blocks.Interfaces.RealInput w(unit = "rad/s") "Speed to drive the inertia" annotation(
              Placement(transformation(extent = {{-140, 10}, {-100, 50}})));
            Modelica.Blocks.Interfaces.RealInput a(unit = "rad/s2") "Angular acceleration to drive the inertia" annotation(
              Placement(transformation(extent = {{-140, -50}, {-100, -10}})));
            Modelica.Blocks.Interfaces.RealOutput tau(unit = "N.m") "Torque needed to drive the flange according to phi, w, a" annotation(
              Placement(transformation(extent = {{-100, -90}, {-120, -70}})));
          equation
            connect(angleToTorque.phi, phi) annotation(
              Line(points = {{-30, 8}, {-84, 8}, {-84, 80}, {-120, 80}}, color = {0, 0, 127}, smooth = Smooth.None));
            connect(angleToTorque.w, w) annotation(
              Line(points = {{-30, 2.8}, {-90, 2.8}, {-90, 30}, {-120, 30}}, color = {0, 0, 127}, smooth = Smooth.None));
            connect(angleToTorque.a, a) annotation(
              Line(points = {{-30, -3.2}, {-80, -3.2}, {-80, -30}, {-120, -30}}, color = {0, 0, 127}, smooth = Smooth.None));
            connect(angleToTorque.flange, inertia.flange_a) annotation(
              Line(points = {{-24, 0}, {-12, 0}}, color = {0, 0, 0}, smooth = Smooth.None));
            connect(angleToTorque.tau, tau) annotation(
              Line(points = {{-29, -8}, {-70, -8}, {-70, -80}, {-110, -80}}, color = {0, 0, 127}, smooth = Smooth.None));
            annotation(
              Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{0, -62}, {96, -94}}, lineColor = {135, 135, 135}, fillColor = {235, 245, 255}, fillPattern = FillPattern.Solid, textString = "to FMU"), Text(extent = {{-94, 96}, {-10, 66}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {245, 245, 245}, horizontalAlignment = TextAlignment.Left, textString = "phi"), Text(extent = {{-94, 46}, {-10, 16}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {245, 245, 245}, horizontalAlignment = TextAlignment.Left, textString = "w"), Text(extent = {{-92, -14}, {-8, -44}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {245, 245, 245}, horizontalAlignment = TextAlignment.Left, textString = "a"), Text(extent = {{-150, -110}, {150, -140}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {245, 245, 245}, textString = "J=%J"), Bitmap(extent = {{-58, -42}, {98, 48}}, fileName = "modelica://Modelica/Resources/Images/Mechanics/Rotational/InverseInertia.png"), Text(extent = {{-90, -64}, {-6, -94}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {245, 245, 245}, horizontalAlignment = TextAlignment.Left, textString = "tau")}));
          end InverseInertia;

          model SpringDamper "Input/output block of a spring/damper model"
            extends Modelica.Blocks.Icons.Block;
            parameter SI.RotationalSpringConstant c = 1e4 "Spring constant";
            parameter SI.RotationalDampingConstant d = 1 "Damping constant";
            parameter SI.Angle phi_rel0 = 0 "Unstretched spring angle";
            Components.AngleToTorqueAdaptor angleToTorque1(use_a = false) annotation(
              Placement(transformation(extent = {{-30, -10}, {-10, 10}})));
            Modelica.Blocks.Interfaces.RealInput phi1(unit = "rad") "Angle of left flange of force element" annotation(
              Placement(transformation(extent = {{-140, 60}, {-100, 100}})));
            Modelica.Blocks.Interfaces.RealInput w1(unit = "rad/s") "Speed to left flange of force element" annotation(
              Placement(transformation(extent = {{-140, 10}, {-100, 50}})));
            Modelica.Blocks.Interfaces.RealOutput tau1(unit = "N.m") "Torque generated by the force element" annotation(
              Placement(transformation(extent = {{-100, -90}, {-120, -70}})));
            SpringDamperNoRelativeStates springDamper(c = c, d = d, phi_rel0 = phi_rel0) annotation(
              Placement(transformation(extent = {{-6, -10}, {14, 10}})));
            Modelica.Blocks.Interfaces.RealInput phi2(unit = "rad") "Angle of left flange of force element" annotation(
              Placement(transformation(extent = {{140, 60}, {100, 100}})));
            Modelica.Blocks.Interfaces.RealInput w2(unit = "rad/s") "Speed to left flange of force element" annotation(
              Placement(transformation(extent = {{140, 10}, {100, 50}})));
            Modelica.Blocks.Interfaces.RealOutput tau2(unit = "N.m") "Torque generated by the force element" annotation(
              Placement(transformation(extent = {{100, -90}, {120, -70}})));
            Components.AngleToTorqueAdaptor angleToTorque2(use_a = false) annotation(
              Placement(transformation(extent = {{38, -10}, {18, 10}})));
          equation
            connect(angleToTorque1.phi, phi1) annotation(
              Line(points = {{-24, 8}, {-84, 8}, {-84, 80}, {-120, 80}}, color = {0, 0, 127}, smooth = Smooth.None));
            connect(angleToTorque1.w, w1) annotation(
              Line(points = {{-24, 2.8}, {-90, 2.8}, {-90, 30}, {-120, 30}}, color = {0, 0, 127}, smooth = Smooth.None));
            connect(angleToTorque1.tau, tau1) annotation(
              Line(points = {{-23, -8}, {-70, -8}, {-70, -80}, {-110, -80}}, color = {0, 0, 127}, smooth = Smooth.None));
            connect(angleToTorque1.flange, springDamper.flange_a) annotation(
              Line(points = {{-18, 0}, {-6, 0}}, color = {0, 0, 0}, smooth = Smooth.None));
            connect(springDamper.flange_b, angleToTorque2.flange) annotation(
              Line(points = {{14, 0}, {26, 0}}, color = {0, 0, 0}, smooth = Smooth.None));
            connect(phi2, angleToTorque2.phi) annotation(
              Line(points = {{120, 80}, {60, 80}, {60, 8}, {32, 8}}, color = {0, 0, 127}, smooth = Smooth.None));
            connect(w2, angleToTorque2.w) annotation(
              Line(points = {{120, 30}, {70, 30}, {70, 2.8}, {32, 2.8}}, color = {0, 0, 127}, smooth = Smooth.None));
            connect(angleToTorque2.tau, tau2) annotation(
              Line(points = {{31, -8}, {60, -8}, {60, -80}, {110, -80}}, color = {0, 0, 127}, smooth = Smooth.None));
            annotation(
              Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-48, -36}, {48, -68}}, lineColor = {135, 135, 135}, fillColor = {235, 245, 255}, fillPattern = FillPattern.Solid, textString = "to FMU"), Text(extent = {{-94, 96}, {-10, 66}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {245, 245, 245}, horizontalAlignment = TextAlignment.Left, textString = "phi1"), Text(extent = {{-150, -118}, {150, -148}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {245, 245, 245}, textString = "c=%c
d=%d"), Bitmap(extent = {{-72, -44}, {84, 46}}, fileName = "modelica://Modelica/Resources/Images/Mechanics/Rotational/SpringDamper.png"), Text(extent = {{12, 96}, {96, 66}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {245, 245, 245}, horizontalAlignment = TextAlignment.Right, textString = "phi2"), Text(extent = {{12, 48}, {96, 18}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {245, 245, 245}, horizontalAlignment = TextAlignment.Right, textString = "w2"), Text(extent = {{10, -60}, {94, -90}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {245, 245, 245}, horizontalAlignment = TextAlignment.Right, textString = "tau2"), Text(extent = {{-94, 46}, {-10, 16}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {245, 245, 245}, horizontalAlignment = TextAlignment.Left, textString = "w1"), Text(extent = {{-90, -64}, {-6, -94}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {245, 245, 245}, horizontalAlignment = TextAlignment.Left, textString = "tau1")}));
          end SpringDamper;

          model Spring "Input/output block of a spring model"
            extends Modelica.Blocks.Icons.Block;
            parameter SI.RotationalSpringConstant c = 1e4 "Spring constant";
            parameter SI.Angle phi_rel0 = 0 "Unstretched spring angle";
            Components.AngleToTorqueAdaptor angleToTorque1(use_a = false, use_w = false) annotation(
              Placement(transformation(extent = {{-30, -10}, {-10, 10}})));
            Modelica.Blocks.Interfaces.RealInput phi1(unit = "rad") "Angle of left flange of force element" annotation(
              Placement(transformation(extent = {{-140, 60}, {-100, 100}})));
            Modelica.Blocks.Interfaces.RealOutput tau1(unit = "N.m") "Torque generated by the force element" annotation(
              Placement(transformation(extent = {{-100, -90}, {-120, -70}})));
            Components.Spring spring(c = c, phi_rel0 = phi_rel0) annotation(
              Placement(transformation(extent = {{-6, -10}, {14, 10}})));
            Modelica.Blocks.Interfaces.RealInput phi2(unit = "rad") "Angle of left flange of force element" annotation(
              Placement(transformation(extent = {{140, 60}, {100, 100}})));
            Modelica.Blocks.Interfaces.RealOutput tau2(unit = "N.m") "Torque generated by the force element" annotation(
              Placement(transformation(extent = {{100, -90}, {120, -70}})));
            Components.AngleToTorqueAdaptor angleToTorque2(use_a = false, use_w = false) annotation(
              Placement(transformation(extent = {{38, -10}, {18, 10}})));
          equation
            connect(angleToTorque1.phi, phi1) annotation(
              Line(points = {{-24, 8}, {-84, 8}, {-84, 80}, {-120, 80}}, color = {0, 0, 127}, smooth = Smooth.None));
            connect(angleToTorque1.tau, tau1) annotation(
              Line(points = {{-23, -8}, {-70, -8}, {-70, -80}, {-110, -80}}, color = {0, 0, 127}, smooth = Smooth.None));
            connect(angleToTorque1.flange, spring.flange_a) annotation(
              Line(points = {{-18, 0}, {-6, 0}}, color = {0, 0, 0}, smooth = Smooth.None));
            connect(spring.flange_b, angleToTorque2.flange) annotation(
              Line(points = {{14, 0}, {26, 0}}, color = {0, 0, 0}, smooth = Smooth.None));
            connect(phi2, angleToTorque2.phi) annotation(
              Line(points = {{120, 80}, {60, 80}, {60, 8}, {32, 8}}, color = {0, 0, 127}, smooth = Smooth.None));
            connect(angleToTorque2.tau, tau2) annotation(
              Line(points = {{31, -8}, {60, -8}, {60, -80}, {110, -80}}, color = {0, 0, 127}, smooth = Smooth.None));
            annotation(
              Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-48, -36}, {48, -68}}, lineColor = {135, 135, 135}, fillColor = {235, 245, 255}, fillPattern = FillPattern.Solid, textString = "to FMU"), Text(extent = {{-94, 96}, {-10, 66}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {245, 245, 245}, horizontalAlignment = TextAlignment.Left, textString = "phi1"), Text(extent = {{-150, -114}, {150, -144}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {245, 245, 245}, textString = "c=%c"), Bitmap(extent = {{-88, -36}, {92, 56}}, fileName = "modelica://Modelica/Resources/Images/Mechanics/Rotational/Spring.png"), Text(extent = {{12, 96}, {96, 66}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {245, 245, 245}, horizontalAlignment = TextAlignment.Right, textString = "phi2"), Text(extent = {{10, -60}, {94, -90}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {245, 245, 245}, horizontalAlignment = TextAlignment.Right, textString = "tau2"), Text(extent = {{-90, -64}, {-6, -94}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {245, 245, 245}, horizontalAlignment = TextAlignment.Left, textString = "tau1")}));
          end Spring;

          model SpringDamperNoRelativeStates "Linear 1D rotational spring and damper in parallel (phi and w are not used as states)"
            parameter SI.RotationalSpringConstant c(final min = 0, start = 1.0e5) "Spring constant";
            parameter SI.RotationalDampingConstant d(final min = 0, start = 0) "Damping constant";
            parameter SI.Angle phi_rel0 = 0 "Unstretched spring angle";
            Modelica.SIunits.AngularVelocity w_rel(start = 0) "Relative angular velocity (= der(phi_rel))";
            extends Modelica.Mechanics.Rotational.Interfaces.PartialCompliant;
            extends Modelica.Thermal.HeatTransfer.Interfaces.PartialElementaryConditionalHeatPortWithoutT;
          protected
            Modelica.SIunits.Torque tau_c "Spring torque";
            Modelica.SIunits.Torque tau_d "Damping torque";
          equation
            w_rel = der(phi_rel);
            tau_c = c * (phi_rel - phi_rel0);
            tau_d = d * w_rel;
            tau = tau_c + tau_d;
            lossPower = tau_d * w_rel;
            annotation(
              Documentation(info = "<html>
<p>
A <b>spring</b> and <b>damper</b> element <b>connected in parallel</b>.
The component can be
connected either between two inertias/gears to describe the shaft elasticity
and damping, or between an inertia/gear and the housing (component Fixed),
to describe a coupling of the element with the housing via a spring/damper.
</p>

<p>
This is the same element as <a href=\"modelica://Modelica.Mechanics.Rotational.Components.SpringDamper\">Rotational.Components.SpringDamper</a>
but with the only difference, that the relative quantities are not used as states. If the relative
states are potentially used as states, \"a_rel = der(w_rel)\" is present, and then exporting this model
as FMU requires to also have the accelerations in the flanges as inputs, which is usually not
desired for a force element.
</p>
</html>"),
              Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-80, 40}, {-60, 40}, {-45, 10}, {-15, 70}, {15, 10}, {45, 70}, {60, 40}, {80, 40}}, color = {0, 0, 0}), Line(points = {{-80, 40}, {-80, -40}}, color = {0, 0, 0}), Line(points = {{-80, -40}, {-50, -40}}, color = {0, 0, 0}), Rectangle(extent = {{-50, -10}, {40, -70}}, lineColor = {0, 0, 0}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-50, -10}, {70, -10}}, color = {0, 0, 0}), Line(points = {{-50, -70}, {70, -70}}, color = {0, 0, 0}), Line(points = {{40, -40}, {80, -40}}, color = {0, 0, 0}), Line(points = {{80, 40}, {80, -40}}, color = {0, 0, 0}), Line(points = {{-90, 0}, {-80, 0}}, color = {0, 0, 0}), Line(points = {{80, 0}, {90, 0}}, color = {0, 0, 0}), Text(origin = {0, -9}, extent = {{-150, -144}, {150, -104}}, lineColor = {0, 0, 0}, textString = "d=%d"), Text(extent = {{-190, 110}, {190, 70}}, lineColor = {0, 0, 255}, textString = "%name"), Text(origin = {0, -7}, extent = {{-150, -108}, {150, -68}}, lineColor = {0, 0, 0}, textString = "c=%c"), Line(visible = useHeatPort, points = {{-100, -100}, {-100, -55}, {-5, -55}}, color = {191, 0, 0}, pattern = LinePattern.Dot, smooth = Smooth.None)}),
              Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-80, 32}, {-58, 32}, {-43, 2}, {-13, 62}, {17, 2}, {47, 62}, {62, 32}, {80, 32}}, color = {0, 0, 0}, thickness = 0.5), Line(points = {{-68, 32}, {-68, 97}}, color = {128, 128, 128}), Line(points = {{72, 32}, {72, 97}}, color = {128, 128, 128}), Line(points = {{-68, 92}, {72, 92}}, color = {128, 128, 128}), Polygon(points = {{62, 95}, {72, 92}, {62, 89}, {62, 95}}, lineColor = {128, 128, 128}, fillColor = {128, 128, 128}, fillPattern = FillPattern.Solid), Text(extent = {{-44, 79}, {29, 91}}, lineColor = {0, 0, 255}, textString = "phi_rel"), Rectangle(extent = {{-50, -20}, {40, -80}}, lineColor = {0, 0, 0}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-50, -80}, {68, -80}}, color = {0, 0, 0}), Line(points = {{-50, -20}, {68, -20}}, color = {0, 0, 0}), Line(points = {{40, -50}, {80, -50}}, color = {0, 0, 0}), Line(points = {{-80, -50}, {-50, -50}}, color = {0, 0, 0}), Line(points = {{-80, 32}, {-80, -50}}, color = {0, 0, 0}), Line(points = {{80, 32}, {80, -50}}, color = {0, 0, 0}), Line(points = {{-96, 0}, {-80, 0}}, color = {0, 0, 0}), Line(points = {{96, 0}, {80, 0}}, color = {0, 0, 0})}));
          end SpringDamperNoRelativeStates;
        end Utilities;
        annotation(
          Documentation(info = "<html>
<p>
This package contains example models to demonstrate the usage of the
Modelica.Mechanics.Rotational package. Open the models and
simulate them according to the provided description in the models.
</p>

</html>"));
      end Examples;

      package Components "Components for 1D rotational mechanical drive trains"
        extends Modelica.Icons.Package;

        model Fixed "Flange fixed in housing at a given angle"
          parameter SI.Angle phi0 = 0 "Fixed offset angle of housing";
          Interfaces.Flange_b flange "(right) flange fixed in housing" annotation(
            Placement(transformation(extent = {{10, -10}, {-10, 10}}, rotation = 0)));
        equation
          flange.phi = phi0;
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-150, -90}, {150, -130}}, lineColor = {0, 0, 255}, textString = "%name"), Line(points = {{-80, -40}, {80, -40}}, color = {0, 0, 0}), Line(points = {{80, -40}, {40, -80}}, color = {0, 0, 0}), Line(points = {{40, -40}, {0, -80}}, color = {0, 0, 0}), Line(points = {{0, -40}, {-40, -80}}, color = {0, 0, 0}), Line(points = {{-40, -40}, {-80, -80}}, color = {0, 0, 0}), Line(points = {{0, -40}, {0, -10}}, color = {0, 0, 0})}),
            Documentation(info = "<html>
<p>
The <b>flange</b> of a 1D rotational mechanical system is <b>fixed</b>
at an angle phi0 in the <b>housing</b>. May be used:
</p>
<ul>
<li> to connect a compliant element, such as a spring or a damper,
     between an inertia or gearbox component and the housing.
<li> to fix a rigid element, such as an inertia, with a specific
     angle to the housing.
</ul>

</html>"),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-80, -40}, {80, -40}}, color = {0, 0, 0}), Line(points = {{80, -40}, {40, -80}}, color = {0, 0, 0}), Line(points = {{40, -40}, {0, -80}}, color = {0, 0, 0}), Line(points = {{0, -40}, {-40, -80}}, color = {0, 0, 0}), Line(points = {{-40, -40}, {-80, -80}}, color = {0, 0, 0}), Line(points = {{0, -40}, {0, -4}}, color = {0, 0, 0})}));
        end Fixed;

        model Inertia "1D-rotational component with inertia"
          Rotational.Interfaces.Flange_a flange_a "Left flange of shaft" annotation(
            Placement(transformation(extent = {{-110, -10}, {-90, 10}}, rotation = 0)));
          Rotational.Interfaces.Flange_b flange_b "Right flange of shaft" annotation(
            Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
          parameter SI.Inertia J(min = 0, start = 1) "Moment of inertia";
          parameter StateSelect stateSelect = StateSelect.default "Priority to use phi and w as states" annotation(
            HideResult = true,
            Dialog(tab = "Advanced"));
          SI.Angle phi(stateSelect = stateSelect) "Absolute rotation angle of component" annotation(
            Dialog(group = "Initialization", showStartAttribute = true));
          SI.AngularVelocity w(stateSelect = stateSelect) "Absolute angular velocity of component (= der(phi))" annotation(
            Dialog(group = "Initialization", showStartAttribute = true));
          SI.AngularAcceleration a "Absolute angular acceleration of component (= der(w))" annotation(
            Dialog(group = "Initialization", showStartAttribute = true));
        equation
          phi = flange_a.phi;
          phi = flange_b.phi;
          w = der(phi);
          a = der(w);
          J * a = flange_a.tau + flange_b.tau;
          annotation(
            Documentation(info = "<html>
<p>
Rotational component with <b>inertia</b> and two rigidly connected flanges.
</p>

</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100.0, -100.0}, {100.0, 100.0}}, initialScale = 0.1), graphics = {Rectangle(lineColor = {64, 64, 64}, fillColor = {192, 192, 192}, fillPattern = FillPattern.HorizontalCylinder, extent = {{-100.0, -10.0}, {-50.0, 10.0}}), Rectangle(lineColor = {64, 64, 64}, fillColor = {192, 192, 192}, fillPattern = FillPattern.HorizontalCylinder, extent = {{50.0, -10.0}, {100.0, 10.0}}), Line(points = {{-80.0, -25.0}, {-60.0, -25.0}}), Line(points = {{60.0, -25.0}, {80.0, -25.0}}), Line(points = {{-70.0, -25.0}, {-70.0, -70.0}}), Line(points = {{70.0, -25.0}, {70.0, -70.0}}), Line(points = {{-80.0, 25.0}, {-60.0, 25.0}}), Line(points = {{60.0, 25.0}, {80.0, 25.0}}), Line(points = {{-70.0, 45.0}, {-70.0, 25.0}}), Line(points = {{70.0, 45.0}, {70.0, 25.0}}), Line(points = {{-70.0, -70.0}, {70.0, -70.0}}), Rectangle(lineColor = {64, 64, 64}, fillColor = {255, 255, 255}, fillPattern = FillPattern.HorizontalCylinder, extent = {{-50.0, -50.0}, {50.0, 50.0}}, radius = 10.0), Text(lineColor = {0, 0, 255}, extent = {{-150.0, 60.0}, {150.0, 100.0}}, textString = "%name"), Text(extent = {{-150.0, -120.0}, {150.0, -80.0}}, textString = "J=%J"), Rectangle(lineColor = {64, 64, 64}, fillColor = {255, 255, 255}, extent = {{-50, -50}, {50, 50}}, radius = 10)}));
        end Inertia;

        model Disc "1-dim. rotational rigid component without inertia, where right flange is rotated by a fixed angle with respect to left flange"
          extends Rotational.Interfaces.PartialTwoFlanges;
          parameter SI.Angle deltaPhi = 0 "Fixed rotation of left flange with respect to right flange (= flange_b.phi - flange_a.phi)";
          SI.Angle phi "Absolute rotation angle of component";
        equation
          flange_a.phi = phi - deltaPhi / 2;
          flange_b.phi = phi + deltaPhi / 2;
          0 = flange_a.tau + flange_b.tau;
          annotation(
            Documentation(info = "<html>
<p>
Rotational component with two rigidly connected flanges without <b>inertia</b>.
The right flange is rotated by the fixed angle \"deltaPhi\" with respect to the left
flange.
</p>

</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100.0, -100.0}, {100.0, 100.0}}, initialScale = 0.1), graphics = {Rectangle(lineColor = {64, 64, 64}, fillColor = {192, 192, 192}, fillPattern = FillPattern.HorizontalCylinder, extent = {{-100.0, -10.0}, {99.0, 10.0}}), Rectangle(lineColor = {64, 64, 64}, fillColor = {192, 192, 192}, fillPattern = FillPattern.HorizontalCylinder, extent = {{-10.0, -50.0}, {10.0, 50.0}}), Text(lineColor = {0, 0, 255}, extent = {{-150.0, 60.0}, {150.0, 100.0}}, textString = "%name"), Rectangle(lineColor = {64, 64, 64}, fillColor = {192, 192, 192}, fillPattern = FillPattern.HorizontalCylinder, extent = {{-30.0, 10.0}, {-10.0, 50.0}}), Rectangle(lineColor = {64, 64, 64}, fillColor = {192, 192, 192}, fillPattern = FillPattern.HorizontalCylinder, extent = {{10.0, -50.0}, {30.0, -10.0}}), Text(extent = {{-160.0, -87.0}, {160.0, -62.0}}, textString = "deltaPhi = %deltaPhi")}));
        end Disc;

        model Spring "Linear 1D rotational spring"
          extends Modelica.Mechanics.Rotational.Interfaces.PartialCompliant;
          parameter SI.RotationalSpringConstant c(final min = 0, start = 1.0e5) "Spring constant";
          parameter SI.Angle phi_rel0 = 0 "Unstretched spring angle";
        equation
          tau = c * (phi_rel - phi_rel0);
          annotation(
            Documentation(info = "<html>
<p>
A <b>linear 1D rotational spring</b>. The component can be connected either
between two inertias/gears to describe the shaft elasticity, or between
a inertia/gear and the housing (component Fixed), to describe
a coupling of the element with the housing via a spring.
</p>

</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-150, 80}, {150, 40}}, textString = "%name", lineColor = {0, 0, 255}), Text(extent = {{-150, -40}, {150, -80}}, lineColor = {0, 0, 0}, textString = "c=%c"), Line(points = {{-100, 0}, {-58, 0}, {-43, -30}, {-13, 30}, {17, -30}, {47, 30}, {62, 0}, {100, 0}}, color = {0, 0, 0}, pattern = LinePattern.Solid, thickness = 0.25, arrow = {Arrow.None, Arrow.None})}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-68, 0}, {-68, 65}}, color = {128, 128, 128}), Line(points = {{72, 0}, {72, 65}}, color = {128, 128, 128}), Line(points = {{-68, 60}, {72, 60}}, color = {128, 128, 128}), Polygon(points = {{62, 63}, {72, 60}, {62, 57}, {62, 63}}, lineColor = {128, 128, 128}, fillColor = {128, 128, 128}, fillPattern = FillPattern.Solid), Text(extent = {{-22, 62}, {18, 87}}, lineColor = {0, 0, 255}, textString = "phi_rel"), Line(points = {{-96, 0}, {-60, 0}, {-42, -32}, {-12, 30}, {18, -30}, {48, 28}, {62, 0}, {96, 0}}, color = {0, 0, 255})}));
        end Spring;

        model Damper "Linear 1D rotational damper"
          extends Modelica.Mechanics.Rotational.Interfaces.PartialCompliantWithRelativeStates;
          parameter SI.RotationalDampingConstant d(final min = 0, start = 0) "Damping constant";
          extends Modelica.Thermal.HeatTransfer.Interfaces.PartialElementaryConditionalHeatPortWithoutT;
        equation
          tau = d * w_rel;
          lossPower = tau * w_rel;
          annotation(
            Documentation(info = "<html>
<p>
<b>Linear, velocity dependent damper</b> element. It can be either connected
between an inertia or gear and the housing (component Fixed), or
between two inertia/gear elements.
</p>

<p>
See also the discussion
<a href=\"modelica://Modelica.Mechanics.Rotational.UsersGuide.StateSelection\">State Selection</a>
in the User's Guide of the Rotational library.
</p>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-90, 0}, {-60, 0}}, color = {0, 0, 0}), Line(points = {{-60, -30}, {-60, 30}}, color = {0, 0, 0}), Line(points = {{-60, -30}, {60, -30}}, color = {0, 0, 0}), Line(points = {{-60, 30}, {60, 30}}, color = {0, 0, 0}), Rectangle(extent = {{-60, 30}, {30, -30}}, lineColor = {0, 0, 0}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{30, 0}, {90, 0}}, color = {0, 0, 0}), Text(extent = {{-150, 80}, {150, 40}}, textString = "%name", lineColor = {0, 0, 255}), Text(extent = {{-150, -50}, {150, -90}}, lineColor = {0, 0, 0}, textString = "d=%d"), Line(visible = useHeatPort, points = {{-100, -100}, {-100, -40}, {-20, -40}, {-20, 0}}, color = {191, 0, 0}, smooth = Smooth.None, pattern = LinePattern.Dot)}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-96, 0}, {-60, 0}}, color = {0, 0, 0}), Line(points = {{-60, -30}, {-60, 30}}, color = {0, 0, 0}), Line(points = {{-60, -30}, {60, -30}}, color = {0, 0, 0}), Line(points = {{-60, 30}, {60, 30}}, color = {0, 0, 0}), Rectangle(extent = {{-60, 30}, {30, -30}}, lineColor = {0, 0, 0}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{30, 0}, {96, 0}}, color = {0, 0, 0}), Line(points = {{-68, 0}, {-68, 65}}, color = {128, 128, 128}), Text(extent = {{-40, 66}, {33, 85}}, lineColor = {0, 0, 255}, textString = "phi_rel"), Line(points = {{-68, 60}, {72, 60}}, color = {128, 128, 128}), Line(points = {{72, 0}, {72, 65}}, color = {128, 128, 128}), Polygon(points = {{62, 63}, {72, 60}, {62, 57}, {62, 63}}, lineColor = {128, 128, 128}, fillColor = {128, 128, 128}, fillPattern = FillPattern.Solid)}));
        end Damper;

        model SpringDamper "Linear 1D rotational spring and damper in parallel"
          parameter SI.RotationalSpringConstant c(final min = 0, start = 1.0e5) "Spring constant";
          parameter SI.RotationalDampingConstant d(final min = 0, start = 0) "Damping constant";
          parameter SI.Angle phi_rel0 = 0 "Unstretched spring angle";
          extends Modelica.Mechanics.Rotational.Interfaces.PartialCompliantWithRelativeStates;
          extends Modelica.Thermal.HeatTransfer.Interfaces.PartialElementaryConditionalHeatPortWithoutT;
        protected
          Modelica.SIunits.Torque tau_c "Spring torque";
          Modelica.SIunits.Torque tau_d "Damping torque";
        equation
          tau_c = c * (phi_rel - phi_rel0);
          tau_d = d * w_rel;
          tau = tau_c + tau_d;
          lossPower = tau_d * w_rel;
          annotation(
            Documentation(info = "<html>
<p>
A <b>spring</b> and <b>damper</b> element <b>connected in parallel</b>.
The component can be
connected either between two inertias/gears to describe the shaft elasticity
and damping, or between an inertia/gear and the housing (component Fixed),
to describe a coupling of the element with the housing via a spring/damper.
</p>

<p>
See also the discussion
<a href=\"modelica://Modelica.Mechanics.Rotational.UsersGuide.StateSelection\">State Selection</a>
in the User's Guide of the Rotational library.
</p>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-80, 40}, {-60, 40}, {-45, 10}, {-15, 70}, {15, 10}, {45, 70}, {60, 40}, {80, 40}}, color = {0, 0, 0}), Line(points = {{-80, 40}, {-80, -40}}, color = {0, 0, 0}), Line(points = {{-80, -40}, {-50, -40}}, color = {0, 0, 0}), Rectangle(extent = {{-50, -10}, {40, -70}}, lineColor = {0, 0, 0}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-50, -10}, {70, -10}}, color = {0, 0, 0}), Line(points = {{-50, -70}, {70, -70}}, color = {0, 0, 0}), Line(points = {{40, -40}, {80, -40}}, color = {0, 0, 0}), Line(points = {{80, 40}, {80, -40}}, color = {0, 0, 0}), Line(points = {{-90, 0}, {-80, 0}}, color = {0, 0, 0}), Line(points = {{80, 0}, {90, 0}}, color = {0, 0, 0}), Text(origin = {0, -9}, extent = {{-150, -144}, {150, -104}}, lineColor = {0, 0, 0}, textString = "d=%d"), Text(extent = {{-190, 110}, {190, 70}}, lineColor = {0, 0, 255}, textString = "%name"), Text(origin = {0, -7}, extent = {{-150, -108}, {150, -68}}, lineColor = {0, 0, 0}, textString = "c=%c"), Line(visible = useHeatPort, points = {{-100, -100}, {-100, -55}, {-5, -55}}, color = {191, 0, 0}, pattern = LinePattern.Dot, smooth = Smooth.None)}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-80, 32}, {-58, 32}, {-43, 2}, {-13, 62}, {17, 2}, {47, 62}, {62, 32}, {80, 32}}, color = {0, 0, 0}, thickness = 0.5), Line(points = {{-68, 32}, {-68, 97}}, color = {128, 128, 128}), Line(points = {{72, 32}, {72, 97}}, color = {128, 128, 128}), Line(points = {{-68, 92}, {72, 92}}, color = {128, 128, 128}), Polygon(points = {{62, 95}, {72, 92}, {62, 89}, {62, 95}}, lineColor = {128, 128, 128}, fillColor = {128, 128, 128}, fillPattern = FillPattern.Solid), Text(extent = {{-44, 79}, {29, 91}}, lineColor = {0, 0, 255}, textString = "phi_rel"), Rectangle(extent = {{-50, -20}, {40, -80}}, lineColor = {0, 0, 0}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-50, -80}, {68, -80}}, color = {0, 0, 0}), Line(points = {{-50, -20}, {68, -20}}, color = {0, 0, 0}), Line(points = {{40, -50}, {80, -50}}, color = {0, 0, 0}), Line(points = {{-80, -50}, {-50, -50}}, color = {0, 0, 0}), Line(points = {{-80, 32}, {-80, -50}}, color = {0, 0, 0}), Line(points = {{80, 32}, {80, -50}}, color = {0, 0, 0}), Line(points = {{-96, 0}, {-80, 0}}, color = {0, 0, 0}), Line(points = {{96, 0}, {80, 0}}, color = {0, 0, 0})}));
        end SpringDamper;

        model ElastoBacklash "Backlash connected in series to linear spring and damper (backlash is modeled with elasticity)"
          parameter SI.RotationalSpringConstant c(final min = Modelica.Constants.small, start = 1.0e5) "Spring constant (c > 0 required)";
          parameter SI.RotationalDampingConstant d(final min = 0, start = 0) "Damping constant";
          parameter SI.Angle b(final min = 0) = 0 "Total backlash";
          parameter SI.Angle phi_rel0 = 0 "Unstretched spring angle";
          extends Modelica.Mechanics.Rotational.Interfaces.PartialCompliantWithRelativeStates;
          extends Modelica.Thermal.HeatTransfer.Interfaces.PartialElementaryConditionalHeatPortWithoutT;
        protected
          final parameter SI.Angle bMax = b / 2 "Backlash in range bMin <= phi_rel - phi_rel0 <= bMax";
          final parameter SI.Angle bMin = -bMax "Backlash in range bMin <= phi_rel - phi_rel0 <= bMax";
          SI.Torque tau_c;
          SI.Torque tau_d;
          SI.Angle phi_diff = phi_rel - phi_rel0;
          // A minimum backlash is defined in order to avoid an infinite
          // number of state events if backlash b is set to zero.
          constant SI.Angle bEps = 1e-10 "Minimum backlash";
        equation
          if initial() then
/* During initialization the characteristic is modified, in order that
        it is a strict monoton rising function. Otherwise, initialization might
        result in a singular system when the characteristic has to be
        inverted. The characteristic is modified in the range 1.5*bMin <= phi_rel - phi_rel0 <= 1.5 bMax,
        so that in this range a linear characteristic is present that approaches the original
        function continuously at its limits, e.g.,
          original:  tau(1.5*bMax) = c*(phi_diff - bMax)
                                   = c*(0.5*bMax)
          initial :  tau(1.5*bMax) = (c/3)*phi_diff
                                   = (c/3)*(3/2)*bMax
                                   = (c/2)*bMax
     */
            tau_c = if phi_diff > 1.5 * bMax then c * (phi_diff - bMax) else if phi_diff < 1.5 * bMin then c * (phi_diff - bMin) else c / 3 * phi_diff;
            tau_d = d * w_rel;
            tau = tau_c + tau_d;
            lossPower = tau_d * w_rel;
          else
/*
     if abs(b) <= bEps then
        tau_c = c*phi_diff;
        tau_d = d*w_rel;
        tau   = tau_c + tau_d;
     elseif phi_diff > bMax then
        tau_c = c*(phi_diff - bMax);
        tau_d = d*w_rel;
        tau   = smooth(0, noEvent(if tau_c + tau_d <= 0 then 0 else tau_c + min(tau_c,tau_d)));
     elseif phi_diff < bMin then
        tau_c = c*(phi_diff - bMin);
        tau_d = d*w_rel;
        tau   = smooth(0, noEvent(if tau_c + tau_d >= 0 then 0 else tau_c + max(tau_c,tau_d)));
     else
        tau_c = 0;
        tau_d = 0;
        tau   = 0;
     end if;

     This is written in the form below, in order that parameter "b"
     is not evaluated during translation (i.e., in the above form
     it cannot be changed anymore after translation).
   */
            tau_c = if abs(b) <= bEps then c * phi_diff else if phi_diff > bMax then c * (phi_diff - bMax) else if phi_diff < bMin then c * (phi_diff - bMin) else 0;
            tau_d = d * w_rel;
            tau = if abs(b) <= bEps then tau_c + tau_d else if phi_diff > bMax then smooth(0, noEvent(if tau_c + tau_d <= 0 then 0 else tau_c + min(tau_c, tau_d))) else if phi_diff < bMin then smooth(0, noEvent(if tau_c + tau_d >= 0 then 0 else tau_c + max(tau_c, tau_d))) else 0;
            lossPower = if abs(b) <= bEps then tau_d * w_rel else if phi_diff > bMax then smooth(0, noEvent(if tau_c + tau_d <= 0 then 0 else min(tau_c, tau_d) * w_rel)) else if phi_diff < bMin then smooth(0, noEvent(if tau_c + tau_d >= 0 then 0 else max(tau_c, tau_d) * w_rel)) else 0;
          end if;
          annotation(
            Documentation(info = "<html>
<p>
This element consists of a <b>backlash</b> element <b>connected in series</b>
to a <b>spring</b> and <b>damper</b> element which are <b>connected in parallel</b>.
The spring constant shall be non-zero, otherwise the component cannot be used.
</p>

<p>
In combination with components IdealGear, the ElastoBacklash model
can be used to model a gear box with backlash, elasticity and damping.
</p>

<p>
During initialization, the backlash characteristic is replaced by a continuous
approximation in the backlash region, in order to reduce problems during
initialization, especially for inverse models.
</p>

<p>
If the backlash b is smaller as 1e-10 rad (especially, if b=0),
then the backlash is ignored and the component reduces to a spring/damper
element in parallel.
</p>

<p>
In the backlash region (-b/2 &le; flange_b.phi - flange_a.phi - phi_rel0 &le; b/2) no torque
is exerted (flange_b.tau = 0). Outside of this region, contact is present and
the contact torque is basically computed with a linear
spring/damper characteristic:
</p>

<pre>
   desiredContactTorque = c*phi_contact + d*<b>der</b>(phi_contact)

            phi_contact = phi_rel - phi_rel0 - b/2 <b>if</b> phi_rel - phi_rel0 &gt;  b/2
                        = phi_rel - phi_rel0 + b/2 <b>if</b> phi_rel - phi_rel0 &lt; -b/2

            phi_rel     = flange_b.phi - flange_a.phi;
</pre>

<p>
This torque characteristic leads to the following difficulties:
</p>

<ol>
<li> If the damper torque becomes larger as the spring torque and with opposite sign,
     the contact torque would be \"pulling/sticking\" which is unphysical, since during
     contact only pushing torques can occur.</li>

<li> When contact occurs with a non-zero relative speed (which is the usual
     situation), the damping torque has a non-zero value and therefore the contact
     torque changes discontinuously at phi_rel = phi_rel0. Again, this is not physical
     because the torque can only change continuously. (Note, this component is not an
     idealized model where a steep characteristic is approximated by a discontinuity,
     but it shall model the steep characteristic.)</li>
</ol>

<p>
In the literature there are several proposals to fix problem (2). However, there
seems to be no proposal to avoid sticking. For this reason, the most simple
approach is used in the ElastoBacklash model, to fix both problems by slight changes
to the linear spring/damper characteristic:
</p>

<pre>
    // Torque characteristic when phi_rel > phi_rel0
    <b>if</b> phi_rel - phi_rel0 &lt; b/2 <b>then</b>
       tau_c = 0;          // spring torque
       tau_d = 0;          // damper torque
       flange_b.tau = 0;
    <b>else</b>
       tau_c = c*(phi_rel - phi_rel0);    // spring torque
       tau_d = d*<b>der</b>(phi_rel);            // damper torque
       flange_b.tau = <b>if</b> tau_c + tau_d &le; 0 <b>then</b> 0 <b>else</b> tau_c + <b>min</b>( tau_c, tau_d );
    <b>end if</b>;
</pre>

<p>
Note, when sticking would occur (tau_c + tau_d &le; 0), then the contact torque
is explicitly set to zero. The \"min(tau_c, tau_d)\" part in the if-expression,
limits the damping torque when it is pushing. This means that at the start of
the contact (phi_rel - phi_rel0 = b/2), the damping torque is zero and is continuous.
The effect of both modifications is that the absolute value of the damping torque
is always limited by the absolute value of the spring torque: |tau_d| &le; |tau_c|.
</p>

<p>
In the next figure, a typical simulation with the ElastoBacklash model is shown
(<a href=\"modelica://Modelica.Mechanics.Rotational.Examples.Backlash\">Examples.Backlash</a>)
where the different effects are visualized:
</p>

<ol>
<li> Curve 1 (elastoBacklash1.tau) is the unmodified contact torque, i.e., the linear spring/damper
     characteristic. A pulling/sticking torque is present at the end of the contact.</li>
<li> Curve 2 (elastoBacklash2.tau) is the contact torque, where the torque is explicitly set to
     zero when pulling/sticking occurs. The contact torque is discontinuous at begin of
     contact.</li>
<li> Curve 3 (elastoBacklash3.tau) is the ElastoBacklash model of this library. No discontinuity and no
     pulling/sticking occurs.</li>
</ol>

<p>
<img src=\"modelica://Modelica/Resources/Images/Mechanics/Rotational/elastoBacklash1.png\">
</p>

<p>
See also the discussion
<a href=\"modelica://Modelica.Mechanics.Rotational.UsersGuide.StateSelection\">State Selection</a>
in the User's Guide of the Rotational library.
</p>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-80, 32}, {-58, 32}, {-48, 0}, {-34, 61}, {-20, 0}, {-8, 60}, {0, 30}, {20, 30}}, color = {0, 0, 0}, pattern = LinePattern.Solid, thickness = 0.25, arrow = {Arrow.None, Arrow.None}), Rectangle(extent = {{-60, -10}, {-10, -50}}, lineColor = {0, 0, 0}, pattern = LinePattern.Solid, lineThickness = 0.25, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-60, -50}, {0, -50}}, color = {0, 0, 0}, pattern = LinePattern.Solid, thickness = 0.25, arrow = {Arrow.None, Arrow.None}), Line(points = {{-60, -10}, {0, -10}}, color = {0, 0, 0}, pattern = LinePattern.Solid, thickness = 0.25, arrow = {Arrow.None, Arrow.None}), Line(points = {{-10, -30}, {20, -30}}, color = {0, 0, 0}, pattern = LinePattern.Solid, thickness = 0.25, arrow = {Arrow.None, Arrow.None}), Line(points = {{-80, -30}, {-60, -30}}, color = {0, 0, 0}, pattern = LinePattern.Solid, thickness = 0.25, arrow = {Arrow.None, Arrow.None}), Line(points = {{-80, 32}, {-80, -30}}, color = {0, 0, 0}, pattern = LinePattern.Solid, thickness = 0.25, arrow = {Arrow.None, Arrow.None}), Line(points = {{20, 30}, {20, -30}}, color = {0, 0, 0}, pattern = LinePattern.Solid, thickness = 0.25, arrow = {Arrow.None, Arrow.None}), Line(points = {{-90, 0}, {-80, 0}}, color = {0, 0, 0}, pattern = LinePattern.Solid, thickness = 0.25, arrow = {Arrow.None, Arrow.None}), Line(points = {{90, 0}, {80, 0}}, color = {0, 0, 0}, pattern = LinePattern.Solid, thickness = 0.25, arrow = {Arrow.None, Arrow.None}), Line(points = {{20, 0}, {60, 0}, {60, -30}}, color = {0, 0, 0}, pattern = LinePattern.Solid, thickness = 0.25, arrow = {Arrow.None, Arrow.None}), Line(points = {{40, -12}, {40, -40}, {80, -40}, {80, 0}}, color = {0, 0, 0}, pattern = LinePattern.Solid, thickness = 0.25, arrow = {Arrow.None, Arrow.None}), Text(extent = {{-150, -130}, {150, -90}}, lineColor = {0, 0, 0}, textString = "b=%b"), Text(extent = {{-150, 100}, {150, 60}}, lineColor = {0, 0, 255}, textString = "%name"), Text(extent = {{-152, -92}, {148, -52}}, lineColor = {0, 0, 0}, textString = "c=%c"), Line(visible = useHeatPort, points = {{-100, -100}, {-100, -43}, {-34, -43}}, color = {191, 0, 0}, pattern = LinePattern.Dot, smooth = Smooth.None)}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-80, 32}, {-58, 32}, {-48, 0}, {-34, 60}, {-20, 0}, {-8, 60}, {0, 30}, {20, 30}}, color = {0, 0, 0}, thickness = 0.5), Line(points = {{-68, 32}, {-68, 97}}, color = {128, 128, 128}), Line(points = {{80, 0}, {80, 96}}, color = {128, 128, 128}), Line(points = {{-68, 92}, {72, 92}}, color = {128, 128, 128}), Polygon(points = {{70, 95}, {80, 92}, {70, 89}, {70, 95}}, lineColor = {128, 128, 128}, fillColor = {128, 128, 128}, fillPattern = FillPattern.Solid), Text(extent = {{-34, 77}, {40, 90}}, lineColor = {128, 128, 128}, textString = "phi_rel"), Rectangle(extent = {{-60, -20}, {-10, -80}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-52, -80}, {0, -80}}, color = {0, 0, 0}, thickness = 0.5), Line(points = {{-52, -20}, {0, -20}}, color = {0, 0, 0}, thickness = 0.5), Line(points = {{-10, -50}, {20, -50}}, color = {0, 0, 0}, thickness = 0.5), Line(points = {{-80, -50}, {-60, -50}}, color = {0, 0, 0}, thickness = 0.5), Line(points = {{-80, 32}, {-80, -50}}, color = {0, 0, 0}, thickness = 0.5), Line(points = {{20, 30}, {20, -50}}, color = {0, 0, 0}, thickness = 0.5), Line(points = {{-96, 0}, {-80, 0}}, color = {0, 0, 0}), Line(points = {{96, 0}, {80, 0}}, color = {0, 0, 0}, thickness = 0.5), Line(points = {{20, 0}, {60, 0}, {60, -30}}, color = {0, 0, 0}, thickness = 0.5), Line(points = {{40, -12}, {40, -40}, {80, -40}, {80, 0}}, color = {0, 0, 0}, thickness = 0.5), Line(points = {{30, 0}, {30, 64}}, color = {128, 128, 128}), Line(points = {{30, 60}, {80, 60}}, color = {128, 128, 128}), Polygon(points = {{70, 63}, {80, 60}, {70, 57}, {70, 63}}, lineColor = {128, 128, 128}, fillColor = {128, 128, 128}, fillPattern = FillPattern.Solid), Text(extent = {{39, 60}, {68, 46}}, lineColor = {160, 160, 164}, textString = "b")}));
        end ElastoBacklash;

        model BearingFriction "Coulomb friction in bearings"
          extends Modelica.Mechanics.Rotational.Interfaces.PartialElementaryTwoFlangesAndSupport2;
          parameter Real tau_pos[:, 2] = [0, 1] "[w,tau] Positive sliding friction characteristic (w>=0)";
          parameter Real peak(final min = 1) = 1 "peak*tau_pos[1,2] = Maximum friction torque for w==0";
          extends Rotational.Interfaces.PartialFriction;
          extends Modelica.Thermal.HeatTransfer.Interfaces.PartialElementaryConditionalHeatPortWithoutT;
          SI.Angle phi "Angle between shaft flanges (flange_a, flange_b) and support";
          SI.Torque tau "Friction torque";
          SI.AngularVelocity w "Absolute angular velocity of flange_a and flange_b";
          SI.AngularAcceleration a "Absolute angular acceleration of flange_a and flange_b";
        equation
// Constant auxiliary variables
          tau0 = Modelica.Math.tempInterpol1(0, tau_pos, 2);
          tau0_max = peak * tau0;
          free = false;
          phi = flange_a.phi - phi_support;
          flange_b.phi = flange_a.phi;
// Angular velocity and angular acceleration of flanges
          w = der(phi);
          a = der(w);
          w_relfric = w;
          a_relfric = a;
// Friction torque
          flange_a.tau + flange_b.tau - tau = 0;
// Friction torque
          tau = if locked then sa * unitTorque else if startForward then Modelica.Math.tempInterpol1(w, tau_pos, 2) else if startBackward then -Modelica.Math.tempInterpol1(-w, tau_pos, 2) else if pre(mode) == Forward then Modelica.Math.tempInterpol1(w, tau_pos, 2) else -Modelica.Math.tempInterpol1(-w, tau_pos, 2);
          lossPower = tau * w_relfric;
          annotation(
            Documentation(info = "<html>
<p>
This element describes <b>Coulomb friction</b> in <b>bearings</b>,
i.e., a frictional torque acting between a flange and the housing.
The positive sliding friction torque \"tau\" has to be defined
by table \"tau_pos\" as function of the absolute angular velocity \"w\".
E.g.
</p>
<pre>
       w | tau
      ---+-----
       0 |   0
       1 |   2
       2 |   5
       3 |   8
</pre>
<p>
gives the following table:
</p>
<pre>
   tau_pos = [0, 0; 1, 2; 2, 5; 3, 8];
</pre>
<p>
Currently, only linear interpolation in the table is supported.
Outside of the table, extrapolation through the last
two table entries is used. It is assumed that the negative
sliding friction force has the same characteristic with negative
values. Friction is modelled in the following way:
</p>
<p>
When the absolute angular velocity \"w\" is not zero, the friction torque
is a function of w and of a constant normal force. This dependency
is defined via table tau_pos and can be determined by measurements,
e.g., by driving the gear with constant velocity and measuring the
needed motor torque (= friction torque).
</p>
<p>
When the absolute angular velocity becomes zero, the elements
connected by the friction element become stuck, i.e., the absolute
angle remains constant. In this phase the friction torque is
calculated from a torque balance due to the requirement, that
the absolute acceleration shall be zero.  The elements begin
to slide when the friction torque exceeds a threshold value,
called the maximum static friction torque, computed via:
</p>
<pre>
   maximum_static_friction = <b>peak</b> * sliding_friction(w=0)  (<b>peak</b> >= 1)
</pre>
<p>
This procedure is implemented in a \"clean\" way by state events and
leads to continuous/discrete systems of equations if friction elements
are dynamically coupled which have to be solved by appropriate
numerical methods. The method is described in:
</p>
<dl>
<dt>Otter M., Elmqvist H., and Mattsson S.E. (1999):
<dd><b>Hybrid Modeling in Modelica based on the Synchronous
    Data Flow Principle</b>. CACSD'99, Aug. 22.-26, Hawaii.
</dl>
<p>
More precise friction models take into account the elasticity of the
material when the two elements are \"stuck\", as well as other effects,
like hysteresis. This has the advantage that the friction element can
be completely described by a differential equation without events. The
drawback is that the system becomes stiff (about 10-20 times slower
simulation) and that more material constants have to be supplied which
requires more sophisticated identification. For more details, see the
following references, especially (Armstrong and Canudas de Witt 1996):
</p>
<dl>
<dt>Armstrong B. (1991):</dt>
<dd><b>Control of Machines with Friction</b>. Kluwer Academic
    Press, Boston MA.<br></dd>
<dt>Armstrong B., and Canudas de Wit C. (1996):</dt>
<dd><b>Friction Modeling and Compensation.</b>
    The Control Handbook, edited by W.S.Levine, CRC Press,
    pp. 1369-1382.<br></dd>
<dt>Canudas de Wit C., Olsson H., Astroem K.J., and Lischinsky P. (1995):</dt>
<dd><b>A new model for control of systems with friction.</b>
    IEEE Transactions on Automatic Control, Vol. 40, No. 3, pp. 419-425.</dd>
</dl>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100.0, -100.0}, {100.0, 100.0}}, initialScale = 0.1), graphics = {Rectangle(lineColor = {64, 64, 64}, fillColor = {192, 192, 192}, fillPattern = FillPattern.HorizontalCylinder, extent = {{-100.0, -10.0}, {100.0, 10.0}}), Rectangle(extent = {{-60.0, -60.0}, {60.0, -10.0}}), Rectangle(fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid, extent = {{-60.0, -25.0}, {60.0, -10.0}}), Rectangle(fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid, extent = {{-60.0, -61.0}, {60.0, -45.0}}), Rectangle(fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, extent = {{-50.0, -50.0}, {50.0, -18.0}}), Polygon(fillColor = {160, 160, 164}, fillPattern = FillPattern.Solid, points = {{60.0, -60.0}, {60.0, -70.0}, {75.0, -70.0}, {75.0, -80.0}, {-75.0, -80.0}, {-75.0, -70.0}, {-60.0, -70.0}, {-60.0, -60.0}, {60.0, -60.0}}), Line(points = {{-75.0, -10.0}, {-75.0, -70.0}}), Line(points = {{75.0, -10.0}, {75.0, -70.0}}), Rectangle(extent = {{-60.0, 10.0}, {60.0, 60.0}}), Rectangle(fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid, extent = {{-60.0, 45.0}, {60.0, 60.0}}), Rectangle(fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid, extent = {{-60.0, 10.0}, {60.0, 25.0}}), Rectangle(fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, extent = {{-50.0, 19.0}, {50.0, 51.0}}), Line(points = {{-75.0, 70.0}, {-75.0, 10.0}}), Polygon(fillColor = {160, 160, 164}, fillPattern = FillPattern.Solid, points = {{60.0, 60.0}, {60.0, 70.0}, {75.0, 70.0}, {75.0, 80.0}, {-75.0, 80.0}, {-75.0, 70.0}, {-60.0, 70.0}, {-60.0, 60.0}, {60.0, 60.0}}), Line(points = {{75.0, 70.0}, {75.0, 10.0}}), Text(lineColor = {0, 0, 255}, extent = {{-150.0, 90.0}, {150.0, 130.0}}, textString = "%name"), Line(points = {{0.0, -80.0}, {0.0, -100.0}}), Line(visible = useHeatPort, points = {{-100.0, -100.0}, {-100.0, -35.0}, {2.0, -35.0}}, color = {191, 0, 0}, pattern = LinePattern.Dot)}));
        end BearingFriction;

        model Brake "Brake based on Coulomb friction"
          extends Modelica.Mechanics.Rotational.Interfaces.PartialElementaryTwoFlangesAndSupport2;
          parameter Real mue_pos[:, 2] = [0, 0.5] "[w,mue] positive sliding friction coefficient (w_rel>=0)";
          parameter Real peak(final min = 1) = 1 "peak*mue_pos[1,2] = maximum value of mue for w_rel==0";
          parameter Real cgeo(final min = 0) = 1 "Geometry constant containing friction distribution assumption";
          parameter SI.Force fn_max(final min = 0, start = 1) "Maximum normal force";
          extends Rotational.Interfaces.PartialFriction;
          extends Modelica.Thermal.HeatTransfer.Interfaces.PartialElementaryConditionalHeatPortWithoutT;
          SI.Angle phi "Angle between shaft flanges (flange_a, flange_b) and support";
          SI.Torque tau "Brake friction torque";
          SI.AngularVelocity w "Absolute angular velocity of flange_a and flange_b";
          SI.AngularAcceleration a "Absolute angular acceleration of flange_a and flange_b";
          Real mue0 "Friction coefficient for w=0 and forward sliding";
          SI.Force fn "Normal force (=fn_max*f_normalized)";
          // Constant auxiliary variable
          Modelica.Blocks.Interfaces.RealInput f_normalized "Normalized force signal 0..1 (normal force = fn_max*f_normalized; brake is active if > 0)" annotation(
            Placement(transformation(origin = {0, 110}, extent = {{20, -20}, {-20, 20}}, rotation = 90)));
        equation
          mue0 = Modelica.Math.tempInterpol1(0, mue_pos, 2);
          phi = flange_a.phi - phi_support;
          flange_b.phi = flange_a.phi;
// Angular velocity and angular acceleration of flanges flange_a and flange_b
          w = der(phi);
          a = der(w);
          w_relfric = w;
          a_relfric = a;
// Friction torque, normal force and friction torque for w_rel=0
          flange_a.tau + flange_b.tau - tau = 0;
          fn = fn_max * f_normalized;
          tau0 = mue0 * cgeo * fn;
          tau0_max = peak * tau0;
          free = fn <= 0;
// Friction torque
          tau = if locked then sa * unitTorque else if free then 0 else cgeo * fn * (if startForward then Modelica.Math.tempInterpol1(w, mue_pos, 2) else if startBackward then -Modelica.Math.tempInterpol1(-w, mue_pos, 2) else if pre(mode) == Forward then Modelica.Math.tempInterpol1(w, mue_pos, 2) else -Modelica.Math.tempInterpol1(-w, mue_pos, 2));
          lossPower = tau * w_relfric;
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100.0, -100.0}, {100.0, 100.0}}, initialScale = 0.1), graphics = {Polygon(lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid, points = {{-37.0, -55.0}, {-37.0, -90.0}, {37.0, -90.0}, {37.0, -55.0}, {33.0, -55.0}, {33.0, -86.0}, {-33.0, -86.0}, {-33.0, -55.0}, {-37.0, -55.0}}), Rectangle(lineColor = {64, 64, 64}, fillColor = {192, 192, 192}, fillPattern = FillPattern.HorizontalCylinder, extent = {{-100.0, -10.0}, {-20.0, 10.0}}), Rectangle(lineColor = {64, 64, 64}, fillColor = {255, 255, 255}, fillPattern = FillPattern.HorizontalCylinder, extent = {{-20.0, -60.0}, {20.0, 60.0}}, radius = 10.0), Rectangle(lineColor = {64, 64, 64}, fillPattern = FillPattern.None, extent = {{-20.0, -60.0}, {20.0, 60.0}}, radius = 10.0), Rectangle(lineColor = {64, 64, 64}, fillColor = {192, 192, 192}, fillPattern = FillPattern.HorizontalCylinder, extent = {{20.0, -10.0}, {100.0, 10.0}}), Polygon(lineColor = {0, 0, 127}, fillColor = {0, 0, 127}, fillPattern = FillPattern.Solid, points = {{40.0, -40.0}, {70.0, -30.0}, {70.0, -50.0}, {40.0, -40.0}}), Rectangle(fillPattern = FillPattern.Solid, extent = {{30.0, -55.0}, {40.0, -25.0}}), Polygon(lineColor = {0, 0, 127}, fillColor = {0, 0, 127}, fillPattern = FillPattern.Solid, points = {{-40.0, -40.0}, {-70.0, -30.0}, {-70.0, -50.0}, {-40.0, -40.0}}), Rectangle(fillPattern = FillPattern.Solid, extent = {{-40.0, -55.0}, {-30.0, -25.0}}), Line(points = {{0.0, 90.0}, {80.0, 70.0}, {80.0, -40.0}, {70.0, -40.0}}, color = {0, 0, 127}), Line(points = {{0.0, 90.0}, {-80.0, 70.0}, {-80.0, -40.0}, {-70.0, -40.0}}, color = {0, 0, 127}), Text(lineColor = {0, 0, 255}, extent = {{-150.0, -180.0}, {150.0, -140.0}}, textString = "%name"), Line(visible = useHeatPort, points = {{-100.0, -98.0}, {-100.0, -70.0}, {0.0, -70.0}, {0.0, -40.0}}, color = {191, 0, 0}, pattern = LinePattern.Dot)}),
            Documentation(info = "<html>
<p>
This component models a <b>brake</b>, i.e., a component where a frictional
torque is acting between the housing and a flange and a controlled normal
force presses the flange to the housing in order to increase friction.
The normal force fn has to be provided as input signal f_normalized in a normalized form
(0 &le; f_normalized &le; 1),
fn = fn_max*f_normalized, where fn_max has to be provided as parameter.
Friction in the brake is modelled in the following way:
</p>
<p>
When the absolute angular velocity \"w\" is not zero, the friction torque
is a function of the velocity dependent friction coefficient  mue(w) , of
the normal force \"fn\", and of a geometry constant \"cgeo\" which takes into
account the geometry of the device and the assumptions on the friction
distributions:
</p>
<pre>
        frictional_torque = <b>cgeo</b> * <b>mue</b>(w) * <b>fn</b>
</pre>
<p>
   Typical values of coefficients of friction:
</p>
<pre>
      dry operation   :  <b>mue</b> = 0.2 .. 0.4
      operating in oil:  <b>mue</b> = 0.05 .. 0.1
</pre>
<p>
   When plates are pressed together, where  <b>ri</b>  is the inner radius,
   <b>ro</b> is the outer radius and <b>N</b> is the number of friction interfaces,
   the geometry constant is calculated in the following way under the
   assumption of a uniform rate of wear at the interfaces:
</p>
<pre>
         <b>cgeo</b> = <b>N</b>*(<b>r0</b> + <b>ri</b>)/2
</pre>
<p>
    The positive part of the friction characteristic <b>mue</b>(w),
    w >= 0, is defined via table mue_pos (first column = w,
    second column = mue). Currently, only linear interpolation in
    the table is supported.
</p>
<p>
   When the absolute angular velocity becomes zero, the elements
   connected by the friction element become stuck, i.e., the absolute
   angle remains constant. In this phase the friction torque is
   calculated from a torque balance due to the requirement, that
   the absolute acceleration shall be zero.  The elements begin
   to slide when the friction torque exceeds a threshold value,
   called the  maximum static friction torque, computed via:
</p>
<pre>
       frictional_torque = <b>peak</b> * <b>cgeo</b> * <b>mue</b>(w=0) * <b>fn</b>   (<b>peak</b> >= 1)
</pre>
<p>
This procedure is implemented in a \"clean\" way by state events and
leads to continuous/discrete systems of equations if friction elements
are dynamically coupled. The method is described in:
</p>
<dl>
<dt>Otter M., Elmqvist H., and Mattsson S.E. (1999):
<dd><b>Hybrid Modeling in Modelica based on the Synchronous
    Data Flow Principle</b>. CACSD'99, Aug. 22.-26, Hawaii.
</dl>
<p>
More precise friction models take into account the elasticity of the
material when the two elements are \"stuck\", as well as other effects,
like hysteresis. This has the advantage that the friction element can
be completely described by a differential equation without events. The
drawback is that the system becomes stiff (about 10-20 times slower
simulation) and that more material constants have to be supplied which
requires more sophisticated identification. For more details, see the
following references, especially (Armstrong and Canudas de Witt 1996):
</p>
<dl>
<dt>Armstrong B. (1991):</dt>
<dd><b>Control of Machines with Friction</b>. Kluwer Academic
    Press, Boston MA.<br></dd>
<dt>Armstrong B., and Canudas de Wit C. (1996):</dt>
<dd><b>Friction Modeling and Compensation.</b>
    The Control Handbook, edited by W.S.Levine, CRC Press,
    pp. 1369-1382.<br></dd>
<dt>Canudas de Wit C., Olsson H., Astroem K.J., and Lischinsky P. (1995):</dt>
<dd><b>A new model for control of systems with friction.</b>
    IEEE Transactions on Automatic Control, Vol. 40, No. 3, pp. 419-425.</dd>
</dl>

<p>
See also the discussion
<a href=\"modelica://Modelica.Mechanics.Rotational.UsersGuide.StateSelection\">State Selection</a>
in the User's Guide of the Rotational library.
</p>
</html>"));
        end Brake;

        model Clutch "Clutch based on Coulomb friction"
          extends Modelica.Mechanics.Rotational.Icons.Clutch;
          extends Modelica.Mechanics.Rotational.Interfaces.PartialCompliantWithRelativeStates;
          parameter Real mue_pos[:, 2] = [0, 0.5] "[w,mue] positive sliding friction coefficient (w_rel>=0)";
          parameter Real peak(final min = 1) = 1 "peak*mue_pos[1,2] = maximum value of mue for w_rel==0";
          parameter Real cgeo(final min = 0) = 1 "Geometry constant containing friction distribution assumption";
          parameter SI.Force fn_max(final min = 0, start = 1) "Maximum normal force";
          extends Rotational.Interfaces.PartialFriction;
          extends Modelica.Thermal.HeatTransfer.Interfaces.PartialElementaryConditionalHeatPortWithoutT;
          Real mue0 "Friction coefficient for w=0 and forward sliding";
          SI.Force fn "Normal force (fn=fn_max*f_normalized)";
          Modelica.Blocks.Interfaces.RealInput f_normalized "Normalized force signal 0..1 (normal force = fn_max*f_normalized; clutch is engaged if > 0)" annotation(
            Placement(transformation(origin = {0, 110}, extent = {{20, -20}, {-20, 20}}, rotation = 90)));
        equation
// Constant auxiliary variable
          mue0 = Modelica.Math.tempInterpol1(0, mue_pos, 2);
// Relative quantities
          w_relfric = w_rel;
          a_relfric = a_rel;
// Normal force and friction torque for w_rel=0
          fn = fn_max * f_normalized;
          free = fn <= 0;
          tau0 = mue0 * cgeo * fn;
          tau0_max = peak * tau0;
// Friction torque
          tau = if locked then sa * unitTorque else if free then 0 else cgeo * fn * (if startForward then Modelica.Math.tempInterpol1(w_rel, mue_pos, 2) else if startBackward then -Modelica.Math.tempInterpol1(-w_rel, mue_pos, 2) else if pre(mode) == Forward then Modelica.Math.tempInterpol1(w_rel, mue_pos, 2) else -Modelica.Math.tempInterpol1(-w_rel, mue_pos, 2));
          lossPower = tau * w_relfric;
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-150, -110}, {150, -70}}, textString = "%name", lineColor = {0, 0, 255}), Line(visible = useHeatPort, points = {{-100, -100}, {-100, -40}, {0, -40}}, color = {191, 0, 0}, pattern = LinePattern.Dot, smooth = Smooth.None)}),
            Documentation(info = "<html>
<p>
This component models a <b>clutch</b>, i.e., a component with
two flanges where friction is present between the two flanges
and these flanges are pressed together via a normal force.
The normal force fn has to be provided as input signal f_normalized in a normalized form
(0 &le; f_normalized &le; 1),
fn = fn_max*f_normalized, where fn_max has to be provided as parameter. Friction in the
clutch is modelled in the following way:
</p>
<p>
When the relative angular velocity is not zero, the friction torque is a
function of the velocity dependent friction coefficient  mue(w_rel) , of
the normal force \"fn\", and of a geometry constant \"cgeo\" which takes into
account the geometry of the device and the assumptions on the friction
distributions:
</p>
<pre>
        frictional_torque = <b>cgeo</b> * <b>mue</b>(w_rel) * <b>fn</b>
</pre>
<p>
   Typical values of coefficients of friction:
</p>
<pre>
      dry operation   :  <b>mue</b> = 0.2 .. 0.4
      operating in oil:  <b>mue</b> = 0.05 .. 0.1
</pre>
<p>
   When plates are pressed together, where  <b>ri</b>  is the inner radius,
   <b>ro</b> is the outer radius and <b>N</b> is the number of friction interfaces,
   the geometry constant is calculated in the following way under the
   assumption of a uniform rate of wear at the interfaces:
</p>
<pre>
         <b>cgeo</b> = <b>N</b>*(<b>r0</b> + <b>ri</b>)/2
</pre>
<p>
    The positive part of the friction characteristic <b>mue</b>(w_rel),
    w_rel >= 0, is defined via table mue_pos (first column = w_rel,
    second column = mue). Currently, only linear interpolation in
    the table is supported.
</p>
<p>
   When the relative angular velocity becomes zero, the elements
   connected by the friction element become stuck, i.e., the relative
   angle remains constant. In this phase the friction torque is
   calculated from a torque balance due to the requirement, that
   the relative acceleration shall be zero.  The elements begin
   to slide when the friction torque exceeds a threshold value,
   called the  maximum static friction torque, computed via:
</p>
<pre>
       frictional_torque = <b>peak</b> * <b>cgeo</b> * <b>mue</b>(w_rel=0) * <b>fn</b>   (<b>peak</b> >= 1)
</pre>
<p>
This procedure is implemented in a \"clean\" way by state events and
leads to continuous/discrete systems of equations if friction elements
are dynamically coupled. The method is described in:
</p>
<dl>
<dt>Otter M., Elmqvist H., and Mattsson S.E. (1999):
<dd><b>Hybrid Modeling in Modelica based on the Synchronous
    Data Flow Principle</b>. CACSD'99, Aug. 22.-26, Hawaii.
</dl>
<p>
More precise friction models take into account the elasticity of the
material when the two elements are \"stuck\", as well as other effects,
like hysteresis. This has the advantage that the friction element can
be completely described by a differential equation without events. The
drawback is that the system becomes stiff (about 10-20 times slower
simulation) and that more material constants have to be supplied which
requires more sophisticated identification. For more details, see the
following references, especially (Armstrong and Canudas de Witt 1996):
</p>
<dl>
<dt>Armstrong B. (1991):</dt>
<dd><b>Control of Machines with Friction</b>. Kluwer Academic
    Press, Boston MA.<br></dd>
<dt>Armstrong B., and Canudas de Wit C. (1996):</dt>
<dd><b>Friction Modeling and Compensation.</b>
    The Control Handbook, edited by W.S.Levine, CRC Press,
    pp. 1369-1382.<br></dd>
<dt>Canudas de Wit C., Olsson H., Astroem K.J., and Lischinsky P. (1995):</dt>
<dd><b>A new model for control of systems with friction.</b>
    IEEE Transactions on Automatic Control, Vol. 40, No. 3, pp. 419-425.</dd>
</dl>

<p>
See also the discussion
<a href=\"modelica://Modelica.Mechanics.Rotational.UsersGuide.StateSelection\">State Selection</a>
in the User's Guide of the Rotational library.
</p>
</html>"));
        end Clutch;

        model OneWayClutch "Series connection of freewheel and clutch"
          extends Modelica.Mechanics.Rotational.Icons.Clutch;
          extends Modelica.Mechanics.Rotational.Interfaces.PartialCompliantWithRelativeStates;
          parameter Real mue_pos[:, 2] = [0, 0.5] "[w,mue] positive sliding friction coefficient (w_rel>=0)";
          parameter Real peak(final min = 1) = 1 "peak*mue_pos[1,2] = maximum value of mue for w_rel==0";
          parameter Real cgeo(final min = 0) = 1 "Geometry constant containing friction distribution assumption";
          parameter SI.Force fn_max(final min = 0, start = 1) "Maximum normal force";
          parameter SI.AngularVelocity w_small = 1e10 "Relative angular velocity near to zero if jumps due to a reinit(..) of the velocity can occur (set to low value only if such impulses can occur)" annotation(
            Dialog(tab = "Advanced"));
          extends Modelica.Thermal.HeatTransfer.Interfaces.PartialElementaryConditionalHeatPortWithoutT;
          Real u "Normalized force input signal (0..1)";
          SI.Force fn "Normal force (fn=fn_max*inPort.signal)";
          Boolean startForward(start = false) "true, if w_rel=0 and start of forward sliding or w_rel > w_small";
          Boolean locked(start = false) "true, if w_rel=0 and not sliding";
          Boolean stuck(start = false) "w_rel=0 (locked or start forward sliding)";
        protected
          SI.Torque tau0 "Friction torque for w=0 and sliding";
          SI.Torque tau0_max "Maximum friction torque for w=0 and locked";
          Real mue0 "Friction coefficient for w=0 and sliding";
          Boolean free "true, if frictional element is not active";
          Real sa(final unit = "1") "Path parameter of tau = f(a_rel) Friction characteristic";
          constant Real eps0 = 1.0e-4 "Relative hysteresis epsilon";
          SI.Torque tau0_max_low "Lowest value for tau0_max";
          parameter Real peak2 = max([peak, 1 + eps0]);
          constant SI.AngularAcceleration unitAngularAcceleration = 1;
          constant SI.Torque unitTorque = 1;
        public
          Modelica.Blocks.Interfaces.RealInput f_normalized "Normalized force signal 0..1 (normal force = fn_max*f_normalized; clutch is engaged if > 0)" annotation(
            Placement(transformation(origin = {0, 110}, extent = {{20, -20}, {-20, 20}}, rotation = 90)));
        equation
// Constant auxiliary variable
          mue0 = Modelica.Math.tempInterpol1(0, mue_pos, 2);
          tau0_max_low = eps0 * mue0 * cgeo * fn_max;
// Normal force and friction torque for w_rel=0
          u = f_normalized;
          free = u <= 0;
          fn = if free then 0 else fn_max * u;
          tau0 = mue0 * cgeo * fn;
          tau0_max = if free then tau0_max_low else peak2 * tau0;
/* Friction characteristic
       (locked is introduced to help the Modelica translator determining
       the different structural configurations, if for each configuration
       special code shall be generated)
    */
          startForward = pre(stuck) and (sa > tau0_max / unitTorque or pre(startForward) and sa > tau0 / unitTorque or w_rel > w_small) or initial() and w_rel > 0;
          locked = pre(stuck) and not startForward;
// acceleration and friction torque
          a_rel = unitAngularAcceleration * (if locked then 0 else sa - tau0 / unitTorque);
          tau = if locked then sa * unitTorque else if free then 0 else cgeo * fn * Modelica.Math.tempInterpol1(w_rel, mue_pos, 2);
// Determine configuration
          stuck = locked or w_rel <= 0;
          lossPower = if stuck then 0 else tau * w_rel;
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-150, -110}, {150, -70}}, textString = "%name", lineColor = {0, 0, 255}), Polygon(points = {{-10, 30}, {50, 0}, {-10, -30}, {-10, 30}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Line(visible = useHeatPort, points = {{-100, -99}, {-100, -40}, {0, -40}}, color = {191, 0, 0}, pattern = LinePattern.Dot, smooth = Smooth.None)}),
            Documentation(info = "<html>
<p>
This component models a <b>one-way clutch</b>, i.e., a component with
two flanges where friction is present between the two flanges
and these flanges are pressed together via a normal force. These
flanges maybe sliding with respect to each other
Parallel connection of ClutchCombi and of FreeWheel.
                     The element is introduced to resolve the ambiguity
                     of the constraint torques of the elements.
<p>
A one-way-clutch is an element where a clutch is connected in parallel
to a free wheel. This special element is provided, because such
a parallel connection introduces an ambiguity into the model
(the constraint torques are not uniquely defined when both
elements are stuck) and this element resolves it by introducing
<b>one</b> constraint torque and not two.
</p>
<p>
Note, initial values have to be chosen for the model, such that the
relative speed of the one-way-clutch >= 0. Otherwise, the configuration
is physically not possible and an error occurs.
</p>
<p>
The normal force fn has to be provided as input signal f_normalized in a normalized form
(0 &le; f_normalized &le; 1),
fn = fn_max*f_normalized, where fn_max has to be provided as parameter. Friction in the
clutch is modelled in the following way:
</p>
<p>
When the relative angular velocity is positive, the friction torque is a
function of the velocity dependent friction coefficient  mue(w_rel) , of
the normal force \"fn\", and of a geometry constant \"cgeo\" which takes into
account the geometry of the device and the assumptions on the friction
distributions:
</p>
<pre>
        frictional_torque = <b>cgeo</b> * <b>mue</b>(w_rel) * <b>fn</b>
</pre>
<p>
   Typical values of coefficients of friction:
</p>
<pre>
      dry operation   :  <b>mue</b> = 0.2 .. 0.4
      operating in oil:  <b>mue</b> = 0.05 .. 0.1
</pre>
<p>
   When plates are pressed together, where  <b>ri</b>  is the inner radius,
   <b>ro</b> is the outer radius and <b>N</b> is the number of friction interfaces,
   the geometry constant is calculated in the following way under the
   assumption of a uniform rate of wear at the interfaces:
</p>
<pre>
         <b>cgeo</b> = <b>N</b>*(<b>r0</b> + <b>ri</b>)/2
</pre>
<p>
    The positive part of the friction characteristic <b>mue</b>(w_rel),
    w_rel >= 0, is defined via table mue_pos (first column = w_rel,
    second column = mue). Currently, only linear interpolation in
    the table is supported.
</p>
<p>
   When the relative angular velocity becomes zero, the elements
   connected by the friction element become stuck, i.e., the relative
   angle remains constant. In this phase the friction torque is
   calculated from a torque balance due to the requirement, that
   the relative acceleration shall be zero.  The elements begin
   to slide when the friction torque exceeds a threshold value,
   called the  maximum static friction torque, computed via:
</p>
<pre>
       frictional_torque = <b>peak</b> * <b>cgeo</b> * <b>mue</b>(w_rel=0) * <b>fn</b>   (<b>peak</b> >= 1)
</pre>
<p>
This procedure is implemented in a \"clean\" way by state events and
leads to continuous/discrete systems of equations if friction elements
are dynamically coupled. The method is described in:
</p>
<dl>
<dt>Otter M., Elmqvist H., and Mattsson S.E. (1999):</dt>
<dd><b>Hybrid Modeling in Modelica based on the Synchronous
    Data Flow Principle</b>. CACSD'99, Aug. 22.-26, Hawaii.</dd>
</dl>

<p>
See also the discussion
<a href=\"modelica://Modelica.Mechanics.Rotational.UsersGuide.StateSelection\">State Selection</a>
in the User's Guide of the Rotational library.
</p>
</html>"));
        end OneWayClutch;

        model IdealGear "Ideal gear without inertia"
          extends Modelica.Mechanics.Rotational.Icons.Gear;
          extends Modelica.Mechanics.Rotational.Interfaces.PartialElementaryTwoFlangesAndSupport2;
          parameter Real ratio(start = 1) "Transmission ratio (flange_a.phi/flange_b.phi)";
          Modelica.SIunits.Angle phi_a "Angle between left shaft flange and support";
          Modelica.SIunits.Angle phi_b "Angle between right shaft flange and support";
        equation
          phi_a = flange_a.phi - phi_support;
          phi_b = flange_b.phi - phi_support;
          phi_a = ratio * phi_b;
          0 = ratio * flange_a.tau + flange_b.tau;
          annotation(
            Documentation(info = "<html>
<p>
This element characterizes any type of gear box which is fixed in the
ground and which has one driving shaft and one driven shaft.
The gear is <b>ideal</b>, i.e., it does not have inertia, elasticity, damping
or backlash. If these effects have to be considered, the gear has to be
connected to other elements in an appropriate way.
</p>

</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-153, 145}, {147, 105}}, lineColor = {0, 0, 255}, textString = "%name"), Text(extent = {{-146, -49}, {154, -79}}, lineColor = {0, 0, 0}, textString = "ratio=%ratio")}));
        end IdealGear;

        model LossyGear "Gear with mesh efficiency and bearing friction (stuck/rolling possible)"
          extends Modelica.Mechanics.Rotational.Icons.Gear;
          extends Modelica.Mechanics.Rotational.Interfaces.PartialElementaryTwoFlangesAndSupport2;
          parameter Real ratio(start = 1) "Transmission ratio (flange_a.phi/flange_b.phi)";
          parameter Real lossTable[:, 5] = [0, 1, 1, 0, 0] "Array for mesh efficiencies and bearing friction depending on speed";
          extends Modelica.Thermal.HeatTransfer.Interfaces.PartialElementaryConditionalHeatPortWithoutT;
          Modelica.SIunits.Angle phi_a "Angle between left shaft flange and support";
          Modelica.SIunits.Angle phi_b "Angle between right shaft flange and support";
          Real sa(final unit = "1") "Path parameter for acceleration and torque loss";
          SI.AngularVelocity w_a "Angular velocity of flange_a with respect to support";
          SI.AngularAcceleration a_a "Angular acceleration of flange_a with respect to support";
          Real interpolation_result[1, 4] "Result of interpolation in lossTable (= [eta_mf1, eta_mf2, tau_bf1, tau_bf2])";
          Real eta_mf1(unit = "1") "Mesh efficiency in case that flange_a is driving";
          Real eta_mf2(unit = "1") "Mesh efficiency in case that flange_b is driving";
          SI.Torque tau_bf_a "Bearing friction torque on flange_a side";
          SI.Torque tau_eta "Torque that determines the driving side (= if forwardSliding then flange_a.tau-tau_bf_a else if backwardSliding then flange_a.tau+tau_bf_a else flange_a.tau)";
          SI.Torque tau_bf1 "Absolute resultant bearing friction torque with respect to flange_a in case that flange_a is driving (= |tau_bf_a*eta_mf1 + tau_bf_b/i|)";
          SI.Torque tau_bf2 "Absolute resultant bearing friction torque with respect to flange_a in case that flange_b is driving (= |tau_bf_a/eta_mf2 + tau_bf_b/i|)";
          SI.Torque quadrant1 "Torque loss if w_a > 0 and flange_a.tau >= 0";
          SI.Torque quadrant2 "Torque loss if w_a > 0 and flange_a.tau < 0";
          SI.Torque quadrant3 "Torque loss if w_a < 0 and flange_a.tau >= 0";
          SI.Torque quadrant4 "Torque loss if w_a < 0 and flange_a.tau < 0";
          // Resultant friction torques at quadrant values for angular velocities near zero
          SI.Torque quadrant1_p "Torque loss at w_a = 0+ to determine driving side (flange_a.tau >= 0)";
          SI.Torque quadrant2_p "Torque loss at w_a = 0+ to determine driving side (flange_a.tau < 0)";
          SI.Torque quadrant3_m "Torque loss at w_a = 0- to determine driving side (flange_a.tau >=0)";
          SI.Torque quadrant4_m "Torque loss at w_a = 0- to determine driving side (flange_a.tau < 0)";
          SI.Torque tauLoss "Torque loss due to friction in the gear teeth and in the bearings";
          SI.Torque tauLossMax "Torque loss for positive speed";
          SI.Torque tauLossMin "Torque loss for negative speed";
          SI.Torque tauLossMax_p "Torque loss for positive speed";
          SI.Torque tauLossMin_m "Torque loss for negative speed";
          Boolean tau_aPos(start = true) "Only for backwards compatibility (was previously: true, if torque of flange_a is not negative)";
          Boolean tau_etaPos(start = true) "true, if torque tau_eta is not negative";
          Boolean startForward(start = false) "true, if starting to roll forward";
          Boolean startBackward(start = false) "true, if starting to roll backward";
          Boolean locked(start = false) "true, if gear is locked";
          Boolean ideal "= true, if losses are neglected (that is lossTable = [0, 1, 1, 0, 0])";
          constant Integer Unknown = 3 "Value of mode is not known";
          constant Integer Free = 2 "Element is not active";
          constant Integer Forward = 1 "w_a > 0 (forward rolling)";
          constant Integer Stuck = 0 "w_a = 0 (forward rolling, locked or backward rolling)";
          constant Integer Backward = -1 "w_a < 0 (backward rolling)";
          Integer mode(final min = Backward, final max = Unknown, start = Free, fixed = true) "Mode of friction element (unknown, not active, forward/backward rolling, stuck)";
          SI.Torque tau_eta_p "tau_eta assuming positive omega";
          SI.Torque tau_eta_m "tau_eta assuming negative omega";
        protected
          constant SI.AngularAcceleration unitAngularAcceleration = 1;
          constant SI.Torque unitTorque = 1;
          // get friction and eta information for omega=0
          parameter Real eta_mf1_0(unit = "1") = Modelica.Math.Vectors.interpolate(lossTable[:, 1], lossTable[:, 2], 0, 1);
          parameter Real eta_mf2_0(unit = "1") = Modelica.Math.Vectors.interpolate(lossTable[:, 1], lossTable[:, 3], 0, 1);
          parameter SI.Torque tau_bf1_0 = abs(Modelica.Math.Vectors.interpolate(lossTable[:, 1], lossTable[:, 4], 0, 1));
          parameter SI.Torque tau_bf2_0 = abs(Modelica.Math.Vectors.interpolate(lossTable[:, 1], lossTable[:, 5], 0, 1));
          parameter SI.Torque tau_bf_a_0 = if Modelica.Math.isEqual(eta_mf1_0, 1.0, Modelica.Constants.eps) and Modelica.Math.isEqual(eta_mf2_0, 1.0, Modelica.Constants.eps) then tau_bf1_0 / 2 else (tau_bf1_0 - tau_bf2_0) / (eta_mf1_0 - 1.0 / eta_mf2_0);
          // For eta_mf1_0=eta_mf2_0=1 the given bearing
          // friction can not be separated into a part
          // on side A or B, so it is done arbitrarily.
          // Calculate tau_bf_a_0 from the following equations
          //  tau_bf1_0=eta_mf1_0*tau_bf_a_0 + 1/ratio a_0
          //  tau_bf2_0=1/eta_mf2*tau_bf_a_0 + 1/ratio tau_bf_a_0
        equation
          assert(abs(ratio) > 0, "Error in initialization of LossyGear: ratio may not be zero");
          ideal = Modelica.Math.Matrices.isEqual(lossTable, [0, 1, 1, 0, 0], Modelica.Constants.eps);
          if ideal then
            interpolation_result = [1, 1, 0, 0];
            eta_mf1 = 1;
            eta_mf2 = 1;
            tau_bf1 = 0;
            tau_bf2 = 0;
          else
            interpolation_result = [Modelica.Math.Vectors.interpolate(lossTable[:, 1], lossTable[:, 2], noEvent(abs(w_a)), 1), Modelica.Math.Vectors.interpolate(lossTable[:, 1], lossTable[:, 3], noEvent(abs(w_a)), 1), Modelica.Math.Vectors.interpolate(lossTable[:, 1], lossTable[:, 4], noEvent(abs(w_a)), 1), Modelica.Math.Vectors.interpolate(lossTable[:, 1], lossTable[:, 5], noEvent(abs(w_a)), 1)];
            eta_mf1 = interpolation_result[1, 1];
            eta_mf2 = interpolation_result[1, 2];
            tau_bf1 = noEvent(abs(interpolation_result[1, 3]));
            tau_bf2 = noEvent(abs(interpolation_result[1, 4]));
          end if;
          if Modelica.Math.isEqual(eta_mf1, 1.0, Modelica.Constants.eps) and Modelica.Math.isEqual(eta_mf2, 1.0, Modelica.Constants.eps) then
// For eta_mf1=eta_mf2=1 the given bearing friction can not be
// separated into a part on side A or B, so it is done arbitrarily.
            tau_bf_a = tau_bf1 / 2;
          else
//calculate tau_bf_a from the following equations
//tau_bf1 = eta_mf1*tau_bf_a + tau_bf_b / ratio
//tau_bf2 = 1/eta_mf2*tau_bf_a + tau_bf_b / ratio
            tau_bf_a = (tau_bf1 - tau_bf2) / (eta_mf1 - 1.0 / eta_mf2);
          end if;
          phi_a = flange_a.phi - phi_support;
          phi_b = flange_b.phi - phi_support;
          phi_a = ratio * phi_b;
// Torque balance (no inertias)
          0 = flange_b.tau + ratio * (flange_a.tau - tauLoss);
// Speed for friction element
          w_a = der(phi_a);
          a_a = der(w_a);
// Determine driving side
//assuming positive omega
          tau_eta_p = flange_a.tau - tau_bf_a_0;
//assuming negative omega
          tau_eta_m = flange_a.tau + tau_bf_a_0;
// assuming w>=0, take value at w=0 to decide rolling/stuck mode
          quadrant1_p = (1 - eta_mf1_0) * flange_a.tau + tau_bf1_0;
          quadrant2_p = (1 - 1 / eta_mf2_0) * flange_a.tau + tau_bf2_0;
          tauLossMax_p = if noEvent(tau_eta_p > 0) then quadrant1_p else quadrant2_p;
// assuming w<=0, take value at w=0 to decide rolling/stuck mode
          quadrant4_m = (1 - 1 / eta_mf2_0) * flange_a.tau - tau_bf2_0;
          quadrant3_m = (1 - eta_mf1_0) * flange_a.tau - tau_bf1_0;
          tauLossMin_m = if noEvent(tau_eta_m > 0) then quadrant4_m else quadrant3_m;
          quadrant1 = (1 - eta_mf1) * flange_a.tau + tau_bf1;
          quadrant2 = (1 - 1 / eta_mf2) * flange_a.tau + tau_bf2;
          quadrant4 = (1 - 1 / eta_mf2) * flange_a.tau - tau_bf2;
          quadrant3 = (1 - eta_mf1) * flange_a.tau - tau_bf1;
//tau eta: only for determination of driving side for calculation of tauloss
          tau_eta = if ideal then flange_a.tau else if locked then flange_a.tau else if startForward or pre(mode) == Forward then flange_a.tau - tau_bf_a else flange_a.tau + tau_bf_a;
// Torque Losses
          tau_etaPos = tau_eta >= 0;
          tau_aPos = tau_etaPos;
          tauLossMax = if tau_etaPos then quadrant1 else quadrant2;
          tauLossMin = if tau_etaPos then quadrant4 else quadrant3;
// Determine rolling/stuck mode when w_rel = 0
          startForward = pre(mode) == Stuck and sa > tauLossMax_p / unitTorque or initial() and w_a > 0;
          startBackward = pre(mode) == Stuck and sa < tauLossMin_m / unitTorque or initial() and w_a < 0;
          locked = not (ideal or pre(mode) == Forward or startForward or pre(mode) == Backward or startBackward);
/* Parameterized curve description a_a = f1(sa), tauLoss = f2(sa)
       In comparison to Modelica.Mechanics.Rotational.FrictionBase it is possible
       to simplify the following expression as mode == Stuck is assured in case
       of startForward or startBackward */
          tauLoss = if ideal then 0 else if locked then sa * unitTorque else if startForward or pre(mode) == Forward then tauLossMax else tauLossMin;
          a_a = unitAngularAcceleration * (if locked then 0 else sa - tauLoss / unitTorque);
/* Finite state machine to fix configuration after the computation above
       The above equations are only dependent on pre(mode) and not on the actual
       value of mode. This prevents loops. So mode can be determined in one step. */
          mode = if ideal then Free else if (pre(mode) == Forward or startForward) and w_a > 0 then Forward else if (pre(mode) == Backward or startBackward) and w_a < 0 then Backward else Stuck;
          lossPower = tauLoss * w_a;
          annotation(
            Documentation(info = "<HTML>
<p>
This component models the gear ratio and the <b>losses</b> of
a standard gear box in a <b>reliable</b> way including the stuck phases
that may occur at zero speed. The gear boxes that can
be handled are fixed in the ground or on a moving support, have one input and one
output shaft, and are essentially described by the equations:
</p>
<blockquote><pre>
             flange_a.phi  = i*flange_b.phi;
-(flange_b.tau - tau_bf_b) = i*eta_mf*(flange_a.tau - tau_bf_a);

// or        -flange_b.tau = i*eta_mf*(flange_a.tau - tau_bf_a - tau_bf_b/(i*eta_mf));
</pre></blockquote>
<p>
where
</p>

<ul>
<li> <b>i</b> is the constant <b>gear ratio</b>, </li>

<li> <b>eta_mf</b> = eta_mf(w_a) is the <b>mesh efficiency</b> due to the
     friction between the teeth of the gear wheels, </li>

<li> <b>tau_bf_a</b> = tau_bf_a(w_a) is the <b>bearing friction torque</b>
     on the flange_a side,</li>

<li> <b>tau_bf_b</b> = tau_bf_b(w_a) is the <b>bearing friction torque</b>
     on the flange_b side, and</li>

<li><b>w_a</b> = der(flange_a.phi) is the speed of flange_a</li>
</ul>

<p>
The loss terms \"eta_mf\", \"tau_bf_a\" and \"tau_bf_b\" are functions of the
<i>absolute value</i> of the input shaft speed w_a and of the energy
flow direction. They are defined by parameter <b>lossTable[:,5]
</b> where the columns of this table have the following
meaning:
</p>

<table BORDER=1 CELLSPACING=0 CELLPADDING=2>
  <tbody>
    <tr>
      <td valign=\"top\">|w_a|</td>
      <td valign=\"top\">eta_mf1</td>
      <td valign=\"top\">eta_mf2</td>
      <td valign=\"top\">|tau_bf1|</td>
      <td valign=\"top\">|tau_bf2|</td>
    </tr>
    <tr>
      <td align=\"center\">...</td>
      <td align=\"center\">...</td>
      <td align=\"center\">...</td>
      <td align=\"center\">...</td>
      <td align=\"center\">...</td>
    </tr>
    <tr>
      <td align=\"center\">...</td>
      <td align=\"center\">...</td>
      <td align=\"center\">...</td>
      <td align=\"center\">...</td>
      <td align=\"center\">...</td>
    </tr>
  </tbody>
</table>

<p>with</p>
<table BORDER=1 CELLSPACING=0 CELLPADDING=2>
  <tbody>
    <tr>
      <td valign=\"top\">|w_a|</td>
      <td valign=\"top\">Absolute value of angular velocity of input shaft flange_a</td>
    </tr>
    <tr>
      <td valign=\"top\">eta_mf1</td>
      <td valign=\"top\">Mesh efficiency in case that flange_a is driving</td>
    </tr>
    <tr>
      <td valign=\"top\">eta_mf2</td>
      <td valign=\"top\">Mesh efficiency in case that flange_b is driving</td>
    </tr>
    <tr>
      <td valign=\"top\">|tau_bf1|</td>
      <td valign=\"top\"> Absolute resultant bearing friction torque with respect to flange_a
                        in case that flange_a is driving<br>
                        (= |tau_bf_a*eta_mf1 + tau_bf_b/i|)
                        </td>
    </tr>
    <tr>
      <td valign=\"top\">|tau_bf2|</td>
      <td valign=\"top\"> Absolute resultant bearing friction torque with respect to flange_a
                        in case that flange_b is driving<br>
                        (= |tau_bf_a/eta_mf2 + tau_bf_b/i|)
                        </td>
    </tr>
  </tbody>
</table>
<p>
With these variables, the mesh efficiency and the bearing friction
are formally defined as:
</p>

<blockquote><pre>
<b>if</b> (flange_a.tau - tau_bf_a)*w_a &gt; 0 <b>or</b>
   (flange_a.tau - tau_bf_a) == 0 <b>and</b> w_a &gt; 0 <b>then</b>
   eta_mf := eta_mf1
   tau_bf := tau_bf1
<b>elseif</b> (flange_a.tau - tau_bf_a)*w_a &lt; 0 <b>or</b>
       (flange_a.tau - tau_bf_a) == 0 <b>and</b> w_a &lt; 0 <b>then</b>
   eta_mf := 1/eta_mf2
   tau_bf := tau_bf2
<b>else</b> // w_a == 0
   eta_mf and tau_bf are computed such that <b>der</b>(w_a) = 0
<b>end if</b>;
-flange_b.tau = i*(eta_mf*flange_a.tau - tau_bf);
</pre></blockquote>

<p>
Note, that the losses are modeled in a physically meaningful way taking
into account that at zero speed the movement may be locked due
to the friction in the gear teeth and/or in the bearings.
Due to this important property, this component can be used in
situations where the combination of the components
Modelica.Mechanics.Rotational.IdealGear and
Modelica.Mechanics.Rotational.GearEfficiency will fail because,
e.g., chattering occurs when using the
Modelica.Mechanics.Rotational.GearEfficiency model.
</p>

<h4>Acknowledgement:</h4>
<ul>
<li> The essential idea to model efficiency
     in this way is from Christoph Pelchen, ZF Friedrichshafen.</li>
<li> The article (Pelchen et.al. 2002), see Literature below,
     and the first implementation of LossyGear (up to version 3.1 of package Modelica)
     contained a bug leading to a non-converging solution in cases where the
     driving side is not obvious.
     This was pointed out by Christian Bertsch and Max Westenkirchner, Bosch,
     and Christian Bertsch proposed a concrete solution how to fix this
     bug, see Literature below.</li>
</ul>

<h4>Literature</h4>

<ul>
<li>Pelchen C.,
<a href=\"http://www.robotic.dlr.de/Christian.Schweiger/\">Schweiger C.</a>,
and <a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Otter M.</a>:
&quot;<a href=\"https://www.modelica.org/events/Conference2002/papers/p33_Pelchen.pdf\">Modeling
and Simulating the Efficiency of Gearboxes and of Planetary Gearboxes</A>,&quot; in
<I>Proceedings of the 2nd International Modelica Conference, Oberpfaffenhofen, Germany,</I>
pp. 257-266, The Modelica Association and Institute of Robotics and Mechatronics,
Deutsches Zentrum f&uuml;r Luft- und Raumfahrt e. V., March 18-19, 2002.</li>

<li>Bertsch C. (2009):
&quot;<a href=\"modelica://Modelica/Resources/Documentation/Mechanics/Lossy-Gear-Bug_Solution.pdf\">Problem
with model LossyGear and a proposed solution</a>&quot;,
Ticket <a href=\"https://trac.modelica.org/Modelica/ticket/108\">#108</a>,
Sept. 11, 2009.</li>
</ul>

</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100.0, -100.0}, {100.0, 100.0}}, initialScale = 0.1), graphics = {Polygon(fillColor = {161, 35, 41}, pattern = LinePattern.None, fillPattern = FillPattern.Solid, points = {{-109.0, 40.0}, {-80.0, 40.0}, {-80.0, 80.0}, {-90.0, 80.0}, {-70.0, 100.0}, {-50.0, 80.0}, {-60.0, 80.0}, {-60.0, 20.0}, {-109.0, 20.0}, {-109.0, 40.0}}), Line(points = {{-80.0, 20.0}, {-60.0, 20.0}}), Text(lineColor = {0, 0, 255}, extent = {{-148.0, 105.0}, {152.0, 145.0}}, textString = "%name"), Text(extent = {{-145.0, -79.0}, {155.0, -49.0}}, textString = "ratio=%ratio"), Line(visible = useHeatPort, points = {{-100.0, -100.0}, {-100.0, -30.0}, {0.0, -30.0}, {0.0, 0.0}}, color = {191, 0, 0}, pattern = LinePattern.Dot)}));
        end LossyGear;

        model IdealPlanetary "Ideal planetary gear box"
          parameter Real ratio(start = 100 / 50) "Number of ring_teeth/sun_teeth (e.g., ratio=100/50)";
          // kinematic relationship
          Interfaces.Flange_a sun "Flange of sun shaft" annotation(
            Placement(transformation(extent = {{-110, -10}, {-90, 10}}, rotation = 0)));
          Interfaces.Flange_a carrier "Flange of carrier shaft" annotation(
            Placement(transformation(extent = {{-110, 30}, {-90, 50}}, rotation = 0)));
          Interfaces.Flange_b ring "Flange of ring shaft" annotation(
            Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
        equation
          (1 + ratio) * carrier.phi = sun.phi + ratio * ring.phi;
// torque balance (no inertias)
          ring.tau = ratio * sun.tau;
          carrier.tau = -(1 + ratio) * sun.tau;
          annotation(
            Documentation(info = "<HTML>
<p>
The IdealPlanetary gear box is an ideal gear without inertia,
elasticity, damping or backlash consisting
of an inner <b>sun</b> wheel, an outer <b>ring</b> wheel and a
<b>planet</b> wheel located between sun and ring wheel. The bearing
of the planet wheel shaft is fixed in the planet <b>carrier</b>.
The component can be connected to other elements at the
sun, ring and/or carrier flanges. It is not possible to connect
to the planet wheel. If inertia shall not be neglected,
the sun, ring and carrier inertias can be easily added by attaching
inertias (= model Inertia) to the corresponding connectors.
The inertias of the planet wheels are always neglected.
</p>
<p>
The icon of the planetary gear signals that the sun and carrier
flanges are on the left side and the ring flange is on the right side
of the gear box. However, this component is generic and is valid
independently how the flanges are actually placed (e.g., sun wheel
may be placed on the right side instead on the left side in reality).
</p>
<p>
The ideal planetary gearbox is uniquely defined by the ratio
of the number of ring teeth zr with respect to the number of
sun teeth zs. For example, if there are 100 ring teeth and
50 sun teeth then ratio = zr/zs = 2. The number of planet teeth
zp has to fulfill the following relationship:
</p>
<pre>
   <b>zp := (zr - zs) / 2</b>
</pre>
<p>
Therefore, in the above example zp = 25 is required.
</p>
<p>
According to the overall convention, the positive direction of all
vectors, especially the absolute angular velocities and cut-torques
in the flanges, are along the axis vector displayed in the icon.
</p>

</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100.0, -100.0}, {100.0, 100.0}}, initialScale = 0.1), graphics = {Rectangle(lineColor = {160, 160, 164}, fillColor = {160, 160, 164}, fillPattern = FillPattern.Solid, extent = {{-50.0, 100.0}, {50.0, 105.0}}), Rectangle(lineColor = {160, 160, 164}, fillColor = {160, 160, 164}, fillPattern = FillPattern.Solid, extent = {{-50.0, -105.0}, {50.0, -100.0}}), Line(points = {{-90.0, 40.0}, {-70.0, 40.0}}), Line(points = {{-80.0, 50.0}, {-60.0, 50.0}}), Line(points = {{-70.0, 50.0}, {-70.0, 40.0}}), Line(points = {{-80.0, 80.0}, {-59.0, 80.0}}), Line(points = {{-70.0, 100.0}, {-70.0, 80.0}}), Text(lineColor = {0, 0, 255}, extent = {{-150.0, 110.0}, {150.0, 150.0}}, textString = "%name"), Text(extent = {{-150.0, -150.0}, {150.0, -110.0}}, textString = "ratio=%ratio"), Rectangle(lineColor = {64, 64, 64}, fillColor = {255, 255, 255}, fillPattern = FillPattern.HorizontalCylinder, extent = {{10.0, -100.0}, {50.0, 100.0}}), Rectangle(lineColor = {64, 64, 64}, fillColor = {192, 192, 192}, fillPattern = FillPattern.HorizontalCylinder, extent = {{-100.0, -10.0}, {-50.0, 10.0}}), Rectangle(lineColor = {64, 64, 64}, fillColor = {192, 192, 192}, fillPattern = FillPattern.HorizontalCylinder, extent = {{50.0, -10.0}, {100.0, 10.0}}), Rectangle(lineColor = {64, 64, 64}, fillColor = {192, 192, 192}, fillPattern = FillPattern.HorizontalCylinder, extent = {{-80.0, 60.0}, {-50.0, 70.0}}), Rectangle(origin = {-30.0, 65.0}, fillColor = {192, 192, 192}, fillPattern = FillPattern.HorizontalCylinder, extent = {{-20.0, -19.0}, {20.0, 19.0}}), Rectangle(origin = {-30.0, 48.0}, fillColor = {153, 153, 153}, fillPattern = FillPattern.Solid, extent = {{-20.0, -2.0}, {20.0, 2.0}}), Rectangle(origin = {-30.0, 82.0}, fillColor = {204, 204, 204}, fillPattern = FillPattern.Solid, extent = {{-20.0, -2.0}, {20.0, 2.0}}), Rectangle(origin = {-30.0, 59.0}, fillColor = {204, 204, 204}, fillPattern = FillPattern.Solid, extent = {{-20.0, -4.0}, {20.0, 4.0}}), Rectangle(origin = {-30.0, 73.0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, extent = {{-20.0, -3.0}, {20.0, 3.0}}), Rectangle(origin = {-30.0, -1.0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.HorizontalCylinder, extent = {{-20.0, -35.0}, {20.0, 35.0}}), Rectangle(origin = {-30.0, 32.0}, fillColor = {153, 153, 153}, fillPattern = FillPattern.Solid, extent = {{-20.0, -2.0}, {20.0, 2.0}}), Rectangle(origin = {-30.0, 23.0}, fillColor = {204, 204, 204}, fillPattern = FillPattern.Solid, extent = {{-20.0, -3.0}, {20.0, 3.0}}), Rectangle(origin = {-30.0, -10.0}, fillColor = {204, 204, 204}, fillPattern = FillPattern.Solid, extent = {{-20.0, -4.0}, {20.0, 4.0}}), Rectangle(origin = {-30.0, -34.0}, fillColor = {102, 102, 102}, fillPattern = FillPattern.Solid, extent = {{-20.0, -2.0}, {20.0, 2.0}}), Rectangle(origin = {-30.0, -25.0}, fillColor = {153, 153, 153}, fillPattern = FillPattern.Solid, extent = {{-20.0, -3.0}, {20.0, 3.0}}), Rectangle(origin = {-30.0, 8.0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, extent = {{-20.0, -4.0}, {20.0, 4.0}}), Rectangle(visible = true, origin = {-30, 65}, fillColor = {192, 192, 192}, extent = {{-20, -19}, {20, 19}}), Rectangle(visible = true, origin = {-30, -1}, fillColor = {255, 255, 255}, extent = {{-20, -35}, {20, 35}}), Rectangle(visible = true, lineColor = {64, 64, 64}, fillColor = {255, 255, 255}, extent = {{10, -100}, {50, 100}})}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(lineColor = {64, 64, 64}, fillColor = {192, 192, 192}, fillPattern = FillPattern.HorizontalCylinder, extent = {{-100, -10}, {-50, 10}}), Rectangle(lineColor = {64, 64, 64}, fillColor = {192, 192, 192}, fillPattern = FillPattern.HorizontalCylinder, extent = {{50, -10}, {100, 10}}), Rectangle(lineColor = {160, 160, 164}, fillColor = {160, 160, 164}, fillPattern = FillPattern.Solid, extent = {{-50, 100}, {50, 105}}), Rectangle(lineColor = {160, 160, 164}, fillColor = {160, 160, 164}, fillPattern = FillPattern.Solid, extent = {{-50, -105}, {50, -100}}), Rectangle(lineColor = {64, 64, 64}, fillColor = {192, 192, 192}, fillPattern = FillPattern.HorizontalCylinder, extent = {{-80, 60}, {-50, 70}}), Line(points = {{-90, 40}, {-70, 40}}), Line(points = {{-80, 50}, {-60, 50}}), Line(points = {{-70, 50}, {-70, 40}}), Line(points = {{-80, 80}, {-59, 80}}), Line(points = {{-70, 90}, {-70, 80}}), Line(points = {{-26, -42}, {-32, -2}}, color = {0, 0, 255}, pattern = LinePattern.Dot), Text(lineColor = {0, 0, 255}, extent = {{58, -78}, {98, -66}}, textString = "ring gear"), Text(lineColor = {0, 0, 255}, extent = {{-112, 87}, {-56, 111}}, textString = "planet carrier "), Text(lineColor = {0, 0, 255}, extent = {{-47, -56}, {-3, -42}}, textString = "sun gear"), Polygon(lineColor = {128, 128, 128}, fillColor = {128, 128, 128}, fillPattern = FillPattern.Solid, points = {{58, 130}, {28, 140}, {28, 120}, {58, 130}}), Line(points = {{-52, 130}, {28, 130}}), Line(points = {{-92, 93}, {-70, 80}}, color = {0, 0, 255}, pattern = LinePattern.Dot), Polygon(lineColor = {128, 128, 128}, fillColor = {128, 128, 128}, fillPattern = FillPattern.Solid, points = {{-7, -86}, {-27, -81}, {-27, -91}, {-7, -86}}), Line(points = {{-97, -86}, {-26, -86}}, color = {128, 128, 128}), Text(lineColor = {128, 128, 128}, extent = {{-96, -84}, {-28, -71}}, textString = "rotation axis"), Rectangle(origin = {-30, 65}, fillColor = {192, 192, 192}, fillPattern = FillPattern.HorizontalCylinder, extent = {{-20, -19}, {20, 19}}), Rectangle(origin = {-30, 48}, fillColor = {153, 153, 153}, fillPattern = FillPattern.Solid, extent = {{-20, -2}, {20, 2}}), Rectangle(origin = {-30, 82}, fillColor = {204, 204, 204}, fillPattern = FillPattern.Solid, extent = {{-20, -2}, {20, 2}}), Rectangle(origin = {-30, 59}, fillColor = {204, 204, 204}, fillPattern = FillPattern.Solid, extent = {{-20, -4}, {20, 4}}), Rectangle(origin = {-30, 73}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, extent = {{-20, -3}, {20, 3}}), Rectangle(origin = {-30, 65}, fillColor = {192, 192, 192}, extent = {{-20, -19}, {20, 19}}), Rectangle(origin = {-30, -1}, fillColor = {255, 255, 255}, fillPattern = FillPattern.HorizontalCylinder, extent = {{-20, -35}, {20, 35}}), Rectangle(origin = {-30, 32}, fillColor = {153, 153, 153}, fillPattern = FillPattern.Solid, extent = {{-20, -2}, {20, 2}}), Rectangle(origin = {-30, 23}, fillColor = {204, 204, 204}, fillPattern = FillPattern.Solid, extent = {{-20, -3}, {20, 3}}), Rectangle(origin = {-30, -10}, fillColor = {204, 204, 204}, fillPattern = FillPattern.Solid, extent = {{-20, -4}, {20, 4}}), Rectangle(origin = {-30, -34}, fillColor = {102, 102, 102}, fillPattern = FillPattern.Solid, extent = {{-20, -2}, {20, 2}}), Rectangle(origin = {-30, -25}, fillColor = {153, 153, 153}, fillPattern = FillPattern.Solid, extent = {{-20, -3}, {20, 3}}), Rectangle(origin = {-30, 8}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, extent = {{-20, -4}, {20, 4}}), Rectangle(origin = {-30, -1}, fillColor = {255, 255, 255}, extent = {{-20, -35}, {20, 35}}), Rectangle(lineColor = {64, 64, 64}, fillColor = {255, 255, 255}, fillPattern = FillPattern.HorizontalCylinder, extent = {{10, -100}, {50, 100}}), Rectangle(lineColor = {64, 64, 64}, fillColor = {255, 255, 255}, extent = {{10, -100}, {50, 100}}), Line(points = {{36, -26}, {64, -60}}, color = {0, 0, 255}, pattern = LinePattern.Dot)}));
        end IdealPlanetary;

        model Gearbox "Realistic model of a gearbox (based on LossyGear)"
          extends Modelica.Mechanics.Rotational.Icons.Gearbox;
          extends Modelica.Mechanics.Rotational.Interfaces.PartialTwoFlangesAndSupport;
          parameter Real ratio(start = 1) "Transmission ratio (flange_a.phi/flange_b.phi)";
          parameter Real lossTable[:, 5] = [0, 1, 1, 0, 0] "Array for mesh efficiencies and bearing friction depending on speed (see docu of LossyGear)";
          parameter SI.RotationalSpringConstant c(final min = Modelica.Constants.small, start = 1.0e5) "Gear elasticity (spring constant)";
          parameter SI.RotationalDampingConstant d(final min = 0, start = 0) "(relative) gear damping";
          parameter SI.Angle b(final min = 0) = 0 "Total backlash";
          parameter StateSelect stateSelect = StateSelect.prefer "Priority to use phi_rel and w_rel as states" annotation(
            HideResult = true,
            Dialog(tab = "Advanced"));
          extends Modelica.Thermal.HeatTransfer.Interfaces.PartialConditionalHeatPort(final T = 293.15);
          Modelica.SIunits.Angle phi_rel(start = 0, stateSelect = stateSelect, nominal = 1e-4) = flange_b.phi - lossyGear.flange_b.phi "Relative rotation angle over gear elasticity (= flange_b.phi - lossyGear.flange_b.phi)";
          Modelica.SIunits.AngularVelocity w_rel(start = 0, stateSelect = stateSelect) = der(phi_rel) "Relative angular velocity over gear elasticity (= der(phi_rel))";
          Modelica.SIunits.AngularAcceleration a_rel(start = 0) = der(w_rel) "Relative angular acceleration over gear elasticity (= der(w_rel))";
          Rotational.Components.LossyGear lossyGear(final ratio = ratio, final lossTable = lossTable, final useSupport = true, final useHeatPort = true) annotation(
            Placement(transformation(extent = {{-60, -20}, {-20, 20}}, rotation = 0)));
          Rotational.Components.ElastoBacklash elastoBacklash(final b = b, final c = c, final phi_rel0 = 0, final d = d, final useHeatPort = true) annotation(
            Placement(transformation(extent = {{20, -20}, {60, 20}}, rotation = 0)));
        equation
          connect(flange_a, lossyGear.flange_a) annotation(
            Line(points = {{-100, 0}, {-90, 0}, {-90, 0}, {-80, 0}, {-80, 0}, {-60, 0}}, color = {0, 0, 0}));
          connect(lossyGear.flange_b, elastoBacklash.flange_a) annotation(
            Line(points = {{-20, 0}, {-10, 0}, {0, 0}, {20, 0}}, color = {0, 0, 0}));
          connect(elastoBacklash.flange_b, flange_b) annotation(
            Line(points = {{60, 0}, {70, 0}, {70, 0}, {80, 0}, {80, 0}, {100, 0}}, color = {0, 0, 0}));
          connect(elastoBacklash.heatPort, internalHeatPort) annotation(
            Line(points = {{20, -20}, {20, -60}, {-100, -60}, {-100, -80}}, color = {191, 0, 0}, smooth = Smooth.None));
          connect(lossyGear.heatPort, internalHeatPort) annotation(
            Line(points = {{-60, -20}, {-60, -60}, {-100, -60}, {-100, -80}}, color = {191, 0, 0}, smooth = Smooth.None));
          connect(lossyGear.support, internalSupport) annotation(
            Line(points = {{-40, -20}, {-40, -40}, {0, -40}, {0, -80}}, color = {0, 0, 0}, smooth = Smooth.None));
          annotation(
            Documentation(info = "<html>
<p>This component models the essential effects of a gearbox, in
particular</p>
<ul>
  <li>in component <b>lossyGear</b>
    <ul>
      <li>gear <b>efficiency</b> due to friction between the teeth</li>
      <li><b>bearing friction</b></li>
    </ul></li>
  <li>in component <b>elastoBacklash</b>
    <ul>
      <li>gear <b>elasticity</b></li>
      <li><b>damping</b></li>
      <li><b>backlash</b></li>
    </ul></li>
</ul>
<p>The inertia of the gear wheels is not modeled. If necessary,
inertia has to be taken into account by connecting components of
model Inertia to the left and/or the right flange of component
Gearbox.
</p>

</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-150, 150}, {150, 110}}, lineColor = {0, 0, 255}, textString = "%name"), Text(extent = {{-150, 70}, {150, 100}}, lineColor = {0, 0, 0}, textString = "ratio=%ratio, c=%c"), Line(visible = useHeatPort, points = {{-100, -100}, {-100, -30}, {0, -30}}, color = {191, 0, 0}, pattern = LinePattern.Dot, smooth = Smooth.None)}));
        end Gearbox;

        model IdealGearR2T "Gearbox transforming rotational into translational motion"
          extends Rotational.Interfaces.PartialElementaryRotationalToTranslational;
          parameter Real ratio(final unit = "rad/m", start = 1) "Transmission ratio (flange_a.phi/flange_b.s)";
        equation
          flangeR.phi - internalSupportR.phi = ratio * (flangeT.s - internalSupportT.s);
          0 = ratio * flangeR.tau + flangeT.f;
          annotation(
            Documentation(info = "<html>
<p>
This is an ideal mass- and inertialess gearbox which transforms a
1D-rotational into a 1D-translational motion. If elasticity, damping
or backlash has to be considered, this ideal gearbox has to be
connected with corresponding elements.
This component defines the kinematic constraint:
</p>

<pre>
  (flangeR.phi - internalSupportR.phi) = ratio*(flangeT.s - internalSupportT.s);
</pre>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100.0, -100.0}, {100.0, 100.0}}, initialScale = 0.1), graphics = {Rectangle(origin = {133.3333, 0.0}, lineColor = {64, 64, 64}, fillColor = {192, 192, 192}, fillPattern = FillPattern.HorizontalCylinder, extent = {{-233.3333, -10.0}, {-163.3333, 10.0}}), Text(lineColor = {0, 0, 255}, extent = {{-150.0, 85.0}, {150.0, 125.0}}, textString = "%name"), Rectangle(fillColor = {160, 160, 164}, fillPattern = FillPattern.Solid, extent = {{95.0, -60.0}, {106.0, -10.0}}), Rectangle(origin = {-0.5889, 0.0}, fillColor = {160, 160, 164}, fillPattern = FillPattern.Solid, extent = {{-74.4111, -80.0}, {106.5889, -60.0}}), Text(extent = {{-150.0, 50.0}, {150.0, 80.0}}, textString = "ratio=%ratio"), Line(points = {{-100.0, 15.0}, {-80.0, 15.0}}), Line(points = {{-100.0, -16.0}, {-80.0, -16.0}}), Line(points = {{-100.0, -16.0}, {-100.0, -100.0}}), Line(points = {{100.0, -80.0}, {100.0, -100.0}}), Polygon(origin = {16.875, -50.0}, fillColor = {160, 160, 164}, fillPattern = FillPattern.Solid, points = {{-84.375, -10.0}, {-79.375, 10.0}, {-69.375, 10.0}, {-64.375, -10.0}, {-54.375, -10.0}, {-49.375, 10.0}, {-39.375, 10.0}, {-34.375, -10.0}, {-24.375, -10.0}, {-19.375, 10.0}, {-9.375, 10.0}, {-4.375, -10.0}, {5.625, -10.0}, {10.625, 10.0}, {20.625, 10.0}, {25.625, -10.0}, {35.625, -10.0}, {40.625, 10.0}, {50.625, 10.0}, {55.625, -10.0}, {65.625, -10.0}, {70.625, 10.0}, {78.125, 10.0}, {78.125, -10.0}}), Polygon(origin = {-20.0, -0.0}, rotation = 10.0, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, points = {{-5.0, 45.0}, {-10.0, 10.0}, {-45.0, 5.0}, {-45.0, -5.0}, {-10.0, -10.0}, {-5.0, -45.0}, {5.0, -45.0}, {10.0, -10.0}, {45.0, -5.0}, {45.0, 5.0}, {10.0, 10.0}, {5.0, 45.0}}), Polygon(origin = {-20.0, -0.0}, rotation = 55.0, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, points = {{-5.0, 45.0}, {-10.0, 10.0}, {-45.0, 5.0}, {-45.0, -5.0}, {-10.0, -10.0}, {-5.0, -45.0}, {5.0, -45.0}, {10.0, -10.0}, {45.0, -5.0}, {45.0, 5.0}, {10.0, 10.0}, {5.0, 45.0}}), Ellipse(origin = {-20.0, -0.0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, extent = {{-30.0, -30.0}, {30.0, 30.0}}), Ellipse(origin = {-20.0, -0.0}, fillColor = {128, 128, 128}, fillPattern = FillPattern.Solid, extent = {{-10.0, -10.0}, {10.0, 10.0}})}));
        end IdealGearR2T;

        model IdealRollingWheel "Simple 1-dim. model of an ideal rolling wheel without inertia"
          extends Rotational.Interfaces.PartialElementaryRotationalToTranslational;
          parameter Modelica.SIunits.Distance radius(start = 0.3) "Wheel radius";
        equation
          (flangeR.phi - internalSupportR.phi) * radius = flangeT.s - internalSupportT.s;
          0 = radius * flangeT.f + flangeR.tau;
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100.0, -100.0}, {100.0, 100.0}}, initialScale = 0.1), graphics = {Rectangle(lineColor = {64, 64, 64}, fillColor = {192, 192, 192}, fillPattern = FillPattern.HorizontalCylinder, extent = {{-100.0, -10.0}, {-46.0, 10.0}}), Ellipse(lineColor = {64, 64, 64}, fillColor = {255, 255, 255}, fillPattern = FillPattern.HorizontalCylinder, extent = {{-50.0, -80.0}, {10.0, 80.0}}), Rectangle(lineColor = {64, 64, 64}, fillColor = {255, 255, 255}, pattern = LinePattern.None, fillPattern = FillPattern.HorizontalCylinder, extent = {{-20.0, -80.0}, {12.0, 80.0}}), Ellipse(lineColor = {64, 64, 64}, fillColor = {128, 128, 128}, fillPattern = FillPattern.Solid, extent = {{-16.0, -80.0}, {44.0, 80.0}}), Ellipse(lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid, extent = {{-2.0, -52.0}, {34.0, 52.0}}), Ellipse(lineColor = {64, 64, 64}, fillColor = {192, 192, 192}, fillPattern = FillPattern.HorizontalCylinder, extent = {{12.0, -10.0}, {20.0, 10.0}}), Rectangle(lineColor = {64, 64, 64}, fillColor = {192, 192, 192}, pattern = LinePattern.None, fillPattern = FillPattern.HorizontalCylinder, extent = {{16.0, -10.0}, {60.0, 10.0}}), Ellipse(fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid, extent = {{56.0, -10.0}, {64.0, 10.0}}), Text(lineColor = {0, 0, 255}, extent = {{-150.0, 90.0}, {150.0, 130.0}}, textString = "%name"), Polygon(lineColor = {0, 127, 0}, fillColor = {0, 127, 0}, fillPattern = FillPattern.Solid, points = {{80.0, 10.0}, {80.0, 26.0}, {60.0, 26.0}, {60.0, 20.0}, {70.0, 20.0}, {70.0, -20.0}, {60.0, -20.0}, {60.0, -26.0}, {80.0, -26.0}, {80.0, -10.0}, {90.0, -10.0}, {90.0, 10.0}, {80.0, 10.0}}), Line(points = {{-100.0, -20.0}, {-60.0, -20.0}}), Line(points = {{-100.0, -20.0}, {-100.0, -100.0}}), Line(points = {{-100.0, 20.0}, {-60.0, 20.0}}), Line(points = {{100.0, -90.0}, {-40.0, -90.0}}, color = {0, 127, 0}), Line(points = {{70.0, -26.0}, {70.0, -50.0}, {100.0, -50.0}, {100.0, -100.0}}, color = {0, 127, 0}), Line(origin = {-2.5, 80.0}, points = {{-17.5, 0.0}, {17.5, 0.0}}, color = {64, 64, 64}), Line(origin = {-2.5, -80.0}, points = {{-17.5, 0.0}, {17.5, 0.0}}, color = {64, 64, 64}), Line(origin = {38.0, 10.0}, points = {{-22.0, 0.0}, {22.0, 0.0}}, color = {64, 64, 64}), Line(origin = {38.0, -10.0}, points = {{-22.0, 0.0}, {22.0, 0.0}}, color = {64, 64, 64})}),
            Documentation(info = "<html>
<p>
A simple kinematic model of a rolling wheel which has no inertia and
no rolling resistance. This component defines the kinematic constraint:
</p>

<pre>
   (flangeR.phi - internalSupportR.phi)*wheelRadius = (flangeT.s - internalSupportT.s);
</pre>
</html>"));
        end IdealRollingWheel;

        model InitializeFlange "Initializes a flange with pre-defined angle, speed and angular acceleration (usually, this is reference data from a control bus)"
          extends Modelica.Blocks.Icons.Block;
          parameter Boolean use_phi_start = true "= true, if initial angle is defined by input phi_start, otherwise not initialized";
          parameter Boolean use_w_start = true "= true, if initial speed is defined by input w_start, otherwise not initialized";
          parameter Boolean use_a_start = true "= true, if initial angular acceleration is defined by input a_start, otherwise not initialized";
          parameter StateSelect stateSelect = StateSelect.default "Priority to use flange angle and speed as states";
          Modelica.Blocks.Interfaces.RealInput phi_start(unit = "rad") if use_phi_start "Initial angle of flange" annotation(
            Placement(transformation(extent = {{-140, 60}, {-100, 100}}, rotation = 0), iconTransformation(extent = {{-140, 60}, {-100, 100}})));
          Modelica.Blocks.Interfaces.RealInput w_start(unit = "rad/s") if use_w_start "Initial speed of flange" annotation(
            Placement(transformation(extent = {{-140, -20}, {-100, 20}}, rotation = 0)));
          Modelica.Blocks.Interfaces.RealInput a_start(unit = "rad/s2") if use_a_start "Initial angular acceleration of flange" annotation(
            Placement(transformation(extent = {{-140, -100}, {-100, -60}}, rotation = 0), iconTransformation(extent = {{-140, -100}, {-100, -60}})));
          Interfaces.Flange_b flange "Flange that is initialized" annotation(
            Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
          Modelica.SIunits.Angle phi_flange(stateSelect = stateSelect) = flange.phi "Flange angle";
          Modelica.SIunits.AngularVelocity w_flange(stateSelect = stateSelect) = der(phi_flange) "= der(phi_flange)";
        protected
          encapsulated model Set_phi_start "Set phi_start"
            import Modelica;
            extends Modelica.Blocks.Icons.Block;
            Modelica.Blocks.Interfaces.RealInput phi_start(unit = "rad") "Start angle" annotation(
              HideResult = true,
              Placement(transformation(extent = {{-140, -20}, {-100, 20}}, rotation = 0)));
            Modelica.Mechanics.Rotational.Interfaces.Flange_b flange annotation(
              Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
          initial equation
            flange.phi = phi_start;
          equation
            flange.tau = 0;
          end Set_phi_start;

          encapsulated model Set_w_start "Set w_start"
            import Modelica;
            extends Modelica.Blocks.Icons.Block;
            Modelica.Blocks.Interfaces.RealInput w_start(unit = "rad/s") "Start angular velocity" annotation(
              HideResult = true,
              Placement(transformation(extent = {{-140, -20}, {-100, 20}}, rotation = 0)));
            Modelica.Mechanics.Rotational.Interfaces.Flange_b flange annotation(
              Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
          initial equation
            der(flange.phi) = w_start;
          equation
            flange.tau = 0;
          end Set_w_start;

          encapsulated model Set_a_start "Set a_start"
            import Modelica;
            extends Modelica.Blocks.Icons.Block;
            Modelica.Blocks.Interfaces.RealInput a_start(unit = "rad/s2") "Start angular acceleration" annotation(
              HideResult = true,
              Placement(transformation(extent = {{-140, -20}, {-100, 20}}, rotation = 0)));
            Modelica.Mechanics.Rotational.Interfaces.Flange_b flange(phi(stateSelect = StateSelect.avoid)) annotation(
              Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
            Modelica.SIunits.AngularVelocity w = der(flange.phi) annotation(
              HideResult = true);
          initial equation
            der(w) = a_start;
          equation
            flange.tau = 0;
          end Set_a_start;

          encapsulated model Set_flange_tau "Set flange.tau to zero"
            import Modelica;
            extends Modelica.Blocks.Icons.Block;
            Modelica.Mechanics.Rotational.Interfaces.Flange_b flange annotation(
              Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
          equation
            flange.tau = 0;
          end Set_flange_tau;
        protected
          Set_phi_start set_phi_start if use_phi_start annotation(
            Placement(transformation(extent = {{-20, 70}, {0, 90}}, rotation = 0)));
          Set_w_start set_w_start if use_w_start annotation(
            Placement(transformation(extent = {{-20, -10}, {0, 10}}, rotation = 0)));
          Set_a_start set_a_start if use_a_start annotation(
            Placement(transformation(extent = {{-20, -90}, {0, -70}}, rotation = 0)));
          Set_flange_tau set_flange_tau annotation(
            Placement(transformation(extent = {{96, -90}, {76, -70}}, rotation = 0)));
        equation
          connect(set_phi_start.phi_start, phi_start) annotation(
            Line(points = {{-22, 80}, {-120, 80}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(set_phi_start.flange, flange) annotation(
            Line(points = {{0, 80}, {60, 80}, {60, 0}, {100, 0}}, color = {0, 0, 0}, smooth = Smooth.None));
          connect(set_w_start.flange, flange) annotation(
            Line(points = {{0, 0}, {100, 0}}, color = {0, 0, 0}, smooth = Smooth.None));
          connect(set_w_start.w_start, w_start) annotation(
            Line(points = {{-22, 0}, {-120, 0}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(set_a_start.a_start, a_start) annotation(
            Line(points = {{-22, -80}, {-120, -80}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(set_a_start.flange, flange) annotation(
            Line(points = {{0, -80}, {60, -80}, {60, 0}, {100, 0}}, color = {0, 0, 0}, smooth = Smooth.None));
          connect(set_flange_tau.flange, flange) annotation(
            Line(points = {{76, -80}, {60, -80}, {60, 0}, {100, 0}}, color = {0, 0, 0}, smooth = Smooth.None));
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-94, 94}, {68, 66}}, lineColor = {0, 0, 0}, textString = "phi_start"), Text(extent = {{-94, 16}, {70, -14}}, lineColor = {0, 0, 0}, textString = "w_start"), Text(extent = {{-92, -66}, {68, -94}}, lineColor = {0, 0, 0}, textString = "a_start")}),
            Documentation(info = "<html>
<p>
This component is used to optionally initialize the angle, speed,
and/or angular acceleration of the flange to which this component
is connected. Via parameters use_phi_start, use_w_start, use_a_start
the corresponding input signals phi_start, w_start, a_start are conditionally
activated. If an input is activated, the corresponding flange property
is initialized with the input value at start time.
</p>

<p>
For example, if \"use_phi_start = true\", then flange.phi is initialized
with the value of the input signal \"phi_start\" at the start time.
</p>

<p>
Additionally, it is optionally possible to define the \"StateSelect\"
attribute of the flange angle and the flange speed via parameter
\"stateSelection\".
</p>

<p>
This component is especially useful when the initial values of a flange
shall be set according to reference signals of a controller that are
provided via a signal bus.
</p>
</html>"));
        end InitializeFlange;

        model RelativeStates "Definition of relative state variables"
          extends Rotational.Interfaces.PartialTwoFlanges;
          parameter StateSelect stateSelect = StateSelect.prefer "Priority to use the relative angle and relative speed as states";
          parameter SI.Angle phi_nominal(displayUnit = "rad", min = 0.0) = 1.0 "Nominal value of the relative angle (used for scaling)";
          SI.Angle phi_rel(start = 0, stateSelect = stateSelect, nominal = if phi_nominal >= Modelica.Constants.eps then phi_nominal else 1) "Relative rotation angle used as state variable";
          SI.AngularVelocity w_rel(start = 0, stateSelect = stateSelect) "Relative angular velocity used as state variable";
          SI.AngularAcceleration a_rel(start = 0) "Relative angular acceleration";
        equation
          phi_rel = flange_b.phi - flange_a.phi;
          w_rel = der(phi_rel);
          a_rel = der(w_rel);
          flange_a.tau = 0;
          flange_b.tau = 0;
          annotation(
            Documentation(info = "<html>
<p>
Usually, the absolute angle and the absolute angular velocity of
Modelica.Mechanics.Rotational.Components.Inertia models are used as state variables.
In some circumstances, relative quantities are better suited, e.g.,
because it may be easier to supply initial values.
In such cases, model <b>RelativeStates</b> allows the definition of state variables
in the following way:
</p>
<ul>
<li> Connect an instance of this model between two flange connectors.</li>
<li> The <b>relative rotation angle</b> and the <b>relative angular velocity</b>
     between the two connectors are used as <b>state variables</b>.
</ul>
<p>
An example is given in the next figure
</p>

<p>
<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/Rotational/relativeStates.png\" ALT=\"relativeStates\">
</p>

<p>
Here, the relative angle and the relative angular velocity between
the two inertias are used as state variables. Additionally, the
simulator selects either the absolute angle and absolute angular
velocity of model inertia1 or of model inertia2 as state variables.
</p>

<p>
See also the discussion
<a href=\"modelica://Modelica.Mechanics.Rotational.UsersGuide.StateSelection\">State Selection</a>
in the User's Guide of the Rotational library.
</p>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100.0, -100.0}, {100.0, 100.0}}, initialScale = 0.1), graphics = {Ellipse(lineColor = {87, 195, 193}, fillColor = {87, 195, 193}, fillPattern = FillPattern.Solid, extent = {{-40.0, -40.0}, {40.0, 40.0}}), Text(lineColor = {0, 0, 255}, extent = {{-40.0, -40.0}, {40.0, 40.0}}, textString = "S"), Line(points = {{-92.0, 0.0}, {-42.0, 0.0}}, pattern = LinePattern.Dot), Line(points = {{40.0, 0.0}, {90.0, 0.0}}, pattern = LinePattern.Dot), Text(lineColor = {0, 0, 255}, extent = {{-150.0, -80.0}, {150.0, -40.0}}, textString = "%name")}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-40, 40}, {40, -40}}, lineColor = {0, 255, 255}, fillColor = {0, 255, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-40, 40}, {40, -40}}, textString = "S", lineColor = {0, 0, 255}), Line(points = {{40, 0}, {96, 0}}, color = {0, 0, 0}, pattern = LinePattern.Dash), Line(points = {{-100, -10}, {-100, -80}}, color = {160, 160, 164}), Line(points = {{100, -10}, {100, -80}}, color = {160, 160, 164}), Polygon(points = {{80, -65}, {80, -55}, {100, -60}, {80, -65}}, lineColor = {160, 160, 164}, fillColor = {160, 160, 164}, fillPattern = FillPattern.Solid), Line(points = {{-100, -60}, {80, -60}}, color = {160, 160, 164}), Text(extent = {{-30, -70}, {30, -90}}, textString = "w_rel", lineColor = {0, 0, 255}), Line(points = {{-76, 80}, {-5, 80}}, color = {128, 128, 128}), Polygon(points = {{14, 80}, {-6, 85}, {-6, 75}, {14, 80}}, lineColor = {128, 128, 128}, fillColor = {128, 128, 128}, fillPattern = FillPattern.Solid), Text(extent = {{18, 87}, {86, 74}}, lineColor = {128, 128, 128}, textString = "rotation axis"), Line(points = {{-96, 0}, {-40, 0}}, color = {0, 0, 0}, pattern = LinePattern.Dash)}));
        end RelativeStates;

        model TorqueToAngleAdaptor "Signal adaptor for a Rotational flange with angle, speed, and acceleration as outputs and torque as input (especially useful for FMUs)"
          parameter Boolean use_w = true "= true, enable the output connector w (angular velocity)" annotation(
            Evaluate = true,
            HideResult = true,
            choices(checkBox = true));
          parameter Boolean use_a = true "= true, enable the output connector a (angular acceleration)" annotation(
            Evaluate = true,
            HideResult = true,
            choices(checkBox = true));
          Modelica.Mechanics.Rotational.Interfaces.Flange_a flange annotation(
            Placement(transformation(extent = {{-30, -10}, {-10, 10}})));
          Modelica.Blocks.Interfaces.RealOutput phi(unit = "rad") "Flange moves with angle phi due to torque tau" annotation(
            Placement(transformation(extent = {{20, 70}, {40, 90}})));
          Modelica.Blocks.Interfaces.RealOutput w(unit = "rad/s") if use_w "Flange moves with speed w due to torque tau" annotation(
            Placement(transformation(extent = {{20, 20}, {40, 40}})));
          Modelica.Blocks.Interfaces.RealOutput a(unit = "rad/s2") if use_a "Flange moves with angular acceleration a due to torque tau" annotation(
            Placement(transformation(extent = {{20, -40}, {40, -20}})));
          Modelica.Blocks.Interfaces.RealInput tau(unit = "N.m") "Torque to drive the flange" annotation(
            Placement(transformation(extent = {{60, -100}, {20, -60}})));
        protected
          Modelica.Blocks.Interfaces.RealInput w_internal "Needed to connect to conditional connector w";
          Modelica.Blocks.Interfaces.RealInput a_internal "Needed to connect to conditional connector a";
        equation
          connect(w, w_internal);
          connect(a, a_internal);
          phi = flange.phi;
          if use_w then
            w_internal = der(phi);
          else
            w_internal = 0.0;
          end if;
          if use_a then
            a_internal = der(w_internal);
          else
            a_internal = 0.0;
          end if;
          flange.tau = tau;
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-20, 100}, {20, -100}}, lineColor = {64, 64, 64}, fillColor = {255, 255, 255}, fillPattern = FillPattern.HorizontalCylinder, radius = 10), Rectangle(extent = {{-20, 100}, {20, -100}}, lineColor = {64, 64, 64}, fillPattern = FillPattern.None, radius = 10), Text(extent = {{-20, 92}, {20, 70}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.Solid, textString = "phi"), Text(visible = use_w, extent = {{-20, 42}, {20, 20}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.Solid, textString = "w"), Text(visible = use_a, extent = {{-20, -18}, {20, -40}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.Solid, textString = "a"), Text(extent = {{-20, -68}, {20, -90}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.Solid, textString = "tau"), Text(extent = {{-150, 150}, {150, 110}}, textString = "%name", lineColor = {0, 0, 255})}),
            Documentation(info = "<html>
<p>
Adaptor between a flange connector and a signal representation of the flange.
This component is used to provide a pure signal interface around a Rotational model
and export this model in form of an input/output block,
especially as FMU (<a href=\"https://www.fmi-standard.org\">Functional Mockup Unit</a>).
Examples of the usage of this adaptor are provided in
<a href=\"modelica://Modelica.Mechanics.Rotational.Examples.GenerationOfFMUs\">Rotational.Examples.GenerationOfFMUs</a>.
This adaptor has torque as input and angle, angular velocity and angular acceleration as output signals.
</p>
</html>"));
        end TorqueToAngleAdaptor;

        model AngleToTorqueAdaptor "Signal adaptor for a Rotational flange with torque as output and angle, speed, and optionally acceleration as inputs (especially useful for FMUs)"
          parameter Boolean use_w = true "= true, enable the input connector w (angular velocity)" annotation(
            Evaluate = true,
            HideResult = true,
            choices(checkBox = true));
          parameter Boolean use_a = true "= true, enable the input connector a (angular acceleration)" annotation(
            Evaluate = true,
            HideResult = true,
            choices(checkBox = true));
          Modelica.Mechanics.Rotational.Interfaces.Flange_b flange annotation(
            Placement(transformation(extent = {{56, -10}, {76, 10}}), iconTransformation(extent = {{10, -10}, {30, 10}})));
          Modelica.Blocks.Interfaces.RealInput phi(unit = "rad") "Angle to drive the flange" annotation(
            Placement(transformation(extent = {{-80, 60}, {-40, 100}}), iconTransformation(extent = {{-60, 60}, {-20, 100}})));
          Modelica.Blocks.Interfaces.RealInput w(unit = "rad/s") if use_w or use_a "Speed to drive the flange (w=der(phi) required)" annotation(
            Placement(transformation(extent = {{-80, 10}, {-40, 50}}), iconTransformation(extent = {{-60, 8}, {-20, 48}})));
          Modelica.Blocks.Interfaces.RealInput a(unit = "rad/s2") if use_a "Angular acceleration to drive the flange (a = der(w) required)" annotation(
            Placement(transformation(extent = {{-80, -50}, {-40, -10}}), iconTransformation(extent = {{-60, -52}, {-20, -12}})));
          Modelica.Blocks.Interfaces.RealOutput tau(unit = "N.m") "Torque needed to drive the flange according to phi, w, a" annotation(
            Placement(transformation(extent = {{-40, -90}, {-60, -70}}), iconTransformation(extent = {{-20, -90}, {-40, -70}})));
        protected
          Modelica.Mechanics.Rotational.Sources.Move move if use_a annotation(
            Placement(transformation(extent = {{10, -10}, {30, 10}})));
          Modelica.Blocks.Routing.Multiplex3 multiplex3 if use_a annotation(
            Placement(transformation(extent = {{-20, -10}, {0, 10}})));
          Modelica.Mechanics.Rotational.Sensors.TorqueSensor torqueSensor annotation(
            Placement(transformation(extent = {{36, -10}, {56, 10}})));
          Modelica.Blocks.Routing.Multiplex2 multiplex2 if use_w and not use_a annotation(
            Placement(transformation(extent = {{-20, 40}, {0, 60}})));
          Move_phi move_phi if not use_w and not use_a annotation(
            Placement(transformation(extent = {{10, 70}, {30, 90}})));
          Move_w move_w if use_w and not use_a annotation(
            Placement(transformation(extent = {{10, 40}, {30, 60}})));

          model Move_phi "Forced movement of a flange according to an angle signal"
            extends Modelica.Mechanics.Rotational.Interfaces.PartialElementaryOneFlangeAndSupport2;
            Modelica.Blocks.Interfaces.RealInput phi(final quantity = "Angle", final unit = "rad", displayUnit = "deg") "Rotation angle of flange with respect to support" annotation(
              Placement(transformation(extent = {{-140, -20}, {-100, 20}}, rotation = 0)));
          equation
            phi = flange.phi - phi_support;
            annotation(
              Documentation(info = "<html>
<p>
Flange <b>flange</b> is <b>forced</b> to move relative to flange support with a predefined motion
according to the input signal u
</p>
<pre>
    u[1]: angle of flange
    u[2]: angular velocity of flange
</pre>
<p>
The user has to guarantee that the input signals are consistent to each other,
i.e., that u[2] is the derivative of u[1].
</p>
<p>
The input signals can be provided from one of the signal generator
blocks of the block library Modelica.Blocks.Sources.
</p>
</html>"),
              Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-80, -60}, {-80, -100}}, lineColor = {0, 0, 0}, textString = "phi"), Rectangle(extent = {{-100, 20}, {100, -20}}, lineColor = {64, 64, 64}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192, 192, 192}), Line(points = {{-30, -32}, {30, -32}}, color = {0, 0, 0}), Line(points = {{0, 52}, {0, 32}}, color = {0, 0, 0}), Line(points = {{-29, 32}, {30, 32}}, color = {0, 0, 0}), Line(points = {{0, -32}, {0, -100}}, color = {0, 0, 0}), Line(points = {{30, -42}, {20, -52}}, color = {0, 0, 0}), Line(points = {{30, -32}, {10, -52}}, color = {0, 0, 0}), Line(points = {{20, -32}, {0, -52}}, color = {0, 0, 0}), Line(points = {{10, -32}, {-10, -52}}, color = {0, 0, 0}), Line(points = {{0, -32}, {-20, -52}}, color = {0, 0, 0}), Line(points = {{-10, -32}, {-30, -52}}, color = {0, 0, 0}), Line(points = {{-20, -32}, {-30, -42}}, color = {0, 0, 0}), Text(extent = {{-150, 100}, {150, 60}}, textString = "%name", lineColor = {0, 0, 255})}));
          end Move_phi;

          model Move_w "Forced movement of a flange according to an angle and speed signal"
            extends Modelica.Mechanics.Rotational.Interfaces.PartialElementaryOneFlangeAndSupport2;
            Modelica.SIunits.Angle phi "Rotation angle of flange with respect to support";
            Modelica.Blocks.Interfaces.RealInput u[2] "Angle and angular velocity of flange with respect to support as input signals" annotation(
              Placement(transformation(extent = {{-140, -20}, {-100, 20}}, rotation = 0)));
          protected
            function position
              extends Modelica.Icons.Function;
              input Real q_qd[2] "Required values for position and speed";
              input Real dummy "Just to have one input signal that should be differentiated to avoid possible problems in the Modelica tool (is not used)";
              output Real q;
            algorithm
              q := q_qd[1];
              annotation(
                derivative(noDerivative = q_qd) = position_der,
                InlineAfterIndexReduction = true);
            end position;

            function position_der
              extends Modelica.Icons.Function;
              input Real q_qd[2] "Required values for position and speed";
              input Real dummy "Just to have one input signal that should be differentiated to avoid possible problems in the Modelica tool (is not used)";
              input Real dummy_der;
              output Real qd;
            algorithm
              qd := q_qd[2];
            end position_der;
          equation
            phi = flange.phi - phi_support;
            phi = position(u, time);
            annotation(
              Documentation(info = "<html>
<p>
Flange <b>flange</b> is <b>forced</b> to move relative to flange support with a predefined motion
according to the input signals:
</p>
<pre>
    u[1]: angle of flange
    u[2]: angular velocity of flange
</pre>
<p>
The user has to guarantee that the input signals are consistent to each other,
i.e., that u[2] is the derivative of u[1].
</p>
<p>
The input signals can be provided from one of the signal generator
blocks of the block library Modelica.Blocks.Sources.
</p>
</html>"),
              Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-80, -60}, {-80, -100}}, lineColor = {0, 0, 0}, textString = "phi,w"), Rectangle(extent = {{-100, 20}, {100, -20}}, lineColor = {64, 64, 64}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192, 192, 192}), Line(points = {{-30, -32}, {30, -32}}, color = {0, 0, 0}), Line(points = {{0, 52}, {0, 32}}, color = {0, 0, 0}), Line(points = {{-29, 32}, {30, 32}}, color = {0, 0, 0}), Line(points = {{0, -32}, {0, -100}}, color = {0, 0, 0}), Line(points = {{30, -42}, {20, -52}}, color = {0, 0, 0}), Line(points = {{30, -32}, {10, -52}}, color = {0, 0, 0}), Line(points = {{20, -32}, {0, -52}}, color = {0, 0, 0}), Line(points = {{10, -32}, {-10, -52}}, color = {0, 0, 0}), Line(points = {{0, -32}, {-20, -52}}, color = {0, 0, 0}), Line(points = {{-10, -32}, {-30, -52}}, color = {0, 0, 0}), Line(points = {{-20, -32}, {-30, -42}}, color = {0, 0, 0}), Text(extent = {{-150, 100}, {150, 60}}, textString = "%name", lineColor = {0, 0, 255})}));
          end Move_w;
        equation
          connect(multiplex3.y, move.u) annotation(
            Line(points = {{1, 0}, {8, 0}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(phi, multiplex3.u1[1]) annotation(
            Line(points = {{-60, 80}, {-32, 80}, {-32, 7}, {-22, 7}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(w, multiplex3.u2[1]) annotation(
            Line(points = {{-60, 30}, {-36, 30}, {-36, 0}, {-22, 0}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(a, multiplex3.u3[1]) annotation(
            Line(points = {{-60, -30}, {-32, -30}, {-32, -7}, {-22, -7}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(flange, torqueSensor.flange_b) annotation(
            Line(points = {{66, 0}, {56, 0}}, color = {0, 0, 0}, smooth = Smooth.None));
          connect(move.flange, torqueSensor.flange_a) annotation(
            Line(points = {{30, 0}, {36, 0}}, color = {0, 0, 0}, smooth = Smooth.None));
          connect(torqueSensor.tau, tau) annotation(
            Line(points = {{38, -11}, {38, -80}, {-50, -80}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(multiplex2.y, move_w.u) annotation(
            Line(points = {{1, 50}, {8, 50}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(phi, multiplex2.u1[1]) annotation(
            Line(points = {{-60, 80}, {-46, 80}, {-46, 80}, {-32, 80}, {-32, 56}, {-22, 56}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(w, multiplex2.u2[1]) annotation(
            Line(points = {{-60, 30}, {-28, 30}, {-28, 44}, {-22, 44}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(move_w.flange, torqueSensor.flange_a) annotation(
            Line(points = {{30, 50}, {36, 50}, {36, 0}}, color = {0, 0, 0}, smooth = Smooth.None));
          connect(phi, move_phi.phi) annotation(
            Line(points = {{-60, 80}, {8, 80}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(move_phi.flange, torqueSensor.flange_a) annotation(
            Line(points = {{30, 80}, {36, 80}, {36, 0}}, color = {0, 0, 0}, smooth = Smooth.None));
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-20, 100}, {20, -100}}, lineColor = {64, 64, 64}, fillColor = {255, 255, 255}, fillPattern = FillPattern.HorizontalCylinder, radius = 10), Rectangle(extent = {{-20, 100}, {20, -100}}, lineColor = {64, 64, 64}, fillPattern = FillPattern.None, radius = 10), Text(extent = {{-20, 90}, {20, 68}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.Solid, textString = "phi"), Text(visible = use_w or use_a, extent = {{-20, 40}, {20, 18}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.Solid, textString = "w"), Text(visible = use_a, extent = {{-20, -20}, {20, -42}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.Solid, textString = "a"), Text(extent = {{-20, -70}, {20, -92}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.Solid, textString = "tau"), Text(extent = {{-150, 150}, {150, 110}}, textString = "%name", lineColor = {0, 0, 255})}),
            Documentation(info = "<html>
<p>
Adaptor between a flange connector and a signal representation of the flange.
This component is used to provide a pure signal interface around a Rotational model
and export this model in form of an input/output block,
especially as FMU (<a href=\"https://www.fmi-standard.org\">Functional Mockup Unit</a>).
Examples of the usage of this adaptor are provided in
<a href=\"modelica://Modelica.Mechanics.Rotational.Examples.GenerationOfFMUs\">Rotational.Examples.GenerationOfFMUs</a>.
This adaptor has angle, angular velocity and angular acceleration as input signals and
torque as output signal. Note, the input signals must be consistent to each other
(w=der(phi), a=der(w)).
</p>
</html>"));
        end AngleToTorqueAdaptor;
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100.0, -100.0}, {100.0, 100.0}}, initialScale = 0.1), graphics = {Rectangle(origin = {13.5135, 76.9841}, lineColor = {64, 64, 64}, fillColor = {255, 255, 255}, fillPattern = FillPattern.HorizontalCylinder, extent = {{-63.5135, -126.9841}, {36.4865, -26.9841}}, radius = 10.0), Rectangle(origin = {13.5135, 76.9841}, lineColor = {64, 64, 64}, fillPattern = FillPattern.None, extent = {{-63.5135, -126.9841}, {36.4865, -26.9841}}, radius = 10.0), Rectangle(origin = {-3.0, 73.0769}, lineColor = {64, 64, 64}, fillColor = {192, 192, 192}, fillPattern = FillPattern.HorizontalCylinder, extent = {{-87.0, -83.0769}, {-47.0, -63.0769}}), Rectangle(origin = {22.3077, 70.0}, lineColor = {64, 64, 64}, fillColor = {192, 192, 192}, fillPattern = FillPattern.HorizontalCylinder, extent = {{27.6923, -80.0}, {67.6923, -60.0}})}),
          Documentation(info = "<html>
<p>
This package contains basic components 1D mechanical rotational drive trains.
</p>
</html>"));
      end Components;

      package Sensors "Sensors to measure variables in 1D rotational mechanical components"
        extends Modelica.Icons.SensorsPackage;

        model AngleSensor "Ideal sensor to measure the absolute flange angle"
          extends Rotational.Interfaces.PartialAbsoluteSensor;
          Modelica.Blocks.Interfaces.RealOutput phi(unit = "rad", displayUnit = "deg") "Absolute angle of flange as output signal" annotation(
            Placement(transformation(extent = {{100, -10}, {120, 10}}, rotation = 0)));
        equation
          phi = flange.phi;
          annotation(
            Documentation(info = "<html>
<p>
Measures the <b>absolute angle phi</b> of a flange in an ideal
way and provides the result as output signal <b>phi</b>
(to be further processed with blocks of the Modelica.Blocks library).
</p>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{70, -30}, {120, -70}}, lineColor = {0, 0, 0}, textString = "phi")}));
        end AngleSensor;

        model SpeedSensor "Ideal sensor to measure the absolute flange angular velocity"
          extends Rotational.Interfaces.PartialAbsoluteSensor;
          Modelica.Blocks.Interfaces.RealOutput w(unit = "rad/s") "Absolute angular velocity of flange as output signal" annotation(
            Placement(transformation(extent = {{100, -10}, {120, 10}}, rotation = 0)));
        equation
          w = der(flange.phi);
          annotation(
            Documentation(info = "<html>
<p>
Measures the <b>absolute angular velocity w</b> of a flange in an ideal
way and provides the result as output signal <b>w</b>
(to be further processed with blocks of the Modelica.Blocks library).
</p>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{70, -30}, {120, -70}}, lineColor = {0, 0, 0}, textString = "w")}));
        end SpeedSensor;

        model AccSensor "Ideal sensor to measure the absolute flange angular acceleration"
          extends Rotational.Interfaces.PartialAbsoluteSensor;
          SI.AngularVelocity w "Absolute angular velocity of flange";
          Modelica.Blocks.Interfaces.RealOutput a(unit = "rad/s2") "Absolute angular acceleration of flange as output signal" annotation(
            Placement(transformation(extent = {{100, -10}, {120, 10}}, rotation = 0)));
        equation
          w = der(flange.phi);
          a = der(w);
          annotation(
            Documentation(info = "<html>
<p>
Measures the <b>absolute angular acceleration a</b> of a flange in an ideal
way and provides the result as output signal <b>a</b> (to be further processed with
blocks of the Modelica.Blocks library).
</p>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{70, -30}, {120, -70}}, lineColor = {0, 0, 0}, textString = "a")}));
        end AccSensor;

        model RelAngleSensor "Ideal sensor to measure the relative angle between two flanges"
          extends Rotational.Interfaces.PartialRelativeSensor;
          Modelica.Blocks.Interfaces.RealOutput phi_rel(unit = "rad", displayUnit = "deg") "Relative angle between two flanges (= flange_b.phi - flange_a.phi) as output signal" annotation(
            Placement(transformation(origin = {0, -110}, extent = {{10, -10}, {-10, 10}}, rotation = 90)));
        equation
          phi_rel = flange_b.phi - flange_a.phi;
          0 = flange_a.tau;
          annotation(
            Documentation(info = "<html>
<p>
Measures the <b>relative angle phi_rel</b> between two flanges
in an ideal way and provides the result as output signal <b>phi_rel</b>
(to be further processed with blocks of the Modelica.Blocks library).
</p>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100.0, -100.0}, {100.0, 100.0}}, initialScale = 0.1), graphics = {Text(extent = {{20.0, -114.0}, {160.0, -84.0}}, textString = "phi_rel"), Line(points = {{0.0, -100.0}, {0.0, -70.0}}, color = {0, 0, 127})}));
        end RelAngleSensor;

        model RelSpeedSensor "Ideal sensor to measure the relative angular velocity between two flanges"
          extends Rotational.Interfaces.PartialRelativeSensor;
          SI.Angle phi_rel "Relative angle between two flanges (flange_b.phi - flange_a.phi)";
          Modelica.Blocks.Interfaces.RealOutput w_rel(unit = "rad/s") "Relative angular velocity between two flanges (= der(flange_b.phi) - der(flange_a.phi)) as output signal" annotation(
            Placement(transformation(origin = {0, -110}, extent = {{10, -10}, {-10, 10}}, rotation = 90)));
        equation
          phi_rel = flange_b.phi - flange_a.phi;
          w_rel = der(phi_rel);
          0 = flange_a.tau;
          annotation(
            Documentation(info = "<html>
<p>
Measures the <b>relative angular velocity w_rel</b> between two flanges
in an ideal way and provides the result as output signal <b>w_rel</b>
(to be further processed with blocks of the Modelica.Blocks library).
</p>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100.0, -100.0}, {100.0, 100.0}}, initialScale = 0.1), graphics = {Text(extent = {{20.0, -118.0}, {160.0, -88.0}}, textString = "w_rel"), Line(points = {{0.0, -100.0}, {0.0, -70.0}}, color = {0, 0, 127})}));
        end RelSpeedSensor;

        model RelAccSensor "Ideal sensor to measure the relative angular acceleration between two flanges"
          extends Rotational.Interfaces.PartialRelativeSensor;
          SI.Angle phi_rel "Relative angle between two flanges (flange_b.phi - flange_a.phi)";
          SI.AngularVelocity w_rel "Relative angular velocity between two flanges";
          Modelica.Blocks.Interfaces.RealOutput a_rel(unit = "rad/s2") "Relative angular acceleration between two flanges as output signal" annotation(
            Placement(transformation(origin = {0, -110}, extent = {{10, -10}, {-10, 10}}, rotation = 90)));
        equation
          phi_rel = flange_b.phi - flange_a.phi;
          w_rel = der(phi_rel);
          a_rel = der(w_rel);
          0 = flange_a.tau;
          annotation(
            Documentation(info = "<html>
<p>
Measures the <b>relative angular acceleration a_rel</b> between two flanges
in an ideal way and provides the result as output signal <b>a_rel</b>
(to be further processed with blocks of the Modelica.Blocks library).
</p>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100.0, -100.0}, {100.0, 100.0}}, initialScale = 0.1), graphics = {Text(extent = {{20.0, -116.0}, {160.0, -86.0}}, textString = "a_rel"), Line(points = {{0.0, -100.0}, {0.0, -70.0}}, color = {0, 0, 127})}));
        end RelAccSensor;

        model TorqueSensor "Ideal sensor to measure the torque between two flanges (= flange_a.tau)"
          extends Rotational.Interfaces.PartialRelativeSensor;
          Modelica.Blocks.Interfaces.RealOutput tau(unit = "N.m") "Torque in flange flange_a and flange_b (tau = flange_a.tau = -flange_b.tau) as output signal" annotation(
            Placement(transformation(origin = {-80, -110}, extent = {{10, -10}, {-10, 10}}, rotation = 90)));
        equation
          flange_a.phi = flange_b.phi;
          flange_a.tau = tau;
          annotation(
            Documentation(info = "<html>
<p>
Measures the <b>cut-torque between two flanges</b> in an ideal way
and provides the result as output signal <b>tau</b>
(to be further processed with blocks of the Modelica.Blocks library).
</p>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100.0, -100.0}, {100.0, 100.0}}, initialScale = 0.1), graphics = {Text(extent = {{-50.0, -120.0}, {50.0, -80.0}}, textString = "tau"), Line(points = {{-80.0, -100.0}, {-80.0, 0.0}}, color = {0, 0, 127})}));
        end TorqueSensor;

        model PowerSensor "Ideal sensor to measure the power between two flanges (= flange_a.tau*der(flange_a.phi))"
          extends Rotational.Interfaces.PartialRelativeSensor;
          Modelica.Blocks.Interfaces.RealOutput power(unit = "W") "Power in flange flange_a as output signal" annotation(
            Placement(transformation(origin = {-80, -110}, extent = {{10, -10}, {-10, 10}}, rotation = 90)));
        equation
          flange_a.phi = flange_b.phi;
          power = flange_a.tau * der(flange_a.phi);
          annotation(
            Documentation(info = "<html>
<p>
Measures the <b>power between two flanges</b> in an ideal way
and provides the result as output signal <b>power</b>
(to be further processed with blocks of the Modelica.Blocks library).
</p>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100.0, -100.0}, {100.0, 100.0}}, initialScale = 0.1), graphics = {Text(extent = {{-50.0, -120.0}, {100.0, -80.0}}, textString = "power"), Line(points = {{-80.0, -100.0}, {-80.0, 0.0}}, color = {0, 0, 127})}));
        end PowerSensor;

        model MultiSensor "Ideal sensor to measure the torque and power between two flanges (= flange_a.tau*der(flange_a.phi)) and the absolute angular velocity"
          extends .Modelica.Mechanics.Rotational.Interfaces.PartialRelativeSensor;
          Modelica.Blocks.Interfaces.RealOutput power(unit = "W") "Power in flange flange_a as output signal" annotation(
            Placement(transformation(origin = {-60, -110}, extent = {{10, -10}, {-10, 10}}, rotation = 90)));
          Modelica.Blocks.Interfaces.RealOutput w(unit = "rad/s") "Absolute angular velocity of flange_a as output signal" annotation(
            Placement(transformation(extent = {{-10, 10}, {10, -10}}, rotation = 270, origin = {60, -110})));
          Modelica.Blocks.Interfaces.RealOutput tau(unit = "N.m") "Torque in flange flange_a and flange_b (tau = flange_a.tau = -flange_b.tau) as output signal" annotation(
            Placement(transformation(origin = {0, -110}, extent = {{10, -10}, {-10, 10}}, rotation = 90)));
        equation
          flange_a.phi = flange_b.phi;
          w = der(flange_a.phi);
          tau = flange_a.tau;
          power = tau * w;
          annotation(
            Documentation(info = "<html>
<p>Measures the <b>absolute angular velocity</b> of a flange_a, the <b>cut-torque</b> and <b>power</b> between two flanges in an ideal way and provides the results as output signals <b>w</b>, <b>tau</b> and <b>power</b>, respectively.
</p>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{60, -68}, {100, -96}}, lineColor = {0, 0, 0}, textString = "w"), Text(extent = {{-140, -68}, {-60, -96}}, lineColor = {0, 0, 0}, textString = "power"), Text(extent = {{-30, -68}, {30, -96}}, lineColor = {0, 0, 0}, textString = "tau"), Line(points = {{-60, -100}, {-60, -60}, {-94, -2}}, color = {0, 0, 0}), Line(points = {{0, -100}, {0, -70}}, color = {0, 0, 127}), Line(points = {{60, -100}, {60, -60}, {50, -50}}, color = {0, 0, 127})}));
        end MultiSensor;
        annotation(
          Documentation(info = "<html>
<p>
This package contains ideal sensor components that provide
the connector variables as signals for further processing with the
Modelica.Blocks library.
</p>
</html>"));
      end Sensors;

      package Sources "Sources to drive 1D rotational mechanical components"
        extends Modelica.Icons.SourcesPackage;

        model Position "Forced movement of a flange according to a reference angle signal"
          extends Modelica.Mechanics.Rotational.Interfaces.PartialElementaryOneFlangeAndSupport2;
          parameter Boolean exact = false "true/false exact treatment/filtering the input signal";
          parameter SI.Frequency f_crit = 50 "if exact=false, critical frequency of filter to filter input signal" annotation(
            Dialog(enable = not exact));
          SI.Angle phi(stateSelect = if exact then StateSelect.default else StateSelect.prefer) "Rotation angle of flange with respect to support";
          SI.AngularVelocity w(start = 0, stateSelect = if exact then StateSelect.default else StateSelect.prefer) "If exact=false, Angular velocity of flange with respect to support else dummy";
          SI.AngularAcceleration a(start = 0) "If exact=false, Angular acceleration of flange with respect to support else dummy";
          Modelica.Blocks.Interfaces.RealInput phi_ref(final quantity = "Angle", final unit = "rad", displayUnit = "deg") "Reference angle of flange with respect to support as input signal" annotation(
            Placement(transformation(extent = {{-140, -20}, {-100, 20}}, rotation = 0)));
        protected
          parameter Modelica.SIunits.AngularFrequency w_crit = 2 * Modelica.Constants.pi * f_crit "Critical frequency";
          constant Real af = 1.3617 "s coefficient of Bessel filter";
          constant Real bf = 0.6180 "s*s coefficient of Bessel filter";
        initial equation
          if not exact then
            phi = phi_ref;
          end if;
        equation
          phi = flange.phi - phi_support;
          if exact then
            phi = phi_ref;
            w = 0;
            a = 0;
          else
// Filter: a = phi_ref*s^2/(1 + (af/w_crit)*s + (bf/w_crit^2)*s^2)
            w = der(phi);
            a = der(w);
            a = ((phi_ref - phi) * w_crit - af * w) * (w_crit / bf);
          end if;
          annotation(
            Documentation(info = "<HTML>
<p>
The input signal <b>phi_ref</b> defines the <b>reference
angle</b> in [rad]. Flange <b>flange</b> is <b>forced</b>
to move according to this reference motion relative to flange support. According to parameter
<b>exact</b> (default = <b>false</b>), this is done in the following way:
<ol>
<li><b>exact=true</b><br>
    The reference angle is treated <b>exactly</b>. This is only possible, if
    the input signal is defined by an analytical function which can be
    differentiated at least twice. If this prerequisite is fulfilled,
    the Modelica translator will differentiate the input signal twice
    in order to compute the reference acceleration of the flange.</li>
<li><b>exact=false</b><br>
    The reference angle is <b>filtered</b> and the second derivative
    of the filtered curve is used to compute the reference acceleration
    of the flange. This second derivative is <b>not</b> computed by
    numerical differentiation but by an appropriate realization of the
    filter. For filtering, a second order Bessel filter is used.
    The critical frequency (also called cut-off frequency) of the
    filter is defined via parameter <b>f_crit</b> in [Hz]. This value
    should be selected in such a way that it is higher as the essential
    low frequencies in the signal.</li>
</ol>
<p>
The input signal can be provided from one of the signal generator
blocks of the block library Modelica.Blocks.Sources.
</p>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100.0, -100.0}, {100.0, 100.0}}, initialScale = 0.1), graphics = {Rectangle(lineColor = {64, 64, 64}, fillColor = {192, 192, 192}, fillPattern = FillPattern.HorizontalCylinder, extent = {{-100.0, -20.0}, {100.0, 20.0}}), Line(points = {{-30.0, -32.0}, {30.0, -32.0}}), Line(points = {{0.0, 52.0}, {0.0, 32.0}}), Line(points = {{-29.0, 32.0}, {30.0, 32.0}}), Line(points = {{0.0, -32.0}, {0.0, -100.0}}), Line(points = {{30.0, -42.0}, {20.0, -52.0}}), Line(points = {{30.0, -32.0}, {10.0, -52.0}}), Line(points = {{20.0, -32.0}, {0.0, -52.0}}), Line(points = {{10.0, -32.0}, {-10.0, -52.0}}), Line(points = {{0.0, -32.0}, {-20.0, -52.0}}), Line(points = {{-10.0, -32.0}, {-30.0, -52.0}}), Line(points = {{-20.0, -32.0}, {-30.0, -42.0}}), Text(extent = {{-172.0, -90.0}, {-56.0, -56.0}}, textString = "phi_ref"), Text(lineColor = {0, 0, 255}, extent = {{-150.0, 60.0}, {150.0, 100.0}}, textString = "%name"), Text(extent = {{30.0, -62.0}, {146.0, -28.0}}, textString = "exact="), Text(extent = {{30.0, -98.0}, {146.0, -64.0}}, textString = "%exact")}));
        end Position;

        model Speed "Forced movement of a flange according to a reference angular velocity signal"
          extends Modelica.Mechanics.Rotational.Interfaces.PartialElementaryOneFlangeAndSupport2;
          parameter Boolean exact = false "true/false exact treatment/filtering the input signal";
          parameter SI.Frequency f_crit = 50 "if exact=false, critical frequency of filter to filter input signal" annotation(
            Dialog(enable = not exact));
          SI.Angle phi(start = 0, fixed = true, stateSelect = StateSelect.prefer) "Rotation angle of flange with respect to support";
          SI.AngularVelocity w(stateSelect = if exact then StateSelect.default else StateSelect.prefer) "Angular velocity of flange with respect to support";
          SI.AngularAcceleration a "If exact=false, angular acceleration of flange with respect to support else dummy";
          Modelica.Blocks.Interfaces.RealInput w_ref(unit = "rad/s") "Reference angular velocity of flange with respect to support as input signal" annotation(
            Placement(transformation(extent = {{-140, -20}, {-100, 20}}, rotation = 0)));
        protected
          parameter Modelica.SIunits.AngularFrequency w_crit = 2 * Modelica.Constants.pi * f_crit "Critical frequency";
        initial equation
          if not exact then
            w = w_ref;
          end if;
        equation
          phi = flange.phi - phi_support;
          w = der(phi);
          if exact then
            w = w_ref;
            a = 0;
          else
// Filter: a = w_ref/(1 + (1/w_crit)*s)
            a = der(w);
            a = (w_ref - w) * w_crit;
          end if;
          annotation(
            Documentation(info = "<html>
<p>
The input signal <b>w_ref</b> defines the <b>reference
speed</b> in [rad/s]. Flange <b>flange</b> is <b>forced</b>
to move relative to flange support according to this reference motion. According to parameter
<b>exact</b> (default = <b>false</b>), this is done in the following way:
<ol>
<li><b>exact=true</b><br>
    The reference speed is treated <b>exactly</b>. This is only possible, if
    the input signal is defined by an analytical function which can be
    differentiated at least once. If this prerequisite is fulfilled,
    the Modelica translator will differentiate the input signal once
    in order to compute the reference acceleration of the flange.</li>
<li><b>exact=false</b><br>
    The reference angle is <b>filtered</b> and the second derivative
    of the filtered curve is used to compute the reference acceleration
    of the flange. This second derivative is <b>not</b> computed by
    numerical differentiation but by an appropriate realization of the
    filter. For filtering, a first order filter is used.
    The critical frequency (also called cut-off frequency) of the
    filter is defined via parameter <b>f_crit</b> in [Hz]. This value
    should be selected in such a way that it is higher as the essential
    low frequencies in the signal.</li>
</ol>
<p>
The input signal can be provided from one of the signal generator
blocks of the block library Modelica.Blocks.Sources.
</p>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100.0, -100.0}, {100.0, 100.0}}, initialScale = 0.1), graphics = {Rectangle(lineColor = {64, 64, 64}, fillColor = {192, 192, 192}, fillPattern = FillPattern.HorizontalCylinder, extent = {{-100.0, -20.0}, {100.0, 20.0}}), Line(points = {{-30.0, -32.0}, {30.0, -32.0}}), Line(points = {{0.0, 52.0}, {0.0, 32.0}}), Line(points = {{-29.0, 32.0}, {30.0, 32.0}}), Line(points = {{0.0, -32.0}, {0.0, -100.0}}), Line(points = {{-10.0, -32.0}, {-30.0, -52.0}}), Line(points = {{0.0, -32.0}, {-20.0, -52.0}}), Line(points = {{10.0, -32.0}, {-10.0, -52.0}}), Line(points = {{20.0, -32.0}, {0.0, -52.0}}), Line(points = {{-20.0, -32.0}, {-30.0, -42.0}}), Line(points = {{30.0, -32.0}, {10.0, -52.0}}), Line(points = {{30.0, -42.0}, {20.0, -52.0}}), Text(extent = {{-158.0, -78.0}, {-54.0, -44.0}}, textString = "w_ref"), Text(lineColor = {0, 0, 255}, extent = {{-0.0, 60.0}, {0.0, 120.0}}, textString = "%name"), Text(extent = {{30.0, -60.0}, {146.0, -26.0}}, textString = "exact="), Text(extent = {{30.0, -96.0}, {146.0, -62.0}}, textString = "%exact")}));
        end Speed;

        model Accelerate "Forced movement of a flange according to an acceleration signal"
          extends Modelica.Mechanics.Rotational.Interfaces.PartialElementaryOneFlangeAndSupport2;
          SI.Angle phi(start = 0, fixed = true, stateSelect = StateSelect.prefer) "Rotation angle of flange with respect to support";
          SI.AngularVelocity w(start = 0, fixed = true, stateSelect = StateSelect.prefer) "Angular velocity of flange with respect to support";
          SI.AngularAcceleration a "Angular acceleration of flange with respect to support";
          Modelica.Blocks.Interfaces.RealInput a_ref(unit = "rad/s2") "Absolute angular acceleration of flange with respect to support as input signal" annotation(
            Placement(transformation(extent = {{-140, -20}, {-100, 20}}, rotation = 0)));
        equation
          phi = flange.phi - phi_support;
          w = der(phi);
          a = der(w);
          a = a_ref;
          annotation(
            Documentation(info = "<html>
<p>
The input signal <b>a</b> defines an <b>angular acceleration</b>
in [rad/s2]. Flange <b>flange</b> is <b>forced</b> to move relative to flange support with
this acceleration. The angular velocity <b>w</b> and the rotation angle
<b>phi</b> of the flange are automatically determined by integration of
the acceleration.
</p>
<p>
The input signal can be provided from one of the signal generator
blocks of the block library Modelica.Blocks.Sources.
</p>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100.0, -100.0}, {100.0, 100.0}}, initialScale = 0.1), graphics = {Rectangle(lineColor = {64, 64, 64}, fillColor = {192, 192, 192}, fillPattern = FillPattern.HorizontalCylinder, extent = {{-100.0, -20.0}, {100.0, 20.0}}), Line(points = {{-30.0, -32.0}, {30.0, -32.0}}), Line(points = {{0.0, 52.0}, {0.0, 32.0}}), Line(points = {{-29.0, 32.0}, {30.0, 32.0}}), Line(points = {{0.0, -32.0}, {0.0, -100.0}}), Line(points = {{30.0, -42.0}, {20.0, -52.0}}), Line(points = {{30.0, -32.0}, {10.0, -52.0}}), Line(points = {{20.0, -32.0}, {0.0, -52.0}}), Line(points = {{10.0, -32.0}, {-10.0, -52.0}}), Line(points = {{0.0, -32.0}, {-20.0, -52.0}}), Line(points = {{-10.0, -32.0}, {-30.0, -52.0}}), Line(points = {{-20.0, -32.0}, {-30.0, -42.0}}), Text(extent = {{-144.0, -86.0}, {-46.0, -50.0}}, textString = "a_ref"), Text(lineColor = {0, 0, 255}, extent = {{-150.0, 60.0}, {150.0, 100.0}}, textString = "%name")}));
        end Accelerate;

        model Move "Forced movement of a flange according to an angle, speed and angular acceleration signal"
          extends Modelica.Mechanics.Rotational.Interfaces.PartialElementaryOneFlangeAndSupport2;
          Modelica.SIunits.Angle phi "Rotation angle of flange with respect to support";
          Modelica.Blocks.Interfaces.RealInput u[3] "Angle, angular velocity and angular acceleration of flange with respect to support as input signals" annotation(
            Placement(transformation(extent = {{-140, -20}, {-100, 20}}, rotation = 0)));
        protected
          function position
            extends Modelica.Icons.Function;
            input Real q_qd_qdd[3] "Required values for position, speed, acceleration";
            input Real dummy "Just to have one input signal that should be differentiated to avoid possible problems in the Modelica tool (is not used)";
            output Real q;
          algorithm
            q := q_qd_qdd[1];
            annotation(
              derivative(noDerivative = q_qd_qdd) = position_der,
              Inline = false);
          end position;

          function position_der
            extends Modelica.Icons.Function;
            input Real q_qd_qdd[3] "Required values for position, speed, acceleration";
            input Real dummy "Just to have one input signal that should be differentiated to avoid possible problems in the Modelica tool (is not used)";
            input Real dummy_der;
            output Real qd;
          algorithm
            qd := q_qd_qdd[2];
            annotation(
              derivative(noDerivative = q_qd_qdd, order = 2) = position_der2,
              Inline = false);
          end position_der;

          function position_der2
            extends Modelica.Icons.Function;
            input Real q_qd_qdd[3] "Required values for position, speed, acceleration";
            input Real dummy "Just to have one input signal that should be differentiated to avoid possible problems in the Modelica tool (is not used)";
            input Real dummy_der;
            input Real dummy_der2;
            output Real qdd;
          algorithm
            qdd := q_qd_qdd[3];
          end position_der2;
        equation
          phi = flange.phi - phi_support;
          phi = position(u, time);
          annotation(
            Documentation(info = "<html>
<p>
Flange <b>flange</b> is <b>forced</b> to move relative to flange support with a predefined motion
according to the input signals:
</p>
<pre>
    u[1]: angle of flange
    u[2]: angular velocity of flange
    u[3]: angular acceleration of flange
</pre>
<p>
The user has to guarantee that the input signals are consistent to each other,
i.e., that u[2] is the derivative of u[1] and that
u[3] is the derivative of u[2]. There are, however,
also applications where by purpose these conditions do not hold. For example,
if only the position dependent terms of a mechanical system shall be
calculated, one may provide angle = angle(t) and set the angular velocity
and the angular acceleration to zero.
</p>
<p>
The input signals can be provided from one of the signal generator
blocks of the block library Modelica.Blocks.Sources.
</p>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100.0, -100.0}, {100.0, 100.0}}, initialScale = 0.1), graphics = {Text(extent = {{-80.0, -100.0}, {-80.0, -60.0}}, textString = "phi,w,a"), Rectangle(lineColor = {64, 64, 64}, fillColor = {192, 192, 192}, fillPattern = FillPattern.HorizontalCylinder, extent = {{-100.0, -20.0}, {100.0, 20.0}}), Line(points = {{-30.0, -32.0}, {30.0, -32.0}}), Line(points = {{0.0, 52.0}, {0.0, 32.0}}), Line(points = {{-29.0, 32.0}, {30.0, 32.0}}), Line(points = {{0.0, -32.0}, {0.0, -100.0}}), Line(points = {{30.0, -42.0}, {20.0, -52.0}}), Line(points = {{30.0, -32.0}, {10.0, -52.0}}), Line(points = {{20.0, -32.0}, {0.0, -52.0}}), Line(points = {{10.0, -32.0}, {-10.0, -52.0}}), Line(points = {{0.0, -32.0}, {-20.0, -52.0}}), Line(points = {{-10.0, -32.0}, {-30.0, -52.0}}), Line(points = {{-20.0, -32.0}, {-30.0, -42.0}}), Text(lineColor = {0, 0, 255}, extent = {{-150.0, 60.0}, {150.0, 100.0}}, textString = "%name")}));
        end Move;

        model Torque "Input signal acting as external torque on a flange"
          extends Modelica.Mechanics.Rotational.Interfaces.PartialElementaryOneFlangeAndSupport2;
          Modelica.Blocks.Interfaces.RealInput tau(unit = "N.m") "Accelerating torque acting at flange (= -flange.tau)" annotation(
            Placement(transformation(extent = {{-140, -20}, {-100, 20}}, rotation = 0)));
        equation
          flange.tau = -tau;
          annotation(
            Documentation(info = "<HTML>
<p>
The input signal <b>tau</b> defines an external
torque in [Nm] which acts (with negative sign) at
a flange connector, i.e., the component connected to this
flange is driven by torque <b>tau</b>.</p>
<p>
The input signal can be provided from one of the signal generator
blocks of Modelica.Blocks.Sources.
</p>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-150, 110}, {150, 70}}, textString = "%name", lineColor = {0, 0, 255}), Text(extent = {{-62, -29}, {-141, -70}}, lineColor = {0, 0, 0}, textString = "tau"), Line(points = {{-88, 0}, {-64, 30}, {-36, 52}, {-2, 62}, {28, 56}, {48, 44}, {64, 28}, {76, 14}, {86, 0}}, color = {0, 0, 0}, thickness = 0.5, smooth = Smooth.Bezier), Polygon(points = {{86, 0}, {66, 58}, {37, 27}, {86, 0}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Line(points = {{-30, -30}, {30, -30}}, color = {0, 0, 0}), Line(points = {{0, -30}, {0, -101}}, color = {0, 0, 0}), Line(points = {{-30, -50}, {-10, -30}}, color = {0, 0, 0}), Line(points = {{-10, -50}, {10, -30}}, color = {0, 0, 0}), Line(points = {{10, -50}, {30, -30}}, color = {0, 0, 0}), Line(points = {{-54, -42}, {-38, -28}, {-16, -16}, {4, -14}, {22, -18}, {36, -26}, {48, -36}, {56, -46}, {64, -58}}, color = {0, 0, 0}, smooth = Smooth.Bezier), Polygon(points = {{-61, -66}, {-44, -42}, {-58, -36}, {-61, -66}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid)}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{14, 86}, {82, 73}}, lineColor = {128, 128, 128}, textString = "rotation axis"), Polygon(points = {{10, 80}, {-10, 85}, {-10, 75}, {10, 80}}, lineColor = {128, 128, 128}, fillColor = {128, 128, 128}, fillPattern = FillPattern.Solid), Line(points = {{-80, 80}, {-9, 80}}, color = {128, 128, 128}), Line(points = {{-88, 0}, {-64, 30}, {-36, 52}, {-2, 62}, {28, 56}, {48, 44}, {64, 28}, {76, 14}, {80, 10}}, color = {0, 0, 0}, thickness = 0.5, smooth = Smooth.Bezier), Polygon(points = {{86, 0}, {66, 58}, {38, 28}, {86, 0}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid)}));
        end Torque;

        model Torque2 "Input signal acting as torque on two flanges"
          extends Rotational.Interfaces.PartialTwoFlanges;
          Modelica.Blocks.Interfaces.RealInput tau(unit = "N.m") "Torque driving the two flanges (a positive value accelerates the flange)" annotation(
            Placement(transformation(origin = {0, 40}, extent = {{-20, -20}, {20, 20}}, rotation = 270)));
        equation
          flange_a.tau = tau;
          flange_b.tau = -tau;
          annotation(
            Documentation(info = "<HTML>
<p>
The input signal <b>tau</b> defines an external
torque in [Nm] which acts at both flange connectors,
i.e., the components connected to these flanges are driven by torque <b>tau</b>.</p>
<p>The input signal can be provided from one of the signal generator
blocks of Modelica.Blocks.Sources.
</p>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-150, -40}, {150, -80}}, textString = "%name", lineColor = {0, 0, 255}), Polygon(points = {{-78, 24}, {-69, 17}, {-89, 0}, {-78, 24}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Line(points = {{-74, 20}, {-70, 23}, {-65, 26}, {-60, 28}, {-56, 29}, {-50, 30}, {-41, 30}, {-35, 29}, {-31, 28}, {-26, 26}, {-21, 23}, {-17, 20}, {-13, 15}, {-10, 9}}, color = {0, 0, 0}, thickness = 0.5, smooth = Smooth.Bezier), Line(points = {{74, 20}, {70, 23}, {65, 26}, {60, 28}, {56, 29}, {50, 30}, {41, 30}, {35, 29}, {31, 28}, {26, 26}, {21, 23}, {17, 20}, {13, 15}, {10, 9}}, color = {0, 0, 0}, thickness = 0.5, smooth = Smooth.Bezier), Polygon(points = {{89, 0}, {78, 24}, {69, 17}, {89, 0}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid)}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{15, -71}, {83, -84}}, lineColor = {128, 128, 128}, textString = "rotation axis"), Polygon(points = {{11, -77}, {-9, -72}, {-9, -82}, {11, -77}}, lineColor = {128, 128, 128}, fillColor = {128, 128, 128}, fillPattern = FillPattern.Solid), Line(points = {{-79, -77}, {-8, -77}}, color = {128, 128, 128}), Line(points = {{-75, 20}, {-71, 23}, {-66, 26}, {-61, 28}, {-57, 29}, {-51, 30}, {-42, 30}, {-36, 29}, {-32, 28}, {-27, 26}, {-22, 23}, {-18, 20}, {-14, 15}, {-11, 9}}, color = {0, 0, 0}, thickness = 0.5, smooth = Smooth.Bezier), Polygon(points = {{-79, 24}, {-70, 17}, {-90, 0}, {-79, 24}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Line(points = {{73, 20}, {69, 23}, {64, 26}, {59, 28}, {55, 29}, {49, 30}, {40, 30}, {34, 29}, {30, 28}, {25, 26}, {20, 23}, {16, 20}, {12, 15}, {9, 9}}, color = {0, 0, 0}, thickness = 0.5, smooth = Smooth.Bezier), Polygon(points = {{88, 0}, {77, 24}, {68, 17}, {88, 0}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid)}));
        end Torque2;

        model LinearSpeedDependentTorque "Linear dependency of torque versus speed"
          extends Modelica.Mechanics.Rotational.Interfaces.PartialTorque;
          parameter Modelica.SIunits.Torque tau_nominal "Nominal torque (if negative, torque is acting as load)";
          parameter Boolean TorqueDirection = true "Same direction of torque in both directions of rotation";
          parameter Modelica.SIunits.AngularVelocity w_nominal(min = Modelica.Constants.eps) "Nominal speed";
          Modelica.SIunits.AngularVelocity w "Angular velocity of flange with respect to support (= der(phi))";
          Modelica.SIunits.Torque tau "Accelerating torque acting at flange (= -flange.tau)";
        equation
          w = der(phi);
          tau = -flange.tau;
          if TorqueDirection then
            tau = tau_nominal * abs(w / w_nominal);
          else
            tau = tau_nominal * (w / w_nominal);
          end if;
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-100, -100}, {100, 100}}, color = {0, 0, 127})}),
            Documentation(info = "<HTML>
<p>
Model of torque, linearly dependent on angular velocity of flange.<br>
Parameter TorqueDirection chooses whether direction of torque is the same in both directions of rotation or not.
</p>
</HTML>"));
        end LinearSpeedDependentTorque;

        model QuadraticSpeedDependentTorque "Quadratic dependency of torque versus speed"
          extends Modelica.Mechanics.Rotational.Interfaces.PartialTorque;
          parameter Modelica.SIunits.Torque tau_nominal "Nominal torque (if negative, torque is acting as load)";
          parameter Boolean TorqueDirection = true "Same direction of torque in both directions of rotation";
          parameter Modelica.SIunits.AngularVelocity w_nominal(min = Modelica.Constants.eps) "Nominal speed";
          Modelica.SIunits.AngularVelocity w "Angular velocity of flange with respect to support (= der(phi))";
          Modelica.SIunits.Torque tau "Accelerating torque acting at flange (= -flange.tau)";
        equation
          w = der(phi);
          tau = -flange.tau;
          if TorqueDirection then
            tau = tau_nominal * (w / w_nominal) ^ 2;
          else
            tau = tau_nominal * smooth(1, if w >= 0 then (w / w_nominal) ^ 2 else -(w / w_nominal) ^ 2);
          end if;
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100.0, -100.0}, {100.0, 100.0}}, initialScale = 0.1), graphics = {Line(points = {{-100.0, -100.0}, {-80.0, -98.0}, {-60.0, -92.0}, {-40.0, -82.0}, {-20.0, -68.0}, {0.0, -50.0}, {20.0, -28.0}, {40.0, -2.0}, {60.0, 28.0}, {80.0, 62.0}, {100.0, 100.0}}, color = {0, 0, 127}, smooth = Smooth.Bezier)}),
            Documentation(info = "<HTML>
<p>
Model of torque, quadratic dependent on angular velocity of flange.<br>
Parameter TorqueDirection chooses whether direction of torque is the same in both directions of rotation or not.
</p>
</HTML>"));
        end QuadraticSpeedDependentTorque;

        model ConstantTorque "Constant torque, not dependent on speed"
          extends Rotational.Interfaces.PartialTorque;
          parameter Modelica.SIunits.Torque tau_constant "Constant torque (if negative, torque is acting as load)";
          Modelica.SIunits.Torque tau "Accelerating torque acting at flange (= -flange.tau)";
        equation
          tau = -flange.tau;
          tau = tau_constant;
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-98, 0}, {100, 0}}, color = {0, 0, 127}), Text(extent = {{-124.0, -40.0}, {120.0, -16.0}}, textString = "%tau_constant")}),
            Documentation(info = "<HTML>
<p>
Model of constant torque, not dependent on angular velocity of flange.<br>
Positive torque acts accelerating.
</p>
</HTML>"));
        end ConstantTorque;

        model ConstantSpeed "Constant speed, not dependent on torque"
          extends Modelica.Mechanics.Rotational.Interfaces.PartialTorque;
          Modelica.SIunits.AngularVelocity w "Angular velocity of flange with respect to support (= der(phi))";
          parameter Modelica.SIunits.AngularVelocity w_fixed "Fixed speed";
        equation
          w = der(phi);
          w = w_fixed;
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}, initialScale = 0.1), graphics = {Line(points = {{0, -100}, {0, 100}}, color = {0, 0, 127}), Text(extent = {{-116.0, -40.0}, {128.0, -16.0}}, textString = "%w_fixed")}),
            Documentation(info = "<HTML>
<p>
Model of <b>fixed</b> angular velocity of flange, not dependent on torque.
</p>
</HTML>"));
        end ConstantSpeed;

        model TorqueStep "Constant torque, not dependent on speed"
          extends Modelica.Mechanics.Rotational.Interfaces.PartialTorque;
          parameter Modelica.SIunits.Torque stepTorque(start = 1) "Height of torque step (if negative, torque is acting as load)";
          parameter Modelica.SIunits.Torque offsetTorque(start = 0) "Offset of torque";
          parameter Modelica.SIunits.Time startTime = 0 "Torque = offset for time < startTime";
          Modelica.SIunits.Torque tau "Accelerating torque acting at flange (= -flange.tau)";
        equation
          tau = -flange.tau;
          tau = offsetTorque + (if time < startTime then 0 else stepTorque);
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}, initialScale = 0.1), graphics = {Line(points = {{-80, -60}, {0, -60}, {0, 60}, {80, 60}}, color = {0, 0, 127}), Text(extent = {{0.0, -60.0}, {100.0, -40.0}}, lineColor = {0, 0, 255}, textString = "time")}),
            Documentation(info = "<HTML>
<p>
Model of a torque step at time .<br>
Positive torque acts accelerating.
</p>
</HTML>"));
        end TorqueStep;
        annotation(
          Documentation(info = "<html>
<p>
This package contains ideal sources to drive 1D mechanical rotational drive trains.
</p>
</html>"));
      end Sources;

      package Interfaces "Connectors and partial models for 1D rotational mechanical components"
        extends Modelica.Icons.InterfacesPackage;

        connector Flange_a "1-dim. rotational flange of a shaft (filled square icon)"
          SI.Angle phi "Absolute rotation angle of flange";
          flow SI.Torque tau "Cut torque in the flange";
          annotation(
            defaultComponentName = "flange_a",
            Documentation(info = "<html>
<p>
This is a connector for 1-dim. rotational mechanical systems and models
the mechanical flange of a shaft. The following variables are defined in this connector:
</p>

<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"> <b>phi</b></td>
      <td valign=\"top\"> Absolute rotation angle of the shaft flange in [rad] </td>
  </tr>
  <tr><td valign=\"top\"> <b>tau</b></td>
      <td valign=\"top\"> Cut-torque in the shaft flange in [Nm] </td>
  </tr>
</table>

<p>
There is a second connector for flanges: Flange_b. The connectors
Flange_a and Flange_b are completely identical. There is only a difference
in the icons, in order to easier identify a flange variable in a diagram.
For a discussion on the actual direction of the cut-torque tau and
of the rotation angle, see section
<a href=\"modelica://Modelica.Mechanics.Rotational.UsersGuide.SignConventions\">Sign Conventions</a>
in the user's guide of Rotational.
</p>

<p>
If needed, the absolute angular velocity w and the
absolute angular acceleration a of the flange can be determined by
differentiation of the flange angle phi:
</p>
<pre>
     w = der(phi);    a = der(w)
</pre>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-100, 100}, {100, -100}}, lineColor = {0, 0, 0}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid)}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-160, 90}, {40, 50}}, lineColor = {0, 0, 0}, textString = "%name"), Ellipse(extent = {{-40, 40}, {40, -40}}, lineColor = {0, 0, 0}, fillColor = {135, 135, 135}, fillPattern = FillPattern.Solid)}));
        end Flange_a;

        connector Flange_b "1-dim. rotational flange of a shaft (non-filled square icon)"
          SI.Angle phi "Absolute rotation angle of flange";
          flow SI.Torque tau "Cut torque in the flange";
          annotation(
            defaultComponentName = "flange_b",
            Documentation(info = "<html>
<p>
This is a connector for 1-dim. rotational mechanical systems and models
the mechanical flange of a shaft. The following variables are defined in this connector:
</p>

<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"> <b>phi</b></td>
      <td valign=\"top\"> Absolute rotation angle of the shaft flange in [rad] </td>
  </tr>
  <tr><td valign=\"top\"> <b>tau</b></td>
      <td valign=\"top\"> Cut-torque in the shaft flange in [Nm] </td>
  </tr>
</table>

<p>
There is a second connector for flanges: Flange_a. The connectors
Flange_a and Flange_b are completely identical. There is only a difference
in the icons, in order to easier identify a flange variable in a diagram.
For a discussion on the actual direction of the cut-torque tau and
of the rotation angle, see section
<a href=\"modelica://Modelica.Mechanics.Rotational.UsersGuide.SignConventions\">Sign Conventions</a>
in the user's guide of Rotational.
</p>

<p>
If needed, the absolute angular velocity w and the
absolute angular acceleration a of the flange can be determined by
differentiation of the flange angle phi:
</p>
<pre>
     w = der(phi);    a = der(w)
</pre>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-98, 100}, {102, -100}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid)}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-40, 40}, {40, -40}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-40, 90}, {160, 50}}, lineColor = {0, 0, 0}, textString = "%name")}));
        end Flange_b;

        connector Support "Support/housing of a 1-dim. rotational shaft"
          SI.Angle phi "Absolute rotation angle of the support/housing";
          flow SI.Torque tau "Reaction torque in the support/housing";
          annotation(
            Documentation(info = "<html>
<p>
This is a connector for 1-dim. rotational mechanical systems and models
the support or housing of a shaft. The following variables are defined in this connector:
</p>

<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"> <b>phi</b></td>
      <td valign=\"top\"> Absolute rotation angle of the support/housing in [rad] </td>
  </tr>
  <tr><td valign=\"top\"> <b>tau</b></td>
      <td valign=\"top\"> Reaction torque in the support/housing in [Nm] </td>
  </tr>
</table>

<p>
The support connector is usually defined as conditional connector.
It is most convenient to utilize it
</p>

<ul>
<li> For models to be build graphically (i.e., the model is build up by drag-and-drop
     from elementary components):<br>
     <a href=\"modelica://Modelica.Mechanics.Rotational.Interfaces.PartialOneFlangeAndSupport\">PartialOneFlangeAndSupport</a>,<br>
     <a href=\"modelica://Modelica.Mechanics.Rotational.Interfaces.PartialTwoFlangesAndSupport\">PartialTwoFlangesAndSupport</a>, <br> &nbsp; </li>

<li> For models to be build textually (i.e., elementary models):<br>
     <a href=\"modelica://Modelica.Mechanics.Rotational.Interfaces.PartialElementaryOneFlangeAndSupport\">PartialElementaryOneFlangeAndSupport</a>,<br>
     <a href=\"modelica://Modelica.Mechanics.Rotational.Interfaces.PartialElementaryTwoFlangesAndSupport\">PartialElementaryTwoFlangesAndSupport</a>,<br>
     <a href=\"modelica://Modelica.Mechanics.Rotational.Interfaces.PartialElementaryRotationalToTranslational\">PartialElementaryRotationalToTranslational</a>.</li>
</ul>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}, initialScale = 0.1), graphics = {Ellipse(extent = {{-100, 100}, {100, -100}}, lineColor = {0, 0, 0}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-150, 150}, {150, -150}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-100, 100}, {100, -100}}, lineColor = {0, 0, 0}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid)}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}, initialScale = 0.1), graphics = {Rectangle(extent = {{-60, 60}, {60, -60}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Text(extent = {{-160, 100}, {40, 60}}, lineColor = {0, 0, 0}, textString = "%name"), Ellipse(extent = {{-40, 40}, {40, -40}}, lineColor = {0, 0, 0}, fillColor = {135, 135, 135}, fillPattern = FillPattern.Solid)}));
        end Support;

        model InternalSupport "Adapter model to utilize conditional support connector"
          input Modelica.SIunits.Torque tau "External support torque (must be computed via torque balance in model where InternalSupport is used; = flange.tau)";
          Modelica.SIunits.Angle phi "External support angle (= flange.phi)";
          Flange_a flange "Internal support flange (must be connected to the conditional support connector for useSupport=true and to conditional fixed model for useSupport=false)" annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}})));
        equation
          flange.tau = tau;
          flange.phi = phi;
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-20, 20}, {20, -20}}, lineColor = {135, 135, 135}, fillColor = {175, 175, 175}, fillPattern = FillPattern.Solid), Text(extent = {{-200, 80}, {200, 40}}, lineColor = {0, 0, 255}, textString = "%name")}),
            Documentation(info = "<html>
<p>
This is an adapter model to utilize a conditional support connector
in an elementary component, i.e., where the component equations are
defined textually:
</p>

<ul>
<li> If <i>useSupport = true</i>, the flange has to be connected to the conditional
     support connector.</li>
<li> If <i>useSupport = false</i>, the flange has to be connected to the conditional
     fixed model.</li>
</ul>

<p>
Variable <b>tau</b> is defined as <b>input</b> and must be provided when using
this component as a modifier (computed via a torque balance in
the model where InternalSupport is used). Usually, model InternalSupport is
utilized via the partial models:
</p>

<blockquote>
<a href=\"modelica://Modelica.Mechanics.Rotational.Interfaces.PartialElementaryOneFlangeAndSupport\">
PartialElementaryOneFlangeAndSupport</a>,<br>
<a href=\"modelica://Modelica.Mechanics.Rotational.Interfaces.PartialElementaryTwoFlangesAndSupport\">
PartialElementaryTwoFlangesAndSupport</a>,<br>
<a href=\"modelica://Modelica.Mechanics.Rotational.Interfaces.PartialElementaryRotationalToTranslational\">
PartialElementaryRotationalToTranslational</a>.
</blockquote>

<p>
Note, the support angle can always be accessed as internalSupport.phi, and
the support torque can always be accessed as internalSupport.tau.
</p>
</html>"));
        end InternalSupport;

        partial model PartialTwoFlanges "Partial model for a component with two rotational 1-dim. shaft flanges"
          Flange_a flange_a "Flange of left shaft" annotation(
            Placement(transformation(extent = {{-110, -10}, {-90, 10}}, rotation = 0)));
          Flange_b flange_b "Flange of right shaft" annotation(
            Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
          annotation(
            Documentation(info = "<html>
<p>
This is a 1-dim. rotational component with two flanges.
It is used e.g., to build up parts of a drive train consisting
of several components.
</p>
</html>"));
        end PartialTwoFlanges;

        partial model PartialOneFlangeAndSupport "Partial model for a component with one rotational 1-dim. shaft flange and a support used for graphical modeling, i.e., the model is build up by drag-and-drop from elementary components"
          parameter Boolean useSupport = false "= true, if support flange enabled, otherwise implicitly grounded" annotation(
            Evaluate = true,
            HideResult = true,
            choices(checkBox = true));
          Flange_b flange "Flange of shaft" annotation(
            Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
          Support support if useSupport "Support/housing of component" annotation(
            Placement(transformation(extent = {{-10, -110}, {10, -90}})));
        protected
          Support internalSupport "Internal support/housing of component (either connected to support, if useSupport=true, or connected to fixed, if useSupport=false)" annotation(
            Placement(transformation(extent = {{-3, -83}, {3, -77}})));
          Components.Fixed fixed if not useSupport "Fixed support/housing, if not useSupport" annotation(
            Placement(transformation(extent = {{10, -94}, {30, -74}})));
        equation
          connect(support, internalSupport) annotation(
            Line(points = {{0, -100}, {0, -80}}, color = {0, 0, 0}, smooth = Smooth.None));
          connect(internalSupport, fixed.flange) annotation(
            Line(points = {{0, -80}, {20, -80}, {20, -84}}, color = {0, 0, 0}, smooth = Smooth.None));
          annotation(
            Documentation(info = "<html>
<p>
This is a 1-dim. rotational component with one flange and a support/housing.
It is used e.g., to build up parts of a drive train graphically consisting
of several components.
</p>

<p>
If <i>useSupport=true</i>, the support connector is conditionally enabled
and needs to be connected.<br>
If <i>useSupport=false</i>, the support connector is conditionally disabled
and instead the component is internally fixed to ground.
</p>
</html>"),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-38, -98}, {-6, -96}}, lineColor = {95, 95, 95}, textString = "(if useSupport)"), Text(extent = {{21, -95}, {61, -96}}, lineColor = {95, 95, 95}, textString = "(if not useSupport)")}),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(visible = not useSupport, points = {{-50, -120}, {-30, -100}}, color = {0, 0, 0}), Line(visible = not useSupport, points = {{-30, -120}, {-10, -100}}, color = {0, 0, 0}), Line(visible = not useSupport, points = {{-10, -120}, {10, -100}}, color = {0, 0, 0}), Line(visible = not useSupport, points = {{10, -120}, {30, -100}}, color = {0, 0, 0}), Line(visible = not useSupport, points = {{-30, -100}, {30, -100}}, color = {0, 0, 0})}));
        end PartialOneFlangeAndSupport;

        partial model PartialTwoFlangesAndSupport "Partial model for a component with two rotational 1-dim. shaft flanges and a support used for graphical modeling, i.e., the model is build up by drag-and-drop from elementary components"
          parameter Boolean useSupport = false "= true, if support flange enabled, otherwise implicitly grounded" annotation(
            Evaluate = true,
            HideResult = true,
            choices(checkBox = true));
          Flange_a flange_a "Flange of left shaft" annotation(
            Placement(transformation(extent = {{-110, -10}, {-90, 10}}, rotation = 0)));
          Flange_b flange_b "Flange of right shaft" annotation(
            Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
          Support support if useSupport "Support/housing of component" annotation(
            Placement(transformation(extent = {{-10, -110}, {10, -90}})));
        protected
          Support internalSupport "Internal support/housing of component (either connected to support, if useSupport=true, or connected to fixed, if useSupport=false)" annotation(
            Placement(transformation(extent = {{-3, -83}, {3, -77}})));
          Components.Fixed fixed if not useSupport "Fixed support/housing, if not useSupport" annotation(
            Placement(transformation(extent = {{10, -97}, {30, -77}})));
        equation
          connect(support, internalSupport) annotation(
            Line(points = {{0, -100}, {0, -80}}, color = {0, 0, 0}, smooth = Smooth.None));
          connect(internalSupport, fixed.flange) annotation(
            Line(points = {{0, -80}, {20, -80}, {20, -87}}, color = {0, 0, 0}, smooth = Smooth.None));
          annotation(
            Documentation(info = "<html>
<p>
This is a 1-dim. rotational component with two flanges and a support/housing.
It is used e.g., to build up parts of a drive train graphically consisting
of several components.
</p>

<p>
If <i>useSupport=true</i>, the support connector is conditionally enabled
and needs to be connected.<br>
If <i>useSupport=false</i>, the support connector is conditionally disabled
and instead the component is internally fixed to ground.
</p>
</html>"),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-38, -98}, {-6, -96}}, lineColor = {95, 95, 95}, textString = "(if useSupport)"), Text(extent = {{24, -97}, {64, -98}}, lineColor = {95, 95, 95}, textString = "(if not useSupport)")}),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(visible = not useSupport, points = {{-50, -120}, {-30, -100}}, color = {0, 0, 0}), Line(visible = not useSupport, points = {{-30, -120}, {-10, -100}}, color = {0, 0, 0}), Line(visible = not useSupport, points = {{-10, -120}, {10, -100}}, color = {0, 0, 0}), Line(visible = not useSupport, points = {{10, -120}, {30, -100}}, color = {0, 0, 0}), Line(visible = not useSupport, points = {{-30, -100}, {30, -100}}, color = {0, 0, 0})}));
        end PartialTwoFlangesAndSupport;

        partial model PartialCompliant "Partial model for the compliant connection of two rotational 1-dim. shaft flanges"
          Modelica.SIunits.Angle phi_rel(start = 0) "Relative rotation angle (= flange_b.phi - flange_a.phi)";
          Modelica.SIunits.Torque tau "Torque between flanges (= flange_b.tau)";
          Flange_a flange_a "Left flange of compliant 1-dim. rotational component" annotation(
            Placement(transformation(extent = {{-110, -10}, {-90, 10}}, rotation = 0)));
          Flange_b flange_b "Right flange of compliant 1-dim. rotational component" annotation(
            Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
        equation
          phi_rel = flange_b.phi - flange_a.phi;
          flange_b.tau = tau;
          flange_a.tau = -tau;
          annotation(
            Documentation(info = "<html>
<p>
This is a 1-dim. rotational component with a compliant connection of two
rotational 1-dim. flanges where inertial effects between the two
flanges are neglected. The basic assumption is that the cut-torques
of the two flanges sum-up to zero, i.e., they have the same absolute value
but opposite sign: flange_a.tau + flange_b.tau = 0. This base class
is used to built up force elements such as springs, dampers, friction.
</p>
</html>"));
        end PartialCompliant;

        partial model PartialCompliantWithRelativeStates "Partial model for the compliant connection of two rotational 1-dim. shaft flanges where the relative angle and speed are used as preferred states"
          Modelica.SIunits.Angle phi_rel(start = 0, stateSelect = stateSelect, nominal = if phi_nominal >= Modelica.Constants.eps then phi_nominal else 1) "Relative rotation angle (= flange_b.phi - flange_a.phi)";
          Modelica.SIunits.AngularVelocity w_rel(start = 0, stateSelect = stateSelect) "Relative angular velocity (= der(phi_rel))";
          Modelica.SIunits.AngularAcceleration a_rel(start = 0) "Relative angular acceleration (= der(w_rel))";
          Modelica.SIunits.Torque tau "Torque between flanges (= flange_b.tau)";
          Flange_a flange_a "Left flange of compliant 1-dim. rotational component" annotation(
            Placement(transformation(extent = {{-110, -10}, {-90, 10}}, rotation = 0)));
          Flange_b flange_b "Right flange of compliant 1-dim. rotational component" annotation(
            Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
          parameter SI.Angle phi_nominal(displayUnit = "rad", min = 0.0) = 1e-4 "Nominal value of phi_rel (used for scaling)" annotation(
            Dialog(tab = "Advanced"));
          parameter StateSelect stateSelect = StateSelect.prefer "Priority to use phi_rel and w_rel as states" annotation(
            HideResult = true,
            Dialog(tab = "Advanced"));
        equation
          phi_rel = flange_b.phi - flange_a.phi;
          w_rel = der(phi_rel);
          a_rel = der(w_rel);
          flange_b.tau = tau;
          flange_a.tau = -tau;
          annotation(
            Documentation(info = "<html>
<p>
This is a 1-dim. rotational component with a compliant connection of two
rotational 1-dim. flanges where inertial effects between the two
flanges are neglected. The basic assumption is that the cut-torques
of the two flanges sum-up to zero, i.e., they have the same absolute value
but opposite sign: flange_a.tau + flange_b.tau = 0. This base class
is used to built up force elements such as springs, dampers, friction.
</p>

<p>
The relative angle and the relative speed are defined as preferred states.
The reason is that for some drive trains, such as drive
trains in vehicles, the absolute angle is quickly increasing during operation.
Numerically, it is better to use relative angles between drive train components
because they remain in a limited size. For this reason, StateSelect.prefer
is set for the relative angle of this component.
</p>

<p>
In order to improve the numerics, a nominal value for the relative angle
can be provided via parameter <b>phi_nominal</b> in the Advanced menu.
The default is 1e-4 rad since relative angles are usually
in this order and the step size control of an integrator would be
practically switched off, if a default of 1 rad would be used.
This nominal value might also be computed from other values, such
as \"phi_nominal = tau_nominal / c\" for a rotational spring, if tau_nominal
and c are more meaningful for the user.
</p>

<p>
See also the discussion
<a href=\"modelica://Modelica.Mechanics.Rotational.UsersGuide.StateSelection\">State Selection</a>
in the User's Guide of the Rotational library.
</p>
</html>"));
        end PartialCompliantWithRelativeStates;

        partial model PartialElementaryOneFlangeAndSupport "Obsolete partial model. Use PartialElementaryOneFlangeAndSupport2."
          extends Modelica.Icons.ObsoleteModel;
          parameter Boolean useSupport = false "= true, if support flange enabled, otherwise implicitly grounded" annotation(
            Evaluate = true,
            HideResult = true,
            choices(checkBox = true));
          Flange_b flange "Flange of shaft" annotation(
            Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
          Support support if useSupport "Support/housing of component" annotation(
            Placement(transformation(extent = {{-10, -110}, {10, -90}})));
        protected
          Rotational.Interfaces.InternalSupport internalSupport(tau = -flange.tau) "Internal support/housing of component as a model with connector flange (flange is either connected to support, if useSupport=true, or connected to fixed, if useSupport=false)" annotation(
            Placement(transformation(extent = {{-10, -90}, {10, -70}})));
          Rotational.Components.Fixed fixed if not useSupport "Fixed support/housing, if not useSupport" annotation(
            Placement(transformation(extent = {{10, -96}, {30, -76}})));
        equation
          connect(internalSupport.flange, support) annotation(
            Line(points = {{0, -80}, {0, -100}}, color = {0, 0, 0}, smooth = Smooth.None));
          connect(internalSupport.flange, fixed.flange) annotation(
            Line(points = {{0, -80}, {20, -80}, {20, -86}}, color = {0, 0, 0}, smooth = Smooth.None));
          annotation(
            Documentation(info = "<html>
<p>
This is a 1-dim. rotational component with one flange and a support/housing.
It is used to build up elementary components of a drive train with
equations in the text layer.
</p>

<p>
If <i>useSupport=true</i>, the support connector is conditionally enabled
and needs to be connected.<br>
If <i>useSupport=false</i>, the support connector is conditionally disabled
and instead the component is internally fixed to ground.
</p>
</html>"),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{25, -97}, {65, -98}}, lineColor = {95, 95, 95}, textString = "(if not useSupport)"), Text(extent = {{-38, -98}, {-6, -96}}, lineColor = {95, 95, 95}, textString = "(if useSupport)")}),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(visible = not useSupport, points = {{-50, -120}, {-30, -100}}, color = {0, 0, 0}), Line(visible = not useSupport, points = {{-30, -120}, {-10, -100}}, color = {0, 0, 0}), Line(visible = not useSupport, points = {{-10, -120}, {10, -100}}, color = {0, 0, 0}), Line(visible = not useSupport, points = {{10, -120}, {30, -100}}, color = {0, 0, 0}), Line(visible = not useSupport, points = {{-30, -100}, {30, -100}}, color = {0, 0, 0})}));
        end PartialElementaryOneFlangeAndSupport;

        partial model PartialElementaryOneFlangeAndSupport2 "Partial model for a component with one rotational 1-dim. shaft flange and a support used for textual modeling, i.e., for elementary models"
          parameter Boolean useSupport = false "= true, if support flange enabled, otherwise implicitly grounded" annotation(
            Evaluate = true,
            HideResult = true,
            choices(checkBox = true));
          Flange_b flange "Flange of shaft" annotation(
            Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
          Support support(phi = phi_support, tau = -flange.tau) if useSupport "Support/housing of component" annotation(
            Placement(transformation(extent = {{-10, -110}, {10, -90}})));
        protected
          Modelica.SIunits.Angle phi_support "Absolute angle of support flange";
        equation
          if not useSupport then
            phi_support = 0;
          end if;
          annotation(
            Documentation(info = "<html>
<p>
This is a 1-dim. rotational component with one flange and a support/housing.
It is used to build up elementary components of a drive train with
equations in the text layer.
</p>

<p>
If <i>useSupport=true</i>, the support connector is conditionally enabled
and needs to be connected.<br>
If <i>useSupport=false</i>, the support connector is conditionally disabled
and instead the component is internally fixed to ground.
</p>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(visible = not useSupport, points = {{-50, -120}, {-30, -100}}, color = {0, 0, 0}), Line(visible = not useSupport, points = {{-30, -120}, {-10, -100}}, color = {0, 0, 0}), Line(visible = not useSupport, points = {{-10, -120}, {10, -100}}, color = {0, 0, 0}), Line(visible = not useSupport, points = {{10, -120}, {30, -100}}, color = {0, 0, 0}), Line(visible = not useSupport, points = {{-30, -100}, {30, -100}}, color = {0, 0, 0})}));
        end PartialElementaryOneFlangeAndSupport2;

        partial model PartialElementaryTwoFlangesAndSupport "Obsolete partial model. Use PartialElementaryTwoFlangesAndSupport2."
          extends Modelica.Icons.ObsoleteModel;
          parameter Boolean useSupport = false "= true, if support flange enabled, otherwise implicitly grounded" annotation(
            Evaluate = true,
            HideResult = true,
            choices(checkBox = true));
          Flange_a flange_a "Flange of left shaft" annotation(
            Placement(transformation(extent = {{-110, -10}, {-90, 10}}, rotation = 0)));
          Flange_b flange_b "Flange of right shaft" annotation(
            Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
          Support support if useSupport "Support/housing of component" annotation(
            Placement(transformation(extent = {{-10, -110}, {10, -90}})));
        protected
          Rotational.Interfaces.InternalSupport internalSupport(tau = (-flange_a.tau) - flange_b.tau) "Internal support/housing of component as a model with connector flange (flange is either connected to support, if useSupport=true, or connected to fixed, if useSupport=false)" annotation(
            Placement(transformation(extent = {{-10, -90}, {10, -70}})));
          Rotational.Components.Fixed fixed if not useSupport "Fixed support/housing, if not useSupport" annotation(
            Placement(transformation(extent = {{10, -97}, {30, -77}})));
        equation
          connect(internalSupport.flange, support) annotation(
            Line(points = {{0, -80}, {0, -100}}, color = {0, 0, 0}, smooth = Smooth.None));
          connect(internalSupport.flange, fixed.flange) annotation(
            Line(points = {{0, -80}, {20, -80}, {20, -87}}, color = {0, 0, 0}, smooth = Smooth.None));
          annotation(
            Documentation(info = "<html>
<p>
This is a 1-dim. rotational component with two flanges and a support/housing.
It is used to build up elementary components of a drive train with
equations in the text layer.
</p>

<p>
If <i>useSupport=true</i>, the support connector is conditionally enabled
and needs to be connected.<br>
If <i>useSupport=false</i>, the support connector is conditionally disabled
and instead the component is internally fixed to ground.
</p>
</html>"),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{24, -97}, {64, -98}}, lineColor = {95, 95, 95}, textString = "(if not useSupport)"), Text(extent = {{-38, -98}, {-6, -96}}, lineColor = {95, 95, 95}, textString = "(if useSupport)")}),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(visible = not useSupport, points = {{-50, -120}, {-30, -100}}, color = {0, 0, 0}), Line(visible = not useSupport, points = {{-30, -120}, {-10, -100}}, color = {0, 0, 0}), Line(visible = not useSupport, points = {{-10, -120}, {10, -100}}, color = {0, 0, 0}), Line(visible = not useSupport, points = {{10, -120}, {30, -100}}, color = {0, 0, 0}), Line(visible = not useSupport, points = {{-30, -100}, {30, -100}}, color = {0, 0, 0})}));
        end PartialElementaryTwoFlangesAndSupport;

        partial model PartialElementaryTwoFlangesAndSupport2 "Partial model for a component with two rotational 1-dim. shaft flanges and a support used for textual modeling, i.e., for elementary models"
          parameter Boolean useSupport = false "= true, if support flange enabled, otherwise implicitly grounded" annotation(
            Evaluate = true,
            HideResult = true,
            choices(checkBox = true));
          Flange_a flange_a "Flange of left shaft" annotation(
            Placement(transformation(extent = {{-110, -10}, {-90, 10}}, rotation = 0)));
          Flange_b flange_b "Flange of right shaft" annotation(
            Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
          Support support(phi = phi_support, tau = (-flange_a.tau) - flange_b.tau) if useSupport "Support/housing of component" annotation(
            Placement(transformation(extent = {{-10, -110}, {10, -90}})));
        protected
          Modelica.SIunits.Angle phi_support "Absolute angle of support flange";
        equation
          if not useSupport then
            phi_support = 0;
          end if;
          annotation(
            Documentation(info = "<html>
<p>
This is a 1-dim. rotational component with two flanges and a support/housing.
It is used to build up elementary components of a drive train with
equations in the text layer.
</p>

<p>
If <i>useSupport=true</i>, the support connector is conditionally enabled
and needs to be connected.<br>
If <i>useSupport=false</i>, the support connector is conditionally disabled
and instead the component is internally fixed to ground.
</p>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(visible = not useSupport, points = {{-50, -120}, {-30, -100}}, color = {0, 0, 0}), Line(visible = not useSupport, points = {{-30, -120}, {-10, -100}}, color = {0, 0, 0}), Line(visible = not useSupport, points = {{-10, -120}, {10, -100}}, color = {0, 0, 0}), Line(visible = not useSupport, points = {{10, -120}, {30, -100}}, color = {0, 0, 0}), Line(visible = not useSupport, points = {{-30, -100}, {30, -100}}, color = {0, 0, 0})}));
        end PartialElementaryTwoFlangesAndSupport2;

        partial model PartialElementaryRotationalToTranslational "Partial model to transform rotational into translational motion"
          parameter Boolean useSupportR = false "= true, if rotational support flange enabled, otherwise implicitly grounded" annotation(
            Evaluate = true,
            HideResult = true,
            choices(checkBox = true));
          parameter Boolean useSupportT = false "= true, if translational support flange enabled, otherwise implicitly grounded" annotation(
            Evaluate = true,
            HideResult = true,
            choices(checkBox = true));
          Rotational.Interfaces.Flange_a flangeR "Flange of rotational shaft" annotation(
            Placement(transformation(extent = {{-110, -10}, {-90, 10}}, rotation = 0)));
          Modelica.Mechanics.Translational.Interfaces.Flange_b flangeT "Flange of translational rod" annotation(
            Placement(transformation(extent = {{90, 10}, {110, -10}}, rotation = 0)));
          Rotational.Interfaces.Support supportR if useSupportR "Rotational support/housing of component" annotation(
            Placement(transformation(extent = {{-110, -110}, {-90, -90}}, rotation = 0), iconTransformation(extent = {{-110, -110}, {-90, -90}})));
          Translational.Interfaces.Support supportT if useSupportT "Translational support/housing of component" annotation(
            Placement(transformation(extent = {{110, -110}, {90, -90}}, rotation = 0), iconTransformation(extent = {{90, -110}, {110, -90}})));
        protected
          Rotational.Interfaces.InternalSupport internalSupportR(tau = -flangeR.tau) annotation(
            Placement(transformation(extent = {{-110, -90}, {-90, -70}})));
          Translational.Interfaces.InternalSupport internalSupportT(f = -flangeT.f) annotation(
            Placement(transformation(extent = {{90, -90}, {110, -70}})));
          Rotational.Components.Fixed fixedR if not useSupportR annotation(
            Placement(transformation(extent = {{-90, -90}, {-70, -70}})));
          Translational.Components.Fixed fixedT if not useSupportT annotation(
            Placement(transformation(extent = {{70, -90}, {90, -70}})));
        equation
          connect(internalSupportR.flange, supportR) annotation(
            Line(points = {{-100, -80}, {-100, -100}}, color = {0, 0, 0}, smooth = Smooth.None));
          connect(internalSupportR.flange, fixedR.flange) annotation(
            Line(points = {{-100, -80}, {-80, -80}}, color = {0, 0, 0}, smooth = Smooth.None));
          connect(fixedT.flange, internalSupportT.flange) annotation(
            Line(points = {{80, -80}, {100, -80}}, color = {0, 127, 0}, smooth = Smooth.None));
          connect(internalSupportT.flange, supportT) annotation(
            Line(points = {{100, -80}, {100, -100}}, color = {0, 127, 0}, smooth = Smooth.None));
          annotation(
            Documentation(info = "<html>

<p>
This is a 1-dim. rotational component with
</p>

<ul>
<li> one rotational flange, </li>
<li> one rotational support/housing, </li>
<li> one translational flange, and </li>
<li> one translational support/housing </li>
</ul>

<p>
This model is used to build up elementary components of a drive train
transforming rotational into translational motion with
equations in the text layer.
</p>

<p>
If <i>useSupportR=true</i>, the rotational support connector is conditionally enabled
and needs to be connected.<br>
If <i>useSupportR=false</i>, the rotational support connector is conditionally disabled
and instead the rotational part is internally fixed to ground.<br>
If <i>useSupportT=true</i>, the translational support connector is conditionally enabled
and needs to be connected.<br>
If <i>useSupportT=false</i>, the translational support connector is conditionally disabled
and instead the translational part is internally fixed to ground.
</p>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(visible = not useSupportT, points = {{85, -110}, {95, -100}}, color = {0, 0, 0}), Line(visible = not useSupportT, points = {{95, -110}, {105, -100}}, color = {0, 0, 0}), Line(visible = not useSupportT, points = {{105, -110}, {115, -100}}, color = {0, 0, 0}), Line(visible = not useSupportT, points = {{85, -100}, {115, -100}}, color = {0, 0, 0}), Line(visible = not useSupportR, points = {{-115, -110}, {-105, -100}}, color = {0, 0, 0}), Line(visible = not useSupportR, points = {{-105, -110}, {-95, -100}}, color = {0, 0, 0}), Line(visible = not useSupportR, points = {{-95, -110}, {-85, -100}}, color = {0, 0, 0}), Line(visible = not useSupportR, points = {{-115, -100}, {-85, -100}}, color = {0, 0, 0})}));
        end PartialElementaryRotationalToTranslational;

        partial model PartialTorque "Partial model of a torque acting at the flange (accelerates the flange)"
          extends Modelica.Mechanics.Rotational.Interfaces.PartialElementaryOneFlangeAndSupport2;
          Modelica.SIunits.Angle phi "Angle of flange with respect to support (= flange.phi - support.phi)";
        equation
          phi = flange.phi - phi_support;
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-96, 96}, {96, -96}}, lineColor = {255, 255, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{0, -62}, {0, -100}}, color = {0, 0, 0}), Line(points = {{-92, 0}, {-76, 36}, {-54, 62}, {-30, 80}, {-14, 88}, {10, 92}, {26, 90}, {46, 80}, {64, 62}}, color = {0, 0, 0}, smooth = Smooth.Bezier), Text(extent = {{-150, 140}, {150, 100}}, lineColor = {0, 0, 255}, textString = "%name"), Polygon(points = {{94, 16}, {80, 74}, {50, 52}, {94, 16}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Line(points = {{-58, -82}, {-42, -68}, {-20, -56}, {0, -54}, {18, -56}, {34, -62}, {44, -72}, {54, -82}, {60, -94}}, color = {0, 0, 0}, smooth = Smooth.Bezier), Polygon(points = {{-65, -98}, {-46, -80}, {-58, -72}, {-65, -98}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Line(visible = not useSupport, points = {{-50, -120}, {-30, -100}}, color = {0, 0, 0}), Line(visible = not useSupport, points = {{-30, -120}, {-10, -100}}, color = {0, 0, 0}), Line(visible = not useSupport, points = {{-10, -120}, {10, -100}}, color = {0, 0, 0}), Line(visible = not useSupport, points = {{10, -120}, {30, -100}}, color = {0, 0, 0}), Line(visible = not useSupport, points = {{-30, -100}, {30, -100}}, color = {0, 0, 0})}),
            Documentation(info = "<HTML>
<p>
Partial model of torque that accelerates the flange.
</p>

<p>
If <i>useSupport=true</i>, the support connector is conditionally enabled
and needs to be connected.<br>
If <i>useSupport=false</i>, the support connector is conditionally disabled
and instead the component is internally fixed to ground.
</p>
</html>"));
        end PartialTorque;

        partial model PartialAbsoluteSensor "Partial model to measure a single absolute flange variable"
          extends Modelica.Icons.RotationalSensor;
          Flange_a flange "Flange of shaft from which sensor information shall be measured" annotation(
            Placement(transformation(extent = {{-110, -10}, {-90, 10}}, rotation = 0)));
        equation
          0 = flange.tau;
          annotation(
            Documentation(info = "<html>
<p>
This is a partial model of a 1-dim. rotational component with one flange of a shaft
in order to measure an absolute kinematic quantity in the flange
and to provide the measured signal as output signal for further processing
with the blocks of package Modelica.Blocks.
</p>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100.0, -100.0}, {100.0, 100.0}}, initialScale = 0.1), graphics = {Line(points = {{-70.0, 0.0}, {-90.0, 0.0}}), Line(points = {{70.0, 0.0}, {100.0, 0.0}}, color = {0, 0, 127}), Text(lineColor = {0, 0, 255}, extent = {{-150.0, 80.0}, {150.0, 120.0}}, textString = "%name")}));
        end PartialAbsoluteSensor;

        partial model PartialRelativeSensor "Partial model to measure a single relative variable between two flanges"
          extends Modelica.Icons.RotationalSensor;
          Flange_a flange_a "Left flange of shaft" annotation(
            Placement(transformation(extent = {{-110, -10}, {-90, 10}}, rotation = 0)));
          Flange_b flange_b "Right flange of shaft" annotation(
            Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
        equation
          0 = flange_a.tau + flange_b.tau;
          annotation(
            Documentation(info = "<html>
<p>
This is a partial model for 1-dim. rotational components with two rigidly connected
flanges in order to measure relative kinematic quantities
between the two flanges or the cut-torque in the flange and
to provide the measured signal as output signal for further processing
with the blocks of package Modelica.Blocks.
</p>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100.0, -100.0}, {100.0, 100.0}}, initialScale = 0.1), graphics = {Line(points = {{-70.0, 0.0}, {-90.0, 0.0}}), Line(points = {{70.0, 0.0}, {90.0, 0.0}}), Text(lineColor = {0, 0, 255}, extent = {{-150.0, 73.0}, {150.0, 113.0}}, textString = "%name")}));
        end PartialRelativeSensor;

        partial model PartialFriction "Partial model of Coulomb friction elements"
          // parameter SI.AngularVelocity w_small=1 "Relative angular velocity near to zero (see model info text)";
          parameter SI.AngularVelocity w_small = 1.0e10 "Relative angular velocity near to zero if jumps due to a reinit(..) of the velocity can occur (set to low value only if such impulses can occur)" annotation(
            Dialog(tab = "Advanced"));
          // Equations to define the following variables have to be defined in subclasses
          SI.AngularVelocity w_relfric "Relative angular velocity between frictional surfaces";
          SI.AngularAcceleration a_relfric "Relative angular acceleration between frictional surfaces";
          //SI.Torque tau "Friction torque (positive, if directed in opposite direction of w_rel)";
          SI.Torque tau0 "Friction torque for w=0 and forward sliding";
          SI.Torque tau0_max "Maximum friction torque for w=0 and locked";
          Boolean free "true, if frictional element is not active";
          // Equations to define the following variables are given in this class
          Real sa(final unit = "1") "Path parameter of friction characteristic tau = f(a_relfric)";
          Boolean startForward(start = false, fixed = true) "true, if w_rel=0 and start of forward sliding";
          Boolean startBackward(start = false, fixed = true) "true, if w_rel=0 and start of backward sliding";
          Boolean locked(start = false) "true, if w_rel=0 and not sliding";
          constant Integer Unknown = 3 "Value of mode is not known";
          constant Integer Free = 2 "Element is not active";
          constant Integer Forward = 1 "w_rel > 0 (forward sliding)";
          constant Integer Stuck = 0 "w_rel = 0 (forward sliding, locked or backward sliding)";
          constant Integer Backward = -1 "w_rel < 0 (backward sliding)";
          Integer mode(final min = Backward, final max = Unknown, start = Unknown, fixed = true);
        protected
          constant SI.AngularAcceleration unitAngularAcceleration = 1 annotation(
            HideResult = true);
          constant SI.Torque unitTorque = 1 annotation(
            HideResult = true);
        equation
/* Friction characteristic
   locked is introduced to help the Modelica translator determining
   the different structural configurations,
   if for each configuration special code shall be generated
*/
          startForward = pre(mode) == Stuck and (sa > tau0_max / unitTorque or pre(startForward) and sa > tau0 / unitTorque) or pre(mode) == Backward and w_relfric > w_small or initial() and w_relfric > 0;
          startBackward = pre(mode) == Stuck and (sa < (-tau0_max / unitTorque) or pre(startBackward) and sa < (-tau0 / unitTorque)) or pre(mode) == Forward and w_relfric < (-w_small) or initial() and w_relfric < 0;
          locked = not free and not (pre(mode) == Forward or startForward or pre(mode) == Backward or startBackward);
          a_relfric / unitAngularAcceleration = if locked then 0 else if free then sa else if startForward then sa - tau0_max / unitTorque else if startBackward then sa + tau0_max / unitTorque else if pre(mode) == Forward then sa - tau0_max / unitTorque else sa + tau0_max / unitTorque;
/* Friction torque has to be defined in a subclass. Example for a clutch:
   tau = if locked then sa else
         if free then   0 else
         cgeo*fn*(if startForward then          Math.tempInterpol1( w_relfric, mue_pos, 2) else
                  if startBackward then        -Math.tempInterpol1(-w_relfric, mue_pos, 2) else
                  if pre(mode) == Forward then  Math.tempInterpol1( w_relfric, mue_pos, 2) else
                                               -Math.tempInterpol1(-w_relfric, mue_pos, 2));
*/
// finite state machine to determine configuration
          mode = if free then Free else if (pre(mode) == Forward or pre(mode) == Free or startForward) and w_relfric > 0 then Forward else if (pre(mode) == Backward or pre(mode) == Free or startBackward) and w_relfric < 0 then Backward else Stuck;
          annotation(
            Documentation(info = "<html>
<p>
Basic model for Coulomb friction that models the stuck phase in a reliable way.
</p>
</html>"));
        end PartialFriction;
        annotation(
          Documentation(info = "<html>
<p>
This package contains connectors and partial models for 1-dim.
rotational mechanical components. The components of this package can
only be used as basic building elements for models.
</p>
</html>"));
      end Interfaces;

      package Icons "Icons for Rotational package"
        extends Modelica.Icons.IconsPackage;

        partial class Gear "Rotational gear icon"
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100.0, -100.0}, {100.0, 100.0}}, initialScale = 0.1), graphics = {Rectangle(origin = {-35.0, 60.0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.HorizontalCylinder, extent = {{-15.0, -40.0}, {15.0, 40.0}}), Rectangle(origin = {-35.0, 0.0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.HorizontalCylinder, extent = {{-15.0, -21.0}, {15.0, 21.0}}), Line(points = {{-80.0, 20.0}, {-60.0, 20.0}}), Line(points = {{-80.0, -20.0}, {-60.0, -20.0}}), Line(points = {{-70.0, -20.0}, {-70.0, -86.0}}), Line(points = {{0.0, 40.0}, {0.0, -86.0}}), Line(points = {{-10.0, 40.0}, {10.0, 40.0}}), Line(points = {{-10.0, 80.0}, {10.0, 80.0}}), Line(points = {{60.0, -20.0}, {80.0, -20.0}}), Line(points = {{60.0, 20.0}, {80.0, 20.0}}), Line(points = {{70.0, -20.0}, {70.0, -86.0}}), Line(points = {{70.0, -86.0}, {-70.0, -86.0}}), Rectangle(origin = {-75.0, 0.0}, lineColor = {64, 64, 64}, fillColor = {191, 191, 191}, fillPattern = FillPattern.HorizontalCylinder, extent = {{-25.0, -10.0}, {25.0, 10.0}}), Rectangle(origin = {75.0, 0.0}, lineColor = {64, 64, 64}, fillColor = {191, 191, 191}, fillPattern = FillPattern.HorizontalCylinder, extent = {{-25.0, -10.0}, {25.0, 10.0}}), Rectangle(origin = {-35.0, -19.0}, fillColor = {153, 153, 153}, fillPattern = FillPattern.Solid, extent = {{-15.0, -2.0}, {15.0, 2.0}}), Rectangle(origin = {-35.0, -8.0}, fillColor = {204, 204, 204}, fillPattern = FillPattern.Solid, extent = {{-15.0, -3.0}, {15.0, 3.0}}), Rectangle(origin = {-35.0, 19.0}, fillColor = {204, 204, 204}, fillPattern = FillPattern.Solid, extent = {{-15.0, -2.0}, {15.0, 2.0}}), Rectangle(origin = {-35.0, 8.0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, extent = {{-15.0, -3.0}, {15.0, 3.0}}), Rectangle(origin = {0.0, 60.0}, lineColor = {64, 64, 64}, fillColor = {191, 191, 191}, fillPattern = FillPattern.HorizontalCylinder, extent = {{-20.0, -10.0}, {20.0, 10.0}}), Rectangle(origin = {-35.0, 98.0}, fillColor = {153, 153, 153}, fillPattern = FillPattern.Solid, extent = {{-15.0, -2.0}, {15.0, 2.0}}), Rectangle(origin = {-35.0, 87.0}, fillColor = {204, 204, 204}, fillPattern = FillPattern.Solid, extent = {{-15.0, -3.0}, {15.0, 3.0}}), Rectangle(origin = {-35.0, 50.0}, fillColor = {204, 204, 204}, fillPattern = FillPattern.Solid, extent = {{-15.0, -4.0}, {15.0, 4.0}}), Rectangle(origin = {-35.0, 22.0}, fillColor = {102, 102, 102}, fillPattern = FillPattern.Solid, extent = {{-15.0, -2.0}, {15.0, 2.0}}), Rectangle(origin = {-35.0, 33.0}, fillColor = {153, 153, 153}, fillPattern = FillPattern.Solid, extent = {{-15.0, -3.0}, {15.0, 3.0}}), Rectangle(origin = {-35.0, 70.0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, extent = {{-15.0, -4.0}, {15.0, 4.0}}), Rectangle(origin = {35.0, 60.0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.HorizontalCylinder, extent = {{-15.0, -21.0}, {15.0, 21.0}}), Rectangle(origin = {35.0, 41.0}, fillColor = {153, 153, 153}, fillPattern = FillPattern.Solid, extent = {{-15.0, -2.0}, {15.0, 2.0}}), Rectangle(origin = {35.0, 52.0}, fillColor = {204, 204, 204}, fillPattern = FillPattern.Solid, extent = {{-15.0, -3.0}, {15.0, 3.0}}), Rectangle(origin = {35.0, 79.0}, fillColor = {204, 204, 204}, fillPattern = FillPattern.Solid, extent = {{-15.0, -2.0}, {15.0, 2.0}}), Rectangle(origin = {35.0, 68.0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, extent = {{-15.0, -3.0}, {15.0, 3.0}}), Rectangle(origin = {35.0, 0.0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.HorizontalCylinder, extent = {{-15.0, -40.0}, {15.0, 40.0}}), Rectangle(origin = {35.0, 38.0}, fillColor = {153, 153, 153}, fillPattern = FillPattern.Solid, extent = {{-15.0, -2.0}, {15.0, 2.0}}), Rectangle(origin = {35.0, 27.0}, fillColor = {204, 204, 204}, fillPattern = FillPattern.Solid, extent = {{-15.0, -3.0}, {15.0, 3.0}}), Rectangle(origin = {35.0, -10.0}, fillColor = {204, 204, 204}, fillPattern = FillPattern.Solid, extent = {{-15.0, -4.0}, {15.0, 4.0}}), Rectangle(origin = {35.0, -38.0}, fillColor = {102, 102, 102}, fillPattern = FillPattern.Solid, extent = {{-15.0, -2.0}, {15.0, 2.0}}), Rectangle(origin = {35.0, -27.0}, fillColor = {153, 153, 153}, fillPattern = FillPattern.Solid, extent = {{-15.0, -3.0}, {15.0, 3.0}}), Rectangle(origin = {35.0, 10.0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, extent = {{-15.0, -4.0}, {15.0, 4.0}}), Rectangle(origin = {-35, 40}, fillColor = {255, 255, 255}, extent = {{-15, -61}, {15, 60}}), Rectangle(origin = {35, 21}, fillColor = {255, 255, 255}, extent = {{-15, -61}, {15, 60}})}),
            Documentation(info = "<html>
<p>
This is the icon of a gear from the rotational package.
</p>
</html>"));
        end Gear;

        model Gearbox "Icon of gear box"
          annotation(
            Icon(coordinateSystem(extent = {{-100.0, -100.0}, {100.0, 100.0}}, preserveAspectRatio = true, initialScale = 0.1), graphics = {Rectangle(origin = {20.0, 0.0}, lineColor = {64, 64, 64}, fillColor = {192, 192, 192}, fillPattern = FillPattern.HorizontalCylinder, extent = {{-120.0, -10.0}, {-80.0, 10.0}}), Polygon(fillColor = {192, 192, 192}, fillPattern = FillPattern.HorizontalCylinder, points = {{-60.0, 10.0}, {-60.0, 20.0}, {-40.0, 40.0}, {-40.0, -40.0}, {-60.0, -20.0}, {-60.0, 10.0}}), Rectangle(lineColor = {64, 64, 64}, fillColor = {255, 255, 255}, fillPattern = FillPattern.HorizontalCylinder, extent = {{-40.0, -60.0}, {40.0, 60.0}}, radius = 10.0), Rectangle(lineColor = {64, 64, 64}, fillPattern = FillPattern.None, extent = {{-40.0, -60.0}, {40.0, 60.0}}, radius = 10.0), Polygon(fillColor = {192, 192, 192}, fillPattern = FillPattern.HorizontalCylinder, points = {{60.0, 20.0}, {40.0, 40.0}, {40.0, -40.0}, {60.0, -20.0}, {60.0, 20.0}}), Rectangle(origin = {-20.0, 0.0}, lineColor = {64, 64, 64}, fillColor = {192, 192, 192}, fillPattern = FillPattern.HorizontalCylinder, extent = {{80.0, -10.0}, {120.0, 10.0}}), Polygon(origin = {0.0, 10.0}, fillColor = {64, 64, 64}, fillPattern = FillPattern.Solid, points = {{-60.0, -90.0}, {-50.0, -90.0}, {-20.0, -30.0}, {20.0, -30.0}, {48.0, -90.0}, {60.0, -90.0}, {60.0, -100.0}, {-60.0, -100.0}, {-60.0, -90.0}})}),
            Documentation(info = "<html>
<p>
This is the icon of a gear box from the rotational package.
</p>
</html>"));
        end Gearbox;

        model Clutch "Icon of a clutch"
          annotation(
            Icon(graphics = {Rectangle(lineColor = {64, 64, 64}, fillColor = {192, 192, 192}, fillPattern = FillPattern.HorizontalCylinder, extent = {{-100.0, -10.0}, {-30.0, 10.0}}), Rectangle(lineColor = {64, 64, 64}, fillColor = {255, 255, 255}, fillPattern = FillPattern.HorizontalCylinder, extent = {{-30.0, -60.0}, {-10.0, 60.0}}), Rectangle(lineColor = {64, 64, 64}, fillPattern = FillPattern.None, extent = {{-30.0, -60.0}, {-10.0, 60.0}}), Rectangle(lineColor = {64, 64, 64}, fillColor = {255, 255, 255}, fillPattern = FillPattern.HorizontalCylinder, extent = {{10.0, -60.0}, {30.0, 60.0}}), Rectangle(lineColor = {64, 64, 64}, fillPattern = FillPattern.None, extent = {{10.0, -60.0}, {30.0, 60.0}}), Rectangle(lineColor = {64, 64, 64}, fillColor = {192, 192, 192}, fillPattern = FillPattern.HorizontalCylinder, extent = {{30.0, -10.0}, {100.0, 10.0}}), Polygon(lineColor = {0, 0, 127}, fillColor = {0, 0, 127}, fillPattern = FillPattern.Solid, points = {{-30.0, 40.0}, {-60.0, 50.0}, {-60.0, 30.0}, {-30.0, 40.0}}), Polygon(lineColor = {0, 0, 127}, fillColor = {0, 0, 127}, fillPattern = FillPattern.Solid, points = {{30.0, 40.0}, {60.0, 50.0}, {60.0, 30.0}, {30.0, 40.0}}), Line(points = {{0.0, 90.0}, {90.0, 70.0}, {90.0, 40.0}, {30.0, 40.0}}, color = {0, 0, 127}), Line(points = {{0.0, 90.0}, {-90.0, 70.0}, {-90.0, 40.0}, {-30.0, 40.0}}, color = {0, 0, 127})}, coordinateSystem(extent = {{-100.0, -100.0}, {100.0, 100.0}}, preserveAspectRatio = true, initialScale = 0.1)),
            Documentation(info = "<html>
<p>
This is the icon of a clutch from the rotational package.
</p>
</html>"));
        end Clutch;
        annotation(
          Documentation(info = "<html>
<p>
This package contains icons for the Rotational library
(that is, all the components have only graphical annotations
without any equations).
</p>
</html>"));
      end Icons;
      annotation(
        Documentation(info = "<html>

<p>
Library <b>Rotational</b> is a <b>free</b> Modelica package providing
1-dimensional, rotational mechanical components to model in a convenient way
drive trains with frictional losses. A typical, simple example is shown
in the next figure:
</p>

<img src=\"modelica://Modelica/Resources/Images/Mechanics/Rotational/driveExample.png\">

<p>
For an introduction, have especially a look at:
</p>
<ul>
<li> <a href=\"modelica://Modelica.Mechanics.Rotational.UsersGuide\">Rotational.UsersGuide</a>
     discusses the most important aspects how to use this library.</li>
<li> <a href=\"modelica://Modelica.Mechanics.Rotational.Examples\">Rotational.Examples</a>
     contains examples that demonstrate the usage of this library.</li>
</ul>

<p>
In version 3.0 of the Modelica Standard Library, the basic design of the
library has changed: Previously, bearing connectors could or could not be connected.
In 3.0, the bearing connector is renamed to \"<b>support</b>\" and this connector
is enabled via parameter \"useSupport\". If the support connector is enabled,
it must be connected, and if it is not enabled, it must not be connected.
</p>

<p>
In version 3.2 of the Modelica Standard Library, all <b>dissipative</b> components
of the Rotational library got an optional <b>heatPort</b> connector to which the
dissipated energy is transported in form of heat. This connector is enabled
via parameter \"useHeatPort\". If the heatPort connector is enabled,
it must be connected, and if it is not enabled, it must not be connected.
Independently, whether the heatPort is enabled or not,
the dissipated power is available from the new variable \"<b>lossPower</b>\" (which is
positive if heat is flowing out of the heatPort). For an example, see
<a href=\"modelica://Modelica.Mechanics.Rotational.Examples.HeatLosses\">Examples.HeatLosses</a>.
</p>

<p>
Copyright &copy; 1998-2015, Modelica Association and DLR.
</p>
<p>
<i>This Modelica package is <u>free</u> software and the use is completely at <u>your own risk</u>; it can be redistributed and/or modified under the terms of the Modelica License 2. For license conditions (including the disclaimer of warranty) see <a href=\"modelica://Modelica.UsersGuide.ModelicaLicense2\">Modelica.UsersGuide.ModelicaLicense2</a> or visit <a href=\"https://www.modelica.org/licenses/ModelicaLicense2\"> https://www.modelica.org/licenses/ModelicaLicense2</a>.</i>
</p>
</html>", revisions = ""),
        Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100.0, -100.0}, {100.0, 100.0}}, initialScale = 0.1), graphics = {Line(visible = true, origin = {-2.0, 46.0}, points = {{-83.0, -66.0}, {-63.0, -66.0}}), Line(visible = true, origin = {29.0, 48.0}, points = {{36.0, -68.0}, {56.0, -68.0}}), Line(visible = true, origin = {-2.0, 49.0}, points = {{-83.0, -29.0}, {-63.0, -29.0}}), Line(visible = true, origin = {29.0, 52.0}, points = {{36.0, -32.0}, {56.0, -32.0}}), Line(visible = true, origin = {-2.0, 49.0}, points = {{-73.0, -9.0}, {-73.0, -29.0}}), Line(visible = true, origin = {29.0, 52.0}, points = {{46.0, -12.0}, {46.0, -32.0}}), Line(visible = true, origin = {-0.0, -47.5}, points = {{-75.0, 27.5}, {-75.0, -27.5}, {75.0, -27.5}, {75.0, 27.5}}), Rectangle(visible = true, origin = {13.5135, 76.9841}, lineColor = {64, 64, 64}, fillColor = {255, 255, 255}, fillPattern = FillPattern.HorizontalCylinder, extent = {{-63.5135, -126.9841}, {36.4865, -26.9841}}, radius = 10.0), Rectangle(visible = true, origin = {13.5135, 76.9841}, lineColor = {64, 64, 64}, fillPattern = FillPattern.None, extent = {{-63.5135, -126.9841}, {36.4865, -26.9841}}, radius = 10.0), Rectangle(visible = true, origin = {-3.0, 73.0769}, lineColor = {64, 64, 64}, fillColor = {192, 192, 192}, fillPattern = FillPattern.HorizontalCylinder, extent = {{-87.0, -83.0769}, {-47.0, -63.0769}}), Rectangle(visible = true, origin = {22.3077, 70.0}, lineColor = {64, 64, 64}, fillColor = {192, 192, 192}, fillPattern = FillPattern.HorizontalCylinder, extent = {{27.6923, -80.0}, {67.6923, -60.0}})}));
    end Rotational;

    package Translational "Library to model 1-dimensional, translational mechanical systems"
      extends Modelica.Icons.Package;
      import SI = Modelica.SIunits;

      package Examples "Demonstration examples of the components of this package"
        extends Modelica.Icons.ExamplesPackage;

        model SignConvention "Examples for the used sign conventions."
          extends Modelica.Icons.Example;
          Translational.Components.Mass mass1(L = 1, s(fixed = true), v(fixed = true), m = 1) annotation(
            Placement(transformation(extent = {{40, 60}, {60, 80}}, rotation = 0)));
          Translational.Sources.Force force1 annotation(
            Placement(transformation(extent = {{-4, 60}, {16, 80}}, rotation = 0)));
          Modelica.Blocks.Sources.Constant constant1(k = 1) annotation(
            Placement(transformation(extent = {{-44, 60}, {-24, 80}}, rotation = 0)));
          Translational.Components.Mass mass2(L = 1, s(fixed = true), v(fixed = true), m = 1) annotation(
            Placement(transformation(extent = {{40, 0}, {60, 20}}, rotation = 0)));
          Translational.Sources.Force force2 annotation(
            Placement(transformation(extent = {{-4, 20}, {16, 40}}, rotation = 0)));
          Modelica.Blocks.Sources.Constant constant2(k = 1) annotation(
            Placement(transformation(extent = {{-44, 20}, {-24, 40}}, rotation = 0)));
          Translational.Components.Mass mass3(L = 1, s(fixed = true), v(fixed = true), m = 1) annotation(
            Placement(transformation(extent = {{-40, -40}, {-20, -20}}, rotation = 0)));
          Translational.Sources.Force force3(useSupport = true) annotation(
            Placement(transformation(extent = {{20, -40}, {0, -20}}, rotation = 0)));
          Modelica.Blocks.Sources.Constant constant3(k = 1) annotation(
            Placement(transformation(extent = {{60, -40}, {40, -20}}, rotation = 0)));
          Translational.Components.Fixed fixed annotation(
            Placement(transformation(extent = {{0, -60}, {20, -40}})));
        equation
          connect(constant1.y, force1.f) annotation(
            Line(points = {{-23, 70}, {-6, 70}}, color = {0, 0, 127}));
          connect(constant2.y, force2.f) annotation(
            Line(points = {{-23, 30}, {-6, 30}}, color = {0, 0, 127}));
          connect(constant3.y, force3.f) annotation(
            Line(points = {{39, -30}, {22, -30}}, color = {0, 0, 127}));
          connect(force1.flange, mass1.flange_a) annotation(
            Line(points = {{16, 70}, {40, 70}}, color = {0, 127, 0}, smooth = Smooth.None));
          connect(force2.flange, mass2.flange_b) annotation(
            Line(points = {{16, 30}, {70, 30}, {70, 10}, {60, 10}}, color = {0, 127, 0}, smooth = Smooth.None));
          connect(mass3.flange_b, force3.flange) annotation(
            Line(points = {{-20, -30}, {0, -30}}, color = {0, 127, 0}, smooth = Smooth.None));
          connect(fixed.flange, force3.support) annotation(
            Line(points = {{10, -50}, {10, -40}}, color = {0, 127, 0}, smooth = Smooth.None));
          annotation(
            Documentation(info = "<html>
<p>
If all arrows point in the same direction a positive force
results in a positive acceleration a, velocity v and position s.
</p>
<p>
For a force of 1 N and a mass of 1 Kg this leads to
</p>
<pre>
        a = 1 m/s2
        v = 1 m/s after 1 s (SlidingMass1.v)
        s = 0.5 m after 1 s (SlidingMass1.s)
</pre>
<p>
The acceleration is not available for plotting.
</p>
<p>
System 1) and 2) are equivalent. It doesn't matter whether the
force pushes at flange_a in system 1 or pulls at flange_b in system 2.
</p><p>
It is of course possible to ignore the arrows and connect the models
in an arbitrary way. But then it is hard see in what direction the
force acts.
</p><p>
In the third system the two arrows are opposed which means that the
force acts in the opposite direction (in the same direction as in
the two other examples).
</p>
</html>"),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-100, 80}, {-82, 60}}, textString = "1)", lineColor = {0, 0, 255}), Text(extent = {{-100, 40}, {-82, 20}}, textString = "2)", lineColor = {0, 0, 255}), Text(extent = {{-100, -20}, {-82, -40}}, textString = "3)", lineColor = {0, 0, 255})}),
            experiment(StopTime = 1.0, Interval = 0.001));
        end SignConvention;

        model InitialConditions "Setting of initial conditions"
          extends Modelica.Icons.Example;
          Translational.Components.Fixed fixed2(s0 = 1) annotation(
            Placement(transformation(extent = {{-100, 30}, {-80, 50}}, rotation = 0)));
          Translational.Components.Spring s2(s_rel0 = 2, c = 1e3) annotation(
            Placement(transformation(extent = {{-60, 30}, {-40, 50}}, rotation = 0)));
          Translational.Components.Mass m3(L = 3, s(start = 4.5, fixed = true), v(fixed = true), m = 1) annotation(
            Placement(transformation(extent = {{-20, 30}, {0, 50}}, rotation = 0)));
          Translational.Components.SpringDamper sd2(s_rel0 = 4, c = 111, d = 1) annotation(
            Placement(transformation(extent = {{20, 30}, {40, 50}}, rotation = 0)));
          Translational.Components.Mass m4(L = 5, s(start = 12.5, fixed = true), v(fixed = true), m = 1) annotation(
            Placement(transformation(extent = {{60, 30}, {80, 50}}, rotation = 0)));
          Translational.Components.Fixed fixed1(s0 = -1) annotation(
            Placement(transformation(extent = {{-100, -70}, {-80, -50}}, rotation = 0)));
          Translational.Components.Spring s1(s_rel0 = 1, c = 1e3, s_rel(start = 0.5, fixed = true)) annotation(
            Placement(transformation(extent = {{-60, -70}, {-40, -50}}, rotation = 0)));
          Translational.Components.Mass m1(L = 1, v(fixed = true), m = 1) annotation(
            Placement(transformation(extent = {{-20, -70}, {0, -50}}, rotation = 0)));
          Translational.Components.SpringDamper sd1(s_rel0 = 1, c = 111, s_rel(start = 1, fixed = true), v_rel(fixed = true), d = 1) annotation(
            Placement(transformation(extent = {{20, -70}, {40, -50}}, rotation = 0)));
          Translational.Components.Mass m2(L = 2, m = 1) annotation(
            Placement(transformation(extent = {{60, -70}, {80, -50}}, rotation = 0)));
        equation
          connect(s2.flange_a, fixed2.flange) annotation(
            Line(points = {{-60, 40}, {-90, 40}}, color = {0, 127, 0}, smooth = Smooth.None));
          connect(s1.flange_a, fixed1.flange) annotation(
            Line(points = {{-60, -60}, {-90, -60}}, color = {0, 127, 0}, smooth = Smooth.None));
          connect(m1.flange_a, s1.flange_b) annotation(
            Line(points = {{-20, -60}, {-40, -60}}, color = {0, 127, 0}, smooth = Smooth.None));
          connect(sd1.flange_a, m1.flange_b) annotation(
            Line(points = {{20, -60}, {0, -60}}, color = {0, 127, 0}, smooth = Smooth.None));
          connect(m2.flange_a, sd1.flange_b) annotation(
            Line(points = {{60, -60}, {40, -60}}, color = {0, 127, 0}, smooth = Smooth.None));
          connect(m4.flange_a, sd2.flange_b) annotation(
            Line(points = {{60, 40}, {40, 40}}, color = {0, 127, 0}, smooth = Smooth.None));
          connect(sd2.flange_a, m3.flange_b) annotation(
            Line(points = {{20, 40}, {0, 40}}, color = {0, 127, 0}, smooth = Smooth.None));
          connect(m3.flange_a, s2.flange_b) annotation(
            Line(points = {{-20, 40}, {-40, 40}}, color = {0, 127, 0}, smooth = Smooth.None));
          annotation(
            Documentation(info = "<html>
<p>
There are several ways to set initial conditions.
In the first system the position of the mass m3 was defined
by using the modifier s(start=4.5), the position of m4 by s(start=12.5).
These positions were chosen such that the system is a rest. To calculate
these values start at the left (Fixed1) with a value of 1 m. The spring
has an unstretched length of 2 m and m3 an length of 3 m, which leads to
</p>

<pre>
        1   m (fixed1)
      + 2   m (spring s2)
      + 3/2 m (half of the length of mass m3)
      -------
        4,5 m = s(start = 4.5) for m3
      + 3/2 m (half of the length of mass m3)
      + 4   m (springDamper 2)
      + 5/2 m (half of length of mass m4)
      -------
       12,5 m = s(start = 12.5) for m4
</pre>

<p>
This selection of initial conditions has the effect that Dymola selects
those variables (m3.s and m4.s) as state variables.
In the second example the length of the springs are given as start values
but they cannot be used as state for pure springs (only for the spring/damper
combination). In this case the system is not at rest.
</p>

<p>
<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/Translational/InitialConditions.png\">
</p>

</html>"),
            experiment(StopTime = 5.0, Interval = 0.001),
            Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-90, 70}, {-90, 50}}, color = {0, 0, 255}, smooth = Smooth.None), Text(extent = {{-100, 82}, {-80, 72}}, lineColor = {255, 0, 0}, textString = "ref= 0 m"), Line(points = {{-10, 70}, {-10, 50}}, color = {0, 0, 255}, smooth = Smooth.None), Text(extent = {{-20, 82}, {0, 72}}, lineColor = {255, 0, 0}, textString = " 4.5 m "), Line(points = {{70, 70}, {70, 50}}, color = {0, 0, 255}, smooth = Smooth.None), Text(extent = {{60, 82}, {80, 72}}, lineColor = {255, 0, 0}, textString = "12.5 m"), Line(points = {{-54, 20}, {-54, 22}, {-60, 20}, {-54, 18}, {-54, 20}, {-46, 20}, {-46, 22}, {-40, 20}, {-46, 18}, {-46, 20}}, color = {0, 0, 255}, smooth = Smooth.None), Line(points = {{-14, 20}, {-14, 22}, {-20, 20}, {-14, 18}, {-14, 20}, {-6, 20}, {-6, 22}, {0, 20}, {-6, 18}, {-6, 20}}, color = {0, 0, 255}, smooth = Smooth.None), Line(points = {{26, 20}, {26, 22}, {20, 20}, {26, 18}, {26, 20}, {34, 20}, {34, 22}, {40, 20}, {34, 18}, {34, 20}}, color = {0, 0, 255}, smooth = Smooth.None), Line(points = {{66, 20}, {66, 22}, {60, 20}, {66, 18}, {66, 20}, {74, 20}, {74, 22}, {80, 20}, {74, 18}, {74, 20}}, color = {0, 0, 255}, smooth = Smooth.None), Text(extent = {{-60, 16}, {-40, 6}}, lineColor = {0, 0, 255}, textString = "  2 m  "), Text(extent = {{20, 16}, {40, 6}}, lineColor = {0, 0, 255}, textString = "  4 m  "), Text(extent = {{-20, 16}, {0, 6}}, lineColor = {0, 0, 0}, textString = "  3 m  "), Text(extent = {{60, 16}, {80, 6}}, lineColor = {0, 0, 0}, textString = "  5 m  "), Line(points = {{-54, -80}, {-54, -78}, {-60, -80}, {-54, -82}, {-54, -80}, {-46, -80}, {-46, -78}, {-40, -80}, {-46, -82}, {-46, -80}}, color = {0, 0, 255}, smooth = Smooth.None), Line(points = {{-14, -80}, {-14, -78}, {-20, -80}, {-14, -82}, {-14, -80}, {-6, -80}, {-6, -78}, {0, -80}, {-6, -82}, {-6, -80}}, color = {0, 0, 255}, smooth = Smooth.None), Line(points = {{26, -80}, {26, -78}, {20, -80}, {26, -82}, {26, -80}, {34, -80}, {34, -78}, {40, -80}, {34, -82}, {34, -80}}, color = {0, 0, 255}, smooth = Smooth.None), Line(points = {{66, -80}, {66, -78}, {60, -80}, {66, -82}, {66, -80}, {74, -80}, {74, -78}, {80, -80}, {74, -82}, {74, -80}}, color = {0, 0, 255}, smooth = Smooth.None), Text(extent = {{-60, -84}, {-40, -94}}, lineColor = {255, 0, 0}, textString = " 0.5 m
 (1 m) "), Text(extent = {{20, -84}, {40, -94}}, lineColor = {255, 0, 0}, textString = " 1 m
 (1 m) "), Text(extent = {{-20, -84}, {0, -94}}, lineColor = {0, 0, 0}, textString = "  1 m  "), Text(extent = {{60, -84}, {80, -94}}, lineColor = {0, 0, 0}, textString = "  2 m  "), Line(points = {{-90, -30}, {-90, -50}}, color = {0, 0, 255}, smooth = Smooth.None), Text(extent = {{-100, -18}, {-80, -28}}, lineColor = {255, 0, 0}, textString = "ref=-1 m"), Line(points = {{-10, -30}, {-10, -50}}, color = {0, 0, 255}, smooth = Smooth.None), Text(extent = {{-20, -18}, {0, -28}}, lineColor = {0, 0, 255}, textString = "  0 m  "), Line(points = {{70, -30}, {70, -50}}, color = {0, 0, 255}, smooth = Smooth.None), Text(extent = {{60, -18}, {80, -28}}, lineColor = {0, 0, 255}, textString = " 2.5 m ")}));
        end InitialConditions;

        model WhyArrows "Use of arrows in Mechanics.Translational"
          extends Modelica.Icons.Example;
          Translational.Components.Fixed fixed annotation(
            Placement(transformation(extent = {{-20, 20}, {0, 40}}, rotation = 0)));
          Translational.Components.Rod rod1(L = 1) annotation(
            Placement(transformation(extent = {{-48, 20}, {-28, 40}}, rotation = 0)));
          Translational.Components.Rod rod2(L = 1) annotation(
            Placement(transformation(extent = {{20, 20}, {40, 40}}, rotation = 0)));
          Translational.Components.Rod rod3(L = 1) annotation(
            Placement(transformation(extent = {{-30, 58}, {-50, 78}}, rotation = 0)));
          Translational.Sensors.PositionSensor positionSensor2 annotation(
            Placement(transformation(extent = {{60, 20}, {80, 40}}, rotation = 0)));
          Translational.Sensors.PositionSensor positionSensor1 annotation(
            Placement(transformation(extent = {{-60, 20}, {-80, 40}}, rotation = 0)));
          Translational.Sensors.PositionSensor positionSensor3 annotation(
            Placement(transformation(extent = {{-60, 58}, {-80, 78}}, rotation = 0)));
          Translational.Components.Fixed fixed1(s0 = -1.9) annotation(
            Placement(transformation(extent = {{-100, -60}, {-80, -40}}, rotation = 0)));
          Translational.Components.Spring spring1(s_rel0 = 2, c = 11) annotation(
            Placement(transformation(extent = {{-80, -60}, {-60, -40}}, rotation = 0)));
          Translational.Components.Mass mass1(L = 2, s(fixed = true), v(fixed = true), m = 1) annotation(
            Placement(transformation(extent = {{-50, -60}, {-30, -40}}, rotation = 0)));
          Translational.Components.Fixed fixed2(s0 = -1.9) annotation(
            Placement(transformation(extent = {{0, -60}, {20, -40}}, rotation = 0)));
          Translational.Components.Spring spring2(s_rel0 = 2, c = 11) annotation(
            Placement(transformation(extent = {{30, -60}, {50, -40}}, rotation = 0)));
          Translational.Components.Mass inertia2(L = 2, m = 1, s(fixed = true), v(fixed = true)) annotation(
            Placement(transformation(extent = {{80, -60}, {60, -40}}, rotation = 0)));
        equation
          connect(spring1.flange_b, mass1.flange_b) annotation(
            Line(points = {{-60, -50}, {-60, -72}, {-30, -72}, {-30, -50}}, color = {0, 191, 0}));
          connect(spring2.flange_b, inertia2.flange_b) annotation(
            Line(points = {{50, -50}, {60, -50}}, color = {0, 191, 0}));
          connect(rod3.flange_b, positionSensor3.flange) annotation(
            Line(points = {{-50, 68}, {-60, 68}}, color = {0, 127, 0}, smooth = Smooth.None));
          connect(rod1.flange_a, positionSensor1.flange) annotation(
            Line(points = {{-48, 30}, {-60, 30}}, color = {0, 127, 0}, smooth = Smooth.None));
          connect(rod1.flange_b, fixed.flange) annotation(
            Line(points = {{-28, 30}, {-10, 30}}, color = {0, 127, 0}, smooth = Smooth.None));
          connect(rod3.flange_a, fixed.flange) annotation(
            Line(points = {{-30, 68}, {-10, 68}, {-10, 30}}, color = {0, 127, 0}, smooth = Smooth.None));
          connect(fixed.flange, rod2.flange_a) annotation(
            Line(points = {{-10, 30}, {20, 30}}, color = {0, 127, 0}, smooth = Smooth.None));
          connect(rod2.flange_b, positionSensor2.flange) annotation(
            Line(points = {{40, 30}, {60, 30}}, color = {0, 127, 0}, smooth = Smooth.None));
          connect(fixed1.flange, spring1.flange_a) annotation(
            Line(points = {{-90, -50}, {-80, -50}}, color = {0, 127, 0}, smooth = Smooth.None));
          connect(fixed2.flange, spring2.flange_a) annotation(
            Line(points = {{10, -50}, {30, -50}}, color = {0, 127, 0}, smooth = Smooth.None));
          annotation(
            Documentation(info = "<html>
<p>
When using the models of the translational sublibrary
it is recommended to make sure that all arrows point in
the same direction because then all component have the
same reference system.
In the example the distance from flange_a of Rod1 to flange_b
of Rod2 is 2 m. The distance from flange_a of Rod1 to flange_b
of Rod3 is also 2 m though it is difficult to see that. Without
the arrows it would be almost impossible to notice.
That all arrows point in the same direction is a sufficient
condition for an easy use of the library. There are cases
where horizontally flipped models can be used without
problems.
</p>
</html>"),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-84, 10}, {88, 2}}, lineColor = {0, 0, 255}, textString = "positionSensor2.s = positionSensor3.s"), Text(extent = {{-78, -4}, {86, -12}}, lineColor = {0, 0, 255}, textString = "positionSensor3.s <>positionSensor1.s"), Text(extent = {{-82, -80}, {92, -88}}, textString = "Both systems are equivalent", lineColor = {0, 0, 255}), Line(points = {{-90, -28}, {90, -28}}, thickness = 0.5, color = {0, 0, 255})}),
            experiment(StopTime = 1.0, Interval = 0.001));
        end WhyArrows;

        model Accelerate "Use of model accelerate."
          extends Modelica.Icons.Example;
          Translational.Sources.Accelerate accelerate annotation(
            Placement(transformation(extent = {{-40, 20}, {-20, 40}}, rotation = 0)));
          Translational.Components.Mass mass(L = 1, m = 1) annotation(
            Placement(transformation(extent = {{0, 20}, {20, 40}}, rotation = 0)));
          Modelica.Blocks.Sources.Constant constantAcc(k = 1) annotation(
            Placement(transformation(extent = {{-80, 20}, {-60, 40}}, rotation = 0)));
        equation
          connect(accelerate.flange, mass.flange_a) annotation(
            Line(points = {{-20, 30}, {0, 30}}, color = {0, 127, 0}, smooth = Smooth.None));
          connect(constantAcc.y, accelerate.a_ref) annotation(
            Line(points = {{-59, 30}, {-42, 30}}, color = {0, 0, 127}, smooth = Smooth.None));
          annotation(
            Documentation(info = "<html>
<p>
Demonstrate usage of component Sources.Accelerate by moving a massing
with a predefined acceleration.
</p>
</html>"),
            experiment(StopTime = 1.0, Interval = 0.001));
        end Accelerate;

        model Damper "Use of damper models."
          extends Modelica.Icons.Example;
          Translational.Components.Mass mass1(L = 1, s(start = 3, fixed = true), v(start = 10, fixed = true), m = 1) annotation(
            Placement(transformation(extent = {{-80, 60}, {-60, 80}}, rotation = 0)));
          Translational.Components.Damper damper1(d = 25) annotation(
            Placement(transformation(extent = {{-20, 60}, {0, 80}}, rotation = 0)));
          Translational.Components.Fixed fixed1(s0 = 4.5) annotation(
            Placement(transformation(extent = {{22, 60}, {42, 80}}, rotation = 0)));
          Translational.Components.Mass mass2(L = 1, s(start = 3, fixed = true), v(start = 10, fixed = true), m = 1) annotation(
            Placement(transformation(extent = {{-80, 0}, {-60, 20}}, rotation = 0)));
          Translational.Components.Damper damper2(d = 25) annotation(
            Placement(transformation(extent = {{-20, 0}, {0, 20}}, rotation = 0)));
          Translational.Components.Fixed fixed2(s0 = 4.5) annotation(
            Placement(transformation(extent = {{20, 0}, {40, 20}}, rotation = 0)));
          Translational.Components.Mass mass3(L = 1, s(start = 3, fixed = true), v(start = 10, fixed = true), m = 1) annotation(
            Placement(transformation(extent = {{-80, -60}, {-60, -40}}, rotation = 0)));
          Translational.Components.Fixed fixed3(s0 = 4.5) annotation(
            Placement(transformation(extent = {{20, -60}, {40, -40}}, rotation = 0)));
          Translational.Components.Spring spring2(s_rel0 = 1, c = 1) annotation(
            Placement(transformation(extent = {{-20, -20}, {0, 0}}, rotation = 0)));
          Translational.Components.SpringDamper springDamper3(s_rel0 = 1, d = 25, c = 1) annotation(
            Placement(transformation(extent = {{-20, -60}, {0, -40}}, rotation = 0)));
        equation
          connect(mass1.flange_b, damper1.flange_a) annotation(
            Line(points = {{-60, 70}, {-20, 70}}, color = {0, 191, 0}));
          connect(mass2.flange_b, damper2.flange_a) annotation(
            Line(points = {{-60, 10}, {-20, 10}}, color = {0, 191, 0}));
          connect(damper2.flange_b, spring2.flange_b) annotation(
            Line(points = {{0, 10}, {0, -10}}, color = {0, 191, 0}));
          connect(damper2.flange_a, spring2.flange_a) annotation(
            Line(points = {{-20, 10}, {-20, -10}}, color = {0, 191, 0}));
          connect(mass3.flange_b, springDamper3.flange_a) annotation(
            Line(points = {{-60, -50}, {-20, -50}}, color = {0, 191, 0}));
          connect(damper1.flange_b, fixed1.flange) annotation(
            Line(points = {{0, 70}, {32, 70}}, color = {0, 127, 0}, smooth = Smooth.None));
          connect(damper2.flange_b, fixed2.flange) annotation(
            Line(points = {{0, 10}, {30, 10}}, color = {0, 127, 0}, smooth = Smooth.None));
          connect(springDamper3.flange_b, fixed3.flange) annotation(
            Line(points = {{0, -50}, {30, -50}}, color = {0, 127, 0}, smooth = Smooth.None));
          annotation(
            Documentation(info = "<html>
<p>
Demonstrate usage of damper components in different variants.
</p>
</html>"),
            experiment(StopTime = 1.0, Interval = 0.001));
        end Damper;

        model Oscillator "Oscillator demonstrates the use of initial conditions."
          extends Modelica.Icons.Example;
          Translational.Components.Mass mass1(L = 1, s(start = -0.5, fixed = true), v(start = 0, fixed = true), m = 1) annotation(
            Placement(transformation(extent = {{-20, 40}, {0, 60}}, rotation = 0)));
          Translational.Components.Spring spring1(s_rel0 = 1, c = 10000) annotation(
            Placement(transformation(extent = {{20, 40}, {40, 60}}, rotation = 0)));
          Translational.Components.Fixed fixed1(s0 = 1) annotation(
            Placement(transformation(extent = {{60, 40}, {80, 60}}, rotation = 0)));
          Translational.Sources.Force force1 annotation(
            Placement(transformation(extent = {{-60, 40}, {-40, 60}}, rotation = 0)));
          Modelica.Blocks.Sources.Sine sine1(freqHz = 15.9155) annotation(
            Placement(transformation(extent = {{-100, 40}, {-80, 60}}, rotation = 0)));
          Translational.Components.Mass mass2(L = 1, s(start = -0.5, fixed = true), v(start = 0, fixed = true), m = 1) annotation(
            Placement(transformation(extent = {{-20, -60}, {0, -40}}, rotation = 0)));
          Translational.Components.Spring spring2(s_rel0 = 1, c = 10000) annotation(
            Placement(transformation(extent = {{20, -60}, {40, -40}}, rotation = 0)));
          Translational.Components.Fixed fixed2(s0 = 1) annotation(
            Placement(transformation(extent = {{60, -60}, {80, -40}}, rotation = 0)));
          Translational.Sources.Force force2 annotation(
            Placement(transformation(extent = {{-60, -60}, {-40, -40}}, rotation = 0)));
          Modelica.Blocks.Sources.Sine sine2(freqHz = 15.9155) annotation(
            Placement(transformation(extent = {{-100, -60}, {-80, -40}}, rotation = 0)));
          Translational.Components.Damper damper1(d = 10) annotation(
            Placement(transformation(extent = {{20, -36}, {40, -16}}, rotation = 0)));
        equation
          connect(mass1.flange_b, spring1.flange_a) annotation(
            Line(points = {{0, 50}, {20, 50}}, color = {0, 191, 0}));
          connect(spring2.flange_a, damper1.flange_a) annotation(
            Line(points = {{20, -50}, {20, -26}}, color = {0, 191, 0}));
          connect(mass2.flange_b, spring2.flange_a) annotation(
            Line(points = {{0, -50}, {20, -50}}, color = {0, 191, 0}));
          connect(damper1.flange_b, spring2.flange_b) annotation(
            Line(points = {{40, -26}, {40, -50}}, color = {0, 191, 0}));
          connect(sine1.y, force1.f) annotation(
            Line(points = {{-79, 50}, {-62, 50}}, color = {0, 0, 127}));
          connect(sine2.y, force2.f) annotation(
            Line(points = {{-79, -50}, {-62, -50}}, color = {0, 0, 127}));
          connect(spring1.flange_b, fixed1.flange) annotation(
            Line(points = {{40, 50}, {70, 50}}, color = {0, 127, 0}, smooth = Smooth.None));
          connect(force2.flange, mass2.flange_a) annotation(
            Line(points = {{-40, -50}, {-20, -50}}, color = {0, 127, 0}, smooth = Smooth.None));
          connect(force1.flange, mass1.flange_a) annotation(
            Line(points = {{-40, 50}, {-20, 50}}, color = {0, 127, 0}, smooth = Smooth.None));
          connect(spring2.flange_b, fixed2.flange) annotation(
            Line(points = {{40, -50}, {70, -50}}, color = {0, 127, 0}, smooth = Smooth.None));
          annotation(
            Documentation(info = "<html>
<p>
A spring - mass system is a mechanical oscillator. If no
damping is included and the system is excited at resonance
frequency infinite amplitudes will result.
The resonant frequency is given by
omega_res = sqrt(c / m)
with:
</p>

<pre>
      c spring stiffness
      m mass
</pre>

<p>
To make sure that the system is initially at rest the initial
conditions s(start=0) and v(start=0) for the SlidingMass
are set.
If damping is added the amplitudes are bounded.
</p>
</html>"),
            experiment(StopTime = 1.0, Interval = 0.001));
        end Oscillator;

        model Sensors "Sensors for translational systems."
          import Modelica;
          extends Modelica.Icons.Example;
          Translational.Sensors.ForceSensor forceSensor annotation(
            Placement(transformation(extent = {{-34, 40}, {-14, 60}}, rotation = 0)));
          Translational.Sensors.SpeedSensor speedSensor1 annotation(
            Placement(transformation(extent = {{40, -40}, {60, -20}}, rotation = 0)));
          Translational.Sensors.PositionSensor positionSensor1 annotation(
            Placement(transformation(extent = {{40, 0}, {60, 20}}, rotation = 0)));
          Translational.Sensors.AccSensor accSensor1 annotation(
            Placement(transformation(extent = {{40, -80}, {60, -60}}, rotation = 0)));
          Translational.Components.Mass mass(L = 1, s(fixed = true), v(fixed = true), m = 1) annotation(
            Placement(transformation(extent = {{20, 40}, {40, 60}}, rotation = 0)));
          Translational.Sources.Force force annotation(
            Placement(transformation(extent = {{-64, 40}, {-44, 60}}, rotation = 0)));
          Modelica.Blocks.Sources.Sine sineForce(amplitude = 10, freqHz = 4) annotation(
            Placement(transformation(extent = {{-100, 40}, {-80, 60}}, rotation = 0)));
          Translational.Sensors.PositionSensor positionSensor2 annotation(
            Placement(transformation(extent = {{60, 40}, {80, 60}}, rotation = 0)));
          Modelica.Mechanics.Translational.Sensors.MultiSensor multiSensor annotation(
            Placement(transformation(extent = {{-8, 40}, {12, 60}})));
        equation
          connect(sineForce.y, force.f) annotation(
            Line(points = {{-79, 50}, {-66, 50}}, color = {0, 0, 127}));
          connect(forceSensor.flange_a, force.flange) annotation(
            Line(points = {{-34, 50}, {-44, 50}}, color = {0, 127, 0}, smooth = Smooth.None));
          connect(mass.flange_a, positionSensor1.flange) annotation(
            Line(points = {{20, 50}, {20, 10}, {40, 10}}, color = {0, 127, 0}, smooth = Smooth.None));
          connect(mass.flange_a, speedSensor1.flange) annotation(
            Line(points = {{20, 50}, {20, -30}, {40, -30}}, color = {0, 127, 0}, smooth = Smooth.None));
          connect(mass.flange_a, accSensor1.flange) annotation(
            Line(points = {{20, 50}, {20, -70}, {40, -70}}, color = {0, 127, 0}, smooth = Smooth.None));
          connect(mass.flange_b, positionSensor2.flange) annotation(
            Line(points = {{40, 50}, {60, 50}}, color = {0, 127, 0}, smooth = Smooth.None));
          connect(forceSensor.flange_b, multiSensor.flange_a) annotation(
            Line(points = {{-14, 50}, {-8, 50}}, color = {0, 127, 0}, smooth = Smooth.None));
          connect(multiSensor.flange_b, mass.flange_a) annotation(
            Line(points = {{12, 50}, {20, 50}}, color = {0, 127, 0}, smooth = Smooth.None));
          annotation(
            Documentation(info = "<html>
<p>
These sensors measure
</p>

<pre>
   force f in N
   position s in m
   velocity v in m/s
   acceleration a in m/s2
</pre>

<p>
The measured velocity and acceleration is independent on
the flange the sensor is connected to. The position
depends on the flange (flange_a or flange_b) and the
length L of the component.
Plot PositionSensor1.s, PositionSensor2.s and SlidingMass1.s
to see the difference.
</p>
</html>"),
            experiment(StopTime = 1.0, Interval = 0.001));
        end Sensors;

        model Friction "Use of model Stop"
          extends Modelica.Icons.Example;
          Modelica.Mechanics.Translational.Components.MassWithStopAndFriction stop1(L = 1, s(fixed = true), v(fixed = true), smax = 25, smin = -25, m = 1, F_prop = 1, F_Coulomb = 5, F_Stribeck = 10, fexp = 2) annotation(
            Placement(transformation(extent = {{20, 60}, {40, 80}}, rotation = 0)));
          Translational.Sources.Force force annotation(
            Placement(transformation(extent = {{-20, 60}, {0, 80}}, rotation = 0)));
          Modelica.Blocks.Sources.Sine sineForce(amplitude = 25, freqHz = 0.25) annotation(
            Placement(transformation(extent = {{-60, 60}, {-40, 80}}, rotation = 0)));
          Modelica.Mechanics.Translational.Components.MassWithStopAndFriction stop2(L = 1, smax = 0.9, smin = -0.9, F_Coulomb = 3, F_Stribeck = 5, s(start = 0, fixed = true), m = 1, F_prop = 1, fexp = 2, v(start = -5, fixed = true)) annotation(
            Placement(transformation(extent = {{42, -60}, {62, -40}}, rotation = 0)));
          Translational.Components.Spring spring(s_rel0 = 1, c = 500) annotation(
            Placement(transformation(extent = {{2, -60}, {22, -40}}, rotation = 0)));
          Translational.Components.Fixed fixed2(s0 = -1.75) annotation(
            Placement(transformation(extent = {{-40, -60}, {-20, -40}}, rotation = 0)));
          Translational.Sources.Force force2 annotation(
            Placement(transformation(extent = {{-22, 0}, {-2, 20}}, rotation = 0)));
          Components.Mass mass(m = 1, L = 1, s(fixed = true), v(fixed = true)) annotation(
            Placement(transformation(extent = {{10, 0}, {30, 20}})));
          Components.SupportFriction supportFriction(f_pos = Examples.Utilities.GenerateStribeckFrictionTable(F_prop = 1, F_Coulomb = 5, F_Stribeck = 10, fexp = 2, v_max = 12, nTable = 50)) annotation(
            Placement(transformation(extent = {{40, 0}, {60, 20}})));
        equation
          connect(spring.flange_b, stop2.flange_a) annotation(
            Line(points = {{22, -50}, {42, -50}}, color = {0, 191, 0}));
          connect(sineForce.y, force.f) annotation(
            Line(points = {{-39, 70}, {-22, 70}}, color = {0, 0, 127}));
          connect(spring.flange_a, fixed2.flange) annotation(
            Line(points = {{2, -50}, {-30, -50}}, color = {0, 127, 0}, smooth = Smooth.None));
          connect(force.flange, stop1.flange_a) annotation(
            Line(points = {{0, 70}, {20, 70}}, color = {0, 127, 0}, smooth = Smooth.None));
          connect(force2.flange, mass.flange_a) annotation(
            Line(points = {{-2, 10}, {10, 10}}, color = {0, 127, 0}, smooth = Smooth.None));
          connect(mass.flange_b, supportFriction.flange_a) annotation(
            Line(points = {{30, 10}, {40, 10}}, color = {0, 127, 0}, smooth = Smooth.None));
          connect(sineForce.y, force2.f) annotation(
            Line(points = {{-39, 70}, {-30, 70}, {-30, 10}, {-24, 10}}, color = {0, 0, 127}, smooth = Smooth.None));
          annotation(
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-100, 80}, {-80, 60}}, textString = "1)", lineColor = {0, 0, 255}), Text(extent = {{-100, 20}, {-80, 0}}, textString = "2)", lineColor = {0, 0, 255}), Text(extent = {{-100, -40}, {-80, -60}}, lineColor = {0, 0, 255}, textString = "3)")}),
            Documentation(info = "<html>
<ol>
<li> Simulate and then plot stop1.f as a function of stop1.v
     This gives the Stribeck curve.</li>
<li> The same model is also available by modeling the system with a Mass and
     a SupportFriction model. The SupportFriction model defines the friction characteristic
     with a table. The table is constructed with function
     Examples.Utilities.GenerateStribeckFrictionTable(..) to generate the
     same friction characteristic as with stop1.
     The simulation results of stop1 and of model mass are therefore identical.</li>
<li> Model stop2 gives an example for a hard stop. However there
     can arise some problems with the used modeling approach (use of
     <b>reinit</b>(..), convergence problems). In this case use the ElastoGap
     to model a stop (see example Preload).</li>
</ol>
</html>"),
            experiment(StopTime = 5.0, Interval = 0.001));
        end Friction;

        model PreLoad "Preload of a spool using ElastoGap models."
          extends Modelica.Icons.Example;
          Translational.Components.ElastoGap innerContactA(c = 1000e3, d = 250, s_rel0 = 0.001) annotation(
            Placement(transformation(extent = {{-70, 20}, {-50, 40}}, rotation = 0)));
          Translational.Components.ElastoGap innerContactB(c = 1000e3, d = 250, s_rel0 = 0.001) annotation(
            Placement(transformation(extent = {{50, 20}, {70, 40}}, rotation = 0)));
          Translational.Components.Mass spool(L = 0.19, m = 0.150, s(start = 0.01475, fixed = true), v(fixed = true)) annotation(
            Placement(transformation(extent = {{0, -40}, {40, 0}}, rotation = 0)));
          Translational.Components.Fixed fixedLe(s0 = -0.0955) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 270, origin = {-80, 90})));
          Translational.Components.Mass springPlateA(m = 10e-3, L = 0.002, s(start = -0.093, fixed = true), v(fixed = true)) annotation(
            Placement(transformation(extent = {{-38, 60}, {-18, 80}}, rotation = 0)));
          Translational.Components.Mass springPlateB(m = 10e-3, s(start = -0.06925, fixed = true), L = 0.002, v(fixed = true)) annotation(
            Placement(transformation(extent = {{20, 60}, {40, 80}}, rotation = 0)));
          Translational.Components.Spring spring(c = 20e3, s_rel0 = 0.025) annotation(
            Placement(transformation(extent = {{-10, 60}, {10, 80}}, rotation = 0)));
          Translational.Components.ElastoGap outerContactA(c = 1000e3, d = 250, s_rel0 = 0.0015) annotation(
            Placement(transformation(extent = {{-70, 60}, {-50, 80}}, rotation = 0)));
          Translational.Components.ElastoGap outerContactB(c = 1000e3, d = 250, s_rel0 = 0.0015) annotation(
            Placement(transformation(extent = {{50, 60}, {70, 80}}, rotation = 0)));
          Translational.Components.Rod rod1(L = 0.007) annotation(
            Placement(transformation(extent = {{-40, 30}, {-20, 50}}, rotation = 0)));
          Translational.Components.Damper friction(d = 2500) annotation(
            Placement(transformation(origin = {-80, 50}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          Translational.Sources.Force force annotation(
            Placement(transformation(extent = {{-38, -30}, {-18, -10}}, rotation = 0)));
          Translational.Components.Rod housing(L = 0.0305) annotation(
            Placement(transformation(extent = {{-10, 80}, {10, 100}}, rotation = 0)));
          Translational.Components.Rod rod3(L = 0.00575) annotation(
            Placement(transformation(extent = {{-40, -2}, {-20, 18}}, rotation = 0)));
          Translational.Components.Rod rod4(L = 0.00575) annotation(
            Placement(transformation(extent = {{20, -2}, {40, 18}}, rotation = 0)));
          Translational.Components.Rod rod2(L = 0.007) annotation(
            Placement(transformation(extent = {{20, 30}, {40, 50}}, rotation = 0)));
          Modelica.Blocks.Sources.Sine sineForce(amplitude = 150, freqHz = 0.01) annotation(
            Placement(transformation(extent = {{-78, -30}, {-58, -10}}, rotation = 0)));
        equation
          connect(outerContactA.flange_b, springPlateA.flange_a) annotation(
            Line(points = {{-50, 70}, {-38, 70}}, color = {0, 191, 0}));
          connect(springPlateA.flange_b, spring.flange_a) annotation(
            Line(points = {{-18, 70}, {-18, 70}, {-10, 70}}, color = {0, 191, 0}));
          connect(spring.flange_b, springPlateB.flange_a) annotation(
            Line(points = {{10, 70}, {20, 70}}, color = {0, 191, 0}));
          connect(springPlateB.flange_b, outerContactB.flange_a) annotation(
            Line(points = {{40, 70}, {40, 70}, {50, 70}}, color = {0, 191, 0}));
          connect(outerContactB.flange_b, housing.flange_b) annotation(
            Line(points = {{70, 70}, {70, 90}, {10, 90}}, color = {0, 191, 0}));
          connect(springPlateA.flange_b, rod1.flange_a) annotation(
            Line(points = {{-18, 70}, {-18, 52}, {-40, 52}, {-40, 40}}, color = {0, 191, 0}));
          connect(innerContactA.flange_a, rod3.flange_a) annotation(
            Line(points = {{-70, 30}, {-80, 30}, {-80, 8}, {-40, 8}}, color = {0, 191, 0}));
          connect(innerContactA.flange_b, rod1.flange_b) annotation(
            Line(points = {{-50, 30}, {-20, 30}, {-20, 40}}, color = {0, 191, 0}));
          connect(rod2.flange_a, innerContactB.flange_a) annotation(
            Line(points = {{20, 40}, {20, 30}, {50, 30}}, color = {0, 191, 0}));
          connect(rod4.flange_b, innerContactB.flange_b) annotation(
            Line(points = {{40, 8}, {70, 8}, {70, 30}}, color = {0, 191, 0}));
          connect(friction.flange_b, rod3.flange_a) annotation(
            Line(points = {{-80, 40}, {-80, 8}, {-40, 8}}, color = {0, 191, 0}));
          connect(rod3.flange_b, rod4.flange_a) annotation(
            Line(points = {{-20, 8}, {20, 8}}, color = {0, 191, 0}));
          connect(rod2.flange_b, springPlateB.flange_a) annotation(
            Line(points = {{40, 40}, {40, 52}, {20, 52}, {20, 70}}, color = {0, 191, 0}));
          connect(spool.flange_a, rod4.flange_a) annotation(
            Line(points = {{0, -20}, {0, 8}, {20, 8}}, color = {0, 191, 0}));
          connect(sineForce.y, force.f) annotation(
            Line(points = {{-57, -20}, {-46, -20}, {-40, -20}}, color = {0, 0, 127}));
          connect(force.flange, spool.flange_a) annotation(
            Line(points = {{-18, -20}, {0, -20}}, color = {0, 127, 0}, smooth = Smooth.None));
          connect(outerContactA.flange_a, fixedLe.flange) annotation(
            Line(points = {{-70, 70}, {-80, 70}, {-80, 90}}, color = {0, 127, 0}, smooth = Smooth.None));
          connect(housing.flange_a, fixedLe.flange) annotation(
            Line(points = {{-10, 90}, {-80, 90}}, color = {0, 127, 0}, smooth = Smooth.None));
          connect(friction.flange_a, fixedLe.flange) annotation(
            Line(points = {{-80, 60}, {-80, 90}}, color = {0, 127, 0}, smooth = Smooth.None));
          annotation(
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-92, -72}, {90, -80}}, textString = "positive force => spool moves in positive direction ", lineColor = {0, 0, 255}), Text(extent = {{-48, -52}, {42, -60}}, textString = "Simulate for 100 s", lineColor = {0, 0, 255}), Text(extent = {{-100, -62}, {88, -70}}, lineColor = {0, 0, 255}, textString = "plot spool.s as a function of force.f")}),
            Documentation(info = "<html>
<p>
When designing hydraulic valves it is often necessary to hold the spool in
a certain position as long as an external force is below a threshold value.
If this force exceeds the threshold value a linear relation between force
and position is desired.
There are designs that need only one spring to accomplish this task. Using
the ElastoGap elements this design can be modelled easily.
Drawing of spool.
</p>

<p>
<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/Translational/PreLoad.png\"><br>
<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/Translational/PreLoad3.png\"><br>
<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/Translational/PreLoad4.png\"><br>
</p>

<p>
Spool position s as a function of working force f.
</p>

<p>
<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/Translational/PreLoad2.png\">
</p>
</html>"),
            experiment(StopTime = 100, Interval = 0.1));
        end PreLoad;

        model ElastoGap "Demonstrate usage of ElastoGap"
          extends Modelica.Icons.Example;
          Components.Fixed fixed annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}})));
          Components.Rod rod1(L = 2) annotation(
            Placement(transformation(extent = {{-40, -10}, {-20, 10}})));
          Components.Rod rod2(L = 2) annotation(
            Placement(transformation(extent = {{20, -10}, {40, 10}})));
          Components.SpringDamper springDamper1(c = 10, s_rel0 = 1, s_rel(fixed = false, start = 1), d = 1.5) annotation(
            Placement(transformation(extent = {{-40, 20}, {-20, 40}})));
          Components.SpringDamper springDamper2(c = 10, s_rel0 = 1, s_rel(fixed = false, start = 1), d = 1.5) annotation(
            Placement(transformation(extent = {{20, 20}, {40, 40}})));
          Components.Mass mass1(s(fixed = true, start = 2), L = 0, m = 1, v(fixed = true)) annotation(
            Placement(transformation(extent = {{-10, 20}, {10, 40}})));
          Components.ElastoGap elastoGap1(c = 10, s_rel(fixed = false, start = 1.5), s_rel0 = 1.5, d = 1.5) annotation(
            Placement(transformation(extent = {{-40, -40}, {-20, -20}})));
          Components.ElastoGap elastoGap2(c = 10, s_rel(fixed = false, start = 1.5), s_rel0 = 1.5, d = 1.5) annotation(
            Placement(transformation(extent = {{20, -40}, {40, -20}})));
          Components.Mass mass2(s(fixed = true, start = 2), L = 0, m = 1, v(fixed = true)) annotation(
            Placement(transformation(extent = {{-10, -40}, {10, -20}})));
          parameter SI.TranslationalDampingConstant d = 1.5 "Damping constant";
        equation
          connect(rod1.flange_b, fixed.flange) annotation(
            Line(points = {{-20, 0}, {0, 0}}, color = {0, 127, 0}, smooth = Smooth.None));
          connect(fixed.flange, rod2.flange_a) annotation(
            Line(points = {{0, 0}, {20, 0}}, color = {0, 127, 0}, smooth = Smooth.None));
          connect(springDamper1.flange_a, rod1.flange_a) annotation(
            Line(points = {{-40, 30}, {-48, 30}, {-48, 0}, {-40, 0}}, color = {0, 127, 0}, smooth = Smooth.None));
          connect(springDamper2.flange_b, rod2.flange_b) annotation(
            Line(points = {{40, 30}, {50, 30}, {50, 0}, {40, 0}}, color = {0, 127, 0}, smooth = Smooth.None));
          connect(springDamper1.flange_b, mass1.flange_a) annotation(
            Line(points = {{-20, 30}, {-10, 30}}, color = {0, 127, 0}, smooth = Smooth.None));
          connect(mass1.flange_b, springDamper2.flange_a) annotation(
            Line(points = {{10, 30}, {20, 30}}, color = {0, 127, 0}, smooth = Smooth.None));
          connect(rod1.flange_a, elastoGap1.flange_a) annotation(
            Line(points = {{-40, 0}, {-48, 0}, {-48, -30}, {-40, -30}}, color = {0, 127, 0}, smooth = Smooth.None));
          connect(rod2.flange_b, elastoGap2.flange_b) annotation(
            Line(points = {{40, 0}, {50, 0}, {50, -30}, {40, -30}}, color = {0, 127, 0}, smooth = Smooth.None));
          connect(elastoGap1.flange_b, mass2.flange_a) annotation(
            Line(points = {{-20, -30}, {-10, -30}}, color = {0, 127, 0}, smooth = Smooth.None));
          connect(mass2.flange_b, elastoGap2.flange_a) annotation(
            Line(points = {{10, -30}, {20, -30}}, color = {0, 127, 0}, smooth = Smooth.None));
          annotation(
            Documentation(info = "<html>
<p>
This model demonstrates the effect of ElastoGaps on eigenfrequency:<br>
Plot mass1.s and mass2.s as well as mass1.v and mass2.v<br><br>
mass1 is moved by both spring forces all the time.<br>
Since elastoGap1 lifts off at s &gt; -0.5 m and elastoGap2 lifts off s &lt; +0.5 m,
mass2 moves freely as long as -0.5 m &lt; s &lt; +0.5 m.
</p>
</html>"),
            experiment(StopTime = 1.0, Interval = 0.01));
        end ElastoGap;

        model Brake "Demonstrate braking of a translational moving mass"
          extends Modelica.Icons.Example;
          Modelica.Mechanics.Translational.Components.Brake brake(fn_max = 1, useSupport = false) annotation(
            Placement(transformation(extent = {{6, 40}, {26, 20}})));
          Modelica.Mechanics.Translational.Components.Mass mass1(m = 1, s(fixed = true), v(start = 1, fixed = true)) annotation(
            Placement(transformation(extent = {{-34, 20}, {-14, 40}})));
          Modelica.Blocks.Sources.Step step(startTime = 0.1, height = 2) annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0, origin = {-24, -10})));
          Modelica.Mechanics.Translational.Components.Brake brake1(fn_max = 1, useSupport = true) annotation(
            Placement(transformation(extent = {{6, -60}, {26, -40}})));
          Modelica.Mechanics.Translational.Components.Mass mass2(m = 1, s(fixed = true), v(start = 1, fixed = true)) annotation(
            Placement(transformation(extent = {{-34, -60}, {-14, -40}})));
          Modelica.Mechanics.Translational.Components.Fixed fixed annotation(
            Placement(transformation(extent = {{6, -80}, {26, -60}})));
        equation
          connect(mass1.flange_b, brake.flange_a) annotation(
            Line(points = {{-14, 30}, {6, 30}}, color = {0, 127, 0}, smooth = Smooth.None));
          connect(step.y, brake.f_normalized) annotation(
            Line(points = {{-13, -10}, {16, -10}, {16, 19}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(mass2.flange_b, brake1.flange_a) annotation(
            Line(points = {{-14, -50}, {6, -50}}, color = {0, 127, 0}, smooth = Smooth.None));
          connect(step.y, brake1.f_normalized) annotation(
            Line(points = {{-13, -10}, {16, -10}, {16, -39}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(fixed.flange, brake1.support) annotation(
            Line(points = {{16, -70}, {16, -60}}, color = {0, 127, 0}, smooth = Smooth.None));
          annotation(
            Documentation(info = "<html>
<p>
This model consists of a mass with an initial velocity of 1 m/s.
After 0.1 s, a brake is activated and it is shown that the mass decelerates until
it arrives at rest and remains at rest. Two versions of this system are present,
one where the brake is implicitly grounded and one where it is explicitly grounded.
</p>
</html>"),
            experiment(StopTime = 2.0, Interval = 0.001));
        end Brake;

        model HeatLosses "Demonstrate the modeling of heat losses"
          extends Modelica.Icons.Example;
          Components.Mass mass1(m = 1, s(fixed = true), L = 0.1, v(fixed = true)) annotation(
            Placement(transformation(extent = {{-60, 20}, {-40, 40}})));
          Components.SpringDamper springDamper(s_rel(fixed = true), v_rel(fixed = true), c = 100, d = 10, useHeatPort = true) annotation(
            Placement(transformation(extent = {{-30, 20}, {-10, 40}})));
          Components.Damper damper(d = 10, useHeatPort = true) annotation(
            Placement(transformation(extent = {{-10, 10}, {10, -10}}, rotation = -90, origin = {-60, -10})));
          Components.ElastoGap elastoGap(c = 100, d = 20, s_rel0 = -0.02, useHeatPort = true) annotation(
            Placement(transformation(extent = {{-90, -10}, {-70, 10}})));
          Components.Fixed fixed1 annotation(
            Placement(transformation(extent = {{-70, -40}, {-50, -20}})));
          Sources.Force force annotation(
            Placement(transformation(extent = {{-90, 20}, {-70, 40}})));
          Blocks.Sources.Sine sine1(freqHz = 1, amplitude = 20) annotation(
            Placement(transformation(extent = {{-120, 20}, {-100, 40}})));
          Components.Mass mass2(m = 1, L = 0.1, s(fixed = false), v(fixed = false)) annotation(
            Placement(transformation(extent = {{0, 20}, {20, 40}})));
          Components.SupportFriction supportFriction(useHeatPort = true) annotation(
            Placement(transformation(extent = {{30, 20}, {50, 40}})));
          Components.Spring spring(c = 100, s_rel(fixed = true)) annotation(
            Placement(transformation(extent = {{60, 20}, {80, 40}})));
          Components.Mass mass3(m = 1, L = 0.1, s(fixed = false), v(fixed = true)) annotation(
            Placement(transformation(extent = {{90, 20}, {110, 40}})));
          Components.Brake brake(fn_max = 10, useHeatPort = true) annotation(
            Placement(transformation(extent = {{120, 20}, {140, 40}})));
          Blocks.Sources.Sine sine2(amplitude = 10, freqHz = 2) annotation(
            Placement(transformation(extent = {{100, 50}, {120, 70}})));
          Components.MassWithStopAndFriction massWithStopAndFriction(L = 0.1, m = 1, F_prop = 0.5, F_Coulomb = 1, F_Stribeck = 2, fexp = 2, smin = 0, smax = 0.4, v(fixed = true), useHeatPort = true) annotation(
            Placement(transformation(extent = {{180, 20}, {200, 40}})));
          Thermal.HeatTransfer.Components.Convection convection annotation(
            Placement(transformation(extent = {{-10, -40}, {10, -60}})));
          Blocks.Sources.Constant const(k = 20) annotation(
            Placement(transformation(extent = {{-30, -90}, {-10, -70}})));
          Thermal.HeatTransfer.Celsius.FixedTemperature TAmbient(T = 25) "Ambient temperature" annotation(
            Placement(transformation(extent = {{40, -60}, {20, -40}})));
          Components.Fixed fixed2 annotation(
            Placement(transformation(extent = {{-120, -10}, {-100, 10}})));
          Components.SpringDamper springDamper1(c = 10000, d = 1000, useHeatPort = true, s_rel(fixed = true)) annotation(
            Placement(transformation(extent = {{150, 20}, {170, 40}})));
        equation
          connect(mass1.flange_b, springDamper.flange_a) annotation(
            Line(points = {{-40, 30}, {-30, 30}}, color = {0, 127, 0}, smooth = Smooth.None));
          connect(sine1.y, force.f) annotation(
            Line(points = {{-99, 30}, {-92, 30}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(force.flange, mass1.flange_a) annotation(
            Line(points = {{-70, 30}, {-60, 30}}, color = {0, 127, 0}, smooth = Smooth.None));
          connect(mass1.flange_a, damper.flange_a) annotation(
            Line(points = {{-60, 30}, {-60, 0}}, color = {0, 127, 0}, smooth = Smooth.None));
          connect(damper.flange_b, fixed1.flange) annotation(
            Line(points = {{-60, -20}, {-60, -30}}, color = {0, 127, 0}, smooth = Smooth.None));
          connect(springDamper.flange_b, mass2.flange_a) annotation(
            Line(points = {{-10, 30}, {0, 30}}, color = {0, 127, 0}, smooth = Smooth.None));
          connect(mass2.flange_b, supportFriction.flange_a) annotation(
            Line(points = {{20, 30}, {30, 30}}, color = {0, 127, 0}, smooth = Smooth.None));
          connect(supportFriction.flange_b, spring.flange_a) annotation(
            Line(points = {{50, 30}, {60, 30}}, color = {0, 127, 0}, smooth = Smooth.None));
          connect(spring.flange_b, mass3.flange_a) annotation(
            Line(points = {{80, 30}, {90, 30}}, color = {0, 127, 0}, smooth = Smooth.None));
          connect(mass3.flange_b, brake.flange_a) annotation(
            Line(points = {{110, 30}, {120, 30}}, color = {0, 127, 0}, smooth = Smooth.None));
          connect(sine2.y, brake.f_normalized) annotation(
            Line(points = {{121, 60}, {130, 60}, {130, 41}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(elastoGap.flange_b, mass1.flange_a) annotation(
            Line(points = {{-70, 0}, {-60, 0}, {-60, 30}}, color = {0, 127, 0}, smooth = Smooth.None));
          connect(const.y, convection.Gc) annotation(
            Line(points = {{-9, -80}, {0, -80}, {0, -60}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(TAmbient.port, convection.fluid) annotation(
            Line(points = {{20, -50}, {10, -50}}, color = {191, 0, 0}, smooth = Smooth.None));
          connect(elastoGap.flange_a, fixed2.flange) annotation(
            Line(points = {{-90, 0}, {-110, 0}}, color = {0, 127, 0}, smooth = Smooth.None));
          connect(elastoGap.heatPort, convection.solid) annotation(
            Line(points = {{-90, -10}, {-90, -50}, {-10, -50}}, color = {191, 0, 0}, smooth = Smooth.None));
          connect(damper.heatPort, convection.solid) annotation(
            Line(points = {{-50, 0}, {-50, -50}, {-10, -50}}, color = {191, 0, 0}, smooth = Smooth.None));
          connect(springDamper.heatPort, convection.solid) annotation(
            Line(points = {{-30, 20}, {-30, -50}, {-10, -50}}, color = {191, 0, 0}, smooth = Smooth.None));
          connect(supportFriction.heatPort, convection.solid) annotation(
            Line(points = {{30, 20}, {30, 0}, {-30, 0}, {-30, -50}, {-10, -50}}, color = {191, 0, 0}, smooth = Smooth.None));
          connect(brake.heatPort, convection.solid) annotation(
            Line(points = {{120, 20}, {120, 0}, {-30, 0}, {-30, -50}, {-10, -50}}, color = {191, 0, 0}, smooth = Smooth.None));
          connect(massWithStopAndFriction.heatPort, convection.solid) annotation(
            Line(points = {{180, 20}, {180, 0}, {-30, 0}, {-30, -50}, {-10, -50}}, color = {191, 0, 0}, smooth = Smooth.None));
          connect(brake.flange_b, springDamper1.flange_a) annotation(
            Line(points = {{140, 30}, {150, 30}}, color = {0, 127, 0}, smooth = Smooth.None));
          connect(springDamper1.flange_b, massWithStopAndFriction.flange_a) annotation(
            Line(points = {{170, 30}, {180, 30}}, color = {0, 127, 0}, smooth = Smooth.None));
          connect(springDamper1.heatPort, convection.solid) annotation(
            Line(points = {{150, 20}, {150, 0}, {-30, 0}, {-30, -50}, {-10, -50}}, color = {191, 0, 0}, smooth = Smooth.None));
          annotation(
            Documentation(info = "<html>
<p>
This model demonstrates how to model the dissipated power of a Translational model,
by enabling the heatPort of all components and connecting these heatPorts via
a convection element to the environment. The total heat flow generated by the
elements and transported to the environment
is present in variable convection.fluid.
</p>
</html>"),
            experiment(StopTime = 2.0, Interval = 0.001),
            Diagram(coordinateSystem(extent = {{-120, -100}, {200, 100}}, preserveAspectRatio = false)));
        end HeatLosses;

        package Utilities "Utility classes used by the Example models"
          extends Modelica.Icons.UtilitiesPackage;

          function GenerateStribeckFrictionTable "Generate Stribeck friction table for example Friction for the SupportFriction"
            extends Modelica.Icons.Function;
            input Real F_prop(final unit = "N.s/m", final min = 0) "Velocity dependent friction coefficient";
            input Modelica.SIunits.Force F_Coulomb "Constant friction: Coulomb force";
            input Modelica.SIunits.Force F_Stribeck "Stribeck effect";
            input Real fexp(final unit = "s/m", final min = 0) "Exponential decay";
            input Real v_max "Generate table from v=0 ... v_max";
            input Integer nTable(min = 2) = 100 "Number of table points";
            output Real table[nTable, 2] "Friction table";
          algorithm
            for i in 1:nTable loop
              table[i, 1] := v_max * (i - 1) / (nTable - 1);
              table[i, 2] := F_Coulomb + F_prop * table[i, 1] + F_Stribeck * exp(-fexp * table[i, 1]);
            end for;
            annotation(
              Documentation(info = "<html>
<p>
Returns a table with the friction characteristic table[nTable,2] = [0, f1; ....; v_max, fn], where the first
column is the velocity v in the range 0..v_max and the second column is the friction force
according to the Stribeck curve:
</p>
<pre>
  F_Coulomb + F_prop*v + F_Stribeck*exp(-fexp*v);
</pre>

</html>"));
          end GenerateStribeckFrictionTable;
          annotation(
            Documentation(info = "<html>
<p>Utility models and functions used in the Examples</p>
</html>"));
        end Utilities;
        annotation(
          Documentation(info = "<html>
<p>
This package contains example models to demonstrate the usage of the
Translational package. Open the models and
simulate them according to the provided description in the models.
</p>

</html>"));
      end Examples;

      package Components "Components for 1D translational mechanical drive trains"
        extends Modelica.Icons.Package;

        model Fixed "Fixed flange"
          parameter SI.Position s0 = 0 "Fixed offset position of housing";
          Interfaces.Flange_b flange annotation(
            Placement(transformation(origin = {0, 0}, extent = {{-10, 10}, {10, -10}}, rotation = 180)));
        equation
          flange.s = s0;
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-80, -40}, {80, -40}}, color = {0, 0, 0}), Line(points = {{80, -40}, {40, -80}}, color = {0, 0, 0}), Line(points = {{40, -40}, {0, -80}}, color = {0, 0, 0}), Line(points = {{0, -40}, {-40, -80}}, color = {0, 0, 0}), Line(points = {{-40, -40}, {-80, -80}}, color = {0, 0, 0}), Line(points = {{0, -40}, {0, -10}}, color = {0, 0, 0}), Text(extent = {{-150, -90}, {150, -130}}, textString = "%name", lineColor = {0, 0, 255})}),
            Documentation(info = "<html>
<p>
The <i>flange</i> of a 1D translational mechanical system <i>fixed</i>
at an position s0 in the <i>housing</i>. May be used:
</p>
<ul>
<li> to connect a compliant element, such as a spring or a damper,
     between a sliding mass and the housing.
<li> to fix a rigid element, such as a sliding mass, at a specific
     position.
</ul>

</html>"));
        end Fixed;

        model Mass "Sliding mass with inertia"
          parameter SI.Mass m(min = 0, start = 1) "Mass of the sliding mass";
          parameter StateSelect stateSelect = StateSelect.default "Priority to use s and v as states" annotation(
            Dialog(tab = "Advanced"));
          extends Translational.Interfaces.PartialRigid(L = 0, s(start = 0, stateSelect = stateSelect));
          SI.Velocity v(start = 0, stateSelect = stateSelect) "Absolute velocity of component";
          SI.Acceleration a(start = 0) "Absolute acceleration of component";
        equation
          v = der(s);
          a = der(v);
          m * a = flange_a.f + flange_b.f;
          annotation(
            Documentation(info = "<html>
<p>
Sliding mass with <i>inertia, without friction</i> and two rigidly connected flanges.
</p>
<p>
The sliding mass has the length L, the position coordinate s is in the middle.
Sign convention: A positive force at flange flange_a moves the sliding mass in the positive direction.
A negative force at flange flange_a moves the sliding mass to the negative direction.
</p>

</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-100, 0}, {-55, 0}}, color = {0, 127, 0}), Line(points = {{55, 0}, {100, 0}}, color = {0, 127, 0}), Rectangle(extent = {{-55, -30}, {56, 30}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.Sphere, fillColor = {255, 255, 255}), Polygon(points = {{50, -90}, {20, -80}, {20, -100}, {50, -90}}, lineColor = {128, 128, 128}, fillColor = {128, 128, 128}, fillPattern = FillPattern.Solid), Line(points = {{-60, -90}, {20, -90}}, color = {0, 0, 0}), Text(extent = {{-150, 85}, {150, 45}}, textString = "%name", lineColor = {0, 0, 255}), Text(extent = {{-150, -45}, {150, -75}}, lineColor = {0, 0, 0}, textString = "m=%m")}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-100, 0}, {-55, 0}}, color = {0, 127, 0}), Line(points = {{55, 0}, {100, 0}}, color = {0, 127, 0}), Rectangle(extent = {{-55, -30}, {55, 30}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.Sphere, fillColor = {255, 255, 255}), Polygon(points = {{50, -90}, {20, -80}, {20, -100}, {50, -90}}, lineColor = {128, 128, 128}, fillColor = {128, 128, 128}, fillPattern = FillPattern.Solid), Line(points = {{-60, -90}, {20, -90}}, color = {0, 0, 0}), Line(points = {{-100, -29}, {-100, -61}}, color = {0, 0, 0}), Line(points = {{100, -61}, {100, -28}}, color = {0, 0, 0}), Line(points = {{-98, -60}, {98, -60}}, color = {0, 0, 0}), Polygon(points = {{-101, -60}, {-96, -59}, {-96, -61}, {-101, -60}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Polygon(points = {{100, -60}, {95, -61}, {95, -59}, {100, -60}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Text(extent = {{-44, -41}, {51, -57}}, textString = "Length L", lineColor = {0, 0, 255}), Line(points = {{0, 30}, {0, 53}}, color = {0, 0, 0}), Line(points = {{-72, 40}, {1, 40}}, color = {0, 0, 0}), Polygon(points = {{-7, 42}, {-7, 38}, {-1, 40}, {-7, 42}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Text(extent = {{-61, 53}, {-9, 42}}, textString = "Position s", lineColor = {0, 0, 255})}));
        end Mass;

        model Rod "Rod without inertia"
          extends Translational.Interfaces.PartialRigid;
        equation
          0 = flange_a.f + flange_b.f;
          annotation(
            Documentation(info = "<html>
<p>
Rod <i>without inertia</i> and two rigidly connected flanges.
</p>

</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-100, 0}, {-55, 0}}, color = {0, 127, 0}), Line(points = {{53, 0}, {99, 0}}, color = {0, 127, 0}), Polygon(points = {{50, -90}, {20, -80}, {20, -100}, {50, -90}}, lineColor = {128, 128, 128}, fillColor = {128, 128, 128}, fillPattern = FillPattern.Solid), Line(points = {{-60, -90}, {20, -90}}, color = {0, 0, 0}), Rectangle(extent = {{-55, 10}, {53, -10}}, lineColor = {160, 160, 164}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Text(extent = {{-150, 80}, {150, 40}}, textString = "%name", lineColor = {0, 0, 255}), Text(extent = {{-150, -30}, {150, -60}}, lineColor = {0, 0, 0}, textString = "L=%L")}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-100, 0}, {-55, 0}}, color = {0, 127, 0}), Line(points = {{54, 0}, {100, 0}}, color = {0, 127, 0}), Polygon(points = {{50, -90}, {20, -80}, {20, -100}, {50, -90}}, lineColor = {128, 128, 128}, fillColor = {128, 128, 128}, fillPattern = FillPattern.Solid), Line(points = {{-60, -90}, {20, -90}}, color = {0, 0, 0}), Rectangle(extent = {{-55, 3}, {53, -4}}, lineColor = {160, 160, 164}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-100, -29}, {-100, -61}}, color = {0, 0, 0}), Line(points = {{100, -61}, {100, -28}}, color = {0, 0, 0}), Line(points = {{-98, -60}, {98, -60}}, color = {0, 0, 0}), Polygon(points = {{-101, -60}, {-96, -59}, {-96, -61}, {-101, -60}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Polygon(points = {{100, -60}, {95, -61}, {95, -59}, {100, -60}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Text(extent = {{-44, -41}, {51, -57}}, textString = "Length L", lineColor = {0, 0, 255})}));
        end Rod;

        model Spring "Linear 1D translational spring"
          extends Translational.Interfaces.PartialCompliant;
          parameter SI.TranslationalSpringConstant c(final min = 0, start = 1) "Spring constant";
          parameter SI.Distance s_rel0 = 0 "Unstretched spring length";
        equation
          f = c * (s_rel - s_rel0);
          annotation(
            Documentation(info = "<html>
<p>
A <i>linear 1D translational spring</i>. The component can be connected either
between two sliding masses, or between
a sliding mass and the housing (model Fixed), to describe
a coupling of the sliding mass with the housing via a spring.
</p>

</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-60, -90}, {20, -90}}, color = {0, 0, 0}), Polygon(points = {{50, -90}, {20, -80}, {20, -100}, {50, -90}}, lineColor = {128, 128, 128}, fillColor = {128, 128, 128}, fillPattern = FillPattern.Solid), Text(extent = {{-150, 90}, {150, 50}}, textString = "%name", lineColor = {0, 0, 255}), Line(points = {{-98, 0}, {-60, 0}, {-44, -30}, {-16, 30}, {14, -30}, {44, 30}, {60, 0}, {100, 0}}, color = {0, 0, 0}), Text(extent = {{-150, -45}, {150, -75}}, lineColor = {0, 0, 0}, textString = "c=%c")}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-100, 0}, {-100, 65}}, color = {128, 128, 128}), Line(points = {{100, 0}, {100, 65}}, color = {128, 128, 128}), Line(points = {{-100, 60}, {100, 60}}, color = {128, 128, 128}), Polygon(points = {{90, 63}, {100, 60}, {90, 57}, {90, 63}}, lineColor = {128, 128, 128}, fillColor = {128, 128, 128}, fillPattern = FillPattern.Solid), Text(extent = {{-56, 66}, {36, 81}}, lineColor = {0, 0, 255}, textString = "s_rel"), Line(points = {{-86, 0}, {-60, 0}, {-44, -30}, {-16, 30}, {14, -30}, {44, 30}, {60, 0}, {84, 0}}, color = {0, 0, 0})}));
        end Spring;

        model Damper "Linear 1D translational damper"
          extends Translational.Interfaces.PartialCompliantWithRelativeStates;
          parameter SI.TranslationalDampingConstant d(final min = 0, start = 0) "Damping constant";
          extends Modelica.Thermal.HeatTransfer.Interfaces.PartialElementaryConditionalHeatPortWithoutT;
        equation
          f = d * v_rel;
          lossPower = f * v_rel;
          annotation(
            Documentation(info = "<html>
<p>
<i>Linear, velocity dependent damper</i> element. It can be either connected
between a sliding mass and the housing (model Fixed), or
between two sliding masses.
</p>

</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-90, 0}, {-60, 0}}, color = {0, 0, 0}), Line(points = {{-60, -30}, {-60, 30}}, color = {0, 0, 0}), Line(points = {{-60, -30}, {60, -30}}, color = {0, 0, 0}), Line(points = {{-60, 30}, {60, 30}}, color = {0, 0, 0}), Rectangle(extent = {{-60, 30}, {30, -30}}, lineColor = {0, 0, 0}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{30, 0}, {90, 0}}, color = {0, 0, 0}), Polygon(points = {{50, -90}, {20, -80}, {20, -100}, {50, -90}}, lineColor = {128, 128, 128}, fillColor = {128, 128, 128}, fillPattern = FillPattern.Solid), Line(points = {{-60, -90}, {20, -90}}, color = {0, 0, 0}), Text(extent = {{-150, 90}, {150, 50}}, textString = "%name", lineColor = {0, 0, 255}), Text(extent = {{-150, -45}, {150, -75}}, lineColor = {0, 0, 0}, textString = "d=%d"), Line(visible = useHeatPort, points = {{-100, -100}, {-100, -20}, {-14, -20}}, color = {191, 0, 0}, pattern = LinePattern.Dot, smooth = Smooth.None)}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-90, 0}, {-60, 0}}, color = {0, 0, 0}), Line(points = {{-60, -30}, {-60, 30}}, color = {0, 0, 0}), Line(points = {{-60, -30}, {60, -30}}, color = {0, 0, 0}), Line(points = {{-60, 30}, {60, 30}}, color = {0, 0, 0}), Rectangle(extent = {{-60, 30}, {30, -30}}, lineColor = {0, 0, 0}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{30, 0}, {90, 0}}, color = {0, 0, 0}), Line(points = {{-50, 60}, {50, 60}}, color = {128, 128, 128}), Polygon(points = {{50, 63}, {60, 60}, {50, 57}, {50, 63}}, lineColor = {128, 128, 128}, fillColor = {128, 128, 128}, fillPattern = FillPattern.Solid), Text(extent = {{-58, 68}, {42, 78}}, lineColor = {128, 128, 128}, textString = "der(s_rel)")}));
        end Damper;

        model SpringDamper "Linear 1D translational spring and damper in parallel"
          extends Translational.Interfaces.PartialCompliantWithRelativeStates;
          parameter SI.TranslationalSpringConstant c(final min = 0, start = 1) "Spring constant";
          parameter SI.TranslationalDampingConstant d(final min = 0, start = 1) "Damping constant";
          parameter SI.Position s_rel0 = 0 "Unstretched spring length";
          extends Modelica.Thermal.HeatTransfer.Interfaces.PartialElementaryConditionalHeatPortWithoutT;
        protected
          Modelica.SIunits.Force f_c "Spring force";
          Modelica.SIunits.Force f_d "Damping force";
        equation
          f_c = c * (s_rel - s_rel0);
          f_d = d * v_rel;
          f = f_c + f_d;
          lossPower = f_d * v_rel;
          annotation(
            Documentation(info = "<html>
<p>
A <i>spring and damper element connected in parallel</i>.
The component can be
connected either between two sliding masses to describe the elasticity
and damping, or between a sliding mass and the housing (model Fixed),
to describe a coupling of the sliding mass with the housing via a spring/damper.
</p>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-80, 40}, {-60, 40}, {-45, 10}, {-15, 70}, {15, 10}, {45, 70}, {60, 40}, {80, 40}}, color = {0, 0, 0}), Line(points = {{-80, 40}, {-80, -70}}, color = {0, 0, 0}), Line(points = {{-80, -70}, {-52, -70}}, color = {0, 0, 0}), Rectangle(extent = {{-52, -49}, {38, -91}}, lineColor = {0, 0, 0}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-52, -49}, {68, -49}}, color = {0, 0, 0}), Line(points = {{-51, -91}, {69, -91}}, color = {0, 0, 0}), Line(points = {{38, -70}, {80, -70}}, color = {0, 0, 0}), Line(points = {{80, 40}, {80, -70}}, color = {0, 0, 0}), Line(points = {{-90, 0}, {-80, 0}}, color = {0, 0, 0}), Line(points = {{80, 0}, {90, 0}}, color = {0, 0, 0}), Polygon(points = {{53, -18}, {23, -8}, {23, -28}, {53, -18}}, lineColor = {128, 128, 128}, fillColor = {128, 128, 128}, fillPattern = FillPattern.Solid), Line(points = {{-57, -18}, {23, -18}}, color = {0, 0, 0}), Text(extent = {{-150, 120}, {150, 80}}, textString = "%name", lineColor = {0, 0, 255}), Text(extent = {{-150, -135}, {150, -165}}, lineColor = {0, 0, 0}, textString = "d=%d"), Text(extent = {{-150, -100}, {150, -130}}, lineColor = {0, 0, 0}, textString = "c=%c"), Line(visible = useHeatPort, points = {{-100, -100}, {-100, -80}, {-5, -80}}, color = {191, 0, 0}, pattern = LinePattern.Dot, smooth = Smooth.None)}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-80, 32}, {-58, 32}, {-43, 2}, {-13, 62}, {17, 2}, {47, 62}, {62, 32}, {80, 32}}, color = {0, 0, 0}, thickness = 0.5), Line(points = {{-100, 31}, {-100, 96}}, color = {128, 128, 128}), Line(points = {{100, 29}, {100, 94}}, color = {128, 128, 128}), Line(points = {{-98, 82}, {100, 82}}, color = {128, 128, 128}), Polygon(points = {{90, 85}, {100, 82}, {90, 79}, {90, 85}}, lineColor = {128, 128, 128}, fillColor = {128, 128, 128}, fillPattern = FillPattern.Solid), Text(extent = {{-63, 83}, {46, 103}}, lineColor = {0, 0, 255}, textString = "s_rel"), Rectangle(extent = {{-52, -28}, {38, -72}}, lineColor = {0, 0, 0}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-51, -72}, {69, -72}}, color = {0, 0, 0}), Line(points = {{-52, -28}, {68, -28}}, color = {0, 0, 0}), Line(points = {{38, -50}, {80, -50}}, color = {0, 0, 0}), Line(points = {{-80, -50}, {-52, -50}}, color = {0, 0, 0}), Line(points = {{-80, 32}, {-80, -50}}, color = {0, 0, 0}), Line(points = {{80, 32}, {80, -50}}, color = {0, 0, 0}), Line(points = {{-90, 0}, {-80, 0}}, color = {0, 0, 0}), Line(points = {{90, 0}, {80, 0}}, color = {0, 0, 0})}));
        end SpringDamper;

        model ElastoGap "1D translational spring damper combination with gap"
          extends Modelica.Mechanics.Translational.Interfaces.PartialCompliantWithRelativeStates;
          parameter SI.TranslationalSpringConstant c(final min = 0, start = 1) "Spring constant";
          parameter SI.TranslationalDampingConstant d(final min = 0, start = 1) "Damping constant";
          parameter SI.Position s_rel0 = 0 "Unstretched spring length";
          parameter Real n(final min = 1) = 1 "Exponent of spring force ( f_c = -c*|s_rel-s_rel0|^n )";
          extends Modelica.Thermal.HeatTransfer.Interfaces.PartialElementaryConditionalHeatPortWithoutT;
          /*
          Please note that initialization might fail due to the nonlinear spring characteristic
          (spring force is zero for s_rel > s_rel0)
          if a positive force is acting on the element and no other force balances this force
          (e.g., when setting both initial velocity and acceleration to 0)
          */
          Boolean contact "=true, if contact, otherwise no contact";
        protected
          Modelica.SIunits.Force f_c "Spring force";
          Modelica.SIunits.Force f_d2 "Linear damping force";
          Modelica.SIunits.Force f_d "Linear damping force which is limited by spring force (|f_d| <= |f_c|)";
        equation
// Modify contact force, so that it is only "pushing" and not
// "pulling/sticking" and that it is continuous
          contact = s_rel < s_rel0;
          f_c = smooth(1, noEvent(if contact then -c * abs(s_rel - s_rel0) ^ n else 0));
          f_d2 = if contact then d * v_rel else 0;
          f_d = smooth(0, noEvent(if contact then if f_d2 < f_c then f_c else if f_d2 > (-f_c) then -f_c else f_d2 else 0));
          f = f_c + f_d;
          lossPower = f_d * v_rel;
          annotation(
            Documentation(info = "<html>
<p>
This component models a spring damper combination that can lift off.
It can be connected between a sliding mass and the housing (model
<a href=\"modelica://Modelica.Mechanics.Translational.Components.Fixed\">Fixed</a>),
to describe the contact of a sliding mass with the housing.
</p>

<p>
As long as s_rel &gt; s_rel0, no force is exerted (s_rel = flange_b.s - flange_a.s).
If s_rel &le; s_rel0, the contact force is basically computed with a linear
spring/damper characteristic. With parameter n&ge;1 (exponent of spring force),
a nonlinear spring force can be modeled:
</p>

<pre>
   desiredContactForce = c*|s_rel - s_rel0|^n + d*<b>der</b>(s_rel)
</pre>

<p>
Note, Hertzian contact is described by:
</p>
<ul>
<li> Contact between two metallic spheres: n=1.5</li>
<li> Contact between two metallic plates: n=1</li>
</ul>

<p>
The above force law leads to the following difficulties:
</p>

<ol>
<li> If the damper force becomes larger as the spring force and with opposite sign,
     the contact force would be \"pulling/sticking\" which is unphysical, since during
     contact only pushing forces can occur.</li>

<li> When contact occurs with a non-zero relative speed (which is the usual
     situation), the damping force has a non-zero value and therefore the contact
     force changes discontinuously at s_rel = s_rel0. Again, this is not physical
     because the force can only change continuously. (Note, this component is not an
     idealized model where a steep characteristic is approximated by a discontinuity,
     but it shall model the steep characteristic.)</li>
</ol>

<p>
In the literature there are several proposals to fix problem (2). Especially, often
the following model is used (see, e.g.,
Lankarani, Nikravesh: Continuous Contact Force Models for Impact
Analysis in Multibody Systems, Nonlinear Dynamics 5, pp. 193-207, 1994,
<a href=\"http://www.springerlink.com/content/h50x61270q06p65n/fulltext.pdf\">pdf-download</a>):
</p>

<pre>
   f = c*s_rel^n + (d*s_rel^n)*<b>der</b>(s_rel)
</pre>

<p>
However, this and other models proposed in literature violate
issue (1), i.e., unphysical pulling forces can occur (if d*<b>der</b>(s_rel)
becomes large enough). Note, if the force law is of the form \"f = f_c + f_d\", then a
necessary condition is that |f_d| &le; |f_c|, otherwise (1) and (2) are violated.
For this reason, the most simplest approach is used in the ElastoGap model
to fix both problems by using this necessary condition in the force law directly.
If s_rel0 = 0, the equations are:
</p>

<pre>
    <b>if</b> s_rel &ge; 0 <b>then</b>
       f = 0;    // contact force
    <b>else</b>
       f_c  = -c*|s_rel|^n;          // contact spring force (Hertzian contact force)
       f_d2 = d*<b>der</b>(s_rel);         // linear contact damper force
       f_d  = <b>if</b> f_d2 &lt;  f_c <b>then</b>  f_c <b>else</b>
              <b>if</b> f_d2 &gt; -f_c <b>then</b> -f_c <b>else</b> f_d2;  // bounded damper force
       f    = f_c + f_d;            // contact force
    <b>end if</b>;
</pre>

<p>
Note, since |f_d| &le; |f_c|, pulling forces cannot occur and the contact force
is always continuous, especially around the start of the penetration at s_rel = s_rel0.
</p>

<p>
In the next figure, a typical simulation with the ElastoGap model is shown
(<a href=\"modelica://Modelica.Mechanics.Translational.Examples.ElastoGap\">Examples.ElastoGap</a>)
where the different effects are visualized:
</p>

<ol>
<li> Curve 1 (elastoGap1.f) is the unmodified contact force, i.e., the linear spring/damper
     characteristic. A pulling/sticking force is present at the end of the contact.</li>
<li> Curve 2 (elastoGap2.f) is the contact force, where the force is explicitly set to
     zero when pulling/sticking occurs. The contact force is discontinuous when contact starts.</li>
<li> Curve 3 (elastoGap3.f) is the ElastoGap model of this library. No discontinuity and no
     pulling/sticking occurs.</li>
</ol>

<p>
<img src=\"modelica://Modelica/Resources/Images/Mechanics/Translational/ElastoGap1.png\">
</p>
</html>"),
            Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-100, 0}, {-50, 0}}, color = {0, 127, 0}), Line(points = {{-48, 34}, {-48, -46}}, color = {0, 0, 0}, thickness = 1), Line(points = {{8, 40}, {8, 2}}, color = {0, 0, 0}), Line(points = {{-2, 0}, {38, 0}, {38, 44}, {-2, 44}}, color = {0, 0, 0}), Line(points = {{38, 22}, {72, 22}}, color = {0, 0, 0}), Line(points = {{-12, -38}, {-12, 20}}, color = {0, 0, 0}, thickness = 1), Line(points = {{-12, 22}, {8, 22}}, color = {0, 0, 0}), Line(points = {{-12, -38}, {-2, -38}}, color = {0, 0, 0}), Line(points = {{72, 0}, {100, 0}}, color = {0, 127, 0}), Line(points = {{72, 22}, {72, -42}}, color = {0, 0, 0}), Line(points = {{-2, -38}, {10, -28}, {22, -48}, {38, -28}, {50, -48}, {64, -28}, {72, -40}}, color = {0, 0, 0}), Rectangle(extent = {{8, 44}, {38, 0}}, lineColor = {0, 0, 0}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Text(extent = {{-64, -80}, {64, -64}}, lineColor = {0, 0, 255}, textString = "s_rel"), Line(points = {{-100, -29}, {-100, -61}}, color = {0, 0, 0}), Line(points = {{100, -61}, {100, -28}}, color = {0, 0, 0}), Line(points = {{-98, -60}, {98, -60}}, color = {0, 0, 0}), Polygon(points = {{-101, -60}, {-96, -59}, {-96, -61}, {-101, -60}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Polygon(points = {{100, -60}, {95, -61}, {95, -59}, {100, -60}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid)}),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-98, 0}, {-48, 0}}, color = {0, 127, 0}), Line(points = {{-48, 38}, {-48, -38}}, color = {0, 0, 0}, thickness = 1), Line(points = {{8, -10}, {8, -48}}, color = {0, 0, 0}), Line(points = {{-2, -50}, {38, -50}, {38, -6}, {-2, -6}}, color = {0, 0, 0}), Line(points = {{38, -28}, {72, -28}}, color = {0, 0, 0}), Line(points = {{-12, -38}, {-12, 36}}, color = {0, 0, 0}, thickness = 1), Line(points = {{-12, -28}, {8, -28}}, color = {0, 0, 0}), Line(points = {{-12, 24}, {-6, 24}}, color = {0, 0, 0}), Line(points = {{72, 0}, {98, 0}}, color = {0, 127, 0}), Line(points = {{72, 22}, {72, -42}}, color = {0, 0, 0}), Line(points = {{-6, 24}, {6, 34}, {18, 14}, {34, 34}, {46, 14}, {60, 34}, {68, 22}}, color = {0, 0, 0}), Rectangle(extent = {{8, -6}, {38, -50}}, lineColor = {0, 0, 0}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-52, -70}, {28, -70}}, color = {0, 0, 0}), Polygon(points = {{58, -70}, {28, -60}, {28, -80}, {58, -70}}, lineColor = {128, 128, 128}, fillColor = {128, 128, 128}, fillPattern = FillPattern.Solid), Text(extent = {{-150, 100}, {150, 60}}, textString = "%name", lineColor = {0, 0, 255}, pattern = LinePattern.Dot), Text(extent = {{-150, -125}, {150, -95}}, lineColor = {0, 0, 0}, textString = "c=%c"), Text(extent = {{-150, -160}, {150, -130}}, lineColor = {0, 0, 0}, textString = "d=%d"), Line(points = {{68, 22}, {72, 22}}, color = {0, 0, 0}), Line(visible = useHeatPort, points = {{-100, -100}, {-100, -44}, {22, -44}, {22, -28}}, color = {191, 0, 0}, pattern = LinePattern.Dot, smooth = Smooth.None)}));
        end ElastoGap;

        model SupportFriction "Coulomb friction in support"
          extends Modelica.Mechanics.Translational.Interfaces.PartialElementaryTwoFlangesAndSupport2;
          extends Modelica.Thermal.HeatTransfer.Interfaces.PartialElementaryConditionalHeatPortWithoutT;
          parameter Real f_pos[:, 2] = [0, 1] "[v, f] Positive sliding friction characteristic (v>=0)";
          parameter Real peak(final min = 1) = 1 "peak*f_pos[1,2] = Maximum friction force for v==0";
          extends Translational.Interfaces.PartialFriction;
          SI.Position s;
          SI.Force f "Friction force";
          SI.Velocity v "Absolute velocity of flange_a and flange_b";
          SI.Acceleration a "Absolute acceleration of flange_a and flange_b";
        equation
// Constant auxiliary variables
          f0 = Modelica.Math.tempInterpol1(0, f_pos, 2);
          f0_max = peak * f0;
          free = false;
          s = flange_a.s - s_support;
          flange_a.s = flange_b.s;
// velocity and acceleration of flanges
          v = der(s);
          a = der(v);
          v_relfric = v;
          a_relfric = a;
// Friction force
          flange_a.f + flange_b.f - f = 0;
// Friction force
          f = if locked then sa * unitForce else if startForward then Modelica.Math.tempInterpol1(v, f_pos, 2) else if startBackward then -Modelica.Math.tempInterpol1(-v, f_pos, 2) else if pre(mode) == Forward then Modelica.Math.tempInterpol1(v, f_pos, 2) else -Modelica.Math.tempInterpol1(-v, f_pos, 2);
          lossPower = f * v_relfric;
          annotation(
            Documentation(info = "<html>
<p>
This element describes <b>Coulomb friction</b> in <b>support</b>,
i.e., a frictional force acting between a flange and the housing.
The positive sliding friction force \"f\" has to be defined
by table \"f_pos\" as function of the absolute velocity \"v\".
E.g.
</p>
<pre>
       v |   f
      ---+-----
       0 |   0
       1 |   2
       2 |   5
       3 |   8
</pre>
<p>
gives the following table:
</p>
<pre>
   f_pos = [0, 0; 1, 2; 2, 5; 3, 8];
</pre>
<p>
Currently, only linear interpolation in the table is supported.
Outside of the table, extrapolation through the last
two table entries is used. It is assumed that the negative
sliding friction force has the same characteristic with negative
values. Friction is modelled in the following way:
</p>
<p>
When the absolute velocity \"v\" is not zero, the friction force
is a function of v and of a constant normal force. This dependency
is defined via table f_pos and can be determined by measurements,
e.g., by driving the gear with constant velocity and measuring the
needed driving force (= friction force).
</p>
<p>
When the absolute velocity becomes zero, the elements
connected by the friction element become stuck, i.e., the absolute
position remains constant. In this phase the friction force is
calculated from a force balance due to the requirement, that
the absolute acceleration shall be zero.  The elements begin
to slide when the friction force exceeds a threshold value,
called the maximum static friction force, computed via:
</p>
<pre>
   maximum_static_friction = <b>peak</b> * sliding_friction(v=0)  (<b>peak</b> >= 1)
</pre>
<p>
This procedure is implemented in a \"clean\" way by state events and
leads to continuous/discrete systems of equations if friction elements
are dynamically coupled which have to be solved by appropriate
numerical methods. The method is described in:
</p>
<dl>
<dt>Otter M., Elmqvist H., and Mattsson S.E. (1999):
<dd><b>Hybrid Modeling in Modelica based on the Synchronous
    Data Flow Principle</b>. CACSD'99, Aug. 22.-26, Hawaii.
</dl>
<p>
More precise friction models take into account the elasticity of the
material when the two elements are \"stuck\", as well as other effects,
like hysteresis. This has the advantage that the friction element can
be completely described by a differential equation without events. The
drawback is that the system becomes stiff (about 10-20 times slower
simulation) and that more material constants have to be supplied which
requires more sophisticated identification. For more details, see the
following references, especially (Armstrong and Canudas de Witt 1996):
</p>
<dl>
<dt>Armstrong B. (1991):
<dd><b>Control of Machines with Friction</b>. Kluwer Academic
    Press, Boston MA.<br><br>
<dt>Armstrong B., and Canudas de Wit C. (1996):
<dd><b>Friction Modeling and Compensation.</b>
    The Control Handbook, edited by W.S.Levine, CRC Press,
    pp. 1369-1382.<br><br>
<dt>Canudas de Wit C., Olsson H., Astroem K.J., and Lischinsky P. (1995):
<dd><b>A new model for control of systems with friction.</b>
    IEEE Transactions on Automatic Control, Vol. 40, No. 3, pp. 419-425.<br><br>
</dl>

</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-90, 10}, {90, -10}}, lineColor = {0, 127, 0}, fillColor = {215, 215, 215}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-48, -10}, {-28, -30}}, lineColor = {95, 95, 95}, fillPattern = FillPattern.Sphere, fillColor = {175, 175, 175}), Ellipse(extent = {{-10, -10}, {10, -30}}, lineColor = {95, 95, 95}, fillPattern = FillPattern.Sphere, fillColor = {175, 175, 175}), Ellipse(extent = {{30, -10}, {50, -30}}, lineColor = {95, 95, 95}, fillPattern = FillPattern.Sphere, fillColor = {175, 175, 175}), Polygon(points = {{-60, -30}, {60, -30}, {60, -12}, {80, -12}, {80, -50}, {-80, -50}, {-80, -12}, {-60, -12}, {-60, -30}}, lineColor = {95, 95, 95}, fillPattern = FillPattern.Solid, smooth = Smooth.None, fillColor = {175, 175, 175}), Text(extent = {{-150, 100}, {150, 60}}, textString = "%name", lineColor = {0, 0, 255}), Line(points = {{-10, -85}, {-5, -80}}, color = {0, 0, 0}), Line(points = {{-5, -90}, {5, -80}}, color = {0, 0, 0}), Line(points = {{-10, -90}, {0, -80}}, color = {0, 0, 0}), Line(points = {{0, -90}, {10, -80}}, color = {0, 0, 0}), Polygon(points = {{-60, -50}, {-20, -80}, {20, -80}, {60, -50}, {-60, -50}}, lineColor = {95, 95, 95}, smooth = Smooth.None, fillColor = {175, 175, 175}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-50, 30}, {-30, 10}}, lineColor = {95, 95, 95}, fillPattern = FillPattern.Sphere, fillColor = {175, 175, 175}), Ellipse(extent = {{-10, 30}, {10, 10}}, lineColor = {95, 95, 95}, fillPattern = FillPattern.Sphere, fillColor = {175, 175, 175}), Ellipse(extent = {{30, 30}, {50, 10}}, lineColor = {95, 95, 95}, fillPattern = FillPattern.Sphere, fillColor = {175, 175, 175}), Polygon(points = {{-60, 30}, {60, 30}, {60, 12}, {80, 12}, {80, 50}, {-80, 50}, {-80, 12}, {-60, 12}, {-60, 30}}, lineColor = {95, 95, 95}, fillPattern = FillPattern.Solid, smooth = Smooth.None, fillColor = {175, 175, 175}), Line(visible = useHeatPort, points = {{-100, -100}, {-100, -20}, {0, -20}}, color = {191, 0, 0}, pattern = LinePattern.Dot, smooth = Smooth.None)}));
        end SupportFriction;

        model Brake "Brake based on Coulomb friction"
          extends Modelica.Mechanics.Translational.Interfaces.PartialElementaryTwoFlangesAndSupport2;
          extends Modelica.Thermal.HeatTransfer.Interfaces.PartialElementaryConditionalHeatPortWithoutT;
          parameter Real mue_pos[:, 2] = [0, 0.5] "[v, f] Positive sliding friction characteristic (v>=0)";
          parameter Real peak(final min = 1) = 1 "peak*mue_pos[1,2] = Maximum friction force for v==0";
          parameter Real cgeo(final min = 0) = 1 "Geometry constant containing friction distribution assumption";
          parameter SI.Force fn_max(final min = 0, start = 1) "Maximum normal force";
          extends Translational.Interfaces.PartialFriction;
          SI.Position s;
          SI.Force f "Brake friction force";
          SI.Velocity v "Absolute velocity of flange_a and flange_b";
          SI.Acceleration a "Absolute acceleration of flange_a and flange_b";
          Real mue0 "Friction coefficient for v=0 and forward sliding";
          SI.Force fn "Normal force (=fn_max*f_normalized)";
          // Constant auxiliary variable
          Modelica.Blocks.Interfaces.RealInput f_normalized "Normalized force signal 0..1 (normal force = fn_max*f_normalized; brake is active if > 0)" annotation(
            Placement(transformation(origin = {0, 110}, extent = {{20, -20}, {-20, 20}}, rotation = 90)));
        equation
          mue0 = Modelica.Math.tempInterpol1(0, mue_pos, 2);
          s = s_a;
          s = s_b;
// velocity and acceleration of flanges flange_a and flange_b
          v = der(s);
          a = der(v);
          v_relfric = v;
          a_relfric = a;
// Friction force, normal force and friction force for v_rel=0
          flange_a.f + flange_b.f - f = 0;
          fn = fn_max * f_normalized;
          f0 = mue0 * cgeo * fn;
          f0_max = peak * f0;
          free = fn <= 0;
// Friction force
          f = if locked then sa * unitForce else if free then 0 else cgeo * fn * (if startForward then Modelica.Math.tempInterpol1(v, mue_pos, 2) else if startBackward then -Modelica.Math.tempInterpol1(-v, mue_pos, 2) else if pre(mode) == Forward then Modelica.Math.tempInterpol1(v, mue_pos, 2) else -Modelica.Math.tempInterpol1(-v, mue_pos, 2));
          lossPower = f * v_relfric;
          annotation(
            Documentation(info = "<html>
<p>
This component models a <b>brake</b>, i.e., a component where a frictional
force is acting between the housing and a flange and a controlled normal
force presses the flange to the housing in order to increase friction.
The normal force fn has to be provided as input signal f_normalized in a normalized form
(0 &le; f_normalized &le; 1),
fn = fn_max*f_normalized, where fn_max has to be provided as parameter.
Friction in the brake is modelled in the following way:
</p>
<p>
When the absolute velocity \"v\" is not zero, the friction force
is a function of the velocity dependent friction coefficient  mue(v) , of
the normal force \"fn\", and of a geometry constant \"cgeo\" which takes into
account the geometry of the device and the assumptions on the friction
distributions:
</p>
<pre>
        frictional_force = <b>cgeo</b> * <b>mue</b>(v) * <b>fn</b>
</pre>
<p>
   Typical values of coefficients of friction:
</p>
<pre>
      dry operation   :  <b>mue</b> = 0.2 .. 0.4
      operating in oil:  <b>mue</b> = 0.05 .. 0.1
</pre>
<p>
    The positive part of the friction characteristic <b>mue</b>(v),
    v >= 0, is defined via table mue_pos (first column = v,
    second column = mue). Currently, only linear interpolation in
    the table is supported.
</p>
<p>
   When the absolute velocity becomes zero, the elements
   connected by the friction element become stuck, i.e., the absolute
   position remains constant. In this phase the friction force is
   calculated from a force balance due to the requirement, that
   the absolute acceleration shall be zero.  The elements begin
   to slide when the friction force exceeds a threshold value,
   called the  maximum static friction force, computed via:
</p>
<pre>
       frictional_force = <b>peak</b> * <b>cgeo</b> * <b>mue</b>(w=0) * <b>fn</b>   (<b>peak</b> >= 1)
</pre>
<p>
This procedure is implemented in a \"clean\" way by state events and
leads to continuous/discrete systems of equations if friction elements
are dynamically coupled. The method is described in:
</p>
<dl>
<dt>Otter M., Elmqvist H., and Mattsson S.E. (1999):
<dd><b>Hybrid Modeling in Modelica based on the Synchronous
    Data Flow Principle</b>. CACSD'99, Aug. 22.-26, Hawaii.
</dl>
<p>
More precise friction models take into account the elasticity of the
material when the two elements are \"stuck\", as well as other effects,
like hysteresis. This has the advantage that the friction element can
be completely described by a differential equation without events. The
drawback is that the system becomes stiff (about 10-20 times slower
simulation) and that more material constants have to be supplied which
requires more sophisticated identification. For more details, see the
following references, especially (Armstrong and Canudas de Witt 1996):
</p>
<dl>
<dt>Armstrong B. (1991):
<dd><b>Control of Machines with Friction</b>. Kluwer Academic
    Press, Boston MA.<br><br>
<dt>Armstrong B., and Canudas de Wit C. (1996):
<dd><b>Friction Modeling and Compensation.</b>
    The Control Handbook, edited by W.S.Levine, CRC Press,
    pp. 1369-1382.<br><br>
<dt>Canudas de Wit C., Olsson H., Astroem K.J., and Lischinsky P. (1995):
<dd><b>A new model for control of systems with friction.</b>
    IEEE Transactions on Automatic Control, Vol. 40, No. 3, pp. 419-425.<br><br>
</dl>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-90, 10}, {90, -10}}, lineColor = {0, 127, 0}, fillColor = {215, 215, 215}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-20, 30}, {20, 20}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-20, -20}, {20, -30}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Polygon(points = {{0, -30}, {10, -50}, {-10, -50}, {0, -30}}, lineColor = {0, 0, 127}, fillColor = {0, 0, 127}, fillPattern = FillPattern.Solid), Polygon(points = {{10, 50}, {-10, 50}, {0, 30}, {10, 50}}, lineColor = {0, 0, 127}, fillColor = {0, 0, 127}, fillPattern = FillPattern.Solid), Line(points = {{0, 90}, {0, 50}}, color = {0, 0, 0}, smooth = Smooth.None), Rectangle(extent = {{20, 28}, {30, 22}}, lineColor = {175, 175, 175}, fillColor = {175, 175, 175}, fillPattern = FillPattern.Solid), Rectangle(extent = {{20, -22}, {30, -28}}, lineColor = {175, 175, 175}, fillColor = {175, 175, 175}, fillPattern = FillPattern.Solid), Rectangle(extent = {{30, 28}, {36, -102}}, lineColor = {175, 175, 175}, fillColor = {175, 175, 175}, fillPattern = FillPattern.Solid), Rectangle(extent = {{14, -96}, {30, -102}}, lineColor = {175, 175, 175}, fillColor = {175, 175, 175}, fillPattern = FillPattern.Solid), Line(points = {{0, -50}, {0, -60}, {-40, -50}, {-40, 48}, {0, 60}, {0, 90}}, color = {0, 0, 0}, smooth = Smooth.None), Text(extent = {{-150, -120}, {150, -160}}, textString = "%name", lineColor = {0, 0, 255}), Line(visible = useHeatPort, points = {{-100, -102}, {-100, -16}, {0, -16}}, color = {191, 0, 0}, pattern = LinePattern.Dot, smooth = Smooth.None)}));
        end Brake;

        model IdealGearR2T "Gearbox transforming rotational into translational motion"
          extends Modelica.Mechanics.Rotational.Components.IdealGearR2T;
          annotation(
            Documentation(info = "<html>
<p>Couples rotational and translational motion, like a toothed wheel with a toothed rack, specifying the ratio of rotational / translational motion.</p>
</html>"));
        end IdealGearR2T;

        model IdealRollingWheel "Simple 1-dim. model of an ideal rolling wheel without inertia"
          extends Modelica.Mechanics.Rotational.Components.IdealRollingWheel;
          annotation(
            Documentation(info = "<html>
<p>Couples rotational and translational motion, like an ideal rolling wheel, specifying the wheel radius.</p>
</html>"));
        end IdealRollingWheel;

        model InitializeFlange "Initializes a flange with pre-defined position, speed and acceleration (usually, this is reference data from a control bus)"
          extends Modelica.Blocks.Icons.Block;
          parameter Boolean use_s_start = true "= true, if initial position is defined by input s_start, otherwise not initialized";
          parameter Boolean use_v_start = true "= true, if initial speed is defined by input v_start, otherwise not initialized";
          parameter Boolean use_a_start = true "= true, if initial acceleration is defined by input a_start, otherwise not initialized";
          parameter StateSelect stateSelect = StateSelect.default "Priority to use flange angle and speed as states";
          Modelica.Blocks.Interfaces.RealInput s_start(unit = "m") if use_s_start "Initial position of flange" annotation(
            Placement(transformation(extent = {{-140, 40}, {-100, 80}}, rotation = 0)));
          Modelica.Blocks.Interfaces.RealInput v_start(unit = "m/s") if use_v_start "Initial speed of flange" annotation(
            Placement(transformation(extent = {{-140, -20}, {-100, 20}}, rotation = 0)));
          Modelica.Blocks.Interfaces.RealInput a_start(unit = "m/s2") if use_a_start "Initial angular acceleration of flange" annotation(
            Placement(transformation(extent = {{-140, -80}, {-100, -40}}, rotation = 0)));
          Interfaces.Flange_b flange "Flange that is initialized" annotation(
            Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
          Modelica.SIunits.Position s_flange(stateSelect = stateSelect) = flange.s "Flange position";
          Modelica.SIunits.Velocity v_flange(stateSelect = stateSelect) = der(s_flange) "= der(s_flange)";
        protected
          encapsulated model Set_s_start "Set s_start"
            import Modelica;
            extends Modelica.Blocks.Icons.Block;
            Modelica.Blocks.Interfaces.RealInput s_start(unit = "m") "Start position" annotation(
              HideResult = true,
              Placement(transformation(extent = {{-140, -20}, {-100, 20}}, rotation = 0)));
            Modelica.Mechanics.Translational.Interfaces.Flange_b flange annotation(
              Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
          initial equation
            flange.s = s_start;
          equation
            flange.f = 0;
          end Set_s_start;

          encapsulated model Set_v_start "Set v_start"
            import Modelica;
            extends Modelica.Blocks.Icons.Block;
            Modelica.Blocks.Interfaces.RealInput v_start(unit = "m/s") "Start velocity" annotation(
              HideResult = true,
              Placement(transformation(extent = {{-140, -20}, {-100, 20}}, rotation = 0)));
            Modelica.Mechanics.Translational.Interfaces.Flange_b flange annotation(
              Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
          initial equation
            der(flange.s) = v_start;
          equation
            flange.f = 0;
          end Set_v_start;

          encapsulated model Set_a_start "Set a_start"
            import Modelica;
            extends Modelica.Blocks.Icons.Block;
            Modelica.Blocks.Interfaces.RealInput a_start(unit = "m/s2") "Start acceleration" annotation(
              HideResult = true,
              Placement(transformation(extent = {{-140, -20}, {-100, 20}}, rotation = 0)));
            Modelica.Mechanics.Translational.Interfaces.Flange_b flange(s(stateSelect = StateSelect.avoid)) annotation(
              Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
            Modelica.SIunits.Velocity v = der(flange.s) annotation(
              HideResult = true);
          initial equation
            der(v) = a_start;
          equation
            flange.f = 0;
          end Set_a_start;

          encapsulated model Set_flange_f "Set flange_f to zero"
            import Modelica;
            extends Modelica.Blocks.Icons.Block;
            Modelica.Mechanics.Translational.Interfaces.Flange_b flange annotation(
              Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
          equation
            flange.f = 0;
          end Set_flange_f;
        protected
          Set_s_start set_s_start if use_s_start annotation(
            Placement(transformation(extent = {{-20, 50}, {0, 70}}, rotation = 0)));
          Set_v_start set_v_start if use_v_start annotation(
            Placement(transformation(extent = {{-20, -10}, {0, 10}}, rotation = 0)));
          Set_a_start set_a_start if use_a_start annotation(
            Placement(transformation(extent = {{-20, -70}, {0, -50}}, rotation = 0)));
          Set_flange_f set_flange_f annotation(
            Placement(transformation(extent = {{20, -100}, {40, -80}}, rotation = 0)));
        equation
          connect(set_s_start.flange, flange) annotation(
            Line(points = {{0, 60}, {60, 60}, {60, 0}, {100, 0}}, color = {0, 0, 0}, smooth = Smooth.None));
          connect(set_v_start.flange, flange) annotation(
            Line(points = {{0, 0}, {100, 0}}, color = {0, 0, 0}, smooth = Smooth.None));
          connect(set_a_start.flange, flange) annotation(
            Line(points = {{0, -60}, {60, -60}, {60, 0}, {100, 0}}, color = {0, 0, 0}, smooth = Smooth.None));
          connect(set_flange_f.flange, flange) annotation(
            Line(points = {{40, -90}, {60, -90}, {60, 0}, {100, 0}}, color = {0, 0, 0}, smooth = Smooth.None));
          connect(s_start, set_s_start.s_start) annotation(
            Line(points = {{-120, 60}, {-22, 60}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(v_start, set_v_start.v_start) annotation(
            Line(points = {{-120, 0}, {-22, 0}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(a_start, set_a_start.a_start) annotation(
            Line(points = {{-120, -60}, {-22, -60}}, color = {0, 0, 127}, smooth = Smooth.None));
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-94, 74}, {68, 46}}, lineColor = {0, 0, 0}, textString = "s_start"), Text(extent = {{-94, 16}, {70, -14}}, lineColor = {0, 0, 0}, textString = "v_start"), Text(extent = {{-94, -46}, {66, -74}}, lineColor = {0, 0, 0}, textString = "a_start")}),
            Documentation(info = "<html>
<p>
This component is used to optionally initialize the position, speed,
and/or acceleration of the flange to which this component
is connected. Via parameters use_s_start, use_v_start, use_a_start
the corresponding input signals s_start, v_start, a_start are conditionally
activated. If an input is activated, the corresponding flange property
is initialized with the input value at start time.
</p>

<p>
For example, if \"use_s_start = true\", then flange.s is initialized
with the value of the input signal \"s_start\" at the start time.
</p>

<p>
Additionally, it is optionally possible to define the \"StateSelect\"
attribute of the flange position and the flange speed via parameter
\"stateSelection\".
</p>

<p>
This component is especially useful when the initial values of a flange
shall be set according to reference signals of a controller that are
provided via a signal bus.
</p>

</html>"));
        end InitializeFlange;

        model MassWithStopAndFriction "Sliding mass with hard stop and Stribeck friction"
          extends PartialFrictionWithStop;
          SI.Velocity v(start = 0, stateSelect = StateSelect.always) "Absolute velocity of flange_a and flange_b";
          SI.Acceleration a(start = 0) "Absolute acceleration of flange_a and flange_b";
          parameter Modelica.SIunits.Mass m(start = 1) "Mass";
          parameter Real F_prop(final unit = "N.s/m", final min = 0, start = 1) "Velocity dependent friction";
          parameter Modelica.SIunits.Force F_Coulomb(start = 5) "Constant friction: Coulomb force";
          parameter Modelica.SIunits.Force F_Stribeck(start = 10) "Stribeck effect";
          parameter Real fexp(final unit = "s/m", final min = 0, start = 2) "Exponential decay";
          extends Modelica.Thermal.HeatTransfer.Interfaces.PartialElementaryConditionalHeatPortWithoutT;
          Integer stopped;

          encapsulated partial model PartialFrictionWithStop "Base model of Coulomb friction elements with stop"
            import SI = Modelica.SIunits;
            import Modelica.Mechanics.Translational.Interfaces.PartialRigid;
            parameter SI.Position smax(start = 25) "Right stop for (right end of) sliding mass";
            parameter SI.Position smin(start = -25) "Left stop for (left end of) sliding mass";
            parameter SI.Velocity v_small = 1e-3 "Relative velocity near to zero (see model info text)" annotation(
              Dialog(tab = "Advanced"));
            // Equations to define the following variables have to be defined in subclasses
            SI.Velocity v_relfric "Relative velocity between frictional surfaces";
            SI.Acceleration a_relfric "Relative acceleration between frictional surfaces";
            SI.Force f "Friction force (positive, if directed in opposite direction of v_rel)";
            SI.Force f0 "Friction force for v=0 and forward sliding";
            SI.Force f0_max "Maximum friction force for v=0 and locked";
            Boolean free "true, if frictional element is not active";
            // Equations to define the following variables are given in this class
            Real sa(unit = "1") "Path parameter of friction characteristic f = f(a_relfric)";
            Boolean startForward(start = false, fixed = true) "= true, if v_rel=0 and start of forward sliding or v_rel > v_small";
            Boolean startBackward(start = false, fixed = true) "= true, if v_rel=0 and start of backward sliding or v_rel < -v_small";
            Boolean locked(start = false) "true, if v_rel=0 and not sliding";
            extends PartialRigid(s(start = 0, stateSelect = StateSelect.always));
            constant Integer Unknown = 3 "Value of mode is not known";
            constant Integer Free = 2 "Element is not active";
            constant Integer Forward = 1 "v_rel > 0 (forward sliding)";
            constant Integer Stuck = 0 "v_rel = 0 (forward sliding, locked or backward sliding)";
            constant Integer Backward = -1 "v_rel < 0 (backward sliding)";
            Integer mode(final min = Backward, final max = Unknown, start = Unknown, fixed = true);
          protected
            constant SI.Acceleration unitAcceleration = 1 annotation(
              HideResult = true);
            constant SI.Force unitForce = 1 annotation(
              HideResult = true);
          equation
/* Friction characteristic
   (locked is introduced to help the Modelica translator determining
   the different structural configurations,
   if for each configuration special code shall be generated)
*/
            startForward = pre(mode) == Stuck and (sa > f0_max / unitForce and s < smax - L / 2 or pre(startForward) and sa > f0 / unitForce and s < smax - L / 2) or pre(mode) == Backward and v_relfric > v_small or initial() and v_relfric > 0;
            startBackward = pre(mode) == Stuck and (sa < (-f0_max / unitForce) and s > smin + L / 2 or pre(startBackward) and sa < (-f0 / unitForce) and s > smin + L / 2) or pre(mode) == Forward and v_relfric < (-v_small) or initial() and v_relfric < 0;
            locked = not free and not (pre(mode) == Forward or startForward or pre(mode) == Backward or startBackward);
            a_relfric / unitAcceleration = if locked then 0 else if free then sa else if startForward then sa - f0_max / unitForce else if startBackward then sa + f0_max / unitForce else if pre(mode) == Forward then sa - f0_max / unitForce else sa + f0_max / unitForce;
/* Friction torque has to be defined in a subclass. Example for a clutch:
   f = if locked then sa else
       if free then   0 else
       cgeo*fn*(if startForward then          Math.tempInterpol1( v_relfric, mue_pos, 2) else
                if startBackward then        -Math.tempInterpol1(-v_relfric, mue_pos, 2) else
                if pre(mode) == Forward then  Math.tempInterpol1( v_relfric, mue_pos, 2) else
                                             -Math.tempInterpol1(-v_relfric, mue_pos, 2));
*/
// finite state machine to determine configuration
            mode = if free then Free else if (pre(mode) == Forward or pre(mode) == Free or startForward) and v_relfric > 0 and s < smax - L / 2 then Forward else if (pre(mode) == Backward or pre(mode) == Free or startBackward) and v_relfric < 0 and s > smin + L / 2 then Backward else Stuck;
            annotation(
              Documentation(info = "<html>
<p>
Basic model for Coulomb friction that models the stuck phase in a reliable way.<br>
Additionally, a left and right stop are handled.
</p>
</html>"));
          end PartialFrictionWithStop;
        equation
// Constant auxiliary variables
          f0 = F_Coulomb + F_Stribeck;
          f0_max = f0 * 1.001;
          free = f0 <= 0 and F_prop <= 0 and s > smin + L / 2 and s < smax - L / 2;
// Velocity and acceleration of flanges
          v = der(s);
          a = der(v);
          v_relfric = v;
          a_relfric = a;
// Equilibrium of forces
          0 = flange_a.f + flange_b.f - f - m * der(v);
// Friction force
          f = if locked then sa * unitForce else if free then 0 else if startForward then F_prop * v + F_Coulomb + F_Stribeck else if startBackward then F_prop * v - F_Coulomb - F_Stribeck else if pre(mode) == Forward then F_prop * v + F_Coulomb + F_Stribeck * exp(-fexp * abs(v)) else F_prop * v - F_Coulomb - F_Stribeck * exp(-fexp * abs(v));
          lossPower = f * v_relfric;
          when initial() then
            assert(s > smin + L / 2 or s >= smin + L / 2 and v >= 0, "Error in initialization of hard stop. (s - L/2) must be >= smin\n" + "(s=" + String(s) + ", L=" + String(L) + ", smin=" + String(smin) + ")");
            assert(s < smax - L / 2 or s <= smax - L / 2 and v <= 0, "Error in initialization of hard stop. (s + L/2) must be <= smax\n" + "(s=" + String(s) + ", L=" + String(L) + ", smax=" + String(smax) + ")");
          end when;
// Define events for hard stops and reinitialize the state variables velocity v and position s
          stopped = if s <= smin + L / 2 then -1 else if s >= smax - L / 2 then +1 else 0;
          when stopped <> 0 then
            reinit(s, if stopped < 0 then smin + L / 2 else smax - L / 2);
            reinit(v, 0);
          end when;
/*
Version 1:

  when not (s < smax - L/2) then
    reinit(s, smax - L/2);
    if (not initial() or v>0) then
      reinit(v, 0);
    end if;
  end when;

  when not (s > smin + L/2) then
    reinit(s, smin + L/2);
    if (not initial() or v<0) then
      reinit(v, 0);
    end if;
  end when;

Version 2:
        stopped := if s <= smin + L/2 then -1 else if s >= smax - L/2 then +1 else 0;
      when (initial()) then
        assert(s > smin + L/2 or s >= smin + L/2 and v >= 0,
          "Error in initialization of hard stop. (s - L/2) must be >= smin\n"+
          "(s=" + String(s) + ", L=" + String(L) + ", smin=" + String(smin) + ")");
        assert(s < smax - L/2 or s <= smax - L/2 and v <= 0,
          "Error in initialization of hard stop. (s + L/2) must be <= smax\n"+
          "(s=" + String(s) + ", L=" + String(L) + ", smax=" + String(smax) + ")");
      end when;
      when stopped <> 0 then
        reinit(s, if stopped < 0 then smin + L/2 else smax - L/2);
        if (not initial() or stopped*v>0) then
           reinit(v, 0);
        end if;
      end when;
*/
          annotation(
            Documentation(info = "<html>
<p>This element describes the <i>Stribeck friction characteristics</i> of a sliding mass,
i. e. the frictional force acting between the sliding mass and the support. Included is a
<i>hard stop</i> for the position. </p>
<p>
The surface is fixed and there is friction between sliding mass and surface.
The frictional force f is given for positive velocity v by:
</p>
<blockquote><pre>
f = F_Coulomb + F_prop * v + F_Stribeck * exp (-fexp * v)
</pre></blockquote>

<p>
<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/Translational/Stribeck.png\">
</p>

<p>
The distance between the left and the right connector is given by parameter L.
The position of the center of gravity, coordinate s, is in the middle between
the two flanges.</p>
<p>
There are hard stops at smax and smin, i. e. if
<i><code>flange_a.s &gt;= smin</code></i> and <i><code>flange_b.s &lt;= xmax </code></i> the sliding mass can move freely.</p>
<p>When the absolute velocity becomes zero, the sliding mass becomes stuck, i.e., the absolute position remains constant. In this phase the
friction force is calculated from a force balance due to the requirement that the
absolute acceleration shall be zero. The elements begin to slide when the friction
force exceeds a threshold value, called the maximum static friction force, computed via:</p>
<blockquote><pre>
   maximum_static_friction =  F_Coulomb + F_Stribeck
</pre></blockquote>
<p>
<font color=\"#ff0000\"> <b>This requires the states Stop.s and Stop.v</b> </font>. If these states are eliminated during the index reduction
the model will not work. To avoid this any inertias should be connected via springs
to the Stop element, other sliding masses, dampers or hydraulic chambers must be avoided.</p>
<p>For more details of the used friction model see the following reference: </p>

<dl>
<dt>Beater P. (1999):</dt>
<dd><a href=\"http://www.springer.de/cgi-bin/search_book.pl?isbn=3-540-65444-5\">
Entwurf hydraulischer Maschinen</a>. Springer Verlag Berlin Heidelberg New York.</dd>
</dl>

<P>The friction model is implemented in a \"clean\" way by state events and leads to
continuous/discrete systems of equations which have to be solved by appropriate
numerical methods. The method is described in: </P>

<dl>
<dt>Otter M., Elmqvist H., and Mattsson S.E. (1999):</dt>
<dd><i>Hybrid Modeling in Modelica based on the Synchronous Data Flow Principle</i>.
    CACSD'99, Aug. 22.-26, Hawaii. </dd>
</dl>

<P>More precise friction models take into account the elasticity of the material when
the two elements are \"stuck\", as well as other effects, like hysteresis. This has
the advantage that the friction element can be completely described by a differential
equation without events. The drawback is that the system becomes stiff (about 10-20 times
slower simulation) and that more material constants have to be supplied which requires more
sophisticated identification. For more details, see the following references, especially
(Armstrong and Canudas de Witt 1996): </P>
<dl>
<dt>
Armstrong B. (1991):</dt>
<DD><i>Control of Machines with Friction</i>. Kluwer Academic Press, Boston MA.<BR>
</DD>
<DT>Armstrong B., and Canudas de Wit C. (1996): </DT>
<DD><i>Friction Modeling and Compensation.</i> The Control Handbook, edited by W.S.Levine, CRC Press, pp. 1369-1382.<BR>
</DD>
<DT>Canudas de Wit C., Olsson H., Astroem K.J., and Lischinsky P. (1995): </DT>
<DD>A<i> new model for control of systems with friction.</i> IEEE Transactions on Automatic Control, Vol. 40, No. 3, pp. 419-425.<BR>
</DD>
</DL>

<h4>Optional heatPort</h4>
<p>
The dissipated energy is transported in form of heat to the optional heatPort connector
that can be enabled via parameter \"useHeatPort\". Independently whether the heatPort is
or is not enabled, the dissipated power is defined with variable \"lossPower\".
If contact occurs at the hard stops, the lossPower is not correctly modelled
at this time instant, because the hard stop would introduce a Dirac impulse
in the lossPower due to the discontinuously changing kinetic energy of the mass
(lossPower is the derivative of the kinetic energy at the time instant of the impact).
</p>

</html>", revisions = "<html>
<h4>Release Notes:</h4>
<ul>
<li><i>First Version from December 7, 1999 by P. Beater (based on Rotational.BearingFriction)</i> </li>
<li><i>July 14, 2001 by P. Beater, assert on initialization added, diagram modified </i> </li>
<li><i>October 11, 2001, by Hans Olsson, Dassault Syst&egrave;mes AB, modified assert to handle start at stops,
modified event logic such if you have friction parameters equal to zero you do not get events
between the stops.</i> </li>
<li><i>June 10, 2002 by P. Beater, StateSelect.always for variables s and v (instead of fixed=true). </i> </li>
</ul>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{80, -100}, {50, -90}, {50, -110}, {80, -100}}, lineColor = {128, 128, 128}, fillColor = {128, 128, 128}, fillPattern = FillPattern.Solid), Line(points = {{-30, -100}, {50, -100}}, color = {0, 0, 0}), Rectangle(extent = {{-30, 30}, {35, -35}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.Sphere, fillColor = {255, 255, 255}), Line(points = {{-90, 0}, {-30, 0}}, color = {0, 127, 0}), Rectangle(extent = {{-70, -45}, {74, -60}}, lineColor = {0, 0, 0}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-63, -15}, {-55, -45}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Rectangle(extent = {{60, -16}, {69, -45}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Line(points = {{35, 0}, {90, 0}}, color = {0, 127, 0}), Text(extent = {{-150, 80}, {150, 40}}, textString = "%name", lineColor = {0, 0, 255}), Line(points = {{-50, -90}, {-30, -70}}, color = {0, 0, 0}), Line(points = {{-30, -70}, {30, -70}}, color = {0, 0, 0}), Line(points = {{-30, -90}, {-10, -70}}, color = {0, 0, 0}), Line(points = {{-10, -90}, {10, -70}}, color = {0, 0, 0}), Line(points = {{10, -90}, {30, -70}}, color = {0, 0, 0}), Text(extent = {{-150, -110}, {150, -140}}, lineColor = {0, 0, 0}, textString = "m=%m"), Line(visible = useHeatPort, points = {{-100, -100}, {-100, -40}, {3, -40}}, color = {191, 0, 0}, pattern = LinePattern.Dot, smooth = Smooth.None)}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{50, -75}, {20, -65}, {20, -85}, {50, -75}}, lineColor = {128, 128, 128}, fillColor = {128, 128, 128}, fillPattern = FillPattern.Solid), Line(points = {{-60, -75}, {20, -75}}, color = {0, 0, 0}), Rectangle(extent = {{-30, 26}, {35, -9}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.Sphere, fillColor = {255, 255, 255}), Line(points = {{-90, 0}, {-30, 0}}, color = {0, 127, 0}), Line(points = {{35, 0}, {90, 0}}, color = {0, 127, 0}), Rectangle(extent = {{-68, -14}, {76, -29}}, lineColor = {0, 0, 0}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-119, 43}, {-111, 17}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Line(points = {{-111, 43}, {-111, 50}}, color = {0, 0, 0}, pattern = LinePattern.Solid, thickness = 0.25, arrow = {Arrow.None, Arrow.None}), Line(points = {{-151, 49}, {-113, 49}}, color = {0, 0, 0}, pattern = LinePattern.Solid, thickness = 0.25, arrow = {Arrow.None, Arrow.None}), Text(extent = {{-149, 51}, {-126, 60}}, textString = "s min", lineColor = {0, 0, 255}), Polygon(points = {{-121, 52}, {-111, 49}, {-121, 46}, {-121, 52}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Rectangle(extent = {{124, 42}, {132, 17}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Line(points = {{124, 39}, {124, 87}}, color = {0, 0, 0}, pattern = LinePattern.Solid, thickness = 0.25, arrow = {Arrow.None, Arrow.None}), Line(points = {{-19, 78}, {121, 78}}, color = {0, 0, 0}, pattern = LinePattern.Solid, thickness = 0.25, arrow = {Arrow.None, Arrow.None}), Text(extent = {{-17, 83}, {6, 92}}, textString = "s max", lineColor = {0, 0, 255}), Polygon(points = {{114, 81}, {124, 78}, {114, 75}, {114, 81}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Line(points = {{5, 26}, {5, 63}}, color = {0, 0, 0}, pattern = LinePattern.Solid, thickness = 0.25, arrow = {Arrow.None, Arrow.None}), Line(points = {{-77, 58}, {-1, 58}}, color = {0, 0, 0}, pattern = LinePattern.Solid, thickness = 0.25, arrow = {Arrow.None, Arrow.None}), Text(extent = {{-75, 60}, {-38, 71}}, textString = "Position s", lineColor = {0, 0, 255}), Polygon(points = {{-5, 61}, {5, 58}, {-5, 55}, {-5, 61}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Line(points = {{-100, -10}, {-100, -60}}, color = {0, 0, 0}), Line(points = {{100, -10}, {100, -60}}, color = {0, 0, 0}), Polygon(points = {{90, -47}, {100, -50}, {90, -53}, {90, -47}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Polygon(points = {{-90, -47}, {-90, -53}, {-100, -50}, {-90, -47}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Line(points = {{-90, -50}, {92, -50}}, color = {0, 0, 0}), Text(extent = {{-11, -46}, {26, -36}}, textString = "Length L", lineColor = {0, 0, 255})}));
        end MassWithStopAndFriction;

        model RelativeStates "Definition of relative state variables"
          extends Translational.Interfaces.PartialTwoFlanges;
          parameter StateSelect stateSelect = StateSelect.prefer "Priority to use the relative angle and relative speed as states";
          SI.Position s_rel(start = 0, stateSelect = StateSelect.prefer) "Relative position used as state variable";
          SI.Velocity v_rel(start = 0, stateSelect = StateSelect.prefer) "Relative velocity used as state variable";
          SI.Acceleration a_rel(start = 0) "Relative angular acceleration";
        equation
          s_rel = flange_b.s - flange_a.s;
          v_rel = der(s_rel);
          a_rel = der(v_rel);
          flange_a.f = 0;
          flange_b.f = 0;
          annotation(
            Documentation(info = "<html>
<p>
Usually, the absolute position and the absolute velocity of
Modelica.Mechanics.Translational.Inertia models are used as state variables.
In some circumstances, relative quantities are better suited, e.g.,
because it may be easier to supply initial values.
In such cases, model <b>RelativeStates</b> allows the definition of state variables
in the following way:
</p>
<ul>
<li> Connect an instance of this model between two flange connectors.</li>
<li> The <b>relative position</b> and the <b>relative velocity</b>
     between the two connectors are used as <b>state variables</b>.
</ul>
<p>
An example is given in the next figure
</p>

<p>
<IMG src=\"modelica://Modelica/Resources/Images/Mechanics/Translational/relativeStates2.png\" ALT=\"relativeStates2\">
</p>

<p>
Here, the relative position and the relative velocity between
the two masses are used as state variables. Additionally, the
simulator selects either the absolute position and absolute
velocity of model mass1 or of model mass2 as state variables.
</p>

</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-40, 40}, {40, -40}}, lineColor = {0, 255, 255}, fillColor = {0, 255, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-40, 40}, {40, -40}}, textString = "S", lineColor = {0, 0, 255}), Line(points = {{-92, 0}, {-42, 0}}, color = {0, 0, 0}, pattern = LinePattern.Dot), Line(points = {{40, 0}, {90, 0}}, color = {0, 0, 0}, pattern = LinePattern.Dot), Text(extent = {{-150, 90}, {150, 50}}, textString = "%name", lineColor = {0, 0, 255})}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-40, 40}, {40, -40}}, lineColor = {0, 255, 255}, fillColor = {0, 255, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-40, 40}, {40, -40}}, textString = "S", lineColor = {0, 0, 255}), Line(points = {{40, 0}, {90, 0}}, color = {0, 0, 0}, pattern = LinePattern.Dash), Line(points = {{-100, -10}, {-100, -80}}, color = {160, 160, 164}), Line(points = {{100, -10}, {100, -80}}, color = {160, 160, 164}), Polygon(points = {{80, -65}, {80, -55}, {100, -60}, {80, -65}}, lineColor = {160, 160, 164}, fillColor = {160, 160, 164}, fillPattern = FillPattern.Solid), Line(points = {{-100, -60}, {80, -60}}, color = {160, 160, 164}), Text(extent = {{-30, -70}, {30, -90}}, textString = "w_rel", lineColor = {0, 0, 255}), Line(points = {{-76, 80}, {-5, 80}}, color = {128, 128, 128}), Polygon(points = {{14, 80}, {-6, 85}, {-6, 75}, {14, 80}}, lineColor = {128, 128, 128}, fillColor = {128, 128, 128}, fillPattern = FillPattern.Solid), Text(extent = {{18, 87}, {86, 74}}, lineColor = {128, 128, 128}, textString = "moving direction"), Line(points = {{-90, 0}, {-40, 0}}, color = {0, 0, 0}, pattern = LinePattern.Dash)}));
        end RelativeStates;
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(origin = {11.5, 31.183}, lineColor = {64, 64, 64}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Sphere, extent = {{-67, -66}, {44, -6}})}),
          Documentation(info = "<html>
<p>
This package contains basic components 1D mechanical translational drive trains.
</p>
</html>"));
      end Components;

      package Sensors "Sensors for 1-dim. translational mechanical quantities"
        extends Modelica.Icons.SensorsPackage;

        model PositionSensor "Ideal sensor to measure the absolute position"
          extends Translational.Interfaces.PartialAbsoluteSensor;
          Modelica.Blocks.Interfaces.RealOutput s(unit = "m") "Absolute position of flange as output signal" annotation(
            Placement(transformation(extent = {{100, -11}, {120, 9}}, rotation = 0), iconTransformation(extent = {{100, -10}, {120, 10}})));
        equation
          s = flange.s;
          annotation(
            Documentation(info = "<html>
<p>
Measures the <i>absolute position s</i> of a flange in an ideal way and provides the result as
output signals (to be further processed with blocks of the
Modelica.Blocks library).
</p>

</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-70, 0}, {-90, 0}}, color = {0, 0, 0}), Text(extent = {{80, -28}, {114, -62}}, lineColor = {0, 0, 0}, textString = "s")}));
        end PositionSensor;

        model SpeedSensor "Ideal sensor to measure the absolute velocity"
          extends Translational.Interfaces.PartialAbsoluteSensor;
          Modelica.Blocks.Interfaces.RealOutput v(unit = "m/s") "Absolute velocity of flange as output signal" annotation(
            Placement(transformation(extent = {{100, -10}, {120, 10}}, rotation = 0)));
        equation
          v = der(flange.s);
          annotation(
            Documentation(info = "<html>
<p>
Measures the <i>absolute velocity v</i> of a flange in an ideal way and provides the result as
output signals (to be further processed with blocks of the
Modelica.Blocks library).
</p>

</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-70, 0}, {-90, 0}}, color = {0, 0, 0}), Text(extent = {{80, -28}, {111, -61}}, lineColor = {0, 0, 0}, textString = "v")}));
        end SpeedSensor;

        model AccSensor "Ideal sensor to measure the absolute acceleration"
          extends Translational.Interfaces.PartialAbsoluteSensor;
          SI.Velocity v "Absolute velocity of flange";
          Modelica.Blocks.Interfaces.RealOutput a(unit = "m/s2") "Absolute acceleration of flange as output signal" annotation(
            Placement(transformation(extent = {{100, -10}, {120, 10}}, rotation = 0)));
        equation
          v = der(flange.s);
          a = der(v);
          annotation(
            Documentation(info = "<html>
<p>
Measures the <i>absolute acceleration a</i>
of a flange in an ideal way and provides the result as
output signals (to be further processed with blocks of the
Modelica.Blocks library).
</p>

</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-70, 0}, {-90, 0}}, color = {0, 0, 0}), Text(extent = {{80, -28}, {115, -60}}, lineColor = {0, 0, 0}, textString = "a")}));
        end AccSensor;

        model RelPositionSensor "Ideal sensor to measure the relative position"
          extends Translational.Interfaces.PartialRelativeSensor;
          Modelica.Blocks.Interfaces.RealOutput s_rel(unit = "m") "Distance between two flanges (= flange_b.s - flange_a.s) as output signal" annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 270, origin = {0, -110})));
        equation
          s_rel = flange_b.s - flange_a.s;
          0 = flange_a.f;
          annotation(
            Documentation(info = "<html>
<p>
Measures the <i>relative position s</i> of a flange in an ideal way and provides the result as
output signals (to be further processed with blocks of the
Modelica.Blocks library).
</p>

</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-70, 0}, {-90, 0}}, color = {0, 0, 0}), Line(points = {{70.4, 0}, {100, 0}}, color = {0, 0, 127}), Text(extent = {{8, -68}, {42, -102}}, lineColor = {0, 0, 0}, textString = "s"), Line(points = {{0, -99}, {0, -60}}, color = {0, 0, 127})}));
        end RelPositionSensor;

        model RelSpeedSensor "Ideal sensor to measure the relative speed"
          extends Translational.Interfaces.PartialRelativeSensor;
          SI.Position s_rel "Distance between the two flanges (flange_b.s - flange_a.s)";
          Modelica.Blocks.Interfaces.RealOutput v_rel(unit = "m/s") "Relative velocity between two flanges (= der(flange_b.s) - der(flange_a.s)) as output signal" annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 270, origin = {0, -110})));
        equation
          s_rel = flange_b.s - flange_a.s;
          v_rel = der(s_rel);
          0 = flange_a.f;
          annotation(
            Documentation(info = "<html>
<p>
Measures the <i>relative speed v</i> of a flange in an ideal way and provides the result as
output signals (to be further processed with blocks of the
Modelica.Blocks library).
</p>

</html>", revisions = "<html>
<p><b>Release Notes:</b></p>
<ul>
<li><i>First Version from August 26, 1999 by P. Beater</i> </li>
</ul>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-70, 0}, {-90, 0}}, color = {0, 0, 0}), Line(points = {{70.4, 0}, {100, 0}}, color = {0, 0, 127}), Text(extent = {{8, -68}, {42, -102}}, lineColor = {0, 0, 0}, textString = "v"), Line(points = {{0, -100}, {0, -61}}, color = {0, 0, 127})}));
        end RelSpeedSensor;

        model RelAccSensor "Ideal sensor to measure the relative acceleration"
          extends Translational.Interfaces.PartialRelativeSensor;
          SI.Position s_rel "Distance between the two flanges (flange_b.s - flange_a.s)";
          SI.Velocity v_rel "Relative velocity between the two flanges (der(flange_b.s) - der(flange_a.s))";
          Modelica.Blocks.Interfaces.RealOutput a_rel(unit = "m/s2") "Relative acceleration between two flanges (= der(v_rel)) as output signal" annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 270, origin = {0, -110})));
        equation
          s_rel = flange_b.s - flange_a.s;
          v_rel = der(s_rel);
          a_rel = der(v_rel);
          0 = flange_a.f;
          annotation(
            Documentation(info = "<html>
<p>
Measures the <i>relative acceleration a</i> of a flange in an ideal way and provides the result as
output signals (to be further processed with blocks of the
Modelica.Blocks library).
</p>

</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-70, 0}, {-90, 0}}, color = {0, 0, 0}), Line(points = {{70.4, 0}, {100, 0}}, color = {0, 0, 127}), Text(extent = {{7, -68}, {41, -102}}, lineColor = {0, 0, 0}, textString = "a"), Line(points = {{0, -99}, {0, -60}}, color = {0, 0, 127})}));
        end RelAccSensor;

        model ForceSensor "Ideal sensor to measure the force between two flanges"
          extends Translational.Interfaces.PartialRelativeSensor;
          Modelica.Blocks.Interfaces.RealOutput f(unit = "N") "Force in flange_a and flange_b (f = flange_a.f = -flange_b.f) as output signal" annotation(
            Placement(transformation(origin = {-80, -110}, extent = {{10, -10}, {-10, 10}}, rotation = 90)));
        equation
          flange_a.s = flange_b.s;
          flange_a.f = f;
          annotation(
            Documentation(info = "<html>
<p>
Measures the <i>cut-force between two flanges</i> in an ideal way
and provides the result as output signal (to be further processed
with blocks of the Modelica.Blocks library).
</p>

</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-40, -70}, {40, -120}}, lineColor = {0, 0, 0}, textString = "f"), Line(points = {{-70, 0}, {-90, 0}}, color = {0, 0, 0}), Line(points = {{70, 0}, {90, 0}}, color = {0, 0, 0}), Line(points = {{-80, -100}, {-80, 0}}, color = {0, 0, 127})}));
        end ForceSensor;

        model PowerSensor "Ideal sensor to measure the power between two flanges (= flange_a.f*der(flange_a.s))"
          extends Translational.Interfaces.PartialRelativeSensor;
          Modelica.Blocks.Interfaces.RealOutput power(unit = "W") "Power in flange flange_a as output signal" annotation(
            Placement(transformation(origin = {-80, -110}, extent = {{10, -10}, {-10, 10}}, rotation = 90)));
        equation
          flange_a.s = flange_b.s;
          power = flange_a.f * der(flange_a.s);
          annotation(
            Documentation(info = "<html>
<p>
Measures the <b>power between two flanges</b> in an ideal way
and provides the result as output signal <b>power</b>
(to be further processed with blocks of the Modelica.Blocks library).
</p>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-75, -79}, {67, -119}}, lineColor = {0, 0, 0}, textString = "power"), Line(points = {{-70, 0}, {-90, 0}}, color = {0, 0, 0}), Line(points = {{70, 0}, {90, 0}}, color = {0, 0, 0}), Line(points = {{-80, -100}, {-80, 0}}, color = {0, 0, 127})}));
        end PowerSensor;

        model MultiSensor "Ideal sensor to measure the absolute velocity, force and power between two flanges"
          extends Translational.Interfaces.PartialRelativeSensor;
          Modelica.Blocks.Interfaces.RealOutput power(unit = "W") "Power in flange flange_a as output signal" annotation(
            Placement(transformation(origin = {-60, -110}, extent = {{10, -10}, {-10, 10}}, rotation = 90)));
          Modelica.Blocks.Interfaces.RealOutput f(unit = "N") "Force in flange_a and flange_b (f = flange_a.f = -flange_b.f) as output signal" annotation(
            Placement(transformation(extent = {{10, -10}, {-10, 10}}, rotation = 90, origin = {0, -110})));
          Modelica.Blocks.Interfaces.RealOutput v(unit = "m/s") "Absolute velocity of flange as output signal as output signal" annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = -90, origin = {60, -110})));
        equation
          flange_a.s = flange_b.s;
          f = flange_a.f;
          v = der(flange_a.s);
          power = f * v;
          annotation(
            Documentation(info = "<html>
<p>
Measures the <b>absolute velocity</b> of a flange_a, the <b>cut-force</b> and <b>power</b> between two flanges in an
ideal way and provides the results as output signals <b>v</b>, <b>f</b> and <b>power</b>, respectively. </p>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-146, -70}, {-56, -100}}, lineColor = {0, 0, 0}, textString = "power"), Line(points = {{-70, 0}, {-90, 0}}, color = {0, 0, 0}), Line(points = {{70, 0}, {90, 0}}, color = {0, 0, 0}), Line(points = {{-60, -100}, {-60, -60}}, color = {0, 0, 127}), Text(extent = {{-28, -71}, {52, -101}}, lineColor = {0, 0, 0}, textString = "f"), Line(points = {{0, -100}, {0, -60}}, color = {0, 0, 127}), Line(points = {{60, -100}, {60, -60}}, color = {0, 0, 127}), Text(extent = {{60, -70}, {114, -101}}, lineColor = {0, 0, 0}, textString = "v")}));
        end MultiSensor;
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-56, -61}, {-56, -81}}, color = {0, 0, 0}), Line(points = {{-36, -61}, {-36, -81}}, color = {0, 0, 0}), Line(points = {{-16, -61}, {-16, -81}}, color = {0, 0, 0}), Line(points = {{4, -61}, {4, -81}}, color = {0, 0, 0}), Line(points = {{24, -61}, {24, -81}}, color = {0, 0, 0}), Line(points = {{44, -61}, {44, -81}}, color = {0, 0, 0})}),
          Documentation(info = "<html>
<p>
This package contains ideal sensor components that provide
the connector variables as signals for further processing with the
Modelica.Blocks library.
</p>
</html>"));
      end Sensors;

      package Sources "Sources to drive 1D translational mechanical components"
        extends Modelica.Icons.SourcesPackage;

        model Position "Forced movement of a flange according to a reference position"
          extends Modelica.Mechanics.Translational.Interfaces.PartialElementaryOneFlangeAndSupport2(s(stateSelect = if exact then StateSelect.default else StateSelect.prefer));
          parameter Boolean exact = false "true/false exact treatment/filtering the input signal";
          parameter SI.Frequency f_crit = 50 "if exact=false, critical frequency of filter to filter input signal" annotation(
            Dialog(enable = not exact));
          SI.Velocity v(start = 0, stateSelect = if exact then StateSelect.default else StateSelect.prefer) "If exact=false, absolute velocity of flange_b else dummy";
          SI.Acceleration a(start = 0) "If exact=false, absolute acceleration of flange_b else dummy";
          Modelica.Blocks.Interfaces.RealInput s_ref(unit = "m") "Reference position of flange as input signal" annotation(
            Placement(transformation(extent = {{-140, -20}, {-100, 20}}, rotation = 0)));
        protected
          parameter Modelica.SIunits.AngularFrequency w_crit = 2 * Modelica.Constants.pi * f_crit "Critical frequency";
          constant Real af = 1.3617 "s coefficient of Bessel filter";
          constant Real bf = 0.6180 "s*s coefficient of Bessel filter";
        initial equation
          if not exact then
            s = s_ref;
          end if;
        equation
          if exact then
            s = s_ref;
            v = 0;
            a = 0;
          else
// Filter: a = s_ref*S^2/(1 + (af/w_crit)*S + (bf/w_crit^2)*S^2)
            v = der(s);
            a = der(v);
            a = ((s_ref - s) * w_crit - af * v) * (w_crit / bf);
          end if;
          annotation(
            Documentation(info = "<HTML>
<p>
The input signal <b>s_ref</b> defines the <b>reference
position</b> in [m]. Flange <b>flange_b</b> is <b>forced</b>
to move relative to the support connector according to this reference motion. According to parameter
<b>exact</b> (default = <b>false</b>), this is done in the following way:
<ol>
<li><b>exact=true</b><br>
    The reference position is treated <b>exactly</b>. This is only possible, if
    the input signal is defined by an analytical function which can be
    differentiated at least twice. If this prerequisite is fulfilled,
    the Modelica translator will differentiate the input signal twice
    in order to compute the reference acceleration of the flange.</li>
<li><b>exact=false</b><br>
    The reference position is <b>filtered</b> and the second derivative
    of the filtered curve is used to compute the reference acceleration
    of the flange. This second derivative is <b>not</b> computed by
    numerical differentiation but by an appropriate realization of the
    filter. For filtering, a second order Bessel filter is used.
    The critical frequency (also called cut-off frequency) of the
    filter is defined via parameter <b>f_crit</b> in [Hz]. This value
    should be selected in such a way that it is higher as the essential
    low frequencies in the signal.</li>
</ol>
<p>
The input signal can be provided from one of the signal generator
blocks of the block library Modelica.Blocks.Sources.
</p>

</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-56, -36}, {-178, -66}}, lineColor = {0, 0, 0}, textString = "s_ref"), Rectangle(extent = {{-100, 20}, {100, -20}}, lineColor = {0, 127, 0}, fillColor = {215, 215, 215}, fillPattern = FillPattern.Solid), Text(extent = {{150, 60}, {-150, 100}}, textString = "%name", lineColor = {0, 0, 255}), Line(points = {{0, 52}, {0, 32}}, color = {0, 0, 0}), Line(points = {{-29, 32}, {30, 32}}, color = {0, 0, 0}), Line(points = {{-30, -32}, {30, -32}}, color = {0, 0, 0}), Line(points = {{0, -32}, {0, -100}}, color = {0, 0, 0}), Line(points = {{30, -42}, {20, -52}}, color = {0, 0, 0}), Line(points = {{30, -32}, {10, -52}}, color = {0, 0, 0}), Line(points = {{20, -32}, {0, -52}}, color = {0, 0, 0}), Line(points = {{10, -32}, {-10, -52}}, color = {0, 0, 0}), Line(points = {{0, -32}, {-20, -52}}, color = {0, 0, 0}), Line(points = {{-10, -32}, {-30, -52}}, color = {0, 0, 0}), Line(points = {{-20, -32}, {-30, -42}}, color = {0, 0, 0}), Text(extent = {{144, -30}, {30, -60}}, lineColor = {0, 0, 0}, textString = "exact="), Text(extent = {{134, -68}, {22, -96}}, lineColor = {0, 0, 0}, textString = "%exact")}));
        end Position;

        model Speed "Forced movement of a flange according to a reference speed"
          extends Modelica.Mechanics.Translational.Interfaces.PartialElementaryOneFlangeAndSupport2(s(start = 0, fixed = true, stateSelect = StateSelect.prefer));
          parameter Boolean exact = false "true/false exact treatment/filtering the input signal";
          parameter SI.Frequency f_crit = 50 "if exact=false, critical frequency of filter to filter input signal" annotation(
            Dialog(enable = not exact));
          SI.Velocity v(stateSelect = if exact then StateSelect.default else StateSelect.prefer) "Absolute velocity of flange_b";
          SI.Acceleration a "If exact=false, absolute acceleration of flange_b else dummy";
          Modelica.Blocks.Interfaces.RealInput v_ref(unit = "m/s") "Reference speed of flange as input signal" annotation(
            Placement(transformation(extent = {{-140, -20}, {-100, 20}}, rotation = 0)));
        protected
          parameter Modelica.SIunits.AngularFrequency w_crit = 2 * Modelica.Constants.pi * f_crit "Critical frequency";
        initial equation
          if not exact then
            v = v_ref;
          end if;
        equation
          v = der(s);
          if exact then
            v = v_ref;
            a = 0;
          else
// Filter: a = v_ref/(1 + (1/w_crit)*s)
            a = der(v);
            a = (v_ref - v) * w_crit;
          end if;
          annotation(
            Documentation(info = "<HTML>
<p>
The input signal <b>v_ref</b> defines the <b>reference
speed</b> in [m/s]. Flange <b>flange_b</b> is <b>forced</b>
to move relative to the support connector  according to this reference motion. According to parameter
<b>exact</b> (default = <b>false</b>), this is done in the following way:
<ol>
<li><b>exact=true</b><br>
    The reference speed is treated <b>exactly</b>. This is only possible, if
    the input signal is defined by an analytical function which can be
    differentiated at least once. If this prerequisite is fulfilled,
    the Modelica translator will differentiate the input signal once
    in order to compute the reference acceleration of the flange.</li>
<li><b>exact=false</b><br>
    The reference speed is <b>filtered</b> and the first derivative
    of the filtered curve is used to compute the reference acceleration
    of the flange. This first derivative is <b>not</b> computed by
    numerical differentiation but by an appropriate realization of the
    filter. For filtering, a first order filter is used.
    The critical frequency (also called cut-off frequency) of the
    filter is defined via parameter <b>f_crit</b> in [Hz]. This value
    should be selected in such a way that it is higher as the essential
    low frequencies in the signal.</li>
</ol>
<p>
The input signal can be provided from one of the signal generator
blocks of the block library Modelica.Blocks.Sources.
</p>

</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-54, -36}, {-174, -68}}, lineColor = {0, 0, 0}, textString = "v_ref"), Line(points = {{-30, -32}, {30, -32}}, color = {0, 0, 0}), Line(points = {{0, -32}, {0, -100}}, color = {0, 0, 0}), Line(points = {{30, -42}, {20, -52}}, color = {0, 0, 0}), Line(points = {{30, -32}, {10, -52}}, color = {0, 0, 0}), Line(points = {{20, -32}, {0, -52}}, color = {0, 0, 0}), Line(points = {{10, -32}, {-10, -52}}, color = {0, 0, 0}), Line(points = {{0, -32}, {-20, -52}}, color = {0, 0, 0}), Line(points = {{-10, -32}, {-30, -52}}, color = {0, 0, 0}), Line(points = {{-20, -32}, {-30, -42}}, color = {0, 0, 0}), Rectangle(extent = {{-100, 20}, {100, -20}}, lineColor = {0, 127, 0}, fillColor = {215, 215, 215}, fillPattern = FillPattern.Solid), Line(points = {{-29, 32}, {30, 32}}, color = {0, 0, 0}), Line(points = {{0, 52}, {0, 32}}, color = {0, 0, 0}), Text(extent = {{150, 60}, {-150, 100}}, textString = "%name", lineColor = {0, 0, 255}), Text(extent = {{146, -38}, {32, -64}}, lineColor = {0, 0, 0}, textString = "exact="), Text(extent = {{140, -76}, {22, -102}}, lineColor = {0, 0, 0}, textString = "%exact")}));
        end Speed;

        model Accelerate "Forced movement of a flange according to an acceleration signal"
          extends Modelica.Mechanics.Translational.Interfaces.PartialElementaryOneFlangeAndSupport2(s(start = 0, fixed = true, stateSelect = StateSelect.prefer));
          SI.Velocity v(start = 0, fixed = true, stateSelect = StateSelect.prefer) "Absolute velocity of flange_b";
          SI.Acceleration a "Absolute acceleration of flange_b";
          Modelica.Blocks.Interfaces.RealInput a_ref(unit = "m/s2") "Absolute acceleration of flange as input signal" annotation(
            Placement(transformation(extent = {{-140, -20}, {-100, 20}}, rotation = 0)));
        equation
          v = der(s);
          a = der(v);
          a = a_ref;
          annotation(
            Documentation(info = "<html>
<p>
The input signal <b>a</b> in [m/s2] moves the 1D translational flange
connector flange_b with a predefined <i>acceleration</i>, i.e., the flange
is <i>forced</i> to move relative to the support connector  with this acceleration. The velocity and the
position of the flange are also predefined and are determined by
integration of the acceleration.
</p>
<p>
The acceleration \"a(t)\" can be provided from one of the signal generator
blocks of the block library Modelica.Blocks.Source.
</p>

</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-56, -40}, {-166, -68}}, lineColor = {0, 0, 0}, textString = "a_ref"), Line(points = {{-30, -32}, {30, -32}}, color = {0, 0, 0}), Line(points = {{0, -32}, {0, -100}}, color = {0, 0, 0}), Line(points = {{30, -42}, {20, -52}}, color = {0, 0, 0}), Line(points = {{30, -32}, {10, -52}}, color = {0, 0, 0}), Line(points = {{20, -32}, {0, -52}}, color = {0, 0, 0}), Line(points = {{10, -32}, {-10, -52}}, color = {0, 0, 0}), Line(points = {{0, -32}, {-20, -52}}, color = {0, 0, 0}), Line(points = {{-10, -32}, {-30, -52}}, color = {0, 0, 0}), Line(points = {{-20, -32}, {-30, -42}}, color = {0, 0, 0}), Rectangle(extent = {{-100, 20}, {100, -20}}, lineColor = {0, 127, 0}, fillColor = {215, 215, 215}, fillPattern = FillPattern.Solid), Line(points = {{-29, 32}, {30, 32}}, color = {0, 0, 0}), Line(points = {{0, 52}, {0, 32}}, color = {0, 0, 0}), Text(extent = {{150, 60}, {-150, 100}}, textString = "%name", lineColor = {0, 0, 255})}));
        end Accelerate;

        model Move "Forced movement of a flange according to a position, velocity and acceleration signal"
          extends Modelica.Mechanics.Translational.Interfaces.PartialElementaryOneFlangeAndSupport2;
          Modelica.Blocks.Interfaces.RealInput u[3] "Position, velocity and acceleration of flange as input signals" annotation(
            Placement(transformation(extent = {{-140, -20}, {-100, 20}}, rotation = 0)));
        protected
          function position
            extends Modelica.Icons.Function;
            input Real q_qd_qdd[3] "Required values for position, speed, acceleration";
            input Real dummy "Just to have one input signal that should be differentiated to avoid possible problems in the Modelica tool (is not used)";
            output Real q;
          algorithm
            q := q_qd_qdd[1];
            annotation(
              derivative(noDerivative = q_qd_qdd) = position_der,
              Inline = false);
          end position;

          function position_der
            extends Modelica.Icons.Function;
            input Real q_qd_qdd[3] "Required values for position, speed, acceleration";
            input Real dummy "Just to have one input signal that should be differentiated to avoid possible problems in the Modelica tool (is not used)";
            input Real dummy_der;
            output Real qd;
          algorithm
            qd := q_qd_qdd[2];
            annotation(
              derivative(noDerivative = q_qd_qdd, order = 2) = position_der2,
              Inline = false);
          end position_der;

          function position_der2
            extends Modelica.Icons.Function;
            input Real q_qd_qdd[3] "Required values for position, speed, acceleration";
            input Real dummy "Just to have one input signal that should be differentiated to avoid possible problems in the Modelica tool (is not used)";
            input Real dummy_der;
            input Real dummy_der2;
            output Real qdd;
          algorithm
            qdd := q_qd_qdd[3];
          end position_der2;
        equation
          s = position(u, time);
          annotation(
            Documentation(info = "<html>
<p>
Flange <b>flange_b</b> is <b>forced</b> to move relative to the support connector  with a predefined motion
according to the input signals:
</p>
<pre>
    u[1]: position of flange
    u[2]: velocity of flange
    u[3]: acceleration of flange
</pre>
<p>
The user has to guarantee that the input signals are consistent to each other,
i.e., that u[2] is the derivative of u[1] and that
u[3] is the derivative of u. There are, however,
also applications where by purpose these conditions do not hold. For example,
if only the position dependent terms of a mechanical system shall be
calculated, one may provide position = position(t) and set the velocity
and the acceleration to zero.
</p>
<p>
The input signals can be provided from one of the signal generator
blocks of the block library Modelica.Blocks.Sources.
</p>

</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-192, -38}, {-32, -70}}, lineColor = {0, 0, 0}, textString = "s,v,a"), Line(points = {{-30, -32}, {30, -32}}, color = {0, 0, 0}), Line(points = {{0, -32}, {0, -100}}, color = {0, 0, 0}), Line(points = {{30, -42}, {20, -52}}, color = {0, 0, 0}), Line(points = {{30, -32}, {10, -52}}, color = {0, 0, 0}), Line(points = {{20, -32}, {0, -52}}, color = {0, 0, 0}), Line(points = {{10, -32}, {-10, -52}}, color = {0, 0, 0}), Line(points = {{0, -32}, {-20, -52}}, color = {0, 0, 0}), Line(points = {{-10, -32}, {-30, -52}}, color = {0, 0, 0}), Line(points = {{-20, -32}, {-30, -42}}, color = {0, 0, 0}), Rectangle(extent = {{-100, 20}, {100, -20}}, lineColor = {0, 127, 0}, fillColor = {215, 215, 215}, fillPattern = FillPattern.Solid), Line(points = {{0, 52}, {0, 32}}, color = {0, 0, 0}), Line(points = {{-29, 32}, {30, 32}}, color = {0, 0, 0}), Text(extent = {{150, 60}, {-150, 100}}, textString = "%name", lineColor = {0, 0, 255})}));
        end Move;

        model Force "External force acting on a drive train element as input signal"
          extends Modelica.Mechanics.Translational.Interfaces.PartialElementaryOneFlangeAndSupport2;
          Modelica.Blocks.Interfaces.RealInput f(unit = "N") "Driving force as input signal" annotation(
            Placement(transformation(extent = {{-140, -20}, {-100, 20}}, rotation = 0)));
        equation
          flange.f = -f;
          annotation(
            Documentation(info = "<html>
<p>
The input signal \"f\" in [N] characterizes an <i>external
force</i> which acts (with positive sign) at a flange,
i.e., the component connected to the flange is driven by force f.
</p>
<p>
Input signal f can be provided from one of the signal generator
blocks of Modelica.Blocks.Source.
</p>

</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{-100, 10}, {20, 10}, {20, 41}, {90, 0}, {20, -41}, {20, -10}, {-100, -10}, {-100, 10}}, lineColor = {0, 127, 0}, fillColor = {215, 215, 215}, fillPattern = FillPattern.Solid), Text(extent = {{-150, -32}, {-80, -62}}, lineColor = {0, 0, 0}, textString = "f"), Text(extent = {{-150, 90}, {150, 50}}, textString = "%name", lineColor = {0, 0, 255}), Line(points = {{-30, -60}, {30, -60}}, color = {0, 0, 0}), Line(points = {{0, -60}, {0, -101}}, color = {0, 0, 0}), Line(points = {{-30, -80}, {-10, -60}}, color = {0, 0, 0}), Line(points = {{-10, -80}, {10, -60}}, color = {0, 0, 0}), Line(points = {{10, -80}, {30, -60}}, color = {0, 0, 0}), Polygon(points = {{-61, -50}, {-30, -40}, {-30, -60}, {-61, -50}}, lineColor = {0, 0, 0}, fillColor = {128, 128, 128}, fillPattern = FillPattern.Solid), Line(points = {{-31, -50}, {50, -50}}, color = {0, 0, 0}), Line(points = {{-50, -80}, {-30, -60}}, color = {0, 0, 0})}));
        end Force;

        model Force2 "Input signal acting as torque on two flanges"
          extends Translational.Interfaces.PartialTwoFlanges;
          Modelica.Blocks.Interfaces.RealInput f(unit = "N") "Driving force as input signal" annotation(
            Placement(transformation(extent = {{-20, -20}, {20, 20}}, rotation = 270, origin = {0, 60}), iconTransformation(extent = {{-20, -20}, {20, 20}}, rotation = 270, origin = {0, 40})));
        equation
          flange_a.f = f;
          flange_b.f = -f;
          annotation(
            Documentation(info = "<html>
<p>
The input signal \"f\" in [N] characterizes an <i>external
force</i> which acts (with positive sign) at both flanges,
i.e., the components connected to these flanges are driven by force f.
</p>
<p>
Input signal s can be provided from one of the signal generator
blocks of Modelica.Blocks.Source.
</p>

</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-150, -40}, {150, -80}}, textString = "%name", lineColor = {0, 0, 255}), Polygon(points = {{90, 0}, {60, -30}, {60, -10}, {10, -10}, {10, 10}, {60, 10}, {60, 31}, {90, 0}}, lineColor = {0, 127, 0}, smooth = Smooth.None, fillColor = {215, 215, 215}, fillPattern = FillPattern.Solid), Polygon(points = {{-90, 0}, {-60, 30}, {-60, 10}, {-10, 10}, {-10, -10}, {-60, -10}, {-60, -30}, {-90, 0}}, lineColor = {0, 127, 0}, smooth = Smooth.None, fillColor = {215, 215, 215}, fillPattern = FillPattern.Solid)}));
        end Force2;

        model LinearSpeedDependentForce "Linear dependency of force versus speed"
          extends Modelica.Mechanics.Translational.Interfaces.PartialForce;
          parameter Modelica.SIunits.Force f_nominal "Nominal force (if negative, force is acting as load)";
          parameter Boolean ForceDirection = true "Same direction of force in both directions of movement";
          parameter Modelica.SIunits.Velocity v_nominal(min = Modelica.Constants.eps) "Nominal speed";
          Modelica.SIunits.Velocity v "Velocity of flange with respect to support (= der(s))";
        equation
          v = der(s);
          if ForceDirection then
            f = -f_nominal * abs(v / v_nominal);
          else
            f = -f_nominal * (v / v_nominal);
          end if;
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-100, -100}, {100, 100}}, color = {0, 0, 255})}),
            Documentation(info = "<HTML>
<p>
Model of force, linearly dependent on velocity of flange.<br>
Parameter ForceDirection chooses whether direction of force is the same in both directions of movement or not.
</p>
</HTML>"));
        end LinearSpeedDependentForce;

        model QuadraticSpeedDependentForce "Quadratic dependency of force versus speed"
          extends Modelica.Mechanics.Translational.Interfaces.PartialForce;
          parameter Modelica.SIunits.Force f_nominal "Nominal force (if negative, force is acting as load)";
          parameter Boolean ForceDirection = true "Same direction of force in both directions of movement";
          parameter Modelica.SIunits.Velocity v_nominal(min = Modelica.Constants.eps) "Nominal speed";
          Modelica.SIunits.Velocity v "Velocity of flange with respect to support (= der(s))";
        equation
          v = der(s);
          if ForceDirection then
            f = -f_nominal * (v / v_nominal) ^ 2;
          else
            f = -f_nominal * smooth(1, if v >= 0 then (v / v_nominal) ^ 2 else -(v / v_nominal) ^ 2);
          end if;
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-100, -100}, {-80, -98}, {-60, -92}, {-40, -82}, {-20, -68}, {0, -50}, {20, -28}, {40, -2}, {60, 28}, {80, 62}, {100, 100}}, color = {0, 0, 127}, smooth = Smooth.Bezier)}),
            Documentation(info = "<HTML>
<p>
Model of force, quadratic dependent on velocity of flange.<br>
Parameter ForceDirection chooses whether direction of force is the same in both directions of movement or not.
</p>
</HTML>"));
        end QuadraticSpeedDependentForce;

        model ConstantForce "Constant force, not dependent on speed"
          extends Modelica.Mechanics.Translational.Interfaces.PartialForce;
          parameter Modelica.SIunits.Force f_constant "Nominal force (if negative, force is acting as load)";
        equation
          f = -f_constant;
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-100, 0}, {98, 0}}, color = {0, 0, 255}), Text(extent = {{-118, 58}, {126, 34}}, lineColor = {0, 0, 0}, textString = "%f_constant")}),
            Documentation(info = "<HTML>
<p>
Model of constant force, not dependent on velocity of flange.<br>
Positive force acts accelerating.
</p>
</HTML>"));
        end ConstantForce;

        model ConstantSpeed "Constant speed, not dependent on force"
          extends Modelica.Mechanics.Translational.Interfaces.PartialForce;
          parameter Modelica.SIunits.Velocity v_fixed "Fixed speed (if negative, force is acting as load)";
          Modelica.SIunits.Velocity v "Velocity of flange with respect to support (= der(s))";
        equation
          v = der(s);
          v = v_fixed;
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{0, -100}, {0, 100}}, color = {0, 0, 255}), Text(extent = {{-120, 60}, {124, 36}}, lineColor = {0, 0, 0}, textString = "%v_fixed")}),
            Documentation(info = "<HTML>
<p>
Model of <b>fixed</b> velocity of flange, not dependent on force.
</p>
</HTML>"));
        end ConstantSpeed;

        model ForceStep "Constant force, not dependent on speed"
          extends Modelica.Mechanics.Translational.Interfaces.PartialForce;
          parameter Modelica.SIunits.Force stepForce(start = 1) "Height of force step (if negative, force is acting as load)";
          parameter Modelica.SIunits.Force offsetForce(start = 0) "Offset of force";
          parameter Modelica.SIunits.Time startTime = 0 "Force = offset for time < startTime";
        equation
          f = (-offsetForce) - (if time < startTime then 0 else stepForce);
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-80, -60}, {0, -60}, {0, 60}, {80, 60}}, color = {0, 0, 255}), Text(extent = {{0, -40}, {100, -60}}, lineColor = {0, 0, 0}, textString = "time")}),
            Documentation(info = "<HTML>
<p>
Model of a force step at time .<br>
Positive force acts accelerating.
</p>
</HTML>"));
        end ForceStep;
        annotation(
          Documentation(info = "<html>
<p>
This package contains ideal sources to drive 1D mechanical translational drive trains.
</p>
</html>"));
      end Sources;

      package Interfaces "Interfaces for 1-dim. translational mechanical components"
        extends Modelica.Icons.InterfacesPackage;

        connector Flange_a "(left) 1D translational flange (flange axis directed INTO cut plane, e. g. from left to right)"
          SI.Position s "Absolute position of flange";
          flow SI.Force f "Cut force directed into flange";
          annotation(
            defaultComponentName = "flange_a",
            Documentation(info = "<html>
<p>
This is a flange for 1D translational mechanical systems. In the cut plane of
the flange a unit vector n, called flange axis, is defined which is directed
INTO the cut plane, i. e. from left to right. All vectors in the cut plane are
resolved with respect to
this unit vector. E.g. force f characterizes a vector which is directed in
the direction of n with value equal to f. When this flange is connected to
other 1D translational flanges, this means that the axes vectors of the connected
flanges are identical.
</p>
<p>
The following variables are transported through this connector:
</p>
<pre>
  s: Absolute position of the flange in [m]. A positive translation
     means that the flange is translated along the flange axis.
  f: Cut-force in direction of the flange axis in [N].
</pre>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, -100}, {100, 100}}, lineColor = {0, 127, 0}, fillColor = {0, 127, 0}, fillPattern = FillPattern.Solid)}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-40, -40}, {40, 40}}, lineColor = {0, 127, 0}, fillColor = {0, 127, 0}, fillPattern = FillPattern.Solid), Text(extent = {{-160, 110}, {40, 50}}, lineColor = {0, 127, 0}, textString = "%name")}));
        end Flange_a;

        connector Flange_b "(right) 1D translational flange (flange axis directed OUT OF cut plane)"
          SI.Position s "Absolute position of flange";
          flow SI.Force f "Cut force directed into flange";
          annotation(
            defaultComponentName = "flange_b",
            Documentation(info = "<html>
<p>
This is a flange for 1D translational mechanical systems. In the cut plane of
the flange a unit vector n, called flange axis, is defined which is directed
OUT OF the cut plane. All vectors in the cut plane are resolved with respect to
this unit vector. E.g. force f characterizes a vector which is directed in
the direction of n with value equal to f. When this flange is connected to
other 1D translational flanges, this means that the axes vectors of the connected
flanges are identical.
</p>
<p>
The following variables are transported through this connector:
<pre>
  s: Absolute position of the flange in [m]. A positive translation
     means that the flange is translated along the flange axis.
  f: Cut-force in direction of the flange axis in [N].
</pre>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, -100}, {100, 100}}, lineColor = {0, 127, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid)}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-40, -40}, {40, 40}}, lineColor = {0, 127, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-40, 110}, {160, 50}}, lineColor = {0, 127, 0}, textString = "%name")}));
        end Flange_b;

        connector Support "Support/housing 1D translational flange"
          SI.Position s "Absolute position of flange";
          flow SI.Force f "Cut force directed into flange";
          annotation(
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-60, 60}, {60, -60}}, fillColor = {175, 175, 175}, fillPattern = FillPattern.Solid, pattern = LinePattern.None), Text(extent = {{-160, 110}, {40, 50}}, lineColor = {0, 127, 0}, textString = "%name"), Rectangle(extent = {{-40, -40}, {40, 40}}, lineColor = {0, 127, 0}, fillColor = {0, 127, 0}, fillPattern = FillPattern.Solid)}),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-90, -90}, {90, 90}}, lineColor = {0, 127, 0}, fillColor = {175, 175, 175}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-150, 150}, {150, -150}}, fillColor = {175, 175, 175}, fillPattern = FillPattern.Solid, pattern = LinePattern.None), Rectangle(extent = {{-90, -90}, {90, 90}}, lineColor = {0, 127, 0}, fillColor = {0, 127, 0}, fillPattern = FillPattern.Solid)}),
            Documentation(info = "<html>
<p>This is a connector for 1-dim. rotational mechanical systems and models the support or housing of a shaft. The following variables are defined in this connector:</p>
<table cellspacing=\"0\" cellpadding=\"2\" border=\"1\"><tr>
<td valign=\"top\"><h4>s</h4></td>
<td valign=\"top\"><p>Absolute position of the support/housing in [m]</p></td>
</tr>
<tr>
<td valign=\"top\"><h4>f</h4></td>
<td valign=\"top\"><p>Reaction force in the support/housing in [N]</p></td>
</tr>
</table>
<p><br/>The support connector is usually defined as conditional connector. It is most convenient to utilize it</p>
<ul>
<li>For models to be build graphically (i.e., the model is build up by drag-and-drop from elementary components):<br/><a href=\"modelica://Modelica.Mechanics.Translational.Interfaces.PartialOneFlangeAndSupport\">PartialOneFlangeAndSupport</a>,<br/><a href=\"modelica://Modelica.Mechanics.Translational.Interfaces.PartialTwoFlangesAndSupport\">PartialTwoFlangesAndSupport</a>, <br/>&nbsp; </li>
<li>For models to be build textually (i.e., elementary models):<br/><a href=\"modelica://Modelica.Mechanics.Translational.Interfaces.PartialElementaryOneFlangeAndSupport\">PartialElementaryOneFlangeAndSupport</a>,<br/><a href=\"modelica://Modelica.Mechanics.Translational.Interfaces.PartialElementaryTwoFlangesAndSupport\">PartialElementaryTwoFlangesAndSupport</a>,<br/><a href=\"modelica://Modelica.Mechanics.Translational.Interfaces.PartialElementaryRotationalToTranslational\">PartialElementaryRotationalToTranslational</a>. </li>
</ul>
</html>"));
        end Support;

        model InternalSupport "Adapter model to utilize conditional support connector"
          input SI.Force f "External support force (must be computed via force balance in model where InternalSupport is used; = flange.f)";
          SI.Position s "External support position (= flange.s)";
          Flange_a flange "Internal support flange (must be connected to the conditional support connector for useSupport=true and to conditional fixed model for useSupport=false)" annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}})));
        equation
          flange.f = f;
          flange.s = s;
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-200, 80}, {200, 40}}, lineColor = {0, 0, 255}, textString = "%name"), Rectangle(extent = {{-20, 20}, {20, -20}}, lineColor = {0, 127, 0}, fillColor = {175, 175, 175}, fillPattern = FillPattern.Solid)}),
            Documentation(info = "<html>
<p>
This is an adapter model to utilize a conditional support connector
in an elementary component, i.e., where the component equations are
defined textually:
</p>

<ul>
<li> If <i>useSupport = true</i>, the flange has to be connected to the conditional
     support connector.</li>
<li> If <i>useSupport = false</i>, the flange has to be connected to the conditional
     fixed model.</li>
</ul>

<p>
Variable <b>f</b> is defined as <b>input</b> and must be provided when using
this component as a modifier (computed via a force balance in
the model where InternalSupport is used). Usually, model InternalSupport is
utilized via the partial models:
</p>

<blockquote>
<a href=\"modelica://Modelica.Mechanics.Translational.Interfaces.PartialElementaryOneFlangeAndSupport\">
PartialElementaryOneFlangeAndSupport</a>,<br>
<a href=\"modelica://Modelica.Mechanics.Translational.Interfaces.PartialElementaryTwoFlangesAndSupport\">
PartialElementaryTwoFlangesAndSupport</a>,<br>
<a href=\"modelica://Modelica.Mechanics.Translational.Interfaces.PartialElementaryRotationalToTranslational\">
PartialElementaryRotationalToTranslational</a>.
</blockquote>

<p>
Note, the support position can always be accessed as internalSupport.s, and
the support force can always be accessed as internalSupport.f.
</p>
</html>"));
        end InternalSupport;

        partial model PartialTwoFlanges "Component with two translational 1D flanges"
          Flange_a flange_a "(left) driving flange (flange axis directed in to cut plane, e. g. from left to right)" annotation(
            Placement(transformation(extent = {{-110, -10}, {-90, 10}}, rotation = 0)));
          Flange_b flange_b "(right) driven flange (flange axis directed out of cut plane)" annotation(
            Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
          annotation(
            Documentation(info = "<html>
<p>
This is a 1D translational component with two flanges.
It is used e.g., to built up parts of a drive train consisting
of several base components.
</p>
</html>"));
        end PartialTwoFlanges;

        partial model PartialOneFlangeAndSupport "Partial model for a component with one translational 1-dim. shaft flange and a support used for graphical modeling, i.e., the model is build up by drag-and-drop from elementary components"
          parameter Boolean useSupport = false "= true, if support flange enabled, otherwise implicitly grounded" annotation(
            Evaluate = true,
            HideResult = true,
            choices(checkBox = true));
          Flange_b flange "Flange of component" annotation(
            Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
          Support support if useSupport "Support/housing of component" annotation(
            Placement(transformation(extent = {{-10, -110}, {10, -90}})));
        protected
          Support internalSupport "Internal support/housing of component (either connected to support, if useSupport=true, or connected to fixed, if useSupport=false)" annotation(
            Placement(transformation(extent = {{-3, -83}, {3, -77}})));
          Components.Fixed fixed if not useSupport "Fixed support/housing, if not useSupport" annotation(
            Placement(transformation(extent = {{10, -94}, {30, -74}})));
        equation
          connect(fixed.flange, internalSupport) annotation(
            Line(points = {{20, -84}, {20, -80}, {0, -80}}, color = {0, 127, 0}, smooth = Smooth.None));
          connect(internalSupport, support) annotation(
            Line(points = {{0, -80}, {0, -100}}, smooth = Smooth.None));
          annotation(
            Documentation(info = "<html>
<p>
This is a 1-dim. translational component with one flange and a support/housing.
It is used e.g., to build up parts of a drive train graphically consisting
of several components.
</p>

<p>
If <i>useSupport=true</i>, the support connector is conditionally enabled
and needs to be connected.<br>
If <i>useSupport=false</i>, the support connector is conditionally disabled
and instead the component is internally fixed to ground.
</p>

</html>"),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-38, -98}, {-6, -96}}, lineColor = {95, 95, 95}, textString = "(if useSupport)"), Text(extent = {{21, -95}, {61, -96}}, lineColor = {95, 95, 95}, textString = "(if not useSupport)")}),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(visible = not useSupport, points = {{-50, -120}, {-30, -100}}, color = {0, 0, 0}), Line(visible = not useSupport, points = {{-30, -120}, {-10, -100}}, color = {0, 0, 0}), Line(visible = not useSupport, points = {{-10, -120}, {10, -100}}, color = {0, 0, 0}), Line(visible = not useSupport, points = {{10, -120}, {30, -100}}, color = {0, 0, 0}), Line(visible = not useSupport, points = {{-30, -100}, {30, -100}}, color = {0, 0, 0})}));
        end PartialOneFlangeAndSupport;

        partial model PartialTwoFlangesAndSupport "Partial model for a component with two translational 1-dim. shaft flanges and a support used for graphical modeling, i.e., the model is build up by drag-and-drop from elementary components"
          parameter Boolean useSupport = false "= true, if support flange enabled, otherwise implicitly grounded" annotation(
            Evaluate = true,
            HideResult = true,
            choices(checkBox = true));
          Flange_a flange_a "Flange of left end" annotation(
            Placement(transformation(extent = {{-110, -10}, {-90, 10}}, rotation = 0)));
          Flange_b flange_b "Flange of right end" annotation(
            Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
          Support support if useSupport "Support/housing of component" annotation(
            Placement(transformation(extent = {{-10, -110}, {10, -90}})));
        protected
          Support internalSupport "Internal support/housing of component (either connected to support, if useSupport=true, or connected to fixed, if useSupport=false)" annotation(
            Placement(transformation(extent = {{-3, -83}, {3, -77}})));
          Components.Fixed fixed if not useSupport "Fixed support/housing, if not useSupport" annotation(
            Placement(transformation(extent = {{10, -97}, {30, -77}})));
        equation
          connect(fixed.flange, internalSupport) annotation(
            Line(points = {{20, -87}, {20, -80}, {0, -80}}, color = {0, 127, 0}, smooth = Smooth.None));
          connect(internalSupport, support) annotation(
            Line(points = {{0, -80}, {0, -100}}, smooth = Smooth.None));
          annotation(
            Documentation(info = "<html>
<p>
This is a 1-dim. translational component with two flanges and a support/housing.
It is used e.g., to build up parts of a drive train graphically consisting
of several components.
</p>

<p>
If <i>useSupport=true</i>, the support connector is conditionally enabled
and needs to be connected.<br>
If <i>useSupport=false</i>, the support connector is conditionally disabled
and instead the component is internally fixed to ground.
</p>

</html>"),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-38, -98}, {-6, -96}}, lineColor = {95, 95, 95}, textString = "(if useSupport)"), Text(extent = {{24, -97}, {64, -98}}, lineColor = {95, 95, 95}, textString = "(if not useSupport)")}),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(visible = not useSupport, points = {{-50, -120}, {-30, -100}}, color = {0, 0, 0}), Line(visible = not useSupport, points = {{-30, -120}, {-10, -100}}, color = {0, 0, 0}), Line(visible = not useSupport, points = {{-10, -120}, {10, -100}}, color = {0, 0, 0}), Line(visible = not useSupport, points = {{10, -120}, {30, -100}}, color = {0, 0, 0}), Line(visible = not useSupport, points = {{-30, -100}, {30, -100}}, color = {0, 0, 0})}));
        end PartialTwoFlangesAndSupport;

        partial model PartialRigid "Rigid connection of two translational 1D flanges"
          SI.Position s "Absolute position of center of component (s = flange_a.s + L/2 = flange_b.s - L/2)";
          parameter SI.Length L(start = 0) "Length of component, from left flange to right flange (= flange_b.s - flange_a.s)";
          Flange_a flange_a "Left flange of translational component" annotation(
            Placement(transformation(extent = {{-110, -10}, {-90, 10}}, rotation = 0)));
          Flange_b flange_b "Right flange of translational component" annotation(
            Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
        equation
          flange_a.s = s - L / 2;
          flange_b.s = s + L / 2;
          annotation(
            Documentation(info = "<html>
<p>
This is a 1-dim. translational component with two <i>rigidly</i> connected flanges.
The fixed distance between the left and the right flange is defined by parameter \"L\".
The forces at the right and left flange can be different.
It is used e.g., to built up sliding masses.
</p>
</html>"));
        end PartialRigid;

        partial model PartialCompliant "Compliant connection of two translational 1D flanges"
          Flange_a flange_a "Left flange of compliant 1-dim. translational component" annotation(
            Placement(transformation(extent = {{-110, -10}, {-90, 10}}, rotation = 0)));
          Flange_b flange_b "Right flange of compliant 1-dim. translational component" annotation(
            Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
          SI.Position s_rel(start = 0) "Relative distance (= flange_b.s - flange_a.s)";
          SI.Force f "Force between flanges (positive in direction of flange axis R)";
        equation
          s_rel = flange_b.s - flange_a.s;
          flange_b.f = f;
          flange_a.f = -f;
          annotation(
            Documentation(info = "<html>
<p>
This is a 1D translational component with a <i>compliant </i>connection of two
translational 1D flanges where inertial effects between the two
flanges are not included. The absolute value of the force at the left and the right
flange is the same. It is used to built up springs, dampers etc.
</p>

</html>"),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{50, -90}, {20, -80}, {20, -100}, {50, -90}}, lineColor = {128, 128, 128}, fillColor = {128, 128, 128}, fillPattern = FillPattern.Solid), Line(points = {{-60, -90}, {20, -90}}, color = {0, 0, 0})}));
        end PartialCompliant;

        partial model PartialCompliantWithRelativeStates "Base model for the compliant connection of two translational 1-dim. shaft flanges where the relative position and relative velocities are used as states"
          parameter StateSelect stateSelect = StateSelect.prefer "Priority to use phi_rel and w_rel as states" annotation(
            HideResult = true,
            Dialog(tab = "Advanced"));
          parameter SI.Distance s_nominal = 1e-4 "Nominal value of s_rel (used for scaling)" annotation(
            Dialog(tab = "Advanced"));
          SI.Position s_rel(start = 0, stateSelect = stateSelect, nominal = s_nominal) "Relative distance (= flange_b.s - flange_a.s)";
          SI.Velocity v_rel(start = 0, stateSelect = stateSelect) "Relative velocity (= der(s_rel))";
          SI.Force f "Forces between flanges (= flange_b.f)";
          Translational.Interfaces.Flange_a flange_a "Left flange of compliant 1-dim. translational component" annotation(
            Placement(transformation(extent = {{-110, -10}, {-90, 10}}, rotation = 0)));
          Translational.Interfaces.Flange_b flange_b "Right flange of compliant 1-dim. translational component" annotation(
            Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
        equation
          s_rel = flange_b.s - flange_a.s;
          v_rel = der(s_rel);
          flange_b.f = f;
          flange_a.f = -f;
          annotation(
            Documentation(info = "<html>
<p>
This is a 1-dim. translational component with a compliant connection of two
translational 1-dim. flanges where inertial effects between the two
flanges are neglected. The basic assumption is that the cut-forces
of the two flanges sum-up to zero, i.e., they have the same absolute value
but opposite sign: flange_a.f + flange_b.f = 0. This base class
is used to built up force elements such as springs, dampers, friction.
</p>

<p>
The difference to base class \"PartialCompliant\" is that the relative
distance and the relative velocity are defined as preferred states.
The reason is that for a large class of drive trains,
the absolute position is quickly increasing during operation.
Numerically, it is better to use relative distances between drive train components
because they remain in a limited size. For this reason, StateSelect.prefer
is set for the relative distance of this component.
</p>

<p>
In order to improve the numerics, a nominal value for the relative distance
should be set, since drive train distances are in a small order and
then step size control of the integrator is practically switched off for
such a variable. A default nominal value of s_nominal = 1e-4 is defined.
This nominal value might also be computed from other values, such
as \"s_nominal = f_nominal / c\" for a spring, if f_nominal
and c have more meaningful values for the user.
</p>

</html>"));
        end PartialCompliantWithRelativeStates;

        partial model PartialElementaryOneFlangeAndSupport "Obsolete partial model. Use PartialElementaryOneFlangeAndSupport2."
          extends Modelica.Icons.ObsoleteModel;
          parameter Boolean useSupport = false "= true, if support flange enabled, otherwise implicitly grounded" annotation(
            Evaluate = true,
            HideResult = true,
            choices(checkBox = true));
          Modelica.SIunits.Length s "Distance between flange and support (= flange.s - support.s)";
          Flange_b flange "Flange of component" annotation(
            Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
        protected
          InternalSupport internalSupport(f = -flange.f) "Internal support/housing of component as a model with connector flange (flange is either connected to support, if useSupport=true, or connected to fixed, if useSupport=false)" annotation(
            Placement(transformation(extent = {{-10, -90}, {10, -70}})));
          Components.Fixed fixed if not useSupport "Fixed support/housing, if not useSupport" annotation(
            Placement(transformation(extent = {{10, -97}, {30, -77}})));
        public
          Support support if useSupport "Support/housing of component" annotation(
            Placement(transformation(extent = {{-10, -110}, {10, -90}})));
        equation
          s = flange.s - internalSupport.s;
          connect(internalSupport.flange, support) annotation(
            Line(points = {{0, -80}, {0, -100}}, color = {0, 127, 0}, smooth = Smooth.None));
          connect(fixed.flange, internalSupport.flange) annotation(
            Line(points = {{20, -87}, {20, -80}, {0, -80}}, color = {0, 127, 0}, smooth = Smooth.None));
          annotation(
            Documentation(info = "<html>
<p>
This is a 1-dim. translational component with one flange and a support/housing.
It is used to build up elementary components of a drive train with
equations in the text layer.
</p>

<p>
If <i>useSupport=true</i>, the support connector is conditionally enabled
and needs to be connected.<br>
If <i>useSupport=false</i>, the support connector is conditionally disabled
and instead the component is internally fixed to ground.
</p>

</html>"),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-38, -98}, {-6, -96}}, lineColor = {95, 95, 95}, textString = "(if useSupport)"), Text(extent = {{24, -97}, {64, -98}}, lineColor = {95, 95, 95}, textString = "(if not useSupport)")}),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(visible = not useSupport, points = {{-50, -120}, {-30, -100}}, color = {0, 0, 0}), Line(visible = not useSupport, points = {{-30, -120}, {-10, -100}}, color = {0, 0, 0}), Line(visible = not useSupport, points = {{-10, -120}, {10, -100}}, color = {0, 0, 0}), Line(visible = not useSupport, points = {{10, -120}, {30, -100}}, color = {0, 0, 0}), Line(visible = not useSupport, points = {{-30, -100}, {30, -100}}, color = {0, 0, 0})}));
        end PartialElementaryOneFlangeAndSupport;

        partial model PartialElementaryOneFlangeAndSupport2 "Partial model for a component with one translational 1-dim. shaft flange and a support used for textual modeling, i.e., for elementary models"
          parameter Boolean useSupport = false "= true, if support flange enabled, otherwise implicitly grounded" annotation(
            Evaluate = true,
            HideResult = true,
            choices(checkBox = true));
          Modelica.SIunits.Length s "Distance between flange and support (= flange.s - support.s)";
          Flange_b flange "Flange of component" annotation(
            Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
          Support support(s = s_support, f = -flange.f) if useSupport "Support/housing of component" annotation(
            Placement(transformation(extent = {{-10, -110}, {10, -90}})));
        protected
          Modelica.SIunits.Length s_support "Absolute position of support flange";
        equation
          s = flange.s - s_support;
          if not useSupport then
            s_support = 0;
          end if;
          annotation(
            Documentation(info = "<html>
<p>
This is a 1-dim. translational component with one flange and a support/housing.
It is used to build up elementary components of a drive train with
equations in the text layer.
</p>

<p>
If <i>useSupport=true</i>, the support connector is conditionally enabled
and needs to be connected.<br>
If <i>useSupport=false</i>, the support connector is conditionally disabled
and instead the component is internally fixed to ground.
</p>

</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(visible = not useSupport, points = {{-50, -120}, {-30, -100}}, color = {0, 0, 0}), Line(visible = not useSupport, points = {{-30, -120}, {-10, -100}}, color = {0, 0, 0}), Line(visible = not useSupport, points = {{-10, -120}, {10, -100}}, color = {0, 0, 0}), Line(visible = not useSupport, points = {{10, -120}, {30, -100}}, color = {0, 0, 0}), Line(visible = not useSupport, points = {{-30, -100}, {30, -100}}, color = {0, 0, 0})}));
        end PartialElementaryOneFlangeAndSupport2;

        partial model PartialElementaryTwoFlangesAndSupport "Obsolete partial model. Use PartialElementaryTwoFlangesAndSupport2."
          extends Modelica.Icons.ObsoleteModel;
          parameter Boolean useSupport = false "= true, if support flange enabled, otherwise implicitly grounded" annotation(
            Evaluate = true,
            HideResult = true,
            choices(checkBox = true));
          Flange_a flange_a "Flange of left shaft" annotation(
            Placement(transformation(extent = {{-110, -10}, {-90, 10}}, rotation = 0)));
          Flange_b flange_b "Flange of right shaft" annotation(
            Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
          Modelica.SIunits.Length s_a "Distance between left flange and support";
          Modelica.SIunits.Length s_b "Distance between right flange and support";
        protected
          InternalSupport internalSupport(f = (-flange_a.f) - flange_b.f) "Internal support/housing of component as a model with connector flange (flange is either connected to support, if useSupport=true, or connected to fixed, if useSupport=false)" annotation(
            Placement(transformation(extent = {{-10, -90}, {10, -70}})));
          Components.Fixed fixed if not useSupport "Fixed support/housing, if not useSupport" annotation(
            Placement(transformation(extent = {{10, -97}, {30, -77}})));
        public
          Support support if useSupport "Support/housing of component" annotation(
            Placement(transformation(extent = {{-10, -110}, {10, -90}})));
        equation
          s_a = flange_a.s - internalSupport.s;
          s_b = flange_b.s - internalSupport.s;
          connect(internalSupport.flange, support) annotation(
            Line(points = {{0, -80}, {0, -100}}, color = {0, 127, 0}, smooth = Smooth.None));
          connect(fixed.flange, internalSupport.flange) annotation(
            Line(points = {{20, -87}, {20, -80}, {0, -80}}, color = {0, 127, 0}, smooth = Smooth.None));
          annotation(
            Documentation(info = "<html>
<p>
This is a 1-dim. translational component with two flanges and an additional support.
It is used e.g., to build up elementary ideal gear components. The component
contains the force balance, i.e., the sum of the forces of the connectors
is zero (therefore, components that are based on PartialGear cannot have
a mass). The support connector needs to be connected
to avoid the unphysical behavior that the
support force is required to be zero (= the default value, if the
connector is not connected).
</p>

</html>"),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-38, -98}, {-6, -96}}, lineColor = {95, 95, 95}, textString = "(if useSupport)"), Text(extent = {{24, -97}, {64, -98}}, lineColor = {95, 95, 95}, textString = "(if not useSupport)")}),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(visible = not useSupport, points = {{-50, -120}, {-30, -100}}, color = {0, 0, 0}), Line(visible = not useSupport, points = {{-30, -120}, {-10, -100}}, color = {0, 0, 0}), Line(visible = not useSupport, points = {{-10, -120}, {10, -100}}, color = {0, 0, 0}), Line(visible = not useSupport, points = {{10, -120}, {30, -100}}, color = {0, 0, 0}), Line(visible = not useSupport, points = {{-30, -100}, {30, -100}}, color = {0, 0, 0})}));
        end PartialElementaryTwoFlangesAndSupport;

        partial model PartialElementaryTwoFlangesAndSupport2 "Partial model for a component with one translational 1-dim. shaft flange and a support used for textual modeling, i.e., for elementary models"
          parameter Boolean useSupport = false "= true, if support flange enabled, otherwise implicitly grounded" annotation(
            Evaluate = true,
            HideResult = true,
            choices(checkBox = true));
          Flange_a flange_a "Flange of left shaft" annotation(
            Placement(transformation(extent = {{-110, -10}, {-90, 10}}, rotation = 0)));
          Flange_b flange_b "Flange of right shaft" annotation(
            Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
          Support support(s = s_support, f = (-flange_a.f) - flange_b.f) if useSupport "Support/housing of component" annotation(
            Placement(transformation(extent = {{-10, -110}, {10, -90}})));
          Modelica.SIunits.Length s_a "Distance between left flange and support";
          Modelica.SIunits.Length s_b "Distance between right flange and support";
        protected
          Modelica.SIunits.Length s_support "Absolute position of support flange";
        equation
          s_a = flange_a.s - s_support;
          s_b = flange_b.s - s_support;
          if not useSupport then
            s_support = 0;
          end if;
          annotation(
            Documentation(info = "<html>
<p>
This is a 1-dim. translational component with two flanges and an additional support.
It is used e.g., to build up elementary ideal gear components. The component
contains the force balance, i.e., the sum of the forces of the connectors
is zero (therefore, components that are based on PartialGear cannot have
a mass). The support connector needs to be connected
to avoid the unphysical behavior that the
support force is required to be zero (= the default value, if the
connector is not connected).
</p>

</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(visible = not useSupport, points = {{-50, -120}, {-30, -100}}, color = {0, 0, 0}), Line(visible = not useSupport, points = {{-30, -120}, {-10, -100}}, color = {0, 0, 0}), Line(visible = not useSupport, points = {{-10, -120}, {10, -100}}, color = {0, 0, 0}), Line(visible = not useSupport, points = {{10, -120}, {30, -100}}, color = {0, 0, 0}), Line(visible = not useSupport, points = {{-30, -100}, {30, -100}}, color = {0, 0, 0})}));
        end PartialElementaryTwoFlangesAndSupport2;

        partial model PartialElementaryRotationalToTranslational "Partial model to transform rotational into translational motion"
          extends Modelica.Mechanics.Rotational.Interfaces.PartialElementaryRotationalToTranslational;
          annotation(
            Documentation(info = "<html>
<p>This is a 1-dim. rotational component with</p>
<ul>
<li>one rotational flange, </li>
<li>one rotational support/housing, </li>
<li>one translational flange, and </li>
<li>one translational support/housing </li>
</ul>
<p>This model is used to build up elementary components of a drive train transforming rotational into translational motion with equations in the text layer.</p>
<p>If <i>useSupportR=true</i>, the rotational support connector is conditionally enabled and needs to be connected.</p>
<p>If <i>useSupportR=false</i>, the rotational support connector is conditionally disabled and instead the rotational part is internally fixed to ground.</p>
<p>If <i>useSupportT=true</i>, the translational support connector is conditionally enabled and needs to be connected.</p>
<p>If <i>useSupportT=false</i>, the translational support connector is conditionally disabled and instead the translational part is internally fixed to ground.</p>
</html>"));
        end PartialElementaryRotationalToTranslational;

        partial model PartialForce "Partial model of a force acting at the flange (accelerates the flange)"
          extends PartialElementaryOneFlangeAndSupport2;
          Modelica.SIunits.Force f "Accelerating force acting at flange (= flange.f)";
        equation
          f = flange.f;
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-96, 96}, {96, -96}}, lineColor = {255, 255, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{0, -60}, {0, -100}}, color = {0, 0, 0}), Text(extent = {{-150, 140}, {150, 100}}, lineColor = {0, 0, 255}, textString = "%name"), Line(points = {{-78, 80}, {51, 80}}, color = {0, 0, 0}), Polygon(points = {{81, 80}, {51, 90}, {51, 70}, {81, 80}}, lineColor = {0, 0, 0}, fillColor = {128, 128, 128}, fillPattern = FillPattern.Solid), Line(points = {{-52, -60}, {77, -60}}, color = {0, 0, 0}), Polygon(points = {{-82, -60}, {-51, -50}, {-51, -70}, {-82, -60}}, lineColor = {0, 0, 0}, fillColor = {128, 128, 128}, fillPattern = FillPattern.Solid), Line(visible = not useSupport, points = {{-50, -120}, {-30, -100}}, color = {0, 0, 0}), Line(visible = not useSupport, points = {{-30, -120}, {-10, -100}}, color = {0, 0, 0}), Line(visible = not useSupport, points = {{-10, -120}, {10, -100}}, color = {0, 0, 0}), Line(visible = not useSupport, points = {{10, -120}, {30, -100}}, color = {0, 0, 0}), Line(visible = not useSupport, points = {{-30, -100}, {30, -100}}, color = {0, 0, 0})}),
            Documentation(info = "<HTML>
<p>
Partial model of force that accelerates the flange.
</p>

<p>
If <i>useSupport=true</i>, the support connector is conditionally enabled
and needs to be connected.<br>
If <i>useSupport=false</i>, the support connector is conditionally disabled
and instead the component is internally fixed to ground.
</p>
</HTML>"));
        end PartialForce;

        partial model PartialAbsoluteSensor "Device to measure a single absolute flange variable"
          extends Modelica.Icons.TranslationalSensor;
          Interfaces.Flange_a flange "Flange to be measured (flange axis directed in to cut plane, e. g. from left to right)" annotation(
            Placement(transformation(extent = {{-110, -10}, {-90, 10}}, rotation = 0)));
        equation
          0 = flange.f;
          annotation(
            Documentation(info = "<html>
<p>
This is the superclass of a 1D translational component with one flange and one
output signal in order to measure an absolute kinematic quantity in the flange
and to provide the measured signal as output signal for further processing
with the Modelica.Blocks blocks.
</p>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-100, -90}, {-20, -90}}, color = {0, 0, 0}), Polygon(points = {{10, -90}, {-20, -80}, {-20, -100}, {10, -90}}, lineColor = {128, 128, 128}, fillColor = {128, 128, 128}, fillPattern = FillPattern.Solid), Line(points = {{-70, 0}, {-90, 0}}, color = {0, 0, 0}), Line(points = {{70, 0}, {100, 0}}, color = {0, 0, 127}), Text(extent = {{-150, 80}, {150, 40}}, textString = "%name", lineColor = {0, 0, 255})}));
        end PartialAbsoluteSensor;

        partial model PartialRelativeSensor "Device to measure a single relative variable between two flanges"
          extends Modelica.Icons.TranslationalSensor;
          Interfaces.Flange_a flange_a "(left) driving flange (flange axis directed in to cut plane, e. g. from left to right)" annotation(
            Placement(transformation(extent = {{-110, -10}, {-90, 10}}, rotation = 0)));
          Interfaces.Flange_b flange_b "(right) driven flange (flange axis directed out of cut plane)" annotation(
            Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
        equation
          0 = flange_a.f + flange_b.f;
          annotation(
            Documentation(info = "<html>
<p>
This is a superclass for 1D translational components with two rigidly connected
flanges and one output signal in order to measure relative kinematic quantities
between the two flanges or the cut-force in the flange and
to provide the measured signal as output signal for further processing
with the Modelica.Blocks blocks.
</p>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-51, 34}, {29, 34}}, color = {0, 0, 0}), Polygon(points = {{59, 34}, {29, 44}, {29, 24}, {59, 34}}, lineColor = {0, 0, 0}, fillColor = {128, 128, 128}, fillPattern = FillPattern.Solid), Line(points = {{-70, 0}, {-90, 0}}, color = {0, 0, 0}), Line(points = {{70, 0}, {90, 0}}, color = {0, 0, 0}), Text(extent = {{-150, 100}, {150, 60}}, textString = "%name", lineColor = {0, 0, 255})}));
        end PartialRelativeSensor;

        partial model PartialFriction "Base model of Coulomb friction elements"
          //extends Translational.Interfaces.PartialRigid;
          parameter SI.Velocity v_small = 1e-3 "Relative velocity near to zero (see model info text)" annotation(
            Dialog(tab = "Advanced"));
          // Equations to define the following variables have to be defined in subclasses
          SI.Velocity v_relfric "Relative velocity between frictional surfaces";
          SI.Acceleration a_relfric "Relative acceleration between frictional surfaces";
          //SI.Force f "Friction force (positive, if directed in opposite direction of v_rel)";
          SI.Force f0 "Friction force for v=0 and forward sliding";
          SI.Force f0_max "Maximum friction force for v=0 and locked";
          Boolean free "true, if frictional element is not active";
          // Equations to define the following variables are given in this class
          Real sa(unit = "1") "Path parameter of friction characteristic f = f(a_relfric)";
          Boolean startForward(start = false, fixed = true) "true, if v_rel=0 and start of forward sliding";
          Boolean startBackward(start = false, fixed = true) "true, if v_rel=0 and start of backward sliding";
          Boolean locked(start = false) "true, if v_rel=0 and not sliding";
          constant Integer Unknown = 3 "Value of mode is not known";
          constant Integer Free = 2 "Element is not active";
          constant Integer Forward = 1 "v_rel > 0 (forward sliding)";
          constant Integer Stuck = 0 "v_rel = 0 (forward sliding, locked or backward sliding)";
          constant Integer Backward = -1 "v_rel < 0 (backward sliding)";
          Integer mode(final min = Backward, final max = Unknown, start = Unknown, fixed = true);
        protected
          constant SI.Acceleration unitAcceleration = 1 annotation(
            HideResult = true);
          constant SI.Force unitForce = 1 annotation(
            HideResult = true);
        equation
/* Friction characteristic
   (locked is introduced to help the Modelica translator determining
   the different structural configurations,
   if for each configuration special code shall be generated)
*/
          startForward = pre(mode) == Stuck and (sa > f0_max / unitForce or pre(startForward) and sa > f0 / unitForce) or pre(mode) == Backward and v_relfric > v_small or initial() and v_relfric > 0;
          startBackward = pre(mode) == Stuck and (sa < (-f0_max / unitForce) or pre(startBackward) and sa < (-f0 / unitForce)) or pre(mode) == Forward and v_relfric < (-v_small) or initial() and v_relfric < 0;
          locked = not free and not (pre(mode) == Forward or startForward or pre(mode) == Backward or startBackward);
          a_relfric / unitAcceleration = if locked then 0 else if free then sa else if startForward then sa - f0_max / unitForce else if startBackward then sa + f0_max / unitForce else if pre(mode) == Forward then sa - f0_max / unitForce else sa + f0_max / unitForce;
/* Friction torque has to be defined in a subclass. Example for a clutch:
   f = if locked then sa else
       if free then   0 else
       cgeo*fn*(if startForward then          Math.tempInterpol1( v_relfric, mue_pos, 2) else
                if startBackward then        -Math.tempInterpol1(-v_relfric, mue_pos, 2) else
                if pre(mode) == Forward then  Math.tempInterpol1( v_relfric, mue_pos, 2) else
                                             -Math.tempInterpol1(-v_relfric, mue_pos, 2));
*/
// finite state machine to determine configuration
          mode = if free then Free else if (pre(mode) == Forward or pre(mode) == Free or startForward) and v_relfric > 0 then Forward else if (pre(mode) == Backward or pre(mode) == Free or startBackward) and v_relfric < 0 then Backward else Stuck;
          annotation(
            Documentation(info = "<html>
<p>
Basic model for Coulomb friction that models the stuck phase in a reliable way.
</p>
</html>"));
        end PartialFriction;
        annotation(
          Documentation(info = "<html>
<p>
This package contains connectors and partial models for 1-dim.
translational mechanical components. The components of this package can
only be used as basic building elements for models.
</p>

</html>"));
      end Interfaces;
      annotation(
        Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(origin = {14, 53}, points = {{-84, -73}, {66, -73}}), Rectangle(origin = {14, 53}, lineColor = {64, 64, 64}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Sphere, extent = {{-81, -65}, {-8, -22}}), Line(visible = true, origin = {14, 53}, points = {{-8, -43}, {-1, -43}, {6, -64}, {17, -23}, {29, -65}, {40, -23}, {50, -44}, {61, -44}}), Line(origin = {14, 53}, points = {{-59, -73}, {-84, -93}}), Line(origin = {14, 53}, points = {{-11, -73}, {-36, -93}}), Line(origin = {14, 53}, points = {{-34, -73}, {-59, -93}}), Line(origin = {14, 53}, points = {{14, -73}, {-11, -93}}), Line(origin = {14, 53}, points = {{39, -73}, {14, -93}}), Line(origin = {14, 53}, points = {{63, -73}, {38, -93}})}),
        Documentation(info = "<html>
<p>
This package contains components to model <i>1-dimensional translational
mechanical</i> systems.
</p>
<p>
The <i>filled</i> and <i>non-filled green squares</i> at the left and
right side of a component represent <i>mechanical flanges</i>.
Drawing a line between such squares means that the corresponding
flanges are <i>rigidly attached</i> to each other. The components of this
library can be usually connected together in an arbitrary way. E.g. it is
possible to connect two springs or two sliding masses with inertia directly
together.
<p> The only <i>connection restriction</i> is that the Coulomb friction
elements (e.g., MassWithStopAndFriction) should be only connected
together provided a compliant element, such as a spring, is in between.
The reason is that otherwise the frictional force is not uniquely
defined if the elements are stuck at the same time instant (i.e., there
does not exist a unique solution) and some simulation systems may not be
able to handle this situation, since this leads to a singularity during
simulation. It can only be resolved in a \"clean way\" by combining the
two connected friction elements into
one component and resolving the ambiguity of the frictional force in the
stuck mode.
</p>
<p> Another restriction arises if the hard stops in model MassWithStopAndFriction are used, i. e.
the movement of the mass is limited by a stop at smax or smin.
<font color=\"#ff0000\"> <b>This requires the states Stop.s and Stop.v</b> </font>. If these states are eliminated during the index reduction
the model will not work. To avoid this any inertias should be connected via springs
to the Stop element, other sliding masses, dampers or hydraulic chambers must be avoided.</p>
<p>
In the <i>icon</i> of every component an <i>arrow</i> is displayed in grey
color. This arrow characterizes the coordinate system in which the vectors
of the component are resolved. It is directed into the positive
translational direction (in the mathematical sense).
In the flanges of a component, a coordinate system is rigidly attached
to the flange. It is called <i>flange frame</i> and is directed in parallel
to the component coordinate system. As a result, e.g., the positive
cut-force of a \"left\" flange (flange_a) is directed into the flange, whereas
the positive cut-force of a \"right\" flange (flange_b) is directed out of the
flange. A flange is described by a Modelica connector containing
the following variables:
</p>
<pre>
   Modelica.SIunits.Position s    \"Absolute position of flange\";
   <b>flow</b> Modelica.SIunits.Force f  \"Cut-force in the flange\";
</pre>

<p>
This library is designed in a fully object oriented way in order that
components can be connected together in every meaningful combination
(e.g., direct connection of two springs or two shafts with inertia).
As a consequence, most models lead to a system of
differential-algebraic equations of <i>index 3</i> (= constraint
equations have to be differentiated twice in order to arrive at
a state space representation) and the Modelica translator or
the simulator has to cope with this system representation.
According to our present knowledge, this requires that the
Modelica translator is able to symbolically differentiate equations
(otherwise it is e.g., not possible to provide consistent initial
conditions; even if consistent initial conditions are present, most
numerical DAE integrators can cope at most with index 2 DAEs).
</p>

<p>
In version 3.2 of the Modelica Standard Library, all <b>dissipative</b> components
of the Translational library got an optional <b>heatPort</b> connector to which the
dissipated energy is transported in form of heat. This connector is enabled
via parameter \"useHeatPort\". If the heatPort connector is enabled,
it must be connected, and if it is not enabled, it must not be connected.
Independently, whether the heatPort is enabled or not,
the dissipated power is available from the new variable \"<b>lossPower</b>\" (which is
positive if heat is flowing out of the heatPort). For an example, see
<a href=\"modelica://Modelica.Mechanics.Translational.Examples.HeatLosses\">Examples.HeatLosses</a>.
</p>

<dl>
<dt><b>Library Officer</b>
<dd><a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a> <br>
    Deutsches Zentrum f&uuml;r Luft und Raumfahrt e.V. (DLR)<br>
    Institut f&uuml;r Robotik und Mechatronik (DLR-RM)<br>
    Abteilung Systemdynamik und Regelungstechnik<br>
    Postfach 1116<br>
    D-82230 Wessling<br>
    Germany<br>
    email: <A HREF=\"mailto:Martin.Otter@dlr.de\">Martin.Otter@dlr.de</A><br><br>
</dl>

<p>
<b>Contributors to this library:</b>
</p>

<ul>
<li> Main author until 2006:<br>
     Peter Beater <br>
     Universit&auml;t Paderborn, Abteilung Soest<br>
     Fachbereich Maschinenbau/Automatisierungstechnik<br>
     L&uuml;becker Ring 2 <br>
     D 59494 Soest <br>
     Germany <br>
     email: <A HREF=\"mailto:info@beater.de\">info@beater.de</A><br><br>
     </li>

<li> <a href=\"http://www.haumer.at/\">Anton Haumer</a><br>
     Technical Consulting &amp; Electrical Engineering<br>
     A-3423 St.Andrae-Woerdern, Austria<br>
     email: <a href=\"mailto:a.haumer@haumer.at\">a.haumer@haumer.at</a><br><br></li>

<li> <a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a> (DLR-RM)</li>
</ul>

<p>
Copyright &copy; 1998-2015, Modelica Association, Anton Haumer and Universit&auml;t Paderborn, FB 12.
</p>
<p>
<i>This Modelica package is <u>free</u> software and the use is completely at <u>your own risk</u>; it can be redistributed and/or modified under the terms of the Modelica License 2. For license conditions (including the disclaimer of warranty) see <a href=\"modelica://Modelica.UsersGuide.ModelicaLicense2\">Modelica.UsersGuide.ModelicaLicense2</a> or visit <a href=\"https://www.modelica.org/licenses/ModelicaLicense2\"> https://www.modelica.org/licenses/ModelicaLicense2</a>.</i>
</p>
</html>", revisions = "<html>
<ul>
<li><i>Version 1.2.0 2010-07-22</i>
       by Anton Haumer and Martin Otter<br>
       heatPort introduced for all dissipative elements, and
       text in icons improved.
       <br></li>

<li><i>Version 1.1.0 2007-11-16</i>
       by Anton Haumer<br>
       Redesign for Modelica 3.0-compliance<br>
       Added new components according to Mechanics.Rotational library
       <br></li>

<li><i>Version 1.01 (July 18, 2001)</i>
       by Peter Beater <br>
       Assert statement added to \"Stop\", small bug fixes in examples.
       <br></li>

<li><i>Version 1.0 (January 5, 2000)</i>
       by Peter Beater <br>
       Realized a first version based on Modelica library Mechanics.Rotational
       by Martin Otter and an existing Dymola library onedof.lib by Peter Beater.</li>
</ul>
</html>"));
    end Translational;
    annotation(
      Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100.0, -100.0}, {100.0, 100.0}}, initialScale = 0.1), graphics = {Rectangle(origin = {8.6, 63.3333}, lineColor = {64, 64, 64}, fillColor = {192, 192, 192}, fillPattern = FillPattern.HorizontalCylinder, extent = {{-4.6, -93.3333}, {41.4, -53.3333}}), Ellipse(origin = {9.0, 46.0}, extent = {{-90.0, -60.0}, {-80.0, -50.0}}), Line(origin = {9.0, 46.0}, points = {{-85.0, -55.0}, {-60.0, -21.0}}, thickness = 0.5), Ellipse(origin = {9.0, 46.0}, extent = {{-65.0, -26.0}, {-55.0, -16.0}}), Line(origin = {9.0, 46.0}, points = {{-60.0, -21.0}, {9.0, -55.0}}, thickness = 0.5), Ellipse(origin = {9.0, 46.0}, fillPattern = FillPattern.Solid, extent = {{4.0, -60.0}, {14.0, -50.0}}), Line(origin = {9.0, 46.0}, points = {{-10.0, -26.0}, {72.0, -26.0}, {72.0, -86.0}, {-10.0, -86.0}})}),
      Documentation(info = "<HTML>
<p>
This package contains components to model the movement
of 1-dim. rotational, 1-dim. translational, and
3-dim. <b>mechanical systems</b>.
</p>

<p>
Note, all <b>dissipative</b> components of the Modelica.Mechanics library have
an optional <b>heatPort</b> connector to which the
dissipated energy is transported in form of heat. This connector is enabled
via parameter \"useHeatPort\". If the heatPort connector is enabled,
it must be connected, and if it is not enabled, it must not be connected.
Independently, whether the heatPort is enabled or not,
the dissipated power is available from variable \"<b>lossPower</b>\" (which is
positive if heat is flowing out of the heatPort).
</p>
</html>"));
  end Mechanics;

  package Fluid "Library of 1-dim. thermo-fluid flow models using the Modelica.Media media description"
    extends Modelica.Icons.Package;
    import SI = Modelica.SIunits;
    import Cv = Modelica.SIunits.Conversions;

    package UsersGuide "User's Guide"
      extends Modelica.Icons.Information;

      class Overview "Overview"
        extends Modelica.Icons.Information;
        annotation(
          Documentation(info = "<html>
<p>
The Modelica.Fluid library provides basic interfaces and
components to model 1-dimensional thermo-fluid flow in networks of pipes.
It is not the intention that this library covers all
application cases because the fluid flow area is too large and
because for special applications it is possible to implement
libraries with simpler component interfaces.
Instead, the goal is that the Modelica.Fluid library provides
a <b>reasonable set of components</b> and that it <b>demonstrates</b>
how to implement components of a fluid flow library in Modelica,
in particular to cope with difficult issues such as connector
design, reversing flow and initialization. It is planned to
include more components in the future. User proposals are
welcome.
</p>
<p>
This library has the following main features:
</p>
<ul>
<li> The connectors Modelica.Fluid.Interfaces.FluidPort_a/_b are designed
     for one-dimensional flow of a <b>single substance</b>
     or of a <b>mixture of substances</b> with optional <b>multiple phases</b>.
     All media models from Modelica.Media can be utilized when
     connecting components. For one substance media, the additional arrays for
     multiple
     substance media have zero dimension and are therefore removed
     from the code during translation. The general connector definition
     therefore does not introduce an overhead for special cases.<br>&nbsp;</li>
<li> All the components of the Modelica.Fluid library are designed
     that they can be utilized for all media models from
     Modelica.Media if this is possible. For example, all media can
     be utilized for the Modelica.Fluid.Sensors/Sources components.
     For some components only special media are possible, since additional
     functionality is required. For example,
     Modelica.Fluid.Components.Evaporator requires a two phase medium
     (extending from Modelica.Media.Interfaces.PartialTwoPhaseMedium).
     <br>&nbsp;</li>
<li> In order to simplify the initialization in the components,
     there is the restriction that only media models are supported
     that have T, (p,T), (p,h), (T,X), (p,T,X) or (p,h,X) as
     independent variables. Other media models would be possible,
     e.g., with (T,d) as independent variables. However, this requires
     to rewrite the code for the component initialization.
     (Note, T is temperature, p is pressure, d is density,
     h is specific enthalpy, and X is a mass fraction vector).
     <br>&nbsp;</li>
<li> All components work for <b>incompressible</b> and <b>compressible</b> media.
     This is implemented by a small change in the initialization of a
     component, if the medium is incompressible. Otherwise, the equations
     of the components are not influenced by this property.<br>&nbsp;</li>
<li> All components allow fluid flow in both directions, i.e.,
     <b>reversing flow</b> is supported. However, it is possible to declare that
     the flow through a component only has the design direction, in order to
     obtain faster simulation code.<br>&nbsp;</li>
<li> Two or more components can be connected together. This means that
     the pressures of all connected ports are equal and the mass flow rates
     sum up to zero. Specific enthalpy, mass fractions and trace substances are
     mixed according to the mass flow rates.<br>&nbsp;</li>
<li> The <b>momentum balance</b> and the <b>energy balance</b> are only fulfilled exactly if
     <b>two ports of equal diameter</b> are connected. In all other cases, the balances
     are approximated, because kinetic and friction effect are neglected. An explicit fitting
     or junction should be used if these are important for the specific problem at hand.
     In all circuits where friction dominates, or components such as pumps determine the flow rate,
     kinetic pressure is typically irrelevant. You can consider the
     <a href=\"modelica://Modelica.Fluid.Examples.Explanatory.MomentumBalanceFittings\">Modelica.Fluid.Examples.Explanatory.MomentumBalanceFittings</a> model (and its documentation)
     to see one case where the momentum balance essentially depends on kinetic pressure,
     so it is necessary to use explicit fittings in order to obtain correct results.
     <br>&nbsp;</li>
<li> Given the above-mentioned limitations, there is no restriction how components can be connected
     together. The resulting simulation performance however often strongly depends on the
     model structure and modeling assumptions made. In particular the direct connection of
     fluid volumes generally results in high-index DAEs for the pressures. The direct
     connection of flow models generally results in systems of implicit nonlinear algebraic
     equations.<br>&nbsp;</li>
</ul>
</html>"));
      end Overview;

      class GettingStarted "Getting started"
        extends Modelica.Icons.Information;
        annotation(
          Documentation(info = "<html>

<p>
Please explore the
<a href=\"modelica://Modelica.Fluid.Examples\">Examples</a>,
which provide simple models for a broad variety of applications.
</p>
</html>"));
      end GettingStarted;

      package ComponentDefinition "Component definition"
        extends Modelica.Icons.Information;

        class FluidConnectors "Fluid connectors"
          extends Modelica.Icons.Information;
          annotation(
            Documentation(info = "<html>
<p>
In this section the design of the fluid connectors is
explained.
</p>
<p>
Fluid connectors represent the points in a device (e.g., the
flanges) through which a fluid can flow into or out of the component, carrying its
thermodynamic properties; these flanges are assumed to be fixed in space.
</p>
<p>
A major design goal is that components can be arbitrarily
connected and that the important balance equations are automatically
fulfilled when 2 or more components are connected together at
one point as shown in the next figure:
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Fluid/UsersGuide/MixingConnections.png\"
     alt=\"MixingConnections.png\">
</p>

<p>
In such a case the balance equations define <b>ideal mixing</b>,
i.e., the upstream discretization scheme of each component uses
values that result from ideal mixing in
an infinitely small time period. If more realistic modelling
is desired that takes into account mixing losses, an explicit
model has to be used in the connection point.
</p>
<h4>Single substance media</h4>
<p>
For a single substance medium, the connector definition in
Modelica.Fluid.Interfaces.FluidPort reduces to
</p>
<pre>
  <b>connector</b> FluidPort
     <b>replaceable package</b> Medium = Modelica.Media.Interfaces.PartialMedium
              \"Medium model of the fluid\";
     <b>flow</b> Medium.MassFlowRate m_flow;
              \"Mass flow rate from the connection point into the component\";
     Medium.AbsolutePressure p
              \"Thermodynamic pressure in the connection point\";
     <b>stream</b> Medium.SpecificEnthalpy h_outflow
               \"Specific thermodynamic enthalpy close to the connection point if m_flow &lt; 0\";
  <b>end</b> FluidPort;
</pre>
<p>
The first statement defines the Medium flowing through the connector.
In a medium, medium specific types such as \"Medium.AbsolutePressure\"
are defined that contain medium specific values for the min, max and
nominal attributes. Furthermore, Medium.MassFlowRate is defined as:
</p>
<pre>
   <b>type</b> MassFlowRate =
      Modelica.SIunits.MassFlowRate(quantity=\"MassFlowRate.\" + mediumName);
</pre>
<p>
With the current library design, it is necessary to explicitly select the medium
model for each component in a circuit. This model is then propagated to the ports,
and a Modelica translator will check that the quantity and unit attributes
of connected interfaces are identical. Therefore, an error occurs,
if connected FluidPorts do not have a medium with the same medium name.
In the future, automatic propagation of fluid models through the ports will be
introduced, but this still not possible with Modelica 3.0.
</p>
<p>
The thermodynamic pressure is an <i>effort</i> variable, which means that the connection
of two or more ports states that the port pressures are the same.
</p>
<p>
The mass flow rate is a <i>flow</i> variable, which means that the connection of two or
more ports states that the sum of all flow rates is zero.
</p>
<p>
The last variable is a <i>stream</i> variable, i.e., a specific quantity carried by the
flow variable. The quantity on the connector always
corresponds to the value close to the connection point, assuming that the fluid is
flowing out of the connector, regardless of the actual direction of the flow. This helps
avoiding singularities when the mass flow goes through zero. The stream properties for the
other flow direction can be inquired with the built-in operator inStream(..), while the
value of the stream variable corresponding to the actual flow direction can be inquired
through the built-in operator actualStream(..).
</p>
<p>
The actual equations corresponding to these operators are introduced and solved automatically
by the tool. In principle, they correspond to the balance equation
sum(flow_variable) = 0 and sum(flow_variable*stream_variable_at_connection) = 0 applied to the set of connected ports. In this case the first equation is the mass balance sum(m_flow) = 0, and the second is the energy balance at the connection point sum(m_flow*h_connection) = 0.
</p>
<p>
In the simpler case of a one-to-one connections between port_a and port_b,
inStream(port_a.h_outflow) just returns port_b.h_outflow. For multiple-way connections,
mixing equations are generated, and special care is taken in order to avoid discontinuities
around zero flow rates. For more details, see this
<a href=\"modelica://Modelica/Resources/Documentation/Fluid/Stream-Connectors-Overview-Rationale.pdf\">presentation</a>
which illustrates the stream concept rationale and the underlying technicalities.
</p>

<p>
A connector should have only the minimal number of variables to
describe the interface, otherwise there will be connection
restrictions in certain cases. Therefore, in the connector
no redundant variables are present, e.g., the temperature T
is not present because it can be computed from the connector
variables pressure p and specific enthalpy h.
</p>

<p>
Here are two simple examples to illustrate modeling with stream connectors. The first
one is a rigid adiabatic volume mixing two flows, where the kinetic and gravitational
terms in the energy balance are neglected for simplicity.
</p>

<pre>
model MixingVolume \"Volume that mixes two flows\"
  replaceable package Medium = Modelica.Media.Interfaces.PartialPureSubstance;
  FluidPort port_a, port_b;
  parameter Modelica.SIunits.Volume V \"Volume of device\";
  Modelica.SIunits.Mass             m \"Mass in device\";
  Modelica.SIunits.Energy           U \"Inner energy in device\";
  Medium.BaseProperties medium(preferredMediumStates=true) \"Medium in the device\";
equation
  // Definition of port variables
  port_a.p         = medium.p;
  port_b.p         = medium.p;
  port_a.h_outflow = medium.h;  // The stream variable always corresponds to the
  port_b.h_outflow = medium.h;  // properties of the fluid holdup (outgoing flow)

  // Total quantities
  m = V*medium.d;
  U = m*medium.u;
   // Mass and energy balance (actualStream(..) is a built-in operator for streams to
  // compute the right h, depending on the flow direction)
  der(m) = port_a.m_flow + port_b.m_flow;
  der(U) = port_a.m_flow*actualStream(port_a.h_outflow) +
           port_b.m_flow*actualStream(port_b.h_outflow);
end MixingVolume;
</pre>

<p>
The second example is the model of a component describing a lumped pressure loss
between two ports, with no energy storage and no heat transfer. An isenthalpic
transformation is assumed (changes in kinetic and potential energy between
inlet and outlet are neglected)
</p>
<pre>
model PressureLoss \"Pressure loss component\"
  replaceable package Medium=Modelica.Media.Interfaces.PartialPureSubstance;
  FluidPort port_a, port_b:
  Medium.ThermodynamicState port_a_state_inflow \"State at port_a if inflowing\";
  Medium.ThermodynamicState port_b_state_inflow \"State at port_b if inflowing\";
  Medium density d_a, d_b \"Density at ports a and b if inflowing\";
  replaceable function f \"Function to compute the mass flow rate\";
equation
  // Medium states for inflowing fluid
  port_a_state_inflow = Medium.setState_phX(port_a.p, inStream(port_a.h_outflow));
  port_b_state_inflow = Medium.setState_phX(port_b.p, inStream(port_b.h_outflow));
  // Mass balance
  0 = port_a.m_flow + port_b.m_flow;
  // Instantaneous propagation of enthalpy flow between the ports with
  // isenthalpic state transformation (no storage and no loss of energy)
  port_a.h_outflow = inStream(port_b.h_outflow);
  port_b.h_outflow = inStream(port_a.h_outflow);
  // (Regularized) Momentum balance
  port_a.m_flow = f(port_a.p, port_b.p, d_a, d_b);
end PressureLoss;
</pre>

<p>
If many such components are connected in series between two models with storage, the
specific enthalpies are propagated in both directions and available to all pressure
loss components, without problems when the mass flow goes through zero. The function
f then uses either d_a or d_b depending on the sign of port_a.p-port_b.p, with a
suitable regularization around zero to avoid discontinuities.
</p>

<p>
Please note that these models are highly idealized in order to explain the stream connector
concept. Device models in the library are much more complete, handling issues such as
initialization, steady vs. dynamic modelling, heat transfer from the outside, etc.
</p>

<h4>Multiple-substance media</h4>
<p>
Modelica.Fluid can handle models where the fluid contains multiple substances, so that its
composition can be characterized by mass fraction vectors.
</p>
<pre>
<b>connector</b> FluidPort
   <b>replaceable package</b> Medium = Modelica.Media.Interfaces.PartialMedium
      \"Medium model of the fluid\";
   <b>flow</b> Medium.MassFlowRate m_flow;
      \"Mass flow rate from the connection point into the component\"
   Medium.AbsolutePressure p
      \"Thermodynamic pressure in the connection point\";
   <b>stream</b> Medium.SpecificEnthalpy h_outflow
       \"Specific thermodynamic enthalpy close to the connection point if m_flow &lt; 0\";
   <b>stream</b> Medium.MassFraction Xi_outflow[Medium.nXi]
       \"Independent mixture mass fractions m_i/m close to the connection point if m_flow &lt; 0\";
   <b>stream</b> Medium.ExtraProperty C_outflow[Medium.nC]
       \"Properties c_i/m close to the connection point if m_flow &lt; 0\";
  <b>end</b> FluidPort;
</pre>
The mass fraction vectors Xi and C are also stream quantities, as they are carried by the mass
flow rate. The corresponding connection equations are sum(m_flow*Xi) and sum(m_flow*C), which correspond to mass balances for the single substances. The vector Xi contains the mass fractions
of the main components of the fluid, and is used together with p and h to determine the
thermodynamic state of the fluid. The vector C contains the mass fraction of the trace components,
which are accounted for in mass balances, but is ignored when computing the fluid properties. This
allows to easily declare and use medium models with trace components starting from existing medium
models (e.g., adding CO<sub>2</sub> traces to Moist Air for air conditioning models).

<h4>Approximations in balance equations at connection point</h4>
<p>
Summing up, when two or more ports of the type FluidPort are connected, the following
equations are generated by the tool:
</p>
<pre>
sum(port_j.m_flow) = 0;               // Total Mass balance
port_j = port_k;                      // Momentum balance
sum(port_j.m_flow*h_connection) = 0;  // Energy balance
sum(port_j.m_flow*Xi_connection) = 0; // Single component mass balances
sum(port_j.m_flow*C_connection) = 0;  // Trace components mass balances
</pre>
<p>
It is <b>very important</b> to bear in mind that
</p>
<ul>
<li> the mass balances are always exact; </li>
<li> the momentum and energy balance are only exact when two port with the same
diameter are connected, because there is no friction and no change in fluid velocity. </li>
</ul>
<p>
In all other cases, i.e., different port diameters and/or multiple port connections:
</p>
<ul>
<li> The momentum balance does not consider friction effects and changes of pressure due to changes
in velocity. </li>
<li> There might thus be errors in the momentum balance of the order of magnitude
of the dynamic pressure &rho;v<sup>2</sup>/2.</li>
<li> The energy balance does not consider the kinetic terms (gravity terms cancel out due
to the infinitesimal size of the connection volume). There might thus be errors in the momentum balance of the order of magnitude of the kinetic energy v^2/2. </li>
</ul>
<p>
In many applications, where fluid speeds are low and thermal phenomena are mainly of interest,
these approximations are commonly made and lead to acceptable results.
In all other cases, explicit fitting and junction models should be used, that model explicitly
all the kinetic phenomena with the appropriate level of detail.
</p></html>"));
        end FluidConnectors;

        class BalanceEquations "Balance equations"
          extends Modelica.Icons.Information;
          annotation(
            Documentation(info = "<html>

<p>
For one-dimensional flow
along the coordinate \"x\", the following partial differential
equations hold
</p>
<table border=1 cellspacing=0 cellpadding=2>
  <tr><td> Mass balance</td>
      <td> <img src=\"modelica://Modelica/Resources/Images/Fluid/UsersGuide/massBalance.png\"
                alt=\"massBalance.png\"></td>
  </tr>
  <tr><td> Momentum balance</td>
      <td> <img src=\"modelica://Modelica/Resources/Images/Fluid/UsersGuide/momentumBalance.png\"
                alt=\"momentumBalance.png\"></td>
  </tr>
  <tr><td> Energy balance 1</td>
      <td> <img src=\"modelica://Modelica/Resources/Images/Fluid/UsersGuide/energyBalance1.png\"
                alt=\"energyBalance1.png\"></td>
  </tr>
  <tr><td> Pipe friction</td>
      <td> <img src=\"modelica://Modelica/Resources/Images/Fluid/UsersGuide/pipeFriction.png\"
                alt=\"pipeFriction.png\"></td>
  </tr>
  <tr><td></td>
      <td>x: independent spatial coordinate (flow is along coordinate x)<br>
          t: time<br>
          v(x,t): mean velocity<br>
          p(x,t): mean pressure<br>
          T(x,t): mean temperature<br>
          &rho;(x,t): mean density<br>
          u(x,t): specific internal energy<br>
          z(x): height over ground<br>
          A(x): area perpendicular to direction x<br>
          g: gravity constant<br>
          f: Fanning friction factor<br>
          S: circumference
  </tr>
</table>
<p>
An alternative energy balance can be derived by multiplying
the momentum balance with \"v\" and subtracting it
from the energy balance 1 above. This results in
the \"energy balance 2\":
</p>
<table border=1 cellspacing=0 cellpadding=2>
  <tr><td> Energy balance 2</td>
      <td> <img src=\"modelica://Modelica/Resources/Images/Fluid/UsersGuide/energyBalance2.png\"
                alt=\"energyBalance2.png\"></td>
  </tr>
</table>

<p>
This formulation separates the internal energy of the fluid from the kinetic energy of fluid flow.
The internal energy is treated by the energy balance 2, the kinetic energy is treated by the momentum balance equally well.
The evaluation of medium properties, which are independent of the kinetic energy, and the formulation of many fluid models is simplified with the energy balance 2.
The overall conservation of energy is achieved by considering the mutual dependencies of energy and momentum balance.
</p>
<p>
Some components in the library, like DynamicPipe, provide a rigorous implementation of mass,
momentum and energy balance, using the energy balance 2 equation. Other components, like Valves and
Fittings, neglect the impact of changes of the kinetic energy and potential energy on the energy
balance, because they are usually irrelevant compared to changes due to heat flows. The StaticPipe
component neglects the effect of kinetic energy, but includes the potential energy in the balance,
which might be substantial.
</p>
<p>
All modelling assumptions and simplifications are stated in the component documentation; please note that some of the assumptions might be stated in the base classes the
component inherits from.
</p>
</html>"));
        end BalanceEquations;

        class UpstreamDiscretization "Upstream discretization"
          extends Modelica.Icons.Information;
          annotation(
            Documentation(info = "<html>

<p>
When implementing a Fluid component, the difficult arises that
the value of intensive quantities (such as p, T, &rho;)
shall be accessed from the
<b>upstream</b> volume. For example, if the fluid flows
from volume A to volume B, then the intensive quantities of
volume B have no influence on the fluid between the
two volumes. On the other hand, if the flow direction is reversed,
the intensive quantities of volume A have no influence
on the fluid between the two volumes.
</p>
<p>
In the Modelica.Fluid library, such a situation is handled
with the following code fragment
(from Interfaces.PartialTwoPortTransport):
</p>
<pre>    <b>replaceable package</b> Medium =
                   Modelica.Media.Interfaces.PartialMedium
                   <b>annotation</b>(choicesAllMatching = <b>true</b>);

    Interfaces.FluidPort_a port_a(<b>redeclare package</b> Medium = Medium);
    Interfaces.FluidPort_b port_b(<b>redeclare package</b> Medium = Medium);

    Medium.ThermodynamicState port_a_state_inflow
                    \"Medium state close to port_a for inflowing mass flow\";
    Medium.ThermodynamicState port_b_state_inflow
                    \"Medium state close to port_b for inflowing mass flow\";

  <b>equation</b>
    // Isenthalpic state transformation (no storage and no loss of energy)
    port_a.h_outflow  = <b>inStream</b>(port_b.h_outflow);
    port_b.h_outflow  = <b>inStream</b>(port_a.h_outflow);

    port_a.Xi_outflow = <b>inStream</b>(port_b.Xi_outflow);
    port_b.Xi_outflow = <b>inStream</b>(port_a.Xi_outflow);

    // Mass balance
    port_a.m_flow + port_b.m_flow = 0;

    // Medium states for inflowing medium
    port_a_state_inflow = Medium.setState_phX(port_a.p, port_b.h_outflow, port_b.Xi_outflow);
    port_b_state_inflow = Medium.setState_phX(port_b.p, port_a.h_outflow, port_a.Xi_outflow);

    // Densities close to the parts when mass flows in to the respective port
    port_a_rho_inflow = Medium.density(port_a_state_inflow);
    port_b_rho_inflow = Medium.density(port_b_state_inflow);

    // Pressure drop correlation (k_ab, k_ba are the loss factors for the two flow
    // directions; e.g., for a circular device: k = 8*zeta/(pi*diameter)^2)^2)
    m_flow = Utilities.regRoot2(port_a.p - port_b.p, dp_small,
                                port_a_rho_inflow/k1, port_b_rho_inflow/k2);
</pre>
<p>
The medium states for inflowing media can be used to compute density and dynamic
viscosity which in turn can be use to formulate the pressure drop equation.
The standard pressure drop equation
</p>

<pre>
   dp = port_a - port_b;
   m_flow = sqrt(2/(zeta*diameter))*if dp >= 0 then  sqrt(dp)
                                               else -sqrt(-dp)
</pre>

<p>
cannot be used, since the function has an infinite derivative at dp=0.
Instead the region around zero mass flow rate must be regularized using
one of the regularization functions of Modelica.Fluid.Utilities.
This requires to have density and/or other medium properties for both
flow directions at the same time. These media properties can be computed
from the medium states of the inflowing fluid at the two ports.
</p>

<p>
If the above component is connected between two volumes, i.e.,
the independent medium variables in port_a and port_b are
usually states, then port_a.h and port_b.h are either states
(i.e., known quantities in the model) or are computed from
states. In either case they are \"known\". In such a situation,
all equations can be directly evaluated without any problems.
Zero or reversed mass flow rate does not pose any problems because
the medium properties are always computed for both flow directions
and are then used in the regularization function.
</p>

<p>
If 3 or more components are connected together, it can be shown
that a system of non-linear algebraic equations appear.
The equations are written by purpose in such a form, that
a tool can select mass flow rates and pressures as iteration
variables of this system. The advantage is that these iteration
variables are continuous and even often differentiable. The
alternative to use the medium states as iteration variables
is not good, because T,h,d are discontinuous for reversing flow
direction.
</p>
</html>"));
        end UpstreamDiscretization;

        class RegularizingCharacteristics "Regularizing characteristics"
          extends Modelica.Icons.Information;
          annotation(
            Documentation(info = "<html>

<p>
Pressure drop equations and other fluid characteristics are usually
computed by <b>semi-empirical</b> equations. Unfortunately, the developers
of semi-empirical equations nearly never take into account that the
equation might be used in a simulation program. As a consequence, these
semi-empirical equations can nearly never be used blindly but must
be slightly modified or adapted in order that obvious
simulation problems are avoided. Below, examples are given to
demonstrate what problems occur and how to regularize the characteristics:
</p>

<h4>Square root function</h4>
<p>
In several empirical formulae, expressions of the following form
are present, e.g., for turbulent flow in a pipe:
</p>
<pre>   y = <b>if</b> x &lt; 0 <b>then</b> -<b>sqrt</b>( <b>abs</b>(x) ) <b>else</b> <b>sqrt</b>(x)
</pre>
<p>
A plot of this characteristic is shown in the next figure:
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Fluid/UsersGuide/sqrt.png\"
     alt=\"sqrt.png\">
</p>

<p>
The difficulty with this function is that the derivative at x=0 is infinity.
In reality, such a function does not exist. E.g., for pipe flow,
the flow becomes laminar for small velocities and therefore around zero the
sqrt() function is replaced by a linear function. Since the laminar region is
usually of not much practical interest, the above approximation is used.
</p>
<p>
The direct implementation above does not work in Modelica, because
an event is generated when x &lt; 0 changes sign. In order to detect
this event, an event iteration takes place. During the event iteration,
the active if-branch is not changed. For example, assume that x is positive
(= \"else\" branch) and shall become negative. During the event iteration
x is slightly negative and the else branch, i.e., sqrt(x), is evaluated.
Since this results in an imaginary number, an error occurs.
It would be possible to fix this, by using the <b>noEvent</b>() operator
to explicitly switch of an event:
</p>
<pre>   y = <b>if</b> <b>noEvent</b>(x &lt; 0) <b>then</b> -<b>sqrt</b>( <b>abs</b>(x) ) <b>else</b> <b>sqrt</b>(x)
</pre>
<p>
Still, it is highly likely that good integrators will not work well
around x=0, because they will recognize that the derivative changes very
sharply and will reduce the step size drastically.
</p>
<p>
There are several solutions around this problem: Around x=0, the sqrt() function
can be replaced by a polynomial of 3rd order which is determined in such a way
that it smoothly touches the sqrt() function, i.e., the whole function is continuous
and continuously differentiable. In the Modelica.Fluid library, implementations of
such critical functions are provided in sublibrary Modelica.Fluid.Utilities.
The above sqrt() type function is computed by function <b>Utilities.regRoot</b>().
This function is defined as:
</p>
<pre>     y := x/(x*x+delta*delta)^0.25;
</pre>
<p>
where \"delta\" is the size of the small region around zero where the
sqrt() function is approximated by another function. The plot of the
function above is practically identical to the one of the original function.
However, it has a finite derivative at x=0 and is differentiable up to
any order. With the default value of delta=0.01, the difference between
the function above and regRoot(x) is 16% around x=0.01, 0.25% around x=0.1
and 0.0025% around x=1.
</p>
</html>"));
        end RegularizingCharacteristics;

        class WallFriction "Wall friction"
          extends Modelica.Icons.Information;
          annotation(
            Documentation(info = "<html>

<p>
One important special case for a pressure loss is the friction at the
wall of a pipe under the assumption of quasi steady state flow (i.e., the
mass flow rate varies only slowly). In this section it is explained how this case is
handled in the Modelica.Fluid library for pipes with
<b>nonuniform roughness</b>, including the smooth pipe
as a special case (see
<a href=\"modelica://Modelica.Fluid.Pipes.BaseClasses.WallFriction\">Pipes.BaseClasses.WallFriction</a>).
The treatment is non-standard in order to get a
numerically well-posed description.
</p>

<p>
For pipes with circular cross section the pressure drop is computed as:
</p>

<pre>
   dp = &lambda;(Re,<font face=\"Symbol\">D</font>)*(L/D)*&rho;*v*|v|/2
      = &lambda;(Re,<font face=\"Symbol\">D</font>)*8*L/(&pi;^2*D^5*&rho;)*m_flow*|m_flow|
      = &lambda;2(Re,<font face=\"Symbol\">D</font>)*k2*sign(m_flow);

with
   Re     = |v|*D*&rho;/&mu;
          = |m_flow|*4/(&pi;*D*&mu;)
   m_flow = A*v*&rho;
   A      = &pi;*(D/2)^2
   &lambda;2     = &lambda;*Re^2
   k2     = L*&mu;^2/(2*D^3*&rho;)
</pre>

<p>
where
</p>
<ul>
<li> L is the length of the pipe.</li>
<li> D is the diameter of the pipe. If the pipe has not a
     circular cross section, D = 4*A/P, where A is the cross section
     area and P is the wetted perimeter.</li>
<li> &lambda; = &lambda;(Re,<font face=\"Symbol\">D</font>) is the \"usual\" wall friction coefficient.</li>
<li> &lambda;2 = &lambda;*Re^2 is the used friction coefficient to get a numerically
     well-posed formulation.</li>
<li> Re = |v|*D*&rho;/&mu; is the Reynolds number.</li>
<li> <font face=\"Symbol\">D</font> = <font face=\"Symbol\">d</font>/D is the relative roughness where
     \"<font face=\"Symbol\">d</font>\" is
     the absolute \"roughness\", i.e., the averaged height of asperities in the pipe
     (<font face=\"Symbol\">d</font> may change over time due to growth of surface asperities during
      service, see <i>[Idelchick 1994, p. 85, Tables 2-1, 2-2])</i>.</li>
<li> &rho; is the upstream density.</li>
<li> &mu; is the upstream dynamic viscosity.</li>
<li> v is the mean velocity.</li>
</ul>
<p>
The first form with &lambda; is used and presented in textbooks,
see \"blue\" curve in the next figure:
</p>

<p>
<IMG src=\"modelica://Modelica/Resources/Images/Fluid/Components/PipeFriction1.png\" ALT=\"PipeFriction1\">
</p>

<p>
This form is not suited for a simulation program since
&lambda; = 64/Re if Re &lt; 2000, i.e., a division by zero occurs for
zero mass flow rate because Re = 0 in this case.
More useful for a simulation model is the friction coefficient
&lambda;2 = &lambda;*Re^2, because &lambda;2 = 64*Re if Re &lt; 2000 and
therefore no problems for zero mass flow rate occur.
The characteristic of &lambda;2 is shown in the next figure and is
used in Modelica.Fluid:
</p>

<p>
<IMG src=\"modelica://Modelica/Resources/Images/Fluid/Components/PipeFriction2.png\" ALT=\"PipeFriction2\">
</p>

<p>
The pressure loss characteristic is divided into three regions:
</p>

<ul>
<li> <b>Region 1</b>:
     For <b>Re &le; 2000</b>, the flow is <b>laminar</b> and the exact solution of the
     3-dim. Navier-Stokes equations (momentum and mass balance) is used under the
     assumptions of steady flow, constant pressure gradient and constant
     density and viscosity (= Hagen-Poiseuille flow) leading to &lambda;2 = 64*Re.
     Therefore:
     <pre> dp = 128*&mu;*L/(&pi;*D^4*&rho;)*m_flow </pre> <br>&nbsp;
</li>

<li> <b>Region 3</b>:
     For <b>Re &ge; 4000</b>, the flow is <b>turbulent</b>.
     Depending on the calculation direction (see \"inverse formulation\"
     below) either of two explicit equations are used. If the pressure drop dp
     is assumed to be known, &lambda;2 = |dp|/k2. The
     Colebrook-White equation
     <i>[Colebrook 1939; Idelchik 1994, p. 83, eq. (2-9)]</i>:
     <pre>1/sqrt(&lambda;) = -2*lg( 2.51/(Re*sqrt(&lambda;)) + 0.27*<font face=\"Symbol\">D</font>) </pre>
     gives an implicit relationship between Re and &lambda;.
     Inserting &lambda;2 = &lambda;*Re^2 allows to solve this equation analytically
     for Re: <pre>Re = -2*sqrt(&lambda;2)*lg(2.51/sqrt(&lambda;2) + 0.27*<font face=\"Symbol\">D</font>)</pre>
     Finally, the mass flow rate m_flow is computed from Re via
     m_flow = Re*&pi;*D*&mu;/4*sign(dp).
     These are the <b>red</b> curves in the diagrams above.<br>
     If the mass flow rate is assumed known (and therefore implicitly
     also the Reynolds number), then &lambda;2 is computed by an
     approximation of the inverse of the Colebrook-White equation
     <i>[Swamee and Jain 1976;
     Miller 1990, p. 191, eq.(8.4)]</i> adapted to &lambda;2:
     <pre> &lambda;2 = 0.25*(Re/lg(<font face=\"Symbol\">D</font>/3.7 + 5.74/Re^0.9))^2 </pre>
     The pressure drop is then computed as dp = k2*&lambda;2*sign(m_flow).
     These are the <b>blue</b> curves in the diagrams above.<br>&nbsp;</li>

<li> <b>Region 2</b>:
     For <b>2000 &le; Re &le; 4000</b> there is a transition region between laminar
     and turbulent flow. The value of &lambda;2 depends on more factors as just
     the Reynolds number and the relative roughness, therefore only crude
     approximations are possible in this area.<br>
     The deviation from the laminar region depends on the
     relative roughness. A laminar flow at Re=2000 is only reached for smooth pipes.
     The deviation Reynolds number Re1 is computed according to
     <i>[Samoilenko 1968; Idelchik 1994, p. 81, sect. 2.1.21]</i> as:
     <pre>Re1 = 745*e^(if <font face=\"Symbol\">D</font> &le; 0.0065 then 1 else 0.0065/<font face=\"Symbol\">D</font>)</pre>
     These are the <b>blue</b> curves in the diagrams above.<br>
     Between Re1=Re1(<font face=\"Symbol\">d</font>/D) and Re2=4000,
     &lambda;2 is approximated by a cubic
     polynomial in the \"lg(&lambda;2) - lg(Re)\" chart (see figures above) such that the
     first derivative is continuous at these two points. In order to avoid
     the solution of non-linear equations, two different cubic polynomials are used
     for the direct and the inverse formulation. This leads to some discrepancies
     in &lambda;2 (= differences between the red and the blue curves).
     This is acceptable, because the transition region is anyway not
     precisely known since the actual friction coefficient depends on
     additional factors and since the operating points are usually
     not in this region.</li>
</ul>
<p>
The absolute roughness <font face=\"Symbol\">d</font> has usually to
be estimated. In <i>[Idelchik 1994, pp. 105-109,
Table 2-5; Miller 1990, p. 190, Table 8-1]</i> many examples are given.
As a short summary:
</p>
<table border=1 cellspacing=0 cellpadding=2>
  <tr><td><b>Smooth pipes</b></td>
      <td>Drawn brass, copper, aluminium, glass, etc.</td>
      <td><font face=\"Symbol\">d</font> = 0.0025 mm</td>
  </tr>
  <tr><td rowspan=\"3\"><b>Steel pipes</b></td>
      <td>New smooth pipes</td>
      <td><font face=\"Symbol\">d</font> = 0.025 mm</td>
  </tr>
  <tr><td>Mortar lined, average finish</td>
      <td><font face=\"Symbol\">d</font> = 0.1 mm</td>
  </tr>
  <tr><td>Heavy rust</td>
      <td><font face=\"Symbol\">d</font> = 1 mm</td>
  </tr>
  <tr><td rowspan=\"3\"><b>Concrete pipes</b></td>
      <td>Steel forms, first class workmanship</td>
      <td><font face=\"Symbol\">d</font> = 0.025 mm</td>
  </tr>
  <tr><td>Steel forms, average workmanship</td>
      <td><font face=\"Symbol\">d</font> = 0.1 mm</td>
  </tr>
  <tr><td>Block linings</td>
      <td><font face=\"Symbol\">d</font> = 1 mm</td>
  </tr>
</table>
<p>
The equations above are valid for incompressible flow.
They can also be applied for <b>compressible</b> flow up to about <b>Ma = 0.6</b>
(Ma is the Mach number) with a maximum error in &lambda; of about 3 %.
The effect of gas compressibility in a wide region can be taken into
account by the following formula derived by Voronin
<i>[Voronin 1959; Idelchick 1994, p. 97, sect. 2.1.81]</i>:
</p>
<pre>
  &lambda;_comp = &lambda;*(1 + (&kappa;-1)/2 * Ma^2)^(-0.47)
</pre>
<p>
where &kappa; is the isentropic coefficient
(for ideal gases, &kappa; is the ratio of specific heat capacities cp/cv).
An appreciable decrease in the coefficient \"&lambda;_comp\" is observed
only in a narrow transonic region and also at supersonic flow velocities
by about 15% <i>[Idelchick 1994, p. 97, sect. 2.1.81]</i>.
This effect is not yet included in Modelica.Fluid.
Another restriction is that the pressure drop model is valid
only for steady state or slowly changing mass flow rate.
For large fluid acceleration, the pressure drop depends additionally
on the frequency of the changing mass flow rate.
</p>

<h4>Inverse formulation</h4>

<p>
In the \"Advanced menu\" it is possible via parameter
\"from_dp\" to define in which form the
pressure drop equation is actually evaluated (<b>default</b> is from_dp = <b>true</b>):
</p>
<pre>
   from_dp = <b>true</b>:   m_flow = f1(dp)
           = <b>false</b>:  dp     = f2(m_flow)
</pre>
<p>
\"from_dp\" can be useful to avoid nonlinear systems of equations
in cases where the inverse pressure loss function is needed.
</p>

<h4>Summary</h4>

<p>
A detailed pressure drop model for pipe wall friction is
provided in the form m_flow = f1(dp, <font face=\"Symbol\">D</font>) or
dp = f2(m_flow, <font face=\"Symbol\">D</font>).
These functions are continuous and differentiable,
are provided in an explicit form without solving non-linear equations,
and do behave well also at small mass flow rates. This pressure drop
model can be used stand-alone in a static momentum balance and in
a dynamic momentum balance as the friction pressure drop term.
It is valid for incompressible and compressible flow up to a Mach number of 0.6.
</p>

<h4>References</h4>

<dl><dt>Colebrook F. (1939):</dt>
    <dd><b>Turbulent flow in pipes with particular reference to the transition
         region between the smooth and rough pipe laws</b>.
         J. Inst. Civ. Eng. no. 4, 14-25.</dd>
    <dt>Idelchik I.E. (1994):</dt>
    <dd><a href=\"http://www.bookfinder.com/dir/i/Handbook_of_Hydraulic_Resistance/0849399084/\"><b>Handbook of Hydraulic Resistance</b></a>. 3rd edition, Begell House, ISBN 0-8493-9908-4</dd>
    <dt>Miller D. S. (1990):</dt>
    <dd><b>Internal flow systems</b>.
    2nd edition. Cranfield:BHRA(Information Services).</dd>
    <dt>Samoilenko L.A. (1968):</dt>
    <dd><b>Investigation of the Hydraulic Resistance of Pipelines in the
        Zone of Transition from Laminar into Turbulent Motion</b>.
        Thesis (Cand. of Technical Science), Leningrad.</dd>
    <dt>Swamee P.K. and Jain A.K. (1976):</dt>
    <dd><b>Explicit equations for pipe-flow problems</b>.
         Proc. ASCE, J.Hydraul. Div., 102 (HY5), pp. 657-664.</dd>
    <dt>Voronin F.S. (1959):</dt>
    <dd><b>Effect of contraction on the friction coefficient in a
           turbulent gas flow</b>.
           Inzh. Fiz. Zh., vol. 2, no. 11, pp. 81-85.</dd>
</dl>

</html>"));
        end WallFriction;

        class ValveCharacteristics "Valve characteristics"
          extends Modelica.Icons.Information;
          annotation(
            Documentation(info = "<html>
<p>
The control valves in
<a href=\"modelica://Modelica.Fluid.Valves\">Modelica.Fluid.Valves</a>
have the parameters <b>Kv</b> and <b>Cv</b>. They are defined
as unit-less variables, but in the description text a unit
is given. The reason for this definition is the following:
</p>

<p>
The basic equation for valves is:
</p>

<pre>
  q = Av*sqrt(dp/rho)
</pre>

<p>
In SI units, [q] is m3/s, [dp] is Pascal, [rho] is [kg/m3], and Av is an area, thus [Av] = m2. Basically, the equation stems from Bernoulli's law. Av is roughly 1.4 times the area of the valve throat. Now, usually valves aren't so big that their throat area is of the order of magnitude of square meters - depending on the applications it is from a few square millimeters to a few square centimeters. Therefore, in the common engineering practice, the following equations are used:
</p>

<p>
Europe:
</p>

<pre>
  q = Kv sqrt(dp/(rho/rho0)) , with [q] = m3/h, [dp] = bar
</pre>

<p>
US:
</p>

<pre>
  q = Cv sqrt(dp/(rho/rho0)) , with [q] = USG/min, [dp] = psi
</pre>

<p>
In both cases rho0 is the density of cold water at 4 &deg;C, 999 kg/m3. Note that these equations use relative, not absolute densities.
</p>

<p>
It turns out that Kv = 1e6/27.7*Av and Cv = 1e6/24*Av, so both US and EU engineers get more or less the same numbers (just by sheer luck), with a range between a few units and a few hundred units for typical industrial applications, and everybody is happy.
</p>

<p>
Now, we've got two problems here. First, depending on the unit, we change the equation: with SI units, we use the density, with non-SI units, we use the relative density. So the quantities (not only the units!) of Av and Cv/Kv are different.
</p>

<p>
Second, the units of Kv and Cv are usually labeled \"m3/h\" and \"USG/min\", but as a matter of fact they are different, as can be seen from the equations above: they are actually
m3/(h*sqrt(bar)) and USG/(min*sqrt(psi)). If I have a valve with Kv = 10 m3/h, it means I get 10 m3/h \"for a pressure drop of 1 bar\". Unfortunately, this is not correct from the point of view of strict dimensional analysis, but nobody uses sqrt(Pa) or sqrt(bar).
</p>

<p>
You might think this is crazy (it is, especially when you try to explain it), but as a matter of fact the valve coefficient is <b>never</b> given in square meters in any catalog or datasheet; Cv is still the most used (even in Europe), followed by Kv. So, it will be very inconvenient for users to type in Av in square meters.
</p>

<p>
The pragmatic approach used in Modelica.Fluid.ControlValves is to accept the fact that m3/h and USG/min are not the real units of Cv and Kv, so we can't use the general unit conversion mechanism, put them just as mnemonic labels in the comment, use non-dimensional coefficients in the interface, and then define properly dimensioned unit conversion within the model
</p>

</html>"));
        end ValveCharacteristics;
        annotation(
          Documentation(info = "<html>

<p>
In this section it is described how the components
of the Modelica.Fluid library are implemented.
If you would like to introduce new components either in
Modelica.Fluid or your own library, you should be aware
of the issues discussed in this section.
</p>
<p>
This section is partly based on the following paper:
</p>
<dl>
<dt> Elmqvist H., Tummescheit H., and Otter M.:</dt>
<dd> <b>Object-Oriented Modeling of Thermo-Fluid Systems</b>.
     Modelica 2003 Conference, Link&ouml;ping, Sweden,
     pp. 269-286, Nov. 3-4, 2003.
     Download from:
     <a href=\"https://www.modelica.org/events/Conference2003/papers/h40_Elmqvist_fluid.pdf\">https://www.modelica.org/events/Conference2003/papers/h40_Elmqvist_fluid.pdf</a>
     </dd>
</dl>
Please note that the design of the connectors has been changed with respect to the design presented in that paper.
</html>"));
      end ComponentDefinition;

      package BuildingSystemModels "Building system models"
        extends Modelica.Icons.Information;

        class SystemComponent "System component"
          extends Modelica.Icons.Information;
          annotation(
            Documentation(info = "<html>
<p>
The Modelica.Fluid library is designed so that each model of a system must
include an instance <code>system</code> of the <code>System</code> component at the top level, in the same way as the <code>World</code> model of the MultiBody Library. The System component contains the parameters that
describe the environment surrounding the components (ambient pressure and
temperature, gravity acceleration), and also provides default settings
for many parameters which are used consistently by the models in the library.
These parameters are then propagated to the individual components
using the inner/outer variable mechanism. In case the system model is structured
hierarchically, it is possible to either put a single System
component at the top level, or possibly to put many of them at different levels,
which will only influence the system components from that level down.
</p>
<p>All the parameters defined in the System model are used as default values for the parameters of the individual components of the system model. Note that it is always possible to override these defaults locally by changing the value of the parameters in the specific component instance.
</p>
<ul>
<li> The <i>General</i> tab of the System model allows to set the default environment variables (pressure, temperature and gravity)
used by all the components.
</li>
<li> The <i>Assumptions</i> tab allows to change the default modelling assumptions
used by all the components (see the section <i>Customizing a system model later</i>)</li>
<li> The <i>Initialization</i> tab allows to define default start values for mass flow rates, pressures and temperatures in the model; this can be useful to help nonlinear solver converge to the solution of any nonlinear system of equations that involves such variables, by providing meaningful guess values. </li>
<li> The <i>Advanced</i> tab contains default values for parameters used in
the advanced settings of some components.</li>
</ul>
<p>
Remember to <b>always add a System component</b> at the top level of
your system model, otherwise you will get errors when compiling the model. The tool will automatically name it <code>system</code>, so that it
is recognized by all other components.
</p>
</html>"));
        end SystemComponent;

        class MediumDefinition "Definition of the medium models"
          extends Modelica.Icons.Information;
          annotation(
            Documentation(info = "<html>

<p>
All the models in Modelica.Fluid compute fluid properties by using medium
models defined by Modelica.Media packages. Custom fluid models can also be
used, provided they extend the interfaces defined in
Modelica.Media.Interfaces.
</p>
<p>
All the components in Modelica.Fluid use a <i>replaceable</i> medium package, called <code>Medium</code>: the model is written for a generic fluid, and a specific fluid model can then be specified when building a system model by redeclaring the package. This can be done in different ways:
<ul>
<li>
If several components use the same medium, it is possible to select
all of them within a GUI, and set them simultaneously (as they are
all named Medium).
</li>
<li>It is also possible to declare one or more (possibly replaceable) medium packages in the model, and then use them to set up the individual components.</li>
</ul>
</html>"));
        end MediumDefinition;

        class CustomizingModel "Customizing a system model"
          extends Modelica.Icons.Information;
          annotation(
            Documentation(info = "<html>
<p>
Once a system model has been built, it is possible to obtain different approximations by
appropriately setting the defaults in the System component (and/or the settings of specific
components.
</p>
<p>
The Assumptions | allowFlowReversal parameter determines whether reversing flow conditions
(i.e., flow direction opposite to design direction) are modelled or not. By default,
reversing flow conditions are considered by the models, but this causes a significant increase
of complexity in the equations, due to the conditional equations depending on the flow direction.
If you know in advance that the flow in a certain component (or in the whole system) will always
be in the design direction, then setting this parameter to false will produce a much faster and
possibly more robust simulation code.
</p>
<p>
The flags in the Assumptions | Dynamics tab allow different degrees of approximation on
the mass, energy, and momentum equations of the components.
</p>
<ul>
<li>DynamicFreeInitial: dynamic equations are considered (nonzero storage), no
initial equations are provided, and the start values are used as guess values.</li>
<li>FixedInitial: dynamic equations are considered (nonzero storage) and initial
equations are included, fixing the states to the start values provided by the
component parameters.</li>
<li>SteadyStateInitial: dynamic equations are considered (nonzero storage), initial
equations are included, declaring that the state derivatives are zero (steady-state
initialization) and the start values are used as guess values for the nonlinear solver. </li>
<li>SteadyState: algebraic (or static) balance equations are considered (no storage)
and the start values are used as guess values for the nonlinear solver.</li>
</ul>
<p>
It is then possible to neglect the storage of mass, momentum, and energy in the whole system
(or just in parts of it) just by a few mouse clicks in a GUI, and also to change the type of
initialization when considering dynamic models. Please note that some combinations of the
options might be contradictory, and will therefore trigger compilation errors.
</p>
</html>"));
        end CustomizingModel;
        annotation(
          Documentation(info = "<html>

<p>
This section is a quick primer explaining how to build a system model using Modelica.Fluid.
It covers some key issues, such as the System component, the definition of medium models in the
system, and the typical customizations available in the Modelica.Fluid models.
</p>
</html>"));
      end BuildingSystemModels;

      class ReleaseNotes "Release notes"
        extends Modelica.Icons.ReleaseNotes;
        annotation(
          Documentation(info = "<html>

<h4>Version 1.1, 2009-06-21</h4>

<p>
The Modelica_Fluid library (revisionId = r2625) was included as Modelica.Fluid in the
Modelica Standard Library 3.1.
</p>

<h4>Version 1.0, 2009-01-28</h4>

<p>
Modelica_Fluid was refactored and finalized for the release:
</p>

<ul>
<li> Refactoring of the code<br>
     This became necessary as the previous release Modelica.Fluid Streams Beta3
     still reflected the long development history, while the basic concepts had been crystallized.
     Please consult the subversion control (SVN) logs for individual changes.</li>

<li> Device oriented package names<br>
     The former sub-packages Junctions and PressureLosses have been combined into the new subpackage Fittings.
     The former Pumps and Volumes.SweptVolume have become the initial version of fluid Machines.
     The former Volumes package is now called Vessels.</li>

<li> Complete implementation of one-dimensional fluid flow<br>
     The balance equations as documented in
     <a href=\"modelica://Modelica.Fluid.UsersGuide.ComponentDefinition.BalanceEquations\">UsersGuide.ComponentDefinition.BalanceEquations</a>
     are now completely implemented. The implementations with generic boundary flow and source terms find in:
     <ul>
     <li><a href=\"modelica://Modelica.Fluid.Interfaces.PartialDistributedVolume\">Interfaces.PartialDistributedVolume</a>,
         <a href=\"modelica://Modelica.Fluid.Interfaces.PartialLumpedVolume\">Interfaces.PartialLumpedVolume</a>:
         Energy, Mass and Substance balances</li>
     <li><a href=\"modelica://Modelica.Fluid.Interfaces.PartialDistributedFlow\">Interfaces.PartialDistributedFlow</a>,
         <a href=\"modelica://Modelica.Fluid.Interfaces.PartialLumpedFlow\">Interfaces.PartialLumpedFlow</a>:
         Momentum balance</li>
     </ul>
     Specific models combine the balances and define the boundary flow and source terms as appropriate.
     For instance
     <ul>
     <li><a href=\"modelica://Modelica.Fluid.Vessels.OpenTank\">Vessels.OpenTank</a> extends from
         <a href=\"modelica://Modelica.Fluid.Interfaces.PartialLumpedVolume\">Interfaces.PartialLumpedVolume</a>,</li>
     <li><a href=\"modelica://Modelica.Fluid.Fittings.SimpleGenericOrifice\">Fittings.SimpleGenericOrifice</a> extends from
         <a href=\"modelica://Modelica.Fluid.Interfaces.PartialLumpedFlow\">Interfaces.PartialLumpedFlow</a>, besides
         <a href=\"modelica://Modelica.Fluid.Interfaces.PartialTwoPortTransport\">Interfaces.PartialTwoPortTransport</a>,</li>
     <li><a href=\"modelica://Modelica.Fluid.Pipes.DynamicPipe\">Pipes.DynamicPipe</a> is based on
         <a href=\"modelica://Modelica.Fluid.Interfaces.PartialDistributedVolume\">Interfaces.PartialDistributedVolume</a> and
         <a href=\"modelica://Modelica.Fluid.Interfaces.PartialDistributedFlow\">Interfaces.PartialDistributedFlow</a>,
         besides <a href=\"modelica://Modelica.Fluid.Interfaces.PartialTwoPort\">Interfaces.PartialTwoPort</a>.</li>
     </ul>
     All non-trivial mass and energy balances of Vessels, Machines and Fittings have been replaced with PartialLumpedVolume.
     The mass and energy balances of Pipes are based on PartialDistributedVolume.<br>
     See <a href=\"modelica://Modelica.Fluid.Examples.BranchingDynamicPipes\">Examples.BranchingDynamicPipes</a>
     for an example utilizing the complete balance equations.

<li> New approach for the connection of distributed flow models<br>
     The staggered grid approach offers different choices for the connection approach. So far the preferred modeling was to put full mass balances
     into the pipes and expose half momentum balances through the ports (ModelStructure a_v_b).
     This resulted in nonlinear equation systems for pressure/flow correlations in connection sets.
     A new default ModelStructure av_vb has been introduced putting full momentum balances into the models and
     exposing half mass balances through the ports (av_vb replaces the former avb). This way the nonlinear equation systems are avoided.
     High-index DAEs need to be treated instead in connection sets.
     Alternatively a Fitting like SuddenExpansion can be introduced to account for different cross flow areas of connected flow models.</li>

<li> New Vessels.BaseClasses.PartialLumpedVessel treating the ports, including hydraulic resistances, for ClosedVolume, SimpleTank and SweptVolume.</li>

<li> Clarification of modeling assumptions<br>
     The documentation has been extended to better explain the modeling assumptions made. In particular the section
     <a href=\"modelica://Modelica.Fluid.UsersGuide.ComponentDefinition.FluidConnectors\">UsersGuide.ComponentDefinition.FluidConnectors</a>
     now makes clear that the ports represent the thermodynamic enthalpy, as opposed to stagnation enthalpy,
     and thermodynamic or static pressure, as opposed to total pressure. An new package Explanatory has been added to the
     examples to show the difference between static pressure and total pressure and possible implications. See
     <a href=\"modelica://Modelica.Fluid.Examples.Explanatory.MomentumBalanceFittings\">Examples.Explanatory.MomentumBalanceFittings</a>.

<li> System (former Ambient)<br>
     The use of the global System object has been extended towards common default values for
     modeling assumptions, initialization, and advanced settings that are different for each application of the library
     but should nevertheless provide default values for reasons of convenience.
     In particular steady-state initialization and complete steady-state simulation can now be specified system-wide.
     A new Types.Init.Dynamics has been introduced, combining steady-state and initial conditions.
     The former Types.Init has become obsolete.
     <br>See <a href=\"modelica://Modelica.Fluid.Examples.HeatingSystem\">Examples.HeatingSystem</a></li>

<li> Extension of pumps for better consideration of zero flow and heat transfer with environment<br>
     The simplified mass and energy balances have been replaced with a rigorous formulation.
     Moreover an optional heat transfer model can be configured for heat exchanged with the environment or the housing.<br>
     See <a href=\"modelica://Modelica.Fluid.Machines.BaseClasses.PartialPump\">Machines.BaseClasses.PartialPump</a></li>

<li> Refinement of valves for flow reversal<br>
     All valves now use upstream discretization for reverting flow conditions.</li>

<li> Finalization of trace substances<br>
     Modelica.Fluid now provides a sound implementation for trace substances,
     which can easily be added to existing Media models, in order to study their evolution in a fluid system.<br>
     See <a href=\"modelica://Modelica.Fluid.Examples.TraceSubstances.RoomCO2WithControls\">Examples.TraceSubstances.RoomCO2WithControls</a></li>

<li> Vectorized ports for volumes<br>
     The ports of models that typically have large volumes, like Vessels and Sources,
     have been vectorized. Formerly the connection of multiple flow models to the same port
     of such volume models resulted in unintended mixing equations for stream variables
     in connection sets outside the volumes. The mixing takes place inside the volumes
     when using multiple ports. Moreover a
     <a href=\"modelica://Modelica.Fluid.Fittings.MultiPort\">Fittings.MultiPort</a>
     has been introduced. It can be attached to components like pipes,
     which do not have vectorized ports on their own.</li>

<li> Inverse parameterization of flow models with nominal operational conditions<br>
     Flow models have been added or extended to support the parameterization with nominal values
     (Machines.ControlledPump, Orifices.SimpleGenericOrifice, Pipes.BaseClasses.FlowModels.NominalTurbulentFlow).
     They are intended for early phases of system modeling, if geometries and flow characteristics
     are of secondary interest. As these models use the same interfaces, base classes and naming conventions,
     they can easily be replaced with more detailed models
     as more information shall be taken into account later on.<br>
     See <a href=\"modelica://Modelica.Fluid.Examples.InverseParameterization\">Examples.InverseParameterization</a></li>

<li> Replaceable HeatTransfer models<br>
     The Vessels and the Machines now have replaceable HeatTransfer models,
     besides the Pipes. All HeatTransfer models are optional.
     The heat transfer models are parameterized with the Medium and the ThermodynamicState
     of involved flow segments.<br>
     See <a href=\"modelica://Modelica.Fluid.Interfaces.PartialHeatTransfer\">Interfaces.PartialHeatTransfer</a>.

<li> All examples are working now (using Dymola 7.1).<br>
     The number of examples has been extended with the former critical test cases
     HeatingSystem and IncompressibleFluidNetwork. Moreover the HeatExchangers have been
     moved into Examples.</li>

</ul>

<h4>Version 1.0 Streams Beta 3, 2008-10-12</h4>

<p>
Modelica.Fluid was further improved:
</p>

<ul>
<li> Volumes, tanks, junctions<br>
     Added asserts to require that ports are connected at most once.
     If a user would perform more than one connection, ideal mixing
     takes place for the connected components and this is nearly never
     what the user would like to have </li>

<li> Ambient<br>
     Renamed Ambient to System, including adaptation of models.<br>
     Introduced default values system.flowDirection and
     as a comment system.initType. system.flowDirection is used in
     two port components as default.</li>

<li> GenericJunction<br>
     Corrected specification of flowDirection.<br>
     Added a HeatPort.</li>

<li> PartialDistributedFlow models<br>
     Adapted determination of velocities to usage of
     upstream properties at ports.<br>
     Corrected and unified initialization of p_start[*] values.</li>

<li> DistributedPipe models<br>
     Changed treatment of port densities and viscosities
     to the treatment of the lumped pipe model. This way events are
     avoided if the mass flow rate crosses or approaches zero.<br>
     Correct determination of Reynolds numbers.<br>
     Added test model DistributedPipeClosingValve.</li>

<li> ControlValves<br>
     Changed flowCharacteristic into valveCharacteristic<br>
     Removed parameter Kv and added dp_nom, m_flow_nom from linear
     and discrete valve interfaces. Added test cases.<br>
     Adapted Examples to new LinearValve and DiscreteValve,
     using nominal values instead of Kv. <br>
     Changed default flow coefficient selection to OpPoint</li>

<li> Fixed units for Kv and Cv in control valve models.</li>

<li> Updated tests for valves.</li>

<li> Bug in Modelica.Fluid.Test.TestComponents.Pumps.TestWaterPump2 corrected
     (complicated redeclaration issue).</li>

<li> Adapted AST_BatchPlant so that \"Check\" is successful.
     Simulation fails after 600 s.</li>

<li> Introduced
     density_pTX(Medium.p_default, Medium.T_default, Medium.X_default)
     as default value for nominal densities (previously it was a literal
     such as 1000).</li>

<li> Pumps<br>
     Updated energy balance equations for pumps (no division by zero anymore,
     fixed several bugs related to Np).<br>
     Added two more test cases for pumps.<br>
     Fixed pump initialization options.</li>

<li> PartialPump<br>
     Explanation for the energy balanced added as comment<br>
     \"h=0\" replaced by \"h=Medium.h_default\" since otherwise an assert is triggered
     if \"h=0\" is not in the medium range.<br>
     Fluid ports positioned in the middle line
     and using the same size as for all other components.</li>

<li> Pumps.Pump <br>
     Resized input connector, so that it has the same size as the
     standard input connectors.<br>
     Changed icon text to input connector to \"N_in [rpm]\".<br>
     Added unit rev/min to the external and internal input connector.</li>

<li> PartialValve<br>
     fillcolor=white added to icon<br>
     made line Thickness = Single,
     since icon does not look nice sometimes</li>

<li> All components<br>
     Changed %name color from black to blue
     (is a conversion bug, since Modelica 2 has blue as default
     color whereas Modelica 3 has black and Dymola is not
     taking care off this).</li>

<li> Sources<br>
     Made icon elements invisible, if corresponding input is disabled.</li>

<li> Valves, Pipes, PressureLosses, HeatExchangers, two port senors<br>
     Added an arrow in the icon for the \"design flow direction\" from
     port_a to port_b.</li>

<li> Moved default initialization in \"System\" in to a comment, since no effect yet</li>

<li> Added the explanation from Francesco for Kv, Cv for valves in the
     User's Guide and added links in the corresponding valves to this description</li>
</ul>

<p>
\"Check\" for the library is successful. \"Check with Simulation\"
(i.e., simulating all test models in the library) is successful
with the exceptions:
</p>

<ul>
<li> Examples.AST_BatchPlant.BatchPlant_StandardWater<br>
     Need to be fixed in a later release (requires quite a lot of work).</li>
<li> Test.TestOverdeterminedSteadyStateInit.Test5<br>
     Test.TestOverdeterminedSteadyStateInit.Test6<br>
     These are test cases where too much initial conditions are given.
     The goal is to work on methods how this can be handled.
     So, this is a principal problem that these models do not simulate.</li>
</ul>

<h4>Version 1.0 Streams Beta 2, 2008-10-08</h4>

<p>
Modelica.Fluid was transformed to Modelica 3 and to Modelica Standard
library 3.0 (by automatic conversion). Further changes:
</p>

<ul>
<li> Emulated enumerations changed to real enumerations.</li>
<li> Improved ControlValves code</li>
<li> Introduced stream connectors with stream keyword (was previously an annotation)</li>
<li> Introduced inStream() instead of inflow() </li>
<li> Introduced m_flow*actualStream(h_outflow) instead of
     streamFlow() or semiLinear(m_flow, inStream(h_outflow), medium.h)</li>
<li> Removed Modelica.Fluid.Media and all references to it (since now available
     in Modelica.Media of MSL3.0).</li>
<li> Fixed PartialLumpedVolume for media with multiple substances</li>
<li> New function \"Utilities.RegFun3\" for regularization with static head</li>
<li> Fix density in static head models with the new RegFun3 functions
     (ticket 7)</li>
<li> Minor bug in MixingVolume corrected:<br>
     V_lumped and Wb_flow have been set as modifiers when extending from PartialLumpedVolume,
     although they are not declared as input. This is not allowed in Modelica 3.
     Fixed by replacing the modifiers by equations.</li>
<li> Modelica.Fluid.Sources.FixedBoundary<br>
     Introduced p_default, T_default, h_default as default values, since
     otherwise warnings will always be printed because parameter value is missing.</li>
<li> Modelica.Fluid.Sources.Boundary_pT<br>
     Modelica.Fluid.Sources.Boundary_ph<br>
     Modelica.Fluid.Sources.MassFlowSource_T<br>
     Changed default values of parameters reference_p, reference_T to
     p_default, T_default (some have been xx_default, some reference_xx,
     it seems best to always use the same approach)</li>
<li> Modelica.Fluid.Pipes.BaseClasses.PartialDistributedFlow<br>
     Added default value for parameter \"rho_nominal\" =
     Medium.density_pTX(Medium.p_default, Medium.T_default, Medium.X_default)
     in order to avoid unnecessary warning messages.
     Should be replaced by \"Medium.rho_default\", once available.</li>
<li> Modelica.Fluid.Pipes.DistributedPipe<br>
     Modelica.Fluid.Pipes.DistributedPipeSb<br>
     Modelica.Fluid.Pipes.DistributedPipeSa<br>
     Added default value for parameter \"mu_nominal\"
    (computed with default values of p,T,X from dynamicViscosity(..))</li>
<li> Modelica.Fluid.Pipes.BaseClasses.PartialDistributedFlowLumpedPressure<br>
     Replaced default value \"rho_nominal=0.01\" by
     Medium.density_pTX(Medium.p_default, Medium.T_default, Medium.X_default)</li>
<li> Modelica.Fluid.Volumes.OpenTank<br>
     Modelica.Fluid.Volumes.Tank<br>
     Corrected icons of ports (wrongly sized by automatic conversion from
     Modelica 2 to Modelica 3).</li>
<li> Examples.BranchingDistributedPipes<br>
     Modelica.Fluid.Test.TestComponents.Junctions.TestGenericJunction<br>
     Modelica.Fluid.Test.TestComponents.Pipes.TestDistributedPipe01<br>
     Parameters dp_nom, m_flow_nom are not defined in junction components.
     Values provided.</li>
<li> PressureLosses.BaseClasses.QuadraticTurbulent.BaseModel<br>
     No default or start values for \"parameter LossFactorData data\"
     Changed the model to \"partial model\" to avoid warning messages</li>
</ul>

<h4>Version 1.0 Streams Beta 1, 2008-05-02</h4>

<p>
Changed connectors to stream connectors and adapted the following sublibraries:
</p>

<ul>
<li> Volumes</li>
<li> PressureLosses </li>
<li> Sensors </li>
<li> Sources </li>
<li> ControlValves </li>
<li> HeatExchangers </li>
<li> Junctions </li>
<li> Pipes </li>
<li> Pumps </li>
<li> Test and Examples (most of the examples and tests are simulating)</li>
</ul>

<p>
Other changes:
</p>

<ul>
<li> Introduced HeatPorts with vectorized icon in Modelica.Fluid.Interfaces</li>
<li> Deleted Modelica.Fluid.WorkInProgress since it seems to be too much work
     to convert it to stream connectors</li>
<li> Added Modelica.Fluid.Media (contains ConstantLiquidWater
     medium because functions are missing in Modelica.Media)</li>
<li> Added two additional test cases with LumpedPipes
    (to identify problems with hierarchically connected stream connectors).</li>
<li> Deleted TestPortVolumes since PortVolumes can no longer be implemented with
     stream connectors</li>
<li> Leakage flow introduced for valves</li>
<li> DrumBoiler Example corrected</li>
<li> Regularization for sensors (T,h,...), in order that no discontinuity
     for bi-directional flow</li>
<li> Density computation in static head corrected</li>
<li> New functions Utilities.regUnitStep, regStep</li>
<li> New components (TestComponents.Sensors.TestOnePortSensors1/.TestOnePortSensors2l,
     TestRegStep)</li>
<li> PartialTwoPortTransport<br>
     <ul>
     <li> Introduced port_a.T, port_b.T (for plotting)</li>
     <li> Removed initialization menu</li>
     <li> Introduced dp_start, m_flow_start</li>
     <li> Removed previous start values of PartialTwoPortTransport in all models</li>
     </ul></li>
<li> PartialPump: Removed p_nom, since no longer needed (only dp_nom)</li>
<li> Made \"%name\" in the icons of all components unified (and better looking)</li>
<li> Changed default value of leakage flow of valves to zero.</li>
<li> Fixed Modelica.Fluid.Junctions.MassFlowRatio so that it compiles
     (inflow(..) currently only supported for scalars, not for vectors)</li>
<li> Added script libraryinfo.mos, in order that Modelica.Fluid appears in the
     Dymola library window automatically (provided library is in MODELICAPATH)</li>
<li> Replaced semiLinear(..) by streamFlow(..) (not yet at all places)</li>
<li> Introduced check-boxes in parameter menu of Sources (is more convenient to use)</li>
<li> TwoPortTransport<br>
     Computation of V_flow and optionally port_a_T, port_b_T.
     Error in temperature calculation corrected</li>
<li> Tank:<br>
     Default of bottom pipe diameter changed from 0 to 0.1, since
     otherwise a division by zero (if not connected and not changed).</li>
<li> Modelica.Fluid.ControlValves.ValveVaporizing:<br>
     Due to changes in PartialTwoPortTransport, port_a_T_inflow does no longer exist
     and the usage to it is removed.
     </li>
<li> Modelica.Fluid.Test.TestComponents.Sensors.TestTemperatureSensor:<br>
     Due to changes in PartialTwoPortTransport,
     p_start does no longer exist and the usage to it is removed.</li>
<li> VersionBuild introduced, as well as automatic update of
     VersionBuild/VersionDate</li>
</ul>

<h4>Version 1.0 Beta 4, 2008-04-26</h4>

<p>
Changes according to the Modelica Design Meetings since the
last beta version. This version is used to \"freeze\" the current
development, in order to change to a version with a new
connector design using stream variables.
</p>

<h4>Version 1.0 Beta 3, 2007-06-05</h4>

<p>
Changes according to the Modelica Design Meetings since the
Modelica'2006 conference, especially, improved initialization,
changed Source components (input connectors must be enabled),
improved tank component, moved test models from Examples to
new package Test, many more test models, etc.
This version is slightly non-backward compatible to version 1.0 Beta 2.
</p>

<h4>Version 1.0 Beta 2, 2006-08-28</h4>

<p>
Package considerably restructured and some new components added.
New examples (ControlledTankSystem, AST_BatchPlant).
</p>

<h4>Version 0.96, 2006-01-08</h4>

<ul>
<li> New package Modelica.Fluid.PressureLosses.</li>
<li> New package Modelica.Fluid.WorkInProgress.</li>
<li> New components in Modelica.Fluid.Components:<br>
     ShortPipe, OpenTank, ValveDiscrete, StaticHead.</li>
<li> New components in Modelica.Fluid.Examples.</li>
<li> Improved users guide.</li>
</ul>

<h4>Version 0.910, 2005-10-25</h4>
<ul>
<li> Changes as decided on 41th-45th Modelica Design Meetings
     (details, see minutes).
</ul>
<h4>Version 0.900, 2004-10-18</h4>
<ul>
<li> Changes as decided on 40th Modelica Design Meeting in Dresden
     (see also minutes)
</ul>
<h4>Version 0.794, 2004-05-31</h4>
<ul>
<li> Sensors.mo, Examples/DrumBoiler.mo: extend sensors with user choice
     for measurement unit.</li>
<li> Components.mo, Types.mo: moved components and types to
     package Examples.</li>
<li> Moved Examples from <b>file</b> Modelica.Fluid/package.mo to
     Modelica.Media/Examples <b>subdirectory</b> and created separate
     file per sub-package. This shall simplify the maintenance of
     examples by different authors</li>
<li> Moved Interfaces from file Modelica.Fluid/package.mo to
     Modelica.Fluid/Interfaces.mo</li>
</ul>
<h4>Version 0.793, 2004-05-18</h4>
<ul>
<li> Removed \"semiLinear\" function since available as
     Modelica 2.1 built-in operator in Dymola.</li>
<li> Minor bug in \"Components.ShortPipe\" corrected.</li>
<li> Bug in \"Components.Orifice\" corrected
     (dp was previously calculated in
      Interfaces.PartialTwoPortTransport,
      but this was removed and not updated in Orifice).</li>
</ul>
<h4>Version 0.792, 2003-11-07</h4>
<p>
This is the first consolidated version made up from
several changes for Modelica'2003.
Modelica.Fluid is still quite far away
from a library that could be included in the Modelica
standard library.
</p>
<h4>Previous Releases</h4>
<ul>
<li><i>Oct., 2003</i><br>
       by Martin Otter: Adapted to latest design of the Modelica.Media
       library.<br>
       by Ruediger Franke: Included sensor components and
       Modelica.Fluid.Examples.DrumBoiler example.</li>
<li><i>Sept., 2003</i><br>
       by Martin Otter: Changes according to the decisions of the
       Modelica design meeting in Dearborn, Sept. 2-4, 2003.
       Fluid library split into two packages: Modelica.Media
       that contains the media models and Modelica.Fluid that
       contains fluid flow components. Modelica.Media is
       independent of Modelica.Fluid and my be used also from
       other packages that may have a different design as
       Modelica.Fluid.</li>
<li><i>Aug., 2003</i><br>
       by Martin Otter: Improved documentation, PortVicinity (now called semiLinear)
       manually expanded, two different volume types,
       replaced number of massFractions from n to n-1 in order
       that usage of model for single substances is easier
       and in order that no special cases have to be treated
       in the equations (previously the massFraction equations had to
       be removed for single substance flow; now they are removed
       automatically, since the dimensions are zero, and not one
       as previously), included asserts to check the validity of
       the medium models, included the dynamic viscosity in the
       medium models, adapted the examples and medium models to the
       changes in Interfaces, improved menus according to the new
       features in Dymola 5.1. Added \"Components.ShortPipe\" that
       contains a detailed model of the frictional losses in pipes
       over a very wide range.</li>
<li><i>Feb., 2003</i><br>
       by Martin Otter: Included several elementary components and
       a model for moist air. Some elementary components, such as
       FixedAmbient, are adapted versions from the SimpleFlow fluid library
       of Anton Haumer.</li>
<li><i>Dec., 2002</i><br>
       by Hubertus Tummescheit:
       Improved version of the high precision water model
       (Copy from ThermoFluid library, code reorganization,
       enhanced documentation, additional functions).</li>
<li><i>Nov. 30, 2002</i><br>
       by Martin Otter: Improved the design from the design meeting:
       Adapted to Modelica standard library 1.5,
       added \"choicesAllMatching=true\" annotation,
       added short documentation to \"Interfaces\",
       added packages \"Examples\" and \"Media\" (previously called \"Properties\")
       from previous versions and adapted them to the updated
       \"Interfaces\" package.</li>
<li><i>Nov. 20-21, 2002</i><br>
       by Hilding Elmqvist, Mike Tiller, Allan Watson, John Batteh, Chuck Newman,
       Jonas Eborn: Improved at the 32nd Modelica Design Meeting.</li>
<li><i>Nov. 11, 2002</i><br>
       by Hilding Elmqvist, Martin Otter: improved version.</li>
<li><i>Nov. 6, 2002</i><br>
       by Hilding Elmqvist: first version of the basic design.</li>
</ul>
</html>"));
      end ReleaseNotes;

      class Contact "Contact"
        extends Modelica.Icons.Contact;
        annotation(
          Documentation(info = "<html>
<dl>
<dt><b>Library Officers:</b><br>&nbsp;</dt>
<dd>
<table border=0 cellspacing=0 cellpadding=2>
<tr>
<td>
    Francesco Casella<br>
    Dipartimento di Elettronica e Informazione<br>
    Politecnico di Milano<br>
    Via Ponzio 34/5<br>
    I-20133 Milano, Italy<br>
    email: <A HREF=\"mailto:casella@elet.polimi.it\">casella@elet.polimi.it</A><br>&nbsp;
</td>
<td></td>
<td>and</td>
<td></td>
<td>
    R&uuml;diger Franke<br>
    ABB AG<br>
    PTSP-E22<br>
    Kallstadter Str. 1<br>
    D-68163, Germany<br>
    email: <A HREF=\"mailto:ruediger.franke@de.abb.com\">ruediger.franke@de.abb.com</A><br>&nbsp;
</td>
</tr>
</table>
</dd>
</dl>
<p><b>Acknowledgements:</b></p>
<p>
The development of this library has been a collaborative effort
and many have contributed.
</p>
<ul>
<li>The previous design of this library (until beginning of 2008) was based on the paper
     Elmqvist H., Tummescheit H., and Otter M.:
     <a href=\"https://www.modelica.org/events/Conference2003/papers/h40_Elmqvist_fluid.pdf\">Object-Oriented Modeling of Thermo-Fluid Systems</a>.
     Modelica 2003 Conference, Link&ouml;ping, Sweden, pp. 269-286, Nov. 3-4, 2003.<br>
     This design has been partly changed, especially by the introduction of the streams
     concept.</li>

<li>The Fluid library development was organized in 2002-2004 by Martin
     Otter, since 2004 it is organized by Francesco Casella, and
     since 2008 it is organized jointly by Francesco Casella and R&uuml;diger Franke.</li>

<li>Francesco Casella included several components of his ThermoPower
     library with some rewriting. The stream connector concept used in Modelica.Fluid is
     based on a similar concept developed by him for the ThermoPower library.</li>

<li>R&uuml;diger Franke initiated the stream connector concept as an extension
     and improved version of the ThermoPower concept. In Nov. 2008 - Jan. 2009 he
     greatly restructured and improved the library.</li>

<li>Michael Wetter introduced trace constituents in Modelica.Fluid consistently and
     provided corresponding examples under Examples.TraceSubstances.</li>

<li>The following people contributed to the fluid component models,
     examples, and the further design of the library
     (alphabetical list):<br>
     John Batteh,
     Francesco Casella, Jonas Eborn, Hilding Elmqvist,
     R&uuml;diger Franke, Manuel Gr&auml;ber, Henning Knigge,
     Sven Erik Mattsson, Chuck Newman, Hans Olsson,
     Martin Otter, Katrin Pr&ouml;l&szlig;,
     Christoph Richter, Michael Sielemann, Mike Tiller, Hubertus Tummescheit,
     Allan Watson, Michael Wetter.</li>
</ul>

<p>
  Partial financial support of ABB and DLR by BMBF (BMBF F&ouml;rderkennzeichen: 01IS07022F) for the further development
     of this library within the <a href=\"http://www.itea2.org\">ITEA</a> project
     <a href=\"http://www.itea2.org/public/project_leaflets/EUROSYSLIB_profile_oct-07.pdf\">EUROSYSLIB</a>
     is highly appreciated.</p>

</html>"));
      end Contact;
      annotation(
        DocumentationClass = true,
        Documentation(info = "<html>
<p>
Library <b>Modelica.Fluid</b> is a <b>free</b> Modelica package providing components for
<b>1-dimensional thermo-fluid flow</b> in networks of pipes. A unique feature is that the
component equations and the media models
as well as pressure loss and heat transfer correlations are decoupled from each other.
All components are implemented such that they can be used for
media from the Modelica.Media library. This means especially that an
incompressible or compressible medium, a single or a multiple
substance medium with one or more phases might be used.
</p>
</html>"));
    end UsersGuide;

    package Examples "Demonstration of the usage of the library"
      extends Modelica.Icons.ExamplesPackage;

      model PumpingSystem "Model of a pumping system for drinking water"
        extends Modelica.Icons.Example;
        replaceable package Medium = Modelica.Media.Water.StandardWaterOnePhase constrainedby Modelica.Media.Interfaces.PartialMedium;
        //replaceable package Medium = Modelica.Media.Water.ConstantPropertyLiquidWater
        //  constrainedby Modelica.Media.Interfaces.PartialMedium;
        Modelica.Fluid.Sources.FixedBoundary source(nPorts = 1, use_T = true, T = Modelica.SIunits.Conversions.from_degC(20), p = system.p_ambient, redeclare package Medium = Medium) annotation(
          Placement(transformation(extent = {{-100, -80}, {-80, -60}})));
        Modelica.Fluid.Pipes.StaticPipe pipe(allowFlowReversal = true, length = 100, height_ab = 50, diameter = 0.3, redeclare package Medium = Medium) annotation(
          Placement(transformation(origin = {-30, -51}, extent = {{-9, -10}, {11, 10}}, rotation = 90)));
        Machines.PrescribedPump pumps(checkValve = true, N_nominal = 1200, redeclare function flowCharacteristic = Modelica.Fluid.Machines.BaseClasses.PumpCharacteristics.quadraticFlow(V_flow_nominal = {0, 0.25, 0.5}, head_nominal = {100, 60, 0}), use_N_in = true, nParallel = 1, energyDynamics = Modelica.Fluid.Types.Dynamics.FixedInitial, V(displayUnit = "l") = 0.05, massDynamics = Modelica.Fluid.Types.Dynamics.FixedInitial, redeclare package Medium = Medium, p_b_start = 600000, T_start = system.T_start) annotation(
          Placement(transformation(extent = {{-68, -80}, {-48, -60}})));
        Modelica.Fluid.Vessels.OpenTank reservoir(T_start = Modelica.SIunits.Conversions.from_degC(20), use_portsData = true, crossArea = 50, level_start = 2.2, height = 3, nPorts = 3, portsData = {Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.3), Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.3), Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.01)}, redeclare package Medium = Medium) annotation(
          Placement(transformation(extent = {{-20, -16}, {0, 4}})));
        Modelica.Fluid.Valves.ValveLinear userValve(allowFlowReversal = false, dp_nominal = 200000, m_flow_nominal = 400, redeclare package Medium = Medium) annotation(
          Placement(transformation(extent = {{58, -38}, {74, -22}})));
        Modelica.Fluid.Sources.FixedBoundary sink(p = system.p_ambient, T = system.T_ambient, nPorts = 2, redeclare package Medium = Medium) annotation(
          Placement(transformation(extent = {{100, -40}, {80, -20}})));
        Modelica.Blocks.Sources.Step valveOpening(startTime = 200, offset = 1e-6) annotation(
          Placement(transformation(extent = {{56, 0}, {76, 20}})));
        Modelica.Blocks.Sources.Constant RelativePressureSetPoint(k = 2e4) annotation(
          Placement(transformation(extent = {{-100, 60}, {-80, 80}})));
        Modelica.Blocks.Logical.OnOffController controller(bandwidth = 4000, pre_y_start = false) annotation(
          Placement(transformation(extent = {{-40, 60}, {-20, 80}})));
        Modelica.Blocks.Logical.TriggeredTrapezoid PumpRPMGenerator(rising = 3, falling = 3, amplitude = 1200, offset = 0.001) annotation(
          Placement(transformation(extent = {{0, 60}, {20, 80}})));
        Modelica.Fluid.Sensors.RelativePressure reservoirPressure(redeclare package Medium = Medium) annotation(
          Placement(transformation(extent = {{10, -12}, {30, -32}})));
        Modelica.Blocks.Continuous.FirstOrder PT1(T = 2, initType = Modelica.Blocks.Types.Init.InitialState, y_start = 0) annotation(
          Placement(transformation(extent = {{40, 60}, {60, 80}})));
        inner Modelica.Fluid.System system(energyDynamics = Modelica.Fluid.Types.Dynamics.FixedInitial) annotation(
          Placement(transformation(extent = {{60, -96}, {80, -76}})));
      equation
        connect(userValve.port_b, sink.ports[1]) annotation(
          Line(points = {{74, -30}, {77, -30}, {77, -28}, {80, -28}}, color = {0, 127, 255}));
        connect(source.ports[1], pumps.port_a) annotation(
          Line(points = {{-80, -70}, {-74, -70}, {-68, -70}}, color = {0, 127, 255}));
        connect(valveOpening.y, userValve.opening) annotation(
          Line(points = {{77, 10}, {98, 10}, {98, -12}, {66, -12}, {66, -23.6}}, color = {0, 0, 127}));
        connect(RelativePressureSetPoint.y, controller.reference) annotation(
          Line(points = {{-79, 70}, {-60, 70}, {-60, 76}, {-42, 76}}, color = {0, 0, 127}));
        connect(controller.y, PumpRPMGenerator.u) annotation(
          Line(points = {{-19, 70}, {-2, 70}}, color = {255, 0, 255}));
        connect(reservoirPressure.p_rel, controller.u) annotation(
          Line(points = {{20, -13}, {20, 50}, {-52, 50}, {-52, 64}, {-42, 64}}, color = {0, 0, 127}));
        connect(reservoirPressure.port_b, sink.ports[2]) annotation(
          Line(points = {{30, -22}, {44, -22}, {44, -48}, {80, -48}, {80, -32}}, color = {0, 127, 255}, pattern = LinePattern.Dot));
        connect(PumpRPMGenerator.y, PT1.u) annotation(
          Line(points = {{21, 70}, {38, 70}}, color = {0, 0, 127}));
        connect(PT1.y, pumps.N_in) annotation(
          Line(points = {{61, 70}, {74, 70}, {74, 30}, {-58, 30}, {-58, -60}}, color = {0, 0, 127}));
        connect(pipe.port_a, pumps.port_b) annotation(
          Line(points = {{-30, -60}, {-30, -70}, {-48, -70}}, color = {0, 127, 255}));
        connect(reservoir.ports[1], pipe.port_b) annotation(
          Line(points = {{-12.6667, -16}, {-12.6667, -30}, {-30, -30}, {-30, -40}}, color = {0, 127, 255}));
        connect(reservoir.ports[3], reservoirPressure.port_a) annotation(
          Line(points = {{-7.33333, -16}, {-7, -16}, {-7, -22}, {10, -22}}, color = {0, 127, 255}, pattern = LinePattern.Dot));
        connect(reservoir.ports[2], userValve.port_a) annotation(
          Line(points = {{-10, -16}, {-10, -30}, {58, -30}}, color = {0, 127, 255}));
        annotation(
          Documentation(info = "<html>
<p>
Water is pumped from a source by a pump (fitted with check valves), through a pipe whose outlet is 50 m higher than the source, into a reservoir. The users are represented by an equivalent valve, connected to the reservoir.
</p>
<p>
The water controller is a simple on-off controller, regulating on the gauge pressure measured at the base of the tower; the output of the controller is the rotational speed of the pump, which is represented by the output of a first-order system. A small but nonzero rotational speed is used to represent the standby state of the pumps, in order to avoid singularities in the flow characteristic.
</p>
<p>
Simulate for 2000 s. When the valve is opened at time t=200, the pump starts turning on and off to keep the reservoir level around 2 meters, which roughly corresponds to a gauge pressure of 200 mbar.
</p>
<p>
If using Dymola, turn off \"Equidistant time grid\" to avoid numerical errors.
</p>

<img src=\"modelica://Modelica/Resources/Images/Fluid/Examples/PumpingSystem.png\" border=\"1\"
     alt=\"PumpingSystem.png\">
</html>", revisions = "<html>
<ul>
<li><i>Jan 2009</i>
    by R&uuml;diger Franke:<br>
       Reduce diameters of pipe and reservoir ports; use separate port for measurement of reservoirPressure, avoiding disturbances due to pressure losses.</li>
<li><i>1 Oct 2007</i>
    by <a href=\"mailto:francesco.casella@polimi.it\">Francesco Casella</a>:<br>
       Parameters updated.</li>
<li><i>2 Nov 2005</i>
    by <a href=\"mailto:francesco.casella@polimi.it\">Francesco Casella</a>:<br>
       Created.</li>
</ul>
</html>"),
          experiment(StopTime = 2000, Interval = 0.4, Tolerance = 1e-006));
      end PumpingSystem;

      model HeatingSystem "Simple model of a heating system"
        extends Modelica.Icons.Example;
        replaceable package Medium = Modelica.Media.Water.StandardWater constrainedby Modelica.Media.Interfaces.PartialMedium;
        Modelica.Fluid.Vessels.OpenTank tank(redeclare package Medium = Medium, crossArea = 0.01, height = 2, level_start = 1, nPorts = 2, massDynamics = Modelica.Fluid.Types.Dynamics.FixedInitial, use_HeatTransfer = true, portsData = {Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.01), Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.01)}, redeclare model HeatTransfer = Modelica.Fluid.Vessels.BaseClasses.HeatTransfer.IdealHeatTransfer(k = 10), ports(each p(start = 1e5)), T_start = Modelica.SIunits.Conversions.from_degC(20)) annotation(
          Placement(transformation(extent = {{-80, 30}, {-60, 50}})));
        Machines.ControlledPump pump(redeclare package Medium = Medium, N_nominal = 1500, use_T_start = true, T_start = Modelica.SIunits.Conversions.from_degC(40), m_flow_start = 0.01, m_flow_nominal = 0.01, control_m_flow = false, allowFlowReversal = false, p_a_start = 110000, p_b_start = 130000, p_a_nominal = 110000, p_b_nominal = 130000) annotation(
          Placement(transformation(extent = {{-50, 10}, {-30, 30}})));
        Modelica.Fluid.Valves.ValveIncompressible valve(redeclare package Medium = Medium, CvData = Modelica.Fluid.Types.CvTypes.OpPoint, m_flow_nominal = 0.01, show_T = true, allowFlowReversal = false, dp_start = 18000, dp_nominal = 10000) annotation(
          Placement(transformation(extent = {{60, -80}, {40, -60}})));
      protected
        Modelica.Blocks.Interfaces.RealOutput m_flow annotation(
          Placement(transformation(extent = {{-6, 34}, {6, 46}})));
      public
        Sensors.MassFlowRate sensor_m_flow(redeclare package Medium = Medium) annotation(
          Placement(transformation(extent = {{-20, 10}, {0, 30}})));
        Modelica.Thermal.HeatTransfer.Sources.FixedTemperature T_ambient(T = system.T_ambient) annotation(
          Placement(transformation(extent = {{-14, -27}, {0, -13}})));
        Modelica.Thermal.HeatTransfer.Components.ThermalConductor wall(G = 1.6e3 / 20) annotation(
          Placement(transformation(origin = {10, -48}, extent = {{8, -10}, {-8, 10}}, rotation = 90)));
        Modelica.Thermal.HeatTransfer.Sources.FixedHeatFlow burner(Q_flow = 1.6e3, T_ref = 343.15, alpha = -0.5) annotation(
          Placement(transformation(extent = {{16, 30}, {36, 50}})));
        inner Modelica.Fluid.System system(m_flow_small = 1e-4, energyDynamics = Modelica.Fluid.Types.Dynamics.SteadyStateInitial) annotation(
          Placement(transformation(extent = {{-90, 70}, {-70, 90}})));
        Pipes.DynamicPipe heater(redeclare package Medium = Medium, use_T_start = true, T_start = Modelica.SIunits.Conversions.from_degC(80), length = 2, redeclare model HeatTransfer = Modelica.Fluid.Pipes.BaseClasses.HeatTransfer.IdealFlowHeatTransfer, diameter = 0.01, nNodes = 1, redeclare model FlowModel = Modelica.Fluid.Pipes.BaseClasses.FlowModels.DetailedPipeFlow, use_HeatTransfer = true, modelStructure = Modelica.Fluid.Types.ModelStructure.a_v_b, p_a_start = 130000) annotation(
          Placement(transformation(extent = {{30, 10}, {50, 30}})));
        Pipes.DynamicPipe radiator(use_T_start = true, redeclare package Medium = Medium, length = 10, T_start = Modelica.SIunits.Conversions.from_degC(40), redeclare model HeatTransfer = Modelica.Fluid.Pipes.BaseClasses.HeatTransfer.IdealFlowHeatTransfer, diameter = 0.01, nNodes = 1, redeclare model FlowModel = Modelica.Fluid.Pipes.BaseClasses.FlowModels.DetailedPipeFlow, use_HeatTransfer = true, modelStructure = Modelica.Fluid.Types.ModelStructure.a_v_b, p_a_start = 110000, state_a(p(start = 110000)), state_b(p(start = 110000))) annotation(
          Placement(transformation(extent = {{20, -80}, {0, -60}})));
      protected
        Modelica.Blocks.Interfaces.RealOutput T_forward annotation(
          Placement(transformation(extent = {{74, 34}, {86, 46}})));
        Modelica.Blocks.Interfaces.RealOutput T_return annotation(
          Placement(transformation(extent = {{-46, -56}, {-58, -44}})));
      public
        Modelica.Fluid.Sensors.Temperature sensor_T_forward(redeclare package Medium = Medium) annotation(
          Placement(transformation(extent = {{50, 30}, {70, 50}})));
        Modelica.Fluid.Sensors.Temperature sensor_T_return(redeclare package Medium = Medium) annotation(
          Placement(transformation(extent = {{-20, -60}, {-40, -40}})));
      protected
        Modelica.Blocks.Interfaces.RealOutput tankLevel annotation(
          Placement(transformation(extent = {{-56, 34}, {-44, 46}})));
      public
        Modelica.Blocks.Sources.Step handle(startTime = 2000, height = 0.9, offset = 0.1) annotation(
          Placement(transformation(extent = {{26, -27}, {40, -13}})));
        Pipes.DynamicPipe pipe(redeclare package Medium = Medium, use_T_start = true, T_start = Modelica.SIunits.Conversions.from_degC(80), redeclare model HeatTransfer = Modelica.Fluid.Pipes.BaseClasses.HeatTransfer.IdealFlowHeatTransfer, diameter = 0.01, redeclare model FlowModel = Modelica.Fluid.Pipes.BaseClasses.FlowModels.DetailedPipeFlow, length = 10, p_a_start = 130000) annotation(
          Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = -90, origin = {80, -20})));
      equation
        tankLevel = tank.level;
        connect(sensor_m_flow.m_flow, m_flow) annotation(
          Line(points = {{-10, 31}, {-10, 40}, {0, 40}}, color = {0, 0, 127}));
        connect(sensor_m_flow.port_b, heater.port_a) annotation(
          Line(points = {{0, 20}, {0, 20}, {30, 20}}, color = {0, 127, 255}));
        connect(T_ambient.port, wall.port_a) annotation(
          Line(points = {{0, -20}, {10, -20}, {10, -40}}, color = {191, 0, 0}));
        connect(sensor_T_forward.T, T_forward) annotation(
          Line(points = {{67, 40}, {80, 40}}, color = {0, 0, 127}));
        connect(radiator.port_a, valve.port_b) annotation(
          Line(points = {{20, -70}, {20, -70}, {40, -70}}, color = {0, 127, 255}));
        connect(sensor_T_return.port, radiator.port_b) annotation(
          Line(points = {{-30, -60}, {-30, -70}, {0, -70}}, color = {0, 127, 255}));
        connect(tank.ports[2], pump.port_a) annotation(
          Line(points = {{-68, 30}, {-68, 20}, {-50, 20}}, color = {0, 127, 255}));
        connect(handle.y, valve.opening) annotation(
          Line(points = {{40.7, -20}, {50, -20}, {50, -62}}, color = {0, 0, 127}));
        connect(pump.port_b, sensor_m_flow.port_a) annotation(
          Line(points = {{-30, 20}, {-20, 20}}, color = {0, 127, 255}));
        connect(sensor_T_return.T, T_return) annotation(
          Line(points = {{-37, -50}, {-52, -50}}, color = {0, 0, 127}));
        connect(burner.port, heater.heatPorts[1]) annotation(
          Line(points = {{36, 40}, {40.1, 40}, {40.1, 24.4}}, color = {191, 0, 0}));
        connect(wall.port_b, radiator.heatPorts[1]) annotation(
          Line(points = {{10, -56}, {10, -65.6}, {9.9, -65.6}}, color = {191, 0, 0}));
        connect(sensor_T_forward.port, heater.port_b) annotation(
          Line(points = {{60, 30}, {60, 20}, {50, 20}}, color = {0, 127, 255}));
        connect(heater.port_b, pipe.port_a) annotation(
          Line(points = {{50, 20}, {80, 20}, {80, -10}}, color = {0, 127, 255}));
        connect(pipe.port_b, valve.port_a) annotation(
          Line(points = {{80, -30}, {80, -70}, {60, -70}}, color = {0, 127, 255}));
        connect(radiator.port_b, tank.ports[1]) annotation(
          Line(points = {{0, -70}, {-72, -70}, {-72, 30}}, color = {0, 127, 255}));
        annotation(
          Documentation(info = "<html>
<p>
Simple heating system with a closed flow cycle.
After 2000s of simulation time the valve fully opens. A simple idealized control is embedded
into the respective components, so that the heating system can be regulated with the valve:
the pump controls the pressure, the burner controls the temperature.
</p>
<p>
One can investigate the temperatures and flows for different settings of <code>system.energyDynamics</code>
(see Assumptions tab of the system object).</p>
<ul>
<li>With <code>system.energyDynamics==Types.Dynamics.FixedInitial</code> the states need to find their steady values during the simulation.</li>
<li>With <code>system.energyDynamics==Types.Dynamics.SteadyStateInitial</code> (default setting) the simulation starts in steady-state.</li>
<li>With <code>system.energyDynamics==Types.Dynamics.SteadyState</code> all but one dynamic states are eliminated.
    The left state <code>tank.m</code> is to account for the closed flow cycle. It is constant as outflow and inflow are equal
    in a steady-state simulation.</li>
</ul>
<p>
Note that a closed flow cycle generally causes circular equalities for the mass flow rates and leaves the pressure undefined.
This is why the tank.massDynamics, i.e., the tank level determining the port pressure, is modified locally to Types.Dynamics.FixedInitial.
</p>
<p>
Also note that the tank is thermally isolated against its ambient. This way the temperature of the tank is also
well defined for zero flow rate in the heating system, e.g., for valveOpening.offset=0 at the beginning of a simulation.
The pipe however is assumed to be perfectly isolated.
If steady-state values shall be obtained with the valve fully closed, then a thermal
coupling between the pipe and its ambient should be defined as well.
</p>
<p>
Moreover it is worth noting that the idealized direct connection between the heater and the pipe, resulting in equal port pressures,
is treated as high-index DAE, as opposed to a nonlinear equation system for connected pressure loss correlations. A pressure loss correlation
could be additionally introduced to model the fitting between the heater and the pipe, e.g., to adapt different diameters.
</p>

<img src=\"modelica://Modelica/Resources/Images/Fluid/Examples/HeatingSystem.png\" border=\"1\"
     alt=\"HeatingSystem.png\">
</html>"),
          experiment(StopTime = 6000),
          __Dymola_Commands(file(ensureSimulated = true) = "modelica://Modelica/Resources/Scripts/Dymola/Fluid/HeatingSystem/plotResults.mos" "plotResults"));
      end HeatingSystem;

      package DrumBoiler "Drum boiler example, see Franke, Rode, Krueger: On-line Optimization of Drum Boiler Startup, 3rd International Modelica Conference, Linkoping, 2003"
        extends Modelica.Icons.ExamplesPackage;

        model DrumBoiler "Complete drum boiler model, including evaporator and supplementary components"
          extends Modelica.Icons.Example;
          parameter Boolean use_inputs = false "use external inputs instead of test data contained internally";
          Modelica.Fluid.Examples.DrumBoiler.BaseClasses.EquilibriumDrumBoiler evaporator(m_D = 300e3, cp_D = 500, V_t = 100, V_l_start = 67, redeclare package Medium = Modelica.Media.Water.StandardWater, energyDynamics = Modelica.Fluid.Types.Dynamics.FixedInitial, massDynamics = Modelica.Fluid.Types.Dynamics.FixedInitial, p_start = 100000) annotation(
            Placement(transformation(extent = {{-46, -30}, {-26, -10}})));
          Modelica.Thermal.HeatTransfer.Sources.PrescribedHeatFlow furnace annotation(
            Placement(transformation(origin = {-36, -53}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
          Modelica.Fluid.Sources.FixedBoundary sink(nPorts = 1, p = Cv.from_bar(0.5), redeclare package Medium = Modelica.Media.Water.StandardWaterOnePhase, T = 500) annotation(
            Placement(transformation(origin = {90, -20}, extent = {{-10, -10}, {10, 10}}, rotation = 180)));
          Modelica.Fluid.Sensors.MassFlowRate massFlowRate(redeclare package Medium = Modelica.Media.Water.StandardWater) annotation(
            Placement(transformation(origin = {30, -20}, extent = {{10, 10}, {-10, -10}}, rotation = 180)));
          Modelica.Fluid.Sensors.Temperature temperature(redeclare package Medium = Modelica.Media.Water.StandardWater) annotation(
            Placement(transformation(origin = {-3, -1}, extent = {{10, 10}, {-10, -10}}, rotation = 180)));
          Modelica.Fluid.Sensors.Pressure pressure(redeclare package Medium = Modelica.Media.Water.StandardWater) annotation(
            Placement(transformation(extent = {{10, 18}, {30, 38}})));
          Modelica.Blocks.Continuous.PI controller(T = 120, k = 10, initType = Modelica.Blocks.Types.Init.InitialState) annotation(
            Placement(transformation(extent = {{-49, 23}, {-63, 37}})));
          Modelica.Fluid.Sources.MassFlowSource_h pump(nPorts = 1, h = 5e5, redeclare package Medium = Modelica.Media.Water.StandardWater, use_m_flow_in = true) annotation(
            Placement(transformation(extent = {{-80, -30}, {-60, -10}})));
          Modelica.Blocks.Math.Feedback feedback annotation(
            Placement(transformation(extent = {{-22, 20}, {-42, 40}})));
          Modelica.Blocks.Sources.Constant levelSetPoint(k = 67) annotation(
            Placement(transformation(extent = {{-38, 48}, {-24, 62}})));
          Modelica.Blocks.Interfaces.RealOutput T_S "steam temperature" annotation(
            Placement(transformation(extent = {{100, 48}, {112, 60}})));
          Modelica.Blocks.Interfaces.RealOutput p_S "steam pressure" annotation(
            Placement(transformation(extent = {{100, 22}, {112, 34}})));
          Modelica.Blocks.Interfaces.RealOutput qm_S "steam flow rate" annotation(
            Placement(transformation(extent = {{100, -2}, {112, 10}})));
          Modelica.Blocks.Interfaces.RealOutput V_l "liquid volume inside drum" annotation(
            Placement(transformation(extent = {{100, 74}, {112, 86}})));
        public
          Modelica.Blocks.Math.Gain MW2W(k = 1e6) annotation(
            Placement(transformation(extent = {{-54, -75.5}, {-44, -64.5}})));
          Modelica.Blocks.Math.Gain Pa2bar(k = 1e-5) annotation(
            Placement(transformation(extent = {{37, 23}, {47, 33}})));
          Modelica.Thermal.HeatTransfer.Celsius.FromKelvin K2degC annotation(
            Placement(transformation(extent = {{38, 49}, {48, 59}})));
          Modelica.Blocks.Nonlinear.Limiter limiter(uMin = 0, uMax = 500) annotation(
            Placement(transformation(origin = {-78, 30}, extent = {{-7, 7}, {7, -7}}, rotation = 180)));
          Modelica.Fluid.Valves.ValveLinear SteamValve(redeclare package Medium = Modelica.Media.Water.StandardWater, dp_nominal = 9000000, m_flow_nominal = 180) annotation(
            Placement(transformation(extent = {{50, -10}, {70, -30}})));
          inner Modelica.Fluid.System system annotation(
            Placement(transformation(extent = {{-90, 70}, {-70, 90}})));
          Modelica.Blocks.Sources.TimeTable q_F_Tab(table = [0, 0; 3600, 400; 7210, 400]) if not use_inputs annotation(
            Placement(transformation(extent = {{-90, -80}, {-70, -60}})));
          Modelica.Blocks.Sources.TimeTable Y_Valve_Tab(table = [0, 0; 900, 1; 7210, 1]) if not use_inputs annotation(
            Placement(transformation(extent = {{30, -80}, {50, -60}})));
          Blocks.Interfaces.RealInput q_F(unit = "MW") if use_inputs "fuel flow rate" annotation(
            Placement(transformation(extent = {{-112, -56}, {-100, -44}})));
          Blocks.Interfaces.RealInput Y_Valve if use_inputs "valve opening" annotation(
            Placement(transformation(extent = {{-112, -96}, {-100, -84}})));
        equation
          connect(furnace.port, evaporator.heatPort) annotation(
            Line(points = {{-36, -43}, {-36, -30}}, color = {191, 0, 0}));
          connect(controller.u, feedback.y) annotation(
            Line(points = {{-47.6, 30}, {-41, 30}}, color = {0, 0, 127}));
          connect(massFlowRate.m_flow, qm_S) annotation(
            Line(points = {{30, -9}, {30, 4}, {106, 4}}, color = {0, 0, 127}));
          connect(evaporator.V, V_l) annotation(
            Line(points = {{-32, -9}, {-32, 16}, {-4, 16}, {-4, 80}, {106, 80}}, color = {0, 0, 127}));
          connect(MW2W.y, furnace.Q_flow) annotation(
            Line(points = {{-43.5, -70}, {-36, -70}, {-36, -63}}, color = {0, 0, 127}));
          connect(pressure.p, Pa2bar.u) annotation(
            Line(points = {{31, 28}, {36, 28}}, color = {0, 0, 127}));
          connect(Pa2bar.y, p_S) annotation(
            Line(points = {{47.5, 28}, {106, 28}}, color = {0, 0, 127}));
          connect(K2degC.Celsius, T_S) annotation(
            Line(points = {{48.5, 54}, {106, 54}}, color = {0, 0, 127}));
          connect(controller.y, limiter.u) annotation(
            Line(points = {{-63.7, 30}, {-69.6, 30}}, color = {0, 0, 127}));
          connect(limiter.y, pump.m_flow_in) annotation(
            Line(points = {{-85.7, 30}, {-90, 30}, {-90, -12}, {-80, -12}}, color = {0, 0, 127}));
          connect(temperature.T, K2degC.Kelvin) annotation(
            Line(points = {{4, -1}, {4, -1}, {8, -1}, {8, 54}, {37, 54}}, color = {0, 0, 127}));
          connect(pressure.port, massFlowRate.port_a) annotation(
            Line(points = {{20, 18}, {20, -20}}, color = {0, 127, 255}));
          connect(pump.ports[1], evaporator.port_a) annotation(
            Line(points = {{-60, -20}, {-46, -20}}, color = {0, 127, 255}));
          connect(massFlowRate.port_b, SteamValve.port_a) annotation(
            Line(points = {{40, -20}, {50, -20}}, color = {0, 127, 255}));
          connect(SteamValve.port_b, sink.ports[1]) annotation(
            Line(points = {{70, -20}, {75, -20}, {80, -20}}, color = {0, 127, 255}));
          connect(evaporator.port_b, massFlowRate.port_a) annotation(
            Line(points = {{-26, -20}, {20, -20}}, color = {0, 127, 255}));
          connect(temperature.port, massFlowRate.port_a) annotation(
            Line(points = {{-3, -11}, {-3, -20}, {20, -20}}, color = {0, 127, 255}));
          connect(q_F_Tab.y, MW2W.u) annotation(
            Line(points = {{-69, -70}, {-55, -70}}, color = {0, 0, 127}));
          connect(Y_Valve_Tab.y, SteamValve.opening) annotation(
            Line(points = {{51, -70}, {60, -70}, {60, -28}}, color = {0, 0, 127}));
          connect(q_F, MW2W.u) annotation(
            Line(points = {{-106, -50}, {-62, -50}, {-62, -70}, {-55, -70}}, color = {0, 0, 127}));
          connect(Y_Valve, SteamValve.opening) annotation(
            Line(points = {{-106, -90}, {60, -90}, {60, -28}}, color = {0, 0, 127}));
          connect(evaporator.V, feedback.u2) annotation(
            Line(points = {{-32, -9}, {-32, 22}}, color = {0, 0, 127}));
          connect(levelSetPoint.y, feedback.u1) annotation(
            Line(points = {{-23.3, 55}, {-16, 55}, {-16, 30}, {-24, 30}}, color = {0, 0, 127}));
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-151, 165}, {138, 102}}, lineColor = {0, 0, 255}, textString = "%name"), Text(extent = {{-79, 67}, {67, 21}}, lineColor = {0, 0, 0}, textString = "drum"), Text(extent = {{-90, -14}, {88, -64}}, lineColor = {0, 0, 0}, textString = "boiler")}),
            experiment(StopTime = 5400),
            Documentation(info = "<html>

<img src=\"modelica://Modelica/Resources/Images/Fluid/Examples/DrumBoiler.png\" border=\"1\"
     alt=\"DrumBoiler.png\">
</html>"));
        end DrumBoiler;

        package BaseClasses "Additional components for drum boiler example"
          extends Modelica.Icons.BasesPackage;

          model EquilibriumDrumBoiler "Simple Evaporator with two states, see Astroem, Bell: Drum-boiler dynamics, Automatica 36, 2000, pp.363-378"
            extends Modelica.Fluid.Interfaces.PartialTwoPort(final port_a_exposesState = true, final port_b_exposesState = true, redeclare replaceable package Medium = Modelica.Media.Water.StandardWater constrainedby Modelica.Media.Interfaces.PartialTwoPhaseMedium);
            import Modelica.Constants;
            import Modelica.Fluid.Types;
            parameter SI.Mass m_D "mass of surrounding drum metal";
            parameter Medium.SpecificHeatCapacity cp_D "specific heat capacity of drum metal";
            parameter SI.Volume V_t "total volume inside drum";
            parameter Medium.AbsolutePressure p_start = system.p_start "Start value of pressure" annotation(
              Dialog(tab = "Initialization"));
            parameter SI.Volume V_l_start = V_t / 2 "Start value of liquid volumeStart value of volume" annotation(
              Dialog(tab = "Initialization"));
            // Assumptions
            parameter Boolean allowFlowReversal = system.allowFlowReversal "allow flow reversal, false restricts to design direction (port_a -> port_b)" annotation(
              Dialog(tab = "Assumptions"),
              Evaluate = true);
            parameter Types.Dynamics energyDynamics = system.energyDynamics "Formulation of energy balance" annotation(
              Evaluate = true,
              Dialog(tab = "Assumptions", group = "Dynamics"));
            parameter Types.Dynamics massDynamics = system.massDynamics "Formulation of mass balance" annotation(
              Evaluate = true,
              Dialog(tab = "Assumptions", group = "Dynamics"));
            Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a heatPort annotation(
              Placement(transformation(extent = {{-10, -110}, {10, -90}})));
            Modelica.Blocks.Interfaces.RealOutput V "liquid volume" annotation(
              Placement(transformation(origin = {40, 110}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
            Medium.SaturationProperties sat "State vector to compute saturation properties";
            Medium.AbsolutePressure p(start = p_start, stateSelect = StateSelect.prefer) "pressure inside drum boiler";
            Medium.Temperature T "temperature inside drum boiler";
            SI.Volume V_v "volume of vapour phase";
            SI.Volume V_l(start = V_l_start, stateSelect = StateSelect.prefer) "volumes of liquid phase";
            Medium.SpecificEnthalpy h_v = Medium.dewEnthalpy(sat) "specific enthalpy of vapour";
            Medium.SpecificEnthalpy h_l = Medium.bubbleEnthalpy(sat) "specific enthalpy of liquid";
            Medium.Density rho_v = Medium.dewDensity(sat) "density in vapour phase";
            Medium.Density rho_l = Medium.bubbleDensity(sat) "density in liquid phase";
            SI.Mass m "total mass of drum boiler";
            SI.Energy U "internal energy";
            Medium.Temperature T_D = heatPort.T "temperature of drum";
            SI.HeatFlowRate q_F = heatPort.Q_flow "heat flow rate from furnace";
            Medium.SpecificEnthalpy h_W = inStream(port_a.h_outflow) "Feed water enthalpy (specific enthalpy close to feedwater port when mass flows in to the boiler)";
            Medium.SpecificEnthalpy h_S = inStream(port_b.h_outflow) "steam enthalpy (specific enthalpy close to steam port when mass flows in to the boiler)";
            SI.MassFlowRate qm_W = port_a.m_flow "feed water mass flow rate";
            SI.MassFlowRate qm_S = port_b.m_flow "steam mass flow rate";
            /*outer Modelica.Fluid.Components.FluidOptions fluidOptions
                "Global default options";*/
          equation
// balance equations
            m = rho_v * V_v + rho_l * V_l + m_D "Total mass";
            U = rho_v * V_v * h_v + rho_l * V_l * h_l - p * V_t + m_D * cp_D * T_D "Total energy";
            if massDynamics == Types.Dynamics.SteadyState then
              0 = qm_W + qm_S "Steady state mass balance";
            else
              der(m) = qm_W + qm_S "Dynamic mass balance";
            end if;
            if energyDynamics == Types.Dynamics.SteadyState then
              0 = q_F + port_a.m_flow * actualStream(port_a.h_outflow) + port_b.m_flow * actualStream(port_b.h_outflow) "Steady state energy balance";
            else
              der(U) = q_F + port_a.m_flow * actualStream(port_a.h_outflow) + port_b.m_flow * actualStream(port_b.h_outflow) "Dynamic energy balance";
            end if;
            V_t = V_l + V_v;
// Properties of saturated liquid and steam
            sat.psat = p;
            sat.Tsat = T;
            sat.Tsat = Medium.saturationTemperature(p);
// ideal heat transfer between metal and water
            T_D = T;
// boundary conditions at the ports
            port_a.p = p;
            port_a.h_outflow = h_l;
            port_b.p = p;
            port_b.h_outflow = h_v;
// liquid volume
            V = V_l;
// Check that two-phase equilibrium is actually possible
            assert(p < Medium.fluidConstants[1].criticalPressure - 10000, "Evaporator model requires subcritical pressure");
          initial equation
// Initial conditions
// Note: p represents the energy as it is constrained by T_sat
            if energyDynamics == Types.Dynamics.FixedInitial then
              p = p_start;
            elseif energyDynamics == Types.Dynamics.SteadyStateInitial then
              der(p) = 0;
            end if;
            if massDynamics == Types.Dynamics.FixedInitial then
              V_l = V_l_start;
            elseif energyDynamics == Types.Dynamics.SteadyStateInitial then
              der(V_l) = 0;
            end if;
            annotation(
              Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, 64}, {100, -64}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.Backward, fillColor = {135, 135, 135}), Rectangle(extent = {{-100, -44}, {100, 44}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {255, 255, 255}), Rectangle(extent = DynamicSelect({{-100, -44}, {100, 44}}, {{-100, -44}, {(-100) + 200 * V_l / V_t, 44}}), lineColor = {0, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {0, 127, 255}), Ellipse(extent = {{18, 0}, {48, -29}}, lineColor = {0, 0, 255}, pattern = LinePattern.None, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-1, 29}, {29, 0}}, lineColor = {0, 0, 255}, pattern = LinePattern.None, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Ellipse(extent = {{43, 31}, {73, 2}}, lineColor = {0, 0, 255}, pattern = LinePattern.None, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-31, 1}, {-1, -28}}, lineColor = {0, 0, 255}, pattern = LinePattern.None, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Ellipse(extent = {{50, 15}, {80, -14}}, lineColor = {0, 0, 255}, pattern = LinePattern.None, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-72, 25}, {-42, -4}}, lineColor = {0, 0, 255}, pattern = LinePattern.None, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Ellipse(extent = {{71, -11}, {101, -40}}, lineColor = {0, 0, 255}, pattern = LinePattern.None, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Ellipse(extent = {{72, 28}, {102, -1}}, lineColor = {0, 0, 255}, pattern = LinePattern.None, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Ellipse(extent = {{71, 40}, {101, 11}}, lineColor = {0, 0, 255}, pattern = LinePattern.None, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{0, -64}, {0, -100}}, color = {191, 0, 0}), Line(points = {{40, 100}, {40, 64}}, color = {0, 0, 127}), Ellipse(extent = {{58, -11}, {88, -40}}, lineColor = {0, 0, 255}, pattern = LinePattern.None, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Ellipse(extent = {{71, 1}, {101, -28}}, lineColor = {0, 0, 255}, pattern = LinePattern.None, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid)}),
              Documentation(revisions = "<html>
<ul>
<li><i>Dec 2008</i>
    by R&uuml;diger Franke:<br>
     Adapt initialization to new Types.Dynamics</li>
<li><i>2 Nov 2005</i>
    by <a href=\"mailto:francesco.casella@polimi.it\">Francesco Casella</a>:<br>
     Initialization options fixed</li>
<li><i>6 Sep 2005</i><br>
    Model by Ruediger Franke<br>
    See Franke, Rode, Krueger: On-line Optimization of Drum Boiler Startup, 3rd International Modelica Conference, Linkoping, 2003.<br>
    Modified after the 45th Design Meeting</li>
</ul>
</html>", info = "<html>
<p>
Model of a simple evaporator with two states. The model assumes two-phase equilibrium inside the component; saturated steam goes out of the steam outlet.</p>
<p>
References: Astroem, Bell: Drum-boiler dynamics, Automatica 36, 2000, pp.363-378</p>
</html>"));
          end EquilibriumDrumBoiler;
        end BaseClasses;
      end DrumBoiler;

      package Tanks "Library demonstrating the usage of the tank model"
        extends Modelica.Icons.ExamplesPackage;

        model ThreeTanks "Demonstrating the usage of SimpleTank"
          import Modelica.Fluid;
          extends Modelica.Icons.Example;
          // replaceable package Medium = Modelica.Fluid.Media.Water.ConstantPropertyLiquidWater extends
          // replaceable package Medium = Modelica.Media.Water.StandardWaterOnePhase extends
          // replaceable package Medium = Modelica.Media.Incompressible.Examples.Glycol47 extends
          replaceable package Medium = Modelica.Media.Water.ConstantPropertyLiquidWater constrainedby Modelica.Media.Interfaces.PartialMedium "Medium in the component" annotation(
             choicesAllMatching = true);
          Modelica.Fluid.Vessels.OpenTank tank1(crossArea = 1, redeclare package Medium = Medium, use_portsData = true, height = 12, level_start = 8, nPorts = 1, portsData = {Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.1)}) annotation(
            Placement(transformation(extent = {{-80, 20}, {-40, 60}})));
          Modelica.Fluid.Vessels.OpenTank tank2(crossArea = 1, redeclare package Medium = Medium, use_portsData = true, height = 12, level_start = 3, nPorts = 1, portsData = {Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.1)}) annotation(
            Placement(transformation(extent = {{-20, 20}, {20, 60}})));
          inner Modelica.Fluid.System system(energyDynamics = Modelica.Fluid.Types.Dynamics.FixedInitial) annotation(
            Placement(transformation(extent = {{70, -90}, {90, -70}})));
          Modelica.Fluid.Vessels.OpenTank tank3(crossArea = 1, redeclare package Medium = Medium, use_portsData = true, height = 12, level_start = 3, nPorts = 1, portsData = {Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.1)}) annotation(
            Placement(transformation(extent = {{40, 10}, {80, 50}})));
          Modelica.Fluid.Pipes.StaticPipe pipe1(redeclare package Medium = Medium, allowFlowReversal = true, height_ab = 2, length = 2, diameter = 0.1) annotation(
            Placement(transformation(origin = {-60, -10}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
          Modelica.Fluid.Pipes.StaticPipe pipe2(redeclare package Medium = Medium, allowFlowReversal = true, height_ab = 2, length = 2, diameter = 0.1) annotation(
            Placement(transformation(origin = {0, -10}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
          Modelica.Fluid.Pipes.StaticPipe pipe3(redeclare package Medium = Medium, allowFlowReversal = true, height_ab = -1, length = 2, diameter = 0.1) annotation(
            Placement(transformation(origin = {60, -20}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
        equation
          connect(pipe1.port_a, pipe2.port_a) annotation(
            Line(points = {{-60, -20}, {-60, -40}, {0, -40}, {0, -30}, {0, -20}}, color = {0, 127, 255}));
          connect(pipe2.port_a, pipe3.port_a) annotation(
            Line(points = {{0, -20}, {0, -20}, {0, -40}, {60, -40}, {60, -30}}, color = {0, 127, 255}));
          connect(pipe3.port_b, tank3.ports[1]) annotation(
            Line(points = {{60, -10}, {60, -10}, {60, 10}}, color = {0, 127, 255}));
          connect(pipe1.port_b, tank1.ports[1]) annotation(
            Line(points = {{-60, 0}, {-60, 10}, {-60, 20}}, color = {0, 127, 255}));
          connect(pipe2.port_b, tank2.ports[1]) annotation(
            Line(points = {{0, 0}, {0, 20}}, color = {0, 127, 255}));
          annotation(
            experiment(StopTime = 200),
            __Dymola_Commands(file = "modelica://Modelica/Resources/Scripts/Dymola/Fluid/ThreeTanks/plot level and port.m_flow.mos" "plot level and port.m_flow"),
            Documentation(info = "<html>

<img src=\"modelica://Modelica/Resources/Images/Fluid/Examples/ThreeTanks.png\" border=\"1\"
     alt=\"ThreeTanks.png\">
</html>"));
        end ThreeTanks;

        model TanksWithOverflow "Two tanks connected with pipes at different heights"
          extends Modelica.Icons.Example;
          import Modelica.Fluid;
          Modelica.Fluid.Vessels.OpenTank upperTank(redeclare package Medium = Modelica.Media.CompressibleLiquids.LinearColdWater, height = 20, level_start = 2, crossArea = 0.2, nPorts = 3, portsData = {Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.1), Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.1), Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.1, height = 10)}) annotation(
            Placement(transformation(extent = {{-40, 20}, {0, 60}})));
          Modelica.Fluid.Sources.MassFlowSource_T massFlowRate(nPorts = 1, redeclare package Medium = Modelica.Media.CompressibleLiquids.LinearColdWater, m_flow = 0.2, use_m_flow_in = true) annotation(
            Placement(transformation(extent = {{-60, -40}, {-40, -20}})));
          inner Modelica.Fluid.System system(energyDynamics = Modelica.Fluid.Types.Dynamics.FixedInitial) annotation(
            Placement(transformation(extent = {{-150, -112}, {-130, -92}})));
          Modelica.Fluid.Sensors.Pressure pressure(redeclare package Medium = Modelica.Media.CompressibleLiquids.LinearColdWater) annotation(
            Placement(transformation(extent = {{40, 16}, {60, 36}})));
          Modelica.Fluid.Pipes.StaticPipe pipe(redeclare package Medium = Modelica.Media.CompressibleLiquids.LinearColdWater, diameter = 0.02, height_ab = -20, length = 200) annotation(
            Placement(transformation(origin = {0, -30}, extent = {{10, -10}, {-10, 10}}, rotation = 90)));
          Modelica.Fluid.Vessels.OpenTank lowerTank(height = 20, redeclare package Medium = Modelica.Media.CompressibleLiquids.LinearColdWater, level_start = 2, crossArea = 1, nPorts = 2, portsData = {Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.1), Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.1, height = 10)}) annotation(
            Placement(transformation(extent = {{40, -60}, {80, -20}})));
          Modelica.Blocks.Logical.Hysteresis hysteresis(uLow = 1.1e5, uHigh = 2.5e5, pre_y_start = true) "mass flow rate signal by pressure control" annotation(
            Placement(transformation(extent = {{-140, -30}, {-120, -10}})));
          Modelica.Blocks.Logical.Switch switch1 annotation(
            Placement(transformation(extent = {{-100, -30}, {-80, -10}})));
          Modelica.Blocks.Sources.Constant m_flow_off(k = 0) annotation(
            Placement(transformation(extent = {{-140, 10}, {-120, 30}})));
          Modelica.Blocks.Sources.Constant m_flow_on(k = 2) annotation(
            Placement(transformation(extent = {{-140, -60}, {-120, -40}})));
          Modelica.Fluid.Pipes.StaticPipe overflow(redeclare package Medium = Modelica.Media.CompressibleLiquids.LinearColdWater, diameter = 0.02, length = 200, height_ab = -20) annotation(
            Placement(transformation(origin = {20, -10}, extent = {{10, -10}, {-10, 10}}, rotation = 90)));
        equation
          connect(massFlowRate.ports[1], upperTank.ports[1]) annotation(
            Line(points = {{-40, -30}, {-25.3333, -30}, {-25.3333, 20}}, color = {0, 127, 255}));
          connect(pressure.p, hysteresis.u) annotation(
            Line(points = {{61, 26}, {70, 26}, {70, 80}, {-150, 80}, {-150, -20}, {-142, -20}}, color = {0, 0, 127}));
          connect(hysteresis.y, switch1.u2) annotation(
            Line(points = {{-119, -20}, {-102, -20}}, color = {255, 0, 255}));
          connect(m_flow_off.y, switch1.u1) annotation(
            Line(points = {{-119, 20}, {-119, 5}, {-102, 5}, {-102, -12}}, color = {0, 0, 127}));
          connect(m_flow_on.y, switch1.u3) annotation(
            Line(points = {{-119, -50}, {-110, -50}, {-110, -28}, {-102, -28}}, color = {0, 0, 127}));
          connect(switch1.y, massFlowRate.m_flow_in) annotation(
            Line(points = {{-79, -20}, {-70, -20}, {-70, -22}, {-60, -22}}, color = {0, 0, 127}));
          connect(upperTank.ports[2], pipe.port_a) annotation(
            Line(points = {{-20, 20}, {-20, 10}, {0, 10}, {0, -20}}, color = {0, 127, 255}));
          connect(pipe.port_a, pressure.port) annotation(
            Line(points = {{0, -20}, {0, 10}, {50, 10}, {50, 16}}, color = {0, 127, 255}));
          connect(pipe.port_b, lowerTank.ports[1]) annotation(
            Line(points = {{0, -40}, {0, -70}, {56, -70}, {56, -60}}, color = {0, 127, 255}));
          connect(upperTank.ports[3], overflow.port_a) annotation(
            Line(points = {{-14.6667, 20}, {0, 20}, {0, 40}, {20, 40}, {20, 0}}, color = {0, 127, 255}));
          connect(overflow.port_b, lowerTank.ports[2]) annotation(
            Line(points = {{20, -20}, {20, -40}, {40, -40}, {40, -60}, {64, -60}}, color = {0, 127, 255}));
          annotation(
            experiment(StopTime = 25000, Interval = 5.0),
            __Dymola_Commands(file = "modelica://Modelica/Resources/Scripts/Dymola/Fluid/TanksWithOverflow/plot level and port.m_flow.mos" "plot level and port.m_flow"),
            Documentation(info = "<html>
<p align=justify>The mass flow rate to the upper tank is controlled by the static pressure at its bottom.
The fluid flows through a pipe and forced by different heights from the upper tank to the lower tank.
</p>
<p>
Additional fluid flows through an overflow pipe if the level of the upper tank exceeds 10m.
Initially the overflow enters the lower tank above its fluid level; later on the fluid level exceeds the overflow port.
</p>
<p>
Note that the number of solver intervals has been increased, accounting for the long simulation time horizon.
Otherwise the simulation may fail due to too large steps subject to events. Alternatively the
simulation accuracy could be increased in order to avoid errors.
</p>

<img src=\"modelica://Modelica/Resources/Images/Fluid/Examples/TanksWithOverflow.png\" border=\"1\"
     alt=\"TanksWithOverflow.png\">
</html>"),
            Diagram(coordinateSystem(extent = {{-160, -120}, {100, 100}})));
        end TanksWithOverflow;

        model EmptyTanks "Show the treatment of empty tanks"
          extends Modelica.Icons.Example;
          Modelica.Fluid.Vessels.OpenTank tank1(redeclare package Medium = Modelica.Media.Water.ConstantPropertyLiquidWater, nPorts = 1, crossArea = 1, level_start = 1, portsData = {Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.1)}, height = 1.1) annotation(
            Placement(transformation(extent = {{-40, 20}, {0, 60}})));
          Modelica.Fluid.Pipes.StaticPipe pipe(redeclare package Medium = Modelica.Media.Water.ConstantPropertyLiquidWater, length = 1, diameter = 0.1, height_ab = -1) annotation(
            Placement(transformation(origin = {-20, -20}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          Modelica.Fluid.Vessels.OpenTank tank2(crossArea = 1, redeclare package Medium = Modelica.Media.Water.ConstantPropertyLiquidWater, nPorts = 1, height = 1.1, portsData = {Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.1, height = 0.5)}, level_start = 1.0e-10) annotation(
            Placement(transformation(extent = {{0, -80}, {40, -40}})));
          inner Modelica.Fluid.System system(energyDynamics = Modelica.Fluid.Types.Dynamics.FixedInitial) annotation(
            Placement(transformation(extent = {{60, 60}, {80, 80}})));
        equation
          connect(tank1.ports[1], pipe.port_a) annotation(
            Line(points = {{-20, 20}, {-20, 5}, {-20, -10}, {-20, -10}}, color = {0, 127, 255}));
          connect(pipe.port_b, tank2.ports[1]) annotation(
            Line(points = {{-20, -30}, {-20, -60}, {0, -60}, {0, -80}, {20, -80}}, color = {0, 127, 255}));
          annotation(
            experiment(StopTime = 50),
            __Dymola_Commands(file = "modelica://Modelica/Resources/Scripts/Dymola/Fluid/EmptyTanks/plot level and port.p.mos" "plot level and port.p"),
            Documentation(info = "<html>
<img src=\"modelica://Modelica/Resources/Images/Fluid/Examples/EmptyTanks.png\" border=\"1\"
     alt=\"EmptyTanks.png\">
</html>"));
        end EmptyTanks;
      end Tanks;

      package ControlledTankSystem "Tank system with controller, start/stop/shut operation and diagram animation"
        extends Modelica.Icons.ExamplesPackage;

        model ControlledTanks "Demonstrating the controller of a tank filling/emptying system"
          extends Modelica.Icons.Example;
          package Medium = Modelica.Media.Water.ConstantPropertyLiquidWater;
          Modelica.Fluid.Examples.ControlledTankSystem.Utilities.TankController tankController(waitTime = 50, maxLevel = 0.9 * tank1.height, minLevel = 0.01) annotation(
            Placement(transformation(extent = {{-60, -20}, {-20, 20}})));
          Modelica.Fluid.Examples.ControlledTankSystem.Utilities.RadioButton start(reset = {stop.on, shut.on}, buttonTimeTable = {20, 280}) annotation(
            Placement(transformation(extent = {{-100, 20}, {-80, 40}})));
          Modelica.Fluid.Examples.ControlledTankSystem.Utilities.RadioButton stop(reset = {start.on, shut.on}, buttonTimeTable = {220, 650}) annotation(
            Placement(transformation(extent = {{-100, -10}, {-80, 10}})));
          Modelica.Fluid.Examples.ControlledTankSystem.Utilities.RadioButton shut(reset = {start.on, stop.on}, buttonTimeTable = {700}) annotation(
            Placement(transformation(extent = {{-100, -40}, {-80, -20}})));
          Modelica.Fluid.Valves.ValveDiscrete valve1(redeclare package Medium = Medium, m_flow_nominal = 40, dp_nominal = 100000) annotation(
            Placement(transformation(origin = {-10, 70}, extent = {{10, -10}, {-10, 10}}, rotation = 180)));
          Modelica.Fluid.Vessels.OpenTank tank1(level_start = 0.05, redeclare package Medium = Medium, crossArea = 6, height = 4, nPorts = 2, portsData = {Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.2, height = 4, zeta_out = 0, zeta_in = 1), Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.2, height = 0, zeta_out = 0, zeta_in = 1)}) annotation(
            Placement(transformation(extent = {{10, 30}, {50, 70}})));
          Modelica.Blocks.Sources.RealExpression level1(y = tank1.level) annotation(
            Placement(transformation(extent = {{-90, -60}, {-55, -40}})));
          Modelica.Fluid.Valves.ValveDiscrete valve2(redeclare package Medium = Medium, dp_nominal(displayUnit = "Pa") = 1, m_flow_nominal = 100) annotation(
            Placement(transformation(origin = {34, 0}, extent = {{10, -10}, {-10, 10}}, rotation = 90)));
          Modelica.Fluid.Valves.ValveDiscrete valve3(redeclare package Medium = Medium, dp_nominal(displayUnit = "Pa") = 1, m_flow_nominal = 10) annotation(
            Placement(transformation(origin = {35, -80}, extent = {{10, -10}, {-10, 10}})));
          Modelica.Fluid.Vessels.OpenTank tank2(level_start = 0.05, redeclare package Medium = Medium, height = 5, crossArea = 6, nPorts = 2, portsData = {Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.2, height = 5, zeta_out = 0, zeta_in = 1), Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.2, height = 0, zeta_out = 0, zeta_in = 1)}) annotation(
            Placement(transformation(extent = {{50, -60}, {90, -20}})));
          Modelica.Fluid.Sources.Boundary_pT ambient1(redeclare package Medium = Medium, nPorts = 1, p = system.p_ambient, T = system.T_ambient) annotation(
            Placement(transformation(extent = {{-10, -90}, {10, -70}})));
          Modelica.Blocks.Sources.RealExpression level2(y = tank2.level) annotation(
            Placement(transformation(extent = {{-70, -80}, {-33, -60}})));
          Modelica.Fluid.Sources.Boundary_pT source(redeclare package Medium = Medium, p = 2.5e6, nPorts = 1, T = system.T_ambient) annotation(
            Placement(transformation(origin = {-40, 70}, extent = {{-10, -10}, {10, 10}})));
          inner Modelica.Fluid.System system(energyDynamics = Modelica.Fluid.Types.Dynamics.FixedInitial) annotation(
            Placement(transformation(extent = {{-90, 70}, {-70, 90}})));
        equation
          connect(shut.on, tankController.shut) annotation(
            Line(points = {{-79, -30}, {-72, -30}, {-72, -12}, {-62, -12}}, color = {255, 0, 255}));
          connect(stop.on, tankController.stop) annotation(
            Line(points = {{-79, 0}, {-62, 0}}, color = {255, 0, 255}));
          connect(start.on, tankController.start) annotation(
            Line(points = {{-79, 30}, {-70, 30}, {-70, 12}, {-62, 12}}, color = {255, 0, 255}));
          connect(tankController.valve1, valve1.open) annotation(
            Line(points = {{-19, 12}, {-10, 12}, {-10, 62}}, color = {255, 0, 255}));
          connect(level1.y, tankController.level1) annotation(
            Line(points = {{-53.25, -50}, {-52, -50}, {-52, -22}}, color = {0, 0, 127}));
          connect(tankController.valve2, valve2.open) annotation(
            Line(points = {{-19, 0}, {-5, 0}, {26, 0}}, color = {255, 0, 255}));
          connect(tankController.valve3, valve3.open) annotation(
            Line(points = {{-19, -12}, {-10, -12}, {-10, -50}, {35, -50}, {35, -72}}, color = {255, 0, 255}));
          connect(level2.y, tankController.level2) annotation(
            Line(points = {{-31.15, -70}, {-28, -70}, {-28, -22}}, color = {0, 0, 127}));
          connect(source.ports[1], valve1.port_a) annotation(
            Line(points = {{-30, 70}, {-20, 70}}, color = {0, 127, 255}));
          connect(valve3.port_b, ambient1.ports[1]) annotation(
            Line(points = {{25, -80}, {10, -80}}, color = {0, 127, 255}));
          connect(tank2.ports[2], valve3.port_a) annotation(
            Line(points = {{74, -60}, {74, -80}, {45, -80}}, color = {0, 127, 255}));
          connect(valve2.port_b, tank2.ports[1]) annotation(
            Line(points = {{34, -10}, {34, -20}, {50, -20}, {50, -60}, {66, -60}}, color = {0, 127, 255}));
          connect(valve1.port_b, tank1.ports[1]) annotation(
            Line(points = {{0, 70}, {10, 70}, {10, 30}, {26, 30}}, color = {0, 127, 255}));
          connect(tank1.ports[2], valve2.port_a) annotation(
            Line(points = {{34, 30}, {34, 10}}, color = {0, 127, 255}));
          annotation(
            experiment(StopTime = 900),
            Documentation(info = "<html>
<p>
With this example, the controller of a tank filling/emptying system
is demonstrated.
</p>

<p>
The basic operation is to fill and empty the two tanks:
</p>
<ol>
<li> Valve 1 is opened and tank 1 is filled.</li>
<li> When tank 1 reaches its fill level limit,
     valve 1 is closed. </li>
<li> After a waiting time, valve 2 is
     opened and the fluid flows from tank 1 into tank 2.</li>
<li> When tank 1 reaches its minimum level, valve 2 is closed. </li>
<li> After a waiting time, valve 3 is opened and
     the fluid flows out of tank 2</li>
<li> When tank 2 reaches its minimum level, valve 3 is closed</li>
</ol>
<p>
The above \"normal\" process can be influenced by three
buttons:
</p>
<ul>
<li> Button <b>start</b> starts the above process.
     When this button is pressed after a \"stop\" or
     \"shut\" operation, the process operation continues.
     </li>
<li> Button <b>stop</b> stops the above process by
     closing all valves. Then, the controller waits for
     further input (either \"start\" or \"shut\" operation).</li>
<li> Button <b>shut</b> is used to shutdown the process,
     by emptying at once both tanks by opening valve 2 and
     valve 3. When this is achieved,
     the process goes back to its start configuration
     where all 3 valves are closed.
     Clicking on \"start\", restarts the process.</li>
</ul>

<p>
The demo-run uses the following button presses:
</p>

<ul>
<li> Button <b>start</b> pressed at 20 s.</li>
<li> Button <b>stop</b> pressed at 220 s </li>
<li> Button <b>start</b> pressed at 280 s </li>
<li> Button <b>stop</b> pressed at 650 s </li>
<li> Button <b>shut</b> pressed at 700 s </li>
<li> Simulate for 900 s</li>
</ul>

<p>
This example is based on
</p>

<dl>
<dt>Dressler I. (2004):</dt>
<dd> <b>Code Generation From JGrafchart to Modelica</b>.
     Master thesis, supervisor: Karl-Erik Arzen,
     Department of Automatic Control, Lund Institute of Technology,
     Lund, Sweden, March 30, 2004<br>&nbsp;</dd>
</dl>

<img src=\"modelica://Modelica/Resources/Images/Fluid/Examples/ControlledTanks.png\" border=\"1\"
     alt=\"ControlledTanks.png\">
</html>"),
            __Dymola_Commands(file = "modelica://Modelica/Resources/Scripts/Dymola/Fluid/ControlledTanks/plot level and ports.m_flow.mos" "plot level and ports.m_flow"));
        end ControlledTanks;

        package Utilities
          extends Modelica.Icons.UtilitiesPackage;

          model TankController "Controller for tank system"
            extends Modelica.StateGraph.Interfaces.PartialStateGraphIcon;
            parameter SI.Height maxLevel "Fill level of tank 1";
            parameter SI.Height minLevel "Lowest level of tank 1 and 2";
            parameter SI.Time waitTime "Wait time, between operations";
            Modelica.StateGraph.InitialStep s1(nIn = 2) annotation(
              Placement(transformation(extent = {{-72, 30}, {-52, 50}})));
            Modelica.Fluid.Examples.ControlledTankSystem.Utilities.NormalOperation normal(maxLevel = maxLevel, minLevel = minLevel, waitTime = waitTime) annotation(
              Placement(transformation(extent = {{-20, 20}, {20, 60}})));
            Modelica.StateGraph.Transition T1(condition = start) annotation(
              Placement(transformation(extent = {{-50, 50}, {-30, 30}})));
            Modelica.StateGraph.Transition T2(condition = level2 < minLevel) annotation(
              Placement(transformation(extent = {{27, 50}, {47, 30}})));
            Modelica.StateGraph.Transition T3(condition = stop) annotation(
              Placement(transformation(origin = {-23, -1}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
            Modelica.StateGraph.Step s2(nOut = 2) annotation(
              Placement(transformation(extent = {{-50, -60}, {-30, -40}})));
            Modelica.StateGraph.Transition T4(condition = start) annotation(
              Placement(transformation(origin = {10, 0}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
            Modelica.StateGraph.Transition T5(condition = shut) annotation(
              Placement(transformation(extent = {{-6, -60}, {14, -40}})));
            Modelica.StateGraph.Step emptyTanks annotation(
              Placement(transformation(extent = {{20, -60}, {40, -40}})));
            Modelica.StateGraph.Transition T6(condition = level1 < minLevel and level2 < minLevel) annotation(
              Placement(transformation(extent = {{45, -60}, {65, -40}})));
            Modelica.Blocks.Interfaces.BooleanInput start annotation(
              Placement(transformation(extent = {{-120, 50}, {-100, 70}})));
            Modelica.Blocks.Interfaces.BooleanInput stop annotation(
              Placement(transformation(extent = {{-120, -10}, {-100, 10}})));
            Modelica.Blocks.Interfaces.BooleanInput shut annotation(
              Placement(transformation(extent = {{-120, -70}, {-100, -50}})));
            Modelica.Blocks.Interfaces.RealInput level1 annotation(
              Placement(transformation(origin = {-60, -110}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
            Modelica.Blocks.Interfaces.RealInput level2 annotation(
              Placement(transformation(origin = {60, -110}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
            Modelica.Blocks.Interfaces.BooleanOutput valve1 annotation(
              Placement(transformation(extent = {{100, 55}, {110, 65}})));
            Modelica.Blocks.Interfaces.BooleanOutput valve2 annotation(
              Placement(transformation(extent = {{100, -5}, {110, 5}})));
            Modelica.Blocks.Interfaces.BooleanOutput valve3 annotation(
              Placement(transformation(extent = {{100, -65}, {110, -55}})));
            Modelica.Blocks.Sources.BooleanExpression setValve1(y = normal.fillTank1.active) annotation(
              Placement(transformation(extent = {{20, 73}, {80, 92}})));
            Modelica.Blocks.Sources.BooleanExpression setValve2(y = normal.fillTank2.active or emptyTanks.active and level1 > minLevel) annotation(
              Placement(transformation(extent = {{-40, -85}, {80, -64}})));
            Modelica.Blocks.Sources.BooleanExpression setValve3(y = normal.emptyTank2.active or emptyTanks.active and level2 > minLevel) annotation(
              Placement(transformation(extent = {{-40, -103}, {80, -83}})));
            inner Modelica.StateGraph.StateGraphRoot stateGraphRoot annotation(
              Placement(transformation(extent = {{-94, 74}, {-74, 94}})));
          equation
            connect(s1.outPort[1], T1.inPort) annotation(
              Line(points = {{-51.5, 40}, {-44, 40}}));
            connect(T1.outPort, normal.inPort) annotation(
              Line(points = {{-38.5, 40}, {-21.3333, 40}}));
            connect(normal.outPort, T2.inPort) annotation(
              Line(points = {{20.6667, 40}, {33, 40}}));
            connect(T5.outPort, emptyTanks.inPort[1]) annotation(
              Line(points = {{5.5, -50}, {19, -50}}));
            connect(emptyTanks.outPort[1], T6.inPort) annotation(
              Line(points = {{40.5, -50}, {51, -50}}));
            connect(setValve1.y, valve1) annotation(
              Line(points = {{83, 82.5}, {90, 82.5}, {90, 60}, {105, 60}}, color = {255, 0, 255}));
            connect(setValve2.y, valve2) annotation(
              Line(points = {{86, -74.5}, {90, -74.5}, {90, 0}, {105, 0}}, color = {255, 0, 255}));
            connect(setValve3.y, valve3) annotation(
              Line(points = {{86, -93}, {95, -93}, {95, -60}, {105, -60}}, color = {255, 0, 255}));
            connect(normal.suspend[1], T3.inPort) annotation(
              Line(points = {{-10, 19.3333}, {-10, 12}, {-23, 12}, {-23, 3}}));
            connect(T3.outPort, s2.inPort[1]) annotation(
              Line(points = {{-23, -2.5}, {-23, -20}, {-60, -20}, {-60, -50}, {-51, -50}}));
            connect(level1, normal.level1) annotation(
              Line(points = {{-60, -110}, {-60, -80}, {-80, -80}, {-80, 20}, {-30, 20}, {-30, 24}, {-22.6667, 24}}, color = {0, 0, 255}));
            connect(s2.outPort[1], T5.inPort) annotation(
              Line(points = {{-29.5, -49.75}, {-30, -49.75}, {-30, -50}, {0, -50}}));
            connect(s2.outPort[2], T4.inPort) annotation(
              Line(points = {{-29.5, -50.25}, {-29, -50}, {-8, -50}, {-8, -25}, {10, -25}, {10, -4}}));
            connect(T2.outPort, s1.inPort[1]) annotation(
              Line(points = {{38.5, 40}, {70, 40}, {70, 70}, {-80, 70}, {-80, 40}, {-73, 40}, {-73, 40.5}}));
            connect(T6.outPort, s1.inPort[2]) annotation(
              Line(points = {{56.5, -50}, {70, -50}, {70, 70}, {-80, 70}, {-80, 40}, {-74, 40}, {-73, 39.5}}));
            connect(T4.outPort, normal.resume[1]) annotation(
              Line(points = {{10, 1.5}, {10, 10}, {10.5, 10}, {10.5, 18.6667}, {10, 18.6667}}));
            annotation(
              Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, 100}, {100, -100}}, lineColor = {0, 0, 0})}),
              Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-100, 68}, {-12, 54}}, lineColor = {0, 0, 0}, textString = "start"), Text(extent = {{-99, 6}, {-14, -9}}, lineColor = {0, 0, 0}, textString = "stop"), Text(extent = {{-99, -54}, {-14, -69}}, lineColor = {0, 0, 0}, textString = "shut"), Text(extent = {{-94, -82}, {-9, -96}}, lineColor = {0, 0, 0}, textString = "level1"), Text(extent = {{11, -83}, {96, -98}}, lineColor = {0, 0, 0}, textString = "level2"), Text(extent = {{10, 68}, {99, 54}}, lineColor = {0, 0, 0}, textString = "valve1"), Text(extent = {{7, 10}, {101, -5}}, lineColor = {0, 0, 0}, textString = "valve2"), Text(extent = {{2, -51}, {102, -67}}, lineColor = {0, 0, 0}, textString = "valve3")}));
          end TankController;

          model NormalOperation "Normal operation of tank system (button start pressed)"
            extends Modelica.StateGraph.PartialCompositeStep;
            parameter SI.Height maxLevel "Fill level of tank 1";
            parameter SI.Height minLevel "Lowest level of tank 1 and 2";
            parameter SI.Time waitTime "Wait time between operations";
            Modelica.Blocks.Interfaces.RealInput level1 annotation(
              Placement(transformation(extent = {{-190, -140}, {-150, -100}})));
            Modelica.StateGraph.Step fillTank1 annotation(
              Placement(transformation(extent = {{-140, -10}, {-120, 10}})));
            Modelica.StateGraph.Transition T1(condition = level1 > maxLevel) annotation(
              Placement(transformation(extent = {{-110, -10}, {-90, 10}})));
            Modelica.StateGraph.Step fillTank2 annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}})));
            Modelica.StateGraph.Transition T3(condition = level1 < minLevel) annotation(
              Placement(transformation(extent = {{20, -10}, {40, 10}})));
            Modelica.StateGraph.Step emptyTank2 annotation(
              Placement(transformation(extent = {{120, -10}, {140, 10}})));
            Modelica.StateGraph.Step wait1 annotation(
              Placement(transformation(extent = {{-80, -10}, {-60, 10}})));
            Modelica.StateGraph.Transition T2(enableTimer = true, waitTime = waitTime) annotation(
              Placement(transformation(extent = {{-50, -10}, {-30, 10}})));
            Modelica.StateGraph.Step wait2 annotation(
              Placement(transformation(extent = {{54, -10}, {74, 10}})));
            Modelica.StateGraph.Transition T4(enableTimer = true, waitTime = waitTime) annotation(
              Placement(transformation(extent = {{82, -10}, {102, 10}})));
          equation
            connect(fillTank1.inPort[1], inPort) annotation(
              Line(points = {{-141, 0}, {-160, 0}}));
            connect(fillTank1.outPort[1], T1.inPort) annotation(
              Line(points = {{-119.5, 0}, {-104, 0}}));
            connect(fillTank2.outPort[1], T3.inPort) annotation(
              Line(points = {{10.5, 0}, {26, 0}}));
            connect(emptyTank2.outPort[1], outPort) annotation(
              Line(points = {{140.5, 0}, {155, 0}}));
            connect(wait1.outPort[1], T2.inPort) annotation(
              Line(points = {{-59.5, 0}, {-44, 0}}));
            connect(T2.outPort, fillTank2.inPort[1]) annotation(
              Line(points = {{-38.5, 0}, {-11, 0}}));
            connect(T1.outPort, wait1.inPort[1]) annotation(
              Line(points = {{-98.5, 0}, {-81, 0}}));
            connect(wait2.outPort[1], T4.inPort) annotation(
              Line(points = {{74.5, 0}, {88, 0}}));
            connect(T3.outPort, wait2.inPort[1]) annotation(
              Line(points = {{31.5, 0}, {53, 0}}));
            connect(T4.outPort, emptyTank2.inPort[1]) annotation(
              Line(points = {{93.5, 0}, {119, 0}}));
          end NormalOperation;

          block RadioButton "Button that sets its output to true when pressed and is reset when an element of 'reset' becomes true"
            parameter Modelica.SIunits.Time buttonTimeTable[:] "Time instants where button is pressed";
            input Boolean reset[:] = {false} "Reset button to false, if an element of reset becomes true" annotation(
              Dialog(group = "Time varying expressions"));
            Modelica.Blocks.Interfaces.BooleanOutput on annotation(
              Placement(transformation(extent = {{100, -10}, {120, 10}})));
          protected
            Modelica.Blocks.Sources.BooleanTable table(table = buttonTimeTable);
          initial equation
            pre(reset) = fill(false, size(reset, 1));
          algorithm
            when pre(reset) then
              on := false;
            end when;
            when change(table.y) then
              on := true;
            end when;
            annotation(
              Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, -100}, {100, 100}}, fillColor = DynamicSelect({192, 192, 192}, if on > 0.5 then {0, 255, 0} else {192, 192, 192}), fillPattern = DynamicSelect(FillPattern.Solid, if on > 0.5 then FillPattern.Solid else FillPattern.Solid), lineColor = {128, 128, 128}, lineThickness = 0.5), Text(extent = {{-80, -40}, {80, 40}}, lineColor = {0, 0, 0}, textString = "%name")}, interaction = {OnMouseDownSetBoolean(on, true)}));
          end RadioButton;
        end Utilities;
      end ControlledTankSystem;

      package AST_BatchPlant "Model of the experimental batch plant at Process Control Laboratory at University of Dortmund (Prof. Engell)"
        extends Modelica.Icons.ExamplesPackage;

        model BatchPlant_StandardWater
          extends Modelica.Icons.Example;
          replaceable package BatchMedium = Modelica.Media.Water.StandardWater constrainedby Modelica.Media.Interfaces.PartialTwoPhaseMedium "Component media";
          /*
            replaceable package BatchMedium =Modelica.Media.Electrolytes.WaterNaCl extends
              Modelica.Media.Interfaces.PartialTwoPhaseMedium "Component media";
          */
          parameter Modelica.SIunits.Length pipeDiameter = 0.01;
          BaseClasses.TankWith3InletOutletArraysWithEvaporatorCondensor B5(redeclare package Medium = BatchMedium, height = 0.5, n_SidePorts = 1, V0 = 0.001, n_BottomPorts = 1, bottom_pipeArea = {0.0001}, top_pipeArea = {0.0001}, n_TopPorts = 1, min_level_for_heating = 0.0001, level_start = 0.0009, crossArea = 0.05, initType = BaseClasses.Init.InitialValues, side_pipeArea = {0.0001}, redeclare model HeatTransfer = Modelica.Fluid.Vessels.BaseClasses.HeatTransfer.IdealHeatTransfer(k = 4.9)) annotation(
            Placement(transformation(extent = {{-110, -60}, {-30, -20}})));
          Modelica.Fluid.Valves.ValveDiscrete V12(redeclare package Medium = BatchMedium, m_flow_nominal = 1, dp_nominal = 100) annotation(
            Placement(transformation(origin = {-90, 2}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
          Modelica.Fluid.Valves.ValveDiscrete V15(redeclare package Medium = BatchMedium, m_flow_nominal = 1, dp_nominal = 100) annotation(
            Placement(transformation(origin = {-90, -82}, extent = {{10, -10}, {-10, 10}}, rotation = 90)));
          Modelica.Thermal.HeatTransfer.Sources.PrescribedHeatFlow HeatB5 annotation(
            Placement(transformation(extent = {{-150, -50}, {-130, -30}})));
          Modelica.Thermal.HeatTransfer.Sources.PrescribedHeatFlow CoolingB7 annotation(
            Placement(transformation(extent = {{-150, -130}, {-130, -110}})));
          Modelica.Thermal.HeatTransfer.Sources.PrescribedHeatFlow CoolingB6 annotation(
            Placement(transformation(origin = {110, -60}, extent = {{-10, -10}, {10, 10}}, rotation = 180)));
          Modelica.Fluid.Examples.AST_BatchPlant.BaseClasses.Controller controller(Transition3(enableTimer = true, waitTime = 60), Transition7(condition = true, enableTimer = true, waitTime = 300)) annotation(
            Placement(transformation(extent = {{60, 38}, {100, 78}})));
          Modelica.Fluid.Valves.ValveDiscrete V11(redeclare package Medium = BatchMedium, m_flow_nominal = 1, dp_nominal = 100) annotation(
            Placement(transformation(extent = {{-50, 80}, {-70, 100}})));
          Modelica.Fluid.Valves.ValveDiscrete V8(redeclare package Medium = BatchMedium, m_flow_nominal = 1, dp_nominal = 100) annotation(
            Placement(transformation(origin = {-90, 160}, extent = {{10, -10}, {-10, 10}}, rotation = 90)));
          Modelica.Fluid.Valves.ValveDiscrete V9(redeclare package Medium = BatchMedium, m_flow_nominal = 1, dp_nominal = 100) annotation(
            Placement(transformation(origin = {70, 160}, extent = {{10, 10}, {-10, -10}}, rotation = 90)));
          Modelica.Fluid.Valves.ValveDiscrete V2(redeclare package Medium = BatchMedium, m_flow_nominal = 1, dp_nominal = 100) annotation(
            Placement(transformation(extent = {{-60, 230}, {-40, 250}})));
          Modelica.Fluid.Valves.ValveDiscrete V4(redeclare package Medium = BatchMedium, m_flow_nominal = 1, dp_nominal = 100) annotation(
            Placement(transformation(extent = {{40, 230}, {20, 250}})));
          Modelica.Fluid.Valves.ValveDiscrete V3(redeclare package Medium = BatchMedium, m_flow_nominal = 1, dp_nominal = 100) annotation(
            Placement(transformation(extent = {{-150, 210}, {-130, 230}})));
          Fittings.TeeJunctionIdeal volume2(redeclare package Medium = BatchMedium) annotation(
            Placement(transformation(origin = {-180, 220}, extent = {{-10, 10}, {10, -10}}, rotation = 90)));
          Modelica.Fluid.Valves.ValveDiscrete V6(redeclare package Medium = BatchMedium, m_flow_nominal = 1, dp_nominal = 100) annotation(
            Placement(transformation(extent = {{130, 210}, {110, 230}})));
          Fittings.TeeJunctionIdeal volume8(redeclare package Medium = BatchMedium) annotation(
            Placement(transformation(origin = {160, 220}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
          Modelica.Fluid.Valves.ValveDiscrete V23(redeclare package Medium = BatchMedium, m_flow_nominal = 1, dp_nominal = 100) annotation(
            Placement(transformation(origin = {-110, -250}, extent = {{-10, 10}, {10, -10}}, rotation = 180)));
          Modelica.Fluid.Valves.ValveDiscrete V1(redeclare package Medium = BatchMedium, m_flow_nominal = 1, dp_nominal = 100) annotation(
            Placement(transformation(origin = {-180, 110}, extent = {{-10, 10}, {10, -10}}, rotation = 90)));
          Modelica.Fluid.Valves.ValveDiscrete V22(redeclare package Medium = BatchMedium, m_flow_nominal = 1, dp_nominal = 100) annotation(
            Placement(transformation(origin = {-180, -170}, extent = {{-10, 10}, {10, -10}}, rotation = 90)));
          Modelica.Fluid.Valves.ValveDiscrete V5(redeclare package Medium = BatchMedium, m_flow_nominal = 1, dp_nominal = 100) annotation(
            Placement(transformation(origin = {160, 110}, extent = {{10, -10}, {-10, 10}}, rotation = 270)));
          Modelica.Fluid.Valves.ValveDiscrete V24(redeclare package Medium = BatchMedium, m_flow_nominal = 1, dp_nominal = 100) annotation(
            Placement(transformation(origin = {90, -250}, extent = {{10, 10}, {-10, -10}}, rotation = 180)));
          Modelica.Fluid.Valves.ValveDiscrete V25(redeclare package Medium = BatchMedium, m_flow_nominal = 1, dp_nominal = 100) annotation(
            Placement(transformation(origin = {160, -170}, extent = {{10, -10}, {-10, 10}}, rotation = 270)));
          Modelica.Fluid.Valves.ValveDiscrete V20(redeclare package Medium = BatchMedium, m_flow_nominal = 1, dp_nominal = 100) annotation(
            Placement(transformation(origin = {60, -210}, extent = {{10, 10}, {-10, -10}}, rotation = 90)));
          Modelica.Fluid.Valves.ValveDiscrete V19(redeclare package Medium = BatchMedium, m_flow_nominal = 1, dp_nominal = 100) annotation(
            Placement(transformation(origin = {10, -200}, extent = {{-10, 10}, {10, -10}}, rotation = 180)));
          Modelica.Fluid.Valves.ValveDiscrete V10(redeclare package Medium = BatchMedium, m_flow_nominal = 1, dp_nominal = 100) annotation(
            Placement(transformation(origin = {-21, -170}, extent = {{10, 10}, {-10, -10}}, rotation = 90)));
          Modelica.Fluid.Valves.ValveDiscrete V21(redeclare package Medium = BatchMedium, m_flow_nominal = 1, dp_nominal = 100) annotation(
            Placement(transformation(origin = {30, -250}, extent = {{10, 10}, {-10, -10}}, rotation = 180)));
          Modelica.Fluid.Fittings.TeeJunctionVolume volume5(redeclare package Medium = BatchMedium, V = 0.001) annotation(
            Placement(transformation(extent = {{50, -260}, {70, -240}})));
          Modelica.Fluid.Valves.ValveDiscrete V18(redeclare package Medium = BatchMedium, m_flow_nominal = 1, dp_nominal = 100) annotation(
            Placement(transformation(origin = {-50, -200}, extent = {{10, 10}, {-10, -10}}, rotation = 180)));
          Machines.PrescribedPump P1(redeclare package Medium = BatchMedium, m_flow_start = 0.1, N_nominal = 200, use_N_in = true, redeclare model Monitoring = Modelica.Fluid.Machines.BaseClasses.PumpMonitoring.PumpMonitoringNPSH, V(displayUnit = "ml") = 0.0001, energyDynamics = Modelica.Fluid.Types.Dynamics.FixedInitial, massDynamics = Modelica.Fluid.Types.Dynamics.FixedInitial, redeclare function flowCharacteristic = Modelica.Fluid.Machines.BaseClasses.PumpCharacteristics.quadraticFlow(V_flow_nominal = {0, 0.1e-3, 0.15e-3}, head_nominal = {10, 5, 0}), p_a_start = 100000, p_b_start = 100000) annotation(
            Placement(transformation(extent = {{-140, -260}, {-160, -240}})));
          Machines.PrescribedPump P2(redeclare package Medium = BatchMedium, m_flow_start = 0.1, N_nominal = 200, use_N_in = true, redeclare model Monitoring = Modelica.Fluid.Machines.BaseClasses.PumpMonitoring.PumpMonitoringNPSH, V(displayUnit = "ml") = 0.0001, energyDynamics = Modelica.Fluid.Types.Dynamics.FixedInitial, massDynamics = Modelica.Fluid.Types.Dynamics.FixedInitial, redeclare function flowCharacteristic = Modelica.Fluid.Machines.BaseClasses.PumpCharacteristics.quadraticFlow(V_flow_nominal = {0, 0.1e-3, 0.15e-3}, head_nominal = {10, 5, 0}), p_a_start = 100000, p_b_start = 100000) annotation(
            Placement(transformation(extent = {{120, -260}, {140, -240}})));
          Modelica.Fluid.Examples.AST_BatchPlant.BaseClasses.TankWithTopPorts B1(level_start = 0.2, redeclare package Medium = BatchMedium, height = 0.5, crossArea = 0.05, V0 = 0.0001, nTopPorts = 1, nPorts = 1, portsData = {Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.011, height = 0)}, stiffCharacteristicForEmptyPort = false) annotation(
            Placement(transformation(extent = {{-110, 180}, {-70, 220}})));
          inner Modelica.Fluid.System system(energyDynamics = Modelica.Fluid.Types.Dynamics.FixedInitial) annotation(
            Placement(transformation(extent = {{180, 250}, {200, 270}})));
          Modelica.Blocks.Logical.TriggeredTrapezoid P1_on(rising = 0, amplitude = 200) annotation(
            Placement(transformation(extent = {{-122, -230}, {-142, -210}})));
          Modelica.Blocks.Logical.TriggeredTrapezoid P2_on(rising = 0, amplitude = 200) annotation(
            Placement(transformation(extent = {{100, -230}, {120, -210}})));
          Modelica.Fluid.Examples.AST_BatchPlant.BaseClasses.TankWithTopPorts B2(level_start = 0.2, redeclare package Medium = BatchMedium, height = 0.5, crossArea = 0.05, V0 = 0.0001, nTopPorts = 1, nPorts = 1, portsData = {Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.011, height = 0)}, stiffCharacteristicForEmptyPort = false) annotation(
            Placement(transformation(extent = {{50, 180}, {90, 220}})));
          Modelica.Fluid.Examples.AST_BatchPlant.BaseClasses.TankWithTopPorts B3(redeclare package Medium = BatchMedium, height = 0.5, crossArea = 0.05, V0 = 0.0001, nTopPorts = 2, nPorts = 2, portsData = {Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.011, height = 0), Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.011, height = 0)}, level_start = 0.02, stiffCharacteristicForEmptyPort = false) annotation(
            Placement(transformation(extent = {{-30, 100}, {10, 140}})));
          Modelica.Fluid.Examples.AST_BatchPlant.BaseClasses.TankWithTopPorts B4(redeclare package Medium = BatchMedium, height = 0.5, crossArea = 0.05, V0 = 0.0001, level_start = 0.015, nTopPorts = 1, nPorts = 1, portsData = {Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.011, height = 0)}, stiffCharacteristicForEmptyPort = false) annotation(
            Placement(transformation(extent = {{-110, 30}, {-70, 70}})));
          Modelica.Fluid.Examples.AST_BatchPlant.BaseClasses.TankWithTopPorts B7(redeclare package Medium = BatchMedium, V0 = 0.0001, nTopPorts = 1, nPorts = 1, portsData = {Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.011, height = 0)}, level_start = 0.009, height = 0.5, crossArea = 0.05, stiffCharacteristicForEmptyPort = false, use_HeatTransfer = true, T_start = 298, redeclare model HeatTransfer = Modelica.Fluid.Vessels.BaseClasses.HeatTransfer.IdealHeatTransfer(k = 4.9)) annotation(
            Placement(transformation(extent = {{-110, -140}, {-70, -100}})));
          Pipes.DynamicPipe pipeB1B2(redeclare package Medium = BatchMedium, length = 1, diameter = pipeDiameter, height_ab = 0) annotation(
            Placement(transformation(extent = {{0, 230}, {-20, 250}})));
          Pipes.StaticPipe pipeB1B3(redeclare package Medium = BatchMedium, length = 1, redeclare model FlowModel = Modelica.Fluid.Pipes.BaseClasses.FlowModels.DetailedPipeFlow, diameter = pipeDiameter, height_ab = -0.1) annotation(
            Placement(transformation(extent = {{-70, 134}, {-50, 154}})));
          Pipes.StaticPipe pipeB2B3(redeclare package Medium = BatchMedium, length = 1, redeclare model FlowModel = Modelica.Fluid.Pipes.BaseClasses.FlowModels.DetailedPipeFlow, diameter = pipeDiameter, height_ab = -0.1) annotation(
            Placement(transformation(extent = {{50, 134}, {30, 154}})));
          Pipes.StaticPipe pipeB1B1(redeclare package Medium = BatchMedium, diameter = pipeDiameter, length = 1, height_ab = -0.5) annotation(
            Placement(transformation(origin = {-8, 10}, extent = {{10, 10}, {-10, -10}}, rotation = 90)));
          Pipes.StaticPipe pipeB6Pump(redeclare package Medium = BatchMedium, length = 0.5, diameter = pipeDiameter, height_ab = -0.5) annotation(
            Placement(transformation(origin = {60, -110}, extent = {{10, 10}, {-10, -10}}, rotation = 90)));
          Pipes.StaticPipe pipeB7Pump(redeclare package Medium = BatchMedium, length = 1, diameter = pipeDiameter, height_ab = -0.1) annotation(
            Placement(transformation(origin = {-90, -170}, extent = {{10, 10}, {-10, -10}}, rotation = 90)));
          Pipes.DynamicPipe pipePump1B1(redeclare package Medium = BatchMedium, diameter = pipeDiameter, height_ab = 3, length = 3) annotation(
            Placement(transformation(origin = {-180, 10}, extent = {{-10, 10}, {10, -10}}, rotation = 90)));
          Pipes.DynamicPipe pipePump2B2(redeclare package Medium = BatchMedium, diameter = pipeDiameter, height_ab = 3, length = 3) annotation(
            Placement(transformation(origin = {160, 10}, extent = {{-10, 10}, {10, -10}}, rotation = 90)));
          Modelica.Fluid.Examples.AST_BatchPlant.BaseClasses.TankWithTopPorts B6(redeclare package Medium = BatchMedium, V0 = 0.0001, nTopPorts = 1, height = 0.5, crossArea = 0.05, level_start = 0.02, nPorts = 1, portsData = {Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.011, height = 0)}, stiffCharacteristicForEmptyPort = false, use_HeatTransfer = true, T_start = 298, redeclare model HeatTransfer = Modelica.Fluid.Vessels.BaseClasses.HeatTransfer.IdealHeatTransfer(k = 4.9)) annotation(
            Placement(transformation(extent = {{80, -80}, {40, -40}})));
          Fittings.MultiPort multiPort(redeclare package Medium = BatchMedium, nPorts_b = 3) annotation(
            Placement(transformation(extent = {{4, 10}, {-4, -10}}, rotation = -90, origin = {-20, -228})));
          Modelica.Fluid.Fittings.TeeJunctionVolume volume4(redeclare package Medium = BatchMedium, V = 0.001) annotation(
            Placement(transformation(extent = {{-30, -260}, {-10, -240}})));
        equation
          controller.sensors.LIS_301 = B3.level;
          controller.sensors.QI_302 = 0;
//B3.medium.X[2];
          controller.sensors.LIS_501 = B5.level;
          controller.sensors.QIS_502 = 0;
//B5.medium.X[2];
          controller.sensors.TI_503 = B5.medium.T;
          controller.sensors.LIS_601 = B6.level;
          controller.sensors.TIS_602 = B6.medium.T;
          controller.sensors.LIS_701 = B7.level;
          controller.sensors.TIS_702 = B7.medium.T;
          P1_on.u = controller.actuators.P1;
          P2_on.u = controller.actuators.P2;
          V1.open = controller.actuators.V1;
          V2.open = controller.actuators.V2;
          V3.open = controller.actuators.V3;
          V4.open = controller.actuators.V4;
          V5.open = controller.actuators.V5;
          V6.open = controller.actuators.V6;
          V8.open = controller.actuators.V8;
          V9.open = controller.actuators.V9;
          V10.open = controller.actuators.V10;
          V11.open = controller.actuators.V11;
          V12.open = controller.actuators.V12;
          V15.open = controller.actuators.V15;
          V18.open = controller.actuators.V18;
          V19.open = controller.actuators.V19;
          V20.open = controller.actuators.V20;
          V21.open = controller.actuators.V21;
          V22.open = controller.actuators.V22;
          V23.open = controller.actuators.V23;
          V24.open = controller.actuators.V24;
          V25.open = controller.actuators.V25;
          HeatB5.Q_flow = if controller.actuators.T5_Heater then 20000 else 0;
          CoolingB6.Q_flow = if controller.actuators.T6_Cooling then -2000 else 0;
          CoolingB7.Q_flow = if controller.actuators.T7_Cooling then -2000 else 0;
          connect(V24.port_b, P2.port_a) annotation(
            Line(points = {{100, -250}, {120, -250}}, color = {0, 127, 255}));
          connect(V15.port_a, B5.BottomFluidPort[1]) annotation(
            Line(points = {{-90, -72}, {-90, -60.4}}, color = {0, 127, 255}));
          connect(P1_on.y, P1.N_in) annotation(
            Line(points = {{-143, -220}, {-150, -220}, {-150, -240}}, color = {0, 0, 127}));
          connect(P2_on.y, P2.N_in) annotation(
            Line(points = {{121, -220}, {130, -220}, {130, -240}}, color = {0, 0, 127}));
          connect(B4.ports[1], V12.port_b) annotation(
            Line(points = {{-90, 29}, {-90, 21}, {-90, 12}}, color = {0, 127, 255}));
          connect(CoolingB7.port, B7.heatPort) annotation(
            Line(points = {{-130, -120}, {-110, -120}}, color = {191, 0, 0}));
          connect(pipeB1B2.port_a, V4.port_b) annotation(
            Line(points = {{0, 240}, {20, 240}}, color = {0, 127, 255}));
          connect(B5.TopFluidPort[1], V12.port_a) annotation(
            Line(points = {{-90, -19.6}, {-90, -8}}, color = {0, 0, 255}));
          connect(V15.port_b, B7.topPorts[1]) annotation(
            Line(points = {{-90, -92}, {-90, -96}, {-90, -99}, {-90, -99}}, color = {0, 127, 255}));
          connect(pipePump1B1.port_a, V22.port_b) annotation(
            Line(points = {{-180, 0}, {-180, -160}}, color = {0, 127, 255}));
          connect(V25.port_b, pipePump2B2.port_a) annotation(
            Line(points = {{160, -160}, {160, 0}}, color = {0, 127, 255}));
          connect(B6.topPorts[1], B5.Condensed) annotation(
            Line(points = {{60, -39}, {60, -28}, {-29.6, -28}}, color = {0, 127, 255}));
          connect(CoolingB6.port, B6.heatPort) annotation(
            Line(points = {{100, -60}, {80, -60}}, color = {191, 0, 0}));
          connect(HeatB5.port, B5.heatPort) annotation(
            Line(points = {{-130, -40}, {-110, -40}}, color = {191, 0, 0}));
          connect(V1.port_b, volume2.port_1) annotation(
            Line(points = {{-180, 120}, {-180, 210}}, color = {0, 127, 255}));
          connect(volume8.port_2, V4.port_a) annotation(
            Line(points = {{160, 230}, {160, 240}, {40, 240}}, color = {0, 127, 255}));
          connect(volume8.port_1, V5.port_b) annotation(
            Line(points = {{160, 210}, {160, 120}}, color = {0, 127, 255}));
          connect(volume5.port_1, V21.port_b) annotation(
            Line(points = {{50, -250}, {40, -250}}, color = {0, 127, 255}));
          connect(volume5.port_2, V24.port_a) annotation(
            Line(points = {{70, -250}, {80, -250}}, color = {0, 127, 255}));
          connect(P2.port_b, V25.port_a) annotation(
            Line(points = {{140, -250}, {160, -250}, {160, -180}}, color = {0, 127, 255}));
          connect(V22.port_a, P1.port_b) annotation(
            Line(points = {{-180, -180}, {-180, -250}, {-160, -250}}, color = {0, 127, 255}));
          connect(V5.port_a, pipePump2B2.port_b) annotation(
            Line(points = {{160, 100}, {160, 20}}, color = {0, 127, 255}));
          connect(V1.port_a, pipePump1B1.port_b) annotation(
            Line(points = {{-180, 100}, {-180, 20}}, color = {0, 127, 255}));
          connect(P1.port_a, V23.port_b) annotation(
            Line(points = {{-140, -250}, {-120, -250}}, color = {0, 127, 255}));
          connect(V23.port_a, volume4.port_1) annotation(
            Line(points = {{-100, -250}, {-30, -250}}, color = {0, 127, 255}));
          connect(volume4.port_2, V21.port_a) annotation(
            Line(points = {{-10, -250}, {20, -250}}, color = {0, 127, 255}));
          connect(multiPort.port_a, volume4.port_3) annotation(
            Line(points = {{-20, -232}, {-20, -240}}, color = {0, 127, 255}));
          connect(multiPort.ports_b[3], V19.port_b) annotation(
            Line(points = {{-17.3333, -224}, {-17.3333, -200}, {0, -200}}, color = {0, 127, 255}));
          connect(V18.port_b, multiPort.ports_b[1]) annotation(
            Line(points = {{-40, -200}, {-22.6667, -200}, {-22.6667, -224}}, color = {0, 127, 255}));
          connect(V10.port_b, multiPort.ports_b[2]) annotation(
            Line(points = {{-21, -180}, {-20, -180}, {-20, -224}}, color = {0, 127, 255}));
          connect(V11.port_b, B4.topPorts[1]) annotation(
            Line(points = {{-70, 90}, {-90, 90}, {-90, 71}}, color = {0, 127, 255}));
          connect(B3.ports[1], V11.port_a) annotation(
            Line(points = {{-12, 99}, {-12, 90}, {-50, 90}}, color = {0, 127, 255}));
          connect(V20.port_b, volume5.port_3) annotation(
            Line(points = {{60, -220}, {60, -240}}, color = {0, 127, 255}));
          connect(V2.port_b, pipeB1B2.port_b) annotation(
            Line(points = {{-40, 240}, {-20, 240}}, color = {0, 127, 255}));
          connect(volume2.port_2, V2.port_a) annotation(
            Line(points = {{-180, 230}, {-180, 240}, {-60, 240}}, color = {0, 127, 255}));
          connect(volume2.port_3, V3.port_a) annotation(
            Line(points = {{-170, 220}, {-150, 220}}, color = {0, 127, 255}));
          connect(V3.port_b, B1.topPorts[1]) annotation(
            Line(points = {{-130, 220}, {-120, 220}, {-120, 230}, {-90, 230}, {-90, 221}}, color = {0, 127, 255}));
          connect(volume8.port_3, V6.port_a) annotation(
            Line(points = {{150, 220}, {130, 220}}, color = {0, 127, 255}));
          connect(V6.port_b, B2.topPorts[1]) annotation(
            Line(points = {{110, 220}, {100, 220}, {100, 230}, {70, 230}, {70, 221}}, color = {0, 127, 255}));
          connect(B2.ports[1], V9.port_a) annotation(
            Line(points = {{70, 179}, {70, 175}, {70, 175}, {70, 170}}, color = {0, 127, 255}));
          connect(V9.port_b, pipeB2B3.port_a) annotation(
            Line(points = {{70, 150}, {70, 144}, {50, 144}}, color = {0, 127, 255}));
          connect(pipeB2B3.port_b, B3.topPorts[2]) annotation(
            Line(points = {{30, 144}, {-8, 144}, {-8, 141}, {-8, 141}}, color = {0, 127, 255}));
          connect(B1.ports[1], V8.port_a) annotation(
            Line(points = {{-90, 179}, {-90, 175}, {-90, 175}, {-90, 170}}, color = {0, 127, 255}));
          connect(V8.port_b, pipeB1B3.port_a) annotation(
            Line(points = {{-90, 150}, {-90, 144}, {-70, 144}}, color = {0, 127, 255}));
          connect(B3.topPorts[1], pipeB1B3.port_b) annotation(
            Line(points = {{-12, 141}, {-12, 141}, {-12, 144}, {-50, 144}}, color = {0, 127, 255}));
          connect(B7.ports[1], pipeB7Pump.port_a) annotation(
            Line(points = {{-90, -141}, {-90, -150}, {-90, -150}, {-90, -160}}, color = {0, 127, 255}));
          connect(pipeB7Pump.port_b, V18.port_a) annotation(
            Line(points = {{-90, -180}, {-90, -200}, {-60, -200}}, color = {0, 127, 255}));
          connect(B6.ports[1], pipeB6Pump.port_a) annotation(
            Line(points = {{60, -81}, {60, -90}, {60, -90}, {60, -100}}, color = {0, 127, 255}));
          connect(pipeB6Pump.port_b, V20.port_a) annotation(
            Line(points = {{60, -120}, {60, -200}}, color = {0, 127, 255}));
          connect(V19.port_a, pipeB6Pump.port_b) annotation(
            Line(points = {{20, -200}, {30, -200}, {30, -160}, {60, -160}, {60, -120}}, color = {0, 127, 255}));
          connect(pipeB1B1.port_b, V10.port_a) annotation(
            Line(points = {{-8, 0}, {-8, -80}, {-21, -80}, {-21, -160}}, color = {0, 127, 255}));
          connect(B3.ports[2], pipeB1B1.port_a) annotation(
            Line(points = {{-8, 99}, {-8, 20}, {-8, 20}}, color = {0, 127, 255}));
          annotation(
            experiment(StopTime = 3600),
            __Dymola_Commands(file = "modelica://Modelica/Resources/Scripts/Dymola/Fluid/AST_BatchPlant_StandardWater/plot level.mos" "plot level"),
            Documentation(info = "<html>
<img src=\"modelica://Modelica/Resources/Images/Fluid/Examples/BatchPlant_StandardWater.png\" border=\"1\"
     alt=\"BatchPlant_StandardWater.png\">
</html>"),
            Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{-200, -280}, {200, 280}})));
        end BatchPlant_StandardWater;

        package BaseClasses
          extends Modelica.Icons.BasesPackage;

          block setReal "Set output signal to a time varying Real expression"
            Modelica.Blocks.Interfaces.RealInput u "Set value of Real input" annotation(
              Dialog(group = "Time varying input signal"),
              Placement(transformation(extent = {{-140, -20}, {-100, 20}})));
            annotation(
              Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, 40}, {100, -40}}, lineColor = {0, 0, 0}, fillColor = {235, 235, 235}, fillPattern = FillPattern.Solid, borderPattern = BorderPattern.Raised), Text(extent = {{-96, 15}, {96, -15}}, lineColor = {0, 0, 0}, textString = "%u"), Text(extent = {{-150, 90}, {140, 50}}, textString = "%name")}),
              Documentation(info = "<html>

</html>"));
          end setReal;

          model TankWith3InletOutletArraysWithEvaporatorCondensor "Tank with Heating and Evaporation"
            import Modelica.Fluid.Examples.AST_BatchPlant.BaseClasses.Init;
            replaceable package Medium = Modelica.Media.Water.WaterIF97_ph constrainedby Modelica.Media.Interfaces.PartialTwoPhaseMedium "Medium in the component" annotation(
               choicesAllMatching = true);
            // parameter for Tank
            parameter Modelica.SIunits.Area crossArea "Tank area";
            parameter SI.Area top_pipeArea[n_TopPorts] "Area of outlet pipe";
            parameter SI.Area side_pipeArea[n_SidePorts] "Area of outlet pipe";
            parameter SI.Area bottom_pipeArea[n_BottomPorts] "Area of outlet pipe";
            parameter Modelica.SIunits.Height height(min = 0) = 10 "Height of Tank";
            parameter SI.Volume V0 = 0 "Volume of the liquid when the level is zero";
            constant Modelica.SIunits.Acceleration g = Modelica.Constants.g_n;
            parameter Real side_heights[n_SidePorts] = zeros(n_SidePorts);
            parameter Real bottom_heights[n_BottomPorts] = zeros(n_BottomPorts);
            parameter Real top_heights[n_TopPorts] = fill(height, n_TopPorts);
            parameter SI.Height level_start(min = 0) "Initial tank level" annotation(
              Dialog(tab = "Initialization"));
            parameter Modelica.Fluid.Examples.AST_BatchPlant.BaseClasses.Init initType = Init.GuessValues "Initialization option" annotation(
              Dialog(tab = "Initialization"));
            parameter Boolean use_T_start = true "Use T_start if true, otherwise h_start" annotation(
              Dialog(tab = "Initialization"),
              Evaluate = true);
            parameter Medium.Temperature T_start = if use_T_start then 293.15 else Medium.temperature_phX(p_ambient, h_start, X_start) "Start value of temperature" annotation(
              Dialog(tab = "Initialization", enable = use_T_start));
            parameter Medium.SpecificEnthalpy h_start = if use_T_start then Medium.specificEnthalpy_pTX(p_ambient, T_start, X_start[1:Medium.nXi]) else 1e4 "Start value of specific enthalpy" annotation(
              Dialog(tab = "Initialization", enable = not use_T_start));
            parameter Medium.MassFraction X_start[Medium.nX] = Medium.reference_X "Start value of mass fractions m_i/m" annotation(
              Dialog(tab = "Initialization", enable = Medium.nXi > 0));
            parameter Medium.AbsolutePressure p_ambient = 101325 "Tank surface pressure";
            parameter Medium.Temperature T_ambient = 293.15 "Tank surface Temperature";
            parameter Integer n_TopPorts = 1 "number of Top connectors";
            parameter Integer n_SidePorts = 1 "number of side connectors";
            parameter Integer n_BottomPorts = 1 "number of bottom connectors";
            Medium.BaseProperties medium(preferredMediumStates = true, p(start = p_ambient), T(start = T_start), Xi(start = X_start[1:Medium.nXi]));
            Modelica.SIunits.Height level(stateSelect = StateSelect.prefer, min = 0, max = height) "Level height of tank";
            SI.Volume V(stateSelect = StateSelect.never) "Actual tank volume";
            SI.Energy U "Internal energy of tank volume";
            Real m(unit = "kg") "Mass of tank volume";
            Real mXi[Medium.nXi](each unit = "kg") "Component masses of the independent substances";
            // additional variables
            Real H_flow_BottomPorts[n_BottomPorts];
            Real H_flow_SidePorts[n_SidePorts];
            Real H_flow_TopPorts[n_TopPorts];
            Real m_flow_BottomPorts[n_BottomPorts];
            Real m_flow_SidePorts[n_SidePorts];
            Real m_flow_TopPorts[n_TopPorts];
            Real m_flow_BottomPorts_pos[n_BottomPorts];
            Real m_flow_SidePorts_pos[n_SidePorts];
            Real m_flow_TopPorts_pos[n_TopPorts];
            Real m_flow_pos;
            Medium.MassFlowRate mXi_flow_topPorts[n_TopPorts, Medium.nXi];
            Medium.MassFlowRate mXi_flowBottomPorts[n_BottomPorts, Medium.nXi];
            Medium.MassFlowRate mXi_flow_sidePorts[n_SidePorts, Medium.nXi];
            // Connectors and InnerTanks
            Modelica.Fluid.Interfaces.FluidPort_b BottomFluidPort[n_BottomPorts](redeclare package Medium = Medium, m_flow(each start = 0)) annotation(
              Placement(transformation(origin = {-100, -102}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
            Modelica.Fluid.Interfaces.FluidPort_a TopFluidPort[n_TopPorts](redeclare package Medium = Medium, m_flow(each start = 0)) annotation(
              Placement(transformation(extent = {{-110, 92}, {-90, 112}})));
            Modelica.Fluid.Interfaces.FluidPort_b SideFluidPort[n_SidePorts](redeclare package Medium = Medium, m_flow(each start = 0)) annotation(
              Placement(transformation(extent = {{0, -10}, {20, 10}})));
            Modelica.Fluid.Examples.AST_BatchPlant.BaseClasses.InnerTank InnerTankTop[n_TopPorts](each h = medium.h, each p_ambient = p_ambient, each d = medium.d, each Xi = medium.Xi, aboveLevel = {level - top_heights[i] for i in 1:n_TopPorts}, pipeArea = {top_pipeArea[i] for i in 1:n_TopPorts}, redeclare package Medium = Medium) annotation(
              Placement(transformation(extent = {{-140, 60}, {-120, 80}})));
            Modelica.Fluid.Examples.AST_BatchPlant.BaseClasses.InnerTank InnerTankSide[n_SidePorts](each h = medium.h, each p_ambient = p_ambient, each d = medium.d, each Xi = medium.Xi, aboveLevel = {level - side_heights[i] for i in 1:n_SidePorts}, pipeArea = {side_pipeArea[i] for i in 1:n_SidePorts}, redeclare package Medium = Medium) annotation(
              Placement(transformation(extent = {{-20, 0}, {0, 20}})));
            Modelica.Fluid.Examples.AST_BatchPlant.BaseClasses.InnerTank InnerTankBottom[n_BottomPorts](each h = medium.h, each p_ambient = p_ambient, each d = medium.d, each Xi = medium.Xi, aboveLevel = {level - bottom_heights[i] for i in 1:n_BottomPorts}, pipeArea = {bottom_pipeArea[i] for i in 1:n_BottomPorts}, redeclare package Medium = Medium) annotation(
              Placement(transformation(extent = {{-80, -80}, {-60, -60}})));
            Modelica.Fluid.Interfaces.FluidPort_b Condensed(redeclare package Medium = Medium) annotation(
              Placement(transformation(extent = {{192, 50}, {212, 70}})));
            // Heat transfer through boundary
            replaceable model HeatTransfer = Modelica.Fluid.Vessels.BaseClasses.HeatTransfer.IdealHeatTransfer constrainedby Modelica.Fluid.Vessels.BaseClasses.HeatTransfer.PartialVesselHeatTransfer "Wall heat transfer" annotation(
               Dialog(tab = "Assumptions", group = "Heat transfer", enable = use_HeatTransfer),
               choicesAllMatching = true);
            HeatTransfer heatTransfer(redeclare final package Medium = Medium, final n = 1, final states = {medium.state}, surfaceAreas = {crossArea + 2 * sqrt(crossArea * Modelica.Constants.pi) * level}, final use_k = true) annotation(
              Placement(transformation(extent = {{-10, -10}, {30, 30}}, rotation = 90, origin = {-50, -10})));
            Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a heatPort annotation(
              Placement(transformation(extent = {{-210, -10}, {-190, 10}})));
            // parameter for Evaporator
            parameter Real min_level_for_heating;
            Medium.SaturationProperties sat "State vector to compute saturation properties";
            Medium.SpecificEnthalpy h_v = Medium.dewEnthalpy(sat) "specific enthalpy of vapour";
            Medium.SpecificEnthalpy h_l = Medium.bubbleEnthalpy(sat) "specific enthalpy of liquid";
            Medium.SpecificEnthalpy h "'is'specific enthalpy of liquid";
            Medium.Density rho_v = Medium.dewDensity(sat) "density in vapour phase";
            Medium.Density rho_l = Medium.bubbleDensity(sat) "density in liquid phase";
            Medium.Density rho "'is' density in liquid phase";
          equation
            H_flow_TopPorts = InnerTankTop.H_flow;
            m_flow_TopPorts = InnerTankTop.m_flow;
            mXi_flow_topPorts = InnerTankTop.mXi_flow;
            H_flow_SidePorts = InnerTankSide.H_flow;
            m_flow_SidePorts = InnerTankSide.m_flow;
            mXi_flow_sidePorts = InnerTankSide.mXi_flow;
            H_flow_BottomPorts = InnerTankBottom.H_flow;
            m_flow_BottomPorts = InnerTankBottom.m_flow;
            mXi_flowBottomPorts = InnerTankBottom.mXi_flow;
            for i in 1:n_BottomPorts loop
              m_flow_BottomPorts_pos[i] = if m_flow_BottomPorts[i] > 0 then m_flow_BottomPorts[i] else 0;
            end for;
            for i in 1:n_SidePorts loop
              m_flow_SidePorts_pos[i] = if m_flow_SidePorts[i] > 0 then m_flow_SidePorts[i] else 0;
            end for;
            for i in 1:n_TopPorts loop
              m_flow_TopPorts_pos[i] = if m_flow_TopPorts[i] > 0 then m_flow_TopPorts[i] else 0;
            end for;
            for i in 1:n_BottomPorts loop
              connect(InnerTankBottom[i].port, BottomFluidPort[i]) annotation(
                Line(points = {{-70, -81}, {-70, -102}, {-100, -102}}, color = {0, 0, 255}));
            end for;
            for i in 1:n_TopPorts loop
              connect(InnerTankTop[i].port, TopFluidPort[i]) annotation(
                Line(points = {{-130, 59}, {-92, 59}, {-92, 102}, {-100, 102}}, color = {0, 0, 255}));
            end for;
            for i in 1:n_SidePorts loop
              connect(InnerTankSide[i].port, SideFluidPort[i]) annotation(
                Line(points = {{-10, -1}, {24, -1}, {24, 0}, {10, 0}}, color = {0, 0, 255}));
            end for;
            medium.p = p_ambient;
// Mass balance
            der(m) = sum(m_flow_BottomPorts) + sum(m_flow_SidePorts) + sum(m_flow_TopPorts) + Condensed.m_flow;
// Energy balance
            U = m * medium.h - p_ambient * V "Internal energy of fluid";
            m = V * medium.d "Mass of fluid";
            V = crossArea * level + V0 "Volume of fluid";
            mXi = m * medium.Xi "Mass of fluid components";
            sat.psat = medium.p;
            sat.Tsat = Medium.saturationTemperature(medium.p);
            if noEvent(medium.T < sat.Tsat) then
              if Medium.singleState then
                der(U) = sum(H_flow_BottomPorts) + sum(H_flow_SidePorts) + sum(H_flow_TopPorts) + Condensed.m_flow * actualStream(Condensed.h_outflow) + heatTransfer.Q_flows[1] "Mechanical work is neglected";
              else
                der(U) = sum(H_flow_BottomPorts) + sum(H_flow_SidePorts) + sum(H_flow_TopPorts) + Condensed.m_flow * actualStream(Condensed.h_outflow) - p_ambient * der(V) + heatTransfer.Q_flows[1];
              end if;
              Condensed.h_outflow = h;
              Condensed.m_flow = 0;
              rho = medium.d;
              h = medium.h;
            else
              if Medium.singleState then
                der(U) = sum(H_flow_BottomPorts) + sum(H_flow_SidePorts) + sum(H_flow_TopPorts) + Condensed.m_flow * actualStream(Condensed.h_outflow) "Mechanical work is neglected";
              else
                der(U) = sum(H_flow_BottomPorts) + sum(H_flow_SidePorts) + sum(H_flow_TopPorts) + Condensed.m_flow * actualStream(Condensed.h_outflow) - p_ambient * der(V);
              end if;
              Condensed.h_outflow = h;
              Condensed.m_flow = -heatTransfer.Q_flows[1] / (h_v - h_l);
              rho = rho_l;
//Density = liquid Density
              h = h_l;
//Enthalpy = liquid Enthalpy
              if noEvent(heatPort.Q_flow > 0.0) then
                assert(noEvent(abs(m_flow_pos) <= 0.01), "Attempt to fill tank while evaporating.");
              end if;
            end if;
            m_flow_pos = sum(m_flow_TopPorts_pos) + sum(m_flow_SidePorts_pos) + sum(m_flow_BottomPorts_pos);
            for i in 1:Medium.nXi loop
              der(mXi[i]) = sum(mXi_flowBottomPorts[:, i]) + sum(mXi_flow_sidePorts[:, i]) + sum(mXi_flow_topPorts[:, i]);
            end for;
            assert(level < height, "
    Tank is overflowing.
    ");
            assert(not (heatPort.Q_flow > 0.0 and level <= min_level_for_heating), "
    Attempting to heat an empty tank
  ");
          initial equation
            if initType == Init.GuessValues then
// no initial equations
            elseif initType == Init.InitialValues then
              level = level_start;
              if use_T_start then
                medium.T = T_start;
              else
                medium.h = h_start;
              end if;
              medium.Xi = X_start[1:Medium.nXi];
            elseif initType == Init.SteadyStateHydraulic then
              der(level) = 0;
              if use_T_start then
                medium.T = T_start;
              else
                medium.h = h_start;
              end if;
              medium.Xi = X_start[1:Medium.nXi];
            else
              assert(false, "Unsupported initialization option");
            end if;
          equation
            connect(heatPort, heatTransfer.heatPorts[1]) annotation(
              Line(points = {{-200, 0}, {-87, 0}, {-87, 0}, {-74, 0}}, color = {191, 0, 0}));
            annotation(
              Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-200, -100}, {200, 100}}), graphics = {Rectangle(extent = {{-200, 100}, {0, -100}}, lineColor = {255, 255, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.VerticalCylinder), Rectangle(extent = DynamicSelect({{-200, -100}, {0, 0}}, {{-200, -100}, {0, (-100) + 200 * level / height}}), lineColor = {0, 0, 0}, fillColor = {85, 170, 255}, fillPattern = FillPattern.VerticalCylinder), Line(points = {{-200, 100}, {-200, -100}, {0, -100}, {0, 100}}), Text(extent = {{-200, 84}, {0, 48}}, lineColor = {0, 0, 255}, textString = "%name"), Polygon(points = {{0, 100}, {200, 70}, {200, 50}, {200, 50}, {0, 80}, {0, 100}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {0, 0, 255}), Polygon(points = {{20, 98}, {30, 74}, {52, 84}, {66, 72}, {86, 78}, {98, 66}, {118, 74}, {130, 60}, {144, 70}, {152, 60}, {168, 66}, {180, 54}, {196, 74}, {190, 76}, {180, 64}, {170, 70}, {156, 66}, {148, 76}, {132, 68}, {120, 80}, {100, 74}, {88, 88}, {70, 78}, {50, 92}, {32, 82}, {28, 100}, {20, 98}, {20, 98}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {170, 255, 255}), Text(extent = {{-193, 30}, {-3, 10}}, lineColor = {0, 0, 0}, textString = "level ="), Text(extent = {{-195, -38}, {-5, -58}}, lineColor = {0, 0, 0}, textString = DynamicSelect("%level_start", String(level, minimumLength = 1, significantDigits = 2)))}),
              Documentation(info = "<html>
<p>This tank has the same geometric variables as TankWith3InletOutletArrays plus the feature of a HeatPort and the possibility of evaporation.
(Assumption: The gas is condensed immediately afterwards so that a liquid boiling fluid is created.)</p>
<p>The tank can be initialized with the following options:</p>
<ul>
<li>GuessValues: no explicit initial conditions
<li>InitialValues: initial values of temperature (or specific enthalpy), composition and level are specified
<li>SteadyStateHydraulic: initial values of temperature (or specific enthalpy) and composition are specified; the initial level is determined so that levels and pressure are at steady state.
</ul>
<p>
Full steady state initialization is not supported, because the corresponding initial equations for temperature/enthalpy are undetermined (the flow rate through the port at steady state is zero).
</p>
</html>"));
          end TankWith3InletOutletArraysWithEvaporatorCondensor;

          model InnerTank
            replaceable package Medium = Modelica.Media.Interfaces.PartialMedium "Medium in the component" annotation(
              choicesAllMatching = true);
            Modelica.Fluid.Interfaces.FluidPort_a port(redeclare package Medium = Medium) annotation(
              Placement(transformation(origin = {0, -110}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
            Boolean m_flow_negative(start = true, fixed = true) "true= mass flow out of tank";
            constant Modelica.SIunits.Acceleration g = Modelica.Constants.g_n;
            input Real aboveLevel;
            input Real d;
            input Real p_ambient;
            input Real h;
            input Medium.MassFraction Xi[Medium.nXi] "Actual mass fractions of fluid in tank" annotation(
              Dialog);
            input Real pipeArea;
            output Real H_flow;
            output Real m_flow;
            output Medium.MassFlowRate mXi_flow[Medium.nXi] "= port.mXi_flow (used to transform vector of connectors in vector of Real numbers)";
          equation
            m_flow_negative = pre(m_flow_negative) and not port.p > p_ambient or port.m_flow < (-1e-6);
            if noEvent(aboveLevel > 0) then
              port.p = aboveLevel * g * d + p_ambient - smooth(2, noEvent(if noEvent(m_flow < 0) then m_flow ^ 2 / (2 * d * pipeArea ^ 2) else 0));
            else
              if pre(m_flow_negative) then
                port.m_flow = 0;
              else
                port.p = p_ambient;
              end if;
            end if;
            H_flow = port.m_flow * actualStream(port.h_outflow);
            m_flow = port.m_flow;
            mXi_flow = port.m_flow * actualStream(port.Xi_outflow);
            port.h_outflow = h;
            port.Xi_outflow = Xi;
          end InnerTank;

          model Controller
            Modelica.Fluid.Examples.AST_BatchPlant.BaseClasses.ControllerUtilities.Port_Sensors sensors annotation(
              Placement(transformation(extent = {{-280, -40}, {-200, 40}})));
            Modelica.Fluid.Examples.AST_BatchPlant.BaseClasses.ControllerUtilities.Port_Actuators actuators annotation(
              Placement(transformation(extent = {{200, -20}, {240, 20}})));
            parameter Real w_dilution = 0.003;
            parameter Real w_concentrate = 0.005;
            parameter Real startTime = 1;
            parameter Real T5_batch_level = 0.211;
            Modelica.StateGraph.InitialStep InitialStep1 annotation(
              Placement(transformation(extent = {{-180, 90}, {-160, 110}})));
            Modelica.StateGraph.Transition Transition1(enableTimer = true, waitTime = startTime) annotation(
              Placement(transformation(extent = {{-150, 90}, {-130, 110}})));
            Modelica.StateGraph.Step Step1 annotation(
              Placement(transformation(extent = {{-120, 90}, {-100, 110}})));
            Modelica.StateGraph.Transition Transition2(condition = LIS_301 >= 0.13) annotation(
              Placement(transformation(extent = {{-90, 90}, {-70, 110}})));
            Modelica.StateGraph.Step Step2 annotation(
              Placement(transformation(extent = {{-60, 90}, {-40, 110}})));
            Modelica.StateGraph.Transition Transition3(condition = true, enableTimer = true, waitTime = 500) annotation(
              Placement(transformation(extent = {{-30, 90}, {-10, 110}})));
            Modelica.StateGraph.Step Step3 annotation(
              Placement(transformation(extent = {{0, 90}, {20, 110}})));
            Modelica.StateGraph.Transition Transition4(condition = LIS_301 <= 0.01) annotation(
              Placement(transformation(extent = {{30, 90}, {50, 110}})));
            Modelica.StateGraph.Step Step4 annotation(
              Placement(transformation(extent = {{60, 90}, {80, 110}})));
            Modelica.StateGraph.Transition Transition5(condition = T5_idle) annotation(
              Placement(transformation(extent = {{90, 90}, {110, 110}})));
            Modelica.StateGraph.Step Step5 annotation(
              Placement(transformation(extent = {{120, 90}, {140, 110}})));
            Modelica.StateGraph.Transition Transition6(condition = LIS_501 >= T5_batch_level) annotation(
              Placement(transformation(extent = {{150, 90}, {170, 110}})));
            Modelica.StateGraph.Step Step6 annotation(
              Placement(transformation(extent = {{-120, 30}, {-100, 50}})));
            Modelica.StateGraph.Transition Transition7(condition = true, enableTimer = true, waitTime = 300) annotation(
              Placement(transformation(extent = {{-90, 30}, {-70, 50}})));
            Modelica.StateGraph.Parallel Parallel1 annotation(
              Placement(transformation(extent = {{-176, -100}, {194, 0}})));
            Modelica.StateGraph.Step Step7 annotation(
              Placement(transformation(extent = {{-122, -80}, {-102, -60}})));
            Modelica.StateGraph.Step Step8 annotation(
              Placement(transformation(extent = {{-62, -80}, {-42, -60}})));
            Modelica.StateGraph.Step Step9 annotation(
              Placement(transformation(extent = {{-2, -80}, {18, -60}})));
            Modelica.StateGraph.Step Step10 annotation(
              Placement(transformation(extent = {{58, -80}, {78, -60}})));
            Modelica.StateGraph.Step Step11 annotation(
              Placement(transformation(extent = {{118, -80}, {138, -60}})));
            Modelica.StateGraph.Step Step12 annotation(
              Placement(transformation(extent = {{-62, -40}, {-42, -20}})));
            Modelica.StateGraph.Step Step13 annotation(
              Placement(transformation(extent = {{-2, -40}, {18, -20}})));
            Modelica.StateGraph.Step Step14 annotation(
              Placement(transformation(extent = {{58, -40}, {78, -20}})));
            Modelica.StateGraph.Transition Transition8(condition = T7_idle) annotation(
              Placement(transformation(extent = {{-92, -80}, {-72, -60}})));
            Modelica.StateGraph.Transition Transition9(condition = LIS_501 <= 0.01) annotation(
              Placement(transformation(extent = {{-32, -80}, {-12, -60}})));
            Modelica.StateGraph.Transition Transition10(condition = TIS_702 <= 298) annotation(
              Placement(transformation(extent = {{28, -80}, {48, -60}})));
            Modelica.StateGraph.Transition Transition11(condition = LIS_701 <= 0.01) annotation(
              Placement(transformation(extent = {{88, -80}, {108, -60}})));
            Modelica.StateGraph.Transition Transition12(condition = TIS_602 <= 298) annotation(
              Placement(transformation(extent = {{-32, -40}, {-12, -20}})));
            Modelica.StateGraph.Transition Transition13(condition = LIS_601 <= 0.01) annotation(
              Placement(transformation(extent = {{28, -40}, {48, -20}})));
            Real LIS_301;
            Real LIS_501;
            Real LIS_601;
            Real LIS_701;
            Real QI_302;
            Real QIS_502;
            Real TIS_602;
            Real TIS_702;
            Boolean T5_idle;
            Boolean T7_idle;
            Modelica.StateGraph.TransitionWithSignal TransitionWithSignal1 annotation(
              Placement(transformation(origin = {-2, -150}, extent = {{-10, -10}, {10, 10}}, rotation = 180)));
            Modelica.Blocks.Sources.BooleanExpression BooleanExpression1(y = time > 2500) annotation(
              Placement(transformation(extent = {{-104, -148}, {-18, -116}})));
            inner Modelica.StateGraph.StateGraphRoot stateGraphRoot annotation(
              Placement(transformation(extent = {{-180, 140}, {-160, 160}})));
          equation
            LIS_301 = sensors.LIS_301;
            LIS_501 = sensors.LIS_501;
            LIS_601 = sensors.LIS_601;
            LIS_701 = sensors.LIS_701;
            QI_302 = sensors.QI_302;
            QIS_502 = sensors.QIS_502;
            TIS_602 = sensors.TIS_602;
            TIS_702 = sensors.TIS_702;
            T5_idle = not actuators.V12 and not actuators.V15 and not actuators.T5_Heater and sensors.LIS_501 < 0.01;
            T7_idle = not actuators.V15 and not actuators.V18 and not actuators.T7_Cooling and sensors.LIS_701 < 0.01;
            actuators.P1 = Step10.active;
            actuators.P2 = Step13.active;
            actuators.T5_Heater = Step6.active;
            actuators.T7_Cooling = Step9.active;
            actuators.T6_Cooling = Step12.active;
            actuators.V1 = Step10.active;
            actuators.V2 = false;
            actuators.V3 = Step10.active;
            actuators.V4 = false;
            actuators.V5 = Step13.active;
            actuators.V6 = Step13.active;
            actuators.V8 = Step1.active;
            actuators.V9 = Step2.active;
            actuators.V10 = false;
            actuators.V11 = Step3.active;
            actuators.V12 = Step5.active;
            actuators.V15 = Step8.active;
            actuators.V18 = Step10.active;
            actuators.V19 = false;
            actuators.V20 = Step13.active;
            actuators.V21 = false;
            actuators.V22 = Step10.active;
            actuators.V23 = Step10.active;
            actuators.V25 = Step13.active;
            actuators.V24 = Step13.active;
            connect(InitialStep1.outPort[1], Transition1.inPort) annotation(
              Line(points = {{-159.5, 100}, {-144, 100}}));
            connect(Transition1.outPort, Step1.inPort[1]) annotation(
              Line(points = {{-138.5, 100}, {-121, 100}}));
            connect(Step1.outPort[1], Transition2.inPort) annotation(
              Line(points = {{-99.5, 100}, {-84, 100}}));
            connect(Transition2.outPort, Step2.inPort[1]) annotation(
              Line(points = {{-78.5, 100}, {-61, 100}}));
            connect(Step2.outPort[1], Transition3.inPort) annotation(
              Line(points = {{-39.5, 100}, {-24, 100}}));
            connect(Transition3.outPort, Step3.inPort[1]) annotation(
              Line(points = {{-18.5, 100}, {-1, 100}}));
            connect(Step3.outPort[1], Transition4.inPort) annotation(
              Line(points = {{20.5, 100}, {36, 100}}));
            connect(Transition4.outPort, Step4.inPort[1]) annotation(
              Line(points = {{41.5, 100}, {59, 100}}));
            connect(Step4.outPort[1], Transition5.inPort) annotation(
              Line(points = {{80.5, 100}, {96, 100}}));
            connect(Transition5.outPort, Step5.inPort[1]) annotation(
              Line(points = {{101.5, 100}, {119, 100}}));
            connect(Step5.outPort[1], Transition6.inPort) annotation(
              Line(points = {{140.5, 100}, {156, 100}}));
            connect(Transition6.outPort, Step6.inPort[1]) annotation(
              Line(points = {{161.5, 100}, {184, 100}, {184, 70}, {-160, 70}, {-160, 40}, {-121, 40}}));
            connect(Step6.outPort[1], Transition7.inPort) annotation(
              Line(points = {{-99.5, 40}, {-84, 40}}));
            connect(Step12.inPort[1], Parallel1.split[1]) annotation(
              Line(points = {{-63, -30}, {-134.375, -30}, {-134.375, -25}}));
            connect(Step12.outPort[1], Transition12.inPort) annotation(
              Line(points = {{-41.5, -30}, {-26, -30}}));
            connect(Transition12.outPort, Step13.inPort[1]) annotation(
              Line(points = {{-20.5, -30}, {-3, -30}}));
            connect(Step13.outPort[1], Transition13.inPort) annotation(
              Line(points = {{18.5, -30}, {34, -30}}));
            connect(Transition13.outPort, Step14.inPort[1]) annotation(
              Line(points = {{39.5, -30}, {57, -30}}));
            connect(Step14.outPort[1], Parallel1.join[1]) annotation(
              Line(points = {{78.5, -30}, {152.375, -30}, {152.375, -25}}));
            connect(Step7.inPort[1], Parallel1.split[2]) annotation(
              Line(points = {{-123, -70}, {-138, -70}, {-138, -75}, {-134.375, -75}}));
            connect(Step7.outPort[1], Transition8.inPort) annotation(
              Line(points = {{-101.5, -70}, {-86, -70}}));
            connect(Transition8.outPort, Step8.inPort[1]) annotation(
              Line(points = {{-80.5, -70}, {-63, -70}}));
            connect(Step8.outPort[1], Transition9.inPort) annotation(
              Line(points = {{-41.5, -70}, {-26, -70}}));
            connect(Transition9.outPort, Step9.inPort[1]) annotation(
              Line(points = {{-20.5, -70}, {-3, -70}}));
            connect(Step9.outPort[1], Transition10.inPort) annotation(
              Line(points = {{18.5, -70}, {34, -70}}));
            connect(Transition10.outPort, Step10.inPort[1]) annotation(
              Line(points = {{39.5, -70}, {57, -70}}));
            connect(Step10.outPort[1], Transition11.inPort) annotation(
              Line(points = {{78.5, -70}, {94, -70}}));
            connect(Transition11.outPort, Step11.inPort[1]) annotation(
              Line(points = {{99.5, -70}, {117, -70}}));
            connect(Step11.outPort[1], Parallel1.join[2]) annotation(
              Line(points = {{138.5, -70}, {154, -70}, {154, -75}, {152.375, -75}}));
            connect(Transition7.outPort, Parallel1.inPort) annotation(
              Line(points = {{-78.5, 40}, {-40, 40}, {-40, 20}, {-190, 20}, {-190, -50}, {-181.55, -50}}));
            connect(TransitionWithSignal1.inPort, Parallel1.outPort) annotation(
              Line(points = {{2, -150}, {208, -150}, {208, -50}, {197.7, -50}}));
            connect(TransitionWithSignal1.outPort, InitialStep1.inPort[1]) annotation(
              Line(points = {{-3.5, -150}, {-194, -150}, {-194, 100}, {-181, 100}}, color = {0, 0, 0}));
            connect(BooleanExpression1.y, TransitionWithSignal1.condition) annotation(
              Line(points = {{-13.7, -132}, {-2, -132}, {-2, -138}}, color = {255, 0, 255}));
            annotation(
              Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-200, -200}, {200, 200}}), graphics = {Rectangle(extent = {{-200, 200}, {200, -200}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-288, 286}, {262, 208}}, lineColor = {0, 0, 255}, textString = "%name"), Line(points = {{-48, 0}, {0, 0}}), Rectangle(extent = {{-170, 60}, {-50, -60}}, lineColor = {0, 0, 0}), Line(points = {{0, 40}, {0, -40}}), Line(points = {{0, 0}, {26, 0}}), Polygon(points = {{26, 10}, {50, 0}, {26, -10}, {26, 10}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Rectangle(extent = {{50, 60}, {170, -60}}, lineColor = {0, 0, 0}), Polygon(points = {{-24, 10}, {0, 0}, {-24, -10}, {-24, 10}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid)}),
              Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{-200, -160}, {200, 160}})));
          end Controller;

          package ControllerUtilities
            extends Modelica.Icons.Package;

            class Adapter_Inference
              Port_IdleTanks idleTanks;
            end Adapter_Inference;

            class Adapter_Superposition
              Port_Actuators actuators;
            end Adapter_Superposition;

            class Block_Recipe_TBD
              parameter Real startTime;
              parameter Real w_dilution = 0.003;
              parameter Real w_concentrat = 0.005;
              parameter Real T3_batch_level = 0.1273;
              parameter Real T5_batch_level = 0.211;
              Boolean trig;
              Boolean S0(start = true);
              Boolean S1;
              Boolean S2;
              Boolean S3;
              Boolean S4;
              Boolean S5;
              Boolean S6;
              Boolean S7;
              Boolean S8;
              Boolean S9;
              Boolean S10;
              Boolean S11;
              Boolean S12;
              Boolean S13;
              Boolean S14;
              Boolean tr0;
              Boolean tr1;
              Boolean tr2;
              Boolean tr3;
              Boolean tr4;
              Boolean tr5;
              Boolean tr6;
              Boolean tr7;
              Boolean tr8;
              Boolean tr9;
              Boolean tr10;
              Boolean tr11;
              Boolean tr12;
              Boolean tr13;
              Port_Actuators act annotation(
                Placement(transformation(extent = {{-110, -10}, {-90, 10}})));
            end Block_Recipe_TBD;

            class BlockMain
              Boolean trig;
              Port_Actuators actuators annotation(
                Placement(transformation(extent = {{90, -10}, {110, 10}})));
              Block_Recipe_TBD Recipe1 annotation(
                Placement(transformation(extent = {{-50, 10}, {-10, 50}})));
              Block_Recipe_TBD Recipe2 annotation(
                Placement(transformation(extent = {{10, 10}, {50, 50}})));
              Adapter_Inference Inference annotation(
                Placement(transformation(extent = {{-50, -50}, {-10, -10}})));
              Adapter_Superposition Superposition annotation(
                Placement(transformation(extent = {{10, -50}, {50, -10}})));
            end BlockMain;

            class Buffer_Recipe_TBD
              Port_Actuators act;
              Boolean S0;
              Boolean S1;
              Boolean S2;
              Boolean S3;
              Boolean S4;
              Boolean S5;
              Boolean S6;
              Boolean S7;
              Boolean S8;
              Boolean S9;
              Boolean S10;
              Boolean S11;
              Boolean S12;
              Boolean S13;
              Boolean S14;
            end Buffer_Recipe_TBD;

            class BufferMain
              Buffer_Recipe_TBD Recipe1;
              Buffer_Recipe_TBD Recipe2;
            end BufferMain;

            connector Port_Actuators
              output Boolean P1;
              output Boolean P2;
              output Boolean T5_Heater;
              output Boolean T7_Cooling;
              output Boolean T6_Cooling;
              output Boolean V1;
              output Boolean V2;
              output Boolean V3;
              output Boolean V4;
              output Boolean V5;
              output Boolean V6;
              output Boolean V8;
              output Boolean V9;
              output Boolean V10;
              output Boolean V11;
              output Boolean V12;
              output Boolean V15;
              output Boolean V18;
              output Boolean V19;
              output Boolean V20;
              output Boolean V21;
              output Boolean V22;
              output Boolean V23;
              output Boolean V24;
              output Boolean V25;
              annotation(
                Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{-100, 100}, {100, 0}, {-100, -100}, {-100, 100}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid)}),
                Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{0, 50}, {100, 0}, {0, -50}, {0, 50}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid)}));
            end Port_Actuators;

            connector Port_IdleTanks
              Boolean T5_idle;
              Boolean T7_idle;
            end Port_IdleTanks;

            connector Port_Sensors
              input Real LIS_301;
              input Real QI_302;
              input Real LIS_501;
              input Real QIS_502;
              input Real TI_503;
              input Real LIS_601;
              input Real TIS_602;
              input Real LIS_701;
              input Real TIS_702;
              annotation(
                Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{-100, 100}, {-100, -100}, {100, 0}, {-100, 100}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {215, 215, 215}, fillPattern = FillPattern.Solid)}),
                Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{0, 50}, {0, -50}, {100, 0}, {0, 50}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {215, 215, 215}, fillPattern = FillPattern.Solid)}));
            end Port_Sensors;
          end ControllerUtilities;

          type Init = enumeration(GuessValues "GuessValues -- Guess values (not fixed) for p, T or h, X, C", InitialValues "InitialValues -- Initial values for p, T or h, X, C", SteadyStateMomentum "SteadyStateMomentum: Steady state momentum", SteadyStateHydraulic "SteadyStateHydraulic -- Hydraulic steady state (der(p)=0), guess value for p, initial values for T or h, X, C", SteadyState "SteadyState -- Steady state (guess values for p, T or h, X, C)") "Enumeration to define initialization options" annotation(
            Documentation(info = "<html>
<p>
Integer type that can have the following values
(to be selected via choices menu):
</p>

<table border=1 cellspacing=0 cellpadding=2>
<tr><th><b>Types.Init.</b></th><th><b>Meaning</b></th></tr>
<tr><td>GuessValues</td>
    <td>GuessValues -- Guess values (not fixed) for p, T or h, X, C</td></tr>

<tr><td>InitialValues</td>
    <td>Initial values for p, T or h, X, C</td></tr>

<tr><td>SteadyStateMomentum</td>
    <td>Steady state momentum</td></tr>

<tr><td>SteadyStateHydraulic</td>
    <td>Hydraulic steady state (der(p)=0), guess value for p,
        initial values for T or h, X, C</td></tr>

<tr><td>SteadyState</td>
    <td>Steady state (guess values for p, T or h, X, C)</td></tr>
</table>
</html>"));

          model TankWithTopPorts "Tank with inlet/outlet ports and with inlet ports at the top"
            import Modelica.Constants;
            import Modelica.Fluid.Fittings.BaseClasses.lossConstant_D_zeta;
            import Modelica.Fluid.Utilities.regRoot2;
            import Modelica.Fluid.Vessels.BaseClasses.VesselPortsData;
            SI.Length level(stateSelect = StateSelect.prefer, start = level_start) "Fluid level in the tank";
            //Tank geometry
            parameter SI.Height height "Maximum level of tank before it overflows";
            parameter SI.Area crossArea "Area of tank";
            parameter SI.Volume V0 = 0 "Volume of the liquid when level = 0";
            //Ambient
            parameter Medium.AbsolutePressure p_ambient = system.p_ambient "Tank surface pressure" annotation(
              Dialog(tab = "Assumptions", group = "Ambient"));
            parameter Medium.Temperature T_ambient = system.T_ambient "Tank surface Temperature" annotation(
              Dialog(tab = "Assumptions", group = "Ambient"));
            //Initialization
            parameter SI.Height level_start(min = 0) = 0.5 * height "Start value of tank level" annotation(
              Dialog(tab = "Initialization"));
            //Mass and energy balance
            extends Modelica.Fluid.Interfaces.PartialLumpedVolume(final fluidVolume = V, final initialize_p = false, final p_start = p_ambient);
            //Port definitions
            parameter Integer nTopPorts = 0 "Number of inlet ports above height (>= 1)" annotation(
              Dialog(connectorSizing = true));
            Vessels.BaseClasses.VesselFluidPorts_a topPorts[nTopPorts](redeclare package Medium = Medium, m_flow(each start = 0, each min = 0)) "Inlet ports over height at top of tank (fluid flows only from the port in to the tank)" annotation(
              Placement(transformation(extent = {{-20, 0}, {20, 10}}, origin = {0, 100})));
            parameter Integer nPorts = 0 "Number of inlet/outlet ports (on bottom and on the side)" annotation(
              Dialog(connectorSizing = true));
            parameter Modelica.Fluid.Vessels.BaseClasses.VesselPortsData portsData[nPorts] "Data of inlet/outlet ports at side and bottom of tank";
            Vessels.BaseClasses.VesselFluidPorts_b ports[nPorts](redeclare package Medium = Medium, m_flow(each start = 0)) "inlet/outlet ports at bottom or side of tank (fluid flows in to or out of port; a port might be above the fluid level)" annotation(
              Placement(transformation(extent = {{-20, 0}, {20, -10}}, origin = {0, -100})));
            // Heat transfer through boundary
            parameter Boolean use_HeatTransfer = false "= true to use the HeatTransfer model" annotation(
              Dialog(tab = "Assumptions", group = "Heat transfer"));
            replaceable model HeatTransfer = Modelica.Fluid.Vessels.BaseClasses.HeatTransfer.IdealHeatTransfer constrainedby Modelica.Fluid.Vessels.BaseClasses.HeatTransfer.PartialVesselHeatTransfer "Wall heat transfer" annotation(
               Dialog(tab = "Assumptions", group = "Heat transfer", enable = use_HeatTransfer),
               choicesAllMatching = true);
            HeatTransfer heatTransfer(redeclare final package Medium = Medium, final n = 1, final states = {medium.state}, surfaceAreas = {crossArea + 2 * sqrt(crossArea * Modelica.Constants.pi) * level}, final use_k = use_HeatTransfer) annotation(
              Placement(transformation(extent = {{-10, -10}, {30, 30}}, rotation = 90, origin = {-50, -10})));
            Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a heatPort if use_HeatTransfer annotation(
              Placement(transformation(extent = {{-110, -10}, {-90, 10}})));
            // Advanced
            parameter Real hysteresisFactor(min = 0) = 0.1 "Hysteresis for empty pipe = diameter*hysteresisFactor" annotation(
              Dialog(tab = "Advanced", group = "Port properties"));
            parameter Boolean stiffCharacteristicForEmptyPort = false "=true, if steep pressure loss characteristic for empty pipe port" annotation(
              Dialog(tab = "Advanced", group = "Port properties"),
              Evaluate = true);
            parameter Real zetaLarge(min = 0) = 1e5 "Large pressure loss factor if mass flows out of empty pipe port" annotation(
              Dialog(tab = "Advanced", group = "Port properties", enable = stiffCharacteristicForEmptyPort));
            parameter SI.MassFlowRate m_flow_small(min = 0) = system.m_flow_small "Regularization range at zero mass flow rate" annotation(
              Dialog(tab = "Advanced", group = "Port properties", enable = stiffCharacteristicForEmptyPort));
            // Tank properties
            SI.Volume V(stateSelect = StateSelect.never) "Actual tank volume";
            Medium.EnthalpyFlowRate H_flow_top[nTopPorts] "Enthalpy flow rates from the top ports in to the tank";
            Medium.EnthalpyFlowRate port_b_H_flow_bottom[nPorts] "Enthalpy flow rates from the bottom ports in to the tank";
            Medium.MassFlowRate mXi_flow_top[nTopPorts, Medium.nXi] "Substance mass flow rates from the top ports into the tank";
            Medium.MassFlowRate port_b_mXi_flow_bottom[nPorts, Medium.nXi] "Substance mass flow rates from the bottom ports into the tank";
            Medium.MassFlowRate mC_flow_top[nTopPorts, Medium.nC] "Trace substance mass flow rates from the top ports into the tank";
            Medium.MassFlowRate port_b_mC_flow_bottom[nPorts, Medium.nC] "Trace substance mass flow rates from the bottom ports into the tank";
          protected
            SI.Area bottomArea[nPorts];
            SI.Diameter ports_emptyPipeHysteresis[nPorts];
            SI.Length levelAbovePort[nPorts] "Height of fluid over bottom ports";
            Boolean ports_m_flow_out[nPorts](each start = true, each fixed = true);
            Boolean aboveLevel[nPorts] "= true, if level >= ports[i].height";
            Real zetas_out[nPorts];
            Modelica.Blocks.Interfaces.RealInput portsData_diameter[nPorts] = portsData.diameter if nPorts > 0;
            Modelica.Blocks.Interfaces.RealInput portsData_diameter2[nPorts];
            Modelica.Blocks.Interfaces.RealInput portsData_height[nPorts] = portsData.height if nPorts > 0;
            Modelica.Blocks.Interfaces.RealInput portsData_height2[nPorts];
          equation
            assert(level <= height, "Tank starts to overflow (level = height = " + String(level) + ")");
            assert(m >= 0, "Mass in tank is zero");
// Compute constant data
            connect(portsData_diameter, portsData_diameter2);
            connect(portsData_height, portsData_height2);
            for i in 1:nPorts loop
              bottomArea[i] = Constants.pi * (portsData_diameter2[i] / 2) ^ 2;
              ports_emptyPipeHysteresis[i] = portsData_diameter2[i] * hysteresisFactor;
            end for;
// Only one connection allowed to a port to avoid unwanted ideal mixing
/*
for i in 1:nTopPorts loop
  assert(cardinality(topPorts[i]) <= 1,"
topPorts[" + String(i) + "] of volume can at most be connected to one component.
If two or more connections are present, ideal mixing takes
place with these connections which is usually not the intention
of the modeller.
");
end for;

for i in 1:nPorts loop
  assert(cardinality(ports[i]) <= 1,"
ports[" + String(i) + "] of volume can at most be connected to one component.
If two or more connections are present, ideal mixing takes
place with these connections which is usually not the intention
of the modeller.
");
end for;
*/
// Total quantities
            medium.p = p_ambient;
            V = crossArea * level + V0 "Volume of fluid";
// Mass balances
            mb_flow = sum(topPorts.m_flow) + sum(ports.m_flow);
            for i in 1:Medium.nXi loop
              mbXi_flow[i] = sum(mXi_flow_top[:, i]) + sum(port_b_mXi_flow_bottom[:, i]);
            end for;
            for i in 1:Medium.nC loop
              mbC_flow[i] = sum(mC_flow_top[:, i]) + sum(port_b_mC_flow_bottom[:, i]);
            end for;
// Energy balance
            Hb_flow = sum(H_flow_top) + sum(port_b_H_flow_bottom);
            Qb_flow = heatTransfer.Q_flows[1];
            if Medium.singleState or energyDynamics == Types.Dynamics.SteadyState then
              Wb_flow = 0 "Mechanical work is neglected, since also neglected in medium model (otherwise unphysical small temperature change, if tank level changes)";
            else
              Wb_flow = -p_ambient * der(V);
            end if;
// Properties at top ports
            for i in 1:nTopPorts loop
// It is assumed that fluid flows only from one of the top ports in to the tank and never vice versa
              H_flow_top[i] = topPorts[i].m_flow * actualStream(topPorts[i].h_outflow);
              mXi_flow_top[i, :] = topPorts[i].m_flow * actualStream(topPorts[i].Xi_outflow);
              mC_flow_top[i, :] = topPorts[i].m_flow * actualStream(topPorts[i].C_outflow);
              topPorts[i].p = p_ambient;
              topPorts[i].h_outflow = h_start;
              topPorts[i].Xi_outflow = X_start[1:Medium.nXi];
              topPorts[i].C_outflow = C_start;
/*
       assert(topPorts[i].m_flow > -1, "Mass flows out of tank via topPorts[" + String(i) + "]\n" +
                                         "This indicates a wrong model");
*/
            end for;
// Properties at bottom ports
            for i in 1:nPorts loop
              port_b_H_flow_bottom[i] = ports[i].m_flow * actualStream(ports[i].h_outflow);
              port_b_mXi_flow_bottom[i, :] = ports[i].m_flow * actualStream(ports[i].Xi_outflow);
              port_b_mC_flow_bottom[i, :] = ports[i].m_flow * actualStream(ports[i].C_outflow);
              aboveLevel[i] = level >= portsData_height2[i] + ports_emptyPipeHysteresis[i] or pre(aboveLevel[i]) and level >= portsData_height2[i] - ports_emptyPipeHysteresis[i];
              levelAbovePort[i] = if aboveLevel[i] then level - portsData_height2[i] else 0;
              ports[i].h_outflow = medium.h;
              ports[i].Xi_outflow = medium.Xi;
              ports[i].C_outflow = C;
              if stiffCharacteristicForEmptyPort then
// If port is above fluid level, use large zeta if fluid flows out of port (= small mass flow rate)
                zetas_out[i] = 1 + (if aboveLevel[i] then 0 else zetaLarge);
                ports[i].p = p_ambient + levelAbovePort[i] * system.g * medium.d + Modelica.Fluid.Utilities.regSquare2(ports[i].m_flow, m_flow_small, lossConstant_D_zeta(portsData_diameter2[i], 0.01) / medium.d, lossConstant_D_zeta(portsData_diameter2[i], zetas_out[i]) / medium.d);
                ports_m_flow_out[i] = false;
              else
// Handling according to Remelhe/Poschlad
                ports_m_flow_out[i] = pre(ports_m_flow_out[i]) and not ports[i].p > p_ambient or ports[i].m_flow < (-1e-6);
                if aboveLevel[i] then
                  ports[i].p = p_ambient + levelAbovePort[i] * system.g * medium.d - smooth(2, noEvent(if ports[i].m_flow < 0 then ports[i].m_flow ^ 2 / (2 * medium.d * bottomArea[i] ^ 2) else 0));
                else
                  if pre(ports_m_flow_out[i]) then
                    ports[i].m_flow = 0;
                  else
                    ports[i].p = p_ambient;
                  end if;
                end if;
                zetas_out[i] = 0;
              end if;
            end for;
          initial equation
            for i in 1:nPorts loop
              pre(aboveLevel[i]) = level_start >= portsData_height2[i];
            end for;
            if massDynamics == Types.Dynamics.FixedInitial then
              level = level_start;
            elseif massDynamics == Types.Dynamics.SteadyStateInitial then
              der(level) = 0;
            end if;
          equation
            connect(heatPort, heatTransfer.heatPorts[1]) annotation(
              Line(points = {{-100, 0}, {-87, 0}, {-87, 0}, {-74, 0}}, color = {191, 0, 0}));
            annotation(
              defaultComponentName = "tank",
              Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}, initialScale = 0.2), graphics = {Rectangle(extent = {{-100, -100}, {100, 100}}, lineColor = {255, 255, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.VerticalCylinder), Rectangle(extent = DynamicSelect({{-100, -100}, {100, 0}}, {{-100, -100}, {100, (-100) + 200 * level / height}}), lineColor = {0, 0, 0}, fillColor = {85, 170, 255}, fillPattern = FillPattern.VerticalCylinder), Text(extent = {{-94, 90}, {95, 60}}, lineColor = {0, 0, 255}, textString = "%name"), Text(extent = {{-95, 41}, {95, 21}}, lineColor = {0, 0, 0}, textString = "level ="), Line(points = {{-100, 100}, {-100, -100}, {100, -100}, {100, 100}}, color = {0, 0, 0}), Text(extent = {{-95, -39}, {95, -59}}, lineColor = {0, 0, 0}, textString = DynamicSelect("%level_start", String(level, minimumLength = 1, significantDigits = 2)))}),
              Documentation(info = "<html>
<p>
Model of a tank that is open to the environment at the fixed pressure
<code>p_ambient</code>.
The tank is filled with a single or multiple-substance liquid,
assumed to have uniform temperature and mass fractions.
</p>

<p>
At the top of the tank over the maximal fill level <b>height</b>
a vector of FluidPorts, called <b>topPorts</b>, is present.
The assumption is made that fluid flows always in to the tank via these
ports (and never back in to the connector).
</p>

<p>
The vector of connectors <b>ports</b> are fluid ports at the bottom
and side of the tank at a definable height. Fluid can flow either out
of or in to this port. The fluid level of the tank may be below
one of these ports. This case is approximated by introducing a
large pressure flow coefficient so that the mass flow rate
through this port is very small in this case.
</p>

<p>
If the tank starts to over flow (i.e., level > height), an
assertion is triggered.
</p>

<p>
When the diagram layer is open in the plot environment, the
level of the tank is dynamically visualized. Note, the speed
of the diagram animation in Dymola can be set via command
<b>animationSpeed</b>(), e.g., animationSpeed(speed = 10)
</p>
</html>", revisions = "<html>
<ul>
<li><i>Dec. 12, 2008</i> by Ruediger Franke: replace energy and mass balances with
   common definition in BaseClasses.PartialLumpedVolume</li>
<li><i>Dec. 8, 2008</i> by Michael Wetter (LBNL):<br>
Implemented trace substances and missing equation for outflow of multi substance media at top port.</li>
<li><i>Jul. 29, 2006</i> by Martin Otter (DLR):<br>
   Improved handling of ports that are above the fluid level and
   simpler implementation.</li>

<li><i>Jan. 6, 2006</i> by Katja Poschlad, Manuel Remelhe (AST Uni Dortmund),
   Martin Otter (DLR):<br>
   Implementation based on former tank model but with several improvements
   (top, bottom, side ports; correctly treating kinetic energy for outlet
   and total dissipation for inlet; ports can be above the fluid level).</li>
</ul>
</html>"));
          end TankWithTopPorts;
        end BaseClasses;

        package Test "Test of used tank models"
          extends Modelica.Icons.ExamplesPackage;

          model OneTank "Tank with one time-varying top inlet mass flow rate and a bottom outlet into the ambient"
            import Modelica.SIunits.Conversions.from_bar;
            extends Modelica.Icons.Example;
            Modelica.Fluid.Examples.AST_BatchPlant.BaseClasses.TankWithTopPorts tank(redeclare package Medium = Modelica.Media.Water.ConstantPropertyLiquidWater, crossArea = 1, height = 1, portsData = {Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.1, height = 0)}, V0 = 0.1, nTopPorts = 1, nPorts = 1, level_start = 0.8, stiffCharacteristicForEmptyPort = false, hysteresisFactor = 0.01) annotation(
              Placement(transformation(extent = {{0, 0}, {40, 40}})));
            Sources.MassFlowSource_T flowSource(nPorts = 1, redeclare package Medium = Modelica.Media.Water.ConstantPropertyLiquidWater, m_flow = 20, T = system.T_ambient, use_m_flow_in = true) annotation(
              Placement(transformation(extent = {{-12, 42}, {8, 62}})));
            inner Modelica.Fluid.System system annotation(
              Placement(transformation(extent = {{70, 72}, {90, 92}})));
            Modelica.Fluid.Sources.Boundary_pT ambient_fixed(nPorts = 1, redeclare package Medium = Modelica.Media.Water.ConstantPropertyLiquidWater, p = system.p_ambient, T = system.T_ambient) annotation(
              Placement(transformation(extent = {{-14, -50}, {6, -30}})));
            Modelica.Fluid.Pipes.StaticPipe pipe(redeclare package Medium = Modelica.Media.Water.ConstantPropertyLiquidWater, length = 1, diameter = 0.1, height_ab = -1) annotation(
              Placement(transformation(origin = {20, -18}, extent = {{10, -10}, {-10, 10}}, rotation = 90)));
            Modelica.Blocks.Sources.TimeTable timeTable(table = [0, 0; 10, 0; 10, 40; 20, 40; 20, 10; 50, 10; 50, 0; 60, 0; 60, 20; 70, 20; 80, 55; 80, 0; 100, 0]) annotation(
              Placement(transformation(extent = {{-60, 60}, {-40, 80}})));
          equation
            connect(flowSource.ports[1], tank.topPorts[1]) annotation(
              Line(points = {{8, 52}, {20, 52}, {20, 41}}, color = {0, 127, 255}));
            connect(tank.ports[1], pipe.port_a) annotation(
              Line(points = {{20, -1}, {20, -8}}, color = {0, 127, 255}));
            connect(pipe.port_b, ambient_fixed.ports[1]) annotation(
              Line(points = {{20, -28}, {20, -40}, {6, -40}}, color = {0, 127, 255}));
            connect(timeTable.y, flowSource.m_flow_in) annotation(
              Line(points = {{-39, 70}, {-24, 70}, {-24, 60}, {-12, 60}}, color = {0, 0, 127}));
            annotation(
              experiment(StopTime = 100));
          end OneTank;

          model TwoTanks
            import Modelica.SIunits.Conversions.from_bar;
            extends Modelica.Icons.Example;
            parameter Boolean stiffCharacteristicForEmptyPort = true;
            inner Modelica.Fluid.System system(energyDynamics = Modelica.Fluid.Types.Dynamics.FixedInitial, massDynamics = Modelica.Fluid.Types.Dynamics.FixedInitial) annotation(
              Placement(transformation(extent = {{40, 62}, {60, 82}})));
            Modelica.Fluid.Examples.AST_BatchPlant.BaseClasses.TankWithTopPorts tank1(redeclare package Medium = Modelica.Media.Water.ConstantPropertyLiquidWater, stiffCharacteristicForEmptyPort = stiffCharacteristicForEmptyPort, crossArea = 1, height = 4, level_start = 3, T_start = Modelica.SIunits.Conversions.from_degC(50), nPorts = 1, portsData = {Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.1, height = 0)}) annotation(
              Placement(transformation(extent = {{-80, 0}, {-40, 40}})));
            Modelica.Fluid.Examples.AST_BatchPlant.BaseClasses.TankWithTopPorts tank2(redeclare package Medium = Modelica.Media.Water.ConstantPropertyLiquidWater, stiffCharacteristicForEmptyPort = stiffCharacteristicForEmptyPort, crossArea = 1, height = 4, level_start = 1, T_start = Modelica.SIunits.Conversions.from_degC(100), nPorts = 1, portsData = {Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.1, height = 0)}) annotation(
              Placement(transformation(extent = {{0, 0}, {40, 40}})));
            Modelica.Fluid.Pipes.StaticPipe pipe(redeclare package Medium = Modelica.Media.Water.ConstantPropertyLiquidWater, length = 1, diameter = 0.1) annotation(
              Placement(transformation(extent = {{-30, -30}, {-10, -10}})));
          equation
            connect(tank1.ports[1], pipe.port_a) annotation(
              Line(points = {{-60, -1}, {-60, -20}, {-30, -20}}, color = {0, 127, 255}));
            connect(pipe.port_b, tank2.ports[1]) annotation(
              Line(points = {{-10, -20}, {20, -20}, {20, -1}}, color = {0, 127, 255}));
            annotation(
              experiment(StopTime = 100));
          end TwoTanks;

          model TankWithEmptyingPipe1 "Demonstrates a tank with one constant top inlet mass flow rate and a bottom outlet into the ambient"
            import Modelica.SIunits.Conversions.from_bar;
            extends Modelica.Icons.Example;
            Sources.MassFlowSource_T flowSource(nPorts = 1, redeclare package Medium = Modelica.Media.Water.ConstantPropertyLiquidWater, m_flow = 50, T = system.T_ambient) annotation(
              Placement(transformation(extent = {{-20, 40}, {0, 60}})));
            inner Modelica.Fluid.System system(energyDynamics = Modelica.Fluid.Types.Dynamics.FixedInitial, massDynamics = Modelica.Fluid.Types.Dynamics.FixedInitial) annotation(
              Placement(transformation(extent = {{-100, 60}, {-80, 80}})));
            Modelica.Fluid.Sources.Boundary_pT ambient_fixed(nPorts = 1, redeclare package Medium = Modelica.Media.Water.ConstantPropertyLiquidWater, p = system.p_ambient, T = system.T_ambient) annotation(
              Placement(transformation(extent = {{-60, -100}, {-40, -80}})));
            Modelica.Fluid.Valves.ValveDiscrete valveDiscrete(redeclare package Medium = Modelica.Media.Water.ConstantPropertyLiquidWater, dp_nominal(displayUnit = "Pa") = 1, m_flow_nominal = 100) annotation(
              Placement(transformation(origin = {-20, -50}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
            Modelica.Blocks.Sources.BooleanConstant open(k = false) annotation(
              Placement(transformation(extent = {{-60, -60}, {-40, -40}})));
            Modelica.Fluid.Examples.AST_BatchPlant.BaseClasses.TankWithTopPorts tank1(redeclare package Medium = Modelica.Media.Water.ConstantPropertyLiquidWater, crossArea = 1, V0 = 0.1, height = 2, level_start = 0.1, nPorts = 2, portsData = {Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.05, height = 0), Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.1, height = 1)}, stiffCharacteristicForEmptyPort = true) annotation(
              Placement(transformation(extent = {{-40, -20}, {0, 20}})));
            Modelica.Fluid.Pipes.StaticPipe pipe(redeclare package Medium = Modelica.Media.Water.ConstantPropertyLiquidWater, length = 1, diameter = 0.1, height_ab = 1) annotation(
              Placement(transformation(origin = {40, 10}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
          equation
            connect(ambient_fixed.ports[1], valveDiscrete.port_a) annotation(
              Line(points = {{-40, -90}, {-20, -90}, {-20, -60}}, color = {0, 127, 255}));
            connect(open.y, valveDiscrete.open) annotation(
              Line(points = {{-39, -50}, {-28, -50}}, color = {255, 0, 255}));
            connect(flowSource.ports[1], pipe.port_b) annotation(
              Line(points = {{0, 50}, {40, 50}, {40, 20}}, color = {0, 127, 255}));
            connect(valveDiscrete.port_b, tank1.ports[1]) annotation(
              Line(points = {{-20, -40}, {-20, -30}, {-20, -21}, {-22, -21}}, color = {0, 127, 255}));
            connect(pipe.port_a, tank1.ports[2]) annotation(
              Line(points = {{40, 0}, {40, -28}, {-18, -28}, {-18, -20}, {-18, -21}}, color = {0, 127, 255}));
            annotation(
              experiment(StopTime = 35));
          end TankWithEmptyingPipe1;

          model TankWithEmptyingPipe2 "Demonstrates a tank with one constant top inlet mass flow rate and a bottom outlet into the ambient"
            import Modelica.SIunits.Conversions.from_bar;
            extends Modelica.Icons.Example;
            inner Modelica.Fluid.System system(energyDynamics = Modelica.Fluid.Types.Dynamics.FixedInitial, massDynamics = Modelica.Fluid.Types.Dynamics.FixedInitial) annotation(
              Placement(transformation(extent = {{-100, 60}, {-80, 80}})));
            Modelica.Fluid.Sources.Boundary_pT ambient_fixed(nPorts = 1, redeclare package Medium = Modelica.Media.Water.ConstantPropertyLiquidWater, p = system.p_ambient, T = system.T_ambient) annotation(
              Placement(transformation(extent = {{-60, -100}, {-40, -80}})));
            Modelica.Fluid.Examples.AST_BatchPlant.BaseClasses.TankWithTopPorts tank1(redeclare package Medium = Modelica.Media.Water.ConstantPropertyLiquidWater, crossArea = 1, V0 = 0.1, height = 2, nPorts = 2, portsData = {Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.05, height = 0), Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.1, height = 1)}, level_start = 2, stiffCharacteristicForEmptyPort = true) annotation(
              Placement(transformation(extent = {{-40, -20}, {0, 20}})));
            Modelica.Fluid.Pipes.StaticPipe pipe1(redeclare package Medium = Modelica.Media.Water.ConstantPropertyLiquidWater, length = 1, diameter = 0.1, height_ab = 1) annotation(
              Placement(transformation(origin = {-20, -60}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
            Modelica.Fluid.Pipes.StaticPipe pipe2(redeclare package Medium = Modelica.Media.Water.ConstantPropertyLiquidWater, length = 1, diameter = 0.1, height_ab = 1) annotation(
              Placement(transformation(origin = {30, -60}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
            Modelica.Fluid.Sources.Boundary_pT ambient_fixed1(nPorts = 1, redeclare package Medium = Modelica.Media.Water.ConstantPropertyLiquidWater, p = system.p_ambient, T = system.T_ambient) annotation(
              Placement(transformation(extent = {{0, -100}, {20, -80}})));
          equation
            connect(tank1.ports[1], pipe1.port_b) annotation(
              Line(points = {{-22, -21}, {-22, -35}, {-20, -35}, {-20, -50}}, color = {0, 127, 255}));
            connect(ambient_fixed.ports[1], pipe1.port_a) annotation(
              Line(points = {{-40, -90}, {-20, -90}, {-20, -70}}, color = {0, 127, 255}));
            connect(tank1.ports[2], pipe2.port_b) annotation(
              Line(points = {{-18, -21}, {-18, -21}, {-18, -40}, {30, -40}, {30, -50}}, color = {0, 127, 255}));
            connect(ambient_fixed1.ports[1], pipe2.port_a) annotation(
              Line(points = {{20, -90}, {30, -90}, {30, -70}}, color = {0, 127, 255}));
            annotation(
              experiment(StopTime = 120));
          end TankWithEmptyingPipe2;

          model TanksWithEmptyingPipe1 "Demonstrates a tank with one constant top inlet mass flow rate and a bottom outlet into the ambient"
            import Modelica.SIunits.Conversions.from_bar;
            extends Modelica.Icons.Example;
            inner Modelica.Fluid.System system(energyDynamics = Modelica.Fluid.Types.Dynamics.FixedInitial, massDynamics = Modelica.Fluid.Types.Dynamics.FixedInitial) annotation(
              Placement(transformation(extent = {{-100, 60}, {-80, 80}})));
            Modelica.Fluid.Sources.Boundary_pT ambient_fixed1(nPorts = 1, redeclare package Medium = Modelica.Media.Water.ConstantPropertyLiquidWater, p = system.p_ambient, T = system.T_ambient) annotation(
              Placement(transformation(extent = {{-100, -80}, {-80, -60}})));
            Modelica.Fluid.Examples.AST_BatchPlant.BaseClasses.TankWithTopPorts tank1(redeclare package Medium = Modelica.Media.Water.ConstantPropertyLiquidWater, crossArea = 1, V0 = 0.1, height = 2, nPorts = 2, portsData = {Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.05, height = 0), Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.1, height = 1)}, level_start = 2, stiffCharacteristicForEmptyPort = true) annotation(
              Placement(transformation(extent = {{-80, 0}, {-40, 40}})));
            Modelica.Fluid.Pipes.StaticPipe pipe1(redeclare package Medium = Modelica.Media.Water.ConstantPropertyLiquidWater, length = 1, diameter = 0.1, height_ab = 1) annotation(
              Placement(transformation(origin = {-60, -40}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
            Modelica.Fluid.Pipes.StaticPipe pipe2(redeclare package Medium = Modelica.Media.Water.ConstantPropertyLiquidWater, length = 1, diameter = 0.1, height_ab = 1) annotation(
              Placement(transformation(origin = {40, -40}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
            Modelica.Fluid.Sources.Boundary_pT ambient_fixed2(nPorts = 1, redeclare package Medium = Modelica.Media.Water.ConstantPropertyLiquidWater, p = system.p_ambient, T = system.T_ambient) annotation(
              Placement(transformation(extent = {{0, -80}, {20, -60}})));
            Modelica.Fluid.Examples.AST_BatchPlant.BaseClasses.TankWithTopPorts tank2(redeclare package Medium = Modelica.Media.Water.ConstantPropertyLiquidWater, crossArea = 1, V0 = 0.1, height = 2, nPorts = 2, portsData = {Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.05, height = 0), Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.1, height = 0.5)}, level_start = 0.1, stiffCharacteristicForEmptyPort = true) annotation(
              Placement(transformation(extent = {{20, 0}, {60, 40}})));
            Modelica.Fluid.Pipes.StaticPipe pipe3(redeclare package Medium = Modelica.Media.Water.ConstantPropertyLiquidWater, length = 1, diameter = 0.1, height_ab = -0.5) annotation(
              Placement(transformation(extent = {{-20, 10}, {0, 30}})));
          equation
            connect(tank1.ports[1], pipe1.port_b) annotation(
              Line(points = {{-62, -1}, {-62, -15}, {-60, -15}, {-60, -30}}, color = {0, 127, 255}));
            connect(ambient_fixed1.ports[1], pipe1.port_a) annotation(
              Line(points = {{-80, -70}, {-60, -70}, {-60, -50}}, color = {0, 127, 255}));
            connect(ambient_fixed2.ports[1], pipe2.port_a) annotation(
              Line(points = {{20, -70}, {40, -70}, {40, -50}}, color = {0, 127, 255}));
            connect(tank2.ports[1], pipe2.port_b) annotation(
              Line(points = {{38, -1}, {38, -15}, {40, -15}, {40, -30}}, color = {0, 127, 255}));
            connect(pipe3.port_a, tank1.ports[2]) annotation(
              Line(points = {{-20, 20}, {-30, 20}, {-30, -10}, {-58, -10}, {-58, 0}, {-58, 0}, {-58, -1}}, color = {0, 127, 255}));
            connect(pipe3.port_b, tank2.ports[2]) annotation(
              Line(points = {{0, 20}, {10, 20}, {10, -8}, {38, -8}, {38, 0}, {42, 0}, {42, -1}}, color = {0, 127, 255}));
            annotation(
              experiment(StopTime = 120));
          end TanksWithEmptyingPipe1;

          model TanksWithEmptyingPipe2 "Demonstrates a tank with one constant top inlet mass flow rate and a bottom outlet into the ambient"
            parameter Boolean stiffCharacteristicForEmptyPort = true;
            import Modelica.SIunits.Conversions.from_bar;
            extends Modelica.Icons.Example;
            replaceable package Medium = Modelica.Media.Water.ConstantPropertyLiquidWater constrainedby Modelica.Media.Interfaces.PartialMedium "Medium in the component" annotation(
               choicesAllMatching = true);
            inner Modelica.Fluid.System system(energyDynamics = Modelica.Fluid.Types.Dynamics.FixedInitial, massDynamics = Modelica.Fluid.Types.Dynamics.FixedInitial) annotation(
              Placement(transformation(extent = {{-100, 60}, {-80, 80}})));
            Modelica.Fluid.Sources.Boundary_pT ambient_fixed(nPorts = 1, redeclare package Medium = Modelica.Media.Water.ConstantPropertyLiquidWater, p = system.p_ambient, T = system.T_ambient) annotation(
              Placement(transformation(extent = {{-16, -102}, {-36, -82}})));
            Modelica.Fluid.Valves.ValveDiscrete valveDiscrete(redeclare package Medium = Modelica.Media.Water.ConstantPropertyLiquidWater, dp_nominal(displayUnit = "Pa") = 1, m_flow_nominal = 100) annotation(
              Placement(transformation(origin = {-60, -78}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
            Modelica.Blocks.Sources.BooleanConstant open(k = false) annotation(
              Placement(transformation(extent = {{-98, -88}, {-78, -68}})));
            Modelica.Fluid.Examples.AST_BatchPlant.BaseClasses.TankWithTopPorts tank3(redeclare package Medium = Medium, crossArea = 1, V0 = 0.1, height = 20, nPorts = 2, portsData = {Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.05, height = 0), Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.05, height = 6.5)}, level_start = 6, nTopPorts = 1, stiffCharacteristicForEmptyPort = stiffCharacteristicForEmptyPort) annotation(
              Placement(transformation(extent = {{-80, -50}, {-40, -10}})));
            Modelica.Fluid.Examples.AST_BatchPlant.BaseClasses.TankWithTopPorts tank1(redeclare package Medium = Medium, crossArea = 1, V0 = 0.1, height = 10, nPorts = 1, portsData = {Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.1, height = 0)}, level_start = 9, stiffCharacteristicForEmptyPort = stiffCharacteristicForEmptyPort) annotation(
              Placement(transformation(extent = {{50, 50}, {90, 90}})));
            Modelica.Fluid.Examples.AST_BatchPlant.BaseClasses.TankWithTopPorts tank2(redeclare package Medium = Medium, crossArea = 1, V0 = 0.1, height = 10, nPorts = 3, portsData = {Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.05, height = 0), Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.05, height = 2), Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.1, height = 3)}, level_start = 1, stiffCharacteristicForEmptyPort = stiffCharacteristicForEmptyPort) annotation(
              Placement(transformation(extent = {{-20, 10}, {20, 50}})));
            Pipes.StaticPipe pipe1(redeclare package Medium = Medium, height_ab = 2, length = 2, diameter = 0.1) annotation(
              Placement(transformation(origin = {70, 30}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
            Pipes.StaticPipe pipe2(redeclare package Medium = Medium, height_ab = 2, length = 2, diameter = 0.1) annotation(
              Placement(transformation(origin = {0, -22}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
            Pipes.StaticPipe pipe3(redeclare package Medium = Medium, height_ab = 2, length = 2, diameter = 0.1) annotation(
              Placement(transformation(origin = {-60, 10}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
          equation
            connect(ambient_fixed.ports[1], valveDiscrete.port_a) annotation(
              Line(points = {{-36, -92}, {-60, -92}, {-60, -88}}, color = {0, 127, 255}));
            connect(open.y, valveDiscrete.open) annotation(
              Line(points = {{-77, -78}, {-68, -78}}, color = {255, 0, 255}));
            connect(valveDiscrete.port_b, tank3.ports[1]) annotation(
              Line(points = {{-60, -68}, {-60, -59}, {-60, -51}, {-62, -51}}, color = {0, 127, 255}));
            connect(pipe1.port_b, tank1.ports[1]) annotation(
              Line(points = {{70, 40}, {70, 45}, {70, 49}}, color = {0, 127, 255}));
            connect(pipe2.port_a, tank3.ports[2]) annotation(
              Line(points = {{0, -32}, {0, -48}, {0, -60}, {-58, -60}, {-58, -51}}, color = {0, 127, 255}));
            connect(pipe3.port_a, tank3.topPorts[1]) annotation(
              Line(points = {{-60, 0}, {-60, -5}, {-60, -9}}, color = {0, 127, 255}));
            connect(pipe3.port_b, tank2.ports[1]) annotation(
              Line(points = {{-60, 20}, {-60, 26}, {-30, 26}, {-30, 0}, {-2, 0}, {-2, 9}, {-2.66667, 9}}, color = {0, 127, 255}));
            connect(pipe1.port_a, tank2.ports[3]) annotation(
              Line(points = {{70, 20}, {70, 0}, {2, 0}, {2, 9}, {2.66667, 9}}, color = {0, 127, 255}));
            connect(pipe2.port_b, tank2.ports[2]) annotation(
              Line(points = {{0, -12}, {0, 9}}, color = {0, 127, 255}));
            annotation(
              experiment(StopTime = 120));
          end TanksWithEmptyingPipe2;
        end Test;
        annotation(
          preferredView = "info",
          Documentation(info = "<html>
<p>
The process under consideration is an evaporation plant for a
student lab at the Process Control Laboratory (AST) of the
University of Dortmund that evaporates a water sodium chloride
mixture so that a higher concentrated solution is produced.
The task of the students is to learn how to program the process
control system. A picture of the batch plant is shown in the figure
below.
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Fluid/Examples/AST_BatchPlant1.jpg\" border=\"1\"
     alt=\"AST_BatchPlant1.jpg\">
</p>

<p>
The flow sheet diagram is shown in the next figure.
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Fluid/Examples/AST_BatchPlant2.png\" border=\"1\"
     alt=\"AST_BatchPlant2.png\">
</p>

<p>
Pure water from tank B1 and concentrated sodium chloride
solution from tank B2 are mixed in a mixing tank B3.
After buffering in tank B4 the mixture flows to the
evaporator B5. Here the water sodium chloride mixture
is evaporated until the desired concentration is reached.
The steam is condensed in the condenser K1 and cooled
afterwards in the cooling tank B6. The concentrated
 solution is also led to a cooling tank B7. The cooled
fluids are pumped back to the charging vessels by the
pumps P1 and P2. Be-tween the tanks several valves are
present that are regulated by a central control system.
</p>
</html>"));
      end AST_BatchPlant;

      model IncompressibleFluidNetwork "Multi-way connections of pipes and incompressible medium model"
        extends Modelica.Icons.Example;
        parameter Types.ModelStructure pipeModelStructure = Modelica.Fluid.Types.ModelStructure.av_vb;
        //parameter Types.ModelStructure pipeModelStructure = Modelica.Fluid.Types.ModelStructure.a_v_b;
        replaceable package Medium = Modelica.Media.Incompressible.Examples.Glycol47 constrainedby Modelica.Media.Interfaces.PartialMedium;
        //replaceable package Medium =
        //    Modelica.Media.Water.StandardWaterOnePhase
        //  constrainedby Modelica.Media.Interfaces.PartialMedium;
        import Modelica.Fluid.Types.Dynamics;
        parameter Dynamics systemMassDynamics = if Medium.singleState then Dynamics.SteadyState else Dynamics.SteadyStateInitial;
        parameter Boolean filteredValveOpening = not Medium.singleState;
        Sources.Boundary_pT source(nPorts = 1, redeclare package Medium = Medium, p = 5.0e5, T = 300) annotation(
          Placement(transformation(extent = {{-98, -6}, {-86, 6}})));
        Pipes.DynamicPipe pipe1(use_T_start = true, length = 10, diameter = 2.5e-2, redeclare package Medium = Medium, modelStructure = pipeModelStructure, redeclare model FlowModel = Modelica.Fluid.Pipes.BaseClasses.FlowModels.TurbulentPipeFlow) annotation(
          Placement(transformation(extent = {{-80, -10}, {-60, 10}})));
        Pipes.DynamicPipe pipe2(use_T_start = true, diameter = 2.5e-2, redeclare package Medium = Medium, length = 0.5, modelStructure = pipeModelStructure, redeclare model FlowModel = Modelica.Fluid.Pipes.BaseClasses.FlowModels.TurbulentPipeFlow) annotation(
          Placement(transformation(origin = {-50, 20}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
        Pipes.DynamicPipe pipe3(use_T_start = true, diameter = 2.5e-2, redeclare package Medium = Medium, length = 0.5, modelStructure = pipeModelStructure, redeclare model FlowModel = Modelica.Fluid.Pipes.BaseClasses.FlowModels.TurbulentPipeFlow) annotation(
          Placement(transformation(origin = {-50, -20}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
        Pipes.DynamicPipe pipe4(use_T_start = true, diameter = 2.5e-2, redeclare package Medium = Medium, length = 2, modelStructure = pipeModelStructure, use_HeatTransfer = true, redeclare model FlowModel = Modelica.Fluid.Pipes.BaseClasses.FlowModels.TurbulentPipeFlow) annotation(
          Placement(transformation(extent = {{-20, -50}, {0, -30}})));
        Pipes.DynamicPipe pipe6(use_T_start = true, diameter = 2.5e-2, redeclare package Medium = Medium, length = 20, modelStructure = pipeModelStructure, redeclare model FlowModel = Modelica.Fluid.Pipes.BaseClasses.FlowModels.TurbulentPipeFlow) annotation(
          Placement(transformation(extent = {{20, -50}, {40, -30}})));
        Valves.ValveIncompressible valve1(redeclare package Medium = Medium, m_flow_nominal = 1, rho_nominal = 1000, CvData = Modelica.Fluid.Types.CvTypes.Av, Av = 2.5e-2 ^ 2 / 4 * Modelica.Constants.pi, dp_nominal = 30000) annotation(
          Placement(transformation(extent = {{-46, 30}, {-26, 50}})));
        Valves.ValveIncompressible valve2(redeclare package Medium = Medium, m_flow_nominal = 1, rho_nominal = 1000, CvData = Modelica.Fluid.Types.CvTypes.Av, Av = 2.5e-2 ^ 2 / 4 * Modelica.Constants.pi, dp_nominal = 30000) annotation(
          Placement(transformation(extent = {{-46, -30}, {-26, -50}})));
        Pipes.DynamicPipe pipe7(use_T_start = true, length = 10, diameter = 2.5e-2, redeclare package Medium = Medium, modelStructure = pipeModelStructure, use_HeatTransfer = true, redeclare model FlowModel = Modelica.Fluid.Pipes.BaseClasses.FlowModels.TurbulentPipeFlow) annotation(
          Placement(transformation(extent = {{-20, 30}, {0, 50}})));
        Valves.ValveIncompressible valve3(redeclare package Medium = Medium, m_flow_nominal = 1, rho_nominal = 1000, CvData = Modelica.Fluid.Types.CvTypes.Av, Av = 2.5e-2 ^ 2 / 4 * Modelica.Constants.pi, dp_nominal = 30000) annotation(
          Placement(transformation(extent = {{80, 0}, {100, 20}})));
        Sources.Boundary_pT sink(nPorts = 1, redeclare package Medium = Medium, T = 300, p = 1.0e5) annotation(
          Placement(transformation(extent = {{118, 4}, {106, 16}})));
        inner Modelica.Fluid.System system(massDynamics = systemMassDynamics, energyDynamics = Modelica.Fluid.Types.Dynamics.FixedInitial, use_eps_Re = true) annotation(
          Placement(transformation(extent = {{90, -92}, {110, -72}})));
        Modelica.Blocks.Sources.Step valveOpening1(offset = 1, startTime = 50, height = -1) annotation(
          Placement(transformation(extent = {{-80, 80}, {-60, 60}})));
        Modelica.Blocks.Sources.Step valveOpening2(offset = 1, height = -0.5, startTime = 100) annotation(
          Placement(transformation(extent = {{-80, -60}, {-60, -80}})));
        Modelica.Blocks.Sources.Step valveOpening3(offset = 1, startTime = 150, height = -0.5) annotation(
          Placement(transformation(extent = {{60, 80}, {80, 60}})));
        Pipes.DynamicPipe pipe8(use_T_start = true, length = 10, diameter = 2.5e-2, redeclare package Medium = Medium, modelStructure = pipeModelStructure, use_HeatTransfer = true, redeclare model FlowModel = Modelica.Fluid.Pipes.BaseClasses.FlowModels.TurbulentPipeFlow) annotation(
          Placement(transformation(origin = {10, 10}, extent = {{-10, 10}, {10, -10}}, rotation = 270)));
        Pipes.DynamicPipe pipe9(use_T_start = true, length = 10, diameter = 2.5e-2, redeclare package Medium = Medium, modelStructure = pipeModelStructure, use_HeatTransfer = true, redeclare model FlowModel = Modelica.Fluid.Pipes.BaseClasses.FlowModels.TurbulentPipeFlow) annotation(
          Placement(transformation(extent = {{20, 30}, {40, 50}})));
        Pipes.DynamicPipe pipe10(use_T_start = true, length = 10, diameter = 2.5e-2, redeclare package Medium = Medium, modelStructure = pipeModelStructure, use_HeatTransfer = true, redeclare model FlowModel = Modelica.Fluid.Pipes.BaseClasses.FlowModels.TurbulentPipeFlow) annotation(
          Placement(transformation(extent = {{20, -30}, {40, -10}})));
        Pipes.DynamicPipe pipe5(use_T_start = true, diameter = 2.5e-2, redeclare package Medium = Medium, length = 20, modelStructure = pipeModelStructure, redeclare model FlowModel = Modelica.Fluid.Pipes.BaseClasses.FlowModels.TurbulentPipeFlow) annotation(
          Placement(transformation(extent = {{20, -70}, {40, -50}})));
        Thermal.HeatTransfer.Sources.FixedHeatFlow[pipe8.nNodes] heat8(Q_flow = 16e3 * pipe8.dxs) annotation(
          Placement(transformation(extent = {{-20, 0}, {0, 20}})));
        Pipes.DynamicPipe pipe11(use_T_start = true, diameter = 2.5e-2, redeclare package Medium = Medium, modelStructure = pipeModelStructure, redeclare model FlowModel = Modelica.Fluid.Pipes.BaseClasses.FlowModels.TurbulentPipeFlow, length = 0.5) annotation(
          Placement(transformation(extent = {{54, 0}, {74, 20}})));
      equation
        connect(source.ports[1], pipe1.port_a) annotation(
          Line(points = {{-86, 0}, {-80, 0}}, color = {0, 127, 255}));
        connect(pipe1.port_b, pipe3.port_a) annotation(
          Line(points = {{-60, 0}, {-50, 0}, {-50, -10}}, color = {0, 127, 255}));
        connect(pipe1.port_b, pipe2.port_a) annotation(
          Line(points = {{-60, 0}, {-50, 0}, {-50, 10}}, color = {0, 127, 255}));
        connect(pipe2.port_b, valve1.port_a) annotation(
          Line(points = {{-50, 30}, {-50, 40}, {-46, 40}}, color = {0, 127, 255}));
        connect(valve2.port_b, pipe4.port_a) annotation(
          Line(points = {{-26, -40}, {-26, -40}, {-20, -40}}, color = {0, 127, 255}));
        connect(pipe3.port_b, valve2.port_a) annotation(
          Line(points = {{-50, -30}, {-50, -40}, {-46, -40}}, color = {0, 127, 255}));
        connect(valve1.port_b, pipe7.port_a) annotation(
          Line(points = {{-26, 40}, {-26, 40}, {-20, 40}}, color = {0, 127, 255}));
        connect(valve3.port_b, sink.ports[1]) annotation(
          Line(points = {{100, 10}, {100, 10}, {106, 10}}, color = {0, 127, 255}));
        connect(valveOpening1.y, valve1.opening) annotation(
          Line(points = {{-59, 70}, {-36, 70}, {-36, 48}}, color = {0, 0, 127}));
        connect(valveOpening2.y, valve2.opening) annotation(
          Line(points = {{-59, -70}, {-36, -70}, {-36, -48}}, color = {0, 0, 127}));
        connect(valveOpening3.y, valve3.opening) annotation(
          Line(points = {{81, 70}, {90, 70}, {90, 18}}, color = {0, 0, 127}));
        connect(pipe7.port_b, pipe9.port_a) annotation(
          Line(points = {{0, 40}, {0, 40}, {20, 40}}, color = {0, 127, 255}));
        connect(pipe7.port_b, pipe8.port_a) annotation(
          Line(points = {{0, 40}, {10, 40}, {10, 20}}, color = {0, 127, 255}));
        connect(pipe8.port_b, pipe10.port_a) annotation(
          Line(points = {{10, 0}, {10, -20}, {20, -20}}, color = {0, 127, 255}));
        connect(pipe4.port_b, pipe6.port_a) annotation(
          Line(points = {{0, -40}, {20, -40}}, color = {0, 127, 255}));
        connect(pipe8.port_b, pipe4.port_b) annotation(
          Line(points = {{10, 0}, {10, -40}, {0, -40}}, color = {0, 127, 255}));
        connect(pipe5.port_a, pipe4.port_b) annotation(
          Line(points = {{20, -60}, {10, -60}, {10, -40}, {0, -40}}, color = {0, 127, 255}));
        connect(heat8.port, pipe8.heatPorts) annotation(
          Line(points = {{0, 10}, {6, 10}, {6, 9.9}, {5.6, 9.9}}, color = {191, 0, 0}));
        connect(pipe5.port_b, pipe6.port_b) annotation(
          Line(points = {{40, -60}, {50, -60}, {50, -40}, {40, -40}}, color = {0, 127, 255}));
        connect(pipe6.port_b, pipe10.port_b) annotation(
          Line(points = {{40, -40}, {50, -40}, {50, -20}, {40, -20}}, color = {0, 127, 255}));
        connect(pipe11.port_b, valve3.port_a) annotation(
          Line(points = {{74, 10}, {80, 10}}, color = {0, 127, 255}));
        connect(pipe9.port_b, pipe11.port_a) annotation(
          Line(points = {{40, 40}, {50, 40}, {50, 10}, {54, 10}}, color = {0, 127, 255}));
        connect(pipe10.port_b, pipe11.port_a) annotation(
          Line(points = {{40, -20}, {50, -20}, {50, 10}, {54, 10}}, color = {0, 127, 255}));
        annotation(
          Documentation(info = "<html>
<p>
This example demonstrates two aspects: the treatment of multi-way connections
and the usage of an incompressible medium model.
</p><p>
Eleven pipe models with nNodes=2 each introduce 22 temperature states and and 22 pressure states.
When configuring <b>pipeModelStructure=a_v_b</b>, the flow models at the pipe ports constitute algebraic loops for the pressures.
A common work-around is to introduce \"mixing volumes\" in critical connections.
</p><p>
Here the problem is treated alternatively with the default <b>pipeModelStructure=av_vb</b> of the
<a href=\"modelica://Modelica.Fluid.Pipes.DynamicPipe\">DynamicPipe</a> model.
Each pipe exposes the states of the outer fluid segments to the respective fluid ports.
Consequently the pressures of all connected pipe segments get lumped together into one mass balance spanning the whole connection set.
Overall this treatment as high-index DAE results in the reduction to 9 pressure states, preventing algebraic loops in connections.
This can be studied with a rigorous medium model like <b>StandardWaterOnePhase</b>.
</p><p>
The pressure dynamics completely disappears with an incompressible medium model, like the used <b>Glycol47</b>.
It appears reasonable to assume steady-state mass balances in this case
(see parameter systemMassDynamics used in system.massDynamics, tab Assumptions).
</p><p>
Note that with the stream concept in the fluid ports, the energy and substance balances of the connected pipe segments remain independent
from each other, despite of pressures being lumped together. The following simulation results can be observed:
</p>
<ol>
<li>The simulation starts with system.T_ambient as initial temperature in all pipes.
    The temperatures upstream or bypassing pipe8 are approaching the value of 26.85 degC from the source, including also pipe9.
    The temperatures downstream of pipe8 take a higher value, depending on the mixing with heated fluid, see e.g. pipe10.</li>
<li>After 50s valve1 fully closes. This causes flow reversal in pipe8. Now heated fluid flows from pipe8 to pipe9.
    Note that the temperature of the connected pipe7 remains unchanged as there is no flow into pipe7.
    The temperature of pipe10 cools down to the source temperature.</li>
<li>After 100s valve2 closes half way, which affects mass flow rates and temperatures.</li>
<li>After 150s valve5 closes half way, which affects mass flow rates and temperatures.</li>
</ol>
<p>
The fluid temperatures in the pipes of interest are exposed through heatPorts.
</p>
<img src=\"modelica://Modelica/Resources/Images/Fluid/Examples/IncompressibleFluidNetwork.png\" border=\"1\"
     alt=\"IncompressibleFluidNetwork.png\">
</html>"),
          experiment(StopTime = 200),
          __Dymola_Commands(file = "modelica://Modelica/Resources/Scripts/Dymola/Fluid/IncompressibleFluidNetwork/plotResults.mos" "plotResults"),
          Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {120, 100}})),
          Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {120, 100}})));
      end IncompressibleFluidNetwork;

      model BranchingDynamicPipes "Multi-way connections of pipes with dynamic momentum balance, pressure wave and flow reversal"
        extends Modelica.Icons.Example;
        replaceable package Medium = Modelica.Media.Air.MoistAir constrainedby Modelica.Media.Interfaces.PartialMedium;
        //replaceable package Medium=Modelica.Media.Water.StandardWater constrainedby Modelica.Media.Interfaces.PartialMedium;
        inner Modelica.Fluid.System system(energyDynamics = Modelica.Fluid.Types.Dynamics.SteadyStateInitial, momentumDynamics = Modelica.Fluid.Types.Dynamics.SteadyStateInitial) annotation(
          Placement(transformation(extent = {{-90, 70}, {-70, 90}})));
        Modelica.Fluid.Sources.Boundary_pT boundary1(nPorts = 1, redeclare package Medium = Medium, p = 150000) annotation(
          Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 90, origin = {0, -80})));
        Pipes.DynamicPipe pipe1(redeclare package Medium = Medium, use_T_start = true, nNodes = 5, diameter = 2.54e-2, m_flow_start = 0.02, height_ab = 50, length = 50, p_a_start = 150000, p_b_start = 130000, modelStructure = Modelica.Fluid.Types.ModelStructure.a_v_b) annotation(
          Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 90, origin = {0, -50})));
        Pipes.DynamicPipe pipe2(redeclare package Medium = Medium, use_T_start = true, nNodes = 5, redeclare model HeatTransfer = Modelica.Fluid.Pipes.BaseClasses.HeatTransfer.LocalPipeFlowHeatTransfer, use_HeatTransfer = true, diameter = 2.54e-2, m_flow_start = 0.01, length = 50, height_ab = 25, p_a_start = 130000, p_b_start = 120000, modelStructure = Modelica.Fluid.Types.ModelStructure.av_vb) annotation(
          Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 90, origin = {-20, -10})));
        Pipes.DynamicPipe pipe3(redeclare package Medium = Medium, use_T_start = true, nNodes = 5, diameter = 2.54e-2, m_flow_start = 0.01, length = 25, height_ab = 25, p_a_start = 130000, p_b_start = 120000, modelStructure = Modelica.Fluid.Types.ModelStructure.a_v_b) annotation(
          Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 90, origin = {20, -10})));
        Pipes.DynamicPipe pipe4(redeclare package Medium = Medium, use_T_start = true, nNodes = 5, diameter = 2.54e-2, m_flow_start = 0.02, height_ab = 50, length = 50, p_a_start = 120000, p_b_start = 100000, modelStructure = Modelica.Fluid.Types.ModelStructure.a_v_b) annotation(
          Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 90, origin = {0, 30})));
        Modelica.Fluid.Sources.Boundary_pT boundary4(nPorts = 1, redeclare package Medium = Medium, use_p_in = true, use_T_in = false, p = 100000) annotation(
          Placement(transformation(extent = {{10, -10}, {-10, 10}}, rotation = 90, origin = {0, 60})));
        Modelica.Blocks.Sources.Ramp ramp1(offset = 1e5, startTime = 2, height = 1e5, duration = 0) annotation(
          Placement(transformation(extent = {{-40, 70}, {-20, 90}})));
        Modelica.Thermal.HeatTransfer.Sources.FixedHeatFlow[pipe2.nNodes] heat2(Q_flow = 200 * pipe2.dxs, alpha = -1e-2 * ones(pipe2.n)) annotation(
          Placement(transformation(extent = {{-60, -20}, {-40, 0}})));
      equation
        connect(ramp1.y, boundary4.p_in) annotation(
          Line(points = {{-19, 80}, {-8, 80}, {-8, 72}}, color = {0, 0, 127}, thickness = 0.5));
        connect(boundary1.ports[1], pipe1.port_a) annotation(
          Line(points = {{0, -70}, {0, -70}, {0, -60}, {0, -60}}, color = {0, 127, 255}, thickness = 0.5));
        connect(pipe1.port_b, pipe2.port_a) annotation(
          Line(points = {{0, -40}, {0, -40}, {0, -30}, {-20, -30}, {-20, -20}}, color = {0, 127, 255}, thickness = 0.5));
        connect(pipe1.port_b, pipe3.port_a) annotation(
          Line(points = {{0, -40}, {0, -40}, {0, -30}, {20, -30}, {20, -20}}, color = {0, 127, 255}, thickness = 0.5));
        connect(pipe2.port_b, pipe4.port_a) annotation(
          Line(points = {{-20, 0}, {-20, 0}, {-20, 10}, {0, 10}, {0, 16}, {0, 20}, {0, 20}}, color = {0, 127, 255}, thickness = 0.5));
        connect(pipe3.port_b, pipe4.port_a) annotation(
          Line(points = {{20, 0}, {20, 0}, {20, 10}, {0, 10}, {0, 16}, {0, 20}, {0, 20}}, color = {0, 127, 255}, thickness = 0.5));
        connect(pipe4.port_b, boundary4.ports[1]) annotation(
          Line(points = {{0, 40}, {0, 50}, {0, 50}}, color = {0, 127, 255}, thickness = 0.5));
        connect(heat2.port, pipe2.heatPorts) annotation(
          Line(points = {{-40, -10}, {-24.4, -10}, {-24.4, -9.9}}, color = {191, 0, 0}, thickness = 0.5));
        annotation(
          Documentation(info = "<html>
<p>
This model demonstrates the use of distributed pipe models with dynamic energy, mass and momentum balances.
At time=2s the pressure of boundary4 jumps, which causes a pressure wave and flow reversal.
</p>
<p>
Change system.momentumDynamics on the Assumptions tab of the system object from SteadyStateInitial to SteadyState,
in order to assume a steady-state momentum balance. This is the default for all models of the library.
</p>
<p>
Change the Medium from MoistAir to StandardWater, in order to investigate a medium with significantly different density.
Note the static head caused by the elevation of the pipes.
</p>
<p>
Note the appropriate use of the modelStructure of the DynamicPipe models (Advanced tab).
The default modelStructure is av_vb, i.e. volumes with a pressure state are exposed at both ports.
In many cases this gives good numerical performance, avoiding algebraic loops in connections,
e.g. if a pipe is connected to a valve or to a vessel with portsData configured.
The price to pay is a high-index DAE if two pipes are connected or if a pipe is connected to a boundary with prescribed pressure.
In such cases one might consider changing the modelStructure.
</p>
<p>
In the BranchingDynamicPipes example, {pipe1,pipe3,pipe4}.modelStructure are configured to a_v_b, while pipe2.modelStructure remains av_vb.
This avoids a high-index DAE and overdetermined initial conditions.
</p>
<img src=\"modelica://Modelica/Resources/Images/Fluid/Examples/BranchingDynamicPipes.png\" border=\"1\"
     alt=\"BranchingDynamicPipes.png\">
</html>"),
          experiment(StopTime = 10),
          __Dymola_Commands(file(ensureSimulated = true) = "modelica://Modelica/Resources/Scripts/Dymola/Fluid/BranchingDynamicPipes/plotResults.mos" "plotResults"));
      end BranchingDynamicPipes;

      model NonCircularPipes "Comparing a circular with a non-circular pipe"
        extends Modelica.Icons.Example;
        replaceable package Medium = Modelica.Media.Water.StandardWater;
        constant Real odim = 0.015;
        constant Real idim = 0.005;
        inner Modelica.Fluid.System system(energyDynamics = Modelica.Fluid.Types.Dynamics.FixedInitial) annotation(
          Placement(transformation(origin = {-80, 80}, extent = {{-10, -10}, {10, 10}})));
        Modelica.Fluid.Sources.Boundary_pT boundary(redeclare package Medium = Medium, nPorts = 2, p = 10.0e5, T = 293.15) annotation(
          Placement(transformation(origin = {-80, 0}, extent = {{-10, -10}, {10, 10}})));
        Modelica.Fluid.Sources.MassFlowSource_T massflowsink1(redeclare package Medium = Medium, nPorts = 1, m_flow = -0.1, T = 293.15) annotation(
          Placement(transformation(origin = {40, 40}, extent = {{10, -10}, {-10, 10}})));
        Modelica.Fluid.Sources.MassFlowSource_T massflowsink2(redeclare package Medium = Medium, nPorts = 1, m_flow = -0.1, T = 293.15) annotation(
          Placement(transformation(origin = {40, -40}, extent = {{10, -10}, {-10, 10}})));
        Modelica.Fluid.Pipes.DynamicPipe circular_pipe(redeclare package Medium = Medium, length = 100, diameter = 0.01, T_start = 293.15, p_a_start = 10.0e5, p_b_start = 10.0e5) annotation(
          Placement(transformation(origin = {-20, 40}, extent = {{-10, -10}, {10, 10}})));
        Modelica.Fluid.Pipes.DynamicPipe annulus_pipe(redeclare package Medium = Medium, length = 100, crossArea = Modelica.Constants.pi * (odim ^ 2 - idim ^ 2) / 4, perimeter = Modelica.Constants.pi * (odim + idim), isCircular = false, diameter = 0, p_a_start = 1000000, p_b_start = 1000000, T_start = 293.15) annotation(
          Placement(transformation(origin = {-20, -40}, extent = {{-10, -10}, {10, 10}})));
      equation
        connect(boundary.ports[1], circular_pipe.port_a) annotation(
          Line(points = {{-70, 2}, {-60, 2}, {-60, 40}, {-30, 40}}, color = {0, 127, 255}));
        connect(boundary.ports[2], annulus_pipe.port_a) annotation(
          Line(points = {{-70, -2}, {-60, -2}, {-60, -40}, {-30, -40}}, color = {0, 127, 255}));
        connect(circular_pipe.port_b, massflowsink1.ports[1]) annotation(
          Line(points = {{-10, 40}, {30, 40}}, color = {0, 127, 255}));
        connect(annulus_pipe.port_b, massflowsink2.ports[1]) annotation(
          Line(points = {{-10, -40}, {30, -40}}, color = {0, 127, 255}));
        annotation(
          experiment(StopTime = 1),
          Documentation(info = "<html>
<p>
In this example two pipes are used to demonstrate the use of circular (default) and non-circular pipes,
where the topmost pipe is circular with a length of 100 m and an inner diameter of 10 mm and the second pipe
is a circular ring pipe with inner diameter of 5 mm and an outer diameter of 15 mm.
</p>
<p>
Both pipes are connected to a pT source (water, 293.15 K, 10 bar) and a mass flow sink (0.1 kg/s inflow).
</p>
<p>
Although the hydraulic diameter of both pipes are the same, the different cross sections lead to different
velocities and by this different outlet pressures (7.324 bar for the circular pipe versus 9.231 bar for the
circular ring pipe).
</p>
<p><img src=\"modelica://Modelica/Resources/Images/Fluid/Examples/NonCircularPipes.png\" border=\"1\"
     alt=\"NonCircularPipes.png\"></p>
</html>", revisions = "<html>
<ul>
<li>
January 6, 2015 by Alexander T&auml;schner:<br/>
First implementation.
</li>
</ul>
</html>"));
      end NonCircularPipes;

      package HeatExchanger "Demo of a heat exchanger model"
        extends Modelica.Icons.ExamplesPackage;

        model HeatExchangerSimulation "simulation for the heat exchanger model"
          extends Modelica.Icons.Example;
          //replaceable package Medium = Modelica.Media.Water.ConstantPropertyLiquidWater;
          replaceable package Medium = Modelica.Media.Water.StandardWaterOnePhase;
          //package Medium = Modelica.Media.Incompressible.Examples.Essotherm650;
          Modelica.Fluid.Examples.HeatExchanger.BaseClasses.BasicHX HEX(c_wall = 500, use_T_start = true, nNodes = 20, m_flow_start_1 = 0.2, m_flow_start_2 = 0.2, k_wall = 100, s_wall = 0.005, crossArea_1 = 4.5e-4, crossArea_2 = 4.5e-4, perimeter_1 = 0.075, perimeter_2 = 0.075, rho_wall = 900, redeclare package Medium_1 = Medium, redeclare package Medium_2 = Medium, modelStructure_1 = Modelica.Fluid.Types.ModelStructure.av_b, modelStructure_2 = Modelica.Fluid.Types.ModelStructure.a_vb, redeclare model HeatTransfer_1 = Modelica.Fluid.Pipes.BaseClasses.HeatTransfer.LocalPipeFlowHeatTransfer(alpha0 = 1000), length = 20, area_h_1 = 0.075 * 20, area_h_2 = 0.075 * 20, redeclare model HeatTransfer_2 = Modelica.Fluid.Pipes.BaseClasses.HeatTransfer.ConstantFlowHeatTransfer(alpha0 = 2000), Twall_start = 300, dT = 10, T_start_1 = 304, T_start_2 = 300) annotation(
            Placement(transformation(extent = {{-26, -14}, {34, 46}})));
          Modelica.Fluid.Sources.Boundary_pT ambient2(nPorts = 1, p = 1e5, T = 280, redeclare package Medium = Medium) annotation(
            Placement(transformation(extent = {{82, -28}, {62, -8}})));
          Modelica.Fluid.Sources.Boundary_pT ambient1(nPorts = 1, p = 1e5, T = 300, redeclare package Medium = Medium) annotation(
            Placement(transformation(extent = {{82, 24}, {62, 44}})));
          Modelica.Fluid.Sources.MassFlowSource_T massFlowRate2(nPorts = 1, m_flow = 0.2, T = 360, redeclare package Medium = Medium, use_m_flow_in = true, use_T_in = false, use_X_in = false) annotation(
            Placement(transformation(extent = {{-66, 24}, {-46, 44}})));
          Modelica.Fluid.Sources.MassFlowSource_T massFlowRate1(nPorts = 1, redeclare package Medium = Medium, m_flow = 0.2, T = 300) annotation(
            Placement(transformation(extent = {{-66, -10}, {-46, 10}})));
          Modelica.Blocks.Sources.Ramp Ramp1(startTime = 50, duration = 5, height = 0.4, offset = -0.2) annotation(
            Placement(transformation(extent = {{-98, 24}, {-78, 44}})));
          inner Modelica.Fluid.System system(energyDynamics = Modelica.Fluid.Types.Dynamics.SteadyStateInitial, use_eps_Re = true) annotation(
            Placement(transformation(extent = {{60, 70}, {80, 90}})));
        equation
          connect(massFlowRate1.ports[1], HEX.port_a1) annotation(
            Line(points = {{-46, 0}, {-40, 0}, {-40, 15.4}, {-29, 15.4}}, color = {0, 127, 255}));
          connect(HEX.port_b1, ambient1.ports[1]) annotation(
            Line(points = {{37, 15.4}, {48.5, 15.4}, {48.5, 34}, {62, 34}}, color = {0, 127, 255}));
          connect(Ramp1.y, massFlowRate2.m_flow_in) annotation(
            Line(points = {{-77, 34}, {-74, 34}, {-74, 42}, {-66, 42}}, color = {0, 0, 127}));
          connect(massFlowRate2.ports[1], HEX.port_b2) annotation(
            Line(points = {{-46, 34}, {-40, 34}, {-40, 29.8}, {-29, 29.8}}, color = {0, 127, 255}));
          connect(HEX.port_a2, ambient2.ports[1]) annotation(
            Line(points = {{37, 2.2}, {42, 2}, {50, 2}, {50, -18}, {62, -18}}, color = {0, 127, 255}));
          annotation(
            experiment(StopTime = 200, Tolerance = 1e-005),
            Documentation(info = "<html>
<p>The simulation start in steady state with counterflow operation. At time t = 50, the mass flow rate on the secondary circuit is changed to a negative value in 5 seconds. After a transient, the heat exchanger operates in co-current flow.</p>
<p><img src=\"modelica://Modelica/Resources/Images/Fluid/Examples/HeatExchanger.png\" alt=\"HeatExchanger.png\"/> </p>
</html>"));
        end HeatExchangerSimulation;

        package BaseClasses "Additional models for heat exchangers"
          extends Modelica.Icons.BasesPackage;

          model BasicHX "Simple heat exchanger model"
            outer Modelica.Fluid.System system "System properties";
            //General
            parameter SI.Length length(min = 0) "Length of flow path for both fluids";
            parameter Integer nNodes(min = 1) = 2 "Spatial segmentation";
            parameter Modelica.Fluid.Types.ModelStructure modelStructure_1 = Types.ModelStructure.av_vb "Determines whether flow or volume models are present at the ports" annotation(
              Evaluate = true,
              Dialog(tab = "General", group = "Fluid 1"));
            parameter Modelica.Fluid.Types.ModelStructure modelStructure_2 = Types.ModelStructure.av_vb "Determines whether flow or volume models are present at the ports" annotation(
              Evaluate = true,
              Dialog(tab = "General", group = "Fluid 2"));
            replaceable package Medium_1 = Modelica.Media.Water.StandardWater constrainedby Modelica.Media.Interfaces.PartialMedium "Fluid 1" annotation(
               choicesAllMatching,
               Dialog(tab = "General", group = "Fluid 1"));
            replaceable package Medium_2 = Modelica.Media.Water.StandardWater constrainedby Modelica.Media.Interfaces.PartialMedium "Fluid 2" annotation(
               choicesAllMatching,
               Dialog(tab = "General", group = "Fluid 2"));
            parameter SI.Area crossArea_1 "Cross sectional area" annotation(
              Dialog(tab = "General", group = "Fluid 1"));
            parameter SI.Area crossArea_2 "Cross sectional area" annotation(
              Dialog(tab = "General", group = "Fluid 2"));
            parameter SI.Length perimeter_1 "Flow channel perimeter" annotation(
              Dialog(tab = "General", group = "Fluid 1"));
            parameter SI.Length perimeter_2 "Flow channel perimeter" annotation(
              Dialog(tab = "General", group = "Fluid 2"));
            final parameter Boolean use_HeatTransfer = true "= true to use the HeatTransfer_1/_2 model";
            // Heat transfer
            replaceable model HeatTransfer_1 = Modelica.Fluid.Pipes.BaseClasses.HeatTransfer.IdealFlowHeatTransfer constrainedby Modelica.Fluid.Pipes.BaseClasses.HeatTransfer.PartialFlowHeatTransfer "Heat transfer model" annotation(
               choicesAllMatching,
               Dialog(tab = "General", group = "Fluid 1", enable = use_HeatTransfer));
            replaceable model HeatTransfer_2 = Modelica.Fluid.Pipes.BaseClasses.HeatTransfer.IdealFlowHeatTransfer constrainedby Modelica.Fluid.Pipes.BaseClasses.HeatTransfer.PartialFlowHeatTransfer "Heat transfer model" annotation(
               choicesAllMatching,
               Dialog(tab = "General", group = "Fluid 2", enable = use_HeatTransfer));
            parameter SI.Area area_h_1 "Heat transfer area" annotation(
              Dialog(tab = "General", group = "Fluid 1"));
            parameter SI.Area area_h_2 "Heat transfer area" annotation(
              Dialog(tab = "General", group = "Fluid 2"));
            //Wall
            parameter SI.Length s_wall(min = 0) "Wall thickness" annotation(
              Dialog(group = "Wall properties"));
            parameter SI.ThermalConductivity k_wall "Thermal conductivity of wall material" annotation(
              Dialog(group = "Wall properties"));
            parameter SI.SpecificHeatCapacity c_wall "Specific heat capacity of wall material" annotation(
              Dialog(tab = "General", group = "Wall properties"));
            parameter SI.Density rho_wall "Density of wall material" annotation(
              Dialog(tab = "General", group = "Wall properties"));
            final parameter SI.Area area_h = (area_h_1 + area_h_2) / 2 "Heat transfer area";
            final parameter SI.Mass m_wall = rho_wall * area_h * s_wall "Wall mass";
            // Assumptions
            parameter Boolean allowFlowReversal = system.allowFlowReversal "allow flow reversal, false restricts to design direction (port_a -> port_b)" annotation(
              Dialog(tab = "Assumptions"),
              Evaluate = true);
            parameter Modelica.Fluid.Types.Dynamics energyDynamics = system.energyDynamics "Formulation of energy balance" annotation(
              Evaluate = true,
              Dialog(tab = "Assumptions", group = "Dynamics"));
            parameter Modelica.Fluid.Types.Dynamics massDynamics = system.massDynamics "Formulation of mass balance" annotation(
              Evaluate = true,
              Dialog(tab = "Assumptions", group = "Dynamics"));
            parameter Modelica.Fluid.Types.Dynamics momentumDynamics = system.momentumDynamics "Formulation of momentum balance, if pressureLoss options available" annotation(
              Evaluate = true,
              Dialog(tab = "Assumptions", group = "Dynamics"));
            //Initialization pipe 1
            parameter SI.Temperature Twall_start "Start value of wall temperature" annotation(
              Dialog(tab = "Initialization", group = "Wall"));
            parameter SI.Temperature dT "Start value for pipe_1.T - pipe_2.T" annotation(
              Dialog(tab = "Initialization", group = "Wall"));
            parameter Boolean use_T_start = true "Use T_start if true, otherwise h_start" annotation(
              Evaluate = true,
              Dialog(tab = "Initialization"));
            parameter Medium_1.AbsolutePressure p_a_start1 = Medium_1.p_default "Start value of pressure" annotation(
              Dialog(tab = "Initialization", group = "Fluid 1"));
            parameter Medium_1.AbsolutePressure p_b_start1 = Medium_1.p_default "Start value of pressure" annotation(
              Dialog(tab = "Initialization", group = "Fluid 1"));
            parameter Medium_1.Temperature T_start_1 = if use_T_start then Medium_1.T_default else Medium_1.temperature_phX((p_a_start1 + p_b_start1) / 2, h_start_1, X_start_1) "Start value of temperature" annotation(
              Evaluate = true,
              Dialog(tab = "Initialization", group = "Fluid 1", enable = use_T_start));
            parameter Medium_1.SpecificEnthalpy h_start_1 = if use_T_start then Medium_1.specificEnthalpy_pTX((p_a_start1 + p_b_start1) / 2, T_start_1, X_start_1) else Medium_1.h_default "Start value of specific enthalpy" annotation(
              Evaluate = true,
              Dialog(tab = "Initialization", group = "Fluid 1", enable = not use_T_start));
            parameter Medium_1.MassFraction X_start_1[Medium_1.nX] = Medium_1.X_default "Start value of mass fractions m_i/m" annotation(
              Dialog(tab = "Initialization", group = "Fluid 1", enable = Medium_1.nXi > 0));
            parameter Medium_1.MassFlowRate m_flow_start_1 = system.m_flow_start "Start value of mass flow rate" annotation(
              Evaluate = true,
              Dialog(tab = "Initialization", group = "Fluid 1"));
            //Initialization pipe 2
            parameter Medium_2.AbsolutePressure p_a_start2 = Medium_2.p_default "Start value of pressure" annotation(
              Dialog(tab = "Initialization", group = "Fluid 2"));
            parameter Medium_2.AbsolutePressure p_b_start2 = Medium_2.p_default "Start value of pressure" annotation(
              Dialog(tab = "Initialization", group = "Fluid 2"));
            parameter Medium_2.Temperature T_start_2 = if use_T_start then Medium_2.T_default else Medium_2.temperature_phX((p_a_start2 + p_b_start2) / 2, h_start_2, X_start_2) "Start value of temperature" annotation(
              Evaluate = true,
              Dialog(tab = "Initialization", group = "Fluid 2", enable = use_T_start));
            parameter Medium_2.SpecificEnthalpy h_start_2 = if use_T_start then Medium_2.specificEnthalpy_pTX((p_a_start2 + p_b_start2) / 2, T_start_2, X_start_2) else Medium_2.h_default "Start value of specific enthalpy" annotation(
              Evaluate = true,
              Dialog(tab = "Initialization", group = "Fluid 2", enable = not use_T_start));
            parameter Medium_2.MassFraction X_start_2[Medium_2.nX] = Medium_2.X_default "Start value of mass fractions m_i/m" annotation(
              Dialog(tab = "Initialization", group = "Fluid 2", enable = Medium_2.nXi > 0));
            parameter Medium_2.MassFlowRate m_flow_start_2 = system.m_flow_start "Start value of mass flow rate" annotation(
              Evaluate = true,
              Dialog(tab = "Initialization", group = "Fluid 2"));
            //Pressure drop and heat transfer
            replaceable model FlowModel_1 = Modelica.Fluid.Pipes.BaseClasses.FlowModels.DetailedPipeFlow constrainedby Modelica.Fluid.Pipes.BaseClasses.FlowModels.PartialStaggeredFlowModel "Characteristic of wall friction" annotation(
               choicesAllMatching,
               Dialog(tab = "General", group = "Fluid 1"));
            replaceable model FlowModel_2 = Modelica.Fluid.Pipes.BaseClasses.FlowModels.DetailedPipeFlow constrainedby Modelica.Fluid.Pipes.BaseClasses.FlowModels.PartialStaggeredFlowModel "Characteristic of wall friction" annotation(
               choicesAllMatching,
               Dialog(tab = "General", group = "Fluid 2"));
            parameter SI.Length roughness_1 = 2.5e-5 "Absolute roughness of pipe (default = smooth steel pipe)" annotation(
              Dialog(tab = "General", group = "Fluid 1"));
            parameter SI.Length roughness_2 = 2.5e-5 "Absolute roughness of pipe (default = smooth steel pipe)" annotation(
              Dialog(tab = "General", group = "Fluid 2"));
            //Display variables
            SI.HeatFlowRate Q_flow_1 "Total heat flow rate of pipe 1";
            SI.HeatFlowRate Q_flow_2 "Total heat flow rate of pipe 2";
            BaseClasses.WallConstProps wall(rho_wall = rho_wall, c_wall = c_wall, T_start = Twall_start, k_wall = k_wall, dT = dT, s = s_wall, energyDynamics = energyDynamics, n = nNodes, area_h = area_h) annotation(
              Placement(transformation(extent = {{-29, -23}, {9, 35}})));
            Pipes.DynamicPipe pipe_1(redeclare final package Medium = Medium_1, final isCircular = false, final diameter = 0, final nNodes = nNodes, final allowFlowReversal = allowFlowReversal, final energyDynamics = energyDynamics, final massDynamics = massDynamics, final momentumDynamics = momentumDynamics, final length = length, final use_HeatTransfer = use_HeatTransfer, redeclare final model HeatTransfer = HeatTransfer_1, final use_T_start = use_T_start, final T_start = T_start_1, final h_start = h_start_1, final X_start = X_start_1, final m_flow_start = m_flow_start_1, final perimeter = perimeter_1, final crossArea = crossArea_1, final p_a_start = p_a_start1, final p_b_start = p_b_start1, final roughness = roughness_1, redeclare final model FlowModel = FlowModel_1, final modelStructure = modelStructure_1) annotation(
              Placement(transformation(extent = {{-40, -80}, {20, -20}})));
            Pipes.DynamicPipe pipe_2(redeclare final package Medium = Medium_2, final nNodes = nNodes, final allowFlowReversal = allowFlowReversal, final energyDynamics = energyDynamics, final massDynamics = massDynamics, final momentumDynamics = momentumDynamics, final length = length, final isCircular = false, final diameter = 0, final use_HeatTransfer = use_HeatTransfer, redeclare final model HeatTransfer = HeatTransfer_2, final use_T_start = use_T_start, final T_start = T_start_2, final h_start = h_start_2, final X_start = X_start_2, final m_flow_start = m_flow_start_2, final perimeter = perimeter_2, final crossArea = crossArea_2, final p_a_start = p_a_start2, final p_b_start = p_b_start2, final roughness = roughness_2, redeclare final model FlowModel = FlowModel_2, final modelStructure = modelStructure_2) annotation(
              Placement(transformation(extent = {{20, 88}, {-40, 28}})));
            Modelica.Fluid.Interfaces.FluidPort_b port_b1(redeclare final package Medium = Medium_1) annotation(
              Placement(transformation(extent = {{100, -12}, {120, 8}})));
            Modelica.Fluid.Interfaces.FluidPort_a port_a1(redeclare final package Medium = Medium_1) annotation(
              Placement(transformation(extent = {{-120, -12}, {-100, 8}})));
            Modelica.Fluid.Interfaces.FluidPort_b port_b2(redeclare final package Medium = Medium_2) annotation(
              Placement(transformation(extent = {{-120, 36}, {-100, 56}})));
            Modelica.Fluid.Interfaces.FluidPort_a port_a2(redeclare final package Medium = Medium_2) annotation(
              Placement(transformation(extent = {{100, -56}, {120, -36}})));
          equation
            Q_flow_1 = sum(pipe_1.heatTransfer.Q_flows);
            Q_flow_2 = sum(pipe_2.heatTransfer.Q_flows);
            connect(pipe_2.port_b, port_b2) annotation(
              Line(points = {{-40, 58}, {-76, 58}, {-76, 46}, {-110, 46}}, color = {0, 127, 255}, thickness = 0.5));
            connect(pipe_1.port_b, port_b1) annotation(
              Line(points = {{20, -50}, {42, -50}, {42, -2}, {110, -2}}, color = {0, 127, 255}, thickness = 0.5));
            connect(pipe_1.port_a, port_a1) annotation(
              Line(points = {{-40, -50}, {-75.3, -50}, {-75.3, -2}, {-110, -2}}, color = {0, 127, 255}, thickness = 0.5));
            connect(pipe_2.port_a, port_a2) annotation(
              Line(points = {{20, 58}, {65, 58}, {65, -46}, {110, -46}}, color = {0, 127, 255}, thickness = 0.5));
            connect(wall.heatPort_b, pipe_1.heatPorts) annotation(
              Line(points = {{-10, -8.5}, {-10, -36.8}, {-9.7, -36.8}}, color = {191, 0, 0}));
            connect(pipe_2.heatPorts[nNodes:(-1):1], wall.heatPort_a[1:nNodes]) annotation(
              Line(points = {{-10.3, 44.8}, {-10.3, 31.7}, {-10, 31.7}, {-10, 20.5}}, color = {127, 0, 0}));
            annotation(
              Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, -26}, {100, -30}}, lineColor = {0, 0, 0}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Forward), Rectangle(extent = {{-100, 30}, {100, 26}}, lineColor = {0, 0, 0}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Forward), Rectangle(extent = {{-100, 60}, {100, 30}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {0, 63, 125}), Rectangle(extent = {{-100, -30}, {100, -60}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {0, 63, 125}), Rectangle(extent = {{-100, 26}, {100, -26}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {0, 128, 255}), Text(extent = {{-150, 110}, {150, 70}}, lineColor = {0, 0, 255}, textString = "%name"), Line(points = {{30, -85}, {-60, -85}}, color = {0, 128, 255}), Polygon(points = {{20, -70}, {60, -85}, {20, -100}, {20, -70}}, lineColor = {0, 128, 255}, fillColor = {0, 128, 255}, fillPattern = FillPattern.Solid), Line(points = {{30, 77}, {-60, 77}}, color = {0, 128, 255}), Polygon(points = {{-50, 92}, {-90, 77}, {-50, 62}, {-50, 92}}, lineColor = {0, 128, 255}, fillColor = {0, 128, 255}, fillPattern = FillPattern.Solid)}),
              Documentation(info = "<html>
<p>Simple model of a heat exchanger consisting of two pipes and one wall in between.
For both fluids geometry parameters, such as heat transfer area and cross section as well as heat transfer and pressure drop correlations may be chosen.
The flow scheme may be concurrent or counterflow, defined by the respective flow directions of the fluids entering the component.
The design flow direction with positive m_flow variables is counterflow.</p>
</html>"));
          end BasicHX;

          model WallConstProps "Pipe wall with capacitance, assuming 1D heat conduction and constant material properties"
            parameter Integer n(min = 1) = 1 "Segmentation perpendicular to heat conduction";
            //Geometry
            parameter SI.Length s "Wall thickness";
            parameter SI.Area area_h "Heat transfer area";
            //Material properties
            parameter SI.Density rho_wall "Density of wall material";
            parameter SI.SpecificHeatCapacity c_wall "Specific heat capacity of wall material";
            parameter SI.ThermalConductivity k_wall "Thermal conductivity of wall material";
            parameter SI.Mass[n] m = fill(rho_wall * area_h * s / n, n) "Distribution of wall mass";
            //Initialization
            outer Modelica.Fluid.System system;
            parameter Modelica.Fluid.Types.Dynamics energyDynamics = system.energyDynamics "Formulation of energy balance" annotation(
              Evaluate = true,
              Dialog(tab = "Assumptions", group = "Dynamics"));
            parameter SI.Temperature T_start "Wall temperature start value";
            parameter SI.Temperature dT "Start value for port_b.T - port_a.T";
            //Temperatures
            SI.Temperature[n] Tb(each start = T_start + 0.5 * dT);
            SI.Temperature[n] Ta(each start = T_start - 0.5 * dT);
            SI.Temperature[n] T(start = ones(n) * T_start, each stateSelect = StateSelect.prefer) "Wall temperature";
            Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a[n] heatPort_a "Thermal port" annotation(
              Placement(transformation(extent = {{-20, 40}, {20, 60}})));
            Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a[n] heatPort_b "Thermal port" annotation(
              Placement(transformation(extent = {{-20, -40}, {20, -60}})));
          initial equation
            if energyDynamics == Modelica.Fluid.Types.Dynamics.SteadyStateInitial then
              der(T) = zeros(n);
            elseif energyDynamics == Modelica.Fluid.Types.Dynamics.FixedInitial then
              T = ones(n) * T_start;
            end if;
          equation
            for i in 1:n loop
              assert(m[i] > 0, "Wall has negative dimensions");
              if energyDynamics == Modelica.Fluid.Types.Dynamics.SteadyState then
                0 = heatPort_a[i].Q_flow + heatPort_b[i].Q_flow;
              else
                c_wall * m[i] * der(T[i]) = heatPort_a[i].Q_flow + heatPort_b[i].Q_flow;
              end if;
              heatPort_a[i].Q_flow = 2 * k_wall / s * (Ta[i] - T[i]) * area_h / n;
              heatPort_b[i].Q_flow = 2 * k_wall / s * (Tb[i] - T[i]) * area_h / n;
            end for;
            Ta = heatPort_a.T;
            Tb = heatPort_b.T;
            annotation(
              Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, 40}, {100, -40}}, lineColor = {0, 0, 0}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Forward), Text(extent = {{-82, 18}, {76, -18}}, lineColor = {0, 0, 0}, textString = "%name")}),
              Documentation(revisions = "<html>
<ul>
<li><i>04 Mar 2006</i>
    by Katrin Pr&ouml;l&szlig;:<br>
       Model added to the Fluid library</li>
</ul>
</html>", info = "<html>
Simple model of circular (or any other closed shape) wall to be used for pipe (or duct) models. Heat conduction is regarded one dimensional, capacitance is lumped at the arithmetic mean temperature. The spatial discretization (parameter <code>n</code>) is meant to correspond to a connected fluid model discretization.
</html>"));
          end WallConstProps;
        end BaseClasses;
      end HeatExchanger;

      package TraceSubstances "Library demonstrating the usage of trace substances"
        extends Modelica.Icons.ExamplesPackage;

        model RoomCO2 "Demonstrates a room volume with CO2 accumulation"
          extends Modelica.Icons.Example;
          package Medium = Modelica.Media.Air.MoistAir(extraPropertiesNames = {"CO2"}, C_nominal = {1.519E-3});
          Modelica.Blocks.Sources.Constant C(k = 0.3 * 1.519E-3) "substance concentration, raising to 1000 PPM CO2" annotation(
            Placement(transformation(extent = {{-94, -28}, {-74, -8}})));
          Sources.FixedBoundary boundary4(nPorts = 1, redeclare package Medium = Medium) annotation(
            Placement(transformation(extent = {{80, -20}, {60, 0}})));
          Sensors.TraceSubstances traceVolume(redeclare package Medium = Medium) annotation(
            Placement(transformation(extent = {{0, 20}, {20, 40}})));
          inner System system(energyDynamics = Modelica.Fluid.Types.Dynamics.FixedInitial) annotation(
            Placement(transformation(extent = {{52, 36}, {72, 56}})));
          Sources.MassFlowSource_T boundary1(use_C_in = true, m_flow = 100 / 1.2 / 3600 * 5, redeclare package Medium = Medium, nPorts = 2, X = Medium.X_default) annotation(
            Placement(transformation(extent = {{-60, -20}, {-40, 0}})));
          Modelica.Fluid.Vessels.ClosedVolume volume(C_start = {1.519E-3}, V = 100, redeclare package Medium = Medium, nPorts = 2, X_start = {0.015, 0.085}, massDynamics = Modelica.Fluid.Types.Dynamics.FixedInitial, use_portsData = false) annotation(
            Placement(transformation(extent = {{-20, 0}, {0, 20}})));
          Modelica.Fluid.Pipes.StaticPipe pipe(redeclare package Medium = Medium, length = 1, diameter = 0.15, redeclare model FlowModel = Modelica.Fluid.Pipes.BaseClasses.FlowModels.DetailedPipeFlow(show_Res = true)) annotation(
            Placement(transformation(extent = {{20, -20}, {40, 0}})));
          Sensors.TraceSubstances traceSource(redeclare package Medium = Medium) annotation(
            Placement(transformation(extent = {{-40, 20}, {-20, 40}})));
        equation
          connect(C.y, boundary1.C_in[1]) annotation(
            Line(points = {{-73, -18}, {-60, -18}}, color = {0, 0, 127}));
          connect(pipe.port_b, boundary4.ports[1]) annotation(
            Line(points = {{40, -10}, {60, -10}}, color = {0, 127, 255}));
          connect(volume.ports[2], pipe.port_a) annotation(
            Line(points = {{-8, 0}, {-8, -10}, {20, -10}}, color = {0, 127, 255}));
          connect(traceVolume.port, pipe.port_a) annotation(
            Line(points = {{10, 20}, {10, -10}, {20, -10}}, color = {0, 127, 255}));
          connect(boundary1.ports[2], volume.ports[1]) annotation(
            Line(points = {{-40, -10.5}, {-12, -10.5}, {-12, 0}}, color = {0, 127, 255}));
          connect(boundary1.ports[1], traceSource.port) annotation(
            Line(points = {{-40, -9.5}, {-30, -9.5}, {-30, 20}}, color = {0, 127, 255}));
          annotation(
            experiment(StopTime = 3600),
            Documentation(info = "<html>
<p>
This example consists of a volume with a carbon dioxide concentration that corresponds to
1.519E-3 kg/kg, which is equal to 1000 PPM.
There is a fresh air stream with a carbon dioxide concentration of about 300 PPM.
The fresh air stream is such that the air exchange rate is about 5 air changes per hour.
After 1 hour of ventilation, the volume's carbon dioxide concentration is close to the
concentration of the fresh air.
</p>
<p>
The nominal value for the trace substance is set to <code>C_nominal={1.519E-3}</code>.
This scales the residual equations that are used by the solver to the right order
of magnitude.
</p>
<p>
<img src=\"modelica://Modelica/Resources/Images/Fluid/Examples/RoomCO2.png\" border=\"1\"
     alt=\"RoomCO2.png\">
</p>
</html>"),
            __Dymola_Commands(file(ensureSimulated = true) = "modelica://Modelica/Resources/Scripts/Dymola/Fluid/RoomCO2/plotConcentrations.mos" "plot concentrations"));
        end RoomCO2;

        model RoomCO2WithControls "Demonstrates a room volume with CO2 controls"
          extends Modelica.Icons.Example;
          package Medium = Modelica.Media.Air.MoistAir(extraPropertiesNames = {"CO2"}, C_nominal = {1.519E-1});
          Modelica.Blocks.Sources.Constant CAtm(k = 0.3 * 1.519E-3) "Atmospheric trace substance concentration, corresponding to 300 PPM CO2" annotation(
            Placement(transformation(extent = {{-100, -48}, {-80, -28}})));
          Sources.FixedBoundary boundary4(nPorts = 1, redeclare package Medium = Medium) annotation(
            Placement(transformation(extent = {{92, -40}, {72, -20}})));
          Sensors.TraceSubstances traceVolume(redeclare package Medium = Medium) annotation(
            Placement(transformation(extent = {{20, 0}, {40, 20}})));
          inner System system(energyDynamics = Modelica.Fluid.Types.Dynamics.FixedInitial, use_eps_Re = true, m_flow_nominal = 0.1, eps_m_flow = 1e-2) annotation(
            Placement(transformation(extent = {{70, 70}, {90, 90}})));
          Sources.MassFlowSource_T freshAir(use_C_in = true, redeclare package Medium = Medium, use_m_flow_in = true, nPorts = 2) annotation(
            Placement(transformation(extent = {{-70, -40}, {-50, -20}})));
          Modelica.Fluid.Vessels.ClosedVolume volume(medium(Xi(nominal = 0.01)), C_start = {1.519E-3}, V = 100, redeclare package Medium = Medium, massDynamics = Modelica.Fluid.Types.Dynamics.FixedInitial, use_portsData = false, nPorts = 4) annotation(
            Placement(transformation(extent = {{0, -20}, {20, 0}})));
          Pipes.DynamicPipe ductOut(mCs_scaled(each nominal = 0.01), mediums(each Xi(nominal = 0.01)), redeclare package Medium = Medium, diameter = 0.15, redeclare model FlowModel = Modelica.Fluid.Pipes.BaseClasses.FlowModels.DetailedPipeFlow(show_Res = true), modelStructure = Modelica.Fluid.Types.ModelStructure.a_v_b, length = 5) "Outlet duct" annotation(
            Placement(transformation(extent = {{40, -40}, {60, -20}})));
          Sensors.TraceSubstances traceDuctIn(redeclare package Medium = Medium) "Trace substance at duct inlet" annotation(
            Placement(transformation(extent = {{-54, 0}, {-34, 20}})));
          Sources.MassFlowSource_T peopleSource(m_flow = 100 / 1.2 / 3600 * 5, redeclare package Medium = Medium, use_m_flow_in = true, use_C_in = false, C = {100}, nPorts = 1) "CO2 emitted by room occupants." annotation(
            Placement(transformation(extent = {{-38, -98}, {-18, -78}})));
          Modelica.Blocks.Sources.CombiTimeTable NumberOfPeople(table = [0, 0; 9 * 3600, 0; 9 * 3600, 10; 11 * 3600, 10; 11 * 3600, 2; 13 * 3600, 2; 13 * 3600, 15; 15 * 3600, 15; 15 * 3600, 5; 18 * 3600, 5; 18 * 3600, 0; 24 * 3600, 0]) "Time table for number of people in the room" annotation(
            Placement(transformation(extent = {{-100, -90}, {-80, -70}})));
          Modelica.Blocks.Math.Gain gain(k = 8.18E-6 / 100) "CO2 mass flow rate, released per 100 person (there is another 100 factor in peopleSource)" annotation(
            Placement(transformation(extent = {{-68, -90}, {-48, -70}})));
          Modelica.Blocks.Math.Gain gain1(k = -100 * 1.2 / 3600 * 5) "Nominal fresh air flow rate (for u=1)" annotation(
            Placement(transformation(extent = {{0, 40}, {20, 60}})));
          Modelica.Blocks.Math.Gain gainSensor(k = 1 / 1.519E-3) "Gain to normalize CO2 measurement signal. y=1 corresponds to 1000 PPM" annotation(
            Placement(transformation(extent = {{60, 0}, {80, 20}})));
          Modelica.Blocks.Sources.Constant CO2Set(k = 1) "Normalized CO2 set point" annotation(
            Placement(transformation(extent = {{-80, 40}, {-60, 60}})));
          Modelica.Blocks.Continuous.LimPID PID(controllerType = Modelica.Blocks.Types.SimpleController.PI, yMax = 0, yMin = -1, Ti = 10, k = 10) annotation(
            Placement(transformation(extent = {{-40, 40}, {-20, 60}})));
          Pipes.DynamicPipe ductIn(mCs_scaled(each nominal = 0.01), mediums(each Xi(nominal = 0.01)), redeclare package Medium = Medium, diameter = 0.15, redeclare model FlowModel = Modelica.Fluid.Pipes.BaseClasses.FlowModels.DetailedPipeFlow(show_Res = true), modelStructure = Modelica.Fluid.Types.ModelStructure.a_v_b, length = 5) "Inlet duct" annotation(
            Placement(transformation(extent = {{-38, -40}, {-18, -20}})));
          Sensors.TraceSubstances traceDuctOut(redeclare package Medium = Medium) "Trace substance at duct outlet" annotation(
            Placement(transformation(extent = {{-20, 0}, {0, 20}})));
        equation
          connect(CAtm.y, freshAir.C_in[1]) annotation(
            Line(points = {{-79, -38}, {-70, -38}}, color = {0, 0, 127}));
          connect(ductOut.port_b, boundary4.ports[1]) annotation(
            Line(points = {{60, -30}, {72, -30}}, color = {0, 127, 255}));
          connect(NumberOfPeople.y[1], gain.u) annotation(
            Line(points = {{-79, -80}, {-70, -80}}, color = {0, 0, 127}));
          connect(gain.y, peopleSource.m_flow_in) annotation(
            Line(points = {{-47, -80}, {-38, -80}}, color = {0, 0, 127}));
          connect(traceVolume.C, gainSensor.u) annotation(
            Line(points = {{41, 10}, {58, 10}}, color = {0, 0, 127}));
          connect(CO2Set.y, PID.u_s) annotation(
            Line(points = {{-59, 50}, {-42, 50}}, color = {0, 0, 127}));
          connect(gainSensor.y, PID.u_m) annotation(
            Line(points = {{81, 10}, {90, 10}, {90, 30}, {-30, 30}, {-30, 38}}, color = {0, 0, 127}));
          connect(PID.y, gain1.u) annotation(
            Line(points = {{-19, 50}, {-2, 50}}, color = {0, 0, 127}));
          connect(gain1.y, freshAir.m_flow_in) annotation(
            Line(points = {{21, 50}, {30, 50}, {30, 70}, {-88, 70}, {-88, -22}, {-70, -22}}, color = {0, 0, 127}));
          connect(ductIn.port_b, volume.ports[1]) annotation(
            Line(points = {{-18, -30}, {7, -30}, {7, -20}}, color = {0, 127, 255}));
          connect(peopleSource.ports[1], volume.ports[2]) annotation(
            Line(points = {{-18, -88}, {9, -88}, {9, -20}}, color = {0, 127, 255}));
          connect(volume.ports[3], ductOut.port_a) annotation(
            Line(points = {{11, -20}, {11, -30}, {40, -30}}, color = {0, 127, 255}));
          connect(volume.ports[4], traceVolume.port) annotation(
            Line(points = {{13, -20}, {13, -26}, {30, -26}, {30, 0}}, color = {0, 127, 255}));
          connect(freshAir.ports[1], traceDuctIn.port) annotation(
            Line(points = {{-50, -29.5}, {-44, -29.5}, {-44, 0}}, color = {0, 127, 255}));
          connect(ductIn.port_a, freshAir.ports[2]) annotation(
            Line(points = {{-38, -30}, {-38, -30.5}, {-50, -30.5}}, color = {0, 127, 255}));
          connect(traceDuctOut.port, ductIn.port_b) annotation(
            Line(points = {{-10, 0}, {-10, -30}, {-18, -30}}, color = {0, 127, 255}));
          annotation(
            experiment(StopTime = 86400, Tolerance = 1e-006),
            __Dymola_Commands(file(ensureSimulated = true) = "modelica://Modelica/Resources/Scripts/Dymola/Fluid/RoomCO2WithControls/plotStatesWithControl.mos" "plot states and controls"),
            Documentation(info = "<html>
<p>
This example illustrates a room volume with a CO2 source and a fresh air supply with feedback
control.
The CO2 emission rate is proportional to the room occupancy, which is defined by a schedule.
The fresh air flow rate is controlled such that the room CO2
concentration does not exceed <code>1000 PPM (=1.519E-3 kg/kg)</code>.
The fresh air has a CO2 concentration of <code>300 PPM</code> which corresponds to a typical
CO2 concentration in the outside air.
</p>

<p>
The CO2 emission from the occupants is implemented as a mass flow source.
Depending on the activity and size, a person emits about <code>8.18E-6 kg/s</code> CO2. In the model,
this value is multiplied by the number of occupants.
Since the mass flow rate associate with the CO2 source model contributes to the volume's energy balance,
this mass flow rate should be kept small. Thus, in the source model, we set the
CO2 concentration to <code>C={100} kg/kg</code>, and scaled the mass flow rate using
</p>

<pre>
  m_flow = 1/100 * nPeo * 8.18E-6 kg/(s*person)
</pre>

<p>
where <code>nPeo</code> is the number of people in the room.
This results in a mass flow rate that is about 5 orders of magnitudes smaller than the supply air flow rate,
and hence its contribution to the volume's energy balance is negligible.
</p>
<p>
The nominal value for the trace substance is set to <code>C_nominal={1.519E-3}</code>.
This scales the residual equations that are used by the solver to the right order
of magnitude.
</p>
<p>
<img src=\"modelica://Modelica/Resources/Images/Fluid/Examples/RoomCO2WithControls.png\" border=\"1\"
     alt=\"RoomCO2WithControls.png\">
</p>
</html>"));
        end RoomCO2WithControls;
      end TraceSubstances;

      model InverseParameterization "Demonstrates the parameterization of a pump and a pipe for given nominal values"
        extends Modelica.Icons.Example;
        replaceable package Medium = Modelica.Media.Water.StandardWater;
        //replaceable package Medium = Modelica.Media.Water.ConstantPropertyLiquidWater;
        //parameter Real eps_m_flow_turbulent = 0.0 "Turbulent flow |m_flow| >= eps_m_flow_nominal*m_flow_nominal";
        parameter Real eps_m_flow_turbulent = 0.1 "Turbulent flow |m_flow| >= eps_m_flow_nominal*m_flow_nominal";
        //parameter Real eps_m_flow_turbulent = 1 "Turbulent flow |m_flow| >= eps_m_flow_nominal*m_flow_nominal";
        Modelica.Fluid.Sources.Boundary_pT source(redeclare package Medium = Medium, nPorts = 1, use_p_in = false, p = 100000) annotation(
          Placement(transformation(extent = {{-76, 14}, {-64, 26}})));
        Modelica.Fluid.Machines.ControlledPump pump(m_flow_nominal = 1, control_m_flow = false, use_p_set = true, redeclare package Medium = Medium, p_a_nominal = 100000, p_b_nominal = 200000) annotation(
          Placement(transformation(extent = {{-40, 10}, {-20, 30}})));
        Modelica.Fluid.Fittings.SimpleGenericOrifice orifice(redeclare package Medium = Medium, diameter = 2.54e-2, use_zeta = false, zeta = 0, dp_nominal = 100000, m_flow_nominal = 1) annotation(
          Placement(transformation(extent = {{20, 10}, {40, 30}})));
        Modelica.Fluid.Sources.Boundary_pT sink(nPorts = 1, redeclare package Medium = Medium, p = 100000) annotation(
          Placement(transformation(extent = {{76, 14}, {64, 26}})));
        inner Modelica.Fluid.System system(m_flow_start = 0.4) annotation(
          Placement(transformation(extent = {{-90, 50}, {-70, 70}})));
        Modelica.Fluid.Pipes.StaticPipe pipe1(redeclare package Medium = Medium, diameter = 2.54e-2, length = 0, redeclare model FlowModel = Modelica.Fluid.Pipes.BaseClasses.FlowModels.NominalTurbulentPipeFlow(show_Res = true, m_flow_nominal = 1, m_flow_turbulent = eps_m_flow_turbulent * 1, dp_nominal = 100000)) annotation(
          Placement(transformation(extent = {{20, -30}, {40, -10}})));
        Modelica.Fluid.Sources.Boundary_pT sink1(nPorts = 1, redeclare package Medium = Medium, p = 200000) annotation(
          Placement(transformation(extent = {{76, -26}, {64, -14}})));
        Modelica.Blocks.Sources.Ramp p_set(height = 0.2e5, offset = 1.9e5, duration = 8, startTime = 1) annotation(
          Placement(transformation(extent = {{-50, 40}, {-30, 60}})));
        Modelica.Fluid.Pipes.StaticPipe pipe2(redeclare package Medium = Medium, diameter = 2.54e-2, length = 1000, redeclare model FlowModel = Modelica.Fluid.Pipes.BaseClasses.FlowModels.NominalLaminarFlow(show_Res = true, dp_nominal = 100000, m_flow_nominal = 1)) annotation(
          Placement(transformation(extent = {{20, -70}, {40, -50}})));
        Modelica.Fluid.Sources.Boundary_pT sink2(nPorts = 1, redeclare package Medium = Medium, p = 200000) annotation(
          Placement(transformation(extent = {{76, -66}, {64, -54}})));
      equation
        connect(orifice.port_b, sink.ports[1]) annotation(
          Line(points = {{40, 20}, {64, 20}}, color = {0, 127, 255}));
        connect(source.ports[1], pump.port_a) annotation(
          Line(points = {{-64, 20}, {-40, 20}}, color = {0, 127, 255}));
        connect(pump.port_b, orifice.port_a) annotation(
          Line(points = {{-20, 20}, {20, 20}}, color = {0, 127, 255}));
        connect(pipe1.port_b, sink1.ports[1]) annotation(
          Line(points = {{40, -20}, {64, -20}}, color = {0, 127, 255}));
        connect(pipe1.port_a, pump.port_b) annotation(
          Line(points = {{20, -20}, {0, -20}, {0, 20}, {-20, 20}}, color = {0, 127, 255}));
        connect(p_set.y, pump.p_set) annotation(
          Line(points = {{-29, 50}, {-25, 50}, {-25, 28.2}}, color = {0, 0, 127}));
        connect(pipe2.port_b, sink2.ports[1]) annotation(
          Line(points = {{40, -60}, {64, -60}}, color = {0, 127, 255}));
        connect(pump.port_b, pipe2.port_a) annotation(
          Line(points = {{-20, 20}, {0, 20}, {0, -60}, {20, -60}}, color = {0, 127, 255}));
        annotation(
          __Dymola_Commands(file(ensureSimulated = true) = "modelica://Modelica/Resources/Scripts/Dymola/Fluid/InverseParameterization/plotResults.mos" "plotResults"),
          experiment(StopTime = 10, Interval = 0.001),
          Documentation(info = "<html>
<p>
A pump, an orifice and two pipes are parameterized with simple nominal values.
Note that pipe1 and pipe2 use the flowModel NominalTurbulentFlow and NominalLaminarFlow, respectively,
which do not require the specification of geometry data.
Instead pathLengths_nominal are obtained internally for given nominal pressure loss and nominal mass flow rate.
</p>
<p>
The pump controls a pressure ramp from 1.9 bar to 2.1 bar.
This causes an appropriate ramp on the mass flow rate of the orifice, which has a boundary pressure of 1 bar.
Flow reversal occurs in the pipes, which have a boundary pressure of 2 bar.
The Command plotResults can be used to see the pump speed N, which is controlled ideally to obtain the pressure ramp.
Moreover the internally obtained nominal design values that fulfill the nominal operating conditions as well as
the Reynolds number, m_flows_turbulent, and dps_fg_turbulent are plotted.
</p>
<p>
Note that the large value for pipe2.flowModel.pathLengths_nominal[1] is only meaningful under the made assumption of laminar flow,
 which is hardly possible for a real pipe.
</p>
<p>
Once the geometries have been designed, the NominalTurbulentPipeFlow correlations can easily be replaced with
TurbulentPipeFlow or DetailedPipeFlow correlations. Similarly the ControlledPump can be replaced with a PrescribedPump
to investigate a real controller or with a Pump with rotational shaft to investigate inertia effects.
</p>
<p>
The model has the parameter <b>eps_m_flow_turbulent</b> that
can be used to change the flow through pipe1 from fully turbulent (eps_m_flow_turbulent=0) to fully laminar (eps_m_flow_turbulent>actual flow).
Invoke plotResults and see pipe1.port_a.m_flow. Relating the actual flow to pipe1.flowModel.m_flows_turbulent[1],
it can be seen that eps_m_flow_turbulent=0.1 is an appropriate choice for the given pipe diameter.
</p>
<img src=\"modelica://Modelica/Resources/Images/Fluid/Examples/InverseParametrization.png\" border=\"1\"
     alt=\"InverseParametrization.png\">
</html>"));
      end InverseParameterization;

      package Explanatory "A set of examples illustrating when special attention has to be paid"
        extends Modelica.Icons.ExamplesPackage;

        model MeasuringTemperature "Differences between using one port with and without explicit junction model and two port sensors for fluid temperature measuring"
          extends Modelica.Icons.Example;
          Modelica.Fluid.Sensors.Temperature T_onePort(redeclare package Medium = Modelica.Media.Water.StandardWater) annotation(
            Placement(transformation(extent = {{-20, 40}, {0, 60}})));
          Modelica.Fluid.Sensors.TemperatureTwoPort T_twoPort(redeclare package Medium = Modelica.Media.Water.StandardWater) annotation(
            Placement(transformation(extent = {{-20, -20}, {0, 0}})));
          inner Modelica.Fluid.System system(energyDynamics = Modelica.Fluid.Types.Dynamics.FixedInitial) annotation(
            Placement(transformation(extent = {{-100, 56}, {-80, 76}})));
          Modelica.Fluid.Vessels.OpenTank openTankCold2(nPorts = 1, redeclare package Medium = Modelica.Media.Water.StandardWater, height = 2, crossArea = 2, portsData = {Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.05)}, level_start = 0.975, T_start = 293.15) annotation(
            Placement(transformation(extent = {{20, 6}, {40, 26}})));
          Modelica.Fluid.Vessels.OpenTank openTankCold1(redeclare package Medium = Modelica.Media.Water.StandardWater, height = 2, crossArea = 2, portsData = {Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.05)}, nPorts = 1, level_start = 0.975, T_start = 293.15) annotation(
            Placement(transformation(extent = {{20, 60}, {40, 80}})));
          Modelica.Fluid.Vessels.OpenTank openTankHot1(nPorts = 1, level_start = 1, redeclare package Medium = Modelica.Media.Water.StandardWater, height = 2, crossArea = 2, portsData = {Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.05)}, T_start = 353.15) annotation(
            Placement(transformation(extent = {{60, 40}, {80, 60}})));
          Modelica.Fluid.Vessels.OpenTank openTankHot2(nPorts = 1, level_start = 1, redeclare package Medium = Modelica.Media.Water.StandardWater, height = 2, crossArea = 2, portsData = {Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.05)}, T_start = 353.15) annotation(
            Placement(transformation(extent = {{60, -10}, {80, 10}})));
          Modelica.Fluid.Sources.MassFlowSource_T mFlow1(nPorts = 1, redeclare package Medium = Modelica.Media.Water.StandardWater, use_m_flow_in = true, use_T_in = false, T = 323.15) annotation(
            Placement(transformation(extent = {{-60, 30}, {-40, 50}})));
          Modelica.Fluid.Sources.MassFlowSource_T mFlow2(nPorts = 1, redeclare package Medium = Modelica.Media.Water.StandardWater, use_m_flow_in = true, T = 323.15) annotation(
            Placement(transformation(extent = {{-60, -20}, {-40, 0}})));
          Modelica.Blocks.Sources.Sine sine(freqHz = 1) annotation(
            Placement(transformation(extent = {{-100, 10}, {-80, 30}})));
          Modelica.Fluid.Sensors.Temperature T_junction(redeclare package Medium = Modelica.Media.Water.StandardWater) annotation(
            Placement(transformation(extent = {{-20, -80}, {0, -60}})));
          Modelica.Fluid.Vessels.OpenTank openTankCold3(nPorts = 1, redeclare package Medium = Modelica.Media.Water.StandardWater, height = 2, crossArea = 2, portsData = {Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.05)}, level_start = 0.975, T_start = 293.15) annotation(
            Placement(transformation(extent = {{20, -60}, {40, -40}})));
          Modelica.Fluid.Vessels.OpenTank openTankHot3(nPorts = 1, level_start = 1, redeclare package Medium = Modelica.Media.Water.StandardWater, height = 2, crossArea = 2, portsData = {Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.05)}, T_start = 353.15) annotation(
            Placement(transformation(extent = {{60, -80}, {80, -60}})));
          Modelica.Fluid.Sources.MassFlowSource_T mFlow3(nPorts = 1, redeclare package Medium = Modelica.Media.Water.StandardWater, use_m_flow_in = true, T = 323.15) annotation(
            Placement(transformation(extent = {{-60, -90}, {-40, -70}})));
          Modelica.Fluid.Fittings.TeeJunctionIdeal junctionIdeal(redeclare package Medium = Modelica.Media.Water.StandardWater) annotation(
            Placement(transformation(extent = {{20, -90}, {40, -70}})));
        equation
          connect(mFlow2.ports[1], T_twoPort.port_a) annotation(
            Line(points = {{-40, -10}, {-20, -10}}, color = {0, 127, 255}));
          connect(mFlow1.ports[1], T_onePort.port) annotation(
            Line(points = {{-40, 40}, {-10, 40}}, color = {0, 127, 255}));
          connect(sine.y, mFlow1.m_flow_in) annotation(
            Line(points = {{-79, 20}, {-70, 20}, {-70, 48}, {-60, 48}}, color = {0, 0, 127}));
          connect(sine.y, mFlow2.m_flow_in) annotation(
            Line(points = {{-79, 20}, {-70, 20}, {-70, -2}, {-60, -2}}, color = {0, 0, 127}));
          connect(mFlow3.ports[1], T_junction.port) annotation(
            Line(points = {{-40, -80}, {-10, -80}}, color = {0, 127, 255}));
          connect(sine.y, mFlow3.m_flow_in) annotation(
            Line(points = {{-79, 20}, {-70, 20}, {-70, -72}, {-60, -72}}, color = {0, 0, 127}));
          connect(T_junction.port, junctionIdeal.port_1) annotation(
            Line(points = {{-10, -80}, {20, -80}}, color = {0, 127, 255}));
          connect(T_twoPort.port_b, openTankCold2.ports[1]) annotation(
            Line(points = {{0, -10}, {0, -10}, {30, -10}, {30, 6}}, color = {0, 127, 255}));
          connect(T_onePort.port, openTankHot1.ports[1]) annotation(
            Line(points = {{-10, 40}, {-10, 40}, {70, 40}}, color = {0, 127, 255}));
          connect(T_twoPort.port_b, openTankHot2.ports[1]) annotation(
            Line(points = {{0, -10}, {30, -10}, {70, -10}}, color = {0, 127, 255}));
          connect(junctionIdeal.port_3, openTankCold3.ports[1]) annotation(
            Line(points = {{30, -70}, {30, -65}, {30, -60}}, color = {0, 127, 255}));
          connect(junctionIdeal.port_2, openTankHot3.ports[1]) annotation(
            Line(points = {{40, -80}, {55.5, -80}, {70, -80}}, color = {0, 127, 255}));
          connect(T_onePort.port, openTankCold1.ports[1]) annotation(
            Line(points = {{-10, 40}, {30, 40}, {30, 60}}, color = {0, 127, 255}));
          annotation(
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{42, 52}, {62, 46}}, lineColor = {0, 0, 0}, textString = "T=80"), Text(extent = {{-4, 76}, {18, 70}}, lineColor = {0, 0, 0}, textString = "T=20"), Text(extent = {{-62, 20}, {-36, 12}}, lineColor = {0, 0, 0}, textString = "T=50")}),
            Documentation(info = "<html>
<p>
This model demonstrates the differences that occur when using
one- and two-port temperature sensors with and without explicit junction models.
As shown in the next figure, the same system is shown in 3 different variations.
In all cases exactly the same fluid system is defined. The only difference is
how the temperature is measured:
</p>

<blockquote>
<img src=\"modelica://Modelica/Resources/Images/Fluid/Examples/MeasuringTemperature1.png\"
     alt=\"MeasuringTemperature1.png\">
</blockquote>

<p>
A pre-defined mass flow rate is present so that fluid flows from the reservoir to the
tanks and after 0.5 s the mass flows from the tanks to the reservoir.
The reservoir has a temperature of 50<sup>0</sup>C whereas the tanks have an
initial temperature of 20<sup>0</sup>C and of 80<sup>0</sup>C. The initial height of the
tanks is made in such a form that fluid always flows out of the cold tank.
When the fluid flows from the reservoir to the tanks, then it mixes with the
cold tank and enters the hot tank.
When the fluid flow from the tanks to the reservoir, then the cold and hot water
from the two tanks first mixes and the flows to the reservoir.
</p>

<p>
A one-port sensor measures the <em>mixing</em> temperature at a connection point.
Therefore T_onePort.T (the blue curve in the figure below) is the
temperature of the mixing point.
A two-port sensor measures the temperature at the <em>upstream</em> side.
Therefore T_twoPort.T (the red curve in the figure below which is identical
to the green curve) shows first the temperature of the reservoir and then
the mixing temperature when fluid flows from the tanks to the reservoir.
The same is measured with T_junction.T (the green curve below), because
the one-port sensor is connected between the mass flow source and the junction
and since the mixing takes place in the junction, the same situation is
present as for T_twoPort.T.
</p>

<blockquote>
<img src=\"modelica://Modelica/Resources/Images/Fluid/Examples/MeasuringTemperature2.png\"
     alt=\"MeasuringTemperature2.png\">
</blockquote>

</html>"),
            experiment(StopTime = 1.1));
        end MeasuringTemperature;

        model MomentumBalanceFittings "Illustrating a case in which kinetic terms play a major role in the momentum balance"
          extends Modelica.Icons.Example;
          Modelica.Fluid.Sources.Boundary_pT leftBoundary1(redeclare package Medium = Modelica.Media.Water.StandardWaterOnePhase, nPorts = 1, p = 100000) annotation(
            Placement(transformation(extent = {{-80, 20}, {-60, 40}})));
          Modelica.Fluid.Sources.Boundary_pT rightBoundary1(redeclare package Medium = Modelica.Media.Water.StandardWaterOnePhase, nPorts = 1, p = 110000) annotation(
            Placement(transformation(extent = {{80, 20}, {60, 40}})));
          Modelica.Fluid.Fittings.AbruptAdaptor suddenExpansion1(diameter_a = 0.1, diameter_b = 0.2, redeclare package Medium = Modelica.Media.Water.StandardWaterOnePhase, show_totalPressures = true, show_portVelocities = true) annotation(
            Placement(transformation(extent = {{-12, 20}, {8, 40}})));
          Modelica.Fluid.Sources.Boundary_pT leftBoundary2(redeclare package Medium = Modelica.Media.Water.StandardWaterOnePhase, nPorts = 1, p = 100000) annotation(
            Placement(transformation(extent = {{-90, -40}, {-70, -20}})));
          Modelica.Fluid.Sources.Boundary_pT rightBoundary2(redeclare package Medium = Modelica.Media.Water.StandardWaterOnePhase, nPorts = 1, p = 110000) annotation(
            Placement(transformation(extent = {{90, -40}, {70, -20}})));
          Modelica.Fluid.Fittings.AbruptAdaptor suddenExpansion2(diameter_a = 0.1, diameter_b = 0.2, redeclare package Medium = Modelica.Media.Water.StandardWaterOnePhase, show_totalPressures = true, show_portVelocities = true) annotation(
            Placement(transformation(extent = {{-10, -40}, {10, -20}})));
          Modelica.Fluid.Fittings.AbruptAdaptor leftAdaptor(diameter_a = 0.1, redeclare package Medium = Modelica.Media.Water.StandardWaterOnePhase, diameter_b = Modelica.Constants.inf) annotation(
            Placement(transformation(extent = {{-40, -40}, {-60, -20}})));
          Modelica.Fluid.Fittings.AbruptAdaptor rightAdaptor(redeclare package Medium = Modelica.Media.Water.StandardWaterOnePhase, diameter_a = 0.2, diameter_b = Modelica.Constants.inf) annotation(
            Placement(transformation(extent = {{40, -40}, {60, -20}})));
          inner System system annotation(
            Placement(transformation(extent = {{-92, 56}, {-72, 76}})));
        equation
          connect(leftBoundary1.ports[1], suddenExpansion1.port_a) annotation(
            Line(points = {{-60, 30}, {-12, 30}}, color = {0, 127, 255}));
          connect(suddenExpansion1.port_b, rightBoundary1.ports[1]) annotation(
            Line(points = {{8, 30}, {60, 30}}, color = {0, 127, 255}));
          connect(leftAdaptor.port_b, leftBoundary2.ports[1]) annotation(
            Line(points = {{-60, -30}, {-70, -30}}, color = {0, 127, 255}));
          connect(leftAdaptor.port_a, suddenExpansion2.port_a) annotation(
            Line(points = {{-40, -30}, {-10, -30}}, color = {0, 127, 255}));
          connect(suddenExpansion2.port_b, rightAdaptor.port_a) annotation(
            Line(points = {{10, -30}, {40, -30}}, color = {0, 127, 255}));
          connect(rightAdaptor.port_b, rightBoundary2.ports[1]) annotation(
            Line(points = {{60, -30}, {70, -30}}, color = {0, 127, 255}));
          annotation(
            Documentation(info = "<html>
<p>
This example shows the use of a sudden expansion / contraction model, which is connected to two boundary conditions prescribing static pressure. Notice that the prescribed static pressure on the right boundary is higher than on the left one. Still, the fluid flows from left to right.
</p>
<p>
The reason for this is that the boundary conditions model infinite reservoirs with an infinite diameter and thus zero flow velocity. The sudden expansion model does however have two ends with finite diameters, and, as explained in the <a href=\"modelica://Modelica.Fluid.UsersGuide.Overview\">Overview</a> of the Users' Guide, the momentum balance is not fulfilled exactly for this type of connections. Using a simple <code>connect()</code>-statement, the difference of the kinetic terms is neglected, which is not reasonable in the present model: At the left boundary condition it is zero, and on the left side of the sudden expansion it has a non-zero value. It is not reasonable to neglect it in the shown model, because there is little friction and therefore these kinetic effects dominate. Consequently, only modelling these effects explicitly leads to the correct results.
</p>
<p>
To do so, two additional sudden expansions / contractions are included in the model. The diameter is set to <code>inf</code> close to the boundaries and the proper values close to the original model. These additional components now introduce <i>exact</i> momentum balances and the results are as expected.
</p>
<p>
The total pressures offer an additional perspective on the model. After setting the parameter <code>show_totalPressures</code> on the Advanced tab of the <code>AbruptAdaptor</code>s to <code>true</code>, the total pressures are included in said models and may be plotted. This allows to confirm that the <b>total</b> pressure <i>always</i> reduces along the flow direction, even in the upper model.
</p>

<img src=\"modelica://Modelica/Resources/Images/Fluid/Examples/MomentumBalanceFittings.png\" border=\"1\"
     alt=\"MomentumBalanceFittings.png\">
</html>"),
            __Dymola_Commands(file = "modelica://Modelica/Resources/Scripts/Dymola/Fluid/MomentumBalanceFittings/Plot the model results.mos" "Plot the model results"),
            experiment(StopTime = 1.1));
        end MomentumBalanceFittings;
      end Explanatory;
      annotation(
        preferredView = "info");
    end Examples;

    model System "System properties and default values (ambient, flow direction, initialization)"
      package Medium = Modelica.Media.Interfaces.PartialMedium "Medium model for default start values" annotation(
        choicesAllMatching = true);
      parameter Modelica.SIunits.AbsolutePressure p_ambient = 101325 "Default ambient pressure" annotation(
        Dialog(group = "Environment"));
      parameter Modelica.SIunits.Temperature T_ambient = 293.15 "Default ambient temperature" annotation(
        Dialog(group = "Environment"));
      parameter Modelica.SIunits.Acceleration g = Modelica.Constants.g_n "Constant gravity acceleration" annotation(
        Dialog(group = "Environment"));
      // Assumptions
      parameter Boolean allowFlowReversal = true "= false to restrict to design flow direction (port_a -> port_b)" annotation(
        Dialog(tab = "Assumptions"),
        Evaluate = true);
      parameter Modelica.Fluid.Types.Dynamics energyDynamics = Modelica.Fluid.Types.Dynamics.DynamicFreeInitial "Default formulation of energy balances" annotation(
        Evaluate = true,
        Dialog(tab = "Assumptions", group = "Dynamics"));
      parameter Modelica.Fluid.Types.Dynamics massDynamics = energyDynamics "Default formulation of mass balances" annotation(
        Evaluate = true,
        Dialog(tab = "Assumptions", group = "Dynamics"));
      final parameter Modelica.Fluid.Types.Dynamics substanceDynamics = massDynamics "Default formulation of substance balances" annotation(
        Evaluate = true,
        Dialog(tab = "Assumptions", group = "Dynamics"));
      final parameter Modelica.Fluid.Types.Dynamics traceDynamics = massDynamics "Default formulation of trace substance balances" annotation(
        Evaluate = true,
        Dialog(tab = "Assumptions", group = "Dynamics"));
      parameter Modelica.Fluid.Types.Dynamics momentumDynamics = Modelica.Fluid.Types.Dynamics.SteadyState "Default formulation of momentum balances, if options available" annotation(
        Evaluate = true,
        Dialog(tab = "Assumptions", group = "Dynamics"));
      // Initialization
      parameter Modelica.SIunits.MassFlowRate m_flow_start = 0 "Default start value for mass flow rates" annotation(
        Dialog(tab = "Initialization"));
      parameter Modelica.SIunits.AbsolutePressure p_start = p_ambient "Default start value for pressures" annotation(
        Dialog(tab = "Initialization"));
      parameter Modelica.SIunits.Temperature T_start = T_ambient "Default start value for temperatures" annotation(
        Dialog(tab = "Initialization"));
      // Advanced
      parameter Boolean use_eps_Re = false "= true to determine turbulent region automatically using Reynolds number" annotation(
        Evaluate = true,
        Dialog(tab = "Advanced"));
      parameter Modelica.SIunits.MassFlowRate m_flow_nominal = if use_eps_Re then 1 else 1e2 * m_flow_small "Default nominal mass flow rate" annotation(
        Dialog(tab = "Advanced", enable = use_eps_Re));
      parameter Real eps_m_flow(min = 0) = 1e-4 "Regularization of zero flow for |m_flow| < eps_m_flow*m_flow_nominal" annotation(
        Dialog(tab = "Advanced", enable = use_eps_Re));
      parameter Modelica.SIunits.AbsolutePressure dp_small(min = 0) = 1 "Default small pressure drop for regularization of laminar and zero flow" annotation(
        Dialog(tab = "Advanced", group = "Classic", enable = not use_eps_Re));
      parameter Modelica.SIunits.MassFlowRate m_flow_small(min = 0) = 1e-2 "Default small mass flow rate for regularization of laminar and zero flow" annotation(
        Dialog(tab = "Advanced", group = "Classic", enable = not use_eps_Re));
    initial equation
//assert(use_eps_Re, "*** Using classic system.m_flow_small and system.dp_small."
//       + " They do not distinguish between laminar flow and regularization of zero flow."
//       + " Absolute small values are error prone for models with local nominal values."
//       + " Moreover dp_small can generally be obtained automatically."
//       + " Please update the model to new system.use_eps_Re = true  (see system, Advanced tab). ***",
//       level=AssertionLevel.warning);
      annotation(
        defaultComponentName = "system",
        defaultComponentPrefixes = "inner",
        missingInnerMessage = "
Your model is using an outer \"system\" component but
an inner \"system\" component is not defined.
For simulation drag Modelica.Fluid.System into your model
to specify system properties.",
        Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, 100}, {100, -100}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-150, 150}, {150, 110}}, lineColor = {0, 0, 255}, textString = "%name"), Line(points = {{-86, -30}, {82, -30}}), Line(points = {{-82, -68}, {-52, -30}}), Line(points = {{-48, -68}, {-18, -30}}), Line(points = {{-14, -68}, {16, -30}}), Line(points = {{22, -68}, {52, -30}}), Line(points = {{74, 84}, {74, 14}}), Polygon(points = {{60, 14}, {88, 14}, {74, -18}, {60, 14}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Text(extent = {{16, 20}, {60, -18}}, lineColor = {0, 0, 0}, textString = "g"), Text(extent = {{-90, 82}, {74, 50}}, lineColor = {0, 0, 0}, textString = "defaults"), Line(points = {{-82, 14}, {-42, -20}, {2, 30}}, thickness = 0.5), Ellipse(extent = {{-10, 40}, {12, 18}}, pattern = LinePattern.None, lineColor = {0, 0, 0}, fillColor = {255, 0, 0}, fillPattern = FillPattern.Solid)}),
        Documentation(info = "<html>
<p>
 A system component is needed in each fluid model to provide system-wide settings, such as ambient conditions and overall modeling assumptions.
 The system settings are propagated to the fluid models using the inner/outer mechanism.
</p>
<p>
 A model should never directly use system parameters.
 Instead a local parameter should be declared, which uses the global setting as default.
 The only exceptions are:</p>
 <ul>
  <li>the gravity system.g,</li>
  <li>the global system.eps_m_flow, which is used to define a local m_flow_small for the local m_flow_nominal:
      <pre>m_flow_small = system.eps_m_flow*m_flow_nominal</pre>
  </li>
 </ul>
<p>
 The global system.m_flow_small and system.dp_small are classic parameters.
 They do not distinguish between laminar flow and regularization of zero flow.
 Absolute small values are error prone for models with local nominal values.
 Moreover dp_small can generally be obtained automatically.
 Consider using the new system.use_eps_Re = true (see Advanced tab).
</p>
</html>"));
    end System;

    package Vessels "Devices for storing fluid"
      extends Modelica.Icons.VariantsPackage;

      model ClosedVolume "Volume of fixed size, closed to the ambient, with inlet/outlet ports"
        import Modelica.Constants.pi;
        // Mass and energy balance, ports
        extends Modelica.Fluid.Vessels.BaseClasses.PartialLumpedVessel(final fluidVolume = V, vesselArea = pi * (3 / 4 * V) ^ (2 / 3), heatTransfer(surfaceAreas = {4 * pi * (3 / 4 * V / pi) ^ (2 / 3)}));
        parameter SI.Volume V "Volume";
      equation
        Wb_flow = 0;
        for i in 1:nPorts loop
          vessel_ps_static[i] = medium.p;
        end for;
        annotation(
          defaultComponentName = "volume",
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-100, 100}, {100, -100}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.Sphere, fillColor = {170, 213, 255}), Text(extent = {{-150, 12}, {150, -18}}, lineColor = {0, 0, 0}, textString = "V=%V")}),
          Documentation(info = "<html>
<p>
Ideally mixed volume of constant size with two fluid ports and one medium model.
The flow properties are computed from the upstream quantities, pressures are equal in both nodes and the medium model if <code>use_portsData=false</code>.
Heat transfer through a thermal port is possible, it equals zero if the port remains unconnected.
A spherical shape is assumed for the heat transfer area, with V=4/3*pi*r^3, A=4*pi*r^2.
Ideal heat transfer is assumed per default; the thermal port temperature is equal to the medium temperature.
</p>
<p>
If <code>use_portsData=true</code>, the port pressures represent the pressures just after the outlet (or just before the inlet) in the attached pipe.
The hydraulic resistances <code>portsData.zeta_in</code> and <code>portsData.zeta_out</code> determine the dissipative pressure drop between volume and port depending on
the direction of mass flow. See <a href=\"modelica://Modelica.Fluid.Vessels.BaseClasses.VesselPortsData\">VesselPortsData</a> and <i>[Idelchik, Handbook of Hydraulic Resistance, 2004]</i>.
</p>
</html>"));
      end ClosedVolume;

      model OpenTank "Simple tank with inlet/outlet ports"
        import Modelica.Constants.pi;
        // Tank properties
        SI.Height level(stateSelect = StateSelect.prefer, start = level_start_eps) "Level height of tank";
        SI.Volume V(stateSelect = StateSelect.never) "Actual tank volume";
        // Tank geometry
        parameter SI.Height height "Height of tank";
        parameter SI.Area crossArea "Area of tank";
        // Ambient
        parameter Medium.AbsolutePressure p_ambient = system.p_ambient "Tank surface pressure" annotation(
          Dialog(tab = "Assumptions", group = "Ambient"));
        parameter Medium.Temperature T_ambient = system.T_ambient "Tank surface Temperature" annotation(
          Dialog(tab = "Assumptions", group = "Ambient"));
        // Initialization
        parameter SI.Height level_start(min = 0) = 0.5 * height "Start value of tank level" annotation(
          Dialog(tab = "Initialization"));
        // Mass and energy balance, ports
        extends Modelica.Fluid.Vessels.BaseClasses.PartialLumpedVessel(final fluidVolume = V, final fluidLevel = level, final fluidLevel_max = height, final vesselArea = crossArea, heatTransfer(surfaceAreas = {crossArea + 2 * sqrt(crossArea * pi) * level}), final initialize_p = false, final p_start = p_ambient);
      protected
        final parameter SI.Height level_start_eps = max(level_start, Modelica.Constants.eps);
      equation
// Total quantities
        V = crossArea * level "Volume of fluid";
        medium.p = p_ambient;
// Source termsEnergy balance
        if Medium.singleState or energyDynamics == Types.Dynamics.SteadyState then
          Wb_flow = 0 "Mechanical work is neglected, since also neglected in medium model (otherwise unphysical small temperature change, if tank level changes)";
        else
          Wb_flow = -p_ambient * der(V);
        end if;
//Determine port properties
        for i in 1:nPorts loop
          vessel_ps_static[i] = max(0, level - portsData_height[i]) * system.g * medium.d + p_ambient;
        end for;
      initial equation
        if massDynamics == Types.Dynamics.FixedInitial then
          level = level_start_eps;
        elseif massDynamics == Types.Dynamics.SteadyStateInitial then
          der(level) = 0;
        end if;
        annotation(
          defaultComponentName = "tank",
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}, initialScale = 0.2), graphics = {Rectangle(extent = {{-100, 100}, {100, -100}}, lineColor = {255, 255, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.VerticalCylinder), Rectangle(extent = DynamicSelect({{-100, -100}, {100, 10}}, {{-100, -100}, {100, (-100) + 200 * level / height}}), lineColor = {0, 0, 0}, fillColor = {85, 170, 255}, fillPattern = FillPattern.VerticalCylinder), Line(points = {{-100, 100}, {-100, -100}, {100, -100}, {100, 100}}, color = {0, 0, 0}), Text(extent = {{-95, 60}, {95, 40}}, lineColor = {0, 0, 0}, textString = "level ="), Text(extent = {{-95, -24}, {95, -44}}, lineColor = {0, 0, 0}, textString = DynamicSelect("%level_start", String(level, minimumLength = 1, significantDigits = 2)))}),
          Documentation(info = "<html>
<p>
Model of a tank that is open to the ambient at the fixed pressure
<code>p_ambient</code>.
</p>
<p>
The vector of connectors <b>ports</b> represents fluid ports at configurable heights, relative to the bottom of tank.
Fluid can flow either out of or in to each port.
</p>
The following assumptions are made:
<ul>
<li>The tank is filled with a single or multiple-substance medium having a density higher than the density of the ambient medium.</li>
<li>The fluid has uniform density, temperature and mass fractions</li>
<li>No liquid is leaving the tank through the open top; the simulation breaks with an assertion if the liquid level growths over the height.</li>
</ul>
<p>
The port pressures represent the pressures just after the outlet (or just before the inlet) in the attached pipe.
The hydraulic resistances <code>portsData.zeta_in</code> and <code>portsData.zeta_out</code> determine the dissipative pressure drop between tank and port depending on
the direction of mass flow. See <a href=\"modelica://Modelica.Fluid.Vessels.BaseClasses.VesselPortsData\">VesselPortsData</a> and <i>[Idelchik, Handbook of Hydraulic Resistance, 2004]</i>.
</p>
<p>
With the setting <code>use_portsData=false</code>, the port pressure represents the static head
at the height of the respective port.
The relationship between pressure drop and mass flow rate at the port must then be provided by connected components;
Heights of ports as well as kinetic and potential energy of fluid entering or leaving are not taken into account anymore.
</p>
</html>", revisions = "<html>
<ul>
<li><i>Dec. 12, 2008</i> by Ruediger Franke: move port definitions
   to BaseClasses.PartialLumpedVessel; also use energy and mass balance from common base class</li>
<li><i>Dec. 8, 2008</i> by Michael Wetter (LBNL):<br>
Implemented trace substances.</li>
<li><i>Jan. 6, 2006</i> by Katja Poschlad, Manuel Remelhe (AST Uni Dortmund),
   Martin Otter (DLR):<br>
   Implementation based on former tank model.</li>
<li><i>Oct. 29, 2007</i> by Carsten Heinrich (ILK Dresden):<br>
Adapted to the new fluid library interfaces:
<ul> <li>FluidPorts_b is used instead of FluidPort_b (due to it is defined as an array of ports)</li>
    <li>Port name changed from port to ports</li></ul>Updated documentation.</li>
<li><i>Apr. 25, 2006</i> by Katrin Pr&ouml;l&szlig; (TUHH):<br>
Limitation to bottom ports only, added inlet and outlet loss factors.</li>
</ul>
</html>"));
      end OpenTank;

      package BaseClasses "Base classes used in the Vessels package (only of interest to build new component models)"
        extends Modelica.Icons.BasesPackage;

        partial model PartialLumpedVessel "Lumped volume with a vector of fluid ports and replaceable heat transfer model"
          extends Modelica.Fluid.Interfaces.PartialLumpedVolume;
          // Port definitions
          parameter Integer nPorts = 0 "Number of ports" annotation(
            Evaluate = true,
            Dialog(connectorSizing = true, tab = "General", group = "Ports"));
          VesselFluidPorts_b ports[nPorts](redeclare each package Medium = Medium) "Fluid inlets and outlets" annotation(
            Placement(transformation(extent = {{-40, -10}, {40, 10}}, origin = {0, -100})));
          // Port properties
          parameter Boolean use_portsData = true "= false to neglect pressure loss and kinetic energy" annotation(
            Evaluate = true,
            Dialog(tab = "General", group = "Ports"));
          parameter Modelica.Fluid.Vessels.BaseClasses.VesselPortsData[if use_portsData then nPorts else 0] portsData "Data of inlet/outlet ports" annotation(
            Dialog(tab = "General", group = "Ports", enable = use_portsData));
          parameter Medium.MassFlowRate m_flow_nominal = if system.use_eps_Re then system.m_flow_nominal else 1e2 * system.m_flow_small "Nominal value for mass flow rates in ports" annotation(
            Dialog(tab = "Advanced", group = "Port properties"));
          parameter SI.MassFlowRate m_flow_small(min = 0) = if system.use_eps_Re then system.eps_m_flow * m_flow_nominal else system.m_flow_small "Regularization range at zero mass flow rate" annotation(
            Dialog(tab = "Advanced", group = "Port properties"));
          parameter Boolean use_Re = system.use_eps_Re "= true, if turbulent region is defined by Re, otherwise by m_flow_small" annotation(
            Dialog(tab = "Advanced", group = "Port properties"),
            Evaluate = true);
          Medium.EnthalpyFlowRate ports_H_flow[nPorts];
          Medium.MassFlowRate ports_mXi_flow[nPorts, Medium.nXi];
          Medium.MassFlowRate[Medium.nXi] sum_ports_mXi_flow "Substance mass flows through ports";
          Medium.ExtraPropertyFlowRate ports_mC_flow[nPorts, Medium.nC];
          Medium.ExtraPropertyFlowRate[Medium.nC] sum_ports_mC_flow "Trace substance mass flows through ports";
          // Heat transfer through boundary
          parameter Boolean use_HeatTransfer = false "= true to use the HeatTransfer model" annotation(
            Dialog(tab = "Assumptions", group = "Heat transfer"));
          replaceable model HeatTransfer = Modelica.Fluid.Vessels.BaseClasses.HeatTransfer.IdealHeatTransfer constrainedby Modelica.Fluid.Vessels.BaseClasses.HeatTransfer.PartialVesselHeatTransfer "Wall heat transfer" annotation(
             Dialog(tab = "Assumptions", group = "Heat transfer", enable = use_HeatTransfer),
             choicesAllMatching = true);
          HeatTransfer heatTransfer(redeclare final package Medium = Medium, final n = 1, final states = {medium.state}, final use_k = use_HeatTransfer) annotation(
            Placement(transformation(extent = {{-10, -10}, {30, 30}}, rotation = 90, origin = {-50, -10})));
          Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a heatPort if use_HeatTransfer annotation(
            Placement(transformation(extent = {{-110, -10}, {-90, 10}})));
          // Conservation of kinetic energy
          Medium.Density[nPorts] portInDensities "densities of the fluid at the device boundary";
          SI.Velocity[nPorts] portVelocities "velocities of fluid flow at device boundary";
          SI.EnergyFlowRate[nPorts] ports_E_flow "flow of kinetic and potential energy at device boundary";
          // Note: should use fluidLevel_start - portsData.height
          Real[nPorts] s(each start = fluidLevel_max) "curve parameters for port flows vs. port pressures; for further details see, Modelica Tutorial: Ideal switching devices";
          Real[nPorts] ports_penetration "penetration of port with fluid, depending on fluid level and port diameter";
          // treatment of pressure losses at ports
          SI.Area[nPorts] portAreas = {Modelica.Constants.pi / 4 * portsData_diameter[i] ^ 2 for i in 1:nPorts};
          Medium.AbsolutePressure[nPorts] vessel_ps_static "static pressures inside the vessel at the height of the corresponding ports, zero flow velocity";
          // determination of turbulent region
          constant SI.ReynoldsNumber Re_turbulent = 100 "cf. suddenExpansion";
          SI.MassFlowRate[nPorts] m_flow_turbulent;
        protected
          input SI.Height fluidLevel = 0 "level of fluid in the vessel for treating heights of ports";
          parameter SI.Height fluidLevel_max = 1 "maximum level of fluid in the vessel";
          parameter SI.Area vesselArea = Modelica.Constants.inf "Area of the vessel used to relate to cross flow area of ports";
          // Treatment of use_portsData=false to neglect portsData and to not require its specification either in this case.
          // Remove portsData conditionally if use_portsData=false. Simplify their use in model equations by always
          // providing portsData_diameter and portsData_height, independent of the use_portsData setting.
          // Note: this moreover serves as work-around if a tool does not support a zero sized portsData record.
          Modelica.Blocks.Interfaces.RealInput[nPorts] portsData_diameter_internal = portsData.diameter if use_portsData and nPorts > 0;
          Modelica.Blocks.Interfaces.RealInput[nPorts] portsData_height_internal = portsData.height if use_portsData and nPorts > 0;
          Modelica.Blocks.Interfaces.RealInput[nPorts] portsData_zeta_in_internal = portsData.zeta_in if use_portsData and nPorts > 0;
          Modelica.Blocks.Interfaces.RealInput[nPorts] portsData_zeta_out_internal = portsData.zeta_out if use_portsData and nPorts > 0;
          Modelica.Blocks.Interfaces.RealInput[nPorts] portsData_diameter;
          Modelica.Blocks.Interfaces.RealInput[nPorts] portsData_height;
          Modelica.Blocks.Interfaces.RealInput[nPorts] portsData_zeta_in;
          Modelica.Blocks.Interfaces.RealInput[nPorts] portsData_zeta_out;
          Modelica.Blocks.Interfaces.BooleanInput[nPorts] regularFlow(each start = true);
          Modelica.Blocks.Interfaces.BooleanInput[nPorts] inFlow(each start = false);
        equation
          mb_flow = sum(ports.m_flow);
          mbXi_flow = sum_ports_mXi_flow;
          mbC_flow = sum_ports_mC_flow;
          Hb_flow = sum(ports_H_flow) + sum(ports_E_flow);
          Qb_flow = heatTransfer.Q_flows[1];
// Only one connection allowed to a port to avoid unwanted ideal mixing
          for i in 1:nPorts loop
            assert(cardinality(ports[i]) <= 1, "
each ports[i] of volume can at most be connected to one component.
If two or more connections are present, ideal mixing takes
place with these connections, which is usually not the intention
of the modeller. Increase nPorts to add an additional port.
            ");
          end for;
// Check for correct solution
          assert(fluidLevel <= fluidLevel_max, "Vessel is overflowing (fluidLevel > fluidLevel_max = " + String(fluidLevel) + ")");
          assert(fluidLevel > (-1e-6 * fluidLevel_max), "Fluid level (= " + String(fluidLevel) + ") is below zero meaning that the solution failed.");
// Boundary conditions
// treatment of conditional portsData
          connect(portsData_diameter, portsData_diameter_internal);
          connect(portsData_height, portsData_height_internal);
          connect(portsData_zeta_in, portsData_zeta_in_internal);
          connect(portsData_zeta_out, portsData_zeta_out_internal);
          if not use_portsData then
            portsData_diameter = zeros(nPorts);
            portsData_height = zeros(nPorts);
            portsData_zeta_in = zeros(nPorts);
            portsData_zeta_out = zeros(nPorts);
          end if;
// actual definition of port variables
          for i in 1:nPorts loop
            portInDensities[i] = Medium.density(Medium.setState_phX(vessel_ps_static[i], inStream(ports[i].h_outflow), inStream(ports[i].Xi_outflow)));
            if use_portsData then
// dp = 0.5*zeta*d*v*|v|
// Note: assume vessel_ps_static for portVelocities to avoid algebraic loops for ports.p
              portVelocities[i] = smooth(0, ports[i].m_flow / portAreas[i] / Medium.density(Medium.setState_phX(vessel_ps_static[i], actualStream(ports[i].h_outflow), actualStream(ports[i].Xi_outflow))));
// Note: the penetration should not go too close to zero as this would prevent a vessel from running empty
              ports_penetration[i] = Utilities.regStep(fluidLevel - portsData_height[i] - 0.1 * portsData_diameter[i], 1, 1e-3, 0.1 * portsData_diameter[i]);
              m_flow_turbulent[i] = if not use_Re then m_flow_small else max(m_flow_small, Modelica.Constants.pi / 8 * portsData_diameter[i] * (Medium.dynamicViscosity(Medium.setState_phX(vessel_ps_static[i], inStream(ports[i].h_outflow), inStream(ports[i].Xi_outflow))) + Medium.dynamicViscosity(medium.state)) * Re_turbulent);
            else
// an infinite port diameter is assumed
              portVelocities[i] = 0;
              ports_penetration[i] = 1;
              m_flow_turbulent[i] = Modelica.Constants.inf;
            end if;
// fluid flow through ports
            regularFlow[i] = fluidLevel >= portsData_height[i];
            inFlow[i] = not regularFlow[i] and (s[i] > 0 or portsData_height[i] >= fluidLevel_max);
            if regularFlow[i] then
// regular operation: fluidLevel is above ports[i]
// Note: >= covers default values of zero as well
              if use_portsData then
/* Without regularization
                 ports[i].p = vessel_ps_static[i] + 0.5*ports[i].m_flow^2/portAreas[i]^2
                              * noEvent(if ports[i].m_flow>0 then zeta_in[i]/portInDensities[i] else -zeta_out[i]/medium.d);
              */
                ports[i].p = homotopy(vessel_ps_static[i] + 0.5 / portAreas[i] ^ 2 * Utilities.regSquare2(ports[i].m_flow, m_flow_turbulent[i], (portsData_zeta_in[i] - 1 + portAreas[i] ^ 2 / vesselArea ^ 2) / portInDensities[i] * ports_penetration[i], (portsData_zeta_out[i] + 1 - portAreas[i] ^ 2 / vesselArea ^ 2) / medium.d / ports_penetration[i]), vessel_ps_static[i]);
/*
                // alternative formulation m_flow=f(dp); not allowing the ideal portsData_zeta_in[i]=1 though
                ports[i].m_flow = smooth(2, portAreas[i]*Utilities.regRoot2(ports[i].p - vessel_ps_static[i], dp_small,
                                       2*portInDensities[i]/portsData_zeta_in[i],
                                       2*medium.d/portsData_zeta_out[i]));
              */
              else
                ports[i].p = vessel_ps_static[i];
              end if;
              s[i] = fluidLevel - portsData_height[i];
            elseif inFlow[i] then
// ports[i] is above fluidLevel and has inflow
              ports[i].p = vessel_ps_static[i];
              s[i] = ports[i].m_flow;
            else
// ports[i] is above fluidLevel, preventing outflow
              ports[i].m_flow = 0;
              s[i] = (ports[i].p - vessel_ps_static[i]) / Medium.p_default * (portsData_height[i] - fluidLevel);
            end if;
            ports[i].h_outflow = medium.h;
            ports[i].Xi_outflow = medium.Xi;
            ports[i].C_outflow = C;
            ports_H_flow[i] = ports[i].m_flow * actualStream(ports[i].h_outflow) "Enthalpy flow";
            ports_E_flow[i] = ports[i].m_flow * (0.5 * portVelocities[i] * portVelocities[i] + system.g * portsData_height[i]) "Flow of kinetic and potential energy";
            ports_mXi_flow[i, :] = ports[i].m_flow * actualStream(ports[i].Xi_outflow) "Component mass flow";
            ports_mC_flow[i, :] = ports[i].m_flow * actualStream(ports[i].C_outflow) "Trace substance mass flow";
          end for;
          for i in 1:Medium.nXi loop
            sum_ports_mXi_flow[i] = sum(ports_mXi_flow[:, i]);
          end for;
          for i in 1:Medium.nC loop
            sum_ports_mC_flow[i] = sum(ports_mC_flow[:, i]);
          end for;
          connect(heatPort, heatTransfer.heatPorts[1]) annotation(
            Line(points = {{-100, 0}, {-87, 0}, {-87, 0}, {-74, 0}}, color = {191, 0, 0}));
          annotation(
            Documentation(info = "<html>
<p>
This base class extends PartialLumpedVolume with a vector of fluid ports and a replaceable wall HeatTransfer model.
</p>
<p>
The following modeling assumption are made:
<ul>
<li>homogeneous medium, i.e., phase separation is not taken into account,</li>
<li>no kinetic energy in the fluid, i.e., kinetic energy dissipates into the internal energy,</li>
<li>pressure loss definitions at vessel ports assume incompressible fluid,</li>
<li>outflow of ambient media is prevented at each port assuming check valve behavior.
    If <code> fluidlevel &lt; portsData_height[i] </code>and &nbsp; <code> ports[i].p &lt; vessel_ps_static[i]</code> mass flow at the port is set to 0.</li>
</ul>
<p>
Each port has a (hydraulic) diameter and a height above the bottom of the vessel, which can be configured using the &nbsp;<b><code>portsData</code></b> record.
Alternatively the impact of port geometries can be neglected with <code>use_portsData=false</code>. This might be useful for early
design studies. Note that this means to assume an infinite port diameter at the bottom of the vessel.
Pressure drops and heights of the ports as well as kinetic and potential energy fluid entering or leaving the vessel are neglected then.
</p>
<p>
The following variables need to be defined by an extending model:
</p>
<ul>
<li><code>input fluidVolume</code>, the volume of the fluid in the vessel,</li>
<li><code>vessel_ps_static[nPorts]</code>, the static pressures inside the vessel at the height of the corresponding ports, at zero flow velocity, and</li>
<li><code>Wb_flow</code>, work term of the energy balance, e.g., p*der(V) if the volume is not constant or stirrer power.</li>
</ul>
<p>
An extending model should define:
</p>
<ul>
<li><code>parameter vesselArea</code> (default: Modelica.Constants.inf m2), the area of the vessel, to be related to cross flow areas of the ports for the consideration of dynamic pressure effects.</li>
</ul>
<p>
Optionally the fluid level may vary in the vessel, which effects the flow through the ports at configurable <code>portsData_height[nPorts]</code>.
This is why an extending model with varying fluid level needs to define:
</p>
<ul>
<li><code>input fluidLevel (default: 0m)</code>, the level the fluid in the vessel, and</li>
<li><code>parameter fluidLevel_max (default: 1m)</code>, the maximum level that must not be exceeded. Ports at or above fluidLevel_max can only receive inflow.</li>
</ul>
<p>
An extending model should not access the <code>portsData</code> record defined in the configuration dialog,
as an access to <code>portsData</code> may fail for <code>use_portsData=false</code> or <code>nPorts=0</code>.
</p>
<p>
Instead the predefined variables
</p>
<ul>
<li><code>portsData_diameter[nPorts]</code>,</li>
<li><code>portsData_height[nPorts]</code>,</li>
<li><code>portsData_zeta_in[nPorts]</code>, and</li>
<li><code>portsData_zeta_out[nPorts]</code></li>
</ul>
<p>
should be used if these values are needed.
</p>
</html>", revisions = "<html>
<ul>
<li><i>Jan. 2009</i> by R&uuml;diger Franke: extended with
   <ul><li>portsData record and threat configurable port heights,</li>
       <li>consideration of kinetic and potential energy of fluid entering or leaving in energy balance</li>
   </ul>
</li>
<li><i>Dec. 2008</i> by R&uuml;diger Franke: derived from OpenTank, in order to make general use of configurable port diameters</li>
</ul>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-150, 110}, {150, 150}}, textString = "%name", lineColor = {0, 0, 255})}));
        end PartialLumpedVessel;

        package HeatTransfer "HeatTransfer models for vessels"
          extends Modelica.Icons.Package;

          partial model PartialVesselHeatTransfer "Base class for vessel heat transfer models"
            extends Modelica.Fluid.Interfaces.PartialHeatTransfer;
            annotation(
              Documentation(info = "<html>
Base class for vessel heat transfer models.
</html>"),
              Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-60, 64}, {60, -56}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.Sphere, fillColor = {232, 0, 0}), Text(extent = {{-38, 26}, {40, -14}}, lineColor = {0, 0, 0}, textString = "%name")}));
          end PartialVesselHeatTransfer;

          model IdealHeatTransfer "IdealHeatTransfer: Ideal heat transfer without thermal resistance"
            extends PartialVesselHeatTransfer;
          equation
            Ts = heatPorts.T;
            annotation(
              Documentation(info = "<html>
Ideal heat transfer without thermal resistance.
</html>"));
          end IdealHeatTransfer;

          model ConstantHeatTransfer "ConstantHeatTransfer: Constant heat transfer coefficient"
            extends PartialVesselHeatTransfer;
            parameter SI.CoefficientOfHeatTransfer alpha0 "constant heat transfer coefficient";
          equation
            Q_flows = {(alpha0 + k) * surfaceAreas[i] * (heatPorts[i].T - Ts[i]) for i in 1:n};
            annotation(
              Documentation(info = "<html>
Simple heat transfer correlation with constant heat transfer coefficient.
</html>"));
          end ConstantHeatTransfer;
          annotation(
            Documentation(info = "<html>
Heat transfer correlations for pipe models
</html>"));
        end HeatTransfer;

        record VesselPortsData "Data to describe inlet/outlet ports at vessels:
            diameter -- Inner (hydraulic) diameter of inlet/outlet port
            height -- Height over the bottom of the vessel
            zeta_out -- Hydraulic resistance out of vessel, default 0.5 for small diameter mounted flush with the wall
            zeta_in -- Hydraulic resistance into vessel, default 1.04 for small diameter mounted flush with the wall"
          extends Modelica.Icons.Record;
          parameter SI.Diameter diameter "Inner (hydraulic) diameter of inlet/outlet port";
          parameter SI.Height height = 0 "Height over the bottom of the vessel";
          parameter Real zeta_out(min = 0) = 0.5 "Hydraulic resistance out of vessel, default 0.5 for small diameter mounted flush with the wall";
          parameter Real zeta_in(min = 0) = 1.04 "Hydraulic resistance into vessel, default 1.04 for small diameter mounted flush with the wall";
          annotation(
            preferredView = "info",
            Documentation(info = "<html>
<h4>Vessel Port Data</h4>
<p>
This record describes the <b>ports</b> of a <b>vessel</b>. The variables in it are mostly self-explanatory (see list below); only the &zeta;
loss factors are discussed further. All data is quoted from Idelchik (1994).
</p>

<h4>Outlet Coefficients</h4>

<p>
If a <b>straight pipe with constant cross section is mounted flush with the wall</b>, its outlet pressure loss coefficient will be <code>&zeta; = 0.5</code> (Idelchik, p. 160, Diagram 3-1, paragraph 2).
</p>
<p>
If a <b>straight pipe with constant cross section is mounted into a vessel such that the entrance into it is at a distance</b> <code>b</code> from the wall (inside) the following table can be used. Herein, &delta; is the tube wall thickness (Idelchik, p. 160, Diagram 3-1, paragraph 1).
</p>

<table border=\"1\" cellspacing=\"0\" cellpadding=\"2\">
  <caption align=\"bottom\">Pressure loss coefficients for outlets, entrance at a distance from wall</caption>
  <tr>
    <td></td> <td>   </td><th colspan=\"5\" align=\"center\"> b / D_hyd  </th>
  </tr>
  <tr>
    <td></td> <td>   </td><th> 0.000 </th><th> 0.005 </th><th> 0.020 </th><th> 0.100 </th><th> 0.500-&#8734; </th>
  </tr>
  <tr>
     <th rowspan=\"5\" valign=\"middle\">&delta; / D_hyd</th> <th> 0.000 </th><td> 0.50 </td><td> 0.63  </td><td> 0.73  </td><td> 0.86  </td><td>      1.00     </td>
  </tr>
  <tr>
              <th> 0.008 </th><td> 0.50 </td><td> 0.55  </td><td> 0.62  </td><td> 0.74  </td><td>      0.88     </td>
  </tr>
  <tr>
              <th> 0.016 </th><td> 0.50 </td><td> 0.51  </td><td> 0.55  </td><td> 0.64  </td><td>      0.77     </td>
  </tr>
  <tr>
              <th> 0.024 </th><td> 0.50 </td><td> 0.50  </td><td> 0.52  </td><td> 0.58  </td><td>      0.68     </td>
  </tr>
  <tr>
              <th> 0.040 </th><td> 0.50 </td><td> 0.50  </td><td> 0.51  </td><td> 0.51  </td><td>      0.54     </td>
  </tr>
</table>

<p>
If a <b>straight pipe with a circular bellmouth inlet (collector) without baffle is mounted flush with the wall</b> then its pressure loss coefficient can be established from the following table. Herein, r is the radius of the bellmouth inlet surface (Idelchik, p. 164 f., Diagram 3-4, paragraph b)
</p>

<table border=\"1\" cellspacing=\"0\" cellpadding=\"2\">
  <caption align=\"bottom\">Pressure loss coefficients for outlets, bellmouth flush with wall</caption>
  <tr>
    <td></td> <th colspan=\"6\" align=\"center\"> r / D_hyd  </th>
  </tr>
  <tr>
    <td></td> <th> 0.01 </th><th> 0.03 </th><th> 0.05 </th><th> 0.08 </th><th> 0.16 </th><th>&ge;0.20</th>
  </tr>
  <tr>
     <th>&zeta;</th> <td> 0.44 </td><td> 0.31 </td><td> 0.22  </td><td> 0.15  </td><td> 0.06  </td><td>      0.03     </td>
  </tr>
</table>

<p>
If a <b>straight pipe with a circular bellmouth inlet (collector) without baffle is mounted at a distance from a wall</b> then its pressure loss coefficient can be established from the following table. Herein, r is the radius of the bellmouth inlet surface (Idelchik, p. 164 f., Diagram 3-4, paragraph a)
</p>

<table border=\"1\" cellspacing=\"0\" cellpadding=\"2\">
  <caption align=\"bottom\">Pressure loss coefficients for outlets, bellmouth at a distance of wall</caption>
  <tr>
    <td></td> <th colspan=\"6\" align=\"center\"> r / D_hyd  </th>
  </tr>
  <tr>
    <td></td> <th> 0.01 </th><th> 0.03 </th><th> 0.05 </th><th> 0.08 </th><th> 0.16 </th><th>&ge;0.20</th>
  </tr>
  <tr>
     <th>&zeta;</th> <td> 0.87 </td><td> 0.61 </td><td> 0.40  </td><td> 0.20  </td><td> 0.06  </td><td>      0.03     </td>
  </tr>
</table>

<h4>Inlet Coefficients</h4>

<p>
If a <b>straight pipe with constant circular cross section is mounted flush with the wall</b>, its vessel inlet pressure loss coefficient will be according to the following table (Idelchik, p. 209 f., Diagram 4-2 with <code>A_port/A_vessel = 0</code> and Idelchik, p. 640, Diagram 11-1, graph a). According to the text, <code>m = 9</code> is appropriate for fully developed turbulent flow.
</p>

<table border=\"1\" cellspacing=\"0\" cellpadding=\"2\">
  <caption align=\"bottom\">Pressure loss coefficients for inlets, circular tube flush with wall</caption>
  <tr>
    <td></td> <th colspan=\"6\" align=\"center\"> m  </th>
  </tr>
  <tr>
    <td></td> <th> 1.0 </th><th> 2.0 </th><th> 3.0 </th><th> 4.0 </th><th> 7.0 </th><th>9.0</th>
  </tr>
  <tr>
     <th>&zeta;</th> <td> 2.70 </td><td> 1.50 </td><td> 1.25  </td><td> 1.15  </td><td> 1.06  </td><td>      1.04     </td>
  </tr>
</table>

<p>
For larger port diameters, relative to the area of the vessel, the inlet pressure loss coefficient will be according to the following table (Idelchik, p. 209 f., Diagram 4-2 with <code>m = 7</code>).
</p>

<table border=\"1\" cellspacing=\"0\" cellpadding=\"2\">
  <caption align=\"bottom\">Pressure loss coefficients for inlets, circular tube flush with wall</caption>
  <tr>
    <td></td> <th colspan=\"6\" align=\"center\"> A_port / A_vessel  </th>
  </tr>
  <tr>
    <td></td> <th> 0.0 </th><th> 0.1 </th><th> 0.2 </th><th> 0.4 </th><th> 0.6 </th><th>0.8</th>
  </tr>
  <tr>
     <th>&zeta;</th> <td> 1.04 </td><td> 0.84 </td><td> 0.67  </td><td> 0.39  </td><td> 0.18  </td><td>      0.06     </td>
  </tr>
</table>

<h4>References</h4>

<dl><dt>Idelchik I.E. (1994):</dt>
    <dd><a href=\"http://www.bookfinder.com/dir/i/Handbook_of_Hydraulic_Resistance/0849399084/\"><b>Handbook
        of Hydraulic Resistance</b></a>. 3rd edition, Begell House, ISBN
        0-8493-9908-4</dd>
</dl>
</html>"));
        end VesselPortsData;

        connector VesselFluidPorts_a "Fluid connector with filled, large icon to be used for horizontally aligned vectors of FluidPorts (vector dimensions must be added after dragging)"
          extends Interfaces.FluidPort;
          annotation(
            defaultComponentName = "ports_b",
            Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{-50, -200}, {50, 200}}, initialScale = 0.2), graphics = {Text(extent = {{-75, 130}, {75, 100}}, textString = "%name"), Rectangle(extent = {{-25, 100}, {25, -100}}, lineColor = {0, 0, 0}), Ellipse(extent = {{-22, 100}, {-10, -100}}, lineColor = {0, 0, 0}, fillColor = {0, 127, 255}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-6, 100}, {6, -100}}, lineColor = {0, 0, 0}, fillColor = {0, 127, 255}, fillPattern = FillPattern.Solid), Ellipse(extent = {{10, 100}, {22, -100}}, lineColor = {0, 0, 0}, fillColor = {0, 127, 255}, fillPattern = FillPattern.Solid)}),
            Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-50, -200}, {50, 200}}, initialScale = 0.2), graphics = {Rectangle(extent = {{-50, 200}, {50, -200}}, lineColor = {0, 127, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-44, 200}, {-20, -200}}, lineColor = {0, 0, 0}, fillColor = {0, 127, 255}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-12, 200}, {12, -200}}, lineColor = {0, 0, 0}, fillColor = {0, 127, 255}, fillPattern = FillPattern.Solid), Ellipse(extent = {{20, 200}, {44, -200}}, lineColor = {0, 0, 0}, fillColor = {0, 127, 255}, fillPattern = FillPattern.Solid)}));
        end VesselFluidPorts_a;

        connector VesselFluidPorts_b "Fluid connector with outlined, large icon to be used for horizontally aligned vectors of FluidPorts (vector dimensions must be added after dragging)"
          extends Interfaces.FluidPort;
          annotation(
            defaultComponentName = "ports_b",
            Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{-50, -200}, {50, 200}}, initialScale = 0.2), graphics = {Text(extent = {{-75, 130}, {75, 100}}, textString = "%name"), Rectangle(extent = {{-25, 100}, {25, -100}}, lineColor = {0, 0, 0}), Ellipse(extent = {{-22, 100}, {-10, -100}}, lineColor = {0, 0, 0}, fillColor = {0, 127, 255}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-20, -69}, {-12, 69}}, lineColor = {0, 127, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-6, 100}, {6, -100}}, lineColor = {0, 0, 0}, fillColor = {0, 127, 255}, fillPattern = FillPattern.Solid), Ellipse(extent = {{10, 100}, {22, -100}}, lineColor = {0, 0, 0}, fillColor = {0, 127, 255}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-4, -69}, {4, 69}}, lineColor = {0, 127, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Ellipse(extent = {{12, -69}, {20, 69}}, lineColor = {0, 127, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid)}),
            Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-50, -200}, {50, 200}}, initialScale = 0.2), graphics = {Rectangle(extent = {{-50, 200}, {50, -200}}, lineColor = {0, 127, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-44, 200}, {-20, -200}}, lineColor = {0, 0, 0}, fillColor = {0, 127, 255}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-12, 200}, {12, -200}}, lineColor = {0, 0, 0}, fillColor = {0, 127, 255}, fillPattern = FillPattern.Solid), Ellipse(extent = {{20, 200}, {44, -200}}, lineColor = {0, 0, 0}, fillColor = {0, 127, 255}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-39, -118.5}, {-25, 113}}, lineColor = {0, 127, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-7, -118.5}, {7, 113}}, lineColor = {0, 127, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Ellipse(extent = {{25, -117.5}, {39, 114}}, lineColor = {0, 127, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid)}));
        end VesselFluidPorts_b;
      end BaseClasses;
      annotation(
        Documentation(info = "<html>

</html>"));
    end Vessels;

    package Pipes "Devices for conveying fluid"
      extends Modelica.Icons.VariantsPackage;

      model StaticPipe "Basic pipe flow model without storage of mass or energy"
        // extending PartialStraightPipe
        extends Modelica.Fluid.Pipes.BaseClasses.PartialStraightPipe;
        // Initialization
        parameter Medium.AbsolutePressure p_a_start = system.p_start "Start value of pressure at port a" annotation(
          Dialog(tab = "Initialization"));
        parameter Medium.AbsolutePressure p_b_start = p_a_start "Start value of pressure at port b" annotation(
          Dialog(tab = "Initialization"));
        parameter Medium.MassFlowRate m_flow_start = system.m_flow_start "Start value for mass flow rate" annotation(
          Evaluate = true,
          Dialog(tab = "Initialization"));
        FlowModel flowModel(redeclare final package Medium = Medium, final n = 2, states = {Medium.setState_phX(port_a.p, inStream(port_a.h_outflow), inStream(port_a.Xi_outflow)), Medium.setState_phX(port_b.p, inStream(port_b.h_outflow), inStream(port_b.Xi_outflow))}, vs = {port_a.m_flow / Medium.density(flowModel.states[1]) / flowModel.crossAreas[1], -port_b.m_flow / Medium.density(flowModel.states[2]) / flowModel.crossAreas[2]} / nParallel, final momentumDynamics = Types.Dynamics.SteadyState, final allowFlowReversal = allowFlowReversal, final p_a_start = p_a_start, final p_b_start = p_b_start, final m_flow_start = m_flow_start, final nParallel = nParallel, final pathLengths = {length}, final crossAreas = {crossArea, crossArea}, final dimensions = {4 * crossArea / perimeter, 4 * crossArea / perimeter}, final roughnesses = {roughness, roughness}, final dheights = {height_ab}, final g = system.g) "Flow model" annotation(
          Placement(transformation(extent = {{-38, -18}, {38, 18}})));
      equation
// Mass balance
        port_a.m_flow = flowModel.m_flows[1];
        0 = port_a.m_flow + port_b.m_flow;
        port_a.Xi_outflow = inStream(port_b.Xi_outflow);
        port_b.Xi_outflow = inStream(port_a.Xi_outflow);
        port_a.C_outflow = inStream(port_b.C_outflow);
        port_b.C_outflow = inStream(port_a.C_outflow);
// Energy balance, considering change of potential energy
// Wb_flow = v*A*dpdx + v*F_fric
//         = m_flow/d/A * (A*dpdx + A*pressureLoss.dp_fg - F_grav)
//         = m_flow/d/A * (-A*g*height_ab*d)
//         = -m_flow*g*height_ab
        port_b.h_outflow = inStream(port_a.h_outflow) - system.g * height_ab;
        port_a.h_outflow = inStream(port_b.h_outflow) + system.g * height_ab;
        annotation(
          defaultComponentName = "pipe",
          Documentation(info = "<html>
<p>Model of a straight pipe with constant cross section and with steady-state mass, momentum and energy balances, i.e., the model does not store mass or energy.
There exist two thermodynamic states, one at each fluid port. The momentum balance is formulated for the two states, taking into account
momentum flows, friction and gravity. The same result can be obtained by using <a href=\"modelica://Modelica.Fluid.Pipes.DynamicPipe\">DynamicPipe</a> with
steady-state dynamic settings. The intended use is to provide simple connections of vessels or other devices with storage, as it is done in:
</p>
<ul>
<li><a href=\"modelica://Modelica.Fluid.Examples.Tanks.EmptyTanks\">Examples.Tanks.EmptyTanks</a></li>
<li><a href=\"modelica://Modelica.Fluid.Examples.InverseParameterization\">Examples.InverseParameterization</a></li>
</ul>
<h4>Numerical Issues</h4>
<p>
With the stream connectors the thermodynamic states on the ports are generally defined by models with storage or by sources placed upstream and downstream of the static pipe.
Other non storage components in the flow path may yield to state transformation. Note that this generally leads to nonlinear equation systems if multiple static pipes,
or other flow models without storage, are directly connected.
</p>
</html>"));
      end StaticPipe;

      model DynamicPipe "Dynamic pipe model with storage of mass and energy"
        import Modelica.Fluid.Types.ModelStructure;
        // extending PartialStraightPipe
        extends Modelica.Fluid.Pipes.BaseClasses.PartialStraightPipe(final port_a_exposesState = modelStructure == ModelStructure.av_b or modelStructure == ModelStructure.av_vb, final port_b_exposesState = modelStructure == ModelStructure.a_vb or modelStructure == ModelStructure.av_vb);
        // extending PartialTwoPortFlow
        extends BaseClasses.PartialTwoPortFlow(final lengths = fill(length / n, n), final crossAreas = fill(crossArea, n), final dimensions = fill(4 * crossArea / perimeter, n), final roughnesses = fill(roughness, n), final dheights = height_ab * dxs);
        // Wall heat transfer
        parameter Boolean use_HeatTransfer = false "= true to use the HeatTransfer model" annotation(
          Dialog(tab = "Assumptions", group = "Heat transfer"));
        replaceable model HeatTransfer = Modelica.Fluid.Pipes.BaseClasses.HeatTransfer.IdealFlowHeatTransfer constrainedby Modelica.Fluid.Pipes.BaseClasses.HeatTransfer.PartialFlowHeatTransfer "Wall heat transfer" annotation(
           Dialog(tab = "Assumptions", group = "Heat transfer", enable = use_HeatTransfer),
           choicesAllMatching = true);
        Interfaces.HeatPorts_a[nNodes] heatPorts if use_HeatTransfer annotation(
          Placement(transformation(extent = {{-10, 45}, {10, 65}}), iconTransformation(extent = {{-30, 36}, {32, 52}})));
        HeatTransfer heatTransfer(redeclare final package Medium = Medium, final n = n, final nParallel = nParallel, final surfaceAreas = perimeter * lengths, final lengths = lengths, final dimensions = dimensions, final roughnesses = roughnesses, final states = mediums.state, final vs = vs, final use_k = use_HeatTransfer) "Heat transfer model" annotation(
          Placement(transformation(extent = {{-45, 20}, {-23, 42}})));
        final parameter Real[n] dxs = lengths / sum(lengths);
      equation
        Qb_flows = heatTransfer.Q_flows;
// Wb_flow = v*A*dpdx + v*F_fric
//         = v*A*dpdx + v*A*flowModel.dp_fg - v*A*dp_grav
        if n == 1 or useLumpedPressure then
          Wb_flows = dxs * (vs * dxs * (crossAreas * dxs) * (port_b.p - port_a.p + sum(flowModel.dps_fg) - system.g * (dheights * mediums.d))) * nParallel;
        else
          if modelStructure == ModelStructure.av_vb or modelStructure == ModelStructure.av_b then
            Wb_flows[2:n - 1] = {vs[i] * crossAreas[i] * ((mediums[i + 1].p - mediums[i - 1].p) / 2 + (flowModel.dps_fg[i - 1] + flowModel.dps_fg[i]) / 2 - system.g * dheights[i] * mediums[i].d) for i in 2:n - 1} * nParallel;
          else
            Wb_flows[2:n - 1] = {vs[i] * crossAreas[i] * ((mediums[i + 1].p - mediums[i - 1].p) / 2 + (flowModel.dps_fg[i] + flowModel.dps_fg[i + 1]) / 2 - system.g * dheights[i] * mediums[i].d) for i in 2:n - 1} * nParallel;
          end if;
          if modelStructure == ModelStructure.av_vb then
            Wb_flows[1] = vs[1] * crossAreas[1] * ((mediums[2].p - mediums[1].p) / 2 + flowModel.dps_fg[1] / 2 - system.g * dheights[1] * mediums[1].d) * nParallel;
            Wb_flows[n] = vs[n] * crossAreas[n] * ((mediums[n].p - mediums[n - 1].p) / 2 + flowModel.dps_fg[n - 1] / 2 - system.g * dheights[n] * mediums[n].d) * nParallel;
          elseif modelStructure == ModelStructure.av_b then
            Wb_flows[1] = vs[1] * crossAreas[1] * ((mediums[2].p - mediums[1].p) / 2 + flowModel.dps_fg[1] / 2 - system.g * dheights[1] * mediums[1].d) * nParallel;
            Wb_flows[n] = vs[n] * crossAreas[n] * ((port_b.p - mediums[n - 1].p) / 1.5 + flowModel.dps_fg[n - 1] / 2 + flowModel.dps_fg[n] - system.g * dheights[n] * mediums[n].d) * nParallel;
          elseif modelStructure == ModelStructure.a_vb then
            Wb_flows[1] = vs[1] * crossAreas[1] * ((mediums[2].p - port_a.p) / 1.5 + flowModel.dps_fg[1] + flowModel.dps_fg[2] / 2 - system.g * dheights[1] * mediums[1].d) * nParallel;
            Wb_flows[n] = vs[n] * crossAreas[n] * ((mediums[n].p - mediums[n - 1].p) / 2 + flowModel.dps_fg[n] / 2 - system.g * dheights[n] * mediums[n].d) * nParallel;
          elseif modelStructure == ModelStructure.a_v_b then
            Wb_flows[1] = vs[1] * crossAreas[1] * ((mediums[2].p - port_a.p) / 1.5 + flowModel.dps_fg[1] + flowModel.dps_fg[2] / 2 - system.g * dheights[1] * mediums[1].d) * nParallel;
            Wb_flows[n] = vs[n] * crossAreas[n] * ((port_b.p - mediums[n - 1].p) / 1.5 + flowModel.dps_fg[n] / 2 + flowModel.dps_fg[n + 1] - system.g * dheights[n] * mediums[n].d) * nParallel;
          else
            assert(false, "Unknown model structure");
          end if;
        end if;
        connect(heatPorts, heatTransfer.heatPorts) annotation(
          Line(points = {{0, 55}, {0, 54}, {-34, 54}, {-34, 38.7}}, color = {191, 0, 0}));
        annotation(
          defaultComponentName = "pipe",
          Documentation(info = "<html>
<p>Model of a straight pipe with distributed mass, energy and momentum balances. It provides the complete balance equations for one-dimensional fluid flow as formulated in <a href=\"modelica://Modelica.Fluid.UsersGuide.ComponentDefinition.BalanceEquations\">UsersGuide.ComponentDefinition.BalanceEquations</a>. </p>
<p>This generic model offers a large number of combinations of possible parameter settings. In order to reduce model complexity, consider defining and/or using a tailored model for the application at hand, such as
<a href=\"modelica://Modelica.Fluid.Examples.HeatExchanger.HeatExchangerSimulation\">HeatExchanger</a>.</p>
<p>DynamicPipe treats the partial differential equations with the finite volume method and a staggered grid scheme for momentum balances. The pipe is split into nNodes equally spaced segments along the flow path. The default value is nNodes=2. This results in two lumped mass and energy balances and one lumped momentum balance across the dynamic pipe. </p>
<p>Note that this generally leads to high-index DAEs for pressure states if dynamic pipes are directly connected to each other, or generally to models with storage exposing a thermodynamic state through the port. This may not be valid if the dynamic pipe is connected to a model with non-differentiable pressure, like a Sources.Boundary_pT with prescribed jumping pressure. The <code><b>modelStructure</b></code> can be configured as appropriate in such situations, in order to place a momentum balance between a pressure state of the pipe and a non-differentiable boundary condition. </p>
<p>The default <code><b>modelStructure</b></code> is <code>av_vb</code> (see Advanced tab). The simplest possible alternative symmetric configuration, avoiding potential high-index DAEs at the cost of the potential introduction of nonlinear equation systems, is obtained with the setting <code>nNodes=1, modelStructure=a_v_b</code>. Depending on the configured model structure, the first and the last pipe segment, or the flow path length of the first and the last momentum balance, are of half size. See the documentation of the base class <a href=\"modelica://Modelica.Fluid.Pipes.BaseClasses.PartialTwoPortFlow\">Pipes.BaseClasses.PartialTwoPortFlow</a>, also covering asymmetric configurations. </p>
<p>The <code><b>HeatTransfer</b></code> component specifies the source term <code>Qb_flows</code> of the energy balance. The default component uses a constant coefficient for the heat transfer between the bulk flow and the segment boundaries exposed through the <code>heatPorts</code>. The <code>HeatTransfer</code> model is replaceable and can be exchanged with any model extended from <a href=\"modelica://Modelica.Fluid.Pipes.BaseClasses.HeatTransfer.PartialFlowHeatTransfer\">BaseClasses.HeatTransfer.PartialFlowHeatTransfer</a>. </p>
<p>The intended use is for complex networks of pipes and other flow devices, like valves. See, e.g., </p>
<ul>
<li><a href=\"modelica://Modelica.Fluid.Examples.BranchingDynamicPipes\">Examples.BranchingDynamicPipes</a>, or </li>
<li><a href=\"modelica://Modelica.Fluid.Examples.IncompressibleFluidNetwork\">Examples.IncompressibleFluidNetwork</a>.</li>
</ul>
</html>"),
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, 44}, {100, -44}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {0, 127, 255}), Ellipse(extent = {{-72, 10}, {-52, -10}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Ellipse(extent = {{50, 10}, {70, -10}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Text(extent = {{-48, 15}, {46, -20}}, lineColor = {0, 0, 0}, textString = "%nNodes")}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, 60}, {100, 50}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Backward), Rectangle(extent = {{-100, -50}, {100, -60}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Backward), Line(points = {{100, 45}, {100, 50}}, arrow = {Arrow.None, Arrow.Filled}, pattern = LinePattern.Dot), Line(points = {{0, 45}, {0, 50}}, arrow = {Arrow.None, Arrow.Filled}, pattern = LinePattern.Dot), Line(points = {{100, -45}, {100, -50}}, arrow = {Arrow.None, Arrow.Filled}, pattern = LinePattern.Dot), Line(points = {{0, -45}, {0, -50}}, arrow = {Arrow.None, Arrow.Filled}, pattern = LinePattern.Dot), Line(points = {{-50, 60}, {-50, 50}}, pattern = LinePattern.Dot), Line(points = {{50, 60}, {50, 50}}, pattern = LinePattern.Dot), Line(points = {{0, -50}, {0, -60}}, pattern = LinePattern.Dot)}));
      end DynamicPipe;

      package BaseClasses "Base classes used in the Pipes package (only of interest to build new component models)"
        extends Modelica.Icons.BasesPackage;

        partial model PartialStraightPipe "Base class for straight pipe models"
          extends Modelica.Fluid.Interfaces.PartialTwoPort;
          // Geometry
          // Note: define nParallel as Real to support inverse calculations
          parameter Real nParallel(min = 1) = 1 "Number of identical parallel pipes" annotation(
            Dialog(group = "Geometry"));
          parameter SI.Length length "Length" annotation(
            Dialog(tab = "General", group = "Geometry"));
          parameter Boolean isCircular = true "= true if cross sectional area is circular" annotation(
            Evaluate,
            Dialog(tab = "General", group = "Geometry"));
          parameter SI.Diameter diameter "Diameter of circular pipe" annotation(
            Dialog(group = "Geometry", enable = isCircular));
          parameter SI.Area crossArea = Modelica.Constants.pi * diameter * diameter / 4 "Inner cross section area" annotation(
            Dialog(tab = "General", group = "Geometry", enable = not isCircular));
          parameter SI.Length perimeter = Modelica.Constants.pi * diameter "Inner perimeter" annotation(
            Dialog(tab = "General", group = "Geometry", enable = not isCircular));
          parameter SI.Height roughness = 2.5e-5 "Average height of surface asperities (default: smooth steel pipe)" annotation(
            Dialog(group = "Geometry"));
          final parameter SI.Volume V = crossArea * length * nParallel "volume size";
          // Static head
          parameter SI.Length height_ab = 0 "Height(port_b) - Height(port_a)" annotation(
            Dialog(group = "Static head"));
          // Pressure loss
          replaceable model FlowModel = Modelica.Fluid.Pipes.BaseClasses.FlowModels.DetailedPipeFlow constrainedby Modelica.Fluid.Pipes.BaseClasses.FlowModels.PartialStaggeredFlowModel "Wall friction, gravity, momentum flow" annotation(
             Dialog(group = "Pressure loss"),
             choicesAllMatching = true);
        equation
          assert(length >= height_ab, "Parameter length must be greater or equal height_ab.");
          annotation(
            defaultComponentName = "pipe",
            Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, 40}, {100, -40}}, fillPattern = FillPattern.Solid, fillColor = {95, 95, 95}, pattern = LinePattern.None), Rectangle(extent = {{-100, 44}, {100, -44}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {0, 127, 255})}),
            Documentation(info = "<html>
<p>
Base class for one dimensional flow models. It specializes a PartialTwoPort with a parameter interface and icon graphics.
</p>
</html>"));
        end PartialStraightPipe;

        partial model PartialTwoPortFlow "Base class for distributed flow models"
          import Modelica.Fluid.Types.ModelStructure;
          // extending PartialTwoPort
          extends Modelica.Fluid.Interfaces.PartialTwoPort(final port_a_exposesState = modelStructure == ModelStructure.av_b or modelStructure == ModelStructure.av_vb, final port_b_exposesState = modelStructure == ModelStructure.a_vb or modelStructure == ModelStructure.av_vb);
          // distributed volume model
          extends Modelica.Fluid.Interfaces.PartialDistributedVolume(final n = nNodes, final fluidVolumes = {crossAreas[i] * lengths[i] for i in 1:n} * nParallel);
          // Geometry parameters
          parameter Real nParallel(min = 1) = 1 "Number of identical parallel flow devices" annotation(
            Dialog(group = "Geometry"));
          parameter SI.Length[n] lengths "lengths of flow segments" annotation(
            Dialog(group = "Geometry"));
          parameter SI.Area[n] crossAreas "cross flow areas of flow segments" annotation(
            Dialog(group = "Geometry"));
          parameter SI.Length[n] dimensions "hydraulic diameters of flow segments" annotation(
            Dialog(group = "Geometry"));
          parameter SI.Height[n] roughnesses "Average heights of surface asperities" annotation(
            Dialog(group = "Geometry"));
          // Static head
          parameter SI.Length[n] dheights = zeros(n) "Differences in heights of flow segments" annotation(
            Dialog(group = "Static head"),
            Evaluate = true);
          // Assumptions
          parameter Types.Dynamics momentumDynamics = system.momentumDynamics "Formulation of momentum balances" annotation(
            Evaluate = true,
            Dialog(tab = "Assumptions", group = "Dynamics"));
          // Initialization
          parameter Medium.MassFlowRate m_flow_start = system.m_flow_start "Start value for mass flow rate" annotation(
            Evaluate = true,
            Dialog(tab = "Initialization"));
          // Discretization
          parameter Integer nNodes(min = 1) = 2 "Number of discrete flow volumes" annotation(
            Dialog(tab = "Advanced"),
            Evaluate = true);
          parameter Types.ModelStructure modelStructure = Types.ModelStructure.av_vb "Determines whether flow or volume models are present at the ports" annotation(
            Dialog(tab = "Advanced"),
            Evaluate = true);
          parameter Boolean useLumpedPressure = false "=true to lump pressure states together" annotation(
            Dialog(tab = "Advanced"),
            Evaluate = true);
          final parameter Integer nFM = if useLumpedPressure then nFMLumped else nFMDistributed "number of flow models in flowModel";
          final parameter Integer nFMDistributed = if modelStructure == Types.ModelStructure.a_v_b then n + 1 else if modelStructure == Types.ModelStructure.a_vb or modelStructure == Types.ModelStructure.av_b then n else n - 1;
          final parameter Integer nFMLumped = if modelStructure == Types.ModelStructure.a_v_b then 2 else 1;
          final parameter Integer iLumped = integer(n / 2) + 1 "Index of control volume with representative state if useLumpedPressure" annotation(
            Evaluate = true);
          // Advanced model options
          parameter Boolean useInnerPortProperties = false "=true to take port properties for flow models from internal control volumes" annotation(
            Dialog(tab = "Advanced"),
            Evaluate = true);
          Medium.ThermodynamicState state_a "state defined by volume outside port_a";
          Medium.ThermodynamicState state_b "state defined by volume outside port_b";
          Medium.ThermodynamicState[nFM + 1] statesFM "state vector for flowModel model";
          // Pressure loss model
          replaceable model FlowModel = Modelica.Fluid.Pipes.BaseClasses.FlowModels.DetailedPipeFlow constrainedby Modelica.Fluid.Pipes.BaseClasses.FlowModels.PartialStaggeredFlowModel "Wall friction, gravity, momentum flow" annotation(
             Dialog(group = "Pressure loss"),
             choicesAllMatching = true);
          FlowModel flowModel(redeclare final package Medium = Medium, final n = nFM + 1, final states = statesFM, final vs = vsFM, final momentumDynamics = momentumDynamics, final allowFlowReversal = allowFlowReversal, final p_a_start = p_a_start, final p_b_start = p_b_start, final m_flow_start = m_flow_start, final nParallel = nParallel, final pathLengths = pathLengths, final crossAreas = crossAreasFM, final dimensions = dimensionsFM, final roughnesses = roughnessesFM, final dheights = dheightsFM, final g = system.g) "Flow model" annotation(
            Placement(transformation(extent = {{-77, -37}, {75, -19}})));
          // Flow quantities
          Medium.MassFlowRate[n + 1] m_flows(each min = if allowFlowReversal then -Modelica.Constants.inf else 0, each start = m_flow_start) "Mass flow rates of fluid across segment boundaries";
          Medium.MassFlowRate[n + 1, Medium.nXi] mXi_flows "Independent mass flow rates across segment boundaries";
          Medium.MassFlowRate[n + 1, Medium.nC] mC_flows "Trace substance mass flow rates across segment boundaries";
          Medium.EnthalpyFlowRate[n + 1] H_flows "Enthalpy flow rates of fluid across segment boundaries";
          SI.Velocity[n] vs = {0.5 * (m_flows[i] + m_flows[i + 1]) / mediums[i].d / crossAreas[i] for i in 1:n} / nParallel "mean velocities in flow segments";
          // Model structure dependent flow geometry
        protected
          SI.Length[nFM] pathLengths "Lengths along flow path";
          SI.Length[nFM] dheightsFM "Differences in heights between flow segments";
          SI.Area[nFM + 1] crossAreasFM "Cross flow areas of flow segments";
          SI.Velocity[nFM + 1] vsFM "Mean velocities in flow segments";
          SI.Length[nFM + 1] dimensionsFM "Hydraulic diameters of flow segments";
          SI.Height[nFM + 1] roughnessesFM "Average heights of surface asperities";
        equation
          assert(nNodes > 1 or modelStructure <> ModelStructure.av_vb, "nNodes needs to be at least 2 for modelStructure av_vb, as flow model disappears otherwise!");
// staggered grid discretization of geometry for flowModel, depending on modelStructure
          if useLumpedPressure then
            if modelStructure <> ModelStructure.a_v_b then
              pathLengths[1] = sum(lengths);
              dheightsFM[1] = sum(dheights);
              if n == 1 then
                crossAreasFM[1:2] = {crossAreas[1], crossAreas[1]};
                dimensionsFM[1:2] = {dimensions[1], dimensions[1]};
                roughnessesFM[1:2] = {roughnesses[1], roughnesses[1]};
              else
// n > 1
                crossAreasFM[1:2] = {sum(crossAreas[1:iLumped - 1]) / (iLumped - 1), sum(crossAreas[iLumped:n]) / (n - iLumped + 1)};
                dimensionsFM[1:2] = {sum(dimensions[1:iLumped - 1]) / (iLumped - 1), sum(dimensions[iLumped:n]) / (n - iLumped + 1)};
                roughnessesFM[1:2] = {sum(roughnesses[1:iLumped - 1]) / (iLumped - 1), sum(roughnesses[iLumped:n]) / (n - iLumped + 1)};
              end if;
            else
              if n == 1 then
                pathLengths[1:2] = {lengths[1] / 2, lengths[1] / 2};
                dheightsFM[1:2] = {dheights[1] / 2, dheights[1] / 2};
                crossAreasFM[1:3] = {crossAreas[1], crossAreas[1], crossAreas[1]};
                dimensionsFM[1:3] = {dimensions[1], dimensions[1], dimensions[1]};
                roughnessesFM[1:3] = {roughnesses[1], roughnesses[1], roughnesses[1]};
              else
// n > 1
                pathLengths[1:2] = {sum(lengths[1:iLumped - 1]), sum(lengths[iLumped:n])};
                dheightsFM[1:2] = {sum(dheights[1:iLumped - 1]), sum(dheights[iLumped:n])};
                crossAreasFM[1:3] = {sum(crossAreas[1:iLumped - 1]) / (iLumped - 1), sum(crossAreas) / n, sum(crossAreas[iLumped:n]) / (n - iLumped + 1)};
                dimensionsFM[1:3] = {sum(dimensions[1:iLumped - 1]) / (iLumped - 1), sum(dimensions) / n, sum(dimensions[iLumped:n]) / (n - iLumped + 1)};
                roughnessesFM[1:3] = {sum(roughnesses[1:iLumped - 1]) / (iLumped - 1), sum(roughnesses) / n, sum(roughnesses[iLumped:n]) / (n - iLumped + 1)};
              end if;
            end if;
          else
            if modelStructure == ModelStructure.av_vb then
//nFM = n-1
              if n == 2 then
                pathLengths[1] = lengths[1] + lengths[2];
                dheightsFM[1] = dheights[1] + dheights[2];
              else
                pathLengths[1:n - 1] = cat(1, {lengths[1] + 0.5 * lengths[2]}, 0.5 * (lengths[2:n - 2] + lengths[3:n - 1]), {0.5 * lengths[n - 1] + lengths[n]});
                dheightsFM[1:n - 1] = cat(1, {dheights[1] + 0.5 * dheights[2]}, 0.5 * (dheights[2:n - 2] + dheights[3:n - 1]), {0.5 * dheights[n - 1] + dheights[n]});
              end if;
              crossAreasFM[1:n] = crossAreas;
              dimensionsFM[1:n] = dimensions;
              roughnessesFM[1:n] = roughnesses;
            elseif modelStructure == ModelStructure.av_b then
//nFM = n
              pathLengths[1:n] = lengths;
              dheightsFM[1:n] = dheights;
              crossAreasFM[1:n + 1] = cat(1, crossAreas[1:n], {crossAreas[n]});
              dimensionsFM[1:n + 1] = cat(1, dimensions[1:n], {dimensions[n]});
              roughnessesFM[1:n + 1] = cat(1, roughnesses[1:n], {roughnesses[n]});
            elseif modelStructure == ModelStructure.a_vb then
//nFM = n
              pathLengths[1:n] = lengths;
              dheightsFM[1:n] = dheights;
              crossAreasFM[1:n + 1] = cat(1, {crossAreas[1]}, crossAreas[1:n]);
              dimensionsFM[1:n + 1] = cat(1, {dimensions[1]}, dimensions[1:n]);
              roughnessesFM[1:n + 1] = cat(1, {roughnesses[1]}, roughnesses[1:n]);
            elseif modelStructure == ModelStructure.a_v_b then
//nFM = n+1;
              pathLengths[1:n + 1] = cat(1, {0.5 * lengths[1]}, 0.5 * (lengths[1:n - 1] + lengths[2:n]), {0.5 * lengths[n]});
              dheightsFM[1:n + 1] = cat(1, {0.5 * dheights[1]}, 0.5 * (dheights[1:n - 1] + dheights[2:n]), {0.5 * dheights[n]});
              crossAreasFM[1:n + 2] = cat(1, {crossAreas[1]}, crossAreas[1:n], {crossAreas[n]});
              dimensionsFM[1:n + 2] = cat(1, {dimensions[1]}, dimensions[1:n], {dimensions[n]});
              roughnessesFM[1:n + 2] = cat(1, {roughnesses[1]}, roughnesses[1:n], {roughnesses[n]});
            else
              assert(false, "Unknown model structure");
            end if;
          end if;
// Source/sink terms for mass and energy balances
          for i in 1:n loop
            mb_flows[i] = m_flows[i] - m_flows[i + 1];
            mbXi_flows[i, :] = mXi_flows[i, :] - mXi_flows[i + 1, :];
            mbC_flows[i, :] = mC_flows[i, :] - mC_flows[i + 1, :];
            Hb_flows[i] = H_flows[i] - H_flows[i + 1];
          end for;
// Distributed flow quantities, upwind discretization
          for i in 2:n loop
            H_flows[i] = semiLinear(m_flows[i], mediums[i - 1].h, mediums[i].h);
            mXi_flows[i, :] = semiLinear(m_flows[i], mediums[i - 1].Xi, mediums[i].Xi);
            mC_flows[i, :] = semiLinear(m_flows[i], Cs[i - 1, :], Cs[i, :]);
          end for;
          H_flows[1] = semiLinear(port_a.m_flow, inStream(port_a.h_outflow), mediums[1].h);
          H_flows[n + 1] = -semiLinear(port_b.m_flow, inStream(port_b.h_outflow), mediums[n].h);
          mXi_flows[1, :] = semiLinear(port_a.m_flow, inStream(port_a.Xi_outflow), mediums[1].Xi);
          mXi_flows[n + 1, :] = -semiLinear(port_b.m_flow, inStream(port_b.Xi_outflow), mediums[n].Xi);
          mC_flows[1, :] = semiLinear(port_a.m_flow, inStream(port_a.C_outflow), Cs[1, :]);
          mC_flows[n + 1, :] = -semiLinear(port_b.m_flow, inStream(port_b.C_outflow), Cs[n, :]);
// Boundary conditions
          port_a.m_flow = m_flows[1];
          port_b.m_flow = -m_flows[n + 1];
          port_a.h_outflow = mediums[1].h;
          port_b.h_outflow = mediums[n].h;
          port_a.Xi_outflow = mediums[1].Xi;
          port_b.Xi_outflow = mediums[n].Xi;
          port_a.C_outflow = Cs[1, :];
          port_b.C_outflow = Cs[n, :];
// The two equations below are not correct if C is stored in volumes.
// C should be treated the same way as Xi.
//port_a.C_outflow = inStream(port_b.C_outflow);
//port_b.C_outflow = inStream(port_a.C_outflow);
          if useInnerPortProperties and n > 0 then
            state_a = Medium.setState_phX(port_a.p, mediums[1].h, mediums[1].Xi);
            state_b = Medium.setState_phX(port_b.p, mediums[n].h, mediums[n].Xi);
          else
            state_a = Medium.setState_phX(port_a.p, inStream(port_a.h_outflow), inStream(port_a.Xi_outflow));
            state_b = Medium.setState_phX(port_b.p, inStream(port_b.h_outflow), inStream(port_b.Xi_outflow));
          end if;
// staggered grid discretization for flowModel, depending on modelStructure
          if useLumpedPressure then
            if modelStructure <> ModelStructure.av_vb then
// all pressures are equal
              fill(mediums[1].p, n - 1) = mediums[2:n].p;
            elseif n > 2 then
// need two pressures
              fill(mediums[1].p, iLumped - 2) = mediums[2:iLumped - 1].p;
              fill(mediums[n].p, n - iLumped) = mediums[iLumped:n - 1].p;
            end if;
            if modelStructure == ModelStructure.av_vb then
              port_a.p = mediums[1].p;
              statesFM[1] = mediums[1].state;
              m_flows[iLumped] = flowModel.m_flows[1];
              statesFM[2] = mediums[n].state;
              port_b.p = mediums[n].p;
              vsFM[1] = vs[1:iLumped - 1] * lengths[1:iLumped - 1] / sum(lengths[1:iLumped - 1]);
              vsFM[2] = vs[iLumped:n] * lengths[iLumped:n] / sum(lengths[iLumped:n]);
            elseif modelStructure == ModelStructure.av_b then
              port_a.p = mediums[1].p;
              statesFM[1] = mediums[iLumped].state;
              statesFM[2] = state_b;
              m_flows[n + 1] = flowModel.m_flows[1];
              vsFM[1] = vs * lengths / sum(lengths);
              vsFM[2] = m_flows[n + 1] / Medium.density(state_b) / crossAreas[n] / nParallel;
            elseif modelStructure == ModelStructure.a_vb then
              m_flows[1] = flowModel.m_flows[1];
              statesFM[1] = state_a;
              statesFM[2] = mediums[iLumped].state;
              port_b.p = mediums[n].p;
              vsFM[1] = m_flows[1] / Medium.density(state_a) / crossAreas[1] / nParallel;
              vsFM[2] = vs * lengths / sum(lengths);
            elseif modelStructure == ModelStructure.a_v_b then
              m_flows[1] = flowModel.m_flows[1];
              statesFM[1] = state_a;
              statesFM[2] = mediums[iLumped].state;
              statesFM[3] = state_b;
              m_flows[n + 1] = flowModel.m_flows[2];
              vsFM[1] = m_flows[1] / Medium.density(state_a) / crossAreas[1] / nParallel;
              vsFM[2] = vs * lengths / sum(lengths);
              vsFM[3] = m_flows[n + 1] / Medium.density(state_b) / crossAreas[n] / nParallel;
            else
              assert(false, "Unknown model structure");
            end if;
          else
            if modelStructure == ModelStructure.av_vb then
//nFM = n-1
              statesFM[1:n] = mediums[1:n].state;
              m_flows[2:n] = flowModel.m_flows[1:n - 1];
              vsFM[1:n] = vs;
              port_a.p = mediums[1].p;
              port_b.p = mediums[n].p;
            elseif modelStructure == ModelStructure.av_b then
//nFM = n
              statesFM[1:n] = mediums[1:n].state;
              statesFM[n + 1] = state_b;
              m_flows[2:n + 1] = flowModel.m_flows[1:n];
              vsFM[1:n] = vs;
              vsFM[n + 1] = m_flows[n + 1] / Medium.density(state_b) / crossAreas[n] / nParallel;
              port_a.p = mediums[1].p;
            elseif modelStructure == ModelStructure.a_vb then
//nFM = n
              statesFM[1] = state_a;
              statesFM[2:n + 1] = mediums[1:n].state;
              m_flows[1:n] = flowModel.m_flows[1:n];
              vsFM[1] = m_flows[1] / Medium.density(state_a) / crossAreas[1] / nParallel;
              vsFM[2:n + 1] = vs;
              port_b.p = mediums[n].p;
            elseif modelStructure == ModelStructure.a_v_b then
//nFM = n+1
              statesFM[1] = state_a;
              statesFM[2:n + 1] = mediums[1:n].state;
              statesFM[n + 2] = state_b;
              m_flows[1:n + 1] = flowModel.m_flows[1:n + 1];
              vsFM[1] = m_flows[1] / Medium.density(state_a) / crossAreas[1] / nParallel;
              vsFM[2:n + 1] = vs;
              vsFM[n + 2] = m_flows[n + 1] / Medium.density(state_b) / crossAreas[n] / nParallel;
            else
              assert(false, "Unknown model structure");
            end if;
          end if;
          annotation(
            defaultComponentName = "pipe",
            Documentation(info = "<html>
<p>Base class for distributed flow models. The total volume is split into nNodes segments along the flow path.
The default value is nNodes=2.
</p>
<h4>Mass and Energy balances</h4>
<p>
The mass and energy balances are inherited from <a href=\"modelica://Modelica.Fluid.Interfaces.PartialDistributedVolume\">Interfaces.PartialDistributedVolume</a>.
One total mass and one energy balance is formed across each segment according to the finite volume approach.
Substance mass balances are added if the medium contains more than one component.
</p>
<p>
An extending model needs to define the geometry and the difference in heights between the flow segments (static head).
Moreover it needs to define two vectors of source terms for the distributed energy balance:
</p>
<ul>
<li><code><b>Qb_flows[nNodes]</b></code>, the heat flow source terms, e.g., conductive heat flows across segment boundaries, and</li>
<li><code><b>Wb_flows[nNodes]</b></code>, the work source terms.</li>
</ul>

<h4>Momentum balance</h4>
<p>
The momentum balance is determined by the <b><code>FlowModel</code></b> component, which can be replaced with any model extended from
<a href=\"modelica://Modelica.Fluid.Pipes.BaseClasses.FlowModels.PartialStaggeredFlowModel\">BaseClasses.FlowModels.PartialStaggeredFlowModel</a>.
The default setting is <a href=\"modelica://Modelica.Fluid.Pipes.BaseClasses.FlowModels.DetailedPipeFlow\">DetailedPipeFlow</a>.
</p>
<p>
This considers
</p>
<ul>
<li>pressure drop due to friction and other dissipative losses, and</li>
<li>gravity effects for non-horizontal devices.</li>
<li>variation of flow velocity along the flow path,
which occur due to changes in the cross sectional area or the fluid density, provided that <code>flowModel.use_Ib_flows</code> is true.
</ul>

<h4>Model Structure</h4>
<p>
The momentum balances are formulated across the segment boundaries along the flow path according to the staggered grid approach.
The configurable <b><code>modelStructure</code></b> determines the formulation of the boundary conditions at <code>port_a</code> and <code>port_b</code>.
The options include (default: av_vb):
</p>
<ul>
<li><code>av_vb</code>: Symmetric setting with nNodes-1 momentum balances between nNodes flow segments.
    The ports <code>port_a</code> and <code>port_b</code> expose the first and the last thermodynamic state, respectively.
    Connecting two or more flow devices therefore may result in high-index DAEs for the pressures of connected flow segments.
<li><code>a_v_b</code>: Alternative symmetric setting with nNodes+1 momentum balances across nNodes flow segments.
    Half momentum balances are placed between <code>port_a</code> and the first flow segment as well as between the last flow segment and <code>port_b</code>.
    Connecting two or more flow devices therefore results in algebraic pressures at the ports.
    The specification of good start values for the port pressures is essential for the solution of large nonlinear equation systems.</li>
<li><code>av_b</code>: Asymmetric setting with nNodes momentum balances, one between nth volume and <code>port_b</code>, potential pressure state at <code>port_a</code></li>
<li><code>a_vb</code>: Asymmetric setting with nNodes momentum balance, one between first volume and <code>port_a</code>, potential pressure state at <code>port_b</code></li>
</ul>
<p>
When connecting two components, e.g., two pipes, the momentum balance across the connection point reduces to
</p>
<pre>pipe1.port_b.p = pipe2.port_a.p</pre>
<p>
This is only true if the flow velocity remains the same on each side of the connection.
Consider using a fitting for any significant change in diameter or fluid density, if the resulting effects,
such as change in kinetic energy, cannot be neglected.
This also allows for taking into account friction losses with respect to the actual geometry of the connection point.
</p>
</html>", revisions = "<html>
<ul>
<li><i>5 Dec 2008</i>
    by Michael Wetter:<br>
       Modified mass balance for trace substances. With the new formulation, the trace substances masses <code>mC</code> are stored
       in the same way as the species <code>mXi</code>.</li>
<li><i>Dec 2008</i>
    by R&uuml;diger Franke:<br>
       Derived model from original DistributedPipe models
    <ul>
    <li>moved mass and energy balances to PartialDistributedVolume</li>
    <li>introduced replaceable pressure loss models</li>
    <li>combined all model structures and lumped pressure into one model</li>
    <li>new ModelStructure av_vb, replacing former avb</li>
    </ul></li>
<li><i>04 Mar 2006</i>
    by Katrin Pr&ouml;l&szlig;:<br>
       Model added to the Fluid library</li>
</ul>
</html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-72, 10}, {-52, -10}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Ellipse(extent = {{50, 10}, {70, -10}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid)}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{-100, -50}, {-100, 50}, {100, 60}, {100, -60}, {-100, -50}}, fillColor = {215, 215, 215}, fillPattern = FillPattern.Solid, pattern = LinePattern.None, lineColor = {0, 0, 0}), Polygon(points = {{-34, -53}, {-34, 53}, {34, 57}, {34, -57}, {-34, -53}}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, pattern = LinePattern.None, lineColor = {0, 0, 0}), Line(points = {{-100, -50}, {-100, 50}}, arrow = {Arrow.Filled, Arrow.Filled}, pattern = LinePattern.Dot), Text(extent = {{-99, 36}, {-69, 30}}, lineColor = {0, 0, 255}, textString = "crossAreas[1]"), Line(points = {{-100, 70}, {-34, 70}}, arrow = {Arrow.Filled, Arrow.Filled}, pattern = LinePattern.Dot), Text(extent = {{0, 36}, {40, 30}}, lineColor = {0, 0, 255}, textString = "crossAreas[2:n-1]"), Line(points = {{100, -60}, {100, 60}}, arrow = {Arrow.Filled, Arrow.Filled}, pattern = LinePattern.Dot), Text(extent = {{100.5, 36}, {130.5, 30}}, lineColor = {0, 0, 255}, textString = "crossAreas[n]"), Line(points = {{-34, 52}, {-34, -53}}, pattern = LinePattern.Dash), Line(points = {{34, 57}, {34, -57}}, pattern = LinePattern.Dash), Line(points = {{34, 70}, {100, 70}}, arrow = {Arrow.Filled, Arrow.Filled}, pattern = LinePattern.Dot), Line(points = {{-34, 70}, {34, 70}}, arrow = {Arrow.Filled, Arrow.Filled}, pattern = LinePattern.Dot), Text(extent = {{-30, 77}, {30, 71}}, lineColor = {0, 0, 255}, textString = "lengths[2:n-1]"), Line(points = {{-100, -70}, {0, -70}}, arrow = {Arrow.None, Arrow.Filled}), Text(extent = {{-80, -63}, {-20, -69}}, lineColor = {0, 0, 255}, textString = "flowModel.dps_fg[1]"), Line(points = {{0, -70}, {100, -70}}, arrow = {Arrow.None, Arrow.Filled}), Text(extent = {{20.5, -63}, {80, -69}}, lineColor = {0, 0, 255}, textString = "flowModel.dps_fg[2:n-1]"), Line(points = {{-95, 0}, {-5, 0}}, arrow = {Arrow.None, Arrow.Filled}), Text(extent = {{-62, 7}, {-32, 1}}, lineColor = {0, 0, 255}, textString = "m_flows[2]"), Line(points = {{5, 0}, {95, 0}}, arrow = {Arrow.None, Arrow.Filled}), Text(extent = {{34, 7}, {64, 1}}, lineColor = {0, 0, 255}, textString = "m_flows[3:n]"), Line(points = {{-150, 0}, {-105, 0}}, arrow = {Arrow.None, Arrow.Filled}), Line(points = {{105, 0}, {150, 0}}, arrow = {Arrow.None, Arrow.Filled}), Text(extent = {{-140, 7}, {-110, 1}}, lineColor = {0, 0, 255}, textString = "m_flows[1]"), Text(extent = {{111, 7}, {141, 1}}, lineColor = {0, 0, 255}, textString = "m_flows[n+1]"), Text(extent = {{35, -92}, {100, -98}}, lineColor = {0, 0, 255}, textString = "(ModelStructure av_vb, n=3)"), Line(points = {{-100, -50}, {-100, -86}}, pattern = LinePattern.Dot), Line(points = {{0, -55}, {0, -86}}, pattern = LinePattern.Dot), Line(points = {{100, -60}, {100, -86}}, pattern = LinePattern.Dot), Ellipse(extent = {{-5, 5}, {5, -5}}, pattern = LinePattern.None, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Text(extent = {{3, -4}, {33, -10}}, lineColor = {0, 0, 255}, textString = "states[2:n-1]"), Ellipse(extent = {{95, 5}, {105, -5}}, pattern = LinePattern.None, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Text(extent = {{104, -4}, {124, -10}}, lineColor = {0, 0, 255}, textString = "states[n]"), Ellipse(extent = {{-105, 5}, {-95, -5}}, pattern = LinePattern.None, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Text(extent = {{-96, -4}, {-76, -10}}, lineColor = {0, 0, 255}, textString = "states[1]"), Text(extent = {{-99.5, 30}, {-69.5, 24}}, lineColor = {0, 0, 255}, textString = "dimensions[1]"), Text(extent = {{-0.5, 30}, {40, 24}}, lineColor = {0, 0, 255}, textString = "dimensions[2:n-1]"), Text(extent = {{100.5, 30}, {130.5, 24}}, lineColor = {0, 0, 255}, textString = "dimensions[n]"), Line(points = {{-34, 73}, {-34, 52}}, pattern = LinePattern.Dot), Line(points = {{34, 73}, {34, 57}}, pattern = LinePattern.Dot), Line(points = {{-100, 50}, {100, 60}}, thickness = 0.5), Line(points = {{-100, -50}, {100, -60}}, thickness = 0.5), Line(points = {{-100, 73}, {-100, 50}}, pattern = LinePattern.Dot), Line(points = {{100, 73}, {100, 60}}, pattern = LinePattern.Dot), Line(points = {{0, -55}, {0, 55}}, arrow = {Arrow.Filled, Arrow.Filled}, pattern = LinePattern.Dot), Line(points = {{-34, 11}, {34, 11}}, arrow = {Arrow.None, Arrow.Filled}), Text(extent = {{5, 18}, {25, 12}}, lineColor = {0, 0, 255}, textString = "vs[2:n-1]"), Text(extent = {{-72, 18}, {-62, 12}}, lineColor = {0, 0, 255}, textString = "vs[1]"), Line(points = {{-100, 11}, {-34, 11}}, arrow = {Arrow.None, Arrow.Filled}), Text(extent = {{63, 18}, {73, 12}}, lineColor = {0, 0, 255}, textString = "vs[n]"), Line(points = {{34, 11}, {100, 11}}, arrow = {Arrow.None, Arrow.Filled}), Text(extent = {{-80, -75}, {-20, -81}}, lineColor = {0, 0, 255}, textString = "flowModel.pathLengths[1]"), Line(points = {{-100, -82}, {0, -82}}, arrow = {Arrow.Filled, Arrow.Filled}), Line(points = {{0, -82}, {100, -82}}, arrow = {Arrow.Filled, Arrow.Filled}), Text(extent = {{15, -75}, {85, -81}}, lineColor = {0, 0, 255}, textString = "flowModel.pathLengths[2:n-1]"), Text(extent = {{-100, 77}, {-37, 71}}, lineColor = {0, 0, 255}, textString = "lengths[1]"), Text(extent = {{34, 77}, {100, 71}}, lineColor = {0, 0, 255}, textString = "lengths[n]")}));
        end PartialTwoPortFlow;

        package FlowModels "Flow models for pipes, including wall friction, static head and momentum flow"
          extends Modelica.Icons.Package;

          partial model PartialStaggeredFlowModel "Base class for momentum balances in flow models"
            //
            // Internal interface
            // (not exposed to GUI; needs to be hard coded when using this model
            //
            replaceable package Medium = Modelica.Media.Interfaces.PartialMedium "Medium in the component" annotation(
              Dialog(tab = "Internal Interface", enable = false));
            parameter Integer n = 2 "Number of discrete flow volumes" annotation(
              Dialog(tab = "Internal Interface", enable = false));
            // Inputs
            input Medium.ThermodynamicState[n] states "Thermodynamic states along design flow";
            input Modelica.SIunits.Velocity[n] vs "Mean velocities of fluid flow";
            // Geometry parameters and inputs
            parameter Real nParallel "number of identical parallel flow devices" annotation(
              Dialog(tab = "Internal Interface", enable = false, group = "Geometry"));
            input SI.Area[n] crossAreas "Cross flow areas at segment boundaries";
            input SI.Length[n] dimensions "Characteristic dimensions for fluid flow (diameters for pipe flow)";
            input SI.Height[n] roughnesses "Average height of surface asperities";
            // Static head
            input SI.Length[n - 1] dheights "Height(states[2:n]) - Height(states[1:n-1])";
            parameter SI.Acceleration g = system.g "Constant gravity acceleration" annotation(
              Dialog(tab = "Internal Interface", enable = false, group = "Static head"));
            // Assumptions
            parameter Boolean allowFlowReversal = system.allowFlowReversal "= true to allow flow reversal, false restricts to design direction (states[1] -> states[n+1])" annotation(
              Dialog(tab = "Internal Interface", enable = false, group = "Assumptions"),
              Evaluate = true);
            parameter Modelica.Fluid.Types.Dynamics momentumDynamics = system.momentumDynamics "Formulation of momentum balance" annotation(
              Dialog(tab = "Internal Interface", enable = false, group = "Assumptions"),
              Evaluate = true);
            // Initialization
            parameter Medium.MassFlowRate m_flow_start = system.m_flow_start "Start value of mass flow rates" annotation(
              Dialog(tab = "Internal Interface", enable = false, group = "Initialization"));
            parameter Medium.AbsolutePressure p_a_start "Start value for p[1] at design inflow" annotation(
              Dialog(tab = "Internal Interface", enable = false, group = "Initialization"));
            parameter Medium.AbsolutePressure p_b_start "Start value for p[n+1] at design outflow" annotation(
              Dialog(tab = "Internal Interface", enable = false, group = "Initialization"));
            //
            // Implementation of momentum balance
            //
            extends Modelica.Fluid.Interfaces.PartialDistributedFlow(final m = n - 1);
            // Advanced parameters
            parameter Boolean useUpstreamScheme = true "= false to average upstream and downstream properties across flow segments" annotation(
              Dialog(group = "Advanced"),
              Evaluate = true);
            parameter Boolean use_Ib_flows = momentumDynamics <> Types.Dynamics.SteadyState "= true to consider differences in flow of momentum through boundaries" annotation(
              Dialog(group = "Advanced"),
              Evaluate = true);
            // Variables
            Medium.Density[n] rhos = if use_rho_nominal then fill(rho_nominal, n) else Medium.density(states);
            Medium.Density[n - 1] rhos_act "Actual density per segment";
            Medium.DynamicViscosity[n] mus = if use_mu_nominal then fill(mu_nominal, n) else Medium.dynamicViscosity(states);
            Medium.DynamicViscosity[n - 1] mus_act "Actual viscosity per segment";
            // Variables
            Modelica.SIunits.Pressure[n - 1] dps_fg(each start = (p_a_start - p_b_start) / (n - 1)) "pressure drop between states";
            // Reynolds Number
            parameter SI.ReynoldsNumber Re_turbulent = 4000 "Start of turbulent regime, depending on type of flow device";
            parameter Boolean show_Res = false "= true, if Reynolds numbers are included for plotting" annotation(
              Evaluate = true,
              Dialog(group = "Diagnostics"));
            SI.ReynoldsNumber[n] Res = Modelica.Fluid.Pipes.BaseClasses.CharacteristicNumbers.ReynoldsNumber(vs, rhos, mus, dimensions) if show_Res "Reynolds numbers";
            Medium.MassFlowRate[n - 1] m_flows_turbulent = {nParallel * (crossAreas[i] + crossAreas[i + 1]) / (dimensions[i] + dimensions[i + 1]) * mus_act[i] * Re_turbulent for i in 1:n - 1} if show_Res "Start of turbulent flow";
          protected
            parameter Boolean use_rho_nominal = false "= true, if rho_nominal is used, otherwise computed from medium" annotation(
              Dialog(group = "Advanced"),
              Evaluate = true);
            parameter SI.Density rho_nominal = Medium.density_pTX(Medium.p_default, Medium.T_default, Medium.X_default) "Nominal density (e.g., rho_liquidWater = 995, rho_air = 1.2)" annotation(
              Dialog(group = "Advanced", enable = use_rho_nominal));
            parameter Boolean use_mu_nominal = false "= true, if mu_nominal is used, otherwise computed from medium" annotation(
              Dialog(group = "Advanced"),
              Evaluate = true);
            parameter SI.DynamicViscosity mu_nominal = Medium.dynamicViscosity(Medium.setState_pTX(Medium.p_default, Medium.T_default, Medium.X_default)) "Nominal dynamic viscosity (e.g., mu_liquidWater = 1e-3, mu_air = 1.8e-5)" annotation(
              Dialog(group = "Advanced", enable = use_mu_nominal));
          equation
            if not allowFlowReversal then
              rhos_act = rhos[1:n - 1];
              mus_act = mus[1:n - 1];
            elseif not useUpstreamScheme then
              rhos_act = 0.5 * (rhos[1:n - 1] + rhos[2:n]);
              mus_act = 0.5 * (mus[1:n - 1] + mus[2:n]);
            else
              for i in 1:n - 1 loop
                rhos_act[i] = noEvent(if m_flows[i] > 0 then rhos[i] else rhos[i + 1]);
                mus_act[i] = noEvent(if m_flows[i] > 0 then mus[i] else mus[i + 1]);
              end for;
            end if;
            if use_Ib_flows then
              Ib_flows = nParallel * {rhos[i] * vs[i] * vs[i] * crossAreas[i] - rhos[i + 1] * vs[i + 1] * vs[i + 1] * crossAreas[i + 1] for i in 1:n - 1};
// alternatively use densities rhos_act of actual streams, together with mass flow rates,
// not conserving momentum if fluid density changes between flow segments:
//Ib_flows = {((rhos[i]*vs[i])^2*crossAreas[i] - (rhos[i+1]*vs[i+1])^2*crossAreas[i+1])/rhos_act[i] for i in 1:n-1};
            else
              Ib_flows = zeros(n - 1);
            end if;
            Fs_p = nParallel * {0.5 * (crossAreas[i] + crossAreas[i + 1]) * (Medium.pressure(states[i + 1]) - Medium.pressure(states[i])) for i in 1:n - 1};
// Note: the equation is written for dps_fg instead of Fs_fg to help the translator
            dps_fg = {Fs_fg[i] / nParallel * 2 / (crossAreas[i] + crossAreas[i + 1]) for i in 1:n - 1};
            annotation(
              Documentation(info = "<html>
<p>
This partial model defines a common interface for <code>m=n-1</code> flow models between <code>n</code> device segments.
The flow models provide a steady-state or dynamic momentum balance using an upwind discretization scheme per default.
Extending models must add pressure loss terms for friction and gravity.
</p>
<p>
The fluid is specified in the interface with the thermodynamic <code>states[n]</code> for a given <code>Medium</code> model.
The geometry is specified with the <code>pathLengths[n-1]</code> between the device segments as well as
with the <code>crossAreas[n]</code> and the <code>roughnesses[n]</code> of the device segments.
Moreover the fluid flow is characterized for different types of devices by the characteristic <code>dimensions[n]</code>
and the average velocities <code>vs[n]</code> of fluid flow in the device segments.
See <a href=\"modelica://Modelica.Fluid.Pipes.BaseClasses.CharacteristicNumbers.ReynoldsNumber\">Pipes.BaseClasses.CharacteristicNumbers.ReynoldsNumber</a>
for example definitions.
</p>
<p>
The parameter <code>Re_turbulent</code> can be specified for the least mass flow rate of the turbulent regime.
It defaults to 4000, which is appropriate for pipe flow.
The <code>m_flows_turbulent[n-1]</code> resulting from <code>Re_turbulent</code> can optionally be calculated together with the Reynolds numbers
<code>Res[n]</code> of the device segments (<code>show_Res=true</code>).
</p>
<p>
Using the thermodynamic states[n] of the device segments, the densities rhos[n] and the dynamic viscosities mus[n]
of the segments as well as the actual densities rhos_act[n-1] and the actual viscosities mus_act[n-1] of the flows are predefined
in this base model. Note that no events are raised on flow reversal. This needs to be treated by an extending model,
e.g., with numerical smoothing or by raising events as appropriate.
</p>
</html>"),
              Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-80, -50}, {-80, 50}, {80, -50}, {80, 50}}, color = {0, 0, 255}, thickness = 1), Text(extent = {{-40, -50}, {40, -90}}, lineColor = {0, 0, 0}, textString = "%name")}));
          end PartialStaggeredFlowModel;

          model NominalLaminarFlow "NominalLaminarFlow: Linear laminar flow for given nominal values"
            extends Modelica.Fluid.Pipes.BaseClasses.FlowModels.PartialStaggeredFlowModel(use_mu_nominal = not show_Res);
            // Operational conditions
            parameter SI.AbsolutePressure dp_nominal "Nominal pressure loss";
            parameter SI.MassFlowRate m_flow_nominal "Mass flow rate for dp_nominal";
            // Inverse parameterization assuming pipe flow and WallFriction.Laminar
            // Laminar.massFlowRate_dp:
            //   m_flow = dp*pi*diameter^4*d/(128*length*mu);
            SI.Length[n - 1] pathLengths_nominal = {(dp_nominal / (n - 1) - g * dheights[i]) * Modelica.Constants.pi * ((dimensions[i] + dimensions[i + 1]) / 2) ^ 4 * rhos_act[i] / (128 * mus_act[i]) / (m_flow_nominal / nParallel) for i in 1:n - 1} if show_Res "Lengths resulting from given nominal values for circular tubes";
          equation
// linear pressure loss
            if not allowFlowReversal or use_rho_nominal or not useUpstreamScheme then
              dps_fg = {g * dheights[i] * rhos_act[i] for i in 1:n - 1} + dp_nominal / (n - 1) / m_flow_nominal * m_flows;
            else
              dps_fg = {g * dheights[i] * (if m_flows[i] > 0 then rhos[i] else rhos[i + 1]) for i in 1:n - 1} + dp_nominal / (n - 1) / m_flow_nominal * m_flows;
            end if;
            annotation(
              Documentation(info = "<html>
<p>
This model defines a simple linear pressure loss assuming laminar flow for
specified <code>dp_nominal</code> and <code>m_flow_nominal</code>.
</p>
<p>
Select <code>show_Res = true</code> to analyze the actual flow and the lengths of a pipe that would fulfill the
specified nominal values for given geometry parameters <code>crossAreas</code>, <code>dimensions</code> and <code>roughnesses</code>.
</p>
</html>"));
          end NominalLaminarFlow;

          partial model PartialGenericPipeFlow "GenericPipeFlow: Pipe flow pressure loss and gravity with replaceable WallFriction package"
            parameter Boolean from_dp = momentumDynamics >= Types.Dynamics.SteadyStateInitial "= true, use m_flow = f(dp), otherwise dp = f(m_flow)" annotation(
              Dialog(group = "Advanced"),
              Evaluate = true);
            extends Modelica.Fluid.Pipes.BaseClasses.FlowModels.PartialStaggeredFlowModel(final Re_turbulent = 4000);
            replaceable package WallFriction = Modelica.Fluid.Pipes.BaseClasses.WallFriction.Detailed constrainedby Modelica.Fluid.Pipes.BaseClasses.WallFriction.PartialWallFriction "Wall friction model" annotation(
               Dialog(group = "Wall friction"),
               choicesAllMatching = true);
            input SI.Length[n - 1] pathLengths_internal "pathLengths used internally; to be defined by extending class";
            input SI.ReynoldsNumber[n - 1] Res_turbulent_internal = Re_turbulent * ones(n - 1) "Re_turbulent used internally; to be defined by extending class";
            // Parameters
            parameter SI.AbsolutePressure dp_nominal "Nominal pressure loss (only for nominal models)";
            parameter SI.MassFlowRate m_flow_nominal "Nominal mass flow rate";
            parameter SI.MassFlowRate m_flow_small = if system.use_eps_Re then system.eps_m_flow * m_flow_nominal else system.m_flow_small "Within regularization if |m_flows| < m_flow_small (may be wider for large discontinuities in static head)" annotation(
              Dialog(enable = not from_dp and WallFriction.use_m_flow_small));
          protected
            parameter SI.AbsolutePressure dp_small(start = 1, fixed = false) "Within regularization if |dp| < dp_small (may be wider for large discontinuities in static head)" annotation(
              Dialog(enable = from_dp and WallFriction.use_dp_small));
            final parameter Boolean constantPressureLossCoefficient = use_rho_nominal and (use_mu_nominal or not WallFriction.use_mu) "= true if the pressure loss does not depend on fluid states" annotation(
              Evaluate = true);
            final parameter Boolean continuousFlowReversal = not useUpstreamScheme or constantPressureLossCoefficient or not allowFlowReversal "= true if the pressure loss is continuous around zero flow" annotation(
              Evaluate = true);
            SI.Length[n - 1] diameters = 0.5 * (dimensions[1:n - 1] + dimensions[2:n]) "mean diameters between segments";
            SI.AbsolutePressure dp_fric_nominal = sum(WallFriction.pressureLoss_m_flow(m_flow_nominal / nParallel, rho_nominal, rho_nominal, mu_nominal, mu_nominal, pathLengths_internal, diameters, (crossAreas[1:n - 1] + crossAreas[2:n]) / 2, (roughnesses[1:n - 1] + roughnesses[2:n]) / 2, m_flow_small / nParallel, Res_turbulent_internal)) "pressure loss for nominal conditions";
          initial equation
// initialize dp_small from flow model
            if system.use_eps_Re then
              dp_small = dp_fric_nominal / m_flow_nominal * m_flow_small;
            else
              dp_small = system.dp_small;
            end if;
          equation
            for i in 1:n - 1 loop
              assert(m_flows[i] > (-m_flow_small) or allowFlowReversal, "Reverting flow occurs even though allowFlowReversal is false");
            end for;
            if continuousFlowReversal then
// simple regularization
              if from_dp and not WallFriction.dp_is_zero then
                m_flows = homotopy(actual = WallFriction.massFlowRate_dp(dps_fg - {g * dheights[i] * rhos_act[i] for i in 1:n - 1}, rhos_act, rhos_act, mus_act, mus_act, pathLengths_internal, diameters, (crossAreas[1:n - 1] + crossAreas[2:n]) / 2, (roughnesses[1:n - 1] + roughnesses[2:n]) / 2, dp_small / (n - 1), Res_turbulent_internal) * nParallel, simplified = m_flow_nominal / dp_nominal * (dps_fg - g * dheights * rho_nominal));
              else
                dps_fg = homotopy(actual = WallFriction.pressureLoss_m_flow(m_flows / nParallel, rhos_act, rhos_act, mus_act, mus_act, pathLengths_internal, diameters, (crossAreas[1:n - 1] + crossAreas[2:n]) / 2, (roughnesses[1:n - 1] + roughnesses[2:n]) / 2, m_flow_small / nParallel, Res_turbulent_internal) + {g * dheights[i] * rhos_act[i] for i in 1:n - 1}, simplified = dp_nominal / m_flow_nominal * m_flows + g * dheights * rho_nominal);
              end if;
            else
// regularization for discontinuous flow reversal and static head
              if from_dp and not WallFriction.dp_is_zero then
                m_flows = homotopy(actual = WallFriction.massFlowRate_dp_staticHead(dps_fg, rhos[1:n - 1], rhos[2:n], mus[1:n - 1], mus[2:n], pathLengths_internal, diameters, g * dheights, (crossAreas[1:n - 1] + crossAreas[2:n]) / 2, (roughnesses[1:n - 1] + roughnesses[2:n]) / 2, dp_small / (n - 1), Res_turbulent_internal) * nParallel, simplified = m_flow_nominal / dp_nominal * (dps_fg - g * dheights * rho_nominal));
              else
                dps_fg = homotopy(actual = WallFriction.pressureLoss_m_flow_staticHead(m_flows / nParallel, rhos[1:n - 1], rhos[2:n], mus[1:n - 1], mus[2:n], pathLengths_internal, diameters, g * dheights, (crossAreas[1:n - 1] + crossAreas[2:n]) / 2, (roughnesses[1:n - 1] + roughnesses[2:n]) / 2, m_flow_small / nParallel, Res_turbulent_internal), simplified = dp_nominal / m_flow_nominal * m_flows + g * dheights * rho_nominal);
              end if;
            end if;
            annotation(
              Documentation(info = "<html>
<p>
This model describes pressure losses due to <b>wall friction</b> in a pipe
and due to <b>gravity</b>.
Correlations of different complexity and validity can be
selected via the replaceable package <b>WallFriction</b> (see parameter menu below).
The details of the pipe wall friction model are described in the
<a href=\"modelica://Modelica.Fluid.UsersGuide.ComponentDefinition.WallFriction\">UsersGuide</a>.
Basically, different variants of the equation
</p>

<pre>
   dp = &lambda;(Re,<font face=\"Symbol\">D</font>)*(L/D)*&rho;*v*|v|/2.
</pre>

<p>

By default, the correlations are computed with media data at the actual time instant.
In order to reduce non-linear equation systems, the parameters
<b>use_mu_nominal</b> and <b>use_rho_nominal</b> provide the option
to compute the correlations with constant media values
at the desired operating point. This might speed-up the
simulation and/or might give a more robust simulation.
</p>
</html>"),
              Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, 64}, {100, -64}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Backward), Rectangle(extent = {{-100, 50}, {100, -49}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{-60, -49}, {-60, 50}}, color = {0, 0, 255}, arrow = {Arrow.Filled, Arrow.Filled}), Text(extent = {{-50, 16}, {6, -10}}, lineColor = {0, 0, 255}, textString = "diameters"), Line(points = {{-100, 74}, {100, 74}}, color = {0, 0, 255}, arrow = {Arrow.Filled, Arrow.Filled}), Text(extent = {{-32, 93}, {32, 74}}, lineColor = {0, 0, 255}, textString = "pathLengths")}));
          end PartialGenericPipeFlow;

          model NominalTurbulentPipeFlow "NominalTurbulentPipeFlow: Quadratic turbulent flow in circular tubes for given nominal values"
            extends Modelica.Fluid.Pipes.BaseClasses.FlowModels.PartialGenericPipeFlow(redeclare package WallFriction = Modelica.Fluid.Pipes.BaseClasses.WallFriction.LaminarAndQuadraticTurbulent, use_mu_nominal = not show_Res, pathLengths_internal = pathLengths_nominal, useUpstreamScheme = false, Res_turbulent_internal = Res_turbulent_nominal);
            import Modelica.Constants.pi;
            parameter SI.MassFlowRate m_flow_turbulent(min = 0) = if system.use_eps_Re then 0.1 * m_flow_nominal else system.m_flow_small "Turbulent flow starting from |m_flows| > m_flow_turbulent (may be wider for large discontinuities in static head)" annotation(
              Dialog(enable = not from_dp and WallFriction.use_m_flow_small));
            // variables for nominal pressure loss
            SI.Length[n - 1] pathLengths_nominal "pathLengths resulting from nominal pressure loss and geometry";
            SI.ReynoldsNumber[n - 1] Res_turbulent_nominal "Re_turbulent resulting from nominal turbulent flow and geometry";
            Real[n - 1] ks_inv "coefficient for quadratic flow";
            Real[n - 1] zetas "coefficient for quadratic flow";
            // Reynolds Number
            Medium.AbsolutePressure[n - 1] dps_fg_turbulent(each min = 0) = {(mus_act[i] * diameters[i] * pi / 4) ^ 2 * Re_turbulent ^ 2 / (ks_inv[i] * rhos_act[i]) for i in 1:n - 1} if show_Res "Start of turbulent flow in circular tubes";
          initial equation
            for i in 1:n loop
              assert(abs(crossAreas[i] - pi / 4 * dimensions[i] ^ 2) < 1e-10 * crossAreas[i], "NominalTurbulentPipeFlow model requires circular tubes");
            end for;
          equation
// Inverse parameterization for WallFriction.QuadraticTurbulent
// Note: the code should be shared with the WallFriction.QuadraticTurbulent model,
//       but this required a re-design of the WallFriction interfaces ...
//   zeta = (length_nominal/diameter)/(2*Math.log10(3.7 /(roughness/diameter)))^2;
//   k_inv = (pi*diameter*diameter)^2/(8*zeta);
//   k = rho*k_inv "Factor in m_flow = sqrt(k*dp)";
//   m_flow_turbulent = pi/4*diameter*mu*Re_turbulent;
            for i in 1:n - 1 loop
              ks_inv[i] = (m_flow_nominal / nParallel) ^ 2 / (dp_nominal / (n - 1) - g * dheights[i] * rhos_act[i]) / rhos_act[i];
              zetas[i] = (pi * diameters[i] * diameters[i]) ^ 2 / (8 * ks_inv[i]);
              pathLengths_nominal[i] = zetas[i] * diameters[i] * (2 * Modelica.Math.log10(3.7 / ((roughnesses[i] + roughnesses[i + 1]) / 2 / diameters[i]))) ^ 2;
              Res_turbulent_nominal[i] = m_flow_turbulent / nParallel / (pi / 4 * diameters[i] * mus_act[i]);
            end for;
            annotation(
              Documentation(info = "<html>
<p>
This model defines the pressure loss assuming turbulent flow for
specified <code>dp_nominal</code> and <code>m_flow_nominal</code>.
It takes into account the fluid density of each flow segment and
obtains appropriate <code>pathLengths_nominal</code> values
for an inverse parameterization of the
<a href=\"modelica://Modelica.Fluid.Pipes.BaseClasses.FlowModels.TurbulentPipeFlow\">
          TurbulentPipeFlow</a>
model. Per default the upstream and downstream densities are averaged with the setting <code>useUpstreamScheme = false</code>,
in order to avoid discontinuous <code>pathLengths_nominal</code> values in the case of flow reversal.
</p>
<p>
The geometry parameters <code>crossAreas</code>, <code>diameters</code> and <code>roughnesses</code> do
not effect simulation results of this nominal pressure loss model.
As the geometry is specified however, the optionally calculated Reynolds number as well as
<code>m_flows_turbulent</code> and <code>dps_fg_turbulent</code> become meaningful
and can be related to <code>m_flow_small</code> and <code>dp_small</code>.
</p>
<p>
<b>Optional Variables if show_Res</b>
</p>
<table border=1 cellspacing=0 cellpadding=2>
<tr><th><b>Type</b></th><th><b>Name</b></th><th><b>Description</b></th></tr>
<tr><td>ReynoldsNumber</td><td>Res[n]</td>
    <td>Reynolds numbers of pipe flow per flow segment</td></tr>
<tr><td>MassFlowRate</td><td>m_flows_turbulent[n-1]</td>
    <td>mass flow rates at start of turbulent region for Re_turbulent=4000</td></tr>
<tr><td>AbsolutePressure</td><td>dps_fg_turbulent[n-1]</td>
    <td>pressure losses due to friction and gravity corresponding to m_flows_turbulent</td></tr>
</table>
</html>", revisions = "<html>
<ul>
<li><i>6 Dec 2008</i>
    by Ruediger Franke:<br />
       Model added to the Fluid library</li>
</ul>
</html>"));
          end NominalTurbulentPipeFlow;

          model TurbulentPipeFlow "TurbulentPipeFlow: Quadratic turbulent flow in circular tubes (using mu to regularize laminar region)"
            extends Modelica.Fluid.Pipes.BaseClasses.FlowModels.PartialGenericPipeFlow(redeclare package WallFriction = Modelica.Fluid.Pipes.BaseClasses.WallFriction.LaminarAndQuadraticTurbulent, use_mu_nominal = not show_Res, pathLengths_internal = pathLengths, dp_nominal(start = if system.use_eps_Re then 1 else 1e3 * dp_small, fixed = not system.use_eps_Re), m_flow_nominal = if system.use_eps_Re then system.m_flow_nominal else 1e2 * m_flow_small, Res_turbulent_internal = if use_Re then Re_turbulent * ones(n - 1) else zeros(n - 1));
            import Modelica.Constants.pi;
            parameter Boolean use_Re = system.use_eps_Re "= true, if turbulent region is defined by Re, otherwise by m_flow_small" annotation(
              Evaluate = true);
          initial equation
            for i in 1:n loop
              assert(abs(crossAreas[i] - pi / 4 * dimensions[i] ^ 2) < 1e-10 * crossAreas[i], "NominalTurbulentPipeFlow model requires circular tubes");
            end for;
// initialize dp_nominal from flow model
            if system.use_eps_Re then
              dp_nominal = dp_fric_nominal + g * sum(dheights) * rho_nominal;
            end if;
            annotation(
              Documentation(info = "<html>
<p>
This model defines only the quadratic turbulent regime of wall friction:
dp = k*m_flow*|m_flow|, where \"k\" depends on density and the roughness
of the pipe and is not a function of the Reynolds number.
This relationship is only valid for large Reynolds numbers.
The turbulent pressure loss correlation might be useful to optimize models that are only facing turbulent flow.
</p>

</html>"));
          end TurbulentPipeFlow;

          model DetailedPipeFlow "DetailedPipeFlow: Detailed characteristic for laminar and turbulent flow"
            extends Modelica.Fluid.Pipes.BaseClasses.FlowModels.PartialGenericPipeFlow(redeclare package WallFriction = Modelica.Fluid.Pipes.BaseClasses.WallFriction.Detailed, pathLengths_internal = pathLengths, dp_nominal(start = 1, fixed = false), m_flow_nominal = if system.use_eps_Re then system.m_flow_nominal else 1e2 * m_flow_small, Res_turbulent_internal = Re_turbulent * ones(n - 1));
          initial equation
// initialize dp_nominal from flow model
            if system.use_eps_Re then
              dp_nominal = dp_fric_nominal + g * sum(dheights) * rho_nominal;
            else
              dp_nominal = 1e3 * dp_small;
            end if;
            annotation(
              Documentation(info = "<html>
<p>
This component defines the complete regime of wall friction.
The details are described in the
<a href=\"modelica://Modelica.Fluid.UsersGuide.ComponentDefinition.WallFriction\">UsersGuide</a>.
The functional relationship of the friction loss factor &lambda; is
displayed in the next figure. Function massFlowRate_dp() defines the \"red curve\"
(\"Swamee and Jain\"), where as function pressureLoss_m_flow() defines the
\"blue curve\" (\"Colebrook-White\"). The two functions are inverses from
each other and give slightly different results in the transition region
between Re = 1500 .. 4000, in order to get explicit equations without
solving a non-linear equation.
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Fluid/Components/PipeFriction1.png\"
     alt=\"PipeFriction1.png\">
</p>

<p>
Additionally to wall friction, this component properly implements static
head. With respect to the latter, two cases can be distinguished. In the case
shown next, the change of elevation with the path from a to b has the opposite
sign of the change of density.</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Fluid/Components/PipeFrictionStaticHead_case-a.png\"
     alt=\"PipeFrictionStaticHead_case-a.png\">
</p>

<p>
In the case illustrated second, the change of elevation with the path from a to
b has the same sign of the change of density.</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Fluid/Components/PipeFrictionStaticHead_case-b.png\"
     alt=\"PipeFrictionStaticHead_case-b.png\">
</p>
</html>"),
              Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, 64}, {100, -64}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Backward), Rectangle(extent = {{-100, 50}, {100, -49}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{-60, -49}, {-60, 50}}, color = {0, 0, 255}, arrow = {Arrow.Filled, Arrow.Filled}), Text(extent = {{-50, 16}, {6, -10}}, lineColor = {0, 0, 255}, textString = "diameters"), Line(points = {{-100, 74}, {100, 74}}, color = {0, 0, 255}, arrow = {Arrow.Filled, Arrow.Filled})}));
          end DetailedPipeFlow;
        end FlowModels;

        package HeatTransfer "Heat transfer for flow models"
          extends Modelica.Icons.Package;

          partial model PartialFlowHeatTransfer "base class for any pipe heat transfer correlation"
            extends Modelica.Fluid.Interfaces.PartialHeatTransfer;
            // Additional inputs provided to flow heat transfer model
            input SI.Velocity[n] vs "Mean velocities of fluid flow in segments";
            // Geometry parameters and inputs for flow heat transfer
            parameter Real nParallel "number of identical parallel flow devices" annotation(
              Dialog(tab = "Internal Interface", enable = false, group = "Geometry"));
            input SI.Length[n] lengths "Lengths along flow path";
            input SI.Length[n] dimensions "Characteristic dimensions for fluid flow (diameter for pipe flow)";
            input SI.Height[n] roughnesses "Average heights of surface asperities";
            annotation(
              Documentation(info = "<html>
Base class for heat transfer models of flow devices.
<p>
The geometry is specified in the interface with the <code>surfaceAreas[n]</code>, the <code>roughnesses[n]</code>
and the lengths[n] along the flow path.
Moreover the fluid flow is characterized for different types of devices by the characteristic <code>dimensions[n+1]</code>
and the average velocities <code>vs[n+1]</code> of fluid flow.
See <a href=\"modelica://Modelica.Fluid.Pipes.BaseClasses.CharacteristicNumbers.ReynoldsNumber\">Pipes.BaseClasses.CharacteristicNumbers.ReynoldsNumber</a>
for example definitions.
</p>
</html>"),
              Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-80, 60}, {80, -60}}, pattern = LinePattern.None, lineColor = {0, 0, 0}, fillColor = {255, 0, 0}, fillPattern = FillPattern.HorizontalCylinder), Text(extent = {{-40, 22}, {38, -18}}, lineColor = {0, 0, 0}, textString = "%name")}));
          end PartialFlowHeatTransfer;

          model IdealFlowHeatTransfer "IdealHeatTransfer: Ideal heat transfer without thermal resistance"
            extends PartialFlowHeatTransfer;
          equation
            Ts = heatPorts.T;
            annotation(
              Documentation(info = "<html>
Ideal heat transfer without thermal resistance.
</html>"));
          end IdealFlowHeatTransfer;

          model ConstantFlowHeatTransfer "ConstantHeatTransfer: Constant heat transfer coefficient"
            extends PartialFlowHeatTransfer;
            parameter SI.CoefficientOfHeatTransfer alpha0 "heat transfer coefficient";
          equation
            Q_flows = {alpha0 * surfaceAreas[i] * (heatPorts[i].T - Ts[i]) * nParallel for i in 1:n};
            annotation(
              Documentation(info = "<html>
<p>
Simple heat transfer correlation with constant heat transfer coefficient, used as default component in distributed pipe models.
</p>
</html>"));
          end ConstantFlowHeatTransfer;

          partial model PartialPipeFlowHeatTransfer "Base class for pipe heat transfer correlation in terms of Nusselt number heat transfer in a circular pipe for laminar and turbulent one-phase flow"
            extends PartialFlowHeatTransfer;
            parameter SI.CoefficientOfHeatTransfer alpha0 = 100 "guess value for heat transfer coefficients";
            SI.CoefficientOfHeatTransfer[n] alphas(each start = alpha0) "CoefficientOfHeatTransfer";
            Real[n] Res "Reynolds numbers";
            Real[n] Prs "Prandtl numbers";
            Real[n] Nus "Nusselt numbers";
            Medium.Density[n] ds "Densities";
            Medium.DynamicViscosity[n] mus "Dynamic viscosities";
            Medium.ThermalConductivity[n] lambdas "Thermal conductivity";
            SI.Length[n] diameters = dimensions "Hydraulic diameters for pipe flow";
          equation
            ds = Medium.density(states);
            mus = Medium.dynamicViscosity(states);
            lambdas = Medium.thermalConductivity(states);
            Prs = Medium.prandtlNumber(states);
            Res = CharacteristicNumbers.ReynoldsNumber(vs, ds, mus, diameters);
            Nus = CharacteristicNumbers.NusseltNumber(alphas, diameters, lambdas);
            Q_flows = {alphas[i] * surfaceAreas[i] * (heatPorts[i].T - Ts[i]) * nParallel for i in 1:n};
            annotation(
              Documentation(info = "<html>
<p>
Base class for heat transfer models that are expressed in terms of the Nusselt number and which can be used in distributed pipe models.
</p>
</html>"));
          end PartialPipeFlowHeatTransfer;

          model LocalPipeFlowHeatTransfer "LocalPipeFlowHeatTransfer: Laminar and turbulent forced convection in pipes, local coefficients"
            extends PartialPipeFlowHeatTransfer;
          protected
            Real[n] Nus_turb "Nusselt number for turbulent flow";
            Real[n] Nus_lam "Nusselt number for laminar flow";
            Real Nu_1;
            Real[n] Nus_2;
            Real[n] Xis;
          equation
            Nu_1 = 3.66;
            for i in 1:n loop
              Nus_turb[i] = smooth(0, Xis[i] / 8 * abs(Res[i]) * Prs[i] / (1 + 12.7 * (Xis[i] / 8) ^ 0.5 * (Prs[i] ^ (2 / 3) - 1)) * (1 + 1 / 3 * (diameters[i] / lengths[i] / (if vs[i] >= 0 then i - 0.5 else n - i + 0.5)) ^ (2 / 3)));
              Xis[i] = (1.8 * Modelica.Math.log10(max(1e-10, Res[i])) - 1.5) ^ (-2);
              Nus_lam[i] = (Nu_1 ^ 3 + 0.7 ^ 3 + (Nus_2[i] - 0.7) ^ 3) ^ (1 / 3);
              Nus_2[i] = smooth(0, 1.077 * (abs(Res[i]) * Prs[i] * diameters[i] / lengths[i] / (if vs[i] >= 0 then i - 0.5 else n - i + 0.5)) ^ (1 / 3));
              Nus[i] = Modelica.Media.Air.MoistAir.Utilities.spliceFunction(Nus_turb[i], Nus_lam[i], Res[i] - 6150, 3850);
            end for;
            annotation(
              Documentation(info = "<html>
<p>
Heat transfer model for laminar and turbulent flow in pipes. Range of validity:
</p>
<ul>
<li>fully developed pipe flow</li>
<li>forced convection</li>
<li>one phase Newtonian fluid</li>
<li>(spatial) constant wall temperature in the laminar region</li>
<li>0 &le; Re &le; 1e6, 0.6 &le; Pr &le; 100, d/L &le; 1</li>
<li>The correlation holds for non-circular pipes only in the turbulent region. Use diameter=4*crossArea/perimeter as characteristic length.</li>
</ul>
<p>
The correlation takes into account the spatial position along the pipe flow, which changes discontinuously at flow reversal. However, the heat transfer coefficient itself is continuous around zero flow rate, but not its derivative.
</p>
<h4>References</h4>
<dl><dt>Verein Deutscher Ingenieure (1997):</dt>
    <dd><b>VDI W&auml;rmeatlas</b>.
         Springer Verlag, Ed. 8, 1997.</dd>
</dl>
</html>"));
          end LocalPipeFlowHeatTransfer;
          annotation(
            Documentation(info = "<html>
<p>
Heat transfer correlations for pipe models
</p>
</html>"));
        end HeatTransfer;

        package CharacteristicNumbers "Functions to compute characteristic numbers"
          extends Modelica.Icons.Package;

          function ReynoldsNumber "Return Reynolds number from v, rho, mu, D"
            extends Modelica.Icons.Function;
            input SI.Velocity v "Mean velocity of fluid flow";
            input SI.Density rho "Fluid density";
            input SI.DynamicViscosity mu "Dynamic (absolute) viscosity";
            input SI.Length D "Characteristic dimension (hydraulic diameter of pipes)";
            output SI.ReynoldsNumber Re "Reynolds number";
          algorithm
            Re := abs(v) * rho * D / mu;
            annotation(
              Documentation(info = "<html>
<p>
Calculation of Reynolds Number
</p>
<pre>
   Re = |v|&rho;D/&mu;
</pre>
<p>
a measure of the relationship between inertial forces (v&rho;) and viscous forces (D/&mu;).
</p>
<p>
The following table gives examples for the characteristic dimension D and the velocity v for different fluid flow devices:
</p>
<table border=1 cellspacing=0 cellpadding=2>
<tr><th><b>Device Type</b></th><th><b>Characteristic Dimension D</b></th><th><b>Velocity v</b></th></tr>
<tr><td>Circular Pipe</td><td>diameter</td>
    <td>m_flow/&rho;/crossArea</td></tr>
<tr><td>Rectangular Duct</td><td>4*crossArea/perimeter</td>
    <td>m_flow/&rho;/crossArea</td></tr>
<tr><td>Wide Duct</td><td>distance between narrow, parallel walls</td>
    <td>m_flow/&rho;/crossArea</td></tr>
<tr><td>Packed Bed</td><td>diameterOfSpericalParticles/(1-fluidFractionOfTotalVolume)</td>
    <td>m_flow/&rho;/crossArea (without particles)</td></tr>
<tr><td>Device with rotating agitator</td><td>diameterOfRotor</td>
    <td>RotationalSpeed*diameterOfRotor</td></tr>
</table>
</html>"));
          end ReynoldsNumber;

          function ReynoldsNumber_m_flow "Return Reynolds number from m_flow, mu, D, A"
            extends Modelica.Icons.Function;
            input SI.MassFlowRate m_flow "Mass flow rate";
            input SI.DynamicViscosity mu "Dynamic viscosity";
            input SI.Length D "Characteristic dimension (hydraulic diameter of pipes or orifices)";
            input SI.Area A = Modelica.Constants.pi / 4 * D * D "Cross sectional area of fluid flow";
            output SI.ReynoldsNumber Re "Reynolds number";
          algorithm
            Re := abs(m_flow) * D / A / mu;
            annotation(
              Documentation(info = "<html>Simplified calculation of Reynolds Number for flow through pipes or orifices;
              using the mass flow rate <code>m_flow</code> instead of the velocity <code>v</code> to express inertial forces.
<pre>
  Re = |m_flow|*diameter/A/&mu;
with
  m_flow = v*&rho;*A
</pre>
See also <a href=\"modelica://Modelica.Fluid.Pipes.BaseClasses.CharacteristicNumbers.ReynoldsNumber\">
          Pipes.BaseClasses.CharacteristicNumbers.ReynoldsNumber</a>.
</html>"));
          end ReynoldsNumber_m_flow;

          function NusseltNumber "Return Nusselt number"
            extends Modelica.Icons.Function;
            input SI.CoefficientOfHeatTransfer alpha "Coefficient of heat transfer";
            input SI.Length D "Characteristic dimension";
            input SI.ThermalConductivity lambda "Thermal conductivity";
            output SI.NusseltNumber Nu "Nusselt number";
          algorithm
            Nu := alpha * D / lambda;
            annotation(
              Documentation(info = "Nusselt number Nu = alpha*D/lambda"));
          end NusseltNumber;
        end CharacteristicNumbers;

        package WallFriction "Different variants for pressure drops due to pipe wall friction"
          extends Modelica.Icons.Package;

          partial package PartialWallFriction "Partial wall friction characteristic (base package of all wall friction characteristics)"
            extends Modelica.Icons.Package;
            import Modelica.Constants.pi;
            // Constants to be set in subpackages
            constant Boolean use_mu = true "= true, if mu_a/mu_b are used in function, otherwise value is not used";
            constant Boolean use_roughness = true "= true, if roughness is used in function, otherwise value is not used";
            constant Boolean use_dp_small = true "= true, if dp_small is used in function, otherwise value is not used";
            constant Boolean use_m_flow_small = true "= true, if m_flow_small is used in function, otherwise value is not used";
            constant Boolean dp_is_zero = false "= true, if no wall friction is present, i.e., dp = 0 (function massFlowRate_dp() cannot be used)";
            constant Boolean use_Re_turbulent = true "= true, if Re_turbulent input is used in function, otherwise value is not used";
            // pressure loss characteristic functions

            replaceable partial function massFlowRate_dp "Return mass flow rate m_flow as function of pressure loss dp, i.e., m_flow = f(dp), due to wall friction"
              extends Modelica.Icons.Function;
              input SI.Pressure dp "Pressure loss (dp = port_a.p - port_b.p)";
              input SI.Density rho_a "Density at port_a";
              input SI.Density rho_b "Density at port_b";
              input SI.DynamicViscosity mu_a "Dynamic viscosity at port_a (dummy if use_mu = false)";
              input SI.DynamicViscosity mu_b "Dynamic viscosity at port_b (dummy if use_mu = false)";
              input SI.Length length "Length of pipe";
              input SI.Diameter diameter "Inner (hydraulic) diameter of pipe";
              input SI.Area crossArea = pi * diameter ^ 2 / 4 "Inner cross section area";
              input SI.Length roughness(min = 0) = 2.5e-5 "Absolute roughness of pipe, with a default for a smooth steel pipe (dummy if use_roughness = false)";
              input SI.AbsolutePressure dp_small = 1 "Regularization of zero flow if |dp| < dp_small (dummy if use_dp_small = false)";
              input SI.ReynoldsNumber Re_turbulent = 4000 "Turbulent flow if Re >= Re_turbulent (dummy if use_Re_turbulent = false)";
              output SI.MassFlowRate m_flow "Mass flow rate from port_a to port_b";
              annotation(
                Documentation(info = "<html>

</html>"));
            end massFlowRate_dp;

            replaceable partial function massFlowRate_dp_staticHead "Return mass flow rate m_flow as function of pressure loss dp, i.e., m_flow = f(dp), due to wall friction and static head"
              extends Modelica.Icons.Function;
              input SI.Pressure dp "Pressure loss (dp = port_a.p - port_b.p)";
              input SI.Density rho_a "Density at port_a";
              input SI.Density rho_b "Density at port_b";
              input SI.DynamicViscosity mu_a "Dynamic viscosity at port_a (dummy if use_mu = false)";
              input SI.DynamicViscosity mu_b "Dynamic viscosity at port_b (dummy if use_mu = false)";
              input SI.Length length "Length of pipe";
              input SI.Diameter diameter "Inner (hydraulic) diameter of pipe";
              input Real g_times_height_ab "Gravity times (Height(port_b) - Height(port_a))";
              input SI.Area crossArea = pi * diameter ^ 2 / 4 "Inner cross section area";
              input SI.Length roughness(min = 0) = 2.5e-5 "Absolute roughness of pipe, with a default for a smooth steel pipe (dummy if use_roughness = false)";
              input SI.AbsolutePressure dp_small = 1 "Regularization of zero flow if |dp| < dp_small (dummy if use_dp_small = false)";
              input SI.ReynoldsNumber Re_turbulent = 4000 "Turbulent flow if Re >= Re_turbulent (dummy if use_Re_turbulent = false)";
              output SI.MassFlowRate m_flow "Mass flow rate from port_a to port_b";
              annotation(
                Documentation(info = "<html>

</html>"));
            end massFlowRate_dp_staticHead;

            replaceable partial function pressureLoss_m_flow "Return pressure loss dp as function of mass flow rate m_flow, i.e., dp = f(m_flow), due to wall friction"
              extends Modelica.Icons.Function;
              input SI.MassFlowRate m_flow "Mass flow rate from port_a to port_b";
              input SI.Density rho_a "Density at port_a";
              input SI.Density rho_b "Density at port_b";
              input SI.DynamicViscosity mu_a "Dynamic viscosity at port_a (dummy if use_mu = false)";
              input SI.DynamicViscosity mu_b "Dynamic viscosity at port_b (dummy if use_mu = false)";
              input SI.Length length "Length of pipe";
              input SI.Diameter diameter "Inner (hydraulic) diameter of pipe";
              input SI.Area crossArea = pi * diameter ^ 2 / 4 "Inner cross section area";
              input SI.Length roughness(min = 0) = 2.5e-5 "Absolute roughness of pipe, with a default for a smooth steel pipe (dummy if use_roughness = false)";
              input SI.MassFlowRate m_flow_small = 0.01 "Regularization of zero flow if |m_flow| < m_flow_small (dummy if use_m_flow_small = false)";
              input SI.ReynoldsNumber Re_turbulent = 4000 "Turbulent flow if Re >= Re_turbulent (dummy if use_Re_turbulent = false)";
              output SI.Pressure dp "Pressure loss (dp = port_a.p - port_b.p)";
              annotation(
                Documentation(info = "<html>

</html>"));
            end pressureLoss_m_flow;

            replaceable partial function pressureLoss_m_flow_staticHead "Return pressure loss dp as function of mass flow rate m_flow, i.e., dp = f(m_flow), due to wall friction and static head"
              extends Modelica.Icons.Function;
              input SI.MassFlowRate m_flow "Mass flow rate from port_a to port_b";
              input SI.Density rho_a "Density at port_a";
              input SI.Density rho_b "Density at port_b";
              input SI.DynamicViscosity mu_a "Dynamic viscosity at port_a (dummy if use_mu = false)";
              input SI.DynamicViscosity mu_b "Dynamic viscosity at port_b (dummy if use_mu = false)";
              input SI.Length length "Length of pipe";
              input SI.Diameter diameter "Inner (hydraulic) diameter of pipe";
              input Real g_times_height_ab "Gravity times (Height(port_b) - Height(port_a))";
              input SI.Area crossArea = pi * diameter ^ 2 / 4 "Inner cross section area";
              input SI.Length roughness(min = 0) = 2.5e-5 "Absolute roughness of pipe, with a default for a smooth steel pipe (dummy if use_roughness = false)";
              input SI.MassFlowRate m_flow_small = 0.01 "Regularization of zero flow if |m_flow| < m_flow_small (dummy if use_m_flow_small = false)";
              input SI.ReynoldsNumber Re_turbulent = 4000 "Turbulent flow if Re >= Re_turbulent (dummy if use_Re_turbulent = false)";
              output SI.Pressure dp "Pressure loss (dp = port_a.p - port_b.p)";
              annotation(
                Documentation(info = "<html>

</html>"));
            end pressureLoss_m_flow_staticHead;
            annotation(
              Documentation(info = "<html>

</html>"));
          end PartialWallFriction;

          package NoFriction "No pipe wall friction, no static head"
            extends Modelica.Icons.Package;
            extends PartialWallFriction(final use_mu = false, final use_roughness = false, final use_dp_small = false, final use_m_flow_small = false, final dp_is_zero = true, final use_Re_turbulent = false);

            redeclare function extends massFlowRate_dp "Return mass flow rate m_flow as function of pressure loss dp, i.e., m_flow = f(dp), due to wall friction"
              algorithm
                assert(false, "function massFlowRate_dp (option: from_dp=true)
cannot be used for WallFriction.NoFriction. Use instead
function pressureLoss_m_flow (option: from_dp=false)");
              annotation(
                Documentation(info = "<html>

</html>"));
            end massFlowRate_dp;

            redeclare function extends pressureLoss_m_flow "Return pressure loss dp as function of mass flow rate m_flow, i.e., dp = f(m_flow), due to wall friction"
              algorithm
                dp := 0;
              annotation(
                Documentation(info = "<html>

</html>"));
            end pressureLoss_m_flow;

            redeclare function extends massFlowRate_dp_staticHead "Return mass flow rate m_flow as function of pressure loss dp, i.e., m_flow = f(dp), due to wall friction and static head"
              algorithm
                assert(false, "function massFlowRate_dp (option: from_dp=true)
cannot be used for WallFriction.NoFriction. Use instead
function pressureLoss_m_flow (option: from_dp=false)");
              annotation(
                Documentation(info = "<html>

</html>"));
            end massFlowRate_dp_staticHead;

            redeclare function extends pressureLoss_m_flow_staticHead "Return pressure loss dp as function of mass flow rate m_flow, i.e., dp = f(m_flow), due to wall friction and static head"
                /* To include only static head:
                protected
                  Real dp_grav_a = g_times_height_ab*rho_a
                    "Static head if mass flows in design direction (a to b)";
                  Real dp_grav_b = g_times_height_ab*rho_b
                    "Static head if mass flows against design direction (b to a)";
                */

              algorithm
//  dp := Utilities.regStep(m_flow, dp_grav_a, dp_grav_a, m_flow_small);
                dp := 0;
                assert(abs(g_times_height_ab) < Modelica.Constants.small, "WallFriction.NoFriction does not consider static head and cannot be used with height_ab<>0!");
              annotation(
                Documentation(info = "<html>

</html>"));
            end pressureLoss_m_flow_staticHead;
            annotation(
              Documentation(info = "<html>
<p>
This component sets the pressure loss due to wall friction
to zero, i.e., it allows to switch off pipe wall friction.
</p>
</html>"));
          end NoFriction;

          package Laminar "Pipe wall friction for laminar flow in circular tubes (linear correlation)"
            extends PartialWallFriction(final use_mu = true, final use_roughness = false, final use_dp_small = false, final use_m_flow_small = false, final use_Re_turbulent = false);

            redeclare function extends massFlowRate_dp "Return mass flow rate m_flow as function of pressure loss dp, i.e., m_flow = f(dp), due to wall friction"
              algorithm
// WARNING: The following equation is only correct for circular tubes!
                m_flow := dp * Modelica.Constants.pi * diameter ^ 4 * (rho_a + rho_b) / (128 * length * (mu_a + mu_b));
              annotation(
                Documentation(info = "<html>

</html>"));
            end massFlowRate_dp;

            redeclare function extends pressureLoss_m_flow "Return pressure loss dp as function of mass flow rate m_flow, i.e., dp = f(m_flow), due to wall friction"
              algorithm
// WARNING: The following equation is only correct for circular tubes!
                dp := m_flow * 128 * length * (mu_a + mu_b) / (Modelica.Constants.pi * diameter ^ 4 * (rho_a + rho_b));
              annotation(
                Documentation(info = "<html>

</html>"));
            end pressureLoss_m_flow;

            redeclare function extends massFlowRate_dp_staticHead "Return mass flow rate m_flow as function of pressure loss dp, i.e., m_flow = f(dp), due to wall friction and static head"
                // WARNING: The following equation is only correct for circular tubes!

              protected
                Real k0inv = Modelica.Constants.pi * diameter ^ 4 / (128 * length) "Constant factor";
                Real dp_grav_a = g_times_height_ab * rho_a "Static head if mass flows in design direction (a to b)";
                Real dp_grav_b = g_times_height_ab * rho_b "Static head if mass flows against design direction (b to a)";
                Real dm_flow_ddp_fric_a = k0inv * rho_a / mu_a "Slope of mass flow rate over dp if flow in design direction (a to b)";
                Real dm_flow_ddp_fric_b = k0inv * rho_b / mu_b "Slope of mass flow rate over dp if flow against design direction (b to a)";
                Real dp_a = max(dp_grav_a, dp_grav_b) + dp_small "Upper end of regularization domain of the m_flow(dp) relation";
                Real dp_b = min(dp_grav_a, dp_grav_b) - dp_small "Lower end of regularization domain of the m_flow(dp) relation";
                SI.MassFlowRate m_flow_a "Value at upper end of regularization domain";
                SI.MassFlowRate m_flow_b "Value at lower end of regularization domain";
                // Properly define zero mass flow conditions
                SI.MassFlowRate m_flow_zero = 0;
                SI.Pressure dp_zero = (dp_grav_a + dp_grav_b) / 2;
                Real dm_flow_ddp_fric_zero;

              algorithm
/*
  dp = 0.5*zeta/(A^2*d) * m_flow * |m_flow|
     = 0.5 * c0/(|m_flow|*(4/pi)/(D_Re*mu)) / ((pi*(D_Re/2)^2)^2*d) * m_flow*|m_flow|
     = 0.5 * c0*(pi/4)*(D_Re*mu) * 16/(pi^2*D_Re^4*d) * m_flow*|m_flow|
     = 2*c0/(pi*D_Re^3) * mu/d * m_flow
     = k0 * mu/d * m_flow
  k0 = 2*c0/(pi*D_Re^3)
*/
                if dp >= dp_a then
                  m_flow := dm_flow_ddp_fric_a * (dp - dp_grav_a);
                elseif dp <= dp_b then
                  m_flow := dm_flow_ddp_fric_b * (dp - dp_grav_b);
                else
                  m_flow_a := dm_flow_ddp_fric_a * (dp_a - dp_grav_a);
                  m_flow_b := dm_flow_ddp_fric_b * (dp_b - dp_grav_b);
                  (m_flow, dm_flow_ddp_fric_zero) := Utilities.regFun3(dp_zero, dp_b, dp_a, m_flow_b, m_flow_a, dm_flow_ddp_fric_b, dm_flow_ddp_fric_a);
                  if dp > dp_zero then
                    m_flow := Utilities.regFun3(dp, dp_zero, dp_a, m_flow_zero, m_flow_a, dm_flow_ddp_fric_zero, dm_flow_ddp_fric_a);
                  else
                    m_flow := Utilities.regFun3(dp, dp_b, dp_zero, m_flow_b, m_flow_zero, dm_flow_ddp_fric_b, dm_flow_ddp_fric_zero);
                  end if;
                end if;
// Positive flow outside regularization
// Negative flow outside regularization
// Include a properly defined zero mass flow point
// Obtain a suitable slope from the linear section slope c (value of m_flow is overwritten later)
// Do regularization
/*
  m_flow := if dp<dp_b then dm_flow_ddp_b*(dp-dp_grav_b) else
              (if dp>dp_a then dm_flow_ddp_a*(dp-dp_grav_a) else
                Modelica.Fluid.Utilities.regFun3(dp, dp_b, dp_a, dm_flow_ddp_b*(dp_b - dp_grav_b), dm_flow_ddp_a*(dp_a - dp_grav_a), dm_flow_ddp_b, dm_flow_ddp_a));
*/
              annotation(
                Documentation(info = "<html>

</html>"));
            end massFlowRate_dp_staticHead;

            redeclare function extends pressureLoss_m_flow_staticHead "Return pressure loss dp as function of mass flow rate m_flow, i.e., dp = f(m_flow), due to wall friction and static head"
                // WARNING: The following equation is only correct for circular tubes!

              protected
                Real k0 = 128 * length / (Modelica.Constants.pi * diameter ^ 4) "Constant factor";
                Real dp_grav_a = g_times_height_ab * rho_a "Static head if mass flows in design direction (a to b)";
                Real dp_grav_b = g_times_height_ab * rho_b "Static head if mass flows against design direction (b to a)";
                Real ddp_dm_flow_a = k0 * mu_a / rho_a "Slope of dp over mass flow rate if flow in design direction (a to b)";
                Real ddp_dm_flow_b = k0 * mu_b / rho_b "Slope of dp over mass flow rate if flow against design direction (b to a)";
                Real m_flow_a = if dp_grav_a >= dp_grav_b then m_flow_small else m_flow_small + (dp_grav_b - dp_grav_a) / ddp_dm_flow_a "Upper end of regularization domain of the dp(m_flow) relation";
                Real m_flow_b = if dp_grav_a >= dp_grav_b then -m_flow_small else (-m_flow_small) - (dp_grav_b - dp_grav_a) / ddp_dm_flow_b "Lower end of regularization domain of the dp(m_flow) relation";
                SI.Pressure dp_a "Value at upper end of regularization domain";
                SI.Pressure dp_b "Value at lower end of regularization domain";
                // Properly define zero mass flow conditions
                SI.MassFlowRate m_flow_zero = 0;
                SI.Pressure dp_zero = (dp_grav_a + dp_grav_b) / 2;
                Real ddp_dm_flow_zero;

              algorithm
/*
  dp = 0.5*zeta/(A^2*d) * m_flow * |m_flow|
     = 0.5 * c0/(|m_flow|*(4/pi)/(D_Re*mu)) / ((pi*(D_Re/2)^2)^2*d) * m_flow*|m_flow|
     = 0.5 * c0*(pi/4)*(D_Re*mu) * 16/(pi^2*D_Re^4*d) * m_flow*|m_flow|
     = 2*c0/(pi*D_Re^3) * mu/d * m_flow
     = k0 * mu/d * m_flow
  k0 = 2*c0/(pi*D_Re^3)
*/
                if m_flow >= m_flow_a then
                  dp := ddp_dm_flow_a * m_flow + dp_grav_a;
                elseif m_flow <= m_flow_b then
                  dp := ddp_dm_flow_b * m_flow + dp_grav_b;
                else
                  dp_a := ddp_dm_flow_a * m_flow_a + dp_grav_a;
                  dp_b := ddp_dm_flow_b * m_flow_b + dp_grav_b;
                  (dp, ddp_dm_flow_zero) := Utilities.regFun3(m_flow_zero, m_flow_b, m_flow_a, dp_b, dp_a, ddp_dm_flow_b, ddp_dm_flow_a);
                  if m_flow > m_flow_zero then
                    dp := Utilities.regFun3(m_flow, m_flow_zero, m_flow_a, dp_zero, dp_a, ddp_dm_flow_zero, ddp_dm_flow_a);
                  else
                    dp := Utilities.regFun3(m_flow, m_flow_b, m_flow_zero, dp_b, dp_zero, ddp_dm_flow_b, ddp_dm_flow_zero);
                  end if;
                end if;
// Positive flow outside regularization
// Negative flow outside regularization
// Regularization parameters
// Include a properly defined zero mass flow point
// Obtain a suitable slope from the linear section slope c (value of dp is overwritten later)
// Do regularization
              annotation(
                Documentation(info = "<html>

</html>"));
            end pressureLoss_m_flow_staticHead;
            annotation(
              Documentation(info = "<html>
<p>
This component defines only the laminar region of wall friction:
dp = k*m_flow, where \"k\" depends on density and dynamic viscosity.
The roughness of the wall does not have an influence on the laminar
flow and therefore argument roughness is ignored.
Since this is a linear relationship, the occurring systems of equations
are usually much simpler (e.g., either linear instead of non-linear).
By using nominal values for density and dynamic viscosity, the
systems of equations can still further be reduced.
</p>

<p>
In <a href=\"modelica://Modelica.Fluid.UsersGuide.ComponentDefinition.WallFriction\">UsersGuide</a> the complete friction regime is illustrated.
This component describes only the <b>Hagen-Poiseuille</b> equation.
</p>
<br>

</html>"));
          end Laminar;

          package QuadraticTurbulent "Pipe wall friction for turbulent flow in circular tubes (simple characteristic, mu not used)"
            import Modelica.Constants.pi;
            extends PartialWallFriction(final use_mu = false, final use_roughness = true, final use_dp_small = true, final use_m_flow_small = true, final use_Re_turbulent = false);

            redeclare function extends massFlowRate_dp "Return mass flow rate m_flow as function of pressure loss dp, i.e., m_flow = f(dp), due to wall friction"
                import Modelica.Math;

              protected
                Real zeta;
                Real k_inv;

              algorithm
/*
   dp = 0.5*zeta*d*v*|v|
      = 0.5*zeta*d*1/(d*A)^2 * m_flow * |m_flow|
      = 0.5*zeta/A^2 *1/d * m_flow * |m_flow|
      = k/d * m_flow * |m_flow|
   k  = 0.5*zeta/A^2
      = 0.5*zeta/(pi*(D/2)^2)^2
      = 8*zeta/(pi*D^2)^2
  */
                assert(roughness > 1e-10, "roughness > 0 required for quadratic turbulent wall friction characteristic");
                zeta := length / diameter / (2 * Math.log10(3.7 / (roughness / diameter))) ^ 2;
// WARNING: The following equation is only correct for circular tubes!
                k_inv := (pi * diameter * diameter) ^ 2 / (8 * zeta);
                m_flow := Modelica.Fluid.Utilities.regRoot2(dp, dp_small, rho_a * k_inv, rho_b * k_inv);
              annotation(
                smoothOrder = 1,
                Documentation(info = "<html>

</html>"));
            end massFlowRate_dp;

            redeclare function extends pressureLoss_m_flow "Return pressure loss dp as function of mass flow rate m_flow, i.e., dp = f(m_flow), due to wall friction"
                import Modelica.Constants.pi;
                import Modelica.Math;

              protected
                Real zeta;
                Real k;

              algorithm
/*
   dp = 0.5*zeta*d*v*|v|
      = 0.5*zeta*d*1/(d*A)^2 * m_flow * |m_flow|
      = 0.5*zeta/A^2 *1/d * m_flow * |m_flow|
      = k/d * m_flow * |m_flow|
   k  = 0.5*zeta/A^2
      = 0.5*zeta/(pi*(D/2)^2)^2
      = 8*zeta/(pi*D^2)^2
  */
                assert(roughness > 1e-10, "roughness > 0 required for quadratic turbulent wall friction characteristic");
                zeta := length / diameter / (2 * Math.log10(3.7 / (roughness / diameter))) ^ 2;
// WARNING: The following equation is only correct for circular tubes!
                k := 8 * zeta / (pi * diameter * diameter) ^ 2;
                dp := Modelica.Fluid.Utilities.regSquare2(m_flow, m_flow_small, k / rho_a, k / rho_b);
              annotation(
                smoothOrder = 1,
                Documentation(info = "<html>

</html>"));
            end pressureLoss_m_flow;

            redeclare function extends massFlowRate_dp_staticHead "Return mass flow rate m_flow as function of pressure loss dp, i.e., m_flow = f(dp), due to wall friction and static head"
                import Modelica.Math;
                import Modelica.Constants.pi;

              protected
                Real zeta = length / diameter / (2 * Math.log10(3.7 / (roughness / diameter))) ^ 2;
                // WARNING: The following equation is only correct for circular tubes!
                Real k_inv = (pi * diameter * diameter) ^ 2 / (8 * zeta);
                SI.Pressure dp_grav_a = g_times_height_ab * rho_a "Static head if mass flows in design direction (a to b)";
                SI.Pressure dp_grav_b = g_times_height_ab * rho_b "Static head if mass flows against design direction (b to a)";
                Real k1 = rho_a * k_inv "Factor in m_flow =  sqrt(k1*(dp-dp_grav_a))";
                Real k2 = rho_b * k_inv "Factor in m_flow = -sqrt(k2*|dp-dp_grav_b|)";
                Real dp_a = max(dp_grav_a, dp_grav_b) + dp_small "Upper end of regularization domain of the m_flow(dp) relation";
                Real dp_b = min(dp_grav_a, dp_grav_b) - dp_small "Lower end of regularization domain of the m_flow(dp) relation";
                SI.MassFlowRate m_flow_a "Value at upper end of regularization domain";
                SI.MassFlowRate m_flow_b "Value at lower end of regularization domain";
                SI.MassFlowRate dm_flow_ddp_fric_a "Derivative at upper end of regularization domain";
                SI.MassFlowRate dm_flow_ddp_fric_b "Derivative at lower end of regularization domain";
                // Properly define zero mass flow conditions
                SI.MassFlowRate m_flow_zero = 0;
                SI.Pressure dp_zero = (dp_grav_a + dp_grav_b) / 2;
                Real dm_flow_ddp_fric_zero;

              algorithm
/*
   dp = 0.5*zeta*d*v*|v|
      = 0.5*zeta*d*1/(d*A)^2 * m_flow * |m_flow|
      = 0.5*zeta/A^2 *1/d * m_flow * |m_flow|
      = k/d * m_flow * |m_flow|
   k  = 0.5*zeta/A^2
      = 0.5*zeta/(pi*(D/2)^2)^2
      = 8*zeta/(pi*D^2)^2
  */
                assert(roughness > 1e-10, "roughness > 0 required for quadratic turbulent wall friction characteristic");
                if dp >= dp_a then
                  m_flow := sqrt(k1 * (dp - dp_grav_a));
                elseif dp <= dp_b then
                  m_flow := -sqrt(k2 * abs(dp - dp_grav_b));
                else
                  m_flow_a := sqrt(k1 * (dp_a - dp_grav_a));
                  m_flow_b := -sqrt(k2 * abs(dp_b - dp_grav_b));
                  dm_flow_ddp_fric_a := k1 / (2 * sqrt(k1 * (dp_a - dp_grav_a)));
                  dm_flow_ddp_fric_b := k2 / (2 * sqrt(k2 * abs(dp_b - dp_grav_b)));
                  (m_flow, dm_flow_ddp_fric_zero) := Utilities.regFun3(dp_zero, dp_b, dp_a, m_flow_b, m_flow_a, dm_flow_ddp_fric_b, dm_flow_ddp_fric_a);
                  if dp > dp_zero then
                    m_flow := Utilities.regFun3(dp, dp_zero, dp_a, m_flow_zero, m_flow_a, dm_flow_ddp_fric_zero, dm_flow_ddp_fric_a);
                  else
                    m_flow := Utilities.regFun3(dp, dp_b, dp_zero, m_flow_b, m_flow_zero, dm_flow_ddp_fric_b, dm_flow_ddp_fric_zero);
                  end if;
                end if;
// Positive flow outside regularization
// Negative flow outside regularization
/*  dm_flow_ddp_fric_a := if abs(dp_a - dp_grav_a)>0 then k1/(2*sqrt(k1*(dp_a - dp_grav_a))) else  Modelica.Constants.inf);
    dm_flow_ddp_fric_b := if abs(dp_b - dp_grav_b)>0 then k2/(2*sqrt(k2*abs(dp_b - dp_grav_b))) else Modelica.Constants.inf; */
// Include a properly defined zero mass flow point
// Obtain a suitable slope from the linear section slope c (value of m_flow is overwritten later)
// Do regularization
              annotation(
                smoothOrder = 1,
                Documentation(info = "<html>

</html>"));
            end massFlowRate_dp_staticHead;

            redeclare function extends pressureLoss_m_flow_staticHead "Return pressure loss dp as function of mass flow rate m_flow, i.e., dp = f(m_flow), due to wall friction and static head"
                import Modelica.Math;
                import Modelica.Constants.pi;

              protected
                Real zeta = length / diameter / (2 * Math.log10(3.7 / (roughness / diameter))) ^ 2;
                // WARNING: The following equation is only correct for circular tubes!
                Real k = 8 * zeta / (pi * diameter * diameter) ^ 2;
                SI.Pressure dp_grav_a = g_times_height_ab * rho_a "Static head if mass flows in design direction (a to b)";
                SI.Pressure dp_grav_b = g_times_height_ab * rho_b "Static head if mass flows against design direction (b to a)";
                Real k1 = k / rho_a "If m_flow >= 0 then dp = k1*m_flow^2 + dp_grav_a";
                Real k2 = k / rho_b "If m_flow < 0 then dp = -k2*m_flow^2 + dp_grav_b";
                Real m_flow_a = if dp_grav_a >= dp_grav_b then m_flow_small else m_flow_small + sqrt((dp_grav_b - dp_grav_a) / k1) "Upper end of regularization domain of the dp(m_flow) relation";
                Real m_flow_b = if dp_grav_a >= dp_grav_b then -m_flow_small else (-m_flow_small) - sqrt((dp_grav_b - dp_grav_a) / k2) "Lower end of regularization domain of the dp(m_flow) relation";
                SI.Pressure dp_a "Value at upper end of regularization domain";
                SI.Pressure dp_b "Value at lower end of regularization domain";
                Real ddp_dm_flow_a "Derivative of pressure drop with mass flow rate at m_flow_a";
                Real ddp_dm_flow_b "Derivative of pressure drop with mass flow rate at m_flow_b";
                // Properly define zero mass flow conditions
                SI.MassFlowRate m_flow_zero = 0;
                SI.Pressure dp_zero = (dp_grav_a + dp_grav_b) / 2;
                Real ddp_dm_flow_zero;

              algorithm
/*
   dp = 0.5*zeta*d*v*|v|
      = 0.5*zeta*d*1/(d*A)^2 * m_flow * |m_flow|
      = 0.5*zeta/A^2 *1/d * m_flow * |m_flow|
      = k/d * m_flow * |m_flow|
   k  = 0.5*zeta/A^2
      = 0.5*zeta/(pi*(D/2)^2)^2
      = 8*zeta/(pi*D^2)^2
  */
                assert(roughness > 1e-10, "roughness > 0 required for quadratic turbulent wall friction characteristic");
                if m_flow >= m_flow_a then
                  dp := k1 * m_flow ^ 2 + dp_grav_a;
                elseif m_flow <= m_flow_b then
                  dp := (-k2 * m_flow ^ 2) + dp_grav_b;
                else
                  dp_a := k1 * m_flow_a ^ 2 + dp_grav_a;
                  ddp_dm_flow_a := 2 * k1 * m_flow_a;
                  dp_b := (-k2 * m_flow_b ^ 2) + dp_grav_b;
                  ddp_dm_flow_b := -2 * k2 * m_flow_b;
                  (dp, ddp_dm_flow_zero) := Utilities.regFun3(m_flow_zero, m_flow_b, m_flow_a, dp_b, dp_a, ddp_dm_flow_b, ddp_dm_flow_a);
                  if m_flow > m_flow_zero then
                    dp := Utilities.regFun3(m_flow, m_flow_zero, m_flow_a, dp_zero, dp_a, ddp_dm_flow_zero, ddp_dm_flow_a);
                  else
                    dp := Utilities.regFun3(m_flow, m_flow_b, m_flow_zero, dp_b, dp_zero, ddp_dm_flow_b, ddp_dm_flow_zero);
                  end if;
                end if;
// Positive flow outside regularization
// Negative flow outside regularization
// Regularization parameters
// Include a properly defined zero mass flow point
// Obtain a suitable slope from the linear section slope c (value of dp is overwritten later)
// Do regularization
              annotation(
                smoothOrder = 1,
                Documentation(info = "<html>

</html>"));
            end pressureLoss_m_flow_staticHead;
            annotation(
              Documentation(info = "<html>
<p>
This component defines only the quadratic turbulent regime of wall friction:
dp = k*m_flow*|m_flow|, where \"k\" depends on density and the roughness
of the pipe and is no longer a function of the Reynolds number.
This relationship is only valid for large Reynolds numbers.
</p>

<p>
In <a href=\"modelica://Modelica.Fluid.UsersGuide.ComponentDefinition.WallFriction\">UsersGuide</a> the complete friction regime is illustrated.
This component describes only the asymptotic behaviour for large
Reynolds numbers, i.e., the values at the right ordinate where
&lambda; is constant.
</p>
<br>

</html>"));
          end QuadraticTurbulent;

          package LaminarAndQuadraticTurbulent "Pipe wall friction for laminar and turbulent flow in circular tubes (simple characteristic)"
            extends PartialWallFriction(final use_mu = true, final use_roughness = true, final use_dp_small = true, final use_m_flow_small = true, final use_Re_turbulent = true);
            import ln = Modelica.Math.log;
            import Modelica.Math.log10;
            import Modelica.Math.exp;
            import Modelica.Constants.pi;

            redeclare function extends massFlowRate_dp "Return mass flow rate m_flow as function of pressure loss dp, i.e., m_flow = f(dp), due to wall friction"
                import Modelica.Math;

              protected
                Real zeta;
                Real k0;
                Real k_inv;
                Real yd0 "Derivative of m_flow=m_flow(dp) at zero";
                SI.AbsolutePressure dp_turbulent;

              algorithm
/*
Turbulent region:
   // WARNING: The following equations are only correct for circular tubes!
   Re = m_flow*(4/pi)/(D_Re*mu)
   dp = 0.5*zeta*rho*v*|v|
      = 0.5*zeta*rho*1/(rho*A)^2 * m_flow * |m_flow|
      = 0.5*zeta/A^2 *1/rho * m_flow * |m_flow|
      = k/rho * m_flow * |m_flow|
   k  = 0.5*zeta/A^2
      = 0.5*zeta/(pi*(D/2)^2)^2
      = 8*zeta/(pi*D^2)^2
   m_flow_turbulent = (pi/4)*D_Re*mu*Re_turbulent
   dp_turbulent     =  k/rho *(D_Re*mu*pi/4)^2 * Re_turbulent^2

   The start of the turbulent region is computed with mean values
   of dynamic viscosity mu and density rho. Otherwise, one has
   to introduce different "delta" values for both flow directions.
   In order to simplify the approach, only one delta is used.

Laminar region:
   // WARNING: The following equation is only correct for circular tubes!
   dp = 0.5*zeta/(A^2*d) * m_flow * |m_flow|
      = 0.5 * c0/(|m_flow|*(4/pi)/(D_Re*mu)) / ((pi*(D_Re/2)^2)^2*d) * m_flow*|m_flow|
      = 0.5 * c0*(pi/4)*(D_Re*mu) * 16/(pi^2*D_Re^4*d) * m_flow*|m_flow|
      = 2*c0/(pi*D_Re^3) * mu/rho * m_flow
      = k0 * mu/rho * m_flow
   k0 = 2*c0/(pi*D_Re^3)

   In order that the derivative of dp=f(m_flow) is continuous
   at m_flow=0, the mean values of mu and d are used in the
   laminar region: mu/rho = (mu_a + mu_b)/(rho_a + rho_b)
   If data.zetaLaminarKnown = false then mu_a and mu_b are potentially zero
   (because dummy values) and therefore the division is only performed
   if zetaLaminarKnown = true.
*/
                assert(roughness > 1e-10, "roughness > 0 required for quadratic turbulent wall friction characteristic");
// WARNING: The following equations are only correct for circular tubes!
                zeta := length / diameter / (2 * Math.log10(3.7 / (roughness / diameter))) ^ 2;
                k0 := 128 * length / (pi * diameter ^ 4);
                k_inv := (pi * diameter * diameter) ^ 2 / (8 * zeta);
                yd0 := (rho_a + rho_b) / (k0 * (mu_a + mu_b));
                dp_turbulent := max(((mu_a + mu_b) * diameter * pi / 8) ^ 2 * Re_turbulent ^ 2 / (k_inv * (rho_a + rho_b) / 2), dp_small);
                m_flow := Modelica.Fluid.Utilities.regRoot2(dp, dp_turbulent, rho_a * k_inv, rho_b * k_inv, use_yd0 = true, yd0 = yd0);
              annotation(
                smoothOrder = 1,
                Documentation(info = "<html>

</html>"));
            end massFlowRate_dp;

            redeclare function extends pressureLoss_m_flow "Return pressure loss dp as function of mass flow rate m_flow, i.e., dp = f(m_flow), due to wall friction"
                import Modelica.Math;
                import Modelica.Constants.pi;

              protected
                Real zeta;
                Real k0;
                Real k;
                Real yd0 "Derivative of dp = f(m_flow) at zero";
                SI.MassFlowRate m_flow_turbulent "The turbulent region is: |m_flow| >= m_flow_turbulent";

              algorithm
/*
Turbulent region:
   Re = m_flow*(4/pi)/(D_Re*mu)
   dp = 0.5*zeta*rho*v*|v|
      = 0.5*zeta*rho*1/(rho*A)^2 * m_flow * |m_flow|
      = 0.5*zeta/A^2 *1/rho * m_flow * |m_flow|
      = k/rho * m_flow * |m_flow|
   k  = 0.5*zeta/A^2
      = 0.5*zeta/(pi*(D/2)^2)^2
      = 8*zeta/(pi*D^2)^2
   m_flow_turbulent = (pi/4)*D_Re*mu*Re_turbulent
   dp_turbulent     =  k/rho *(D_Re*mu*pi/4)^2 * Re_turbulent^2

   The start of the turbulent region is computed with mean values
   of dynamic viscosity mu and density rho. Otherwise, one has
   to introduce different "delta" values for both flow directions.
   In order to simplify the approach, only one delta is used.

Laminar region:
   dp = 0.5*zeta/(A^2*d) * m_flow * |m_flow|
      = 0.5 * c0/(|m_flow|*(4/pi)/(D_Re*mu)) / ((pi*(D_Re/2)^2)^2*d) * m_flow*|m_flow|
      = 0.5 * c0*(pi/4)*(D_Re*mu) * 16/(pi^2*D_Re^4*d) * m_flow*|m_flow|
      = 2*c0/(pi*D_Re^3) * mu/rho * m_flow
      = k0 * mu/rho * m_flow
   k0 = 2*c0/(pi*D_Re^3)

   In order that the derivative of dp=f(m_flow) is continuous
   at m_flow=0, the mean values of mu and d are used in the
   laminar region: mu/rho = (mu_a + mu_b)/(rho_a + rho_b)
*/
                assert(roughness > 1e-10, "roughness > 0 required for quadratic turbulent wall friction characteristic");
// WARNING: The following equations are only correct for circular tubes!
                zeta := length / diameter / (2 * Math.log10(3.7 / (roughness / diameter))) ^ 2;
                k0 := 128 * length / (pi * diameter ^ 4);
                k := 8 * zeta / (pi * diameter * diameter) ^ 2;
                yd0 := k0 * (mu_a + mu_b) / (rho_a + rho_b);
                m_flow_turbulent := max(pi / 8 * diameter * (mu_a + mu_b) * Re_turbulent, m_flow_small);
                dp := Modelica.Fluid.Utilities.regSquare2(m_flow, m_flow_turbulent, k / rho_a, k / rho_b, use_yd0 = true, yd0 = yd0);
              annotation(
                smoothOrder = 1,
                Documentation(info = "<html>

</html>"));
            end pressureLoss_m_flow;

            redeclare function extends massFlowRate_dp_staticHead "Return mass flow rate m_flow as function of pressure loss dp, i.e., m_flow = f(dp), due to wall friction and static head"
                import Modelica.Math;

              protected
                Real Delta = roughness / diameter "Relative roughness";
                SI.ReynoldsNumber Re1 = min(745 * exp(if Delta <= 0.0065 then 1 else 0.0065 / Delta), Re_turbulent) "Boundary between laminar regime and transition";
                SI.ReynoldsNumber Re2 = Re_turbulent "Boundary between transition and turbulent regime";
                SI.Pressure dp_a "Upper end of regularization domain of the m_flow(dp) relation";
                SI.Pressure dp_b "Lower end of regularization domain of the m_flow(dp) relation";
                SI.MassFlowRate m_flow_a "Value at upper end of regularization domain";
                SI.MassFlowRate m_flow_b "Value at lower end of regularization domain";
                SI.MassFlowRate dm_flow_ddp_fric_a "Derivative at upper end of regularization domain";
                SI.MassFlowRate dm_flow_ddp_fric_b "Derivative at lower end of regularization domain";
                SI.Pressure dp_grav_a = g_times_height_ab * rho_a "Static head if mass flows in design direction (a to b)";
                SI.Pressure dp_grav_b = g_times_height_ab * rho_b "Static head if mass flows against design direction (b to a)";
                // Properly define zero mass flow conditions
                SI.MassFlowRate m_flow_zero = 0;
                SI.Pressure dp_zero = (dp_grav_a + dp_grav_b) / 2;
                Real dm_flow_ddp_fric_zero;

              algorithm
                assert(roughness > 1e-10, "roughness > 0 required for quadratic turbulent wall friction characteristic");
                dp_a := max(dp_grav_a, dp_grav_b) + dp_small;
                dp_b := min(dp_grav_a, dp_grav_b) - dp_small;
                if dp >= dp_a then
                  m_flow := Internal.m_flow_of_dp_fric(dp - dp_grav_a, rho_a, rho_b, mu_a, mu_b, length, diameter, crossArea, Re1, Re2, Delta);
                elseif dp <= dp_b then
                  m_flow := Internal.m_flow_of_dp_fric(dp - dp_grav_b, rho_a, rho_b, mu_a, mu_b, length, diameter, crossArea, Re1, Re2, Delta);
                else
                  (m_flow_a, dm_flow_ddp_fric_a) := Internal.m_flow_of_dp_fric(dp_a - dp_grav_a, rho_a, rho_b, mu_a, mu_b, length, diameter, crossArea, Re1, Re2, Delta);
                  (m_flow_b, dm_flow_ddp_fric_b) := Internal.m_flow_of_dp_fric(dp_b - dp_grav_b, rho_a, rho_b, mu_a, mu_b, length, diameter, crossArea, Re1, Re2, Delta);
                  (m_flow, dm_flow_ddp_fric_zero) := Utilities.regFun3(dp_zero, dp_b, dp_a, m_flow_b, m_flow_a, dm_flow_ddp_fric_b, dm_flow_ddp_fric_a);
                  if dp > dp_zero then
                    m_flow := Utilities.regFun3(dp, dp_zero, dp_a, m_flow_zero, m_flow_a, dm_flow_ddp_fric_zero, dm_flow_ddp_fric_a);
                  else
                    m_flow := Utilities.regFun3(dp, dp_b, dp_zero, m_flow_b, m_flow_zero, dm_flow_ddp_fric_b, dm_flow_ddp_fric_zero);
                  end if;
                end if;
// Positive flow outside regularization
// Negative flow outside regularization
// Regularization parameters
// Include a properly defined zero mass flow point
// Obtain a suitable slope from the linear section slope c (value of m_flow is overwritten later)
// Do regularization
              annotation(
                smoothOrder = 1,
                Documentation(info = "<html>

</html>"));
            end massFlowRate_dp_staticHead;

            redeclare function extends pressureLoss_m_flow_staticHead "Return pressure loss dp as function of mass flow rate m_flow, i.e., dp = f(m_flow), due to wall friction and static head"
                import Modelica.Math;

              protected
                Real Delta = roughness / diameter "Relative roughness";
                SI.ReynoldsNumber Re1 = min(745 * exp(if Delta <= 0.0065 then 1 else 0.0065 / Delta), Re_turbulent) "Boundary between laminar regime and transition";
                SI.ReynoldsNumber Re2 = Re_turbulent "Boundary between transition and turbulent regime";
                SI.MassFlowRate m_flow_a "Upper end of regularization domain of the dp(m_flow) relation";
                SI.MassFlowRate m_flow_b "Lower end of regularization domain of the dp(m_flow) relation";
                SI.Pressure dp_a "Value at upper end of regularization domain";
                SI.Pressure dp_b "Value at lower end of regularization domain";
                SI.Pressure dp_grav_a = g_times_height_ab * rho_a "Static head if mass flows in design direction (a to b)";
                SI.Pressure dp_grav_b = g_times_height_ab * rho_b "Static head if mass flows against design direction (b to a)";
                Real ddp_dm_flow_a "Derivative of pressure drop with mass flow rate at m_flow_a";
                Real ddp_dm_flow_b "Derivative of pressure drop with mass flow rate at m_flow_b";
                // Properly define zero mass flow conditions
                SI.MassFlowRate m_flow_zero = 0;
                SI.Pressure dp_zero = (dp_grav_a + dp_grav_b) / 2;
                Real ddp_dm_flow_zero;

              algorithm
                assert(roughness > 1e-10, "roughness > 0 required for quadratic turbulent wall friction characteristic");
                m_flow_a := if dp_grav_a < dp_grav_b then Internal.m_flow_of_dp_fric(dp_grav_b - dp_grav_a, rho_a, rho_b, mu_a, mu_b, length, diameter, crossArea, Re1, Re2, Delta) + m_flow_small else m_flow_small;
                m_flow_b := if dp_grav_a < dp_grav_b then Internal.m_flow_of_dp_fric(dp_grav_a - dp_grav_b, rho_a, rho_b, mu_a, mu_b, length, diameter, crossArea, Re1, Re2, Delta) - m_flow_small else -m_flow_small;
                if m_flow >= m_flow_a then
                  dp := Internal.dp_fric_of_m_flow(m_flow, rho_a, rho_b, mu_a, mu_b, length, diameter, crossArea, Re1, Re2, Delta) + dp_grav_a;
                elseif m_flow <= m_flow_b then
                  dp := Internal.dp_fric_of_m_flow(m_flow, rho_a, rho_b, mu_a, mu_b, length, diameter, crossArea, Re1, Re2, Delta) + dp_grav_b;
                else
                  (dp_a, ddp_dm_flow_a) := Internal.dp_fric_of_m_flow(m_flow_a, rho_a, rho_b, mu_a, mu_b, length, diameter, crossArea, Re1, Re2, Delta);
                  dp_a := dp_a + dp_grav_a "Adding dp_grav to dp_fric to get dp";
                  (dp_b, ddp_dm_flow_b) := Internal.dp_fric_of_m_flow(m_flow_b, rho_a, rho_b, mu_a, mu_b, length, diameter, crossArea, Re1, Re2, Delta);
                  dp_b := dp_b + dp_grav_b "Adding dp_grav to dp_fric to get dp";
                  (dp, ddp_dm_flow_zero) := Utilities.regFun3(m_flow_zero, m_flow_b, m_flow_a, dp_b, dp_a, ddp_dm_flow_b, ddp_dm_flow_a);
                  if m_flow > m_flow_zero then
                    dp := Utilities.regFun3(m_flow, m_flow_zero, m_flow_a, dp_zero, dp_a, ddp_dm_flow_zero, ddp_dm_flow_a);
                  else
                    dp := Utilities.regFun3(m_flow, m_flow_b, m_flow_zero, dp_b, dp_zero, ddp_dm_flow_b, ddp_dm_flow_zero);
                  end if;
                end if;
// Positive flow outside regularization
// Negative flow outside regularization
// Regularization parameters
// Include a properly defined zero mass flow point
// Obtain a suitable slope from the linear section slope c (value of dp is overwritten later)
// Do regularization
              annotation(
                smoothOrder = 1,
                Documentation(info = "<html>

</html>"));
            end pressureLoss_m_flow_staticHead;

            package Internal "Functions to calculate mass flow rate from friction pressure drop and vice versa"
              extends Modelica.Icons.InternalPackage;

              function m_flow_of_dp_fric "Calculate mass flow rate as function of pressure drop due to friction"
                extends Modelica.Icons.Function;
                input SI.Pressure dp_fric "Pressure loss due to friction (dp = port_a.p - port_b.p)";
                input SI.Density rho_a "Density at port_a";
                input SI.Density rho_b "Density at port_b";
                input SI.DynamicViscosity mu_a "Dynamic viscosity at port_a (dummy if use_mu = false)";
                input SI.DynamicViscosity mu_b "Dynamic viscosity at port_b (dummy if use_mu = false)";
                input SI.Length length "Length of pipe";
                input SI.Diameter diameter "Inner (hydraulic) diameter of pipe";
                input SI.Area crossArea "Inner cross section area";
                input SI.ReynoldsNumber Re1 "Boundary between laminar regime and transition";
                input SI.ReynoldsNumber Re2 "Boundary between transition and turbulent regime";
                input Real Delta "Relative roughness";
                output SI.MassFlowRate m_flow "Mass flow rate from port_a to port_b";
                output Real dm_flow_ddp_fric "Derivative of mass flow rate with dp_fric";
              protected
                SI.DynamicViscosity mu "Upstream viscosity";
                SI.Density rho "Upstream density";
                Real zeta;
                Real k0;
                Real k_inv;
                Real dm_flow_ddp_laminar "Derivative of m_flow=m_flow(dp) in laminar regime";
                SI.AbsolutePressure dp_fric_turbulent "The turbulent region is: |dp_fric| >= dp_fric_turbulent, simple quadratic correlation";
                SI.AbsolutePressure dp_fric_laminar "The laminar region is: |dp_fric| <= dp_fric_laminar";
              algorithm
/*
Turbulent region:
   Re = m_flow*(4/pi)/(D_Re*mu)
   dp = 0.5*zeta*rho*v*|v|
      = 0.5*zeta*rho*1/(rho*A)^2 * m_flow * |m_flow|
      = 0.5*zeta/A^2 *1/rho * m_flow * |m_flow|
      = k/rho * m_flow * |m_flow|
   k  = 0.5*zeta/A^2
      = 0.5*zeta/(pi*(D/2)^2)^2
      = 8*zeta/(pi*D^2)^2
   dp_fric_turbulent     =  k/rho *(D_Re*mu*pi/4)^2 * Re_turbulent^2

Laminar region:
   dp = 0.5*zeta/(A^2*d) * m_flow * |m_flow|
      = 0.5 * c0/(|m_flow|*(4/pi)/(D_Re*mu)) / ((pi*(D_Re/2)^2)^2*d) * m_flow*|m_flow|
      = 0.5 * c0*(pi/4)*(D_Re*mu) * 16/(pi^2*D_Re^4*d) * m_flow*|m_flow|
      = 2*c0/(pi*D_Re^3) * mu/rho * m_flow
      = k0 * mu/rho * m_flow
   k0 = 2*c0/(pi*D_Re^3)
*/
// Determine upstream density and upstream viscosity
                if dp_fric >= 0 then
                  rho := rho_a;
                  mu := mu_a;
                else
                  rho := rho_b;
                  mu := mu_b;
                end if;
// Quadratic turbulent
// WARNING: The following equations are only correct for circular tubes!
                zeta := length / diameter / (2 * log10(3.7 / Delta)) ^ 2;
                k_inv := (pi * diameter * diameter) ^ 2 / (8 * zeta);
                dp_fric_turbulent := sign(dp_fric) * (mu * diameter * pi / 4) ^ 2 * Re2 ^ 2 / (k_inv * rho);
// Laminar
                k0 := 128 * length / (pi * diameter ^ 4);
                dm_flow_ddp_laminar := rho / (k0 * mu);
                dp_fric_laminar := sign(dp_fric) * pi * k0 * mu ^ 2 / rho * diameter / 4 * Re1;
                if abs(dp_fric) > abs(dp_fric_turbulent) then
                  m_flow := sign(dp_fric) * sqrt(rho * k_inv * abs(dp_fric));
                  dm_flow_ddp_fric := 0.5 * rho * k_inv * (rho * k_inv * abs(dp_fric)) ^ (-0.5);
                elseif abs(dp_fric) < abs(dp_fric_laminar) then
                  m_flow := dm_flow_ddp_laminar * dp_fric;
                  dm_flow_ddp_fric := dm_flow_ddp_laminar;
                else
                  (m_flow, dm_flow_ddp_fric) := Utilities.cubicHermite_withDerivative(dp_fric, dp_fric_laminar, dp_fric_turbulent, dm_flow_ddp_laminar * dp_fric_laminar, sign(dp_fric_turbulent) * sqrt(rho * k_inv * abs(dp_fric_turbulent)), dm_flow_ddp_laminar, if abs(dp_fric_turbulent) > 0 then 0.5 * rho * k_inv * (rho * k_inv * abs(dp_fric_turbulent)) ^ (-0.5) else Modelica.Constants.inf);
                end if;
// Preliminary testing seems to indicate that the log-log transform is not required here
                annotation(
                  smoothOrder = 1);
              end m_flow_of_dp_fric;

              function dp_fric_of_m_flow "Calculate pressure drop due to friction as function of mass flow rate"
                extends Modelica.Icons.Function;
                input SI.MassFlowRate m_flow "Mass flow rate from port_a to port_b";
                input SI.Density rho_a "Density at port_a";
                input SI.Density rho_b "Density at port_b";
                input SI.DynamicViscosity mu_a "Dynamic viscosity at port_a (dummy if use_mu = false)";
                input SI.DynamicViscosity mu_b "Dynamic viscosity at port_b (dummy if use_mu = false)";
                input SI.Length length "Length of pipe";
                input SI.Diameter diameter "Inner (hydraulic) diameter of pipe";
                input SI.Area crossArea "Inner cross section area";
                input SI.ReynoldsNumber Re1 "Boundary between laminar regime and transition";
                input SI.ReynoldsNumber Re2 "Boundary between transition and turbulent regime";
                input Real Delta "Relative roughness";
                output SI.Pressure dp_fric "Pressure loss due to friction (dp_fric = port_a.p - port_b.p - dp_grav)";
                output Real ddp_fric_dm_flow "Derivative of pressure drop with mass flow rate";
              protected
                SI.DynamicViscosity mu "Upstream viscosity";
                SI.Density rho "Upstream density";
                Real zeta;
                Real k0;
                Real k;
                Real ddp_fric_dm_flow_laminar "Derivative of dp_fric = f(m_flow) at zero";
                SI.MassFlowRate m_flow_turbulent "The turbulent region is: |m_flow| >= m_flow_turbulent";
                SI.MassFlowRate m_flow_laminar "The laminar region is: |m_flow| <= m_flow_laminar";
              algorithm
/*
Turbulent region:
   Re = m_flow*(4/pi)/(D_Re*mu)
   dp = 0.5*zeta*rho*v*|v|
      = 0.5*zeta*rho*1/(rho*A)^2 * m_flow * |m_flow|
      = 0.5*zeta/A^2 *1/rho * m_flow * |m_flow|
      = k/rho * m_flow * |m_flow|
   k  = 0.5*zeta/A^2
      = 0.5*zeta/(pi*(D/2)^2)^2
      = 8*zeta/(pi*D^2)^2
   m_flow_turbulent = (pi/4)*D_Re*mu*Re_turbulent

Laminar region:
   dp = 0.5*zeta/(A^2*d) * m_flow * |m_flow|
      = 0.5 * c0/(|m_flow|*(4/pi)/(D_Re*mu)) / ((pi*(D_Re/2)^2)^2*d) * m_flow*|m_flow|
      = 0.5 * c0*(pi/4)*(D_Re*mu) * 16/(pi^2*D_Re^4*d) * m_flow*|m_flow|
      = 2*c0/(pi*D_Re^3) * mu/rho * m_flow
      = k0 * mu/rho * m_flow
   k0 = 2*c0/(pi*D_Re^3)
*/
// Determine upstream density and upstream viscosity
                if m_flow >= 0 then
                  rho := rho_a;
                  mu := mu_a;
                else
                  rho := rho_b;
                  mu := mu_b;
                end if;
// Turbulent
// WARNING: The following equations are only correct for circular tubes!
                zeta := length / diameter / (2 * log10(3.7 / Delta)) ^ 2;
                k := 8 * zeta / (pi * diameter * diameter) ^ 2;
                m_flow_turbulent := sign(m_flow) * (pi / 4) * diameter * mu * Re2;
// Laminar
                k0 := 128 * length / (pi * diameter ^ 4);
                ddp_fric_dm_flow_laminar := k0 * mu / rho;
                m_flow_laminar := sign(m_flow) * (pi / 4) * diameter * mu * Re1;
                if abs(m_flow) > abs(m_flow_turbulent) then
                  dp_fric := k / rho * m_flow * abs(m_flow);
                  ddp_fric_dm_flow := 2 * k / rho * abs(m_flow);
                elseif abs(m_flow) < abs(m_flow_laminar) then
                  dp_fric := ddp_fric_dm_flow_laminar * m_flow;
                  ddp_fric_dm_flow := ddp_fric_dm_flow_laminar;
                else
                  (dp_fric, ddp_fric_dm_flow) := Utilities.cubicHermite_withDerivative(m_flow, m_flow_laminar, m_flow_turbulent, ddp_fric_dm_flow_laminar * m_flow_laminar, k / rho * m_flow_turbulent * abs(m_flow_turbulent), ddp_fric_dm_flow_laminar, 2 * k / rho * abs(m_flow_turbulent));
                end if;
// Preliminary testing seems to indicate that the log-log transform is not required here
                annotation(
                  smoothOrder = 1);
              end dp_fric_of_m_flow;
            end Internal;
            annotation(
              Documentation(info = "<html>
<p>
This component defines the quadratic turbulent regime of wall friction:
dp = k*m_flow*|m_flow|, where \"k\" depends on density and the roughness
of the pipe and is no longer a function of the Reynolds number.
This relationship is only valid for large Reynolds numbers.
At Re=4000, a polynomial is constructed that approaches
the constant &lambda; (for large Reynolds-numbers) at Re=4000
smoothly and has a derivative at zero mass flow rate that is
identical to laminar wall friction.
</p>
</html>"));
          end LaminarAndQuadraticTurbulent;

          package Detailed "Pipe wall friction for laminar and turbulent flow (detailed characteristic)"
            extends PartialWallFriction(final use_mu = true, final use_roughness = true, final use_dp_small = true, final use_m_flow_small = true, final use_Re_turbulent = true);
            import ln = Modelica.Math.log;
            import Modelica.Math.log10;
            import Modelica.Math.exp;

            redeclare function extends massFlowRate_dp "Return mass flow rate m_flow as function of pressure loss dp, i.e., m_flow = f(dp), due to wall friction"
                import Modelica.Math;

              protected
                Real Delta = roughness / diameter "Relative roughness";
                SI.ReynoldsNumber Re1 = min((745 * Math.exp(if Delta <= 0.0065 then 1 else 0.0065 / Delta)) ^ 0.97, Re_turbulent) "Re leaving laminar curve";
                SI.ReynoldsNumber Re2 = Re_turbulent "Re entering turbulent curve";
                SI.DynamicViscosity mu "Upstream viscosity";
                SI.Density rho "Upstream density";
                SI.ReynoldsNumber Re "Reynolds number";
                Real lambda2 "Modified friction coefficient (= lambda*Re^2)";

                function interpolateInRegion2
                  input Real Re_turbulent;
                  input SI.ReynoldsNumber Re1;
                  input SI.ReynoldsNumber Re2;
                  input Real Delta;
                  input Real lambda2;
                  output SI.ReynoldsNumber Re;
                  // point lg(lambda2(Re1)) with derivative at lg(Re1)
                protected
                  Real x1 = Math.log10(64 * Re1);
                  Real y1 = Math.log10(Re1);
                  Real yd1 = 1;
                  // Point lg(lambda2(Re2)) with derivative at lg(Re2)
                  Real aux1 = 0.5 / Math.log(10) * 5.74 * 0.9;
                  Real aux2 = Delta / 3.7 + 5.74 / Re2 ^ 0.9;
                  Real aux3 = Math.log10(aux2);
                  Real L2 = 0.25 * (Re2 / aux3) ^ 2;
                  Real aux4 = 2.51 / sqrt(L2) + 0.27 * Delta;
                  Real aux5 = -2 * sqrt(L2) * Math.log10(aux4);
                  Real x2 = Math.log10(L2);
                  Real y2 = Math.log10(aux5);
                  Real yd2 = 0.5 + 2.51 / Math.log(10) / (aux5 * aux4);
                  // Constants: Cubic polynomial between lg(Re1) and lg(Re2)
                  Real diff_x = x2 - x1;
                  Real m = (y2 - y1) / diff_x;
                  Real c2 = (3 * m - 2 * yd1 - yd2) / diff_x;
                  Real c3 = (yd1 + yd2 - 2 * m) / (diff_x * diff_x);
                  Real lambda2_1 = 64 * Re1;
                  Real dx;
                algorithm
                  dx := Math.log10(lambda2 / lambda2_1);
                  Re := Re1 * (lambda2 / lambda2_1) ^ (1 + dx * (c2 + dx * c3));
                  annotation(
                    smoothOrder = 1);
                end interpolateInRegion2;

              algorithm
// Determine upstream density, upstream viscosity, and lambda2
                rho := if dp >= 0 then rho_a else rho_b;
                mu := if dp >= 0 then mu_a else mu_b;
                lambda2 := abs(dp) * 2 * diameter ^ 3 * rho / (length * mu * mu);
// Determine Re under the assumption of laminar flow
                Re := lambda2 / 64;
// Modify Re, if turbulent flow
                if Re > Re1 then
                  Re := -2 * sqrt(lambda2) * Math.log10(2.51 / sqrt(lambda2) + 0.27 * Delta);
                  if Re < Re2 then
                    Re := interpolateInRegion2(Re, Re1, Re2, Delta, lambda2);
                  end if;
                end if;
// Determine mass flow rate
                m_flow := crossArea / diameter * mu * (if dp >= 0 then Re else -Re);
              annotation(
                smoothOrder = 1,
                Documentation(info = "<html>

</html>"));
            end massFlowRate_dp;

            redeclare function extends pressureLoss_m_flow "Return pressure loss dp as function of mass flow rate m_flow, i.e., dp = f(m_flow), due to wall friction"
                import Modelica.Math;
                import Modelica.Constants.pi;

              protected
                Real Delta = roughness / diameter "Relative roughness";
                SI.ReynoldsNumber Re1 = min(745 * Math.exp(if Delta <= 0.0065 then 1 else 0.0065 / Delta), Re_turbulent) "Re leaving laminar curve";
                SI.ReynoldsNumber Re2 = Re_turbulent "Re entering turbulent curve";
                SI.DynamicViscosity mu "Upstream viscosity";
                SI.Density rho "Upstream density";
                SI.ReynoldsNumber Re "Reynolds number";
                Real lambda2 "Modified friction coefficient (= lambda*Re^2)";

                function interpolateInRegion2
                  input SI.ReynoldsNumber Re;
                  input SI.ReynoldsNumber Re1;
                  input SI.ReynoldsNumber Re2;
                  input Real Delta;
                  output Real lambda2;
                  // point lg(lambda2(Re1)) with derivative at lg(Re1)
                protected
                  Real x1 = Math.log10(Re1);
                  Real y1 = Math.log10(64 * Re1);
                  Real yd1 = 1;
                  // Point lg(lambda2(Re2)) with derivative at lg(Re2)
                  Real aux1 = 0.5 / Math.log(10) * 5.74 * 0.9;
                  Real aux2 = Delta / 3.7 + 5.74 / Re2 ^ 0.9;
                  Real aux3 = Math.log10(aux2);
                  Real L2 = 0.25 * (Re2 / aux3) ^ 2;
                  Real aux4 = 2.51 / sqrt(L2) + 0.27 * Delta;
                  Real aux5 = -2 * sqrt(L2) * Math.log10(aux4);
                  Real x2 = Math.log10(Re2);
                  Real y2 = Math.log10(L2);
                  Real yd2 = 2 + 4 * aux1 / (aux2 * aux3 * Re2 ^ 0.9);
                  // Constants: Cubic polynomial between lg(Re1) and lg(Re2)
                  Real diff_x = x2 - x1;
                  Real m = (y2 - y1) / diff_x;
                  Real c2 = (3 * m - 2 * yd1 - yd2) / diff_x;
                  Real c3 = (yd1 + yd2 - 2 * m) / (diff_x * diff_x);
                  Real dx;
                algorithm
                  dx := Math.log10(Re / Re1);
                  lambda2 := 64 * Re1 * (Re / Re1) ^ (1 + dx * (c2 + dx * c3));
                  annotation(
                    smoothOrder = 1);
                end interpolateInRegion2;

              algorithm
// Determine upstream density and upstream viscosity
                rho := if m_flow >= 0 then rho_a else rho_b;
                mu := if m_flow >= 0 then mu_a else mu_b;
// Determine Re, lambda2 and pressure drop
                Re := diameter * abs(m_flow) / (crossArea * mu);
                lambda2 := if Re <= Re1 then 64 * Re else if Re >= Re2 then 0.25 * (Re / Math.log10(Delta / 3.7 + 5.74 / Re ^ 0.9)) ^ 2 else interpolateInRegion2(Re, Re1, Re2, Delta);
                dp := length * mu * mu / (2 * rho * diameter * diameter * diameter) * (if m_flow >= 0 then lambda2 else -lambda2);
              annotation(
                smoothOrder = 1,
                Documentation(info = "<html>

</html>"));
            end pressureLoss_m_flow;

            redeclare function extends massFlowRate_dp_staticHead "Return mass flow rate m_flow as function of pressure loss dp, i.e., m_flow = f(dp), due to wall friction and static head"
              protected
                Real Delta = roughness / diameter "Relative roughness";
                SI.ReynoldsNumber Re "Reynolds number";
                SI.ReynoldsNumber Re1 = min((745 * exp(if Delta <= 0.0065 then 1 else 0.0065 / Delta)) ^ 0.97, Re_turbulent) "Boundary between laminar regime and transition";
                SI.ReynoldsNumber Re2 = Re_turbulent "Boundary between transition and turbulent regime";
                SI.Pressure dp_a "Upper end of regularization domain of the m_flow(dp) relation";
                SI.Pressure dp_b "Lower end of regularization domain of the m_flow(dp) relation";
                SI.MassFlowRate m_flow_a "Value at upper end of regularization domain";
                SI.MassFlowRate m_flow_b "Value at lower end of regularization domain";
                SI.MassFlowRate dm_flow_ddp_fric_a "Derivative at upper end of regularization domain";
                SI.MassFlowRate dm_flow_ddp_fric_b "Derivative at lower end of regularization domain";
                SI.Pressure dp_grav_a = g_times_height_ab * rho_a "Static head if mass flows in design direction (a to b)";
                SI.Pressure dp_grav_b = g_times_height_ab * rho_b "Static head if mass flows against design direction (b to a)";
                // Properly define zero mass flow conditions
                SI.MassFlowRate m_flow_zero = 0;
                SI.Pressure dp_zero = (dp_grav_a + dp_grav_b) / 2;
                Real dm_flow_ddp_fric_zero;

              algorithm
                dp_a := max(dp_grav_a, dp_grav_b) + dp_small;
                dp_b := min(dp_grav_a, dp_grav_b) - dp_small;
                if dp >= dp_a then
                  m_flow := Internal.m_flow_of_dp_fric(dp - dp_grav_a, rho_a, rho_b, mu_a, mu_b, length, diameter, crossArea, Re1, Re2, Delta);
                elseif dp <= dp_b then
                  m_flow := Internal.m_flow_of_dp_fric(dp - dp_grav_b, rho_a, rho_b, mu_a, mu_b, length, diameter, crossArea, Re1, Re2, Delta);
                else
                  (m_flow_a, dm_flow_ddp_fric_a) := Internal.m_flow_of_dp_fric(dp_a - dp_grav_a, rho_a, rho_b, mu_a, mu_b, length, diameter, crossArea, Re1, Re2, Delta);
                  (m_flow_b, dm_flow_ddp_fric_b) := Internal.m_flow_of_dp_fric(dp_b - dp_grav_b, rho_a, rho_b, mu_a, mu_b, length, diameter, crossArea, Re1, Re2, Delta);
                  (m_flow, dm_flow_ddp_fric_zero) := Utilities.regFun3(dp_zero, dp_b, dp_a, m_flow_b, m_flow_a, dm_flow_ddp_fric_b, dm_flow_ddp_fric_a);
                  if dp > dp_zero then
                    m_flow := Utilities.regFun3(dp, dp_zero, dp_a, m_flow_zero, m_flow_a, dm_flow_ddp_fric_zero, dm_flow_ddp_fric_a);
                  else
                    m_flow := Utilities.regFun3(dp, dp_b, dp_zero, m_flow_b, m_flow_zero, dm_flow_ddp_fric_b, dm_flow_ddp_fric_zero);
                  end if;
                end if;
// Positive flow outside regularization
// Negative flow outside regularization
// Regularization parameters
// Include a properly defined zero mass flow point
// Obtain a suitable slope from the linear section slope c (value of m_flow is overwritten later)
// Do regularization
              annotation(
                smoothOrder = 1);
            end massFlowRate_dp_staticHead;

            redeclare function extends pressureLoss_m_flow_staticHead "Return pressure loss dp as function of mass flow rate m_flow, i.e., dp = f(m_flow), due to wall friction and static head"
              protected
                Real Delta = roughness / diameter "Relative roughness";
                SI.ReynoldsNumber Re1 = min(745 * exp(if Delta <= 0.0065 then 1 else 0.0065 / Delta), Re_turbulent) "Boundary between laminar regime and transition";
                SI.ReynoldsNumber Re2 = Re_turbulent "Boundary between transition and turbulent regime";
                SI.MassFlowRate m_flow_a "Upper end of regularization domain of the dp(m_flow) relation";
                SI.MassFlowRate m_flow_b "Lower end of regularization domain of the dp(m_flow) relation";
                SI.Pressure dp_a "Value at upper end of regularization domain";
                SI.Pressure dp_b "Value at lower end of regularization domain";
                SI.Pressure dp_grav_a = g_times_height_ab * rho_a "Static head if mass flows in design direction (a to b)";
                SI.Pressure dp_grav_b = g_times_height_ab * rho_b "Static head if mass flows against design direction (b to a)";
                Real ddp_dm_flow_a "Derivative of pressure drop with mass flow rate at m_flow_a";
                Real ddp_dm_flow_b "Derivative of pressure drop with mass flow rate at m_flow_b";
                // Properly define zero mass flow conditions
                SI.MassFlowRate m_flow_zero = 0;
                SI.Pressure dp_zero = (dp_grav_a + dp_grav_b) / 2;
                Real ddp_dm_flow_zero;

              algorithm
                m_flow_a := if dp_grav_a < dp_grav_b then Internal.m_flow_of_dp_fric(dp_grav_b - dp_grav_a, rho_a, rho_b, mu_a, mu_b, length, diameter, crossArea, Re1, Re2, Delta) + m_flow_small else m_flow_small;
                m_flow_b := if dp_grav_a < dp_grav_b then Internal.m_flow_of_dp_fric(dp_grav_a - dp_grav_b, rho_a, rho_b, mu_a, mu_b, length, diameter, crossArea, Re1, Re2, Delta) - m_flow_small else -m_flow_small;
                if m_flow >= m_flow_a then
                  dp := Internal.dp_fric_of_m_flow(m_flow, rho_a, rho_b, mu_a, mu_b, length, diameter, crossArea, Re1, Re2, Delta) + dp_grav_a;
                elseif m_flow <= m_flow_b then
                  dp := Internal.dp_fric_of_m_flow(m_flow, rho_a, rho_b, mu_a, mu_b, length, diameter, crossArea, Re1, Re2, Delta) + dp_grav_b;
                else
                  (dp_a, ddp_dm_flow_a) := Internal.dp_fric_of_m_flow(m_flow_a, rho_a, rho_b, mu_a, mu_b, length, diameter, crossArea, Re1, Re2, Delta);
                  dp_a := dp_a + dp_grav_a "Adding dp_grav to dp_fric to get dp";
                  (dp_b, ddp_dm_flow_b) := Internal.dp_fric_of_m_flow(m_flow_b, rho_a, rho_b, mu_a, mu_b, length, diameter, crossArea, Re1, Re2, Delta);
                  dp_b := dp_b + dp_grav_b "Adding dp_grav to dp_fric to get dp";
                  (dp, ddp_dm_flow_zero) := Utilities.regFun3(m_flow_zero, m_flow_b, m_flow_a, dp_b, dp_a, ddp_dm_flow_b, ddp_dm_flow_a);
                  if m_flow > m_flow_zero then
                    dp := Utilities.regFun3(m_flow, m_flow_zero, m_flow_a, dp_zero, dp_a, ddp_dm_flow_zero, ddp_dm_flow_a);
                  else
                    dp := Utilities.regFun3(m_flow, m_flow_b, m_flow_zero, dp_b, dp_zero, ddp_dm_flow_b, ddp_dm_flow_zero);
                  end if;
                end if;
// Positive flow outside regularization
// Negative flow outside regularization
// Regularization parameters
// Include a properly defined zero mass flow point
// Obtain a suitable slope from the linear section slope c (value of dp is overwritten later)
// Do regularization
              annotation(
                smoothOrder = 1);
            end pressureLoss_m_flow_staticHead;

            package Internal "Functions to calculate mass flow rate from friction pressure drop and vice versa"
              extends Modelica.Icons.InternalPackage;

              function m_flow_of_dp_fric "Calculate mass flow rate as function of pressure drop due to friction"
                extends Modelica.Icons.Function;
                input SI.Pressure dp_fric "Pressure loss due to friction (dp = port_a.p - port_b.p)";
                input SI.Density rho_a "Density at port_a";
                input SI.Density rho_b "Density at port_b";
                input SI.DynamicViscosity mu_a "Dynamic viscosity at port_a (dummy if use_mu = false)";
                input SI.DynamicViscosity mu_b "Dynamic viscosity at port_b (dummy if use_mu = false)";
                input SI.Length length "Length of pipe";
                input SI.Diameter diameter "Inner (hydraulic) diameter of pipe";
                input SI.Area crossArea "Inner cross section area";
                input SI.ReynoldsNumber Re1 "Boundary between laminar regime and transition";
                input SI.ReynoldsNumber Re2 "Boundary between transition and turbulent regime";
                input Real Delta "Relative roughness";
                output SI.MassFlowRate m_flow "Mass flow rate from port_a to port_b";
                output Real dm_flow_ddp_fric "Derivative of mass flow rate with dp_fric";
              protected
                function interpolateInRegion2_withDerivative "Interpolation in log-log space using a cubic Hermite polynomial, where x=log10(lambda2), y=log10(Re)"
                  input Real lambda2 "Known independent variable";
                  input SI.ReynoldsNumber Re1 "Boundary between laminar regime and transition";
                  input SI.ReynoldsNumber Re2 "Boundary between transition and turbulent regime";
                  input Real Delta "Relative roughness";
                  input SI.Pressure dp_fric "Pressure loss due to friction (dp = port_a.p - port_b.p)";
                  output SI.ReynoldsNumber Re "Unknown return variable";
                  output Real dRe_ddp "Derivative of return value";
                  // point lg(lambda2(Re1)) with derivative at lg(Re1)
                protected
                  Real x1 = log10(64 * Re1);
                  Real y1 = log10(Re1);
                  Real y1d = 1;
                  // Point lg(lambda2(Re2)) with derivative at lg(Re2)
                  Real aux2 = Delta / 3.7 + 5.74 / Re2 ^ 0.9;
                  Real aux3 = log10(aux2);
                  Real L2 = 0.25 * (Re2 / aux3) ^ 2;
                  Real aux4 = 2.51 / sqrt(L2) + 0.27 * Delta;
                  Real aux5 = -2 * sqrt(L2) * log10(aux4);
                  Real x2 = log10(L2);
                  Real y2 = log10(aux5);
                  Real y2d = 0.5 + 2.51 / log(10) / (aux5 * aux4);
                  // Point of interest in transformed space
                  Real x = log10(lambda2);
                  Real y;
                  Real dy_dx "Derivative in transformed space";
                algorithm
// Interpolation
                  (y, dy_dx) := Utilities.cubicHermite_withDerivative(x, x1, x2, y1, y2, y1d, y2d);
// Return value
                  Re := 10 ^ y;
// Derivative of return value
                  dRe_ddp := Re / abs(dp_fric) * dy_dx;
                  annotation(
                    smoothOrder = 1);
                end interpolateInRegion2_withDerivative;

                SI.DynamicViscosity mu "Upstream viscosity";
                SI.Density rho "Upstream density";
                Real lambda2 "Modified friction coefficient (= lambda*Re^2)";
                SI.ReynoldsNumber Re "Reynolds number";
                Real dRe_ddp "dRe/ddp";
                Real aux1;
                Real aux2;
              algorithm
// Determine upstream density and upstream viscosity
                if dp_fric >= 0 then
                  rho := rho_a;
                  mu := mu_a;
                else
                  rho := rho_b;
                  mu := mu_b;
                end if;
// Positive mass flow rate
                lambda2 := abs(dp_fric) * 2 * diameter ^ 3 * rho / (length * mu * mu) "Known as lambda2=f(dp)";
                aux1 := 2 * diameter ^ 3 * rho / (length * mu ^ 2);
// Determine Re and dRe/ddp under the assumption of laminar flow
                Re := lambda2 / 64 "Hagen-Poiseuille";
                dRe_ddp := aux1 / 64 "Hagen-Poiseuille";
// Modify Re, if turbulent flow
                if Re > Re1 then
                  Re := -2 * sqrt(lambda2) * log10(2.51 / sqrt(lambda2) + 0.27 * Delta) "Colebrook-White";
                  aux2 := sqrt(aux1 * abs(dp_fric));
                  dRe_ddp := 1 / log(10) * ((-2 * log(2.51 / aux2 + 0.27 * Delta) * aux1 / (2 * aux2)) + 2 * 2.51 / (2 * abs(dp_fric) * (2.51 / aux2 + 0.27 * Delta)));
                  if Re < Re2 then
                    (Re, dRe_ddp) := interpolateInRegion2_withDerivative(lambda2, Re1, Re2, Delta, dp_fric);
                  end if;
                end if;
// Determine mass flow rate
                m_flow := crossArea / diameter * mu * (if dp_fric >= 0 then Re else -Re);
// Determine derivative of mass flow rate with dp_fric
                dm_flow_ddp_fric := crossArea / diameter * mu * dRe_ddp;
                annotation(
                  smoothOrder = 1);
              end m_flow_of_dp_fric;

              function dp_fric_of_m_flow "Calculate pressure drop due to friction as function of mass flow rate"
                extends Modelica.Icons.Function;
                input SI.MassFlowRate m_flow "Mass flow rate from port_a to port_b";
                input SI.Density rho_a "Density at port_a";
                input SI.Density rho_b "Density at port_b";
                input SI.DynamicViscosity mu_a "Dynamic viscosity at port_a (dummy if use_mu = false)";
                input SI.DynamicViscosity mu_b "Dynamic viscosity at port_b (dummy if use_mu = false)";
                input SI.Length length "Length of pipe";
                input SI.Diameter diameter "Inner (hydraulic) diameter of pipe";
                input SI.Area crossArea "Inner cross section area";
                input SI.ReynoldsNumber Re1 "Boundary between laminar regime and transition";
                input SI.ReynoldsNumber Re2 "Boundary between transition and turbulent regime";
                input Real Delta "Relative roughness";
                output SI.Pressure dp_fric "Pressure loss due to friction (dp_fric = port_a.p - port_b.p - dp_grav)";
                output Real ddp_fric_dm_flow "Derivative of pressure drop with mass flow rate";
              protected
                function interpolateInRegion2 "Interpolation in log-log space using a cubic Hermite polynomial, where x=log10(Re), y=log10(lambda2)"
                  input SI.ReynoldsNumber Re "Known independent variable";
                  input SI.ReynoldsNumber Re1 "Boundary between laminar regime and transition";
                  input SI.ReynoldsNumber Re2 "Boundary between transition and turbulent regime";
                  input Real Delta "Relative roughness";
                  input SI.MassFlowRate m_flow "Mass flow rate from port_a to port_b";
                  output Real lambda2 "Unknown return value";
                  output Real dlambda2_dm_flow "Derivative of return value";
                  // point lg(lambda2(Re1)) with derivative at lg(Re1)
                protected
                  Real x1 = log10(Re1);
                  Real y1 = log10(64 * Re1);
                  Real y1d = 1;
                  // Point lg(lambda2(Re2)) with derivative at lg(Re2)
                  Real aux2 = Delta / 3.7 + 5.74 / Re2 ^ 0.9;
                  Real aux3 = log10(aux2);
                  Real L2 = 0.25 * (Re2 / aux3) ^ 2;
                  Real x2 = log10(Re2);
                  Real y2 = log10(L2);
                  Real y2d = 2 + 2 * 5.74 * 0.9 / (log(aux2) * Re2 ^ 0.9 * aux2);
                  // Point of interest in transformed space
                  Real x = log10(Re);
                  Real y;
                  Real dy_dx "Derivative in transformed space";
                algorithm
// Interpolation
                  (y, dy_dx) := Utilities.cubicHermite_withDerivative(x, x1, x2, y1, y2, y1d, y2d);
// Return value
                  lambda2 := 10 ^ y;
// Derivative of return value
                  dlambda2_dm_flow := lambda2 / abs(m_flow) * dy_dx;
                  annotation(
                    smoothOrder = 1);
                end interpolateInRegion2;

                SI.DynamicViscosity mu "Upstream viscosity";
                SI.Density rho "Upstream density";
                SI.ReynoldsNumber Re "Reynolds number";
                Real lambda2 "Modified friction coefficient (= lambda*Re^2)";
                Real dlambda2_dm_flow "dlambda2/dm_flow";
                Real aux1;
                Real aux2;
              algorithm
// Determine upstream density and upstream viscosity
                if m_flow >= 0 then
                  rho := rho_a;
                  mu := mu_a;
                else
                  rho := rho_b;
                  mu := mu_b;
                end if;
// Determine Reynolds number
                Re := abs(m_flow) * diameter / (crossArea * mu);
                aux1 := diameter / (crossArea * mu);
// Use correlation for lambda2 depending on actual conditions
                if Re <= Re1 then
                  lambda2 := 64 * Re "Hagen-Poiseuille";
                  dlambda2_dm_flow := 64 * aux1 "Hagen-Poiseuille";
                elseif Re >= Re2 then
                  lambda2 := 0.25 * (Re / log10(Delta / 3.7 + 5.74 / Re ^ 0.9)) ^ 2 "Swamee-Jain";
                  aux2 := Delta / 3.7 + 5.74 / (aux1 * abs(m_flow)) ^ 0.9;
                  dlambda2_dm_flow := 0.5 * aux1 * Re * log(10) ^ 2 * (1 / log(aux2) ^ 2 + 5.74 * 0.9 / (log(aux2) ^ 3 * Re ^ 0.9 * aux2)) "Swamee-Jain";
                else
                  (lambda2, dlambda2_dm_flow) := interpolateInRegion2(Re, Re1, Re2, Delta, m_flow);
                end if;
// Compute pressure drop from lambda2
                dp_fric := length * mu * mu / (2 * rho * diameter * diameter * diameter) * (if m_flow >= 0 then lambda2 else -lambda2);
// Compute derivative from dlambda2/dm_flow
                ddp_fric_dm_flow := length * mu ^ 2 / (2 * diameter ^ 3 * rho) * dlambda2_dm_flow;
                annotation(
                  smoothOrder = 1);
              end dp_fric_of_m_flow;
            end Internal;
            annotation(
              Documentation(info = "<html>
<p>
This component defines the complete regime of wall friction.
The details are described in the
<a href=\"modelica://Modelica.Fluid.UsersGuide.ComponentDefinition.WallFriction\">UsersGuide</a>.
The functional relationship of the friction loss factor &lambda; is
displayed in the next figure. Function massFlowRate_dp() defines the \"red curve\"
(\"Swamee and Jain\"), where as function pressureLoss_m_flow() defines the
\"blue curve\" (\"Colebrook-White\"). The two functions are inverses from
each other and give slightly different results in the transition region
between Re = 1500 .. 4000, in order to get explicit equations without
solving a non-linear equation.
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Fluid/Components/PipeFriction1.png\"
     alt=\"PipeFriction1.png\">
</p>

<p>
Additionally to wall friction, this component properly implements static
head. With respect to the latter, two cases can be distinguished. In the case
shown next, the change of elevation with the path from a to b has the opposite
sign of the change of density.</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Fluid/Components/PipeFrictionStaticHead_case-a.png\"
     alt=\"PipeFrictionStaticHead_case-a.png\">
</p>

<p>
In the case illustrated second, the change of elevation with the path from a to
b has the same sign of the change of density.</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Fluid/Components/PipeFrictionStaticHead_case-b.png\"
     alt=\"PipeFrictionStaticHead_case-b.png\">
</p>

</html>"));
          end Detailed;

          model TestWallFrictionAndGravity "Pressure loss in pipe due to wall friction and gravity (only for test purposes; if needed use Pipes.StaticPipe instead)"
            extends Modelica.Fluid.Interfaces.PartialTwoPortTransport(dp_start = dp_fric_nominal + system.g * height_ab * rho_nominal, m_flow_small = if system.use_eps_Re then system.eps_m_flow * m_flow_nominal else system.m_flow_small);
            replaceable package WallFriction = Modelica.Fluid.Pipes.BaseClasses.WallFriction.QuadraticTurbulent constrainedby Modelica.Fluid.Pipes.BaseClasses.WallFriction.PartialWallFriction "Characteristic of wall friction" annotation(
               choicesAllMatching = true);
            parameter SI.Length length "Length of pipe";
            parameter SI.Diameter diameter "Inner (hydraulic) diameter of pipe";
            parameter SI.Area crossArea = Modelica.Constants.pi * diameter * diameter / 4 "Inner cross section area";
            parameter SI.Length height_ab = 0.0 "Height(port_b) - Height(port_a)" annotation(
              Evaluate = true);
            parameter SI.Length roughness(min = 0) = 2.5e-5 "Absolute roughness of pipe (default = smooth steel pipe)" annotation(
              Dialog(enable = WallFriction.use_roughness));
            parameter Modelica.SIunits.MassFlowRate m_flow_nominal = if system.use_eps_Re then system.m_flow_nominal else 1e2 * system.m_flow_small "Nominal mass flow rate" annotation(
              Dialog(group = "Nominal operating point"));
            parameter Boolean use_nominal = false "= true, if mu_nominal and rho_nominal are used, otherwise computed from medium" annotation(
              Evaluate = true);
            parameter SI.DynamicViscosity mu_nominal = Medium.dynamicViscosity(Medium.setState_pTX(Medium.p_default, Medium.T_default, Medium.X_default)) "Nominal dynamic viscosity (e.g., mu_liquidWater = 1e-3, mu_air = 1.8e-5)" annotation(
              Dialog(enable = use_nominal));
            parameter SI.Density rho_nominal = Medium.density_pTX(Medium.p_default, Medium.T_default, Medium.X_default) "Nominal density (e.g., rho_liquidWater = 995, rho_air = 1.2)" annotation(
              Dialog(enable = use_nominal));
            parameter Boolean show_Re = false "= true, if Reynolds number is included for plotting" annotation(
              Evaluate = true,
              Dialog(tab = "Advanced"));
            parameter Boolean from_dp = true "= true, use m_flow = f(dp), otherwise dp = f(m_flow)" annotation(
              Evaluate = true,
              Dialog(tab = "Advanced"));
          protected
            parameter Medium.AbsolutePressure dp_fric_nominal(min = 0) = WallFriction.pressureLoss_m_flow(m_flow_nominal, rho_nominal, rho_nominal, mu_nominal, mu_nominal, length, diameter, crossArea, roughness, m_flow_small) "Nominal pressure loss (excluding static head)";
            parameter Medium.AbsolutePressure dp_small(min = 0) = if system.use_eps_Re then dp_fric_nominal / m_flow_nominal * m_flow_small else system.dp_small "Within regularization if |dp| < dp_small (may be wider for large discontinuities in static head)" annotation(
              Dialog(tab = "Advanced", enable = from_dp and WallFriction.use_dp_small));
          public
            SI.ReynoldsNumber Re = Modelica.Fluid.Pipes.BaseClasses.CharacteristicNumbers.ReynoldsNumber_m_flow(m_flow, noEvent(if m_flow > 0 then mu_a else mu_b), diameter) if show_Re "Reynolds number of pipe flow";
          protected
            SI.DynamicViscosity mu_a = if not WallFriction.use_mu then 1.e-10 else if use_nominal then mu_nominal else Medium.dynamicViscosity(state_a);
            SI.DynamicViscosity mu_b = if not WallFriction.use_mu then 1.e-10 else if use_nominal then mu_nominal else Medium.dynamicViscosity(state_b);
            SI.Density rho_a = if use_nominal then rho_nominal else Medium.density(state_a);
            SI.Density rho_b = if use_nominal then rho_nominal else Medium.density(state_b);
            Real g_times_height_ab(final unit = "m2/s2") = system.g * height_ab "Gravity times height_ab = dp_grav/d";
            // Currently not in use (means to widen the regularization domain in case of large difference in static head)
            final parameter Boolean use_x_small_staticHead = false "Use dp_/m_flow_small_staticHead only if static head actually exists" annotation(
              Evaluate = true);
            /*abs(height_ab)>0*/
            SI.AbsolutePressure dp_small_staticHead = noEvent(max(dp_small, 0.015 * abs(g_times_height_ab * (rho_a - rho_b)))) "Heuristic for large discontinuities in static head";
            SI.MassFlowRate m_flow_small_staticHead = noEvent(max(m_flow_small, ((-5.55e-7 * (rho_a + rho_b) / 2) + 5.5e-4) * abs(g_times_height_ab * (rho_a - rho_b)))) "Heuristic for large discontinuities in static head";
          equation
            if from_dp and not WallFriction.dp_is_zero then
              m_flow = WallFriction.massFlowRate_dp_staticHead(dp, rho_a, rho_b, mu_a, mu_b, length, diameter, g_times_height_ab, crossArea, roughness, if use_x_small_staticHead then dp_small_staticHead else dp_small);
            else
              dp = WallFriction.pressureLoss_m_flow_staticHead(m_flow, rho_a, rho_b, mu_a, mu_b, length, diameter, g_times_height_ab, crossArea, roughness, if use_x_small_staticHead then m_flow_small_staticHead else m_flow_small);
            end if;
// Energy balance, considering change of potential energy
            port_a.h_outflow = inStream(port_b.h_outflow) + system.g * height_ab;
            port_b.h_outflow = inStream(port_a.h_outflow) - system.g * height_ab;
            annotation(
              defaultComponentName = "pipeFriction",
              Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, 60}, {100, -60}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192, 192, 192}), Rectangle(extent = {{-100, 44}, {100, -45}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {0, 127, 255}), Text(extent = {{-150, 80}, {150, 120}}, lineColor = {0, 0, 255}, textString = "%name")}),
              Documentation(info = "<html>
<p>
This model describes pressure losses due to <b>wall friction</b> in a pipe
and due to gravity.
It is assumed that no mass or energy is stored in the pipe.
Correlations of different complexity and validity can be
selected via the replaceable package <b>WallFriction</b> (see parameter menu below).
The details of the pipe wall friction model are described in the
<a href=\"modelica://Modelica.Fluid.UsersGuide.ComponentDefinition.WallFriction\">UsersGuide</a>.
Basically, different variants of the equation
</p>

<pre>
   dp = &lambda;(Re,<font face=\"Symbol\">D</font>)*(L/D)*&rho;*v*|v|/2
</pre>

<p>
are used, where the friction loss factor &lambda; is shown
in the next figure:
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Fluid/Components/PipeFriction1.png\"
     alt=\"PipeFriction1.png\">
</p>

<p>
By default, the correlations are computed with media data
at the actual time instant.
In order to reduce non-linear equation systems, parameter
<b>use_nominal</b> provides the option
to compute the correlations with constant media values
at the desired operating point. This might speed-up the
simulation and/or might give a more robust simulation.
</p>
</html>"),
              Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, 64}, {100, -64}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Backward), Rectangle(extent = {{-100, 50}, {100, -49}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{-60, -49}, {-60, 50}}, color = {0, 0, 255}, arrow = {Arrow.Filled, Arrow.Filled}), Text(extent = {{-50, 16}, {6, -10}}, lineColor = {0, 0, 255}, textString = "diameter"), Line(points = {{-100, 74}, {100, 74}}, color = {0, 0, 255}, arrow = {Arrow.Filled, Arrow.Filled}), Text(extent = {{-34, 92}, {34, 74}}, lineColor = {0, 0, 255}, textString = "length")}));
          end TestWallFrictionAndGravity;
          annotation(
            Documentation(info = "<html>
<p>
This package provides functions to compute
pressure losses due to <b>wall friction</b> in a pipe.
Every correlation is defined by a package that is derived
by inheritance from the package WallFriction.PartialWallFriction.
The details of the underlying pipe wall friction model are described in the
<a href=\"modelica://Modelica.Fluid.UsersGuide.ComponentDefinition.WallFriction\">UsersGuide</a>.
Basically, different variants of the equation
</p>

<pre>
   dp = &lambda;(Re,<font face=\"Symbol\">D</font>)*(L/D)*&rho;*v*|v|/2
</pre>

<p>
are used, where the friction loss factor &lambda; is shown
in the next figure:
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Fluid/Components/PipeFriction1.png\"
     alt=\"PipeFriction1.png\">
</p>

</html>"));
        end WallFriction;
      end BaseClasses;
      annotation(
        Documentation(info = "<html>

</html>"));
    end Pipes;

    package Machines "Devices for converting between energy held in a fluid and mechanical energy"
      extends Modelica.Icons.VariantsPackage;

      model SweptVolume "varying cylindric volume depending on the position of the piston"
        import Modelica.Constants.pi;
        parameter SI.Area pistonCrossArea "Cross sectional area of piston";
        parameter SI.Volume clearance "Remaining volume at zero piston stroke";
        SI.Volume V "fluid volume";
        // Mass and energy balance, ports
        extends Modelica.Fluid.Vessels.BaseClasses.PartialLumpedVessel(final fluidVolume = V, heatTransfer(surfaceAreas = {pistonCrossArea + 2 * sqrt(pistonCrossArea * pi) * (flange.s + clearance / pistonCrossArea)}));
        Modelica.Mechanics.Translational.Interfaces.Flange_b flange "translation flange for piston" annotation(
          Placement(transformation(extent = {{-10, 90}, {10, 110}})));
      equation
        assert(flange.s >= 0, "Piston stroke (given by flange.s) must not be smaller than zero!");
// volume size
        V = clearance + flange.s * pistonCrossArea;
        0 = flange.f + (medium.p - system.p_ambient) * pistonCrossArea;
// energy balances
        Wb_flow = medium.p * pistonCrossArea * (-der(flange.s));
// definition of port pressures
        for i in 1:nPorts loop
          vessel_ps_static[i] = medium.p;
        end for;
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-50, 36}, {50, -90}}, lineColor = {0, 0, 255}, pattern = LinePattern.None, lineThickness = 1, fillColor = {170, 213, 255}, fillPattern = FillPattern.Solid), Polygon(points = {{-52, 62}, {-48, 62}, {-48, -30}, {-52, -30}, {-52, 62}}, lineColor = {95, 95, 95}, fillColor = {135, 135, 135}, fillPattern = FillPattern.Backward), Polygon(points = {{48, 60}, {52, 60}, {52, -34}, {48, -34}, {48, 60}}, lineColor = {95, 95, 95}, fillColor = {135, 135, 135}, fillPattern = FillPattern.Backward), Rectangle(extent = {{-48, 40}, {48, 30}}, lineColor = {95, 95, 95}, fillColor = {135, 135, 135}, fillPattern = FillPattern.Forward), Rectangle(extent = {{-6, 92}, {6, 40}}, lineColor = {95, 95, 95}, fillColor = {135, 135, 135}, fillPattern = FillPattern.Forward), Polygon(points = {{-48, -90}, {48, -90}, {48, 70}, {52, 70}, {52, -94}, {-52, -94}, {-52, 70}, {-48, 70}, {-48, -90}}, lineColor = {95, 95, 95}, fillColor = {135, 135, 135}, fillPattern = FillPattern.Backward), Line(visible = use_HeatTransfer, points = {{-100, 0}, {-52, 0}}, color = {198, 0, 0})}),
          Documentation(info = "<html>
<p> Mixing volume with varying size. The size of the volume is given by:</p>
<ul>
  <li>cross sectional piston area</li>
  <li>piston stroke given by the flange position s</li>
  <li>clearance (volume at flange position = 0)</li>
</ul>
<p>Losses are neglected. The shaft power is completely converted into mechanical work on the fluid.</p>

<p> The flange position has to be equal or greater than zero. Otherwise the simulation stops. The force of the flange results from the pressure difference between medium and ambient pressure and the cross sectional piston area. For using the component, a top level instance of the ambient model with the inner attribute is needed.</p>
<p> The pressure at both fluid ports equals the medium pressure in the volume. No suction nor discharge valve is included in the model.</p>
<p>The thermal port is directly connected to the medium. The temperature of the thermal port equals the medium temperature. The heat capacity of the cylinder and the piston are not includes in the model.</p>
</html>", revisions = "<html>
<ul>
<li><i>29 Oct 2007</i>
    by Carsten Heinrich:<br>
       Model added to the Fluid library</li>
</ul>
</html>"));
      end SweptVolume;

      model Pump "Centrifugal pump with mechanical connector for the shaft"
        extends Modelica.Fluid.Machines.BaseClasses.PartialPump;
        SI.Angle phi "Shaft angle";
        SI.AngularVelocity omega "Shaft angular velocity";
        Modelica.Mechanics.Rotational.Interfaces.Flange_a shaft annotation(
          Placement(transformation(extent = {{-10, 90}, {10, 110}})));
      equation
        phi = shaft.phi;
        omega = der(phi);
        N = Modelica.SIunits.Conversions.to_rpm(omega);
        W_single = omega * shaft.tau;
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-10, 100}, {10, 78}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.VerticalCylinder, fillColor = {95, 95, 95})}),
          Documentation(info = "<html>
<p>This model describes a centrifugal pump (or a group of <code>nParallel</code> pumps) with a mechanical rotational connector for the shaft, to be used when the pump drive has to be modelled explicitly. In the case of <code>nParallel</code> pumps, the mechanical connector is relative to a single pump.
<p>The model extends <code>PartialPump</code>
 </html>", revisions = "<html>
<ul>
<li><i>31 Oct 2005</i>
    by <a href=\"mailto:francesco.casella@polimi.it\">Francesco Casella</a>:<br>
       Model added to the Fluid library</li>
</ul>
</html>"));
      end Pump;

      model ControlledPump "Centrifugal pump with ideally controlled mass flow rate"
        import Modelica.SIunits.Conversions.NonSIunits.AngularVelocity_rpm;
        extends Modelica.Fluid.Machines.BaseClasses.PartialPump(N_nominal = 1500, N(start = N_nominal), redeclare replaceable function flowCharacteristic = Modelica.Fluid.Machines.BaseClasses.PumpCharacteristics.quadraticFlow(V_flow_nominal = {0, V_flow_op, 1.5 * V_flow_op}, head_nominal = {2 * head_op, head_op, 0}));
        // nominal values
        parameter Medium.AbsolutePressure p_a_nominal "Nominal inlet pressure for predefined pump characteristics";
        parameter Medium.AbsolutePressure p_b_nominal "Nominal outlet pressure, fixed if not control_m_flow and not use_p_set";
        parameter Medium.MassFlowRate m_flow_nominal "Nominal mass flow rate, fixed if control_m_flow and not use_m_flow_set";
        // what to control
        parameter Boolean control_m_flow = true "= false to control outlet pressure port_b.p instead of m_flow" annotation(
          Evaluate = true);
        parameter Boolean use_m_flow_set = false "= true to use input signal m_flow_set instead of m_flow_nominal" annotation(
          Dialog(enable = control_m_flow));
        parameter Boolean use_p_set = false "= true to use input signal p_set instead of p_b_nominal" annotation(
          Dialog(enable = not control_m_flow));
        // exemplary characteristics
        final parameter SI.VolumeFlowRate V_flow_op = m_flow_nominal / rho_nominal "operational volume flow rate according to nominal values";
        final parameter SI.Height head_op = (p_b_nominal - p_a_nominal) / (rho_nominal * g) "operational pump head according to nominal values";
        Modelica.Blocks.Interfaces.RealInput m_flow_set if use_m_flow_set "Prescribed mass flow rate" annotation(
          Placement(transformation(extent = {{-20, -20}, {20, 20}}, rotation = -90, origin = {-50, 82})));
        Modelica.Blocks.Interfaces.RealInput p_set if use_p_set "Prescribed outlet pressure" annotation(
          Placement(transformation(extent = {{-20, -20}, {20, 20}}, rotation = -90, origin = {50, 82})));
      protected
        Modelica.Blocks.Interfaces.RealInput m_flow_set_internal "Needed to connect to conditional connector";
        Modelica.Blocks.Interfaces.RealInput p_set_internal "Needed to connect to conditional connector";
      equation
// Ideal control
        if control_m_flow then
          m_flow = m_flow_set_internal;
        else
          dp_pump = p_set_internal - port_a.p;
        end if;
// Internal connector value when use_m_flow_set = false
        if not use_m_flow_set then
          m_flow_set_internal = m_flow_nominal;
        end if;
        if not use_p_set then
          p_set_internal = p_b_nominal;
        end if;
        connect(m_flow_set, m_flow_set_internal);
        connect(p_set, p_set_internal);
        annotation(
          defaultComponentName = "pump",
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(visible = use_p_set, extent = {{82, 108}, {176, 92}}, textString = "p_set"), Text(visible = use_m_flow_set, extent = {{-20, 108}, {170, 92}}, textString = "m_flow_set")}),
          Documentation(info = "<html>
<p>
This model describes a centrifugal pump (or a group of <code>nParallel</code> pumps)
with ideally controlled mass flow rate or pressure.
</p>
<p>
Nominal values are used to predefine an exemplary pump characteristics and to define the operation of the pump.
The input connectors <code>m_flow_set</code> or <code>p_set</code> can optionally be enabled to provide time varying set points.
</p>
<p>
Use this model if the pump characteristics is of secondary interest.
The actual characteristics can be configured later on for the appropriate rotational speed N.
Then the model can be replaced with a Pump with rotational shaft or with a PrescribedPump.
</p>
</html>", revisions = "<html>
<ul>
<li><i>15 Dec 2008</i>
    by Ruediger Franke:<br />
       Model added to the Fluid library</li>
</ul>
</html>"));
      end ControlledPump;

      model PrescribedPump "Centrifugal pump with ideally controlled speed"
        extends Modelica.Fluid.Machines.BaseClasses.PartialPump;
        parameter Boolean use_N_in = false "Get the rotational speed from the input connector";
        parameter Modelica.SIunits.Conversions.NonSIunits.AngularVelocity_rpm N_const = N_nominal "Constant rotational speed" annotation(
          Dialog(enable = not use_N_in));
        Modelica.Blocks.Interfaces.RealInput N_in(unit = "rev/min") if use_N_in "Prescribed rotational speed" annotation(
          Placement(transformation(extent = {{-20, -20}, {20, 20}}, rotation = -90, origin = {0, 100}), iconTransformation(extent = {{-20, -20}, {20, 20}}, rotation = -90, origin = {0, 100})));
      protected
        Modelica.Blocks.Interfaces.RealInput N_in_internal(unit = "rev/min") "Needed to connect to conditional connector";
      equation
// Connect statement active only if use_p_in = true
        connect(N_in, N_in_internal);
// Internal connector value when use_p_in = false
        if not use_N_in then
          N_in_internal = N_const;
        end if;
// Set N with a lower limit to avoid singularities at zero speed
        N = max(N_in_internal, 1e-3) "Rotational speed";
        annotation(
          defaultComponentName = "pump",
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(visible = use_N_in, extent = {{14, 98}, {178, 82}}, textString = "N_in [rpm]")}),
          Documentation(info = "<html>
<p>This model describes a centrifugal pump (or a group of <code>nParallel</code> pumps) with prescribed speed, either fixed or provided by an external signal.
<p>The model extends <code>PartialPump</code>
<p>If the <code>N_in</code> input connector is wired, it provides rotational speed of the pumps (rpm); otherwise, a constant rotational speed equal to <code>n_const</code> (which can be different from <code>N_nominal</code>) is assumed.</p>
</html>", revisions = "<html>
<ul>
<li><i>31 Oct 2005</i>
    by <a href=\"mailto:francesco.casella@polimi.it\">Francesco Casella</a>:<br>
       Model added to the Fluid library</li>
</ul>
</html>"));
      end PrescribedPump;

      package BaseClasses "Base classes used in the Machines package (only of interest to build new component models)"
        extends Modelica.Icons.BasesPackage;

        partial model PartialPump "Base model for centrifugal pumps"
          import NonSI = Modelica.SIunits.Conversions.NonSIunits;
          import Modelica.Constants;
          extends Modelica.Fluid.Interfaces.PartialTwoPort(port_b_exposesState = energyDynamics <> Types.Dynamics.SteadyState or massDynamics <> Types.Dynamics.SteadyState, port_a(p(start = p_a_start), m_flow(start = m_flow_start, min = if allowFlowReversal and not checkValve then -Constants.inf else 0)), port_b(p(start = p_b_start), m_flow(start = -m_flow_start, max = if allowFlowReversal and not checkValve then +Constants.inf else 0)));
          // Initialization
          parameter Medium.AbsolutePressure p_a_start = system.p_start "Guess value for inlet pressure" annotation(
            Dialog(tab = "Initialization"));
          parameter Medium.AbsolutePressure p_b_start = p_a_start "Guess value for outlet pressure" annotation(
            Dialog(tab = "Initialization"));
          parameter Medium.MassFlowRate m_flow_start = system.m_flow_start "Guess value of m_flow = port_a.m_flow" annotation(
            Dialog(tab = "Initialization"));
          final parameter SI.VolumeFlowRate V_flow_single_init = m_flow_start / rho_nominal / nParallel "Used for simplified initialization model";
          final parameter SI.Height delta_head_init = flowCharacteristic(V_flow_single_init) - flowCharacteristic(0) "Used for simplified initialization model";
          // Characteristic curves
          parameter Integer nParallel(min = 1) = 1 "Number of pumps in parallel" annotation(
            Dialog(group = "Characteristics"));
          replaceable function flowCharacteristic = PumpCharacteristics.baseFlow "Head vs. V_flow characteristic at nominal speed and density" annotation(
            Dialog(group = "Characteristics"),
            choicesAllMatching = true);
          parameter NonSI.AngularVelocity_rpm N_nominal "Nominal rotational speed for flow characteristic" annotation(
            Dialog(group = "Characteristics"));
          parameter Medium.Density rho_nominal = Medium.density_pTX(Medium.p_default, Medium.T_default, Medium.X_default) "Nominal fluid density for characteristic" annotation(
            Dialog(group = "Characteristics"));
          parameter Boolean use_powerCharacteristic = false "Use powerCharacteristic (vs. efficiencyCharacteristic)" annotation(
            Evaluate = true,
            Dialog(group = "Characteristics"));
          replaceable function powerCharacteristic = PumpCharacteristics.quadraticPower(V_flow_nominal = {0, 0, 0}, W_nominal = {0, 0, 0}) "Power consumption vs. V_flow at nominal speed and density" annotation(
            Dialog(group = "Characteristics", enable = use_powerCharacteristic),
            choicesAllMatching = true);
          replaceable function efficiencyCharacteristic = PumpCharacteristics.constantEfficiency(eta_nominal = 0.8) constrainedby PumpCharacteristics.baseEfficiency "Efficiency vs. V_flow at nominal speed and density" annotation(
             Dialog(group = "Characteristics", enable = not use_powerCharacteristic),
             choicesAllMatching = true);
          // Assumptions
          parameter Boolean checkValve = false "= true to prevent reverse flow" annotation(
            Dialog(tab = "Assumptions"),
            Evaluate = true);
          parameter SI.Volume V = 0 "Volume inside the pump" annotation(
            Dialog(tab = "Assumptions"),
            Evaluate = true);
          // Energy and mass balance
          extends Modelica.Fluid.Interfaces.PartialLumpedVolume(final fluidVolume = V, energyDynamics = Types.Dynamics.SteadyState, massDynamics = Types.Dynamics.SteadyState, final p_start = p_b_start);
          // Heat transfer through boundary, e.g., to add a housing
          parameter Boolean use_HeatTransfer = false "= true to use a HeatTransfer model, e.g., for a housing" annotation(
            Dialog(tab = "Assumptions", group = "Heat transfer"));
          replaceable model HeatTransfer = Modelica.Fluid.Vessels.BaseClasses.HeatTransfer.IdealHeatTransfer constrainedby Modelica.Fluid.Vessels.BaseClasses.HeatTransfer.PartialVesselHeatTransfer "Wall heat transfer" annotation(
             Dialog(tab = "Assumptions", group = "Heat transfer", enable = use_HeatTransfer),
             choicesAllMatching = true);
          HeatTransfer heatTransfer(redeclare final package Medium = Medium, final n = 1, surfaceAreas = {4 * Modelica.Constants.pi * (3 / 4 * V / Modelica.Constants.pi) ^ (2 / 3)}, final states = {medium.state}, final use_k = use_HeatTransfer) annotation(
            Placement(transformation(extent = {{-10, -10}, {30, 30}}, rotation = 180, origin = {50, -10})));
          Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a heatPort if use_HeatTransfer annotation(
            Placement(transformation(extent = {{30, -70}, {50, -50}})));
          // Variables
          final parameter SI.Acceleration g = system.g;
          Medium.Density rho = medium.d;
          SI.Pressure dp_pump = port_b.p - port_a.p "Pressure increase";
          SI.Height head = dp_pump / (rho * g) "Pump head";
          SI.MassFlowRate m_flow = port_a.m_flow "Mass flow rate (total)";
          SI.MassFlowRate m_flow_single = m_flow / nParallel "Mass flow rate (single pump)";
          SI.VolumeFlowRate V_flow "Volume flow rate (total)";
          SI.VolumeFlowRate V_flow_single(start = m_flow_start / rho_nominal / nParallel) "Volume flow rate (single pump)";
          NonSI.AngularVelocity_rpm N(start = N_nominal) "Shaft rotational speed";
          SI.Power W_single "Power Consumption (single pump)";
          SI.Power W_total = W_single * nParallel "Power Consumption (total)";
          Real eta "Global Efficiency";
          final constant Medium.MassFlowRate unit_m_flow = 1 annotation(
            HideResult = true);
          Real s(start = m_flow_start / unit_m_flow) "Curvilinear abscissa for the flow curve in parametric form (either mass flow rate or head)";
          // Diagnostics
          replaceable model Monitoring = Modelica.Fluid.Machines.BaseClasses.PumpMonitoring.PumpMonitoringBase constrainedby Modelica.Fluid.Machines.BaseClasses.PumpMonitoring.PumpMonitoringBase "Optional pump monitoring" annotation(
             Dialog(tab = "Advanced", group = "Diagnostics"),
             choicesAllMatching = true);
          final parameter Boolean show_NPSHa = false "obsolete -- remove modifier and specify Monitoring for NPSH instead" annotation(
            Dialog(tab = "Advanced", group = "Obsolete"));
          Monitoring monitoring(redeclare final package Medium = Medium, final state_in = Medium.setState_phX(port_a.p, inStream(port_a.h_outflow), inStream(port_a.Xi_outflow)), final state = medium.state) "Monitoring model" annotation(
            Placement(transformation(extent = {{-64, -42}, {-20, 0}})));
        protected
          constant SI.Height unitHead = 1;
          constant SI.MassFlowRate unitMassFlowRate = 1;
        equation
// Flow equations
          V_flow = homotopy(m_flow / rho, m_flow / rho_nominal);
          V_flow_single = V_flow / nParallel;
          if not checkValve then
// Regular flow characteristics without check valve
            head = homotopy((N / N_nominal) ^ 2 * flowCharacteristic(V_flow_single * N_nominal / N), N / N_nominal * (flowCharacteristic(0) + delta_head_init * V_flow_single / V_flow_single_init));
            s = 0;
          else
// Flow characteristics when check valve is open
            head = homotopy(if s > 0 then (N / N_nominal) ^ 2 * flowCharacteristic(V_flow_single * N_nominal / N) else (N / N_nominal) ^ 2 * flowCharacteristic(0) - s * unitHead, N / N_nominal * (flowCharacteristic(0) + delta_head_init * V_flow_single / V_flow_single_init));
            V_flow_single = homotopy(if s > 0 then s * unitMassFlowRate / rho else 0, s * unitMassFlowRate / rho_nominal);
          end if;
// Power consumption
          if use_powerCharacteristic then
            W_single = homotopy((N / N_nominal) ^ 3 * (rho / rho_nominal) * powerCharacteristic(V_flow_single * N_nominal / N), N / N_nominal * V_flow_single / V_flow_single_init * powerCharacteristic(V_flow_single_init));
            eta = dp_pump * V_flow_single / W_single;
          else
            eta = homotopy(efficiencyCharacteristic(V_flow_single * (N_nominal / N)), efficiencyCharacteristic(V_flow_single_init));
            W_single = homotopy(dp_pump * V_flow_single / eta, dp_pump * V_flow_single_init / eta);
          end if;
// Energy balance
          Wb_flow = W_total;
          Qb_flow = heatTransfer.Q_flows[1];
          Hb_flow = port_a.m_flow * actualStream(port_a.h_outflow) + port_b.m_flow * actualStream(port_b.h_outflow);
// Ports
          port_a.h_outflow = medium.h;
          port_b.h_outflow = medium.h;
          port_b.p = medium.p "outlet pressure is equal to medium pressure, which includes Wb_flow";
// Mass balance
          mb_flow = port_a.m_flow + port_b.m_flow;
          mbXi_flow = port_a.m_flow * actualStream(port_a.Xi_outflow) + port_b.m_flow * actualStream(port_b.Xi_outflow);
          port_a.Xi_outflow = medium.Xi;
          port_b.Xi_outflow = medium.Xi;
          mbC_flow = port_a.m_flow * actualStream(port_a.C_outflow) + port_b.m_flow * actualStream(port_b.C_outflow);
          port_a.C_outflow = C;
          port_b.C_outflow = C;
          connect(heatTransfer.heatPorts[1], heatPort) annotation(
            Line(points = {{40, -34}, {40, -60}}, color = {127, 0, 0}));
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, 46}, {100, -46}}, lineColor = {0, 0, 0}, fillColor = {0, 127, 255}, fillPattern = FillPattern.HorizontalCylinder), Polygon(points = {{-48, -60}, {-72, -100}, {72, -100}, {48, -60}, {-48, -60}}, lineColor = {0, 0, 255}, pattern = LinePattern.None, fillColor = {0, 0, 0}, fillPattern = FillPattern.VerticalCylinder), Ellipse(extent = {{-80, 80}, {80, -80}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.Sphere, fillColor = {0, 100, 199}), Polygon(points = {{-28, 30}, {-28, -30}, {50, -2}, {-28, 30}}, lineColor = {0, 0, 0}, pattern = LinePattern.None, fillPattern = FillPattern.HorizontalCylinder, fillColor = {255, 255, 255})}),
            Documentation(info = "<html>
<p>This is the base model for pumps.
<p>The model describes a centrifugal pump, or a group of <code>nParallel</code> identical pumps. The pump model is based on the theory of kinematic similarity: the pump characteristics are given for nominal operating conditions (rotational speed and fluid density), and then adapted to actual operating condition, according to the similarity equations.

<p><b>Pump characteristics</b></p>
<p> The nominal hydraulic characteristic (head vs. volume flow rate) is given by the the replaceable function <code>flowCharacteristic</code>.
<p> The pump energy balance can be specified in two alternative ways:
<ul>
<li><code>use_powerCharacteristic = false</code> (default option): the replaceable function <code>efficiencyCharacteristic</code> (efficiency vs. volume flow rate in nominal conditions) is used to determine the efficiency, and then the power consumption.
    The default is a constant efficiency of 0.8.</li>
<li><code>use_powerCharacteristic = true</code>: the replaceable function <code>powerCharacteristic</code> (power consumption vs. volume flow rate in nominal conditions) is used to determine the power consumption, and then the efficiency.
    Use <code>powerCharacteristic</code> to specify a non-zero power consumption for zero flow rate.
</ul>
<p>
Several functions are provided in the package <code>PumpCharacteristics</code> to specify the characteristics as a function of some operating points at nominal conditions.
<p>Depending on the value of the <code>checkValve</code> parameter, the model either supports reverse flow conditions, or includes a built-in check valve to avoid flow reversal.
</p>
<p>It is possible to take into account the mass and energy storage of the fluid inside the pump by specifying its volume <code>V</code>, and by selecting appropriate dynamic mass and energy balance assumptions (see below);
this is recommended to avoid singularities in the computation of the outlet enthalpy in case of zero flow rate.
If zero flow rate conditions are always avoided, this dynamic effect can be neglected by leaving the default value <code>V = 0</code>, thus avoiding fast state variables in the model.
</p>

<p><b>Dynamics options</b></p>
<p>
Steady-state mass and energy balances are assumed per default, neglecting the holdup of fluid in the pump; this configuration works well if the flow rate is always positive.
Dynamic mass and energy balance can be used by setting the corresponding dynamic parameters. This is recommended to avoid singularities at zero or reversing mass flow rate. If the initial conditions imply non-zero mass flow rate, it is possible to use the <code>SteadyStateInitial</code> condition, otherwise it is recommended to use <code>FixedInitial</code> in order to avoid undetermined initial conditions.
</p>

<p><b>Heat transfer</b></p>
<p>
The Boolean parameter <code>use_HeatTransfer</code> can be set to true if heat exchanged with the environment
should be taken into account or to model a housing. This might be desirable if a pump with realistic
<code>powerCharacteristic</code> for zero flow operates while a valve prevents fluid flow.
</p>

<p><b>Diagnostics of Cavitation</b></p>
<p>The replaceable Monitoring submodel can be configured to PumpMonitoringNPSH,
in order to compute the Net Positive Suction Head available and check for cavitation,
provided a two-phase medium model is used (see Advanced tab).
</p>
</html>", revisions = "<html>
<ul>
<li><i>8 Jan 2013</i>
    by R&uuml;diger Franke:<br>
    moved NPSH diagnostics from PartialPump to replaceable sub-model PumpMonitoring.PumpMonitoringNPSH (see ticket #646)</li>
<li><i>Dec 2008</i>
    by R&uuml;diger Franke:<br>
    <ul>
    <li>Replaced simplified mass and energy balances with rigorous formulation (base class PartialLumpedVolume)</li>
    <li>Introduced optional HeatTransfer model defining Qb_flow</li>
    <li>Enabled events when the checkValve is operating to support the opening of a discrete valve before port_a</li>
    </ul></li>
<li><i>31 Oct 2005</i>
    by <a href=\"mailto:francesco.casella@polimi.it\">Francesco Casella</a>:<br>
       Model added to the Fluid library</li>
</ul>
</html>"));
        end PartialPump;

        package PumpCharacteristics "Functions for pump characteristics"
          extends Modelica.Icons.Package;
          import NonSI = Modelica.SIunits.Conversions.NonSIunits;

          partial function baseFlow "Base class for pump flow characteristics"
            extends Modelica.Icons.Function;
            input SI.VolumeFlowRate V_flow "Volumetric flow rate";
            output SI.Height head "Pump head";
          end baseFlow;

          partial function basePower "Base class for pump power consumption characteristics"
            extends Modelica.Icons.Function;
            input SI.VolumeFlowRate V_flow "Volumetric flow rate";
            output SI.Power consumption "Power consumption";
          end basePower;

          partial function baseEfficiency "Base class for efficiency characteristics"
            extends Modelica.Icons.Function;
            input SI.VolumeFlowRate V_flow "Volumetric flow rate";
            output Real eta "Efficiency";
          end baseEfficiency;

          function linearFlow "Linear flow characteristic"
            extends baseFlow;
            input SI.VolumeFlowRate V_flow_nominal[2] "Volume flow rate for two operating points (single pump)" annotation(
              Dialog);
            input SI.Height head_nominal[2] "Pump head for two operating points" annotation(
              Dialog);
            /* Linear system to determine the coefficients:
              head_nominal[1] = c[1] + V_flow_nominal[1]*c[2];
              head_nominal[2] = c[1] + V_flow_nominal[2]*c[2];
              */
          protected
            Real c[2] = Modelica.Math.Matrices.solve([ones(2), V_flow_nominal], head_nominal) "Coefficients of linear head curve";
          algorithm
            assert(c[2] <= (-Modelica.Constants.small), "Wrong pump curve -- head_nominal must be monotonically decreasing with increasing V_flow_nominal", level = AssertionLevel.warning);
// Flow equation: head = q*c[1] + c[2];
            head := c[1] + V_flow * c[2];
          end linearFlow;

          function quadraticFlow "Quadratic flow characteristic, including linear extrapolation"
            extends baseFlow;
            input SI.VolumeFlowRate V_flow_nominal[3] "Volume flow rate for three operating points (single pump)" annotation(
              Dialog);
            input SI.Height head_nominal[3] "Pump head for three operating points" annotation(
              Dialog);
          protected
            Real V_flow_nominal2[3] = {V_flow_nominal[1] ^ 2, V_flow_nominal[2] ^ 2, V_flow_nominal[3] ^ 2} "Squared nominal flow rates";
            /* Linear system to determine the coefficients:
              head_nominal[1] = c[1] + V_flow_nominal[1]*c[2] + V_flow_nominal[1]^2*c[3];
              head_nominal[2] = c[1] + V_flow_nominal[2]*c[2] + V_flow_nominal[2]^2*c[3];
              head_nominal[3] = c[1] + V_flow_nominal[3]*c[2] + V_flow_nominal[3]^2*c[3];
              */
            Real c[3] = Modelica.Math.Matrices.solve([ones(3), V_flow_nominal, V_flow_nominal2], head_nominal) "Coefficients of quadratic head curve";
            SI.VolumeFlowRate V_flow_min = min(V_flow_nominal);
            SI.VolumeFlowRate V_flow_max = max(V_flow_nominal);
          algorithm
            assert(max(c[2] .+ 2 * c[3] * V_flow_nominal) <= (-Modelica.Constants.small), "Wrong pump curve -- head_nominal must be monotonically decreasing with increasing V_flow_nominal", level = AssertionLevel.warning);
            if V_flow < V_flow_min then
              head := max(head_nominal) + (V_flow - V_flow_min) * (c[2] + 2 * c[3] * V_flow_min);
            elseif V_flow > V_flow_max then
              head := min(head_nominal) + (V_flow - V_flow_max) * (c[2] + 2 * c[3] * V_flow_max);
            else
              head := c[1] + V_flow * (c[2] + V_flow * c[3]);
            end if;
// Flow equation: head  = c[1] + V_flow*c[2] + V_flow^2*c[3];
            annotation(
              Documentation(revisions = "<html>
<ul>
<li><i>Jan 2013</i>
    by R&uuml;diger Franke:<br>
    Extended with linear extrapolation outside specified points</li>
</ul>
</html>"));
          end quadraticFlow;

          function polynomialFlow "Polynomial flow characteristic, including linear extrapolation"
            extends baseFlow;
            input SI.VolumeFlowRate V_flow_nominal[:] "Volume flow rate for N operating points (single pump)" annotation(
              Dialog);
            input SI.Height head_nominal[:] "Pump head for N operating points" annotation(
              Dialog);
          protected
            Integer N = size(V_flow_nominal, 1) "Number of nominal operating points";
            Real V_flow_nominal_pow[N, N] = {{if j > 1 then V_flow_nominal[i] ^ (j - 1) else 1 for j in 1:N} for i in 1:N} "Rows: different operating points; columns: increasing powers";
            /* Linear system to determine the coefficients (example N=3):
              head_nominal[1] = c[1] + V_flow_nominal[1]*c[2] + V_flow_nominal[1]^2*c[3];
              head_nominal[2] = c[1] + V_flow_nominal[2]*c[2] + V_flow_nominal[2]^2*c[3];
              head_nominal[3] = c[1] + V_flow_nominal[3]*c[2] + V_flow_nominal[3]^2*c[3];
              */
            Real c[size(V_flow_nominal, 1)] = Modelica.Math.Matrices.solve(V_flow_nominal_pow, head_nominal) "Coefficients of polynomial head curve";
            SI.VolumeFlowRate V_flow_min = min(V_flow_nominal);
            SI.VolumeFlowRate V_flow_max = max(V_flow_nominal);
            Real max_dhdV = c[2] + max(sum((i - 1) * V_flow_nominal .^ (i - 2) * c[i] for i in 3:N));
            Real poly;
          algorithm
            assert(max_dhdV <= (-Modelica.Constants.small), "Wrong pump curve -- head_nominal must be monotonically decreasing with increasing V_flow_nominal", level = AssertionLevel.warning);
            if V_flow < V_flow_min then
              poly := c[N] * (N - 1);
              for i in 1:N - 2 loop
                poly := V_flow_min * poly + c[N - i] * (N - i - 1);
              end for;
              head := max(head_nominal) + (V_flow - V_flow_min) * poly;
            elseif V_flow > V_flow_max then
              poly := c[N] * (N - 1);
              for i in 1:N - 2 loop
                poly := V_flow_max * poly + c[N - i] * (N - i - 1);
              end for;
              head := min(head_nominal) + (V_flow - V_flow_max) * poly;
            else
              poly := c[N];
              for i in 1:N - 1 loop
                poly := V_flow * poly + c[N - i];
              end for;
              head := poly;
            end if;
//head := max(head_nominal) + (V_flow-V_flow_min)*(c[2]+sum((i-1)*V_flow_min^(i-2)*c[i] for i in 3:N));
//head := min(head_nominal) + (V_flow-V_flow_max)*(c[2]+sum((i-1)*V_flow_max^(i-2)*c[i] for i in 3:N));
// Flow equation (example N=3): head  = c[1] + V_flow*c[2] + V_flow^2*c[3];
// Note: the implementation is numerically efficient only for low values of Na
//head := sum(V_flow^(i-1)*c[i] for i in 1:N);
            annotation(
              Documentation(revisions = "<html>
<ul>
<li><i>Jan 2013</i>
    by R&uuml;diger Franke:<br>
    Extended with linear extrapolation outside specified points and reformulated polynomial evaluation</li>
</ul>
</html>"));
          end polynomialFlow;

          function constantEfficiency "Constant efficiency characteristic"
            extends baseEfficiency;
            input Real eta_nominal "Nominal efficiency" annotation(
              Dialog);
          algorithm
            eta := eta_nominal;
          end constantEfficiency;

          function linearPower "Linear power consumption characteristic"
            extends basePower;
            input SI.VolumeFlowRate V_flow_nominal[2] "Volume flow rate for two operating points (single pump)" annotation(
              Dialog);
            input SI.Power W_nominal[2] "Power consumption for two operating points" annotation(
              Dialog);
            /* Linear system to determine the coefficients:
              W_nominal[1] = c[1] + V_flow_nominal[1]*c[2];
              W_nominal[2] = c[1] + V_flow_nominal[2]*c[2];
              */
          protected
            Real c[2] = Modelica.Math.Matrices.solve([ones(3), V_flow_nominal], W_nominal) "Coefficients of linear power consumption curve";
          algorithm
            consumption := c[1] + V_flow * c[2];
          end linearPower;

          function quadraticPower "Quadratic power consumption characteristic"
            extends basePower;
            input SI.VolumeFlowRate V_flow_nominal[3] "Volume flow rate for three operating points (single pump)" annotation(
              Dialog);
            input SI.Power W_nominal[3] "Power consumption for three operating points" annotation(
              Dialog);
          protected
            Real V_flow_nominal2[3] = {V_flow_nominal[1] ^ 2, V_flow_nominal[2] ^ 2, V_flow_nominal[3] ^ 2} "Squared nominal flow rates";
            /* Linear system to determine the coefficients:
              W_nominal[1] = c[1] + V_flow_nominal[1]*c[2] + V_flow_nominal[1]^2*c[3];
              W_nominal[2] = c[1] + V_flow_nominal[2]*c[2] + V_flow_nominal[2]^2*c[3];
              W_nominal[3] = c[1] + V_flow_nominal[3]*c[2] + V_flow_nominal[3]^2*c[3];
              */
            Real c[3] = Modelica.Math.Matrices.solve([ones(3), V_flow_nominal, V_flow_nominal2], W_nominal) "Coefficients of quadratic power consumption curve";
          algorithm
            consumption := c[1] + V_flow * c[2] + V_flow ^ 2 * c[3];
          end quadraticPower;
        end PumpCharacteristics;

        package PumpMonitoring "Monitoring of pump operation"
          extends Modelica.Icons.Package;

          model PumpMonitoringBase "Interface for pump monitoring"
            outer Modelica.Fluid.System system "System wide properties";
            //
            // Internal interface
            // (not exposed to GUI; needs to be hard coded when using this model
            //
            replaceable package Medium = Modelica.Media.Interfaces.PartialMedium "Medium in the component" annotation(
              Dialog(tab = "Internal Interface", enable = false));
            // Inputs
            input Medium.ThermodynamicState state_in "Thermodynamic state of inflow";
            input Medium.ThermodynamicState state "Thermodynamic state in the pump";
          end PumpMonitoringBase;

          model PumpMonitoringNPSH "Monitor Net Positive Suction Head (NPSH)"
            extends PumpMonitoringBase(redeclare replaceable package Medium = Modelica.Media.Interfaces.PartialTwoPhaseMedium);
            Medium.Density rho_in = Medium.density(state_in) "Liquid density at the inlet port_a";
            SI.Length NPSHa = NPSPa / (rho_in * system.g) "Net Positive Suction Head available";
            SI.Pressure NPSPa = assertPositiveDifference(Medium.pressure(state_in), Medium.saturationPressure(Medium.temperature(state_in)), "Cavitation occurs at the pump inlet") "Net Positive Suction Pressure available";
            SI.Pressure NPDPa = assertPositiveDifference(Medium.pressure(state), Medium.saturationPressure(Medium.temperature(state)), "Cavitation occurs in the pump") "Net Positive Discharge Pressure available";
          end PumpMonitoringNPSH;

          function assertPositiveDifference
            extends Modelica.Icons.Function;
            input SI.Pressure p;
            input SI.Pressure p_sat;
            input String message;
            output SI.Pressure dp;
          algorithm
            dp := p - p_sat;
            assert(p >= p_sat, message);
          end assertPositiveDifference;
        end PumpMonitoring;
      end BaseClasses;
      annotation(
        Documentation(info = "<html>

</html>"));
    end Machines;

    package Valves "Components for the regulation and control of fluid flow"
      extends Modelica.Icons.VariantsPackage;

      model ValveIncompressible "Valve for (almost) incompressible fluids"
        extends BaseClasses.PartialValve;
        import Modelica.Fluid.Types.CvTypes;
        import Modelica.Constants.pi;
        constant SI.ReynoldsNumber Re_turbulent = 4000 "cf. straight pipe for fully open valve -- dp_turbulent increases for closing valve";
        parameter Boolean use_Re = system.use_eps_Re "= true, if turbulent region is defined by Re, otherwise by m_flow_small" annotation(
          Dialog(tab = "Advanced"),
          Evaluate = true);
        //SI.MassFlowRate m_flow_turbulent=if not use_Re then m_flow_small else
        //  max(m_flow_small,
        //      (Modelica.Constants.pi/8)*sqrt(max(relativeFlowCoefficient,0.001)*Av*4/pi)*(Medium.dynamicViscosity(state_a) + Medium.dynamicViscosity(state_b))*Re_turbulent);
        //SI.AbsolutePressure dp_turbulent_=if not use_Re then dp_small else
        //  max(dp_small, m_flow_turbulent^2/(max(relativeFlowCoefficient,0.001)^2*Av^2*(Medium.density(state_a) + Medium.density(state_b))/2));
        // substitute m_flow_turbulent into dp_turbulent
        SI.AbsolutePressure dp_turbulent = if not use_Re then dp_small else max(dp_small, (Medium.dynamicViscosity(state_a) + Medium.dynamicViscosity(state_b)) ^ 2 * pi / 8 * Re_turbulent ^ 2 / (max(relativeFlowCoefficient, 0.001) * Av * (Medium.density(state_a) + Medium.density(state_b))));
      protected
        Real relativeFlowCoefficient;
      initial equation
        if CvData == CvTypes.OpPoint then
          m_flow_nominal = valveCharacteristic(opening_nominal) * Av * sqrt(rho_nominal) * Utilities.regRoot(dp_nominal, dp_small) "Determination of Av by the operating point";
        end if;
      equation
// m_flow = valveCharacteristic(opening)*Av*sqrt(d)*sqrt(dp);
        relativeFlowCoefficient = valveCharacteristic(opening_actual);
        if checkValve then
          m_flow = homotopy(relativeFlowCoefficient * Av * sqrt(Medium.density(state_a)) * Utilities.regRoot2(dp, dp_turbulent, 1.0, 0.0, use_yd0 = true, yd0 = 0.0), relativeFlowCoefficient * m_flow_nominal * dp / dp_nominal);
/* In Modelica 3.1 (Disadvantage: Unnecessary event at dp=0, and instead of smooth=2)
    m_flow = valveCharacteristic(opening)*Av*sqrt(Medium.density(state_a))*
                  (if dp>=0 then Utilities.regRoot(dp, dp_turbulent) else 0);
    */
        elseif not allowFlowReversal then
          m_flow = homotopy(relativeFlowCoefficient * Av * sqrt(Medium.density(state_a)) * Utilities.regRoot(dp, dp_turbulent), relativeFlowCoefficient * m_flow_nominal * dp / dp_nominal);
        else
          m_flow = homotopy(relativeFlowCoefficient * Av * Utilities.regRoot2(dp, dp_turbulent, Medium.density(state_a), Medium.density(state_b)), relativeFlowCoefficient * m_flow_nominal * dp / dp_nominal);
/* In Modelica 3.1 (Disadvantage: Unnecessary event at dp=0, and instead of smooth=2)
    m_flow = smooth(0, Utilities.regRoot(dp, dp_turbulent)*(if dp>=0 then sqrt(Medium.density(state_a)) else sqrt(Medium.density(state_b))));
    */
        end if;
        annotation(
          Documentation(info = "<html>
<p>
Valve model according to the IEC 534/ISA S.75 standards for valve sizing, incompressible fluids.<
/p>

<p>
The parameters of this model are explained in detail in
<a href=\"modelica://Modelica.Fluid.Valves.BaseClasses.PartialValve\">PartialValve</a>
(the base model for valves).
</p>

<p>
This model assumes that the fluid has a low compressibility, which is always the case for liquids.
It can also be used with gases, provided that the pressure drop is lower than 0.2 times the absolute pressure at the inlet, so that the fluid density does not change much inside the valve.</p>

<p>
If <code>checkValve</code> is false, the valve supports reverse flow, with a symmetric flow characteristic curve. Otherwise, reverse flow is stopped (check valve behaviour).
</p>

<p>
The treatment of parameters <b>Kv</b> and <b>Cv</b> is
explained in detail in the
<a href=\"modelica://Modelica.Fluid.UsersGuide.ComponentDefinition.ValveCharacteristics\">User's Guide</a>.
</p>

</html>", revisions = "<html>
<ul>
<li><i>2 Nov 2005</i>
    by <a href=\"mailto:francesco.casella@polimi.it\">Francesco Casella</a>:<br>
       Adapted from the ThermoPower library.</li>
</ul>
</html>"));
      end ValveIncompressible;

      model ValveVaporizing "Valve for possibly vaporizing (almost) incompressible fluids, accounts for choked flow conditions"
        import Modelica.Fluid.Types.CvTypes;
        import Modelica.Constants.pi;
        extends BaseClasses.PartialValve(redeclare replaceable package Medium = Modelica.Media.Water.WaterIF97_ph constrainedby Modelica.Media.Interfaces.PartialTwoPhaseMedium);
        parameter Real Fl_nominal = 0.9 "Liquid pressure recovery factor";
        replaceable function FlCharacteristic = Modelica.Fluid.Valves.BaseClasses.ValveCharacteristics.one constrainedby Modelica.Fluid.Valves.BaseClasses.ValveCharacteristics.baseFun "Pressure recovery characteristic";
        Real Ff "Ff coefficient (see IEC/ISA standard)";
        Real Fl "Pressure recovery coefficient Fl (see IEC/ISA standard)";
        SI.Pressure dpEff "Effective pressure drop";
        Medium.Temperature T_in "Inlet temperature";
        Medium.AbsolutePressure p_sat "Saturation pressure";
        Medium.AbsolutePressure p_in "Inlet pressure";
        Medium.AbsolutePressure p_out "Outlet pressure";
        constant SI.ReynoldsNumber Re_turbulent = 4000 "cf. straight pipe for fully open valve -- dp_turbulent increases for closing valve";
        parameter Boolean use_Re = system.use_eps_Re "= true, if turbulent region is defined by Re, otherwise by m_flow_small" annotation(
          Dialog(tab = "Advanced"),
          Evaluate = true);
        //SI.Diameter diameter = Utilities.regRoot(4/pi*valveCharacteristic(opening_actual)*Av, 0.04/pi*valveCharacteristic(opening_nominal)*Av);
        SI.AbsolutePressure dp_turbulent = if not use_Re then dp_small else max(dp_small, (Medium.dynamicViscosity(state_a) + Medium.dynamicViscosity(state_b)) ^ 2 * pi / 8 * Re_turbulent ^ 2 / (valveCharacteristic(opening_actual) * Av * (Medium.density(state_a) + Medium.density(state_b))));
      initial equation
        assert(not CvData == CvTypes.OpPoint, "OpPoint option not supported for vaporizing valve");
      equation
        p_in = port_a.p;
        p_out = port_b.p;
        T_in = Medium.temperature(state_a);
        p_sat = Medium.saturationPressure(T_in);
        Ff = 0.96 - 0.28 * sqrt(p_sat / Medium.fluidConstants[1].criticalPressure);
        Fl = Fl_nominal * FlCharacteristic(opening_actual);
        dpEff = if p_out < (1 - Fl ^ 2) * p_in + Ff * Fl ^ 2 * p_sat then Fl ^ 2 * (p_in - Ff * p_sat) else dp "Effective pressure drop, accounting for possible choked conditions";
// m_flow = valveCharacteristic(opening)*Av*sqrt(d)*sqrt(dpEff);
        if checkValve then
          m_flow = homotopy(valveCharacteristic(opening_actual) * Av * sqrt(Medium.density(state_a)) * Utilities.regRoot2(dpEff, dp_turbulent, 1.0, 0.0, use_yd0 = true, yd0 = 0.0), valveCharacteristic(opening_actual) * m_flow_nominal * dp / dp_nominal);
/* In Modelica 3.1 (Disadvantage: Unnecessary event at dpEff=0, and instead of smooth=2)
    m_flow = valveCharacteristic(opening)*Av*sqrt(Medium.density(state_a))*
                  (if dpEff>=0 then Utilities.regRoot(dpEff, dp_turbulent) else 0);
   */
        elseif not allowFlowReversal then
          m_flow = homotopy(valveCharacteristic(opening_actual) * Av * sqrt(Medium.density(state_a)) * Utilities.regRoot(dpEff, dp_turbulent), valveCharacteristic(opening_actual) * m_flow_nominal * dp / dp_nominal);
        else
          m_flow = homotopy(valveCharacteristic(opening_actual) * Av * Utilities.regRoot2(dpEff, dp_turbulent, Medium.density(state_a), Medium.density(state_b)), valveCharacteristic(opening_actual) * m_flow_nominal * dp / dp_nominal);
/* In Modelica 3.1 (Disadvantage: Unnecessary event at dp=0, and instead of smooth=2)
     m_flow = valveCharacteristic(opening)*Av*
      smooth(0, Utilities.regRoot(dpEff, dp_turbulent)*(if dpEff>=0 then sqrt(Medium.density(state_a)) else sqrt(Medium.density(state_b))));
   */
        end if;
        annotation(
          Documentation(info = "<html>
<p>Valve model according to the IEC 534/ISA S.75 standards for valve sizing, incompressible fluid at the inlet, and possibly two-phase fluid at the outlet, including choked flow conditions.</p>

<p>
The parameters of this model are explained in detail in
<a href=\"modelica://Modelica.Fluid.Valves.BaseClasses.PartialValve\">PartialValve</a>
(the base model for valves).
</p>

<p>The model operating range includes choked flow operation, which takes place for low outlet pressures due to flashing in the vena contracta; otherwise, non-choking conditions are assumed.
<p>This model requires a two-phase medium model, to describe the liquid and (possible) two-phase conditions.
<p>The default liquid pressure recovery coefficient <code>Fl</code> is constant and given by the parameter <code>Fl_nominal</code>. The relative change (per unit) of the recovery coefficient can be specified as a given function of the valve opening by replacing the <code>FlCharacteristic</code> function.
<p>If <code>checkValve</code> is false, the valve supports reverse flow, with a symmetric flow characteristic curve. Otherwise, reverse flow is stopped (check valve behaviour).</p>

<p>
The treatment of parameters <b>Kv</b> and <b>Cv</b> is
explained in detail in the
<a href=\"modelica://Modelica.Fluid.UsersGuide.ComponentDefinition.ValveCharacteristics\">User's Guide</a>.
</p>

</html>", revisions = "<html>
<ul>
<li><i>2 Nov 2005</i>
    by <a href=\"mailto:francesco.casella@polimi.it\">Francesco Casella</a>:<br>
       Adapted from the ThermoPower library.</li>
</ul>
</html>"));
      end ValveVaporizing;

      model ValveCompressible "Valve for compressible fluids, accounts for choked flow conditions"
        extends BaseClasses.PartialValve;
        import Modelica.Fluid.Types.CvTypes;
        import Modelica.Constants.pi;
        parameter Medium.AbsolutePressure p_nominal "Nominal inlet pressure" annotation(
          Dialog(group = "Nominal operating point"));
        parameter Real Fxt_full = 0.5 "Fk*xt critical ratio at full opening";
        replaceable function xtCharacteristic = Modelica.Fluid.Valves.BaseClasses.ValveCharacteristics.one constrainedby Modelica.Fluid.Valves.BaseClasses.ValveCharacteristics.baseFun "Critical ratio characteristic";
        Real Fxt;
        Real x "Pressure drop ratio";
        Real xs "Saturated pressure drop ratio";
        Real Y "Compressibility factor";
        Medium.AbsolutePressure p "Inlet pressure";
        constant SI.ReynoldsNumber Re_turbulent = 4000 "cf. straight pipe for fully open valve -- dp_turbulent increases for closing valve";
        parameter Boolean use_Re = system.use_eps_Re "= true, if turbulent region is defined by Re, otherwise by m_flow_small" annotation(
          Dialog(tab = "Advanced"),
          Evaluate = true);
        SI.AbsolutePressure dp_turbulent = if not use_Re then dp_small else max(dp_small, (Medium.dynamicViscosity(state_a) + Medium.dynamicViscosity(state_b)) ^ 2 * pi / 8 * Re_turbulent ^ 2 / (max(valveCharacteristic(opening_actual), 0.001) * Av * Y * (Medium.density(state_a) + Medium.density(state_b))));
      protected
        parameter Real Fxt_nominal(fixed = false) "Nominal Fxt";
        parameter Real x_nominal(fixed = false) "Nominal pressure drop ratio";
        parameter Real xs_nominal(fixed = false) "Nominal saturated pressure drop ratio";
        parameter Real Y_nominal(fixed = false) "Nominal compressibility factor";
      initial equation
        if CvData == CvTypes.OpPoint then
// Determination of Av by the nominal operating point conditions
          Fxt_nominal = Fxt_full * xtCharacteristic(opening_nominal);
          x_nominal = dp_nominal / p_nominal;
          xs_nominal = smooth(0, if x_nominal > Fxt_nominal then Fxt_nominal else x_nominal);
          Y_nominal = 1 - abs(xs_nominal) / (3 * Fxt_nominal);
          m_flow_nominal = valveCharacteristic(opening_nominal) * Av * Y_nominal * sqrt(rho_nominal) * Utilities.regRoot(p_nominal * xs_nominal, dp_small);
        else
// Dummy values
          Fxt_nominal = 0;
          x_nominal = 0;
          xs_nominal = 0;
          Y_nominal = 0;
        end if;
      equation
        p = max(port_a.p, port_b.p);
        Fxt = Fxt_full * xtCharacteristic(opening_actual);
        x = dp / p;
        xs = max(-Fxt, min(x, Fxt));
        Y = 1 - abs(xs) / (3 * Fxt);
// m_flow = valveCharacteristic(opening)*Av*Y*sqrt(d)*sqrt(p*xs);
        if checkValve then
          m_flow = homotopy(valveCharacteristic(opening_actual) * Av * Y * sqrt(Medium.density(state_a)) * (if xs >= 0 then Utilities.regRoot(p * xs, dp_turbulent) else 0), valveCharacteristic(opening_actual) * m_flow_nominal * dp / dp_nominal);
        elseif not allowFlowReversal then
          m_flow = homotopy(valveCharacteristic(opening_actual) * Av * Y * sqrt(Medium.density(state_a)) * Utilities.regRoot(p * xs, dp_turbulent), valveCharacteristic(opening_actual) * m_flow_nominal * dp / dp_nominal);
        else
          m_flow = homotopy(valveCharacteristic(opening_actual) * Av * Y * Utilities.regRoot2(p * xs, dp_turbulent, Medium.density(state_a), Medium.density(state_b)), valveCharacteristic(opening_actual) * m_flow_nominal * dp / dp_nominal);
/* alternative formulation using smooth(0, ...) -- should not be used as regRoot2 has continuous derivatives
   -- cf. ModelicaTest.Fluid.TestPipesAndValves.DynamicPipeInitialization --
    m_flow = homotopy(valveCharacteristic(opening_actual)*Av*Y*
                        smooth(0, Utilities.regRoot(p*xs, dp_turbulent)*
                        (if xs>=0 then sqrt(Medium.density(state_a)) else sqrt(Medium.density(state_b)))),
                      valveCharacteristic(opening_actual)*m_flow_nominal*dp/dp_nominal);
*/
        end if;
        annotation(
          Documentation(info = "<html>
<p>Valve model according to the IEC 534/ISA S.75 standards for valve sizing, compressible fluid, no phase change, also covering choked-flow conditions.</p>

<p>
The parameters of this model are explained in detail in
<a href=\"modelica://Modelica.Fluid.Valves.BaseClasses.PartialValve\">PartialValve</a>
(the base model for valves).
</p>

<p>This model can be used with gases and vapours, with arbitrary pressure ratio between inlet and outlet.</p>

<p>The product Fk*xt is given by the parameter <code>Fxt_full</code>, and is assumed constant by default. The relative change (per unit) of the xt coefficient with the valve opening can be specified by replacing the <code>xtCharacteristic</code> function.
<p>If <code>checkValve</code> is false, the valve supports reverse flow, with a symmetric flow characteristic curve. Otherwise, reverse flow is stopped (check valve behaviour).</p>

<p>
The treatment of parameters <b>Kv</b> and <b>Cv</b> is
explained in detail in the
<a href=\"modelica://Modelica.Fluid.UsersGuide.ComponentDefinition.ValveCharacteristics\">User's Guide</a>.
</p>

</html>", revisions = "<html>
<ul>
<li><i>2 Nov 2005</i>
    by <a href=\"mailto:francesco.casella@polimi.it\">Francesco Casella</a>:<br>
       Adapted from the ThermoPower library.</li>
</ul>
</html>"));
      end ValveCompressible;

      model ValveLinear "Valve for water/steam flows with linear pressure drop"
        extends Modelica.Fluid.Interfaces.PartialTwoPortTransport;
        parameter SI.AbsolutePressure dp_nominal "Nominal pressure drop at full opening" annotation(
          Dialog(group = "Nominal operating point"));
        parameter Medium.MassFlowRate m_flow_nominal "Nominal mass flowrate at full opening";
        final parameter Types.HydraulicConductance k = m_flow_nominal / dp_nominal "Hydraulic conductance at full opening";
        Modelica.Blocks.Interfaces.RealInput opening(min = 0, max = 1) "=1: completely open, =0: completely closed" annotation(
          Placement(transformation(origin = {0, 90}, extent = {{-20, -20}, {20, 20}}, rotation = 270), iconTransformation(extent = {{-20, -20}, {20, 20}}, rotation = 270, origin = {0, 80})));
      equation
        m_flow = opening * k * dp;
// Isenthalpic state transformation (no storage and no loss of energy)
        port_a.h_outflow = inStream(port_b.h_outflow);
        port_b.h_outflow = inStream(port_a.h_outflow);
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{0, 50}, {0, 0}}), Rectangle(extent = {{-20, 60}, {20, 50}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Polygon(points = {{-100, 50}, {100, -50}, {100, 50}, {0, 0}, {-100, -50}, {-100, 50}}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Polygon(points = DynamicSelect({{-100, 0}, {100, -0}, {100, 0}, {0, 0}, {-100, -0}, {-100, 0}}, {{-100, 50 * opening}, {-100, 50 * opening}, {100, -50 * opening}, {100, 50 * opening}, {0, 0}, {-100, -50 * opening}, {-100, 50 * opening}}), fillColor = {0, 255, 0}, lineColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Polygon(points = {{-100, 50}, {100, -50}, {100, 50}, {0, 0}, {-100, -50}, {-100, 50}}, lineColor = {0, 0, 0})}),
          Documentation(info = "<html>
<p>This very simple model provides a pressure drop which is proportional to the flowrate and to the <code>opening</code> input, without computing any fluid property. It can be used for testing purposes, when
a simple model of a variable pressure loss is needed.</p>
<p>A medium model must be nevertheless be specified, so that the fluid ports can be connected to other components using the same medium model.</p>
<p>The model is adiabatic (no heat losses to the ambient) and neglects changes in kinetic energy from the inlet to the outlet.</p>
</html>", revisions = "<html>
<ul>
<li><i>2 Nov 2005</i>
    by <a href=\"mailto:francesco.casella@polimi.it\">Francesco Casella</a>:<br>
       Adapted from the ThermoPower library.</li>
</ul>
</html>"));
      end ValveLinear;

      model ValveDiscrete "Valve for water/steam flows with linear pressure drop"
        extends Modelica.Fluid.Interfaces.PartialTwoPortTransport;
        parameter SI.AbsolutePressure dp_nominal "Nominal pressure drop at full opening=1" annotation(
          Dialog(group = "Nominal operating point"));
        parameter Medium.MassFlowRate m_flow_nominal "Nominal mass flowrate at full opening=1";
        final parameter Types.HydraulicConductance k = m_flow_nominal / dp_nominal "Hydraulic conductance at full opening=1";
        Modelica.Blocks.Interfaces.BooleanInput open annotation(
          Placement(transformation(origin = {0, 80}, extent = {{-20, -20}, {20, 20}}, rotation = 270)));
        parameter Real opening_min(min = 0) = 0 "Remaining opening if closed, causing small leakage flow";
      equation
        m_flow = if open then 1 * k * dp else opening_min * k * dp;
// Isenthalpic state transformation (no storage and no loss of energy)
        port_a.h_outflow = inStream(port_b.h_outflow);
        port_b.h_outflow = inStream(port_a.h_outflow);
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{0, 50}, {0, 0}}), Rectangle(extent = {{-20, 60}, {20, 50}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Polygon(points = {{-100, 50}, {100, -50}, {100, 50}, {0, 0}, {-100, -50}, {-100, 50}}, fillColor = DynamicSelect({255, 255, 255}, if open > 0.5 then {0, 255, 0} else {255, 255, 255}), lineColor = {0, 0, 0}, fillPattern = FillPattern.Solid)}),
          Documentation(info = "<html>
<p>
This very simple model provides a (small) pressure drop which is proportional to the flowrate if the Boolean open signal is <b>true</b>. Otherwise, the mass flow rate is zero. If opening_min > 0, a small leakage mass flow rate occurs when open = <b>false</b>.
</p>
<p>This model can be used for simplified modelling of on-off valves, when it is not important to accurately describe the pressure loss when the valve is open. Although the medium model is not used to determine the pressure loss, it must be nevertheless be specified, so that the fluid ports can be connected to other components using the same medium model.</p>
<p>The model is adiabatic (no heat losses to the ambient) and neglects changes in kinetic energy from the inlet to the outlet.</p>
<p>
In a diagram animation, the valve is shown in \"green\", when
it is open.
</p>
</html>", revisions = "<html>
<ul>
<li><i>Nov 2005</i>
    by Katja Poschlad (based on ValveLinear).</li>
</ul>
</html>"));
      end ValveDiscrete;

      package BaseClasses "Base classes used in the Valves package (only of interest to build new component models)"
        extends Modelica.Icons.BasesPackage;

        partial model PartialValve "Base model for valves"
          import Modelica.Fluid.Types.CvTypes;
          extends Modelica.Fluid.Interfaces.PartialTwoPortTransport(dp_start = dp_nominal, m_flow_small = if system.use_eps_Re then system.eps_m_flow * m_flow_nominal else system.m_flow_small, m_flow_start = m_flow_nominal);
          parameter Modelica.Fluid.Types.CvTypes CvData = Modelica.Fluid.Types.CvTypes.OpPoint "Selection of flow coefficient" annotation(
            Dialog(group = "Flow Coefficient"));
          parameter SI.Area Av(fixed = if CvData == Modelica.Fluid.Types.CvTypes.Av then true else false, start = m_flow_nominal / sqrt(rho_nominal * dp_nominal) * valveCharacteristic(opening_nominal)) "Av (metric) flow coefficient" annotation(
            Dialog(group = "Flow Coefficient", enable = CvData == Modelica.Fluid.Types.CvTypes.Av));
          parameter Real Kv = 0 "Kv (metric) flow coefficient [m3/h]" annotation(
            Dialog(group = "Flow Coefficient", enable = CvData == Modelica.Fluid.Types.CvTypes.Kv));
          parameter Real Cv = 0 "Cv (US) flow coefficient [USG/min]" annotation(
            Dialog(group = "Flow Coefficient", enable = CvData == Modelica.Fluid.Types.CvTypes.Cv));
          parameter SI.Pressure dp_nominal "Nominal pressure drop" annotation(
            Dialog(group = "Nominal operating point"));
          parameter Medium.MassFlowRate m_flow_nominal "Nominal mass flowrate" annotation(
            Dialog(group = "Nominal operating point"));
          parameter Medium.Density rho_nominal = Medium.density_pTX(Medium.p_default, Medium.T_default, Medium.X_default) "Nominal inlet density" annotation(
            Dialog(group = "Nominal operating point", enable = CvData == Modelica.Fluid.Types.CvTypes.OpPoint));
          parameter Real opening_nominal(min = 0, max = 1) = 1 "Nominal opening" annotation(
            Dialog(group = "Nominal operating point", enable = CvData == Modelica.Fluid.Types.CvTypes.OpPoint));
          parameter Boolean filteredOpening = false "= true, if opening is filtered with a 2nd order CriticalDamping filter" annotation(
            Dialog(group = "Filtered opening"),
            choices(checkBox = true));
          parameter Modelica.SIunits.Time riseTime = 1 "Rise time of the filter (time to reach 99.6 % of an opening step)" annotation(
            Dialog(group = "Filtered opening", enable = filteredOpening));
          parameter Real leakageOpening(min = 0, max = 1) = 1e-3 "The opening signal is limited by leakageOpening (to improve the numerics)" annotation(
            Dialog(group = "Filtered opening", enable = filteredOpening));
          parameter Boolean checkValve = false "Reverse flow stopped" annotation(
            Dialog(tab = "Assumptions"));
          replaceable function valveCharacteristic = Modelica.Fluid.Valves.BaseClasses.ValveCharacteristics.linear constrainedby Modelica.Fluid.Valves.BaseClasses.ValveCharacteristics.baseFun "Inherent flow characteristic" annotation(
             choicesAllMatching = true);
        protected
          parameter SI.Pressure dp_small = if system.use_eps_Re then dp_nominal / m_flow_nominal * m_flow_small else system.dp_small "Regularisation of zero flow" annotation(
            Dialog(tab = "Advanced"));
        public
          constant SI.Area Kv2Av = 27.7e-6 "Conversion factor";
          constant SI.Area Cv2Av = 24.0e-6 "Conversion factor";
          Modelica.Blocks.Interfaces.RealInput opening(min = 0, max = 1) "Valve position in the range 0..1" annotation(
            Placement(transformation(origin = {0, 90}, extent = {{-20, -20}, {20, 20}}, rotation = 270), iconTransformation(extent = {{-20, -20}, {20, 20}}, rotation = 270, origin = {0, 80})));
          Modelica.Blocks.Interfaces.RealOutput opening_filtered if filteredOpening "Filtered valve position in the range 0..1" annotation(
            Placement(transformation(extent = {{60, 40}, {80, 60}}), iconTransformation(extent = {{60, 50}, {80, 70}})));
          Modelica.Blocks.Continuous.Filter filter(order = 2, f_cut = 5 / (2 * Modelica.Constants.pi * riseTime)) if filteredOpening annotation(
            Placement(transformation(extent = {{34, 44}, {48, 58}})));
        protected
          Modelica.Blocks.Interfaces.RealOutput opening_actual annotation(
            Placement(transformation(extent = {{60, 10}, {80, 30}})));

          block MinLimiter "Limit the signal above a threshold"
            parameter Real uMin = 0 "Lower limit of input signal";
            extends Modelica.Blocks.Interfaces.SISO;
          equation
            y = smooth(0, noEvent(if u < uMin then uMin else u));
            annotation(
              Documentation(info = "<html>
<p>
The block passes its input signal as output signal
as long as the input is above uMin. If this is not the case,
y=uMin is passed as output.
</p>
</html>"),
              Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{0, -90}, {0, 68}}, color = {192, 192, 192}), Polygon(points = {{0, 90}, {-8, 68}, {8, 68}, {0, 90}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-90, 0}, {68, 0}}, color = {192, 192, 192}), Polygon(points = {{90, 0}, {68, -8}, {68, 8}, {90, 0}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-80, -70}, {-50, -70}, {50, 70}, {64, 90}}), Text(extent = {{-150, -150}, {150, -110}}, lineColor = {0, 0, 0}, textString = "uMin=%uMin"), Text(extent = {{-150, 150}, {150, 110}}, textString = "%name", lineColor = {0, 0, 255})}),
              Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{0, -60}, {0, 50}}, color = {192, 192, 192}), Polygon(points = {{0, 60}, {-5, 50}, {5, 50}, {0, 60}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-60, 0}, {50, 0}}, color = {192, 192, 192}), Polygon(points = {{60, 0}, {50, -5}, {50, 5}, {60, 0}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-50, -40}, {-30, -40}, {30, 40}, {50, 40}}), Text(extent = {{46, -6}, {68, -18}}, lineColor = {128, 128, 128}, textString = "u"), Text(extent = {{-30, 70}, {-5, 50}}, lineColor = {128, 128, 128}, textString = "y"), Text(extent = {{-58, -54}, {-28, -42}}, lineColor = {128, 128, 128}, textString = "uMin"), Text(extent = {{26, 40}, {66, 56}}, lineColor = {128, 128, 128}, textString = "uMax")}));
          end MinLimiter;

          MinLimiter minLimiter(uMin = leakageOpening) annotation(
            Placement(transformation(extent = {{10, 44}, {24, 58}})));
        initial equation
          if CvData == CvTypes.Kv then
            Av = Kv * Kv2Av "Unit conversion";
          elseif CvData == CvTypes.Cv then
            Av = Cv * Cv2Av "Unit conversion";
          end if;
        equation
// Isenthalpic state transformation (no storage and no loss of energy)
          port_a.h_outflow = inStream(port_b.h_outflow);
          port_b.h_outflow = inStream(port_a.h_outflow);
          connect(filter.y, opening_filtered) annotation(
            Line(points = {{48.7, 51}, {60, 51}, {60, 50}, {70, 50}}, color = {0, 0, 127}));
          if filteredOpening then
            connect(filter.y, opening_actual);
          else
            connect(opening, opening_actual);
          end if;
          connect(minLimiter.y, filter.u) annotation(
            Line(points = {{24.7, 51}, {32.6, 51}}, color = {0, 0, 127}));
          connect(minLimiter.u, opening) annotation(
            Line(points = {{8.6, 51}, {0, 51}, {0, 90}}, color = {0, 0, 127}));
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{0, 52}, {0, 0}}), Rectangle(extent = {{-20, 60}, {20, 52}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Polygon(points = {{-100, 50}, {100, -50}, {100, 50}, {0, 0}, {-100, -50}, {-100, 50}}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Polygon(points = DynamicSelect({{-100, 0}, {100, -0}, {100, 0}, {0, 0}, {-100, -0}, {-100, 0}}, {{-100, 50 * opening_actual}, {-100, 50 * opening_actual}, {100, -50 * opening}, {100, 50 * opening_actual}, {0, 0}, {-100, -50 * opening_actual}, {-100, 50 * opening}}), fillColor = {0, 255, 0}, lineColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Polygon(points = {{-100, 50}, {100, -50}, {100, 50}, {0, 0}, {-100, -50}, {-100, 50}}, lineColor = {0, 0, 0}), Ellipse(visible = filteredOpening, extent = {{-40, 94}, {40, 14}}, lineColor = {0, 0, 127}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(visible = filteredOpening, points = {{-20, 25}, {-20, 63}, {0, 41}, {20, 63}, {20, 25}}, thickness = 0.5), Line(visible = filteredOpening, points = {{40, 60}, {60, 60}}, color = {0, 0, 127})}),
            Documentation(info = "<html>
<p>This is the base model for the <code>ValveIncompressible</code>, <code>ValveVaporizing</code>, and <code>ValveCompressible</code> valve models. The model is based on the IEC 534 / ISA S.75 standards for valve sizing.</p>
<p>The model optionally supports reverse flow conditions (assuming symmetrical behaviour) or check valve operation, and has been suitably regularized, compared to the equations in the standard, in order to avoid numerical singularities around zero pressure drop operating conditions.</p>
<p>The model assumes adiabatic operation (no heat losses to the ambient); changes in kinetic energy
from inlet to outlet are neglected in the energy balance.</p>
<p><b>Modelling options</b></p>
<p>The following options are available to specify the valve flow coefficient in fully open conditions:
<ul><li><code>CvData = Modelica.Fluid.Types.CvTypes.Av</code>: the flow coefficient is given by the metric <code>Av</code> coefficient (m^2).
<li><code>CvData = Modelica.Fluid.Types.CvTypes.Kv</code>: the flow coefficient is given by the metric <code>Kv</code> coefficient (m^3/h).
<li><code>CvData = Modelica.Fluid.Types.CvTypes.Cv</code>: the flow coefficient is given by the US <code>Cv</code> coefficient (USG/min).
<li><code>CvData = Modelica.Fluid.Types.CvTypes.OpPoint</code>: the flow is computed from the nominal operating point specified by <code>p_nominal</code>, <code>dp_nominal</code>, <code>m_flow_nominal</code>, <code>rho_nominal</code>, <code>opening_nominal</code>.
</ul>
<p>The nominal pressure drop <code>dp_nominal</code> must always be specified; to avoid numerical singularities, the flow characteristic is modified for pressure drops less than <code>b*dp_nominal</code> (the default value is 1% of the nominal pressure drop). Increase this parameter if numerical problems occur in valves with very low pressure drops.
<p>If <code>checkValve</code> is true, then the flow is stopped when the outlet pressure is higher than the inlet pressure; otherwise, reverse flow takes place. Use this option only when needed, as it increases the numerical complexity of the problem.
<p>The valve opening characteristic <code>valveCharacteristic</code>, linear by default, can be replaced by any user-defined function. Quadratic and equal percentage with customizable rangeability are already provided by the library. The characteristics for constant port_a.p and port_b.p pressures with continuously changing opening are shown in the next two figures:
</p>

<blockquote>
<img src=\"modelica://Modelica/Resources/Images/Fluid/Components/ValveCharacteristics1a.png\"
     alt=\"ValveCharacteristics1a.png\"><br>
<img src=\"modelica://Modelica/Resources/Images/Fluid/Components/ValveCharacteristics1b.png\"
     alt=\"Components/ValveCharacteristics1b.png\">
</blockquote>

<p>
The treatment of parameters <b>Kv</b> and <b>Cv</b> is
explained in detail in the
<a href=\"modelica://Modelica.Fluid.UsersGuide.ComponentDefinition.ValveCharacteristics\">User's Guide</a>.
</p>

<p>
With the optional parameter \"filteredOpening\", the opening can be filtered with a
<b>second order, criticalDamping</b> filter so that the
opening demand is delayed by parameter \"riseTime\". The filtered opening is then available
via the output signal \"opening_filtered\" and is used to control the valve equations.
This approach approximates the driving device of a valve. The \"riseTime\" parameter
is used to compute the cut-off frequency of the filter by the equation: f_cut = 5/(2*pi*riseTime).
It defines the time that is needed until opening_filtered reaches 99.6 % of
a step input of opening. The icon of a valve changes in the following way
(left image: filteredOpening=false, right image: filteredOpening=true):
</p>

<blockquote>
<img src=\"modelica://Modelica/Resources/Images/Fluid/Components/FilteredValveIcon.png\"
     alt=\"FilteredValveIcon.png\">
</blockquote>

<p>
If \"filteredOpening = <b>true</b>\", the input signal \"opening\" is limited
by parameter <b>leakageOpening</b>, i.e., if \"opening\" becomes smaller as
\"leakageOpening\", then \"leakageOpening\" is used instead of \"opening\" as input
for the filter. The reason is that \"opening=0\" might structurally change the equations of the
fluid network leading to a singularity. If a small leakage flow is introduced
(which is often anyway present in reality), the singularity might be avoided.
</p>

<p>
In the next figure, \"opening\" and \"filtered_opening\" are shown in the case that
filteredOpening = <b>true</b>, riseTime = 1 s, and leakageOpening = 0.02.
</p>

<blockquote>
<img src=\"modelica://Modelica/Resources/Images/Fluid/Components/ValveFilteredOpening.png\"
     alt=\"ValveFilteredOpening.png\">
</blockquote>

</html>", revisions = "<html>
<ul>
<li><i>Sept. 5, 2010</i>
    by <a href=\"mailto:martin.otter@dlr.de\">Martin Otter</a>:<br>
    Optional filtering of opening introduced, based on a proposal
    from Mike Barth (Universitaet der Bundeswehr Hamburg) +
    Documentation improved.</li>
<li><i>2 Nov 2005</i>
    by <a href=\"mailto:francesco.casella@polimi.it\">Francesco Casella</a>:<br>
       Adapted from the ThermoPower library.</li>
</ul>
</html>"));
        end PartialValve;

        package ValveCharacteristics "Functions for valve characteristics"
          extends Modelica.Icons.VariantsPackage;

          partial function baseFun "Base class for valve characteristics"
            extends Modelica.Icons.Function;
            input Real pos(min = 0, max = 1) "Opening position (0: closed, 1: fully open)";
            output Real rc "Relative flow coefficient (per unit)";
            annotation(
              Documentation(info = "<html>
<p>
This is a partial function that defines the interface of valve
characteristics. The function returns \"rc = valveCharacteristic\" as function of the
opening \"pos\" (in the range 0..1):
</p>

<blockquote><pre>
    dp = (zeta_TOT/2) * rho * velocity^2
m_flow =    sqrt(2/zeta_TOT) * Av * sqrt(rho * dp)
m_flow = valveCharacteristic * Av * sqrt(rho * dp)
m_flow =                  rc * Av * sqrt(rho * dp)
</pre></blockquote>

</html>"));
          end baseFun;

          function linear "Linear characteristic"
            extends baseFun;
          algorithm
            rc := pos;
          end linear;

          function one "Constant characteristic"
            extends baseFun;
          algorithm
            rc := 1;
          end one;

          function quadratic "Quadratic characteristic"
            extends baseFun;
          algorithm
            rc := pos * pos;
          end quadratic;

          function equalPercentage "Equal percentage characteristic"
            extends baseFun;
            input Real rangeability = 20 "Rangeability" annotation(
              Dialog);
            input Real delta = 0.01 annotation(
              Dialog);
          algorithm
            rc := if pos > delta then rangeability ^ (pos - 1) else pos / delta * rangeability ^ (delta - 1);
            annotation(
              Documentation(info = "<html>
This characteristic is such that the relative change of the flow coefficient is proportional to the change in the opening position:
<p> d(rc)/d(pos) = k d(pos).
<p> The constant k is expressed in terms of the rangeability, i.e., the ratio between the maximum and the minimum useful flow coefficient:
<p> rangeability = exp(k) = rc(1.0)/rc(0.0).
<p> The theoretical characteristic has a non-zero opening when pos = 0; the implemented characteristic is modified so that the valve closes linearly when pos &lt; delta.
</html>"));
          end equalPercentage;
        end ValveCharacteristics;
      end BaseClasses;
      annotation(
        Documentation(info = "<html>

</html>"));
    end Valves;

    package Fittings "Adaptors for connections of fluid components and the regulation of fluid flow"
      package Bends "Flow models for bends"
        extends Modelica.Icons.VariantsPackage;

        model CurvedBend "Curved bend flow model"
          extends Modelica.Fluid.Dissipation.Utilities.Icons.PressureLoss.Bend_i;
          extends Modelica.Fluid.Interfaces.PartialPressureLoss;
          parameter Modelica.Fluid.Fittings.BaseClasses.Bends.CurvedBend.Geometry geometry "Geometry of curved bend" annotation(
            Placement(transformation(extent = {{-20, 0}, {0, 20}})));
        protected
          parameter Medium.AbsolutePressure dp_small(min = 0) = Modelica.Fluid.Dissipation.PressureLoss.Bend.dp_curvedOverall_DP(geometry, Modelica.Fluid.Dissipation.PressureLoss.Bend.dp_curvedOverall_IN_var(rho = Medium.density(state_dp_small), eta = Medium.dynamicViscosity(state_dp_small)), m_flow_small) "Default small pressure drop for regularization of laminar and zero flow (calculated from m_flow_small)";
        equation
          if allowFlowReversal then
            m_flow = Modelica.Fluid.Fittings.BaseClasses.Bends.CurvedBend.massFlowRate(dp, geometry, d_a, d_b, eta_a, eta_b, dp_small, m_flow_small);
          else
            m_flow = Modelica.Fluid.Dissipation.PressureLoss.Bend.dp_curvedOverall_MFLOW(geometry, Modelica.Fluid.Dissipation.PressureLoss.Bend.dp_curvedOverall_IN_var(rho = d_a, eta = eta_a), dp);
          end if;
          annotation(
            Documentation(info = "<html>
<p>
This component models a <b>curved bend</b> in the overall flow regime for incompressible and single-phase fluid flow through circular cross sectional area considering surface roughness. It is expected that also compressible fluid flow can be handled up to about Ma = 0.3. It is assumed that neither mass nor energy is stored in this component.
In the model basically a function is called to compute the mass flow rate as a function
of pressure loss for a curved bend. Also the inverse of this function is defined, and a tool
might use this inverse function instead, in order to avoid the solution of a nonlinear equation.
</p>

<p>
The details of the model are described in the
<a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.PressureLoss.Bend.dp_curvedOverall\">documentation of the underlying function</a>.
</p>
</html>"));
        end CurvedBend;

        model EdgedBend "Edged bend flow model"
          extends Modelica.Fluid.Dissipation.Utilities.Icons.PressureLoss.Bend_i;
          extends Modelica.Fluid.Interfaces.PartialPressureLoss;
          parameter Modelica.Fluid.Fittings.BaseClasses.Bends.EdgedBend.Geometry geometry "Geometry of curved bend" annotation(
            Placement(transformation(extent = {{-20, 0}, {0, 20}})));
        protected
          parameter Medium.AbsolutePressure dp_small(min = 0) = Modelica.Fluid.Dissipation.PressureLoss.Bend.dp_edgedOverall_DP(Modelica.Fluid.Dissipation.PressureLoss.Bend.dp_edgedOverall_IN_con(d_hyd = geometry.d_hyd, delta = geometry.delta, K = geometry.K), Modelica.Fluid.Dissipation.PressureLoss.Bend.dp_edgedOverall_IN_var(rho = Medium.density(state_dp_small), eta = Medium.dynamicViscosity(state_dp_small)), m_flow_small) "Default small pressure drop for regularization of laminar and zero flow (calculated from m_flow_small)";
        equation
          if allowFlowReversal then
            m_flow = Modelica.Fluid.Fittings.BaseClasses.Bends.EdgedBend.massFlowRate(dp, geometry, d_a, d_b, eta_a, eta_b, dp_small, m_flow_small);
          else
            m_flow = Modelica.Fluid.Dissipation.PressureLoss.Bend.dp_edgedOverall_MFLOW(Modelica.Fluid.Dissipation.PressureLoss.Bend.dp_edgedOverall_IN_con(d_hyd = geometry.d_hyd, delta = geometry.delta, K = geometry.K), Modelica.Fluid.Dissipation.PressureLoss.Bend.dp_edgedOverall_IN_var(rho = d_a, eta = eta_a), dp);
          end if;
          annotation(
            Documentation(info = "<html>
<p>
This component models an <b>edged bend</b> in the overall flow regime for incompressible and single-phase fluid flow through circular cross sectional area considering surface roughness. It is expected that also compressible fluid flow can be handled up to about Ma = 0.3. It is assumed that neither mass nor energy is stored in this component.
In the model basically a function is called to compute the mass flow rate as a function
of pressure loss for an edged bend. Also the inverse of this function is defined, and a tool
might use this inverse function instead, in order to avoid the solution of a nonlinear equation.
</p>

<p>
The details of the model are described in the
<a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.PressureLoss.Bend.dp_edgedOverall\">documentation of the underlying function</a>.
</p>

</html>"));
        end EdgedBend;
      end Bends;

      package Orifices "Flow models for orifices"
        extends Modelica.Icons.VariantsPackage;

        model ThickEdgedOrifice "Thicked edged orifice flow model"
          extends Modelica.Fluid.Dissipation.Utilities.Icons.PressureLoss.Orifice_i;
          extends Modelica.Fluid.Interfaces.PartialPressureLoss;
          parameter Modelica.Fluid.Fittings.BaseClasses.Orifices.ThickEdgedOrifice.Geometry geometry "Geometry of thick edged orifice" annotation(
            Placement(transformation(extent = {{-20, 0}, {0, 20}})),
            choices(choice = Modelica.Fluid.Fittings.BaseClasses.Orifices.ThickEdgedOrifice.Choices.circular(), choice = Modelica.Fluid.Fittings.BaseClasses.Orifices.ThickEdgedOrifice.Choices.rectangular(), choice = Modelica.Fluid.Fittings.BaseClasses.Orifices.ThickEdgedOrifice.Choices.general()));
        protected
          parameter Medium.AbsolutePressure dp_small(min = 0) = Modelica.Fluid.Dissipation.PressureLoss.Orifice.dp_thickEdgedOverall_DP(Modelica.Fluid.Dissipation.PressureLoss.Orifice.dp_thickEdgedOverall_IN_con(A_0 = geometry.venaCrossArea, A_1 = geometry.crossArea, C_0 = geometry.venaPerimeter, C_1 = geometry.perimeter, L = geometry.venaLength, dp_smooth = 1e-10), Modelica.Fluid.Dissipation.PressureLoss.Orifice.dp_thickEdgedOverall_IN_var(rho = Medium.density(state_dp_small), eta = Medium.dynamicViscosity(state_dp_small)), m_flow_small) "Default small pressure drop for regularization of laminar and zero flow (calculated from m_flow_small)";
        equation
          if allowFlowReversal then
            m_flow = Modelica.Fluid.Fittings.BaseClasses.Orifices.ThickEdgedOrifice.massFlowRate(dp, geometry, d_a, d_b, eta_a, eta_b, dp_small, m_flow_small);
          else
            m_flow = Modelica.Fluid.Dissipation.PressureLoss.Orifice.dp_thickEdgedOverall_MFLOW(Modelica.Fluid.Dissipation.PressureLoss.Orifice.dp_thickEdgedOverall_IN_con(A_0 = geometry.venaCrossArea, A_1 = geometry.crossArea, C_0 = geometry.venaPerimeter, C_1 = geometry.perimeter, L = geometry.venaLength, dp_smooth = dp_small), Modelica.Fluid.Dissipation.PressureLoss.Orifice.dp_thickEdgedOverall_IN_var(rho = d_a, eta = eta_a), dp);
          end if;
          annotation(
            Documentation(info = "<html>
<p>
This component models a <b>thick edged orifice</b> with sharp corners in the overall flow regime for incompressible and single-phase fluid flow through an arbitrary shaped cross sectional area (square, circular, etc.) considering  influence of surface roughness. It is expected that also compressible fluid flow can be handled up to about Ma = 0.3. It is assumed that neither mass nor energy is stored in this component.
In the model basically a function is called to compute the mass flow rate as a function
of pressure loss for a thick edged orifice. Also the inverse of this function is defined, and a tool
might use this inverse function instead, in order to avoid the solution of a nonlinear equation.
</p>

<p>
The details of the model are described in the
<a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.PressureLoss.Orifice.dp_thickEdgedOverall\">documentation of the underlying function</a>.
</p>

</html>"));
        end ThickEdgedOrifice;
      end Orifices;

      package GenericResistances "Flow models for generic resistances"
        extends Modelica.Icons.VariantsPackage;

        model VolumeFlowRate "Flow model for generic resistance parameterized with the volume flow rate"
          extends Modelica.Fluid.Dissipation.Utilities.Icons.PressureLoss.General_i;
          extends Modelica.Fluid.Interfaces.PartialTwoPortTransport;
          parameter Real a(unit = "(Pa.s2)/m6") "Coefficient for quadratic term" annotation(
            Dialog(group = "dp = a*V_flow^2 + b*V_flow"));
          parameter Real b(unit = "(Pa.s)/m3") "Coefficient for linear term" annotation(
            Dialog(group = "dp = a*V_flow^2 + b*V_flow"));
        protected
          parameter Medium.ThermodynamicState state_dp_small = Medium.setState_pTX(Medium.reference_p, Medium.reference_T, Medium.reference_X) "Medium state to compute dp_small";
          parameter Medium.AbsolutePressure dp_small(min = 0) = Modelica.Fluid.Dissipation.PressureLoss.General.dp_volumeFlowRate_DP(Modelica.Fluid.Dissipation.PressureLoss.General.dp_volumeFlowRate_IN_con(a = a, b = b, dp_min = 1e-10), Modelica.Fluid.Dissipation.PressureLoss.General.dp_volumeFlowRate_IN_var(rho = Medium.density(state_dp_small)), m_flow_small) "Default small pressure drop for regularization of laminar and zero flow (calculated from m_flow_small)";
          Medium.Density d_a "Density at port_a when fluid is flowing from port_a to port_b";
          Medium.Density d_b "If allowFlowReversal=true then Density at port_b when fluid is flowing from port_b to port_a else d_a";
        equation
// Isenthalpic state transformation (no storage and no loss of energy)
          port_a.h_outflow = inStream(port_b.h_outflow);
          port_b.h_outflow = inStream(port_a.h_outflow);
// Medium properties
          d_a = Medium.density(state_a);
          if allowFlowReversal then
            d_b = Medium.density(state_b);
          else
            d_b = d_a;
          end if;
          if allowFlowReversal then
            m_flow = Modelica.Fluid.Fittings.BaseClasses.GenericResistances.VolumeFlowRate.massFlowRate(dp, a, b, d_a, d_b, dp_small, m_flow_small);
          else
            m_flow = Modelica.Fluid.Dissipation.PressureLoss.General.dp_volumeFlowRate_MFLOW(Modelica.Fluid.Dissipation.PressureLoss.General.dp_volumeFlowRate_IN_con(a = a, b = b, dp_min = dp_small), Modelica.Fluid.Dissipation.PressureLoss.General.dp_volumeFlowRate_IN_var(rho = d_a), dp);
          end if;
          annotation(
            Documentation(info = "<html>
<p>
This component models a generic resistance parameterized
with the volume flow rate:
</p>

<pre>
    dp     = a*V_flow^2 + b*V_flow
    m_flow = rho*V_flow
</pre>

<p>
with
</p>

<table>
<tr><td><b> a              </b></td><td> as quadratic coefficient [Pa*s^2/m^6],</td></tr>
<tr><td><b> b              </b></td><td> as linear coefficient [Pa*s/m3],</td></tr>
<tr><td><b> dp             </b></td><td> as pressure loss [Pa],</td></tr>
<tr><td><b> m_flow         </b></td><td> as mass flow rate [kg/s],</td></tr>
<tr><td><b> rho            </b></td><td> as density of fluid [kg/m3],</td></tr>
<tr><td><b> V_flow         </b></td><td> as volume flow rate [m3/s].</td></tr>
</table>

<p>
The geometry parameters of energy devices necessary for the pressure loss calculations are often not exactly known. Therefore the modelling of the detailed pressure loss calculation has to be simplified. This components use a linear and a quadratic dependence of the pressure loss on the volume flow rate. It is assumed that neither mass nor energy is stored in this component.
In the model basically a function is called to compute the mass flow rate as a function
of pressure loss. Also the inverse of this function is defined, and a tool
might use this inverse function instead, in order to avoid the solution of a nonlinear equation.
</p>

<p>
The details of the model are described in the
<a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.PressureLoss.General.dp_volumeFlowRate\">documentation of the underlying function</a>.
</p>

</html>"));
        end VolumeFlowRate;
        annotation(
          Documentation(info = "<html>
<p>
The geometry parameters of energy devices necessary for the pressure loss
calculations are often not exactly known. Therefore the modelling of the detailed
pressure loss calculation has to be simplified.
In this package components are present that provide different forms of
such approximations.
</p>
</html>"));
      end GenericResistances;

      extends Modelica.Icons.VariantsPackage;

      model SimpleGenericOrifice "Simple generic orifice defined by pressure loss coefficient and diameter (only for flow from port_a to port_b)"
        extends Modelica.Fluid.Interfaces.PartialTwoPortTransport(dp_start = dp_nominal, m_flow_small = if system.use_eps_Re then system.eps_m_flow * m_flow_nominal else system.m_flow_small, m_flow(stateSelect = if momentumDynamics == Types.Dynamics.SteadyState then StateSelect.default else StateSelect.prefer));
        extends Modelica.Fluid.Interfaces.PartialLumpedFlow(final pathLength = 0, final momentumDynamics = Types.Dynamics.SteadyState);
        parameter SI.Diameter diameter "Diameter of orifice";
        parameter Real zeta "Loss factor for flow of port_a -> port_b" annotation(
          Dialog(enable = use_zeta));
        parameter Boolean use_zeta = true "= false to obtain zeta from dp_nominal and m_flow_nominal";
        // Operational conditions
        parameter SI.MassFlowRate m_flow_nominal = if system.use_eps_Re then system.m_flow_nominal else 1e2 * system.m_flow_small "Mass flow rate for dp_nominal" annotation(
          Dialog(group = "Nominal operating point"));
        parameter SI.Pressure dp_nominal = if not system.use_eps_Re then 1e3 else BaseClasses.lossConstant_D_zeta(diameter, zeta) / Medium.density_pTX(Medium.p_default, Medium.T_default, Medium.X_default) * m_flow_nominal ^ 2 "Nominal pressure drop" annotation(
          Dialog(group = "Nominal operating point"));
        parameter Boolean use_Re = system.use_eps_Re "= true, if turbulent region is defined by Re, otherwise by m_flow_small" annotation(
          Dialog(tab = "Advanced"),
          Evaluate = true);
        parameter Boolean from_dp = true "= true, use m_flow = f(dp) else dp = f(m_flow)" annotation(
          Evaluate = true,
          Dialog(tab = "Advanced"));
      protected
        parameter Medium.AbsolutePressure dp_small(min = 0) = if system.use_eps_Re then dp_nominal / m_flow_nominal * m_flow_small else system.dp_small "Regularization of zero flow if |dp| < dp_small" annotation(
          Dialog(tab = "Advanced", enable = not use_Re and from_dp));
        // Variables
      public
        Real zeta_nominal;
        Medium.Density d = 0.5 * (Medium.density(state_a) + Medium.density(state_b));
        Modelica.SIunits.Pressure dp_fg(start = dp_start) "pressure loss due to friction and gravity";
        Modelica.SIunits.Area A_mean = Modelica.Constants.pi / 4 * diameter ^ 2 "mean cross flow area";
        constant SI.ReynoldsNumber Re_turbulent = 10000 "cf. sharpEdgedOrifice";
        SI.MassFlowRate m_flow_turbulent = if not use_Re then m_flow_small else max(m_flow_small, Modelica.Constants.pi / 8 * diameter * (Medium.dynamicViscosity(state_a) + Medium.dynamicViscosity(state_b)) * Re_turbulent);
        SI.AbsolutePressure dp_turbulent = if not use_Re then dp_small else max(dp_small, BaseClasses.lossConstant_D_zeta(diameter, zeta_nominal) / d * m_flow_turbulent ^ 2);
      equation
        if use_zeta then
          zeta_nominal = zeta;
        else
          zeta_nominal = 2 * A_mean ^ 2 * d * dp_nominal / m_flow_nominal ^ 2;
        end if;
        Ib_flow = 0;
        F_p = A_mean * (Medium.pressure(state_b) - Medium.pressure(state_a));
        F_fg = A_mean * dp_fg;
/*
   dp = 0.5*zeta*d*v*|v|
      = 0.5*zeta*d*1/(d*A)^2 * m_flow * |m_flow|
      = 0.5*zeta/A^2 *1/d * m_flow * |m_flow|
      = k/d * m_flow * |m_flow|
   k  = 0.5*zeta/A^2
      = 0.5*zeta/(pi*(D/2)^2)^2
      = 8*zeta/(pi*D^2)^2
  */
        if from_dp then
          m_flow = homotopy(Utilities.regRoot2(dp_fg, dp_turbulent, Medium.density(state_a) / BaseClasses.lossConstant_D_zeta(diameter, zeta_nominal), Medium.density(state_b) / BaseClasses.lossConstant_D_zeta(diameter, zeta_nominal)), m_flow_nominal * dp_fg / dp_nominal);
        else
          dp_fg = homotopy(Utilities.regSquare2(m_flow, m_flow_turbulent, BaseClasses.lossConstant_D_zeta(diameter, zeta_nominal) / Medium.density(state_a), BaseClasses.lossConstant_D_zeta(diameter, zeta_nominal) / Medium.density(state_b)), dp_nominal * m_flow / m_flow_nominal);
        end if;
// Isenthalpic state transformation (no storage and no loss of energy)
        port_a.h_outflow = inStream(port_b.h_outflow);
        port_b.h_outflow = inStream(port_a.h_outflow);
        annotation(
          defaultComponentName = "orifice",
          Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-60, -50}, {-60, 50}, {60, -50}, {60, 50}}, thickness = 0.5), Line(points = {{-60, 0}, {-100, 0}}, color = {0, 127, 255}), Line(points = {{60, 0}, {100, 0}}, color = {0, 127, 255}), Text(extent = {{-173, 104}, {175, 62}}, lineColor = {0, 0, 0}, textString = "zeta=%zeta")}),
          Documentation(info = "<html>
<p>
This pressure drop component defines a
simple, generic orifice, where the loss factor &zeta; is provided
for one flow direction (e.g., from loss table of a book):
</p>

<pre>   &Delta;p = 0.5*&zeta;*&rho;*v*|v|
      = 8*&zeta;/(&pi;^2*D^4*&rho;) * m_flow*|m_flow|
</pre>

<p>
where
</p>
<ul>
<li> &Delta;p is the pressure drop: &Delta;p = port_a.p - port_b.p</li>
<li> D is the diameter of the orifice at the position where
     &zeta; is defined (either at port_a or port_b). If the orifice has not a
     circular cross section, D = 4*A/P, where A is the cross section
     area and P is the wetted perimeter.</li>
<li> &zeta; is the loss factor with respect to D
     that depends on the geometry of
     the orifice. In the turbulent flow regime, it is assumed that
     &zeta; is constant.<br>
     For small mass flow rates, the flow is laminar and is approximated
     by a polynomial that has a finite derivative for m_flow=0.</li>
<li> v is the mean velocity.</li>
<li> &rho; is the upstream density.</li>
</ul>

<p>
Since the pressure loss factor zeta is provided only for a mass flow
from port_a to port_b, the pressure loss is not correct when the
flow is reversing. If reversing flow only occurs in a short time interval,
this is most likely uncritical. If significant reversing flow
can appear, this component should not be used.
</p>
</html>"));
      end SimpleGenericOrifice;

      model SharpEdgedOrifice "Pressure drop due to sharp edged orifice (for both flow directions)"
        import NonSI = Modelica.SIunits.Conversions.NonSIunits;
        extends BaseClasses.QuadraticTurbulent.BaseModel(final data = BaseClasses.QuadraticTurbulent.LossFactorData.sharpEdgedOrifice(diameter, leastDiameter, length, alpha));
        parameter SI.Length length "Length of orifice";
        parameter SI.Diameter diameter "Inner diameter of pipe (= same at port_a and port_b)";
        parameter SI.Diameter leastDiameter "Smallest diameter of orifice";
        parameter NonSI.Angle_deg alpha "Angle of orifice";
        annotation(
          defaultComponentName = "orifice",
          Documentation(info = "<html>
</html>"),
          Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, 44}, {100, -44}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {0, 127, 255}), Polygon(points = {{-25, 44}, {-25, 7}, {35, 37}, {35, 44}, {-25, 44}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.Backward, fillColor = {175, 175, 175}), Polygon(points = {{-25, -7}, {-25, -44}, {35, -44}, {35, -36}, {-25, -7}}, lineColor = {0, 0, 0}, fillColor = {175, 175, 175}, fillPattern = FillPattern.Backward)}),
          Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, 60}, {100, -60}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Polygon(points = {{-30, 60}, {-30, 12}, {30, 50}, {30, 60}, {-30, 60}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Backward), Polygon(points = {{-30, -10}, {-30, -60}, {30, -60}, {30, -50}, {-30, -10}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Backward), Line(points = {{-82, -60}, {-82, 60}}, color = {0, 0, 255}, arrow = {Arrow.Filled, Arrow.Filled}), Text(extent = {{-78, 16}, {-44, -8}}, lineColor = {0, 0, 255}, textString = "diameter"), Line(points = {{-30, -10}, {-30, 12}}, color = {0, 0, 255}, arrow = {Arrow.Filled, Arrow.Filled}), Text(extent = {{-24, 14}, {8, -10}}, lineColor = {0, 0, 255}, textString = "leastDiameter"), Text(extent = {{-20, 84}, {18, 70}}, lineColor = {0, 0, 255}, textString = "length"), Line(points = {{30, 68}, {-30, 68}}, color = {0, 0, 255}, arrow = {Arrow.Filled, Arrow.Filled}), Line(points = {{16, 40}, {32, 18}, {36, -2}, {34, -20}, {20, -42}}, color = {0, 0, 255}, arrow = {Arrow.Filled, Arrow.Filled}), Text(extent = {{38, 8}, {92, -6}}, lineColor = {0, 0, 255}, textString = "alpha")}));
      end SharpEdgedOrifice;

      model AbruptAdaptor "Pressure drop in pipe due to suddenly expanding or reducing area (for both flow directions)"
        extends BaseClasses.QuadraticTurbulent.BaseModelNonconstantCrossSectionArea(final data = BaseClasses.QuadraticTurbulent.LossFactorData.suddenExpansion(diameter_a, diameter_b));
        parameter SI.Diameter diameter_a "Inner diameter of pipe at port_a";
        parameter SI.Diameter diameter_b "Inner diameter of pipe at port_b";
        annotation(
          Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{0, 40}, {-100, 40}, {-100, -40}, {0, -40}, {0, -100}, {100, -100}, {100, 100}, {0, 100}, {0, 40}}), Rectangle(extent = {{-100, 40}, {0, -40}}, lineColor = {255, 255, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Rectangle(extent = {{0, 100}, {100, -100}}, lineColor = {255, 255, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{0, 40}, {-100, 40}, {-100, -40}, {0, -40}, {0, -100}, {100, -100}, {100, 100}, {0, 100}, {0, 40}}), Line(points = {{-60, -40}, {-60, 40}}, color = {0, 0, 255}, arrow = {Arrow.Filled, Arrow.Filled}), Text(extent = {{-50, 16}, {-26, -10}}, lineColor = {0, 0, 255}, textString = "diameter_a"), Line(points = {{34, -100}, {34, 100}}, color = {0, 0, 255}, arrow = {Arrow.Filled, Arrow.Filled}), Text(extent = {{54, 16}, {78, -10}}, lineColor = {0, 0, 255}, textString = "diameter_b")}),
          Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = DynamicSelect({{-100, 22}, {0, -22}}, {{-100, max(0.1, min(1, diameter_a / max(diameter_a, diameter_b))) * 60}, {0, -max(0.1, min(1, diameter_a / max(diameter_a, diameter_b))) * 60}}), lineColor = {0, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {0, 127, 255}), Rectangle(extent = DynamicSelect({{0, 60}, {100, -60}}, {{0, max(0.1, min(1, diameter_b / max(diameter_a, diameter_b))) * 60}, {100, -max(0.1, min(1, diameter_b / max(diameter_a, diameter_b))) * 60}}), lineColor = {0, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {0, 127, 255})}),
          Documentation(info = "<html>

</html>"));
      end AbruptAdaptor;

      model MultiPort "Multiply a port; useful if multiple connections shall be made to a port exposing a state"
        function positiveMax
          extends Modelica.Icons.Function;
          input Real x;
          output Real y;
        algorithm
          y := max(x, 1e-10);
        end positiveMax;

        import Modelica.Constants;
        replaceable package Medium = Modelica.Media.Interfaces.PartialMedium annotation(
          choicesAllMatching);
        // Ports
        parameter Integer nPorts_b = 0 "Number of outlet ports (mass is distributed evenly between the outlet ports" annotation(
          Dialog(connectorSizing = true));
        Modelica.Fluid.Interfaces.FluidPort_a port_a(redeclare package Medium = Medium) annotation(
          Placement(transformation(extent = {{-50, -10}, {-30, 10}})));
        Modelica.Fluid.Interfaces.FluidPorts_b ports_b[nPorts_b](redeclare each package Medium = Medium) annotation(
          Placement(transformation(extent = {{30, 40}, {50, -40}})));
        Medium.MassFraction ports_b_Xi_inStream[nPorts_b, Medium.nXi] "inStream mass fractions at ports_b";
        Medium.ExtraProperty ports_b_C_inStream[nPorts_b, Medium.nC] "inStream extra properties at ports_b";
      equation
// Only one connection allowed to a port to avoid unwanted ideal mixing
        for i in 1:nPorts_b loop
          assert(cardinality(ports_b[i]) <= 1, "
each ports_b[i] of boundary shall at most be connected to one component.
If two or more connections are present, ideal mixing takes
place with these connections, which is usually not the intention
of the modeller. Increase nPorts_b to add an additional port.
          ");
        end for;
// mass and momentum balance
        0 = port_a.m_flow + sum(ports_b.m_flow);
        ports_b.p = fill(port_a.p, nPorts_b);
// mixing at port_a
        port_a.h_outflow = sum({positiveMax(ports_b[j].m_flow) * inStream(ports_b[j].h_outflow) for j in 1:nPorts_b}) / sum({positiveMax(ports_b[j].m_flow) for j in 1:nPorts_b});
        for j in 1:nPorts_b loop
// expose stream values from port_a to ports_b
          ports_b[j].h_outflow = inStream(port_a.h_outflow);
          ports_b[j].Xi_outflow = inStream(port_a.Xi_outflow);
          ports_b[j].C_outflow = inStream(port_a.C_outflow);
          ports_b_Xi_inStream[j, :] = inStream(ports_b[j].Xi_outflow);
          ports_b_C_inStream[j, :] = inStream(ports_b[j].C_outflow);
        end for;
        for i in 1:Medium.nXi loop
          port_a.Xi_outflow[i] = positiveMax(ports_b.m_flow) * ports_b_Xi_inStream[:, i] / sum(positiveMax(ports_b.m_flow));
        end for;
        for i in 1:Medium.nC loop
          port_a.C_outflow[i] = positiveMax(ports_b.m_flow) * ports_b_C_inStream[:, i] / sum(positiveMax(ports_b.m_flow));
        end for;
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-40, -100}, {40, 100}}), graphics = {Line(points = {{-40, 0}, {40, 0}}, color = {0, 128, 255}, thickness = 1), Line(points = {{-40, 0}, {40, 26}}, color = {0, 128, 255}, thickness = 1), Line(points = {{-40, 0}, {40, -26}}, color = {0, 128, 255}, thickness = 1), Text(extent = {{-150, 100}, {150, 60}}, lineColor = {0, 0, 255}, textString = "%name")}),
          Documentation(info = "<html>
<p>
This model is useful if multiple connections shall be made to a port of a volume model exposing a state,
like a pipe with ModelStructure av_vb.
The mixing is shifted into the volume connected to port_a and the result is propagated back to each ports_b.
</p>
<p>
If multiple connections were directly made to the volume,
then ideal mixing would take place in the connection set, outside the volume. This is normally not intended.
</p>
</html>"));
      end MultiPort;

      model TeeJunctionIdeal "Splitting/joining component with static balances for an infinitesimal control volume"
        extends Modelica.Fluid.Fittings.BaseClasses.PartialTeeJunction;
      equation
        connect(port_1, port_2) annotation(
          Line(points = {{-100, 0}, {100, 0}}, color = {0, 127, 255}));
        connect(port_1, port_3) annotation(
          Line(points = {{-100, 0}, {0, 0}, {0, 100}}, color = {0, 127, 255}));
        annotation(
          Documentation(info = "<html>
  This model is the simplest implementation for a splitting/joining component for
  three flows. Its use is not required. It just formulates the balance
  equations in the same way that the connect semantics would formulate them anyways.
  The main advantage of using this component is, that the user does not get
  confused when looking at the specific enthalpy at each port which might be confusing
  when not using a splitting/joining component. The reason for the confusion is that one examines the mixing
  enthalpy of the infinitesimal control volume introduced with the connect statement when
  looking at the specific enthalpy in the connector which
  might not be equal to the specific enthalpy at the port in the \"real world\".</html>"));
      end TeeJunctionIdeal;

      model TeeJunctionVolume "Splitting/joining component with static balances for a dynamic control volume"
        extends Modelica.Fluid.Fittings.BaseClasses.PartialTeeJunction;
        extends Modelica.Fluid.Interfaces.PartialLumpedVolume(final fluidVolume = V);
        parameter SI.Volume V "Mixing volume inside junction";
      equation
// Only one connection allowed to a port to avoid unwanted ideal mixing
        assert(cardinality(port_1) <= 1, "
port_1 of volume can at most be connected to one component.
If two or more connections are present, ideal mixing takes
place with these connections which is usually not the intention
of the modeller.
        ");
        assert(cardinality(port_2) <= 1, "
port_2 of volume can at most be connected to one component.
If two or more connections are present, ideal mixing takes
place with these connections which is usually not the intention
of the modeller.
        ");
        assert(cardinality(port_3) <= 1, "
port_3 of volume can at most be connected to one component.
If two or more connections are present, ideal mixing takes
place with these connections which is usually not the intention
of the modeller.
        ");
// Boundary conditions
        port_1.h_outflow = medium.h;
        port_2.h_outflow = medium.h;
        port_3.h_outflow = medium.h;
        port_1.Xi_outflow = medium.Xi;
        port_2.Xi_outflow = medium.Xi;
        port_3.Xi_outflow = medium.Xi;
        port_1.C_outflow = C;
        port_2.C_outflow = C;
        port_3.C_outflow = C;
// Mass balances
        mb_flow = port_1.m_flow + port_2.m_flow + port_3.m_flow "Mass balance";
        mbXi_flow = port_1.m_flow * actualStream(port_1.Xi_outflow) + port_2.m_flow * actualStream(port_2.Xi_outflow) + port_3.m_flow * actualStream(port_3.Xi_outflow) "Component mass balances";
        mbC_flow = port_1.m_flow * actualStream(port_1.C_outflow) + port_2.m_flow * actualStream(port_2.C_outflow) + port_3.m_flow * actualStream(port_3.C_outflow) "Trace substance mass balances";
// Momentum balance (suitable for compressible media)
        port_1.p = medium.p;
        port_2.p = medium.p;
        port_3.p = medium.p;
// Energy balance
        Hb_flow = port_1.m_flow * actualStream(port_1.h_outflow) + port_2.m_flow * actualStream(port_2.h_outflow) + port_3.m_flow * actualStream(port_3.h_outflow);
        Qb_flow = 0;
        Wb_flow = 0;
        annotation(
          Documentation(info = "<html>
  This model introduces a mixing volume into a junction.
  This might be useful to examine the non-ideal mixing taking place in a real junction.</html>"),
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-9, 10}, {11, -10}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid)}));
      end TeeJunctionVolume;

      package BaseClasses "Base classes used in the Fittings package (only of interest to build new component models)"
        extends Modelica.Icons.BasesPackage;

        function lossConstant_D_zeta "Return the loss constant 8*zeta/(pi^2*D^4)"
          extends Modelica.Icons.Function;
          input SI.Diameter D "Diameter at port_a or port_b";
          input Real zeta "Constant pressure loss factor with respect to D (i.e., either port_a or port_b)";
          output Real k "Loss constant (= 8*zeta/(pi^2*D^4))";
        algorithm
          k := 8 * zeta / (Modelica.Constants.pi * Modelica.Constants.pi * D * D * D * D);
          annotation(
            Documentation(info = "<html>

</html>"));
        end lossConstant_D_zeta;

        package QuadraticTurbulent "Pressure loss components that are mainly defined by a quadratic turbulent regime with constant loss factor data"
          extends Modelica.Icons.Package;

          record LossFactorData "Data structure defining constant loss factor data for dp = zeta*rho*v*|v|/2 and functions providing the data for some loss types"
            extends Modelica.Icons.Record;
            SI.Diameter diameter_a "Diameter at port_a" annotation(
              Dialog);
            SI.Diameter diameter_b "Diameter at port_b" annotation(
              Dialog);
            Real zeta1 "Loss factor for flow port_a -> port_b" annotation(
              Dialog);
            Real zeta2 "Loss factor for flow port_b -> port_a" annotation(
              Dialog);
            SI.ReynoldsNumber Re_turbulent "Loss factors suited for Re >= Re_turbulent" annotation(
              Dialog);
            SI.Diameter D_Re "Diameter used to compute Re" annotation(
              Dialog);
            Boolean zeta1_at_a = true "dp = zeta1*(if zeta1_at_a then rho_a*v_a^2/2 else rho_b*v_b^2/2)" annotation(
              Dialog);
            Boolean zeta2_at_a = false "dp = -zeta2*(if zeta2_at_a then rho_a*v_a^2/2 else rho_b*v_b^2/2)" annotation(
              Dialog);
            Boolean zetaLaminarKnown = false "= true, if zeta = c0/Re in laminar region" annotation(
              Dialog);
            Real c0 = 1 "zeta = c0/Re; dp = zeta*rho_Re*v_Re^2/2, Re=v_Re*D_Re*rho_Re/mu_Re)" annotation(
              Dialog(enable = zetaLaminarKnown));

            encapsulated function wallFriction "Return pressure loss data due to friction in a straight pipe with walls of nonuniform roughness (not useful for smooth pipes, since zeta is no function of Re)"
              import Modelica.Fluid.Fittings.BaseClasses.QuadraticTurbulent.LossFactorData;
              import lg = Modelica.Math.log10;
              import SI = Modelica.SIunits;
              input SI.Length length "Length of pipe" annotation(
                Dialog);
              input SI.Diameter diameter "Inner diameter of pipe" annotation(
                Dialog);
              input SI.Length roughness(min = 1e-10) "Absolute roughness of pipe (> 0 required, details see info layer)" annotation(
                Dialog);
              output LossFactorData data "Pressure loss factors for both flow directions";
            protected
              Real Delta = roughness / diameter "relative roughness";
            algorithm
              data.diameter_a := diameter;
              data.diameter_b := diameter;
              data.zeta1 := length / diameter / (2 * lg(3.7 / Delta)) ^ 2;
              data.zeta2 := data.zeta1;
              data.Re_turbulent := 4000 ">= 560/Delta flow does not depend on Re, but interpolation is bad";
              data.D_Re := diameter;
              data.zeta1_at_a := true;
              data.zeta2_at_a := false;
              data.zetaLaminarKnown := true;
              data.c0 := 64 * (length / diameter);
              annotation(
                Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, 50}, {100, -50}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid)}),
                Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, 64}, {100, -64}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Backward), Rectangle(extent = {{-100, 50}, {100, -49}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{-60, -49}, {-60, 50}}, color = {0, 0, 255}, arrow = {Arrow.Filled, Arrow.Filled}), Text(extent = {{-50, 16}, {6, -10}}, lineColor = {0, 0, 255}, textString = "diameter"), Line(points = {{-100, 74}, {100, 74}}, color = {0, 0, 255}, arrow = {Arrow.Filled, Arrow.Filled}), Text(extent = {{-34, 92}, {34, 74}}, lineColor = {0, 0, 255}, textString = "length")}),
                Documentation(info = "<html>
<p>
Friction in straight pipe with walls of nonuniform roughness
(commercial pipes) in the region that does not depend on the Reynolds-number
</p>
<p>
The loss factors are given for mass flow rates from
port_a to port_b as:
</p>
<pre>
  turbulent flow (Idelchik 1994, diagram 2-5, p. 117)
     zeta = (L/D)/(2*lg(3.7 / &Delta;))^2, for Re >= 560/&Delta;
&nbsp;
     for Re &ge; 560/&Delta; the loss factor does not depend on the
     Reynolds number. For Re &ge; 4000, the flow is turbulent,
     but depends both on &Delta; and slightly on Re.
&nbsp;
  laminar flow (Idelchick 1994, diagram 2-1, p. 110):
     zeta = 64*(L/D)/Re
</pre>
<p>
where
</p>
<ul>
<li> D is the inner pipe diameter</li>
<li> L is the length of the pipe</li>
<li> &Delta; = &delta;/D is the relative roughness where &delta; is
     the absolute \"roughness\", i.e., the averaged height of asperities in the pipe.
     (&delta; may change over time due to growth of surface asperities during
      service, see [Idelchick 1994, p. 85, Tables 2-1, 2-2]).</li>
</ul>

<p>
Since the LossFactorData record can only describe loss factors that depend
on geometry (but, e.g., not on the Reynolds number), only the region
with Re &ge; 560/&Delta; is described by this data. Still, the turbulent
region with the above zeta is defined to start at Re=4000, since otherwise
the approximation for Re &lt; 560/&Delta; is too bad.
</p>

<p>
The absolute roughness <font face=\"Symbol\">d</font> has usually to
be estimated. In <i>[Idelchik 1994, pp. 105-109,
Table 2-5; Miller 1990, p. 190, Table 8-1]</i> many examples are given.
As a short summary:
</p>
<table border=1 cellspacing=0 cellpadding=2>
  <tr><td><b>Smooth pipes</b></td>
      <td>Drawn brass, copper, aluminium, glass, etc.</td>
      <td><font face=\"Symbol\">d</font> = 0.0025 mm</td>
  </tr>
  <tr><td rowspan=\"3\"><b>Steel pipes</b></td>
      <td>New smooth pipes</td>
      <td><font face=\"Symbol\">d</font> = 0.025 mm</td>
  </tr>
  <tr><td>Mortar lined, average finish</td>
      <td><font face=\"Symbol\">d</font> = 0.1 mm</td>
  </tr>
  <tr><td>Heavy rust</td>
      <td><font face=\"Symbol\">d</font> = 1 mm</td>
  </tr>
  <tr><td rowspan=\"3\"><b>Concrete pipes</b></td>
      <td>Steel forms, first class workmanship</td>
      <td><font face=\"Symbol\">d</font> = 0.025 mm</td>
  </tr>
  <tr><td>Steel forms, average workmanship</td>
      <td><font face=\"Symbol\">d</font> = 0.1 mm</td>
  </tr>
  <tr><td>Block linings</td>
      <td><font face=\"Symbol\">d</font> = 1 mm</td>
  </tr>
</table>
</html>"));
            end wallFriction;

            encapsulated function suddenExpansion "Return pressure loss data for sudden expansion or contraction in a pipe (for both flow directions)"
              import Modelica.Fluid.Fittings.BaseClasses.QuadraticTurbulent.LossFactorData;
              import SI = Modelica.SIunits;
              input SI.Diameter diameter_a "Inner diameter of pipe at port_a" annotation(
                Dialog);
              input SI.Diameter diameter_b "Inner diameter of pipe at port_b" annotation(
                Dialog);
              output LossFactorData data "Pressure loss factors for both flow directions";
            protected
              Real A_rel;
            algorithm
              data.diameter_a := diameter_a;
              data.diameter_b := diameter_b;
              data.Re_turbulent := 100;
              data.zetaLaminarKnown := true;
              data.c0 := 30;
              if diameter_a <= diameter_b then
                A_rel := (diameter_a / diameter_b) ^ 2;
                data.zeta1 := (1 - A_rel) ^ 2;
                data.zeta2 := 0.5 * (1 - A_rel) ^ 0.75;
                data.zeta1_at_a := true;
                data.zeta2_at_a := true;
                data.D_Re := diameter_a;
              else
                A_rel := (diameter_b / diameter_a) ^ 2;
                data.zeta1 := 0.5 * (1 - A_rel) ^ 0.75;
                data.zeta2 := (1 - A_rel) ^ 2;
                data.zeta1_at_a := false;
                data.zeta2_at_a := false;
                data.D_Re := diameter_b;
              end if;
              annotation(
                Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, 40}, {0, -40}}, lineColor = {255, 255, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Rectangle(extent = {{0, 100}, {100, -100}}, lineColor = {255, 255, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{0, 40}, {-100, 40}, {-100, -40}, {0, -40}, {0, -100}, {100, -100}, {100, 100}, {0, 100}, {0, 40}})}),
                Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{0, 40}, {-100, 40}, {-100, -40}, {0, -40}, {0, -100}, {100, -100}, {100, 100}, {0, 100}, {0, 40}}), Rectangle(extent = {{-100, 40}, {0, -40}}, lineColor = {255, 255, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Rectangle(extent = {{0, 100}, {100, -100}}, lineColor = {255, 255, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{0, 40}, {-100, 40}, {-100, -40}, {0, -40}, {0, -100}, {100, -100}, {100, 100}, {0, 100}, {0, 40}}), Line(points = {{-60, -40}, {-60, 40}}, color = {0, 0, 255}, arrow = {Arrow.Filled, Arrow.Filled}), Text(extent = {{-50, 16}, {-26, -10}}, lineColor = {0, 0, 255}, textString = "diameter_a"), Line(points = {{34, -100}, {34, 100}}, color = {0, 0, 255}, arrow = {Arrow.Filled, Arrow.Filled}), Text(extent = {{54, 16}, {78, -10}}, lineColor = {0, 0, 255}, textString = "diameter_b")}),
                Documentation(info = "<html>
<p>
The loss factors are given for mass flow rates from
port_a to port_b as:
</p>
<pre>
   A_a &lt; A_b (Idelchik 1994, diagram 4-1, p. 208):
      zeta = dp/(rho_a*v_a^2/2)
           = (1 - A_a/A_b)^2 for Re_a &ge; 3.3e3 (turbulent flow)
      zeta = 30/Re           for Re_a &lt; 10    (laminar flow)
&nbsp;
   A_a &gt; A_b (Idelchik 1994, diagram 4-9, p. 216 and diagram 4-10, p. 217)
      zeta = dp/(rho_b*v_b^2/2)
           = 0.5*(1 - A_b/A_a)^0.75 for Re_b &ge; 1e4 (turbulent flow)
      zeta = 30/Re                  for Re_a &lt; 10  (laminar flow)
</pre>
</html>"));
            end suddenExpansion;

            encapsulated function sharpEdgedOrifice "Return pressure loss data for sharp edged orifice (for both flow directions)"
              import NonSI = Modelica.SIunits.Conversions.NonSIunits;
              import Modelica.Fluid.Fittings.BaseClasses.QuadraticTurbulent.LossFactorData;
              import SI = Modelica.SIunits;
              input SI.Diameter diameter "Inner diameter of pipe (= same at port_a and port_b)" annotation(
                Dialog);
              input SI.Diameter leastDiameter "Smallest diameter of orifice" annotation(
                Dialog);
              input SI.Diameter length "Length of orifice" annotation(
                Dialog);
              input NonSI.Angle_deg alpha "Angle of orifice" annotation(
                Dialog);
              output LossFactorData data "Pressure loss factors for both flow directions";
            protected
              Real D_rel = leastDiameter / diameter;
              Real LD = length / leastDiameter;
              Real k = 0.13 + 0.34 * 10 ^ (-(3.4 * LD + 88.4 * LD ^ 2.3));
            algorithm
              data.diameter_a := diameter;
              data.diameter_b := diameter;
              data.zeta1 := (1 - D_rel + 0.707 * (1 - D_rel) ^ 0.375) ^ 2 * (1 / D_rel) ^ 2;
              data.zeta2 := k * (1 - D_rel) ^ 0.75 + (1 - D_rel) ^ 2 + 2 * sqrt(k * (1 - D_rel) ^ 0.375) + 1 - D_rel;
              data.Re_turbulent := 1e4;
              data.D_Re := leastDiameter;
              data.zeta1_at_a := true;
              data.zeta2_at_a := false;
              data.zetaLaminarKnown := false;
              data.c0 := 0;
              annotation(
                Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, 60}, {100, -60}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Polygon(points = {{-30, 60}, {-30, 12}, {30, 50}, {30, 60}, {-30, 60}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Backward), Polygon(points = {{-30, -10}, {-30, -60}, {30, -60}, {30, -50}, {-30, -10}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Backward)}),
                Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, 60}, {100, -60}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Polygon(points = {{-30, 60}, {-30, 12}, {30, 50}, {30, 60}, {-30, 60}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Backward), Polygon(points = {{-30, -10}, {-30, -60}, {30, -60}, {30, -50}, {-30, -10}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Backward), Line(points = {{-82, -60}, {-82, 60}}, color = {0, 0, 255}, arrow = {Arrow.Filled, Arrow.Filled}), Text(extent = {{-78, 16}, {-44, -8}}, lineColor = {0, 0, 255}, textString = "diameter"), Line(points = {{-30, -10}, {-30, 12}}, color = {0, 0, 255}, arrow = {Arrow.Filled, Arrow.Filled}), Text(extent = {{-24, 14}, {8, -10}}, lineColor = {0, 0, 255}, textString = "leastDiameter"), Text(extent = {{-20, 84}, {18, 70}}, lineColor = {0, 0, 255}, textString = "length"), Line(points = {{30, 68}, {-30, 68}}, color = {0, 0, 255}, arrow = {Arrow.Filled, Arrow.Filled}), Line(points = {{16, 40}, {32, 18}, {36, -2}, {34, -20}, {20, -42}}, color = {0, 0, 255}, arrow = {Arrow.Filled, Arrow.Filled}), Text(extent = {{38, 8}, {92, -6}}, lineColor = {0, 0, 255}, textString = "alpha")}),
                Documentation(info = "<html>
<p>
Loss factor for mass flow rate from port_a to port_b
(Idelchik 1994, diagram 4-14, p. 221):
</p>
<pre>
   zeta = [(1-A0/A1) + 0.707*(1-A0/A1)^0.375]^2*(A1/A0)^2
          for Re(A0) >= 1e5,  independent of alpha
</pre>
<p>
Loss factor for mass flow rate from port_b to port_a
(Idelchik 1994, diagram 4-13, p. 220, with A2=A1):
</p>
<pre>
   zeta = k*(1 - A0/A1)^0.75 + (1 - A0/A1)^2 + 2*sqrt(k*(1-A0/A1)^0.375) + (1- A0/A1)
          k  = 0.13 + 0.34*10^(-(3.4*LD+88.4*LD^2.3))
               (there is a typing error in the formula in diagram 4-13, the above
                equation corresponds to table (a) in diagram 4-12)
          LD = L/D0
          for Re(A0) >= 1e4, 40 deg &le; alpha &le; 60 deg
                             for other values of alpha, k is given as table
                             in diagram 3-7 (this is not yet included in the function)
</pre
</html>"));
            end sharpEdgedOrifice;
            annotation(
              preferredView = "info",
              Documentation(info = "<html>
<p>
This record defines the pressure loss factors of a pipe
segment (orifice, bending etc.) with a minimum amount of data.
If available, data should be provided for <b>both flow directions</b>,
i.e., flow from port_a to port_b and from port_b to port_a,
as well as for the <b>laminar</b> and the <b>turbulent</b> region.
It is also an option to provide the loss factor <b>only</b> for the
<b>turbulent</b> region for a flow from port_a to port_b.
</p>
<p>
The following equations are used:
</p>
<pre>   &Delta;p = 0.5*&zeta;*&rho;*v*|v|
      = 0.5*&zeta;/A^2 * (1/&rho;) * m_flow*|m_flow|
      = 8*&zeta;/(&pi;^2*D^4*&rho;) * m_flow*|m_flow|
        Re = |v|*D*&rho;/&mu;
</pre>
<table border=1 cellspacing=0 cellpadding=2>
<tr><td><b>flow type</b></td>
    <td><b>&zeta;</b> = </td>
    <td><b>flow region</b></td></tr>
<tr><td>turbulent</td>
    <td><b>zeta1</b> = const.</td>
    <td>Re &ge;  Re_turbulent, v &ge; 0</td></tr>
<tr><td></td>
    <td><b>zeta2</b> = const.</td>
    <td>Re &ge; Re_turbulent, v &lt; 0</td></tr>
<tr><td>laminar</td>
    <td><b>c0</b>/Re</td>
    <td>both flow directions, Re small; c0 = const.</td></tr>
</table>
<p>
where
</p>
<ul>
<li> &Delta;p is the pressure drop: &Delta;p = port_a.p - port_b.p</li>
<li> v is the mean velocity.</li>
<li> &rho; is the density.</li>
<li> &zeta; is the loss factor that depends on the geometry of
     the pipe. In the turbulent flow regime, it is assumed that
     &zeta; is constant and is given by \"zeta1\" and
     \"zeta2\" depending on the flow direction.<br>
     When the Reynolds number Re is below \"Re_turbulent\", the
     flow is laminar for small flow velocities. For higher
     velocities there is a transition region from
     laminar to turbulent flow. The loss factor for
     laminar flow at small velocities is defined by the often occurring
     approximation c0/Re. If c0 is different for the two
     flow directions, the mean value has to be used
     (c0 = (c0_ab + c0_ba)/2).</li>
<li> The equation \"&Delta;p = 0.5*&zeta;*&rho;*v*|v|\" is either with
     respect to port_a or to port_b, depending on the definition
     of the particular loss factor &zeta; (in some references loss
     factors are defined with respect to port_a, in other references
     with respect to port_b).</li>
<li> Re = |v|*D_Re*&rho;/&mu; = |m_flow|*D_Re/(A_Re*&mu;)
     is the Reynolds number at the smallest cross
     section area. This is often at port_a or at port_b, but can
     also be between the two ports. In the record, the diameter
     D_Re of this smallest cross section area has to be provided, as
     well, as Re_turbulent, the absolute value of the
     Reynolds number at which
     the turbulent flow starts. If Re_turbulent is different for
     the two flow directions, use the smaller value as Re_turbulent.</li>
<li> D is the diameter of the pipe. If the pipe has not a
     circular cross section, D = 4*A/P, where A is the cross section
     area and P is the wetted perimeter.</li>
<li> A is the cross section area with A = &pi;(D/2)^2.</li>
<li> &mu; is the dynamic viscosity.</li>
</ul>
<p>
The laminar and the transition region is usually of
not much technical interest because the operating point is
mostly in the turbulent regime. For simplification and for
numerical reasons, this whole region is described by two
polynomials of third order, one polynomial for m_flow &ge; 0
and one for m_flow &lt; 0. The polynomials start at
Re = |m_flow|*4/(&pi;*D_Re*&mu;), where D_Re is the
smallest diameter between port_a and port_b.
The common derivative
of the two polynomials at Re = 0 is
computed from the equation \"c0/Re\". Note, the pressure drop
equation above in the laminar region is always defined
with respect to the smallest diameter D_Re.
</p>
<p>
If no data for c0 is available, the derivative at Re = 0 is computed in such
a way, that the second derivatives of the two polynomials
are identical at Re = 0. The polynomials are constructed, such that
they smoothly touch the characteristic curves in the turbulent
regions. The whole characteristic is therefore <b>continuous</b>
and has a <b>finite</b>, <b>continuous first derivative everywhere</b>.
In some cases, the constructed polynomials would \"vibrate\". This is
avoided by reducing the derivative at Re=0 in such a way that
the polynomials are guaranteed to be monotonically increasing.
The used sufficient criteria for monotonicity follows from:
</p>

<dl>
<dt> Fritsch F.N. and Carlson R.E. (1980):</dt>
<dd> <b>Monotone piecewise cubic interpolation</b>.
     SIAM J. Numerc. Anal., Vol. 17, No. 2, April 1980, pp. 238-246</dd>
</dl>
</html>"));
          end LossFactorData;

          function massFlowRate_dp "Return mass flow rate from constant loss factor data and pressure drop (m_flow = f(dp))"
            //import Modelica.Fluid.PressureLosses.BaseClasses.lossConstant_D_zeta;
            extends Modelica.Icons.Function;
            input SI.Pressure dp "Pressure drop (dp = port_a.p - port_b.p)";
            input SI.Density rho_a "Density at port_a";
            input SI.Density rho_b "Density at port_b";
            input LossFactorData data "Constant loss factors for both flow directions" annotation(
              choices(choice = Modelica.Fluid.Fittings.Utilities.QuadraticTurbulent.LossFactorData.wallFriction(), choice = Modelica.Fluid.Fittings.Utilities.QuadraticTurbulent.LossFactorData.suddenExpansion(), choice = Modelica.Fluid.Fittings.Utilities.QuadraticTurbulent.LossFactorData.sharpEdgedOrifice()));
            input SI.AbsolutePressure dp_small = 1 "Turbulent flow if |dp| >= dp_small";
            output SI.MassFlowRate m_flow "Mass flow rate from port_a to port_b";
          protected
            Real k1 = lossConstant_D_zeta(if data.zeta1_at_a then data.diameter_a else data.diameter_b, data.zeta1);
            Real k2 = lossConstant_D_zeta(if data.zeta2_at_a then data.diameter_a else data.diameter_b, data.zeta2);
          algorithm
/*
   dp = 0.5*zeta*rho*v*|v|
      = 0.5*zeta*rho*1/(rho*A)^2 * m_flow * |m_flow|
      = 0.5*zeta/A^2 *1/rho * m_flow * |m_flow|
      = k/rho * m_flow * |m_flow|
   k  = 0.5*zeta/A^2
      = 0.5*zeta/(pi*(D/2)^2)^2
      = 8*zeta/(pi*D^2)^2
  */
            m_flow := Utilities.regRoot2(dp, dp_small, rho_a / k1, rho_b / k2);
            annotation(
              smoothOrder = 1,
              Documentation(info = "<html>
<p>
Compute mass flow rate from constant loss factor and pressure drop (m_flow = f(dp)).
For small pressure drops (dp &lt; dp_small), the characteristic is approximated by
a polynomial in order to have a finite derivative at zero mass flow rate.
</p>
</html>"));
          end massFlowRate_dp;

          function massFlowRate_dp_and_Re "Return mass flow rate from constant loss factor data, pressure drop and Re (m_flow = f(dp))"
            extends Modelica.Icons.Function;
            import Modelica.Constants.pi;
            input SI.Pressure dp "Pressure drop (dp = port_a.p - port_b.p)";
            input SI.Density rho_a "Density at port_a";
            input SI.Density rho_b "Density at port_b";
            input SI.DynamicViscosity mu_a "Dynamic viscosity at port_a";
            input SI.DynamicViscosity mu_b "Dynamic viscosity at port_b";
            input LossFactorData data "Constant loss factors for both flow directions" annotation(
              choices(choice = BaseClasses.PressureLosses.QuadraticTurbulent.LossFactorData.wallFriction(), choice = BaseClasses.PressureLosses.QuadraticTurbulent.LossFactorData.suddenExpansion(), choice = BaseClasses.PressureLosses.QuadraticTurbulent.LossFactorData.sharpEdgedOrifice()));
            output SI.MassFlowRate m_flow "Mass flow rate from port_a to port_b";
          protected
            Real k0 = 2 * data.c0 / (pi * data.D_Re ^ 3);
            Real k1 = lossConstant_D_zeta(if data.zeta1_at_a then data.diameter_a else data.diameter_b, data.zeta1);
            Real k2 = lossConstant_D_zeta(if data.zeta2_at_a then data.diameter_a else data.diameter_b, data.zeta2);
            Real yd0 "Derivative of m_flow=m_flow(dp) at zero, if data.zetaLaminarKnown";
            SI.AbsolutePressure dp_turbulent "The turbulent region is: |dp| >= dp_turbulent";
          algorithm
/*
Turbulent region:
   Re = m_flow*(4/pi)/(D_Re*mu)
   dp = 0.5*zeta*rho*v*|v|
      = 0.5*zeta*rho*1/(rho*A)^2 * m_flow * |m_flow|
      = 0.5*zeta/A^2 *1/rho * m_flow * |m_flow|
      = k/rho * m_flow * |m_flow|
   k  = 0.5*zeta/A^2
      = 0.5*zeta/(pi*(D/2)^2)^2
      = 8*zeta/(pi*D^2)^2
   m_flow_turbulent = (pi/4)*D_Re*mu*Re_turbulent
   dp_turbulent     =  k/rho *(D_Re*mu*pi/4)^2 * Re_turbulent^2

   The start of the turbulent region is computed with mean values
   of dynamic viscosity mu and density rho. Otherwise, one has
   to introduce different "delta" values for both flow directions.
   In order to simplify the approach, only one delta is used.

Laminar region:
   dp = 0.5*zeta/(A^2*d) * m_flow * |m_flow|
      = 0.5 * c0/(|m_flow|*(4/pi)/(D_Re*mu)) / ((pi*(D_Re/2)^2)^2*d) * m_flow*|m_flow|
      = 0.5 * c0*(pi/4)*(D_Re*mu) * 16/(pi^2*D_Re^4*d) * m_flow*|m_flow|
      = 2*c0/(pi*D_Re^3) * mu/rho * m_flow
      = k0 * mu/rho * m_flow
   k0 = 2*c0/(pi*D_Re^3)

   In order that the derivative of dp=f(m_flow) is continuous
   at m_flow=0, the mean values of mu and d are used in the
   laminar region: mu/rho = (mu_a + mu_b)/(rho_a + rho_b)
   If data.zetaLaminarKnown = false then mu_a and mu_b are potentially zero
   (because dummy values) and therefore the division is only performed
   if zetaLaminarKnown = true.
*/
            dp_turbulent := (k1 + k2) / (rho_a + rho_b) * ((mu_a + mu_b) * data.D_Re * pi / 8) ^ 2 * data.Re_turbulent ^ 2;
            yd0 := if data.zetaLaminarKnown then (rho_a + rho_b) / (k0 * (mu_a + mu_b)) else 0;
            m_flow := Utilities.regRoot2(dp, dp_turbulent, rho_a / k1, rho_b / k2, data.zetaLaminarKnown, yd0);
            annotation(
              smoothOrder = 1,
              Documentation(info = "<html>
<p>
Compute mass flow rate from constant loss factor and pressure drop (m_flow = f(dp)).
If the Reynolds-number Re &ge; data.Re_turbulent, the flow
is treated as a turbulent flow with constant loss factor zeta.
If the Reynolds-number Re &lt; data.Re_turbulent, the flow
is laminar and/or in a transition region between laminar and
turbulent. This region is approximated by two
polynomials of third order, one polynomial for m_flow &ge; 0
and one for m_flow &lt; 0.
The common derivative
of the two polynomials at Re = 0 is
computed from the equation \"data.c0/Re\".
</p>
<p>
If no data for c0 is available, the derivative at Re = 0 is computed in such
a way, that the second derivatives of the two polynomials
are identical at Re = 0. The polynomials are constructed, such that
they smoothly touch the characteristic curves in the turbulent
regions. The whole characteristic is therefore <b>continuous</b>
and has a <b>finite</b>, <b>continuous first derivative everywhere</b>.
In some cases, the constructed polynomials would \"vibrate\". This is
avoided by reducing the derivative at Re=0 in such a way that
the polynomials are guaranteed to be monotonically increasing.
The used sufficient criteria for monotonicity follows from:
</p>

<dl>
<dt> Fritsch F.N. and Carlson R.E. (1980):</dt>
<dd> <b>Monotone piecewise cubic interpolation</b>.
     SIAM J. Numerc. Anal., Vol. 17, No. 2, April 1980, pp. 238-246</dd>
</dl>
</html>"));
          end massFlowRate_dp_and_Re;

          function pressureLoss_m_flow "Return pressure drop from constant loss factor and mass flow rate (dp = f(m_flow))"
            extends Modelica.Icons.Function;
            input SI.MassFlowRate m_flow "Mass flow rate from port_a to port_b";
            input SI.Density rho_a "Density at port_a";
            input SI.Density rho_b "Density at port_b";
            input LossFactorData data "Constant loss factors for both flow directions" annotation(
              choices(choice = BaseClasses.PressureLosses.QuadraticTurbulent.LossFactorData.wallFriction(), choice = BaseClasses.PressureLosses.QuadraticTurbulent.LossFactorData.suddenExpansion(), choice = BaseClasses.PressureLosses.QuadraticTurbulent.LossFactorData.sharpEdgedOrifice()));
            input SI.MassFlowRate m_flow_small = 0.01 "Turbulent flow if |m_flow| >= m_flow_small";
            output SI.Pressure dp "Pressure drop (dp = port_a.p - port_b.p)";
          protected
            Real k1 = lossConstant_D_zeta(if data.zeta1_at_a then data.diameter_a else data.diameter_b, data.zeta1);
            Real k2 = lossConstant_D_zeta(if data.zeta2_at_a then data.diameter_a else data.diameter_b, data.zeta2);
          algorithm
/*
   dp = 0.5*zeta*rho*v*|v|
      = 0.5*zeta*rho*1/(rho*A)^2 * m_flow * |m_flow|
      = 0.5*zeta/A^2 *1/rho * m_flow * |m_flow|
      = k/rho * m_flow * |m_flow|
   k  = 0.5*zeta/A^2
      = 0.5*zeta/(pi*(D/2)^2)^2
      = 8*zeta/(pi*D^2)^2
  */
            dp := Utilities.regSquare2(m_flow, m_flow_small, k1 / rho_a, k2 / rho_b);
            annotation(
              smoothOrder = 1,
              Documentation(info = "<html>
<p>
Compute pressure drop from constant loss factor and mass flow rate (dp = f(m_flow)).
For small mass flow rates(|m_flow| &lt; m_flow_small), the characteristic is approximated by
a polynomial in order to have a finite derivative at zero mass flow rate.
</p>
</html>"));
          end pressureLoss_m_flow;

          function pressureLoss_m_flow_and_Re "Return pressure drop from constant loss factor, mass flow rate and Re (dp = f(m_flow))"
            extends Modelica.Icons.Function;
            import Modelica.Constants.pi;
            input SI.MassFlowRate m_flow "Mass flow rate from port_a to port_b";
            input SI.Density rho_a "Density at port_a";
            input SI.Density rho_b "Density at port_b";
            input SI.DynamicViscosity mu_a "Dynamic viscosity at port_a";
            input SI.DynamicViscosity mu_b "Dynamic viscosity at port_b";
            input LossFactorData data "Constant loss factors for both flow directions" annotation(
              choices(choice = BaseClasses.PressureLosses.QuadraticTurbulent.LossFactorData.wallFriction(), choice = BaseClasses.PressureLosses.QuadraticTurbulent.LossFactorData.suddenExpansion(), choice = BaseClasses.PressureLosses.QuadraticTurbulent.LossFactorData.sharpEdgedOrifice()));
            output SI.Pressure dp "Pressure drop (dp = port_a.p - port_b.p)";
          protected
            Real k0 = 2 * data.c0 / (pi * data.D_Re ^ 3);
            Real k1 = lossConstant_D_zeta(if data.zeta1_at_a then data.diameter_a else data.diameter_b, data.zeta1);
            Real k2 = lossConstant_D_zeta(if data.zeta2_at_a then data.diameter_a else data.diameter_b, data.zeta2);
            Real yd0 "Derivative of dp = f(m_flow) at zero, if data.zetaLaminarKnown";
            SI.MassFlowRate m_flow_turbulent "The turbulent region is: |m_flow| >= m_flow_turbulent";
          algorithm
/*
Turbulent region:
   Re = m_flow*(4/pi)/(D_Re*mu)
   dp = 0.5*zeta*rho*v*|v|
      = 0.5*zeta*rho*1/(rho*A)^2 * m_flow * |m_flow|
      = 0.5*zeta/A^2 *1/rho * m_flow * |m_flow|
      = k/rho * m_flow * |m_flow|
   k  = 0.5*zeta/A^2
      = 0.5*zeta/(pi*(D/2)^2)^2
      = 8*zeta/(pi*D^2)^2
   m_flow_turbulent = (pi/4)*D_Re*mu*Re_turbulent
   dp_turbulent     =  k/rho *(D_Re*mu*pi/4)^2 * Re_turbulent^2

   The start of the turbulent region is computed with mean values
   of dynamic viscosity mu and density rho. Otherwise, one has
   to introduce different "delta" values for both flow directions.
   In order to simplify the approach, only one delta is used.

Laminar region:
   dp = 0.5*zeta/(A^2*d) * m_flow * |m_flow|
      = 0.5 * c0/(|m_flow|*(4/pi)/(D_Re*mu)) / ((pi*(D_Re/2)^2)^2*d) * m_flow*|m_flow|
      = 0.5 * c0*(pi/4)*(D_Re*mu) * 16/(pi^2*D_Re^4*d) * m_flow*|m_flow|
      = 2*c0/(pi*D_Re^3) * mu/rho * m_flow
      = k0 * mu/rho * m_flow
   k0 = 2*c0/(pi*D_Re^3)

   In order that the derivative of dp=f(m_flow) is continuous
   at m_flow=0, the mean values of mu and d are used in the
   laminar region: mu/rho = (mu_a + mu_b)/(rho_a + rho_b)
   If data.zetaLaminarKnown = false then mu_a and mu_b are potentially zero
   (because dummy values) and therefore the division is only performed
   if zetaLaminarKnown = true.
*/
            m_flow_turbulent := pi / 8 * data.D_Re * (mu_a + mu_b) * data.Re_turbulent;
            yd0 := if data.zetaLaminarKnown then k0 * (mu_a + mu_b) / (rho_a + rho_b) else 0;
            dp := Utilities.regSquare2(m_flow, m_flow_turbulent, k1 / rho_a, k2 / rho_b, data.zetaLaminarKnown, yd0);
            annotation(
              smoothOrder = 1,
              Documentation(info = "<html>
<p>
Compute pressure drop from constant loss factor and mass flow rate (dp = f(m_flow)).
If the Reynolds-number Re &ge; data.Re_turbulent, the flow
is treated as a turbulent flow with constant loss factor zeta.
If the Reynolds-number Re &lt; data.Re_turbulent, the flow
is laminar and/or in a transition region between laminar and
turbulent. This region is approximated by two
polynomials of third order, one polynomial for m_flow &ge; 0
and one for m_flow &lt; 0.
The common derivative
of the two polynomials at Re = 0 is
computed from the equation \"data.c0/Re\".
</p>
<p>
If no data for c0 is available, the derivative at Re = 0 is computed in such
a way, that the second derivatives of the two polynomials
are identical at Re = 0. The polynomials are constructed, such that
they smoothly touch the characteristic curves in the turbulent
regions. The whole characteristic is therefore <b>continuous</b>
and has a <b>finite</b>, <b>continuous first derivative everywhere</b>.
In some cases, the constructed polynomials would \"vibrate\". This is
avoided by reducing the derivative at Re=0 in such a way that
the polynomials are guaranteed to be monotonically increasing.
The used sufficient criteria for monotonicity follows from:
</p>

<dl>
<dt> Fritsch F.N. and Carlson R.E. (1980):</dt>
<dd> <b>Monotone piecewise cubic interpolation</b>.
     SIAM J. Numerc. Anal., Vol. 17, No. 2, April 1980, pp. 238-246</dd>
</dl>
</html>"));
          end pressureLoss_m_flow_and_Re;

          partial model BaseModel "Generic pressure drop component with constant turbulent loss factor data and without an icon"
            extends Modelica.Fluid.Interfaces.PartialTwoPortTransport(dp_start = dp_nominal, m_flow_small = if system.use_eps_Re then system.eps_m_flow * m_flow_nominal else system.m_flow_small, m_flow(stateSelect = if momentumDynamics == Types.Dynamics.SteadyState then StateSelect.default else StateSelect.prefer));
            extends Modelica.Fluid.Interfaces.PartialLumpedFlow(final pathLength = 0, final momentumDynamics = Types.Dynamics.SteadyState);
            parameter LossFactorData data "Loss factor data";
            parameter Modelica.SIunits.MassFlowRate m_flow_nominal = if system.use_eps_Re then system.m_flow_nominal else 1e2 * system.m_flow_small "Nominal mass flow rate" annotation(
              Dialog(group = "Nominal operating point"));
            // Advanced
            parameter Boolean use_Re = system.use_eps_Re "= true, if turbulent region is defined by Re, otherwise by m_flow_small" annotation(
              Evaluate = true,
              Dialog(tab = "Advanced"));
            parameter Boolean from_dp = true "= true, use m_flow = f(dp) else dp = f(m_flow)" annotation(
              Evaluate = true,
              Dialog(tab = "Advanced"));
          protected
            parameter Medium.ThermodynamicState state_nominal = Medium.setState_pTX(Medium.reference_p, Medium.reference_T, Medium.reference_X) "Medium state to compute nominal pressure drop";
            parameter Modelica.SIunits.Pressure dp_nominal = pressureLoss_m_flow(m_flow_nominal, Medium.density(state_nominal), Medium.density(state_nominal), data, m_flow_small) "Nominal pressure loss";
            parameter Medium.AbsolutePressure dp_small(min = 0) = if system.use_eps_Re then dp_nominal / m_flow_nominal * m_flow_small else system.dp_small "Regularization of zero flow if |dp| < dp_small" annotation(
              Dialog(tab = "Advanced", enable = not use_Re and from_dp));
            //parameter Medium.MassFlowRate m_flow_small = system.m_flow_small
            //  "Turbulent flow if |m_flow| >= m_flow_small"
            //  annotation(Dialog(tab = "Advanced", enable=not from_dp));
            // Diagnostics
          public
            parameter Boolean show_Re = false "= true, if Reynolds number is included for plotting" annotation(
              Evaluate = true,
              Dialog(tab = "Advanced", group = "Diagnostics"));
            SI.ReynoldsNumber Re = Modelica.Fluid.Pipes.BaseClasses.CharacteristicNumbers.ReynoldsNumber_m_flow(m_flow, noEvent(if m_flow > 0 then Medium.dynamicViscosity(state_a) else Medium.dynamicViscosity(state_b)), data.D_Re) if show_Re "Reynolds number at diameter data.D_Re";
            // Variables
            Modelica.SIunits.Pressure dp_fg "pressure loss due to friction and gravity";
            Modelica.SIunits.Area A_mean = Modelica.Constants.pi / 4 * (data.diameter_a ^ 2 + data.diameter_b ^ 2) / 2 "mean cross flow area";
          equation
            Ib_flow = 0;
            F_p = A_mean * (Medium.pressure(state_b) - Medium.pressure(state_a));
            F_fg = A_mean * dp_fg;
            if from_dp then
              m_flow = homotopy(if use_Re then massFlowRate_dp_and_Re(dp_fg, Medium.density(state_a), Medium.density(state_b), Medium.dynamicViscosity(state_a), Medium.dynamicViscosity(state_b), data) else massFlowRate_dp(dp_fg, Medium.density(state_a), Medium.density(state_b), data, dp_small), m_flow_nominal * dp_fg / dp_nominal);
            else
              dp_fg = homotopy(if use_Re then pressureLoss_m_flow_and_Re(m_flow, Medium.density(state_a), Medium.density(state_b), Medium.dynamicViscosity(state_a), Medium.dynamicViscosity(state_b), data) else pressureLoss_m_flow(m_flow, Medium.density(state_a), Medium.density(state_b), data, m_flow_small), dp_nominal * m_flow / m_flow_nominal);
            end if;
// Isenthalpic state transformation (no storage and no loss of energy)
            port_a.h_outflow = inStream(port_b.h_outflow);
            port_b.h_outflow = inStream(port_a.h_outflow);
            annotation(
              Documentation(info = "<html>
<p>
This model computes the pressure loss of a pipe
segment (orifice, bending etc.) with a minimum amount of data
provided via parameter <b>data</b>.
If available, data should be provided for <b>both flow directions</b>,
i.e., flow from port_a to port_b and from port_b to port_a,
as well as for the <b>laminar</b> and the <b>turbulent</b> region.
It is also an option to provide the loss factor <b>only</b> for the
<b>turbulent</b> region for a flow from port_a to port_b.
</p>
<p>
The following equations are used:
</p>
<pre>   &Delta;p = 0.5*&zeta;*&rho;*v*|v|
      = 0.5*&zeta;/A^2 * (1/&rho;) * m_flow*|m_flow|
        Re = |v|*D*&rho;/&mu;
</pre>
<table border=1 cellspacing=0 cellpadding=2>
<tr><td><b>flow type</b></td>
    <td><b>&zeta;</b> = </td>
    <td><b>flow region</b></td></tr>
<tr><td>turbulent</td>
    <td><b>zeta1</b> = const.</td>
    <td>Re &ge;  Re_turbulent, v &ge; 0</td></tr>
<tr><td></td>
    <td><b>zeta2</b> = const.</td>
    <td>Re &ge; Re_turbulent, v &lt; 0</td></tr>
<tr><td>laminar</td>
    <td><b>c0</b>/Re</td>
    <td>both flow directions, Re small; c0 = const.</td></tr>
</table>
<p>
where
</p>
<ul>
<li> &Delta;p is the pressure drop: &Delta;p = port_a.p - port_b.p</li>
<li> v is the mean velocity.</li>
<li> &rho; is the density.</li>
<li> &zeta; is the loss factor that depends on the geometry of
     the pipe. In the turbulent flow regime, it is assumed that
     &zeta; is constant and is given by \"zeta1\" and
     \"zeta2\" depending on the flow direction.<br>
     When the Reynolds number Re is below \"Re_turbulent\", the
     flow is laminar for small flow velocities. For higher
     velocities there is a transition region from
     laminar to turbulent flow. The loss factor for
     laminar flow at small velocities is defined by the often occurring
     approximation c0/Re. If c0 is different for the two
     flow directions, the mean value has to be used
     (c0 = (c0_ab + c0_ba)/2).</li>
<li> The equation \"&Delta;p = 0.5*&zeta;*&rho;*v*|v|\" is either with
     respect to port_a or to port_b, depending on the definition
     of the particular loss factor &zeta; (in some references loss
     factors are defined with respect to port_a, in other references
     with respect to port_b).</li>
<li> Re = |v|*D_Re*&rho;/&mu; = |m_flow|*D_Re/(A_Re*&mu;)
     is the Reynolds number at the smallest cross
     section area. This is often at port_a or at port_b, but can
     also be between the two ports. In the record, the diameter
     D_Re of this smallest cross section area has to be provided, as
     well, as Re_turbulent, the absolute value of the
     Reynolds number at which
     the turbulent flow starts. If Re_turbulent is different for
     the two flow directions, use the smaller value as Re_turbulent.</li>
<li> D is the diameter of the pipe. If the pipe has not a
     circular cross section, D = 4*A/P, where A is the cross section
     area and P is the wetted perimeter.</li>
<li> A is the cross section area with A = &pi;(D/2)^2.</li>
<li> &mu; is the dynamic viscosity.</li>
</ul>
<p>
The laminar and the transition region is usually of
not much technical interest because the operating point is
mostly in the turbulent regime. For simplification and for
numerical reasons, this whole region is described by two
polynomials of third order, one polynomial for m_flow &ge; 0
and one for m_flow &lt; 0. The polynomials start at
Re = |m_flow|*4/(&pi;*D_Re*&mu;), where D_Re is the
smallest diameter between port_a and port_b.
The common derivative
of the two polynomials at Re = 0 is
computed from the equation \"c0/Re\". Note, the pressure drop
equation above in the laminar region is always defined
with respect to the smallest diameter D_Re.
</p>
<p>
If no data for c0 is available, the derivative at Re = 0 is computed in such
a way, that the second derivatives of the two polynomials
are identical at Re = 0. The polynomials are constructed, such that
they smoothly touch the characteristic curves in the turbulent
regions. The whole characteristic is therefore <b>continuous</b>
and has a <b>finite</b>, <b>continuous first derivative everywhere</b>.
In some cases, the constructed polynomials would \"vibrate\". This is
avoided by reducing the derivative at Re=0 in such a way that
the polynomials are guaranteed to be monotonically increasing.
The used sufficient criteria for monotonicity follows from:
</p>

<dl>
<dt> Fritsch F.N. and Carlson R.E. (1980):</dt>
<dd> <b>Monotone piecewise cubic interpolation</b>.
     SIAM J. Numerc. Anal., Vol. 17, No. 2, April 1980, pp. 238-246</dd>
</dl>
</html>"));
          end BaseModel;

          model TestWallFriction "Pressure drop in pipe due to wall friction (only for test purposes; if needed use Pipes.StaticPipe instead)"
            extends BaseModel(final data = LossFactorData.wallFriction(length, diameter, roughness));
            parameter SI.Length length "Length of pipe";
            parameter SI.Diameter diameter "Inner diameter of pipe";
            parameter SI.Length roughness(min = 1e-10) "Absolute roughness of pipe (> 0 required, details see info layer)";
            annotation(
              Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-150, 80}, {150, 120}}, lineColor = {0, 0, 0}, textString = "%name"), Rectangle(extent = {{-100, 60}, {100, -60}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192, 192, 192}), Rectangle(extent = {{-100, 34}, {100, -36}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {0, 127, 255}), Text(extent = {{-134, -66}, {130, -92}}, lineColor = {0, 0, 0}, textString = "quad. turbulent")}),
              Documentation(info = "<html>

</html>"));
          end TestWallFriction;

          partial model BaseModelNonconstantCrossSectionArea "Generic pressure drop component with constant turbulent loss factor data and without an icon, for non-constant cross section area"
            extends Modelica.Fluid.Interfaces.PartialTwoPortTransport(final dp_start = dp_nominal, m_flow_small = if system.use_eps_Re then system.eps_m_flow * m_flow_nominal else system.m_flow_small, m_flow(stateSelect = if momentumDynamics == Types.Dynamics.SteadyState then StateSelect.default else StateSelect.prefer));
            extends Modelica.Fluid.Interfaces.PartialLumpedFlow(final pathLength = 0, final momentumDynamics = Types.Dynamics.SteadyState);
            parameter LossFactorData data "Loss factor data";
            parameter Modelica.SIunits.MassFlowRate m_flow_nominal = if system.use_eps_Re then system.m_flow_nominal else 1e2 * system.m_flow_small "Nominal mass flow rate" annotation(
              Dialog(group = "Nominal operating point"));
            // Advanced
            /// Other settings than the final values are not yet implemented ///
            final parameter Boolean use_Re = false "= true, if turbulent region is defined by Re, otherwise by m_flow_small" annotation(
              Evaluate = true,
              Dialog(tab = "Advanced"));
            final parameter Boolean from_dp = false "= true, use m_flow = f(dp) else dp = f(m_flow)" annotation(
              Evaluate = true,
              Dialog(tab = "Advanced"));
            // End not yet implemented /////////////////////////////////////////
          protected
            parameter Medium.ThermodynamicState state_nominal = Medium.setState_pTX(Medium.reference_p, Medium.reference_T, Medium.reference_X) "Medium state to compute nominal pressure drop" annotation(
              HideResult = true);
            parameter Modelica.SIunits.Pressure dp_nominal = pressureLoss_m_flow(m_flow_nominal, Medium.density(state_nominal), Medium.density(state_nominal), data, m_flow_small) "Nominal pressure loss";
            parameter Medium.AbsolutePressure dp_small(min = 0) = if system.use_eps_Re then dp_nominal / m_flow_nominal * m_flow_small else system.dp_small "Regularization of zero flow if |dp| < dp_small" annotation(
              Dialog(tab = "Advanced", enable = not use_Re and from_dp));
            //parameter Medium.MassFlowRate m_flow_small = system.m_flow_small
            //  "Turbulent flow if |m_flow| >= m_flow_small"
            //  annotation(Dialog(tab = "Advanced", enable=not from_dp));
            // Diagnostics
          public
            parameter Boolean show_Re = false "= true, if Reynolds number is included for plotting" annotation(
              Evaluate = true,
              Dialog(tab = "Advanced", group = "Diagnostics"));
            SI.ReynoldsNumber Re = Modelica.Fluid.Pipes.BaseClasses.CharacteristicNumbers.ReynoldsNumber_m_flow(m_flow, noEvent(if m_flow > 0 then Medium.dynamicViscosity(state_a) else Medium.dynamicViscosity(state_b)), data.D_Re) if show_Re "Reynolds number at diameter data.D_Re";
            parameter Boolean show_totalPressures = false "= true, if total pressures are included for plotting" annotation(
              Evaluate = true,
              Dialog(tab = "Advanced", group = "Diagnostics"));
            SI.AbsolutePressure p_total_a = port_a.p + 0.5 * m_flow ^ 2 / ((Modelica.Constants.pi / 4 * data.diameter_a ^ 2) ^ 2 * noEvent(if port_a.m_flow > 0 then Medium.density(state_a) else Medium.density(state_b))) if show_totalPressures "Total pressure at port_a";
            SI.AbsolutePressure p_total_b = port_b.p + 0.5 * m_flow ^ 2 / ((Modelica.Constants.pi / 4 * data.diameter_b ^ 2) ^ 2 * noEvent(if port_b.m_flow > 0 then Medium.density(state_b) else Medium.density(state_a))) if show_totalPressures "Total pressure at port_a";
            parameter Boolean show_portVelocities = false "= true, if port velocities are included for plotting" annotation(
              Evaluate = true,
              Dialog(tab = "Advanced", group = "Diagnostics"));
            SI.Velocity v_a = port_a.m_flow / (Modelica.Constants.pi / 4 * data.diameter_a ^ 2 * noEvent(if port_a.m_flow > 0 then Medium.density(state_a) else Medium.density(state_b))) if show_portVelocities "Fluid velocity into port_a";
            SI.Velocity v_b = port_b.m_flow / (Modelica.Constants.pi / 4 * data.diameter_b ^ 2 * noEvent(if port_b.m_flow > 0 then Medium.density(state_b) else Medium.density(state_a))) if show_portVelocities "Fluid velocity into port_b";
            // Variables
            Modelica.SIunits.Pressure dp_fg "pressure loss due to friction and gravity";
            Modelica.SIunits.Area A_mean = Modelica.Constants.pi / 4 * (data.diameter_a ^ 2 + data.diameter_b ^ 2) / 2 "mean cross flow area";
            Medium.ThermodynamicState state_b_des "Thermodynamic state at port b for flow a -> b";
            Medium.ThermodynamicState state_a_nondes "Thermodynamic state at port a for flow a <- b";
          equation
            Ib_flow = 0;
            F_p = A_mean * (Medium.pressure(state_b) - Medium.pressure(state_a));
            F_fg = A_mean * dp_fg;
            if from_dp then
              m_flow = if use_Re then massFlowRate_dp_and_Re(dp_fg, Medium.density(state_a), Medium.density(state_b), Medium.dynamicViscosity(state_a), Medium.dynamicViscosity(state_b), data) else massFlowRate_dp(dp_fg, Medium.density(state_a), Medium.density(state_b), data, dp_small);
            else
              dp_fg = if use_Re then pressureLoss_m_flow_and_Re(m_flow, Medium.density(state_a), Medium.density(state_b), Medium.dynamicViscosity(state_a), Medium.dynamicViscosity(state_b), data) else pressureLoss_m_flow_totalPressure(m_flow, Medium.density(state_a), Medium.density(state_b_des), Medium.density(state_b), Medium.density(state_a_nondes), data, m_flow_small);
            end if;
// Isenthalpic state transformation (no storage and no loss of energy)
            port_a.h_outflow = inStream(port_b.h_outflow);
            port_b.h_outflow = inStream(port_a.h_outflow);
// medium states for downstream properties, may want to change this neglecting the only difference from state_a, state_b, which is in pressure
// This will remove the extra interaction variables
            state_b_des = Medium.setState_phX(port_b.p, inStream(port_a.h_outflow), inStream(port_a.Xi_outflow));
            state_a_nondes = Medium.setState_phX(port_a.p, inStream(port_b.h_outflow), inStream(port_b.Xi_outflow));
            annotation(
              Documentation(info = "<html>
<p>
This model computes the pressure loss of a pipe
segment (orifice, bending etc.) with a minimum amount of data
provided via parameter <b>data</b>.
If available, data should be provided for <b>both flow directions</b>,
i.e., flow from port_a to port_b and from port_b to port_a,
as well as for the <b>laminar</b> and the <b>turbulent</b> region.
It is also an option to provide the loss factor <b>only</b> for the
<b>turbulent</b> region for a flow from port_a to port_b.
</p>
<p>
The following equations are used:
</p>
<pre>   &Delta;p = 0.5*&zeta;*&rho;*v*|v|
      = 0.5*&zeta;/A^2 * (1/&rho;) * m_flow*|m_flow|
        Re = |v|*D*&rho;/&mu;
</pre>
<table border=1 cellspacing=0 cellpadding=2>
<tr><td><b>flow type</b></td>
    <td><b>&zeta;</b> = </td>
    <td><b>flow region</b></td></tr>
<tr><td>turbulent</td>
    <td><b>zeta1</b> = const.</td>
    <td>Re &ge;  Re_turbulent, v &ge; 0</td></tr>
<tr><td></td>
    <td><b>zeta2</b> = const.</td>
    <td>Re &ge; Re_turbulent, v &lt; 0</td></tr>
<tr><td>laminar</td>
    <td><b>c0</b>/Re</td>
    <td>both flow directions, Re small; c0 = const.</td></tr>
</table>
<p>
where
</p>
<ul>
<li> &Delta;p is the pressure drop: &Delta;p = port_a.p - port_b.p</li>
<li> v is the mean velocity.</li>
<li> &rho; is the density.</li>
<li> &zeta; is the loss factor that depends on the geometry of
     the pipe. In the turbulent flow regime, it is assumed that
     &zeta; is constant and is given by \"zeta1\" and
     \"zeta2\" depending on the flow direction.<br>
     When the Reynolds number Re is below \"Re_turbulent\", the
     flow is laminar for small flow velocities. For higher
     velocities there is a transition region from
     laminar to turbulent flow. The loss factor for
     laminar flow at small velocities is defined by the often occurring
     approximation c0/Re. If c0 is different for the two
     flow directions, the mean value has to be used
     (c0 = (c0_ab + c0_ba)/2).</li>
<li> The equation \"&Delta;p = 0.5*&zeta;*&rho;*v*|v|\" is either with
     respect to port_a or to port_b, depending on the definition
     of the particular loss factor &zeta; (in some references loss
     factors are defined with respect to port_a, in other references
     with respect to port_b).</li>

<li> Re = |v|*D_Re*&rho;/&mu; = |m_flow|*D_Re/(A_Re*&mu;)
     is the Reynolds number at the smallest cross
     section area. This is often at port_a or at port_b, but can
     also be between the two ports. In the record, the diameter
     D_Re of this smallest cross section area has to be provided, as
     well, as Re_turbulent, the absolute value of the
     Reynolds number at which
     the turbulent flow starts. If Re_turbulent is different for
     the two flow directions, use the smaller value as Re_turbulent.</li>
<li> D is the diameter of the pipe. If the pipe has not a
     circular cross section, D = 4*A/P, where A is the cross section
     area and P is the wetted perimeter.</li>
<li> A is the cross section area with A = &pi;(D/2)^2.</li>
<li> &mu; is the dynamic viscosity.</li>
</ul>
<p>
The laminar and the transition region is usually of
not much technical interest because the operating point is
mostly in the turbulent regime. For simplification and for
numerical reasons, this whole region is described by two
polynomials of third order, one polynomial for m_flow &ge; 0
and one for m_flow &lt; 0. The polynomials start at
Re = |m_flow|*4/(&pi;*D_Re*&mu;), where D_Re is the
smallest diameter between port_a and port_b.
The common derivative
of the two polynomials at Re = 0 is
computed from the equation \"c0/Re\". Note, the pressure drop
equation above in the laminar region is always defined
with respect to the smallest diameter D_Re.
</p>
<p>
If no data for c0 is available, the derivative at Re = 0 is computed in such
a way, that the second derivatives of the two polynomials
are identical at Re = 0. The polynomials are constructed, such that
they smoothly touch the characteristic curves in the turbulent
regions. The whole characteristic is therefore <b>continuous</b>
and has a <b>finite</b>, <b>continuous first derivative everywhere</b>.
In some cases, the constructed polynomials would \"vibrate\". This is
avoided by reducing the derivative at Re=0 in such a way that
the polynomials are guaranteed to be monotonically increasing.
The used sufficient criteria for monotonicity follows from:
</p>

<dl>
<dt> Fritsch F.N. and Carlson R.E. (1980):</dt>
<dd> <b>Monotone piecewise cubic interpolation</b>.
     SIAM J. Numerc. Anal., Vol. 17, No. 2, April 1980, pp. 238-246</dd>
</dl>
</html>"));
          end BaseModelNonconstantCrossSectionArea;

          function pressureLoss_m_flow_totalPressure "Return pressure drop from constant loss factor and mass flow rate (dp = f(m_flow))"
            extends Modelica.Icons.Function;
            input SI.MassFlowRate m_flow "Mass flow rate from port_a to port_b";
            input SI.Density rho_a_des "Density at port_a, mass flow in design direction a -> b";
            input SI.Density rho_b_des "Density at port_b, mass flow in design direction a -> b";
            input SI.Density rho_b_nondes "Density at port_b, mass flow against design direction a <- b";
            input SI.Density rho_a_nondes "Density at port_a, mass flow against design direction a <- b";
            input LossFactorData data "Constant loss factors for both flow directions" annotation(
              choices(choice = BaseClasses.PressureLosses.QuadraticTurbulent.LossFactorData.wallFriction(), choice = BaseClasses.PressureLosses.QuadraticTurbulent.LossFactorData.suddenExpansion(), choice = BaseClasses.PressureLosses.QuadraticTurbulent.LossFactorData.sharpEdgedOrifice()));
            input SI.MassFlowRate m_flow_small = 0.01 "Turbulent flow if |m_flow| >= m_flow_small";
            output SI.Pressure dp "Pressure drop (dp = port_a.p - port_b.p)";
          protected
            SI.Area A_a = Modelica.Constants.pi * data.diameter_a ^ 2 / 4 "Cross section area at port_a";
            SI.Area A_b = Modelica.Constants.pi * data.diameter_b ^ 2 / 4 "Cross section area at port_b";
          algorithm
            dp := 1 / 2 * m_flow ^ 2 * (if m_flow > 0 then data.zeta1 / (if data.zeta1_at_a then rho_a_des * A_a ^ 2 else rho_b_des * A_b ^ 2) - 1 / (rho_a_des * A_a ^ 2) + 1 / (rho_b_des * A_b ^ 2) else (-data.zeta2 / (if data.zeta2_at_a then rho_a_nondes * A_a ^ 2 else rho_b_nondes * A_b ^ 2)) - 1 / (rho_a_nondes * A_a ^ 2) + 1 / (rho_b_nondes * A_b ^ 2));
            annotation(
              smoothOrder = 1,
              Documentation(info = "<html>
<p>
Compute pressure drop from constant loss factor and mass flow rate (dp = f(m_flow)).
For small mass flow rates(|m_flow| &lt; m_flow_small), the characteristic is approximated by
a polynomial in order to have a finite derivative at zero mass flow rate.
</p>
</html>"));
          end pressureLoss_m_flow_totalPressure;
          annotation(
            Documentation(info = "<html>
<p>
This library provides pressure loss factors of a pipe
segment (orifice, bending etc.) with a minimum amount of data.
If available, data can be provided for <b>both flow directions</b>,
i.e., flow from port_a to port_b and from port_b to port_a,
as well as for the <b>laminar</b> and the <b>turbulent</b> region.
It is also an option to provide the loss factor <b>only</b> for the
<b>turbulent</b> region for a flow from port_a to port_b.
Basically, the pressure drop is defined by the following
equation:
</p>
<pre>   &Delta;p = 0.5*&zeta;*&rho;*v*|v|
      = 0.5*&zeta;/A^2 * (1/&rho;) * m_flow*|m_flow|
      = 8*&zeta;/(&pi;^2*D^4*&rho;) * m_flow*|m_flow|
</pre>
<p>
where
</p>
<ul>
<li> &Delta;p is the pressure drop: &Delta;p = port_a.p - port_b.p</li>
<li> v is the mean velocity.</li>
<li> &rho; is the density.</li>
<li> &zeta; is the loss factor that depends on the geometry of
     the pipe. In the turbulent flow regime, it is assumed that
     &zeta; is constant and is given by \"zeta1\" and
     \"zeta2\" depending on the flow direction.</li>
<li> D is the diameter of the pipe segment. If this is not a
     circular cross section, D = 4*A/P, where A is the cross section
     area and P is the wetted perimeter.</li>
</ul>

</html>"));
        end QuadraticTurbulent;

        partial model PartialTeeJunction "Base class for a splitting/joining component with three ports"
          import Modelica.Fluid.Types;
          import Modelica.Fluid.Types.PortFlowDirection;
          replaceable package Medium = Modelica.Media.Interfaces.PartialMedium "Medium in the component" annotation(
            choicesAllMatching = true);
          Modelica.Fluid.Interfaces.FluidPort_a port_1(redeclare package Medium = Medium, m_flow(min = if portFlowDirection_1 == PortFlowDirection.Entering then 0.0 else -Modelica.Constants.inf, max = if portFlowDirection_1 == PortFlowDirection.Leaving then 0.0 else Modelica.Constants.inf)) annotation(
            Placement(transformation(extent = {{-110, -10}, {-90, 10}})));
          Modelica.Fluid.Interfaces.FluidPort_b port_2(redeclare package Medium = Medium, m_flow(min = if portFlowDirection_2 == PortFlowDirection.Entering then 0.0 else -Modelica.Constants.inf, max = if portFlowDirection_2 == PortFlowDirection.Leaving then 0.0 else Modelica.Constants.inf)) annotation(
            Placement(transformation(extent = {{90, -10}, {110, 10}})));
          Modelica.Fluid.Interfaces.FluidPort_a port_3(redeclare package Medium = Medium, m_flow(min = if portFlowDirection_3 == PortFlowDirection.Entering then 0.0 else -Modelica.Constants.inf, max = if portFlowDirection_3 == PortFlowDirection.Leaving then 0.0 else Modelica.Constants.inf)) annotation(
            Placement(transformation(extent = {{-10, 90}, {10, 110}})));
        protected
          parameter PortFlowDirection portFlowDirection_1 = PortFlowDirection.Bidirectional "Flow direction for port_1" annotation(
            Dialog(tab = "Advanced"));
          parameter PortFlowDirection portFlowDirection_2 = PortFlowDirection.Bidirectional "Flow direction for port_2" annotation(
            Dialog(tab = "Advanced"));
          parameter PortFlowDirection portFlowDirection_3 = PortFlowDirection.Bidirectional "Flow direction for port_3" annotation(
            Dialog(tab = "Advanced"));
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, 44}, {100, -44}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {0, 127, 255}), Text(extent = {{-150, -89}, {150, -129}}, lineColor = {0, 0, 255}, textString = "%name"), Rectangle(extent = {{-44, 100}, {44, 44}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.VerticalCylinder, fillColor = {0, 127, 255}), Rectangle(extent = {{-22, 82}, {21, -4}}, fillPattern = FillPattern.Solid, fillColor = {0, 128, 255}, pattern = LinePattern.None, lineColor = {0, 0, 0})}));
        end PartialTeeJunction;

        package Bends "Pressure loss functions for bends"
          extends Modelica.Icons.VariantsPackage;

          package CurvedBend "Pressure loss functions for curved bends"
            extends Modelica.Icons.Package;

            function massFlowRate "Return mass flow rate m_flow as function of pressure loss dp for a curved bend"
              extends Modelica.Icons.Function;
              input SI.Pressure dp "Pressure loss";
              input Geometry geometry "Geometry of bend";
              input SI.Density d_a "Density at port_a when fluid is flowing from port_a to port_b";
              input SI.Density d_b "Density at port_b when fluid is flowing from port_b to port_a";
              input SI.DynamicViscosity eta_a "Dynamic viscosity at port_a when fluid is flowing from port_a to port_b";
              input SI.DynamicViscosity eta_b "Dynamic viscosity at port_b when fluid is flowing from port_b to port_a";
              input SI.AbsolutePressure dp_small "Small pressure drop used for regularization if m_flow=f(...,dp_small,..,dp)";
              input SI.MassFlowRate m_flow_small "Small mass flow rate used for regularization if dp=f_inv(...,m_flow_small,m_flow)";
              output SI.MassFlowRate m_flow "Mass flow rate (= port_a.m_flow)";
            algorithm
              m_flow := Modelica.Fluid.Dissipation.PressureLoss.Bend.dp_curvedOverall_MFLOW(geometry, Modelica.Fluid.Dissipation.PressureLoss.Bend.dp_curvedOverall_IN_var(rho = Modelica.Fluid.Utilities.regStep(dp, d_a, d_b, dp_small), eta = Modelica.Fluid.Utilities.regStep(dp, eta_a, eta_b, dp_small)), dp);
              annotation(
                Inline = false,
                LateInline = true,
                inverse(dp = Modelica.Fluid.Fittings.BaseClasses.Bends.CurvedBend.pressureLoss(m_flow, geometry, d_a, d_b, eta_a, eta_b, dp_small, m_flow_small)),
                Documentation(info = "<html>
<p>
This function returns the mass flow rate m_flow as function of pressure loss dp for a curved bend.
The details of the function are described
<a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.PressureLoss.Bend.dp_curvedOverall\">here</a>.
</p>

<p>
The bend characteristic is valid for constant density and constant dynamic viscosity.
It can be approximately also used for compressible media. This is performed by providing
the upstream density and upstream dynamic viscosity. In order to be able to regularize density
and dynamic viscosity around zero mass flow rate, the two quantities have to be given if
fluid flows from port_a to port_b (d_a, eta_a) and if fluid flows from port_b to port_a
(d_b, eta_b).
</p>

</html>"));
            end massFlowRate;

            function pressureLoss "Return pressure loss dp as function of mass flow rate m_flow for a curved bend"
              extends Modelica.Icons.Function;
              input SI.MassFlowRate m_flow "Mass flow rate (= port_a.m_flow)";
              input Geometry geometry "Geometry of bend";
              input SI.Density d_a "Density at port_a when fluid is flowing from port_a to port_b";
              input SI.Density d_b "Density at port_b when fluid is flowing from port_b to port_a";
              input SI.DynamicViscosity eta_a "Dynamic viscosity at port_a when fluid is flowing from port_a to port_b";
              input SI.DynamicViscosity eta_b "Dynamic viscosity at port_b when fluid is flowing from port_b to port_a";
              input SI.AbsolutePressure dp_small "Small pressure drop used for regularization if m_flow=f(...,dp_small,..,dp)";
              input SI.MassFlowRate m_flow_small "Small mass flow rate used for regularization if dp=f_inv(...,m_flow_small,m_flow)";
              output SI.Pressure dp "Pressure loss";
            algorithm
              dp := Modelica.Fluid.Dissipation.PressureLoss.Bend.dp_curvedOverall_DP(geometry, Modelica.Fluid.Dissipation.PressureLoss.Bend.dp_curvedOverall_IN_var(rho = Modelica.Fluid.Utilities.regStep(m_flow, d_a, d_b, m_flow_small), eta = Modelica.Fluid.Utilities.regStep(m_flow, eta_a, eta_b, m_flow_small)), m_flow);
              annotation(
                Inline = true,
                Documentation(info = "<html>
<p>
This function returns the pressure loss dp as function of mass flow rate m_flow for a curved bend.
The details of the function are described
<a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.PressureLoss.Bend.dp_curvedOverall\">here</a>.
</p>

<p>
The bend characteristic is valid for constant density and constant dynamic viscosity.
It can be approximately also used for compressible media. This is performed by providing
the upstream density and upstream dynamic viscosity. In order to be able to regularize density
and dynamic viscosity around zero mass flow rate, the two quantities have to be given if
fluid flows from port_a to port_b (d_a, eta_a) and if fluid flows from port_b to port_a
(d_b, eta_b).
</p>
</html>"));
            end pressureLoss;

            record Geometry "Geometric data for a curved bend"
              extends Modelica.Icons.Record;
              SI.Diameter d_hyd "Hydraulic diameter" annotation(
                Dialog);
              SI.Radius R_0 "Curvature radius" annotation(
                Dialog);
              SI.Angle delta = 1.5707963267949 "Angle of turning" annotation(
                Dialog);
              Modelica.Fluid.Types.Roughness K = 2.5e-5 "Absolute roughness, with a default for a smooth steel pipe" annotation(
                Dialog);
              annotation(
                Documentation(info = "<html>
<p>
This record is used to define the geometric (constant) data of a curved bend.
The details of the record are described
<a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.PressureLoss.Bend.dp_curvedOverall\">here</a>.
</p>
</html>"));
            end Geometry;
            annotation(
              Documentation(info = "<html>
<p>
This package contains utility functions and records
for the CurvedBend fitting component.
</p>

</html>"));
          end CurvedBend;

          package EdgedBend "Pressure loss functions for edged bends"
            extends Modelica.Icons.Package;

            function massFlowRate "Return mass flow rate m_flow as function of pressure loss dp for a curved bend"
              extends Modelica.Icons.Function;
              input SI.Pressure dp "Pressure loss";
              input Geometry geometry "Geometry of bend";
              input SI.Density d_a "Density at port_a when fluid is flowing from port_a to port_b";
              input SI.Density d_b "Density at port_b when fluid is flowing from port_b to port_a";
              input SI.DynamicViscosity eta_a "Dynamic viscosity at port_a when fluid is flowing from port_a to port_b";
              input SI.DynamicViscosity eta_b "Dynamic viscosity at port_b when fluid is flowing from port_b to port_a";
              input SI.AbsolutePressure dp_small "Small pressure drop used for regularization if m_flow=f(...,dp_small,..,dp)";
              input SI.MassFlowRate m_flow_small "Small mass flow rate used for regularization if dp=f_inv(...,m_flow_small,m_flow)";
              output SI.MassFlowRate m_flow "Mass flow rate (= port_a.m_flow)";
            algorithm
              m_flow := Modelica.Fluid.Dissipation.PressureLoss.Bend.dp_edgedOverall_MFLOW(Modelica.Fluid.Dissipation.PressureLoss.Bend.dp_edgedOverall_IN_con(d_hyd = geometry.d_hyd, delta = geometry.delta, K = geometry.K), Modelica.Fluid.Dissipation.PressureLoss.Bend.dp_edgedOverall_IN_var(rho = Modelica.Fluid.Utilities.regStep(dp, d_a, d_b, dp_small), eta = Modelica.Fluid.Utilities.regStep(dp, eta_a, eta_b, dp_small)), dp);
              annotation(
                Inline = false,
                LateInline = true,
                inverse(dp = Modelica.Fluid.Fittings.BaseClasses.Bends.EdgedBend.pressureLoss(m_flow, geometry, d_a, d_b, eta_a, eta_b, dp_small, m_flow_small)),
                Documentation(info = "<html>
<p>
This function returns the mass flow rate m_flow as function of pressure loss dp for an edged bend.
The details of the function are described
<a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.PressureLoss.Bend.dp_edgedOverall\">here</a>.
</p>

<p>
The bend characteristic is valid for constant density and constant dynamic viscosity.
It can be approximately also used for compressible media. This is performed by providing
the upstream density and upstream dynamic viscosity. In order to be able to regularize density
and dynamic viscosity around zero mass flow rate, the two quantities have to be given if
fluid flows from port_a to port_b (d_a, eta_a) and if fluid flows from port_b to port_a
(d_b, eta_b).
</p>

</html>"));
            end massFlowRate;

            function pressureLoss "Return pressure loss dp as function of mass flow rate m_flow for a curved bend"
              extends Modelica.Icons.Function;
              input SI.MassFlowRate m_flow "Mass flow rate (= port_a.m_flow)";
              input Geometry geometry "Geometry of bend";
              input SI.Density d_a "Density at port_a when fluid is flowing from port_a to port_b";
              input SI.Density d_b "Density at port_b when fluid is flowing from port_b to port_a";
              input SI.DynamicViscosity eta_a "Dynamic viscosity at port_a when fluid is flowing from port_a to port_b";
              input SI.DynamicViscosity eta_b "Dynamic viscosity at port_b when fluid is flowing from port_b to port_a";
              input SI.AbsolutePressure dp_small "Small pressure drop used for regularization if m_flow=f(...,dp_small,..,dp)";
              input SI.MassFlowRate m_flow_small "Small mass flow rate used for regularization if dp=f_inv(...,m_flow_small,m_flow)";
              output SI.Pressure dp "Pressure loss";
            algorithm
              dp := Modelica.Fluid.Dissipation.PressureLoss.Bend.dp_edgedOverall_DP(Modelica.Fluid.Dissipation.PressureLoss.Bend.dp_edgedOverall_IN_con(d_hyd = geometry.d_hyd, delta = geometry.delta, K = geometry.K), Modelica.Fluid.Dissipation.PressureLoss.Bend.dp_edgedOverall_IN_var(rho = Modelica.Fluid.Utilities.regStep(m_flow, d_a, d_b, m_flow_small), eta = Modelica.Fluid.Utilities.regStep(m_flow, eta_a, eta_b, m_flow_small)), m_flow);
              annotation(
                Inline = true,
                Documentation(info = "<html>
<p>
This function returns the pressure loss dp as function of mass flow rate m_flow for an edged bend.
The details of the function are described
<a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.PressureLoss.Bend.dp_edgedOverall\">here</a>.
</p>

<p>
The bend characteristic is valid for constant density and constant dynamic viscosity.
It can be approximately also used for compressible media. This is performed by providing
the upstream density and upstream dynamic viscosity. In order to be able to regularize density
and dynamic viscosity around zero mass flow rate, the two quantities have to be given if
fluid flows from port_a to port_b (d_a, eta_a) and if fluid flows from port_b to port_a
(d_b, eta_b).
</p>
</html>"));
            end pressureLoss;

            record Geometry "Geometric data for a curved bend"
              extends Modelica.Icons.Record;
              SI.Diameter d_hyd "Hydraulic diameter" annotation(
                Dialog);
              SI.Angle delta "Angle of turning" annotation(
                Dialog);
              Modelica.Fluid.Types.Roughness K = 2.5e-5 "Absolute roughness, with a default for a smooth steel pipe" annotation(
                Dialog);
              annotation(
                Documentation(info = "<html>
<p>
This record is used to define the geometric (constant) data of an edged bend.
The details of the record are described
<a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.PressureLoss.Bend.dp_edgedOverall\">here</a>.
</p>
</html>"));
            end Geometry;
            annotation(
              Documentation(info = "<html>
<p>
This package contains utility functions and records
for the EdgedBend fitting component.
</p>

</html>"));
          end EdgedBend;
        end Bends;

        package Orifices "Pressure loss functions for orifices"
          extends Modelica.Icons.VariantsPackage;

          package ThickEdgedOrifice "Pressure loss functions for thick edged orifices"
            extends Modelica.Icons.Package;

            function massFlowRate "Return mass flow rate m_flow as function of pressure loss dp for a thick edged orifice"
              extends Modelica.Icons.Function;
              input SI.Pressure dp "Pressure loss";
              input Geometry geometry "Geometry of bend";
              input SI.Density d_a "Density at port_a when fluid is flowing from port_a to port_b";
              input SI.Density d_b "Density at port_b when fluid is flowing from port_b to port_a";
              input SI.DynamicViscosity eta_a "Dynamic viscosity at port_a when fluid is flowing from port_a to port_b";
              input SI.DynamicViscosity eta_b "Dynamic viscosity at port_b when fluid is flowing from port_b to port_a";
              input SI.AbsolutePressure dp_small "Small pressure drop used for regularization if m_flow=f(...,dp_small,..,dp)";
              input SI.MassFlowRate m_flow_small "Small mass flow rate used for regularization if dp=f_inv(...,m_flow_small,m_flow)";
              output SI.MassFlowRate m_flow "Mass flow rate (= port_a.m_flow)";
            algorithm
              m_flow := Modelica.Fluid.Dissipation.PressureLoss.Orifice.dp_thickEdgedOverall_MFLOW(Modelica.Fluid.Dissipation.PressureLoss.Orifice.dp_thickEdgedOverall_IN_con(A_0 = geometry.venaCrossArea, A_1 = geometry.crossArea, C_0 = geometry.venaPerimeter, C_1 = geometry.perimeter, L = geometry.venaLength, dp_smooth = dp_small), Modelica.Fluid.Dissipation.PressureLoss.Orifice.dp_thickEdgedOverall_IN_var(rho = Modelica.Fluid.Utilities.regStep(dp, d_a, d_b, dp_small), eta = Modelica.Fluid.Utilities.regStep(dp, eta_a, eta_b, dp_small)), dp);
              annotation(
                Inline = false,
                LateInline = true,
                inverse(dp = Modelica.Fluid.Fittings.BaseClasses.Orifices.ThickEdgedOrifice.pressureLoss(m_flow, geometry, d_a, d_b, eta_a, eta_b, dp_small, m_flow_small)),
                Documentation(info = "<html>
<p>
This function returns the mass flow rate m_flow as function of pressure loss dp for a thick edged orifice.
The details of the function are described
<a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.PressureLoss.Orifice.dp_thickEdgedOverall\">here</a>.
</p>

<p>
The orifice characteristic is valid for constant density and constant dynamic viscosity.
It can be approximately also used for compressible media. This is performed by providing
the upstream density and upstream dynamic viscosity. In order to be able to regularize density
and dynamic viscosity around zero mass flow rate, the two quantities have to be given if
fluid flows from port_a to port_b (d_a, eta_a) and if fluid flows from port_b to port_a
(d_b, eta_b).
</p>

</html>"));
            end massFlowRate;

            function pressureLoss "Return pressure loss dp as function of mass flow rate m_flow for a thick edged orifice"
              extends Modelica.Icons.Function;
              input SI.MassFlowRate m_flow "Mass flow rate (= port_a.m_flow)";
              input Geometry geometry "Geometry of bend";
              input SI.Density d_a "Density at port_a when fluid is flowing from port_a to port_b";
              input SI.Density d_b "Density at port_b when fluid is flowing from port_b to port_a";
              input SI.DynamicViscosity eta_a "Dynamic viscosity at port_a when fluid is flowing from port_a to port_b";
              input SI.DynamicViscosity eta_b "Dynamic viscosity at port_b when fluid is flowing from port_b to port_a";
              input SI.AbsolutePressure dp_small "Small pressure drop used for regularization if m_flow=f(...,dp_small,..,dp)";
              input SI.MassFlowRate m_flow_small "Small mass flow rate used for regularization if dp=f_inv(...,m_flow_small,m_flow)";
              output SI.Pressure dp "Pressure loss";
            algorithm
              dp := Modelica.Fluid.Dissipation.PressureLoss.Orifice.dp_thickEdgedOverall_DP(Modelica.Fluid.Dissipation.PressureLoss.Orifice.dp_thickEdgedOverall_IN_con(A_0 = geometry.venaCrossArea, A_1 = geometry.crossArea, C_0 = geometry.venaPerimeter, C_1 = geometry.perimeter, L = geometry.venaLength, dp_smooth = dp_small), Modelica.Fluid.Dissipation.PressureLoss.Orifice.dp_thickEdgedOverall_IN_var(rho = Modelica.Fluid.Utilities.regStep(m_flow, d_a, d_b, m_flow_small), eta = Modelica.Fluid.Utilities.regStep(m_flow, eta_a, eta_b, m_flow_small)), m_flow);
              annotation(
                Inline = true,
                Documentation(info = "<html>
<p>
This function returns the pressure loss dp as function of mass flow rate m_flow for a thick edged orifice.
The details of the function are described
<a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.PressureLoss.Orifice.dp_thickEdgedOverall\">here</a>.
</p>

<p>
The orifice characteristic is valid for constant density and constant dynamic viscosity.
It can be approximately also used for compressible media. This is performed by providing
the upstream density and upstream dynamic viscosity. In order to be able to regularize density
and dynamic viscosity around zero mass flow rate, the two quantities have to be given if
fluid flows from port_a to port_b (d_a, eta_a) and if fluid flows from port_b to port_a
(d_b, eta_b).
</p>
</html>"));
            end pressureLoss;

            record Geometry "Geometric data for a thick edged orifice"
              extends Modelica.Icons.Record;
              SI.Area crossArea "Inner cross sectional area" annotation(
                Dialog);
              SI.Length perimeter "Inner perimeter" annotation(
                Dialog);
              SI.Area venaCrossArea "Cross sectional area of vena contraction" annotation(
                Dialog);
              SI.Length venaPerimeter "Perimeter of vena contraction" annotation(
                Dialog);
              SI.Length venaLength "Length of vena contraction" annotation(
                Dialog);
              annotation(
                Documentation(info = "<html>
<p>
This record is used to define the geometric (constant) data of a thick edged orifice.
The details of the record are described
<a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.PressureLoss.Orifice.dp_thickEdgedOverall\">here</a>.
</p>
</html>"));
            end Geometry;

            package Choices "Choices for Geometry"
              extends Modelica.Icons.Package;

              function circular "Circular cross section"
                import Modelica.Constants.pi;
                input SI.Diameter diameter "Inner diameter of circular orifice" annotation(
                  Dialog);
                input SI.Diameter venaDiameter "Diameter of vena contraction" annotation(
                  Dialog);
                input SI.Length venaLength "Length of vena contraction" annotation(
                  Dialog);
                output ThickEdgedOrifice.Geometry geometry "Geometry of circular thick edged orifice";
              algorithm
                geometry.crossArea := diameter ^ 2 * pi / 4;
                geometry.perimeter := pi * diameter;
                geometry.venaCrossArea := venaDiameter ^ 2 * pi / 4;
                geometry.venaPerimeter := pi * venaDiameter;
                geometry.venaLength := venaLength;
                annotation(
                  Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-80, 80}, {80, -80}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid)}),
                  Documentation(revisions = "", info = "<html>
<p>
Function that returns the ThickEdgedOrifice.Geometry for a circular
cross section of the orifice.
</p>
</html>"));
              end circular;

              function rectangular "Rectangular cross section"
                import Modelica.Constants.pi;
                input SI.Length width "Inner width of rectangular orifice" annotation(
                  Dialog);
                input SI.Length height "Inner height of rectangular orifice" annotation(
                  Dialog);
                input SI.Length venaWidth "Width of vena contraction" annotation(
                  Dialog);
                input SI.Length venaHeight "Height of vena contraction" annotation(
                  Dialog);
                input SI.Length venaLength "Length of vena contraction" annotation(
                  Dialog);
                output ThickEdgedOrifice.Geometry geometry "Geometry of circular thick edged orifice";
              algorithm
                geometry.crossArea := width * height;
                geometry.perimeter := 2 * width + 2 * height;
                geometry.venaCrossArea := venaWidth * venaHeight;
                geometry.venaPerimeter := 2 * venaWidth + 2 * venaHeight;
                geometry.venaLength := venaLength;
                annotation(
                  Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-80, 60}, {80, -60}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid)}),
                  Documentation(revisions = "", info = "<html>
<p>
Function that returns the ThickEdgedOrifice.Geometry for a rectangular
cross section of the orifice.
</p>
</html>"));
              end rectangular;

              function general "General cross section"
                import Modelica.Constants.pi;
                input SI.Area crossArea "Inner cross sectional area" annotation(
                  Dialog);
                input SI.Length perimeter "Inner perimeter" annotation(
                  Dialog);
                input SI.Area venaCrossArea "Cross sectional area of vena contraction" annotation(
                  Dialog);
                input SI.Length venaPerimeter "Perimeter of vena contraction" annotation(
                  Dialog);
                input SI.Length venaLength "Length of vena contraction" annotation(
                  Dialog);
                output ThickEdgedOrifice.Geometry geometry "Geometry of circular thick edged orifice";
              algorithm
                geometry.crossArea := crossArea;
                geometry.perimeter := perimeter;
                geometry.venaCrossArea := venaCrossArea;
                geometry.venaPerimeter := venaPerimeter;
                geometry.venaLength := venaLength;
                annotation(
                  Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{-80, 8}, {0, 80}, {80, 40}, {20, -20}, {40, -80}, {-60, -80}, {-80, 8}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid)}),
                  Documentation(revisions = "", info = "<html>
<p>
Function that returns the ThickEdgedOrifice.Geometry for a general
cross section of the orifice.
</p>
</html>"));
              end general;
              annotation(
                Documentation(info = "<html>
<p>
Choices to compute the \"Geometry\" record for a ThickEdgedOrifice.
</p>
</html>"));
            end Choices;
            annotation(
              Documentation(info = "<html>
<p>
This package contains utility functions and records
for the ThickEdgedOrifice fitting component.
</p>
</html>"));
          end ThickEdgedOrifice;
        end Orifices;

        package GenericResistances "Pressure loss functions for generic, geometry independent flow resistances"
          extends Modelica.Icons.VariantsPackage;

          package VolumeFlowRate "Pressure loss functions for generic resistances parameterized with the volume flow rate"
            extends Modelica.Icons.Package;

            function massFlowRate "Return mass flow rate m_flow as function of pressure loss dp for a curved bend"
              extends Modelica.Icons.Function;
              input SI.Pressure dp "Pressure loss";
              input Real a(unit = "(Pa.s2)/m6") "Coefficient for quadratic term (dp = a*V_flow^2 + b*V_flow)";
              input Real b(unit = "(Pa.s)/m3") "Coefficient for linear term (dp = a*V_flow^2 + b*V_flow)";
              input SI.Density d_a "Density at port_a when fluid is flowing from port_a to port_b";
              input SI.Density d_b "Density at port_b when fluid is flowing from port_b to port_a";
              input SI.AbsolutePressure dp_small "Small pressure drop used for regularization if m_flow=f(...,dp_small,..,dp)";
              input SI.MassFlowRate m_flow_small "Small mass flow rate used for regularization if dp=f_inv(...,m_flow_small,m_flow)";
              output SI.MassFlowRate m_flow "Mass flow rate (= port_a.m_flow)";
            algorithm
              m_flow := Modelica.Fluid.Dissipation.PressureLoss.General.dp_volumeFlowRate_MFLOW(Modelica.Fluid.Dissipation.PressureLoss.General.dp_volumeFlowRate_IN_con(a = a, b = b, dp_min = dp_small), Modelica.Fluid.Dissipation.PressureLoss.General.dp_volumeFlowRate_IN_var(rho = Modelica.Fluid.Utilities.regStep(dp, d_a, d_b, dp_small)), dp);
              annotation(
                LateInline = true,
                inverse(dp = Modelica.Fluid.Fittings.BaseClasses.GenericResistances.VolumeFlowRate.pressureLoss(m_flow, a, b, d_a, d_b, dp_small, m_flow_small)),
                Documentation(info = "<html>
<p>
This function returns the mass flow rate m_flow as function of pressure loss dp for an edged bend.
The details of the function are described
<a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.PressureLoss.General.dp_volumeFlowRate\">here</a>.
</p>

<p>
The bend characteristic is valid for constant density and constant dynamic viscosity.
It can be approximately also used for compressible media. This is performed by providing
the upstream density and upstream dynamic viscosity. In order to be able to regularize density
and dynamic viscosity around zero mass flow rate, the two quantities have to be given if
fluid flows from port_a to port_b (d_a, eta_a) and if fluid flows from port_b to port_a
(d_b, eta_b).
</p>

</html>"));
            end massFlowRate;

            function pressureLoss "Return pressure loss dp as function of mass flow rate m_flow for a curved bend"
              extends Modelica.Icons.Function;
              input SI.MassFlowRate m_flow "Mass flow rate (= port_a.m_flow)";
              input Real a(unit = "(Pa.s2)/m6") "Coefficient for quadratic term (dp = a*V_flow^2 + b*V_flow)";
              input Real b(unit = "(Pa.s)/m3") "Coefficient for linear term (dp = a*V_flow^2 + b*V_flow)";
              input SI.Density d_a "Density at port_a when fluid is flowing from port_a to port_b";
              input SI.Density d_b "Density at port_b when fluid is flowing from port_b to port_a";
              input SI.AbsolutePressure dp_small "Small pressure drop used for regularization if m_flow=f(...,dp_small,..,dp)";
              input SI.MassFlowRate m_flow_small "Small mass flow rate used for regularization if dp=f_inv(...,m_flow_small,m_flow)";
              output SI.Pressure dp "Pressure loss";
            algorithm
              dp := Modelica.Fluid.Dissipation.PressureLoss.General.dp_volumeFlowRate_DP(Modelica.Fluid.Dissipation.PressureLoss.General.dp_volumeFlowRate_IN_con(a = a, b = b, dp_min = dp_small), Modelica.Fluid.Dissipation.PressureLoss.General.dp_volumeFlowRate_IN_var(rho = Modelica.Fluid.Utilities.regStep(m_flow, d_a, d_b, m_flow_small)), m_flow);
              annotation(
                Inline = true,
                Documentation(info = "<html>
<p>
This function returns the pressure loss dp as function of mass flow rate m_flow for an edged bend.
The details of the function are described
<a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.PressureLoss.General.dp_volumeFlowRate\">here</a>.
</p>

<p>
The bend characteristic is valid for constant density and constant dynamic viscosity.
It can be approximately also used for compressible media. This is performed by providing
the upstream density and upstream dynamic viscosity. In order to be able to regularize density
and dynamic viscosity around zero mass flow rate, the two quantities have to be given if
fluid flows from port_a to port_b (d_a, eta_a) and if fluid flows from port_b to port_a
(d_b, eta_b).
</p>
</html>"));
            end pressureLoss;
            annotation(
              Documentation(info = "<html>
<p>
This package contains utility functions
for the VolumeFlowRate fitting component.
</p>
</html>"));
          end VolumeFlowRate;
        end GenericResistances;
      end BaseClasses;
      annotation(
        Documentation(info = "<html>
<p>
This sublibrary contains models and functions providing pressure
loss correlations. All models in this library have the property
that no mass and no energy is stored in the component. Therefore,
none of the models have a state.
</p>

<p>
All functions are continuous and have a finite, non-zero, smooth, first derivative.
The functions are all guaranteed to be strict monotonically increasing.
The mentioned properties guarantee that a unique inverse of every
function exists. Note, the usual quadratic pressure loss correlation
</p>

<ul>
<li> in the form m_flow = f(dp) has an infinite derivative at zero
     mass flow rate and is therefore problematic to use.</li>
<li> in the form dp = f(m_flow) has a zero derivative at zero mass flow rate
     and is therefore problematic to invert, since the inverse function has
     then an infinite derivative at zero mass flow rate.</li>
</ul>
<p>
The two mentioned problems are solved in this package by approximating
the characteristics around zero mass flow rates with appropriate
polynomials. The monotonicity is guaranteed using results from:
</p>

<dl>
<dt> Fritsch F.N. and Carlson R.E. (1980):</dt>
<dd> <b>Monotone piecewise cubic interpolation</b>.
     SIAM J. Numerc. Anal., Vol. 17, No. 2, April 1980, pp. 238-246</dd>
</dl>

</html>", revisions = "<html>
<ul>
<li><i>Jan. 3, 2006</i>
    by <a href=\"mailto:Martin.Otter@DLR.de\">Martin Otter</a>:<br>
    New design and implementation based on previous iterations.</li>
</ul>
</html>"));
    end Fittings;

    package Sources "Define fixed or prescribed boundary conditions"
      extends Modelica.Icons.SourcesPackage;

      model FixedBoundary "Boundary source component"
        import Modelica.Media.Interfaces.Choices.IndependentVariables;
        extends Sources.BaseClasses.PartialSource;
        parameter Boolean use_p = true "select p or d" annotation(
          Evaluate = true,
          Dialog(group = "Boundary pressure or Boundary density"));
        parameter Medium.AbsolutePressure p = Medium.p_default "Boundary pressure" annotation(
          Dialog(group = "Boundary pressure or Boundary density", enable = use_p));
        parameter Medium.Density d = if use_T then Medium.density_pTX(Medium.p_default, Medium.T_default, Medium.X_default) else Medium.density_phX(Medium.p_default, Medium.h_default, Medium.X_default) "Boundary density" annotation(
          Dialog(group = "Boundary pressure or Boundary density", enable = not use_p));
        parameter Boolean use_T = true "select T or h" annotation(
          Evaluate = true,
          Dialog(group = "Boundary temperature or Boundary specific enthalpy"));
        parameter Medium.Temperature T = Medium.T_default "Boundary temperature" annotation(
          Dialog(group = "Boundary temperature or Boundary specific enthalpy", enable = use_T));
        parameter Medium.SpecificEnthalpy h = Medium.h_default "Boundary specific enthalpy" annotation(
          Dialog(group = "Boundary temperature or Boundary specific enthalpy", enable = not use_T));
        parameter Medium.MassFraction X[Medium.nX](quantity = Medium.substanceNames) = Medium.X_default "Boundary mass fractions m_i/m" annotation(
          Dialog(group = "Only for multi-substance flow", enable = Medium.nXi > 0));
        parameter Medium.ExtraProperty C[Medium.nC](quantity = Medium.extraPropertiesNames) = fill(0, Medium.nC) "Boundary trace substances" annotation(
          Dialog(group = "Only for trace-substance flow", enable = Medium.nC > 0));
      protected
        Medium.ThermodynamicState state;
      equation
        Modelica.Fluid.Utilities.checkBoundary(Medium.mediumName, Medium.substanceNames, Medium.singleState, use_p, X, "FixedBoundary");
        if use_p or Medium.singleState then
// p given
          if use_T then
// p,T,X given
            state = Medium.setState_pTX(p, T, X);
          else
// p,h,X given
            state = Medium.setState_phX(p, h, X);
          end if;
          if Medium.ThermoStates == IndependentVariables.dTX then
            medium.d = Medium.density(state);
          else
            medium.p = Medium.pressure(state);
          end if;
          if Medium.ThermoStates == IndependentVariables.ph or Medium.ThermoStates == IndependentVariables.phX then
            medium.h = Medium.specificEnthalpy(state);
          else
            medium.T = Medium.temperature(state);
          end if;
        else
// d given
          if use_T then
// d,T,X given
            state = Medium.setState_dTX(d, T, X);
            if Medium.ThermoStates == IndependentVariables.dTX then
              medium.d = Medium.density(state);
            else
              medium.p = Medium.pressure(state);
            end if;
            if Medium.ThermoStates == IndependentVariables.ph or Medium.ThermoStates == IndependentVariables.phX then
              medium.h = Medium.specificEnthalpy(state);
            else
              medium.T = Medium.temperature(state);
            end if;
          else
// d,h,X given
            medium.d = d;
            medium.h = h;
            state = Medium.setState_dTX(d, T, X);
          end if;
        end if;
        medium.Xi = X[1:Medium.nXi];
        ports.C_outflow = fill(C, nPorts);
        annotation(
          defaultComponentName = "boundary",
          Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-100, 100}, {100, -100}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.Sphere, fillColor = {0, 127, 255}), Text(extent = {{-150, 110}, {150, 150}}, textString = "%name", lineColor = {0, 0, 255})}),
          Documentation(info = "<html>
<p>
Model <b>FixedBoundary</b> defines constant values for boundary conditions:
</p>
<ul>
<li> Boundary pressure or boundary density.</li>
<li> Boundary temperature or boundary specific enthalpy.</li>
<li> Boundary composition (only for multi-substance or trace-substance flow).</li>
</ul>
<p>
Note, that boundary temperature, density, specific enthalpy,
mass fractions and trace substances have only an effect if the mass flow
is from the Boundary into the port. If mass is flowing from
the port into the boundary, the boundary definitions,
with exception of boundary pressure, do not have an effect.
</p>
</html>"));
      end FixedBoundary;

      model Boundary_pT "Boundary with prescribed pressure, temperature, composition and trace substances"
        import Modelica.Media.Interfaces.Choices.IndependentVariables;
        extends Sources.BaseClasses.PartialSource;
        parameter Boolean use_p_in = false "Get the pressure from the input connector" annotation(
          Evaluate = true,
          HideResult = true,
          choices(checkBox = true));
        parameter Boolean use_T_in = false "Get the temperature from the input connector" annotation(
          Evaluate = true,
          HideResult = true,
          choices(checkBox = true));
        parameter Boolean use_X_in = false "Get the composition from the input connector" annotation(
          Evaluate = true,
          HideResult = true,
          choices(checkBox = true));
        parameter Boolean use_C_in = false "Get the trace substances from the input connector" annotation(
          Evaluate = true,
          HideResult = true,
          choices(checkBox = true));
        parameter Medium.AbsolutePressure p = Medium.p_default "Fixed value of pressure" annotation(
          Evaluate = true,
          Dialog(enable = not use_p_in));
        parameter Medium.Temperature T = Medium.T_default "Fixed value of temperature" annotation(
          Evaluate = true,
          Dialog(enable = not use_T_in));
        parameter Medium.MassFraction X[Medium.nX] = Medium.X_default "Fixed value of composition" annotation(
          Evaluate = true,
          Dialog(enable = not use_X_in and Medium.nXi > 0));
        parameter Medium.ExtraProperty C[Medium.nC](quantity = Medium.extraPropertiesNames) = fill(0, Medium.nC) "Fixed values of trace substances" annotation(
          Evaluate = true,
          Dialog(enable = not use_C_in and Medium.nC > 0));
        Modelica.Blocks.Interfaces.RealInput p_in if use_p_in "Prescribed boundary pressure" annotation(
          Placement(transformation(extent = {{-140, 60}, {-100, 100}})));
        Modelica.Blocks.Interfaces.RealInput T_in if use_T_in "Prescribed boundary temperature" annotation(
          Placement(transformation(extent = {{-140, 20}, {-100, 60}})));
        Modelica.Blocks.Interfaces.RealInput X_in[Medium.nX] if use_X_in "Prescribed boundary composition" annotation(
          Placement(transformation(extent = {{-140, -60}, {-100, -20}})));
        Modelica.Blocks.Interfaces.RealInput C_in[Medium.nC] if use_C_in "Prescribed boundary trace substances" annotation(
          Placement(transformation(extent = {{-140, -100}, {-100, -60}})));
      protected
        Modelica.Blocks.Interfaces.RealInput p_in_internal "Needed to connect to conditional connector";
        Modelica.Blocks.Interfaces.RealInput T_in_internal "Needed to connect to conditional connector";
        Modelica.Blocks.Interfaces.RealInput X_in_internal[Medium.nX] "Needed to connect to conditional connector";
        Modelica.Blocks.Interfaces.RealInput C_in_internal[Medium.nC] "Needed to connect to conditional connector";
      equation
        Modelica.Fluid.Utilities.checkBoundary(Medium.mediumName, Medium.substanceNames, Medium.singleState, true, X_in_internal, "Boundary_pT");
        connect(p_in, p_in_internal);
        connect(T_in, T_in_internal);
        connect(X_in, X_in_internal);
        connect(C_in, C_in_internal);
        if not use_p_in then
          p_in_internal = p;
        end if;
        if not use_T_in then
          T_in_internal = T;
        end if;
        if not use_X_in then
          X_in_internal = X;
        end if;
        if not use_C_in then
          C_in_internal = C;
        end if;
        medium.p = p_in_internal;
        if Medium.ThermoStates == IndependentVariables.ph or Medium.ThermoStates == IndependentVariables.phX then
          medium.h = Medium.specificEnthalpy(Medium.setState_pTX(p_in_internal, T_in_internal, X_in_internal));
        else
          medium.T = T_in_internal;
        end if;
        medium.Xi = X_in_internal[1:Medium.nXi];
        ports.C_outflow = fill(C_in_internal, nPorts);
        annotation(
          defaultComponentName = "boundary",
          Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-100, 100}, {100, -100}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.Sphere, fillColor = {0, 127, 255}), Text(extent = {{-150, 120}, {150, 160}}, textString = "%name", lineColor = {0, 0, 255}), Line(visible = use_p_in, points = {{-100, 80}, {-58, 80}}, color = {0, 0, 255}), Line(visible = use_T_in, points = {{-100, 40}, {-92, 40}}, color = {0, 0, 255}), Line(visible = use_X_in, points = {{-100, -40}, {-92, -40}}, color = {0, 0, 255}), Line(visible = use_C_in, points = {{-100, -80}, {-60, -80}}, color = {0, 0, 255}), Text(visible = use_p_in, extent = {{-152, 134}, {-68, 94}}, lineColor = {0, 0, 0}, textString = "p"), Text(visible = use_X_in, extent = {{-164, 4}, {-62, -36}}, lineColor = {0, 0, 0}, textString = "X"), Text(visible = use_C_in, extent = {{-164, -90}, {-62, -130}}, lineColor = {0, 0, 0}, textString = "C"), Text(visible = use_T_in, extent = {{-162, 34}, {-60, -6}}, lineColor = {0, 0, 0}, textString = "T")}),
          Documentation(info = "<html>
<p>
Defines prescribed values for boundary conditions:
</p>
<ul>
<li> Prescribed boundary pressure.</li>
<li> Prescribed boundary temperature.</li>
<li> Boundary composition (only for multi-substance or trace-substance flow).</li>
</ul>
<p>If <code>use_p_in</code> is false (default option), the <code>p</code> parameter
is used as boundary pressure, and the <code>p_in</code> input connector is disabled; if <code>use_p_in</code> is true, then the <code>p</code> parameter is ignored, and the value provided by the input connector is used instead.</p>
<p>The same thing goes for the temperature, composition and trace substances.</p>
<p>
Note, that boundary temperature,
mass fractions and trace substances have only an effect if the mass flow
is from the boundary into the port. If mass is flowing from
the port into the boundary, the boundary definitions,
with exception of boundary pressure, do not have an effect.
</p>
</html>"));
      end Boundary_pT;

      model Boundary_ph "Boundary with prescribed pressure, specific enthalpy, composition and trace substances"
        import Modelica.Media.Interfaces.Choices.IndependentVariables;
        extends Sources.BaseClasses.PartialSource;
        parameter Boolean use_p_in = false "Get the pressure from the input connector" annotation(
          Evaluate = true,
          HideResult = true,
          choices(checkBox = true));
        parameter Boolean use_h_in = false "Get the specific enthalpy from the input connector" annotation(
          Evaluate = true,
          HideResult = true,
          choices(checkBox = true));
        parameter Boolean use_X_in = false "Get the composition from the input connector" annotation(
          Evaluate = true,
          HideResult = true,
          choices(checkBox = true));
        parameter Boolean use_C_in = false "Get the trace substances from the input connector" annotation(
          Evaluate = true,
          HideResult = true,
          choices(checkBox = true));
        parameter Medium.AbsolutePressure p = Medium.p_default "Fixed value of pressure" annotation(
          Evaluate = true,
          Dialog(enable = not use_p_in));
        parameter Medium.SpecificEnthalpy h = Medium.h_default "Fixed value of specific enthalpy" annotation(
          Evaluate = true,
          Dialog(enable = not use_h_in));
        parameter Medium.MassFraction X[Medium.nX] = Medium.X_default "Fixed value of composition" annotation(
          Evaluate = true,
          Dialog(enable = not use_X_in and Medium.nXi > 0));
        parameter Medium.ExtraProperty C[Medium.nC](quantity = Medium.extraPropertiesNames) = fill(0, Medium.nC) "Fixed values of trace substances" annotation(
          Evaluate = true,
          Dialog(enable = not use_C_in and Medium.nC > 0));
        Modelica.Blocks.Interfaces.RealInput p_in if use_p_in "Prescribed boundary pressure" annotation(
          Placement(transformation(extent = {{-140, 60}, {-100, 100}})));
        Modelica.Blocks.Interfaces.RealInput h_in if use_h_in "Prescribed boundary specific enthalpy" annotation(
          Placement(transformation(extent = {{-140, 20}, {-100, 60}})));
        Modelica.Blocks.Interfaces.RealInput X_in[Medium.nX] if use_X_in "Prescribed boundary composition" annotation(
          Placement(transformation(extent = {{-140, -60}, {-100, -20}})));
        Modelica.Blocks.Interfaces.RealInput C_in[Medium.nC] if use_C_in "Prescribed boundary trace substances" annotation(
          Placement(transformation(extent = {{-140, -100}, {-100, -60}})));
      protected
        Modelica.Blocks.Interfaces.RealInput p_in_internal "Needed to connect to conditional connector";
        Modelica.Blocks.Interfaces.RealInput h_in_internal "Needed to connect to conditional connector";
        Modelica.Blocks.Interfaces.RealInput X_in_internal[Medium.nX] "Needed to connect to conditional connector";
        Modelica.Blocks.Interfaces.RealInput C_in_internal[Medium.nC] "Needed to connect to conditional connector";
      equation
        Modelica.Fluid.Utilities.checkBoundary(Medium.mediumName, Medium.substanceNames, Medium.singleState, true, X_in_internal, "Boundary_ph");
        connect(p_in, p_in_internal);
        connect(h_in, h_in_internal);
        connect(X_in, X_in_internal);
        connect(C_in, C_in_internal);
        if not use_p_in then
          p_in_internal = p;
        end if;
        if not use_h_in then
          h_in_internal = h;
        end if;
        if not use_X_in then
          X_in_internal = X;
        end if;
        if not use_C_in then
          C_in_internal = C;
        end if;
        medium.p = p_in_internal;
        if Medium.ThermoStates == IndependentVariables.ph or Medium.ThermoStates == IndependentVariables.phX then
          medium.h = h_in_internal;
        else
          medium.T = Medium.temperature(Medium.setState_phX(p_in_internal, h_in_internal, X_in_internal));
        end if;
        medium.Xi = X_in_internal[1:Medium.nXi];
        ports.C_outflow = fill(C_in_internal, nPorts);
        annotation(
          defaultComponentName = "boundary",
          Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-100, 100}, {100, -100}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.Sphere, fillColor = {0, 127, 255}), Text(extent = {{-150, 110}, {150, 150}}, textString = "%name", lineColor = {0, 0, 255}), Line(visible = use_p_in, points = {{-100, 80}, {-60, 80}}, color = {0, 0, 255}), Line(visible = use_h_in, points = {{-100, 40}, {-92, 40}}, color = {0, 0, 255}), Line(visible = use_X_in, points = {{-100, -40}, {-92, -40}}, color = {0, 0, 255}), Line(visible = use_C_in, points = {{-100, -80}, {-60, -80}}, color = {0, 0, 255}), Text(visible = use_p_in, extent = {{-150, 134}, {-72, 94}}, lineColor = {0, 0, 0}, textString = "p"), Text(visible = use_h_in, extent = {{-166, 34}, {-64, -6}}, lineColor = {0, 0, 0}, textString = "h"), Text(visible = use_X_in, extent = {{-164, 4}, {-62, -36}}, lineColor = {0, 0, 0}, textString = "X"), Text(visible = use_C_in, extent = {{-164, -90}, {-62, -130}}, lineColor = {0, 0, 0}, textString = "C")}),
          Documentation(info = "<html>
<p>
Defines prescribed values for boundary conditions:
</p>
<ul>
<li> Prescribed boundary pressure.</li>
<li> Prescribed boundary temperature.</li>
<li> Boundary composition (only for multi-substance or trace-substance flow).</li>
</ul>
<p>If <code>use_p_in</code> is false (default option), the <code>p</code> parameter
is used as boundary pressure, and the <code>p_in</code> input connector is disabled; if <code>use_p_in</code> is true, then the <code>p</code> parameter is ignored, and the value provided by the input connector is used instead.</p>
<p>The same thing goes for the specific enthalpy and composition</p>
<p>
Note, that boundary temperature,
mass fractions and trace substances have only an effect if the mass flow
is from the boundary into the port. If mass is flowing from
the port into the boundary, the boundary definitions,
with exception of boundary pressure, do not have an effect.
</p>
</html>"));
      end Boundary_ph;

      model MassFlowSource_T "Ideal flow source that produces a prescribed mass flow with prescribed temperature, mass fraction and trace substances"
        import Modelica.Media.Interfaces.Choices.IndependentVariables;
        extends Sources.BaseClasses.PartialFlowSource;
        parameter Boolean use_m_flow_in = false "Get the mass flow rate from the input connector" annotation(
          Evaluate = true,
          HideResult = true,
          choices(checkBox = true));
        parameter Boolean use_T_in = false "Get the temperature from the input connector" annotation(
          Evaluate = true,
          HideResult = true,
          choices(checkBox = true));
        parameter Boolean use_X_in = false "Get the composition from the input connector" annotation(
          Evaluate = true,
          HideResult = true,
          choices(checkBox = true));
        parameter Boolean use_C_in = false "Get the trace substances from the input connector" annotation(
          Evaluate = true,
          HideResult = true,
          choices(checkBox = true));
        parameter Medium.MassFlowRate m_flow = 0 "Fixed mass flow rate going out of the fluid port" annotation(
          Evaluate = true,
          Dialog(enable = not use_m_flow_in));
        parameter Medium.Temperature T = Medium.T_default "Fixed value of temperature" annotation(
          Evaluate = true,
          Dialog(enable = not use_T_in));
        parameter Medium.MassFraction X[Medium.nX] = Medium.X_default "Fixed value of composition" annotation(
          Evaluate = true,
          Dialog(enable = not use_X_in and Medium.nXi > 0));
        parameter Medium.ExtraProperty C[Medium.nC](quantity = Medium.extraPropertiesNames) = fill(0, Medium.nC) "Fixed values of trace substances" annotation(
          Evaluate = true,
          Dialog(enable = not use_C_in and Medium.nC > 0));
        Modelica.Blocks.Interfaces.RealInput m_flow_in if use_m_flow_in "Prescribed mass flow rate" annotation(
          Placement(transformation(extent = {{-120, 60}, {-80, 100}}), iconTransformation(extent = {{-120, 60}, {-80, 100}})));
        Modelica.Blocks.Interfaces.RealInput T_in if use_T_in "Prescribed fluid temperature" annotation(
          Placement(transformation(extent = {{-140, 20}, {-100, 60}}), iconTransformation(extent = {{-140, 20}, {-100, 60}})));
        Modelica.Blocks.Interfaces.RealInput X_in[Medium.nX] if use_X_in "Prescribed fluid composition" annotation(
          Placement(transformation(extent = {{-140, -60}, {-100, -20}})));
        Modelica.Blocks.Interfaces.RealInput C_in[Medium.nC] if use_C_in "Prescribed boundary trace substances" annotation(
          Placement(transformation(extent = {{-120, -100}, {-80, -60}})));
      protected
        Modelica.Blocks.Interfaces.RealInput m_flow_in_internal "Needed to connect to conditional connector";
        Modelica.Blocks.Interfaces.RealInput T_in_internal "Needed to connect to conditional connector";
        Modelica.Blocks.Interfaces.RealInput X_in_internal[Medium.nX] "Needed to connect to conditional connector";
        Modelica.Blocks.Interfaces.RealInput C_in_internal[Medium.nC] "Needed to connect to conditional connector";
      equation
        Utilities.checkBoundary(Medium.mediumName, Medium.substanceNames, Medium.singleState, true, X_in_internal, "MassFlowSource_T");
        connect(m_flow_in, m_flow_in_internal);
        connect(T_in, T_in_internal);
        connect(X_in, X_in_internal);
        connect(C_in, C_in_internal);
        if not use_m_flow_in then
          m_flow_in_internal = m_flow;
        end if;
        if not use_T_in then
          T_in_internal = T;
        end if;
        if not use_X_in then
          X_in_internal = X;
        end if;
        if not use_C_in then
          C_in_internal = C;
        end if;
        if Medium.ThermoStates == IndependentVariables.ph or Medium.ThermoStates == IndependentVariables.phX then
          medium.h = Medium.specificEnthalpy(Medium.setState_pTX(medium.p, T_in_internal, X_in_internal));
        else
          medium.T = T_in_internal;
        end if;
        sum(ports.m_flow) = -m_flow_in_internal;
        medium.Xi = X_in_internal[1:Medium.nXi];
        ports.C_outflow = fill(C_in_internal, nPorts);
        annotation(
          defaultComponentName = "boundary",
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{35, 45}, {100, -45}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {0, 127, 255}), Ellipse(extent = {{-100, 80}, {60, -80}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Polygon(points = {{-60, 70}, {60, 0}, {-60, -68}, {-60, 70}}, lineColor = {0, 0, 255}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-54, 32}, {16, -30}}, lineColor = {255, 0, 0}, textString = "m"), Text(extent = {{-150, 130}, {150, 170}}, textString = "%name", lineColor = {0, 0, 255}), Ellipse(extent = {{-26, 30}, {-18, 22}}, lineColor = {255, 0, 0}, fillColor = {255, 0, 0}, fillPattern = FillPattern.Solid), Text(visible = use_m_flow_in, extent = {{-185, 132}, {-45, 100}}, lineColor = {0, 0, 0}, textString = "m_flow"), Text(visible = use_T_in, extent = {{-111, 71}, {-71, 37}}, lineColor = {0, 0, 0}, textString = "T"), Text(visible = use_X_in, extent = {{-153, -44}, {-33, -72}}, lineColor = {0, 0, 0}, textString = "X"), Text(visible = use_C_in, extent = {{-155, -98}, {-35, -126}}, lineColor = {0, 0, 0}, textString = "C")}),
          Documentation(info = "<html>
<p>
Models an ideal flow source, with prescribed values of flow rate, temperature, composition and trace substances:
</p>
<ul>
<li> Prescribed mass flow rate.</li>
<li> Prescribed temperature.</li>
<li> Boundary composition (only for multi-substance or trace-substance flow).</li>
</ul>
<p>If <code>use_m_flow_in</code> is false (default option), the <code>m_flow</code> parameter
is used as boundary pressure, and the <code>m_flow_in</code> input connector is disabled; if <code>use_m_flow_in</code> is true, then the <code>m_flow</code> parameter is ignored, and the value provided by the input connector is used instead.</p>
<p>The same thing goes for the temperature and composition</p>
<p>
Note, that boundary temperature,
mass fractions and trace substances have only an effect if the mass flow
is from the boundary into the port. If mass is flowing from
the port into the boundary, the boundary definitions,
with exception of boundary flow rate, do not have an effect.
</p>
</html>"));
      end MassFlowSource_T;

      model MassFlowSource_h "Ideal flow source that produces a prescribed mass flow with prescribed specific enthalpy, mass fraction and trace substances"
        import Modelica.Media.Interfaces.Choices.IndependentVariables;
        extends Sources.BaseClasses.PartialFlowSource;
        parameter Boolean use_m_flow_in = false "Get the mass flow rate from the input connector" annotation(
          Evaluate = true,
          HideResult = true,
          choices(checkBox = true));
        parameter Boolean use_h_in = false "Get the specific enthalpy from the input connector" annotation(
          Evaluate = true,
          HideResult = true,
          choices(checkBox = true));
        parameter Boolean use_X_in = false "Get the composition from the input connector" annotation(
          Evaluate = true,
          HideResult = true,
          choices(checkBox = true));
        parameter Boolean use_C_in = false "Get the trace substances from the input connector" annotation(
          Evaluate = true,
          HideResult = true,
          choices(checkBox = true));
        parameter Medium.MassFlowRate m_flow = 0 "Fixed mass flow rate going out of the fluid port" annotation(
          Evaluate = true,
          Dialog(enable = not use_m_flow_in));
        parameter Medium.SpecificEnthalpy h = Medium.h_default "Fixed value of specific enthalpy" annotation(
          Evaluate = true,
          Dialog(enable = not use_h_in));
        parameter Medium.MassFraction X[Medium.nX] = Medium.X_default "Fixed value of composition" annotation(
          Evaluate = true,
          Dialog(enable = not use_X_in and Medium.nXi > 0));
        parameter Medium.ExtraProperty C[Medium.nC](quantity = Medium.extraPropertiesNames) = fill(0, Medium.nC) "Fixed values of trace substances" annotation(
          Evaluate = true,
          Dialog(enable = not use_C_in and Medium.nC > 0));
        Modelica.Blocks.Interfaces.RealInput m_flow_in if use_m_flow_in "Prescribed mass flow rate" annotation(
          Placement(transformation(extent = {{-120, 60}, {-80, 100}})));
        Modelica.Blocks.Interfaces.RealInput h_in if use_h_in "Prescribed fluid specific enthalpy" annotation(
          Placement(transformation(extent = {{-140, 20}, {-100, 60}}), iconTransformation(extent = {{-140, 20}, {-100, 60}})));
        Modelica.Blocks.Interfaces.RealInput X_in[Medium.nX] if use_X_in "Prescribed fluid composition" annotation(
          Placement(transformation(extent = {{-140, -60}, {-100, -20}})));
        Modelica.Blocks.Interfaces.RealInput C_in[Medium.nC] if use_C_in "Prescribed boundary trace substances" annotation(
          Placement(transformation(extent = {{-120, -100}, {-80, -60}}), iconTransformation(extent = {{-120, -100}, {-80, -60}})));
      protected
        Modelica.Blocks.Interfaces.RealInput m_flow_in_internal "Needed to connect to conditional connector";
        Modelica.Blocks.Interfaces.RealInput h_in_internal "Needed to connect to conditional connector";
        Modelica.Blocks.Interfaces.RealInput X_in_internal[Medium.nX] "Needed to connect to conditional connector";
        Modelica.Blocks.Interfaces.RealInput C_in_internal[Medium.nC] "Needed to connect to conditional connector";
      equation
        Utilities.checkBoundary(Medium.mediumName, Medium.substanceNames, Medium.singleState, true, X_in_internal, "MassFlowSource_h");
        connect(m_flow_in, m_flow_in_internal);
        connect(h_in, h_in_internal);
        connect(X_in, X_in_internal);
        connect(C_in, C_in_internal);
        if not use_m_flow_in then
          m_flow_in_internal = m_flow;
        end if;
        if not use_h_in then
          h_in_internal = h;
        end if;
        if not use_X_in then
          X_in_internal = X;
        end if;
        if not use_C_in then
          C_in_internal = C;
        end if;
        if Medium.ThermoStates == IndependentVariables.ph or Medium.ThermoStates == IndependentVariables.phX then
          medium.h = h_in_internal;
        else
          medium.T = Medium.temperature(Medium.setState_phX(medium.p, h_in_internal, X_in_internal));
        end if;
        sum(ports.m_flow) = -m_flow_in_internal;
        medium.Xi = X_in_internal[1:Medium.nXi];
        ports.C_outflow = fill(C_in_internal, nPorts);
        annotation(
          defaultComponentName = "boundary",
          Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{36, 45}, {100, -45}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {0, 127, 255}), Ellipse(extent = {{-100, 80}, {60, -80}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Polygon(points = {{-60, 70}, {60, 0}, {-60, -68}, {-60, 70}}, lineColor = {0, 0, 255}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-54, 32}, {16, -30}}, lineColor = {255, 0, 0}, textString = "m"), Ellipse(extent = {{-26, 30}, {-18, 22}}, lineColor = {255, 0, 0}, fillColor = {255, 0, 0}, fillPattern = FillPattern.Solid), Text(visible = use_m_flow_in, extent = {{-185, 132}, {-45, 100}}, lineColor = {0, 0, 0}, textString = "m_flow"), Text(visible = use_h_in, extent = {{-113, 72}, {-73, 38}}, lineColor = {0, 0, 0}, textString = "h"), Text(visible = use_X_in, extent = {{-153, -44}, {-33, -72}}, lineColor = {0, 0, 0}, textString = "X"), Text(visible = use_X_in, extent = {{-155, -98}, {-35, -126}}, lineColor = {0, 0, 0}, textString = "C"), Text(extent = {{-150, 110}, {150, 150}}, textString = "%name", lineColor = {0, 0, 255})}),
          Documentation(info = "<html>
<p>
Models an ideal flow source, with prescribed values of flow rate, temperature and composition:
</p>
<ul>
<li> Prescribed mass flow rate.</li>
<li> Prescribed specific enthalpy.</li>
<li> Boundary composition (only for multi-substance or trace-substance flow).</li>
</ul>
<p>If <code>use_m_flow_in</code> is false (default option), the <code>m_flow</code> parameter
is used as boundary pressure, and the <code>m_flow_in</code> input connector is disabled; if <code>use_m_flow_in</code> is true, then the <code>m_flow</code> parameter is ignored, and the value provided by the input connector is used instead.</p>
<p>The same thing goes for the temperature and composition</p>
<p>
Note, that boundary temperature,
mass fractions and trace substances have only an effect if the mass flow
is from the boundary into the port. If mass is flowing from
the port into the boundary, the boundary definitions,
with exception of boundary flow rate, do not have an effect.
</p>
</html>"));
      end MassFlowSource_h;

      package BaseClasses "Base classes used in the Sources package (only of interest to build new component models)"
        extends Modelica.Icons.BasesPackage;

        partial model PartialSource "Partial component source with one fluid connector"
          import Modelica.Constants;
          parameter Integer nPorts = 0 "Number of ports" annotation(
            Dialog(connectorSizing = true));
          replaceable package Medium = Modelica.Media.Interfaces.PartialMedium "Medium model within the source" annotation(
            choicesAllMatching = true);
          Medium.BaseProperties medium "Medium in the source";
          Interfaces.FluidPorts_b ports[nPorts](redeclare each package Medium = Medium, m_flow(each max = if flowDirection == Types.PortFlowDirection.Leaving then 0 else +Constants.inf, each min = if flowDirection == Types.PortFlowDirection.Entering then 0 else -Constants.inf)) annotation(
            Placement(transformation(extent = {{90, 40}, {110, -40}})));
        protected
          parameter Types.PortFlowDirection flowDirection = Types.PortFlowDirection.Bidirectional "Allowed flow direction" annotation(
            Evaluate = true,
            Dialog(tab = "Advanced"));
        equation
// Only one connection allowed to a port to avoid unwanted ideal mixing
          for i in 1:nPorts loop
            assert(cardinality(ports[i]) <= 1, "
each ports[i] of boundary shall at most be connected to one component.
If two or more connections are present, ideal mixing takes
place with these connections, which is usually not the intention
of the modeller. Increase nPorts to add an additional port.
            ");
            ports[i].p = medium.p;
            ports[i].h_outflow = medium.h;
            ports[i].Xi_outflow = medium.Xi;
          end for;
          annotation(
            defaultComponentName = "boundary",
            Documentation(info = "<html>
<p>
Partial component to model the <b>volume interface</b> of a <b>source</b>
component, such as a mass flow source. The essential
features are:
</p>
<ul>
<li> The pressure in the connection port (= ports.p) is identical to the
     pressure in the volume.</li>
<li> The outflow enthalpy rate (= port.h_outflow) and the composition of the
     substances (= port.Xi_outflow) are identical to the respective values in the volume.</li>
</ul>
</html>"));
        end PartialSource;

        partial model PartialFlowSource "Partial component source with one fluid connector"
          import Modelica.Constants;
          parameter Integer nPorts = 0 "Number of ports" annotation(
            Dialog(connectorSizing = true));
          replaceable package Medium = Modelica.Media.Interfaces.PartialMedium "Medium model within the source" annotation(
            choicesAllMatching = true);
          Medium.BaseProperties medium "Medium in the source";
          Interfaces.FluidPort_b ports[nPorts](redeclare each package Medium = Medium, m_flow(each max = if flowDirection == Types.PortFlowDirection.Leaving then 0 else +Constants.inf, each min = if flowDirection == Types.PortFlowDirection.Entering then 0 else -Constants.inf)) annotation(
            Placement(transformation(extent = {{90, 10}, {110, -10}})));
        protected
          parameter Types.PortFlowDirection flowDirection = Types.PortFlowDirection.Bidirectional "Allowed flow direction" annotation(
            Evaluate = true,
            Dialog(tab = "Advanced"));
        equation
          assert(abs(sum(abs(ports.m_flow)) - max(abs(ports.m_flow))) <= Modelica.Constants.small, "FlowSource only supports one connection with flow");
// Only one connection allowed to a port to avoid unwanted ideal mixing
          for i in 1:nPorts loop
            assert(cardinality(ports[i]) <= 1, "
each ports[i] of boundary shall at most be connected to one component.
If two or more connections are present, ideal mixing takes
place with these connections, which is usually not the intention
of the modeller. Increase nPorts to add an additional port.
            ");
            ports[i].p = medium.p;
            ports[i].h_outflow = medium.h;
            ports[i].Xi_outflow = medium.Xi;
          end for;
          annotation(
            defaultComponentName = "boundary",
            Documentation(info = "<html>
<p>
Partial component to model the <b>volume interface</b> of a <b>source</b>
component, such as a mass flow source. The essential
features are:
</p>
<ul>
<li> The pressure in the connection port (= ports.p) is identical to the
     pressure in the volume.</li>
<li> The outflow enthalpy rate (= port.h_outflow) and the composition of the
     substances (= port.Xi_outflow) are identical to the respective values in the volume.</li>
</ul>
</html>"));
        end PartialFlowSource;
      end BaseClasses;
      annotation(
        Documentation(info = "<html>
<p>
Package <b>Sources</b> contains generic sources for fluid connectors
to define fixed or prescribed ambient conditions.
</p>
</html>"));
    end Sources;

    package Sensors "Ideal sensor components to extract signals from a fluid connector"
      extends Modelica.Icons.SensorsPackage;

      model Pressure "Ideal pressure sensor"
        extends Sensors.BaseClasses.PartialAbsoluteSensor;
        extends Modelica.Icons.RotationalSensor;
        Modelica.Blocks.Interfaces.RealOutput p(final quantity = "Pressure", final unit = "Pa", displayUnit = "bar", min = 0) "Pressure at port" annotation(
          Placement(transformation(extent = {{100, -10}, {120, 10}})));
      equation
        p = port.p;
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{70, 0}, {100, 0}}, color = {0, 0, 127}), Line(points = {{0, -70}, {0, -100}}, color = {0, 127, 255}), Text(extent = {{-150, 80}, {150, 120}}, textString = "%name", lineColor = {0, 0, 255}), Text(extent = {{151, -20}, {57, -50}}, lineColor = {0, 0, 0}, textString = "p")}),
          Documentation(info = "<html>
<p>
This component monitors the absolute pressure at its fluid port. The sensor is
ideal, i.e., it does not influence the fluid.
</p>
</html>"));
      end Pressure;

      model Density "Ideal one port density sensor"
        extends Sensors.BaseClasses.PartialAbsoluteSensor;
        extends Modelica.Icons.RotationalSensor;
        Modelica.Blocks.Interfaces.RealOutput d(final quantity = "Density", final unit = "kg/m3", displayUnit = "g/cm3", min = 0) "Density in port medium" annotation(
          Placement(transformation(extent = {{100, -10}, {120, 10}})));
      equation
        d = Medium.density(Medium.setState_phX(port.p, inStream(port.h_outflow), inStream(port.Xi_outflow)));
        annotation(
          defaultComponentName = "density",
          Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{0, -70}, {0, -100}}, color = {0, 0, 127}), Text(extent = {{-150, 80}, {150, 120}}, textString = "%name", lineColor = {0, 0, 255}), Text(extent = {{154, -31}, {56, -61}}, lineColor = {0, 0, 0}, textString = "d"), Line(points = {{70, 0}, {100, 0}}, color = {0, 0, 127})}),
          Documentation(info = "<html>
<p>
This component monitors the density of the fluid passing its port.
The sensor is ideal, i.e., it does not influence the fluid.
</p>

</html>"));
      end Density;

      model DensityTwoPort "Ideal two port density sensor"
        extends Sensors.BaseClasses.PartialFlowSensor;
        extends Modelica.Icons.RotationalSensor;
        Modelica.Blocks.Interfaces.RealOutput d(final quantity = "Density", final unit = "kg/m3", displayUnit = "g/cm3", min = 0) "Density of the passing fluid" annotation(
          Placement(transformation(origin = {0, 110}, extent = {{10, -10}, {-10, 10}}, rotation = 270)));
      protected
        Medium.Density rho_a_inflow "Density of inflowing fluid at port_a";
        Medium.Density rho_b_inflow "Density of inflowing fluid at port_b or rho_a_inflow, if uni-directional flow";
      equation
        if allowFlowReversal then
          rho_a_inflow = Medium.density(Medium.setState_phX(port_b.p, port_b.h_outflow, port_b.Xi_outflow));
          rho_b_inflow = Medium.density(Medium.setState_phX(port_a.p, port_a.h_outflow, port_a.Xi_outflow));
          d = Modelica.Fluid.Utilities.regStep(port_a.m_flow, rho_a_inflow, rho_b_inflow, m_flow_small);
        else
          d = Medium.density(Medium.setState_phX(port_b.p, port_b.h_outflow, port_b.Xi_outflow));
          rho_a_inflow = d;
          rho_b_inflow = d;
        end if;
        annotation(
          defaultComponentName = "density",
          Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{102, 124}, {6, 95}}, lineColor = {0, 0, 0}, textString = "d"), Line(points = {{0, 100}, {0, 70}}, color = {0, 0, 127}), Line(points = {{-100, 0}, {-70, 0}}, color = {0, 128, 255}), Line(points = {{70, 0}, {100, 0}}, color = {0, 128, 255})}),
          Documentation(info = "<html>
<p>
This component monitors the density of the fluid flowing from port_a to port_b.
The sensor is ideal, i.e., it does not influence the fluid.
</p>
</html>"));
      end DensityTwoPort;

      model Temperature "Ideal one port temperature sensor"
        extends Sensors.BaseClasses.PartialAbsoluteSensor;
        Modelica.Blocks.Interfaces.RealOutput T(final quantity = "ThermodynamicTemperature", final unit = "K", displayUnit = "degC", min = 0) "Temperature in port medium" annotation(
          Placement(transformation(extent = {{60, -10}, {80, 10}})));
      equation
        T = Medium.temperature(Medium.setState_phX(port.p, inStream(port.h_outflow), inStream(port.Xi_outflow)));
        annotation(
          defaultComponentName = "temperature",
          Documentation(info = "<html>
<p>
This component monitors the temperature of the fluid passing its port.
The sensor is ideal, i.e., it does not influence the fluid.
</p>
</html>"),
          Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{0, -70}, {0, -100}}, color = {0, 0, 127}), Ellipse(extent = {{-20, -98}, {20, -60}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {191, 0, 0}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-12, 40}, {12, -68}}, lineColor = {191, 0, 0}, fillColor = {191, 0, 0}, fillPattern = FillPattern.Solid), Polygon(points = {{-12, 40}, {-12, 80}, {-10, 86}, {-6, 88}, {0, 90}, {6, 88}, {10, 86}, {12, 80}, {12, 40}, {-12, 40}}, lineColor = {0, 0, 0}, lineThickness = 0.5), Line(points = {{-12, 40}, {-12, -64}}, thickness = 0.5), Line(points = {{12, 40}, {12, -64}}, thickness = 0.5), Line(points = {{-40, -20}, {-12, -20}}), Line(points = {{-40, 20}, {-12, 20}}), Line(points = {{-40, 60}, {-12, 60}}), Line(points = {{12, 0}, {60, 0}}, color = {0, 0, 127})}),
          Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-20, -88}, {20, -50}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {191, 0, 0}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-12, 50}, {12, -58}}, lineColor = {191, 0, 0}, fillColor = {191, 0, 0}, fillPattern = FillPattern.Solid), Polygon(points = {{-12, 50}, {-12, 90}, {-10, 96}, {-6, 98}, {0, 100}, {6, 98}, {10, 96}, {12, 90}, {12, 50}, {-12, 50}}, lineColor = {0, 0, 0}, lineThickness = 0.5), Line(points = {{-12, 50}, {-12, -54}}, thickness = 0.5), Line(points = {{12, 50}, {12, -54}}, thickness = 0.5), Line(points = {{-40, -10}, {-12, -10}}), Line(points = {{-40, 30}, {-12, 30}}), Line(points = {{-40, 70}, {-12, 70}}), Text(extent = {{126, -30}, {6, -60}}, lineColor = {0, 0, 0}, textString = "T"), Text(extent = {{-150, 110}, {150, 150}}, textString = "%name", lineColor = {0, 0, 255}), Line(points = {{12, 0}, {60, 0}}, color = {0, 0, 127})}));
      end Temperature;

      model TemperatureTwoPort "Ideal two port temperature sensor"
        extends Sensors.BaseClasses.PartialFlowSensor;
        Modelica.Blocks.Interfaces.RealOutput T(final quantity = "ThermodynamicTemperature", final unit = "K", min = 0, displayUnit = "degC") "Temperature of the passing fluid" annotation(
          Placement(transformation(origin = {0, 110}, extent = {{10, -10}, {-10, 10}}, rotation = 270)));
      protected
        Medium.Temperature T_a_inflow "Temperature of inflowing fluid at port_a";
        Medium.Temperature T_b_inflow "Temperature of inflowing fluid at port_b or T_a_inflow, if uni-directional flow";
      equation
        if allowFlowReversal then
          T_a_inflow = Medium.temperature(Medium.setState_phX(port_b.p, port_b.h_outflow, port_b.Xi_outflow));
          T_b_inflow = Medium.temperature(Medium.setState_phX(port_a.p, port_a.h_outflow, port_a.Xi_outflow));
          T = Modelica.Fluid.Utilities.regStep(port_a.m_flow, T_a_inflow, T_b_inflow, m_flow_small);
        else
          T = Medium.temperature(Medium.setState_phX(port_b.p, port_b.h_outflow, port_b.Xi_outflow));
          T_a_inflow = T;
          T_b_inflow = T;
        end if;
        annotation(
          defaultComponentName = "temperature",
          Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{0, 100}, {0, 50}}, color = {0, 0, 127}), Line(points = {{-92, 0}, {100, 0}}, color = {0, 128, 255}), Ellipse(extent = {{-20, -68}, {20, -30}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {191, 0, 0}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-12, 50}, {12, -34}}, lineColor = {191, 0, 0}, fillColor = {191, 0, 0}, fillPattern = FillPattern.Solid), Polygon(points = {{-12, 50}, {-12, 70}, {-10, 76}, {-6, 78}, {0, 80}, {6, 78}, {10, 76}, {12, 70}, {12, 50}, {-12, 50}}, lineColor = {0, 0, 0}, lineThickness = 0.5), Line(points = {{-12, 50}, {-12, -35}}, thickness = 0.5), Line(points = {{12, 50}, {12, -34}}, thickness = 0.5), Line(points = {{-40, -10}, {-12, -10}}), Line(points = {{-40, 20}, {-12, 20}}), Line(points = {{-40, 50}, {-12, 50}}), Text(extent = {{94, 122}, {0, 92}}, lineColor = {0, 0, 0}, textString = "T")}),
          Documentation(info = "<html>
<p>
This component monitors the temperature of the passing fluid.
The sensor is ideal, i.e., it does not influence the fluid.
</p>
</html>"));
      end TemperatureTwoPort;

      model SpecificEnthalpy "Ideal one port specific enthalpy sensor"
        extends Sensors.BaseClasses.PartialAbsoluteSensor;
        extends Modelica.Icons.RotationalSensor;
        Modelica.Blocks.Interfaces.RealOutput h_out(final quantity = "SpecificEnergy", final unit = "J/kg") "Specific enthalpy in port medium" annotation(
          Placement(transformation(extent = {{100, -10}, {120, 10}})));
      equation
        h_out = inStream(port.h_outflow);
        annotation(
          defaultComponentName = "specificEnthalpy",
          Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{0, -70}, {0, -100}}, color = {0, 0, 127}), Text(extent = {{-150, 80}, {150, 120}}, textString = "%name", lineColor = {0, 0, 255}), Text(extent = {{168, -30}, {52, -60}}, lineColor = {0, 0, 0}, textString = "h"), Line(points = {{70, 0}, {100, 0}}, color = {0, 0, 127})}),
          Documentation(info = "<html>
<p>
This component monitors the specific enthalpy of the fluid passing its port.
The sensor is ideal, i.e., it does not influence the fluid.
</p>
</html>"));
      end SpecificEnthalpy;

      model SpecificEnthalpyTwoPort "Ideal two port sensor for the specific enthalpy"
        extends Sensors.BaseClasses.PartialFlowSensor;
        extends Modelica.Icons.RotationalSensor;
        Modelica.Blocks.Interfaces.RealOutput h_out(final quantity = "SpecificEnergy", final unit = "J/kg") "Specific enthalpy of the passing fluid" annotation(
          Placement(transformation(origin = {0, 110}, extent = {{10, -10}, {-10, 10}}, rotation = 270)));
      equation
        if allowFlowReversal then
          h_out = Modelica.Fluid.Utilities.regStep(port_a.m_flow, port_b.h_outflow, port_a.h_outflow, m_flow_small);
        else
          h_out = port_b.h_outflow;
        end if;
        annotation(
          defaultComponentName = "specificEnthalpy",
          Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{102, 120}, {0, 90}}, lineColor = {0, 0, 0}, textString = "h"), Line(points = {{0, 100}, {0, 70}}, color = {0, 0, 127}), Line(points = {{-100, 0}, {-70, 0}}, color = {0, 128, 255}), Line(points = {{70, 0}, {100, 0}}, color = {0, 128, 255})}),
          Documentation(info = "<html>
<p>
This component monitors the specific enthalpy of a passing fluid.
The sensor is ideal, i.e., it does not influence the fluid.
</p>
</html>"));
      end SpecificEnthalpyTwoPort;

      model SpecificEntropy "Ideal one port specific entropy sensor"
        extends Sensors.BaseClasses.PartialAbsoluteSensor;
        extends Modelica.Icons.RotationalSensor;
        Modelica.Blocks.Interfaces.RealOutput s(final quantity = "SpecificEntropy", final unit = "J/(kg.K)") "Specific entropy in port medium" annotation(
          Placement(transformation(extent = {{100, -10}, {120, 10}})));
      equation
        s = Medium.specificEntropy(Medium.setState_phX(port.p, inStream(port.h_outflow), inStream(port.Xi_outflow)));
        annotation(
          defaultComponentName = "specificEntropy",
          Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{0, -70}, {0, -100}}, color = {0, 0, 127}), Text(extent = {{-150, 80}, {150, 120}}, textString = "%name", lineColor = {0, 0, 255}), Text(extent = {{156, -24}, {54, -54}}, lineColor = {0, 0, 0}, textString = "s"), Line(points = {{70, 0}, {100, 0}}, color = {0, 0, 127})}),
          Documentation(info = "<html>
<p>
This component monitors the specific entropy of the fluid passing its port.
The sensor is ideal, i.e., it does not influence the fluid.
</p>
</html>"));
      end SpecificEntropy;

      model SpecificEntropyTwoPort "Ideal two port sensor for the specific entropy"
        extends Sensors.BaseClasses.PartialFlowSensor;
        extends Modelica.Icons.RotationalSensor;
        Modelica.Blocks.Interfaces.RealOutput s(final quantity = "SpecificEntropy", final unit = "J/(kg.K)") "Specific entropy of the passing fluid" annotation(
          Placement(transformation(origin = {0, 110}, extent = {{10, -10}, {-10, 10}}, rotation = 270)));
      protected
        Medium.SpecificEntropy s_a_inflow "Specific entropy of inflowing fluid at port_a";
        Medium.SpecificEntropy s_b_inflow "Specific entropy of inflowing fluid at port_b or s_a_inflow, if uni-directional flow";
      equation
        if allowFlowReversal then
          s_a_inflow = Medium.specificEntropy(Medium.setState_phX(port_b.p, port_b.h_outflow, port_b.Xi_outflow));
          s_b_inflow = Medium.specificEntropy(Medium.setState_phX(port_a.p, port_a.h_outflow, port_a.Xi_outflow));
          s = Modelica.Fluid.Utilities.regStep(port_a.m_flow, s_a_inflow, s_b_inflow, m_flow_small);
        else
          s = Medium.specificEntropy(Medium.setState_phX(port_b.p, port_b.h_outflow, port_b.Xi_outflow));
          s_a_inflow = s;
          s_b_inflow = s;
        end if;
        annotation(
          defaultComponentName = "specificEntropy",
          Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{120, 120}, {0, 90}}, lineColor = {0, 0, 0}, textString = "s"), Line(points = {{0, 100}, {0, 70}}, color = {0, 0, 127}), Line(points = {{-100, 0}, {-70, 0}}, color = {0, 128, 255}), Line(points = {{70, 0}, {100, 0}}, color = {0, 128, 255})}),
          Documentation(info = "<html>
<p>
This component monitors the specific entropy of the passing fluid.
The sensor is ideal, i.e., it does not influence the fluid.
</p>
</html>"));
      end SpecificEntropyTwoPort;

      model MassFractions "Ideal one port mass fraction sensor"
        extends Modelica.Fluid.Sensors.BaseClasses.PartialAbsoluteSensor;
        extends Modelica.Icons.RotationalSensor;
        parameter String substanceName = "water" "Name of mass fraction";
        Modelica.Blocks.Interfaces.RealOutput Xi "Mass fraction in port medium" annotation(
          Placement(transformation(extent = {{100, -10}, {120, 10}})));
      protected
        parameter Integer ind(fixed = false) "Index of species in vector of independent mass fractions";
        Medium.MassFraction XiVec[Medium.nXi] "Mass fraction vector, needed because indexed argument for the operator inStream is not supported";
      initial algorithm
        ind := -1;
        for i in 1:Medium.nXi loop
          if Modelica.Utilities.Strings.isEqual(Medium.substanceNames[i], substanceName) then
            ind := i;
          end if;
        end for;
        assert(ind > 0, "Mass fraction '" + substanceName + "' is not present in medium '" + Medium.mediumName + "'.\n" + "Check sensor parameter and medium model.");
      equation
        XiVec = inStream(port.Xi_outflow);
        Xi = XiVec[ind];
        annotation(
          defaultComponentName = "massFraction",
          Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{0, -70}, {0, -100}}, color = {0, 0, 127}), Text(extent = {{-150, 80}, {150, 120}}, textString = "%name", lineColor = {0, 0, 255}), Text(extent = {{160, -30}, {60, -60}}, lineColor = {0, 0, 0}, textString = "Xi"), Line(points = {{70, 0}, {100, 0}}, color = {0, 0, 127})}),
          Documentation(info = "<html>
<p>
This component monitors the mass fraction contained in the fluid passing its port.
The sensor is ideal, i.e., it does not influence the fluid.
</p>
</html>", revisions = "<html>
<ul>
<li>2011-12-14: Stefan Wischhusen: Initial Release.</li>
</ul>
</html>"));
      end MassFractions;

      model MassFractionsTwoPort "Ideal two port sensor for mass fraction"
        extends Modelica.Fluid.Sensors.BaseClasses.PartialFlowSensor;
        extends Modelica.Icons.RotationalSensor;
        Modelica.Blocks.Interfaces.RealOutput Xi "Mass fraction in port medium" annotation(
          Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 90, origin = {0, 110}), iconTransformation(extent = {{-10, -10}, {10, 10}}, rotation = 90, origin = {0, 110})));
        parameter String substanceName = "water" "Name of mass fraction";
      protected
        parameter Integer ind(fixed = false) "Index of species in vector of independent mass fractions";
      initial algorithm
        ind := -1;
        for i in 1:Medium.nC loop
          if Modelica.Utilities.Strings.isEqual(Medium.substanceNames[i], substanceName) then
            ind := i;
          end if;
        end for;
        assert(ind > 0, "Mass fraction '" + substanceName + "' is not present in medium '" + Medium.mediumName + "'.\n" + "Check sensor parameter and medium model.");
      equation
        if allowFlowReversal then
          Xi = Modelica.Fluid.Utilities.regStep(port_a.m_flow, port_b.Xi_outflow[ind], port_a.Xi_outflow[ind], m_flow_small);
        else
          Xi = port_b.Xi_outflow[ind];
        end if;
        annotation(
          defaultComponentName = "massFraction",
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{82, 122}, {0, 92}}, lineColor = {0, 0, 0}, textString = "Xi"), Line(points = {{0, 100}, {0, 70}}, color = {0, 0, 127}), Line(points = {{-100, 0}, {-70, 0}}, color = {0, 128, 255}), Line(points = {{70, 0}, {100, 0}}, color = {0, 128, 255})}),
          Documentation(info = "<html>
<p>
This component monitors the mass fraction of the passing fluid.
The sensor is ideal, i.e., it does not influence the fluid.
</p> </html>", revisions = "<html>
<ul>
<li>2011-12-14: Stefan Wischhusen: Initial Release.</li>
</ul>
</html>"));
      end MassFractionsTwoPort;

      model TraceSubstances "Ideal one port trace substances sensor"
        extends Sensors.BaseClasses.PartialAbsoluteSensor;
        extends Modelica.Icons.RotationalSensor;
        parameter String substanceName = "CO2" "Name of trace substance";
        Modelica.Blocks.Interfaces.RealOutput C "Trace substance in port medium" annotation(
          Placement(transformation(extent = {{100, -10}, {120, 10}})));
      protected
        parameter Integer ind(fixed = false) "Index of species in vector of auxiliary substances";
        Medium.ExtraProperty CVec[Medium.nC](quantity = Medium.extraPropertiesNames) "Trace substances vector, needed because indexed argument for the operator inStream is not supported";
      initial algorithm
        ind := -1;
        for i in 1:Medium.nC loop
          if Modelica.Utilities.Strings.isEqual(Medium.extraPropertiesNames[i], substanceName) then
            ind := i;
          end if;
        end for;
        assert(ind > 0, "Trace substance '" + substanceName + "' is not present in medium '" + Medium.mediumName + "'.\n" + "Check sensor parameter and medium model.");
      equation
        CVec = inStream(port.C_outflow);
        C = CVec[ind];
        annotation(
          defaultComponentName = "traceSubstance",
          Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{0, -70}, {0, -100}}, color = {0, 0, 127}), Text(extent = {{-150, 80}, {150, 120}}, textString = "%name", lineColor = {0, 0, 255}), Text(extent = {{160, -30}, {60, -60}}, lineColor = {0, 0, 0}, textString = "C"), Line(points = {{70, 0}, {100, 0}}, color = {0, 0, 127})}),
          Documentation(info = "<html>
<p>
This component monitors the trace substances contained in the fluid passing its port.
The sensor is ideal, i.e., it does not influence the fluid.
</p>
</html>"));
      end TraceSubstances;

      model TraceSubstancesTwoPort "Ideal two port sensor for trace substance"
        extends Sensors.BaseClasses.PartialFlowSensor;
        extends Modelica.Icons.RotationalSensor;
        Modelica.Blocks.Interfaces.RealOutput C "Trace substance of the passing fluid" annotation(
          Placement(transformation(origin = {0, 110}, extent = {{10, -10}, {-10, 10}}, rotation = 270)));
        parameter String substanceName = "CO2" "Name of trace substance";
      protected
        parameter Integer ind(fixed = false) "Index of species in vector of auxiliary substances";
      initial algorithm
        ind := -1;
        for i in 1:Medium.nC loop
          if Modelica.Utilities.Strings.isEqual(Medium.extraPropertiesNames[i], substanceName) then
            ind := i;
          end if;
        end for;
        assert(ind > 0, "Trace substance '" + substanceName + "' is not present in medium '" + Medium.mediumName + "'.\n" + "Check sensor parameter and medium model.");
      equation
        if allowFlowReversal then
          C = Modelica.Fluid.Utilities.regStep(port_a.m_flow, port_b.C_outflow[ind], port_a.C_outflow[ind], m_flow_small);
        else
          C = port_b.C_outflow[ind];
        end if;
        annotation(
          defaultComponentName = "traceSubstance",
          Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{82, 122}, {0, 92}}, lineColor = {0, 0, 0}, textString = "C"), Line(points = {{0, 100}, {0, 70}}, color = {0, 0, 127}), Line(points = {{-100, 0}, {-70, 0}}, color = {0, 128, 255}), Line(points = {{70, 0}, {100, 0}}, color = {0, 128, 255})}),
          Documentation(info = "<html>
<p>
This component monitors the trace substance of the passing fluid.
The sensor is ideal, i.e., it does not influence the fluid.
</p>
</html>"));
      end TraceSubstancesTwoPort;

      model MassFlowRate "Ideal sensor for mass flow rate"
        extends Sensors.BaseClasses.PartialFlowSensor;
        extends Modelica.Icons.RotationalSensor;
        Modelica.Blocks.Interfaces.RealOutput m_flow(quantity = "MassFlowRate", final unit = "kg/s") "Mass flow rate from port_a to port_b" annotation(
          Placement(transformation(origin = {0, 110}, extent = {{10, -10}, {-10, 10}}, rotation = 270)));
      equation
        m_flow = port_a.m_flow;
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{70, 0}, {100, 0}}, color = {0, 128, 255}), Text(extent = {{162, 120}, {2, 90}}, lineColor = {0, 0, 0}, textString = "m_flow"), Line(points = {{0, 100}, {0, 70}}, color = {0, 0, 127}), Line(points = {{-100, 0}, {-70, 0}}, color = {0, 128, 255})}),
          Documentation(info = "<html>
<p>
This component monitors the mass flow rate flowing from port_a to port_b.
The sensor is ideal, i.e., it does not influence the fluid.
</p>
</html>"));
      end MassFlowRate;

      model VolumeFlowRate "Ideal sensor for volume flow rate"
        extends Sensors.BaseClasses.PartialFlowSensor;
        extends Modelica.Icons.RotationalSensor;
        Modelica.Blocks.Interfaces.RealOutput V_flow(final quantity = "VolumeFlowRate", final unit = "m3/s") "Volume flow rate from port_a to port_b" annotation(
          Placement(transformation(origin = {0, 110}, extent = {{10, -10}, {-10, 10}}, rotation = 270)));
      protected
        Medium.Density rho_a_inflow "Density of inflowing fluid at port_a";
        Medium.Density rho_b_inflow "Density of inflowing fluid at port_b or rho_a_inflow, if uni-directional flow";
        Medium.Density d "Density of the passing fluid";
      equation
        if allowFlowReversal then
          rho_a_inflow = Medium.density(Medium.setState_phX(port_b.p, port_b.h_outflow, port_b.Xi_outflow));
          rho_b_inflow = Medium.density(Medium.setState_phX(port_a.p, port_a.h_outflow, port_a.Xi_outflow));
          d = Modelica.Fluid.Utilities.regStep(port_a.m_flow, rho_a_inflow, rho_b_inflow, m_flow_small);
        else
          d = Medium.density(Medium.setState_phX(port_b.p, port_b.h_outflow, port_b.Xi_outflow));
          rho_a_inflow = d;
          rho_b_inflow = d;
        end if;
        V_flow = port_a.m_flow / d;
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{160, 120}, {0, 90}}, lineColor = {0, 0, 0}, textString = "V_flow"), Line(points = {{0, 100}, {0, 70}}, color = {0, 0, 127}), Line(points = {{-100, 0}, {-70, 0}}, color = {0, 128, 255}), Line(points = {{70, 0}, {100, 0}}, color = {0, 128, 255})}),
          Documentation(info = "<html>
<p>
This component monitors the volume flow rate flowing from port_a to port_b.
The sensor is ideal, i.e., it does not influence the fluid.
</p>
</html>"));
      end VolumeFlowRate;

      model RelativePressure "Ideal relative pressure sensor"
        extends Sensors.BaseClasses.PartialRelativeSensor;
        Modelica.Blocks.Interfaces.RealOutput p_rel(final quantity = "Pressure", final unit = "Pa", displayUnit = "bar") "Relative pressure signal" annotation(
          Placement(transformation(origin = {0, -90}, extent = {{10, -10}, {-10, 10}}, rotation = 90)));
      equation
// Relative pressure
        p_rel = port_a.p - port_b.p;
        annotation(
          Icon(graphics = {Line(points = {{0, -30}, {0, -80}}, color = {0, 0, 127}), Text(extent = {{130, -70}, {4, -100}}, lineColor = {0, 0, 0}, textString = "p_rel")}),
          Documentation(info = "<html>
<p>
The relative pressure \"port_a.p - port_b.p\" is determined between
the two ports of this component and is provided as output signal. The
sensor should be connected in parallel with other equipment, no flow
through the sensor is allowed.
</p>
</html>"));
      end RelativePressure;

      model RelativeTemperature "Ideal relative temperature sensor"
        extends Sensors.BaseClasses.PartialRelativeSensor;
        Modelica.Blocks.Interfaces.RealOutput T_rel(final quantity = "ThermodynamicTemperature", final unit = "K", displayUnit = "degC", min = 0) "Relative temperature signal" annotation(
          Placement(transformation(origin = {0, -90}, extent = {{10, -10}, {-10, 10}}, rotation = 90)));
      equation
// Relative temperature
        T_rel = Medium.temperature(Medium.setState_phX(port_a.p, inStream(port_a.h_outflow), inStream(port_a.Xi_outflow))) - Medium.temperature(Medium.setState_phX(port_b.p, inStream(port_b.h_outflow), inStream(port_b.Xi_outflow)));
        annotation(
          Icon(graphics = {Line(points = {{0, -30}, {0, -80}}, color = {0, 0, 127}), Text(extent = {{128, -70}, {10, -100}}, lineColor = {0, 0, 0}, textString = "T_rel")}),
          Documentation(info = "<html>
<p>
The relative temperature \"T(port_a) - T(port_b)\" is determined between
the two ports of this component and is provided as output signal. The
sensor should be connected in parallel with other equipment, no flow
through the sensor is allowed.
</p>
</html>"));
      end RelativeTemperature;

      package BaseClasses "Base classes used in the Sensors package (only of interest to build new component models)"
        extends Modelica.Icons.BasesPackage;

        partial model PartialAbsoluteSensor "Partial component to model a sensor that measures a potential variable"
          replaceable package Medium = Modelica.Media.Interfaces.PartialMedium "Medium in the sensor" annotation(
            choicesAllMatching = true);
          Modelica.Fluid.Interfaces.FluidPort_a port(redeclare package Medium = Medium, m_flow(min = 0)) annotation(
            Placement(transformation(origin = {0, -100}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
        equation
          port.m_flow = 0;
          port.h_outflow = Medium.h_default;
          port.Xi_outflow = Medium.X_default[1:Medium.nXi];
          port.C_outflow = zeros(Medium.nC);
          annotation(
            Documentation(info = "<html>
<p>
Partial component to model an <b>absolute sensor</b>. Can be used for pressure sensor models.
Use for other properties such as temperature or density is discouraged, because the enthalpy at the connector can have different meanings, depending on the connection topology. Use <code>PartialFlowSensor</code> instead.
as signal.
</p>
</html>"));
        end PartialAbsoluteSensor;

        model PartialRelativeSensor "Partial component to model a sensor that measures the difference between two potential variables"
          extends Modelica.Icons.TranslationalSensor;
          replaceable package Medium = Modelica.Media.Interfaces.PartialMedium "Medium in the sensor" annotation(
            choicesAllMatching = true);
          Modelica.Fluid.Interfaces.FluidPort_a port_a(m_flow(min = 0), redeclare package Medium = Medium) annotation(
            Placement(transformation(extent = {{-110, -10}, {-90, 10}})));
          Modelica.Fluid.Interfaces.FluidPort_b port_b(m_flow(min = 0), redeclare package Medium = Medium) annotation(
            Placement(transformation(extent = {{110, -12}, {90, 8}}), iconTransformation(extent = {{110, -10}, {90, 10}})));
        equation
// Zero flow equations for connectors
          port_a.m_flow = 0;
          port_b.m_flow = 0;
// No contribution of specific quantities
          port_a.h_outflow = Medium.h_default;
          port_b.h_outflow = Medium.h_default;
          port_a.Xi_outflow = Medium.X_default[1:Medium.nXi];
          port_b.Xi_outflow = Medium.X_default[1:Medium.nXi];
          port_a.C_outflow = zeros(Medium.nC);
          port_b.C_outflow = zeros(Medium.nC);
          annotation(
            Icon(graphics = {Line(points = {{-100, 0}, {-70, 0}}, color = {0, 127, 255}), Line(points = {{70, 0}, {100, 0}}, color = {0, 127, 255}), Text(extent = {{-150, 40}, {150, 80}}, textString = "%name", lineColor = {0, 0, 255}), Line(points = {{32, 3}, {-58, 3}}, color = {0, 128, 255}), Polygon(points = {{22, 18}, {62, 3}, {22, -12}, {22, 18}}, lineColor = {0, 128, 255}, fillColor = {0, 128, 255}, fillPattern = FillPattern.Solid)}),
            Documentation(info = "<html>
<p>
The relative pressure \"port_a.p - port_b.p\" is determined between
the two ports of this component and is provided as output signal. The
sensor should be connected in parallel with other equipment, no flow
through the sensor is allowed.
</p>
</html>"));
        end PartialRelativeSensor;

        partial model PartialFlowSensor "Partial component to model sensors that measure flow properties"
          extends Modelica.Fluid.Interfaces.PartialTwoPort;
          parameter Medium.MassFlowRate m_flow_nominal = system.m_flow_nominal "Nominal value of m_flow = port_a.m_flow" annotation(
            Dialog(tab = "Advanced"));
          parameter Medium.MassFlowRate m_flow_small(min = 0) = if system.use_eps_Re then system.eps_m_flow * m_flow_nominal else system.m_flow_small "Regularization for bi-directional flow in the region |m_flow| < m_flow_small (m_flow_small > 0 required)" annotation(
            Dialog(tab = "Advanced"));
        equation
// mass balance
          0 = port_a.m_flow + port_b.m_flow;
// momentum equation (no pressure loss)
          port_a.p = port_b.p;
// isenthalpic state transformation (no storage and no loss of energy)
          port_a.h_outflow = inStream(port_b.h_outflow);
          port_b.h_outflow = inStream(port_a.h_outflow);
          port_a.Xi_outflow = inStream(port_b.Xi_outflow);
          port_b.Xi_outflow = inStream(port_a.Xi_outflow);
          port_a.C_outflow = inStream(port_b.C_outflow);
          port_b.C_outflow = inStream(port_a.C_outflow);
          annotation(
            Documentation(info = "<html>
<p>
Partial component to model a <b>sensor</b> that measures any intensive properties
of a flow, e.g., to get temperature or density in the flow
between fluid connectors.<br>
The model includes zero-volume balance equations. Sensor models inheriting from
this partial class should add a medium instance to calculate the measured property.
</p>
</html>"));
        end PartialFlowSensor;
      end BaseClasses;
      annotation(
        preferredView = "info",
        Documentation(info = "<html>
<p align = justify>
Package <b>Sensors</b> consists of idealized sensor components that
provide variables of a medium model and/or fluid ports as
output signals. These signals can be, e.g., further processed
with components of the Modelica.Blocks library.
Also more realistic sensor models can be built, by further
processing (e.g., by attaching block Modelica.Blocks.FirstOrder to
model the time constant of the sensor).
</p>

<p align = justify>For the thermodynamic state variables temperature, specific enthalpy, specific entropy and density
the fluid library provides two different types of sensors: <b>regular one port</b> and <b>two port</b> sensors.</p>

<ul>
<li>The <b>regular one port</b> sensors have the advantage of easy introduction and removal from a model, as no connections have to be broken.
A potential drawback is that the obtained value jumps as flow reverts.
</li>

<li>The <b>two port</b> sensors offer the advantages of an adjustable regularized step function around zero flow.
Moreover the obtained result is restricted to the value flowing into port_a if allowFlowReversal is false.</li>
</ul>

<p>
<a href=\"modelica://Modelica.Fluid.Examples.Explanatory.MeasuringTemperature\">Modelica.Fluid.Examples.Explanatory.MeasuringTemperature</a>
demonstrates the differences between one- and two-port sensor at hand of a
simple example.
</p>
</html>", revisions = "<html>
<ul>
<li><i>22 Dec 2008</i>
    by R;uumldiger Franke<br>
    <ul>
    <li>flow sensors based on Interfaces.PartialTwoPort</li>
    <li>adapted docu to stream connectors, i.e., less need for two port sensors</li>
    </ul>
<li><i>4 Dec 2008</i>
    by Michael Wetter<br>
       included sensors for trace substance</li>
<li><i>31 Oct 2007</i>
    by Carsten Heinrich<br>
       updated sensor models, included one and two port sensors for thermodynamic state variables</li>
</ul>
</html>"));
    end Sensors;

    package Interfaces "Interfaces for steady state and unsteady, mixed-phase, multi-substance, incompressible and compressible flow"
      extends Modelica.Icons.InterfacesPackage;

      connector FluidPort "Interface for quasi one-dimensional fluid flow in a piping network (incompressible or compressible, one or more phases, one or more substances)"
        replaceable package Medium = Modelica.Media.Interfaces.PartialMedium "Medium model" annotation(
          choicesAllMatching = true);
        flow Medium.MassFlowRate m_flow "Mass flow rate from the connection point into the component";
        Medium.AbsolutePressure p "Thermodynamic pressure in the connection point";
        stream Medium.SpecificEnthalpy h_outflow "Specific thermodynamic enthalpy close to the connection point if m_flow < 0";
        stream Medium.MassFraction Xi_outflow[Medium.nXi] "Independent mixture mass fractions m_i/m close to the connection point if m_flow < 0";
        stream Medium.ExtraProperty C_outflow[Medium.nC] "Properties c_i/m close to the connection point if m_flow < 0";
      end FluidPort;

      connector FluidPort_a "Generic fluid connector at design inlet"
        extends FluidPort;
        annotation(
          defaultComponentName = "port_a",
          Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-40, 40}, {40, -40}}, lineColor = {0, 0, 0}, fillColor = {0, 127, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-150, 110}, {150, 50}}, textString = "%name")}),
          Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-100, 100}, {100, -100}}, lineColor = {0, 127, 255}, fillColor = {0, 127, 255}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-100, 100}, {100, -100}}, lineColor = {0, 0, 0}, fillColor = {0, 127, 255}, fillPattern = FillPattern.Solid)}));
      end FluidPort_a;

      connector FluidPort_b "Generic fluid connector at design outlet"
        extends FluidPort;
        annotation(
          defaultComponentName = "port_b",
          Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-40, 40}, {40, -40}}, lineColor = {0, 0, 0}, fillColor = {0, 127, 255}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-30, 30}, {30, -30}}, lineColor = {0, 127, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-150, 110}, {150, 50}}, textString = "%name")}),
          Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-100, 100}, {100, -100}}, lineColor = {0, 127, 255}, fillColor = {0, 127, 255}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-100, 100}, {100, -100}}, lineColor = {0, 0, 0}, fillColor = {0, 127, 255}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-80, 80}, {80, -80}}, lineColor = {0, 127, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid)}));
      end FluidPort_b;

      connector FluidPorts_a "Fluid connector with filled, large icon to be used for vectors of FluidPorts (vector dimensions must be added after dragging)"
        extends FluidPort;
        annotation(
          defaultComponentName = "ports_a",
          Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{-50, -200}, {50, 200}}, initialScale = 0.2), graphics = {Text(extent = {{-75, 130}, {75, 100}}, textString = "%name"), Rectangle(extent = {{25, -100}, {-25, 100}}, lineColor = {0, 127, 255}), Ellipse(extent = {{-25, 90}, {25, 40}}, lineColor = {0, 0, 0}, fillColor = {0, 127, 255}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-25, 25}, {25, -25}}, lineColor = {0, 0, 0}, fillColor = {0, 127, 255}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-25, -40}, {25, -90}}, lineColor = {0, 0, 0}, fillColor = {0, 127, 255}, fillPattern = FillPattern.Solid)}),
          Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-50, -200}, {50, 200}}, initialScale = 0.2), graphics = {Rectangle(extent = {{50, -200}, {-50, 200}}, lineColor = {0, 127, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-50, 180}, {50, 80}}, lineColor = {0, 0, 0}, fillColor = {0, 127, 255}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-50, 50}, {50, -50}}, lineColor = {0, 0, 0}, fillColor = {0, 127, 255}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-50, -80}, {50, -180}}, lineColor = {0, 0, 0}, fillColor = {0, 127, 255}, fillPattern = FillPattern.Solid)}));
      end FluidPorts_a;

      connector FluidPorts_b "Fluid connector with outlined, large icon to be used for vectors of FluidPorts (vector dimensions must be added after dragging)"
        extends FluidPort;
        annotation(
          defaultComponentName = "ports_b",
          Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{-50, -200}, {50, 200}}, initialScale = 0.2), graphics = {Text(extent = {{-75, 130}, {75, 100}}, textString = "%name"), Rectangle(extent = {{-25, 100}, {25, -100}}, lineColor = {0, 0, 0}), Ellipse(extent = {{-25, 90}, {25, 40}}, lineColor = {0, 0, 0}, fillColor = {0, 127, 255}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-25, 25}, {25, -25}}, lineColor = {0, 0, 0}, fillColor = {0, 127, 255}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-25, -40}, {25, -90}}, lineColor = {0, 0, 0}, fillColor = {0, 127, 255}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-15, -50}, {15, -80}}, lineColor = {0, 127, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-15, 15}, {15, -15}}, lineColor = {0, 127, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-15, 50}, {15, 80}}, lineColor = {0, 127, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid)}),
          Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-50, -200}, {50, 200}}, initialScale = 0.2), graphics = {Rectangle(extent = {{-50, 200}, {50, -200}}, lineColor = {0, 127, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-50, 180}, {50, 80}}, lineColor = {0, 0, 0}, fillColor = {0, 127, 255}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-50, 50}, {50, -50}}, lineColor = {0, 0, 0}, fillColor = {0, 127, 255}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-50, -80}, {50, -180}}, lineColor = {0, 0, 0}, fillColor = {0, 127, 255}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-30, 30}, {30, -30}}, lineColor = {0, 127, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-30, 100}, {30, 160}}, lineColor = {0, 127, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-30, -100}, {30, -160}}, lineColor = {0, 127, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid)}));
      end FluidPorts_b;

      partial model PartialTwoPort "Partial component with two ports"
        import Modelica.Constants;
        outer Modelica.Fluid.System system "System wide properties";
        replaceable package Medium = Modelica.Media.Interfaces.PartialMedium "Medium in the component" annotation(
          choicesAllMatching = true);
        parameter Boolean allowFlowReversal = system.allowFlowReversal "= true to allow flow reversal, false restricts to design direction (port_a -> port_b)" annotation(
          Dialog(tab = "Assumptions"),
          Evaluate = true);
        Modelica.Fluid.Interfaces.FluidPort_a port_a(redeclare package Medium = Medium, m_flow(min = if allowFlowReversal then -Constants.inf else 0)) "Fluid connector a (positive design flow direction is from port_a to port_b)" annotation(
          Placement(transformation(extent = {{-110, -10}, {-90, 10}})));
        Modelica.Fluid.Interfaces.FluidPort_b port_b(redeclare package Medium = Medium, m_flow(max = if allowFlowReversal then +Constants.inf else 0)) "Fluid connector b (positive design flow direction is from port_a to port_b)" annotation(
          Placement(transformation(extent = {{110, -10}, {90, 10}}), iconTransformation(extent = {{110, -10}, {90, 10}})));
        // Model structure, e.g., used for visualization
      protected
        parameter Boolean port_a_exposesState = false "= true if port_a exposes the state of a fluid volume";
        parameter Boolean port_b_exposesState = false "= true if port_b.p exposes the state of a fluid volume";
        parameter Boolean showDesignFlowDirection = true "= false to hide the arrow in the model icon";
        annotation(
          Documentation(info = "<html>
<p>
This partial model defines an interface for components with two ports.
The treatment of the design flow direction and of flow reversal are predefined based on the parameter <code><b>allowFlowReversal</b></code>.
The component may transport fluid and may have internal storage for a given fluid <code><b>Medium</b></code>.
</p>
<p>
An extending model providing direct access to internal storage of mass or energy through port_a or port_b
should redefine the protected parameters <code><b>port_a_exposesState</b></code> and <code><b>port_b_exposesState</b></code> appropriately.
This will be visualized at the port icons, in order to improve the understanding of fluid model diagrams.
</p>
</html>"),
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{20, -70}, {60, -85}, {20, -100}, {20, -70}}, lineColor = {0, 128, 255}, fillColor = {0, 128, 255}, fillPattern = FillPattern.Solid, visible = showDesignFlowDirection), Polygon(points = {{20, -75}, {50, -85}, {20, -95}, {20, -75}}, lineColor = {255, 255, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, visible = allowFlowReversal), Line(points = {{55, -85}, {-60, -85}}, color = {0, 128, 255}, visible = showDesignFlowDirection), Text(extent = {{-149, -114}, {151, -154}}, lineColor = {0, 0, 255}, textString = "%name"), Ellipse(extent = {{-110, 26}, {-90, -24}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid, visible = port_a_exposesState), Ellipse(extent = {{90, 25}, {110, -25}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid, visible = port_b_exposesState)}));
      end PartialTwoPort;

      partial model PartialTwoPortTransport "Partial element transporting fluid between two ports without storage of mass or energy"
        extends PartialTwoPort(final port_a_exposesState = false, final port_b_exposesState = false);
        // Advanced
        // Note: value of dp_start shall be refined by derived model, basing on local dp_nominal
        parameter Medium.AbsolutePressure dp_start(min = -Modelica.Constants.inf) = 0.01 * system.p_start "Guess value of dp = port_a.p - port_b.p" annotation(
          Dialog(tab = "Advanced"));
        parameter Medium.MassFlowRate m_flow_start = system.m_flow_start "Guess value of m_flow = port_a.m_flow" annotation(
          Dialog(tab = "Advanced"));
        // Note: value of m_flow_small shall be refined by derived model, basing on local m_flow_nominal
        parameter Medium.MassFlowRate m_flow_small = if system.use_eps_Re then system.eps_m_flow * system.m_flow_nominal else system.m_flow_small "Small mass flow rate for regularization of zero flow" annotation(
          Dialog(tab = "Advanced"));
        // Diagnostics
        parameter Boolean show_T = true "= true, if temperatures at port_a and port_b are computed" annotation(
          Dialog(tab = "Advanced", group = "Diagnostics"));
        parameter Boolean show_V_flow = true "= true, if volume flow rate at inflowing port is computed" annotation(
          Dialog(tab = "Advanced", group = "Diagnostics"));
        // Variables
        Medium.MassFlowRate m_flow(min = if allowFlowReversal then -Modelica.Constants.inf else 0, start = m_flow_start) "Mass flow rate in design flow direction";
        Modelica.SIunits.Pressure dp(start = dp_start) "Pressure difference between port_a and port_b (= port_a.p - port_b.p)";
        Modelica.SIunits.VolumeFlowRate V_flow = m_flow / Modelica.Fluid.Utilities.regStep(m_flow, Medium.density(state_a), Medium.density(state_b), m_flow_small) if show_V_flow "Volume flow rate at inflowing port (positive when flow from port_a to port_b)";
        Medium.Temperature port_a_T = Modelica.Fluid.Utilities.regStep(port_a.m_flow, Medium.temperature(state_a), Medium.temperature(Medium.setState_phX(port_a.p, port_a.h_outflow, port_a.Xi_outflow)), m_flow_small) if show_T "Temperature close to port_a, if show_T = true";
        Medium.Temperature port_b_T = Modelica.Fluid.Utilities.regStep(port_b.m_flow, Medium.temperature(state_b), Medium.temperature(Medium.setState_phX(port_b.p, port_b.h_outflow, port_b.Xi_outflow)), m_flow_small) if show_T "Temperature close to port_b, if show_T = true";
      protected
        Medium.ThermodynamicState state_a "state for medium inflowing through port_a";
        Medium.ThermodynamicState state_b "state for medium inflowing through port_b";
      equation
// medium states
        state_a = Medium.setState_phX(port_a.p, inStream(port_a.h_outflow), inStream(port_a.Xi_outflow));
        state_b = Medium.setState_phX(port_b.p, inStream(port_b.h_outflow), inStream(port_b.Xi_outflow));
// Pressure drop in design flow direction
        dp = port_a.p - port_b.p;
// Design direction of mass flow rate
        m_flow = port_a.m_flow;
        assert(m_flow > (-m_flow_small) or allowFlowReversal, "Reverting flow occurs even though allowFlowReversal is false");
// Mass balance (no storage)
        port_a.m_flow + port_b.m_flow = 0;
// Transport of substances
        port_a.Xi_outflow = inStream(port_b.Xi_outflow);
        port_b.Xi_outflow = inStream(port_a.Xi_outflow);
        port_a.C_outflow = inStream(port_b.C_outflow);
        port_b.C_outflow = inStream(port_a.C_outflow);
        annotation(
          Documentation(info = "<html>
<p>
This component transports fluid between its two ports, without storing mass or energy.
Energy may be exchanged with the environment though, e.g., in the form of work.
<code>PartialTwoPortTransport</code> is intended as base class for devices like orifices, valves and simple fluid machines.</p>
<p>
Three equations need to be added by an extending class using this component:
</p>
<ul>
<li>the momentum balance specifying the relationship between the pressure drop <code>dp</code> and the mass flow rate <code>m_flow</code>,</li>
<li><code>port_b.h_outflow</code> for flow in design direction, and</li>
<li><code>port_a.h_outflow</code> for flow in reverse direction.</li>
</ul>
<p>
Moreover appropriate values shall be assigned to the following parameters:
</p>
<ul>
<li><code>dp_start</code> for a guess of the pressure drop</li>
<li><code>m_flow_small</code> for regularization of zero flow.</li>
</ul>
</html>"));
      end PartialTwoPortTransport;

      connector HeatPorts_a "HeatPort connector with filled, large icon to be used for vectors of HeatPorts (vector dimensions must be added after dragging)"
        extends Modelica.Thermal.HeatTransfer.Interfaces.HeatPort;
        annotation(
          defaultComponentName = "heatPorts_a",
          Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-200, -50}, {200, 50}}, initialScale = 0.2), graphics = {Rectangle(extent = {{-201, 50}, {200, -50}}, lineColor = {127, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-171, 45}, {-83, -45}}, lineColor = {127, 0, 0}, fillColor = {127, 0, 0}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-45, 45}, {43, -45}}, lineColor = {127, 0, 0}, fillColor = {127, 0, 0}, fillPattern = FillPattern.Solid), Rectangle(extent = {{82, 45}, {170, -45}}, lineColor = {127, 0, 0}, fillColor = {127, 0, 0}, fillPattern = FillPattern.Solid)}));
      end HeatPorts_a;

      connector HeatPorts_b "HeatPort connector with filled, large icon to be used for vectors of HeatPorts (vector dimensions must be added after dragging)"
        extends Modelica.Thermal.HeatTransfer.Interfaces.HeatPort;
        annotation(
          defaultComponentName = "heatPorts_b",
          Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-200, -50}, {200, 50}}, initialScale = 0.2), graphics = {Rectangle(extent = {{-200, 50}, {200, -51}}, lineColor = {127, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-170, 44}, {-82, -46}}, lineColor = {127, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-44, 46}, {44, -44}}, lineColor = {127, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Rectangle(extent = {{82, 45}, {170, -45}}, lineColor = {127, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid)}));
      end HeatPorts_b;

      partial model PartialHeatTransfer "Common interface for heat transfer models"
        // Parameters
        replaceable package Medium = Modelica.Media.Interfaces.PartialMedium "Medium in the component" annotation(
          Dialog(tab = "Internal Interface", enable = false));
        parameter Integer n = 1 "Number of heat transfer segments" annotation(
          Dialog(tab = "Internal Interface", enable = false),
          Evaluate = true);
        // Inputs provided to heat transfer model
        input Medium.ThermodynamicState[n] states "Thermodynamic states of flow segments";
        input SI.Area[n] surfaceAreas "Heat transfer areas";
        // Outputs defined by heat transfer model
        output SI.HeatFlowRate[n] Q_flows "Heat flow rates";
        // Parameters
        parameter Boolean use_k = false "= true to use k value for thermal isolation" annotation(
          Dialog(tab = "Internal Interface", enable = false));
        parameter SI.CoefficientOfHeatTransfer k = 0 "Heat transfer coefficient to ambient" annotation(
          Dialog(group = "Ambient"),
          Evaluate = true);
        parameter SI.Temperature T_ambient = system.T_ambient "Ambient temperature" annotation(
          Dialog(group = "Ambient"));
        outer Modelica.Fluid.System system "System wide properties";
        // Heat ports
        Modelica.Fluid.Interfaces.HeatPorts_a[n] heatPorts "Heat port to component boundary" annotation(
          Placement(transformation(extent = {{-10, 60}, {10, 80}}), iconTransformation(extent = {{-20, 60}, {20, 80}})));
        // Variables
        SI.Temperature[n] Ts = Medium.temperature(states) "Temperatures defined by fluid states";
      equation
        if use_k then
          Q_flows = heatPorts.Q_flow + {k * surfaceAreas[i] * (T_ambient - heatPorts[i].T) for i in 1:n};
        else
          Q_flows = heatPorts.Q_flow;
        end if;
        annotation(
          Documentation(info = "<html>
<p>
This component is a common interface for heat transfer models. The heat flow rates <code>Q_flows[n]</code> through the boundaries of n flow segments
are obtained as function of the thermodynamic <code>states</code> of the flow segments for a given fluid <code>Medium</code>,
the <code>surfaceAreas[n]</code> and the boundary temperatures <code>heatPorts[n].T</code>.
</p>
<p>
The heat loss coefficient <code>k</code> can be used to model a thermal isolation between <code>heatPorts.T</code> and <code>T_ambient</code>.</p>
<p>
An extending model implementing this interface needs to define one equation: the relation between the predefined fluid temperatures <code>Ts[n]</code>,
the boundary temperatures <code>heatPorts[n].T</code>, and the heat flow rates <code>Q_flows[n]</code>.
</p>
</html>"));
      end PartialHeatTransfer;

      partial model PartialLumpedVolume "Lumped volume with mass and energy balance"
        import Modelica.Fluid.Types;
        import Modelica.Fluid.Types.Dynamics;
        import Modelica.Media.Interfaces.Choices.IndependentVariables;
        outer Modelica.Fluid.System system "System properties";
        replaceable package Medium = Modelica.Media.Interfaces.PartialMedium "Medium in the component" annotation(
          choicesAllMatching = true);
        // Inputs provided to the volume model
        input SI.Volume fluidVolume "Volume";
        // Assumptions
        parameter Types.Dynamics energyDynamics = system.energyDynamics "Formulation of energy balance" annotation(
          Evaluate = true,
          Dialog(tab = "Assumptions", group = "Dynamics"));
        parameter Types.Dynamics massDynamics = system.massDynamics "Formulation of mass balance" annotation(
          Evaluate = true,
          Dialog(tab = "Assumptions", group = "Dynamics"));
        final parameter Types.Dynamics substanceDynamics = massDynamics "Formulation of substance balance" annotation(
          Evaluate = true,
          Dialog(tab = "Assumptions", group = "Dynamics"));
        final parameter Types.Dynamics traceDynamics = massDynamics "Formulation of trace substance balance" annotation(
          Evaluate = true,
          Dialog(tab = "Assumptions", group = "Dynamics"));
        // Initialization
        parameter Medium.AbsolutePressure p_start = system.p_start "Start value of pressure" annotation(
          Dialog(tab = "Initialization"));
        parameter Boolean use_T_start = true "= true, use T_start, otherwise h_start" annotation(
          Dialog(tab = "Initialization"),
          Evaluate = true);
        parameter Medium.Temperature T_start = if use_T_start then system.T_start else Medium.temperature_phX(p_start, h_start, X_start) "Start value of temperature" annotation(
          Dialog(tab = "Initialization", enable = use_T_start));
        parameter Medium.SpecificEnthalpy h_start = if use_T_start then Medium.specificEnthalpy_pTX(p_start, T_start, X_start) else Medium.h_default "Start value of specific enthalpy" annotation(
          Dialog(tab = "Initialization", enable = not use_T_start));
        parameter Medium.MassFraction X_start[Medium.nX] = Medium.X_default "Start value of mass fractions m_i/m" annotation(
          Dialog(tab = "Initialization", enable = Medium.nXi > 0));
        parameter Medium.ExtraProperty C_start[Medium.nC](quantity = Medium.extraPropertiesNames) = fill(0, Medium.nC) "Start value of trace substances" annotation(
          Dialog(tab = "Initialization", enable = Medium.nC > 0));
        Medium.BaseProperties medium(preferredMediumStates = true, p(start = p_start), h(start = h_start), T(start = T_start), Xi(start = X_start[1:Medium.nXi]));
        SI.Energy U "Internal energy of fluid";
        SI.Mass m "Mass of fluid";
        SI.Mass[Medium.nXi] mXi "Masses of independent components in the fluid";
        SI.Mass[Medium.nC] mC "Masses of trace substances in the fluid";
        // C need to be added here because unlike for Xi, which has medium.Xi,
        // there is no variable medium.C
        Medium.ExtraProperty C[Medium.nC] "Trace substance mixture content";
        // variables that need to be defined by an extending class
        SI.MassFlowRate mb_flow "Mass flows across boundaries";
        SI.MassFlowRate[Medium.nXi] mbXi_flow "Substance mass flows across boundaries";
        Medium.ExtraPropertyFlowRate[Medium.nC] mbC_flow "Trace substance mass flows across boundaries";
        SI.EnthalpyFlowRate Hb_flow "Enthalpy flow across boundaries or energy source/sink";
        SI.HeatFlowRate Qb_flow "Heat flow across boundaries or energy source/sink";
        SI.Power Wb_flow "Work flow across boundaries or source term";
      protected
        parameter Boolean initialize_p = not Medium.singleState "= true to set up initial equations for pressure";
        Real[Medium.nC] mC_scaled(min = fill(Modelica.Constants.eps, Medium.nC)) "Scaled masses of trace substances in the fluid";
      equation
        assert(not (energyDynamics <> Dynamics.SteadyState and massDynamics == Dynamics.SteadyState) or Medium.singleState, "Bad combination of dynamics options and Medium not conserving mass if fluidVolume is fixed.");
// Total quantities
        m = fluidVolume * medium.d;
        mXi = m * medium.Xi;
        U = m * medium.u;
        mC = m * C;
// Energy and mass balances
        if energyDynamics == Dynamics.SteadyState then
          0 = Hb_flow + Qb_flow + Wb_flow;
        else
          der(U) = Hb_flow + Qb_flow + Wb_flow;
        end if;
        if massDynamics == Dynamics.SteadyState then
          0 = mb_flow;
        else
          der(m) = mb_flow;
        end if;
        if substanceDynamics == Dynamics.SteadyState then
          zeros(Medium.nXi) = mbXi_flow;
        else
          der(mXi) = mbXi_flow;
        end if;
        if traceDynamics == Dynamics.SteadyState then
          zeros(Medium.nC) = mbC_flow;
        else
          der(mC_scaled) = mbC_flow ./ Medium.C_nominal;
        end if;
        mC = mC_scaled .* Medium.C_nominal;
      initial equation
// initialization of balances
        if energyDynamics == Dynamics.FixedInitial then
/*
    if use_T_start then
      medium.T = T_start;
    else
      medium.h = h_start;
    end if;
    */
          if Medium.ThermoStates == IndependentVariables.ph or Medium.ThermoStates == IndependentVariables.phX then
            medium.h = h_start;
          else
            medium.T = T_start;
          end if;
        elseif energyDynamics == Dynamics.SteadyStateInitial then
/*
    if use_T_start then
      der(medium.T) = 0;
    else
      der(medium.h) = 0;
    end if;
    */
          if Medium.ThermoStates == IndependentVariables.ph or Medium.ThermoStates == IndependentVariables.phX then
            der(medium.h) = 0;
          else
            der(medium.T) = 0;
          end if;
        end if;
        if massDynamics == Dynamics.FixedInitial then
          if initialize_p then
            medium.p = p_start;
          end if;
        elseif massDynamics == Dynamics.SteadyStateInitial then
          if initialize_p then
            der(medium.p) = 0;
          end if;
        end if;
        if substanceDynamics == Dynamics.FixedInitial then
          medium.Xi = X_start[1:Medium.nXi];
        elseif substanceDynamics == Dynamics.SteadyStateInitial then
          der(medium.Xi) = zeros(Medium.nXi);
        end if;
        if traceDynamics == Dynamics.FixedInitial then
          mC_scaled = m * C_start[1:Medium.nC] ./ Medium.C_nominal;
        elseif traceDynamics == Dynamics.SteadyStateInitial then
          der(mC_scaled) = zeros(Medium.nC);
        end if;
        annotation(
          Documentation(info = "<html>
<p>
Interface and base class for an ideally mixed fluid volume with the ability to store mass and energy.
The following boundary flow and source terms are part of the energy balance and must be specified in an extending class:
</p>
<ul>
<li><code><b>Qb_flow</b></code>, e.g., convective or latent heat flow rate across segment boundary, and</li>
<li><code><b>Wb_flow</b></code>, work term, e.g., p*der(fluidVolume) if the volume is not constant.</li>
</ul>
<p>
The component volume <code><b>fluidVolume</b></code> is an input that needs to be set in the extending class to complete the model.
</p>
<p>
Further source terms must be defined by an extending class for fluid flow across the segment boundary:
</p>
<ul>
<li><code><b>Hb_flow</b></code>, enthalpy flow,</li>
<li><code><b>mb_flow</b></code>, mass flow,</li>
<li><code><b>mbXi_flow</b></code>, substance mass flow, and</li>
<li><code><b>mbC_flow</b></code>, trace substance mass flow.</li>
</ul>
</html>"));
      end PartialLumpedVolume;

      partial model PartialLumpedFlow "Base class for a lumped momentum balance"
        outer Modelica.Fluid.System system "System properties";
        replaceable package Medium = Modelica.Media.Interfaces.PartialMedium "Medium in the component";
        parameter Boolean allowFlowReversal = system.allowFlowReversal "= true to allow flow reversal, false restricts to design direction (m_flow >= 0)" annotation(
          Dialog(tab = "Assumptions"),
          Evaluate = true);
        // Inputs provided to the flow model
        input SI.Length pathLength "Length flow path";
        // Variables defined by the flow model
        Medium.MassFlowRate m_flow(min = if allowFlowReversal then -Modelica.Constants.inf else 0, start = m_flow_start, stateSelect = if momentumDynamics == Types.Dynamics.SteadyState then StateSelect.default else StateSelect.prefer) "mass flow rates between states";
        // Parameters
        parameter Modelica.Fluid.Types.Dynamics momentumDynamics = system.momentumDynamics "Formulation of momentum balance" annotation(
          Dialog(tab = "Assumptions", group = "Dynamics"),
          Evaluate = true);
        parameter Medium.MassFlowRate m_flow_start = system.m_flow_start "Start value of mass flow rates" annotation(
          Dialog(tab = "Initialization"));
        // Total quantities
        SI.Momentum I "Momenta of flow segments";
        // Source terms and forces to be defined by an extending model (zero if not used)
        SI.Force Ib_flow "Flow of momentum across boundaries";
        SI.Force F_p "Pressure force";
        SI.Force F_fg "Friction and gravity force";
      equation
// Total quantities
        I = m_flow * pathLength;
// Momentum balances
        if momentumDynamics == Types.Dynamics.SteadyState then
          0 = Ib_flow - F_p - F_fg;
        else
          der(I) = Ib_flow - F_p - F_fg;
        end if;
      initial equation
        if momentumDynamics == Types.Dynamics.FixedInitial then
          m_flow = m_flow_start;
        elseif momentumDynamics == Types.Dynamics.SteadyStateInitial then
          der(m_flow) = 0;
        end if;
        annotation(
          Documentation(info = "<html>
<p>
Interface and base class for a momentum balance, defining the mass flow rate <code><b>m_flow</b></code>
of a given <code>Medium</code> in a flow model.
</p>
<p>
The following boundary flow and force terms are part of the momentum balance and must be specified in an extending model (to zero if not considered):
</p>
<ul>
<li><code><b>Ib_flow</b></code>, the flow of momentum across model boundaries,</li>
<li><code><b>F_p[m]</b></code>, pressure force, and</li>
<li><code><b>F_fg[m]</b></code>, friction and gravity forces.</li>
</ul>
<p>
The length of the flow path <code><b>pathLength</b></code> is an input that needs to be set in an extending class to complete the model.
</p>
</html>"));
      end PartialLumpedFlow;

      partial model PartialDistributedVolume "Base class for distributed volume models"
        import Modelica.Fluid.Types;
        import Modelica.Fluid.Types.Dynamics;
        import Modelica.Media.Interfaces.Choices.IndependentVariables;
        outer Modelica.Fluid.System system "System properties";
        replaceable package Medium = Modelica.Media.Interfaces.PartialMedium "Medium in the component" annotation(
          choicesAllMatching = true);
        // Discretization
        parameter Integer n = 2 "Number of discrete volumes";
        // Inputs provided to the volume model
        input SI.Volume[n] fluidVolumes "Discretized volume, determine in inheriting class";
        // Assumptions
        parameter Types.Dynamics energyDynamics = system.energyDynamics "Formulation of energy balances" annotation(
          Evaluate = true,
          Dialog(tab = "Assumptions", group = "Dynamics"));
        parameter Types.Dynamics massDynamics = system.massDynamics "Formulation of mass balances" annotation(
          Evaluate = true,
          Dialog(tab = "Assumptions", group = "Dynamics"));
        final parameter Types.Dynamics substanceDynamics = massDynamics "Formulation of substance balances" annotation(
          Evaluate = true,
          Dialog(tab = "Assumptions", group = "Dynamics"));
        final parameter Types.Dynamics traceDynamics = massDynamics "Formulation of trace substance balances" annotation(
          Evaluate = true,
          Dialog(tab = "Assumptions", group = "Dynamics"));
        //Initialization
        parameter Medium.AbsolutePressure p_a_start = system.p_start "Start value of pressure at port a" annotation(
          Dialog(tab = "Initialization"));
        parameter Medium.AbsolutePressure p_b_start = p_a_start "Start value of pressure at port b" annotation(
          Dialog(tab = "Initialization"));
        final parameter Medium.AbsolutePressure[n] ps_start = if n > 1 then linspace(p_a_start, p_b_start, n) else {(p_a_start + p_b_start) / 2} "Start value of pressure";
        parameter Boolean use_T_start = true "Use T_start if true, otherwise h_start" annotation(
          Evaluate = true,
          Dialog(tab = "Initialization"));
        parameter Medium.Temperature T_start = if use_T_start then system.T_start else Medium.temperature_phX((p_a_start + p_b_start) / 2, h_start, X_start) "Start value of temperature" annotation(
          Evaluate = true,
          Dialog(tab = "Initialization", enable = use_T_start));
        parameter Medium.SpecificEnthalpy h_start = if use_T_start then Medium.specificEnthalpy_pTX((p_a_start + p_b_start) / 2, T_start, X_start) else Medium.h_default "Start value of specific enthalpy" annotation(
          Evaluate = true,
          Dialog(tab = "Initialization", enable = not use_T_start));
        parameter Medium.MassFraction X_start[Medium.nX] = Medium.X_default "Start value of mass fractions m_i/m" annotation(
          Dialog(tab = "Initialization", enable = Medium.nXi > 0));
        parameter Medium.ExtraProperty C_start[Medium.nC](quantity = Medium.extraPropertiesNames) = fill(0, Medium.nC) "Start value of trace substances" annotation(
          Dialog(tab = "Initialization", enable = Medium.nC > 0));
        // Total quantities
        SI.Energy[n] Us "Internal energy of fluid";
        SI.Mass[n] ms "Fluid mass";
        SI.Mass[n, Medium.nXi] mXis "Substance mass";
        SI.Mass[n, Medium.nC] mCs "Trace substance mass";
        // C need to be added here because unlike for Xi, which has medium[:].Xi,
        // there is no variable medium[:].C
        SI.Mass[n, Medium.nC] mCs_scaled "Scaled trace substance mass";
        Medium.ExtraProperty Cs[n, Medium.nC] "Trace substance mixture content";
        Medium.BaseProperties[n] mediums(each preferredMediumStates = true, p(start = ps_start), each h(start = h_start), each T(start = T_start), each Xi(start = X_start[1:Medium.nXi]));
        //Source terms, have to be defined by an extending model (to zero if not used)
        Medium.MassFlowRate[n] mb_flows "Mass flow rate, source or sink";
        Medium.MassFlowRate[n, Medium.nXi] mbXi_flows "Independent mass flow rates, source or sink";
        Medium.ExtraPropertyFlowRate[n, Medium.nC] mbC_flows "Trace substance mass flow rates, source or sink";
        SI.EnthalpyFlowRate[n] Hb_flows "Enthalpy flow rate, source or sink";
        SI.HeatFlowRate[n] Qb_flows "Heat flow rate, source or sink";
        SI.Power[n] Wb_flows "Mechanical power, p*der(V) etc.";
      protected
        parameter Boolean initialize_p = not Medium.singleState "= true to set up initial equations for pressure";
      equation
        assert(not (energyDynamics <> Dynamics.SteadyState and massDynamics == Dynamics.SteadyState) or Medium.singleState, "Bad combination of dynamics options and Medium not conserving mass if fluidVolumes are fixed.");
// Total quantities
        for i in 1:n loop
          ms[i] = fluidVolumes[i] * mediums[i].d;
          mXis[i, :] = ms[i] * mediums[i].Xi;
          mCs[i, :] = ms[i] * Cs[i, :];
          Us[i] = ms[i] * mediums[i].u;
        end for;
// Energy and mass balances
        if energyDynamics == Dynamics.SteadyState then
          for i in 1:n loop
            0 = Hb_flows[i] + Wb_flows[i] + Qb_flows[i];
          end for;
        else
          for i in 1:n loop
            der(Us[i]) = Hb_flows[i] + Wb_flows[i] + Qb_flows[i];
          end for;
        end if;
        if massDynamics == Dynamics.SteadyState then
          for i in 1:n loop
            0 = mb_flows[i];
          end for;
        else
          for i in 1:n loop
            der(ms[i]) = mb_flows[i];
          end for;
        end if;
        if substanceDynamics == Dynamics.SteadyState then
          for i in 1:n loop
            zeros(Medium.nXi) = mbXi_flows[i, :];
          end for;
        else
          for i in 1:n loop
            der(mXis[i, :]) = mbXi_flows[i, :];
          end for;
        end if;
        if traceDynamics == Dynamics.SteadyState then
          for i in 1:n loop
            zeros(Medium.nC) = mbC_flows[i, :];
          end for;
        else
          for i in 1:n loop
            der(mCs_scaled[i, :]) = mbC_flows[i, :] ./ Medium.C_nominal;
            mCs[i, :] = mCs_scaled[i, :] .* Medium.C_nominal;
          end for;
        end if;
      initial equation
// initialization of balances
        if energyDynamics == Dynamics.FixedInitial then
/*
    if use_T_start then
      mediums.T = fill(T_start, n);
    else
      mediums.h = fill(h_start, n);
    end if;
    */
          if Medium.ThermoStates == IndependentVariables.ph or Medium.ThermoStates == IndependentVariables.phX then
            mediums.h = fill(h_start, n);
          else
            mediums.T = fill(T_start, n);
          end if;
        elseif energyDynamics == Dynamics.SteadyStateInitial then
/*
    if use_T_start then
      der(mediums.T) = zeros(n);
    else
      der(mediums.h) = zeros(n);
    end if;
    */
          if Medium.ThermoStates == IndependentVariables.ph or Medium.ThermoStates == IndependentVariables.phX then
            der(mediums.h) = zeros(n);
          else
            der(mediums.T) = zeros(n);
          end if;
        end if;
        if massDynamics == Dynamics.FixedInitial then
          if initialize_p then
            mediums.p = ps_start;
          end if;
        elseif massDynamics == Dynamics.SteadyStateInitial then
          if initialize_p then
            der(mediums.p) = zeros(n);
          end if;
        end if;
        if substanceDynamics == Dynamics.FixedInitial then
          mediums.Xi = fill(X_start[1:Medium.nXi], n);
        elseif substanceDynamics == Dynamics.SteadyStateInitial then
          for i in 1:n loop
            der(mediums[i].Xi) = zeros(Medium.nXi);
          end for;
        end if;
        if traceDynamics == Dynamics.FixedInitial then
          Cs = fill(C_start[1:Medium.nC], n);
        elseif traceDynamics == Dynamics.SteadyStateInitial then
          for i in 1:n loop
            der(mCs[i, :]) = zeros(Medium.nC);
          end for;
        end if;
        annotation(
          Documentation(info = "<html>
<p>
Interface and base class for <code><b>n</b></code> ideally mixed fluid volumes with the ability to store mass and energy.
It is intended to model a one-dimensional spatial discretization of fluid flow according to the finite volume method.
The following boundary flow and source terms are part of the energy balance and must be specified in an extending class:
</p>
<ul>
<li><code><b>Qb_flows[n]</b></code>, heat flow term, e.g., conductive heat flows across segment boundaries, and</li>
<li><code><b>Wb_flows[n]</b></code>, work term.</li>
</ul>
<p>
The component volumes <code><b>fluidVolumes[n]</b></code> are an input that needs to be set in an extending class to complete the model.
</p>
<p>
Further source terms must be defined by an extending class for fluid flow across the segment boundary:
</p>
<ul>
<li><code><b>Hb_flows[n]</b></code>, enthalpy flow,</li>
<li><code><b>mb_flows[n]</b></code>, mass flow,</li>
<li><code><b>mbXi_flows[n]</b></code>, substance mass flow, and</li>
<li><code><b>mbC_flows[n]</b></code>, trace substance mass flow.</li>
</ul>
</html>"));
      end PartialDistributedVolume;

      partial model PartialDistributedFlow "Base class for a distributed momentum balance"
        outer Modelica.Fluid.System system "System properties";
        replaceable package Medium = Modelica.Media.Interfaces.PartialMedium "Medium in the component";
        parameter Boolean allowFlowReversal = system.allowFlowReversal "= true to allow flow reversal, false restricts to design direction (m_flows >= zeros(m))" annotation(
          Dialog(tab = "Assumptions"),
          Evaluate = true);
        // Discretization
        parameter Integer m = 1 "Number of flow segments";
        // Inputs provided to the flow model
        input SI.Length[m] pathLengths "Lengths along flow path";
        // Variables defined by momentum model
        Medium.MassFlowRate[m] m_flows(each min = if allowFlowReversal then -Modelica.Constants.inf else 0, each start = m_flow_start, each stateSelect = if momentumDynamics == Types.Dynamics.SteadyState then StateSelect.default else StateSelect.prefer) "mass flow rates between states";
        // Parameters
        parameter Modelica.Fluid.Types.Dynamics momentumDynamics = system.momentumDynamics "Formulation of momentum balance" annotation(
          Dialog(tab = "Assumptions", group = "Dynamics"),
          Evaluate = true);
        parameter Medium.MassFlowRate m_flow_start = system.m_flow_start "Start value of mass flow rates" annotation(
          Dialog(tab = "Initialization"));
        // Total quantities
        SI.Momentum[m] Is "Momenta of flow segments";
        // Source terms and forces to be defined by an extending model (zero if not used)
        SI.Force[m] Ib_flows "Flow of momentum across boundaries";
        SI.Force[m] Fs_p "Pressure forces";
        SI.Force[m] Fs_fg "Friction and gravity forces";
      equation
// Total quantities
        Is = {m_flows[i] * pathLengths[i] for i in 1:m};
// Momentum balances
        if momentumDynamics == Types.Dynamics.SteadyState then
          zeros(m) = Ib_flows - Fs_p - Fs_fg;
        else
          der(Is) = Ib_flows - Fs_p - Fs_fg;
        end if;
      initial equation
        if momentumDynamics == Types.Dynamics.FixedInitial then
          m_flows = fill(m_flow_start, m);
        elseif momentumDynamics == Types.Dynamics.SteadyStateInitial then
          der(m_flows) = zeros(m);
        end if;
        annotation(
          Documentation(info = "<html>
<p>
Interface and base class for <code><b>m</b></code> momentum balances, defining the mass flow rates <code><b>m_flows[m]</b></code>
of a given <code>Medium</code> in <code><b>m</b></code> flow segments.
</p>
<p>
The following boundary flow and force terms are part of the momentum balances and must be specified in an extending model (to zero if not considered):
</p>
<ul>
<li><code><b>Ib_flows[m]</b></code>, the flows of momentum across segment boundaries,</li>
<li><code><b>Fs_p[m]</b></code>, pressure forces, and</li>
<li><code><b>Fs_fg[m]</b></code>, friction and gravity forces.</li>
</ul>
<p>
The lengths along the flow path <code><b>pathLengths[m]</b></code> are an input that needs to be set in an extending class to complete the model.
</p>
</html>"));
      end PartialDistributedFlow;

      partial model PartialPressureLoss "Base flow model for pressure loss functions with the same area at port_a and at port_b"
        extends Modelica.Fluid.Interfaces.PartialTwoPortTransport;
      protected
        parameter Medium.ThermodynamicState state_dp_small = Medium.setState_pTX(Medium.reference_p, Medium.reference_T, Medium.reference_X) "Medium state to compute dp_small";
        Medium.Density d_a "Density at port_a when fluid is flowing from port_a to port_b";
        Medium.Density d_b "If allowFlowReversal=true then Density at port_b when fluid is flowing from port_b to port_a else d_a";
        Medium.DynamicViscosity eta_a "Dynamic viscosity at port_a when fluid is flowing from port_a to port_b";
        Medium.DynamicViscosity eta_b "If allowFlowReversal=true then Dynamic viscosity at port_b when fluid is flowing from port_b to port_a else eta_a";
      equation
// Isenthalpic state transformation (no storage and no loss of energy)
        port_a.h_outflow = inStream(port_b.h_outflow);
        port_b.h_outflow = inStream(port_a.h_outflow);
// Medium properties
        d_a = Medium.density(state_a);
        eta_a = Medium.dynamicViscosity(state_a);
        if allowFlowReversal then
          d_b = Medium.density(state_b);
          eta_b = Medium.dynamicViscosity(state_b);
        else
          d_b = d_a;
          eta_b = eta_a;
        end if;
      end PartialPressureLoss;
      annotation(
        Documentation(info = "<html>

</html>", revisions = "<html>
<ul>
<li><i>June 9th, 2008</i>
       by Michael Sielemann: Introduced stream keyword after decision at 57th Design Meeting (Lund).</li>
<li><i>May 30, 2007</i>
       by Christoph Richter: moved everything back to its original position in Modelica.Fluid.</li>
<li><i>Apr. 20, 2007</i>
       by Christoph Richter: moved parts of the original package from Modelica.Fluid
       to the development branch of Modelica 2.2.2.</li>
<li><i>Nov. 2, 2005</i>
       by Francesco Casella: restructured after 45th Design Meeting.</li>
<li><i>Nov. 20-21, 2002</i>
       by Hilding Elmqvist, Mike Tiller, Allan Watson, John Batteh, Chuck Newman,
       Jonas Eborn: Improved at the 32nd Modelica Design Meeting.
<li><i>Nov. 11, 2002</i>
       by Hilding Elmqvist, Martin Otter: improved version.</li>
<li><i>Nov. 6, 2002</i>
       by Hilding Elmqvist: first version.</li>
<li><i>Aug. 11, 2002</i>
       by Martin Otter: Improved according to discussion with Hilding
       Elmqvist and Hubertus Tummescheit.<br>
       The PortVicinity model is manually
       expanded in the base models.<br>
       The Volume used for components is renamed
       PartialComponentVolume.<br>
       A new volume model \"Fluid.Components.PortVolume\"
       introduced that has the medium properties of the port to which it is
       connected.<br>
       Fluid.Interfaces.PartialTwoPortTransport is a component
       for elementary two port transport elements, whereas PartialTwoPort
       is a component for a container component.</li>
</ul>
</html>"));
    end Interfaces;

    package Types "Common types for fluid models"
      extends Modelica.Icons.TypesPackage;
      type HydraulicConductance = Modelica.Icons.TypeReal(final quantity = "HydraulicConductance", final unit = "kg/(s.Pa)") "Real type for hydraulic conductance";
      type HydraulicResistance = Modelica.Icons.TypeReal(final quantity = "HydraulicResistance", final unit = "Pa.s/kg") "Real type for hydraulic resistance";
      type Roughness = Modelica.Icons.TypeReal(final quantity = "Length", final unit = "m", displayUnit = "mm") "Real type for roughness of a pipe" annotation(
        Documentation(info = "<html>
<p>
This Real type defines the absolute roughness of the inner surface of a
pipe or fitting, i.e., the absolute average height of surface asperities.
It has usually to
be estimated. In <i>[Idelchik 1994, pp. 105-109,
Table 2-5; Miller 1990, p. 190, Table 8-1]</i> many examples are given.
As a short summary:
</p>

<table border=1 cellspacing=0 cellpadding=2>
  <tr><td colspan=\"2\" align=\"center\"><b>Type of pipe</b></td>
      <td align=\"center\">Roughness</td></tr>

  <tr><td><b>Smooth pipes</b></td>
      <td>Drawn brass, copper, aluminium, glass, etc.</td>
      <td> 0.0025 mm</td>
  </tr>
  <tr><td rowspan=\"3\"><b>Steel pipes</b></td>
      <td>New smooth pipes</td>
      <td>0.025 mm</td>
  </tr>
  <tr><td>Mortar lined, average finish</td>
      <td>0.1 mm</td>
  </tr>
  <tr><td>Heavy rust</td>
      <td>1 mm</td>
  </tr>
  <tr><td rowspan=\"3\"><b>Concrete pipes</b></td>
      <td>Steel forms, first class workmanship</td>
      <td>0.025 mm</td>
  </tr>
  <tr><td>Steel forms, average workmanship</td>
      <td>0.1 mm</td>
  </tr>
  <tr><td>Block linings</td>
      <td>1 mm</td>
  </tr>
</table>

<h4>References</h4>

<dl>
    <dt>Idelchik I.E. (1994):</dt>
    <dd><a href=\"http://www.bookfinder.com/dir/i/Handbook_of_Hydraulic_Resistance/0849399084/\"><b>Handbook
        of Hydraulic Resistance</b></a>. 3rd edition, Begell House, ISBN
        0-8493-9908-4</dd>
    <dt>Miller D. S. (1990):</dt>
    <dd><b>Internal flow systems</b>.
    2nd edition. Cranfield:BHRA(Information Services).</dd>
</dl>
</html>"));
      type Dynamics = enumeration(DynamicFreeInitial "DynamicFreeInitial -- Dynamic balance, Initial guess value", FixedInitial "FixedInitial -- Dynamic balance, Initial value fixed", SteadyStateInitial "SteadyStateInitial -- Dynamic balance, Steady state initial with guess value", SteadyState "SteadyState -- Steady state balance, Initial guess value") "Enumeration to define definition of balance equations" annotation(
        Documentation(info = "<html>
<p>
Enumeration to define the formulation of balance equations
(to be selected via choices menu):
</p>

<table border=1 cellspacing=0 cellpadding=2>
<tr><th><b>Dynamics.</b></th><th><b>Meaning</b></th></tr>
<tr><td>DynamicFreeInitial</td><td>Dynamic balance, Initial guess value</td></tr>

<tr><td>FixedInitial</td><td>Dynamic balance, Initial value fixed</td></tr>

<tr><td>SteadyStateInitial</td><td>Dynamic balance, Steady state initial with guess value</td></tr>

<tr><td>SteadyState</td><td>Steady state balance, Initial guess value</td></tr>
</table>

<p>
The enumeration \"Dynamics\" is used for the mass, energy and momentum balance equations
respectively. The exact meaning for the three balance equations is stated in the following
tables:
</p>

<table border=1 cellspacing=0 cellpadding=2>
<tr><td colspan=\"3\"><b>Mass balance</b> </td>
<tr><td><b>Dynamics.</b></td>
    <td><b>Balance equation</b></td>
    <td><b>Initial condition</b></td></tr>

<tr><td> DynamicFreeInitial</td>
    <td> no restrictions </td>
    <td> no initial conditions </td></tr>

<tr><td> FixedInitial</td>
    <td> no restrictions </td>
    <td> <b>if</b> Medium.singleState <b>then</b> <br>
         &nbsp;&nbsp;no initial condition<br>
         <b>else</b> p=p_start </td></tr>

<tr><td> SteadyStateInitial</td>
    <td> no restrictions </td>
    <td> <b>if</b> Medium.singleState <b>then</b> <br>
         &nbsp;&nbsp;no initial condition<br>
         <b>else</b> <b>der</b>(p)=0 </td></tr>

<tr><td> SteadyState</td>
    <td> <b>der</b>(m)=0  </td>
    <td> no initial conditions </td></tr>
</table>

&nbsp;<br>

<table border=1 cellspacing=0 cellpadding=2>
<tr><td colspan=\"3\"><b>Energy balance</b> </td>
<tr><td><b>Dynamics.</b></td>
    <td><b>Balance equation</b></td>
    <td><b>Initial condition</b></td></tr>

<tr><td> DynamicFreeInitial</td>
    <td> no restrictions </td>
    <td> no initial conditions </td></tr>

<tr><td> FixedInitial</td>
    <td> no restrictions </td>
    <td> T=T_start or h=h_start </td></tr>

<tr><td> SteadyStateInitial</td>
    <td> no restrictions </td>
    <td> <b>der</b>(T)=0 or <b>der</b>(h)=0 </td></tr>

<tr><td> SteadyState</td>
    <td> <b>der</b>(U)=0  </td>
    <td> no initial conditions </td></tr>
</table>

&nbsp;<br>

<table border=1 cellspacing=0 cellpadding=2>
<tr><td colspan=\"3\"><b>Momentum balance</b> </td>
<tr><td><b>Dynamics.</b></td>
    <td><b>Balance equation</b></td>
    <td><b>Initial condition</b></td></tr>

<tr><td> DynamicFreeInitial</td>
    <td> no restrictions </td>
    <td> no initial conditions </td></tr>

<tr><td> FixedInitial</td>
    <td> no restrictions </td>
    <td> m_flow = m_flow_start </td></tr>

<tr><td> SteadyStateInitial</td>
    <td> no restrictions </td>
    <td> <b>der</b>(m_flow)=0 </td></tr>

<tr><td> SteadyState</td>
    <td> <b>der</b>(m_flow)=0 </td>
    <td> no initial conditions </td></tr>
</table>

<p>
In the tables above, the equations are given for one-substance fluids. For multiple-substance
fluids and for trace substances, equivalent equations hold.
</p>

<p>
Medium.singleState is a medium property and defines whether the medium is only
described by one state (+ the mass fractions in case of a multi-substance fluid). In such
a case one initial condition less must be provided. For example, incompressible
media have Medium.singleState = <b>true</b>.
</p>

</html>"));
      type CvTypes = enumeration(Av "Av (metric) flow coefficient", Kv "Kv (metric) flow coefficient", Cv "Cv (US) flow coefficient", OpPoint "Av defined by operating point") "Enumeration to define the choice of valve flow coefficient" annotation(
        Documentation(info = "<html>

<p>
Enumeration to define the choice of valve flow coefficient
(to be selected via choices menu):
</p>

<table border=1 cellspacing=0 cellpadding=2>
<tr><th><b>CvTypes.</b></th>
    <th><b>Meaning</b></th></tr>

<tr><td>Av</td>
    <td>Av (metric) flow coefficient</td></tr>

<tr><td>Kv</td>
    <td>Kv (metric) flow coefficient</td></tr>

<tr><td>Cv</td>
    <td>Cv (US) flow coefficient</td></tr>

<tr><td>OpPoint</td>
    <td>Av defined by operating point</td></tr>

</table>

<p>
The details of the coefficients are explained in the
<a href=\"modelica://Modelica.Fluid.UsersGuide.ComponentDefinition.ValveCharacteristics\">
   User's Guide </a>.
</p>

</html>"));
      type PortFlowDirection = enumeration(Entering "Fluid flow is only entering", Leaving "Fluid flow is only leaving", Bidirectional "No restrictions on fluid flow (flow reversal possible)") "Enumeration to define whether flow reversal is allowed" annotation(
        Documentation(info = "<html>

<p>
Enumeration to define the assumptions on the model for the
direction of fluid flow at a port (to be selected via choices menu):
</p>

<table border=1 cellspacing=0 cellpadding=2>
<tr><th><b>PortFlowDirection.</b></th>
    <th><b>Meaning</b></th></tr>

<tr><td>Entering</td>
    <td>Fluid flow is only entering the port from the outside</td></tr>

<tr><td>Leaving</td>
    <td>Fluid flow is only leaving the port to the outside</td></tr>

<tr><td>Bidirectional</td>
    <td>No restrictions on fluid flow (flow reversal possible)</td></tr>
</table>

<p>
The default is \"PortFlowDirection.Bidirectional\". If you are completely sure that
the flow is only in one direction, then the other settings may
make the simulation of your model faster.
</p>

</html>"));
      type ModelStructure = enumeration(av_vb "av_vb: port_a - volume - flow model - volume - port_b", a_v_b "a_v_b: port_a - flow model - volume - flow model - port_b", av_b "av_b: port_a - volume - flow model - port_b", a_vb "a_vb: port_a - flow model - volume - port_b") "Enumeration with choices for model structure in distributed pipe model" annotation(
        Documentation(info = "<html>

<p>
Enumeration to define the discretization structure of
distributed pipe models according to the staggered grid scheme:
</p>

<table border=1 cellspacing=0 cellpadding=2>
<tr><th><b>ModelStructure.</b></th>
    <th><b>Meaning</b></th></tr>

<tr><td>av_vb</td>
    <td>port_a - volume - flow model - volume - port_b</td></tr>

<tr><td>a_v_b</td>
    <td>port_a - flow model - volume - flow model - port_b</td></tr>

<tr><td>av_b</td>
    <td>port_a - volume - flow model - port_b</td></tr>

<tr><td>a_vb</td>
    <td>port_a - flow model - volume - port_b</td></tr>
</table>

<p>
The default is \"ModelStructure.av_vb\", i.e., the distributed pipe
has \"volumes\" at its both ends. The advantage is that connections
of the pipe to flow models (like fittings) lead to the desirable structure
of alternating volume and flow models, which means that no non-linear
algebraic equations occur.
</p>

<p>
Direct connections of distributed pipes with
this option means that two volumes are directly connected together. Due to the
stream concept this means that the pressures of the two connected volumes
are identical, but the temperatures are not set equal
(this corresponds to volumes that are connected together with a very
short distance and it needs some time until different volume temperatures
are equilibrated). Since the pressures of the volumes are identical, the number
of states is reduced and index reduction takes place (which means that medium
equations depending on pressure are differentiated and the number of required
initial conditions is reduced by one).
</p>

<p>
The default option \"av_vb\" cannot be used, if the dynamic pipe is connected to a model with non-differentiable pressure, like a Sources.Boundary_pT with prescribed jumping pressure. The modelStructure can be configured as appropriate in such situations, in order to place a momentum balance between a pressure state of the pipe and a non-differentiable boundary condition
(e.g., if the jumping pressure component is connected to port_a, use model structure
ModelStructure.a_vb).
</p>

</html>"));
      annotation(
        preferredView = "info",
        Documentation(info = "<html>

</html>"));
    end Types;

    package Dissipation "Functions for convective heat transfer and pressure loss characteristics"
      extends Modelica.Icons.BasesPackage;
      import PI = Modelica.Constants.pi;
      import REC = Modelica.Fluid.Dissipation.Utilities.Records;
      import TYP = Modelica.Fluid.Dissipation.Utilities.Types;

      package UsersGuide "User's guide"
        extends Modelica.Icons.Information;

        class GettingStarted "Getting Started"
          extends Modelica.Icons.Information;
          annotation(
            Documentation(info = "<html>
<p>
The <b> Fluid.Dissipation </b> library provides convective heat transfer and pressure loss
(HTPL) correlations for a broad range of energy devices to build up thermohydraulic
energy systems.
</p>
<p>
This section introduces an implementation method for the integration of the provided HTPL
functions by Fluid.Dissipation into own application models. Additionally you can find
ready-to-use application models integrated into Modelica.Fluid as thermohydraulic
framework <a href=\"modelica://Modelica.Fluid.Fittings\"> (see
package Fittings)</a>.<br />
In the following the implementation method is described in 5 steps for a straight pipe as
example. Generally the implementation method can be used for all HTPL correlations
throughout the library in the same manner.
</p>

<h4>Step 1: Use/Create model with missing pressure loss correlation</h4>

<p>
All thermohydraulic systems using pressure loss calculations can be modelled for an <b>
incompressible case </b>, where the pressure loss (DP) is calculated in dependence of a
known mass flow rate (m_flow)
</p>
<pre>
   DP = f(m_flow,...)
</pre>
<p>
or a <b> compressible case </b> , where the mass flow rate (M_FLOW) is calculated in
dependence of a known pressure loss (dp)
</p>
<pre>
   M_FLOW = f(dp,...).
</pre>
<p>
In both cases one target variable (DP for the compressible or M_FLOW for the
incompressible case) is calculated as a function of the corresponding input variable
(m_flow or dp respectively). Both functions for these cases can be found in the library
for the pressure loss device of interest enlarged with a corresponding underscore
describing its intended use (functionname_MFLOW for compressible or functionname_DP for
incompressible calculation).
</p>
<p>
To create a simplified thermohydraulic model, the pressure loss (dp) and the mass flow
rate (M_FLOW) have to be defined as unknown variables and only a functional correlation
between them is still missing. Here the implementation for the compressible case of a
flow model will be explained as example.
</p>
<pre>
   model straightPipe
    //compressible case M_FLOW = f(dp)
     Modelica.SIunits.Pressure dp \"Input pressure loss\";
     Modelica.SIunits.MassFlowRate M_FLOW \"Output mass flow rate\";
   end straightPipe

   equation
  end straightPipe
</pre>

<h4>Step 2: Choose pressure loss <b> function </b> of interest</h4>

<p>
The HTPL correlations are modelled with functions for several devices. The pressure loss
of a straight pipe to be modelled can be found by browsing through the <b>
Fluid.Dissipation </b> library and looking up the function of interest, here:
</p>
<pre>
   Fluid.Dissipation.PressureLoss.StraightPipe.dp_overall_MFLOW
</pre>

<p>
This HTPL correlation for the compressible case of a straight pipe have to be dragged and
dropped in the equation section of the <b> equation layer </b> of the model in Step 1.
</p>
<pre>
   model straightPipe
    //compressible case M_FLOW = f(dp)
     Modelica.SIunits.Pressure dp \"Input pressure loss\";
     Modelica.SIunits.MassFlowRate M_FLOW \"Output mass flow rate\";

    equation
    Fluid.Dissipation.PressureLoss.StraightPipe.dp_overall_<b>MFLOW</b>
   end straightPipe
</pre>

<h4>Step 3: Choose corresponding pressure loss <b> records </b>
</h4>
<p>
The chosen function in Step 2 still needs its corresponding input values provided by
records. These input records are spitted into one for input parameters (e.g., for
geometry) and one for input variables (e.g., for fluid properties). The name of these
input records are identical with the corresponding function but with the extension <b>
_IN_con </b> for parameters and <b> _IN_var </b> for variables as input. These
corresponding input record for the chosen function have to be dragged and dropped on the
<b> diagram layer </b> of the model in Step 1.
</p>
<pre>
  Input parameter record:
Fluid.Dissipation.PressureLoss.StraightPipe.dp_overall<b>_IN_con</b> IN_con
  Input variable record:
Fluid.Dissipation.PressureLoss.StraightPipe.dp_overall<b>_IN_var</b> IN_var
</pre>
<p>
Now the equation layer of the model in Step 1 should look similar to the following
(without comments and annotation):
</p>
<pre>
  model straightPipe
   ...
   //records
   Fluid.Dissipation.PressureLoss.StraightPipe.dp_overall_IN_con <b>IN_con</b>;
   Fluid.Dissipation.PressureLoss.StraightPipe.dp_overall_IN_var <b>IN_var</b>;

   equation
   Fluid.Dissipation.PressureLoss.StraightPipe.dp_overall_MFLOW
  end straightPipe
</pre>

<h4>Step 4: Build function-record construction </h4>
<p>
Now the input record have to be assigned to the chosen function in the equation layer.
The resulting function-record implementation for the compressible case looks like the
following:
</p>
<pre>
model straightPipe
   ...
  equation
  //compressible case
  M_FLOW = Fluid.Dissipation.PressureLoss.StraightPipe.dp_overall_MFLOW(IN_con,IN_var,dp);
end straightPipe
</pre>

<p>
Here the compressible case for the unknown mass flow rate (M_FLOW) is calculated by the
known pressure difference (dp) out of the interfaces of the thermohydraulic framework and
the input records (IN_con,IN_var) provide data like geometry and fluid properties for
example.
</p>
<h4>Step 5: Assign record variables </h4>
<p>
In the last step the variables of the input records for the function have to be assigned.
The assignment of the record variables can either be done directly in the record on the
diagram layer or in the equation layer.
The assignment of the input record in the equation layer results into the following
model:
</p>
<pre>
model straightPipe
 ...
//compressible fluid flow
  //input record

Fluid.Dissipation.Examples.Applications.PressureLoss.BaseClasses.StraightPipe.Overall.Pres
sureLossInput_con
    IN_con(
    d_hyd=d_hyd,
    L=L,
    roughness=roughness,
    K=K);

Fluid.Dissipation.Examples.Applications.PressureLoss.BaseClasses.StraightPipe.Overall.Pres
sureLossInput_var
    IN_var(
    eta=eta,
    rho=rho);
 ...
end straight Pipe;
</pre>

<p>
If the implementation of a HTPL correlation is done in an existing application model, the
unknown variables out of Step 1 (M_FLOW and dp for compressible or DP and m_flow for
incompressible case) have to be adjusted to the model variables (typically the interface
variables). The implementation of HTPL correlation into <b> Modelica.Fluid </b> can be
found for <a href=\"modelica://Modelica.Fluid.Fittings\"> flow
models of several devices</a>.
</p>
 </html>"));
        end GettingStarted;

        class ReleaseNotes "Release notes"
          extends Modelica.Icons.ReleaseNotes;
          annotation(
            Documentation(info = "<html>
<h4>Version 1.0 Beta 4-6, 2010-01-12</h4>

<p>
Fluid.Dissipation was improved for the release as follows:
</p>

<ul>
  <li> Changed structure for input records of all heat transfer and pressure loss
functions:
    <ul>
      <li> Reduced amount of input records for compressible and incompressible functions as well as for their combinational one to improve usability of library.</li>
      <li> Splitting input records of one function into one with parameters (e.g., for geometry) and one with variables (e.g., fluid properties) to ease work of IDE-solver.</li>
    </ul>
  </li>
  <li> Improved Modelica.Fluid application models for available heat transfer and pressure
loss functions:
  <ul>
    <li> Flattened inheritance with one base flow model for all energy devices.</li>
    <li> Implemented smooth state of fluid density and dynamic viscosity for reverse flow.</li>
  </ul>
  </li>
<li> Adaption of complete library due to structure change.</li>
</ul>

<h4>Version 1.0 Beta 3, 2009-07-03</h4>

<p>
Fluid.Dissipation was improved for the release as follows:
</p>

<ul>
<li> Changed flow models structure: <br>
Now that a future feature for the automatic choice of using either a mass flow rate (compressible case) or a pressure loss (incompressible case) function for calculation is supported if implemented by IDE. Due to that no manual selection of a compressible or incompressible calculation in the Modelica.Fluid flow models is possible anymore. Therefore nonlinear equations will be created from the Modelica.Fluid flow models, if the future feature is not supported and the mass flow rate is known at a fluid port instead of the pressure loss.
</li>
<li> Changed structure and amount of records used as input for function calls due to
changed structure of flow model.
</li>
<li> Changed structure of function calls due to changed structure of flow model.
</li>
<li> Finished validation of all available heat transfer and pressure loss functions.
</li>
<li> Included scripts for verification of all available heat transfer and pressure loss functions.
</li>
</ul>

<h4>Version 1.0 Beta 2, 2009-04-22</h4>

<p>
Fluid.Dissipation was improved for the release as follows:
</p>

<ul>
<li> Support of analytical Jacobians at inverse calculation of heat transfer and pressure loss functions.
</li>
<li> Included Modelica.Fluid application models for available heat transfer and pressure loss functions.
</li>
<li> Adaption of complete library to Modelica Standard nomenclature.</li>
</ul>

<h4>Version 1.0 Beta 1, 2008-10-08</h4>

Initial release of Fluid.Dissipation.

</html>"));
        end ReleaseNotes;

        class Contact "Contact"
          extends Modelica.Icons.Contact;
          annotation(
            Documentation(info = "<html>
<dl>
<dt>Maintainer and co-author:</dt>
<dd>Stefan Wischhusen<br />
    XRG Simulation GmbH<br />
    Hamburg, Germany<br />
    email: <A HREF=\"mailto:wischhusen@xrg-simulation.de\">wischhusen@xrg-simulation.de</A></dd>
</dl>
<h4>  Acknowledgements </h4>
<p>
The following people contributed to the Modelica.Fluid.Dissipation library (alphabetical list):
J&ouml;rg Eiden, Ole Engel, Nina Peci, Sven Rutkowski, Thorben Vahlenkamp, Stefan
Wischhusen.
</p>
<p>
The development of the Modelica.Fluid.Dissipation library was founded within the ITEA research
project EuroSysLib-D by German Federal Ministry of Education and Research (promotional
reference 01IS07022B). The project is started in October 2007 and ended in June 2010.
</p>
</html>"));
        end Contact;
        annotation(
          DocumentationClass = true,
          Documentation(info = "<html>
</html>"));
      end UsersGuide;

      package HeatTransfer "Package for calculation of heat transfer"
        extends Modelica.Icons.VariantsPackage;

        package Channel
          extends Modelica.Icons.VariantsPackage;

          function kc_evenGapLaminar "Mean heat transfer coefficient of even gap | laminar flow regime | considering boundary layer development | heat transfer at ONE or BOTH sides | identical and constant wall temperatures"
            extends Modelica.Icons.Function;
            //SOURCE: VDI-Waermeatlas, 9th edition, Springer-Verlag, 2002, Section Gb 6-10
            //input records
            input Modelica.Fluid.Dissipation.HeatTransfer.Channel.kc_evenGapLaminar_IN_con IN_con "Input record for function kc_evenGapLaminar" annotation(
              Dialog(group = "Constant inputs"));
            input Modelica.Fluid.Dissipation.HeatTransfer.Channel.kc_evenGapLaminar_IN_var IN_var "Input record for function kc_evenGapLaminar" annotation(
              Dialog(group = "Variable inputs"));
            //output variables
            output SI.CoefficientOfHeatTransfer kc "Convective heat transfer coefficient" annotation(
              Dialog(group = "Output"));
            output SI.PrandtlNumber Pr "Prandtl number" annotation(
              Dialog(group = "Output"));
            output SI.ReynoldsNumber Re "Reynolds number" annotation(
              Dialog(group = "Output"));
            output SI.NusseltNumber Nu "Nusselt number" annotation(
              Dialog(group = "Output"));
            output Real failureStatus "0== boundary conditions fulfilled | 1== failure >> check if still meaningful results" annotation(
              Dialog(group = "Output"));
          protected
            type TYP = Modelica.Fluid.Dissipation.Utilities.Types.kc_evenGap;
            Real MIN = Modelica.Constants.eps;
            Real laminar = 2200 "Maximum Reynolds number of laminar flow regime";
            SI.Area A_cross = IN_con.s * IN_con.h "Cross sectional area of gap";
            SI.Diameter d_hyd = 2 * IN_con.s "Hydraulic diameter";
            Real prandtlMax = if IN_con.target == TYP.UndevOne then 10 else if IN_con.target == TYP.UndevBoth then 1000 else 0 "Maximum Prandtl number";
            Real prandtlMin = if IN_con.target == TYP.UndevOne or IN_con.target == TYP.UndevBoth then 0.1 else 0 "Minimum Prandtl number";
            SI.Velocity velocity = abs(IN_var.m_flow) / max(MIN, IN_var.rho * A_cross) "Mean velocity in gap";
            //failure status
            Real fstatus[2] "Check of expected boundary conditions";
            //Documentation
          algorithm
            Pr := abs(IN_var.eta * IN_var.cp / max(MIN, IN_var.lambda));
            Re := max(1, abs(IN_var.rho * velocity * d_hyd / max(MIN, IN_var.eta)));
            kc := Modelica.Fluid.Dissipation.HeatTransfer.Channel.kc_evenGapLaminar_KC(IN_con, IN_var);
            Nu := kc * d_hyd / max(MIN, IN_var.lambda);
//failure status
            fstatus[1] := if Re > laminar then 1 else 0;
            fstatus[2] := if IN_con.target == TYP.UndevOne or IN_con.target == TYP.UndevBoth then if Pr > prandtlMax or Pr < prandtlMin then 1 else 0 else 0;
            failureStatus := 0;
            for i in 1:size(fstatus, 1) loop
              if fstatus[i] == 1 then
                failureStatus := 1;
              end if;
            end for;
            annotation(
              Inline = false,
              Documentation(info = "<html>
<p>Calculation of the mean convective heat transfer coefficient <b>kc </b>for a laminar fluid flow through an even gap at different fluid flow and heat transfer situations. Note that additionally a failure status is observed in this function to check if the intended boundary conditions are fulfilled.
<a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.HeatTransfer.Channel.kc_evenGapLaminar\">See more information.</a> </p>
</html>"),
              smoothOrder(normallyConstant = IN_con) = 2);
          end kc_evenGapLaminar;

          function kc_evenGapLaminar_KC "Mean heat transfer coefficient of even gap | laminar flow regime | considering boundary layer development | heat transfer at ONE or BOTH sides | identical and constant wall temperatures"
            extends Modelica.Icons.Function;
            //SOURCE: VDI-Waermeatlas, 9th edition, Springer-Verlag, 2002, Section Gb 6-10
            //input records
            input Modelica.Fluid.Dissipation.HeatTransfer.Channel.kc_evenGapLaminar_IN_con IN_con "Input record for function kc_evenGapLaminar_KC" annotation(
              Dialog(group = "Constant inputs"));
            input Modelica.Fluid.Dissipation.HeatTransfer.Channel.kc_evenGapLaminar_IN_var IN_var "Input record for function kc_evenGapLaminar_KC" annotation(
              Dialog(group = "Variable inputs"));
            //output variables
            output SI.CoefficientOfHeatTransfer kc "Output for function kc_evenGapLaminar_KC";
          protected
            type TYP = Modelica.Fluid.Dissipation.Utilities.Types.kc_evenGap;
            Real MIN = Modelica.Constants.eps;
            SI.Area A_cross = max(MIN, IN_con.s * IN_con.h) "Cross sectional area of gap";
            SI.Diameter d_hyd = 2 * IN_con.s "Hydraulic diameter";
            SI.Velocity velocity = abs(IN_var.m_flow) / max(MIN, IN_var.rho * A_cross) "Mean velocity in gap";
            SI.ReynoldsNumber Re = max(1, IN_var.rho * velocity * d_hyd / max(MIN, IN_var.eta));
            SI.PrandtlNumber Pr = abs(IN_var.eta * IN_var.cp / max(MIN, IN_var.lambda));
            //variables for mean Nusselt number
            //SOURCE: p.Gb 7, eq. 36/37
            SI.NusseltNumber Nu_1 = if IN_con.target == TYP.DevOne or IN_con.target == TYP.UndevOne then 4.861 else if IN_con.target == TYP.DevBoth or IN_con.target == TYP.UndevBoth then 7.541 else 0 "First Nusselt number";
            //SOURCE: p.Gb 7, eq. 38
            SI.NusseltNumber Nu_2 = 1.841 * (Re * Pr * d_hyd / max(IN_con.L, MIN)) ^ (1 / 3) "Second Nusselt number";
            //SOURCE: p.Gb 7, eq. 42
            SI.NusseltNumber Nu_3 = if IN_con.target == TYP.UndevOne or IN_con.target == TYP.UndevBoth then (2 / (1 + 22 * Pr)) ^ (1 / 6) * (Re * Pr * d_hyd / max(IN_con.L, MIN)) ^ 0.5 else 0 "Third mean Nusselt number";
            SI.NusseltNumber Nu = (Nu_1 ^ 3 + Nu_2 ^ 3 + Nu_3 ^ 3) ^ (1 / 3);
            //Documentation
          algorithm
            kc := Nu * (IN_var.lambda / max(MIN, d_hyd));
            annotation(
              Inline = false,
              Documentation(info = "<html>
<p>
Calculation of the mean convective heat transfer coefficient <b> kc </b> for a laminar fluid flow through an even gap at different fluid flow and heat transfer situations.
<a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.HeatTransfer.Channel.kc_evenGapLaminar\">See more information.</a>
</p>
</html>"),
              smoothOrder(normallyConstant = IN_con) = 2);
          end kc_evenGapLaminar_KC;

          record kc_evenGapLaminar_IN_con "Input record for function kc_evenGapLaminar and kc_evenGapLaminar_KC"
            extends Modelica.Fluid.Dissipation.HeatTransfer.Channel.kc_evenGapOverall_IN_con;
            annotation(
              Documentation(info = "<html>
<p>
This record is used as <b> input record </b> for the heat transfer function <a href=\"modelica://Modelica.Fluid.Dissipation.HeatTransfer.Channel.kc_evenGapLaminar\"> kc_evenGapLaminar</a> and
<a href=\"modelica://Modelica.Fluid.Dissipation.HeatTransfer.Channel.kc_evenGapLaminar_KC\"> kc_evenGapLaminar_KC</a>.</p>
</html>"));
          end kc_evenGapLaminar_IN_con;

          record kc_evenGapLaminar_IN_var "Input record for function kc_evenGapLaminar and kc_evenGapLaminar_KC"
            extends Modelica.Fluid.Dissipation.HeatTransfer.Channel.kc_evenGapOverall_IN_var;
            annotation(
              Documentation(info = "<html>
<p>
This record is used as <b> input record </b> for the heat transfer function <a href=\"Modelica://Modelica.Fluid.Dissipation.HeatTransfer.Channel.kc_evenGapLaminar\"> kc_evenGapLaminar</a> and
<a href=\"modelica://Modelica.Fluid.Dissipation.HeatTransfer.Channel.kc_evenGapLaminar_KC\"> kc_evenGapLaminar_KC</a>.</p>
</html>"));
          end kc_evenGapLaminar_IN_var;

          function kc_evenGapOverall "Mean heat transfer coefficient of even gap | overall flow regime | considering boundary layer development | heat transfer at ONE or BOTH sides | identical and constant wall temperatures | surface roughness"
            extends Modelica.Icons.Function;
            //SOURCE: VDI-Waermeatlas, 9th edition, Springer-Verlag, 2002, Section Gb 6-10
            import MIN = Modelica.Constants.eps;
            // import SMOOTH = Modelica.Fluid.Dissipation.Utilities.Functions.Stepsmoother;
            //input records
            input Modelica.Fluid.Dissipation.HeatTransfer.Channel.kc_evenGapOverall_IN_con IN_con "Input record for function kc_evenGapOverall" annotation(
              Dialog(group = "Constant inputs"));
            input Modelica.Fluid.Dissipation.HeatTransfer.Channel.kc_evenGapOverall_IN_var IN_var "Input record for function kc_evenGapOverall" annotation(
              Dialog(group = "Variable inputs"));
            //output variables
            output SI.CoefficientOfHeatTransfer kc "Convective heat transfer coefficient" annotation(
              Dialog(group = "Output"));
            output SI.PrandtlNumber Pr "Prandtl number" annotation(
              Dialog(group = "Output"));
            output SI.ReynoldsNumber Re "Reynolds number" annotation(
              Dialog(group = "Output"));
            output SI.NusseltNumber Nu "Nusselt number" annotation(
              Dialog(group = "Output"));
            output Real failureStatus "0== boundary conditions fulfilled | 1== failure >> check if still meaningful results" annotation(
              Dialog(group = "Output"));
          protected
            type TYP = Modelica.Fluid.Dissipation.Utilities.Types.kc_evenGap;
            Real MIN = Modelica.Constants.eps;
            Real laminar = 2200 "Maximum Reynolds number for laminar regime";
            Real turbulent = 1e4 "Minimum Reynolds number for turbulent regime";
            SI.Area A_cross = IN_con.s * IN_con.h "Cross sectional area of gap";
            SI.Diameter d_hyd = 2 * IN_con.s "Hydraulic diameter";
            Real prandtlMax = if IN_con.target == TYP.UndevOne then 10 else if IN_con.target == TYP.UndevBoth then 1000 else 0 "Maximum Prandtl number";
            Real prandtlMin = if IN_con.target == TYP.UndevOne or IN_con.target == TYP.UndevBoth then 0.1 else 0 "Minimum Prandtl number";
            SI.Velocity velocity = abs(IN_var.m_flow) / max(MIN, IN_var.rho * A_cross) "Mean velocity in gap";
            //failure status
            Real fstatus[2] "Check of expected boundary conditions";
            //Documentation
          algorithm
            Pr := abs(IN_var.eta * IN_var.cp / max(MIN, IN_var.lambda));
            Re := max(1e-3, abs(IN_var.rho * velocity * d_hyd / max(MIN, IN_var.eta)));
            kc := Modelica.Fluid.Dissipation.HeatTransfer.Channel.kc_evenGapOverall_KC(IN_con, IN_var);
            Nu := kc * d_hyd / max(MIN, IN_var.lambda);
//failure status
            fstatus[1] := if IN_con.target == TYP.UndevOne or IN_con.target == TYP.UndevBoth then if Pr > prandtlMax or Pr < prandtlMin then 1 else 0 else 0;
            fstatus[2] := if d_hyd / IN_con.L > 1.0 then 1 else 0;
            failureStatus := 0;
            for i in 1:size(fstatus, 1) loop
              if fstatus[i] == 1 then
                failureStatus := 1;
              end if;
            end for;
            annotation(
              Inline = false,
              smoothOrder(normallyConstant = IN_con) = 2,
              Documentation(info = "<html>
<p>
Calculation of the mean convective heat transfer coefficient <b> kc </b> for an overall fluid flow through an even gap at different fluid flow and heat transfer situations. Note that additionally a failure status is observed in this function to check if the intended boundary conditions are fulfilled. <a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.HeatTransfer.Channel.kc_evenGapOverall\">See more information.</a>
</p>
</html>"));
          end kc_evenGapOverall;

          function kc_evenGapOverall_KC "Mean heat transfer coefficient of even gap | overall flow regime | considering boundary layer development | heat transfer at ONE or BOTH sides | identical and constant wall temperatures | surface roughness"
            extends Modelica.Icons.Function;
            //SOURCE: VDI-Waermeatlas, 9th edition, Springer-Verlag, 2002, Section Gb 6-10
            import SMOOTH = Modelica.Fluid.Dissipation.Utilities.Functions.General.Stepsmoother;
            //input records
            input Modelica.Fluid.Dissipation.HeatTransfer.Channel.kc_evenGapOverall_IN_con IN_con "Input record for function kc_evenGapOverall_KC" annotation(
              Dialog(group = "Constant inputs"));
            input Modelica.Fluid.Dissipation.HeatTransfer.Channel.kc_evenGapOverall_IN_var IN_var "Input record for function kc_evenGapOverall_KC" annotation(
              Dialog(group = "Variable inputs"));
            //output variables
            output SI.CoefficientOfHeatTransfer kc "Output for function kc_evenGapOverall_KC";
          protected
            Real MIN = Modelica.Constants.eps;
            Real laminar = 2200 "Maximum Reynolds number for laminar regime";
            Real turbulent = 1e4 "Minimum Reynolds number for turbulent regime";
            SI.Area A_cross = max(MIN, IN_con.s * IN_con.h) "Cross sectional area of gap";
            SI.Diameter d_hyd = 2 * IN_con.s "Hydraulic diameter";
            SI.Velocity velocity = abs(IN_var.m_flow) / max(MIN, IN_var.rho * A_cross) "Mean velocity in gap";
            SI.ReynoldsNumber Re = max(1, IN_var.rho * velocity * d_hyd / max(MIN, IN_var.eta));
            SI.PrandtlNumber Pr = abs(IN_var.eta * IN_var.cp / max(MIN, IN_var.lambda));
            kc_evenGapTurbulent_IN_con IN_con_turb(h = IN_con.h, s = IN_con.s, L = IN_con.L);
          algorithm
            kc := SMOOTH(laminar, turbulent, Re) * Dissipation.HeatTransfer.Channel.kc_evenGapLaminar_KC(IN_con, IN_var) + SMOOTH(turbulent, laminar, Re) * Dissipation.HeatTransfer.Channel.kc_evenGapTurbulent_KC(IN_con_turb, IN_var);
            annotation(
              Inline = false,
              Documentation(info = "<html>
<p>
Calculation of the mean convective heat transfer coefficient <b> kc </b> for an overall fluid flow through an even gap at different fluid flow and heat transfer situations. <a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.HeatTransfer.Channel.kc_evenGapOverall\">See more information.</a>
</p>
</html>"),
              smoothOrder(normallyConstant = IN_con) = 2);
          end kc_evenGapOverall_KC;

          record kc_evenGapOverall_IN_con "Input record for function kc_evenGapOverall and kc_evenGapOverall_KC"
            //even gap variables
            extends Modelica.Fluid.Dissipation.Utilities.Records.HeatTransfer.EvenGap;
            annotation(
              Documentation(info = "<html>
This record is used as <b> input record </b> for the heat transfer function <a href=\"modelica://Modelica.Fluid.Dissipation.HeatTransfer.Channel.kc_evenGapOverall\"> kc_evenGapOverall</a> and
<a href=\"modelica://Modelica.Fluid.Dissipation.HeatTransfer.Channel.kc_evenGapOverall_KC\"> kc_evenGapOverall_KC</a>.
</html>"));
          end kc_evenGapOverall_IN_con;

          record kc_evenGapOverall_IN_var "Input record for function kc_evenGapOverall and kc_evenGapOverall_KC"
            //fluid property variables
            extends Modelica.Fluid.Dissipation.Utilities.Records.General.FluidProperties;
            //input variable (mass flow rate)
            SI.MassFlowRate m_flow annotation(
              Dialog(group = "Input"));
            annotation(
              Documentation(info = "<html>
This record is used as <b> input record </b> for the heat transfer function <a href=\"Modelica://Modelica.Fluid.Dissipation.HeatTransfer.Channel.kc_evenGapOverall\"> kc_evenGapOverall</a> and
<a href=\"Modelica://Modelica.Fluid.Dissipation.HeatTransfer.Channel.kc_evenGapOverall_KC\"> kc_evenGapOverall_KC</a>.
</html>"));
          end kc_evenGapOverall_IN_var;

          function kc_evenGapTurbulent "Mean heat transfer coefficient of even gap | turbulent flow regime | developed fluid flow | heat transfer at BOTH sides | identical and constant wall temperatures"
            extends Modelica.Icons.Function;
            //SOURCE: VDI-Waermeatlas, 9th edition, Springer-Verlag, 2002, Section Gb 7
            import MIN = Modelica.Constants.eps;
            //input records
            input Modelica.Fluid.Dissipation.HeatTransfer.Channel.kc_evenGapTurbulent_IN_con IN_con "Input record for function kc_evenGapTurbulent" annotation(
              Dialog(group = "Constant inputs"));
            input Modelica.Fluid.Dissipation.HeatTransfer.Channel.kc_evenGapTurbulent_IN_var IN_var "Input record for function kc_evenGapTurbulent" annotation(
              Dialog(group = "Variable inputs"));
            //output variables
            output SI.CoefficientOfHeatTransfer kc "Convective heat transfer coefficient" annotation(
              Dialog(group = "Output"));
            output SI.PrandtlNumber Pr "Prandtl number" annotation(
              Dialog(group = "Output"));
            output SI.ReynoldsNumber Re "Reynolds number" annotation(
              Dialog(group = "Output"));
            output SI.NusseltNumber Nu "Nusselt number" annotation(
              Dialog(group = "Output"));
            output Real failureStatus "0== boundary conditions fulfilled | 1== failure >> check if still meaningful results" annotation(
              Dialog(group = "Output"));
          protected
            Real MIN = Modelica.Constants.eps;
            Real prandtlMax = 100 "Maximum Prandtl number";
            Real prandtlMin = 0.6 "Minimum Prandtl number";
            Real turbulentMax = 1e6 "Maximum Reynolds number for turbulent flow regime";
            Real turbulentMin = 3e4 "Minimum Reynolds number for turbulent flow regime";
            SI.Area A_cross = max(MIN, IN_con.s * IN_con.h) "Cross sectional area of gap";
            SI.Diameter d_hyd = 2 * IN_con.s "Hydraulic diameter";
            SI.Velocity velocity = abs(IN_var.m_flow) / max(MIN, IN_var.rho * A_cross) "Mean velocity in gap";
            //failure status
            Real fstatus[3] "check of expected boundary conditions";
            //Documentation
          algorithm
            Pr := abs(IN_var.eta * IN_var.cp / max(MIN, IN_var.lambda));
            Re := max(1, abs(IN_var.rho * velocity * d_hyd / max(MIN, IN_var.eta)));
            kc := Modelica.Fluid.Dissipation.HeatTransfer.Channel.kc_evenGapTurbulent_KC(IN_con, IN_var);
            Nu := kc * d_hyd / max(MIN, IN_var.lambda);
//failure status
            fstatus[1] := if Re > turbulentMax or Re < turbulentMin then 1 else 0;
            fstatus[2] := if Pr > prandtlMax or Pr < prandtlMin then 1 else 0;
            fstatus[3] := if d_hyd / max(MIN, IN_con.L) > 1.0 then 1 else 0;
            failureStatus := 0;
            for i in 1:size(fstatus, 1) loop
              if fstatus[i] == 1 then
                failureStatus := 1;
              end if;
            end for;
            annotation(
              Inline = false,
              Documentation(info = "<html>
<p>
Calculation of the mean convective heat transfer coefficient <b> kc </b> for a developed turbulent fluid flow through an even gap at heat transfer from both sides. Note that additionally a failure status is observed in this function to check if the intended boundary conditions are fulfilled. <a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.HeatTransfer.Channel.kc_evenGapTurbulent\">See more information.</a>
</p>
</html>"),
              smoothOrder(normallyConstant = IN_con) = 2);
          end kc_evenGapTurbulent;

          function kc_evenGapTurbulent_KC "Mean heat transfer coefficient of even gap | turbulent flow regime | developed fluid flow | heat transfer at BOTH sides | identical and constant wall temperatures"
            extends Modelica.Icons.Function;
            //SOURCE: VDI-Waermeatlas, 9th edition, Springer-Verlag, 2002, Section Gb 7
            //input records
            input Modelica.Fluid.Dissipation.HeatTransfer.Channel.kc_evenGapTurbulent_IN_con IN_con "Input record for function kc_evenGapTurbulent_KC" annotation(
              Dialog(group = "Constant inputs"));
            input Modelica.Fluid.Dissipation.HeatTransfer.Channel.kc_evenGapTurbulent_IN_var IN_var "Input record for function kc_evenGapTurbulent_KC" annotation(
              Dialog(group = "Variable inputs"));
            //output variables
            output SI.CoefficientOfHeatTransfer kc "Output for function kc_evenGapTurbulent_KC";
          protected
            Real MIN = Modelica.Constants.eps;
            SI.Area A_cross = max(MIN, IN_con.s * IN_con.h) "Cross sectional area of gap";
            SI.Diameter d_hyd = 2 * IN_con.s "Hydraulic diameter";
            SI.Velocity velocity = abs(IN_var.m_flow) / max(MIN, IN_var.rho * A_cross) "Mean velocity in gap";
            SI.ReynoldsNumber Re = max(2.6, IN_var.rho * velocity * d_hyd / max(MIN, IN_var.eta));
            SI.PrandtlNumber Pr = abs(IN_var.eta * IN_var.cp / max(MIN, IN_var.lambda));
            //SOURCE: p.Ga 5, eq. 27
            Real zeta = 1 / max(MIN, 1.8 * Modelica.Math.log10(abs(Re)) - 1.5) ^ 2 "Pressure loss coefficient";
            //SOURCE: p.Gb 5, eq. 26
            //assumption according to Gb 7, sec. 2.4
            SI.NusseltNumber Nu = abs(zeta / 8 * Re * Pr / (1 + 12.7 * (zeta / 8) ^ 0.5 * (Pr ^ (2 / 3) - 1)) * (1 + (d_hyd / max(MIN, IN_con.L)) ^ (2 / 3)));
            //Documentation
          algorithm
            kc := Nu * (IN_var.lambda / max(MIN, d_hyd));
            annotation(
              Inline = false,
              Documentation(info = "<html>
<p>
Calculation of the mean convective heat transfer coefficient <b> kc </b> for a developed turbulent fluid flow through an even gap at heat transfer from both sides. <a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.HeatTransfer.Channel.kc_evenGapTurbulent\">See more information.</a>
</p>
</html>"),
              smoothOrder(normallyConstant = IN_con) = 2);
          end kc_evenGapTurbulent_KC;

          record kc_evenGapTurbulent_IN_con "Input record for function kc_evenGapTurbulent and kc_evenGapTurbulent_KC"
            extends Modelica.Fluid.Dissipation.HeatTransfer.Channel.kc_evenGapOverall_IN_con(final target = Modelica.Fluid.Dissipation.Utilities.Types.kc_evenGap.DevBoth);
            annotation(
              Documentation(info = "<html>
<p>
This record is used as <b> input record </b> for the heat transfer function <a href=\"modelica://Modelica.Fluid.Dissipation.HeatTransfer.Channel.kc_evenGapTurbulent\"> kc_evenGapTurbulent</a> and
<a href=\"Modelica://Modelica.Fluid.Dissipation.HeatTransfer.Channel.kc_evenGapTurbulent_KC\"> kc_evenGapTurbulent_KC</a>.
</p>
</html>"));
          end kc_evenGapTurbulent_IN_con;

          record kc_evenGapTurbulent_IN_var "Input record for function kc_evenGapTurbulent and kc_evenGapTurbulent_KC"
            extends Modelica.Fluid.Dissipation.HeatTransfer.Channel.kc_evenGapOverall_IN_var;
            annotation(
              Documentation(info = "<html>
<p>
This record is used as <b> input record </b> for the heat transfer function <a href=\"modelica://Modelica.Fluid.Dissipation.HeatTransfer.Channel.kc_evenGapTurbulent\"> kc_evenGapTurbulent</a> and
<a href=\"Modelica://Modelica.Fluid.Dissipation.HeatTransfer.Channel.kc_evenGapTurbulent_KC\"> kc_evenGapTurbulent_KC</a>.
</p>
</html>"));
          end kc_evenGapTurbulent_IN_var;
          annotation(
            preferredView = "info",
            Documentation(info = "<html>
<h4>Even gap</h4>
<h5>Laminar flow</h5>
<p>Calculation of the mean convective heat transfer coefficient <b>kc </b>for a laminar fluid flow through an even gap at different fluid flow and heat transfer situations. <a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.HeatTransfer.Channel.kc_evenGapLaminar\">See more information.</a> </p>

<h5>Turbulent flow</h5>
<p>Calculation of the mean convective heat transfer coefficient <b>kc </b>for a laminar fluid flow through an even gap at different fluid flow and heat transfer situations. <a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.HeatTransfer.Channel.kc_evenGapTurbulent\">See more information.</a></p>

<h5>Overall flow</h5>
<p>Calculation of the mean convective heat transfer coefficient <b>kc </b>for a laminar fluid flow through an even gap at different fluid flow and heat transfer situations. <a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.HeatTransfer.Channel.kc_evenGapOverall\">See more information.</a></p>
</html>"));
        end Channel;

        package General
          extends Modelica.Icons.VariantsPackage;

          function kc_approxForcedConvection "Mean convective heat transfer coefficient for forced convection | approximation | turbulent regime | hydrodynamically developed fluid flow"
            extends Modelica.Icons.Function;
            //SOURCE: A Bejan and A.D. Kraus. Heat Transfer handbook.John Wiley & Sons, 2nd edition, 2003. (p.424 ff)
            //Notation of equations according to SOURCE
            //input records
            input Modelica.Fluid.Dissipation.HeatTransfer.General.kc_approxForcedConvection_IN_con IN_con "Input record for function kc_approxForcedConvection" annotation(
              Dialog(group = "Constant inputs"));
            input Modelica.Fluid.Dissipation.HeatTransfer.General.kc_approxForcedConvection_IN_var IN_var "Input record for function kc_approxForcedConvection" annotation(
              Dialog(group = "Variable inputs"));
            //output variables
            output SI.CoefficientOfHeatTransfer kc "Convective heat transfer coefficient" annotation(
              Dialog(group = "Output"));
            output SI.PrandtlNumber Pr "Prandtl number" annotation(
              Dialog(group = "Output"));
            output SI.ReynoldsNumber Re "Reynolds number" annotation(
              Dialog(group = "Output"));
            output SI.NusseltNumber Nu "Nusselt number" annotation(
              Dialog(group = "Output"));
            output Real failureStatus "0== boundary conditions fulfilled | 1== failure >> check if still meaningful results" annotation(
              Dialog(group = "Output"));
          protected
            type TYP = Modelica.Fluid.Dissipation.Utilities.Types.kc_general;
            Real MIN = Modelica.Constants.eps;
            Real prandtlMax[3] = {120, 16700, 500} "Maximum Prandtl number";
            Real prandtlMin[3] = {0.7, 0.7, 1.5} "Minimum Prandtl number";
            Real reynoldsMax[3] = {1.24e5, 1e6, 1e6} "Maximum Reynolds number";
            Real reynoldsMin[3] = {2500, 1e4, 3e3} "Minimum Reynolds number";
            SI.Diameter d_hyd = max(MIN, 4 * IN_con.A_cross / max(MIN, IN_con.perimeter)) "Hydraulic diameter";
            //failure status
            Real fstatus[2] "Check of expected boundary conditions";
          algorithm
            Pr := Modelica.Fluid.Dissipation.Utilities.Functions.General.PrandtlNumber(IN_var.cp, IN_var.eta, IN_var.lambda);
            Re := max(1, Modelica.Fluid.Dissipation.Utilities.Functions.General.ReynoldsNumber(IN_con.A_cross, IN_con.perimeter, IN_var.rho, IN_var.eta, abs(IN_var.m_flow))) "Reynolds number";
            kc := Modelica.Fluid.Dissipation.HeatTransfer.General.kc_approxForcedConvection_KC(IN_con, IN_var);
            Nu := kc * d_hyd / max(MIN, IN_var.lambda);
//failure status
            fstatus[1] := if IN_con.target == TYP.Rough then if Pr > prandtlMax[1] or Pr < prandtlMin[1] then 1 else 0 else if IN_con.target == TYP.Middle then if Pr > prandtlMax[2] or Pr < prandtlMin[2] then 1 else 0 else if IN_con.target == TYP.Finest then if Pr > prandtlMax[3] or Pr < prandtlMin[3] then 1 else 0 else 0;
            fstatus[2] := if IN_con.target == TYP.Rough then if Re > reynoldsMax[1] or Re < reynoldsMin[1] then 1 else 0 else if IN_con.target == TYP.Middle then if Re > reynoldsMax[2] or Re < reynoldsMin[2] then 1 else 0 else if IN_con.target == TYP.Finest then if Re > reynoldsMax[3] or Re < reynoldsMin[3] then 1 else 0 else 0;
            failureStatus := 0;
            for i in 1:size(fstatus, 1) loop
              if fstatus[i] == 1 then
                failureStatus := 1;
              end if;
            end for;
            annotation(
              Inline = false,
              Documentation(info = "<html>
<p>
Approximate calculation of the mean convective heat transfer coefficient <b> kc </b> for forced convection with a fully developed fluid flow in a turbulent regime.
</p>

<p>
A detailed documentation for this convective heat transfer calculation can be found in its underlying function
<a href=\"modelica://Modelica.Fluid.Dissipation.HeatTransfer.General.kc_approxForcedConvection_KC\">kc_approxForcedConvection_KC</a> .
Note that additionally a failure status is observed in this function to check if the intended boundary conditions are fulfilled.  <a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.HeatTransfer.General.kc_approxForcedConvection\">See more information</a> .
</p>
</html>"),
              smoothOrder(normallyConstant = IN_con) = 2);
          end kc_approxForcedConvection;

          function kc_approxForcedConvection_KC "Mean convective heat transfer coefficient for forced convection | approximation | turbulent regime | hydrodynamically developed fluid flow"
            extends Modelica.Icons.Function;
            //SOURCE: A Bejan and A.D. Kraus. Heat Transfer handbook.John Wiley & Sons, 2nd edition, 2003. (p.424 ff)
            //Notation of equations according to SOURCE
            //type =
            //input records
            input Modelica.Fluid.Dissipation.HeatTransfer.General.kc_approxForcedConvection_IN_con IN_con "Input record for function kc_approxForcedConvection_KC" annotation(
              Dialog(group = "Constant inputs"));
            input Modelica.Fluid.Dissipation.HeatTransfer.General.kc_approxForcedConvection_IN_var IN_var "Input record for function kc_approxForcedConvection_KC" annotation(
              Dialog(group = "Variable inputs"));
            //output variables
            output SI.CoefficientOfHeatTransfer kc "Output for function kc_approxForcedConvection_KC";
          protected
            type TYP = Modelica.Fluid.Dissipation.Utilities.Types.kc_general;
            Real MIN = Modelica.Constants.eps;
            SI.Diameter d_hyd = max(MIN, 4 * IN_con.A_cross / max(MIN, IN_con.perimeter)) "Hydraulic diameter";
            SI.PrandtlNumber Pr = max(MIN, abs(IN_var.eta * IN_var.cp / max(MIN, IN_var.lambda))) "Prandtl number";
            SI.ReynoldsNumber Re = max(1, 4 * abs(IN_var.m_flow) / max(MIN, IN_con.perimeter * IN_var.eta)) "Reynolds number";
          algorithm
            kc := IN_var.lambda / d_hyd * (if IN_con.target == TYP.Rough then 0.023 * Re ^ (4 / 5) * Pr ^ IN_con.exp_Pr else if IN_con.target == TYP.Middle then 0.023 * Re ^ (4 / 5) * Pr ^ (1 / 3) * (IN_var.eta / IN_var.eta_wall) ^ 0.14 else if IN_con.target == TYP.Finest and Pr <= 1.5 then 0.0214 * max(1, abs(Re ^ 0.8 - 100)) * Pr ^ 0.4 else if IN_con.target == TYP.Finest then 0.012 * max(1, abs(Re ^ 0.87 - 280)) * Pr ^ 0.4 else 0);
//Documentation
            annotation(
              Inline = false,
              Documentation(info = "<html>
<p>
Approximate calculation of the mean convective heat transfer coefficient <b> kc </b> for forced convection with a fully developed fluid flow in a turbulent regime.
<a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.HeatTransfer.General.kc_approxForcedConvection\">See more information</a> .
</p>
</html>"),
              smoothOrder(normallyConstant = IN_con) = 2);
          end kc_approxForcedConvection_KC;

          record kc_approxForcedConvection_IN_con "Input record for function kc_approxForcedConvection and kc_approxForcedConvection_KC"
            //generic variables
            extends Modelica.Fluid.Dissipation.Utilities.Records.HeatTransfer.General;
            parameter Real exp_Pr = 0.4 "Exponent for Prandtl number w.r.t. Dittus/Boelter | 0.4 for heating | 0.3 for cooling" annotation(
              Dialog(group = "Generic variables", enable = if target == 1 then true else false));
            annotation(
              Documentation(info = "<html>
This record is used as <b> input record </b> for the heat transfer function <a href=\"Modelica://Modelica.Fluid.Dissipation.HeatTransfer.General.kc_approxForcedConvection\"> kc_approxForcedConvection</a> and
<a href=\"Modelica://Modelica.Fluid.Dissipation.HeatTransfer.General.kc_approxForcedConvection_KC\"> kc_approxForcedConvection_KC</a>.
</html>"));
          end kc_approxForcedConvection_IN_con;

          record kc_approxForcedConvection_IN_var "Input record for function kc_approxForcedConvection and kc_approxForcedConvection_KC"
            //fluid property variables
            extends Modelica.Fluid.Dissipation.Utilities.Records.General.FluidProperties;
            SI.DynamicViscosity eta_wall "Dynamic viscosity of fluid at wall temperature" annotation(
              Dialog(group = "Fluid properties", enable = if target == 2 then true else false));
            //input variable (mass flow rate)
            SI.MassFlowRate m_flow annotation(
              Dialog(group = "Input"));
            annotation(
              Documentation(info = "<html>
This record is used as <b> input record </b> for the heat transfer function <a href=\"Modelica://Modelica.Fluid.Dissipation.HeatTransfer.General.kc_approxForcedConvection\"> kc_approxForcedConvection</a> and
<a href=\"Modelica://Modelica.Fluid.Dissipation.HeatTransfer.General.kc_approxForcedConvection_KC\"> kc_approxForcedConvection_KC</a>.
</html>"));
          end kc_approxForcedConvection_IN_var;
          annotation(
            preferredView = "info",
            Documentation(info = "<html>
<h4>General heat transfer</h4>
<h5>Approximated forced convection</h5>
<p>Approximate calculation of the mean convective heat transfer coefficient <b> kc </b> for forced convection with a fully developed fluid flow in a turbulent regime. <a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.HeatTransfer.General.kc_approxForcedConvection\">See more information.</a> </p>
</html>"));
        end General;

        package HeatExchanger
          extends Modelica.Icons.VariantsPackage;

          function kc_flatTube
            extends Modelica.Icons.Function;
            //SOURCE: A.M. Jacobi, Y. Park, D. Tafti, X. Zhang. AN ASSESSMENT OF THE STATE OF THE ART, AND POTENTIAL DESIGN IMPROVEMENTS, FOR FLAT-TUBE HEAT EXCHANGERS IN AIR CONDITIONING AND REFRIGERATION APPLICATIONS - PHASE I
            //icon
            //input records
            input Modelica.Fluid.Dissipation.HeatTransfer.HeatExchanger.kc_flatTube_IN_con IN_con "Input record for function kc_flatTube" annotation(
              Dialog(group = "Constant inputs"));
            input Modelica.Fluid.Dissipation.HeatTransfer.HeatExchanger.kc_flatTube_IN_var IN_var "Input record for function kc_flatTube" annotation(
              Dialog(group = "Variable inputs"));
            //output variables
            output SI.CoefficientOfHeatTransfer kc "Convective heat transfer coefficient" annotation(
              Dialog(group = "Output"));
            output SI.PrandtlNumber Pr "Prandtl number" annotation(
              Dialog(group = "Output"));
            output SI.ReynoldsNumber Re "Reynolds number" annotation(
              Dialog(group = "Output"));
            output SI.NusseltNumber Nu "Nusselt number" annotation(
              Dialog(group = "Output"));
            output Real failureStatus "0== boundary conditions fulfilled | 1== failure >> check if still meaningful results" annotation(
              Dialog(group = "Output"));
          protected
            type TYP = Modelica.Fluid.Dissipation.Utilities.Types.HTXGeometry_flatTubes;
            SI.Area A_c = if IN_con.geometry == TYP.LouverFin then IN_con.A_fr * ((IN_con.F_l - IN_con.delta_f) * (IN_con.F_p - IN_con.delta_f) / ((IN_con.F_l + IN_con.D_m) * IN_con.F_p)) else if IN_con.geometry == TYP.RectangularFin then IN_con.A_fr * (h * s / ((h + t + IN_con.D_m) * (s + t))) else 0 "Minimum flow cross-sectional area";
            SI.Length h = if IN_con.geometry == TYP.RectangularFin then IN_con.D_h * (1 + IN_con.alpha) / (2 * IN_con.alpha) else 0 "Free flow height";
            SI.Length l = if IN_con.geometry == TYP.RectangularFin then t / IN_con.delta else 0 "Fin length";
            SI.Length s = if IN_con.geometry == TYP.RectangularFin then h * IN_con.alpha else 0 "Lateral fin spacing (free flow width)";
            SI.Length t = if IN_con.geometry == TYP.RectangularFin then s * IN_con.gamma else 0 "Fin thickness";
          algorithm
            kc := Modelica.Fluid.Dissipation.HeatTransfer.HeatExchanger.kc_flatTube_KC(IN_con, IN_var);
            Pr := abs(IN_var.eta * IN_var.cp / IN_var.lambda);
            if IN_con.geometry == TYP.LouverFin then
              Re := max(1e-3, abs(IN_var.m_flow) * IN_con.L_p / (IN_var.eta * A_c));
              Nu := max(1e-3, kc * IN_con.L_p / IN_var.lambda);
            elseif IN_con.geometry == TYP.RectangularFin then
              Re := max(1e-3, abs(IN_var.m_flow) * IN_con.D_h / (IN_var.eta * A_c));
              Nu := max(1e-3, kc * IN_con.D_h / IN_var.lambda);
            end if;
            failureStatus := if IN_con.geometry == TYP.LouverFin then if Re < 100 or Re > 3000 then 1 else 0 else if IN_con.geometry == TYP.RectangularFin then if Re < 300 or Re > 5000 then 1 else 0 else 0;
            annotation(
              Inline = false,
              Documentation(info = "<html>
<p>
Calculation of the mean convective heat transfer coefficient <b> kc </b> for the air-side heat transfer of heat exchangers with flat tubes and several fin geometries. Note that additionally a failure status is observed in this function to check if the intended boundary conditions are fulfilled.
<a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.HeatTransfer.HeatExchanger.kc_flatTube\">See more information</a> .
</p>

</html>"),
              smoothOrder(normallyConstant = IN_con) = 2);
          end kc_flatTube;

          function kc_flatTube_KC
            extends Modelica.Icons.Function;
            //SOURCE: A.M. Jacobi, Y. Park, D. Tafti, X. Zhang. AN ASSESSMENT OF THE STATE OF THE ART, AND POTENTIAL DESIGN IMPROVEMENTS, FOR FLAT-TUBE HEAT EXCHANGERS IN AIR CONDITIONING AND REFRIGERATION APPLICATIONS - PHASE I
            //input records
            input Modelica.Fluid.Dissipation.HeatTransfer.HeatExchanger.kc_flatTube_IN_con IN_con "Input record for function kc_flatTube_KC" annotation(
              Dialog(group = "Constant inputs"));
            input Modelica.Fluid.Dissipation.HeatTransfer.HeatExchanger.kc_flatTube_IN_var IN_var "Input record for function kc_flatTube_KC" annotation(
              Dialog(group = "Variable inputs"));
            //output variables
            output SI.CoefficientOfHeatTransfer kc "Output for function kc_flatTubePlateFin_KC";
          protected
            type TYP = Modelica.Fluid.Dissipation.Utilities.Types.HTXGeometry_flatTubes;
            Real MIN = Modelica.Constants.eps;
            Real Phi = IN_con.Phi * 180 / PI "Louver angle";
            SI.ReynoldsNumber Re_Dh = max(1e-3, abs(IN_var.m_flow) * IN_con.D_h / (IN_var.eta * A_c)) "Reynolds number based on hydraulic diameter";
            SI.ReynoldsNumber Re_Lp = max(1e-3, abs(IN_var.m_flow) * IN_con.L_p / (IN_var.eta * A_c)) "Reynolds number based on louver pitch";
            SI.PrandtlNumber Pr = IN_var.eta * IN_var.cp / IN_var.lambda "Prandtl number";
            Real j "Colburn j factor";
            SI.Area A_c = if IN_con.geometry == TYP.LouverFin then IN_con.A_fr * ((IN_con.F_l - IN_con.delta_f) * (IN_con.F_p - IN_con.delta_f) / ((IN_con.F_l + IN_con.D_m) * IN_con.F_p)) else if IN_con.geometry == TYP.RectangularFin then IN_con.A_fr * (h * s / ((h + t + IN_con.D_m) * (s + t))) else 0 "Minimum flow cross-sectional area";
            SI.Length h = if IN_con.geometry == TYP.RectangularFin then IN_con.D_h * (1 + IN_con.alpha) / (2 * IN_con.alpha) else 0 "Free flow height";
            SI.Length l = if IN_con.geometry == TYP.RectangularFin then t / IN_con.delta else 0 "Fin length";
            SI.Length s = if IN_con.geometry == TYP.RectangularFin then h * IN_con.alpha else 0 "Lateral fin spacing (free flow width)";
            SI.Length t = if IN_con.geometry == TYP.RectangularFin then s * IN_con.gamma else 0 "Fin thickness";
          algorithm
            if IN_con.geometry == TYP.LouverFin then
              j := Re_Lp ^ (-0.49) * (Phi / 90) ^ 0.27 * (IN_con.F_p / IN_con.L_p) ^ (-0.14) * (IN_con.F_l / IN_con.L_p) ^ (-0.29) * (IN_con.T_d / IN_con.L_p) ^ (-0.23) * (IN_con.L_l / IN_con.L_p) ^ 0.68 * (IN_con.T_p / IN_con.L_p) ^ (-0.28) * (IN_con.delta_f / IN_con.L_p) ^ (-0.05);
              kc := j * (Re_Lp * Pr ^ (1 / 3) * IN_var.lambda / IN_con.L_p);
            elseif IN_con.geometry == TYP.RectangularFin then
              j := 0.6522 * Re_Dh ^ (-0.5403) * (s / h) ^ (-0.1541) * (t / l) ^ 0.1499 * (t / s) ^ (-0.0678) * (1 + 5.269e-5 * Re_Dh ^ 1.340 * (s / h) ^ 0.504 * (t / l) ^ 0.456 * (t / s) ^ (-1.055)) ^ 0.1;
              kc := j * (Re_Dh * Pr ^ (1 / 3) * IN_var.lambda / IN_con.D_h);
            end if;
            annotation(
              Inline = false,
              Documentation(info = "<html>
<p>
Calculation of the mean convective heat transfer coefficient <b> kc </b> for the air-side heat transfer of heat exchangers with flat tubes and several fin geometries.
<a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.HeatTransfer.HeatExchanger.kc_flatTube\">See more information</a> .
</p>
</html>"),
              smoothOrder(normallyConstant = IN_con) = 2);
          end kc_flatTube_KC;

          record kc_flatTube_IN_con "Input record for function kc_flatTube and kc_flatTube_KC"
            extends Modelica.Icons.Record;
          protected
            type TYP = Modelica.Fluid.Dissipation.Utilities.Types.HTXGeometry_flatTubes;
          public
            Modelica.Fluid.Dissipation.Utilities.Types.HTXGeometry_flatTubes geometry = Dissipation.Utilities.Types.HTXGeometry_flatTubes.LouverFin "Choice of fin geometry" annotation(
              Dialog(group = "HeatExchanger"));
            SI.Area A_fr = 0 "Frontal area" annotation(
              Dialog(group = "HeatExchanger"));
            SI.Length D_h = 0 "Hydraulic diameter" annotation(
              Dialog(group = "HeatExchanger", enable = if geometry == 2 then true else false));
            SI.Length D_m = 0 "Major tube diameter for flat tube" annotation(
              Dialog(group = "HeatExchanger"));
            SI.Length F_l = 0 "Fin length" annotation(
              Dialog(group = "HeatExchanger", enable = if geometry == 1 then true else false));
            SI.Length F_p = 0 "Fin pitch, fin spacing + fin thickness" annotation(
              Dialog(group = "HeatExchanger", enable = if geometry == 1 then true else false));
            SI.Length L_l = 0 "Louver length" annotation(
              Dialog(group = "HeatExchanger", enable = if geometry == 1 then true else false));
            SI.Length L_p = 0 "Louver pitch" annotation(
              Dialog(group = "HeatExchanger", enable = if geometry == 1 then true else false));
            SI.Length T_d = 0 "Tube depth" annotation(
              Dialog(group = "HeatExchanger", enable = if geometry == 1 then true else false));
            SI.Length T_p = 0 "Tube pitch" annotation(
              Dialog(group = "HeatExchanger", enable = if geometry == 1 then true else false));
            Real alpha = 0 "Lateral fin spacing (s) / free flow height (h)" annotation(
              Dialog(group = "HeatExchanger", enable = if geometry == 2 then true else false));
            Real gamma = 0 "Fin thickness (t) / lateral fin spacing (s)" annotation(
              Dialog(group = "HeatExchanger", enable = if geometry == 2 then true else false));
            Real delta = 0 "Fin thickness (t) / Fin length (l)" annotation(
              Dialog(group = "HeatExchanger", enable = if geometry == 2 then true else false));
            SI.Length delta_f = 0 "Fin thickness" annotation(
              Dialog(group = "HeatExchanger", enable = if geometry == 1 then true else false));
            SI.Angle Phi = 0 "Louver angle" annotation(
              Dialog(group = "HeatExchanger", enable = if geometry == 1 then true else false));
            annotation(
              Documentation(info = "<html>
This record is used as <b> input record </b> for the heat transfer function <a href=\"Modelica://Modelica.Fluid.Dissipation.HeatTransfer.HeatExchanger.kc_flatTube\"> kc_flatTube</a> and
<a href=\"Modelica://Modelica.Fluid.Dissipation.HeatTransfer.HeatExchanger.kc_flatTube_KC\"> kc_flatTube_KC</a>.
</html>"));
          end kc_flatTube_IN_con;

          record kc_flatTube_IN_var "Input record for function kc_flatTube and kc_flatTube_KC"
            extends Modelica.Icons.Record;
            //fluid property variables
            extends Modelica.Fluid.Dissipation.Utilities.Records.General.FluidProperties;
            //input variable (mass flow rate)
            SI.MassFlowRate m_flow annotation(
              Dialog(group = "Input"));
            annotation(
              Documentation(info = "<html>
This record is used as <b> input record </b> for the heat transfer function <a href=\"modelica://Modelica.Fluid.Dissipation.HeatTransfer.HeatExchanger.kc_flatTube\"> kc_flatTube</a> and
<a href=\"modelica://Modelica.Fluid.Dissipation.HeatTransfer.HeatExchanger.kc_flatTube_KC\"> kc_flatTube_KC</a>.
</html>"));
          end kc_flatTube_IN_var;

          function kc_roundTube
            extends Modelica.Icons.Function;
            //SOURCE: A.M. Jacobi, Y. Park, D. Tafti, X. Zhang. AN ASSESSMENT OF THE STATE OF THE ART, AND POTENTIAL DESIGN IMPROVEMENTS, FOR FLAT-TUBE HEAT EXCHANGERS IN AIR CONDITIONING AND REFRIGERATION APPLICATIONS - PHASE I
            //input records
            input Modelica.Fluid.Dissipation.HeatTransfer.HeatExchanger.kc_roundTube_IN_con IN_con "Input record for function kc_roundTube" annotation(
              Dialog(group = "Constant inputs"));
            input Modelica.Fluid.Dissipation.HeatTransfer.HeatExchanger.kc_roundTube_IN_var IN_var "Input record for function kc_roundTube" annotation(
              Dialog(group = "Variable inputs"));
            //output variables
            output SI.CoefficientOfHeatTransfer kc "Convective heat transfer coefficient" annotation(
              Dialog(group = "Output"));
            output SI.PrandtlNumber Pr "Prandtl number" annotation(
              Dialog(group = "Output"));
            output SI.ReynoldsNumber Re "Reynolds number" annotation(
              Dialog(group = "Output"));
            output SI.NusseltNumber Nu "Nusselt number" annotation(
              Dialog(group = "Output"));
            output Real failureStatus "0== boundary conditions fulfilled | 1== failure >> check if still meaningful results" annotation(
              Dialog(group = "Output"));
          protected
            type TYP = Modelica.Fluid.Dissipation.Utilities.Types.HTXGeometry_roundTubes;
            SI.Area A_c = IN_con.A_fr * ((IN_con.F_p * IN_con.P_t - IN_con.F_p * IN_con.D_c - (IN_con.P_t - IN_con.D_c) * IN_con.delta_f) / (IN_con.F_p * IN_con.P_t)) "Minimum flow cross-sectional area";
            SI.Area A_tot = if IN_con.geometry == TYP.LouverFin then IN_con.A_fr * ((IN_con.N * PI * IN_con.D_c * (IN_con.F_p - IN_con.delta_f) + 2 * (IN_con.P_t * IN_con.L - IN_con.N * PI * IN_con.D_c ^ 2 / 4)) / (IN_con.P_t * IN_con.F_p)) else 0 "Total heat transfer area";
            SI.Length D_h = if IN_con.geometry == TYP.LouverFin then 4 * A_c * IN_con.L / A_tot else 0 "Hydraulic diameter";
            /*SI.Length D_h=
                if IN_con.geometry==2 then
                  4*A_c/(IN_con.A_fr*(2*(IN_con.P_t-IN_con.D_c+IN_con.F_p)/(IN_con.F_p*(IN_con.P_t-IN_con.D_c)))) else
                  0 "Hydraulic diameter";*/
          algorithm
            kc := Modelica.Fluid.Dissipation.HeatTransfer.HeatExchanger.kc_roundTube_KC(IN_con, IN_var);
            Pr := abs(IN_var.eta * IN_var.cp / IN_var.lambda);
            if IN_con.geometry == TYP.PlainFin or IN_con.geometry == TYP.LouverFin or IN_con.geometry == TYP.SlitFin or IN_con.geometry == TYP.WavyFin then
              Re := max(1e-3, abs(IN_var.m_flow) * IN_con.D_c / (IN_var.eta * A_c));
              Nu := max(1e-3, kc * IN_con.D_c / IN_var.lambda);
            end if;
            failureStatus := if IN_con.geometry == TYP.PlainFin then if Re < 300 or Re > 8000 then 1 else 0 else if IN_con.geometry == TYP.LouverFin then if Re < 300 or Re > 7000 then 1 else 0 else if IN_con.geometry == TYP.SlitFin then if Re < 400 or Re > 7000 then 1 else 0 else if IN_con.geometry == TYP.WavyFin then if Re < 350 or Re > 7000 then 1 else 0 else 0;
            annotation(
              Inline = false,
              Documentation(info = "<html>
<p>
Calculation of the mean convective heat transfer coefficient <b> kc </b> for the air-side heat transfer of heat exchangers with round tubes and several fin geometries. Note that additionally a failure status is observed in this function to check if the intended boundary conditions are fulfilled.
<a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.HeatTransfer.HeatExchanger.kc_roundTube\">See more information</a> .
</p>
</html>"),
              smoothOrder(normallyConstant = IN_con) = 2);
          end kc_roundTube;

          function kc_roundTube_KC
            extends Modelica.Icons.Function;
            //SOURCE: A.M. Jacobi, Y. Park, D. Tafti, X. Zhang. AN ASSESSMENT OF THE STATE OF THE ART, AND POTENTIAL DESIGN IMPROVEMENTS, FOR FLAT-TUBE HEAT EXCHANGERS IN AIR CONDITIONING AND REFRIGERATION APPLICATIONS - PHASE I
            import SMOOTH = Modelica.Fluid.Dissipation.Utilities.Functions.General.Stepsmoother;
            //input records
            input Modelica.Fluid.Dissipation.HeatTransfer.HeatExchanger.kc_roundTube_IN_con IN_con "Input record for function kc_roundTube_KC" annotation(
              Dialog(group = "Constant inputs"));
            input Modelica.Fluid.Dissipation.HeatTransfer.HeatExchanger.kc_roundTube_IN_var IN_var "Input record for function kc_roundTube_KC" annotation(
              Dialog(group = "Variable inputs"));
            //output variables
            output SI.CoefficientOfHeatTransfer kc "Output for function kc_roundTube_KC";
          protected
            type TYP = Modelica.Fluid.Dissipation.Utilities.Types.HTXGeometry_roundTubes;
            Real MIN = Modelica.Constants.eps;
            SI.ReynoldsNumber Re_Dc = max(1e-3, abs(IN_var.m_flow) * IN_con.D_c / (IN_var.eta * A_c)) "Reynolds number based on fin collar diameter";
            SI.PrandtlNumber Pr = IN_var.eta * IN_var.cp / IN_var.lambda "Prandtl number";
            Real j "Colburn j factor";
            SI.Area A_c = IN_con.A_fr * ((IN_con.F_p * IN_con.P_t - IN_con.F_p * IN_con.D_c - (IN_con.P_t - IN_con.D_c) * IN_con.delta_f) / (IN_con.F_p * IN_con.P_t)) "Minimum flow cross-sectional area";
            SI.Area A_tot = if IN_con.geometry == TYP.LouverFin then IN_con.A_fr * ((IN_con.N * PI * IN_con.D_c * (IN_con.F_p - IN_con.delta_f) + 2 * (IN_con.P_t * IN_con.L - IN_con.N * PI * IN_con.D_c ^ 2 / 4)) / (IN_con.P_t * IN_con.F_p)) else 0 "Total heat transfer area";
            SI.Length D_h = if IN_con.geometry == TYP.LouverFin then 4 * A_c * IN_con.L / A_tot else 0 "Hydraulic diameter";
            /*SI.Length D_h=
                if IN_con.geometry==2 then
                  4*A_c/(IN_con.A_fr*(2*(IN_con.P_t-IN_con.D_c+IN_con.F_p-IN_con.delta_f)/(IN_con.F_p*IN_con.P_t))) else
                  0 "Hydraulic diameter";*/
            Real J1 = 0 "Exponent for computation of Colburn j factor";
            Real J2 = 0 "Exponent for computation of Colburn j factor";
            Real J3 = 0 "Exponent for computation of Colburn j factor";
            Real J4 = 0 "Exponent for computation of Colburn j factor";
            Real J5 = 0 "Exponent for computation of Colburn j factor";
            Real J6 = 0 "Exponent for computation of Colburn j factor";
            Real J7 = 0 "Exponent for computation of Colburn j factor";
            Real J8 = 0 "Exponent for computation of Colburn j factor";
          algorithm
            if IN_con.geometry == TYP.PlainFin then
              j := 0.991 * (2.24 * Re_Dc ^ (-0.092) * (IN_con.N / 4) ^ (-0.031)) ^ (0.607 * (4 - IN_con.N)) * (0.14 * Re_Dc ^ (-0.328) * (IN_con.P_t / IN_con.P_l) ^ (-0.502) * (IN_con.F_p / IN_con.D_c) ^ 0.0312) * (2.55 * (IN_con.P_l / IN_con.D_c) ^ (-1.28));
              kc := j * (Re_Dc * Pr ^ (1 / 3) * IN_var.lambda / IN_con.D_c);
            elseif IN_con.geometry == TYP.LouverFin then
              if Re_Dc < 900 then
                J1 := (-0.991) - 0.1055 * (IN_con.P_l / IN_con.P_t) ^ 3.1 * log(IN_con.L_h / IN_con.L_p);
                J2 := (-0.7344) + 2.1059 * IN_con.N ^ 0.55 / (log(Re_Dc) - 3.2);
                J3 := 0.08485 * (IN_con.P_l / IN_con.P_t) ^ (-4.4) * IN_con.N ^ (-0.68);
                J4 := -0.1741 * log(IN_con.N);
                j := 14.3117 * Re_Dc ^ J1 * (IN_con.F_p / IN_con.D_c) ^ J2 * (IN_con.L_h / IN_con.L_p) ^ J3 * (IN_con.F_p / IN_con.P_l) ^ J4 * (IN_con.P_l / IN_con.P_t) ^ (-1.724);
              elseif Re_Dc > 1100 then
                J5 := (-0.6027) + 0.02593 * (IN_con.P_l / D_h) ^ 0.52 * IN_con.N ^ (-0.5) * log(IN_con.L_h / IN_con.L_p);
                J6 := (-0.4776) + 0.40774 * IN_con.N ^ 0.7 / (log(Re_Dc) - 4.4);
                J7 := -0.58655 * (IN_con.F_p / D_h) ^ 2.3 * (IN_con.P_l / IN_con.P_t) ^ (-1.6) * IN_con.N ^ (-0.65);
                J8 := 0.0814 * (log(Re_Dc) - 3);
                j := 1.1373 * Re_Dc ^ J5 * (IN_con.F_p / IN_con.P_l) ^ J6 * (IN_con.L_h / IN_con.L_p) ^ J7 * (IN_con.P_l / IN_con.P_t) ^ J8 * IN_con.N ^ 0.3545;
              else
                J1 := (-0.991) - 0.1055 * (IN_con.P_l / IN_con.P_t) ^ 3.1 * log(IN_con.L_h / IN_con.L_p);
                J2 := (-0.7344) + 2.1059 * IN_con.N ^ 0.55 / (log(Re_Dc) - 3.2);
                J3 := 0.08485 * (IN_con.P_l / IN_con.P_t) ^ (-4.4) * IN_con.N ^ (-0.68);
                J4 := -0.1741 * log(IN_con.N);
                J5 := (-0.6027) + 0.02593 * (IN_con.P_l / D_h) ^ 0.52 * IN_con.N ^ (-0.5) * log(IN_con.L_h / IN_con.L_p);
                J6 := (-0.4776) + 0.40774 * IN_con.N ^ 0.7 / (log(Re_Dc) - 4.4);
                J7 := -0.58655 * (IN_con.F_p / D_h) ^ 2.3 * (IN_con.P_l / IN_con.P_t) ^ (-1.6) * IN_con.N ^ (-0.65);
                J8 := 0.0814 * (log(Re_Dc) - 3);
                j := SMOOTH(900, 1100, Re_Dc) * (14.3117 * Re_Dc ^ J1 * (IN_con.F_p / IN_con.D_c) ^ J2 * (IN_con.L_h / IN_con.L_p) ^ J3 * (IN_con.F_p / IN_con.P_l) ^ J4 * (IN_con.P_l / IN_con.P_t) ^ (-1.724)) + SMOOTH(1100, 900, Re_Dc) * (1.1373 * Re_Dc ^ J5 * (IN_con.F_p / IN_con.P_l) ^ J6 * (IN_con.L_h / IN_con.L_p) ^ J7 * (IN_con.P_l / IN_con.P_t) ^ J8 * IN_con.N ^ 0.3545);
              end if;
              kc := SMOOTH(100, 0, Re_Dc) * j * (Re_Dc * Pr ^ (1 / 3) * IN_var.lambda / IN_con.D_c);
            elseif IN_con.geometry == TYP.SlitFin then
              J1 := (-0.674) + 0.1316 * IN_con.N / log(Re_Dc) - 0.3769 * IN_con.F_p / IN_con.D_c - 1.8857 * IN_con.N / Re_Dc;
              J2 := (-0.0178) + 0.996 * IN_con.N / log(Re_Dc) + 26.7 * IN_con.N / Re_Dc;
              J3 := 1.865 + 1244.03 * IN_con.F_p / (Re_Dc * IN_con.D_c) - 14.37 / log(Re_Dc);
              j := 1.6409 * Re_Dc ^ J1 * (IN_con.S_p / IN_con.S_h) ^ 1.16 * (IN_con.P_t / IN_con.P_l) ^ 1.37 * (IN_con.F_p / IN_con.D_c) ^ J2 * IN_con.N ^ J3;
              kc := j * (Re_Dc * Pr ^ (1 / 3) * IN_var.lambda / IN_con.D_c);
            elseif IN_con.geometry == TYP.WavyFin then
              if Re_Dc < exp(2.921) then
                j := 1.201 / log(exp(2.921) ^ (A_c / IN_con.A_fr)) ^ 2.921;
                kc := j * (exp(2.921) * Pr ^ (1 / 3) * IN_var.lambda / IN_con.D_c);
              else
                j := 1.201 / log(Re_Dc ^ (A_c / IN_con.A_fr)) ^ 2.921;
                kc := j * (Re_Dc * Pr ^ (1 / 3) * IN_var.lambda / IN_con.D_c);
              end if;
            end if;
            annotation(
              Inline = false,
              Documentation(info = "<html>
<p>
Calculation of the mean convective heat transfer coefficient <b> kc </b> for the air-side heat transfer of heat exchangers with round tubes and several fin geometries.<a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.HeatTransfer.HeatExchanger.kc_roundTube\">See more information</a> .
</p>
</html>"),
              smoothOrder(normallyConstant = IN_con) = 2);
          end kc_roundTube_KC;

          record kc_roundTube_IN_con "Input record for function kc_roundTube and kc_roundTube_KC"
            extends Modelica.Icons.Record;
            Modelica.Fluid.Dissipation.Utilities.Types.HTXGeometry_roundTubes geometry = Dissipation.Utilities.Types.HTXGeometry_roundTubes.PlainFin "Choice of fin geometry" annotation(
              Dialog(group = "HeatExchanger"));
            SI.Area A_fr = 0 "Frontal area" annotation(
              Dialog(group = "HeatExchanger"));
            SI.Length D_c = 0 "Fin collar diameter" annotation(
              Dialog(group = "HeatExchanger"));
            SI.Length F_p = 0 "Fin pitch, fin spacing + fin thickness" annotation(
              Dialog(group = "HeatExchanger"));
            SI.Length L = 0 "Heat exchanger length" annotation(
              Dialog(group = "HeatExchanger", enable = if geometry == 2 then true else false));
            SI.Length L_h = 0 "Louver height" annotation(
              Dialog(group = "HeatExchanger", enable = if geometry == 2 then true else false));
            SI.Length L_p = 0 "Louver pitch" annotation(
              Dialog(group = "HeatExchanger", enable = if geometry == 2 then true else false));
            Integer N = 0 "Number of tube rows" annotation(
              Dialog(group = "HeatExchanger", enable = if geometry == 1 or geometry == 2 or geometry == 3 then true else false));
            SI.Length P_d = 0 "Pattern depth of wavy fin, wave height" annotation(
              Dialog(group = "HeatExchanger", enable = if geometry == 4 then true else false));
            SI.Length P_l = 0 "Longitudinal tube pitch" annotation(
              Dialog(group = "HeatExchanger", enable = if geometry == 1 or geometry == 2 or geometry == 3 then true else false));
            SI.Length P_t = 0 "Transverse tube pitch" annotation(
              Dialog(group = "HeatExchanger"));
            SI.Length S_h = 0 "Slit height" annotation(
              Dialog(group = "HeatExchanger", enable = if geometry == 3 then true else false));
            SI.Length S_p = 0 "Slit pitch" annotation(
              Dialog(group = "HeatExchanger", enable = if geometry == 3 then true else false));
            SI.Length X_f = 0 "Half wave length of wavy fin" annotation(
              Dialog(group = "HeatExchanger", enable = if geometry == 4 then true else false));
            SI.Length delta_f = 0 "Fin thickness" annotation(
              Dialog(group = "HeatExchanger"));
            annotation(
              Documentation(info = "<html>
This record is used as <b> input record </b> for the heat transfer function <a href=\"Modelica://Modelica.Fluid.Dissipation.HeatTransfer.HeatExchanger.kc_roundTube\"> kc_roundTube</a> and
<a href=\"Modelica://Modelica.Fluid.Dissipation.HeatTransfer.HeatExchanger.kc_roundTube_KC\"> kc_roundTube_KC</a>.
</html>"));
          end kc_roundTube_IN_con;

          record kc_roundTube_IN_var "Input record for function kc_roundTube and kc_roundTube_KC"
            extends Modelica.Icons.Record;
            //fluid property variables
            extends Modelica.Fluid.Dissipation.Utilities.Records.General.FluidProperties;
            //input variable (mass flow rate)
            SI.MassFlowRate m_flow annotation(
              Dialog(group = "Input"));
            annotation(
              Documentation(info = "<html>
This record is used as <b> input record </b> for the heat transfer function <a href=\"Modelica://Modelica.Fluid.Dissipation.HeatTransfer.HeatExchanger.kc_roundTube\"> kc_roundTube</a> and
<a href=\"Modelica://Modelica.Fluid.Dissipation.HeatTransfer.HeatExchanger.kc_roundTube_KC\"> kc_roundTube_KC</a>.
</html>"));
          end kc_roundTube_IN_var;
          annotation(
            preferredView = "info",
            Documentation(info = "<html>
<h4>Heat exchanger</h4>
<h5>Flat tube heat exchanger</h5>
<p>
Calculation of the mean convective heat transfer coefficient <b> kc </b> for the air-side heat transfer of heat exchangers with flat tubes and several fin geometries.
<a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.HeatTransfer.HeatExchanger.kc_flatTube\">See more information</a> .
</p>

<h5>Round tube heat exchanger</h5>
<p>
Calculation of the mean convective heat transfer coefficient <b> kc </b> for the air-side heat transfer of heat exchangers with round tubes and several fin geometries.
<a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.HeatTransfer.HeatExchanger.kc_roundTube\">See more information</a> .
</p>
</html>"));
        end HeatExchanger;

        package HelicalPipe
          extends Modelica.Icons.VariantsPackage;

          function kc_laminar "Mean heat transfer coefficient of helical pipe | laminar flow regime"
            extends Modelica.Icons.Function;
            //SOURCE: VDI-Waermeatlas, 9th edition, Springer-Verlag, 2002, section Gc1 - Gc2
            //Notation of equations according to SOURCE
            //input records
            input Modelica.Fluid.Dissipation.HeatTransfer.HelicalPipe.kc_laminar_IN_con IN_con "Input record for function kc_laminar" annotation(
              Dialog(group = "Constant inputs"));
            input Modelica.Fluid.Dissipation.HeatTransfer.HelicalPipe.kc_laminar_IN_var IN_var "Input record for function kc_laminar" annotation(
              Dialog(group = "Variable inputs"));
            //output variables
            output SI.CoefficientOfHeatTransfer kc "Convective heat transfer coefficient" annotation(
              Dialog(group = "Output"));
            output SI.PrandtlNumber Pr "Prandtl number" annotation(
              Dialog(group = "Output"));
            output SI.ReynoldsNumber Re "Reynolds number" annotation(
              Dialog(group = "Output"));
            output SI.NusseltNumber Nu "Nusselt number" annotation(
              Dialog(group = "Output"));
            output Real failureStatus "0== boundary conditions fulfilled | 1== failure >> check if still meaningful results" annotation(
              Dialog(group = "Output"));
          protected
            Real MIN = Modelica.Constants.eps;
            SI.Diameter d_hyd = IN_con.d_hyd "Hydraulic diameter";
            SI.Area A_cross = PI * IN_con.d_hyd ^ 2 / 4 "Circular cross sectional area";
            SI.Diameter d_s = IN_con.L / (IN_con.n_nt * PI) "Mean coil diameter";
            SI.Diameter d_w = sqrt(max(MIN, d_s ^ 2 - (IN_con.h / PI) ^ 2)) "Mean helical pipe diameter";
            SI.Diameter d_coil = max(d_w, d_w * (1 + (IN_con.h / (PI * d_w)) ^ 2)) "Mean curvature diameter of helical pipe";
            SI.ReynoldsNumber Re_crit = 2300 * (1 + 8.6 * (IN_con.d_hyd / d_coil) ^ 0.45) "Critical Reynolds number";
            SI.Velocity velocity = abs(IN_var.m_flow) / max(MIN, IN_var.rho * A_cross) "Mean velocity";
            //failure status
            Real fstatus[1] "Check of expected boundary conditions";
            //Documentation
          algorithm
            Pr := abs(IN_var.eta * IN_var.cp / max(MIN, IN_var.lambda));
            Re := max(1, abs(IN_var.rho * velocity * IN_con.d_hyd / max(MIN, IN_var.eta)));
            kc := Modelica.Fluid.Dissipation.HeatTransfer.HelicalPipe.kc_laminar_KC(IN_con, IN_var);
            Nu := kc * IN_con.d_hyd / max(MIN, IN_var.lambda);
//failure status
            fstatus[1] := if Re > Re_crit then 1 else 0;
            failureStatus := 0;
            for i in 1:size(fstatus, 1) loop
              if fstatus[i] == 1 then
                failureStatus := 1;
              end if;
            end for;
            annotation(
              Inline = false,
              Documentation(info = "<html>
<p>
Calculation of the mean convective heat transfer coefficient <b> kc </b> for a helical pipe in the laminar flow regime. Note that additionally a failure status is observed in this function to check if the intended boundary conditions are fulfilled.
<a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.HeatTransfer.HelicalPipe.kc_laminar\">See more information</a> .
</p>
</html>"),
              smoothOrder(normallyConstant = IN_con) = 2);
          end kc_laminar;

          function kc_laminar_KC "Mean heat transfer coefficient of helical pipe | hydrodynamically developed laminar flow regime"
            extends Modelica.Icons.Function;
            //SOURCE: VDI-Waermeatlas, 9th edition, Springer-Verlag, 2002, section Gc1 - Gc2
            //Notation of equations according to SOURCE
            //input records
            input Modelica.Fluid.Dissipation.HeatTransfer.HelicalPipe.kc_laminar_IN_con IN_con "Input record for function kc_laminar_KC" annotation(
              Dialog(group = "Constant inputs"));
            input Modelica.Fluid.Dissipation.HeatTransfer.HelicalPipe.kc_laminar_IN_var IN_var "Input record for function kc_laminar_KC" annotation(
              Dialog(group = "Variable inputs"));
            //output variables
            output SI.CoefficientOfHeatTransfer kc "Output for function kc_laminar_KC";
          protected
            Real MIN = Modelica.Constants.eps;
            SI.Diameter d_hyd = IN_con.d_hyd "Hydraulic diameter";
            SI.Area A_cross = PI * IN_con.d_hyd ^ 2 / 4 "Circular cross sectional area";
            SI.Diameter d_s = IN_con.L / (IN_con.n_nt * PI) "Mean coil diameter";
            SI.Diameter d_w = sqrt(max(MIN, d_s ^ 2 - (IN_con.h / PI) ^ 2)) "Mean helical pipe diameter";
            SI.Diameter d_coil = max(d_w, d_w * (1 + (IN_con.h / (PI * d_w)) ^ 2)) "Mean curvature diameter of helical pipe";
            SI.Velocity velocity = abs(IN_var.m_flow) / max(MIN, IN_var.rho * A_cross) "Mean velocity";
            SI.ReynoldsNumber Re = max(1, IN_var.rho * velocity * IN_con.d_hyd / max(MIN, IN_var.eta));
            SI.PrandtlNumber Pr = abs(IN_var.eta * IN_var.cp / max(MIN, IN_var.lambda));
            Real m = 0.5 + 0.2903 * (IN_con.d_hyd / d_coil) ^ 0.194 "Exponent for actual Reynolds number";
            //Documentation
          algorithm
            kc := IN_var.lambda / IN_con.d_hyd * (3.66 + 0.08 * (1 + 0.8 * (IN_con.d_hyd / d_coil) ^ 0.9) * Re ^ m * Pr ^ (1 / 3));
            annotation(
              Inline = false,
              Documentation(info = "<html>
<p>
Calculation of the mean convective heat transfer coefficient <b> kc </b> for a helical pipe in the laminar flow regime.
<a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.HeatTransfer.HelicalPipe.kc_laminar\">See more information</a> .
</p>
</html>"),
              smoothOrder(normallyConstant = IN_con) = 2);
          end kc_laminar_KC;

          record kc_laminar_IN_con "Input record for function kc_laminar and kc_laminar_KC"
            extends Modelica.Fluid.Dissipation.HeatTransfer.HelicalPipe.kc_overall_IN_con;
            annotation(
              Documentation(info = "<html>
This record is used as <b> input record </b> for the heat transfer function <a href=\"Modelica://Modelica.Fluid.Dissipation.HeatTransfer.HelicalPipe.kc_laminar\"> kc_laminar</a> and
<a href=\"Modelica://Modelica.Fluid.Dissipation.HeatTransfer.HelicalPipe.kc_laminar_KC\"> kc_laminar_KC</a>.
</html>"));
          end kc_laminar_IN_con;

          record kc_laminar_IN_var "Input record for function kc_laminar and kc_laminar_KC"
            extends Modelica.Fluid.Dissipation.HeatTransfer.HelicalPipe.kc_overall_IN_var;
            annotation(
              Documentation(info = "<html>
This record is used as <b> input record </b> for the heat transfer function <a href=\"Modelica://Modelica.Fluid.Dissipation.HeatTransfer.HelicalPipe.kc_laminar\"> kc_laminar</a> and
<a href=\"Modelica://Modelica.Fluid.Dissipation.HeatTransfer.HelicalPipe.kc_laminar_KC\"> kc_laminar_KC</a>.
</html>"));
          end kc_laminar_IN_var;

          function kc_overall "Mean heat transfer coefficient of helical pipe | overall flow regime"
            extends Modelica.Icons.Function;
            //SOURCE: VDI-Waermeatlas, 9th edition, Springer-Verlag, 2002, section Gc1 - Gc2
            //Notation of equations according to SOURCE
            //input records
            input Modelica.Fluid.Dissipation.HeatTransfer.HelicalPipe.kc_overall_IN_con IN_con "Input record for function kc_overall" annotation(
              Dialog(group = "Constant inputs"));
            input Modelica.Fluid.Dissipation.HeatTransfer.HelicalPipe.kc_overall_IN_var IN_var "Input record for function kc_overall" annotation(
              Dialog(group = "Variable inputs"));
            //output variables
            output SI.CoefficientOfHeatTransfer kc "Convective heat transfer coefficient" annotation(
              Dialog(group = "Output"));
            output SI.PrandtlNumber Pr "Prandtl number" annotation(
              Dialog(group = "Output"));
            output SI.ReynoldsNumber Re "Reynolds number" annotation(
              Dialog(group = "Output"));
            output SI.NusseltNumber Nu "Nusselt number" annotation(
              Dialog(group = "Output"));
            output Real failureStatus "0== boundary conditions fulfilled | 1== failure >> check if still meaningful results" annotation(
              Dialog(group = "Output"));
          protected
            Real MIN = Modelica.Constants.eps;
            SI.Area A_cross = PI * IN_con.d_hyd ^ 2 / 4 "Cross sectional area";
            SI.Velocity velocity = abs(IN_var.m_flow) / max(MIN, IN_var.rho * A_cross) "Mean velocity";
            //Documentation
          algorithm
            Pr := abs(IN_var.eta * IN_var.cp / max(MIN, IN_var.lambda));
            Re := max(MIN, IN_var.rho * velocity * IN_con.d_hyd / max(MIN, IN_var.eta));
            kc := Modelica.Fluid.Dissipation.HeatTransfer.HelicalPipe.kc_overall_KC(IN_con, IN_var);
            Nu := kc * IN_con.d_hyd / max(MIN, IN_var.lambda);
            failureStatus := 0;
            annotation(
              Inline = false,
              Documentation(info = "<html>
<p>
Calculation of the mean convective heat transfer coefficient <b> kc </b> of a helical pipe in a hydrodynamically developed laminar and turbulent flow regime. Note that additionally a failure status is observed in this function to check if the intended boundary conditions are fulfilled.
<a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.HeatTransfer.HelicalPipe.kc_overall\">See more information</a> .
</p>
</html>"),
              smoothOrder(normallyConstant = IN_con) = 2);
          end kc_overall;

          function kc_overall_KC "Mean heat transfer coefficient of helical pipe | overall flow regime"
            extends Modelica.Icons.Function;
            //SOURCE: VDI-Waermeatlas, 9th edition, Springer-Verlag, 2002, section Gc1 - Gc2
            //Notation of equations according to SOURCE
            import SMOOTH = Modelica.Fluid.Dissipation.Utilities.Functions.General.Stepsmoother;
            //input records
            input Modelica.Fluid.Dissipation.HeatTransfer.HelicalPipe.kc_overall_IN_con IN_con "Input record for function kc_overall_KC" annotation(
              Dialog(group = "Constant inputs"));
            input Modelica.Fluid.Dissipation.HeatTransfer.HelicalPipe.kc_overall_IN_var IN_var "Input record for function kc_overall_KC" annotation(
              Dialog(group = "Variable inputs"));
            //output variables
            output SI.CoefficientOfHeatTransfer kc "Output for function kc_overall_KC";
          protected
            Real MIN = Modelica.Constants.eps;
            Real laminar = 2e3 "Maximum Reynolds number for laminar regime";
            Real turbulent = 2.2e4 "Minimum Reynolds number for turbulent regime";
            SI.Area A_cross = PI * IN_con.d_hyd ^ 2 / 4 "Cross sectional area";
            SI.Diameter d_s = max(1e-6, IN_con.L / (IN_con.n_nt * PI)) "Mean coil diameter";
            SI.Diameter d_w = sqrt(max(MIN, abs(d_s ^ 2 - (IN_con.h / PI) ^ 2))) "Mean helical pipe diameter";
            SI.Diameter d_coil = d_w * (1 + (IN_con.h / (PI * d_w)) ^ 2) "Mean curvature diameter of helical pipe";
            SI.ReynoldsNumber Re_crit = min(4e3, 2300 * (1 + 8.6 * (IN_con.d_hyd / d_coil) ^ 0.45)) "Critical Reynolds number";
            SI.Velocity velocity = abs(IN_var.m_flow) / max(MIN, IN_var.rho * A_cross) "Mean velocity";
            SI.ReynoldsNumber Re = max(1e-3, IN_var.rho * velocity * IN_con.d_hyd / max(MIN, IN_var.eta));
            SI.PrandtlNumber Pr = abs(IN_var.eta * IN_var.cp / max(MIN, IN_var.lambda));
            //Documentation
          algorithm
            kc := SMOOTH(Re_crit, turbulent, Re) * Dissipation.HeatTransfer.HelicalPipe.kc_laminar_KC(IN_con, IN_var) + SMOOTH(turbulent, Re_crit, Re) * Dissipation.HeatTransfer.HelicalPipe.kc_turbulent_KC(IN_con, IN_var);
            annotation(
              Inline = false,
              Documentation(info = "<html>
<p>
Calculation of the mean convective heat transfer coefficient <b> kc </b> of a helical pipe in a hydrodynamically developed laminar and turbulent flow regime.
<a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.HeatTransfer.HelicalPipe.kc_overall\">See more information</a> .
</p>
</html>"),
              smoothOrder(normallyConstant = IN_con) = 2);
          end kc_overall_KC;

          record kc_overall_IN_con "Input record for function kc_overall and kc_overall_KC"
            //helical pipe variables
            extends Modelica.Fluid.Dissipation.Utilities.Records.HeatTransfer.HelicalPipe;
            annotation(
              Documentation(info = "<html>
This record is used as <b> input record </b> for the heat transfer function <a href=\"Modelica://Modelica.Fluid.Dissipation.HeatTransfer.HelicalPipe.kc_overall\"> kc_overall</a> and
<a href=\"Modelica://Modelica.Fluid.Dissipation.HeatTransfer.HelicalPipe.kc_overall_KC\"> kc_overall_KC</a>.
</html>"));
          end kc_overall_IN_con;

          record kc_overall_IN_var "Input record for function kc_overall and kc_overall_KC"
            //fluid property variables
            extends Modelica.Fluid.Dissipation.Utilities.Records.General.FluidProperties;
            //input variable (mass flow rate)
            SI.MassFlowRate m_flow annotation(
              Dialog(group = "Input"));
            annotation(
              Documentation(info = "<html>
This record is used as <b> input record </b> for the heat transfer function <a href=\"Modelica://Modelica.Fluid.Dissipation.HeatTransfer.HelicalPipe.kc_overall\"> kc_overall</a> and
<a href=\"Modelica://Modelica.Fluid.Dissipation.HeatTransfer.HelicalPipe.kc_overall_KC\"> kc_overall_KC</a>.
</html>"));
          end kc_overall_IN_var;

          function kc_turbulent "Mean heat transfer coefficient of helical pipe | hydrodynamically developed turbulent flow regime"
            extends Modelica.Icons.Function;
            //SOURCE: VDI-Waermeatlas, 9th edition, Springer-Verlag, 2002, section Gc1 - Gc2
            //Notation of equations according to SOURCE
            //input records
            input Modelica.Fluid.Dissipation.HeatTransfer.HelicalPipe.kc_turbulent_IN_con IN_con "Input record for function kc_turbulent" annotation(
              Dialog(group = "Constant inputs"));
            input Modelica.Fluid.Dissipation.HeatTransfer.HelicalPipe.kc_turbulent_IN_var IN_var "Input record for function kc_turbulent" annotation(
              Dialog(group = "Variable inputs"));
            //output variables
            output SI.CoefficientOfHeatTransfer kc "Convective heat transfer coefficient" annotation(
              Dialog(group = "Output"));
            output SI.PrandtlNumber Pr "Prandtl number" annotation(
              Dialog(group = "Output"));
            output SI.ReynoldsNumber Re "Reynolds number" annotation(
              Dialog(group = "Output"));
            output SI.NusseltNumber Nu "Nusselt number" annotation(
              Dialog(group = "Output"));
            output Real failureStatus "0== boundary conditions fulfilled | 1== failure >> check if still meaningful results" annotation(
              Dialog(group = "Output"));
          protected
            Real MIN = Modelica.Constants.eps;
            Real turbulent = 2.2e4 "Minimum Reynolds number for turbulent regime";
            SI.Area A_cross = PI * IN_con.d_hyd ^ 2 / 4 "Cross sectional area";
            SI.Velocity velocity = abs(IN_var.m_flow) / max(MIN, IN_var.rho * A_cross) "Mean velocity";
            //failure status
            Real fstatus[1] "Check of expected boundary conditions";
            //Documentation
          algorithm
            Pr := abs(IN_var.eta * IN_var.cp / IN_var.lambda);
            Re := max(1e-3, abs(IN_var.rho * velocity * IN_con.d_hyd / IN_var.eta));
            kc := Modelica.Fluid.Dissipation.HeatTransfer.HelicalPipe.kc_turbulent_KC(IN_con, IN_var);
            Nu := kc * IN_con.d_hyd / max(MIN, IN_var.lambda);
//failure status
            fstatus[1] := if Re < turbulent then 1 else 0;
            failureStatus := 0;
            for i in 1:size(fstatus, 1) loop
              if fstatus[i] == 1 then
                failureStatus := 1;
              end if;
            end for;
            annotation(
              Inline = false,
              Documentation(info = "<html>
<p>
Calculation of the mean convective heat transfer coefficient <b> kc </b> of a helical pipe for turbulent flow regime. Note that additionally a failure status is observed in this function to check if the intended boundary conditions are fulfilled.
<a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.HeatTransfer.HelicalPipe.kc_turbulent\">See more information</a> .
</p>
</html>"),
              smoothOrder(normallyConstant = IN_con) = 2);
          end kc_turbulent;

          function kc_turbulent_KC "Mean heat transfer coefficient of helical pipe | hydrodynamically developed turbulent flow regime"
            extends Modelica.Icons.Function;
            //SOURCE: VDI-Waermeatlas, 9th edition, Springer-Verlag, 2002, section Gc1 - Gc2
            //Notation of equations according to SOURCE
            //input records
            input Modelica.Fluid.Dissipation.HeatTransfer.HelicalPipe.kc_turbulent_IN_con IN_con "Input record for function kc_turbulent_KC" annotation(
              Dialog(group = "Constant inputs"));
            input Modelica.Fluid.Dissipation.HeatTransfer.HelicalPipe.kc_turbulent_IN_var IN_var "Input record for function kc_turbulent_KC" annotation(
              Dialog(group = "Variable inputs"));
            //output variables
            output SI.CoefficientOfHeatTransfer kc "Output for function kc_turbulent_KC";
          protected
            Real MIN = Modelica.Constants.eps;
            Real turbulent = 2.2e4 "Minimum Reynolds number for turbulent regime";
            SI.Diameter d_hyd = IN_con.d_hyd "Hydraulic diameter";
            SI.Area A_cross = PI * IN_con.d_hyd ^ 2 / 4 "Circular cross sectional area";
            SI.Diameter d_s = IN_con.L / (IN_con.n_nt * PI) "Mean coil diameter";
            SI.Diameter d_w = sqrt(max(MIN, d_s ^ 2 - (IN_con.h / PI) ^ 2)) "Mean helical pipe diameter";
            SI.Diameter d_coil = max(d_w, d_w * (1 + (IN_con.h / (PI * d_w)) ^ 2)) "Mean curvature diameter of helical pipe";
            SI.Velocity velocity = abs(IN_var.m_flow) / max(MIN, IN_var.rho * A_cross) "Mean velocity";
            SI.ReynoldsNumber Re = max(1e-3, IN_var.rho * velocity * IN_con.d_hyd / max(MIN, IN_var.eta));
            SI.PrandtlNumber Pr = abs(IN_var.eta * IN_var.cp / max(MIN, IN_var.lambda));
            Real zeta_TOT = 0.3164 * max(turbulent, Re) ^ (-0.25) + 0.03 * sqrt(IN_con.d_hyd / d_coil) "Pressure loss coefficient";
            //Documentation
          algorithm
            kc := IN_var.lambda / IN_con.d_hyd * (zeta_TOT / 8) * Re * Pr / (1 + 12.7 * sqrt(zeta_TOT / 8) * (Pr ^ (2 / 3) - 1));
            annotation(
              Inline = false,
              Documentation(info = "<html>
<p>
Calculation of the mean convective heat transfer coefficient <b> kc </b> of a helical pipe for turbulent flow regime.
<a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.HeatTransfer.HelicalPipe.kc_turbulent\">See more information</a> .
</p>
</html>"),
              smoothOrder(normallyConstant = IN_con) = 2);
          end kc_turbulent_KC;

          record kc_turbulent_IN_con "Input record for function kc_turbulent and kc_turbulent_KC"
            extends Modelica.Fluid.Dissipation.HeatTransfer.HelicalPipe.kc_overall_IN_con;
            annotation(
              Documentation(info = "<html>
This record is used as <b> input record </b> for the heat transfer function <a href=\"Modelica://Modelica.Fluid.Dissipation.HeatTransfer.HelicalPipe.kc_turbulent\">  kc_turbulent</a> and
<a href=\"Modelica://Modelica.Fluid.Dissipation.HeatTransfer.HelicalPipe.kc_turbulent_KC\">  kc_turbulent_KC</a>.
</html>"));
          end kc_turbulent_IN_con;

          record kc_turbulent_IN_var "Input record for function kc_turbulent and kc_turbulent_KC"
            extends Modelica.Fluid.Dissipation.HeatTransfer.HelicalPipe.kc_overall_IN_var;
            annotation(
              Documentation(info = "<html>
This record is used as <b> input record </b> for the heat transfer function <a href=\"Modelica://Modelica.Fluid.Dissipation.HeatTransfer.HelicalPipe.kc_turbulent\">  kc_turbulent</a> and
<a href=\"Modelica://Modelica.Fluid.Dissipation.HeatTransfer.HelicalPipe.kc_turbulent_KC\">  kc_turbulent_KC</a>.
</html>"));
          end kc_turbulent_IN_var;
          annotation(
            preferredView = "info",
            Documentation(info = "<html>
<h4>Helical pipe</h4>
<h5>Laminar flow</h5>
<p>Calculation of the mean convective heat transfer coefficient <b> kc </b> for a helical pipe in the laminar flow regime.
<a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.HeatTransfer.HelicalPipe.kc_laminar\">See more information</a>.</p>

<h5>Turbulent flow</h5>
<p>Calculation of the mean convective heat transfer coefficient <b> kc </b> of a helical pipe for turbulent flow regime.
<a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.HeatTransfer.HelicalPipe.kc_turbulent\">See more information</a>.</p>

<h5>Overall flow</h5>
<p>Calculation of the mean convective heat transfer coefficient <b> kc </b> of a helical pipe in a hydrodynamically developed laminar and turbulent flow regime.
<a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.HeatTransfer.HelicalPipe.kc_overall\">See more information</a>.</p>
</html>"));
        end HelicalPipe;

        package Plate
          extends Modelica.Icons.VariantsPackage;

          function kc_laminar "Mean heat transfer coefficient of plate | laminar regime"
            extends Modelica.Icons.Function;
            //SOURCE: VDI-Waermeatlas, Aufl. 9, Springer-Verlag, 2002, Section Gd 1
            //Notation of equations according to SOURCE
            //input records
            input Modelica.Fluid.Dissipation.HeatTransfer.Plate.kc_laminar_IN_con IN_con "Input record for function kc_laminar" annotation(
              Dialog(group = "Constant inputs"));
            input Modelica.Fluid.Dissipation.HeatTransfer.Plate.kc_laminar_IN_var IN_var "Input record for function kc_laminar" annotation(
              Dialog(group = "Variable inputs"));
            //output variables
            output SI.CoefficientOfHeatTransfer kc "Convective heat transfer coefficient" annotation(
              Dialog(group = "Output"));
            output SI.PrandtlNumber Pr "Prandtl number" annotation(
              Dialog(group = "Output"));
            output SI.ReynoldsNumber Re "Reynolds number" annotation(
              Dialog(group = "Output"));
            output SI.NusseltNumber Nu "Nusselt number" annotation(
              Dialog(group = "Output"));
            output Real failureStatus "0== boundary conditions fulfilled | 1== failure >> check if still meaningful results" annotation(
              Dialog(group = "Output"));
          protected
            Real MIN = Modelica.Constants.eps;
            Real laminar = 1e5 "Maximum Reynolds number of laminar flow regime";
            Real prandtlMax = 2000 "Maximum Prandtl number";
            Real prandtlMin = 0.6 "Minimum Prandtl number";
            //failure status
            Real fstatus[2] "Check of expected boundary conditions";
            //Documentation
          algorithm
            Pr := IN_var.eta * IN_var.cp / max(MIN, IN_var.lambda);
            Re := max(1e-3, abs(IN_var.rho * IN_var.velocity * IN_con.L / max(MIN, IN_var.eta)));
            kc := Modelica.Fluid.Dissipation.HeatTransfer.Plate.kc_laminar_KC(IN_con, IN_var);
            Nu := kc * IN_con.L / max(MIN, IN_var.lambda);
//failure status
            fstatus[1] := if Re > laminar then 1 else 0;
            fstatus[2] := if Pr > prandtlMax or Pr < prandtlMin then 1 else 0;
            failureStatus := 0;
            for i in 1:size(fstatus, 1) loop
              if fstatus[i] == 1 then
                failureStatus := 1;
              end if;
            end for;
            annotation(
              Inline = false,
              Documentation(info = "<html>
<p>
Calculation of the mean convective heat transfer coefficient <b> kc </b> for a laminar fluid flow over an even surface. Note that additionally a failure status is observed in this function to check if the intended boundary conditions are fulfilled. <a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.HeatTransfer.Plate.kc_laminar\">See more information</a> .
</p>
</html>"),
              smoothOrder(normallyConstant = IN_con) = 2);
          end kc_laminar;

          function kc_laminar_KC "Mean heat transfer coefficient of plate | laminar regime"
            extends Modelica.Icons.Function;
            //SOURCE: VDI-Waermeatlas, Aufl. 9, Springer-Verlag, 2002, Section Gd 1
            //Notation of equations according to SOURCE
            //input records
            input Modelica.Fluid.Dissipation.HeatTransfer.Plate.kc_laminar_IN_con IN_con "Input record for function kc_laminar_KC" annotation(
              Dialog(group = "Constant inputs"));
            input Modelica.Fluid.Dissipation.HeatTransfer.Plate.kc_laminar_IN_var IN_var "Input record for function kc_laminar_KC" annotation(
              Dialog(group = "Variable inputs"));
            //output variables
            output SI.CoefficientOfHeatTransfer kc "Output for function kc_laminar_KC";
          protected
            Real MIN = Modelica.Constants.eps;
            SI.Length L = max(MIN, IN_con.L);
            SI.SpecificHeatCapacityAtConstantPressure cp = IN_var.cp;
            SI.DynamicViscosity eta = max(MIN, IN_var.eta);
            SI.ThermalConductivity lambda = max(MIN, IN_var.lambda);
            SI.Density rho = IN_var.rho;
            SI.Velocity velocity = abs(IN_var.velocity) "Mean velocity";
            SI.ReynoldsNumber Re = max(1e-3, rho * velocity * L / eta);
            SI.PrandtlNumber Pr = eta * cp / lambda;
            //Documentation
          algorithm
            kc := lambda / L * (0.664 * abs(Re) ^ 0.5 * Pr ^ (1 / 3));
            annotation(
              Inline = true,
              Documentation(info = "<html>
<p>
Calculation of the mean convective heat transfer coefficient <b> kc </b> for a laminar fluid flow over an even surface.
Generally this function is numerically best used for the calculation of the mean convective heat transfer coefficient <b> kc </b> at known fluid velocity.
 <a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.HeatTransfer.Plate.kc_laminar\">See more information</a> .
</p>
</html>"),
              smoothOrder(normallyConstant = IN_con) = 2);
          end kc_laminar_KC;

          record kc_laminar_IN_con "Input record for function kc_laminar and kc_laminar_KC"
            extends Modelica.Fluid.Dissipation.HeatTransfer.Plate.kc_overall_IN_con;
            annotation(
              Documentation(info = "<html>
<p>
This record is used as <b> input record </b> for the heat transfer function <a href=\"modelica://Modelica.Fluid.Dissipation.HeatTransfer.Plate.kc_laminar\">  kc_laminar</a> and
<a href=\"modelica://Modelica.Fluid.Dissipation.HeatTransfer.Plate.kc_laminar_KC\">  kc_laminar_KC</a>.
</p>
</html>"));
          end kc_laminar_IN_con;

          record kc_laminar_IN_var "Input record for function kc_laminar and kc_laminar_KC"
            extends Modelica.Fluid.Dissipation.HeatTransfer.Plate.kc_overall_IN_var;
            annotation(
              Documentation(info = "<html>
<p>
This record is used as <b> input record </b> for the heat transfer function <a href=\"modelica://Modelica.Fluid.Dissipation.HeatTransfer.Plate.kc_laminar\">  kc_laminar</a> and
<a href=\"modelica://Modelica.Fluid.Dissipation.HeatTransfer.Plate.kc_laminar_KC\">  kc_laminar_KC</a>.
</p>
</html>"));
          end kc_laminar_IN_var;

          function kc_overall "Mean heat transfer coefficient of even plate | overall regime | constant wall temperature"
            extends Modelica.Icons.Function;
            //SOURCE: VDI-Waermeatlas, Aufl. 9, Springer-Verlag, 2002, Section Gd 1
            //Notation of equations according to SOURCE
            //input records
            input Modelica.Fluid.Dissipation.HeatTransfer.Plate.kc_overall_IN_con IN_con "Input record for function kc_overall" annotation(
              Dialog(group = "Constant inputs"));
            input Modelica.Fluid.Dissipation.HeatTransfer.Plate.kc_overall_IN_var IN_var "Input record for function kc_overall" annotation(
              Dialog(group = "Variable inputs"));
            //output variables
            output SI.CoefficientOfHeatTransfer kc "Convective heat transfer coefficient" annotation(
              Dialog(group = "Output"));
            output SI.PrandtlNumber Pr "Prandtl number" annotation(
              Dialog(group = "Output"));
            output SI.ReynoldsNumber Re "Reynolds number" annotation(
              Dialog(group = "Output"));
            output SI.NusseltNumber Nu "Nusselt number" annotation(
              Dialog(group = "Output"));
            output Real failureStatus "0== boundary conditions fulfilled | 1== failure >> check if still meaningful results" annotation(
              Dialog(group = "Output"));
          protected
            Real MIN = Modelica.Constants.eps;
            Real prandtlMax = 2000 "Maximum Prandtl number";
            Real prandtlMin = 0.6 "Minimum Prandtl number";
            Real reynoldsMax = 1e7 "Maximum Reynolds number";
            Real reynoldsMin = 1e1 "Minimum Reynolds number";
            //failure status
            Real fstatus[2] "Check of expected boundary conditions";
            //Documentation
          algorithm
            Pr := IN_var.eta * IN_var.cp / max(MIN, IN_var.lambda);
            Re := max(1e-3, abs(IN_var.rho * IN_var.velocity * IN_con.L / max(MIN, IN_var.eta)));
            kc := Modelica.Fluid.Dissipation.HeatTransfer.Plate.kc_overall_KC(IN_con, IN_var);
            Nu := kc * IN_con.L / max(MIN, IN_var.lambda);
//failure status
            fstatus[1] := if Re > reynoldsMax or Re < reynoldsMin then 1 else 0;
            fstatus[2] := if Pr > prandtlMax or Pr < prandtlMin then 1 else 0;
            failureStatus := 0;
            for i in 1:size(fstatus, 1) loop
              if fstatus[i] == 1 then
                failureStatus := 1;
              end if;
            end for;
            annotation(
              Inline = false,
              Documentation(info = "<html>
<p>
Calculation of the mean convective heat transfer coefficient <b> kc </b> for a laminar or turbulent fluid flow over an even surface. Note that additionally a failure status is observed in this function to check if the intended boundary conditions are fulfilled.  <a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.HeatTransfer.Plate.kc_overall\">See more information</a> .
</p>
</html>"),
              smoothOrder(normallyConstant = IN_con) = 2);
          end kc_overall;

          function kc_overall_KC "Mean heat transfer coefficient of even plate | overall regime | constant wall temperature"
            extends Modelica.Icons.Function;
            //SOURCE: VDI-Waermeatlas, Aufl. 9, Springer-Verlag, 2002, Section Gd 1
            //Notation of equations according to SOURCE
            //input records
            input Modelica.Fluid.Dissipation.HeatTransfer.Plate.kc_overall_IN_con IN_con "Input record for function kc_overall_KC" annotation(
              Dialog(group = "Constant inputs"));
            input Modelica.Fluid.Dissipation.HeatTransfer.Plate.kc_overall_IN_var IN_var "Input record for function kc_overall_KC" annotation(
              Dialog(group = "Variable inputs"));
            //output variables
            output SI.CoefficientOfHeatTransfer kc "Output for function kc_overall_KC";
          protected
            SI.CoefficientOfHeatTransfer kc_lam = Modelica.Fluid.Dissipation.HeatTransfer.Plate.kc_laminar_KC(IN_con, IN_var);
            SI.CoefficientOfHeatTransfer kc_turb = Modelica.Fluid.Dissipation.HeatTransfer.Plate.kc_turbulent_KC(IN_con, IN_var);
            //Documentation
          algorithm
            kc := sqrt(kc_lam ^ 2 + kc_turb ^ 2);
            annotation(
              Inline = true,
              Documentation(info = "<html>
<p>
Calculation of the mean convective heat transfer coefficient <b> kc </b> for a laminar or turbulent fluid flow over an even surface.
Generally this function is numerically best used for the calculation of the mean convective heat transfer coefficient <b> kc </b> at known fluid velocity.
 <a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.HeatTransfer.Plate.kc_overall\">See more information</a> .
</p>
</html>"),
              smoothOrder(normallyConstant = IN_con) = 2);
          end kc_overall_KC;

          record kc_overall_IN_con "Input record for function kc_overall and function kc_overall_KC"
            //plate variables
            extends Modelica.Fluid.Dissipation.Utilities.Records.HeatTransfer.Plate;
            annotation(
              Documentation(info = "<html>
This record is used as <b> input record </b> for the heat transfer function <a href=\"modelica://Modelica.Fluid.Dissipation.HeatTransfer.Plate.kc_overall\"> kc_overall</a> and
<a href=\"modelica://Modelica.Fluid.Dissipation.HeatTransfer.Plate.kc_overall_KC\"> kc_overall_KC</a>.
</html>"));
          end kc_overall_IN_con;

          record kc_overall_IN_var "Input record for function kc_overall and function kc_overall_KC"
            //fluid property variables
            extends Modelica.Fluid.Dissipation.Utilities.Records.General.FluidProperties;
            //input variable (fluid flow velocity)
            Modelica.SIunits.Velocity velocity annotation(
              Dialog(group = "Input"));
            annotation(
              Documentation(info = "<html>
This record is used as <b> input record </b> for the heat transfer function <a href=\"modelica://Modelica.Fluid.Dissipation.HeatTransfer.Plate.kc_overall\"> kc_overall</a> and
<a href=\"modelica://Modelica.Fluid.Dissipation.HeatTransfer.Plate.kc_overall_KC\"> kc_overall_KC</a>.
</html>"));
          end kc_overall_IN_var;

          function kc_turbulent "Mean heat transfer coefficient of even plate | turbulent regime | constant wall temperature"
            extends Modelica.Icons.Function;
            //SOURCE: VDI-Waermeatlas, Aufl. 9, Springer-Verlag, 2002, Section Gd 1
            //Notation of equations according to SOURCE
            //input records
            input Modelica.Fluid.Dissipation.HeatTransfer.Plate.kc_turbulent_IN_con IN_con "Input record for function kc_turbulent" annotation(
              Dialog(group = "Constant inputs"));
            input Modelica.Fluid.Dissipation.HeatTransfer.Plate.kc_turbulent_IN_var IN_var "Input record for function kc_turbulent" annotation(
              Dialog(group = "Variable inputs"));
            //output variables
            output SI.CoefficientOfHeatTransfer kc "Convective heat transfer coefficient" annotation(
              Dialog(group = "Output"));
            output SI.PrandtlNumber Pr "Prandtl number" annotation(
              Dialog(group = "Output"));
            output SI.ReynoldsNumber Re "Reynolds number" annotation(
              Dialog(group = "Output"));
            output SI.NusseltNumber Nu "Nusselt number" annotation(
              Dialog(group = "Output"));
            output Real failureStatus "0== boundary conditions fulfilled | 1== failure >> check if still meaningful results" annotation(
              Dialog(group = "Output"));
          protected
            Real MIN = Modelica.Constants.eps;
            Real prandtlMax = 2000 "Maximum Prandtl number";
            Real prandtlMin = 0.6 "Minimum Prandtl number";
            Real reynoldsMax = 1e7 "Maximum Reynolds number";
            Real reynoldsMin = 5e5 "Minimum Reynolds number";
            //failure status
            Real fstatus[2] "Check of expected boundary conditions";
            //Documentation
          algorithm
            Pr := abs(IN_var.eta * IN_var.cp / max(MIN, IN_var.lambda));
            Re := max(1e-3, abs(IN_var.rho * IN_var.velocity * IN_con.L / max(MIN, IN_var.eta)));
            kc := Modelica.Fluid.Dissipation.HeatTransfer.Plate.kc_turbulent_KC(IN_con, IN_var);
            Nu := kc * IN_con.L / max(MIN, IN_var.lambda);
//failure status
            fstatus[1] := if Re > reynoldsMax or Re < reynoldsMin then 1 else 0;
            fstatus[2] := if Pr > prandtlMax or Pr < prandtlMin then 1 else 0;
            failureStatus := 0;
            for i in 1:size(fstatus, 1) loop
              if fstatus[i] == 1 then
                failureStatus := 1;
              end if;
            end for;
            annotation(
              Inline = false,
              Documentation(info = "<html>
<p>
Calculation of the mean convective heat transfer coefficient <b> kc </b> for a hydrodynamically developed turbulent fluid flow over an even surface. Note that additionally a failure status is observed in this function to check if the intended boundary conditions are fulfilled.  <a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.HeatTransfer.Plate.kc_turbulent\">See more information</a> .
</p>
</html>"),
              smoothOrder(normallyConstant = IN_con) = 2);
          end kc_turbulent;

          function kc_turbulent_KC "Mean heat transfer coefficient of even plate | turbulent regime | constant wall temperature"
            extends Modelica.Icons.Function;
            //SOURCE: VDI-Waermeatlas, Aufl. 9, Springer-Verlag, 2002, Section Gd 1
            //Notation of equations according to SOURCE
            //input records
            input Modelica.Fluid.Dissipation.HeatTransfer.Plate.kc_turbulent_IN_con IN_con "Input record for function kc_turbulent_KC" annotation(
              Dialog(group = "Constant inputs"));
            input Modelica.Fluid.Dissipation.HeatTransfer.Plate.kc_turbulent_IN_var IN_var "Input record for function kc_turbulent_KC" annotation(
              Dialog(group = "Variable inputs"));
            //output variables
            output SI.CoefficientOfHeatTransfer kc "Output for function kc_turbulent_KC";
          protected
            Real MIN = Modelica.Constants.eps;
            SI.Length L = max(MIN, IN_con.L);
            SI.SpecificHeatCapacityAtConstantPressure cp = IN_var.cp;
            SI.DynamicViscosity eta = IN_var.eta;
            SI.ThermalConductivity lambda = max(MIN, IN_var.lambda);
            SI.Density rho = IN_var.rho;
            SI.Velocity velocity = abs(IN_var.velocity) "Mean velocity";
            SI.ReynoldsNumber Re = max(1e-3, abs(rho * velocity * L / eta));
            SI.PrandtlNumber Pr = abs(eta * cp / lambda);
            //Documentation
          algorithm
            kc := abs(lambda / L) * (0.037 * Re ^ 0.8 * Pr) / (1 + 2.443 / max(Re ^ 0.1, 1e-6) * (Pr ^ (2 / 3) - 1));
            annotation(
              Inline = true,
              Documentation(info = "<html>
<p>
Calculation of the mean convective heat transfer coefficient <b> kc </b> for a hydrodynamically developed turbulent fluid flow over an even surface.
Generally this function is numerically best used for the calculation of the mean convective heat transfer coefficient <b> kc </b> at known fluid velocity.
 <a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.HeatTransfer.Plate.kc_turbulent\">See more information</a> .
</p>
</html>"),
              smoothOrder(normallyConstant = IN_con) = 2);
          end kc_turbulent_KC;

          record kc_turbulent_IN_con "Input record for function kc_turbulent and kc_turbulent_KC"
            extends Modelica.Fluid.Dissipation.HeatTransfer.Plate.kc_overall_IN_con;
            annotation(
              Documentation(info = "<html>
This record is used as <b> input record </b> for the heat transfer function <a href=\"modelica://Modelica.Fluid.Dissipation.HeatTransfer.Plate.kc_turbulent\">  kc_turbulent</a> and
<a href=\"modelica://Modelica.Fluid.Dissipation.HeatTransfer.Plate.kc_turbulent_KC\">  kc_turbulent_KC</a>.
</html>"));
          end kc_turbulent_IN_con;

          record kc_turbulent_IN_var "Input record for function kc_turbulent and kc_turbulent_KC"
            extends Modelica.Fluid.Dissipation.HeatTransfer.Plate.kc_overall_IN_var;
            annotation(
              Documentation(info = "<html>
This record is used as <b> input record </b> for the heat transfer function <a href=\"modelica://Modelica.Fluid.Dissipation.HeatTransfer.Plate.kc_turbulent\">  kc_turbulent</a> and
<a href=\"modelica://Modelica.Fluid.Dissipation.HeatTransfer.Plate.kc_turbulent_KC\">  kc_turbulent_KC</a>.
</html>"));
          end kc_turbulent_IN_var;
          annotation(
            preferredView = "info",
            Documentation(info = "<html>
<h4>Even plate</h4>
<h5>Laminar flow</h5>
<p>Calculation of the mean convective heat transfer coefficient <b> kc </b> for a laminar fluid flow over an even surface. <a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.HeatTransfer.Plate.kc_laminar\">See more information</a> .</p>

<h5>Turbulent flow</h5>
<p>Calculation of the mean convective heat transfer coefficient <b> kc </b> for a hydrodynamically developed turbulent fluid flow over an even surface. <a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.HeatTransfer.Plate.kc_turbulent\">See more information</a>.</p>

<h5>Overall flow</h5>
<p>
Calculation of the mean convective heat transfer coefficient <b> kc </b> for an laminar and turbulent fluid flow over an even surface. <a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.HeatTransfer.Plate.kc_overall\">See more information</a>.
</p>
</html>"));
        end Plate;

        package StraightPipe
          extends Modelica.Icons.VariantsPackage;

          function kc_laminar "Mean heat transfer coefficient of straight pipe | uniform wall temperature or uniform heat flux | hydrodynamically developed or undeveloped laminar flow regime"
            extends Modelica.Icons.Function;
            //input records
            input Modelica.Fluid.Dissipation.HeatTransfer.StraightPipe.kc_laminar_IN_con IN_con "Input record for function kc_laminar" annotation(
              Dialog(group = "Constant inputs"));
            input Modelica.Fluid.Dissipation.HeatTransfer.StraightPipe.kc_laminar_IN_var IN_var "Input record for function kc_laminar" annotation(
              Dialog(group = "Variable inputs"));
            //output variables
            output SI.CoefficientOfHeatTransfer kc "Convective heat transfer coefficient" annotation(
              Dialog(group = "Output"));
            output SI.PrandtlNumber Pr "Prandtl number" annotation(
              Dialog(group = "Output"));
            output SI.ReynoldsNumber Re "Reynolds number" annotation(
              Dialog(group = "Output"));
            output SI.NusseltNumber Nu "Nusselt number" annotation(
              Dialog(group = "Output"));
            output Real failureStatus "0== boundary conditions fulfilled | 1== failure >> check if still meaningful results" annotation(
              Dialog(group = "Output"));
          protected
            Real MIN = Modelica.Constants.eps;
            Real laminar = 2e3 "Maximum Reynolds number of laminar flow regime";
            Real prandtlMax = 1000 "Maximum Prandtl number";
            Real prandtlMin = 0.6 "Minimum Prandtl number";
            SI.Area A_cross = PI * IN_con.d_hyd ^ 2 / 4 "Cross sectional area";
            SI.Velocity velocity = abs(IN_var.m_flow) / max(MIN, IN_var.rho * A_cross) "Mean velocity";
            //failure status
            Real fstatus[2] "check of expected boundary conditions";
            //Documentation
          algorithm
            Pr := abs(IN_var.eta * IN_var.cp / max(MIN, IN_var.lambda));
            Re := max(1e-3, IN_var.rho * velocity * IN_con.d_hyd / max(MIN, IN_var.eta));
            kc := Modelica.Fluid.Dissipation.HeatTransfer.StraightPipe.kc_laminar_KC(IN_con, IN_var);
            Nu := kc * IN_con.d_hyd / max(MIN, IN_var.lambda);
//failure status
            fstatus[1] := if Re > laminar then 1 else 0;
            fstatus[2] := if Pr > prandtlMax or Pr < prandtlMin then 1 else 0;
            failureStatus := 0;
            for i in 1:size(fstatus, 1) loop
              if fstatus[i] == 1 then
                failureStatus := 1;
              end if;
            end for;
            annotation(
              Inline = false,
              Documentation(info = "<html>
<p>
Calculation of mean convective heat transfer coefficient <b> kc </b> of a straight pipe at an uniform wall temperature <b> or </b> uniform heat flux <b>and</b> for a hydrodynamically developed <b>or</b> undeveloped laminar fluid flow. Note that additionally a failure status is observed in this function to check if the intended boundary conditions are fulfilled.  <a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.HeatTransfer.StraightPipe.kc_laminar\">See more information</a> .
</p>
</html>"),
              smoothOrder(normallyConstant = IN_con) = 2);
          end kc_laminar;

          function kc_laminar_KC "Mean heat transfer coefficient of straight pipe | uniform wall temperature or uniform heat flux | hydrodynamically developed or undeveloped laminar flow regime"
            extends Modelica.Icons.Function;
            //input records
            input Modelica.Fluid.Dissipation.HeatTransfer.StraightPipe.kc_laminar_IN_con IN_con "Input record for function kc_laminar_KC" annotation(
              Dialog(group = "Constant inputs"));
            input Modelica.Fluid.Dissipation.HeatTransfer.StraightPipe.kc_laminar_IN_var IN_var "Input record for function kc_laminar_KC" annotation(
              Dialog(group = "Variable inputs"));
            //output variables
            output SI.CoefficientOfHeatTransfer kc "Output for function kc_laminar_KC";
          protected
            type TYP = Modelica.Fluid.Dissipation.Utilities.Types.HeatTransferBoundary;
            Real MIN = Modelica.Constants.eps;
            SI.Area A_cross = PI * IN_con.d_hyd ^ 2 / 4 "Circular cross sectional area";
            Real Nu0 = if IN_con.target == TYP.UWTuDFF or IN_con.target == TYP.UWTuUFF then 0.7 else if IN_con.target == TYP.UHFuDFF or IN_con.target == TYP.UHFuUFF then 0.6 else 0 "Help variable for mean Nusselt number";
            Real Nu1 = if IN_con.target == TYP.UWTuDFF or IN_con.target == TYP.UWTuUFF then 3.66 else if IN_con.target == TYP.UHFuDFF or IN_con.target == TYP.UHFuUFF then 4.364 else 0 "Help variable for mean Nusselt number";
            SI.Velocity velocity = abs(IN_var.m_flow) / max(MIN, IN_var.rho * A_cross) "Mean velocity";
            SI.ReynoldsNumber Re = max(1e-3, IN_var.rho * velocity * IN_con.d_hyd / max(MIN, IN_var.eta));
            SI.PrandtlNumber Pr = abs(IN_var.eta * IN_var.cp / max(MIN, IN_var.lambda));
            SI.NusseltNumber Nu2 = if IN_con.target == TYP.UWTuDFF or IN_con.target == TYP.UWTuUFF then 1.615 * (Re * Pr * IN_con.d_hyd / IN_con.L) ^ (1 / 3) else if IN_con.target == TYP.UHFuDFF or IN_con.target == TYP.UHFuUFF then 1.953 * (Re * Pr * IN_con.d_hyd / IN_con.L) ^ (1 / 3) else 0 "Help variable for mean Nusselt number";
            SI.NusseltNumber Nu3 = if IN_con.target == TYP.UWTuUFF then (2 / (1 + 22 * Pr)) ^ (1 / 6) * (Re * Pr * IN_con.d_hyd / IN_con.L) ^ 0.5 else if IN_con.target == TYP.UHFuUFF then 0.924 * Pr ^ (1 / 3) * (Re * IN_con.d_hyd / IN_con.L) ^ (1 / 2) else 0 "Help variable for mean Nusselt number";
            SI.NusseltNumber Nu = (Nu1 ^ 3 + Nu0 ^ 3 + (Nu2 - Nu0) ^ 3 + Nu3 ^ 3) ^ (1 / 3) "Mean Nusselt number";
            //Documentation
          algorithm
            kc := Nu * IN_var.lambda / max(MIN, IN_con.d_hyd);
            annotation(
              Inline = false,
              Documentation(info = "<html>
<p>
Calculation of mean convective heat transfer coefficient <b> kc </b> of a straight pipe at an uniform wall temperature <b> or </b> uniform heat flux <b>and</b> for a hydrodynamically developed <b>or</b> undeveloped laminar fluid flow. <a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.HeatTransfer.StraightPipe.kc_laminar\">See more information</a> .
</p>
</html>"),
              smoothOrder(normallyConstant = IN_con) = 2);
          end kc_laminar_KC;

          record kc_laminar_IN_con "Input record for function kc_laminar and kc_laminar_KC"
            extends Utilities.Records.HeatTransfer.StraightPipe;
            //choices
            Modelica.Fluid.Dissipation.Utilities.Types.HeatTransferBoundary target = Dissipation.Utilities.Types.HeatTransferBoundary.UWTuDFF "Choice of heat transfer boundary condition" annotation(
              Dialog(group = "Choices"));
            annotation(
              Documentation(info = "<html>
<p>
This record is used as <b> input record </b> for the heat transfer function <a href=\"modelica://Modelica.Fluid.Dissipation.HeatTransfer.StraightPipe.kc_laminar\"> kc_laminar</a> and
<a href=\"modelica://Modelica.Fluid.Dissipation.HeatTransfer.StraightPipe.kc_laminar_KC\"> kc_laminar_KC</a>.
</p>
</html>"));
          end kc_laminar_IN_con;

          record kc_laminar_IN_var "Input record for function kc_laminar and kc_laminar_KC"
            extends Modelica.Fluid.Dissipation.HeatTransfer.StraightPipe.kc_overall_IN_var;
            annotation(
              Documentation(info = "<html>
<p>
This record is used as <b> input record </b> for the heat transfer function <a href=\"modelica://Modelica.Fluid.Dissipation.HeatTransfer.StraightPipe.kc_laminar\"> kc_laminar</a> and
<a href=\"modelica://Modelica.Fluid.Dissipation.HeatTransfer.StraightPipe.kc_laminar_KC\"> kc_laminar_KC</a>.
</p>
</html>"));
          end kc_laminar_IN_var;

          function kc_overall "Mean heat transfer coefficient of straight pipe | uniform wall temperature or uniform heat flux | hydrodynamically developed or undeveloped overall flow regime| pressure loss dependence"
            extends Modelica.Icons.Function;
            //input records
            input Modelica.Fluid.Dissipation.HeatTransfer.StraightPipe.kc_overall_IN_con IN_con "Input record for function kc_overall" annotation(
              Dialog(group = "Constant inputs"));
            input Modelica.Fluid.Dissipation.HeatTransfer.StraightPipe.kc_overall_IN_var IN_var "Input record for function kc_overall" annotation(
              Dialog(group = "Variable inputs"));
            //output variables
            output SI.CoefficientOfHeatTransfer kc "Convective heat transfer coefficient" annotation(
              Dialog(group = "Output"));
            output SI.PrandtlNumber Pr "Prandtl number" annotation(
              Dialog(group = "Output"));
            output SI.ReynoldsNumber Re "Reynolds number" annotation(
              Dialog(group = "Output"));
            output SI.NusseltNumber Nu "Nusselt number" annotation(
              Dialog(group = "Output"));
            output Real failureStatus "0== boundary conditions fulfilled | 1== failure >> check if still meaningful results" annotation(
              Dialog(group = "Output"));
          protected
            type TYP = Modelica.Fluid.Dissipation.Utilities.Types.Roughness;
            Real MIN = Modelica.Constants.eps;
            SI.Area A_cross = PI * IN_con.d_hyd ^ 2 / 4 "Cross sectional area";
            SI.Velocity velocity = abs(IN_var.m_flow) / max(MIN, IN_var.rho * A_cross) "Mean velocity";
            //failure status
            Real fstatus[3] "Check of expected boundary conditions";
          algorithm
            Pr := abs(IN_var.eta * IN_var.cp / max(MIN, IN_var.lambda));
            Re := max(1e-3, IN_var.rho * velocity * IN_con.d_hyd / max(MIN, IN_var.eta));
            kc := Modelica.Fluid.Dissipation.HeatTransfer.StraightPipe.kc_overall_KC(IN_con, IN_var);
            Nu := kc * IN_con.d_hyd / max(MIN, IN_var.lambda);
//failure status
            if IN_con.roughness == TYP.Neglected then
              if Re < 2e4 or Re > 1e6 then
                fstatus[1] := 1;
              else
                fstatus[1] := 0;
              end if;
            elseif IN_con.roughness == TYP.Considered then
              if Re < 1e4 or Re > 1e6 then
                fstatus[1] := 1;
              else
                fstatus[1] := 0;
              end if;
            else
              assert(false, "No choice of roughness is selected");
            end if;
            fstatus[2] := if Pr < 0.6 or Pr > 1e3 then 1 else 0;
            fstatus[3] := if IN_con.d_hyd / max(MIN, IN_con.L) > 1 then 1 else 0;
            failureStatus := 0;
            for i in 1:size(fstatus, 1) loop
              if fstatus[i] == 1 then
                failureStatus := 1;
              end if;
            end for;
            annotation(
              Inline = false,
              Documentation(info = "<html>
<p>
Calculation of mean convective heat transfer coefficient <b> kc </b> of a straight pipe at an uniform wall temperature <b> or </b> uniform heat flux <b> and </b> for a hydrodynamically developed <b>or</b> undeveloped overall fluid flow with neglect <b> or </b> consideration of pressure loss influence. Note that additionally a failure status is observed in this function to check if the intended boundary conditions are fulfilled.  <a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.HeatTransfer.StraightPipe.kc_overall\">See more information</a> .
</p>
</html>"),
              smoothOrder(normallyConstant = IN_con) = 2);
          end kc_overall;

          function kc_overall_KC "Mean heat transfer coefficient of straight pipe | uniform wall temperature or uniform heat flux | hydrodynamically developed or undeveloped overall flow regime| pressure loss dependence"
            extends Modelica.Icons.Function;
            import SMOOTH = Modelica.Fluid.Dissipation.Utilities.Functions.General.Stepsmoother;
            //input records
            input Modelica.Fluid.Dissipation.HeatTransfer.StraightPipe.kc_overall_IN_con IN_con "Input record for function kc_overall_KC" annotation(
              Dialog(group = "Constant inputs"));
            input Modelica.Fluid.Dissipation.HeatTransfer.StraightPipe.kc_overall_IN_var IN_var "Input record for function kc_overall_KC" annotation(
              Dialog(group = "Variable inputs"));
            //output variables
            output SI.CoefficientOfHeatTransfer kc "Output for function kc_overall_KC";
          protected
            Real MIN = Modelica.Constants.eps;
            Real laminar = 2200 "Maximum Reynolds number for laminar regime";
            Real turbulent = 1e4 "Minimum Reynolds number for turbulent regime";
            SI.Area A_cross = PI * IN_con.d_hyd ^ 2 / 4 "Cross sectional area";
            SI.Velocity velocity = abs(IN_var.m_flow) / max(MIN, IN_var.rho * A_cross) "Mean velocity";
            SI.ReynoldsNumber Re = max(1e-3, IN_var.rho * velocity * IN_con.d_hyd / max(MIN, IN_var.eta));
            SI.PrandtlNumber Pr = abs(IN_var.eta * IN_var.cp / max(MIN, IN_var.lambda));
            kc_turbulent_IN_con IN_con_turb(d_hyd = IN_con.d_hyd, L = IN_con.L, roughness = IN_con.roughness, K = IN_con.K);
            kc_laminar_IN_con IN_con_lam(d_hyd = IN_con.d_hyd, L = IN_con.L, target = IN_con.target);
          algorithm
            kc := SMOOTH(laminar, turbulent, Re) * Dissipation.HeatTransfer.StraightPipe.kc_laminar_KC(IN_con_lam, IN_var) + SMOOTH(turbulent, laminar, Re) * Dissipation.HeatTransfer.StraightPipe.kc_turbulent_KC(IN_con_turb, IN_var);
            annotation(
              Inline = false,
              Documentation(info = "<html>
<p>
Calculation of mean convective heat transfer coefficient <b> kc </b> of a straight pipe at an uniform wall temperature <b> or </b> uniform heat flux <b> and </b> for a hydrodynamically developed <b>or</b> undeveloped overall fluid flow with neglect <b> or </b> consideration of pressure loss influence.<a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.HeatTransfer.StraightPipe.kc_overall\">See more information</a> .
</p>
</html>"),
              smoothOrder(normallyConstant = IN_con) = 2);
          end kc_overall_KC;

          record kc_overall_IN_con "Input record for function kc_overall and kc_overall_KC"
            //choices
            Modelica.Fluid.Dissipation.Utilities.Types.HeatTransferBoundary target = Dissipation.Utilities.Types.HeatTransferBoundary.UWTuDFF "Choice of heat transfer boundary condition" annotation(
              Dialog(group = "Choices"));
            extends kc_turbulent_IN_con;
            annotation(
              Documentation(info = "<html>
This record is used as <b> input record </b> for the heat transfer function <a href=\"modelica://Modelica.Fluid.Dissipation.HeatTransfer.StraightPipe.kc_overall\"> kc_overall</a> and
<a href=\"modelica://Modelica.Fluid.Dissipation.HeatTransfer.StraightPipe.kc_overall_KC\"> kc_overall_KC</a>.
</html>"));
          end kc_overall_IN_con;

          record kc_overall_IN_var "Input record for function kc_overall and kc_overall_KC"
            //fluid property variables
            extends Modelica.Fluid.Dissipation.Utilities.Records.General.FluidProperties;
            //input variable (mass flow rate)
            SI.MassFlowRate m_flow annotation(
              Dialog(group = "Input"));
            annotation(
              Documentation(info = "<html>
This record is used as <b> input record </b> for the heat transfer function <a href=\"modelica://Modelica.Fluid.Dissipation.HeatTransfer.StraightPipe.kc_overall\"> kc_overall</a> and
<a href=\"modelica://Modelica.Fluid.Dissipation.HeatTransfer.StraightPipe.kc_overall_KC\"> kc_overall_KC</a>.
</html>"));
          end kc_overall_IN_var;

          function kc_turbulent "Mean heat transfer coefficient of straight pipe | hydrodynamically developed turbulent flow regime | pressure loss dependence"
            extends Modelica.Icons.Function;
            //input records
            input Modelica.Fluid.Dissipation.HeatTransfer.StraightPipe.kc_turbulent_IN_con IN_con "Input record for function kc_turbulent" annotation(
              Dialog(group = "Constant inputs"));
            input Modelica.Fluid.Dissipation.HeatTransfer.StraightPipe.kc_turbulent_IN_var IN_var "Input record for function kc_turbulent" annotation(
              Dialog(group = "Variable inputs"));
            //output variables
            output SI.CoefficientOfHeatTransfer kc "Convective heat transfer coefficient" annotation(
              Dialog(group = "Output"));
            output SI.PrandtlNumber Pr "Prandtl number" annotation(
              Dialog(group = "Output"));
            output SI.ReynoldsNumber Re "Reynolds number" annotation(
              Dialog(group = "Output"));
            output SI.NusseltNumber Nu "Nusselt number" annotation(
              Dialog(group = "Output"));
            output Real failureStatus "0== boundary conditions fulfilled | 1== failure >> check if still meaningful results" annotation(
              Dialog(group = "Output"));
          protected
            type TYP = Modelica.Fluid.Dissipation.Utilities.Types.Roughness;
            Real MIN = Modelica.Constants.eps;
            SI.Area A_cross = PI * IN_con.d_hyd ^ 2 / 4 "Cross sectional area";
            SI.Velocity velocity = abs(IN_var.m_flow) / (IN_var.rho * A_cross) "Mean velocity";
            //failure status
            Real fstatus[3] "check of expected boundary conditions";
            //Documentation
          algorithm
            Pr := abs(IN_var.eta * IN_var.cp / max(MIN, IN_var.lambda));
            Re := max(1e-3, IN_var.rho * velocity * IN_con.d_hyd / max(MIN, IN_var.eta));
            kc := Modelica.Fluid.Dissipation.HeatTransfer.StraightPipe.kc_turbulent_KC(IN_con, IN_var);
            Nu := kc * IN_con.d_hyd / max(MIN, IN_var.lambda);
//failure status
            if IN_con.roughness == TYP.Neglected then
              if Re < 2e4 or Re > 1e6 then
                fstatus[1] := 1;
              else
                fstatus[1] := 0;
              end if;
            elseif IN_con.roughness == TYP.Considered then
              if Re < 1e4 or Re > 1e6 then
                fstatus[1] := 1;
              else
                fstatus[1] := 0;
              end if;
            else
              assert(false, "No choice of roughness is selected");
            end if;
            fstatus[2] := if Pr <= 0.6 or Pr >= 1e3 then 1 else 0;
            fstatus[3] := if IN_con.d_hyd / max(MIN, IN_con.L) > 1 then 1 else 0;
            failureStatus := 0;
            for i in 1:size(fstatus, 1) loop
              if fstatus[i] == 1 then
                failureStatus := 1;
              end if;
            end for;
            annotation(
              Inline = false,
              Documentation(info = "<html>
<p>
Calculation of mean convective heat transfer coefficient <b> kc </b> of a straight pipe for a hydrodynamically developed turbulent fluid flow at uniform wall temperature <b> or </b> uniform heat flux with neglecting <b> or </b> considering of pressure loss influence. Note that additionally a failure status is observed in this function to check if the intended boundary conditions are fulfilled.  <a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.HeatTransfer.StraightPipe.kc_turbulent\">See more information</a> .
</p>
</html>"),
              smoothOrder(normallyConstant = IN_con) = 2);
          end kc_turbulent;

          function kc_turbulent_KC "Mean heat transfer coefficient of straight pipe | hydrodynamically developed turbulent flow regime | pressure loss dependence"
            extends Modelica.Icons.Function;
            //input records
            input Modelica.Fluid.Dissipation.HeatTransfer.StraightPipe.kc_turbulent_IN_con IN_con "Input record for function kc_turbulent_KC" annotation(
              Dialog(group = "Constant inputs"));
            input Modelica.Fluid.Dissipation.HeatTransfer.StraightPipe.kc_turbulent_IN_var IN_var "Input record for function kc_turbulent_KC" annotation(
              Dialog(group = "Variable inputs"));
            //output variables
            output SI.CoefficientOfHeatTransfer kc "Output for function kc_turbulent_KC";
          protected
            type TYP = Modelica.Fluid.Dissipation.Utilities.Types.Roughness;
            Real MIN = Modelica.Constants.eps;
            SI.Area A_cross = PI * IN_con.d_hyd ^ 2 / 4 "Circular cross sectional area";
            SI.Velocity velocity = abs(IN_var.m_flow) / (IN_var.rho * A_cross) "Mean velocity";
            SI.ReynoldsNumber Re = max(2.6, IN_var.rho * velocity * IN_con.d_hyd / IN_var.eta);
            SI.PrandtlNumber Pr = abs(IN_var.eta * IN_var.cp / IN_var.lambda);
            Real zeta = abs(1 / max(MIN, 1.8 * Modelica.Math.log10(abs(Re)) - 1.5) ^ 2) "Pressure loss coefficient";
            //Documentation
          algorithm
            kc := if IN_con.roughness == TYP.Neglected then abs(IN_var.lambda / IN_con.d_hyd) * 0.023 * Re ^ 0.8 * Pr ^ (1 / 3) else if IN_con.roughness == TYP.Considered then abs(IN_var.lambda / IN_con.d_hyd) * (abs(zeta) / 8) * abs(Re) * abs(Pr) / (1 + 12.7 * (abs(zeta) / 8) ^ 0.5 * (abs(Pr) ^ (2 / 3) - 1)) * (1 + (IN_con.d_hyd / IN_con.L) ^ (2 / 3)) else 0;
            annotation(
              Inline = false,
              Documentation(info = "<html>
<p>
Calculation of mean convective heat transfer coefficient <b> kc </b> of a straight pipe for a hydrodynamically developed turbulent fluid flow at uniform wall temperature <b> or </b> uniform heat flux with neglecting <b> or </b> considering of pressure loss influence. <a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.HeatTransfer.StraightPipe.kc_turbulent\">See more information</a> .
</p>
</html>"),
              smoothOrder(normallyConstant = IN_con) = 2);
          end kc_turbulent_KC;

          record kc_turbulent_IN_con "Input record for function kc_turbulent and kc_turbulent_KC"
            extends Utilities.Records.HeatTransfer.StraightPipe;
            Modelica.Fluid.Dissipation.Utilities.Types.Roughness roughness = Dissipation.Utilities.Types.Roughness.Considered "Choice of considering surface roughness" annotation(
              Dialog(group = "Choices"));
            SI.Length K = 0 "Roughness (average height of surface asperities)" annotation(
              Dialog(group = "Straight pipe", enable = roughness == Modelica.Fluid.Dissipation.Utilities.Types.Roughness.Considered));
            annotation(
              Documentation(info = "<html>
This record is used as <b> input record </b> for the heat transfer function <a href=\"modelica://Modelica.Fluid.Dissipation.HeatTransfer.StraightPipe.kc_turbulent\">kc_turbulent</a> and
<a href=\"modelica://Modelica.Fluid.Dissipation.HeatTransfer.StraightPipe.kc_turbulent_KC\">kc_turbulent_KC</a>.
</html>"));
          end kc_turbulent_IN_con;

          record kc_turbulent_IN_var "Input record for function kc_turbulent and kc_turbulent_KC"
            extends Modelica.Fluid.Dissipation.HeatTransfer.StraightPipe.kc_overall_IN_var;
            annotation(
              Documentation(info = "<html>
This record is used as <b> input record </b> for the heat transfer function <a href=\"modelica://Modelica.Fluid.Dissipation.HeatTransfer.StraightPipe.kc_turbulent\">kc_turbulent</a> and
<a href=\"modelica://Modelica.Fluid.Dissipation.HeatTransfer.StraightPipe.kc_turbulent_KC\">kc_turbulent_KC</a>.
</html>"));
          end kc_turbulent_IN_var;

          function kc_twoPhaseOverall_KC "Local two phase heat transfer coefficient of straight pipe | horizontal or vertical boiling | horizontal condensation"
            extends Modelica.Icons.Function;
            //SOURCE_1: Bejan,A.: HEAT TRANSFER HANDBOOK, Wiley, 2003.
            //SOURCE_2: Gungor, K.E. and R.H.S. Winterton: A general correlation for flow boiling in tubes and annuli, Int.J. Heat Mass Transfer, Vol.29, p.351-358, 1986.
            //input records
            input Modelica.Fluid.Dissipation.HeatTransfer.StraightPipe.kc_twoPhaseOverall_KC_IN_con IN_con annotation(
              Dialog(group = "Constant inputs"));
            input Modelica.Fluid.Dissipation.HeatTransfer.StraightPipe.kc_twoPhaseOverall_KC_IN_var IN_var annotation(
              Dialog(group = "Variable inputs"));
            //output variables
            output SI.CoefficientOfHeatTransfer kc "Local two phase heat transfer coefficient";
          protected
            Real MIN = Modelica.Constants.eps;
            //Documentation
          algorithm
            kc := if IN_con.target == Modelica.Fluid.Dissipation.Utilities.Types.TwoPhaseHeatTransferTarget.BoilHor then Modelica.Fluid.Dissipation.Utilities.Functions.HeatTransfer.TwoPhase.kc_twoPhase_boilingHorizontal_KC(IN_con, IN_var) else if IN_con.target == Modelica.Fluid.Dissipation.Utilities.Types.TwoPhaseHeatTransferTarget.BoilVer then Modelica.Fluid.Dissipation.Utilities.Functions.HeatTransfer.TwoPhase.kc_twoPhase_boilingVertical_KC(IN_con, IN_var) else if IN_con.target == Modelica.Fluid.Dissipation.Utilities.Types.TwoPhaseHeatTransferTarget.CondHor then Modelica.Fluid.Dissipation.Utilities.Functions.HeatTransfer.TwoPhase.kc_twoPhase_condensationHorizontal_KC(IN_con, IN_var) else MIN;
            annotation(
              Inline = false,
              smoothOrder(normallyConstant = IN_con) = 2,
              Documentation(info = "<html>
<p>Calculation of local <b>two phase</b> heat transfer coefficient <b>kc_2ph </b>for (horizontal/vertical) <b>boiling</b> or (horizontal) <b>condensation</b> for an overall flow regime. <a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.HeatTransfer.StraightPipe.kc_twoPhaseOverall\">See more information</a> .</p>
</html>"));
          end kc_twoPhaseOverall_KC;

          record kc_twoPhaseOverall_KC_IN_con "Input record for function kc_twoPhaseOverall_KC"
            extends Modelica.Fluid.Dissipation.Utilities.Records.HeatTransfer.TwoPhaseFlowHT_IN_con;
            annotation(
              Documentation(info = "<html>
This record is used as <b> input record </b> for the heat transfer function <a href=\"modelica://Modelica.Fluid.Dissipation.HeatTransfer.StraightPipe.kc_twoPhaseOverall_KC\"> kc_twoPhaseOverall_KC</a>.
</html>"));
          end kc_twoPhaseOverall_KC_IN_con;

          record kc_twoPhaseOverall_KC_IN_var "Input record for function kc_twoPhaseOverall_KC"
            extends Modelica.Fluid.Dissipation.Utilities.Records.HeatTransfer.TwoPhaseFlowHT_IN_var;
            annotation(
              Documentation(info = "<html>
This record is used as <b>input record</b> for the heat transfer function <a href=\"modelica://Modelica.Fluid.Dissipation.HeatTransfer.StraightPipe.kc_twoPhaseOverall_KC\"> kc_twoPhaseOverall_KC</a>.
</html>"));
          end kc_twoPhaseOverall_KC_IN_var;
          annotation(
            preferredView = "info",
            Documentation(info = "<html>
<h4>Straight pipe</h4>
<h5>Laminar flow</h5>
<p>Calculation of mean convective heat transfer coefficient <b>kc</b> of a straight pipe at an uniform wall temperature <b>or</b> uniform heat flux <b>and</b> for a hydrodynamically developed <b>or</b> undeveloped laminar fluid flow. <a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.HeatTransfer.StraightPipe.kc_laminar\">See more information</a>.</p>

<h5>Turbulent flow</h5>
<p>Calculation of mean convective heat transfer coefficient <b>kc</b> of a straight pipe for a hydrodynamically developed turbulent fluid flow at uniform wall temperature <b>or</b> <a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.HeatTransfer.StraightPipe.kc_twoPhaseOverall\">See more information</a>.</p>
</html>"));
        end StraightPipe;
        annotation(
          preferredView = "info");
      end HeatTransfer;

      package PressureLoss "Package for calculation of pressure loss"
        extends Modelica.Icons.VariantsPackage;

        package Bend "Package for pressure loss calculation of bends"
          extends Modelica.Icons.VariantsPackage;

          function dp_curvedOverall_DP "Pressure loss of curved bend | calculate pressure loss | overall flow regime | surface roughness"
            extends Modelica.Icons.Function;
            //SOURCE_1: Idelchik, I.E.: HANDBOOK OF HYDRAULIC RESISTANCE, 3rd edition, 2006.
            //SOURCE_2: Miller, D.S.: INTERNAL FLOW SYSTEMS, 2nd edition, 1984.
            //SOURCE_3: VDI-Waermeatlas, 9th edition, Springer-Verlag, 2002, Section Lac 6 (Verification)
            //Notation of equations according to SOURCES
            import FD = Modelica.Fluid.Dissipation.PressureLoss.Bend;
            import SMOOTH = Modelica.Fluid.Dissipation.Utilities.Functions.General.Stepsmoother;
            //input records
            input Modelica.Fluid.Dissipation.PressureLoss.Bend.dp_curvedOverall_IN_con IN_con "Input record for function dp_curvedOverall_DP" annotation(
              Dialog(group = "Constant inputs"));
            input Modelica.Fluid.Dissipation.PressureLoss.Bend.dp_curvedOverall_IN_var IN_var "Input record for function dp_curvedOverall_DP" annotation(
              Dialog(group = "Variable inputs"));
            input SI.MassFlowRate m_flow "Mass flow rate" annotation(
              Dialog(group = "Input"));
            //output variables
            output SI.Pressure DP "Output for function dp_curvedOverall_DP";
          protected
            Real MIN = Modelica.Constants.eps;
            SI.Diameter d_hyd = max(MIN, IN_con.d_hyd) "Hydraulic diameter";
            SI.Area A_cross = PI * IN_con.d_hyd ^ 2 / 4 "Circular cross sectional area";
            Real frac_RD = max(MIN, IN_con.R_0 / d_hyd) "Relative curvature radius";
            Real k = max(MIN, abs(IN_con.K) / d_hyd) "Relative roughness";
            Real delta = IN_con.delta * 180 / PI "Angle of turning";
            SI.Length L = IN_con.delta * IN_con.R_0 "Length of flow path";
            //SOURCE_1: p.336, sec.15: definition of flow regime boundaries
            SI.ReynoldsNumber Re_min = 1 "Minimum Reynolds number";
            SI.ReynoldsNumber Re_lam_max = 6.5e3 "Maximum Reynolds number for laminar regime (6.5e3)";
            SI.ReynoldsNumber Re_turb_min = 4e4 "Minimum Reynolds number for turbulent regime (4e4)";
            SI.ReynoldsNumber Re_turb_max = 3e5 "Maximum Reynolds number for turbulent regime (3e5)";
            SI.ReynoldsNumber Re_turb_const = 1e6 "Reynolds number for independence on pressure loss coefficient (1e6)";
            SI.ReynoldsNumber Re_lam_leave = min(Re_lam_max, max(1e2, 754 * Modelica.Math.exp(if k <= 0.007 then 0.0065 / 0.007 else 0.0065 / k))) "Start of transition regime for increasing Reynolds number (leaving laminar regime)";
            //SOURCE_1: p.357, diag. 6-1: coefficients for local resistance coefficient [zeta_LOC]:
            //IN_con.R_0/IN_con.d_hyd <= 3
            Real A1 = if delta <= 70 then 0.9 * sin(delta * PI / 180) else if delta >= 100 then 0.7 + 0.35 * delta / 90 else 1.0 "Coefficient considering effect for angle of turning on zeta_LOC";
            Real A2 = if frac_RD > 2.0 then 6e2 else if frac_RD <= 2.0 and frac_RD > 0.55 then if frac_RD > 1.0 then 1e3 else if frac_RD <= 1.0 and frac_RD > 0.7 then 3e3 else 6e3 else 4e3 "Coefficient considering laminar regime on zeta_LOC";
            Real B1 = if frac_RD >= 1.0 then 0.21 * frac_RD ^ (-0.5) else 0.21 * frac_RD ^ (-2.5) "Coefficient considering relative curvature radius (R_0/d_hyd) on zeta_LOC";
            Real C1 = 1.0 "Considering relative elongation of cross sectional area on zeta_LOC (here: circular cross sectional area)";
            TYP.LocalResistanceCoefficient zeta_LOC_sharp_turb = max(MIN, A1 * B1 * C1) "Local resistance coefficient for turbulent regime (Re > Re_turb_max)";
            SI.ReynoldsNumber Re = max(Re_min, 4 * abs(m_flow) / (PI * IN_con.d_hyd * IN_var.eta)) "Reynolds number";
            //mass flow rate boundaries for w.r.t flow regimes
            SI.MassFlowRate m_flow_smooth = Re_min * PI * IN_con.d_hyd * IN_var.eta / 4;
            //SOURCE_1: p.357, diag. 6-1, sec. 2 / p.336, sec. 15 (turbulent regime + hydraulically rough):
            //IN_con.R_0/IN_con.d_hyd < 3
            Real C_Re = if frac_RD > 0.7 then 11.5 / Re ^ 0.19 else if frac_RD <= 0.7 and frac_RD >= 0.55 then 5.45 / Re ^ 0.131 else 1 + 4400 / Re "Correction factor for hydraulically rough turbulent regime (Re_turb_min < Re < Re_turb_max)";
            //SOURCE_1: p.357, diag. 6-1
            //IN_con.R_0/IN_con.d_hyd < 3
            TYP.LocalResistanceCoefficient zeta_LOC_sharp = if Re < Re_lam_leave then A2 / Re + zeta_LOC_sharp_turb else if Re < Re_turb_min then SMOOTH(Re_lam_leave, Re_turb_min, Re) * (A2 / max(Re_lam_leave, Re) + zeta_LOC_sharp_turb) + SMOOTH(Re_turb_min, Re_lam_leave, Re) * (C_Re * zeta_LOC_sharp_turb) else if Re < Re_turb_max then SMOOTH(Re_turb_min, Re_turb_max, Re) * (C_Re * zeta_LOC_sharp_turb) + SMOOTH(Re_turb_max, Re_turb_min, Re) * zeta_LOC_sharp_turb else zeta_LOC_sharp_turb "Local resistance coefficient for R_0/d_hyd < 3";
            TYP.LocalResistanceCoefficient zeta_LOC = zeta_LOC_sharp "Local resistance coefficient";
            //SOURCE_2: p.191, eq. 8.4: considering surface roughness
            //restriction of lambda_FRI at maximum Reynolds number Re=1e6 (SOURCE_2: p.207, sec. 9.2.4)
            TYP.DarcyFrictionFactor lambda_FRI_rough = 0.25 / Modelica.Math.log10(k / (3.7 * IN_con.d_hyd) + 5.74 / min(1e6, max(Re_lam_leave, Re)) ^ 0.9) ^ 2 "Darcy friction factor considering surface roughness";
            //SOURCE_2: p.207, sec. 9.2.4: correction factors CF w.r.t.surface roughness
            Real CF_fri = 1 + SMOOTH(Re_lam_max, Re_lam_leave, Re) * min(1.4, lambda_FRI_rough * L / d_hyd / zeta_LOC) + SMOOTH(Re_lam_leave, Re_lam_max, Re) "Correction factor for surface roughness";
            TYP.PressureLossCoefficient zeta_TOT = max(1, CF_fri) * zeta_LOC "Pressure loss coefficient";
            //Documentation
          algorithm
            DP := zeta_TOT * (IN_var.rho / 2) * Modelica.Fluid.Dissipation.Utilities.Functions.General.SmoothPower(m_flow, m_flow_smooth, 2) / max(MIN, (IN_var.rho * A_cross) ^ 2);
            annotation(
              Inline = false,
              smoothOrder(normallyConstant = IN_con) = 2,
              inverse(m_flow = Modelica.Fluid.Dissipation.PressureLoss.Bend.dp_curvedOverall_MFLOW(IN_con, IN_var, DP)),
              Documentation(info = "<html>
<p>Calculation of pressure loss in curved bends at overall flow regime for incompressible and single-phase fluid flow through circular cross sectional area considering surface roughness. </p>

<p>Generally this function is numerically best used for the <b>incompressible case </b>, where the mass flow rate (m_flow) is known (as state variable) in the used model and the corresponding pressure loss (DP) has to be calculated. On the other hand the function <a href=\"modelica://Modelica.Fluid.Dissipation.PressureLoss.Bend.dp_curvedOverall_MFLOW\">dp_curvedOverall_MFLOW</a> is numerically best used for the <b>compressible case </b>if the pressure loss (dp) is known (out of pressures as state variable) and the mass flow rate (M_FLOW) has to be calculated. <a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.PressureLoss.Bend.dp_curvedOverall\">See more information</a>.</p>
</html>", revisions = "<html>
2014-12-12 Stefan Wischhusen: Factor A2 corrected for R/D > 0.55-0.7. The factor is now 6e3 instead of 4e3.<br>
</html>"));
          end dp_curvedOverall_DP;

          function dp_curvedOverall_MFLOW "Pressure loss of curved bend | calculate mass flow rate | overall flow regime | surface roughness"
            extends Modelica.Icons.Function;
            //SOURCE_1: Idelchik, I.E.: HANDBOOK OF HYDRAULIC RESISTANCE, 3rd edition, 2006.
            //SOURCE_2: Miller, D.S.: INTERNAL FLOW SYSTEMS, 2nd edition, 1984.
            //SOURCE_3: VDI-Waermeatlas, 9th edition, Springer-Verlag, 2002, Section Lac 6 (Verification)
            //Notation of equations according to SOURCES
            import FD = Modelica.Fluid.Dissipation.PressureLoss.Bend;
            import SMOOTH = Modelica.Fluid.Dissipation.Utilities.Functions.General.Stepsmoother;
            //input records
            input Modelica.Fluid.Dissipation.PressureLoss.Bend.dp_curvedOverall_IN_con IN_con "Input record for function dp_curvedOverall_MFLOW" annotation(
              Dialog(group = "Constant inputs"));
            input Modelica.Fluid.Dissipation.PressureLoss.Bend.dp_curvedOverall_IN_var IN_var "Input record for function dp_curvedOverall_MFLOW" annotation(
              Dialog(group = "Variable inputs"));
            input SI.Pressure dp "Pressure loss" annotation(
              Dialog(group = "Input"));
            //output variables
            output SI.MassFlowRate M_FLOW "Output for function dp_curvedOverall_MFLOW";
          protected
            Real MIN = Modelica.Constants.eps;
            SI.Diameter d_hyd = max(MIN, IN_con.d_hyd) "Hydraulic diameter";
            SI.Area A_cross = PI * IN_con.d_hyd ^ 2 / 4 "Circular cross sectional area";
            Real frac_RD = max(MIN, IN_con.R_0 / d_hyd) "Relative curvature radius";
            Real k = max(MIN, abs(IN_con.K) / d_hyd) "Relative roughness";
            Real delta = IN_con.delta * 180 / PI "Angle of turning";
            SI.Length L = IN_con.delta * IN_con.R_0 "Length of flow path";
            //SOURCE_1: p.336, sec.15: definition of flow regime boundaries
            SI.ReynoldsNumber Re_min = 1 "Minimum Reynolds number";
            SI.ReynoldsNumber Re_lam_max = 6.5e3 "Maximum Reynolds number for laminar regime (6.5e3)";
            SI.ReynoldsNumber Re_turb_min = 4e4 "Minimum Reynolds number for turbulent regime (4e4)";
            SI.ReynoldsNumber Re_turb_max = 3e5 "Maximum Reynolds number for turbulent regime (3e5)";
            SI.ReynoldsNumber Re_turb_const = 1e6 "Reynolds number for independence on pressure loss coefficient (1e6)";
            SI.ReynoldsNumber Re_lam_leave = min(Re_lam_max, max(1e2, 754 * Modelica.Math.exp(if k <= 0.007 then 0.0065 / 0.007 else 0.0065 / k))) "Start of transition regime for increasing Reynolds number (leaving laminar regime)";
            //SOURCE_1: p.357, diag. 6-1: coefficients for local resistance coefficient [zeta_LOC]:
            //IN_con.R_0/IN_con.d_hyd <= 3
            Real A1 = if delta <= 70 then 0.9 * sin(delta / 180 * PI) else if delta >= 100 then 0.7 + 0.35 * delta / 90 else 1.0 "Coefficient considering effect for angle of turning on zeta_LOC";
            Real A2 = if frac_RD > 2.0 then 6e2 else if frac_RD <= 2.0 and frac_RD > 0.55 then if frac_RD > 1.0 then 1e3 else if frac_RD <= 1.0 and frac_RD > 0.7 then 3e3 else 6e3 else 4e3 "Coefficient considering laminar regime on zeta_LOC";
            Real B1 = if frac_RD >= 1.0 then 0.21 * frac_RD ^ (-0.5) else 0.21 * frac_RD ^ (-2.5) "Coefficient considering relative curvature radius (R_0/d_hyd) on zeta_LOC";
            Real C1 = 1.0 "Considering relative elongation of cross sectional area on zeta_LOC (here: circular cross sectional area)";
            TYP.LocalResistanceCoefficient zeta_LOC_sharp_turb = max(MIN, A1 * B1 * C1) "Local resistance coefficient for turbulent regime (Re > Re_turb_max)";
            //SOURCE_1: p.357, diag. 6-1: pressure loss boundaries for w.r.t flow regimes
            //IN_con.R_0/d_hyd <=3
            SI.AbsolutePressure dp_lam_max = (zeta_LOC_sharp_turb + A2 / Re_lam_leave) * IN_var.rho / 2 * (Re_lam_leave * IN_var.eta / (IN_var.rho * d_hyd)) ^ 2 "Maximum pressure loss for laminar regime";
            SI.AbsolutePressure dp_turb_min = zeta_LOC_sharp_turb * (if frac_RD > 0.7 then 11.5 / Re_turb_min ^ 0.19 else if frac_RD <= 0.7 and frac_RD >= 0.55 then 5.45 / Re_turb_min ^ 0.131 else 1 + 4400 / Re_turb_min) * IN_var.rho / 2 * (Re_turb_min * IN_var.eta / (IN_var.rho * d_hyd)) ^ 2 "Minimum pressure loss for turbulent regime";
            SI.AbsolutePressure dp_turb_max = zeta_LOC_sharp_turb * (if frac_RD > 0.7 then 11.5 / Re_turb_max ^ 0.19 else if frac_RD <= 0.7 and frac_RD >= 0.55 then 5.45 / Re_turb_max ^ 0.131 else 1 + 4400 / Re_turb_max) * IN_var.rho / 2 * (Re_turb_max * IN_var.eta / (IN_var.rho * d_hyd)) ^ 2 "Maximum pressure loss for turbulent regime";
            SI.AbsolutePressure dp_turb_const = zeta_LOC_sharp_turb * IN_var.rho / 2 * (Re_turb_const * IN_var.eta / (IN_var.rho * d_hyd)) ^ 2 "Pressure loss for independence of Reynolds number on pressure loss coefficient";
            //SOURCE_1: p.357, diag. 6-1: mean velocities for assumed flow regime
            //IN_con.R_0/d_hyd <=3
            SI.Velocity v_lam = if 1e7 * sqrt(abs(zeta_LOC_sharp_turb * abs(dp) * IN_var.rho * d_hyd ^ 2)) < abs(A2 * IN_var.eta) then 2 * abs(dp) * d_hyd / A2 / IN_var.eta else ((-A2 / 2 * IN_var.eta) + 0.5 * sqrt(max(MIN, (A2 * IN_var.eta) ^ 2 + 8 * zeta_LOC_sharp_turb * abs(dp) * IN_var.rho * d_hyd ^ 2))) / zeta_LOC_sharp_turb / IN_var.rho / d_hyd "Mean velocity in laminar regime (Re < Re_lam_leave)";
            SI.Velocity v_tra = if 1e7 * sqrt(abs(zeta_LOC_sharp_turb * abs(dp_lam_max) * IN_var.rho * d_hyd ^ 2)) < abs(A2 * IN_var.eta) then 2 * abs(dp_lam_max) * d_hyd / A2 / IN_var.eta else ((-A2 / 2 * IN_var.eta) + 0.5 * sqrt(max(MIN, (A2 * IN_var.eta) ^ 2 + 8 * zeta_LOC_sharp_turb * abs(dp_lam_max) * IN_var.rho * d_hyd ^ 2))) / zeta_LOC_sharp_turb / IN_var.rho / d_hyd "Mean velocity in transition regime (Re_lam_leave < Re_turb_min)";
            SI.Velocity v_turb = if frac_RD > 0.7 then (max(MIN, abs(dp)) / (IN_var.rho / 2 * 11.5 * zeta_LOC_sharp_turb) * (IN_var.rho * IN_con.d_hyd / max(MIN, IN_var.eta)) ^ 0.19) ^ (1 / (2 - 0.19)) else if frac_RD > 0.55 and frac_RD < 0.7 then (max(MIN, abs(dp)) / (IN_var.rho / 2 * 5.45 * zeta_LOC_sharp_turb) * (IN_var.rho * IN_con.d_hyd / max(MIN, IN_var.eta)) ^ 0.131) ^ (1 / (2 - 0.131)) else (-2200 / (IN_var.rho * IN_con.d_hyd / IN_var.eta)) + ((-2200 / (IN_var.rho * IN_con.d_hyd / max(MIN, IN_var.eta))) ^ 2 + 2 * abs(max(MIN, dp)) / max(MIN, IN_var.rho)) ^ 0.5 "Mean velocity in turbulent regime with dependence on pressure loss coefficient (Re_turb_min < Re < Re_turb_max)";
            SI.Velocity v_turb_const = sqrt(max(MIN, 2 * abs(dp) / (IN_var.rho * zeta_LOC_sharp_turb))) "Mean velocity in turbulent regime with independence on pressure loss coefficient (Re > Re_turb_max)";
            //mean velocity under smooth conditions w.r.t flow regime
            SI.Velocity v_smooth = if dp < dp_lam_max then v_lam else if dp < dp_turb_min then SMOOTH(dp_lam_max, dp_turb_min, dp) * v_lam + SMOOTH(dp_turb_min, dp_lam_max, dp) * v_turb else if dp < dp_turb_max then v_turb else SMOOTH(dp_turb_max, dp_turb_const, dp) * v_turb + SMOOTH(dp_turb_const, dp_turb_max, dp) * v_turb_const "Mean velocity under smooth conditions for R_0/d_hyd < 3";
            SI.ReynoldsNumber Re_smooth = max(Re_min, IN_var.rho * v_smooth * d_hyd / IN_var.eta) "Reynolds number under smooth conditions";
            //SOURCE_2: p.191, eq. 8.4: considering surface roughness
            //restriction of lambda_FRI at maximum Reynolds number Re=1e6 (SOURCE_2: p.207, sec. 9.2.4)
            TYP.DarcyFrictionFactor lambda_FRI_rough = 0.25 / Modelica.Math.log10(k / (3.7 * IN_con.d_hyd) + 5.74 / min(1e6, max(Re_lam_leave, Re_smooth)) ^ 0.9) ^ 2 "Darcy friction factor considering surface roughness";
            TYP.DarcyFrictionFactor lambda_FRI_smooth = 0.25 / Modelica.Math.log10(5.74 / max(Re_lam_leave, Re_smooth) ^ 0.9) ^ 2 "Darcy friction factor neglecting surface roughness";
            //SOURCE_2: p.207, sec. 9.2.4: correction factors CF w.r.t.surface roughness
            Real CF_3 = 1 + SMOOTH(6e3, 1e3, Re_smooth) * min(1.4, lambda_FRI_rough * L / d_hyd / zeta_LOC_sharp_turb) + SMOOTH(1e3, 6e3, Re_smooth) "Correction factor for surface roughness";
            SI.Velocity velocity = v_smooth / max(1, CF_3) ^ 0.5 "Corrected velocity considering surface roughness";
            //Documentation
          algorithm
            M_FLOW := sign(dp) * IN_var.rho * A_cross * abs(velocity);
            annotation(
              Inline = false,
              smoothOrder(normallyConstant = IN_con) = 2,
              inverse(dp = Modelica.Fluid.Dissipation.PressureLoss.Bend.dp_curvedOverall_DP(IN_con, IN_var, M_FLOW)),
              Documentation(info = "<html>
<p>
Calculation of pressure loss in curved bends at overall flow regime for incompressible and single-phase fluid flow through circular cross sectional area considering surface roughness.
</p>

<p>
Generally this function is numerically best used for the <b> compressible case </b> if the pressure loss (dp) is known (out of pressures as state variable) and the mass flow rate (M_FLOW) has to be calculated. On the other hand the function <a href=\"modelica://Modelica.Fluid.Dissipation.PressureLoss.Bend.dp_curvedOverall_DP\">dp_curvedOverall_DP</a> is numerically best used for the <b> incompressible case </b>, where the mass flow rate (m_flow) is known (as state variable) in the used model and the corresponding pressure loss (DP) has to be calculated.
<a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.PressureLoss.Bend.dp_curvedOverall\">See more information</a> .</p>
</html>", revisions = "<html>
2014-12-01 Stefan Wischhusen: Introduced an expansion in variables v_lam and v_tra for numerical improvement at close to zero flows.<br>
2014-12-12 Stefan Wischhusen: Factor A2 corrected for R/D > 0.55-0.7. The factor is now 6e3 instead of 4e3.<br>
</html>"));
          end dp_curvedOverall_MFLOW;

          record dp_curvedOverall_IN_con "Input record for function dp_curvedOverall_DP and dp_curvedOverall_MFLOW"
            //bend variables
            extends Modelica.Fluid.Dissipation.Utilities.Records.PressureLoss.Bend;
            annotation(
              Documentation(info = "<html>
This record is used as <b> input record </b> for the pressure loss function
<a href=\"modelica://Modelica.Fluid.Dissipation.PressureLoss.Bend.dp_curvedOverall_DP\"> dp_curvedOverall_DP </a>
and <a href=\"modelica://Modelica.Fluid.Dissipation.PressureLoss.Bend.dp_curvedOverall_MFLOW\"> dp_curvedOverall_MFLOW </a>.
</html>"));
          end dp_curvedOverall_IN_con;

          record dp_curvedOverall_IN_var "Input record for function dp_curvedOverall_DP and dp_curvedOverall_MFLOW"
            //fluid property variables
            extends Modelica.Fluid.Dissipation.Utilities.Records.General.PressureLoss;
            annotation(
              Documentation(info = "<html>
This record is used as <b> input record </b> for the pressure loss function
<a href=\"modelica://Modelica.Fluid.Dissipation.PressureLoss.Bend.dp_curvedOverall_DP\"> dp_curvedOverall_DP </a>
and <a href=\"modelica://Modelica.Fluid.Dissipation.PressureLoss.Bend.dp_curvedOverall_MFLOW\"> dp_curvedOverall_MFLOW </a>.
</html>"));
          end dp_curvedOverall_IN_var;

          function dp_edgedOverall_DP "Pressure loss of edged bend | calculate pressure loss | overall flow regime | surface roughness"
            extends Modelica.Icons.Function;
            //SOURCE_1: Idelchik, I.E.: HANDBOOK OF HYDRAULIC RESISTANCE, 3rd edition, 2006.
            //SOURCE_2: Miller, D.S.: INTERNAL FLOW SYSTEMS, 2nd edition, 1984.
            //SOURCE_3: VDI-Waermeatlas, 9th edition, Springer-Verlag, 2002, Section Lac 6 (Verification)
            //Notation of equations according to SOURCES
            import FD = Modelica.Fluid.Dissipation.PressureLoss.Bend;
            import SMOOTH = Modelica.Fluid.Dissipation.Utilities.Functions.General.Stepsmoother;
            //input records
            input Modelica.Fluid.Dissipation.PressureLoss.Bend.dp_edgedOverall_IN_con IN_con "Input record for function dp_edgedOverall_DP" annotation(
              Dialog(group = "Constant inputs"));
            input Modelica.Fluid.Dissipation.PressureLoss.Bend.dp_edgedOverall_IN_var IN_var "Input record for function dp_edgedOverall_DP" annotation(
              Dialog(group = "Variable inputs"));
            input SI.MassFlowRate m_flow "Mass flow rate" annotation(
              Dialog(group = "Input"));
            //output variables
            output SI.Pressure DP "Output for function dp_edgedOverall_DP";
          protected
            Real MIN = Modelica.Constants.eps;
            SI.Diameter d_hyd = max(MIN, IN_con.d_hyd) "Hydraulic diameter";
            SI.Area A_cross = PI * d_hyd ^ 2 / 4 "Circular cross sectional area";
            Real k = max(MIN, abs(IN_con.K) / IN_con.d_hyd) "Relative roughness";
            Real delta = IN_con.delta * 180 / PI "Angle of turning";
            //definition of flow regime boundaries
            //SOURCE_2: p.207, sec. 9.2.4
            //SOURCE_3: p.Lac 6, fig. 16
            SI.ReynoldsNumber Re_min = 1 "Minimum Reynolds number";
            SI.ReynoldsNumber Re_lam_min = 5e2 "Start of transition regime for roughness contribution";
            SI.ReynoldsNumber Re_lam_max = 1e4 "End of transition regime for roughness contribution";
            SI.ReynoldsNumber Re_turb_min = 1e5 "Minimum Reynolds number for Reynolds-dependent transition regime";
            SI.ReynoldsNumber Re_turb_max = 2e5 "Maximum Reynolds number for Reynolds-dependent transition regime (k_Re=1)";
            SI.ReynoldsNumber Re_turb_const = 1e6 "Reynolds number for independence on pressure loss coefficient (1e6)";
            //SOURCE_1: p. 81, sec. 2-2-21: start of transition regime
            SI.ReynoldsNumber Re_lam_leave = min(Re_lam_max, max(Re_lam_min, 754 * Modelica.Math.exp(if k <= 0.007 then 0.0065 / 0.007 else 0.0065 / k))) "Start of transition regime for increasing Reynolds number (leaving laminar regime)";
            //SOURCE_1: p.366, diag. 6-7
            Real A = 0.95 + 33.5 / max(MIN, delta) "Coefficient considering effect of angle of turning on zeta_LOC";
            Real C1 = 1 "Considering relative elongation of cross sectional area on zeta_LOC (here: circular cross sectional area)";
            //SOURCE_1: p.366, diag. 6-7
            TYP.LocalResistanceCoefficient zeta_LOC = max(MIN, 0.95 * sin(PI / 180 * delta / 2) ^ 2 + 2.05 * sin(PI / 180 * delta / 2) ^ 4) "Local resistance coefficient";
            //SOURCE_1: p.365: Correction w.r.t effect of Reynolds number in laminar regime
            Real B = 24.8 "Coefficient considering effect of Reynolds number on zeta_TOT";
            Real exp = 0.263 "Exponent for Reynolds number correction in laminar regime";
            Real v_min = Re_min * IN_var.eta / (IN_var.rho * d_hyd) "Minimum mean velocity for linear interpolation";
            SI.Velocity velocity = m_flow / (IN_var.rho * A_cross) "Mean velocity";
            SI.ReynoldsNumber Re = max(Re_min, IN_var.rho * abs(velocity) * d_hyd / IN_var.eta) "Reynolds number";
            //SOURCE_2: p.191, eq. 8.4: considering surface roughness
            TYP.DarcyFrictionFactor lambda_FRI_rough = 0.25 / Modelica.Math.log10(k / (3.7 * IN_con.d_hyd) + 5.74 / max(Re_lam_min, Re) ^ 0.9) ^ 2 "Darcy friction factor considering surface roughness";
            TYP.DarcyFrictionFactor lambda_FRI_smooth = 0.25 / Modelica.Math.log10(5.74 / max(Re_lam_min, Re) ^ 0.9) ^ 2 "Darcy friction factor neglecting surface roughness";
            //SOURCE_3: Lac 6, Figure 18
            Real CF_fri = SMOOTH(Re_lam_leave, Re_lam_min, Re) * max(1, min(1.4, lambda_FRI_rough / lambda_FRI_smooth)) + SMOOTH(Re_lam_min, Re_lam_leave, Re) "Correction factor for surface roughness";
            //SOURCE_2: p.208, diag. 9.3: Correction w.r.t effect of Reynolds number
            Real CF_Re = SMOOTH(Re_turb_min, Re_turb_max, Re) * B / Re ^ exp + SMOOTH(Re_turb_max, Re_turb_min, Re) "Correction factor for Reynolds number";
            TYP.PressureLossCoefficient zeta_TOT = A * C1 * zeta_LOC * CF_Re * CF_fri "Pressure loss coefficient";
          algorithm
            DP := zeta_TOT * (IN_var.rho / 2) * Modelica.Fluid.Dissipation.Utilities.Functions.General.SmoothPower(velocity, v_min, 2);
            annotation(
              Inline = false,
              smoothOrder(normallyConstant = IN_con) = 2,
              inverse(m_flow = Modelica.Fluid.Dissipation.PressureLoss.Bend.dp_edgedOverall_MFLOW(IN_con, IN_var, DP)),
              Documentation(info = "<html>
<p>
Calculation of pressure loss in edged bends with sharp corners at overall flow regime for incompressible and single-phase fluid flow through circular cross sectional area considering surface roughness.
</p>

<p>
There are larger pressure losses in an edged bend compared to a curved bend under the same conditions. The effect of a sharp corner in an edged bend on the pressure loss is much larger than the influence of surface roughness.
</p>

<p>
Generally this function is numerically best used for the <b>incompressible case</b>, where the mass flow rate (m_flow) is known (as state variable) in the used model and the corresponding pressure loss (DP) has to be calculated. On the other hand the function <a href=\"modelica://Modelica.Fluid.Dissipation.PressureLoss.Bend.dp_edgedOverall_MFLOW\">dp_edgedOverall_MFLOW</a> is numerically best used for the <b> compressible case </b> if the pressure loss (dp) is known (out of pressures as state variable) and the mass flow rate (M_FLOW) has to be calculated.
<a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.PressureLoss.Bend.dp_edgedOverall\">See more information</a> .</p>
</html>"));
          end dp_edgedOverall_DP;

          function dp_edgedOverall_MFLOW "Pressure loss of edged bend | calculate mass flow rate | overall flow regime | surface roughness"
            extends Modelica.Icons.Function;
            //SOURCE_1: Idelchik, I.E.: HANDBOOK OF HYDRAULIC RESISTANCE, 3rd edition, 2006.
            //SOURCE_2: Miller, D.S.: INTERNAL FLOW SYSTEMS, 2nd edition, 1984.
            //SOURCE_3: VDI-Waermeatlas, 9th edition, Springer-Verlag, 2002, Section Lac 6 (Verification)
            //Notation of equations according to SOURCES
            import FD = Modelica.Fluid.Dissipation.PressureLoss.Bend;
            import SMOOTH = Modelica.Fluid.Dissipation.Utilities.Functions.General.Stepsmoother;
            //input records
            input Modelica.Fluid.Dissipation.PressureLoss.Bend.dp_edgedOverall_IN_con IN_con "Input record for function dp_edgedOverall_MFLOW" annotation(
              Dialog(group = "Constant inputs"));
            input Modelica.Fluid.Dissipation.PressureLoss.Bend.dp_edgedOverall_IN_var IN_var "Input record for function dp_edgedOverall_MFLOW" annotation(
              Dialog(group = "Variable inputs"));
            input SI.Pressure dp "Pressure loss" annotation(
              Dialog(group = "Input"));
            //output variables
            output SI.MassFlowRate M_FLOW "Output for function dp_edgedOverall_MFLOW";
          protected
            Real MIN = Modelica.Constants.eps;
            SI.Diameter d_hyd = IN_con.d_hyd "Hydraulic diameter";
            SI.Area A_cross = PI * d_hyd ^ 2 / 4 "Circular cross sectional area";
            Real k = max(MIN, abs(IN_con.K) / IN_con.d_hyd) "Relative roughness";
            Real delta = IN_con.delta * 180 / PI "Angle of turning";
            //definition of flow regime boundaries
            //SOURCE_2: p.207, sec. 9.2.4
            //SOURCE_3: p.Lac 6, fig. 16
            SI.ReynoldsNumber Re_min = 1 "Minimum Reynolds number";
            SI.ReynoldsNumber Re_lam_min = 500 "Start of transition regime for roughness contribution";
            SI.ReynoldsNumber Re_lam_max = 1e4 "End of transition regime for roughness contribution";
            SI.ReynoldsNumber Re_turb_min = 1e5 "Minimum Reynolds number for Reynolds-dependent transition regime";
            SI.ReynoldsNumber Re_turb_max = 2e5 "Maximum Reynolds number for Reynolds-dependent transition regime (k_Re=1)";
            SI.ReynoldsNumber Re_turb_const = 1e6 "Reynolds number for independence on pressure loss coefficient (1e6)";
            //SOURCE_1: p. 81, sec. 2-2-21: start of transition regime
            SI.ReynoldsNumber Re_lam_leave = min(Re_lam_max, max(Re_lam_min, 754 * Modelica.Math.exp(if k <= 0.007 then 0.0065 / 0.007 else 0.0065 / k))) "Start of transition regime for increasing Reynolds number (leaving laminar regime)";
            //SOURCE_1: p.366, diag. 6-7
            Real A = 0.95 + 33.5 / max(MIN, delta) "Coefficient considering effect of angle of turning on zeta_LOC";
            Real C1 = 1 "Considering relative elongation of cross sectional area on zeta_LOC (here: circular cross sectional area)";
            //SOURCE_1: p.366, diag. 6-7
            TYP.LocalResistanceCoefficient zeta_LOC = max(MIN, 0.95 * sin(PI / 180 * delta / 2) ^ 2 + 2.05 * sin(PI / 180 * delta / 2) ^ 4) "Local resistance coefficient";
            //SOURCE_1: p.365: Correction w.r.t effect of Reynolds number
            Real B = 24.8 "Coefficient considering effect of Reynolds number on zeta_TOT";
            Real exp = 0.263 "Exponent for Reynolds number correction";
            Real pow = 2 - exp "pressure loss = f(mass flow rate^pow)";
            //   Real k_Re = B/(max(MIN, velocity)*IN_con.d_hyd*IN_var.rho)^exp*IN_var.eta^exp;
            SI.Velocity v_min = Re_min * IN_var.eta / (IN_var.rho * d_hyd) "Minimum mean velocity for regularization";
            SI.Pressure dp_min = A * C1 * zeta_LOC * (B / 2) * (d_hyd / IN_var.eta) ^ (-exp) * IN_var.rho ^ (1 - exp) * v_min ^ pow "Linear smoothing of mass flow rate for decreasing pressure loss";
            SI.Velocity v_lam_min = Re_lam_min * IN_var.eta / (IN_var.rho * d_hyd) "Mean velocity for starting of transition to roughness regime";
            SI.Velocity v_lam_leave = Re_lam_leave * IN_var.eta / (IN_var.rho * d_hyd) "Mean velocity for end of transition to roughness regime";
            SI.Pressure dp_lam_min = A * C1 * zeta_LOC * (B / 2) * (d_hyd / IN_var.eta) ^ (-exp) * IN_var.rho ^ (1 - exp) * v_lam_min ^ pow "Pressure loss for starting of transition to roughness regime";
            TYP.DarcyFrictionFactor lambda_lam_leave_rough = 0.25 / Modelica.Math.log10(k / (3.7 * IN_con.d_hyd) + 5.74 / Re_lam_leave ^ 0.9) ^ 2 "Darcy friction factor considering surface roughness at Re_lem_leave";
            TYP.DarcyFrictionFactor lambda_lam_leave_smooth = 0.25 / Modelica.Math.log10(5.74 / Re_lam_leave ^ 0.9) ^ 2 "Darcy friction factor neglecting surface roughness at Re_lam_leave";
            SI.Pressure dp_lam_leave = A * C1 * zeta_LOC * (B / 2) * max(1, min(1.4, lambda_lam_leave_rough / lambda_lam_leave_smooth)) * (d_hyd / IN_var.eta) ^ (-exp) * IN_var.rho ^ (1 - exp) * v_lam_leave ^ pow "Pressure loss at end of transition to surface roughness regime";
            TYP.DarcyFrictionFactor lambda_turb_min_rough = 0.25 / Modelica.Math.log10(k / (3.7 * IN_con.d_hyd) + 5.74 / Re_turb_min ^ 0.9) ^ 2 "Darcy friction factor considering surface roughness at starting transition to constant turbulent regime";
            TYP.DarcyFrictionFactor lambda_turb_min_smooth = 0.25 / Modelica.Math.log10(5.74 / Re_turb_min ^ 0.9) ^ 2 "Darcy friction factor neglecting surface roughness at starting transition to constant turbulent regime";
            TYP.DarcyFrictionFactor lambda_turb_max_rough = 0.25 / Modelica.Math.log10(k / (3.7 * IN_con.d_hyd) + 5.74 / Re_turb_max ^ 0.9) ^ 2 "Darcy friction factor considering surface roughness at starting transition to constant turbulent regime";
            TYP.DarcyFrictionFactor lambda_turb_max_smooth = 0.25 / Modelica.Math.log10(5.74 / Re_turb_max ^ 0.9) ^ 2 "Darcy friction factor neglecting surface roughness at starting transition to constant turbulent regime";
            SI.Velocity v_turb_min = Re_turb_min * IN_var.eta / (IN_var.rho * d_hyd) "Mean velocity for starting of transition to constant turbulent regime";
            SI.Velocity v_turb_max = Re_turb_max * IN_var.eta / (IN_var.rho * d_hyd) "Mean velocity for end of transition to constant turbulent regime";
            SI.Pressure dp_turb_min = A * C1 * zeta_LOC * (B / 2) * max(1, min(1.4, lambda_turb_min_rough / lambda_turb_min_smooth)) * (d_hyd / IN_var.eta) ^ (-exp) * IN_var.rho ^ (1 - exp) * v_turb_min ^ pow "Pressure loss at starting of transition to constant turbulent regime";
            SI.Pressure dp_turb_max = A * C1 * zeta_LOC * max(1, min(1.4, lambda_turb_max_rough / lambda_turb_max_smooth)) * IN_var.rho / 2 * v_turb_max ^ 2 "Pressure loss at end of transition to constant turbulent regime";
            SI.Velocity v_turb = (A * C1 * zeta_LOC * IN_var.rho / 2) ^ (-0.5) * Modelica.Fluid.Dissipation.Utilities.Functions.General.SmoothPower(abs(dp), dp_min, 0.5) "Mean velocity under turbulent conditions";
            SI.Velocity v_lam = (2 * (d_hyd / IN_var.eta) ^ exp / (A * C1 * zeta_LOC * B * IN_var.rho ^ (1 - exp))) ^ (1 / pow) * Modelica.Fluid.Dissipation.Utilities.Functions.General.SmoothPower(abs(dp), dp_min, 1 / pow) "Mean velocity under laminar conditions";
            //mean velocity under smooth conditions w.r.t flow regime
            SI.Velocity v_smooth = if abs(dp) > dp_turb_max then v_turb else if abs(dp) < dp_turb_min then v_lam else SMOOTH(dp_turb_max, dp_turb_min, abs(dp)) * v_turb + SMOOTH(dp_turb_min, dp_turb_max, abs(dp)) * v_lam "Mean velocity under smooth conditions";
            SI.ReynoldsNumber Re_smooth = max(Re_min, IN_var.rho * v_smooth * d_hyd / IN_var.eta) "Reynolds number under smooth conditions";
            //SOURCE_2: p.191, eq. 8.4: considering surface roughness
            TYP.DarcyFrictionFactor lambda_FRI_rough = 0.25 / Modelica.Math.log10(k / (3.7 * IN_con.d_hyd) + 5.74 / max(Re_lam_min, Re_smooth) ^ 0.9) ^ 2 "Darcy friction factor considering surface roughness";
            TYP.DarcyFrictionFactor lambda_FRI_smooth = 0.25 / Modelica.Math.log10(5.74 / max(Re_lam_min, Re_smooth) ^ 0.9) ^ 2 "Darcy friction factor neglecting surface roughness";
            //SOURCE_3: Lac 6, Figure 18
            Real CF_fri = SMOOTH(dp_lam_leave, dp_lam_min, abs(dp)) * max(1, min(1.4, lambda_FRI_rough / lambda_FRI_smooth)) + SMOOTH(dp_lam_min, dp_lam_leave, abs(dp)) "Correction factor for surface roughness";
            SI.Velocity velocity = v_smooth / max(1, CF_fri) ^ 0.5 "Corrected velocity considering surface roughness";
          algorithm
            M_FLOW := sign(dp) * IN_var.rho * A_cross * velocity;
            annotation(
              Inline = false,
              smoothOrder(normallyConstant = IN_con) = 2,
              inverse(dp = Modelica.Fluid.Dissipation.PressureLoss.Bend.dp_edgedOverall_DP(IN_con, IN_var, M_FLOW)),
              Documentation(info = "<html>
<p>
Calculation of pressure loss in edged bends with sharp corners at overall flow regime for incompressible and single-phase fluid flow through circular cross sectional area considering surface roughness.
</p>

<p>
There are larger pressure losses in an edged bend compared to a curved bend under the same conditions. The effect of a sharp corner in an edged bend on the pressure loss is much larger than the influence of surface roughness.
</p>

<p>
Generally this function is numerically best used for the <b> compressible case </b> if the pressure loss (dp) is known (out of pressures as state variable) and the mass flow rate (M_FLOW) has to be calculated. On the other hand the function <a href=\"modelica://Modelica.Fluid.Dissipation.PressureLoss.Bend.dp_edgedOverall_DP\">dp_edgedOverall_DP</a> is numerically best used for the <b> incompressible case </b>, where the mass flow rate (m_flow) is known (as state variable) in the used model and the corresponding pressure loss (DP) has to be calculated.
<a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.PressureLoss.Bend.dp_edgedOverall\">See more information</a> .</p>
</html>", revisions = "<html>
2014-12-02 Stefan Wischhusen: Changed transition regimes for laminar to highly turbulent domain.
</html>"));
          end dp_edgedOverall_MFLOW;

          record dp_edgedOverall_IN_con "Input record for function dp_edgedOverall_DP and dp_edgedOverall_MFLOW"
            //edged bend variables
            extends Modelica.Fluid.Dissipation.Utilities.Records.PressureLoss.EdgedBend;
            annotation(
              Documentation(info = "<html>
This record is used as <b> input record </b> for the  pressure loss function
<a href=\"modelica://Modelica.Fluid.Dissipation.PressureLoss.Bend.dp_edgedOverall_DP\"> dp_edgedOverall_DP </a> and
<a href=\"modelica://Modelica.Fluid.Dissipation.PressureLoss.Bend.dp_edgedOverall_MFLOW\"> dp_edgedOverall_MFLOW </a>.
</html>"));
          end dp_edgedOverall_IN_con;

          record dp_edgedOverall_IN_var "Input record for function dp_edgedOverall_DP and  dp_edgedOverall_MFLOW"
            //fluid property variables
            extends Modelica.Fluid.Dissipation.Utilities.Records.General.PressureLoss;
            annotation(
              Documentation(info = "<html>
This record is used as <b> input record </b> for the  pressure loss function
<a href=\"modelica://Modelica.Fluid.Dissipation.PressureLoss.Bend.dp_edgedOverall_DP\"> dp_edgedOverall_DP </a> and
<a href=\"modelica://Modelica.Fluid.Dissipation.PressureLoss.Bend.dp_edgedOverall_MFLOW\"> dp_edgedOverall_MFLOW </a>.
</html>"));
          end dp_edgedOverall_IN_var;
          annotation(
            preferredView = "info",
            Documentation(info = "<html>
<h4>Bend</h4>
<h5>Curved bend overall flow</h5>
<p>Calculation of pressure loss in curved bends at overall flow regime for incompressible and single-phase fluid flow through circular cross sectional area considering surface roughness.  <a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.PressureLoss.Bend.dp_curvedOverall\">See more information</a>. </p>

<h5>Edged bend overall flow</h5>
<p>
Calculation of pressure loss in edged bends with sharp corners at overall flow regime for incompressible and single-phase fluid flow through circular cross sectional area considering surface roughness.
<a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.PressureLoss.Bend.dp_edgedOverall\">See more information</a> .</p>

</html>"));
        end Bend;

        package Channel "Package for pressure loss calculation of channels"
          extends Modelica.Icons.VariantsPackage;

          function dp_internalFlowOverall_DP "Pressure loss of internal flow | calculate pressure loss | overall flow regime | surface roughness | several geometries"
            extends Modelica.Icons.Function;
            //SOURCE_1: Idelchik, I.E.: HANDBOOK OF HYDRAULIC RESISTANCE, 3rd edition, 2006.
            //SOURCE_2: Miller, D.S.: INTERNAL FLOW SYSTEMS, 1978.
            //SOURCE_3: VDI-Waermeatlas, 9th edition, Springer-Verlag, 2002
            //Notation of equations according to SOURCES
            import FD = Modelica.Fluid.Dissipation.PressureLoss.Channel;
            import SMOOTH = Modelica.Fluid.Dissipation.Utilities.Functions.General.Stepsmoother;
            //input records
            input Modelica.Fluid.Dissipation.PressureLoss.Channel.dp_internalFlowOverall_IN_con IN_con "Input record for function dp_internalFlowOverall_DP" annotation(
              Dialog(group = "Constant inputs"));
            input Modelica.Fluid.Dissipation.PressureLoss.Channel.dp_internalFlowOverall_IN_var IN_var "Input record for function dp_internalFlowOverall_DP" annotation(
              Dialog(group = "Variable inputs"));
            input SI.MassFlowRate m_flow "Mass flow rate" annotation(
              Dialog(group = "Input"));
            //output variables
            output SI.Pressure DP "Output for function dp_internalFlowOverall_DP";
          protected
            type TYP = Modelica.Fluid.Dissipation.Utilities.Types.GeometryOfInternalFlow;
            Real MIN = Modelica.Constants.eps;
            SI.Area A_cross = max(MIN, if IN_con.geometry == TYP.Annular then PI / 4 * (IN_con.D_ann ^ 2 - IN_con.d_ann ^ 2) else if IN_con.geometry == TYP.Circular then PI / 4 * IN_con.d_cir ^ 2 else if IN_con.geometry == TYP.Elliptical then PI * IN_con.a_ell * IN_con.b_ell else if IN_con.geometry == TYP.Rectangular then IN_con.a_rec * IN_con.b_rec else if IN_con.geometry == TYP.Isosceles then 0.5 * (IN_con.a_tri * IN_con.h_tri) else 0) "Cross sectional area";
            SI.Length perimeter = max(MIN, if IN_con.geometry == TYP.Annular then PI * (IN_con.D_ann + IN_con.d_ann) else if IN_con.geometry == TYP.Circular then PI * IN_con.d_cir else if IN_con.geometry == TYP.Elliptical then PI * (IN_con.a_ell + IN_con.b_ell) else if IN_con.geometry == TYP.Rectangular then 2 * (IN_con.a_rec + IN_con.b_rec) else if IN_con.geometry == TYP.Isosceles then IN_con.a_tri + 2 * (IN_con.h_tri ^ 2 + (IN_con.a_tri / 2) ^ 2) ^ 0.5 else 0) "Perimeter";
            SI.Diameter d_hyd = 4 * A_cross / perimeter "Hydraulic diameter";
            Real beta = IN_con.beta * 180 / PI "Top angle";
            //SOURCE_2: p.138, sec 8.5
            Real Dd_ann = min(max(MIN, IN_con.d_ann), IN_con.D_ann) / max(MIN, max(IN_con.d_ann, IN_con.D_ann)) "Ratio of small to large diameter of annular geometry";
            Real CF_ann = 98.7378 * Dd_ann ^ 0.0589 "Correction factor for annular geometry";
            Real ab_rec = min(IN_con.a_rec, IN_con.b_rec) / max(MIN, max(IN_con.a_rec, IN_con.b_rec)) "Aspect ratio of rectangular geometry";
            Real CF_rec = (-59.85 * ab_rec ^ 3) + 148.67 * ab_rec ^ 2 - 128.1 * ab_rec + 96.1 "Correction factor for rectangular geometry";
            Real ab_ell = min(IN_con.a_ell, IN_con.b_ell) / max(MIN, max(IN_con.a_ell, IN_con.b_ell)) "Ratio of small to large length of annular geometry";
            Real CF_ell = (-169.2211 * ab_ell ^ 4) + 260.9028 * ab_ell ^ 3 - 113.7890 * ab_ell ^ 2 + 9.2588 * ab_ell ^ 1 + 78.7124 "Correction factor for elliptical geometry";
            Real CF_tri = (-0.0013 * min(90, beta) ^ 2) + 0.1577 * min(90, beta) + 48.5575 "Correction factor for triangular geometry";
            Real CF_lam = if IN_con.geometry == TYP.Annular then CF_ann else if IN_con.geometry == TYP.Circular then 64 else if IN_con.geometry == TYP.Elliptical then CF_ell else if IN_con.geometry == TYP.Rectangular then CF_rec else if IN_con.geometry == TYP.Isosceles then CF_tri else 0 "Correction factor for laminar flow";
            //SOURCE_1: p.81, fig. 2-3, sec 21-22: definition of flow regime boundaries
            Real k = max(MIN, abs(IN_con.K) / d_hyd) "Relative roughness";
            SI.ReynoldsNumber Re_lam_min = 1e3 "Minimum Reynolds number for laminar regime";
            SI.ReynoldsNumber Re_lam_max = 2090 * (1 / max(0.007, k)) ^ 0.0635 "Maximum Reynolds number for laminar regime";
            SI.ReynoldsNumber Re_lam_leave = min(Re_lam_max, max(Re_lam_min, 754 * Modelica.Math.exp(if k <= 0.007 then 0.0065 / 0.007 else 0.0065 / k))) "Start of transition regime for increasing Reynolds number (leaving laminar regime)";
            //Adapted mass flow rate for function dp_turbulent of a straight pipe
            SI.MassFlowRate m_flow_turb = m_flow * (PI / 4 * d_hyd ^ 2) / A_cross "Mass flow rate for turbulent calculation";
            SI.Velocity velocity = m_flow / (IN_var.rho * A_cross) "Velocity of internal flow";
            SI.ReynoldsNumber Re = IN_var.rho * abs(velocity) * d_hyd / IN_var.eta;
          protected
            Modelica.Fluid.Dissipation.PressureLoss.StraightPipe.dp_overall_IN_con IN_2_con(final roughness = IN_con.roughness, final d_hyd = d_hyd, final K = IN_con.K, final L = IN_con.L) "Input record for turbulent regime" annotation(
              Placement(transformation(extent = {{-100, 20}, {-80, 40}})));
            Modelica.Fluid.Dissipation.PressureLoss.StraightPipe.dp_overall_IN_var IN_2_var(final eta = IN_var.eta, final rho = IN_var.rho) "Input record for turbulent regime" annotation(
              Placement(transformation(extent = {{-80, 20}, {-60, 40}})));
            //Documentation
          algorithm
            DP := SMOOTH(Re_lam_min, Re_lam_max, Re) * (CF_lam / 2) * IN_con.L / d_hyd ^ 2 * velocity * IN_var.eta + SMOOTH(Re_lam_max, Re_lam_min, Re) * Dissipation.PressureLoss.StraightPipe.dp_turbulent_DP(IN_2_con, IN_2_var, m_flow_turb);
            annotation(
              Inline = false,
              smoothOrder(normallyConstant = IN_con) = 2,
              inverse(m_flow = Modelica.Fluid.Dissipation.PressureLoss.Channel.dp_internalFlowOverall_MFLOW(IN_con, IN_var, DP)),
              Documentation(info = "<html>
<p>
Calculation of pressure loss for an internal flow through different geometries at overall flow regime for incompressible and single-phase fluid flow considering surface roughness.
</p>

<p>
Generally this function is numerically best used for the <b> incompressible case </b>, where the mass flow rate (m_flow) is known (as state variable) in the used model and the corresponding pressure loss (DP) has to be calculated. On the other hand the function <a href=\"modelica://Modelica.Fluid.Dissipation.PressureLoss.Channel.dp_internalFlowOverall_MFLOW\">dp_internalFlowOverall_MFLOW</a> is numerically best used for the <b> compressible case </b> if the pressure loss (dp) is known (out of pressures as state variable) and the mass flow rate (M_FLOW) has to be calculated.  <a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.PressureLoss.Channel.dp_internalFlowOverall\">See more information</a>.
</p>
</html>"));
          end dp_internalFlowOverall_DP;

          function dp_internalFlowOverall_MFLOW "Pressure loss of internal flow | calculate mass flow rate | overall flow regime | surface roughness | several geometries"
            extends Modelica.Icons.Function;
            import FD = Modelica.Fluid.Dissipation.PressureLoss.Channel;
            import SMOOTH = Modelica.Fluid.Dissipation.Utilities.Functions.General.Stepsmoother;
            //input records
            input Modelica.Fluid.Dissipation.PressureLoss.Channel.dp_internalFlowOverall_IN_con IN_con "Input record for function dp_internalFlowOverall_MFLOW" annotation(
              Dialog(group = "Constant inputs"));
            input Modelica.Fluid.Dissipation.PressureLoss.Channel.dp_internalFlowOverall_IN_var IN_var "Input record for function dp_internalFlowOverall_MFLOW" annotation(
              Dialog(group = "Variable inputs"));
            input SI.Pressure dp "Pressure loss" annotation(
              Dialog(group = "Input"));
            //output variables
            output SI.MassFlowRate M_FLOW "Output of function dp_overall_MFLOW";
          protected
            type TYP1 = Modelica.Fluid.Dissipation.Utilities.Types.GeometryOfInternalFlow;
            type TYP2 = Modelica.Fluid.Dissipation.Utilities.Types.Roughness;
            Real MIN = Modelica.Constants.eps;
            SI.Area A_cross = max(MIN, if IN_con.geometry == TYP1.Annular then PI / 4 * (IN_con.D_ann ^ 2 - IN_con.d_ann ^ 2) else if IN_con.geometry == TYP1.Circular then PI / 4 * IN_con.d_cir ^ 2 else if IN_con.geometry == TYP1.Elliptical then PI * IN_con.a_ell * IN_con.b_ell else if IN_con.geometry == TYP1.Rectangular then IN_con.a_rec * IN_con.b_rec else if IN_con.geometry == TYP1.Isosceles then 0.5 * (IN_con.a_tri * IN_con.h_tri) else 0) "Cross sectional area";
            SI.Length perimeter = max(MIN, if IN_con.geometry == TYP1.Annular then PI * (IN_con.D_ann + IN_con.d_ann) else if IN_con.geometry == TYP1.Circular then PI * IN_con.d_cir else if IN_con.geometry == TYP1.Elliptical then PI * (IN_con.a_ell + IN_con.b_ell) else if IN_con.geometry == TYP1.Rectangular then 2 * (IN_con.a_rec + IN_con.b_rec) else if IN_con.geometry == TYP1.Isosceles then IN_con.a_tri + 2 * (IN_con.h_tri ^ 2 + (IN_con.a_tri / 2) ^ 2) ^ 0.5 else 0) "Perimeter";
            SI.Diameter d_hyd = 4 * A_cross / perimeter "Hydraulic diameter";
            Real beta = IN_con.beta * 180 / PI "Top angle";
            //SOURCE_2: p.138, sec 8.5
            Real Dd_ann = min(max(MIN, IN_con.d_ann), IN_con.D_ann) / max(MIN, max(IN_con.d_ann, IN_con.D_ann)) "Ratio of small to large diameter of annular geometry";
            Real CF_ann = 98.7378 * Dd_ann ^ 0.0589 "Correction factor for annular geometry";
            Real ab_rec = min(IN_con.a_rec, IN_con.b_rec) / max(MIN, max(IN_con.a_rec, IN_con.b_rec)) "Aspect ratio of rectangular geometry";
            Real CF_rec = (-59.85 * ab_rec ^ 3) + 148.67 * ab_rec ^ 2 - 128.1 * ab_rec + 96.1 "Correction factor for rectangular geometry";
            Real ab_ell = min(IN_con.a_ell, IN_con.b_ell) / max(MIN, max(IN_con.a_ell, IN_con.b_ell)) "Ratio of small to large length of annular geometry";
            Real CF_ell = (-169.2211 * ab_ell ^ 4) + 260.9028 * ab_ell ^ 3 - 113.7890 * ab_ell ^ 2 + 9.2588 * ab_ell ^ 1 + 78.7124 "Correction factor for elliptical geometry";
            Real CF_tri = (-0.0013 * min(90, beta) ^ 2) + 0.1577 * min(90, beta) + 48.5575 "Correction factor for triangular geometry";
            Real CF_lam = if IN_con.geometry == TYP1.Annular then CF_ann else if IN_con.geometry == TYP1.Circular then 64 else if IN_con.geometry == TYP1.Elliptical then CF_ell else if IN_con.geometry == TYP1.Rectangular then CF_rec else if IN_con.geometry == TYP1.Isosceles then CF_tri else 0 "Correction factor for laminar flow";
            //SOURCE_1: p.81, fig. 2-3, sec 21-22: definition of flow regime boundaries
            Real k = max(MIN, abs(IN_con.K) / d_hyd) "Relative roughness";
            SI.ReynoldsNumber Re_lam_min = 1e3 "Minimum Reynolds number for laminar regime";
            SI.ReynoldsNumber Re_lam_max = 2090 * (1 / max(0.007, k)) ^ 0.0635 "Maximum Reynolds number for laminar regime";
            SI.ReynoldsNumber Re_turb_min = 4e3 "Minimum Reynolds number for turbulent regime";
            SI.ReynoldsNumber Re_lam_leave = min(Re_lam_max, max(Re_lam_min, 754 * Modelica.Math.exp(if k <= 0.007 then 0.0065 / 0.007 else 0.0065 / k))) "Start of transition regime for increasing Reynolds number (leaving laminar regime)";
            //determining Darcy friction factor out of pressure loss calculation for straight pipe:
            //dp = lambda_FRI*L/d_hyd*(rho/2)*velocity^2 and assuming lambda_FRI == lambda_FRI_calc/Re^2
            TYP.DarcyFrictionFactor lambda_FRI_calc = 2 * abs(dp) * d_hyd ^ 3 * IN_var.rho / (IN_con.L * IN_var.eta ^ 2) "Adapted Darcy friction factor";
            //SOURCE_3: p.Lab 1, eq. 5: determine Re assuming laminar regime
            SI.ReynoldsNumber Re_lam = lambda_FRI_calc / CF_lam "Reynolds number assuming laminar regime";
            //SOURCE_3: p.Lab 2, eq. 10: determine Re assuming turbulent regime (Colebrook-White)
            SI.ReynoldsNumber Re_turb = if IN_con.roughness == TYP2.Neglected then (max(MIN, lambda_FRI_calc) / 0.3164) ^ (1 / 1.75) else -2 * sqrt(max(lambda_FRI_calc, MIN)) * Modelica.Math.log10(2.51 / sqrt(max(lambda_FRI_calc, MIN)) + k / 3.7) "Reynolds number assuming turbulent regime";
            //determine actual flow regime
            SI.ReynoldsNumber Re_check = if Re_lam < Re_lam_leave then Re_lam else Re_turb;
            //determine Re for transition regime
            SI.ReynoldsNumber Re_trans = if Re_lam >= Re_lam_leave then Modelica.Fluid.Dissipation.Utilities.Functions.General.CubicInterpolation_DP(Re_check, Re_lam_leave, Re_turb_min, k, lambda_FRI_calc) else 0;
            //determine actual Re
            SI.ReynoldsNumber Re = if Re_lam < Re_lam_leave then Re_lam else if Re_turb > Re_turb_min then Re_turb else Re_trans;
            Modelica.Fluid.Dissipation.PressureLoss.StraightPipe.dp_overall_IN_con IN_2_con(final roughness = IN_con.roughness, final d_hyd = d_hyd, final K = IN_con.K, final L = IN_con.L) "Input record for turbulent regime" annotation(
              Placement(transformation(extent = {{-100, 20}, {-80, 40}})));
            Modelica.Fluid.Dissipation.PressureLoss.StraightPipe.dp_overall_IN_var IN_2_var(final eta = IN_var.eta, final rho = IN_var.rho) "Input record for turbulent regime" annotation(
              Placement(transformation(extent = {{-80, 20}, {-60, 40}})));
            //Documentation
          algorithm
            M_FLOW := SMOOTH(Re_lam_min, Re_turb, Re) * IN_var.rho * A_cross * (dp * (2 / CF_lam) * (d_hyd ^ 2 / IN_con.L) * (1 / IN_var.eta)) + SMOOTH(Re_turb, Re_lam_min, Re) * (A_cross / (PI / 4 * d_hyd ^ 2)) * Modelica.Fluid.Dissipation.PressureLoss.StraightPipe.dp_turbulent_MFLOW(IN_2_con, IN_2_var, dp);
            annotation(
              Inline = false,
              smoothOrder(normallyConstant = IN_con) = 2,
              inverse(dp = Modelica.Fluid.Dissipation.PressureLoss.Channel.dp_internalFlowOverall_DP(IN_con, IN_var, M_FLOW)),
              Documentation(info = "<html>
<p>
Calculation of pressure loss for an internal flow through different geometries at overall flow regime for incompressible and single-phase fluid flow considering surface roughness.
</p>

<p>
Generally this function is numerically best used for the <b> compressible case </b> if the pressure loss (dp) is known (out of pressures as state variable) and the mass flow rate (M_FLOW) has to be calculated. On the other hand the function <a href=\"modelica://Modelica.Fluid.Dissipation.PressureLoss.Channel.dp_internalFlowOverall_DP\">dp_internalFlowOverall_DP</a> is numerically best used for the <b> incompressible case </b>, where the mass flow rate (m_flow) is known (as state variable) in the used model and the corresponding pressure loss (DP) has to be calculated. </p>

<p>
The pressure loss calculation for internal fluid flow in different geometries is further documented  <a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.PressureLoss.Channel.dp_internalFlowOverall\">here</a>.
</p>
</html>"));
          end dp_internalFlowOverall_MFLOW;

          record dp_internalFlowOverall_IN_con "Input record for function dp_internalFlowOverall_DP and dp_internalFlowOverall_MFLOW"
            //channel variables
            Modelica.Fluid.Dissipation.Utilities.Types.Roughness roughness = Dissipation.Utilities.Types.Roughness.Considered "Choice of considering surface roughness" annotation(
              Dialog(group = "Channel"));
            extends Modelica.Fluid.Dissipation.Utilities.Records.PressureLoss.Geometry;
            annotation(
              Documentation(info = "<html>
This record is used as <b> input record </b> for the pressure loss function
<a href=\"modelica://Modelica.Fluid.Dissipation.PressureLoss.Channel.dp_internalFlowOverall_DP\"> dp_internalFlowOverall_DP</a> and
<a href=\"modelica://Modelica.Fluid.Dissipation.PressureLoss.Channel.dp_internalFlowOverall_MFLOW\"> dp_internalFlowOverall_MFLOW</a>.
</html>"));
          end dp_internalFlowOverall_IN_con;

          record dp_internalFlowOverall_IN_var "Input record for function dp_internalFlowOverall_DP and dp_internalFlowOverall_MFLOW"
            //fluid property variables
            extends Modelica.Fluid.Dissipation.Utilities.Records.General.PressureLoss;
            annotation(
              Documentation(info = "<html>
This record is used as <b> input record </b> for the pressure loss function
<a href=\"modelica://Modelica.Fluid.Dissipation.PressureLoss.Channel.dp_internalFlowOverall_DP\"> dp_internalFlowOverall_DP</a> and
<a href=\"modelica://Modelica.Fluid.Dissipation.PressureLoss.Channel.dp_internalFlowOverall_MFLOW\"> dp_internalFlowOverall_MFLOW</a>.
</html>"));
          end dp_internalFlowOverall_IN_var;
          annotation(
            preferredView = "info",
            Documentation(info = "<html>
<h4>Channel</h4>
<h5>Internal overall flow</h5>
<p>
Calculation of pressure loss for an internal flow through different geometries at laminar and turbulent flow regime considering surface roughness.
<a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.PressureLoss.Channel.dp_internalFlowOverall\">See more information</a>.
</p>
</html>"));
        end Channel;

        package General "Package for generic pressure loss calculations"
          extends Modelica.Icons.VariantsPackage;

          function dp_idealGas_DP "Generic pressure loss | calculate pressure loss | ideal gas | mean density"
            extends Modelica.Icons.Function;
            import FD = Modelica.Fluid.Dissipation.PressureLoss.General;
            //input records
            input Modelica.Fluid.Dissipation.PressureLoss.General.dp_idealGas_IN_con IN_con "Input record for function dp_idealGas_DP" annotation(
              Dialog(group = "Constant inputs"));
            input Modelica.Fluid.Dissipation.PressureLoss.General.dp_idealGas_IN_var IN_var "Input record for function dp_idealGas_DP" annotation(
              Dialog(group = "Variable inputs"));
            input SI.MassFlowRate m_flow "Mass flow rate" annotation(
              Dialog(group = "Input"));
            //output variables
            output SI.Pressure DP "Output for function dp_idealGas_DP";
          protected
            Real Km_internal = IN_con.Km "Coefficient for pressure loss";
            SI.Density rho_internal = IN_var.p_m / (IN_con.R_s * IN_var.T_m) "Mean density";
            SI.VolumeFlowRate V_flow = m_flow / rho_internal "Volume flow rate [m3/s]";
            SI.VolumeFlowRate V_flow_min = (IN_con.R_s / Km_internal) ^ (1 / IN_con.exp) * rho_internal ^ (1 / IN_con.exp - 1) * IN_con.dp_smooth ^ (1 / IN_con.exp) "Start of approximation for decreasing volume flow rate";
            //Documentation
          algorithm
            DP := Km_internal / IN_con.R_s * rho_internal ^ (IN_con.exp - 1) * Modelica.Fluid.Dissipation.Utilities.Functions.General.SmoothPower(V_flow, V_flow_min, IN_con.exp);
            annotation(
              Inline = true,
              smoothOrder(normallyConstant = IN_con) = 2,
              inverse(m_flow = Modelica.Fluid.Dissipation.PressureLoss.General.dp_idealGas_MFLOW(IN_con, IN_var, DP)),
              Documentation(info = "<html>
<p>
Calculation of a generic pressure loss for an <b> ideal gas </b> using mean density.
</p>

<p>
Generally this  function is numerically best used for the <b> incompressible case </b>, where the mass flow rate (m_flow) is known (as state variable) in the used model and the corresponding pressure loss (DP) has to be calculated. On the other hand the  function <a href=\"modelica://Modelica.Fluid.Dissipation.PressureLoss.General.dp_idealGas_MFLOW\">dp_idealGas_MFLOW</a> is numerically best used for the <b> compressible case </b> if the pressure loss (dp) is known (out of pressures as state variable) and the mass flow rate (M_FLOW) has to be calculated.  <a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.PressureLoss.General.dp_idealGas\">See more information</a>.
</p>
</html>"));
          end dp_idealGas_DP;

          function dp_idealGas_MFLOW "Generic pressure loss | calculate mass flow rate | ideal gas | mean density"
            extends Modelica.Icons.Function;
            import FD = Modelica.Fluid.Dissipation.PressureLoss.General;
            //input records
            input Modelica.Fluid.Dissipation.PressureLoss.General.dp_idealGas_IN_con IN_con "Input record for function dp_idealGas_MFLOW" annotation(
              Dialog(group = "Constant inputs"));
            input Modelica.Fluid.Dissipation.PressureLoss.General.dp_idealGas_IN_var IN_var "Input record for function dp_idealGas_MFLOW" annotation(
              Dialog(group = "Variable inputs"));
            input SI.Pressure dp "Pressure loss" annotation(
              Dialog(group = "Input"));
            //output variables
            output SI.MassFlowRate M_FLOW "Output for function dp_idealGas_MFLOW";
          protected
            Real Km_internal = IN_con.Km "Coefficient for pressure loss";
            SI.Density rho_internal = IN_var.p_m / (IN_con.R_s * IN_var.T_m) "Mean density";
            //Documentation
          algorithm
            M_FLOW := (IN_con.R_s / Km_internal) ^ (1 / IN_con.exp) * rho_internal ^ (1 / IN_con.exp) * Dissipation.Utilities.Functions.General.SmoothPower(dp, IN_con.dp_smooth, 1 / IN_con.exp);
            annotation(
              Inline = true,
              smoothOrder(normallyConstant = IN_con) = 2,
              inverse(dp = Modelica.Fluid.Dissipation.PressureLoss.General.dp_idealGas_DP(IN_con, IN_var, M_FLOW)),
              Documentation(info = "<html>
<p>
Calculation of a generic pressure loss for an <b> ideal gas </b> using mean density.
</p>

<p>
Generally this  function is numerically best used for the <b> compressible case </b> if the pressure loss (dp) is known (out of pressures as state variable) and the mass flow rate (M_FLOW) has to be calculated. On the other hand the  function <a href=\"modelica://Modelica.Fluid.Dissipation.PressureLoss.General.dp_idealGas_DP\">dp_idealGas_DP</a> is numerically best used for the <b> incompressible case </b>, where the mass flow rate (m_flow) is known (as state variable) in the used model and the corresponding pressure loss (DP) has to be calculated.  <a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.PressureLoss.General.dp_idealGas\">See more information</a>.
</p>
</html>"));
          end dp_idealGas_MFLOW;

          record dp_idealGas_IN_con "Input record for function dp_idealGas_DP and dp_idealGas_MFLOW"
            //generic variables
            extends Modelica.Fluid.Dissipation.Utilities.Records.General.IdealGas_con;
            //linearisation
            SI.Pressure dp_smooth(min = Modelica.Constants.eps) = 1 "Start linearisation for smaller pressure loss" annotation(
              Dialog(group = "Linearisation"));
            annotation(
              Documentation(info = "<html>
This record is used as <b> input record </b> for the pressure loss function
<a href=\"modelica://Modelica.Fluid.Dissipation.PressureLoss.General.dp_idealGas_DP\"> dp_idealGas_DP </a> and
<a href=\"modelica://Modelica.Fluid.Dissipation.PressureLoss.General.dp_idealGas_MFLOW\"> dp_idealGas_MFLOW </a>.
</html>"));
          end dp_idealGas_IN_con;

          record dp_idealGas_IN_var "Input record for function dp_idealGas_DP and dp_idealGas_MFLOW"
            //generic variables
            extends Modelica.Fluid.Dissipation.Utilities.Records.General.IdealGas_var;
            annotation(
              Documentation(info = "<html>
This record is used as <b> input record </b> for the pressure loss function
<a href=\"modelica://Modelica.Fluid.Dissipation.PressureLoss.General.dp_idealGas_DP\"> dp_idealGas_DP</a> and
<a href=\"modelica://Modelica.Fluid.Dissipation.PressureLoss.General.dp_idealGas_MFLOW\"> dp_idealGas_MFLOW</a>.
</html>"));
          end dp_idealGas_IN_var;

          function dp_nominalDensityViscosity_DP "Generic pressure loss | calculate mass flow rate | nominal operation point | pressure loss law (exponent) | density and dynamic viscosity dependence"
            extends Modelica.Icons.Function;
            import FD = Modelica.Fluid.Dissipation.PressureLoss.General;
            //input records
            input Modelica.Fluid.Dissipation.PressureLoss.General.dp_nominalDensityViscosity_IN_con IN_con "Input record for function dp_nominalDensityViscosity_DP" annotation(
              Dialog(group = "Constant inputs"));
            input Modelica.Fluid.Dissipation.PressureLoss.General.dp_nominalDensityViscosity_IN_var IN_var "Input record for function dp_nominalDensityViscosity_DP" annotation(
              Dialog(group = "Variable inputs"));
            input SI.MassFlowRate m_flow "Mass flow rate" annotation(
              Dialog(group = "Input"));
            //output variables
            output SI.Pressure DP "Output for function dp_nominalDensityViscosity_DP";
          protected
            SI.MassFlowRate m_flow_smooth = (max(1, 0.01 * IN_con.dp_nom) * IN_var.rho / IN_con.rho_nom * (1 / IN_var.eta * IN_con.eta_nom) ^ IN_con.exp_eta * (1 / IN_con.m_flow_nom)) ^ (1 / IN_con.exp) "Start of approximation for decreasing mass flow rate";
            //Documentation
          algorithm
            DP := if IN_con.exp > 1.0 or IN_con.exp < 1.0 then Modelica.Fluid.Dissipation.Utilities.Functions.General.SmoothPower(m_flow, m_flow_smooth, IN_con.exp) * (IN_var.eta / IN_con.eta_nom) ^ IN_con.exp_eta * IN_con.rho_nom / IN_var.rho * IN_con.dp_nom * (1 / IN_con.m_flow_nom) ^ IN_con.exp else m_flow / IN_con.m_flow_nom * (IN_var.eta / IN_con.eta_nom) ^ IN_con.exp_eta * IN_con.rho_nom / IN_var.rho * IN_con.dp_nom;
            annotation(
              Inline = true,
              smoothOrder(normallyConstant = IN_con) = 2,
              inverse(m_flow = Modelica.Fluid.Dissipation.PressureLoss.General.dp_nominalDensityViscosity_MFLOW(IN_con, IN_var, DP)),
              Documentation(info = "<html>
<p>
Calculation of a generic pressure loss in dependence of nominal fluid variables (e.g., nominal density, nominal dynamic viscosity) at an operation point via interpolation.
This generic function considers the pressure loss law via a pressure loss exponent and the influence of density and dynamic viscosity on pressure loss.
</p>

<p>
Generally this function is numerically best used for the <b> incompressible case </b>, where the mass flow rate (m_flow) is known (as state variable) in the used model and the corresponding pressure loss (DP) has to be calculated. On the other hand the function <a href=\"modelica://Modelica.Fluid.Dissipation.PressureLoss.General.dp_nominalDensityViscosity_MFLOW\">dp_nominalDensityViscosity_MFLOW</a> is numerically best used for the <b> compressible case </b> if the pressure loss (dp) is known (out of pressures as state variable) and the mass flow rate (M_FLOW) has to be calculated.  <a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.PressureLoss.General.dp_nominalDensityViscosity\">See more information</a>.
</p>
</html>"));
          end dp_nominalDensityViscosity_DP;

          function dp_nominalDensityViscosity_MFLOW "Generic pressure loss | calculate M_FLOW (compressible) | nominal operation point | pressure loss law (exponent) | density and dynamic viscosity dependence"
            extends Modelica.Icons.Function;
            import FD = Modelica.Fluid.Dissipation.PressureLoss.General;
            //input records
            input Modelica.Fluid.Dissipation.PressureLoss.General.dp_nominalDensityViscosity_IN_con IN_con "Input record for function dp_nominalDensityViscosity_MFLOW" annotation(
              Dialog(group = "Constant inputs"));
            input Modelica.Fluid.Dissipation.PressureLoss.General.dp_nominalDensityViscosity_IN_var IN_var "Input record for function dp_nominalDensityViscosity_MFLOW" annotation(
              Dialog(group = "Variable inputs"));
            input SI.Pressure dp "Pressure loss" annotation(
              Dialog(group = "Input"));
            //output variables
            output SI.MassFlowRate M_FLOW "Output for function dp_nominalDensityViscosity_MFLOW";
            //Documentation
          algorithm
            M_FLOW := if IN_con.exp > 1.0 or IN_con.exp < 1.0 then Modelica.Fluid.Dissipation.Utilities.Functions.General.SmoothPower(dp, 0.01 * IN_con.dp_nom, 1 / IN_con.exp) * (IN_con.eta_nom / IN_var.eta) ^ (IN_con.exp_eta / IN_con.exp) * (1 / IN_con.dp_nom * IN_var.rho / IN_con.rho_nom) ^ (1 / IN_con.exp) * IN_con.m_flow_nom else dp / IN_con.dp_nom * (IN_con.eta_nom / IN_var.eta) ^ IN_con.exp_eta * IN_var.rho / IN_con.rho_nom * IN_con.m_flow_nom;
            annotation(
              Inline = true,
              smoothOrder(normallyConstant = IN_con) = 2,
              inverse(dp = Modelica.Fluid.Dissipation.PressureLoss.General.dp_nominalDensityViscosity_DP(IN_con, IN_var, M_FLOW)),
              Documentation(info = "<html>
<p>
Calculation of a generic pressure loss in dependence of nominal fluid variables (e.g., nominal density, nominal dynamic viscosity) at an operation point via interpolation.
This generic function considers the pressure loss law via a pressure loss exponent and the influence of density and dynamic viscosity on pressure loss.
</p>

<p>
Generally this function is numerically best used for the <b> compressible case </b> if the pressure loss (dp) is known (out of pressures as state variable) and the mass flow rate (M_FLOW) has to be calculated. On the other hand the function <a href=\"modelica://Modelica.Fluid.Dissipation.PressureLoss.General.dp_nominalDensityViscosity_DP\">dp_genericDensityViscosity_DP</a> is numerically best used for the <b> incompressible case </b>, where the mass flow rate (m_flow) is known (as state variable) in the used model and the corresponding pressure loss (DP) has to be calculated. <a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.PressureLoss.General.dp_nominalDensityViscosity\">See more information</a>.
</p>
</html>"));
          end dp_nominalDensityViscosity_MFLOW;

          record dp_nominalDensityViscosity_IN_con "Output record for function dp_nominalDensityViscosity_DP and dp_nominalDensityViscosity_MFLOW"
            //generic variables
            extends Modelica.Fluid.Dissipation.Utilities.Records.General.NominalDensityViscosity;
            annotation(
              Documentation(info = "<html>
This record is used as <b> input record </b> for the pressure loss function
<a href=\"modelica://Modelica.Fluid.Dissipation.PressureLoss.General.dp_nominalDensityViscosity_DP\"> dp_nominalDensityViscosity_DP</a> and
<a href=\"modelica://Modelica.Fluid.Dissipation.PressureLoss.General.dp_nominalDensityViscosity_MFLOW\"> dp_nominalDensityViscosity_MFLOW</a>.
</html>"));
          end dp_nominalDensityViscosity_IN_con;

          record dp_nominalDensityViscosity_IN_var "Output record for function dp_nominalDensityViscosity_DP and dp_nominalDensityViscosity_MFLOW"
            //fluid property variables
            extends Modelica.Fluid.Dissipation.Utilities.Records.General.PressureLoss;
            annotation(
              Documentation(info = "<html>
This record is used as <b> input record </b> for the pressure loss function
<a href=\"modelica://Modelica.Fluid.Dissipation.PressureLoss.General.dp_nominalDensityViscosity_DP\"> dp_nominalDensityViscosity_DP</a> and
<a href=\"modelica://Modelica.Fluid.Dissipation.PressureLoss.General.dp_nominalDensityViscosity_MFLOW\"> dp_nominalDensityViscosity_MFLOW</a>.
</html>"));
          end dp_nominalDensityViscosity_IN_var;

          function dp_nominalPressureLossLawDensity_DP "Generic pressure loss | calculate pressure loss | nominal operation point | pressure loss law (coefficient and exponent) | density dependence"
            extends Modelica.Icons.Function;
            import FD = Modelica.Fluid.Dissipation.PressureLoss.General;
            //input records
            input Modelica.Fluid.Dissipation.PressureLoss.General.dp_nominalPressureLossLawDensity_IN_con IN_con "Input record for function dp_nominalPressureLossLawDensity_DP" annotation(
              Dialog(group = "Constant inputs"));
            input Modelica.Fluid.Dissipation.PressureLoss.General.dp_nominalPressureLossLawDensity_IN_var IN_var "Input record for function dp_nominalPressureLossLawDensity_DP" annotation(
              Dialog(group = "Variable inputs"));
            input SI.MassFlowRate m_flow "Mass flow rate" annotation(
              Dialog(group = "Input"));
            //output variables
            output SI.Pressure DP "Output for function dp_nominalPressureLossLawDensity_yesAJac_DP";
          protected
            Real exp_density = if IN_con.target == Dissipation.Utilities.Types.MassOrVolumeFlowRate.MassFlowRate then 1 - IN_con.exp else 1 "Exponent of density fraction (rho/rho_nom)";
            SI.MassFlowRate m_flow_nom = if IN_con.target == Dissipation.Utilities.Types.MassOrVolumeFlowRate.MassFlowRate then IN_con.m_flow_nom else IN_var.rho * IN_con.V_flow_nom "Nominal mean flow velocity at operation point";
            SI.MassFlowRate m_flow_linear = (0.01 * (IN_con.zeta_TOT_nom / IN_var.zeta_TOT) * (IN_con.rho_nom / IN_var.rho) ^ exp_density * (IN_con.A_cross / IN_con.A_cross_nom) ^ IN_con.exp * IN_con.m_flow_nom) ^ (1 / IN_con.exp) "Start of approximation for decreasing mass flow rate";
            //Documentation
          algorithm
            DP := if IN_con.exp > 1.0 or IN_con.exp < 1.0 then Modelica.Fluid.Dissipation.Utilities.Functions.General.SmoothPower(m_flow, m_flow_linear, IN_con.exp) * IN_con.dp_nom * (IN_var.zeta_TOT / IN_con.zeta_TOT_nom) * (IN_var.rho / IN_con.rho_nom) ^ exp_density * (IN_con.A_cross_nom / IN_con.A_cross) ^ IN_con.exp * (1 / IN_con.m_flow_nom) ^ IN_con.exp else IN_con.dp_nom * (IN_var.zeta_TOT / IN_con.zeta_TOT_nom) * (IN_var.rho / IN_con.rho_nom) ^ exp_density * (IN_con.A_cross_nom / IN_con.A_cross) ^ 1 * (m_flow / IN_con.m_flow_nom) ^ 1;
            annotation(
              Inline = false,
              smoothOrder(normallyConstant = IN_con) = 2,
              inverse(m_flow = Modelica.Fluid.Dissipation.PressureLoss.General.dp_nominalPressureLossLawDensity_MFLOW(IN_con, IN_var, DP)),
              Documentation(info = "<html>
<p>
Calculation of a generic pressure loss in dependence of nominal fluid variables (e.g., nominal density) via interpolation from an operation point.
This generic function considers the pressure loss law via a nominal pressure loss (dp_nom), a pressure loss coefficient (zeta_TOT) and a pressure loss law exponent (exp) as well as the influence of density on pressure loss.
</p>

<p>
Generally this function is numerically best used for the <b> incompressible case </b>, where the mass flow rate (m_flow) is known (as state variable) in the used model and the corresponding pressure loss (DP) has to be calculated. On the other hand the function <a href=\"modelica://Modelica.Fluid.Dissipation.PressureLoss.General.dp_nominalPressureLossLawDensity_MFLOW\">dp_nominalPressureLossLawDensity_MFLOW</a> is numerically best used for the <b> compressible case </b> if the pressure loss (dp) is known (out of pressures as state variable) and the mass flow rate (M_FLOW) has to be calculated. <a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.PressureLoss.General.dp_nominalPressureLossLawDensity\">See more information</a>.
</p>
</html>"));
          end dp_nominalPressureLossLawDensity_DP;

          function dp_nominalPressureLossLawDensity_MFLOW "Generic pressure loss | calculate mass flow rate | nominal operation point | pressure loss law (coefficient and exponent) | density dependence"
            extends Modelica.Icons.Function;
            import FD = Modelica.Fluid.Dissipation.PressureLoss.General;
            //input records
            input Modelica.Fluid.Dissipation.PressureLoss.General.dp_nominalPressureLossLawDensity_IN_con IN_con "Input record for function dp_nominalPressureLossLawDensity_MFLOW" annotation(
              Dialog(group = "Constant inputs"));
            input Modelica.Fluid.Dissipation.PressureLoss.General.dp_nominalPressureLossLawDensity_IN_var IN_var "Input record for function dp_nominalPressureLossLawDensity_MFLOW" annotation(
              Dialog(group = "Variable inputs"));
            input SI.Pressure dp "Pressure loss" annotation(
              Dialog(group = "Input"));
            //output variables
            output SI.MassFlowRate M_FLOW "Output for function dp_nominalPressurelosslawDensity_MFLOW";
          protected
            Real exp_density = if IN_con.target == Dissipation.Utilities.Types.MassOrVolumeFlowRate.MassFlowRate then 1 - IN_con.exp else 1 "Exponent of density fraction (rho/rho_nom)";
            SI.MassFlowRate m_flow_nom = if IN_con.target == Dissipation.Utilities.Types.MassOrVolumeFlowRate.MassFlowRate then IN_con.m_flow_nom else IN_var.rho * IN_con.V_flow_nom "Nominal mean flow velocity at operation point";
            //Documentation
          algorithm
            M_FLOW := if IN_con.exp > 1.0 or IN_con.exp < 1.0 then Modelica.Fluid.Dissipation.Utilities.Functions.General.SmoothPower(dp, 0.01 * IN_con.dp_nom, 1 / IN_con.exp) * IN_con.m_flow_nom * (IN_con.A_cross / IN_con.A_cross_nom) * (IN_con.rho_nom / IN_var.rho) ^ (exp_density / IN_con.exp) * (1 / IN_con.dp_nom * (IN_con.zeta_TOT_nom / IN_var.zeta_TOT)) ^ (1 / IN_con.exp) else IN_con.m_flow_nom * (IN_con.A_cross / IN_con.A_cross_nom) * (IN_con.rho_nom / IN_var.rho) ^ (exp_density / 1) * (dp / IN_con.dp_nom * (IN_con.zeta_TOT_nom / IN_var.zeta_TOT)) ^ (1 / 1);
            annotation(
              Inline = false,
              smoothOrder(normallyConstant = IN_con) = 2,
              inverse(dp = Modelica.Fluid.Dissipation.PressureLoss.General.dp_nominalPressureLossLawDensity_DP(IN_con, IN_var, M_FLOW)),
              Documentation(info = "<html>
<p>
Calculation of a generic pressure loss in dependence of nominal fluid variables (e.g., nominal density) via interpolation from an operation point.
This generic function considers the pressure loss law via a nominal pressure loss (dp_nom), a pressure loss coefficient (zeta_TOT) and a pressure loss law exponent (exp) as well as the influence of density on pressure loss.
</p>

<p>
Generally this function is numerically best used for the <b> compressible case </b> if the pressure loss (dp) is known (out of pressures as state variable) and the mass flow rate (M_FLOW) has to be calculated. On the other hand the function <a href=\"modelica://Modelica.Fluid.Dissipation.PressureLoss.General.dp_nominalPressureLossLawDensity_DP\">dp_nominalPressurelosslawDensity_DP</a> is numerically best used for the <b> incompressible case </b>, where the mass flow rate (m_flow) is known (as state variable) in the used model and the corresponding pressure loss (DP) has to be calculated. <a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.PressureLoss.General.dp_nominalPressureLossLawDensity\">See more information</a>.
</p>
</html>"));
          end dp_nominalPressureLossLawDensity_MFLOW;

          record dp_nominalPressureLossLawDensity_IN_con "Input record for function dp_nominalPressureLossLawDensity_DP and dp_nominalPressureLossLawDensity_MFLOW"
            //generic variables
            extends Modelica.Fluid.Dissipation.Utilities.Records.General.NominalPressureLossLawDensity_con;
            annotation(
              Documentation(info = "<html>
This record is used as <b> input record </b> for the pressure loss function
<a href=\"modelica://Modelica.Fluid.Dissipation.PressureLoss.General.dp_nominalPressureLossLawDensity_DP\"> dp_nominalPressureLosslawDensity_DP</a> and
<a href=\"modelica://Modelica.Fluid.Dissipation.PressureLoss.General.dp_nominalPressureLossLawDensity_MFLOW\"> dp_nominalPressureLosslawDensity_MFLOW</a>.
</html>"));
          end dp_nominalPressureLossLawDensity_IN_con;

          record dp_nominalPressureLossLawDensity_IN_var "Input record for function dp_nominalPressureLossLawDensity_DP and dp_nominalPressureLossLawDensity_MFLOW"
            //generic variables
            extends Modelica.Fluid.Dissipation.Utilities.Records.General.NominalPressureLossLawDensity_var;
            //fluid property variables
            SI.Density rho "Density of fluid" annotation(
              Dialog(group = "Fluid properties"));
            annotation(
              Documentation(info = "<html>
This record is used as <b> input record </b> for the pressure loss function
<a href=\"modelica://Modelica.Fluid.Dissipation.PressureLoss.General.dp_nominalPressureLossLawDensity_DP\"> dp_nominalPressureLosslawDensity_DP</a> and
<a href=\"modelica://Modelica.Fluid.Dissipation.PressureLoss.General.dp_nominalPressureLossLawDensity_MFLOW\"> dp_nominalPressureLosslawDensity_MFLOW</a>.
</html>"));
          end dp_nominalPressureLossLawDensity_IN_var;

          function dp_pressureLossCoefficient_DP "Generic pressure loss | calculate pressure loss | pressure loss coefficient (zeta_TOT)"
            extends Modelica.Icons.Function;
            import FD = Modelica.Fluid.Dissipation.PressureLoss.General;
            //input records
            input Modelica.Fluid.Dissipation.PressureLoss.General.dp_pressureLossCoefficient_IN_con IN_con "Input record for dp_pressureLossCoefficient_DP" annotation(
              Dialog(group = "Constant inputs"));
            input Modelica.Fluid.Dissipation.PressureLoss.General.dp_pressureLossCoefficient_IN_var IN_var "Input record for dp_pressureLossCoefficient_DP" annotation(
              Dialog(group = "Variable inputs"));
            input SI.MassFlowRate m_flow "Mass flow rate" annotation(
              Dialog(group = "Input"));
            //output variables
            output SI.Pressure DP "Output for function dp_pressureLossCoefficient_DP";
            //Documentation
          algorithm
            DP := 0.5 * IN_var.zeta_TOT * Modelica.Fluid.Dissipation.Utilities.Functions.General.SmoothPower(m_flow, (IN_con.dp_smooth / (0.5 * IN_var.zeta_TOT * IN_var.rho)) ^ 0.5 * IN_var.rho * IN_con.A_cross, 2) / (IN_var.rho * IN_con.A_cross ^ 2);
            annotation(
              Inline = true,
              smoothOrder(normallyConstant = IN_con) = 2,
              inverse(m_flow = Modelica.Fluid.Dissipation.PressureLoss.General.dp_pressureLossCoefficient_MFLOW(IN_con, IN_var, DP)),
              Documentation(info = "<html>
<p>
Calculation of a generic pressure loss in dependence of a pressure loss coefficient.
</p>

<p>
Generally this function is numerically best used for the <b> incompressible case </b>, where the mass flow rate (m_flow) is known (as state variable) in the used model and the corresponding pressure loss (DP) has to be calculated. On the other hand the  function <a href=\"modelica://Modelica.Fluid.Dissipation.PressureLoss.General.dp_pressureLossCoefficient_MFLOW\">dp_pressureLossCoefficient_MFLOW</a> is numerically best used for the <b> compressible case </b> if the pressure loss (dp) is known (out of pressures as state variable) and the mass flow rate (M_FLOW) has to be calculated. <a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.PressureLoss.General.dp_pressureLossCoefficient\">See more information</a>.
</p>
</html>"));
          end dp_pressureLossCoefficient_DP;

          function dp_pressureLossCoefficient_MFLOW "Generic pressure loss | calculate mass flow rate | pressure loss coefficient (zeta_TOT)"
            extends Modelica.Icons.Function;
            import FD = Modelica.Fluid.Dissipation.PressureLoss.General;
            //input records
            input Modelica.Fluid.Dissipation.PressureLoss.General.dp_pressureLossCoefficient_IN_con IN_con "Input record for function dp_pressureLossCoefficient_MFLOW" annotation(
              Dialog(group = "Constant inputs"));
            input Modelica.Fluid.Dissipation.PressureLoss.General.dp_pressureLossCoefficient_IN_var IN_var "Input record for function dp_pressureLossCoefficient_MFLOW" annotation(
              Dialog(group = "Variable inputs"));
            input SI.Pressure dp "Pressure loss" annotation(
              Dialog(group = "Input"));
            //output variables
            output SI.MassFlowRate M_FLOW "Output for function dp_pressureLossCoefficientt_MFLOW";
            //Documentation
          algorithm
            M_FLOW := IN_var.rho * IN_con.A_cross * Modelica.Fluid.Dissipation.Utilities.Functions.General.SmoothPower(dp, IN_con.dp_smooth, 0.5) / (0.5 * IN_var.zeta_TOT * IN_var.rho) ^ 0.5;
            annotation(
              Inline = true,
              smoothOrder(normallyConstant = IN_con) = 2,
              inverse(dp = Modelica.Fluid.Dissipation.PressureLoss.General.dp_pressureLossCoefficient_DP(IN_con, IN_var, M_FLOW)),
              Documentation(info = "<html>
<p>
Calculation of a generic pressure loss in dependence of  a pressure loss coefficient.
</p>

<p>
Generally this  function is numerically best used for the <b> compressible case </b> if the pressure loss (dp) is known (out of pressures as state variable) and the mass flow rate (M_FLOW) has to be calculated. On the other hand the  function <a href=\"modelica://Modelica.Fluid.Dissipation.PressureLoss.General.dp_pressureLossCoefficient_DP\">dp_pressureLossCoefficient_DP</a> is numerically best used for the <b> incompressible case </b>, where the mass flow rate (m_flow) is known (as state variable) in the used model and the corresponding pressure loss (DP) has to be calculated. <a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.PressureLoss.General.dp_pressureLossCoefficient\">See more information</a>.
</p>
</html>"));
          end dp_pressureLossCoefficient_MFLOW;

          record dp_pressureLossCoefficient_IN_con "Input record for function dp_pressureLossCoefficient_DP and dp_pressureLossCoefficient_MFLOW"
            extends Modelica.Icons.Record;
            //generic variables
            SI.Area A_cross = Modelica.Constants.pi * 0.1 ^ 2 / 4 "Cross sectional area" annotation(
              Dialog(group = "Generic variables"));
            //linearisation
            SI.Pressure dp_smooth = 1 "Start linearisation for decreasing pressure loss" annotation(
              Dialog(group = "Linearisation"));
            annotation(
              Documentation(info = "<html>
This record is used as <b> input record </b> for the pressure loss function
<a href=\"modelica://Modelica.Fluid.Dissipation.PressureLoss.General.dp_pressureLossCoefficient_DP\"> dp_pressureLossCoefficient_DP</a> and
<a href=\"modelica://Modelica.Fluid.Dissipation.PressureLoss.General.dp_pressureLossCoefficient_MFLOW\"> dp_pressureLossCoefficient_MFLOW</a>.
</html>"));
          end dp_pressureLossCoefficient_IN_con;

          record dp_pressureLossCoefficient_IN_var "Input record for function dp_pressureLossCoefficient_DP and dp_pressureLossCoefficient_MFLOW"
            extends Modelica.Icons.Record;
            //generic variables
            TYP.PressureLossCoefficient zeta_TOT = 0.02 * 1 / 0.1 "Pressure loss coefficient" annotation(
              Dialog(group = "Generic variables"));
            //fluid property variables
            SI.Density rho "Density of fluid" annotation(
              Dialog(group = "FluidProperties"));
            annotation(
              Documentation(info = "<html>
This record is used as <b> input record </b> for the pressure loss function
<a href=\"modelica://Modelica.Fluid.Dissipation.PressureLoss.General.dp_pressureLossCoefficient_DP\"> dp_pressureLossCoefficient_DP</a> and
<a href=\"modelica://Modelica.Fluid.Dissipation.PressureLoss.General.dp_pressureLossCoefficient_MFLOW\"> dp_pressureLossCoefficient_MFLOW</a>.
</html>"));
          end dp_pressureLossCoefficient_IN_var;

          function dp_volumeFlowRate_DP "Generic pressure loss | calculate pressure loss | quadratic function (dp=a*V_flow^2 + b*V_flow)"
            extends Modelica.Icons.Function;
            import FD = Modelica.Fluid.Dissipation.PressureLoss.General;
            //input records
            input Modelica.Fluid.Dissipation.PressureLoss.General.dp_volumeFlowRate_IN_con IN_con "Input record for function dp_volumeFlowRate_DP" annotation(
              Dialog(group = "Constant inputs"));
            input Modelica.Fluid.Dissipation.PressureLoss.General.dp_volumeFlowRate_IN_var IN_var "Input record for function dp_volumeFlowRate_DP" annotation(
              Dialog(group = "Variable inputs"));
            input SI.MassFlowRate m_flow "Mass flow rate" annotation(
              Dialog(group = "Input"));
            //output variables
            output SI.Pressure DP "Output for function dp_volumeFlowRate_DP";
          protected
            Real a = max(Modelica.Constants.eps, abs(IN_con.a));
            Real b = max(Modelica.Constants.eps, abs(IN_con.b));
            SI.VolumeFlowRate V_flow = m_flow / max(Modelica.Constants.eps, IN_var.rho) "Volume flow rate";
            SI.Pressure dp_min = IN_con.dp_min "Start of approximation for decreasing pressure loss";
            SI.VolumeFlowRate V_flow_smooth = if IN_con.a > 0 then -(b / (2 * a) + ((-b / (2 * a)) ^ 2 + dp_min / a) ^ 0.5) else dp_min / b "Start of approximation for decreasing volume flow rate";
            //Documentation
          algorithm
            DP := a * Dissipation.Utilities.Functions.General.SmoothPower(V_flow, V_flow_smooth, 2) + b * V_flow;
            annotation(
              Inline = false,
              smoothOrder(normallyConstant = IN_con) = 2,
              inverse(m_flow = Modelica.Fluid.Dissipation.PressureLoss.General.dp_volumeFlowRate_MFLOW(IN_con, IN_var, DP)),
              Documentation(info = "<html>
<p>
Calculation of a generic pressure loss with linear or quadratic dependence on volume flow rate.
The function can be used to calculate pressure loss at known mass flow rate <b> or </b> mass flow rate at known pressure loss.
</p>

<p>
Generally this  function is numerically best used for the <b> incompressible case </b>, where the mass flow rate (m_flow) is known (as state variable) in the used model and the corresponding pressure loss (DP) has to be calculated. On the other hand the  function <a href=\"modelica://Modelica.Fluid.Dissipation.PressureLoss.General.dp_volumeFlowRate_MFLOW\">dp_volumeFlowRate_MFLOW</a> is numerically best used for the <b> compressible case </b> if the pressure loss (dp) is known (out of pressures as state variable) and the mass flow rate (M_FLOW) has to be calculated. <a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.PressureLoss.General.dp_volumeFlowRate\">See more information</a>.
</p>
</html>"));
          end dp_volumeFlowRate_DP;

          function dp_volumeFlowRate_MFLOW "Generic pressure loss | calculate mass flow rate | quadratic function (dp=a*V_flow^2 + b*V_flow)"
            extends Modelica.Icons.Function;
            import FD = Modelica.Fluid.Dissipation.PressureLoss.General;
            //input records
            input Modelica.Fluid.Dissipation.PressureLoss.General.dp_volumeFlowRate_IN_con IN_con "Input record for function dp_volumeFlowRate_MFLOW" annotation(
              Dialog(group = "Constant inputs"));
            input Modelica.Fluid.Dissipation.PressureLoss.General.dp_volumeFlowRate_IN_var IN_var "Input record for function dp_volumeFlowRate_MFLOW" annotation(
              Dialog(group = "Variable inputs"));
            input SI.Pressure dp "Pressure loss" annotation(
              Dialog(group = "Input"));
            //output variables
            output SI.MassFlowRate M_FLOW "Output for function dp_volumeFlowRate_MFLOW";
          protected
            Real a = max(Modelica.Constants.eps, abs(IN_con.a));
            Real b = max(Modelica.Constants.eps, abs(IN_con.b));
            SI.Pressure dp_min = IN_con.dp_min "Start of approximation for decreasing pressure loss";
            //Documentation
          algorithm
            M_FLOW := IN_var.rho * ((-b / (2 * a)) + Modelica.Fluid.Dissipation.Utilities.Functions.General.SmoothPower((b / (2 * a)) ^ 2 + 1 / a * dp, (b / (2 * a)) ^ 2 + 1 / a * dp_min, 0.5));
            annotation(
              Inline = true,
              smoothOrder(normallyConstant = IN_con) = 2,
              inverse(dp = Modelica.Fluid.Dissipation.PressureLoss.General.dp_volumeFlowRate_DP(IN_con, IN_var, M_FLOW)),
              Documentation(info = "<html>
<p>
Calculation of a generic pressure loss with linear or quadratic dependence on volume flow rate.
The function can be used to calculate pressure loss at known mass flow rate <b> or </b> mass flow rate at known pressure loss.
</p>

<p>
Generally this  function is numerically best used for the <b> compressible case </b> if the pressure loss (dp) is known (out of pressures as state variable) and the mass flow rate (M_FLOW) has to be calculated. On the other hand the  function <a href=\"modelica://Modelica.Fluid.Dissipation.PressureLoss.General.dp_volumeFlowRate_DP\">dp_volumeFlowRate_DP</a> is numerically best used for the <b> incompressible case </b>, where the mass flow rate (m_flow) is known (as state variable) in the used model and the corresponding pressure loss (DP) has to be calculated. <a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.PressureLoss.General.dp_volumeFlowRate\">See more information</a>.
</p>
</html>"));
          end dp_volumeFlowRate_MFLOW;

          record dp_volumeFlowRate_IN_con "Input record for function dp_volumeFlowRate_DP and dp_volumeFlowRate_MFLOW"
            //generic variables
            extends Modelica.Fluid.Dissipation.Utilities.Records.General.QuadraticVFLOW;
            SI.Pressure dp_min = 0.1 "Start of approximation for decreasing pressure loss";
            annotation(
              Documentation(info = "<html>
This record is used as <b> input record </b> for the pressure loss function
 <a href=\"modelica://Modelica.Fluid.Dissipation.PressureLoss.General.dp_volumeFlowRate_DP\"> dp_volumeFlowRate_DP</a> and
 <a href=\"modelica://Modelica.Fluid.Dissipation.PressureLoss.General.dp_volumeFlowRate_MFLOW\"> dp_volumeFlowRate_MFLOW</a>.
</html>"));
          end dp_volumeFlowRate_IN_con;

          record dp_volumeFlowRate_IN_var "Input record for function dp_volumeFlowRate_DP and dp_volumeFlowRate_MFLOW"
            extends Modelica.Icons.Record;
            SI.Density rho "Density of fluid" annotation(
              Dialog(group = "Fluid properties"));
            annotation(
              Documentation(info = "<html>
This record is used as <b> input record </b> for the pressure loss function
 <a href=\"modelica://Modelica.Fluid.Dissipation.PressureLoss.General.dp_volumeFlowRate_DP\"> dp_volumeFlowRate_DP</a> and
 <a href=\"modelica://Modelica.Fluid.Dissipation.PressureLoss.General.dp_volumeFlowRate_MFLOW\"> dp_volumeFlowRate_MFLOW</a>.
</html>"));
          end dp_volumeFlowRate_IN_var;
          annotation(
            preferredView = "info",
            Documentation(info = "<html>
<h4>General</h4>
<h5>General pressure loss for ideal gas</h5>
<p>
Calculation of a generic pressure loss for an <b> ideal gas </b> using mean density.
<a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.PressureLoss.General.dp_idealGas\">See more information</a>.
</p>

<h5>Generic pressure loss depending on density and viscosity </h5>
<p>
Calculation of a generic pressure loss in dependence of nominal fluid variables (e.g., nominal density, nominal dynamic viscosity) at an operation point via interpolation.
This generic function considers the pressure loss law via a pressure loss exponent and the influence of density and dynamic viscosity on pressure loss.
<a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.PressureLoss.General.dp_nominalDensityViscosity\">See more information</a>.
</p>

<h5>Generic pressure loss depending on density</h5>
<p>
Calculation of a generic pressure loss in dependence of nominal fluid variables (e.g., nominal density) via interpolation from an operation point.
<a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.PressureLoss.General.dp_nominalPressureLossLawDensity\">See more information</a>.
</p>

<h5>Generic pressure loss depending on pressure loss coefficient</h5>
<p>
Calculation of a generic pressure loss in dependence of a pressure loss coefficient.
 <a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.PressureLoss.General.dp_pressureLossCoefficient\">See more information</a>.
</p>

<h5>Generic pressure loss depending on volume flow rate</h5>
<p>
Calculation of a generic pressure loss with linear or quadratic dependence on volume flow rate.
 <a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.PressureLoss.General.dp_volumeFlowRate\">See more information</a>.
</p>
</html>"));
        end General;

        package Orifice "Package for pressure loss calculation of orifices"
          extends Modelica.Icons.VariantsPackage;

          function dp_suddenChange_DP "Pressure loss of orifice with sudden change in cross sectional area | calculate pressure loss | turbulent flow regime | smooth surface | arbitrary cross sectional area | without baffles | sharp edge"
            extends Modelica.Icons.Function;
            //SOURCE_1: Idelchik, I.E.: HANDBOOK OF HYDRAULIC RESISTANCE, 3rd edition, 2006.
            //Notation of equations according to SOURCES
            import FD = Modelica.Fluid.Dissipation.PressureLoss.Orifice;
            import SMOOTH = Modelica.Fluid.Dissipation.Utilities.Functions.General.Stepsmoother;
            //input records
            input Modelica.Fluid.Dissipation.PressureLoss.Orifice.dp_suddenChange_IN_con IN_con "Input record for function dp_suddenChange_DP" annotation(
              Dialog(group = "Constant inputs"));
            input Modelica.Fluid.Dissipation.PressureLoss.Orifice.dp_suddenChange_IN_var IN_var "Input record for function dp_suddenChange_DP" annotation(
              Dialog(group = "Variable inputs"));
            input SI.MassFlowRate m_flow "Mass flow rate" annotation(
              Dialog(group = "Input"));
            //output variables
            output SI.Pressure DP "Output for function dp_suddenChange_DP";
          protected
            Real MIN = Modelica.Constants.eps;
            SI.ReynoldsNumber Re_min = 1 "Minimum Reynolds number for linear smoothing";
            //restriction of local resistance coefficient zeta_LOC >> numerical improvement
            TYP.LocalResistanceCoefficient zeta_LOC_min = 1e-3 "Minimal local resistance coefficient";
            SI.Area A_1 = max(MIN, min(IN_con.A_1, IN_con.A_2)) "Small cross sectional area of orifice";
            SI.Area A_2 = max(MIN, max(IN_con.A_1, IN_con.A_2)) "Large cross sectional area of orifice";
            SI.Length C_1 = max(MIN, min(IN_con.C_1, IN_con.C_2)) "Perimeter of small cross sectional area of orifice";
            SI.Length C_2 = max(MIN, max(IN_con.C_1, IN_con.C_2)) "perimeter of large cross sectional area of orifice";
            SI.Diameter d_hyd = 4 * A_1 / C_1 "Hydraulic diameter of small cross sectional area of orifice";
            //sudden expansion  :  SOURCE_1, section 4, diagram 4-1, page 208
            //assumption of Re >= 3.3e3 for sudden expansion
            TYP.LocalResistanceCoefficient zeta_LOC_exp = max(zeta_LOC_min, (1 - A_1 / A_2) ^ 2);
            //sudden contraction:  SOURCE_1, section 4, diagram 4-9, page 216 / 217
            //assumption of Re >= 1.0e4 for sudden contraction
            TYP.LocalResistanceCoefficient zeta_LOC_con = max(zeta_LOC_min, 0.5 * (1 - A_1 / A_2) ^ 0.75);
            SI.Velocity velocity_1 = m_flow / (IN_var.rho * A_1) "Mean velocity in smaller cross sectional area";
            //determine Reynolds number for small cross sectional area of orifice
            SI.ReynoldsNumber Re = IN_var.rho * d_hyd * velocity_1 / IN_var.eta;
            //actual local resistance coefficient
            TYP.LocalResistanceCoefficient zeta_LOC = zeta_LOC_exp * SMOOTH(Re_min, 0, Re) + zeta_LOC_con * SMOOTH(-Re_min, 0, Re) + zeta_LOC_min * SMOOTH(0, Re_min, abs(Re));
            //Documentation
          algorithm
            DP := zeta_LOC * IN_var.rho / 2 * (IN_var.eta / IN_var.rho / d_hyd) ^ 2 * Modelica.Fluid.Dissipation.Utilities.Functions.General.SmoothPower(Re, Re_min, 2);
            annotation(
              Inline = false,
              smoothOrder(normallyConstant = IN_con) = 2,
              inverse(m_flow = Modelica.Fluid.Dissipation.PressureLoss.Orifice.dp_suddenChange_MFLOW(IN_con, IN_var, DP)),
              Documentation(info = "<html>
<p>
Calculation of the local pressure loss at a sudden change of the cross sectional areas (sudden expansion or sudden contraction) with sharp corners at turbulent flow regime for incompressible and single-phase fluid flow through arbitrary shaped cross sectional area (square, circular, etc.) considering a smooth surface. The flow direction determines the type of the transition. In case of the design flow a sudden expansion will be considered. At flow reversal a sudden contraction will be considered.
</p>

<p>
Generally this  function is numerically best used for the <b> incompressible case </b>, where the mass flow rate (m_flow) is known (as state variable) in the used model and the corresponding pressure loss (DP) has to be calculated. On the other hand the  function <a href=\"modelica://Modelica.Fluid.Dissipation.PressureLoss.Orifice.dp_suddenChange_MFLOW\">dp_suddenChange_MFLOW</a> is numerically best used for the <b> compressible case </b> if the pressure loss (dp) is known (out of pressures as state variable) and the mass flow rate (M_FLOW) has to be calculated.  <a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.PressureLoss.Orifice.dp_suddenChange\">See more information</a>.
</p>
</html>"));
          end dp_suddenChange_DP;

          function dp_suddenChange_MFLOW "Pressure loss of orifice with sudden change in cross sectional area | calculate mass flow rate | turbulent flow regime | smooth surface | arbitrary cross sectional area | without baffles | sharp edge"
            extends Modelica.Icons.Function;
            //SOURCE_1: Idelchik, I.E.: HANDBOOK OF HYDRAULIC RESISTANCE, 3rd edition, 2006.
            //Notation of equations according to SOURCES
            import FD = Modelica.Fluid.Dissipation.PressureLoss.Orifice;
            import SMOOTH = Modelica.Fluid.Dissipation.Utilities.Functions.General.Stepsmoother;
            //input records
            input Modelica.Fluid.Dissipation.PressureLoss.Orifice.dp_suddenChange_IN_con IN_con "Input record for function dp_suddenChange_MFLOW" annotation(
              Dialog(group = "Constant inputs"));
            input Modelica.Fluid.Dissipation.PressureLoss.Orifice.dp_suddenChange_IN_var IN_var "Input record for function dp_suddenChange_MFLOW" annotation(
              Dialog(group = "Variable inputs"));
            input SI.Pressure dp "Pressure loss" annotation(
              Dialog(group = "Input"));
            //output variables
            output SI.MassFlowRate M_FLOW "Output for function dp_suddenChange_MFLOW";
          protected
            Real MIN = Modelica.Constants.eps;
            SI.Pressure dp_min = 1 "Pressure loss for linear smoothing";
            //restriction of local resistance coefficient zeta_LOC >> numerical improvement
            TYP.LocalResistanceCoefficient zeta_LOC_min = 1e-3 "Minimal local resistance coefficient";
            SI.Area A_1 = max(MIN, min(IN_con.A_1, IN_con.A_2)) "Small cross sectional area of orifice";
            SI.Area A_2 = max(MIN, max(IN_con.A_1, IN_con.A_2)) "Large cross sectional area of orifice";
            //sudden expansion  :  SOURCE_1, section 4, diagram 4-1, page 208
            //assumption of Re >= 3.3e3 for sudden expansion
            TYP.LocalResistanceCoefficient zeta_LOC_exp = max(zeta_LOC_min, (1 - A_1 / A_2) ^ 2);
            //sudden contraction:  SOURCE_1, section 4, diagram 4-9, page 216 / 217
            //assumption of Re >= 1.0e4 for sudden contraction
            TYP.LocalResistanceCoefficient zeta_LOC_con = max(zeta_LOC_min, 0.5 * (1 - A_1 / A_2) ^ 0.75);
            //actual local resistance coefficient
            TYP.LocalResistanceCoefficient zeta_LOC = max(zeta_LOC_min, zeta_LOC_exp * SMOOTH(dp_min, 0, dp) + zeta_LOC_con * SMOOTH(-dp_min, 0, dp)) + zeta_LOC_min * SMOOTH(0, dp_min, abs(dp));
            //Documentation
          algorithm
            M_FLOW := IN_var.rho * A_1 * Modelica.Fluid.Dissipation.Utilities.Functions.General.SmoothPower(dp, dp_min, 0.5) * max(MIN, 2 / (IN_var.rho * zeta_LOC)) ^ 0.5;
            annotation(
              Inline = false,
              smoothOrder(normallyConstant = IN_con) = 2,
              inverse(dp = Modelica.Fluid.Dissipation.PressureLoss.Orifice.dp_suddenChange_DP(IN_con, IN_var, M_FLOW)),
              Documentation(info = "<html>
<p>
Calculation of the local pressure loss at a sudden change of the cross sectional areas (sudden expansion or sudden contraction) with sharp corners at turbulent flow regime for incompressible and single-phase fluid flow through arbitrary shaped cross sectional area (square, circular, etc.) considering a smooth surface. The flow direction determines the type of the transition. In case of the design flow a sudden expansion will be considered. At flow reversal a sudden contraction will be considered.
</p>

<p>
Generally this  function is numerically best used for the <b> compressible case </b> if the pressure loss (dp) is known (out of pressures as state variable) and the mass flow rate (M_FLOW) has to be calculated. On the other hand the  function <a href=\"modelica://Modelica.Fluid.Dissipation.PressureLoss.Orifice.dp_suddenChange_DP\">dp_suddenChange_DP</a> is numerically best used for the <b> incompressible case </b>, where the mass flow rate (m_flow) is known (as state variable) in the used model and the corresponding pressure loss (DP) has to be calculated.  <a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.PressureLoss.Orifice.dp_suddenChange\">See more information</a>.
</p>
</html>"));
          end dp_suddenChange_MFLOW;

          record dp_suddenChange_IN_con "Input record for function dp_suddenChange_DP and dp_suddenChange_MFLOW"
            //orifice variables
            extends Modelica.Fluid.Dissipation.Utilities.Records.PressureLoss.SuddenChange;
            annotation(
              Documentation(info = "<html>
This record is used as <b> input record </b> for the pressure loss functions
<a href=\"modelica://Modelica.Fluid.Dissipation.PressureLoss.Orifice.dp_suddenChange_DP\"> dp_suddenChange_DP</a> and
<a href=\"modelica://Modelica.Fluid.Dissipation.PressureLoss.Orifice.dp_suddenChange_MFLOW\"> dp_suddenChange_MFLOW</a>.
</html>"));
          end dp_suddenChange_IN_con;

          record dp_suddenChange_IN_var "Input record for function dp_suddenChange_DP and dp_suddenChange_MFLOW"
            //fluid property variables
            extends Modelica.Fluid.Dissipation.Utilities.Records.General.PressureLoss;
            annotation(
              Documentation(info = "<html>
This record is used as <b> input record </b> for the pressure loss functions
<a href=\"modelica://Modelica.Fluid.Dissipation.PressureLoss.Orifice.dp_suddenChange_DP\"> dp_suddenChange_DP</a> and
<a href=\"modelica://Modelica.Fluid.Dissipation.PressureLoss.Orifice.dp_suddenChange_MFLOW\"> dp_suddenChange_MFLOW</a>.
</html>"));
          end dp_suddenChange_IN_var;

          function dp_thickEdgedOverall_DP "Pressure loss of thick and sharp edged orifice | calculate pressure loss | overall flow regime | constant influence of friction  | arbitrary cross sectional area"
            extends Modelica.Icons.Function;
            //SOURCE_1: Idelchik, I.E.: HANDBOOK OF HYDRAULIC RESISTANCE, 3rd edition, 2006.
            //Notation of equations according to SOURCES
            import FD = Modelica.Fluid.Dissipation.PressureLoss.Orifice;
            import SMOOTH = Modelica.Fluid.Dissipation.Utilities.Functions.General.Stepsmoother;
            //input records
            input Modelica.Fluid.Dissipation.PressureLoss.Orifice.dp_thickEdgedOverall_IN_con IN_con "Input record for function dp_thickEdgedOverall_DP" annotation(
              Dialog(group = "Constant inputs"));
            input Modelica.Fluid.Dissipation.PressureLoss.Orifice.dp_thickEdgedOverall_IN_var IN_var "Input record for function dp_thickEdgedOverall_DP" annotation(
              Dialog(group = "Variable inputs"));
            input SI.MassFlowRate m_flow "Mass flow rate" annotation(
              Dialog(group = "Input"));
            //output variables
            output SI.Pressure DP "Output for function dp_thickEdgedOverall_DP";
          protected
            Real MIN = Modelica.Constants.eps;
            TYP.DarcyFrictionFactor lambda_FRI = 0.02 "Assumption for Darcy friction factor in vena contraction according to SOURCE_1";
            SI.ReynoldsNumber Re_min = 1;
            SI.ReynoldsNumber Re_lim = 1e3 "Limitation for laminar regime if dp is target";
            SI.Area A_0 = IN_con.A_0 "Cross sectional area of vena contraction";
            SI.Area A_1 = IN_con.A_1 "Cross sectional area of large cross sectional area";
            SI.Diameter d_hyd_0 = max(MIN, 4 * A_0 / IN_con.C_0) "Hydraulic diameter of vena contraction";
            SI.Diameter d_hyd_1 = max(MIN, 4 * A_1 / IN_con.C_1) "Hydraulic diameter of large cross sectional area";
            SI.Length l = IN_con.L "Length of vena contraction";
            Real l_bar = IN_con.L / d_hyd_0;
            //SOURCE_1, section 4, diagram 4-15, page 222:
            Real phi = 0.25 + 0.535 * min(l_bar, 2.4) ^ 8 / (0.05 + min(l_bar, 2.4) ^ 8);
            Real tau = max(2.4 - l_bar, 0) * 10 ^ (-phi);
            TYP.PressureLossCoefficient zeta_TOT_1 = max(MIN, (0.5 * (1 - A_0 / A_1) ^ 0.75 + tau * (1 - A_0 / A_1) ^ 1.375 + (1 - A_0 / A_1) ^ 2 + lambda_FRI * l / d_hyd_0) * (A_1 / A_0) ^ 2) "Pressure loss coefficient w.r.t. to flow velocity in large cross sectional area";
            SI.Velocity v_0 = m_flow / (IN_var.rho * A_0) "Mean velocity in vena contraction";
            SI.ReynoldsNumber Re = IN_var.rho * v_0 * d_hyd_0 / max(MIN, IN_var.eta) "Reynolds number in vena contraction";
            //Documentation
          algorithm
            DP := zeta_TOT_1 * IN_var.rho / 2 * (IN_var.eta / IN_var.rho / d_hyd_1) ^ 2 * Modelica.Fluid.Dissipation.Utilities.Functions.General.SmoothPower(Re, Re_min, 2) * (d_hyd_1 / d_hyd_0 * A_0 / A_1) ^ 2;
            annotation(
              Inline = false,
              smoothOrder(normallyConstant = IN_con) = 2,
              inverse(m_flow = Modelica.Fluid.Dissipation.PressureLoss.Orifice.dp_thickEdgedOverall_MFLOW(IN_con, IN_var, DP)),
              Documentation(info = "<html>
<p>
Calculation of pressure loss in thick edged orifices with sharp corners at overall flow regime for incompressible and single-phase fluid flow through an arbitrary shaped cross sectional area (square, circular, etc.) considering constant influence of surface roughness.
</p>

<p>
Generally this  function is numerically best used for the <b> incompressible case </b>, where the mass flow rate (m_flow) is known (as state variable) in the used model and the corresponding pressure loss (DP) has to be calculated. On the other hand the  function <a href=\"modelica://Modelica.Fluid.Dissipation.PressureLoss.Orifice.dp_thickEdgedOverall_MFLOW\">dp_thickEdgedOverall_MFLOW</a> is numerically best used for the <b> compressible case </b> if the pressure loss (dp) is known (out of pressures as state variable) and the mass flow rate (M_FLOW) has to be calculated.  <a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.PressureLoss.Orifice.dp_thickEdgedOverall\">See more information</a>.
</p>
</html>"));
          end dp_thickEdgedOverall_DP;

          function dp_thickEdgedOverall_MFLOW "Pressure loss of thick and sharp edged orifice | calculate mass flow rate | overall flow regime | constant influence of friction  | arbitrary cross sectional area"
            extends Modelica.Icons.Function;
            //SOURCE_1: Idelchik, I.E.: HANDBOOK OF HYDRAULIC RESISTANCE, 3rd edition, 2006.
            //Notation of equations according to SOURCES
            import FD = Modelica.Fluid.Dissipation.PressureLoss.Orifice;
            import SMOOTH = Modelica.Fluid.Dissipation.Utilities.Functions.General.Stepsmoother;
            //input records
            input Modelica.Fluid.Dissipation.PressureLoss.Orifice.dp_thickEdgedOverall_IN_con IN_con "Input record for function dp_thickEdgedOverall_MFLOW" annotation(
              Dialog(group = "Constant inputs"));
            input Modelica.Fluid.Dissipation.PressureLoss.Orifice.dp_thickEdgedOverall_IN_var IN_var "Input record for function dp_thickEdgedOverall_MFLOW" annotation(
              Dialog(group = "Variable inputs"));
            input SI.Pressure dp "Pressure loss" annotation(
              Dialog(group = "Input"));
            //output variables
            output SI.MassFlowRate M_FLOW "Output for function dp_thickEdgedOverall_MFLOW";
          protected
            Real MIN = Modelica.Constants.eps;
            TYP.DarcyFrictionFactor lambda_FRI = 0.02 "Assumption for Darcy friction factor in vena contraction according to SOURCE_1";
            SI.ReynoldsNumber Re_lim = 1e3 "Limitation for laminar regime if dp is target";
            SI.Area A_0 = IN_con.A_0 "Cross sectional area of vena contraction";
            SI.Area A_1 = IN_con.A_1 "Large cross sectional area";
            SI.Diameter d_hyd_0 = max(MIN, 4 * A_0 / IN_con.C_0) "Hydraulic diameter of vena contraction";
            SI.Diameter d_hyd_1 = max(MIN, 4 * A_1 / IN_con.C_1) "Hydraulic diameter of large cross sectional area";
            SI.Length l = IN_con.L "Length of vena contraction";
            Real l_bar = IN_con.L / d_hyd_0;
            //SOURCE_1, section 4, diagram 4-15, page 222:
            Real phi = 0.25 + 0.535 * min(l_bar, 2.4) ^ 8 / (0.05 + min(l_bar, 2.4) ^ 8);
            Real tau = max(2.4 - l_bar, 0) * 10 ^ (-phi);
            TYP.PressureLossCoefficient zeta_TOT_1 = max(MIN, (0.5 * (1 - A_0 / A_1) ^ 0.75 + tau * (1 - A_0 / A_1) ^ 1.375 + (1 - A_0 / A_1) ^ 2 + lambda_FRI * l / d_hyd_0) * (A_1 / A_0) ^ 2) "Pressure loss coefficient w.r.t. to flow velocity in large cross sectional area";
            //Documentation
          algorithm
            M_FLOW := IN_var.rho * A_1 * Modelica.Fluid.Dissipation.Utilities.Functions.General.SmoothPower(dp, IN_con.dp_smooth, 0.5) / (0.5 * IN_var.rho * zeta_TOT_1) ^ 0.5;
            annotation(
              Inline = false,
              smoothOrder(normallyConstant = IN_con) = 2,
              inverse(dp = Modelica.Fluid.Dissipation.PressureLoss.Orifice.dp_thickEdgedOverall_DP(IN_con, IN_var, M_FLOW)),
              Documentation(info = "<html>
<p>
Calculation of pressure loss in thick edged orifices with sharp corners at overall flow regime for incompressible and single-phase fluid flow through an arbitrary shaped cross sectional area (square, circular, etc.) considering constant influence of surface roughness.
</p>

<p>
Generally this  function is numerically best used for the <b> compressible case </b>, where the pressure loss (dp) is known (out of pressures as state variable) in the used model and the corresponding mass flow rate (M_FLOW) has to be calculated. On the other hand the  function <a href=\"modelica://Modelica.Fluid.Dissipation.PressureLoss.Orifice.dp_thickEdgedOverall_DP\">dp_thickEdgedOverall_DP</a> is numerically best used for the <b> incompressible case </b> if the mass flow rate (m_flow) is known (as state variable) and the pressure loss (DP) has to be calculated.  <a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.PressureLoss.Orifice.dp_thickEdgedOverall\">See more information</a>.
</p>
</html>"));
          end dp_thickEdgedOverall_MFLOW;

          record dp_thickEdgedOverall_IN_con "Input record for function dp_thickEdgedOverall_DP and dp_thickEdgedOverall_MFLOW"
            //orifice variables
            extends Modelica.Fluid.Dissipation.Utilities.Records.PressureLoss.Orifice;
            //linearisation
            SI.Pressure dp_smooth(min = Modelica.Constants.eps) = 1 "Start linearisation for decreasing pressure loss" annotation(
              Dialog(group = "Linearisation"));
            annotation(
              Documentation(info = "<html>
This record is used as <b> input record </b> for the pressure loss functions
<a href=\"modelica://Modelica.Fluid.Dissipation.PressureLoss.Orifice.dp_thickEdgedOverall_DP\"> dp_thickEdgedOverall_DP</a> and
<a href=\"modelica://Modelica.Fluid.Dissipation.PressureLoss.Orifice.dp_thickEdgedOverall_MFLOW\"> dp_thickEdgedOverall_MFLOW</a>.
</html>"));
          end dp_thickEdgedOverall_IN_con;

          record dp_thickEdgedOverall_IN_var "Input record for function dp_thickEdgedOverall_DP and dp_thickEdgedOverall_MFLOW"
            //fluid property variables
            extends Modelica.Fluid.Dissipation.Utilities.Records.General.PressureLoss;
            annotation(
              Documentation(info = "<html>
This record is used as <b> input record </b> for the pressure loss functions
<a href=\"modelica://Modelica.Fluid.Dissipation.PressureLoss.Orifice.dp_thickEdgedOverall_DP\"> dp_thickEdgedOverall_DP </a> and
<a href=\"modelica://Modelica.Fluid.Dissipation.PressureLoss.Orifice.dp_thickEdgedOverall_MFLOW\"> dp_thickEdgedOverall_MFLOW </a>.
</html>"));
          end dp_thickEdgedOverall_IN_var;
          annotation(
            preferredView = "info",
            Documentation(info = "<html>
<h4>Orifice</h4>
<h5>Sudden change</h5>
<p>
Calculation of the local pressure loss at a sudden change of the cross sectional areas (sudden expansion or sudden contraction) with sharp corners at turbulent flow regime for incompressible and single-phase fluid flow through arbitrary shaped cross sectional area (square, circular, etc.) considering a smooth surface.
<a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.PressureLoss.Orifice.dp_suddenChange\">See more information</a>.
</p>

<h5>Thick edged orifice</h5>
<p>
Calculation of pressure loss in thick edged orifices with sharp corners at overall flow regime for incompressible and single-phase fluid flow through an arbitrary shaped cross sectional area (square, circular, etc.) considering constant influence of surface roughness.
<a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.PressureLoss.Orifice.dp_thickEdgedOverall\">See more information</a>.
</p>
</html>"));
        end Orifice;

        package StraightPipe "Package for pressure loss calculation of straight pipes"
          extends Modelica.Icons.VariantsPackage;

          function dp_laminar_DP "Pressure loss of straight pipe | calculate pressure loss| laminar flow regime (Hagen-Poiseuille)"
            extends Modelica.Icons.Function;
            //SOURCE_1: Idelchik, I.E.: HANDBOOK OF HYDRAULIC RESISTANCE, 3rd edition, 2006.
            import FD = Modelica.Fluid.Dissipation.PressureLoss.StraightPipe;
            //input records
            input Modelica.Fluid.Dissipation.PressureLoss.StraightPipe.dp_laminar_IN_con IN_con "Input record for function dp_laminar_DP" annotation(
              Dialog(group = "Constant inputs"));
            input Modelica.Fluid.Dissipation.PressureLoss.StraightPipe.dp_laminar_IN_var IN_var "Input record for function dp_laminar_DP" annotation(
              Dialog(group = "Variable inputs"));
            input SI.MassFlowRate m_flow "Mass flow rate" annotation(
              Dialog(group = "Input"));
            //output variables
            output SI.Pressure DP "Output for function dp_laminar_DP";
          protected
            Real MIN = Modelica.Constants.eps;
            SI.Diameter d_hyd = IN_con.d_hyd "Hydraulic diameter";
            SI.Area A_cross = PI * IN_con.d_hyd ^ 2 / 4 "Circular cross sectional area";
            SI.Velocity velocity = m_flow / max(MIN, IN_var.rho * A_cross) "Mean velocity";
            //Documentation
          algorithm
            DP := 32 * IN_var.eta * velocity * IN_con.L / d_hyd ^ 2;
            annotation(
              Inline = true,
              smoothOrder(normallyConstant = IN_con) = 2,
              inverse(m_flow = Modelica.Fluid.Dissipation.PressureLoss.StraightPipe.dp_laminar_MFLOW(IN_con, IN_var, DP)),
              Documentation(info = "<html>
<p>
Calculation of pressure loss in a straight pipe for <b> laminar </b> flow regime of an incompressible and single-phase fluid flow only.
</p>

<p>
Generally this  function is numerically best used for the <b> incompressible case </b>, where the mass flow rate (m_flow) is known (as state variable) in the used model and the corresponding pressure loss (DP) has to be calculated. On the other hand the  function <a href=\"modelica://Modelica.Fluid.Dissipation.PressureLoss.StraightPipe.dp_laminar_MFLOW\">dp_laminar_MFLOW</a> is numerically best used for the <b> compressible case </b> if the pressure loss (dp) is known (out of pressures as state variable) and the mass flow rate (M_FLOW) has to be calculated.   <a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.PressureLoss.StraightPipe.dp_laminar\">See more information</a>.
</p>
</html>"));
          end dp_laminar_DP;

          function dp_laminar_MFLOW "Pressure loss of straight pipe | calculate mass flow rate | laminar flow regime (Hagen-Poiseuille)"
            extends Modelica.Icons.Function;
            //SOURCE_1: Idelchik, I.E.: HANDBOOK OF HYDRAULIC RESISTANCE, 3rd edition, 2006.
            import FD = Modelica.Fluid.Dissipation.PressureLoss.StraightPipe;
            //input records
            input Modelica.Fluid.Dissipation.PressureLoss.StraightPipe.dp_laminar_IN_con IN_con "Input record for function dp_laminar_MFLOW" annotation(
              Dialog(group = "Constant inputs"));
            input Modelica.Fluid.Dissipation.PressureLoss.StraightPipe.dp_laminar_IN_var IN_var "Input record for function dp_laminar_MFLOW" annotation(
              Dialog(group = "Variable inputs"));
            input SI.Pressure dp "Pressure loss" annotation(
              Dialog(group = "Input"));
            //output variables
            output SI.MassFlowRate M_FLOW "Output for function dp_laminar_MFLOW";
          protected
            Real MIN = Modelica.Constants.eps;
            SI.Diameter d_hyd = max(MIN, IN_con.d_hyd) "Hydraulic diameter";
            SI.Area A_cross = PI * IN_con.d_hyd ^ 2 / 4 "Circular cross sectional area";
            //Documentation
          algorithm
            M_FLOW := IN_var.rho * A_cross * (dp * d_hyd ^ 2 / (32 * IN_var.eta * IN_con.L));
            annotation(
              Inline = true,
              smoothOrder(normallyConstant = IN_con) = 2,
              inverse(dp = Modelica.Fluid.Dissipation.PressureLoss.StraightPipe.dp_laminar_DP(IN_con, IN_var, M_FLOW)),
              Documentation(info = "<html>
<p>
Calculation of pressure loss in a straight pipe for <b> laminar </b> flow regime of an incompressible and single-phase fluid flow only.
</p>

<p>
Generally this  function is numerically best used for the <b> compressible case </b>, where the pressure loss (dp) is known (out of pressures as state variable) in the used model and the corresponding mass flow rate (M_FLOW) has to be calculated. On the other hand the  function <a href=\"modelica://Modelica.Fluid.Dissipation.PressureLoss.StraightPipe.dp_laminar_DP\">dp_laminar_DP</a> is numerically best used for the <b> incompressible case </b> if the mass flow rate (m_flow) is known (as state variable) and the pressure loss (DP) has to be calculated.   <a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.PressureLoss.StraightPipe.dp_laminar\">See more information</a>.
</p>
</html>"));
          end dp_laminar_MFLOW;

          record dp_laminar_IN_con "Input record for function dp_laminar_DP and dp_laminar_MFLOW"
            extends Utilities.Records.PressureLoss.StraightPipe;
            annotation(
              Documentation(info = "<html>
This record is used as <b> input record </b> for the pressure loss function
<a href=\"modelica://Modelica.Fluid.Dissipation.PressureLoss.StraightPipe.dp_laminar_DP\"> dp_laminar_DP</a> and
<a href=\"modelica://Modelica.Fluid.Dissipation.PressureLoss.StraightPipe.dp_laminar_MFLOW\"> dp_laminar_MFLOW</a>.
</html>"));
          end dp_laminar_IN_con;

          record dp_laminar_IN_var "Input record for function dp_laminar_DP and dp_laminar_MFLOW"
            extends Modelica.Fluid.Dissipation.PressureLoss.StraightPipe.dp_overall_IN_var;
            annotation(
              Documentation(info = "<html>
This record is used as <b> input record </b> for the pressure loss function
<a href=\"modelica://Modelica.Fluid.Dissipation.PressureLoss.StraightPipe.dp_laminar_DP\"> dp_laminar_DP</a> and
<a href=\"modelica://Modelica.Fluid.Dissipation.PressureLoss.StraightPipe.dp_laminar_MFLOW\"> dp_laminar_MFLOW</a>.
</html>"));
          end dp_laminar_IN_var;

          function dp_overall_DP "Pressure loss of straight pipe | calculate pressure loss | overall flow regime | surface roughness"
            extends Modelica.Icons.Function;
            import FD = Modelica.Fluid.Dissipation.PressureLoss.StraightPipe;
            import SMOOTH = Modelica.Fluid.Dissipation.Utilities.Functions.General.Stepsmoother;
            //input records
            input Modelica.Fluid.Dissipation.PressureLoss.StraightPipe.dp_overall_IN_con IN_con "Input record for function dp_overall_DP" annotation(
              Dialog(group = "Constant inputs"));
            input Modelica.Fluid.Dissipation.PressureLoss.StraightPipe.dp_overall_IN_var IN_var "Input record for function dp_overall_DP" annotation(
              Dialog(group = "Variable inputs"));
            input SI.MassFlowRate m_flow "Mass flow rate" annotation(
              Dialog(group = "Input"));
            //output variables
            output SI.Pressure DP "Output for function dp_overall_DP";
          protected
            Real MIN = Modelica.Constants.eps;
            SI.Diameter d_hyd = max(MIN, IN_con.d_hyd) "Hydraulic diameter";
            SI.Area A_cross = PI * IN_con.d_hyd ^ 2 / 4 "Circular cross sectional area";
            Real k = max(MIN, abs(IN_con.K) / IN_con.d_hyd) "Relative roughness";
            SI.Length perimeter = PI * IN_con.d_hyd "Perimeter";
            //SOURCE_1: p.81, fig. 2-3, sec 21-22: definition of flow regime boundaries
            SI.ReynoldsNumber Re_lam_min = 1e3 "Minimum Reynolds number for laminar regime";
            SI.ReynoldsNumber Re_lam_max = 2090 * (1 / max(0.007, k)) ^ 0.0635 "Maximum Reynolds number for laminar regime";
            SI.ReynoldsNumber Re_lam_leave = min(Re_lam_max, max(Re_lam_min, 754 * Modelica.Math.exp(if k <= 0.007 then 0.0065 / 0.007 else 0.0065 / k))) "Start of transition regime for increasing Reynolds number (leaving laminar regime)";
            SI.ReynoldsNumber Re = Modelica.Fluid.Dissipation.Utilities.Functions.General.ReynoldsNumber(A_cross, perimeter, IN_var.rho, IN_var.eta, m_flow);
            dp_laminar_IN_con IN_con_lam(d_hyd = IN_con.d_hyd, L = IN_con.L);
          algorithm
            DP := SMOOTH(Re_lam_min, Re_lam_max, Re) * Dissipation.PressureLoss.StraightPipe.dp_laminar_DP(IN_con_lam, IN_var, m_flow) + SMOOTH(Re_lam_max, Re_lam_min, Re) * Dissipation.PressureLoss.StraightPipe.dp_turbulent_DP(IN_con, IN_var, m_flow);
            annotation(
              Inline = false,
              smoothOrder(normallyConstant = IN_con) = 2,
              inverse(m_flow = Modelica.Fluid.Dissipation.PressureLoss.StraightPipe.dp_overall_MFLOW(IN_con, IN_var, DP)),
              Documentation(info = "<html>
<p>
Calculation of pressure loss in a straight pipe for <b> overall </b> flow regime of an incompressible and single-phase fluid flow only considering surface roughness.
</p>

<p>
Generally this  function is numerically best used for the <b> incompressible case </b>, where the mass flow rate (m_flow) is known (as state variable) in the used model and the corresponding pressure loss (DP) has to be calculated. On the other hand the  function <a href=\"modelica://Modelica.Fluid.Dissipation.PressureLoss.StraightPipe.dp_overall_MFLOW\">dp_overall_MFLOW</a> is numerically best used for the <b> compressible case </b> if the pressure loss (dp) is known (out of pressures as state variable) and the mass flow rate (M_FLOW) has to be calculated.   <a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.PressureLoss.StraightPipe.dp_overall\">See more information</a>.
</p>
</html>"));
          end dp_overall_DP;

          function dp_overall_MFLOW "Pressure loss of straight pipe | calculate mass flow rate | overall flow regime | surface roughness"
            extends Modelica.Icons.Function;
            import FD = Modelica.Fluid.Dissipation.PressureLoss.StraightPipe;
            import SMOOTH = Modelica.Fluid.Dissipation.Utilities.Functions.General.Stepsmoother;
            import Modelica.Fluid.Dissipation.Utilities.Types.Roughness;
            //input records
            input Modelica.Fluid.Dissipation.PressureLoss.StraightPipe.dp_overall_IN_con IN_con "Input record for function dp_overall_MFLOW" annotation(
              Dialog(group = "Constant inputs"));
            input Modelica.Fluid.Dissipation.PressureLoss.StraightPipe.dp_overall_IN_var IN_var "Input record for function dp_overall_MFLOW" annotation(
              Dialog(group = "Variable inputs"));
            input SI.Pressure dp "Pressure loss" annotation(
              Dialog(group = "Input"));
            //output variables
            output SI.MassFlowRate M_FLOW "Output of function dp_overall_MFLOW";
          protected
            Real MIN = Modelica.Constants.eps;
            SI.Diameter d_hyd = max(MIN, IN_con.d_hyd) "Hydraulic diameter";
            SI.Area A_cross = max(MIN, PI * IN_con.d_hyd ^ 2 / 4) "Circular cross sectional area";
            Real k = max(MIN, abs(IN_con.K) / IN_con.d_hyd) "Relative roughness";
            //SOURCE_1: p.81, fig. 2-3, sec 21-22: definition of flow regime boundaries
            SI.ReynoldsNumber Re_lam_min = 1e3 "Minimum Reynolds number for laminar regime";
            SI.ReynoldsNumber Re_lam_max = 2090 * (1 / max(0.007, k)) ^ 0.0635 "Maximum Reynolds number for laminar regime";
            SI.ReynoldsNumber Re_turb_min = 4e3 "Minimum Reynolds number for turbulent regime";
            SI.ReynoldsNumber Re_lam_leave = min(Re_lam_max, max(Re_lam_min, 754 * Modelica.Math.exp(if k <= 0.007 then 0.0065 / 0.007 else 0.0065 / k))) "Start of transition regime for increasing Reynolds number (leaving laminar regime)";
            //determining Darcy friction factor out of pressure loss calculation for straight pipe:
            //dp = lambda_FRI*L/d_hyd*(rho/2)*velocity^2 and assuming lambda_FRI == lambda_FRI_calc/Re^2
            TYP.DarcyFrictionFactor lambda_FRI_calc = 2 * abs(dp) * d_hyd ^ 3 * IN_var.rho / (IN_con.L * IN_var.eta ^ 2) "Adapted Darcy friction factor";
            //SOURCE_3: p.Lab 1, eq. 5: determine Re assuming laminar regime (Blasius)
            SI.ReynoldsNumber Re_lam = lambda_FRI_calc / 64 "Reynolds number assuming laminar regime";
            //SOURCE_3: p.Lab 2, eq. 10: determine Re assuming turbulent regime (Colebrook-White)
            SI.ReynoldsNumber Re_turb = if IN_con.roughness == Roughness.Neglected then (max(MIN, lambda_FRI_calc) / 0.3164) ^ (1 / 1.75) else -2 * sqrt(max(lambda_FRI_calc, MIN)) * Modelica.Math.log10(2.51 / sqrt(max(lambda_FRI_calc, MIN)) + k / 3.7) "Reynolds number assuming turbulent regime";
            //determine actual flow regime
            SI.ReynoldsNumber Re_check = if Re_lam < Re_lam_leave then Re_lam else Re_turb;
            //determine Re for transition regime
            SI.ReynoldsNumber Re_trans = if Re_lam >= Re_lam_leave then Modelica.Fluid.Dissipation.Utilities.Functions.General.CubicInterpolation_DP(Re_check, Re_lam_leave, Re_turb_min, k, lambda_FRI_calc) else 0;
            //determine actual Re
            SI.ReynoldsNumber Re = if Re_lam < Re_lam_leave then Re_lam else if Re_turb > Re_turb_min then Re_turb else Re_trans;
            dp_laminar_IN_con IN_con_lam(d_hyd = IN_con.d_hyd, L = IN_con.L);
          algorithm
            M_FLOW := SMOOTH(Re_lam_min, Re_turb, Re) * Dissipation.PressureLoss.StraightPipe.dp_laminar_MFLOW(IN_con_lam, IN_var, dp) + SMOOTH(Re_turb, Re_lam_min, Re) * Dissipation.PressureLoss.StraightPipe.dp_turbulent_MFLOW(IN_con, IN_var, dp);
            annotation(
              Inline = false,
              smoothOrder(normallyConstant = IN_con) = 2,
              inverse(dp = Modelica.Fluid.Dissipation.PressureLoss.StraightPipe.dp_overall_DP(IN_con, IN_var, M_FLOW)),
              Documentation(info = "<html>
<p>
Calculation of pressure loss in a straight pipe for <b> overall </b> flow regime of an incompressible and single-phase fluid flow only considering surface roughness.
</p>

<p>
Generally this  function is numerically best used for the <b> compressible case </b>, where the pressure loss (dp) is known (out of pressures as state variable) in the used model and the corresponding mass flow rate (M_FLOW) has to be calculated. On the other hand the  function <a href=\"modelica://Modelica.Fluid.Dissipation.PressureLoss.StraightPipe.dp_overall_DP\">dp_overall_DP</a> is numerically best used for the <b> incompressible case </b> if the mass flow rate (m_flow) is known (as state variable) and the pressure loss (DP) has to be calculated.   <a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.PressureLoss.StraightPipe.dp_overall\">See more information</a>.
</p>
</html>"));
          end dp_overall_MFLOW;

          record dp_overall_IN_con "Input record for function dp_overall_DP and dp_overall_MFLOW"
            //straight pipe variables
            extends dp_turbulent_IN_con;
            annotation(
              Documentation(info = "<html>
This record is used as <b> input record </b> for the pressure loss function
<a href=\"modelica://Modelica.Fluid.Dissipation.PressureLoss.StraightPipe.dp_overall_DP\"> dp_overall_DP</a> and
<a href=\"modelica://Modelica.Fluid.Dissipation.PressureLoss.StraightPipe.dp_overall_MFLOW\"> dp_overall_MFLOW</a>.
</html>"));
          end dp_overall_IN_con;

          record dp_overall_IN_var "Input record for function dp_overall_DP and dp_overall_MFLOW"
            //fluid property variables
            extends Modelica.Fluid.Dissipation.Utilities.Records.General.PressureLoss;
            annotation(
              Documentation(info = "<html>
This record is used as <b> input record </b> for the pressure loss function
<a href=\"modelica://Modelica.Fluid.Dissipation.PressureLoss.StraightPipe.dp_overall_DP\"> dp_overall_DP</a> and
<a href=\"modelica://Modelica.Fluid.Dissipation.PressureLoss.StraightPipe.dp_overall_MFLOW\"> dp_overall_MFLOW</a>.
</html>"));
          end dp_overall_IN_var;

          function dp_turbulent_DP "Pressure loss of straight pipe | calculate pressure loss | turbulent flow regime | surface roughness"
            extends Modelica.Icons.Function;
            //SOURCE_1: Idelchik, I.E.: HANDBOOK OF HYDRAULIC RESISTANCE, 3rd edition, 2006.
            //SOURCE_2: Miller, D.S.: INTERNAL FLOW SYSTEMS, 2nd edition, 1984.
            //SOURCE_3: VDI-Waermeatlas, 9th edition, Springer-Verlag, 2002.
            import FD = Modelica.Fluid.Dissipation.PressureLoss.StraightPipe;
            //input records
            input Modelica.Fluid.Dissipation.PressureLoss.StraightPipe.dp_turbulent_IN_con IN_con "Input record for function dp_turbulent_DP" annotation(
              Dialog(group = "Constant inputs"));
            input Modelica.Fluid.Dissipation.PressureLoss.StraightPipe.dp_turbulent_IN_var IN_var "Input record for function dp_turbulent_DP" annotation(
              Dialog(group = "Variable inputs"));
            input SI.MassFlowRate m_flow "Mass flow rate" annotation(
              Dialog(group = "Input"));
            //output variables
            output SI.Pressure DP "Output for function dp_turbulent_DP";
          protected
            type TYP1 = Modelica.Fluid.Dissipation.Utilities.Types.Roughness;
            Real MIN = Modelica.Constants.eps;
            SI.ReynoldsNumber Re_min = 1;
            SI.Velocity v_min = Re_min * IN_var.eta / (IN_var.rho * IN_con.d_hyd);
            SI.Diameter d_hyd = IN_con.d_hyd "Hydraulic diameter";
            SI.Area A_cross = PI * IN_con.d_hyd ^ 2 / 4 "Circular cross sectional area";
            Real k = max(MIN, abs(IN_con.K) / IN_con.d_hyd) "Relative roughness";
            //SOURCE_1: p.81, fig. 2-3, sec 21-22: definition of flow regime boundaries
            SI.ReynoldsNumber Re_lam_min = 1e3 "Minimum Reynolds number for laminar regime";
            SI.ReynoldsNumber Re_lam_max = 2090 * (1 / max(0.007, k)) ^ 0.0635 "Maximum Reynolds number for laminar regime";
            SI.ReynoldsNumber Re_turb_min = 4e3 "Minimum Reynolds number for turbulent regime";
            SI.ReynoldsNumber Re_lam_leave = min(Re_lam_max, max(Re_lam_min, 754 * Modelica.Math.exp(if k <= 0.007 then 0.0065 / 0.007 else 0.0065 / k))) "Start of transition regime for increasing Reynolds number (leaving laminar regime)";
            SI.Velocity velocity = m_flow / (IN_var.rho * A_cross) "Mean velocity";
            SI.ReynoldsNumber Re = max(Re_min, IN_var.rho * abs(velocity) * d_hyd / IN_var.eta);
            //SOURCE_2: p.191, eq. 8.4: determining Darcy friction factor
            //assuming lambda_FRI == lambda_FRI_calc/Re^2
            TYP.DarcyFrictionFactor lambda_FRI_smooth = 0.3164 * Re ^ 1.75 "Darcy friction factor neglecting surface roughness (Blasius)";
            //here with lambda_FRI_rough == lambda_FRI*Re^2
            TYP.DarcyFrictionFactor lambda_FRI_rough = 0.25 * (max(Re, Re_lam_leave) / Modelica.Math.log10(k / 3.7 + 5.74 / max(Re, Re_lam_leave) ^ 0.9)) ^ 2 "Darcy friction factor considering surface roughness";
            TYP.DarcyFrictionFactor lambda_FRI = if IN_con.roughness == TYP1.Neglected then lambda_FRI_smooth else lambda_FRI_rough "Darcy friction factor";
            TYP.DarcyFrictionFactor lambda_FRI_calc = if Re < Re_lam_leave then 64 / Re else if Re > Re_turb_min then lambda_FRI / Re ^ 2 else Modelica.Fluid.Dissipation.Utilities.Functions.General.CubicInterpolation_MFLOW(Re, Re_lam_leave, Re_turb_min, k) / Re ^ 2 "Darcy friction factor";
            TYP.PressureLossCoefficient zeta_TOT = lambda_FRI_calc * IN_con.L / d_hyd "Pressure loss coefficient";
            //Documentation
          algorithm
            DP := zeta_TOT * (IN_var.rho / 2) * Modelica.Fluid.Dissipation.Utilities.Functions.General.SmoothPower(velocity, v_min, 2);
            annotation(
              Inline = false,
              smoothOrder(normallyConstant = IN_con) = 2,
              inverse(m_flow = Modelica.Fluid.Dissipation.PressureLoss.StraightPipe.dp_turbulent_MFLOW(IN_con, IN_var, DP)),
              Documentation(info = "<html>
<p>
Calculation of pressure loss in a straight pipe for <b> turbulent </b> flow regime of an incompressible and single-phase fluid flow only considering surface roughness.
</p>

<p>
Generally this  function is numerically best used for the <b> incompressible case </b>, where the mass flow rate (m_flow) is known (as state variable) in the used model and the corresponding pressure loss (DP) has to be calculated. On the other hand the  function <a href=\"modelica://Modelica.Fluid.Dissipation.PressureLoss.StraightPipe.dp_turbulent_MFLOW\">dp_turbulent_MFLOW</a> is numerically best used for the <b> compressible case </b> if the pressure loss (dp) is known (out of pressures as state variable) and the mass flow rate (M_FLOW) has to be calculated.   <a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.PressureLoss.StraightPipe.dp_turbulent\">See more information</a>.
</p>
</html>"));
          end dp_turbulent_DP;

          function dp_turbulent_MFLOW "Pressure loss of straight pipe | calculate mass flow rate | turbulent flow regime | surface roughness"
            extends Modelica.Icons.Function;
            //SOURCE_1: Idelchik, I.E.: HANDBOOK OF HYDRAULIC RESISTANCE, 3rd edition, 2006.
            //SOURCE_2: Miller, D.S.: INTERNAL FLOW SYSTEMS, 2nd edition, 1984.
            //SOURCE_3: VDI-Waermeatlas, 9th edition, Springer-Verlag, 2002.
            import FD = Modelica.Fluid.Dissipation.PressureLoss.StraightPipe;
            //input records
            input Modelica.Fluid.Dissipation.PressureLoss.StraightPipe.dp_turbulent_IN_con IN_con "Input record for function dp_turbulent_MFLOW" annotation(
              Dialog(group = "Constant inputs"));
            input Modelica.Fluid.Dissipation.PressureLoss.StraightPipe.dp_turbulent_IN_var IN_var "Input record for function dp_turbulent_MFLOW" annotation(
              Dialog(group = "Variable inputs"));
            input SI.Pressure dp "Pressure loss" annotation(
              Dialog(group = "Input"));
            //output variables
            output SI.MassFlowRate M_FLOW "Mass flow rate";
          protected
            type TYP1 = Modelica.Fluid.Dissipation.Utilities.Types.Roughness;
            Real MIN = Modelica.Constants.eps;
            SI.ReynoldsNumber Re_min = 1;
            SI.Diameter d_hyd = max(MIN, IN_con.d_hyd) "Hydraulic diameter";
            SI.Area A_cross = PI * IN_con.d_hyd ^ 2 / 4 "Circular cross sectional area";
            Real k = max(MIN, abs(IN_con.K) / IN_con.d_hyd) "Relative roughness";
            //SOURCE_1: p.81, fig. 2-3, sec 21-22: definition of flow regime boundaries
            SI.ReynoldsNumber Re_lam_min = 1e3 "Minimum Reynolds number for laminar regime";
            SI.ReynoldsNumber Re_lam_max = 2090 * (1 / max(0.007, k)) ^ 0.0635 "Maximum Reynolds number for laminar regime";
            SI.ReynoldsNumber Re_turb_min = 4e3 "Minimum Reynolds number for turbulent regime";
            SI.ReynoldsNumber Re_lam_leave = min(Re_lam_max, max(Re_lam_min, 754 * Modelica.Math.exp(if k <= 0.007 then 0.0065 / 0.007 else 0.0065 / k))) "Start of transition regime for increasing Reynolds number (leaving laminar regime)";
            //determining Darcy friction factor out of pressure loss calculation for straight pipe:
            //dp = lambda_FRI*L/d_hyd*(rho/2)*velocity^2 and assuming lambda_FRI == lambda_FRI_calc/Re^2
            TYP.DarcyFrictionFactor lambda_FRI_calc = 2 * abs(dp) * d_hyd ^ 3 * IN_var.rho / (IN_con.L * IN_var.eta ^ 2) "Adapted Darcy friction factor";
            //SOURCE_3: p.Lab 1, eq. 5: determine Re assuming laminar regime (Hagen-Poiseuille)
            SI.ReynoldsNumber Re_lam = lambda_FRI_calc / 64 "Reynolds number assuming laminar regime";
            //SOURCE_3: p.Lab 2, eq. 10: determine Re assuming turbulent regime (Colebrook-White)
            SI.ReynoldsNumber Re_turb = if IN_con.roughness == TYP1.Neglected then (max(MIN, lambda_FRI_calc) / 0.3164) ^ (1 / 1.75) else -2 * sqrt(max(lambda_FRI_calc, MIN)) * Modelica.Math.log10(2.51 / sqrt(max(lambda_FRI_calc, MIN)) + k / 3.7) "Reynolds number assuming turbulent regime";
            //determine actual flow regime
            SI.ReynoldsNumber Re_check = if Re_lam < Re_lam_leave then Re_lam else Re_turb;
            //determine Re for transition regime
            SI.ReynoldsNumber Re_trans = if Re_lam >= Re_lam_leave then Modelica.Fluid.Dissipation.Utilities.Functions.General.CubicInterpolation_DP(Re_check, Re_lam_leave, Re_turb_min, k, lambda_FRI_calc) else 0;
            //determine actual Re
            SI.ReynoldsNumber Re = if Re_lam < Re_lam_leave then Re_lam else if Re_turb > Re_turb_min then Re_turb else Re_trans;
            //determine velocity
            SI.Velocity velocity = (if dp >= 0 then Re else -Re) * IN_var.eta / (IN_var.rho * d_hyd) "Mean velocity";
            //Documentation
          algorithm
            M_FLOW := IN_var.rho * A_cross * velocity;
            annotation(
              Inline = false,
              smoothOrder(normallyConstant = IN_con) = 2,
              inverse(dp = Modelica.Fluid.Dissipation.PressureLoss.StraightPipe.dp_turbulent_DP(IN_con, IN_var, M_FLOW)),
              Documentation(info = "<html>
<p>
Calculation of pressure loss in a straight pipe for <b> turbulent </b> flow regime of an incompressible and single-phase fluid flow only considering surface roughness.
</p>

<p>
Generally this function is numerically best used for the <b> compressible case </b> if the pressure loss (dp) is known (out of pressures as state variable) and the mass flow rate (M_FLOW) has to be calculated. On the other hand the  function <a href=\"modelica://Modelica.Fluid.Dissipation.PressureLoss.StraightPipe.dp_turbulent_DP\">dp_turbulent_DP</a> is numerically best used for the <b> incompressible case </b>, where the mass flow rate (m_flow) is known (as state variable) in the used model and the corresponding pressure loss (DP) has to be calculated.   <a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.PressureLoss.StraightPipe.dp_turbulent\">See more information</a>.
</p>
</html>"));
          end dp_turbulent_MFLOW;

          record dp_turbulent_IN_con "Input record for function dp_turbulent_DP and dp_turbulent_MFLOW"
            Modelica.Fluid.Dissipation.Utilities.Types.Roughness roughness = Dissipation.Utilities.Types.Roughness.Neglected "Choice of considering surface roughness" annotation(
              Dialog(group = "Straight pipe"));
            extends Utilities.Records.PressureLoss.StraightPipe;
            SI.Length K = 0 "Roughness (average height of surface asperities)" annotation(
              Dialog(group = "Straight pipe"));
            annotation(
              Documentation(info = "<html>
This record is used as <b> input record </b> for the pressure loss function
<a href=\"modelica://Modelica.Fluid.Dissipation.PressureLoss.StraightPipe.dp_turbulent_DP\"> dp_turbulent_DP</a> and
<a href=\"modelica://Modelica.Fluid.Dissipation.PressureLoss.StraightPipe.dp_turbulent_MFLOW\"> dp_turbulent_MFLOW</a>.
</html>"));
          end dp_turbulent_IN_con;

          record dp_turbulent_IN_var "Input record for function dp_turbulent_DP and dp_turbulent_MFLOW"
            extends Modelica.Fluid.Dissipation.PressureLoss.StraightPipe.dp_overall_IN_var;
            annotation(
              Documentation(info = "<html>
This record is used as <b> input record </b> for the pressure loss function
<a href=\"modelica://Modelica.Fluid.Dissipation.PressureLoss.StraightPipe.dp_turbulent_DP\"> dp_turbulent_DP</a> and
<a href=\"modelica://Modelica.Fluid.Dissipation.PressureLoss.StraightPipe.dp_turbulent_MFLOW\"> dp_turbulent_MFLOW</a>.
</html>"));
          end dp_turbulent_IN_var;

          function dp_twoPhaseOverall_DP "Pressure loss of straight pipe for two phase flow | calculate (frictional, momentum, geodetic) pressure loss"
            extends Modelica.Icons.Function;
            //SOURCE_1: Friedel,L.:IMPROVED FRICTION PRESSURE DROP CORRELATIONS FOR HORIZONTAL AND VERTICAL TWO PHASE PIPE FLOW, 3R International, Vol. 18, Issue 7, pp. 485-491, 1979
            //SOURCE_2: Chisholm,D.:PRESSURE GRADIENTS DUE TO FRICTION DURING THE FLOW OF EVAPORATING TWO-PHASE MIXTURES IN SMOOTH TUBES AND CHANNELS, Int. J. Heat Mass Transfer, Vol. 16, pp. 347-358, Pergamon Press 1973
            //SOURCE_3: VDI-Waermeatlas, 10th edition, Springer-Verlag, 2006.
            //SOURCE 4: J.M. Jensen and H. Tummescheit. Moving boundary models for dynamic simulations of two-phase flows. In Proceedings of the 2nd International Modelica Conference, pp. 235-244, Oberpfaffenhofen, Germany, 2002. The Modelica Association.
            //SOURCE_5: Thome, J.R., Engineering Data Book 3, Swiss Federal Institute of Technology Lausanne (EPFL), 2009.
            //input records
            input Modelica.Fluid.Dissipation.PressureLoss.StraightPipe.dp_twoPhaseOverall_IN_con IN_con "Input record for function dp_twoPhaseOverall_DP" annotation(
              Dialog(group = "Constant inputs"));
            input Modelica.Fluid.Dissipation.PressureLoss.StraightPipe.dp_twoPhaseOverall_IN_var IN_var "Input record for function dp_twoPhaseOverall_DP" annotation(
              Dialog(group = "Variable inputs"));
            input SI.MassFlowRate m_flow "Mass flow rate" annotation(
              Dialog(group = "Input"));
            //output variables
            output SI.Pressure DP "Two phase pressure loss";
          protected
            type TYP = Modelica.Fluid.Dissipation.Utilities.Types.TwoPhaseFrictionalPressureLoss;
            Real MIN = Modelica.Constants.eps;
            SI.Area A_cross = max(MIN, IN_con.A_cross) "Cross sectional area";
            SI.Diameter d_hyd = max(MIN, 4 * A_cross / max(MIN, IN_con.perimeter)) "Hydraulic diameter";
            Real mdot_A = abs(m_flow) / A_cross "Mass flux";
            Real xflowEnd = min(1, max(0, abs(IN_var.x_flow_end))) "Mass flow rate quality at end of length";
            Real xflowSta = min(1, max(0, abs(IN_var.x_flow_sta))) "Mass flow rate quality at start of length";
            Real x_flow = (xflowEnd + xflowSta) / 2 "Mean mass flow rate quality over length";
            //SOURCE_5: p.17-1 to 17-5, sec. 17.1 to 17.2: Considering cross sectional void fraction [epsilon=A_g/(A_g+A_l)]
            Real epsilon = Modelica.Fluid.Dissipation.Utilities.Functions.PressureLoss.TwoPhase.VoidFraction(IN_con.voidFractionApproach, true, IN_var.rho_g, IN_var.rho_l, x_flow) "Void fraction";
            //SOURCE_1: Considering frictional pressure loss w.r.t. to correlation of Friedel
            //SOURCE_2: Considering frictional pressure loss w.r.t. to correlation of Chisholm
            SI.Pressure DP_fric = if IN_con.frictionalPressureLoss == TYP.Friedel then Modelica.Fluid.Dissipation.Utilities.Functions.PressureLoss.TwoPhase.dp_twoPhaseFriedel_DP(IN_con, IN_var, m_flow) else if IN_con.frictionalPressureLoss == TYP.Chisholm then Modelica.Fluid.Dissipation.Utilities.Functions.PressureLoss.TwoPhase.dp_twoPhaseChisholm_DP(IN_con, IN_var, m_flow) else 0 "Frictional pressure loss";
            //SOURCE_3: p.Lba 4, eq. 22: Considering momentum pressure loss assuming heterogeneous approach for two phase flow
            //Evaporation >> positive momentum pressure loss (assumed vice versa at condensation)
            SI.Pressure DP_mom = if IN_con.momentumPressureLoss then Modelica.Fluid.Dissipation.Utilities.Functions.PressureLoss.TwoPhase.dp_twoPhaseMomentum_DP(IN_con.voidFractionApproach, IN_con.massFlowRateCorrection, IN_con.A_cross, IN_con.perimeter, IN_var.rho_g, IN_var.rho_l, IN_var.x_flow_end, IN_var.x_flow_sta, abs(m_flow)) else 0 "Momentum pressure loss";
            //SOURCE_3: p.Lbb 1, eq. 4: Considering geodetic pressure loss assuming constant void fraction for flow length
            SI.Pressure DP_geo = if IN_con.geodeticPressureLoss then Modelica.Fluid.Dissipation.Utilities.Functions.PressureLoss.TwoPhase.dp_twoPhaseGeodetic_DP(IN_con.voidFractionApproach, true, IN_con.length, IN_con.phi, IN_var.rho_g, IN_var.rho_l, IN_var.x_flow) else 0 "Geodetic pressure loss";
            //Documentation
          algorithm
            DP := DP_fric + DP_mom + DP_geo;
            annotation(
              Inline = false,
              smoothOrder(normallyConstant = IN_con) = 2,
              Documentation(info = "<html>
<p>
Calculation of pressure loss for <b>two phase flow</b> in a horizontal <b>or</b> vertical straight pipe for an overall flow regime considering frictional, momentum and geodetic pressure loss.
</p>

<p>
Generally the pressure loss for two phase flow in a horizontal or a vertical straight pipe can be calculated for the following fluid flow regimes:
</p>
<p>
<b>Horizontal fluid flow</b> [(a) bubble flow, (b) stratified flow, (c) wavy flow, (d) slug flow, (e) annular flow]:
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Fluid/Dissipation/pressureLoss/straightPipe/pic_twoPhaseFlowRegimes_horizontal.png\" alt=\"pic_twoPhaseFlowRegimes_horizontal\"/>
</p>

<p>
<b>Vertical fluid flow</b> [(a) bubble flow, (b) plug slug flow, (c) foam flow, (d) annular streak flow, (e) annular flow]:
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Fluid/Dissipation/pressureLoss/straightPipe/pic_twoPhaseFlowRegimes_vertical.png\" alt=\"pic_twoPhaseFlowRegimes_vertical\"/>
</p>

<p>
 <a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.PressureLoss.StraightPipe.dp_twoPhaseOverall\">See more information</a>.
</p>
</html>"));
          end dp_twoPhaseOverall_DP;

          record dp_twoPhaseOverall_IN_con "Input record for function dp_twoPhaseOverall_DP"
            //choices
            Modelica.Fluid.Dissipation.Utilities.Types.TwoPhaseFrictionalPressureLoss frictionalPressureLoss = Dissipation.Utilities.Types.TwoPhaseFrictionalPressureLoss.Friedel "Choice of frictional pressure loss approach" annotation(
              Dialog(group = "Choices"));
            Modelica.Fluid.Dissipation.Utilities.Types.VoidFractionApproach voidFractionApproach = Dissipation.Utilities.Types.VoidFractionApproach.Homogeneous "Choice of void fraction approach" annotation(
              Dialog(group = "Choices"));
            Boolean momentumPressureLoss = false "Considering momentum pressure loss" annotation(
              Dialog(group = "Choices"));
            Boolean massFlowRateCorrection = false "Consider heterogeneous mass flow rate correction" annotation(
              Dialog(group = "Choices", enable = if momentumPressureLoss then true else false));
            Boolean geodeticPressureLoss = false "Considering geodetic pressure loss" annotation(
              Dialog(group = "Choices"));
            extends Modelica.Fluid.Dissipation.Utilities.Records.General.TwoPhaseFlow_con;
            SI.Angle phi = 0 "Tilt angle to horizontal" annotation(
              Dialog(group = "Geometry"));
            annotation(
              Documentation(info = "<html>
This record is used as <b> input record </b> for the pressure loss function <a href=\"modelica://Modelica.Fluid.Dissipation.PressureLoss.StraightPipe.dp_twoPhaseOverall_DP\"> dp_twoPhaseOverall_DP</a>.
</html>"));
          end dp_twoPhaseOverall_IN_con;

          record dp_twoPhaseOverall_IN_var "Input record for function dp_twoPhaseOverall_DP"
            Real x_flow_end = 0 "Mass flow rate quality at end of length" annotation(
              Dialog(group = "Fluid properties"));
            Real x_flow_sta = 0 "Mass flow rate quality at start of length" annotation(
              Dialog(group = "Fluid properties"));
            extends Modelica.Fluid.Dissipation.Utilities.Records.General.TwoPhaseFlow_var(final x_flow = (x_flow_end + x_flow_sta) / 2);
            annotation(
              Documentation(info = "<html>
This record is used as <b> input record </b> for the pressure loss function <a href=\"modelica://Modelica.Fluid.Dissipation.PressureLoss.StraightPipe.dp_twoPhaseOverall_DP\"> dp_twoPhaseOverall_DP</a>.
</html>"));
          end dp_twoPhaseOverall_IN_var;
          annotation(
            preferredView = "info",
            Documentation(info = "<html>
<h4>Straight Pipe</h4>
<h5>Laminar flow</h5>
<p>
Calculation of pressure loss in a straight pipe for <b> laminar </b> flow regime of single-phase fluid flow only.
<a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.PressureLoss.StraightPipe.dp_laminar\">See more information</a>.
</p>

<h5>Turbulent flow</h5>
<p>
Calculation of pressure loss in a straight pipe for <b> turbulent </b> flow regime of single-phase fluid flow only considering surface roughness.
<a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.PressureLoss.StraightPipe.dp_turbulent\">See more information</a>.
</p>

<h5>Overall flow</h5>
<p>
Calculation of pressure loss in a straight pipe for <b> laminar or turbulent </b> flow regime of single-phase fluid flow only considering surface roughness.
<a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.PressureLoss.StraightPipe.dp_overall\">See more information</a>.
</p>

<h5>Two phase overall flow</h5>
<p>
Calculation of pressure loss for <b>two phase flow</b> in a horizontal <b>or</b> vertical straight pipe for an overall flow regime considering frictional, momentum and geodetic pressure loss.
 <a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.PressureLoss.StraightPipe.dp_twoPhaseOverall\">See more information</a>.
</p>
</html>"));
        end StraightPipe;

        package Valve "Package for pressure loss calculation of valves"
          extends Modelica.Icons.VariantsPackage;

          function dp_severalGeometryOverall_DP "Pressure loss of valve | calculate pressure loss | several geometries | overall flow regime"
            extends Modelica.Icons.Function;
            import FD = Modelica.Fluid.Dissipation.PressureLoss.Valve;
            import SMOOTH = Modelica.Fluid.Dissipation.Utilities.Functions.General.Stepsmoother;
            import TYP = Modelica.Fluid.Dissipation.Utilities.Types;
            //input records
            input Modelica.Fluid.Dissipation.PressureLoss.Valve.dp_severalGeometryOverall_IN_con IN_con "Input record for function dp_severalGeometryOverall_DP" annotation(
              Dialog(group = "Constant inputs"));
            input Modelica.Fluid.Dissipation.PressureLoss.Valve.dp_severalGeometryOverall_IN_var IN_var "Input record for function dp_severalGeometryOverall_DP" annotation(
              Dialog(group = "Variable inputs"));
            input SI.MassFlowRate m_flow "Mass flow rate" annotation(
              Dialog(group = "Input"));
            //output variables
            output SI.Pressure DP "Pressure loss";
          protected
            type TYP1 = Modelica.Fluid.Dissipation.Utilities.Types.ValveCoefficient;
            type TYP2 = Modelica.Fluid.Dissipation.Utilities.Types.ValveGeometry;
            Real MIN = Modelica.Constants.eps;
            SI.Area Av = if IN_con.valveCoefficient == TYP1.AV then IN_con.Av else if IN_con.valveCoefficient == TYP1.KV then IN_con.Kv * 27.7e-6 else if IN_con.valveCoefficient == TYP1.CV then IN_con.Cv * 24e-6 else if IN_con.valveCoefficient == TYP1.OP then IN_con.m_flow_nominal / max(MIN, IN_con.opening_nominal * (IN_con.rho_nominal * IN_con.dp_nominal) ^ 0.5) else MIN "Av (metric) flow coefficient [Av]=m^2";
            TYP.PressureLossCoefficient zeta_bal = SMOOTH(0.05, 0, IN_var.opening) * 10 ^ ((-3.8397 * IN_var.opening) + 2.9449) + SMOOTH(0, 0.05, IN_var.opening) * IN_con.zeta_TOT_max "Ball valves";
            TYP.PressureLossCoefficient zeta_dia = SMOOTH(0.05, 0, IN_var.opening) * 10 ^ (2.2596 * exp(-1.8816 * IN_var.opening)) + SMOOTH(0, 0.05, IN_var.opening) * IN_con.zeta_TOT_max "Diaphragm valves";
            TYP.PressureLossCoefficient zeta_but = SMOOTH(0.05, 0, IN_var.opening) * 619.81 * exp(-7.3211 * IN_var.opening) + SMOOTH(0, 0.05, IN_var.opening) * IN_con.zeta_TOT_max "Butterfly valves";
            TYP.PressureLossCoefficient zeta_gat = SMOOTH(0.05, 0, IN_var.opening) * 51.45 * exp(-6.046 * IN_var.opening) + SMOOTH(0, 0.05, IN_var.opening) * IN_con.zeta_TOT_max "Gate valves";
            TYP.PressureLossCoefficient zeta_slu = SMOOTH(0.05, 0, IN_var.opening) * 248.89 * exp(-7.8265 * IN_var.opening) + SMOOTH(0, 0.05, IN_var.opening) * IN_con.zeta_TOT_max "Sluice valves";
            TYP.PressureLossCoefficient zeta_TOT = if IN_con.geometry == TYP2.Ball then zeta_bal else if IN_con.geometry == TYP2.Diaphragm then zeta_dia else if IN_con.geometry == TYP2.Butterfly then zeta_but else if IN_con.geometry == TYP2.Gate then zeta_gat else if IN_con.geometry == TYP2.Sluice then zeta_slu else 0 "Pressure loss coefficient of chosen valve";
            Real valveCharacteristic = (2 / min(IN_con.zeta_TOT_max, max(MIN, max(IN_con.zeta_TOT_min, abs(zeta_TOT))))) ^ 0.5 "Valve characteristic considering opening of chosen valve";
            SI.MassFlowRate m_flow_small = valveCharacteristic * Av * IN_var.rho ^ 0.5 * IN_con.dp_small ^ 0.5 "Mass flow rate at linearisation";
            //Documentation
          algorithm
            DP := 1 / ((valveCharacteristic * Av) ^ 2 * IN_var.rho) * Modelica.Fluid.Dissipation.Utilities.Functions.General.SmoothPower(m_flow, m_flow_small, 2);
            annotation(
              Inline = false,
              smoothOrder(normallyConstant = IN_con) = 2,
              inverse(m_flow = Modelica.Fluid.Dissipation.PressureLoss.Valve.dp_severalGeometryOverall_MFLOW(IN_con, IN_var, DP)),
              Documentation(info = "<html>
<p>
Calculation of pressure loss for a valve with different geometries at overall flow regime for incompressible and single-phase fluid flow in dependence of its opening.
</p>

<p>
Generally this function is numerically best used for the <b> incompressible case </b>, where the mass flow rate (m_flow) is known (as state variable) in the used model and the corresponding pressure loss (DP) has to be calculated. On the other hand the function <a href=\"modelica://Modelica.Fluid.Dissipation.PressureLoss.Valve.dp_severalGeometryOverall_MFLOW\">dp_severalGeometryOverall_MFLOW</a> is numerically best used for the <b> compressible case </b> if the pressure loss (dp) is known (out of pressures as state variable) and the mass flow rate (M_FLOW) has to be calculated. <a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.PressureLoss.Valve.dp_severalGeometryOverall\">See more information</a>.
</p>
</html>"));
          end dp_severalGeometryOverall_DP;

          function dp_severalGeometryOverall_MFLOW "Pressure loss of valve | calculate mass flow rate | several geometries | overall flow regime"
            extends Modelica.Icons.Function;
            import FD = Modelica.Fluid.Dissipation.PressureLoss.Valve;
            import SMOOTH = Modelica.Fluid.Dissipation.Utilities.Functions.General.Stepsmoother;
            import TYP = Modelica.Fluid.Dissipation.Utilities.Types;
            //input records
            input Modelica.Fluid.Dissipation.PressureLoss.Valve.dp_severalGeometryOverall_IN_con IN_con "Input record for function dp_severalGeometryOverall_MFLOW" annotation(
              Dialog(group = "Constant inputs"));
            input Modelica.Fluid.Dissipation.PressureLoss.Valve.dp_severalGeometryOverall_IN_var IN_var "Input record for function dp_severalGeometryOverall_MFLOW" annotation(
              Dialog(group = "Variable inputs"));
            input SI.Pressure dp "Pressure loss" annotation(
              Dialog(group = "Input"));
            //output variables
            output SI.MassFlowRate M_FLOW "Mass flow rate";
          protected
            type TYP1 = Modelica.Fluid.Dissipation.Utilities.Types.ValveCoefficient;
            type TYP2 = Modelica.Fluid.Dissipation.Utilities.Types.ValveGeometry;
            Real MIN = Modelica.Constants.eps;
            SI.Area Av = if IN_con.valveCoefficient == TYP1.AV then IN_con.Av else if IN_con.valveCoefficient == TYP1.KV then IN_con.Kv * 27.7e-6 else if IN_con.valveCoefficient == TYP1.CV then IN_con.Cv * 24e-6 else if IN_con.valveCoefficient == TYP1.OP then IN_con.m_flow_nominal / max(MIN, IN_con.opening_nominal * (IN_con.rho_nominal * IN_con.dp_nominal) ^ 0.5) else MIN "Av (metric) flow coefficient [Av]=m^2";
            TYP.PressureLossCoefficient zeta_bal = SMOOTH(0.05, 0, IN_var.opening) * 10 ^ ((-3.8397 * IN_var.opening) + 2.9449) + SMOOTH(0, 0.05, IN_var.opening) * IN_con.zeta_TOT_max "Ball valves";
            TYP.PressureLossCoefficient zeta_dia = SMOOTH(0.05, 0, IN_var.opening) * 10 ^ (2.2596 * exp(-1.8816 * IN_var.opening)) + SMOOTH(0, 0.05, IN_var.opening) * IN_con.zeta_TOT_max "Diaphragm valves";
            TYP.PressureLossCoefficient zeta_but = SMOOTH(0.05, 0, IN_var.opening) * 619.81 * exp(-7.3211 * IN_var.opening) + SMOOTH(0, 0.05, IN_var.opening) * IN_con.zeta_TOT_max "Butterfly valves";
            TYP.PressureLossCoefficient zeta_gat = SMOOTH(0.05, 0, IN_var.opening) * 51.45 * exp(-6.046 * IN_var.opening) + SMOOTH(0, 0.05, IN_var.opening) * IN_con.zeta_TOT_max "Gate valves";
            TYP.PressureLossCoefficient zeta_slu = SMOOTH(0.05, 0, IN_var.opening) * 248.89 * exp(-7.8265 * IN_var.opening) + SMOOTH(0, 0.05, IN_var.opening) * IN_con.zeta_TOT_max "Sluice valves";
            TYP.PressureLossCoefficient zeta_TOT = if IN_con.geometry == TYP2.Ball then zeta_bal else if IN_con.geometry == TYP2.Diaphragm then zeta_dia else if IN_con.geometry == TYP2.Butterfly then zeta_but else if IN_con.geometry == TYP2.Gate then zeta_gat else if IN_con.geometry == TYP2.Sluice then zeta_slu else 0 "Pressure loss coefficient of chosen valve";
            Real valveCharacteristic = (2 / min(IN_con.zeta_TOT_max, max(MIN, max(IN_con.zeta_TOT_min, abs(zeta_TOT))))) ^ 0.5 "Valve characteristic considering opening of chosen valve";
            //Documentation
          algorithm
            M_FLOW := valveCharacteristic * Av * IN_var.rho ^ 0.5 * Modelica.Fluid.Dissipation.Utilities.Functions.General.SmoothPower(dp, IN_con.dp_small, 0.5);
            annotation(
              Inline = false,
              smoothOrder(normallyConstant = IN_con) = 2,
              inverse(dp = Modelica.Fluid.Dissipation.PressureLoss.Valve.dp_severalGeometryOverall_DP(IN_con, IN_var, M_FLOW)),
              Documentation(info = "<html>
<p>
Calculation of pressure loss for a valve with different geometries at overall flow regime for incompressible and single-phase fluid flow in dependence of its opening.
</p>

<p>
Generally this  function is numerically best used for the <b> compressible case </b>, where the pressure loss (dp) is known (out of pressures as state variable) in the used model and the corresponding mass flow rate (M_FLOW) has to be calculated. On the other hand the  function <a href=\"modelica://Modelica.Fluid.Dissipation.PressureLoss.Valve.dp_severalGeometryOverall_DP\">dp_severalGeometryOverall_DP</a> is numerically best used for the <b> incompressible case </b> if the mass flow rate (m_flow) is known (as state variable) and the pressure loss (DP) has to be calculated. <a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.PressureLoss.Valve.dp_severalGeometryOverall\">See more information</a>.
</p>
</html>"));
          end dp_severalGeometryOverall_MFLOW;

          record dp_severalGeometryOverall_IN_con "Input record for function dp_severalGeometryOverall_DP and dp_severalGeometryOverall_MFLOW"
            extends Modelica.Icons.Record;
            Modelica.Fluid.Dissipation.Utilities.Types.ValveGeometry geometry = Dissipation.Utilities.Types.ValveGeometry.Ball "Choice of geometry for valve" annotation(
              Dialog(group = "Valve"));
            Modelica.Fluid.Dissipation.Utilities.Types.ValveCoefficient valveCoefficient = Modelica.Fluid.Dissipation.Utilities.Types.ValveCoefficient.AV "Choice of valve coefficient" annotation(
              Dialog(group = "Valve"));
            //valve variables
            Real Av = PI * 0.1 ^ 2 / 4 "Av (metric) flow coefficient [Av]=m^2" annotation(
              Dialog(group = "Valve", enable = if valveCoefficient == 1 then true else false));
            Real Kv = Av / 27.7e-6 "Kv (metric) flow coefficient [Kv]=m^3/h" annotation(
              Dialog(group = "Valve", enable = if valveCoefficient == 2 then true else false));
            Real Cv = Av / 24.6e-6 "Cv (US) flow coefficient [Cv]=USG/min" annotation(
              Dialog(group = "Valve", enable = if valveCoefficient == 3 then true else false));
            SI.Pressure dp_nominal = 1e3 "Nominal pressure loss" annotation(
              Dialog(group = "Valve", enable = if valveCoefficient == 4 then true else false));
            SI.MassFlowRate m_flow_nominal = opening_nominal * Av * (rho_nominal * dp_nominal) ^ 0.5 "Nominal mass flow rate" annotation(
              Dialog(group = "Valve", enable = if valveCoefficient == 4 then true else false));
            SI.Density rho_nominal = 1000 "Nominal inlet density" annotation(
              Dialog(group = "Valve", enable = if valveCoefficient == 4 then true else false));
            Real opening_nominal = 0.5 "Nominal opening" annotation(
              Dialog(group = "Valve", enable = if valveCoefficient == 4 then true else false));
            Real zeta_TOT_min = 1e-3 "Minimal pressure loss coefficient at full opening" annotation(
              Dialog(group = "Valve"));
            Real zeta_TOT_max = 1e8 "Maximum pressure loss coefficient at closed opening" annotation(
              Dialog(group = "Valve"));
            //numerical aspects
            SI.Pressure dp_small = 0.01 * dp_nominal "Linearisation for a pressure loss smaller then dp_small" annotation(
              Dialog(group = "Linearisation"));
            annotation(
              Documentation(info = "<html>
This record is used as <b> input record </b> for the pressure loss function
<a href=\"modelica://Modelica.Fluid.Dissipation.PressureLoss.Valve.dp_severalGeometryOverall_DP\"> dp_severalGeometryOverall_DP</a> and
<a href=\"modelica://Modelica.Fluid.Dissipation.PressureLoss.Valve.dp_severalGeometryOverall_MFLOW\"> dp_severalGeometryOverall_MFLOW</a>.
</html>"));
          end dp_severalGeometryOverall_IN_con;

          record dp_severalGeometryOverall_IN_var "Input record for function dp_severalGeometryOverall_DP and dp_severalGeometryOverall_MFLOW"
            extends Modelica.Icons.Record;
            //valve variables
            Real opening = 1 "Opening of valve | 0==closed and 1== fully opened" annotation(
              Dialog(group = "Valve"));
            //fluid property variables
            SI.DynamicViscosity eta "Dynamic viscosity of fluid" annotation(
              Dialog(group = "Fluid properties"));
            SI.Density rho "Density of fluid" annotation(
              Dialog(group = "Fluid properties"));
            annotation(
              Documentation(info = "<html>
This record is used as <b> input record </b> for the pressure loss function
<a href=\"modelica://Modelica.Fluid.Dissipation.PressureLoss.Valve.dp_severalGeometryOverall_DP\"> dp_severalGeometryOverall_DP</a> and
<a href=\"modelica://Modelica.Fluid.Dissipation.PressureLoss.Valve.dp_severalGeometryOverall_MFLOW\"> dp_severalGeometryOverall_MFLOW</a>.
</html>"));
          end dp_severalGeometryOverall_IN_var;
          annotation(
            preferredView = "info",
            Documentation(info = "<html>
<h4>Valve</h4>
<h5>Several geometries</h5>
<p>
Calculation of pressure loss for a valve with different geometries at overall flow regime for incompressible and single-phase fluid flow in dependence of its opening.
<a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.PressureLoss.Valve.dp_severalGeometryOverall\">See more information</a>.
</p>

</html>"));
        end Valve;
      end PressureLoss;

      package Utilities "Package for utilities (should not be used directly)"
        extends Modelica.Icons.UtilitiesPackage;

        package SharedDocumentation "Shared Documentation"
          extends Modelica.Icons.Information;

          package HeatTransfer
            import Modelica;
            extends Modelica.Icons.Information;

            package Channel
              extends Modelica.Icons.Information;

              class kc_evenGapLaminar
                extends Modelica.Icons.Information;
                annotation(
                  Documentation(info = "<html>
<p>
Calculation of the mean convective heat transfer coefficient <b> kc </b> for a laminar fluid flow through an even gap at different fluid flow and heat transfer situations.
</p>
<h4>Functions <b>kc_evenGapLaminar</b> and <b>kc_evenGapLaminar_KC</b></h4>
<p>
There are basically three differences:
</p>
<ul>
<li>
The function <b>kc_evenGapLaminar</b>  is using <b>kc_evenGapLaminar_KC</b> but offers additional output variables like e.g. Reynolds number or Nusselt number and failure status (an output of <b>1</b> means that the function is not valid for the inputs).</li>
<li>
Generally the  function <b>kc_evenGapLaminar_KC</b> is numerically best used for the calculation of the mean convective heat transfer coefficient <b> kc </b> at known mass flow rate.</li>
<li>
You can perform an inverse calculation from <b>kc_evenGapLaminar_KC</b>, where an unknown mass flow rate is calculated out of a given mean convective heat transfer coefficient <b> kc </b>
</li>
</ul>


<h4>Restriction</h4>
<ul>
<li> laminar regime (Reynolds number &le; 2200)</li>
<li> developed fluid flow
   <ul>
   <li> heat transfer from one side of the gap (target=Modelica.Fluid.Dissipation.Utilities.Types.kc_evenGap.DevOne)</li>
   <li> heat transfer from both sides of the gap (target=Modelica.Fluid.Dissipation.Utilities.Types.kc_evenGap.DevBoth)</li>
   </ul></li>
<li> undeveloped fluid flow
   <ul>
   <li> heat transfer from one side of the gap (target=Modelica.Fluid.Dissipation.Utilities.Types.kc_evenGap.UndevOne)
       <ul>
       <li> Prandtl number 0.1 &le; Pr &le; 10 </li>
       </ul></li>
   <li> heat transfer from both sides of the gap (target=Modelica.Fluid.Dissipation.Utilities.Types.kc_evenGap.UndevBoth)
       <ul>
       <li> Prandtl number 0.1 &le; Pr &le; 1000</li>
       </ul></li>
   </ul></li>
</ul>

<h4>Geometry</h4>
<p>
<img src=\"modelica://Modelica/Resources/Images/Fluid/Dissipation/heatTransfer/channel/pic_gap.png\" alt=\"pic_gap\"/>
</p>

<h4>Calculation</h4>
<p>
The mean convective heat transfer coefficient <b> kc</b> for an even gap is calculated through the corresponding Nusselt number <b> Nu_lam</b> according to <i>[VDI 2002, p. Gb 7, eq. 43]</i> :
</p>
<pre>
    Nu_lam = [(Nu_1)^3 + (Nu_2)^3 + (Nu_3)^3]^(1/3)
</pre>

<p>
with the corresponding mean convective heat transfer coefficient <b> kc </b> :
</p>

<pre>
    kc =  Nu_lam * lambda / d_hyd
</pre>

<p>
with
</p>

<table>
<tr><td><b> cp                      </b></td><td> as specific heat capacity at constant pressure [J/(kg.K)],</td></tr>
<tr><td><b> d_hyd = 2*s             </b></td><td> as hydraulic diameter of gap [m],</td></tr>
<tr><td><b> eta                     </b></td><td> as dynamic viscosity of fluid [Pa.s],</td></tr>
<tr><td><b> h                       </b></td><td> as height of cross sectional area in gap [m],</td></tr>
<tr><td><b> kc                      </b></td><td> as mean convective heat transfer coefficient [W/(m2.K)],</td></tr>
<tr><td><b> lambda                  </b></td><td> as heat conductivity of fluid [W/(m.K)],</td></tr>
<tr><td><b> L                       </b></td><td> as overflowed length of gap (normal to cross sectional area) [m] ,</td></tr>
<tr><td><b> Nu_lam                  </b></td><td> as mean Nusselt number [-], </td></tr>
<tr><td><b> Pr = eta*cp/lambda      </b></td><td> as Prandtl number [-],</td></tr>
<tr><td><b> rho                     </b></td><td> as fluid density [kg/m3],</td></tr>
<tr><td><b> s                       </b></td><td> as distance between parallel plates of cross sectional area [m],</td></tr>
<tr><td><b> Re = rho*v*d_hyd/eta    </b></td><td> as Reynolds number [-],</td></tr>
<tr><td><b> v                       </b></td><td> as mean velocity in gap [m/s].</td></tr>
</table>

<p>
The summands for the mean Nusselt number <b> Nu_lam </b> at a chosen fluid flow and heat transfer situation are calculated as follows:
</p>
<ul>
<li> developed fluid flow
   <ul>
   <li> heat transfer from one side of the gap (target=1)
        <ul>
            <li> Nu_1 = 4.861</li>
            <li> Nu_2 = 1.841*(Re*Pr*d_hyd/L)^(1/3)</li>
            <li> Nu_3 = 0</li>
        </ul></li>
   <li> heat transfer from both sides of the gap (target=2)
        <ul>
            <li> Nu_1 = 7.541</li>
            <li> Nu_2 = 1.841*(Re*Pr*d_hyd/L)^(1/3)</li>
            <li> Nu_3 = 0</li>
        </ul></li>
   </ul></li>
<li> undeveloped fluid flow
   <ul>
   <li> heat transfer from one side of the gap (target=3)
        <ul>
            <li> Nu_1 = 4.861</li>
            <li> Nu_2 = 1.841*(Re*Pr*d_hyd/L)^(1/3)</li>
            <li> Nu_3 = [2/(1+22*Pr)]^(1/6)*(Re*Pr*d_hyd/L)^(1/2)</li>
        </ul></li>
   <li> heat transfer from both sides of the gap (target=4)
        <ul>
            <li> Nu_1 = 7.541</li>
            <li> Nu_2 = 1.841*(Re*Pr*d_hyd/L)^(1/3)</li>
            <li> Nu_3 = [2/(1+22*Pr)]^(1/6)*(Re*Pr*d_hyd/L)^(1/2)</li>
        </ul></li>
   </ul></li>
</ul>

<p>
Note that the fluid properties shall be calculated with an arithmetic mean temperature out of the fluid flow temperatures at the entrance and the exit of the gap.
</p>

<h4>Verification</h4>
<p>
The mean Nusselt number <b> Nu_lam </b> representing the mean convective heat transfer coefficient <b> kc </b> in dependence of
the chosen fluid flow and heat transfer situations (targets) is shown in the figure below.
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Fluid/Dissipation/heatTransfer/channel/fig_channel_kc_evenGapLaminar.png\" alt=\"fig_channel_kc_evenGapLamina\"/>
</p>

<h4>References</h4>
<dl>
<dt>Bejan,A.:</dt>
    <dd><b>Heat transfer handbook</b>.
    Wiley, 2003.</dd>
<dt>VDI:</dt>
    <dd><b>VDI - W&auml;rmeatlas: Berechnungsbl&auml;tter f&uuml;r den W&auml;rme&uuml;bergang</b>.
    Springer Verlag, 9th edition, 2002.</dd>
</dl>
</html>"));
              end kc_evenGapLaminar;

              class kc_evenGapOverall
                extends Modelica.Icons.Information;
                annotation(
                  Documentation(info = "<html>
<p>
Calculation of the mean convective heat transfer coefficient <b> kc </b> for an laminar or turbulent fluid flow through an even gap at different fluid flow and heat transfer situations.
</p>

<h4>Functions <b>kc_evenGapOverall</b> and <b>kc_evenGapOverall_KC</b></h4>
<p>
There are basically three differences:
</p>
<ul>
<li>
The function <b>kc_evenGapOverall</b> is using <b>kc_evenGapOverall_KC</b> but offers additional output variables like e.g. Reynolds number or Nusselt number and failure status (an output of <b>1</b> means that the function is not valid for the inputs).</li>
<li>
Generally the  function <b>kc_evenGapOverall_KC</b> is numerically best used for the calculation of the mean convective heat transfer coefficient <b> kc </b> at known mass flow rate.</li>
<li>
You can perform an inverse calculation from <b>kc_evenGapOverall_KC</b>, where an unknown mass flow rate is calculated out of a given mean convective heat transfer coefficient <b> kc </b>
</li>
</ul>

<h4>Restriction</h4>

<ul>
<li> developed fluid flow
   <ul>
   <li> heat transfer from one side of the gap (target=Modelica.Fluid.Dissipation.Utilities.Types.kc_evenGap.DevOne) </li>
   <li> heat transfer from both sides of the gap (target=Modelica.Fluid.Dissipation.Utilities.Types.kc_evenGap.DevBoth) </li>
   </ul></li>
<li> undeveloped fluid flow
   <ul>
   <li> heat transfer from one side of the gap (target=Modelica.Fluid.Dissipation.Utilities.Types.kc_evenGap.UndevOne)
       <ul>
       <li> Prandtl number 0.1 &le; Pr &le; 10 </li>
       </ul></li>
   <li> heat transfer from both sides of the gap (target=Modelica.Fluid.Dissipation.Utilities.Types.kc_evenGap.UndevBoth)
   <ul>
       <li> Prandtl number 0.1 &le; Pr &le; 1000 </li>
       </ul></li>
   </ul></li>
<li> turbulent regime always calculated for developed fluid flow and heat transfer from both sides of the gap (target=Modelica.Fluid.Dissipation.Utilities.Types.kc_evenGap.DevBoth)</li>
</ul>

<h4>Geometry and Calculation </h4>

<p>This heat transfer function enables a calculation of heat transfer coefficient for laminar and turbulent flow regime. The geometry, constant and fluid parameters of the function are the same as for
<a href=\"modelica://Modelica.Fluid.Dissipation.HeatTransfer.Channel.kc_evenGapLaminar\">kc_evenGapLaminar</a>  and <a href=\"modelica://Modelica.Fluid.Dissipation.HeatTransfer.Channel.kc_evenGapTurbulent\">kc_evenGapTurbulent</a>.
</p>
<p>
The calculation conditions for laminar and turbulent flow is equal to the calculation in <b>kc_evenGapLaminar</b>
and <b>kc_evenGapTurbulent</b>. A smooth transition between both functions is carried out between 2200 &le; Re &le; 30000 (see figure below). </p>

<h4>Verification</h4>
<p>
The mean Nusselt number <b> Nu </b> representing the mean convective heat transfer coefficient <b> kc </b> for Prandtl numbers of different fluids in dependence of
the chosen fluid flow and heat transfer situations (targets) is shown in the figures below.
</p>

<ul>
   <li> Target 1: Developed fluid flow and heat transfer from one side of the gap</li>
   <li> Target 2: Developed fluid flow and heat transfer from both sides of the gap</li>
   <li> Target 3: Undeveloped fluid flow and heat transfer from one side of the gap</li>
   <li> Target 4: Undeveloped fluid flow and heat transfer from both sides of the gap</li>
</ul>
<p>
The verification for all targets is shown in the following figure w.r.t. the reference:
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Fluid/Dissipation/heatTransfer/channel/fig_channel_kc_evenGapOverall.png\" alt=\"fig_channel_kc_evenGapOverall\"/>
</p>

<h4>References</h4>
<dl>
<dt>Bejan,A.:</dt>
    <dd><b>Heat transfer handbook</b>.
    Wiley, 2003.</dd>
<dt>VDI:</dt>
    <dd><b>VDI - W&auml;rmeatlas: Berechnungsbl&auml;tter f&uuml;r den W&auml;rme&uuml;bergang</b>.
    Springer Verlag, 9th edition, 2002.</dd>
</dl>

</html>"));
              end kc_evenGapOverall;

              class kc_evenGapTurbulent
                extends Modelica.Icons.Information;
                annotation(
                  Documentation(info = "<html>
<p>
Calculation of the mean convective heat transfer coefficient <b> kc </b> for a developed turbulent fluid flow through an even gap at heat transfer from both sides.
</p>
<h4>Functions <b>kc_evenGapTurbulent</b> and <b>kc_evenGapTurbulent_KC</b></h4>
<p>
There are basically three differences:
</p>
<ul>
<li>
The function <b>kc_evenGapTurbulent</b> is using <b>kc_evenGapTurbulent_KC</b> but offers additional output variables like e.g. Reynolds number or Nusselt number and failure status (an output of <b>1</b> means that the function is not valid for the inputs).</li>
<li>
Generally the  function <b>kc_evenGapTurbulent_KC</b> is numerically best used for the calculation of the mean convective heat transfer coefficient <b> kc </b> at known mass flow rate.</li>
<li>
You can perform an inverse calculation from <b>kc_evenGapTurbulent_KC</b>, where an unknown mass flow rate is calculated out of a given mean convective heat transfer coefficient <b> kc </b>
</li>
</ul>

<h4>Restriction</h4>

<ul>
<li> identical and constant wall temperatures</li>
<li> hydraulic diameter per gap length (d_hyd / L) &le; 1</li>
<li> 0.5 &le; Prandtl number Pr &le; 100) </li>
<li> turbulent regime (3e4 &le; Reynolds number &le; 1e6)</li>
<li> developed fluid flow</li>
<li> heat transfer from both sides of the gap (target = Modelica.Fluid.Dissipation.Utilities.Types.kc_evenGap.DevBoth)</li>
</ul>

<h4>Geometry</h4>
<p>
<img src=\"modelica://Modelica/Resources/Images/Fluid/Dissipation/heatTransfer/channel/pic_gap.png\" alt=\"pic_gap\"/>
</p>

<h4>Calculation</h4>
<p>
The mean convective heat transfer coefficient <b> kc</b> for an even gap is calculated through the corresponding Nusselt number <b> Nu_turb</b> according to <i> Gnielinski in [VDI 2002, p. Gb 7, sec. 2.4]</i>
</p>
<pre>
    Nu_turb =(zeta/8)*Re*Pr/{1+12.7*[zeta/8]^(0.5)*[Pr^(2/3) -1]}*{1+[d_hyd/L]^(2/3)}
</pre>

<p>
where the pressure loss coefficient <b> zeta </b> according to <i> Konakov in [VDI 2002, p. Ga 5, eq. 27]</i> is determined by
</p>
<pre>
    zeta =  1/[1.8*log10(Re) - 1.5]^2
</pre>

<p>
resulting to the corresponding mean convective heat transfer coefficient <b> kc </b>
</p>
<pre>
    kc =  Nu_turb * lambda / d_hyd
</pre>

<p>
with
</p>

<table>
<tr><td><b> cp                      </b></td><td> as specific heat capacity at constant pressure [J/(kg.K)],</td></tr>
<tr><td><b> d_hyd = 2*s             </b></td><td> as hydraulic diameter of gap [m],</td></tr>
<tr><td><b> eta                     </b></td><td> as dynamic viscosity of fluid [Pa.s],</td></tr>
<tr><td><b> h                       </b></td><td> as height of cross sectional area in gap [m],</td></tr>
<tr><td><b> kc                      </b></td><td> as mean convective heat transfer coefficient [W/(m2.K)],</td></tr>
<tr><td><b> lambda                  </b></td><td> as heat conductivity of fluid [W/(m.K)],</td></tr>
<tr><td><b> L                       </b></td><td> as overflowed length of gap (normal to cross sectional area) [m] ,</td></tr>
<tr><td><b> Nu_turb                 </b></td><td> as mean Nusselt number for turbulent regime [-], </td></tr>
<tr><td><b> Pr = eta*cp/lambda      </b></td><td> as Prandtl number [-],</td></tr>
<tr><td><b> rho                     </b></td><td> as fluid density [kg/m3],</td></tr>
<tr><td><b> s                       </b></td><td> as distance between parallel plates of cross sectional area [m],</td></tr>
<tr><td><b> Re = rho*v*d_hyd/eta    </b></td><td> as Reynolds number [-],</td></tr>
<tr><td><b> v                       </b></td><td> as mean velocity in gap [m/s],</td></tr>
<tr><td><b> zeta                    </b></td><td> as pressure loss coefficient [-].</td></tr>
</table>

<p>
Note that the fluid flow properties shall be calculated with an arithmetic mean temperature out of the fluid flow temperatures at the entrance and the exit of the gap.
</p>

<h4>Verification</h4>
<p>
The mean Nusselt number <b> Nu_turb </b> representing the mean convective heat transfer coefficient <b> kc </b> in dependence of
the chosen fluid flow and heat transfer situations (targets) is shown in the figure below.
</p>
<ul>
   <li> Target 2: Developed fluid flow and heat transfer from both sides of the gap</li>
</ul>

<p>
<img src=\"modelica://Modelica/Resources/Images/Fluid/Dissipation/heatTransfer/channel/fig_channel_kc_evenGapTurbulent.png\" alt=\"fig_channel_kc_evenGapTurbulent\"/>
</p>

<h4>References</h4>
<dl>
<dt>VDI:</dt>
    <dd><b>VDI - W&auml;rmeatlas: Berechnungsbl&auml;tter f&uuml;r den W&auml;rme&uuml;bergang</b>.
    Springer Verlag, 9th edition, 2002.</dd>
</dl>

</html>"));
              end kc_evenGapTurbulent;
            end Channel;

            package General
              extends Modelica.Icons.Information;

              class kc_approxForcedConvection
                extends Modelica.Icons.Information;
                annotation(
                  Documentation(info = "<html>
<p>
Approximate calculation of the mean convective heat transfer coefficient <b> kc </b> for forced convection with a fully developed fluid flow in a turbulent regime.
</p>

<h4>Functions <b>kc_approxForcedConvection</b> and <b>kc_approxForcedConvection_KC</b></h4>
<p>
There are basically three differences:
</p>
<ul>
<li>
The function <b>kc_approxForcedConvection</b> is using <b>kc_approxForcedConvection_KC</b> but offers additional output variables like e.g. Reynolds number or Nusselt number and failure status (an output of <b>1</b> means that the function is not valid for the inputs).</li>
<li>
Generally the  function <b>kc_approxForcedConvection_KC</b> is numerically best used for the calculation of the mean convective heat transfer coefficient <b> kc </b> at known mass flow rate.</li>
<li>
You can perform an inverse calculation from <b>kc_approxForcedConvection_KC</b>, where an unknown mass flow rate is calculated out of a given mean convective heat transfer coefficient <b> kc </b>
</li>
</ul>

<h4>Restriction</h4>

<ul>
<li> Constant wall temperature <b> or </b> constant heat flux
<li> Turbulent regime (Reynolds number 2500 &lt; Re &lt; 1e6)
<li> Prandtl number 0.5 &le; Pr &le; 500
</ul>

<h4>Calculation</h4>
<p>
The mean convective heat transfer coefficient <b> kc </b> is approximated through different Nusselt number <b> Nu </b> correlations out of <i>[Bejan 2003, p. 424 ff]</i>. <br>
Roughest approximation according to Dittus/Boelter (1930):
</p>
<pre>
    Nu_1 = 0.023 * Re^(4/5) * Pr^(exp_Pr)
</pre>

<p>
Middle approximation according to Sieder/Tate (1936) considering the temperature dependence of the fluid properties:
</p>
<pre>
    Nu_2 = 0.023 * Re^(4/5) * Pr^(1/3) * (eta/eta_wall)^(0.14)
</pre>

<p>
Finest approximation according to Gnielinski (1976):
</p>
<pre>
    Nu_3 = 0.0214 * [Re^(0.8) - 100] * Pr^(0.4) for Pr &le; 1.5
         = 0.012 * [Re^(0.87) - 280] * Pr^(0.4) for Pr &gt; 1.5
</pre>

<p>
The mean convective heat transfer coefficient <b> kc </b> is calculated by:
</p>

<pre>
    kc =  Nu * lambda / d_hyd
</pre>

<p>
with
</p>

<table>
<tr><td><b> eta                          </b></td><td> as dynamic viscosity of fluid [Pa.s],</td></tr>
<tr><td><b> eta_wall                     </b></td><td> as dynamic viscosity of fluid at wall temperature [Pa.s],</td></tr>
<tr><td><b> exp_Pr                       </b></td><td> as exponent for Prandtl number w.r.t. Dittus/Boelter (0.4 for heating or 0.3 for cooling) [-],</td></tr>
<tr><td><b> kc                           </b></td><td> as mean convective heat transfer coefficient [W/(m2.K)],</td></tr>
<tr><td><b> lambda                       </b></td><td> as heat conductivity of fluid [W/(m.K)],</td></tr>
<tr><td><b> d_hyd                        </b></td><td> as hydraulic diameter [m],</td></tr>
<tr><td><b> Nu_1/2/3                     </b></td><td> as mean Nusselt number [-], </td></tr>
<tr><td><b> Pr                           </b></td><td> as Prandtl number [-],</td></tr>
<tr><td><b> Re                           </b></td><td> as Reynolds number [-].</td></tr>
</table>

<h4>Verification</h4>
<p>
The mean Nusselt number <b> Nu </b> representing the mean convective heat transfer coefficient <b> kc </b> for Prandtl numbers of different fluids is shown in the figure below. </p>
<p><b>Dittus/Boelter</b> (target = Modelica.Fluid.Dissipation.Utilities.Types.kc_general.Rough)
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Fluid/Dissipation/heatTransfer/general/fig_kc_approxForcedConvection_T1.png\" alt=\"fig_kc_approxForcedConvection_T1\"/>
</p>

<p>
<b>Sieder/Tate</b> (Target = Modelica.Fluid.Dissipation.Utilities.Types.kc_general.Middle)
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Fluid/Dissipation/heatTransfer/general/fig_kc_approxForcedConvection_T2.png\" alt=\"fig_kc_approxForcedConvection_T2\"/>
</p>

<p>
<b>Gnielinski</b> (Target = Modelica.Fluid.Dissipation.Utilities.Types.kc_general.Finest)
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Fluid/Dissipation/heatTransfer/general/fig_kc_approxForcedConvection_T3.png\" alt=\"fig_kc_approxForcedConvection_T3\"/>
</p>

<p>
Note that all fluid properties shall be calculated with the mean temperature of the fluid between the entrance and the outlet of the generic device.
</p>

<h4>References</h4>
<dl>
<dt>Bejan,A.:</dt>
    <dd><b>Heat transfer handbook</b>.
    Wiley, 2003.</dd>
</dl>

</html>"));
              end kc_approxForcedConvection;
            end General;

            package HeatExchanger
              extends Modelica.Icons.Information;

              class kc_flatTube
                extends Modelica.Icons.Information;
                annotation(
                  Documentation(info = "<html>
<p>
Calculation of the mean convective heat transfer coefficient <b> kc </b> for the air-side heat transfer of heat exchangers with flat tubes and several fin geometries.
</p>
<h4>Functions <b>kc_flatTube</b> and <b>kc_flatTube_KC</b></h4>
<p>
There are basically three differences:
</p>
<ul>
<li>
The function <b>kc_flatTube</b> is using <b>kc_flatTube_KC</b> but offers additional output variables like e.g. Reynolds number or Nusselt number and failure status (an output of <b>1</b> means that the function is not valid for the inputs).</li>
<li>
Generally the  function <b>kc_flatTube_KC</b> is numerically best used for the calculation of the mean convective heat transfer coefficient <b> kc </b> at known mass flow rate.</li>
<li>
You can perform an inverse calculation from <b>kc_flatTube_KC</b>, where an unknown mass flow rate is calculated out of a given mean convective heat transfer coefficient <b> kc </b>
</li>
</ul>

<h4>Restriction</h4>
<ul>
<li> According to the kind of fin geometry the calculation is valid in a range of <b> Re</b> from 100 to 5000. </li>
<li> medium = air </li>
</ul>

<h4>Geometry </h4>

<p>
<img src=\"modelica://Modelica/Resources/Images/Fluid/Dissipation/heatTransfer/heatExchanger/pic_flatTube.png\" width=\"850\" alt=\"pic_flatTube\"/>
</p>

<h4>Calculation</h4>
<p>
The mean convective heat transfer coefficient <b> kc </b> for heat exchanger is calculated through the corresponding Coulburn factor <b> j </b>:
</p>
<pre>
    j = f(geometry, Re)
</pre>

<p>
with the resulting mean convective heat transfer coefficient <b> kc </b>
</p>

<pre>
    kc =  j * Re_L_p * Pr^(1/3) * lambda / L_p (Louver fin)
</pre>

 <p>
or
</p>

<pre>
    kc =  j * Re_D_h * Pr^(1/3) * lambda / D_h (Rectangular offset strip fin)
</pre>

<p>
with
</p>

<table>
<tr><td><b> D_h                        </b></td><td> as hydraulic diameter [m],</td></tr>
<tr><td><b> kc                         </b></td><td> as mean convective heat transfer coefficient [W/(m2K)],</td></tr>
<tr><td><b> lambda                     </b></td><td> as heat conductivity of fluid [W/(mK)],</td></tr>
<tr><td><b> L_p                        </b></td><td> as louver pitch [m],</td></tr>
<tr><td><b> Nu_D_h = kc*D_h/lambda     </b></td><td> as mean Nusselt number based on hydraulic diameter [-], </td></tr>
<tr><td><b> Nu_L_p = kc*L_p/lambda     </b></td><td> as mean Nusselt number based on louver pitch [-], </td></tr>
<tr><td><b> Pr = eta*cp/lambda         </b></td><td> as Prandtl number [-],</td></tr>
<tr><td><b> Re_D_h = rho*v*D_h/eta     </b></td><td> as Reynolds number based on hydraulic diameter [-],</td></tr>
<tr><td><b> Re_L_p = rho*v*L_p/eta     </b></td><td> as Reynolds number based on louver pitch [-],</td></tr>
</table>

<h4>Verification</h4>
<p>
The mean Nusselt number <b> Nu </b> representing the mean convective heat transfer coefficient <b> kc </b> is shown below for different fin geometries at similar dimensions.
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Fluid/Dissipation/heatTransfer/heatExchanger/fig_flatTube_kc.png\" alt=\"fig_flatTube_kc\"/>
</p>

<h4>References</h4>
<dl>
 <dt>Y.-J. CHANG and C.-C. WANG:</dt>
    <dd><b>A generalized heat transfer correlation for louver fin geometry</b>.
    In International Journal of Heat and Mass Transfer, volume 40, No. 3,  pages 533-544, 1997.</dd>
 <dt>Y.-J. CHANG and C.-C. WANG:</dt>
    <dd><b>Air Side Performance of Brazed Aluminium Heat Exchangers</b>.
    In Journal of Enhanced Heat Transfer, volume 3, No. 1,  pages 15-28, 1996.</dd>
 <dt>R.-M. Manglik, A.-E. Bergles:</dt>
    <dd><b>Heat Transfer and Pressure Drop Correlations for the Rectangular Offset Strip Fin Compact Heat Exchanger</b>.
    In Experimental Thermal and Fluid Science, volume 10, pages 171-180, 1995.</dd>
</dl>

</html>"));
              end kc_flatTube;

              class kc_roundTube
                extends Modelica.Icons.Information;
                annotation(
                  Documentation(info = "<html>
<p>
Calculation of the mean convective heat transfer coefficient <b> kc </b> for the air-side heat transfer of heat exchangers with round tubes and several fin geometries.
</p>
<h4>Functions <b>kc_roundTube</b> and <b>kc_roundTube_KC</b></h4>
<p>
There are basically three differences:
</p>
<ul>
<li>
The function <b>kc_roundTube</b> is using <b>kc_roundTube_KC</b> but offers additional output variables like e.g. Reynolds number or Nusselt number and failure status (an output of <b>1</b> means that the function is not valid for the inputs).</li>
<li>
Generally the  function <b>kc_roundTube_KC</b> is numerically best used for the calculation of the mean convective heat transfer coefficient <b> kc </b> at known mass flow rate.</li>
<li>
You can perform an inverse calculation from <b>kc_roundTube_KC</b>, where an unknown mass flow rate is calculated out of a given mean convective heat transfer coefficient <b> kc </b>
</li>
</ul>

<h4>Restriction</h4>
<ul>
<li> According to the kind of fin geometry the calculation is valid in a range of <b> Re</b> from 300 to 8000. </li>
<li> medium = air </li>
</ul>

<h4>Geometry </h4>
<p>
<img src=\"modelica://Modelica/Resources/Images/Fluid/Dissipation/heatTransfer/heatExchanger/pic_roundTube.png\" width=\"850\" alt=\"pic_roundTube\"/>
</p>

<h4>Calculation</h4>
<p>
The mean convective heat transfer coefficient <b> kc </b> for heat exchanger is calculated through the corresponding Coulburn factor <b> j </b>:
</p>

<pre>
    j = f(geometry, Re)
</pre>

<p>
with the resulting mean convective heat transfer coefficient <b> kc </b>
</p>

<pre>
    kc =  j * Re * Pr^(1/3) * lambda / D_c
</pre>

<p>
with
</p>

<table>
<tr><td><b> D_c                        </b></td><td> as fin collar diameter [m],</td></tr>
<tr><td><b> kc                         </b></td><td> as mean convective heat transfer coefficient [W/(m2K)],</td></tr>
<tr><td><b> lambda                     </b></td><td> as heat conductivity of fluid [W/(mK)],</td></tr>
<tr><td><b> Nu = kc*D_c/lambda         </b></td><td> as mean Nusselt number [-], </td></tr>
<tr><td><b> Pr = eta*cp/lambda         </b></td><td> as Prandtl number [-],</td></tr>
<tr><td><b> Re = rho*v*D_c/eta         </b></td><td> as Reynolds number [-],</td></tr>
</table>

<h4>Verification</h4>
<p>
The mean Nusselt number <b> Nu </b> representing the mean convective heat transfer coefficient <b> kc </b> is shown below for different fin geometries at similar dimensions.
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Fluid/Dissipation/heatTransfer/heatExchanger/fig_roundTube_kc.png\" alt=\"fig_roundTube_kc\"/>
</p>

<h4>References</h4>
<dl>
 <dt>C.-C. Wang, C.-T. Chang:</dt>
    <dd><b>Heat and mass transfer for plate fin-and-tube heat exchangers, with and without hydrophilic coating</b>.
    In International Journal of Heat and Mass Transfer, volume 41, pages 3109-3120, 1998.</dd>
 <dt>C.-C. Wang, C.-J. Lee, C.-T. Chang, S.-P. Lina:</dt>
    <dd><b>Heat transfer and friction correlation for compact louvered fin-and-tube heat exchangers</b>.
    In International Journal of Heat and Mass Transfer, volume 42, pages 1945-1956, 1999.</dd>
 <dt>C.-C. Wang, W.-H. Tao, C.-J. Chang:</dt>
    <dd><b>An investigation of the airside performance of the slit fin-and-tube heat exchangers</b>.
    In International Journal of Refrigeration, volume 22, pages 595-603, 1999.</dd>
 <dt>C.-C. Wang, W.-L. Fu, C.-T. Chang:</dt>
    <dd><b>Heat Transfer and Friction Characteristics of Typical Wavy Fin-and-Tube Heat Exchangers</b>.
    In Experimental Thermal and Fluid Science, volume 14, pages 174-186, 1997.</dd>
</dl>
</html>"));
              end kc_roundTube;
            end HeatExchanger;

            package HelicalPipe
              extends Modelica.Icons.Information;

              class kc_laminar
                extends Modelica.Icons.Information;
                annotation(
                  Documentation(info = "<html>
<p>
Calculation of the mean convective heat transfer coefficient <b> kc </b> for a helical pipe in the laminar flow regime.
</p>
<h4>Functions <b>kc_laminar</b> and <b>kc_laminar_KC</b></h4>
<p>
There are basically three differences:
</p>
<ul>
<li>
The function <b>kc_laminar</b> is using <b>kc_laminar_KC</b> but offers additional output variables like e.g. Reynolds number or Nusselt number and failure status (an output of <b>1</b> means that the function is not valid for the inputs).</li>
<li>
Generally the  function <b>kc_laminar_KC</b> is numerically best used for the calculation of the mean convective heat transfer coefficient <b> kc </b> at known mass flow rate.</li>
<li>
You can perform an inverse calculation from <b>kc_laminar_KC</b>, where an unknown mass flow rate is calculated out of a given mean convective heat transfer coefficient <b> kc </b>
</li>
</ul>

<h4>Restriction</h4>

<ul>
  <li> laminar regime (Reynolds number &le; critical Reynolds number <b> Re_crit</b>) </li>
  <li> neglect influence of heat transfer direction (heating/cooling) according to <i>Sieder and Tate </i> </li>
</ul>

<p>
The critical Reynolds number <b> Re_crit </b> in a helical pipe depends on its mean curvature diameter <b> d_coil </b>.
The smaller the mean curvature diameter of the helical pipe, the
earlier the turbulent regime will start due to vortexes out of higher
centrifugal forces.
</p>

<h4>Geometry </h4>

<p>
<img src=\"modelica://Modelica/Resources/Images/Fluid/Dissipation/heatTransfer/helicalPipe/pic_helicalPipe.png\" alt=\"pic_helicalPipe\"/>
</p>

<h4>Calculation</h4>
<p>
The mean convective heat transfer coefficient <b> kc </b> for helical pipes is calculated through the corresponding Nusselt number <b> Nu </b> according to <i>[VDI 2002, p. Gc 2, eq. 5]</i> :
</p>

<pre>    Nu = 3.66 + 0.08*[1 + 0.8*(d_hyd/d_coil)^0.9]*Re^m*Pr^(1/3)<br></pre>

<p>
with the exponent <b> m </b> for the Reynolds number
</p>

<pre>    m = 0.5 + 0.2903*(d_hyd/d_coil)^0.194</pre>

<p>
and the resulting mean convective heat transfer coefficient <b> kc </b>
</p>

<pre>    kc =  Nu * lambda / d_hyd</pre>

<p>
with
</p>

<table>

  <tbody>
    <tr>
      <td><b> d_mean                     </b></td>
      <td> as mean diameter of helical pipe [m],</td>
    </tr>

    <tr>
      <td><b> d_coil = f(geometry)       </b></td>
      <td> as mean curvature diameter of helical pipe [m],</td>
    </tr>

    <tr>
      <td><b> d_hyd                      </b></td>
      <td> as hydraulic diameter of the helical pipe [m],</td>
    </tr>

    <tr>
      <td><b> h                          </b></td>
      <td> as slope of helical pipe [m],</td>
    </tr>

    <tr>
      <td><b> kc                         </b></td>
      <td> as mean convective heat transfer coefficient [W/(m2K)],</td>
    </tr>

    <tr>
      <td><b> lambda                     </b></td>
      <td> as heat conductivity of fluid [W/(mK)],</td>
    </tr>

    <tr>
      <td><b> L                          </b></td>
      <td> as total length of helical pipe [m],</td>
    </tr>

    <tr>
      <td><b> Nu = kc*d_hyd/lambda       </b></td>
      <td> as mean Nusselt number [-], </td>
    </tr>

    <tr>
      <td><b> Pr = eta*cp/lambda         </b></td>
      <td> as Prandtl number [-],</td>
    </tr>

    <tr>
      <td><b> Re = rho*v*d_hyd/eta       </b></td>
      <td> as Reynolds number [-],</td>
    </tr>

    <tr>
      <td><b> Re_crit = f(geometry)      </b></td>
      <td> as critical Reynolds number [-].</td>
    </tr>

  </tbody>
</table>

<h4>Verification</h4>
<p>
The mean Nusselt number <b> Nu </b> representing the mean convective heat transfer coefficient <b> kc </b> is shown below for different numbers of turns <b> n_nt </b> at constant total length of the helical pipe.
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Fluid/Dissipation/heatTransfer/helicalPipe/fig_helicalPipe_kc_laminar.png\" alt=\"fig_helicalPipe_kc_laminar\"/>
</p>

<p>The convective heat transfer of a helical pipe is enhanced compared
to a straight pipe due to occurring turbulences resulting out of
centrifugal forces. The higher the number of turns, the better is the
convective heat transfer for the same length of a pipe.
</p>
<p>
Note that the ratio of hydraulic diameter to total length of helical pipe <b> d_hyd/L </b> has no remarkable influence on the coefficient of heat transfer <b> kc </b>.
</p>
<h4>References</h4>
<dl>
 <dt>GNIELINSKI, V.:</dt>
    <dd><b>Heat transfer and pressure drop in helically coiled tubes.</b>.
    In 8th International Heat Transfer Conference, volume 6, pages 2847-2854, Washington,1986. Hemisphere.</dd>
</dl>
</html>"));
              end kc_laminar;

              class kc_overall
                extends Modelica.Icons.Information;
                annotation(
                  Documentation(info = "<html>
<p>
Calculation of the mean convective heat transfer coefficient <b> kc </b> of a helical pipe in a hydrodynamically developed laminar and turbulent flow regime.
</p>
<h4>Functions <b>kc_overall</b> and <b>kc_overall_KC</b></h4>
<p>
There are basically three differences:
</p>
<ul>
<li>
The function <b>kc_overall</b> is using <b>kc_overall_KC</b> but offers additional output variables like e.g. Reynolds number or Nusselt number and failure status (an output of <b>1</b> means that the function is not valid for the inputs).</li>
<li>
Generally the  function <b>kc_overall_KC</b> is numerically best used for the calculation of the mean convective heat transfer coefficient <b> kc </b> at known mass flow rate.</li>
<li>
You can perform an inverse calculation from <b>kc_overall_KC</b>, where an unknown mass flow rate is calculated out of a given mean convective heat transfer coefficient <b> kc </b>
</li>
</ul>

<h4>Geometry and Calculation </h4>

<p>This heat transfer function enables a calculation of heat transfer coefficient for laminar and turbulent flow regime. The geometry, constant and fluid parameters of the function are the same as for
<a href=\"modelica://Modelica.Fluid.Dissipation.HeatTransfer.HelicalPipe.kc_laminar\">kc_laminar</a>  and <a href=\"modelica://Modelica.Fluid.Dissipation.HeatTransfer.HelicalPipe.kc_turbulent\">kc_turbulent</a>.
</p>
<p>
The calculation conditions for laminar and turbulent flow is equal to the calculation in <b>kc_laminar</b>
and <b>kc_turbulent</b>. A smooth transition between both functions is carried out between 2200 &le; Re &le; 30000 (see figure below). </p>

<h4>Verification</h4>
<p>
The mean Nusselt number <b> Nu </b> representing the mean convective heat transfer coefficient <b> kc </b> is shown below for different numbers of turns <b> n_nt </b> at constant total length of the helical pipe.
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Fluid/Dissipation/heatTransfer/helicalPipe/fig_helicalPipe_kc_overall.png\" alt=\"fig_helicalPipe_kc_overall\"/>
</p>

<p>
The convective heat transfer of a helical pipe is enhanced compared to a straight pipe due to occurring turbulences resulting out of centrifugal forces. The higher the number of turns, the better is the convective heat transfer for the same length of a pipe.
</p>

<p>
Note that the ratio of hydraulic diameter to total length of helical pipe <b> d_hyd/L </b> has no remarkable influence on the coefficient of heat transfer <b> kc </b>.
</p>

<h4>References</h4>
<dl>
 <dt>GNIELINSKI, V.:</dt>
    <dd><b>Heat transfer and pressure drop in helically coiled tubes.</b>.
    In 8th International Heat Transfer Conference, volume 6, pages 2847?2854, Washington,1986. Hemisphere.</dd>
</dl>
</html>"));
              end kc_overall;

              class kc_turbulent
                extends Modelica.Icons.Information;
                annotation(
                  Documentation(info = "<html>
<p>
Calculation of the mean convective heat transfer coefficient <b> kc </b> of a helical pipe for turbulent flow regime.
</p>
<h4>Functions <b>kc_turbulent</b> and <b>kc_turbulent_KC</b></h4>

<p>
There are basically three differences:
</p>
<ul>
<li>
The function <b>kc_turbulent</b> is using <b>kc_turbulent_KC</b> but offers additional output variables like e.g. Reynolds number or Nusselt number and failure status (an output of <b>1</b> means that the function is not valid for the inputs).
</li>
<li>
Generally the  function <b>kc_turbulent_KC</b> is numerically best used for the calculation of the mean convective heat transfer coefficient <b> kc </b> at known mass flow rate.</li>
<li>
You can perform an inverse calculation from <b>kc_turbulent_KC</b>, where an unknown mass flow rate is calculated out of a given mean convective heat transfer coefficient <b> kc </b>
</li>
</ul>

<p>
The critical Reynolds number <b> Re_crit </b> in a helical pipe depends on its mean curvature diameter. The smaller the mean curvature diameter of the helical pipe <b> d_mean </b>, the earlier the turbulent regime will start due to vortexes out of higher centrifugal forces.
</p>
<h4>Geometry </h4>

<p>
<img src=\"modelica://Modelica/Resources/Images/Fluid/Dissipation/heatTransfer/helicalPipe/pic_helicalPipe.png\" alt=\"pic_helicalPipe\"/>
</p>

<h4>Calculation</h4>
<p>
The mean convective heat transfer coefficient <b> kc </b> for helical pipes is calculated through the corresponding Nusselt number <b> Nu </b> according to <i>[VDI 2002, p. Ga 2, eq. 6]</i>:
</p>
<pre>
    Nu = (zeta_TOT/8)*Re*Pr/{1 + 12.7*(zeta_TOT/8)^0.5*[Pr^(2/3)-1]},
</pre>

<p>
where the influence of the pressure loss on the heat transfer calculation is considered through
</p>
<pre>
    zeta_TOT = 0.3164*Re^(-0.25) + 0.03*(d_hyd/d_coil)^(0.5) and
</pre>

<p>
and the resulting mean convective heat transfer coefficient <b> kc </b>
</p>

<pre>
    kc =  Nu * lambda / d_hyd
</pre>

<p>
with
</p>

<table>
<tr><td><b> d_mean                     </b></td><td> as mean diameter of helical pipe [m],</td></tr>
<tr><td><b> d_coil = f(geometry)       </b></td><td> as mean curvature diameter of helical pipe [m],</td></tr>
<tr><td><b> d_hyd                      </b></td><td> as hydraulic diameter of the helical pipe [m],</td></tr>
<tr><td><b> h                          </b></td><td> as slope of helical pipe [m],</td></tr>
<tr><td><b> kc                         </b></td><td> as mean convective heat transfer coefficient [W/(m2K)],</td></tr>
<tr><td><b> lambda                     </b></td><td> as heat conductivity of fluid [W/(mK)],</td></tr>
<tr><td><b> L                          </b></td><td> as total length of helical pipe [m],</td></tr>
<tr><td><b> Nu = kc*d_hyd/lambda       </b></td><td> as mean Nusselt number [-], </td></tr>
<tr><td><b> Pr = eta*cp/lambda         </b></td><td> as Prandtl number [-],</td></tr>
<tr><td><b> Re = rho*v*d_hyd/eta       </b></td><td> as Reynolds number [-],</td></tr>
<tr><td><b> Re_crit = f(geometry)      </b></td><td> as critical Reynolds number [-].</td></tr>
</table>

<h4>Verification</h4>
<p>
The mean Nusselt number <b> Nu </b> representing the mean convective heat transfer coefficient <b> kc </b> is shown below for different numbers of turns <b> n_nt </b> at constant total length of the helical pipe.
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Fluid/Dissipation/heatTransfer/helicalPipe/fig_helicalPipe_kc_turbulent.png\" alt=\"fig_helicalPipe_kc_turbulent\"/>
</p>

<p>
The convective heat transfer of a helical pipe is enhanced compared to a straight pipe due to occurring turbulences resulting out of centrifugal forces. The higher the number of turns, the better is the convective heat transfer for the same length of a pipe.
</p>

<p>
Note that the ratio of hydraulic diameter to total length of helical pipe <b> d_hyd/L </b> has no remarkable influence on the coefficient of heat transfer <b> kc </b>.
</p>

<h4>References</h4>
<dl>
 <dt>GNIELINSKI, V.:</dt>
    <dd><b>Heat transfer and pressure drop in helically coiled tubes.</b>.
    In 8th International Heat Transfer Conference, volume 6, pages 2847?2854, Washington,1986. Hemisphere.</dd>
</dl>
</html>"));
              end kc_turbulent;
            end HelicalPipe;

            package Plate
              extends Modelica.Icons.Information;

              class kc_laminar
                extends Modelica.Icons.Information;
                annotation(
                  Documentation(info = "<html>
<p>
Calculation of the mean convective heat transfer coefficient <b> kc </b> for a laminar fluid flow over an even surface.
</p>

<h4>Functions <b>kc_laminar</b> and <b>kc_laminar_KC</b></h4>
<p>
There are basically three differences:
</p>

<ul>
<li>
The function <b>kc_laminar</b> is using <b>kc_laminar_KC</b> but offers additional output variables like e.g. Reynolds number or Nusselt number and failure status (an output of <b>1</b> means that the function is not valid for the inputs).</li>
<li>
Generally the  function <b>kc_laminar_KC</b> is numerically best used for the calculation of the mean convective heat transfer coefficient <b> kc </b> at known mass flow rate.</li>
<li>
You can perform an inverse calculation from <b>kc_laminar_KC</b>, where an unknown mass flow rate is calculated out of a given mean convective heat transfer coefficient <b> kc </b>
</li>
</ul>

<h4>Restriction</h4>

<ul>
<li> laminar regime (Reynolds number &le; 1e5)</li>
<li> Prandtl number 0.6 &le; Pr &le; 2000</li>
</ul>

<h4>Geometry </h4>

<p>
<img src=\"modelica://Modelica/Resources/Images/Fluid/Dissipation/heatTransfer/plate/pic_plate.png\" alt=\"pic_plate\"/>
</p>

<h4>Calculation</h4>
<p>
The mean convective heat transfer coefficient <b> kc</b> for flat plate is calculated through the corresponding Nusselt number <b> Nu_lam</b> according to <i>[VDI 2002, p. Gd 1, eq. 1]</i> :
</p>
<pre>
    Nu_lam = 0.664 * Re^(0.5) * (Pr)^(1/3)
</pre>

<p>
and the corresponding mean convective heat transfer coefficient <b> kc </b>:
</p>
<pre>
    kc =  Nu_lam * lambda / L
</pre>

<p>
with
</p>

<table>
<tr><td><b> cp                 </b></td><td> as specific heat capacity at constant pressure [J/(kg.K)],</td></tr>
<tr><td><b> eta                </b></td><td> as dynamic viscosity of fluid [Pa.s],</td></tr>
<tr><td><b> kc                 </b></td><td> as mean convective heat transfer coefficient [W/(m2.K)],</td></tr>
<tr><td><b> lambda         </b></td><td> as heat conductivity of fluid [W/(m.K)],</td></tr>
<tr><td><b> L              </b></td><td> as length of plate [m],</td></tr>
<tr><td><b> Nu_lam                  </b></td><td> as mean Nusselt number for laminar regime [-], </td></tr>
<tr><td><b> Pr = eta*cp/lambda           </b></td><td> as Prandtl number [-],</td></tr>
<tr><td><b> rho          </b></td><td> as fluid density [kg/m3],</td></tr>
<tr><td><b> Re = rho*v*L/eta    </b></td><td> as Reynolds number [-].</td></tr>
</table>

<h4>Verification</h4>
<p>
The mean Nusselt number <b> Nu </b> in the laminar regime representing the mean convective heat transfer coefficient <b> kc </b> for Prandtl numbers of different fluids is shown in the figure below.
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Fluid/Dissipation/heatTransfer/plate/fig_plate_kc_laminar.png\" width=\"600\" alt=\"fig_plate_kc_laminar\"/>
</p>

<p>
Note that this function is best used in the laminar regime up to a Reynolds number <b> Re </b> smaller than 2300. There is a deviation w.r.t. literature due to the neglect of the turbulence influence in the transition regime even though this function is used inside its cited restrictions for a higher Reynolds number. The function
<a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.HeatTransfer.Plate.kc_overall\">kc_overall</a> is recommended for the simulation of a Reynolds number higher than 2300.
</p>
<h4>References</h4>
<dl>
<dt>VDI:</dt>
    <dd><b>VDI - W&auml;rmeatlas: Berechnungsbl&auml;tter f&uuml;r den W&auml;rme&uuml;bergang</b>.
    Springer Verlag, 9th edition, 2002.</dd>
</dl>

</html>"));
              end kc_laminar;

              class kc_overall
                extends Modelica.Icons.Information;
                annotation(
                  Documentation(info = "<html>
<p>
Calculation of the mean convective heat transfer coefficient <b> kc </b> for a laminar or turbulent fluid flow over an even surface.
</p>
<h4>Functions <b>kc_overall</b> and <b>kc_overall_KC</b></h4>
<p>
There are basically three differences:
</p>
<ul>
<li>
The function <b>kc_overall</b> is using <b>kc_overall_KC</b> but offers additional output variables like e.g. Reynolds number or Nusselt number and failure status (an output of <b>1</b> means that the function is not valid for the inputs).</li>
<li>
Generally the  function <b>kc_overall_KC</b> is numerically best used for the calculation of the mean convective heat transfer coefficient <b> kc </b> at known mass flow rate.</li>
<li>
You can perform an inverse calculation from <b>kc_overall_KC</b>, where an unknown mass flow rate is calculated out of a given mean convective heat transfer coefficient <b> kc </b>
</li>
</ul>

<h4>Restriction</h4>
<ul>
<li> constant wall temperature</li>
<li> overall regime (Reynolds number 1e1 &lt; Re &lt; 1e7)</li>
<li> Prandtl number 0.6 &le; Pr &le; 2000</li>
</ul>

<h4>Geometry and Calculation </h4>
<p>This heat transfer function enables a calculation of heat transfer coefficient for laminar and turbulent flow regime. The geometry, constant and fluid parameters of the function are the same as for
<a href=\"modelica://Modelica.Fluid.Dissipation.HeatTransfer.Plate.kc_laminar\">kc_laminar</a>  and <a href=\"modelica://Modelica.Fluid.Dissipation.HeatTransfer.Plate.kc_turbulent\">kc_turbulent</a>.
</p>
<p>
The calculation conditions for laminar and turbulent flow is equal to the calculation in <b>kc_laminar</b>
and <b>kc_turbulent</b>. A smooth transition between both functions is carried out between 1e5 &le; Re &le; 5e5 (see figure below).
</p>
<h4>Verification</h4>
<p>
The mean Nusselt number <b> Nu = sqrt(Nu_lam^2 + Nu_turb^2) </b> representing the mean convective heat transfer coefficient <b> kc </b> for Prandtl numbers of different fluids is shown in the figure below.
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Fluid/Dissipation/heatTransfer/plate/fig_plate_kc_overall.png\" alt=\"fig_plate_kc_overall\"/>
</p>

<h4>References</h4>
<dl>
<dt>VDI:</dt>
    <dd><b>VDI - W&auml;rmeatlas: Berechnungsbl&auml;tter f&uuml;r den W&auml;rme&uuml;bergang</b>.
    Springer Verlag, 9th edition, 2002.</dd>
</dl>

</html>"));
              end kc_overall;

              class kc_turbulent
                extends Modelica.Icons.Information;
                annotation(
                  Documentation(info = "<html>
<p>
Calculation of the mean convective heat transfer coefficient <b> kc </b> for a hydrodynamically developed turbulent fluid flow over an even surface.
</p>
<h4>Functions <b>kc_turbulent</b> and <b>kc_turbulent_KC</b></h4>
<p>
There are basically three differences:
</p>
<ul>
<li>
The function <b>kc_turbulent</b> is using <b>kc_turbulent_KC</b> but offers additional output variables like e.g. Reynolds number or Nusselt number and failure status (an output of <b>1</b> means that the function is not valid for the inputs).</li>
<li>
Generally the  function <b>kc_turbulent_KC</b> is numerically best used for the calculation of the mean convective heat transfer coefficient <b> kc </b> at known mass flow rate.</li>
<li>
You can perform an inverse calculation from <b>kc_turbulent_KC</b>, where an unknown mass flow rate is calculated out of a given mean convective heat transfer coefficient <b> kc </b>
</li>
</ul>

<h4>Restriction</h4>
<ul>
<li> constant wall temperature</li>
<li> turbulent regime (Reynolds number 5e5 &lt; Re &lt; 1e7)</li>
<li> Prandtl number 0.6 &le; Pr &le; 2000</li>
</ul>

<h4>Geometry </h4>

<p>
<img src=\"modelica://Modelica/Resources/Images/Fluid/Dissipation/heatTransfer/plate/pic_plate.png\" alt=\"pic_plate\"/>
</p>

<h4>Calculation</h4>
<p>
The mean convective heat transfer coefficient <b> kc</b> for flat plate is calculated through the corresponding Nusselt number <b> Nu_turb</b> according to <i>[VDI 2002, p. Gd 1, eq. 2]</i>:
</p>

<pre>
    Nu_turb = (0.037 * Re^0.8 * Pr) / (1 + 2.443/Re^0.1 * (Pr^(2/3)-1))
</pre>

<p>
and the corresponding mean convective heat transfer coefficient <b> kc </b>:
</p>

<pre>
    kc =  Nu_turb * lambda / L
</pre>

<p>
with
</p>

<table>
<tr><td><b> cp          </b></td><td> as specific heat capacity at constant pressure [J/(kg.K)],</td></tr>
<tr><td><b> eta          </b></td><td> as dynamic viscosity of fluid [Pa.s],</td></tr>
<tr><td><b> kc          </b></td><td> as mean convective heat transfer coefficient [W/(m2.K)],</td></tr>
<tr><td><b> lambda         </b></td><td> as heat conductivity of fluid [W/(m.K)],</td></tr>
<tr><td><b> L              </b></td><td> as length of plate [m],</td></tr>
<tr><td><b> Nu_turb                 </b></td><td> as mean Nusselt number for turbulent regime [-], </td></tr>
<tr><td><b> Pr = eta*cp/lambda           </b></td><td> as Prandtl number [-],</td></tr>
<tr><td><b> rho          </b></td><td> as fluid density [kg/m3],</td></tr>
<tr><td><b> Re = v*rho*L/eta    </b></td><td> as Reynolds number [-].</td></tr>
</table>

<h4>Verification</h4>
<p>
The mean Nusselt number in turbulent regime <b> Nu </b> representing the mean convective heat transfer coefficient <b> kc </b> for Prandtl numbers of different fluids is shown in the figure below.
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Fluid/Dissipation/heatTransfer/plate/fig_plate_kc_turbulent.png\" alt=\"fig_plate_kc_turbulent\"/>
</p>


<h4>References</h4>
<dl>

<dt>VDI:</dt>
    <dd><b>VDI - W&auml;rmeatlas: Berechnungsbl&auml;tter f&uuml;r den W&auml;rme&uuml;bergang</b>.
    Springer Verlag, 9th edition, 2002.</dd>
</dl>

</html>"));
              end kc_turbulent;
            end Plate;

            package StraightPipe
              extends Modelica.Icons.Information;

              class kc_laminar
                extends Modelica.Icons.Information;
                annotation(
                  Documentation(info = "<html>
<p>
Calculation of mean convective heat transfer coefficient <b> kc </b> of a straight pipe at an uniform wall temperature <b> or </b> uniform heat flux <b>and</b> for a hydrodynamically developed <b>or</b> undeveloped laminar fluid flow.
</p>
<h4>Functions <b>kc_laminar</b> and <b>kc_laminar_KC</b></h4>
<p>
There are basically three differences:
</p>
<ul>
<li>
The function <b>kc_laminar</b> is using <b>kc_laminar_KC</b> but offers additional output variables like e.g. Reynolds number or Nusselt number and failure status (an output of <b>1</b> means that the function is not valid for the inputs).</li>
<li>
Generally the  function <b>kc_laminar_KC</b> is numerically best used for the calculation of the mean convective heat transfer coefficient <b> kc </b> at known mass flow rate.</li>
<li>
You can perform an inverse calculation from <b>kc_laminar_KC</b>, where an unknown mass flow rate is calculated out of a given mean convective heat transfer coefficient <b> kc </b>
</li>
</ul>

<h4>Restriction</h4>
<ul>
<li> circular cross sectional area</li>
<li> uniform wall temperature (UWT) <b> or </b> uniform heat flux (UHF)</li>
<li> hydrodynamically developed fluid flow (DFF) <b> or </b> hydrodynamically undeveloped fluid flow (UFF)</li>
<li> 0.6 &le; Prandtl number &le; 1000</li>
<li> laminar regime (Reynolds number &le; 2000)</li>
</ul>

<h4>Geometry </h4>

<p>
<img src=\"modelica://Modelica/Resources/Images/Fluid/Dissipation/pressureLoss/straightPipe/pic_straightPipe.png\" alt=\"pic_straightPipe\"/>
</p>

<h4>Calculation</h4>
<p>
The mean convective heat transfer coefficient <b> kc </b> of a straight pipe in the laminar regime can be calculated for the following four heat transfer boundary conditions through its corresponding Nusselt number <b> Nu </b>:
</p>
<p>
<b> Uniform wall temperature in developed fluid flow (heatTransferBoundary == Modelica.Fluid.Dissipation.Utilities.Types.HeatTransferBoundary.UWTuDFF)</b> according to <i>[VDI 2002, p. Ga 2, eq. 6]</i> :
</p>

<pre>
    Nu_TD = [3.66^3 + 0.7^3 + {1.615*(Re*Pr*d_hyd/L)^1/3 - 0.7}^3]^1/3
</pre>

<p>
<b> Uniform heat flux in developed fluid flow (heatTransferBoundary == Modelica.Fluid.Dissipation.Utilities.Types.HeatTransferBoundary.UHFuDFF)</b> according to <i>[VDI 2002, p. Ga 4, eq. 19]</i> :
</p>
<pre>
    Nu_qD = [4.364^3 + 0.6^3 + {1.953*(Re*Pr*d_hyd/L)^1/3 - 0.6}^3]^1/3
</pre>

<p>
<b> Uniform wall temperature in undeveloped fluid flow (heatTransferBoundary == Modelica.Fluid.Dissipation.Utilities.Types.HeatTransferBoundary.UWTuUFF)</b> according to <i>[VDI 2002, p. Ga 2, eq. 12]</i> :
</p>
<pre>
    Nu_TU = [3.66^3 + 0.7^3 + {1.615*(Re*Pr*d_hyd/L)^1/3 - 0.7}^3 + {(2/[1+22*Pr])^1/6*(Re*Pr*d_hyd/L)^0.5}^3]^1/3
</pre>

<p>
<b> Uniform heat flux in developed fluid flow (heatTransferBoundary == Modelica.Fluid.Dissipation.Utilities.Types.HeatTransferBoundary.UHFuUFF)</b> according to <i>[VDI 2002, p. Ga 5, eq. 25]</i> :
</p>

<pre>
    Nu_qU = [4.364^3 + 0.6^3 + {1.953*(Re*Pr*d_hyd/L)^1/3 - 0.6}^3 + {0.924*Pr^1/3*[Re*d_hyd/L]^0.5}^3]^1/3.
</pre>

<p>
The corresponding mean convective heat transfer coefficient <b> kc </b> is determined w.r.t. the chosen heat transfer boundary by:
</p>

<pre>
    kc =  Nu * lambda / d_hyd
</pre>

<p>
with
</p>

<table>
<tr><td><b> d_hyd                    </b></td><td> as hydraulic diameter of straight pipe [m],</td></tr>
<tr><td><b> kc                       </b></td><td> as mean convective heat transfer coefficient [W/(m2K)],</td></tr>
<tr><td><b> lambda                   </b></td><td> as heat conductivity of fluid [W/(mK)],</td></tr>
<tr><td><b> L                        </b></td><td> as length of straight pipe [m],</td></tr>
<tr><td><b> Nu = kc*d_hyd/lambda     </b></td><td> as mean Nusselt number [-], </td></tr>
<tr><td><b> Pr = eta*cp/lambda       </b></td><td> as Prandtl number [-],</td></tr>
<tr><td><b> Re = rho*v*d_hyd/eta     </b></td><td> as Reynolds number [-],</td></tr>
<tr><td><b> v                        </b></td><td> as mean velocity [m/s].</td></tr>
</table>

<h4>Verification</h4>
<p>The mean Nusselt number <b> Nu </b> representing the mean convective heat transfer coefficient <b> kc </b> depending on four different heat transfer boundary conditions is shown in the figures below.
</p>
<p>
This verification has been done with the fluid properties of Water (Prandtl number Pr = 7) and a diameter to pipe length fraction of 0.1.
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Fluid/Dissipation/heatTransfer/straightPipe/fig_straightPipe_kc_laminar.png\" alt=\"fig_straightPipe_kc_laminar\"/>
</p>

<h4>References</h4>
<dl>
 <dt>Bejan,A.:</dt>
    <dd><b>Heat transfer handbook</b>.
    Wiley, 2003.</dd>
 <dt>VDI:</dt>
    <dd><b>VDI - W&auml;rmeatlas: Berechnungsbl&auml;tter f&uuml;r den W&auml;rme&uuml;bergang</b>.
    Springer Verlag, 9th edition, 2002.</dd>
</dl>

</html>"));
              end kc_laminar;

              class kc_overall
                extends Modelica.Icons.Information;
                annotation(
                  Documentation(info = "<html>
<p>
Calculation of mean convective heat transfer coefficient <b> kc </b> of a straight pipe at an uniform wall temperature <b> or </b> uniform heat flux <b> and </b> for a hydrodynamically developed <b>or</b> undeveloped laminar or turbulent fluid flow with neglect <b> or </b> consideration of pressure loss influence.
</p>
<h4>Functions <b>kc_overall</b> and <b>kc_overall_KC</b></h4>
<p>
There are basically three differences:
</p>
<ul>
<li>
The function <b>kc_overall</b> is using <b>kc_overall_KC</b> but offers additional output variables like e.g. Reynolds number or Nusselt number and failure status (an output of <b>1</b> means that the function is not valid for the inputs).</li>
<li>
Generally the  function <b>kc_overall_KC</b> is numerically best used for the calculation of the mean convective heat transfer coefficient <b> kc </b> at known mass flow rate.</li>
<li>
You can perform an inverse calculation from <b>kc_overall_KC</b>, where an unknown mass flow rate is calculated out of a given mean convective heat transfer coefficient <b> kc </b>
</li>
</ul>

<h4>Restriction</h4>
<ul>
<li> circular cross sectional area </li>
<li> uniform wall temperature (UWT) <b> or </b> uniform heat flux (UHF) </li>
<li> hydrodynamically developed fluid flow </li>
<li> hydraulic diameter / length &le; 1 </li>
<li> 0.6 &le; Prandtl number &le; 1000 </li>
</ul>

<h4>Geometry and Calculation </h4>

<p>This heat transfer function enables a calculation of heat transfer coefficient for laminar and turbulent flow regime. The geometry, constant and fluid parameters of the function are the same as for
<a href=\"modelica://Modelica.Fluid.Dissipation.HeatTransfer.HelicalPipe.kc_laminar\">kc_laminar</a>  and <a href=\"modelica://Modelica.Fluid.Dissipation.HeatTransfer.HelicalPipe.kc_turbulent\">kc_turbulent</a>.
</p>
<p>
The calculation conditions for laminar and turbulent flow is equal to the calculation in <b>kc_laminar</b>
and <b>kc_turbulent</b>. A smooth transition between both functions is carried out between 2200 &le; Re &le; 10000 (see figure below). </p>

<h4>Verification</h4>
<p>
The mean Nusselt number <b> Nu </b> representing the mean convective heat transfer coefficient <b> kc </b> is shown for the fluid properties of Water (Prandtl number Pr = 7) and a diameter to pipe length fraction of 0.1 in the figure below.
</p>

<p>
The following verification considers pressure loss influence (roughness =2).
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Fluid/Dissipation/heatTransfer/straightPipe/fig_straightPipe_kc_overall.png\" alt=\"fig_straightPipe_kc_overall\"/>
</p>

<h4>References</h4>
<dl>
 <dt>Bejan,A.:</dt>
    <dd><b>Heat transfer handbook</b>.
    Wiley, 2003.</dd>
 <dt>VDI:</dt>
    <dd><b>VDI - W&auml;rmeatlas: Berechnungsbl&auml;tter f&uuml;r den W&auml;rme&uuml;bergang</b>.
    Springer Verlag, 9th edition, 2002.</dd>
</dl>

</html>"));
              end kc_overall;

              class kc_turbulent
                extends Modelica.Icons.Information;
                annotation(
                  Documentation(info = "<html>
<p>
Calculation of mean convective heat transfer coefficient <b> kc </b> of a straight pipe for a hydrodynamically developed turbulent fluid flow at uniform wall temperature <b> or </b> uniform heat flux with neglecting <b> or </b> considering of pressure loss influence.
</p>
<h4>Functions <b>kc_turbulent</b> and <b>kc_turbulent_KC</b></h4>
<p>
There are basically three differences:
</p>
<ul>
<li>
The function <b>kc_turbulent</b> is using <b>kc_turbulent_KC</b> but offers additional output variables like e.g. Reynolds number or Nusselt number and failure status (an output of <b>1</b> means that the function is not valid for the inputs).</li>
<li>
Generally the  function <b>kc_turbulent_KC</b> is numerically best used for the calculation of the mean convective heat transfer coefficient <b> kc </b> at known mass flow rate.</li>
<li>
You can perform an inverse calculation from <b>kc_turbulent_KC</b>, where an unknown mass flow rate is calculated out of a given mean convective heat transfer coefficient <b> kc </b>
</li>
</ul>

<h4>Restriction</h4>

<ul>
<li> circular cross sectional area </li>
<li> hydrodynamically developed fluid flow </li>
<li> hydraulic diameter / length &le; 1 </li>
<li> 0.6 &le; Prandtl number &le; 1000 </li>
<li> turbulent flow regime (1e4 &le; Reynolds number &le; 1e6) </li>
</ul>

<h4>Geometry </h4>

<p>
<img src=\"modelica://Modelica/Resources/Images/Fluid/Dissipation/pressureLoss/straightPipe/pic_straightPipe.png\" alt=\"pic_straightPipe\"/>
</p>

<h4>Calculation</h4>
<p>
<b>Neglect pressure loss influence (roughness == 1):</b>
</p>
<p>
The mean convective heat transfer coefficient <b> kc </b> for smooth straight pipes is calculated through its corresponding Nusselt number <b> Nu </b> according to <i> [Dittus and Boelter in Bejan 2003, p. 424, eq. 5.76]</i>
</p>

<pre>
    Nu = 0.023 * Re^(4/5) * Pr^(1/3).
</pre>

<p>
<b>Consider pressure loss influence (roughness == 2):</b>
</p>

<p>
The mean convective heat transfer coefficient <b> kc </b> for rough straight pipes is calculated through its corresponding Nusselt number <b> Nu </b> according to <i>[Gnielinski in VDI 2002, p. Ga 5, eq. 26]</i>
</p>

<pre>
    Nu = (zeta/8)*Re*Pr/(1 + 12.7*(zeta/8)^0.5*(Pr^(2/3)-1))*(1+(d_hyd/L)^(2/3)),
</pre>

<p>
where the influence of the pressure loss on the heat transfer calculation is considered through
</p>

<pre>
    zeta =  (1.8*log10(Re)-1.5)^-2.
</pre>

<p>
The mean convective heat transfer coefficient <b> kc </b> in dependence of the chosen calculation (neglecting or considering of pressure loss influence) results into:
</p>

<pre>
    kc =  Nu * lambda / d_hyd
</pre>

<p>
with
</p>

<table>
<tr><td><b> d_hyd                      </b></td><td> as hydraulic diameter of straight pipe [m],</td></tr>
<tr><td><b> kc                         </b></td><td> as mean convective heat transfer coefficient [W/(m2K)],</td></tr>
<tr><td><b> lambda                     </b></td><td> as heat conductivity of fluid [W/(mK)],</td></tr>
<tr><td><b> L                          </b></td><td> as length of straight pipe [m],</td></tr>
<tr><td><b> Nu = kc*d_hyd/lambda       </b></td><td> as mean Nusselt number [-], </td></tr>
<tr><td><b> Pr = eta*cp/lambda         </b></td><td> as Prandtl number [-],</td></tr>
<tr><td><b> Re = rho*v*d_hyd/eta       </b></td><td> as Reynolds number [-],</td></tr>
<tr><td><b> v                          </b></td><td> as mean velocity [m/s],</td></tr>
<tr><td><b> zeta                       </b></td><td> as pressure loss coefficient [-]. </td></tr>
</table>

<p>
Note that there is no significant difference for the calculation of the mean Nusselt number <b> Nu </b> at a uniform wall temperature (UWT) or a uniform heat flux (UHF) as heat transfer boundary in the turbulent regime (Bejan 2003, p.303).
</p>

<h4>Verification</h4>
<p>
The mean Nusselt number <b> Nu </b> representing the mean convective heat transfer coefficient <b> kc </b> for Prandtl numbers of different fluids is shown in the figures below.
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Fluid/Dissipation/heatTransfer/straightPipe/fig_straightPipe_kc_turbulent.png\" alt=\"fig_straightPipe_kc_turbulent\"/>
</p>

<p>
Note that the higher the Prandtl number <b> Pr </b> there is a higher difference in Nusselt numbers <b> Nu </b> comparing the neglect and consideration of pressure loss.
</p>

<h4>References</h4>
<dl>
  <dt>Bejan,A.:</dt>
    <dd><b>Heat transfer handbook</b>.
    Wiley, 2003.</dd>
 <dt>VDI:</dt>
    <dd><b>VDI - W&auml;rmeatlas: Berechnungsbl&auml;tter f&uuml;r den W&auml;rme&uuml;bergang</b>.
    Springer Verlag, 9th edition, 2002.</dd>
</dl>

</html>"));
              end kc_turbulent;

              class kc_twoPhaseOverall
                extends Modelica.Icons.Information;
                annotation(
                  Documentation(info = "<html>
<p>
Calculation of local <b>two phase</b> heat transfer coefficient <b>kc_2ph </b>for (horizontal/vertical) <b>boiling</b> or (horizontal) <b>condensation</b> for an overall flow regime.
</p>

<h4>Restriction</h4>
<ul>
  <li>circular cross sectional area </li>
  <li>no subcooled boiling </li>
  <li>film condensation </li>
</ul>

<h4>Geometry </h4>

<p>
<img src=\"modelica://Modelica/Resources/Images/Fluid/Dissipation/pressureLoss/straightPipe/pic_straightPipe.png\" alt=\"pic_straightPipe\"/>
</p>

<h4>Calculation</h4>
<p><b>Boiling in a horizontal pipe (target = Modelica.Fluid.Dissipation.Utilities.Types.TwoPhaseHeatTransferTarget.BoilHor):</b></p>
<p>
The local two phase heat transfer coefficient <b> kc_2ph </b> during boiling in a <b>horizontal</b> straight pipe for an overall regime is calculated according to <i>[Gungor/Winterton 1986, p.354, eq. 2]</i> :
</p>

<pre>
    kc_2ph = E_fc*E_fc_hor*kc_fc+S_nb+S_nb_hor*kc_nb
</pre>

<p>
with
</p>

<table>
<tr><td><b> Bo</b>=qdot_A/(mdot_A*dh_lv)  </td><td> as boiling number [-],</td></tr>
<tr><td><b> dh_lv                         </b></td><td> as evaporation enthalpy [J/kg],</td></tr>
<tr><td><b> E_fc</b>=f(Bo,Fr_l,X_tt)      </td><td> as forced convection enhancement factor [-],</td></tr>
<tr><td><b> E_fc_hor</b> =f(Fr_l)         </td><td> as forced convection enhancement factor for horizontal straight pipes [-],</td></tr>
<tr><td><b> Fr_l                          </b></td><td> as Froude number assuming total mass flow rate flowing as liquid [-],</td></tr>
<tr><td><b> kc_2ph                        </b></td><td> as local two phase heat transfer coefficient [W/(m2K)],</td></tr>
<tr><td><b> kc_fc                         </b></td><td> as heat transfer coefficient considering forced convection [W/(m2K)],</td></tr>
<tr><td><b> kc_nb                         </b></td><td> as heat transfer coefficient considering nucleate boiling [W/(m2K)],</td></tr>
<tr><td><b> mdot_A                        </b></td><td> as total mass flow rate density [kg/(m2s)],</td></tr>
<tr><td><b> qdot_A                        </b></td><td> as heat flow rate density [W/m2],</td></tr>
<tr><td><b> Re_l                          </b></td><td> as Reynolds number assuming liquid mass flow rate flowing alone [-],</td></tr>
<tr><td><b> S_nb</b> =f(E_fc,Re_l)        </td><td> as suppression factor of nucleate boiling [-],</td></tr>
<tr><td><b> S_nb_hor</b> =f(Fr_l)         </td><td> as suppression factor of nucleate boiling for horizontal straight pipes [-],</td></tr>
<tr><td><b> x_flow                        </b></td><td> as mass flow rate quality [-],</td></tr>
<tr><td><b> X_tt</b> = f(x_flow)          </td><td> as Martinelli parameter [-].</td></tr>
</table>

<p><b>Boiling in a vertical pipe (target = Modelica.Fluid.Dissipation.Utilities.Types.TwoPhaseHeatTransferTarget.BoilVer):</b></p>
<p>
The local two phase heat transfer coefficient <b> kc_2ph </b> during boiling in a <b>vertical</b> straight pipe for an overall regime is calculated out of the correlations for boiling in a horizontal straight pipe, where the horizontal correction factors <b> E_fc_hor,S_nb_hor</b> are unity.
</p>
<p>
Please note that the correlations named above are not valid for subcooled boiling due to a different driving temperature for nucleate boiling and forced convection. At subcooled boiling there is no enhancement factor (no vapour generation) but the suppression factor remains effective.
</p>

<p><b>Condensation in a horizontal pipe (target = Modelica.Fluid.Dissipation.Utilities.Types.TwoPhaseHeatTransferTarget.CondHor):</b></p>
<p>
The local two phase heat transfer coefficient <b> kc_2ph </b> during condensation in a <b>horizontal</b> straight pipe for an overall regime is calculated according to <i>[Shah 1979, p.548, eq. 8]</i> :
</p>

<pre>
  kc_2ph = kc_1ph*[(1 - x_flow)^0.8 + 3.8*x_flow^0.76*(1 - x_flow)^0.04/p_red^0.38]
</pre>

<p>
where the convective heat transfer coefficient <b> kc_1ph </b> assuming the total mass flow rate is flowing as liquid according to <i>[Shah 1979, p.548, eq. 5]</i> :
</p>

<pre>
  kc_1ph = 0.023*Re_l^0.8*Pr_l^0.4*lambda_l/d_hyd
 </pre>

<p>
with
</p>

<table>
<tr><td><b> d_hyd                        </b></td><td> as hydraulic diameter [m],</td></tr>
<tr><td><b> kc_2ph                       </b></td><td> as local two phase heat transfer coefficient [W/(m2K)],</td></tr>
<tr><td><b> kc_1ph                       </b></td><td> as convective heat transfer coefficient assuming total mass flow rate is flowing as liquid [W/(m2K)],</td></tr>
<tr><td><b> lambda_l                     </b></td><td> as thermal conductivity of fluid  [W/(mK)],</td></tr>
<tr><td><b> pressure                     </b></td><td> as thermodynamic pressure of fluid [Pa],</td></tr>
<tr><td><b> p_crit                       </b></td><td> as critical pressure of fluid [Pa],</td></tr>
<tr><td><b> p_red</b> = pressure/p_crit  </td><td> as reduced pressure [-],</td></tr>
<tr><td><b> Pr_l                         </b></td><td> as Prandtl number assuming [-],</td></tr>
<tr><td><b> Re_l                         </b></td><td> as Reynolds number assuming <b>total</b> mass flow rate is flowing as liquid [-],</td></tr>
<tr><td><b> x_flow                       </b></td><td> as mass flow rate quality [-],</td></tr>
</table>

<h4>Verification</h4>
<p>The local two phase heat transfer coefficient <b>kc_2ph </b> during for horizontal and vertical boiling as well as for horizontal condensation is shown for a straight pipe in the figures below.</p>

<p><b>Boiling in a horizontal pipe (target = Modelica.Fluid.Dissipation.Utilities.Types.TwoPhaseHeatTransferTarget.BoilVer):</b></p>
<p>
Here the validation of the two phase heat transfer coefficient is shown for boiling in a horizontal straight pipe.
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Fluid/Dissipation/heatTransfer/straightPipe/fig_kc_twoPhaseOverall_KC_4.png\" alt=\"fig_kc_twoPhaseOverall\"/>
</p>

<p>The two phase heat transfer coefficient (<b>kc_2ph </b>) w.r.t. <i>Gungor/Winterton</i> is shown in dependence of the mass flow rate quality (<b>x_flow </b>) for different mass flow rate densities (<b>mdot_A </b>). The validation has been done with measurement results from <i>Kattan/Thome</i> for R134a as medium.</p>
<p>
The two phase heat transfer coefficient increases with increasing mass flow rate quality up to a maximum value. After that there is a rapid decrease of (<b>kc_2ph </b>) with increasing (<b>x_flow </b>). This can be explained with a partial dryout of the pipe wall for a high mass flow rate quality.
</p>

<p><b>Condensation in a horizontal pipe (target = Modelica.Fluid.Dissipation.Utilities.Types.TwoPhaseHeatTransferTarget.CondHor):</b></p>
<p>
Here the validation of the two phase heat transfer coefficient is shown for condensation in a horizontal straight pipe.
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Fluid/Dissipation/heatTransfer/straightPipe/fig_kc_twoPhaseOverall_KC_2.png\" alt=\"fig_kc_twoPhaseOverall\"/>
</p>

<p>The two phase heat transfer coefficient (<b>kc_2ph </b>) w.r.t. <i>Shah</i> is shown in dependence of the mass flow rate quality (<b>x_flow </b>) for different mass flow rate densities (<b>mdot_A </b>). The validation has been done with measurement results from <i>Dobson/Chato</i> for R134a as medium.</p>

<h4>References</h4>
<dl><dt>Bejan,A.: </dt>
<dd><b>Heat transfer handbook</b>. Wiley, 2003. </dd>
<dt>M.K. Dobson and J.C. Chato: </dt>
<dd><b>Condensation in smooth horizontal tubes</b>. Journal of HeatTransfer, Vol.120, p.193-213, 1998. </dd>
<dt>Gungor, K.E. and R.H.S. Winterton: </dt>
<dd><b>A general correlation for flow boiling in tubes and annuli</b>, Int.J. Heat Mass Transfer, Vol.29, p.351-358, 1986. </dd>
<dt>N. Kattan and J.R. Thome: </dt>
<dd><b>Flow boiling in horizontal pipes: Part 2 - new heat transfer data for five refrigerants.</b>. Journal of Heat Transfer, Vol.120. p.148-155, 1998. </dd>
<dt>Shah, M.M.: </dt>
<dd><b>A general correlation for heat transfer during film condensation inside pipes</b>. Int. J. Heat Mass Transfer, Vol.22, p.547-556, 1979.</dd>
</dl></html>"));
              end kc_twoPhaseOverall;
            end StraightPipe;
          end HeatTransfer;

          package PressureLoss
            extends Modelica.Icons.Information;

            package Bend
              extends Modelica.Icons.Information;

              class dp_curvedOverall
                extends Modelica.Icons.Information;
                annotation(
                  Documentation(info = "<html>
<p>
Calculation of pressure loss in curved bends at overall flow regime for incompressible and single-phase fluid flow through circular cross sectional area considering surface roughness.
</p>
<h4>Restriction</h4>
<p>
This function shall be used inside of the restricted limits according to the referenced literature.
</p>

<ul>
<li><b>circular cross sectional area </b></li>
<li><b>0.5 &le; curvature radius / diameter &le; 3 </b><i>[Idelchik 2006, p. 357, diag. 6-1] </i></li>
<li><b>length of bend along curved axis / diameter &ge; 10 </b><i>[Idelchik 2006, p. 357, diag. 6-1] </i></li>
<li><b>angle of curvature smaller than 180&deg; (delta &le; 180) </b><i>[Idelchik 2006, p. 357, diag. 6-1] </i></li>
</ul>

<h4>Geometry</h4>
<p>
<img src=\"modelica://Modelica/Resources/Images/Fluid/Dissipation/pressureLoss/bend/pic_circularBend.png\" alt=\"pic_circularBend\"/>
</p>

<h4>Calculation</h4>
<p>
The pressure loss <b>dp </b>for curved bends is determined by:
</p>
<pre>
    dp = zeta_TOT * (rho/2) * velocity^2
</pre>
<p>
with
</p>
<table cellspacing=\"2\" cellpadding=\"0\" border=\"0\"><tr>
<td><p><b>rho </b></p></td>
<td><p>as density of fluid [kg/m3],</p></td>
</tr>
<tr>
<td><p><b>velocity </b></p></td>
<td><p>as mean velocity [m/s],</p></td>
</tr>
<tr>
<td><p><b>zeta_TOT </b></p></td>
<td><p>as pressure loss coefficient [-].</p></td>
</tr>
</table>
<p>
<b>Curved bends with relative curvature radius R_0/d_hyd &le; 3 </b>according to <i>[Idelchik 2006, p. 357, diag. 6-1]</i>
</p>
<p>
The pressure loss of curved bends is similar to its calculation in straight pipes. There are three different flow regimes observed (laminar,transition,turbulent). The turbulent regime is further separated into sections with a dependence or independence of the local resistance coefficient (<b>zeta_LOC </b>) on Reynolds number. The local resistance coefficient (<b>zeta_LOC</b>) of a curved bend is calculated in dependence of the flow regime as follows:
</p>
<ul>
<li><p><b>Laminar regime (Re &le; Re_lam_leave)</b>: </p>
<pre>      zeta_LOC = A2/Re + A1*B1*C1</pre></li>
<li><p><b>Transition regime (Re_lam_leave &le; 4e4)</b></p>
<p> This calculation is done using a smoothing function interpolating between the laminar and the first turbulent flow regime.</p></li>
<li><p><b>Turbulent regime (4e4 &le; 3e5) with dependence </b> of local resistance coefficient on Reynolds number: </p>
<pre>      zeta_LOC = k_Re * (A1*B1*C1)</pre>
<p>where <b>k_Re</b> depends on the relative curvature radius <b>R_0/d_hyd </b></p>
<pre>
      k_Re = 1 + 4400/Re              for 0.50 &lt; r/d_hyd &lt; 0.55
      k_Re = 5.45/Re^(0.118)          for 0.55 &le; r/d_hyd &lt; 0.70
      k_Re = 11.5/Re^(0.19)           for 0.70 &le; r/d_hyd &lt; 3.00</pre></li>
<li><p><b>Turbulent regime (Re &ge; 3e5) with independence </b>of local resistance coefficient on Reynolds number </p>
<pre>      zeta_LOC = A1*B1*C1</pre></li>
</ul>

<p>with </p>
<table cellspacing=\"2\" cellpadding=\"0\" border=\"0\"><tr>
<td><p><b>A1 </b></p></td>
<td><p>as coefficient considering effect of angle of turning (delta) [-],</p></td>
</tr>
<tr>
<td><p><b>A2 </b></p></td>
<td><p>as coefficient considering effect for laminar regime [-],</p></td>
</tr>
<tr>
<td><p><b>B1 </b></p></td>
<td><p>as coefficient considering effect of relative curvature radius (R_0/d_hyd) [-],</p></td>
</tr>
<tr>
<td><p><b>C1=1 </b></p></td>
<td><p>as coefficient considering relative elongation of cross sectional area (here: circular cross sectional area) [-],</p></td>
</tr>
<tr>
<td><p><b>k_Re </b></p></td>
<td><p>as coefficient considering influence of laminar regime in transition regime [-],</p></td>
</tr>
<tr>
<td><p><b>Re </b></p></td>
<td><p>as Reynolds number [-].</p></td>
</tr>
</table>
<p><br/><br/>The pressure loss coefficient <b>zeta_TOT </b>of a curved bend including pressure loss due to friction is determined by its local resistance coefficient <b>zeta_LOC </b>multiplied with a correction factor <b>CF </b>for surface roughness according to <i>[Miller, p. 209, eq. 9.4]:</i> </p>
<pre>    zeta_TOT = CF*zeta_LOC </pre>
<p>where the correction factor <b>CF </b>is determined from the Darcy friction factor of a straight pipe having the bend flow path length </p>
<pre>    CF = 1 + (lambda_FRI_rough * pi * delta/d_hyd) / zeta_LOC</pre>
<p>and the Darcy friction factors <b>lambda_FRI_rough </b>is calculated with an approximated Colebrook-White law according to <i>[Miller, p. 191, eq. 8.4]:</i> </p>
<pre>    lambda_FRI_rough = 0.25*(lg(K/(3.7*d_hyd) + 5.74/Re^0.9))^-2</pre>
<p>with </p>
<table cellspacing=\"2\" cellpadding=\"0\" border=\"0\"><tr>
<td><p><b>delta </b></p></td>
<td><p>as curvature radiant [rad],</p></td>
</tr>
<tr>
<td><p><b>d_hyd </b></p></td>
<td><p>as hydraulic diameter [m],</p></td>
</tr>
<tr>
<td><p><b>K</b> </p></td>
<td><p>as absolute roughness (average height of surface asperities) [m],</p></td>
</tr>
<tr>
<td><p><b>lambda_FRI_rough </b></p></td>
<td><p>as Darcy friction factor[-],</p></td>
</tr>
<tr>
<td><p><b>Re </b></p></td>
<td><p>as Reynolds number [m],</p></td>
</tr>
<tr>
<td><p><b>zeta_LOC </b></p></td>
<td><p>as local resistance coefficient [-],</p></td>
</tr>
<tr>
<td><p><b>zeta_TOT </b></p></td>
<td><p>as pressure loss coefficient [-].</p></td>
</tr>
</table>
<p><br/>The correction for surface roughness through <b>CF </b>is used only in the turbulent regime, where the fluid flow is influenced by surface asperities not covered by a laminar boundary layer. The turbulent regime starts at <b>Re &ge; 4e4 </b>according to <i>[Idelchik 2006, p. 336, sec. 15]</i>. There is no correction due to roughness in the laminar regime up to <b>Re &le; 6.5e3 </b>according to <i>[Idelchik 2006, p. 336, sec. 15]</i>. </p>
<p>Nevertheless the transition point from the laminar to the transition regime is shifted to smaller Reynolds numbers for an increasing absolute roughness. This effect is considered according to <i>[Samoilenko in Idelchik 2006, p. 81, sec. 2-1-21]</i> as: </p>
<pre>    Re_lam_leave = 754*exp(if k &le; 0.007 then 0.0065/0.007 else 0.0065/k)</pre>
<p>with </p>
<table cellspacing=\"2\" cellpadding=\"0\" border=\"0\"><tr>
<td><p><b>k = K /d_hyd </b></p></td>
<td><p>as relative roughness [-],</p></td>
</tr>
<tr>
<td><p><b>Re_lam_leave </b></p></td>
<td><p>as Reynolds number for leaving laminar regime [-].</p></td>
</tr>
</table>
<p>Note that the beginning of the laminar regime cannot be beneath <b>Re &le; 1e2 </b>. </p>
<h4>Verification</h4>
<p>The pressure loss coefficient <b>zeta_TOT </b>of a curved bend in dependence of the Reynolds number <b>Re </b>for different relative curvature radii <b>R_0/d_hyd </b>and different angles of turning <b>delta </b>is shown in the figures below. </p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Fluid/Dissipation/pressureLoss/bend/fig_bend_dp_curvedOverall_DPvsMFLOW.png\" alt=\"fig_bend_dp_curvedOverall_DPvsMFLOW\"/>
</p>

<p>There are deviations of the pressure loss coefficient <b>zeta_TOT</b> comparing different references. Usually these deviations in the transition regime have to be accepted due to an uncertainty for the determination of comparable boundary conditions in the different references. Nevertheless these calculations cover the usual range of pressure loss coefficients for a curved bend. The pressure loss coefficient <b>zeta_TOT </b>for the same geometry can be adjusted via varying the average height of surface asperities <b>K</b> for calibration. </p>
<p>The pressure loss in dependence of the mass flow rate of water is shown for different relative curvature radii: </p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Fluid/Dissipation/pressureLoss/bend/fig_bend_dp_curvedOverall_DPvsMFLOWwrtRD.png\" alt=\"fig_bend_dp_curvedOverall_DPvsMFLOWwrtRD\"/>
</p>

<p>The pressure loss in dependence of the mass flow rate of water is shown for different angles of turning: </p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Fluid/Dissipation/pressureLoss/bend/fig_bend_dp_curvedOverall_DPvsMFLOWwrtDelta.png\" alt=\"fig_bend_dp_curvedOverall_DPvsMFLOWwrtRD\"/>
</p>

<p>Note that there is a small deviation between the compressible and incompressible calculation due to the lack of a direct analytical invertibility.</p>

<h4>References</h4>
<dl><dt>Elmqvist,H., M.Otter and S.E. Cellier: </dt>
<dd><b>Inline integration: A new mixed symbolic / numeric approach for solving differential-algebraic equation systems.</b>. In Proceedings of European Simulation MultiConference, Praque, 1995.</dd>
<dt>Idelchik,I.E.: </dt>
<dd><b>Handbook of hydraulic resistance</b>. Jaico Publishing House, Mumbai, 3rd edition, 2006.</dd>
<dt>Miller,D.S.: </dt>
<dd><b>Internal flow systems</b>. volume 5th of BHRA Fluid Engineering Series.BHRA Fluid Engineering, 1984. </dd>
<dt>Samoilenko,L.A.: </dt>
<dd><b>Investigation of the hydraulic resistance of pipelines in the zone of transition from laminar into turbulent motion</b>. PhD thesis, Leningrad State University, 1968.</dd>
<dt>VDI: </dt>
<dd><b>VDI - W&auml;rmeatlas: Berechnungsbl&auml;tter f&uuml;r den W&auml;rme&uuml;bergang</b>. Springer Verlag, 9th edition, 2002. </dd>
</dl></html>"));
              end dp_curvedOverall;

              class dp_edgedOverall
                extends Modelica.Icons.Information;
                annotation(
                  Documentation(info = "<html>
<p>
Calculation of pressure loss in edged bends with sharp corners at overall flow regime for incompressible and single-phase fluid flow through circular cross sectional area considering surface roughness.
</p>
<h4>Restriction</h4>
<p>
This function shall be used inside of the restricted limits according to the referenced literature.
</p>
<ul>
 <li>
      <b> circular cross sectional area </b> <i>[Idelchik 2006, p. 366, diag. 6-7] </i>
 </li>
 <li>
      <b> edged bend with sharp corners at turning </b> <i>[Idelchik 2006, p. 366, diag. 6-7] </i>
 </li>
 <li>
      <b> 0&deg; &le; angle of turning &le; 180&deg; </b> <i>[Idelchik 2006, p. 338, sec. 19] </i>
 </li>
 <li>
      <b> length of edged bend along edged axis / diameter &ge; 10 </b> <i>[Idelchik 2006, p. 366, diag. 6-7] </i>
 </li>
</ul>

<h4>Geometry</h4>

<p>
<img src=\"modelica://Modelica/Resources/Images/Fluid/Dissipation/pressureLoss/bend/pic_edgedBend.png\" alt=\"pic_edgedBend\"/>
</p>

<h4>Calculation</h4>
<p>The pressure loss <b>dp</b> for edged bends is determined by:
</p>

<pre>
    dp = zeta_TOT * (rho/2) * velocity^2
</pre>

<p>
with
</p>

<table>
<tr><td><b> rho            </b></td><td> as density of fluid [kg/m3],</td></tr>
<tr><td><b> velocity       </b></td><td> as mean velocity [m/s],</td></tr>
<tr><td><b> zeta_TOT       </b></td><td> as pressure loss coefficient [-].</td></tr>
</table>

<p>
The pressure loss coefficient <b> zeta_TOT </b> of an edged bend can be calculated for different angles of turning <b> delta </b> by:
</p>

<pre>
    zeta_TOT = A * C1 * zeta_LOC * CF_Fri* CF_Re <i>[Idelchik 2006, p. 366, diag. 6-7] </i> and <i>[Miller 1984, p. 149, sec. 9.4]</i>
</pre>

<p>
with
</p>

<table>
<tr><td><b> A              </b></td><td> as coefficient considering effect for angle of turning [-],</td></tr>
<tr><td><b> C1             </b></td><td> as coefficient considering relative elongation of cross sectional area (here: circular cross sectional area) [-],</td></tr>
<tr><td><b> CF_Fri         </b></td><td> as correction factor considering surface roughness [-],</td></tr>
<tr><td><b> CF_Re          </b></td><td> as correction factor considering Reynolds number [-],</td></tr>
<tr><td><b> delta          </b></td><td> as angle of turning [deg].</td></tr>
</table>

<p>
The correction factor <b> CF_Fri </b> regarding the influence of surface roughness is determined as ratio of the Darcy friction factor for rough surfaces to smooth surfaces according to <i>[Miller, p. 207, eq. 9.3]:</i>
</p>
<pre>
    CF_Fri = lambda_FRI_rough / lambda_FRI_smooth
</pre>

<p>
and the Darcy friction factors <b>lambda_FRI</b> are calculated with an approximated Colebrook-White law according to <i>[Miller, p. 191, eq. 8.4]:</i>
</p>
<pre>
    lambda_FRI = 0.25*(lg(K/(3.7*d_hyd) + 5.74/Re^0.9))^-2
</pre>

<p>
with
</p>

<table>
<tr><td><b> d_hyd              </b></td><td> as hydraulic diameter [m],</td></tr>
<tr><td><b> K                  </b></td><td> as absolute roughness (average height of surface asperities) [m],</td></tr>
<tr><td><b> lambda_FRI         </b></td><td> as Darcy friction factor[-],</td></tr>
<tr><td><b> Re                 </b></td><td> as Reynolds number [m],</td></tr>
<tr><td><b> zeta_TOT           </b></td><td> as pressure loss coefficient [-].</td></tr>
</table>

<p>
Note that the Darcy friction factor for a smooth surface <b> lambda_FRI_smooth </b> is calculated with the previous equation and an absolute roughness of <b> K = 0 </b>.
</p>

<p>
The correction for surface roughness through <b> CF_Fri </b> is used only in the turbulent regime, where the fluid flow is influenced by surface asperities not covered by a laminar boundary layer. Here the correction according to friction starts at <b> Re &ge; Re_lam_leave </b> according to <i>[Idelchik 2006, p. 336, sec. 15]</i>. Here the end of the laminar regime is restricted to a Reynolds number smaller than 2e3 w.r.t <i>[VDI, p. Lac 6, fig. 16]</i>.
</p>

<p>
Nevertheless the transition point from the laminar to the transition regime is shifted to smaller Reynolds numbers for an increasing absolute roughness. This effect is considered according to <i>[Samoilenko in Idelchik 2006, p. 81, sec. 2-1-21]</i> as:
</p>

<pre>
    Re_lam_leave = 754*exp(if k &le; 0.007 then 0.0065/0.007 else 0.0065/k)
</pre>

<p>
with
</p>

<table>
<tr><td><b> k = K /d_hyd       </b></td><td> as relative roughness [-],</td></tr>
<tr><td><b> Re_lam_leave       </b></td><td> as Reynolds number for leaving laminar regime [-].</td></tr>
</table>

<p>
Note that the beginning of the laminar regime cannot be beneath <b> Re &le; 5e2 </b>.
</p>

<p>
In addition the influence or decreasing Reynolds numbers <b> Re </b> on the pressure loss coefficient <b> zeta_TOT </b> in the laminar and turbulent regime is considered through a second correction factor <b> CF_Re </b> according to <i>[Miller 1984, p. 149, sec. 9.4]</i> by:
</p>

<pre>
   CF_Re = B/Re^exp for Re &le; 2e5
</pre>

<p>
with
</p>

<table>
<tr><td><b> B = f(Geometry)  </b></td><td> as coefficient considering effect of Reynolds number in laminar regime [-],</td></tr>
<tr><td><b> exp              </b></td><td> as exponent for Reynolds number in laminar regime [-],</td></tr>
<tr><td><b> Re               </b></td><td> as Reynolds number [-], </td></tr>
</table>


<p>
Note that the coefficient <b> B </b> considers the influence of the angle of turning <b> delta </b> on the pressure loss coefficient <b> zeta_TOT </b> in the laminar regime according to <i>[Idelchik 2006, p. 340, sec. 28]</i>.
</p>

<p>
Note that the correction of the pressure loss coefficient <b> zeta_TOT </b> is influenced by the correction factor <b> CF_Re </b> only for decreasing Reynolds numbers <b> Re </b> out of the turbulent fluid flow regime at <b> Re &le; 2e5 </b> into transition and laminar fluid flow regime.
</p>

<h4>Verification</h4>
<p>
The pressure loss coefficient <b> zeta_TOT </b> of a edged bend in dependence of the Reynolds number <b> Re </b> for different angles of turning <b> delta </b> is shown in the figures below.
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Fluid/Dissipation/pressureLoss/bend/fig_bend_dp_edgedOverall_ZETAvsRE.png\" alt=\"fig_bend_dp_edgedOverall_ZETAvsRE\"/>
</p>

<p>
Pressure loss calculation of edged bends is complex and there are large differences in literature data. Nevertheless these calculations cover the usual range of pressure loss coefficients for an edged bend.
</p>

<p>
The validation of the pressure loss coefficient for an edged bends shows four possible flow regimes:
</p>
<ul>
 <li>
      <b> laminar regime </b> for Re &le; 5e2</li>
 <li>
      <b> transition regime </b> for 5e2 &le; Re &le; 1e3 ... 1e4</li>
 <li>
      <b> turbulent regime dependent on Reynolds number </b> for 2e3 ... 1e4 &le; Re &le; 1e5</li>
 <li>
      <b> turbulent regime independent of Reynolds number</b> for Re &ge; 2e5</li>
</ul>

<p>
<b> Incompressible case </b> [Pressure loss = f(m_flow)]:
</p>
<p>
The pressure loss in dependence of the mass flow rate of water is shown for different angles of turning:
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Fluid/Dissipation/pressureLoss/bend/fig_bend_dp_edgedOverall_DPvsMFLOWwrtDelta.png\" alt=\"fig_bend_dp_edgedOverall_DPvsMFLOWwrtDelta\"/>
</p>

<p>
<b> Compressible case </b> [Mass flow rate = f(dp)]:
</p>
<p>
The mass flow rate in dependence of the pressure loss of water is shown for different angles of turning:
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Fluid/Dissipation/pressureLoss/bend/fig_bend_dp_edgedOverall_MFLOWvsDPwrtDelta.png\" alt=\"fig_bend_dp_edgedOverall_MFLOWvsDPwrtDelta\"/>
</p>

<h4>References</h4>
<dl>
<dt>Idelchik,I.E.:</dt>
    <dd><b>Handbook of hydraulic resistance</b>.
    Jaico Publishing House,Mumbai,3rd edition, 2006.</dd>
<dt>Miller,D.S.:</dt>
    <dd><b>Internal flow systems</b>.
    volume 5th of BHRA Fluid Engineering Series.BHRA Fluid Engineering, 1984.</dd>
 <dt>Samoilenko,L.A.:</dt>
    <dd><b>Investigation of the hydraulic resistance of pipelines in the
        zone of transition from laminar into turbulent motion</b>.
        PhD thesis, Leningrad State University, 1968.</dd>
<dt>VDI:</dt>
    <dd><b>VDI - W&auml;rmeatlas: Berechnungsbl&auml;tter f&uuml;r den W&auml;rme&uuml;bergang</b>.
    Springer Verlag, 9th edition, 2002.</dd>
</dl>
</html>"));
              end dp_edgedOverall;
              annotation(
                DocumentationClass = true);
            end Bend;

            package Channel
              extends Modelica.Icons.Information;

              class dp_internalFlowOverall
                extends Modelica.Icons.Information;
                annotation(
                  Documentation(info = "<html>
<p>
Calculation of pressure loss for an internal flow through different geometries at overall flow regime for single-phase fluid flow considering surface roughness.
</p>
<h4>Restriction</h4>
<p>
This function shall be used inside of the restricted limits according to the referenced literature.
</p>
<ul>
 <li>
      <b> developed fluid flow </b>
 </li>
</ul>

<h4>Geometry</h4>

<p>
<img src=\"modelica://Modelica/Resources/Images/Fluid/Dissipation/pressureLoss/channel/pic-pLchannel.png\" alt=\"pic-pLchannel\"/>
</p>

<h4>Calculation</h4>
<p>
The pressure loss <b>dp</b> for channels is determined by:
</p>

<pre>
    dp = zeta_TOT * (rho/2) * velocity^2
</pre>

<p>
with
</p>

<table>
<tr><td><b> rho            </b></td><td> as density of fluid [kg/m3],</td></tr>
<tr><td><b> velocity       </b></td><td> as mean velocity [m/s],</td></tr>
<tr><td><b> zeta_TOT       </b></td><td> as pressure loss coefficient [-].</td></tr>
</table>

<p>
<b>Channels with different shape of its cross sectional area are calculated </b> according to <i>[Miller 1978, p. 138, fig. 8.5-8-6]</i>
</p>
<p>
The pressure loss of these channels is similar to its calculation in straight pipes. There are three different flow regimes observed (laminar,transition,turbulent). The pressure loss coefficient (<b>zeta_TOT</b>) of a channel is calculated in dependence of the flow regime as follows:
</p>

<ul>
  <li><b>Laminar regime (Re &le; Re_lam_leave)</b>:
     <pre>
      zeta_TOT = CF_lam/Re * (L/d_hyd)
     </pre></li>
  <li><p><b>Transition regime (Re_lam_leave &le; 4e4)</b></p>
        <p>This calculation is done using a smoothing function interpolating between the laminar and the turbulent flow regime.</p></li>
  <li> <p><b>Turbulent regime (Re &ge; 4e3)</b>:</p>
        <p>The turbulent regime can be calculated with the pressure loss correlations for a straight pipe with the hydraulic diameter of
           the chosen geometry instead of the internal diameter of a straight pipe according to <i>[VDI 2002, p. Lab 4, sec. 2.1]</i> .
           The documentation of turbulent fluid flow for a straight pipe is shown in
           <a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.PressureLoss.StraightPipe.dp_turbulent\">dp_turbulent</a>.</p></li>
</ul>

<p>
with
</p>

<table>
<tr><td><b> CF_lam        </b></td><td> as correction factor considering the geometry for laminar regime [-],</td></tr>
<tr><td><b> L             </b></td><td> as length of geometry perpendicular to cross sectional area [m],</td></tr>
<tr><td><b> d_hyd         </b></td><td> as hydraulic diameter of geometry [m],</td></tr>
<tr><td><b> Re            </b></td><td> as Reynolds number [-],</td></tr>
<tr><td><b> zeta_TOT      </b></td><td> as pressure loss coefficient [-].</td></tr>
</table>

<p>
Note that the beginning of the laminar regime depends on the chosen surface roughness of the channel and cannot be beneath <b>Re &le; 1e3</b>.
</p>
<h4>Verification</h4>
<p>
The Darcy friction factor (<b>lambda_FRI</b>) of a channel with different shapes of its cross sectional area are shown in dependence of the Reynolds number (<b>Re</b>) in the figures below.
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Fluid/Dissipation/pressureLoss/channel/fig_channel_dp_internalFlowOverall_lambdaVsRe.png\" alt=\"fig_channel_dp_internalFlowOverall_lambdaVsRe\"/>
</p>

<p>
The Darcy friction factor (<b>lambda_FRI</b>) for different geometries has been obtained at the same hydraulic diameter and the same mean velocity of the internal flow. Note that there is no difference of the Darcy friction factor in the turbulent regime if using the same hydraulic diameter for all geometries. Roughness can be considered but it is not used for this validation.
</p>
<h4>References</h4>
<dl>
<dt>Miller,D.S.:</dt>
    <dd><b>Internal flow systems</b>.
    Volume 5th of BHRA Fluid Engineering Series.BHRA Fluid Engineering, 1978.</dd>
<dt>VDI:</dt>
    <dd><b>VDI - W&auml;rmeatlas: Berechnungsbl&auml;tter f&uuml;r den W&auml;rme&uuml;bergang</b>.
    Springer Verlag, 9th edition, 2002.</dd>
</dl>
</html>"));
              end dp_internalFlowOverall;
              annotation(
                DocumentationClass = true);
            end Channel;

            package General
              extends Modelica.Icons.Information;

              class dp_idealGas
                extends Modelica.Icons.Information;
                annotation(
                  Documentation(info = "<html>
<p>
Calculation of a generic pressure loss for an <b> ideal gas </b> using mean density.
</p>
<h4>Restriction</h4>
<p>
This function shall be used inside of the restricted limits according to the referenced literature.
</p>
<ul>
 <li>
    <b> ideal gas </b>
 </li>
 <li>
    mean density of ideal gas
 </li>
</ul>

<h4>Calculation</h4>
<p>
The geometry parameters of energy devices necessary for the pressure loss calculations are often not exactly known.
Therefore the modelling of the detailed pressure loss calculation has to be simplified.
</p>
<p>
The pressure loss <b>dp</b> for the compressible case [Mass flow rate = f(dp)] is determined by (Eq.1):
</p>

<pre>
    m_flow = (R_s/Km)^(1/exp)*(rho_m)^(1/exp)*dp^(1/exp)
</pre>

<p>
for the underlying base equation using ideal gas law as follows:
</p>

<pre>
    dp^2 = p_2^2 - p_1^2 = Km*m_flow^exp*(T_2 + T_1)
    dp   = p_2 - p_1     = Km*m_flow^exp*T_m/p_m, Eq.2 with [dp] = Pa, [m_flow] = kg/s
</pre>

<p>
so that the coefficient <b> Km </b> is calculated out of Eq.2:
</p>

<pre>
    Km = dp*R_s*rho_m / m_flow^exp , [Km] = [Pa^2/{(kg/s)^exp*K}]
</pre>

<p>
where the mean density <b> rho_m </b> is calculated according to the ideal gas law out of an arithmetic mean pressure and temperature:
</p>

<pre>
   rho_m = p_m / (R_s*T_m) , p_m = (p_1 + p_2)/2 and T_m = (T_1 + T_2)/2.
</pre>

<p>
with
</p>

<table>
<tr><td><b> exp                    </b></td><td> as exponent of pressure loss law [-],</td></tr>
<tr><td><b> dp                     </b></td><td> as pressure loss [Pa],</td></tr>
<tr><td><b> Km                     </b></td><td> as coefficient w.r.t. mass flow rate! [Km] = [Pa^2/{(kg/s)^exp*K}],</td></tr>
<tr><td><b> m_flow                 </b></td><td> as mass flow rate [kg/s],</td></tr>
<tr><td><b> p_m = (p_2 + p_1)/2    </b></td><td> as mean pressure of ideal gas [Pa],</td></tr>
<tr><td><b> T_m = (T_2 + T_1)/2    </b></td><td> as mean temperature of ideal gas [K],</td></tr>
<tr><td><b> rho_m = p_m/(R_s*T_m)  </b></td><td> as mean density of ideal gas [kg/m3],</td></tr>
<tr><td><b> R_s                    </b></td><td> as specific gas constant of ideal gas [J/(kgK)],</td></tr>
<tr><td><b> V_flow                 </b></td><td> as volume flow rate of ideal gas [m^3/s].</td></tr>
</table>

<p>
Furthermore the coefficient <b> Km </b> can be defined more detailed w.r.t. the definition of pressure loss if <b> Km </b> is not given as (e.g., measured) value. Generally pressure loss can be calculated due to local losses <b> Km,LOC </b> or frictional losses <b> Km,FRI </b>.
</p>
<p>
Pressure loss due to local losses gives the following definition of <b> Km </b>:
</p>

<pre>
    dp        = zeta_LOC * (rho_m/2)*velocity^2 is leading to
      Km,LOC  = (8/&pi;^2)*R_s*zeta_LOC/(d_hyd)^4, considering the cross sectional area of pipes.
</pre>

<p>
and pressure loss due to friction is leading to
</p>

<pre>
    dp        = lambda_FRI*L/d_hyd * (rho_m/2)*velocity^2
      Km,FRI  = (8/&pi;^2)*R_s*lambda_FRI*L/(d_hyd)^5, considering the cross sectional area of pipes.
</pre>

<p>
with
</p>

<table>
<tr><td><b> dp                    </b></td><td> as pressure loss [Pa],</td></tr>
<tr><td><b> d_hyd                 </b></td><td> as hydraulic diameter of pipe [m],</td></tr>
<tr><td><b> Km,i                  </b></td><td> as coefficients w.r.t. mass flow rate! [Km] = [Pa^2/{(kg/s)^exp*K}],</td></tr>
<tr><td><b> lambda_FRI            </b></td><td> as Darcy friction factor [-],</td></tr>
<tr><td><b> L                     </b></td><td> as length of pipe [m],</td></tr>
<tr><td><b> rho_m = p_m/(R_s*T_m) </b></td><td> as mean density of ideal gas [kg/m3],</td></tr>
<tr><td><b> velocity              </b></td><td> as mean velocity [m/s],</td></tr>
<tr><td><b> zeta_LOC              </b></td><td> as local resistance coefficient [-].</td></tr>
</table>

<p>
Note that the variables of this function are delivered in SI units so that the coefficient Km shall be given in SI units too.
</p>
<h4>Verification</h4>
<p>
<b> Compressible case </b> [Mass flow rate = f(dp)]:
</p>
<p>
The mass flow rate <b>m_flow</b> for different coefficients <b> Km </b> as parameter is shown in dependence of its pressure loss <b>dp</b> in the figure below.
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Fluid/Dissipation/pressureLoss/general/fig_general_dp_idealGas_MFLOWvsDP.png\" alt=\"fig_general_dp_idealGas_MFLOWvsDP\"/>
</p>

<p>
Note that the verification for <a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.PressureLoss.General.dp_idealGas\">dp_idealGas</a> is also valid for this inverse calculation due to using the same functions.
</p>
<h4>References</h4>
<dl>
<dt>Elmqvist, H., M.Otter and S.E. Cellier:</dt>
    <dd><b>Inline integration: A new mixed
    symbolic / numeric approach for solving differential-algebraic equation systems.</b>.
    In Proceedings of European Simulation MultiConference, Praque, 1995.</dd>
</dl>
</html>"));
              end dp_idealGas;

              class dp_nominalDensityViscosity
                extends Modelica.Icons.Information;
                annotation(
                  Documentation(info = "<html>
<p>
Calculation of a generic pressure loss in dependence of nominal fluid variables (e.g., nominal density, nominal dynamic viscosity) at an operation point via interpolation.
This generic function considers the pressure loss law via a pressure loss exponent and the influence of density and dynamic viscosity on pressure loss.
</p>
<h4>Calculation</h4>
<p>
The geometry parameters of energy devices necessary for the pressure loss calculations are often not exactly known.
Therefore the modelling of the detailed pressure loss calculation has to be simplified.
This function uses nominal variables (e.g., nominal pressure loss) at a known operation point of the energy device to interpolate the actual pressure loss according to a pressure loss law (exponent).
</p>
<p>
The generic pressure loss <b>dp</b> is determined for:
</p>
<ul>
 <li>
 compressible case [Mass flow rate = f(dp)]:
  <pre>
   m_flow = m_flow_nom*[(dp/dp_nom)*(rho/rho_nom)]^(1/exp)*(eta_nom/eta)^(exp_eta/exp)
   </pre>
 </li>
 <li>
 incompressible case [Pressure loss = f(m_flow)]:
  <pre>
   dp = dp_nom*(m_flow/m_flow_nom)^exp*(rho_nom/rho)*(eta/eta_nom)^exp_eta
   </pre>
  </li>
</ul>

<p>
with
</p>

<table>
<tr><td><b> dp             </b></td><td> as pressure loss [Pa],</td></tr>
<tr><td><b> dp_nom         </b></td><td> as nominal pressure loss [Pa],</td></tr>
<tr><td><b> eta            </b></td><td> as dynamic viscosity of fluid [kg/(ms)].</td></tr>
<tr><td><b> eta_nom        </b></td><td> as nominal dynamic viscosity of fluid [kg/(ms)].</td></tr>
<tr><td><b> m_flow         </b></td><td> as mass flow rate [kg/s],</td></tr>
<tr><td><b> m_flow_nom     </b></td><td> as nominal mass flow rate [kg/s],</td></tr>
<tr><td><b> exp            </b></td><td> as exponent of pressure loss calculation [-],</td></tr>
<tr><td><b> exp_eta        </b></td><td> as exponent of dynamic viscosity dependence [-],</td></tr>
<tr><td><b> rho            </b></td><td> as fluid density [kg/m3],</td></tr>
<tr><td><b> rho_nom        </b></td><td> as nominal fluid density [kg/m3].</td></tr>
</table>

<p>
To avoid numerical difficulties this pressure loss function is linear smoothed for
</p>
<ul>
 <li>
 small mass flow rates, where
 <pre>
    m_flow &le; (0.01*rho/rho_nom*(1/eta*eta_nom)^(exp_eta))^(1/exp) and
  </pre>
 </li>
 <li> small pressure losses, where
 <pre>
    dp &le; 0.01*dp_nom)
 </pre>
 </li>
</ul>
<p>
Note that the density (rho) and dynamic viscosity (eta) of the fluid are defined through the definition of the kinematic viscosity (nue).
</p>

<pre>
    nue = eta / rho
</pre>

<p>
Therefore if you set both the exponent of dynamic viscosity (exp_eta == 1) and additionally a relation of density and dynamic viscosity there will be no difference for varying densities because the dynamic viscosities will vary in the same manner.
</p>
<h4>Verification</h4>
<p>
<b> Incompressible case </b> [Pressure loss = f(m_flow)]:
</p>
<p>
The generic pressure loss <b> DP </b> in dependence of the mass flow rate <b>m_flow</b> with different fluid densities and dynamic viscosity dependence as parameters is shown for a turbulent pressure loss regime (exp == 2) in the figure below.
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Fluid/Dissipation/pressureLoss/general/fig_general_dp_nominalDensityViscosity_DPvsMFLOW.png\" alt=\"fig_general_dp_nominalDensityViscosity_DPvsMFLOW\"/>
</p>

<p>
<b> Compressible case </b> [Mass flow rate = f(dp)]:
</p>
<p>
The generic mass flow rate <b> M_FLOW </b> in dependence of the pressure loss <b>dp</b> at different fluid densities and dynamic viscosity as parameters is shown for a turbulent pressure loss regime (exp == 2) in the figure below.
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Fluid/Dissipation/pressureLoss/general/fig_general_dp_nominalDensityViscosity_MFLOWvsDP.png\" alt=\"fig_general_dp_nominalDensityViscosity_MFLOWvsDP\"/>
</p>

<h4>References</h4>
<dl>
<dt>Elmqvist, H., M.Otter and S.E. Cellier:</dt>
    <dd><b>Inline integration: A new mixed
    symbolic / numeric approach for solving differential-algebraic equation systems.</b>.
    In Proceedings of European Simulation MultiConference, Praque, 1995.</dd>
<dt>Wischhusen, S.:</dt>
    <dd><b>Dynamische Simulation zur wirtschaftlichen Bewertung von komplexen Energiesystemen.</b>.
    PhD thesis, Technische Universit&auml;t Hamburg-Harburg, 2005.</dd>
</dl>

</html>"));
              end dp_nominalDensityViscosity;

              class dp_nominalPressureLossLawDensity
                extends Modelica.Icons.Information;
                annotation(
                  Documentation(info = "<html>
<p>
Calculation of a generic pressure loss in dependence of nominal fluid variables (e.g., nominal density) via interpolation from an operation point.
This generic function considers the pressure loss law via a nominal pressure loss (dp_nom), a pressure loss coefficient (zeta_TOT) and a pressure loss law exponent (exp) as well as the influence of density on pressure loss.
</p>
<h4>Calculation</h4>
<p>
The geometry parameters of energy devices necessary for the pressure loss calculations are often not exactly known.
Therefore the modelling of the detailed pressure loss calculation have to be simplified.
This function uses nominal variables (e.g., nominal pressure loss) at a known operation point of the energy device to interpolate the actual pressure loss according to a pressure loss law (exponent).
</p>
<p>
In the following the pressure loss <b>dp</b> is generally determined from a known operation point via a law of similarity:
</p>

<pre>
   dp/dp_nom = (zeta_TOT/zeta_TOT_nom)*(rho/rho_nom)*(v/v_nom)^exp
</pre>

<p>
with
</p>

<table>
<tr><td><b> dp             </b></td><td> as pressure loss [Pa],</td></tr>
<tr><td><b> dp_nom         </b></td><td> as nominal pressure loss [Pa],</td></tr>
<tr><td><b> m_flow         </b></td><td> as mass flow rate [kg/s],</td></tr>
<tr><td><b> m_flow_nom     </b></td><td> as nominal mass flow rate [kg/s],</td></tr>
<tr><td><b> exp            </b></td><td> as exponent of pressure loss calculation [-],</td></tr>
<tr><td><b> rho            </b></td><td> as fluid density [kg/m3],</td></tr>
<tr><td><b> rho_nom        </b></td><td> as nominal fluid density [kg/m3],</td></tr>
<tr><td><b> v              </b></td><td> as mean flow velocity [m/s],</td></tr>
<tr><td><b> v_nom          </b></td><td> as nominal mean flow velocity [m/s],</td></tr>
<tr><td><b> zeta_TOT       </b></td><td> as pressure loss coefficient [-],</td></tr>
<tr><td><b> zeta_TOT_nom   </b></td><td> as nominal pressure loss coefficient [-].</td></tr>
</table>

<p>
The fraction of mean flow velocities (v/v_nom) can be calculated through its corresponding <b> mass flow rates </b>, densities and cross sectional areas:
</p>

<pre>
   v/v_nom = (m_flow/m_flow_nom)*(A_cross_nom/A_cross)*(rho_nom/rho)
</pre>

<p>
<b> or </b> through its corresponding <b> volume flow rates </b>, densities and cross sectional areas:
</p>

<pre>
    v/v_nom = (V_flow/V_flow_nom)*(A_cross_nom/A_cross).
</pre>

<p>
with
</p>

<table>
<tr><td><b> A_cross        </b></td><td> as cross sectional area [m2],</td></tr>
<tr><td><b> A_cross_nom    </b></td><td> as nominal cross sectional area [m2],</td></tr>
<tr><td><b> rho            </b></td><td> as fluid density [kg/m3],</td></tr>
<tr><td><b> rho_nom        </b></td><td> as nominal fluid density [kg/m3],</td></tr>
<tr><td><b> v              </b></td><td> as mean flow velocity [m/s],</td></tr>
<tr><td><b> v_nom          </b></td><td> as nominal mean flow velocity [m/s],</td></tr>
<tr><td><b> V_flow         </b></td><td> as volume flow rate [m3/s],</td></tr>
<tr><td><b> V_flow_nom     </b></td><td> as nominal volume flow rate [m3/s].</td></tr>
</table>

<p>
Here the <b> compressible case </b> [Mass flow rate = f(dp)] determines the unknown mass flow rate out of a given pressure loss:
</p>

<pre>
   m_flow = m_flow_nom*(A_cross/A_cross_nom)*(rho_nom/rho)^(exp_density/exp)*[(dp/dp_nom)*(zeta_TOT_nom/zeta_TOT)]^(1/exp);
</pre>

<p>
where the exponent for the fraction of densities is determined w.r.t. the chosen nominal mass flow rate or nominal volume flow rate to:
</p>

<pre>
  exp_density = if NominalMassFlowRate == Modelica.Fluid.Dissipation.Utilities.Types.MassOrVolumeFlowRate.MassFlowRate then 1-exp else 1
</pre>

<p>
with
</p>

<table>
<tr><td><b> NominalMassFlowRate    </b></td><td> as reference for pressure loss law (mass flow rate of volume flow rate),</td></tr>
<tr><td><b> exp                    </b></td><td> as exponent of pressure loss calculation [-],</td></tr>
<tr><td><b> exp_density            </b></td><td> as exponent for density [-].</td></tr>
</table>

<p>
To avoid numerical difficulties this pressure loss function is linear smoothed for small pressure losses, with
</p>

<pre>
   dp &le; 0.01*dp_nom
</pre>

<p>
Note that the input and output arguments for functions throughout this library always use mass flow rates. Here you can choose <b> NominalMassFlowRate == Modelica.Fluid.Dissipation.Utilities.Types.MassOrVolumeFlowRate.MassFlowRate </b> for using a nominal mass flow rate or <b> NominalMassFlowRate == Modelica.Fluid.Dissipation.Utilities.Types.MassOrVolumeFlowRate.VolumeFlowRate </b> for using a nominal volume flow rate. The output argument will always be a mass flow rate for further use as flow model in a thermo-hydraulic framework.
</p>

<p>
Note that the pressure loss coefficients (zeta_TOT,zeta_TOT_nom) refer to its mean flow velocities (v,v_nom) in the pressure loss law to obtain its corresponding pressure loss.
</p>

<h4>Verification</h4>
<p>
<b> Compressible case </b> [Mass flow rate = f(dp)]:
</p>
<p>
The generic mass flow rate <b> M_FLOW </b> in dependence of the pressure loss <b>dp</b> is shown for a turbulent pressure loss regime (exp == 2) in the figure below.
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Fluid/Dissipation/pressureLoss/general/fig_general_dp_nominalPressureLossLawDensity_MFLOWvsDP.png\" alt=\"fig_general_dp_nominalPressureLossLawDensity_MFLOWvsDP\"/>
</p>

<p>
Note that the verification for <a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.PressureLoss.General.dp_nominalPressureLossLawDensity\">dp_nominalPressureLossLawDensity</a> is also valid for this inverse calculation due to using the same functions.
</p>

<h4>References</h4>
<dl>
<dt>Elmqvist, H., M.Otter and S.E. Cellier:</dt>
    <dd><b>Inline integration: A new mixed
    symbolic / numeric approach for solving differential-algebraic equation systems.</b>.
    In Proceedings of European Simulation MultiConference, Praque, 1995.</dd>
<dt>Wischhusen, S.:</dt>
    <dd><b>Dynamische Simulation zur wirtschaftlichen Bewertung von komplexen Energiesystemen.</b>.
    PhD thesis, Technische Universit&auml;t Hamburg-Harburg, 2005.</dd>
</dl>

</html>"));
              end dp_nominalPressureLossLawDensity;

              class dp_pressureLossCoefficient
                extends Modelica.Icons.Information;
                annotation(
                  Documentation(info = "<html>
<p>
Calculation of a generic pressure loss in dependence of a pressure loss coefficient.
</p>
<h4>Calculation</h4>
<p>
The mass flow rate <b>m_flow</b> is determined by:
</p>

<pre>
    m_flow = rho*A_cross*(dp/(zeta_TOT *(rho/2))^0.5
</pre>

<p>
with
</p>

<table>
<tr><td><b> A_cross        </b></td><td> as cross sectional area [m2],</td></tr>
<tr><td><b> dp             </b></td><td> as pressure loss [Pa],</td></tr>
<tr><td><b> rho            </b></td><td> as density of fluid [kg/m3],</td></tr>
<tr><td><b> m_flow         </b></td><td> as mass flow rate [kg/s],</td></tr>
<tr><td><b> zeta_TOT       </b></td><td> as pressure loss coefficient [-].</td></tr>
</table>

<h4>Verification</h4>
<p>
<b> Compressible case </b> [Mass flow rate = f(dp)]:
</p>
<p>
The mass flow rate <b> M_FLOW </b> in dependence of the pressure loss <b>dp</b> for a constant pressure loss coefficient <b> zeta_TOT </b> is shown in the figure below.
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Fluid/Dissipation/pressureLoss/general/fig_general_dp_pressureLossCoefficient_MFLOWvsDP.png\" alt=\"fig_general_dp_pressureLossCoefficient_MFLOWvsDP\"/>
</p>

<p>
Note that the verification for <a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.PressureLoss.General.dp_pressureLossCoefficient\">dp_pressureLossCoefficient</a> is also valid for this inverse calculation due to using the same functions.
</p>

<h4>References</h4>
<dl>
 <dt>Elmqvist, H., M.Otter and S.E. Cellier:</dt>
    <dd><b>Inline integration: A new mixed
    symbolic / numeric approach for solving differential-algebraic equation systems.</b>.
    In Proceedings of European Simulation MultiConference, Praque, 1995.</dd>
 <dt>Wischhusen, S.:</dt>
    <dd><b>Dynamische Simulation zur wirtschaftlichen Bewertung von komplexen Energiesystemen.</b>.
    PhD thesis, Technische Universit&auml;t Hamburg-Harburg, 2005.</dd>
</dl>
</html>"));
              end dp_pressureLossCoefficient;

              class dp_volumeFlowRate
                extends Modelica.Icons.Information;
                annotation(
                  Documentation(info = "<html>
<p>
Calculation of a generic pressure loss with linear or quadratic dependence on volume flow rate.
</p>
<h4>Calculation</h4>
<p>
The geometry parameters of energy devices necessary for the pressure loss calculations are often not exactly known.
Therefore the modelling of the detailed pressure loss calculation has to be simplified. This function uses as
quadratic dependence of the pressure loss on the volume flow rate.
</p>
<p>
The mass flow rate <b>m_flow</b> for the compressible case [Mass flow rate = f(dp)] is determined to <i> [see Wischhusen] </i>:
</p>
<pre>
 m_flow = rho*[-b/(2a) + {[b/(2a)]^2 + dp/a}^0.5]
</pre>
<p>
with
</p>

<table>
<tr><td><b> a              </b></td><td> as quadratic coefficient [Pa*s^2/m^6],</td></tr>
<tr><td><b> b              </b></td><td> as linear coefficient [Pa*s/m3],</td></tr>
<tr><td><b> dp             </b></td><td> as pressure loss [Pa],</td></tr>
<tr><td><b> m_flow         </b></td><td> as mass flow rate [kg/s],</td></tr>
<tr><td><b> rho            </b></td><td> as density of fluid [kg/m3].</td></tr>
</table>

<p>
Note that the coefficients <b> a,b </b> have to be positive values so that there will be a positive (linear or quadratic) pressure loss at positive volume flow rate and vice versa.
</p>
<h4>Verification</h4>
<p><b> Compressible case </b> [Mass flow rate = f(dp)]:</p>
<p>
The generic pressure loss <b>dp</b> for different coefficients <b> a </b> as parameter is shown in dependence of the volume flow rate <b> V_flow </b> in the figure below.
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Fluid/Dissipation/pressureLoss/general/fig_general_dp_volumeFlowRate_MFLOWvsDP.png\" alt=\"fig_general_dp_volumeFlowRate_MFLOWvsDP\"/>
</p>

<p>
Note that the verification for <a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.PressureLoss.General.dp_volumeFlowRate\">dp_volumeFlowRate</a> is also valid for this inverse calculation due to using the same functions.
</p>

<h4>References</h4>
<dl>
<dt>Elmqvist, H., M.Otter and S.E. Cellier:</dt>
    <dd><b>Inline integration: A new mixed
    symbolic / numeric approach for solving differential-algebraic equation systems.</b>.
    In Proceedings of European Simulation MultiConference, Praque, 1995.</dd>
<dt>Wischhusen, S.:</dt>
    <dd><b>Dynamische Simulation zur wirtschaftlichen Bewertung von komplexen Energiesystemen.</b>.
    PhD thesis, Technische Universit&auml;t Hamburg-Harburg, 2005.</dd>
</dl>
</html>"));
              end dp_volumeFlowRate;
              annotation(
                DocumentationClass = true);
            end General;

            package Orifice
              extends Modelica.Icons.Information;

              class dp_suddenChange
                extends Modelica.Icons.Information;
                annotation(
                  Documentation(info = "<html>
<h4>Restriction</h4>
<p>
This function shall be used within the restricted limits according to the referenced literature.
</p>
<ul>
 <li>
      <b>Smooth surface</b></li>
 <li>
      <b>Turbulent flow regime</b></li>
 <li>
      <b>Reynolds number for sudden expansion Re &gt; 3.3e3 </b> <i>[Idelchik 2006, p. 208, diag. 4-1] </i></li>
 <li>
      <b>Reynolds number for sudden contraction Re &gt; 1e4 </b> <i>[Idelchik 2006, p. 216-217, diag. 4-9] </i></li>
</ul>

<h4>Geometry </h4>

<p>
<img src=\"modelica://Modelica/Resources/Images/Fluid/Dissipation/pressureLoss/orifice/pic_suddenChangeSection.png\" alt=\"pic_suddenChangeSection\"/>
</p>

<h4>Calculation</h4>
<p>
The local pressure loss <b>dp</b> is generally determined by:
</p>

<pre>
    dp = 0.5 * zeta_LOC * rho * |v_1|*v_1
</pre>

<p>
with
</p>

<table>
<tr><td><b> rho              </b></td><td> as density of fluid [kg/m3],</td></tr>
<tr><td><b> v_1             </b></td><td> as average flow velocity in small cross sectional area [m/s].</td></tr>
<tr><td><b> zeta_LOC         </b></td><td> as local resistance coefficient [-],</td></tr>
</table>

<p>
The local resistance coefficient <b> zeta_LOC </b> of a sudden expansion can be calculated for different ratios of cross sectional areas by:
</p>

<pre>
    zeta_LOC = (1 - A_1/A_2)^2  <i>[Idelchik 2006, p. 208, diag. 4-1] </i>
</pre>

<p>
and for sudden contraction:
</p>

<pre>
    zeta_LOC = 0.5*(1 - A_1/A_2)^0.75  <i>[Idelchik 2006, p. 216-217, diag. 4-9] </i>
</pre>

<p>
with
</p>

<table>
<tr><td><b> A_1       </b></td><td> small cross sectional area [m^2],</td></tr>
<tr><td><b> A_2       </b></td><td> large cross sectional area [m^2].</td></tr>
</table>

<h4>Verification</h4>
<p>
The local resistance coefficient <b> zeta_LOC </b> of a sudden expansion in dependence of the cross sectional area ratio <b> A_1/A_2 </b> is shown in the figure below.
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Fluid/Dissipation/pressureLoss/orifice/fig_orifice_suddenChangeExpansion.png\" alt=\"fig_orifice_suddenChangeExpansion\"/>
</p>

<p>
The local resistance coefficient <b> zeta_LOC </b> of a sudden contraction in dependence of the cross sectional area ratio <b> A_1/A_2 </b> is shown in the figure below.
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Fluid/Dissipation/pressureLoss/orifice/fig_orifice_suddenChangeContraction.png\" alt=\"fig_orifice_suddenChangeContraction\"/>
</p>

<h4>References</h4>
<dl>
<dt>Elmqvist, H., M.Otter and S.E. Cellier:</dt>
    <dd><b>Inline integration: A new mixed
    symbolic / numeric approach for solving differential-algebraic equation systems.</b>.
    In Proceedings of European Simulation MultiConference, Praque, 1995.</dd>
<dt>Idelchik,I.E.:</dt>
    <dd><b>Handbook of hydraulic resistance</b>.
    Jaico Publishing House, Mumbai, 3rd edition, 2006.</dd>
</dl>
</html>"));
              end dp_suddenChange;

              class dp_thickEdgedOverall
                extends Modelica.Icons.Information;
                annotation(
                  Documentation(info = "<html>
<h4>Restriction</h4>
<p>
This function shall be used within the restricted limits according to the referenced literature.
</p>
<ul>
 <li>
      <b> Reynolds number (for vena contraction) Re &gt; 1e3 </b> <i>[Idelchik 2006, p. 222, diag. 4-15] </i>
 <li>
      <b> Relative length of vena contraction (L/d_hyd_0) &gt; 0.015 </b> <i>[Idelchik 2006, p. 222, diag. 4-15] </i>
 <li>
      <b> Darcy friction factor lambda_FRI = 0.02 </b> <i>[Idelchik 2006, p. 222, sec. 4-15] </i>
</ul>

<h4>Geometry</h4>

<p>
<img src=\"modelica://Modelica/Resources/Images/Fluid/Dissipation/pressureLoss/orifice/pic_thickEdged.png\" alt=\"pic_thickEdged\"/>
</p>

<h4>Calculation</h4>
<p>
The pressure loss <b>dp</b> for a thick edged orifice is determined by:
</p>

<pre>
    dp = zeta_TOT * (rho/2) * (velocity_1)^2
</pre>

<p>
with
</p>

<table>
<tr><td><b> rho            </b></td><td> as density of fluid [kg/m3],</td></tr>
<tr><td><b> velocity_1     </b></td><td> as mean velocity in large cross sectional area [m/s],</td></tr>
<tr><td><b> zeta_TOT       </b></td><td> as pressure loss coefficient [-].</td></tr>
</table>

<p>
The pressure loss coefficient <b> zeta_TOT </b> of a thick edged orifice can be calculated for different cross sectional areas <b> A_0 </b> and relative length of orifice <b> l_bar </b>=L/d_hyd_0 by:
</p>

<pre>
    zeta_TOT = (0.5*(1 - A_0/A_1)^0.75 + tau*(1 - A_0/A_1)^1.375 + (1 - A_0/A_1)^2 + lambda_FRI*l_bar)*(A_1/A_0)^2 <i>[Idelchik 2006, p. 222, diag. 4-15] </i>
</pre>

<p>
with
</p>

<table>
<tr><td><b> A_0       </b></td><td> cross sectional area of vena contraction [m2],</td></tr>
<tr><td><b> A_1       </b></td><td> large cross sectional area of orifice [m2],</td></tr>
<tr><td><b> d_hyd_0   </b></td><td> hydraulic diameter of vena contraction [m],</td></tr>
<tr><td><b> lambda_FRI</b></td><td> as constant Darcy friction factor [-],</td></tr>
<tr><td><b> l_bar     </b></td><td> relative length of orifice [-],</td></tr>
<tr><td><b> L         </b></td><td> length of vena contraction [m],</td></tr>
<tr><td><b> tau       </b></td><td> geometry parameter [-].</td></tr>
</table>

<p>
The geometry factor <b> tau </b> is determined by <i>[Idelchik 2006, p. 219, diag. 4-12]</i>:
</p>

<pre>
    tau = (2.4 - l_bar)*10^(-phi)
    phi = 0.25 + 0.535*l_bar^8 / (0.05 + l_bar^8) .
</pre>

<h4>Verification</h4>
<p>
The pressure loss coefficient <b> zeta_TOT </b> of a thick edged orifice in dependence of a relative length <b>(l_bar = L /d_hyd)</b> with different ratios of cross sectional areas <b> A_0/A_1 </b> is shown in the figure below.
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Fluid/Dissipation/pressureLoss/orifice/fig_orifice_thickEdgedOverall_ZETAvsLENGTH.png\" alt=\"fig_orifice_thickEdgedOverall_ZETAvsLENGTH\"/>
</p>

<p>
<b> Incompressible case </b> [Pressure loss = f(m_flow)]:
</p>
<p>
The pressure loss <b> DP </b> of an thick edged orifice in dependence of the mass flow rate <b>m_flow</b> of water for different ratios <b>A_0/A_1</b> (where <b> A_0 </b> = 0.001 m^2) is shown in the figure below.
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Fluid/Dissipation/pressureLoss/orifice/fig_orifice_thickEdgedOverall_DPvsMFLOW.png\" alt=\"fig_orifice_thickEdgedOverall_DPvsMFLOW\"/>
</p>

<p>
<b>And for the compressible case </b> [Mass flow rate = f(dp)]:
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Fluid/Dissipation/pressureLoss/orifice/fig_orifice_thickEdgedOverall_MFLOWvsDP.png\" alt=\"fig_orifice_thickEdgedOverall_MFLOWvsDP\"/>
</p>

<h4>References</h4>
<dl>
 <dt>Elmqvist,H., M.Otter and S.E. Cellier:</dt>
    <dd><b>Inline integration: A new mixed
symbolic / numeric approach for solving differential-algebraic equation systems.</b>.
    In Proceedings of European Simulation MultiConference, Praque, 1995.</dd>
<dt>Idelchik,I.E.:</dt>
    <dd><b>Handbook of hydraulic resistance</b>.
    Jaico Publishing House,Mumbai, 3rd edition, 2006.</dd>
</dl>
</html>"));
              end dp_thickEdgedOverall;
              annotation(
                DocumentationClass = true);
            end Orifice;

            package StraightPipe
              extends Modelica.Icons.Information;

              class dp_laminar
                extends Modelica.Icons.Information;
                annotation(
                  Documentation(info = "<html>
<p>
Calculation of pressure loss in a straight pipe for <b> laminar </b> flow regime of single-phase fluid flow only.
</p>
<h4>Restriction</h4>
<p>
This function shall be used inside of the restricted limits according to the referenced literature.
</p>
<ul>
 <li><b> circular cross sectional area </b></li>
 <li><b> laminar flow regime (Reynolds number Re &le; 2000) <i>[VDI-W&auml;rmeatlas 2002, p. Lab, eq. 3] </i> </b></li>
</ul>

<h4>Geometry </h4>

<p>
<img src=\"modelica://Modelica/Resources/Images/Fluid/Dissipation/pressureLoss/straightPipe/pic_straightPipe.png\" alt=\"pic_straightPipe\"/>
</p>

<h4>Calculation</h4>
<p>
The pressure loss <b>dp</b> for straight pipes is determined by:
</p>

<pre>
    dp = lambda_FRI * (L/d_hyd) * (rho/2) * velocity^2
</pre>

<p>
with
</p>

<table>
<tr><td><b> lambda_FRI     </b></td><td> as Darcy friction factor [-].</td></tr>
<tr><td><b> L              </b></td><td> as length of straight pipe [m],</td></tr>
<tr><td><b> d_hyd          </b></td><td> as hydraulic diameter of straight pipe [m],</td></tr>
<tr><td><b> rho            </b></td><td> as density of fluid [kg/m3],</td></tr>
<tr><td><b> velocity       </b></td><td> as mean velocity [m/s].</td></tr>
</table>

<p>
The Darcy friction factor <b>lambda_FRI</b> of straight pipes for the laminar flow regime is calculated by <b>Hagen-Poiseuilles</b> law according to <i>[Idelchik 2006, p. 77, eq. 2-3]</i> as follows:
</p>
<ul>
 <li><b> Laminar flow regime </b> is restricted to a Reynolds number <b> Re </b> &le; 2000</li>
 <li>and calculated through:
     <pre>
     lambda_FRI = 64/Re
     </pre>
     <p>
     with
     </p>
     <table>
     <tr><td><b> lambda_FRI     </b></td><td> as Darcy friction factor [-],</td></tr>
     <tr><td><b> Re             </b></td><td> as Reynolds number [-].</td></tr>
     </table>
      </li>
</ul>

<p>
The Darcy friction factor <b>lambda_FRI</b> in the laminar regime is independent
of the surface roughness <b> K </b> as long as the relative roughness <b>k = surface roughness/hydraulic diameter</b> is smaller than 0.007.
A higher relative roughness <b> k </b> than 0.007 leads to an earlier leaving of the laminar regime to the transition regime at some value of Reynolds number <b> Re_lam_leave </b>. This earlier leaving is not modelled here because only laminar fluid flow is considered.
</p>

<h4>Verification</h4>
<p>
The Darcy friction factor <b>lambda_FRI</b> in dependence of Reynolds number is shown in the figure below.
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Fluid/Dissipation/pressureLoss/straightPipe/fig_straightPipe_laminar_lambdavsRe_ver.png\" alt=\"fig_straightPipe_laminar_lambdavsRe_ver\"/>
</p>

<p>
The pressure loss <b>dp</b> for the laminar regime in dependence of the mass flow rate of water is shown in the figure below.
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Fluid/Dissipation/pressureLoss/straightPipe/fig_straightPipe_dp_laminar_DPvsMFLOW.png\" alt=\"fig_straightPipe_dp_laminar_DPvsMFLOW\"/>
</p>

<p>
Note that this pressure loss function shall not be used for the modelling outside of the laminar flow regime at <i><b> Re </b> &gt; 2000</i> even though it could be used for that.
</p>
<p>
If the whole flow
regime shall be modelled, the pressure loss function <a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.PressureLoss.StraightPipe.dp_overall\">dp_overall</a> can be used.
</p>

<h4>References</h4>
<dl>
 <dt>Elmqvist,H., M.Otter and S.E. Cellier:</dt>
    <dd><b>Inline integration: A new mixed
    symbolic / numeric approach for solving differential-algebraic equation systems.</b>.
    In Proceedings of European Simulation MultiConference, Praque, 1995.</dd>
<dt>Idelchik,I.E.:</dt>
    <dd><b>Handbook of hydraulic resistance</b>.
    Jaico Publishing House, Mumbai, 3rd edition, 2006.</dd>
 <dt>VDI:</dt>
    <dd><b>VDI - W&auml;rmeatlas: Berechnungsbl&auml;tter f&uuml;r den W&auml;rme&uuml;bergang</b>.
    Springer Verlag, 9th edition, 2002.</dd>
</dl>
</html>"));
              end dp_laminar;

              class dp_overall
                extends Modelica.Icons.Information;
                annotation(
                  Documentation(info = "<html>
<p>
Calculation of pressure loss in a straight pipe for <b> laminar or turbulent </b> flow regime of single-phase fluid flow only considering surface roughness.
</p>
<h4>Restriction</h4>
<p>
This function shall be used within the restricted limits according to the referenced literature.
</p>
<ul>
 <li>
      <b> circular cross sectional area </b></li>
</ul>

<h4>Geometry </h4>

<p>
<img src=\"modelica://Modelica/Resources/Images/Fluid/Dissipation/pressureLoss/straightPipe/pic_straightPipe.png\" alt=\"pic_straightPipe\"/>
</p>

<h4>Calculation</h4>
The pressure loss <b>dp</b> for straight pipes is determined by:

<pre>
    dp = lambda_FRI * (L/d_hyd) * (rho/2) * velocity^2
</pre>

<p>
with
</p>

<table>
<tr><td><b> lambda_FRI     </b></td><td> as Darcy friction factor [-],</td></tr>
<tr><td><b> L              </b></td><td> as length of straight pipe [m],</td></tr>
<tr><td><b> d_hyd          </b></td><td> as hydraulic diameter of straight pipe [m],</td></tr>
<tr><td><b> rho            </b></td><td> as density of fluid [kg/m3],</td></tr>
<tr><td><b> velocity       </b></td><td> as mean velocity [m/s].</td></tr>
</table>

<p>
The Darcy friction factor <b>lambda_FRI</b> for straight pipes is calculated depending on the fluid flow regime (with corresponding Reynolds number <b>Re</b>) and the absolute surface roughness <b> K </b>.
</p>
<p>

<b> The Laminar regime </b> is calculated for <b> Re </b> &le; 2000 by the Hagen-Poiseuille law according to <i>[Idelchik 2006, p. 77, eq. 2-3]</i>
</p>

<pre>
    lambda_FRI = 64/Re
</pre>

<p>
The Darcy friction factor <b>lambda_FRI</b> in the laminar regime is independent of the surface roughness <b> k </b> as long as the relative roughness <b> k </b> is smaller than 0.007. A greater relative roughness <b> k </b> than 0.007 is leading to an earlier leaving of the Hagen-Poiseuille law at some value of Reynolds number <b> Re_lam_leave </b>. The leaving of the laminar regime in dependence of the relative roughness <b> k </b> is calculated according to <i>[Samoilenko in Idelchik 2006, p. 81, sect. 2-1-21]</i> as:
</p>
<pre>
    Re_lam_leave = 754*exp(if k &le; 0.007 then 0.93 else 0.0065/k)
</pre>

<p>
<b>The Transition regime</b> is calculated for 2000 &lt; <b>Re</b> &le; 4000  by a cubic interpolation between the equations of the laminar and turbulent flow regime. Different cubic
interpolation equations for the calculation of either pressure loss <b>dp</b> or mass flow rate <b>m_flow</b> results in a deviation of the Darcy friction factor <b>lambda_FRI</b> through the
transition regime. This deviation can be neglected due to the uncertainty in determination of the fluid flow in the transition regime.
</p>

<p>
<b> Turbulent regime </b> can be calculated for a smooth surface (Blasius law) <b> or </b> a rough surface (Colebrook-White law):
</p>

<p>
<b> Smooth surface (roughness = Modelica.Fluid.Dissipation.Utilities.Types.Roughness.Neglected) </b> w.r.t. <b> Blasius </b> law in the turbulent regime according to <i>[Idelchik 2006, p. 77, sec. 15]</i>:
</p>
<pre>
    lambda_FRI = 0.3164*Re^(-0.25)
</pre>

<p>
with
</p>

<table>
<tr><td><b> lambda_FRI     </b></td><td> as Darcy friction factor [-].</td></tr>
<tr><td><b> Re             </b></td><td> as Reynolds number [-].</td></tr>
</table>

<p>
Note that the Darcy friction factor <b>lambda_FRI</b> for smooth straight pipes in the turbulent regime is independent
of the surface roughness <b> K </b> .
</p>

<p>
<b> Rough surface (roughness = Modelica.Fluid.Dissipation.Utilities.Types.Roughness.Considered) </b> w.r.t. <b> Colebrook-White </b> law in the turbulent regime according to <i>[Miller 1984, p. 191, eq. 8.4]</i>:
</p>
<pre>
    lambda_FRI = 0.25/{lg[k/(3.7*d_hyd) + 5.74/(Re)^0.9]}^2
</pre>

<p>
with
</p>

<table>
<tr><td><b> d_hyd          </b></td><td> as hydraulic diameter [-],</td></tr>
<tr><td><b> k= K/d_hyd     </b></td><td> as relative roughness [-],</td></tr>
<tr><td><b> K              </b></td><td> as roughness (average height of surface asperities [m],</td></tr>
<tr><td><b> lambda_FRI     </b></td><td> as Darcy friction factor [-],</td></tr>
<tr><td><b> Re             </b></td><td> as Reynolds number [-].</td></tr>
</table>

<h4>Verification</h4>
<p>
The Darcy friction factor <b>lambda_FRI</b> in dependence of Reynolds number for different values of relative roughness <b> k </b> is shown in the figure below.
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Fluid/Dissipation/pressureLoss/straightPipe/fig_straightPipe_dp_overall_lambdavsRe_ver.png\" alt=\"fig_straightPipe_dp_overall_lambdavsRe_ver\"/>
</p>

<p>
The pressure loss <b>dp</b> for the turbulent regime in dependence of the mass flow rate of water is shown in the figure below.
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Fluid/Dissipation/pressureLoss/straightPipe/fig_straightPipe_dp_overall_DPvsMFLOW.png\" alt=\"fig_straightPipe_dp_overall_DPvsMFLOW\"/>
</p>

<p>
And the mass flow rate <b>m_flow</b> for the turbulent regime in dependence of the pressure loss of water is shown in the figure below.
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Fluid/Dissipation/pressureLoss/straightPipe/fig_straightPipe_dp_overall_MFLOWvsDP.png\" alt=\"fig_straightPipe_dp_overall_MFLOWvsDP\"/>
</p>

<h4>References</h4>
<dl>
 <dt>Idelchik,I.E.:</dt>
    <dd><b>Handbook of hydraulic resistance</b>.
    Jaico Publishing House, Mumbai, 3rd edition, 2006.</dd>
 <dt>Miller,D.S.:</dt>
    <dd><b>Internal flow systems</b>.
    volume 5th of BHRA Fluid Engineering Series.BHRA Fluid Engineering, 1984.
 <dt>Samoilenko,L.A.:</dt>
    <dd><b>Investigation of the hydraulic resistance of pipelines in the
        zone of transition from laminar into turbulent motion</b>.
        PhD thesis, Leningrad State University, 1968.</dd>
 <dt>VDI:</dt>
    <dd><b>VDI - W&auml;rmeatlas: Berechnungsbl&auml;tter f&uuml;r den W&auml;rme&uuml;bergang</b>.
    Springer Verlag, 9th edition, 2002.</dd>
</dl>
</html>"));
              end dp_overall;

              class dp_turbulent
                extends Modelica.Icons.Information;
                annotation(
                  Documentation(info = "<html>
<p>
Calculation of pressure loss in a straight pipe for <b> turbulent </b> flow regime of single-phase fluid flow only considering surface roughness.
</p>
<h4>Restriction</h4>
<p>
This function shall be used within the restricted limits according to the referenced literature.
</p>
<ul>
 <li>
      <b> circular cross sectional area </b></li>
 <li>
      <b> turbulent flow regime (Reynolds number Re &ge; 4e3) <i>[VDI-W&auml;rmeatlas 2002, p. Lab 3, fig. 1] </i> </b></li>
</ul>


<h4>Geometry </h4>

<p>
<img src=\"modelica://Modelica/Resources/Images/Fluid/Dissipation/pressureLoss/straightPipe/pic_straightPipe.png\" alt=\"pic_straightPipe\"/>
</p>

<h4>Calculation</h4>
<p>
The pressure loss <b>dp</b> for straight pipes is determined by:
</p>

<pre>
    dp = lambda_FRI * (L/d_hyd) * (rho/2) * velocity^2
</pre>

<p>
with
</p>

<table>
<tr><td><b> lambda_FRI     </b></td><td> as Darcy friction factor [-].</td></tr>
<tr><td><b> L              </b></td><td> as length of straight pipe [m],</td></tr>
<tr><td><b> d_hyd          </b></td><td> as hydraulic diameter of straight pipe [m],</td></tr>
<tr><td><b> rho            </b></td><td> as density of fluid [kg/m3],</td></tr>
<tr><td><b> velocity       </b></td><td> as mean velocity [m/s].</td></tr>
</table>

<p>
The Darcy friction factor <b>lambda_FRI</b> for a straight pipe in the turbulent regime can be calculated for a smooth surface (Blasius law) <b> or </b> a rough surface (Colebrook-White law).
</p>
<p>
<b> Smooth surface (roughness = Modelica.Fluid.Dissipation.Utilities.Types.Roughness.Neglected) </b> w.r.t. <b> Blasius </b> law in the turbulent regime according to <i>[Idelchik 2006, p. 77, sec. 15]</i>:
</p>

<pre>
    lambda_FRI = 0.3164*Re^(-0.25)
</pre>

<p>
with
</p>

<table>
<tr><td><b> lambda_FRI     </b></td><td> as Darcy friction factor [-].</td></tr>
<tr><td><b> Re             </b></td><td> as Reynolds number [-].</td></tr>
</table>

<p>
Note that the Darcy friction factor <b>lambda_FRI</b> for smooth straight pipes in the turbulent regime is independent
of the surface roughness <b> K </b> .
</p>

<p>
<b> Rough surface (roughness = Modelica.Fluid.Dissipation.Utilities.Types.Roughness.Considered) </b> w.r.t. <b> Colebrook-White </b> law in the turbulent regime according to <i>[Miller 1984, p. 191, eq. 8.4]</i>:
</p>

<pre>
    lambda_FRI = 0.25/{lg[k/(3.7*d_hyd) + 5.74/(Re)^0.9]}^2
</pre>

<p>
with
</p>

<table>
<tr><td><b> d_hyd          </b></td><td> as hydraulic diameter [-],</td></tr>
<tr><td><b> k= K/d_hyd     </b></td><td> as relative roughness [-],</td></tr>
<tr><td><b> K              </b></td><td> as roughness (average height of surface asperities [m].</td></tr>
<tr><td><b> lambda_FRI     </b></td><td> as Darcy friction factor [-],</td></tr>
<tr><td><b> Re             </b></td><td> as Reynolds number [-].</td></tr>
</table>

<h4>Verification</h4>
<p>
The Darcy friction factor <b>lambda_FRI</b> in dependence of Reynolds number for different values of relative roughness <b> k </b> is shown in the figure below.
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Fluid/Dissipation/pressureLoss/straightPipe/fig_straightPipe_turbulent.png\" alt=\"fig_straightPipe_turbulent\"/>
</p>

<p>
Note that this pressure loss function shall not be used for the modelling outside of the turbulent flow regime at <b> Re </b> &lt; 4e3 even though it could be used for that.
</p>

<p>
If the overall flow regime shall be modelled, the pressure loss function <a href=\"modelica://Modelica.Fluid.Dissipation.Utilities.SharedDocumentation.PressureLoss.StraightPipe.dp_overall\">dp_overall</a> can be used.
</p>

<h4>References</h4>
<dl>
 <dt>Idelchik,I.E.:</dt>
    <dd><b>Handbook of hydraulic resistance</b>.
    Jaico Publishing House, Mumbai, 3rd edition, 2006.</dd>
 <dt>VDI:</dt>
    <dd><b>VDI - W&auml;rmeatlas: Berechnungsbl&auml;tter f&uuml;r den W&auml;rme&uuml;bergang</b>.
    Springer Verlag, 9th edition, 2002.</dd>
</dl>
</html>"));
              end dp_turbulent;

              class dp_twoPhaseOverall
                extends Modelica.Icons.Information;
                annotation(
                  Documentation(info = "<html>
<p>
Calculation of pressure loss for <b>two phase flow</b> in a horizontal <b>or</b> vertical straight pipe for an overall flow regime considering frictional, momentum and geodetic pressure loss.
</p>
<h4>Restriction</h4>
<p>
This function shall be used within the restricted limits according to the referenced literature.
</p>
<ul>
 <li> <b> circular cross sectional area </b> </li>
 <li> <b> neglecting of surface roughness </b> </li>
 <li> <b> horizontal flow or vertical upflow </b> </li>
 <li> <b> usage of mass flow rate quality (see Calculation) </b> </li>
 <li> <b> two phase pressure loss for mean constant mass flow rate quality (x_flow) over (increment) length </b></li>
 <li> <b> usage of two phase pressure loss function for discretization at boiling or condensation considering variable mass flow rate quality</b> </li>
</ul>

<h4>Geometry </h4>

<p>
<img src=\"modelica://Modelica/Resources/Images/Fluid/Dissipation/pressureLoss/straightPipe/pic_straightPipe.png\" alt=\"pic_straightPipe\"/>
</p>

<h4>Calculation</h4>
<p>
The two phase pressure loss <b> dp_2ph </b> of straight pipes is determined by:
</p>

<pre>
    dp_2ph = dp_fri + dp_mom + dp_geo
</pre>

<p>
with
</p>

<table>
<tr><td><b> dp_fri        </b></td><td> as frictional pressure loss [Pa],</td></tr>
<tr><td><b> dp_mom        </b></td><td> as momentum pressure loss [Pa],</td></tr>
<tr><td><b> dp_geo        </b></td><td> as geodetic pressure loss [Pa].</td></tr>
</table>

<p>
<b>Definition of quality for two phase flow</b>:
</p>
<p>
Different definitions of the quality exist for two phase flow. Static quality, mass flow rate quality and thermodynamic quality can be used to describe the fraction of gas and liquid in two phase flow.
Here the mass flow rate quality <b>(x_flow)</b> is used to describe two phase flow as follows:
</p>

<pre>
    x_flow = mdot_g/(mdot_g+mdot_l)
</pre>

<p>
with
</p>

<table>
<tr><td><b> x_flow        </b></td><td> as mass flow rate quality [-],</td></tr>
<tr><td><b> mdot_g        </b></td><td> as gaseous mass flow rate [kg/s],</td></tr>
<tr><td><b> mdot_l        </b></td><td> as liquid mass flow rate [kg/s].</td></tr>
</table>

<p>
Note that mass flow rate quality <b>(x_flow)</b> is only equal to the static quality, if a difference between the velocity of gas and liquid phase is neglected (homogeneous approach).
Additionally the thermodynamic quality is only equal to the mass flow rate quality <b>(x_flow)</b> in the two phase regime for thermodynamic equilibrium of the phases.
</p>
<p>
<b>Frictional pressure loss</b>:
</p>
<p>
The frictional pressure loss <b>dp_fri</b> of a straight pipe is calculated either by the correlation of <b>Friedel</b> (frictionalPressureLoss==Friedel) or by the correlation of <b>Chisholm</b> (frictionalPressureLoss==Chisholm).
Both correlations can be used for the above named two phase flow regimes.
The two phase frictional pressure loss results from a frictional pressure loss assuming one phase liquid fluid flow and a two phase multiplier taking into account the effects of two phase flow:
</p>

<pre>
    dp_fri = dp_1ph*phi_i
</pre>

<p>
with
</p>

<table>
<tr><td><b> dp_1ph  </b></td><td> as frictional pressure loss assuming one phase liquid fluid flow [Pa],</td></tr>
<tr><td><b> phi_i   </b></td><td> as two phase multiplier [-].</td></tr>
</table>

<p>
The liquid frictional pressure loss is calculated with the <b>total mass flow rate</b> assumed to flow as liquid.
</p>
<p>
The correlations of Friedel and Chisholm differ in their calculation of the two phase multiplier:
</p>
<pre>
    phi_friedel = (1 - x_flow)^2 + x_flow^2*(rho_l/rho_g)*(lambda_g/lambda_l)
                + 3.43*x_flow^0.685*(1 - x_flow)^0.24*(rho_l/rho_g)^0.8*(eta_g/eta_l)^0.22*(1 - eta_g/eta_l)^0.89*(1/Fr_l^(0.048))*(1/We_l^(0.0334))
</pre>
<pre>
    phi_chisholm = 1 + (gamma^2 - 1)*(B*x_flow^((2 - n_exp)/2)*(1 - x_flow)^((2 -n_exp)/2) + x_flow^(2 - n_exp))
</pre>

<p>
with
</p>

<table>
<tr><td><b> B              </b></td><td> as Lockhart-Martinelli coefficient [-],</td></tr>
<tr><td><b> eta_l          </b></td><td> as dynamic viscosity of the liquid phase [Pas],</td></tr>
<tr><td><b> eta_g          </b></td><td> as dynamic viscosity of the gaseous phase [Pas],</td></tr>
<tr><td><b> gamma          </b></td><td> as physical property coefficient [-],</td></tr>
<tr><td><b> n_exp</b> =0.2     </td><td> as exponent in Chisholm correlation [-],</td></tr>
<tr><td><b> phi_i          </b></td><td> as two phase multiplier [-],</td></tr>
<tr><td><b> rho_l          </b></td><td> as density of the liquid phase [kg/m3],</td></tr>
<tr><td><b> rho_g          </b></td><td> as density of the gaseous phase [kg/m3],</td></tr>
<tr><td><b> Re_l           </b></td><td> as Reynolds number of the liquid phase [-],</td></tr>
<tr><td><b> Re_g           </b></td><td> as Reynolds number of the gaseous phase [-],</td></tr>
<tr><td><b> Fr_l           </b></td><td> as Froude number of the liquid phase [-],</td></tr>
<tr><td><b> We_l           </b></td><td> as Weber number of the liquid phase [-],</td></tr>
<tr><td><b> x_flow         </b></td><td> as mass flow rate quality [-].</td></tr>
</table>

<p>
Note that the (mean constant) mass flow rate quality <b>(x_flow)</b> used for frictional pressure loss is calculated as arithmetic mean value out of the mass flow rate quality at the end and at the start of the straight pipe length.
</p>
<p>
<b>Momentum pressure loss</b>:
</p>
<p>
The momentum pressure loss <b> dp_mom </b> can be considered (momentumPressureLoss = true) for a homogeneous or heterogeneous two phase flow depending on the approach used for the void fraction <b>(epsilon)</b>.
At evaporation the liquid phase having a slow velocity has to be accelerated to the higher velocity of the gas. The difference in static pressure at the outlet and the inlet causes a positive momentum pressure loss at evaporation (assumed vice versa for condensation).
The momentum pressure loss occurs for a changing mass flow rate quality due to condensation or evaporation according to <i>[VDI 2006, p.Lba 4, eq. 22]</i> :
</p>

<pre>
    dp_mom = mdot_A^2*[[((1-x_flow)^2/(rho_l*(1-epsilon)) + x_flow^2/(rho_g*epsilon))]_out - [((1-x_flow)^2/(rho_l*(1-epsilon)) + x_flow^2/(rho_g*epsilon))]_in]
</pre>

<p>
with
</p>

<table>
<tr><td><b> mdot_A         </b></td><td> as total mass flow rate density [kg/(m2s)],</td></tr>
<tr><td><b> epsilon        </b></td><td> as void fraction [-],</td></tr>
<tr><td><b> rho_l          </b></td><td> as density of the liquid phase [kg/m3],</td></tr>
<tr><td><b> rho_g          </b></td><td> as density of the gaseous phase [kg/m3],</td></tr>
<tr><td><b> x_flow         </b></td><td> as mass flow rate quality [-].</td></tr>
</table>

<p>
Note that a momentum pressure loss is only considered for a variable mass flow rate quality <b>(x_flow)</b> during evaporation or condensation. Momentum pressure loss does not occur under adiabatic conditions for a corresponding constant mass flow rate quality (evaporation due to pressure loss is not considered).
</p>

<p>
<b>Void fraction approach</b>:
</p>
<p>
The void fraction is one of the most important parameter used to characterize two phase flow. There are several analytical and empirical approaches according to <i>[Thome, J.R]</i> :
</p>

<ul>
 <li>    <b> homogeneous approach </b> </li>
 <li>     <b> momentum flux approach (heterogeneous model) </b></li>
 <li>     <b> Kinetic energy flow approach by Zivi (heterogeneous model) </b></li>
 <li>     <b> Empirical momentum flux approach by Chisholm (heterogeneous model) </b></li>
</ul>

<p>
These approaches for the void fraction <b>epsilon</b> imply a correlation for the slip ratio. The slip ratio is defined as ratio of the velocity from the gaseous phase to the liquid phase at two phase flow.
The effects of different fluid flow velocities of the phases on two phase pressure loss can be considered with the slip ratio in the heterogeneous approaches. The slip ratio for the homogeneous approach is unity, so that there is no difference in the velocities of the two phases (e.g., usable for bubble flow).
</p>
<p>
<b>Geodetic pressure loss</b>:
</p>
<p>
The geodetic pressure loss <b>dp_geo</b> can be considered (geodeticPressureLoss=true) for two phase flow according to <i>[VDI 2006, p.Lbb 1, eq. 4]</i> :
</p>

<pre>
    dp_geo = (epsilon*rho_g +(1-epsilon)*rho_l)*g*L*sin(phi)
</pre>

<p>
with
</p>

<table>
<tr><td><b> epsilon        </b></td><td> as void fraction [-],</td></tr>
<tr><td><b> rho_l          </b></td><td> as density of the liquid phase [kg/m3],</td></tr>
<tr><td><b> rho_g          </b></td><td> as density of the gaseous phase [kg/m3],</td></tr>
<tr><td><b> g              </b></td><td> as acceleration of gravity [m/s2],</td></tr>
<tr><td><b> L              </b></td><td> as length of straight pipe [m],</td></tr>
<tr><td><b> phi            </b></td><td> as angle to horizontal [rad].</td></tr>
</table>

<h4>Verification</h4>
<p>
The two phase pressure loss for a horizontal pipe calculated by the correlation of <i> Friedel </i> neglecting momentum and geodetic pressure loss is shown in the figure below.
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Fluid/Dissipation/pressureLoss/straightPipe/fig_dp_twoPhaseOverall_1.png\" alt=\"fig_dp_twoPhaseOverall_1\"/>
</p>

<p>
The two phase pressure loss for a horizontal pipe calculated by the correlation of <i> Chisholm </i> neglecting momentum and geodetic pressure loss is shown in the figure below.
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Fluid/Dissipation/pressureLoss/straightPipe/fig_dp_twoPhaseOverall_4.png\" alt=\"fig_dp_twoPhaseOverall_4\"/>
</p>

<h4>References</h4>
<dl>
<dt>Chisholm,D.:</dt>
    <dd><b>Pressure gradients due to friction during the flow of evaporating two-phase mixtures in smooth tubes and channels</b>.
    Volume 16th of International Journal of Heat and Mass Transfer, 1973.</dd>
 <dt>Friedel,L.:</dt>
    <dd><b>IMPROVED FRICTION PRESSURE DROP CORRELATIONS FOR HORIZONTAL AND VERTICAL TWO PHASE PIPE FLOW</b>.3R International, Vol. 18, Issue 7, pp. 485-491, 1979.</dd>
 <dt>VDI:</dt>
    <dd><b>VDI - W&auml;rmeatlas: Berechnungsbl&auml;tter f&uuml;r den W&auml;rme&uuml;bergang</b>.
    Springer Verlag, 10th edition, 2006.</dd>
<dt>J.M. Jensen and H. Tummescheit:</dt>
    <dd><b>Moving boundary models for dynamic simulations of two-phase flows</b>.
    In Proceedings of the 2nd International Modelica Conference, pages 235-244, Oberpfaffenhofen, Germany, 2002. The Modelica Association.</dd>
<dt>Thome, J.R.:</dt>
    <dd><b>Engineering Data Book 3</b>.Swiss Federal Institute of Technology Lausanne (EPFL), 2009.</dd>
</dl>
</html>"));
              end dp_twoPhaseOverall;
              annotation(
                DocumentationClass = true);
            end StraightPipe;

            package Valve
              extends Modelica.Icons.Information;

              class dp_severalGeometryOverall
                extends Modelica.Icons.Information;
                annotation(
                  Documentation(info = "<html>
<p>
Calculation of pressure loss for a valve with different geometries at overall flow regime for incompressible and single-phase fluid flow in dependence of its opening.
</p>
<h4>Restriction</h4>
<p>
This function shall be used inside of the restricted limits according to the referenced literature.
</p>
<ul>
 <li>
      <b> developed fluid flow </b>
 </li>
 <li>
      <b> ball valve </b>
 </li>
 <li>
      <b> diaphragm valve </b>
 </li>
 <li>
      <b> butterfly valve </b>
 </li>
 <li>
      <b> gate valve </b>
 </li>
 <li>
      <b> sluice valve </b>
 </li>
</ul>

<h4>Geometry</h4>
<p>
Wide variations in valve geometry are possible and a manufacturer will not necessarily maintain geometric similarity between valves of the same type but of different size. Here pressure loss can be estimated for the following types of a valve:
</p>
<ul>
 <li>
      <b> ball valve </b>
 </li>
 <li>
      <b> diaphragm valve </b>
 </li>
 <li>
      <b> butterfly valve </b>
 </li>
 <li>
      <b> gate valve </b>
 </li>
 <li>
      <b> sluice valve </b>
 </li>
</ul>

<h4>Calculation</h4>
<p>
The mass flow rate <b>m_flow</b> for valves out of pressure loss is determined by:
</p>

<pre>
    m_flow = [rho * dp * Av^2 / (zeta_TOT/2]^0.5
    m_flow = (2/zeta_TOT)^0.5 * Av * (rho * dp)^0.5
    m_flow = valveCharacteristic * Av * (rho * dp)^0.5
</pre>

<p>
with
</p>

<table>
<tr><td><b> rho                        </b></td><td> as density of fluid [kg/m3],</td></tr>
<tr><td><b> Av                         </b></td><td> as (metric) flow coefficient (cross sectional area) [m^2],</td></tr>
<tr><td><b> m_flow                     </b></td><td> as mass flow rate [kg/s],</td></tr>
<tr><td><b> valveCharacteristic        </b></td><td> as coefficient of a valve in dependence of its opening [-],</td></tr>
<tr><td><b> velocity                   </b></td><td> as mean velocity [m/s],</td></tr>
<tr><td><b> zeta_TOT                   </b></td><td> as pressure loss coefficient [-].</td></tr>
</table>

<p>
The <b>valveCharacteristic</b> is determined out of a correlation for the pressure loss coefficient (<b>zeta_TOT</b>) in dependence of its opening. The reason for introducing an additional variable
<b>valveCharacteristic</b> is a different definition of the following pressure loss correlations of valves.
</p>

<h4>Verification</h4>
<p>
The pressure loss coefficient (<b>zeta_TOT</b>) of a valve with different geometries are shown in dependence of the <b>opening</b> in the figure below.
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Fluid/Dissipation/pressureLoss/valve/fig_valve_dp_severalGeometryOverall_zetaVsOpening.png\" alt=\"fig_valve_dp_severalGeometryOverall_zetaVsOpening\">
</p>

<p>
Note that the pressure loss coefficients (<b>zeta_TOT</b>) are numerically optimized for very small openings (opening &le; 5%). At openings smaller than 5% the pressure loss coefficient is smoothly set
to a maximum value (<b>zeta_TOT_max</b>) to be adjusted as parameter. Therefore a very small leakage mass flow rate can be adjusted for a given pressure difference at almost closed valves. A very small
leakage mass flow rate can often be neglected in system simulation with valves, whereas the numerical behaviour of the simulation is improved.
</p>
<p>
The mass flow rate of different valves at a constant opening of 50% in dependence of pressure loss is shown in the figure below.
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Fluid/Dissipation/pressureLoss/valve/fig_valve_dp_severalGeometryOverall_MFLOWvsDP.png\" alt=\"fig_valve_dp_severalGeometryOverall_MFLOWvsDP\"/>
</p>

<h4>References</h4>
<dl>
<dt>Miller,D.S.:</dt>
    <dd><b>Internal flow systems</b>.
    Volume 5th of BHRA Fluid Engineering Series. BHRA Fluid Engineering, 1978.</dd>
</dl>
</html>"));
              end dp_severalGeometryOverall;
              annotation(
                DocumentationClass = true);
            end Valve;
            annotation(
              DocumentationClass = true);
          end PressureLoss;
          annotation(
            DocumentationClass = true,
            Documentation(info = "<html>
<p>
This package contains documentation that is used multiple times
in several functions. To avoid duplication of the documentation, links
are used in the respective functions to link to the corresponding shared
documentation available in this package.
</p>

</html>"));
        end SharedDocumentation;

        package Functions "Package for utility functions"
          extends Modelica.Icons.Package;

          package PressureLoss "Package for utility pressure loss functions"
            extends Modelica.Icons.Package;

            package TwoPhase "Package with utility functions to compute two phase pressure loss characteristics"
              extends Modelica.Icons.Package;

              function dp_twoPhaseChisholm_DP "Frictional pressure loss of straight pipe for two phase flow according to Chisholm correlation | calculate pressure loss | overall flow regime"
                extends Modelica.Icons.Function;
                //SOURCE_1: Chisholm,D.:PRESSURE GRADIENTS DUE TO FRICTION DURING THE FLOW OF EVAPORATING TWO-PHASE MIXTURES IN SMOOTH TUBES AND CHANNELS, Int. J. Heat Mass Transfer, Vol. 16, pp. 347-358, Pergamon Press 1973
                //records
                input Modelica.Fluid.Dissipation.Utilities.Records.General.TwoPhaseFlow_con IN_con annotation(
                  Dialog(group = "Constant inputs"));
                input Modelica.Fluid.Dissipation.Utilities.Records.General.TwoPhaseFlow_var IN_var annotation(
                  Dialog(group = "Variable inputs"));
                input SI.MassFlowRate m_flow "Mass flow rate" annotation(
                  Dialog(group = "Input"));
                output SI.Pressure DP "Output for function dp_twoPhaseChisholm_DP";
              protected
                Real MIN = Modelica.Constants.eps;
                Modelica.Fluid.Dissipation.PressureLoss.StraightPipe.dp_overall_IN_con IN_con_1ph(final roughness = Dissipation.Utilities.Types.Roughness.Neglected, final d_hyd = 4 * abs(IN_con.A_cross) / max(MIN, abs(IN_con.perimeter)), final K = 0, final L = abs(IN_con.length));
                Modelica.Fluid.Dissipation.PressureLoss.StraightPipe.dp_overall_IN_var IN_var_1ph(final eta = IN_var.eta_l, final rho = IN_var.rho_l);
              algorithm
                DP := Modelica.Fluid.Dissipation.PressureLoss.StraightPipe.dp_overall_DP(IN_con_1ph, IN_var_1ph, m_flow) * Modelica.Fluid.Dissipation.Utilities.Functions.PressureLoss.TwoPhase.TwoPhaseMultiplierChisholm(IN_con, IN_var, m_flow);
                annotation(
                  Inline = false);
              end dp_twoPhaseChisholm_DP;

              function dp_twoPhaseFriedel_DP "Frictional pressure loss of straight pipe for two phase flow according to Friedel correlation | calculate pressure loss| overall flow regime"
                extends Modelica.Icons.Function;
                //SOURCE_1: Friedel,L.:IMPROVED FRICTION PRESSURE DROP CORRELATIONS FOR HORIZONTAL AND VERTICAL TWO PHASE PIPE FLOW, 3R International, Vol. 18, Issue 7, pp. 485-491, 1979
                //SOURCE_2: VDI-Waermeatlas, 10th edition, Springer-Verlag, 2006.
                import SMOOTH = Modelica.Fluid.Dissipation.Utilities.Functions.General.Stepsmoother;
                import SMOOTH2 = Modelica.Fluid.Dissipation.Utilities.Functions.General.SmoothPower;
                //records
                input Modelica.Fluid.Dissipation.Utilities.Records.General.TwoPhaseFlow_con IN_con annotation(
                  Dialog(group = "Constant inputs"));
                input Modelica.Fluid.Dissipation.Utilities.Records.General.TwoPhaseFlow_var IN_var annotation(
                  Dialog(group = "Variable inputs"));
                input SI.MassFlowRate m_flow "Mass flow rate" annotation(
                  Dialog(group = "Input"));
                output SI.Pressure DP "Output for function dp_twoPhaseFriedel_DP";
              protected
                Real MIN = Modelica.Constants.eps;
                SI.Area A_cross = max(MIN, IN_con.A_cross) "Cross sectional area";
                SI.Diameter d_hyd = max(MIN, 4 * A_cross / max(MIN, IN_con.perimeter)) "Hydraulic diameter";
                Real mdot_A = abs(m_flow) / A_cross "Mass flux";
                SI.ReynoldsNumber Re_liq = max(1, mdot_A * d_hyd / max(MIN, IN_var.eta_l)) "Reynolds number assuming (total) mass flux flowing as liquid";
                SI.ReynoldsNumber Re_lam_leave = 1055 "Maximum Reynolds number for laminar regime (1055)";
                SI.ReynoldsNumber Re_turb = 1100 "Minimum Reynolds number for turbulent regime (1100)";
                SI.ReynoldsNumber Re_smooth = m_flow / A_cross * d_hyd / max(MIN, abs(IN_var.eta_l)) "Reynolds number for smoothing";
                TYP.DarcyFrictionFactor lambda_FRI_lam = 64 / Re_liq "Darcy friction factor for laminar regime";
                TYP.DarcyFrictionFactor lambda_FRI_turb = (0.86859 * Modelica.Math.log(max(1, Re_liq / max(MIN, 1.964 * Modelica.Math.log(Re_liq) - 3.8215)))) ^ (-2) "Darcy friction factor for turbulent regime";
                TYP.DarcyFrictionFactor lambda_FRI = lambda_FRI_lam * SMOOTH(Re_lam_leave, Re_turb, Re_liq) + lambda_FRI_turb * SMOOTH(Re_turb, Re_lam_leave, Re_liq);
                TYP.PressureLossCoefficient zeta_FRI = lambda_FRI * IN_con.length / d_hyd "Pressure loss coefficient";
                SI.Pressure DP_liq = zeta_FRI * mdot_A ^ 2 / (2 * max(MIN, IN_var.rho_l)) "Frictional pressure loss assuming (total) mass flux flowing as liquid";
              algorithm
                DP := SMOOTH2(Re_smooth, 1, 0) * DP_liq * Modelica.Fluid.Dissipation.Utilities.Functions.PressureLoss.TwoPhase.TwoPhaseMultiplierFriedel(IN_con, IN_var, m_flow);
                annotation(
                  Inline = false);
              end dp_twoPhaseFriedel_DP;

              function dp_twoPhaseGeodetic_DP "Geodetic pressure loss of straight pipe for two phase flow | calculate pressure loss"
                extends Modelica.Icons.Function;
                //SOURCE_1: VDI-Waermeatlas, 10th edition, Springer-Verlag, 2006.
                import PI = Modelica.Constants.pi;
                input Modelica.Fluid.Dissipation.Utilities.Types.VoidFractionApproach voidFractionApproach = Dissipation.Utilities.Types.VoidFractionApproach.Homogeneous "Choice of void fraction approach";
                input Boolean crossSectionalAveraged = true "true == cross sectional averaged void fraction | false == volumetric" annotation(
                  Dialog);
                //geometry
                input SI.Length length = 1 "Length in fluid flow direction" annotation(
                  Dialog(group = "Geometry"));
                input SI.Angle phi = 0 "Tilt angle to horizontal" annotation(
                  Dialog(group = "Geometry"));
                //fluid properties
                input SI.Density rho_g(min = Modelica.Constants.eps) "Density of gaseous phase" annotation(
                  Dialog(group = "Fluid properties"));
                input SI.Density rho_l(min = Modelica.Constants.eps) "Density of liquid phase" annotation(
                  Dialog(group = "Fluid properties"));
                input Real x_flow(min = 0, max = 1) = 0 "Mass flow rate quality" annotation(
                  Dialog(group = "Fluid properties"));
                output SI.Pressure DP_geo "Geodetic pressure loss";
              protected
                Real xflow = min(1, max(0, abs(x_flow))) "Mass flow rate quality";
                Real eps = Modelica.Fluid.Dissipation.Utilities.Functions.PressureLoss.TwoPhase.VoidFraction(voidFractionApproach, crossSectionalAveraged, rho_g, rho_l, xflow) "Void fraction";
              algorithm
//SOURCE_1: p.Lbb 1, eq. 4: Considering geodetic pressure loss assuming constant void fraction for flow length
                DP_geo := (eps * rho_g + (1 - eps) * rho_l) * 9.81 * length * sin(phi);
                annotation(
                  Inline = false);
              end dp_twoPhaseGeodetic_DP;

              function dp_twoPhaseMomentum_DP "Momentum pressure loss of straight pipe for two phase flow | calculate pressure loss"
                extends Modelica.Icons.Function;
                //SOURCE_1: VDI-Waermeatlas, 10th edition, Springer-Verlag, 2006.
                //SOURCE_2: Thome, J.R., Engineering Data Book 3, Swiss Federal Institute of Technology Lausanne (EPFL), 2009.
                //SOURCE 3: J.M. Jensen and H. Tummescheit. Moving boundary models for dynamic simulations of two-phase flows. In Proceedings of the 2nd International Modelica Conference, pp. 235-244, Oberpfaffenhofen, Germany, 2002. The Modelica Association.
                import PI = Modelica.Constants.pi;
                import MIN = Modelica.Constants.eps;
                import SMOOTH = Modelica.Fluid.Dissipation.Utilities.Functions.General.SmoothPower;
                //choices
                input Modelica.Fluid.Dissipation.Utilities.Types.VoidFractionApproach voidFractionApproach = Modelica.Fluid.Dissipation.Utilities.Types.VoidFractionApproach.Homogeneous "Choice of void fraction approach" annotation(
                  Dialog(group = "Choices"));
                //SOURCE_3: p.52, eq. 4.6: heterogeneous effects on momentum pressure loss considered through corrected mass flow rate
                input Boolean massFlowRateCorrection = false "Consider heterogeneous mass flow rate correction" annotation(
                  Dialog(group = "Choices"));
                //geometry
                input SI.Area A_cross(min = Modelica.Constants.eps) = PI * 0.1 ^ 2 / 4 "Cross sectional area" annotation(
                  Dialog(group = "Geometry"));
                input SI.Length perimeter(min = Modelica.Constants.eps) = PI * 0.1 "Perimeter" annotation(
                  Dialog(group = "Geometry"));
                //fluid properties
                input SI.Density rho_g(min = Modelica.Constants.eps) "Density of gas" annotation(
                  Dialog(group = "Fluid properties"));
                input SI.Density rho_l(min = Modelica.Constants.eps) "Density of liquid" annotation(
                  Dialog(group = "Fluid properties"));
                input Real x_flow_end(min = 0, max = 1) = 0 "Mass flow rate quality at end of length" annotation(
                  Dialog(group = "Fluid properties"));
                input Real x_flow_sta(min = 0, max = 1) = 0 "Mass flow rate quality at start of length" annotation(
                  Dialog(group = "Fluid properties"));
                input SI.MassFlowRate m_flow "Mass flow rate" annotation(
                  Dialog(group = "Input"));
                output SI.Pressure DP_mom "Momentum pressure loss";
              protected
                Real MIN = Modelica.Constants.eps;
                SI.Area Across = max(MIN, A_cross) "Cross sectional area";
                SI.Diameter d_hyd = max(MIN, 4 * A_cross / max(MIN, perimeter)) "Hydraulic diameter";
                Real mdot_A = abs(m_flow) / Across "Mass flux";
                Real xflowEnd = min(1, max(0, abs(x_flow_end))) "Mass flow rate quality at end of length";
                Real xflowSta = min(1, max(0, abs(x_flow_sta))) "Mass flow rate quality at start of length";
                Real xflowMean = (xflowEnd + xflowSta) / 2 "Mean mass flow rate quality over length";
                Real delta_xflow = xflowEnd - xflowSta "Difference of mass flow rate quality between end and start of length (pos >> evaporation, neg >> condensation";
                //SOURCE_2: Considering void fraction approaches
                Real eps_end = Modelica.Fluid.Dissipation.Utilities.Functions.PressureLoss.TwoPhase.VoidFraction(voidFractionApproach, true, rho_g, rho_l, xflowEnd) "Void fraction at end of length";
                Real eps_sta = Modelica.Fluid.Dissipation.Utilities.Functions.PressureLoss.TwoPhase.VoidFraction(voidFractionApproach, true, rho_g, rho_l, xflowSta) "Void fraction at start of length";
                //SOURCE_2: p.17-6, eq. 17.3.3: Considering mean two phase density at end and start of length
                SI.Density rho_end = Modelica.Fluid.Dissipation.Utilities.Functions.PressureLoss.TwoPhase.TwoPhaseDensity(voidFractionApproach, massFlowRateCorrection, rho_g, rho_l, eps_end, xflowEnd) "Mean two phase density at end of length";
                SI.Density rho_sta = Modelica.Fluid.Dissipation.Utilities.Functions.PressureLoss.TwoPhase.TwoPhaseDensity(voidFractionApproach, massFlowRateCorrection, rho_g, rho_l, eps_sta, xflowSta) "Mean two phase density at start of length";
                SI.Velocity meanVelEnd = abs(m_flow) / max(MIN, rho_end * A_cross) "Mean velocity of two phase flow at end of length";
                SI.Velocity meanVelSta = abs(m_flow) / max(MIN, rho_sta * A_cross) "Mean velocity of two phase flow at start of length";
                //SOURCE 3: p.15, eq. 2.26: Considering velocity difference for heterogeneous approach using slip ratio
                Real SR = Dissipation.Utilities.Functions.PressureLoss.TwoPhase.SlipRatio(voidFractionApproach, rho_g, rho_l, xflowMean) "Slip ratio for velocity void fraction approach";
                SI.Velocity deltaVelEnd = meanVelEnd * (SR - 1) / (xflowEnd * (SR - 1) + 1) "Velocity difference of two phases at end of length";
                SI.Velocity deltaVelSta = meanVelSta * (SR - 1) / (xflowSta * (SR - 1) + 1) "Velocity difference of two phases at start of length";
                //SOURCE 3: p.52, eq. 4.6: Considering of corrected mass flow rate for heterogeneous approach
                SI.MassFlowRate mdotCorEnd = xflowEnd * (1 - xflowEnd) * rho_end * deltaVelEnd * Across "Correction mass flow rate at end of length";
                SI.MassFlowRate mdotCorSta = xflowSta * (1 - xflowSta) * rho_sta * deltaVelSta * Across "Correction mass flow rate at start of length";
                //SOURCE 3: p.53, eq. 4.13: Calculation of heterogeneous approach with correction of mass flow rate for considering velocity difference between fluid phases
                SI.Pressure dp_mom_cor = SMOOTH(delta_xflow, 0.05, 0) * abs(mdot_A * meanVelEnd + mdotCorEnd * deltaVelEnd / Across) - abs(mdot_A * meanVelSta + mdotCorEnd * deltaVelSta / Across) "Momentum pressure loss using mass flow rate correction";
              algorithm
//SOURCE_1: p.Lba 4, eq. 22: Considering momentum pressure loss assuming heterogeneous approach for two phase flow
//Momentum pressure loss occurs for a changing mass flow rate quality due to condensation or evaporation
//At evaporation the liquid phase with a slow velocity has to be accelerated to the higher velocity of the gas
//The difference in static pressure at the outlet and the inlet causes a positive momentum pressure loss at evaporation (assumed vice versa for condensation)
                DP_mom := if massFlowRateCorrection then dp_mom_cor else mdot_A ^ 2 * SMOOTH(delta_xflow, 0.05, 0) * abs(1 / max(MIN, rho_end) - 1 / max(MIN, rho_sta));
                annotation(
                  Inline = false,
                  Documentation(revisions = "<html>
2012-11-28 Corrected an error in momentum pressure loss calculation. Stefan Wischhusen.
</html>"));
              end dp_twoPhaseMomentum_DP;

              function TwoPhaseMultiplierFriedel "Calculation of two phase multiplier according to Friedel | constant mass flow rate quality | horizontal flow | vertical upflow and downflow"
                extends Modelica.Icons.Function;
                //SOURCE_1: Friedel,L.:IMPROVED FRICTION PRESSURE DROP CORRELATIONS FOR HORIZONTAL AND VERTICAL TWO PHASE PIPE FLOW, 3R International, Vol. 18, Issue 7, pp. 485-491, 1979
                //SOURCE_2: VDI-Waermeatlas, 10th edition, Springer-Verlag, 2006.
                import Modelica.Math.log;
                import SMOOTH = Modelica.Fluid.Dissipation.Utilities.Functions.General.Stepsmoother;
                input Modelica.Fluid.Dissipation.Utilities.Records.General.TwoPhaseFlow_con IN_con annotation(
                  Dialog(group = "Constant inputs"));
                input Modelica.Fluid.Dissipation.Utilities.Records.General.TwoPhaseFlow_var IN_var(final sigma = 0) annotation(
                  Dialog(group = "Variable inputs"));
                input SI.MassFlowRate m_flow "Mass flow rate" annotation(
                  Dialog(group = "Input"));
                output Real phi "Two phase multiplier w.r.t. Friedel";
              protected
                Real MIN = Modelica.Constants.eps;
                SI.Area A_cross = max(MIN, IN_con.A_cross) "Cross sectional area";
                SI.Diameter d_hyd = max(MIN, 4 * A_cross / max(MIN, IN_con.perimeter)) "Hydraulic diameter";
                //SOURCE_2: p.Lba 4, sec. 3.3: Correlation based on constant mass flow rate quality (x_flow) for increment (dx)
                //Pressure loss for total length (e.g., L=n*dx) can be achieved by discretization
                Real mdot_A = abs(m_flow) / A_cross "Mass flux";
                Real x_flow = max(0, min(1, abs(IN_var.x_flow))) "Mass flow rate quality";
                //SOURCE_1: p.490 (Appendix): Characteristic numbers based on total mass flow rate flowing as liquid
                SI.FroudeNumber Fr_l = max(MIN, mdot_A ^ 2 / max(MIN, 9.81 * IN_var.rho_l ^ 2 * d_hyd)) "Froude number based on liquid flow";
                SI.ReynoldsNumber Re_g = max(1, mdot_A * d_hyd / max(MIN, IN_var.eta_g)) "Reynolds number based on gas flow";
                SI.ReynoldsNumber Re_l = max(1, mdot_A * d_hyd / max(MIN, IN_var.eta_l)) "Reynolds number based on liquid flow";
                SI.WeberNumber We_l = max(MIN, mdot_A ^ 2 * d_hyd / max(MIN, IN_var.sigma * IN_var.rho_l)) "Weber number based on liquid flow";
                //SOURCE_1: p.490 (Appendix): Smoothing for sudden change from assumed laminar to assumed turbulent regime (numerical improvement at Re=1055)
                SI.ReynoldsNumber Re_lam_max = 1025 "Maximum Reynolds number assuming laminar regime";
                SI.ReynoldsNumber Re_turb_min = 1075 "Minimum Reynolds number assuming turbulent regime";
                //SOURCE_2: p.Lbb 2, eq. 9-10: Considering influence of Reynolds number on Darcy friction factor for smooth straight pipes
                //Correlation based on neglection of surface roughness
                //Correlation based on assumption that total mass flow rate is flowing as gas
                TYP.DarcyFrictionFactor lambda_lam_g = 64 / Re_g "Darcy friction factor of gas for assumed laminar regime";
                TYP.DarcyFrictionFactor lambda_turb_g = 1 / max(MIN, 0.86859 * log(max(1, Re_g / max(MIN, 1.964 * log(Re_g) - 3.8215)))) ^ 2 "Darcy friction factor of gas for assumed turbulent regime";
                TYP.DarcyFrictionFactor lambda_g = lambda_lam_g * SMOOTH(Re_lam_max, Re_turb_min, Re_g) + lambda_turb_g * SMOOTH(Re_turb_min, Re_lam_max, Re_g) "Darcy friction factor of gas for overall regime";
                //Correlation based on assumption that total mass flow rate is flowing as liquid
                TYP.DarcyFrictionFactor lambda_lam_l = 64 / Re_l "Darcy friction factor of liquid for assumed laminar regime";
                TYP.DarcyFrictionFactor lambda_turb_l = 1 / max(MIN, 0.86859 * log(max(1, Re_l / max(MIN, 1.964 * log(Re_l) - 3.8215)))) ^ 2 "Darcy friction factor of liquid for assumed turbulent regime";
                TYP.DarcyFrictionFactor lambda_l = lambda_lam_l * SMOOTH(Re_lam_max, Re_turb_min, Re_l) + lambda_turb_l * SMOOTH(Re_turb_min, Re_lam_max, Re_l) "Darcy friction factor of liquid for overall regime";
                Real A = (1 - x_flow) ^ 2 + x_flow ^ 2 * (IN_var.rho_l / max(MIN, IN_var.rho_g)) * (lambda_g / max(MIN, lambda_l)) "Summand for two phase multiplier";
                //SOURCE_1: p.490 (Appendix): Two phase multiplier for vertical downflow for future usage
                Real phi_vdo = A + 38.5 * x_flow ^ 0.76 * (1 - x_flow) ^ 0.314 * (IN_var.rho_l / max(MIN, IN_var.rho_g)) ^ 0.86 * (IN_var.eta_g / max(MIN, IN_var.eta_l)) ^ 0.73 * (1 - IN_var.eta_g / max(MIN, IN_var.eta_l)) ^ 6.84 * (1 / Fr_l ^ 0.0001) * (1 / We_l ^ 0.087);
                //SOURCE_1: p.490 (Appendix): Two phase multiplier for horizontal and vertical upflow (failure in SOURCE_2)
                Real phi_vup = A + 3.43 * x_flow ^ 0.685 * (1 - x_flow) ^ 0.24 * (IN_var.rho_l / max(MIN, IN_var.rho_g)) ^ 0.8 * (IN_var.eta_g / max(MIN, IN_var.eta_l)) ^ 0.22 * (1 - IN_var.eta_g / max(MIN, IN_var.eta_l)) ^ 0.89 * (1 / Fr_l ^ 0.048) * (1 / We_l ^ 0.0334);
              algorithm
                phi := phi_vup;
                annotation(
                  Inline = false);
              end TwoPhaseMultiplierFriedel;

              function TwoPhaseMultiplierChisholm "Calculation of two phase multiplier according to Chisholm | constant mass flow rate quality"
                extends Modelica.Icons.Function;
                //SOURCE_1: Chisholm,D.:PRESSURE GRADIENTS DUE TO FRICTION DURING THE FLOW OF EVAPORATING TWO-PHASE MIXTURES IN SMOOTH TUBES AND CHANNELS, Int. J. Heat Mass Transfer, Vol. 16, pp. 347-358, Pergamon Press 1973
                //SOURCE_2: VDI-Waermeatlas, 10th edition, Springer-Verlag, 2006.
                import SMOOTH = Modelica.Fluid.Dissipation.Utilities.Functions.General.Stepsmoother;
                input Modelica.Fluid.Dissipation.Utilities.Records.General.TwoPhaseFlow_con IN_con annotation(
                  Dialog(group = "Constant inputs"));
                input Modelica.Fluid.Dissipation.Utilities.Records.General.TwoPhaseFlow_var IN_var annotation(
                  Dialog(group = "Variable inputs"));
                input SI.MassFlowRate m_flow "Mass flow rate" annotation(
                  Dialog(group = "Input"));
                output Real phi "Two phase multiplier w.r.t. Chisholm";
              protected
                Real MIN = Modelica.Constants.eps;
                SI.Area A_cross = max(MIN, IN_con.A_cross) "Cross sectional area";
                SI.Diameter d_hyd = max(MIN, 4 * A_cross / max(MIN, IN_con.perimeter)) "Hydraulic diameter";
                Real mdot_A = abs(m_flow) / A_cross "Mass flux";
                Real x_flow = max(0, min(1, abs(IN_var.x_flow))) "Mass flow rate quality";
                //SOURCE_1: p.357, Appendix 1: Considering Darcy friction factor (lambda_FRI) of smooth pipes for overall flow regime
                Real n_exp = 0.2 "Exponent for Reynolds number (lambda_FRI= A/Re^n)";
                //SOURCE_1: p.349, eq. 21: Considering effect of physical properties (failure in SOURCE_2)
                Real gamma = max(1, abs(IN_var.rho_l / max(MIN, IN_var.rho_g)) ^ 0.5 * (IN_var.eta_g / max(MIN, IN_var.eta_l)) ^ (n_exp / 2));
                //SOURCE: p. 353, tab. 2: Considering effect of mass flux on two phase multiplier
                Real B_gamma_1 = SMOOTH(450, 550, mdot_A) * 4.8 + SMOOTH(550, 450, mdot_A) * 2400 / max(MIN, mdot_A) - SMOOTH(1950, 1850, mdot_A) * 2400 / max(MIN, mdot_A) + SMOOTH(1950, 1850, mdot_A) * 55 / max(MIN, mdot_A ^ 0.5) "Coefficient B for gamma <= 9.5";
                Real B_gamma_2 = SMOOTH(550, 650, mdot_A) * 520 / max(1, max(9.5, gamma) * mdot_A ^ 0.5) + SMOOTH(650, 550, mdot_A) * 21 / max(9.5, gamma) "Coefficient B for 9.5 <= gamma <= 28";
                Real B_gamma = SMOOTH(9.0, 10, gamma) * B_gamma_1 + SMOOTH(10, 9.0, gamma) * B_gamma_2 - SMOOTH(28.5, 27.7, gamma) * B_gamma_2 + SMOOTH(28.5, 27.5, gamma) * 15000 / max(MIN, gamma ^ 2 * mdot_A ^ 0.5) "Coefficient B for gamma";
                //SOURCE_1: p. 350, eq. 24/26: Considering two phase multiplier w.r.t. Chisholm
              algorithm
                phi := 1 + (gamma ^ 2 - 1) * (B_gamma * x_flow ^ ((2 - n_exp) / 2) * (1 - x_flow) ^ ((2 - n_exp) / 2) + x_flow ^ (2 - n_exp));
                annotation(
                  Inline = false);
              end TwoPhaseMultiplierChisholm;

              function TwoPhaseDensity "Calculation of mean density for two phase flow"
                extends Modelica.Icons.Function;
                //SOURCE_1: VDI-Waermeatlas, 10th edition, Springer-Verlag, 2006.
                input Modelica.Fluid.Dissipation.Utilities.Types.VoidFractionApproach voidFractionApproach = Modelica.Fluid.Dissipation.Utilities.Types.VoidFractionApproach.Homogeneous "Choice of void fraction approach" annotation(
                  Dialog(group = "Choices"));
                //SOURCE_3: p.52, eq. 4.6: heterogeneous effects on momentum pressure loss considered through corrected mass flow rate
                input Boolean massFlowRateCorrection = false "Consider heterogeneous mass flow rate correction" annotation(
                  Dialog(group = "Choices"));
                input SI.Density rho_g(min = Modelica.Constants.eps) "Density of gaseous phase" annotation(
                  Dialog);
                input SI.Density rho_l(min = Modelica.Constants.eps) "Density of liquid phase" annotation(
                  Dialog);
                input Real epsilon_A(min = 0, max = 1) "Void fraction (cross sectional averaged)" annotation(
                  Dialog(enable = not twoPhaseDensityApproach == Modelica.Fluid.Dissipation.Utilities.Types.TwoPhaseDensityApproach.Homogeneous));
                input Real x_flow(min = 0, max = 1) "Mass flow rate quality" annotation(
                  Dialog);
                output SI.Density rho_2ph "Mean density of two phase flow";
              protected
                Real MIN = Modelica.Constants.eps;
                Real epsilonA = min(1, max(0, abs(epsilon_A))) "Void fraction (cross sectional averaged)";
                Real xflow = min(1, max(0, abs(x_flow))) "Mass flow rate quality";
                //SOURCE_1: p.Lba 3, eq. 17: Mean two phase density assuming homogeneous approach
                SI.Density rho_hom = 1 / max(MIN, x_flow / max(MIN, rho_g) + (1 - x_flow) / max(MIN, rho_l));
                //SOURCE_1: p.Lbb 7, tab. 2: Mean two phase density assuming momentum flux approach
                SI.Density rho_mom = 1 / max(MIN, x_flow ^ 2 / max(MIN, rho_g * epsilonA) + (1 - x_flow) ^ 2 / max(MIN, rho_l * (1 - epsilonA)));
                //SOURCE_1: p.Lbb 7, tab. 2: Mean two phase density assuming kinetic energy flow approach from Zivi (corrected formula!)
                SI.Density rho_kin = 1 / max(MIN, rho_hom * (x_flow ^ 3 / max(MIN, rho_g ^ 2 * epsilonA ^ 2) + (1 - x_flow) ^ 3 / max(MIN, rho_l ^ 2 * (1 - epsilonA) ^ 2)));
              algorithm
                rho_2ph := if not massFlowRateCorrection then rho_hom else if voidFractionApproach == Modelica.Fluid.Dissipation.Utilities.Types.VoidFractionApproach.Homogeneous then rho_hom else if voidFractionApproach == Modelica.Fluid.Dissipation.Utilities.Types.VoidFractionApproach.Momentum then rho_mom else if voidFractionApproach == Modelica.Fluid.Dissipation.Utilities.Types.VoidFractionApproach.Energy then rho_kin else MIN;
                annotation(
                  Inline = false,
                  Documentation(info = "<html>
<p>
The gaseous and the liquid part of a fluid in a two phase flow are often discontinuously distributed. This complex behaviour is simplified for engineering calculations. The two phase flow of different fluid flow situations (e.g., bubble or stratified flow) is modelled as if the gaseous and the liquid phase are continuously distributed.
</p>

<p>
A <b> mean density </b> assuming a continuous distribution out of a discontinuous two phase fluid flow situation can be calculated with a <b> homogeneous or a heterogeneous approach </b> (see <a href=\"modelica://Modelica.Fluid.Dissipation.PressureLoss.StraightPipe.dp_twoPhaseOverall_DP\">dp_twoPhaseOverall_DP</a>).</p>
<p>
The following <b> modelling approaches </b> can be used to calculate the mean density of two phase flow:
</p>
<ul>
<li>        <b> homogeneous density </b> (homogeneous approach) </li>
<li>         <b> momentum flux density </b> (heterogeneous approach) </li>
<li>         <b> kinetic energy flow density </b> (heterogeneous approach) </li>
</ul>


<p>
The heterogeneous approaches are analytically derived by minimising the momentum flux or the kinetic energy flow assuming implicitly that the two-phase flow will tend towards the minimum of this quantity.
</p>

<h4>References</h4>
<dl>
 <dt>VDI:</dt>
    <dd><b>VDI - W&auml;rmeatlas: Berechnungsbl&auml;tter f&uuml;r den W&auml;rme&uuml;bergang</b>.
    Springer Verlag, 10th edition, 2006.</dd>
</dl>
</html>", revisions = "<html>
<ul>
<li><i>2 May 2011</i>
    by Stefan Wischhusen:<br>
       Corrected a logical error in use of input <code>massFlowRateCorrection</code>.</li>
</ul>
</html>"));
              end TwoPhaseDensity;

              function VoidFraction "Calculation of (cross sectional) void fraction for two phase flow"
                extends Modelica.Icons.Function;
                //SOURCE_1: VDI-Waermeatlas, 10th edition, Springer-Verlag, 2006.
                //SOURCE_2: Thome, J.R., Engineering Data Book 3, Swiss Federal Institute of Technology Lausanne (EPFL), 2009.
                input Modelica.Fluid.Dissipation.Utilities.Types.VoidFractionApproach voidFractionApproach = Dissipation.Utilities.Types.VoidFractionApproach.Homogeneous "Choice of void fraction approach" annotation(
                  Dialog(group = "Choices"));
                input Boolean crossSectionalAveraged = true "true == cross sectional averaged void fraction | false == volumetric" annotation(
                  Dialog);
                input SI.Density rho_g(min = Modelica.Constants.eps) "Density of gaseous phase" annotation(
                  Dialog);
                input SI.Density rho_l(min = Modelica.Constants.eps) "Density of liquid phase" annotation(
                  Dialog);
                input Real x_flow(min = 0, max = 1) = 0 "Mass flow rate quality" annotation(
                  Dialog);
                output Real epsilon "Void fraction";
                output Real slipRatio "Slip ratio";
              protected
                Real MIN = Modelica.Constants.eps;
                Real xflow = min(1, max(0, abs(x_flow))) "Mass flow rate quality";
                Real SR = Dissipation.Utilities.Functions.PressureLoss.TwoPhase.SlipRatio(voidFractionApproach, rho_g, rho_l, xflow) "Slip ratio for void fraction approach";
                //SOURCE_2: p.17-5, eq. 17.2.5: (Heterogeneous) cross sectional void fraction [epsilon_A=A_g/(A_g+A_l)]
                Real epsilon_A = rho_l * x_flow / max(MIN, rho_l * x_flow + rho_g * (1 - x_flow) * SR);
              algorithm
                epsilon := if crossSectionalAveraged then epsilon_A else epsilon_A / (1 / max(MIN, SR) * (1 - epsilon_A) + epsilon_A);
                slipRatio := SR;
                annotation(
                  Inline = false);
              end VoidFraction;

              function SlipRatio "Calculation of (analytical/empirical) slip ratio"
                extends Modelica.Icons.Function;
                //SOURCE_1: VDI-Waermeatlas, 10th edition, Springer-Verlag, 2006.
                //SOURCE_2: Thome, J.R., Engineering Data Book 3, Swiss Federal Institute of Technology Lausanne (EPFL), 2009.
                //SOURCE 3: J.M. Jensen and H. Tummescheit. Moving boundary models for dynamic simulations of two-phase flows. In Proceedings of the 2nd International Modelica Conference, pp. 235-244, Oberpfaffenhofen, Germany, 2002. The Modelica Association.
                input Modelica.Fluid.Dissipation.Utilities.Types.VoidFractionApproach voidFractionApproach = Dissipation.Utilities.Types.VoidFractionApproach.Homogeneous "Choice of void fraction approach" annotation(
                  Dialog(group = "Choices"));
                input SI.Density rho_g(min = Modelica.Constants.eps) "Density of gaseous phase" annotation(
                  Dialog);
                input SI.Density rho_l(min = Modelica.Constants.eps) "Density of liquid phase" annotation(
                  Dialog);
                input Real x_flow = 0 "Mass flow rate quality" annotation(
                  Dialog);
                output Real SR "Slip ratio";
              protected
                Real MIN = Modelica.Constants.eps;
                //SOURCE_1: p.Lba 3, sec. 3.2
                Real SR_hom = 1 "Slip ratio w.r.t. homogeneous approach";
                //SOURCE_2: p.17-6, eq. 17.3.4
                Real SR_mom = abs(rho_l / max(MIN, rho_g)) ^ 0.5 "Slip ratio w.r.t. momentum flux approach (heterogeneous)";
                //SOURCE_2: p.17-7, eq. 17.3.13
                Real SR_kin = abs(rho_l / max(MIN, rho_g)) ^ (1 / 3) "Slip ratio w.r.t. kinetic energy flow approach from Zivi (heterogeneous)";
                //SOURCE_2: p.17-10, eq. 17.4.3
                Real SR_chi = (1 - x_flow * (1 - abs(rho_l) / max(MIN, abs(rho_g)))) ^ 0.5 "Empirical slip ratio w.r.t. momentum flux approach from Chisholm (heterogeneous)";
              algorithm
                SR := if voidFractionApproach == Modelica.Fluid.Dissipation.Utilities.Types.VoidFractionApproach.Homogeneous then SR_hom else if voidFractionApproach == Modelica.Fluid.Dissipation.Utilities.Types.VoidFractionApproach.Momentum then SR_mom else if voidFractionApproach == Modelica.Fluid.Dissipation.Utilities.Types.VoidFractionApproach.Energy then SR_kin else if voidFractionApproach == Modelica.Fluid.Dissipation.Utilities.Types.VoidFractionApproach.Chisholm then SR_chi else 1;
                annotation(
                  Inline = false);
              end SlipRatio;
            end TwoPhase;
          end PressureLoss;

          package HeatTransfer "Package for utility heat transfer functions"
            extends Modelica.Icons.Package;

            package TwoPhase "Package with utility functions to compute two phase heat transfer characteristics"
              extends Modelica.Icons.Package;

              function kc_twoPhase_condensationHorizontal_KC "Local two phase heat transfer coefficient of straight pipe | horizontal condensation"
                extends Modelica.Icons.Function;
                //SOURCE_1: M.M. Shah. A general correlation for heat transfer during film condensation inside pipes.Int. J. Heat Mass Transfer, Vol.22, p.547-556, 1979.
                //records
                input Modelica.Fluid.Dissipation.Utilities.Records.HeatTransfer.TwoPhaseFlowHT_IN_con IN_con annotation(
                  Dialog(group = "Constant inputs"));
                input Modelica.Fluid.Dissipation.Utilities.Records.HeatTransfer.TwoPhaseFlowHT_IN_var IN_var annotation(
                  Dialog(group = "Variable inputs"));
                output SI.CoefficientOfHeatTransfer kc "Local two phase heat transfer coefficient";
              protected
                Real MIN = Modelica.Constants.eps;
                SI.Area A_cross = max(MIN, IN_con.A_cross) "Cross sectional area";
                SI.Diameter d_hyd = max(MIN, 4 * A_cross / max(MIN, IN_con.perimeter)) "Hydraulic diameter";
                Real x_flow = max(0, min(1, abs(IN_var.x_flow))) "Mass flow rate quality";
                Real p_red = max(MIN, abs(IN_var.pressure) / max(MIN, abs(IN_con.p_crit))) "Reduced pressure";
                SI.Velocity velocity = abs(IN_var.m_flow) / max(MIN, IN_var.rho_l * A_cross) "Mean velocity";
                SI.ReynoldsNumber Re_l = max(1, IN_var.rho_l * velocity * d_hyd / max(MIN, IN_var.eta_l)) "Reynolds number assuming (total) mass flux flowing as liquid";
                SI.PrandtlNumber Pr_l = abs(IN_var.eta_l * IN_var.cp_l / max(MIN, IN_var.lambda_l)) "Prandtl number assuming (total) mass flux flowing as liquid";
                //SOURCE_1: p.548, eq. 8: Considering two phase multiplier for condensation w.r.t. Shah
                SI.CoefficientOfHeatTransfer kc_1ph = 0.023 * Re_l ^ 0.8 * Pr_l ^ 0.4 * IN_var.lambda_l / d_hyd;
              algorithm
                kc := kc_1ph * ((1 - x_flow) ^ 0.8 + 3.8 * x_flow ^ 0.76 * (1 - x_flow) ^ 0.04 / p_red ^ 0.38);
                annotation(
                  Inline = false,
                  smoothOrder = 5);
              end kc_twoPhase_condensationHorizontal_KC;

              function kc_twoPhase_boilingVertical_KC "Local two phase heat transfer coefficient of straight pipe | vertical boiling"
                extends Modelica.Icons.Function;
                //SOURCE_1: Bejan,A.: HEAT TRANSFER HANDBOOK, Wiley, 2003.
                //SOURCE_2: Gungor, K.E. and R.H.S. Winterton: A general correlation for flow boiling in tubes and annuli, Int.J. Heat Mass Transfer, Vol.29, p.351-358, 1986.
                //records
                input Modelica.Fluid.Dissipation.Utilities.Records.HeatTransfer.TwoPhaseFlowHT_IN_con IN_con annotation(
                  Dialog(group = "Constant inputs"));
                input Modelica.Fluid.Dissipation.Utilities.Records.HeatTransfer.TwoPhaseFlowHT_IN_var IN_var annotation(
                  Dialog(group = "Variable inputs"));
                output SI.CoefficientOfHeatTransfer kc "Local two phase heat transfer coefficient";
              protected
                Real MIN = Modelica.Constants.eps;
                SI.Area A_cross = max(MIN, IN_con.A_cross) "Cross sectional area";
                SI.Diameter d_hyd = max(MIN, 4 * A_cross / max(MIN, IN_con.perimeter)) "Hydraulic diameter";
                Real mdot_A = abs(IN_var.m_flow) / A_cross "Mass flux";
                Real x_flow = max(0, min(1, abs(IN_var.x_flow))) "Mass flow rate quality";
                Real p_red = max(MIN, abs(IN_var.pressure) / max(MIN, abs(IN_con.p_crit))) "Reduced pressure";
                //SOURCE_1: p.674, sec. 9.8.3: Considering nucleate and convective boiling w.r.t. equation of Gungor-Winterton
                SI.MassFlowRate mdot_l = abs(IN_var.m_flow) * (1 - x_flow) "Mass flow rate of liquid only";
                SI.Velocity velocity_l = mdot_l / max(MIN, IN_var.rho_l * A_cross) "Mean velocity assuming liquid mass flow rate flows alone";
                SI.ReynoldsNumber Re_l = max(1, IN_var.rho_l * velocity_l * d_hyd / max(MIN, IN_var.eta_l)) "Reynolds number assuming liquid mass flow rate flows alone";
                SI.PrandtlNumber Pr_l = abs(IN_var.eta_l * IN_var.cp_l / max(MIN, IN_var.lambda_l)) "Prandtl number assuming liquid mass flow rate flows alone";
                //SOURCE_1: p.674, eq. 9.98: Considering effect of heat flux on nucleate boiling with Boiling number
                //Boiling number (Bo) is defined as ratio of actual heat flux to maximum heat flux necessary for complete evaporation of liquid
                Real Bo = abs(IN_var.qdot_A) / max(MIN, mdot_A * IN_var.dh_lg) "Boiling number";
                //SOURCE_1: p.673, eq. 9.94: Considering of Martinelli parameter w.r.t. equation of Chen
                Real X_tt = abs(((1 - x_flow) / max(MIN, x_flow)) ^ 0.9 * (IN_var.rho_g / max(MIN, IN_var.rho_l)) ^ 0.5 * (IN_var.eta_l / max(MIN, IN_var.eta_g)) ^ 0.1) "Martinelli parameter";
                //SOURCE_1: p.675, eq. 9.105: Considering of convection enhancement factor w.r.t. equation of of Gungor-Winterton
                Real E_fc = 1 + 24000 * Bo ^ 1.16 + 1.37 * (1 / max(MIN, X_tt)) ^ 0.86 "Enhancement factor for forced convection";
                //SOURCE_1: p.675, eq. 9.105: Considering of boiling suppression factor w.r.t. equation of of Gungor-Winterton
                Real S_nb = 1 / max(MIN, 1 + 1.15e-6 * E_fc ^ 2 * Re_l ^ 1.17) "Suppression factor for nucleate boiling";
                //SOURCE_1: p.672, eq. 9.91: Considering effect of forced convective boiling ew.r.t. equation of Dittus-Boelter
                SI.CoefficientOfHeatTransfer kc_fc = 0.023 * Re_l ^ 0.8 * Pr_l ^ 0.4 * (IN_var.lambda_l / d_hyd) "Convective heat transfer coefficient assuming liquid mass flow rate only";
                //SOURCE_1: p.675, eq. 9.107: Considering effect of nucleate boiling w.r.t. equation of Cooper
                SI.CoefficientOfHeatTransfer kc_nb = 55 * p_red ^ 0.12 * (1 / max(MIN, Modelica.Math.log10(1 / p_red)) ^ 0.55) * (1 / max(MIN, IN_con.MM) ^ 0.5) * IN_var.qdot_A ^ 0.67 "Nucleate boiling heat transfer coefficient";
                //SOURCE_2: p.354, sec. final equations: Calculation of two phase heat transfer coefficient for vertical pipes w.r.t. equation of Gungor-Winterton
              algorithm
                kc := E_fc * kc_fc + S_nb * kc_nb;
                annotation(
                  Inline = false,
                  smoothOrder = 5);
              end kc_twoPhase_boilingVertical_KC;

              function kc_twoPhase_boilingHorizontal_KC "Local two phase heat transfer coefficient of straight pipe | horizontal boiling"
                extends Modelica.Icons.Function;
                //SOURCE_1: Bejan,A.: HEAT TRANSFER HANDBOOK, Wiley, 2003.
                //SOURCE_2: Gungor, K.E. and R.H.S. Winterton: A general correlation for flow boiling in tubes and annuli, Int.J. Heat Mass Transfer, Vol.29, p.351-358, 1986.
                import SMOOTH = Modelica.Fluid.Dissipation.Utilities.Functions.General.Stepsmoother;
                //records
                input Modelica.Fluid.Dissipation.Utilities.Records.HeatTransfer.TwoPhaseFlowHT_IN_con IN_con annotation(
                  Dialog(group = "Constant inputs"));
                input Modelica.Fluid.Dissipation.Utilities.Records.HeatTransfer.TwoPhaseFlowHT_IN_var IN_var annotation(
                  Dialog(group = "Variable inputs"));
                output SI.CoefficientOfHeatTransfer kc "Local two phase heat transfer coefficient";
              protected
                Real MIN = Modelica.Constants.eps;
                SI.Area A_cross = max(MIN, IN_con.A_cross) "Cross sectional area";
                SI.Diameter d_hyd = max(MIN, 4 * A_cross / max(MIN, IN_con.perimeter)) "Hydraulic diameter";
                Real mdot_A = abs(IN_var.m_flow) / A_cross "Mass flux";
                Real x_flow = max(0, min(1, abs(IN_var.x_flow))) "Mass flow rate quality";
                Real p_red = max(MIN, abs(IN_var.pressure) / max(MIN, abs(IN_con.p_crit))) "Reduced pressure";
                //SOURCE_1: p.674, sec. 9.8.3: Considering nucleate and convective boiling w.r.t. equation of Gungor-Winterton
                SI.MassFlowRate mdot_l = abs(IN_var.m_flow) * (1 - x_flow) "Mass flow rate of liquid only";
                SI.Velocity velocity_l = mdot_l / max(MIN, IN_var.rho_l * A_cross) "Mean velocity assuming liquid mass flow rate flows alone";
                SI.ReynoldsNumber Re_l = max(1, IN_var.rho_l * velocity_l * d_hyd / max(MIN, IN_var.eta_l)) "Reynolds number assuming liquid mass flow rate flows alone";
                SI.PrandtlNumber Pr_l = abs(IN_var.eta_l * IN_var.cp_l / max(MIN, IN_var.lambda_l)) "Prandtl number assuming liquid mass flow rate flows alone";
                //SOURCE_1: p.352, sec. Nomenclature: Considering effect of stratification w.r.t. Froude number
                SI.FroudeNumber Fr_l = abs(mdot_A ^ 2 / max(MIN, IN_var.rho_l ^ 2 * 9.81 * d_hyd)) "Froude number assuming (total) mass flux flowing as liquid";
                //SOURCE_1: p.674, eq. 9.98: Considering effect of heat flux on nucleate boiling with Boiling number
                //Boiling number (Bo) is defined as ratio of actual heat flux to maximum heat flux necessary for complete evaporation of liquid
                Real Bo = abs(IN_var.qdot_A) / max(MIN, mdot_A * IN_var.dh_lg) "Boiling number";
                //SOURCE_1: p.673, eq. 9.94: Considering of Martinelli parameter w.r.t. equation of Chen
                Real X_tt = abs(((1 - x_flow) / max(MIN, x_flow)) ^ 0.9 * (IN_var.rho_g / max(MIN, IN_var.rho_l)) ^ 0.5 * (IN_var.eta_l / max(MIN, IN_var.eta_g)) ^ 0.1) "Martinelli parameter";
                //SOURCE_1: p.675, eq. 9.105: Considering of convection enhancement factor w.r.t. equation of Gungor-Winterton
                Real E_fc = 1 + 24000 * Bo ^ 1.16 + 1.37 * (1 / max(MIN, X_tt)) ^ 0.86 "Enhancement factor for forced convection";
                //SOURCE_1: p.675, eq. 9.105: Considering of boiling suppression factor w.r.t. equation of Gungor-Winterton
                Real S_nb = 1 / max(MIN, 1 + 1.15e-6 * E_fc ^ 2 * Re_l ^ 1.17) "Suppression factor for nucleate boiling";
                //SOURCE_1: p.680, eq. 9.123: Considering correction of convection enhancement factor for horizontal pipes
                Real E_fc_hor = SMOOTH(0.049, 0.051, Fr_l) * Fr_l ^ max(0, abs(0.1 - 2 * Fr_l)) + SMOOTH(0.051, 0.049, Fr_l) "Correction of enhancement factor for forced convection in horizontal pipes";
                //SOURCE_1: p.680, eq. 9.124: Considering correction of boiling suppression factor for horizontal pipes
                Real S_nb_hor = SMOOTH(0.049, 0.051, Fr_l) * Fr_l ^ 0.5 + SMOOTH(0.051, 0.049, Fr_l) "Correction of suppression factor for nucleate boiling in horizontal pipes";
                //SOURCE_1: p.672, eq. 9.91: Considering effect of forced convective boiling ew.r.t. equation of Dittus-Boelter
                SI.CoefficientOfHeatTransfer kc_fc = 0.023 * Re_l ^ 0.8 * Pr_l ^ 0.4 * (IN_var.lambda_l / d_hyd) "Convective heat transfer coefficient assuming liquid mass flow rate only";
                //SOURCE_1: p.675, eq. 9.107: Considering effect of nucleate boiling w.r.t. equation of Cooper
                SI.CoefficientOfHeatTransfer kc_nb = 55 * p_red ^ 0.12 * (1 / max(MIN, Modelica.Math.log10(1 / p_red)) ^ 0.55) * (1 / max(MIN, IN_con.MM ^ 0.5)) * abs(IN_var.qdot_A) ^ 0.67 "Nucleate boiling heat transfer coefficient";
                //SOURCE_2: p.354, sec. final equations: Calculation of two phase heat transfer coefficient for horizontal pipes w.r.t. equation of Gungor-Winterton
              algorithm
                kc := E_fc * E_fc_hor * kc_fc + S_nb * S_nb_hor * kc_nb;
                annotation(
                  Inline = false,
                  smoothOrder = 5);
              end kc_twoPhase_boilingHorizontal_KC;
            end TwoPhase;
          end HeatTransfer;

          package General
            extends Modelica.Icons.Package;

            function CubicInterpolation_DP
              extends Modelica.Icons.Function;
              import Modelica.Math;
              input Real Re_turbulent;
              input SI.ReynoldsNumber Re1;
              input SI.ReynoldsNumber Re2;
              input Real Delta;
              input Real lambda2;
              output SI.ReynoldsNumber Re;
              // point lg(lambda2(Re1)) with derivative at lg(Re1)
            protected
              Real x1 = Math.log10(64 * Re1);
              Real y1 = Math.log10(Re1);
              Real yd1 = 1;
              // Point lg(lambda2(Re2)) with derivative at lg(Re2)
              Real aux1 = 0.5 / Math.log(10) * 5.74 * 0.9;
              Real aux2 = Delta / 3.7 + 5.74 / Re2 ^ 0.9;
              Real aux3 = Math.log10(aux2);
              Real L2 = 0.25 * (Re2 / aux3) ^ 2;
              Real aux4 = 2.51 / sqrt(L2) + 0.27 * Delta;
              Real aux5 = -2 * sqrt(L2) * Math.log10(aux4);
              Real x2 = Math.log10(L2);
              Real y2 = Math.log10(aux5);
              Real yd2 = 0.5 + 2.51 / Math.log(10) / (aux5 * aux4);
              // Constants: Cubic polynomial between lg(Re1) and lg(Re2)
              Real diff_x = x2 - x1;
              Real m = (y2 - y1) / diff_x;
              Real c2 = (3 * m - 2 * yd1 - yd2) / diff_x;
              Real c3 = (yd1 + yd2 - 2 * m) / (diff_x * diff_x);
              Real lambda2_1 = 64 * Re1;
              Real dx = Math.log10(lambda2 / lambda2_1);
            algorithm
              Re := Re1 * (lambda2 / lambda2_1) ^ (1 + dx * (c2 + dx * c3));
              annotation(
                Inline = false,
                smoothOrder = 5);
            end CubicInterpolation_DP;

            function CubicInterpolation_MFLOW
              extends Modelica.Icons.Function;
              import Modelica.Math;
              input SI.ReynoldsNumber Re;
              input SI.ReynoldsNumber Re1;
              input SI.ReynoldsNumber Re2;
              input Real Delta;
              output Real lambda2;
              // point lg(lambda2(Re1)) with derivative at lg(Re1)
            protected
              Real x1 = Math.log10(Re1);
              Real y1 = Math.log10(64 * Re1);
              Real yd1 = 1;
              // Point lg(lambda2(Re2)) with derivative at lg(Re2)
              Real aux1 = 0.5 / Math.log(10) * 5.74 * 0.9;
              Real aux2 = Delta / 3.7 + 5.74 / Re2 ^ 0.9;
              Real aux3 = Math.log10(aux2);
              Real L2 = 0.25 * (Re2 / aux3) ^ 2;
              Real aux4 = 2.51 / sqrt(L2) + 0.27 * Delta;
              Real aux5 = -2 * sqrt(L2) * Math.log10(aux4);
              Real x2 = Math.log10(Re2);
              Real y2 = Math.log10(L2);
              Real yd2 = 2 + 4 * aux1 / (aux2 * aux3 * Re2 ^ 0.9);
              // Constants: Cubic polynomial between lg(Re1) and lg(Re2)
              Real diff_x = x2 - x1;
              Real m = (y2 - y1) / diff_x;
              Real c2 = (3 * m - 2 * yd1 - yd2) / diff_x;
              Real c3 = (yd1 + yd2 - 2 * m) / (diff_x * diff_x);
              Real dx = Math.log10(Re / Re1);
            algorithm
              lambda2 := 64 * Re1 * (Re / Re1) ^ (1 + dx * (c2 + dx * c3));
              annotation(
                Inline = false,
                smoothOrder = 5);
            end CubicInterpolation_MFLOW;

            function LambertW "Closed approximation of Lambert's w function for solving f(x) = x exp(x) for x"
              extends Modelica.Icons.Function;
              input Real y "f(x)";
              output Real x "W(y)";
            protected
              Real xl;
            algorithm
              if y <= 500.0 then
                xl := Modelica.Math.log(y + 1.0);
                x := 0.665 * (1 + 0.0195 * xl) * xl + 0.04;
              else
                xl := 0;
                x := Modelica.Math.log(y - 4.0) - (1.0 - 1.0 / Modelica.Math.log(y)) * Modelica.Math.log(Modelica.Math.log(y));
              end if;
              assert(y > (-1 / Modelica.Math.exp(1)), "Lambert-w-function is only valid for inputs y > -1/Modelica.Math.exp(1)!");
              annotation(
                Inline = false,
                smoothOrder = 5,
                Documentation(info = "<html>

<p>
This function calculates an approximation of the <b> inverse </b> for
</p>
<pre>
    f(x) = y = x * exp( x )
</pre>

<p>
within &infin; > y > -1/e.  The relative deviation of this approximation for Lambert's w function <b>x = W(y)</b> is displayed in the following graph.
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Fluid/Dissipation/utilities/LambertW_deviation.png\" alt=\"LambertW_deviation\"/>
</p>

<p>
For y > 10 and higher values the relative deviation is smaller 2%.
</p>

</html>"));
            end LambertW;

            function LambertWIter "Iterative form of Lambert's w function for solving f(x) = x exp(x) for x"
              extends Modelica.Icons.Function;
              input Real y "f(x)";
              output Real x "W(y)";
              output Integer iter;
            protected
              Real w;
              Real prec = 1e-12;
              Real c1;
              Real c2;
              Real dw;
              Real w1;
              /*Real wTimesExpW;
                Real wPlusOneTimesExpW;*/
              Real dev;
              Integer i;
            algorithm
              w := if y > 0.1 then Modelica.Fluid.Dissipation.Utilities.Functions.General.LambertW(y) else sqrt(5.43656 * max(y, -1 / Modelica.Math.exp(1)) + 2) - 1;
              dev := 1;
              i := 0;
              while prec < dev and i < 100 loop
                c1 := Modelica.Math.exp(w);
                c2 := w * c1 - y;
                w1 := if w <> 1 then w + 1 else w;
                dw := c2 / (c1 * w1 - (w + 2) * c2 / (2 * w1));
                w := w - dw;
                dev := abs((y - w * c1) / (w + 1) * c1);
                i := i + 1;
              end while;
/*wTimesExpW := w*Modelica.Math.exp(w);
                wPlusOneTimesExpW := (w+1)*Modelica.Math.exp(w);
                w := w-(wTimesExpW-y)/(wPlusOneTimesExpW-(w+2)*(wTimesExpW-y)/(2*w+2));
                dev := abs((y-wTimesExpW)/wPlusOneTimesExpW);
                i := i+1;*/
//dev := abs(dw)/(2+abs(w));
              x := w;
              iter := i;
              annotation(
                Inline = false,
                smoothOrder = 5,
                Documentation(info = "<html>

<p>
This function calculates an approximation of the <b> inverse </b> for
</p>
<pre>
    f(x) = y = x * exp( x )
</pre>

<p>
within &infin; > y > -1/e. Please note, that for negative inputs <b>two</b> solutions exists. The function currently delivers the result x = -1 ... 0 for that particular range.
</p>

</html>"));
            end LambertWIter;

            function PrandtlNumber "calculation of Prandtl number"
              extends Modelica.Icons.Function;
              import MIN = Modelica.Constants.eps;
              //fluid properties
              input SI.SpecificHeatCapacityAtConstantPressure cp "specific heat capacity of fluid at constant pressure";
              input SI.DynamicViscosity eta "dynamic viscosity of fluid";
              input SI.ThermalConductivity lambda "thermal conductivity of fluid";
              output SI.PrandtlNumber Pr "Prandtl number";
            algorithm
              Pr := eta * cp / max(MIN, lambda);
              annotation(
                Inline = true,
                smoothOrder = 1);
            end PrandtlNumber;

            function ReynoldsNumber "calculation of Reynolds number"
              extends Modelica.Icons.Function;
              import MIN = Modelica.Constants.eps;
              //geometry
              input SI.Area A_cross "Cross sectional area";
              input SI.Length perimeter "Wetted perimeter";
              //fluid properties
              input SI.Density rho "Density of fluid";
              input SI.DynamicViscosity eta "Dynamic viscosity of fluid";
              input SI.MassFlowRate m_flow "Mass flow rate";
              output SI.ReynoldsNumber Re "Reynolds number";
              output SI.Velocity velocity "Mean velocity";
            protected
              SI.Diameter d_hyd = 4 * A_cross / max(MIN, perimeter) "Hydraulic diameter";
            algorithm
              Re := 4 * abs(m_flow) / max(MIN, perimeter * eta);
              velocity := m_flow / max(MIN, rho * A_cross);
              annotation(
                Inline = true,
                smoothOrder = 1);
            end ReynoldsNumber;

            function SmoothPower "Limiting the derivative of function y = if x>=0 then x^pow else -(-x)^pow"
              extends Modelica.Icons.Function;
              input Real x "input variable";
              input Real deltax "range for interpolation";
              input Real pow "exponent for x";
              output Real y "output variable";
            protected
              Real adeltax = abs(deltax);
              Real C3 = (pow - 1) / 2 * adeltax ^ (pow - 3);
              Real C1 = (3 - pow) / 2 * adeltax ^ (pow - 1);
            algorithm
              y := if x >= adeltax then x ^ pow else if x <= (-adeltax) then -(-x) ^ pow else (C1 + C3 * x * x) * x;
              annotation(
                derivative(zeroDerivative = deltax, zeroDerivative = pow) = SmoothPower_der,
                Inline = false,
                smoothOrder = 1,
                Documentation(info = "<html>
<p>
The function is used to limit the derivative of the following function at x=0:
</p>
<pre>
   y = <b>if</b> x &ge; 0 <b>then</b> x<sup><b>pow</b></sup> <b>else</b> -(-x)<sup><b>pow</b></sup>;  // pow &gt; 0
</pre>

<p>
by approximating the function in the range -<b>deltax</b>&lt; x &lt; <b>deltax</b>
with a third order polynomial that has the same derivative at <b>abs</b>(x)=deltax, as the
function above.
</p>

<h4>Example </h4>
<p>
In the picture below the input x is increased from -1 to 1. The range of interpolation is defined by the same range. Displayed is the output of the function SmoothPower compared to
</p>
<pre>
y=x*|x|
</pre>
<p>
For |x| &gt; 1 both functions return identical results.
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Fluid/Dissipation/utilities/SmoothPower.png\" alt=\"SmoothPower\"/>
</p>

<h4>References</h4>
<dl>
<dt>ThermoFluid Library</dt>
    <dd><b><a href=\"http://sourceforge.net/projects/thermofluid/\">http://sourceforge.net/projects/thermofluid/</a></b></dd>
</dl>
</html>", revisions = "<html>
2014-04-29 Stefan Wischhusen: Introduced deltax and pow as zero derivatives.
</html>"));
            end SmoothPower;

            function SmoothPower_der "The derivative of function SmoothPower"
              extends Modelica.Icons.Function;
              input Real x "input variable";
              input Real deltax "range of interpolation";
              input Real pow "exponent for x";
              input Real dx "derivative of x";
              output Real dy "derivative of SmoothPower";
            protected
              Real C3;
              Real C1;
              Real adeltax;
            algorithm
              adeltax := abs(deltax);
              if x >= adeltax then
                dy := dx * pow * x ^ (pow - 1);
              elseif x <= (-adeltax) then
                dy := dx * pow * (-x) ^ (pow - 1);
              else
                C3 := (pow - 1) / 2 * adeltax ^ (pow - 3);
                C1 := (3 - pow) / 2 * adeltax ^ (pow - 1);
                dy := (C1 + 3 * C3 * x * x) * dx;
              end if;
              annotation(
                Documentation(revisions = "<html>
 2014-04-29 Stefan Wischhusen: Corrected branch for x<=-adeltax, removed dpow and ddeltax.
 2015-10-13 Stefan Wischhusen: Removed noEvent() from if clause.
</html>"));
            end SmoothPower_der;

            function Stepsmoother "Continuous interpolation for x"
              extends Modelica.Icons.Function;
              input Real func "input value for that result = 100%";
              input Real nofunc "input value for that result = 0%";
              input Real x "input variable for continuous interpolation";
              output Real result "output value";
            protected
              Real m = Modelica.Constants.pi / (func - nofunc);
              Real b = (-Modelica.Constants.pi / 2) - m * nofunc;
            algorithm
              result := if x >= func and func > nofunc or x <= func and nofunc > func then 1 else if x <= nofunc and func > nofunc or x >= nofunc and nofunc > func then 0 else (1 + Modelica.Math.tanh(Modelica.Math.tan(m * x + b))) / 2;
              annotation(
                Inline = false,
                derivative = Stepsmoother_der,
                Documentation(info = "<html>
<p>
The function is used for continuous fading of variable inputs within a defined range. It allows a differentiable and smooth transition between function outputs, e.g., laminar and turbulent pressure drop or correlations for certain ranges.
</p>
<h4>Function</h4>
<p>
The tanh-function is used, since it provides an existing derivative and the derivative is zero at the borders [<b>nofunc</b>, <b>func</b>] of the interpolation domain (smooth derivative for transitions).<br>
<br>
In order to work correctly, the internal interpolation range in terms of the external arbitrary input <b> x </b> needs to be scaled such that:
</p>
<pre>
f(func)   = 0.5 &pi;
f(nofunc) = -0.5 &pi;
</pre>

<h4>Example </h4>
<p>
In the picture below the input x is increased from 0 to 1. The range of interpolation is defined by:
</p>
<ul>
<li> func = 0.75</li>
<li> nofunc = 0.25</li>
</ul>

<p>
<img src=\"modelica://Modelica/Resources/Images/Fluid/Dissipation/utilities/Stepsmoother.png\" alt=\"Stepsmoother\"/>
</p>

<h4>References</h4>
<dl>
<dt>Wischhusen, St.</dt>
    <dd><b>Simulation von K&auml;ltemaschinen-Prozessen mit MODELICA / DYMOLA</b>.
    Diploma thesis, Hamburg University of Technology, Institute of Thermofluiddynamics, 2000.</dd>
</dl></html>"),
                smoothOrder = 5);
            end Stepsmoother;

            function Stepsmoother_der "Derivative of function Stepsmoother"
              extends Modelica.Icons.Function;
              input Real func "input for that result = 100%";
              input Real nofunc "input for that result = 0%";
              input Real x "input for interpolation";
              input Real dfunc "derivative of func";
              input Real dnofunc "derivative of nofunc";
              input Real dx "derivative of x";
              output Real dresult;
            protected
              Real m = Modelica.Constants.pi / (func - nofunc);
              Real b = (-Modelica.Constants.pi / 2) - m * nofunc;
            algorithm
              dresult := if x >= func and func > nofunc or x <= func and nofunc > func or x <= nofunc and func > nofunc or x >= nofunc and nofunc > func then 0 else (1 - Modelica.Math.tanh(Modelica.Math.tan(m * x + b)) ^ 2) * (1 + Modelica.Math.tan(m * x + b) ^ 2) * ((-m ^ 2 / Modelica.Constants.pi * (dfunc - dnofunc) * x) + m * dx + m ^ 2 / Modelica.Constants.pi * (dfunc - dnofunc) * nofunc - m * dnofunc) / 2;
            end Stepsmoother_der;
          end General;
        end Functions;

        package Icons "Icons for Fluid.Dissipation and Fluid.Fittings libraries"
          extends Modelica.Icons.IconsPackage;

          package HeatTransfer "Icons for heat transfer calculation"
            extends Modelica.Icons.Package;

            partial model Gap1_d "Geometry figure for gap"
              annotation(
                Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, -40}, {60, -60}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.Forward, fillColor = {255, 255, 170}, lineThickness = 1), Polygon(points = {{60, -40}, {60, -60}, {100, -20}, {100, 0}, {60, -40}}, lineColor = {0, 0, 0}, lineThickness = 1, fillColor = {255, 255, 170}, fillPattern = FillPattern.Forward), Rectangle(extent = {{-100, 40}, {60, 20}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.Forward, fillColor = {255, 255, 170}, lineThickness = 1), Polygon(points = {{60, 40}, {60, 20}, {100, 60}, {100, 80}, {60, 40}}, lineColor = {0, 0, 0}, lineThickness = 1, fillColor = {255, 255, 170}, fillPattern = FillPattern.Forward), Polygon(points = {{100, 60}, {100, 0}, {60, 0}, {60, 20}, {100, 60}}, lineThickness = 1, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, pattern = LinePattern.None, lineColor = {0, 0, 0}), Polygon(points = {{-100, -40}, {-100, 20}, {60, 20}, {60, 0}, {-60, 0}, {-100, -40}}, lineThickness = 1, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, pattern = LinePattern.None, lineColor = {0, 0, 0}), Polygon(points = {{-100, 40}, {-60, 80}, {100, 80}, {60, 40}, {-100, 40}}, lineColor = {0, 0, 0}, lineThickness = 1, fillColor = {255, 255, 170}, fillPattern = FillPattern.Forward), Line(points = {{-100, 20}, {60, 20}, {100, 60}}, thickness = 1), Line(points = {{-100, -40}, {60, -40}, {100, 0}}, thickness = 1), Line(points = {{20, 80}, {-20, 40}}, arrow = {Arrow.Filled, Arrow.Filled}, thickness = 0.5), Rectangle(extent = {{-4, 66}, {4, 56}}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, lineThickness = 1, pattern = LinePattern.None), Text(extent = {{-10, 66}, {10, 54}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, textString = "h"), Line(points = {{60, -66}, {-100, -66}}, arrow = {Arrow.Filled, Arrow.Filled}, thickness = 0.5), Rectangle(extent = {{-22, -62}, {-14, -72}}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, lineThickness = 1, pattern = LinePattern.None), Text(extent = {{-28, -62}, {-8, -74}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, textString = "L"), Polygon(points = {{-100, -40}, {-60, 0}, {100, 0}, {60, -40}, {-100, -40}}, lineColor = {0, 0, 0}, lineThickness = 1, fillColor = {255, 255, 170}, fillPattern = FillPattern.Forward), Line(points = {{-80, 20}, {-80, -40}}, arrow = {Arrow.Filled, Arrow.Filled}, thickness = 0.5), Rectangle(extent = {{-84, -4}, {-76, -14}}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, pattern = LinePattern.None), Text(extent = {{-90, -2}, {-70, -14}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, textString = "s"), Line(points = {{26, -10}, {-24, -10}}, arrow = {Arrow.Filled, Arrow.None}, thickness = 1), Rectangle(extent = {{-4, -4}, {4, -14}}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, lineThickness = 1, pattern = LinePattern.None), Text(extent = {{-10, -2}, {10, -14}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, textString = "v"), Ellipse(extent = {{78, 52}, {82, 48}}, pattern = LinePattern.None, lineThickness = 0.5, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 0}), Ellipse(extent = {{78, -28}, {82, -32}}, pattern = LinePattern.None, lineThickness = 0.5, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 0}), Line(points = {{80, -28}, {80, 48}}, arrow = {Arrow.Filled, Arrow.Filled}, thickness = 0.5), Rectangle(extent = {{76, 16}, {84, 6}}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, lineThickness = 1, pattern = LinePattern.None), Text(extent = {{70, 16}, {90, 4}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, textString = "T_wall"), Text(extent = {{-30, 36}, {-10, 24}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, textString = "wall 1"), Text(extent = {{-30, -44}, {-10, -56}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, textString = "wall 2")}));
            end Gap1_d;

            partial model HelicalPipe1_d "Geometry figure for helical pipe"
              annotation(
                Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-60, 82}, {-60, -84}}, thickness = 0.5, pattern = LinePattern.Dash), Line(points = {{-62, -26}, {60, -36}}, thickness = 1), Line(points = {{-62, 24}, {60, 14}}, thickness = 1), Line(points = {{-60, -16}, {62, -26}}, thickness = 1), Line(points = {{-60, 34}, {62, 24}}, thickness = 1), Ellipse(extent = {{38, 64}, {78, 24}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 170}, fillPattern = FillPattern.Forward), Ellipse(extent = {{38, 14}, {78, -26}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 170}, fillPattern = FillPattern.Forward), Ellipse(extent = {{-78, 24}, {-38, -16}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 170}, fillPattern = FillPattern.Forward), Ellipse(extent = {{-78, -26}, {-38, -66}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 170}, fillPattern = FillPattern.Forward), Polygon(points = {{-60, -26}, {60, -36}, {60, -76}, {-60, -66}, {-60, -26}}, lineThickness = 0.5, fillPattern = FillPattern.Forward, fillColor = {255, 255, 170}, pattern = LinePattern.None), Line(points = {{-60, -46}, {60, -56}}, thickness = 0.5, pattern = LinePattern.Dash), Polygon(points = {{-60, 24}, {60, 14}, {60, -26}, {-60, -16}, {-60, 24}}, lineThickness = 0.5, fillPattern = FillPattern.Forward, fillColor = {255, 255, 170}, pattern = LinePattern.None), Line(points = {{-60, 4}, {60, -6}}, thickness = 0.5, pattern = LinePattern.Dash), Ellipse(extent = {{40, -36}, {80, -76}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 170}, fillPattern = FillPattern.Forward), Polygon(points = {{-62, 74}, {58, 64}, {58, 24}, {-62, 34}, {-62, 74}}, lineThickness = 0.5, fillPattern = FillPattern.Forward, fillColor = {255, 255, 170}, pattern = LinePattern.None), Line(points = {{-60, 54}, {60, 44}}, thickness = 0.5, pattern = LinePattern.Dash), Ellipse(extent = {{-80, 74}, {-40, 34}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 170}, fillPattern = FillPattern.Forward), Polygon(points = {{-60, 24}, {60, 64}, {60, 24}, {-60, -16}, {-60, 24}}, lineThickness = 0.5, fillPattern = FillPattern.Forward, fillColor = {255, 255, 170}, pattern = LinePattern.None), Line(points = {{8, 28}, {-8, 22}}, arrow = {Arrow.None, Arrow.Filled}, thickness = 1), Line(points = {{12, -24}, {-4, -30}}, arrow = {Arrow.None, Arrow.Filled}, thickness = 1), Polygon(points = {{-60, -26}, {60, 14}, {60, -26}, {-60, -66}, {-60, -26}}, lineThickness = 0.5, fillPattern = FillPattern.Forward, fillColor = {255, 255, 170}, pattern = LinePattern.None), Line(points = {{60, 24}, {-60, -16}}, thickness = 0.5), Line(points = {{60, 14}, {-60, -26}}, thickness = 0.5), Line(points = {{60, -26}, {-60, -66}}, thickness = 0.5), Line(points = {{-60, 74}, {60, 64}}, thickness = 0.5), Line(points = {{-60, 24}, {60, 64}}, thickness = 0.5), Line(points = {{-60, -66}, {60, -76}}, thickness = 0.5), Line(points = {{10, -22}, {-6, -28}}, arrow = {Arrow.None, Arrow.Filled}, thickness = 1), Ellipse(extent = {{46, -42}, {74, -70}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-74, 68}, {-46, 40}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{-74, 78}, {-46, 78}}, arrow = {Arrow.Filled, Arrow.Filled}), Line(points = {{2, 3}, {-14, 3}}, arrow = {Arrow.None, Arrow.Filled}, thickness = 1, origin = {-38, 55}, rotation = 180), Line(points = {{0, 82}, {0, -86}}, thickness = 0.5, pattern = LinePattern.DashDot), Line(points = {{60, -32}, {60, -82}}, thickness = 0.5, pattern = LinePattern.DashDot), Line(points = {{-60, -82}, {60, -82}}, arrow = {Arrow.Filled, Arrow.Filled}), Rectangle(extent = {{-18, -76}, {18, -88}}, fillPattern = FillPattern.Solid, fillColor = {255, 255, 255}, lineThickness = 0.5, pattern = LinePattern.None), Text(extent = {{-14, -76}, {12, -86}}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, pattern = LinePattern.None, textString = "d_mean"), Line(points = {{46, -56}, {88, -56}}, thickness = 0.5, pattern = LinePattern.DashDot), Line(points = {{60, 18}, {60, -30}}, thickness = 0.5, pattern = LinePattern.DashDot), Line(points = {{46, -6}, {88, -6}}, thickness = 0.5, pattern = LinePattern.DashDot), Line(points = {{84, -6}, {84, -56}}, arrow = {Arrow.Filled, Arrow.Filled}), Rectangle(extent = {{74, -24}, {98, -36}}, fillPattern = FillPattern.Solid, fillColor = {255, 255, 255}, lineThickness = 0.5, pattern = LinePattern.None), Line(points = {{-74, 82}, {-74, 52}}, thickness = 0.5, pattern = LinePattern.DashDot), Line(points = {{-46, 82}, {-46, 52}}, thickness = 0.5, pattern = LinePattern.DashDot), Text(extent = {{72, -26}, {98, -36}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, textString = "h"), Rectangle(extent = {{-68, 84}, {-50, 76}}, fillPattern = FillPattern.Solid, fillColor = {255, 255, 255}, lineThickness = 0.5, pattern = LinePattern.None), Text(extent = {{-72, 86}, {-50, 78}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, textString = "d_hyd"), Rectangle(extent = {{-36, 56}, {-30, 48}}, fillPattern = FillPattern.Solid, fillColor = {255, 255, 255}, lineThickness = 0.5, pattern = LinePattern.None), Line(points = {{-40, 54}, {-40, 50}}, thickness = 1), Text(extent = {{-44, 56}, {-22, 48}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, textString = "L")}));
            end HelicalPipe1_d;

            partial model Plate1_d "Geometry figure 1 for plate"
              annotation(
                Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, 10}, {100, -10}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.Forward, fillColor = {255, 255, 170}), Line(points = {{-100, -20}, {100, -20}}, arrow = {Arrow.Filled, Arrow.Filled}), Text(extent = {{-14, -20}, {12, -30}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, textString = "L"), Line(points = {{-20, 16}, {20, 16}}, arrow = {Arrow.None, Arrow.Filled}), Text(extent = {{-14, 26}, {12, 16}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, textString = "velocity")}));
            end Plate1_d;

            partial model Plate2_d "Geometry figure 2 for plate"
              annotation(
                Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, -20}, {60, -40}}, fillPattern = FillPattern.Forward, fillColor = {255, 255, 170}, lineThickness = 0.5, lineColor = {0, 0, 0}), Polygon(points = {{-100, -20}, {-60, 20}, {100, 20}, {60, -20}, {-100, -20}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 170}, fillPattern = FillPattern.Forward, lineThickness = 0.5), Polygon(points = {{60, -20}, {60, -40}, {100, 0}, {100, 20}, {60, -20}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 170}, fillPattern = FillPattern.Forward), Line(points = {{-20, 0}, {20, 0}}, thickness = 1, arrow = {Arrow.None, Arrow.Filled}), Text(extent = {{-14, 10}, {12, 0}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, textString = "v"), Line(points = {{-100, -48}, {60, -48}}, arrow = {Arrow.Filled, Arrow.Filled}), Rectangle(extent = {{-26, -44}, {-18, -54}}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, lineThickness = 1, pattern = LinePattern.None), Text(extent = {{-34, -44}, {-8, -54}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, textString = "L")}));
            end Plate2_d;

            partial model Channel_i "Icon for heat transfer in a channel"
              annotation(
                Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Bitmap(extent = {{-100, -100}, {100, 100}}, fileName = "modelica://Modelica/Resources/Images/Fluid/Dissipation/heatTransfer/channel/icon_channel.png")}));
            end Channel_i;

            partial model General_i "Icon for general heat transfer component"
              annotation(
                Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Bitmap(extent = {{-100, -100}, {100, 100}}, fileName = "modelica://Modelica/Resources/Images/Fluid/Dissipation/heatTransfer/general/icon_general.png")}));
            end General_i;

            partial model HeatExchanger_i "Icon for heat transfer of a heat exchanger"
              annotation(
                Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Bitmap(extent = {{-100, -100}, {100, 100}}, fileName = "modelica://Modelica/Resources/Images/Fluid/Dissipation/heatTransfer/heatExchanger/icon_heatExchanger.png")}));
            end HeatExchanger_i;

            partial model HelicalPipe_i "Icon for heat transfer in a helical pipe"
              annotation(
                Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Bitmap(extent = {{-100, -100}, {100, 100}}, fileName = "modelica://Modelica/Resources/Images/Fluid/Dissipation/heatTransfer/helicalPipe/icon_helicalPipe.png")}));
            end HelicalPipe_i;

            partial model Plate_i "Icon for heat transfer of a plate"
              annotation(
                Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Bitmap(extent = {{-100, -100}, {100, 100}}, fileName = "modelica://Modelica/Resources/Images/Fluid/Dissipation/heatTransfer/plate/icon_plate.png")}));
            end Plate_i;

            partial model StraightPipe_i "Icon for heat transfer of a straight pipe"
              annotation(
                Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Bitmap(extent = {{-100, -100}, {100, 100}}, fileName = "modelica://Modelica/Resources/Images/Fluid/Dissipation/heatTransfer/straightPipe/icon_straightPipe.png")}));
            end StraightPipe_i;
          end HeatTransfer;

          package PressureLoss "Icons for pressure loss calculation"
            extends Modelica.Icons.Package;

            partial model BendEdged_d "Geometry figure of edged bend"
              annotation(
                Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, 10}, {0, -20}}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, pattern = LinePattern.None), Polygon(points = {{0, 10}, {100, -48}, {100, -80}, {0, -20}, {0, 10}}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, pattern = LinePattern.None), Ellipse(extent = {{-6, 16}, {8, 2}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Polygon(points = {{0, 40}, {100, -20}, {100, -48}, {0, 10}, {0, 40}}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, pattern = LinePattern.None), Rectangle(extent = {{-100, 40}, {0, 10}}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, pattern = LinePattern.None), Polygon(points = {{-100, -40}, {-100, -20}, {0, -20}, {0, -40}, {-100, -40}}, fillColor = {255, 255, 170}, fillPattern = FillPattern.Forward, lineColor = {0, 0, 0}), Polygon(points = {{0, -40}, {0, -20}, {100, -80}, {100, -100}, {0, -40}}, fillColor = {255, 255, 170}, fillPattern = FillPattern.Forward, lineColor = {0, 0, 0}), Polygon(points = {{0, 40}, {0, 60}, {100, 0}, {100, -20}, {0, 40}}, fillColor = {255, 255, 170}, fillPattern = FillPattern.Forward, lineColor = {0, 0, 0}), Polygon(points = {{-100, 40}, {-100, 60}, {0, 60}, {0, 40}, {-100, 40}}, fillColor = {255, 255, 170}, fillPattern = FillPattern.Forward, lineColor = {0, 0, 0}), Line(points = {{0, 10}, {100, -48}}, arrow = {Arrow.None, Arrow.Filled}, thickness = 0.5), Line(points = {{-100, 10}, {0, 10}}, thickness = 0.5, arrow = {Arrow.Filled, Arrow.None}), Line(points = {{-78, 40}, {-78, -20}}, thickness = 0.5, arrow = {Arrow.Filled, Arrow.Filled}), Rectangle(extent = {{-90, 32}, {-62, 18}}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, pattern = LinePattern.None), Text(extent = {{-90, 30}, {-64, 20}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, textString = "d_hyd"), Text(extent = {{-10, 20}, {16, 10}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, textString = "delta"), Rectangle(extent = {{-54, 18}, {-44, 2}}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, pattern = LinePattern.None), Text(extent = {{-58, 14}, {-40, 4}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, textString = "L"), Ellipse(extent = {{-2, 6}, {0, 4}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Line(points = {{-2, 4}, {2, 10}}, thickness = 0.5)}));
            end BendEdged_d;

            partial model Channel_d "Geometry figure for channel"
              annotation(
                Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-92, 80}, {-60, 80}}, thickness = 0.5, pattern = LinePattern.Dot), Ellipse(extent = {{20, 80}, {-20, 40}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 170}, fillPattern = FillPattern.Forward), Ellipse(extent = {{80, 74}, {40, 46}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 170}, fillPattern = FillPattern.Forward), Ellipse(extent = {{14, 74}, {-14, 46}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{0, 74}, {0, 46}}, thickness = 0.5, arrow = {Arrow.Filled, Arrow.Filled}), Rectangle(extent = {{-10, 66}, {10, 56}}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, pattern = LinePattern.None), Text(extent = {{-9, 66}, {9, 56}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 170}, fillPattern = FillPattern.Solid, textString = "d_cir"), Ellipse(extent = {{-40, 80}, {-80, 40}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, lineThickness = 0.5), Ellipse(extent = {{-46, 74}, {-74, 46}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 170}, fillPattern = FillPattern.Forward), Line(points = {{0, 14}, {0, -14}}, thickness = 0.5, arrow = {Arrow.Filled, Arrow.Filled}, origin = {-60, 36}, rotation = 90), Line(points = {{-92, 40}, {-60, 40}}, thickness = 0.5, pattern = LinePattern.Dot), Line(points = {{-90, 80}, {-90, 40}}, thickness = 0.5, arrow = {Arrow.Filled, Arrow.Filled}), Rectangle(extent = {{-98, 64}, {-82, 54}}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, pattern = LinePattern.None), Text(extent = {{-99, 65}, {-81, 55}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 170}, fillPattern = FillPattern.Solid, textString = "D_ann"), Line(points = {{-16, 0}, {16, 0}}, thickness = 0.5, pattern = LinePattern.Dot, origin = {-74, 46}, rotation = 270), Line(points = {{-16, 0}, {16, 0}}, thickness = 0.5, pattern = LinePattern.Dot, origin = {-46, 46}, rotation = 90), Rectangle(extent = {{-68, 38}, {-52, 28}}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, pattern = LinePattern.None), Text(extent = {{-69, 39}, {-51, 29}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 170}, fillPattern = FillPattern.Solid, textString = "d_ann"), Ellipse(extent = {{78, 72}, {42, 48}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{-22, 0}, {22, 0}}, thickness = 0.5, pattern = LinePattern.Dot, origin = {60, 60}, rotation = 90), Line(points = {{0, 4}, {0, -14}}, thickness = 0.5, arrow = {Arrow.Filled, Arrow.Filled}, origin = {46, 42}, rotation = 90), Rectangle(extent = {{46, 46}, {55, 39}}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, pattern = LinePattern.None), Text(extent = {{46, 46}, {55, 39}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 170}, fillPattern = FillPattern.Solid, textString = "a_ell"), Line(points = {{0, 0}, {22, 0}}, thickness = 0.5, pattern = LinePattern.Dot, origin = {42, 38}, rotation = 90), Line(points = {{60, 72}, {92, 72}}, thickness = 0.5, pattern = LinePattern.Dot), Line(points = {{38, 60}, {92, 60}}, thickness = 0.5, pattern = LinePattern.Dot), Line(points = {{0, 4}, {0, -8}}, thickness = 0.5, arrow = {Arrow.Filled, Arrow.Filled}, origin = {88, 68}, rotation = 360), Rectangle(extent = {{82, 68}, {94, 65}}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, pattern = LinePattern.None), Text(extent = {{83, 69}, {95, 63}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 170}, fillPattern = FillPattern.Solid, textString = "b_ell"), Rectangle(extent = {{-60, 24}, {-20, -6}}, fillColor = {255, 255, 170}, fillPattern = FillPattern.Forward, lineColor = {0, 0, 0}, lineThickness = 0.5), Rectangle(extent = {{-56, 20}, {-24, -2}}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 0}, lineThickness = 0.5), Line(points = {{-16, 0}, {16, 0}}, thickness = 0.5, pattern = LinePattern.Dot, origin = {-56, -2}, rotation = 270), Line(points = {{-16, 0}, {16, 0}}, thickness = 0.5, pattern = LinePattern.Dot, origin = {-24, -2}, rotation = 270), Line(points = {{0, 14}, {0, -18}}, thickness = 0.5, arrow = {Arrow.Filled, Arrow.Filled}, origin = {-42, -14}, rotation = 90), Rectangle(extent = {{-50, -10}, {-30, -20}}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, pattern = LinePattern.None), Text(extent = {{-48, -9}, {-31, -18}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 170}, fillPattern = FillPattern.Solid, textString = "a_rec"), Line(points = {{-16, 0}, {16, 0}}, thickness = 0.5, pattern = LinePattern.Dot, origin = {-31, 20}, rotation = 360), Line(points = {{-16, 0}, {16, 0}}, thickness = 0.5, pattern = LinePattern.Dot, origin = {-31, -2}, rotation = 360), Line(points = {{0, 12}, {0, -10}}, thickness = 0.5, arrow = {Arrow.Filled, Arrow.Filled}, origin = {-16, 10}, rotation = 180), Rectangle(extent = {{-19, 15}, {0, 3}}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, pattern = LinePattern.None), Text(extent = {{-18, 13}, {-1, 4}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 170}, fillPattern = FillPattern.Solid, textString = "b_rec"), Polygon(points = {{20, -10}, {40, 30}, {60, -10}, {20, -10}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 170}, fillPattern = FillPattern.Forward), Polygon(points = {{26, -6}, {40, 22}, {54, -6}, {26, -6}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{-16, 0}, {-4, 0}}, thickness = 0.5, pattern = LinePattern.Dot, origin = {26, -22}, rotation = 270), Line(points = {{0, 20}, {0, -8}}, thickness = 0.5, arrow = {Arrow.Filled, Arrow.Filled}, origin = {46, -16}, rotation = 90), Line(points = {{-16, 0}, {-4, 0}}, thickness = 0.5, pattern = LinePattern.Dot, origin = {54, -22}, rotation = 270), Line(points = {{-16, 0}, {12, 0}}, thickness = 0.5, pattern = LinePattern.Dot, origin = {56, 22}, rotation = 360), Line(points = {{-16, 0}, {12, 0}}, thickness = 0.5, pattern = LinePattern.Dot, origin = {56, -6}, rotation = 360), Line(points = {{0, 20}, {0, -8}}, thickness = 0.5, arrow = {Arrow.Filled, Arrow.Filled}, origin = {64, 14}, rotation = 180), Rectangle(extent = {{55, 15}, {74, 3}}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, pattern = LinePattern.None), Text(extent = {{56, 13}, {73, 4}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 170}, fillPattern = FillPattern.Solid, textString = "h_tri"), Rectangle(extent = {{31, -11}, {50, -23}}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, pattern = LinePattern.None), Text(extent = {{32, -11}, {49, -20}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 170}, fillPattern = FillPattern.Solid, textString = "a_tri"), Line(points = {{36, 14}, {38, 12}, {42, 12}, {44, 14}}, thickness = 0.5), Line(points = {{-12, -2}, {0, -8}}, thickness = 0.5, arrow = {Arrow.Filled, Arrow.None}, origin = {40, 24}, rotation = 360), Text(extent = {{12, 27}, {29, 18}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 170}, fillPattern = FillPattern.Solid, textString = "beta")}));
            end Channel_d;

            partial model OrificeSuddenChangeSection_d "Geometry figure for orifice with sudden change of cross sectional area"
              annotation(
                Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, 60}, {100, -60}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 170}, fillPattern = FillPattern.Backward), Rectangle(extent = {{-100, 20}, {0, -20}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Rectangle(extent = {{0, 40}, {100, -42}}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 0}), Text(extent = {{-80, 88}, {86, 76}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, textString = "sudden expansion"), Text(extent = {{-82, -76}, {86, -88}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, textString = "sudden contraction"), Line(points = {{-20, 0}, {20, 0}}, thickness = 0.5, arrow = {Arrow.None, Arrow.Filled}, origin = {0, -72}, rotation = 180), Line(points = {{-20, 0}, {20, 0}}, thickness = 0.5, arrow = {Arrow.Filled, Arrow.Filled}, origin = {-48, 0}, rotation = 90), Line(points = {{-62, 0}, {20, 0}}, thickness = 0.5, arrow = {Arrow.Filled, Arrow.Filled}, origin = {52, 20}, rotation = 90), Rectangle(extent = {{42, 6}, {62, -8}}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, pattern = LinePattern.None), Text(extent = {{38, 4}, {64, -6}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, textString = "A_2"), Rectangle(extent = {{-58, 6}, {-38, -8}}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, pattern = LinePattern.None), Text(extent = {{-62, 4}, {-36, -6}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, textString = "A_1"), Rectangle(extent = {{0, 20}, {0, -20}}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, pattern = LinePattern.None), Text(extent = {{-44, 10}, {40, 2}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, textString = "design flow direction"), Line(points = {{-24, -4}, {16, -4}}, thickness = 0.5, arrow = {Arrow.None, Arrow.Filled}), Line(points = {{-20, 70}, {20, 70}}, thickness = 0.5, arrow = {Arrow.None, Arrow.Filled})}));
            end OrificeSuddenChangeSection_d;

            partial model OrificeThickEdged_d "Geometry figure for orifice with thick edged vena contraction"
              annotation(
                Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, 60}, {100, -60}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 170}, fillPattern = FillPattern.Backward), Rectangle(extent = {{-40, 20}, {40, -20}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Rectangle(extent = {{40, 40}, {100, -42}}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 0}), Line(points = {{-20, 0}, {20, 0}}, thickness = 0.5, arrow = {Arrow.Filled, Arrow.Filled}, origin = {-48, 0}, rotation = 90), Line(points = {{-62, 0}, {20, 0}}, thickness = 0.5, arrow = {Arrow.Filled, Arrow.Filled}, origin = {70, 20}, rotation = 90), Rectangle(extent = {{60, 6}, {80, -8}}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, pattern = LinePattern.None), Text(extent = {{56, 4}, {82, -6}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, textString = "A_1"), Rectangle(extent = {{0, 20}, {0, -20}}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, pattern = LinePattern.None), Line(points = {{-40, -32}, {40, -32}}, thickness = 0.5, arrow = {Arrow.Filled, Arrow.Filled}), Rectangle(extent = {{-100, 40}, {-40, -42}}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 0}), Line(points = {{-62, 0}, {20, 0}}, thickness = 0.5, arrow = {Arrow.Filled, Arrow.Filled}, origin = {-70, 20}, rotation = 90), Rectangle(extent = {{-80, 6}, {-60, -8}}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, pattern = LinePattern.None), Text(extent = {{-84, 4}, {-58, -6}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, textString = "A_1"), Line(points = {{-42, 0}, {-2, 0}}, thickness = 0.5, arrow = {Arrow.Filled, Arrow.Filled}, origin = {0, 22}, rotation = 90), Rectangle(extent = {{-2, 6}, {18, -8}}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, pattern = LinePattern.None), Text(extent = {{-14, 4}, {12, -6}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, textString = "A_0"), Rectangle(extent = {{-44, 20}, {-40, -20}}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, pattern = LinePattern.None), Rectangle(extent = {{40, 20}, {44, -20}}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, pattern = LinePattern.None), Rectangle(extent = {{-4, -28}, {4, -38}}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, pattern = LinePattern.None), Text(extent = {{-12, -28}, {14, -38}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, textString = "L")}));
            end OrificeThickEdged_d;

            partial model StraightPipe_d "Geometry figure for straight pipe"
              annotation(
                Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{98, 46}, {62, -54}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillPattern = FillPattern.Forward, fillColor = {255, 255, 170}), Polygon(points = {{-80, -54}, {-80, 46}, {80, 46}, {80, -54}, {-80, -54}}, fillColor = {255, 255, 170}, fillPattern = FillPattern.Forward, lineThickness = 0.5, pattern = LinePattern.None), Line(points = {{-80, 52}, {80, 52}}, thickness = 0.5, arrow = {Arrow.Filled, Arrow.Filled}), Rectangle(extent = {{-4, 58}, {6, 48}}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, pattern = LinePattern.None), Text(extent = {{-8, 58}, {10, 48}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, textString = "L"), Ellipse(extent = {{90, 26}, {70, -34}}, lineColor = {0, 0, 0}, pattern = LinePattern.Dash, lineThickness = 0.5), Line(points = {{0, 26}, {0, -34}}, thickness = 0.5, arrow = {Arrow.Filled, Arrow.Filled}), Rectangle(extent = {{-6, -74}, {10, -86}}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, pattern = LinePattern.None), Ellipse(extent = {{-62, 46}, {-98, -54}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 170}, fillPattern = FillPattern.Forward), Ellipse(extent = {{-70, 26}, {-90, -34}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, lineThickness = 0.5), Line(points = {{-80, 46}, {80, 46}}, thickness = 0.5), Line(points = {{-80, -54}, {80, -54}}, thickness = 0.5), Line(points = {{-80, 26}, {80, 26}}, pattern = LinePattern.Dash, thickness = 0.5), Line(points = {{-80, -34}, {80, -34}}, thickness = 0.5, pattern = LinePattern.Dash), Line(points = {{-62, -4}, {98, -4}}, thickness = 0.5), Rectangle(extent = {{-12, 2}, {10, -8}}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, pattern = LinePattern.None), Text(extent = {{-14, 2}, {12, -8}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 170}, fillPattern = FillPattern.Solid, textString = "d_hyd")}));
            end StraightPipe_d;

            partial model Valve_d "Icon for valve"
              annotation(
                Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-6, -74}, {10, -86}}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, pattern = LinePattern.None), Line(points = {{-60, 0}, {-100, 0}}, color = {0, 127, 255}), Polygon(points = {{-60, 50}, {-60, -50}, {60, -50}, {60, 50}, {-60, 50}}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, pattern = LinePattern.None), Line(points = {{60, 0}, {100, 0}}, color = {0, 127, 255}), Line(points = {{-60, 50}, {-60, -50}, {60, 50}, {60, -50}, {-60, 50}}, thickness = 0.5)}));
            end Valve_d;

            model FlowModel "Icon for flow model in Modelica.Fluid applications"
              annotation(
                Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{-60, 50}, {-60, -50}, {60, -50}, {60, 50}, {-60, 50}}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, pattern = LinePattern.None), Line(points = {{-60, 0}, {-100, 0}}, color = {0, 127, 255}), Line(points = {{-60, 50}, {-60, -50}, {60, 50}, {60, -50}, {-60, 50}}, thickness = 0.5), Line(points = {{60, 0}, {100, 0}}, color = {0, 127, 255})}));
            end FlowModel;

            partial model Bend_i "Icon for bend"
              annotation(
                Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Bitmap(extent = {{-100, -100}, {100, 100}}, fileName = "modelica://Modelica/Resources/Images/Fluid/Dissipation/pressureLoss/bend/icon_bend.png")}));
            end Bend_i;

            partial model Channel_i "Icon for channel"
              annotation(
                Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Bitmap(extent = {{-100, -100}, {100, 100}}, fileName = "modelica://Modelica/Resources/Images/Fluid/Dissipation/pressureLoss/channel/icon_channel.png")}));
            end Channel_i;

            partial model General_i "Icon for general pressure drop"
              annotation(
                Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Bitmap(extent = {{-100, -100}, {100, 100}}, fileName = "modelica://Modelica/Resources/Images/Fluid/Dissipation/pressureLoss/general/icon_general.png")}));
            end General_i;

            partial model HeatExchanger_i
              annotation(
                Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Bitmap(extent = {{-100, -100}, {100, 100}}, fileName = "modelica://Modelica/Resources/Images/Fluid/Dissipation/pressureLoss/heatExchanger/icon_heatExchanger.png")}));
            end HeatExchanger_i;

            partial model Orifice_i "Icon for orifice"
              annotation(
                Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Bitmap(extent = {{-100, -100}, {100, 100}}, fileName = "modelica://Modelica/Resources/Images/Fluid/Dissipation/pressureLoss/orifice/icon_orifice.png")}));
            end Orifice_i;

            partial model StraightPipe_i "Icon for straight pipe"
              annotation(
                Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Bitmap(extent = {{-100, -100}, {100, 100}}, fileName = "modelica://Modelica/Resources/Images/Fluid/Dissipation/pressureLoss/straightPipe/icon_straightPipe.png")}));
            end StraightPipe_i;

            partial model Valve_i "Icon for valve"
              annotation(
                Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Bitmap(extent = {{-100, -100}, {100, 100}}, fileName = "modelica://Modelica/Resources/Images/Fluid/Dissipation/pressureLoss/valve/icon_valve.png")}));
            end Valve_i;
          end PressureLoss;
        end Icons;

        package Records "Package for base records"
          extends Modelica.Icons.Package;

          package General
            extends Modelica.Icons.Package;

            record PressureLoss "Base record for fluid properties for pressure loss"
              extends Modelica.Icons.Record;
              SI.DynamicViscosity eta "Dynamic viscosity of fluid" annotation(
                Dialog(group = "Fluid properties"));
              SI.Density rho "Density of fluid" annotation(
                Dialog(group = "Fluid properties"));
            end PressureLoss;

            record FluidProperties "Base record for fluid properties"
              extends Modelica.Icons.Record;
              SI.SpecificHeatCapacityAtConstantPressure cp "Specific heat capacity of fluid at constant pressure" annotation(
                Dialog(group = "Fluid properties"));
              SI.DynamicViscosity eta "Dynamic viscosity of fluid" annotation(
                Dialog(group = "Fluid properties"));
              SI.ThermalConductivity lambda "Thermal conductivity of fluid" annotation(
                Dialog(group = "Fluid properties"));
              SI.Density rho "Density of fluid" annotation(
                Dialog(group = "Fluid properties"));
            end FluidProperties;

            record IdealGas_con "Base record for generic pressure loss function | ideal gas | mean density"
              extends Modelica.Icons.Record;
              Real exp = 2 "Exponent of pressure loss law" annotation(
                Dialog(group = "Generic variables"));
              SI.SpecificHeatCapacity R_s "Specific gas constant of ideal gas" annotation(
                Dialog(group = "Fluid properties"));
              Real Km = 6824.86 "Coefficient for pressure loss law [(Pa)^2/{(kg/s)^exp*K}]" annotation(
                Dialog(group = "Generic variables"));
            end IdealGas_con;

            record IdealGas_var "Base record for generic pressure loss function | ideal gas | mean density"
              extends Modelica.Icons.Record;
              SI.Density rho_m "Mean density of ideal gas" annotation(
                Dialog(group = "Fluid properties", enable = useMeanDensity));
              SI.Temp_K T_m "Mean temperature of ideal gas" annotation(
                Dialog(group = "Fluid properties", enable = not useMeanDensity));
              SI.Pressure p_m "Mean pressure of ideal gas" annotation(
                Dialog(group = "Fluid properties", enable = not useMeanDensity));
            end IdealGas_var;

            record NominalDensityViscosity "Base record for generic pressure loss function"
              extends Modelica.Icons.Record;
              SI.Pressure dp_nom = 2 "Nominal pressure loss (at nominal values of mass flow rate and density)" annotation(
                Dialog(group = "Generic variables"));
              Real exp = 2 "Exponent of pressure loss law" annotation(
                Dialog(group = "Generic variables"));
              SI.MassFlowRate m_flow_nom = 1 "Nominal mass flow rate (at nominal values of pressure loss and density)" annotation(
                Dialog(group = "Generic variables"));
              SI.Density rho_nom "Nominal density (at nominal values of mass flow rate and pressure loss)" annotation(
                Dialog(group = "Generic variables"));
              Real exp_eta = 1 "Exponent for dynamic viscosity dependence" annotation(
                Dialog(group = "Generic variables"));
              SI.DynamicViscosity eta_nom "Dynamic viscosity at nominal pressure loss" annotation(
                Dialog(group = "Generic variables"));
            end NominalDensityViscosity;

            record NominalPressureLossLawDensity_con "Base record for generic pressure loss function"
              extends Modelica.Icons.Record;
              //NominalMassFlowRate
              Modelica.Fluid.Dissipation.Utilities.Types.MassOrVolumeFlowRate target = Dissipation.Utilities.Types.MassOrVolumeFlowRate.MassFlowRate "MassFlowRate == use nominal mass flow rate | VolumeFlowRate == use nominal volume flow rate" annotation(
                Dialog(group = "Generic variables"));
              SI.Area A_cross = A_cross_nom "Cross sectional area" annotation(
                Dialog(group = "Generic variables"));
              SI.Area A_cross_nom = Modelica.Constants.pi * 0.1 ^ 2 / 4 "Nominal cross sectional area" annotation(
                Dialog(group = "Generic variables"));
              SI.Pressure dp_nom = 2 "Nominal pressure loss (at nominal values of mass flow rate and density)" annotation(
                Dialog(group = "Generic variables"));
              SI.MassFlowRate m_flow_nom = 1 "Nominal mass flow rate (at nominal values of pressure loss and density)" annotation(
                Dialog(group = "Generic variables", enable = target == Modelica.Fluid.Dissipation.Utilities.Types.MassOrVolumeFlowRate.MassFlowRate));
              Real exp = 2 "Exponent of pressure loss law" annotation(
                Dialog(group = "Generic variables"));
              SI.VolumeFlowRate V_flow_nom = m_flow_nom / rho_nom "Nominal volume flow rate (at nominal values of pressure loss and density)" annotation(
                Dialog(group = "Generic variables", enable = target == Modelica.Fluid.Dissipation.Utilities.Types.MassOrVolumeFlowRate.VolumeFlowRate));
              SI.Density rho_nom "Nominal density (at nominal values of mass flow rate and pressure loss)" annotation(
                Dialog(group = "Generic variables"));
              Types.PressureLossCoefficient zeta_TOT_nom = 0.02 * 1 / 0.1 "Nominal pressure loss coefficient (for nominal values)" annotation(
                Dialog(group = "Generic variables"));
            end NominalPressureLossLawDensity_con;

            record NominalPressureLossLawDensity_var "Base record for generic pressure loss function"
              extends Modelica.Icons.Record;
              TYP.PressureLossCoefficient zeta_TOT = 0.2 "Pressure loss coefficient" annotation(
                Dialog(group = "Generic variables"));
            end NominalPressureLossLawDensity_var;

            record QuadraticVFLOW "Base record for generic pressure loss function | quadratic function (dp=a*Vdot^2 + b*Vdot)"
              extends Modelica.Icons.Record;
              Real a(unit = "(Pa.s2)/m6") = 15 "Coefficient for quadratic term" annotation(
                Dialog(group = "Generic variables"));
              Real b(unit = "(Pa.s)/m3") = 0 "Coefficient for linear term" annotation(
                Dialog(group = "Generic variables"));
            end QuadraticVFLOW;

            record TwoPhaseFlow_con "Base record for two phase Flow"
              extends Modelica.Icons.Record;
              SI.Area A_cross = PI * 0.1 ^ 2 / 4 "Cross sectional area" annotation(
                Dialog(group = "Geometry"));
              SI.Length perimeter = PI * 0.1 "Wetted perimeter" annotation(
                Dialog(group = "Geometry"));
              SI.Length length = 1 "Length in fluid flow direction" annotation(
                Dialog(group = "Geometry"));
            end TwoPhaseFlow_con;

            record TwoPhaseFlow_var "Base record for two phase flow"
              extends Modelica.Icons.Record;
              SI.Density rho_g "Density of gas" annotation(
                Dialog(group = "Fluid properties"));
              SI.Density rho_l "Density of liquid" annotation(
                Dialog(group = "Fluid properties"));
              SI.DynamicViscosity eta_g "Dynamic viscosity of gas" annotation(
                Dialog(group = "Fluid properties"));
              SI.DynamicViscosity eta_l "Dynamic viscosity of liquid" annotation(
                Dialog(group = "Fluid properties"));
              SI.SurfaceTension sigma "Surface tension" annotation(
                Dialog(group = "Fluid properties"));
              //input variables
              Real x_flow = 0 "Mean mass flow rate quality over length" annotation(
                Dialog(group = "Input"));
            end TwoPhaseFlow_var;

            record IdealGas "Base record for generic pressure loss function | ideal gas | mean density"
              extends Modelica.Icons.Record;
              parameter Real exp(min = Modelica.Constants.eps) = 2 "Exponent of pressure loss law" annotation(
                Dialog(group = "Generic variables"));
              parameter SI.SpecificHeatCapacity R_s(min = 1) "Specific gas constant of ideal gas" annotation(
                Dialog(group = "Fluid properties"));
              Real Km(min = Modelica.Constants.eps) = R_s * 2e3 / (10 ^ exp / rho_m) "Coefficient for pressure loss law [(Pa)^2/{(kg/s)^exp*K}]" annotation(
                Dialog(group = "Generic variables"));
              SI.Density rho_m = p_m / (R_s * T_m) "Mean density of ideal gas" annotation(
                Dialog(group = "Fluid properties", enable = useMeanDensity));
              SI.Temp_K T_m "Mean temperature of ideal gas" annotation(
                Dialog(group = "Fluid properties", enable = not useMeanDensity));
              SI.Pressure p_m "Mean pressure of ideal gas" annotation(
                Dialog(group = "Fluid properties", enable = not useMeanDensity));
            end IdealGas;

            record NominalPressureLossLawDensity "Base record for generic pressure loss function"
              extends Modelica.Icons.Record;
              //NominalMassFlowRate
              Modelica.Fluid.Dissipation.Utilities.Types.MassOrVolumeFlowRate target = Dissipation.Utilities.Types.MassOrVolumeFlowRate.MassFlowRate "MassFlowRate == use nominal mass flow rate | VolumeFlowRate == use nominal volume flow rate" annotation(
                Dialog(group = "Generic variables"));
              parameter SI.Area A_cross = A_cross_nom "Cross sectional area" annotation(
                Dialog(group = "Generic variables"));
              parameter SI.Area A_cross_nom = Modelica.Constants.pi * 0.1 ^ 2 / 4 "Nominal cross sectional area" annotation(
                Dialog(group = "Generic variables"));
              parameter SI.Pressure dp_nom(min = Modelica.Constants.eps) = 2 "Nominal pressure loss (at nominal values of mass flow rate and density)" annotation(
                Dialog(group = "Generic variables"));
              parameter SI.MassFlowRate m_flow_nom(min = Modelica.Constants.eps) = 1 "Nominal mass flow rate (at nominal values of pressure loss and density)" annotation(
                Dialog(group = "Generic variables", enable = target == Modelica.Fluid.Dissipation.Utilities.Types.MassOrVolumeFlowRate.MassFlowRate));
              parameter Real exp(min = Modelica.Constants.eps) = 2 "Exponent of pressure loss law" annotation(
                Dialog(group = "Generic variables"));
              SI.VolumeFlowRate V_flow_nom(min = Modelica.Constants.eps) = m_flow_nom / rho_nom "Nominal volume flow rate (at nominal values of pressure loss and density)" annotation(
                Dialog(group = "Generic variables", enable = not target == Modelica.Fluid.Dissipation.Utilities.Types.MassOrVolumeFlowRate.MassFlowRate));
              SI.Density rho_nom(min = Modelica.Constants.eps) "Nominal density (at nominal values of mass flow rate and pressure loss)" annotation(
                Dialog(group = "Generic variables"));
              Types.PressureLossCoefficient zeta_TOT = zeta_TOT_nom "Pressure loss coefficient" annotation(
                Dialog(group = "Generic variables"));
              parameter Types.PressureLossCoefficient zeta_TOT_nom = 0.02 * 1 / 0.1 "Nominal pressure loss coefficient (for nominal values)" annotation(
                Dialog(group = "Generic variables"));
            end NominalPressureLossLawDensity;

            record TwoPhaseFlow "base record for two phase Flow"
              extends Modelica.Icons.Record;
              SI.Density rho_l "Density of liquid" annotation(
                Dialog(group = "Fluid properties"));
              SI.Density rho_g "Density of gas" annotation(
                Dialog(group = "Fluid properties", enable = if KC == 1 or KC == 2 then true else false));
              SI.DynamicViscosity eta_l "Dynamic viscosity of liquid" annotation(
                Dialog(group = "Fluid properties"));
              SI.DynamicViscosity eta_g "Dynamic viscosity of gas" annotation(
                Dialog(group = "Fluid properties", enable = if KC == 1 or KC == 2 then true else false));
              SI.MassFraction x = 0.5 "Vapour fraction" annotation(
                Dialog(group = "Fluid properties"));
              SI.SurfaceTension sigma "Surface Tension" annotation(
                Dialog(group = "Fluid properties", enable = if DP_fric == 1 then true else false));
              Real n = 0.25 "Exponent in Blasius equation (0.2-0.25)" annotation(
                Dialog(group = "others", enable = if DP_fric == 2 then true else false));
            end TwoPhaseFlow;
          end General;

          package HeatTransfer
            extends Modelica.Icons.Package;

            record EvenGap "Input for even gap"
              extends Modelica.Icons.Record;
              //choices
              Modelica.Fluid.Dissipation.Utilities.Types.kc_evenGap target = Dissipation.Utilities.Types.kc_evenGap.DevBoth "Target variable of calculation" annotation(
                Dialog(group = "Even gap"));
              SI.Length h = 0.1 "Height of cross sectional area" annotation(
                Dialog(group = "Even gap"));
              SI.Length s = 0.05 "Distance between parallel plates in cross sectional area" annotation(
                Dialog(group = "Even gap"));
              SI.Length L = 1 "Overflowed length of gap" annotation(
                Dialog(group = "Even gap"));
            end EvenGap;

            record General "Input for generic correlation"
              extends Modelica.Icons.Record;
              //choices
              Modelica.Fluid.Dissipation.Utilities.Types.kc_general target = Dissipation.Utilities.Types.kc_general.Finest "Target correlation" annotation(
                Dialog(group = "Generic variables"));
              //geometry
              SI.Area A_cross = Modelica.Constants.pi * 0.1 ^ 2 / 4 "Cross sectional area" annotation(
                Dialog(group = "Generic variables"));
              SI.Length perimeter = Modelica.Constants.pi * 0.1 "Wetted perimeter" annotation(
                Dialog(group = "Generic variables"));
            end General;

            record HelicalPipe "Input for helical pipe"
              extends Modelica.Icons.Record;
              Real n_nt = 1 "Total number of turns" annotation(
                Dialog(group = "HelicalPipe"));
              SI.Diameter d_hyd = 0.1 "Hydraulic diameter" annotation(
                Dialog(group = "HelicalPipe"));
              SI.Length h = 0.01 "Distance between turns" annotation(
                Dialog(group = "HelicalPipe"));
              SI.Length L = 1 "Total length of helical pipe" annotation(
                Dialog(group = "HelicalPipe"));
            end HelicalPipe;

            record Plate "Input for plate"
              extends Modelica.Icons.Record;
              SI.Length L = 1 "Length of plate" annotation(
                Dialog(group = "Plate"));
            end Plate;

            record StraightPipe "Input for straight pipe"
              extends Modelica.Icons.Record;
              SI.Diameter d_hyd = 0.1 "Hydraulic diameter" annotation(
                Dialog(group = "Straight pipe"));
              SI.Length L = 1 "Length" annotation(
                Dialog(group = "Straight pipe"));
            end StraightPipe;

            record TwoPhaseFlowHT_IN_con "Base record for two phase heat transfer coefficient"
              extends Modelica.Icons.Record;
              //choices
              Modelica.Fluid.Dissipation.Utilities.Types.TwoPhaseHeatTransferTarget target = Modelica.Fluid.Dissipation.Utilities.Types.TwoPhaseHeatTransferTarget.BoilHor "Choice of (horizontal/vertical) boiling or (horizontal) condensation in pipe" annotation(
                Dialog(group = "Choices"));
              SI.Area A_cross = Modelica.Constants.pi * 0.1 ^ 2 / 4 "Cross sectional area" annotation(
                Dialog(group = "Geometry"));
              SI.Length perimeter = Modelica.Constants.pi * 0.1 "Wetted perimeter" annotation(
                Dialog(group = "Geometry"));
              Modelica.Fluid.Dissipation.Utilities.Types.MolarMass_gpmol MM = 18.02 "Molar mass of fluid" annotation(
                Dialog(group = "Fluid properties", enable = if target == Modelica.Fluid.Dissipation.Utilities.Types.TwoPhaseHeatTransferTarget.BoilHor or target == Modelica.Fluid.Dissipation.Utilities.Types.TwoPhaseHeatTransferTarget.BoilVer then true else false));
              SI.Pressure p_crit "Critical pressure of fluid" annotation(
                Dialog(group = "Fluid properties"));
              annotation(
                Documentation(revisions = "<html>
<ul>
<li><i>13 May 2011</i>
    by Stefan Wischhusen:<br>
       Corrected the required unit of parameter MM.</li>
</ul>
</html>"));
            end TwoPhaseFlowHT_IN_con;

            record TwoPhaseFlowHT_IN_var "Base record for two phase heat transfer coefficient"
              extends Modelica.Icons.Record;
              //choices
              Modelica.Fluid.Dissipation.Utilities.Types.TwoPhaseHeatTransferTarget target = Modelica.Fluid.Dissipation.Utilities.Types.TwoPhaseHeatTransferTarget.BoilHor "Choice of (horizontal/vertical) boiling or (horizontal) condensation in pipe" annotation(
                Dialog(group = "Choices"));
              //fluid properties
              SI.SpecificHeatCapacityAtConstantPressure cp_l "Specific heat capacity of liquid" annotation(
                Dialog(group = "Fluid properties"));
              SI.ThermalConductivity lambda_l "Thermal conductivity of liquid" annotation(
                Dialog(group = "Fluid properties"));
              SI.Density rho_g "Density of gas" annotation(
                Dialog(group = "Fluid properties", enable = if target == Modelica.Fluid.Dissipation.Utilities.Types.TwoPhaseHeatTransferTarget.BoilHor or target == Modelica.Fluid.Dissipation.Utilities.Types.TwoPhaseHeatTransferTarget.BoilVer then true else false));
              SI.Density rho_l "Density of liquid" annotation(
                Dialog(group = "Fluid properties"));
              SI.DynamicViscosity eta_g "Dynamic viscosity of gas" annotation(
                Dialog(group = "Fluid properties", enable = if target == Modelica.Fluid.Dissipation.Utilities.Types.TwoPhaseHeatTransferTarget.BoilHor or target == Modelica.Fluid.Dissipation.Utilities.Types.TwoPhaseHeatTransferTarget.BoilVer then true else false));
              SI.DynamicViscosity eta_l "Dynamic viscosity of liquid" annotation(
                Dialog(group = "Fluid properties"));
              SI.Pressure pressure "Mean pressure of fluid" annotation(
                Dialog(group = "Fluid properties"));
              SI.SpecificEnthalpy dh_lg "Evaporation enthalpy of fluid" annotation(
                Dialog(group = "Fluid properties", enable = if target == Modelica.Fluid.Dissipation.Utilities.Types.TwoPhaseHeatTransferTarget.BoilHor or target == Modelica.Fluid.Dissipation.Utilities.Types.TwoPhaseHeatTransferTarget.BoilVer then true else false));
              //input variables
              SI.MassFlowRate m_flow "Mass flow rate" annotation(
                Dialog(group = "Input"));
              SI.HeatFlux qdot_A = 0 "Heat flux at boiling" annotation(
                Dialog(group = "Input", enable = if target == Modelica.Fluid.Dissipation.Utilities.Types.TwoPhaseHeatTransferTarget.BoilHor or target == Modelica.Fluid.Dissipation.Utilities.Types.TwoPhaseHeatTransferTarget.BoilVer then true else false));
              Real x_flow = 0 "Mass flow rate quality" annotation(
                Dialog(group = "Input"));
            end TwoPhaseFlowHT_IN_var;
          end HeatTransfer;

          package PressureLoss
            extends Modelica.Icons.Package;

            record Bend "Input for bend"
              extends EdgedBend;
              SI.Radius R_0 = 0.5 * d_hyd "Curvature radius" annotation(
                Dialog(group = "Bend"));
            end Bend;

            record Geometry "Input for several geometries of internal flow"
              extends Modelica.Icons.Record;
              Modelica.Fluid.Dissipation.Utilities.Types.GeometryOfInternalFlow geometry = Modelica.Fluid.Dissipation.Utilities.Types.GeometryOfInternalFlow.Circular "Choice of geometry for internal flow" annotation(
                Dialog(group = "Channel"));
              SI.Length K = 0 "Roughness (average height of surface asperities)" annotation(
                Dialog(group = "Channel"));
              SI.Length L = 1 "Length" annotation(
                Dialog(group = "Channel"));
              //geometry variables
              //annular(1)
              SI.Diameter d_ann = d_cir "Small diameter" annotation(
                Dialog(group = "Annular cross sectional area", enable = geometry == Modelica.Fluid.Dissipation.Utilities.Types.GeometryOfInternalFlow.Annular));
              SI.Diameter D_ann = 2 * d_ann "Large diameter" annotation(
                Dialog(group = "Annular cross sectional area", enable = geometry == Modelica.Fluid.Dissipation.Utilities.Types.GeometryOfInternalFlow.Annular));
              //circular(2)
              SI.Diameter d_cir = 0.1 "Internal diameter" annotation(
                Dialog(group = "Circular cross sectional area", enable = geometry == Modelica.Fluid.Dissipation.Utilities.Types.GeometryOfInternalFlow.Circular));
              //elliptical(3)
              SI.Length a_ell = 3 / 4 * d_cir "Half length of long base line" annotation(
                Dialog(group = "Elliptical cross sectional area", enable = geometry == Modelica.Fluid.Dissipation.Utilities.Types.GeometryOfInternalFlow.Elliptical));
              SI.Length b_ell = 0.5 * a_ell "Half length of short base line" annotation(
                Dialog(group = "Elliptical cross sectional area", enable = geometry == Modelica.Fluid.Dissipation.Utilities.Types.GeometryOfInternalFlow.Elliptical));
              //rectangular(4)
              SI.Length a_rec = d_cir "Horizontal length" annotation(
                Dialog(group = "Rectangular cross sectional area", enable = geometry == Modelica.Fluid.Dissipation.Utilities.Types.GeometryOfInternalFlow.Rectangular));
              SI.Length b_rec = a_rec "Vertical length" annotation(
                Dialog(group = "Rectangular cross sectional area", enable = geometry == Modelica.Fluid.Dissipation.Utilities.Types.GeometryOfInternalFlow.Rectangular));
              //triangular(5)
              SI.Length a_tri = d_cir * (1 + 2 ^ 0.5) "Length of base line" annotation(
                Dialog(group = "Rectangular cross sectional area", enable = geometry == Modelica.Fluid.Dissipation.Utilities.Types.GeometryOfInternalFlow.Rectangular));
              SI.Length h_tri = 0.5 * a_tri "Height to top angle perpendicular to base line" annotation(
                Dialog(group = "Triangle cross sectional area", enable = geometry == Modelica.Fluid.Dissipation.Utilities.Types.GeometryOfInternalFlow.Isosceles));
              SI.Angle beta = 90 * PI / 180 "Top angle" annotation(
                Dialog(group = "Triangle cross sectional area", enable = geometry == Modelica.Fluid.Dissipation.Utilities.Types.GeometryOfInternalFlow.Isosceles));
            end Geometry;

            record Orifice "Input for orifice"
              extends Modelica.Icons.Record;
              SI.Area A_0 = 0.1 * A_1 "Cross sectional area of vena contraction" annotation(
                Dialog(group = "Orifice"));
              SI.Length C_0 = 0.1 * C_1 "Perimeter of vena contraction" annotation(
                Dialog(group = "Orifice"));
              SI.Area A_1 = PI * 0.01 ^ 2 / 4 "Large cross sectional area of orifice" annotation(
                Dialog(group = "Orifice"));
              SI.Length C_1 = PI * 0.01 "Large perimeter of orifice" annotation(
                Dialog(group = "Orifice"));
              SI.Length L = 1e-3 "Length of vena contraction" annotation(
                Dialog(group = "Orifice"));
            end Orifice;

            record PressureLossInput "Input for pressure loss calculation"
              extends Modelica.Icons.Record;
              //target variables
              Modelica.Fluid.Dissipation.Utilities.Types.PressureLossTarget target = Dissipation.Utilities.Types.PressureLossTarget.PressureLoss "Target variable of calculation" annotation(
                Dialog(group = "Input"));
              SI.Pressure dp = 0 "Pressure loss" annotation(
                Dialog(group = "Input", enable = target == Modelica.Fluid.Dissipation.Utilities.Types.PressureLossTarget.pressureLoss));
              SI.MassFlowRate m_flow = 0 "Mass flow rate" annotation(
                Dialog(group = "Input", enable = target == Modelica.Fluid.Dissipation.Utilities.Types.PressureLossTarget.massFlowRate));
            end PressureLossInput;

            record StraightPipe "Input for straight pipe"
              extends Modelica.Icons.Record;
              SI.Diameter d_hyd = 0.1 "Hydraulic diameter" annotation(
                Dialog(group = "Straight pipe"));
              SI.Length L = 1 "Length" annotation(
                Dialog(group = "Straight pipe"));
            end StraightPipe;

            record Tjunction "Input for T-junction"
              extends Modelica.Icons.Record;
              //T-junction variables
              Boolean united_converging_crossection = true "true == A_cross_total = 2*A_cross_branch | false == A_cross_total > 2*A_cross_branch" annotation(
                Dialog(group = "T-junction"));
              Boolean velocity_reference_branches = true "true == pressure loss coefficients w.r.t. velocity in each passage | false == w.r.t. velocity in total passage" annotation(
                Dialog(group = "T-junction"));
              Integer alpha = 90 "Angle of branching" annotation(
                Dialog(group = "T-junction"));
              SI.Diameter d_hyd[3] = ones(3) * 0.1 "Hydraulic diameter of passages [side,straight,total]" annotation(
                Dialog(group = "T-junction"));
              //restrictions
              SI.MassFlowRate m_flow_min = 1e-3 "Restriction for smoothing at reverse fluid flow" annotation(
                Dialog(group = "Restrictions"));
              SI.Velocity v_max = 2e2 "Restriction for maximum fluid flow velocity" annotation(
                Dialog(group = "Restrictions"));
              Real zeta_TOT_max = 1e3 "Restriction for maximum value of pressure loss coefficient" annotation(
                Dialog(group = "Restrictions"));
            end Tjunction;

            record Diffuser "Input for diffuser"
              extends Modelica.Icons.Record;
              SI.Angle alpha = PI * 45 / 180 "Diffuser diverging angle" annotation(
                Dialog(group = "Diffuser"));
              SI.Area A_1 = PI * 0.01 ^ 2 / 4 "Small constant cross sectional area before diffuser section" annotation(
                Dialog(group = "Diffuser"));
              SI.Area A_2 = 2 * A_1 "Large constant cross sectional area after diffuser section" annotation(
                Dialog(group = "Diffuser"));
              SI.Length C_1 = PI * 0.01 "Small perimeter before diffuser section" annotation(
                Dialog(group = "Diffuser"));
              SI.Length C_2 = 2 * C_1 "Large perimeter after diffuser section" annotation(
                Dialog(group = "Diffuser"));
              SI.Length L_1 = 0.1 "Length of straight pipe before diffuser section" annotation(
                Dialog(group = "Diffuser"));
              SI.Length L_2 = L_1 "Length of straight pipe after diffuser section" annotation(
                Dialog(group = "Diffuser"));
              SI.Length L_d = L_1 "Length of diffuser section (parallel to bulk fluid flow)" annotation(
                Dialog(group = "Diffuser"));
              //numerical aspects
              SI.Pressure dp_small = 1 "Linearisation for a pressure loss smaller then dp_small" annotation(
                Dialog(group = "Numerical aspects"));
              Real zeta_TOT_min = 1e-3 "Minimal pressure loss coefficient for infinite Reynolds number" annotation(
                Dialog(group = "Numerical aspects"));
              Real zeta_TOT_max = 1e8 "Maximum pressure loss coefficient for Reynolds number approaching zero" annotation(
                Dialog(group = "Numerical aspects"));
            end Diffuser;

            record EdgedBend "Input for bend"
              extends Modelica.Icons.Record;
              SI.Diameter d_hyd(min = Modelica.Constants.eps) = 0.1 "Hydraulic diameter" annotation(
                Dialog(group = "Bend"));
              SI.Angle delta = 90 * PI / 180 "Angle of turning" annotation(
                Dialog(group = "Bend"));
              SI.Length K = 0 "Roughness (absolute average height of surface asperities)" annotation(
                Dialog(group = "Bend"));
            end EdgedBend;

            record SuddenChange "Input for sudden change of diameter"
              extends Modelica.Icons.Record;
              SI.Area A_1 = PI * 0.01 ^ 2 / 4 "Small cross sectional area of orifice" annotation(
                Dialog(group = "Orifice"));
              SI.Area A_2 = A_1 "Large cross sectional area of orifice" annotation(
                Dialog(group = "Orifice"));
              SI.Length C_1 = PI * 0.01 "Small perimeter of orifice" annotation(
                Dialog(group = "Orifice"));
              SI.Length C_2 = C_1 "Large perimeter of orifice" annotation(
                Dialog(group = "Orifice"));
            end SuddenChange;
          end PressureLoss;
        end Records;

        package Types "Package for types"
          extends Modelica.Icons.TypesPackage;
          type DarcyFrictionFactor = Modelica.Icons.TypeReal(final quantity = "Darcy friction factor | lambda_fri = zeta_fri / (length/diameter)", final unit = "1", min = 0, max = 1111);
          type FrictionalResistanceCoefficient = Modelica.Icons.TypeReal(final quantity = "Frictional resistance coefficient | zeta_fri", final unit = "1", min = 0, max = 1111);
          type LocalResistanceCoefficient = Modelica.Icons.TypeReal(final quantity = "Local resistance coefficient | zeta_loc", final unit = "1", min = 0, max = 1111);
          type PressureLossCoefficient = Modelica.Icons.TypeReal(final quantity = "Pressure loss coefficient | zeta_tot = zeta_loc + zeta_fri", final unit = "1", min = 0, max = 1111);
          type TwoPhaseFrictionalPressureLoss = enumeration(Friedel "Friedel correlation for frictional pressure loss", Chisholm "Chisholm correlation for frictional pressure loss");
          type Roughness = enumeration(Neglected "Neglect surface roughness", Considered "Consider surface roughness");
          type TwoPhaseHeatTransferTarget = enumeration(BoilHor "Boiling horizontal", BoilVer "Boiling vertical", CondHor "Condensation horizontal");
          type PressureLossTarget = enumeration(PressureLoss "Calculate pressure loss from mass flow rate", MassFlowRate "Calculate mass flow rate from pressure loss");
          type GeometryOfInternalFlow = enumeration(Annular "Annular geometry", Circular "Circular geometry", Elliptical "Elliptical geometry", Rectangular "Rectangular geometry", Isosceles "Isosceles triangular geometry");
          type kc_evenGap = enumeration(DevOne "Hydrodynamically DEVELOPED laminar flow regime AND heat transfer at ONE side", DevBoth "Hydrodynamically DEVELOPED laminar flow regime AND heat transfer at BOTH sides", UndevOne "Hydrodynamic and thermal START of laminar flow regime AND heat transfer at ONE side", UndevBoth "Hydrodynamic and thermal START of laminar flow regime AND heat transfer at BOTH side");
          type kc_general = enumeration(Rough "Roughest approximation w.r.t. Dittus/Boelter (1930)", Middle "Middle approximation w.r.t. Sieder/Tate (1936)", Finest "Finest approximation w.r.t. Gnielinski (1976)");
          type HeatTransferBoundary = enumeration(UWTuDFF "Uniform wall temperature in developed fluid flow (UWT+DFF)", UHFuDFF "Uniform heat flux in developed fluid flow (UHF+DFF)", UWTuUFF "Uniform wall temperature in undeveloped fluid flow (UWT+UFF)", UHFuUFF "Uniform heat flux in undeveloped fluid flow (UHF+UFF)");
          type MassOrVolumeFlowRate = enumeration(MassFlowRate "Mass flow rate", VolumeFlowRate "Volume flow rate");
          type VoidFractionApproach = enumeration(Homogeneous "Homogeneous approach", Momentum "Analytical momentum flux approach (heterogeneous)", Energy "Kinetic energy flow approach w.r.t. Zivi (heterogeneous)", Chisholm "Empirical momentum flux approach w.r.t. Chisholm (heterogeneous)");
          type OrificeGeometry = enumeration(SharpEdged "Sharp edged shape of orifice inlet", ThickEdged "Thick edged shape of orifice inlet", TiltedEdged "Tilted edged shape of orifice inlet", RoundedEdged "Rounded edged shape of orifice inlet");
          type ValveGeometry = enumeration(Ball "Ball valve", Diaphragm "Diaphragm valve", Butterfly "Butterfly valve", Gate "Gate valve", Sluice "Sluice valve");
          type ValveCoefficient = enumeration(AV "Av (metric) flow coefficient", KV "Kv (metric) flow coefficient", CV "Cv (US) flow coefficient", OP "Av defined by operating point");
          type FluidFlowRegime = enumeration(Laminar "Laminar fluid flow regime", Overall "Overall fluid flow regime", Turbulent "Turbulent fluid flow regime");
          type HTXGeometry_flatTubes = enumeration(LouverFin "Louver fin", RectangularFin "Rectangular offset strip fin");
          type HTXGeometry_roundTubes = enumeration(PlainFin "Plain fin", LouverFin "Louver fin", SlitFin "Slit fin", WavyFin "Wavy fin (Herringbone wavy fin)");
          type MolarMass_gpmol = Real(final quantity = "Molar mass", final unit = "g/mol");
        end Types;
      end Utilities;
      annotation(
        Documentation(info = "<html>
<p>
<img src=\"modelica://Modelica/Resources/Images/Fluid/Dissipation/FD2.png\" alt=\"FD2\"/>
</p>

<h4>  Library description </h4>

This library contains <b>convective heat transfer</b> and  <b>pressure loss</b> functions written in
Modelica&reg;. Generally the pressure loss calculations are based on incompressible fluids and total pressure difference. For devices with non changing cross sectional area, the calculated total pressure loss is equal to the static pressure difference. Geodetic pressure loss is not considered throughout the library. The functions supplied may be used separately.<br>
<br>
The library is a non-commercial product of XRG Simulation GmbH. It makes use of external,
non-commercial models supplied by Modelica Standard Library. In order to work correctly,
ensure that this library is always loaded with <b> Modelica Standard Library version
3.1</b> especially to be able to use Modelica.Fluid applications.<br>

<h4>  Acknowledgements </h4>
The following people contributed to the Fluid.Dissipation library (alphabetical list):
J&ouml;rg Eiden, Ole Engel, Nina Peci, Sven Rutkowski, Thorben Vahlenkamp, Stefan
Wischhusen.

<p>
The development of the Fluid.Dissipation library is founded within the ITEA research
project EuroSysLib-D by German Federal Ministry of Education and Research (promotional
reference 01IS07022B). The project is started in October 2007 and will end in March 2010.
</p>

<h4>  License condition </h4>
<p>
<b>Licensed by XRG-Simulation GmbH under the Modelica License 2</b><br>
Copyright &copy; 2007-2013, XRG Simulation GmbH.
</p>

<p>
<i>This Modelica package is <u>free</u> software and the use is completely at <u>your own risk</u>; it can be redistributed and/or modified under the terms of the Modelica License 2. For license conditions (including the disclaimer of warranty) see <a href=\"modelica://Modelica.UsersGuide.ModelicaLicense2\">Modelica.UsersGuide.ModelicaLicense2</a> or visit <a href=\"https://www.modelica.org/licenses/ModelicaLicense2\"> https://www.modelica.org/licenses/ModelicaLicense2</a>.</i>
</p>

<h4>  Contact </h4>

XRG Simulation GmbH<br>
Harburger Schlossstrasse 6-12<br>
21079 Hamburg<br>
Germany<br>
<br>
<a href=mailto:info@xrg-simulation.de>info@xrg-simulation.de</a> </html>"));
    end Dissipation;

    package Utilities "Utility models to construct fluid components (should not be used directly)"
      extends Modelica.Icons.UtilitiesPackage;

      function checkBoundary "Check whether boundary definition is correct"
        extends Modelica.Icons.Function;
        input String mediumName;
        input String substanceNames[:] "Names of substances";
        input Boolean singleState;
        input Boolean define_p;
        input Real X_boundary[:];
        input String modelName = "??? boundary ???";
      protected
        Integer nX = size(X_boundary, 1);
        String X_str;
      algorithm
        assert(not singleState or singleState and define_p, "
Wrong value of parameter define_p (= false) in model \"" + modelName + "\":
The selected medium \"" + mediumName + "\" has Medium.singleState=true.
Therefore, an boundary density cannot be defined and
define_p = true is required.
        ");
        for i in 1:nX loop
          assert(X_boundary[i] >= 0.0, "
Wrong boundary mass fractions in medium \"" + mediumName + "\" in model \"" + modelName + "\":
The boundary value X_boundary(" + String(i) + ") = " + String(X_boundary[i]) + "
is negative. It must be positive.
          ");
        end for;
        if nX > 0 and abs(sum(X_boundary) - 1.0) > 1.e-10 then
          X_str := "";
          for i in 1:nX loop
            X_str := X_str + "   X_boundary[" + String(i) + "] = " + String(X_boundary[i]) + " \"" + substanceNames[i] + "\"\n";
          end for;
          Modelica.Utilities.Streams.error("The boundary mass fractions in medium \"" + mediumName + "\" in model \"" + modelName + "\"\n" + "do not sum up to 1. Instead, sum(X_boundary) = " + String(sum(X_boundary)) + ":\n" + X_str);
        end if;
      end checkBoundary;

      function regRoot "Anti-symmetric square root approximation with finite derivative in the origin"
        extends Modelica.Icons.Function;
        input Real x;
        input Real delta = 0.01 "Range of significant deviation from sqrt(abs(x))*sgn(x)";
        output Real y;
      algorithm
        y := x / (x * x + delta * delta) ^ 0.25;
        annotation(
          derivative(zeroDerivative = delta) = regRoot_der,
          Documentation(info = "<html>
<p>
This function approximates sqrt(abs(x))*sgn(x), such that the derivative is finite and smooth in x=0.
</p>
<table border=1 cellspacing=0 cellpadding=2>
<tr><th>Function</th><th>Approximation</th><th>Range</th></tr>
<tr><td>y = regRoot(x)</td><td>y ~= sqrt(abs(x))*sgn(x)</td><td>abs(x) &gt;&gt;delta</td></tr>
<tr><td>y = regRoot(x)</td><td>y ~= x/sqrt(delta)</td><td>abs(x) &lt;&lt; delta</td></tr>
</table>
<p>
With the default value of delta=0.01, the difference between sqrt(x) and regRoot(x) is 16% around x=0.01, 0.25% around x=0.1 and 0.0025% around x=1.
</p>
</html>", revisions = "<html>
<ul>
<li><i>15 Mar 2005</i>
    by <a href=\"mailto:francesco.casella@polimi.it\">Francesco Casella</a>:<br>
       Created. </li>
</ul>
</html>"));
      end regRoot;

      function regRoot_der "Derivative of regRoot"
        extends Modelica.Icons.Function;
        input Real x;
        input Real delta = 0.01 "Range of significant deviation from sqrt(x)";
        input Real dx "Derivative of x";
        output Real dy;
      algorithm
        dy := dx * 0.5 * (x * x + 2 * delta * delta) / (x * x + delta * delta) ^ 1.25;
        annotation(
          Documentation(info = "<html>
</html>", revisions = "<html>
<ul>
<li><i>15 Mar 2005</i>
    by <a href=\"mailto:francesco.casella@polimi.it\">Francesco Casella</a>:<br>
       Created. </li>
</ul>
</html>"));
      end regRoot_der;

      function regSquare "Anti-symmetric square approximation with non-zero derivative in the origin"
        extends Modelica.Icons.Function;
        input Real x;
        input Real delta = 0.01 "Range of significant deviation from x^2*sgn(x)";
        output Real y;
      algorithm
        y := x * sqrt(x * x + delta * delta);
        annotation(
          Documentation(info = "<html>
<p>
This function approximates x^2*sgn(x), such that the derivative is non-zero in x=0.
</p>
<table border=1 cellspacing=0 cellpadding=2>
<tr><th>Function</th><th>Approximation</th><th>Range</th></tr>
<tr><td>y = regSquare(x)</td><td>y ~= x^2*sgn(x)</td><td>abs(x) &gt;&gt;delta</td></tr>
<tr><td>y = regSquare(x)</td><td>y ~= x*delta</td><td>abs(x) &lt;&lt; delta</td></tr>
</table>
<p>
With the default value of delta=0.01, the difference between x^2 and regSquare(x) is 41% around x=0.01, 0.4% around x=0.1 and 0.005% around x=1.
</p>
</html>", revisions = "<html>
<ul>
<li><i>15 Mar 2005</i>
    by <a href=\"mailto:francesco.casella@polimi.it\">Francesco Casella</a>:<br>
       Created. </li>
</ul>
</html>"));
      end regSquare;

      function regPow "Anti-symmetric power approximation with non-zero derivative in the origin"
        extends Modelica.Icons.Function;
        input Real x;
        input Real a;
        input Real delta = 0.01 "Range of significant deviation from x^a*sgn(x)";
        output Real y;
      algorithm
        y := x * (x * x + delta * delta) ^ ((a - 1) / 2);
        annotation(
          Documentation(info = "<html>
<p>
This function approximates abs(x)^a*sign(x), such that the derivative is positive, finite and smooth in x=0.
</p>
<table border=1 cellspacing=0 cellpadding=2>
<tr><th>Function</th><th>Approximation</th><th>Range</th></tr>
<tr><td>y = regPow(x)</td><td>y ~= abs(x)^a*sgn(x)</td><td>abs(x) &gt;&gt;delta</td></tr>
<tr><td>y = regPow(x)</td><td>y ~= x*delta^(a-1)</td><td>abs(x) &lt;&lt; delta</td></tr>
</table>
</html>", revisions = "<html>
<ul>
<li><i>15 Mar 2005</i>
    by <a href=\"mailto:francesco.casella@polimi.it\">Francesco Casella</a>:<br>
       Created. </li>
</ul>
</html>"));
      end regPow;

      function regRoot2 "Anti-symmetric approximation of square root with discontinuous factor so that the first derivative is finite and continuous"
        extends Modelica.Icons.Function;
        input Real x "abscissa value";
        input Real x_small(min = 0) = 0.01 "approximation of function for |x| <= x_small";
        input Real k1(min = 0) = 1 "y = if x>=0 then sqrt(k1*x) else -sqrt(k2*|x|)";
        input Real k2(min = 0) = 1 "y = if x>=0 then sqrt(k1*x) else -sqrt(k2*|x|)";
        input Boolean use_yd0 = false "= true, if yd0 shall be used";
        input Real yd0(min = 0) = 1 "Desired derivative at x=0: dy/dx = yd0";
        output Real y "ordinate value";
      protected
        encapsulated function regRoot2_utility "Interpolating with two 3-order polynomials with a prescribed derivative at x=0"
          import Modelica;
          extends Modelica.Icons.Function;
          import Modelica.Fluid.Utilities.evaluatePoly3_derivativeAtZero;
          input Real x;
          input Real x1 "approximation of function abs(x) < x1";
          input Real k1 "y = if x>=0 then sqrt(k1*x) else -sqrt(k2*|x|); k1 >= k2";
          input Real k2 "y = if x>=0 then sqrt(k1*x) else -sqrt(k2*|x|))";
          input Boolean use_yd0 "= true, if yd0 shall be used";
          input Real yd0(min = 0) "Desired derivative at x=0: dy/dx = yd0";
          output Real y;
        protected
          Real x2;
          Real xsqrt1;
          Real xsqrt2;
          Real y1;
          Real y2;
          Real y1d;
          Real y2d;
          Real w;
          Real y0d;
          Real w1;
          Real w2;
          Real sqrt_k1 = if k1 > 0 then sqrt(k1) else 0;
          Real sqrt_k2 = if k2 > 0 then sqrt(k2) else 0;
        algorithm
          if k2 > 0 then
            x2 := -x1 * (k2 / k1);
          elseif k1 > 0 then
            x2 := -x1;
          else
            y := 0;
            return;
          end if;
// Since k1 >= k2 required, k2 > 0 means that k1 > 0
          if x <= x2 then
            y := -sqrt_k2 * sqrt(abs(x));
          else
            y1 := sqrt_k1 * sqrt(x1);
            y2 := -sqrt_k2 * sqrt(abs(x2));
            y1d := sqrt_k1 / sqrt(x1) / 2;
            y2d := sqrt_k2 / sqrt(abs(x2)) / 2;
            if use_yd0 then
              y0d := yd0;
            else
              w := x2 / x1;
              y0d := ((3 * y2 - x2 * y2d) / w - (3 * y1 - x1 * y1d) * w) / (2 * x1 * (1 - w));
            end if;
            w1 := sqrt_k1 * sqrt(8.75 / x1);
            w2 := sqrt_k2 * sqrt(8.75 / abs(x2));
            y0d := smooth(2, min(y0d, 0.9 * min(w1, w2)));
            y := y1 * (if x >= 0 then evaluatePoly3_derivativeAtZero(x / x1, 1, 1, y1d * x1 / y1, y0d * x1 / y1) else evaluatePoly3_derivativeAtZero(x / x1, x2 / x1, y2 / y1, y2d * x1 / y1, y0d * x1 / y1));
          end if;
/* Determine derivative, such that first and second derivative
              of left and right polynomial are identical at x=0:
           _
           Basic equations:
              y_right = a1*(x/x1) + a2*(x/x1)^2 + a3*(x/x1)^3
              y_left  = b1*(x/x2) + b2*(x/x2)^2 + b3*(x/x2)^3
              yd_right*x1 = a1 + 2*a2*(x/x1) + 3*a3*(x/x1)^2
              yd_left *x2 = b1 + 2*b2*(x/x2) + 3*b3*(x/x2)^2
              ydd_right*x1^2 = 2*a2 + 6*a3*(x/x1)
              ydd_left *x2^2 = 2*b2 + 6*b3*(x/x2)
           _
           Conditions (6 equations for 6 unknowns):
                     y1 = a1 + a2 + a3
                     y2 = b1 + b2 + b3
                 y1d*x1 = a1 + 2*a2 + 3*a3
                 y2d*x2 = b1 + 2*b2 + 3*b3
                    y0d = a1/x1 = b1/x2
                   y0dd = 2*a2/x1^2 = 2*b2/x2^2
           _
           Derived equations:
              b1 = a1*x2/x1
              b2 = a2*(x2/x1)^2
              b3 = y2 - b1 - b2
                 = y2 - a1*(x2/x1) - a2*(x2/x1)^2
              a3 = y1 - a1 - a2
           _
           Remaining equations
              y1d*x1 = a1 + 2*a2 + 3*(y1 - a1 - a2)
                     = 3*y1 - 2*a1 - a2
              y2d*x2 = a1*(x2/x1) + 2*a2*(x2/x1)^2 +
                       3*(y2 - a1*(x2/x1) - a2*(x2/x1)^2)
                     = 3*y2 - 2*a1*(x2/x1) - a2*(x2/x1)^2
              y0d    = a1/x1
           _
           Solving these equations results in y0d below
           (note, the denominator "(1-w)" is always non-zero, because w is negative)
           */
/* Modify derivative y0d, such that the polynomial is
           monotonically increasing. A sufficient condition is
             0 <= y0d <= sqrt(8.75*k_i/|x_i|)
        */
/* Perform interpolation in scaled polynomial:
           y_new = y/y1
           x_new = x/x1
        */
          annotation(
            smoothOrder = 2);
        end regRoot2_utility;
      algorithm
        y := smooth(2, if x >= x_small then sqrt(k1 * x) else if x <= (-x_small) then -sqrt(k2 * abs(x)) else if k1 >= k2 then regRoot2_utility(x, x_small, k1, k2, use_yd0, yd0) else -regRoot2_utility(-x, x_small, k2, k1, use_yd0, yd0));
        annotation(
          smoothOrder = 2,
          Documentation(info = "<html>
<p>
Approximates the function
</p>
<pre>
   y = <b>if</b> x &ge; 0 <b>then</b> <b>sqrt</b>(k1*x) <b>else</b> -<b>sqrt</b>(k2*<b>abs</b>(x)), with k1, k2 &ge; 0
</pre>
<p>
in such a way that within the region -x_small &le; x &le; x_small,
the function is described by two polynomials of third order
(one in the region -x_small .. 0 and one within the region 0 .. x_small)
such that
</p>
<ul>
<li> The derivative at x=0 is finite. </li>
<li> The overall function is continuous with a
     continuous first derivative everywhere.</li>
<li> If parameter use_yd0 = <b>false</b>, the two polynomials
     are constructed such that the second derivatives at x=0
     are identical. If use_yd0 = <b>true</b>, the derivative
     at x=0 is explicitly provided via the additional argument
     yd0. If necessary, the derivative yd0 is automatically
     reduced in order that the polynomials are strict monotonically
     increasing <i>[Fritsch and Carlson, 1980]</i>.</li>
</ul>
<p>
Typical screenshots for two different configurations
are shown below. The first one with k1=k2=1:
</p>
<p>
<img src=\"modelica://Modelica/Resources/Images/Fluid/Components/regRoot2_a.png\"
     alt=\"regRoot2_a.png\">
</p>
<p>
and the second one with k1=1 and k2=3:
</p>
<p>
<img src=\"modelica://Modelica/Resources/Images/Fluid/Components/regRoot2_b.png\"
      alt=\"regRoot2_b.png\">
</p>

<p>
The (smooth) derivative of the function with
k1=1, k2=3 is shown in the next figure:
<p>
<img src=\"modelica://Modelica/Resources/Images/Fluid/Components/regRoot2_c.png\"
     alt=\"regRoot2_c.png\">
</p>

<p>
<b>Literature</b>
</p>

<dl>
<dt> Fritsch F.N. and Carlson R.E. (1980):</dt>
<dd> <b>Monotone piecewise cubic interpolation</b>.
     SIAM J. Numerc. Anal., Vol. 17, No. 2, April 1980, pp. 238-246</dd>
</dl>
</html>", revisions = "<html>
<ul>
<li><i>Sept., 2019</i>
    by <a href=\"mailto:Martin.Otter@DLR.de\">Martin Otter</a>:<br>
    Improved so that k1=0 and/or k2=0 is also possible.</li>
<li><i>Nov., 2005</i>
    by <a href=\"mailto:Martin.Otter@DLR.de\">Martin Otter</a>:<br>
    Designed and implemented.</li>
</ul>
</html>"));
      end regRoot2;

      function regSquare2 "Anti-symmetric approximation of square with discontinuous factor so that the first derivative is non-zero and is continuous"
        extends Modelica.Icons.Function;
        input Real x "abscissa value";
        input Real x_small(min = 0) = 0.01 "approximation of function for |x| <= x_small";
        input Real k1(min = 0) = 1 "y = (if x>=0 then k1 else k2)*x*|x|";
        input Real k2(min = 0) = 1 "y = (if x>=0 then k1 else k2)*x*|x|";
        input Boolean use_yd0 = false "= true, if yd0 shall be used";
        input Real yd0(min = 0) = 1 "Desired derivative at x=0: dy/dx = yd0";
        output Real y "ordinate value";
      protected
        encapsulated function regSquare2_utility "Interpolating with two 3-order polynomials with a prescribed derivative at x=0"
          import Modelica;
          extends Modelica.Icons.Function;
          import Modelica.Fluid.Utilities.evaluatePoly3_derivativeAtZero;
          input Real x;
          input Real x1 "approximation of function abs(x) < x1";
          input Real k1 "y = (if x>=0 then k1 else -k2)*x*|x|; k1 >= k2";
          input Real k2 "y = (if x>=0 then k1 else -k2)*x*|x|";
          input Boolean use_yd0 = false "= true, if yd0 shall be used";
          input Real yd0(min = 0) = 1 "Desired derivative at x=0: dy/dx = yd0";
          output Real y;
        protected
          Real x2;
          Real y1;
          Real y2;
          Real y1d;
          Real y2d;
          Real w;
          Real w1;
          Real w2;
          Real y0d;
          Real ww;
        algorithm
// x2 :=-x1*(k2/k1)^2;
          x2 := -x1;
          if x <= x2 then
            y := -k2 * x ^ 2;
          else
            y1 := k1 * x1 ^ 2;
            y2 := -k2 * x2 ^ 2;
            y1d := k1 * 2 * x1;
            y2d := -k2 * 2 * x2;
            if use_yd0 then
              y0d := yd0;
            else
              w := x2 / x1;
              y0d := ((3 * y2 - x2 * y2d) / w - (3 * y1 - x1 * y1d) * w) / (2 * x1 * (1 - w));
            end if;
            w1 := sqrt(5) * k1 * x1;
            w2 := sqrt(5) * k2 * abs(x2);
            ww := 0.9 * (if w1 < w2 then w1 else w2);
            if ww < y0d then
              y0d := ww;
            end if;
            y := if x >= 0 then evaluatePoly3_derivativeAtZero(x, x1, y1, y1d, y0d) else evaluatePoly3_derivativeAtZero(x, x2, y2, y2d, y0d);
          end if;
/* Determine derivative, such that first and second derivative
              of left and right polynomial are identical at x=0:
              see derivation in function regRoot2
           */
/* Modify derivative y0d, such that the polynomial is
           monotonically increasing. A sufficient condition is
             0 <= y0d <= sqrt(5)*k_i*|x_i|
        */
// y0d :=min(y0d, 0.9*min(w1, w2));
          annotation(
            smoothOrder = 2);
        end regSquare2_utility;
      algorithm
        y := smooth(2, if x >= x_small then k1 * x ^ 2 else if x <= (-x_small) then -k2 * x ^ 2 else if k1 >= k2 then regSquare2_utility(x, x_small, k1, k2, use_yd0, yd0) else -regSquare2_utility(-x, x_small, k2, k1, use_yd0, yd0));
        annotation(
          smoothOrder = 2,
          Documentation(info = "<html>
<p>
Approximates the function
</p>
<pre>
   y = <b>if</b> x &ge; 0 <b>then</b> k1*x*x <b>else</b> -k2*x*x, with k1, k2 > 0
</pre>
<p>
in such a way that within the region -x_small &le; x &le; x_small,
the function is described by two polynomials of third order
(one in the region -x_small .. 0 and one within the region 0 .. x_small)
such that
</p>

<ul>
<li> The derivative at x=0 is non-zero (in order that the
     inverse of the function does not have an infinite derivative). </li>
<li> The overall function is continuous with a
     continuous first derivative everywhere.</li>
<li> If parameter use_yd0 = <b>false</b>, the two polynomials
     are constructed such that the second derivatives at x=0
     are identical. If use_yd0 = <b>true</b>, the derivative
     at x=0 is explicitly provided via the additional argument
     yd0. If necessary, the derivative yd0 is automatically
     reduced in order that the polynomials are strict monotonically
     increasing <i>[Fritsch and Carlson, 1980]</i>.</li>
</ul>

<p>
A typical screenshot for k1=1, k2=3 is shown in the next figure:
</p>
<p>
<img src=\"modelica://Modelica/Resources/Images/Fluid/Components/regSquare2_b.png\"
     alt=\"regSquare2_b.png\">
</p>

<p>
The (smooth, non-zero) derivative of the function with
k1=1, k2=3 is shown in the next figure:
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Fluid/Components/regSquare2_c.png\"
     alt=\"regSquare2_b.png\">
</p>

<p>
<b>Literature</b>
</p>

<dl>
<dt> Fritsch F.N. and Carlson R.E. (1980):</dt>
<dd> <b>Monotone piecewise cubic interpolation</b>.
     SIAM J. Numerc. Anal., Vol. 17, No. 2, April 1980, pp. 238-246</dd>
</dl>
</html>", revisions = "<html>
<ul>
<li><i>Nov., 2005</i>
    by <a href=\"mailto:Martin.Otter@DLR.de\">Martin Otter</a>:<br>
    Designed and implemented.</li>
</ul>
</html>"));
      end regSquare2;

      function regStep "Approximation of a general step, such that the characteristic is continuous and differentiable"
        extends Modelica.Icons.Function;
        input Real x "Abscissa value";
        input Real y1 "Ordinate value for x > 0";
        input Real y2 "Ordinate value for x < 0";
        input Real x_small(min = 0) = 1e-5 "Approximation of step for -x_small <= x <= x_small; x_small >= 0 required";
        output Real y "Ordinate value to approximate y = if x > 0 then y1 else y2";
      algorithm
        y := smooth(1, if x > x_small then y1 else if x < (-x_small) then y2 else if x_small > 0 then x / x_small * ((x / x_small) ^ 2 - 3) * (y2 - y1) / 4 + (y1 + y2) / 2 else (y1 + y2) / 2);
        annotation(
          Documentation(revisions = "<html>
<ul>
<li><i>April 29, 2008</i>
    by <a href=\"mailto:Martin.Otter@DLR.de\">Martin Otter</a>:<br>
    Designed and implemented.</li>
<li><i>August 12, 2008</i>
    by <a href=\"mailto:Michael.Sielemann@dlr.de\">Michael Sielemann</a>:<br>
    Minor modification to cover the limit case <code>x_small -> 0</code> without division by zero.</li>
</ul>
</html>", info = "<html>
<p>
This function is used to approximate the equation
</p>
<pre>
    y = <b>if</b> x &gt; 0 <b>then</b> y1 <b>else</b> y2;
</pre>

<p>
by a smooth characteristic, so that the expression is continuous and differentiable:
</p>

<pre>
   y = <b>smooth</b>(1, <b>if</b> x &gt;  x_small <b>then</b> y1 <b>else</b>
                 <b>if</b> x &lt; -x_small <b>then</b> y2 <b>else</b> f(y1, y2));
</pre>

<p>
In the region -x_small &lt; x &lt; x_small a 2nd order polynomial is used
for a smooth transition from y1 to y2.
</p>
</html>"));
      end regStep;

      function evaluatePoly3_derivativeAtZero "Evaluate polynomial of order 3 that passes the origin with a predefined derivative"
        extends Modelica.Icons.Function;
        input Real x "Value for which polynomial shall be evaluated";
        input Real x1 "Abscissa value";
        input Real y1 "y1=f(x1)";
        input Real y1d "First derivative at y1";
        input Real y0d "First derivative at f(x=0)";
        output Real y;
      protected
        Real a1;
        Real a2;
        Real a3;
        Real xx;
      algorithm
        a1 := x1 * y0d;
        a2 := 3 * y1 - x1 * y1d - 2 * a1;
        a3 := y1 - a2 - a1;
        xx := x / x1;
        y := xx * (a1 + xx * (a2 + xx * a3));
        annotation(
          smoothOrder = 3,
          Documentation(info = "<html>

</html>"));
      end evaluatePoly3_derivativeAtZero;

      function regFun3 "Co-monotonic and C1 smooth regularization function"
        extends Modelica.Icons.Function;
        input Real x "Abscissa value";
        input Real x0 "Lower abscissa value";
        input Real x1 "Upper abscissa value";
        input Real y0 "Ordinate value at lower abscissa value";
        input Real y1 "Ordinate value at upper abscissa value";
        input Real y0d "Derivative at lower abscissa value";
        input Real y1d "Derivative at upper abscissa value";
        output Real y "Ordinate value";
        output Real c "Slope of linear section between two cubic polynomials or dummy linear section slope if single cubic is used";
      protected
        Real h0 "Width of interval i=0";
        Real Delta0 "Slope of secant on interval i=0";
        Real xstar "Inflection point of cubic polynomial S0";
        Real mu "Distance of inflection point and left limit x0";
        Real eta "Distance of right limit x1 and inflection point";
        Real omega "Slope of cubic polynomial S0 at inflection point";
        Real rho "Weighting factor of eta and eta_tilde, mu and mu_tilde";
        Real theta0 "Slope metric";
        Real mu_tilde "Distance of start of linear section and left limit x0";
        Real eta_tilde "Distance of right limit x1 and end of linear section";
        Real xi1 "Start of linear section";
        Real xi2 "End of linear section";
        Real a1 "Leading coefficient of cubic on the left";
        Real a2 "Leading coefficient of cubic on the right";
        Real const12 "Integration constant of left cubic, linear section";
        Real const3 "Integration constant of right cubic";
        Real aux01;
        Real aux02;
        Boolean useSingleCubicPolynomial = false "Indicate to override further logic and use single cubic";
      algorithm
// Check arguments: Data point position
        assert(x0 < x1, "regFun3(): Data points not sorted appropriately (x0 = " + String(x0) + " > x1 = " + String(x1) + "). Please flip arguments.");
// Check arguments: Data point derivatives
        if y0d * y1d >= 0 then
        else
          assert(abs(y0d) < Modelica.Constants.eps or abs(y1d) < Modelica.Constants.eps, "regFun3(): Derivatives at data points do not allow co-monotone interpolation, as both are non-zero, of opposite sign and have an absolute value larger than machine eps (y0d = " + String(y0d) + ", y1d = " + String(y1d) + "). Please correct arguments.");
        end if;
// Derivatives at data points allow co-monotone interpolation, nothing to do
// Strictly speaking, derivatives at data points do not allow co-monotone interpolation, however, they may be numerically zero so assert this
        h0 := x1 - x0;
        Delta0 := (y1 - y0) / h0;
        if abs(Delta0) <= 0 then
          y := y0 + Delta0 * (x - x0);
          c := 0;
        elseif abs(y1d + y0d - 2 * Delta0) < 100 * Modelica.Constants.eps then
          y := y0 + (x - x0) * (y0d + (x - x0) / h0 * ((-2 * y0d) - y1d + 3 * Delta0 + (x - x0) * (y0d + y1d - 2 * Delta0) / h0));
          aux01 := (x0 + x1) / 2;
          c := 3 * (y0d + y1d - 2 * Delta0) * (aux01 - x0) ^ 2 / h0 ^ 2 + 2 * ((-2 * y0d) - y1d + 3 * Delta0) * (aux01 - x0) / h0 + y0d;
        else
          xstar := 1 / 3 * ((-3 * x0 * y0d) - 3 * x0 * y1d + 6 * x0 * Delta0 - 2 * h0 * y0d - h0 * y1d + 3 * h0 * Delta0) / ((-y0d) - y1d + 2 * Delta0);
          mu := xstar - x0;
          eta := x1 - xstar;
          omega := 3 * (y0d + y1d - 2 * Delta0) * (xstar - x0) ^ 2 / h0 ^ 2 + 2 * ((-2 * y0d) - y1d + 3 * Delta0) * (xstar - x0) / h0 + y0d;
          aux01 := 0.25 * sign(Delta0) * min(abs(omega), abs(Delta0)) "Slope c if not using plain cubic S0";
          if abs(y0d - y1d) <= 100 * Modelica.Constants.eps then
            aux02 := y0d;
            if y1 > y0 + y0d * (x1 - x0) then
              useSingleCubicPolynomial := true;
            end if;
          elseif abs(y1d + y0d - 2 * Delta0) < 100 * Modelica.Constants.eps then
            aux02 := (6 * Delta0 * (y1d + y0d - 3 / 2 * Delta0) - y1d * y0d - y1d ^ 2 - y0d ^ 2) * (if y1d + y0d - 2 * Delta0 >= 0 then 1 else -1) * Modelica.Constants.inf;
          else
            aux02 := (6 * Delta0 * (y1d + y0d - 3 / 2 * Delta0) - y1d * y0d - y1d ^ 2 - y0d ^ 2) / (3 * (y1d + y0d - 2 * Delta0));
          end if;
          if (mu > 0 and eta < h0 and Delta0 * omega <= 0 or abs(aux01) < abs(aux02) and aux02 * Delta0 >= 0 or abs(aux01) < abs(0.1 * Delta0)) and not useSingleCubicPolynomial then
            c := aux01;
            if abs(c) < abs(aux02) and aux02 * Delta0 >= 0 then
              c := aux02;
            end if;
            if abs(c) < abs(0.1 * Delta0) then
              c := 0.1 * Delta0;
            end if;
            theta0 := (y0d * mu + y1d * eta) / h0;
            if abs(theta0 - c) < 1e-6 then
              c := (1 - 1e-6) * theta0;
            end if;
            rho := 3 * (Delta0 - c) / (theta0 - c);
            mu_tilde := rho * mu;
            eta_tilde := rho * eta;
            xi1 := x0 + mu_tilde;
            xi2 := x1 - eta_tilde;
            a1 := (y0d - c) / max(mu_tilde ^ 2, 100 * Modelica.Constants.eps);
            a2 := (y1d - c) / max(eta_tilde ^ 2, 100 * Modelica.Constants.eps);
            const12 := y0 - a1 / 3 * (x0 - xi1) ^ 3 - c * x0;
            const3 := y1 - a2 / 3 * (x1 - xi2) ^ 3 - c * x1;
            if x < xi1 then
              y := a1 / 3 * (x - xi1) ^ 3 + c * x + const12;
            elseif x < xi2 then
              y := c * x + const12;
            else
              y := a2 / 3 * (x - xi2) ^ 3 + c * x + const3;
            end if;
          else
            y := y0 + (x - x0) * (y0d + (x - x0) / h0 * ((-2 * y0d) - y1d + 3 * Delta0 + (x - x0) * (y0d + y1d - 2 * Delta0) / h0));
            aux01 := (x0 + x1) / 2;
            c := 3 * (y0d + y1d - 2 * Delta0) * (aux01 - x0) ^ 2 / h0 ^ 2 + 2 * ((-2 * y0d) - y1d + 3 * Delta0) * (aux01 - x0) / h0 + y0d;
          end if;
        end if;
// Points (x0,y0) and (x1,y1) on horizontal line
// Degenerate case as we cannot fulfill the C1 goal an comonotone behaviour at the same time
// y == y0 == y1 with additional term to assist automatic differentiation
// Inflection point at +/- infinity, thus S0 is co-monotone and can be returned directly
// Provide a "dummy linear section slope" as the slope of the cubic at x:=(x0+x1)/2
// Points (x0,y0) and (x1,y1) not on horizontal line and inflection point of S0 not at +/- infinity
// Do actual interpolation
// y0 == y1 (value and sign equal) -> resolve indefinite 0/0
// If y1 is above the linear extension through (x0/y0)
// with slope y0d (when slopes are identical)
//  -> then always used single cubic polynomial
// (y1d+y0d-2*Delta0) approximately 0 -> avoid division by 0
// Okay, no guarding necessary
//aux02 := -1/3*(y0d^2+y0d*y1d-6*y0d*Delta0+y1d^2-6*y1d*Delta0+9*Delta0^2)/(y0d+y1d-2*Delta0);
//aux02 := -1/3*(6*y1d*y0*x1+y0d*y1d*x1^2-6*y0d*x0*y0+y0d^2*x0^2+y0d^2*x1^2+y1d^2*x1^2+y1d^2*x0^2-2*y0d*x0*y1d*x1-2*x0*y0d^2*x1+y0d*y1d*x0^2+6*y0d*x0*y1-6*y0d*y1*x1+6*y0d*y0*x1-2*x0*y1d^2*x1-6*y1d*y1*x1+6*y1d*x0*y1-6*y1d*x0*y0-18*y1*y0+9*y1^2+9*y0^2)/(y0d*x1^2-2*x0*y0d*x1+y1d*x1^2-2*x0*y1d*x1-2*y1*x1+2*y0*x1+y0d*x0^2+y1d*x0^2+2*x0*y1-2*x0*y0);
// Test criteria (also used to avoid saddle points that lead to integrator contraction):
//
//  1. Cubic is not monotonic (from Gasparo Morandi)
//       ((mu > 0) and (eta < h0) and (Delta0*omega <= 0))
//
//  2. Cubic may be monotonic but the linear section slope c is either too close
//     to zero or the end point of the linear section is left of the start point
//     Note however, that the suggested slope has to have the same sign as Delta0.
//       (abs(aux01)<abs(aux02) and aux02*Delta0>=0)
//
//  3. Cubic may be monotonic but the resulting slope in the linear section
//     is too close to zero (less than 1/10 of Delta0).
//       (c < Delta0 / 10)
//
// NOT monotonic using plain cubic S0, use piecewise function S0 tilde instead
// Avoid saddle points that are co-monotonic but lead to integrator contraction
// Slightly reduce c in order to avoid ill-posed problem
// Do actual interpolation
// Cubic S0 is monotonic, use it as is
// Provide a "dummy linear section slope" as the slope of the cubic at x:=(x0+x1)/2
        annotation(
          smoothOrder = 1,
          Documentation(revisions = "<html>
<ul>
<li><i>May 2008</i> by <a href=\"mailto:Michael.Sielemann@dlr.de\">Michael Sielemann</a>:<br/>Designed and implemented.</li>
<li><i>February 2011</i> by <a href=\"mailto:Michael.Sielemann@dlr.de\">Michael Sielemann</a>:<br/>If the inflection point of the cubic S0 was at +/- infinity, the test criteria of <i>[Gasparo and Morandi, 1991]</i> result in division by zero. This case is handled properly now.</li>
<li><i>March 2013</i> by <a href=\"mailto:Michael.Sielemann@dlr.de\">Michael Sielemann</a>:<br/>If the arguments prescribed a degenerate case with points <code>(x0,y0)</code> and <code>(x1,y1)</code> on horizontal line, then return value <code>c</code> was undefined. This was corrected. Furthermore, an additional term was included for the computation of <code>y</code> in this case to assist automatic differentiation.</li>
</ul>
</html>", info = "<html>
<p>
Approximates a function in a region between <code>x0</code> and <code>x1</code>
such that
</p>
<ul>
<li> The overall function is continuous with a
     continuous first derivative everywhere.</li>
<li> The function is co-monotone with the given
     data points.</li>
</ul>
<p>
In this region, a continuation is constructed from the given points
<code>(x0, y0)</code>, <code>(x1, y1)</code> and the respective
derivatives. For this purpose, a single polynomial of third order or two
cubic polynomials with a linear section in between are used <i>[Gasparo
and Morandi, 1991]</i>. This algorithm was extended with two additional
conditions to avoid saddle points with zero/infinite derivative that lead to
integrator step size reduction to zero.
</p>
<p>
This function was developed for pressure loss correlations properly
addressing the static head on top of the established requirements
for monotonicity and smoothness. In this case, the present function
allows to implement the exact solution in the limit of
<code>x1-x0 -> 0</code> or <code>y1-y0 -> 0</code>.
</p>
<p>
Typical screenshots for two different configurations
are shown below. The first one illustrates five different settings of <code>xi</code> and <code>yid</code>:
</p>
<p>
<img src=\"modelica://Modelica/Resources/Images/Fluid/Components/regFun3_a.png\"
      alt=\"regFun3_a.png\">
</p>
<p>
The second graph shows the continuous derivative of this regularization function:
</p>
<p>
<img src=\"modelica://Modelica/Resources/Images/Fluid/Components/regFun3_b.png\"
     alt=\"regFun3_a.png\">
</p>

<p>
<b>Literature</b>
</p>

<dl>
<dt> Gasparo M. G. and Morandi R. (1991):</dt>
<dd> <b>Piecewise cubic monotone interpolation with assigned slopes</b>.
     Computing, Vol. 46, Issue 4, December 1991, pp. 355 - 365.</dd>
</dl>
</html>"));
      end regFun3;

      function cubicHermite "Evaluate a cubic Hermite spline"
        extends Modelica.Icons.Function;
        input Real x "Abscissa value";
        input Real x1 "Lower abscissa value";
        input Real x2 "Upper abscissa value";
        input Real y1 "Lower ordinate value";
        input Real y2 "Upper ordinate value";
        input Real y1d "Lower gradient";
        input Real y2d "Upper gradient";
        output Real y "Interpolated ordinate value";
      protected
        Real h "Distance between x1 and x2";
        Real t "abscissa scaled with h, i.e., t=[0..1] within x=[x1..x2]";
        Real h00 "Basis function 00 of cubic Hermite spline";
        Real h10 "Basis function 10 of cubic Hermite spline";
        Real h01 "Basis function 01 of cubic Hermite spline";
        Real h11 "Basis function 11 of cubic Hermite spline";
        Real aux3 "t cube";
        Real aux2 "t square";
      algorithm
        h := x2 - x1;
        if abs(h) > 0 then
          t := (x - x1) / h;
          aux3 := t ^ 3;
          aux2 := t ^ 2;
          h00 := 2 * aux3 - 3 * aux2 + 1;
          h10 := aux3 - 2 * aux2 + t;
          h01 := (-2 * aux3) + 3 * aux2;
          h11 := aux3 - aux2;
          y := y1 * h00 + h * y1d * h10 + y2 * h01 + h * y2d * h11;
        else
          y := (y1 + y2) / 2;
        end if;
// Regular case
// Degenerate case, x1 and x2 are identical, return step function
        annotation(
          smoothOrder = 3,
          Documentation(revisions = "<html>
<ul>
<li><i>May 2008</i>
    by <a href=\"mailto:Michael.Sielemann@dlr.de\">Michael Sielemann</a>:<br>
    Designed and implemented.</li>
</ul>
</html>"));
      end cubicHermite;

      function cubicHermite_withDerivative "Evaluate a cubic Hermite spline, return value and derivative"
        extends Modelica.Icons.Function;
        input Real x "Abscissa value";
        input Real x1 "Lower abscissa value";
        input Real x2 "Upper abscissa value";
        input Real y1 "Lower ordinate value";
        input Real y2 "Upper ordinate value";
        input Real y1d "Lower gradient";
        input Real y2d "Upper gradient";
        output Real y "Interpolated ordinate value";
        output Real dy_dx "Derivative dy/dx at abscissa value x";
      protected
        Real h "Distance between x1 and x2";
        Real t "abscissa scaled with h, i.e., t=[0..1] within x=[x1..x2]";
        Real h00 "Basis function 00 of cubic Hermite spline";
        Real h10 "Basis function 10 of cubic Hermite spline";
        Real h01 "Basis function 01 of cubic Hermite spline";
        Real h11 "Basis function 11 of cubic Hermite spline";
        Real h00d "d/dt h00";
        Real h10d "d/dt h10";
        Real h01d "d/dt h01";
        Real h11d "d/dt h11";
        Real aux3 "t cube";
        Real aux2 "t square";
      algorithm
        h := x2 - x1;
        if abs(h) > 0 then
          t := (x - x1) / h;
          aux3 := t ^ 3;
          aux2 := t ^ 2;
          h00 := 2 * aux3 - 3 * aux2 + 1;
          h10 := aux3 - 2 * aux2 + t;
          h01 := (-2 * aux3) + 3 * aux2;
          h11 := aux3 - aux2;
          h00d := 6 * (aux2 - t);
          h10d := 3 * aux2 - 4 * t + 1;
          h01d := 6 * (t - aux2);
          h11d := 3 * aux2 - 2 * t;
          y := y1 * h00 + h * y1d * h10 + y2 * h01 + h * y2d * h11;
          dy_dx := y1 * h00d / h + y1d * h10d + y2 * h01d / h + y2d * h11d;
        else
          y := (y1 + y2) / 2;
          dy_dx := sign(y2 - y1) * Modelica.Constants.inf;
        end if;
// Regular case
// Degenerate case, x1 and x2 are identical, return step function
        annotation(
          smoothOrder = 3,
          Documentation(revisions = "<html>
<ul>
<li><i>May 2008</i>
    by <a href=\"mailto:Michael.Sielemann@dlr.de\">Michael Sielemann</a>:<br>
    Designed and implemented.</li>
</ul>
</html>"));
      end cubicHermite_withDerivative;
      annotation(
        Documentation(info = "<html>

</html>"));
    end Utilities;

    package Icons
      extends Modelica.Icons.IconsPackage;

      partial class VariantLibrary "This icon will be removed in future Modelica versions, use Modelica.Icons.VariantsPackage instead."
        // extends Modelica.Icons.VariantsPackage;
        // extends Modelica.Icons.ObsoleteModel;
        annotation(
          Documentation(info = "<html>
<p>
This icon will be removed in future versions of the Modelica Standard Library.
Instead the icon <a href=\"modelica://Modelica.Icons.VariantsPackage\">Modelica.Icons.VariantsPackage</a> shall be used.
</p>
</html>"),
          Icon(graphics = {Rectangle(lineColor = {200, 200, 200}, fillColor = {248, 248, 248}, fillPattern = FillPattern.HorizontalCylinder, extent = {{-100, -100}, {100, 100}}, radius = 25.0), Rectangle(lineColor = {128, 128, 128}, extent = {{-100, -100}, {100, 100}}, radius = 25.0), Ellipse(origin = {10, 10}, fillColor = {76, 76, 76}, pattern = LinePattern.None, fillPattern = FillPattern.Solid, extent = {{-80.0, -80.0}, {-20.0, -20.0}}), Ellipse(origin = {10, 10}, pattern = LinePattern.None, fillPattern = FillPattern.Solid, extent = {{0.0, -80.0}, {60.0, -20.0}}), Ellipse(origin = {10, 10}, fillColor = {128, 128, 128}, pattern = LinePattern.None, fillPattern = FillPattern.Solid, extent = {{0.0, 0.0}, {60.0, 60.0}}), Ellipse(origin = {10, 10}, lineColor = {128, 128, 128}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, extent = {{-80.0, 0.0}, {-20.0, 60.0}})}));
      end VariantLibrary;

      partial package BaseClassLibrary "This icon will be removed in future Modelica versions, use Modelica.Icons.BasePackage instead."
        extends Modelica.Icons.BasesPackage;
        // extends Modelica.Icons.ObsoleteModel;
        annotation(
          Documentation(info = "<html>
<p>
This icon will be removed in future versions of the Modelica Standard Library.
Instead the icon <a href=\"modelica://Modelica.Icons.BasesPackage\">Modelica.Icons.BasesPackage</a> shall be used.
</p>
</html>"));
      end BaseClassLibrary;
    end Icons;
    annotation(
      Icon(graphics = {Polygon(points = {{-70, 26}, {68, -44}, {68, 26}, {2, -10}, {-70, -42}, {-70, 26}}, lineColor = {0, 0, 0}), Line(points = {{2, 42}, {2, -10}}), Rectangle(extent = {{-18, 50}, {22, 42}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid)}),
      preferredView = "info",
      Documentation(info = "<html>
<p>
Library <b>Modelica.Fluid</b> is a <b>free</b> Modelica package providing components for
<b>1-dimensional thermo-fluid flow</b> in networks of vessels, pipes, fluid machines, valves and fittings.
A unique feature is that the component equations and the media models
as well as pressure loss and heat transfer correlations are decoupled from each other.
All components are implemented such that they can be used for
media from the Modelica.Media library. This means especially that an
incompressible or compressible medium, a single or a multiple
substance medium with one or more phases might be used.
</p>

<p>
In the next figure, several features of the library are demonstrated with
a simple heating system with a closed flow cycle. By just changing one configuration parameter in the system object the equations are changed between steady-state and dynamic simulation with fixed or steady-state initial conditions.
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Fluid/UsersGuide/HeatingSystem.png\" border=\"1\"
     alt=\"HeatingSystem.png\">
</p>

<p>
With respect to previous versions, the design
of the connectors has been changed in a non-backward compatible way,
using the recently developed concept
of stream connectors that results in much more reliable simulations
(see also <a href=\"modelica://Modelica/Resources/Documentation/Fluid/Stream-Connectors-Overview-Rationale.pdf\">Stream-Connectors-Overview-Rationale.pdf</a>).
This extension was included in Modelica 3.1.
As of Jan. 2009, the stream concept is supported in Dymola 7.1.
It is recommended to use Dymola 7.2 (available since Feb. 2009), or a later Dymola version,
since this version supports a new annotation to connect very
conveniently to vectors of connectors.
Other tool vendors will support the stream concept as well.
</p>

<p>
The following parts are useful, when newly starting with this library:
</p>
<ul>
<li> <a href=\"modelica://Modelica.Fluid.UsersGuide\">Modelica.Fluid.UsersGuide</a>.</li>
<li> <a href=\"modelica://Modelica.Fluid.UsersGuide.ReleaseNotes\">Modelica.Fluid.UsersGuide.ReleaseNotes</a>
     summarizes the changes of the library releases.</li>
<li> <a href=\"modelica://Modelica.Fluid.Examples\">Modelica.Fluid.Examples</a>
     contains examples that demonstrate the usage of this library.</li>
</ul>
<p>
<b>Licensed by the Modelica Association under the Modelica License 2</b><br>
Copyright &copy; 2002-2015, ABB, DLR, Dassault Syst&egrave;mes AB, Modelon, TU Braunschweig, TU Hamburg-Harburg, Politecnico di Milano.
</p>
<p>
<i>This Modelica package is <u>free</u> software and the use is completely at <u>your own risk</u>; it can be redistributed and/or modified under the terms of the Modelica License 2. For license conditions (including the disclaimer of warranty) see <a href=\"modelica://Modelica.UsersGuide.ModelicaLicense2\">Modelica.UsersGuide.ModelicaLicense2</a> or visit <a href=\"https://www.modelica.org/licenses/ModelicaLicense2\"> https://www.modelica.org/licenses/ModelicaLicense2</a>.</i>
</p>
</html>"));
  end Fluid;

  package Media "Library of media property models"
    extends Modelica.Icons.Package;
    import SI = Modelica.SIunits;
    import Cv = Modelica.SIunits.Conversions;

    package UsersGuide "User's Guide of Media Library"
      extends Modelica.Icons.Information;

      package MediumUsage "Medium usage"
        extends Modelica.Icons.Information;

        class BasicUsage "Basic usage"
          extends Modelica.Icons.Information;
          annotation(
            Documentation(info = "<HTML>
<h4>Basic usage of medium model</h4>
<p>
Media models in Modelica.Media are provided by packages, inheriting from the
partial package Modelica.Media.Interfaces.PartialMedium. Every package defines:
</p>
<ul>
<li> Medium <b>constants</b> (such as the number of chemical substances,
     molecular data, critical properties, etc.).</li>
<li> A BaseProperties <b>model</b>, to compute the basic thermodynamic
     properties of the fluid;</li>
<li> <b>setState_XXX</b> functions to compute the thermodynamic state record from
     different input arguments (such as density, temperature, and composition which
     would be setState_dTX);</li>
<li> <b>Functions</b> to compute additional properties (such as saturation
     properties, viscosity, thermal conductivity, etc.).</li>
</ul>
<p>
There are - as stated above - two different basic ways of using the Media library which
will be described in more details in the following section. One way is to use the model BaseProperties.
Every instance of BaseProperties for any medium model provides <b>3+nXi
equations</b> for the following <b>5+nXi variables</b> that are declared in
the medium model (nXi is the number of independent mass fractions, see
explanation below):
</p>
<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"><b>Variable</b></td>
      <td valign=\"top\"><b>Unit</b></td>
      <td valign=\"top\"><b>Description</b></td></tr>
  <tr><td valign=\"top\">T</td>
      <td valign=\"top\">K</td>
      <td valign=\"top\">temperature</td></tr>
  <tr><td valign=\"top\">p</td>
      <td valign=\"top\">Pa</td>
      <td valign=\"top\">absolute pressure</td></tr>
  <tr><td valign=\"top\">d</td>
      <td valign=\"top\">kg/m3</td>
      <td valign=\"top\">density</td></tr>
  <tr><td valign=\"top\">u</td>
      <td valign=\"top\">J/kg</td>
      <td valign=\"top\">specific internal energy</td></tr>
  <tr><td valign=\"top\">h</td>
      <td valign=\"top\">J/kg</td>
      <td valign=\"top\">specific enthalpy (h = u + p/d)</td></tr>
  <tr><td valign=\"top\">Xi[nXi]</td>
      <td valign=\"top\">kg/kg</td>
      <td valign=\"top\">independent mass fractions m_i/m</td></tr>
  <tr><td valign=\"top\">X[nX]</td>
      <td valign=\"top\">kg/kg</td>
      <td valign=\"top\">All mass fractions m_i/m. X is defined in BaseProperties by:<br>
          X = <b>if</b> reducedX <b>then</b> vector([Xi; 1-<b>sum</b>(Xi)])
          <b>else</b> Xi </td></tr>
</table>
<p>
<b>Two</b> variables out of p, d, h, or u, as well as the
<b>mass fractions</b> Xi are the <b>independent</b> variables and the
medium model basically provides equations to compute
the remaining variables, including the full mass fraction vector X
(more details to Xi and X are given further below).
</p>
<p>
In a component, the most basic usage of a medium model is as follows
</p>
<pre>
  <b>model</b> Pump
    <b>replaceable package</b> Medium = Modelica.Media.Interfaces.PartialMedium
                         \"Medium model\" <b>annotation</b> (choicesAllMatching = <b>true</b>);
    Medium.BaseProperties medium_a \"Medium properties at location a (e.g., port_a)\";
    // Use medium variables (medium_a.p, medium_a.T, medium_a.h, ...)
     ...
  <b>end</b> Pump;
</pre>
<p>
The second way is to use the setState_XXX functions to compute the thermodynamic state
record from which all other thermodynamic state variables can be computed (see
<a href=\"modelica://Modelica.Media.UsersGuide.MediumDefinition.BasicDefinition\">
Basic definition of medium</a> for further details on ThermodynamicState). The setState_XXX functions
accept either X or Xi (see explanation below) and will decide internally which of these two compositions
is provided by the user. The four fundamental setState_XXX functions are provided in PartialMedium
</p>
<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"><b>Function</b></td>
      <td valign=\"top\"><b>Description</b></td>
      <td valign=\"top\"><b>Short-form for<br>single component medium</b></td></tr>
  <tr><td valign=\"top\">setState_dTX</td>
      <td valign=\"top\">computes ThermodynamicState from density, temperature, and composition X or Xi</td>
      <td valign=\"top\">setState_dT</td></tr>
  <tr><td valign=\"top\">setState_phX</td>
      <td valign=\"top\">computes ThermodynamicState from pressure, specific enthalpy, and composition X or Xi</td>
      <td valign=\"top\">setState_ph</td></tr>
  <tr><td valign=\"top\">setState_psX</td>
      <td valign=\"top\">computes ThermodynamicState from pressure, specific entropy, and composition X or Xi</td>
      <td valign=\"top\">setState_ps</td></tr>
  <tr><td valign=\"top\">setState_pTX</td>
      <td valign=\"top\">computes ThermodynamicState from pressure, temperature, and composition X or Xi</td>
      <td valign=\"top\">setState_pT</td></tr>
</table>
<p>
The simple example that explained the basic usage of BaseProperties would then become
<pre>
  <b>model</b> Pump
    <b>replaceable package</b> Medium = Modelica.Media.Interfaces.PartialMedium
                         \"Medium model\" <b>annotation</b> (choicesAllMatching = <b>true</b>);
    Medium.ThermodynamicState state_a \"Thermodynamic state record at location a (e.g., port_a)\";
    // Compute medium variables from thermodynamic state record (pressure(state_a), temperature(state_a),
    // specificEnthalpy(state_a), ...)
    ...
  <b>end</b> Pump;
</pre>
<p>
All media models are directly or indirectly a subpackage of package
Modelica.Media.Interfaces.PartialMedium. Therefore,
a medium model in a component should inherit from this
partial package. Via the annotation \"choicesAllMatching = true\" it
is defined that the tool should display a selection box with
all loaded packages that inherit from PartialMedium. An example
is given in the next figure:
</p>

<p>
<IMG src=\"modelica://Modelica/Resources/Images/Media/UsersGuide/mediumMenu.png\" ALT=\"medium selection menu\">
</p>

<p>
A selected medium model leads, e.g., to the following equation:
</p>
<pre>
  Pump pump(<b>redeclare package</b> Medium = Modelica.Media.Water.SimpleLiquidWater);
</pre>
<p>
Usually, a medium model is associated with the variables of a
fluid connector. Therefore, equations have to be defined in a model
that relate the variables in the connector with the variables
in the medium model:
</p>
<pre>
  <b>model</b> Pump
    <b>replaceable package</b> Medium = Modelica.Media.Interfaces.PartialMedium
                         \"Medium model\" <b>annotation</b> (choicesAllMatching = <b>true</b>);
    Medium.BaseProperties medium_a \"Medium properties of port_a\";
    // definition of the fluid port port_a
     ...
  <b>equation</b>
    medium.p = port_a.p;
    medium.h = port_a.h;
    medium.Xi = port_a.Xi;
     ...
  <b>end</b> Pump;
</pre>
in the case of using BaseProperties or
<pre>
  <b>model</b> Pump
    <b>replaceable package</b> Medium = Modelica.Media.Interfaces.PartialMedium
                         \"Medium model\" <b>annotation</b> (choicesAllMatching = <b>true</b>);
    Medium.ThermodynamicState state_a \"Thermodynamic state record of medium at port_a\";
    // definition of the fluid port port_a
     ...
  <b>equation</b>
    state_a = Medium.setState_phX(port_a.p, port_a.h, port_a.Xi) // if port_a contains the variables
                                                                 // p, h, and Xi
     ...
  <b>end</b> Pump;
</pre>
<p>
in the case of using ThermodynamicState.
</p>
<p>
If a component model shall treat both single and multiple
substance fluids, equations for the mass fractions have to be
present (above: medium.Xi = port_a.Xi) in the model. According
to the Modelica semantics, the equations of the mass fractions
are ignored, if the dimension of Xi is zero, i.e., for a single-component
medium. Note, by specific techniques sketched in section
\"Medium definition\", the independent variables in the medium model
need not to be the same as the variables in the connector and still
get the same efficiency, as if the same variables would be used.
</p>

<p>
If a fluid consists of a single
substance, <b>nXi = 0</b> and the vector of mass fractions Xi is not
present. If a fluid consists of nS substances,
the medium model may define the number of independent
mass fractions <b>nXi</b> to be <b>nS</b>, <b>nS-1</b>, or zero.
In all cases, balance equations for nXi substances have to be
given in the corresponding component (see discussion below).
Note, that if nXi = nS, the constraint \"sum(Xi)=1\" between the mass
fractions is <b>not</b> present in the model; in that case, it is necessary to
provide consistent start values for Xi such that sum(Xi) = 1.
</p>

<p>
The reason for this definition of Xi is that a fluid component library
can be implemented by using only the independent mass fractions Xi and
then via the medium it is defined how Xi is interpreted:
</p>

<ul>
<li> If Xi = nS, then the constraint equation sum(X) = 1 is neglected
     during simulation. By making sure that the initial conditions of X
     fulfill this constraint, it can usually be guaranteed that small
     errors in sum(X) = 1 remain small although this constraint equation is
     not explicitly used during the simulation. This approach is usually useful
     if components of the mixture can become very small. If such a small
     quantity is computed via the equation 1 - sum(X[1:nX-1]), there might
     be large numerical errors and it is better to compute it via
     the corresponding balance equation.</li>
<li> If Xi = nS-1, then the true independent mass fractions are used
     in the fluid component and the last component of X is computed via
     X[nX] = 1 - sum(Xi). This is useful for, e.g., MoistAir, where the
     number of states should be as small as possible without introducing
     numerical problems.</li>
<li> If Xi = 0, then the reference value of composition reference_X is
     assumed. This case is useful to avoid composition states in all
     the cases when the composition will always be constant, e.g., with
     circuits having fixed composition sources.</li>
</ul>

<p>
The full vector of mass fractions <b>X[nX]</b> is computed in
PartialMedium.BaseProperties based on Xi, reference_X, and the information whether Xi = nS or nS-1. For single-substance media, nX = 0, so there's also no X vector. For multiple-substance media, nX = nS, and X always contains the full vector of mass fractions. In order to reduce confusion for the user of a fluid component library, \"Xi\" has the annotation \"HideResult=true\", meaning, that this variable is not shown in the plot window. Only X is shown in the plot window and this vector always contains all mass fractions.
</p>
</HTML>"));
        end BasicUsage;

        class BalanceVolume "Balance volume"
          extends Modelica.Icons.Information;
          annotation(
            Documentation(info = "<HTML>
<p>
Fluid libraries usually have balance volume components with one fluid connector
port that fulfill the mass and energy balance and on a different grid components that
fulfill the momentum balance. A balance volume component, called junction
volume below, should be primarily implemented in the following way
(see also the implementation in
<a href=\"modelica://Modelica.Media.Examples.Tests.Components.PortVolume\">
Modelica.Media.Examples.Tests.Components.PortVolume</a>):
</p>
<pre>
  <b>model</b> JunctionVolume
    <b>import</b> SI=Modelica.SIunits;
    <b>import</b> Modelica.Media.Examples.Tests.Components.FluidPort_a;

    <b>parameter</b> SI.Volume V = 1e-6 \"Fixed size of junction volume\";
    <b>replaceable package</b> Medium = Modelica.Media.Interfaces.PartialMedium
                           \"Medium model\" <b>annotation</b> (choicesAllMatching = <b>true</b>);

    FluidPort_a port(<b>redeclare package</b> Medium = Medium);
    Medium.BaseProperties medium(preferredMediumStates = <b>true</b>);

    SI.Energy U               \"Internal energy of junction volume\";
    SI.Mass   M               \"Mass of junction volume\";
    SI.Mass   MX[Medium.nXi] \"Independent substance masses of junction volume\";
  <b>equation</b>
    medium.p   = port.p;
    medium.h   = port.h;
    medium.Xi = port.Xi;

    M  = V*medium.d;                  // mass of JunctionVolume
    MX = M*medium.Xi;                // mass fractions in JunctionVolume
    U  = M*medium.u;                  // internal energy in JunctionVolume

    <b>der</b>(M)  = port.m_flow;    // mass balance
    <b>der</b>(MX) = port.mX_flow;   // substance mass balance
    <b>der</b>(U)  = port.H_flow;    // energy balance
  <b>end</b> JunctionVolume;
</pre>
<p>
Assume the Modelica.Media.Air.SimpleAir medium model is used with
the JunctionVolume model above. This medium model uses pressure p
and temperature T as independent variables. If the flag
\"preferredMediumStates\" is set to <b>true</b> in the declaration
of \"medium\", then the independent variables of this medium model
get the attribute \"stateSelect = StateSelect.prefer\", i.e., the
Modelica translator should use these variables as states, if this
is possible. Basically, this means that
constraints between the
potential states p,T and the potential states U,M are present.
A Modelica tool will therefore <b>automatically</b>
differentiate medium equations and will use the following
equations for code generation (note the equations related to X are
removed, because SimpleAir consists of a single substance only):
</p>
<pre>
    M  = V*medium.d;
    U  = M*medium.u;

    // balance equations
    <b>der</b>(M)  = port.m_flow;
    <b>der</b>(U)  = port.H_flow;

    // abbreviations introduced to get simpler terms
    p = medium.p;
    T = medium.T;
    d = medium.d;
    u = medium.u;
    h = medium.h;

    // medium equations
    d = fd(p,T);
    h = fh(p,T);
    u = h - p/d;

    // equations derived <b>automatically</b> by a Modelica tool due to index reduction
    <b>der</b>(U) = <b>der</b>(M)*u + M*<b>der</b>(u);
    <b>der</b>(M) = V*<b>der</b>(d);
    <b>der</b>(u) = <b>der</b>(h) - <b>der</b>(p)/d - p/<b>der</b>(d);
    <b>der</b>(d) = <b>der</b>(fd,p)*<b>der</b>(p) + <b>der</b>(fd,T)*<b>der</b>(T);
    <b>der</b>(h) = <b>der</b>(fh,p)*<b>der</b>(p) + <b>der</b>(fd,T)*<b>der</b>(T);
</pre>
<p>
Note, that \"der(y,x)\" is an operator that characterizes
in the example above the partial derivative of y with respect to x
(this operator will be included in one of the next Modelica language
releases).
All media models in this library are written in such a way that
at least the partial derivatives of the medium variables with
respect to the independent variables are provided, either because
the equations are directly given (= symbolic differentiation is possible)
or because the derivative of the corresponding function (such as fd above)
is provided. A Modelica tool will transform the equations above
in differential equations with p and T as states, i.e., will
generate equations to compute <b>der</b>(p) and <b>der</b>(T) as function of p and T.
</p>

<p>
Note, when preferredMediumStates = <b>false</b>, no differentiation
will take place and the Modelica translator will use the variables
appearing differentiated as states, i.e., M and U. This has the
disadvantage that for many media non-linear systems of equations are
present to compute the intrinsic properties p, d, T, u, h from
M and U.
</p>
</HTML>"));
        end BalanceVolume;

        class ShortPipe "Short pipe"
          extends Modelica.Icons.Information;
          annotation(
            Documentation(info = "<HTML>
<p>
Fluid libraries have components with two ports that store
neither mass nor energy and fulfill the
momentum equation between their two ports, e.g., a short pipe. In most
cases this means that an equation is present relating the pressure
drop between the two ports and the mass flow rate from one to the
other port. Since no mass or energy is stored, no differential
equations for thermodynamic variables are present. A component model of this type
has therefore usually the following structure
(see also the implementation in
<a href=\"modelica://Modelica.Media.Examples.Tests.Components.ShortPipe\">
Modelica.Media.Examples.Tests.Components.ShortPipe</a>):
</p>
<pre>
  <b>model</b> ShortPipe
    <b>import</b> SI=Modelica.SIunits;
    <b>import</b> Modelica.Media.Examples.Tests.Components;

    // parameters defining the pressure drop equation

    <b>replaceable package</b> Medium = Modelica.Media.Interfaces.PartialMedium
                           \"Medium model\" <b>annotation</b> (choicesAllMatching = <b>true</b>);

    Component.FluidPort_a port_a (<b>redeclare package</b> Medium = Medium);
    Component.FluidPort_b port_b (<b>redeclare package</b> Medium = Medium);

    SI.Pressure dp = port_a.p - port_b.p \"Pressure drop\";
    Medium.BaseProperties medium_a \"Medium properties in port_a\";
    Medium.BaseProperties medium_b \"Medium properties in port_b\";
  <b>equation</b>
    // define media models of the ports
    medium_a.p   = port_a.p;
    medium_a.h   = port_a.h;
    medium_a.Xi = port_a.Xi;

    medium_b.p   = port_b.p;
    medium_b.h   = port_b.h;
    medium_b.Xi = port_b.Xi;

    // Handle reverse and zero flow (semiLinear is a built-in Modelica operator)
    port_a.H_flow   = <b>semiLinear</b>(port_a.m_flow, port_a.h, port_b.h);
    port_a.mXi_flow = <b>semiLinear</b>(port_a.m_flow, port_a.Xi, port_b.Xi);

    // Energy, mass and substance mass balance
    port_a.H_flow + port_b.H_flow = 0;
    port_a.m_flow + port_b.m_flow = 0;
    port_a.mXi_flow + port_b.mXi_flow = zeros(Medium.nXi);

    // Provide equation: port_a.m_flow = f(dp)
  <b>end</b> ShortPipe;
</pre>

<p>
The <b>semiLinear</b>(..) operator is basically defined as:
</p>
<pre>
    semiLinear(m_flow, ha, hb) = if m_flow &ge; 0 then m_flow*ha else m_flow*hb;
</pre>

<p>
that is, it computes the enthalpy flow rate either from the port_a or
from the port_b properties, depending on flow direction. The exact
details of this operator are given in
<a href=\"modelica://ModelicaReference.Operators.'semiLinear()'\">
ModelicaReference.Operators.'semiLinear()'</a>. Especially, rules
are defined in the Modelica specification that m_flow = 0 can be treated
in a \"meaningful way\". Especially, if n fluid components (such as pipes)
are connected together and the fluid connector from above is used,
a linear system of equations appear between
medium1.h, medium2.h, medium3.h, ..., port1.h, port2.h, port3.h, ...,
port1.H_flow, port2.H_flow, port3.H_flow, .... The rules for the
semiLinear(..) operator allow the following solution of this
linear system of equations:
</p>

<ul>
<li> n = 2 (two components are connected):<br>
     The linear system of equations can be analytically solved
     with the result
     <pre>
     medium1.h = medium2.h = port1.h = port2.h
     0 = port1.H_flow + port2.H_flow
     </pre>
     Therefore, no problems with zero mass flow rate are present.</li>

<li> n &gt; 2 (more than two components are connected together):<br>
     The linear system of equations is solved numerically during simulation.
     For m_flow = 0, the linear system becomes singular and has an
     infinite number of solutions. The simulator could use the solution t
     that is closest to the solution in the previous time step
     (\"least squares solution\"). Physically, the solution is determined
     by diffusion which is usually neglected. If diffusion is included,
     the linear system is regular.</li>
</ul>

</HTML>"));
        end ShortPipe;

        class OptionalProperties "Optional properties"
          extends Modelica.Icons.Information;
          annotation(
            Documentation(info = "<HTML>
<p>
In some cases additional medium properties are needed.
A component that needs these optional properties has to call
one of the functions listed in the following table. They are
defined as partial functions within package
<a href=\"modelica://Modelica.Media.Interfaces.PartialMedium\">PartialMedium</a>,
and then (optionally) implemented in actual medium packages.
If a component calls such an optional function and the
medium package does not provide a new implementation for this
function, an error message is printed at translation time,
since the function is \"partial\", i.e., incomplete.
The argument of all functions is the <b>state</b> record,
automatically defined by the BaseProperties model or specifically computed using the
setState_XXX functions, which contains the
minimum number of thermodynamic variables needed to compute all the additional
properties. In the table it is assumed that there is a declaration of the
form:
</p>
<pre>
   <b>replaceable package</b> Medium = Modelica.Media.Interfaces.PartialMedium;
   Medium.ThermodynamicState state;
</pre>

<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"><b>Function call</b></td>
      <td valign=\"top\"><b>Unit</b></td>
      <td valign=\"top\"><b>Description</b></td></tr>
  <tr><td valign=\"top\">Medium.dynamicViscosity(state)</td>
      <td valign=\"top\">Pa.s</td>
      <td valign=\"top\">dynamic viscosity</td></tr>
  <tr><td valign=\"top\">Medium.thermalConductivity(state)</td>
      <td valign=\"top\">W/(m.K)</td>
      <td valign=\"top\">thermal conductivity</td></tr>
  <tr><td valign=\"top\">Medium.prandtlNumber(state)</td>
      <td valign=\"top\">1</td>
      <td valign=\"top\">Prandtl number</td></tr>
  <tr><td valign=\"top\">Medium.specificEntropy(state)</td>
      <td valign=\"top\">J/(kg.K)</td>
      <td valign=\"top\">specific entropy</td></tr>
  <tr><td valign=\"top\">Medium.specificHeatCapacityCp(state)</td>
      <td valign=\"top\">J/(kg.K)</td>
      <td valign=\"top\">specific heat capacity at constant pressure</td></tr>
  <tr><td valign=\"top\">Medium.specificHeatCapacityCv(state)</td>
      <td valign=\"top\">J/(kg.K)</td>
      <td valign=\"top\">specific heat capacity at constant density</td></tr>
  <tr><td valign=\"top\">Medium.isentropicExponent(state)</td>
      <td valign=\"top\">1</td>
      <td valign=\"top\">isentropic exponent</td></tr>
  <tr><td valign=\"top\">Medium.isentropicEnthatlpy(pressure, state)</td>
      <td valign=\"top\">J/kg</td>
      <td valign=\"top\">isentropic enthalpy</td></tr>
  <tr><td valign=\"top\">Medium.velocityOfSound(state)</td>
      <td valign=\"top\">m/s</td>
      <td valign=\"top\">velocity of sound</td></tr>
  <tr><td valign=\"top\">Medium.isobaricExpansionCoefficient(state)</td>
      <td valign=\"top\">1/K</td>
      <td valign=\"top\">isobaric expansion coefficient</td></tr>
  <tr><td valign=\"top\">Medium.isothermalCompressibility(state)</td>
      <td valign=\"top\">1/Pa</td>
      <td valign=\"top\">isothermal compressibility</td></tr>
  <tr><td valign=\"top\">Medium.density_derp_h(state)</td>
      <td valign=\"top\">kg/(m3.Pa)</td>
      <td valign=\"top\">derivative of density by pressure at constant enthalpy</td></tr>
  <tr><td valign=\"top\">Medium.density_derh_p(state)</td>
      <td valign=\"top\">kg2/(m3.J)</td>
      <td valign=\"top\">derivative of density by enthalpy at constant pressure</td></tr>
  <tr><td valign=\"top\">Medium.density_derp_T(state)</td>
      <td valign=\"top\">kg/(m3.Pa)</td>
      <td valign=\"top\">derivative of density by pressure at constant temperature</td></tr>
  <tr><td valign=\"top\">Medium.density_derT_p(state)</td>
      <td valign=\"top\">kg/(m3.K)</td>
      <td valign=\"top\">derivative of density by temperature at constant pressure</td></tr>
  <tr><td valign=\"top\">Medium.density_derX(state)</td>
      <td valign=\"top\">kg/m3</td>
      <td valign=\"top\">derivative of density by mass fraction</td></tr>
  <tr><td valign=\"top\">Medium.molarMass(state)</td>
      <td valign=\"top\">kg/mol</td>
      <td valign=\"top\">molar mass</td></tr>
</table>
<p>
There are also some short forms provided for user convenience that allow the computation of certain
thermodynamic state variables without using the ThermodynamicState record explicitly. Those short forms
are for example useful to compute consistent start values in the initial equation section. Let's
consider the function temperature_phX(p,h,X) as an example. This function computes the temperature
from pressure, specific enthalpy, and composition X (or Xi) and is a short form for writing
</p>
<pre>
  temperature(setState_phX(p,h,X))
</pre>
<p>
The following functions are predefined in PartialMedium (other functions can be added in the actual
medium implementation package if they are useful)
</p>
<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\">Medium.specificEnthalpy_pTX(p,T,X)</td>
      <td valign=\"top\">J/kg</td>
      <td valign=\"top\">Specific enthalpy at p, T, X </td></tr>
  <tr><td valign=\"top\">Medium.temperature_phX(p,h,X)</td>
      <td valign=\"top\">K</td>
      <td valign=\"top\">Temperature at p, h, X</td></tr>
  <tr><td valign=\"top\">Medium.density_phX(p,h,X)</td>
      <td valign=\"top\">kg/m3</td>
      <td valign=\"top\">Density at p, h, X</td></tr>
  <tr><td valign=\"top\">Medium.temperature_psX(p,s,X)</td>
      <td valign=\"top\">K</td>
      <td valign=\"top\">Temperature at p, s, X</td></tr>
  <tr><td valign=\"top\">Medium.specificEnthalpy_psX(p,s,X)</td>
      <td valign=\"top\">J/(kg.K)</td>
      <td valign=\"top\">Specific entropy at p, s, X</td></tr>
</table>
<p>
Assume for example that the dynamic viscosity eta is needed in
the pressure drop equation of a short pipe. Then, the
model of a short pipe has to be changed to:
</p>
<pre>
  <b>model</b> ShortPipe
      ...
    Medium.BaseProperties medium_a \"Medium properties in port_a\";
    Medium.BaseProperties medium_b \"Medium properties in port_b\";
      ...
    Medium.DynamicViscosity eta;
      ...
    eta = <b>if</b> port_a.m_flow &gt; 0 <b>then</b>
               Medium.dynamicViscosity(medium_a.state)
          <b>else</b>
               Medium.dynamicViscosity(medium_b.state);
    // use eta in the pressure drop equation: port_a.m_flow = f(dp, eta)
  <b>end</b> ShortPipe;
</pre>

<p>
Note, \"Medium.DynamicViscosity\" is a type defined in Modelica.Interfaces.PartialMedium
as
</p>

<pre>
  <b>import</b> SI = Modelica.SIunits;
  <b>type</b> DynamicViscosity = SI.DynamicViscosity (
                                     min=0,
                                     max=1.e8,
                                     nominal=1.e-3,
                                     start=1.e-3);
</pre>

<p>
Every medium model may modify the attributes, to provide, e.g.,
min, max, nominal, and start values adapted to the medium.
Also, other types, such as AbsolutePressure, Density, MassFlowRate,
etc. are defined in PartialMedium. Whenever possible, these medium
specific types should be used in a model in order that medium information,
e.g., about nominal or start values, are automatically utilized.
</p>
</HTML>"));
        end OptionalProperties;

        class Constants "Constants"
          extends Modelica.Icons.Information;
          annotation(
            Documentation(info = "<HTML>
<p>
Every medium model provides the following <b>constants</b>. For example,
if a medium is declared as:
</p>
<pre>
   <b>replaceable package</b> Medium = Modelica.Media.Interfaces.PartialMedium;
</pre>
<p>
then constants \"Medium.mediumName\", \"Medium.nX\", etc. are defined:
</p>
<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"><b>Type</b></td>
      <td valign=\"top\"><b>Name</b></td>
      <td valign=\"top\"><b>Description</b></td></tr>
  <tr><td valign=\"top\">String</td><td valign=\"top\">mediumName</td>
      <td valign=\"top\">Unique name of the medium (is usually used to check whether
          the media in different components connected together
          are the same, by providing Medium.mediumName as quantity
          attribute of the mass flow rate in the connector)</td></tr>
  <tr><td valign=\"top\">String</td><td valign=\"top\">substanceNames[nS]</td>
      <td valign=\"top\">Names of the substances that make up the medium.
          If only one substance is present, substanceNames = {mediumName}.</td></tr>
  <tr><td valign=\"top\">String</td><td valign=\"top\">extraPropertiesNames[nC]</td>
      <td valign=\"top\">Names of the extra transported substances, outside of mass and
          energy balances.</td></tr>
  <tr><td valign=\"top\">Boolean</td><td valign=\"top\">singleState</td>
      <td valign=\"top\">= <b>true</b>, if u and d are not a function of pressure, and thus only
          a function of a single thermal variable (temperature or enthalpy) and
          of Xi for a multiple substance medium. Usually, this flag is
          <b>true</b> for incompressible media. It is used in a model to determine
          whether 1+nXi (singleState=<b>true</b>) or 2+nXi (singleState=<b>false</b>)
          initial conditions have to be provided for a volume element that
          contains mass and energy balance.</td></tr>
  <tr><td valign=\"top\">AbsolutePressure</td><td valign=\"top\">reference_p</td>
      <td valign=\"top\">Reference pressure for the medium</td></tr>
  <tr><td valign=\"top\">MassFraction</td><td valign=\"top\">reference_X[nX]</td>
      <td valign=\"top\">Reference composition for the medium</td></tr>
  <tr><td valign=\"top\">AbsolutePressure</td><td valign=\"top\">p_default</td>
      <td valign=\"top\">Default value for pressure of medium (for initialization)</td></tr>
  <tr><td valign=\"top\">Temperature</td><td valign=\"top\">T_default</td>
      <td valign=\"top\">Default value for temperature of medium (for initialization)</td></tr>
  <tr><td valign=\"top\">SpecificEnthalpy</td><td valign=\"top\">h_default</td>
      <td valign=\"top\">Default value for specific enthalpy of medium (for initialization)</td></tr>
  <tr><td valign=\"top\">MassFraction</td><td valign=\"top\">X_default[nX]</td>
      <td valign=\"top\">Default value for mass fractions of medium (for initialization)</td></tr>
  <tr><td valign=\"top\">Integer</td><td valign=\"top\">nS</td>
      <td valign=\"top\">number of substances contained in the medium.</td></tr>
  <tr><td valign=\"top\">Integer</td><td valign=\"top\">nX</td>
      <td valign=\"top\">Size of the full mass fraction vector X nX=nS.</td></tr>
  <tr><td valign=\"top\">Integer</td><td valign=\"top\">nXi</td>
      <td valign=\"top\">Number of independent mass fractions. If there is a single substance,
          then nXi = 0. </td></tr>
  <tr><td valign=\"top\">Boolean</td><td valign=\"top\">reducedX</td>
      <td valign=\"top\">= <b>true</b>, if the medium has a single substance, or if the medium model
          has multiple substances and contains the equation sum(X) = 1.
          In both cases, nXi = nS - 1 (unless fixedX = true).<br>
          = <b>false</b>, if the medium has multiple substances and does not contain the
          equation sum(X)=1, i.e., nXi = nX = nS (unless fixedX = true).
       </td></tr>
  <tr><td valign=\"top\">Boolean</td><td valign=\"top\">fixedX</td>
      <td valign=\"top\">= <b>false</b>: the composition of the medium can vary, and is
          determined by nXi independent mass fractions (see reducedX above).<br>
          = <b>true</b>: the composition of the medium is always reference_X,
          and nXi = 0.</td></tr>
  <tr><td valign=\"top\">FluidConstants</td><td valign=\"top\">fluidConstants[nS]</td>
      <td valign=\"top\">Critical, triple, molecular and other
          standard data that are provided for
          every substance of a medium.</td></tr>
</table>

<p>
The record FluidConstants that is defined in PartialMedium contains the following elements
</p>

<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"><b>Type</b></td>
      <td valign=\"top\"><b>Name</b></td>
      <td valign=\"top\"><b>Description</b></td></tr>

  <tr><td valign=\"top\">String</td>
      <td valign=\"top\">iupacName</td>
      <td valign=\"top\">complete IUPAC name</td></tr>

  <tr><td valign=\"top\">String</td>
      <td valign=\"top\">casRegistryNumber</td>
      <td valign=\"top\">chemical abstracts sequencing number</td></tr>

  <tr><td valign=\"top\">String</td>
      <td valign=\"top\">chemicalFormula</td>
      <td valign=\"top\">Chemical formula, (brutto, nomenclature according to Hill)</td></tr>

  <tr><td valign=\"top\">String</td>
      <td valign=\"top\">structureFormula</td>
      <td valign=\"top\">Chemical structure formula</td></tr>

  <tr><td valign=\"top\">MolarMass</td>
      <td valign=\"top\">molarMass</td>
      <td valign=\"top\">molar mass</td></tr>
</table>

<p> This record is extended in the partial packages further down the hierarchy (such as
PartialTwoPhaseMedium or PartialMixtureMedium) and may contain some or all of the following
elements</p>

<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\">Temperature</td>
      <td valign=\"top\">criticalTemperature</td>
      <td valign=\"top\">critical temperature</td></tr>

  <tr><td valign=\"top\">AbsolutePressure</td>
      <td valign=\"top\">criticalPressure</td>
      <td valign=\"top\">critical pressure</td></tr>

  <tr><td valign=\"top\">MolarVolume</td>
      <td valign=\"top\">criticalMolarVolume</td>
      <td valign=\"top\">critical molar Volume</td></tr>

  <tr><td valign=\"top\">Real</td>
      <td valign=\"top\">acentricFactor</td>
      <td valign=\"top\">Pitzer acentric factor</td></tr>

  <tr><td valign=\"top\">Temperature</td>
      <td valign=\"top\">triplePointTemperature</td>
      <td valign=\"top\">triple point temperature</td></tr>

  <tr><td valign=\"top\">AbsolutePressure</td>
      <td valign=\"top\">triplePointPressure</td>
      <td valign=\"top\">triple point pressure</td></tr>

  <tr><td valign=\"top\">Temperature</td>
      <td valign=\"top\">meltingPoint</td>
      <td valign=\"top\">melting point at 101325 Pa</td></tr>

  <tr><td valign=\"top\">Temperature</td>
      <td valign=\"top\">normalBoilingPoint</td>
      <td valign=\"top\">normal boiling point (at 101325 Pa)</td></tr>

  <tr><td valign=\"top\">DipoleMoment</td>
      <td valign=\"top\">dipoleMoment</td>
      <td valign=\"top\">dipole moment of molecule in Debye (1 debye = 3.33564e10-30 C.m)</td></tr>

  <tr><td valign=\"top\">Boolean</td>
      <td valign=\"top\">hasIdealGasHeatCapacity</td>
      <td valign=\"top\">true if ideal gas heat capacity is available</td></tr>

  <tr><td valign=\"top\">Boolean</td>
      <td valign=\"top\">hasCriticalData</td>
      <td valign=\"top\">true if critical data are known</td></tr>

  <tr><td valign=\"top\">Boolean</td>
      <td valign=\"top\">hasDipoleMoment</td>
      <td valign=\"top\">true if a dipole moment known</td></tr>

  <tr><td valign=\"top\">Boolean</td>
      <td valign=\"top\">hasFundamentalEquation</td>
      <td valign=\"top\">true if a fundamental equation</td></tr>

  <tr><td valign=\"top\">Boolean</td>
      <td valign=\"top\">hasLiquidHeatCapacity</td>
      <td valign=\"top\">true if liquid heat capacity is available</td></tr>

  <tr><td valign=\"top\">Boolean</td>
      <td valign=\"top\">hasSolidHeatCapacity</td>
      <td valign=\"top\">true if solid heat capacity is available</td></tr>

  <tr><td valign=\"top\">Boolean</td>
      <td valign=\"top\">hasAccurateViscosityData</td>
      <td valign=\"top\">true if accurate data for a viscosity function is available</td></tr>

  <tr><td valign=\"top\">Boolean</td>
      <td valign=\"top\">hasAccurateConductivityData</td>
      <td valign=\"top\">true if accurate data for thermal conductivity is available</td></tr>

  <tr><td valign=\"top\">Boolean</td>
      <td valign=\"top\">hasVapourPressureCurve</td>
      <td valign=\"top\">true if vapour pressure data, e.g., Antoine coefficients are known</td></tr>

  <tr><td valign=\"top\">Boolean</td>
      <td valign=\"top\">hasAcentricFactor</td>
      <td valign=\"top\">true if Pitzer accentric factor is known</td></tr>

  <tr><td valign=\"top\">SpecificEnthalpy</td>
      <td valign=\"top\">HCRIT0</td>
      <td valign=\"top\">Critical specific enthalpy of the fundamental equation</td></tr>

  <tr><td valign=\"top\">SpecificEntropy</td>
      <td valign=\"top\">SCRIT0</td>
      <td valign=\"top\">Critical specific entropy of the fundamental equation</td></tr>

  <tr><td valign=\"top\">SpecificEnthalpy</td>
      <td valign=\"top\">deltah</td>
      <td valign=\"top\">Difference between specific enthalpy model
          (h_m) and f.eq. (h_f) (h_m - h_f)</td></tr>

  <tr><td valign=\"top\">SpecificEntropy</td>
      <td valign=\"top\">deltas</td>
      <td valign=\"top\">Difference between specific enthalpy model (s_m) and f.eq.
          (s_f) (s_m - s_f)</td></tr>

</table>

</html>"));
        end Constants;

        class TwoPhase "Two-phase media"
          extends Modelica.Icons.Information;
          annotation(
            Documentation(info = "<HTML>
<p>
Models for media which can exist in one-phase or two-phase conditions inherit
from <a href=\"modelica://Modelica.Media.Interfaces.PartialTwoPhaseMedium\">
Modelica.Media.Interfaces.PartialTwoPhaseMedium</a>
(which inherits from PartialMedium). The basic usage of these
media models is the same as described in the previous sections. However, additional
functionalities are provided, which apply only to potentially two-phase media.
</p>
<p>
The following additional medium <b>constants</b> are provided:
</p>
<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"><b>Type</b></td>
      <td valign=\"top\"><b>Name</b></td>
      <td valign=\"top\"><b>Description</b></td></tr>
  <tr><td valign=\"top\">Boolean</td>
      <td valign=\"top\">smoothModel</td>
      <td valign=\"top\">If this flag is false (default value), then events are triggered
          whenever the saturation boundary is crossed; otherwise, no events
      are generated.</td></tr>
  <tr><td valign=\"top\">Boolean</td>
      <td valign=\"top\">onePhase</td>
      <td valign=\"top\">If this flag is true, then the medium model assumes it will be never
          called in the two-phase region. This can be useful to speed up
      the computations in a two-phase medium, when the user is sure it will
      always work in the one-phase region. Default value: false.</td></tr>
</table>
<p>
The setState_ph(), setState_ps(), setState_dT() and setState_pT() functions have
one extra input, named <i>phase</i>. If the phase input is not specified, or if
it is given a value of zero, then the setState function will determine the phase,
based on the other input values. An input phase = 1 will force the setState
function to return a state vector corresponding to a one-phase state, while
phase = 2 will force the setState value to return a state vector corresponding
to a two-phase state, as shown in the following example;
</p>
<pre>
   <b>replaceable package</b> Medium = Modelica.Media.Interfaces.PartialTwoPhaseMedium;
   Medium.ThermodynamicState state, state1, state2;
 <b>equation</b>
   // Set the state, given the pressure and the specific enthalpy
   // the phase is determined by the (p, h) values, and can be retrieved
   // from the state record
   state = Medium.setState_ph(p, h);
   phase = state1.phase;

   // Force the computation of the state with one-phase
   // equations of state, irrespective of the (p, h) values
   state1 = Medium.setState_ph(p, h, 1);

   // Force the computation of the state with 2-phase
   // equations of state, irrespective of the (p, h) values
   state2 = Medium.setState_ph(p, h, 2);
</pre>
<p>
This feature can be used for the following purposes:
</p>
<ul>
<li> saving computational time, if one knows in advance the phase of the medium;
<li> unambiguously determine the phase, when the two inputs correspond to a point on the saturation boundary (the derivative functions have substantially different values on either side);
<li> get the properties of metastable states, like superheated water or subcooled vapour.
</ul>
<p>
Many additional optional functions are defined to compute properties of
saturated media, either liquid (bubble point) or vapour (dew point).
The argument to such functions is a SaturationProperties record, which can be
set starting from either the saturation pressure or the saturation temperature,
as shown in the following example.
</p>
<pre>
   <b>replaceable package</b> Medium = Modelica.Media.Interfaces.PartialTwoPhaseMedium;
   Medium.SaturationProperties sat_p;
   Medium.SaturationProperties sat_T;
 <b>equation</b>
   // Set sat_p to saturation properties at pressure p
   sat_p = Medium.setSat_p(p);

   // Compute saturation properties at pressure p
   saturationTemperature_p = Medium.saturationTemperature_sat(sat_p);
   bubble_density_p =        Medium.bubbleDensity(sat_p);
   dew_enthalpy_p   =        Medium.dewEnthalpy(sat_p);

   // Set sat_T to saturation properties at temperature T
   sat_T = Medium.setSat_T(T);

   // Compute saturation properties at temperature T
   saturationTemperature_T = Medium.saturationPressure_sat(sat_T);
   bubble_density_T =        Medium.bubbleDensity(sat_T);
   dew_enthalpy_T =          Medium.dewEnthalpy(sat_T);
</pre>
<p>With reference to a model defining a pressure p, a temperature T, and a
SaturationProperties record sat, the following functions are provided:
</p>
<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"><b>Function call</b></td>
      <td valign=\"top\"><b>Unit</b></td>
      <td valign=\"top\"><b>Description</b></td></tr>
  <tr><td valign=\"top\">Medium.saturationPressure(T)</td>
      <td valign=\"top\">Pa</td>
      <td valign=\"top\">Saturation pressure at temperature T</td></tr>
  <tr><td valign=\"top\">Medium.saturationTemperature(p)</td>
      <td valign=\"top\">K</td>
      <td valign=\"top\">Saturation temperature at pressure p</td></tr>
  <tr><td valign=\"top\">Medium.saturationTemperature_derp(p)</td>
      <td valign=\"top\">K/Pa</td>
      <td valign=\"top\">Derivative of saturation temperature with respect to pressure</td></tr>
  <tr><td valign=\"top\">Medium.saturationTemperature_sat(sat)</td>
      <td valign=\"top\">K</td>
      <td valign=\"top\">Saturation temperature</td></tr>
  <tr><td valign=\"top\">Medium.saturationPressure_sat(sat)</td>
      <td valign=\"top\">Pa</td>
      <td valign=\"top\">Saturation pressure</td></tr>
  <tr><td valign=\"top\">Medium.bubbleEnthalpy(sat)</td>
      <td valign=\"top\">J/kg</td>
      <td valign=\"top\">Specific enthalpy at bubble point</td></tr>
  <tr><td valign=\"top\">Medium.dewEnthalpy(sat)</td>
      <td valign=\"top\">J/kg</td>
      <td valign=\"top\">Specific enthalpy at dew point</td></tr>
  <tr><td valign=\"top\">Medium.bubbleEntropy(sat)</td>
      <td valign=\"top\">J/(kg.K)</td>
      <td valign=\"top\">Specific entropy at bubble point</td></tr>
  <tr><td valign=\"top\">Medium.dewEntropy(sat)</td>
      <td valign=\"top\">J/(kg.K)</td>
      <td valign=\"top\">Specific entropy at dew point</td></tr>
  <tr><td valign=\"top\">Medium.bubbleDensity(sat)</td>
      <td valign=\"top\">kg/m3</td>
      <td valign=\"top\">Density at bubble point</td></tr>
  <tr><td valign=\"top\">Medium.dewDensity(sat)</td>
      <td valign=\"top\">kg/m3</td>
      <td valign=\"top\">Density at dew point</td></tr>
  <tr><td valign=\"top\">Medium.saturationTemperature_derp_sat(sat)</td>
      <td valign=\"top\">K/Pa</td>
      <td valign=\"top\">Derivative of saturation temperature with respect to pressure</td></tr>
  <tr><td valign=\"top\">Medium.dBubbleDensity_dPressure(sat)</td>
      <td valign=\"top\">kg/(m3.Pa)</td>
      <td valign=\"top\">Derivative of density at bubble point with respect to pressure</td></tr>
  <tr><td valign=\"top\">Medium.dDewDensity_dPressure(sat)</td>
      <td valign=\"top\">kg/(m3.Pa)</td>
      <td valign=\"top\">Derivative of density at dew point with respect to pressure</td></tr>
  <tr><td valign=\"top\">Medium.dBubbleEnthalpy_dPressure(sat)</td>
      <td valign=\"top\">J/(kg.Pa)</td>
      <td valign=\"top\">Derivative of specific enthalpy at bubble point with respect to pressure</td></tr>
  <tr><td valign=\"top\">Medium.dDewEnthalpy_dPressure(sat)</td>
      <td valign=\"top\">J/(kg.Pa)</td>
      <td valign=\"top\">Derivative of specific enthalpy at dew point with respect to pressure</td></tr>
  <tr><td valign=\"top\">Medium.surfaceTension(sat)</td>
      <td valign=\"top\">N/m</td>
      <td valign=\"top\">Surface tension between liquid and vapour phase</td></tr>
</table>
<p>
Sometimes it can be necessary to compute fluid properties in the thermodynamic
plane, just inside or outside the saturation dome. In this case, it is possible
to obtain an instance of a ThermodynamicState state vector, and then use it
to call the additional functions already defined for one-phase media.
</p>
<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"><b>Function call</b></td>
      <td valign=\"top\"><b>Description</b></td></tr>
  <tr><td valign=\"top\">Medium.setBubbleState(sat, phase)</td>
      <td valign=\"top\">Obtain the thermodynamic state vector
          corresponding to the bubble point. If phase==1 (default), the state is
      on the one-phase side; if phase==2, the state is on the two-phase
      side </td></tr>
  <tr><td valign=\"top\">Medium.setDewState(sat, phase)</td>
      <td valign=\"top\">Obtain the thermodynamic state vector
          corresponding to the dew point. If phase==1 (default), the state is
      on the one-phase side; if phase==2, the state is on the two-phase
      side </td></tr>
  </table>
<p>
Here are some examples:
</p>
<pre>
   <b>replaceable package</b> Medium = Modelica.Media.Interfaces.PartialTwoPhaseMedium;
   Medium.SaturationProperties sat;
   Medium.ThermodynamicState   dew_1;    // dew point, one-phase side
   Medium.ThermodynamicState   bubble_2; // bubble point, two phase side
 <b>equation</b>
   // Set sat to saturation properties at pressure p
   sat = setSat_p(p);

   // Compute dew point properties, (default) one-phase side
   dew_1 = setDewState(sat);
   cpDew = Medium.specificHeatCapacityCp(dew_1);
   drho_dp_h_1 = Medium.density_derp_h(dew_1);

   // Compute bubble point properties, two-phase side
   bubble_2    = setBubbleState(sat, 2);
   drho_dp_h_2 = Medium.density_derp_h(bubble_2);
</pre>
</html>"));
        end TwoPhase;

        class Initialization "Initialization"
          extends Modelica.Icons.Information;
          annotation(
            Documentation(info = "<HTML>

<p>
When a medium model is used in a balance volume, differential
equations for the independent medium variables are present and
therefore initial conditions have to be provided.
The following possibilities exist:
</p>

<h4>Steady state initialization</h4>
<p>
Modelica has currently no language element to define
steady state initialization. In the Modelica simulation
environment Dymola, the option
</p>
<pre>
   Advanced.DefaultSteadyStateInitialization = <b>true</b>
</pre>
<p>
can be set before translation. Then, missing initial
conditions are provided by automatically setting appropriate
state derivatives to zero.
</p>

<h4>Explicit start values or initial equations</h4>
<p>
Explicit start values can be defined with the \"start\" and
\"fixed\" attributes. The number of independent variables nx
need to be known which can be deduced from the medium
constants (nx = nXi + <b>if</b> singleState <b>then</b> 1 <b>else</b> 2).
Then, start values or initial equations can be defined
for nx variables (= p, T, d, u, h, Xi) from Medium.BaseProperties,
e.g., in the form:
</p>
<pre>
     <b>replaceable</b> package Medium = Medium.Interfaces.PartialMedium;
     Medium.BaseProperties medium1 (p(start=1e5, fixed=<b>not</b> Medium.singleState),
                                    T(start=300, fixed=<b>true</b>));
     Medium.BaseProperties medium2;
  <b>initial equation</b>
     <b>if not</b> Medium.singleState <b>then</b>
        medium2.p = 1e5;
     <b>end if</b>;
     medium2.T = 300;
  <b>equation</b>
  </pre>
<p>
If initial conditions are not provided for the independent
medium variables, non-linear systems of equations may
occur to compute the initial values of the independent
medium variables from the provided initial conditions.
</p>

<h4>Guess values</h4>
<p>
If non-linear systems of equations occur during initialization,
e.g., in case of steady state initialization, guess values
for the iteration variables of the non-linear system of equations
have to be provided via the \"start\" attribute (and fixed=false).
Unfortunately, it is usually not known in
advance which variables are selected as iteration variables of
a non-linear system of equations. One of the following possibilities
exist:
</p>
<ul>
<li> Do not supply start values and hope that the medium specific
     types have meaningful start values, such as in \"Medium.AbsolutePressure\"
     </li>
<li> Supply start values on all variables of the BaseProperties model,
     i.e., on p, T, d, u, h, Xi.</li>
<li> Determine the iteration variables of the non-linear systems of
     equations and provide start values for these variables.
     In the Modelica simulation environment Dymola, the iteration
     variables can be determined by setting the command
     <code>Advanced.OutputModelicaCode = <b>true</b></code>
     and by inspection of the file \"dsmodel.mof\" that is generated
     when this option is set (search for \"nonlinear\").</li>
</ul>
</HTML>"));
        end Initialization;
        annotation(
          DocumentationClass = true,
          Documentation(info = "<HTML>
<p>
Content:
</p>
<ol>
<li><a href=\"modelica://Modelica.Media.UsersGuide.MediumUsage.BasicUsage\">Basic usage of medium model</a></li>
<li><a href=\"modelica://Modelica.Media.UsersGuide.MediumUsage.BalanceVolume\">Medium model for a balance volume</a></li>
<li><a href=\"modelica://Modelica.Media.UsersGuide.MediumUsage.ShortPipe\">Medium model for a pressure loss</a></li>
<li><a href=\"modelica://Modelica.Media.UsersGuide.MediumUsage.OptionalProperties\">Optional medium properties</a></li>
<li><a href=\"modelica://Modelica.Media.UsersGuide.MediumUsage.Constants\">Constants provided by medium model</a></li>
<li><a href=\"modelica://Modelica.Media.UsersGuide.MediumUsage.TwoPhase\">Two-phase media</a></li>
<li><a href=\"modelica://Modelica.Media.UsersGuide.MediumUsage.Initialization\">Initialization</a></li>
</ol>

<p>
A good demonstration how to use the media from Modelica.Media is
given in package Modelica.Media.Examples.Tests. Under
<a href=\"modelica://Modelica.Media.Examples.Tests.Components\">
Tests.Components</a> the most basic components of a Fluid library
are defined. Under Tests.MediaTestModels these basic components are used to test
all media models with some very simple piping networks.
</p>

</HTML>"));
      end MediumUsage;

      package MediumDefinition "Medium definition"
        extends Modelica.Icons.Information;

        class BasicStructure "Basic structure"
          extends Modelica.Icons.Information;
          annotation(
            Documentation(info = "<HTML>
<p>
A medium model of Modelica.Media is essentially a <b>package</b>
that contains the following definitions:
</p>
<ul>
<li>
Definition of <b>constants</b>, such as the medium name.</li>
<li>
A <b>model</b> in the package that contains the 3 basic
thermodynamic equations that relate the 5+nXi primary medium variables.</li>
<li><b>Optional functions</b> to compute medium properties that are
only needed in certain circumstances, such as dynamic viscosity. These optional
functions need not be provided by every medium model.</li>
<li>
<b>Type</b> definitions, which are adapted to the particular
medium. For example, a type <b>Temperature</b> is defined where the attributes
<b>min</b>
and <b>max</b> define the validity region of the medium, and a suitable default
start value is given. In a device model, it is advisable to use these type
definitions, e.g., for parameters, in order that medium limits are checked as
early as possible, and that iteration variables of non-linear
systems of equations get reasonable start values.</li>
</ul>
<p>
Note, although we use the term <b>medium model</b>, it
is actually a Modelica <b>package</b> that contains all the constants and
definitions required for a complete <b>medium model</b>. The basic interface to a
medium is defined by Modelica.Media.Interfaces.PartialMedium that has the
following structure:</p>
<pre>
<b>partial package</b> PartialMedium
  <b>import</b> SI = Modelica.SIunits;
  <b>constant</b> String           mediumName = \"\";
  <b>constant</b> String           substanceNames[:] = {mediumName};
  <b>constant</b> String           extraPropertiesNames[:] = fill(\"\",0);
  <b>constant</b> Boolean          singleState = <b>false</b>;
  <b>constant</b> Boolean          reducedX = <b>true</b>;
  <b>constant</b> Boolean          fixedX = <b>false</b>;
  <b>constant</b> AbsolutePressure reference_p = 101325;
  <b>constant</b> MassFraction     reference_X[nX]=fill(1/nX,nX);
  <b>constant</b> AbsolutePressure p_default = 101325;
  <b>constant</b> Temperature      T_default = Modelica.SIunits.Conversions.from_degC(20);
  <b>constant</b> SpecificEnthalpy h_default =
                            specificEnthalpy_pTX(p_default, T_default, X_default);
  <b>constant</b> MassFraction     X_default[nX]=reference_X;
  <b>final constant</b> Integer    nS  = size(substanceNames,1);
  <b>final constant</b> Integer    nX  = nS;
  <b>final constant</b> Integer    nXi = <b>if</b> fixedX <b>then</b> 0
                                  <b>else if</b> reducedX <b>or</b> nS == 1
                                  <b>then</b> nS-1 <b>else</b> nS;
  <b>final constant</b> Integer    nC  = size(extraPropertiesNames,1);
  <b>constant</b> FluidConstants[nS] fluidConstants;

  <b>replaceable record</b> BasePropertiesRecord
    AbsolutePressure p;
    Density d;
    Temperature T;
    SpecificEnthalpy h;
    SpecificInternalEnergy u;
    MassFraction[nX] X;
    MassFraction[nXi] Xi;
    SpecificHeatCapacity R;
    MolarMass MM;
  <b>end</b> BasePropertiesRecord;

  <b>replaceable partial model</b> BaseProperties
    <b>extends</b> BasePropertiesRecord;
    ThermodynamicState state;
    <b>parameter</b> Boolean preferredMediumStates=false;
    SI.Conversions.NonSIunits.Temperature_degC T_degC =
       Modelica.SIunits.Conversions.to_degC(T)
    SI.Conversions.NonSIunits.Pressure_bar p_bar =
       Modelica.SIunits.Conversions.to_bar(p)
  <b>equation</b>
    Xi = X[1:nXi];
    <b>if</b> nX > 1 <b>then</b>
       <b>if</b> fixedX <b>then</b>
          X = reference_X;
       <b>elseif</b> reducedX <b>then</b>
          X[nX] = 1 - sum(Xi);
       <b>end if</b>;
    <b>end if</b>;
    // equations such as
    //    d = d(p,T);
    //    u = u(p,T);
    //    h = u + p/d;
    //    state.p = p;
    //    state.T = T;
    // will go here in actual media implementations, but are not present
    // in the base class since the ThermodynamicState record is still empty
   <b>end</b> BaseProperties

  <b>replaceable record</b> ThermodynamicState
     // there are no \"standard\" thermodynamic variables in the base class
     // but they will be defined here in actual media extending PartialMedium
     // Example:
     //    AbsolutePressure p \"Absolute pressure of medium\";
     //    Temperature      T \"Temperature of medium\";
  <b>end</b> ThermodynamicState;

  // optional medium properties
  <b>replaceable partial function</b> dynamicViscosity
    <b>input</b>  ThermodynamicState state;
    <b>output</b> DynamicViscosity eta;
  <b>end</b> dynamicViscosity;

  // other optional functions

  // medium specific types
  <b>type</b> AbsolutePressure = SI.AbsolutePressure (
                               min     = 0,
                               max     = 1.e8,
                               nominal = 1.e5,
                               start   = 1.e5);
  <b>type</b> DynamicViscosity = ...;
  // other type definitions
<b>end</b> PartialMedium;
</pre>
<p>
We will discuss all parts of this package in the
following paragraphs. An actual medium model should extend from PartialMedium
and has to provide implementations of the various parts.
</p>

<p>
Some of the constants at the beginning of the package do not have a value yet
(this is valid in Modelica), but a value has to be provided when extending from
package PartialMedium. A given value can be modified until the model is
translated or the <b>final</b> prefix is set.
The reason to use constants instead of parameters in the model BaseProperties
is that some of these constants are used in a context where parameters
are not allowed. For example, in connector definitions the
number of independent mass fractions nXi is used as dimension
of a vector Xi. When defining the
connector, only <i>constants</i> in packages can be accessed, but not
<i>parameters</i> in a model, because a connector cannot contain an instance
of BaseProperties.
</p>

<p>The record BasePropertiesRecord contains the variables
primarily used in balance equations. Three equations for these variables have
to be provided by every medium in model BaseProperties, plus two equations
for the gas constant and the molar mass.
</p>

<p>Optional medium properties are defined by functions, such as the function
dynamicViscosity (see code Section above) to compute the dynamic viscosity.
The argument of those functions is the ThermodynamicState record, defined
in BaseProperties, which contains the minimum number of thermodynamic variables
needed as an input to compute all the optional properties.
This construction simplifies the usage
considerably as demonstrated in the following code fragment:
</p>

<pre>
  <b>replaceable package</b> Medium = Modelica.Media.Interfaces.PartialMedium;
  Medium.BaseProperties   medium;
  Medium.DynamicViscosity eta;
  ...
  U   = m*medium.u; //Internal energy
  eta = Medium.dynamicViscosity(medium.state);
</pre>

<p>Medium is the medium package that satisfies the
requirements of a PartialMedium (when using the model above, a value for
Medium has to be provided by a redeclaration). The medium component is an
instance of the model Medium.BaseProperties and contains the core medium
equations. Variables in this model can be accessed just by dot-notation, such
as medium.u or medium.T. If an optional medium variable has to be computed, the
corresponding function from the actual Medium package is called, such as
Medium.dynamicViscosity. The medium.state vector can be given as input argument
to this function, and its fields are kept consistent to those of BaseProperties
by suitable equations, contained in BaseProperties itself (see above).
</p>

<p>If a medium model does not provide implementations of all
optional functions and one of these functions is called in a model, an error
occurs during translation since the optional functions which have not been
redeclared have the <i>partial</i> attribute. For example, if function
dynamicViscosity is not provided in the medium model when it is used, only
simple pressure drop loss models without a reference to the viscosity can be
used and not the sophisticated ones.
</p>

<p>At the bottom of the PartialMedium package type declarations
are present, that are used in all other parts of the PartialMedium package and
that should be used in all models and connectors where a medium model is
accessed. The reason is that minimum, maximum, nominal, and start
values are defined and these values can be adapted to the particular medium at
hand. For example, the nominal value of AbsolutePressure is 10<sup>5</sup>
Pa. If a simple model of water steam is used that is only valid above 100 &deg;C,
then the minimum value in the Temperature type should be set to this value. The
minimum and maximum values are also important for parameters in order to get an
early message if data outside of the validity region is given. The nominal
attribute is important as a scaling value if the variable is used as a state in
a differential equation or as an iteration variable in a non-linear system of
equations. The start attribute can be very useful to provide a meaningful
default start or guess value if the variable is used, e.g., as iteration
variable in a non-linear system of equations. Note, that all these attributes
can be set specifically for a medium in the following way:
</p>

<pre>
<b>package</b> MyMedium
  <b>extends</b> Modelica.Media.Interfaces.PartialMedium(
     ...
     Temperature(min=373));
<b>end</b> MyMedium;
</pre>

<p>
The type PartialMedium.MassFlowRate is defined as
</p>

<pre>
<b>type</b> MassFlowRate = Modelica.SIunits.MassFlowRate
     (quantity = \"MassFlowRate.\" + mediumName);
</pre>

<p>Note that the constant mediumName, that has to be
defined in every medium model, is used in the quantity attribute. For example,
if mediumName = SimpleLiquidWater, then the quantity attribute has the value
MassFlowRate.SimpleLiquidWater. This type should be used in a connector
definition of a fluid library:
</p>

<pre>
<b>connector</b> FluidPort
  <b>replaceable package</b> Medium = Modelica.Media.Interfaces.PartialMedium;
  <b>flow</b> Medium.MassFlowRate m_flow;
  ...
<b>end</b> FluidPort;
</pre>

<p>In the model where this connector is used, the actual
Medium has to be defined. Connectors can only be connected together, if the
corresponding attributes are either not defined or have identical values. Since
mediumName is part of the quantity attribute of MassFlowRate, it is not
possible to connect connectors with different media models together. In Dymola
this is already checked when models are connected together in the diagram layer
of the graphical user interface.</p>
</html>"));
        end BasicStructure;

        class BasicDefinition "Basic definition"
          extends Modelica.Icons.Information;
          annotation(
            Documentation(info = "<HTML>
<p>
Let's now walk through the definition of a new medium model. Please refer to
<a href=\"modelica://Modelica.Media.Interfaces.TemplateMedium\">
Modelica.Media.Interfaces.TemplateMedium</a> to obtain a template of the new
medium model code. For the moment being, consider a single-substance medium
model.
</p>
<p>
The new medium model is obtained by extending Modelica.Media.Interfaces.PartialMedium, and
setting the following package constants:
</p>
<ul>
<li>mediumName is a String containing the name of the medium.
<li>substanceNames is a vector of strings containing the names of the substances
    that make up the medium. In this case, it will contain only mediumName.</li>
<li>singleState can be set to true if u and d in BaseProperties do not depend
    on pressure. In other words, density does not depend on pressure
    (incompressible fluid), and it is assumed that also u does not depend on
    pressure. This setting can be useful for fluids having high density and
    low compressibility (e.g., liquids at moderate pressure); fast states
    resulting from the low compressibility effects are automatically avoided.</li>
<li>reducedX = true for single-substance media, which do not need mass
    fractions at all.</li>
</ul>
<p>
It is also possible to change the default min, max, nominal, and start
attributes of Medium-defined types (see TemplateMedium).</p>
<p>
All other package constants, such as nX, nXi, nS, are automatically set
by the declarations of the base package Interfaces.PartialMedium.</p>
<p>
The second step is to provide an implementation to the BaseProperties model,
partially defined in the base class Interfaces.PartialMedium. In the case of
single-substance media, two independent state variables must be selected among
p, T, d, u, h, and three equations must be written to provide the values of
the remaining variables. Two equations must then be added to compute the molar
mass MM and the gas constant R.</p>
<p>
The third step is to consider the optional functions that are going to be
implemented, among the partial functions defined by the base class PartialMedium.
A minimal set of state variables that could be provided as an input to
<i>all</i> those functions must be selected, and included in the redeclaration
of the ThermodynamicState record. Subsequently, equations must be added to
BaseProperties in order that the instance of that record inside BaseProperties
(named \"state\") is kept updated. For example, assume that all additional
properties can be computed as a function of p and T. Then, ThermodynamicState
should be redeclared as follows:</p>
<pre>
  <b>redeclare replaceable record</b> ThermodynamicState
    AbsolutePressure p \"Absolute pressure of medium\";
    Temperature T \"Temperature of medium\";
  <b>end</b> ThermodynamicState;
</pre>
<p>
and the following equations should be added to BaseProperties:
</p>
<pre>
  state.p = p;
  state.T = T;
</pre>
<p>
The additional functions can now be implemented by redeclaring the functions
defined in the base class and adding their algorithms, e.g.:
</p>
<pre>
    <b>redeclare function extends</b> dynamicViscosity \"Return dynamic viscosity\"
    <b>algorithm</b>
      eta := 10 - state.T*0.3 + state.p*0.2;
    <b>end</b> dynamicViscosity;
</pre>
</html>"));
        end BasicDefinition;

        class MultipleSubstances "Multiple Substances"
          extends Modelica.Icons.Information;
          annotation(
            Documentation(info = "<HTML>
<p>
When writing the model of a multiple-substance medium, a fundamental issue
concerns how to consider the mass fractions of the fluid. If there are nS
substances, there are also nS mass fractions; however, one of them is redundant,
as sum(X) = 1. Therefore there are basically two options, concerning the number
of independent mass fractions nXi:
</p>
<ul>
<li> <i>Reduced-state models</i>: reducedX = <b>true</b> and nXi = nS - 1. In this
case, the number of independent mass fractions nXi is the minimum possible.
The full state vector X is provided by equations declared in the base class
Interfaces.PartialMedium.BaseProperties: the first nXi elements are equal to
Xi, and the last one is 1 - sum(Xi).</li>
<li> <i>Full-state models</i>: reducedX = <b>false</b> and nXi = nS. In this case,
Xi = X, i.e., all the elements of the composition vector are considered as
independent variables, and the constraint sum(X) = 1 is never written explicitly.
Although this kind of model is heavier, as it provides one extra state variable,
it can be less prone to numerical and/or symbolic problems, which can be
caused by that constraint.</li>
<li> <i>Fixed-composition models</i>: fixedX = <b>true</b> and nXi = 0. In this case X = reference_X, i.e., all the elements of the composition vector are fixed.</li>
</ul>
<p> The medium implementer can declare the value reducedX as <b>final</b>. In
this way only one implementation must be given. For instance,
Modelica.Media.IdealGases models declare <b>final</b> reducedX = <b>false</b>, so that the
implementation can always assume nXi = nX. The same is true for Air.MoistAir,
which declares <b>final</b> reducedX = <b>true</b>, and always assumes nXi = nX - 1 = 1.</p>
<p>It is also possible to leave reducedX modifiable. In this case, the
BaseProperties model and all additional functions should check for the actual
value of reducedX, and provide the corresponding implementation.</p>

<p>If fixedX is left modifiable, then the implementation should also handle the
case fixedX = true properly.</p>
<p>Fluid connectors should always use composition vectors of size Xi, such as
in the Modelica_Fluid library:</p>
<pre>
<b>connector</b> FluidPort
  <b>replaceable package</b> Medium = Modelica.Media.Interfaces.PartialMedium;
  Medium.AbsolutePressure      p;
  <b>flow</b> Medium.MassFlowRate     m_flow;

  Medium.SpecificEnthalpy      h;
  <b>flow</b> Medium.EnthalpyFlowRate H_flow;

  Medium.MassFraction          Xi    [Medium.nXi];
  <b>flow</b> Medium.MassFlowRate     mX_flow[Medium.nXi];
<b>end</b> FluidPort;
</pre>
<p>
For further details, refer to the implementation of
<a href=\"modelica://Modelica.Media.IdealGases.Common.MixtureGasNasa\">
      MixtureGasNasa model</a> and
<a href=\"modelica://Modelica.Media.Air.MoistAir\">
      MoistAir model</a>.
</p>
</html>"));
        end MultipleSubstances;

        class SpecificEnthalpyAsFunction "Specific enthalpy as function"
          extends Modelica.Icons.Information;
          annotation(
            Documentation(info = "<html>
<p>
If pressure p and specific enthalpy h are <b>not</b> used as
independent medium variables, the specific enthalpy should
be computed by a Modelica function that has as input arguments
only the independent medium variables. It should <b>not</b> be
computed by an equation. For example, if p and T are used
as independent medium variables, a function h_pT(p,T) should
be defined that is called to compute h:
</p>

<pre>
    h = h_pT(p,T);
</pre>

<p>
The reason for this rule requires a longer explanation.
In short, if h is not a computed by a Modelica function and
this function is non-linear in the independent medium variables,
then non-linear systems of equations will occur at
every connection point, if the FluidPort connectors from the
Modelica_Fluid library are used (these are the same as in
Modelica.Media.Examples.Tests.Components.FluidPort).
Only, if the above rule is fulfilled, a tool is able to
remove these non-linear system of equations in most cases.
</p>

<p>
The basic idea of the FluidPort connector is that 2 or more
components can be connected together at a point and that
automatically the mass and energy balance is fulfilled
in the connection point, i.e., the ideal mixing equations
are generated. Note, the momentum balance is only correct for
straight line connections. If \"ideal mixing\" is not sufficient,
a special component to define the mixing equations must be introduced.
</p>

<p>
The mass and momentum balance equations in a component are
derived from the partial differential equations along the
flow direction of a pipe:
</p>

<p>
<IMG src=\"modelica://Modelica/Resources/Images/Media/UsersGuide/BalanceEquations1.png\">
</p>

<p>
Note, F<sub>F</sub> is the fanning friction factor.
The energy balance can be given in different forms.
Usually, it is given as:
</p>

<p>
<IMG src=\"modelica://Modelica/Resources/Images/Media/UsersGuide/EnergyBalance1.png\">
</p>

<p>
This form describes the change of the internal energy, kinetic
energy and potential energy of a volume as function of the
in and out flowing fluid. Multiplying the momentum balance
with the flow velocity v and subtracting it from the energy
balance above, results in the following alternative form
of the energy balance:
</p>

<p>
<IMG src=\"modelica://Modelica/Resources/Images/Media/UsersGuide/EnergyBalance2.png\">
</p>

<p>
This form has the advantage that the kinetic and potential
energy is no longer part of the energy balance and therefore
the energy balance is substantially simpler (e.g., additional non-linear
systems of equations occur in the first form since the velocity
is present in the energy balance; in the second form this is not
the case and it is still valid also for high speeds).
</p>

<p>
Assume now that the second form of the energy balance above
is used in all components and that the following FluidPort connector
is used in all components:
</p>

<pre>
<b>connector</b> FluidPort
  <b>replaceable package</b> Medium = Modelica.Media.Interfaces.PartialMedium;
  Medium.AbsolutePressure      p;
  <b>flow</b> Medium.MassFlowRate     m_flow;

  Medium.SpecificEnthalpy      h;
  <b>flow</b> Medium.EnthalpyFlowRate H_flow;

  Medium.MassFraction          Xi    [Medium.nXi];
  <b>flow</b> Medium.MassFlowRate     mX_flow[Medium.nXi];
<b>end</b> FluidPort;
</pre>

<p>
As an example, assume that 3 components
are connected together and that the medium is a single substance
fluid. This will result in the following
connection equations:
</p>

<pre>
    p1=p2=p3;
    h1=h2=h3;
    0 = m_flow1 + m_flow2 + m_flow3;
    0 = H_flow1 + H_flow2 + H_flow3;
</pre>

<p>
These are the mass balance and the
energy balance (form 2) of an infinitesimal volume
in the connection point under the assumption that no
mass or energy is stored in this volume. In other words,
the connection equations are the equations that describe
ideal mixing. Under the assumption that the velocity
vectors of the 3 flows are identical (especially, they are
parallel), also the momentum balance is fulfilled:
</p>

<pre>
   0 = m_flow1*v1 + m_flow2*v2 + m_flow3*v3;
     = v*(m_flow1 + m_flow2 + m_flow3);
     = 0;
</pre>

<p>
With the above connector it is therefore possible to
connect components together in a nearly arbitrary fashion,
because every connection fulfills automatically the
balance equations. This approach has, however, one drawback:
If two components are connected together, then the medium
variables on both sides of the connector are identical.
However, due to the connector, only the two equations
</p>

<pre>
   p1 = p2;
   h1 = h2;
</pre>

<p>
are present. Assume, that p,T are the independent medium variables
and that the medium properties are computed at one side of the
connections. This means, the following equations are basically
present:
</p>

<pre>
    h1 = h(p1,T1);
    h2 = h(p2,T2);
    p1 = p2;
    h1 = h2;
</pre>

<p>
These equations can be solved in the following way:
</p>

<pre>
    h1 := h(p1,T1)
    p2 := p1;
    h2 := h1;
    0  := h2 - h(p2,T2);   // non-linear system of equations for T2
</pre>

<p>
This means that T2 is computed by solving a non-linear system
of equations. If h1 and h2 are provided as Modelica functions,
a Modelica translator, such as Dymola, can replace
this non-linear system of equations by the equation:
</p>

<pre>
   T2 := T1;
</pre>

<p>
because after alias substitution there are two function calls
</p>

<pre>
    h1 := h(p1,T1);
    h1 := h(p1,T2);
</pre>

<p>
Since the left hand side of the function call and the first
argument are the same, the second arguments T1 and T2 must also be
identical and therefore T2 := T1. This type of analysis seems
to be only possible, if the specific enthalpy is defined as a function
of the independent medium variables.
</p>

</html>"));
        end SpecificEnthalpyAsFunction;

        class StaticStateSelection "Static State Selection"
          extends Modelica.Icons.Information;
          annotation(
            Documentation(info = "<html>
<p>
Without pre-caution when implementing a medium model,
it is very easy that non-linear algebraic
systems of equations occur when using the medium model.
In this section it is explained how to avoid non-linear
systems of equations that result from unnecessary
dynamic state selections.
</p>
<p>
A medium model should be implemented in such a way that
a tool is able to select states of a medium in a balance volume
statically (during translation). This is only possible if the
medium equations are written in a specific way. Otherwise,
a tool has to dynamically select states during simulation.
Since medium equations are usually non-linear, this means that
non-linear algebraic systems of equations would occur in every
balance volume.
</p>
<p>
It is assumed that medium equations in a balance volume
are defined in the following way:
</p>
<pre>
    <b>package</b> Medium = Modelica.Media.Interfaces.PartialMedium;
    Medium.BaseProperties medium;
  <b>equation</b>
     // mass balance
       <b>der</b>(M)  = port_a.m_flow + port_b.m_flow;
       <b>der</b>(MX) = port_a_mX_flow + port_b_mX_flow;
             M = V*medium.d;
            MX = M*medium.X;

     // Energy balance
     U = M*medium.u;
     <b>der</b>(U) = port_a.H_flow+port_b.H_flow;
</pre>
<p>
<b>Single Substance Media</b>
</p>
<p>
A medium consisting of a single substance
has to define two of \"p,T,d,u,h\" with
stateSelect=StateSelect.prefer if BaseProperties.preferredMediumstates = <b>true</b>
and has to provide the other three variables as function of these
states. This results in:
</p>
<ul>
<li> static state selection (no dynamic choices).</li>
<li> a linear system of equations in the two
     state derivatives.</li>
</ul>
<p>
<b>Example for a single substance medium</b>
</p>
<p>
p, T are preferred states (i.e., StateSelect.prefer is set)
and there are three equations written in the form:
</p>
<pre>
   d = fd(p,T)
   u = fu(p,T)
   h = fh(p,T)
</pre>
<p>
Index reduction leads to the equations:
</p>
<pre>
   <b>der</b>(M) = V*<b>der</b>(d)
   <b>der</b>(U) = <b>der</b>(M)*u + M*<b>der</b>(u)
   <b>der</b>(d) = <b>der</b>(fd,p)*<b>der</b>(p) + <b>der</b>(fd,T)*<b>der</b>(T)
   <b>der</b>(u) = <b>der</b>(fu,p)*<b>der</b>(p) + <b>der</b>(fu,T)*<b>der</b>(T)
</pre>
<p>
Note, that <b>der</b>(y,x) is the partial derivative of y with respect to x
and that this operator will be introduced in a future version
of the Modelica language.
The above equations imply, that if p,T are provided from the
integrator as states, all functions, such as fd(p,T)
or <b>der</b>(fd,p) can be evaluated as function of the states.
The overall system results in a linear system
of equations in <b>der</b>(p) and <b>der</b>(T) after eliminating
<b>der</b>(M), <b>der</b>(U), <b>der</b>(d), <b>der</b>(u) via tearing.
</p>
<p>
<b>Counter Example for a single substance medium</b>
</p>
<p>
An ideal gas with one substance is written in the form
</p>
<pre>
  <b>redeclare model extends</b> BaseProperties(
     T(stateSelect=if preferredMediumStates then StateSelect.prefer else StateSelect.default),
     p(stateSelect=if preferredMediumStates then StateSelect.prefer else StateSelect.default)
  <b>equation</b>
     h = h(T);
     u = h - R*T;
     p = d*R*T;
      ...
  <b>end</b> BaseProperties;
</pre>
<p>
If p, T are preferred states, these equations are <b>not</b>
written in the recommended form, because d is not a
function of p and T. If p,T would be states, it would be
necessary to solve for the density:
</p>
<pre>
   d = p/(R*T)
</pre>
<p>
If T or R are zero, this results in a division by zero.
A tool does not know that R or T cannot become zero.
Therefore, a tool must assume that p, T <b>cannot</b> always be
selected as states and has to either use another static
state selection or use dynamic state selection. The only
other choice for static state selection is d,T, because
h,u,p are given as functions of d,T.
However, as potential states only variables appearing differentiated and variables
declared with StateSelect.prefer or StateSelect.always
are used. Since \"d\" does not appear differentiated and has
StateSelect.default, it cannot be selected as a state.
As a result, the tool has to select states dynamically
during simulation. Since the equations above are non-linear
and they are utilized in the dynamic state
selection, a non-linear system of equations is present
in every balance volume.
</p>
<p>
To summarize, for single substance ideal gas media there
are the following two possibilities to get static
state selection and linear systems of equations:
</p>
<ol>
<li> Use p,T as preferred states and write the equation
     for d in the form: d = p/(T*R)</li>
<li> Use d,T as preferred states and write the equation
     for p in the form: p = d*T*R</li>
</ol>
<p>
All other settings (other/no preferred states etc.) lead
to dynamic state selection and non-linear systems of
equations for a balance volume.
</p>
<p>
<b>Multiple Substance Media</b>
</p>
<p>
A medium consisting of multiple substance
has to define two of \"p,T,d,u,h\" as well
as the mass fractions Xi with
stateSelect=StateSelect.prefer (if BaseProperties.preferredMediumStates = <b>true</b>)
and has to provide
the other three variables as functions of these
states. Only then, static selection is possible
for a tool.
</p>
<p>
<b>Example for a multiple substance medium:</b>
</p>
<p>
p, T and Xi are defined as preferred states and
the equations are written in the form:
</p>
<pre>
   d = fp(p,T,Xi);
   u = fu(p,T,Xi);
   h = fh(p,T,Xi);
</pre>
<p>
Since the balance equations are written in the form:
</p>
<pre>
              M = V*medium.d;
            MXi = M*medium.Xi;
</pre>
<p>
The variables M and MXi appearing differentiated in the
balance equations are provided as functions of d and Xi
and since d is given as a function of p, T and Xi, it
is possible to compute M and MXi directly from the desired
states. This means that static state selection is possible.
</p>
</html>"));
        end StaticStateSelection;

        class TestOfMedium "Test of medium"
          extends Modelica.Icons.Information;
          annotation(
            Documentation(info = "<html>
<p>
After implementation of a new medium model, it should
be tested. A basic test is already provided with model
Modelica.Media.Examples.Tests.Components.PartialTestModel
which might be used in the following way:
</p>

<pre>
  <b>model</b> TestOfMyMedium
     <b>extends</b> Modelica.Media.Examples.Tests.Components.PartialTestModel(
              <b>redeclare package</b> Medium = MyMedium);
  <b>end</b> TestOfMyMedium;
</pre>

<p>
It might be necessary to adapt or change initial values
depending on the validity range of the medium.
The model above should translate and simulate.
If the medium model is written according to the
suggestions given in the previous sections (and the Modelica
translator has appropriate algorithms implemented),
there should be only static state selection everywhere
and no non-linear system of equations, provided h is an independent
medium variable or is only a function of T. If h is a function
of, say h=h(p,T), one non-linear system of equations occurs that
cannot be avoided.
</p>

<p>
The test model above can be used to test the most basic
properties. Of course, more tests should be performed.
</p>

</html>"));
        end TestOfMedium;
        annotation(
          DocumentationClass = true,
          Documentation(info = "<HTML>
<p>
If a new medium model shall be introduced, copy package
<a href=\"modelica://Modelica.Media.Interfaces.TemplateMedium\">
Modelica.Media.Interfaces.TemplateMedium</a> to the desired
location, remove the
\"partial\" keyword from the package and provide
the information that is requested in the comments of the
Modelica source.
A more detailed description for the different parts of the
TemplateMedium package is given here:
</p>
<ol>
<li> <a href=\"modelica://Modelica.Media.UsersGuide.MediumDefinition.BasicStructure\">
      Basic structure of medium interface</a></li>
<li> <a href=\"modelica://Modelica.Media.UsersGuide.MediumDefinition.BasicDefinition\">
      Basic definition of medium model</a></li>
<li> <a href=\"modelica://Modelica.Media.UsersGuide.MediumDefinition.MultipleSubstances\">
      Multiple Substances</a></li>
<li> <a href=\"modelica://Modelica.Media.UsersGuide.MediumDefinition.SpecificEnthalpyAsFunction\">
      Specific enthalpy as function</a></li>
<li> <a href=\"modelica://Modelica.Media.UsersGuide.MediumDefinition.StaticStateSelection\">
      Static State Selection</a></li>
<li> <a href=\"modelica://Modelica.Media.UsersGuide.MediumDefinition.TestOfMedium\">
      Test of medium model</a></li>
</ol>
</HTML>"));
      end MediumDefinition;

      class ReleaseNotes "Release notes"
        extends Modelica.Icons.ReleaseNotes;
        annotation(
          Documentation(info = "<HTML>
<h4>Version included in Modelica 3.0</h4>
<p> See top-level release notes for MSL.</p>
<h4>Version 1.0, 2005-03-01</h4>
<p>
Many improvements in the library, e.g., providing mixtures
of the ideal gases, table based media, test suite for all media, improved and
updated User's Guide.
</p>
<h4>Version 0.9, 2004-10-18</h4>
<ul>
<li> Changed the redeclaration/extends within packages from the
     experimental feature to the language keywords introduced
     in Modelica 2.1.</li>
<li> Re-introduced package \"Water.SaltWater\" in order to test
     substance mixtures (this medium model does not describe
     real mixing of water and salt). </li>
<li> Started to improve the documentation in
     Modelica.Media.UsersGuide.MediumDefinition.BasicStructure</li>
</ul>
<h4>Version 0.792, 2003-10-28</h4>
<p>
This is the first version made available for the public
for the Modelica'2003 conference (for evaluation).
</p>
</html>"));
      end ReleaseNotes;

      class Future "Future"
        extends Modelica.Icons.Information;
        annotation(
          Documentation(info = "<html>

<h4>Planned changes for version 3.1</h4>

<p>Several changes are planned for version 3.1 of Modelica.Media. In order to have an easy transition, the current version is moved
  to the ObsoleteModelica-package and current users can continue to use it for many years. The goal for the new version is to cover many more fluids, be easier to use for users and less for implementers that want to include their own models. A beta-version of the new media library will be available in the first quarter of 2009.</p>

<p>The main user-visible changes are:</p>
<ul>
<li>No distinction between pure fluids and mixtures on the interface level: pure fluids are mixtures with 1 component</li>
<li>Compressibility will in the future be a flag that can be set for those media where it makes sense. It will thus be possible to treat an ideal gas as incompressible, which is adequate for certain applications, and also use the same medium model for thermal-only applications, and thermal-hydraulic ones.</li>
<li>All functions will have derivatives to enable analytic Jacobians.</li>
<li>Mole-fraction based media for process applications</li>
</ul>
<p>For further information on the upcoming version, please contact the <a href=\"modelica://Modelica.Media.UsersGuide.Contact\">Author</a></p>

<h4>Forward compatibility</h4>
<p> In order to simplify transition to the upcoming version of
Modelica.Media for Modelica 3.1, a new enumeration has been
introduced:  <a href=\"modelica://Modelica.Media.Interfaces.Choices.IndependentVariables\">IndependentVariables</a> with the
entries ph, pT, pTX, phX and dTX. Users are advised to use this enumeration to determine the independent variables of a medium. If they are going to use the function based interface of the current Modelica.Media (and not use the BaseProperties from Modelica.Media), they should not have any trouble using the next version without any need for changes in their code.</p>

</html>"));
      end Future;

      class Contact "Contact"
        extends Modelica.Icons.Contact;
        annotation(
          Documentation(info = "<html>
<dl>
<dt><b>Main author and maintainer:</b>
<dd>Hubertus Tummescheit<br>
    Modelon AB<br>
    Ideon Science Park<br>
    SE-22730 Lund, Sweden<br>
    email: <A HREF=\"mailto:Hubertus.Tummescheit@Modelon.se\">Hubertus.Tummescheit@Modelon.se</A><br>
</dl>
<p><b>Acknowledgements:</b></p>
<p>
The development of this library has been a collaborative effort
and many have contributed:
</p>
<ul>
<li> The essential parts of the media models have been implemented
     in the ThermoFluid library by Hubertus Tummescheit with
     help from Jonas Eborn and Falko Jens Wagner. These media models
     have been converted to the Modelica.Media interface definition
     and have been improved by Hubertus Tummescheit. </li>
<li> The effort for the development of the Modelica.Media library has been
     organized by Martin Otter who also contributed to the design,
     implemented part of the generic models, contributed to the User's Guide
     and provided the generic test suite Modelica.Media.Examples.Tests.</li>
<li> The basic idea for the medium model interface based on packages
     is from Michael Tiller who also contributed to the design.</li>
<li> The first design of the medium model interface is from
     Hilding Elmqvist. The design and the implementation has been further
     improved at the Modelica design meetings in<br>
     Dearborn, Nov. 20-22, 2002<br>
     Dearborn, Sept. 2-4, 2003<br>
     Lund Jan. 28-30, 2004<br>
     Munich, May 26-28, 2004<br>
     Lund, Aug. 30-31, 2004<br>
     Dearborn, Nov. 15-17, 2004<br>
     Cremona Jan. 31 - Feb. 2, 2005.</li>
<li> Hans Olsson, Sven Erik Mattsson and Hilding Elmqvist developed
     symbolic transformation algorithms and implemented them in Dymola
     to improve the efficiency considerably (e.g., to avoid non-linear
        systems of equations).</li>
<li>Katrin Pr&ouml;&szlig; implemented the moist air model</li>
<li> R&uuml;diger Franke performed the first realistic tests of the Modelica.Media
        and Modelica_Fluid libraries and gave valuable feedback.</li>
<li> Francesco Casella has been the most relentless bug-hunter and tester of the
     water and ideal gas properties. He also contributed to the User's Guide.</li>
<li> John Batteh, Daniel Bouskela, Jonas Eborn, Andreas Idebrant, Charles Newman,
     Gerhart Schmitz, and the users of the ThermoFluid library provided
     many useful comments and feedback.</li>
</ul>
</html>"));
      end Contact;
      annotation(
        DocumentationClass = true,
        Documentation(info = "<HTML>
<p>
Library <b>Modelica.Media</b> is a <b>free</b> Modelica package providing
a standardized interface to fluid media models and specific
media models based on this interface.
A fluid medium model defines <b>algebraic</b> equations
for the intensive thermodynamic variables used in the <b>mass</b>
and <b>energy</b> balance of component models. Optionally, additional
medium properties can be computed such as dynamic viscosity or thermal
conductivity. Medium models are defined for <b>single</b> and
<b>multiple substance</b> fluids with <b>one</b> and
<b>multiple phases</b>.
</p>
<p>
A large part of the library provides specific medium models
that can be directly utilized. This library can be used in
all types of Modelica fluid libraries that may have different connectors
and design philosophies. It is particularly utilized
in the Modelica_Fluid library (the Modelica_Fluid library is currently
under development to provide 1D therm-fluid flow components for
single and multiple substance flow with one and multiple phases).
The Modelica.Media library has the following
main features:
</p>
<ul>
<li> Balance equations and media model equations
     are decoupled.
     This means that the used medium model does usually not have an
     influence on how the balance equations are formulated.
     For example, the same balance equations are used for media
     that use pressure and temperature, or pressure and specific
     enthalpy as independent variables, as well as for
     incompressible and compressible media models.
     A Modelica tool will have enough information to
     generate as efficient code as a traditional
     (coupled) definition. This feature is described in more
     detail in section
     <a href=\"modelica://Modelica.Media.UsersGuide.MediumDefinition.StaticStateSelection\">Static State Selection</a>.</li>
<li> Optional variables, such as dynamic viscosity, are only computed if
     needed in the corresponding component.</li>
<li> The independent variables of a medium model do not
     influence the definition of a fluid connector port.
     Especially, the media models are implemented in such a way
     that a connector may have the minimum number of independent
     medium variables in a connector and still get the same
     efficiency as if all medium variables are passed by the
     connector from one component to the next one (the latter
     approach has the restriction that a fluid port can only
     connect two components and not more). Note, the Modelica_Fluid
     library uses the first approach, i.e., having a set of
     independent medium variables in a connector.</li>
<li> The medium models are implemented with regards to
     efficient dynamic simulation. For example, two phase
     medium models trigger state events at phase boundaries
     (because the medium variables are not differentiable
     at this point).</li>
</ul>
<p>
This User's Guide has the following main parts:
</p>
<ul>
<li> <a href=\"modelica://Modelica.Media.UsersGuide.MediumUsage\">Medium usage</a>
     describes how to use a medium model from
     this library in a component model.</li>
<li> <a href=\"modelica://Modelica.Media.UsersGuide.MediumDefinition\">Medium definition</a>
     describes how a new fluid medium
     model has to be implemented.</li>
<li> <a href=\"modelica://Modelica.Media.UsersGuide.ReleaseNotes\">ReleaseNotes</a>
     summarizes the changes of the library releases.</li>
<li><a href=\"modelica://Modelica.Media.UsersGuide.Future\">Future</a>
    provides information about the upcoming version 3.1 of Modelica.Media and
    some changes to this version to enable forward compatibility.</li>
<li><a href=\"modelica://Modelica.Media.UsersGuide.Contact\">Contact</a>
    provides information about the authors of the library as well as
    acknowledgements.</li>
</ul>
</HTML>"));
    end UsersGuide;

    package Examples "Demonstrate usage of property models (currently: simple tests)"
      extends Modelica.Icons.ExamplesPackage;

      model SimpleLiquidWater "Example for Water.SimpleLiquidWater medium model"
        extends Modelica.Icons.Example;
        parameter SI.Volume V = 1 "Volume";
        parameter SI.EnthalpyFlowRate H_flow_ext = 1.e6 "Constant enthalpy flow rate into the volume";
        package Medium = Water.ConstantPropertyLiquidWater(SpecificEnthalpy(max = 1e6)) "Medium model";
        Medium.BaseProperties medium(T(start = 300, fixed = true));
        Medium.BaseProperties medium2;
        Medium.ThermodynamicState state;
        Real m_flow_ext2;
        Real der_p;
        Real der_T;
        SI.Mass m(start = 1.0);
        SI.InternalEnergy U;
        // Use type declarations from the Medium
        Medium.MassFlowRate m_flow_ext;
        Medium.DynamicViscosity eta = Medium.dynamicViscosity(state);
        Medium.SpecificHeatCapacity cv = Medium.specificHeatCapacityCv(state);
      equation
        medium.p = 1.0e5;
        m = medium.d * V;
        U = m * medium.u;
// Mass balance
        der(m) = m_flow_ext;
// Energy balance
        der(U) = H_flow_ext;
// Smooth state
        medium2.p = 1e5 * time / 10;
        medium2.T = 330;
        m_flow_ext2 = time - 30;
        state = Medium.setSmoothState(m_flow_ext2, medium.state, medium2.state, 10);
        der_p = der(state.p);
        der_T = der(state.T);
        annotation(
          experiment(StopTime = 100),
          Documentation(info = "<html>

</html>"));
      end SimpleLiquidWater;

      model IdealGasH2O "IdealGas H20 medium model"
        extends Modelica.Icons.Example;
        package Medium = IdealGases.SingleGases.H2O "Medium model";
        Medium.ThermodynamicState state "Thermodynamic state record";
        Medium.ThermodynamicState state2;
        Medium.SpecificHeatCapacity cp = Medium.specificHeatCapacityCp(state);
        Medium.SpecificHeatCapacity cv = Medium.specificHeatCapacityCv(state);
        Medium.IsentropicExponent k = Medium.isentropicExponent(state);
        Medium.SpecificEntropy s = Medium.specificEntropy(state);
        //  Medium.SpecificEntropy s2=Medium.specificEntropy(state2);
        Medium.VelocityOfSound a = Medium.velocityOfSound(state);
        Real beta = Medium.isobaricExpansionCoefficient(state);
        Real gamma = Medium.isothermalCompressibility(state);
        Medium.SpecificEnthalpy h_is = Medium.isentropicEnthalpyApproximation(2.0, state);
        Medium.ThermodynamicState smoothState;
        Real m_flow_ext;
        Real der_p;
        Real der_T;
      equation
        state.p = 100000.0;
        state.T = 200 + 1000 * time;
        state2.p = 2.0e5;
        state2.T = 500.0;
//  s2 = s;
// Smooth state
        m_flow_ext = time - 0.5;
        smoothState = Medium.setSmoothState(m_flow_ext, state, state2, 0.1);
        der_p = der(smoothState.p);
        der_T = der(smoothState.T);
        annotation(
          Documentation(info = "<html>
<p>An example for using ideal gas properties and how to compute
  isentropic enthalpy changes.
  The function that is implemented is approximate, but usually
   very good: the second medium record medium2
   is given to compare the approximation.
</p>
</html>"),
          experiment(StopTime = 1));
      end IdealGasH2O;

      model WaterIF97 "WaterIF97 medium model"
        extends Modelica.Icons.Example;
        package Medium = Water.StandardWater "Medium model";
        Medium.BaseProperties medium(p(start = 1.e5, fixed = true, stateSelect = StateSelect.prefer), h(start = 1.0e5, fixed = true, stateSelect = StateSelect.prefer), T(start = 275.0), d(start = 999.0));
        Modelica.SIunits.Volume V(start = 0.1, fixed = true);
        parameter Modelica.SIunits.VolumeFlowRate dV = 0.0 "Fixed time derivative of volume";
        parameter Medium.MassFlowRate m_flow_ext = 0 "Fixed mass flow rate into volume";
        parameter Medium.EnthalpyFlowRate H_flow_ext = 10000 "Fixed enthalpy flow rate into volume";
        Modelica.SIunits.Mass m "Mass of volume";
        Modelica.SIunits.InternalEnergy U "Internal energy of volume";
        Medium.ThermodynamicState state2;
        Medium.ThermodynamicState state;
        Real m_flow_ext2;
        Real der_p;
        Real der_T;
      equation
        der(V) = dV;
        m = medium.d * V;
        U = m * medium.u;
// Mass balance
        der(m) = m_flow_ext;
// Energy balance
        der(U) = H_flow_ext;
// smooth states
        m_flow_ext2 = time - 0.5;
        state2 = Medium.setState_pT(1e5 * (1 + time), 300 + 200 * time);
        state = Medium.setSmoothState(m_flow_ext2, medium.state, state2, 0.05);
        der_p = der(state.p);
        der_T = der(state.T);
        annotation(
          Documentation(info = "<html>

</html>"),
          experiment(StopTime = 1));
      end WaterIF97;

      model MixtureGases "Test gas mixtures"
        extends Modelica.Icons.Example;
        parameter Modelica.SIunits.Volume V = 1 "Fixed size of volume 1 and volume 2";
        parameter Modelica.SIunits.MassFlowRate m_flow_ext = 0.01 "Fixed mass flow rate in to volume 1 and in to volume 2";
        parameter Modelica.SIunits.EnthalpyFlowRate H_flow_ext = 5000 "Fixed enthalpy flow rate in to volume and in to volume 2";
        package Medium1 = Modelica.Media.IdealGases.MixtureGases.CombustionAir "Medium model";
        Medium1.BaseProperties medium1(p(start = 1.e5, fixed = true, stateSelect = StateSelect.prefer), T(start = 300, fixed = true, stateSelect = StateSelect.prefer), X(start = {0.8, 0.2}));
        Real m1(quantity = Medium1.mediumName, start = 1.0);
        SI.InternalEnergy U1;
        Medium1.SpecificHeatCapacity cp1 = Medium1.specificHeatCapacityCp(medium1.state);
        Medium1.DynamicViscosity eta1 = Medium1.dynamicViscosity(medium1.state);
        Medium1.ThermalConductivity lambda1 = Medium1.thermalConductivity(medium1.state);
        package Medium2 = Modelica.Media.IdealGases.MixtureGases.SimpleNaturalGas "Medium model";
        Medium2.BaseProperties medium2(p(start = 1.e5, fixed = true, stateSelect = StateSelect.prefer), T(start = 300, fixed = true, stateSelect = StateSelect.prefer), X(start = {0.1, 0.1, 0.1, 0.2, 0.2, 0.3}));
        Real m2(quantity = Medium2.mediumName, start = 1.0);
        SI.InternalEnergy U2;
        Medium2.SpecificHeatCapacity cp2 = Medium2.specificHeatCapacityCp(medium2.state);
        Medium2.DynamicViscosity eta2 = Medium2.dynamicViscosity(medium2.state);
        Medium2.ThermalConductivity lambda2 = Medium2.thermalConductivity(medium2.state);
        Medium2.ThermodynamicState state2 = Medium2.setState_pTX(1.005e5, 302, {0.3, 0.2, 0.2, 0.1, 0.1, 0.1});
        Medium2.ThermodynamicState smoothState;
        Real m_flow_ext2;
        Real der_p;
        Real der_T;
      equation
        medium1.X = {0.8, 0.2};
        m1 = medium1.d * V;
        U1 = m1 * medium1.u;
        der(m1) = m_flow_ext;
        der(U1) = H_flow_ext;
        medium2.X = {0.1, 0.1, 0.1, 0.2, 0.2, 0.3};
        m2 = medium2.d * V;
        U2 = m2 * medium2.u;
        der(m2) = m_flow_ext;
        der(U2) = H_flow_ext;
// Smooth state
        m_flow_ext2 = time - 0.5;
        smoothState = Medium2.setSmoothState(m_flow_ext2, medium2.state, state2, 0.2);
        der_p = der(smoothState.p);
        der_T = der(smoothState.T);
        annotation(
          Documentation(info = "<html>

</html>"),
          experiment(StopTime = 1));
      end MixtureGases;

      model MoistAir "Ideal gas flue gas model"
        extends Modelica.Icons.Example;
        package Medium = Air.MoistAir "Medium model";
        Medium.BaseProperties medium(T(start = 274.0, fixed = true), X(start = {0.95, 0.05}), p(start = 1.0e5, fixed = true));
        //  Medium.SpecificEntropy s=Medium.specificEntropy(medium);
        //  Medium.SpecificEnthalpy h_is = Medium.isentropicEnthalpyApproximation(medium, 2.0e5);
        parameter Medium.MolarMass[2] MMx = {Medium.dryair.MM, Medium.steam.MM} "Vector of molar masses (consisting of dry air and of steam)";
        Medium.MolarMass MM = 1 / ((1 - medium.X[1]) / MMx[1] + medium.X[1] / MMx[2]) "Molar mass of gas part of mixture";
        //  Real[4] dddX=Medium.density_derX(medium,MM);
        Medium.ThermodynamicState state1;
        Medium.ThermodynamicState state2;
        Medium.ThermodynamicState smoothState;
        Real m_flow_ext;
        Real der_p;
        Real der_T;
      protected
        constant Modelica.SIunits.Time unitTime = 1;
      equation
        der(medium.p) = 0.0;
        der(medium.T) = 90;
        medium.X[Medium.Air] = 0.95;
//    medium.X[Medium.Water] = 0.05;
// one simple assumption only for quick testing:
//  medium.X_liquidWater = if medium.X_sat < medium.X[2] then medium.X[2] - medium.X_sat else 0.0;
// Smooth state
        m_flow_ext = time - 0.5;
        state1.p = 1.e5 * (1 + time);
        state1.T = 300 + 10 * time;
        state1.X = {time, 1 - time} / unitTime;
        state2.p = 1.e5 * (1 + time / 2);
        state2.T = 340 - 20 * time;
        state2.X = {0.5 * time, 1 - 0.5 * time} / unitTime;
        smoothState = Medium.setSmoothState(m_flow_ext, state1, state2, 0.2);
        der_p = der(smoothState.p);
        der_T = der(smoothState.T);
        annotation(
          Documentation(info = "<html>
<p>An example for using ideal gas properties and how to compute isentropic enthalpy changes.
The function that is implemented is approximate, but usually very good: the second medium record medium2
is given to compare the approximation.
</p>
</html>"),
          experiment(StopTime = 1.0, Tolerance = 1e-005));
      end MoistAir;

      model PsychrometricData "Produces plot data for psychrometric charts"
        extends Modelica.Icons.Example;
        package Medium = Modelica.Media.Air.MoistAir "Used medium package";
        parameter SIunits.Pressure p_const = 1e5 "Pressure";
        parameter Integer n_T = 11 "Number of isotherms";
        parameter SIunits.Temperature T_min = 253.15 "Lowest isotherm";
        parameter SIunits.Temperature T_step = 10 "Temperature step between two isotherms";
        parameter Integer n_h = 16 "Number of lines with constant specific enthalpy";
        parameter SIunits.SpecificEnthalpy h_min = -20e3 "Lowest line of constant enthalpy";
        parameter SIunits.SpecificEnthalpy h_step = 1e4 "Enthalpy step between two lines of constant enthalpy";
        parameter Integer n_phi = 10 "Number of lines with constant relative humidity";
        parameter Real phi_min = 0.1 "Lowest line of constant humidity";
        parameter Real phi_step = 0.1 "Step between two lines of constant humidity";
        parameter SIunits.MassFraction x_min = 0.00 "Minimum diagram absolute humidity";
        parameter SIunits.MassFraction x_max = 0.03 "Maximum diagram absolute humidity";
        parameter SIunits.Time t = 1 "Simulation time";
        final parameter SIunits.Temperature[n_T] T_const = {T_min - T_step + i * T_step for i in 1:n_T} "Constant temperatures";
        final parameter SIunits.SpecificEnthalpy[n_h] h_const = {(i - 1) * h_step + h_min for i in 1:n_h} "Constant enthalpies";
        final parameter Real[n_phi] phi_const = {(i - 1) * phi_step + phi_min for i in 1:n_phi} "Constant relative humidities";
        final parameter Real diagSlope = Medium.enthalpyOfVaporization(273.15) "Rotation of diagram that zero degrees isotherm becomes horizontal outside the fog region";
        final parameter SIunits.MassFraction x_start = x_min "Initial absolute humidity in kg water/kg dry air";
        SIunits.MassFraction x(start = x_start) "Absolute humidity in kg water/kg dry air";
        SIunits.SpecificEnthalpy[n_T] hx_T "h_1+x for const T";
        SIunits.SpecificEnthalpy[n_h] hx_h(start = h_const, each fixed = true) "Const h_1+x";
        SIunits.SpecificEnthalpy[n_phi] hx_phi "h_1+x for const phi";
        SIunits.SpecificEnthalpy[n_T] y_T "Chart enthalpy for const T";
        SIunits.SpecificEnthalpy[n_h] y_h "Chart enthalpy for const h";
        SIunits.SpecificEnthalpy[n_phi] y_phi "Chart enthalpy for const phi";
        Medium.BaseProperties[n_T] medium_T "Medium properties for const T";
        Medium.BaseProperties[n_phi] medium_phi "Medium properties for const phi";
      protected
        SIunits.Pressure[n_phi] ps_phi "Saturation pressure for constant-phi-lines";
        SIunits.Temperature[n_phi] T_phi(each start = 290);
        Boolean[n_T] fog(start = fill(false, n_T)) "Triggers events at intersection of isotherms with phi=1";
        SIunits.Pressure[n_T] pd "Steam partial pressure along isotherms";
      initial equation
        x = x_min;
      equation
        der(x) = (x_max - x_min) / t;
        for i in 1:n_T loop
          medium_T[i].T = T_const[i];
          medium_T[i].p = p_const;
          medium_T[i].Xi = {x / (1 + x)};
          hx_T[i] = medium_T[i].h * (medium_T[i].x_water + 1);
          y_T[i] = hx_T[i] - diagSlope * x;
//trigger events
          pd[i] = medium_T[i].Xi[1] * medium_T[i].MM / Medium.MMX[1] * p_const;
          fog[i] = pd[i] >= Medium.saturationPressure(T_const[i]);
        end for;
        for i in 1:n_h loop
          der(hx_h[i]) = 0.0;
          y_h[i] = hx_h[i] - diagSlope * x;
        end for;
        for i in 1:n_phi loop
          medium_phi[i].p = p_const;
          ps_phi[i] = p_const * x / phi_const[i] / (Medium.k_mair + x);
          T_phi[i] = if x < 5e-6 then 200 else Medium.saturationTemperature(ps_phi[i]);
          medium_phi[i].T = T_phi[i];
          medium_phi[i].Xi = {x / (1 + x)};
          hx_phi[i] = medium_phi[i].h * (medium_phi[i].x_water + 1);
          y_phi[i] = hx_phi[i] - diagSlope * x;
        end for;
        annotation(
          Documentation(info = "<html>
<p>This model produces psychrometric data from the moist air model in this library to be plotted in charts. The two most common chart varieties are the Mollier Diagram and the Psychrometric Chart. The first is widely used in some European countries while the second is more common in the Anglo-American world. Specific enthalpy is plotted over absolute humidity in the Mollier Diagram, it is the other way round in the Psychrometric Chart.<br>
It must be noted that the relationship of both axis variables is not right-angled, the absolute humidity follows a slope which equals the enthalpy of vaporization at 0 &deg;C. For better reading and in order to reduce the fog region the humidity axis is rotated to obtain a right-angled plot. Both charts usually contain additional information as isochores or auxiliary scales for e.g., heat ratios. Those information are omitted in this model and the charts below. Other important features of psychrometric chart data are that all mass specific variables (like absolute humidity, specific enthalpy etc.) are expressed in terms of kg dry air and that their baseline of 0 enthalpy is found at 0 &deg;C and zero humidity.</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Media/Air/Mollier.png\"><br>
<img src=\"modelica://Modelica/Resources/Images/Media/Air/PsycroChart.png\">
</p>

<p>
<b>Legend:</b> blue - constant specific enthalpy, red - constant temperature, black - constant relative humidity</p>

<p>The model provides data for lines of constant specific enthalpy, temperature and relative humidity in a Mollier Diagram or Psychrometric Chart as they were used for the figures above. For limitations and ranges of validity please refer to the <a href=\"modelica://Modelica.Media.Air.MoistAir\">MoistAir package description</a>. Absolute humidity <b>x</b> is increased with time in this model. The specific enthalpies adjusted for plotting are then obtained from:</p>
<ul>
<li><b>y_h</b>: constant specific enthalpy</li>
<li><b>y_T</b>: constant temperature</li>
<li><b>y_phi</b>: constant relative humidity</li>
</ul>
</html>"));
      end PsychrometricData;

      package TwoPhaseWater "Extension of the StandardWater package"
        extends Modelica.Media.Water.StandardWater;

        redeclare model extends BaseProperties "Make StandardWater.BaseProperties non replaceable in order that inheritance is possible in model ExtendedProperties"
        end BaseProperties;

        model ExtendedProperties "Plenty of two-phase properties"
          extends BaseProperties;
          ThermodynamicState dew "Dew line Properties";
          ThermodynamicState bubble "Bubble line Properties";
          ThermodynamicState bubble2 "Bubble line Properties, on the 2-phase side";
          DynamicViscosity eta "Viscosity (McAdams mixture rules if in 2-phase)";
          DynamicViscosity eta_d "Dew line viscosity";
          DynamicViscosity eta_b "Bubble line viscosity";
          ThermalConductivity lambda_d "Dew line thermal conductivity";
          ThermalConductivity lambda_b "Bubble line thermal conductivity";
          SpecificHeatCapacity cp_d "Dew line Specific heat capacity";
          SpecificHeatCapacity cp_b "Bubble line Specific heat capacity";
          Real ddhp;
          Real ddhp_d;
          Real ddhp_b "Derivatives";
          Real ddph;
          Real ddph_d;
          Real ddph_b "Derivatives";
          Real ddhp_b2;
          Real ddph_b2 "Derivatives";
          // no derivatives yet, ... sat should be temporary
          MassFraction x "Steam mass fraction";
          Real dTp;
          Real dTp2;
          SpecificEntropy s_b;
          SpecificEntropy s_d;
        equation
          eta = if phase == 1 then dynamicViscosity(state) else 1 / (x / eta_d + (1 - x) / eta_b);
          dew = setDewState(sat);
          bubble = setBubbleState(sat);
          bubble2 = setBubbleState(sat, 2);
          x = (h - bubble.h) / max(dew.h - bubble.h, 1e-6);
          eta_d = dynamicViscosity(dew);
          eta_b = dynamicViscosity(bubble);
          lambda_d = thermalConductivity(dew);
          lambda_b = thermalConductivity(bubble);
          cp_d = specificHeatCapacityCp(dew);
          cp_b = specificHeatCapacityCp(bubble);
          s_d = specificEntropy(dew);
          s_b = specificEntropy(bubble);
          ddph = density_derp_h(state);
          ddph_d = density_derp_h(dew);
          ddph_b = density_derp_h(bubble);
          ddhp = density_derh_p(state);
          ddhp_d = density_derh_p(dew);
          ddhp_b = density_derh_p(bubble);
          ddhp_b2 = density_derh_p(bubble2);
          ddph_b2 = density_derp_h(bubble2);
          dTp = saturationTemperature_derp(p);
          dTp2 = (1 / dew.d - 1 / bubble.d) / max(s_d - s_b, 1e-6);
          annotation(
            Documentation(info = ""));
        end ExtendedProperties;

        model TestTwoPhaseStates "Test the above model"
          extends Modelica.Icons.Example;
          ExtendedProperties medium(p(start = 2000.0, fixed = true), h(start = 8.0e5, fixed = true));
          parameter Real dh = 80000.0 "80 kJ/second";
          parameter Real dp = 1.0e6 "10 bars per second";
        equation
          der(medium.p) = dp;
          der(medium.h) = dh;
          annotation(
            experiment(StopTime = 22, Interval = 0.01),
            Documentation(info = "<html>
<p> For details see the documentation of the example package TwoPhaseWater</p>
</html>"));
        end TestTwoPhaseStates;
        annotation(
          Documentation(info = "<html>
<h4>Example: TwoPhaseWater</h4>
The TwoPhaseWater package demonstrates how to extend the parsimonious
BaseProperties with a minimal set of properties from the standard water
package with most properties that are needed in two-phase situations.
The model also demonstrates how to compute additional&nbsp; properties
for the medium model. In this scenario, that builds a new medium model
with many more properties than the default, the standard BaseProperties
is used as a basis. For additional properties, a user has to:<br>
<ol>
<li>Declare a new variable of the wanted type, e.g., \"<span style=\"color: rgb(51, 51, 255);\">DynamicViscosity
eta</span>\".</li>
<li>Compute that variable by calling the function form the package,
e.g., <span style=\"color: rgb(51, 51, 255);\">eta =
dynamicViscosity(state)</span>. Note that the instance of
ThermodynamicState is used as an input to the function. This instance
\"state\" is declared in PartialMedium and thus available in every medium
model. A user does not have to know what actual variables are required
to compute the dynamic viscosity, because the state instance is
guaranteed to contain what is needed.</li>
<li><span style=\"color: rgb(255, 0, 0);\">Attention</span>: Many
properties are not well defined in the two phase region and the
functions might return undesired values if called there. It is the
user's responsibility&nbsp; to take care of such situations. The example
uses one of several possible models to compute an averaged viscosity
for two-phase flows. </li>
</ol>
In two phase models, properties are often needed on the phase boundary
just outside the two phase dome, right on the border.. To compute the
thermodynamic state there, two auxiliary functions are provided: <b>setDewState(sat)</b> and <b>setBubbleState(sat)</b>. They take an
instance of SaturationProperties as input. By default they are in
one-phase, but with the optional phase argument set to 2, the output is
forced to be just inside the phase boundary. This is only needed when
derivatives like cv are computed with are different on both sides of
the boundaries. The usual steps to compute properties on the phase
boundary are: <br>
<ol>
<li>Declare an instance of ThermodynamicState, e.g., \"ThermodynamicState&nbsp; dew\".</li>
<li>Compute the state, using an instance of SaturationProperties,
e.g., dew = setDewState(sat)</li>
<li>Compute properties on the phase boundary to your full desire,
e.g., \"cp_d = specificHeatCapacityCp(dew)\". <br>
</li>
</ol>
<p>The sample model TestTwoPhaseStates test the extended properties</p>

<p>
The same procedure can be used to compute properties at other state
points, e.g., when an isentropic reference state is computed.
</p>
</html>"));
      end TwoPhaseWater;

      package ReferenceAir "Examples for detailed dry air and moist air medium models"
        extends Modelica.Icons.ExamplesPackage;

        model DryAir1 "Example 1 for dry air"
          extends Modelica.Icons.Example;
          extends Modelica.Media.Examples.Tests.Components.PartialTestModel(redeclare package Medium = Modelica.Media.Air.ReferenceAir.Air_pT);
          annotation(
            experiment(StopTime = 1.01));
        end DryAir1;

        model DryAir2 "Example 2 for dry air"
          extends Modelica.Icons.Example;
          extends Modelica.Media.Examples.Tests.Components.PartialTestModel2(redeclare package Medium = Modelica.Media.Air.ReferenceAir.Air_pT);
          annotation(
            experiment(StopTime = 1.01));
        end DryAir2;

        model MoistAir "Ideal gas flue gas model"
          extends Modelica.Icons.Example;
          parameter Modelica.SIunits.Temperature T_start = 274 "Initial guess value for temperature";
          parameter Modelica.SIunits.Pressure p_start = 1e5 "Initial guess value for pressure";
          package Medium = Modelica.Media.Air.ReferenceMoistAir "Medium model";
          Medium.BaseProperties medium(T(start = T_start, fixed = true), X(start = {0.95, 0.05}), p(start = p_start, fixed = true));
          parameter Medium.MolarMass[2] MMx = {Medium.dryair.MM, Medium.steam.MM} "Vector of molar masses (consisting of dry air and of steam)";
          Medium.MolarMass MM = 1 / ((1 - medium.X[1]) / MMx[1] + medium.X[1] / MMx[2]) "Molar mass of gas part of mixture";
          Medium.ThermodynamicState state1;
          Medium.ThermodynamicState state2;
          Medium.ThermodynamicState smoothState(T(start = T_start), p(start = p_start));
          Real m_flow_ext;
          Real der_p;
          Real der_T;
        protected
          constant Modelica.SIunits.Time unitTime = 1;
        equation
          der(medium.p) = 0.0;
          der(medium.T) = 90;
          medium.X[Medium.Air] = 0.95;
          m_flow_ext = time - 0.5;
          state1.p = 1.e5 * (1 + time);
          state1.T = 300 + 10 * time;
          state1.X = {time, 1 - time} / unitTime;
          state2.p = 1.e5 * (1 + time / 2);
          state2.T = 340 - 20 * time;
          state2.X = {0.5 * time, 1 - 0.5 * time} / unitTime;
          smoothState = Medium.setSmoothState(m_flow_ext, state1, state2, 0.2);
          der_p = der(smoothState.p);
          der_T = der(smoothState.T);
          annotation(
            experiment(StopTime = 1.0, Tolerance = 1e-005));
        end MoistAir;

        model MoistAir1 "Example 1 for moist air"
          extends Modelica.Icons.Example;
          extends Modelica.Media.Examples.Tests.Components.PartialTestModel(redeclare package Medium = Modelica.Media.Air.ReferenceMoistAir);
          annotation(
            experiment(StopTime = 1.01));
        end MoistAir1;

        model MoistAir2 "Example 2 for moist air"
          extends Modelica.Icons.Example;
          extends Modelica.Media.Examples.Tests.Components.PartialTestModel2(redeclare package Medium = Modelica.Media.Air.ReferenceMoistAir);
          annotation(
            experiment(StopTime = 1.01));
        end MoistAir2;

        model Inverse_sh_T "Solve h = h_pT(p, T), s = s_pT(p, T) for T, if h or s is given"
          extends Modelica.Icons.Example;
          import Medium = Modelica.Media.Air.ReferenceAir.Air_pT;
          parameter Modelica.SIunits.Temperature T_min = 300 "Vary temperature linearly from T_min (time=0) upto T_max (time=1)";
          parameter Modelica.SIunits.Temperature T_max = 500 "Vary temperature linearly from T_min (time=0) upto T_max (time=1)";
          parameter Modelica.SIunits.Pressure p = 1.0e5 "Fixed pressure in model";
          final parameter Modelica.SIunits.SpecificEnthalpy h_min = Medium.specificEnthalpy(Medium.setState_pT(p, T_min)) "Specific enthalpy at T_min";
          final parameter Modelica.SIunits.SpecificEnthalpy h_max = Medium.specificEnthalpy(Medium.setState_pT(p, T_max)) "Specific enthalpy at T_max";
          final parameter Modelica.SIunits.SpecificEntropy s_min = Medium.specificEntropy(Medium.setState_pT(p, T_min)) "Specific entropy at T_min";
          final parameter Modelica.SIunits.SpecificEntropy s_max = Medium.specificEntropy(Medium.setState_pT(p, T_max)) "Specific entropy at T_max";
          Modelica.SIunits.SpecificEnthalpy h1 "Pre-defined specific enthalpy";
          Modelica.SIunits.SpecificEnthalpy h2 "Specific enthalpy computed from T (= h1 required)";
          Modelica.SIunits.SpecificEntropy s1 "Pre-defined specific entropy";
          Modelica.SIunits.SpecificEntropy s2 "Specific entropy computed from T (= h1 required)";
          Modelica.SIunits.Temperature Th "Temperature computed from h1";
          Modelica.SIunits.Temperature Ts "Temperature computed from s1";
        protected
          constant Modelica.SIunits.Time timeUnit = 1.0;
        equation
// Define specific enthalpy and specific entropy
          h1 = if time < 0 then h_min else if time > 1 then h_max else h_min + time / timeUnit * (h_max - h_min);
          s1 = if time < 0 then s_min else if time > 1 then s_max else s_min + time / timeUnit * (s_max - s_min);
// Solve for temperature
          Th = Medium.temperature_phX(p, h1, fill(0.0, 0));
          Ts = Medium.temperature_psX(p, s1, fill(0.0, 0));
// Check (h2 must be identical to h1)
          h2 = Medium.specificEnthalpy_pTX(p, Th, fill(0.0, 0));
          s2 = Medium.specificEntropy_pTX(p, Ts, fill(0, 0));
          assert(abs(h1 - h2) < 1e-3, "Inverse for h not correctly computed");
          assert(abs(s1 - s2) < 1e-3, "Inverse for s not correctly computed");
          annotation(
            experiment(StopTime = 1));
        end Inverse_sh_T;

        model Inverse_sh_TX "Solve h = h_pTX(p, T, X), s = s_pTX(p, T, X) for T, if h or s is given"
          extends Modelica.Icons.Example;
          import Medium = Modelica.Media.Air.ReferenceMoistAir;
          parameter Modelica.SIunits.Temperature T_min = 300 "Vary temperature linearly from T_min (time=0) upto T_max (time=1)";
          parameter Modelica.SIunits.Temperature T_max = 500 "Vary temperature linearly from T_min (time=0) upto T_max (time=1)";
          parameter Modelica.SIunits.Pressure p = 1.0e5 "Fixed pressure in model";
          parameter Modelica.SIunits.MassFraction[:] X = Modelica.Media.Air.ReferenceMoistAir.reference_X "Mass fraction vector";
          final parameter Modelica.SIunits.SpecificEnthalpy h_min = Modelica.Media.Air.ReferenceMoistAir.specificEnthalpy(Modelica.Media.Air.ReferenceMoistAir.setState_pTX(p, T_min, X)) "Specific enthalpy at T_min";
          final parameter Modelica.SIunits.SpecificEnthalpy h_max = Modelica.Media.Air.ReferenceMoistAir.specificEnthalpy(Modelica.Media.Air.ReferenceMoistAir.setState_pTX(p, T_max, X)) "Specific enthalpy at T_max";
          final parameter Modelica.SIunits.SpecificEntropy s_min = Modelica.Media.Air.ReferenceMoistAir.specificEntropy(Modelica.Media.Air.ReferenceMoistAir.setState_pTX(p, T_min, X)) "Specific entropy at T_min";
          final parameter Modelica.SIunits.SpecificEntropy s_max = Modelica.Media.Air.ReferenceMoistAir.specificEntropy(Modelica.Media.Air.ReferenceMoistAir.setState_pTX(p, T_max, X)) "Specific entropy at T_max";
          Modelica.SIunits.SpecificEnthalpy h1 "Pre-defined specific enthalpy";
          Modelica.SIunits.SpecificEnthalpy h2 "Specific enthalpy computed from T (= h1 required)";
          Modelica.SIunits.SpecificEntropy s1 "Pre-defined specific entropy";
          Modelica.SIunits.SpecificEntropy s2 "Specific entropy computed from T (= h1 required)";
          Modelica.SIunits.Temperature Th "Temperature computed from h1";
          Modelica.SIunits.Temperature Ts "Temperature computed from s1";
        protected
          constant Modelica.SIunits.Time timeUnit = 1.0;
        equation
// Define specific enthalpy
          h1 = if time < 0 then h_min else if time > 1 then h_max else h_min + time / timeUnit * (h_max - h_min);
          s1 = if time < 0 then s_min else if time > 1 then s_max else s_min + time / timeUnit * (s_max - s_min);
// Solve for temperature
          Th = Medium.temperature_phX(p, h1, X);
          Ts = Medium.temperature_psX(p, s1, X);
// Check (h2 must be identical to h1)
          h2 = Medium.specificEnthalpy_pTX(p, Th, X);
          s2 = Medium.specificEntropy_pTX(p, Ts, X);
          assert(abs(h1 - h2) < 1e-3, "Inverse for h not correctly computed");
          assert(abs(s1 - s2) < 1e-3, "Inverse for s not correctly computed");
          annotation(
            experiment(StopTime = 1),
            Documentation(info = "<html>

</html>"));
        end Inverse_sh_TX;
      end ReferenceAir;

      package R134a "Examples for R134a"
        extends Modelica.Icons.ExamplesPackage;

        model R134a1 "Example 1 for R134a"
          extends Modelica.Icons.Example;
          extends Modelica.Media.Examples.Tests.Components.PartialTestModel(redeclare package Medium = Modelica.Media.R134a.R134a_ph, h_start = 107390, fixedMassFlowRate(use_T_ambient = false), volume(use_T_start = false), ambient(use_T_ambient = false));
          annotation(
            experiment(StopTime = 1.01));
        end R134a1;

        model R134a2 "Example 2 for R134a"
          extends Modelica.Icons.Example;
          extends Modelica.Media.Examples.Tests.Components.PartialTestModel2(redeclare package Medium = Modelica.Media.R134a.R134a_ph, h_start = 107390, fixedMassFlowRate(use_T_ambient = false), volume(use_T_start = false), ambient(use_T_ambient = false));
          annotation(
            experiment(StopTime = 1.01));
        end R134a2;
      end R134a;

      package TestOnly "Examples to show the testing of media"
        extends Modelica.Icons.ExamplesPackage;

        model MixIdealGasAir "Ideal gas air medium model"
          extends Modelica.Icons.Example;
          package Medium = IdealGases.MixtureGases.CombustionAir "Medium model";
          Medium.BaseProperties medium(T(start = 200.0, fixed = true), X(start = {0.2, 0.8}), p(start = 1.0e5, fixed = true));
          Medium.BaseProperties medium2(T(start = 300.0, fixed = true), X(start = {0.2, 0.8}, each fixed = true), p(start = 2.0e5, fixed = true));
          Medium.SpecificHeatCapacity cp = Medium.specificHeatCapacityCp(medium.state);
          Medium.SpecificHeatCapacity cv = Medium.specificHeatCapacityCv(medium.state);
          Medium.IsentropicExponent gamma = Medium.isentropicExponent(medium.state);
          Medium.SpecificEntropy s = Medium.specificEntropy(medium.state);
          Medium.SpecificEntropy s2 = Medium.specificEntropy(medium2.state);
          Medium.VelocityOfSound a = Medium.velocityOfSound(medium.state);
          Medium.DynamicViscosity eta = Medium.dynamicViscosity(medium.state);
          Medium.ThermalConductivity lambda = Medium.thermalConductivity(medium.state);
          Real beta = Medium.isobaricExpansionCoefficient(medium.state);
          Real gamma2 = Medium.isothermalCompressibility(medium2.state);
          Medium.SpecificEnthalpy h_is = Medium.isentropicEnthalpyApproximation(2.0e5, medium.state);
        equation
          der(medium.p) = 1000.0;
          der(medium.T) = 1000;
          medium.X = {0.2, 0.8};
          der(medium2.p) = 1.0e3;
          der(medium2.T) = 0.0;
          der(medium2.X) = {0.0, 0.0};
//  s2 = s;
          annotation(
            Documentation(info = "<html>
<p>An example for using ideal gas properties and how to compute isentropic enthalpy changes.
The function that is implemented is approximate, but usually very good: the second medium record medium2
is given to compare the approximation.
</p>
</html>"),
            experiment(StopTime = 1));
        end MixIdealGasAir;

        model FlueGas "Ideal gas flue gas model"
          extends Modelica.Icons.Example;
          package Medium = IdealGases.MixtureGases.FlueGasLambdaOnePlus "Medium model";
          Medium.ThermodynamicState state(T(start = 200.0, fixed = true), X(start = {0.2, 0.3, 0.4, 0.1}), p(start = 1.0e5, fixed = true));
          Medium.BaseProperties medium2(T(start = 300.0, fixed = true), X(start = {0.2, 0.1, 0.3, 0.4}, each fixed = true), p(start = 2.0e5, fixed = true));
          Medium.SpecificHeatCapacity cp = Medium.specificHeatCapacityCp(state);
          Medium.SpecificHeatCapacity cv = Medium.specificHeatCapacityCv(state);
          Medium.IsentropicExponent gamma = Medium.isentropicExponent(state);
          Medium.SpecificEntropy s = Medium.specificEntropy(state);
          Medium.SpecificEntropy s2 = Medium.specificEntropy(medium2.state);
          Medium.VelocityOfSound a = Medium.velocityOfSound(state);
          Real beta = Medium.isobaricExpansionCoefficient(state);
          Real gamma2 = Medium.isothermalCompressibility(medium2.state);
          Medium.SpecificEnthalpy h_is = Medium.isentropicEnthalpyApproximation(2.0e5, medium2.state);
          parameter Medium.MolarMass[4] MMx = Medium.data.MM "Molar masses of flue gas";
          Medium.MolarMass MM = 1 / sum(state.X[j] / MMx[j] for j in 1:4) "Molar mass";
          Real[4] dddX = Medium.density_derX(medium2.state);
        equation
          der(state.p) = 1000.0;
          der(state.T) = 1000;
          state.X = {0.2, 0.2, 0.4, 0.2};
          der(medium2.p) = 1.0e3;
          der(medium2.T) = 0.0;
          der(medium2.X[1:Medium.nX]) = {0.0, 0.0, 0.0, 0.0};
          annotation(
            Documentation(info = "<html>
<p>An example for using ideal gas properties and how to compute isentropic enthalpy changes.
The function that is implemented is approximate, but usually very good: the second medium record medium2
is given to compare the approximation.
</p>
</html>"),
            experiment(StopTime = 1));
        end FlueGas;

        package N2AsMix "Air and steam mixture (no condensation!, pseudo-mixture)"
          extends IdealGases.Common.MixtureGasNasa(mediumName = "Nitrogen", data = {IdealGases.Common.SingleGasesData.N2}, fluidConstants = {IdealGases.Common.FluidData.N2}, substanceNames = {"Nitrogen"}, reference_X = {1.0});
          annotation(
            Documentation(info = "<html>

</html>"));
        end N2AsMix;

        model IdealGasN2 "Test IdealGas.SingleMedia.N2 medium model"
          extends Modelica.Icons.Example;
          parameter Modelica.SIunits.Volume V = 1 "Size of fixed volume";
          parameter Medium.MassFlowRate m_flow_ext = 0.01 "Mass flow rate into volume";
          parameter Medium.EnthalpyFlowRate H_flow_ext = 5000 "Enthalpy flow rate into volume";
          package Medium = IdealGases.SingleGases.N2 "Medium model";
          // initType=Medium.Choices.Init.SteadyState,
          Medium.BaseProperties medium(preferredMediumStates = true, p(start = 1.e5, fixed = true), T(start = 300, fixed = true));
          Real m(quantity = Medium.mediumName, start = 1.0);
          SI.InternalEnergy U;
          Medium.SpecificHeatCapacity cp = Medium.specificHeatCapacityCp(medium.state);
          Medium.SpecificHeatCapacity cv = Medium.specificHeatCapacityCv(medium.state);
          Medium.IsentropicExponent gamma = Medium.isentropicExponent(medium.state);
          Medium.SpecificEntropy s = Medium.specificEntropy(medium.state);
          Medium.VelocityOfSound a = Medium.velocityOfSound(medium.state);
        equation
          m = medium.d * V;
          U = m * medium.u;
// Mass balance
          der(m) = m_flow_ext;
// Energy balance
          der(U) = H_flow_ext;
          annotation(
            Documentation(info = "<html>

</html>"),
            experiment(StopTime = 1));
        end IdealGasN2;

        //   package TestMedia "Test interfaces of media"
        //     extends Modelica.Icons.Package;
        //     model TemplateMedium "Test Interfaces.TemplateMedium"
        //       extends Modelica.Icons.Example;
        //       package Medium = Interfaces.TemplateMedium "Medium model";
        //       Medium.ThermodynamicState state;
        //
        //       Medium.DynamicViscosity eta=Medium.dynamicViscosity(state);
        //       Medium.ThermalConductivity lambda=Medium.thermalConductivity(state);
        //       Medium.SpecificEntropy s=Medium.specificEntropy(state);
        //       Medium.SpecificHeatCapacity cp=Medium.specificHeatCapacityCp(state);
        //       Medium.SpecificHeatCapacity cv=Medium.specificHeatCapacityCv(state);
        //       Medium.IsentropicExponent gamma=Medium.isentropicExponent(state);
        //       Medium.VelocityOfSound a=Medium.velocityOfSound(state);
        //     equation
        //       state.p = 1.0e5;
        //       state.T = 300 + time/1000;
        //       annotation (Documentation(info="<html>
        //
        // </html>"),
        //          experiment(StopTime=1.01));
        //     end TemplateMedium;
        //
        //     annotation (Documentation(info="<html>
        //
        // </html>"));
        //   end TestMedia;

        model IdealGasN2Mix "Test IdealGas.SingleMedia.N2 medium model"
          extends Modelica.Icons.Example;
          parameter Modelica.SIunits.Volume V = 1 "Size of volume";
          parameter Medium.MassFlowRate m_flow_ext = 0.01 "Mass flow rate flowing into volume";
          parameter Medium.EnthalpyFlowRate H_flow_ext = 5000 "Enthalpy flow rate flowing into volume";
          package Medium = N2AsMix "Medium model";
          // initType=Medium.Choices.Init.SteadyState,
          Medium.BaseProperties medium(preferredMediumStates = true, p(start = 1.e5, fixed = true), T(start = 300, fixed = true));
          Real m(quantity = Medium.mediumName, start = 1.0);
          SI.InternalEnergy U;
          Medium.SpecificHeatCapacity cp = Medium.specificHeatCapacityCp(medium.state);
          Medium.SpecificHeatCapacity cv = Medium.specificHeatCapacityCv(medium.state);
          Medium.IsentropicExponent gamma = Medium.isentropicExponent(medium.state);
          Medium.SpecificEntropy s = Medium.specificEntropy(medium.state);
          Medium.VelocityOfSound a = Medium.velocityOfSound(medium.state);
        equation
          m = medium.d * V;
          U = m * medium.u;
          medium.X = {1.0};
// Mass balance
          der(m) = m_flow_ext;
// Energy balance
          der(U) = H_flow_ext;
          annotation(
            Documentation(info = "<html>

</html>"),
            experiment(StopTime = 1));
        end IdealGasN2Mix;
        annotation(
          Documentation(info = "<html>

</html>"));
      end TestOnly;

      package Tests "Library to test that all media models simulate and fulfill the expected structural properties"
        extends Modelica.Icons.ExamplesPackage;

        package Components "Functions, connectors and models needed for the media model tests"
          extends Modelica.Icons.Library;

          connector FluidPort "Interface for quasi one-dimensional fluid flow in a piping network (incompressible or compressible, one or more phases, one or more substances)"
            // extends Modelica.Icons.ObsoleteModel;
            replaceable package Medium = Modelica.Media.Interfaces.PartialMedium "Medium model" annotation(
              choicesAllMatching = true);
            Medium.AbsolutePressure p "Pressure in the connection point";
            flow Medium.MassFlowRate m_flow "Mass flow rate from the connection point into the component";
            Medium.SpecificEnthalpy h "Specific mixture enthalpy in the connection point";
            flow Medium.EnthalpyFlowRate H_flow "Enthalpy flow rate into the component (if m_flow > 0, H_flow = m_flow*h)";
            Medium.MassFraction Xi[Medium.nXi] "Independent mixture mass fractions m_i/m in the connection point";
            flow Medium.MassFlowRate mXi_flow[Medium.nXi] "Mass flow rates of the independent substances from the connection point into the component (if m_flow > 0, mX_flow = m_flow*X)";
            Medium.ExtraProperty C[Medium.nC] "Properties c_i/m in the connection point";
            flow Medium.ExtraPropertyFlowRate mC_flow[Medium.nC] "Flow rates of auxiliary properties from the connection point into the component (if m_flow > 0, mC_flow = m_flow*C)";
            annotation(
              Documentation(info = "<html>

</html>"));
          end FluidPort;

          connector FluidPort_a "Fluid connector with filled icon"
            extends FluidPort;
            annotation(
              Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-100, 100}, {100, -100}}, lineColor = {0, 127, 255}, fillColor = {0, 127, 255}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-100, 100}, {100, -100}}, lineColor = {0, 0, 0}, fillColor = {0, 127, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-88, 206}, {112, 112}}, textString = "%name", lineColor = {0, 0, 255})}),
              Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-100, 100}, {100, -100}}, lineColor = {0, 127, 255}, fillColor = {0, 127, 255}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-100, 100}, {100, -100}}, lineColor = {0, 0, 0}, fillColor = {0, 127, 255}, fillPattern = FillPattern.Solid)}),
              Documentation(info = "<html>Modelica.Media.Examples.Tests.Components.FluidPort_a
</html>"));
          end FluidPort_a;

          connector FluidPort_b "Fluid connector with outlined icon"
            extends FluidPort;
            annotation(
              Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-100, 100}, {100, -100}}, lineColor = {0, 127, 255}, fillColor = {0, 127, 255}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-100, 100}, {100, -100}}, lineColor = {0, 0, 0}, fillColor = {0, 127, 255}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-80, 80}, {80, -80}}, lineColor = {0, 127, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-88, 192}, {112, 98}}, textString = "%name", lineColor = {0, 0, 255})}),
              Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-100, 100}, {100, -100}}, lineColor = {0, 127, 255}, fillColor = {0, 127, 255}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-100, 100}, {100, -100}}, lineColor = {0, 0, 0}, fillColor = {0, 127, 255}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-80, 80}, {80, -80}}, lineColor = {0, 127, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid)}),
              Documentation(info = "<html>

</html>"));
          end FluidPort_b;

          model PortVolume "Fixed volume associated with a port by the finite volume method"
            // extends Modelica.Icons.ObsoleteModel;
            replaceable package Medium = Modelica.Media.Interfaces.PartialMedium "Medium model" annotation(
              choicesAllMatching = true);
            parameter SI.Volume V = 1e-6 "Fixed size of junction volume";
            parameter Boolean use_p_start = true "Select p_start or d_start" annotation(
              Evaluate = true,
              Dialog(group = "Initial pressure or initial density"));
            parameter Medium.AbsolutePressure p_start = 101325 "Initial pressure" annotation(
              Dialog(group = "Initial pressure or initial density", enable = use_p_start));
            parameter Medium.Density d_start = 1 "Initial density" annotation(
              Dialog(group = "Initial pressure or initial density", enable = not use_p_start));
            parameter Boolean use_T_start = true "Select T_start or h_start" annotation(
              Evaluate = true,
              Dialog(group = "Initial temperature or initial specific enthalpy"));
            parameter Medium.Temperature T_start = Modelica.SIunits.Conversions.from_degC(20) "Initial temperature" annotation(
              Dialog(group = "Initial temperature or initial specific enthalpy", enable = use_T_start));
            parameter Medium.SpecificEnthalpy h_start = 1.e4 "Initial specific enthalpy" annotation(
              Dialog(group = "Initial temperature or initial specific enthalpy", enable = not use_T_start));
            parameter Medium.MassFraction X_start[Medium.nX] "Initial mass fractions m_i/m" annotation(
              Dialog(group = "Only for multi-substance flow", enable = Medium.nX > 0));
            FluidPort_a port(redeclare package Medium = Medium) annotation(
              Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0)));
            Medium.BaseProperties medium(preferredMediumStates = true);
            SI.Energy U "Internal energy of port volume";
            SI.Mass m "Mass of junction volume";
            SI.Mass mXi[Medium.nXi] "Independent substance masses of junction volume";
          initial equation
            if not Medium.singleState then
              if use_p_start then
                medium.p = p_start;
              else
                medium.d = d_start;
              end if;
            end if;
            if use_T_start then
              medium.T = T_start;
            else
              medium.h = h_start;
            end if;
            medium.Xi = X_start[1:Medium.nXi];
          equation
// Connect port to medium variables
            medium.p = port.p;
            medium.h = port.h;
            medium.Xi = port.Xi;
// Total quantities
            m = V * medium.d;
            mXi = m * medium.Xi;
            U = m * medium.u;
// Mass and energy balance
            der(m) = port.m_flow;
            der(mXi) = port.mXi_flow;
            der(U) = port.H_flow;
            annotation(
              Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-100, 100}, {100, -100}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.Sphere, fillColor = {170, 213, 255}), Text(extent = {{-144, 178}, {146, 116}}, textString = "%name", lineColor = {0, 0, 255}), Text(extent = {{-130, -108}, {144, -150}}, lineColor = {0, 0, 0}, textString = "V=%V")}),
              Documentation(info = "<html>
<p>
This component models the <b>volume</b> of <b>fixed size</b> that is
associated with the <b>fluid port</b> to which it is connected.
This means that all medium properties inside the volume, are identical
to the port medium properties. In particular, the specific enthalpy
inside the volume (= medium.h) is always identical to the specific enthalpy
in the port (port.h = medium.h). Usually, this model is used when
discretizing a component according to the finite volume method into
volumes in internal ports that only store energy and mass and into
transport elements that just transport energy, mass and momentum
between the internal ports without storing these quantities during the
transport.
</p>
</html>"));
          end PortVolume;

          model FixedMassFlowRate "Ideal pump that produces a constant mass flow rate from a large reservoir at fixed temperature and mass fraction"
            // extends Modelica.Icons.ObsoleteModel;
            parameter Medium.MassFlowRate m_flow "Fixed mass flow rate from an infinite reservoir to the fluid port";
            parameter Boolean use_T_ambient = true "Select T_ambient or h_ambient" annotation(
              Evaluate = true,
              Dialog(group = "Ambient temperature or ambient specific enthalpy"));
            parameter Medium.Temperature T_ambient = Modelica.SIunits.Conversions.from_degC(20) "Ambient temperature" annotation(
              Dialog(group = "Ambient temperature or ambient specific enthalpy", enable = use_T_ambient));
            parameter Medium.SpecificEnthalpy h_ambient = 1.e4 "Ambient specific enthalpy" annotation(
              Dialog(group = "Ambient temperature or ambient specific enthalpy", enable = not use_T_ambient));
            parameter Medium.MassFraction X_ambient[Medium.nX] "Ambient mass fractions m_i/m of reservoir";
            replaceable package Medium = Modelica.Media.Interfaces.PartialMedium "Medium model" annotation(
              choicesAllMatching = true);
            Medium.BaseProperties medium "Medium in the source";
            FluidPort_b port(redeclare package Medium = Medium) annotation(
              Placement(transformation(extent = {{100, -10}, {120, 10}}, rotation = 0)));
          equation
            if use_T_ambient then
              medium.T = T_ambient;
            else
              medium.h = h_ambient;
            end if;
            medium.Xi = X_ambient[1:Medium.nXi];
            medium.p = port.p;
            port.m_flow = -m_flow;
            port.mXi_flow = semiLinear(port.m_flow, port.Xi, medium.Xi);
            port.H_flow = semiLinear(port.m_flow, port.h, medium.h);
            annotation(
              Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{20, 60}, {100, -60}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192, 192, 192}), Rectangle(extent = {{38, 40}, {100, -40}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {0, 127, 255}), Ellipse(extent = {{-100, 80}, {60, -80}}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Polygon(points = {{-60, 70}, {60, 0}, {-60, -68}, {-60, 70}}, lineColor = {0, 0, 255}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-54, 32}, {16, -30}}, lineColor = {255, 0, 0}, textString = "m"), Text(extent = {{-142, 142}, {156, 88}}, textString = "%name", lineColor = {0, 0, 255}), Text(extent = {{-154, -88}, {150, -132}}, lineColor = {0, 0, 0}, textString = "%m_flow"), Ellipse(extent = {{-26, 30}, {-18, 22}}, lineColor = {255, 0, 0}, fillColor = {255, 0, 0}, fillPattern = FillPattern.Solid)}),
              Documentation(info = "<html>

</html>"));
          end FixedMassFlowRate;

          model FixedAmbient "Ambient pressure, temperature and mass fraction source"
            // extends Modelica.Icons.ObsoleteModel;
            replaceable package Medium = Modelica.Media.Interfaces.PartialMedium "Medium model" annotation(
              choicesAllMatching = true);
            parameter Boolean use_p_ambient = true "Select p_ambient or d_ambient" annotation(
              Evaluate = true,
              Dialog(group = "Ambient pressure or ambient density"));
            parameter Medium.AbsolutePressure p_ambient = 101325 "Ambient pressure" annotation(
              Dialog(group = "Ambient pressure or ambient density", enable = use_p_ambient));
            parameter Medium.Density d_ambient = 1 "Ambient density" annotation(
              Dialog(group = "Ambient pressure or ambient density", enable = not use_p_ambient));
            parameter Boolean use_T_ambient = true "Select T_ambient or h_ambient" annotation(
              Evaluate = true,
              Dialog(group = "Ambient temperature or ambient specific enthalpy"));
            parameter Medium.Temperature T_ambient = Modelica.SIunits.Conversions.from_degC(20) "Ambient temperature" annotation(
              Dialog(group = "Ambient temperature or ambient specific enthalpy", enable = use_T_ambient));
            parameter Medium.SpecificEnthalpy h_ambient = 1.e4 "Ambient specific enthalpy" annotation(
              Dialog(group = "Ambient temperature or ambient specific enthalpy", enable = not use_T_ambient));
            parameter Medium.MassFraction X_ambient[Medium.nX] "Ambient mass fractions m_i/m" annotation(
              Dialog(group = "Only for multi-substance flow", enable = Medium.nX > 0));
            Medium.BaseProperties medium "Medium in the source";
            FluidPort_b port(redeclare package Medium = Medium) annotation(
              Placement(transformation(extent = {{100, -10}, {120, 10}}, rotation = 0)));
          equation
            if use_p_ambient or Medium.singleState then
              medium.p = p_ambient;
            else
              medium.d = d_ambient;
            end if;
            if use_T_ambient then
              medium.T = T_ambient;
            else
              medium.h = h_ambient;
            end if;
            medium.Xi = X_ambient[1:Medium.nXi];
            port.p = medium.p;
            port.H_flow = semiLinear(port.m_flow, port.h, medium.h);
            port.mXi_flow = semiLinear(port.m_flow, port.Xi, medium.Xi);
            annotation(
              Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-100, 80}, {100, -80}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.Sphere, fillColor = {0, 127, 255}), Text(extent = {{-136, 144}, {132, 82}}, textString = "%name", lineColor = {0, 0, 255})}),
              Documentation(info = "<html>
<p>
Model <b>FixedAmbient_pt</b> defines constant values for ambient conditions:
</p>
<ul>
<li> Ambient pressure.</li>
<li> Ambient temperature.</li>
<li> Ambient mass fractions (only for multi-substance flow).</li>
</ul>
<p>
Note, that ambient temperature
and mass fractions have only an effect if the mass flow
is from the ambient into the port. If mass is flowing from
the port into the ambient, the ambient definitions,
with exception of ambient pressure, do not have an effect.
</p>
</html>"));
          end FixedAmbient;

          model ShortPipe "Simple pressure loss in pipe"
            // extends Modelica.Icons.ObsoleteModel;
            replaceable package Medium = Modelica.Media.Interfaces.PartialMedium "Medium model" annotation(
              choicesAllMatching = true);
            parameter Medium.AbsolutePressure dp_nominal(min = 1.e-10) "Nominal pressure drop";
            parameter Medium.MassFlowRate m_flow_nominal(min = 1.e-10) "Nominal mass flow rate at nominal pressure drop";
            FluidPort_a port_a(redeclare package Medium = Medium) annotation(
              Placement(transformation(extent = {{-120, -10}, {-100, 10}}, rotation = 0)));
            FluidPort_b port_b(redeclare package Medium = Medium) annotation(
              Placement(transformation(extent = {{120, -10}, {100, 10}}, rotation = 0)));
            // Medium.BaseProperties medium_a(p=port_a.p, h=port_a.h, Xi=port_a.Xi)
            //   "Medium properties in port_a";
            // Medium.BaseProperties medium_b(p=port_b.p, h=port_b.h, Xi=port_b.Xi)
            //   "Medium properties in port_b";
            Medium.MassFlowRate m_flow "Mass flow rate from port_a to port_b (m_flow > 0 is design flow direction)";
            Modelica.SIunits.Pressure dp "Pressure drop from port_a to port_b";
          equation
/* Handle reverse and zero flow */
            port_a.H_flow = semiLinear(port_a.m_flow, port_a.h, port_b.h);
            port_a.mXi_flow = semiLinear(port_a.m_flow, port_a.Xi, port_b.Xi);
/* Energy, mass and substance mass balance */
            port_a.H_flow + port_b.H_flow = 0;
            port_a.m_flow + port_b.m_flow = 0;
            port_a.mXi_flow + port_b.mXi_flow = zeros(Medium.nXi);
// Design direction of mass flow rate
            m_flow = port_a.m_flow;
// Pressure drop
            dp = port_a.p - port_b.p;
            m_flow = m_flow_nominal / dp_nominal * dp;
            annotation(
              Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, 60}, {100, -60}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192, 192, 192}), Rectangle(extent = {{-100, 34}, {100, -36}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {0, 127, 255}), Text(extent = {{-150, 140}, {150, 80}}, lineColor = {0, 0, 0}, textString = "%name"), Text(extent = {{-136, -62}, {122, -108}}, lineColor = {0, 0, 0}, textString = "k=%m_flow_nominal/%dp_nominal")}),
              Documentation(info = "<html>
<p>
Model <b>ShortPipe</b> defines a simple pipe model
with pressure loss due to friction. It is assumed that
no mass or energy is stored in the pipe.
</p>
</html>"));
          end ShortPipe;

          partial model PartialTestModel "Basic test model to test a medium"
            import SI = Modelica.SIunits;
            // extends Modelica.Icons.ObsoleteModel;
            replaceable package Medium = Modelica.Media.Interfaces.PartialMedium "Medium model" annotation(
              choicesAllMatching = true);
            parameter SI.AbsolutePressure p_start = Medium.p_default "Initial value of pressure";
            parameter SI.Temperature T_start = Medium.T_default "Initial value of temperature";
            parameter SI.SpecificEnthalpy h_start = Medium.h_default "Initial value of specific enthalpy";
            parameter Real X_start[Medium.nX] = Medium.X_default "Initial value of mass fractions";
            /*
              parameter SI.AbsolutePressure p_start = 1.0e5 "Initial value of pressure";
              parameter SI.Temperature T_start = 300 "Initial value of temperature";
              parameter SI.Density h_start = 1 "Initial value of specific enthalpy";
              parameter Real X_start[Medium.nX] = Medium.reference_X
                "Initial value of mass fractions";
            */
            PortVolume volume(redeclare package Medium = Medium, p_start = p_start, T_start = T_start, h_start = h_start, X_start = X_start, V = 0.1) annotation(
              Placement(transformation(extent = {{-40, 0}, {-20, 20}}, rotation = 0)));
            FixedMassFlowRate fixedMassFlowRate(redeclare package Medium = Medium, T_ambient = 1.2 * T_start, h_ambient = 1.2 * h_start, m_flow = 1, X_ambient = 0.5 * X_start) annotation(
              Placement(transformation(extent = {{-80, 0}, {-60, 20}}, rotation = 0)));
            FixedAmbient ambient(redeclare package Medium = Medium, T_ambient = T_start, h_ambient = h_start, X_ambient = X_start, p_ambient = p_start) annotation(
              Placement(transformation(extent = {{60, 0}, {40, 20}}, rotation = 0)));
            ShortPipe shortPipe(redeclare package Medium = Medium, m_flow_nominal = 1, dp_nominal = 0.1e5) annotation(
              Placement(transformation(extent = {{0, 0}, {20, 20}}, rotation = 0)));
          equation
            connect(fixedMassFlowRate.port, volume.port) annotation(
              Line(points = {{-59, 10}, {-30, 10}}, color = {0, 127, 255}));
            connect(volume.port, shortPipe.port_a) annotation(
              Line(points = {{-30, 10}, {-1, 10}}, color = {0, 127, 255}));
            connect(shortPipe.port_b, ambient.port) annotation(
              Line(points = {{21, 10}, {39, 10}}, color = {0, 127, 255}));
            annotation(
              Documentation(info = "<html>

</html>"));
          end PartialTestModel;

          partial model PartialTestModel2 "Slightly larger test model to test a medium"
            import SI = Modelica.SIunits;
            // extends Modelica.Icons.ObsoleteModel;
            replaceable package Medium = Modelica.Media.Interfaces.PartialMedium "Medium model" annotation(
              choicesAllMatching = true);
            parameter SI.AbsolutePressure p_start = 1.0e5 "Initial value of pressure";
            parameter SI.Temperature T_start = 300 "Initial value of temperature";
            parameter SI.SpecificEnthalpy h_start = 1 "Initial value of specific enthalpy";
            parameter Real X_start[Medium.nX] = Medium.reference_X "Initial value of mass fractions";
            PortVolume volume(redeclare package Medium = Medium, p_start = p_start, T_start = T_start, h_start = h_start, X_start = X_start, V = 0.1) annotation(
              Placement(transformation(extent = {{-60, 0}, {-40, 20}}, rotation = 0)));
            FixedMassFlowRate fixedMassFlowRate(redeclare package Medium = Medium, T_ambient = 1.2 * T_start, h_ambient = 1.2 * h_start, m_flow = 1, X_ambient = 0.5 * X_start) annotation(
              Placement(transformation(extent = {{-100, 0}, {-80, 20}}, rotation = 0)));
            FixedAmbient ambient(redeclare package Medium = Medium, T_ambient = T_start, h_ambient = h_start, X_ambient = X_start, p_ambient = p_start) annotation(
              Placement(transformation(extent = {{92, 0}, {72, 20}}, rotation = 0)));
            ShortPipe shortPipe(redeclare package Medium = Medium, m_flow_nominal = 1, dp_nominal = 0.1e5) annotation(
              Placement(transformation(extent = {{-30, 0}, {-10, 20}}, rotation = 0)));
            PortVolume volume1(redeclare package Medium = Medium, p_start = p_start, T_start = T_start, h_start = h_start, X_start = X_start, V = 0.1) annotation(
              Placement(transformation(extent = {{0, 0}, {20, 20}}, rotation = 0)));
            ShortPipe shortPipe1(redeclare package Medium = Medium, m_flow_nominal = 1, dp_nominal = 0.1e5) annotation(
              Placement(transformation(extent = {{36, 0}, {56, 20}}, rotation = 0)));
          equation
            connect(fixedMassFlowRate.port, volume.port) annotation(
              Line(points = {{-79, 10}, {-50, 10}}, color = {0, 127, 255}));
            connect(volume.port, shortPipe.port_a) annotation(
              Line(points = {{-50, 10}, {-31, 10}}, color = {0, 127, 255}));
            connect(volume1.port, shortPipe1.port_a) annotation(
              Line(points = {{10, 10}, {35, 10}}, color = {0, 127, 255}));
            connect(shortPipe.port_b, volume1.port) annotation(
              Line(points = {{-9, 10}, {10, 10}}, color = {0, 127, 255}));
            connect(shortPipe1.port_b, ambient.port) annotation(
              Line(points = {{57, 10}, {71, 10}}, color = {0, 127, 255}));
            annotation(
              Documentation(info = "<html>

</html>"));
          end PartialTestModel2;
          annotation(
            Documentation(info = "<html>

</html>"));
        end Components;

        package MediaTestModels "Test models to test all media"
          extends Modelica.Icons.ExamplesPackage;

          package Air "Test models of library Modelica.Media.Air"
            extends Modelica.Icons.ExamplesPackage;

            model SimpleAir "Test Modelica.Media.Air.SimpleAir"
              extends Modelica.Icons.Example;
              extends Modelica.Media.Examples.Tests.Components.PartialTestModel(redeclare package Medium = Modelica.Media.Air.SimpleAir);
              annotation(
                Documentation(info = "<html>

</html>"),
                experiment(StopTime = 1.01));
            end SimpleAir;

            model DryAirNasa "Test Modelica.Media.Air.DryAirNasa"
              extends Modelica.Icons.Example;
              extends Modelica.Media.Examples.Tests.Components.PartialTestModel(redeclare package Medium = Modelica.Media.Air.DryAirNasa);
              annotation(
                Documentation(info = "<html>

</html>"),
                experiment(StopTime = 1.01));
            end DryAirNasa;

            model MoistAir "Test Modelica.Media.Air.MoistAir"
              extends Modelica.Icons.Example;
              extends Modelica.Media.Examples.Tests.Components.PartialTestModel(redeclare package Medium = Modelica.Media.Air.MoistAir);
              annotation(
                Documentation(info = "<html>

</html>"),
                experiment(StopTime = 1.01));
            end MoistAir;
            annotation(
              Documentation(info = "<html>

</html>"));
          end Air;

          package IdealGases "Test models of library Modelica.Media.IdealGases"
            extends Modelica.Icons.ExamplesPackage;

            model Air "Test single gas Modelica.Media.IdealGases.SingleGases.Air"
              extends Modelica.Icons.Example;
              extends Modelica.Media.Examples.Tests.Components.PartialTestModel(redeclare package Medium = Modelica.Media.Air.DryAirNasa);
              annotation(
                Documentation(info = "<html>

</html>"),
                experiment(StopTime = 1.01));
            end Air;

            model Nitrogen "Test single gas Modelica.Media.IdealGases.SingleGases.N2"
              extends Modelica.Icons.Example;
              extends Modelica.Media.Examples.Tests.Components.PartialTestModel(redeclare package Medium = Modelica.Media.IdealGases.SingleGases.N2);
              annotation(
                Documentation(info = "<html>

</html>"),
                experiment(StopTime = 1.01));
            end Nitrogen;

            model SimpleNaturalGas "Test mixture gas Modelica.Media.IdealGases.MixtureGases.SimpleNaturalGas"
              extends Modelica.Icons.Example;
              extends Modelica.Media.Examples.Tests.Components.PartialTestModel(redeclare package Medium = Modelica.Media.IdealGases.MixtureGases.SimpleNaturalGas);
              annotation(
                Documentation(info = "<html>

</html>"),
                experiment(StopTime = 1.01));
            end SimpleNaturalGas;

            model SimpleNaturalGasFixedComposition "Test mixture gas Modelica.Media.IdealGases.MixtureGases.SimpleNaturalGas"
              extends Modelica.Icons.Example;
              extends Modelica.Media.Examples.Tests.Components.PartialTestModel(redeclare package Medium = Modelica.Media.IdealGases.MixtureGases.SimpleNaturalGasFixedComposition);
              annotation(
                experiment(StopTime = 1.01));
            end SimpleNaturalGasFixedComposition;
          end IdealGases;

          package Incompressible "Test models of library Modelica.Media.Incompressible"
            extends Modelica.Icons.ExamplesPackage;

            model Glycol47 "Test Modelica.Media.Incompressible.Examples.Glycol47"
              extends Modelica.Icons.Example;
              extends Modelica.Media.Examples.Tests.Components.PartialTestModel(redeclare package Medium = Modelica.Media.Incompressible.Examples.Glycol47(final singleState = true, final enthalpyOfT = true));
              annotation(
                Documentation(info = "<html>

</html>"),
                experiment(StopTime = 1.01));
            end Glycol47;

            model Essotherm650 "Test Modelica.Media.Incompressible.Examples.Essotherm65"
              extends Modelica.Icons.Example;
              extends Modelica.Media.Examples.Tests.Components.PartialTestModel(redeclare package Medium = Modelica.Media.Incompressible.Examples.Essotherm650);
              annotation(
                Documentation(info = "<html>

</html>"),
                experiment(StopTime = 1.01));
            end Essotherm650;
            annotation(
              Documentation(info = "<html>

</html>"));
          end Incompressible;

          package Water "Test models of library Modelica.Media.Water"
            extends Modelica.Icons.ExamplesPackage;

            model ConstantPropertyLiquidWater "Test Modelica.Media.Water.ConstantPropertyLiquidWater"
              extends Modelica.Icons.Example;
              extends Modelica.Media.Examples.Tests.Components.PartialTestModel(redeclare package Medium = Modelica.Media.Water.ConstantPropertyLiquidWater);
              annotation(
                Documentation(info = "<html>

</html>"),
                experiment(StopTime = 1.01));
            end ConstantPropertyLiquidWater;

            model IdealSteam "Test Modelica.Media.Water.IdealSteam"
              extends Modelica.Icons.Example;
              extends Modelica.Media.Examples.Tests.Components.PartialTestModel(redeclare package Medium = Modelica.Media.Water.IdealSteam);
              annotation(
                Documentation(info = "<html>

</html>"),
                experiment(StopTime = 1.01));
            end IdealSteam;

            model WaterIF97OnePhase_ph "Test Modelica.Media.Water.WaterIF97OnePhase_ph"
              extends Modelica.Icons.Example;
              extends Modelica.Media.Examples.Tests.Components.PartialTestModel(redeclare package Medium = Modelica.Media.Water.WaterIF97OnePhase_ph, fixedMassFlowRate(use_T_ambient = false, h_ambient = 363755), ambient(use_T_ambient = false, h_ambient = 112570));
              annotation(
                Documentation(info = "<html>

</html>"),
                experiment(StopTime = 1.01));
            end WaterIF97OnePhase_ph;

            model WaterIF97_pT "Test Modelica.Media.Water.WaterIF97_pT"
              extends Modelica.Icons.Example;
              extends Modelica.Media.Examples.Tests.Components.PartialTestModel(redeclare package Medium = Modelica.Media.Water.WaterIF97_pT);
              annotation(
                Documentation(info = "<html>

</html>"),
                experiment(StopTime = 1.01));
            end WaterIF97_pT;

            model WaterIF97_ph "Test Modelica.Media.Water.WaterIF97_ph"
              extends Modelica.Icons.Example;
              extends Modelica.Media.Examples.Tests.Components.PartialTestModel(redeclare package Medium = Modelica.Media.Water.WaterIF97_ph, ambient(use_T_ambient = false, h_ambient = 112570), fixedMassFlowRate(use_T_ambient = false, h_ambient = 363755));
              annotation(
                Documentation(info = "<html>

</html>"),
                experiment(StopTime = 1.01));
            end WaterIF97_ph;

            /*
                    model WaterIF97_dT "Test Modelica.Media.Water.WaterIF97_dT"
                      extends Modelica.Media.Examples.Tests.Components.PartialTestModel(
                         redeclare package Medium = Modelica.Media.Water.WaterIF97_dT,
                          ambient(use_p_ambient=false, d_ambient=996.557));
                    end WaterIF97_dT;
            */
            annotation(
              Documentation(info = "<html>

</html>"));
          end Water;

          package LinearFluid "Test models of library Modelica.Media.Incompressible"
            extends Modelica.Icons.ExamplesPackage;

            model LinearColdWater "Test Modelica.Media.Incompressible.Examples.Glycol47"
              extends Modelica.Icons.Example;
              extends Modelica.Media.Examples.Tests.Components.PartialTestModel(redeclare package Medium = Modelica.Media.CompressibleLiquids.LinearColdWater);
              annotation(
                Documentation(info = "<html>

</html>"),
                experiment(StopTime = 1.01));
            end LinearColdWater;

            model LinearWater_pT "Test Modelica.Media.Incompressible.Examples.Essotherm65"
              extends Modelica.Icons.Example;
              extends Modelica.Media.Examples.Tests.Components.PartialTestModel(redeclare package Medium = Modelica.Media.CompressibleLiquids.LinearWater_pT_Ambient);
              annotation(
                Documentation(info = "<html>

</html>"),
                experiment(StopTime = 1.01));
            end LinearWater_pT;
            annotation(
              Documentation(info = "<html>

</html>"));
          end LinearFluid;
          annotation(
            Documentation(info = "<html>

</html>"));
        end MediaTestModels;
        annotation(
          Documentation(info = "<html>

</html>"));
      end Tests;

      package SolveOneNonlinearEquation "Demonstrate how to solve one non-linear algebraic equation in one unknown"
        extends Modelica.Icons.ExamplesPackage;

        model Inverse_sine "Solve y = A*sin(w*x) for x, given y"
          import Modelica.Utilities.Streams.print;
          extends Modelica.Icons.Example;
          parameter Real y_zero = 0.5 "Desired value of A*sin(w*x)";
          parameter Real x_min = -1.7 "Minimum value of x_zero";
          parameter Real x_max = 1.7 "Maximum value of x_zero";
          parameter Real A = 1;
          parameter Real w = 1;
          parameter Inverse_sine_definition.f_nonlinear_Data data = Inverse_sine_definition.f_nonlinear_Data(A = A, w = w);
          Real x_zero "y_zero = A*sin(w*x_zero)";

          encapsulated package Inverse_sine_definition "Define sine as non-linear equation to be solved"
            import Modelica;
            extends Modelica.Media.Common.OneNonLinearEquation;

            redeclare record extends f_nonlinear_Data "Data for nonlinear equation"
                Real A;
                Real w;
              annotation(
                Documentation(info = "<html>

</html>"));
            end f_nonlinear_Data;

            redeclare function extends f_nonlinear "Non-linear equation to be solved"
              algorithm
                y := f_nonlinear_data.A * Modelica.Math.sin(f_nonlinear_data.w * x);
            end f_nonlinear;

            // Dummy definition has to be added for current Dymola (advice from Hans)

            redeclare function extends solve "Solution algorithm of non-linear equation"
            end solve;
            annotation(
              Documentation(info = "<html>

</html>"));
          end Inverse_sine_definition;
        equation
          x_zero = Inverse_sine_definition.solve(y_zero, x_min, x_max, f_nonlinear_data = data);
          print("x_zero = " + String(x_zero) + ", y_zero = " + String(y_zero) + ", A*sin(w*x_zero) = " + String(data.A * Modelica.Math.sin(data.w * x_zero)));
          annotation(
            experiment(StopTime = 0),
            Documentation(info = "<html>
<p>
This models solves the following non-linear equation
</p>

<pre>
   y = A*sin(w*x); -> determine x for given y
</pre>

<p>
Translate model \"Inverse_sine\"
and simulate for 0 sec. The result is printed to the
output window.
</p>

</html>"));
        end Inverse_sine;

        model Inverse_sh_T "Solve h = h_T(T), s = s_T(T) for T, if h or s is given for ideal gas NASA"
          extends Modelica.Icons.Example;
          replaceable package Medium = Modelica.Media.Air.DryAirNasa constrainedby Modelica.Media.IdealGases.Common.SingleGasNasa "Medium model" annotation(
             choicesAllMatching = true);
          parameter SI.Temperature T_min = 300 "Vary temperature linearly from T_min (time=0) upto T_max (time=1)";
          parameter SI.Temperature T_max = 500 "Vary temperature linearly from T_min (time=0) upto T_max (time=1)";
          parameter SI.Pressure p = 1.0e5 "Fixed pressure in model";
          final parameter SI.SpecificEnthalpy h_min = Medium.specificEnthalpy(Medium.setState_pT(p, T_min)) "Specific enthalpy at T_min";
          final parameter SI.SpecificEnthalpy h_max = Medium.specificEnthalpy(Medium.setState_pT(p, T_max)) "Specific enthalpy at T_max";
          final parameter SI.SpecificEntropy s_min = Medium.specificEntropy(Medium.setState_pT(p, T_min)) "Specific entropy at T_min";
          final parameter SI.SpecificEntropy s_max = Medium.specificEntropy(Medium.setState_pT(p, T_max)) "Specific entropy at T_max";
          SI.SpecificEnthalpy h1(start = h_min, fixed = true) "Pre-defined specific enthalpy";
          SI.SpecificEnthalpy h2 "Specific enthalpy computed from T (= h1 required)";
          SI.SpecificEntropy s1(start = s_min, fixed = true) "Pre-defined specific entropy";
          SI.SpecificEntropy s2 "Specific entropy computed from T (= h1 required)";
          SI.Temperature Th "Temperature computed from h1";
          SI.Temperature Ts "Temperature computed from s1";
        protected
          constant SI.Time timeUnit = 1.0;
        equation
// Define specific enthalpy and specific entropy
          der(h1) = if time < 1.0 then 1 / timeUnit * (h_max - h_min) else 0.0;
          der(s1) = if time < 1.0 then 1 / timeUnit * (s_max - s_min) else 0.0;
// Solve for temperature
          Th = Medium.temperature_phX(p, h1, fill(0.0, 0));
          Ts = Medium.temperature_psX(p, s1, fill(0.0, 0));
// Check (h2 must be identical to h1)
          h2 = Medium.specificEnthalpy_pTX(p, Th, fill(0.0, 0));
          s2 = Medium.specificEntropy(Medium.setState_pT(p, Ts));
          annotation(
            experiment(StopTime = 1),
            Documentation(info = "<html>
                               </html>"));
        end Inverse_sh_T;

        model InverseIncompressible_sh_T "Inverse computation for incompressible media"
          extends Modelica.Icons.Example;
          replaceable package Medium = Modelica.Media.Incompressible.Examples.Glycol47 "Medium model" annotation(
            choicesAllMatching = true);
          parameter SI.Temperature T_min = Medium.T_min "Vary temperature linearly from T_min (time=0) upto T_max (time=1)";
          parameter SI.Temperature T_max = Medium.T_max "Vary temperature linearly from T_min (time=0) upto T_max (time=1)";
          parameter SI.Pressure p = 1.0e5 "Fixed pressure in model";
          final parameter SI.SpecificEnthalpy h_min = Medium.h_T(Medium.T_min) "Specific enthalpy at T_min";
          final parameter SI.SpecificEnthalpy h_max = Medium.h_T(Medium.T_max) "Specific enthalpy at T_max";
          final parameter SI.SpecificEntropy s_min = Medium.specificEntropy(Medium.setState_pT(p, T_min)) "Specific entropy at T_min";
          final parameter SI.SpecificEntropy s_max = Medium.specificEntropy(Medium.setState_pT(p, T_max)) "Specific entropy at T_max";
          SI.SpecificEnthalpy h1(start = h_min, fixed = true) "Pre-defined specific enthalpy";
          SI.SpecificEnthalpy h2 "Specific enthalpy computed from T (= h1 required)";
          SI.SpecificEntropy s1(start = s_min, fixed = true) "Pre-defined specific entropy";
          SI.SpecificEntropy s2 "Specific entropy computed from T (= h1 required)";
          SI.Temperature Th "Temperature computed from h1";
          SI.Temperature Ts "Temperature computed from s1";
        protected
          constant SI.Time timeUnit = 1.0;
        equation
// Define specific enthalpy
          der(h1) = if time < 1.0 then 1 / timeUnit * (h_max - h_min) else 0.0;
          der(s1) = if time < 1.0 then 1 / timeUnit * (s_max - s_min) else 0.0;
// Solve for temperature
          Th = Medium.temperature_phX(p, h1, fill(0.0, 0));
          Ts = Medium.temperature_psX(p, s1, fill(0.0, 0));
// Check (h2 must be identical to h1)
          h2 = Medium.specificEnthalpy_pTX(p, Th, fill(0.0, 0));
          s2 = Medium.specificEntropy(Medium.setState_pT(p, Ts));
          annotation(
            experiment(StopTime = 1),
            Documentation(info = "<html>

                               </html>"));
        end InverseIncompressible_sh_T;

        model Inverse_sh_TX "Solve h = h_TX(TX) for T, if h is given for ideal gas NASA"
          extends Modelica.Icons.Example;
          replaceable package Medium = Modelica.Media.IdealGases.MixtureGases.FlueGasLambdaOnePlus constrainedby Modelica.Media.IdealGases.Common.MixtureGasNasa "Medium model" annotation(
             choicesAllMatching = true);
          parameter SI.Temperature T_min = 300 "Vary temperature linearly from T_min (time=0) upto T_max (time=1)";
          parameter SI.Temperature T_max = 500 "Vary temperature linearly from T_min (time=0) upto T_max (time=1)";
          parameter SI.Pressure p = 1.0e5 "Fixed pressure in model";
          final parameter SI.SpecificEnthalpy h_min = Medium.h_TX(T_min, X) "Specific enthalpy at T_min";
          final parameter SI.SpecificEnthalpy h_max = Medium.h_TX(T_max, X) "Specific enthalpy at T_max";
          final parameter SI.SpecificEntropy s_min = Medium.specificEntropy(Medium.setState_pTX(p, T_min, Medium.reference_X)) "Specific entropy at T_min";
          final parameter SI.SpecificEntropy s_max = Medium.specificEntropy(Medium.setState_pTX(p, T_max, Medium.reference_X)) "Specific entropy at T_max";
          SI.SpecificEnthalpy h1(start = h_min, fixed = true) "Pre-defined specific enthalpy";
          SI.SpecificEnthalpy h2 "Specific enthalpy computed from T (= h1 required)";
          SI.SpecificEntropy s1(start = s_min, fixed = true) "Pre-defined specific entropy";
          SI.SpecificEntropy s2 "Specific entropy computed from T (= h1 required)";
          SI.Temperature Th "Temperature computed from h1";
          SI.Temperature Ts "Temperature computed from s1";
          parameter SI.MassFraction[4] X = Medium.reference_X "Mass fraction vector";
        protected
          constant SI.Time timeUnit = 1.0;
        equation
// Define specific enthalpy
          der(h1) = if time < 1.0 then 1 / timeUnit * (h_max - h_min) else 0.0;
          der(s1) = if time < 1.0 then 1 / timeUnit * (s_max - s_min) else 0.0;
// Solve for temperature
          Th = Medium.temperature_phX(p, h1, X);
          Ts = Medium.temperature_psX(p, s1, X);
// Check (h2 must be identical to h1)
          h2 = Medium.specificEnthalpy_pTX(p, Th, X);
          s2 = Medium.specificEntropy(Medium.setState_pTX(p, Ts, X));
          annotation(
            experiment(StopTime = 1),
            Documentation(info = "<html>

</html>"));
        end Inverse_sh_TX;
        annotation(
          Documentation(info = "<html>
<p>
This package demonstrates how to solve one non-linear algebraic
equation in one unknown with function
Modelica.Media.Common.OneNonLinearEquation.
</p>

</html>"));
      end SolveOneNonlinearEquation;
      annotation(
        Documentation(info = "<html>
<h4>Examples</h4>
Physical properties for fluids are needed in so many different variants
that a library can only provide models for the most common situations.
With the following examples we are going to demonstrate how to use the
existing packages and functions in Modelica.Media to customize these
models for advanced applications. The high level functions try to
abstract as much as possible form the fact that different media are
based on different variables, e.g., ideal gases need pressure and
temperature, while many refrigerants are based on Helmholtz functions
of density and temperature, and many water properties are based on
pressure and specific enthalpy. Medium properties are needed in control
volumes in the dynamic state equations and in many thermodynamic state
locations that are independent of the dynamic states of a control
volume, e.g., at a wall temperature, an isentropic reference state or at
a phase boundary. The general structure of the library is such that:<br>
<ul>
<li>Each medium has a model called BaseProperties. BaseProperties
contains the minimum set of medium properties needed in a dynamic
control volume model.</li>
<li>Each instance of BaseProperties contains a \"state\" record that is
an input to all the functions to compute properties. If these functions
need further inputs, like e.g., the molarMass, these are accessible as
constants in the package.</li>
<li>The simplest way to compute properties at any other reference
point is to declare an instance of ThermodynamicState and use that as
input to arbitrary property functions.<br>
</li>
</ul>

<p>
A small library of generic volume, pipe, pump and ambient models
is provided in Modelica.Media.Examples.Tests.Components to demonstrate
how fluid components should be implemented that are using Modelica.Media
models. This library is also used to test all media models in
Modelica.Media.Examples.Tests.MediaTestModels.
</p>
</html>"));
    end Examples;

    package Interfaces "Interfaces for media models"
      extends Modelica.Icons.InterfacesPackage;

      partial package TemplateMedium "Template for media models"
        /* For a new medium, make a copy of this package and remove
             the "partial" keyword from the package definition above.
             The statement below extends from PartialMedium and sets some
             package constants. Provide values for these constants
             that are appropriate for your medium model. Note that other
             constants (such as nX, nXi) are automatically defined by
             definitions given in the base class Interfaces.PartialMedium"
          */
        extends Modelica.Media.Interfaces.PartialMedium(final mediumName = "NameOfMedium", final substanceNames = {mediumName}, final singleState = false, final reducedX = true, final fixedX = true, Temperature(min = 273, max = 450, start = 323));
        // Provide medium constants here
        constant SpecificHeatCapacity cp_const = 123456 "Constant specific heat capacity at constant pressure";
        /* The vector substanceNames is mandatory, as the number of
                 substances is determined based on its size. Here we assume
                 a single-component medium.
                 singleState is true if u and d do not depend on pressure, but only
                 on a thermal variable (temperature or enthalpy). Otherwise, set it
                 to false.
                 For a single-substance medium, just set reducedX and fixedX to true, and there's
                 no need to bother about medium compositions at all. Otherwise, set
                 final reducedX = true if the medium model has nS-1 independent mass
                 fraction, or reducedX = false if the medium model has nS independent
                 mass fractions (nS = number of substances).
                 If a mixture has a fixed composition set fixedX=true, otherwise false.
                 The modifiers for reducedX and fixedX should normally be final
                 since the other equations are based on these values.

                 It is also possible to redeclare the min, max, and start attributes of
                 Medium types, defined in the base class Interfaces.PartialMedium
                 (the example of Temperature is shown here). Min and max attributes
                 should be set in accordance to the limits of validity of the medium
                 model, while the start attribute should be a reasonable default value
                 for the initialization of nonlinear solver iterations */
        /* Provide an implementation of model BaseProperties,
             that is defined in PartialMedium. Select two independent
             variables from p, T, d, u, h. The other independent
             variables are the mass fractions "Xi", if there is more
             than one substance. Provide 3 equations to obtain the remaining
             variables as functions of the independent variables.
             It is also necessary to provide two additional equations to set
             the gas constant R and the molar mass MM of the medium.
             Finally, the thermodynamic state vector, defined in the base class
             Interfaces.PartialMedium.BaseProperties, should be set, according to
             its definition (see ThermodynamicState below).
             The computation of vector X[nX] from Xi[nXi] is already included in
             the base class Interfaces.PartialMedium.BaseProperties, so it should not
             be repeated here.
             The code fragment below is for a single-substance medium with
             p,T as independent variables.
          */

        redeclare model extends BaseProperties(final standardOrderComponents = true) "Base properties of medium"
          equation
            d = 1;
            h = cp_const * T;
            u = h - p / d;
            MM = 0.024;
            R = 8.3144 / MM;
            state.p = p;
            state.T = T;
        end BaseProperties;

        /* Provide implementations of the following optional properties.
             If not available, delete the corresponding function.
             The record "ThermodynamicState" contains the input arguments
             of all the function and is defined together with the used
             type definitions in PartialMedium. The record most often contains two of the
             variables "p, T, d, h" (e.g., medium.T)
          */

        redeclare replaceable record ThermodynamicState "A selection of variables that uniquely defines the thermodynamic state"
          extends Modelica.Icons.Record;
          AbsolutePressure p "Absolute pressure of medium";
          Temperature T "Temperature of medium";
          annotation(
            Documentation(info = "<html>

</html>"));
        end ThermodynamicState;

        redeclare function extends dynamicViscosity "Return dynamic viscosity"
          algorithm
            eta := 10 - state.T * 0.3 + state.p * 0.2;
          annotation(
            Documentation(info = "<html>

</html>"));
        end dynamicViscosity;

        redeclare function extends thermalConductivity "Return thermal conductivity"
          algorithm
            lambda := 0;
          annotation(
            Documentation(info = "<html>

</html>"));
        end thermalConductivity;

        redeclare function extends specificEntropy "Return specific entropy"
          algorithm
            s := 0;
          annotation(
            Documentation(info = "<html>

</html>"));
        end specificEntropy;

        redeclare function extends specificHeatCapacityCp "Return specific heat capacity at constant pressure"
          algorithm
            cp := 0;
          annotation(
            Documentation(info = "<html>

</html>"));
        end specificHeatCapacityCp;

        redeclare function extends specificHeatCapacityCv "Return specific heat capacity at constant volume"
          algorithm
            cv := 0;
          annotation(
            Documentation(info = "<html>

</html>"));
        end specificHeatCapacityCv;

        redeclare function extends isentropicExponent "Return isentropic exponent"
            extends Modelica.Icons.Function;

          algorithm
            gamma := 1;
          annotation(
            Documentation(info = "<html>

</html>"));
        end isentropicExponent;

        redeclare function extends velocityOfSound "Return velocity of sound"
            extends Modelica.Icons.Function;

          algorithm
            a := 0;
          annotation(
            Documentation(info = "<html>

</html>"));
        end velocityOfSound;
        annotation(
          Documentation(info = "<HTML>
<p>
This package is a <b>template</b> for <b>new medium</b> models. For a new
medium model just make a copy of this package, remove the
\"partial\" keyword from the package and provide
the information that is requested in the comments of the
Modelica source.
</p>
</HTML>"));
      end TemplateMedium;

      partial package PartialMedium "Partial medium properties (base package of all media packages)"
        extends Modelica.Media.Interfaces.Types;
        extends Modelica.Icons.MaterialPropertiesPackage;
        // Constants to be set in Medium
        constant Modelica.Media.Interfaces.Choices.IndependentVariables ThermoStates "Enumeration type for independent variables";
        constant String mediumName = "unusablePartialMedium" "Name of the medium";
        constant String substanceNames[:] = {mediumName} "Names of the mixture substances. Set substanceNames={mediumName} if only one substance.";
        constant String extraPropertiesNames[:] = fill("", 0) "Names of the additional (extra) transported properties. Set extraPropertiesNames=fill(\"\",0) if unused";
        constant Boolean singleState "= true, if u and d are not a function of pressure";
        constant Boolean reducedX = true "= true if medium contains the equation sum(X) = 1.0; set reducedX=true if only one substance (see docu for details)";
        constant Boolean fixedX = false "= true if medium contains the equation X = reference_X";
        constant AbsolutePressure reference_p = 101325 "Reference pressure of Medium: default 1 atmosphere";
        constant Temperature reference_T = 298.15 "Reference temperature of Medium: default 25 deg Celsius";
        constant MassFraction reference_X[nX] = fill(1 / nX, nX) "Default mass fractions of medium";
        constant AbsolutePressure p_default = 101325 "Default value for pressure of medium (for initialization)";
        constant Temperature T_default = Modelica.SIunits.Conversions.from_degC(20) "Default value for temperature of medium (for initialization)";
        constant SpecificEnthalpy h_default = specificEnthalpy_pTX(p_default, T_default, X_default) "Default value for specific enthalpy of medium (for initialization)";
        constant MassFraction X_default[nX] = reference_X "Default value for mass fractions of medium (for initialization)";
        final constant Integer nS = size(substanceNames, 1) "Number of substances" annotation(
          Evaluate = true);
        constant Integer nX = nS "Number of mass fractions" annotation(
          Evaluate = true);
        constant Integer nXi = if fixedX then 0 else if reducedX then nS - 1 else nS "Number of structurally independent mass fractions (see docu for details)" annotation(
          Evaluate = true);
        final constant Integer nC = size(extraPropertiesNames, 1) "Number of extra (outside of standard mass-balance) transported properties" annotation(
          Evaluate = true);
        constant Real C_nominal[nC](min = fill(Modelica.Constants.eps, nC)) = 1.0e-6 * ones(nC) "Default for the nominal values for the extra properties";
        replaceable record FluidConstants = Modelica.Media.Interfaces.Types.Basic.FluidConstants "Critical, triple, molecular and other standard data of fluid";

        replaceable record ThermodynamicState "Minimal variable set that is available as input argument to every medium function"
          extends Modelica.Icons.Record;
        end ThermodynamicState;

        replaceable partial model BaseProperties "Base properties (p, d, T, h, u, R, MM and, if applicable, X and Xi) of a medium"
          InputAbsolutePressure p "Absolute pressure of medium";
          InputMassFraction[nXi] Xi(start = reference_X[1:nXi]) "Structurally independent mass fractions";
          InputSpecificEnthalpy h "Specific enthalpy of medium";
          Density d "Density of medium";
          Temperature T "Temperature of medium";
          MassFraction[nX] X(start = reference_X) "Mass fractions (= (component mass)/total mass  m_i/m)";
          SpecificInternalEnergy u "Specific internal energy of medium";
          SpecificHeatCapacity R "Gas constant (of mixture if applicable)";
          MolarMass MM "Molar mass (of mixture or single fluid)";
          ThermodynamicState state "Thermodynamic state record for optional functions";
          parameter Boolean preferredMediumStates = false "= true if StateSelect.prefer shall be used for the independent property variables of the medium" annotation(
            Evaluate = true,
            Dialog(tab = "Advanced"));
          parameter Boolean standardOrderComponents = true "If true, and reducedX = true, the last element of X will be computed from the other ones";
          SI.Conversions.NonSIunits.Temperature_degC T_degC = Modelica.SIunits.Conversions.to_degC(T) "Temperature of medium in [degC]";
          SI.Conversions.NonSIunits.Pressure_bar p_bar = Modelica.SIunits.Conversions.to_bar(p) "Absolute pressure of medium in [bar]";
          // Local connector definition, used for equation balancing check
          connector InputAbsolutePressure = input SI.AbsolutePressure "Pressure as input signal connector";
          connector InputSpecificEnthalpy = input SI.SpecificEnthalpy "Specific enthalpy as input signal connector";
          connector InputMassFraction = input SI.MassFraction "Mass fraction as input signal connector";
        equation
          if standardOrderComponents then
            Xi = X[1:nXi];
            if fixedX then
              X = reference_X;
            end if;
            if reducedX and not fixedX then
              X[nX] = 1 - sum(Xi);
            end if;
            for i in 1:nX loop
              assert(X[i] >= (-1.e-5) and X[i] <= 1 + 1.e-5, "Mass fraction X[" + String(i) + "] = " + String(X[i]) + "of substance " + substanceNames[i] + "\nof medium " + mediumName + " is not in the range 0..1");
            end for;
          end if;
          assert(p >= 0.0, "Pressure (= " + String(p) + " Pa) of medium \"" + mediumName + "\" is negative\n(Temperature = " + String(T) + " K)");
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, 100}, {100, -100}}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, lineColor = {0, 0, 255}), Text(extent = {{-152, 164}, {152, 102}}, textString = "%name", lineColor = {0, 0, 255})}),
            Documentation(info = "<html>
<p>
Model <b>BaseProperties</b> is a model within package <b>PartialMedium</b>
and contains the <b>declarations</b> of the minimum number of
variables that every medium model is supposed to support.
A specific medium inherits from model <b>BaseProperties</b> and provides
the equations for the basic properties.</p>
<p>
The BaseProperties model contains the following <b>7+nXi variables</b>
(nXi is the number of independent mass fractions defined in package
PartialMedium):
</p>
<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"><b>Variable</b></td>
      <td valign=\"top\"><b>Unit</b></td>
      <td valign=\"top\"><b>Description</b></td></tr>
  <tr><td valign=\"top\">T</td>
      <td valign=\"top\">K</td>
      <td valign=\"top\">temperature</td></tr>
  <tr><td valign=\"top\">p</td>
      <td valign=\"top\">Pa</td>
      <td valign=\"top\">absolute pressure</td></tr>
  <tr><td valign=\"top\">d</td>
      <td valign=\"top\">kg/m3</td>
      <td valign=\"top\">density</td></tr>
  <tr><td valign=\"top\">h</td>
      <td valign=\"top\">J/kg</td>
      <td valign=\"top\">specific enthalpy</td></tr>
  <tr><td valign=\"top\">u</td>
      <td valign=\"top\">J/kg</td>
      <td valign=\"top\">specific internal energy</td></tr>
  <tr><td valign=\"top\">Xi[nXi]</td>
      <td valign=\"top\">kg/kg</td>
      <td valign=\"top\">independent mass fractions m_i/m</td></tr>
  <tr><td valign=\"top\">R</td>
      <td valign=\"top\">J/kg.K</td>
      <td valign=\"top\">gas constant</td></tr>
  <tr><td valign=\"top\">M</td>
      <td valign=\"top\">kg/mol</td>
      <td valign=\"top\">molar mass</td></tr>
</table>
<p>
In order to implement an actual medium model, one can extend from this
base model and add <b>5 equations</b> that provide relations among
these variables. Equations will also have to be added in order to
set all the variables within the ThermodynamicState record state.</p>
<p>
If standardOrderComponents=true, the full composition vector X[nX]
is determined by the equations contained in this base class, depending
on the independent mass fraction vector Xi[nXi].</p>
<p>Additional <b>2 + nXi</b> equations will have to be provided
when using the BaseProperties model, in order to fully specify the
thermodynamic conditions. The input connector qualifier applied to
p, h, and nXi indirectly declares the number of missing equations,
permitting advanced equation balance checking by Modelica tools.
Please note that this doesn't mean that the additional equations
should be connection equations, nor that exactly those variables
should be supplied, in order to complete the model.
For further information, see the Modelica.Media User's guide, and
Section 4.7 (Balanced Models) of the Modelica 3.0 specification.</p>
</html>"));
        end BaseProperties;

        replaceable partial function setState_pTX "Return thermodynamic state as function of p, T and composition X or Xi"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input Temperature T "Temperature";
          input MassFraction X[:] = reference_X "Mass fractions";
          output ThermodynamicState state "Thermodynamic state record";
        end setState_pTX;

        replaceable partial function setState_phX "Return thermodynamic state as function of p, h and composition X or Xi"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SpecificEnthalpy h "Specific enthalpy";
          input MassFraction X[:] = reference_X "Mass fractions";
          output ThermodynamicState state "Thermodynamic state record";
        end setState_phX;

        replaceable partial function setState_psX "Return thermodynamic state as function of p, s and composition X or Xi"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SpecificEntropy s "Specific entropy";
          input MassFraction X[:] = reference_X "Mass fractions";
          output ThermodynamicState state "Thermodynamic state record";
        end setState_psX;

        replaceable partial function setState_dTX "Return thermodynamic state as function of d, T and composition X or Xi"
          extends Modelica.Icons.Function;
          input Density d "Density";
          input Temperature T "Temperature";
          input MassFraction X[:] = reference_X "Mass fractions";
          output ThermodynamicState state "Thermodynamic state record";
        end setState_dTX;

        replaceable partial function setSmoothState "Return thermodynamic state so that it smoothly approximates: if x > 0 then state_a else state_b"
          extends Modelica.Icons.Function;
          input Real x "m_flow or dp";
          input ThermodynamicState state_a "Thermodynamic state if x > 0";
          input ThermodynamicState state_b "Thermodynamic state if x < 0";
          input Real x_small(min = 0) "Smooth transition in the region -x_small < x < x_small";
          output ThermodynamicState state "Smooth thermodynamic state for all x (continuous and differentiable)";
          annotation(
            Documentation(info = "<html>
<p>
This function is used to approximate the equation
</p>
<pre>
    state = <b>if</b> x &gt; 0 <b>then</b> state_a <b>else</b> state_b;
</pre>

<p>
by a smooth characteristic, so that the expression is continuous and differentiable:
</p>

<pre>
   state := <b>smooth</b>(1, <b>if</b> x &gt;  x_small <b>then</b> state_a <b>else</b>
                      <b>if</b> x &lt; -x_small <b>then</b> state_b <b>else</b> f(state_a, state_b));
</pre>

<p>
This is performed by applying function <b>Media.Common.smoothStep</b>(..)
on every element of the thermodynamic state record.
</p>

<p>
If <b>mass fractions</b> X[:] are approximated with this function then this can be performed
for all <b>nX</b> mass fractions, instead of applying it for nX-1 mass fractions and computing
the last one by the mass fraction constraint sum(X)=1. The reason is that the approximating function has the
property that sum(state.X) = 1, provided sum(state_a.X) = sum(state_b.X) = 1.
This can be shown by evaluating the approximating function in the abs(x) &lt; x_small
region (otherwise state.X is either state_a.X or state_b.X):
</p>

<pre>
    X[1]  = smoothStep(x, X_a[1] , X_b[1] , x_small);
    X[2]  = smoothStep(x, X_a[2] , X_b[2] , x_small);
       ...
    X[nX] = smoothStep(x, X_a[nX], X_b[nX], x_small);
</pre>

<p>
or
</p>

<pre>
    X[1]  = c*(X_a[1]  - X_b[1])  + (X_a[1]  + X_b[1])/2
    X[2]  = c*(X_a[2]  - X_b[2])  + (X_a[2]  + X_b[2])/2;
       ...
    X[nX] = c*(X_a[nX] - X_b[nX]) + (X_a[nX] + X_b[nX])/2;
    c     = (x/x_small)*((x/x_small)^2 - 3)/4
</pre>

<p>
Summing all mass fractions together results in
</p>

<pre>
    sum(X) = c*(sum(X_a) - sum(X_b)) + (sum(X_a) + sum(X_b))/2
           = c*(1 - 1) + (1 + 1)/2
           = 1
</pre>

</html>"));
        end setSmoothState;

        replaceable partial function dynamicViscosity "Return dynamic viscosity"
          extends Modelica.Icons.Function;
          input ThermodynamicState state "Thermodynamic state record";
          output DynamicViscosity eta "Dynamic viscosity";
        end dynamicViscosity;

        replaceable partial function thermalConductivity "Return thermal conductivity"
          extends Modelica.Icons.Function;
          input ThermodynamicState state "Thermodynamic state record";
          output ThermalConductivity lambda "Thermal conductivity";
        end thermalConductivity;

        replaceable function prandtlNumber "Return the Prandtl number"
          extends Modelica.Icons.Function;
          input ThermodynamicState state "Thermodynamic state record";
          output PrandtlNumber Pr "Prandtl number";
        algorithm
          Pr := dynamicViscosity(state) * specificHeatCapacityCp(state) / thermalConductivity(state);
        end prandtlNumber;

        replaceable partial function pressure "Return pressure"
          extends Modelica.Icons.Function;
          input ThermodynamicState state "Thermodynamic state record";
          output AbsolutePressure p "Pressure";
        end pressure;

        replaceable partial function temperature "Return temperature"
          extends Modelica.Icons.Function;
          input ThermodynamicState state "Thermodynamic state record";
          output Temperature T "Temperature";
        end temperature;

        replaceable partial function density "Return density"
          extends Modelica.Icons.Function;
          input ThermodynamicState state "Thermodynamic state record";
          output Density d "Density";
        end density;

        replaceable partial function specificEnthalpy "Return specific enthalpy"
          extends Modelica.Icons.Function;
          input ThermodynamicState state "Thermodynamic state record";
          output SpecificEnthalpy h "Specific enthalpy";
        end specificEnthalpy;

        replaceable partial function specificInternalEnergy "Return specific internal energy"
          extends Modelica.Icons.Function;
          input ThermodynamicState state "Thermodynamic state record";
          output SpecificEnergy u "Specific internal energy";
        end specificInternalEnergy;

        replaceable partial function specificEntropy "Return specific entropy"
          extends Modelica.Icons.Function;
          input ThermodynamicState state "Thermodynamic state record";
          output SpecificEntropy s "Specific entropy";
        end specificEntropy;

        replaceable partial function specificGibbsEnergy "Return specific Gibbs energy"
          extends Modelica.Icons.Function;
          input ThermodynamicState state "Thermodynamic state record";
          output SpecificEnergy g "Specific Gibbs energy";
        end specificGibbsEnergy;

        replaceable partial function specificHelmholtzEnergy "Return specific Helmholtz energy"
          extends Modelica.Icons.Function;
          input ThermodynamicState state "Thermodynamic state record";
          output SpecificEnergy f "Specific Helmholtz energy";
        end specificHelmholtzEnergy;

        replaceable partial function specificHeatCapacityCp "Return specific heat capacity at constant pressure"
          extends Modelica.Icons.Function;
          input ThermodynamicState state "Thermodynamic state record";
          output SpecificHeatCapacity cp "Specific heat capacity at constant pressure";
        end specificHeatCapacityCp;

        function heatCapacity_cp = specificHeatCapacityCp "Alias for deprecated name";

        replaceable partial function specificHeatCapacityCv "Return specific heat capacity at constant volume"
          extends Modelica.Icons.Function;
          input ThermodynamicState state "Thermodynamic state record";
          output SpecificHeatCapacity cv "Specific heat capacity at constant volume";
        end specificHeatCapacityCv;

        function heatCapacity_cv = specificHeatCapacityCv "Alias for deprecated name";

        replaceable partial function isentropicExponent "Return isentropic exponent"
          extends Modelica.Icons.Function;
          input ThermodynamicState state "Thermodynamic state record";
          output IsentropicExponent gamma "Isentropic exponent";
        end isentropicExponent;

        replaceable partial function isentropicEnthalpy "Return isentropic enthalpy"
          extends Modelica.Icons.Function;
          input AbsolutePressure p_downstream "Downstream pressure";
          input ThermodynamicState refState "Reference state for entropy";
          output SpecificEnthalpy h_is "Isentropic enthalpy";
          annotation(
            Documentation(info = "<html>
<p>
This function computes an isentropic state transformation:
</p>
<ol>
<li> A medium is in a particular state, refState.</li>
<li> The enthalpy at another state (h_is) shall be computed
     under the assumption that the state transformation from refState to h_is
     is performed with a change of specific entropy ds = 0 and the pressure of state h_is
     is p_downstream and the composition X upstream and downstream is assumed to be the same.</li>
</ol>

</html>"));
        end isentropicEnthalpy;

        replaceable partial function velocityOfSound "Return velocity of sound"
          extends Modelica.Icons.Function;
          input ThermodynamicState state "Thermodynamic state record";
          output VelocityOfSound a "Velocity of sound";
        end velocityOfSound;

        replaceable partial function isobaricExpansionCoefficient "Return overall the isobaric expansion coefficient beta"
          extends Modelica.Icons.Function;
          input ThermodynamicState state "Thermodynamic state record";
          output IsobaricExpansionCoefficient beta "Isobaric expansion coefficient";
          annotation(
            Documentation(info = "<html>
<pre>
beta is defined as  1/v * der(v,T), with v = 1/d, at constant pressure p.
</pre>
</html>"));
        end isobaricExpansionCoefficient;

        function beta = isobaricExpansionCoefficient "Alias for isobaricExpansionCoefficient for user convenience";

        replaceable partial function isothermalCompressibility "Return overall the isothermal compressibility factor"
          extends Modelica.Icons.Function;
          input ThermodynamicState state "Thermodynamic state record";
          output SI.IsothermalCompressibility kappa "Isothermal compressibility";
          annotation(
            Documentation(info = "<html>
<pre>

kappa is defined as - 1/v * der(v,p), with v = 1/d at constant temperature T.

</pre>
</html>"));
        end isothermalCompressibility;

        function kappa = isothermalCompressibility "Alias of isothermalCompressibility for user convenience";
        // explicit derivative functions for finite element models

        replaceable partial function density_derp_h "Return density derivative w.r.t. pressure at const specific enthalpy"
          extends Modelica.Icons.Function;
          input ThermodynamicState state "Thermodynamic state record";
          output DerDensityByPressure ddph "Density derivative w.r.t. pressure";
        end density_derp_h;

        replaceable partial function density_derh_p "Return density derivative w.r.t. specific enthalpy at constant pressure"
          extends Modelica.Icons.Function;
          input ThermodynamicState state "Thermodynamic state record";
          output DerDensityByEnthalpy ddhp "Density derivative w.r.t. specific enthalpy";
        end density_derh_p;

        replaceable partial function density_derp_T "Return density derivative w.r.t. pressure at const temperature"
          extends Modelica.Icons.Function;
          input ThermodynamicState state "Thermodynamic state record";
          output DerDensityByPressure ddpT "Density derivative w.r.t. pressure";
        end density_derp_T;

        replaceable partial function density_derT_p "Return density derivative w.r.t. temperature at constant pressure"
          extends Modelica.Icons.Function;
          input ThermodynamicState state "Thermodynamic state record";
          output DerDensityByTemperature ddTp "Density derivative w.r.t. temperature";
        end density_derT_p;

        replaceable partial function density_derX "Return density derivative w.r.t. mass fraction"
          extends Modelica.Icons.Function;
          input ThermodynamicState state "Thermodynamic state record";
          output Density[nX] dddX "Derivative of density w.r.t. mass fraction";
        end density_derX;

        replaceable partial function molarMass "Return the molar mass of the medium"
          extends Modelica.Icons.Function;
          input ThermodynamicState state "Thermodynamic state record";
          output MolarMass MM "Mixture molar mass";
        end molarMass;

        replaceable function specificEnthalpy_pTX "Return specific enthalpy from p, T, and X or Xi"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input Temperature T "Temperature";
          input MassFraction X[:] = reference_X "Mass fractions";
          output SpecificEnthalpy h "Specific enthalpy";
        algorithm
          h := specificEnthalpy(setState_pTX(p, T, X));
          annotation(
            inverse(T = temperature_phX(p, h, X)));
        end specificEnthalpy_pTX;

        replaceable function specificEntropy_pTX "Return specific enthalpy from p, T, and X or Xi"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input Temperature T "Temperature";
          input MassFraction X[:] = reference_X "Mass fractions";
          output SpecificEntropy s "Specific entropy";
        algorithm
          s := specificEntropy(setState_pTX(p, T, X));
          annotation(
            inverse(T = temperature_psX(p, s, X)));
        end specificEntropy_pTX;

        replaceable function density_pTX "Return density from p, T, and X or Xi"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input Temperature T "Temperature";
          input MassFraction X[:] "Mass fractions";
          output Density d "Density";
        algorithm
          d := density(setState_pTX(p, T, X));
        end density_pTX;

        replaceable function temperature_phX "Return temperature from p, h, and X or Xi"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SpecificEnthalpy h "Specific enthalpy";
          input MassFraction X[:] = reference_X "Mass fractions";
          output Temperature T "Temperature";
        algorithm
          T := temperature(setState_phX(p, h, X));
        end temperature_phX;

        replaceable function density_phX "Return density from p, h, and X or Xi"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SpecificEnthalpy h "Specific enthalpy";
          input MassFraction X[:] = reference_X "Mass fractions";
          output Density d "Density";
        algorithm
          d := density(setState_phX(p, h, X));
        end density_phX;

        replaceable function temperature_psX "Return temperature from p,s, and X or Xi"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SpecificEntropy s "Specific entropy";
          input MassFraction X[:] = reference_X "Mass fractions";
          output Temperature T "Temperature";
        algorithm
          T := temperature(setState_psX(p, s, X));
          annotation(
            inverse(s = specificEntropy_pTX(p, T, X)));
        end temperature_psX;

        replaceable function density_psX "Return density from p, s, and X or Xi"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SpecificEntropy s "Specific entropy";
          input MassFraction X[:] = reference_X "Mass fractions";
          output Density d "Density";
        algorithm
          d := density(setState_psX(p, s, X));
        end density_psX;

        replaceable function specificEnthalpy_psX "Return specific enthalpy from p, s, and X or Xi"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SpecificEntropy s "Specific entropy";
          input MassFraction X[:] = reference_X "Mass fractions";
          output SpecificEnthalpy h "Specific enthalpy";
        algorithm
          h := specificEnthalpy(setState_psX(p, s, X));
        end specificEnthalpy_psX;

        type MassFlowRate = SI.MassFlowRate(quantity = "MassFlowRate." + mediumName, min = -1.0e5, max = 1.e5) "Type for mass flow rate with medium specific attributes";
        // Only for backwards compatibility to version 3.2 (
        // (do not use these definitions in new models, but use Modelica.Media.Interfaces.Choices instead)
        package Choices = Modelica.Media.Interfaces.Choices annotation(
          obsolete = "Use Modelica.Media.Interfaces.Choices");
        annotation(
          Documentation(info = "<html>
<p>
<b>PartialMedium</b> is a package and contains all <b>declarations</b> for
a medium. This means that constants, models, and functions
are defined that every medium is supposed to support
(some of them are optional). A medium package
inherits from <b>PartialMedium</b> and provides the
equations for the medium. The details of this package
are described in
<a href=\"modelica://Modelica.Media.UsersGuide\">Modelica.Media.UsersGuide</a>.
</p>
</html>", revisions = "<html>

</html>"));
      end PartialMedium;

      partial package PartialPureSubstance "Base class for pure substances of one chemical substance"
        extends PartialMedium(final reducedX = true, final fixedX = true);

        replaceable function setState_pT "Return thermodynamic state from p and T"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input Temperature T "Temperature";
          output ThermodynamicState state "Thermodynamic state record";
        algorithm
          state := setState_pTX(p, T, fill(0, 0));
        end setState_pT;

        replaceable function setState_ph "Return thermodynamic state from p and h"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SpecificEnthalpy h "Specific enthalpy";
          output ThermodynamicState state "Thermodynamic state record";
        algorithm
          state := setState_phX(p, h, fill(0, 0));
        end setState_ph;

        replaceable function setState_ps "Return thermodynamic state from p and s"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SpecificEntropy s "Specific entropy";
          output ThermodynamicState state "Thermodynamic state record";
        algorithm
          state := setState_psX(p, s, fill(0, 0));
        end setState_ps;

        replaceable function setState_dT "Return thermodynamic state from d and T"
          extends Modelica.Icons.Function;
          input Density d "Density";
          input Temperature T "Temperature";
          output ThermodynamicState state "Thermodynamic state record";
        algorithm
          state := setState_dTX(d, T, fill(0, 0));
        end setState_dT;

        replaceable function density_ph "Return density from p and h"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SpecificEnthalpy h "Specific enthalpy";
          output Density d "Density";
        algorithm
          d := density_phX(p, h, fill(0, 0));
        end density_ph;

        replaceable function temperature_ph "Return temperature from p and h"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SpecificEnthalpy h "Specific enthalpy";
          output Temperature T "Temperature";
        algorithm
          T := temperature_phX(p, h, fill(0, 0));
        end temperature_ph;

        replaceable function pressure_dT "Return pressure from d and T"
          extends Modelica.Icons.Function;
          input Density d "Density";
          input Temperature T "Temperature";
          output AbsolutePressure p "Pressure";
        algorithm
          p := pressure(setState_dTX(d, T, fill(0, 0)));
        end pressure_dT;

        replaceable function specificEnthalpy_dT "Return specific enthalpy from d and T"
          extends Modelica.Icons.Function;
          input Density d "Density";
          input Temperature T "Temperature";
          output SpecificEnthalpy h "Specific enthalpy";
        algorithm
          h := specificEnthalpy(setState_dTX(d, T, fill(0, 0)));
        end specificEnthalpy_dT;

        replaceable function specificEnthalpy_ps "Return specific enthalpy from p and s"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SpecificEntropy s "Specific entropy";
          output SpecificEnthalpy h "Specific enthalpy";
        algorithm
          h := specificEnthalpy_psX(p, s, fill(0, 0));
        end specificEnthalpy_ps;

        replaceable function temperature_ps "Return temperature from p and s"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SpecificEntropy s "Specific entropy";
          output Temperature T "Temperature";
        algorithm
          T := temperature_psX(p, s, fill(0, 0));
        end temperature_ps;

        replaceable function density_ps "Return density from p and s"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SpecificEntropy s "Specific entropy";
          output Density d "Density";
        algorithm
          d := density_psX(p, s, fill(0, 0));
        end density_ps;

        replaceable function specificEnthalpy_pT "Return specific enthalpy from p and T"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input Temperature T "Temperature";
          output SpecificEnthalpy h "Specific enthalpy";
        algorithm
          h := specificEnthalpy_pTX(p, T, fill(0, 0));
        end specificEnthalpy_pT;

        replaceable function density_pT "Return density from p and T"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input Temperature T "Temperature";
          output Density d "Density";
        algorithm
          d := density(setState_pTX(p, T, fill(0, 0)));
        end density_pT;

        redeclare replaceable partial model extends BaseProperties(final standardOrderComponents = true)
        end BaseProperties;
      end PartialPureSubstance;

      partial package PartialLinearFluid "Generic pure liquid model with constant cp, compressibility and thermal expansion coefficients"
        extends Interfaces.PartialPureSubstance(ThermoStates = Modelica.Media.Interfaces.Choices.IndependentVariables.pTX, singleState = false);
        constant SpecificHeatCapacity cp_const "Specific heat capacity at constant pressure";
        constant IsobaricExpansionCoefficient beta_const "Thermal expansion coefficient at constant pressure";
        constant SI.IsothermalCompressibility kappa_const "Isothermal compressibility";
        constant MolarMass MM_const "Molar mass";
        constant Density reference_d "Density in reference conditions";
        constant SpecificEnthalpy reference_h "Specific enthalpy in reference conditions";
        constant SpecificEntropy reference_s "Specific entropy in reference conditions";
        constant Boolean constantJacobian "If true, entries in thermodynamic Jacobian are constant, taken at reference conditions";

        redeclare record ThermodynamicState "A selection of variables that uniquely defines the thermodynamic state"
          extends Modelica.Icons.Record;
          AbsolutePressure p "Absolute pressure of medium";
          Temperature T "Temperature of medium";
        end ThermodynamicState;

        redeclare model extends BaseProperties(T(stateSelect = if preferredMediumStates then StateSelect.prefer else StateSelect.default), p(stateSelect = if preferredMediumStates then StateSelect.prefer else StateSelect.default)) "Base properties of medium"
          equation
            d = (1 + (p - reference_p) * kappa_const - (T - reference_T) * beta_const) * reference_d;
            h = reference_h + (T - reference_T) * cp_const + (p - reference_p) * (1 - beta_const * reference_T) / reference_d;
            u = h - p / d;
            p = state.p;
            T = state.T;
            MM = MM_const;
            R = 8.3144 / MM;
        end BaseProperties;

        redeclare function extends setState_pTX "Set the thermodynamic state record from p and T (X not needed)"
          algorithm
            state := ThermodynamicState(p = p, T = T);
        end setState_pTX;

        redeclare function extends setState_phX "Set the thermodynamic state record from p and h (X not needed)"
          algorithm
            state := ThermodynamicState(p = p, T = (h - reference_h - (p - reference_p) * ((1 - beta_const * reference_T) / reference_d)) / cp_const + reference_T);
        end setState_phX;

        redeclare function extends setState_psX "Set the thermodynamic state record from p and s (X not needed)"
          algorithm
            state := ThermodynamicState(p = p, T = reference_T * cp_const / (cp_const - s + reference_s + (p - reference_p) * (-beta_const / reference_d)));
        end setState_psX;

        redeclare function extends setState_dTX "Set the thermodynamic state record from d and T (X not needed)"
          algorithm
            state := ThermodynamicState(p = (d - reference_d + (state.T - reference_T) * beta_const * reference_d) / (reference_d * kappa_const) + reference_p, T = T);
        end setState_dTX;

        redeclare function extends setSmoothState "Return thermodynamic state so that it smoothly approximates: if x > 0 then state_a else state_b"
          algorithm
            state := ThermodynamicState(p = Media.Common.smoothStep(x, state_a.p, state_b.p, x_small), T = Media.Common.smoothStep(x, state_a.T, state_b.T, x_small));
        end setSmoothState;

        redeclare function extends pressure "Return the pressure from the thermodynamic state"
          algorithm
            p := state.p;
        end pressure;

        redeclare function extends temperature "Return the temperature from the thermodynamic state"
          algorithm
            T := state.T;
        end temperature;

        redeclare function extends density "Return the density from the thermodynamic state"
          algorithm
            d := (1 + (state.p - reference_p) * kappa_const - (state.T - reference_T) * beta_const) * reference_d;
        end density;

        redeclare function extends specificEnthalpy "Return the specific enthalpy from the thermodynamic state"
          algorithm
            h := reference_h + (state.T - reference_T) * cp_const + (state.p - reference_p) * (1 - beta_const * reference_T) / reference_d;
        end specificEnthalpy;

        redeclare function extends specificEntropy "Return the specific entropy from the thermodynamic state"
          algorithm
            s := reference_s + (state.T - reference_T) * cp_const / state.T + (state.p - reference_p) * (-beta_const / reference_d);
        end specificEntropy;

        redeclare function extends specificInternalEnergy "Return the specific internal energy from the thermodynamic state"
          algorithm
            u := specificEnthalpy(state) - state.p / reference_d;
        end specificInternalEnergy;

        redeclare function extends specificGibbsEnergy "Return specific Gibbs energy from the thermodynamic state"
            extends Modelica.Icons.Function;

          algorithm
            g := specificEnthalpy(state) - state.T * specificEntropy(state);
        end specificGibbsEnergy;

        redeclare function extends specificHelmholtzEnergy "Return specific Helmholtz energy from the thermodynamic state"
            extends Modelica.Icons.Function;

          algorithm
            f := specificInternalEnergy(state) - state.T * specificEntropy(state);
        end specificHelmholtzEnergy;

        redeclare function extends velocityOfSound "Return velocity of sound from the thermodynamic state"
            extends Modelica.Icons.Function;

          algorithm
            a := sqrt(max(0, 1 / (kappa_const * density(state) - beta_const * beta_const * state.T / cp_const)));
        end velocityOfSound;

        redeclare function extends isentropicExponent "Return isentropic exponent from the thermodynamic state"
            extends Modelica.Icons.Function;

          algorithm
            gamma := 1 / (state.p * kappa_const) * cp_const / specificHeatCapacityCv(state);
        end isentropicExponent;

        redeclare function extends isentropicEnthalpy "Return isentropic enthalpy"
            /* Previous wrong equation:

            protected
              SpecificEntropy s_upstream = specificEntropy(refState)
                "Specific entropy at component inlet";
              ThermodynamicState downstreamState "State at downstream location";
            algorithm
              downstreamState.p := p_downstream;
              downstreamState.T := reference_T*cp_const/
                (s_upstream -reference_s -(p_downstream-reference_p)*(-beta_const/reference_d) - cp_const);
              h_is := specificEnthalpy(downstreamState);
            */

          algorithm
/* s := reference_s + (refState.T-reference_T)*cp_const/refState.T +
                        (refState.p-reference_p)*(-beta_const/reference_d)
        = reference_s + (state.T-reference_T)*cp_const/state.T +
                        (p_downstream-reference_p)*(-beta_const/reference_d);

      (state.T-reference_T)*cp_const/state.T
     = (refState.T-reference_T)*cp_const/refState.T + (refState.p-reference_p)*(-beta_const/reference_d)
       - (p_downstream-reference_p)*(-beta_const/reference_d)
     = (refState.T-reference_T)*cp_const/refState.T + (refState.p-p_downstream)*(-beta_const/reference_d)

     (x - reference_T)/x = k
     x - reference_T = k*x
     (1-k)*x = reference_T
     x = reference_T/(1-k);

     state.T = reference_T/(1 - ((refState.T-reference_T)*cp_const/refState.T + (refState.p-p_downstream)*(-beta_const/reference_d))/cp_const)
  */
            h_is := specificEnthalpy(setState_pTX(p_downstream, reference_T / (1 - ((refState.T - reference_T) / refState.T + (refState.p - p_downstream) * (-beta_const / (reference_d * cp_const)))), reference_X));
          annotation(
            Documentation(info = "<html>
<p>
A minor approximation is used: the reference density is used instead of the real
one, which would require a numeric solution.
</p>
</html>"));
        end isentropicEnthalpy;

        redeclare function extends specificHeatCapacityCp "Return specific heat capacity at constant volume"
          algorithm
            cp := cp_const;
        end specificHeatCapacityCp;

        redeclare function extends specificHeatCapacityCv "Return specific heat capacity at constant volume from the thermodynamic state"
          algorithm
            cv := if constantJacobian then cp_const - reference_T * beta_const * beta_const / (kappa_const * reference_d) else state.T * beta_const * beta_const / (kappa_const * reference_d);
        end specificHeatCapacityCv;

        redeclare function extends isothermalCompressibility "Return the isothermal compressibility kappa"
          algorithm
            kappa := kappa_const;
        end isothermalCompressibility;

        redeclare function extends isobaricExpansionCoefficient "Return the isobaric expansion coefficient"
          algorithm
            beta := beta_const;
        end isobaricExpansionCoefficient;

        redeclare function extends density_derp_h "Return density derivative w.r.t. pressure at const specific enthalpy"
          algorithm
            ddph := if constantJacobian then kappa_const * reference_d + beta_const * (1 - reference_T * beta_const) / cp_const else kappa_const * density(state) + beta_const * (1 - temperature(state) * beta_const) / cp_const;
        end density_derp_h;

        redeclare function extends density_derh_p "Return density derivative w.r.t. specific enthalpy at constant pressure"
          algorithm
            ddhp := if constantJacobian then -beta_const * reference_d / cp_const else -beta_const * density(state) / cp_const;
        end density_derh_p;

        redeclare function extends density_derp_T "Return density derivative w.r.t. pressure at const temperature"
          algorithm
            ddpT := if constantJacobian then kappa_const * reference_d else kappa_const * density(state);
        end density_derp_T;

        redeclare function extends density_derT_p "Return density derivative w.r.t. temperature at constant pressure"
          algorithm
            ddTp := if constantJacobian then -beta_const * reference_d else -beta_const * density(state);
        end density_derT_p;

        redeclare function extends density_derX "Returns the partial derivative of density with respect to mass fractions at constant pressure and temperature"
          algorithm
            dddX := fill(0, nX);
        end density_derX;

        redeclare function extends molarMass "Return molar mass"
          algorithm
            MM := MM_const;
        end molarMass;

        function T_ph "Return temperature from pressure and specific enthalpy"
          extends Modelica.Icons.Function;
          input SpecificEnthalpy h "Specific enthalpy";
          input AbsolutePressure p "Pressure";
          output Temperature T "Temperature";
        algorithm
          T := (h - reference_h - (p - reference_p) * ((1 - beta_const * reference_T) / reference_d)) / cp_const + reference_T;
        end T_ph;

        function T_ps "Return temperature from pressure and specific entropy"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SpecificEntropy s "Specific entropy";
          output Temperature T "Temperature";
        algorithm
          T := reference_T * cp_const / (s - reference_s - (p - reference_p) * (-beta_const / reference_d) - cp_const);
        end T_ps;
        annotation(
          Documentation(info = "<html>
<h4>Linear Compressibility Fluid Model</h4>
<p>This linear compressibility fluid model is based on the assumptions that:
</p>
<ul>
<li>The specific heat capacity at constant pressure (cp) is constant</li>
<li>The isobaric expansion coefficient (beta) is constant</li>
<li>The isothermal compressibility (kappa) is constant</li>
<li>Pressure and temperature are used as states</li>
<li>The influence of density on specific enthalpy (h), entropy (s), inner energy (u) and heat capacity (cv) at constant volume is neglected.
</ul>
<p>
That means that the density is a linear function in temperature and in pressure.
In order to define the complete model, a number of constant reference values are needed which
are computed at the reference values of the states pressure p and temperature T. The model can
be interpreted as a linearization of a full non-linear fluid model (but it is not linear in all
thermodynamic coordinates). Reference values are needed for
</p>
<ol>
<li>the density (reference_d),</li>
<li>the specific enthalpy (reference_h),</li>
<li>the specific entropy (reference_s).</li>
</ol>
<p>
Apart from that, a user needs to define the molar mass, MM_const.
Note that it is possible to define a fluid by computing the reference
values from a full non-linear fluid model by computing the package constants
using the standard functions defined in a fluid package (see example in liquids package).
</p>
<p>
In order to avoid numerical inversion of the temperature in the T_ph and T_ps functions, the density
is always taken to be the reference density in the computation of h, s, u and cv. For liquids (and this
model is intended only for liquids) the relative error of doing so is 1e-3 to 1e-4 at most. The model would
be more \"correct\" based on the other assumptions, if occurrences of reference_d in the computations of h,s,u
and cv would be replaced by a call to density(state). That would require a numerical solution for T_ps, while T_ph can be solved symbolically from a quadratic function. Errors from this approximation are small because liquid density varies little.</p>
<h4>Efficiency considerations</h4>
<p>One of the main reasons to use a simple, linear fluid model is to achieve high performance
in simulations. There are a number of possible compromises and possibilities to improve performance.
Some of them can be influenced by a flag. The following rules where used in this model:</p>
<ul>
<li>All forward evaluations (using the ThermodynamicState record as input) are exactly following
the assumptions above.</li>
<li>If the flag <b>constantJacobian</b> is set to true in the package, all functions that
typically appear in thermodynamic Jacobians (specificHeatCapacityCv, density_derp_h, density_derh_p,
density_derp_T, density_derT_p) are evaluated at reference conditions (that means using the reference
density) instead of the density of the current pressure and temperature. This makes it possible to evaluate
the thermodynamic Jacobian at compile time.</li>
<li>For inverse functions using other inputs than the states (e.g pressure p and specific enthalpy h),
the inversion is using the reference state whenever that is necessary to achieve a symbolic inversion.</li>
<li>If <b>constantJacobian</b> is set to false, the above list of functions is computed exactly according
to the above list of assumptions</li>
</ul>
<dl>
<dt><b>Authors:</b></dt>
<dd>Francesco Casella<br>
    Dipartimento di Elettronica e Informazione<br>
    Politecnico di Milano<br>
    Via Ponzio 34/5<br>
    I-20133 Milano, Italy<br>
    email: <A HREF=\"mailto:casella@elet.polimi.it\">casella@elet.polimi.it</A><br>
    and <br>
    Hubertus Tummescheit<br>
    Modelon AB<br>
    Ideon Science Park<br>
    SE-22730 Lund, Sweden<br>
    email: <A HREF=\"mailto:Hubertus.Tummescheit@Modelon.se\">Hubertus.Tummescheit@Modelon.se</A>
</dd>
</dl>
</html>"));
      end PartialLinearFluid;

      partial package PartialMixtureMedium "Base class for pure substances of several chemical substances"
        extends PartialMedium(redeclare replaceable record FluidConstants = Modelica.Media.Interfaces.Types.IdealGas.FluidConstants);

        redeclare replaceable record extends ThermodynamicState "Thermodynamic state variables"
            AbsolutePressure p "Absolute pressure of medium";
            Temperature T "Temperature of medium";
            MassFraction X[nX] "Mass fractions (= (component mass)/total mass  m_i/m)";
        end ThermodynamicState;

        constant FluidConstants[nS] fluidConstants "Constant data for the fluid";

        replaceable function gasConstant "Return the gas constant of the mixture (also for liquids)"
          extends Modelica.Icons.Function;
          input ThermodynamicState state "Thermodynamic state";
          output SI.SpecificHeatCapacity R "Mixture gas constant";
        end gasConstant;

        function moleToMassFractions "Return mass fractions X from mole fractions"
          extends Modelica.Icons.Function;
          input SI.MoleFraction moleFractions[:] "Mole fractions of mixture";
          input MolarMass[:] MMX "Molar masses of components";
          output SI.MassFraction X[size(moleFractions, 1)] "Mass fractions of gas mixture";
        protected
          MolarMass Mmix = moleFractions * MMX "Molar mass of mixture";
        algorithm
          for i in 1:size(moleFractions, 1) loop
            X[i] := moleFractions[i] * MMX[i] / Mmix;
          end for;
          annotation(
            smoothOrder = 5);
        end moleToMassFractions;

        function massToMoleFractions "Return mole fractions from mass fractions X"
          extends Modelica.Icons.Function;
          input SI.MassFraction X[:] "Mass fractions of mixture";
          input SI.MolarMass[:] MMX "Molar masses of components";
          output SI.MoleFraction moleFractions[size(X, 1)] "Mole fractions of gas mixture";
        protected
          Real invMMX[size(X, 1)] "Inverses of molar weights";
          SI.MolarMass Mmix "Molar mass of mixture";
        algorithm
          for i in 1:size(X, 1) loop
            invMMX[i] := 1 / MMX[i];
          end for;
          Mmix := 1 / (X * invMMX);
          for i in 1:size(X, 1) loop
            moleFractions[i] := Mmix * X[i] / MMX[i];
          end for;
          annotation(
            smoothOrder = 5);
        end massToMoleFractions;
      end PartialMixtureMedium;

      partial package PartialCondensingGases "Base class for mixtures of condensing and non-condensing gases"
        extends PartialMixtureMedium(ThermoStates = Modelica.Media.Interfaces.Choices.IndependentVariables.pTX);

        replaceable partial function saturationPressure "Return saturation pressure of condensing fluid"
          extends Modelica.Icons.Function;
          input Temperature Tsat "Saturation temperature";
          output AbsolutePressure psat "Saturation pressure";
        end saturationPressure;

        replaceable partial function enthalpyOfVaporization "Return vaporization enthalpy of condensing fluid"
          extends Modelica.Icons.Function;
          input Temperature T "Temperature";
          output SpecificEnthalpy r0 "Vaporization enthalpy";
        end enthalpyOfVaporization;

        replaceable partial function enthalpyOfLiquid "Return liquid enthalpy of condensing fluid"
          extends Modelica.Icons.Function;
          input Temperature T "Temperature";
          output SpecificEnthalpy h "Liquid enthalpy";
        end enthalpyOfLiquid;

        replaceable partial function enthalpyOfGas "Return enthalpy of non-condensing gas mixture"
          extends Modelica.Icons.Function;
          input Temperature T "Temperature";
          input MassFraction[:] X "Vector of mass fractions";
          output SpecificEnthalpy h "Specific enthalpy";
        end enthalpyOfGas;

        replaceable partial function enthalpyOfCondensingGas "Return enthalpy of condensing gas (most often steam)"
          extends Modelica.Icons.Function;
          input Temperature T "Temperature";
          output SpecificEnthalpy h "Specific enthalpy";
        end enthalpyOfCondensingGas;

        replaceable partial function enthalpyOfNonCondensingGas "Return enthalpy of the non-condensing species"
          extends Modelica.Icons.Function;
          input Temperature T "Temperature";
          output SpecificEnthalpy h "Specific enthalpy";
        end enthalpyOfNonCondensingGas;
      end PartialCondensingGases;

      partial package PartialRealCondensingGases "Base class for mixtures of real condensing and non-condensing gases"
        extends Modelica.Media.Interfaces.PartialMixtureMedium(redeclare replaceable record FluidConstants = Modelica.Media.Interfaces.Types.TwoPhase.FluidConstants);

        replaceable partial function saturationPressure "Return saturation pressure of condensing fluid"
          extends Modelica.Icons.Function;
          input ThermodynamicState state "Thermodynamic state record";
          output AbsolutePressure psat "Saturation pressure";
        end saturationPressure;

        replaceable partial function saturationTemperature "Return saturation temperature of condensing fluid"
          extends Modelica.Icons.Function;
          input ThermodynamicState state "Thermodynamic state record";
          output Temperature Tsat "Saturation temperature";
        end saturationTemperature;

        replaceable partial function massFractionSaturation "Return saturation mass fractions"
          extends Modelica.Icons.Function;
          input ThermodynamicState state "Thermodynamic state record";
          output MassFraction[:] Xsat "Saturation mass fractions";
        end massFractionSaturation;

        replaceable partial function massFraction_pTphi "Return mass fractions as a function of pressure, temperature and relative humidity"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input Temperature T "Temperature";
          input Real phi "Relative humidity";
          output MassFraction[:] X "Mass fractions";
        end massFraction_pTphi;

        replaceable partial function relativeHumidity "Return relativ humidity"
          extends Modelica.Icons.Function;
          input ThermodynamicState state "Thermodynamic state record";
          output Real phi "Relative humidity";
        end relativeHumidity;

        replaceable partial function enthalpyOfVaporization "Return vaporization enthalpy of condensing fluid"
          extends Modelica.Icons.Function;
          input ThermodynamicState state "Thermodynamic state record";
          output SpecificEnthalpy r0 "Vaporization enthalpy";
        end enthalpyOfVaporization;

        replaceable partial function enthalpyOfLiquid "Return liquid enthalpy of condensing fluid"
          extends Modelica.Icons.Function;
          input ThermodynamicState state "Thermodynamic state record";
          output SpecificEnthalpy h "Liquid enthalpy";
        end enthalpyOfLiquid;

        replaceable partial function enthalpyOfGas "Return enthalpy of non-condensing gas mixture"
          extends Modelica.Icons.Function;
          input ThermodynamicState state "Thermodynamic state record";
          output SpecificEnthalpy h "Specific enthalpy";
        end enthalpyOfGas;

        replaceable partial function enthalpyOfCondensingGas "Return enthalpy of condensing gas (most often steam)"
          extends Modelica.Icons.Function;
          input ThermodynamicState state "Thermodynamic state record";
          output SpecificEnthalpy h "Specific enthalpy";
        end enthalpyOfCondensingGas;

        replaceable partial function enthalpyOfNonCondensingGas "Return enthalpy of the non-condensing species"
          extends Modelica.Icons.Function;
          input ThermodynamicState state "Thermodynamic state record";
          output SpecificEnthalpy h "Specific enthalpy";
        end enthalpyOfNonCondensingGas;

        replaceable partial function specificEntropy_phX "Return specific entropy as a functio of pressure, specific enthalpy and mass fractions"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SpecificEnthalpy h "Specific enthalpy";
          input MassFraction X[:] = reference_X "Mass fractions";
          output SpecificEntropy s "Specific entropy";
        algorithm
          s := specificEntropy(setState_phX(p, h, X));
        end specificEntropy_phX;
      end PartialRealCondensingGases;

      partial package PartialTwoPhaseMedium "Base class for two phase medium of one substance"
        extends PartialPureSubstance(redeclare record FluidConstants = Modelica.Media.Interfaces.Types.TwoPhase.FluidConstants);
        constant Boolean smoothModel = false "True if the (derived) model should not generate state events";
        constant Boolean onePhase = false "True if the (derived) model should never be called with two-phase inputs";
        constant FluidConstants[nS] fluidConstants "Constant data for the fluid";

        redeclare replaceable record extends ThermodynamicState "Thermodynamic state of two phase medium"
            FixedPhase phase(min = 0, max = 2) "Phase of the fluid: 1 for 1-phase, 2 for two-phase, 0 for not known, e.g., interactive use";
        end ThermodynamicState;

        redeclare replaceable partial model extends BaseProperties "Base properties (p, d, T, h, u, R, MM, sat) of two phase medium"
            SaturationProperties sat "Saturation properties at the medium pressure";
        end BaseProperties;

        replaceable partial function setDewState "Return the thermodynamic state on the dew line"
          extends Modelica.Icons.Function;
          input SaturationProperties sat "Saturation point";
          input FixedPhase phase(min = 1, max = 2) = 1 "Phase: default is one phase";
          output ThermodynamicState state "Complete thermodynamic state info";
        end setDewState;

        replaceable partial function setBubbleState "Return the thermodynamic state on the bubble line"
          extends Modelica.Icons.Function;
          input SaturationProperties sat "Saturation point";
          input FixedPhase phase(min = 1, max = 2) = 1 "Phase: default is one phase";
          output ThermodynamicState state "Complete thermodynamic state info";
        end setBubbleState;

        redeclare replaceable partial function extends setState_dTX "Return thermodynamic state as function of d, T and composition X or Xi"
            input FixedPhase phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
        end setState_dTX;

        redeclare replaceable partial function extends setState_phX "Return thermodynamic state as function of p, h and composition X or Xi"
            input FixedPhase phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
        end setState_phX;

        redeclare replaceable partial function extends setState_psX "Return thermodynamic state as function of p, s and composition X or Xi"
            input FixedPhase phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
        end setState_psX;

        redeclare replaceable partial function extends setState_pTX "Return thermodynamic state as function of p, T and composition X or Xi"
            input FixedPhase phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
        end setState_pTX;

        replaceable function setSat_T "Return saturation property record from temperature"
          extends Modelica.Icons.Function;
          input Temperature T "Temperature";
          output SaturationProperties sat "Saturation property record";
        algorithm
          sat.Tsat := T;
          sat.psat := saturationPressure(T);
        end setSat_T;

        replaceable function setSat_p "Return saturation property record from pressure"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          output SaturationProperties sat "Saturation property record";
        algorithm
          sat.psat := p;
          sat.Tsat := saturationTemperature(p);
        end setSat_p;

        replaceable partial function bubbleEnthalpy "Return bubble point specific enthalpy"
          extends Modelica.Icons.Function;
          input SaturationProperties sat "Saturation property record";
          output SI.SpecificEnthalpy hl "Boiling curve specific enthalpy";
        end bubbleEnthalpy;

        replaceable partial function dewEnthalpy "Return dew point specific enthalpy"
          extends Modelica.Icons.Function;
          input SaturationProperties sat "Saturation property record";
          output SI.SpecificEnthalpy hv "Dew curve specific enthalpy";
        end dewEnthalpy;

        replaceable partial function bubbleEntropy "Return bubble point specific entropy"
          extends Modelica.Icons.Function;
          input SaturationProperties sat "Saturation property record";
          output SI.SpecificEntropy sl "Boiling curve specific entropy";
        end bubbleEntropy;

        replaceable partial function dewEntropy "Return dew point specific entropy"
          extends Modelica.Icons.Function;
          input SaturationProperties sat "Saturation property record";
          output SI.SpecificEntropy sv "Dew curve specific entropy";
        end dewEntropy;

        replaceable partial function bubbleDensity "Return bubble point density"
          extends Modelica.Icons.Function;
          input SaturationProperties sat "Saturation property record";
          output Density dl "Boiling curve density";
        end bubbleDensity;

        replaceable partial function dewDensity "Return dew point density"
          extends Modelica.Icons.Function;
          input SaturationProperties sat "Saturation property record";
          output Density dv "Dew curve density";
        end dewDensity;

        replaceable partial function saturationPressure "Return saturation pressure"
          extends Modelica.Icons.Function;
          input Temperature T "Temperature";
          output AbsolutePressure p "Saturation pressure";
        end saturationPressure;

        replaceable partial function saturationTemperature "Return saturation temperature"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          output Temperature T "Saturation temperature";
        end saturationTemperature;

        replaceable function saturationPressure_sat "Return saturation temperature"
          extends Modelica.Icons.Function;
          input SaturationProperties sat "Saturation property record";
          output AbsolutePressure p "Saturation pressure";
        algorithm
          p := sat.psat;
        end saturationPressure_sat;

        replaceable function saturationTemperature_sat "Return saturation temperature"
          extends Modelica.Icons.Function;
          input SaturationProperties sat "Saturation property record";
          output Temperature T "Saturation temperature";
        algorithm
          T := sat.Tsat;
        end saturationTemperature_sat;

        replaceable partial function saturationTemperature_derp "Return derivative of saturation temperature w.r.t. pressure"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          output DerTemperatureByPressure dTp "Derivative of saturation temperature w.r.t. pressure";
        end saturationTemperature_derp;

        replaceable function saturationTemperature_derp_sat "Return derivative of saturation temperature w.r.t. pressure"
          extends Modelica.Icons.Function;
          input SaturationProperties sat "Saturation property record";
          output DerTemperatureByPressure dTp "Derivative of saturation temperature w.r.t. pressure";
        algorithm
          dTp := saturationTemperature_derp(sat.psat);
        end saturationTemperature_derp_sat;

        replaceable partial function surfaceTension "Return surface tension sigma in the two phase region"
          extends Modelica.Icons.Function;
          input SaturationProperties sat "Saturation property record";
          output SurfaceTension sigma "Surface tension sigma in the two phase region";
        end surfaceTension;

        redeclare replaceable function extends molarMass "Return the molar mass of the medium"
          algorithm
            MM := fluidConstants[1].molarMass;
        end molarMass;

        replaceable partial function dBubbleDensity_dPressure "Return bubble point density derivative"
          extends Modelica.Icons.Function;
          input SaturationProperties sat "Saturation property record";
          output DerDensityByPressure ddldp "Boiling curve density derivative";
        end dBubbleDensity_dPressure;

        replaceable partial function dDewDensity_dPressure "Return dew point density derivative"
          extends Modelica.Icons.Function;
          input SaturationProperties sat "Saturation property record";
          output DerDensityByPressure ddvdp "Saturated steam density derivative";
        end dDewDensity_dPressure;

        replaceable partial function dBubbleEnthalpy_dPressure "Return bubble point specific enthalpy derivative"
          extends Modelica.Icons.Function;
          input SaturationProperties sat "Saturation property record";
          output DerEnthalpyByPressure dhldp "Boiling curve specific enthalpy derivative";
        end dBubbleEnthalpy_dPressure;

        replaceable partial function dDewEnthalpy_dPressure "Return dew point specific enthalpy derivative"
          extends Modelica.Icons.Function;
          input SaturationProperties sat "Saturation property record";
          output DerEnthalpyByPressure dhvdp "Saturated steam specific enthalpy derivative";
        end dDewEnthalpy_dPressure;

        redeclare replaceable function specificEnthalpy_pTX "Return specific enthalpy from pressure, temperature and mass fraction"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input Temperature T "Temperature";
          input MassFraction X[:] "Mass fractions";
          input FixedPhase phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          output SpecificEnthalpy h "Specific enthalpy at p, T, X";
        algorithm
          h := specificEnthalpy(setState_pTX(p, T, X, phase));
        end specificEnthalpy_pTX;

        redeclare replaceable function temperature_phX "Return temperature from p, h, and X or Xi"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SpecificEnthalpy h "Specific enthalpy";
          input MassFraction X[:] "Mass fractions";
          input FixedPhase phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          output Temperature T "Temperature";
        algorithm
          T := temperature(setState_phX(p, h, X, phase));
        end temperature_phX;

        redeclare replaceable function density_phX "Return density from p, h, and X or Xi"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SpecificEnthalpy h "Specific enthalpy";
          input MassFraction X[:] "Mass fractions";
          input FixedPhase phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          output Density d "Density";
        algorithm
          d := density(setState_phX(p, h, X, phase));
        end density_phX;

        redeclare replaceable function temperature_psX "Return temperature from p, s, and X or Xi"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SpecificEntropy s "Specific entropy";
          input MassFraction X[:] "Mass fractions";
          input FixedPhase phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          output Temperature T "Temperature";
        algorithm
          T := temperature(setState_psX(p, s, X, phase));
        end temperature_psX;

        redeclare replaceable function density_psX "Return density from p, s, and X or Xi"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SpecificEntropy s "Specific entropy";
          input MassFraction X[:] "Mass fractions";
          input FixedPhase phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          output Density d "Density";
        algorithm
          d := density(setState_psX(p, s, X, phase));
        end density_psX;

        redeclare replaceable function specificEnthalpy_psX "Return specific enthalpy from p, s, and X or Xi"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SpecificEntropy s "Specific entropy";
          input MassFraction X[:] "Mass fractions";
          input FixedPhase phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          output SpecificEnthalpy h "Specific enthalpy";
        algorithm
          h := specificEnthalpy(setState_psX(p, s, X, phase));
        end specificEnthalpy_psX;

        redeclare replaceable function setState_pT "Return thermodynamic state from p and T"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input Temperature T "Temperature";
          input FixedPhase phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          output ThermodynamicState state "Thermodynamic state record";
        algorithm
          state := setState_pTX(p, T, fill(0, 0), phase);
        end setState_pT;

        redeclare replaceable function setState_ph "Return thermodynamic state from p and h"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SpecificEnthalpy h "Specific enthalpy";
          input FixedPhase phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          output ThermodynamicState state "Thermodynamic state record";
        algorithm
          state := setState_phX(p, h, fill(0, 0), phase);
        end setState_ph;

        redeclare replaceable function setState_ps "Return thermodynamic state from p and s"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SpecificEntropy s "Specific entropy";
          input FixedPhase phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          output ThermodynamicState state "Thermodynamic state record";
        algorithm
          state := setState_psX(p, s, fill(0, 0), phase);
        end setState_ps;

        redeclare replaceable function setState_dT "Return thermodynamic state from d and T"
          extends Modelica.Icons.Function;
          input Density d "Density";
          input Temperature T "Temperature";
          input FixedPhase phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          output ThermodynamicState state "Thermodynamic state record";
        algorithm
          state := setState_dTX(d, T, fill(0, 0), phase);
        end setState_dT;

        replaceable function setState_px "Return thermodynamic state from pressure and vapour quality"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input MassFraction x "Vapour quality";
          output ThermodynamicState state "Thermodynamic state record";
        algorithm
          state := setState_ph(p, (1 - x) * bubbleEnthalpy(setSat_p(p)) + x * dewEnthalpy(setSat_p(p)), 2);
        end setState_px;

        replaceable function setState_Tx "Return thermodynamic state from temperature and vapour quality"
          extends Modelica.Icons.Function;
          input Temperature T "Temperature";
          input MassFraction x "Vapour quality";
          output ThermodynamicState state "Thermodynamic state record";
        algorithm
          state := setState_ph(saturationPressure_sat(setSat_T(T)), (1 - x) * bubbleEnthalpy(setSat_T(T)) + x * dewEnthalpy(setSat_T(T)), 2);
        end setState_Tx;

        replaceable function vapourQuality "Return vapour quality"
          extends Modelica.Icons.Function;
          input ThermodynamicState state "Thermodynamic state record";
          output MassFraction x "Vapour quality";
        protected
          constant SpecificEnthalpy eps = 1e-8;
        algorithm
          x := min(max((specificEnthalpy(state) - bubbleEnthalpy(setSat_p(pressure(state)))) / (dewEnthalpy(setSat_p(pressure(state))) - bubbleEnthalpy(setSat_p(pressure(state))) + eps), 0), 1);
        end vapourQuality;

        redeclare replaceable function density_ph "Return density from p and h"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SpecificEnthalpy h "Specific enthalpy";
          input FixedPhase phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          output Density d "Density";
        algorithm
          d := density_phX(p, h, fill(0, 0), phase);
        end density_ph;

        redeclare replaceable function temperature_ph "Return temperature from p and h"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SpecificEnthalpy h "Specific enthalpy";
          input FixedPhase phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          output Temperature T "Temperature";
        algorithm
          T := temperature_phX(p, h, fill(0, 0), phase);
        end temperature_ph;

        redeclare replaceable function pressure_dT "Return pressure from d and T"
          extends Modelica.Icons.Function;
          input Density d "Density";
          input Temperature T "Temperature";
          input FixedPhase phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          output AbsolutePressure p "Pressure";
        algorithm
          p := pressure(setState_dTX(d, T, fill(0, 0), phase));
        end pressure_dT;

        redeclare replaceable function specificEnthalpy_dT "Return specific enthalpy from d and T"
          extends Modelica.Icons.Function;
          input Density d "Density";
          input Temperature T "Temperature";
          input FixedPhase phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          output SpecificEnthalpy h "Specific enthalpy";
        algorithm
          h := specificEnthalpy(setState_dTX(d, T, fill(0, 0), phase));
        end specificEnthalpy_dT;

        redeclare replaceable function specificEnthalpy_ps "Return specific enthalpy from p and s"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SpecificEntropy s "Specific entropy";
          input FixedPhase phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          output SpecificEnthalpy h "Specific enthalpy";
        algorithm
          h := specificEnthalpy_psX(p, s, fill(0, 0));
        end specificEnthalpy_ps;

        redeclare replaceable function temperature_ps "Return temperature from p and s"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SpecificEntropy s "Specific entropy";
          input FixedPhase phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          output Temperature T "Temperature";
        algorithm
          T := temperature_psX(p, s, fill(0, 0), phase);
        end temperature_ps;

        redeclare replaceable function density_ps "Return density from p and s"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SpecificEntropy s "Specific entropy";
          input FixedPhase phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          output Density d "Density";
        algorithm
          d := density_psX(p, s, fill(0, 0), phase);
        end density_ps;

        redeclare replaceable function specificEnthalpy_pT "Return specific enthalpy from p and T"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input Temperature T "Temperature";
          input FixedPhase phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          output SpecificEnthalpy h "Specific enthalpy";
        algorithm
          h := specificEnthalpy_pTX(p, T, fill(0, 0), phase);
        end specificEnthalpy_pT;

        redeclare replaceable function density_pT "Return density from p and T"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input Temperature T "Temperature";
          input FixedPhase phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          output Density d "Density";
        algorithm
          d := density(setState_pTX(p, T, fill(0, 0), phase));
        end density_pT;
      end PartialTwoPhaseMedium;

      partial package PartialSimpleMedium "Medium model with linear dependency of u, h from temperature. All other quantities, especially density, are constant."
        extends Interfaces.PartialPureSubstance(final ThermoStates = Modelica.Media.Interfaces.Choices.IndependentVariables.pT, final singleState = true);
        constant SpecificHeatCapacity cp_const "Constant specific heat capacity at constant pressure";
        constant SpecificHeatCapacity cv_const "Constant specific heat capacity at constant volume";
        constant Density d_const "Constant density";
        constant DynamicViscosity eta_const "Constant dynamic viscosity";
        constant ThermalConductivity lambda_const "Constant thermal conductivity";
        constant VelocityOfSound a_const "Constant velocity of sound";
        constant Temperature T_min "Minimum temperature valid for medium model";
        constant Temperature T_max "Maximum temperature valid for medium model";
        constant Temperature T0 = reference_T "Zero enthalpy temperature";
        constant MolarMass MM_const "Molar mass";
        constant FluidConstants[nS] fluidConstants "Fluid constants";

        redeclare record extends ThermodynamicState "Thermodynamic state"
            AbsolutePressure p "Absolute pressure of medium";
            Temperature T "Temperature of medium";
        end ThermodynamicState;

        redeclare replaceable model extends BaseProperties(T(stateSelect = if preferredMediumStates then StateSelect.prefer else StateSelect.default), p(stateSelect = if preferredMediumStates then StateSelect.prefer else StateSelect.default)) "Base properties"
          equation
            assert(T >= T_min and T <= T_max, "
Temperature T (= " + String(T) + " K) is not
in the allowed range (" + String(T_min) + " K <= T <= " + String(T_max) + " K)
required from medium model \"" + mediumName + "\".
            ");
// h = cp_const*(T-T0);
            h = specificEnthalpy_pTX(p, T, X);
            u = cv_const * (T - T0);
            d = d_const;
            R = 0;
            MM = MM_const;
            state.T = T;
            state.p = p;
          annotation(
            Documentation(info = "<HTML>
<p>
This is the most simple incompressible medium model, where
specific enthalpy h and specific internal energy u are only
a function of temperature T and all other provided medium
quantities are assumed to be constant.
Note that the (small) influence of the pressure term p/d is neglected.
</p>
</HTML>"));
        end BaseProperties;

        redeclare function setState_pTX "Return thermodynamic state from p, T, and X or Xi"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input Temperature T "Temperature";
          input MassFraction X[:] = reference_X "Mass fractions";
          output ThermodynamicState state "Thermodynamic state record";
        algorithm
          state := ThermodynamicState(p = p, T = T);
        end setState_pTX;

        redeclare function setState_phX "Return thermodynamic state from p, h, and X or Xi"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SpecificEnthalpy h "Specific enthalpy";
          input MassFraction X[:] = reference_X "Mass fractions";
          output ThermodynamicState state "Thermodynamic state record";
        algorithm
          state := ThermodynamicState(p = p, T = T0 + h / cp_const);
        end setState_phX;

        redeclare replaceable function setState_psX "Return thermodynamic state from p, s, and X or Xi"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SpecificEntropy s "Specific entropy";
          input MassFraction X[:] = reference_X "Mass fractions";
          output ThermodynamicState state "Thermodynamic state record";
        algorithm
          state := ThermodynamicState(p = p, T = Modelica.Math.exp(s / cp_const + Modelica.Math.log(reference_T))) "Here the incompressible limit is used, with cp as heat capacity";
        end setState_psX;

        redeclare function setState_dTX "Return thermodynamic state from d, T, and X or Xi"
          extends Modelica.Icons.Function;
          input Density d "Density";
          input Temperature T "Temperature";
          input MassFraction X[:] = reference_X "Mass fractions";
          output ThermodynamicState state "Thermodynamic state record";
        algorithm
          assert(false, "Pressure can not be computed from temperature and density for an incompressible fluid!");
        end setState_dTX;

        redeclare function extends setSmoothState "Return thermodynamic state so that it smoothly approximates: if x > 0 then state_a else state_b"
          algorithm
            state := ThermodynamicState(p = Media.Common.smoothStep(x, state_a.p, state_b.p, x_small), T = Media.Common.smoothStep(x, state_a.T, state_b.T, x_small));
        end setSmoothState;

        redeclare function extends dynamicViscosity "Return dynamic viscosity"
          algorithm
            eta := eta_const;
        end dynamicViscosity;

        redeclare function extends thermalConductivity "Return thermal conductivity"
          algorithm
            lambda := lambda_const;
        end thermalConductivity;

        redeclare function extends pressure "Return pressure"
          algorithm
            p := state.p;
        end pressure;

        redeclare function extends temperature "Return temperature"
          algorithm
            T := state.T;
        end temperature;

        redeclare function extends density "Return density"
          algorithm
            d := d_const;
        end density;

        redeclare function extends specificEnthalpy "Return specific enthalpy"
          algorithm
            h := cp_const * (state.T - T0);
        end specificEnthalpy;

        redeclare function extends specificHeatCapacityCp "Return specific heat capacity at constant pressure"
          algorithm
            cp := cp_const;
        end specificHeatCapacityCp;

        redeclare function extends specificHeatCapacityCv "Return specific heat capacity at constant volume"
          algorithm
            cv := cv_const;
        end specificHeatCapacityCv;

        redeclare function extends isentropicExponent "Return isentropic exponent"
          algorithm
            gamma := cp_const / cv_const;
        end isentropicExponent;

        redeclare function extends velocityOfSound "Return velocity of sound"
          algorithm
            a := a_const;
        end velocityOfSound;

        redeclare function specificEnthalpy_pTX "Return specific enthalpy from p, T, and X or Xi"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input Temperature T "Temperature";
          input MassFraction X[nX] "Mass fractions";
          output SpecificEnthalpy h "Specific enthalpy";
        algorithm
          h := cp_const * (T - T0);
          annotation(
            Documentation(info = "<html>
<p>
This function computes the specific enthalpy of the fluid, but neglects the (small) influence of the pressure term p/d.
</p>
</html>"));
        end specificEnthalpy_pTX;

        redeclare function temperature_phX "Return temperature from p, h, and X or Xi"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SpecificEnthalpy h "Specific enthalpy";
          input MassFraction X[nX] "Mass fractions";
          output Temperature T "Temperature";
        algorithm
          T := T0 + h / cp_const;
        end temperature_phX;

        redeclare function density_phX "Return density from p, h, and X or Xi"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SpecificEnthalpy h "Specific enthalpy";
          input MassFraction X[nX] "Mass fractions";
          output Density d "Density";
        algorithm
          d := density(setState_phX(p, h, X));
        end density_phX;

        redeclare function extends specificInternalEnergy "Return specific internal energy"
            extends Modelica.Icons.Function;

          algorithm
//  u := cv_const*(state.T - T0) - reference_p/d_const;
            u := cv_const * (state.T - T0);
          annotation(
            Documentation(info = "<html>
<p>
This function computes the specific internal energy of the fluid, but neglects the (small) influence of the pressure term p/d.
</p>
</html>"));
        end specificInternalEnergy;

        redeclare function extends specificEntropy "Return specific entropy"
            extends Modelica.Icons.Function;

          algorithm
            s := cv_const * Modelica.Math.log(state.T / T0);
        end specificEntropy;

        redeclare function extends specificGibbsEnergy "Return specific Gibbs energy"
            extends Modelica.Icons.Function;

          algorithm
            g := specificEnthalpy(state) - state.T * specificEntropy(state);
        end specificGibbsEnergy;

        redeclare function extends specificHelmholtzEnergy "Return specific Helmholtz energy"
            extends Modelica.Icons.Function;

          algorithm
            f := specificInternalEnergy(state) - state.T * specificEntropy(state);
        end specificHelmholtzEnergy;

        redeclare function extends isentropicEnthalpy "Return isentropic enthalpy"
          algorithm
            h_is := cp_const * (temperature(refState) - T0);
        end isentropicEnthalpy;

        redeclare function extends isobaricExpansionCoefficient "Returns overall the isobaric expansion coefficient beta"
          algorithm
            beta := 0.0;
        end isobaricExpansionCoefficient;

        redeclare function extends isothermalCompressibility "Returns overall the isothermal compressibility factor"
          algorithm
            kappa := 0;
        end isothermalCompressibility;

        redeclare function extends density_derp_T "Returns the partial derivative of density with respect to pressure at constant temperature"
          algorithm
            ddpT := 0;
        end density_derp_T;

        redeclare function extends density_derT_p "Returns the partial derivative of density with respect to temperature at constant pressure"
          algorithm
            ddTp := 0;
        end density_derT_p;

        redeclare function extends density_derX "Returns the partial derivative of density with respect to mass fractions at constant pressure and temperature"
          algorithm
            dddX := fill(0, nX);
        end density_derX;

        redeclare function extends molarMass "Return the molar mass of the medium"
          algorithm
            MM := MM_const;
        end molarMass;
      end PartialSimpleMedium;

      partial package PartialSimpleIdealGasMedium "Medium model of Ideal gas with constant cp and cv. All other quantities, e.g., transport properties, are constant."
        extends Interfaces.PartialPureSubstance(redeclare replaceable record FluidConstants = Modelica.Media.Interfaces.Types.Basic.FluidConstants, ThermoStates = Modelica.Media.Interfaces.Choices.IndependentVariables.pT, final singleState = false);
        constant SpecificHeatCapacity cp_const "Constant specific heat capacity at constant pressure";
        constant SpecificHeatCapacity cv_const = cp_const - R_gas "Constant specific heat capacity at constant volume";
        constant SpecificHeatCapacity R_gas "Medium specific gas constant";
        constant MolarMass MM_const "Molar mass";
        constant DynamicViscosity eta_const "Constant dynamic viscosity";
        constant ThermalConductivity lambda_const "Constant thermal conductivity";
        constant Temperature T_min "Minimum temperature valid for medium model";
        constant Temperature T_max "Maximum temperature valid for medium model";
        constant Temperature T0 = reference_T "Zero enthalpy temperature";
        constant FluidConstants[nS] fluidConstants "Fluid constants";

        redeclare record extends ThermodynamicState "Thermodynamic state of ideal gas"
            AbsolutePressure p "Absolute pressure of medium";
            Temperature T "Temperature of medium";
        end ThermodynamicState;

        redeclare replaceable model extends BaseProperties(T(stateSelect = if preferredMediumStates then StateSelect.prefer else StateSelect.default), p(stateSelect = if preferredMediumStates then StateSelect.prefer else StateSelect.default)) "Base properties of ideal gas"
          equation
            assert(T >= T_min and T <= T_max, "
Temperature T (= " + String(T) + " K) is not
in the allowed range (" + String(T_min) + " K <= T <= " + String(T_max) + " K)
required from medium model \"" + mediumName + "\".
            ");
            h = specificEnthalpy_pTX(p, T, X);
            u = h - R * T;
            R = R_gas;
            d = p / (R * T);
            MM = MM_const;
            state.T = T;
            state.p = p;
          annotation(
            Documentation(info = "<HTML>
<p>
This is the most simple incompressible medium model, where
specific enthalpy h and specific internal energy u are only
a function of temperature T and all other provided medium
quantities are assumed to be constant.
</p>
</HTML>"));
        end BaseProperties;

        redeclare function setState_pTX "Return thermodynamic state from p, T, and X or Xi"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input Temperature T "Temperature";
          input MassFraction X[:] = reference_X "Mass fractions";
          output ThermodynamicState state "Thermodynamic state record";
        algorithm
          state := ThermodynamicState(p = p, T = T);
        end setState_pTX;

        redeclare function setState_phX "Return thermodynamic state from p, h, and X or Xi"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SpecificEnthalpy h "Specific enthalpy";
          input MassFraction X[:] = reference_X "Mass fractions";
          output ThermodynamicState state "Thermodynamic state record";
        algorithm
          state := ThermodynamicState(p = p, T = T0 + h / cp_const);
        end setState_phX;

        redeclare replaceable function setState_psX "Return thermodynamic state from p, s, and X or Xi"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SpecificEntropy s "Specific entropy";
          input MassFraction X[:] = reference_X "Mass fractions";
          output ThermodynamicState state "Thermodynamic state record";
        algorithm
          state := ThermodynamicState(p = p, T = Modelica.Math.exp(s / cp_const + Modelica.Math.log(reference_T) + R_gas * Modelica.Math.log(p / reference_p)));
        end setState_psX;

        redeclare function setState_dTX "Return thermodynamic state from d, T, and X or Xi"
          extends Modelica.Icons.Function;
          input Density d "Density";
          input Temperature T "Temperature";
          input MassFraction X[:] = reference_X "Mass fractions";
          output ThermodynamicState state "Thermodynamic state record";
        algorithm
          state := ThermodynamicState(p = d * R_gas * T, T = T);
        end setState_dTX;

        redeclare function extends setSmoothState "Return thermodynamic state so that it smoothly approximates: if x > 0 then state_a else state_b"
          algorithm
            state := ThermodynamicState(p = Media.Common.smoothStep(x, state_a.p, state_b.p, x_small), T = Media.Common.smoothStep(x, state_a.T, state_b.T, x_small));
        end setSmoothState;

        redeclare function extends pressure "Return pressure of ideal gas"
          algorithm
            p := state.p;
        end pressure;

        redeclare function extends temperature "Return temperature of ideal gas"
          algorithm
            T := state.T;
        end temperature;

        redeclare function extends density "Return density of ideal gas"
          algorithm
            d := state.p / (R_gas * state.T);
        end density;

        redeclare function extends specificEnthalpy "Return specific enthalpy"
            extends Modelica.Icons.Function;

          algorithm
            h := cp_const * (state.T - T0);
        end specificEnthalpy;

        redeclare function extends specificInternalEnergy "Return specific internal energy"
            extends Modelica.Icons.Function;

          algorithm
            u := cp_const * (state.T - T0) - R_gas * state.T;
        end specificInternalEnergy;

        redeclare function extends specificEntropy "Return specific entropy"
            extends Modelica.Icons.Function;

          algorithm
            s := cp_const * Modelica.Math.log(state.T / T0) - R_gas * Modelica.Math.log(state.p / reference_p);
        end specificEntropy;

        redeclare function extends specificGibbsEnergy "Return specific Gibbs energy"
            extends Modelica.Icons.Function;

          algorithm
            g := cp_const * (state.T - T0) - state.T * specificEntropy(state);
        end specificGibbsEnergy;

        redeclare function extends specificHelmholtzEnergy "Return specific Helmholtz energy"
            extends Modelica.Icons.Function;

          algorithm
            f := specificInternalEnergy(state) - state.T * specificEntropy(state);
        end specificHelmholtzEnergy;

        redeclare function extends dynamicViscosity "Return dynamic viscosity"
          algorithm
            eta := eta_const;
        end dynamicViscosity;

        redeclare function extends thermalConductivity "Return thermal conductivity"
          algorithm
            lambda := lambda_const;
        end thermalConductivity;

        redeclare function extends specificHeatCapacityCp "Return specific heat capacity at constant pressure"
          algorithm
            cp := cp_const;
        end specificHeatCapacityCp;

        redeclare function extends specificHeatCapacityCv "Return specific heat capacity at constant volume"
          algorithm
            cv := cv_const;
        end specificHeatCapacityCv;

        redeclare function extends isentropicExponent "Return isentropic exponent"
          algorithm
            gamma := cp_const / cv_const;
        end isentropicExponent;

        redeclare function extends velocityOfSound "Return velocity of sound"
          algorithm
            a := sqrt(cp_const / cv_const * R_gas * state.T);
        end velocityOfSound;

        redeclare function specificEnthalpy_pTX "Return specific enthalpy from p, T, and X or Xi"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input Temperature T "Temperature";
          input MassFraction X[nX] "Mass fractions";
          output SpecificEnthalpy h "Specific enthalpy at p, T, X";
        algorithm
          h := cp_const * (T - T0);
        end specificEnthalpy_pTX;

        redeclare function temperature_phX "Return temperature from p, h, and X or Xi"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SpecificEnthalpy h "Specific enthalpy";
          input MassFraction X[nX] "Mass fractions";
          output Temperature T "Temperature";
        algorithm
          T := h / cp_const + T0;
        end temperature_phX;

        redeclare function density_phX "Return density from p, h, and X or Xi"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SpecificEnthalpy h "Specific enthalpy";
          input MassFraction X[nX] "Mass fractions";
          output Density d "Density";
        algorithm
          d := density(setState_phX(p, h, X));
        end density_phX;

        redeclare function extends isentropicEnthalpy "Return isentropic enthalpy"
          algorithm
/*  s = cp_const*log(refState.T/T0) - R_gas*log(refState.p/reference_p)
          = cp_const*log(state.T/T0) - R_gas*log(p_downstream/reference_p)

        log(state.T) = log(refState.T) +
                       (R_gas/cp_const)*(log(p_downstream/reference_p) - log(refState.p/reference_p))
                     = log(refState.T) + (R_gas/cp_const)*log(p_downstream/refState.p)
                     = log(refState.T) + log( (p_downstream/refState.p)^(R_gas/cp_const) )
                     = log( refState.T*(p_downstream/refState.p)^(R_gas/cp_const) )
        state.T = refstate.T*(p_downstream/refstate.p)^(R_gas/cp_const)
    */
            h_is := cp_const * (refState.T * (p_downstream / refState.p) ^ (R_gas / cp_const) - T0);
        end isentropicEnthalpy;

        redeclare function extends isobaricExpansionCoefficient "Returns overall the isobaric expansion coefficient beta"
          algorithm
/* beta = 1/v * der(v,T), with v = 1/d, at constant pressure p:
       v = R*T/p
       der(v,T) = R/p
       beta = p/(R*T)*R/p
            = 1/T
    */
            beta := 1 / state.T;
        end isobaricExpansionCoefficient;

        redeclare function extends isothermalCompressibility "Returns overall the isothermal compressibility factor"
          algorithm
/* kappa = - 1/v * der(v,p), with v = 1/d at constant temperature T.
       v = R*T/p
       der(v,T) = -R*T/p^2
       kappa = p/(R*T)*R*T/p^2
             = 1/p
    */
            kappa := 1 / state.p;
        end isothermalCompressibility;

        redeclare function extends density_derp_T "Returns the partial derivative of density with respect to pressure at constant temperature"
          algorithm
/*  d = p/(R*T)
        ddpT = 1/(R*T)
    */
            ddpT := 1 / (R_gas * state.T);
        end density_derp_T;

        redeclare function extends density_derT_p "Returns the partial derivative of density with respect to temperature at constant pressure"
          algorithm
/*  d = p/(R*T)
        ddpT = -p/(R*T^2)
    */
            ddTp := -state.p / (R_gas * state.T * state.T);
        end density_derT_p;

        redeclare function extends density_derX "Returns the partial derivative of density with respect to mass fractions at constant pressure and temperature"
          algorithm
            dddX := fill(0, nX);
        end density_derX;

        redeclare function extends molarMass "Returns the molar mass of the medium"
          algorithm
            MM := MM_const;
        end molarMass;
      end PartialSimpleIdealGasMedium;

      package Choices "Types, constants to define menu choices"
        extends Modelica.Icons.Package;
        type IndependentVariables = enumeration(T "Temperature", pT "Pressure, Temperature", ph "Pressure, Specific Enthalpy", phX "Pressure, Specific Enthalpy, Mass Fraction", pTX "Pressure, Temperature, Mass Fractions", dTX "Density, Temperature, Mass Fractions") "Enumeration defining the independent variables of a medium";
        type Init = enumeration(NoInit "NoInit (no initialization)", InitialStates "InitialStates (initialize medium states)", SteadyState "SteadyState (initialize in steady state)", SteadyMass "SteadyMass (initialize density or pressure in steady state)") "Enumeration defining initialization for fluid flow" annotation(
          Evaluate = true);
        type ReferenceEnthalpy = enumeration(ZeroAt0K "The enthalpy is 0 at 0 K (default), if the enthalpy of formation is excluded", ZeroAt25C "The enthalpy is 0 at 25 degC, if the enthalpy of formation is excluded", UserDefined "The user-defined reference enthalpy is used at 293.15 K (25 degC)") "Enumeration defining the reference enthalpy of a medium" annotation(
          Evaluate = true);
        type ReferenceEntropy = enumeration(ZeroAt0K "The entropy is 0 at 0 K (default)", ZeroAt0C "The entropy is 0 at 0 degC", UserDefined "The user-defined reference entropy is used at 293.15 K (25 degC)") "Enumeration defining the reference entropy of a medium" annotation(
          Evaluate = true);
        type pd = enumeration(default "Default (no boundary condition for p or d)", p_known "p_known (pressure p is known)", d_known "d_known (density d is known)") "Enumeration defining whether p or d are known for the boundary condition" annotation(
          Evaluate = true);
        type Th = enumeration(default "Default (no boundary condition for T or h)", T_known "T_known (temperature T is known)", h_known "h_known (specific enthalpy h is known)") "Enumeration defining whether T or h are known as boundary condition" annotation(
          Evaluate = true);
        annotation(
          Documentation(info = "<html>
<p>
Enumerations and data types for all types of fluids
</p>

<p>
Note: Reference enthalpy might have to be extended with enthalpy of formation.
</p>
</html>"));
      end Choices;

      package Types "Types to be used in fluid models"
        extends Modelica.Icons.Package;
        type AbsolutePressure = SI.AbsolutePressure(min = 0, max = 1.e8, nominal = 1.e5, start = 1.e5) "Type for absolute pressure with medium specific attributes";
        type Density = SI.Density(min = 0, max = 1.e5, nominal = 1, start = 1) "Type for density with medium specific attributes";
        type DynamicViscosity = SI.DynamicViscosity(min = 0, max = 1.e8, nominal = 1.e-3, start = 1.e-3) "Type for dynamic viscosity with medium specific attributes";
        type EnthalpyFlowRate = SI.EnthalpyFlowRate(nominal = 1000.0, min = -1.0e8, max = 1.e8) "Type for enthalpy flow rate with medium specific attributes";
        type MassFraction = Real(quantity = "MassFraction", final unit = "kg/kg", min = 0, max = 1, nominal = 0.1) "Type for mass fraction with medium specific attributes";
        type MoleFraction = Real(quantity = "MoleFraction", final unit = "mol/mol", min = 0, max = 1, nominal = 0.1) "Type for mole fraction with medium specific attributes";
        type MolarMass = SI.MolarMass(min = 0.001, max = 0.25, nominal = 0.032) "Type for molar mass with medium specific attributes";
        type MolarVolume = SI.MolarVolume(min = 1e-6, max = 1.0e6, nominal = 1.0) "Type for molar volume with medium specific attributes";
        type IsentropicExponent = SI.RatioOfSpecificHeatCapacities(min = 1, max = 500000, nominal = 1.2, start = 1.2) "Type for isentropic exponent with medium specific attributes";
        type SpecificEnergy = SI.SpecificEnergy(min = -1.0e8, max = 1.e8, nominal = 1.e6) "Type for specific energy with medium specific attributes";
        type SpecificInternalEnergy = SpecificEnergy "Type for specific internal energy with medium specific attributes";
        type SpecificEnthalpy = SI.SpecificEnthalpy(min = -1.0e10, max = 1.e10, nominal = 1.e6) "Type for specific enthalpy with medium specific attributes";
        type SpecificEntropy = SI.SpecificEntropy(min = -1.e7, max = 1.e7, nominal = 1.e3) "Type for specific entropy with medium specific attributes";
        type SpecificHeatCapacity = SI.SpecificHeatCapacity(min = 0, max = 1.e7, nominal = 1.e3, start = 1.e3) "Type for specific heat capacity with medium specific attributes";
        type SurfaceTension = SI.SurfaceTension "Type for surface tension with medium specific attributes";
        type Temperature = SI.Temperature(min = 1, max = 1.e4, nominal = 300, start = 300) "Type for temperature with medium specific attributes";
        type ThermalConductivity = SI.ThermalConductivity(min = 0, max = 500, nominal = 1, start = 1) "Type for thermal conductivity with medium specific attributes";
        type PrandtlNumber = SI.PrandtlNumber(min = 1e-3, max = 1e5, nominal = 1.0) "Type for Prandtl number with medium specific attributes";
        type VelocityOfSound = SI.Velocity(min = 0, max = 1.e5, nominal = 1000, start = 1000) "Type for velocity of sound with medium specific attributes";
        type ExtraProperty = Real(min = 0.0, start = 1.0) "Type for unspecified, mass-specific property transported by flow";
        type CumulativeExtraProperty = Real(min = 0.0, start = 1.0) "Type for conserved integral of unspecified, mass specific property";
        type ExtraPropertyFlowRate = Real(unit = "kg/s") "Type for flow rate of unspecified, mass-specific property";
        type IsobaricExpansionCoefficient = Real(min = 0, max = 1.0e8, unit = "1/K") "Type for isobaric expansion coefficient with medium specific attributes";
        type DipoleMoment = Real(min = 0.0, max = 2.0, unit = "debye", quantity = "ElectricDipoleMoment") "Type for dipole moment with medium specific attributes";
        type DerDensityByPressure = SI.DerDensityByPressure "Type for partial derivative of density with respect to pressure with medium specific attributes";
        type DerDensityByEnthalpy = SI.DerDensityByEnthalpy "Type for partial derivative of density with respect to enthalpy with medium specific attributes";
        type DerEnthalpyByPressure = SI.DerEnthalpyByPressure "Type for partial derivative of enthalpy with respect to pressure with medium specific attributes";
        type DerDensityByTemperature = SI.DerDensityByTemperature "Type for partial derivative of density with respect to temperature with medium specific attributes";
        type DerTemperatureByPressure = Real(final unit = "K/Pa") "Type for partial derivative of temperature with respect to pressure with medium specific attributes";

        replaceable record SaturationProperties "Saturation properties of two phase medium"
          extends Modelica.Icons.Record;
          AbsolutePressure psat "Saturation pressure";
          Temperature Tsat "Saturation temperature";
        end SaturationProperties;

        record FluidLimits "Validity limits for fluid model"
          extends Modelica.Icons.Record;
          Temperature TMIN "Minimum temperature";
          Temperature TMAX "Maximum temperature";
          Density DMIN "Minimum density";
          Density DMAX "Maximum density";
          AbsolutePressure PMIN "Minimum pressure";
          AbsolutePressure PMAX "Maximum pressure";
          SpecificEnthalpy HMIN "Minimum enthalpy";
          SpecificEnthalpy HMAX "Maximum enthalpy";
          SpecificEntropy SMIN "Minimum entropy";
          SpecificEntropy SMAX "Maximum entropy";
          annotation(
            Documentation(info = "<html>
          <p>The minimum pressure mostly applies to the liquid state only.
          The minimum density is also arbitrary, but is reasonable for technical
          applications to limit iterations in non-linear systems. The limits in
          enthalpy and entropy are used as safeguards in inverse iterations.</p>
          </html>"));
        end FluidLimits;

        type FixedPhase = Integer(min = 0, max = 2) "Phase of the fluid: 1 for 1-phase, 2 for two-phase, 0 for not known, e.g., interactive use";

        package Basic "The most basic version of a record used in several degrees of detail"
          extends Icons.Package;

          record FluidConstants "Critical, triple, molecular and other standard data of fluid"
            extends Modelica.Icons.Record;
            String iupacName "Complete IUPAC name (or common name, if non-existent)";
            String casRegistryNumber "Chemical abstracts sequencing number (if it exists)";
            String chemicalFormula "Chemical formula, (brutto, nomenclature according to Hill";
            String structureFormula "Chemical structure formula";
            MolarMass molarMass "Molar mass";
          end FluidConstants;
        end Basic;

        package IdealGas "The ideal gas version of a record used in several degrees of detail"
          extends Icons.Package;

          record FluidConstants "Extended fluid constants"
            extends Modelica.Media.Interfaces.Types.Basic.FluidConstants;
            Temperature criticalTemperature "Critical temperature";
            AbsolutePressure criticalPressure "Critical pressure";
            MolarVolume criticalMolarVolume "Critical molar Volume";
            Real acentricFactor "Pitzer acentric factor";
            //   Temperature triplePointTemperature "Triple point temperature";
            //   AbsolutePressure triplePointPressure "Triple point pressure";
            Temperature meltingPoint "Melting point at 101325 Pa";
            Temperature normalBoilingPoint "Normal boiling point (at 101325 Pa)";
            DipoleMoment dipoleMoment "Dipole moment of molecule in Debye (1 debye = 3.33564e10-30 C.m)";
            Boolean hasIdealGasHeatCapacity = false "True if ideal gas heat capacity is available";
            Boolean hasCriticalData = false "True if critical data are known";
            Boolean hasDipoleMoment = false "True if a dipole moment known";
            Boolean hasFundamentalEquation = false "True if a fundamental equation";
            Boolean hasLiquidHeatCapacity = false "True if liquid heat capacity is available";
            Boolean hasSolidHeatCapacity = false "True if solid heat capacity is available";
            Boolean hasAccurateViscosityData = false "True if accurate data for a viscosity function is available";
            Boolean hasAccurateConductivityData = false "True if accurate data for thermal conductivity is available";
            Boolean hasVapourPressureCurve = false "True if vapour pressure data, e.g., Antoine coefficents are known";
            Boolean hasAcentricFactor = false "True if Pitzer accentric factor is known";
            SpecificEnthalpy HCRIT0 = 0.0 "Critical specific enthalpy of the fundamental equation";
            SpecificEntropy SCRIT0 = 0.0 "Critical specific entropy of the fundamental equation";
            SpecificEnthalpy deltah = 0.0 "Difference between specific enthalpy model (h_m) and f.eq. (h_f) (h_m - h_f)";
            SpecificEntropy deltas = 0.0 "Difference between specific enthalpy model (s_m) and f.eq. (s_f) (s_m - s_f)";
          end FluidConstants;
        end IdealGas;

        package TwoPhase "The two phase fluid version of a record used in several degrees of detail"
          extends Icons.Package;

          record FluidConstants "Extended fluid constants"
            extends Modelica.Media.Interfaces.Types.Basic.FluidConstants;
            Temperature criticalTemperature "Critical temperature";
            AbsolutePressure criticalPressure "Critical pressure";
            MolarVolume criticalMolarVolume "Critical molar Volume";
            Real acentricFactor "Pitzer acentric factor";
            Temperature triplePointTemperature "Triple point temperature";
            AbsolutePressure triplePointPressure "Triple point pressure";
            Temperature meltingPoint "Melting point at 101325 Pa";
            Temperature normalBoilingPoint "Normal boiling point (at 101325 Pa)";
            DipoleMoment dipoleMoment "Dipole moment of molecule in Debye (1 debye = 3.33564e10-30 C.m)";
            Boolean hasIdealGasHeatCapacity = false "True if ideal gas heat capacity is available";
            Boolean hasCriticalData = false "True if critical data are known";
            Boolean hasDipoleMoment = false "True if a dipole moment known";
            Boolean hasFundamentalEquation = false "True if a fundamental equation";
            Boolean hasLiquidHeatCapacity = false "True if liquid heat capacity is available";
            Boolean hasSolidHeatCapacity = false "True if solid heat capacity is available";
            Boolean hasAccurateViscosityData = false "True if accurate data for a viscosity function is available";
            Boolean hasAccurateConductivityData = false "True if accurate data for thermal conductivity is available";
            Boolean hasVapourPressureCurve = false "True if vapour pressure data, e.g., Antoine coefficents are known";
            Boolean hasAcentricFactor = false "True if Pitzer accentric factor is known";
            SpecificEnthalpy HCRIT0 = 0.0 "Critical specific enthalpy of the fundamental equation";
            SpecificEntropy SCRIT0 = 0.0 "Critical specific entropy of the fundamental equation";
            SpecificEnthalpy deltah = 0.0 "Difference between specific enthalpy model (h_m) and f.eq. (h_f) (h_m - h_f)";
            SpecificEntropy deltas = 0.0 "Difference between specific enthalpy model (s_m) and f.eq. (s_f) (s_m - s_f)";
          end FluidConstants;
        end TwoPhase;
      end Types;
      annotation(
        Documentation(info = "<HTML>
<p>
This package provides basic interfaces definitions of media models for different
kind of media.
</p>
</HTML>"));
    end Interfaces;

    package Common "Data structures and fundamental functions for fluid properties"
      extends Modelica.Icons.Package;
      type Rate = Real(final quantity = "Rate", final unit = "s-1");
      type MolarFlowRate = Real(final quantity = "MolarFlowRate", final unit = "mol/s");
      type MolarReactionRate = Real(final quantity = "MolarReactionRate", final unit = "mol/(m3.s)");
      type MolarEnthalpy = Real(final quantity = "MolarEnthalpy", final unit = "J/mol");
      type DerDensityByEntropy = Real(final quantity = "DerDensityByEntropy", final unit = "kg2.K/(m3.J)");
      type DerEnergyByPressure = Real(final quantity = "DerEnergyByPressure", final unit = "J/Pa");
      type DerEnergyByMoles = Real(final quantity = "DerEnergyByMoles", final unit = "J/mol");
      type DerEntropyByTemperature = Real(final quantity = "DerEntropyByTemperature", final unit = "J/K2");
      type DerEntropyByPressure = Real(final quantity = "DerEntropyByPressure", final unit = "J/(K.Pa)");
      type DerEntropyByMoles = Real(final quantity = "DerEntropyByMoles", final unit = "J/(mol.K)");
      type DerPressureByDensity = Real(final quantity = "DerPressureByDensity", final unit = "Pa.m3/kg");
      type DerPressureBySpecificVolume = Real(final quantity = "DerPressureBySpecificVolume", final unit = "Pa.kg/m3");
      type DerPressureByTemperature = Real(final quantity = "DerPressureByTemperature", final unit = "Pa/K");
      type DerVolumeByTemperature = Real(final quantity = "DerVolumeByTemperature", final unit = "m3/K");
      type DerVolumeByPressure = Real(final quantity = "DerVolumeByPressure", final unit = "m3/Pa");
      type DerVolumeByMoles = Real(final quantity = "DerVolumeByMoles", final unit = "m3/mol");
      type IsenthalpicExponent = Real(final quantity = "IsenthalpicExponent", unit = "1");
      type IsentropicExponent = Real(final quantity = "IsentropicExponent", unit = "1");
      type IsobaricVolumeExpansionCoefficient = Real(final quantity = "IsobaricVolumeExpansionCoefficient", unit = "1/K");
      type IsochoricPressureCoefficient = Real(final quantity = "IsochoricPressureCoefficient", unit = "1/K");
      type IsothermalCompressibility = Real(final quantity = "IsothermalCompressibility", unit = "1/Pa");
      type JouleThomsonCoefficient = Real(final quantity = "JouleThomsonCoefficient", unit = "K/Pa");
      // introduce min-manx-nominal values
      constant Real MINPOS = 1.0e-9 "Minimal value for physical variables which are always > 0.0";
      constant SI.Area AMIN = MINPOS "Minimal init area";
      constant SI.Area AMAX = 1.0e5 "Maximal init area";
      constant SI.Area ANOM = 1.0 "Nominal init area";
      constant SI.AmountOfSubstance MOLMIN = -1.0 * MINPOS "Minimal Mole Number";
      constant SI.AmountOfSubstance MOLMAX = 1.0e8 "Maximal Mole Number";
      constant SI.AmountOfSubstance MOLNOM = 1.0 "Nominal Mole Number";
      constant SI.Density DMIN = 1e-6 "Minimal init density";
      constant SI.Density DMAX = 30.0e3 "Maximal init density";
      constant SI.Density DNOM = 1.0 "Nominal init density";
      constant SI.ThermalConductivity LAMMIN = MINPOS "Minimal thermal conductivity";
      constant SI.ThermalConductivity LAMNOM = 1.0 "Nominal thermal conductivity";
      constant SI.ThermalConductivity LAMMAX = 1000.0 "Maximal thermal conductivity";
      constant SI.DynamicViscosity ETAMIN = MINPOS "Minimal init dynamic viscosity";
      constant SI.DynamicViscosity ETAMAX = 1.0e8 "Maximal init dynamic viscosity";
      constant SI.DynamicViscosity ETANOM = 100.0 "Nominal init dynamic viscosity";
      constant SI.Energy EMIN = -1.0e10 "Minimal init energy";
      constant SI.Energy EMAX = 1.0e10 "Maximal init energy";
      constant SI.Energy ENOM = 1.0e3 "Nominal init energy";
      constant SI.Entropy SMIN = -1.0e6 "Minimal init entropy";
      constant SI.Entropy SMAX = 1.0e6 "Maximal init entropy";
      constant SI.Entropy SNOM = 1.0e3 "Nominal init entropy";
      constant SI.MassFlowRate MDOTMIN = -1.0e5 "Minimal init mass flow rate";
      constant SI.MassFlowRate MDOTMAX = 1.0e5 "Maximal init mass flow rate";
      constant SI.MassFlowRate MDOTNOM = 1.0 "Nominal init mass flow rate";
      constant SI.MassFraction MASSXMIN = -1.0 * MINPOS "Minimal init mass fraction";
      constant SI.MassFraction MASSXMAX = 1.0 "Maximal init mass fraction";
      constant SI.MassFraction MASSXNOM = 0.1 "Nominal init mass fraction";
      constant SI.Mass MMIN = -1.0 * MINPOS "Minimal init mass";
      constant SI.Mass MMAX = 1.0e8 "Maximal init mass";
      constant SI.Mass MNOM = 1.0 "Nominal init mass";
      constant SI.MolarMass MMMIN = 0.001 "Minimal initial molar mass";
      constant SI.MolarMass MMMAX = 250.0 "Maximal initial molar mass";
      constant SI.MolarMass MMNOM = 0.2 "Nominal initial molar mass";
      constant SI.MoleFraction MOLEYMIN = -1.0 * MINPOS "Minimal init mole fraction";
      constant SI.MoleFraction MOLEYMAX = 1.0 "Maximal init mole fraction";
      constant SI.MoleFraction MOLEYNOM = 0.1 "Nominal init mole fraction";
      constant SI.MomentumFlux GMIN = -1.0e8 "Minimal init momentum flux";
      constant SI.MomentumFlux GMAX = 1.0e8 "Maximal init momentum flux";
      constant SI.MomentumFlux GNOM = 1.0 "Nominal init momentum flux";
      constant SI.Power POWMIN = -1.0e8 "Minimal init power or heat";
      constant SI.Power POWMAX = 1.0e8 "Maximal init power or heat";
      constant SI.Power POWNOM = 1.0e3 "Nominal init power or heat";
      constant SI.Pressure PMIN = 1.0e4 "Minimal init pressure";
      constant SI.Pressure PMAX = 1.0e8 "Maximal init pressure";
      constant SI.Pressure PNOM = 1.0e5 "Nominal init pressure";
      constant SI.Pressure COMPPMIN = -1.0 * MINPOS "Minimal init pressure";
      constant SI.Pressure COMPPMAX = 1.0e8 "Maximal init pressure";
      constant SI.Pressure COMPPNOM = 1.0e5 "Nominal init pressure";
      constant SI.RatioOfSpecificHeatCapacities KAPPAMIN = 1.0 "Minimal init isentropic exponent";
      constant SI.RatioOfSpecificHeatCapacities KAPPAMAX = 1.7 "Maximal init isentropic exponent";
      constant SI.RatioOfSpecificHeatCapacities KAPPANOM = 1.2 "Nominal init isentropic exponent";
      constant SI.SpecificEnergy SEMIN = -1.0e8 "Minimal init specific energy";
      constant SI.SpecificEnergy SEMAX = 1.0e8 "Maximal init specific energy";
      constant SI.SpecificEnergy SENOM = 1.0e6 "Nominal init specific energy";
      constant SI.SpecificEnthalpy SHMIN = -1.0e8 "Minimal init specific enthalpy";
      constant SI.SpecificEnthalpy SHMAX = 1.0e8 "Maximal init specific enthalpy";
      constant SI.SpecificEnthalpy SHNOM = 1.0e6 "Nominal init specific enthalpy";
      constant SI.SpecificEntropy SSMIN = -1.0e6 "Minimal init specific entropy";
      constant SI.SpecificEntropy SSMAX = 1.0e6 "Maximal init specific entropy";
      constant SI.SpecificEntropy SSNOM = 1.0e3 "Nominal init specific entropy";
      constant SI.SpecificHeatCapacity CPMIN = MINPOS "Minimal init specific heat capacity";
      constant SI.SpecificHeatCapacity CPMAX = 1.0e6 "Maximal init specific heat capacity";
      constant SI.SpecificHeatCapacity CPNOM = 1.0e3 "Nominal init specific heat capacity";
      constant SI.Temperature TMIN = 1.0 "Minimal init temperature";
      constant SI.Temperature TMAX = 6000.0 "Maximal init temperature";
      constant SI.Temperature TNOM = 320.0 "Nominal init temperature";
      constant SI.ThermalConductivity LMIN = MINPOS "Minimal init thermal conductivity";
      constant SI.ThermalConductivity LMAX = 500.0 "Maximal init thermal conductivity";
      constant SI.ThermalConductivity LNOM = 1.0 "Nominal init thermal conductivity";
      constant SI.Velocity VELMIN = -1.0e5 "Minimal init speed";
      constant SI.Velocity VELMAX = 1.0e5 "Maximal init speed";
      constant SI.Velocity VELNOM = 1.0 "Nominal init speed";
      constant SI.Volume VMIN = 0.0 "Minimal init volume";
      constant SI.Volume VMAX = 1.0e5 "Maximal init volume";
      constant SI.Volume VNOM = 1.0e-3 "Nominal init volume";

      package ThermoFluidSpecial "Property records used by the ThermoFluid library"
        record FixedIGProperties "Constant properties for ideal gases"
          extends Modelica.Icons.Record;
          parameter Integer nspecies(min = 1) "Number of components";
          SI.MolarMass[nspecies] MM "Molar mass of components";
          Real[nspecies] invMM "Inverse of molar mass of components";
          SI.SpecificHeatCapacity[nspecies] R "Gas constant";
          SI.SpecificEnthalpy[nspecies] Hf "Enthalpy of formation at 298.15K";
          SI.SpecificEnthalpy[nspecies] H0 "H0(298.15K) - H0(0K)";
        end FixedIGProperties;

        record ThermoBaseVars
          extends Modelica.Icons.Record;
          parameter Integer n(min = 1) "Discretization number";
          parameter Integer nspecies(min = 1) "Number of species";
          SI.Pressure[n] p(min = PMIN, max = PMAX, nominal = PNOM, start = fill(1.0e5, n)) "Pressure";
          SI.Temperature[n] T(min = TMIN, max = TMAX, nominal = TNOM) "Temperature";
          SI.Density[n] d(min = DMIN, max = DMAX, nominal = DNOM) "Density";
          SI.SpecificEnthalpy[n] h(min = SHMIN, max = SHMAX, nominal = SHNOM) "Specific enthalpy";
          SI.SpecificEntropy[n] s(min = SSMIN, max = SSMAX, nominal = SSNOM) "Specific entropy";
          SI.RatioOfSpecificHeatCapacities[n] kappa "Ratio of cp/cv";
          SI.Mass[n] M(min = MMIN, max = MMAX, nominal = MNOM) "Total mass";
          SI.Energy[n] U(min = EMIN, max = EMAX, nominal = ENOM) "Inner energy";
          SI.MassFlowRate[n] dM(min = MDOTMIN, max = MDOTMAX, nominal = MDOTNOM) "Change in total mass";
          SI.Power[n] dU(min = POWMIN, max = POWMAX, nominal = POWNOM) "Change in inner energy";
          SI.Volume[n] V(min = VMIN, max = VMAX, nominal = VNOM) "Volume";
          SI.MassFraction[n, nspecies] mass_x(min = MASSXMIN, max = MASSXMAX, nominal = MASSXNOM) "Mass fraction";
          SI.MoleFraction[n, nspecies] mole_y(min = MOLEYMIN, max = MOLEYMAX, nominal = MOLEYNOM) "Mole fraction";
          SI.Mass[n, nspecies] M_x(min = MMIN, max = MMAX, nominal = MNOM) "Component mass";
          SI.MassFlowRate[n, nspecies] dM_x(min = MDOTMIN, max = MDOTMAX, nominal = MDOTNOM) "Rate of change in component mass";
          MolarFlowRate[n, nspecies] dZ(min = -1.0e6, max = 1.0e6, nominal = 0.0) "Rate of change in component moles";
          MolarFlowRate[n, nspecies] rZ(min = -1.0e6, max = 1.0e6, nominal = 0.0) "Reaction(source) mole rates";
          SI.MolarMass[n] MM(min = MMMIN, max = MMMAX, nominal = MMNOM) "Molar mass of mixture";
          SI.AmountOfSubstance[n] Moles(min = MOLMIN, max = MOLMAX, nominal = MOLNOM) "Total moles";
          SI.AmountOfSubstance[n, nspecies] Moles_z(min = MOLMIN, max = MOLMAX, nominal = MOLNOM) "Mole vector";
          annotation(
            Documentation(info = "<HTML>
                         <h4>Model description</h4>
                              <p>
                              <b>ThermoBaseVars</b> is inherited by all medium property models
                              and by all models defining the dynamic states for the conservation
                              of mass and energy. Thus it is a good choice as a restricting class
                              for any medium model or dynamic state model.
                           </p>
                              </html>"));
        end ThermoBaseVars;

        record ThermoProperties "Thermodynamic base property data for all state models"
          extends Modelica.Icons.Record;
          parameter Integer nspecies(min = 1) "Number of species";
          SI.Temperature T(min = TMIN, max = TMAX, nominal = TNOM) "Temperature";
          SI.Density d(min = DMIN, max = DMAX, nominal = DNOM) "Density";
          SI.Pressure p(min = PMIN, max = PMAX, nominal = PNOM) "Pressure";
          SI.Volume V(min = VMIN, max = VMAX, nominal = VNOM) "Volume";
          SI.SpecificEnthalpy h(min = SHMIN, max = SHMAX, nominal = SHNOM) "Specific enthalpy";
          SI.SpecificEnergy u(min = SEMIN, max = SEMAX, nominal = SENOM) "Specific inner energy";
          SI.SpecificEntropy s(min = SSMIN, max = SSMAX, nominal = SSNOM) "Specific entropy";
          SI.SpecificGibbsFreeEnergy g(min = SHMIN, max = SHMAX, nominal = SHNOM) "Specific Gibbs free energy";
          SI.SpecificHeatCapacity cp(min = CPMIN, max = CPMAX, nominal = CPNOM) "Heat capacity at constant pressure";
          SI.SpecificHeatCapacity cv(min = CPMIN, max = CPMAX, nominal = CPNOM) "Heat capacity at constant volume";
          SI.SpecificHeatCapacity R(min = CPMIN, max = CPMAX, nominal = CPNOM) "Gas constant";
          SI.MolarMass MM(min = MMMIN, max = MMMAX, nominal = MMNOM) "Molar mass of mixture";
          SI.MassFraction[nspecies] mass_x(min = MASSXMIN, max = MASSXMAX, nominal = MASSXNOM) "Mass fraction";
          SI.MoleFraction[nspecies] mole_y(min = MOLEYMIN, max = MOLEYMAX, nominal = MOLEYNOM) "Mole fraction";
          SI.RatioOfSpecificHeatCapacities kappa "Ratio of cp/cv";
          SI.DerDensityByTemperature ddTp "Derivative of density by temperature at constant pressure";
          SI.DerDensityByPressure ddpT "Derivative of density by pressure at constant temperature";
          Real dupT(unit = "m3.kg-1") "Derivative of inner energy by pressure at constant T";
          Real dudT(unit = "(J.m3)/(kg2)") "Derivative of inner energy by density at constant T";
          SI.SpecificHeatCapacity duTp "Derivative of inner energy by temperature at constant p";
          SI.SpecificEnergy ddx[nspecies] "Derivative vector of density by change in mass composition";
          SI.SpecificEnergy[nspecies] compu(min = SEMIN, max = SEMAX, nominal = SENOM) "Inner energy of the components";
          SI.Pressure[nspecies] compp(min = COMPPMIN, max = COMPPMAX, nominal = COMPPNOM) "partial pressures of the components";
          SI.Velocity a(min = VELMIN, max = VELMAX, nominal = VELNOM) "Speed of sound";
          SI.HeatCapacity dUTZ "Derivative of inner energy by temperature at constant moles";
          SI.MolarInternalEnergy[nspecies] dUZT "Derivative of inner energy by moles at constant temperature";
          SI.SpecificEnthalpy[nspecies] dHMxT(min = SEMIN, max = SEMAX, nominal = SENOM) "Derivative of total enthalpy w.r.t. component mass at constant T";
          Real dpT "Derivative of pressure w.r.t. temperature";
          Real dpZ[nspecies] "Derivative of pressure w.r.t. moles";
          annotation(
            Documentation(info = "<HTML>
        <h4>Model description</h4>
        <p>
        A base class for medium property models which work with most of the
        versions of dynamic states that are available in the ThermoFluid
        library. Currently used by all ideal gas models.
     </p>
        </html>"));
        end ThermoProperties;

        record ThermoProperties_ph "Thermodynamic property data for pressure p and specific enthalpy h as dynamic states"
          extends Modelica.Icons.Record;
          SI.Temperature T(min = 1.0e-9, max = 10000.0, nominal = 298.15) "Temperature";
          SI.Density d(min = 1.0e-9, max = 10000.0, nominal = 10.0) "Density";
          SI.SpecificEnergy u(min = -1.0e8, max = 1.0e8, nominal = 1.0e6) "Specific inner energy";
          SI.SpecificEntropy s(min = -1.0e6, max = 1.0e6, nominal = 1.0e3) "Specific entropy";
          SI.SpecificHeatCapacity cp(min = 1.0, max = 1.0e6, nominal = 1000.0) "Heat capacity at constant pressure";
          SI.SpecificHeatCapacity cv(min = 1.0, max = 1.0e6, nominal = 1000.0) "Heat capacity at constant volume";
          SI.SpecificHeatCapacity R(min = 1.0, max = 1.0e6, nominal = 1000.0) "Gas constant";
          SI.RatioOfSpecificHeatCapacities kappa "Ratio of cp/cv";
          SI.Velocity a(min = 1.0, max = 10000.0, nominal = 300.0) "Speed of sound";
          SI.DerDensityByEnthalpy ddhp "Derivative of density by enthalpy at constant pressure";
          SI.DerDensityByPressure ddph "Derivative of density by pressure at constant enthalpy";
          Real duph(unit = "m3/kg") "Derivative of inner energy by pressure at constant enthalpy";
          Real duhp(unit = "1") "Derivative of inner energy by enthalpy at constant pressure";
          annotation(
            Documentation(info = "<HTML>
<h4>Model description</h4>
<p>
A base class for medium property models which
use pressure and enthalpy as dynamic states.
This is the preferred model for fluids that can also be in the
two phase and liquid regions.
</p>
</html>"));
        end ThermoProperties_ph;

        record ThermoProperties_pT "Thermodynamic property data for pressure p and temperature T as dynamic states"
          extends Modelica.Icons.Record;
          SI.Density d(min = 1.0e-9, max = 10000.0, nominal = 10.0) "Density";
          SI.SpecificEnthalpy h(min = -1.0e8, max = 1.0e8, nominal = 1.0e6) "Specific enthalpy";
          SI.SpecificEnergy u(min = -1.0e8, max = 1.0e8, nominal = 1.0e6) "Specific inner energy";
          SI.SpecificEntropy s(min = -1.0e6, max = 1.0e6, nominal = 1.0e3) "Specific entropy";
          SI.SpecificHeatCapacity cp(min = 1.0, max = 1.0e6, nominal = 1000.0) "Heat capacity at constant pressure";
          SI.SpecificHeatCapacity cv(min = 1.0, max = 1.0e6, nominal = 1000.0) "Heat capacity at constant volume";
          SI.SpecificHeatCapacity R(min = 1.0, max = 1.0e6, nominal = 1000.0) "Gas constant";
          SI.RatioOfSpecificHeatCapacities kappa "Ratio of cp/cv";
          SI.Velocity a(min = 1.0, max = 10000.0, nominal = 300.0) "Speed of sound";
          SI.DerDensityByTemperature ddTp "Derivative of density by temperature at constant pressure";
          SI.DerDensityByPressure ddpT "Derivative of density by pressure at constant temperature";
          Real dupT(unit = "m3.kg-1") "Derivative of inner energy by pressure at constant T";
          SI.SpecificHeatCapacity duTp "Derivative of inner energy by temperature at constant p";
          annotation(
            Documentation(info = "<HTML>
<h4>Model description</h4>
<p>
A base class for medium property models which use pressure and temperature as dynamic states.
This is a reasonable model for fluids that can also be in the gas and
liquid regions, but never in the two-phase region.
</p>
</html>"));
        end ThermoProperties_pT;

        record ThermoProperties_dT "Thermodynamic property data for density d and temperature T as dynamic states"
          extends Modelica.Icons.Record;
          SI.Pressure p(min = 1.0, max = 1.0e9, nominal = 1.0e5) "Pressure";
          SI.SpecificEnthalpy h(min = -1.0e8, max = 1.0e8, nominal = 1.0e6) "Specific enthalpy";
          SI.SpecificEnergy u(min = -1.0e8, max = 1.0e8, nominal = 1.0e6) "Specific inner energy";
          SI.SpecificEntropy s(min = -1.0e6, max = 1.0e6, nominal = 1.0e3) "Specific entropy";
          SI.SpecificHeatCapacity cp(min = 1.0, max = 1.0e6, nominal = 1000.0) "Heat capacity at constant pressure";
          SI.SpecificHeatCapacity cv(min = 1.0, max = 1.0e6, nominal = 1000.0) "Heat capacity at constant volume";
          SI.SpecificHeatCapacity R(min = 1.0, max = 1.0e6, nominal = 1000.0) "Gas constant";
          SI.RatioOfSpecificHeatCapacities kappa "Ratio of cp/cv";
          SI.Velocity a(min = 1.0, max = 10000.0, nominal = 300.0) "Speed of sound";
          Real dudT(unit = "m5/(kg.s2)") "Derivative of inner energy by density at constant T";
          annotation(
            Documentation(info = "<HTML>
<h4>Model description</h4>
<p>
A base class for medium property models which use density and temperature as dynamic states.
This is a reasonable model for fluids that can be in the gas, liquid
and two-phase region. The model is numerically not well suited for
liquids except if the pressure is always above approx. 80% of the
critical pressure.
</p>
</html>"));
        end ThermoProperties_dT;

        //   record GibbsDerivs
        //     "Derivatives of dimensionless Gibbs-function w.r.t. dimensionless pressure and temperature"
        //     extends Modelica.Icons.Record;
        //     Real pi "Dimensionless pressure";
        //     Real tau "Dimensionless temperature";
        //     Real g "Dimensionless Gibbs-function";
        //     Real gpi "Derivative of g w.r.t. pi";
        //     Real gpipi "2nd derivative of g w.r.t. pi";
        //     Real gtau "Derivative of g w.r.t. tau";
        //     Real gtautau "2nd derivative of g w.r.t. tau";
        //     Real gtaupi "Mixed derivative of g w.r.t. pi and tau";
        //   end GibbsDerivs;
        //   record HelmholtzDerivs
        //     "Derivatives of dimensionless Helmholtz-function w.r.t. dimensionless pressure, density and temperature"
        //     extends Modelica.Icons.Record;
        //     Real delta "Dimensionless density";
        //     Real tau "Dimensionless temperature";
        //     Real f "Dimensionless Helmholtz-function";
        //     Real fdelta "Derivative of f w.r.t. delta";
        //     Real fdeltadelta "2nd derivative of f w.r.t. delta";
        //     Real ftau "Derivative of f w.r.t. tau";
        //     Real ftautau "2nd derivative of f w.r.t. tau";
        //     Real fdeltatau "Mixed derivative of f w.r.t. delta and tau";
        //   end HelmholtzDerivs;

        record TransportProps "Record with transport properties"
          extends Modelica.Icons.Record;
          SI.DynamicViscosity eta;
          SI.ThermalConductivity lam;
        end TransportProps;

        function gibbsToProps_ph "Calculate property record for pressure and specific enthalpy as states from dimensionless Gibbs function"
          extends Modelica.Icons.Function;
          input GibbsDerivs g "Dimensionless derivatives of Gibbs function";
          output ThermoProperties_ph pro "Property record for pressure and specific enthalpy as dynamic states";
        protected
          Real vt(unit = "m3.kg-1.K-1") "Derivative of specific volume w.r.t. temperature";
          Real vp(unit = "m4.kg-2.s2") "Derivative of specific volume w.r.t. pressure";
        algorithm
          pro.T := g.T;
          pro.R := g.R;
          pro.d := g.p / (pro.R * pro.T * g.pi * g.gpi);
          pro.u := g.T * g.R * (g.tau * g.gtau - g.pi * g.gpi);
          pro.s := pro.R * (g.tau * g.gtau - g.g);
          pro.cp := -pro.R * g.tau * g.tau * g.gtautau;
          pro.cv := pro.R * ((-g.tau * g.tau * g.gtautau) + (g.gpi - g.tau * g.gtaupi) * (g.gpi - g.tau * g.gtaupi) / g.gpipi);
          pro.a := abs(g.R * g.T * (g.gpi * g.gpi / ((g.gpi - g.tau * g.gtaupi) * (g.gpi - g.tau * g.gtaupi) / (g.tau * g.tau * g.gtautau) - g.gpipi))) ^ 0.5;
          vt := g.R / g.p * (g.pi * g.gpi - g.tau * g.pi * g.gtaupi);
          vp := g.R * g.T / (g.p * g.p) * g.pi * g.pi * g.gpipi;
          pro.kappa := -1 / (pro.d * g.p) * pro.cp / (vp * pro.cp + vt * vt * g.T);
          pro.ddhp := -pro.d * pro.d * vt / pro.cp;
          pro.ddph := -pro.d * pro.d * (vp * pro.cp - vt / pro.d + g.T * vt * vt) / pro.cp;
          pro.duph := (-1 / pro.d) + g.p / (pro.d * pro.d) * pro.ddph;
          pro.duhp := 1 + g.p / (pro.d * pro.d) * pro.ddhp;
        end gibbsToProps_ph;

        function gibbsToBoundaryProps "Calculate phase boundary property record from dimensionless Gibbs function"
          extends Modelica.Icons.Function;
          input GibbsDerivs g "Dimensionless derivatives of Gibbs function";
          output PhaseBoundaryProperties sat "Phase boundary properties";
        protected
          Real vt(unit = "m3.kg-1.K-1") "Derivative of specific volume w.r.t. temperature";
          Real vp(unit = "m4.kg-2.s2") "Derivative of specific volume w.r.t. pressure";
        algorithm
          sat.d := g.p / (g.R * g.T * g.pi * g.gpi);
          sat.h := g.R * g.T * g.tau * g.gtau;
          sat.u := g.T * g.R * (g.tau * g.gtau - g.pi * g.gpi);
          sat.s := g.R * (g.tau * g.gtau - g.g);
          sat.cp := -g.R * g.tau * g.tau * g.gtautau;
          sat.cv := g.R * ((-g.tau * g.tau * g.gtautau) + (g.gpi - g.tau * g.gtaupi) * (g.gpi - g.tau * g.gtaupi) / g.gpipi);
          vt := g.R / g.p * (g.pi * g.gpi - g.tau * g.pi * g.gtaupi);
          vp := g.R * g.T / (g.p * g.p) * g.pi * g.pi * g.gpipi;
// sat.kappa := -1/(sat.d*g.p)*sat.cp/(vp*sat.cp + vt*vt*g.T);
          sat.pt := -g.p / g.T * (g.gpi - g.tau * g.gtaupi) / (g.gpipi * g.pi);
          sat.pd := -g.R * g.T * g.gpi * g.gpi / g.gpipi;
        end gibbsToBoundaryProps;

        function gibbsToProps_dT "Calculate property record for density and temperature as states from dimensionless Gibbs function"
          extends Modelica.Icons.Function;
          input GibbsDerivs g "Dimensionless derivatives of Gibbs function";
          output ThermoProperties_dT pro "Property record for density and temperature as dynamic states";
        protected
          Real vt(unit = "m3.kg-1.K-1") "Derivative of specific volume w.r.t. temperature";
          Real vp(unit = "m4.kg-2.s2") "Derivative of specific volume w.r.t. pressure";
          Modelica.SIunits.Density d;
        algorithm
          pro.R := g.R;
          pro.p := g.p;
          pro.u := g.T * g.R * (g.tau * g.gtau - g.pi * g.gpi);
          pro.h := g.R * g.T * g.tau * g.gtau;
          pro.s := pro.R * (g.tau * g.gtau - g.g);
          pro.cp := -pro.R * g.tau * g.tau * g.gtautau;
          pro.cv := pro.R * ((-g.tau * g.tau * g.gtautau) + (g.gpi - g.tau * g.gtaupi) * (g.gpi - g.tau * g.gtaupi) / g.gpipi);
          vt := g.R / g.p * (g.pi * g.gpi - g.tau * g.pi * g.gtaupi);
          vp := g.R * g.T / (g.p * g.p) * g.pi * g.pi * g.gpipi;
          pro.kappa := -1 / (g.p / (pro.R * g.T * g.pi * g.gpi) * g.p) * pro.cp / (vp * pro.cp + vt * vt * g.T);
          pro.a := abs(g.R * g.T * (g.gpi * g.gpi / ((g.gpi - g.tau * g.gtaupi) * (g.gpi - g.tau * g.gtaupi) / (g.tau * g.tau * g.gtautau) - g.gpipi))) ^ 0.5;
          d := g.p / (pro.R * g.T * g.pi * g.gpi);
          pro.dudT := (pro.p - g.T * vt / vp) / (d * d);
        end gibbsToProps_dT;

        function gibbsToProps_pT "Calculate property record for pressure and temperature as states from dimensionless Gibbs function"
          extends Modelica.Icons.Function;
          input GibbsDerivs g "Dimensionless derivatives of Gibbs function";
          output ThermoProperties_pT pro "Property record for pressure and temperature as dynamic states";
        protected
          Real vt(unit = "m3.kg-1.K-1") "Derivative of specific volume w.r.t. temperature";
          Real vp(unit = "m4.kg-2.s2") "Derivative of specific volume w.r.t. pressure";
        algorithm
          pro.R := g.R;
          pro.d := g.p / (pro.R * g.T * g.pi * g.gpi);
          pro.u := g.T * g.R * (g.tau * g.gtau - g.pi * g.gpi);
          pro.h := g.R * g.T * g.tau * g.gtau;
          pro.s := pro.R * (g.tau * g.gtau - g.g);
          pro.cp := -pro.R * g.tau * g.tau * g.gtautau;
          pro.cv := pro.R * ((-g.tau * g.tau * g.gtautau) + (g.gpi - g.tau * g.gtaupi) * (g.gpi - g.tau * g.gtaupi) / g.gpipi);
          vt := g.R / g.p * (g.pi * g.gpi - g.tau * g.pi * g.gtaupi);
          vp := g.R * g.T / (g.p * g.p) * g.pi * g.pi * g.gpipi;
          pro.kappa := -1 / (pro.d * g.p) * pro.cp / (vp * pro.cp + vt * vt * g.T);
          pro.a := abs(g.R * g.T * (g.gpi * g.gpi / ((g.gpi - g.tau * g.gtaupi) * (g.gpi - g.tau * g.gtaupi) / (g.tau * g.tau * g.gtautau) - g.gpipi))) ^ 0.5;
          pro.ddpT := -pro.d * pro.d * vp;
          pro.ddTp := -pro.d * pro.d * vt;
          pro.duTp := pro.cp - g.p * vt;
          pro.dupT := (-g.T * vt) - g.p * vp;
        end gibbsToProps_pT;

        function helmholtzToProps_ph "Calculate property record for pressure and specific enthalpy as states from dimensionless Helmholtz function"
          extends Modelica.Icons.Function;
          input HelmholtzDerivs f "Dimensionless derivatives of Helmholtz function";
          output ThermoProperties_ph pro "Property record for pressure and specific enthalpy as dynamic states";
        protected
          SI.Pressure p "Pressure";
          DerPressureByDensity pd "Derivative of pressure w.r.t. density";
          DerPressureByTemperature pt "Derivative of pressure w.r.t. temperature";
          DerPressureBySpecificVolume pv "Derivative of pressure w.r.t. specific volume";
        algorithm
          pro.d := f.d;
          pro.T := f.T;
          pro.R := f.R;
          pro.s := f.R * (f.tau * f.ftau - f.f);
          pro.u := f.R * f.T * f.tau * f.ftau;
          p := pro.d * pro.R * pro.T * f.delta * f.fdelta;
          pd := f.R * f.T * f.delta * (2.0 * f.fdelta + f.delta * f.fdeltadelta);
          pt := f.R * f.d * f.delta * (f.fdelta - f.tau * f.fdeltatau);
          pv := -pd * f.d * f.d;
// calculating cp near the critical point may be troublesome (cp -> inf).
          pro.cp := f.R * ((-f.tau * f.tau * f.ftautau) + (f.delta * f.fdelta - f.delta * f.tau * f.fdeltatau) ^ 2 / (2 * f.delta * f.fdelta + f.delta * f.delta * f.fdeltadelta));
          pro.cv := f.R * (-f.tau * f.tau * f.ftautau);
          pro.kappa := 1 / (f.d * f.R * f.d * f.T * f.delta * f.fdelta) * (((-pv * pro.cv) + pt * pt * f.T) / pro.cv);
          pro.a := abs(f.R * f.T * (2 * f.delta * f.fdelta + f.delta * f.delta * f.fdeltadelta - (f.delta * f.fdelta - f.delta * f.tau * f.fdeltatau) * (f.delta * f.fdelta - f.delta * f.tau * f.fdeltatau) / (f.tau * f.tau * f.ftautau))) ^ 0.5;
          pro.ddph := f.d * (pro.cv * f.d + pt) / (f.d * f.d * pd * pro.cv + f.T * pt * pt);
          pro.ddhp := -f.d * f.d * pt / (f.d * f.d * pd * pro.cv + f.T * pt * pt);
          pro.duph := (-1 / pro.d) + p / (pro.d * pro.d) * pro.ddph;
          pro.duhp := 1 + p / (pro.d * pro.d) * pro.ddhp;
        end helmholtzToProps_ph;

        function helmholtzToProps_pT "Calculate property record for pressure and temperature as states from dimensionless Helmholtz function"
          extends Modelica.Icons.Function;
          input HelmholtzDerivs f "Dimensionless derivatives of Helmholtz function";
          output ThermoProperties_pT pro "Property record for pressure and temperature as dynamic states";
        protected
          DerPressureByDensity pd "Derivative of pressure w.r.t. density";
          DerPressureByTemperature pt "Derivative of pressure w.r.t. temperature";
          DerPressureBySpecificVolume pv "Derivative of pressure w.r.t. specific volume";
          IsobaricVolumeExpansionCoefficient alpha "Isobaric volume expansion coefficient";
          // beta in Bejan
          IsothermalCompressibility gamma "Isothermal compressibility";
          // kappa in Bejan
          SI.Pressure p "Pressure";
        algorithm
          pro.d := f.d;
          pro.R := f.R;
          pro.s := f.R * (f.tau * f.ftau - f.f);
          pro.h := f.R * f.T * (f.tau * f.ftau + f.delta * f.fdelta);
          pro.u := f.R * f.T * f.tau * f.ftau;
          pd := f.R * f.T * f.delta * (2.0 * f.fdelta + f.delta * f.fdeltadelta);
          pt := f.R * f.d * f.delta * (f.fdelta - f.tau * f.fdeltatau);
          pv := -f.d * f.d * pd;
          alpha := -f.d * pt / pv;
          gamma := -f.d / pv;
          p := f.R * f.d * f.T * f.delta * f.fdelta;
// calculating cp near the critical point may be troublesome (cp -> inf).
          pro.cp := f.R * ((-f.tau * f.tau * f.ftautau) + (f.delta * f.fdelta - f.delta * f.tau * f.fdeltatau) ^ 2 / (2 * f.delta * f.fdelta + f.delta * f.delta * f.fdeltadelta));
          pro.cv := f.R * (-f.tau * f.tau * f.ftautau);
          pro.kappa := 1 / (f.d * f.R * f.d * f.T * f.delta * f.fdelta) * (((-pv * pro.cv) + pt * pt * f.T) / pro.cv);
          pro.a := abs(f.R * f.T * (2 * f.delta * f.fdelta + f.delta * f.delta * f.fdeltadelta - (f.delta * f.fdelta - f.delta * f.tau * f.fdeltatau) * (f.delta * f.fdelta - f.delta * f.tau * f.fdeltatau) / (f.tau * f.tau * f.ftautau))) ^ 0.5;
          pro.ddTp := -pt / pd;
          pro.ddpT := 1 / pd;
//problem with units in last two lines
          pro.dupT := gamma * p / f.d - alpha * f.T / f.d;
          pro.duTp := pro.cp - alpha * p / f.d;
        end helmholtzToProps_pT;

        function helmholtzToProps_dT "Calculate property record for density and temperature as states from dimensionless Helmholtz function"
          extends Modelica.Icons.Function;
          input HelmholtzDerivs f "Dimensionless derivatives of Helmholtz function";
          output ThermoProperties_dT pro "Property record for density and temperature as dynamic states";
        protected
          DerPressureByTemperature pt "Derivative of pressure w.r.t. temperature";
          DerPressureBySpecificVolume pv "Derivative of pressure w.r.t. pressure";
        algorithm
          pro.p := f.R * f.d * f.T * f.delta * f.fdelta;
          pro.R := f.R;
          pro.s := f.R * (f.tau * f.ftau - f.f);
          pro.h := f.R * f.T * (f.tau * f.ftau + f.delta * f.fdelta);
          pro.u := f.R * f.T * f.tau * f.ftau;
          pv := -f.d * f.d * f.R * f.T * f.delta * (2.0 * f.fdelta + f.delta * f.fdeltadelta);
          pt := f.R * f.d * f.delta * (f.fdelta - f.tau * f.fdeltatau);
// calculating cp near the critical point may be troublesome (cp -> inf).
          pro.cp := f.R * ((-f.tau * f.tau * f.ftautau) + (f.delta * f.fdelta - f.delta * f.tau * f.fdeltatau) ^ 2 / (2 * f.delta * f.fdelta + f.delta * f.delta * f.fdeltadelta));
          pro.cv := f.R * (-f.tau * f.tau * f.ftautau);
          pro.kappa := 1 / (f.d * pro.p) * (((-pv * pro.cv) + pt * pt * f.T) / pro.cv);
          pro.a := abs(f.R * f.T * (2 * f.delta * f.fdelta + f.delta * f.delta * f.fdeltadelta - (f.delta * f.fdelta - f.delta * f.tau * f.fdeltatau) * (f.delta * f.fdelta - f.delta * f.tau * f.fdeltatau) / (f.tau * f.tau * f.ftautau))) ^ 0.5;
          pro.dudT := (pro.p - f.T * pt) / (f.d * f.d);
        end helmholtzToProps_dT;

        function TwoPhaseToProps_ph "Compute property record for pressure and specific enthalpy as states from saturation properties"
          extends Modelica.Icons.Function;
          input SaturationProperties sat "Saturation property record";
          output ThermoProperties_ph pro "Property record for pressure and specific enthalpy as dynamic states";
        protected
          Real dht(unit = "(J/kg)/K") "Derivative of specific enthalpy w.r.t. temperature";
          Real dhd(unit = "(J/kg)/(kg/m3)") "Derivative of specific enthalpy w.r.t. density";
          Real detph(unit = "m4.s4/(K.s8)") "Thermodynamic determinant";
        algorithm
          pro.d := sat.d;
          pro.T := sat.T;
          pro.u := sat.u;
          pro.s := sat.s;
          pro.cv := sat.cv;
          pro.R := sat.R;
          pro.cp := Modelica.Constants.inf;
          pro.kappa := -1 / (sat.d * sat.p) * sat.dpT * sat.dpT * sat.T / sat.cv;
          pro.a := Modelica.Constants.inf;
          dht := sat.cv + sat.dpT / sat.d;
          dhd := -sat.T * sat.dpT / (sat.d * sat.d);
          detph := -sat.dpT * dhd;
          pro.ddph := dht / detph;
          pro.ddhp := -sat.dpT / detph;
        end TwoPhaseToProps_ph;

        function TwoPhaseToProps_dT "Compute property record for density and temperature as states from saturation properties"
          extends Modelica.Icons.Function;
          input SaturationProperties sat "Saturation properties";
          output ThermoProperties_dT pro "Property record for density and temperature as dynamic states";
        algorithm
          pro.p := sat.p;
          pro.h := sat.h;
          pro.u := sat.u;
          pro.s := sat.s;
          pro.cv := sat.cv;
          pro.cp := Modelica.Constants.inf;
          pro.R := sat.R;
          pro.kappa := -1 / (sat.d * sat.p) * sat.dpT * sat.dpT * sat.T / sat.cv;
          pro.a := Modelica.Constants.inf;
          pro.dudT := (sat.p - sat.T * sat.dpT) / (sat.d * sat.d);
        end TwoPhaseToProps_dT;
      end ThermoFluidSpecial;
    public
      record SaturationProperties "Properties in the two phase region"
        extends Modelica.Icons.Record;
        SI.Temp_K T "Temperature";
        SI.Density d "Density";
        SI.Pressure p "Pressure";
        SI.SpecificEnergy u "Specific inner energy";
        SI.SpecificEnthalpy h "Specific enthalpy";
        SI.SpecificEntropy s "Specific entropy";
        SI.SpecificHeatCapacity cp "Heat capacity at constant pressure";
        SI.SpecificHeatCapacity cv "Heat capacity at constant volume";
        SI.SpecificHeatCapacity R "Gas constant";
        SI.RatioOfSpecificHeatCapacities kappa "Isentropic expansion coefficient";
        PhaseBoundaryProperties liq "Thermodynamic base properties on the boiling curve";
        PhaseBoundaryProperties vap "Thermodynamic base properties on the dew curve";
        Real dpT(unit = "Pa/K") "Derivative of saturation pressure w.r.t. temperature";
        SI.MassFraction x "Vapour mass fraction";
      end SaturationProperties;

      record SaturationBoundaryProperties "Properties on both phase boundaries, including some derivatives"
        extends Modelica.Icons.Record;
        SI.Temp_K T "Saturation temperature";
        SI.Density dl "Liquid density";
        SI.Density dv "Vapour density";
        SI.SpecificEnthalpy hl "Liquid specific enthalpy";
        SI.SpecificEnthalpy hv "Vapour specific enthalpy";
        Real dTp "Derivative of temperature w.r.t. saturation pressure";
        Real ddldp "Derivative of density along boiling curve";
        Real ddvdp "Derivative of density along dew curve";
        Real dhldp "Derivative of specific enthalpy along boiling curve";
        Real dhvdp "Derivative of specific enthalpy along dew curve";
        SI.MassFraction x "Vapour mass fraction";
      end SaturationBoundaryProperties;

      record IF97BaseTwoPhase "Intermediate property data record for IF 97"
        extends Modelica.Icons.Record;
        Integer phase(start = 0) "Phase: 2 for two-phase, 1 for one phase, 0 if unknown";
        Integer region(min = 1, max = 5) "IF 97 region";
        SI.Pressure p "Pressure";
        SI.Temperature T "Temperature";
        SI.SpecificEnthalpy h "Specific enthalpy";
        SI.SpecificHeatCapacity R "Gas constant";
        SI.SpecificHeatCapacity cp "Specific heat capacity";
        SI.SpecificHeatCapacity cv "Specific heat capacity";
        SI.Density rho "Density";
        SI.SpecificEntropy s "Specific entropy";
        DerPressureByTemperature pt "Derivative of pressure w.r.t. temperature";
        DerPressureByDensity pd "Derivative of pressure w.r.t. density";
        Real vt "Derivative of specific volume w.r.t. temperature";
        Real vp "Derivative of specific volume w.r.t. pressure";
        Real x "Dryness fraction";
        Real dpT "dp/dT derivative of saturation curve";
      end IF97BaseTwoPhase;

      record IF97PhaseBoundaryProperties "Thermodynamic base properties on the phase boundary for IF97 steam tables"
        extends Modelica.Icons.Record;
        Boolean region3boundary "True if boundary between 2-phase and region 3";
        SI.SpecificHeatCapacity R "Specific heat capacity";
        SI.Temperature T "Temperature";
        SI.Density d "Density";
        SI.SpecificEnthalpy h "Specific enthalpy";
        SI.SpecificEntropy s "Specific entropy";
        SI.SpecificHeatCapacity cp "Heat capacity at constant pressure";
        SI.SpecificHeatCapacity cv "Heat capacity at constant volume";
        DerPressureByTemperature dpT "dp/dT derivative of saturation curve";
        DerPressureByTemperature pt "Derivative of pressure w.r.t. temperature";
        DerPressureByDensity pd "Derivative of pressure w.r.t. density";
        Real vt(unit = "m3/(kg.K)") "Derivative of specific volume w.r.t. temperature";
        Real vp(unit = "m3/(kg.Pa)") "Derivative of specific volume w.r.t. pressure";
      end IF97PhaseBoundaryProperties;

      record GibbsDerivs "Derivatives of dimensionless Gibbs-function w.r.t. dimensionless pressure and temperature"
        extends Modelica.Icons.Record;
        SI.Pressure p "Pressure";
        SI.Temperature T "Temperature";
        SI.SpecificHeatCapacity R "Specific heat capacity";
        Real pi(unit = "1") "Dimensionless pressure";
        Real tau(unit = "1") "Dimensionless temperature";
        Real g(unit = "1") "Dimensionless Gibbs-function";
        Real gpi(unit = "1") "Derivative of g w.r.t. pi";
        Real gpipi(unit = "1") "2nd derivative of g w.r.t. pi";
        Real gtau(unit = "1") "Derivative of g w.r.t. tau";
        Real gtautau(unit = "1") "2nd derivative of g w.r.t. tau";
        Real gtaupi(unit = "1") "Mixed derivative of g w.r.t. pi and tau";
      end GibbsDerivs;

      record HelmholtzDerivs "Derivatives of dimensionless Helmholtz-function w.r.t. dimensionless pressure, density and temperature"
        extends Modelica.Icons.Record;
        SI.Density d "Density";
        SI.Temperature T "Temperature";
        SI.SpecificHeatCapacity R "Specific heat capacity";
        Real delta(unit = "1") "Dimensionless density";
        Real tau(unit = "1") "Dimensionless temperature";
        Real f(unit = "1") "Dimensionless Helmholtz-function";
        Real fdelta(unit = "1") "Derivative of f w.r.t. delta";
        Real fdeltadelta(unit = "1") "2nd derivative of f w.r.t. delta";
        Real ftau(unit = "1") "Derivative of f w.r.t. tau";
        Real ftautau(unit = "1") "2nd derivative of f w.r.t. tau";
        Real fdeltatau(unit = "1") "Mixed derivative of f w.r.t. delta and tau";
      end HelmholtzDerivs;

      record TwoPhaseTransportProps "Defines properties on both phase boundaries, needed in the two phase region"
        extends Modelica.Icons.Record;
        SI.Density d_vap "Density on the dew line";
        SI.Density d_liq "Density on the bubble line";
        SI.DynamicViscosity eta_vap "Dynamic viscosity on the dew line";
        SI.DynamicViscosity eta_liq "Dynamic viscosity on the bubble line";
        SI.ThermalConductivity lam_vap "Thermal conductivity on the dew line";
        SI.ThermalConductivity lam_liq "Thermal conductivity on the bubble line";
        SI.SpecificHeatCapacity cp_vap "Cp on the dew line";
        SI.SpecificHeatCapacity cp_liq "Cp on the bubble line";
        SI.MassFraction x "Steam quality";
      end TwoPhaseTransportProps;

      record PhaseBoundaryProperties "Thermodynamic base properties on the phase boundary"
        extends Modelica.Icons.Record;
        SI.Density d "Density";
        SI.SpecificEnthalpy h "Specific enthalpy";
        SI.SpecificEnergy u "Inner energy";
        SI.SpecificEntropy s "Specific entropy";
        SI.SpecificHeatCapacity cp "Heat capacity at constant pressure";
        SI.SpecificHeatCapacity cv "Heat capacity at constant volume";
        DerPressureByTemperature pt "Derivative of pressure w.r.t. temperature";
        DerPressureByDensity pd "Derivative of pressure w.r.t. density";
      end PhaseBoundaryProperties;

      record NewtonDerivatives_ph "Derivatives for fast inverse calculations of Helmholtz functions: p & h"
        extends Modelica.Icons.Record;
        SI.Pressure p "Pressure";
        SI.SpecificEnthalpy h "Specific enthalpy";
        DerPressureByDensity pd "Derivative of pressure w.r.t. density";
        DerPressureByTemperature pt "Derivative of pressure w.r.t. temperature";
        Real hd "Derivative of specific enthalpy w.r.t. density";
        Real ht "Derivative of specific enthalpy w.r.t. temperature";
      end NewtonDerivatives_ph;

      record NewtonDerivatives_ps "Derivatives for fast inverse calculation of Helmholtz functions: p & s"
        extends Modelica.Icons.Record;
        SI.Pressure p "Pressure";
        SI.SpecificEntropy s "Specific entropy";
        DerPressureByDensity pd "Derivative of pressure w.r.t. density";
        DerPressureByTemperature pt "Derivative of pressure w.r.t. temperature";
        Real sd "Derivative of specific entropy w.r.t. density";
        Real st "Derivative of specific entropy w.r.t. temperature";
      end NewtonDerivatives_ps;

      record NewtonDerivatives_pT "Derivatives for fast inverse calculations of Helmholtz functions:p & T"
        extends Modelica.Icons.Record;
        SI.Pressure p "Pressure";
        DerPressureByDensity pd "Derivative of pressure w.r.t. density";
      end NewtonDerivatives_pT;

      record ExtraDerivatives "Additional thermodynamic derivatives"
        extends Modelica.Icons.Record;
        IsentropicExponent kappa "Isentropic expansion coefficient";
        // k in Bejan
        IsenthalpicExponent theta "Isenthalpic exponent";
        // same as kappa, except derivative at const h
        IsobaricVolumeExpansionCoefficient alpha "Isobaric volume expansion coefficient";
        // beta in Bejan
        IsochoricPressureCoefficient beta "Isochoric pressure coefficient";
        // kT in Bejan
        IsothermalCompressibility gamma "Isothermal compressibility";
        // kappa in Bejan
        JouleThomsonCoefficient mu "Joule-Thomson coefficient";
        // mu_J in Bejan
      end ExtraDerivatives;

      record BridgmansTables "Calculates all entries in Bridgmans tables if first seven variables given"
        extends Modelica.Icons.Record;
        // the first 7 need to calculated in a function!
        SI.SpecificVolume v "Specific volume";
        SI.Pressure p "Pressure";
        SI.Temperature T "Temperature";
        SI.SpecificEntropy s "Specific entropy";
        SI.SpecificHeatCapacity cp "Heat capacity at constant pressure";
        IsobaricVolumeExpansionCoefficient alpha "Isobaric volume expansion coefficient";
        // beta in Bejan
        IsothermalCompressibility gamma "Isothermal compressibility";
        // kappa in Bejan
        // Derivatives at constant pressure
        Real dTp = 1 "Coefficient in Bridgmans table, see info for usage";
        Real dpT = -dTp "Coefficient in Bridgmans table, see info for usage";
        Real dvp = alpha * v "Coefficient in Bridgmans table, see info for usage";
        Real dpv = -dvp "Coefficient in Bridgmans table, see info for usage";
        Real dsp = cp / T "Coefficient in Bridgmans table, see info for usage";
        Real dps = -dsp "Coefficient in Bridgmans table, see info for usage";
        Real dup = cp - alpha * p * v "Coefficient in Bridgmans table, see info for usage";
        Real dpu = -dup "Coefficient in Bridgmans table, see info for usage";
        Real dhp = cp "Coefficient in Bridgmans table, see info for usage";
        Real dph = -dhp "Coefficient in Bridgmans table, see info for usage";
        Real dfp = (-s) - alpha * p * v "Coefficient in Bridgmans table, see info for usage";
        Real dpf = -dfp "Coefficient in Bridgmans table, see info for usage";
        Real dgp = -s "Coefficient in Bridgmans table, see info for usage";
        Real dpg = -dgp "Coefficient in Bridgmans table, see info for usage";
        // Derivatives at constant Temperature
        Real dvT = gamma * v "Coefficient in Bridgmans table, see info for usage";
        Real dTv = -dvT "Coefficient in Bridgmans table, see info for usage";
        Real dsT = alpha * v "Coefficient in Bridgmans table, see info for usage";
        Real dTs = -dsT "Coefficient in Bridgmans table, see info for usage";
        Real duT = alpha * T * v - gamma * p * v "Coefficient in Bridgmans table, see info for usage";
        Real dTu = -duT "Coefficient in Bridgmans table, see info for usage";
        Real dhT = (-v) + alpha * T * v "Coefficient in Bridgmans table, see info for usage";
        Real dTh = -dhT "Coefficient in Bridgmans table, see info for usage";
        Real dfT = -gamma * p * v "Coefficient in Bridgmans table, see info for usage";
        Real dTf = -dfT "Coefficient in Bridgmans table, see info for usage";
        Real dgT = -v "Coefficient in Bridgmans table, see info for usage";
        Real dTg = -dgT "Coefficient in Bridgmans table, see info for usage";
        // Derivatives at constant v
        Real dsv = alpha * alpha * v * v - gamma * v * cp / T "Coefficient in Bridgmans table, see info for usage";
        Real dvs = -dsv "Coefficient in Bridgmans table, see info for usage";
        Real duv = T * alpha * alpha * v * v - gamma * v * cp "Coefficient in Bridgmans table, see info for usage";
        Real dvu = -duv "Coefficient in Bridgmans table, see info for usage";
        Real dhv = T * alpha * alpha * v * v - alpha * v * v - gamma * v * cp "Coefficient in Bridgmans table, see info for usage";
        Real dvh = -dhv "Coefficient in Bridgmans table, see info for usage";
        Real dfv = gamma * v * s "Coefficient in Bridgmans table, see info for usage";
        Real dvf = -dfv "Coefficient in Bridgmans table, see info for usage";
        Real dgv = gamma * v * s - alpha * v * v "Coefficient in Bridgmans table, see info for usage";
        Real dvg = -dgv "Coefficient in Bridgmans table, see info for usage";
        // Derivatives at constant s
        Real dus = dsv * p "Coefficient in Bridgmans table, see info for usage";
        Real dsu = -dus "Coefficient in Bridgmans table, see info for usage";
        Real dhs = -v * cp / T "Coefficient in Bridgmans table, see info for usage";
        Real dsh = -dhs "Coefficient in Bridgmans table, see info for usage";
        Real dfs = alpha * v * s + dus "Coefficient in Bridgmans table, see info for usage";
        Real dsf = -dfs "Coefficient in Bridgmans table, see info for usage";
        Real dgs = alpha * v * s - v * cp / T "Coefficient in Bridgmans table, see info for usage";
        Real dsg = -dgs "Coefficient in Bridgmans table, see info for usage";
        // Derivatives at constant u
        Real dhu = p * alpha * v * v + gamma * v * cp * p - v * cp - p * T * alpha * alpha * v * v "Coefficient in Bridgmans table, see info for usage";
        Real duh = -dhu "Coefficient in Bridgmans table, see info for usage";
        Real dfu = s * T * alpha * v - gamma * v * cp * p - gamma * v * s * p + p * T * alpha * alpha * v * v "Coefficient in Bridgmans table, see info for usage";
        Real duf = -dfu "Coefficient in Bridgmans table, see info for usage";
        Real dgu = alpha * v * v * p + alpha * v * s * T - v * cp - gamma * v * s * p "Coefficient in Bridgmans table, see info for usage";
        Real dug = -dgu "Coefficient in Bridgmans table, see info for usage";
        //  Derivatives at constant h
        Real dfh = (s - v * alpha * p) * (v - v * alpha * T) - gamma * v * cp * p "Coefficient in Bridgmans table, see info for usage";
        Real dhf = -dfh "Coefficient in Bridgmans table, see info for usage";
        Real dgh = alpha * v * s * T - v * (s + cp) "Coefficient in Bridgmans table, see info for usage";
        Real dhg = -dgh "Coefficient in Bridgmans table, see info for usage";
        // Derivatives at constant g
        Real dfg = gamma * v * s * p - v * s - alpha * v * v * p "Coefficient in Bridgmans table, see info for usage";
        Real dgf = -dfg "Coefficient in Bridgmans table, see info for usage";
        annotation(
          Documentation(info = "<HTML>
<p>
Important: the phase equilibrium conditions are not yet considered.
this means that Bridgman's tables do not yet work in the two phase region.
Some derivatives are 0 or infinity anyways.
Idea: Do not use the values in Bridgmans table directly, all
derivatives are calculated as the quotient of two entries in the
table. The last letter indicates which variable is held constant in
taking the derivative. The second letters are the two variables
involved in the derivative and the first letter is always a d to remind
of differentiation.
</p>

<pre>
Example 1: Get the derivative of specific entropy s w.r.t. Temperature at
constant specific volume (between identical to constant density)
constant volume  --> last letter v
Temperature      --> second letter T
Specific entropy --> second letter s
--> the needed value is dsv/dTv
Known variables:
Temperature T
pressure p
specific volume v
specific inner energy u
specific enthalpy h
specific entropy s
specific Helmholtz energy f
specific gibbs enthalpy g
Not included but useful:
density d
In order to convert derivatives involving density use the following
rules:
at constant density == at constant specific volume
ddx/dyx = -d*d*dvx/dyx with y,x any of T,p,u,h,s,f,g
dyx/ddx = -1/(d*d)dyx/dvx with y,x any of T,p,u,h,s,f,g
Usage example assuming water as the medium:
model BridgmansTablesForWater
extends ThermoFluid.BaseClasses.MediumModels.Water.WaterSteamMedium_ph;
Real derOfsByTAtConstantv \"derivative of sp. entropy by temperature at constant sp. volume\"
ThermoFluid.BaseClasses.MediumModels.Common.ExtraDerivatives dpro;
ThermoFluid.BaseClasses.MediumModels.Common.BridgmansTables bt;
equation
dpro = ThermoFluid.BaseClasses.MediumModels.SteamIF97.extraDerivs_pT(p[1],T[1]);
bt.p = p[1];
bt.T = T[1];
bt.v = 1/pro[1].d;
bt.s = pro[1].s;
bt.cp = pro[1].cp;
bt.alpha = dpro.alpha;
bt.gamma = dpro.gamma;
derOfsByTAtConstantv =  bt.dsv/bt.dTv;
                ...
end BridgmansTablesForWater;
                </pre>

                </html>"));
      end BridgmansTables;

      record FundamentalConstants "Constants of the medium"
        extends Modelica.Icons.Record;
        Modelica.SIunits.MolarHeatCapacity R_bar;
        Modelica.SIunits.SpecificHeatCapacity R;
        Modelica.SIunits.MolarMass MM;
        Modelica.SIunits.MolarDensity rhored;
        Modelica.SIunits.Temperature Tred;
        Modelica.SIunits.AbsolutePressure pred;
        Modelica.SIunits.SpecificEnthalpy h_off;
        Modelica.SIunits.SpecificEntropy s_off;
      end FundamentalConstants;

      record AuxiliaryProperties "Intermediate property data record"
        extends Modelica.Icons.Record;
        SI.Pressure p "Pressure";
        SI.Temperature T "Temperature";
        SI.SpecificEnthalpy h "Specific enthalpy";
        SI.SpecificHeatCapacity R "Gas constant";
        SI.SpecificHeatCapacity cp "Specific heat capacity";
        SI.SpecificHeatCapacity cv "Specific heat capacity";
        SI.Density rho "Density";
        SI.SpecificEntropy s "Specific entropy";
        SI.DerPressureByTemperature pt "Derivative of pressure w.r.t. temperature";
        SI.DerPressureByDensity pd "Derivative of pressure w.r.t. density";
        Real vt "Derivative of specific volume w.r.t. temperature";
        Real vp "Derivative of specific volume w.r.t. pressure";
      end AuxiliaryProperties;

      record GibbsDerivs2 "Derivatives of Gibbs function w.r.t. pressure and temperature"
        extends Modelica.Icons.Record;
        SI.Pressure p "Pressure";
        SI.Temperature T "Temperature";
        SI.SpecificHeatCapacity R "Specific heat capacity";
        Real pi(unit = "1") "Dimensionless pressure";
        Real theta(unit = "1") "Dimensionless temperature";
        Real g(unit = "J/kg") "Gibbs function";
        Real gp(unit = "m3/kg") "Derivative of g w.r.t. p";
        Real gpp(unit = "m3/(kg.Pa)") "2nd derivative of g w.r.t. p";
        Real gT(unit = "J/(kg.K)") "Derivative of g w.r.t. T";
        Real gTT(unit = "J/(kg.K2)") "2nd derivative of g w.r.t. T";
        Real gTp(unit = "m3/(kg.K)") "Mixed derivative of g w.r.t. T and p";
      end GibbsDerivs2;

      record NewtonDerivatives_dT "Derivatives for fast inverse calculations of Gibbs function"
        extends Modelica.Icons.Record;
        SI.SpecificVolume v "Specific volume";
        Real vp "Derivative of specific volume w.r.t. pressure";
      end NewtonDerivatives_dT;

      function gibbsToBridgmansTables "Calculates base coefficients for Bridgman's tables from gibbs enthalpy"
        extends Modelica.Icons.Function;
        input GibbsDerivs g "Dimensionless derivatives of Gibbs function";
        output SI.SpecificVolume v "Specific volume";
        output SI.Pressure p = g.p "Pressure";
        output SI.Temperature T = g.T "Temperature";
        output SI.SpecificEntropy s "Specific entropy";
        output SI.SpecificHeatCapacity cp "Heat capacity at constant pressure";
        output IsobaricVolumeExpansionCoefficient alpha "Isobaric volume expansion coefficient";
        // beta in Bejan
        output IsothermalCompressibility gamma "Isothermal compressibility";
        // kappa in Bejan
      protected
        Real vt(unit = "m3/(kg.K)") "Derivative of specific volume w.r.t. temperature";
        Real vp(unit = "m4.kg-2.s2") "Derivative of specific volume w.r.t. pressure";
      algorithm
        vt := g.R / g.p * (g.pi * g.gpi - g.tau * g.pi * g.gtaupi);
        vp := g.R * g.T / (g.p * g.p) * g.pi * g.pi * g.gpipi;
        v := g.R * g.T * g.pi * g.gpi / g.p;
        s := g.R * (g.tau * g.gtau - g.g);
        cp := -g.R * g.tau * g.tau * g.gtautau;
        alpha := vt / v;
        gamma := -vp / v;
      end gibbsToBridgmansTables;

      function helmholtzToBridgmansTables "Calculates base coefficients for Bridgmans tables from Helmholtz energy"
        extends Modelica.Icons.Function;
        input HelmholtzDerivs f "Dimensionless derivatives of Helmholtz function";
        output SI.SpecificVolume v = 1 / f.d "Specific volume";
        output SI.Pressure p "Pressure";
        output SI.Temperature T = f.T "Temperature";
        output SI.SpecificEntropy s "Specific entropy";
        output SI.SpecificHeatCapacity cp "Heat capacity at constant pressure";
        output IsobaricVolumeExpansionCoefficient alpha "Isobaric volume expansion coefficient";
        // beta in Bejan
        output IsothermalCompressibility gamma "Isothermal compressibility";
        // kappa in Bejan
      protected
        DerPressureByTemperature pt "Derivative of pressure w.r.t. temperature";
        DerPressureBySpecificVolume pv "Derivative of pressure w.r.t. specific volume";
        SI.SpecificHeatCapacity cv "Isochoric specific heat capacity";
      algorithm
        p := f.R * f.d * f.T * f.delta * f.fdelta;
        pv := -f.d * f.d * f.R * f.T * f.delta * (2.0 * f.fdelta + f.delta * f.fdeltadelta);
        pt := f.R * f.d * f.delta * (f.fdelta - f.tau * f.fdeltatau);
        s := f.R * (f.tau * f.ftau - f.f);
        alpha := -f.d * pt / pv;
        gamma := -f.d / pv;
        cp := f.R * ((-f.tau * f.tau * f.ftautau) + (f.delta * f.fdelta - f.delta * f.tau * f.fdeltatau) ^ 2 / (2 * f.delta * f.fdelta + f.delta * f.delta * f.fdeltadelta));
      end helmholtzToBridgmansTables;

      function gibbsToBoundaryProps "Calculate phase boundary property record from dimensionless Gibbs function"
        extends Modelica.Icons.Function;
        input GibbsDerivs g "Dimensionless derivatives of Gibbs function";
        output PhaseBoundaryProperties sat "Phase boundary properties";
      protected
        Real vt "Derivative of specific volume w.r.t. temperature";
        Real vp "Derivative of specific volume w.r.t. pressure";
      algorithm
        sat.d := g.p / (g.R * g.T * g.pi * g.gpi);
        sat.h := g.R * g.T * g.tau * g.gtau;
        sat.u := g.T * g.R * (g.tau * g.gtau - g.pi * g.gpi);
        sat.s := g.R * (g.tau * g.gtau - g.g);
        sat.cp := -g.R * g.tau * g.tau * g.gtautau;
        sat.cv := g.R * ((-g.tau * g.tau * g.gtautau) + (g.gpi - g.tau * g.gtaupi) * (g.gpi - g.tau * g.gtaupi) / g.gpipi);
        vt := g.R / g.p * (g.pi * g.gpi - g.tau * g.pi * g.gtaupi);
        vp := g.R * g.T / (g.p * g.p) * g.pi * g.pi * g.gpipi;
// sat.kappa := -1/(sat.d*g.p)*sat.cp/(vp*sat.cp + vt*vt*g.T);
        sat.pt := -g.p / g.T * (g.gpi - g.tau * g.gtaupi) / (g.gpipi * g.pi);
        sat.pd := -g.R * g.T * g.gpi * g.gpi / g.gpipi;
      end gibbsToBoundaryProps;

      function helmholtzToBoundaryProps "Calculate phase boundary property record from dimensionless Helmholtz function"
        extends Modelica.Icons.Function;
        input HelmholtzDerivs f "Dimensionless derivatives of Helmholtz function";
        output PhaseBoundaryProperties sat "Phase boundary property record";
      protected
        SI.Pressure p "Pressure";
      algorithm
        p := f.R * f.d * f.T * f.delta * f.fdelta;
        sat.d := f.d;
        sat.h := f.R * f.T * (f.tau * f.ftau + f.delta * f.fdelta);
        sat.s := f.R * (f.tau * f.ftau - f.f);
        sat.u := f.R * f.T * f.tau * f.ftau;
        sat.cp := f.R * ((-f.tau * f.tau * f.ftautau) + (f.delta * f.fdelta - f.delta * f.tau * f.fdeltatau) ^ 2 / (2 * f.delta * f.fdelta + f.delta * f.delta * f.fdeltadelta));
        sat.cv := f.R * (-f.tau * f.tau * f.ftautau);
        sat.pt := f.R * f.d * f.delta * (f.fdelta - f.tau * f.fdeltatau);
        sat.pd := f.R * f.T * f.delta * (2.0 * f.fdelta + f.delta * f.fdeltadelta);
      end helmholtzToBoundaryProps;

      function cv2Phase "Compute isochoric specific heat capacity inside the two-phase region"
        extends Modelica.Icons.Function;
        input PhaseBoundaryProperties liq "Properties on the boiling curve";
        input PhaseBoundaryProperties vap "Properties on the condensation curve";
        input SI.MassFraction x "Vapour mass fraction";
        input SI.Temperature T "Temperature";
        input SI.Pressure p "Properties";
        output SI.SpecificHeatCapacity cv "Isochoric specific heat capacity";
      protected
        Real dpT "Derivative of pressure w.r.t. temperature";
        Real dxv "Derivative of vapour mass fraction w.r.t. specific volume";
        Real dvTl "Derivative of liquid specific volume w.r.t. temperature";
        Real dvTv "Derivative of vapour specific volume w.r.t. temperature";
        Real duTl "Derivative of liquid specific inner energy w.r.t. temperature";
        Real duTv "Derivative of vapour specific inner energy w.r.t. temperature";
        Real dxt "Derivative of vapour mass fraction w.r.t. temperature";
      algorithm
        dxv := if liq.d <> vap.d then liq.d * vap.d / (liq.d - vap.d) else 0.0;
        dpT := (vap.s - liq.s) * dxv;
// wrong at critical point
        dvTl := (liq.pt - dpT) / liq.pd / liq.d / liq.d;
        dvTv := (vap.pt - dpT) / vap.pd / vap.d / vap.d;
        dxt := -dxv * (dvTl + x * (dvTv - dvTl));
        duTl := liq.cv + (T * liq.pt - p) * dvTl;
        duTv := vap.cv + (T * vap.pt - p) * dvTv;
        cv := duTl + x * (duTv - duTl) + dxt * (vap.u - liq.u);
      end cv2Phase;

      function cvdpT2Phase "Compute isochoric specific heat capacity inside the two-phase region and derivative of pressure w.r.t. temperature"
        extends Modelica.Icons.Function;
        input PhaseBoundaryProperties liq "Properties on the boiling curve";
        input PhaseBoundaryProperties vap "Properties on the condensation curve";
        input SI.MassFraction x "Vapour mass fraction";
        input SI.Temperature T "Temperature";
        input SI.Pressure p "Properties";
        output SI.SpecificHeatCapacity cv "Isochoric specific heat capacity";
        output Real dpT "Derivative of pressure w.r.t. temperature";
      protected
        Real dxv "Derivative of vapour mass fraction w.r.t. specific volume";
        Real dvTl "Derivative of liquid specific volume w.r.t. temperature";
        Real dvTv "Derivative of vapour specific volume w.r.t. temperature";
        Real duTl "Derivative of liquid specific inner energy w.r.t. temperature";
        Real duTv "Derivative of vapour specific inner energy w.r.t. temperature";
        Real dxt "Derivative of vapour mass fraction w.r.t. temperature";
      algorithm
        dxv := if liq.d <> vap.d then liq.d * vap.d / (liq.d - vap.d) else 0.0;
        dpT := (vap.s - liq.s) * dxv;
// wrong at critical point
        dvTl := (liq.pt - dpT) / liq.pd / liq.d / liq.d;
        dvTv := (vap.pt - dpT) / vap.pd / vap.d / vap.d;
        dxt := -dxv * (dvTl + x * (dvTv - dvTl));
        duTl := liq.cv + (T * liq.pt - p) * dvTl;
        duTv := vap.cv + (T * vap.pt - p) * dvTv;
        cv := duTl + x * (duTv - duTl) + dxt * (vap.u - liq.u);
      end cvdpT2Phase;

      function gibbsToExtraDerivs "Compute additional thermodynamic derivatives from dimensionless Gibbs function"
        extends Modelica.Icons.Function;
        input GibbsDerivs g "Dimensionless derivatives of Gibbs function";
        output ExtraDerivatives dpro "Additional property derivatives";
      protected
        Real vt "Derivative of specific volume w.r.t. temperature";
        Real vp "Derivative of specific volume w.r.t. pressure";
        SI.Density d "Density";
        SI.SpecificVolume v "Specific volume";
        SI.SpecificHeatCapacity cv "Isochoric heat capacity";
        SI.SpecificHeatCapacity cp "Isobaric heat capacity";
      algorithm
        d := g.p / (g.R * g.T * g.pi * g.gpi);
        v := 1 / d;
        vt := g.R / g.p * (g.pi * g.gpi - g.tau * g.pi * g.gtaupi);
        vp := g.R * g.T / (g.p * g.p) * g.pi * g.pi * g.gpipi;
        cp := -g.R * g.tau * g.tau * g.gtautau;
        cv := g.R * ((-g.tau * g.tau * g.gtautau) + (g.gpi - g.tau * g.gtaupi) * (g.gpi - g.tau * g.gtaupi) / g.gpipi);
        dpro.kappa := -1 / (d * g.p) * cp / (vp * cp + vt * vt * g.T);
        dpro.theta := cp / (d * g.p * ((-vp * cp) + vt * v - g.T * vt * vt));
        dpro.alpha := d * vt;
        dpro.beta := -vt / (g.p * vp);
        dpro.gamma := -d * vp;
        dpro.mu := -(v - g.T * vt) / cp;
      end gibbsToExtraDerivs;

      function helmholtzToExtraDerivs "Compute additional thermodynamic derivatives from dimensionless Helmholtz function"
        extends Modelica.Icons.Function;
        input HelmholtzDerivs f "Dimensionless derivatives of Helmholtz function";
        output ExtraDerivatives dpro "Additional property derivatives";
      protected
        SI.Pressure p "Pressure";
        SI.SpecificVolume v "Specific volume";
        DerPressureByTemperature pt "Derivative of pressure w.r.t. temperature";
        DerPressureBySpecificVolume pv "Derivative of pressure w.r.t. specific volume";
        SI.SpecificHeatCapacity cv "Isochoric specific heat capacity";
      algorithm
        v := 1 / f.d;
        p := f.R * f.d * f.T * f.delta * f.fdelta;
        pv := -f.d * f.d * f.R * f.T * f.delta * (2.0 * f.fdelta + f.delta * f.fdeltadelta);
        pt := f.R * f.d * f.delta * (f.fdelta - f.tau * f.fdeltatau);
        cv := f.R * (-f.tau * f.tau * f.ftautau);
        dpro.kappa := 1 / (f.d * p) * (((-pv * cv) + pt * pt * f.T) / cv);
        dpro.theta := -1 / (f.d * p) * (((-pv * cv) + f.T * pt * pt) / (cv + pt * v));
        dpro.alpha := -f.d * pt / pv;
        dpro.beta := pt / p;
        dpro.gamma := -f.d / pv;
        dpro.mu := (v * pv + f.T * pt) / (pt * pt * f.T - pv * cv);
      end helmholtzToExtraDerivs;

      function Helmholtz_ph "Function to calculate analytic derivatives for computing d and t given p and h"
        extends Modelica.Icons.Function;
        input HelmholtzDerivs f "Dimensionless derivatives of Helmholtz function";
        output NewtonDerivatives_ph nderivs "Derivatives for Newton iteration to calculate d and t from p and h";
      protected
        SI.SpecificHeatCapacity cv "Isochoric heat capacity";
      algorithm
        cv := -f.R * (f.tau * f.tau * f.ftautau);
        nderivs.p := f.d * f.R * f.T * f.delta * f.fdelta;
        nderivs.h := f.R * f.T * (f.tau * f.ftau + f.delta * f.fdelta);
        nderivs.pd := f.R * f.T * f.delta * (2.0 * f.fdelta + f.delta * f.fdeltadelta);
        nderivs.pt := f.R * f.d * f.delta * (f.fdelta - f.tau * f.fdeltatau);
        nderivs.ht := cv + nderivs.pt / f.d;
        nderivs.hd := (nderivs.pd - f.T * nderivs.pt / f.d) / f.d;
      end Helmholtz_ph;

      function Helmholtz_pT "Function to calculate analytic derivatives for computing d and t given p and t"
        extends Modelica.Icons.Function;
        input HelmholtzDerivs f "Dimensionless derivatives of Helmholtz function";
        output NewtonDerivatives_pT nderivs "Derivatives for Newton iteration to compute d and t from p and t";
      algorithm
        nderivs.p := f.d * f.R * f.T * f.delta * f.fdelta;
        nderivs.pd := f.R * f.T * f.delta * (2.0 * f.fdelta + f.delta * f.fdeltadelta);
      end Helmholtz_pT;

      function Helmholtz_ps "Function to calculate analytic derivatives for computing d and t given p and s"
        extends Modelica.Icons.Function;
        input HelmholtzDerivs f "Dimensionless derivatives of Helmholtz function";
        output NewtonDerivatives_ps nderivs "Derivatives for Newton iteration to compute d and t from p and s";
      protected
        SI.SpecificHeatCapacity cv "Isochoric heat capacity";
      algorithm
        cv := -f.R * (f.tau * f.tau * f.ftautau);
        nderivs.p := f.d * f.R * f.T * f.delta * f.fdelta;
        nderivs.s := f.R * (f.tau * f.ftau - f.f);
        nderivs.pd := f.R * f.T * f.delta * (2.0 * f.fdelta + f.delta * f.fdeltadelta);
        nderivs.pt := f.R * f.d * f.delta * (f.fdelta - f.tau * f.fdeltatau);
        nderivs.st := cv / f.T;
        nderivs.sd := -nderivs.pt / (f.d * f.d);
      end Helmholtz_ps;

      function smoothStep "Approximation of a general step, such that the characteristic is continuous and differentiable"
        extends Modelica.Icons.Function;
        input Real x "Abscissa value";
        input Real y1 "Ordinate value for x > 0";
        input Real y2 "Ordinate value for x < 0";
        input Real x_small(min = 0) = 1e-5 "Approximation of step for -x_small <= x <= x_small; x_small > 0 required";
        output Real y "Ordinate value to approximate y = if x > 0 then y1 else y2";
      algorithm
        y := smooth(1, if x > x_small then y1 else if x < (-x_small) then y2 else if abs(x_small) > 0 then x / x_small * ((x / x_small) ^ 2 - 3) * (y2 - y1) / 4 + (y1 + y2) / 2 else (y1 + y2) / 2);
        annotation(
          Inline = true,
          smoothOrder = 1,
          Documentation(revisions = "<html>
<ul>
<li><i>April 29, 2008</i>
    by <a href=\"mailto:Martin.Otter@DLR.de\">Martin Otter</a>:<br>
    Designed and implemented.</li>
<li><i>August 12, 2008</i>
    by <a href=\"mailto:Michael.Sielemann@dlr.de\">Michael Sielemann</a>:<br>
    Minor modification to cover the limit case <code>x_small -> 0</code> without division by zero.</li>
</ul>
</html>", info = "<html>
<p>
This function is used to approximate the equation
</p>
<pre>
    y = <b>if</b> x &gt; 0 <b>then</b> y1 <b>else</b> y2;
</pre>

<p>
by a smooth characteristic, so that the expression is continuous and differentiable:
</p>

<pre>
   y = <b>smooth</b>(1, <b>if</b> x &gt;  x_small <b>then</b> y1 <b>else</b>
                 <b>if</b> x &lt; -x_small <b>then</b> y2 <b>else</b> f(y1, y2));
</pre>

<p>
In the region -x_small &lt; x &lt; x_small a 2nd order polynomial is used
for a smooth transition from y1 to y2.
</p>

<p>
If <b>mass fractions</b> X[:] are approximated with this function then this can be performed
for all <b>nX</b> mass fractions, instead of applying it for nX-1 mass fractions and computing
the last one by the mass fraction constraint sum(X)=1. The reason is that the approximating function has the
property that sum(X) = 1, provided sum(X_a) = sum(X_b) = 1
(and y1=X_a[i], y2=X_b[i]).
This can be shown by evaluating the approximating function in the abs(x) &lt; x_small
region (otherwise X is either X_a or X_b):
</p>

<pre>
    X[1]  = smoothStep(x, X_a[1] , X_b[1] , x_small);
    X[2]  = smoothStep(x, X_a[2] , X_b[2] , x_small);
       ...
    X[nX] = smoothStep(x, X_a[nX], X_b[nX], x_small);
</pre>

<p>
or
</p>

<pre>
    X[1]  = c*(X_a[1]  - X_b[1])  + (X_a[1]  + X_b[1])/2
    X[2]  = c*(X_a[2]  - X_b[2])  + (X_a[2]  + X_b[2])/2;
       ...
    X[nX] = c*(X_a[nX] - X_b[nX]) + (X_a[nX] + X_b[nX])/2;
    c     = (x/x_small)*((x/x_small)^2 - 3)/4
</pre>

<p>
Summing all mass fractions together results in
</p>

<pre>
    sum(X) = c*(sum(X_a) - sum(X_b)) + (sum(X_a) + sum(X_b))/2
           = c*(1 - 1) + (1 + 1)/2
           = 1
</pre>
</html>"));
      end smoothStep;

      function Gibbs2_ph "Function to calculate analytic derivatives for computing T given p and h"
        extends Modelica.Icons.Function;
        input Modelica.Media.Common.GibbsDerivs2 g "Dimensionless derivatives of Gibbs function";
        output Modelica.Media.Common.NewtonDerivatives_ph nderivs "Derivatives for Newton iteration to calculate d and t from p and h";
      algorithm
        nderivs.h := g.g - g.T * g.gT;
        nderivs.ht := -g.T * g.gTT;
//dummy values - DO NOT USE
        nderivs.p := 0.0;
        nderivs.pd := 0.0;
        nderivs.pt := 0.0;
        nderivs.hd := 0.0;
      end Gibbs2_ph;

      function Gibbs2_dT "Function to calculate analytic derivatives for computing p given d and T"
        extends Modelica.Icons.Function;
        input Modelica.Media.Common.GibbsDerivs2 g "Dimensionless derivatives of Gibbs function";
        output Modelica.Media.Common.NewtonDerivatives_dT nderivs "Derivatives for Newton iteration to compute p from d and T";
      algorithm
        nderivs.v := g.gp;
        nderivs.vp := nderivs.v * g.gpp / g.gp;
      end Gibbs2_dT;

      function Gibbs2_ps "Function to calculate analytic derivatives for computing d and t given p and s"
        extends Modelica.Icons.Function;
        input Modelica.Media.Common.GibbsDerivs2 g "Dimensionless derivatives of Gibbs function";
        output Modelica.Media.Common.NewtonDerivatives_ps nderivs "Derivatives for Newton iteration to compute T from p and s";
      algorithm
        nderivs.s := -g.gT;
        nderivs.st := -g.gTT;
//dummy values - DO NOT USE
        nderivs.p := 0.0;
        nderivs.pd := 0.0;
        nderivs.pt := 0.0;
        nderivs.sd := 0.0;
      end Gibbs2_ps;

      package OneNonLinearEquation "Determine solution of a non-linear algebraic equation in one unknown without derivatives in a reliable and efficient way"
        extends Modelica.Icons.Package;

        replaceable record f_nonlinear_Data "Data specific for function f_nonlinear"
          extends Modelica.Icons.Record;
        end f_nonlinear_Data;

        replaceable partial function f_nonlinear "Nonlinear algebraic equation in one unknown: y = f_nonlinear(x,p,X)"
          extends Modelica.Icons.Function;
          input Real x "Independent variable of function";
          input Real p = 0.0 "Disregarded variables (here always used for pressure)";
          input Real[:] X = fill(0, 0) "Disregarded variables (her always used for composition)";
          input f_nonlinear_Data f_nonlinear_data "Additional data for the function";
          output Real y "= f_nonlinear(x)";
          // annotation(derivative(zeroDerivative=y)); // this must hold for all replaced functions
        end f_nonlinear;

        replaceable function solve "Solve f_nonlinear(x_zero)=y_zero; f_nonlinear(x_min) - y_zero and f_nonlinear(x_max)-y_zero must have different sign"
          import Modelica.Utilities.Streams.error;
          extends Modelica.Icons.Function;
          input Real y_zero "Determine x_zero, such that f_nonlinear(x_zero) = y_zero";
          input Real x_min "Minimum value of x";
          input Real x_max "Maximum value of x";
          input Real pressure = 0.0 "Disregarded variables (here always used for pressure)";
          input Real[:] X = fill(0, 0) "Disregarded variables (here always used for composition)";
          input f_nonlinear_Data f_nonlinear_data "Additional data for function f_nonlinear";
          input Real x_tol = 100 * Modelica.Constants.eps "Relative tolerance of the result";
          output Real x_zero "f_nonlinear(x_zero) = y_zero";
        protected
          constant Real eps = Modelica.Constants.eps "Machine epsilon";
          constant Real x_eps = 1e-10 "Slight modification of x_min, x_max, since x_min, x_max are usually exactly at the borders T_min/h_min and then small numeric noise may make the interval invalid";
          Real x_min2 = x_min - x_eps;
          Real x_max2 = x_max + x_eps;
          Real a = x_min2 "Current best minimum interval value";
          Real b = x_max2 "Current best maximum interval value";
          Real c "Intermediate point a <= c <= b";
          Real d;
          Real e "b - a";
          Real m;
          Real s;
          Real p;
          Real q;
          Real r;
          Real tol;
          Real fa "= f_nonlinear(a) - y_zero";
          Real fb "= f_nonlinear(b) - y_zero";
          Real fc;
          Boolean found = false;
        algorithm
// Check that f(x_min) and f(x_max) have different sign
          fa := f_nonlinear(x_min2, pressure, X, f_nonlinear_data) - y_zero;
          fb := f_nonlinear(x_max2, pressure, X, f_nonlinear_data) - y_zero;
          fc := fb;
          if fa > 0.0 and fb > 0.0 or fa < 0.0 and fb < 0.0 then
            error("The arguments x_min and x_max to OneNonLinearEquation.solve(..)\n" + "do not bracket the root of the single non-linear equation:\n" + "  x_min  = " + String(x_min2) + "\n" + "  x_max  = " + String(x_max2) + "\n" + "  y_zero = " + String(y_zero) + "\n" + "  fa = f(x_min) - y_zero = " + String(fa) + "\n" + "  fb = f(x_max) - y_zero = " + String(fb) + "\n" + "fa and fb must have opposite sign which is not the case");
          end if;
// Initialize variables
          c := a;
          fc := fa;
          e := b - a;
          d := e;
// Search loop
          while not found loop
            if abs(fc) < abs(fb) then
              a := b;
              b := c;
              c := a;
              fa := fb;
              fb := fc;
              fc := fa;
            end if;
            tol := 2 * eps * abs(b) + x_tol;
            m := (c - b) / 2;
            if abs(m) <= tol or fb == 0.0 then
              found := true;
              x_zero := b;
            else
              if abs(e) < tol or abs(fa) <= abs(fb) then
                e := m;
                d := e;
              else
                s := fb / fa;
                if a == c then
                  p := 2 * m * s;
                  q := 1 - s;
                else
                  q := fa / fc;
                  r := fb / fc;
                  p := s * (2 * m * q * (q - r) - (b - a) * (r - 1));
                  q := (q - 1) * (r - 1) * (s - 1);
                end if;
                if p > 0 then
                  q := -q;
                else
                  p := -p;
                end if;
                s := e;
                e := d;
                if 2 * p < 3 * m * q - abs(tol * q) and p < abs(0.5 * s * q) then
                  d := p / q;
                else
                  e := m;
                  d := e;
                end if;
              end if;
              a := b;
              fa := fb;
              b := b + (if abs(d) > tol then d else if m > 0 then tol else -tol);
              fb := f_nonlinear(b, pressure, X, f_nonlinear_data) - y_zero;
              if fb > 0 and fc > 0 or fb < 0 and fc < 0 then
                c := a;
                fc := fa;
                e := b - a;
                d := e;
              end if;
            end if;
          end while;
// root found (interval is small enough)
// Determine if a bisection is needed
// linear interpolation
// inverse quadratic interpolation
// interpolation successful
// use bi-section
// Best guess value is defined as "a"
// initialize variables
        end solve;
        annotation(
          Documentation(info = "<html>
<p>
This function should currently only be used in Modelica.Media,
since it might be replaced in the future by another strategy,
where the tool is responsible for the solution of the non-linear
equation.
</p>

<p>
This library determines the solution of one non-linear algebraic equation \"y=f(x)\"
in one unknown \"x\" in a reliable way. As input, the desired value y of the
non-linear function has to be given, as well as an interval x_min, x_max that
contains the solution, i.e., \"f(x_min) - y\" and \"f(x_max) - y\" must
have a different sign. If possible, a smaller interval is computed by
inverse quadratic interpolation (interpolating with a quadratic polynomial
through the last 3 points and computing the zero). If this fails,
bisection is used, which always reduces the interval by a factor of 2.
The inverse quadratic interpolation method has superlinear convergence.
This is roughly the same convergence rate as a globally convergent Newton
method, but without the need to compute derivatives of the non-linear
function. The solver function is a direct mapping of the Algol 60 procedure
\"zero\" to Modelica, from:
</p>

<dl>
<dt> Brent R.P.:</dt>
<dd> <b>Algorithms for Minimization without derivatives</b>.
     Prentice Hall, 1973, pp. 58-59.</dd>
</dl>

<p>
Due to current limitations of the
Modelica language (not possible to pass a function reference to a function),
the construction to use this solver on a user-defined function is a bit
complicated (this method is from Hans Olsson, Dassault Syst&egrave;mes AB). A user has to
provide a package in the following way:
</p>

<pre>
  <b>package</b> MyNonLinearSolver
    <b>extends</b> OneNonLinearEquation;

    <b>redeclare record extends</b> Data
      // Define data to be passed to user function
      ...
    <b>end</b> Data;

    <b>redeclare function extends</b> f_nonlinear
    <b>algorithm</b>
       // Compute the non-linear equation: y = f(x, Data)
    <b>end</b> f_nonlinear;

    // Dummy definition that has to be present for current Dymola
    <b>redeclare function extends</b> solve
    <b>end</b> solve;
  <b>end</b> MyNonLinearSolver;

  x_zero = MyNonLinearSolver.solve(y_zero, x_min, x_max, data=data);
</pre>
</html>"));
      end OneNonLinearEquation;
      annotation(
        Documentation(info = "<HTML><h4>Package description</h4>
      <p>Package Modelica.Media.Common provides records and functions shared by many of the property sub-packages.
      High accuracy fluid property models share a lot of common structure, even if the actual models are different.
      Common data structures and computations shared by these property models are collected in this library.
   </p>

</html>", revisions = "<html>
      <ul>
      <li>First implemented: <i>July, 2000</i>
      by <a href=\"http://www.control.lth.se/~hubertus/\">Hubertus Tummescheit</a>
      for the ThermoFluid Library with help from Jonas Eborn and Falko Jens Wagner
      </li>
      <li>Code reorganization, enhanced documentation, additional functions: <i>December, 2002</i>
      by <a href=\"http://www.control.lth.se/~hubertus/\">Hubertus Tummescheit</a> and move to Modelica
                            properties library.</li>
      <li>Inclusion into Modelica.Media: September 2003 </li>
      </ul>

      <address>Author: Hubertus Tummescheit, <br>
      Lund University<br>
      Department of Automatic Control<br>
      Box 118, 22100 Lund, Sweden<br>
      email: hubertus@control.lth.se
      </address>
</html>"));
    end Common;

    package Air "Medium models for air"
      extends Modelica.Icons.VariantsPackage;

      package SimpleAir "Air: Simple dry air model (0..100 degC)"
        extends Modelica.Icons.MaterialProperty;
        extends Interfaces.PartialSimpleIdealGasMedium(mediumName = "SimpleAir", cp_const = 1005.45, MM_const = 0.0289651159, R_gas = Constants.R / 0.0289651159, eta_const = 1.82e-5, lambda_const = 0.026, T_min = Cv.from_degC(0), T_max = Cv.from_degC(100), fluidConstants = airConstants, Temperature(min = Modelica.SIunits.Conversions.from_degC(0), max = Modelica.SIunits.Conversions.from_degC(100)));
        import Modelica.Constants;
        constant Modelica.Media.Interfaces.Types.Basic.FluidConstants[nS] airConstants = {Modelica.Media.Interfaces.Types.Basic.FluidConstants(iupacName = "simple air", casRegistryNumber = "not a real substance", chemicalFormula = "N2, O2", structureFormula = "N2, O2", molarMass = Modelica.Media.IdealGases.Common.SingleGasesData.N2.MM)} "Constant data for the fluid";
        annotation(
          Documentation(info = "<html>
                            <h4>Simple Ideal gas air model for low temperatures</h4>
                            <p>This model demonstrates how to use the PartialSimpleIdealGas base class to build a
                            simple ideal gas model with a limited temperature validity range.</p>
                            </html>"));
      end SimpleAir;

      package DryAirNasa "Air: Detailed dry air model as ideal gas (200..6000 K)"
        extends Modelica.Icons.MaterialProperty;
        extends IdealGases.Common.SingleGasNasa(mediumName = "Air", data = IdealGases.Common.SingleGasesData.Air, fluidConstants = {IdealGases.Common.FluidData.N2});

        redeclare function dynamicViscosity "Return dynamic viscosity of dry air (simple polynomial, moisture influence small, valid from 123.15 K to 1273.15 K, outside of this range linear extrapolation is used)"
          extends Modelica.Icons.Function;
          input ThermodynamicState state "Thermodynamic state record";
          output DynamicViscosity eta "Dynamic viscosity";
          import Modelica.Media.Incompressible.TableBased.Polynomials_Temp;
        algorithm
          eta := 1e-6 * Polynomials_Temp.evaluateWithRange({9.7391102886305869E-15, -3.1353724870333906E-11, 4.3004876595642225E-08, -3.8228016291758240E-05, 5.0427874367180762E-02, 1.7239260139242528E+01}, Cv.to_degC(123.15), Cv.to_degC(1273.15), Cv.to_degC(state.T));
          annotation(
            smoothOrder = 2,
            Documentation(info = "<html>
<p>Dynamic viscosity is computed from temperature using a simple polynomial for dry air. Range of validity is from 123.15 K to 1273.15 K. The influence of pressure is neglected. </p>
<p>Source: VDI Waermeatlas, 8th edition. </p>
</html>"));
        end dynamicViscosity;

        redeclare function thermalConductivity "Return thermal conductivity of dry air (simple polynomial, moisture influence small, valid from 123.15 K to 1273.15 K, outside of this range linear extrapolation is used)"
          extends Modelica.Icons.Function;
          input ThermodynamicState state "Thermodynamic state record";
          input Integer method = 1 "Dummy for compatibility reasons";
          output ThermalConductivity lambda "Thermal conductivity";
          import Modelica.Media.Incompressible.TableBased.Polynomials_Temp;
          import Cv = Modelica.SIunits.Conversions;
        algorithm
          lambda := 1e-3 * Polynomials_Temp.evaluateWithRange({6.5691470817717812E-15, -3.4025961923050509E-11, 5.3279284846303157E-08, -4.5340839289219472E-05, 7.6129675309037664E-02, 2.4169481088097051E+01}, Cv.to_degC(123.15), Cv.to_degC(1273.15), Cv.to_degC(state.T));
          annotation(
            smoothOrder = 2,
            Documentation(info = "<html>
<p>Thermal conductivity is computed from temperature using a simple polynomial for dry air. Range of validity is from 123.15 K to 1273.15 K. The influence of pressure is neglected. </p>
<p>Source: VDI Waermeatlas, 8th edition. </p>
</html>"));
        end thermalConductivity;
        annotation(
          Documentation(info = "<html>
<p>
    <IMG src=\"modelica://Modelica/Resources/Images/Media/IdealGases/SingleGases/Air.png\">
</p>

<p>
Ideal gas medium model for dry air based on the package <a href=\"modelica://Modelica.Media.IdealGases\">IdealGases</a> with additional functions for dynamic viscosity and thermal conductivity in a limited temperature range.
</p>
</html>"));
      end DryAirNasa;

      package ReferenceAir "ReferenceAir: Detailed dry air model with a large operating range (130 ... 2000 K, 0 ... 2000 MPa) based on Helmholtz equations of state"
        extends Modelica.Icons.VariantsPackage;
        import SI = Modelica.SIunits;
        constant Modelica.Media.Interfaces.Types.TwoPhase.FluidConstants airConstants(each chemicalFormula = "N2+O2+Ar", each structureFormula = "N2+O2+Ar", each casRegistryNumber = "1", each iupacName = "air", each molarMass = 0.02896546, each criticalTemperature = 132.5306, each criticalPressure = 3.786e6, each criticalMolarVolume = 0.02896546 / 342.68, each triplePointTemperature = 63.05 "From N2", each triplePointPressure = 0.1253e5 "From N2", each normalBoilingPoint = 78.903, each meltingPoint = 0, each acentricFactor = 0.0335, each dipoleMoment = 0.0, each hasCriticalData = true, each hasFundamentalEquation = true, each hasAccurateViscosityData = true, each hasAcentricFactor = true);
      protected
        type MolarHeatCapacity = SI.MolarHeatCapacity(min = 0, max = 3.e5, nominal = 3.e1, start = 3.e1) "Type for molar heat capacity with medium specific attributes";
        type MolarDensity = Real(final quantity = "MolarDensity", final unit = "mol/m3", min = 0);
        type IsothermalExpansionCoefficient = Real(min = 0, max = 1e8, unit = "1");
      public
        package Air_ph "ReferenceAir.Air_ph: Detailed dry air model (130 ... 2000 K) explicit in p and h"
          extends Modelica.Icons.MaterialProperty;
          extends Modelica.Media.Air.ReferenceAir.Air_Base(ThermoStates = Modelica.Media.Interfaces.Choices.IndependentVariables.ph, final ph_explicit = true, final dT_explicit = false, final pT_explicit = false);
          annotation(
            Documentation(info = "<html>
<h4>Usage</h4>
<p>
The package Air_ph can be used as any other medium model (see <a href=\"modelica://Modelica.Media.UsersGuide\">User's Guide of Media Library</a> for further information).
</p>
</html>"));
        end Air_ph;

        package Air_pT "ReferenceAir.Air_pT: Detailed dry air model (130 ... 2000 K) explicit in p and T"
          extends Modelica.Icons.MaterialProperty;
          extends Modelica.Media.Air.ReferenceAir.Air_Base(ThermoStates = Modelica.Media.Interfaces.Choices.IndependentVariables.pT, final ph_explicit = false, final dT_explicit = false, final pT_explicit = true);
          annotation(
            Documentation(info = "<html>
<h4>Usage</h4>
<p>
The package Air_pT can be used as any other medium model (see <a href=\"modelica://Modelica.Media.UsersGuide\">User's Guide of Media Library</a> for further information).
</p>
</html>"));
        end Air_pT;
      public
        package Air_dT "ReferenceAir.Air_dT: Detailed dry air model (130 ... 2000 K) explicit in d and T"
          extends Modelica.Icons.MaterialProperty;
          extends Modelica.Media.Air.ReferenceAir.Air_Base(ThermoStates = Modelica.Media.Interfaces.Choices.IndependentVariables.dTX, final ph_explicit = false, final dT_explicit = true, final pT_explicit = false);
          annotation(
            Documentation(info = "<html>
<h4>Usage</h4>
<p>
The package Air_dT can be used as any other medium model (see <a href=\"modelica://Modelica.Media.UsersGuide\">User's Guide of Media Library</a> for further information).
</p>
</html>"));
        end Air_dT;
      public
        partial package Air_Base "Properties of dry air calculated using the equation of state by Lemmon et. al."
          extends Modelica.Media.Interfaces.PartialPureSubstance(mediumName = "Air", substanceNames = {"air"}, singleState = false, SpecificEnthalpy(start = 1.0e5, nominal = 5.0e5), Density(start = 1.0, nominal = 1.2), AbsolutePressure(start = 1e5, nominal = 1e5, min = 1.0, max = 2000e6), Temperature(start = 273.15, nominal = 293.15, min = 130, max = 2000));
          constant Boolean ph_explicit "True if explicit in pressure and specific enthalpy";
          constant Boolean dT_explicit "True if explicit in density and temperature";
          constant Boolean pT_explicit "True if explicit in pressure and temperature";

          redeclare record extends ThermodynamicState "Thermodynamic state"
              SpecificEnthalpy h "Specific enthalpy";
              Density d "Density";
              Temperature T "Temperature";
              AbsolutePressure p "Pressure";
          end ThermodynamicState;

          redeclare model extends BaseProperties(h(stateSelect = if ph_explicit and preferredMediumStates then StateSelect.prefer else StateSelect.default), d(stateSelect = if dT_explicit and preferredMediumStates then StateSelect.prefer else StateSelect.default), T(stateSelect = if (pT_explicit or dT_explicit) and preferredMediumStates then StateSelect.prefer else StateSelect.default), p(stateSelect = if (pT_explicit or ph_explicit) and preferredMediumStates then StateSelect.prefer else StateSelect.default)) "Base properties of air"
            equation
              MM = ReferenceAir.Air_Utilities.Basic.Constants.MM;
              if dT_explicit then
                p = pressure_dT(d, T);
                h = specificEnthalpy_dT(d, T);
              elseif ph_explicit then
                d = density_ph(p, h);
                T = temperature_ph(p, h);
              else
                h = specificEnthalpy_pT(p, T);
                d = density_pT(p, T);
              end if;
              u = h - p / d;
              R = Constants.R;
              h = state.h;
              p = state.p;
              T = state.T;
              d = state.d;
          end BaseProperties;

          redeclare function density_ph "Computes density as a function of pressure and specific enthalpy"
            extends Modelica.Icons.Function;
            input AbsolutePressure p "Pressure";
            input SpecificEnthalpy h "Specific enthalpy";
            output Density d "Density";
          algorithm
            d := Air_Utilities.rho_ph(p, h);
            annotation(
              Inline = true);
          end density_ph;

          redeclare function temperature_ph "Computes temperature as a function of pressure and specific enthalpy"
            extends Modelica.Icons.Function;
            input AbsolutePressure p "Pressure";
            input SpecificEnthalpy h "Specific enthalpy";
            output Temperature T "Temperature";
          algorithm
            T := Air_Utilities.T_ph(p, h);
            annotation(
              Inline = true);
          end temperature_ph;

          redeclare function temperature_ps "Compute temperature from pressure and specific enthalpy"
            extends Modelica.Icons.Function;
            input AbsolutePressure p "Pressure";
            input SpecificEntropy s "Specific entropy";
            output Temperature T "Temperature";
          algorithm
            T := Air_Utilities.T_ps(p, s);
            annotation(
              Inline = true);
          end temperature_ps;

          redeclare function density_ps "Computes density as a function of pressure and specific enthalpy"
            extends Modelica.Icons.Function;
            input AbsolutePressure p "Pressure";
            input SpecificEntropy s "Specific entropy";
            output Density d "Density";
          algorithm
            d := Air_Utilities.rho_ps(p, s);
            annotation(
              Inline = true);
          end density_ps;

          redeclare function pressure_dT "Computes pressure as a function of density and temperature"
            extends Modelica.Icons.Function;
            input Density d "Density";
            input Temperature T "Temperature";
            output AbsolutePressure p "Pressure";
          algorithm
            p := Air_Utilities.p_dT(d, T);
            annotation(
              Inline = true);
          end pressure_dT;

          redeclare function specificEnthalpy_dT "Computes specific enthalpy as a function of density and temperature"
            extends Modelica.Icons.Function;
            input Density d "Density";
            input Temperature T "Temperature";
            output SpecificEnthalpy h "Specific enthalpy";
          algorithm
            h := Air_Utilities.h_dT(d, T);
            annotation(
              Inline = true);
          end specificEnthalpy_dT;

          redeclare function specificEnthalpy_pT "Computes specific enthalpy as a function of pressure and temperature"
            extends Modelica.Icons.Function;
            input AbsolutePressure p "Pressure";
            input Temperature T "Temperature";
            output SpecificEnthalpy h "Specific enthalpy";
          algorithm
            h := Air_Utilities.h_pT(p, T);
            annotation(
              Inline = true);
          end specificEnthalpy_pT;

          redeclare function specificEnthalpy_ps "Computes specific enthalpy as a function of pressure and temperature"
            extends Modelica.Icons.Function;
            input AbsolutePressure p "Pressure";
            input SpecificEntropy s "Specific entropy";
            output SpecificEnthalpy h "Specific enthalpy";
          algorithm
            h := Air_Utilities.h_ps(p, s);
            annotation(
              Inline = true);
          end specificEnthalpy_ps;

          redeclare function density_pT "Computes density as a function of pressure and temperature"
            extends Modelica.Icons.Function;
            input AbsolutePressure p "Pressure";
            input Temperature T "Temperature";
            output Density d "Density";
          algorithm
            d := Air_Utilities.rho_pT(p, T);
            annotation(
              Inline = true);
          end density_pT;

          redeclare function extends dynamicViscosity "Return dynamic viscosity as a function of the thermodynamic state record"
            algorithm
              eta := Air_Utilities.Transport.eta_dT(state.d, state.T);
            annotation(
              Inline = true);
          end dynamicViscosity;

          redeclare function extends thermalConductivity "Thermal conductivity of air"
            algorithm
              lambda := Air_Utilities.Transport.lambda_dT(state.d, state.T);
            annotation(
              Inline = true);
          end thermalConductivity;

          redeclare function extends pressure "Return pressure of ideal gas"
            algorithm
              p := state.p;
            annotation(
              Inline = true);
          end pressure;

          redeclare function extends temperature "Return temperature of ideal gas"
            algorithm
              T := state.T;
            annotation(
              Inline = true);
          end temperature;

          redeclare function extends density "Return density of ideal gas"
            algorithm
              d := state.d;
            annotation(
              Inline = true);
          end density;

          redeclare function extends specificEnthalpy "Return specific enthalpy"
            algorithm
              h := state.h;
            annotation(
              Inline = true);
          end specificEnthalpy;

          redeclare function extends specificInternalEnergy "Return specific internal energy"
            algorithm
              u := state.h - state.p / state.d;
            annotation(
              Inline = true);
          end specificInternalEnergy;

          redeclare function extends specificGibbsEnergy "Return specific Gibbs energy"
            algorithm
              g := state.h - state.T * specificEntropy(state);
            annotation(
              Inline = true);
          end specificGibbsEnergy;

          redeclare function extends specificHelmholtzEnergy "Return specific Helmholtz energy"
            algorithm
              f := state.h - state.p / state.d - state.T * specificEntropy(state);
            annotation(
              Inline = true);
          end specificHelmholtzEnergy;

          redeclare function extends specificEntropy "Specific entropy of air"
            algorithm
              if dT_explicit then
                s := Air_Utilities.s_dT(state.d, state.T);
              elseif pT_explicit then
                s := Air_Utilities.s_pT(state.p, state.T);
              else
                s := Air_Utilities.s_ph(state.p, state.h);
              end if;
          end specificEntropy;

          redeclare function extends specificHeatCapacityCp "Specific heat capacity at constant pressure of air"
            algorithm
              if dT_explicit then
                cp := Air_Utilities.cp_dT(state.d, state.T);
              elseif pT_explicit then
                cp := Air_Utilities.cp_pT(state.p, state.T);
              else
                cp := Air_Utilities.cp_ph(state.p, state.h);
              end if;
          end specificHeatCapacityCp;

          redeclare function extends specificHeatCapacityCv "Specific heat capacity at constant volume of air"
            algorithm
              if dT_explicit then
                cv := Air_Utilities.cv_dT(state.d, state.T);
              elseif pT_explicit then
                cv := Air_Utilities.cv_pT(state.p, state.T);
              else
                cv := Air_Utilities.cv_ph(state.p, state.h);
              end if;
          end specificHeatCapacityCv;

          redeclare function extends isentropicExponent "Return isentropic exponent"
            algorithm
              if dT_explicit then
                gamma := Air_Utilities.isentropicExponent_dT(state.d, state.T);
              elseif pT_explicit then
                gamma := Air_Utilities.isentropicExponent_pT(state.p, state.T);
              else
                gamma := Air_Utilities.isentropicExponent_ph(state.p, state.h);
              end if;
          end isentropicExponent;

          redeclare function extends isothermalCompressibility "Isothermal compressibility of air"
            algorithm
              if dT_explicit then
                kappa := Air_Utilities.kappa_dT(state.d, state.T);
              elseif pT_explicit then
                kappa := Air_Utilities.kappa_pT(state.p, state.T);
              else
                kappa := Air_Utilities.kappa_ph(state.p, state.h);
              end if;
          end isothermalCompressibility;

          redeclare function extends isobaricExpansionCoefficient "Isobaric expansion coefficient of air"
            algorithm
              if dT_explicit then
                beta := Air_Utilities.beta_dT(state.d, state.T);
              elseif pT_explicit then
                beta := Air_Utilities.beta_pT(state.p, state.T);
              else
                beta := Air_Utilities.beta_ph(state.p, state.h);
              end if;
          end isobaricExpansionCoefficient;

          redeclare function extends velocityOfSound "Return velocity of sound as a function of the thermodynamic state record"
            algorithm
              if dT_explicit then
                a := Air_Utilities.velocityOfSound_dT(state.d, state.T);
              elseif pT_explicit then
                a := Air_Utilities.velocityOfSound_pT(state.p, state.T);
              else
                a := Air_Utilities.velocityOfSound_ph(state.p, state.h);
              end if;
          end velocityOfSound;

          redeclare function extends density_derh_p "Density derivative by specific enthalpy"
            algorithm
              ddhp := Air_Utilities.ddhp(state.p, state.h);
            annotation(
              Inline = true);
          end density_derh_p;

          redeclare function extends density_derp_h "Density derivative by pressure"
            algorithm
              ddph := Air_Utilities.ddph(state.p, state.h);
            annotation(
              Inline = true);
          end density_derp_h;

          //   redeclare function extends density_derT_p
          //     "Density derivative by temperature"
          //   algorithm
          //     ddTp := IF97_Utilities.ddTp(state.p, state.h, state.phase);
          //   end density_derT_p;
          //
          //   redeclare function extends density_derp_T
          //     "Density derivative by pressure"
          //   algorithm
          //     ddpT := IF97_Utilities.ddpT(state.p, state.h, state.phase);
          //   end density_derp_T;

          redeclare function extends setState_dTX "Return thermodynamic state of air as function of d and T"
            algorithm
              state := ThermodynamicState(d = d, T = T, h = specificEnthalpy_dT(d, T), p = pressure_dT(d, T));
            annotation(
              Inline = true);
          end setState_dTX;

          redeclare function extends setState_phX "Return thermodynamic state of air as function of p and h"
            algorithm
              state := ThermodynamicState(d = density_ph(p, h), T = temperature_ph(p, h), h = h, p = p);
            annotation(
              Inline = true);
          end setState_phX;

          redeclare function extends setState_psX "Return thermodynamic state of air as function of p and s"
            algorithm
              state := ThermodynamicState(d = density_ps(p, s), T = temperature_ps(p, s), h = specificEnthalpy_ps(p, s), p = p);
            annotation(
              Inline = true);
          end setState_psX;

          redeclare function extends setState_pTX "Return thermodynamic state of air as function of p and T"
            algorithm
              state := ThermodynamicState(d = density_pT(p, T), T = T, h = specificEnthalpy_pT(p, T), p = p);
            annotation(
              Inline = true);
          end setState_pTX;

          redeclare function extends setSmoothState "Return thermodynamic state so that it smoothly approximates: if x > 0 then state_a else state_b"
              import Modelica.Media.Common.smoothStep;

            algorithm
              state := ThermodynamicState(p = smoothStep(x, state_a.p, state_b.p, x_small), h = smoothStep(x, state_a.h, state_b.h, x_small), d = density_ph(smoothStep(x, state_a.p, state_b.p, x_small), smoothStep(x, state_a.h, state_b.h, x_small)), T = temperature_ph(smoothStep(x, state_a.p, state_b.p, x_small), smoothStep(x, state_a.h, state_b.h, x_small)));
            annotation(
              Inline = true);
          end setSmoothState;

          redeclare function extends isentropicEnthalpy
            algorithm
              h_is := specificEnthalpy_psX(p_downstream, specificEntropy(refState), reference_X);
            annotation(
              Inline = true);
          end isentropicEnthalpy;

          redeclare function extends molarMass "Return the molar mass of the medium"
            algorithm
              MM := Modelica.Media.Air.ReferenceAir.airConstants.molarMass;
            annotation(
              Inline = true);
          end molarMass;
          annotation(
            Documentation(info = "<HTML>
<p>
This model calculates medium properties
for air in the <b>liquid</b>, <b>gas</b> and <b>two phase</b> regions.
Three variable pairs can be the independent variables of the model:
</p>
<ol>
<li>Pressure <b>p</b> and specific enthalpy <b>h</b> are the most natural choice for general applications. This is the recommended choice for most general purpose applications.</li>
<li>Pressure <b>p</b> and temperature <b>T</b> are the most natural choice for applications where air is always in the same phase (liquid or gas).</li>
<li>Density <b>d</b> and temperature <b>T</b> are explicit variables of the Helmholtz function in the near-critical region and can be the best choice for applications with super-critical or near-critial states.</li>
</ol>
<p>
The following quantities are always computed:
</p>
<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"><b>Variable</b></td>
      <td valign=\"top\"><b>Unit</b></td>
      <td valign=\"top\"><b>Description</b></td></tr>
  <tr><td valign=\"top\">T</td>
      <td valign=\"top\">K</td>
      <td valign=\"top\">temperature</td></tr>
  <tr><td valign=\"top\">u</td>
      <td valign=\"top\">J/kg</td>
      <td valign=\"top\">specific internal energy</td></tr>
  <tr><td valign=\"top\">d</td>
      <td valign=\"top\">kg/m^3</td>
      <td valign=\"top\">density</td></tr>
  <tr><td valign=\"top\">p</td>
      <td valign=\"top\">Pa</td>
      <td valign=\"top\">pressure</td></tr>
  <tr><td valign=\"top\">h</td>
      <td valign=\"top\">J/kg</td>
      <td valign=\"top\">specific enthalpy</td></tr>
</table>
<p>
In some cases additional medium properties are needed.
A component that needs these optional properties has to call
one of the functions listed in
<a href=\"modelica://Modelica.Media.UsersGuide.MediumUsage.OptionalProperties\">
Modelica.Media.UsersGuide.MediumUsage.OptionalProperties</a> and in
<a href=\"modelica://Modelica.Media.UsersGuide.MediumUsage.TwoPhase\">
Modelica.Media.UsersGuide.MediumUsage.TwoPhase</a>.
</p>
<p>Many further properties can be computed. Using the well-known Bridgman's Tables, all first partial derivatives of the standard thermodynamic variables can be computed easily.</p>
</html>"));
        end Air_Base;

        package Air_Utilities "Low level and utility computation for high accuracy dry air properties"
          extends Modelica.Icons.UtilitiesPackage;
          record iter = Inverses.accuracy;

          package Basic "Fundamental equation of state"
            extends Modelica.Icons.BasesPackage;
            constant Modelica.Media.Common.FundamentalConstants Constants(final R_bar = 8.31451, final R = 287.117, final MM = 28.9586E-003, final rhored = 10447.7, final Tred = 132.6312, final pred = 3785020, h_off = 1589557.62320524, s_off = 6610.41237132543);

            function Helmholtz "Helmholtz equation of state"
              extends Modelica.Icons.Function;
              input SI.Density d "Density";
              input SI.Temperature T "Temperature (K)";
              output Modelica.Media.Common.HelmholtzDerivs f "Dimensionless Helmholtz function and dervatives w.r.t. delta and tau";
            protected
              final constant Real[13] N_0 = {0.605719400E-007, -0.210274769E-004, -0.158860716E-003, -0.13841928076E002, 0.17275266575E002, -0.195363420E-003, 0.2490888032E001, 0.791309509, 0.212236768, 0.197938904, 0.2536365E002, 0.1690741E002, 0.8731279E002};
              final constant Real[19] N = {0.118160747229, 0.713116392079, -0.161824192067E001, 0.714140178971E-001, -0.865421396646E-001, 0.134211176704, 0.112626704218E-001, -0.420533228842E-001, 0.349008431982E-001, 0.164957183186E-003, -0.101365037912, -0.173813690970, -0.472103183731E-001, -0.122523554253E-001, -0.146629609713, -0.316055879821E-001, 0.233594806142E-003, 0.148287891978E-001, -0.938782884667E-002};
              final constant Integer[19] i = {1, 1, 1, 2, 3, 3, 4, 4, 4, 6, 1, 3, 5, 6, 1, 3, 11, 1, 3};
              final constant Real[19] j = {0, 0.33, 1.01, 0, 0, 0.15, 0, 0.2, 0.35, 1.35, 1.6, 0.8, 0.95, 1.25, 3.6, 6, 3.25, 3.5, 15};
              final constant Integer[19] l = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 2, 2, 2, 3, 3};
            algorithm
              f.d := d;
              f.T := T;
              f.R := ReferenceAir.Air_Utilities.Basic.Constants.R;
//Reduced density
              f.delta := d / (ReferenceAir.Air_Utilities.Basic.Constants.MM * ReferenceAir.Air_Utilities.Basic.Constants.rhored);
//Reciprocal reduced temperature
              f.tau := ReferenceAir.Air_Utilities.Basic.Constants.Tred / T;
//Dimensionless Helmholtz equation
              f.f := 0;
//Ideal-gas part
              for k in 1:5 loop
                f.f := f.f + N_0[k] * f.tau ^ (k - 4);
              end for;
              f.f := f.f + log(f.delta) + N_0[6] * f.tau * sqrt(f.tau) + N_0[7] * log(f.tau) + N_0[8] * log(1 - exp(-N_0[11] * f.tau)) + N_0[9] * log(1 - exp(-N_0[12] * f.tau)) + N_0[10] * log(2 / 3 + exp(N_0[13] * f.tau));
//Residual part
              for k in 1:10 loop
                f.f := f.f + N[k] * f.delta ^ i[k] * f.tau ^ j[k];
              end for;
              for k in 11:19 loop
                f.f := f.f + N[k] * f.delta ^ i[k] * f.tau ^ j[k] * exp(-f.delta ^ l[k]);
              end for;
//First derivative of f w.r.t. delta
              f.fdelta := 0;
//Ideal-gas part
              f.fdelta := 1 / f.delta;
//Residual part
              for k in 1:10 loop
                f.fdelta := f.fdelta + i[k] * N[k] * f.delta ^ (i[k] - 1) * f.tau ^ j[k];
              end for;
              for k in 11:19 loop
                f.fdelta := f.fdelta + N[k] * f.delta ^ (i[k] - 1) * f.tau ^ j[k] * exp(-f.delta ^ l[k]) * (i[k] - l[k] * f.delta ^ l[k]);
              end for;
//Second derivative of f w.r.t. delta
              f.fdeltadelta := 0;
//Ideal-gas part
              f.fdeltadelta := -1 / f.delta ^ 2;
//Residual part
              for k in 1:10 loop
                f.fdeltadelta := f.fdeltadelta + i[k] * (i[k] - 1) * N[k] * f.delta ^ (i[k] - 2) * f.tau ^ j[k];
              end for;
              for k in 11:19 loop
                f.fdeltadelta := f.fdeltadelta + N[k] * f.delta ^ (i[k] - 2) * f.tau ^ j[k] * exp(-f.delta ^ l[k]) * ((i[k] - l[k] * f.delta ^ l[k]) * (i[k] - 1 - l[k] * f.delta ^ l[k]) - l[k] ^ 2 * f.delta ^ l[k]);
              end for;
//First derivative of f w.r.t. tau
              f.ftau := 0;
//Ideal-gas part
              for k in 1:5 loop
                f.ftau := f.ftau + (k - 4) * N_0[k] * f.tau ^ (k - 5);
              end for;
              f.ftau := f.ftau + 1.5 * N_0[6] * sqrt(f.tau) + N_0[7] / f.tau + N_0[8] * N_0[11] / (exp(N_0[11] * f.tau) - 1) + N_0[9] * N_0[12] / (exp(N_0[12] * f.tau) - 1) + N_0[10] * N_0[13] / (2 / 3 * exp(-N_0[13] * f.tau) + 1);
//Residual part
              for k in 1:10 loop
                f.ftau := f.ftau + j[k] * N[k] * f.delta ^ i[k] * f.tau ^ (j[k] - 1);
              end for;
              for k in 11:19 loop
                f.ftau := f.ftau + j[k] * N[k] * f.delta ^ i[k] * f.tau ^ (j[k] - 1) * exp(-f.delta ^ l[k]);
              end for;
//Second derivative of f w.r.t. tau
              f.ftautau := 0;
//Ideal-gas part
              for k in 1:3 loop
                f.ftautau := f.ftautau + (k - 4) * (k - 5) * N_0[k] * f.tau ^ (k - 6);
              end for;
              f.ftautau := f.ftautau + 0.75 * N_0[6] / sqrt(f.tau) - N_0[7] / f.tau ^ 2 - N_0[8] * N_0[11] ^ 2 * exp(N_0[11] * f.tau) / (exp(N_0[11] * f.tau) - 1) ^ 2 - N_0[9] * N_0[12] ^ 2 * exp(N_0[12] * f.tau) / (exp(N_0[12] * f.tau) - 1) ^ 2 + 2 / 3 * N_0[10] * N_0[13] ^ 2 * exp(-N_0[13] * f.tau) / (2 / 3 * exp(-N_0[13] * f.tau) + 1) ^ 2;
//Residual part
              for k in 1:10 loop
                f.ftautau := f.ftautau + j[k] * (j[k] - 1) * N[k] * f.delta ^ i[k] * f.tau ^ (j[k] - 2);
              end for;
              for k in 11:19 loop
                f.ftautau := f.ftautau + j[k] * (j[k] - 1) * N[k] * f.delta ^ i[k] * f.tau ^ (j[k] - 2) * exp(-f.delta ^ l[k]);
              end for;
//Mixed derivative of f w.r.t. delta and tau
              f.fdeltatau := 0;
//Residual part (Ideal-gas part is zero)
              for k in 1:10 loop
                f.fdeltatau := f.fdeltatau + i[k] * j[k] * N[k] * f.delta ^ (i[k] - 1) * f.tau ^ (j[k] - 1);
              end for;
              for k in 11:19 loop
                f.fdeltatau := f.fdeltatau + j[k] * N[k] * f.delta ^ (i[k] - 1) * f.tau ^ (j[k] - 1) * exp(-f.delta ^ l[k]) * (i[k] - l[k] * f.delta ^ l[k]);
              end for;
            end Helmholtz;
          end Basic;

          package Inverses "Inverse function"
            extends Modelica.Icons.BasesPackage;

            record accuracy "Accuracy of the iterations"
              extends Modelica.Icons.Record;
              constant Real delp = 1E-001 "Accuracy of p";
              constant Real delh = 1E-009 "Accuracy of h";
              constant Real dels = 1E-006 "Accuracy of s";
            end accuracy;

            function dofpT "Compute d for given p and T"
              extends Modelica.Icons.Function;
              input SI.Pressure p "Pressure";
              input SI.Temperature T "Temperature (K)";
              input SI.Pressure delp "Iteration converged if (p-pre(p) < delp)";
              output SI.Density d "Density";
            protected
              Integer i = 0 "Loop counter";
              Real dp "Pressure difference";
              SI.Density deld "Density step";
              Modelica.Media.Common.HelmholtzDerivs f "Dimensionless Helmholtz function and dervatives w.r.t. delta and tau";
              Modelica.Media.Common.NewtonDerivatives_pT nDerivs "Derivatives needed in Newton iteration";
              Boolean found = false "Flag for iteration success";
            algorithm
              d := p / (ReferenceAir.Air_Utilities.Basic.Constants.R * T);
              while i < 100 and not found loop
                f := Basic.Helmholtz(d, T);
                nDerivs := Modelica.Media.Common.Helmholtz_pT(f);
                dp := nDerivs.p - p;
                if abs(dp) <= delp then
                  found := true;
                end if;
                deld := dp / nDerivs.pd;
                d := d - deld;
                i := i + 1;
              end while;
            end dofpT;

            function dTofph "Return d and T as a function of p and h"
              extends Modelica.Icons.Function;
              input SI.Pressure p "Pressure";
              input SI.SpecificEnthalpy h "Specific enthalpy";
              input SI.Pressure delp "Iteration accuracy";
              input SI.SpecificEnthalpy delh "Iteration accuracy";
              output SI.Density d "Density";
              output SI.Temperature T "Temperature (K)";
            protected
              SI.Temperature Tguess "Initial temperature";
              SI.Density dguess "Initial density";
              Integer i "Iteration counter";
              Real dh "Newton-error in h-direction";
              Real dp "Newton-error in p-direction";
              Real det "Determinant of directional derivatives";
              Real deld "Newton-step in d-direction";
              Real delt "Newton-step in T-direction";
              Modelica.Media.Common.HelmholtzDerivs f "Dimensionless Helmholtz function and dervatives w.r.t. delta and tau";
              Modelica.Media.Common.NewtonDerivatives_ph nDerivs "Derivatives needed in Newton iteration";
              Boolean found = false "Flag for iteration success";
            algorithm
// Stefan Wischhusen: better guess for high temperatures:
              T := h / 1000 + 273.15;
              d := p / (ReferenceAir.Air_Utilities.Basic.Constants.R * T);
              i := 0;
              while i < 100 and not found loop
                f := Basic.Helmholtz(d, T);
                nDerivs := Modelica.Media.Common.Helmholtz_ph(f);
                dh := nDerivs.h - ReferenceAir.Air_Utilities.Basic.Constants.h_off - h;
                dp := nDerivs.p - p;
                if abs(dh) <= delh and abs(dp) <= delp then
                  found := true;
                end if;
                det := nDerivs.ht * nDerivs.pd - nDerivs.pt * nDerivs.hd;
                delt := (nDerivs.pd * dh - nDerivs.hd * dp) / det;
                deld := (nDerivs.ht * dp - nDerivs.pt * dh) / det;
                T := T - delt;
                d := d - deld;
                i := i + 1;
              end while;
            end dTofph;

            function dTofps "Return d and T as a function of p and s"
              extends Modelica.Icons.Function;
              input SI.Pressure p "Pressure";
              input SI.SpecificEntropy s "Specific entropy";
              input SI.Pressure delp "Iteration accuracy";
              input SI.SpecificEntropy dels "Iteration accuracy";
              output SI.Density d "Density";
              output SI.Temperature T "Temperature (K)";
            protected
              SI.Temperature Tguess "Initial temperature";
              SI.Density dguess "Initial density";
              Integer i "Iteration counter";
              Real ds "Newton-error in s-direction";
              Real dp "Newton-error in p-direction";
              Real det "Determinant of directional derivatives";
              Real deld "Newton-step in d-direction";
              Real delt "Newton-step in T-direction";
              Modelica.Media.Common.HelmholtzDerivs f "Dimensionless Helmholtz function and dervatives w.r.t. delta and tau";
              Modelica.Media.Common.NewtonDerivatives_ps nDerivs "Derivatives needed in Newton iteration";
              Boolean found = false "Flag for iteration success";
            algorithm
              T := 273.15;
              d := p / (ReferenceAir.Air_Utilities.Basic.Constants.R * T);
              i := 0;
              while i < 100 and not found loop
                f := Basic.Helmholtz(d, T);
                nDerivs := Modelica.Media.Common.Helmholtz_ps(f);
                ds := nDerivs.s - ReferenceAir.Air_Utilities.Basic.Constants.s_off - s;
                dp := nDerivs.p - p;
                if abs(ds) <= dels and abs(dp) <= delp then
                  found := true;
                end if;
                det := nDerivs.st * nDerivs.pd - nDerivs.pt * nDerivs.sd;
                delt := (nDerivs.pd * ds - nDerivs.sd * dp) / det;
                deld := (nDerivs.st * dp - nDerivs.pt * ds) / det;
                T := T - delt;
                d := d - deld;
                i := i + 1;
              end while;
            end dTofps;
          end Inverses;

          package Transport "Transport properties for air"
            extends Modelica.Icons.BasesPackage;

            function eta_dT "Return dynamic viscosity as a function of d and T"
              extends Modelica.Icons.Function;
              input SI.Density d "Density";
              input SI.Temperature T "Temperature";
              output SI.DynamicViscosity eta "Dynamic viscosity";
            protected
              Real delta = d / (ReferenceAir.Air_Utilities.Basic.Constants.MM * ReferenceAir.Air_Utilities.Basic.Constants.rhored) "Reduced density";
              Real tau = ReferenceAir.Air_Utilities.Basic.Constants.Tred / T "Reciprocal reduced temperature";
              Real Omega "Collision integral";
              SI.DynamicViscosity eta_0 = 0 "Dilute gas viscosity";
              SI.DynamicViscosity eta_r = 0 "Residual fluid viscosity";
              final constant Real[5] b = {0.431, -0.4623, 0.08406, 0.005341, -0.00331};
              final constant Real[5] Nvis = {10.72, 1.122, 0.002019, -8.876, -0.02916};
              final constant Real[5] tvis = {0.2, 0.05, 2.4, 0.6, 3.6};
              final constant Integer[5] dvis = {1, 4, 9, 1, 8};
              final constant Integer[5] lvis = {0, 0, 0, 1, 1};
              final constant Integer[5] gammavis = {0, 0, 0, 1, 1};
            algorithm
              Omega := exp(Modelica.Media.Incompressible.TableBased.Polynomials_Temp.evaluate({b[5], b[4], b[3], b[2], b[1]}, log(T / 103.3)));
              eta_0 := 0.0266958 * sqrt(1000 * ReferenceAir.Air_Utilities.Basic.Constants.MM * T) / (0.36 ^ 2 * Omega);
              for i in 1:5 loop
                eta_r := eta_r + Nvis[i] * tau ^ tvis[i] * delta ^ dvis[i] * exp(-gammavis[i] * delta ^ lvis[i]);
              end for;
              eta := (eta_0 + eta_r) * 1E-006;
            end eta_dT;

            function lambda_dT "Return thermal conductivity as a function of d and T"
              extends Modelica.Icons.Function;
              input SI.Density d "Density";
              input SI.Temperature T "Temperature";
              output SI.ThermalConductivity lambda "Thermal conductivity";
            protected
              Modelica.Media.Common.HelmholtzDerivs f "Dimensionless Helmholtz function and dervatives w.r.t. delta and tau";
              SI.ThermalConductivity lambda_0 = 0 "Dilute gas thermal conductivity";
              SI.ThermalConductivity lambda_r = 0 "Residual fluid thermal conductivity";
              SI.ThermalConductivity lambda_c = 0 "Thermal conductivity critical enhancement";
              Real Omega "Collision integral";
              SI.DynamicViscosity eta_0 = 0 "Dilute gas viscosity";
              Real pddT;
              Real pddTref;
              Real pdTp;
              Real xi;
              Real xiref;
              Real Omega_tilde;
              Real Omega_0_tilde;
              Real cv;
              Real cp;
              final constant Real[5] b = {0.431, -0.4623, 0.08406, 0.005341, -0.00331};
              final constant Real[9] Ncon = {1.308, 1.405, -1.036, 8.743, 14.76, -16.62, 3.793, -6.142, -0.3778};
              final constant Real[9] tcon = {0.0, -1.1, -0.3, 0.1, 0.0, 0.5, 2.7, 0.3, 1.3};
              final constant Integer[9] dcon = {0, 0, 0, 1, 2, 3, 7, 7, 11};
              final constant Integer[9] lcon = {0, 0, 0, 0, 0, 2, 2, 2, 2};
              final constant Integer[9] gammacon = {0, 0, 0, 0, 0, 1, 1, 1, 1};
            algorithm
//chi_tilde in at the reference temperature 265.262
              f := Basic.Helmholtz(d, 265.262);
              pddTref := ReferenceAir.Air_Utilities.Basic.Constants.R_bar * 265.262 * (1 + 2 * f.delta * (f.fdelta - 1 / f.delta) + f.delta ^ 2 * (f.fdeltadelta + 1 / f.delta ^ 2));
              xiref := ReferenceAir.Air_Utilities.Basic.Constants.pred * (d / ReferenceAir.Air_Utilities.Basic.Constants.MM) / ReferenceAir.Air_Utilities.Basic.Constants.rhored ^ 2 / pddTref;
//calculating f at the given state
              f := Basic.Helmholtz(d, T);
              Omega := exp(Modelica.Media.Incompressible.TableBased.Polynomials_Temp.evaluate({b[5], b[4], b[3], b[2], b[1]}, log(T / 103.3)));
//Ideal-gas part of dynamic viscosity
              eta_0 := 0.0266958 * sqrt(1000 * ReferenceAir.Air_Utilities.Basic.Constants.MM * T) / (0.36 ^ 2 * Omega);
//Ideal-gas part of thermal conductivity
              lambda_0 := Ncon[1] * eta_0 + Ncon[2] * f.tau ^ tcon[2] + Ncon[3] * f.tau ^ tcon[3];
//Residual part of thermal conductivity
              for i in 4:9 loop
                lambda_r := lambda_r + Ncon[i] * f.tau ^ tcon[i] * f.delta ^ dcon[i] * exp(-gammacon[i] * f.delta ^ lcon[i]);
              end for;
//Derivative of p w.r.t. d at constant temperature
              pddT := ReferenceAir.Air_Utilities.Basic.Constants.R * T * (1 + 2 * f.delta * (f.fdelta - 1 / f.delta) + f.delta ^ 2 * (f.fdeltadelta + 1 / f.delta ^ 2));
//chi_tilde at the given state
              xi := ReferenceAir.Air_Utilities.Basic.Constants.pred * (d / ReferenceAir.Air_Utilities.Basic.Constants.MM) / ReferenceAir.Air_Utilities.Basic.Constants.rhored ^ 2 / (pddT * ReferenceAir.Air_Utilities.Basic.Constants.MM);
//Thermal conductivity critical enhancement
              xi := xi - xiref * 265.262 / T;
              if xi <= 0 then
                lambda_c := 0;
              else
                xi := 0.11 * (xi / 0.055) ^ (0.63 / 1.2415);
                pdTp := ReferenceAir.Air_Utilities.Basic.Constants.R * d * (1 + f.delta * (f.fdelta - 1 / f.delta) - f.delta * f.tau * f.fdeltatau);
                cv := ReferenceAir.Air_Utilities.Basic.Constants.R * (-f.tau * f.tau * f.ftautau);
                cp := cv + T * pdTp * pdTp / (d * d * pddT);
                Omega_tilde := 2 / Modelica.Constants.pi * ((cp - cv) / cp * atan(xi / 0.31) + cv / cp * xi / 0.31);
                Omega_0_tilde := 2 / Modelica.Constants.pi * (1 - exp(-1 / (0.31 / xi + 1 / 3 * (xi / 0.31) ^ 2 * (ReferenceAir.Air_Utilities.Basic.Constants.rhored / (d / ReferenceAir.Air_Utilities.Basic.Constants.MM)) ^ 2)));
                lambda_c := d * cp * 1.380658E-023 * 1.01 * T / (6 * Modelica.Constants.pi * xi * eta_dT(d, T)) * (Omega_tilde - Omega_0_tilde) * 1E012;
              end if;
//Derivative of p w.r.t. T at constant p
//Specific isochoric heat capacity
//Specific isobaric heat capacity
              lambda := (lambda_0 + lambda_r + lambda_c) / 1000;
            end lambda_dT;
          end Transport;

          function airBaseProp_ps "Intermediate property record for air"
            extends Modelica.Icons.Function;
            input Modelica.SIunits.Pressure p "Pressure";
            input Modelica.SIunits.SpecificEntropy s "Specific entropy";
            output Common.AuxiliaryProperties aux "Auxiliary record";
          protected
            Modelica.Media.Common.HelmholtzDerivs f "Dimensionless Helmholtz funcion and dervatives w.r.t. delta and tau";
          algorithm
            aux.p := p;
            aux.s := s;
            aux.R := ReferenceAir.Air_Utilities.Basic.Constants.R;
            (aux.rho, aux.T) := Inverses.dTofps(p = p, s = s, delp = iter.delp, dels = iter.dels);
            f := Basic.Helmholtz(aux.rho, aux.T);
            aux.h := aux.R * aux.T * (f.tau * f.ftau + f.delta * f.fdelta) - ReferenceAir.Air_Utilities.Basic.Constants.h_off;
            aux.pd := aux.R * aux.T * f.delta * (2 * f.fdelta + f.delta * f.fdeltadelta);
            aux.pt := aux.R * aux.rho * f.delta * (f.fdelta - f.tau * f.fdeltatau);
            aux.cv := aux.R * (-f.tau * f.tau * f.ftautau);
            aux.cp := aux.cv + aux.T * aux.pt * aux.pt / (aux.rho * aux.rho * aux.pd);
            aux.vp := -1 / (aux.rho * aux.rho) * 1 / aux.pd;
            aux.vt := aux.pt / (aux.rho * aux.rho * aux.pd);
          end airBaseProp_ps;

          function rho_props_ps "Density as function of pressure and specific entropy"
            extends Modelica.Icons.Function;
            input Modelica.SIunits.Pressure p "Pressure";
            input Modelica.SIunits.SpecificEntropy s "Specific entropy";
            input Common.AuxiliaryProperties aux "Auxiliary record";
            output Modelica.SIunits.Density rho "Density";
          algorithm
            rho := aux.rho;
            annotation(
              Inline = false,
              LateInline = true);
          end rho_props_ps;

          function rho_ps "Density as function of pressure and specific entropy"
            extends Modelica.Icons.Function;
            input Modelica.SIunits.Pressure p "Pressure";
            input Modelica.SIunits.SpecificEntropy s "Specific entropy";
            output Modelica.SIunits.Density rho "Density";
          algorithm
            rho := rho_props_ps(p, s, Air_Utilities.airBaseProp_ps(p, s));
          end rho_ps;

          function T_props_ps "Temperature as function of pressure and specific entropy"
            extends Modelica.Icons.Function;
            input Modelica.SIunits.Pressure p "Pressure";
            input Modelica.SIunits.SpecificEntropy s "Specific entropy";
            input Common.AuxiliaryProperties aux "Auxiliary record";
            output Modelica.SIunits.Temperature T "Temperature";
          algorithm
            T := aux.T;
            annotation(
              Inline = false,
              LateInline = true);
          end T_props_ps;

          function T_ps "Temperature as function of pressure and specific entropy"
            extends Modelica.Icons.Function;
            input Modelica.SIunits.Pressure p "Pressure";
            input Modelica.SIunits.SpecificEntropy s "Specific entropy";
            output Modelica.SIunits.Temperature T "Temperature";
          algorithm
            T := T_props_ps(p, s, Air_Utilities.airBaseProp_ps(p, s));
          end T_ps;

          function h_props_ps "Specific enthalpy as function or pressure and temperature"
            extends Modelica.Icons.Function;
            input Modelica.SIunits.Pressure p "Pressure";
            input Modelica.SIunits.SpecificEntropy s "Specific entropy";
            input Common.AuxiliaryProperties aux "Auxiliary record";
            output Modelica.SIunits.SpecificEnthalpy h "Specific enthalpy";
          algorithm
            h := aux.h;
            annotation(
              Inline = false,
              LateInline = true);
          end h_props_ps;

          function h_ps "Specific enthalpy as function or pressure and temperature"
            extends Modelica.Icons.Function;
            input Modelica.SIunits.Pressure p "Pressure";
            input Modelica.SIunits.SpecificEntropy s "Specific entropy";
            output Modelica.SIunits.SpecificEnthalpy h "Specific enthalpy";
          algorithm
            h := h_props_ps(p, s, Air_Utilities.airBaseProp_ps(p, s));
          end h_ps;

          function airBaseProp_ph "Intermediate property record for air"
            extends Modelica.Icons.Function;
            input Modelica.SIunits.Pressure p "Pressure";
            input Modelica.SIunits.SpecificEnthalpy h "Specific enthalpy";
            output Common.AuxiliaryProperties aux "Auxiliary record";
          protected
            Modelica.Media.Common.HelmholtzDerivs f "Dimensionless Helmholtz funcion and dervatives w.r.t. delta and tau";
            Integer error "Error flag for inverse iterations";
          algorithm
            aux.p := p;
            aux.h := h;
            aux.R := ReferenceAir.Air_Utilities.Basic.Constants.R;
            (aux.rho, aux.T) := Inverses.dTofph(p, h, delp = iter.delp, delh = iter.delh);
            f := Basic.Helmholtz(aux.rho, aux.T);
            aux.s := aux.R * (f.tau * f.ftau - f.f) - ReferenceAir.Air_Utilities.Basic.Constants.s_off;
            aux.pd := aux.R * aux.T * f.delta * (2 * f.fdelta + f.delta * f.fdeltadelta);
            aux.pt := aux.R * aux.rho * f.delta * (f.fdelta - f.tau * f.fdeltatau);
            aux.cv := aux.R * (-f.tau * f.tau * f.ftautau);
            aux.cp := aux.cv + aux.T * aux.pt * aux.pt / (aux.rho * aux.rho * aux.pd);
            aux.vp := -1 / (aux.rho * aux.rho) * 1 / aux.pd;
            aux.vt := aux.pt / (aux.rho * aux.rho * aux.pd);
          end airBaseProp_ph;

          function rho_props_ph "Density as function of pressure and specific enthalpy"
            extends Modelica.Icons.Function;
            input Modelica.SIunits.Pressure p "Pressure";
            input Modelica.SIunits.SpecificEnthalpy h "Specific enthalpy";
            input Common.AuxiliaryProperties aux "Auxiliary record";
            output Modelica.SIunits.Density rho "Density";
          algorithm
            rho := aux.rho;
            annotation(
              derivative(noDerivative = aux) = rho_ph_der,
              Inline = false,
              LateInline = true);
          end rho_props_ph;

          function rho_ph "Density as function of pressure and specific enthalpy"
            extends Modelica.Icons.Function;
            input Modelica.SIunits.Pressure p "Pressure";
            input Modelica.SIunits.SpecificEnthalpy h "Specific enthalpy";
            output Modelica.SIunits.Density rho "Density";
          algorithm
            rho := rho_props_ph(p, h, Air_Utilities.airBaseProp_ph(p, h));
          end rho_ph;

          function rho_ph_der "Derivative function of rho_ph"
            extends Modelica.Icons.Function;
            input Modelica.SIunits.Pressure p "Pressure";
            input Modelica.SIunits.SpecificEnthalpy h "Specific enthalpy";
            input Common.AuxiliaryProperties aux "Auxiliary record";
            input Real p_der "Derivative of pressure";
            input Real h_der "Derivative of specific enthalpy";
            output Real rho_der "Derivative of density";
          algorithm
            rho_der := aux.rho * (aux.cv * aux.rho + aux.pt) / (aux.rho * aux.rho * aux.pd * aux.cv + aux.T * aux.pt * aux.pt) * p_der + (-aux.rho * aux.rho * aux.pt / (aux.rho * aux.rho * aux.pd * aux.cv + aux.T * aux.pt * aux.pt)) * h_der;
          end rho_ph_der;

          function T_props_ph "Temperature as function of pressure and specific enthalpy"
            extends Modelica.Icons.Function;
            input Modelica.SIunits.Pressure p "Pressure";
            input Modelica.SIunits.SpecificEnthalpy h "Specific enthalpy";
            input Common.AuxiliaryProperties aux "Auxiliary record";
            output Modelica.SIunits.Temperature T "Temperature";
          algorithm
            T := aux.T;
            annotation(
              derivative(noDerivative = aux) = T_ph_der,
              Inline = false,
              LateInline = true);
          end T_props_ph;

          function T_ph "Temperature as function of pressure and specific enthalpy"
            extends Modelica.Icons.Function;
            input Modelica.SIunits.Pressure p "Pressure";
            input Modelica.SIunits.SpecificEnthalpy h "Specific enthalpy";
            output Modelica.SIunits.Temperature T "Temperature";
          algorithm
            T := T_props_ph(p, h, Air_Utilities.airBaseProp_ph(p, h));
          end T_ph;

          function T_ph_der "Derivative function of T_ph"
            extends Modelica.Icons.Function;
            input Modelica.SIunits.Pressure p "Pressure";
            input Modelica.SIunits.SpecificEnthalpy h "Specific enthalpy";
            input Common.AuxiliaryProperties aux "Auxiliary record";
            input Real p_der "Derivative of pressure";
            input Real h_der "Derivative of specific enthalpy";
            output Real T_der "Derivative of temperature";
          algorithm
            T_der := ((-aux.rho * aux.pd) + aux.T * aux.pt) / (aux.rho * aux.rho * aux.pd * aux.cv + aux.T * aux.pt * aux.pt) * p_der + aux.rho * aux.rho * aux.pd / (aux.rho * aux.rho * aux.pd * aux.cv + aux.T * aux.pt * aux.pt) * h_der;
          end T_ph_der;

          function s_props_ph "Specific entropy as function of pressure and specific enthalpy"
            extends Modelica.Icons.Function;
            input Modelica.SIunits.Pressure p "Pressure";
            input Modelica.SIunits.SpecificEnthalpy h "Specific enthalpy";
            input Common.AuxiliaryProperties aux "Auxiliary record";
            output Modelica.SIunits.SpecificEntropy s "Specific entropy";
          algorithm
            s := aux.s;
            annotation(
              derivative(noDerivative = aux) = s_ph_der,
              Inline = false,
              LateInline = true);
          end s_props_ph;

          function s_ph "Specific entropy as function of pressure and specific enthalpy"
            extends Modelica.Icons.Function;
            input Modelica.SIunits.Pressure p "Pressure";
            input Modelica.SIunits.SpecificEnthalpy h "Specific enthalpy";
            output Modelica.SIunits.SpecificEntropy s "Specific entropy";
          algorithm
            s := s_props_ph(p, h, Air_Utilities.airBaseProp_ph(p, h));
          end s_ph;

          function s_ph_der "Specific entropy as function of pressure and specific enthalpy"
            extends Modelica.Icons.Function;
            input Modelica.SIunits.Pressure p "Pressure";
            input Modelica.SIunits.SpecificEnthalpy h "Specific enthalpy";
            input Common.AuxiliaryProperties aux "Auxiliary record";
            input Real p_der "Derivative of pressure";
            input Real h_der "Derivative of specific enthalpy";
            output Real s_der "Derivative of entropy";
          algorithm
            s_der := (-1 / (aux.rho * aux.T) * p_der) + 1 / aux.T * h_der;
          end s_ph_der;

          function cv_props_ph "Specific heat capacity at constant volume as function of pressure and specific enthalpy"
            extends Modelica.Icons.Function;
            input Modelica.SIunits.Pressure p "Pressure";
            input Modelica.SIunits.SpecificEnthalpy h "Specific enthalpy";
            input Common.AuxiliaryProperties aux "Auxiliary record";
            output Modelica.SIunits.SpecificHeatCapacity cv "Specific heat capacity";
          algorithm
            cv := aux.cv;
            annotation(
              Inline = false,
              LateInline = true);
          end cv_props_ph;

          function cv_ph "Specific heat capacity at constant volume as function of pressure and specific enthalpy"
            extends Modelica.Icons.Function;
            input Modelica.SIunits.Pressure p "Pressure";
            input Modelica.SIunits.SpecificEnthalpy h "Specific enthalpy";
            output Modelica.SIunits.SpecificHeatCapacity cv "Specific heat capacity";
          algorithm
            cv := cv_props_ph(p, h, Air_Utilities.airBaseProp_ph(p, h));
          end cv_ph;

          function cp_props_ph "Specific heat capacity at constant pressure as function of pressure and specific enthalpy"
            extends Modelica.Icons.Function;
            input Modelica.SIunits.Pressure p "Pressure";
            input Modelica.SIunits.SpecificEnthalpy h "Specific enthalpy";
            input Common.AuxiliaryProperties aux "Auxiliary record";
            output Modelica.SIunits.SpecificHeatCapacity cp "Specific heat capacity";
          algorithm
            cp := aux.cp;
            annotation(
              Inline = false,
              LateInline = true);
          end cp_props_ph;

          function cp_ph "Specific heat capacity at constant pressure as function of pressure and specific enthalpy"
            extends Modelica.Icons.Function;
            input Modelica.SIunits.Pressure p "Pressure";
            input Modelica.SIunits.SpecificEnthalpy h "Specific enthalpy";
            output Modelica.SIunits.SpecificHeatCapacity cp "Specific heat capacity";
          algorithm
            cp := cp_props_ph(p, h, Air_Utilities.airBaseProp_ph(p, h));
          end cp_ph;

          function beta_props_ph "Isobaric expansion coefficient as function of pressure and specific enthalpy"
            extends Modelica.Icons.Function;
            input Modelica.SIunits.Pressure p "Pressure";
            input Modelica.SIunits.SpecificEnthalpy h "Specific enthalpy";
            input Common.AuxiliaryProperties aux "Auxiliary record";
            output Modelica.SIunits.RelativePressureCoefficient beta "Isobaric expansion coefficient";
          algorithm
            beta := aux.pt / (aux.rho * aux.pd);
            annotation(
              Inline = false,
              LateInline = true);
          end beta_props_ph;

          function beta_ph "Isobaric expansion coefficient as function of pressure and specific enthalpy"
            extends Modelica.Icons.Function;
            input Modelica.SIunits.Pressure p "Pressure";
            input Modelica.SIunits.SpecificEnthalpy h "Specific enthalpy";
            output Modelica.SIunits.RelativePressureCoefficient beta "Isobaric expansion coefficient";
          algorithm
            beta := beta_props_ph(p, h, Air_Utilities.airBaseProp_ph(p, h));
          end beta_ph;

          function kappa_props_ph "Isothermal compressibility factor as function of pressure and specific enthalpy"
            extends Modelica.Icons.Function;
            input Modelica.SIunits.Pressure p "Pressure";
            input Modelica.SIunits.SpecificEnthalpy h "Specific enthalpy";
            input Common.AuxiliaryProperties aux "Auxiliary record";
            output Modelica.SIunits.IsothermalCompressibility kappa "Isothermal compressibility factor";
          algorithm
            kappa := 1 / (aux.rho * aux.pd);
            annotation(
              Inline = false,
              LateInline = true);
          end kappa_props_ph;

          function kappa_ph "Isothermal compressibility factor as function of pressure and specific enthalpy"
            extends Modelica.Icons.Function;
            input Modelica.SIunits.Pressure p "Pressure";
            input Modelica.SIunits.SpecificEnthalpy h "Specific enthalpy";
            output Modelica.SIunits.IsothermalCompressibility kappa "Isothermal compressibility factor";
          algorithm
            kappa := kappa_props_ph(p, h, Air_Utilities.airBaseProp_ph(p, h));
          end kappa_ph;

          function velocityOfSound_props_ph "Speed of sound as function of pressure and specific enthalpy"
            extends Modelica.Icons.Function;
            input Modelica.SIunits.Pressure p "Pressure";
            input Modelica.SIunits.SpecificEnthalpy h "Specific enthalpy";
            input Common.AuxiliaryProperties aux "Auxiliary record";
            output Modelica.SIunits.Velocity a "Speed of sound";
          algorithm
            a := sqrt(max(0, aux.pd + aux.pt * aux.pt * aux.T / (aux.rho * aux.rho * aux.cv)));
            annotation(
              Inline = false,
              LateInline = true);
          end velocityOfSound_props_ph;

          function velocityOfSound_ph
            extends Modelica.Icons.Function;
            input Modelica.SIunits.Pressure p "Pressure";
            input Modelica.SIunits.SpecificEnthalpy h "Specific enthalpy";
            output Modelica.SIunits.Velocity a "Speed of sound";
          algorithm
            a := velocityOfSound_props_ph(p, h, Air_Utilities.airBaseProp_ph(p, h));
          end velocityOfSound_ph;

          function isentropicExponent_props_ph "Isentropic exponent as function of pressure and specific enthalpy"
            extends Modelica.Icons.Function;
            input Modelica.SIunits.Pressure p "Pressure";
            input Modelica.SIunits.SpecificEnthalpy h "Specific enthalpy";
            input Common.AuxiliaryProperties aux "Auxiliary record";
            output Real gamma "Isentropic exponent";
          algorithm
            gamma := 1 / (aux.rho * p) * ((aux.pd * aux.cv * aux.rho * aux.rho + aux.pt * aux.pt * aux.T) / aux.cv);
            annotation(
              Inline = false,
              LateInline = true);
          end isentropicExponent_props_ph;

          function isentropicExponent_ph "Isentropic exponent as function of pressure and specific enthalpy"
            extends Modelica.Icons.Function;
            input Modelica.SIunits.Pressure p "Pressure";
            input Modelica.SIunits.SpecificEnthalpy h "Specific enthalpy";
            output Real gamma "Isentropic exponent";
          algorithm
            gamma := isentropicExponent_props_ph(p, h, Air_Utilities.airBaseProp_ph(p, h));
            annotation(
              Inline = false,
              LateInline = true);
          end isentropicExponent_ph;

          function ddph_props "Density derivative by pressure"
            extends Modelica.Icons.Function;
            input Modelica.SIunits.Pressure p "Pressure";
            input Modelica.SIunits.SpecificEnthalpy h "Specific enthalpy";
            input Common.AuxiliaryProperties aux "Auxiliary record";
            output Modelica.SIunits.DerDensityByPressure ddph "Density derivative by pressure";
          algorithm
            ddph := aux.rho * (aux.cv * aux.rho + aux.pt) / (aux.rho * aux.rho * aux.pd * aux.cv + aux.T * aux.pt * aux.pt);
            annotation(
              Inline = false,
              LateInline = true);
          end ddph_props;

          function ddph "Density derivative by pressure"
            extends Modelica.Icons.Function;
            input Modelica.SIunits.Pressure p "Pressure";
            input Modelica.SIunits.SpecificEnthalpy h "Specific enthalpy";
            output Modelica.SIunits.DerDensityByPressure ddph "Density derivative by pressure";
          algorithm
            ddph := ddph_props(p, h, Air_Utilities.airBaseProp_ph(p, h));
          end ddph;

          function ddhp_props "Density derivative by specific enthalpy"
            extends Modelica.Icons.Function;
            input Modelica.SIunits.Pressure p "Pressure";
            input Modelica.SIunits.SpecificEnthalpy h "Specific enthalpy";
            input Common.AuxiliaryProperties aux "Auxiliary record";
            output Modelica.SIunits.DerDensityByEnthalpy ddhp "Density derivative by specific enthalpy";
          algorithm
            ddhp := -aux.rho * aux.rho * aux.pt / (aux.rho * aux.rho * aux.pd * aux.cv + aux.T * aux.pt * aux.pt);
            annotation(
              Inline = false,
              LateInline = true);
          end ddhp_props;

          function ddhp "Density derivative by specific enthalpy"
            extends Modelica.Icons.Function;
            input Modelica.SIunits.Pressure p "Pressure";
            input Modelica.SIunits.SpecificEnthalpy h "Specific enthalpy";
            output Modelica.SIunits.DerDensityByEnthalpy ddhp "Density derivative by specific enthalpy";
          algorithm
            ddhp := ddhp_props(p, h, Air_Utilities.airBaseProp_ph(p, h));
          end ddhp;

          function airBaseProp_pT "Intermediate property record for air (p and T prefered states)"
            extends Modelica.Icons.Function;
            input Modelica.SIunits.Pressure p "Pressure";
            input Modelica.SIunits.Temperature T "Temperature";
            output Common.AuxiliaryProperties aux "Auxiliary record";
          protected
            Modelica.Media.Common.HelmholtzDerivs f "Dimensionless Helmholtz funcion and dervatives w.r.t. delta and tau";
          algorithm
            aux.p := p;
            aux.T := T;
            aux.R := ReferenceAir.Air_Utilities.Basic.Constants.R;
            aux.rho := Inverses.dofpT(p = p, T = T, delp = iter.delp);
            f := Basic.Helmholtz(aux.rho, T);
            aux.h := aux.R * T * (f.tau * f.ftau + f.delta * f.fdelta) - ReferenceAir.Air_Utilities.Basic.Constants.h_off;
            aux.s := aux.R * (f.tau * f.ftau - f.f) - ReferenceAir.Air_Utilities.Basic.Constants.s_off;
            aux.pd := aux.R * T * f.delta * (2 * f.fdelta + f.delta * f.fdeltadelta);
            aux.pt := aux.R * aux.rho * f.delta * (f.fdelta - f.tau * f.fdeltatau);
            aux.cv := aux.R * (-f.tau * f.tau * f.ftautau);
            aux.cp := aux.cv + aux.T * aux.pt * aux.pt / (aux.rho * aux.rho * aux.pd);
            aux.vp := -1 / (aux.rho * aux.rho) * 1 / aux.pd;
            aux.vt := aux.pt / (aux.rho * aux.rho * aux.pd);
          end airBaseProp_pT;

          function rho_props_pT "Density as function or pressure and temperature"
            extends Modelica.Icons.Function;
            input Modelica.SIunits.Pressure p "Pressure";
            input Modelica.SIunits.Temperature T "Temperature";
            input Common.AuxiliaryProperties aux "Auxiliary record";
            output Modelica.SIunits.Density rho "Density";
          algorithm
            rho := aux.rho;
            annotation(
              derivative(noDerivative = aux) = rho_pT_der,
              Inline = false,
              LateInline = true);
          end rho_props_pT;

          function rho_pT "Density as function or pressure and temperature"
            extends Modelica.Icons.Function;
            input Modelica.SIunits.Pressure p "Pressure";
            input Modelica.SIunits.Temperature T "Temperature";
            output Modelica.SIunits.Density rho "Density";
          algorithm
            rho := rho_props_pT(p, T, Air_Utilities.airBaseProp_pT(p, T));
          end rho_pT;

          function rho_pT_der "Derivative function of rho_pT"
            extends Modelica.Icons.Function;
            input Modelica.SIunits.Pressure p "Pressure";
            input Modelica.SIunits.Temperature T "Temperature";
            input Common.AuxiliaryProperties aux "Auxiliary record";
            input Real p_der "Derivative of pressure";
            input Real T_der "Derivative of temperature";
            output Real rho_der "Derivative of density";
          algorithm
            rho_der := 1 / aux.pd * p_der - aux.pt / aux.pd * T_der;
          end rho_pT_der;

          function h_props_pT "Specific enthalpy as function or pressure and temperature"
            extends Modelica.Icons.Function;
            input Modelica.SIunits.Pressure p "Pressure";
            input Modelica.SIunits.Temperature T "Temperature";
            input Common.AuxiliaryProperties aux "Auxiliary record";
            output Modelica.SIunits.SpecificEnthalpy h "Specific enthalpy";
          algorithm
            h := aux.h;
            annotation(
              derivative(noDerivative = aux) = h_pT_der,
              Inline = false,
              LateInline = true);
          end h_props_pT;

          function h_pT "Specific enthalpy as function or pressure and temperature"
            extends Modelica.Icons.Function;
            input Modelica.SIunits.Pressure p "Pressure";
            input Modelica.SIunits.Temperature T "Temperature";
            output Modelica.SIunits.SpecificEnthalpy h "Specific enthalpy";
          algorithm
            h := h_props_pT(p, T, Air_Utilities.airBaseProp_pT(p, T));
          end h_pT;

          function h_pT_der "Derivative function of h_pT"
            extends Modelica.Icons.Function;
            input Modelica.SIunits.Pressure p "Pressure";
            input Modelica.SIunits.Temperature T "Temperature";
            input Common.AuxiliaryProperties aux "Auxiliary record";
            input Real p_der "Derivative of pressure";
            input Real T_der "Derivative of temperature";
            output Real h_der "Derivative of specific enthalpy";
          algorithm
            h_der := ((-aux.rho * aux.pd) + T * aux.pt) / (aux.rho * aux.rho * aux.pd) * p_der + (aux.rho * aux.rho * aux.pd * aux.cv + aux.T * aux.pt * aux.pt) / (aux.rho * aux.rho * aux.pd) * T_der;
          end h_pT_der;

          function s_props_pT "Specific entropy as function of pressure and temperature"
            extends Modelica.Icons.Function;
            input Modelica.SIunits.Pressure p "Pressure";
            input Modelica.SIunits.Temperature T "Temperature";
            input Common.AuxiliaryProperties aux "Auxiliary record";
            output Modelica.SIunits.SpecificEntropy s "Specific entropy";
          algorithm
            s := aux.s;
            annotation(
              Inline = false,
              LateInline = true);
          end s_props_pT;

          function s_pT "Temperature as function of pressure and temperature"
            extends Modelica.Icons.Function;
            input Modelica.SIunits.Pressure p "Pressure";
            input Modelica.SIunits.Temperature T "Temperature";
            output Modelica.SIunits.SpecificEntropy s "Specific entropy";
          algorithm
            s := s_props_pT(p, T, Air_Utilities.airBaseProp_pT(p, T));
          end s_pT;

          function cv_props_pT "Specific heat capacity at constant volume as function of pressure and temperature"
            extends Modelica.Icons.Function;
            input Modelica.SIunits.Pressure p "Pressure";
            input Modelica.SIunits.Temperature T "Temperature";
            input Common.AuxiliaryProperties aux "Auxiliary record";
            output Modelica.SIunits.SpecificHeatCapacity cv "Specific heat capacity";
          algorithm
            cv := aux.cv;
            annotation(
              Inline = false,
              LateInline = true);
          end cv_props_pT;

          function cv_pT "Specific heat capacity at constant volume as function of pressure and temperature"
            extends Modelica.Icons.Function;
            input Modelica.SIunits.Pressure p "Pressure";
            input Modelica.SIunits.Temperature T "Temperature";
            output Modelica.SIunits.SpecificHeatCapacity cv "Specific heat capacity";
          algorithm
            cv := cv_props_pT(p, T, Air_Utilities.airBaseProp_pT(p, T));
          end cv_pT;

          function cp_props_pT "Specific heat capacity at constant pressure as function of pressure and temperature"
            extends Modelica.Icons.Function;
            input Modelica.SIunits.Pressure p "Pressure";
            input Modelica.SIunits.Temperature T "Temperature";
            input Common.AuxiliaryProperties aux "Auxiliary record";
            output Modelica.SIunits.SpecificHeatCapacity cp "Specific heat capacity";
          algorithm
            cp := aux.cp;
            annotation(
              Inline = false,
              LateInline = true);
          end cp_props_pT;

          function cp_pT "Specific heat capacity at constant pressure as function of pressure and temperature"
            extends Modelica.Icons.Function;
            input Modelica.SIunits.Pressure p "Pressure";
            input Modelica.SIunits.Temperature T "Temperature";
            output Modelica.SIunits.SpecificHeatCapacity cp "Specific heat capacity";
          algorithm
            cp := cp_props_pT(p, T, Air_Utilities.airBaseProp_pT(p, T));
          end cp_pT;

          function beta_props_pT "Isobaric expansion coefficient as function of pressure and temperature"
            extends Modelica.Icons.Function;
            input Modelica.SIunits.Pressure p "Pressure";
            input Modelica.SIunits.Temperature T "Temperature";
            input Common.AuxiliaryProperties aux "Auxiliary record";
            output Modelica.SIunits.RelativePressureCoefficient beta "Isobaric expansion coefficient";
          algorithm
            beta := aux.pt / (aux.rho * aux.pd);
            annotation(
              Inline = false,
              LateInline = true);
          end beta_props_pT;

          function beta_pT "Isobaric expansion coefficient as function of pressure and temperature"
            extends Modelica.Icons.Function;
            input Modelica.SIunits.Pressure p "Pressure";
            input Modelica.SIunits.Temperature T "Temperature";
            output Modelica.SIunits.RelativePressureCoefficient beta "Isobaric expansion coefficient";
          algorithm
            beta := beta_props_pT(p, T, Air_Utilities.airBaseProp_pT(p, T));
          end beta_pT;

          function kappa_props_pT "Isothermal compressibility factor as function of pressure and temperature"
            extends Modelica.Icons.Function;
            input Modelica.SIunits.Pressure p "Pressure";
            input Modelica.SIunits.Temperature T "Temperature";
            input Common.AuxiliaryProperties aux "Auxiliary record";
            output Modelica.SIunits.IsothermalCompressibility kappa "Isothermal compressibility factor";
          algorithm
            kappa := 1 / (aux.rho * aux.pd);
            annotation(
              Inline = false,
              LateInline = true);
          end kappa_props_pT;

          function kappa_pT "Isothermal compressibility factor as function of pressure and temperature"
            extends Modelica.Icons.Function;
            input Modelica.SIunits.Pressure p "Pressure";
            input Modelica.SIunits.Temperature T "Temperature";
            output Modelica.SIunits.IsothermalCompressibility kappa "Isothermal compressibility factor";
          algorithm
            kappa := kappa_props_pT(p, T, Air_Utilities.airBaseProp_pT(p, T));
          end kappa_pT;

          function velocityOfSound_props_pT "Speed of sound as function of pressure and temperature"
            extends Modelica.Icons.Function;
            input Modelica.SIunits.Pressure p "Pressure";
            input Modelica.SIunits.Temperature T "Temperature";
            input Common.AuxiliaryProperties aux "Auxiliary record";
            output Modelica.SIunits.Velocity a "Speed of sound";
          algorithm
            a := sqrt(max(0, (aux.pd * aux.rho * aux.rho * aux.cv + aux.pt * aux.pt * aux.T) / (aux.rho * aux.rho * aux.cv)));
            annotation(
              Inline = false,
              LateInline = true);
          end velocityOfSound_props_pT;

          function velocityOfSound_pT "Speed of sound as function of pressure and temperature"
            extends Modelica.Icons.Function;
            input Modelica.SIunits.Pressure p "Pressure";
            input Modelica.SIunits.Temperature T "Temperature";
            output Modelica.SIunits.Velocity a "Speed of sound";
          algorithm
            a := velocityOfSound_props_pT(p, T, Air_Utilities.airBaseProp_pT(p, T));
          end velocityOfSound_pT;

          function isentropicExponent_props_pT "Isentropic exponent as function of pressure and temperature"
            extends Modelica.Icons.Function;
            input Modelica.SIunits.Pressure p "Pressure";
            input Modelica.SIunits.Temperature T "Temperature";
            input Common.AuxiliaryProperties aux "Auxiliary record";
            output Real gamma "Isentropic exponent";
          algorithm
            gamma := 1 / (aux.rho * p) * ((aux.pd * aux.cv * aux.rho * aux.rho + aux.pt * aux.pt * aux.T) / aux.cv);
            annotation(
              Inline = false,
              LateInline = true);
          end isentropicExponent_props_pT;

          function isentropicExponent_pT "Isentropic exponent as function of pressure and temperature"
            extends Modelica.Icons.Function;
            input Modelica.SIunits.Pressure p "Pressure";
            input Modelica.SIunits.Temperature T "Temperature";
            output Real gamma "Isentropic exponent";
          algorithm
            gamma := isentropicExponent_props_pT(p, T, Air_Utilities.airBaseProp_pT(p, T));
          end isentropicExponent_pT;

          function airBaseProp_dT "Intermediate property record for air (d and T prefered states)"
            extends Modelica.Icons.Function;
            input Modelica.SIunits.Density d "Density";
            input Modelica.SIunits.Temperature T "Temperature";
            output Common.AuxiliaryProperties aux "Auxiliary record";
          protected
            Modelica.Media.Common.HelmholtzDerivs f "Dimensionless Helmholtz funcion and dervatives w.r.t. delta and tau";
          algorithm
            aux.rho := d;
            aux.T := T;
            aux.R := ReferenceAir.Air_Utilities.Basic.Constants.R;
            f := Basic.Helmholtz(d, T);
            aux.p := aux.R * d * T * f.delta * f.fdelta;
            aux.h := aux.R * T * (f.tau * f.ftau + f.delta * f.fdelta) - ReferenceAir.Air_Utilities.Basic.Constants.h_off;
            aux.s := aux.R * (f.tau * f.ftau - f.f) - ReferenceAir.Air_Utilities.Basic.Constants.s_off;
            aux.pd := aux.R * T * f.delta * (2 * f.fdelta + f.delta * f.fdeltadelta);
            aux.pt := aux.R * d * f.delta * (f.fdelta - f.tau * f.fdeltatau);
            aux.cv := aux.R * (-f.tau * f.tau * f.ftautau);
            aux.cp := aux.cv + aux.T * aux.pt * aux.pt / (d * d * aux.pd);
            aux.vp := -1 / (aux.rho * aux.rho) * 1 / aux.pd;
            aux.vt := aux.pt / (aux.rho * aux.rho * aux.pd);
          end airBaseProp_dT;

          function h_props_dT "Specific enthalpy as function of density and temperature"
            extends Modelica.Icons.Function;
            input Modelica.SIunits.Density d "Density";
            input Modelica.SIunits.Temperature T "Temperature";
            input Common.AuxiliaryProperties aux "Auxiliary record";
            output Modelica.SIunits.SpecificEnthalpy h "Specific enthalpy";
          algorithm
            h := aux.h;
            annotation(
              derivative(noDerivative = aux) = h_dT_der,
              Inline = false,
              LateInline = true);
          end h_props_dT;

          function h_dT "Specific enthalpy as function of density and temperature"
            extends Modelica.Icons.Function;
            input Modelica.SIunits.Density d "Density";
            input Modelica.SIunits.Temperature T "Temperature";
            output Modelica.SIunits.SpecificEnthalpy h "Specific enthalpy";
          algorithm
            h := h_props_dT(d, T, Air_Utilities.airBaseProp_dT(d, T));
          end h_dT;

          function h_dT_der "Derivative function of h_dT"
            extends Modelica.Icons.Function;
            input Modelica.SIunits.Density d "Density";
            input Modelica.SIunits.Temperature T "Temperature";
            input Common.AuxiliaryProperties aux "Auxiliary record";
            input Real d_der "Derivative of density";
            input Real T_der "Derivative of temperature";
            output Real h_der "Derivative of specific enthalpy";
          algorithm
            h_der := ((-d * aux.pd) + T * aux.pt) / (d * d) * d_der + (aux.cv * d + aux.pt) / d * T_der;
          end h_dT_der;

          function p_props_dT "Pressure as function of density and temperature"
            extends Modelica.Icons.Function;
            input Modelica.SIunits.Density d "Density";
            input Modelica.SIunits.Temperature T "Temperature";
            input Common.AuxiliaryProperties aux "Auxiliary record";
            output Modelica.SIunits.Pressure p "Pressure";
          algorithm
            p := aux.p;
            annotation(
              derivative(noDerivative = aux) = p_dT_der,
              Inline = false,
              LateInline = true);
          end p_props_dT;

          function p_dT "Pressure as function of density and temperature"
            extends Modelica.Icons.Function;
            input Modelica.SIunits.Density d "Density";
            input Modelica.SIunits.Temperature T "Temperature";
            output Modelica.SIunits.Pressure p "Pressure";
          algorithm
            p := p_props_dT(d, T, Air_Utilities.airBaseProp_dT(d, T));
          end p_dT;

          function p_dT_der "Derivative function of p_dT"
            extends Modelica.Icons.Function;
            input Modelica.SIunits.Density d "Density";
            input Modelica.SIunits.Temperature T "Temperature";
            input Common.AuxiliaryProperties aux "Auxiliary record";
            input Real d_der "Derivative of density";
            input Real T_der "Derivative of temperature";
            output Real p_der "Derivative of pressure";
          algorithm
            p_der := aux.pd * d_der + aux.pt * T_der;
          end p_dT_der;

          function s_props_dT "Specific entropy as function of density and temperature"
            extends Modelica.Icons.Function;
            input Modelica.SIunits.Density d "Density";
            input Modelica.SIunits.Temperature T "Temperature";
            input Common.AuxiliaryProperties aux "Auxiliary record";
            output Modelica.SIunits.SpecificEntropy s "Specific entropy";
          algorithm
            s := aux.s;
            annotation(
              Inline = false,
              LateInline = true);
          end s_props_dT;

          function s_dT "Temperature as function of density and temperature"
            extends Modelica.Icons.Function;
            input Modelica.SIunits.Density d "Density";
            input Modelica.SIunits.Temperature T "Temperature";
            output Modelica.SIunits.SpecificEntropy s "Specific entropy";
          algorithm
            s := s_props_dT(d, T, Air_Utilities.airBaseProp_dT(d, T));
          end s_dT;

          function cv_props_dT "Specific heat capacity at constant volume as function of density and temperature"
            extends Modelica.Icons.Function;
            input Modelica.SIunits.Density d "Density";
            input Modelica.SIunits.Temperature T "Temperature";
            input Common.AuxiliaryProperties aux "Auxiliary record";
            output Modelica.SIunits.SpecificHeatCapacity cv "Specific heat capacity";
          algorithm
            cv := aux.cv;
            annotation(
              Inline = false,
              LateInline = true);
          end cv_props_dT;

          function cv_dT "Specific heat capacity at constant volume as function of density and temperature"
            extends Modelica.Icons.Function;
            input Modelica.SIunits.Density d "Density";
            input Modelica.SIunits.Temperature T "Temperature";
            output Modelica.SIunits.SpecificHeatCapacity cv "Specific heat capacity";
          algorithm
            cv := cv_props_dT(d, T, Air_Utilities.airBaseProp_dT(d, T));
          end cv_dT;

          function cp_props_dT "Specific heat capacity at constant pressure as function of density and temperature"
            extends Modelica.Icons.Function;
            input Modelica.SIunits.Density d "Density";
            input Modelica.SIunits.Temperature T "Temperature";
            input Common.AuxiliaryProperties aux "Auxiliary record";
            output Modelica.SIunits.SpecificHeatCapacity cp "Specific heat capacity";
          algorithm
            cp := aux.cp;
            annotation(
              Inline = false,
              LateInline = true);
          end cp_props_dT;

          function cp_dT "Specific heat capacity at constant pressure as function of density and temperature"
            extends Modelica.Icons.Function;
            input Modelica.SIunits.Density d "Density";
            input Modelica.SIunits.Temperature T "Temperature";
            output Modelica.SIunits.SpecificHeatCapacity cp "Specific heat capacity";
          algorithm
            cp := cp_props_dT(d, T, Air_Utilities.airBaseProp_dT(d, T));
          end cp_dT;

          function beta_props_dT "Isobaric expansion coefficient as function of density and temperature"
            extends Modelica.Icons.Function;
            input Modelica.SIunits.Density d "Density";
            input Modelica.SIunits.Temperature T "Temperature";
            input Common.AuxiliaryProperties aux "Auxiliary record";
            output Modelica.SIunits.RelativePressureCoefficient beta "Isobaric expansion coefficient";
          algorithm
            beta := aux.pt / (aux.rho * aux.pd);
            annotation(
              Inline = false,
              LateInline = true);
          end beta_props_dT;

          function beta_dT "Isobaric expansion coefficient as function of density and temperature"
            extends Modelica.Icons.Function;
            input Modelica.SIunits.Density d "Density";
            input Modelica.SIunits.Temperature T "Temperature";
            output Modelica.SIunits.RelativePressureCoefficient beta "Isobaric expansion coefficient";
          algorithm
            beta := beta_props_dT(d, T, Air_Utilities.airBaseProp_dT(d, T));
          end beta_dT;

          function kappa_props_dT "Isothermal compressibility factor as function of density and temperature"
            extends Modelica.Icons.Function;
            input Modelica.SIunits.Density d "Density";
            input Modelica.SIunits.Temperature T "Temperature";
            input Common.AuxiliaryProperties aux "Auxiliary record";
            output Modelica.SIunits.IsothermalCompressibility kappa "Isothermal compressibility factor";
          algorithm
            kappa := 1 / (aux.rho * aux.pd);
            annotation(
              Inline = false,
              LateInline = true);
          end kappa_props_dT;

          function kappa_dT "Isothermal compressibility factor as function of density and temperature"
            extends Modelica.Icons.Function;
            input Modelica.SIunits.Density d "Density";
            input Modelica.SIunits.Temperature T "Temperature";
            output Modelica.SIunits.IsothermalCompressibility kappa "Isothermal compressibility factor";
          algorithm
            kappa := kappa_props_dT(d, T, Air_Utilities.airBaseProp_dT(d, T));
          end kappa_dT;

          function velocityOfSound_props_dT "Speed of sound as function of density and temperature"
            extends Modelica.Icons.Function;
            input Modelica.SIunits.Density d "Density";
            input Modelica.SIunits.Temperature T "Temperature";
            input Common.AuxiliaryProperties aux "Auxiliary record";
            output Modelica.SIunits.Velocity a "Speed of sound";
          algorithm
            a := sqrt(max(0, (aux.pd * aux.rho * aux.rho * aux.cv + aux.pt * aux.pt * aux.T) / (aux.rho * aux.rho * aux.cv)));
            annotation(
              Inline = false,
              LateInline = true);
          end velocityOfSound_props_dT;

          function velocityOfSound_dT "Speed of sound as function of density and temperature"
            extends Modelica.Icons.Function;
            input Modelica.SIunits.Density d "Density";
            input Modelica.SIunits.Temperature T "Temperature";
            output Modelica.SIunits.Velocity a "Speed of sound";
          algorithm
            a := velocityOfSound_props_dT(d, T, Air_Utilities.airBaseProp_dT(d, T));
          end velocityOfSound_dT;

          function isentropicExponent_props_dT "Isentropic exponent as function of density and temperature"
            extends Modelica.Icons.Function;
            input Modelica.SIunits.Density d "Density";
            input Modelica.SIunits.Temperature T "Temperature";
            input Common.AuxiliaryProperties aux "Auxiliary record";
            output Real gamma "Isentropic exponent";
          algorithm
            gamma := 1 / (aux.rho * aux.p) * ((aux.pd * aux.cv * aux.rho * aux.rho + aux.pt * aux.pt * aux.T) / aux.cv);
            annotation(
              Inline = false,
              LateInline = true);
          end isentropicExponent_props_dT;

          function isentropicExponent_dT "Isentropic exponent as function of density and temperature"
            extends Modelica.Icons.Function;
            input Modelica.SIunits.Density d "Density";
            input Modelica.SIunits.Temperature T "Temperature";
            output Real gamma "Isentropic exponent";
          algorithm
            gamma := isentropicExponent_props_dT(d, T, Air_Utilities.airBaseProp_dT(d, T));
          end isentropicExponent_dT;
        protected
          package ThermoFluidSpecial
            function air_ph "Calculate the property record for dynamic simulation properties using p,h as states"
              extends Modelica.Icons.Function;
              input Modelica.SIunits.Pressure p "Pressure";
              input Modelica.SIunits.SpecificEnthalpy h "Specific enthalpy";
              output Modelica.Media.Common.ThermoFluidSpecial.ThermoProperties_ph pro "Property record for dynamic simulation";
            protected
              Modelica.Media.Common.HelmholtzDerivs f "Dimensionless Helmholtz funcion and dervatives w.r.t. delta and tau";
              Modelica.SIunits.Temperature T "Temperature";
              Modelica.SIunits.Density d "Density";
            algorithm
              (d, T) := Air_Utilities.Inverses.dTofph(p = p, h = h, delp = 1.0e-7, delh = 1.0e-6);
              f := Air_Utilities.Basic.Helmholtz(d, T);
              pro := Modelica.Media.Common.ThermoFluidSpecial.helmholtzToProps_ph(f);
            end air_ph;

            function air_dT "Calculate property record for dynamic simulation properties using d and T as dynamic states"
              extends Modelica.Icons.Function;
              input Modelica.SIunits.Density d "Density";
              input Modelica.SIunits.Temperature T "Temperature";
              output Modelica.Media.Common.ThermoFluidSpecial.ThermoProperties_dT pro "Property record for dynamic simulation";
            protected
              Modelica.SIunits.Pressure p "Pressure";
              Modelica.Media.Common.HelmholtzDerivs f "Dimensionless Helmholtz funcion and dervatives w.r.t. delta and tau";
            algorithm
              f := Air_Utilities.Basic.Helmholtz(d, T);
              pro := Modelica.Media.Common.ThermoFluidSpecial.helmholtzToProps_dT(f);
            end air_dT;

            function air_pT "Calculate property record for dynamic simulation properties using p and T as dynamic states"
              extends Modelica.Icons.Function;
              input Modelica.SIunits.Pressure p "Pressure";
              input Modelica.SIunits.Temperature T "Temperature";
              output Modelica.Media.Common.ThermoFluidSpecial.ThermoProperties_pT pro "Property record for dynamic simulation";
            protected
              Modelica.SIunits.Density d "Density";
              Modelica.Media.Common.HelmholtzDerivs f "Dimensionless Helmholtz funcion and dervatives w.r.t. delta and tau";
            algorithm
              d := Modelica.Media.Air.ReferenceAir.Air_Utilities.Inverses.dofpT(p = p, T = T, delp = 1e-7);
              f := Air_Utilities.Basic.Helmholtz(d, T);
              pro := Modelica.Media.Common.ThermoFluidSpecial.helmholtzToProps_pT(f);
            end air_pT;
          end ThermoFluidSpecial;
        end Air_Utilities;
        annotation(
          Documentation(info = "<html>
<p>
Calculation of fluid properties of air in the fluid region of 130 Kelvin to 2000 Kelvin at pressures up to 2000 MPa. To use this package in your model, select <a href=\"modelica://Modelica.Media.Air.ReferenceAir.Air_dT\">
Air_dT</a>, <a href=\"modelica://Modelica.Media.Air.ReferenceAir.Air_pT\">
Air_pT</a> or <a href=\"modelica://Modelica.Media.Air.ReferenceAir.Air_ph\">Air_ph</a> according to which variables you choose to determine your state.
</p>

<h4>Restriction</h4>
<p>
The functions provided by this package shall be used inside of the restricted limits according to the referenced literature.
</p>

<ul>
<li>
<b>p &le; 2000 MPa</b>
</li>
<li>
<b>130 K &le; T &le; 2000 K</b>
</li>
</ul>

<h4>References</h4>
<dl>
<dt>Lemmon, E. W., Jacobsen, R. T., Penoncello, S. G., Friend, D. G.:</dt>
<dd><b>Thermodynamic Properties of Air and Mixtures of Nitrogen, Argon,
and Oxygen From 60 to 2000 K at Pressures to 2000 MPa</b>. J. Phys. Chem. Ref. Data, Vol. 29, No. 3, 2000.
</dd>
<dt>Lemmon, E. W., Jacobsen, R. T.:</dt>
<dd><b>Viscosity and Thermal Conductivity Equations for
Nitrogen, Oxygen, Argon, and Air</b>. International Journal of Thermophysics, Vol. 25, No. 1, January 2004
</dd>
</dl>


<h4>Verification</h4>
<p>
The verification report for the development of this library is provided
<a href=\"modelica://Modelica/Resources/Documentation/Media/MoMoLib_VerificationResults_XRG.pdf\">here</a>.
</p>

<h4>Acknowledgment</h4>
<p>
This library was developed by XRG Simulation GmbH as part of the <a href=\"http://www.cleansky.eu/\">Clean Sky</a> JTI project (Project title: MoMoLib-Modelica Model Library Development for Media, Magnetic Systems and Wavelets; Project number: 296369; Theme: JTI-CS-2011-1-SGO-02-026: Modelica Model Library Development Part I). The partial financial support for the development of this library by the European Union is highly appreciated.
</p>

<p>
Some parts of this library refer to the ThermoFluid library developed at Lund University (<a href=\"http://thermofluid.sourceforge.net/\">http://thermofluid.sourceforge.net</a>).
</p>

<h4>Disclaimer</h4>
<p>
In no event will XRG Simulation GmbH be liable for any direct, indirect, incidental, special, exemplary, or consequential damages, arising in any way out of the use of this software, even if advised of the possibility of such damage.
</p>
<h4> Copyright (C) 2013, XRG Simulation GmbH </h4>

</html>"));
      end ReferenceAir;

      package MoistAir "Air: Moist air model (190 ... 647 K)"
        extends Interfaces.PartialCondensingGases(mediumName = "Moist air", substanceNames = {"water", "air"}, final reducedX = true, final singleState = false, reference_X = {0.01, 0.99}, fluidConstants = {IdealGases.Common.FluidData.H2O, IdealGases.Common.FluidData.N2}, Temperature(min = 190, max = 647));
        import Modelica.Media.IdealGases.Common.Functions;
        constant Integer Water = 1 "Index of water (in substanceNames, massFractions X, etc.)";
        constant Integer Air = 2 "Index of air (in substanceNames, massFractions X, etc.)";
        //     constant SI.Pressure psat_low=saturationPressureWithoutLimits(200.0);
        //     constant SI.Pressure psat_high=saturationPressureWithoutLimits(422.16);
        constant Real k_mair = steam.MM / dryair.MM "Ratio of molar weights";
        constant IdealGases.Common.DataRecord dryair = IdealGases.Common.SingleGasesData.Air;
        constant IdealGases.Common.DataRecord steam = IdealGases.Common.SingleGasesData.H2O;
        constant SI.MolarMass[2] MMX = {steam.MM, dryair.MM} "Molar masses of components";
        import Modelica.Media.Interfaces;
        import Modelica.Math;
        import Modelica.Constants;
        import Modelica.Media.IdealGases.Common.SingleGasNasa;
        import Modelica.Media.Interfaces.Choices.ReferenceEnthalpy;

        redeclare record extends ThermodynamicState "ThermodynamicState record for moist air"
        end ThermodynamicState;

        redeclare replaceable model extends BaseProperties(T(stateSelect = if preferredMediumStates then StateSelect.prefer else StateSelect.default), p(stateSelect = if preferredMediumStates then StateSelect.prefer else StateSelect.default), Xi(each stateSelect = if preferredMediumStates then StateSelect.prefer else StateSelect.default), final standardOrderComponents = true) "Moist air base properties record"
            /* p, T, X = X[Water] are used as preferred states, since only then all
                 other quantities can be computed in a recursive sequence.
                 If other variables are selected as states, static state selection
                 is no longer possible and non-linear algebraic equations occur.
                  */
            MassFraction x_water "Mass of total water/mass of dry air";
            Real phi "Relative humidity";

          protected
            MassFraction X_liquid "Mass fraction of liquid or solid water";
            MassFraction X_steam "Mass fraction of steam water";
            MassFraction X_air "Mass fraction of air";
            MassFraction X_sat "Steam water mass fraction of saturation boundary in kg_water/kg_moistair";
            MassFraction x_sat "Steam water mass content of saturation boundary in kg_water/kg_dryair";
            AbsolutePressure p_steam_sat "partial saturation pressure of steam";

          equation
            assert(T >= 190 and T <= 647, "
Temperature T is not in the allowed range
190.0 K <= (T =" + String(T) + " K) <= 647.0 K
required from medium model \"" + mediumName + "\".");
            MM = 1 / (Xi[Water] / MMX[Water] + (1.0 - Xi[Water]) / MMX[Air]);
            p_steam_sat = min(saturationPressure(T), 0.999 * p);
            X_sat = min(p_steam_sat * k_mair / max(100 * Constants.eps, p - p_steam_sat) * (1 - Xi[Water]), 1.0) "Water content at saturation with respect to actual water content";
            X_liquid = max(Xi[Water] - X_sat, 0.0);
            X_steam = Xi[Water] - X_liquid;
            X_air = 1 - Xi[Water];
            h = specificEnthalpy_pTX(p, T, Xi);
            R = dryair.R * (X_air / (1 - X_liquid)) + steam.R * X_steam / (1 - X_liquid);
//
            u = h - R * T;
            d = p / (R * T);
/* Note, u and d are computed under the assumption that the volume of the liquid
         water is negligible with respect to the volume of air and of steam
      */
            state.p = p;
            state.T = T;
            state.X = X;
// these x are per unit mass of DRY air!
            x_sat = k_mair * p_steam_sat / max(100 * Constants.eps, p - p_steam_sat);
            x_water = Xi[Water] / max(X_air, 100 * Constants.eps);
            phi = p / p_steam_sat * Xi[Water] / (Xi[Water] + k_mair * X_air);
          annotation(
            Documentation(info = "<html>
<p>This model computes thermodynamic properties of moist air from three independent (thermodynamic or/and numerical) state variables. Preferred numerical states are temperature T, pressure p and the reduced composition vector Xi, which contains the water mass fraction only. As an EOS the <b>ideal gas law</b> is used and associated restrictions apply. The model can also be used in the <b>fog region</b>, when moisture is present in its liquid state. However, it is assumed that the liquid water volume is negligible compared to that of the gas phase. Computation of thermal properties is based on property data of <a href=\"modelica://Modelica.Media.Air.DryAirNasa\"> dry air</a> and water (source: VDI-W&auml;rmeatlas), respectively. Besides the standard thermodynamic variables <b>absolute and relative humidity</b>, x_water and phi, respectively, are given by the model. Upper case X denotes absolute humidity with respect to mass of moist air while absolute humidity with respect to mass of dry air only is denoted by a lower case x throughout the model. See <a href=\"modelica://Modelica.Media.Air.MoistAir\">package description</a> for further information.</p>
</html>"));
        end BaseProperties;

        redeclare function setState_pTX "Return thermodynamic state as function of pressure p, temperature T and composition X"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input Temperature T "Temperature";
          input MassFraction X[:] = reference_X "Mass fractions";
          output ThermodynamicState state "Thermodynamic state";
        algorithm
          state := if size(X, 1) == nX then ThermodynamicState(p = p, T = T, X = X) else ThermodynamicState(p = p, T = T, X = cat(1, X, {1 - sum(X)}));
          annotation(
            smoothOrder = 2,
            Documentation(info = "<html>
The <a href=\"modelica://Modelica.Media.Air.MoistAir.ThermodynamicState\">thermodynamic state record</a> is computed from pressure p, temperature T and composition X.
</html>"));
        end setState_pTX;

        redeclare function setState_phX "Return thermodynamic state as function of pressure p, specific enthalpy h and composition X"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SpecificEnthalpy h "Specific enthalpy";
          input MassFraction X[:] = reference_X "Mass fractions";
          output ThermodynamicState state "Thermodynamic state";
        algorithm
          state := if size(X, 1) == nX then ThermodynamicState(p = p, T = T_phX(p, h, X), X = X) else ThermodynamicState(p = p, T = T_phX(p, h, X), X = cat(1, X, {1 - sum(X)}));
          annotation(
            smoothOrder = 2,
            Documentation(info = "<html>
The <a href=\"modelica://Modelica.Media.Air.MoistAir.ThermodynamicState\">thermodynamic state record</a> is computed from pressure p, specific enthalpy h and composition X.
</html>"));
        end setState_phX;

        redeclare function setState_dTX "Return thermodynamic state as function of density d, temperature T and composition X"
          extends Modelica.Icons.Function;
          input Density d "Density";
          input Temperature T "Temperature";
          input MassFraction X[:] = reference_X "Mass fractions";
          output ThermodynamicState state "Thermodynamic state";
        algorithm
          state := if size(X, 1) == nX then ThermodynamicState(p = d * ({steam.R, dryair.R} * X) * T, T = T, X = X) else ThermodynamicState(p = d * ({steam.R, dryair.R} * cat(1, X, {1 - sum(X)})) * T, T = T, X = cat(1, X, {1 - sum(X)}));
          annotation(
            smoothOrder = 2,
            Documentation(info = "<html>
The <a href=\"modelica://Modelica.Media.Air.MoistAir.ThermodynamicState\">thermodynamic state record</a> is computed from density d, temperature T and composition X.
</html>"));
        end setState_dTX;

        redeclare function extends setSmoothState "Return thermodynamic state so that it smoothly approximates: if x > 0 then state_a else state_b"
          algorithm
            state := ThermodynamicState(p = Media.Common.smoothStep(x, state_a.p, state_b.p, x_small), T = Media.Common.smoothStep(x, state_a.T, state_b.T, x_small), X = Media.Common.smoothStep(x, state_a.X, state_b.X, x_small));
        end setSmoothState;

        function Xsaturation "Return absolute humidity per unit mass of moist air at saturation as a function of the thermodynamic state record"
          extends Modelica.Icons.Function;
          input ThermodynamicState state "Thermodynamic state record";
          output MassFraction X_sat "Steam mass fraction of sat. boundary";
        algorithm
          X_sat := k_mair / (state.p / min(saturationPressure(state.T), 0.999 * state.p) - 1 + k_mair);
          annotation(
            smoothOrder = 2,
            Documentation(info = "<html>
Absolute humidity per unit mass of moist air at saturation is computed from pressure and temperature in the state record. Note, that unlike X_sat in the BaseProperties model this mass fraction refers to mass of moist air at saturation.
</html>"));
        end Xsaturation;

        function xsaturation "Return absolute humidity per unit mass of dry air at saturation as a function of the thermodynamic state record"
          extends Modelica.Icons.Function;
          input ThermodynamicState state "Thermodynamic state record";
          output MassFraction x_sat "Absolute humidity per unit mass of dry air";
        algorithm
          x_sat := k_mair * saturationPressure(state.T) / max(100 * Constants.eps, state.p - saturationPressure(state.T));
          annotation(
            smoothOrder = 2,
            Documentation(info = "<html>
Absolute humidity per unit mass of dry air at saturation is computed from pressure and temperature in the thermodynamic state record.
</html>"));
        end xsaturation;

        function xsaturation_pT "Return absolute humidity per unit mass of dry air at saturation as a function of pressure p and temperature T"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SI.Temperature T "Temperature";
          output MassFraction x_sat "Absolute humidity per unit mass of dry air";
        algorithm
          x_sat := k_mair * saturationPressure(T) / max(100 * Constants.eps, p - saturationPressure(T));
          annotation(
            smoothOrder = 2,
            Documentation(info = "<html>
Absolute humidity per unit mass of dry air at saturation is computed from pressure and temperature.
</html>"));
        end xsaturation_pT;

        function massFraction_pTphi "Return steam mass fraction as a function of relative humidity phi and temperature T"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input Temperature T "Temperature";
          input Real phi "Relative humidity (0 ... 1.0)";
          output MassFraction X_steam "Absolute humidity, steam mass fraction";
        protected
          constant Real k = 0.621964713077499 "Ratio of molar masses";
          AbsolutePressure psat = saturationPressure(T) "Saturation pressure";
        algorithm
          X_steam := phi * k / (k * phi + p / psat - phi);
          annotation(
            smoothOrder = 2,
            Documentation(info = "<html>
Absolute humidity per unit mass of moist air is computed from temperature, pressure and relative humidity.
</html>"));
        end massFraction_pTphi;

        function relativeHumidity_pTX "Return relative humidity as a function of pressure p, temperature T and composition X"
          extends Modelica.Icons.Function;
          input SI.Pressure p "Pressure";
          input SI.Temperature T "Temperature";
          input SI.MassFraction[:] X "Composition";
          output Real phi "Relative humidity";
        protected
          SI.Pressure p_steam_sat "Saturation pressure";
          SI.MassFraction X_air "Dry air mass fraction";
        algorithm
          p_steam_sat := min(saturationPressure(T), 0.999 * p);
          X_air := 1 - X[Water];
          phi := max(0.0, min(1.0, p / p_steam_sat * X[Water] / (X[Water] + k_mair * X_air)));
          annotation(
            smoothOrder = 2,
            Documentation(info = "<html>
Relative humidity is computed from pressure, temperature and composition with 1.0 as the upper limit at saturation. Water mass fraction is the first entry in the composition vector.
</html>"));
        end relativeHumidity_pTX;

        function relativeHumidity "Return relative humidity as a function of the thermodynamic state record"
          extends Modelica.Icons.Function;
          input ThermodynamicState state "Thermodynamic state";
          output Real phi "Relative humidity";
        algorithm
          phi := relativeHumidity_pTX(state.p, state.T, state.X);
          annotation(
            smoothOrder = 2,
            Documentation(info = "<html>
Relative humidity is computed from the thermodynamic state record with 1.0 as the upper limit at saturation.
</html>"));
        end relativeHumidity;

        /*
            redeclare function setState_psX "Return thermodynamic state as function of p, s and composition X"
              extends Modelica.Icons.Function;
              input AbsolutePressure p "Pressure";
              input SpecificEntropy s "Specific entropy";
              input MassFraction X[:]=reference_X "Mass fractions";
              output ThermodynamicState state;
            algorithm
              state := if size(X,1) == nX then ThermodynamicState(p=p,T=T_psX(s,p,X),X=X)
                else ThermodynamicState(p=p,T=T_psX(p,s,X), X=cat(1,X,{1-sum(X)}));
            end setState_psX;
        */

        redeclare function extends gasConstant "Return ideal gas constant as a function from thermodynamic state, only valid for phi<1"
          algorithm
            R := dryair.R * (1 - state.X[Water]) + steam.R * state.X[Water];
          annotation(
            smoothOrder = 2,
            Documentation(info = "<html>
The ideal gas constant for moist air is computed from <a href=\"modelica://Modelica.Media.Air.MoistAir.ThermodynamicState\">thermodynamic state</a> assuming that all water is in the gas phase.
</html>"));
        end gasConstant;

        function gasConstant_X "Return ideal gas constant as a function from composition X"
          extends Modelica.Icons.Function;
          input SI.MassFraction X[:] "Gas phase composition";
          output SI.SpecificHeatCapacity R "Ideal gas constant";
        algorithm
          R := dryair.R * (1 - X[Water]) + steam.R * X[Water];
          annotation(
            smoothOrder = 2,
            Documentation(info = "<html>
The ideal gas constant for moist air is computed from the gas phase composition. The first entry in composition vector X is the steam mass fraction of the gas phase.
</html>"));
        end gasConstant_X;

        function saturationPressureLiquid "Return saturation pressure of water as a function of temperature T in the range of 273.16 to 647.096 K"
          extends Modelica.Icons.Function;
          input SI.Temperature Tsat "Saturation temperature";
          output SI.AbsolutePressure psat "Saturation pressure";
        protected
          SI.Temperature Tcritical = 647.096 "Critical temperature";
          SI.AbsolutePressure pcritical = 22.064e6 "Critical pressure";
          Real r1 = 1 - Tsat / Tcritical "Common subexpression";
          Real a[:] = {-7.85951783, 1.84408259, -11.7866497, 22.6807411, -15.9618719, 1.80122502} "Coefficients a[:]";
          Real n[:] = {1.0, 1.5, 3.0, 3.5, 4.0, 7.5} "Coefficients n[:]";
        algorithm
          psat := exp((a[1] * r1 ^ n[1] + a[2] * r1 ^ n[2] + a[3] * r1 ^ n[3] + a[4] * r1 ^ n[4] + a[5] * r1 ^ n[5] + a[6] * r1 ^ n[6]) * Tcritical / Tsat) * pcritical;
          annotation(
            derivative = saturationPressureLiquid_der,
            Inline = false,
            smoothOrder = 5,
            Documentation(info = "<html>
<p>Saturation pressure of water above the triple point temperature is computed from temperature. </p>
<p>Source: A Saul, W Wagner: &quot;International equations for the saturation properties of ordinary water substance&quot;, equation 2.1 </p>
</html>"));
        end saturationPressureLiquid;

        function saturationPressureLiquid_der "Derivative function for 'saturationPressureLiquid'"
          extends Modelica.Icons.Function;
          input SI.Temperature Tsat "Saturation temperature";
          input Real dTsat(unit = "K/s") "Saturation temperature derivative";
          output Real psat_der(unit = "Pa/s") "Saturation pressure derivative";
        protected
          SI.Temperature Tcritical = 647.096 "Critical temperature";
          SI.AbsolutePressure pcritical = 22.064e6 "Critical pressure";
          Real r1 = 1 - Tsat / Tcritical "Common subexpression 1";
          Real r1_der = -1 / Tcritical * dTsat "Derivative of common subexpression 1";
          Real a[:] = {-7.85951783, 1.84408259, -11.7866497, 22.6807411, -15.9618719, 1.80122502} "Coefficients a[:]";
          Real n[:] = {1.0, 1.5, 3.0, 3.5, 4.0, 7.5} "Coefficients n[:]";
          Real r2 = a[1] * r1 ^ n[1] + a[2] * r1 ^ n[2] + a[3] * r1 ^ n[3] + a[4] * r1 ^ n[4] + a[5] * r1 ^ n[5] + a[6] * r1 ^ n[6] "Common subexpression 2";
        algorithm
// Approach used here is based on Baehr: "Thermodynamik", 12th edition p.204ff, "Method of Wagner"
//psat := exp(((a[1]*r1^n[1] + a[2]*r1^n[2] + a[3]*r1^n[3] + a[4]*r1^n[4] + a[5]*r1^n[5] + a[6]*r1^n[6])*Tcritical)/Tsat) * pcritical;
          psat_der := exp(r2 * Tcritical / Tsat) * pcritical * ((a[1] * (r1 ^ (n[1] - 1) * n[1] * r1_der) + a[2] * (r1 ^ (n[2] - 1) * n[2] * r1_der) + a[3] * (r1 ^ (n[3] - 1) * n[3] * r1_der) + a[4] * (r1 ^ (n[4] - 1) * n[4] * r1_der) + a[5] * (r1 ^ (n[5] - 1) * n[5] * r1_der) + a[6] * (r1 ^ (n[6] - 1) * n[6] * r1_der)) * Tcritical / Tsat - r2 * Tcritical * dTsat / Tsat ^ 2);
          annotation(
            Inline = false,
            smoothOrder = 5,
            Documentation(info = "<html>
<p>Saturation pressure of water above the triple point temperature is computed from temperature. </p>
<p>Source: A Saul, W Wagner: &quot;International equations for the saturation properties of ordinary water substance&quot;, equation 2.1 </p>
</html>"));
        end saturationPressureLiquid_der;

        function sublimationPressureIce "Return sublimation pressure of water as a function of temperature T between 190 and 273.16 K"
          extends Modelica.Icons.Function;
          input SI.Temperature Tsat "Sublimation temperature";
          output SI.AbsolutePressure psat "Sublimation pressure";
        protected
          SI.Temperature Ttriple = 273.16 "Triple point temperature";
          SI.AbsolutePressure ptriple = 611.657 "Triple point pressure";
          Real r1 = Tsat / Ttriple "Common subexpression";
          Real a[:] = {-13.9281690, 34.7078238} "Coefficients a[:]";
          Real n[:] = {-1.5, -1.25} "Coefficients n[:]";
        algorithm
          psat := exp(a[1] - a[1] * r1 ^ n[1] + a[2] - a[2] * r1 ^ n[2]) * ptriple;
          annotation(
            Inline = false,
            smoothOrder = 5,
            derivative = sublimationPressureIce_der,
            Documentation(info = "<html>
<p>Sublimation pressure of water below the triple point temperature is computed from temperature.</p>
<p>Source: W Wagner, A Saul, A Pruss: &quot;International equations for the pressure along the melting and along the sublimation curve of ordinary water substance&quot;, equation 3.5</p>
</html>"));
        end sublimationPressureIce;

        function sublimationPressureIce_der "Derivative function for 'sublimationPressureIce'"
          extends Modelica.Icons.Function;
          input SI.Temperature Tsat "Sublimation temperature";
          input Real dTsat(unit = "K/s") "Sublimation temperature derivative";
          output Real psat_der(unit = "Pa/s") "Sublimation pressure derivative";
        protected
          SI.Temperature Ttriple = 273.16 "Triple point temperature";
          SI.AbsolutePressure ptriple = 611.657 "Triple point pressure";
          Real r1 = Tsat / Ttriple "Common subexpression 1";
          Real r1_der = dTsat / Ttriple "Derivative of common subexpression 1";
          Real a[:] = {-13.9281690, 34.7078238} "Coefficients a[:]";
          Real n[:] = {-1.5, -1.25} "Coefficients n[:]";
        algorithm
//psat := exp(a[1] - a[1]*r1^n[1] + a[2] - a[2]*r1^n[2]) * ptriple;
          psat_der := exp(a[1] - a[1] * r1 ^ n[1] + a[2] - a[2] * r1 ^ n[2]) * ptriple * ((-a[1] * (r1 ^ (n[1] - 1) * n[1] * r1_der)) - a[2] * (r1 ^ (n[2] - 1) * n[2] * r1_der));
          annotation(
            Inline = false,
            smoothOrder = 5,
            Documentation(info = "<html>
<p>Sublimation pressure of water below the triple point temperature is computed from temperature.</p>
<p>Source: W Wagner, A Saul, A Pruss: &quot;International equations for the pressure along the melting and along the sublimation curve of ordinary water substance&quot;, equation 3.5</p>
</html>"));
        end sublimationPressureIce_der;

        redeclare function extends saturationPressure "Return saturation pressure of water as a function of temperature T between 190 and 647.096 K"
          algorithm
            psat := Utilities.spliceFunction(saturationPressureLiquid(Tsat), sublimationPressureIce(Tsat), Tsat - 273.16, 1.0);
          annotation(
            Inline = false,
            smoothOrder = 5,
            derivative = saturationPressure_der,
            Documentation(info = "<html>
Saturation pressure of water in the liquid and the solid region is computed using correlations. Functions for the
<a href=\"modelica://Modelica.Media.Air.MoistAir.sublimationPressureIce\">solid</a> and the <a href=\"modelica://Modelica.Media.Air.MoistAir.saturationPressureLiquid\"> liquid</a> region, respectively, are combined using the first derivative continuous <a href=\"modelica://Modelica.Media.Air.MoistAir.Utilities.spliceFunction\">spliceFunction</a>. This functions range of validity is from 190 to 647.096 K. For more information on the type of correlation used, see the documentation of the linked functions.
</html>"));
        end saturationPressure;

        function saturationPressure_der "Derivative function for 'saturationPressure'"
          extends Modelica.Icons.Function;
          input Temperature Tsat "Saturation temperature";
          input Real dTsat(unit = "K/s") "Time derivative of saturation temperature";
          output Real psat_der(unit = "Pa/s") "Saturation pressure";
        algorithm
/*psat := Utilities.spliceFunction(saturationPressureLiquid(Tsat),sublimationPressureIce(Tsat),Tsat-273.16,1.0);*/
          psat_der := Utilities.spliceFunction_der(saturationPressureLiquid(Tsat), sublimationPressureIce(Tsat), Tsat - 273.16, 1.0, saturationPressureLiquid_der(Tsat = Tsat, dTsat = dTsat), sublimationPressureIce_der(Tsat = Tsat, dTsat = dTsat), dTsat, 0);
          annotation(
            Inline = false,
            smoothOrder = 5,
            Documentation(info = "<html>
Derivative function of <a href=\"modelica://Modelica.Media.Air.MoistAir.saturationPressure\">saturationPressure</a>
</html>"));
        end saturationPressure_der;

        function saturationTemperature "Return saturation temperature of water as a function of (partial) pressure p"
          extends Modelica.Icons.Function;
          input SI.Pressure p "Pressure";
          input SI.Temperature T_min = 190 "Lower boundary of solution";
          input SI.Temperature T_max = 647 "Upper boundary of solution";
          output SI.Temperature T "Saturation temperature";
        protected
          package Internal
            extends Modelica.Media.Common.OneNonLinearEquation;

            redeclare record extends f_nonlinear_Data
                // Define data to be passed to user function
            end f_nonlinear_Data;

            redeclare function extends f_nonlinear
              algorithm
                y := saturationPressure(x);
// Compute the non-linear equation: y = f(x, Data)
            end f_nonlinear;

            // Dummy definition

            redeclare function extends solve
            end solve;
          end Internal;
        algorithm
          T := Internal.solve(p, T_min, T_max, f_nonlinear_data = Internal.f_nonlinear_Data());
          annotation(
            Documentation(info = "<html>
 Computes saturation temperature from (partial) pressure via numerical inversion of the function <a href=\"modelica://Modelica.Media.Air.MoistAir.saturationPressure\">saturationPressure</a>. Therefore additional inputs are required (or the defaults are used) for upper and lower temperature bounds.
</html>"));
        end saturationTemperature;

        redeclare function extends enthalpyOfVaporization "Return enthalpy of vaporization of water as a function of temperature T, 273.16 to 647.096 K"
          protected
            Real Tcritical = 647.096 "Critical temperature";
            Real dcritical = 322 "Critical density";
            Real pcritical = 22.064e6 "Critical pressure";
            Real n[:] = {1, 1.5, 3, 3.5, 4, 7.5} "Powers in equation (1)";
            Real a[:] = {-7.85951783, 1.84408259, -11.7866497, 22.6807411, -15.9618719, 1.80122502} "Coefficients in equation (1) of [1]";
            Real m[:] = {1 / 3, 2 / 3, 5 / 3, 16 / 3, 43 / 3, 110 / 3} "Powers in equation (2)";
            Real b[:] = {1.99274064, 1.09965342, -0.510839303, -1.75493479, -45.5170352, -6.74694450e5} "Coefficients in equation (2) of [1]";
            Real o[:] = {2 / 6, 4 / 6, 8 / 6, 18 / 6, 37 / 6, 71 / 6} "Powers in equation (3)";
            Real c[:] = {-2.03150240, -2.68302940, -5.38626492, -17.2991605, -44.7586581, -63.9201063} "Coefficients in equation (3) of [1]";
            Real tau = 1 - T / Tcritical "Temperature expression";
            Real r1 = a[1] * Tcritical * tau ^ n[1] / T + a[2] * Tcritical * tau ^ n[2] / T + a[3] * Tcritical * tau ^ n[3] / T + a[4] * Tcritical * tau ^ n[4] / T + a[5] * Tcritical * tau ^ n[5] / T + a[6] * Tcritical * tau ^ n[6] / T "Expression 1";
            Real r2 = a[1] * n[1] * tau ^ n[1] + a[2] * n[2] * tau ^ n[2] + a[3] * n[3] * tau ^ n[3] + a[4] * n[4] * tau ^ n[4] + a[5] * n[5] * tau ^ n[5] + a[6] * n[6] * tau ^ n[6] "Expression 2";
            Real dp = dcritical * (1 + b[1] * tau ^ m[1] + b[2] * tau ^ m[2] + b[3] * tau ^ m[3] + b[4] * tau ^ m[4] + b[5] * tau ^ m[5] + b[6] * tau ^ m[6]) "Density of saturated liquid";
            Real dpp = dcritical * exp(c[1] * tau ^ o[1] + c[2] * tau ^ o[2] + c[3] * tau ^ o[3] + c[4] * tau ^ o[4] + c[5] * tau ^ o[5] + c[6] * tau ^ o[6]) "Density of saturated vapor";

          algorithm
            r0 := -(dp - dpp) * exp(r1) * pcritical * (r2 + r1 * tau) / (dp * dpp * tau) "Difference of equations (7) and (6)";
          annotation(
            smoothOrder = 2,
            Documentation(info = "<html>
<p>Enthalpy of vaporization of water is computed from temperature in the region of 273.16 to 647.096 K.</p>
<p>Source: W Wagner, A Pruss: \"International equations for the saturation properties of ordinary water substance. Revised according to the international temperature scale of 1990\" (1993).</p>
</html>"));
        end enthalpyOfVaporization;

        function HeatCapacityOfWater "Return specific heat capacity of water (liquid only) as a function of temperature T"
          extends Modelica.Icons.Function;
          input Temperature T "Temperature";
          output SpecificHeatCapacity cp_fl "Specific heat capacity of liquid";
        algorithm
          cp_fl := 1e3 * (4.2166 - (T - 273.15) * (0.0033166 + (T - 273.15) * (0.00010295 - (T - 273.15) * (1.3819e-6 + (T - 273.15) * 7.3221e-9))));
          annotation(
            Documentation(info = "<html>
The specific heat capacity of water (liquid and solid) is calculated using a
                 polynomial approach and data from VDI-Waermeatlas 8. Edition (Db1)
</html>"),
            smoothOrder = 2);
        end HeatCapacityOfWater;

        redeclare function extends enthalpyOfLiquid "Return enthalpy of liquid water as a function of temperature T(use enthalpyOfWater instead)"
          algorithm
            h := (T - 273.15) * 1e3 * (4.2166 - 0.5 * (T - 273.15) * (0.0033166 + 0.333333 * (T - 273.15) * (0.00010295 - 0.25 * (T - 273.15) * (1.3819e-6 + 0.2 * (T - 273.15) * 7.3221e-9))));
          annotation(
            Inline = false,
            smoothOrder = 5,
            Documentation(info = "<html>
Specific enthalpy of liquid water is computed from temperature using a polynomial approach. Kept for compatibility reasons, better use <a href=\"modelica://Modelica.Media.Air.MoistAir.enthalpyOfWater\">enthalpyOfWater</a> instead.
</html>"));
        end enthalpyOfLiquid;

        redeclare function extends enthalpyOfGas "Return specific enthalpy of gas (air and steam) as a function of temperature T and composition X"
          algorithm
            h := Modelica.Media.IdealGases.Common.Functions.h_Tlow(data = steam, T = T, refChoice = ReferenceEnthalpy.UserDefined, h_off = 46479.819 + 2501014.5) * X[Water] + Modelica.Media.IdealGases.Common.Functions.h_Tlow(data = dryair, T = T, refChoice = ReferenceEnthalpy.UserDefined, h_off = 25104.684) * (1.0 - X[Water]);
          annotation(
            Inline = false,
            smoothOrder = 5,
            Documentation(info = "<html>
Specific enthalpy of moist air is computed from temperature, provided all water is in the gaseous state. The first entry in the composition vector X must be the mass fraction of steam. For a function that also covers the fog region please refer to <a href=\"modelica://Modelica.Media.Air.MoistAir.h_pTX\">h_pTX</a>.
</html>"));
        end enthalpyOfGas;

        redeclare function extends enthalpyOfCondensingGas "Return specific enthalpy of steam as a function of temperature T"
          algorithm
            h := Modelica.Media.IdealGases.Common.Functions.h_Tlow(data = steam, T = T, refChoice = ReferenceEnthalpy.UserDefined, h_off = 46479.819 + 2501014.5);
          annotation(
            Inline = false,
            smoothOrder = 5,
            Documentation(info = "<html>
Specific enthalpy of steam is computed from temperature.
</html>"));
        end enthalpyOfCondensingGas;

        redeclare function extends enthalpyOfNonCondensingGas "Return specific enthalpy of dry air as a function of temperature T"
          algorithm
            h := Modelica.Media.IdealGases.Common.Functions.h_Tlow(data = dryair, T = T, refChoice = ReferenceEnthalpy.UserDefined, h_off = 25104.684);
          annotation(
            Inline = false,
            smoothOrder = 1,
            Documentation(info = "<html>
Specific enthalpy of dry air is computed from temperature.
</html>"));
        end enthalpyOfNonCondensingGas;

        function enthalpyOfWater "Computes specific enthalpy of water (solid/liquid) near atmospheric pressure from temperature T"
          extends Modelica.Icons.Function;
          input SIunits.Temperature T "Temperature";
          output SIunits.SpecificEnthalpy h "Specific enthalpy of water";
        algorithm
/*simple model assuming constant properties:
  heat capacity of liquid water:4200 J/kg
  heat capacity of solid water: 2050 J/kg
  enthalpy of fusion (liquid=>solid): 333000 J/kg*/
          h := Utilities.spliceFunction(4200 * (T - 273.15), 2050 * (T - 273.15) - 333000, T - 273.16, 0.1);
          annotation(
            derivative = enthalpyOfWater_der,
            Documentation(info = "<html>
Specific enthalpy of water (liquid and solid) is computed from temperature using constant properties as follows:<br>
<ul>
<li>  heat capacity of liquid water:4200 J/kg
<li>  heat capacity of solid water: 2050 J/kg
<li>  enthalpy of fusion (liquid=>solid): 333000 J/kg
</ul>
Pressure is assumed to be around 1 bar. This function is usually used to determine the specific enthalpy of the liquid or solid fraction of moist air.
</html>"));
        end enthalpyOfWater;

        function enthalpyOfWater_der "Derivative function of enthalpyOfWater"
          extends Modelica.Icons.Function;
          input SIunits.Temperature T "Temperature";
          input Real dT(unit = "K/s") "Time derivative of temperature";
          output Real dh(unit = "J/(kg.s)") "Time derivative of specific enthalpy";
        algorithm
/*simple model assuming constant properties:
  heat capacity of liquid water:4200 J/kg
  heat capacity of solid water: 2050 J/kg
  enthalpy of fusion (liquid=>solid): 333000 J/kg*/
//h:=Utilities.spliceFunction(4200*(T-273.15),2050*(T-273.15)-333000,T-273.16,0.1);
          dh := Utilities.spliceFunction_der(4200 * (T - 273.15), 2050 * (T - 273.15) - 333000, T - 273.16, 0.1, 4200 * dT, 2050 * dT, dT, 0);
          annotation(
            Documentation(info = "<html>
Derivative function for <a href=\"modelica://Modelica.Media.Air.MoistAir.enthalpyOfWater\">enthalpyOfWater</a>.

</html>"));
        end enthalpyOfWater_der;

        redeclare function extends pressure "Returns pressure of ideal gas as a function of the thermodynamic state record"
          algorithm
            p := state.p;
          annotation(
            smoothOrder = 2,
            Documentation(info = "<html>
Pressure is returned from the thermodynamic state record input as a simple assignment.
</html>"));
        end pressure;

        redeclare function extends temperature "Return temperature of ideal gas as a function of the thermodynamic state record"
          algorithm
            T := state.T;
          annotation(
            smoothOrder = 2,
            Documentation(info = "<html>
Temperature is returned from the thermodynamic state record input as a simple assignment.
</html>"));
        end temperature;

        function T_phX "Return temperature as a function of pressure p, specific enthalpy h and composition X"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SpecificEnthalpy h "Specific enthalpy";
          input MassFraction[:] X "Mass fractions of composition";
          output Temperature T "Temperature";
        protected
          package Internal "Solve h(data,T) for T with given h (use only indirectly via temperature_phX)"
            extends Modelica.Media.Common.OneNonLinearEquation;

            redeclare record extends f_nonlinear_Data "Data to be passed to non-linear function"
                extends Modelica.Media.IdealGases.Common.DataRecord;
            end f_nonlinear_Data;

            redeclare function extends f_nonlinear
              algorithm
                y := h_pTX(p, x, X);
            end f_nonlinear;

            // Dummy definition has to be added for current Dymola

            redeclare function extends solve
            end solve;
          end Internal;
        algorithm
          T := Internal.solve(h, 190, 647, p, X[1:nXi], steam);
          annotation(
            Documentation(info = "<html>
Temperature is computed from pressure, specific enthalpy and composition via numerical inversion of function <a href=\"modelica://Modelica.Media.Air.MoistAir.h_pTX\">h_pTX</a>.
</html>"));
        end T_phX;

        redeclare function extends density "Returns density of ideal gas as a function of the thermodynamic state record"
          algorithm
            d := state.p / (gasConstant(state) * state.T);
          annotation(
            smoothOrder = 2,
            Documentation(info = "<html>
Density is computed from pressure, temperature and composition in the thermodynamic state record applying the ideal gas law.
</html>"));
        end density;

        redeclare function extends specificEnthalpy "Return specific enthalpy of moist air as a function of the thermodynamic state record"
          algorithm
            h := h_pTX(state.p, state.T, state.X);
          annotation(
            smoothOrder = 2,
            Documentation(info = "<html>
Specific enthalpy of moist air is computed from the thermodynamic state record. The fog region is included for both, ice and liquid fog.
</html>"));
        end specificEnthalpy;

        function h_pTX "Return specific enthalpy of moist air as a function of pressure p, temperature T and composition X"
          extends Modelica.Icons.Function;
          input SI.Pressure p "Pressure";
          input SI.Temperature T "Temperature";
          input SI.MassFraction X[:] "Mass fractions of moist air";
          output SI.SpecificEnthalpy h "Specific enthalpy at p, T, X";
        protected
          SI.AbsolutePressure p_steam_sat "partial saturation pressure of steam";
          SI.MassFraction X_sat "Absolute humidity per unit mass of moist air";
          SI.MassFraction X_liquid "Mass fraction of liquid water";
          SI.MassFraction X_steam "Mass fraction of steam water";
          SI.MassFraction X_air "Mass fraction of air";
        algorithm
          p_steam_sat := saturationPressure(T);
//p_steam_sat :=min(saturationPressure(T), 0.999*p);
          X_sat := min(p_steam_sat * k_mair / max(100 * Constants.eps, p - p_steam_sat) * (1 - X[Water]), 1.0);
          X_liquid := max(X[Water] - X_sat, 0.0);
          X_steam := X[Water] - X_liquid;
          X_air := 1 - X[Water];
/* h        := {SingleGasNasa.h_Tlow(data=steam,  T=T, refChoice=ReferenceEnthalpy.UserDefined, h_off=46479.819+2501014.5),
               SingleGasNasa.h_Tlow(data=dryair, T=T, refChoice=ReferenceEnthalpy.UserDefined, h_off=25104.684)}*
    {X_steam, X_air} + enthalpyOfLiquid(T)*X_liquid;*/
          h := {Modelica.Media.IdealGases.Common.Functions.h_Tlow(data = steam, T = T, refChoice = ReferenceEnthalpy.UserDefined, h_off = 46479.819 + 2501014.5), Modelica.Media.IdealGases.Common.Functions.h_Tlow(data = dryair, T = T, refChoice = ReferenceEnthalpy.UserDefined, h_off = 25104.684)} * {X_steam, X_air} + enthalpyOfWater(T) * X_liquid;
          annotation(
            derivative = h_pTX_der,
            Inline = false,
            Documentation(info = "<html>
Specific enthalpy of moist air is computed from pressure, temperature and composition with X[1] as the total water mass fraction. The fog region is included for both, ice and liquid fog.
</html>"));
        end h_pTX;

        function h_pTX_der "Derivative function of h_pTX"
          extends Modelica.Icons.Function;
          input SI.Pressure p "Pressure";
          input SI.Temperature T "Temperature";
          input SI.MassFraction X[:] "Mass fractions of moist air";
          input Real dp(unit = "Pa/s") "Pressure derivative";
          input Real dT(unit = "K/s") "Temperature derivative";
          input Real dX[:](each unit = "1/s") "Composition derivative";
          output Real h_der(unit = "J/(kg.s)") "Time derivative of specific enthalpy";
        protected
          SI.AbsolutePressure p_steam_sat "partial saturation pressure of steam";
          SI.MassFraction X_sat "Absolute humidity per unit mass of moist air";
          SI.MassFraction X_liquid "Mass fraction of liquid water";
          SI.MassFraction X_steam "Mass fraction of steam water";
          SI.MassFraction X_air "Mass fraction of air";
          SI.MassFraction x_sat "Absolute humidity per unit mass of dry air at saturation";
          Real dX_steam(unit = "1/s") "Time derivative of steam mass fraction";
          Real dX_air(unit = "1/s") "Time derivative of dry air mass fraction";
          Real dX_liq(unit = "1/s") "Time derivative of liquid/solid water mass fraction";
          Real dps(unit = "Pa/s") "Time derivative of saturation pressure";
          Real dx_sat(unit = "1/s") "Time derivative of absolute humidity per unit mass of dry air";
        algorithm
          p_steam_sat := saturationPressure(T);
          x_sat := p_steam_sat * k_mair / max(100 * Modelica.Constants.eps, p - p_steam_sat);
          X_sat := min(x_sat * (1 - X[Water]), 1.0);
          X_liquid := Utilities.smoothMax(X[Water] - X_sat, 0.0, 1e-5);
          X_steam := X[Water] - X_liquid;
          X_air := 1 - X[Water];
          dX_air := -dX[Water];
          dps := saturationPressure_der(Tsat = T, dTsat = dT);
          dx_sat := k_mair * (dps * (p - p_steam_sat) - p_steam_sat * (dp - dps)) / (p - p_steam_sat) / (p - p_steam_sat);
          dX_liq := Utilities.smoothMax_der(X[Water] - X_sat, 0.0, 1e-5, (1 + x_sat) * dX[Water] - (1 - X[Water]) * dx_sat, 0, 0);
          dX_steam := dX[Water] - dX_liq;
          h_der := X_steam * Modelica.Media.IdealGases.Common.Functions.h_Tlow_der(data = steam, T = T, refChoice = ReferenceEnthalpy.UserDefined, h_off = 46479.819 + 2501014.5, dT = dT) + dX_steam * Modelica.Media.IdealGases.Common.Functions.h_Tlow(data = steam, T = T, refChoice = ReferenceEnthalpy.UserDefined, h_off = 46479.819 + 2501014.5) + X_air * Modelica.Media.IdealGases.Common.Functions.h_Tlow_der(data = dryair, T = T, refChoice = ReferenceEnthalpy.UserDefined, h_off = 25104.684, dT = dT) + dX_air * Modelica.Media.IdealGases.Common.Functions.h_Tlow(data = dryair, T = T, refChoice = ReferenceEnthalpy.UserDefined, h_off = 25104.684) + X_liquid * enthalpyOfWater_der(T = T, dT = dT) + dX_liq * enthalpyOfWater(T);
          annotation(
            Inline = false,
            smoothOrder = 1,
            Documentation(info = "<html>
Derivative function for <a href=\"modelica://Modelica.Media.Air.MoistAir.h_pTX\">h_pTX</a>.
</html>"));
        end h_pTX_der;

        redeclare function extends isentropicExponent "Return isentropic exponent (only for gas fraction!)"
          algorithm
            gamma := specificHeatCapacityCp(state) / specificHeatCapacityCv(state);
        end isentropicExponent;

        function isentropicEnthalpyApproximation "Approximate calculation of h_is from upstream properties, downstream pressure, gas part only"
          extends Modelica.Icons.Function;
          input AbsolutePressure p2 "Downstream pressure";
          input ThermodynamicState state "Thermodynamic state at upstream location";
          output SpecificEnthalpy h_is "Isentropic enthalpy";
        protected
          SpecificEnthalpy h "Specific enthalpy at upstream location";
          IsentropicExponent gamma = isentropicExponent(state) "Isentropic exponent";
        protected
          MassFraction[nX] X "Complete X-vector";
        algorithm
          X := state.X;
//  X := if reducedX then cat(1,state.X,{1-sum(state.X)}) else state.X;
          h := {Modelica.Media.IdealGases.Common.Functions.h_Tlow(data = steam, T = state.T, refChoice = ReferenceEnthalpy.UserDefined, h_off = 46479.819 + 2501014.5), Modelica.Media.IdealGases.Common.Functions.h_Tlow(data = dryair, T = state.T, refChoice = ReferenceEnthalpy.UserDefined, h_off = 25104.684)} * X;
          h_is := h + gamma / (gamma - 1.0) * (state.T * gasConstant(state)) * ((p2 / state.p) ^ ((gamma - 1) / gamma) - 1.0);
        end isentropicEnthalpyApproximation;

        redeclare function extends specificInternalEnergy "Return specific internal energy of moist air as a function of the thermodynamic state record"
            extends Modelica.Icons.Function;
            output SI.SpecificInternalEnergy u "Specific internal energy";

          algorithm
            u := specificInternalEnergy_pTX(state.p, state.T, state.X);
          annotation(
            smoothOrder = 2,
            Documentation(info = "<html>
Specific internal energy is determined from the thermodynamic state record, assuming that the liquid or solid water volume is negligible.
</html>"));
        end specificInternalEnergy;

        function specificInternalEnergy_pTX "Return specific internal energy of moist air as a function of pressure p, temperature T and composition X"
          extends Modelica.Icons.Function;
          input SI.Pressure p "Pressure";
          input SI.Temperature T "Temperature";
          input SI.MassFraction X[:] "Mass fractions of moist air";
          output SI.SpecificInternalEnergy u "Specific internal energy";
        protected
          SI.AbsolutePressure p_steam_sat "partial saturation pressure of steam";
          SI.MassFraction X_liquid "Mass fraction of liquid water";
          SI.MassFraction X_steam "Mass fraction of steam water";
          SI.MassFraction X_air "Mass fraction of air";
          SI.MassFraction X_sat "Absolute humidity per unit mass of moist air";
          Real R_gas "Ideal gas constant";
        algorithm
          p_steam_sat := saturationPressure(T);
          X_sat := min(p_steam_sat * k_mair / max(100 * Constants.eps, p - p_steam_sat) * (1 - X[Water]), 1.0);
          X_liquid := max(X[Water] - X_sat, 0.0);
          X_steam := X[Water] - X_liquid;
          X_air := 1 - X[Water];
          R_gas := dryair.R * X_air / (1 - X_liquid) + steam.R * X_steam / (1 - X_liquid);
          u := X_steam * Modelica.Media.IdealGases.Common.Functions.h_Tlow(data = steam, T = T, refChoice = ReferenceEnthalpy.UserDefined, h_off = 46479.819 + 2501014.5) + X_air * Modelica.Media.IdealGases.Common.Functions.h_Tlow(data = dryair, T = T, refChoice = ReferenceEnthalpy.UserDefined, h_off = 25104.684) + enthalpyOfWater(T) * X_liquid - R_gas * T;
          annotation(
            derivative = specificInternalEnergy_pTX_der,
            Documentation(info = "<html>
Specific internal energy is determined from pressure p, temperature T and composition X, assuming that the liquid or solid water volume is negligible.
</html>"));
        end specificInternalEnergy_pTX;

        function specificInternalEnergy_pTX_der "Derivative function for specificInternalEnergy_pTX"
          extends Modelica.Icons.Function;
          input SI.Pressure p "Pressure";
          input SI.Temperature T "Temperature";
          input SI.MassFraction X[:] "Mass fractions of moist air";
          input Real dp(unit = "Pa/s") "Pressure derivative";
          input Real dT(unit = "K/s") "Temperature derivative";
          input Real dX[:](each unit = "1/s") "Mass fraction derivatives";
          output Real u_der(unit = "J/(kg.s)") "Specific internal energy derivative";
        protected
          SI.AbsolutePressure p_steam_sat "partial saturation pressure of steam";
          SI.MassFraction X_liquid "Mass fraction of liquid water";
          SI.MassFraction X_steam "Mass fraction of steam water";
          SI.MassFraction X_air "Mass fraction of air";
          SI.MassFraction X_sat "Absolute humidity per unit mass of moist air";
          SI.SpecificHeatCapacity R_gas "Ideal gas constant";
          SI.MassFraction x_sat "Absolute humidity per unit mass of dry air at saturation";
          Real dX_steam(unit = "1/s") "Time derivative of steam mass fraction";
          Real dX_air(unit = "1/s") "Time derivative of dry air mass fraction";
          Real dX_liq(unit = "1/s") "Time derivative of liquid/solid water mass fraction";
          Real dps(unit = "Pa/s") "Time derivative of saturation pressure";
          Real dx_sat(unit = "1/s") "Time derivative of absolute humidity per unit mass of dry air";
          Real dR_gas(unit = "J/(kg.K.s)") "Time derivative of ideal gas constant";
        algorithm
          p_steam_sat := saturationPressure(T);
          x_sat := p_steam_sat * k_mair / max(100 * Modelica.Constants.eps, p - p_steam_sat);
          X_sat := min(x_sat * (1 - X[Water]), 1.0);
          X_liquid := Utilities.spliceFunction(X[Water] - X_sat, 0.0, X[Water] - X_sat, 1e-6);
          X_steam := X[Water] - X_liquid;
          X_air := 1 - X[Water];
          R_gas := steam.R * X_steam / (1 - X_liquid) + dryair.R * X_air / (1 - X_liquid);
          dX_air := -dX[Water];
          dps := saturationPressure_der(Tsat = T, dTsat = dT);
          dx_sat := k_mair * (dps * (p - p_steam_sat) - p_steam_sat * (dp - dps)) / (p - p_steam_sat) / (p - p_steam_sat);
          dX_liq := Utilities.spliceFunction_der(X[Water] - X_sat, 0.0, X[Water] - X_sat, 1e-6, (1 + x_sat) * dX[Water] - (1 - X[Water]) * dx_sat, 0.0, (1 + x_sat) * dX[Water] - (1 - X[Water]) * dx_sat, 0.0);
          dX_steam := dX[Water] - dX_liq;
          dR_gas := (steam.R * (dX_steam * (1 - X_liquid) + dX_liq * X_steam) + dryair.R * (dX_air * (1 - X_liquid) + dX_liq * X_air)) / (1 - X_liquid) / (1 - X_liquid);
          u_der := X_steam * Modelica.Media.IdealGases.Common.Functions.h_Tlow_der(data = steam, T = T, refChoice = ReferenceEnthalpy.UserDefined, h_off = 46479.819 + 2501014.5, dT = dT) + dX_steam * Modelica.Media.IdealGases.Common.Functions.h_Tlow(data = steam, T = T, refChoice = ReferenceEnthalpy.UserDefined, h_off = 46479.819 + 2501014.5) + X_air * Modelica.Media.IdealGases.Common.Functions.h_Tlow_der(data = dryair, T = T, refChoice = ReferenceEnthalpy.UserDefined, h_off = 25104.684, dT = dT) + dX_air * Modelica.Media.IdealGases.Common.Functions.h_Tlow(data = dryair, T = T, refChoice = ReferenceEnthalpy.UserDefined, h_off = 25104.684) + X_liquid * enthalpyOfWater_der(T = T, dT = dT) + dX_liq * enthalpyOfWater(T) - dR_gas * T - R_gas * dT;
          annotation(
            Documentation(info = "<html>
Derivative function for <a href=\"modelica://Modelica.Media.Air.MoistAir.specificInternalEnergy_pTX\">specificInternalEnergy_pTX</a>.
</html>"));
        end specificInternalEnergy_pTX_der;

        redeclare function extends specificEntropy "Return specific entropy from thermodynamic state record, only valid for phi<1"
          algorithm
            s := s_pTX(state.p, state.T, state.X);
          annotation(
            Inline = false,
            smoothOrder = 2,
            Documentation(info = "<html>
Specific entropy is calculated from the thermodynamic state record, assuming ideal gas behavior and including entropy of mixing. Liquid or solid water is not taken into account, the entire water content X[1] is assumed to be in the vapor state (relative humidity below 1.0).
</html>"));
        end specificEntropy;

        redeclare function extends specificGibbsEnergy "Return specific Gibbs energy as a function of the thermodynamic state record, only valid for phi<1"
            extends Modelica.Icons.Function;

          algorithm
            g := h_pTX(state.p, state.T, state.X) - state.T * specificEntropy(state);
          annotation(
            smoothOrder = 2,
            Documentation(info = "<html>
The Gibbs Energy is computed from the thermodynamic state record for moist air with a water content below saturation.
</html>"));
        end specificGibbsEnergy;

        redeclare function extends specificHelmholtzEnergy "Return specific Helmholtz energy as a function of the thermodynamic state record, only valid for phi<1"
            extends Modelica.Icons.Function;

          algorithm
            f := h_pTX(state.p, state.T, state.X) - gasConstant(state) * state.T - state.T * specificEntropy(state);
          annotation(
            smoothOrder = 2,
            Documentation(info = "<html>
The Specific Helmholtz Energy is computed from the thermodynamic state record for moist air with a water content below saturation.
</html>"));
        end specificHelmholtzEnergy;

        redeclare function extends specificHeatCapacityCp "Return specific heat capacity at constant pressure as a function of the thermodynamic state record"
          protected
            Real dT(unit = "s/K") = 1.0;

          algorithm
            cp := h_pTX_der(state.p, state.T, state.X, 0.0, 1.0, zeros(size(state.X, 1))) * dT "Definition of cp: dh/dT @ constant p";
//      cp:= SingleGasNasa.cp_Tlow(dryair, state.T)*(1-state.X[Water])
//        + SingleGasNasa.cp_Tlow(steam, state.T)*state.X[Water];
          annotation(
            Inline = false,
            smoothOrder = 2,
            Documentation(info = "<html>
The specific heat capacity at constant pressure <b>cp</b> is computed from temperature and composition for a mixture of steam (X[1]) and dry air. All water is assumed to be in the vapor state.
</html>"));
        end specificHeatCapacityCp;

        redeclare function extends specificHeatCapacityCv "Return specific heat capacity at constant volume as a function of the thermodynamic state record"
          algorithm
            cv := Modelica.Media.IdealGases.Common.Functions.cp_Tlow(dryair, state.T) * (1 - state.X[Water]) + Modelica.Media.IdealGases.Common.Functions.cp_Tlow(steam, state.T) * state.X[Water] - gasConstant(state);
          annotation(
            Inline = false,
            smoothOrder = 2,
            Documentation(info = "<html>
The specific heat capacity at constant density <b>cv</b> is computed from temperature and composition for a mixture of steam (X[1]) and dry air. All water is assumed to be in the vapor state.
</html>"));
        end specificHeatCapacityCv;

        redeclare function extends dynamicViscosity "Return dynamic viscosity as a function of the thermodynamic state record, valid from 123.15 K to 1273.15 K"
            import Modelica.Media.Incompressible.TableBased.Polynomials_Temp;

          algorithm
            eta := 1e-6 * Polynomials_Temp.evaluateWithRange({9.7391102886305869E-15, -3.1353724870333906E-11, 4.3004876595642225E-08, -3.8228016291758240E-05, 5.0427874367180762E-02, 1.7239260139242528E+01}, Cv.to_degC(123.15), Cv.to_degC(1273.15), Cv.to_degC(state.T));
          annotation(
            smoothOrder = 2,
            Documentation(info = "<html>
<p>Dynamic viscosity is computed from temperature using a simple polynomial for dry air. Range of validity is from 123.15 K to 1273.15 K. The influence of pressure and moisture is neglected. </p>
<p>Source: VDI Waermeatlas, 8th edition. </p>
</html>"));
        end dynamicViscosity;

        redeclare function extends thermalConductivity "Return thermal conductivity as a function of the thermodynamic state record, valid from 123.15 K to 1273.15 K"
            import Modelica.Media.Incompressible.TableBased.Polynomials_Temp;
            import Cv = Modelica.SIunits.Conversions;

          algorithm
            lambda := 1e-3 * Polynomials_Temp.evaluateWithRange({6.5691470817717812E-15, -3.4025961923050509E-11, 5.3279284846303157E-08, -4.5340839289219472E-05, 7.6129675309037664E-02, 2.4169481088097051E+01}, Cv.to_degC(123.15), Cv.to_degC(1273.15), Cv.to_degC(state.T));
          annotation(
            smoothOrder = 2,
            Documentation(info = "<html>
<p>Thermal conductivity is computed from temperature using a simple polynomial for dry air. Range of validity is from 123.15 K to 1273.15 K. The influence of pressure and moisture is neglected. </p>
<p>Source: VDI Waermeatlas, 8th edition. </p>
</html>"));
        end thermalConductivity;

        redeclare function extends velocityOfSound
          algorithm
            a := sqrt(isentropicExponent(state) * gasConstant(state) * temperature(state));
          annotation(
            Documentation(revisions = "<html>
<p>2012-01-12        Stefan Wischhusen: Initial Release.</p>
</html>"));
        end velocityOfSound;

        redeclare function extends isobaricExpansionCoefficient
          algorithm
            beta := 1 / temperature(state);
          annotation(
            Documentation(revisions = "<html>
<p>2012-01-12        Stefan Wischhusen: Initial Release.</p>
</html>"));
        end isobaricExpansionCoefficient;

        redeclare function extends isothermalCompressibility
          algorithm
            kappa := 1 / pressure(state);
          annotation(
            Documentation(revisions = "<html>
<p>2012-01-12        Stefan Wischhusen: Initial Release.</p>
</html>"));
        end isothermalCompressibility;

        redeclare function extends density_derp_h
          algorithm
            ddph := 1 / (gasConstant(state) * temperature(state));
          annotation(
            Documentation(revisions = "<html>
<p>2012-01-12        Stefan Wischhusen: Initial Release.</p>
</html>"));
        end density_derp_h;

        redeclare function extends density_derh_p
          algorithm
            ddhp := -density(state) / (specificHeatCapacityCp(state) * temperature(state));
          annotation(
            Documentation(revisions = "<html>
<p>2012-01-12        Stefan Wischhusen: Initial Release.</p>
</html>"));
        end density_derh_p;

        redeclare function extends density_derp_T
          algorithm
            ddpT := 1 / (gasConstant(state) * temperature(state));
          annotation(
            Documentation(revisions = "<html>
<p>2012-01-12        Stefan Wischhusen: Initial Release.</p>
</html>"));
        end density_derp_T;

        redeclare function extends density_derT_p
          algorithm
            ddTp := -density(state) / temperature(state);
          annotation(
            Documentation(revisions = "<html>
<p>2012-01-12        Stefan Wischhusen: Initial Release.</p>
</html>"));
        end density_derT_p;

        redeclare function extends density_derX
          algorithm
            dddX[Water] := pressure(state) * (steam.R - dryair.R) / ((steam.R - dryair.R) * state.X[Water] * temperature(state) + dryair.R * temperature(state)) ^ 2;
            dddX[Air] := pressure(state) * (dryair.R - steam.R) / ((dryair.R - steam.R) * state.X[Air] * temperature(state) + steam.R * temperature(state)) ^ 2;
          annotation(
            Documentation(revisions = "<html>
<p>2012-01-12        Stefan Wischhusen: Initial Release.</p>
</html>"));
        end density_derX;

        redeclare function extends molarMass
          algorithm
            MM := Modelica.Media.Air.MoistAir.gasConstant(state) / Modelica.Constants.R;
          annotation(
            Documentation(revisions = "<html>
<p>2012-01-12        Stefan Wischhusen: Initial Release.</p>
</html>"));
        end molarMass;

        function T_psX "Return temperature as a function of pressure p, specific entropy s and composition X"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SpecificEntropy s "Specific entropy";
          input MassFraction[:] X "Mass fractions of composition";
          output Temperature T "Temperature";
        protected
          package Internal "Solve s(data,T) for T with given s"
            extends Modelica.Media.Common.OneNonLinearEquation;

            redeclare record extends f_nonlinear_Data "Data to be passed to non-linear function"
                extends Modelica.Media.IdealGases.Common.DataRecord;
            end f_nonlinear_Data;

            redeclare function extends f_nonlinear
              algorithm
                y := s_pTX(p, x, X);
            end f_nonlinear;

            // Dummy definition has to be added for current Dymola

            redeclare function extends solve
            end solve;
          end Internal;
        algorithm
          T := Internal.solve(s, 190, 647, p, X[1:nX], steam);
          annotation(
            Documentation(info = "<html>
Temperature is computed from pressure, specific entropy and composition via numerical inversion of function <a href=\"modelica://Modelica.Media.Air.MoistAir.specificEntropy\">specificEntropy</a>.
</html>", revisions = "<html>
<p>2012-01-12        Stefan Wischhusen: Initial Release.</p>
</html>"));
        end T_psX;

        redeclare function extends setState_psX
          algorithm
            state := if size(X, 1) == nX then ThermodynamicState(p = p, T = T_psX(p, s, X), X = X) else ThermodynamicState(p = p, T = T_psX(p, s, X), X = cat(1, X, {1 - sum(X)}));
          annotation(
            smoothOrder = 2,
            Documentation(info = "<html>
The <a href=\"modelica://Modelica.Media.Air.MoistAir.ThermodynamicState\">thermodynamic state record</a> is computed from pressure p, specific enthalpy h and composition X.
</html>", revisions = "<html>
<p>2012-01-12        Stefan Wischhusen: Initial Release.</p>
</html>"));
        end setState_psX;

        function s_pTX "Return specific entropy of moist air as a function of pressure p, temperature T and composition X (only valid for phi<1)"
          extends Modelica.Icons.Function;
          input SI.Pressure p "Pressure";
          input SI.Temperature T "Temperature";
          input SI.MassFraction X[:] "Mass fractions of moist air";
          output SI.SpecificEntropy s "Specific entropy at p, T, X";
        protected
          MoleFraction[2] Y = massToMoleFractions(X, {steam.MM, dryair.MM}) "Molar fraction";
        algorithm
          s := Modelica.Media.IdealGases.Common.Functions.s0_Tlow(dryair, T) * (1 - X[Water]) + Modelica.Media.IdealGases.Common.Functions.s0_Tlow(steam, T) * X[Water] - Modelica.Constants.R * (Utilities.smoothMax(X[Water] / MMX[Water] * Modelica.Math.log(max(Y[Water], Modelica.Constants.eps) * p / reference_p), 0.0, 1e-9) - Utilities.smoothMax((1 - X[Water]) / MMX[Air] * Modelica.Math.log(max(Y[Air], Modelica.Constants.eps) * p / reference_p), 0.0, 1e-9));
          annotation(
            derivative = s_pTX_der,
            Inline = false,
            Documentation(info = "<html>
Specific entropy of moist air is computed from pressure, temperature and composition with X[1] as the total water mass fraction.
</html>", revisions = "<html>
<p>2012-01-12        Stefan Wischhusen: Initial Release.</p>
</html>"),
            Icon(graphics = {Text(extent = {{-100, 100}, {100, -100}}, lineColor = {255, 127, 0}, textString = "f")}));
        end s_pTX;

        function s_pTX_der "Return specific entropy of moist air as a function of pressure p, temperature T and composition X (only valid for phi<1)"
          extends Modelica.Icons.Function;
          input SI.Pressure p "Pressure";
          input SI.Temperature T "Temperature";
          input SI.MassFraction X[:] "Mass fractions of moist air";
          input Real dp(unit = "Pa/s") "Derivative of pressure";
          input Real dT(unit = "K/s") "Derivative of temperature";
          input Real dX[nX](unit = "1/s") "Derivative of mass fractions";
          output Real ds(unit = "J/(kg.K.s)") "Specific entropy at p, T, X";
        protected
          MoleFraction[2] Y = massToMoleFractions(X, {steam.MM, dryair.MM}) "Molar fraction";
        algorithm
          ds := Modelica.Media.IdealGases.Common.Functions.s0_Tlow_der(dryair, T, dT) * (1 - X[Water]) + Modelica.Media.IdealGases.Common.Functions.s0_Tlow_der(steam, T, dT) * X[Water] + Modelica.Media.IdealGases.Common.Functions.s0_Tlow(dryair, T) * dX[Air] + Modelica.Media.IdealGases.Common.Functions.s0_Tlow(steam, T) * dX[Water] - Modelica.Constants.R * (1 / MMX[Water] * Utilities.smoothMax_der(X[Water] * Modelica.Math.log(max(Y[Water], Modelica.Constants.eps) * p / reference_p), 0.0, 1e-9, (Modelica.Math.log(max(Y[Water], Modelica.Constants.eps) * p / reference_p) + X[Water] / Y[Water] * (X[Air] * MMX[Water] / (X[Air] * MMX[Water] + X[Water] * MMX[Air]) ^ 2)) * dX[Water] + X[Water] * reference_p / p * dp, 0, 0) - 1 / MMX[Air] * Utilities.smoothMax_der((1 - X[Water]) * Modelica.Math.log(max(Y[Air], Modelica.Constants.eps) * p / reference_p), 0.0, 1e-9, (Modelica.Math.log(max(Y[Air], Modelica.Constants.eps) * p / reference_p) + X[Air] / Y[Air] * (X[Water] * MMX[Air] / (X[Air] * MMX[Water] + X[Water] * MMX[Air]) ^ 2)) * dX[Air] + X[Air] * reference_p / p * dp, 0, 0));
          annotation(
            Inline = false,
            smoothOrder = 1,
            Documentation(info = "<html>
Specific entropy of moist air is computed from pressure, temperature and composition with X[1] as the total water mass fraction.
</html>", revisions = "<html>
<p>2012-01-12        Stefan Wischhusen: Initial Release.</p>
</html>"),
            Icon(graphics = {Text(extent = {{-100, 100}, {100, -100}}, lineColor = {255, 127, 0}, textString = "f")}));
        end s_pTX_der;

        redeclare function extends isentropicEnthalpy "Isentropic enthalpy (only valid for phi<1)"
            extends Modelica.Icons.Function;

          algorithm
            h_is := Modelica.Media.Air.MoistAir.h_pTX(p_downstream, Modelica.Media.Air.MoistAir.T_psX(p_downstream, Modelica.Media.Air.MoistAir.specificEntropy(refState), refState.X), refState.X);
          annotation(
            Icon(graphics = {Text(extent = {{-100, 100}, {100, -100}}, lineColor = {255, 127, 0}, textString = "f")}),
            Documentation(revisions = "<html>
<p>2012-01-12        Stefan Wischhusen: Initial Release.</p>
</html>"));
        end isentropicEnthalpy;

        package Utilities "Utility functions"
          extends Modelica.Icons.UtilitiesPackage;

          function spliceFunction "Spline interpolation of two functions"
            extends Modelica.Icons.Function;
            input Real pos "Returned value for x-deltax >= 0";
            input Real neg "Returned value for x+deltax <= 0";
            input Real x "Function argument";
            input Real deltax = 1 "Region around x with spline interpolation";
            output Real out;
          protected
            Real scaledX;
            Real scaledX1;
            Real y;
          algorithm
            scaledX1 := x / deltax;
            scaledX := scaledX1 * Modelica.Math.asin(1);
            if scaledX1 <= (-0.999999999) then
              y := 0;
            elseif scaledX1 >= 0.999999999 then
              y := 1;
            else
              y := (Modelica.Math.tanh(Modelica.Math.tan(scaledX)) + 1) / 2;
            end if;
            out := pos * y + (1 - y) * neg;
            annotation(
              derivative = spliceFunction_der);
          end spliceFunction;

          function spliceFunction_der "Derivative of spliceFunction"
            extends Modelica.Icons.Function;
            input Real pos;
            input Real neg;
            input Real x;
            input Real deltax = 1;
            input Real dpos;
            input Real dneg;
            input Real dx;
            input Real ddeltax = 0;
            output Real out;
          protected
            Real scaledX;
            Real scaledX1;
            Real dscaledX1;
            Real y;
          algorithm
            scaledX1 := x / deltax;
            scaledX := scaledX1 * Modelica.Math.asin(1);
            dscaledX1 := (dx - scaledX1 * ddeltax) / deltax;
            if scaledX1 <= (-0.99999999999) then
              y := 0;
            elseif scaledX1 >= 0.9999999999 then
              y := 1;
            else
              y := (Modelica.Math.tanh(Modelica.Math.tan(scaledX)) + 1) / 2;
            end if;
            out := dpos * y + (1 - y) * dneg;
            if abs(scaledX1) < 1 then
              out := out + (pos - neg) * dscaledX1 * Modelica.Math.asin(1) / 2 / (Modelica.Math.cosh(Modelica.Math.tan(scaledX)) * Modelica.Math.cos(scaledX)) ^ 2;
            end if;
          end spliceFunction_der;

          function smoothMax
            extends Modelica.Icons.Function;
            import Modelica.Math;
            input Real x1 "First argument of smooth max operator";
            input Real x2 "Second argument of smooth max operator";
            input Real dx "Approximate difference between x1 and x2, below which regularization starts";
            output Real y "Result of smooth max operator";
          algorithm
            y := max(x1, x2) + Math.log(exp(4 / dx * (x1 - max(x1, x2))) + exp(4 / dx * (x2 - max(x1, x2)))) / (4 / dx);
            annotation(
              smoothOrder = 2,
              Documentation(info = "<html>
<p>An implementation of Kreisselmeier Steinhauser smooth maximum</p>
</html>"));
          end smoothMax;

          function smoothMax_der
            extends Modelica.Icons.Function;
            import Modelica.Math.exp;
            import Modelica.Math.log;
            input Real x1 "First argument of smooth max operator";
            input Real x2 "Second argument of smooth max operator";
            input Real dx "Approximate difference between x1 and x2, below which regularization starts";
            input Real dx1;
            input Real dx2;
            input Real ddx;
            output Real dy "Derivative of smooth max operator";
          algorithm
            dy := (if x1 > x2 then dx1 else dx2) + 0.25 * (((4 * (dx1 - (if x1 > x2 then dx1 else dx2)) / dx - 4 * (x1 - max(x1, x2)) * ddx / dx ^ 2) * exp(4 * (x1 - max(x1, x2)) / dx) + (4 * (dx2 - (if x1 > x2 then dx1 else dx2)) / dx - 4 * (x2 - max(x1, x2)) * ddx / dx ^ 2) * exp(4 * (x2 - max(x1, x2)) / dx)) * dx / (exp(4 * (x1 - max(x1, x2)) / dx) + exp(4 * (x2 - max(x1, x2)) / dx)) + log(exp(4 * (x1 - max(x1, x2)) / dx) + exp(4 * (x2 - max(x1, x2)) / dx)) * ddx);
            annotation(
              Documentation(info = "<html>
<p>An implementation of Kreisselmeier Steinhauser smooth maximum</p>
</html>"));
          end smoothMax_der;
        end Utilities;
        annotation(
          Documentation(info = "<html>
<h4>Thermodynamic Model</h4>
<p>This package provides a full thermodynamic model of moist air including the fog region and temperatures below zero degC.
The governing assumptions in this model are:</p>
<ul>
<li>the perfect gas law applies</li>
<li>water volume other than that of steam is neglected</li></ul>
<p>All extensive properties are expressed in terms of the total mass in order to comply with other media in this library. However, for moist air it is rather common to express the absolute humidity in terms of mass of dry air only, which has advantages when working with charts. In addition, care must be taken, when working with mass fractions with respect to total mass, that all properties refer to the same water content when being used in mathematical operations (which is always the case if based on dry air only). Therefore two absolute humidities are computed in the <b>BaseProperties</b> model: <b>X</b> denotes the absolute humidity in terms of the total mass while <b>x</b> denotes the absolute humidity per unit mass of dry air. In addition, the relative humidity <b>phi</b> is also computed.</p>
<p>At the triple point temperature of water of 0.01 &deg;C or 273.16 K and a relative humidity greater than 1 fog may be present as liquid and as ice resulting in a specific enthalpy somewhere between those of the two isotherms for solid and liquid fog, respectively. For numerical reasons a coexisting mixture of 50% solid and 50% liquid fog is assumed in the fog region at the triple point in this model.</p>

<h4>Range of validity</h4>
<p>From the assumptions mentioned above it follows that the <b>pressure</b> should be in the region around <b>atmospheric</b> conditions or below (a few bars may still be fine though). Additionally a very high water content at low temperatures would yield incorrect densities, because the volume of the liquid or solid phase would not be negligible anymore. The model does not provide information on limits for water drop size in the fog region or transport information for the actual condensation or evaporation process in combination with surfaces. All excess water which is not in its vapour state is assumed to be still present in the air regarding its energy but not in terms of its spatial extent.<br><br>
The thermodynamic model may be used for <b>temperatures</b> ranging from <b>190 ... 647 K</b>. This holds for all functions unless otherwise stated in their description. However, although the model works at temperatures above the saturation temperature it is questionable to use the term \"relative humidity\" in this region. Please note, that although several functions compute pure water properties, they are designed to be used within the moist air medium model where properties are dominated by air and steam in their vapor states, and not for pure liquid water applications.</p>

<h4>Transport Properties</h4>
<p>Several additional functions that are not needed to describe the thermodynamic system, but are required to model transport processes, like heat and mass transfer, may be called. They usually neglect the moisture influence unless otherwise stated.</p>

<h4>Application</h4>
<p>The model's main area of application is all processes that involve moist air cooling under near atmospheric pressure with possible moisture condensation. This is the case in all domestic and industrial air conditioning applications. Another large domain of moist air applications covers all processes that deal with dehydration of bulk material using air as a transport medium. Engineering tasks involving moist air are often performed (or at least visualized) by using charts that contain all relevant thermodynamic data for a moist air system. These so called psychrometric charts can be generated from the medium properties in this package. The model <a href=\"modelica://Modelica.Media.Examples.PsychrometricData\">PsychrometricData</a> may be used for this purpose in order to obtain data for figures like those below (the plotting itself is not part of the model though).</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Media/Air/Mollier.png\"><br>
<img src=\"modelica://Modelica/Resources/Images/Media/Air/PsycroChart.png\">
</p>

<p>
<b>Legend:</b> blue - constant specific enthalpy, red - constant temperature, black - constant relative humidity</p>

</html>"));
      end MoistAir;

      package ReferenceMoistAir "ReferenceMoistAir: Detailed moist air model (143.15 ... 2000 K)"
        extends Modelica.Media.Interfaces.PartialRealCondensingGases(mediumName = "Moist air", substanceNames = {"Water", "Air"}, final fixedX = false, final reducedX = true, final singleState = false, reference_X = {0.01, 0.99}, fluidConstants = {Utilities.Water95_Utilities.waterConstants, Modelica.Media.Air.ReferenceAir.airConstants}, ThermoStates = Modelica.Media.Interfaces.Choices.IndependentVariables.pTX);
        constant Integer Water = 1 "Index of water (in substanceNames, massFractions X, etc.)";
        constant Integer Air = 2 "Index of air (in substanceNames, massFractions X, etc.)";
        constant Boolean useEnhancementFactor = false "Use the enhancement factor in the calculations";
        constant Boolean useDissociation = true "Take dissociation into account for high temperatures";
        constant Real k_mair = steam.MM / dryair.MM "Ratio of molar weights";
        constant Common.FundamentalConstants dryair = ReferenceAir.Air_Utilities.Basic.Constants;
        constant Common.FundamentalConstants steam = Utilities.Water95_Utilities.Constants;
        constant SI.MolarMass[2] MMX = {steam.MM, dryair.MM} "Molar masses of components";
        import Modelica.Media.Interfaces;
        import Modelica.Math;
        import SI = Modelica.SIunits;
        import Cv = Modelica.SIunits.Conversions;
        import Modelica.Constants;
        import Modelica.Media.IdealGases.Common.SingleGasNasa;

        redeclare record extends ThermodynamicState "ThermodynamicState record for moist air"
        end ThermodynamicState;

        redeclare replaceable model extends BaseProperties(T(stateSelect = if preferredMediumStates then StateSelect.prefer else StateSelect.default), p(stateSelect = if preferredMediumStates then StateSelect.prefer else StateSelect.default), Xi(each stateSelect = if preferredMediumStates then StateSelect.prefer else StateSelect.default), final standardOrderComponents = true) "Moist air base properties record"
            MassFraction x_water "Mass of total water/mass of dry air";
            Real phi "Relative humidity";

          protected
            MassFraction X_liquid "Mass fraction of liquid or solid water";
            MassFraction X_steam "Mass fraction of steam water";
            MassFraction X_air "Mass fraction of air";
            MassFraction X_sat "Steam water mass fraction of saturation boundary in kg_water/kg_moistair";
            MassFraction x_sat "Steam water mass content of saturation boundary in kg_water/kg_dryair";
            AbsolutePressure p_steam_sat "partial saturation pressure of steam";

          equation
            assert(T >= 143.15 and T <= 2000, "Temperature T is not in the allowed range 143.15 K <= (T =" + String(T) + " K) <= 2000 K required from medium model \"" + mediumName + "\".");
            MM = 1 / (Xi[Water] / MMX[Water] + (1.0 - Xi[Water]) / MMX[Air]);
            p_steam_sat = Modelica.Media.Air.ReferenceMoistAir.Utilities.pds_pT(p, T);
            X_sat = k_mair / (p / p_steam_sat - 1 + k_mair);
            X_liquid = Xi[Water] - X_sat;
            X_steam = Xi[Water] - X_liquid;
            X_air = 1 - Xi[Water];
            h = specificEnthalpy_pTX(p, T, Xi);
            R = dryair.R * (X_air / (1 - X_liquid)) + steam.R * X_steam / (1 - X_liquid);
            u = Modelica.Media.Air.ReferenceMoistAir.Utilities.u_pTX(p, T, Xi);
            d = Modelica.Media.Air.ReferenceMoistAir.Utilities.rho_pTX(p, T, Xi);
            state.p = p;
            state.T = T;
            state.X = X;
// these x are per unit mass of DRY air!
            x_sat = k_mair * p_steam_sat / max(100 * Constants.eps, p - p_steam_sat);
            x_water = Xi[Water] / max(X_air, 100 * Constants.eps);
            phi = Modelica.Media.Air.ReferenceMoistAir.Utilities.phi_pTX(p, T, Xi);
        end BaseProperties;

        redeclare function extends setState_pTX "Return thermodynamic state as function of pressure p, temperature T and composition X"
          algorithm
            state := if size(X, 1) == nX then ThermodynamicState(p = p, T = T, X = X) else ThermodynamicState(p = p, T = T, X = cat(1, X, {1 - sum(X)}));
        end setState_pTX;

        redeclare function extends setState_phX "Return thermodynamic state as function of pressure p, specific enthalpy h and composition X"
          algorithm
            state := if size(X, 1) == nX then ThermodynamicState(p = p, T = Modelica.Media.Air.ReferenceMoistAir.Utilities.Inverses.T_phX(p, h, X), X = X) else ThermodynamicState(p = p, T = Modelica.Media.Air.ReferenceMoistAir.Utilities.Inverses.T_phX(p, h, X), X = cat(1, X, {1 - sum(X)}));
        end setState_phX;

        redeclare function extends setState_psX "Return thermodynamic state as function of pressure p, specific enthalpy h and composition X"
          algorithm
            state := if size(X, 1) == nX then ThermodynamicState(p = p, T = Modelica.Media.Air.ReferenceMoistAir.Utilities.Inverses.T_psX(p, s, X), X = X) else ThermodynamicState(p = p, T = Modelica.Media.Air.ReferenceMoistAir.Utilities.Inverses.T_psX(p, s, X), X = cat(1, X, {1 - sum(X)}));
        end setState_psX;

        redeclare function extends setState_dTX "Return thermodynamic state as function of density d, temperature T and composition X"
          algorithm
            state := if size(X, 1) == nX then ThermodynamicState(p = Modelica.Media.Air.ReferenceMoistAir.Utilities.Inverses.p_dTX(d, T, X), T = T, X = X) else ThermodynamicState(p = Modelica.Media.Air.ReferenceMoistAir.Utilities.Inverses.p_dTX(d, T, X), T = T, X = cat(1, X, {1 - sum(X)}));
        end setState_dTX;

        redeclare function extends setSmoothState "Return thermodynamic state so that it smoothly approximates: if x > 0 then state_a else state_b"
          algorithm
            state := ThermodynamicState(p = Modelica.Media.Common.smoothStep(x, state_a.p, state_b.p, x_small), T = Modelica.Media.Common.smoothStep(x, state_a.T, state_b.T, x_small), X = Modelica.Media.Common.smoothStep(x, state_a.X, state_b.X, x_small));
        end setSmoothState;

        function Xsaturation "Return absolute humitity per unit mass of moist air at saturation as a function of the thermodynamic state record"
          extends Modelica.Icons.Function;
          input ThermodynamicState state "Thermodynamic state record";
          output MassFraction X_sat "Steam mass fraction of sat. boundary";
        protected
          MassFraction[:] X;
        algorithm
          X := massFractionSaturation(state);
          X_sat := X[1];
        end Xsaturation;

        function xsaturation "Return absolute humitity per unit mass of dry air at saturation as a function of the thermodynamic state record"
          extends Modelica.Icons.Function;
          input ThermodynamicState state "Thermodynamic state record";
          output MassFraction x_sat "Absolute humidity per unit mass of dry air";
        algorithm
          x_sat := Modelica.Media.Air.ReferenceMoistAir.Utilities.xws_pT(state.p, state.T);
          assert(x_sat > (-1), "Calculation of absolute humidity is meaningless\nfor input pressure p = " + String(state.p) + " Pa and temperature T = " + String(state.T) + " K.");
        end xsaturation;

        redeclare function extends massFraction_pTphi "Return mass fractions as a function of pressure, temperature and relative humidity"
          protected
            Real pds;

          algorithm
            assert(phi < 1.0 and phi > 0, "Illegal input phi = " + String(phi) + ". Relative humidity is only defined in the range\n 0 <= phi <= 1.0.");
            pds := Modelica.Media.Air.ReferenceMoistAir.Utilities.pds_pT(p, T);
            assert(pds > (-1), "Calculation of mass fraction of steam is meaningless\nfor input pressure p = " + String(p) + " Pa and temperature T = " + String(T) + " K.");
            X := {phi * k_mair / (p / pds - phi), 1 - phi * k_mair / (p / pds - phi)};
        end massFraction_pTphi;

        function massFractionWaterVapor "Return mass fraction of water vapor"
          extends Modelica.Icons.Function;
          input ThermodynamicState state "Thermodynamic state record";
          output MassFraction X "Mass fraction of water varpor";
        protected
          Real xw;
          Real xws;
        algorithm
          xw := state.X[1] / (1 - state.X[1]);
          xws := Utilities.xws_pT(state.p, state.T);
          X := if xw <= xws then xw / (1 + xw) else xws / (1 + xw);
        end massFractionWaterVapor;

        function massFractionWaterNonVapor "Return mass fraction of liquid and solid water"
          extends Modelica.Icons.Function;
          input ThermodynamicState state "Thermodynamic state record";
          output MassFraction X "Mass fraction of water varpor";
        protected
          Real xw;
          Real xws;
        algorithm
          xw := state.X[1] / (1 - state.X[1]);
          xws := Utilities.xws_pT(state.p, state.T);
          X := if xw <= xws then 0 else (xw - xws) / (1 + xw);
        end massFractionWaterNonVapor;

        redeclare function extends massFractionSaturation "Return saturation mass fractions"
          protected
            AbsolutePressure pds;

          algorithm
            pds := Utilities.pds_pT(state.p, state.T);
            Xsat := {k_mair / (state.p / pds - 1 + k_mair), (state.p / pds - 1) / (state.p / pds - 1 + k_mair)};
            assert(Xsat[1] > (-1), "Calculation of saturation mass fraction is meaningless\nfor input pressure p = " + String(state.p) + " Pa and temperature T = " + String(state.T) + " K.");
        end massFractionSaturation;

        function massFractionSaturation_ppsat "Return mass fvraction at saturation boundary given pressure and saturation pressure"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Ambient pressure";
          input AbsolutePressure psat "Saturation pressure";
          output MassFraction[:] X "Mass fraction";
        algorithm
          X := {k_mair / (p / psat - 1 + k_mair), (p / psat - 1) / (p / psat - 1 + k_mair)};
        end massFractionSaturation_ppsat;

        function massFraction_waterContent "Return mass fractions as a function of pressure, temperature and absolute humidity in kg(water)/kg(dry air)"
          extends Modelica.Icons.Function;
          input Real xw "Water content in kg(water)/kg(dry air)";
          output MassFraction[:] X "Mass fractions";
        algorithm
          X := {xw / (1 + xw), 1 / (1 + xw)};
        end massFraction_waterContent;

        function waterContent_X "Return water content in kg(water)/kg(dry air) given mass fractions"
          extends Modelica.Icons.Function;
          input MassFraction[:] X "Mass fractions";
          output Real xw "Water content in kg(water)/kg(dry air)";
        algorithm
          xw := X[1] / (1 - X[1]);
        end waterContent_X;

        redeclare function extends relativeHumidity "Return relativ humidity"
          algorithm
            phi := Utilities.phi_pTX(state.p, state.T, state.X);
            assert(phi > (-1), "Calculation of relative humidity is meaningless\nfor input pressure p = " + String(state.p) + " Pa and temperature T = " + String(state.T) + " K.");
        end relativeHumidity;

        redeclare function extends gasConstant "Return ideal gas constant as a function from thermodynamic state, only valid for phi<1"
          algorithm
            R := dryair.R * (1 - state.X[Water]) + steam.R * state.X[Water];
        end gasConstant;

        function saturationPressureLiquid "Return saturation pressure of water as a function of temperature T"
          extends Modelica.Icons.Function;
          input ThermodynamicState state "Thermodynamic state record";
          output AbsolutePressure psat "Saturation pressure";
        algorithm
          psat := Modelica.Media.Air.ReferenceMoistAir.Utilities.Water95_Utilities.psat(state.T);
        end saturationPressureLiquid;

        function sublimationPressureIce "Return sublimation pressure of water as a function of temperature T between 223.16 and 273.16 K"
          extends Modelica.Icons.Function;
          input ThermodynamicState state "Thermodynamic state record";
          output AbsolutePressure psat "Sublimation pressure";
        algorithm
          psat := Modelica.Media.Air.ReferenceMoistAir.Utilities.Ice09_Utilities.Basic.psub(state.T);
        end sublimationPressureIce;

        redeclare function extends saturationPressure "Return saturation pressure of condensing fluid"
          algorithm
            psat := Utilities.pds_pT(state.p, state.T);
            assert(psat > (-1), "Calculation of saturation pressure is meaningless\nfor input temperature T = " + String(state.T) + " K.");
        end saturationPressure;

        redeclare function extends saturationTemperature "Return saturation temperature of condensing fluid"
          protected
            partial function Tsat_res
              extends Modelica.Math.Nonlinear.Interfaces.partialScalarFunction;
              input SI.Pressure p "Pressure";
            algorithm
              y := Modelica.Media.Air.ReferenceMoistAir.Utilities.pds_pT(p = p, T = u) - p;
            end Tsat_res;

          algorithm
            Tsat := Modelica.Math.Nonlinear.solveOneNonlinearEquation(function Tsat_res(p = state.p), 50.0, 673.15, 1e-9);
        end saturationTemperature;

        redeclare function extends enthalpyOfVaporization "Return enthalpy of vaporization of water"
          protected
            AbsolutePressure p_liq;

          algorithm
            p_liq := saturationPressureLiquid(state);
            r0 := Modelica.Media.Water.IF97_Utilities.hv_p(p_liq) - Modelica.Media.Water.IF97_Utilities.hl_p(p_liq);
        end enthalpyOfVaporization;

        redeclare function extends enthalpyOfLiquid "Return enthalpy of liquid water"
          protected
            Real xw;
            Real xws;

          algorithm
            xw := state.X[1] / (1 - state.X[1]);
            xws := Utilities.xws_pT(state.p, state.T);
            if xws > xw and state.T > 273.15 then
              h := Modelica.Media.Water.IF97_Utilities.h_pT(state.p, state.T, region = 1);
            else
              h := 0;
            end if;
        end enthalpyOfLiquid;

        redeclare function extends enthalpyOfGas "Return specific enthalpy of gas (air and steam)"
          protected
            Real xw;
            Real xws;
            Real pd;
            Real pl;

          algorithm
            pd := Utilities.pd_pTX(state.p, state.T, state.X);
            pl := state.p - pd;
            xw := state.X[1] / (1 - state.X[1]);
            xws := Utilities.xws_pT(state.p, state.T);
            if xw <= xws or xws == (-1) then
              h := Modelica.Media.Air.ReferenceAir.Air_Utilities.h_pT(pl, state.T) + xw * Utilities.IF97_new.h_pT(pd, state.T);
            else
              if state.T < 273.16 then
                h := Modelica.Media.Air.ReferenceAir.Air_Utilities.h_pT(pl, state.T) + xws * Utilities.IF97_new.h_pT(pd, state.T);
              else
                h := Modelica.Media.Air.ReferenceAir.Air_Utilities.h_pT(pl, state.T) + xws * Utilities.IF97_new.h_pT(pd, state.T);
              end if;
            end if;
          annotation(
            Inline = false,
            LateInline = true);
        end enthalpyOfGas;

        redeclare function extends enthalpyOfCondensingGas "Return specific enthalpy of steam"
          protected
            Real xw;
            Real pd;

          algorithm
            pd := Utilities.pd_pTX(state.p, state.T, state.X);
            xw := state.X[1] / (1 - state.X[1]);
            h := xw * Utilities.IF97_new.h_pT(pd, state.T);
          annotation(
            Inline = false,
            LateInline = true);
        end enthalpyOfCondensingGas;

        redeclare function extends enthalpyOfNonCondensingGas "Return specific enthalpy of dry air"
          algorithm
            h := Modelica.Media.Air.ReferenceAir.Air_Utilities.h_pT(state.p, state.T);
        end enthalpyOfNonCondensingGas;

        function enthalpyOfDryAir = enthalpyOfNonCondensingGas "Return specific enthalpy of dry air";

        function enthalpyOfWater "Return specific enthalpy of water (solid + liquid + steam)"
          extends Modelica.Icons.Function;
          input ThermodynamicState state "Thermodynamic state record";
          output SI.SpecificEnthalpy h "Specific enthalpy of water";
        algorithm
          h := specificEnthalpy(state) - enthalpyOfNonCondensingGas(state);
        end enthalpyOfWater;

        function enthalpyOfWaterVapor = enthalpyOfCondensingGas "Return specific enthalpy of steam";

        function enthalpyOfWaterNonVapor "Return enthalpy of liquid and solid water"
          extends Modelica.Icons.Function;
          input ThermodynamicState state "Thermodynamic state record";
          output SI.SpecificEnthalpy h "Specific enthalpy of water";
        algorithm
          h := enthalpyOfWater(state) - enthalpyOfWaterVapor(state);
        end enthalpyOfWaterNonVapor;

        redeclare function extends pressure "Returns pressure of ideal gas as a function of the thermodynamic state record"
          algorithm
            p := state.p;
        end pressure;

        redeclare function extends temperature "Return temperature of ideal gas as a function of the thermodynamic state record"
          algorithm
            T := state.T;
        end temperature;

        redeclare function extends density "Returns density as a function of the thermodynamic state record"
          algorithm
            d := Utilities.rho_pTX(state.p, state.T, state.X);
        end density;

        redeclare function extends specificEnthalpy "Return specific enthalpy of moist air as a function of the thermodynamic state record"
          algorithm
            h := Modelica.Media.Air.ReferenceMoistAir.Utilities.h_pTX(state.p, state.T, state.X);
        end specificEnthalpy;

        redeclare function extends specificInternalEnergy "Return specific internal energy of moist air as a function of the thermodynamic state record"
          algorithm
            u := Utilities.u_pTX(state.p, state.T, state.X);
        end specificInternalEnergy;

        redeclare function extends specificEntropy "Return specific entropy from thermodynamic state record, only valid for phi<1"
          algorithm
            s := Utilities.s_pTX(state.p, state.T, state.X);
        end specificEntropy;

        redeclare function extends specificGibbsEnergy "Return specific Gibbs energy as a function of the thermodynamic state record, only valid for phi<1"
          algorithm
            g := Utilities.h_pTX(state.p, state.T, state.X) - state.T * Utilities.s_pTX(state.p, state.T, state.X);
        end specificGibbsEnergy;

        redeclare function extends specificHelmholtzEnergy "Return specific Helmholtz energy as a function of the thermodynamic state record, only valid for phi<1"
          algorithm
            f := Utilities.u_pTX(state.p, state.T, state.X) - state.T * Utilities.s_pTX(state.p, state.T, state.X);
        end specificHelmholtzEnergy;

        redeclare function extends specificHeatCapacityCp "Return specific heat capacity at constant pressure as a function of the thermodynamic state record"
          algorithm
            cp := Utilities.cp_pTX(state.p, state.T, state.X);
        end specificHeatCapacityCp;

        redeclare function extends specificHeatCapacityCv "Return specific heat capacity at constant volume as a function of the thermodynamic state record"
          algorithm
            cv := Utilities.cv_pTX(state.p, state.T, state.X);
        end specificHeatCapacityCv;

        redeclare function extends isentropicExponent "Return isentropic exponent"
          algorithm
            gamma := specificHeatCapacityCp(state) / specificHeatCapacityCv(state);
        end isentropicExponent;

        redeclare function extends isentropicEnthalpy "Return isentropic enthalpy"
          protected
            MassFraction[nX] X "Complete X-vector";

          algorithm
/*X := if reducedX then cat(
      1,
      refState.X,
      {1 - sum(refState.X)}) else refState.X;*/
            X := refState.X;
            h_is := specificEnthalpy(setState_psX(p_downstream, specificEntropy(refState), X));
          annotation(
            Documentation(revisions = "<html>
2013-07-18 Stefan Wischhusen: Changed internal calculation of X.
</html>"));
        end isentropicEnthalpy;

        redeclare function extends velocityOfSound "Return velocity of sound"
          algorithm
            a := sqrt(max(0, gasConstant(state) * state.T * specificHeatCapacityCp(state) / specificHeatCapacityCv(state)));
        end velocityOfSound;

        redeclare function extends molarMass "Return the molar mass of the medium"
          algorithm
            MM := 1 / (state.X[1] * steam.MM + state.X[2] * dryair.MM);
        end molarMass;

        redeclare function extends dynamicViscosity "Return dynamic viscosity as a function of the thermodynamic state record, valid from 73.15 K to 373.15 K"
          algorithm
            eta := Utilities.Transport.eta_pTX(state.p, state.T, state.X);
        end dynamicViscosity;

        redeclare function extends thermalConductivity "Return thermal conductivity as a function of the thermodynamic state record, valid from 73.15 K to 373.15 K"
          algorithm
            lambda := Utilities.Transport.lambda_pTX(state.p, state.T, state.X);
        end thermalConductivity;

        package Utilities "Utilitiy package for moist air"
          extends Modelica.Icons.UtilitiesPackage;
          final constant MoleFraction[4] MMX = {18.015257E-003, 28.01348E-003, 31.9988E-003, 39.948E-003};
          final constant Real[3] Xi_Air = {0.7557, 0.2316, 0.0127};

          package Inverses "Compute inverse function"
            extends Modelica.Icons.BasesPackage;

            function T_phX "Return temperature as a function of pressure, specific enthalpy and mass fractions"
              extends Modelica.Icons.Function;
              input Modelica.SIunits.AbsolutePressure p "Pressure";
              input Modelica.SIunits.SpecificEnthalpy h "Specific enthalpy";
              input Modelica.SIunits.MassFraction X[:] = Modelica.Media.Air.ReferenceMoistAir.reference_X "Mass fractions";
              output Modelica.SIunits.Temperature T "Temperature";
            protected
              MassFraction[nX] Xfull = if size(X, 1) == nX then X else cat(1, X, {1 - sum(X)});

              function T_phX_res
                extends Modelica.Math.Nonlinear.Interfaces.partialScalarFunction;
                input Modelica.SIunits.AbsolutePressure p "Pressure";
                input Modelica.SIunits.SpecificEnthalpy h "Specific enthalpy";
                input Modelica.SIunits.MassFraction X[:] = Modelica.Media.Air.ReferenceMoistAir.reference_X "Mass fractions";
              algorithm
                y := Modelica.Media.Air.ReferenceMoistAir.Utilities.h_pTX(p = p, T = u, X = X) - h;
              end T_phX_res;
            algorithm
              T := Modelica.Math.Nonlinear.solveOneNonlinearEquation(function T_phX_res(p = p, h = h, X = Xfull), 173.15, 2000.0, 1e-9);
              annotation(
                inverse(h = Modelica.Media.Air.ReferenceMoistAir.Utilities.h_pTX(p = p, T = T, X = X)));
            end T_phX;

            function T_psX "Return temperature as function of pressure, specific entropy and mass fractions"
              extends Modelica.Icons.Function;
              input Modelica.SIunits.AbsolutePressure p "Pressure";
              input Modelica.SIunits.SpecificEntropy s "Specific entropy";
              input Modelica.SIunits.MassFraction X[:] = Modelica.Media.Air.ReferenceMoistAir.reference_X "Mass fractions";
              output Modelica.SIunits.Temperature T "Temperature";
            protected
              MassFraction[nX] Xfull = if size(X, 1) == nX then X else cat(1, X, {1 - sum(X)});

              function T_psX_res
                extends Modelica.Math.Nonlinear.Interfaces.partialScalarFunction;
                input Modelica.SIunits.AbsolutePressure p "Pressure";
                input Modelica.SIunits.SpecificEntropy s "Specific entropy";
                input Modelica.SIunits.MassFraction X[:] = Modelica.Media.Air.ReferenceMoistAir.reference_X "Mass fractions";
              algorithm
                y := Modelica.Media.Air.ReferenceMoistAir.Utilities.s_pTX(p = p, T = u, X = X) - s;
              end T_psX_res;
            algorithm
              T := Modelica.Math.Nonlinear.solveOneNonlinearEquation(function T_psX_res(p = p, s = s, X = Xfull), 173.15, 2000.0, 1e-9);
              annotation(
                inverse(s = Modelica.Media.Air.ReferenceMoistAir.Utilities.s_pTX(p = p, T = T, X = X)));
            end T_psX;

            function p_dTX "Return pressure as function of density, temperature and mass fractions"
              extends Modelica.Icons.Function;
              input Modelica.SIunits.Density d "Density";
              input Modelica.SIunits.Temperature T "Temperature";
              input Modelica.SIunits.MassFraction X[:] = Modelica.Media.Air.ReferenceMoistAir.reference_X "Mass fractions";
              output Modelica.SIunits.AbsolutePressure p "Pressure";
            protected
              MassFraction[nX] Xfull = if size(X, 1) == nX then X else cat(1, X, {1 - sum(X)});

              function p_dTX_res
                extends Modelica.Math.Nonlinear.Interfaces.partialScalarFunction;
                input Modelica.SIunits.Density d "Density";
                input Modelica.SIunits.Temperature T "Temperature";
                input Modelica.SIunits.MassFraction X[:] = Modelica.Media.Air.ReferenceMoistAir.reference_X "Mass fractions";
              algorithm
                y := Modelica.Media.Air.ReferenceMoistAir.Utilities.rho_pTX(p = u, T = T, X = X) - d;
              end p_dTX_res;
            algorithm
              p := Modelica.Math.Nonlinear.solveOneNonlinearEquation(function p_dTX_res(d = d, T = T, X = Xfull), 611.2, 1e7, 1e-9);
              annotation(
                inverse(d = Modelica.Media.Air.ReferenceMoistAir.Utilities.rho_pTX(p = p, T = T, X = X)),
                Documentation(revisions = "<html>
2013-07-18 Stefan Wischhusen: Corrected inverse intervall of pressure to complete range of medium model.
</html>"));
            end p_dTX;
          end Inverses;

          package Transport "Package for transport properties of moist air"
            extends Modelica.Icons.BasesPackage;

            record coef "Coefficients for polynomials used to calculate transport properties"
              extends Modelica.Icons.Record;
              Real sigma = 2.52;
              Real epsilon = 775;
              Real M = 18.0152;
              Real R = 0.46144;
              Real[5] w = {0.69339511, -0.002597963, 1.2864772, 0.1576848, 0.02543632};
              Real[5] a = {0.4159259E+001, -0.1725577E-002, 0.5702012E-005, -0.4596049E-008, 0.1424309E-011};
            end coef;

            function eta_pTX "Dynamic viscosity"
              extends Modelica.Icons.Function;
              input Modelica.SIunits.AbsolutePressure p "Pressure";
              input Modelica.SIunits.Temperature T "Temperature";
              input Modelica.SIunits.MassFraction X[:] = Modelica.Media.Air.ReferenceMoistAir.reference_X "Mass fractions";
              output Modelica.SIunits.DynamicViscosity eta "Dynamic viscosity";
            protected
              Real ya;
              Real yd;
              Real yf;
              Real va;
              Real vd;
              Real vf;
              Real xw;
              Real xws;
              Real pd;
              Real pl;
              Real da;
              Real dd;
              Real df;
              Real Omega;
              Real Tred;
              Real etad;
              Modelica.Media.Air.ReferenceMoistAir.Utilities.Transport.coef coef;
            algorithm
              xw := X[1] / (1 - X[1]);
              xws := Modelica.Media.Air.ReferenceMoistAir.Utilities.xws_pT(p, T);
              pd := Modelica.Media.Air.ReferenceMoistAir.Utilities.pd_pTX(p, T, X);
              pl := p - pd;
              da := Modelica.Media.Air.ReferenceAir.Air_Utilities.rho_pT(pl, T);
              if xw <= xws or xws == (-1) then
                if T < 273.16 then
                  dd := pd / (Modelica.Media.Air.ReferenceMoistAir.steam.R * T);
                  ya := da / (da + dd);
                  yd := 1 - ya;
                  Tred := T / coef.epsilon;
                  Omega := coef.w[1] + coef.w[2] * Tred + coef.w[3] * Modelica.Math.exp(coef.w[4] * Tred) / (coef.w[5] + Tred);
                  etad := 2.6695E-006 * sqrt(T * coef.M) / (coef.sigma ^ 2 * Omega);
                  eta := ya * Modelica.Media.Air.ReferenceAir.Air_Utilities.Transport.eta_dT(da, T) + yd * etad;
                else
                  dd := Modelica.Media.Air.ReferenceMoistAir.Utilities.IF97_new.rho_pT(pd, T);
                  ya := da / (da + dd);
                  yd := 1 - ya;
                  eta := ya * Modelica.Media.Air.ReferenceAir.Air_Utilities.Transport.eta_dT(da, T) + yd * Modelica.Media.Air.ReferenceMoistAir.Utilities.IF97_new.visc_dT(dd, T);
                end if;
              else
                if T < 273.16 then
                  dd := pd / (Modelica.Media.Air.ReferenceMoistAir.steam.R * T);
                  ya := da / (da + dd);
                  yd := 1 - ya;
                  Tred := T / coef.epsilon;
                  Omega := coef.w[1] + coef.w[2] * Tred + coef.w[3] * Modelica.Math.exp(coef.w[4] * Tred) / (coef.w[5] + Tred);
                  etad := 2.6695E-006 * sqrt(T * coef.M) / (coef.sigma ^ 2 * Omega);
                  eta := ya * Modelica.Media.Air.ReferenceAir.Air_Utilities.Transport.eta_dT(da, T) + yd * etad;
                else
                  dd := Modelica.Media.Air.ReferenceMoistAir.Utilities.IF97_new.rho_pT(pd, T);
                  df := Modelica.Media.Water.IF97_Utilities.rho_pT(p, T);
                  yf := (xw - xws) / df / ((1 + xws) / (da + dd) + (xw - xws) / df);
                  ya := (1 - yf) / (1 + dd / da);
                  yd := 1 - (ya + yf);
                  eta := ya * Modelica.Media.Air.ReferenceAir.Air_Utilities.Transport.eta_dT(da, T) + yd * Modelica.Media.Air.ReferenceMoistAir.Utilities.IF97_new.visc_dT(dd, T) + yf * Modelica.Media.Air.ReferenceMoistAir.Utilities.IF97_new.visc_dT(df, T);
                end if;
              end if;
            end eta_pTX;

            function lambda_pTX "Thermal conductivity"
              extends Modelica.Icons.Function;
              input Modelica.SIunits.AbsolutePressure p "Pressure";
              input Modelica.SIunits.Temperature T "Temperature";
              input Modelica.SIunits.MassFraction X[:] = Modelica.Media.Air.ReferenceMoistAir.reference_X "Mass fractions";
              output Modelica.SIunits.ThermalConductivity lambda "Thermal conductivity";
            protected
              Real ya;
              Real yd;
              Real yf;
              Real va;
              Real vd;
              Real vf;
              Real xw;
              Real xws;
              Real pd;
              Real pl;
              Real da;
              Real dd;
              Real df;
              Real Omega;
              Real Tred;
              Real cp;
              Real Eu;
              Real lambdad;
              Modelica.Media.Air.ReferenceMoistAir.Utilities.Transport.coef coef;
            algorithm
              xw := X[1] / (1 - X[1]);
              xws := Modelica.Media.Air.ReferenceMoistAir.Utilities.xws_pT(p, T);
              pd := Modelica.Media.Air.ReferenceMoistAir.Utilities.pd_pTX(p, T, X);
              pl := p - pd;
              da := Modelica.Media.Air.ReferenceAir.Air_Utilities.rho_pT(pl, T);
              if xw <= xws or xws == (-1) then
                if T < 273.16 then
                  dd := pd / (Modelica.Media.Air.ReferenceMoistAir.steam.R * T);
                  ya := da / (da + dd);
                  yd := 1 - ya;
                  Tred := T / coef.epsilon;
                  Omega := coef.w[1] + coef.w[2] * Tred + coef.w[3] * Modelica.Math.exp(coef.w[4] * Tred) / (coef.w[5] + Tred);
                  cp := coef.a[1] + coef.a[2] * T + coef.a[3] * T ^ 2 + coef.a[4] * T ^ 3 + coef.a[5] * T ^ 4;
                  Eu := 0.35424 * cp + 0.1144;
                  Eu := 0;
                  lambdad := 0.083232 * sqrt(T / coef.M) / (coef.sigma ^ 2 * Omega) * Eu;
                  lambda := ya * Modelica.Media.Air.ReferenceAir.Air_Utilities.Transport.lambda_dT(da, T) + yd * lambdad;
                else
                  dd := Modelica.Media.Air.ReferenceMoistAir.Utilities.IF97_new.rho_pT(pd, T);
                  ya := da / (da + dd);
                  yd := 1 - ya;
                  lambda := ya * Modelica.Media.Air.ReferenceAir.Air_Utilities.Transport.lambda_dT(da, T) + yd * Modelica.Media.Air.ReferenceMoistAir.Utilities.IF97_new.cond_dT(dd, T);
                end if;
              else
                if T < 273.16 then
                  dd := pd / (Modelica.Media.Air.ReferenceMoistAir.steam.R * T);
                  df := Modelica.Media.Air.ReferenceMoistAir.Utilities.Ice09_Utilities.rho_pT(p, T);
                  yf := (xw - xws) / df / ((1 + xws) / (da + dd) + (xw - xws) / df);
                  ya := (1 - yf) / (1 + dd / da);
                  yd := 1 - (ya + yf);
                  Tred := T / coef.epsilon;
                  Omega := coef.w[1] + coef.w[2] * Tred + coef.w[3] * Modelica.Math.exp(coef.w[4] * Tred) / (coef.w[5] + Tred);
                  cp := coef.a[1] + coef.a[2] * T + coef.a[3] * T ^ 2 + coef.a[4] * T ^ 3 + coef.a[5] * T ^ 4;
                  Eu := 0.35424 * cp + 0.1144;
                  lambdad := 0.083232 * sqrt(T / coef.M) / (coef.sigma ^ 2 * Omega) * Eu;
                  lambda := ya * Modelica.Media.Air.ReferenceAir.Air_Utilities.Transport.lambda_dT(da, T) + yd * lambdad + yf * 2.21;
                else
                  dd := Modelica.Media.Air.ReferenceMoistAir.Utilities.IF97_new.rho_pT(pd, T);
                  df := Modelica.Media.Water.IF97_Utilities.rho_pT(p, T);
                  yf := (xw - xws) / df / ((1 + xws) / (da + dd) + (xw - xws) / df);
                  ya := (1 - yf) / (1 + dd / da);
                  yd := 1 - (ya + yf);
                  lambda := ya * Modelica.Media.Air.ReferenceAir.Air_Utilities.Transport.lambda_dT(da, T) + yd * Modelica.Media.Air.ReferenceMoistAir.Utilities.IF97_new.cond_dT(dd, T) + yf * Modelica.Media.Air.ReferenceMoistAir.Utilities.IF97_new.cond_dT(df, T);
                end if;
              end if;
            end lambda_pTX;
          end Transport;

          package VirialCoefficients "Virial and cross-virial coefficients of air and water"
            extends Modelica.Icons.BasesPackage;

            function Baa_dT "Second molar virial coefficient of dry air"
              extends Modelica.Icons.Function;
              input Modelica.SIunits.Density d "Density";
              input Modelica.SIunits.Temperature T "Temperature";
              output Modelica.SIunits.MolarVolume baa "Second virial coefficient";
            protected
              final constant Real[19] N = {0.118160747229, 0.713116392079, -0.161824192067E+001, 0.714140178971E-001, -0.865421396646E-001, 0.134211176704, 0.112626704218E-001, -0.420533228842E-001, 0.349008431982E-001, 0.164957183186E-003, -0.101365037912, -0.173813690970, -0.472103183731E-001, -0.122523554253E-001, -0.146629609713, -0.316055879821E-001, 0.233594806142E-003, 0.148287891978E-001, -0.938782884667E-002};
              final constant Integer[19] i = {1, 1, 1, 2, 3, 3, 4, 4, 4, 6, 1, 3, 5, 6, 1, 3, 11, 1, 3};
              final constant Real[19] j = {0, 0.33, 1.01, 0, 0, 0.15, 0, 0.2, 0.35, 1.35, 1.6, 0.8, 0.95, 1.25, 3.6, 6, 3.25, 3.5, 15};
              Real tau = ReferenceAir.Air_Utilities.Basic.Constants.Tred / T;
            algorithm
              baa := 0;
              for k in 1:19 loop
                baa := if i[k] == 1 then baa + N[k] * tau ^ j[k] else baa;
              end for;
              baa := 1 / ReferenceAir.Air_Utilities.Basic.Constants.rhored * baa;
            end Baa_dT;

            function Baw_dT "Second molar cross-virial coefficient"
              extends Modelica.Icons.Function;
              input Modelica.SIunits.Density d "Density";
              input Modelica.SIunits.Temperature T "Temperature";
              output Modelica.SIunits.MolarVolume baw "Second cross-virial coefficient";
            protected
              final constant Real[3] a = {0.665687E+002, -0.238834E+003, -0.176755E+003};
              final constant Real[3] b = {-0.237, -1.048, -3.183};
              Real theta;
            algorithm
              baw := 0;
              theta := T / 100;
              for k in 1:3 loop
                baw := baw + a[k] * theta ^ b[k];
              end for;
              baw := baw * 1E-006;
            end Baw_dT;

            function Bww_dT "Second molar virial coefficient of water"
              extends Modelica.Icons.Function;
              input Modelica.SIunits.Density d "Density";
              input Modelica.SIunits.Temperature T "Temperature";
              output Modelica.SIunits.MolarVolume bww "Second virial coefficient";
            protected
              final constant Real[56] N = {0.12533547935523E-001, 0.78957634722828E+001, -0.87803203303561E+001, 0.31802509345418, -0.26145533859358, -0.78199751687981E-002, 0.88089493102134E-002, -0.66856572307965, 0.20433810950965, -0.66212605039687E-004, -0.19232721156002, -0.25709043003438, 0.16074868486251, -0.40092828925807E-001, 0.39343422603254E-006, -0.75941377088144E-005, 0.56250979351888E-003, -0.15608652257135E-004, 0.11537996422951E-008, 0.36582165144204E-006, -0.13251180074668E-011, -0.62639586912454E-009, -0.10793600908932, 0.17611491008752E-001, 0.22132295167546, -0.40247669763528, 0.58083399985759, 0.49969146990806E-002, -0.31358700712549E-001, -0.74315929710341, 0.4780732991548, 0.20527940895948E-001, -0.13636435110343, 0.14180634400617E-001, 0.83326504880713E-002, -0.29052336009585E-001, 0.38615085574206E-001, -0.20393486513704E-001, -0.16554050063734E-002, 0.19955571979541E-002, 0.15870308324157E-003, -0.1638856834253E-004, 0.43613615723811E-001, 0.34994005463765E-001, -0.76788197844621E-001, 0.22446277332006E-001, -0.62689710414685E-004, -0.55711118565645E-009, -0.19905718354408, 0.31777497330738, -0.11841182425981, -0.31306260323435E+002, 0.31546140237781E+002, -0.25213154341695E+004, -0.14874640856724, 0.31806110878444};
              final constant Integer[51] c = {0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 3, 3, 3, 3, 4, 6, 6, 6, 6};
              final constant Integer[54] dd = {1, 1, 1, 2, 2, 3, 4, 1, 1, 1, 2, 2, 3, 4, 4, 5, 7, 9, 10, 11, 13, 15, 1, 2, 2, 2, 3, 4, 4, 4, 5, 6, 6, 7, 9, 9, 9, 9, 9, 10, 10, 12, 3, 4, 4, 5, 14, 3, 6, 6, 6, 3, 3, 3};
              final constant Real[54] t = {-0.5, 0.875, 1.0, 0.5, 0.75, 0.375, 1.0, 4.0, 6.0, 12.0, 1.0, 5.0, 4.0, 2.0, 13.0, 9.0, 3.0, 4.0, 11.0, 4.0, 13.0, 1.0, 7.0, 1.0, 9.0, 10.0, 10.0, 3.0, 7.0, 10.0, 10.0, 6.0, 10.0, 10.0, 1.0, 2.0, 3.0, 4.0, 8.0, 6.0, 9.0, 8.0, 16.0, 22.0, 23.0, 23.0, 10.0, 50.0, 44.0, 46.0, 50.0, 0.0, 1.0, 4.0};
              final constant Integer[54] alpha = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 20, 20, 20};
              final constant Real[56] beta = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 150, 150, 250, 0.3, 0.3};
              final constant Real[54] gamma = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21, 1.21, 1.25};
              final constant Integer[54] epsilon = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1};
              final constant Real[56] a = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 3.5, 3.5};
              final constant Real[56] b = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.85, 0.95};
              final constant Real[56] AA = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.32, 0.32};
              final constant Real[56] BB = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.2, 0.2};
              final constant Integer[56] CC = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 28, 32};
              final constant Integer[56] DD = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 700, 800};
              Real Delta55 = 0;
              Real Delta56 = 0;
              Real theta55 = 0;
              Real theta56 = 0;
              Real psi55 = 0;
              Real psi56 = 0;
              Real Delta55delta = 0;
              Real Delta56delta = 0;
              Real Deltab55delta = 0;
              Real Deltab56delta = 0;
              Real psi55delta = 0;
              Real psi56delta = 0;
              Real tau = Water95_Utilities.Constants.Tred / T;
            algorithm
              bww := 0;
              theta55 := 1 - tau + AA[55];
              theta56 := 1 - tau + AA[56];
              psi55 := exp((-CC[55]) - DD[55] * (tau - 1) ^ 2);
              psi56 := exp((-CC[56]) - DD[56] * (tau - 1) ^ 2);
              Delta55 := theta55 ^ 2 + BB[55];
              Delta56 := theta56 ^ 2 + BB[56];
              Delta55delta := -(AA[55] * theta55 * 2 / beta[55] + 2 * BB[55] * a[55]);
              Delta56delta := -(AA[56] * theta56 * 2 / beta[56] + 2 * BB[56] * a[56]);
              Deltab55delta := b[55] * Delta55 ^ (b[55] - 1) * Delta55delta;
              Deltab56delta := b[56] * Delta56 ^ (b[56] - 1) * Delta56delta;
              psi55delta := 2 * CC[55] * psi55;
              psi56delta := 2 * CC[56] * psi56;
              for k in 1:7 loop
                bww := if dd[k] == 1 then bww + N[k] * tau ^ t[k] else bww;
              end for;
              for k in 8:51 loop
                bww := if dd[k] == 1 then bww + N[k] * tau ^ t[k] * dd[k] else bww;
              end for;
              for k in 52:54 loop
                bww := if dd[k] == 1 then bww + N[k] * tau ^ t[k] * exp((-alpha[k] * epsilon[k] ^ 2) - beta[k] * (tau - gamma[k]) ^ 2) else bww;
              end for;
              bww := (bww + N[55] * Delta55 ^ b[55] * psi55 + N[56] * Delta56 ^ b[56] * psi56) / Modelica.Media.Air.ReferenceMoistAir.Utilities.Water95_Utilities.Constants.rhored * Modelica.Media.Air.ReferenceMoistAir.Utilities.Water95_Utilities.Constants.MM;
            end Bww_dT;

            function Caaa_dT "Third molar virial coefficient of dry air"
              extends Modelica.Icons.Function;
              input Modelica.SIunits.Density d "Density";
              input Modelica.SIunits.Temperature T "Temperature";
              output Modelica.SIunits.MolarVolume caaa "Third virial coefficient";
            protected
              final constant Real[19] N = {0.118160747229, 0.713116392079, -0.161824192067E+001, 0.714140178971E-001, -0.865421396646E-001, 0.134211176704, 0.112626704218E-001, -0.420533228842E-001, 0.349008431982E-001, 0.164957183186E-003, -0.101365037912, -0.173813690970, -0.472103183731E-001, -0.122523554253E-001, -0.146629609713, -0.316055879821E-001, 0.233594806142E-003, 0.148287891978E-001, -0.938782884667E-002};
              final constant Integer[19] i = {1, 1, 1, 2, 3, 3, 4, 4, 4, 6, 1, 3, 5, 6, 1, 3, 11, 1, 3};
              final constant Real[19] j = {0, 0.33, 1.01, 0, 0, 0.15, 0, 0.2, 0.35, 1.35, 1.6, 0.8, 0.95, 1.25, 3.6, 6, 3.25, 3.5, 15};
              final constant Integer[19] l = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 2, 2, 2, 3, 3};
              Real tau = ReferenceAir.Air_Utilities.Basic.Constants.Tred / T;
            algorithm
              caaa := 0;
              for k in 1:10 loop
                caaa := if i[k] == 2 then caaa + 2 * N[k] * tau ^ j[k] else caaa;
              end for;
              for k in 11:19 loop
                caaa := if i[k] == 2 then caaa + 2 * N[k] * tau ^ j[k] elseif i[k] == 1 and l[k] == 1 then caaa - 2 * N[k] * tau ^ j[k] else caaa;
              end for;
              caaa := 1 / ReferenceAir.Air_Utilities.Basic.Constants.rhored ^ 2 * caaa;
            end Caaa_dT;

            function Caaw_dT "Third molar cross-virial coefficient"
              extends Modelica.Icons.Function;
              input Modelica.SIunits.Density d "Density";
              input Modelica.SIunits.Temperature T "Temperature";
              output Real caaw "Third cross-virial coefficient";
            protected
              final constant Real[5] c = {0.482737E+003, 0.105678E+006, -0.656394E+008, 0.294442E+011, -0.319317E+013};
              Real theta = T;
            algorithm
              caaw := 0;
              for k in 1:5 loop
                caaw := caaw + c[k] * theta ^ (1 - k);
              end for;
              caaw := caaw * 1E-012;
            end Caaw_dT;

            function Caww_dT "Third molar cross-virial coefficient"
              extends Modelica.Icons.Function;
              input Modelica.SIunits.Density d "Density";
              input Modelica.SIunits.Temperature T "Temperature";
              output Real caww "Third cross-virial coefficient";
            protected
              final constant Real[4] dd = {-0.1072887E+002, 0.347804E+004, -0.383383E+006, 0.33406E+008};
              Real theta = T;
            algorithm
              caww := 0;
              for k in 1:4 loop
                caww := caww + dd[k] * theta ^ (1 - k);
              end for;
              caww := -exp(caww) * 1E-012;
            end Caww_dT;

            function Cwww_dT "Third molar virial coefficient of water"
              extends Modelica.Icons.Function;
              input Modelica.SIunits.Density d "Density";
              input Modelica.SIunits.Temperature T "Temperature";
              output Modelica.SIunits.MolarVolume cwww "Third virial coefficient";
            protected
              final constant Real[56] N = {0.12533547935523E-001, 0.78957634722828E+001, -0.87803203303561E+001, 0.31802509345418, -0.26145533859358, -0.78199751687981E-002, 0.88089493102134E-002, -0.66856572307965, 0.20433810950965, -0.66212605039687E-004, -0.19232721156002, -0.25709043003438, 0.16074868486251, -0.40092828925807E-001, 0.39343422603254E-006, -0.75941377088144E-005, 0.56250979351888E-003, -0.15608652257135E-004, 0.11537996422951E-008, 0.36582165144204E-006, -0.13251180074668E-011, -0.62639586912454E-009, -0.10793600908932, 0.17611491008752E-001, 0.22132295167546, -0.40247669763528, 0.58083399985759, 0.49969146990806E-002, -0.31358700712549E-001, -0.74315929710341, 0.4780732991548, 0.20527940895948E-001, -0.13636435110343, 0.14180634400617E-001, 0.83326504880713E-002, -0.29052336009585E-001, 0.38615085574206E-001, -0.20393486513704E-001, -0.16554050063734E-002, 0.19955571979541E-002, 0.15870308324157E-003, -0.1638856834253E-004, 0.43613615723811E-001, 0.34994005463765E-001, -0.76788197844621E-001, 0.22446277332006E-001, -0.62689710414685E-004, -0.55711118565645E-009, -0.19905718354408, 0.31777497330738, -0.11841182425981, -0.31306260323435E+002, 0.31546140237781E+002, -0.25213154341695E+004, -0.14874640856724, 0.31806110878444};
              final constant Integer[51] c = {0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 3, 3, 3, 3, 4, 6, 6, 6, 6};
              final constant Integer[54] dd = {1, 1, 1, 2, 2, 3, 4, 1, 1, 1, 2, 2, 3, 4, 4, 5, 7, 9, 10, 11, 13, 15, 1, 2, 2, 2, 3, 4, 4, 4, 5, 6, 6, 7, 9, 9, 9, 9, 9, 10, 10, 12, 3, 4, 4, 5, 14, 3, 6, 6, 6, 3, 3, 3};
              final constant Real[54] t = {-0.5, 0.875, 1.0, 0.5, 0.75, 0.375, 1.0, 4.0, 6.0, 12.0, 1.0, 5.0, 4.0, 2.0, 13.0, 9.0, 3.0, 4.0, 11.0, 4.0, 13.0, 1.0, 7.0, 1.0, 9.0, 10.0, 10.0, 3.0, 7.0, 10.0, 10.0, 6.0, 10.0, 10.0, 1.0, 2.0, 3.0, 4.0, 8.0, 6.0, 9.0, 8.0, 16.0, 22.0, 23.0, 23.0, 10.0, 50.0, 44.0, 46.0, 50.0, 0.0, 1.0, 4.0};
              final constant Integer[54] alpha = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 20, 20, 20};
              final constant Real[56] beta = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 150, 150, 250, 0.3, 0.3};
              final constant Real[54] gamma = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.21, 1.21, 1.25};
              final constant Integer[54] epsilon = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1};
              final constant Real[56] a = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 3.5, 3.5};
              final constant Real[56] b = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.85, 0.95};
              final constant Real[56] AA = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.32, 0.32};
              final constant Real[56] BB = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.2, 0.2};
              final constant Integer[56] CC = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 28, 32};
              final constant Integer[56] DD = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 700, 800};
              Real Delta55 = 0;
              Real Delta56 = 0;
              Real theta55 = 0;
              Real theta56 = 0;
              Real psi55 = 0;
              Real psi56 = 0;
              Real Delta55delta = 0;
              Real Delta56delta = 0;
              Real Deltab55delta = 0;
              Real Deltab56delta = 0;
              Real psi55delta = 0;
              Real psi56delta = 0;
              Real Delta55deltadelta = 0;
              Real Delta56deltadelta = 0;
              Real Deltab55deltadelta = 0;
              Real Deltab56deltadelta = 0;
              Real psi55deltadelta = 0;
              Real psi56deltadelta = 0;
              Real tau = Water95_Utilities.Constants.Tred / T;
            algorithm
              cwww := 0;
              theta55 := 1 - tau + AA[55];
              theta56 := 1 - tau + AA[56];
              psi55 := exp((-CC[55]) - DD[55] * (tau - 1) ^ 2);
              psi56 := exp((-CC[56]) - DD[56] * (tau - 1) ^ 2);
              Delta55 := theta55 ^ 2 + BB[55];
              Delta56 := theta56 ^ 2 + BB[56];
              Delta55delta := -(AA[55] * theta55 * 2 / beta[55] + 2 * BB[55] * a[55]);
              Delta56delta := -(AA[56] * theta56 * 2 / beta[56] + 2 * BB[56] * a[56]);
              Deltab55delta := b[55] * Delta55 ^ (b[55] - 1) * Delta55delta;
              Deltab56delta := b[56] * Delta56 ^ (b[56] - 1) * Delta56delta;
              psi55delta := 2 * CC[55] * psi55;
              psi56delta := 2 * CC[56] * psi56;
              Delta55deltadelta := (-Delta55delta) + AA[55] ^ 2 * 2 / beta[55] ^ 2 + AA[55] * theta55 * 4 / beta[55] * (1 / (2 * beta[55]) - 1) + 4 * BB[55] * a[55] * (a[55] - 1);
              Delta56deltadelta := (-Delta56delta) + AA[56] ^ 2 * 2 / beta[56] ^ 2 + AA[56] * theta56 * 4 / beta[56] * (1 / (2 * beta[56]) - 1) + 4 * BB[56] * a[56] * (a[56] - 1);
              Deltab55deltadelta := b[55] * (Delta55 ^ (b[55] - 1) * Delta55deltadelta + (b[55] - 1) * Delta55 ^ (b[55] - 2) * Delta55delta ^ 2);
              Deltab56deltadelta := b[56] * (Delta56 ^ (b[56] - 1) * Delta56deltadelta + (b[56] - 1) * Delta56 ^ (b[56] - 2) * Delta56delta ^ 2);
              psi55deltadelta := (2 * CC[55] - 1) * 2 * CC[55] * psi55;
              psi56deltadelta := (2 * CC[56] - 1) * 2 * CC[56] * psi56;
              cwww := 0;
              for k in 1:7 loop
                cwww := if dd[k] == 2 then cwww + 2 * N[k] * tau ^ t[k] else cwww;
              end for;
              for k in 8:51 loop
                cwww := if dd[k] == 2 then cwww + 2 * N[k] * tau ^ t[k] elseif dd[k] == 1 and c[k] == 1 then cwww - 2 * N[k] * tau ^ t[k] else cwww;
              end for;
              cwww := cwww + N[55] * (Delta55 ^ b[55] * 2 * psi55delta + 2 * Deltab55delta * psi55) + N[56] * (Delta56 ^ b[56] * 2 * psi56delta + 2 * Deltab56delta * psi56);
              cwww := Modelica.Media.Air.ReferenceMoistAir.Utilities.Water95_Utilities.Constants.MM ^ 2 / Modelica.Media.Air.ReferenceMoistAir.Utilities.Water95_Utilities.Constants.rhored ^ 2 * cwww * 1E-006;
            end Cwww_dT;
          end VirialCoefficients;

          package ReactionIndices "Parameters and equations for determining reaction variables (dissociation VDI 4670)"
            extends Modelica.Icons.BasesPackage;
            final constant Real[6] AA = {20413.2, 1075.5, 165.95, 1491.75, 3235.34, 4.5542};
            final constant Modelica.SIunits.Temperature[6] BB = {-33086.5, -30283.3, -19526.8, -27488.0, -30807.8, -10973.6};
            final constant Modelica.SIunits.MolarHeatCapacity[6] CC = {-19.5, -65.2, -18.7, -3.6, -21.8, -5.6};
            final constant Modelica.SIunits.MolarInternalEnergy[6] DD = {-1.15E+005, 3.03E+005, 5.72E+004, 3.93E+005, 1.5E+005, 1.62E+004};
            final constant Real[6] EE(each unit = "J.K/mol") = {9.483E+009, 7.277E+009, 3.136E+009, 5.826E+009, 7.659E+009, 9.94E+008};
            final constant Modelica.SIunits.AbsolutePressure p0 = 101325 "Reference pressure";

            function U2 "Reaction index for formation of H2"
              extends Modelica.Icons.Function;
              input Modelica.SIunits.AbsolutePressure p "Pressure";
              input Modelica.SIunits.Temperature T "Temperature";
              input Modelica.SIunits.MoleFraction[4] moleFraction "Mole fractions";
              output Real u "Reaction index for H2";
            algorithm
              u := AA[2] * moleFraction[1] / sqrt(moleFraction[3]) * (p / p0) ^ (-0.5) * Modelica.Math.exp(BB[2] / T);
              annotation(
                derivative = U2_der,
                Inline = false,
                LateInline = true);
            end U2;

            function U3 "Reaction index for formation of OH"
              extends Modelica.Icons.Function;
              input Modelica.SIunits.AbsolutePressure p "Pressure";
              input Modelica.SIunits.Temperature T "Temperature";
              input Modelica.SIunits.MoleFraction[4] moleFraction "Mole fractions";
              output Real u "Reaction index for OH";
            algorithm
              u := AA[3] * sqrt(moleFraction[1]) * sqrt(sqrt(moleFraction[3])) * (p / p0) ^ (-0.25) * Modelica.Math.exp(BB[3] / T);
              annotation(
                derivative = U3_der,
                Inline = false,
                LateInline = true);
            end U3;

            function U4 "Reaction index for formation of H"
              extends Modelica.Icons.Function;
              input Modelica.SIunits.AbsolutePressure p "Pressure";
              input Modelica.SIunits.Temperature T "Temperature";
              input Modelica.SIunits.MoleFraction[4] moleFraction "Mole fractions";
              output Real u "Reaction index for H";
            algorithm
              u := AA[4] * sqrt(U2(p, T, moleFraction)) * (p / p0) ^ (-0.5) * Modelica.Math.exp(BB[4] / T);
              annotation(
                derivative = U4_der,
                Inline = false,
                LateInline = true);
            end U4;

            function U5 "Reaction index for formation of O"
              extends Modelica.Icons.Function;
              input Modelica.SIunits.AbsolutePressure p "Pressure";
              input Modelica.SIunits.Temperature T "Temperature";
              input Modelica.SIunits.MoleFraction[4] moleFraction "Mole fractions";
              output Real u "Reaction index for O";
            algorithm
              u := AA[5] * sqrt(moleFraction[3]) * (p / p0) ^ (-0.5) * Modelica.Math.exp(BB[5] / T);
              annotation(
                derivative = U5_der,
                Inline = false,
                LateInline = true);
            end U5;

            function U6 "Reaction index for formation of NO"
              extends Modelica.Icons.Function;
              input Modelica.SIunits.AbsolutePressure p "Pressure";
              input Modelica.SIunits.Temperature T "Temperature";
              input Modelica.SIunits.MoleFraction[4] moleFraction "Mole fractions";
              output Real u "Reaction index for NO";
            algorithm
              u := AA[6] * sqrt(moleFraction[2] * moleFraction[3]) * Modelica.Math.exp(BB[6] / T);
              annotation(
                derivative = U6_der,
                Inline = false,
                LateInline = true);
            end U6;

            function V2 "Energy index for formation of H2"
              extends Modelica.Icons.Function;
              input Modelica.SIunits.Temperature T "Temperature";
              output Modelica.SIunits.MolarHeatCapacity v "Energy index for H2";
            algorithm
              v := CC[2] + DD[2] / T + EE[2] / T ^ 2;
              annotation(
                derivative = V2_der,
                Inline = false,
                LateInline = true);
            end V2;

            function V3 "Energy index for formation of OH"
              extends Modelica.Icons.Function;
              input Modelica.SIunits.Temperature T "Temperature";
              output Modelica.SIunits.MolarHeatCapacity v "Energy index for OH";
            algorithm
              v := CC[3] + DD[3] / T + EE[3] / T ^ 2;
              annotation(
                derivative = V3_der,
                Inline = false,
                LateInline = true);
            end V3;

            function V4 "Energy index for formation of H"
              extends Modelica.Icons.Function;
              input Modelica.SIunits.Temperature T "Temperature";
              output Modelica.SIunits.MolarHeatCapacity v "Energy index for H";
            algorithm
              v := CC[4] + DD[4] / T + EE[4] / T ^ 2;
              annotation(
                derivative = V4_der,
                Inline = false,
                LateInline = true);
            end V4;

            function V5 "Energy index for formation of O"
              extends Modelica.Icons.Function;
              input Modelica.SIunits.Temperature T "Temperature";
              output Modelica.SIunits.MolarHeatCapacity v "Energy index for O";
            algorithm
              v := CC[5] + DD[5] / T + EE[5] / T ^ 2;
              annotation(
                derivative = V5_der,
                Inline = false,
                LateInline = true);
            end V5;

            function V6 "Energy index for formation of NO"
              extends Modelica.Icons.Function;
              input Modelica.SIunits.Temperature T "Temperature";
              output Modelica.SIunits.MolarHeatCapacity v "Energy index for NO";
            algorithm
              v := CC[6] + DD[6] / T + EE[6] / T ^ 2;
              annotation(
                derivative = V6_der,
                Inline = false,
                LateInline = true);
            end V6;

            function U2_der "Derivative reaction index for formation of H2"
              extends Modelica.Icons.Function;
              input Modelica.SIunits.AbsolutePressure p "Pressure";
              input Modelica.SIunits.Temperature T "Temperature";
              input Modelica.SIunits.MoleFraction[4] moleFraction "Mole fractions";
              input Real p_der "Derivative of pressure";
              input Real T_der "Derivative of temperature";
              input Real[4] moleFraction_der "Derivative of mole fractions";
              output Real u_der "Derivative of reaction index for H2";
            protected
              Real o[4];
            algorithm
              o[1] := AA[2] * sqrt(moleFraction[3]) * (p / p0) ^ (-0.5) * Modelica.Math.exp(BB[2] / T);
              o[2] := -0.5 * AA[2] * moleFraction[1] / moleFraction[3] ^ 1.5 * (p / p0) ^ (-0.5) * Modelica.Math.exp(BB[2] / T);
              o[3] := -0.5 * AA[2] * moleFraction[1] / sqrt(moleFraction[3]) * sqrt(p0) * p ^ (-1.5) * Modelica.Math.exp(BB[2] / T);
              o[4] := -BB[2] * AA[2] * moleFraction[1] / sqrt(moleFraction[3]) * (p / p0) ^ (-0.5) * Modelica.Math.exp(BB[2] / T) / T ^ 2;
              u_der := o[1] * moleFraction_der[1] + o[2] * moleFraction_der[3] + o[3] * p_der + o[4] * T_der;
            end U2_der;

            function U3_der "Derivative of reaction index for formation of OH"
              extends Modelica.Icons.Function;
              input Modelica.SIunits.AbsolutePressure p "Pressure";
              input Modelica.SIunits.Temperature T "Temperature";
              input Modelica.SIunits.MoleFraction[4] moleFraction "Mole fractions";
              input Real p_der "Derivative of pressure";
              input Real T_der "Derivative of temperature";
              input Real[4] moleFraction_der "Derivative of mole fractions";
              output Real u_der "Derivative of reaction index for OH";
            protected
              Real o[4];
            algorithm
              o[1] := 0.5 * AA[3] / sqrt(moleFraction[1]) * sqrt(sqrt(moleFraction[3])) * (p / p0) ^ (-0.25) * Modelica.Math.exp(BB[3] / T);
              o[2] := 0.25 * AA[3] * sqrt(moleFraction[1]) / moleFraction[3] ^ 0.75 * (p / p0) ^ (-0.25) * Modelica.Math.exp(BB[3] / T);
              o[3] := -0.25 * AA[3] * sqrt(moleFraction[1]) * sqrt(sqrt(moleFraction[3])) * sqrt(sqrt(p0)) * p ^ (-1.25) * Modelica.Math.exp(BB[3] / T);
              o[4] := BB[3] * AA[3] * moleFraction[1] / sqrt(sqrt(moleFraction[3])) * (p / p0) ^ (-0.25) * Modelica.Math.exp(BB[3] / T) / T ^ 2;
              u_der := o[1] * moleFraction_der[1] + o[2] * moleFraction_der[3] + o[3] * p_der + o[4] * T_der;
            end U3_der;

            function U4_der "Derivative of reaction index for formation of H"
              extends Modelica.Icons.Function;
              input Modelica.SIunits.AbsolutePressure p "Pressure";
              input Modelica.SIunits.Temperature T "Temperature";
              input Modelica.SIunits.MoleFraction[4] moleFraction "Mole fractions";
              input Real p_der "Derivative of pressure";
              input Real T_der "Derivative of temperature";
              input Real[4] moleFraction_der "Derivative of mole fractions";
              output Real u_der "Derivative of reaction index for H";
            protected
              Real o[3];
            algorithm
              o[1] := 0.5 * AA[4] / sqrt(U2(p, T, moleFraction)) * U2_der(p, T, moleFraction, p_der, T_der, moleFraction_der) * (p / p0) ^ (-0.5) * Modelica.Math.exp(BB[4] / T);
              o[2] := -0.5 * AA[4] * sqrt(U2(p, T, moleFraction)) * sqrt(p0) * p ^ (-1.5) * Modelica.Math.exp(BB[4] / T);
              o[3] := BB[4] * AA[4] * sqrt(U2(p, T, moleFraction)) * (p / p0) ^ (-0.5) * Modelica.Math.exp(BB[4] / T) / T ^ 2;
              u_der := o[1] + o[2] * p_der + o[3] * T_der;
            end U4_der;

            function U5_der "Derivative of reaction index for formation of O"
              extends Modelica.Icons.Function;
              input Modelica.SIunits.AbsolutePressure p "Pressure";
              input Modelica.SIunits.Temperature T "Temperature";
              input Modelica.SIunits.MoleFraction[4] moleFraction "Mole fractions";
              input Real p_der "Derivative of pressure";
              input Real T_der "Derivative of temperature";
              input Real[4] moleFraction_der "Derivative of mole fractions";
              output Real u_der "Derivative of reaction index for O";
            protected
              Real o[3];
            algorithm
              o[1] := 0.5 * AA[5] / sqrt(moleFraction[3]) * (p / p0) ^ (-0.5) * Modelica.Math.exp(BB[5] / T);
              o[2] := -0.5 * AA[5] * sqrt(moleFraction[3]) * sqrt(p0) * p ^ (-1.5) * Modelica.Math.exp(BB[5] / T);
              o[3] := BB[5] * AA[5] * sqrt(moleFraction[3]) * (p / p0) ^ (-0.5) * Modelica.Math.exp(BB[5] / T) / T ^ 2;
              u_der := o[1] * moleFraction[3] + o[2] * p_der + o[3] * T_der;
            end U5_der;

            function U6_der "Derivative of reaction index for formation of NO"
              extends Modelica.Icons.Function;
              input Modelica.SIunits.AbsolutePressure p "Pressure";
              input Modelica.SIunits.Temperature T "Temperature";
              input Modelica.SIunits.MoleFraction[4] moleFraction "Mole fractions";
              input Real p_der "Derivative of pressure";
              input Real T_der "Derivative of temperature";
              input Real[4] moleFraction_der "Derivative of mole fractions";
              output Real u_der "Derivative of reaction index for NO";
            protected
              Real o[3];
            algorithm
              o[1] := 0.5 * AA[6] / sqrt(moleFraction[2] * moleFraction[3]) * moleFraction[3] * Modelica.Math.exp(BB[6] / T);
              o[2] := 0.5 * AA[6] / sqrt(moleFraction[2] * moleFraction[3]) * moleFraction[2] * Modelica.Math.exp(BB[6] / T);
              o[3] := BB[6] * AA[6] * sqrt(moleFraction[2] * moleFraction[3]) * Modelica.Math.exp(BB[6] / T) / T ^ 2;
              u_der := o[1] * moleFraction[2] + o[2] * moleFraction[3] + o[3] * T_der;
            end U6_der;

            function V2_der "Derivative of energy index for formation of H2"
              extends Modelica.Icons.Function;
              input Modelica.SIunits.Temperature T "Temperature";
              input Real T_der "Derivative of temperature";
              output Real v_der "Derivative energy index for H2";
            algorithm
              v_der := ((-DD[2] / T ^ 2) - 2 * EE[2] / T ^ 3) * T_der;
            end V2_der;

            function V3_der "Derivative energy index for formation of OH"
              extends Modelica.Icons.Function;
              input Modelica.SIunits.Temperature T "Temperature";
              input Real T_der "Derivative of temperature";
              output Real v_der "Derivative energy index for OH";
            algorithm
              v_der := ((-DD[3] / T ^ 2) - 2 * EE[3] / T ^ 3) * T_der;
            end V3_der;

            function V4_der "Derivative of energy index for formation of H"
              extends Modelica.Icons.Function;
              input Modelica.SIunits.Temperature T "Temperature";
              input Real T_der "Derivative of temperature";
              output Real v_der "Derivative energy index for H";
            algorithm
              v_der := ((-DD[4] / T ^ 2) - 2 * EE[4] / T ^ 3) * T_der;
            end V4_der;

            function V5_der "Derivative of energy index for formation of O"
              extends Modelica.Icons.Function;
              input Modelica.SIunits.Temperature T "Temperature";
              input Real T_der "Derivative of temperature";
              output Real v_der "Derivative energy index for O";
            algorithm
              v_der := ((-DD[5] / T ^ 2) - 2 * EE[5] / T ^ 3) * T_der;
            end V5_der;

            function V6_der "Derivative of energy index for formation of NO"
              extends Modelica.Icons.Function;
              input Modelica.SIunits.Temperature T "Temperature";
              input Real T_der "Derivative of temperature";
              output Real v_der "Derivative energy index for NO";
            algorithm
              v_der := ((-DD[6] / T ^ 2) - 2 * EE[6] / T ^ 3) * T_der;
            end V6_der;
          end ReactionIndices;

          package IF97_new "Workaround for IF97"
            extends Modelica.Icons.BasesPackage;
            final constant MolarMass molarMass = 0.018015257;

            function g2 "Gibbs function for region 2: g(p,T)"
              extends Modelica.Icons.Function;
              input Modelica.SIunits.Pressure p "Pressure";
              input Modelica.SIunits.Temperature T "Temperature (K)";
              output Modelica.Media.Common.GibbsDerivs g "Dimensionless Gibbs funcion and dervatives w.r.t. pi and tau";
            protected
              Real tau2 "Dimensionless temperature";
              Real[55] o "Vector of auxiliary variables";
            algorithm
              g.p := p;
              g.T := T;
              g.R := Modelica.Media.Water.IF97_Utilities.BaseIF97.data.RH2O;
              g.pi := p / Modelica.Media.Water.IF97_Utilities.BaseIF97.data.PSTAR2;
              g.tau := Modelica.Media.Water.IF97_Utilities.BaseIF97.data.TSTAR2 / T;
              tau2 := (-0.5) + g.tau;
              o[1] := tau2 * tau2;
              o[2] := o[1] * tau2;
              o[3] := -0.050325278727930 * o[2];
              o[4] := (-0.057581259083432) + o[3];
              o[5] := o[4] * tau2;
              o[6] := (-0.045996013696365) + o[5];
              o[7] := o[6] * tau2;
              o[8] := (-0.0178348622923580) + o[7];
              o[9] := o[8] * tau2;
              o[10] := o[1] * o[1];
              o[11] := o[10] * o[10];
              o[12] := o[11] * o[11];
              o[13] := o[10] * o[11] * o[12] * tau2;
              o[14] := o[1] * o[10] * tau2;
              o[15] := o[10] * o[11] * tau2;
              o[16] := o[1] * o[12] * tau2;
              o[17] := o[1] * o[11] * tau2;
              o[18] := o[1] * o[10] * o[11];
              o[19] := o[10] * o[11] * o[12];
              o[20] := o[1] * o[10];
              o[21] := g.pi * g.pi;
              o[22] := o[21] * o[21];
              o[23] := o[21] * o[22];
              o[24] := o[10] * o[12] * tau2;
              o[25] := o[12] * o[12];
              o[26] := o[11] * o[12] * o[25] * tau2;
              o[27] := o[10] * o[12];
              o[28] := o[1] * o[10] * o[11] * tau2;
              o[29] := o[10] * o[12] * o[25] * tau2;
              o[30] := o[1] * o[10] * o[25] * tau2;
              o[31] := o[1] * o[11] * o[12];
              o[32] := o[1] * o[12];
              o[33] := g.tau * g.tau;
              o[34] := o[33] * o[33];
              o[35] := -0.000053349095828174 * o[13];
              o[36] := (-0.087594591301146) + o[35];
              o[37] := o[2] * o[36];
              o[38] := (-0.0078785554486710) + o[37];
              o[39] := o[1] * o[38];
              o[40] := (-0.00037897975032630) + o[39];
              o[41] := o[40] * tau2;
              o[42] := (-0.000066065283340406) + o[41];
              o[43] := o[42] * tau2;
              o[44] := 5.7870447262208e-6 * tau2;
              o[45] := -0.301951672367580 * o[2];
              o[46] := (-0.172743777250296) + o[45];
              o[47] := o[46] * tau2;
              o[48] := (-0.091992027392730) + o[47];
              o[49] := o[48] * tau2;
              o[50] := o[1] * o[11];
              o[51] := o[10] * o[11];
              o[52] := o[11] * o[12] * o[25];
              o[53] := o[10] * o[12] * o[25];
              o[54] := o[1] * o[10] * o[25];
              o[55] := o[11] * o[12] * tau2;
              g.g := g.pi * ((-0.00177317424732130) + o[9] + g.pi * (tau2 * ((-0.000033032641670203) + ((-0.000189489875163150) + o[1] * ((-0.0039392777243355) + ((-0.043797295650573) - 0.0000266745479140870 * o[13]) * o[2])) * tau2) + g.pi * (2.04817376923090e-8 + (4.3870667284435e-7 + o[1] * ((-0.000032277677238570) + ((-0.00150339245421480) - 0.040668253562649 * o[13]) * o[2])) * tau2 + g.pi * (g.pi * (2.29220763376610e-6 * o[14] + g.pi * (((-1.67147664510610e-11) + o[15] * ((-0.00211714723213550) - 23.8957419341040 * o[16])) * o[2] + g.pi * ((-5.9059564324270e-18) + o[17] * ((-1.26218088991010e-6) - 0.038946842435739 * o[18]) + g.pi * (o[11] * (1.12562113604590e-11 - 8.2311340897998 * o[19]) + g.pi * (1.98097128020880e-8 * o[15] + g.pi * (o[10] * (1.04069652101740e-19 + ((-1.02347470959290e-13) - 1.00181793795110e-9 * o[10]) * o[20]) + o[23] * (o[13] * ((-8.0882908646985e-11) + 0.106930318794090 * o[24]) + o[21] * ((-0.33662250574171 * o[26]) + o[21] * (o[27] * (8.9185845355421e-25 + (3.06293168762320e-13 - 4.2002467698208e-6 * o[15]) * o[28]) + g.pi * ((-5.9056029685639e-26 * o[24]) + g.pi * (3.7826947613457e-6 * o[29] + g.pi * ((-1.27686089346810e-15 * o[30]) + o[31] * (7.3087610595061e-29 + o[18] * (5.5414715350778e-17 - 9.4369707241210e-7 * o[32])) * g.pi)))))))))))) + tau2 * ((-7.8847309559367e-10) + (1.27907178522850e-8 + 4.8225372718507e-7 * tau2) * tau2))))) + ((-0.0056087911830200) + g.tau * (0.071452738814550 + g.tau * ((-0.40710498239280) + g.tau * (1.42408197144400 + g.tau * ((-4.3839511194500) + g.tau * ((-9.6927686002170) + g.tau * (10.0866556801800 + ((-0.284086326077200) + 0.0212684635330700 * g.tau) * g.tau) + Modelica.Math.log(g.pi))))))) / (o[34] * g.tau);
              g.gpi := (1.00000000000000 + g.pi * ((-0.00177317424732130) + o[9] + g.pi * (o[43] + g.pi * (6.1445213076927e-8 + (1.31612001853305e-6 + o[1] * ((-0.000096833031715710) + ((-0.0045101773626444) - 0.122004760687947 * o[13]) * o[2])) * tau2 + g.pi * (g.pi * (0.0000114610381688305 * o[14] + g.pi * (((-1.00288598706366e-10) + o[15] * ((-0.0127028833928130) - 143.374451604624 * o[16])) * o[2] + g.pi * ((-4.1341695026989e-17) + o[17] * ((-8.8352662293707e-6) - 0.272627897050173 * o[18]) + g.pi * (o[11] * (9.0049690883672e-11 - 65.849072718398 * o[19]) + g.pi * (1.78287415218792e-7 * o[15] + g.pi * (o[10] * (1.04069652101740e-18 + ((-1.02347470959290e-12) - 1.00181793795110e-8 * o[10]) * o[20]) + o[23] * (o[13] * ((-1.29412653835176e-9) + 1.71088510070544 * o[24]) + o[21] * ((-6.0592051033508 * o[26]) + o[21] * (o[27] * (1.78371690710842e-23 + (6.1258633752464e-12 - 0.000084004935396416 * o[15]) * o[28]) + g.pi * ((-1.24017662339842e-24 * o[24]) + g.pi * (0.000083219284749605 * o[29] + g.pi * ((-2.93678005497663e-14 * o[30]) + o[31] * (1.75410265428146e-27 + o[18] * (1.32995316841867e-15 - 0.0000226487297378904 * o[32])) * g.pi)))))))))))) + tau2 * ((-3.15389238237468e-9) + (5.1162871409140e-8 + 1.92901490874028e-6 * tau2) * tau2)))))) / g.pi;
              g.gpipi := ((-1.00000000000000) + o[21] * (o[43] + g.pi * (1.22890426153854e-7 + (2.63224003706610e-6 + o[1] * ((-0.000193666063431420) + ((-0.0090203547252888) - 0.244009521375894 * o[13]) * o[2])) * tau2 + g.pi * (g.pi * (0.000045844152675322 * o[14] + g.pi * (((-5.0144299353183e-10) + o[15] * ((-0.063514416964065) - 716.87225802312 * o[16])) * o[2] + g.pi * ((-2.48050170161934e-16) + o[17] * ((-0.000053011597376224) - 1.63576738230104 * o[18]) + g.pi * (o[11] * (6.3034783618570e-10 - 460.94350902879 * o[19]) + g.pi * (1.42629932175034e-6 * o[15] + g.pi * (o[10] * (9.3662686891566e-18 + ((-9.2112723863361e-12) - 9.0163614415599e-8 * o[10]) * o[20]) + o[23] * (o[13] * ((-1.94118980752764e-8) + 25.6632765105816 * o[24]) + o[21] * ((-103.006486756963 * o[26]) + o[21] * (o[27] * (3.3890621235060e-22 + (1.16391404129682e-10 - 0.00159609377253190 * o[15]) * o[28]) + g.pi * ((-2.48035324679684e-23 * o[24]) + g.pi * (0.00174760497974171 * o[29] + g.pi * ((-6.4609161209486e-13 * o[30]) + o[31] * (4.0344361048474e-26 + o[18] * (3.05889228736295e-14 - 0.00052092078397148 * o[32])) * g.pi)))))))))))) + tau2 * ((-9.4616771471240e-9) + (1.53488614227420e-7 + o[44]) * tau2))))) / o[21];
              g.gtau := (0.0280439559151000 + g.tau * ((-0.285810955258200) + g.tau * (1.22131494717840 + g.tau * ((-2.84816394288800) + g.tau * (4.3839511194500 + o[33] * (10.0866556801800 + ((-0.56817265215440) + 0.063805390599210 * g.tau) * g.tau)))))) / (o[33] * o[34]) + g.pi * ((-0.0178348622923580) + o[49] + g.pi * ((-0.000033032641670203) + ((-0.00037897975032630) + o[1] * ((-0.0157571108973420) + ((-0.306581069554011) - 0.00096028372490713 * o[13]) * o[2])) * tau2 + g.pi * (4.3870667284435e-7 + o[1] * ((-0.000096833031715710) + ((-0.0090203547252888) - 1.42338887469272 * o[13]) * o[2]) + g.pi * ((-7.8847309559367e-10) + g.pi * (0.0000160454534363627 * o[20] + g.pi * (o[1] * ((-5.0144299353183e-11) + o[15] * ((-0.033874355714168) - 836.35096769364 * o[16])) + g.pi * (((-0.0000138839897890111) - 0.97367106089347 * o[18]) * o[50] + g.pi * (o[14] * (9.0049690883672e-11 - 296.320827232793 * o[19]) + g.pi * (2.57526266427144e-7 * o[51] + g.pi * (o[2] * (4.1627860840696e-19 + ((-1.02347470959290e-12) - 1.40254511313154e-8 * o[10]) * o[20]) + o[23] * (o[19] * ((-2.34560435076256e-9) + 5.3465159397045 * o[24]) + o[21] * ((-19.1874828272775 * o[52]) + o[21] * (o[16] * (1.78371690710842e-23 + (1.07202609066812e-11 - 0.000201611844951398 * o[15]) * o[28]) + g.pi * ((-1.24017662339842e-24 * o[27]) + g.pi * (0.000200482822351322 * o[53] + g.pi * ((-4.9797574845256e-14 * o[54]) + (1.90027787547159e-27 + o[18] * (2.21658861403112e-15 - 0.000054734430199902 * o[32])) * o[55] * g.pi)))))))))))) + (2.55814357045700e-8 + 1.44676118155521e-6 * tau2) * tau2))));
              g.gtautau := ((-0.168263735490600) + g.tau * (1.42905477629100 + g.tau * ((-4.8852597887136) + g.tau * (8.5444918286640 + g.tau * ((-8.7679022389000) + o[33] * ((-0.56817265215440) + 0.127610781198420 * g.tau) * g.tau))))) / (o[33] * o[34] * g.tau) + g.pi * ((-0.091992027392730) + ((-0.34548755450059) - 1.50975836183790 * o[2]) * tau2 + g.pi * ((-0.00037897975032630) + o[1] * ((-0.047271332692026) + ((-1.83948641732407) - 0.033609930371750 * o[13]) * o[2]) + g.pi * (((-0.000193666063431420) + ((-0.045101773626444) - 48.395221739552 * o[13]) * o[2]) * tau2 + g.pi * (2.55814357045700e-8 + 2.89352236311042e-6 * tau2 + g.pi * (0.000096272720618176 * o[10] * tau2 + g.pi * (((-1.00288598706366e-10) + o[15] * ((-0.50811533571252) - 28435.9329015838 * o[16])) * tau2 + g.pi * (o[11] * ((-0.000138839897890111) - 23.3681054614434 * o[18]) * tau2 + g.pi * ((6.3034783618570e-10 - 10371.2289531477 * o[19]) * o[20] + g.pi * (3.09031519712573e-6 * o[17] + g.pi * (o[1] * (1.24883582522088e-18 + ((-9.2112723863361e-12) - 1.82330864707100e-7 * o[10]) * o[20]) + o[23] * (o[1] * o[11] * o[12] * ((-6.5676921821352e-8) + 261.979281045521 * o[24]) * tau2 + o[21] * ((-1074.49903832754 * o[1] * o[10] * o[12] * o[25] * tau2) + o[21] * ((3.3890621235060e-22 + (3.6448887082716e-10 - 0.0094757567127157 * o[15]) * o[28]) * o[32] + g.pi * ((-2.48035324679684e-23 * o[16]) + g.pi * (0.0104251067622687 * o[1] * o[12] * o[25] * tau2 + g.pi * (o[11] * o[12] * (4.7506946886790e-26 + o[18] * (8.6446955947214e-14 - 0.00311986252139440 * o[32])) * g.pi - 1.89230784411972e-12 * o[10] * o[25] * tau2))))))))))))))));
              g.gtaupi := (-0.0178348622923580) + o[49] + g.pi * ((-0.000066065283340406) + ((-0.00075795950065260) + o[1] * ((-0.0315142217946840) + ((-0.61316213910802) - 0.00192056744981426 * o[13]) * o[2])) * tau2 + g.pi * (1.31612001853305e-6 + o[1] * ((-0.000290499095147130) + ((-0.0270610641758664) - 4.2701666240781 * o[13]) * o[2]) + g.pi * ((-3.15389238237468e-9) + g.pi * (0.000080227267181813 * o[20] + g.pi * (o[1] * ((-3.00865796119098e-10) + o[15] * ((-0.203246134285008) - 5018.1058061618 * o[16])) + g.pi * (((-0.000097187928523078) - 6.8156974262543 * o[18]) * o[50] + g.pi * (o[14] * (7.2039752706938e-10 - 2370.56661786234 * o[19]) + g.pi * (2.31773639784430e-6 * o[51] + g.pi * (o[2] * (4.1627860840696e-18 + ((-1.02347470959290e-11) - 1.40254511313154e-7 * o[10]) * o[20]) + o[23] * (o[19] * ((-3.7529669612201e-8) + 85.544255035272 * o[24]) + o[21] * ((-345.37469089099 * o[52]) + o[21] * (o[16] * (3.5674338142168e-22 + (2.14405218133624e-10 - 0.0040322368990280 * o[15]) * o[28]) + g.pi * ((-2.60437090913668e-23 * o[27]) + g.pi * (0.0044106220917291 * o[53] + g.pi * ((-1.14534422144089e-12 * o[54]) + (4.5606669011318e-26 + o[18] * (5.3198126736747e-14 - 0.00131362632479764 * o[32])) * o[55] * g.pi)))))))))))) + (1.02325742818280e-7 + o[44]) * tau2)));
            end g2;

            function h_pT "Specific enthalpy as function or pressure and temperature"
              extends Modelica.Icons.Function;
              input Modelica.SIunits.Pressure p "Pressure";
              input Modelica.SIunits.Temperature T "Temperature";
              input Integer region = 0 "If 0, region is unknown, otherwise known and this input";
              output Modelica.SIunits.SpecificEnthalpy h "Specific enthalpy";
            protected
              Modelica.Media.Common.GibbsDerivs g;
            algorithm
              g := Modelica.Media.Air.ReferenceMoistAir.Utilities.IF97_new.g2(p, T);
              h := g.R * T * g.tau * g.gtau;
              annotation(
                derivative(noDerivative = region) = h_pT_der,
                Inline = false,
                LateInline = true);
            end h_pT;

            function s_pT "Temperature as function of pressure and temperature"
              extends Modelica.Icons.Function;
              input Modelica.SIunits.Pressure p "Pressure";
              input Modelica.SIunits.Temperature T "Temperature";
              input Integer region = 0 "If 0, region is unknown, otherwise known and this input";
              output Modelica.SIunits.SpecificEntropy s "Specific entropy";
            protected
              Modelica.Media.Common.GibbsDerivs g;
            algorithm
              g := Modelica.Media.Air.ReferenceMoistAir.Utilities.IF97_new.g2(p, T);
              s := g.R * (g.tau * g.gtau - g.g);
            end s_pT;

            function cp_pT "Specific heat capacity at constant pressure as function of pressure and temperature"
              extends Modelica.Icons.Function;
              input Modelica.SIunits.Pressure p "Pressure";
              input Modelica.SIunits.Temperature T "Temperature";
              input Integer region = 0 "If 0, region is unknown, otherwise known and this input";
              output Modelica.SIunits.SpecificHeatCapacity cp "Specific heat capacity";
            protected
              Modelica.Media.Common.GibbsDerivs g;
            algorithm
              g := Modelica.Media.Air.ReferenceMoistAir.Utilities.IF97_new.g2(p, T);
              cp := -g.R * g.tau * g.tau * g.gtautau;
            end cp_pT;

            function cv_pT "Specific heat capacity at constant volume as function of pressure and temperature"
              extends Modelica.Icons.Function;
              input Modelica.SIunits.Pressure p "Pressure";
              input Modelica.SIunits.Temperature T "Temperature";
              input Integer region = 0 "If 0, region is unknown, otherwise known and this input";
              output Modelica.SIunits.SpecificHeatCapacity cv "Specific heat capacity";
            protected
              Modelica.Media.Common.GibbsDerivs g;
            algorithm
              g := Modelica.Media.Air.ReferenceMoistAir.Utilities.IF97_new.g2(p, T);
              cv := g.R * ((-g.tau * g.tau * g.gtautau) + (g.gpi - g.tau * g.gtaupi) * (g.gpi - g.tau * g.gtaupi) / g.gpipi);
            end cv_pT;

            function rho_pT "Density as function or pressure and temperature"
              extends Modelica.Icons.Function;
              input Modelica.SIunits.Pressure p "Pressure";
              input Modelica.SIunits.Temperature T "Temperature";
              output Modelica.SIunits.Density rho "Density";
            protected
              Modelica.Media.Common.GibbsDerivs g;
            algorithm
              g := Modelica.Media.Air.ReferenceMoistAir.Utilities.IF97_new.g2(p, T);
              rho := p / (g.R * T * g.pi * g.gpi);
              annotation(
                derivative = rho_pT_der,
                Inline = false,
                LateInline = true);
            end rho_pT;

            function rho_pT_der "Derivative function of rho_pT"
              extends Modelica.Icons.Function;
              input SI.Pressure p "Pressure";
              input SI.Temperature T "Temperature";
              input Real p_der "Derivative of pressure";
              input Real T_der "Derivative of temperature";
              output Real rho_der "Derivative of density";
            protected
              Modelica.Media.Common.GibbsDerivs g;
              SI.Density d;
              Real vp;
              Real vt;
            algorithm
              g := Modelica.Media.Air.ReferenceMoistAir.Utilities.IF97_new.g2(p, T);
              vt := g.R / p * (g.pi * g.gpi - g.tau * g.pi * g.gtaupi);
              vp := g.R * T / (p * p) * g.pi * g.pi * g.gpipi;
              d := p / (g.R * T * g.pi * g.gpi);
              rho_der := (-d ^ 2 * vp) * p_der + (-d ^ 2 * vt) * T_der;
            end rho_pT_der;

            function visc_dT "Dynamic viscosity eta(d,T), industrial formulation"
              extends Modelica.Icons.Function;
              input SI.Density d "Density";
              input SI.Temperature T "Temperature (K)";
              output SI.DynamicViscosity eta "Dynamic viscosity";
            protected
              constant Real n0 = 1.0 "Viscosity coefficient";
              constant Real n1 = 0.978197 "Viscosity coefficient";
              constant Real n2 = 0.579829 "Viscosity coefficient";
              constant Real n3 = -0.202354 "Viscosity coefficient";
              constant Real[42] nn = array(0.5132047, 0.3205656, 0.0, 0.0, -0.7782567, 0.1885447, 0.2151778, 0.7317883, 1.241044, 1.476783, 0.0, 0.0, -0.2818107, -1.070786, -1.263184, 0.0, 0.0, 0.0, 0.1778064, 0.460504, 0.2340379, -0.4924179, 0.0, 0.0, -0.0417661, 0.0, 0.0, 0.1600435, 0.0, 0.0, 0.0, -0.01578386, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -0.003629481, 0.0, 0.0) "Viscosity coefficients";
              constant SI.Density rhostar = 317.763 "Scaling density";
              constant SI.DynamicViscosity etastar = 55.071e-6 "Scaling viscosity";
              constant SI.Temperature tstar = 647.226 "Scaling temperature";
              Integer i "Auxiliary variable";
              Integer j "Auxiliary variable";
              Real delta "Dimensionless density";
              Real deltam1 "Dimensionless density";
              Real tau "Dimensionless temperature";
              Real taum1 "Dimensionless temperature";
              Real Psi0 "Auxiliary variable";
              Real Psi1 "Auxiliary variable";
              Real tfun "Auxiliary variable";
              Real rhofun "Auxiliary variable";
            algorithm
              delta := d / rhostar;
              deltam1 := delta - 1.0;
              tau := tstar / T;
              taum1 := tau - 1.0;
              Psi0 := 1 / (n0 + (n1 + (n2 + n3 * tau) * tau) * tau) / tau ^ 0.5;
              Psi1 := 0.0;
              tfun := 1.0;
              for i in 1:6 loop
                if i <> 1 then
                  tfun := tfun * taum1;
                end if;
                rhofun := 1.;
                for j in 0:6 loop
                  if j <> 0 then
                    rhofun := rhofun * deltam1;
                  end if;
                  Psi1 := Psi1 + nn[i + j * 6] * tfun * rhofun;
                end for;
              end for;
              eta := etastar * Psi0 * Modelica.Math.exp(delta * Psi1);
            end visc_dT;

            function cond_dT "Thermal conductivity"
              extends Modelica.Icons.Function;
              input SI.Density d "Density";
              input SI.Temperature T "Temperature (K)";
              output SI.ThermalConductivity lambda "Thermal conductivity";
            protected
              Integer region(min = 1, max = 5) "IF97 region, valid values:1,2,3, and 5";
              constant Real n0 = 1.0 "Conductivity coefficient";
              constant Real n1 = 6.978267 "Conductivity coefficient";
              constant Real n2 = 2.599096 "Conductivity coefficient";
              constant Real n3 = -0.998254 "Conductivity coefficient";
              constant Real[30] nn = array(1.3293046, 1.7018363, 5.2246158, 8.7127675, -1.8525999, -0.40452437, -2.2156845, -10.124111, -9.5000611, 0.9340469, 0.2440949, 1.6511057, 4.9874687, 4.3786606, 0.0, 0.018660751, -0.76736002, -0.27297694, -0.91783782, 0.0, -0.12961068, 0.37283344, -0.43083393, 0.0, 0.0, 0.044809953, -0.1120316, 0.13333849, 0.0, 0.0) "Conductivity coefficient";
              constant SI.ThermalConductivity lamstar = 0.4945 "Scaling conductivity";
              constant SI.Density rhostar = 317.763 "Scaling density";
              constant SI.Temperature tstar = 647.226 "Scaling temperature";
              constant SI.Pressure pstar = 22.115e6 "Scaling pressure";
              constant SI.DynamicViscosity etastar = 55.071e-6 "Scaling viscosity";
              Integer i "Auxiliary variable";
              Integer j "Auxiliary variable";
              Real delta "Dimensionless density";
              Real tau "Dimensionless temperature";
              Real deltam1 "Dimensionless density";
              Real taum1 "Dimensionless temperature";
              Real Lam0 "Part of thermal conductivity";
              Real Lam1 "Part of thermal conductivity";
              Real Lam2 "Part of thermal conductivity";
              Real tfun "Auxiliary variable";
              Real rhofun "Auxiliary variable";
              Real dpitau "Auxiliary variable";
              Real ddelpi "Auxiliary variable";
              Real d2 "Auxiliary variable";
              Modelica.Media.Common.GibbsDerivs g "Dimensionless Gibbs funcion and dervatives w.r.t. pi and tau";
              Modelica.Media.Common.HelmholtzDerivs f "Dimensionless Helmholtz function and dervatives w.r.t. delta and tau";
              Real Tc = T - 273.15 "Celsius temperature for region check";
              Real Chi "Symmetrized compressibility";
              // slightly different variables for industrial use
              constant SI.Density rhostar2 = 317.7 "Reference density";
              constant SI.Temperature Tstar2 = 647.25 "Reference temperature";
              constant SI.ThermalConductivity lambdastar = 1 "Reference thermal conductivity";
              parameter Real TREL = T / Tstar2 "Relative temperature";
              parameter Real rhoREL = d / rhostar2 "Relative density";
              Real lambdaREL "Relative thermal conductivity";
              Real deltaTREL "Relative temperature increment";
              constant Real[:] C = {0.642857, -4.11717, -6.17937, 0.00308976, 0.0822994, 10.0932};
              constant Real[:] dpar = {0.0701309, 0.0118520, 0.00169937, -1.0200};
              constant Real[:] b = {-0.397070, 0.400302, 1.060000};
              constant Real[:] B = {-0.171587, 2.392190};
              constant Real[:] a = {0.0102811, 0.0299621, 0.0156146, -0.00422464};
              Real Q;
              Real S;
              Real lambdaREL2 "Function, part of the interpolating equation of the thermal conductivity";
              Real lambdaREL1 "Function, part of the interpolating equation of the thermal conductivity";
              Real lambdaREL0 "Function, part of the interpolating equation of the thermal conductivity";
            algorithm
              deltaTREL := abs(TREL - 1) + C[4];
              Q := 2 + C[5] / deltaTREL ^ (3 / 5);
              if TREL >= 1 then
                S := 1 / deltaTREL;
              else
                S := C[6] / deltaTREL ^ (3 / 5);
              end if;
              lambdaREL2 := (dpar[1] / TREL ^ 10 + dpar[2]) * rhoREL ^ (9 / 5) * Modelica.Math.exp(C[1] * (1 - rhoREL ^ (14 / 5))) + dpar[3] * S * rhoREL ^ Q * Modelica.Math.exp(Q / (1 + Q) * (1 - rhoREL ^ (1 + Q))) + dpar[4] * Modelica.Math.exp(C[2] * TREL ^ (3 / 2) + C[3] / rhoREL ^ 5);
              lambdaREL1 := b[1] + b[2] * rhoREL + b[3] * Modelica.Math.exp(B[1] * (rhoREL + B[2]) ^ 2);
              lambdaREL0 := TREL ^ (1 / 2) * sum(a[i] * TREL ^ (i - 1) for i in 1:4);
              lambdaREL := lambdaREL0 + lambdaREL1 + lambdaREL2;
              lambda := lambdaREL * lambdastar;
            end cond_dT;

            function h_pT_der "Derivative function of h_pT"
              extends Modelica.Icons.Function;
              input SI.Pressure p "Pressure";
              input SI.Temperature T "Temperature";
              input Integer region = 0 "If 0, region is unknown, otherwise known and this input";
              input Real p_der "Derivative of pressure";
              input Real T_der "Derivative of temperature";
              output Real h_der "Derivative of specific enthalpy";
            protected
              Modelica.Media.Common.GibbsDerivs g;
              SI.Density rho;
              Real vt;
            algorithm
//region 2
              g := Modelica.Media.Air.ReferenceMoistAir.Utilities.IF97_new.g2(p, T);
              vt := g.R / p * (g.pi * g.gpi - g.tau * g.pi * g.gtaupi);
              rho := max(p / (g.R * T * g.pi * g.gpi), 1e-9);
              h_der := (1 / rho - T * vt) * p_der - g.R * g.tau * g.tau * g.gtautau * T_der;
            end h_pT_der;
          end IF97_new;

          package Water95_Utilities "Utility functions from IAPWS95 required for air temperatures below 273.15 K"
            extends Modelica.Icons.BasesPackage;
            constant Common.FundamentalConstants Constants(R_bar = 8.314371, R = 461.51805, MM = 18.015268E-003, rhored = 322, Tred = 647.096, pred = 22064000, h_off = 0.0, s_off = 0.0);
            constant Modelica.Media.Interfaces.Types.TwoPhase.FluidConstants waterConstants(each chemicalFormula = "H2O", each structureFormula = "H2O", each casRegistryNumber = "7732-18-5", each iupacName = "oxidane", each molarMass = 0.018015268, each criticalTemperature = 647.096, each criticalPressure = 22064.0e3, each criticalMolarVolume = 1 / 322.0 * 0.018015268, each triplePointTemperature = 273.16, each triplePointPressure = 611.657, each normalBoilingPoint = 373.124, each meltingPoint = 273.15, each acentricFactor = 0.344, each dipoleMoment = 1.8, each hasCriticalData = true, each hasIdealGasHeatCapacity = false, each hasDipoleMoment = true, each hasFundamentalEquation = true, each hasLiquidHeatCapacity = true, each hasSolidHeatCapacity = false, each hasAccurateViscosityData = true, each hasAccurateConductivityData = true, each hasVapourPressureCurve = false, each hasAcentricFactor = true, each HCRIT0 = 0.0, each SCRIT0 = 0.0, each deltah = 0.0, each deltas = 0.0);

            function psat "Saturation pressure"
              extends Modelica.Icons.Function;
              input SI.Temperature T "Temperature";
              output SI.AbsolutePressure p "Pressure";
            protected
              Real theta_s;
              Real A;
              Real B;
              Real C;
              final constant Real[10] n = {0.11670521452767E+004, -0.72421316703206E+006, -0.17073846940092E+002, 0.1202082470247E+005, -0.32325550322333E+007, 0.1491510861353E+002, -0.48232657361591E+004, 0.40511340542057E+006, -0.23855557567849, 0.65017534844798E+003};
            algorithm
              theta_s := min(T, 553) + n[9] / (min(T, 553) - n[10]);
              A := theta_s ^ 2 + n[1] * theta_s + n[2];
              B := n[3] * theta_s ^ 2 + n[4] * theta_s + n[5];
              C := n[6] * theta_s ^ 2 + n[7] * theta_s + n[8];
              p := (2 * C / ((-B) + sqrt(B ^ 2 - 4 * A * C))) ^ 4 * 1E+006;
              annotation(
                derivative = psat_der,
                Inline = false,
                LateInline = true);
            end psat;

            function Tsat "Saturation temperature"
              extends Modelica.Icons.Function;
              input SI.AbsolutePressure p "Pressure";
              output SI.Temperature T "Temperature";
            protected
              Real beta;
              Real D;
              Real E;
              Real F;
              Real G;
              final constant Real[10] n = {0.11670521452767E+004, -0.72421316703206E+006, -0.17073846940092E+002, 0.1202082470247E+005, -0.32325550322333E+007, 0.1491510861353E+002, -0.48232657361591E+004, 0.40511340542057E+006, -0.23855557567849, 0.65017534844798E+003};
            algorithm
              beta := (p * 1E-006) ^ 0.25;
              E := beta ^ 2 + n[3] * beta + n[6];
              F := n[1] * beta ^ 2 + n[4] * beta + n[7];
              G := n[2] * beta ^ 2 + n[5] * beta + n[8];
              D := 2 * G / ((-F) - sqrt(F ^ 2 - 4 * E * G));
              T := (n[10] + D - sqrt((n[10] + D) ^ 2 - 4 * (n[9] + n[10] * D))) / 2;
              annotation(
                derivative = Tsat_der,
                Inline = false,
                LateInline = true);
            end Tsat;

            function psat_der "Saturation pressure"
              extends Modelica.Icons.Function;
              input SI.Temperature T "Temperature";
              input Real T_der "Derivative of temperature";
              output Real p_der "Derivative of pressure w.r.t. temperature";
            protected
              Real theta_s, theta_s_der;
              Real A, A_der;
              Real B, B_der;
              Real C, C_der, o_der[3];
              final constant Real[10] n = {0.11670521452767E+004, -0.72421316703206E+006, -0.17073846940092E+002, 0.1202082470247E+005, -0.32325550322333E+007, 0.1491510861353E+002, -0.48232657361591E+004, 0.40511340542057E+006, -0.23855557567849, 0.65017534844798E+003};
            algorithm
              theta_s := min(T, 553) + n[9] / (min(T, 553) - n[10]);
              theta_s_der := 1 - n[9] / (min(T, 553) - n[10]) ^ 2;
              A := theta_s ^ 2 + n[1] * theta_s + n[2];
              B := n[3] * theta_s ^ 2 + n[4] * theta_s + n[5];
              C := n[6] * theta_s ^ 2 + n[7] * theta_s + n[8];
              A_der := 2 * theta_s * theta_s_der + n[1] * theta_s_der;
              B_der := 2 * n[3] * theta_s * theta_s_der + n[4] * theta_s_der;
              C_der := 2 * n[6] * theta_s * theta_s_der + n[7] * theta_s_der;
              o_der[1] := 2 * B * B_der - 4 * (A * C_der + A_der * C);
              o_der[2] := (-B_der) + 0.5 / sqrt(B ^ 2 - 4 * A * C) * o_der[1];
              o_der[3] := (2 * C_der * ((-B) + sqrt(B ^ 2 - 4 * A * C)) - 2 * C * o_der[2]) / ((-B) + sqrt(B ^ 2 - 4 * A * C)) ^ 2;
              p_der := 4 * (2 * C / ((-B) + sqrt(B ^ 2 - 4 * A * C))) ^ 3 * o_der[3] * 1E+006 * T_der;
            end psat_der;

            function Tsat_der "Derivative of saturation temperature"
              extends Modelica.Icons.Function;
              input SI.AbsolutePressure p "Pressure";
              input Real p_der "Pressure derivative";
              output Real T_der "Temperature derivative";
            protected
              Real beta, beta_der;
              Real D, D_der;
              Real E, E_der;
              Real F, F_der;
              Real G, G_der;
              Real o_der[2];
              final constant Real[10] n = {0.11670521452767E+004, -0.72421316703206E+006, -0.17073846940092E+002, 0.1202082470247E+005, -0.32325550322333E+007, 0.1491510861353E+002, -0.48232657361591E+004, 0.40511340542057E+006, -0.23855557567849, 0.65017534844798E+003};
            algorithm
              beta := (p * 1E-006) ^ 0.25;
              beta_der := 0.25 / (p * 1E-006) ^ 0.75 * 1E-006;
              E := beta ^ 2 + n[3] * beta + n[6];
              E_der := 2 * beta * beta_der + n[3];
              F := n[1] * beta ^ 2 + n[4] * beta + n[7];
              F_der := 2 * n[1] * beta * beta_der + n[4];
              G := n[2] * beta ^ 2 + n[5] * beta + n[8];
              G_der := 2 * n[2] * beta * beta_der + n[5];
              D := 2 * G / ((-F) - sqrt(F ^ 2 - 4 * E * G));
              o_der[1] := 2 * F * F_der - 4 * (E * G_der + E_der * G);
              o_der[2] := (-F_der) - 0.5 / sqrt(F ^ 2 - 4 * E * G) * o_der[1];
              D_der := (2 * G_der * ((-F) - sqrt(F ^ 2 - 4 * E * G)) - 2 * G * o_der[2]) / ((-F) - sqrt(F ^ 2 - 4 * E * G)) ^ 2;
              T_der := (D_der - 0.5 / sqrt((n[10] + D) ^ 2 - 4 * (n[9] + n[10] * D)) * (2 * D * D_der - 2 * n[10] * D_der)) / 2 * p_der;
            end Tsat_der;
          end Water95_Utilities;

          package Ice09_Utilities "Utility functions from IAPWS09 required for ice in air"
            extends Modelica.Icons.BasesPackage;

            package Basic "Fundamental equation of state"
              extends Modelica.Icons.BasesPackage;

              record IceConstants
                extends Common.FundamentalConstants;
                Modelica.SIunits.AbsolutePressure p0;
              end IceConstants;
            protected
              record MyComplex "Complex number with function"
                Real re "Real part of complex number" annotation(
                  Dialog);
                Real im "Imaginary part of complex number" annotation(
                  Dialog);
              end MyComplex;

              package MyComplexF
                function toReal "Extract Real part from Complex number"
                  input MyComplex c "Complex number";
                  output Real result = c.re "Real number";
                algorithm

                  annotation(
                    Inline = true,
                    Documentation(info = "<html>
<p>This function returns the real part of a complex number.</p>
</html>"));
                end toReal;

                function '-' "Subtract two complex numbers"
                  input MyComplex c1 "Complex number 1";
                  input MyComplex c2 "Complex number 2";
                  output MyComplex c3 "= c1 - c2";
                algorithm
                  c3 := MyComplex(c1.re - c2.re, c1.im - c2.im);
                  annotation(
                    Inline = true,
                    Documentation(info = "<html>
<p>This function returns the difference of two given Complex numbers.</p>
</html>"));
                end '-';

                function '*' "Multiplication"
                  input MyComplex c1 "Complex number 1";
                  input MyComplex c2 "Complex number 2";
                  output MyComplex c3 "= c1*c2";
                algorithm
                  c3 := MyComplex(c1.re * c2.re - c1.im * c2.im, c1.re * c2.im + c1.im * c2.re);
                  annotation(
                    Inline = true,
                    Documentation(info = "<html>
<p>This function returns the product of two given Complex numbers.</p>
</html>"));
                end '*';

                function '+' "Add two complex numbers"
                  input MyComplex c1 "Complex number 1";
                  input MyComplex c2 "Complex number 2";
                  output MyComplex c3 "= c1 + c2";
                algorithm
                  c3 := MyComplex(c1.re + c2.re, c1.im + c2.im);
                  annotation(
                    Inline = true,
                    Documentation(info = "<html>
<p>This function returns the sum of two given Complex numbers.</p>
</html>"));
                end '+';

                function '/' "Divide two complex numbers"
                  input MyComplex c1 "Complex number 1";
                  input MyComplex c2 "Complex number 2";
                  output MyComplex c3 "= c1/c2";
                algorithm
                  c3 := MyComplex(((+c1.re * c2.re) + c1.im * c2.im) / (c2.re * c2.re + c2.im * c2.im), ((-c1.re * c2.im) + c1.im * c2.re) / (c2.re * c2.re + c2.im * c2.im));
                  annotation(
                    Inline = true,
                    Documentation(info = "<html>
<p>This function returns the quotient of two given Complex numbers.</p>
</html>"));
                end '/';

                function '^' "Complex power of complex number"
                  input MyComplex c1 "Complex number";
                  input MyComplex c2 "Complex exponent";
                  output MyComplex c3 "= c1^c2";
                protected
                  Real lnz = 0.5 * log(c1.re * c1.re + c1.im * c1.im);
                  Real phi = atan2(c1.im, c1.re);
                  Real re = lnz * c2.re - phi * c2.im;
                  Real im = lnz * c2.im + phi * c2.re;
                algorithm
                  c3 := MyComplex(exp(re) * cos(im), exp(re) * sin(im));
                  annotation(
                    Inline = true,
                    Documentation(info = "<html>
<p>This function returns the given Complex numbers c1 to the power of the Complex number c2.</p>
</html>"));
                end '^';

                function 'log' "Logarithm of complex number"
                  import Modelica.Math.atan3;
                  input MyComplex c1 "Complex number";
                  output MyComplex c2 "= log(c1)";
                protected
                  Real phi = atan3(c1.im, c1.re, 0);
                algorithm
                  c2 := MyComplex(log((c1.re ^ 2 + c1.im ^ 2) ^ 0.5), phi);
                  annotation(
                    Inline = true,
                    Documentation(info = "<html>
<p>This function returns the Complex natural logarithm of the Complex input.</p>
</html>"));
                end 'log';
              end MyComplexF;
            public
              constant IceConstants Constants(R_bar = 8.314472, R = 461.52364, MM = 18.015268E-003, rhored = 1.0, Tred = 273.16, pred = 611.657, p0 = 101325, h_off = 0.0, s_off = 0.0);

              function Gibbs "Gibbs equation of state"
                extends Modelica.Icons.Function;
                input SI.AbsolutePressure p "Absolute pressure";
                input SI.Temperature T "Temperature";
                output Common.GibbsDerivs2 g "Gibbs function and dervatives w.r.t. T and p";
              protected
                final constant Real[5] g_0 = {-0.632020233449497E+006, 0.655022213658955, -0.189369929326131E-007, 0.339746123271053E-014, -0.556464869058991E-021} "Coefficients of EOS";
                final constant Real s_0 = -0.332733756492168E+004 "Coefficient of EOS";
                final MyComplex[2] t = {MyComplex(0.368017112855051E-001, 0.510878114959572E-001), MyComplex(0.337315741065416, 0.335449415919309)} "Coefficients of EOS";
                final constant MyComplex r_1 = MyComplex(0.447050716285388E+002, 0.656876847463481E+002) "Coefficients of EOS";
                final MyComplex[3] r_2 = {MyComplex(-0.725974574329220E+002, -0.781008427112870E+002), MyComplex(-0.557107698030123E-004, 0.464578634580806E-004), MyComplex(0.234801409215913E-010, -0.285651142904972E-010)} "Coefficients of EOS";
                final Real pi0 = Constants.p0 / Constants.pred "Reduced pressure";
                //temporary variables needed for calculation
                Real g0 = 0.0 "Help variable";
                Real g0p = 0.0 "Help variable";
                Real g0pp = 0.0 "Help variable";
                MyComplex r2 "Help variable";
                MyComplex r2p "Help variable";
                MyComplex r2pp "Help variable";
                MyComplex o[12] "Help variable";
              algorithm
                g.p := p;
                g.T := T;
                g.R := Modelica.Media.Air.ReferenceMoistAir.Utilities.Ice09_Utilities.Basic.Constants.R;
//Reduced pressure
                g.pi := g.p / Modelica.Media.Air.ReferenceMoistAir.Utilities.Ice09_Utilities.Basic.Constants.pred;
//Reduced temperature
                g.theta := g.T / Modelica.Media.Air.ReferenceMoistAir.Utilities.Ice09_Utilities.Basic.Constants.Tred;
//calculate temporary values
//0^0 may result in an error, so we have to use a workaround
                g0 := g_0[1];
                for k in 2:5 loop
                  g0 := g0 + g_0[k] * (g.pi - pi0) ^ (k - 1);
                end for;
                r2 := r_2[1];
                for k in 2:3 loop
                  r2 := MyComplexF.'+'(r2, MyComplexF.'*'(r_2[k], MyComplex((g.pi - pi0) ^ (k - 1), 0)));
                end for;
//First derivative of g0 w.r.t. pi
                g0p := g_0[2] / Modelica.Media.Air.ReferenceMoistAir.Utilities.Ice09_Utilities.Basic.Constants.pred;
                for k in 3:5 loop
                  g0p := g0p + g_0[k] * (k - 1) / Modelica.Media.Air.ReferenceMoistAir.Utilities.Ice09_Utilities.Basic.Constants.pred * (g.pi - pi0) ^ (k - 2);
                end for;
//First derivative of r2 w.r.t. pi
                r2p := MyComplexF.'+'(MyComplexF.'/'(r_2[2], MyComplex(Modelica.Media.Air.ReferenceMoistAir.Utilities.Ice09_Utilities.Basic.Constants.pred, 0)), MyComplexF.'*'(MyComplexF.'/'(r_2[3], MyComplex(Modelica.Media.Air.ReferenceMoistAir.Utilities.Ice09_Utilities.Basic.Constants.pred / 2, 0)), MyComplex(g.pi - pi0, 0)));
//Second derivative of g0 w.r.t. pi
                g0pp := g_0[3] * 2 / Modelica.Media.Air.ReferenceMoistAir.Utilities.Ice09_Utilities.Basic.Constants.pred ^ 2;
                for k in 4:5 loop
                  g0pp := g0pp + g_0[k] * (k - 1) * (k - 2) / Modelica.Media.Air.ReferenceMoistAir.Utilities.Ice09_Utilities.Basic.Constants.pred ^ 2 * (g.pi - pi0) ^ (k - 3);
                end for;
//Second derivative of g0 w.r.t. pi
                r2pp := MyComplexF.'*'(r_2[3], MyComplex(2 / Modelica.Media.Air.ReferenceMoistAir.Utilities.Ice09_Utilities.Basic.Constants.pred ^ 2, 0));
                o[1] := MyComplexF.'*'(MyComplexF.'-'(t[1], MyComplex(g.theta, 0)), MyComplexF.'log'(MyComplexF.'-'(t[1], MyComplex(g.theta, 0))));
                o[2] := MyComplexF.'*'(MyComplexF.'+'(t[1], MyComplex(g.theta, 0)), MyComplexF.'log'(MyComplexF.'+'(t[1], MyComplex(g.theta, 0))));
                o[3] := MyComplexF.'*'(MyComplexF.'-'(t[2], MyComplex(g.theta, 0)), MyComplexF.'log'(MyComplexF.'-'(t[2], MyComplex(g.theta, 0))));
                o[4] := MyComplexF.'*'(MyComplexF.'+'(t[2], MyComplex(g.theta, 0)), MyComplexF.'log'(MyComplexF.'+'(t[2], MyComplex(g.theta, 0))));
                o[5] := MyComplexF.'*'(MyComplexF.'*'(MyComplex(2, 0), t[1]), MyComplexF.'log'(t[1]));
                o[6] := MyComplexF.'*'(MyComplexF.'*'(MyComplex(2, 0), t[2]), MyComplexF.'log'(t[2]));
                o[7] := MyComplexF.'/'(MyComplex(g.theta ^ 2, 0), t[1]);
                o[8] := MyComplexF.'/'(MyComplex(g.theta ^ 2, 0), t[2]);
                o[9] := MyComplexF.'-'(MyComplexF.'-'(MyComplexF.'+'(o[1], o[2]), o[5]), o[7]);
                o[10] := MyComplexF.'-'(MyComplexF.'-'(MyComplexF.'+'(o[3], o[4]), o[6]), o[8]);
//   //Gibbs equation
//   g.g := g0 - s_0*Modelica.Media.Air.ReferenceMoistAir.Utilities.Ice09_Utilities.Basic.Constants.Tred
//     *g.theta + Modelica.Media.Air.ReferenceMoistAir.Utilities.Ice09_Utilities.Basic.Constants.Tred
//     *GibbsComplex.fromReal(r_1*((t[1] - g.theta)*GibbsComplexMath.log(t[1] - g.theta)
//      + (t[1] + g.theta)*GibbsComplexMath.log(t[1] + g.theta) - 2*t[1]*
//     GibbsComplexMath.log(t[1]) - g.theta^2/t[1]) + r2*((t[2] - g.theta)*
//     GibbsComplexMath.log(t[2] - g.theta) + (t[2] + g.theta)*
//     GibbsComplexMath.log(t[2] + g.theta) - 2*t[2]*GibbsComplexMath.log(t[2]) -
//     g.theta^2/t[2]));
//   //Gibbs equation
                g.g := g0 - s_0 * Modelica.Media.Air.ReferenceMoistAir.Utilities.Ice09_Utilities.Basic.Constants.Tred * g.theta + Modelica.Media.Air.ReferenceMoistAir.Utilities.Ice09_Utilities.Basic.Constants.Tred * MyComplexF.toReal(MyComplexF.'+'(MyComplexF.'*'(r_1, o[9]), MyComplexF.'*'(r2, o[10])));
// //First derivative of g w.r.t. p
// g.gp := g0p + MoMoLib.Water.IAPWS09.Ice09_Utilities.Basic.Constants.Tred*
//   ComplexMath.real(r2p*((t[2] - g.theta)*ComplexMath.log(t[2] - g.theta) + (t[
//   2] + g.theta)*ComplexMath.log(t[2] + g.theta) - 2*t[2]*ComplexMath.log(t[2])
//    - g.theta^2/t[2]));
// //First derivative of g w.r.t. p
                g.gp := g0p + Modelica.Media.Air.ReferenceMoistAir.Utilities.Ice09_Utilities.Basic.Constants.Tred * MyComplexF.toReal(MyComplexF.'*'(r2p, o[10]));
//   //Second derivative of g w.r.t. p
//   g.gpp := g0pp + Modelica.Media.Air.ReferenceMoistAir.Utilities.Ice09_Utilities.Basic.Constants.Tred
//     *GibbsComplex.fromReal(r2pp*((t[2] - g.theta)*GibbsComplexMath.log(t[2] - g.theta)
//      + (t[2] + g.theta)*GibbsComplexMath.log(t[2] + g.theta) - 2*t[2]*
//     GibbsComplexMath.log(t[2]) - g.theta^2/t[2]));
// //Second derivative of g w.r.t. p
                g.gpp := g0pp + Modelica.Media.Air.ReferenceMoistAir.Utilities.Ice09_Utilities.Basic.Constants.Tred * MyComplexF.toReal(MyComplexF.'*'(r2pp, o[10]));
//   //First derivative of g w.r.t. T
//   g.gT := -s_0 + GibbsComplex.fromReal(r_1*(
//      + GibbsComplexMath.log(t[1] + g.theta) - 2*g.theta/t[1] -GibbsComplexMath.log(t[1] - g.theta)) + r2*(-
//     GibbsComplexMath.log(t[2] - g.theta) + GibbsComplexMath.log(t[2] + g.theta) -
//     2*g.theta/t[2]));
                o[11] := MyComplexF.'-'(MyComplexF.'-'(MyComplexF.'log'(MyComplexF.'+'(t[1], MyComplex(g.theta, 0))), MyComplexF.'/'(MyComplex(2 * g.theta, 0), t[1])), MyComplexF.'log'(MyComplexF.'-'(t[1], MyComplex(g.theta, 0))));
                o[12] := MyComplexF.'-'(MyComplexF.'-'(MyComplexF.'log'(MyComplexF.'+'(t[2], MyComplex(g.theta, 0))), MyComplexF.'log'(MyComplexF.'-'(t[2], MyComplex(g.theta, 0)))), MyComplexF.'/'(MyComplex(2 * g.theta, 0), t[2]));
//   //First derivative of g w.r.t. T
                g.gT := (-s_0) + MyComplexF.toReal(MyComplexF.'+'(MyComplexF.'*'(r_1, o[11]), MyComplexF.'*'(r2, o[12])));
//   //Second derivative of g w.r.t. T
//   g.gTT := 1/Modelica.Media.Air.ReferenceMoistAir.Utilities.Ice09_Utilities.Basic.Constants.Tred
//     *GibbsComplex.fromReal(r_1*(1/(t[1] - g.theta) + 1/(t[1] + g.theta) - 2/t[1])
//      + r2*(1/(t[2] - g.theta) + 1/(t[2] + g.theta) - 2/t[2]));
//   //Second derivative of g w.r.t. T
                g.gTT := 1 / Modelica.Media.Air.ReferenceMoistAir.Utilities.Ice09_Utilities.Basic.Constants.Tred * MyComplexF.toReal(MyComplexF.'+'(MyComplexF.'*'(r_1, MyComplexF.'-'(MyComplexF.'+'(MyComplexF.'/'(MyComplex(1, 0), MyComplexF.'-'(t[1], MyComplex(g.theta, 0))), MyComplexF.'/'(MyComplex(1, 0), MyComplexF.'+'(t[1], MyComplex(g.theta, 0)))), MyComplexF.'/'(MyComplex(2, 0), t[1]))), MyComplexF.'*'(r2, MyComplexF.'-'(MyComplexF.'+'(MyComplexF.'/'(MyComplex(1, 0), MyComplexF.'-'(t[2], MyComplex(g.theta, 0))), MyComplexF.'/'(MyComplex(1, 0), MyComplexF.'+'(t[2], MyComplex(g.theta, 0)))), MyComplexF.'/'(MyComplex(2, 0), t[2])))));
//  //Mixed derivative of g w.r.t. T and p
// g.gTp := ComplexMath.real(r2p*(-ComplexMath.log(t[2] - g.theta) +
//   ComplexMath.log(t[2] + g.theta) - 2*g.theta/t[2]));
//  //Mixed derivative of g w.r.t. T and p
                g.gTp := MyComplexF.toReal(MyComplexF.'*'(r2p, MyComplexF.'-'(MyComplexF.'log'(MyComplexF.'+'(t[2], MyComplex(g.theta, 0))), MyComplexF.'+'(MyComplexF.'/'(MyComplex(2 * g.theta, 0), t[2]), MyComplexF.'log'(MyComplexF.'-'(t[2], MyComplex(g.theta, 0)))))));
              end Gibbs;

              function psub "Sublimation pressure"
                extends Modelica.Icons.Function;
                input SI.Temperature T "Temperature";
                output SI.AbsolutePressure p_sub "Pressure";
              protected
                final constant Real[3] a = {-0.212144006E+002, 0.273203819E+002, -0.610598130E+001};
                final constant Real[3] b = {0.333333333E-002, 0.120666667E+001, 0.170333333E+001};
                Real theta = 0;
                Real sum = 0;
              algorithm
/*assert(T >= 50 and T < 273.16, "IAPWS-95 medium function psub: input temperature T = "
     + String(T) + " K\n" + "must be in the range 50 <= T < 273.16 K.");*/
                theta := T / Modelica.Media.Air.ReferenceMoistAir.Utilities.Ice09_Utilities.Basic.Constants.Tred;
                for k in 1:3 loop
                  sum := sum + a[k] * theta ^ b[k];
                end for;
                p_sub := exp(sum / theta) * Modelica.Media.Air.ReferenceMoistAir.Utilities.Ice09_Utilities.Basic.Constants.pred;
                annotation(
                  derivative = psub_der,
                  Inline = false,
                  LateInline = true);
              end psub;

              function Tsub "Sublimation temperature"
                extends Modelica.Icons.Function;
                input Modelica.SIunits.AbsolutePressure p "Pressure";
                output Modelica.SIunits.Temperature T_sub "Temperature";
              protected
                function Tsub_res
                  extends Modelica.Math.Nonlinear.Interfaces.partialScalarFunction;
                  input Modelica.SIunits.AbsolutePressure p "Pressure";
                algorithm
                  y := Modelica.Media.Air.ReferenceMoistAir.Utilities.Ice09_Utilities.Basic.psub(u) - p;
                end Tsub_res;
              algorithm
                T_sub := Modelica.Math.Nonlinear.solveOneNonlinearEquation(function Tsub_res(p = p), 143.15, 2000.0, 1e-9);
              end Tsub;

              function psub_der "Derivative of sublimation pressure"
                extends Modelica.Icons.Function;
                input SI.Temperature T "Temperature";
                input Real T_der "Derivative of temperature";
                output Real p_sub_der "Derivative of pressure";
              protected
                final constant Real[3] a = {-0.212144006E+002, 0.273203819E+002, -0.610598130E+001};
                final constant Real[3] b = {0.333333333E-002, 0.120666667E+001, 0.170333333E+001};
                Real theta;
                Real theta_der;
                Real sum = 0;
                Real sum_der = 0;
              algorithm
/*assert(T >= 50 and T < 273.16, "IAPWS-95 medium function psub: input temperature T = "
     + String(T) + " K\n" + "must be in the range 50 <= T < 273.16 K.");*/
                theta := T / Modelica.Media.Air.ReferenceMoistAir.Utilities.Ice09_Utilities.Basic.Constants.Tred;
                theta_der := 1 / Modelica.Media.Air.ReferenceMoistAir.Utilities.Ice09_Utilities.Basic.Constants.Tred;
                for k in 1:3 loop
                  sum := sum + a[k] * theta ^ b[k];
                  sum_der := sum_der + a[k] * b[k] * theta ^ (b[k] - 1) * theta_der;
                end for;
                p_sub_der := (sum_der * theta - sum * theta_der) / theta ^ 2 * exp(sum / theta) * Modelica.Media.Air.ReferenceMoistAir.Utilities.Ice09_Utilities.Basic.Constants.pred * T_der;
              end psub_der;
            end Basic;

            function ice09BaseProp_pT "Intermediate property record for water (p and T prefered states)"
              extends Modelica.Icons.Function;
              input Modelica.SIunits.Pressure p "Pressure";
              input Modelica.SIunits.Temperature T "Temperature";
              output Common.AuxiliaryProperties aux "Auxiliary record";
            protected
              Common.GibbsDerivs2 g "Gibbs funcion and dervatives w.r.t. p and T";
            algorithm
              aux.p := p;
              aux.T := T;
              aux.R := Modelica.Media.Air.ReferenceMoistAir.Utilities.Ice09_Utilities.Basic.Constants.R;
              g := Basic.Gibbs(aux.p, T);
              aux.rho := 1 / g.gp;
              aux.h := g.g - g.T * g.gT - Modelica.Media.Air.ReferenceMoistAir.Utilities.Ice09_Utilities.Basic.Constants.h_off;
              aux.s := (-g.gT) - Modelica.Media.Air.ReferenceMoistAir.Utilities.Ice09_Utilities.Basic.Constants.s_off;
              aux.cp := -g.T * g.gTT;
              aux.cv := aux.cp - g.T / g.gpp * g.gTp ^ 2;
              aux.vt := 1 / aux.rho * g.gTp / g.gp;
              aux.vp := 1 / aux.rho * g.gpp / g.gp;
              aux.pd := -1 / (aux.rho * aux.rho * aux.vp);
              aux.pt := -aux.vt / aux.vp;
            end ice09BaseProp_pT;

            function rho_props_pT "Density as function or pressure and temperature"
              extends Modelica.Icons.Function;
              input Modelica.SIunits.Pressure p "Pressure";
              input Modelica.SIunits.Temperature T "Temperature";
              input Common.AuxiliaryProperties aux "Auxiliary record";
              output Modelica.SIunits.Density rho "Density";
            algorithm
              rho := aux.rho;
              annotation(
                derivative(noDerivative = aux) = rho_pT_der,
                Inline = false,
                LateInline = true);
            end rho_props_pT;

            function rho_pT "Density as function or pressure and temperature"
              extends Modelica.Icons.Function;
              input Modelica.SIunits.Pressure p "Pressure";
              input Modelica.SIunits.Temperature T "Temperature";
              output Modelica.SIunits.Density rho "Density";
            algorithm
              rho := rho_props_pT(p, T, Modelica.Media.Air.ReferenceMoistAir.Utilities.Ice09_Utilities.ice09BaseProp_pT(p, T));
            end rho_pT;

            function rho_pT_der "Derivative function of rho_pT"
              extends Modelica.Icons.Function;
              input Modelica.SIunits.Pressure p "Pressure";
              input Modelica.SIunits.Temperature T "Temperature";
              input Common.AuxiliaryProperties aux "Auxiliary record";
              input Real p_der "Derivative of pressure";
              input Real T_der "Derivative of temperature";
              output Real rho_der "Derivative of density";
            algorithm
              rho_der := (-aux.rho * aux.rho * aux.vp) * p_der + (-aux.rho * aux.rho * aux.vt) * T_der;
            end rho_pT_der;

            function h_props_pT "Specific enthalpy as function or pressure and temperature"
              extends Modelica.Icons.Function;
              input Modelica.SIunits.Pressure p "Pressure";
              input Modelica.SIunits.Temperature T "Temperature";
              input Common.AuxiliaryProperties aux "Auxiliary record";
              output Modelica.SIunits.SpecificEnthalpy h "Specific enthalpy";
            algorithm
              h := aux.h;
              annotation(
                derivative(noDerivative = aux) = h_pT_der,
                Inline = false,
                LateInline = true);
            end h_props_pT;

            function h_pT "Specific enthalpy as function or pressure and temperature"
              extends Modelica.Icons.Function;
              input Modelica.SIunits.Pressure p "Pressure";
              input Modelica.SIunits.Temperature T "Temperature";
              output Modelica.SIunits.SpecificEnthalpy h "Specific enthalpy";
            algorithm
              h := h_props_pT(p, T, Modelica.Media.Air.ReferenceMoistAir.Utilities.Ice09_Utilities.ice09BaseProp_pT(p, T));
            end h_pT;

            function h_pT_der "Derivative function of h_pT"
              extends Modelica.Icons.Function;
              input Modelica.SIunits.Pressure p "Pressure";
              input Modelica.SIunits.Temperature T "Temperature";
              input Common.AuxiliaryProperties aux "Auxiliary record";
              input Real p_der "Derivative of pressure";
              input Real T_der "Derivative of temperature";
              output Real h_der "Derivative of specific enthalpy";
            algorithm
              h_der := (1 / aux.rho - aux.T * aux.vt) * p_der + aux.cp * T_der;
            end h_pT_der;

            function s_props_pT "Specific entropy as function of pressure and temperature"
              extends Modelica.Icons.Function;
              input Modelica.SIunits.Pressure p "Pressure";
              input Modelica.SIunits.Temperature T "Temperature";
              input Common.AuxiliaryProperties aux "Auxiliary record";
              output Modelica.SIunits.SpecificEntropy s "Specific entropy";
            algorithm
              s := aux.s;
              annotation(
                Inline = false,
                LateInline = true);
            end s_props_pT;

            function s_pT "Temperature as function of pressure and temperature"
              extends Modelica.Icons.Function;
              input Modelica.SIunits.Pressure p "Pressure";
              input Modelica.SIunits.Temperature T "Temperature";
              output Modelica.SIunits.SpecificEntropy s "Specific entropy";
            algorithm
              s := s_props_pT(p, T, Modelica.Media.Air.ReferenceMoistAir.Utilities.Ice09_Utilities.ice09BaseProp_pT(p, T));
            end s_pT;

            function kappa_props_pT "Isothermal compressibility factor as function of pressure and temperature"
              extends Modelica.Icons.Function;
              input Modelica.SIunits.Pressure p "Pressure";
              input Modelica.SIunits.Temperature T "Temperature";
              input Common.AuxiliaryProperties aux "Auxiliary record";
              output Modelica.SIunits.IsothermalCompressibility kappa "Isothermal compressibility factor";
            algorithm
              kappa := -aux.vp * aux.rho;
              annotation(
                Inline = false,
                LateInline = true);
            end kappa_props_pT;

            function kappa_pT "Isothermal compressibility factor as function of pressure and temperature"
              extends Modelica.Icons.Function;
              input Modelica.SIunits.Pressure p "Pressure";
              input Modelica.SIunits.Temperature T "Temperature";
              output Modelica.SIunits.IsothermalCompressibility kappa "Isothermal compressibility factor";
            algorithm
              kappa := kappa_props_pT(p, T, Modelica.Media.Air.ReferenceMoistAir.Utilities.Ice09_Utilities.ice09BaseProp_pT(p, T));
            end kappa_pT;
          end Ice09_Utilities;

          function beta_H "Henry's law constant"
            extends Modelica.Icons.Function;
            input Modelica.SIunits.AbsolutePressure p "Pressure";
            input Modelica.SIunits.Temperature T "Temperature";
            output Real beta_H(unit = "1/Pa") "Henry's law constant";
          protected
            Real[3] A = {-9.67578, -9.44833, -8.40954};
            Real[3] B = {4.72162, 4.43822, 4.29587};
            Real[3] C = {11.70585, 11.42005, 10.52779};
            Real[3] psi = {0.7812, 0.2095, 0.0093};
            Real[3] beta = fill(0, 3);
            Real Tr = T / 647.096;
            Real tau = 1 - Tr;
          algorithm
            if T < 273.15 or T > Modelica.Media.Air.ReferenceMoistAir.Utilities.Water95_Utilities.Tsat(p) then
              beta_H := 0;
            else
              for k in 1:3 loop
                beta[k] := Modelica.Media.Air.ReferenceMoistAir.Utilities.Water95_Utilities.psat(T) * exp(A[k] / Tr + B[k] * tau ^ 0.355 / Tr + C[k] * Tr ^ (-0.41) * exp(tau));
              end for;
              beta_H := 1 / 1.01325 * (psi[1] / beta[1] + psi[2] / beta[2] + psi[3] / beta[3]);
            end if;
          end beta_H;

          function f_pT "Enhancement factor as function of pressure and temperature"
            extends Modelica.Icons.Function;
            input Modelica.SIunits.AbsolutePressure p "Pressure";
            input Modelica.SIunits.Temperature T "Temperature";
            output Real f "Vapor-pressure enhancement factor";
          protected
            function f_res
              extends Modelica.Math.Nonlinear.Interfaces.partialScalarFunction;
              input Modelica.SIunits.AbsolutePressure p "Pressure";
              input Modelica.SIunits.Temperature T "Temperature";
            protected
              Real x = u;
              Real p_ws = 0;
              Real kappa_T = 0;
              Real v_ws = 0;
              Real beta_H = 0;
              Real psi_ws = 0;
              Real baa = 0;
              Real baw = 0;
              Real bww = 0;
              Real caaa = 0;
              Real caaw = 0;
              Real caww = 0;
              Real cwww = 0;
              final constant Real R_bar = 8.314472;
            algorithm
//p_ws is the saturation pressure of water
              p_ws := if T >= 273.16 then Modelica.Media.Air.ReferenceMoistAir.Utilities.Water95_Utilities.psat(T) else Modelica.Media.Air.ReferenceMoistAir.Utilities.Ice09_Utilities.Basic.psub(T);
//kappa_T is the isothermal compressibility
              if p < p_ws then
                kappa_T := 0;
              else
                kappa_T := if T >= 273.16 then Modelica.Media.Water.IF97_Utilities.kappa_pT(p, T) else Modelica.Media.Air.ReferenceMoistAir.Utilities.Ice09_Utilities.kappa_pT(p, T);
              end if;
//v_ws is the molar volume of saturated water
              v_ws := if T >= 273.16 then Modelica.Media.Air.ReferenceMoistAir.Utilities.IF97_new.molarMass / Modelica.Media.Water.IF97_Utilities.rho_pT(p, T) else Modelica.Media.Air.ReferenceMoistAir.Utilities.Ice09_Utilities.Basic.Constants.MM / Modelica.Media.Air.ReferenceMoistAir.Utilities.Ice09_Utilities.rho_pT(p, T);
//beta_H is Henry's law constant
              beta_H := Modelica.Media.Air.ReferenceMoistAir.Utilities.beta_H(p, T);
//calculating the virial coefficients baa, baw, bww, caaa, caaw, caww, cwww
              baa := Modelica.Media.Air.ReferenceMoistAir.Utilities.VirialCoefficients.Baa_dT(0, T);
              baw := Modelica.Media.Air.ReferenceMoistAir.Utilities.VirialCoefficients.Baw_dT(0, T);
              bww := Modelica.Media.Air.ReferenceMoistAir.Utilities.VirialCoefficients.Bww_dT(0, T);
              caaa := Modelica.Media.Air.ReferenceMoistAir.Utilities.VirialCoefficients.Caaa_dT(0, T);
              caaw := Modelica.Media.Air.ReferenceMoistAir.Utilities.VirialCoefficients.Caaw_dT(0, T);
              caww := Modelica.Media.Air.ReferenceMoistAir.Utilities.VirialCoefficients.Caww_dT(0, T);
              cwww := Modelica.Media.Air.ReferenceMoistAir.Utilities.VirialCoefficients.Cwww_dT(0, T);
              y := ((1 + kappa_T * p_ws) * (p - p_ws) - kappa_T * (p ^ 2 - p_ws ^ 2) / 2) / (R_bar * T) * v_ws + log(1 - beta_H * (1 - x * p_ws / p) * p) + (1 - x * p_ws / p) ^ 2 * p / (R_bar * T) * baa - 2 * (1 - x * p_ws / p) ^ 2 * p / (R_bar * T) * baw - (p - p_ws - (1 - x * p_ws / p) ^ 2 * p) / (R_bar * T) * bww + (1 - x * p_ws / p) ^ 3 * p ^ 2 / (R_bar * T) ^ 2 * caaa + 3 * (1 - x * p_ws / p) ^ 2 * (1 - 2 * (1 - x * p_ws / p)) * p ^ 2 / (2 * (R_bar * T) ^ 2) * caaw - 3 * (1 - x * p_ws / p) ^ 2 * x * p_ws / p * p ^ 2 / (R_bar * T) ^ 2 * caww - ((3 - 2 * x * p_ws / p) * (x * p_ws / p) ^ 2 * p ^ 2 - p_ws ^ 2) / (2 * (R_bar * T) ^ 2) * cwww - (1 - x * p_ws / p) ^ 2 * ((-2) + 3 * x * p_ws / p) * x * p_ws / p * p ^ 2 / (R_bar * T) ^ 2 * baa * bww - 2 * (1 - x * p_ws / p) ^ 3 * ((-1) + 3 * x * p_ws / p) * p ^ 2 / (R_bar * T) ^ 2 * baa * baw + 6 * (1 - x * p_ws / p) ^ 2 * (x * p_ws / p) ^ 2 * p ^ 2 / (R_bar * T) ^ 2 * bww * baw - 3 * (1 - x * p_ws / p) ^ 4 * p ^ 2 / (2 * (R_bar * T) ^ 2) * baa ^ 2 - 2 * (1 - x * p_ws / p) ^ 2 * x * p_ws / p * ((-2) + 3 * x * p_ws / p) * p ^ 2 / (R_bar * T) ^ 2 * baw ^ 2 - (p_ws ^ 2 - (4 - 3 * x * p_ws / p) * (x * p_ws / p) ^ 3 * p ^ 2) / (2 * (R_bar * T) ^ 2) * bww ^ 2 - log(x);
            end f_res;

            Real xmax;
          algorithm
            if useEnhancementFactor == false or T >= Modelica.Media.Air.ReferenceMoistAir.Utilities.Water95_Utilities.Tsat(p) then
              f := 1;
            else
              xmax := if T < 273.16 then 120.0 else 8.0;
              f := Modelica.Math.Nonlinear.solveOneNonlinearEquation(function f_res(p = p, T = T), 0.99999, xmax, 1e-9);
            end if;
          end f_pT;

          function rho_pTX "Return density as a function of pressure p, temperature T and composition X"
            extends Modelica.Icons.Function;
            input Modelica.SIunits.AbsolutePressure p "Pressure";
            input Modelica.SIunits.Temperature T "Temperature";
            input Modelica.SIunits.MassFraction X[:] = Modelica.Media.Air.ReferenceMoistAir.reference_X "Mass fractions";
            output Modelica.SIunits.Density d "Density";
          protected
            Real pd;
            Real pl;
            Real xw;
            Real xws;
          algorithm
            if X[1] == 0 then
              d := Modelica.Media.Air.ReferenceAir.Air_Utilities.rho_pT(p, T);
            else
              xw := X[1] / (1 - X[1]);
              xws := xws_pT(p, T);
              pd := pd_pTX(p, T, X);
              pl := p - pd;
              if xw <= xws or xws == (-1) then
                if T < 273.16 then
                  d := Modelica.Media.Air.ReferenceAir.Air_Utilities.rho_pT(pl, T) + pd / (.Modelica.Media.Air.ReferenceMoistAir.steam.R * T);
                else
                  d := Modelica.Media.Air.ReferenceAir.Air_Utilities.rho_pT(pl, T) + IF97_new.rho_pT(pd, T);
                end if;
              else
                if T < 273.16 then
                  d := (1 + xw) / ((1 + xws) / (Modelica.Media.Air.ReferenceAir.Air_Utilities.rho_pT(pl, T) + pd / (.Modelica.Media.Air.ReferenceMoistAir.steam.R * T)) + (xw - xws) / Modelica.Media.Air.ReferenceMoistAir.Utilities.Ice09_Utilities.rho_pT(p, T));
                else
                  d := (1 + xw) / ((1 + xws) / (Modelica.Media.Air.ReferenceAir.Air_Utilities.rho_pT(pl, T) + IF97_new.rho_pT(pd, T)) + (xw - xws) / Modelica.Media.Water.IF97_Utilities.rho_pT(p, T));
                end if;
              end if;
            end if;
            annotation(
              derivative = rho_pTX_der,
              Inline = false,
              LateInline = true);
          end rho_pTX;

          function pds_pT "Saturation partial pressure of steam"
            extends Modelica.Icons.Function;
            input Modelica.SIunits.AbsolutePressure p "Pressure";
            input Modelica.SIunits.Temperature T "Temperature";
            output Modelica.SIunits.AbsolutePressure pds "Pressure";
          protected
            Real Tlim;
          algorithm
            if T >= 273.16 then
              pds := Modelica.Media.Air.ReferenceMoistAir.Utilities.Water95_Utilities.psat(T);
              Tlim := Modelica.Media.Air.ReferenceMoistAir.Utilities.Water95_Utilities.Tsat(p);
            else
              pds := Modelica.Media.Air.ReferenceMoistAir.Utilities.Ice09_Utilities.Basic.psub(T);
              Tlim := Modelica.Media.Air.ReferenceMoistAir.Utilities.Ice09_Utilities.Basic.Tsub(p);
            end if;
            if T <= Tlim then
              pds := Modelica.Media.Air.ReferenceMoistAir.Utilities.f_pT(p, T) * pds;
            else
              pds := -1;
            end if;
            annotation(
              derivative = pds_pT_der,
              Inline = false,
              LateInline = true);
          end pds_pT;

          function pd_pTX "partial pressure of steam"
            extends Modelica.Icons.Function;
            input Modelica.SIunits.AbsolutePressure p "Pressure";
            input Modelica.SIunits.Temperature T "Temperature";
            input Modelica.SIunits.MassFraction X[:] = Modelica.Media.Air.ReferenceMoistAir.reference_X "Mass fractions";
            output Modelica.SIunits.AbsolutePressure pd "partial pressure";
          protected
            Real xw;
            Real xws;
            Real pds;
          algorithm
            if X[1] == 0 then
              pd := 0;
            else
              xw := X[1] / (1 - X[1]);
              pds := Modelica.Media.Air.ReferenceMoistAir.Utilities.pds_pT(p, T);
              pd := xw / (Modelica.Media.Air.ReferenceMoistAir.k_mair + xw) * p;
              xws := Modelica.Media.Air.ReferenceMoistAir.Utilities.xws_pT(p, T);
              pd := if xw <= xws or xws == (-1) then pd else pds;
            end if;
            annotation(
              derivative = pd_pTX_der,
              Inline = false,
              LateInline = true);
          end pd_pTX;

          function xws_pT "Humidity ration (absolute) of saturated humid air"
            extends Modelica.Icons.Function;
            input Modelica.SIunits.AbsolutePressure p "Pressure";
            input Modelica.SIunits.Temperature T "Temperature";
            output Real xws "Absolute humidity ratio";
          protected
            Real pds;
            Real Tlim;
          algorithm
            pds := Modelica.Media.Air.ReferenceMoistAir.Utilities.pds_pT(p, T);
            Tlim := if T <= 273.16 then Modelica.Media.Air.ReferenceMoistAir.Utilities.Ice09_Utilities.Basic.Tsub(p) else Modelica.Media.Air.ReferenceMoistAir.Utilities.Water95_Utilities.Tsat(p);
            xws := if T <= Tlim then Modelica.Media.Air.ReferenceMoistAir.k_mair * pds / (p - pds) else -1;
            annotation(
              derivative = xws_pT_der,
              Inline = false,
              LateInline = true);
          end xws_pT;

          function phi_pTX "Relative humidity"
            extends Modelica.Icons.Function;
            input Modelica.SIunits.AbsolutePressure p "Pressure";
            input Modelica.SIunits.Temperature T "Temperature";
            input Modelica.SIunits.MassFraction X[:] = Modelica.Media.Air.ReferenceMoistAir.reference_X "Mass fractions";
            output Real phi "Relative humidity";
          protected
            Real xw;
            Real pd;
            Real pds;
          algorithm
            if X[1] == 0 then
              phi := 0;
            else
              xw := X[1] / (1 - X[1]);
              if T >= 273.16 then
                pds := Modelica.Media.Water.IF97_Utilities.BaseIF97.Basic.psat(T);
              else
                pds := Modelica.Media.Air.ReferenceMoistAir.Utilities.Ice09_Utilities.Basic.psub(T);
              end if;
              pds := Modelica.Media.Air.ReferenceMoistAir.Utilities.f_pT(p, T) * pds;
              pd := xw / (Modelica.Media.Air.ReferenceMoistAir.k_mair + xw) * p;
              if pd <= pds then
                phi := pd / pds;
              else
                phi := -1;
              end if;
            end if;
            annotation(
              Inline = false,
              LateInline = true);
          end phi_pTX;

          function cp_pTX "Specific isobaric heat capacity"
            extends Modelica.Icons.Function;
            input Modelica.SIunits.AbsolutePressure p "Pressure";
            input Modelica.SIunits.Temperature T "Temperature";
            input Modelica.SIunits.MassFraction X[:] = Modelica.Media.Air.ReferenceMoistAir.reference_X "Mass fractions";
            output Modelica.SIunits.SpecificHeatCapacity cp "Specific heat capacity";
          protected
            Real xw;
            Real xws;
            Real pd;
            Real pl;
          algorithm
            if X[1] == 0.0 then
              if T >= 773.15 then
                cp := Modelica.Media.Air.ReferenceAir.Air_Utilities.cp_pT(p, T) + Utilities.cp_dis_pTX(p, T, X);
              else
                cp := Modelica.Media.Air.ReferenceAir.Air_Utilities.cp_pT(p, T);
              end if;
            else
              pd := Modelica.Media.Air.ReferenceMoistAir.Utilities.pd_pTX(p, T, X);
              pl := p - pd;
              xw := X[1] / (1 - X[1]);
              xws := Modelica.Media.Air.ReferenceMoistAir.Utilities.xws_pT(p, T);
              if xw <= xws or xws == (-1) then
                if T >= 773.15 then
                  cp := X[1] * Modelica.Media.Air.ReferenceMoistAir.Utilities.IF97_new.cp_pT(pd, T) + X[2] * Modelica.Media.Air.ReferenceAir.Air_Utilities.cp_pT(pl, T) + Modelica.Media.Air.ReferenceMoistAir.Utilities.cp_dis_pTX(p, T, X);
                else
                  cp := X[1] * Modelica.Media.Air.ReferenceMoistAir.Utilities.IF97_new.cp_pT(pd, T) + X[2] * Modelica.Media.Air.ReferenceAir.Air_Utilities.cp_pT(pl, T);
                end if;
              else
                cp := -1;
              end if;
            end if;
            annotation(
              Inline = false,
              LateInline = true);
          end cp_pTX;

          function cv_pTX "Specific isochoric heat capacity"
            extends Modelica.Icons.Function;
            input Modelica.SIunits.AbsolutePressure p "Pressure";
            input Modelica.SIunits.Temperature T "Temperature";
            input Modelica.SIunits.MassFraction X[:] = Modelica.Media.Air.ReferenceMoistAir.reference_X "Mass fractions";
            output Modelica.SIunits.SpecificHeatCapacity cv "Specific heat capacity";
          protected
            Real xw;
            Real xws;
            Real pd;
            Real pl;
          algorithm
            if X[1] == 0 then
              cv := Modelica.Media.Air.ReferenceAir.Air_Utilities.cv_pT(p, T);
            else
              pd := Modelica.Media.Air.ReferenceMoistAir.Utilities.pd_pTX(p, T, X);
              pl := p - pd;
              xw := X[1] / (1 - X[1]);
              xws := Modelica.Media.Air.ReferenceMoistAir.Utilities.xws_pT(p, T);
              if xw <= xws or xws == (-1) then
                cv := X[1] * Modelica.Media.Air.ReferenceMoistAir.Utilities.IF97_new.cv_pT(pd, T) + X[2] * Modelica.Media.Air.ReferenceAir.Air_Utilities.cv_pT(pl, T);
              else
                cv := -1;
              end if;
            end if;
            annotation(
              Inline = false,
              LateInline = true);
          end cv_pTX;

          function h_pTX "Specific enthalpy of moist air"
            extends Modelica.Icons.Function;
            input Modelica.SIunits.AbsolutePressure p "Pressure";
            input Modelica.SIunits.Temperature T "Temperature";
            input Modelica.SIunits.MassFraction X[:] = Modelica.Media.Air.ReferenceMoistAir.reference_X "Mass fractions";
            output Modelica.SIunits.SpecificEnthalpy h "Specific enthalpy";
          protected
            Real xw;
            Real xws;
            Real pd;
            Real pl;
          algorithm
            if X[1] == 0 then
              h := Modelica.Media.Air.ReferenceAir.Air_Utilities.h_pT(p, T);
            else
              pd := Modelica.Media.Air.ReferenceMoistAir.Utilities.pd_pTX(p, T, X);
              pl := p - pd;
              xw := X[1] / (1 - X[1]);
              xws := Modelica.Media.Air.ReferenceMoistAir.Utilities.xws_pT(p, T);
              if xw <= xws or xws == (-1) then
                if T >= 773.15 then
                  h := Modelica.Media.Air.ReferenceAir.Air_Utilities.h_pT(pl, T) + xw * Modelica.Media.Air.ReferenceMoistAir.Utilities.IF97_new.h_pT(pd, T) + (1 + xw) * Modelica.Media.Air.ReferenceMoistAir.Utilities.h_dis_pTX(p, T, X);
                else
                  h := Modelica.Media.Air.ReferenceAir.Air_Utilities.h_pT(pl, T) + xw * Modelica.Media.Air.ReferenceMoistAir.Utilities.IF97_new.h_pT(pd, T);
                end if;
              else
                if T < 273.16 then
                  h := Modelica.Media.Air.ReferenceAir.Air_Utilities.h_pT(pl, T) + xws * Modelica.Media.Air.ReferenceMoistAir.Utilities.IF97_new.h_pT(pd, T) + (xw - xws) * Modelica.Media.Air.ReferenceMoistAir.Utilities.Ice09_Utilities.h_pT(p, T);
                else
                  h := Modelica.Media.Air.ReferenceAir.Air_Utilities.h_pT(pl, T) + xws * Modelica.Media.Air.ReferenceMoistAir.Utilities.IF97_new.h_pT(pd, T) + (xw - xws) * Modelica.Media.Water.IF97_Utilities.h_pT(p, T);
                end if;
              end if;
              h := h / (1 + xw);
            end if;
            annotation(
              derivative = h_pTX_der,
              Inline = false,
              LateInline = true);
          end h_pTX;

          function h_dis_pTX
            extends Modelica.Icons.Function;
            input Modelica.SIunits.AbsolutePressure p "Pressure";
            input Modelica.SIunits.Temperature T "Temperature";
            input Modelica.SIunits.MassFraction X[:] = Modelica.Media.Air.ReferenceMoistAir.reference_X "Mass fractions";
            output Real u "Reaction index";
          protected
            Real uges;
            Real invMMX[4] "Inverses of molar weights";
            SI.MolarMass Mmix "Molar mass of mixture";
            MassFraction[4] massFraction "Mass fractions of components";
            Modelica.SIunits.MoleFraction[4] Y "Mole fractions of individual components (H2O, N2, O2, Ar) of moist air";
          algorithm
            if useDissociation == false then
              u := 0;
            else
              massFraction := {X[1], X[2] * Xi_Air[1], X[2] * Xi_Air[2], X[2] * Xi_Air[3]};
              for i in 1:4 loop
                invMMX[i] := 1 / MMX[i];
              end for;
              Mmix := 1 / (massFraction * invMMX);
              for i in 1:4 loop
                Y[i] := Mmix * massFraction[i] / MMX[i];
              end for;
              uges := 1 + Modelica.Media.Air.ReferenceMoistAir.Utilities.ReactionIndices.U2(p, T, Y) + Modelica.Media.Air.ReferenceMoistAir.Utilities.ReactionIndices.U3(p, T, Y) + Modelica.Media.Air.ReferenceMoistAir.Utilities.ReactionIndices.U4(p, T, Y) + Modelica.Media.Air.ReferenceMoistAir.Utilities.ReactionIndices.U5(p, T, Y) + Modelica.Media.Air.ReferenceMoistAir.Utilities.ReactionIndices.U6(p, T, Y);
              u := -T ^ 2 * (Modelica.Media.Air.ReferenceMoistAir.Utilities.ReactionIndices.U2(p, T, Y) * Modelica.Media.Air.ReferenceMoistAir.Utilities.ReactionIndices.V2(T) / Modelica.Media.Air.ReferenceMoistAir.Utilities.ReactionIndices.BB[2] + Modelica.Media.Air.ReferenceMoistAir.Utilities.ReactionIndices.U3(p, T, Y) * Modelica.Media.Air.ReferenceMoistAir.Utilities.ReactionIndices.V3(T) / Modelica.Media.Air.ReferenceMoistAir.Utilities.ReactionIndices.BB[3] + Modelica.Media.Air.ReferenceMoistAir.Utilities.ReactionIndices.U4(p, T, Y) * Modelica.Media.Air.ReferenceMoistAir.Utilities.ReactionIndices.V4(T) / Modelica.Media.Air.ReferenceMoistAir.Utilities.ReactionIndices.BB[4] + Modelica.Media.Air.ReferenceMoistAir.Utilities.ReactionIndices.U5(p, T, Y) * Modelica.Media.Air.ReferenceMoistAir.Utilities.ReactionIndices.V5(T) / Modelica.Media.Air.ReferenceMoistAir.Utilities.ReactionIndices.BB[5] + Modelica.Media.Air.ReferenceMoistAir.Utilities.ReactionIndices.U6(p, T, Y) * Modelica.Media.Air.ReferenceMoistAir.Utilities.ReactionIndices.V6(T) / Modelica.Media.Air.ReferenceMoistAir.Utilities.ReactionIndices.BB[6]) / uges * sum(massFraction[j] / MMX[j] for j in 1:4);
            end if;
          end h_dis_pTX;

          function s_pTX "Specific entropy of moist air"
            extends Modelica.Icons.Function;
            input Modelica.SIunits.AbsolutePressure p "Pressure";
            input Modelica.SIunits.Temperature T "Temperature";
            input Modelica.SIunits.MassFraction X[:] = Modelica.Media.Air.ReferenceMoistAir.reference_X "Mass fractions";
            output Modelica.SIunits.SpecificEntropy s "Specific entropy";
          protected
            Real xw;
            Real xws;
            Real pd;
            Real pl;
          algorithm
            if X[1] == 0 then
              s := Modelica.Media.Air.ReferenceAir.Air_Utilities.s_pT(p, T);
            else
              pd := Modelica.Media.Air.ReferenceMoistAir.Utilities.pd_pTX(p, T, X);
              pl := p - pd;
              xw := X[1] / (1 - X[1]);
              xws := Modelica.Media.Air.ReferenceMoistAir.Utilities.xws_pT(p, T);
              if xw <= xws or xws == (-1) then
                if T >= 773.15 then
                  s := Modelica.Media.Air.ReferenceAir.Air_Utilities.s_pT(pl, T) + xw * Modelica.Media.Air.ReferenceMoistAir.Utilities.IF97_new.s_pT(pd, T) + (1 + xw) * Modelica.Media.Air.ReferenceMoistAir.Utilities.s_dis_pTX(p, T, X);
                else
                  s := Modelica.Media.Air.ReferenceAir.Air_Utilities.s_pT(pl, T) + xw * Modelica.Media.Air.ReferenceMoistAir.Utilities.IF97_new.s_pT(pd, T);
                end if;
              else
                if T < 273.16 then
                  s := Modelica.Media.Air.ReferenceAir.Air_Utilities.s_pT(pl, T) + xws * Modelica.Media.Air.ReferenceMoistAir.Utilities.IF97_new.s_pT(pd, T) + (xw - xws) * Modelica.Media.Air.ReferenceMoistAir.Utilities.Ice09_Utilities.s_pT(p, T);
                else
                  s := Modelica.Media.Air.ReferenceAir.Air_Utilities.s_pT(pl, T) + xws * Modelica.Media.Air.ReferenceMoistAir.Utilities.IF97_new.s_pT(pd, T) + (xw - xws) * Modelica.Media.Water.IF97_Utilities.s_pT(p, T);
                end if;
              end if;
              s := s / (1 + xw);
            end if;
            annotation(
              Inline = false,
              LateInline = true);
          end s_pTX;

          function u_pTX "Internal energy"
            extends Modelica.Icons.Function;
            input Modelica.SIunits.AbsolutePressure p "Pressure";
            input Modelica.SIunits.Temperature T "Temperature";
            input Modelica.SIunits.MassFraction X[:] = Modelica.Media.Air.ReferenceMoistAir.reference_X "Mass fractions";
            output Modelica.SIunits.SpecificEnergy u "Specific entropy";
          algorithm
            u := Modelica.Media.Air.ReferenceMoistAir.Utilities.h_pTX(p, T, X) - p / Modelica.Media.Air.ReferenceMoistAir.Utilities.rho_pTX(p, T, X);
            annotation(
              derivative = u_pTX_der,
              Inline = false,
              LateInline = true);
          end u_pTX;

          function cp_dis_pTX
            extends Modelica.Icons.Function;
            input Modelica.SIunits.AbsolutePressure p "Pressure";
            input Modelica.SIunits.Temperature T "Temperature";
            input Modelica.SIunits.MassFraction X[:] = Modelica.Media.Air.ReferenceMoistAir.reference_X "Mass fractions";
            output Real u "Reaction index";
          protected
            Real uges;
            Real invMMX[4] "Inverses of molar weights";
            SI.MolarMass Mmix "Molar mass of mixture";
            MassFraction[4] massFraction "Mass fractions of components";
            Modelica.SIunits.MoleFraction[4] Y "Mole fractions of individual components (H2O, N2, O2, Ar) of moist air";
          algorithm
            if useDissociation == false then
              u := 0;
            else
              massFraction := {X[1], X[2] * Xi_Air[1], X[2] * Xi_Air[2], X[2] * Xi_Air[3]};
              for i in 1:4 loop
                invMMX[i] := 1 / MMX[i];
              end for;
              Mmix := 1 / (massFraction * invMMX);
              for i in 1:4 loop
                Y[i] := Mmix * massFraction[i] / MMX[i];
              end for;
              uges := 1 + Modelica.Media.Air.ReferenceMoistAir.Utilities.ReactionIndices.U2(p, T, Y) + Modelica.Media.Air.ReferenceMoistAir.Utilities.ReactionIndices.U3(p, T, Y) + Modelica.Media.Air.ReferenceMoistAir.Utilities.ReactionIndices.U4(p, T, Y) + Modelica.Media.Air.ReferenceMoistAir.Utilities.ReactionIndices.U5(p, T, Y) + Modelica.Media.Air.ReferenceMoistAir.Utilities.ReactionIndices.U6(p, T, Y);
              u := (Modelica.Media.Air.ReferenceMoistAir.Utilities.ReactionIndices.U2(p, T, Y) * Modelica.Media.Air.ReferenceMoistAir.Utilities.ReactionIndices.V2(T) + Modelica.Media.Air.ReferenceMoistAir.Utilities.ReactionIndices.U3(p, T, Y) * Modelica.Media.Air.ReferenceMoistAir.Utilities.ReactionIndices.V3(T) + Modelica.Media.Air.ReferenceMoistAir.Utilities.ReactionIndices.U4(p, T, Y) * Modelica.Media.Air.ReferenceMoistAir.Utilities.ReactionIndices.V4(T) + Modelica.Media.Air.ReferenceMoistAir.Utilities.ReactionIndices.U5(p, T, Y) * Modelica.Media.Air.ReferenceMoistAir.Utilities.ReactionIndices.V5(T) + Modelica.Media.Air.ReferenceMoistAir.Utilities.ReactionIndices.U6(p, T, Y) * Modelica.Media.Air.ReferenceMoistAir.Utilities.ReactionIndices.V6(T)) / uges * sum(massFraction[j] / MMX[j] for j in 1:4);
            end if;
          end cp_dis_pTX;

          function s_dis_pTX
            extends Modelica.Icons.Function;
            input Modelica.SIunits.AbsolutePressure p "Pressure";
            input Modelica.SIunits.Temperature T "Temperature";
            input Modelica.SIunits.MassFraction X[:] = Modelica.Media.Air.ReferenceMoistAir.reference_X "Mass fractions";
            output Real u "Reaction index";
          protected
            Real uges;
            Real invMMX[4] "Inverses of molar weights";
            SI.MolarMass Mmix "Molar mass of mixture";
            MassFraction[4] massFraction "Mass fractions of components";
            Modelica.SIunits.MoleFraction[4] Y "Mole fractions of individual components (H2O, N2, O2, Ar) of moist air";
          algorithm
            if useDissociation == false then
              u := 0;
            else
              massFraction := {X[1], X[2] * Xi_Air[1], X[2] * Xi_Air[2], X[2] * Xi_Air[3]};
              for i in 1:4 loop
                invMMX[i] := 1 / MMX[i];
              end for;
              Mmix := 1 / (massFraction * invMMX);
              for i in 1:4 loop
                Y[i] := Mmix * massFraction[i] / MMX[i];
              end for;
              uges := 1 + Modelica.Media.Air.ReferenceMoistAir.Utilities.ReactionIndices.U2(p, T, Y) + Modelica.Media.Air.ReferenceMoistAir.Utilities.ReactionIndices.U3(p, T, Y) + Modelica.Media.Air.ReferenceMoistAir.Utilities.ReactionIndices.U4(p, T, Y) + Modelica.Media.Air.ReferenceMoistAir.Utilities.ReactionIndices.U5(p, T, Y) + Modelica.Media.Air.ReferenceMoistAir.Utilities.ReactionIndices.U6(p, T, Y);
              u := -T * (Modelica.Media.Air.ReferenceMoistAir.Utilities.ReactionIndices.U2(p, T, Y) * Modelica.Media.Air.ReferenceMoistAir.Utilities.ReactionIndices.V2(T) / Modelica.Media.Air.ReferenceMoistAir.Utilities.ReactionIndices.BB[2] + Modelica.Media.Air.ReferenceMoistAir.Utilities.ReactionIndices.U3(p, T, Y) * Modelica.Media.Air.ReferenceMoistAir.Utilities.ReactionIndices.V3(T) / Modelica.Media.Air.ReferenceMoistAir.Utilities.ReactionIndices.BB[3] + Modelica.Media.Air.ReferenceMoistAir.Utilities.ReactionIndices.U4(p, T, Y) * Modelica.Media.Air.ReferenceMoistAir.Utilities.ReactionIndices.V4(T) / Modelica.Media.Air.ReferenceMoistAir.Utilities.ReactionIndices.BB[4] + Modelica.Media.Air.ReferenceMoistAir.Utilities.ReactionIndices.U5(p, T, Y) * Modelica.Media.Air.ReferenceMoistAir.Utilities.ReactionIndices.V5(T) / Modelica.Media.Air.ReferenceMoistAir.Utilities.ReactionIndices.BB[5] + Modelica.Media.Air.ReferenceMoistAir.Utilities.ReactionIndices.U6(p, T, Y) * Modelica.Media.Air.ReferenceMoistAir.Utilities.ReactionIndices.V6(T) / Modelica.Media.Air.ReferenceMoistAir.Utilities.ReactionIndices.BB[6]) / uges * sum(massFraction[j] / MMX[j] for j in 1:4);
            end if;
          end s_dis_pTX;

          function pd_pTX_der "Derivative of partial pressure of steam"
            extends Modelica.Icons.Function;
            input Modelica.SIunits.AbsolutePressure p "Pressure";
            input Modelica.SIunits.Temperature T "Temperature";
            input Modelica.SIunits.MassFraction X[:] = Modelica.Media.Air.ReferenceMoistAir.reference_X "Mass fractions";
            input Real p_der "Derivative of pressure";
            input Real T_der "Derivative of temperature";
            input Real X_der[:] "Derivative of mass fractions";
            output Real pd_der "Derivative of partial pressure";
          protected
            Real xw, xw_der;
            Real xws;
            Real pds, pds_der;
          algorithm
            if X[1] == 0 then
              pd_der := 0;
            else
              xw := X[1] / (1 - X[1]) "d(xw)/dt = d(xw)/dX[1] * dX[1]/dt";
              xw_der := (X_der[1] * (1 - X[1]) + X[1] * X_der[1]) / (1 - X[1]) ^ 2;
              pds_der := Modelica.Media.Air.ReferenceMoistAir.Utilities.pds_pT_der(p, T, p_der, T_der);
              pd_der := (xw_der * (Modelica.Media.Air.ReferenceMoistAir.k_mair + xw) - xw * xw_der) * p + xw / (Modelica.Media.Air.ReferenceMoistAir.k_mair + xw) * p_der;
              xws := Modelica.Media.Air.ReferenceMoistAir.Utilities.xws_pT(p, T);
              pd_der := if xw <= xws or xws == (-1) then pd_der else pds_der;
            end if;
//pd := 0;
//pd := xw/(Modelica.Media.Air.ReferenceMoistAir.k_mair + xw)*p;
          end pd_pTX_der;

          function xws_pT_der "Derivative of humidity ration (absolute) of saturated humid air"
            extends Modelica.Icons.Function;
            input Modelica.SIunits.AbsolutePressure p "Pressure";
            input Modelica.SIunits.Temperature T "Temperature";
            input Real p_der "Derivative of pressure";
            input Real T_der "Derivative of temperature";
            output Real xws_der "Derivative of absolute humidity ratio";
          protected
            Real pds, pds_der;
            Real Tlim;
          algorithm
            pds := Modelica.Media.Air.ReferenceMoistAir.Utilities.pds_pT(p, T);
            pds_der := Modelica.Media.Air.ReferenceMoistAir.Utilities.pds_pT_der(p, T, p_der, T_der);
            Tlim := if T <= 273.16 then Modelica.Media.Air.ReferenceMoistAir.Utilities.Ice09_Utilities.Basic.Tsub(p) else Modelica.Media.Air.ReferenceMoistAir.Utilities.Water95_Utilities.Tsat(p);
            if T <= Tlim then
              xws_der := Modelica.Media.Air.ReferenceMoistAir.k_mair * ((pds_der * (p - pds) + pds * pds_der) / (p - pds) ^ 2);
            else
              xws_der := 0;
            end if;
          end xws_pT_der;

          function pds_pT_der "Derivative of saturation partial pressure of steam"
            extends Modelica.Icons.Function;
            input Modelica.SIunits.AbsolutePressure p "Pressure";
            input Modelica.SIunits.Temperature T "Temperature";
            input Real p_der "Derivative of pressure";
            input Real T_der "Derivative of temperature";
            output Real pds_der "Derivative of pressure";
          protected
            Real Tlim;
          algorithm
            if T >= 273.16 then
              pds_der := Modelica.Media.Air.ReferenceMoistAir.Utilities.Water95_Utilities.psat_der(T, T_der);
              Tlim := Modelica.Media.Air.ReferenceMoistAir.Utilities.Water95_Utilities.Tsat(p);
            else
              pds_der := Modelica.Media.Air.ReferenceMoistAir.Utilities.Ice09_Utilities.Basic.psub_der(T, T_der);
              Tlim := Modelica.Media.Air.ReferenceMoistAir.Utilities.Ice09_Utilities.Basic.Tsub(p);
            end if;
            if T <= Tlim then
              pds_der := pds_der;
            else
              pds_der := 0;
            end if;
          end pds_pT_der;

          function rho_pTX_der "Derivative of density as a function of pressure p, temperature T and composition X"
            extends Modelica.Icons.Function;
            input Modelica.SIunits.AbsolutePressure p "Pressure";
            input Modelica.SIunits.Temperature T "Temperature";
            input Modelica.SIunits.MassFraction X[:] = Modelica.Media.Air.ReferenceMoistAir.reference_X "Mass fractions";
            input Real p_der "Derivative of pressure";
            input Real T_der "Derivative of temperature";
            input Real X_der[:] "Derivative of mass fractions";
            output Real d_der "Derivative of density";
          protected
            Real pd, pd_der;
            Real pl, pl_der;
            Real xw, xw_der;
            Real xws, xws_der;
            Real o[5];
          algorithm
            if X[1] == 0 then
              d_der := Modelica.Media.Air.ReferenceAir.Air_Utilities.rho_pT_der(p, T, Modelica.Media.Air.ReferenceAir.Air_Utilities.airBaseProp_pT(p, T), p_der, T_der);
            else
              xw := X[1] / (1 - X[1]);
              xw_der := X_der[1] / (1 - X[1]) ^ 2;
              xws := xws_pT(p, T);
              xws_der := xws_pT_der(p, T, p_der, T_der);
              pd := pd_pTX(p, T, X);
              pd_der := Modelica.Media.Air.ReferenceMoistAir.Utilities.pd_pTX_der(p, T, X, p_der, T_der, X_der);
              pl := p - pd;
              pl_der := p_der - pd_der;
              if xw <= xws or xws == (-1) then
                if T < 273.16 then
                  d_der := Modelica.Media.Air.ReferenceAir.Air_Utilities.rho_pT_der(pl, T, Modelica.Media.Air.ReferenceAir.Air_Utilities.airBaseProp_pT(pl, T), pl_der, T_der) + Modelica.Media.Air.ReferenceMoistAir.steam.R * (pd_der * T - pd * T_der) / (Modelica.Media.Air.ReferenceMoistAir.steam.R * T) ^ 2;
                else
                  d_der := Modelica.Media.Air.ReferenceAir.Air_Utilities.rho_pT_der(pl, T, Modelica.Media.Air.ReferenceAir.Air_Utilities.airBaseProp_pT(pl, T), pl_der, T_der) + IF97_new.rho_pT_der(pd, T, pd_der, T_der);
                end if;
              else
                if T < 273.16 then
                  o[1] := Modelica.Media.Air.ReferenceAir.Air_Utilities.rho_pT(pl, T);
                  o[2] := Modelica.Media.Air.ReferenceMoistAir.Utilities.Ice09_Utilities.rho_pT(p, T);
                  o[3] := (1 + xws) / (Modelica.Media.Air.ReferenceAir.Air_Utilities.rho_pT(pl, T) + pd / (.Modelica.Media.Air.ReferenceMoistAir.steam.R * T)) + (xw - xws) / Modelica.Media.Air.ReferenceMoistAir.Utilities.Ice09_Utilities.rho_pT(p, T);
                  o[4] := Modelica.Media.Air.ReferenceAir.Air_Utilities.rho_pT_der(pl, T, Modelica.Media.Air.ReferenceAir.Air_Utilities.airBaseProp_pT(p, T), p_der, T_der);
                  o[5] := (xws_der * o[1] - (1 + xws) * o[4]) / o[1] ^ 2 + (pd_der * T - pd * T_der) / Modelica.Media.Air.ReferenceMoistAir.steam.R / T ^ 2 + (xw_der * o[2] - xw * Modelica.Media.Air.ReferenceMoistAir.Utilities.Ice09_Utilities.rho_pT_der(p, T, Modelica.Media.Air.ReferenceMoistAir.Utilities.Ice09_Utilities.ice09BaseProp_pT(p, T), p_der, T_der)) / o[2] ^ 2 - (xws_der * o[2] - xws * Modelica.Media.Air.ReferenceMoistAir.Utilities.Ice09_Utilities.rho_pT_der(p, T, Modelica.Media.Air.ReferenceMoistAir.Utilities.Ice09_Utilities.ice09BaseProp_pT(p, T), p_der, T_der)) / o[2] ^ 2;
                  d_der := (xw_der * o[3] - (1 + xw) * o[5]) / o[3] ^ 2;
                else
                  o[1] := Modelica.Media.Air.ReferenceAir.Air_Utilities.rho_pT(pl, T) + IF97_new.rho_pT(pd, T);
                  o[2] := Modelica.Media.Water.IF97_Utilities.rho_pT(p, T);
                  o[3] := (1 + xws) / (Modelica.Media.Air.ReferenceAir.Air_Utilities.rho_pT(pl, T) + IF97_new.rho_pT(pd, T)) + (xw - xws) / Modelica.Media.Water.IF97_Utilities.rho_pT(p, T);
                  o[4] := Modelica.Media.Air.ReferenceAir.Air_Utilities.rho_pT_der(pl, T, Modelica.Media.Air.ReferenceAir.Air_Utilities.airBaseProp_pT(p, T), p_der, T_der) + IF97_new.rho_pT_der(pd, T, pd_der, T_der);
                  o[5] := (xws_der * o[1] - (1 + xws) * o[4]) / o[1] ^ 2 + (xw_der * o[2] - xw * Modelica.Media.Water.IF97_Utilities.rho_pT_der(p, T, Modelica.Media.Water.IF97_Utilities.waterBaseProp_pT(p, T), p_der, T_der)) / o[2] ^ 2 - (xws_der * o[2] - xws * Modelica.Media.Water.IF97_Utilities.rho_pT_der(p, T, Modelica.Media.Water.IF97_Utilities.waterBaseProp_pT(p, T), p_der, T_der)) / o[2] ^ 2;
                  d_der := (xw_der * o[3] - (1 + xw) * o[5]) / o[3] ^ 2;
                end if;
              end if;
            end if;
          end rho_pTX_der;

          function h_dis_pTX_der
            extends Modelica.Icons.Function;
            import Modelica.Media.Air.ReferenceMoistAir.Utilities;
            input Modelica.SIunits.AbsolutePressure p "Pressure";
            input Modelica.SIunits.Temperature T "Temperature";
            input Modelica.SIunits.MassFraction X[:] = Modelica.Media.Air.ReferenceMoistAir.reference_X "Mass fractions";
            input Real p_der "Derivative of pressure";
            input Real T_der "Derivative of temperature";
            input Real X_der[:] "Derivative of mass fractions";
            output Real u_der "Derivative of reaction index";
          protected
            Real uges, uges_der, o[7], l;
            Real invMMX[4] "Inverses of molar weights";
            SI.MolarMass Mmix "Molar mass of mixture";
            Real Mmix_der "Derivative of molar mass of mixture";
            MassFraction[4] massFraction "Mass fractions of components";
            Real[4] massFraction_der "Derivative of mass fractions of components";
            Modelica.SIunits.MoleFraction[4] Y "Mole fractions of individual components (H2O, N2, O2, Ar) of moist air";
            Real[4] Y_der "Derivative of mole fractions of individual components (H2O, N2, O2, Ar) of moist air";
          algorithm
            if useDissociation == false then
              u_der := 0;
            else
              massFraction := {X[1], X[2] * Xi_Air[1], X[2] * Xi_Air[2], X[2] * Xi_Air[3]};
              massFraction_der := {X_der[1], X_der[2] * Xi_Air[1], X_der[2] * Xi_Air[2], X_der[2] * Xi_Air[3]};
              for i in 1:4 loop
                invMMX[i] := 1 / MMX[i];
              end for;
              Mmix := 1 / (massFraction * invMMX);
              Mmix_der := -1 / (massFraction * invMMX) ^ 2 * massFraction_der * invMMX;
              for i in 1:4 loop
                Y[i] := Mmix * massFraction[i] / MMX[i];
                Y_der[i] := (Mmix_der * massFraction[i] + Mmix * massFraction_der[i]) / MMX[i];
              end for;
              uges := 1 + Utilities.ReactionIndices.U2(p, T, Y) + Utilities.ReactionIndices.U3(p, T, Y) + Utilities.ReactionIndices.U4(p, T, Y) + Utilities.ReactionIndices.U5(p, T, Y) + Utilities.ReactionIndices.U6(p, T, Y);
              uges_der := Utilities.ReactionIndices.U2_der(p, T, Y, p_der, T_der, Y_der) + Utilities.ReactionIndices.U3_der(p, T, Y, p_der, T_der, Y_der) + Utilities.ReactionIndices.U4_der(p, T, Y, p_der, T_der, Y_der) + Utilities.ReactionIndices.U5_der(p, T, Y, p_der, T_der, Y_der) + Utilities.ReactionIndices.U6_der(p, T, Y, p_der, T_der, Y_der);
              o[1] := (Utilities.ReactionIndices.U2_der(p, T, Y, p_der, T_der, Y_der) * Utilities.ReactionIndices.V2(T) + Utilities.ReactionIndices.U2(p, T, Y) * Utilities.ReactionIndices.V2_der(T, T_der)) / Utilities.ReactionIndices.BB[2];
              o[2] := (Utilities.ReactionIndices.U3_der(p, T, Y, p_der, T_der, Y_der) * Utilities.ReactionIndices.V3(T) + Utilities.ReactionIndices.U3(p, T, Y) * Utilities.ReactionIndices.V3_der(T, T_der)) / Utilities.ReactionIndices.BB[3];
              o[3] := (Utilities.ReactionIndices.U4_der(p, T, Y, p_der, T_der, Y_der) * Utilities.ReactionIndices.V4(T) + Utilities.ReactionIndices.U4(p, T, Y) * Utilities.ReactionIndices.V4_der(T, T_der)) / Utilities.ReactionIndices.BB[4];
              o[4] := (Utilities.ReactionIndices.U5_der(p, T, Y, p_der, T_der, Y_der) * Utilities.ReactionIndices.V5(T) + Utilities.ReactionIndices.U5(p, T, Y) * Utilities.ReactionIndices.V5_der(T, T_der)) / Utilities.ReactionIndices.BB[5];
              o[5] := (Utilities.ReactionIndices.U6_der(p, T, Y, p_der, T_der, Y_der) * Utilities.ReactionIndices.V6(T) + Utilities.ReactionIndices.U6(p, T, Y) * Utilities.ReactionIndices.V6_der(T, T_der)) / Utilities.ReactionIndices.BB[6];
              l := Utilities.ReactionIndices.U2(p, T, Y) * Utilities.ReactionIndices.V2(T) / Utilities.ReactionIndices.BB[2] + Utilities.ReactionIndices.U3(p, T, Y) * Utilities.ReactionIndices.V3(T) / Utilities.ReactionIndices.BB[3] + Utilities.ReactionIndices.U4(p, T, Y) * Utilities.ReactionIndices.V4(T) / Utilities.ReactionIndices.BB[4] + Utilities.ReactionIndices.U5(p, T, Y) * Utilities.ReactionIndices.V5(T) / Utilities.ReactionIndices.BB[5] + Utilities.ReactionIndices.U6(p, T, Y) * Utilities.ReactionIndices.V6(T) / Utilities.ReactionIndices.BB[6];
              o[6] := (-2 * T * (Utilities.ReactionIndices.U2(p, T, Y) * Utilities.ReactionIndices.V2(T) / Utilities.ReactionIndices.BB[2] + Utilities.ReactionIndices.U3(p, T, Y) * Utilities.ReactionIndices.V3(T) / Utilities.ReactionIndices.BB[3] + Utilities.ReactionIndices.U4(p, T, Y) * Utilities.ReactionIndices.V4(T) / Utilities.ReactionIndices.BB[4] + Utilities.ReactionIndices.U5(p, T, Y) * Utilities.ReactionIndices.V5(T) / Utilities.ReactionIndices.BB[5] + Utilities.ReactionIndices.U6(p, T, Y) * Utilities.ReactionIndices.V6(T) / Utilities.ReactionIndices.BB[6])) - T ^ 2 * sum(o[1:5]);
              o[7] := uges_der * sum(massFraction[j] / MMX[j] for j in 1:4) + uges * sum(massFraction_der[j] / MMX[j] for j in 1:4);
              u_der := (o[6] * (uges * sum(massFraction[j] / MMX[j] for j in 1:4)) - l * o[7]) / (uges * sum(massFraction[j] / MMX[j] for j in 1:4)) ^ 2;
            end if;
//u := 0;
          end h_dis_pTX_der;

          function h_pTX_der "Derivative of specific enthalpy of moist air"
            extends Modelica.Icons.Function;
            input Modelica.SIunits.AbsolutePressure p "Pressure";
            input Modelica.SIunits.Temperature T "Temperature";
            input Modelica.SIunits.MassFraction X[:] = Modelica.Media.Air.ReferenceMoistAir.reference_X "Mass fractions";
            input Real p_der "Derivative of pressure";
            input Real T_der "Derivative of temperature";
            input Real X_der[:] "Derivative of mass fractions";
            output Real h_der "Derivative of specific enthalpy";
          protected
            Modelica.SIunits.SpecificEnthalpy h;
            Real xw, xw_der;
            Real xws, xws_der;
            Real pd, pd_der;
            Real pl, pl_der;
          algorithm
            if X[1] == 0 then
              h_der := Modelica.Media.Air.ReferenceAir.Air_Utilities.h_pT_der(p, T, Modelica.Media.Air.ReferenceAir.Air_Utilities.airBaseProp_pT(p, T), p_der, T_der);
            else
              pd := Modelica.Media.Air.ReferenceMoistAir.Utilities.pd_pTX(p, T, X);
              pd_der := Modelica.Media.Air.ReferenceMoistAir.Utilities.pd_pTX_der(p, T, X, p_der, T_der, X_der);
              pl := p - pd;
              pl_der := p_der - pd_der;
              xw := X[1] / (1 - X[1]);
              xw_der := X_der[1] / (1 - X[1]) ^ 2;
              xws := Modelica.Media.Air.ReferenceMoistAir.Utilities.xws_pT(p, T);
              xws_der := xws_pT_der(p, T, p_der, T_der);
              if xw <= xws or xws == (-1) then
                if T >= 773.15 then
                  h_der := Modelica.Media.Air.ReferenceAir.Air_Utilities.h_pT_der(pl, T, Modelica.Media.Air.ReferenceAir.Air_Utilities.airBaseProp_pT(pl, T), pl_der, T_der) + xw_der * Modelica.Media.Air.ReferenceMoistAir.Utilities.IF97_new.h_pT(pd, T) + xw * Modelica.Media.Air.ReferenceMoistAir.Utilities.IF97_new.h_pT_der(pd, T, 0, pd_der, T_der) + xw_der * Modelica.Media.Air.ReferenceMoistAir.Utilities.h_dis_pTX(p, T, X) + xw * Modelica.Media.Air.ReferenceMoistAir.Utilities.h_dis_pTX_der(p, T, X, p_der, T_der, X_der);
                else
                  h_der := Modelica.Media.Air.ReferenceAir.Air_Utilities.h_pT_der(pl, T, Modelica.Media.Air.ReferenceAir.Air_Utilities.airBaseProp_pT(pl, T), pl_der, T_der) + xw_der * Modelica.Media.Air.ReferenceMoistAir.Utilities.IF97_new.h_pT(pd, T) + xw * Modelica.Media.Air.ReferenceMoistAir.Utilities.IF97_new.h_pT_der(pd, T, 0, pd_der, T_der);
                end if;
              else
                if T < 273.16 then
                  h_der := Modelica.Media.Air.ReferenceAir.Air_Utilities.h_pT_der(pl, T, Modelica.Media.Air.ReferenceAir.Air_Utilities.airBaseProp_pT(pl, T), pl_der, T_der) + xws_der * Modelica.Media.Air.ReferenceMoistAir.Utilities.IF97_new.h_pT(pd, T) + xws * Modelica.Media.Air.ReferenceMoistAir.Utilities.IF97_new.h_pT_der(pd, T, 0, pd_der, T_der) + xw_der * Modelica.Media.Air.ReferenceMoistAir.Utilities.Ice09_Utilities.h_pT(p, T) + xw * Modelica.Media.Air.ReferenceMoistAir.Utilities.Ice09_Utilities.h_pT_der(p, T, Modelica.Media.Air.ReferenceMoistAir.Utilities.Ice09_Utilities.ice09BaseProp_pT(p, T), p_der, T_der) - xws_der * Modelica.Media.Air.ReferenceMoistAir.Utilities.Ice09_Utilities.h_pT(p, T) - xws * Modelica.Media.Air.ReferenceMoistAir.Utilities.Ice09_Utilities.h_pT_der(p, T, Modelica.Media.Air.ReferenceMoistAir.Utilities.Ice09_Utilities.ice09BaseProp_pT(p, T), p_der, T_der);
                else
                  h_der := Modelica.Media.Air.ReferenceAir.Air_Utilities.h_pT_der(pl, T, Modelica.Media.Air.ReferenceAir.Air_Utilities.airBaseProp_pT(p, T), pl_der, T_der) + xws_der * Modelica.Media.Air.ReferenceMoistAir.Utilities.IF97_new.h_pT(pd, T) + xws * Modelica.Media.Air.ReferenceMoistAir.Utilities.IF97_new.h_pT_der(pd, T, 0, pd_der, T_der) + xw_der * Modelica.Media.Water.IF97_Utilities.h_pT(p, T) + xw * Modelica.Media.Water.IF97_Utilities.h_pT_der(p, T, Modelica.Media.Water.IF97_Utilities.waterBaseProp_pT(p, T), p_der, T_der) - xws_der * Modelica.Media.Water.IF97_Utilities.h_pT(p, T) - xws * Modelica.Media.Water.IF97_Utilities.h_pT_der(p, T, Modelica.Media.Water.IF97_Utilities.waterBaseProp_pT(p, T), p_der, T_der);
                end if;
              end if;
              if xw <= xws or xws == (-1) then
                if T >= 773.15 then
                  h := Modelica.Media.Air.ReferenceAir.Air_Utilities.h_pT(pl, T) + xw * Modelica.Media.Air.ReferenceMoistAir.Utilities.IF97_new.h_pT(pd, T) + (1 + xw) * Modelica.Media.Air.ReferenceMoistAir.Utilities.h_dis_pTX(p, T, X);
                else
                  h := Modelica.Media.Air.ReferenceAir.Air_Utilities.h_pT(pl, T) + xw * Modelica.Media.Air.ReferenceMoistAir.Utilities.IF97_new.h_pT(pd, T);
                end if;
              else
                if T < 273.16 then
                  h := Modelica.Media.Air.ReferenceAir.Air_Utilities.h_pT(pl, T) + xws * Modelica.Media.Air.ReferenceMoistAir.Utilities.IF97_new.h_pT(pd, T) + (xw - xws) * Modelica.Media.Air.ReferenceMoistAir.Utilities.Ice09_Utilities.h_pT(p, T);
                else
                  h := Modelica.Media.Air.ReferenceAir.Air_Utilities.h_pT(pl, T) + xws * Modelica.Media.Air.ReferenceMoistAir.Utilities.IF97_new.h_pT(pd, T) + (xw - xws) * Modelica.Media.Water.IF97_Utilities.h_pT(p, T);
                end if;
              end if;
              h_der := (h_der * (1 + xw) - h * xw_der) / (1 + xw) ^ 2;
            end if;
          end h_pTX_der;

          function u_pTX_der "Derivative of internal energy"
            extends Modelica.Icons.Function;
            input Modelica.SIunits.AbsolutePressure p "Pressure";
            input Modelica.SIunits.Temperature T "Temperature";
            input Modelica.SIunits.MassFraction X[:] = Modelica.Media.Air.ReferenceMoistAir.reference_X "Mass fractions";
            input Real p_der "Derivative of pressure";
            input Real T_der "Derivative of temperature";
            input Real X_der[:] "Derivative of mass fractions";
            output Real u_der "Derivative of specific entropy";
          algorithm
            u_der := Modelica.Media.Air.ReferenceMoistAir.Utilities.h_pTX_der(p, T, X, p_der, T_der, X_der) - (p_der * Modelica.Media.Air.ReferenceMoistAir.Utilities.rho_pTX(p, T, X) - p * Modelica.Media.Air.ReferenceMoistAir.Utilities.rho_pTX_der(p, T, X, p_der, T_der, X_der)) / Modelica.Media.Air.ReferenceMoistAir.Utilities.rho_pTX(p, T, X) ^ 2;
          end u_pTX_der;
        end Utilities;
        annotation(
          Documentation(info = "<html>
<p>
Calculation of fluid properties for moist air in the region from 143.15 Kelvin to 2000 Kelvin at pressures up to 10 MPa. This model of moist air is based on
the diploma thesis of Hellriegel [10] with small modifications. Moist air is treated as an ideal mixture of the real fluids air and water.
</p>

<h4>Restriction</h4>
<p>
The functions provided by this package shall be used inside of the restricted limits according to the referenced literature.
</p>

<ul>
<li>
<b>611.2 Pa &le; p &le; 10 MPa</b>
</li>
<li>
<b>143.15 K &le; T &le; 2000 K</b>
</li>
</ul>

<h4>Usage</h4>
<p>
The package MoistAir can be used as any other medium model (see <a href=\"modelica://Modelica.Media.UsersGuide\">User's Guide of Media Library</a> for further information). The package defines two boolean constants <b>useEnhancementFactor</b> and <b>useDissociation</b>, which give the user fine grained control of the calculations.
</p>
<table border=1 cellspacing=0 cellpadding=2>
<tr>
<td valign=\"top\"><b>Constant</b></td>
<td valign=\"top\"><b>Default Value</b></td>
<td valign=\"top\"><b>Meaning</b></td>
</tr>
<tr>
<td valign=\"top\">useEnhancementFactor</td>
<td valign=\"top\">false</td>
<td valign=\"top\">The enhancement factor is used in the calculation of the saturation partial pressure of water in moist air. It is always very close to 1 except for high pressures (&gt;2 MPa) and low temperatures (&lt;233.15 K). For pressures less than 1 MPa this factor can be safely set to 1. Its calculation is very expensive, since it can only be calculated by an iterative method.</td>
</tr>
<tr>
<td valign=\"top\">useDissociation</td>
<td valign=\"top\">true</td>
<td valign=\"top\">The effect of dissociation is taken into account for temperatures greater than 773.15 K.</td>
</tr>
</table>

<h4>Calculation algorithms</h4>
<h5>Nomenclature</h5>
<table width=\"25%\">
<colgroup>
<col width=\"1*\">
<col width=\"2*\">
</colgroup>
<tr>
<td>p</td>
<td>Mixture pressure in Pa</td>
</tr>
<tr>
<td>T</td>
<td>Temperature in K</td>
</tr>
<tr>
<td>x<sub>w</sub></td>
<td>Absolutue humidity in kg(water)/kg(dry air)</td>
</tr>
<tr>
<td>x<sub>ws</sub></td>
<td>Absolutue humidity on saturation boundary in kg(water)/kg(dry air)</td>
</tr>
<tr>
<td>&phi;</td>
<td>Relative humidity (only defined for unsaturated humid air)</td>
</tr>
</table>

<h5>Unsaturated and saturated humid air (0 &le; x<sub>w</sub> &le; x<sub>ws</sub>)</h5>
<p>
Ideal mixture of dry air and steam
</p>

<ul>
<li>Dry air:
<ul>
<li>d,h,u,s,c<sub>p</sub> from [1]</li>
<li>&lambda;, &eta; from [2]</li>
</ul>
</li>

<li>Steam:
<ul>
<li>d,h,u,s,c<sub>p</sub> from [4]</li>
<li>&lambda;, &eta; for 273.15 K &le; T &le; 1073.15 K from [5] and [6]</li>
<li>&lambda;, &eta; for T &lt; 273.15 K or T &gt; 1073.15 K from [12]</li>
</ul>
</li>
</ul>

<h5>Supersaturated humid air (liquid fog and ice fog)</h5>
<p>
<b>Liquid fog (x<sub>w</sub> &gt; x<sub>wsw</sub>) and T &ge; 273.16 K</b>
</p>

<p>
Ideal mixture of saturated humid air and water
</p>

<ul>
<li>Saturated humid air (see above)</li>
<li>d,h,u,s of liquid droplets from [4]</li>
<li>c<sub>p</sub> is not defined</li>
<li>&lambda;, &eta; of liquid droplets from [5] and [6]</li>
</ul>

<p>
<b>Ice fog (x<sub>w</sub> &gt; x<sub>wsw</sub>) and T &lt; 273.16 K</b>
</p>

<p>
Ideal mixture of saturated humid air and ice
</p>

<ul>
<li>Saturated humid air (see above)</li>
<li>d,h,u,s of ice crystals from [7]</li>
<li>c<sub>p</sub> is not defined</li>
<li>&lambda; of ice as constant value</li>
<li>&eta; of ice is neglected</li>
</ul>


<h5>Saturation pressure of water in moist air</h5>
<p>
The saturation pressure p<sub>ds</sub> of water in moist air is calculated by p<sub>ds</sub> = f*p<sub>sat</sub>, where
</p>

<ul>
<li>f is the enhancement factor from [9] and [3]</li>
<li>p<sub>sat</sub> for T &ge; 273.16 K is the saturation pressure from [4]</li>
<li>p<sub>sat</sub> for T &lt; 273.16 K is the saturation pressure from [8]</li>
</ul>


<h5>Dissociation</h5>
<p>
For temperatures above 773.15 K effects of dissociation are taken into account. Dissociation is modeled according to [11].
For high temperatures the calculated values for moist air with 0 kg(water)/kg(dry air) (i.e. dry air) may differ from those
calculated by the package Modelica.Media.Air.ReferenceAir, because there no dissociation is considered.
</p>

<h4>References</h4>
<dl>
<dd>[1] <b>Thermodynamic Properties of Air and Mixtures of Nitrogen, Argon,
and Oxygen From 60 to 2000 K at Pressures to 2000 MPa</b>. J. Phys. Chem. Ref. Data, Vol. 29, No. 3, 2000.
</dd>
<dd>[2] <b>Viscosity and Thermal Conductivity Equations for
Nitrogen, Oxygen, Argon, and Air</b>. International Journal of Thermophysics, Vol. 25, No. 1, January 2004
</dd>
<dd>[3] <b>Revised Release on the IAPWS Formulation 1995 for the Thermodynamic
Properties of Ordinary Water Substance for General and Scientific Use</b>. 2009 International Association for the Properties of Water and Steam.
</dd>
<dd>[4] <b>Revised Release on the IAPWS Industrial Formulation 1997
for the Thermodynamic Properties of Water and Steam</b>. 2007 International Association for the Properties of Water and Steam.
</dd>
<dd>[5] <b>Release on the IAPWS Formulation 2008 for the Viscosity of Ordinary Water Substance</b>. 2008 International Association for the Properties of Water and Steam
</dd>
<dd>[6] <b>Release on the IAPWS Formulation 2011 for the Thermal Conductivity of
Ordinary Water Substance</b>. 2011 International Association for the Properties of Water and Steam.
</dd>
<dd>[7] <b>Revised Release on the Equation of State 2006 for H2O Ice Ih</b>. 2009 International Association for the Properties of Water and Steam.
</dd>
<dd>[8] <b>Revised Release on the Pressure along the Melting and
Sublimation Curves of Ordinary Water Substance</b>. 2011 International Association for the Properties of Water and Steam.
</dd>
<dd>[9] <b>Determination of Thermodynamic and Transport Properties
of Humid Air for Power-Cycle Calculations</b>. 2009 PTB, Braunschweig, Germany.
</dd>
<dd>[10] <b>Berechnung der thermodynamischen Zustandsfunktionen von
feuchter Luft in energietechnischen Prozessmodellierungen</b>. 2001 Diplomarbeit, Zittau.
</dd>
<dd>[11] <b>Thermodynamische Stoffwerte von feuchter Luft und Verbrennungsgasen</b>. 2003 VDI-Richtlinie 4670.
</dd>
<dd>[12] <b>W&auml;rme&uuml;bertragung in Dampferzeugern und W&auml;rmetauschern</b>. 1985 FDBR-Fachbuchreihe, Bd. 2, Vulkan Verlag Essen.
</dd>
</dl>

<h4>References</h4>
<dl>
<dt>Lemmon, E. W., Jacobsen, R. T., Penoncello, S. G., Friend, D. G.:</dt>
<dd><b>Thermodynamic Properties of Air and Mixtures of Nitrogen, Argon,
and Oxygen From 60 to 2000 K at Pressures to 2000 MPa</b>. J. Phys. Chem. Ref. Data, Vol. 29, No. 3, 2000.
</dd>
<dt>Lemmon, E. W., Jacobsen, R. T.:</dt>
<dd><b>Viscosity and Thermal Conductivity Equations for
Nitrogen, Oxygen, Argon, and Air</b>. International Journal of Thermophysics, Vol. 25, No. 1, January 2004
</dd>
</dl>


<h4>Verification</h4>
<p>
The verification report for the development of this library is provided
<a href=\"modelica://Modelica/Resources/Documentation/Media/MoMoLib_VerificationResults_XRG.pdf\">here</a>.
</p>

<h4>Acknowledgment</h4>
<p>
This library was developed by XRG Simulation GmbH as part of the <a href=\"http://www.cleansky.eu/\">Clean Sky</a> JTI project (Project title: MoMoLib-Modelica Model Library Development for Media, Magnetic Systems and Wavelets; Project number: 296369; Theme: JTI-CS-2011-1-SGO-02-026: Modelica Model Library Development Part I). The partial financial support for the development of this library by the European Union is highly appreciated.
</p>

<p>
Some parts of this library refer to the ThermoFluid library developed at Lund University (<a href=\"http://thermofluid.sourceforge.net/\">http://thermofluid.sourceforge.net</a>).
</p>

<h4>Disclaimer</h4>
<p>
In no event will XRG Simulation GmbH be liable for any direct, indirect, incidental, special, exemplary, or consequential damages, arising in any way out of the use of this software, even if advised of the possibility of such damage.
</p>
<h4> Copyright (C) 2013, XRG Simulation GmbH </h4>
</html>"));
      end ReferenceMoistAir;
      annotation(
        Documentation(info = "<html>
  <p>This package contains different medium models for air:</p>
<ul>
<li><b>SimpleAir</b><br>
    Simple dry air medium in a limited temperature range.</li>
<li><b>DryAirNasa</b><br>
    Dry air as an ideal gas from Media.IdealGases.MixtureGases.Air.</li>
<li><b>MoistAir</b><br>
    Moist air as an ideal gas mixture of steam and dry air with fog below and above the triple point temperature.</li>
</ul>
</html>"));
    end Air;

    package CompressibleLiquids "Compressible liquid models"
      extends Modelica.Icons.VariantsPackage;

      package Common "Base classes for compressible liquids"
        extends Modelica.Icons.Package;

        partial package LinearWater_pT "Base class for liquid, linear compressibility water models"
          extends Media.Interfaces.PartialLinearFluid(constantJacobian = true, reference_d = Modelica.Media.Water.StandardWater.density(state), reference_h = Modelica.Media.Water.StandardWater.specificEnthalpy(state), reference_s = Modelica.Media.Water.StandardWater.specificEntropy(state), cp_const = Modelica.Media.Water.StandardWater.specificHeatCapacityCp(state), beta_const = Modelica.Media.Water.StandardWater.isobaricExpansionCoefficient(state), kappa_const = Modelica.Media.Water.StandardWater.isothermalCompressibility(state), MM_const = Modelica.Media.Water.StandardWater.molarMass(state));
          constant Modelica.Media.Water.StandardWater.ThermodynamicState state = Modelica.Media.Water.StandardWater.setState_pT(reference_p, reference_T);
        end LinearWater_pT;
      end Common;

      package LinearColdWater "Cold water model with linear compressibility"
        extends Media.Interfaces.PartialLinearFluid(constantJacobian = true, reference_p = 101325, reference_T = 278.15, reference_d = 997.05, reference_h = 104929, reference_s = 100.0, cp_const = 4181.9, beta_const = 2.5713e-4, kappa_const = 4.5154e-10, MM_const = 0.018015268);

        redeclare function extends dynamicViscosity "Dynamic viscosity of water"
          algorithm
            eta := 1.5e-3;
        end dynamicViscosity;

        redeclare function extends thermalConductivity "Thermal conductivity of water"
          algorithm
            lambda := 0.572;
        end thermalConductivity;
      end LinearColdWater;

      package LinearWater_pT_Ambient "Liquid, linear compressibility water model at 1.01325 bar and 25 degree Celsius"
        extends Modelica.Media.CompressibleLiquids.Common.LinearWater_pT(reference_p = 101325, reference_T = 298.15);

        redeclare function extends dynamicViscosity "Dynamic viscosity of water"
          algorithm
            eta := 8.9e-4;
        end dynamicViscosity;

        redeclare function extends thermalConductivity "Thermal conductivity of water"
          algorithm
            lambda := 0.608;
        end thermalConductivity;
        annotation(
          Documentation(info = "<html>
<h4>Water model with linear compressibility at ambient conditions</h4>
</html>"));
      end LinearWater_pT_Ambient;
      annotation(
        Documentation(info = "<html>

<h4>Fluid models with linear compressibility, using PartialLinearFluid as base class.</h4>
<p>The linear compressibility fluid models contained in this package are based on the assumptions that:
</p>
<ul>
<li>The specific heat capacity at constant pressure (cp) is constant</li>
<li>The isobaric expansion coefficient (beta) is constant</li>
<li>The isothermal compressibility (kappa) is constant</li>
<li>Pressure and temperature are used as states</li>
</ul>
<p>
This results in models that are only valid for small temperature ranges, but sufficient to model compressibility and e.g., the \"water hammer\" effect. Another advantage id that only 3 values need to be measured to have an initial model. Hydraulic fluids can often be approximated by this type of model.
</p>
<p>
That means that the density is a linear function in temperature and in pressure.
In order to define the complete model, a number of constant reference values are needed which
are computed at the reference values of the states pressure p and temperature T. The model can
be interpreted as a linearization of a full non-linear fluid model (but it is not linear in all
thermodynamic coordinates). Reference values are needed for
</p>
<ol>
<li>the density (reference_d),</li>
<li>the specific enthalpy (reference_h),</li>
<li>the specific entropy (reference_s).</li>
</ol>
<p>
Apart from that, a user needs to define the molar mass, MM_const.
Note that it is possible to define a fluid by computing the reference
values from a full non-linear fluid model by computing the package constants
using the standard functions defined in a fluid package (see example in Common, LinearWater_pT).
</p>

</html>"));
    end CompressibleLiquids;

    package IdealGases "Data and models of ideal gases (single, fixed and dynamic mixtures) from NASA source"
      extends Modelica.Icons.VariantsPackage;

      package Common "Common packages and data for the ideal gas models"
        extends Modelica.Icons.Package;

        record DataRecord "Coefficient data record for properties of ideal gases based on NASA source"
          extends Modelica.Icons.Record;
          String name "Name of ideal gas";
          SI.MolarMass MM "Molar mass";
          SI.SpecificEnthalpy Hf "Enthalpy of formation at 298.15K";
          SI.SpecificEnthalpy H0 "H0(298.15K) - H0(0K)";
          SI.Temperature Tlimit "Temperature limit between low and high data sets";
          Real alow[7] "Low temperature coefficients a";
          Real blow[2] "Low temperature constants b";
          Real ahigh[7] "High temperature coefficients a";
          Real bhigh[2] "High temperature constants b";
          SI.SpecificHeatCapacity R "Gas constant";
          annotation(
            Documentation(info = "<HTML>
<p>
This data record contains the coefficients for the
ideal gas equations according to:
</p>
<blockquote>
  <p>McBride B.J., Zehe M.J., and Gordon S. (2002): <b>NASA Glenn Coefficients
  for Calculating Thermodynamic Properties of Individual Species</b>. NASA
  report TP-2002-211556</p>
</blockquote>
<p>
The equations have the following structure:
</p>
<IMG src=\"modelica://Modelica/Resources/Images/Media/IdealGases/singleEquations.png\">
<p>
The polynomials for h(T) and s0(T) are derived via integration from the one for cp(T)  and contain the integration constants b1, b2 that define the reference specific enthalpy and entropy. For entropy differences the reference pressure p0 is arbitrary, but not for absolute entropies. It is chosen as 1 standard atmosphere (101325 Pa).
</p>
<p>
For most gases, the region of validity is from 200 K to 6000 K.
The equations are split into two regions that are separated
by Tlimit (usually 1000 K). In both regions the gas is described
by the data above. The two branches are continuous and in most
gases also differentiable at Tlimit.
</p>
</HTML>"));
        end DataRecord;

        package Functions "Basic Functions for ideal gases: cp, h, s, thermal conductivity, viscosity"
          extends Modelica.Icons.Package;
          constant Boolean excludeEnthalpyOfFormation = true "If true, enthalpy of formation Hf is not included in specific enthalpy h";
          constant Modelica.Media.Interfaces.Choices.ReferenceEnthalpy referenceChoice = Modelica.Media.Interfaces.Choices.ReferenceEnthalpy.ZeroAt0K "Choice of reference enthalpy";
          constant Modelica.Media.Interfaces.Types.SpecificEnthalpy h_offset = 0.0 "User defined offset for reference enthalpy, if referenceChoice = UserDefined";
          constant Integer methodForThermalConductivity(min = 1, max = 2) = 1;

          function cp_T "Compute specific heat capacity at constant pressure from temperature and gas data"
            extends Modelica.Icons.Function;
            input IdealGases.Common.DataRecord data "Ideal gas data";
            input SI.Temperature T "Temperature";
            output SI.SpecificHeatCapacity cp "Specific heat capacity at temperature T";
          algorithm
            cp := smooth(0, if T < data.Tlimit then data.R * (1 / (T * T) * (data.alow[1] + T * (data.alow[2] + T * (1. * data.alow[3] + T * (data.alow[4] + T * (data.alow[5] + T * (data.alow[6] + data.alow[7] * T))))))) else data.R * (1 / (T * T) * (data.ahigh[1] + T * (data.ahigh[2] + T * (1. * data.ahigh[3] + T * (data.ahigh[4] + T * (data.ahigh[5] + T * (data.ahigh[6] + data.ahigh[7] * T))))))));
            annotation(
              Inline = true,
              smoothOrder = 2);
          end cp_T;

          function cp_Tlow "Compute specific heat capacity at constant pressure, low T region"
            extends Modelica.Icons.Function;
            input IdealGases.Common.DataRecord data "Ideal gas data";
            input SI.Temperature T "Temperature";
            output SI.SpecificHeatCapacity cp "Specific heat capacity at temperature T";
          algorithm
            cp := data.R * (1 / (T * T) * (data.alow[1] + T * (data.alow[2] + T * (1. * data.alow[3] + T * (data.alow[4] + T * (data.alow[5] + T * (data.alow[6] + data.alow[7] * T)))))));
            annotation(
              Inline = false,
              derivative(zeroDerivative = data) = cp_Tlow_der);
          end cp_Tlow;

          function cp_Tlow_der "Compute specific heat capacity at constant pressure, low T region"
            extends Modelica.Icons.Function;
            input IdealGases.Common.DataRecord data "Ideal gas data";
            input SI.Temperature T "Temperature";
            input Real dT "Temperature derivative";
            output Real cp_der "Derivative of specific heat capacity";
          algorithm
            cp_der := dT * data.R / (T * T * T) * ((-2 * data.alow[1]) + T * ((-data.alow[2]) + T * T * (data.alow[4] + T * (2. * data.alow[5] + T * (3. * data.alow[6] + 4. * data.alow[7] * T)))));
            annotation(
              smoothOrder = 2);
          end cp_Tlow_der;

          function h_T "Compute specific enthalpy from temperature and gas data; reference is decided by the
              refChoice input, or by the referenceChoice package constant by default"
            import Modelica.Media.Interfaces.Choices;
            extends Modelica.Icons.Function;
            input IdealGases.Common.DataRecord data "Ideal gas data";
            input SI.Temperature T "Temperature";
            input Boolean exclEnthForm = excludeEnthalpyOfFormation "If true, enthalpy of formation Hf is not included in specific enthalpy h";
            input Modelica.Media.Interfaces.Choices.ReferenceEnthalpy refChoice = referenceChoice "Choice of reference enthalpy";
            input SI.SpecificEnthalpy h_off = h_offset "User defined offset for reference enthalpy, if referenceChoice = UserDefined";
            output SI.SpecificEnthalpy h "Specific enthalpy at temperature T";
          algorithm
            h := smooth(0, (if T < data.Tlimit then data.R * (((-data.alow[1]) + T * (data.blow[1] + data.alow[2] * Math.log(T) + T * (1. * data.alow[3] + T * (0.5 * data.alow[4] + T * (1 / 3 * data.alow[5] + T * (0.25 * data.alow[6] + 0.2 * data.alow[7] * T)))))) / T) else data.R * (((-data.ahigh[1]) + T * (data.bhigh[1] + data.ahigh[2] * Math.log(T) + T * (1. * data.ahigh[3] + T * (0.5 * data.ahigh[4] + T * (1 / 3 * data.ahigh[5] + T * (0.25 * data.ahigh[6] + 0.2 * data.ahigh[7] * T)))))) / T)) + (if exclEnthForm then -data.Hf else 0.0) + (if refChoice == Choices.ReferenceEnthalpy.ZeroAt0K then data.H0 else 0.0) + (if refChoice == Choices.ReferenceEnthalpy.UserDefined then h_off else 0.0));
            annotation(
              Inline = false,
              smoothOrder = 2);
          end h_T;

          function h_T_der "Derivative function for h_T"
            import Modelica.Media.Interfaces.Choices;
            extends Modelica.Icons.Function;
            input IdealGases.Common.DataRecord data "Ideal gas data";
            input SI.Temperature T "Temperature";
            input Boolean exclEnthForm = excludeEnthalpyOfFormation "If true, enthalpy of formation Hf is not included in specific enthalpy h";
            input Modelica.Media.Interfaces.Choices.ReferenceEnthalpy refChoice = referenceChoice "Choice of reference enthalpy";
            input SI.SpecificEnthalpy h_off = h_offset "User defined offset for reference enthalpy, if referenceChoice = UserDefined";
            input Real dT "Temperature derivative";
            output Real h_der "Specific enthalpy at temperature T";
          algorithm
            h_der := dT * Modelica.Media.IdealGases.Common.Functions.cp_T(data, T);
            annotation(
              Inline = true,
              smoothOrder = 1);
          end h_T_der;

          function h_Tlow "Compute specific enthalpy, low T region; reference is decided by the
              refChoice input, or by the referenceChoice package constant by default"
            import Modelica.Media.Interfaces.Choices;
            extends Modelica.Icons.Function;
            input IdealGases.Common.DataRecord data "Ideal gas data";
            input SI.Temperature T "Temperature";
            input Boolean exclEnthForm = excludeEnthalpyOfFormation "If true, enthalpy of formation Hf is not included in specific enthalpy h";
            input Modelica.Media.Interfaces.Choices.ReferenceEnthalpy refChoice = referenceChoice "Choice of reference enthalpy";
            input SI.SpecificEnthalpy h_off = h_offset "User defined offset for reference enthalpy, if referenceChoice = UserDefined";
            output SI.SpecificEnthalpy h "Specific enthalpy at temperature T";
          algorithm
            h := data.R * (((-data.alow[1]) + T * (data.blow[1] + data.alow[2] * Math.log(T) + T * (1. * data.alow[3] + T * (0.5 * data.alow[4] + T * (1 / 3 * data.alow[5] + T * (0.25 * data.alow[6] + 0.2 * data.alow[7] * T)))))) / T) + (if exclEnthForm then -data.Hf else 0.0) + (if refChoice == Choices.ReferenceEnthalpy.ZeroAt0K then data.H0 else 0.0) + (if refChoice == Choices.ReferenceEnthalpy.UserDefined then h_off else 0.0);
            annotation(
              Inline = false,
              smoothOrder = 2);
          end h_Tlow;

          function h_Tlow_der "Compute specific enthalpy, low T region; reference is decided by the
              refChoice input, or by the referenceChoice package constant by default"
            import Modelica.Media.Interfaces.Choices;
            extends Modelica.Icons.Function;
            input IdealGases.Common.DataRecord data "Ideal gas data";
            input SI.Temperature T "Temperature";
            input Boolean exclEnthForm = excludeEnthalpyOfFormation "If true, enthalpy of formation Hf is not included in specific enthalpy h";
            input Modelica.Media.Interfaces.Choices.ReferenceEnthalpy refChoice = referenceChoice "Choice of reference enthalpy";
            input SI.SpecificEnthalpy h_off = h_offset "User defined offset for reference enthalpy, if referenceChoice = UserDefined";
            input Real dT(unit = "K/s") "Temperature derivative";
            output Real h_der(unit = "J/(kg.s)") "Derivative of specific enthalpy at temperature T";
          algorithm
            h_der := dT * Modelica.Media.IdealGases.Common.Functions.cp_Tlow(data, T);
            annotation(
              Inline = true,
              smoothOrder = 2);
          end h_Tlow_der;

          function s0_T "Compute specific entropy from temperature and gas data"
            extends Modelica.Icons.Function;
            input IdealGases.Common.DataRecord data "Ideal gas data";
            input SI.Temperature T "Temperature";
            output SI.SpecificEntropy s "Specific entropy at temperature T";
          algorithm
            s := if T < data.Tlimit then data.R * (data.blow[2] - 0.5 * data.alow[1] / (T * T) - data.alow[2] / T + data.alow[3] * Math.log(T) + T * (data.alow[4] + T * (0.5 * data.alow[5] + T * (1 / 3 * data.alow[6] + 0.25 * data.alow[7] * T)))) else data.R * (data.bhigh[2] - 0.5 * data.ahigh[1] / (T * T) - data.ahigh[2] / T + data.ahigh[3] * Math.log(T) + T * (data.ahigh[4] + T * (0.5 * data.ahigh[5] + T * (1 / 3 * data.ahigh[6] + 0.25 * data.ahigh[7] * T))));
            annotation(
              Inline = true,
              smoothOrder = 2);
          end s0_T;

          function s0_Tlow "Compute specific entropy, low T region"
            extends Modelica.Icons.Function;
            input IdealGases.Common.DataRecord data "Ideal gas data";
            input SI.Temperature T "Temperature";
            output SI.SpecificEntropy s "Specific entropy at temperature T";
          algorithm
            s := data.R * (data.blow[2] - 0.5 * data.alow[1] / (T * T) - data.alow[2] / T + data.alow[3] * Math.log(T) + T * (data.alow[4] + T * (0.5 * data.alow[5] + T * (1 / 3 * data.alow[6] + 0.25 * data.alow[7] * T))));
            annotation(
              Inline = true);
          end s0_Tlow;

          function s0_Tlow_der "Compute derivative of specific entropy, low T region"
            extends Modelica.Icons.Function;
            input IdealGases.Common.DataRecord data "Ideal gas data";
            input SI.Temperature T "Temperature";
            input Real T_der "Temperature derivative";
            output SI.SpecificEntropy s "Specific entropy at temperature T";
          algorithm
            s := data.R * (data.blow[2] - 0.5 * data.alow[1] / (T * T) - data.alow[2] / T + data.alow[3] * Math.log(T) + T * (data.alow[4] + T * (0.5 * data.alow[5] + T * (1 / 3 * data.alow[6] + 0.25 * data.alow[7] * T))));
            annotation(
              Inline = true);
          end s0_Tlow_der;

          function dynamicViscosityLowPressure "Dynamic viscosity of low pressure gases"
            extends Modelica.Icons.Function;
            input SI.Temp_K T "Gas temperature";
            input SI.Temp_K Tc "Critical temperature of gas";
            input SI.MolarMass M "Molar mass of gas";
            input SI.MolarVolume Vc "Critical molar volume of gas";
            input Real w "Acentric factor of gas";
            input Interfaces.PartialMedium.DipoleMoment mu "Dipole moment of gas molecule";
            input Real k = 0.0 "Special correction for highly polar substances";
            output SI.DynamicViscosity eta "Dynamic viscosity of gas";
          protected
            parameter Real Const1_SI = 40.785 * 10 ^ (-9.5) "Constant in formula for eta converted to SI units";
            parameter Real Const2_SI = 131.3 / 1000.0 "Constant in formula for mur converted to SI units";
            Real mur = Const2_SI * mu / sqrt(Vc * Tc) "Dimensionless dipole moment of gas molecule";
            Real Fc = 1 - 0.2756 * w + 0.059035 * mur ^ 4 + k "Factor to account for molecular shape and polarities of gas";
            Real Tstar "Dimensionless temperature defined by equation below";
            Real Ov "Viscosity collision integral for the gas";
          algorithm
            Tstar := 1.2593 * T / Tc;
            Ov := 1.16145 * Tstar ^ (-0.14874) + 0.52487 * Modelica.Math.exp(-0.7732 * Tstar) + 2.16178 * Modelica.Math.exp(-2.43787 * Tstar);
            eta := Const1_SI * Fc * sqrt(M * T) / (Vc ^ (2 / 3) * Ov);
            annotation(
              smoothOrder = 2,
              Documentation(info = "<html>
<p>
The used formula are based on the method of Chung et al (1984, 1988) referred to in ref [1] chapter 9.
The formula 9-4.10 is the one being used. The Formula is given in non-SI units, the following conversion constants were used to
transform the formula to SI units:
</p>

<ul>
<li> <b>Const1_SI:</b> The factor 10^(-9.5) =10^(-2.5)*1e-7 where the
     factor 10^(-2.5) originates from the conversion of g/mol->kg/mol + cm^3/mol->m^3/mol
      and the factor 1e-7 is due to conversion from microPoise->Pa.s.</li>
<li>  <b>Const2_SI:</b> The factor 1/3.335641e-27 = 1e-3/3.335641e-30
      where the factor 3.335641e-30 comes from debye->C.m and
      1e-3 is due to conversion from cm^3/mol->m^3/mol</li>
</ul>

<h4>References:</h4>
<p>
[1] Bruce E. Poling, John E. Prausnitz, John P. O'Connell, \"The Properties of Gases and Liquids\" 5th Ed. Mc Graw Hill.
</p>

<h4>Author</h4>
<p>T. Skoglund, Lund, Sweden, 2004-08-31</p>

</html>"));
          end dynamicViscosityLowPressure;

          function thermalConductivityEstimate "Thermal conductivity of polyatomic gases(Eucken and Modified Eucken correlation)"
            extends Modelica.Icons.Function;
            input Interfaces.PartialMedium.SpecificHeatCapacity Cp "Constant pressure heat capacity";
            input Interfaces.PartialMedium.DynamicViscosity eta "Dynamic viscosity";
            input Integer method(min = 1, max = 2) = 1 "1: Eucken Method, 2: Modified Eucken Method";
            input IdealGases.Common.DataRecord data "Ideal gas data";
            output Interfaces.PartialMedium.ThermalConductivity lambda "Thermal conductivity [W/(m.k)]";
          algorithm
            lambda := if method == 1 then eta * (Cp - data.R + 9 / 4 * data.R) else eta * (Cp - data.R) * (1.32 + 1.77 / (Cp / Modelica.Constants.R - 1.0));
            annotation(
              smoothOrder = 2,
              Documentation(info = "<html>
<p>
This function provides two similar methods for estimating the
thermal conductivity of polyatomic gases.
The Eucken method (input method == 1) gives good results for low temperatures,
but it tends to give an underestimated value of the thermal conductivity
(lambda) at higher temperatures.<br>
The Modified Eucken method (input method == 2) gives good results for
high-temperatures, but it tends to give an overestimated value of the
thermal conductivity (lambda) at low temperatures.
</p>
</html>"));
          end thermalConductivityEstimate;
        end Functions;

        partial package SingleGasNasa "Medium model of an ideal gas based on NASA source"
          extends Interfaces.PartialPureSubstance(ThermoStates = Modelica.Media.Interfaces.Choices.IndependentVariables.pT, redeclare final record FluidConstants = Modelica.Media.Interfaces.Types.IdealGas.FluidConstants, mediumName = data.name, substanceNames = {data.name}, singleState = false, Temperature(min = 200, max = 6000, start = 500, nominal = 500), SpecificEnthalpy(start = if Functions.referenceChoice == ReferenceEnthalpy.ZeroAt0K then data.H0 else if Functions.referenceChoice == ReferenceEnthalpy.UserDefined then Functions.h_offset else 0, nominal = 1.0e5), Density(start = 10, nominal = 10), AbsolutePressure(start = 10e5, nominal = 10e5));

          redeclare record extends ThermodynamicState "Thermodynamic state variables for ideal gases"
              AbsolutePressure p "Absolute pressure of medium";
              Temperature T "Temperature of medium";
          end ThermodynamicState;

          import Modelica.Math;
          import Modelica.Media.Interfaces.Choices.ReferenceEnthalpy;
          constant IdealGases.Common.DataRecord data "Data record of ideal gas substance";
          constant FluidConstants[nS] fluidConstants "Constant data for the fluid";

          redeclare model extends BaseProperties(T(stateSelect = if preferredMediumStates then StateSelect.prefer else StateSelect.default), p(stateSelect = if preferredMediumStates then StateSelect.prefer else StateSelect.default)) "Base properties of ideal gas medium"
            equation
              assert(T >= 200 and T <= 6000, "
Temperature T (= " + String(T) + " K) is not in the allowed range
200 K <= T <= 6000 K required from medium model \"" + mediumName + "\".
              ");
              MM = data.MM;
              R = data.R;
              h = Modelica.Media.IdealGases.Common.Functions.h_T(data, T, Modelica.Media.IdealGases.Common.Functions.excludeEnthalpyOfFormation, Modelica.Media.IdealGases.Common.Functions.referenceChoice, Modelica.Media.IdealGases.Common.Functions.h_offset);
              u = h - R * T;
// Has to be written in the form d=f(p,T) in order that static
// state selection for p and T is possible
              d = p / (R * T);
// connect state with BaseProperties
              state.T = T;
              state.p = p;
          end BaseProperties;

          redeclare function setState_pTX "Return thermodynamic state as function of p, T and composition X"
            extends Modelica.Icons.Function;
            input AbsolutePressure p "Pressure";
            input Temperature T "Temperature";
            input MassFraction X[:] = reference_X "Mass fractions";
            output ThermodynamicState state;
          algorithm
            state := ThermodynamicState(p = p, T = T);
            annotation(
              Inline = true,
              smoothOrder = 2);
          end setState_pTX;

          redeclare function setState_phX "Return thermodynamic state as function of p, h and composition X"
            extends Modelica.Icons.Function;
            input AbsolutePressure p "Pressure";
            input SpecificEnthalpy h "Specific enthalpy";
            input MassFraction X[:] = reference_X "Mass fractions";
            output ThermodynamicState state;
          algorithm
            state := ThermodynamicState(p = p, T = T_h(h));
            annotation(
              Inline = true,
              smoothOrder = 2);
          end setState_phX;

          redeclare function setState_psX "Return thermodynamic state as function of p, s and composition X"
            extends Modelica.Icons.Function;
            input AbsolutePressure p "Pressure";
            input SpecificEntropy s "Specific entropy";
            input MassFraction X[:] = reference_X "Mass fractions";
            output ThermodynamicState state;
          algorithm
            state := ThermodynamicState(p = p, T = T_ps(p, s));
            annotation(
              Inline = true,
              smoothOrder = 2);
          end setState_psX;

          redeclare function setState_dTX "Return thermodynamic state as function of d, T and composition X"
            extends Modelica.Icons.Function;
            input Density d "Density";
            input Temperature T "Temperature";
            input MassFraction X[:] = reference_X "Mass fractions";
            output ThermodynamicState state;
          algorithm
            state := ThermodynamicState(p = d * data.R * T, T = T);
            annotation(
              Inline = true,
              smoothOrder = 2);
          end setState_dTX;

          redeclare function extends setSmoothState "Return thermodynamic state so that it smoothly approximates: if x > 0 then state_a else state_b"
            algorithm
              state := ThermodynamicState(p = Media.Common.smoothStep(x, state_a.p, state_b.p, x_small), T = Media.Common.smoothStep(x, state_a.T, state_b.T, x_small));
            annotation(
              Inline = true,
              smoothOrder = 2);
          end setSmoothState;

          redeclare function extends pressure "Return pressure of ideal gas"
            algorithm
              p := state.p;
            annotation(
              Inline = true,
              smoothOrder = 2);
          end pressure;

          redeclare function extends temperature "Return temperature of ideal gas"
            algorithm
              T := state.T;
            annotation(
              Inline = true,
              smoothOrder = 2);
          end temperature;

          redeclare function extends density "Return density of ideal gas"
            algorithm
              d := state.p / (data.R * state.T);
            annotation(
              Inline = true,
              smoothOrder = 2);
          end density;

          redeclare function extends specificEnthalpy "Return specific enthalpy"
              extends Modelica.Icons.Function;

            algorithm
              h := Modelica.Media.IdealGases.Common.Functions.h_T(data, state.T);
            annotation(
              Inline = true,
              smoothOrder = 2);
          end specificEnthalpy;

          redeclare function extends specificInternalEnergy "Return specific internal energy"
              extends Modelica.Icons.Function;

            algorithm
              u := Modelica.Media.IdealGases.Common.Functions.h_T(data, state.T) - data.R * state.T;
            annotation(
              Inline = true,
              smoothOrder = 2);
          end specificInternalEnergy;

          redeclare function extends specificEntropy "Return specific entropy"
              extends Modelica.Icons.Function;

            algorithm
              s := Modelica.Media.IdealGases.Common.Functions.s0_T(data, state.T) - data.R * Modelica.Math.log(state.p / reference_p);
            annotation(
              Inline = true,
              smoothOrder = 2);
          end specificEntropy;

          redeclare function extends specificGibbsEnergy "Return specific Gibbs energy"
              extends Modelica.Icons.Function;

            algorithm
              g := Modelica.Media.IdealGases.Common.Functions.h_T(data, state.T) - state.T * specificEntropy(state);
            annotation(
              Inline = true,
              smoothOrder = 2);
          end specificGibbsEnergy;

          redeclare function extends specificHelmholtzEnergy "Return specific Helmholtz energy"
              extends Modelica.Icons.Function;

            algorithm
              f := Modelica.Media.IdealGases.Common.Functions.h_T(data, state.T) - data.R * state.T - state.T * specificEntropy(state);
            annotation(
              Inline = true,
              smoothOrder = 2);
          end specificHelmholtzEnergy;

          redeclare function extends specificHeatCapacityCp "Return specific heat capacity at constant pressure"
            algorithm
              cp := Modelica.Media.IdealGases.Common.Functions.cp_T(data, state.T);
            annotation(
              Inline = true,
              smoothOrder = 2);
          end specificHeatCapacityCp;

          redeclare function extends specificHeatCapacityCv "Compute specific heat capacity at constant volume from temperature and gas data"
            algorithm
              cv := Modelica.Media.IdealGases.Common.Functions.cp_T(data, state.T) - data.R;
            annotation(
              Inline = true,
              smoothOrder = 2);
          end specificHeatCapacityCv;

          redeclare function extends isentropicExponent "Return isentropic exponent"
            algorithm
              gamma := specificHeatCapacityCp(state) / specificHeatCapacityCv(state);
            annotation(
              Inline = true,
              smoothOrder = 2);
          end isentropicExponent;

          redeclare function extends velocityOfSound "Return velocity of sound"
              extends Modelica.Icons.Function;

            algorithm
              a := sqrt(max(0, data.R * state.T * Modelica.Media.IdealGases.Common.Functions.cp_T(data, state.T) / specificHeatCapacityCv(state)));
            annotation(
              Inline = true,
              smoothOrder = 2);
          end velocityOfSound;

          function isentropicEnthalpyApproximation "Approximate method of calculating h_is from upstream properties and downstream pressure"
            extends Modelica.Icons.Function;
            input SI.Pressure p2 "Downstream pressure";
            input ThermodynamicState state "Properties at upstream location";
            input Boolean exclEnthForm = Functions.excludeEnthalpyOfFormation "If true, enthalpy of formation Hf is not included in specific enthalpy h";
            input ReferenceEnthalpy refChoice = Functions.referenceChoice "Choice of reference enthalpy";
            input SpecificEnthalpy h_off = Functions.h_offset "User defined offset for reference enthalpy, if referenceChoice = UserDefined";
            output SI.SpecificEnthalpy h_is "Isentropic enthalpy";
          protected
            IsentropicExponent gamma = isentropicExponent(state) "Isentropic exponent";
          algorithm
            h_is := Modelica.Media.IdealGases.Common.Functions.h_T(data, state.T, exclEnthForm, refChoice, h_off) + gamma / (gamma - 1.0) * state.p / density(state) * ((p2 / state.p) ^ ((gamma - 1) / gamma) - 1.0);
            annotation(
              Inline = true,
              smoothOrder = 2);
          end isentropicEnthalpyApproximation;

          redeclare function extends isentropicEnthalpy "Return isentropic enthalpy"
              input Boolean exclEnthForm = Functions.excludeEnthalpyOfFormation "If true, enthalpy of formation Hf is not included in specific enthalpy h";
              input ReferenceEnthalpy refChoice = Functions.referenceChoice "Choice of reference enthalpy";
              input SpecificEnthalpy h_off = Functions.h_offset "User defined offset for reference enthalpy, if referenceChoice = UserDefined";

            algorithm
              h_is := isentropicEnthalpyApproximation(p_downstream, refState, exclEnthForm, refChoice, h_off);
            annotation(
              Inline = true,
              smoothOrder = 2);
          end isentropicEnthalpy;

          redeclare function extends isobaricExpansionCoefficient "Returns overall the isobaric expansion coefficient beta"
            algorithm
              beta := 1 / state.T;
            annotation(
              Inline = true,
              smoothOrder = 2);
          end isobaricExpansionCoefficient;

          redeclare function extends isothermalCompressibility "Returns overall the isothermal compressibility factor"
            algorithm
              kappa := 1.0 / state.p;
            annotation(
              Inline = true,
              smoothOrder = 2);
          end isothermalCompressibility;

          redeclare function extends density_derp_T "Returns the partial derivative of density with respect to pressure at constant temperature"
            algorithm
              ddpT := 1 / (state.T * data.R);
            annotation(
              Inline = true,
              smoothOrder = 2);
          end density_derp_T;

          redeclare function extends density_derT_p "Returns the partial derivative of density with respect to temperature at constant pressure"
            algorithm
              ddTp := -state.p / (state.T * state.T * data.R);
            annotation(
              Inline = true,
              smoothOrder = 2);
          end density_derT_p;

          redeclare function extends density_derX "Returns the partial derivative of density with respect to mass fractions at constant pressure and temperature"
            algorithm
              dddX := fill(0, nX);
            annotation(
              Inline = true,
              smoothOrder = 2);
          end density_derX;

          redeclare replaceable function extends dynamicViscosity "Dynamic viscosity"
            algorithm
              assert(fluidConstants[1].hasCriticalData, "Failed to compute dynamicViscosity: For the species \"" + mediumName + "\" no critical data is available.");
              assert(fluidConstants[1].hasDipoleMoment, "Failed to compute dynamicViscosity: For the species \"" + mediumName + "\" no critical data is available.");
              eta := Modelica.Media.IdealGases.Common.Functions.dynamicViscosityLowPressure(state.T, fluidConstants[1].criticalTemperature, fluidConstants[1].molarMass, fluidConstants[1].criticalMolarVolume, fluidConstants[1].acentricFactor, fluidConstants[1].dipoleMoment);
            annotation(
              smoothOrder = 2);
          end dynamicViscosity;

          redeclare replaceable function extends thermalConductivity "Thermal conductivity of gas"
              //  input IdealGases.Common.DataRecord data "Ideal gas data";
              input Integer method = Functions.methodForThermalConductivity "1: Eucken Method, 2: Modified Eucken Method";

            algorithm
              assert(fluidConstants[1].hasCriticalData, "Failed to compute thermalConductivity: For the species \"" + mediumName + "\" no critical data is available.");
              lambda := Modelica.Media.IdealGases.Common.Functions.thermalConductivityEstimate(specificHeatCapacityCp(state), dynamicViscosity(state), method = method, data = data);
            annotation(
              smoothOrder = 2);
          end thermalConductivity;

          redeclare function extends molarMass "Return the molar mass of the medium"
            algorithm
              MM := data.MM;
            annotation(
              Inline = true,
              smoothOrder = 2);
          end molarMass;

          function T_h "Compute temperature from specific enthalpy"
            extends Modelica.Icons.Function;
            input SpecificEnthalpy h "Specific enthalpy";
            output Temperature T "Temperature";
          protected
            package Internal "Solve h(data,T) for T with given h (use only indirectly via temperature_phX)"
              extends Modelica.Media.Common.OneNonLinearEquation;

              redeclare record extends f_nonlinear_Data "Data to be passed to non-linear function"
                  extends Modelica.Media.IdealGases.Common.DataRecord;
              end f_nonlinear_Data;

              redeclare function extends f_nonlinear
                algorithm
                  y := Modelica.Media.IdealGases.Common.Functions.h_T(f_nonlinear_data, x);
              end f_nonlinear;

              // Dummy definition has to be added for current Dymola

              redeclare function extends solve
              end solve;
            end Internal;
          algorithm
            T := Internal.solve(h, 200, 6000, 1.0e5, {1}, data);
          end T_h;

          function T_ps "Compute temperature from pressure and specific entropy"
            extends Modelica.Icons.Function;
            input AbsolutePressure p "Pressure";
            input SpecificEntropy s "Specific entropy";
            output Temperature T "Temperature";
          protected
            package Internal "Solve h(data,T) for T with given h (use only indirectly via temperature_phX)"
              extends Modelica.Media.Common.OneNonLinearEquation;

              redeclare record extends f_nonlinear_Data "Data to be passed to non-linear function"
                  extends Modelica.Media.IdealGases.Common.DataRecord;
              end f_nonlinear_Data;

              redeclare function extends f_nonlinear
                algorithm
                  y := Modelica.Media.IdealGases.Common.Functions.s0_T(f_nonlinear_data, x) - data.R * Modelica.Math.log(p / reference_p);
              end f_nonlinear;

              // Dummy definition has to be added for current Dymola

              redeclare function extends solve
              end solve;
            end Internal;
          algorithm
            T := Internal.solve(s, 200, 6000, p, {1}, data);
          end T_ps;

          // the functions below are not strictly necessary, there are just here for compatibility reasons

          function dynamicViscosityLowPressure "Dynamic viscosity of low pressure gases"
            extends Modelica.Icons.Function;
            input SI.Temp_K T "Gas temperature";
            input SI.Temp_K Tc "Critical temperature of gas";
            input SI.MolarMass M "Molar mass of gas";
            input SI.MolarVolume Vc "Critical molar volume of gas";
            input Real w "Acentric factor of gas";
            input DipoleMoment mu "Dipole moment of gas molecule";
            input Real k = 0.0 "Special correction for highly polar substances";
            output SI.DynamicViscosity eta "Dynamic viscosity of gas";
          protected
            parameter Real Const1_SI = 40.785 * 10 ^ (-9.5) "Constant in formula for eta converted to SI units";
            parameter Real Const2_SI = 131.3 / 1000.0 "Constant in formula for mur converted to SI units";
            Real mur = Const2_SI * mu / sqrt(Vc * Tc) "Dimensionless dipole moment of gas molecule";
            Real Fc = 1 - 0.2756 * w + 0.059035 * mur ^ 4 + k "Factor to account for molecular shape and polarities of gas";
            Real Tstar "Dimensionless temperature defined by equation below";
            Real Ov "Viscosity collision integral for the gas";
          algorithm
            eta := Functions.dynamicViscosityLowPressure(T, Tc, M, Vc, w, mu, k);
            annotation(
              smoothOrder = 2,
              Documentation(info = "<html>
<p>
The used formula are based on the method of Chung et al (1984, 1988) referred to in ref [1] chapter 9.
The formula 9-4.10 is the one being used. The Formula is given in non-SI units, the following conversion constants were used to
transform the formula to SI units:
</p>

<ul>
<li> <b>Const1_SI:</b> The factor 10^(-9.5) =10^(-2.5)*1e-7 where the
     factor 10^(-2.5) originates from the conversion of g/mol->kg/mol + cm^3/mol->m^3/mol
      and the factor 1e-7 is due to conversion from microPoise->Pa.s.</li>
<li>  <b>Const2_SI:</b> The factor 1/3.335641e-27 = 1e-3/3.335641e-30
      where the factor 3.335641e-30 comes from debye->C.m and
      1e-3 is due to conversion from cm^3/mol->m^3/mol</li>
</ul>

<h4>References:</h4>
<p>
[1] Bruce E. Poling, John E. Prausnitz, John P. O'Connell, \"The Properties of Gases and Liquids\" 5th Ed. Mc Graw Hill.
</p>

<h4>Author</h4>
<p>T. Skoglund, Lund, Sweden, 2004-08-31</p>

</html>"));
          end dynamicViscosityLowPressure;

          function thermalConductivityEstimate "Thermal conductivity of polyatomic gases(Eucken and Modified Eucken correlation)"
            extends Modelica.Icons.Function;
            input Interfaces.PartialMedium.SpecificHeatCapacity Cp "Constant pressure heat capacity";
            input Interfaces.PartialMedium.DynamicViscosity eta "Dynamic viscosity";
            input Integer method(min = 1, max = 2) = 1 "1: Eucken Method, 2: Modified Eucken Method";
            input IdealGases.Common.DataRecord data "Ideal gas data";
            output Interfaces.PartialMedium.ThermalConductivity lambda "Thermal conductivity [W/(m.k)]";
          algorithm
            lambda := Functions.thermalConductivityEstimate(Cp, eta, method, data);
            annotation(
              smoothOrder = 2,
              Documentation(info = "<html>
<p>
This function provides two similar methods for estimating the
thermal conductivity of polyatomic gases.
The Eucken method (input method == 1) gives good results for low temperatures,
but it tends to give an underestimated value of the thermal conductivity
(lambda) at higher temperatures.<br>
The Modified Eucken method (input method == 2) gives good results for
high-temperatures, but it tends to give an overestimated value of the
thermal conductivity (lambda) at low temperatures.
</p>
</html>"));
          end thermalConductivityEstimate;
          annotation(
            Documentation(info = "<HTML>
<p>
This model calculates medium properties
for an ideal gas of a single substance, or for an ideal
gas consisting of several substances where the
mass fractions are fixed. Independent variables
are temperature <b>T</b> and pressure <b>p</b>.
Only density is a function of T and p. All other quantities
are solely a function of T. The properties
are valid in the range:
</p>
<pre>
   200 K &le; T &le; 6000 K
</pre>
<p>
The following quantities are always computed:
</p>
<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"><b>Variable</b></td>
      <td valign=\"top\"><b>Unit</b></td>
      <td valign=\"top\"><b>Description</b></td></tr>
  <tr><td valign=\"top\">h</td>
      <td valign=\"top\">J/kg</td>
      <td valign=\"top\">specific enthalpy h = h(T)</td></tr>
  <tr><td valign=\"top\">u</td>
      <td valign=\"top\">J/kg</td>
      <td valign=\"top\">specific internal energy u = u(T)</td></tr>
  <tr><td valign=\"top\">d</td>
      <td valign=\"top\">kg/m^3</td>
      <td valign=\"top\">density d = d(p,T)</td></tr>
</table>
<p>
For the other variables, see the functions in
Modelica.Media.IdealGases.Common.SingleGasNasa.
Note, dynamic viscosity and thermal conductivity are only provided
for gases that use a data record from Modelica.Media.IdealGases.FluidData.
Currently these are the following gases:
</p>
<pre>
  Ar
  C2H2_vinylidene
  C2H4
  C2H5OH
  C2H6
  C3H6_propylene
  C3H7OH
  C3H8
  C4H8_1_butene
  C4H9OH
  C4H10_n_butane
  C5H10_1_pentene
  C5H12_n_pentane
  C6H6
  C6H12_1_hexene
  C6H14_n_heptane
  C7H14_1_heptene
  C8H10_ethylbenz
  CH3OH
  CH4
  CL2
  CO
  CO2
  F2
  H2
  H2O
  He
  N2
  N2O
  NH3
  NO
  O2
  SO2
  SO3
</pre>
<p>
<b>Sources for model and literature:</b><br>
Original Data: Computer program for calculation of complex chemical
equilibrium compositions and applications. Part 1: Analysis
Document ID: 19950013764 N (95N20180) File Series: NASA Technical Reports
Report Number: NASA-RP-1311  E-8017  NAS 1.61:1311
Authors: Gordon, Sanford (NASA Lewis Research Center)
 Mcbride, Bonnie J. (NASA Lewis Research Center)
Published: Oct 01, 1994.
</p>
<p><b>Known limits of validity:</b></br>
The data is valid for
temperatures between 200K and 6000K.  A few of the data sets for
monatomic gases have a discontinuous 1st derivative at 1000K, but
this never caused problems so far.
</p>
<p>
This model has been copied from the ThermoFluid library
and adapted to the Modelica.Media package.
</p>
</HTML>"));
        end SingleGasNasa;

        partial package MixtureGasNasa "Medium model of a mixture of ideal gases based on NASA source"
          import Modelica.Math;
          import Modelica.Media.Interfaces.Choices.ReferenceEnthalpy;
          extends Modelica.Media.Interfaces.PartialMixtureMedium(ThermoStates = Modelica.Media.Interfaces.Choices.IndependentVariables.pTX, substanceNames = data[:].name, reducedX = false, singleState = false, reference_X = fill(1 / nX, nX), SpecificEnthalpy(start = if referenceChoice == ReferenceEnthalpy.ZeroAt0K then 3e5 else if referenceChoice == ReferenceEnthalpy.UserDefined then h_offset else 0, nominal = 1.0e5), Density(start = 10, nominal = 10), AbsolutePressure(start = 10e5, nominal = 10e5), Temperature(min = 200, max = 6000, start = 500, nominal = 500));

          redeclare record extends ThermodynamicState "Thermodynamic state variables"
          end ThermodynamicState;

          //   redeclare record extends FluidConstants "Fluid constants"
          //   end FluidConstants;
          constant Modelica.Media.IdealGases.Common.DataRecord[:] data "Data records of ideal gas substances";
          // ={Common.SingleGasesData.N2,Common.SingleGasesData.O2}
          constant Boolean excludeEnthalpyOfFormation = true "If true, enthalpy of formation Hf is not included in specific enthalpy h";
          constant ReferenceEnthalpy referenceChoice = ReferenceEnthalpy.ZeroAt0K "Choice of reference enthalpy";
          constant SpecificEnthalpy h_offset = 0.0 "User defined offset for reference enthalpy, if referenceChoice = UserDefined";
          //   constant FluidConstants[nX] fluidConstants
          //     "Additional data needed for transport properties";
          constant MolarMass[nX] MMX = data[:].MM "Molar masses of components";
          constant Integer methodForThermalConductivity(min = 1, max = 2) = 1;

          redeclare replaceable model extends BaseProperties(T(stateSelect = if preferredMediumStates then StateSelect.prefer else StateSelect.default), p(stateSelect = if preferredMediumStates then StateSelect.prefer else StateSelect.default), Xi(each stateSelect = if preferredMediumStates then StateSelect.prefer else StateSelect.default), final standardOrderComponents = true) "Base properties (p, d, T, h, u, R, MM, X, and Xi of NASA mixture gas"
            equation
              assert(T >= 200 and T <= 6000, "
Temperature T (=" + String(T) + " K = 200 K) is not in the allowed range
200 K <= T <= 6000 K
required from medium model \"" + mediumName + "\".");
              MM = molarMass(state);
              h = h_TX(T, X);
              R = data.R * X;
              u = h - R * T;
              d = p / (R * T);
// connect state with BaseProperties
              state.T = T;
              state.p = p;
              state.X = if fixedX then reference_X else X;
          end BaseProperties;

          redeclare function setState_pTX "Return thermodynamic state as function of p, T and composition X"
            extends Modelica.Icons.Function;
            input AbsolutePressure p "Pressure";
            input Temperature T "Temperature";
            input MassFraction X[:] = reference_X "Mass fractions";
            output ThermodynamicState state;
          algorithm
            state := if size(X, 1) == 0 then ThermodynamicState(p = p, T = T, X = reference_X) else if size(X, 1) == nX then ThermodynamicState(p = p, T = T, X = X) else ThermodynamicState(p = p, T = T, X = cat(1, X, {1 - sum(X)}));
            annotation(
              Inline = true,
              smoothOrder = 2);
          end setState_pTX;

          redeclare function setState_phX "Return thermodynamic state as function of p, h and composition X"
            extends Modelica.Icons.Function;
            input AbsolutePressure p "Pressure";
            input SpecificEnthalpy h "Specific enthalpy";
            input MassFraction X[:] = reference_X "Mass fractions";
            output ThermodynamicState state;
          algorithm
            state := if size(X, 1) == 0 then ThermodynamicState(p = p, T = T_hX(h, reference_X), X = reference_X) else if size(X, 1) == nX then ThermodynamicState(p = p, T = T_hX(h, X), X = X) else ThermodynamicState(p = p, T = T_hX(h, X), X = cat(1, X, {1 - sum(X)}));
            annotation(
              Inline = true,
              smoothOrder = 2);
          end setState_phX;

          redeclare function setState_psX "Return thermodynamic state as function of p, s and composition X"
            extends Modelica.Icons.Function;
            input AbsolutePressure p "Pressure";
            input SpecificEntropy s "Specific entropy";
            input MassFraction X[:] = reference_X "Mass fractions";
            output ThermodynamicState state;
          algorithm
            state := if size(X, 1) == 0 then ThermodynamicState(p = p, T = T_psX(p, s, reference_X), X = reference_X) else if size(X, 1) == nX then ThermodynamicState(p = p, T = T_psX(p, s, X), X = X) else ThermodynamicState(p = p, T = T_psX(p, s, X), X = cat(1, X, {1 - sum(X)}));
            annotation(
              Inline = true,
              smoothOrder = 2);
          end setState_psX;

          redeclare function setState_dTX "Return thermodynamic state as function of d, T and composition X"
            extends Modelica.Icons.Function;
            input Density d "Density";
            input Temperature T "Temperature";
            input MassFraction X[:] = reference_X "Mass fractions";
            output ThermodynamicState state;
          algorithm
            state := if size(X, 1) == 0 then ThermodynamicState(p = d * (data.R * reference_X) * T, T = T, X = reference_X) else if size(X, 1) == nX then ThermodynamicState(p = d * (data.R * X) * T, T = T, X = X) else ThermodynamicState(p = d * (data.R * cat(1, X, {1 - sum(X)})) * T, T = T, X = cat(1, X, {1 - sum(X)}));
            annotation(
              Inline = true,
              smoothOrder = 2);
          end setState_dTX;

          redeclare function extends setSmoothState "Return thermodynamic state so that it smoothly approximates: if x > 0 then state_a else state_b"
            algorithm
              state := ThermodynamicState(p = Media.Common.smoothStep(x, state_a.p, state_b.p, x_small), T = Media.Common.smoothStep(x, state_a.T, state_b.T, x_small), X = Media.Common.smoothStep(x, state_a.X, state_b.X, x_small));
            annotation(
              Inline = true,
              smoothOrder = 2);
          end setSmoothState;

          redeclare function extends pressure "Return pressure of ideal gas"
            algorithm
              p := state.p;
            annotation(
              Inline = true,
              smoothOrder = 2);
          end pressure;

          redeclare function extends temperature "Return temperature of ideal gas"
            algorithm
              T := state.T;
            annotation(
              Inline = true,
              smoothOrder = 2);
          end temperature;

          redeclare function extends density "Return density of ideal gas"
            algorithm
              d := state.p / (state.X * data.R * state.T);
            annotation(
              Inline = true,
              smoothOrder = 3);
          end density;

          redeclare function extends specificEnthalpy "Return specific enthalpy"
              extends Modelica.Icons.Function;

            algorithm
              h := h_TX(state.T, state.X);
            annotation(
              Inline = true,
              smoothOrder = 2);
          end specificEnthalpy;

          redeclare function extends specificInternalEnergy "Return specific internal energy"
              extends Modelica.Icons.Function;

            algorithm
              u := h_TX(state.T, state.X) - gasConstant(state) * state.T;
            annotation(
              Inline = true,
              smoothOrder = 2);
          end specificInternalEnergy;

          redeclare function extends specificEntropy "Return specific entropy"
            protected
              Real[nX] Y(unit = "mol/mol") = massToMoleFractions(state.X, data.MM) "Molar fractions";

            algorithm
              s := s_TX(state.T, state.X) - sum(state.X[i] * Modelica.Constants.R / MMX[i] * (if state.X[i] < Modelica.Constants.eps then Y[i] else Modelica.Math.log(Y[i] * state.p / reference_p)) for i in 1:nX);
            annotation(
              Inline = true,
              smoothOrder = 2);
          end specificEntropy;

          redeclare function extends specificGibbsEnergy "Return specific Gibbs energy"
              extends Modelica.Icons.Function;

            algorithm
              g := h_TX(state.T, state.X) - state.T * specificEntropy(state);
            annotation(
              Inline = true,
              smoothOrder = 2);
          end specificGibbsEnergy;

          redeclare function extends specificHelmholtzEnergy "Return specific Helmholtz energy"
              extends Modelica.Icons.Function;

            algorithm
              f := h_TX(state.T, state.X) - gasConstant(state) * state.T - state.T * specificEntropy(state);
            annotation(
              Inline = true,
              smoothOrder = 2);
          end specificHelmholtzEnergy;

          function h_TX "Return specific enthalpy"
            import Modelica.Media.Interfaces.Choices;
            extends Modelica.Icons.Function;
            input SI.Temperature T "Temperature";
            input MassFraction X[:] = reference_X "Independent Mass fractions of gas mixture";
            input Boolean exclEnthForm = excludeEnthalpyOfFormation "If true, enthalpy of formation Hf is not included in specific enthalpy h";
            input Modelica.Media.Interfaces.Choices.ReferenceEnthalpy refChoice = referenceChoice "Choice of reference enthalpy";
            input SI.SpecificEnthalpy h_off = h_offset "User defined offset for reference enthalpy, if referenceChoice = UserDefined";
            output SI.SpecificEnthalpy h "Specific enthalpy at temperature T";
          algorithm
            h := (if fixedX then reference_X else X) * {Modelica.Media.IdealGases.Common.Functions.h_T(data[i], T, exclEnthForm, refChoice, h_off) for i in 1:nX};
            annotation(
              Inline = false,
              smoothOrder = 2);
          end h_TX;

          function h_TX_der "Return specific enthalpy derivative"
            import Modelica.Media.Interfaces.Choices;
            extends Modelica.Icons.Function;
            input SI.Temperature T "Temperature";
            input MassFraction X[nX] "Independent Mass fractions of gas mixture";
            input Boolean exclEnthForm = excludeEnthalpyOfFormation "If true, enthalpy of formation Hf is not included in specific enthalpy h";
            input Modelica.Media.Interfaces.Choices.ReferenceEnthalpy refChoice = referenceChoice "Choice of reference enthalpy";
            input SI.SpecificEnthalpy h_off = h_offset "User defined offset for reference enthalpy, if referenceChoice = UserDefined";
            input Real dT "Temperature derivative";
            input Real dX[nX] "Independent mass fraction derivative";
            output Real h_der "Specific enthalpy at temperature T";
          algorithm
            h_der := if fixedX then dT * sum(Modelica.Media.IdealGases.Common.Functions.cp_T(data[i], T) * reference_X[i] for i in 1:nX) else dT * sum(Modelica.Media.IdealGases.Common.Functions.cp_T(data[i], T) * X[i] for i in 1:nX) + sum(Modelica.Media.IdealGases.Common.Functions.h_T(data[i], T) * dX[i] for i in 1:nX);
            annotation(
              Inline = false,
              smoothOrder = 1);
          end h_TX_der;

          redeclare function extends gasConstant "Return gasConstant"
            algorithm
              R := data.R * state.X;
            annotation(
              Inline = true,
              smoothOrder = 3);
          end gasConstant;

          redeclare function extends specificHeatCapacityCp "Return specific heat capacity at constant pressure"
            algorithm
              cp := {Modelica.Media.IdealGases.Common.Functions.cp_T(data[i], state.T) for i in 1:nX} * state.X;
            annotation(
              Inline = true,
              smoothOrder = 1);
          end specificHeatCapacityCp;

          redeclare function extends specificHeatCapacityCv "Return specific heat capacity at constant volume from temperature and gas data"
            algorithm
              cv := {Modelica.Media.IdealGases.Common.Functions.cp_T(data[i], state.T) for i in 1:nX} * state.X - data.R * state.X;
            annotation(
              Inline = true,
              smoothOrder = 1);
          end specificHeatCapacityCv;

          function MixEntropy "Return mixing entropy of ideal gases / R"
            extends Modelica.Icons.Function;
            input SI.MoleFraction x[:] "Mole fraction of mixture";
            output Real smix "Mixing entropy contribution, divided by gas constant";
          algorithm
            smix := sum(if x[i] > Modelica.Constants.eps then -x[i] * Modelica.Math.log(x[i]) else x[i] for i in 1:size(x, 1));
            annotation(
              Inline = true,
              smoothOrder = 2);
          end MixEntropy;

          function s_TX "Return temperature dependent part of the entropy, expects full entropy vector"
            extends Modelica.Icons.Function;
            input Temperature T "Temperature";
            input MassFraction[nX] X "Mass fraction";
            output SpecificEntropy s "Specific entropy";
          algorithm
            s := sum(Modelica.Media.IdealGases.Common.Functions.s0_T(data[i], T) * X[i] for i in 1:size(X, 1));
            annotation(
              Inline = true,
              smoothOrder = 2);
          end s_TX;

          redeclare function extends isentropicExponent "Return isentropic exponent"
            algorithm
              gamma := specificHeatCapacityCp(state) / specificHeatCapacityCv(state);
            annotation(
              Inline = true,
              smoothOrder = 2);
          end isentropicExponent;

          redeclare function extends velocityOfSound "Return velocity of sound"
              extends Modelica.Icons.Function;
              input ThermodynamicState state "Properties at upstream location";

            algorithm
              a := sqrt(max(0, gasConstant(state) * state.T * specificHeatCapacityCp(state) / specificHeatCapacityCv(state)));
            annotation(
              Inline = true,
              smoothOrder = 2);
          end velocityOfSound;

          function isentropicEnthalpyApproximation "Approximate method of calculating h_is from upstream properties and downstream pressure"
            extends Modelica.Icons.Function;
            input AbsolutePressure p2 "Downstream pressure";
            input ThermodynamicState state "Thermodynamic state at upstream location";
            output SpecificEnthalpy h_is "Isentropic enthalpy";
          protected
            SpecificEnthalpy h "Specific enthalpy at upstream location";
            SpecificEnthalpy h_component[nX] "Specific enthalpy at upstream location";
            IsentropicExponent gamma = isentropicExponent(state) "Isentropic exponent";
          protected
            MassFraction[nX] X "Complete X-vector";
          algorithm
            X := if reducedX then cat(1, state.X, {1 - sum(state.X)}) else state.X;
            h_component := {Modelica.Media.IdealGases.Common.Functions.h_T(data[i], state.T, excludeEnthalpyOfFormation, referenceChoice, h_offset) for i in 1:nX};
            h := h_component * X;
            h_is := h + gamma / (gamma - 1.0) * (state.T * gasConstant(state)) * ((p2 / state.p) ^ ((gamma - 1) / gamma) - 1.0);
            annotation(
              smoothOrder = 2);
          end isentropicEnthalpyApproximation;

          redeclare function extends isentropicEnthalpy "Return isentropic enthalpy"
              input Boolean exact = false "Flag whether exact or approximate version should be used";

            algorithm
              h_is := if exact then specificEnthalpy_psX(p_downstream, specificEntropy(refState), refState.X) else isentropicEnthalpyApproximation(p_downstream, refState);
            annotation(
              Inline = true,
              smoothOrder = 2);
          end isentropicEnthalpy;

          function gasMixtureViscosity "Return viscosities of gas mixtures at low pressures (Wilke method)"
            extends Modelica.Icons.Function;
            input MoleFraction[:] yi "Mole fractions";
            input MolarMass[:] M "Mole masses";
            input DynamicViscosity[:] eta "Pure component viscosities";
            output DynamicViscosity etam "Viscosity of the mixture";
          protected
            Real fi[size(yi, 1), size(yi, 1)];
          algorithm
            for i in 1:size(eta, 1) loop
              assert(fluidConstants[i].hasDipoleMoment, "Dipole moment for " + fluidConstants[i].chemicalFormula + " not known. Can not compute viscosity.");
              assert(fluidConstants[i].hasCriticalData, "Critical data for " + fluidConstants[i].chemicalFormula + " not known. Can not compute viscosity.");
              for j in 1:size(eta, 1) loop
                if i == 1 then
                  fi[i, j] := (1 + (eta[i] / eta[j]) ^ (1 / 2) * (M[j] / M[i]) ^ (1 / 4)) ^ 2 / (8 * (1 + M[i] / M[j])) ^ (1 / 2);
                elseif j < i then
                  fi[i, j] := eta[i] / eta[j] * M[j] / M[i] * fi[j, i];
                else
                  fi[i, j] := (1 + (eta[i] / eta[j]) ^ (1 / 2) * (M[j] / M[i]) ^ (1 / 4)) ^ 2 / (8 * (1 + M[i] / M[j])) ^ (1 / 2);
                end if;
              end for;
            end for;
            etam := sum(yi[i] * eta[i] / sum(yi[j] * fi[i, j] for j in 1:size(eta, 1)) for i in 1:size(eta, 1));
            annotation(
              smoothOrder = 2,
              Documentation(info = "<html>

<p>
Simplification of the kinetic theory (Chapman and Enskog theory)
approach neglecting the second-order effects.<br>
<br>
This equation has been extensively tested (Amdur and Mason, 1958;
Bromley and Wilke, 1951; Cheung, 1958; Dahler, 1959; Gandhi and Saxena,
1964; Ranz and Brodowsky, 1962; Saxena and Gambhir, 1963a; Strunk, et
al., 1964; Vanderslice, et al. 1962; Wright and Gray, 1962). In most
cases, only nonpolar mixtures were compared, and very good results
obtained. For some systems containing hydrogen as one component, less
satisfactory agreement was noted. Wilke's method predicted mixture
viscosities that were larger than experimental for the H2-N2 system,
but for H2-NH3, it underestimated the viscosities. <br>
Gururaja, et al. (1967) found that this method also overpredicted in
the H2-O2 case but was quite accurate for the H2-CO2 system. <br>
Wilke's approximation has proved reliable even for polar-polar gas
mixtures of aliphatic alcohols (Reid and Belenyessy, 1960). The
principal reservation appears to lie in those cases where Mi&gt;&gt;Mj
and etai&gt;&gt;etaj.<br>
</p>

</html>"));
          end gasMixtureViscosity;

          redeclare replaceable function extends dynamicViscosity "Return mixture dynamic viscosity"
            protected
              DynamicViscosity[nX] etaX "Component dynamic viscosities";

            algorithm
              for i in 1:nX loop
                etaX[i] := Modelica.Media.IdealGases.Common.Functions.dynamicViscosityLowPressure(state.T, fluidConstants[i].criticalTemperature, fluidConstants[i].molarMass, fluidConstants[i].criticalMolarVolume, fluidConstants[i].acentricFactor, fluidConstants[i].dipoleMoment);
              end for;
              eta := gasMixtureViscosity(massToMoleFractions(state.X, fluidConstants[:].molarMass), fluidConstants[:].molarMass, etaX);
            annotation(
              smoothOrder = 2);
          end dynamicViscosity;

          function mixtureViscosityChung "Return the viscosity of gas mixtures without access to component viscosities (Chung, et. al. rules)"
            extends Modelica.Icons.Function;
            input Temperature T "Temperature";
            input Temperature[:] Tc "Critical temperatures";
            input MolarVolume[:] Vcrit "Critical volumes (m3/mol)";
            input Real[:] w "Acentric factors";
            input Real[:] mu "Dipole moments (debyes)";
            input MolarMass[:] MolecularWeights "Molecular weights (kg/mol)";
            input MoleFraction[:] y "Molar Fractions";
            input Real[:] kappa = zeros(nX) "Association Factors";
            output DynamicViscosity etaMixture "Mixture viscosity (Pa.s)";
          protected
            constant Real[size(y, 1)] Vc = Vcrit * 1000000 "Critical volumes (cm3/mol)";
            constant Real[size(y, 1)] M = MolecularWeights * 1000 "Molecular weights (g/mol)";
            Integer n = size(y, 1) "Number of mixed elements";
            Real sigmam3 "Mixture sigma3 in Angstrom";
            Real sigma[size(y, 1), size(y, 1)];
            Real edivkm;
            Real edivk[size(y, 1), size(y, 1)];
            Real Mm;
            Real Mij[size(y, 1), size(y, 1)];
            Real wm "Accentric factor";
            Real wij[size(y, 1), size(y, 1)];
            Real kappam "Correlation for highly polar substances such as alcohols and acids";
            Real kappaij[size(y, 1), size(y, 1)];
            Real mum;
            Real Vcm;
            Real Tcm;
            Real murm "Dimensionless dipole moment of the mixture";
            Real Fcm "Factor to correct for shape and polarity";
            Real omegav;
            Real Tmstar;
            Real etam "Mixture viscosity in microP";
          algorithm
//combining rules
            for i in 1:n loop
              for j in 1:n loop
                Mij[i, j] := 2 * M[i] * M[j] / (M[i] + M[j]);
                if i == j then
                  sigma[i, j] := 0.809 * Vc[i] ^ (1 / 3);
                  edivk[i, j] := Tc[i] / 1.2593;
                  wij[i, j] := w[i];
                  kappaij[i, j] := kappa[i];
                else
                  sigma[i, j] := (0.809 * Vc[i] ^ (1 / 3) * 0.809 * Vc[j] ^ (1 / 3)) ^ (1 / 2);
                  edivk[i, j] := (Tc[i] / 1.2593 * Tc[j] / 1.2593) ^ (1 / 2);
                  wij[i, j] := (w[i] + w[j]) / 2;
                  kappaij[i, j] := (kappa[i] * kappa[j]) ^ (1 / 2);
                end if;
              end for;
            end for;
//mixing rules
            sigmam3 := sum(sum(y[i] * y[j] * sigma[i, j] ^ 3 for j in 1:n) for i in 1:n);
//(epsilon/k)m
            edivkm := sum(sum(y[i] * y[j] * edivk[i, j] * sigma[i, j] ^ 3 for j in 1:n) for i in 1:n) / sigmam3;
            Mm := (sum(sum(y[i] * y[j] * edivk[i, j] * sigma[i, j] ^ 2 * Mij[i, j] ^ (1 / 2) for j in 1:n) for i in 1:n) / (edivkm * sigmam3 ^ (2 / 3))) ^ 2;
            wm := sum(sum(y[i] * y[j] * wij[i, j] * sigma[i, j] ^ 3 for j in 1:n) for i in 1:n) / sigmam3;
            mum := (sigmam3 * sum(sum(y[i] * y[j] * mu[i] ^ 2 * mu[j] ^ 2 / sigma[i, j] ^ 3 for j in 1:n) for i in 1:n)) ^ (1 / 4);
            Vcm := sigmam3 / 0.809 ^ 3;
            Tcm := 1.2593 * edivkm;
            murm := 131.3 * mum / (Vcm * Tcm) ^ (1 / 2);
            kappam := sigmam3 * sum(sum(y[i] * y[j] * kappaij[i, j] for j in 1:n) for i in 1:n);
            Fcm := 1 - 0.275 * wm + 0.059035 * murm ^ 4 + kappam;
            Tmstar := T / edivkm;
            omegav := 1.16145 * Tmstar ^ (-0.14874) + 0.52487 * Math.exp(-0.77320 * Tmstar) + 2.16178 * Math.exp(-2.43787 * Tmstar);
            etam := 26.69 * Fcm * (Mm * T) ^ (1 / 2) / (sigmam3 ^ (2 / 3) * omegav);
            etaMixture := etam * 1e7;
            annotation(
              smoothOrder = 2,
              Documentation(info = "<html>

<p>
Equation to estimate the viscosity of gas mixtures at low pressures.<br>
It is a simplification of an extension of the rigorous kinetic theory
of Chapman and Enskog to determine the viscosity of multicomponent
mixtures, at low pressures and with a factor to correct for molecule
shape and polarity.
</p>

<p>
The input argument Kappa is a special correction for highly polar substances such as
alcohols and acids.<br>
Values of kappa for a few such materials:
</p>

<table style=\"text-align: left; width: 302px; height: 200px;\" border=\"1\"
cellspacing=\"0\" cellpadding=\"2\">
<tbody>
<tr>
<td style=\"vertical-align: top;\">Compound <br>
</td>
<td style=\"vertical-align: top; text-align: center;\">Kappa<br>
</td>
<td style=\"vertical-align: top;\">Compound<br>
</td>
<td style=\"vertical-align: top;\">Kappa<br>
</td>
</tr>
<tr>
<td style=\"vertical-align: top;\">Methanol<br>
</td>
<td style=\"vertical-align: top;\">0.215<br>
</td>
<td style=\"vertical-align: top;\">n-Pentanol<br>
</td>
<td style=\"vertical-align: top;\">0.122<br>
</td>
</tr>
<tr>
<td style=\"vertical-align: top;\">Ethanol<br>
</td>
<td style=\"vertical-align: top;\">0.175<br>
</td>
<td style=\"vertical-align: top;\">n-Hexanol<br>
</td>
<td style=\"vertical-align: top;\">0.114<br>
</td>
</tr>
<tr>
<td style=\"vertical-align: top;\">n-Propanol<br>
</td>
<td style=\"vertical-align: top;\">0.143<br>
</td>
<td style=\"vertical-align: top;\">n-Heptanol<br>
</td>
<td style=\"vertical-align: top;\">0.109<br>
</td>
</tr>
<tr>
<td style=\"vertical-align: top;\">i-Propanol<br>
</td>
<td style=\"vertical-align: top;\">0.143<br>
</td>
<td style=\"vertical-align: top;\">Acetic Acid<br>
</td>
<td style=\"vertical-align: top;\">0.0916<br>
</td>
</tr>
<tr>
<td style=\"vertical-align: top;\">n-Butanol<br>
</td>
<td style=\"vertical-align: top;\">0.132<br>
</td>
<td style=\"vertical-align: top;\">Water<br>
</td>
<td style=\"vertical-align: top;\">0.076<br>
</td>
</tr>
<tr>
<td style=\"vertical-align: top;\">i-Butanol<br>
</td>
<td style=\"vertical-align: top;\">0.132</td>
<td style=\"vertical-align: top;\"><br>
</td>
<td style=\"vertical-align: top;\"><br>
</td>
</tr>
</tbody>
</table>
<p>
Chung, et al. (1984) suggest that for other alcohols not shown in the
table:<br>
&nbsp;&nbsp;&nbsp;&nbsp; <br>
&nbsp;&nbsp;&nbsp; kappa = 0.0682 + 4.704*[(number of -OH
groups)]/[molecular weight]<br>
<br>
<span style=\"font-weight: normal;\">S.I. units relation for the
debyes:&nbsp;</span><br>
&nbsp;&nbsp;&nbsp; &nbsp;&nbsp; &nbsp;&nbsp; &nbsp;&nbsp; &nbsp;&nbsp;
&nbsp;&nbsp; &nbsp;&nbsp; &nbsp;&nbsp; &nbsp;&nbsp; &nbsp;&nbsp;
&nbsp;&nbsp; &nbsp;&nbsp; &nbsp;&nbsp; &nbsp;&nbsp; &nbsp;&nbsp;
&nbsp;&nbsp; &nbsp;&nbsp; &nbsp;&nbsp; 1 debye = 3.162e-25 (J.m^3)^(1/2)<br>
</p>
<h4>References</h4>
<p>
[1] THE PROPERTIES OF GASES AND LIQUIDS, Fifth Edition,<br>
&nbsp;&nbsp;&nbsp; &nbsp;&nbsp;&nbsp; &nbsp; Bruce E. Poling, John M.
Prausnitz, John P. O'Connell.<br>
[2] Chung, T.-H., M. Ajlan, L. L. Lee, and K. E. Starling: Ind. Eng.
Chem. Res., 27: 671 (1988).<br>
[3] Chung, T.-H., L. L. Lee, and K. E. Starling; Ing. Eng. Chem.
Fundam., 23: 3 ()1984).<br>
</p>
</html>"));
          end mixtureViscosityChung;

          function lowPressureThermalConductivity "Return thermal conductivities of low-pressure gas mixtures (Mason and Saxena Modification)"
            extends Modelica.Icons.Function;
            input MoleFraction[:] y "Mole fraction of the components in the gas mixture";
            input Temperature T "Temperature";
            input Temperature[:] Tc "Critical temperatures";
            input AbsolutePressure[:] Pc "Critical pressures";
            input MolarMass[:] M "Molecular weights";
            input ThermalConductivity[:] lambda "Thermal conductivities of the pure gases";
            output ThermalConductivity lambdam "Thermal conductivity of the gas mixture";
          protected
            MolarMass[size(y, 1)] gamma;
            Real[size(y, 1)] Tr "Reduced temperature";
            Real[size(y, 1), size(y, 1)] A "Mason and Saxena Modification";
            constant Real epsilon = 1.0 "Numerical constant near unity";
          algorithm
            for i in 1:size(y, 1) loop
              gamma[i] := 210 * (Tc[i] * M[i] ^ 3 / Pc[i] ^ 4) ^ (1 / 6);
              Tr[i] := T / Tc[i];
            end for;
            for i in 1:size(y, 1) loop
              for j in 1:size(y, 1) loop
                A[i, j] := epsilon * (1 + (gamma[j] * (Math.exp(0.0464 * Tr[i]) - Math.exp(-0.2412 * Tr[i])) / (gamma[i] * (Math.exp(0.0464 * Tr[j]) - Math.exp(-0.2412 * Tr[j])))) ^ (1 / 2) * (M[i] / M[j]) ^ (1 / 4)) ^ 2 / (8 * (1 + M[i] / M[j])) ^ (1 / 2);
              end for;
            end for;
            lambdam := sum(y[i] * lambda[i] / sum(y[j] * A[i, j] for j in 1:size(y, 1)) for i in 1:size(y, 1));
            annotation(
              smoothOrder = 2,
              Documentation(info = "<html>

<p>
This function applies the Masson and Saxena modification of the
Wassiljewa Equation for the thermal conductivity for gas mixtures of
n elements at low pressure.
</p>

<p>
For nonpolar gas mixtures errors will generally be less than 3 to 4%.
For mixtures of nonpolar-polar and polar-polar gases, errors greater
than 5 to 8% may be expected. For mixtures in which the sizes and
polarities of the constituent molecules are not greatly different, the
thermal conductivity can be estimated satisfactorily by a mole fraction
average of the pure component conductivities.
</p>

</html>"));
          end lowPressureThermalConductivity;

          redeclare replaceable function extends thermalConductivity "Return thermal conductivity for low pressure gas mixtures"
              input Integer method = methodForThermalConductivity "Method to compute single component thermal conductivity";

            protected
              ThermalConductivity[nX] lambdaX "Component thermal conductivities";
              DynamicViscosity[nX] eta "Component thermal dynamic viscosities";
              SpecificHeatCapacity[nX] cp "Component heat capacity";

            algorithm
              for i in 1:nX loop
                assert(fluidConstants[i].hasCriticalData, "Critical data for " + fluidConstants[i].chemicalFormula + " not known. Can not compute thermal conductivity.");
                eta[i] := Modelica.Media.IdealGases.Common.Functions.dynamicViscosityLowPressure(state.T, fluidConstants[i].criticalTemperature, fluidConstants[i].molarMass, fluidConstants[i].criticalMolarVolume, fluidConstants[i].acentricFactor, fluidConstants[i].dipoleMoment);
                cp[i] := Modelica.Media.IdealGases.Common.Functions.cp_T(data[i], state.T);
                lambdaX[i] := Modelica.Media.IdealGases.Common.Functions.thermalConductivityEstimate(Cp = cp[i], eta = eta[i], method = method, data = data[i]);
              end for;
              lambda := lowPressureThermalConductivity(massToMoleFractions(state.X, fluidConstants[:].molarMass), state.T, fluidConstants[:].criticalTemperature, fluidConstants[:].criticalPressure, fluidConstants[:].molarMass, lambdaX);
            annotation(
              smoothOrder = 2);
          end thermalConductivity;

          redeclare function extends isobaricExpansionCoefficient "Return isobaric expansion coefficient beta"
            algorithm
              beta := 1 / state.T;
            annotation(
              Inline = true,
              smoothOrder = 2);
          end isobaricExpansionCoefficient;

          redeclare function extends isothermalCompressibility "Return isothermal compressibility factor"
            algorithm
              kappa := 1.0 / state.p;
            annotation(
              Inline = true,
              smoothOrder = 2);
          end isothermalCompressibility;

          redeclare function extends density_derp_T "Return density derivative by pressure at constant temperature"
            algorithm
              ddpT := 1 / (state.T * gasConstant(state));
            annotation(
              Inline = true,
              smoothOrder = 2);
          end density_derp_T;

          redeclare function extends density_derT_p "Return density derivative by temperature at constant pressure"
            algorithm
              ddTp := -state.p / (state.T * state.T * gasConstant(state));
            annotation(
              Inline = true,
              smoothOrder = 2);
          end density_derT_p;

          redeclare function density_derX "Return density derivative by mass fraction"
            extends Modelica.Icons.Function;
            input ThermodynamicState state "Thermodynamic state record";
            output Density[nX] dddX "Derivative of density w.r.t. mass fraction";
          algorithm
            dddX := {-state.p / (state.T * gasConstant(state)) * molarMass(state) / data[i].MM for i in 1:nX};
            annotation(
              Inline = true,
              smoothOrder = 2);
          end density_derX;

          redeclare function extends molarMass "Return molar mass of mixture"
            algorithm
              MM := 1 / sum(state.X[j] / data[j].MM for j in 1:size(state.X, 1));
            annotation(
              Inline = true,
              smoothOrder = 2);
          end molarMass;

          function T_hX "Return temperature from specific enthalpy and mass fraction"
            extends Modelica.Icons.Function;
            input SpecificEnthalpy h "Specific enthalpy";
            input MassFraction[:] X "Mass fractions of composition";
            input Boolean exclEnthForm = excludeEnthalpyOfFormation "If true, enthalpy of formation Hf is not included in specific enthalpy h";
            input Modelica.Media.Interfaces.Choices.ReferenceEnthalpy refChoice = referenceChoice "Choice of reference enthalpy";
            input SI.SpecificEnthalpy h_off = h_offset "User defined offset for reference enthalpy, if referenceChoice = UserDefined";
            output Temperature T "Temperature";
          protected
            MassFraction[nX] Xfull = if size(X, 1) == nX then X else cat(1, X, {1 - sum(X)});

            package Internal "Solve h(data,T) for T with given h (use only indirectly via temperature_phX)"
              extends Modelica.Media.Common.OneNonLinearEquation;

              redeclare record extends f_nonlinear_Data "Data to be passed to non-linear function"
                  extends Modelica.Media.IdealGases.Common.DataRecord;
              end f_nonlinear_Data;

              redeclare function extends f_nonlinear
                algorithm
                  y := h_TX(x, X);
              end f_nonlinear;

              // Dummy definition has to be added for current Dymola

              redeclare function extends solve
              end solve;
            end Internal;
          algorithm
            T := Internal.solve(h, 200, 6000, 1.0e5, Xfull, data[1]);
            annotation(
              inverse(h = h_TX(T, X, exclEnthForm, refChoice, h_off)));
          end T_hX;

          function T_psX "Return temperature from pressure, specific entropy and mass fraction"
            extends Modelica.Icons.Function;
            input AbsolutePressure p "Pressure";
            input SpecificEntropy s "Specific entropy";
            input MassFraction[:] X "Mass fractions of composition";
            output Temperature T "Temperature";
          protected
            MassFraction[nX] Xfull = if size(X, 1) == nX then X else cat(1, X, {1 - sum(X)});

            package Internal "Solve h(data,T) for T with given h (use only indirectly via temperature_phX)"
              extends Modelica.Media.Common.OneNonLinearEquation;

              redeclare record extends f_nonlinear_Data "Data to be passed to non-linear function"
                  extends Modelica.Media.IdealGases.Common.DataRecord;
              end f_nonlinear_Data;

              redeclare function extends f_nonlinear "Note that this function always sees the complete mass fraction vector"
                protected
                  MassFraction[nX] Xfull = if size(X, 1) == nX then X else cat(1, X, {1 - sum(X)});
                  Real[nX] Y(unit = "mol/mol") = massToMoleFractions(if size(X, 1) == nX then X else cat(1, X, {1 - sum(X)}), data.MM) "Molar fractions";

                algorithm
                  y := s_TX(x, Xfull) - sum(Xfull[i] * Modelica.Constants.R / MMX[i] * (if Xfull[i] < Modelica.Constants.eps then Y[i] else Modelica.Math.log(Y[i] * p / reference_p)) for i in 1:nX);
// s_TX(x,X)- data[:].R*X*(Modelica.Math.log(p/reference_p)
//       + MixEntropy(massToMoleFractions(X,data[:].MM)));
              end f_nonlinear;

              // Dummy definition has to be added for current Dymola

              redeclare function extends solve
              end solve;
            end Internal;
          algorithm
            T := Internal.solve(s, 200, 6000, p, Xfull, data[1]);
          end T_psX;

          //   redeclare function extends specificEnthalpy_psX
          //   protected
          //     Temperature T "Temperature";
          //   algorithm
          //     T := temperature_psX(p,s,X);
          //     h := specificEnthalpy_pTX(p,T,X);
          //   end extends;
          //   redeclare function extends density_phX
          //     "Compute density from pressure, specific enthalpy and mass fraction"
          //     protected
          //     Temperature T "Temperature";
          //     SpecificHeatCapacity R "Gas constant";
          //   algorithm
          //     T := temperature_phX(p,h,X);
          //     R := if (not reducedX) then
          //       sum(data[i].R*X[i] for i in 1:size(substanceNames, 1)) else
          //       sum(data[i].R*X[i] for i in 1:size(substanceNames, 1)-1) + data[end].R*(1-sum(X[i]));
          //     d := p/(R*T);
          //   end density_phX;
          annotation(
            Documentation(info = "<HTML>
<p>
This model calculates the medium properties for single component ideal gases.
</p>
<p>
<b>Sources for model and literature:</b><br>
Original Data: Computer program for calculation of complex chemical
equilibrium compositions and applications. Part 1: Analysis
Document ID: 19950013764 N (95N20180) File Series: NASA Technical Reports
Report Number: NASA-RP-1311  E-8017  NAS 1.61:1311
Authors: Gordon, Sanford (NASA Lewis Research Center)
 Mcbride, Bonnie J. (NASA Lewis Research Center)
Published: Oct 01, 1994.
</p>
<p><b>Known limits of validity:</b></br>
The data is valid for
temperatures between 200 K and 6000 K.  A few of the data sets for
monatomic gases have a discontinuous 1st derivative at 1000 K, but
this never caused problems so far.
</p>
<p>
This model has been copied from the ThermoFluid library.
It has been developed by Hubertus Tummescheit.
</p>
</HTML>"));
        end MixtureGasNasa;

        package FluidData "Critical data, dipole moments and related data"
          extends Modelica.Icons.Package;
          import Modelica.Media.Interfaces.PartialMixtureMedium;
          import Modelica.Media.IdealGases.Common.SingleGasesData;
          constant Modelica.Media.Interfaces.Types.IdealGas.FluidConstants N2(chemicalFormula = "N2", iupacName = "unknown", structureFormula = "unknown", casRegistryNumber = "7727-37-9", meltingPoint = 63.15, normalBoilingPoint = 77.35, criticalTemperature = 126.20, criticalPressure = 33.98e5, criticalMolarVolume = 90.10e-6, acentricFactor = 0.037, dipoleMoment = 0.0, molarMass = SingleGasesData.N2.MM, hasDipoleMoment = true, hasIdealGasHeatCapacity = true, hasCriticalData = true, hasAcentricFactor = true);
          constant Modelica.Media.Interfaces.Types.IdealGas.FluidConstants O2(chemicalFormula = "O2", iupacName = "unknown", structureFormula = "unknown", casRegistryNumber = "7782-44-7", meltingPoint = 54.36, normalBoilingPoint = 90.17, criticalTemperature = 154.58, criticalPressure = 50.43e5, criticalMolarVolume = 73.37e-6, acentricFactor = 0.022, dipoleMoment = 0.0, molarMass = SingleGasesData.O2.MM, hasDipoleMoment = true, hasIdealGasHeatCapacity = true, hasCriticalData = true, hasAcentricFactor = true);
          constant Modelica.Media.Interfaces.Types.IdealGas.FluidConstants CL2(chemicalFormula = "CL2", iupacName = "unknown", structureFormula = "unknown", casRegistryNumber = "7782-50-5", meltingPoint = 172.19, normalBoilingPoint = 239.12, criticalTemperature = 417.00, criticalPressure = 77.00e5, criticalMolarVolume = 124.00e-6, acentricFactor = 0.069, dipoleMoment = 0.0, molarMass = SingleGasesData.CL2.MM, hasDipoleMoment = true, hasIdealGasHeatCapacity = true, hasCriticalData = true, hasAcentricFactor = true);
          constant Modelica.Media.Interfaces.Types.IdealGas.FluidConstants F2(chemicalFormula = "F2", iupacName = "unknown", structureFormula = "unknown", casRegistryNumber = "7782-41-4", meltingPoint = 53.48, normalBoilingPoint = 84.95, criticalTemperature = 144.30, criticalPressure = 52.15e5, criticalMolarVolume = 66.20e-6, acentricFactor = 0.051, dipoleMoment = 0.0, molarMass = SingleGasesData.F2.MM, hasDipoleMoment = true, hasIdealGasHeatCapacity = true, hasCriticalData = true, hasAcentricFactor = true);
          constant Modelica.Media.Interfaces.Types.IdealGas.FluidConstants CO2(chemicalFormula = "CO2", iupacName = "unknown", structureFormula = "unknown", casRegistryNumber = "124-38-9", meltingPoint = 216.58, normalBoilingPoint = -1.0, criticalTemperature = 304.12, criticalPressure = 73.74e5, criticalMolarVolume = 94.07e-6, acentricFactor = 0.225, dipoleMoment = 0.0, molarMass = SingleGasesData.CO2.MM, hasDipoleMoment = true, hasIdealGasHeatCapacity = true, hasCriticalData = true, hasAcentricFactor = true);
          // does not exist!
          constant Modelica.Media.Interfaces.Types.IdealGas.FluidConstants CO(chemicalFormula = "CO", iupacName = "unknown", structureFormula = "unknown", casRegistryNumber = "630-08-0", meltingPoint = 68.15, normalBoilingPoint = 81.66, criticalTemperature = 132.85, criticalPressure = 34.94e5, criticalMolarVolume = 93.10e-6, acentricFactor = 0.045, dipoleMoment = 0.1, molarMass = SingleGasesData.CO.MM, hasDipoleMoment = true, hasIdealGasHeatCapacity = true, hasCriticalData = true, hasAcentricFactor = true);
          constant Modelica.Media.Interfaces.Types.IdealGas.FluidConstants H2(chemicalFormula = "H2", iupacName = "unknown", structureFormula = "unknown", casRegistryNumber = "800000-51-5", meltingPoint = 13.56, normalBoilingPoint = 20.38, criticalTemperature = 33.25, criticalPressure = 12.97e5, criticalMolarVolume = 65.00e-6, acentricFactor = -0.216, dipoleMoment = 0.0, molarMass = SingleGasesData.H2.MM, hasDipoleMoment = true, hasIdealGasHeatCapacity = true, hasCriticalData = true, hasAcentricFactor = true);
          constant Modelica.Media.Interfaces.Types.IdealGas.FluidConstants H2O(chemicalFormula = "H2O", iupacName = "oxidane", structureFormula = "H2O", casRegistryNumber = "7732-18-5", meltingPoint = 273.15, normalBoilingPoint = 373.124, criticalTemperature = 647.096, criticalPressure = 220.64e5, criticalMolarVolume = 55.95e-6, acentricFactor = 0.344, dipoleMoment = 1.8, molarMass = SingleGasesData.H2O.MM, hasDipoleMoment = true, hasIdealGasHeatCapacity = true, hasCriticalData = true, hasAcentricFactor = true);
          constant Modelica.Media.Interfaces.Types.IdealGas.FluidConstants N2O(chemicalFormula = "N2O", iupacName = "unknown", structureFormula = "unknown", casRegistryNumber = "10024-97-2", meltingPoint = 182.33, normalBoilingPoint = 184.67, criticalTemperature = 309.60, criticalPressure = 72.55e5, criticalMolarVolume = 97.00e-6, acentricFactor = 0.142, dipoleMoment = 0.2, molarMass = SingleGasesData.N2O.MM, hasDipoleMoment = true, hasIdealGasHeatCapacity = true, hasCriticalData = true, hasAcentricFactor = true);
          constant Modelica.Media.Interfaces.Types.IdealGas.FluidConstants NO(chemicalFormula = "NO", iupacName = "unknown", structureFormula = "unknown", casRegistryNumber = "10102-43-9", meltingPoint = 109.51, normalBoilingPoint = 121.38, criticalTemperature = 180.00, criticalPressure = 64.80e5, criticalMolarVolume = 58.00e-6, acentricFactor = 0.582, dipoleMoment = 0.2, molarMass = SingleGasesData.NO.MM, hasDipoleMoment = true, hasIdealGasHeatCapacity = true, hasCriticalData = true, hasAcentricFactor = true);
          constant Modelica.Media.Interfaces.Types.IdealGas.FluidConstants NO2(chemicalFormula = "NO2", iupacName = "unknown", structureFormula = "unknown", casRegistryNumber = "10102-44-0", meltingPoint = 261.95, normalBoilingPoint = 294.0, criticalTemperature = 431.35, criticalPressure = 101.33e5, criticalMolarVolume = 82.5e-6, acentricFactor = 0.849, dipoleMoment = 0.32, molarMass = SingleGasesData.NO2.MM, hasDipoleMoment = true, hasIdealGasHeatCapacity = true, hasCriticalData = true, hasAcentricFactor = true);
          constant Modelica.Media.Interfaces.Types.IdealGas.FluidConstants NH3(chemicalFormula = "NH3", iupacName = "unknown", structureFormula = "unknown", casRegistryNumber = "7664-41-7", meltingPoint = 195.41, normalBoilingPoint = 239.82, criticalTemperature = 405.40, criticalPressure = 113.53e5, criticalMolarVolume = 72.47e-6, acentricFactor = 0.257, dipoleMoment = 1.5, molarMass = SingleGasesData.NH3.MM, hasDipoleMoment = true, hasIdealGasHeatCapacity = true, hasCriticalData = true, hasAcentricFactor = true);
          constant Modelica.Media.Interfaces.Types.IdealGas.FluidConstants SO2(chemicalFormula = "SO2", iupacName = "unknown", structureFormula = "unknown", casRegistryNumber = "7446-09-5", meltingPoint = 197.67, normalBoilingPoint = 263.13, criticalTemperature = 430.80, criticalPressure = 78.84e5, criticalMolarVolume = 122.00e-6, acentricFactor = 0.245, dipoleMoment = 1.6, molarMass = SingleGasesData.SO2.MM, hasDipoleMoment = true, hasIdealGasHeatCapacity = true, hasCriticalData = true, hasAcentricFactor = true);
          constant Modelica.Media.Interfaces.Types.IdealGas.FluidConstants SO3(chemicalFormula = "SO3", iupacName = "unknown", structureFormula = "unknown", casRegistryNumber = "7446-11-9", meltingPoint = 289.95, normalBoilingPoint = 317.90, criticalTemperature = 490.90, criticalPressure = 82.10e5, criticalMolarVolume = 126.50e-6, acentricFactor = 0.422, dipoleMoment = 0.0, molarMass = SingleGasesData.SO3.MM, hasDipoleMoment = true, hasIdealGasHeatCapacity = true, hasCriticalData = true, hasAcentricFactor = true);
          constant Modelica.Media.Interfaces.Types.IdealGas.FluidConstants Ar(chemicalFormula = "Ar", iupacName = "unknown", structureFormula = "unknown", casRegistryNumber = "7440-37-1", meltingPoint = 83.80, normalBoilingPoint = 87.27, criticalTemperature = 150.86, criticalPressure = 48.98e5, criticalMolarVolume = 74.57e-6, acentricFactor = -0.002, dipoleMoment = 0.0, molarMass = SingleGasesData.Ar.MM, hasDipoleMoment = true, hasIdealGasHeatCapacity = true, hasCriticalData = true, hasAcentricFactor = true);
          constant Modelica.Media.Interfaces.Types.IdealGas.FluidConstants He(chemicalFormula = "He", iupacName = "unknown", structureFormula = "unknown", casRegistryNumber = "7440-59-7", meltingPoint = 2.15, normalBoilingPoint = 4.30, criticalTemperature = 5.19, criticalPressure = 2.27e5, criticalMolarVolume = 57.30e-6, acentricFactor = -0.390, dipoleMoment = 0.0, molarMass = SingleGasesData.He.MM, hasDipoleMoment = true, hasIdealGasHeatCapacity = true, hasCriticalData = true, hasAcentricFactor = true);
          constant Modelica.Media.Interfaces.Types.IdealGas.FluidConstants Ne(chemicalFormula = "Ne", iupacName = "unknown", structureFormula = "unknown", casRegistryNumber = "7440-01-9", meltingPoint = 24.56, normalBoilingPoint = 27.07, criticalTemperature = 44.40, criticalPressure = 27.60e5, criticalMolarVolume = 41.70e-6, acentricFactor = -0.016, dipoleMoment = 0.0, molarMass = SingleGasesData.Ne.MM, hasDipoleMoment = true, hasIdealGasHeatCapacity = true, hasCriticalData = true, hasAcentricFactor = true);
          constant Modelica.Media.Interfaces.Types.IdealGas.FluidConstants CH4(chemicalFormula = "CH4", iupacName = "unknown", structureFormula = "unknown", casRegistryNumber = "74-82-8", meltingPoint = 90.69, normalBoilingPoint = 111.66, criticalTemperature = 190.56, criticalPressure = 45.99e5, criticalMolarVolume = 98.60e-6, acentricFactor = 0.011, dipoleMoment = 0.0, molarMass = SingleGasesData.CH4.MM, hasDipoleMoment = true, hasIdealGasHeatCapacity = true, hasCriticalData = true, hasAcentricFactor = true);
          constant Modelica.Media.Interfaces.Types.IdealGas.FluidConstants C2H6(chemicalFormula = "C2H6", iupacName = "unknown", structureFormula = "unknown", casRegistryNumber = "74-84-0", meltingPoint = 90.35, normalBoilingPoint = 184.55, criticalTemperature = 305.32, criticalPressure = 48.72e5, criticalMolarVolume = 145.50e-6, acentricFactor = 0.099, dipoleMoment = 0.0, molarMass = SingleGasesData.C2H6.MM, hasDipoleMoment = true, hasIdealGasHeatCapacity = true, hasCriticalData = true, hasAcentricFactor = true);
          constant Modelica.Media.Interfaces.Types.IdealGas.FluidConstants C3H8(chemicalFormula = "C3H8", iupacName = "unknown", structureFormula = "unknown", casRegistryNumber = "74-98-6", meltingPoint = 91.45, normalBoilingPoint = 231.02, criticalTemperature = 369.83, criticalPressure = 42.48e5, criticalMolarVolume = 200.00e-6, acentricFactor = 0.152, dipoleMoment = 0.0, molarMass = SingleGasesData.C3H8.MM, hasDipoleMoment = true, hasIdealGasHeatCapacity = true, hasCriticalData = true, hasAcentricFactor = true);
          constant Modelica.Media.Interfaces.Types.IdealGas.FluidConstants C4H10_n_butane(chemicalFormula = "C4H10", iupacName = "unknown", structureFormula = "unknown", casRegistryNumber = "106-97-8", meltingPoint = 134.79, normalBoilingPoint = 272.66, criticalTemperature = 425.12, criticalPressure = 37.96e5, criticalMolarVolume = 255.00e-6, acentricFactor = 0.20, dipoleMoment = 0.0, molarMass = SingleGasesData.C4H10_n_butane.MM, hasDipoleMoment = true, hasIdealGasHeatCapacity = true, hasCriticalData = true, hasAcentricFactor = true);
          constant Modelica.Media.Interfaces.Types.IdealGas.FluidConstants C5H12_n_pentane(chemicalFormula = "C5H12", iupacName = "unknown", structureFormula = "unknown", casRegistryNumber = "109-66-0", meltingPoint = 143.43, normalBoilingPoint = 309.22, criticalTemperature = 469.70, criticalPressure = 33.70e5, criticalMolarVolume = 311.00e-6, acentricFactor = 0.252, dipoleMoment = 0.0, molarMass = SingleGasesData.C5H12_n_pentane.MM, hasDipoleMoment = true, hasIdealGasHeatCapacity = true, hasCriticalData = true, hasAcentricFactor = true);
          constant Modelica.Media.Interfaces.Types.IdealGas.FluidConstants C6H14_n_hexane(chemicalFormula = "C6H14", iupacName = "unknown", structureFormula = "unknown", casRegistryNumber = "110-54-3", meltingPoint = 177.84, normalBoilingPoint = 341.88, criticalTemperature = 507.60, criticalPressure = 30.25e5, criticalMolarVolume = 368.00e-6, acentricFactor = 0.300, dipoleMoment = 0.0, molarMass = SingleGasesData.C6H14_n_hexane.MM, hasDipoleMoment = true, hasIdealGasHeatCapacity = true, hasCriticalData = true, hasAcentricFactor = true);
          constant Modelica.Media.Interfaces.Types.IdealGas.FluidConstants C7H16_n_heptane(chemicalFormula = "C7H16", iupacName = "unknown", structureFormula = "unknown", casRegistryNumber = "142-82-5", meltingPoint = 182.59, normalBoilingPoint = 371.57, criticalTemperature = 540.20, criticalPressure = 27.40e5, criticalMolarVolume = 428.00e-6, acentricFactor = 0.350, dipoleMoment = 0.0, molarMass = SingleGasesData.C7H16_n_heptane.MM, hasDipoleMoment = true, hasIdealGasHeatCapacity = true, hasCriticalData = true, hasAcentricFactor = true);
          constant Modelica.Media.Interfaces.Types.IdealGas.FluidConstants C2H4(chemicalFormula = "C2H4", iupacName = "unknown", structureFormula = "unknown", casRegistryNumber = "74-85-1", meltingPoint = 103.99, normalBoilingPoint = 169.42, criticalTemperature = 282.34, criticalPressure = 50.41e5, criticalMolarVolume = 131.10e-6, acentricFactor = 0.087, dipoleMoment = 0.0, molarMass = SingleGasesData.C2H4.MM, hasDipoleMoment = true, hasIdealGasHeatCapacity = true, hasCriticalData = true, hasAcentricFactor = true);
          constant Modelica.Media.Interfaces.Types.IdealGas.FluidConstants C3H6_propylene(chemicalFormula = "C3H6", iupacName = "unknown", structureFormula = "unknown", casRegistryNumber = "115-07-1", meltingPoint = 87.89, normalBoilingPoint = 225.46, criticalTemperature = 364.90, criticalPressure = 46.00e5, criticalMolarVolume = 184.60e-6, acentricFactor = 0.142, dipoleMoment = 0.4, molarMass = SingleGasesData.C3H6_propylene.MM, hasDipoleMoment = true, hasIdealGasHeatCapacity = true, hasCriticalData = true, hasAcentricFactor = true);
          constant Modelica.Media.Interfaces.Types.IdealGas.FluidConstants C4H8_1_butene(chemicalFormula = "C4H8", iupacName = "unknown", structureFormula = "unknown", casRegistryNumber = "106-98-9", meltingPoint = 87.79, normalBoilingPoint = 266.92, criticalTemperature = 419.50, criticalPressure = 40.20e5, criticalMolarVolume = 240.80e-6, acentricFactor = 0.194, dipoleMoment = 0.3, molarMass = SingleGasesData.C4H8_1_butene.MM, hasDipoleMoment = true, hasIdealGasHeatCapacity = true, hasCriticalData = true, hasAcentricFactor = true);
          constant Modelica.Media.Interfaces.Types.IdealGas.FluidConstants C5H10_1_pentene(chemicalFormula = "C5H10", iupacName = "unknown", structureFormula = "unknown", casRegistryNumber = "109-67-1", meltingPoint = 106.95, normalBoilingPoint = 303.11, criticalTemperature = 464.80, criticalPressure = 35.60e5, criticalMolarVolume = 298.40e-6, acentricFactor = 0.237, dipoleMoment = 0.4, molarMass = SingleGasesData.C5H10_1_pentene.MM, hasDipoleMoment = true, hasIdealGasHeatCapacity = true, hasCriticalData = true, hasAcentricFactor = true);
          constant Modelica.Media.Interfaces.Types.IdealGas.FluidConstants C6H12_1_hexene(chemicalFormula = "C6H12", iupacName = "unknown", structureFormula = "unknown", casRegistryNumber = "592-41-6", meltingPoint = 133.34, normalBoilingPoint = 336.63, criticalTemperature = 504.00, criticalPressure = 31.43e5, criticalMolarVolume = 355.10e-6, acentricFactor = 0.281, dipoleMoment = 0.4, molarMass = SingleGasesData.C6H12_1_hexene.MM, hasDipoleMoment = true, hasIdealGasHeatCapacity = true, hasCriticalData = true, hasAcentricFactor = true);
          constant Modelica.Media.Interfaces.Types.IdealGas.FluidConstants C7H14_1_heptene(chemicalFormula = "C7H14", iupacName = "unknown", structureFormula = "unknown", casRegistryNumber = "592-76-7", meltingPoint = 153.45, normalBoilingPoint = 366.79, criticalTemperature = 537.30, criticalPressure = 29.20e5, criticalMolarVolume = 409.00e-6, acentricFactor = 0.343, dipoleMoment = 0.3, molarMass = SingleGasesData.C7H14_1_heptene.MM, hasDipoleMoment = true, hasIdealGasHeatCapacity = true, hasCriticalData = true, hasAcentricFactor = true);
          constant Modelica.Media.Interfaces.Types.IdealGas.FluidConstants C2H2_vinylidene(chemicalFormula = "C2H2", iupacName = "unknown", structureFormula = "unknown", casRegistryNumber = "74-86-2", meltingPoint = 192.35, normalBoilingPoint = 188.40, criticalTemperature = 308.30, criticalPressure = 61.14e5, criticalMolarVolume = 112.20e-6, acentricFactor = 0.189, dipoleMoment = 0.0, molarMass = SingleGasesData.C2H2_vinylidene.MM, hasDipoleMoment = true, hasIdealGasHeatCapacity = true, hasCriticalData = true, hasAcentricFactor = true);
          constant Modelica.Media.Interfaces.Types.IdealGas.FluidConstants C6H6(chemicalFormula = "C6H6", iupacName = "unknown", structureFormula = "unknown", casRegistryNumber = "71-43-2", meltingPoint = 278.68, normalBoilingPoint = 353.24, criticalTemperature = 562.05, criticalPressure = 48.95e5, criticalMolarVolume = 256.00e-6, acentricFactor = 0.210, dipoleMoment = 0.0, molarMass = SingleGasesData.C6H6.MM, hasDipoleMoment = true, hasIdealGasHeatCapacity = true, hasCriticalData = true, hasAcentricFactor = true);
          constant Modelica.Media.Interfaces.Types.IdealGas.FluidConstants C8H18_n_octane(chemicalFormula = "C8H18", iupacName = "unknown", structureFormula = "unknown", casRegistryNumber = "111-65-9", meltingPoint = 216.39, normalBoilingPoint = 398.82, criticalTemperature = 568.70, criticalPressure = 24.90e5, criticalMolarVolume = 492.00e-6, acentricFactor = 0.399, dipoleMoment = 0.0, molarMass = SingleGasesData.C8H18_n_octane.MM, hasDipoleMoment = true, hasIdealGasHeatCapacity = true, hasCriticalData = true, hasAcentricFactor = true);
          constant Modelica.Media.Interfaces.Types.IdealGas.FluidConstants C8H10_ethylbenz(chemicalFormula = "C8H10", iupacName = "unknown", structureFormula = "unknown", casRegistryNumber = "100-41-4", meltingPoint = 178.18, normalBoilingPoint = 409.36, criticalTemperature = 617.15, criticalPressure = 36.09e5, criticalMolarVolume = 374.00e-6, acentricFactor = 0.304, dipoleMoment = 0.4, molarMass = SingleGasesData.C8H10_ethylbenz.MM, hasDipoleMoment = true, hasIdealGasHeatCapacity = true, hasCriticalData = true, hasAcentricFactor = true);
          constant Modelica.Media.Interfaces.Types.IdealGas.FluidConstants CH3OH(chemicalFormula = "CH3OH", iupacName = "unknown", structureFormula = "unknown", casRegistryNumber = "67-56-1", meltingPoint = 175.49, normalBoilingPoint = 337.69, criticalTemperature = 512.64, criticalPressure = 80.97e5, criticalMolarVolume = 118.00e-6, acentricFactor = 0.565, dipoleMoment = 1.7, molarMass = SingleGasesData.CH3OH.MM, hasDipoleMoment = true, hasIdealGasHeatCapacity = true, hasCriticalData = true, hasAcentricFactor = true);
          constant Modelica.Media.Interfaces.Types.IdealGas.FluidConstants C2H5OH(chemicalFormula = "C2H5OH", iupacName = "unknown", structureFormula = "unknown", casRegistryNumber = "64-17-5", meltingPoint = 159.05, normalBoilingPoint = 351.80, criticalTemperature = 513.92, criticalPressure = 61.48e5, criticalMolarVolume = 167.00e-6, acentricFactor = 0.649, dipoleMoment = 1.7, molarMass = SingleGasesData.C2H5OH.MM, hasDipoleMoment = true, hasIdealGasHeatCapacity = true, hasCriticalData = true, hasAcentricFactor = true);
          constant Modelica.Media.Interfaces.Types.IdealGas.FluidConstants C3H7OH(chemicalFormula = "C3H7OH", iupacName = "unknown", structureFormula = "unknown", casRegistryNumber = "71-23-8", meltingPoint = 147.00, normalBoilingPoint = 370.93, criticalTemperature = 536.78, criticalPressure = 51.75e5, criticalMolarVolume = 219.00e-6, acentricFactor = 0.629, dipoleMoment = 1.7, molarMass = 60.1e-3, hasDipoleMoment = true, hasIdealGasHeatCapacity = true, hasCriticalData = true, hasAcentricFactor = true);
          constant Modelica.Media.Interfaces.Types.IdealGas.FluidConstants C4H9OH(chemicalFormula = "C4H9OH", iupacName = "unknown", structureFormula = "unknown", casRegistryNumber = "71-36-3", meltingPoint = 183.35, normalBoilingPoint = 390.88, criticalTemperature = 563.05, criticalPressure = 44.23e5, criticalMolarVolume = 275.00e-6, acentricFactor = 0.589, dipoleMoment = 1.8, molarMass = 74.12e-3, hasDipoleMoment = true, hasIdealGasHeatCapacity = true, hasCriticalData = true, hasAcentricFactor = true);
          annotation(
            Documentation(info = "<html>
<p>
This package contains FluidConstants data records for the following 37 gases
(see also the description in
<a href=\"modelica://Modelica.Media.IdealGases\">Modelica.Media.IdealGases</a>):
</p>
<pre>
Argon             Methane          Methanol       Carbon Monoxide  Carbon Dioxide
Acetylene         Ethylene         Ethanol        Ethane           Propylene
Propane           1-Propanol       1-Butene       N-Butane         1-Pentene
N-Pentane         Benzene          1-Hexene       N-Hexane         1-Heptane
N-Heptane         Ethylbenzene     N-Octane       Chlorine         Fluorine
Hydrogen          Steam            Helium         Ammonia          Nitric Oxide
Nitrogen Dioxide  Nitrogen         Nitrous        Oxide            Neon Oxygen
Sulfur Dioxide    Sulfur Trioxide
</pre>

</html>"));
        end FluidData;

        package SingleGasesData "Ideal gas data based on the NASA Glenn coefficients"
          extends Modelica.Icons.Package;
          constant IdealGases.Common.DataRecord Ag(name = "Ag", MM = 0.1078682, Hf = 2641186.188329832, H0 = 57453.70739476509, Tlimit = 1000, alow = {0, 0, 2.5, 0, 0, 0, 0}, blow = {33520.0237, 6.56281935}, ahigh = {-330992.637, 982.0086420000001, 1.381179917, 0.0006170899989999999, -1.6881146e-007, 2.008826848e-011, -5.627285655e-016}, bhigh = {27267.19171, 14.56862733}, R = 77.07991789980736);
          constant IdealGases.Common.DataRecord Agplus(name = "Agplus", MM = 0.1078676514, Hf = 9475442.514362559, H0 = 57454.94519963192, Tlimit = 1000, alow = {3691132.75, -43169.82999999999, 202.8445385, -0.465841374, 0.000558620051, -3.154880975e-007, 6.578702060000001e-011}, blow = {337157.447, -1142.924427}, ahigh = {-53274323.49999999, 131071.0631, -109.8820208, 0.0482600276, -1.093661557e-005, 1.26383591e-009, -5.852542535e-014}, bhigh = {-743912.2509999999, 844.6266189999999}, R = 77.08030991764042);
          constant IdealGases.Common.DataRecord Agminus(name = "Agminus", MM = 0.1078687486, Hf = 1419120.273357839, H0 = 57453.41519610434, Tlimit = 1000, alow = {0, 0, 2.5, 0, 0, 0, 0}, blow = {17665.65919, 5.8696798}, ahigh = {0, 0, 2.5, 0, 0, 0, 0}, bhigh = {17665.65919, 5.8696798}, R = 77.07952588596174);
          constant IdealGases.Common.DataRecord Air(name = "Air", MM = 0.0289651159, Hf = -4333.833858403446, H0 = 298609.6803431054, Tlimit = 1000, alow = {10099.5016, -196.827561, 5.00915511, -0.00576101373, 1.06685993e-005, -7.94029797e-009, 2.18523191e-012}, blow = {-176.796731, -3.921504225}, ahigh = {241521.443, -1257.8746, 5.14455867, -0.000213854179, 7.06522784e-008, -1.07148349e-011, 6.57780015e-016}, bhigh = {6462.26319, -8.147411905}, R = 287.0512249529787);
          constant IdealGases.Common.DataRecord AL(name = "AL", MM = 0.026981538, Hf = 12230585.22460803, H0 = 256422.4100197698, Tlimit = 1000, alow = {5006.60889, 18.61304407, 2.412531111, 0.0001987604647, -2.432362152e-007, 1.538281506e-010, -3.944375734e-014}, blow = {38874.1268, 6.086585765}, ahigh = {-29208.20938, 116.7751876, 2.356906505, 7.73723152e-005, -1.529455262e-008, -9.97167026e-013, 5.053278264e-016}, bhigh = {38232.8865, 6.600920155}, R = 308.1541163442944);
          constant IdealGases.Common.DataRecord ALplus(name = "ALplus", MM = 0.0269809894, Hf = 33839201.16732265, H0 = 229696.0985426279, Tlimit = 1000, alow = {0, 0, 2.5, 0, 0, 0, 0}, blow = {109064.4788, 3.79100578}, ahigh = {-4181.18325, -9.94855727, 2.548615878, -5.878760040000001e-005, 3.132291294e-008, -7.74889463e-012, 7.27444769e-016}, bhigh = {109101.1485, 3.48866729}, R = 308.1603819910326);
          constant IdealGases.Common.DataRecord ALminus(name = "ALminus", MM = 0.0269820866, Hf = 10417656.61666804, H0 = 250396.2388142361, Tlimit = 1000, alow = {29108.01723, -383.698375, 4.6551428, -0.00604582184, 8.57725964e-006, -5.47626e-009, 1.322714061e-012}, blow = {34906.1698, -5.9570977}, ahigh = {633981.432, -2383.438463, 5.46997113, -0.001299840355, 2.88830547e-007, -3.25324051e-011, 1.472436088e-015}, bhigh = {47803.0654, -15.36906816}, R = 308.1478509523426);
          constant IdealGases.Common.DataRecord ALBr(name = "ALBr", MM = 0.106885538, Hf = 134022.0133429089, H0 = 89545.4631102666, Tlimit = 1000, alow = {8176.158640000001, -251.6942718, 5.40329633, -0.00174721292, 2.07730477e-006, -1.261267579e-009, 3.16633597e-013}, blow = {1635.024429, -2.323594886}, ahigh = {-610339.5599999999, 2010.066834, 1.769617961, 0.001929888914, -6.64104783e-007, 1.172854627e-010, -7.17874276e-015}, bhigh = {-12178.12867, 22.04450764}, R = 77.7885591968485);
          constant IdealGases.Common.DataRecord ALBr2(name = "ALBr2", MM = 0.186789538, Hf = -753051.4101919348, H0 = 71727.1167510463, Tlimit = 1000, alow = {31993.7587, -711.917897, 9.478258110000001, -0.00487553167, 5.51651299e-006, -3.34005304e-009, 8.36847684e-013}, blow = {-15405.91306, -17.42171366}, ahigh = {-352378.29, 467.154417, 7.11190819, -0.00055517092, 3.16630113e-007, -5.52102833e-011, 3.17672595e-015}, bhigh = {-22650.04078, -2.69561036}, R = 44.51251440003026);
          constant IdealGases.Common.DataRecord ALBr3(name = "ALBr3", MM = 0.266693538, Hf = -1539132.395476339, H0 = 67280.73028901059, Tlimit = 1000, alow = {47189.4884, -1053.853163, 13.50747168, -0.006639869929999999, 7.26927683e-006, -4.27804949e-009, 1.045573281e-012}, blow = {-46994.4033, -36.67936779999999}, ahigh = {-89355.2744, -26.78035134, 10.02063997, -8.458764399999999e-006, 1.904243317e-009, -2.215195785e-013, 1.038219389e-017}, bhigh = {-52492.4764, -15.79666403}, R = 31.17612845947546);
          constant IdealGases.Common.DataRecord ALC(name = "ALC", MM = 0.038992238, Hf = 17497931.48574852, H0 = 232305.4398672885, Tlimit = 1000, alow = {41399.2188, -585.867789, 5.96244572, -0.002006064322, 1.665566136e-006, -7.12164921e-010, 1.271188744e-013}, blow = {83834.3611, -8.00216505}, ahigh = {1937001.487, -6749.117789999999, 13.47525643, -0.00585080265, 1.926461091e-006, -2.585922351e-010, 1.222659806e-014}, bhigh = {122536.3649, -61.52244872}, R = 213.2340287828567);
          constant IdealGases.Common.DataRecord ALC2(name = "ALC2", MM = 0.051002938, Hf = 13246605.04851701, H0 = 240432.7374238716, Tlimit = 1000, alow = {15240.87465, -264.9987059, 6.2804149, -0.0001786037647, 3.79190774e-006, -3.99945007e-009, 1.299752388e-012}, blow = {80927.63279999999, -6.247291862}, ahigh = {146601.7895, -1362.764489, 8.457759149999999, -0.000367890168, 7.9031745e-008, -8.880023720000001e-012, 4.05363395e-016}, bhigh = {87059.24230000001, -21.27956728}, R = 163.019471544953);
          constant IdealGases.Common.DataRecord ALCL(name = "ALCL", MM = 0.06243453800000001, Hf = -816974.7808496637, H0 = 149326.1950620985, Tlimit = 1000, alow = {23808.81392, -445.715942, 5.99285399, -0.002777300261, 3.101582418e-006, -1.815268376e-009, 4.414662339999999e-013}, blow = {-5202.73603, -7.383298016}, ahigh = {-899346.1220000001, 2765.942061, 1.003500696, 0.002267806833, -7.2805416e-007, 1.182380673e-010, -6.66049813e-015}, bhigh = {-24961.64448, 26.1692548}, R = 133.1710342759323);
          constant IdealGases.Common.DataRecord ALCLplus(name = "ALCLplus", MM = 0.0624339894, Hf = 13804168.45507553, H0 = 146625.821735492, Tlimit = 1000, alow = {34699.5522, -547.990003, 6.10601113, -0.002674711519, 2.696426573e-006, -1.463885068e-009, 3.35872906e-013}, blow = {105171.951, -7.816586372}, ahigh = {-754724.0819999999, 1251.545253, 4.44242491, -0.0009786116429999999, 6.347511329999999e-007, -1.158320344e-010, 6.870022460000001e-015}, bhigh = {93183.32699999999, 4.047521108}, R = 133.1722044338881);
          constant IdealGases.Common.DataRecord ALCL2(name = "ALCL2", MM = 0.09788753800000001, Hf = -2460724.387613059, H0 = 131241.0063883719, Tlimit = 1000, alow = {53405.4595, -967.805798, 10.06252671, -0.00553952845, 5.82342056e-006, -3.30654245e-009, 7.8315621e-013}, blow = {-26076.2711, -23.93364216}, ahigh = {430345.306, -1552.370585, 8.760657419999999, -0.0009467450059999999, 2.40184488e-007, -2.427836709e-011, 8.390123470000001e-016}, bhigh = {-21458.84709, -18.03641668}, R = 84.93902461823076);
          constant IdealGases.Common.DataRecord ALCL3(name = "ALCL3", MM = 0.133340538, Hf = -4384854.5368851, H0 = 122999.3762287055, Tlimit = 1000, alow = {77506.0097, -1440.779717, 14.01744141, -0.00638163124, 5.87167472e-006, -2.908872278e-009, 5.994050889999999e-013}, blow = {-65793.43180000001, -44.94017799}, ahigh = {-137863.0916, -55.7920729, 10.04190387, -1.682165339e-005, 3.72466466e-009, -4.27552678e-013, 1.982341329e-017}, bhigh = {-73434.0747, -20.45130429}, R = 62.35517063835456);
          constant IdealGases.Common.DataRecord ALF(name = "ALF", MM = 0.0459799412, Hf = -5742948.753488184, H0 = 193391.0041624847, Tlimit = 1000, alow = {30207.71686, -308.0986949, 3.88641314, 0.00356434369, -5.85847128e-006, 4.38810992e-009, -1.256906273e-012}, blow = {-31175.72862, 2.032567172}, ahigh = {-711122.905, 1903.013316, 2.191607179, 0.001421471467, -4.17963477e-007, 6.1538044e-011, -2.945403999e-015}, bhigh = {-45407.2896, 16.13163743}, R = 180.8282434254179);
          constant IdealGases.Common.DataRecord ALFplus(name = "ALFplus", MM = 0.0459793926, Hf = 15055300.25205248, H0 = 191666.0160491116, Tlimit = 1000, alow = {36430.3812, -252.4634478, 3.105310486, 0.00573327043, -8.839786990000001e-006, 6.34357959e-009, -1.645011685e-012}, blow = {83702.37449999999, 6.76401558}, ahigh = {1505101.126, -2236.668821, 3.098216724, 0.00311541393, -1.168427068e-006, 1.773587168e-010, -9.665841379999999e-015}, bhigh = {98850.8385, 5.99189459}, R = 180.8304009653229);
          constant IdealGases.Common.DataRecord ALFCL(name = "ALFCL", MM = 0.0814329412, Hf = -5359132.392481975, H0 = 148996.1534141419, Tlimit = 1000, alow = {41047.7948, -576.964642, 6.5626427, 0.00410628584, -7.21392112e-006, 5.48640145e-009, -1.580656193e-012}, blow = {-51148.287, -6.041227053}, ahigh = {69706.36350000001, -622.4485910000001, 7.57347541, -0.0002236155999, 2.382317949e-008, 4.44378576e-012, -5.486977849999999e-016}, bhigh = {-51009.43199999999, -10.91164505}, R = 102.1020716859482);
          constant IdealGases.Common.DataRecord ALFCL2(name = "ALFCL2", MM = 0.1168859412, Hf = -6770663.741722942, H0 = 134646.3042383407, Tlimit = 1000, alow = {69556.07340000001, -1188.857902, 11.5667259, 0.000457495949, -3.40737283e-006, 3.35682254e-009, -1.085329228e-012}, blow = {-91620.4926, -32.06065081}, ahigh = {-172036.0526, -119.3340759, 10.08894431, -3.55099381e-005, 7.83095261e-009, -8.961734229999999e-013, 4.14533026e-017}, bhigh = {-98041.8547, -21.27046859}, R = 71.13320827671959);
          constant IdealGases.Common.DataRecord ALF2(name = "ALF2", MM = 0.0649783444, Hf = -9722688.08683282, H0 = 178535.3583123919, Tlimit = 1000, alow = {29949.51991, -219.4199915, 3.41674876, 0.01272953936, -1.883312206e-005, 1.330702256e-008, -3.68011863e-012}, blow = {-76075.3765, 8.759626916}, ahigh = {-214689.6245, 30.9684015, 6.81639962, 0.0001820129379, -7.79367744e-008, 1.472534201e-011, -8.803695299999999e-016}, bhigh = {-78840.02680000001, -7.915370474}, R = 127.9575845887511);
          constant IdealGases.Common.DataRecord ALF2minus(name = "ALF2minus", MM = 0.064978893, Hf = -13130889.82602397, H0 = 174392.244570864, Tlimit = 1000, alow = {123336.9037, -1348.195504, 8.61117805, 0.0002245141297, -2.424673201e-006, 2.21413243e-009, -6.60271396e-013}, blow = {-97084.37460000001, -21.92617549}, ahigh = {-159007.6965, -206.0486516, 7.15528938, -6.260167690000001e-005, 1.391935746e-008, -1.603901265e-012, 7.461508519999999e-017}, bhigh = {-104047.9429, -11.22696648}, R = 127.9565042759347);
          constant IdealGases.Common.DataRecord ALF2CL(name = "ALF2CL", MM = 0.1004313444, Hf = -9948369.714345874, H0 = 147638.3402869194, Tlimit = 1000, alow = {56774.12089999999, -886.9829470000001, 8.75274655, 0.0081957912, -1.380480934e-005, 1.032703706e-008, -2.948968416e-012}, blow = {-117793.6778, -18.5834736}, ahigh = {-211542.2988, -195.9188681, 10.1457685, -5.81321935e-005, 1.281071982e-008, -1.465366277e-012, 6.77600245e-017}, bhigh = {-122715.311, -23.59853398}, R = 82.78762023621782);
          constant IdealGases.Common.DataRecord ALF3(name = "ALF3", MM = 0.0839767476, Hf = -14400140.18832994, H0 = 167233.459277244, Tlimit = 1000, alow = {44102.6352, -566.7495739999999, 5.83307285, 0.01603535069, -2.413263602e-005, 1.71410616e-008, -4.7475432e-012}, blow = {-144334.9991, -5.461613658}, ahigh = {-249397.216, -291.1202519, 10.21662932, -8.64298597e-005, 1.905713108e-008, -2.181058411e-012, 1.009055287e-016}, bhigh = {-147569.4276, -27.03983244}, R = 99.00921668940654);
          constant IdealGases.Common.DataRecord ALF4minus(name = "ALF4minus", MM = 0.1029756994, Hf = -18952051.96343634, H0 = 163017.4409866645, Tlimit = 1000, alow = {231201.0037, -3235.37678, 20.17865547, -0.0091081935, 6.67955263e-006, -2.606075106e-009, 4.11006051e-013}, blow = {-221178.4563, -86.77268282}, ahigh = {-321741.161, -261.2097043, 13.19792654, -8.01013541e-005, 1.786204421e-008, -2.062751964e-012, 9.61258177e-017}, bhigh = {-238157.7915, -42.31634321999999}, R = 80.74207845584199);
          constant IdealGases.Common.DataRecord ALH(name = "ALH", MM = 0.027989478, Hf = 8905160.860806337, H0 = 309691.5205063846, Tlimit = 1000, alow = {-37591.1403, 508.900223, 1.128086896, 0.003988660910000001, -2.150790303e-007, -2.176790819e-009, 1.020805902e-012}, blow = {26444.31827, 16.50021856}, ahigh = {6802018.430000001, -21784.16933, 30.32713047, -0.01503343597, 4.49214236e-006, -6.17845037e-010, 3.11520526e-014}, bhigh = {165830.1221, -187.6766425}, R = 297.0570583702919);
          constant IdealGases.Common.DataRecord ALHCL(name = "ALHCL", MM = 0.063442478, Hf = 165850.4890051741, H0 = 171597.5217739761, Tlimit = 1000, alow = {35871.6976, -591.624999, 6.94454238, -0.002511543148, 6.47133255e-006, -5.64227714e-009, 1.715950436e-012}, blow = {2750.930479, -9.903745880000001}, ahigh = {-168391.533, -886.230179, 8.062379399999999, -0.0009200551470000001, 3.94981722e-007, -6.40108676e-011, 3.5783541e-015}, bhigh = {3641.50485, -18.00686894}, R = 131.0552844420737);
          constant IdealGases.Common.DataRecord ALHCL2(name = "ALHCL2", MM = 0.098895478, Hf = -3552022.520180347, H0 = 138289.3968114498, Tlimit = 1000, alow = {118482.8615, -2054.157118, 15.74696513, -0.01476257296, 2.065336978e-005, -1.42019885e-008, 3.82627986e-012}, blow = {-34342.7746, -57.61184061}, ahigh = {96080.63939999999, -1485.540161, 11.03592709, -0.00039569552, 8.46584373e-008, -9.48333652e-012, 4.31903353e-016}, bhigh = {-36692.1639, -32.36862171}, R = 84.07332840840307);
          constant IdealGases.Common.DataRecord ALHF(name = "ALHF", MM = 0.0469878812, Hf = -3886416.653322091, H0 = 224996.7593771817, Tlimit = 1000, alow = {-9289.545099999999, 133.5515868, 2.77244868, 0.00675141503, -4.13384614e-006, 5.86570407e-010, 2.276158011e-013}, blow = {-23846.85845, 12.28752581}, ahigh = {734216.9250000001, -3462.16413, 10.39714588, -0.001750737965, 4.615687860000001e-007, -5.39041474e-011, 2.340478515e-015}, bhigh = {-2965.279722, -37.13172313}, R = 176.9492853829723);
          constant IdealGases.Common.DataRecord ALHFCL(name = "ALHFCL", MM = 0.08244088120000001, Hf = -6735062.943504781, H0 = 152019.6754034696, Tlimit = 1000, alow = {129037.3237, -2142.099873, 14.8543442, -0.0114032693, 1.564521768e-005, -1.067574855e-008, 2.857255247e-012}, blow = {-58183.0307, -54.83424986}, ahigh = {40689.8713, -1502.129854, 11.05214126, -0.000403158773, 8.64525222e-008, -9.70065952e-012, 4.423629509999999e-016}, bhigh = {-61298.8811, -34.03564016}, R = 100.8537497292059);
          constant IdealGases.Common.DataRecord ALHF2(name = "ALHF2", MM = 0.0659862844, Hf = -11597852.32580848, H0 = 186401.9941695641, Tlimit = 1000, alow = {105018.1431, -1398.654202, 9.166334839999999, 0.00386728687, -4.83084109e-006, 3.099287181e-009, -8.440702990000001e-013}, blow = {-86590.48800000001, -25.57299683}, ahigh = {-4419.9473, -1612.826324, 11.13336052, -0.000435181266, 9.34439616e-008, -1.049431425e-011, 4.78833398e-016}, bhigh = {-86069.16189999999, -36.76383785}, R = 126.0030334425073);
          constant IdealGases.Common.DataRecord ALH2(name = "ALH2", MM = 0.028997418, Hf = 9544813.058873035, H0 = 347990.7762822194, Tlimit = 1000, alow = {14551.82996, -215.3768996, 5.14437023, -0.00396522203, 1.340900203e-005, -1.216744854e-008, 3.74310713e-012}, blow = {33110.3794, -3.608799711}, ahigh = {143291.0601, -2365.907684, 9.085667190000001, -0.001308536612, 4.77719136e-007, -7.32472189e-011, 3.99790094e-015}, bhigh = {44859.5185, -32.20814285}, R = 286.7314600217164);
          constant IdealGases.Common.DataRecord ALH2CL(name = "ALH2CL", MM = 0.064450418, Hf = -1650034.387674569, H0 = 174603.2275539315, Tlimit = 1000, alow = {93038.2898, -1328.188016, 9.42476359, -0.00327766059, 1.004231609e-005, -8.94812309e-009, 2.708031665e-012}, blow = {-7647.43793, -26.82512215}, ahigh = {316692.993, -3094.037914, 12.1587202, -0.0008248269589999999, 1.765027123e-007, -1.977372723e-011, 9.00617628e-016}, bhigh = {2411.845642, -47.18390855}, R = 129.0057110257997);
          constant IdealGases.Common.DataRecord ALH2F(name = "ALH2F", MM = 0.0479958212, Hf = -6597571.331897535, H0 = 224286.2134839355, Tlimit = 1000, alow = {89060.39290000001, -975.080393, 5.87077232, 0.00689666369, -4.04581728e-006, 7.21931866e-010, 7.329846739999999e-014}, blow = {-34253.02600000001, -9.387539141}, ahigh = {271151.4962, -3169.62348, 12.21401118, -0.000846560026, 1.812341995e-007, -2.030947985e-011, 9.251825820000001e-016}, bhigh = {-22592.17746, -49.41337333}, R = 173.2332480645211);
          constant IdealGases.Common.DataRecord ALH3(name = "ALH3", MM = 0.030005358, Hf = 4295768.77569666, H0 = 346957.3667476322, Tlimit = 1000, alow = {14812.07909, -28.3649534, 2.507597126, 0.00731592051, 2.331766687e-006, -6.38920989e-009, 2.456885069e-012}, blow = {14631.89428, 8.3131877}, ahigh = {588545.855, -4595.78566, 13.20893344, -0.001226849004, 2.626580824e-007, -2.943725869e-011, 1.341177648e-015}, bhigh = {39917.1695, -61.81829295}, R = 277.0995766822712);
          constant IdealGases.Common.DataRecord ALI(name = "ALI", MM = 0.153886008, Hf = 437954.1575995656, H0 = 63365.84545100423, Tlimit = 1000, alow = {1870.854131, -154.3184003, 5.07439188, -0.00112058483, 1.386302156e-006, -8.54078228e-010, 2.180358492e-013}, blow = {7517.44369, 0.6688267621999999}, ahigh = {475281.792, -1198.028588, 5.43368536, -0.0001004479306, -1.005928846e-007, 4.568173190000001e-011, -3.960201360000001e-015}, bhigh = {14646.27816, -3.141199746}, R = 54.03007140194319);
          constant IdealGases.Common.DataRecord ALI2(name = "ALI2", MM = 0.280790478, Hf = -120420.5863419628, H0 = 49585.17859711753, Tlimit = 1000, alow = {14234.13595, -464.657569, 8.72267491, -0.00356051929, 4.18868229e-006, -2.616026962e-009, 6.72019551e-013}, blow = {-3846.13333, -10.73076578}, ahigh = {-335599.902, 474.334794, 7.10650139, -0.0005529965340000001, 3.161480314e-007, -5.51548916e-011, 3.17415582e-015}, bhigh = {-9785.48885, -0.5602275943999999}, R = 29.61094713475291);
          constant IdealGases.Common.DataRecord ALI3(name = "ALI3", MM = 0.407694948, Hf = -469298.0227952199, H0 = 46612.22095889204, Tlimit = 1000, alow = {27060.32439, -755.993681, 12.71396349, -0.00546688409, 6.29761892e-006, -3.86420961e-009, 9.77413244e-013}, blow = {-22209.66921, -28.47883861}, ahigh = {-63574.2747, -15.7877228, 10.01249733, -5.22867312e-006, 1.19617352e-009, -1.40919449e-013, 6.67127639e-018}, bhigh = {-26114.55525, -12.47999813}, R = 20.39385584930034);
          constant IdealGases.Common.DataRecord ALN(name = "ALN", MM = 0.040988238, Hf = 10706217.98868251, H0 = 226043.1638949691, Tlimit = 1000, alow = {27166.45079, -254.349111, 3.53898772, 0.00529625616, -9.845700609999999e-006, 8.45220904e-009, -2.592304568e-012}, blow = {53099.94910000001, 5.39943522}, ahigh = {3821214.22, -10677.76595, 14.49174222, -0.00372722713, 7.922770960000001e-007, -8.086892210000001e-011, 2.893539736e-015}, bhigh = {120523.634, -72.88585119000001}, R = 202.8501932676394);
          constant IdealGases.Common.DataRecord ALO(name = "ALO", MM = 0.042980938, Hf = 1566252.602490899, H0 = 204465.1980373253, Tlimit = 1000, alow = {-7683.3911, 295.7969549, 0.480810844, 0.01169224855, -1.595428871e-005, 1.060766814e-008, -2.647888708e-012}, blow = {5843.67217, 21.60997839}, ahigh = {15657.21161, 3855.74101, -5.92607978, 0.009050960419999999, -2.930661549e-006, 4.238529070000001e-010, -2.280655341e-014}, bhigh = {-13316.94655, 68.30663436}, R = 193.4455688240215);
          constant IdealGases.Common.DataRecord ALOplus(name = "ALOplus", MM = 0.0429803894, Hf = 23103397.01110293, H0 = 211485.1942220886, Tlimit = 1000, alow = {28291.78513, -417.90871, 5.31329987, -0.001410883046, 2.526524021e-006, -2.005474816e-009, 5.76255355e-013}, blow = {120364.8109, -3.41056593}, ahigh = {27108.46446, -699.8653420000001, 5.78781584, -0.000687189546, 2.075047303e-007, -2.655281004e-011, 1.28312966e-015}, bhigh = {121868.2226, -7.05103588}, R = 193.4480379556543);
          constant IdealGases.Common.DataRecord ALOminus(name = "ALOminus", MM = 0.04298148659999999, Hf = -6349752.034867962, H0 = 203453.1304461675, Tlimit = 1000, alow = {20780.75267, 2.859351226, 1.90191237, 0.00730104566, -9.13976987e-006, 5.65298319e-009, -1.396787103e-012}, blow = {-33592.6773, 12.96339858}, ahigh = {-69719.4829, -232.0553036, 4.67278433, -4.26644704e-005, 1.632506336e-008, -1.727134051e-012, 8.136840640000001e-017}, bhigh = {-33077.1893, -2.155922595}, R = 193.4430997554189);
          constant IdealGases.Common.DataRecord ALOCL(name = "ALOCL", MM = 0.078433938, Hf = -3844822.505278264, H0 = 151797.0065458144, Tlimit = 1000, alow = {-5144.627790000001, -58.86297070000001, 4.45231693, 0.009288235090000001, -1.246894823e-005, 8.17807354e-009, -2.130739934e-012}, blow = {-37596.8365, 2.476862818}, ahigh = {-125802.7706, -242.0565961, 7.68113002, -7.268741699999999e-005, 1.611337534e-008, 1.852889749e-012, 8.60727822e-017}, bhigh = {-37543.2605, -14.7581832}, R = 106.0060505950881);
          constant IdealGases.Common.DataRecord ALOCL2(name = "ALOCL2", MM = 0.113886938, Hf = -3532526.873274967, H0 = 138316.950799046, Tlimit = 1000, alow = {69026.3713, -1189.2975, 11.61842739, 0.0002818279805, -3.145010615e-006, 3.16838065e-009, -1.032375462e-012}, blow = {-44833.2307, -31.65475132}, ahigh = {-170771.3922, -118.0013804, 10.08798517, -3.51385804e-005, 7.751187029999999e-009, -8.8724706e-013, 4.10481544e-017}, bhigh = {-51249.4414, -20.5925296}, R = 73.00637058132162);
          constant IdealGases.Common.DataRecord ALOF(name = "ALOF", MM = 0.0619793412, Hf = -9233554.147555217, H0 = 177526.9434454718, Tlimit = 1000, alow = {7030.57836, -115.457543, 3.37882359, 0.01283929158, -1.749748388e-005, 1.162605473e-008, -3.064218613e-012}, blow = {-69593.817, 5.994804586}, ahigh = {-164102.0297, -327.777029, 7.74451974, -9.78864944e-005, 2.165743208e-008, -2.486541919e-012, 1.153633944e-016}, bhigh = {-69740.85090000001, -17.17253748}, R = 134.1490864378533);
          constant IdealGases.Common.DataRecord ALOF2(name = "ALOF2", MM = 0.08097774440000001, Hf = -9553856.763142934, H0 = 173570.9867463288, Tlimit = 1000, alow = {47560.8529, -612.431836, 6.05022366, 0.01559291479, -2.366136797e-005, 1.688366194e-008, -4.690583099999999e-012}, blow = {-91718.2749, -4.956258224}, ahigh = {-249668.1413, -283.7821503, 10.21108125, -8.418303430000001e-005, 1.855520828e-008, -2.122974022e-012, 9.8193191e-017}, bhigh = {-95218.0419, -25.22534812}, R = 102.6760137813866);
          constant IdealGases.Common.DataRecord ALOF2minus(name = "ALOF2minus", MM = 0.08097829299999999, Hf = -12006793.21555963, H0 = 173836.4749180376, Tlimit = 1000, alow = {154060.5112, -1928.054597, 12.55806423, -0.000490403537, -2.341783347e-006, 2.4151386e-009, -7.47180925e-013}, blow = {-109143.0174, -42.6582175}, ahigh = {-238988.1459, -288.0143725, 10.2170024, -8.747061999999999e-005, 1.944846334e-008, -2.241045051e-012, 1.042591053e-016}, bhigh = {-119057.6566, -26.00779629}, R = 102.6753181868134);
          constant IdealGases.Common.DataRecord ALOH(name = "ALOH", MM = 0.043988878, Hf = -4382066.098617018, H0 = 235330.6442596694, Tlimit = 1000, alow = {58764.9318, -944.942269, 7.82059918, 0.000585888847, -4.08366681e-006, 4.587229340000001e-009, -1.563936726e-012}, blow = {-19932.83011, -20.65043885}, ahigh = {788206.811, -2263.671626, 7.82395488, 0.0001821171456, -8.26372932e-008, 1.265414876e-011, -6.87597253e-016}, bhigh = {-10398.08093, -22.09032458}, R = 189.0130500714294);
          constant IdealGases.Common.DataRecord ALOHCL(name = "ALOHCL", MM = 0.07944187800000001, Hf = -4705147.327962211, H0 = 175208.823739036, Tlimit = 1000, alow = {16363.60341, -191.5959416, 4.71117047, 0.01388068475, -1.999018454e-005, 1.43964735e-008, -4.03333045e-012}, blow = {-45680.83719999999, 4.644207506}, ahigh = {796711.551, -2843.470502, 10.9555997, -0.0001099328162, -1.344506283e-008, 4.41589377e-012, -2.973546112e-016}, bhigh = {-29698.61461, -32.96113996}, R = 104.661070575396);
          constant IdealGases.Common.DataRecord ALOHCL2(name = "ALOHCL2", MM = 0.114894878, Hf = -6311381.504752545, H0 = 149265.1917868784, Tlimit = 1000, alow = {39719.2986, -775.155496, 9.49405153, 0.01074401359, -1.676464791e-005, 1.259706368e-008, -3.61317798e-012}, blow = {-85847.9693, -19.93481642}, ahigh = {738592.976, -2863.204228, 13.97062327, -0.0001160311527, -1.208248796e-008, 4.25832763e-012, -2.900049181e-016}, bhigh = {-72930.48, -46.81834857}, R = 72.36590651151568);
          constant IdealGases.Common.DataRecord ALOHF(name = "ALOHF", MM = 0.0629872812, Hf = -9116309.897179686, H0 = 211393.1693244763, Tlimit = 1000, alow = {-1556.003336, 288.6581999, 0.7703793290000001, 0.02450406329, -3.41781101e-005, 2.388932902e-008, -6.570480510000001e-012}, blow = {-71772.22559999999, 24.53071801}, ahigh = {755711.3130000001, -2936.702803, 11.02455104, -0.000137307803, -7.432816159999999e-009, 3.72998122e-012, -2.657027781e-016}, bhigh = {-53401.309, -35.27397302}, R = 132.0023954296348);
          constant IdealGases.Common.DataRecord ALOHF2(name = "ALOHF2", MM = 0.0819856844, Hf = -13923293.06456335, H0 = 188185.2681100507, Tlimit = 1000, alow = {17382.63538, -130.7687299, 3.31238192, 0.02784734099, -3.98236321e-005, 2.809344456e-008, -7.7645742e-012}, blow = {-138413.3568, 10.04739945}, ahigh = {653011.992, -3034.017232, 14.09719788, -0.0001663681799, -1.011564183e-009, 2.993805003e-012, -2.315956955e-016}, bhigh = {-122305.2213, -51.5121069}, R = 101.4137048540635);
          constant IdealGases.Common.DataRecord ALO2(name = "ALO2", MM = 0.058980338, Hf = -655436.5795597849, H0 = 226543.7848118131, Tlimit = 1000, alow = {43384.8045, -473.5292259999999, 6.00171767, 0.007094420880000001, -1.129107996e-005, 8.252691679999999e-009, -2.327652976e-012}, blow = {-3826.1458, -4.83002248}, ahigh = {118721.6642, -833.56254, 8.309301189999999, -0.000353866722, 5.96706946e-008, 4.0148977e-014, -3.51570252e-016}, bhigh = {-2033.107586, -17.15063884}, R = 140.9702331648218);
          constant IdealGases.Common.DataRecord ALO2minus(name = "ALO2minus", MM = 0.0589808866, Hf = -7673201.423187831, H0 = 180476.720741597, Tlimit = 1000, alow = {117867.8641, -1507.186304, 9.52474975, -0.000520798902, -1.586902345e-006, 1.700455028e-009, -5.30141977e-013}, blow = {-48254.6927, -30.81215859}, ahigh = {-187241.0758, -233.8853263, 7.67607002, -7.093570299999999e-005, 1.576717569e-008, -1.816494162e-012, 8.449707959999999e-017}, bhigh = {-55946.40560000001, -17.73751567}, R = 140.9689219558121);
          constant IdealGases.Common.DataRecord AL_OH_2(name = "AL_OH_2", MM = 0.060996218, Hf = -8322822.080542764, H0 = 229973.8485425441, Tlimit = 1000, alow = {4397.31691, 132.7680339, 1.093947024, 0.03054059173, -4.37594335e-005, 3.17461743e-008, -9.004259400000001e-012}, blow = {-63154.3739, 21.01977655}, ahigh = {1669643.735, -5924.73828, 15.49480264, -0.000538348808, 5.413044529999999e-008, -1.196022328e-012, -1.082001733e-016}, bhigh = {-26975.25061, -66.17911543}, R = 136.3112709709314);
          constant IdealGases.Common.DataRecord AL_OH_2CL(name = "AL_OH_2CL", MM = 0.096449218, Hf = -8906829.332716828, H0 = 178560.6701342047, Tlimit = 1000, alow = {24778.79709, -445.457721, 5.9284702, 0.02730162734, -4.04140537e-005, 2.97421041e-008, -8.477150599999999e-012}, blow = {-103377.803, -3.679940779}, ahigh = {1603911.487, -5767.23224, 18.17772266, -0.000369181615, 1.137918702e-008, 4.0830794e-012, -3.64132343e-016}, bhigh = {-71186.9985, -77.57541774000001}, R = 86.20569634893255);
          constant IdealGases.Common.DataRecord AL_OH_2F(name = "AL_OH_2F", MM = 0.07999462119999999, Hf = -13371257.86652266, H0 = 205914.4821602081, Tlimit = 1000, alow = {21328.12662, -193.2900224, 3.069174664, 0.0355501733, -5.18115833e-005, 3.75352028e-008, -1.059220701e-011}, blow = {-129579.7992, 10.01897011}, ahigh = {1559238.236, -5841.27303, 18.23229919, -0.000390780542, 1.610928938e-008, 3.54478406e-012, -3.39345388e-016}, bhigh = {-96231.0089, -79.71351802}, R = 103.9378882639174);
          constant IdealGases.Common.DataRecord AL_OH_3(name = "AL_OH_3", MM = 0.078003558, Hf = -12982325.01137961, H0 = 225563.6467249353, Tlimit = 1000, alow = {-14024.75452, 369.607346, -0.297960065, 0.0492985935, -6.98157228e-005, 5.0148017e-008, -1.412480165e-011}, blow = {-125526.076, 27.11490074}, ahigh = {2477063.616, -8968.617249999999, 22.80320998, -0.0008320292249999999, 8.66349443e-008, -2.420287082e-012, -1.331746442e-016}, bhigh = {-70152.7864, -112.1899198}, R = 106.5909326854039);
          constant IdealGases.Common.DataRecord ALS(name = "ALS", MM = 0.059046538, Hf = 3940654.556241722, H0 = 153914.3412607866, Tlimit = 1000, alow = {26117.42801, -347.067389, 4.43620459, 0.00334016821, -8.41019967e-006, 8.50818511e-009, -2.780868507e-012}, blow = {28637.79062, 0.7522705491}, ahigh = {8909844.290000001, -25076.77454, 28.93456134, -0.00983976682, 2.045319809e-006, -2.08797089e-010, 8.152792520000001e-015}, bhigh = {188576.1615, -178.6036457}, R = 140.8121844501705);
          constant IdealGases.Common.DataRecord ALS2(name = "ALS2", MM = 0.09111153799999999, Hf = 2727813.452122826, H0 = 158363.90556814, Tlimit = 1000, alow = {42523.3541, -776.46033, 9.80106896, -0.00391068009, 3.87569597e-006, -2.082025593e-009, 4.68293589e-013}, blow = {31679.6154, -21.78097896}, ahigh = {-68525.6354, -26.71667739, 7.52019386, -8.148565100000001e-006, 1.811865152e-009, -2.086958125e-013, 9.703246379999999e-018}, bhigh = {27583.62737, -7.873490263}, R = 91.25597243238283);
          constant IdealGases.Common.DataRecord AL2(name = "AL2", MM = 0.053963076, Hf = 9289717.324490547, H0 = 187889.9935207548, Tlimit = 1000, alow = {-5281.50965, -17.27374523, 4.60407701, -0.000261646777, 6.30231997e-007, -3.29093859e-010, 8.888365139999999e-014}, blow = {59007.0639, 3.060188921}, ahigh = {-2320724.102, 9218.70789, -9.44695187, 0.00999992001, -3.154798085e-006, 4.36154481e-010, -2.24115724e-014}, bhigh = {2904.589544, 99.60320745000001}, R = 154.0770581721472);
          constant IdealGases.Common.DataRecord AL2Br6(name = "AL2Br6", MM = 0.533387076, Hf = -1766864.728083513, H0 = 71650.00938267952, Tlimit = 1000, alow = {64833.197, -1947.687811, 28.86989822, -0.01365825785, 1.557883731e-005, -9.486942000000001e-009, 2.385519396e-012}, blow = {-110152.6833, -102.5934782}, ahigh = {-173182.949, -43.42299780000001, 22.0341421, -1.421187503e-005, 3.23863134e-009, -3.80387976e-013, 1.796525455e-017}, bhigh = {-120233.8355, -62.0105495}, R = 15.58806422973773);
          constant IdealGases.Common.DataRecord AL2C2(name = "AL2C2", MM = 0.077984476, Hf = 6988288.284452922, H0 = 207284.8575657545, Tlimit = 1000, alow = {11109.87147, -350.243423, 8.598856, 0.001832528671, 1.035743392e-006, -2.09164989e-009, 7.716068700000001e-013}, blow = {64927.6419, -16.45080239}, ahigh = {159765.1967, -1644.007203, 11.64643675, -0.000437921543, 9.36964011e-008, -1.049615854e-011, 4.78046893e-016}, bhigh = {72048.89810000001, -36.48164252}, R = 106.6170143914284);
          constant IdealGases.Common.DataRecord AL2CL6(name = "AL2CL6", MM = 0.266681076, Hf = -4863023.134044952, H0 = 128044.6686063318, Tlimit = 1000, alow = {134093.5279, -3001.037953, 31.43000879, -0.01700408496, 1.786540089e-005, -1.015419943e-008, 2.409630451e-012}, blow = {-147183.0046, -127.4445324}, ahigh = {-275297.5554, -92.48235969999999, 22.07058224, -2.870537915e-005, 6.42332569e-009, -7.4365852e-013, 3.47206467e-017}, bhigh = {-162915.7014, -70.91602129}, R = 31.17758531917728);
          constant IdealGases.Common.DataRecord AL2F6(name = "AL2F6", MM = 0.1679534952, Hf = -15673929.42829332, H0 = 155186.3447019232, Tlimit = 1000, alow = {191506.1289, -3168.24025, 22.45374802, 0.01271532036, -2.488697302e-005, 1.955252096e-008, -5.71208489e-012}, blow = {-304996.255, -93.96473100999999}, ahigh = {-585298.2609999999, -545.0571990000001, 22.4068632, -0.0001627323704, 3.59522504e-008, -4.12120144e-012, 1.909095384e-016}, bhigh = {-321968.391, -85.89867941}, R = 49.50460834470327);
          constant IdealGases.Common.DataRecord AL2I6(name = "AL2I6", MM = 0.815389896, Hf = -598176.4520172568, H0 = 50185.76536297918, Tlimit = 1000, alow = {25875.71514, -1280.719742, 26.7771582, -0.009913018470000002, 1.168760413e-005, -7.30391853e-009, 1.874532749e-012}, blow = {-58911.0555, -83.30209321}, ahigh = {-121011.2399, -24.67542774, 22.0199265, -8.46202259e-006, 1.957831901e-009, -2.326573151e-013, 1.108910373e-017}, bhigh = {-65485.2375, -55.28536481}, R = 10.19692792465017);
          constant IdealGases.Common.DataRecord AL2O(name = "AL2O", MM = 0.069962476, Hf = -2124157.040982941, H0 = 182619.5230711961, Tlimit = 1000, alow = {7776.5307, -129.4235361, 4.91250952, 0.008604223449999999, -1.217703648e-005, 8.31463487e-009, -2.237722201e-012}, blow = {-18865.12879, -0.02806368311}, ahigh = {-117107.4351, -178.3009166, 7.63321536, -5.33593177e-005, 1.180702791e-008, -1.355444579e-012, 6.28732389e-017}, bhigh = {-19475.80149, -14.15764167}, R = 118.8418774658576);
          constant IdealGases.Common.DataRecord AL2Oplus(name = "AL2Oplus", MM = 0.0699619274, Hf = 9276048.732756898, H0 = 185545.8887772151, Tlimit = 1000, alow = {68289.25719999999, -909.850417, 8.89656301, -0.000777255438, -4.034655619999999e-007, 6.97731505e-010, -2.417262484e-013}, blow = {80850.07550000001, -21.76216731}, ahigh = {-110222.5105, -121.4571732, 7.59159595, -3.69453916e-005, 8.21852032e-009, -9.473649719999999e-013, 4.40862135e-017}, bhigh = {76149.43580000001, -12.82233856}, R = 118.842809353477);
          constant IdealGases.Common.DataRecord AL2O2(name = "AL2O2", MM = 0.08596187600000001, Hf = -4689236.737923215, H0 = 184298.1300221973, Tlimit = 1000, alow = {-19405.60042, 250.8489836, 3.62140379, 0.01951385302, -2.560329071e-005, 1.662721576e-008, -4.3123962e-012}, blow = {-51726.9728, 9.923995945}, ahigh = {-194061.1656, -460.975243, 10.84375637, -0.0001376042893, 3.044733119e-008, -3.49619392e-012, 1.622305079e-016}, bhigh = {-49630.5578, -29.4653809}, R = 96.72278441201074);
          constant IdealGases.Common.DataRecord AL2O2plus(name = "AL2O2plus", MM = 0.0859613274, Hf = 6484762.995877144, H0 = 174207.8147574069, Tlimit = 1000, alow = {82920.3499, -1757.427015, 15.25328567, -0.008983131330000001, 8.9539545e-006, -4.8405862e-009, 1.096696856e-012}, blow = {73116.7714, -55.1709255}, ahigh = {-165200.5184, -60.2160697, 10.04626917, -1.892194088e-005, 4.2533012e-009, -4.942390419999999e-013, 2.314507373e-017}, bhigh = {63862.19, -23.45753361}, R = 96.72340169097949);
          constant IdealGases.Common.DataRecord AL2O3(name = "AL2O3", MM = 0.101961276, Hf = -5363708.178779559, H0 = 192210.9134844487, Tlimit = 1000, alow = {-7443.37432, 88.29004210000001, 5.26466264, 0.02507678848, -3.43454165e-005, 2.30251698e-008, -6.12252928e-012}, blow = {-68726.85950000001, 2.202324298}, ahigh = {-277778.4969, -491.746593, 13.86703888, -0.000146938194, 3.25040649e-008, -3.73086735e-012, 1.730444284e-016}, bhigh = {-67907.5785, -43.75559873}, R = 81.54538983996238);
          constant IdealGases.Common.DataRecord AL2S(name = "AL2S", MM = 0.086028076, Hf = 2565198.959000316, H0 = 162783.2639195604, Tlimit = 1000, alow = {40946.2437, -779.7172469999999, 9.814423100000001, -0.00394033982, 3.91246543e-006, -2.105891677e-009, 4.74600108e-013}, blow = {28339.63305, -24.65067552}, ahigh = {-70434.3737, -26.76376869, 7.52023382, -8.16612306e-006, 1.81602604e-009, -2.091989298e-013, 9.727537699999999e-018}, bhigh = {24227.18377, -10.66580982}, R = 96.64835466040181);
          constant IdealGases.Common.DataRecord AL2S2(name = "AL2S2", MM = 0.118093076, Hf = 1145599.09507311, H0 = 153264.8789671632, Tlimit = 1000, alow = {66145.8732, -1111.319537, 12.57007304, -0.001456053256, -5.85162711e-007, 1.390055053e-009, -5.4964682e-013}, blow = {19144.56895, -36.00277092}, ahigh = {-137780.6456, -77.050325, 10.55723566, -2.277554487e-005, 5.00768285e-009, -5.715862119999999e-013, 2.63799924e-017}, bhigh = {13133.36091, -22.56851594}, R = 70.40609222508525);
          constant IdealGases.Common.DataRecord Ar(name = "Ar", MM = 0.039948, Hf = 0, H0 = 155137.3785921698, Tlimit = 1000, alow = {0, 0, 2.5, 0, 0, 0, 0}, blow = {-745.375, 4.37967491}, ahigh = {20.10538475, -0.05992661069999999, 2.500069401, -3.99214116e-008, 1.20527214e-011, -1.819015576e-015, 1.078576636e-019}, bhigh = {-744.993961, 4.37918011}, R = 208.1323720837088);
          constant IdealGases.Common.DataRecord Arplus(name = "Arplus", MM = 0.0399474514, Hf = 38219669.92868035, H0 = 155353.7906050247, Tlimit = 1000, alow = {-57312.0917, 793.079147, -1.717121217, 0.01044184018, -1.180207501e-005, 6.52813478e-009, -1.44755813e-012}, blow = {179057.223, 29.4915095}, ahigh = {-383596.54, 816.20197, 2.301342628, -4.95298377e-006, 1.205108477e-008, -2.185050286e-012, 1.265493898e-016}, bhigh = {177181.1455, 7.94750748}, R = 208.135230374171);
          constant IdealGases.Common.DataRecord B(name = "B", MM = 0.010811, Hf = 53241953.5658126, H0 = 584225.326056794, Tlimit = 1000, alow = {118.2394638, -0.0700991691, 2.500236159, -4.5842137e-007, 5.12318583e-010, -3.057217674e-013, 7.533815325e-017}, blow = {68483.59080000001, 4.20950192}, ahigh = {-107265.961, 322.530716, 2.126407232, 0.0002106579339, -5.93712916e-008, 7.37742799e-012, -2.282443381e-016}, bhigh = {66434.13099999999, 6.87706967}, R = 769.0752011839794);
          constant IdealGases.Common.DataRecord Bplus(name = "Bplus", MM = 0.0108104514, Hf = 127868437.3901352, H0 = 573281.1490184396, Tlimit = 1000, alow = {0.07849791190000001, -0.000894748095, 2.500004085, -9.577271229999999e-009, 1.218136411e-011, -7.98675252e-015, 2.113769829e-018}, blow = {165508.026, 2.419053631}, ahigh = {-8911.54803, 4.58779009, 2.531500086, -4.9039491e-005, 2.853326582e-008, -7.38217591e-012, 7.12072156e-016}, bhigh = {165452.6303, 2.23866978}, R = 769.1142295871198);
          constant IdealGases.Common.DataRecord Bminus(name = "Bminus", MM = 0.0108115486, Hf = 50189988.32415183, H0 = 580175.7206178586, Tlimit = 1000, alow = {22.01568105, -0.00474046888, 2.500014238, -2.497056599e-008, 2.556360708e-011, -1.41527478e-014, 3.27177071e-018}, blow = {64517.9188, 4.61636729}, ahigh = {21.18018248, 0.0002070697496, 2.499999729, 1.456693631e-010, -3.88857176e-014, 5.09059863e-018, -2.601566168e-022}, bhigh = {64517.8914, 4.61645583}, R = 769.0361767416002);
          constant IdealGases.Common.DataRecord BBr(name = "BBr", MM = 0.090715, Hf = 2656146.81144243, H0 = 99179.38598908672, Tlimit = 1000, alow = {37960.935, -497.609602, 5.2670082, -3.74284019e-005, -1.13154437e-006, 1.261945702e-009, -4.28881188e-013}, blow = {30381.25367, -4.35340733}, ahigh = {253685.8027, -640.59095, 4.66682347, 0.000417548639, -2.963714868e-007, 7.51911954e-011, -4.83536832e-015}, bhigh = {31890.8434, -0.8255870910000001}, R = 91.65487515846331);
          constant IdealGases.Common.DataRecord BBr2(name = "BBr2", MM = 0.170619, Hf = 573374.6944947515, H0 = 71508.54828594705, Tlimit = 1000, alow = {65165.5071, -902.959076, 8.063817800000001, 0.00112984901, -4.19548239e-006, 3.96565877e-009, -1.283029835e-012}, blow = {14704.66408, -13.36453164}, ahigh = {-419163.337, 415.768847, 7.14816983, -0.000568959075, 3.19544177e-007, -5.55317699e-011, 3.19113676e-015}, bhigh = {6118.98205, -5.50275962}, R = 48.73121985241972);
          constant IdealGases.Common.DataRecord BBr3(name = "BBr3", MM = 0.250523, Hf = -819485.6360493847, H0 = 62680.06131173585, Tlimit = 1000, alow = {39680.7369, -633.108716, 8.054551569999999, 0.009777571299999999, -1.614866025e-005, 1.216119883e-008, -3.51653961e-012}, blow = {-23667.23308, -11.0609939}, ahigh = {-190104.6795, -150.3627687, 10.10999896, -4.32309511e-005, 9.409696190000001e-009, -1.06521847e-012, 4.883048770000001e-017}, bhigh = {-27425.53062, -19.97496003}, R = 33.18845774639455);
          constant IdealGases.Common.DataRecord BC(name = "BC", MM = 0.0228217, Hf = 36726532.42308855, H0 = 382754.3522174071, Tlimit = 1000, alow = {-39157.58, 728.453806, -1.552743361, 0.01552898673, -1.976857863e-005, 1.272066405e-008, -3.22998278e-012}, blow = {96449.13680000001, 32.48204466}, ahigh = {-2346280.674, 6450.7513, -2.619532384, 0.0033391605, -4.50802214e-007, 1.351919576e-011, 8.265104270000001e-016}, bhigh = {57866.8507, 50.37884876}, R = 364.323078473558);
          constant IdealGases.Common.DataRecord BC2(name = "BC2", MM = 0.0348324, Hf = 23003259.17823635, H0 = 335544.6366027033, Tlimit = 1000, alow = {-29874.22175, 364.312658, 2.963469415, 0.00656175015, -3.53976724e-006, 1.927356029e-010, 3.118724866e-013}, blow = {93048.3573, 10.83424193}, ahigh = {1525485.889, -5987.862380000001, 14.08384855, -0.0037474998, 1.081927223e-006, -1.378744645e-010, 6.473822e-015}, bhigh = {131120.4013, -63.37215498}, R = 238.6993718491979);
          constant IdealGases.Common.DataRecord BCL(name = "BCL", MM = 0.046264, Hf = 3959308.25263704, H0 = 191538.4964551271, Tlimit = 1000, alow = {22024.58989, -170.0511155, 3.066075869, 0.00559507018, -8.46087041e-006, 6.08473341e-009, -1.702889433e-012}, blow = {21974.02537, 6.388978917}, ahigh = {-74212.6254, 263.8090127, 3.60022765, 0.001018866266, -4.666184119999999e-007, 9.849002029999999e-011, -6.46881817e-015}, bhigh = {19127.2648, 5.235317877}, R = 179.7179664533979);
          constant IdealGases.Common.DataRecord BCLplus(name = "BCLplus", MM = 0.0462634514, Hf = 26679375.67667076, H0 = 191518.7417253525, Tlimit = 1000, alow = {65144.0542, -684.554101, 5.52727682, -0.000445835745, -4.55772884e-007, 6.37085816e-010, -2.13436612e-013}, blow = {150942.4507, -6.918246393}, ahigh = {-216942.3542, 358.997516, 4.03520837, 0.000327445243, -8.63457845e-008, 1.207061044e-011, -5.262253270000001e-016}, bhigh = {144547.7224, 3.488222017}, R = 179.7200975800954);
          constant IdealGases.Common.DataRecord BCLOH(name = "BCLOH", MM = 0.06327134, Hf = -3698441.996012729, H0 = 196479.4644779137, Tlimit = 1000, alow = {-28984.27288, 689.404658, -2.288138069, 0.0334942862, -4.72522598e-005, 3.32698131e-008, -9.224238860000001e-012}, blow = {-32619.811, 39.76886651}, ahigh = {702773.844, -2857.391274, 10.9117584, -8.10770934e-005, -2.124440337e-008, 5.40991244e-012, -3.46415384e-016}, bhigh = {-13072.21888, -35.85168799}, R = 131.4097662543578);
          constant IdealGases.Common.DataRecord BCL_OH_2(name = "BCL_OH_2", MM = 0.08027867999999999, Hf = -10032397.8670302, H0 = 168975.1625213569, Tlimit = 1000, alow = {80726.23009999999, -859.461982, 2.705039614, 0.0387879962, -5.72173435e-005, 4.14585033e-008, -1.168536754e-011}, blow = {-93799.10739999999, 7.134982695}, ahigh = {1423878.191, -5849.70407, 17.92165219, -0.0002009909087, -3.39343578e-008, 9.838560759999999e-012, -6.47267882e-016}, bhigh = {-64742.6176, -81.00399519}, R = 103.5701135095894);
          constant IdealGases.Common.DataRecord BCL2(name = "BCL2", MM = 0.081717, Hf = -745028.4518521237, H0 = 140896.2761726446, Tlimit = 1000, alow = {35988.7942, -360.28265, 4.15768545, 0.0115287745, -1.816567395e-005, 1.341714418e-008, -3.84082286e-012}, blow = {-6765.107050000001, 5.182311874}, ahigh = {350140.313, -1662.20359, 8.83953472, -0.000977254411, 2.467356248e-007, -2.501151018e-011, 8.722954559999999e-016}, bhigh = {568.2909440000001, -21.48348344}, R = 101.7471517554487);
          constant IdealGases.Common.DataRecord BCL2plus(name = "BCL2plus", MM = 0.0817164514, Hf = 8227417.985994433, H0 = 157236.4166562426, Tlimit = 1000, alow = {80659.5624, -1149.602601, 10.16765297, -0.00358649491, 2.837145924e-006, -1.225075731e-009, 2.224794198e-013}, blow = {84786.1741, -29.37256994}, ahigh = {302929.3331, -1305.533159, 8.920390060000001, -0.000728991964, 1.727833266e-007, -1.523567337e-011, 4.22691144e-016}, bhigh = {86534.15770000001, -22.67466616}, R = 101.747834830723);
          constant IdealGases.Common.DataRecord BCL2OH(name = "BCL2OH", MM = 0.09872433999999999, Hf = -6127338.445615337, H0 = 142333.3799952474, Tlimit = 1000, alow = {20997.5517, -247.0699169, 2.578761742, 0.03120362393, -4.54531405e-005, 3.24201177e-008, -9.039822660000001e-012}, blow = {-73090.2862, 12.73658611}, ahigh = {587811.379, -2982.969317, 14.00662004, -0.0001194189832, -1.269775742e-008, 4.42289219e-012, -3.004089365e-016}, bhigh = {-58241.6807, -51.50858228}, R = 84.2190689752902);
          constant IdealGases.Common.DataRecord BF(name = "BF", MM = 0.0298094032, Hf = -3587186.341254897, H0 = 291675.4133474233, Tlimit = 1000, alow = {-52389.5473, 811.8476640000001, -1.141614903, 0.01161249417, -1.175212617e-005, 6.01923278e-009, -1.238293129e-012}, blow = {-17745.41998, 30.05086287}, ahigh = {-374638.978, 560.449391, 3.60918611, 0.000618721693, -1.77893877e-007, 2.426601527e-011, -9.394651579999999e-016}, bhigh = {-18191.79292, 3.71660929}, R = 278.9211157370638);
          constant IdealGases.Common.DataRecord BFCL(name = "BFCL", MM = 0.0652624032, Hf = -4277869.865509336, H0 = 169056.6001100003, Tlimit = 1000, alow = {-23635.12735, 438.316898, 0.391809572, 0.01719990692, -2.139592424e-005, 1.337389186e-008, -3.37116469e-012}, blow = {-36871.6659, 26.68102752}, ahigh = {-108630.9561, -523.343848, 7.29750474, -3.054410701e-005, -3.154598306e-008, 1.052290533e-011, -7.615432260000001e-016}, bhigh = {-33058.9121, -11.86586595}, R = 127.4006409865091);
          constant IdealGases.Common.DataRecord BFCL2(name = "BFCL2", MM = 0.1007154032, Hf = -6384326.325171282, H0 = 131369.3097542005, Tlimit = 1000, alow = {4402.55124, -157.1398929, 4.01632231, 0.01743396078, -2.25278906e-005, 1.433673443e-008, -3.64875558e-012}, blow = {-78224.52619999999, 6.886094746}, ahigh = {-223361.3309, -644.060296, 10.47861848, -0.0001911892367, 4.22488779e-008, -4.84726692e-012, 2.247974725e-016}, bhigh = {-77406.2648, -28.14630564}, R = 82.5541251469666);
          constant IdealGases.Common.DataRecord BFOH(name = "BFOH", MM = 0.0468167432, Hf = -9646798.284764072, H0 = 255744.4448634778, Tlimit = 1000, alow = {-75639.5367, 1354.838128, -5.64821011, 0.0387027466, -5.03765918e-005, 3.33959264e-008, -8.847822949999999e-012}, blow = {-61944.4611, 58.0036282}, ahigh = {725131.809, -3215.62085, 11.17909777, -0.0001883881929, 2.57970808e-009, 2.6651581e-012, -2.186647438e-016}, bhigh = {-37168.0808, -39.7846349}, R = 177.5961212099008);
          constant IdealGases.Common.DataRecord BF_OH_2(name = "BF_OH_2", MM = 0.06382408319999999, Hf = -16449739.44568937, H0 = 201886.4878892612, Tlimit = 1000, alow = {13818.00676, 252.9637004, -3.84455204, 0.0525514085, -7.19311773e-005, 4.94830991e-008, -1.345922496e-011}, blow = {-128312.3258, 42.9365937}, ahigh = {1422037.64, -6257.297570000001, 18.22460686, -0.000322139645, -7.1267399e-009, 6.75869977e-012, -5.04255454e-016}, bhigh = {-91862.5419, -85.27895049999999}, R = 130.2717028295677);
          constant IdealGases.Common.DataRecord BF2(name = "BF2", MM = 0.04880780639999999, Hf = -10232521.00098479, H0 = 217428.3743266118, Tlimit = 1000, alow = {-67876.51760000001, 1085.903536, -3.023320961, 0.02326503687, -2.641444147e-005, 1.515620683e-008, -3.51855918e-012}, blow = {-66409.18250000001, 44.31968310000001}, ahigh = {-115309.1296, -810.9800119999999, 7.60270923, -0.0002409209242, 5.32847186e-008, -6.11879794e-012, 2.839984178e-016}, bhigh = {-57962.2217, -16.55644047}, R = 170.3512739716162);
          constant IdealGases.Common.DataRecord BF2plus(name = "BF2plus", MM = 0.0488072578, Hf = 6609394.064339341, H0 = 217436.5755906082, Tlimit = 1000, alow = {-29383.56477, 295.5698598, 1.894274526, 0.01280508319, -1.388757254e-005, 7.62436675e-009, -1.706339371e-012}, blow = {35989.9244, 13.84803311}, ahigh = {-176940.9966, -534.368518, 7.72805291, -1.958622052e-006, -2.419680893e-008, 5.73253381e-012, -3.21635423e-016}, bhigh = {39087.4505, -19.36560315}, R = 170.3531887423513);
          constant IdealGases.Common.DataRecord BF2minus(name = "BF2minus", MM = 0.048808355, Hf = -15034371.94308229, H0 = 213766.0857449509, Tlimit = 1000, alow = {21097.65318, 143.95646, 0.2604646296, 0.01817373751, -2.241565287e-005, 1.369712886e-008, -3.35595268e-012}, blow = {-89718.0178, 23.51692604}, ahigh = {-170992.0627, -615.621979, 7.45887137, -0.0001836703248, 4.064363959999999e-008, -4.66770095e-012, 2.16626484e-016}, bhigh = {-87414.0453, -16.20604664}, R = 170.3493592439245);
          constant IdealGases.Common.DataRecord BF2CL(name = "BF2CL", MM = 0.08426080640000001, Hf = -10538707.59062662, H0 = 146256.5281122208, Tlimit = 1000, alow = {4500.10125, -119.6284966, 3.19863514, 0.01743426933, -1.994070287e-005, 1.12513643e-008, -2.544020371e-012}, blow = {-107677.9851, 10.08195015}, ahigh = {-206681.7398, -1024.447292, 10.75843981, -0.0003023235319, 6.67252147e-008, -7.6499419e-012, 3.5462245e-016}, bhigh = {-104693.6725, -32.29995036}, R = 98.67543826402306);
          constant IdealGases.Common.DataRecord BF2OH(name = "BF2OH", MM = 0.0658151464, Hf = -16595227.41713448, H0 = 188343.9858153989, Tlimit = 1000, alow = {-7522.52165, 436.712169, -3.128973474, 0.04323957680000001, -5.78549404e-005, 3.88585813e-008, -1.038574523e-011}, blow = {-134425.9458, 41.3917055}, ahigh = {576075.9250000001, -3611.90019, 14.4689764, -0.0003028293733, 2.764337508e-008, -1.90549317e-013, -8.69497194e-017}, bhigh = {-113358.9041, -58.9698151}, R = 126.3306769762044);
          constant IdealGases.Common.DataRecord BF3(name = "BF3", MM = 0.0678062096, Hf = -16753627.82703017, H0 = 171831.2536378674, Tlimit = 1000, alow = {3465.584, 21.33198651, 1.641245191, 0.01993755064, -2.15011993e-005, 1.145669081e-008, -2.442285789e-012}, blow = {-137945.5591, 16.25533544}, ahigh = {-181976.7014, -1405.347931, 11.03412258, -0.000410459105, 9.031277570000001e-008, -1.03305736e-011, 4.780551830000001e-016}, bhigh = {-132313.6863, -37.3838608}, R = 122.6210998822739);
          constant IdealGases.Common.DataRecord BF4minus(name = "BF4minus", MM = 0.0868051614, Hf = -20289876.42663378, H0 = 158926.5289932403, Tlimit = 1000, alow = {206848.52, -2463.190805, 12.06414196, 0.01102766923, -1.75172076e-005, 1.201346256e-008, -3.150588626e-012}, blow = {-201056.9206, -46.1082694}, ahigh = {-437324.171, -775.928941, 13.58222575, -0.0002341024178, 5.19691594e-008, -5.982245780000001e-012, 2.781163206e-016}, bhigh = {-212725.1877, -49.533697}, R = 95.783152359878);
          constant IdealGases.Common.DataRecord BH(name = "BH", MM = 0.01181894, Hf = 37966789.23829041, H0 = 730954.2141681063, Tlimit = 1000, alow = {20630.8255, -368.250252, 6.07133787, -0.00872832107, 1.458566459e-005, -1.036840401e-008, 2.779462579e-012}, blow = {54604.5992, -13.00392582}, ahigh = {-1098531.663, -174.5890126, 8.442426810000001, -0.00544019667, 2.718307052e-006, -4.83981221e-010, 2.868523222e-014}, bhigh = {50167.3567, -29.71030686}, R = 703.4871147497153);
          constant IdealGases.Common.DataRecord BHCL(name = "BHCL", MM = 0.04727194, Hf = 2991590.931110507, H0 = 217989.9322938725, Tlimit = 1000, alow = {47632.7491, -442.001652, 3.97564588, 0.008541698759999999, -1.377785182e-005, 1.131521587e-008, -3.56965918e-012}, blow = {18222.78428, 3.028452399}, ahigh = {349999.172, -2238.404757, 8.71110605, -0.001088882906, 4.18930109e-007, -6.56871126e-011, 3.6200676e-015}, bhigh = {28412.40364, -25.89566225}, R = 175.8859907166916);
          constant IdealGases.Common.DataRecord BHCL2(name = "BHCL2", MM = 0.08272494, Hf = -3044831.81251023, H0 = 141984.170674527, Tlimit = 1000, alow = {44199.1481, -238.4739577, 1.356682235, 0.02517866897, -3.6329104e-005, 2.644335581e-008, -7.62504411e-012}, blow = {-30038.77914, 17.87024616}, ahigh = {411099.606, -2858.027441, 11.79547506, -0.000632281797, 1.269551549e-007, -1.352862344e-011, 5.92165351e-016}, bhigh = {-16172.8876, -42.17900972}, R = 100.5074406823384);
          constant IdealGases.Common.DataRecord BHF(name = "BHF", MM = 0.0308173432, Hf = -2466544.325599099, H0 = 325857.3243912862, Tlimit = 1000, alow = {-49083.6226, 961.0760210000001, -2.919850316, 0.02149156633, -2.595677863e-005, 1.693492714e-008, -4.55364884e-012}, blow = {-14669.25195, 41.6070931}, ahigh = {1184821.3, -4677.43518, 10.67328148, -0.001606366598, 3.7045347e-007, -3.79788275e-011, 1.432441145e-015}, bhigh = {18026.40937, -42.85931169999999}, R = 269.798468545465);
          constant IdealGases.Common.DataRecord BHFCL(name = "BHFCL", MM = 0.0662703432, Hf = -7288880.616510825, H0 = 172242.0384266246, Tlimit = 1000, alow = {576.116972, 304.215625, -0.9808900360000001, 0.02803843648, -3.67673402e-005, 2.499824138e-008, -6.87342164e-012}, blow = {-60502.0739, 31.32842092}, ahigh = {439077.892, -3137.181709, 12.00286161, -0.000715366934, 1.453878608e-007, -1.56519707e-011, 6.910075059999999e-016}, bhigh = {-42323.6834, -44.66542881}, R = 125.4629386014708);
          constant IdealGases.Common.DataRecord BHF2(name = "BHF2", MM = 0.0498157464, Hf = -14846988.44138969, H0 = 213975.4549577521, Tlimit = 1000, alow = {-83535.40280000001, 1616.659766, -8.161405670000001, 0.0415378679, -4.92106328e-005, 3.054165306e-008, -7.78681398e-012}, blow = {-97480.65459999999, 70.3863075}, ahigh = {466087.6960000001, -3728.10049, 12.44152658, -0.000890684319, 1.841732568e-007, -2.010774349e-011, 8.979196590000001e-016}, bhigh = {-69781.4293, -51.0321804}, R = 166.904495081499);
          constant IdealGases.Common.DataRecord BH2(name = "BH2", MM = 0.01282688, Hf = 25642171.6738599, H0 = 781449.5029188704, Tlimit = 1000, alow = {28125.57296, -300.0083489, 4.824221, -0.0002186429819, 1.485457398e-006, 3.89373968e-010, -6.331629389999999e-013}, blow = {39919.8842, -5.04268285}, ahigh = {1360117.365, -4917.70449, 9.75897103, -0.000741587064, 1.269760019e-007, -1.187742442e-011, 4.68255251e-016}, bhigh = {68902.92660000001, -41.9049012}, R = 648.2068905298874);
          constant IdealGases.Common.DataRecord BH2CL(name = "BH2CL", MM = 0.04827988, Hf = -1674522.679012458, H0 = 215967.8524470235, Tlimit = 1000, alow = {16466.38117, 127.7320942, 0.1300268921, 0.01938143193, -2.316389616e-005, 1.607896164e-008, -4.66833765e-012}, blow = {-11121.12215, 23.08500112}, ahigh = {1427215.437, -6265.56434, 13.82183005, -0.001313843556, 2.587131606e-007, -2.71384527e-011, 1.172827257e-015}, bhigh = {26393.87516, -63.99725308}, R = 172.2140154449431);
          constant IdealGases.Common.DataRecord BH2F(name = "BH2F", MM = 0.0318252832, Hf = -10179225.42791387, H0 = 318549.3098769974, Tlimit = 1000, alow = {-88379.2044, 1705.285929, -8.061246130000001, 0.0364318868, -4.15248255e-005, 2.620135756e-008, -6.930886410000001e-012}, blow = {-47872.7341, 68.6208767}, ahigh = {1435082.111, -6625.32017, 14.09165008, -0.00142246906, 2.828724718e-007, -2.992498948e-011, 1.302618926e-015}, bhigh = {-793.719267, -68.013564}, R = 261.2536689068646);
          constant IdealGases.Common.DataRecord BH3(name = "BH3", MM = 0.01383482, Hf = 7571229.622069531, H0 = 727129.0121591751, Tlimit = 1000, alow = {-66196.3507, 1262.658391, -4.6543559, 0.02461795131, -2.501537437e-005, 1.562330756e-008, -4.32394948e-012}, blow = {5667.58798, 46.66504620000001}, ahigh = {1855778.95, -8002.492370000001, 15.05692199, -0.001790456689, 3.6125111e-007, -3.86603591e-011, 1.698508879e-015}, bhigh = {59675.3707, -79.94046159999999}, R = 600.9815812565686);
          constant IdealGases.Common.DataRecord BH3NH3(name = "BH3NH3", MM = 0.03086534, Hf = -3725862.083489118, H0 = 410866.3633706935, Tlimit = 1000, alow = {-181106.3598, 3010.341543, -15.24271135, 0.0621563775, -5.95988185e-005, 3.26742936e-008, -7.75124309e-012}, blow = {-29342.78877, 108.7399608}, ahigh = {4354925.96, -18242.35232, 31.6299983, -0.00320824361, 5.88507545e-007, -5.80853553e-011, 2.3825786e-015}, bhigh = {94381.18030000001, -189.7865799}, R = 269.3789214698429);
          constant IdealGases.Common.DataRecord BH4(name = "BH4", MM = 0.01484276, Hf = 17194273.6391345, H0 = 725665.3749033199, Tlimit = 1000, alow = {27342.74578, -84.73619380000001, 1.551871695, 0.01439765491, -6.11115236e-006, -1.324417559e-010, 5.13300815e-013}, blow = {30220.48074, 12.50358328}, ahigh = {1354714.146, -7870.76275, 18.2617061, -0.001948275329, 4.07270689e-007, -4.4827109e-011, 2.014115885e-015}, bhigh = {74702.49489999999, -96.8608689}, R = 560.1702109311207);
          constant IdealGases.Common.DataRecord BI(name = "BI", MM = 0.13771547, Hf = 2367108.807746871, H0 = 66384.71335137585, Tlimit = 1000, alow = {35174.4953, -544.5819300000001, 6.02751101, -0.002387237926, 2.244796277e-006, -1.110001347e-009, 2.280204403e-013}, blow = {40719.1013, -7.28804984}, ahigh = {2132872.626, -6283.684960000001, 11.23964343, -0.00325885675, 7.06428215e-007, -4.23006893e-011, -4.20807847e-016}, bhigh = {77976.3481, -46.6404261}, R = 60.37427748676311);
          constant IdealGases.Common.DataRecord BI2(name = "BI2", MM = 0.26461994, Hf = 899767.1301716719, H0 = 47949.61407670186, Tlimit = 1000, alow = {66409.75900000001, -1044.411994, 9.73419724, -0.00395255754, 3.16556778e-006, -1.273764644e-009, 1.879968132e-013}, blow = {32057.5089, -20.12407058}, ahigh = {-392402.223, 442.290235, 7.12962358, -0.000561963406, 3.18076052e-007, -5.537079720000001e-011, 3.1839597e-015}, bhigh = {22919.72638, -3.20179324}, R = 31.42042886110548);
          constant IdealGases.Common.DataRecord BI3(name = "BI3", MM = 0.39152441, Hf = 54658.1501776607, H0 = 43249.08375444586, Tlimit = 1000, alow = {62431.1489, -1002.408628, 11.06650075, 0.001609588058, -5.14725025e-006, 4.73251402e-009, -1.512999796e-012}, blow = {5160.34815, -24.20687643}, ahigh = {-150617.6506, -81.67210609999999, 10.05937867, -2.319396215e-005, 5.01996673e-009, -5.65434963e-013, 2.580651348e-017}, bhigh = {-426.168791, -16.17962779}, R = 21.23615230018481);
          constant IdealGases.Common.DataRecord BN(name = "BN", MM = 0.0248177, Hf = 23157923.90108673, H0 = 361077.2956398055, Tlimit = 1000, alow = {-45697.0758, 670.4286070000001, 0.0361508908, 0.007339487509999999, -4.96903349e-006, 1.22903138e-009, 6.34028152e-014}, blow = {64854.6508, 25.39869896}, ahigh = {-227693.2705, -102.5649298, 4.41458681, 0.0002561670989, -1.612994942e-008, -6.526052929999999e-013, 5.74946612e-017}, bhigh = {67844.6513, -0.6778568656}, R = 335.0218593987356);
          constant IdealGases.Common.DataRecord BO(name = "BO", MM = 0.0268104, Hf = 761137.6182376987, H0 = 323535.0461015128, Tlimit = 1000, alow = {-11662.16822, 92.17579389999999, 3.65549849, -0.00311854292, 9.00832983e-006, -8.017789990000001e-009, 2.472952292e-012}, blow = {873.8284780000001, 4.48284739}, ahigh = {17886.00589, -630.901963, 4.5745284, 0.0001988001643, -9.70296348e-008, 1.870854291e-011, -1.030218131e-015}, bhigh = {4841.311089999999, -3.39889058}, R = 310.1211470175753);
          constant IdealGases.Common.DataRecord BOminus(name = "BOminus", MM = 0.0268109486, Hf = -10361105.8356958, H0 = 323525.4421397086, Tlimit = 1000, alow = {-82420.23209999999, 920.3636899999999, -0.2302659981, 0.006229904119999999, -3.157624391e-006, 1.184578107e-010, 2.819228693e-013}, blow = {-39111.4122, 25.99226838}, ahigh = {212280.4369, -1262.124688, 5.38316731, -0.00031810491, 7.28898825e-008, -8.141708919999999e-012, 3.71536439e-016}, bhigh = {-27068.92261, -9.77375367}, R = 310.1148013837899);
          constant IdealGases.Common.DataRecord BOCL(name = "BOCL", MM = 0.0622634, Hf = -5115954.075749156, H0 = 170371.5666025305, Tlimit = 1000, alow = {70523.8064, -1315.301429, 11.30087727, -0.0120711205, 1.880516131e-005, -1.373823975e-008, 3.85489016e-012}, blow = {-33553.991, -36.98462517}, ahigh = {155714.8487, -1446.254968, 8.50944805, -0.000385900738, 8.26226882e-008, -9.261027459999999e-012, 4.21999801e-016}, bhigh = {-32035.8904, -23.72713292}, R = 133.5370699319344);
          constant IdealGases.Common.DataRecord BOCL2(name = "BOCL2", MM = 0.0977164, Hf = -3700151.786189422, H0 = 134774.2037160599, Tlimit = 1000, alow = {7322.66736, -202.7726965, 4.16192409, 0.01733309456, -2.263908718e-005, 1.45227519e-008, -3.71959312e-012}, blow = {-44144.4188, 6.52042411}, ahigh = {-229752.7041, -619.456793, 10.46075518, -0.0001841620739, 4.07121543e-008, -4.67224655e-012, 2.167237536e-016}, bhigh = {-43715.3842, -27.43570215}, R = 85.08778465027366);
          constant IdealGases.Common.DataRecord BOF(name = "BOF", MM = 0.0458088032, Hf = -12944636.61080759, H0 = 218043.1336830909, Tlimit = 1000, alow = {72268.02649999999, -1131.38663, 8.81605452, -0.00462769457, 7.80129958e-006, -5.681205839999999e-009, 1.534732094e-012}, blow = {-67111.16009999999, -25.47957229}, ahigh = {193702.0297, -1739.112297, 8.695650860000001, -0.000451889557, 9.59101908e-008, -1.067806769e-011, 4.840007660000001e-016}, bhigh = {-63301.5239, -27.02458617}, R = 181.5038031816557);
          constant IdealGases.Common.DataRecord BOF2(name = "BOF2", MM = 0.0648072064, Hf = -12849924.35656045, H0 = 179169.2412774639, Tlimit = 1000, alow = {7227.79413, 21.48783624, 1.157840301, 0.02250585296, -2.610731536e-005, 1.508824929e-008, -3.51836485e-012}, blow = {-101399.5733, 20.10127136}, ahigh = {-220502.8861, -1254.23869, 10.9255138, -0.0003680144, 8.10716494e-008, -9.28127506e-012, 4.2975493e-016}, bhigh = {-96804.47579999999, -34.8285333}, R = 128.2954853613317);
          constant IdealGases.Common.DataRecord BOH(name = "BOH", MM = 0.02781834, Hf = -242882.1417812853, H0 = 360608.2893515573, Tlimit = 1000, alow = {-75214.1027, 1391.444703, -5.63033018, 0.02966358811, -3.83083518e-005, 2.550606424e-008, -6.79585721e-012}, blow = {-8341.300280000001, 55.17690039999999}, ahigh = {844749.8570000001, -3016.982887, 8.0838647, -0.0001609855388, -2.191830959e-009, 3.127133119e-012, -2.376665451e-016}, bhigh = {16476.98495, -26.03227947}, R = 298.8845488264217);
          constant IdealGases.Common.DataRecord BO2(name = "BO2", MM = 0.0428098, Hf = -7220822.031404024, H0 = 251628.2953903078, Tlimit = 1000, alow = {-41410.9064, 719.885461, -1.477629435, 0.02277415194, -2.786326934e-005, 1.718462069e-008, -4.27899144e-012}, blow = {-41776.5769, 32.5845801}, ahigh = {-38344.5234, -956.326114, 8.200962779999999, -0.0002062130802, 9.87228899e-009, 8.15836676e-012, -7.52751966e-016}, bhigh = {-34235.6402, -22.24772278}, R = 194.2188938046896);
          constant IdealGases.Common.DataRecord BO2minus(name = "BO2minus", MM = 0.0428103486, Hf = -16689749.42661411, H0 = 224187.4059395069, Tlimit = 1000, alow = {53242.7969, -732.631303, 5.90247, 0.0024374652, -4.87880395e-007, -8.47572476e-010, 4.08078118e-013}, blow = {-83442.9368, -10.53931911}, ahigh = {119922.6372, -1715.766982, 8.701038219999999, -0.000460124212, 9.867011870000001e-008, -1.107288516e-011, 5.05018324e-016}, bhigh = {-78257.89689999999, -28.39356505}, R = 194.2164049558802);
          constant IdealGases.Common.DataRecord B_OH_2(name = "B_OH_2", MM = 0.04482568, Hf = -9486607.721288333, H0 = 267071.4197754501, Tlimit = 1000, alow = {1491.659222, 364.818484, -2.906006389, 0.0413683446, -5.78258422e-005, 4.069783560000001e-008, -1.126499791e-011}, blow = {-53754.8366, 38.8853518}, ahigh = {1557023.406, -5784.71076, 14.87391235, -0.0001821804765, -3.80354164e-008, 1.030332746e-011, -6.68588983e-016}, bhigh = {-18068.10301, -65.902689}, R = 185.4845704515805);
          constant IdealGases.Common.DataRecord BS(name = "BS", MM = 0.042876, Hf = 6379312.552476911, H0 = 203472.9219143577, Tlimit = 1000, alow = {-38394.901, 698.941749, -1.21398046, 0.01396391264, -1.689881284e-005, 1.038889985e-008, -2.582063989e-012}, blow = {28656.85472, 31.54819945}, ahigh = {1358760.165, -4364.03596, 9.034307589999999, -0.002114548699, 4.18927531e-007, -1.354787322e-011, -1.360684605e-015}, bhigh = {59130.6545, -33.36696868}, R = 193.9190222968561);
          constant IdealGases.Common.DataRecord BS2(name = "BS2", MM = 0.07494100000000001, Hf = 852236.559426749, H0 = 180936.4700230848, Tlimit = 1000, alow = {18202.64374, -515.067189, 8.547515000000001, -0.001626155268, 1.866387194e-006, -1.277165468e-009, 3.68198069e-013}, blow = {8186.839750000001, -17.78384854}, ahigh = {512040.251, -1938.581076, 9.814873459999999, -0.001339731606, 3.75430116e-007, -4.40038378e-011, 1.887662968e-015}, bhigh = {17365.35, -27.57546712}, R = 110.9469048985202);
          constant IdealGases.Common.DataRecord B2(name = "B2", MM = 0.021622, Hf = 39652691.98039035, H0 = 407229.0259920451, Tlimit = 1000, alow = {-151641.8096, 2630.168946, -13.81358321, 0.05216225190000001, -6.93049474e-005, 4.46941039e-008, -1.128496456e-011}, blow = {89952.5105, 98.13118490000001}, ahigh = {1094594.495, -2602.735739, 6.90945621, -0.0006405949889999999, 1.951732355e-007, -2.555902119e-011, 1.053557323e-015}, bhigh = {118780.7611, -19.49045025}, R = 384.5376005919897);
          constant IdealGases.Common.DataRecord B2C(name = "B2C", MM = 0.0336327, Hf = 23799237.46829722, H0 = 348562.9759133224, Tlimit = 1000, alow = {-41665.1987, 579.777952, 1.411017091, 0.01174705713, -1.066474425e-005, 4.71723262e-009, -7.97001326e-013}, blow = {91968.742, 17.69254387}, ahigh = {1159241.019, -4567.79881, 12.28672097, -0.002500830701, 6.40997985e-007, -6.932935770000001e-011, 2.672933741e-015}, bhigh = {122244.2137, -51.99066186}, R = 247.2139316795856);
          constant IdealGases.Common.DataRecord B2CL4(name = "B2CL4", MM = 0.163434, Hf = -2998152.159281422, H0 = 132090.8256543926, Tlimit = 1000, alow = {42855.5513, -728.074881, 9.315763670000001, 0.02331629623, -3.51124761e-005, 2.464057658e-008, -6.73547996e-012}, blow = {-58190.2324, -16.22083878}, ahigh = {-642798.703, 507.856299, 14.86725022, 2.026983515e-006, 6.27271881e-009, -1.218822573e-012, 7.241208409999999e-017}, bhigh = {-68200.66, -41.39110669}, R = 50.87357587772434);
          constant IdealGases.Common.DataRecord B2F4(name = "B2F4", MM = 0.09761561279999999, Hf = -14731250.0403624, H0 = 181126.6916515224, Tlimit = 1000, alow = {-59716.9564, 818.9425440000001, -0.643767129, 0.0385896607, -4.31305319e-005, 2.404463223e-008, -5.39355935e-012}, blow = {-179004.1225, 35.5217997}, ahigh = {38558.6589, -2928.842655, 17.92063736, -0.000952399827, 1.614708845e-007, -1.428613504e-011, 5.21873967e-016}, bhigh = {-161305.1924, -71.8480325}, R = 85.17563698580808);
          constant IdealGases.Common.DataRecord B2H(name = "B2H", MM = 0.02262994, Hf = 35186239.86630102, H0 = 447000.6106953885, Tlimit = 1000, alow = {87755.7974, -1343.931564, 9.83209915, -0.00560713775, 5.730651049999999e-006, -2.266436197e-009, 1.97235208e-013}, blow = {100990.8698, -32.8246148}, ahigh = {617554.5750000001, -2627.897466, 9.08490507, -0.000538941833, 1.0507147e-007, -1.092459556e-011, 4.68485466e-016}, bhigh = {109953.7555, -31.6402991}, R = 367.4102538495462);
          constant IdealGases.Common.DataRecord B2H2(name = "B2H2", MM = 0.02363788, Hf = 19235140.33407395, H0 = 447300.6462508482, Tlimit = 1000, alow = {142161.7322, -2138.011262, 13.17732399, -0.00894265807, 1.127804935e-005, -5.7878862e-009, 1.000813871e-012}, blow = {63723.19250000001, -53.4698274}, ahigh = {1240614.592, -5368.96502, 13.79292454, -0.001136749748, 2.245617156e-007, -2.361530467e-011, 1.022604572e-015}, bhigh = {85065.1149, -64.354613}, R = 351.7435573748576);
          constant IdealGases.Common.DataRecord B2H3(name = "B2H3", MM = 0.02464582, Hf = 14244716.58885767, H0 = 484407.5384791417, Tlimit = 1000, alow = {94251.4716, -1075.866284, 6.38399939, 0.01167858327, -1.429661252e-005, 1.128776882e-008, -3.68132506e-012}, blow = {46353.3508, -14.00002845}, ahigh = {1771506.902, -7872.26093, 17.89447015, -0.001709304785, 3.40957543e-007, -3.61454368e-011, 1.575671409e-015}, bhigh = {87377.1189, -90.5462946}, R = 337.3583025437985);
          constant IdealGases.Common.DataRecord B2H3_db(name = "B2H3_db", MM = 0.02464582, Hf = 14339479.35187387, H0 = 483296.0315380052, Tlimit = 1000, alow = {60640.1484, -752.446633, 5.58448384, 0.01080468494, -8.64903281e-006, 4.74833331e-009, -1.309096028e-012}, blow = {44917.923, -8.87368682}, ahigh = {1497731.017, -7219.31414, 17.50463766, -0.0015796988, 3.16410347e-007, -3.36722515e-011, 1.472919654e-015}, bhigh = {83243.4967, -87.79549259999999}, R = 337.3583025437985);
          constant IdealGases.Common.DataRecord B2H4(name = "B2H4", MM = 0.02565376, Hf = 8231232.341769785, H0 = 481015.6094077438, Tlimit = 1000, alow = {40001.323, -307.1548876, 1.763071673, 0.02646361882, -3.066168202e-005, 2.163781936e-008, -6.49183541e-012}, blow = {25810.33714, 10.1526713}, ahigh = {2292078.842, -10598.32517, 22.68612155, -0.002363586598, 4.76217307e-007, -5.0901004e-011, 2.23391159e-015}, bhigh = {86385.91979999999, -124.6794566}, R = 324.1034452649437);
          constant IdealGases.Common.DataRecord B2H4_db(name = "B2H4_db", MM = 0.02565376, Hf = 8183301.31723381, H0 = 449394.0849216645, Tlimit = 1000, alow = {-17814.96923, 571.6048040000001, -2.763085348, 0.03125348696, -2.709524346e-005, 1.341728634e-008, -3.004352798e-012}, blow = {21581.38307, 36.7587294}, ahigh = {1886854.256, -10100.45614, 22.47110812, -0.002319968667, 4.73187996e-007, -5.11067303e-011, 2.262693782e-015}, bhigh = {82334.87, -123.7455797}, R = 324.1034452649437);
          constant IdealGases.Common.DataRecord B2H5(name = "B2H5", MM = 0.0266617, Hf = 9556175.67521951, H0 = 458364.9579734226, Tlimit = 1000, alow = {38309.6545, -40.6614805, -1.422523787, 0.0367459527, -3.85198971e-005, 2.44449444e-008, -6.82172294e-012}, blow = {30089.56452, 28.02425895}, ahigh = {2696691.012, -13200.19835, 27.35260413, -0.002961879012, 5.98499943e-007, -6.41374855e-011, 2.821187823e-015}, bhigh = {106452.4901, -155.9654379}, R = 311.8507822081863);
          constant IdealGases.Common.DataRecord B2H5_db(name = "B2H5_db", MM = 0.0266617, Hf = 10320070.21307719, H0 = 437613.3554874596, Tlimit = 1000, alow = {14244.46027, 420.45582, -3.70045125, 0.036720531, -3.19350373e-005, 1.695013434e-008, -4.25703814e-012}, blow = {30466.84683, 42.45495820000001}, ahigh = {2571729.448, -13568.29858, 27.81376038, -0.00319236503, 6.56116526e-007, -7.12733394e-011, 3.16949265e-015}, bhigh = {110637.0527, -159.7214457}, R = 311.8507822081863);
          constant IdealGases.Common.DataRecord B2H6(name = "B2H6", MM = 0.02766964, Hf = 1322749.410545276, H0 = 431242.0761527797, Tlimit = 1000, alow = {-10528.44558, 1041.795556, -8.960518860000001, 0.0549248088, -5.30519705e-005, 3.011904756e-008, -7.688768300000001e-012}, blow = {-925.9374349999999, 68.12592429999999}, ahigh = {2835765.414, -15676.00163, 32.2612233, -0.00373860973, 7.71880646e-007, -8.41445454e-011, 3.75222338e-015}, bhigh = {93583.7855, -192.0223395}, R = 300.4907906282843);
          constant IdealGases.Common.DataRecord B2O(name = "B2O", MM = 0.0376214, Hf = 5124690.149755193, H0 = 313199.8809188387, Tlimit = 1000, alow = {-56995.4676, 1018.000465, -2.636481947, 0.02746009279, -3.63243272e-005, 2.406890217e-008, -6.38412972e-012}, blow = {17038.74394, 38.5534611}, ahigh = {-162872.322, -387.628981, 7.78772352, -0.0001146822108, 2.528097612e-008, -2.893783941e-012, 1.339210475e-016}, bhigh = {22630.35354, -19.08513011}, R = 221.0037903958917);
          constant IdealGases.Common.DataRecord B2O2(name = "B2O2", MM = 0.0536208, Hf = -8536080.886521647, H0 = 249839.9501685913, Tlimit = 1000, alow = {81743.9169, -1732.702797, 16.05560926, -0.02160057288, 3.56685457e-005, -2.660198794e-008, 7.531833240000001e-012}, blow = {-48996.32900000001, -61.72702390000001}, ahigh = {460578.966, -2990.079203, 12.56764079, -0.000785097495, 1.672537624e-007, -1.86769478e-011, 8.486190669999999e-016}, bhigh = {-40157.4815, -48.7441371}, R = 155.0605735087876);
          constant IdealGases.Common.DataRecord B2O3(name = "B2O3", MM = 0.06962019999999999, Hf = -11999136.32824956, H0 = 207105.5383351384, Tlimit = 1000, alow = {73796.11910000001, -1263.620592, 10.72681512, 0.000384138372, 5.97605838e-006, -6.55289135e-009, 2.123951064e-012}, blow = {-96281.83140000001, -30.88078011}, ahigh = {390503.53, -3691.34821, 15.55502598, -0.0009707645510000001, 2.068887872e-007, -2.310858356e-011, 1.050136734e-015}, bhigh = {-82630.5441, -63.9086344}, R = 119.4261435617824);
          constant IdealGases.Common.DataRecord B2_OH_4(name = "B2_OH_4", MM = 0.08965136, Hf = -13998533.10646933, H0 = 216874.2225438633, Tlimit = 1000, alow = {-17287.17249, 849.591348, -8.715959570000001, 0.0936236261, -0.0001278784165, 8.81367116e-008, -2.400605571e-011}, blow = {-156433.7133, 71.2016931}, ahigh = {2983925.656, -12074.69577, 31.6261057, -0.000516368244, -4.23209397e-008, 1.67197728e-011, -1.156366726e-015}, bhigh = {-82809.76969999999, -165.3010498}, R = 92.74228522579023);
          constant IdealGases.Common.DataRecord B2S(name = "B2S", MM = 0.053687, Hf = 11590528.52645892, H0 = 245547.4323393, Tlimit = 1000, alow = {66386.09299999999, -920.1185690000001, 8.699831339999999, 0.000764558901, -3.70902603e-006, 3.64237398e-009, -1.197766071e-012}, blow = {77703.758, -22.51712444}, ahigh = {-121150.1483, -64.73398450000001, 7.54682561, -1.82054145e-005, 3.92395117e-009, -4.40386238e-013, 2.003651718e-017}, bhigh = {72585.3187, -13.94971229}, R = 154.8693724737832);
          constant IdealGases.Common.DataRecord B2S2(name = "B2S2", MM = 0.085752, Hf = 1612989.924433249, H0 = 169898.2181173617, Tlimit = 1000, alow = {-70615.4997, 718.294478, 1.919797788, 0.02036407461, -2.309813846e-005, 1.311616725e-008, -2.996394781e-012}, blow = {11008.29019, 18.55402586}, ahigh = {-164153.5014, -774.286564, 11.07731737, -0.0002313278554, 5.12561429e-008, -5.89408123e-012, 2.738673834e-016}, bhigh = {17330.03515, -33.59766998}, R = 96.95951114842804);
          constant IdealGases.Common.DataRecord B2S3(name = "B2S3", MM = 0.117817, Hf = 150691.8101801947, H0 = 169229.0331615981, Tlimit = 1000, alow = {-34312.685, 558.761537, 3.41407741, 0.02398333891, -2.841617637e-005, 1.690005199e-008, -4.060070709999999e-012}, blow = {-3027.690575, 18.46767864}, ahigh = {-190585.5086, -934.8342479999999, 13.69333613, -0.000276664725, 6.1101558e-008, -7.00804857e-012, 3.24952462e-016}, bhigh = {2906.43376, -39.46870479}, R = 70.57107208637125);
          constant IdealGases.Common.DataRecord B3H7_C2v(name = "B3H7_C2v", MM = 0.03948858, Hf = 4457469.450661431, H0 = 375258.8469881672, Tlimit = 1000, alow = {108600.9469, -792.15368, -0.9577368039999999, 0.0565611929, -6.4050345e-005, 4.28132296e-008, -1.227580455e-011}, blow = {24306.38819, 21.02582666}, ahigh = {3598592.14, -18637.01968, 39.9399883, -0.00427541016, 8.705358820000001e-007, -9.38510502e-011, 4.14786531e-015}, bhigh = {127215.4092, -237.9769856}, R = 210.5538360710869);
          constant IdealGases.Common.DataRecord B3H7_Cs(name = "B3H7_Cs", MM = 0.03948858, Hf = 4034541.682683955, H0 = 356642.9078989419, Tlimit = 1000, alow = {52981.69330000001, 407.174537, -9.63573053, 0.07944312000000001, -9.306456819999999e-005, 6.105147350000001e-008, -1.685927691e-011}, blow = {17070.69083, 68.73966560000001}, ahigh = {3281576.45, -18110.96595, 39.6383246, -0.00417842411, 8.52682906e-007, -9.20960024e-011, 4.076496770000001e-015}, bhigh = {121362.2564, -235.7090217}, R = 210.5538360710869);
          constant IdealGases.Common.DataRecord B3H9(name = "B3H9", MM = 0.04150446, Hf = 3346840.315474529, H0 = 417543.9217857551, Tlimit = 1000, alow = {84832.14900000001, -520.616304, -0.466570121, 0.0534698386, -4.15579682e-005, 1.944135439e-008, -4.54122735e-012}, blow = {18051.011, 21.10804198}, ahigh = {3863999.5, -22294.33407, 48.7504615, -0.00541844926, 1.125767651e-006, -1.233226406e-010, 5.52034818e-015}, bhigh = {142218.7123, -295.8240102}, R = 200.3271937521895);
          constant IdealGases.Common.DataRecord B3N3H6(name = "B3N3H6", MM = 0.08050073999999999, Hf = -6360189.980862289, H0 = 201505.5762220323, Tlimit = 1000, alow = {-155262.1082, 4010.21127, -33.824913, 0.1624575582, -0.0002065528647, 1.368862991e-007, -3.6734709e-011}, blow = {-80512.534, 199.5000751}, ahigh = {3739245.17, -19412.38951, 44.4507003, -0.00321030805, 5.71746542e-007, -5.48669214e-011, 2.191943197e-015}, bhigh = {47584.2616, -264.1156625}, R = 103.2844170128126);
          constant IdealGases.Common.DataRecord B3O3CL3(name = "B3O3CL3", MM = 0.1867902, Hf = -8758393.325774049, H0 = 130905.0528346776, Tlimit = 1000, alow = {-42844.52800000001, 1005.623966, -4.55651963, 0.08117967139999999, -0.0001038385803, 6.62142233e-008, -1.695748504e-011}, blow = {-204090.7277, 54.66113803}, ahigh = {-741405.981, -2527.447455, 26.87263667, -0.000746338936, 1.64633411e-007, -1.886231553e-011, 8.737926310000001e-016}, bhigh = {-192350.9858, -118.5636255}, R = 44.51235664397812);
          constant IdealGases.Common.DataRecord B3O3FCL2(name = "B3O3FCL2", MM = 0.1703356032, Hf = -11059388.90995162, H0 = 137658.4728001245, Tlimit = 1000, alow = {-37191.2007, 867.3745889999999, -4.09577846, 0.0781556397, -9.767035169999999e-005, 6.09761009e-008, -1.532880319e-011}, blow = {-233138.3552, 51.98088158}, ahigh = {-716236.083, -2851.210308, 27.10953133, -0.000840091964, 1.852318684e-007, -2.121694694e-011, 9.827291560000002e-016}, bhigh = {-220272.9252, -121.0679365}, R = 48.81229668842362);
          constant IdealGases.Common.DataRecord B3O3F2CL(name = "B3O3F2CL", MM = 0.1538810064, Hf = -13860169.47053188, H0 = 144663.6691609264, Tlimit = 1000, alow = {-52770.1554, 1104.904696, -5.85418105, 0.07953943099999999, -9.59266077e-005, 5.79985631e-008, -1.416698348e-011}, blow = {-264040.0115, 60.71247222}, ahigh = {-681078.749, -3370.00902, 27.48953579, -0.000990544256, 2.18290844e-007, -2.49957348e-011, 1.157532106e-015}, bhigh = {-247211.4785, -126.1501548}, R = 54.03182754333741);
          constant IdealGases.Common.DataRecord B3O3F3(name = "B3O3F3", MM = 0.1374264096, Hf = -17337999.9152652, H0 = 154269.7510741051, Tlimit = 1000, alow = {-72515.4978, 1400.996228, -7.9067162, 0.0821716185, -9.63315266e-005, 5.66819426e-008, -1.349024305e-011}, blow = {-295346.1728, 70.01536830000001}, ahigh = {-662982.291, -3784.94687, 27.79526578, -0.001112096232, 2.450806704e-007, -2.80648498e-011, 1.299750466e-015}, bhigh = {-274886.5475, -131.5605015}, R = 60.50126772721858);
          constant IdealGases.Common.DataRecord B4H4(name = "B4H4", MM = 0.04727576, Hf = 6899738.893674052, H0 = 315946.3115981636, Tlimit = 1000, alow = {217179.1579, -2598.282557, 9.845440610000001, 0.0302630914, -4.128334159999999e-005, 3.089368344e-008, -9.35338426e-012}, blow = {50791.4348, -37.7075721}, ahigh = {1937200.886, -10781.85932, 28.79625241, -0.002401547707, 4.83775678e-007, -5.17049023e-011, 2.269138182e-015}, bhigh = {98365.95389999999, -156.1814117}, R = 175.8717786874288);
          constant IdealGases.Common.DataRecord B4H10(name = "B4H10", MM = 0.0533234, Hf = 1239605.876594516, H0 = 290408.6011019553, Tlimit = 1000, alow = {-32966.04820000001, 2440.532922, -26.85332563, 0.1406499347, -0.0001586710574, 9.87459413e-008, -2.601426018e-011}, blow = {-3091.924018, 158.9905532}, ahigh = {4030474.13, -25606.29762, 56.8723366, -0.00617856786, 1.280621032e-006, -1.400254585e-010, 6.258866560000001e-015}, bhigh = {150600.9087, -352.07142}, R = 155.9253911040931);
          constant IdealGases.Common.DataRecord B4H12(name = "B4H12", MM = 0.05533928000000001, Hf = 3401491.924000457, H0 = 393006.8117980573, Tlimit = 1000, alow = {71521.70299999999, -159.0351575, -4.71977166, 0.088112289, -8.60494688e-005, 5.12353551e-008, -1.37986525e-011}, blow = {21941.86547, 41.878974}, ahigh = {6158360.8, -32020.8673, 66.60443599999999, -0.00740636928, 1.512993908e-006, -1.635619071e-010, 7.24538514e-015}, bhigh = {205458.6275, -414.210686}, R = 150.2453953141421);
          constant IdealGases.Common.DataRecord B5H9(name = "B5H9", MM = 0.06312646, Hf = 1159893.965224725, H0 = 254293.9521715617, Tlimit = 1000, alow = {89984.7372, 104.2533357, -11.95549834, 0.1007310686, -9.866680210000001e-005, 5.29553518e-008, -1.232119583e-011}, blow = {8374.341780000001, 76.6309048}, ahigh = {3169017.62, -22825.03203, 55.1200898, -0.005563411420000001, 1.157837073e-006, -1.270332301e-010, 5.69426203e-015}, bhigh = {133445.2571, -337.106046}, R = 131.7113616065276);
          constant IdealGases.Common.DataRecord Ba(name = "Ba", MM = 0.137327, Hf = 1347149.504467439, H0 = 45128.98410363585, Tlimit = 1000, alow = {2222.563526, -34.0797785, 2.706751118, -0.0006382894490000001, 1.063003846e-006, -9.10262427e-010, 3.148062219e-013}, blow = {21665.49702, 5.10254588}, ahigh = {-19265792.28, 60065.0104, -66.3396413, 0.0350756593, -7.80760183e-006, 8.0851268e-010, -3.199486918e-014}, bhigh = {-358966.372, 500.75834}, R = 60.54506397139674);
          constant IdealGases.Common.DataRecord Baplus(name = "Baplus", MM = 0.1373264514, Hf = 5054011.961456684, H0 = 45129.16438762649, Tlimit = 1000, alow = {-54231.6755, 674.468078, -0.8940272500000001, 0.008796642950000001, -1.222812901e-005, 8.387953870000001e-009, -2.037964358e-012}, blow = {79417.7464, 26.07063698}, ahigh = {8794971.85, -19518.17883, 14.85542861, -0.00094042335, -7.03125779e-007, 1.667412753e-010, -1.07011731e-014}, bhigh = {214680.0732, -92.28264190000002}, R = 60.54530584047409);
          constant IdealGases.Common.DataRecord BaBr(name = "BaBr", MM = 0.217231, Hf = -346749.2070652899, H0 = 46955.19055751711, Tlimit = 1000, alow = {-691.2557439999999, -62.4550016, 4.79667765, -0.000683800494, 1.04394768e-006, -7.4412243e-010, 2.172187003e-013}, blow = {-10113.49165, 5.19645487}, ahigh = {-1056014.042, 1034.740207, 6.73572938, -0.0037840263, 1.952196896e-006, -3.34783188e-010, 1.893189803e-014}, bhigh = {-19502.33645, -5.06927964}, R = 38.27479503385797);
          constant IdealGases.Common.DataRecord BaBr2(name = "BaBr2", MM = 0.297135, Hf = -1388307.473034143, H0 = 52116.91318760833, Tlimit = 1000, alow = {-5143.37406, -38.7051846, 7.16244697, -0.000369671531, 4.68971527e-007, -3.107902475e-010, 8.36358205e-014}, blow = {-51533.1826, 0.9741407000000001}, ahigh = {-8928.25121, -0.753655889, 7.00065821, -2.94982171e-007, 7.092392109999999e-011, -8.670038099999999e-015, 4.22174231e-019}, bhigh = {-51726.6949, 1.910185706}, R = 27.98213606609791);
          constant IdealGases.Common.DataRecord BaCL(name = "BaCL", MM = 0.17278, Hf = -788811.6738048386, H0 = 57165.86989234866, Tlimit = 1000, alow = {-2661.946867, -77.2499634, 4.74610605, -0.000352354888, 3.99919835e-007, -1.936413749e-010, 4.124048529999999e-014}, blow = {-17363.28141, 3.904455084}, ahigh = {-1075410.089, 1209.303069, 6.2725586, -0.00328811031, 1.717751087e-006, -2.910607065e-010, 1.632658438e-014}, bhigh = {-27791.73737, -3.574185216}, R = 48.12172705174211);
          constant IdealGases.Common.DataRecord BaCLplus(name = "BaCLplus", MM = 0.1727794514, Hf = 2018166.090785492, H0 = 55957.05925479053, Tlimit = 1000, alow = {-895.155669, -133.4692684, 4.93541612, -0.000757405451, 8.355724e-007, -4.68525851e-010, 1.095343103e-013}, blow = {41251.5599, 1.896278378}, ahigh = {-18291.98157, -2.931616688, 4.50228071, 3.67178849e-005, 2.396147871e-009, 5.4794586e-014, 1.174342316e-018}, bhigh = {40551.00150000001, 4.503765564}, R = 48.12187984525572);
          constant IdealGases.Common.DataRecord BaCL2(name = "BaCL2", MM = 0.208233, Hf = -2397801.438772913, H0 = 70109.50713863796, Tlimit = 1000, alow = {-1157.501999, -205.4518828, 7.83751405, -0.001867700704, 2.335378734e-006, -1.531373341e-009, 4.08852509e-013}, blow = {-61156.6528, -5.978149218}, ahigh = {-22165.98768, -4.00248381, 7.00342674, -1.515820808e-006, 3.61184921e-010, -4.38675906e-014, 2.125803293e-018}, bhigh = {-62190.9527, -1.132439694}, R = 39.92869525963705);
          constant IdealGases.Common.DataRecord BaF(name = "BaF", MM = 0.1563254032, Hf = -2040575.501295109, H0 = 59760.67746358449, Tlimit = 1000, alow = {25303.17292, -471.69951, 6.18503622, -0.0033666788, 4.011834559999999e-006, -2.506578601e-009, 6.47725816e-013}, blow = {-37318.7214, -6.22071538}, ahigh = {1868380.434, -7071.099600000001, 14.66959284, -0.0069959502, 2.366313845e-006, -3.2877343e-010, 1.647896238e-014}, bhigh = {3571.78068, -66.6239092}, R = 53.18695381429856);
          constant IdealGases.Common.DataRecord BaFplus(name = "BaFplus", MM = 0.1563248546, Hf = 857593.7226555399, H0 = 58862.77024562158, Tlimit = 1000, alow = {28407.19922, -486.357182, 5.94605839, -0.002429844077, 2.445346729e-006, -1.316757471e-009, 2.97488632e-013}, blow = {17306.39076, -5.95120693}, ahigh = {-40544.2367, -17.01997704, 4.51305534, 2.632753508e-005, 2.612088236e-009, -9.502197929999999e-014, 6.515176160000001e-018}, bhigh = {14744.8657, 2.783167711}, R = 53.18714046640156);
          constant IdealGases.Common.DataRecord BaF2(name = "BaF2", MM = 0.1753238064, Hf = -4631450.227286418, H0 = 76641.14346994921, Tlimit = 1000, alow = {34883.9552, -764.845956, 9.79775038, -0.00575645751, 6.7756038e-006, -4.24361475e-009, 1.093687692e-012}, blow = {-95903.56449999999, -20.82035292}, ahigh = {-55086.3366, -16.91114228, 7.01356488, -5.73190977e-006, 1.321203959e-009, -1.565528167e-013, 7.444983019999999e-018}, bhigh = {-99835.29700000001, -4.38958149}, R = 47.42351977592018);
          constant IdealGases.Common.DataRecord BaH(name = "BaH", MM = 0.13833494, Hf = 1514694.010059932, H0 = 63115.68863224288, Tlimit = 1000, alow = {-37652.6888, 698.668012, -1.313524351, 0.01460648882, -1.802974829e-005, 1.133031115e-008, -2.865589059e-012}, blow = {20974.78889, 32.308026}, ahigh = {-6755466.57, 17307.00212, -11.36812458, 0.00561774474, -3.18078885e-007, -1.052342502e-010, 1.111101879e-014}, bhigh = {-89540.70379999999, 118.2377538}, R = 60.10391879303957);
          constant IdealGases.Common.DataRecord BaI(name = "BaI", MM = 0.26423147, Hf = -38745.10102827645, H0 = 39348.54920952451, Tlimit = 1000, alow = {-4186.28554, 10.33094613, 4.42890023, 0.0003026019976, -2.591429408e-007, 1.7416054e-010, -3.370062760000001e-014}, blow = {-2636.169733, 8.201924679999999}, ahigh = {-3533859.66, 7394.258, 1.285151488, -0.002303429302, 2.214839289e-006, -4.68109681e-010, 3.011371472e-014}, bhigh = {-53402.2959, 37.1044387}, R = 31.46662280613282);
          constant IdealGases.Common.DataRecord BaI2(name = "BaI2", MM = 0.39113594, Hf = -737441.7318950542, H0 = 40370.70845496836, Tlimit = 1000, alow = {-4241.592729999999, -18.62214382, 7.07855675, -0.0001793922821, 2.281437677e-007, -1.514647881e-010, 4.08149836e-014}, blow = {-36703.5149, 3.16313897}, ahigh = {-6048.2248, -0.362272945, 7.00031745, -1.425793502e-007, 3.43334942e-011, -4.201754979999999e-015, 2.047703474e-019}, bhigh = {-36796.5075, 3.61547532}, R = 21.25724370918203);
          constant IdealGases.Common.DataRecord BaO(name = "BaO", MM = 0.1533264, Hf = -769263.4275636812, H0 = 58790.31921443405, Tlimit = 1000, alow = {37643.985, -507.15553, 5.39284728, -0.000411932469, -6.52789662e-007, 9.430588419999999e-010, -3.43660303e-013}, blow = {-12755.52614, -3.75221376}, ahigh = {13184816.98, -38542.5325, 46.8674161, -0.02188646633, 5.335677450000001e-006, -5.1523043e-010, 1.4330834e-014}, bhigh = {230690.2396, -302.8332772}, R = 54.22726940696449);
          constant IdealGases.Common.DataRecord BaOplus(name = "BaOplus", MM = 0.1533258514, Hf = 3337367.321477009, H0 = 61762.48762705387, Tlimit = 1000, alow = {356641.097, -3779.24671, 16.73544137, -0.0130661161, 2.686157445e-006, 4.044077950000001e-009, -2.016508813e-012}, blow = {79832.68490000001, -72.8511855}, ahigh = {-331767.923, 1594.253495, 3.22724781, 0.00057226415, -1.147415658e-007, 1.382873231e-011, -6.38443703e-016}, bhigh = {50252.4761, 14.09092365}, R = 54.22746343217109);
          constant IdealGases.Common.DataRecord BaOH(name = "BaOH", MM = 0.15433434, Hf = -1453056.481143471, H0 = 72728.99213486773, Tlimit = 1000, alow = {37762.3151, -889.139022, 9.531718, -0.0054588259, 4.94178301e-006, -1.809327537e-009, 2.050048305e-013}, blow = {-24418.52965, -24.89410158}, ahigh = {2637336.694, -8365.410400000001, 15.99803975, -0.005098446049999999, 1.594657431e-006, -2.180478542e-010, 1.084610688e-014}, bhigh = {23935.50647, -74.9070579}, R = 53.87311728549849);
          constant IdealGases.Common.DataRecord BaOHplus(name = "BaOHplus", MM = 0.1543337914, Hf = 1308976.544717996, H0 = 73237.2340332462, Tlimit = 1000, alow = {27976.32165, -768.07694, 9.05297844, -0.0043685779, 3.52349726e-006, -8.38636087e-010, -6.47545747e-014}, blow = {26232.80802, -22.64561641}, ahigh = {876673.843, -2335.8196, 7.97316606, 0.0001038679898, -6.31948578e-008, 1.028729445e-011, -5.74180857e-016}, bhigh = {37725.1747, -19.30805001}, R = 53.87330878466322);
          constant IdealGases.Common.DataRecord Ba_OH_2(name = "Ba_OH_2", MM = 0.17134168, Hf = -3540682.407222807, H0 = 101485.6688693609, Tlimit = 1000, alow = {57028.149, -1577.384797, 16.59935325, -0.01039598275, 9.664889600000001e-006, -3.67362144e-009, 4.62578499e-013}, blow = {-68351.58899999999, -58.51911699999999}, ahigh = {1762908.112, -4676.205419999999, 13.95135494, 0.0002051842928, -1.25726553e-007, 2.048917625e-011, -1.144045777e-015}, bhigh = {-45459.6441, -49.2945719}, R = 48.52568271771352);
          constant IdealGases.Common.DataRecord BaS(name = "BaS", MM = 0.169392, Hf = 229475.5065174271, H0 = 56414.20492112969, Tlimit = 1000, alow = {13770.75147, -329.350056, 5.81154584, -0.00284897376, 3.57633602e-006, -2.325649347e-009, 6.17280531e-013}, blow = {4964.43245, -3.497962615}, ahigh = {6104262.07, -22276.17479, 35.9751351, -0.0215779374, 7.31987235e-006, -1.089038636e-009, 5.88410195e-014}, bhigh = {140413.8006, -214.1776331}, R = 49.08420704637764);
          constant IdealGases.Common.DataRecord Ba2(name = "Ba2", MM = 0.274654, Hf = 1296046.432966569, H0 = 41357.9885965615, Tlimit = 1000, alow = {-12941.77083, -727.633008, 14.64866569, -0.0396633165, 5.89871106e-005, -4.23530056e-008, 1.189594362e-011}, blow = {43867.1223, -41.4567367}, ahigh = {216011.4547, 195.0676877, 2.253699771, 0.0001507750613, -4.742515e-008, 7.04895196e-012, -3.54744017e-016}, bhigh = {41677.6297, 23.75906459}, R = 30.27253198569837);
          constant IdealGases.Common.DataRecord Be(name = "Be", MM = 0.009012181999999999, Hf = 35951337.86690061, H0 = 687672.3084376237, Tlimit = 1000, alow = {-0.000411290152, 5.36496736e-006, 2.499999972, 7.56920369e-011, -1.097852652e-013, 8.00211024e-017, -2.303022777e-020}, blow = {38222.6459, 2.146172983}, ahigh = {-692628.584, 2466.773005, -0.9776613340000001, 0.002458939515, -9.047950419999999e-007, 1.587880407e-010, -9.415600603e-015}, bhigh = {23002.12917, 26.23234754}, R = 922.581456965694);
          constant IdealGases.Common.DataRecord Beplus(name = "Beplus", MM = 0.0090116334, Hf = 136457096.4460228, H0 = 687714.1717726777, Tlimit = 1000, alow = {0, 0, 2.5, 0, 0, 0, 0}, blow = {147152.8569, 2.839228698}, ahigh = {-94781.35979999999, 276.8325398, 2.191388413, 0.0001648824289, -4.28016682e-008, 4.54235047e-012, -6.270417825e-017}, bhigh = {145385.0986, 5.05550384}, R = 922.6376208335329);
          constant IdealGases.Common.DataRecord Beplusplus(name = "Beplusplus", MM = 0.009011084800000001, Hf = 332146638.7709501, H0 = 687756.0402050593, Tlimit = 1000, alow = {0, 0, 2.5, 0, 0, 0, 0}, blow = {359227.916, 2.145990203}, ahigh = {0, 0, 2.5, 0, 0, 0, 0}, bhigh = {359227.916, 2.145990203}, R = 922.6937915399486);
          constant IdealGases.Common.DataRecord BeBr(name = "BeBr", MM = 0.08891618200000001, Hf = 1489563.598221075, H0 = 100893.9857539092, Tlimit = 1000, alow = {37462.8806, -472.233907, 5.03940904, 0.0005798853219999999, -1.937634343e-006, 1.793372625e-009, -5.67229558e-013}, blow = {17231.35679, -2.573912886}, ahigh = {821723.8139999999, -2850.509348, 8.017266019999999, -0.002105663212, 6.84913634e-007, -9.98912932e-011, 5.192456840000001e-015}, bhigh = {32293.7287, -23.26810811}, R = 93.50909826515043);
          constant IdealGases.Common.DataRecord BeBr2(name = "BeBr2", MM = 0.168820182, Hf = -1386460.411469051, H0 = 76042.45445014389, Tlimit = 1000, alow = {-21186.98678, 209.0382611, 3.73117131, 0.01115501549, -1.542821761e-005, 1.052354444e-008, -2.850006057e-012}, blow = {-30904.56277, 9.458917720000001}, ahigh = {-104133.1811, -154.3443092, 7.61446541, -4.55670063e-005, 1.003180548e-008, -1.146879501e-012, 5.30179538e-017}, bhigh = {-29845.27608, -11.4936098}, R = 49.25046224627338);
          constant IdealGases.Common.DataRecord BeCL(name = "BeCL", MM = 0.044465182, Hf = 1274999.998875525, H0 = 199281.9235508808, Tlimit = 1000, alow = {20161.38947, -141.3988978, 2.898223773, 0.00606992551, -9.162038470000001e-006, 6.61262066e-009, -1.859126853e-012}, blow = {6626.691440000001, 7.90018105}, ahigh = {280884.5146, -1103.100122, 5.67118983, -0.0005443000450000001, 1.459036606e-007, -1.230971735e-011, 9.90080476e-017}, bhigh = {12246.49804, -8.38237137}, R = 186.9883721604918);
          constant IdealGases.Common.DataRecord BeCL2(name = "BeCL2", MM = 0.079918182, Hf = -4523865.920773824, H0 = 151165.3505831752, Tlimit = 1000, alow = {-22299.57657, 152.8781667, 3.71499471, 0.0106777655, -1.397766942e-005, 9.066693559999998e-009, -2.350292311e-012}, blow = {-45904.2823, 6.682555572}, ahigh = {-115408.746, -248.4300053, 7.68567214, -7.44496037e-005, 1.649479787e-008, -1.895997075e-012, 8.804973950000001e-017}, bhigh = {-44687.4728, -15.00337407}, R = 104.0373015492269);
          constant IdealGases.Common.DataRecord BeF(name = "BeF", MM = 0.0280105852, Hf = -6091439.853245195, H0 = 310993.3240523657, Tlimit = 1000, alow = {-46644.9723, 788.717885, -1.463363201, 0.01381790494, -1.591299964e-005, 9.35293152e-009, -2.229211181e-012}, blow = {-25226.27476, 31.976343}, ahigh = {-185794.1543, 47.76082770000001, 4.26998602, 0.0002310668473, -6.657214920000001e-008, 1.152149888e-011, -6.33558644e-016}, bhigh = {-22579.5096, -0.218478727}, R = 296.8332128955307);
          constant IdealGases.Common.DataRecord BeF2(name = "BeF2", MM = 0.0470089884, Hf = -16945445.18213883, H0 = 231440.0792338684, Tlimit = 1000, alow = {7490.27024, -287.8053102, 5.43471282, 0.00390323346, -2.214865601e-006, -1.965806119e-010, 4.04236227e-013}, blow = {-95916.15640000001, -5.61661692}, ahigh = {-64820.1807, -768.455606, 8.062860840000001, -0.0002227502549, 4.89159866e-008, -5.58788672e-012, 2.583409761e-016}, bhigh = {-93954.9549, -21.25660266}, R = 176.8698345357289);
          constant IdealGases.Common.DataRecord BeH(name = "BeH", MM = 0.010020122, Hf = 34156480.72947615, H0 = 863073.7230544697, Tlimit = 1000, alow = {-1615.149125, -59.3528608, 4.5072691, -0.00526418961, 1.145319967e-005, -9.247883590000001e-009, 2.700364753e-012}, blow = {40301.93040000001, -3.48511705}, ahigh = {-2424081.636, 6597.39846, -3.62631648, 0.00457963435, -1.163329056e-006, 1.297785691e-010, -5.29188581e-015}, bhigh = {-2489.846221, 52.1601869}, R = 829.7775216708938);
          constant IdealGases.Common.DataRecord BeHplus(name = "BeHplus", MM = 0.0100195734, Hf = 117591728.1069072, H0 = 862706.4900787092, Tlimit = 1000, alow = {-32206.1102, 252.0229596, 3.20771957, -0.00219128729, 6.90253449e-006, -5.76952263e-009, 1.656414969e-012}, blow = {139253.066, 3.30704477}, ahigh = {462234.1589999999, -1760.169629, 5.37990555, 2.742462444e-005, -1.16720817e-007, 3.55717791e-011, -2.569695765e-015}, bhigh = {151585.9734, -13.69222641}, R = 829.822954338555);
          constant IdealGases.Common.DataRecord BeH2(name = "BeH2", MM = 0.011028062, Hf = 14608088.25703011, H0 = 835967.6432722267, Tlimit = 1000, alow = {90653.14900000001, -1320.65722, 9.90130806, -0.01255772231, 2.205704068e-005, -1.633609388e-008, 4.50510324e-012}, blow = {24645.61869, -36.3582675}, ahigh = {607426.201, -3407.53425, 9.79237036, -0.000825461482, 1.915656246e-007, -1.962774813e-011, 8.929908660000001e-016}, bhigh = {37931.052, -42.5465358}, R = 753.9377272271411);
          constant IdealGases.Common.DataRecord BeI(name = "BeI", MM = 0.135916652, Hf = 1526334.749622879, H0 = 66872.66693414432, Tlimit = 1000, alow = {35300.5481, -519.095026, 5.71450524, -0.00140907364, 7.66477212e-007, -2.601945105e-011, -8.230960639999999e-014}, blow = {26379.03657, -5.14099784}, ahigh = {-532820.662, 1139.497861, 3.62917166, 0.0001704033769, 1.067048688e-007, -3.58945788e-011, 2.72276956e-015}, bhigh = {15854.74307, 9.370951270000001}, R = 61.17331377468009);
          constant IdealGases.Common.DataRecord BeI2(name = "BeI2", MM = 0.262821122, Hf = -246401.2424389543, H0 = 51139.53131970877, Tlimit = 1000, alow = {359.272618, -48.6605901, 5.24157118, 0.007830305059999999, -1.169525374e-005, 8.389583910000001e-009, -2.356636003e-012}, blow = {-9333.22092, 2.896318281}, ahigh = {-93923.72840000001, -89.5921438, 7.56555479, -2.577727326e-005, 5.61426533e-009, -6.35966385e-013, 2.917080336e-017}, bhigh = {-9814.768620000001, -9.03204281}, R = 31.63547867359002);
          constant IdealGases.Common.DataRecord BeN(name = "BeN", MM = 0.023018882, Hf = 18549988.65713808, H0 = 379046.2542881101, Tlimit = 1000, alow = {-42477.8363, 759.0188900000001, -1.561469235, 0.01496945721, -1.83560888e-005, 1.146861461e-008, -2.897470581e-012}, blow = {46830.0332, 32.56973759}, ahigh = {-59096.9616, -307.8649431, 4.72795915, -3.92815883e-005, 2.006201069e-008, -2.300018068e-012, 1.066101831e-016}, bhigh = {51560.9571, -3.031380625}, R = 361.2022512648529);
          constant IdealGases.Common.DataRecord BeO(name = "BeO", MM = 0.025011582, Hf = 5155223.847895747, H0 = 347363.2335611557, Tlimit = 1000, alow = {-48694.58319999999, 721.347362, -0.378415732, 0.008792203849999999, -7.07015126e-006, 2.25017562e-009, -2.799117872e-014}, blow = {11014.66639, 25.74081184}, ahigh = {-35034724.5, 105585.9473, -116.5011722, 0.0647736948, -1.649656058e-005, 2.0055584e-009, -9.426891790000001e-014}, bhigh = {-657001.8000000001, 864.0898030000001}, R = 332.4248742042786);
          constant IdealGases.Common.DataRecord BeOH(name = "BeOH", MM = 0.026019522, Hf = -3832419.404168916, H0 = 415013.38879323, Tlimit = 1000, alow = {-38530.2452, 449.8059289999999, 1.532096459, 0.01299912603, -1.708979996e-005, 1.169968182e-008, -3.16318371e-012}, blow = {-15590.46567, 15.4554293}, ahigh = {855033.031, -2646.537838, 8.204784630000001, 1.110814513e-005, -4.265364e-008, 7.926511920000001e-012, -4.64541952e-016}, bhigh = {3121.216271, -25.71641308}, R = 319.5474536388486);
          constant IdealGases.Common.DataRecord BeOHplus(name = "BeOHplus", MM = 0.0260189734, Hf = 29208835.73369579, H0 = 367074.5902680389, Tlimit = 1000, alow = {71061.19160000001, -806.8981600000001, 5.60310781, 0.004932070859999999, -7.722847159999999e-006, 5.910889809999999e-009, -1.695201901e-012}, blow = {94407.8242, -10.88099852}, ahigh = {813487.439, -2814.653683, 8.381145009999999, -6.94207328e-005, -2.369535806e-008, 5.67810638e-012, -3.58325356e-016}, bhigh = {107312.9932, -29.01093335}, R = 319.5541911734304);
          constant IdealGases.Common.DataRecord Be_OH_2(name = "Be_OH_2", MM = 0.043026862, Hf = -14848036.46614991, H0 = 353868.6600012801, Tlimit = 1000, alow = {7863.08983, -739.7142249999999, 10.43303334, 0.00451424953, -6.95060153e-006, 5.77703004e-009, -1.727199382e-012}, blow = {-75856.7696, -33.0146661}, ahigh = {1722889.403, -5242.47719, 14.86877785, 3.93149836e-005, -8.92027631e-008, 1.630912925e-011, -9.505371139999999e-016}, bhigh = {-46428.2384, -64.7680402}, R = 193.2390979383995);
          constant IdealGases.Common.DataRecord BeS(name = "BeS", MM = 0.041077182, Hf = 6015390.880513663, H0 = 213746.5515526357, Tlimit = 1000, alow = {-7390.57908, 282.8134792, 0.65520194, 0.01076012495, -1.398380157e-005, 8.81841889e-009, -2.110588578e-012}, blow = {27515.93294, 19.80493561}, ahigh = {-16557083.61, 54314.6612, -63.5185925, 0.0396409199, -1.078978696e-005, 1.394030343e-009, -6.93695504e-014}, bhigh = {-312155.9628, 480.3454935}, R = 202.4109638290182);
          constant IdealGases.Common.DataRecord Be2(name = "Be2", MM = 0.018024364, Hf = 35371196.34290564, H0 = 545823.3089389452, Tlimit = 1000, alow = {-158087.381, 2035.567774, -3.69158063, 0.01028818435, -9.60385451e-006, 4.707519419999999e-009, -9.374688790000001e-013}, blow = {65269.68369999999, 48.7932499}, ahigh = {101003.4484, 141.4207488, 2.292070182, 0.0001543331824, -5.801043e-008, 1.003129697e-011, -5.68281834e-016}, bhigh = {75501.3973, 12.38758616}, R = 461.290728482847);
          constant IdealGases.Common.DataRecord Be2CL4(name = "Be2CL4", MM = 0.159836364, Hf = -5127775.385330963, H0 = 141653.5225989, Tlimit = 1000, alow = {121615.2589, -2000.824823, 17.23384275, 0.00581044828, -1.362970089e-005, 1.156602362e-008, -3.5522558e-012}, blow = {-92064.75470000001, -61.37959773999999}, ahigh = {-335492.008, -231.6199283, 16.17047912, -6.73440595e-005, 1.472073032e-008, -1.672352806e-012, 7.68877397e-017}, bhigh = {-103107.5197, -50.47009374}, R = 52.01865077461346);
          constant IdealGases.Common.DataRecord Be2F4(name = "Be2F4", MM = 0.0940179768, Hf = -18418816.28322808, H0 = 210965.0587588479, Tlimit = 1000, alow = {-6327.70187, -69.0253372, 5.47093723, 0.03097341347, -4.09431006e-005, 2.668113685e-008, -6.942909560000001e-012}, blow = {-210597.9269, -0.2073546195}, ahigh = {-368183.495, -943.919148, 16.70099855, -0.0002797974015, 6.178061719999999e-008, -7.08308019e-012, 3.28279104e-016}, bhigh = {-208904.5725, -60.9266576}, R = 88.43491726786446);
          constant IdealGases.Common.DataRecord Be2O(name = "Be2O", MM = 0.034023764, Hf = -1088468.724389224, H0 = 328899.1188629218, Tlimit = 1000, alow = {-426.367357, -115.9556261, 4.08267777, 0.01015885064, -1.326015538e-005, 8.48895536e-009, -2.168039077e-012}, blow = {-5362.25812, 1.253737886}, ahigh = {-141264.6378, -349.864459, 7.76093268, -0.0001044953518, 2.313342334e-008, -2.657722263e-012, 1.233820439e-016}, bhigh = {-5172.38549, -18.54887619}, R = 244.3724921205073);
          constant IdealGases.Common.DataRecord Be2OF2(name = "Be2OF2", MM = 0.0720205704, Hf = -16725415.99309522, H0 = 230279.6813172699, Tlimit = 1000, alow = {-27834.91921, 178.2881461, 4.09425136, 0.02270142496, -2.672688189e-005, 1.572575618e-008, -3.73607058e-012}, blow = {-148008.1903, 7.35349477}, ahigh = {-228182.6506, -1130.355396, 13.83400563, -0.000331608064, 7.30492101e-008, -8.36270438e-012, 3.87218498e-016}, bhigh = {-143139.0587, -47.3915023}, R = 115.4457949141708);
          constant IdealGases.Common.DataRecord Be2O2(name = "Be2O2", MM = 0.050023164, Hf = -8228893.898034918, H0 = 218477.4237791116, Tlimit = 1000, alow = {-30317.1953, 1163.353177, -8.73315595, 0.0537200293, -7.27943919e-005, 4.89492825e-008, -1.312147056e-011}, blow = {-55469.3688, 69.53282640000001}, ahigh = {-396057.6640000001, -907.4948830000001, 10.67160422, -0.0002670905218, 5.87762999e-008, -6.71866409e-012, 3.105949075e-016}, bhigh = {-48611.1433, -36.1654689}, R = 166.2124371021393);
          constant IdealGases.Common.DataRecord Be3O3(name = "Be3O3", MM = 0.075034746, Hf = -13643287.71100258, H0 = 201583.0506043161, Tlimit = 1000, alow = {-27380.72346, 874.38326, -6.59960612, 0.0653302114, -8.762305740000001e-005, 5.81724479e-008, -1.5406005e-011}, blow = {-128467.9012, 58.3720312}, ahigh = {-589080.936, -1392.823588, 17.03450699, -0.000412733074, 9.10752915e-008, -1.043465231e-011, 4.83305377e-016}, bhigh = {-121904.5534, -70.0061451}, R = 110.8082914014262);
          constant IdealGases.Common.DataRecord Be4O4(name = "Be4O4", MM = 0.100046328, Hf = -16485308.50627521, H0 = 158265.6686810135, Tlimit = 1000, alow = {-68932.5564, 1687.565069, -14.79616594, 0.1007319774, -0.0001291057622, 8.243136760000001e-008, -2.111283987e-011}, blow = {-207287.7151, 100.2507262}, ahigh = {-884133.8459999999, -2726.394165, 24.02941774, -0.0008114675460000001, 1.794255972e-007, -2.059359498e-011, 9.552905330000001e-016}, bhigh = {-192371.0587, -113.662075}, R = 83.10621855106966);
          constant IdealGases.Common.DataRecord Br(name = "Br", MM = 0.079904, Hf = 1400055.066079295, H0 = 77560.92310772926, Tlimit = 1000, alow = {-3700.29391, 61.4521542, 2.092120721, 0.00137681887, -2.445566658e-006, 2.050975161e-009, -5.144249091e-013}, blow = {12425.08647, 8.996166199999999}, ahigh = {-4789717.399999999, 16920.51999, -20.24085357, 0.01395620355, -3.65623056e-006, 4.489781e-010, -2.122507526e-014}, bhigh = {-92070.54960000001, 166.1695929}, R = 104.0557669203044);
          constant IdealGases.Common.DataRecord Brplus(name = "Brplus", MM = 0.07990345139999999, Hf = 15743087.58832913, H0 = 77561.53071505495, Tlimit = 1000, alow = {39811.8742, -475.158147, 4.73457898, -0.00516932479, 5.90585752e-006, -2.885129283e-009, 5.054894290000001e-013}, blow = {152905.2046, -5.76928322}, ahigh = {1741149.829, -5455.46567, 8.470663350000001, -0.002683465574, 6.59045456e-007, -7.9537703e-011, 3.735898818e-015}, bhigh = {185178.1643, -36.3790903}, R = 104.0564813449348);
          constant IdealGases.Common.DataRecord Brminus(name = "Brminus", MM = 0.0799045486, Hf = -2740776.036371977, H0 = 77560.39059833948, Tlimit = 1000, alow = {0, 0, 2.5, 0, 0, 0, 0}, blow = {-27084.92743, 5.41955617}, ahigh = {0, 0, 2.5, 0, 0, 0, 0}, bhigh = {-27084.92743, 5.41955617}, R = 104.055052505484);
          constant IdealGases.Common.DataRecord BrCL(name = "BrCL", MM = 0.115357, Hf = 128202.9959170228, H0 = 81547.82978059414, Tlimit = 1000, alow = {16532.91583, -357.876928, 5.70062216, -0.002201257188, 2.415146969e-006, -1.371032976e-009, 3.22780342e-013}, blow = {2252.619279, -4.155659669}, ahigh = {-128486.3067, -1138.379626, 7.9640408, -0.00343805667, 1.524999514e-006, -2.761469114e-010, 1.694450407e-014}, bhigh = {5938.264260000001, -19.25044389}, R = 72.07600752446753);
          constant IdealGases.Common.DataRecord BrF(name = "BrF", MM = 0.09890240319999999, Hf = -595045.6520352784, H0 = 91212.22243465163, Tlimit = 1000, alow = {38361.4319, -518.461197, 5.45832705, -0.00056568431, -4.12131197e-007, 7.85991348e-010, -3.03616152e-013}, blow = {-5595.53991, -4.9011295}, ahigh = {3771388.47, -12674.9343, 20.8055593, -0.0102635936, 3.30820359e-006, -4.90319225e-010, 2.64827508e-014}, bhigh = {70626.0972, -113.02536}, R = 84.06744154827575);
          constant IdealGases.Common.DataRecord BrF3(name = "BrF3", MM = 0.1368992096, Hf = -1867067.025053153, H0 = 107463.1478369032, Tlimit = 1000, alow = {111645.5127, -1959.42433, 15.26220126, -0.0079993623, 6.98203026e-006, -3.24275796e-009, 6.16475146e-013}, blow = {-23453.41551, -55.226336}, ahigh = {-188653.224, -82.45232970000001, 10.06180694, -2.477319092e-005, 5.47857686e-009, -6.28265526e-013, 2.910628214e-017}, bhigh = {-33868.0037, -22.98937369}, R = 60.73425861474075);
          constant IdealGases.Common.DataRecord BrF5(name = "BrF5", MM = 0.174896016, Hf = -2451742.525684519, H0 = 109633.9610160131, Tlimit = 1000, alow = {221136.6805, -4024.80922, 27.34900212, -0.01832254979, 1.721754649e-005, -8.75350863e-009, 1.860196959e-012}, blow = {-35374.4767, -124.4296466}, ahigh = {-375764.764, -157.5896472, 16.11869838, -4.776497799999999e-005, 1.059755365e-008, -1.218527723e-012, 5.65753545e-017}, bhigh = {-56672.66800000001, -55.4083931}, R = 47.53951628034798);
          constant IdealGases.Common.DataRecord BrO(name = "BrO", MM = 0.0959034, Hf = 1311736.601622049, H0 = 94480.48765737189, Tlimit = 1000, alow = {14554.31116, 122.5152439, -0.3058728672, 0.02117309202, -3.49426119e-005, 2.622639719e-008, -7.50805954e-012}, blow = {13891.49932, 25.27867438}, ahigh = {-2392612.795, 6932.79313, -2.513183892, 0.00344654456, -7.569536839999999e-007, 6.51429134e-011, -1.715768736e-015}, bhigh = {-30844.99064, 53.6104836}, R = 86.6963215068496);
          constant IdealGases.Common.DataRecord OBrO(name = "OBrO", MM = 0.1119028, Hf = 1357915.31579192, H0 = 101826.1383986817, Tlimit = 1000, alow = {34702.13230000001, -340.134182, 3.93225162, 0.01215938586, -1.901501702e-005, 1.398631495e-008, -3.99289976e-012}, blow = {18759.55788, 6.36123518}, ahigh = {-162450.466, -145.3689372, 7.10582213, -4.14136207e-005, 8.981833259999999e-009, -1.013712869e-012, 4.63513314e-017}, bhigh = {16500.75386, -9.11181792}, R = 74.300839657274);
          constant IdealGases.Common.DataRecord BrOO(name = "BrOO", MM = 0.1119028, Hf = 965123.3034383413, H0 = 114837.6984311385, Tlimit = 1000, alow = {-44176.199, 495.963716, 3.23988495, 0.00594150211, -3.40269677e-006, -3.44477816e-011, 4.81195795e-013}, blow = {8815.22198, 16.07446878}, ahigh = {6172.483480000001, -658.504966, 7.48395533, -0.0001920530746, 4.22694617e-008, -4.83739261e-012, 2.23974132e-016}, bhigh = {14603.5347, -9.85038024}, R = 74.300839657274);
          constant IdealGases.Common.DataRecord BrO3(name = "BrO3", MM = 0.1279022, Hf = 1726480.740753482, H0 = 102428.5821510498, Tlimit = 1000, alow = {106390.1881, -1501.928813, 9.70365743, 0.00813300076, -1.531857845e-005, 1.199117619e-008, -3.51274787e-012}, blow = {32331.3599, -27.3512543}, ahigh = {-282285.1113, -235.8973461, 10.17525279, -6.97845105e-005, 1.535701661e-008, -1.75445495e-012, 8.10410015e-017}, bhigh = {24011.4014, -25.88197197}, R = 65.00648151478239);
          constant IdealGases.Common.DataRecord Br2(name = "Br2", MM = 0.159808, Hf = 193419.6035242291, H0 = 60855.00725871045, Tlimit = 1000, alow = {7497.04754, -235.0884557, 5.49193432, -0.002227573303, 2.932401703e-006, -1.954889514e-009, 5.31230789e-013}, blow = {3521.47505, -1.96415157}, ahigh = {-4311698.57, 11112.68634, -5.55577561, 0.00363051659, -2.754164226e-007, -6.21750676e-011, 7.37534162e-015}, bhigh = {-70365.8416, 78.7847802}, R = 52.02788346015218);
          constant IdealGases.Common.DataRecord BrBrO(name = "BrBrO", MM = 0.1758074, Hf = 955591.1753430174, H0 = 74726.64973146751, Tlimit = 1000, alow = {16174.98037, -246.4406776, 5.91285845, 0.00498035428, -8.09995825e-006, 6.072776260000001e-009, -1.753715313e-012}, blow = {19740.12093, 2.009704963}, ahigh = {-85371.0175, -64.8704607, 7.04715935, -1.843088186e-005, 3.99226786e-009, -4.50068795e-013, 2.055867395e-017}, bhigh = {18215.66655, -3.162186125}, R = 47.29307185021791);
          constant IdealGases.Common.DataRecord BrOBr(name = "BrOBr", MM = 0.1758074, Hf = 612255.2634303221, H0 = 70526.37147241812, Tlimit = 1000, alow = {64050.6891, -1062.722566, 9.907866289999999, -0.00447049247, 3.91370107e-006, -1.805444867e-009, 3.36613255e-013}, blow = {16429.21537, -23.50319399}, ahigh = {-96866.60249999999, -39.5739527, 7.02943438, -1.171881908e-005, 2.576958549e-009, -2.941186078e-013, 1.357198784e-017}, bhigh = {10769.42321, -5.69777938}, R = 47.29307185021791);
          constant IdealGases.Common.DataRecord C(name = "C", MM = 0.0120107, Hf = 59670127.46967287, H0 = 544172.696012722, Tlimit = 1000, alow = {649.503147, -0.964901086, 2.504675479, -1.281448025e-005, 1.980133654e-008, -1.606144025e-011, 5.314483411e-015}, blow = {85457.6311, 4.747924288}, ahigh = {-128913.6472, 171.9528572, 2.646044387, -0.000335306895, 1.74209274e-007, -2.902817829e-011, 1.642182385e-015}, bhigh = {84105.9785, 4.130047418}, R = 692.2554055966764);
          constant IdealGases.Common.DataRecord Cplus(name = "Cplus", MM = 0.0120101514, Hf = 150659589.6867712, H0 = 553638.482858759, Tlimit = 1000, alow = {2258.535929, -1.574575687, 2.50363773, -5.20287837e-006, 4.51690839e-009, -2.181431053e-012, 4.495047033e-016}, blow = {216895.1913, 4.345699505}, ahigh = {12551.12551, -34.1187467, 2.543383218, -2.805120849e-005, 9.751641969999999e-009, -1.736855394e-012, 1.246191931e-016}, bhigh = {217100.1786, 4.063913515}, R = 692.2870264566357);
          constant IdealGases.Common.DataRecord Cminus(name = "Cminus", MM = 0.0120112486, Hf = 48980273.04172191, H0 = 517751.0021730797, Tlimit = 1000, alow = {4.67129153, -0.001986169369, 2.500008638, -1.976750928e-008, 2.478947477e-011, -1.610664044e-014, 4.23650681e-018}, blow = {70012.18549999999, 4.879570141}, ahigh = {4.25317572, 0.0005778186479999999, 2.499999424, 2.836136231e-010, -7.32725342e-014, 9.478507810000001e-018, -4.830487319999999e-022}, bhigh = {70012.17170000001, 4.879624211}, R = 692.22378762521);
          constant IdealGases.Common.DataRecord CBr(name = "CBr", MM = 0.0919147, Hf = 5335727.756278375, H0 = 104609.1212831027, Tlimit = 1000, alow = {4324.44688, -159.6366559, 4.85785659, -0.000438943916, 4.40886363e-007, -2.281330618e-010, 5.44561822e-014}, blow = {58476.7725, 0.1351169646}, ahigh = {1021862.102, -3243.48436, 8.38822656, -0.002242155508, 6.988355369999999e-007, -9.719134370000001e-011, 4.71697624e-015}, bhigh = {78059.24059999999, -25.18386915}, R = 90.45856647522105);
          constant IdealGases.Common.DataRecord CBr2(name = "CBr2", MM = 0.1718187, Hf = 1959177.429464895, H0 = 70986.5631622169, Tlimit = 1000, alow = {72481.97839999999, -1128.26123, 9.822303379999999, -0.0038727481, 2.904819596e-006, -1.061380159e-009, 1.297474642e-013}, blow = {44377.71599999999, -23.61121795}, ahigh = {-744143.141, 421.542783, 8.796797740000001, -0.002726562507, 1.336438546e-006, -2.304854483e-010, 1.337493953e-014}, bhigh = {33646.1814, -15.99565135}, R = 48.39096093731357);
          constant IdealGases.Common.DataRecord CBr3(name = "CBr3", MM = 0.2517227, Hf = 933566.9766771132, H0 = 61903.25703641347, Tlimit = 1000, alow = {85635.1626, -1275.764123, 11.17190031, 0.002341705749, -6.54067141e-006, 5.73671714e-009, -1.777823046e-012}, blow = {32432.1589, -27.66468801}, ahigh = {-192649.8949, -132.4072178, 10.09808665, -3.89504082e-005, 8.55044501e-009, -9.747493679999999e-013, 4.49424559e-017}, bhigh = {25415.76129, -18.66868947}, R = 33.03028292641069);
          constant IdealGases.Common.DataRecord CBr4(name = "CBr4", MM = 0.3316267000000001, Hf = 239727.3802139574, H0 = 61434.48039618039, Tlimit = 1000, alow = {102308.3739, -1735.600709, 16.51317914, -0.003147664747, 2.924135486e-007, 1.360605681e-009, -6.43175865e-013}, blow = {15005.02543, -55.3647676}, ahigh = {-205978.5072, -104.8517002, 13.0777255, -3.086857556e-005, 6.77530981e-009, -7.72183754e-013, 3.55921824e-017}, bhigh = {5615.89394, -32.8378763}, R = 25.0717810116013);
          constant IdealGases.Common.DataRecord CCL(name = "CCL", MM = 0.0474637, Hf = 9114566.668843769, H0 = 197943.1228496725, Tlimit = 1000, alow = {17070.70049, -14.63803497, 2.334108251, 0.00720209913, -1.034120045e-005, 7.2281067e-009, -1.985123755e-012}, blow = {51233.503, 12.00820386}, ahigh = {590050.763, -2075.31146, 6.88001723, -0.001294796228, 3.88313798e-007, -5.00698163e-011, 2.218229619e-015}, bhigh = {63589.2205, -16.10664472}, R = 175.1753866639137);
          constant IdealGases.Common.DataRecord CCL2(name = "CCL2", MM = 0.08291670000000001, Hf = 2688724.310060578, H0 = 137761.8501459899, Tlimit = 1000, alow = {75096.23150000001, -1034.728559, 8.061704539999999, 0.001487134213, -4.56481711e-006, 3.95488581e-009, -1.166197091e-012}, blow = {30524.09293, -17.39691692}, ahigh = {-6437455.17, 19584.13353, -16.04038625, 0.01238833906, -3.013367274e-006, 3.47346892e-010, -1.553272793e-014}, bhigh = {-99612.42159999999, 155.6949315}, R = 100.2749988844225);
          constant IdealGases.Common.DataRecord CCL2Br2(name = "CCL2Br2", MM = 0.2427247, Hf = 41198.93855054718, H0 = 77008.69338802355, Tlimit = 1000, alow = {101849.5619, -1740.854039, 15.21338304, 0.00112825651, -5.875405490000001e-006, 5.65988697e-009, -1.821576512e-012}, blow = {6918.57219, -50.15385818999999}, ahigh = {-255334.7012, -159.8005477, 13.1186093, -4.717443870000001e-005, 1.036912808e-008, -1.183321687e-012, 5.46061269e-017}, bhigh = {-2588.63698, -34.61705319}, R = 34.25474210082452);
          constant IdealGases.Common.DataRecord CCL3(name = "CCL3", MM = 0.1183697, Hf = 600897.0200988936, H0 = 121652.7540409412, Tlimit = 1000, alow = {34144.7165, -554.796973, 6.84554439, 0.01242006495, -2.036896881e-005, 1.554979993e-008, -4.56894011e-012}, blow = {9388.56882, -7.143598465}, ahigh = {-542757.254, 680.902062, 9.047723619999999, 0.0001324277766, 6.30126304e-008, -2.6566392e-011, 2.105668053e-015}, bhigh = {386.621615, -15.28745849}, R = 70.24155674974254);
          constant IdealGases.Common.DataRecord CCL3Br(name = "CCL3Br", MM = 0.1982737, Hf = -216871.9300643504, H0 = 90705.67099922984, Tlimit = 1000, alow = {104966.8463, -1793.828482, 14.89250892, 0.00239712831, -7.78961547e-006, 7.01661711e-009, -2.196069746e-012}, blow = {910.1349309999999, -50.58348139}, ahigh = {-277232.4186, -184.0310333, 13.13673812, -5.44391001e-005, 1.197658923e-008, -1.367818632e-012, 6.316164790000001e-017}, bhigh = {-8895.857689999999, -36.69528209}, R = 41.93431604897675);
          constant IdealGases.Common.DataRecord CCL4(name = "CCL4", MM = 0.1538227, Hf = -621494.746874161, H0 = 111547.3333909754, Tlimit = 1000, alow = {109338.5626, -1861.731846, 14.49632467, 0.00394948516, -1.010805379e-005, 8.64551417e-009, -2.642368852e-012}, blow = {-4948.00623, -51.8028475}, ahigh = {-304307.8255, -216.8170491, 13.16132386, -6.43112489e-005, 1.416482497e-008, -1.61934332e-012, 7.48397435e-017}, bhigh = {-15123.2007, -39.968443}, R = 54.05230827439643);
          constant IdealGases.Common.DataRecord CF(name = "CF", MM = 0.0310091032, Hf = 7813834.487157951, H0 = 292337.0257286254, Tlimit = 1000, alow = {-45827.8668, 797.263589, -1.364936319, 0.01312359375, -1.457012892e-005, 8.25327668e-009, -1.896351845e-012}, blow = {24382.5962, 32.4806528}, ahigh = {-132980.71, -121.4340891, 4.45241362, 0.0001293786948, -3.67055059e-008, 6.590212389999999e-012, -3.73126161e-016}, bhigh = {28159.34108, -0.664046873}, R = 268.1300373756052);
          constant IdealGases.Common.DataRecord CFplus(name = "CFplus", MM = 0.0310085546, Hf = 36943485.87921605, H0 = 280487.0821034657, Tlimit = 1000, alow = {-59752.132, 927.6863649999999, -1.866581982, 0.01384340912, -1.51342886e-005, 8.51855634e-009, -1.959577655e-012}, blow = {132351.941, 34.1201319}, ahigh = {-80537.6278, -339.027509, 4.7083735, -3.96314831e-005, 2.497996218e-008, -5.23679043e-012, 5.31394913e-016}, bhigh = {138133.7068, -3.92051473}, R = 268.1347811032766);
          constant IdealGases.Common.DataRecord CFBr3(name = "CFBr3", MM = 0.2707211032, Hf = -443260.6050343547, H0 = 67387.07763953881, Tlimit = 1000, alow = {51492.4428, -936.479664, 10.44260886, 0.01205370047, -1.876290291e-005, 1.344488618e-008, -3.73054576e-012}, blow = {-12432.46293, -23.63948551}, ahigh = {-272915.8956, -332.816465, 13.24822519, -9.92707754e-005, 2.193640054e-008, -2.515424945e-012, 1.165680878e-016}, bhigh = {-17301.85479, -36.2681843}, R = 30.71231574384335);
          constant IdealGases.Common.DataRecord CFCL(name = "CFCL", MM = 0.0664621032, Hf = 388879.8692124447, H0 = 164039.0760309253, Tlimit = 1000, alow = {15419.97945, -82.74039809999999, 2.501701822, 0.01390738086, -1.8957347e-005, 1.261934581e-008, -3.33345382e-012}, blow = {2411.755114, 13.31517094}, ahigh = {-268163.0077, -2.328091539, 6.83463383, 0.0001565270633, -5.967755160000001e-008, 9.82092446e-012, -5.05889721e-016}, bhigh = {301.7669604, -9.005085128999999}, R = 125.1009462487188);
          constant IdealGases.Common.DataRecord CFCLBr2(name = "CFCLBr2", MM = 0.2262701032, Hf = -773411.9422985264, H0 = 77332.45688465308, Tlimit = 1000, alow = {58987.9974, -1050.714861, 10.39349401, 0.01273690622, -1.999097478e-005, 1.43820423e-008, -4.00013035e-012}, blow = {-18377.99602, -24.17479131}, ahigh = {-296081.1596, -353.12762, 13.26346169, -0.0001053860298, 2.32908508e-008, -2.670979639e-012, 1.237844667e-016}, bhigh = {-23875.94306, -36.84977271}, R = 36.74578250689551);
          constant IdealGases.Common.DataRecord CFCL2(name = "CFCL2", MM = 0.1019151032, Hf = -1030269.280049161, H0 = 129685.9207811703, Tlimit = 1000, alow = {26113.98792, -335.35659, 4.09467308, 0.01960244711, -2.762775193e-005, 1.880124302e-008, -5.0474103e-012}, blow = {-12512.98871, 6.870882561}, ahigh = {-265311.8323, -446.972417, 10.33334881, -0.0001333673368, 2.948755634e-008, -3.3833062e-012, 1.568746348e-016}, bhigh = {-13931.60489, -25.56817972}, R = 81.58233410884678);
          constant IdealGases.Common.DataRecord CFCL2Br(name = "CFCL2Br", MM = 0.1818191032, Hf = -1292493.450160192, H0 = 92017.68519117852, Tlimit = 1000, alow = {65299.59220000001, -1135.235485, 10.09526872, 0.01401380147, -2.191705096e-005, 1.573404132e-008, -4.36934034e-012}, blow = {-25044.88145, -24.51093103}, ahigh = {-321545.505, -388.412962, 13.28990796, -0.0001160048581, 2.564498787e-008, -2.941639768e-012, 1.363542881e-016}, bhigh = {-30974.21144, -38.71356695}, R = 45.72936426187367);
          constant IdealGases.Common.DataRecord CFCL3(name = "CFCL3", MM = 0.1373681032, Hf = -2065253.820873898, H0 = 116938.2383959423, Tlimit = 1000, alow = {74001.74000000001, -1252.282466, 10.01327229, 0.01475107601, -2.315635251e-005, 1.664207025e-008, -4.62292996e-012}, blow = {-30205.08601, -27.08387087}, ahigh = {-345046.478, -417.782145, 13.31196736, -0.0001248759531, 2.761391353e-008, -3.16820764e-012, 1.468834601e-016}, bhigh = {-36740.6719, -41.56886057000001}, R = 60.52694771430753);
          constant IdealGases.Common.DataRecord CF2(name = "CF2", MM = 0.0500075064, Hf = -3731439.806404744, H0 = 206987.1654308282, Tlimit = 1000, alow = {-37970.2627, 873.1800030000001, -3.46019157, 0.02746253741, -3.4746219e-005, 2.204470693e-008, -5.62175085e-012}, blow = {-27467.97157, 44.5677807}, ahigh = {-108642.8547, -585.498914, 7.01864895, 0.000392918615, -2.603822675e-007, 6.142196389999999e-011, -4.17283326e-015}, bhigh = {-21529.45157, -13.56143285}, R = 166.2644790462898);
          constant IdealGases.Common.DataRecord CF2plus(name = "CF2plus", MM = 0.0500069578, Hf = 18984184.21686112, H0 = 206811.2009805164, Tlimit = 1000, alow = {-28923.73827, 604.935461, -0.972186666, 0.01808478341, -1.980065782e-005, 1.101640055e-008, -2.500457119e-012}, blow = {110275.567, 32.4759147}, ahigh = {-64889.6607, -1064.283593, 7.7857021, -0.0003158347626, 7.130804079999999e-008, -8.53318359e-012, 4.25163882e-016}, bhigh = {117844.6465, -18.19424707}, R = 166.2663030463333);
          constant IdealGases.Common.DataRecord CF2Br2(name = "CF2Br2", MM = 0.2098155064, Hf = -1811114.948175251, H0 = 77591.96295512693, Tlimit = 1000, alow = {12614.04028, -281.5725114, 5.09803213, 0.02502548939, -3.43885319e-005, 2.297615553e-008, -6.08016143e-012}, blow = {-46427.6375, 3.094167118}, ahigh = {-332766.93, -618.435249, 13.46170049, -0.0001849172548, 4.09259134e-008, -4.69981043e-012, 2.180811799e-016}, bhigh = {-47151.8661, -41.0536924}, R = 39.62753822469625);
          constant IdealGases.Common.DataRecord CF2CL(name = "CF2CL", MM = 0.0854605064, Hf = -3217860.642117609, H0 = 145473.8981045869, Tlimit = 1000, alow = {8914.558349999999, -17.68816959, 1.607015025, 0.02473626554, -3.27073894e-005, 2.12635183e-008, -5.509831110000001e-012}, blow = {-34273.0625, 19.2968069}, ahigh = {-280234.6654, -686.031485, 10.51143635, -0.0002046954963, 4.52889718e-008, -5.20022903e-012, 2.41297145e-016}, bhigh = {-33081.7948, -28.74899785}, R = 97.29022621377774);
          constant IdealGases.Common.DataRecord CF2CLBr(name = "CF2CLBr", MM = 0.1653645064, Hf = -2630552.404926478, H0 = 93903.11946651207, Tlimit = 1000, alow = {26966.99365, -467.883227, 5.24480942, 0.02530121349, -3.508443520000001e-005, 2.354353958e-008, -6.24616113e-012}, blow = {-51983.8273, 0.8532887803}, ahigh = {-359282.884, -653.928359, 13.48831524, -0.0001956099469, 4.32977597e-008, -4.97264437e-012, 2.30757505e-016}, bhigh = {-53651.4117, -42.24913908000001}, R = 50.27966509262958);
          constant IdealGases.Common.DataRecord CF2CL2(name = "CF2CL2", MM = 0.1209135064, Hf = -4059099.885635274, H0 = 123072.2228066988, Tlimit = 1000, alow = {38412.5875, -613.949982, 5.26966719, 0.02574105783, -3.58737648e-005, 2.41160013e-008, -6.4022369e-012}, blow = {-57845.4133, -1.957454856}, ahigh = {-382744.855, -693.37748, 13.51791274, -0.0002075114435, 4.59402161e-008, -5.276874600000001e-012, 2.449037044e-016}, bhigh = {-60215.2777, -44.79580046}, R = 68.76379858255437);
          constant IdealGases.Common.DataRecord CF3(name = "CF3", MM = 0.0690059096, Hf = -6773332.932053692, H0 = 166528.7808915426, Tlimit = 1000, alow = {-29783.07106, 715.367883, -3.49818538, 0.0359545799, -4.50797443e-005, 2.82180845e-008, -7.098047020000001e-012}, blow = {-60599.9703, 45.0259264}, ahigh = {-299730.5557, -1046.989457, 10.77923191, -0.0003116087076, 6.89135143e-008, -7.91122564e-012, 3.67059302e-016}, bhigh = {-54253.044, -34.1703879}, R = 120.489274733073);
          constant IdealGases.Common.DataRecord CF3plus(name = "CF3plus", MM = 0.06900536099999999, Hf = 6138906.801748347, H0 = 167251.9327882366, Tlimit = 1000, alow = {-35874.6354, 374.35067, 0.778741393, 0.01929435768, -1.89281577e-005, 9.36390876e-009, -1.885759656e-012}, blow = {47755.8172, 22.24044005}, ahigh = {-29109.19531, -1996.658183, 11.44279678, -0.000565339304, 1.232297113e-007, -1.399758046e-011, 6.44301795e-016}, bhigh = {59023.458, -40.80349990000001}, R = 120.4902326356934);
          constant IdealGases.Common.DataRecord CF3Br(name = "CF3Br", MM = 0.1489099096, Hf = -4356996.80258217, H0 = 96998.43374292129, Tlimit = 1000, alow = {-5439.48901, 124.361519, 0.923194195, 0.0348104438, -4.54964549e-005, 2.932836552e-008, -7.54800036e-012}, blow = {-80233.9676, 22.32999096}, ahigh = {-383408.851, -971.9093300000001, 13.72477396, -0.0002901710612, 6.421892200000001e-008, -7.37568877e-012, 3.42315425e-016}, bhigh = {-77653.0631, -47.1736721}, R = 55.83558557206995);
          constant IdealGases.Common.DataRecord CF3CL(name = "CF3CL", MM = 0.1044589096, Hf = -6741406.766512906, H0 = 132020.8209410603, Tlimit = 1000, alow = {14994.09978, -136.5768572, 1.45291237, 0.0340476507, -4.47377764e-005, 2.888115112e-008, -7.43417133e-012}, blow = {-85471.6664, 17.27327095}, ahigh = {-406604.2119999999, -1022.626507, 13.76252628, -0.0003052722979, 6.75596604e-008, -7.75931525e-012, 3.60119229e-016}, bhigh = {-84105.5937, -49.13411939}, R = 79.59562311954289);
          constant IdealGases.Common.DataRecord CF4(name = "CF4", MM = 0.0880043128, Hf = -10603116.71452538, H0 = 144655.558289866, Tlimit = 1000, alow = {9817.458500000001, 116.3343483, -1.288338636, 0.0395956691, -4.99624421e-005, 3.125339346e-008, -7.841700320000001e-012}, blow = {-113850.2297, 29.38656548}, ahigh = {-416445.678, -1414.797167, 14.05124837, -0.000419909386, 9.27892161e-008, -1.06457583e-011, 4.937099680000001e-016}, bhigh = {-109469.1149, -54.87105}, R = 94.47800608244737);
          constant IdealGases.Common.DataRecord CHplus(name = "CHplus", MM = 0.0130180914, Hf = 125254229.2029076, H0 = 662777.9552999605, Tlimit = 1000, alow = {30196.07105, -461.814131, 6.22564119, -0.0077757118, 1.094489485e-005, -6.67548791e-009, 1.565232409e-012}, blow = {197249.1928, -14.31512811}, ahigh = {-7102094.67, 18283.54883, -13.12691402, 0.006191717360000001, -2.909421253e-007, -1.134243575e-010, 1.105962085e-014}, bhigh = {75412.9604, 124.3984829}, R = 638.6859443927395);
          constant IdealGases.Common.DataRecord CHBr3(name = "CHBr3", MM = 0.25273064, Hf = 66236.5275536041, H0 = 62938.94954723337, Tlimit = 1000, alow = {43465.7686, -499.963442, 5.62022211, 0.02079346321, -2.921336328e-005, 2.066942448e-008, -5.78122091e-012}, blow = {2627.830062, 1.247904015}, ahigh = {627423.495, -3378.7111, 14.87852612, -0.000593274527, 1.082421577e-007, -1.061012665e-011, 4.31915245e-016}, bhigh = {18772.34659, -53.1190965}, R = 32.89855159627658);
          constant IdealGases.Common.DataRecord CHCL(name = "CHCL", MM = 0.04847164, Hf = 6129357.290159771, H0 = 210441.4663914817, Tlimit = 1000, alow = {-269991.2334, 4351.19973, -22.75911813, 0.07550621760000001, -9.07199797e-005, 5.25697186e-008, -1.189769182e-011}, blow = {14168.6268, 152.0980812}, ahigh = {-954806.1900000001, 2174.413794, 4.86764537, 0.0008321641859999999, -1.536948638e-007, 1.529236537e-011, -6.596159359999999e-016}, bhigh = {18801.2181, 2.674761385}, R = 171.5327147998293);
          constant IdealGases.Common.DataRecord CHCLBr2(name = "CHCLBr2", MM = 0.20827964, Hf = 48012.37413316059, H0 = 73416.85437904541, Tlimit = 1000, alow = {38153.8025, -408.4175110000001, 4.63924402, 0.02325990646, -3.22298886e-005, 2.254671151e-008, -6.25650774e-012}, blow = {1483.809044, 6.175235281}, ahigh = {604286.53, -3438.24101, 14.94162631, -0.000622842797, 1.153724426e-007, -1.147291377e-011, 4.733482859999999e-016}, bhigh = {18224.60958, -54.07258924}, R = 39.91975403836881);
          constant IdealGases.Common.DataRecord CHCL2(name = "CHCL2", MM = 0.08392464, Hf = 1141500.27929819, H0 = 152517.7826202174, Tlimit = 1000, alow = {56385.5613, -656.245542, 6.06881802, 0.0097441226, -1.275310131e-005, 8.55238672e-009, -2.2315827e-012}, blow = {13304.47798, -4.533535456}, ahigh = {884939.3709999999, -3526.96973, 11.86372153, -0.000500237612, 6.1338604e-008, -1.885433405e-012, -1.234221421e-016}, bhigh = {30711.36475, -40.88905858}, R = 99.0706900857722);
          constant IdealGases.Common.DataRecord CHCL2Br(name = "CHCL2Br", MM = 0.16382864, Hf = -274677.248129509, H0 = 89910.37830748031, Tlimit = 1000, alow = {32940.2265, -327.530117, 3.77323524, 0.02536466143, -3.47266699e-005, 2.404666618e-008, -6.62173093e-012}, blow = {-5425.56508, 9.433412003999999}, ahigh = {592487.644, -3509.47561, 14.99349593, -0.000643306257, 1.198581809e-007, -1.198476716e-011, 4.96994807e-016}, bhigh = {11972.51405, -56.02687781}, R = 50.75102863577455);
          constant IdealGases.Common.DataRecord CHCL3(name = "CHCL3", MM = 0.11937764, Hf = -860295.1105416392, H0 = 119833.9488031427, Tlimit = 1000, alow = {33953.3329, -304.7428785, 2.923672263, 0.02830547858, -3.71242469e-005, 2.551365915e-008, -6.98765955e-012}, blow = {-12350.63157, 11.15556408}, ahigh = {613605.274, -3715.08717, 15.10777247, 0.0002362584336, 1.297140438e-007, -1.267494791e-011, 5.259022309999999e-016}, bhigh = {6203.31345, -59.92576539}, R = 69.64848693607949);
          constant IdealGases.Common.DataRecord CHF(name = "CHF", MM = 0.03201704320000001, Hf = 3398190.123939989, H0 = 311746.9635672041, Tlimit = 1000, alow = {-78685.85829999999, 1300.218466, -3.94768525, 0.02114995848, -2.239962738e-005, 1.283155181e-008, -2.904778136e-012}, blow = {5824.38972, 47.8545377}, ahigh = {3994085.42, -7962.756200000001, 6.75559509, 0.00494983657, -2.101763812e-006, 3.34823295e-010, -1.88682505e-014}, bhigh = {67061.34940000001, -23.81240379}, R = 259.6889396707313);
          constant IdealGases.Common.DataRecord CHFBr2(name = "CHFBr2", MM = 0.1918250432, Hf = -912289.6420648375, H0 = 74857.56687685709, Tlimit = 1000, alow = {-18201.9798, 436.224967, -0.373385417, 0.033886314, -4.37129853e-005, 2.889569295e-008, -7.683915110000001e-012}, blow = {-24656.11586, 33.2047879}, ahigh = {593575.1630000001, -3742.65658, 15.17184031, -0.0007158039030000001, 1.360636972e-007, -1.385897355e-011, 5.844209110000001e-016}, bhigh = {-2347.676027, -57.6578343}, R = 43.34403819907494);
          constant IdealGases.Common.DataRecord CHFCL(name = "CHFCL", MM = 0.06747004320000001, Hf = -1232320.301819519, H0 = 165198.5751211139, Tlimit = 1000, alow = {-71130.0851, 1377.872745, -6.48461724, 0.0389097249, -4.80073522e-005, 3.073300019e-008, -7.98285341e-012}, blow = {-17517.93745, 63.73895867}, ahigh = {662216.358, -3809.00798, 12.23880469, -0.000746718275, 1.43476785e-007, -1.475406631e-011, 6.27378068e-016}, bhigh = {10189.87746, -46.58365563}, R = 123.2320538961801);
          constant IdealGases.Common.DataRecord CHFCLBr(name = "CHFCLBr", MM = 0.1473740432, Hf = -1560654.746289813, H0 = 93552.3359516542, Tlimit = 1000, alow = {-19592.40614, 490.172327, -1.259408503, 0.0361770847, -4.65424496e-005, 3.065240659e-008, -8.1239036e-012}, blow = {-31399.00705, 37.10450945}, ahigh = {580011.7509999999, -3823.33785, 15.22809062, -0.000737390631, 1.407097155e-007, -1.438245319e-011, 6.08386715e-016}, bhigh = {-8552.249899999999, -59.03677705}, R = 56.41747908562503);
          constant IdealGases.Common.DataRecord CHFCL2(name = "CHFCL2", MM = 0.1029230432, Hf = -2768087.603534833, H0 = 129167.984026341, Tlimit = 1000, alow = {-17349.59015, 492.49132, -1.854635884, 0.0378278628, -4.86334304e-005, 3.19719763e-008, -8.45905799e-012}, blow = {-37887.4927, 38.01530544}, ahigh = {564011.302, -3887.0833, 15.27760246, -0.0007576487420000001, 1.452498792e-007, -1.490796515e-011, 6.3289985e-016}, bhigh = {-14847.36185, -61.67535516}, R = 80.78338670809922);
          constant IdealGases.Common.DataRecord CHF2(name = "CHF2", MM = 0.0510154464, Hf = -4682895.414201452, H0 = 214052.8167562991, Tlimit = 1000, alow = {-146969.0117, 2553.397313, -13.0262763, 0.0543812846, -6.71341879e-005, 4.28765173e-008, -1.110824216e-011}, blow = {-41793.7216, 99.39930250000001}, ahigh = {552680.655, -3696.00917, 12.08610573, -0.00064985881, 1.12084622e-007, -9.814147009999999e-012, 3.34919239e-016}, bhigh = {-9437.06842, -47.044162}, R = 162.9795010477454);
          constant IdealGases.Common.DataRecord CHF2Br(name = "CHF2Br", MM = 0.1309194464, Hf = -3223356.129315255, H0 = 100592.4204702412, Tlimit = 1000, alow = {-77145.1016, 1422.544046, -6.68439365, 0.0473838915, -5.83058875e-005, 3.69572047e-008, -9.49548528e-012}, blow = {-58785.00930000001, 66.0863909}, ahigh = {576770.684, -4177.71608, 15.50150581, -0.000849014729, 1.657177491e-007, -1.727828054e-011, 7.43562723e-016}, bhigh = {-29661.47348, -63.4938014}, R = 63.50830398867315);
          constant IdealGases.Common.DataRecord CHF2CL(name = "CHF2CL", MM = 0.08646844640000001, Hf = -5583539.662162821, H0 = 143029.0067059653, Tlimit = 1000, alow = {-72405.7941, 1365.510973, -7.15322208, 0.0485498797, -5.94878167e-005, 3.75096619e-008, -9.58952442e-012}, blow = {-65659.32799999999, 66.5721223}, ahigh = {562949.373, -4298.673049999999, 15.58191398, -0.000878927848, 1.720248689e-007, -1.797910869e-011, 7.753391699999999e-016}, bhigh = {-36340.6716, -66.11261479999999}, R = 96.15613956491762);
          constant IdealGases.Common.DataRecord CHF3(name = "CHF3", MM = 0.0700138496, Hf = -9902326.524836596, H0 = 165208.5418254162, Tlimit = 1000, alow = {-109513.226, 2042.273879, -11.66213079, 0.0578580777, -6.857135060000001e-005, 4.21211838e-008, -1.053196888e-011}, blow = {-93954.70050000001, 89.3592065}, ahigh = {568523.2020000001, -4728.3617, 15.86728516, -0.000738234865, 1.970841706e-007, -2.062115571e-011, 8.970599639999999e-016}, bhigh = {-59231.9637, -71.6127322}, R = 118.7546756463453);
          constant IdealGases.Common.DataRecord CHI3(name = "CHI3", MM = 0.39373205, Hf = 535576.4154835757, H0 = 43574.2708778724, Tlimit = 1000, alow = {52529.2359, -724.216647, 8.01862334, 0.01470995487, -2.163843469e-005, 1.594571452e-008, -4.60143796e-012}, blow = {26781.86915, -8.605054839999999}, ahigh = {629420.543, -3184.06157, 14.79622178, -0.00057495555, 1.062025541e-007, -1.052804696e-011, 4.32972002e-016}, bhigh = {41036.098, -49.0135739}, R = 21.11708203586678);
          constant IdealGases.Common.DataRecord CH2(name = "CH2", MM = 0.01402658, Hf = 27830341.89374745, H0 = 0, Tlimit = 1000, alow = {32189.2173, -287.7601815, 4.20358382, 0.00345540596, -6.74619334e-006, 7.65457164e-009, -2.870328419e-012}, blow = {0, 47336.2471}, ahigh = {10027.417, 2550418.031, -7971.62539, 12.28924487, -0.001699122922, 2.991728605e-007, -2.767007492e-011}, bhigh = {1.05134174e-015, 0}, R = 592.7654495964092);
          constant IdealGases.Common.DataRecord CH2Br2(name = "CH2Br2", MM = 0.17383458, Hf = -84965.83361032081, H0 = 72565.5792995847, Tlimit = 1000, alow = {4797.30801, 361.385924, -1.819592338, 0.0347704834, -4.49854618e-005, 3.068623685e-008, -8.43484634e-012}, blow = {-4481.49706, 38.2712359}, ahigh = {1528284.441, -6673.414949999999, 16.70213316, -0.001166351237, 2.122634493e-007, -2.07547224e-011, 8.4285777e-016}, bhigh = {36007.0664, -74.48065490000001}, R = 47.82979312861688);
          constant IdealGases.Common.DataRecord CH2CL(name = "CH2CL", MM = 0.04947958, Hf = 2409074.612193556, H0 = 221908.1083550022, Tlimit = 1000, alow = {-31885.8563, 633.316321, -1.164065495, 0.0216058608, -2.545462163e-005, 1.693887757e-008, -4.6600786e-012}, blow = {10201.4254, 32.30835289}, ahigh = {1662438.334, -6441.12572, 13.59753722, -0.00114053681, 2.087760159e-007, -2.052347186e-011, 8.37577227e-016}, bhigh = {52129.0612, -61.48586271}, R = 168.0384514177364);
          constant IdealGases.Common.DataRecord CH2CLBr(name = "CH2CLBr", MM = 0.12938358, Hf = -347803.0210634147, H0 = 94221.74745821687, Tlimit = 1000, alow = {-13079.48755, 645.586038, -3.57734991, 0.0383368723, -4.86425205e-005, 3.26162233e-008, -8.852877389999999e-012}, blow = {-9402.27253, 47.48806758}, ahigh = {1525016.309, -6823.33416, 16.82769102, -0.001219735119, 2.244964495e-007, -2.219063248e-011, 9.104705129999999e-016}, bhigh = {33202.3992, -76.37571062000001}, R = 64.26218844771493);
          constant IdealGases.Common.DataRecord CH2CL2(name = "CH2CL2", MM = 0.08493258000000001, Hf = -1118534.25387525, H0 = 139570.4333955238, Tlimit = 1000, alow = {-25098.41179, 868.766738, -5.09466921, 0.0415004999, -5.19977215e-005, 3.44594426e-008, -9.270292519999999e-012}, blow = {-16389.7884, 53.9689032}, ahigh = {1529279.337, -6976.95476, 16.94154931, -0.001265053995, 2.344766734e-007, -2.333227421e-011, 9.632834730000001e-016}, bhigh = {28063.18171, -79.49453509999999}, R = 97.89496563038588);
          constant IdealGases.Common.DataRecord CH2F(name = "CH2F", MM = 0.0330249832, Hf = -962907.3785569707, H0 = 336987.3023886958, Tlimit = 1000, alow = {-85697.1253, 1392.226749, -4.38205259, 0.02645916948, -2.848145663e-005, 1.732706028e-008, -4.44206144e-012}, blow = {-11694.44586, 50.494992}, ahigh = {2535399.502, -9358.43902, 17.00366206, -0.003062919929, 7.612869080000001e-007, -9.664554980000001e-011, 4.8447681e-015}, bhigh = {52283.9131, -87.0755157}, R = 251.7630955221803);
          constant IdealGases.Common.DataRecord CH2FBr(name = "CH2FBr", MM = 0.1129289832, Hf = -1903851.375507647, H0 = 102919.4248514229, Tlimit = 1000, alow = {-92561.0953, 1811.985865, -9.69834687, 0.0503584867, -6.07027268e-005, 3.8782508e-008, -1.012615992e-011}, blow = {-35375.0962, 81.4039605}, ahigh = {1539975.462, -7231.55136, 17.14566439, -0.001350184394, 2.538022824e-007, -2.558977386e-011, 1.069310946e-015}, bhigh = {15096.04224, -80.62053710000001}, R = 73.62566955265032);
          constant IdealGases.Common.DataRecord CH2FCL(name = "CH2FCL", MM = 0.0684779832, Hf = -3880079.225230454, H0 = 164320.8002656247, Tlimit = 1000, alow = {-107134.0332, 2081.328396, -11.55723697, 0.0544148211, -6.51862274e-005, 4.13315988e-008, -1.071935103e-011}, blow = {-42647.5621, 90.35923546999999}, ahigh = {1536120.418, -7378.93973, 17.25797178, -0.001395645457, 2.639247747e-007, -2.675653563e-011, 1.123582778e-015}, bhigh = {9813.71046, -83.13642553}, R = 121.4181786825755);
          constant IdealGases.Common.DataRecord CH2F2(name = "CH2F2", MM = 0.0520233864, Hf = -8694166.821866099, H0 = 205548.4607976231, Tlimit = 1000, alow = {-181991.75, 3174.42789, -17.14256906, 0.06411353830000001, -7.31359212e-005, 4.426384159999999e-008, -1.103877216e-011}, blow = {-70270.5895, 120.7331926}, ahigh = {1546609.496, -7876.88333, 17.68770469, -0.001581298721, 3.068917255e-007, -3.18340546e-011, 1.363827177e-015}, bhigh = {-9800.13596, -89.0742036}, R = 159.8218142908129);
          constant IdealGases.Common.DataRecord CH2I2(name = "CH2I2", MM = 0.26783552, Hf = 438964.9289235423, H0 = 49472.25819786711, Tlimit = 1000, alow = {27381.33757, -89.7256804, 1.387869408, 0.02733482611, -3.61940877e-005, 2.53918761e-008, -7.145585160000001e-012}, blow = {13387.67276, 22.41870165}, ahigh = {1512892.716, -6442.9124, 16.59026137, -0.00113555152, 2.07375025e-007, -2.033902231e-011, 8.282283700000001e-016}, bhigh = {50582.1903, -71.2523514}, R = 31.0432014394506);
          constant IdealGases.Common.DataRecord CH3(name = "CH3", MM = 0.01503452, Hf = 9754753.726756824, H0 = 0, Tlimit = 1000, alow = {-28761.88806, 509.326866, 0.2002143949, 0.01363605829, -1.433989346e-005, 1.013556725e-008, -3.027331936e-012}, blow = {0, 14082.71825}, ahigh = {10366.34, 2760802.663, -9336.53117, 14.87729606, -0.001439429774, 2.444477951e-007, -2.224555778e-011}, bhigh = {8.39506576e-016, 0}, R = 553.0254374599256);
          constant IdealGases.Common.DataRecord CH3Br(name = "CH3Br", MM = 0.09493852, Hf = -397520.4163705101, H0 = 111784.5001164965, Tlimit = 1000, alow = {-71155.85769999999, 1524.705928, -8.230445209999999, 0.0423997321, -4.94697989e-005, 3.19443334e-008, -8.531237080000001e-012}, blow = {-12517.53591, 70.48127769999999}, ahigh = {2524874.348, -10118.76098, 18.65902163, -0.00179767369, 3.29852011e-007, -3.25091203e-011, 1.330179925e-015}, bhigh = {55405.7639, -97.7866446}, R = 87.57743432275962);
          constant IdealGases.Common.DataRecord CH3CL(name = "CH3CL", MM = 0.05048752, Hf = -1621588.859979654, H0 = 206311.2626645159, Tlimit = 1000, alow = {-98419.71100000001, 1983.700841, -10.84512305, 0.0477798005, -5.51551626e-005, 3.50617614e-008, -9.23610396e-012}, blow = {-19946.89506, 83.99658330999999}, ahigh = {2522463.305, -10301.15447, 18.82725852, -0.001872291294, 3.47337286e-007, -3.45888722e-011, 1.428941079e-015}, bhigh = {51114.1741, -100.6571389}, R = 164.6837079737725);
          constant IdealGases.Common.DataRecord CH3F(name = "CH3F", MM = 0.0340329232, Hf = -6984413.257806781, H0 = 297806.5957025989, Tlimit = 1000, alow = {-202982.1878, 3447.33113, -17.68994275, 0.059452758, -6.46952825e-005, 3.85941804e-008, -9.626541530000001e-012}, blow = {-45779.26220000001, 122.8382176}, ahigh = {2561903.188, -10860.52758, 19.2944692, -0.002070973131, 3.92912453e-007, -3.99450342e-011, 1.681447081e-015}, bhigh = {35635.0851, -106.1158456}, R = 244.3067247305985);
          constant IdealGases.Common.DataRecord CH3I(name = "CH3I", MM = 0.14193899, Hf = 96980.82253509061, H0 = 76200.58449056177, Tlimit = 1000, alow = {-45164.6274, 1086.208429, -5.66317627, 0.0368317171, -4.3213935e-005, 2.833483666e-008, -7.684383690000001e-012}, blow = {-4303.83065, 56.88562690000001}, ahigh = {2511915.982, -9960.289989999999, 18.56907132, -0.001768191331, 3.24220626e-007, -3.19294201e-011, 1.305407821e-015}, bhigh = {60669.29949999999, -95.9077704}, R = 58.57778754097095);
          constant IdealGases.Common.DataRecord CH2OH(name = "CH2OH", MM = 0.03103392, Hf = -573565.9562182283, H0 = 379616.8837194915, Tlimit = 1000, alow = {-156007.6238, 2685.446279, -13.4202242, 0.0575713947, -7.28444999e-005, 4.836648860000001e-008, -1.293492601e-011}, blow = {-15968.2041, 99.630337}, ahigh = {2250349.506, -8173.186060000001, 15.99639179, -0.0008704133719999999, 6.06918395e-008, 4.40834946e-012, -5.7023095e-016}, bhigh = {46453.1343, -78.3515845}, R = 267.9156226477351);
          constant IdealGases.Common.DataRecord CH2OHplus(name = "CH2OHplus", MM = 0.0310333714, Hf = 23084826.67790326, H0 = 327035.0446036295, Tlimit = 1000, alow = {-107708.0841, 2252.082711, -11.88167865, 0.0460231696, -4.87973688e-005, 2.876413471e-008, -7.1002265e-012}, blow = {74844.4777, 90.2792857}, ahigh = {2603333.487, -10099.5381, 17.30843898, -0.000694639032, 3.009715083e-008, 5.22148598e-012, -5.09018379e-016}, bhigh = {146540.2426, -92.2395528}, R = 267.9203587915685);
          constant IdealGases.Common.DataRecord CH3O(name = "CH3O", MM = 0.03103392, Hf = 418896.4848784814, H0 = 364183.7705323724, Tlimit = 1000, alow = {86571.17660000001, -663.1685250000001, 2.257455672, 0.02266283789, -2.970566403e-005, 2.199341353e-008, -6.58804338e-012}, blow = {4174.102129999999, 8.174777900000001}, ahigh = {2101188.243, -8841.968800000001, 18.22645731, -0.001743485034, 3.34043427e-007, -3.43067316e-011, 1.473897771e-015}, bhigh = {53095.82060000001, -94.2250059}, R = 267.9156226477351);
          constant IdealGases.Common.DataRecord CH4(name = "CH4", MM = 0.01604246, Hf = -4650159.63885838, H0 = 624355.7409524474, Tlimit = 1000, alow = {-176685.0998, 2786.18102, -12.0257785, 0.0391761929, -3.61905443e-005, 2.026853043e-008, -4.976705489999999e-012}, blow = {-23313.1436, 89.0432275}, ahigh = {3730042.76, -13835.01485, 20.49107091, -0.001961974759, 4.72731304e-007, -3.72881469e-011, 1.623737207e-015}, bhigh = {75320.6691, -121.9124889}, R = 518.2791167938085);
          constant IdealGases.Common.DataRecord CH3OH(name = "CH3OH", MM = 0.03204186, Hf = -6271171.523750494, H0 = 356885.5553329301, Tlimit = 1000, alow = {-241664.2886, 4032.14719, -20.46415436, 0.0690369807, -7.59893269e-005, 4.59820836e-008, -1.158706744e-011}, blow = {-44332.61169999999, 140.014219}, ahigh = {3411570.76, -13455.00201, 22.61407623, -0.002141029179, 3.73005054e-007, -3.49884639e-011, 1.366073444e-015}, bhigh = {56360.8156, -127.7814279}, R = 259.4878075117987);
          constant IdealGases.Common.DataRecord CH3OOH(name = "CH3OOH", MM = 0.04804126, Hf = -2893346.261109721, H0 = 0, Tlimit = 1000, alow = {-149797.4156, 2656.222273, -13.77060625, 0.0658867383, -7.75180165e-005, 4.9688007e-008, -1.31436764e-011}, blow = {0, -30584.14201}, ahigh = {13918.692, 3060740.61, -12829.59627, 25.41021168, -0.002394481095, 4.44342991e-007, -4.4166595e-011}, bhigh = {1.819673372e-015, 0}, R = 173.069399095694);
          constant IdealGases.Common.DataRecord CI(name = "CI", MM = 0.13891517, Hf = 4104669.15888308, H0 = 68344.68834469267, Tlimit = 1000, alow = {104301.1064, -1715.427168, 12.88874952, -0.01828504834, 2.135468356e-005, -1.286980869e-008, 3.16704766e-012}, blow = {75507.84700000001, -44.9681132}, ahigh = {-240822.9894, 344.738704, 4.9769687, -0.000844481539, 5.061239719999999e-007, -1.047700525e-010, 6.74084314e-015}, bhigh = {64513.4202, 1.090913159}, R = 59.85287279999731);
          constant IdealGases.Common.DataRecord CI2(name = "CI2", MM = 0.26581964, Hf = 1762073.084592245, H0 = 47562.82492896311, Tlimit = 1000, alow = {60282.89810000001, -1043.205435, 10.17465942, -0.00553567171, 5.63031901e-006, -3.108249657e-009, 7.20246956e-013}, blow = {59648.9655, -23.10319208}, ahigh = {-702865.553, 308.2750393, 9.04642705, -0.002946736644, 1.424347233e-006, -2.45318341e-010, 1.425270291e-014}, bhigh = {50211.34209999999, -15.60349377}, R = 31.27862185051489);
          constant IdealGases.Common.DataRecord CI3(name = "CI3", MM = 0.39272411, Hf = 1033763.755935433, H0 = 42858.20649004717, Tlimit = 1000, alow = {85967.84250000001, -1347.703793, 12.72103659, -0.002442876357, 2.511609978e-007, 1.024658583e-009, -4.87269296e-013}, blow = {53107.1651, -32.384929}, ahigh = {-153651.1918, -83.5704437, 10.06203125, -2.466447336e-005, 5.41905475e-009, -6.181417e-013, 2.851262022e-017}, bhigh = {45825.8953, -14.94825345}, R = 21.17127975667193);
          constant IdealGases.Common.DataRecord CI4(name = "CI4", MM = 0.51962858, Hf = 515644.0009516028, H0 = 43022.17172119363, Tlimit = 1000, alow = {89190.4387, -1651.906475, 17.96124325, -0.00851811425, 8.50868225e-006, -4.59909773e-009, 1.039492403e-012}, blow = {36893.9215, -58.0595312}, ahigh = {-144676.277, -52.8036517, 13.03985589, -1.606214814e-005, 3.56755047e-009, -4.10536698e-013, 1.907272766e-017}, bhigh = {28177.46618, -28.10414436}, R = 16.0007981085259);
          constant IdealGases.Common.DataRecord CN(name = "CN", MM = 0.0260174, Hf = 16861160.30041434, H0 = 333319.3939440528, Tlimit = 1000, alow = {3949.14857, -139.1590572, 4.93083532, -0.006304670510000001, 1.256836472e-005, -9.878300500000001e-009, 2.843137221e-012}, blow = {52284.55379999999, -2.763115585}, ahigh = {-2228006.27, 5040.733389999999, -0.2121897722, 0.001354901134, 1.325929798e-007, -6.93700637e-011, 5.49495227e-015}, bhigh = {17844.96132, 32.82563919}, R = 319.5735161853222);
          constant IdealGases.Common.DataRecord CNplus(name = "CNplus", MM = 0.0260168514, Hf = 69143297.79352164, H0 = 333710.7886929008, Tlimit = 1000, alow = {-830290.9570000001, 8775.6875, -29.7744356, 0.0497689706, -1.302225951e-005, -2.058325353e-008, 1.126843895e-011}, blow = {170386.0539, 203.9918818}, ahigh = {-7153463.08, 18572.50421, -10.84534159, 0.00610668143, -1.191208566e-006, 1.184848778e-010, -4.799838730000001e-015}, bhigh = {92426.44959999999, 113.5340573}, R = 319.5802548189978);
          constant IdealGases.Common.DataRecord CNminus(name = "CNminus", MM = 0.0260179486, Hf = 2455424.32964911, H0 = 333273.93075102, Tlimit = 1000, alow = {-46065.4139, 429.417475, 2.32878188, -0.0001235303004, 4.47846277e-006, -4.40315129e-009, 1.349001191e-012}, blow = {4362.07834, 11.42928617}, ahigh = {351796.472, -1630.477359, 5.60987575, -0.000397560597, 8.856147079999999e-008, -9.722872320000001e-012, 4.43420569e-016}, bhigh = {16479.76581, -11.75502699}, R = 319.5667778358206);
          constant IdealGases.Common.DataRecord CNN(name = "CNN", MM = 0.0400241, Hf = 15827565.29191162, H0 = 259285.2806184274, Tlimit = 1000, alow = {-73576.9236, 965.2438659999999, -1.704121157, 0.02037239025, -2.183423906e-005, 1.176082777e-008, -2.551355221e-012}, blow = {70217.2983, 35.2815091}, ahigh = {-181714.8765, -672.986349, 7.85794834, -6.13688072e-005, -1.088178985e-008, 4.45665581e-012, -2.836496278e-016}, bhigh = {77234.8898, -19.66012324}, R = 207.7366386751982);
          constant IdealGases.Common.DataRecord CO(name = "CO", MM = 0.0280101, Hf = -3946262.098314536, H0 = 309570.6191695138, Tlimit = 1000, alow = {14890.45326, -292.2285939, 5.72452717, -0.008176235030000001, 1.456903469e-005, -1.087746302e-008, 3.027941827e-012}, blow = {-13031.31878, -7.85924135}, ahigh = {461919.725, -1944.704863, 5.91671418, -0.0005664282830000001, 1.39881454e-007, -1.787680361e-011, 9.62093557e-016}, bhigh = {-2466.261084, -13.87413108}, R = 296.8383547363272);
          constant IdealGases.Common.DataRecord COplus(name = "COplus", MM = 0.0280095514, Hf = 44548703.62543543, H0 = 309576.6824741077, Tlimit = 1000, alow = {-21787.86658, 128.8857032, 3.76905755, -0.00343173013, 8.19394575e-006, -6.463814690000001e-009, 1.803727574e-012}, blow = {148234.5898, 3.99054707}, ahigh = {231684.7506, -1057.646148, 4.55425778, 0.000449552032, -2.489507047e-007, 5.26756642e-011, -3.28951027e-015}, bhigh = {155505.0724, -3.87346264}, R = 296.8441686645506);
          constant IdealGases.Common.DataRecord COCL(name = "COCL", MM = 0.0634631, Hf = -252115.0085640317, H0 = 182007.3239409988, Tlimit = 1000, alow = {25131.7574, -596.918967, 8.327671349999999, -0.00705613259, 1.313150734e-005, -1.037059653e-008, 3.033665179e-012}, blow = {-705.277032, -15.80716775}, ahigh = {344372.024, -1793.14347, 8.392755899999999, -0.000537476959, 9.113555710000001e-008, -3.111441728e-012, -2.040435218e-016}, bhigh = {6914.470149999999, -19.98919104}, R = 131.0126987178376);
          constant IdealGases.Common.DataRecord COCL2(name = "COCL2", MM = 0.09891610000000001, Hf = -2219052.307966044, H0 = 130197.4299431538, Tlimit = 1000, alow = {93193.2145, -1577.971273, 12.08353907, -0.00480901561, 7.688477319999999e-006, -5.8588412e-009, 1.687786559e-012}, blow = {-20542.52334, -38.3476732}, ahigh = {-25458.81891, -1305.958516, 10.92922584, -0.000360121016, 7.78701765e-008, -8.79245203e-012, 4.02869661e-016}, bhigh = {-22198.4734, -32.3330345}, R = 84.05580082514373);
          constant IdealGases.Common.DataRecord COFCL(name = "COFCL", MM = 0.08246150319999999, Hf = -5208404.471578928, H0 = 144355.821056631, Tlimit = 1000, alow = {72621.739, -1019.738175, 7.29091199, 0.00742069171, -7.902548079999999e-006, 4.209427650000001e-009, -9.3144129e-013}, blow = {-48043.8646, -13.16923671}, ahigh = {-53168.8791, -1581.009678, 11.12696501, -0.00043730008, 9.464190710000001e-008, -1.069291346e-011, 4.901756640000001e-016}, bhigh = {-45996.6723, -35.25879824}, R = 100.8285281901095);
          constant IdealGases.Common.DataRecord COF2(name = "COF2", MM = 0.06600690640000001, Hf = -9695955.088723866, H0 = 168678.2430391253, Tlimit = 1000, alow = {52633.9315, -461.860339, 2.774114516, 0.01831931082, -2.130172554e-005, 1.266924542e-008, -3.101675983e-012}, blow = {-75642.55099999999, 9.467181460000001}, ahigh = {-40738.0685, -1974.009812, 11.40363654, -0.000543711147, 1.175232744e-007, -1.326564192e-011, 6.07677213e-016}, bhigh = {-69077.9541, -40.09695}, R = 125.9636673413314);
          constant IdealGases.Common.DataRecord COHCL(name = "COHCL", MM = 0.06447104000000001, Hf = -2547064.170207274, H0 = 170716.6814743488, Tlimit = 1000, alow = {6332.94687, 81.2036559, 1.374648391, 0.01650970564, -1.692917241e-005, 9.68404683e-009, -2.37446939e-012}, blow = {-21203.5658, 19.3837965}, ahigh = {831895.285, -4416.87084, 12.70661114, -0.0009361408629999999, 1.855335611e-007, -1.958378539e-011, 8.512046829999999e-016}, bhigh = {4330.49644, -51.45071542}, R = 128.9644466724905);
          constant IdealGases.Common.DataRecord COHF(name = "COHF", MM = 0.0480164432, Hf = -7801293.391093992, H0 = 217629.5098842307, Tlimit = 1000, alow = {-45858.285, 1048.202675, -4.77657422, 0.03080719931, -3.4158959e-005, 2.034926806e-008, -5.05419462e-012}, blow = {-50859.8373, 52.3224741}, ahigh = {857885.316, -4791.95912, 12.93975218, -0.001018285299, 2.021278474e-007, -2.136698091e-011, 9.299588910000002e-016}, bhigh = {-18789.33107, -55.2744914}, R = 173.1588482172291);
          constant IdealGases.Common.DataRecord COS(name = "COS", MM = 0.0600751, Hf = -2358714.342547911, H0 = 165486.1498357889, Tlimit = 1000, alow = {85478.76430000001, -1319.464821, 9.735257239999999, -0.00687083096, 1.082331416e-005, -7.70559734e-009, 2.078570344e-012}, blow = {-11916.57685, -29.91988593}, ahigh = {195909.8567, -1756.167688, 8.71043034, -0.000413942496, 1.015243648e-007, -1.159609663e-011, 5.691053860000001e-016}, bhigh = {-8927.09669, -26.36328016}, R = 138.4013010382005);
          constant IdealGases.Common.DataRecord CO2(name = "CO2", MM = 0.0440095, Hf = -8941478.544405185, H0 = 212805.6215135368, Tlimit = 1000, alow = {49436.5054, -626.411601, 5.30172524, 0.002503813816, -2.127308728e-007, -7.68998878e-010, 2.849677801e-013}, blow = {-45281.9846, -7.04827944}, ahigh = {117696.2419, -1788.791477, 8.29152319, -9.22315678e-005, 4.86367688e-009, -1.891053312e-012, 6.330036589999999e-016}, bhigh = {-39083.5059, -26.52669281}, R = 188.9244822140674);
          constant IdealGases.Common.DataRecord CO2plus(name = "CO2plus", MM = 0.0440089514, Hf = 21465814.54335674, H0 = 240089.5423288817, Tlimit = 1000, alow = {-73830.3098, 1086.211742, -2.771112737, 0.02318463595, -2.570240315e-005, 1.450335497e-008, -3.33447042e-012}, blow = {107178.4918, 40.54885210000001}, ahigh = {-169505.1682, -806.646973, 8.00282846, -0.0001577214041, 2.566759314e-008, -2.404195965e-012, 1.6774468e-016}, bhigh = {115438.9478, -21.33567772}, R = 188.9268372797449);
          constant IdealGases.Common.DataRecord COOH(name = "COOH", MM = 0.04501744, Hf = -4731499.614371675, H0 = 240203.5522233161, Tlimit = 1000, alow = {-11283.80671, 377.517943, -0.5992550409999999, 0.02181894272, -2.425918417e-005, 1.451245206e-008, -3.59623338e-012}, blow = {-28410.42565, 29.34561769}, ahigh = {929318.873, -4483.030570000001, 12.42199567, -0.0007463139639999999, 1.332996131e-007, -1.28271055e-011, 5.1379979e-016}, bhigh = {-851.8232680000001, -50.6806551}, R = 184.694465078423);
          constant IdealGases.Common.DataRecord CP(name = "CP", MM = 0.042984461, Hf = 12101159.18401303, H0 = 202750.1287034866, Tlimit = 1000, alow = {-45239.4011, 775.8828140000001, -1.453947907, 0.01397713706, -1.624489706e-005, 9.514083879999999e-009, -2.210281109e-012}, blow = {57926.3467, 33.1164792}, ahigh = {-5449937.15, 16883.45926, -15.956922, 0.01136028761, -2.84654133e-006, 3.37936404e-010, -1.554457397e-014}, bhigh = {-45545.17739999999, 145.1324059}, R = 193.429714054109);
          constant IdealGases.Common.DataRecord CS(name = "CS", MM = 0.0440757, Hf = 6319810.643960278, H0 = 197577.7582658926, Tlimit = 1000, alow = {-49248.4412, 816.69681, -1.542998408, 0.01380324735, -1.574407905e-005, 9.16971493e-009, -2.169700595e-012}, blow = {28651.82876, 33.08541327}, ahigh = {-971957.476, 2339.201284, 1.709390402, 0.001577178949, -4.146335910000001e-007, 4.50475708e-011, -5.94545773e-016}, bhigh = {16810.20727, 18.7404822}, R = 188.6407249346012);
          constant IdealGases.Common.DataRecord CS2(name = "CS2", MM = 0.07614069999999999, Hf = 1532688.824767831, H0 = 140059.4294510032, Tlimit = 1000, alow = {16135.60482, -464.948147, 6.29793879, 0.001888896706, 3.031927747e-007, -1.737645373e-009, 7.79398939e-013}, blow = {14777.61119, -9.303382129999999}, ahigh = {-1390419.724, 3354.9755, 3.019247723, 0.002876437543, -9.076812719999999e-007, 1.374091042e-010, -6.99957557e-015}, bhigh = {-10138.98046, 15.65113703}, R = 109.1987859318341);
          constant IdealGases.Common.DataRecord C2(name = "C2", MM = 0.0240214, Hf = 34571562.10712116, H0 = 423335.9421182778, Tlimit = 1000, alow = {555963.451, -9980.12644, 66.8162037, -0.1743432724, 0.0002448523051, -1.70346758e-007, 4.68452773e-011}, blow = {144586.9634, -344.82297}, ahigh = {-968926.793, 3561.09299, -0.5064138930000001, 0.002945154879, -7.13944119e-007, 8.67065725e-011, -4.07690681e-015}, bhigh = {76817.96829999999, 33.3998524}, R = 346.1277027983382);
          constant IdealGases.Common.DataRecord C2plus(name = "C2plus", MM = 0.0240208514, Hf = 83459803.26076201, H0 = 361565.2024723819, Tlimit = 1000, alow = {-99134.2384, 1347.170609, -3.47675316, 0.01676429424, -1.865908025e-005, 1.091134647e-008, -2.434913818e-012}, blow = {233545.48, 44.0664462}, ahigh = {3836292.81, -6242.062449999999, 2.779245639, 0.006065865859999999, -2.452799858e-006, 3.8829425e-010, -2.190639912e-014}, bhigh = {285744.7553, 0.729738349}, R = 346.1356078327848);
          constant IdealGases.Common.DataRecord C2minus(name = "C2minus", MM = 0.0240219486, Hf = 20013651.34883354, H0 = 361174.0306529504, Tlimit = 1000, alow = {-118192.866, 1438.18971, -3.19613135, 0.01465548163, -1.545537278e-005, 9.061235610000001e-009, -2.135962274e-012}, blow = {49653.1779, 42.2560888}, ahigh = {4478136.25, -11541.45714, 13.10143499, -0.001862700578, 4.00693125e-008, 3.7102136e-011, -3.33726687e-015}, bhigh = {132535.6168, -69.75964399999999}, R = 346.1197981249531);
          constant IdealGases.Common.DataRecord C2CL(name = "C2CL", MM = 0.0594744, Hf = 8980049.752498554, H0 = 181264.9812356241, Tlimit = 1000, alow = {47258.8941, -898.0223609999999, 9.016877149999999, -0.00633378283, 1.08706005e-005, -8.089068920000001e-009, 2.248275223e-012}, blow = {67022.15700000001, -23.54893403}, ahigh = {213736.8408, -1630.519518, 8.62349025, -0.000425351786, 9.04001864e-008, -1.007541495e-011, 4.570764750000001e-016}, bhigh = {71710.9328, -24.13014598}, R = 139.7991740984356);
          constant IdealGases.Common.DataRecord C2CL2(name = "C2CL2", MM = 0.09492680000000001, Hf = 2387102.48317651, H0 = 0, Tlimit = 1000, alow = {38711.6908, -933.6870729999999, 11.19377861, -0.0037462882, 6.77047256e-006, -4.91038835e-009, 1.299068749e-012}, blow = {0, 29481.53198}, ahigh = {14592.612, 296197.8472, -2044.286552, 11.87529315, -0.000511501145, 1.072661556e-007, -1.183411171e-011}, bhigh = {5.32649834e-016, 0}, R = 87.58824694396103);
          constant IdealGases.Common.DataRecord C2CL3(name = "C2CL3", MM = 0.1303804, Hf = 1459357.326714752, H0 = 123870.2903197106, Tlimit = 1000, alow = {46750.1604, -885.1019640000001, 9.03227055, 0.01242122796, -1.554614347e-005, 9.51388712e-009, -2.341473263e-012}, blow = {24958.63399, -17.79070154}, ahigh = {-220402.8219, -1072.926248, 13.78367377, -0.0003093322915, 6.77752895e-008, -7.72730182e-012, 3.56669022e-016}, bhigh = {24310.20015, -43.42161815}, R = 63.77087353620636);
          constant IdealGases.Common.DataRecord C2CL4(name = "C2CL4", MM = 0.1658322, Hf = -145930.6455561706, H0 = 0, Tlimit = 1000, alow = {37462.583, -848.177563, 10.0915611, 0.01845463613, -2.390899444e-005, 1.514828257e-008, -3.84111932e-012}, blow = {0, -1598.288471}, ahigh = {19605.643, -300131.5659, -1132.407598, 16.8307247, -0.000328879249, 7.22090703e-008, -8.24533269e-012}, bhigh = {3.81011278e-016, 0}, R = 50.13786224870682);
          constant IdealGases.Common.DataRecord C2CL6(name = "C2CL6", MM = 0.2367376, Hf = -626009.5565723401, H0 = 0, Tlimit = 1000, alow = {159345.1813, -2606.697136, 20.28011847, 0.01769364822, -3.133105185e-005, 2.389395238e-008, -6.8930383e-012}, blow = {0, -9038.00764}, ahigh = {27129.39, -557738.632, -497.649124, 22.37041989, -0.0001477759098, 3.25754658e-008, -3.72709921e-012}, bhigh = {1.723801704e-016, 0}, R = 35.12104541061496);
          constant IdealGases.Common.DataRecord C2F(name = "C2F", MM = 0.0430198032, Hf = 8225223.16884983, H0 = 240982.8783224188, Tlimit = 1000, alow = {12497.97213, -348.387675, 5.75508435, 0.000852868538, 2.353546435e-006, -2.804737097e-009, 9.083575639999999e-013}, blow = {42815.2531, -6.43720096}, ahigh = {289877.6676, -2016.644658, 8.87504571, -0.000516684113, 1.092094306e-007, -1.212216787e-011, 5.48228768e-016}, bhigh = {52412.8792, -27.7308958}, R = 193.2708051067979);
          constant IdealGases.Common.DataRecord C2FCL(name = "C2FCL", MM = 0.0784728032, Hf = 430294.2882509389, H0 = 176832.0543441476, Tlimit = 1000, alow = {26976.75344, -741.4403179999999, 9.6296043, -0.000386554493, 3.003347997e-006, -2.710191661e-009, 7.73758689e-013}, blow = {5500.62039, -25.15774889}, ahigh = {347145.489, -2348.4611, 12.06875617, -0.000580469823, 1.21273434e-007, -1.334214186e-011, 5.99255661e-016}, bhigh = {15020.61224, -42.52445687}, R = 105.9535490125068);
          constant IdealGases.Common.DataRecord C2FCL3(name = "C2FCL3", MM = 0.1493788032, Hf = -1111268.777389696, H0 = 125398.7419816201, Tlimit = 1000, alow = {12372.82217, -470.396184, 7.58162179, 0.02305727066, -2.792144322e-005, 1.679699826e-008, -4.07757244e-012}, blow = {-20313.33976, -9.274637513}, ahigh = {-282950.5545, -1473.853145, 17.07879074, -0.000426521339, 9.357199850000001e-008, -1.067932796e-011, 4.93326161e-016}, bhigh = {-17384.33314, -61.97054796}, R = 55.66032008482446);
          constant IdealGases.Common.DataRecord C2F2(name = "C2F2", MM = 0.0620182064, Hf = -2332632.567071465, H0 = 213911.684488831, Tlimit = 1000, alow = {17763.13572, -611.328094, 8.6636068, 0.0009156381209999999, 2.363808425e-006, -2.753750054e-009, 8.713935969999999e-013}, blow = {-16496.11017, -21.65150307}, ahigh = {419389.094, -2715.359989, 12.30403413, -0.0006649305540000001, 1.385254301e-007, -1.520813764e-011, 6.81982103e-016}, bhigh = {-4179.221939999999, -46.7038623}, R = 134.0650186878026);
          constant IdealGases.Common.DataRecord C2F2CL2(name = "C2F2CL2", MM = 0.1329242064, Hf = -2541576.204588121, H0 = 134934.2567901161, Tlimit = 1000, alow = {-115043.3214, 2032.113595, -11.19750271, 0.08342451400000001, -0.0001163698616, 7.83309289e-008, -2.070004069e-011}, blow = {-52082.5681, 88.88322147}, ahigh = {-870256.566, 726.472448, 15.30689121, 0.0003045597224, -7.04546804e-008, 8.28592434e-012, -3.89858107e-016}, bhigh = {-52088.8871, -49.98300753}, R = 62.55047312435939);
          constant IdealGases.Common.DataRecord C2F3(name = "C2F3", MM = 0.0810166096, Hf = -2816472.993952588, H0 = 174823.0032079743, Tlimit = 1000, alow = {-26147.75016, 210.1767714, 2.32862508, 0.02368943293, -2.416431591e-005, 1.224916093e-008, -2.488670434e-012}, blow = {-30285.66585, 16.9960879}, ahigh = {-124260.3837, -2137.292446, 14.55051371, -0.0006093891980000001, 1.331427672e-007, -1.515124358e-011, 6.98411206e-016}, bhigh = {-19771.46179, -54.21039870000001}, R = 102.6267581555277);
          constant IdealGases.Common.DataRecord C2F3CL(name = "C2F3CL", MM = 0.1164696096, Hf = -4423471.511318606, H0 = 147061.1008212738, Tlimit = 1000, alow = {-8122.270309999999, -157.4007092, 5.04181029, 0.02621692321, -2.843001445e-005, 1.537643979e-008, -3.36073067e-012}, blow = {-63540.49950000001, 2.792301872}, ahigh = {-216272.4659, -2195.103372, 17.59605428, -0.000628261729, 1.37416661e-007, -1.565004037e-011, 7.218309900000001e-016}, bhigh = {-55151.095, -69.21931823999999}, R = 71.38748063597872);
          constant IdealGases.Common.DataRecord C2F4(name = "C2F4", MM = 0.1000150128, Hf = -6594010.054458545, H0 = 163281.0269459867, Tlimit = 1000, alow = {-9991.530069999999, -129.1088427, 4.50422905, 0.0259202964, -2.63030872e-005, 1.316489777e-008, -2.625017169e-012}, blow = {-80904.47080000001, 3.27424147}, ahigh = {-162991.5758, -2603.903955, 17.88488423, -0.000739703801, 1.61446034e-007, -1.835820392e-011, 8.45764107e-016}, bhigh = {-70063.7166, -74.54165860000001}, R = 83.13223952314488);
          constant IdealGases.Common.DataRecord C2F6(name = "C2F6", MM = 0.1380118192, Hf = -9738296.384980919, H0 = 146886.2603037118, Tlimit = 1000, alow = {-37953.717, 799.82764, -4.67156181, 0.0750145099, -9.812116100000001e-005, 6.31980759e-008, -1.622597311e-011}, blow = {-167521.1878, 50.537592}, ahigh = {-1011551.484, -942.2140710000001, 22.02553906, -0.000131451875, 1.8660453e-008, -3.46711861e-012, 2.488311205e-016}, bhigh = {-165743.4402, -93.02161150000001}, R = 60.24463736653651);
          constant IdealGases.Common.DataRecord C2H(name = "C2H", MM = 0.02502934, Hf = 22621470.72196071, H0 = 417457.3520516323, Tlimit = 1000, alow = {13436.69487, -506.797072, 7.77210741, -0.00651233982, 1.030117855e-005, -5.880147670000001e-009, 1.226901861e-012}, blow = {68922.69989999999, -18.71881626}, ahigh = {3922334.57, -12047.51703, 17.5617292, -0.00365544294, 6.98768543e-007, -6.82516201e-011, 2.719262793e-015}, bhigh = {143326.6627, -95.6163438}, R = 332.1890229626511);
          constant IdealGases.Common.DataRecord C2HCL(name = "C2HCL", MM = 0.06048204, Hf = 3743259.982632861, H0 = 0, Tlimit = 1000, alow = {132978.5007, -2174.542126, 14.97980854, -0.01290343389, 1.602596678e-005, -9.152049509999999e-009, 2.021223607e-012}, blow = {0, 36048.0132}, ahigh = {11787.625, 1152145.326, -4461.192999999999, 12.81366744, -0.000679734676, 1.151382756e-007, -1.046641954e-011}, bhigh = {3.94986052e-016, 0}, R = 137.4700985614903);
          constant IdealGases.Common.DataRecord C2HCL3(name = "C2HCL3", MM = 0.13138744, Hf = -133193.8577994974, H0 = 0, Tlimit = 1000, alow = {39592.3125, -517.875358, 5.15502877, 0.02816922486, -3.62474172e-005, 2.39207427e-008, -6.36240196e-012}, blow = {0, -1534.346259}, ahigh = {16604.624, 604925.466, -4236.992319999999, 18.46273557, -0.000813719506, 1.551211545e-007, -1.584630634e-011}, bhigh = {6.70093249e-016, 0}, R = 63.28209149976589);
          constant IdealGases.Common.DataRecord C2HF(name = "C2HF", MM = 0.0440277432, Hf = 946955.282504691, H0 = 259970.3543287678, Tlimit = 1000, alow = {91611.6911, -1537.245636, 11.33665074, -0.00462472931, 5.87798546e-006, -2.690992345e-009, 3.52471953e-013}, blow = {10859.0282, -40.23994380000001}, ahigh = {1234347.179, -4819.58196, 13.02337752, -0.000749543582, 1.285452942e-007, -1.184559365e-011, 4.53678476e-016}, bhigh = {32368.1171, -56.4079182}, R = 188.8462000477917);
          constant IdealGases.Common.DataRecord C2HFCL2(name = "C2HFCL2", MM = 0.1149337432, Hf = -1467349.755645999, H0 = 141466.6358834818, Tlimit = 1000, alow = {31173.99823, -136.9505457, 1.176399869, 0.040807766, -5.4181378e-005, 3.61360066e-008, -9.625862350000001e-012}, blow = {-21151.42871, 21.4643316}, ahigh = {407124.352, -3808.20452, 18.2114405, -0.000727896606, 1.381532849e-007, -1.405342513e-011, 5.92000972e-016}, bhigh = {-2676.849943, -75.67353566999999}, R = 72.34143575687527);
          constant IdealGases.Common.DataRecord C2HF2CL(name = "C2HF2CL", MM = 0.09847914640000001, Hf = -3388067.54726298, H0 = 154984.6293143743, Tlimit = 1000, alow = {170443.7675, -1667.236861, 4.7887885, 0.0444483001, -7.32219777e-005, 5.62934755e-008, -1.654638029e-011}, blow = {-32918.0976, -5.78859364}, ahigh = {583037.47, -3810.31741, 17.99003171, -0.000588821086, 1.002861284e-007, -9.149788100000001e-012, 3.45998516e-016}, bhigh = {-22058.75409, -75.90948533}, R = 84.42875780247421);
          constant IdealGases.Common.DataRecord C2HF3(name = "C2HF3", MM = 0.0820245496, Hf = -5986012.75343059, H0 = 174673.461419409, Tlimit = 1000, alow = {-14109.69036, 368.762492, -1.144797568, 0.0395630602, -4.60200136e-005, 2.785149385e-008, -6.90376008e-012}, blow = {-62264.1033, 32.8966169}, ahigh = {691741.378, -5260.30164, 19.15276614, -0.001072493938, 2.099276744e-007, -2.195126633e-011, 9.472702399999999e-016}, bhigh = {-32476.4021, -87.5045695}, R = 101.3656526070093);
          constant IdealGases.Common.DataRecord C2H2_vinylidene(name = "C2H2_vinylidene", MM = 0.02603728, Hf = 15930556.80163212, H0 = 417638.4015534649, Tlimit = 1000, alow = {-14660.42239, 278.9475593, 1.276229776, 0.01395015463, -1.475702649e-005, 9.476298110000001e-009, -2.567602217e-012}, blow = {47361.1018, 16.58225704}, ahigh = {1940838.725, -6892.718150000001, 13.39582494, -0.0009368968669999999, 1.470804368e-007, -1.220040365e-011, 4.12239166e-016}, bhigh = {91071.1293, -63.3750293}, R = 319.3295152181795);
          constant IdealGases.Common.DataRecord C2H2CL2(name = "C2H2CL2", MM = 0.09694328000000001, Hf = 35175.20760593204, H0 = 153517.366030941, Tlimit = 1000, alow = {-12037.24514, 462.903119, -1.318184584, 0.039304496, -4.85152902e-005, 3.17841146e-008, -8.476938410000001e-012}, blow = {-3251.80686, 34.89218118}, ahigh = {1561121.185, -7358.096350000001, 20.11869185, -0.001310978834, 2.411885716e-007, -2.38416674e-011, 9.78541575e-016}, bhigh = {41222.009, -95.50252712000001}, R = 85.76635739991467);
          constant IdealGases.Common.DataRecord C2H2FCL(name = "C2H2FCL", MM = 0.08048868319999999, Hf = -2050996.406411579, H0 = 167344.1838591292, Tlimit = 1000, alow = {226015.4427, -2557.428772, 9.2362412, 0.02879408894, -5.16397547e-005, 4.25219103e-008, -1.311612713e-011}, blow = {-8180.627839999999, -32.34787581}, ahigh = {1527690.968, -6689.712229999999, 19.5055383, -0.001040920895, 1.780587502e-007, -1.633098515e-011, 6.213465560000001e-016}, bhigh = {17052.66451, -91.94078300999999}, R = 103.2998884991077);
          constant IdealGases.Common.DataRecord C2H2F2(name = "C2H2F2", MM = 0.06403408640000001, Hf = -5253452.011458697, H0 = 194891.2477964236, Tlimit = 1000, alow = {57029.3581, -676.28604, 4.11741171, 0.01822618762, -6.86133154e-006, -3.29575674e-009, 2.180965597e-012}, blow = {-38386.5078, 1.487528685}, ahigh = {-822719.275, -2915.718833, 18.30163995, -0.001039630129, 2.546725128e-007, -3.157578705e-011, 1.55154096e-015}, bhigh = {-31100.28589, -84.00834700000002}, R = 129.8444698353657);
          constant IdealGases.Common.DataRecord CH2CO_ketene(name = "CH2CO_ketene", MM = 0.04203668, Hf = -1179353.245784396, H0 = 0, Tlimit = 1000, alow = {35495.9809, -406.306283, 3.71892192, 0.01583501817, -1.726195691e-005, 1.157376959e-008, -3.30584263e-012}, blow = {0, -5209.99258}, ahigh = {11795.805, 2013564.915, -8200.88746, 17.59694074, -0.001464544521, 2.695886969e-007, -2.66567484e-011}, bhigh = {1.094204522e-015, 0}, R = 197.7908816776206);
          constant IdealGases.Common.DataRecord O_CH_2O(name = "O_CH_2O", MM = 0.05803608, Hf = -3652900.058032865, H0 = 0, Tlimit = 1000, alow = {-229245.9698, 3724.09805, -18.93769993, 0.0751174414, -8.083855420000001e-005, 4.35823319e-008, -9.36453933e-012}, blow = {0, -44544.8601}, ahigh = {13682.443, 267806.3593, -4436.61748, 17.81696797, -0.000709717378, 1.272621878e-007, -1.237226678e-011}, bhigh = {5.02505752e-016, 0}, R = 143.2638455250596);
          constant IdealGases.Common.DataRecord HO_CO_2OH(name = "HO_CO_2OH", MM = 0.09003488, Hf = -8127961.074641295, H0 = 0, Tlimit = 1000, alow = {30725.13274, -391.617469, 3.17838864, 0.0374128975, -4.00975396e-005, 2.288662646e-008, -5.386771549999999e-012}, blow = {0, -87979.4255}, ahigh = {17321.662, 1805560.696, -9240.33315, 26.25742604, -0.001418458557, 2.526840574e-007, -2.521005198e-011}, bhigh = {1.040036111e-015, 0}, R = 92.34723253921148);
          constant IdealGases.Common.DataRecord C2H3_vinyl(name = "C2H3_vinyl", MM = 0.02704522, Hf = 11080953.19616553, H0 = 389044.3856622353, Tlimit = 1000, alow = {-33478.9687, 1064.104103, -6.40385706, 0.0393451548, -4.76004609e-005, 3.17007135e-008, -8.633406430000001e-012}, blow = {30391.22649, 58.0922618}, ahigh = {2718080.093, -10309.56829, 18.36579807, -0.001580131153, 2.680594939e-007, -2.439003999e-011, 9.20909639e-016}, bhigh = {97650.55589999999, -97.6008686}, R = 307.4285215649937);
          constant IdealGases.Common.DataRecord CH2BrminusCOOH(name = "CH2BrminusCOOH", MM = 0.13894802, Hf = -2760024.935943672, H0 = 0, Tlimit = 1000, alow = {-78324.84789999999, 1648.865353, -9.34953363, 0.06879635440000001, -8.374880770000001e-005, 5.40603842e-008, -1.423159923e-011}, blow = {0, -55411.8762}, ahigh = {16862.437, 2486349.85, -11402.56346, 27.56020307, -0.00184318701, 3.26539274e-007, -3.122663159e-011}, bhigh = {1.223452495e-015, 0}, R = 59.83872242296076);
          constant IdealGases.Common.DataRecord C2H3CL(name = "C2H3CL", MM = 0.06249792, Hf = 352011.7149498736, H0 = 0, Tlimit = 1000, alow = {-16888.95457, 854.510455, -6.51496755, 0.0494468193, -6.11775667e-005, 4.067293850000001e-008, -1.101392611e-011}, blow = {0, -2069.321321}, ahigh = {11819.647, 2456176.938, -10474.52327, 21.78736181, -0.001816891849, 3.29636995e-007, -3.21439569e-011}, bhigh = {1.302254061e-015, 0}, R = 133.0359794373957);
          constant IdealGases.Common.DataRecord CH2CLminusCOOH(name = "CH2CLminusCOOH", MM = 0.09449671999999999, Hf = -4525024.783929008, H0 = 0, Tlimit = 1000, alow = {-112250.6518, 2168.288658, -12.27791286, 0.0751438825, -9.083158770000001e-005, 5.81236419e-008, -1.518247503e-011}, blow = {0, -63143.1455}, ahigh = {16513.941, 2472883.176, -11522.75963, 27.66275606, -0.001886430963, 3.36177605e-007, -3.23127225e-011}, bhigh = {1.273275883e-015, 0}, R = 87.98688462414359);
          constant IdealGases.Common.DataRecord C2H3F(name = "C2H3F", MM = 0.0460436232, Hf = -3042766.625715937, H0 = 246199.7386860728, Tlimit = 1000, alow = {-45791.3496, 1411.541724, -10.21500877, 0.0577581657, -7.08067749e-005, 4.64085076e-008, -1.240455865e-011}, blow = {-24027.88827, 78.6082839}, ahigh = {2478832.34, -10759.32307, 21.94796688, -0.00186911411, 3.39528067e-007, -3.31538293e-011, 1.345081799e-015}, bhigh = {45634.5555, -118.0410277}, R = 180.5781435549581);
          constant IdealGases.Common.DataRecord CH3CN(name = "CH3CN", MM = 0.04105192000000001, Hf = 1618194.715375066, H0 = 294594.9422097675, Tlimit = 1000, alow = {-99659.88380000001, 1739.278534, -7.89842082, 0.0429489432, -4.49997388e-005, 2.717105086e-008, -7.026117590000001e-012}, blow = {-1461.161333, 68.52508274}, ahigh = {2923231.393, -12337.92258, 23.24477222, -0.002411565845, 4.622157170000001e-007, -4.74060124e-011, 2.010639467e-015}, bhigh = {80585.65550000001, -129.2249102}, R = 202.5355208721054);
          constant IdealGases.Common.DataRecord CH3CO_acetyl(name = "CH3CO_acetyl", MM = 0.04304462, Hf = -232317.0700542832, H0 = 302859.3817299352, Tlimit = 1000, alow = {-71938.94130000001, 1464.465167, -6.63227613, 0.04108468379999999, -4.22625664e-005, 2.485766819e-008, -6.29255848e-012}, blow = {-9309.37081, 64.22897619999999}, ahigh = {2485388.15, -11207.14204, 22.77525438, -0.00231426055, 4.53618917e-007, -4.74263555e-011, 2.044663903e-015}, bhigh = {63800.8841, -121.5350925}, R = 193.1593774088376);
          constant IdealGases.Common.DataRecord C2H4(name = "C2H4", MM = 0.02805316, Hf = 1871446.924339362, H0 = 374955.5843263291, Tlimit = 1000, alow = {-116360.5836, 2554.85151, -16.09746428, 0.0662577932, -7.885081859999999e-005, 5.12522482e-008, -1.370340031e-011}, blow = {-6176.19107, 109.3338343}, ahigh = {3408763.67, -13748.47903, 23.65898074, -0.002423804419, 4.43139566e-007, -4.35268339e-011, 1.775410633e-015}, bhigh = {88204.2938, -137.1278108}, R = 296.3827247982046);
          constant IdealGases.Common.DataRecord C2H4O_ethylen_o(name = "C2H4O_ethylen_o", MM = 0.04405256, Hf = -1194816.373895183, H0 = 245856.6539606325, Tlimit = 1000, alow = {-172823.3345, 3816.6788, -26.29851977, 0.1014103162, -0.0001240578373, 8.03404035e-008, -2.120942544e-011}, blow = {-24375.19333, 165.4885056}, ahigh = {3151809.957, -14236.46316, 27.08080476, -0.002606238456, 4.853891929999999e-007, -4.85214476e-011, 2.011778721e-015}, bhigh = {76625.61440000001, -156.3952401}, R = 188.7398144398419);
          constant IdealGases.Common.DataRecord CH3CHO_ethanal(name = "CH3CHO_ethanal", MM = 0.04405256, Hf = -3772538.98524853, H0 = 292757.4697134514, Tlimit = 1000, alow = {-137390.4369, 2559.937679, -13.40470172, 0.05922128619999999, -6.24000605e-005, 3.70332441e-008, -9.34269741e-012}, blow = {-33187.3131, 100.7417652}, ahigh = {3321176.59, -14497.19957, 27.08421279, -0.002879320054, 5.55630992e-007, -5.73267488e-011, 2.443965239e-015}, bhigh = {65077.5564, -153.6236027}, R = 188.7398144398419);
          constant IdealGases.Common.DataRecord CH3COOH(name = "CH3COOH", MM = 0.06005196, Hf = -7197917.270310578, H0 = 226427.097466927, Tlimit = 1000, alow = {-32191.9198, 1196.329795, -8.70582402, 0.0569625759, -5.75788716e-005, 3.35211522e-008, -8.614438229999999e-012}, blow = {-58401.12869999999, 72.82413919999999}, ahigh = {2103514.223, -14678.22192, 33.8280283, -0.00569485868, 1.343221353e-006, -1.606041158e-010, 7.652794250000001e-015}, bhigh = {29242.28407, -193.527885}, R = 138.4546316223484);
          constant IdealGases.Common.DataRecord OHCH2COOH(name = "OHCH2COOH", MM = 0.07605136, Hf = -7665872.115896415, H0 = 223625.7050498505, Tlimit = 1000, alow = {-313897.858, 5693.06877, -36.8516029, 0.1563502811, -0.0002039487993, 1.340232412e-007, -3.50913026e-011}, blow = {-98016.40089999999, 226.9492355}, ahigh = {1946628.253, -9804.020200000001, 28.44111243, -0.000787640475, 6.416275950000001e-008, 1.069321262e-012, -3.23717828e-016}, bhigh = {-16946.5047, -147.4109363}, R = 109.3270652885103);
          constant IdealGases.Common.DataRecord C2H5(name = "C2H5", MM = 0.0290611, Hf = 4083060.861426443, H0 = 419296.7919314823, Tlimit = 1000, alow = {-141131.2551, 2714.285088, -15.34977725, 0.06451672580000001, -7.259143960000001e-005, 4.59911601e-008, -1.218367535e-011}, blow = {598.141884, 109.096652}, ahigh = {4169220.4, -16629.82142, 27.95442134, -0.003051715761, 5.685160040000001e-007, -5.6828636e-011, 2.355648561e-015}, bhigh = {113701.0087, -163.9357995}, R = 286.1031413126138);
          constant IdealGases.Common.DataRecord C2H5Br(name = "C2H5Br", MM = 0.1089651, Hf = -583673.1210268242, H0 = 124526.1097360531, Tlimit = 1000, alow = {-137417.2662, 2861.429418, -18.24108956, 0.08566230600000001, -0.0001047174473, 6.906074570000001e-008, -1.862276022e-011}, blow = {-21984.80205, 125.8435579}, ahigh = {2378649.403, -12670.33647, 27.77558646, -0.002010898783, 4.43608341e-007, -5.34819809e-011, 2.63983585e-015}, bhigh = {64140.6205, -152.8277085}, R = 76.30399091085127);
          constant IdealGases.Common.DataRecord C2H6(name = "C2H6", MM = 0.03006904, Hf = -2788633.890539904, H0 = 395476.3437741943, Tlimit = 1000, alow = {-186204.4161, 3406.19186, -19.51705092, 0.0756583559, -8.20417322e-005, 5.0611358e-008, -1.319281992e-011}, blow = {-27029.3289, 129.8140496}, ahigh = {5025782.13, -20330.22397, 33.2255293, -0.00383670341, 7.23840586e-007, -7.3191825e-011, 3.065468699e-015}, bhigh = {111596.395, -203.9410584}, R = 276.5127187299628);
          constant IdealGases.Common.DataRecord CH3N2CH3(name = "CH3N2CH3", MM = 0.05808244, Hf = 2560143.134482642, H0 = 284509.3801155737, Tlimit = 1000, alow = {-373849.232, 5880.45313, -29.86398524, 0.1087380861, -0.0001167950177, 6.916894889999999e-008, -1.719950055e-011}, blow = {-11899.84084, 194.8246321}, ahigh = {4993357.09, -21609.96161, 39.6444992, -0.00419645011, 8.023361980000001e-007, -8.212260020000001e-011, 3.47723744e-015}, bhigh = {144996.261, -237.2109745}, R = 143.1494957856454);
          constant IdealGases.Common.DataRecord C2H5OH(name = "C2H5OH", MM = 0.04606844, Hf = -5100020.751733725, H0 = 315659.1801241805, Tlimit = 1000, alow = {-234279.1392, 4479.18055, -27.44817302, 0.1088679162, -0.0001305309334, 8.437346399999999e-008, -2.234559017e-011}, blow = {-50222.29, 176.4829211}, ahigh = {4694817.65, -19297.98213, 34.4758404, -0.00323616598, 5.78494772e-007, -5.56460027e-011, 2.2262264e-015}, bhigh = {86016.22709999999, -203.4801732}, R = 180.4808671619877);
          constant IdealGases.Common.DataRecord CH3OCH3(name = "CH3OCH3", MM = 0.04606844, Hf = -3996445.288792067, H0 = 311588.1284454173, Tlimit = 1000, alow = {-269310.3242, 4300.709709999999, -21.52788028, 0.08131833390000001, -8.29567132e-005, 4.80191151e-008, -1.188699808e-011}, blow = {-44102.3709, 146.7666934}, ahigh = {4933577.19, -20830.94065, 36.2905061, -0.004108351640000001, 7.90322031e-007, -8.13143563e-011, 3.45816611e-015}, bhigh = {101330.1012, -218.5447466}, R = 180.4808671619877);
          constant IdealGases.Common.DataRecord CH3O2CH3(name = "CH3O2CH3", MM = 0.06206784, Hf = -2021981.109701901, H0 = 276365.9247687692, Tlimit = 1000, alow = {-228578.4757, 3820.14257, -19.76647823, 0.0884074386, -9.641284560000001e-005, 5.90720083e-008, -1.526491225e-011}, blow = {-34920.1696, 138.6769151}, ahigh = {5316368.47, -22212.67874, 40.3433509, -0.00461274809, 8.792987200000001e-007, -9.068221189999999e-011, 3.865664890000001e-015}, bhigh = {116159.6028, -239.5296055}, R = 133.9578113238676);
          constant IdealGases.Common.DataRecord CCN(name = "CCN", MM = 0.0380281, Hf = 21157945.62441983, H0 = 290272.8245692001, Tlimit = 1000, alow = {-16962.81385, 98.3789163, 3.81266294, 0.00534689423, -2.473598508e-006, -3.73056422e-010, 4.48175686e-013}, blow = {94800.72570000001, 5.553165572}, ahigh = {79486.74890000001, -1344.786906, 8.309986459999999, -0.0002220105361, 1.753683113e-008, 2.545998719e-012, -2.645649117e-016}, bhigh = {102318.7495, -22.5979394}, R = 218.6402160507625);
          constant IdealGases.Common.DataRecord CNC(name = "CNC", MM = 0.0380281, Hf = 18010748.86728498, H0 = 298637.3497492644, Tlimit = 1000, alow = {-70751.9271, 1007.523898, -1.576789967, 0.02052532634, -2.278935009e-005, 1.283362343e-008, -2.933174091e-012}, blow = {76133.2485, 34.87094132}, ahigh = {-90313.13559999999, -831.32365, 8.11473595, -0.0002447691991, 5.39750877e-008, -6.18398486e-012, 2.865172411e-016}, bhigh = {84518.33600000001, -21.02937255}, R = 218.6402160507625);
          constant IdealGases.Common.DataRecord OCCN(name = "OCCN", MM = 0.05402754, Hf = 3886906.566539954, H0 = 251618.9150940428, Tlimit = 1000, alow = {24288.01276, -552.586462, 8.58783817, -0.00337938777, 1.119841826e-005, -1.008408077e-008, 3.086448824e-012}, blow = {25996.20072, -16.59592359}, ahigh = {935913.1680000001, -4441.082289999999, 13.68959297, -0.001647530917, 3.81987344e-007, -3.945161e-011, 1.509598839e-015}, bhigh = {49512.2278, -54.1708968}, R = 153.8932181624409);
          constant IdealGases.Common.DataRecord C2N2(name = "C2N2", MM = 0.0520348, Hf = 5940255.367561709, H0 = 244358.6407558019, Tlimit = 1000, alow = {108240.4484, -1928.137871, 15.53891898, -0.01821159329, 2.77884084e-005, -1.899434373e-008, 4.94967772e-012}, blow = {44490.9759, -60.90964741}, ahigh = {793442.372, -3997.37627, 13.1449743, -0.0008747782000000001, 2.059156733e-007, -2.200469389e-011, 9.97448577e-016}, bhigh = {58636.323, -54.73201251}, R = 159.7867580926611);
          constant IdealGases.Common.DataRecord C2O(name = "C2O", MM = 0.0400208, Hf = 7272185.113740855, H0 = 262006.7065126135, Tlimit = 1000, alow = {-3959.92942, -111.7516348, 4.59396006, 0.00371060202, -7.01476018e-007, -1.371129839e-009, 7.123853999999999e-013}, blow = {34101.0974, 0.4622805600000001}, ahigh = {-634805.659, 1184.133091, 4.87917334, 0.001757538773, -3.95755227e-007, 3.98917994e-011, -1.546043135e-015}, bhigh = {24898.03938, 0.980904059}, R = 207.7537680406189);
          constant IdealGases.Common.DataRecord C3(name = "C3", MM = 0.0360321, Hf = 23311121.08370037, H0 = 336065.5082551392, Tlimit = 1000, alow = {-43546.1448, 666.018322, 1.451033157, 0.00743451312, -3.81015299e-006, -2.336961396e-011, 4.40705453e-013}, blow = {96351.70199999999, 20.25173297}, ahigh = {4508098.93, -14610.33761, 22.81974644, -0.008544340610000001, 2.146069341e-006, -2.103867761e-010, 6.351589060000001e-015}, bhigh = {191197.6065, -127.1869723}, R = 230.7518018655588);
          constant IdealGases.Common.DataRecord C3H3_1_propynl(name = "C3H3_1_propynl", MM = 0.03905592, Hf = 11521940.84789195, H0 = 317493.4811419114, Tlimit = 1000, alow = {-65058.5935, 1350.858921, -5.82543393, 0.0375661048, -3.73490334e-005, 2.117676603e-008, -5.139113250000001e-012}, blow = {46565.1053, 57.8147755}, ahigh = {4550654.87, -16405.74172, 27.12605991, -0.00447460038, 1.037712415e-006, -1.250211369e-010, 6.02658205e-015}, bhigh = {153408.7662, -156.5931809}, R = 212.8863434787863);
          constant IdealGases.Common.DataRecord C3H3_2_propynl(name = "C3H3_2_propynl", MM = 0.03905592, Hf = 8495511.051845662, H0 = 338745.0609280232, Tlimit = 1000, alow = {61885.78320000001, -890.957867, 6.34755882, 0.01633173115, -1.949975695e-005, 1.417349778e-008, -4.19986632e-012}, blow = {42717.8583, -12.31400729}, ahigh = {2989723.833, -11189.54446, 22.22225052, -0.002068106902, 4.12188364e-007, -4.43898059e-011, 1.970824701e-015}, bhigh = {106187.8289, -118.6744583}, R = 212.8863434787863);
          constant IdealGases.Common.DataRecord C3H4_allene(name = "C3H4_allene", MM = 0.04006386, Hf = 4765392.051589637, H0 = 314611.9470265721, Tlimit = 1000, alow = {-16451.55745, 962.945781, -7.53232668, 0.05518219110000001, -6.73358512e-005, 4.53270905e-008, -1.251837614e-011}, blow = {17724.94269, 61.5196976}, ahigh = {3479355.1, -14304.12453, 27.02534756, -0.002557412369, 4.70664675e-007, -4.65168807e-011, 1.908219044e-015}, bhigh = {107231.2354, -154.8846158}, R = 207.5304775925235);
          constant IdealGases.Common.DataRecord C3H4_propyne(name = "C3H4_propyne", MM = 0.04006386, Hf = 4615131.941854829, H0 = 325243.6485151456, Tlimit = 1000, alow = {-35638.844, 832.81391, -4.07375944, 0.04113929609999999, -4.47044495e-005, 2.847458197e-008, -7.69529824e-012}, blow = {17102.06236, 45.1672095}, ahigh = {3710441.42, -14891.45507, 27.32397127, -0.00264526477, 4.858300350000001e-007, -4.79412848e-011, 1.964338121e-015}, bhigh = {110489.8462, -156.7992462}, R = 207.5304775925235);
          constant IdealGases.Common.DataRecord C3H4_cyclo(name = "C3H4_cyclo", MM = 0.04006386, Hf = 6916457.875002559, H0 = 283888.3472536096, Tlimit = 1000, alow = {-19695.20627, 1505.379338, -14.18206573, 0.07642632960000001, -9.765583660000001e-005, 6.61200382e-008, -1.811251145e-011}, blow = {26256.31782, 96.0463189}, ahigh = {3168399.58, -13710.44699, 26.64303646, -0.00242040805, 4.42799413e-007, -4.3518202e-011, 1.775948955e-015}, bhigh = {113368.3702, -152.2619086}, R = 207.5304775925235);
          constant IdealGases.Common.DataRecord C3H5_allyl(name = "C3H5_allyl", MM = 0.04107180000000001, Hf = 3983131.97863254, H0 = 310157.4072721429, Tlimit = 1000, alow = {-43159.9614, 1441.600907, -11.97014426, 0.0731979646, -9.066357849999999e-005, 6.077059450000001e-008, -1.658826363e-011}, blow = {12321.5746, 85.63173239999999}, ahigh = {4094570.59, -16766.76186, 31.23006342, -0.002885449982, 5.21134354e-007, -5.05828422e-011, 2.039932554e-015}, bhigh = {118572.0481, -182.3070197}, R = 202.437487521852);
          constant IdealGases.Common.DataRecord C3H6_propylene(name = "C3H6_propylene", MM = 0.04207974, Hf = 475288.1077687267, H0 = 322020.9535515191, Tlimit = 1000, alow = {-191246.2174, 3542.07424, -21.14878626, 0.0890148479, -0.0001001429154, 6.267959389999999e-008, -1.637870781e-011}, blow = {-15299.61824, 140.7641382}, ahigh = {5017620.34, -20860.84035, 36.4415634, -0.00388119117, 7.27867719e-007, -7.321204500000001e-011, 3.052176369e-015}, bhigh = {126124.5355, -219.5715757}, R = 197.588483198803);
          constant IdealGases.Common.DataRecord C3H6_cyclo(name = "C3H6_cyclo", MM = 0.04207974, Hf = 1266642.807203657, H0 = 271151.4139583562, Tlimit = 1000, alow = {-156578.777, 4111.12987, -32.3344746, 0.1306337881, -0.0001645563833, 1.095708326e-007, -2.956394783e-011}, blow = {-12452.71686, 193.1559109}, ahigh = {4785000.67, -20421.18175, 36.3149578, -0.00356131944, 6.47624124e-007, -6.328430100000001e-011, 2.568705857e-015}, bhigh = {126827.4126, -222.3729099}, R = 197.588483198803);
          constant IdealGases.Common.DataRecord C3H6O_propylox(name = "C3H6O_propylox", MM = 0.05807914, Hf = -1613660.25736607, H0 = 248031.4446804825, Tlimit = 1000, alow = {-229280.8804, 4495.75054, -29.41117945, 0.1213113827, -0.0001440060464, 9.202051750000001e-008, -2.416278343e-011}, blow = {-33176.9721, 184.6878218}, ahigh = {4789729.989999999, -21068.95971, 39.5464773, -0.00391092998, 7.32553151e-007, -7.35708597e-011, 3.0606185e-015}, bhigh = {112034.454, -237.2004192}, R = 143.1576294001599);
          constant IdealGases.Common.DataRecord C3H6O_acetone(name = "C3H6O_acetone", MM = 0.05807914, Hf = -3738857.014756073, H0 = 278812.5478442002, Tlimit = 1000, alow = {-227780.2525, 4215.28001, -24.15785316, 0.0990748332, -0.0001084940903, 6.583355959999999e-008, -1.676046146e-011}, blow = {-47262.4394, 160.7926432}, ahigh = {5001601.92, -21701.55542, 39.6449399, -0.00417994505, 7.96242953e-007, -8.122558050000001e-011, 3.42878096e-015}, bhigh = {101514.5028, -236.8533477}, R = 143.1576294001599);
          constant IdealGases.Common.DataRecord C3H6O_propanal(name = "C3H6O_propanal", MM = 0.05807914, Hf = -3202526.759177219, H0 = 298339.0594282216, Tlimit = 1000, alow = {-265578.1702, 4250.64045, -21.09249262, 0.09194256120000001, -0.0001004653044, 6.13331482e-008, -1.576298535e-011}, blow = {-44503.7105, 145.6678605}, ahigh = {4830933.489999999, -20754.51152, 39.2485518, -0.004095514, 7.88169216e-007, -8.107684080000001e-011, 3.43710543e-015}, bhigh = {99449.37879999999, -231.6690627}, R = 143.1576294001599);
          constant IdealGases.Common.DataRecord C3H7_n_propyl(name = "C3H7_n_propyl", MM = 0.04308768, Hf = 2332453.267384088, H0 = 344879.5108021598, Tlimit = 1000, alow = {-189533.7073, 3949.51726, -26.06216089, 0.1121920441, -0.0001365292213, 9.02366272e-008, -2.44105699e-011}, blow = {-7227.87744, 167.3705556}, ahigh = {5646512.94, -22910.87136, 39.8727518, -0.004106232870000001, 7.56255777e-007, -7.47826302e-011, 3.068983677e-015}, bhigh = {148300.6853, -240.378119}, R = 192.9663421191394);
          constant IdealGases.Common.DataRecord C3H7_i_propyl(name = "C3H7_i_propyl", MM = 0.04308768, Hf = 2165352.137780452, H0 = 343652.9188853984, Tlimit = 1000, alow = {-295206.3445, 5294.4323, -31.05287013, 0.1143871563, -0.0001291752393, 8.05784376e-008, -2.093908432e-011}, blow = {-14768.15514, 198.808236}, ahigh = {5807002.520000001, -24112.19997, 40.852884, -0.00451785133, 8.499427170000001e-007, -8.573514339999999e-011, 3.58338396e-015}, bhigh = {154650.405, -248.7098372}, R = 192.9663421191394);
          constant IdealGases.Common.DataRecord C3H8(name = "C3H8", MM = 0.04409562, Hf = -2373931.923397381, H0 = 334301.1845620949, Tlimit = 1000, alow = {-243314.4337, 4656.27081, -29.39466091, 0.1188952745, -0.0001376308269, 8.814823909999999e-008, -2.342987994e-011}, blow = {-35403.3527, 184.1749277}, ahigh = {6420731.680000001, -26597.91134, 45.3435684, -0.00502066392, 9.471216939999999e-007, -9.57540523e-011, 4.00967288e-015}, bhigh = {145558.2459, -281.8374734}, R = 188.5555073270316);
          constant IdealGases.Common.DataRecord C3H8O_1propanol(name = "C3H8O_1propanol", MM = 0.06009502, Hf = -4246608.121604752, H0 = 291517.6498818038, Tlimit = 1000, alow = {-261697.3337, 5192.37666, -32.9648116, 0.1354568128, -0.0001593156164, 1.01949816e-007, -2.688552974e-011}, blow = {-56128.5435, 208.5024431}, ahigh = {6308672.12, -26422.10376, 47.1511259, -0.004642511930000001, 8.59346536e-007, -8.68209182e-011, 3.64222401e-015}, bhigh = {125500.3155, -285.9463804}, R = 138.3554244594644);
          constant IdealGases.Common.DataRecord C3H8O_2propanol(name = "C3H8O_2propanol", MM = 0.06009502, Hf = -4537813.615837053, H0 = 287291.2763819697, Tlimit = 1000, alow = {-338651.024, 6106.048000000001, -37.9141804, 0.1530494531, -0.0001864354461, 1.213257738e-007, -3.22043349e-011}, blow = {-62799.5935, 233.432261}, ahigh = {6001074.75, -25058.7683, 46.22096120000001, -0.00427246631, 7.69367877e-007, -7.45058484e-011, 2.998959935e-015}, bhigh = {114851.8732, -279.6132222}, R = 138.3554244594644);
          constant IdealGases.Common.DataRecord CNCOCN(name = "CNCOCN", MM = 0.08004498, Hf = 3092011.516524834, H0 = 214226.5261356802, Tlimit = 1000, alow = {113105.2075, -1978.834961, 16.26886206, -0.008330803440000001, 1.884127045e-005, -1.530681289e-008, 4.4181253e-012}, blow = {36802.6174, -59.6332728}, ahigh = {700052.2440000001, -5086.002009999999, 19.4675349, -0.001302852727, 2.75360075e-007, -3.056290852e-011, 1.382140838e-015}, bhigh = {55393.3888, -86.2755086}, R = 103.8724976881748);
          constant IdealGases.Common.DataRecord C3O2(name = "C3O2", MM = 0.06803090000000001, Hf = -1376403.957613379, H0 = 221737.2252902725, Tlimit = 1000, alow = {157987.3382, -2529.493506, 18.01761578, -0.01786032042, 2.978671986e-005, -2.182900022e-008, 6.013797219999999e-012}, blow = {-1121.054014, -72.77721170000001}, ahigh = {696869.9889999999, -4624.73319, 16.63905725, -0.001175486554, 2.478106444e-007, -2.745165984e-011, 1.239566766e-015}, bhigh = {12525.80069, -72.75968780000001}, R = 122.2161106203211);
          constant IdealGases.Common.DataRecord C4(name = "C4", MM = 0.0480428, Hf = 21520472.20395147, H0 = 273042.8284779405, Tlimit = 1000, alow = {39037.805, -894.8280779999999, 10.50952925, -0.00655289446, 1.243940464e-005, -8.645341370000001e-009, 2.263638846e-012}, blow = {126642.5869, -30.77594475}, ahigh = {920068.513, -1530.3118, 6.0500692, 0.00525274367, -1.779154772e-006, 2.589873632e-010, -1.385553481e-014}, bhigh = {133438.9611, -7.26114882}, R = 173.0638513991691);
          constant IdealGases.Common.DataRecord C4H2_butadiyne(name = "C4H2_butadiyne", MM = 0.05005868, Hf = 8989449.98150171, H0 = 287539.743357196, Tlimit = 1000, alow = {246754.2569, -3897.85564, 23.66080456, -0.02208077805, 2.78110114e-005, -1.57734001e-008, 3.42316546e-012}, blow = {70869.0782, -110.917356}, ahigh = {2328179.913, -8925.186090000001, 21.14326883, -0.001368871276, 2.327503159e-007, -2.124517624e-011, 8.053313019999999e-016}, bhigh = {105778.8416, -108.8313574}, R = 166.0945114813255);
          constant IdealGases.Common.DataRecord C4H4_1_3minuscyclo(name = "C4H4_1_3minuscyclo", MM = 0.05207456, Hf = 7393245.377397331, H0 = 232427.0046640816, Tlimit = 1000, alow = {-27784.28049, 1768.176915, -17.57895171, 0.09383512919999999, -0.0001195524281, 7.97808619e-008, -2.1527511e-011}, blow = {38116.2712, 112.8478124}, ahigh = {2991498.949, -14167.81502, 30.01978876, -0.002579200423, 4.78999045e-007, -4.77532971e-011, 1.974923662e-015}, bhigh = {127466.692, -172.7532057}, R = 159.6647576090898);
          constant IdealGases.Common.DataRecord C4H6_butadiene(name = "C4H6_butadiene", MM = 0.05409044, Hf = 2033631.081573749, H0 = 279716.7114928257, Tlimit = 1000, alow = {-91811.30530000001, 3312.57053, -29.85828611, 0.1479201147, -0.0002056618326, 1.466496826e-007, -4.14528573e-011}, blow = {-2077.309444, 178.0687329}, ahigh = {-23619031.88, 56513.2337, -32.7573832, 0.02293070572, -2.297106441e-006, 4.29259621e-011, 4.23676604e-015}, bhigh = {-367135.862, 301.3437302}, R = 153.7142607824969);
          constant IdealGases.Common.DataRecord C4H6_1butyne(name = "C4H6_1butyne", MM = 0.05409044, Hf = 3054144.133418031, H0 = 296170.635698286, Tlimit = 1000, alow = {-55970.3997, 1433.191711, -9.691210720000001, 0.0715000239, -8.1579678e-005, 5.29036497e-008, -1.435655372e-011}, blow = {11849.87013, 76.6022536}, ahigh = {6364402.7, -23920.87731, 40.7375041, -0.00317672649, 1.199856984e-007, 3.20180251e-011, -2.854392633e-015}, bhigh = {163433.5546, -246.0284791}, R = 153.7142607824969);
          constant IdealGases.Common.DataRecord C4H6_2butyne(name = "C4H6_2butyne", MM = 0.05409044, Hf = 2693636.805320866, H0 = 307632.9199762472, Tlimit = 1000, alow = {-265075.6405, 4490.72869, -24.00723889, 0.0981957955, -0.0001079717182, 6.675555770000001e-008, -1.740766886e-011}, blow = {-5328.36371, 159.5035268}, ahigh = {3981304.33, -18730.32899, 36.5456149, -0.002369686378, 3.99021181e-007, -3.69870727e-011, 1.442072006e-015}, bhigh = {126146.0214, -215.5866205}, R = 153.7142607824969);
          constant IdealGases.Common.DataRecord C4H6_cyclo(name = "C4H6_cyclo", MM = 0.05409044, Hf = 2896999.913478241, H0 = 232158.6587204689, Tlimit = 1000, alow = {-204670.7734, 4919.47057, -37.5327816, 0.1497293031, -0.0001860139375, 1.219909019e-007, -3.25407359e-011}, blow = {-3915.95054, 223.3282138}, ahigh = {4517367.819999999, -20973.11985, 40.0803917, -0.00394979398, 7.44848499e-007, -7.52975417e-011, 3.153253502e-015}, bhigh = {140646.9848, -243.4836999}, R = 153.7142607824969);
          constant IdealGases.Common.DataRecord C4H8_1_butene(name = "C4H8_1_butene", MM = 0.05610631999999999, Hf = -9624.584182316718, H0 = 305134.9651875226, Tlimit = 1000, alow = {-272149.2014, 5100.079250000001, -31.8378625, 0.1317754442, -0.0001527359339, 9.714761109999999e-008, -2.56020447e-011}, blow = {-25230.96386, 200.6932108}, ahigh = {6257948.609999999, -26603.76305, 47.6492005, -0.00438326711, 7.12883844e-007, -5.991020839999999e-011, 2.051753504e-015}, bhigh = {156925.2657, -291.3869761}, R = 148.1913623991023);
          constant IdealGases.Common.DataRecord C4H8_cis2_buten(name = "C4H8_cis2_buten", MM = 0.05610631999999999, Hf = -131892.4499058217, H0 = 299431.5078942979, Tlimit = 1000, alow = {-277387.0877, 5382.38404, -33.751881, 0.1322980623, -0.0001490975922, 9.277722e-008, -2.408282948e-011}, blow = {-27158.84347, 211.4462085}, ahigh = {6461018.35, -27753.76432, 48.6353236, -0.00486238635, 8.412626100000001e-007, -7.63389037e-011, 2.861702826e-015}, bhigh = {163085.6187, -300.3105998}, R = 148.1913623991023);
          constant IdealGases.Common.DataRecord C4H8_isobutene(name = "C4H8_isobutene", MM = 0.05610631999999999, Hf = -304778.4991066961, H0 = 303174.4017429766, Tlimit = 1000, alow = {-232720.5032, 3941.99424, -22.24581184, 0.1012790864, -0.0001073194065, 6.45469691e-008, -1.646330345e-011}, blow = {-22337.66063, 147.9597621}, ahigh = {6484970.99, -27325.04764, 48.3632108, -0.00476800405, 8.23387584e-007, -7.449252999999999e-011, 2.782303056e-015}, bhigh = {159594.1773, -298.2986237}, R = 148.1913623991023);
          constant IdealGases.Common.DataRecord C4H8_cyclo(name = "C4H8_cyclo", MM = 0.05610631999999999, Hf = 506181.8347736941, H0 = 241223.5020938818, Tlimit = 1000, alow = {-304765.5983, 6519.48273, -46.6298759, 0.1743593052, -0.0002090964176, 1.343528679e-007, -3.5427274e-011}, blow = {-27000.31171, 273.8348195}, ahigh = {4456213.810000001, -23010.18492, 44.9244846, -0.003080145176, 5.317165260000001e-007, -5.08107938e-011, 2.048919092e-015}, bhigh = {135548.7603, -277.1801965}, R = 148.1913623991023);
          constant IdealGases.Common.DataRecord C4H9_n_butyl(name = "C4H9_n_butyl", MM = 0.05711426, Hf = 1164857.95316266, H0 = 346620.9664626662, Tlimit = 1000, alow = {-223956.0407, 4676.554099999999, -29.85424449, 0.1345493704, -0.000160066035, 1.045338096e-007, -2.812951048e-011}, blow = {-15252.97704, 190.1651559}, ahigh = {7198686.94, -29592.41524, 52.4204281, -0.00544157244, 9.917758369999999e-007, -9.464455870000001e-011, 3.72948704e-015}, bhigh = {183456.6472, -321.420917}, R = 145.5761135660341);
          constant IdealGases.Common.DataRecord C4H9_i_butyl(name = "C4H9_i_butyl", MM = 0.05711426, Hf = 1003602.252747387, H0 = 320707.9983177581, Tlimit = 1000, alow = {-239946.1281, 4697.44454, -30.8747256, 0.1391655831, -0.0001670233968, 1.092423088e-007, -2.936209962e-011}, blow = {-16381.51036, 193.6662372}, ahigh = {6752936.06, -28374.23721, 51.4068761, -0.00498239901, 8.80150663e-007, -8.12573869e-011, 3.099140009e-015}, bhigh = {174344.848, -314.9940745}, R = 145.5761135660341);
          constant IdealGases.Common.DataRecord C4H9_s_butyl(name = "C4H9_s_butyl", MM = 0.05711426, Hf = 1243122.120465187, H0 = 307062.5269416079, Tlimit = 1000, alow = {-335106.289, 6312.945949999999, -40.0302526, 0.1563875047, -0.0001842331788, 1.182727848e-007, -3.131156589e-011}, blow = {-22160.45659, 248.1295714}, ahigh = {7224744.35, -30352.91099, 52.8855183, -0.00565208355, 1.060032692e-006, -1.066129835e-010, 4.443810960000001e-015}, bhigh = {188349.4279, -325.571988}, R = 145.5761135660341);
          constant IdealGases.Common.DataRecord C4H9_t_butyl(name = "C4H9_t_butyl", MM = 0.05711426, Hf = 905203.0088457768, H0 = 297818.3556961081, Tlimit = 1000, alow = {-472346.195, 8090.198770000001, -46.8367534, 0.1575300095, -0.0001686559436, 9.98104013e-008, -2.487608823e-011}, blow = {-33193.6741, 289.4838706}, ahigh = {7151064.95, -31712.2441, 54.4251032, -0.006392331160000001, 1.241097612e-006, -1.2872483e-010, 5.51261874e-015}, bhigh = {193450.6997, -339.9325290000001}, R = 145.5761135660341);
          constant IdealGases.Common.DataRecord C4H10_n_butane(name = "C4H10_n_butane", MM = 0.0581222, Hf = -2164233.28779709, H0 = 330832.0228759407, Tlimit = 1000, alow = {-317587.254, 6176.331819999999, -38.9156212, 0.1584654284, -0.0001860050159, 1.199676349e-007, -3.20167055e-011}, blow = {-45403.63390000001, 237.9488665}, ahigh = {7682322.45, -32560.5151, 57.3673275, -0.00619791681, 1.180186048e-006, -1.221893698e-010, 5.250635250000001e-015}, bhigh = {177452.656, -358.791876}, R = 143.0515706563069);
          constant IdealGases.Common.DataRecord C4H10_isobutane(name = "C4H10_isobutane", MM = 0.0581222, Hf = -2322520.482707124, H0 = 308599.8121199817, Tlimit = 1000, alow = {-383446.933, 7000.03964, -44.400269, 0.1746183447, -0.0002078195348, 1.339792433e-007, -3.55168163e-011}, blow = {-50340.18889999999, 265.8966497}, ahigh = {7528018.92, -32025.1706, 57.00161, -0.00606001309, 1.143975809e-006, -1.157061835e-010, 4.84604291e-015}, bhigh = {172850.0802, -357.617689}, R = 143.0515706563069);
          constant IdealGases.Common.DataRecord C4N2(name = "C4N2", MM = 0.0760562, Hf = 6958012.627504397, H0 = 234029.5860166561, Tlimit = 1000, alow = {158780.2866, -2987.184206, 23.48081602, -0.02607502448, 4.04283003e-005, -2.804912444e-008, 7.39765205e-012}, blow = {75052.9947, -101.757825}, ahigh = {1167686.152, -6198.644179999999, 20.62070093, -0.001518619449, 3.16236168e-007, -3.4699228e-011, 1.555154128e-015}, bhigh = {96674.09389999999, -96.69734738000001}, R = 109.3201080253812);
          constant IdealGases.Common.DataRecord C5(name = "C5", MM = 0.0600535, Hf = 17499801.5436236, H0 = 269623.4357697719, Tlimit = 1000, alow = {-12008.01119, -555.3702910000001, 11.23828271, -0.00434788452, 1.73898749e-005, -1.707945418e-008, 5.57454191e-012}, blow = {126240.4627, -32.6230899}, ahigh = {217205.5356, -2958.510027, 15.61080967, -0.0008200361920000001, 1.776898025e-007, -2.009853583e-011, 9.222677770000001e-016}, bhigh = {139520.8064, -64.33077179999999}, R = 138.4510811193353);
          constant IdealGases.Common.DataRecord C5H6_1_3cyclo(name = "C5H6_1_3cyclo", MM = 0.06610114, Hf = 2031735.004872836, H0 = 204769.9782484841, Tlimit = 1000, alow = {-188625.9728, 4738.22087, -38.8895801, 0.1667438533, -0.0002141149581, 1.438132328e-007, -3.90647811e-011}, blow = {-5667.022010000001, 227.9898557}, ahigh = {4428478.19, -21235.47976, 43.0981901, -0.003914783460000001, 7.31184701e-007, -7.32724119e-011, 3.044403403e-015}, bhigh = {138254.2782, -260.5959678}, R = 125.78409389006);
          constant IdealGases.Common.DataRecord C5H8_cyclo(name = "C5H8_cyclo", MM = 0.06811702, Hf = 497672.9751242788, H0 = 218112.5803800577, Tlimit = 1000, alow = {-263111.4588, 5987.75349, -45.222446, 0.1804626339, -0.0002177216062, 1.402022671e-007, -3.6990943e-011}, blow = {-23795.04749, 266.0136401}, ahigh = {4569848.1, -24060.18294, 48.8100646, -0.00341305498, 6.04700166e-007, -5.927495780000001e-011, 2.44786539e-015}, bhigh = {141398.3093, -298.9533527}, R = 122.0615934167408);
          constant IdealGases.Common.DataRecord C5H10_1_pentene(name = "C5H10_1_pentene", MM = 0.07013290000000001, Hf = -303423.9279995551, H0 = 309127.3852927798, Tlimit = 1000, alow = {-534054.813, 9298.917380000001, -56.6779245, 0.2123100266, -0.000257129829, 1.666834304e-007, -4.43408047e-011}, blow = {-47906.8218, 339.60364}, ahigh = {3744014.97, -21044.85321, 47.3612699, -0.00042442012, -3.89897505e-008, 1.367074243e-011, -9.31319423e-016}, bhigh = {115409.1373, -278.6177449000001}, R = 118.5530899192818);
          constant IdealGases.Common.DataRecord C5H10_cyclo(name = "C5H10_cyclo", MM = 0.07013290000000001, Hf = -1099341.393269065, H0 = 214212.4024530569, Tlimit = 1000, alow = {-414111.971, 8627.5928, -62.0295998, 0.2259910921, -0.000268230333, 1.706289935e-007, -4.46405092e-011}, blow = {-49315.2214, 358.391623}, ahigh = {7501938.73, -35058.6485, 63.2248075, -0.00694035658, 1.337306593e-006, -1.377905033e-010, 5.86735764e-015}, bhigh = {195492.5511, -402.65509}, R = 118.5530899192818);
          constant IdealGases.Common.DataRecord C5H11_pentyl(name = "C5H11_pentyl", MM = 0.07114084, Hf = 643933.9203754131, H0 = 343290.8579656918, Tlimit = 1000, alow = {-465371.592, 8564.22042, -52.9524289, 0.2094288859, -0.0002561602906, 1.692755961e-007, -4.596788110000001e-011}, blow = {-36417.0427, 319.686224}, ahigh = {5697252.899999999, -28917.06175, 58.1102968, -0.00359399501, 4.41996763e-007, -1.509664551e-011, -6.62696443e-016}, bhigh = {171700.955, -352.247712}, R = 116.873402113329);
          constant IdealGases.Common.DataRecord C5H11_t_pentyl(name = "C5H11_t_pentyl", MM = 0.07114084, Hf = 458245.9245631623, H0 = 276124.417423241, Tlimit = 1000, alow = {-515218.198, 9144.32783, -56.0239789, 0.1998204194, -0.0002239112591, 1.375455547e-007, -3.52383305e-011}, blow = {-40362.6661, 340.2811}, ahigh = {8602108.26, -38052.05770000001, 66.4969037, -0.007533860630000001, 1.451612787e-006, -1.495593207e-010, 6.368022329999999e-015}, bhigh = {228185.8805, -416.940855}, R = 116.873402113329);
          constant IdealGases.Common.DataRecord C5H12_n_pentane(name = "C5H12_n_pentane", MM = 0.07214878, Hf = -2034130.029641527, H0 = 335196.2430965569, Tlimit = 1000, alow = {-276889.4625, 5834.28347, -36.1754148, 0.1533339707, -0.0001528395882, 8.191092e-008, -1.792327902e-011}, blow = {-46653.7525, 226.5544053}, ahigh = {-2530779.286, -8972.59326, 45.3622326, -0.002626989916, 3.135136419e-006, -5.31872894e-010, 2.886896868e-014}, bhigh = {14846.16529, -251.6550384}, R = 115.2406457877736);
          constant IdealGases.Common.DataRecord C5H12_i_pentane(name = "C5H12_i_pentane", MM = 0.07214878, Hf = -2130320.152329672, H0 = 305036.3429568733, Tlimit = 1000, alow = {-423190.339, 6497.1891, -36.8112697, 0.1532424729, -0.0001548790714, 8.74989712e-008, -2.07054771e-011}, blow = {-51554.1659, 230.9518218}, ahigh = {11568885.94, -45562.4687, 74.9544363, -0.007845415580000001, 1.444393314e-006, -1.464370213e-010, 6.230285000000001e-015}, bhigh = {254492.7135, -480.198578}, R = 115.2406457877736);
          constant IdealGases.Common.DataRecord CH3C_CH3_2CH3(name = "CH3C_CH3_2CH3", MM = 0.07214878, Hf = -2327412.882102788, H0 = 321266.6936294696, Tlimit = 1000, alow = {-8973222.270000001, 128922.5617, -719.934483, 2.056862183, -0.002953159699, 2.158893146e-006, -6.26831877e-010}, blow = {-639493.2019999999, 4020.80636}, ahigh = {16847055.2, -59794.3057, 86.85451479999999, -0.0096219176, 1.653363091e-006, -1.674727926e-010, 7.37211936e-015}, bhigh = {345849.682, -576.046697}, R = 115.2406457877736);
          constant IdealGases.Common.DataRecord C6D5_phenyl(name = "C6D5_phenyl", MM = 0.08213471, Hf = 3844172.579412528, H0 = 193816.6823746014, Tlimit = 1000, alow = {201283.7008, -1979.349332, 2.6282675, 0.0595186526, -6.08128045e-005, 3.38139165e-008, -8.125429080000001e-012}, blow = {46972.47489999999, 0.335470098}, ahigh = {1411628.125, -13625.69472, 40.2898494, -0.00349098927, 7.37961213e-007, -8.19224405e-011, 3.70533097e-015}, bhigh = {108807.3731, -229.3621057}, R = 101.2296993560944);
          constant IdealGases.Common.DataRecord C6D6(name = "C6D6", MM = 0.084148812, Hf = 691125.3601536288, H0 = 193997.8190066427, Tlimit = 1000, alow = {276291.1236, -2865.868902, 5.39507043, 0.05939315170000001, -6.02363118e-005, 3.38139857e-008, -8.306046370000001e-012}, blow = {20470.83778, -20.11790696}, ahigh = {1758057.871, -15721.21558, 44.6816249, -0.004003361919999999, 8.4454344e-007, -9.36055698e-011, 4.22847553e-015}, bhigh = {89620.45359999999, -261.5332268}, R = 98.80676627971884);
          constant IdealGases.Common.DataRecord C6H2(name = "C6H2", MM = 0.07408007999999999, Hf = 9044266.690856706, H0 = 264682.6785284249, Tlimit = 1000, alow = {290372.2964, -4929.7515, 31.8932321, -0.03119447315, 4.32576368e-005, -2.732517022e-008, 6.67444611e-012}, blow = {101189.8682, -153.0593012}, ahigh = {2592848.577, -10833.61978, 28.47459495, -0.001876727704, 3.41213428e-007, -3.33739289e-011, 1.356853889e-015}, bhigh = {141851.7294, -149.4853494}, R = 112.2362718830757);
          constant IdealGases.Common.DataRecord C6H5_phenyl(name = "C6H5_phenyl", MM = 0.07710389999999999, Hf = 4373319.637528064, H0 = 183578.6905720723, Tlimit = 1000, alow = {-121127.8245, 3529.04558, -31.16903422, 0.146755063, -0.0001831398296, 1.192576957e-007, -3.14926586e-011}, blow = {24209.72928, 186.8799946}, ahigh = {3670279.23, -18946.01209, 41.8058182, -0.00350391415, 6.56182174e-007, -6.594714449999999e-011, 2.748094212e-015}, bhigh = {147674.4628, -247.5301142}, R = 107.8346490903833);
          constant IdealGases.Common.DataRecord C6H5O_phenoxy(name = "C6H5O_phenoxy", MM = 0.0931033, Hf = 512334.1492729044, H0 = 174087.2128055611, Tlimit = 1000, alow = {-129226.4217, 3406.74068, -29.11367361, 0.1459180378, -0.0001780202758, 1.138615885e-007, -2.967142152e-011}, blow = {-10550.26758, 177.0682011}, ahigh = {3678640.34, -19729.80806, 45.4118441, -0.00375106439, 7.11445078e-007, -7.233328850000001e-011, 3.045637067e-015}, bhigh = {116359.5961, -268.1058539}, R = 89.30373037260763);
          constant IdealGases.Common.DataRecord C6H6(name = "C6H6", MM = 0.07811184, Hf = 1061042.730525872, H0 = 181735.4577743912, Tlimit = 1000, alow = {-167734.0902, 4404.50004, -37.1737791, 0.1640509559, -0.0002020812374, 1.307915264e-007, -3.4442841e-011}, blow = {-10354.55401, 216.9853345}, ahigh = {4538575.72, -22605.02547, 46.940073, -0.004206676830000001, 7.90799433e-007, -7.9683021e-011, 3.32821208e-015}, bhigh = {139146.4686, -286.8751333}, R = 106.4431717393932);
          constant IdealGases.Common.DataRecord C6H5OH_phenol(name = "C6H5OH_phenol", MM = 0.09411124, Hf = -1024309.104842312, H0 = 185915.0936700016, Tlimit = 1000, alow = {-120941.8144, 3378.29227, -29.91846148, 0.1567802942, -0.0001970937198, 1.291815064e-007, -3.42435126e-011}, blow = {-27793.87017, 179.9708977}, ahigh = {4462081.569999999, -21655.21026, 48.1501505, -0.00356828243, 6.32717573e-007, -6.039905720000001e-011, 2.399168678e-015}, bhigh = {111372.6204, -286.0454446}, R = 88.34727924103434);
          constant IdealGases.Common.DataRecord C6H10_cyclo(name = "C6H10_cyclo", MM = 0.08214360000000001, Hf = -55999.49356979728, H0 = 210251.1212072517, Tlimit = 1000, alow = {-375028.221, 7643.87675, -55.1043476, 0.2166231405, -0.0002545452198, 1.607607304e-007, -4.18524841e-011}, blow = {-36610.7301, 320.240946}, ahigh = {7510128.98, -35568.5821, 67.03034940000001, -0.00704535187, 1.3581664e-006, -1.400074492e-010, 5.964551700000001e-015}, bhigh = {206027.4332, -423.819766}, R = 101.218743760926);
          constant IdealGases.Common.DataRecord C6H12_1_hexene(name = "C6H12_1_hexene", MM = 0.08415948000000001, Hf = -498458.4030224521, H0 = 311788.9986962847, Tlimit = 1000, alow = {-666883.165, 11768.64939, -72.70998330000001, 0.2709398396, -0.00033332464, 2.182347097e-007, -5.85946882e-011}, blow = {-62157.8054, 428.682564}, ahigh = {733290.696, -14488.48641, 46.7121549, 0.00317297847, -5.24264652e-007, 4.28035582e-011, -1.472353254e-015}, bhigh = {66977.4041, -262.3643854}, R = 98.79424159940152);
          constant IdealGases.Common.DataRecord C6H12_cyclo(name = "C6H12_cyclo", MM = 0.08415948000000001, Hf = -1465075.5921971, H0 = 208471.0361803566, Tlimit = 1000, alow = {-567998.704, 10342.38704, -68.0004125, 0.2387797658, -0.0002511890049, 1.425293184e-007, -3.40783319e-011}, blow = {-64046.3516, 393.480821}, ahigh = {5225149.47, -33641.9458, 71.74607469999999, -0.006698979119999999, 1.318443254e-006, -1.390794789e-010, 6.06010224e-015}, bhigh = {173253.7609, -454.681417}, R = 98.79424159940152);
          constant IdealGases.Common.DataRecord C6H13_n_hexyl(name = "C6H13_n_hexyl", MM = 0.08516742000000001, Hf = 294713.635801108, H0 = 340306.1875069128, Tlimit = 1000, alow = {-1427278.22, 22488.28093, -129.749224, 0.427979733, -0.000555601318, 3.79125694e-007, -1.048462404e-010}, blow = {-106026.1015, 749.718676}, ahigh = {5967938.62, -32990.2316, 68.6907344, -0.00422500906, 5.496523820000001e-007, -2.292851471e-011, -5.08634189e-016}, bhigh = {190697.8683, -418.5362}, R = 97.62503079229123);
          constant IdealGases.Common.DataRecord C6H14_n_hexane(name = "C6H14_n_hexane", MM = 0.08617535999999999, Hf = -1936980.593988816, H0 = 333065.0431863586, Tlimit = 1000, alow = {-581592.67, 10790.97724, -66.3394703, 0.2523715155, -0.0002904344705, 1.802201514e-007, -4.617223680000001e-011}, blow = {-72715.4457, 393.828354}, ahigh = {-3106625.684, -7346.087920000001, 46.94131760000001, 0.001693963977, 2.068996667e-006, -4.21214168e-010, 2.452345845e-014}, bhigh = {523.750312, -254.9967718}, R = 96.48317105956971);
          constant IdealGases.Common.DataRecord C7H7_benzyl(name = "C7H7_benzyl", MM = 0.09113048, Hf = 2309874.808077385, H0 = 203607.9695838319, Tlimit = 1000, alow = {-183676.4826, 4102.46566, -32.024061, 0.1588249575, -0.0001894466924, 1.203649671e-007, -3.136589637e-011}, blow = {5266.33323, 193.875172}, ahigh = {5297322.16, -25999.09398, 55.0975079, -0.00497766147, 9.46315243e-007, -9.639008860000001e-011, 4.06447042e-015}, bhigh = {173838.2912, -334.382955}, R = 91.23700434805129);
          constant IdealGases.Common.DataRecord C7H8(name = "C7H8", MM = 0.09213842, Hf = 544506.8409030674, H0 = 194710.794910527, Tlimit = 1000, alow = {-287796.222, 6133.941519999999, -45.74706759999999, 0.1936895724, -0.0002304305304, 1.459301178e-007, -3.7907961e-011}, blow = {-23084.02499, 269.3915042}, ahigh = {6184538.350000001, -29902.84056, 59.8200597, -0.00569698396, 1.080748416e-006, -1.098702235e-010, 4.62474022e-015}, bhigh = {178204.7857, -369.808225}, R = 90.23892530390688);
          constant IdealGases.Common.DataRecord C7H8O_cresol_mx(name = "C7H8O_cresol_mx", MM = 0.10813782, Hf = -1223420.261292488, H0 = 201949.1792973078, Tlimit = 1000, alow = {-244141.7503, 5080.87784, -37.8904804, 0.186494507, -0.0002269511868, 1.458989279e-007, -3.82240335e-011}, blow = {-40936.57829999999, 228.1352234}, ahigh = {6017373.45, -28498.82774, 60.81543070000001, -0.004996807109999999, 9.13427995e-007, -8.979444109999999e-011, 3.66775697e-015}, bhigh = {147221.84, -367.485014}, R = 76.88773455947235);
          constant IdealGases.Common.DataRecord C7H14_1_heptene(name = "C7H14_1_heptene", MM = 0.09818605999999999, Hf = -639194.6066478277, H0 = 313588.3036756949, Tlimit = 1000, alow = {-744940.284, 13321.79893, -82.81694379999999, 0.3108065994, -0.000378677992, 2.446841042e-007, -6.488763869999999e-011}, blow = {-72178.8501, 485.667149}, ahigh = {-1927608.174, -9125.024420000002, 47.4817797, 0.00606766053, -8.684859080000001e-007, 5.81399526e-011, -1.473979569e-015}, bhigh = {26009.14656, -256.2880707}, R = 84.68077851377274);
          constant IdealGases.Common.DataRecord C7H15_n_heptyl(name = "C7H15_n_heptyl", MM = 0.099194, Hf = 44256.70907514567, H0 = 338155.5336008226, Tlimit = 1000, alow = {-1671733.521, 26400.1025, -152.6867707, 0.5027121410000001, -0.000652101403, 4.44348811e-007, -1.227815006e-010}, blow = {-127375.4623, 878.3933319999999}, ahigh = {5444527.57, -34568.2929, 76.38651949999999, -0.003298972, 2.343496957e-007, 2.467674021e-011, -3.162012849e-015}, bhigh = {193907.9354, -464.142466}, R = 83.82031171240196);
          constant IdealGases.Common.DataRecord C7H16_n_heptane(name = "C7H16_n_heptane", MM = 0.10020194, Hf = -1874015.612871368, H0 = 331540.487140269, Tlimit = 1000, alow = {-612743.289, 11840.85437, -74.87188599999999, 0.2918466052, -0.000341679549, 2.159285269e-007, -5.65585273e-011}, blow = {-80134.0894, 440.721332}, ahigh = {9135632.469999999, -39233.1969, 78.8978085, -0.00465425193, 2.071774142e-006, -3.4425393e-010, 1.976834775e-014}, bhigh = {205070.8295, -485.110402}, R = 82.97715593131233);
          constant IdealGases.Common.DataRecord C7H16_2_methylh(name = "C7H16_2_methylh", MM = 0.10020194, Hf = -1942078.167348856, H0 = 308576.8598891399, Tlimit = 1000, alow = {-710477.777, 11912.5112, -73.45339440000001, 0.2902952369, -0.000346276768, 2.260184498e-007, -6.12881392e-011}, blow = {-82021.477, 432.004229}, ahigh = {1289912.969, -1784.340963, 10.83537673, 0.05270609239999999, -1.886832314e-005, 2.432255843e-009, -1.135553789e-013}, bhigh = {-16375.29884, -29.8186241}, R = 82.97715593131233);
          constant IdealGases.Common.DataRecord C8H8_styrene(name = "C8H8_styrene", MM = 0.10414912, Hf = 1423919.85645198, H0 = 201057.8677957144, Tlimit = 1000, alow = {-268693.052, 6167.99947, -48.3605494, 0.2182873229, -0.0002738561832, 1.810084981e-007, -4.86775027e-011}, blow = {-11406.39978, 281.7679014}, ahigh = {-6629183.62, 15145.94166, 1.609822364, 0.033833186, -1.093737395e-005, 1.338825116e-009, -6.03253492e-014}, bhigh = {-89973.2415, 43.1128279}, R = 79.83237880454487);
          constant IdealGases.Common.DataRecord C8H10_ethylbenz(name = "C8H10_ethylbenz", MM = 0.106165, Hf = 281825.4603682946, H0 = 209862.0072528611, Tlimit = 1000, alow = {-469494, 9307.16836, -65.2176947, 0.2612080237, -0.000318175348, 2.051355473e-007, -5.40181735e-011}, blow = {-40738.7021, 378.090436}, ahigh = {5551564.100000001, -28313.80598, 60.6124072, 0.001042112857, -1.327426719e-006, 2.166031743e-010, -1.142545514e-014}, bhigh = {164224.1062, -369.176982}, R = 78.31650732350586);
          constant IdealGases.Common.DataRecord C8H16_1_octene(name = "C8H16_1_octene", MM = 0.11221264, Hf = -744924.9924072726, H0 = 315026.8989304592, Tlimit = 1000, alow = {-928190.522, 16409.74476, -101.5939534, 0.374800141, -0.00045908294, 2.96533534e-007, -7.84044521e-011}, blow = {-89524.2608, 590.759427}, ahigh = {-4409336.07, -4383.678800000001, 49.39154259999999, 0.007912339629999999, -7.88866951e-007, 9.97021235e-012, 1.913144872e-015}, bhigh = {-11226.19342, -257.7650649}, R = 74.09568119955115);
          constant IdealGases.Common.DataRecord C8H17_n_octyl(name = "C8H17_n_octyl", MM = 0.11322058, Hf = -144143.4057306543, H0 = 336537.7566516617, Tlimit = 1000, alow = {-1934340.995, 30549.7983, -176.7903454, 0.5801596649999999, -0.000751741401, 5.11246903e-007, -1.410193662e-010}, blow = {-149889.4706, 1013.724329}, ahigh = {5632173.390000001, -38211.4367, 86.37927500000001, -0.00360893158, 2.544260445e-007, 2.908638837e-011, -3.67954974e-015}, bhigh = {210313.547, -526.242283}, R = 73.43604846398067);
          constant IdealGases.Common.DataRecord C8H18_n_octane(name = "C8H18_n_octane", MM = 0.11422852, Hf = -1827477.060895125, H0 = 330740.51909278, Tlimit = 1000, alow = {-698664.715, 13385.01096, -84.1516592, 0.327193666, -0.000377720959, 2.339836988e-007, -6.01089265e-011}, blow = {-90262.2325, 493.922214}, ahigh = {6365406.949999999, -31053.64657, 69.6916234, 0.01048059637, -4.12962195e-006, 5.543226319999999e-010, -2.651436499e-014}, bhigh = {150096.8785, -416.989565}, R = 72.78805678301707);
          constant IdealGases.Common.DataRecord C8H18_isooctane(name = "C8H18_isooctane", MM = 0.11422852, Hf = -1961068.916939482, H0 = 281628.440953275, Tlimit = 1000, alow = {-168875.8565, 3126.903227, -21.23502828, 0.1489151508, -0.0001151180135, 4.47321617e-008, -5.55488207e-012}, blow = {-44680.6062, 141.7455793}, ahigh = {13527650.32, -46633.7034, 77.95313179999999, 0.01423729984, -5.073593909999999e-006, 7.24823297e-010, -3.81919011e-014}, bhigh = {254117.8017, -493.388719}, R = 72.78805678301707);
          constant IdealGases.Common.DataRecord C9H19_n_nonyl(name = "C9H19_n_nonyl", MM = 0.12724716, Hf = -291008.4594422383, H0 = 335284.496722756, Tlimit = 1000, alow = {-2194880.612, 34685.6578, -200.9261419, 0.658050311, -0.000852593001, 5.79463896e-007, -1.597637099e-010}, blow = {-172319.4608, 1149.114017}, ahigh = {5277361.74, -40257.0196, 94.57296720000001, -0.002940301447, 6.97810699e-009, 6.80525024e-011, -5.90709533e-015}, bhigh = {216532.0614, -575.44495}, R = 65.34112038335474);
          constant IdealGases.Common.DataRecord C10H8_naphthale(name = "C10H8_naphthale", MM = 0.12817052, Hf = 1174841.141317052, H0 = 161605.6172667475, Tlimit = 1000, alow = {-260284.5316, 6237.40957, -52.2609504, 0.2397692776, -0.0002912244803, 1.854944401e-007, -4.81661927e-011}, blow = {-11147.0088, 297.2139517}, ahigh = {5906172.11, -31632.2924, 70.3034203, -0.00601886554, 1.142052144e-006, -1.161605689e-010, 4.89284402e-015}, bhigh = {196256.7046, -434.7848950000001}, R = 64.87039297336079);
          constant IdealGases.Common.DataRecord C10H21_n_decyl(name = "C10H21_n_decyl", MM = 0.14127374, Hf = -408710.0688351565, H0 = 334273.0220067792, Tlimit = 1000, alow = {-2446511.152, 38700.813, -224.4388176, 0.734362497, -0.00095135256, 6.46297033e-007, -1.781502862e-010}, blow = {-194163.3889, 1280.987834}, ahigh = {4967237.76, -42424.6844, 102.8853417, -0.00232484818, -2.2842339e-007, 1.056127364e-010, -8.0680659e-015}, bhigh = {223542.989, -625.5191159999999}, R = 58.85362700810497);
          constant IdealGases.Common.DataRecord C12H9_o_bipheny(name = "C12H9_o_bipheny", MM = 0.15319986, Hf = 2791973.830785485, H0 = 173559.8648719392, Tlimit = 1000, alow = {-359584.082, 7661.37856, -58.7329046, 0.2697557882, -0.000322975668, 2.0329071e-007, -5.23131672e-011}, blow = {14584.14642, 339.268711}, ahigh = {6736469.42, -36321.9435, 82.102396, -0.00750234286, 1.456251733e-006, -1.507145019e-010, 6.43647043e-015}, bhigh = {255552.4521, -504.246297}, R = 54.27206003974155);
          constant IdealGases.Common.DataRecord C12H10_biphenyl(name = "C12H10_biphenyl", MM = 0.1542078, Hf = 1181068.66189648, H0 = 173684.6190659617, Tlimit = 1000, alow = {-367103.405, 8128.41259, -63.9099457, 0.2901422744, -0.000350959074, 2.230989996e-007, -5.78847029e-011}, blow = {-16792.63066, 363.346419}, ahigh = {7480385.359999999, -39280.8723, 86.61482219999999, -0.007946398570000001, 1.531868544e-006, -1.576450171e-010, 6.70060273e-015}, bhigh = {243805.0641, -538.138149}, R = 53.91732454519163);
          constant IdealGases.Common.DataRecord Ca(name = "Ca", MM = 0.040078, Hf = 4436349.119217526, H0 = 154634.1633814063, Tlimit = 1000, alow = {0, 0, 2.5, 0, 0, 0, 0}, blow = {20638.92786, 4.38454833}, ahigh = {7547341.239999999, -21486.42662, 25.30849567, -0.01103773705, 2.293249636e-006, -1.209075383e-010, -4.015333268e-015}, bhigh = {158586.2323, -160.9512955}, R = 207.4572583462249);
          constant IdealGases.Common.DataRecord Caplus(name = "Caplus", MM = 0.0400774514, Hf = 19308306.81513845, H0 = 154636.2800904052, Tlimit = 1000, alow = {0, 0, 2.5, 0, 0, 0, 0}, blow = {92324.1779, 5.07767498}, ahigh = {3747070.82, -11747.07738, 16.72546969, -0.00833479771, 2.394593294e-006, -2.988243468e-010, 1.356563002e-014}, bhigh = {166432.9088, -95.8282126}, R = 207.4600981239042);
          constant IdealGases.Common.DataRecord CaBr(name = "CaBr", MM = 0.119982, Hf = -207270.0988481606, H0 = 82146.64699704957, Tlimit = 1000, alow = {569.3832620000001, -125.7513537, 5.00150833, -0.001039868753, 1.377594927e-006, -8.94901396e-010, 2.395658928e-013}, blow = {-3728.10398, 1.784864847}, ahigh = {2783236.402, -8458.203729999999, 14.3175656, -0.00543078168, 1.522218659e-006, -1.831824807e-010, 7.86428866e-015}, bhigh = {49312.7068, -65.37001100000001}, R = 69.29766131586405);
          constant IdealGases.Common.DataRecord CaBr2(name = "CaBr2", MM = 0.199886, Hf = -1937090.641665749, H0 = 78060.49448185465, Tlimit = 1000, alow = {1572.504318, -223.8551236, 8.38845789, -0.001944164774, 2.397926088e-006, -1.55657235e-009, 4.12430958e-013}, blow = {-47721.0661, -10.74965233}, ahigh = {-22198.46426, -4.46315533, 7.50375511, -1.641654626e-006, 3.87942806e-010, -4.68342426e-014, 2.259325541e-018}, bhigh = {-48854.3682, -5.59032425}, R = 41.59606975976306);
          constant IdealGases.Common.DataRecord CaCL(name = "CaCL", MM = 0.075531, Hf = -1373911.175543816, H0 = 127035.4556407303, Tlimit = 1000, alow = {6395.335260000001, -224.9042269, 5.28327839, -0.001447983237, 1.643742771e-006, -9.43010428e-010, 2.237516133e-013}, blow = {-12701.69, -1.391964289}, ahigh = {1629182.545, -4766.22302, 9.65892977, -0.002523044131, 5.83354216e-007, -4.10172699e-011, 8.813177639999999e-017}, bhigh = {16625.99241, -33.98731174}, R = 110.0802584369332);
          constant IdealGases.Common.DataRecord CaCLplus(name = "CaCLplus", MM = 0.0755304514, Hf = 6185459.034606007, H0 = 123938.2636603705, Tlimit = 1000, alow = {2285.284542, -200.2872721, 4.93699019, -0.000386097303, 8.686661880000001e-008, 1.47020568e-010, -7.68931587e-014}, blow = {55882.7795, -0.6359815341}, ahigh = {177028.0396, -589.137239, 5.1033688, -0.0002356761168, 5.48373287e-008, 2.137973112e-014, -5.57288731e-016}, bhigh = {58534.17249999999, -2.191253644}, R = 110.0810579823968);
          constant IdealGases.Common.DataRecord CaCL2(name = "CaCL2", MM = 0.110984, Hf = -4372193.081885677, H0 = 133866.3501045195, Tlimit = 1000, alow = {11068.0203, -424.910825, 9.13471186, -0.00349718531, 4.242184629999999e-006, -2.719673231e-009, 7.13826863e-013}, blow = {-58503.4676, -18.08678294}, ahigh = {-35957.6878, -8.796495630000001, 7.50725691, -3.129465934e-006, 7.322886480000001e-010, -8.776403220000001e-014, 4.21048604e-018}, bhigh = {-60667.7966, -8.553523451}, R = 74.91595184891516);
          constant IdealGases.Common.DataRecord CaF(name = "CaF", MM = 0.0590764032, Hf = -4678751.159989374, H0 = 154581.3472950229, Tlimit = 1000, alow = {31842.354, -491.8235089999999, 5.70112089, -0.001465138218, 9.09293081e-007, -1.367585222e-010, -5.10632338e-014}, blow = {-31976.9411, -5.99666226}, ahigh = {519588.2669999999, -1512.321567, 5.86927625, -0.000388252448, -2.484582298e-008, 3.74991596e-011, -3.47097645e-015}, bhigh = {-24888.14584, -8.6146637}, R = 140.7409989374573);
          constant IdealGases.Common.DataRecord CaFplus(name = "CaFplus", MM = 0.0590758546, Hf = 4412371.344688089, H0 = 152399.1156955688, Tlimit = 1000, alow = {41583.408, -562.766166, 5.68752215, -0.001264710572, 6.66908648e-007, -6.92675282e-011, -3.97871462e-014}, blow = {33051.195, -6.97448501}, ahigh = {-139867.0845, 219.1526375, 4.21432434, 0.0002142798048, -5.7786726e-008, 9.98575631e-012, -6.05962188e-016}, bhigh = {28416.05897, 2.740342729}, R = 140.7423059098666);
          constant IdealGases.Common.DataRecord CaF2(name = "CaF2", MM = 0.0780748064, Hf = -10129111.36722332, H0 = 164093.7914640798, Tlimit = 1000, alow = {59093.5229, -1019.733042, 10.13165309, -0.00550350644, 5.62931998e-006, -3.115401465e-009, 7.205525940000001e-013}, blow = {-91926.08189999999, -26.3950595}, ahigh = {-82804.0944, -31.15950071, 7.02357737, -9.521000020000001e-006, 2.118160895e-009, -2.440713888e-013, 1.135135468e-017}, bhigh = {-97294.66770000001, -7.54624334}, R = 106.4936614431362);
          constant IdealGases.Common.DataRecord CaH(name = "CaH", MM = 0.04108594, Hf = 5583640.169848858, H0 = 211875.5223806489, Tlimit = 1000, alow = {-45137.8223, 762.942921, -1.280874223, 0.01318774659, -1.481595334e-005, 8.53657322e-009, -1.989958945e-012}, blow = {23003.78814, 30.53421525}, ahigh = {-2696952.529, 8607.05975, -7.02745482, 0.007467916309999999, -2.318610699e-006, 3.42307242e-010, -1.892679792e-014}, bhigh = {-27738.19107, 78.45822010000001}, R = 202.367817311713);
          constant IdealGases.Common.DataRecord CaI(name = "CaI", MM = 0.16698247, Hf = 72961.67076699728, H0 = 59995.04019793216, Tlimit = 1000, alow = {-826.179018, -85.08975899999999, 4.86417169, -0.000786316965, 1.100493506e-006, -7.34803061e-010, 2.010892767e-013}, blow = {523.675344, 3.58539168}, ahigh = {1771071.309, -5683.64373, 11.53857476, -0.0041940202, 1.291359961e-006, -1.730043579e-010, 8.38076235e-015}, bhigh = {35832.932, -44.0591757}, R = 49.79248420507854);
          constant IdealGases.Common.DataRecord CaI2(name = "CaI2", MM = 0.29388694, Hf = -882378.4581921197, H0 = 54616.22418471539, Tlimit = 1000, alow = {-1097.851401, -145.9821761, 8.08934184, -0.001305332923, 1.624199331e-006, -1.061194001e-009, 2.825551217e-013}, blow = {-32726.9298, -6.98171494}, ahigh = {-16237.27104, -2.859530722, 7.50243219, -1.071178707e-006, 2.544559095e-010, -3.083631004e-014, 1.491836724e-018}, bhigh = {-33463.4117, -3.56730574}, R = 28.29139668472509);
          constant IdealGases.Common.DataRecord CaO(name = "CaO", MM = 0.0560774, Hf = 677729.4953047038, H0 = 159656.2608109506, Tlimit = 1000, alow = {38897.3307, -483.567735, 5.07771325, 0.000307623525, -1.159759897e-006, 8.493433339999999e-010, -1.495333366e-013}, blow = {5937.643480000001, -3.95532073}, ahigh = {-49131061.7, 149586.595, -168.1654149, 0.09381950259999999, -2.455529428e-005, 3.07498072e-009, -1.485914237e-013}, bhigh = {-946151.172, 1235.694769}, R = 148.2677870229362);
          constant IdealGases.Common.DataRecord CaOplus(name = "CaOplus", MM = 0.0560768514, Hf = 12665431.88977975, H0 = 163402.7191476731, Tlimit = 1000, alow = {109806.0332, -1459.992448, 9.88077794, -0.009157564600000001, 8.707582560000001e-006, -4.39041108e-009, 9.264854659999999e-013}, blow = {91500.57350000001, -30.09947701}, ahigh = {939784.313, -2993.362243, 8.33619182, -0.002303295087, 7.37396753e-007, -1.052888279e-010, 5.25713841e-015}, bhigh = {102809.8401, -24.61547836}, R = 148.2692375271269);
          constant IdealGases.Common.DataRecord CaOH(name = "CaOH", MM = 0.05708534000000001, Hf = -3035934.444815429, H0 = 193564.5298775482, Tlimit = 1000, alow = {46200.289, -928.567282, 9.175828770000001, -0.0039628288, 2.505447308e-006, 3.85206821e-011, -3.35277847e-013}, blow = {-17980.09717, -25.3370485}, ahigh = {1979972.994, -5598.88099, 11.51348706, -0.001668264707, 3.31257391e-007, -1.789056647e-011, -3.58071641e-016}, bhigh = {13401.96822, -46.46084260000001}, R = 145.6498638704788);
          constant IdealGases.Common.DataRecord CaOHplus(name = "CaOHplus", MM = 0.0570847914, Hf = 6533055.317427332, H0 = 194407.6299103372, Tlimit = 1000, alow = {48843.4266, -983.2415100000001, 9.61018295, -0.00520436066, 4.26304982e-006, -1.197590878e-009, 9.028411889999999e-015}, blow = {47950.56340000001, -28.27077334}, ahigh = {863761.537, -2347.302046, 7.9819112, 0.0001003160947, -6.24007417e-008, 1.019540904e-011, -5.69892528e-016}, bhigh = {58305.80119999999, -21.52181937}, R = 145.6512636043372);
          constant IdealGases.Common.DataRecord Ca_OH_2(name = "Ca_OH_2", MM = 0.07409268000000001, Hf = -8075546.315776401, H0 = 223939.5443652463, Tlimit = 1000, alow = {83892.57539999999, -1791.902135, 16.21891031, -0.00784085717, 5.09511161e-006, -6.95548755e-011, -6.13215264e-013}, blow = {-66004.0563, -60.7091392}, ahigh = {1721854.884, -4702.21767, 13.96947691, 0.0001983791405, -1.243050592e-007, 2.033402404e-011, -1.137157819e-015}, bhigh = {-44437.6135, -52.8744486}, R = 112.2171852873995);
          constant IdealGases.Common.DataRecord CaS(name = "CaS", MM = 0.072143, Hf = 1683812.885519038, H0 = 129743.8836754779, Tlimit = 1000, alow = {23209.90615, -451.8446080000001, 6.17233375, -0.00359885202, 4.823731709999999e-006, -3.63173354e-009, 1.212849867e-012}, blow = {15545.97691, -7.686936462}, ahigh = {-15683532.14, 52938.5581, -63.2246724, 0.0402772638, -1.114855081e-005, 1.464389344e-009, -7.37607897e-014}, bhigh = {-317202.267, 479.9606012}, R = 115.2498787131115);
          constant IdealGases.Common.DataRecord Ca2(name = "Ca2", MM = 0.08015600000000001, Hf = 4263752.382853436, H0 = 140639.9520934178, Tlimit = 1000, alow = {-85822.2862, 158.818896, 11.03952055, -0.0333319676, 5.34593881e-005, -4.011573239999999e-008, 1.160486682e-011}, blow = {37703.508, -23.97744561}, ahigh = {240596.6267, 57.7580382, 2.347436675, 0.0001199275034, -4.32915031e-008, 7.01530269e-012, -3.70566032e-016}, bhigh = {40818.754, 18.95399607}, R = 103.7286291731124);
          constant IdealGases.Common.DataRecord Cd(name = "Cd", MM = 0.112411, Hf = 994564.5888747543, H0 = 55131.86431932818, Tlimit = 1000, alow = {-0.0001081751543, 1.816433041e-006, 2.499999989, 3.129989231e-011, -4.60071016e-014, 3.40704874e-017, -9.989497436e-021}, blow = {12700.99766, 5.93154976}, ahigh = {-269975.7467, 786.600114, 1.628169079, 0.000459412329, -1.150420443e-007, 1.074836707e-011, 8.790199554999999e-017}, bhigh = {7675.14826, 12.20006052}, R = 73.96493225751928);
          constant IdealGases.Common.DataRecord Cdplus(name = "Cdplus", MM = 0.1124104514, Hf = 8769240.899961371, H0 = 55132.13338097137, Tlimit = 1000, alow = {0.000409834494, -4.34358162e-006, 2.500000019, -4.389036810000001e-011, 5.5639692e-014, -3.63822826e-017, 9.607881994999999e-021}, blow = {117812.9439, 6.62468945}, ahigh = {14848.80812, -46.6915368, 2.557883006, -3.6247017e-005, 1.21374757e-008, -2.072802814e-012, 1.420665367e-016}, bhigh = {118107.0109, 6.21641448}, R = 73.96529323073246);
          constant IdealGases.Common.DataRecord CL(name = "CL", MM = 0.03545300000000001, Hf = 3421459.396948072, H0 = 176898.6545567371, Tlimit = 1000, alow = {22762.15854, -216.8413293, 2.745185115, 0.002451101694, -5.45801199e-006, 4.41798688e-009, -1.288134004e-012}, blow = {15013.57068, 3.102963457}, ahigh = {-169793.293, 608.172646, 2.12866409, 0.0001307367034, -2.644883596e-008, 2.842504775e-012, -1.252911731e-016}, bhigh = {9934.3874, 8.844772103}, R = 234.5209714269596);
          constant IdealGases.Common.DataRecord CLplus(name = "CLplus", MM = 0.0354524514, Hf = 38891517.52705033, H0 = 180138.0369426302, Tlimit = 1000, alow = {103469.7859, -1293.758873, 8.186702690000001, -0.0099160146, 9.20847237e-006, -4.50742624e-009, 9.18212788e-013}, blow = {171475.878, -27.66417931}, ahigh = {40564.0948, -49.6016572, 3.10165363, -0.000586873829, 2.252039316e-007, -3.29970302e-011, 1.708780842e-015}, bhigh = {165298.2337, 2.574942598}, R = 234.5246004624662);
          constant IdealGases.Common.DataRecord CLminus(name = "CLminus", MM = 0.0354535486, Hf = -6599000.135066875, H0 = 174804.1661477012, Tlimit = 1000, alow = {0, 0, 2.5, 0, 0, 0, 0}, blow = {-28883.89093, 4.200642023}, ahigh = {0, 0, 2.5, 0, 0, 0, 0}, bhigh = {-28883.89093, 4.200642023}, R = 234.5173425037629);
          constant IdealGases.Common.DataRecord CLCN(name = "CLCN", MM = 0.06147039999999999, Hf = 2183164.580025509, H0 = 173563.8941669487, Tlimit = 1000, alow = {72740.43429999999, -1297.344947, 11.07676527, -0.01108430216, 1.614009477e-005, -1.088916772e-008, 2.830952246e-012}, blow = {20843.88986, -35.97370046}, ahigh = {346757.312, -1957.85737, 8.807806149999999, -0.0004388362779999999, 1.024761895e-007, -1.110675026e-011, 4.98617942e-016}, bhigh = {25819.23944, -26.36885363}, R = 135.2597673026367);
          constant IdealGases.Common.DataRecord CLF(name = "CLF", MM = 0.0544514032, Hf = -1022948.71622335, H0 = 163597.3818210069, Tlimit = 1000, alow = {33522.1081, -368.83119, 4.27228862, 0.002549434508, -4.45683089e-006, 3.41385412e-009, -9.8386852e-013}, blow = {-5839.36969, 0.2318014199}, ahigh = {3045867.173, -9979.32826, 16.84162254, -0.007465850620000001, 2.336213612e-006, -3.36586546e-010, 1.763082415e-014}, bhigh = {54471.20800000001, -87.07987094000001}, R = 152.6952752615198);
          constant IdealGases.Common.DataRecord CLF3(name = "CLF3", MM = 0.0924482096, Hf = -1780456.330221889, H0 = 148498.8087860168, Tlimit = 1000, alow = {128517.5171, -2140.445721, 14.93096474, -0.00604247104, 3.71896861e-006, -8.0532011e-010, -7.98560053e-014}, blow = {-11384.5964, -55.94838353}, ahigh = {-229495.6235, -122.0741, 10.09124644, -3.64972475e-005, 8.05891075e-009, -9.23084995e-013, 4.27256949e-017}, bhigh = {-22828.40447, -25.11605479}, R = 89.93653891161999);
          constant IdealGases.Common.DataRecord CLF5(name = "CLF5", MM = 0.130445016, Hf = -1824523.521849237, H0 = 137455.5851179473, Tlimit = 1000, alow = {245970.3939, -4315.34807, 27.27005972, -0.01670193839, 1.421668208e-005, -6.426479740000001e-009, 1.183099799e-012}, blow = {-10714.23726, -126.7466837}, ahigh = {-426932.7449999999, -205.805739, 16.15465278, -6.21268818e-005, 1.376637959e-008, -1.581335709e-012, 7.33642249e-017}, bhigh = {-33614.666, -57.57531175}, R = 63.73928460402045);
          constant IdealGases.Common.DataRecord CLO(name = "CLO", MM = 0.0514524, Hf = 1975051.018028314, H0 = 185066.4886380422, Tlimit = 1000, alow = {-16872.68145, 257.3812247, 2.17584612, 0.006432061130000001, -8.568249500000001e-006, 5.764971250000001e-009, -1.545771209e-012}, blow = {9829.518979999999, 13.86010503}, ahigh = {409376.052, -1765.985112, 7.08790063, -0.001828450169, 7.1038181e-007, -1.209332942e-010, 7.07644104e-015}, bhigh = {21518.91784, -16.68645548}, R = 161.5954163459819);
          constant IdealGases.Common.DataRecord CLO2(name = "CLO2", MM = 0.06745180000000001, Hf = 1556667.131195906, H0 = 160123.9107036432, Tlimit = 1000, alow = {-11272.77696, 390.303203, -0.384301853, 0.02108677947, -2.793137755e-005, 1.841289286e-008, -4.83977915e-012}, blow = {9756.93367, 29.132627}, ahigh = {-163379.3802, -316.148067, 7.00978726, 0.0002837971144, -1.179925338e-007, 2.920383252e-011, -2.024030202e-015}, bhigh = {11849.46452, -10.91168827}, R = 123.2653835776065);
          constant IdealGases.Common.DataRecord CL2(name = "CL2", MM = 0.07090600000000001, Hf = 0, H0 = 129482.8364313316, Tlimit = 1000, alow = {34628.1517, -554.7126520000001, 6.20758937, -0.002989632078, 3.17302729e-006, -1.793629562e-009, 4.260043590000001e-013}, blow = {1534.069331, -9.438331107}, ahigh = {6092569.42, -19496.27662, 28.54535795, -0.01449968764, 4.46389077e-006, -6.35852586e-010, 3.32736029e-014}, bhigh = {121211.7724, -169.0778824}, R = 117.2604857134798);
          constant IdealGases.Common.DataRecord CL2O(name = "CL2O", MM = 0.08690539999999999, Hf = 909034.4213363037, H0 = 134577.1609129007, Tlimit = 1000, alow = {77988.554, -1182.537879, 9.52651466, -0.002596163176, 8.696781310000001e-007, 4.484098269999999e-010, -3.044511967e-013}, blow = {13767.29572, -24.84690718}, ahigh = {-127174.3461, -67.4196322, 7.05002245, -1.988084386e-005, 4.366209050000001e-009, -4.978580680000001e-013, 2.295679186e-017}, bhigh = {7387.2893, -8.797477254}, R = 95.67267396502405);
          constant IdealGases.Common.DataRecord Co(name = "Co", MM = 0.0589332, Hf = 7269953.099441402, H0 = 107914.8595358813, Tlimit = 1000, alow = {-2598.939184, 246.1989844, -0.610605837, 0.01393005772, -2.210012979e-005, 1.623755261e-008, -4.534904351e-012}, blow = {49846.1376, 22.57584199}, ahigh = {1381841.305, -3756.03668, 6.65713065, -0.001269246675, 1.464092329e-007, 6.57494657e-012, -1.102384178e-015}, bhigh = {74944.42909999999, -22.58500836}, R = 141.0829888755405);
          constant IdealGases.Common.DataRecord Coplus(name = "Coplus", MM = 0.05893265139999999, Hf = 20243503.02691455, H0 = 106757.5758181482, Tlimit = 1000, alow = {102849.416, -874.473126, 4.27950028, 0.002225857835, -7.45727457e-006, 7.27922195e-009, -2.347541963e-012}, blow = {147489.5959, -5.67901922}, ahigh = {2907386.174, -8619.705749999999, 11.88134934, -0.00351064742, 5.74800468e-007, -2.534065135e-011, 2.976607469e-016}, bhigh = {197741.9221, -60.9653344}, R = 141.0843022073838);
          constant IdealGases.Common.DataRecord Cominus(name = "Cominus", MM = 0.0589337486, Hf = 6081648.978968902, H0 = 107014.6079253475, Tlimit = 1000, alow = {34594.9376, -135.5041712, 1.116330898, 0.009042761829999999, -1.454296726e-005, 9.99494063e-009, -2.595633259e-012}, blow = {43370.3782, 12.70494975}, ahigh = {-574139.417, 1763.109207, 1.415561988, 0.000377686969, -7.53417998e-008, 7.99570154e-012, -3.49044e-016}, bhigh = {30834.93114, 16.34360353}, R = 141.0816755681481);
          constant IdealGases.Common.DataRecord Cr(name = "Cr", MM = 0.0519961, Hf = 7644419.485307553, H0 = 119190.2469608298, Tlimit = 1000, alow = {1335.658217, -21.02424026, 2.631908173, -0.000424626325, 7.43919416e-007, -6.76393163e-010, 2.507855625e-013}, blow = {47158.6664, 6.00542545}, ahigh = {-11202207.89, 34011.63690000001, -36.5706217, 0.02110296902, -5.51818014e-006, 7.17360171e-010, -3.505127367e-014}, bhigh = {-168899.344, 286.4481267}, R = 159.9056852340849);
          constant IdealGases.Common.DataRecord Crplus(name = "Crplus", MM = 0.0519955514, Hf = 20319944.67895959, H0 = 119191.5045255199, Tlimit = 1000, alow = {181.9187467, -2.188843517, 2.510676511, -2.706791825e-005, 3.76849263e-008, -2.736784742e-011, 8.115389931999999e-015}, blow = {126338.0825, 6.50627617}, ahigh = {3342330.79, -10642.61051, 15.57884307, -0.00770897148, 2.158300274e-006, -2.36810811e-010, 8.952805604e-015}, bhigh = {193299.767, -86.0435667}, R = 159.9073723833997);
          constant IdealGases.Common.DataRecord Crminus(name = "Crminus", MM = 0.0519966486, Hf = 6289317.423431017, H0 = 119188.989422676, Tlimit = 1000, alow = {0, 0, 2.5, 0, 0, 0, 0}, blow = {38586.2787, 6.56683537}, ahigh = {0, 0, 2.5, 0, 0, 0, 0}, bhigh = {38586.2787, 6.56683537}, R = 159.9039981203712);
          constant IdealGases.Common.DataRecord CrN(name = "CrN", MM = 0.0660028, Hf = 7651323.883229196, H0 = 132991.1609810493, Tlimit = 1000, alow = {-8239.129220000001, 300.8144202, 0.514787492, 0.01129636791, -1.515183504e-005, 1.010997229e-008, -2.68777575e-012}, blow = {58456.2812, 22.98028696}, ahigh = {1110672.49, -3690.47854, 8.59905668, -0.002125587223, 5.28235848e-007, -4.92113915e-011, 1.404331106e-015}, bhigh = {82548.04580000001, -27.99968411}, R = 125.9715042392141);
          constant IdealGases.Common.DataRecord CrO(name = "CrO", MM = 0.0679955, Hf = 2744024.501621431, H0 = 144849.5562206323, Tlimit = 1000, alow = {9373.33411, 136.9743818, 1.621443428, 0.008814095959999999, -1.23284536e-005, 8.497960940000001e-009, -2.315804197e-012}, blow = {20909.48871, 17.81935787}, ahigh = {1092367.332, -3749.75865, 9.00787021, -0.002545445236, 6.928051680000001e-007, -6.390831950000001e-011, 1.659741645e-015}, bhigh = {44470.9821, -29.42600453}, R = 122.2797391003817);
          constant IdealGases.Common.DataRecord CrO2(name = "CrO2", MM = 0.0839949, Hf = -1286307.085311132, H0 = 127315.6941671459, Tlimit = 1000, alow = {35486.299, -229.8628537, 2.286289393, 0.01616929338, -2.34519891e-005, 1.631365714e-008, -4.44426962e-012}, blow = {-12789.1284, 14.42954956}, ahigh = {-432710.914, 191.5584657, 7.18824737, -0.000569484619, 3.54636613e-007, -5.65512306e-011, 2.908946349e-015}, bhigh = {-17433.39545, -10.0642472}, R = 98.9878194985648);
          constant IdealGases.Common.DataRecord CrO3(name = "CrO3", MM = 0.09999429999999999, Hf = -3220554.411601461, H0 = 130408.1532647361, Tlimit = 1000, alow = {41830.2006, -505.934885, 4.43271567, 0.01995079387, -2.920597649e-005, 2.03933028e-008, -5.580086630000001e-012}, blow = {-37697.024, 0.8653827279999999}, ahigh = {-628331.401, 692.8158179999999, 8.971274599999999, 0.000682336564, -2.235048825e-007, 3.36678579e-011, -1.614026492e-015}, bhigh = {-47238.802, -19.45305345}, R = 83.1494595191926);
          constant IdealGases.Common.DataRecord CrO3minus(name = "CrO3minus", MM = 0.09999484859999999, Hf = -6328834.473579072, H0 = 134244.3054611516, Tlimit = 1000, alow = {187345.5703, -2300.722991, 13.43953022, -0.001663085846, -1.443973096e-006, 2.045898933e-009, -6.8375553e-013}, blow = {-66301.1158, -49.306442}, ahigh = {-649968.203, 452.201762, 9.9805759, -0.000421413673, 2.631277945e-007, -4.594543650000001e-011, 2.659097549e-015}, bhigh = {-83500.1973, -24.648109}, R = 83.14900333775795);
          constant IdealGases.Common.DataRecord Cs(name = "Cs", MM = 0.13290545, Hf = 575597.1632465035, H0 = 46630.35263038498, Tlimit = 1000, alow = {54.6658407, -0.827934604, 2.50494221, -1.49462069e-005, 2.425976774e-008, -2.013172322e-011, 6.704271991e-015}, blow = {8459.321389999999, 6.848825772}, ahigh = {6166040.899999999, -18961.75522, 24.83229903, -0.01251977234, 3.30901739e-006, -3.35401202e-010, 9.626500908000001e-015}, bhigh = {128511.1231, -152.2942188}, R = 62.559300615588);
          constant IdealGases.Common.DataRecord Csplus(name = "Csplus", MM = 0.1329049014, Hf = 3449096.483058675, H0 = 46630.54510945222, Tlimit = 1000, alow = {0, 0, 2.5, 0, 0, 0, 0}, blow = {54387.3782, 6.182757992}, ahigh = {0, 0, 2.5, 0, 0, 0, 0}, bhigh = {54387.3782, 6.182757992}, R = 62.55955884558522);
          constant IdealGases.Common.DataRecord Csminus(name = "Csminus", MM = 0.1329059986, Hf = 186577.1918589685, H0 = 46630.16015290675, Tlimit = 1000, alow = {0, 0, 2.5, 0, 0, 0, 0}, blow = {2237.029001, 6.182770382}, ahigh = {0, 0, 2.5, 0, 0, 0, 0}, bhigh = {2237.029001, 6.182770382}, R = 62.5590423877226);
          constant IdealGases.Common.DataRecord CsBO2(name = "CsBO2", MM = 0.17571525, Hf = -3909176.386227149, H0 = 82326.91243361063, Tlimit = 1000, alow = {41936.0958, -666.4237400000001, 8.134835880000001, 0.002902960302, -7.88146792e-007, -8.327694539999999e-010, 4.44171911e-013}, blow = {-81223.2007, -11.21622595}, ahigh = {89755.79890000001, -1656.134283, 11.16468934, -0.000447636699, 9.621091809999999e-008, -1.081470115e-011, 4.93844674e-016}, bhigh = {-76104.79670000001, -30.36616012}, R = 47.31787366207544);
          constant IdealGases.Common.DataRecord CsBr(name = "CsBr", MM = 0.21280945, Hf = -971897.0421661255, H0 = 48898.79185346327, Tlimit = 1000, alow = {1639.263647, -69.4747797, 4.86005872, -0.0008587780150000001, 1.37517755e-006, -9.89176698e-010, 2.895433432e-013}, blow = {-25895.50471, 4.46535569}, ahigh = {-152832.9963, 1259.712657, 1.704268408, 0.002776255318, -1.228389838e-006, 2.698111786e-010, -1.98353448e-014}, bhigh = {-33278.4203, 24.84116659}, R = 39.07003189942927);
          constant IdealGases.Common.DataRecord CsCL(name = "CsCL", MM = 0.16835845, Hf = -1438768.757968489, H0 = 60175.90444673255, Tlimit = 1000, alow = {-25381.65292, 297.2839326, 2.667813848, 0.00558327241, -8.52900509e-006, 6.595191390000001e-009, -1.989715823e-012}, blow = {-31892.5145, 15.11365709}, ahigh = {-3674923.48, 11861.52941, -10.63287842, 0.009804117919999999, -3.27779746e-006, 5.53423339e-010, -3.4012315e-014}, bhigh = {-104865.8658, 111.4096064}, R = 49.38553425741328);
          constant IdealGases.Common.DataRecord CsF(name = "CsF", MM = 0.1519038532, Hf = -2397667.184389764, H0 = 63494.87387460162, Tlimit = 1000, alow = {18436.85799, -404.240939, 6.38317886, -0.00467432337, 6.63013401e-006, -4.76090553e-009, 1.374569797e-012}, blow = {-43184.8959, -7.22659323}, ahigh = {-1850863.231, 5625.298800000001, -2.250022212, 0.00410924354, -1.250243837e-006, 1.941483152e-010, -1.071166179e-014}, bhigh = {-80798.2779, 51.3555945}, R = 54.73509608115722);
          constant IdealGases.Common.DataRecord CsH(name = "CsH", MM = 0.13391339, Hf = 865858.9406182609, H0 = 66058.4128293668, Tlimit = 1000, alow = {16205.44411, -70.8701628, 2.480847135, 0.00702198599, -1.007126309e-005, 7.09063903e-009, -1.950395735e-012}, blow = {13427.77328, 9.89424717}, ahigh = {-911214.6649999999, 3576.47275, -1.258058765, 0.00431485515, -1.407820502e-006, 2.154598897e-010, -1.254525606e-014}, bhigh = {-9158.717270000001, 39.0880003}, R = 62.08842894650043);
          constant IdealGases.Common.DataRecord CsI(name = "CsI", MM = 0.25980992, Hf = -586274.2769791085, H0 = 40607.09845105222, Tlimit = 1000, alow = {-4072.68565, 23.30073667, 4.38727769, 0.000341208429, -2.175588153e-007, 8.906786999999999e-011, -2.182639837e-015}, blow = {-19787.66885, 8.074657820000001}, ahigh = {4511259.05, -13417.06362, 19.80984712, -0.008359855550000002, 2.348201245e-006, -2.874285248e-010, 1.20808775e-014}, bhigh = {65734.3913, -102.1032592}, R = 32.0021344835486);
          constant IdealGases.Common.DataRecord CsLi(name = "CsLi", MM = 0.13984645, Hf = 1159459.707414811, H0 = 73946.27464622805, Tlimit = 1000, alow = {1368.709568, -74.5129706, 4.84562093, -0.0005573667210000001, 4.95332676e-007, 4.36906891e-010, -4.60814577e-013}, blow = {18505.77392, 2.112012017}, ahigh = {7481630.23, -28852.97839, 46.3110499, -0.02784288108, 8.78741349e-006, -1.264901197e-009, 6.72018628e-014}, bhigh = {194513.7819, -285.7846194}, R = 59.45429433496524);
          constant IdealGases.Common.DataRecord CsNO2(name = "CsNO2", MM = 0.17891095, Hf = -1175667.716257725, H0 = 89319.13893476056, Tlimit = 1000, alow = {-71060.1044, 1272.448257, -2.447362349, 0.03064441948, -3.69922594e-005, 2.259121812e-008, -5.58063591e-012}, blow = {-33133.7092, 49.54693336}, ahigh = {-163350.3154, -846.8879579999999, 10.62853461, -0.0002508799215, 5.54125577e-008, -6.35559195e-012, 2.946880031e-016}, bhigh = {-24030.40739, -24.43488888}, R = 46.4726837569193);
          constant IdealGases.Common.DataRecord CsNO3(name = "CsNO3", MM = 0.19491035, Hf = -1634014.76627588, H0 = 84980.65905684332, Tlimit = 1000, alow = {-26772.19779, 901.169269, -3.26534082, 0.04298377699999999, -5.38675138e-005, 3.38452752e-008, -8.56522366e-012}, blow = {-44053.0414, 51.18862689}, ahigh = {-314751.7676, -1367.011655, 14.01012848, -0.000401864713, 8.8536147e-008, -1.013466457e-011, 4.69175873e-016}, bhigh = {-35490.2667, -45.00692791}, R = 42.65792965843015);
          constant IdealGases.Common.DataRecord CsNa(name = "CsNa", MM = 0.15589522, Hf = 807640.7281762712, H0 = 68649.50060688198, Tlimit = 1000, alow = {25217.38609, -429.5362489999999, 7.22309474, -0.008460463000000001, 1.443905157e-005, -1.143624855e-008, 3.18372193e-012}, blow = {15790.99265, -8.838790299999999}, ahigh = {3879556.79, -17801.67288, 34.913558, -0.02320080793, 7.861975840000001e-006, -1.171011257e-009, 6.323927369999999e-014}, bhigh = {119577.6556, -200.1754433}, R = 53.33371991777555);
          constant IdealGases.Common.DataRecord CsO(name = "CsO", MM = 0.14890485, Hf = 252425.0754760506, H0 = 66049.68206206849, Tlimit = 1000, alow = {8683.95881, 425.027817, -3.4840553, 0.0380196983, -6.651927559999999e-005, 5.17347219e-008, -1.512567066e-011}, blow = {1969.690015, 42.4079614}, ahigh = {837554.4350000001, -2418.205772, 8.908065349999999, -0.00337818004, 1.312755917e-006, -2.150508925e-010, 1.21913898e-014}, bhigh = {18094.93447, -24.6085858}, R = 55.83748279522124);
          constant IdealGases.Common.DataRecord CsOH(name = "CsOH", MM = 0.14991279, Hf = -1707659.499899908, H0 = 78943.82460629277, Tlimit = 1000, alow = {9386.960789999999, -500.935426, 8.30135198, -0.00323559684, 2.612406777e-006, -4.95061341e-010, -1.038364927e-013}, blow = {-30257.22427, -17.42194837}, ahigh = {896717.113, -2323.978587, 7.95964487, 0.0001101149275, -6.466013969999999e-008, 1.045968201e-011, -5.82251417e-016}, bhigh = {-17362.58234, -18.64239624}, R = 55.46205897442107);
          constant IdealGases.Common.DataRecord CsRb(name = "CsRb", MM = 0.21837325, Hf = 510489.8699817858, H0 = 50244.85370804345, Tlimit = 1000, alow = {-4910.92268, -9.20901555, 5.19337789, -0.00439024699, 1.254790436e-005, -1.414878514e-008, 5.10144131e-012}, blow = {12004.98171, 5.36154293}, ahigh = {-13286933.78, 34145.7947, -23.73696406, 0.0070704654, 3.86345097e-007, -3.158132986e-010, 2.667747207e-014}, bhigh = {-212400.8997, 223.9970251}, R = 38.07459017988696);
          constant IdealGases.Common.DataRecord Cs2(name = "Cs2", MM = 0.2658109, Hf = 411587.083148208, H0 = 41492.40305796338, Tlimit = 1000, alow = {-46741.5873, 595.201951, 1.895333975, 0.00408206581, 2.531487119e-006, -9.085139030000001e-009, 4.29017993e-012}, blow = {8857.282570000001, 23.91592727}, ahigh = {-25927395.9, 75398.1891, -74.840868, 0.0369286679, -8.08249724e-006, 8.171850729999999e-010, -3.0892859e-014}, bhigh = {-471504.572, 586.093375}, R = 31.279650307794);
          constant IdealGases.Common.DataRecord Cs2Br2(name = "Cs2Br2", MM = 0.4256189, Hf = -1329426.08751632, H0 = 51955.23507062304, Tlimit = 1000, alow = {-6321.28518, -12.13153529, 10.04967112, -0.0001102100044, 1.365293456e-007, -8.85695707e-011, 2.339447704e-014}, blow = {-70997.71709999999, -7.53942479}, ahigh = {-7545.84665, -0.2087093009, 10.00018042, -8.03426073e-008, 1.923604258e-011, -2.34469814e-015, 1.139365834e-019}, bhigh = {-71058.8354, -7.25179304}, R = 19.53501594971464);
          constant IdealGases.Common.DataRecord Cs2CO3(name = "Cs2CO3", MM = 0.3258198, Hf = -2475134.841406201, H0 = 65002.37554623753, Tlimit = 1000, alow = {-46731.3, 943.6549060000001, -0.2374670436, 0.0416848461, -5.09437221e-005, 3.132078394e-008, -7.78042167e-012}, blow = {-103916.2411, 40.55853980000001}, ahigh = {-303714.9985, -1517.0784, 17.11941656, -0.000444970012, 9.798325240000001e-008, -1.121256063e-011, 5.189734780000001e-016}, bhigh = {-94204.1477, -56.8200626}, R = 25.51862102917011);
          constant IdealGases.Common.DataRecord Cs2CL2(name = "Cs2CL2", MM = 0.3367169, Hf = -1914541.260031796, H0 = 62202.73470087186, Tlimit = 1000, alow = {-10779.56357, -53.4610695, 10.21710683, -0.000478968703, 5.90894653e-007, -3.82141652e-010, 1.006995985e-013}, blow = {-80295.1517, -12.07856486}, ahigh = {-16251.43966, -0.900613735, 10.00077173, -3.41619408e-007, 8.14496808e-011, -9.897475279999999e-015, 4.79833081e-019}, bhigh = {-80565.13219999999, -10.81977337}, R = 24.69276712870664);
          constant IdealGases.Common.DataRecord Cs2F2(name = "Cs2F2", MM = 0.3038077064, Hf = -2935601.985111462, H0 = 63818.65762968019, Tlimit = 1000, alow = {-9942.67748, -253.6723355, 11.00684073, -0.002185640152, 2.664791874e-006, -1.708229209e-009, 4.47115198e-013}, blow = {-109058.2601, -20.00699602}, ahigh = {-36779.4642, -4.35575348, 10.00366687, -1.603814186e-006, 3.79146974e-010, -4.57868743e-014, 2.209385787e-018}, bhigh = {-110345.6448, -14.15086452}, R = 27.36754804057861);
          constant IdealGases.Common.DataRecord Cs2I2(name = "Cs2I2", MM = 0.5196198399999999, Hf = -873779.3710879093, H0 = 43464.2815024153, Tlimit = 1000, alow = {-4449.28697, -5.32179366, 10.02183919, -4.85337485e-005, 6.019338399999999e-008, -3.90822639e-011, 1.032981014e-014}, blow = {-57578.2338, -5.27126709}, ahigh = {-4983.37601, -0.09461179879999999, 10.00008229, -3.68096204e-008, 8.844660910000001e-012, -1.081178562e-015, 5.26614198e-020}, bhigh = {-57605.01040000001, -5.14486489}, R = 16.0010672417743);
          constant IdealGases.Common.DataRecord Cs2O(name = "Cs2O", MM = 0.2818103, Hf = -506920.240317689, H0 = 49997.65799901566, Tlimit = 1000, alow = {19105.33804, -496.218153, 8.7596063, -0.00351162279, 4.01647983e-006, -2.450899853e-009, 6.17240489e-013}, blow = {-16776.64956, -11.59258822}, ahigh = {-41191.4741, -10.79255391, 7.00850047, -3.54295578e-006, 8.08169912e-010, -9.499419930000001e-014, 4.48912254e-018}, bhigh = {-19343.7785, -1.20397045}, R = 29.503790315684);
          constant IdealGases.Common.DataRecord Cs2Oplus(name = "Cs2Oplus", MM = 0.2818097514, Hf = 1006707.271805216, H0 = 51267.65815670069, Tlimit = 1000, alow = {683.705609, -243.5575007, 7.78303747, -0.001414552725, 1.470799739e-006, -8.218821330000001e-010, 1.911749489e-013}, blow = {33241.9487, -4.75042356}, ahigh = {-31530.85844, -5.77155903, 7.00448263, -1.848241331e-006, 4.1808823e-010, -4.88237627e-014, 2.295419936e-018}, bhigh = {31962.2731, -0.0543435069}, R = 29.50384775081279);
          constant IdealGases.Common.DataRecord Cs2O2(name = "Cs2O2", MM = 0.2978097, Hf = -829620.7410302619, H0 = 58515.54197193711, Tlimit = 1000, alow = {22745.0518, -566.7912570000001, 10.01728332, 0.002547081016, -4.90921669e-006, 3.84878891e-009, -1.124566008e-012}, blow = {-29473.3436, -18.41275291}, ahigh = {-119003.2094, -92.6107526, 10.06915777, -2.766227147e-005, 6.11075296e-009, -7.003595480000001e-013, 3.24374252e-017}, bhigh = {-32556.1074, -17.24868948}, R = 27.91874139761062);
          constant IdealGases.Common.DataRecord Cs2O2H2(name = "Cs2O2H2", MM = 0.29982558, Hf = -2177932.916864532, H0 = 79837.59424396011, Tlimit = 1000, alow = {-10859.81041, -688.306842, 16.43543354, -0.00403922058, 2.331117156e-006, 8.43965163e-010, -6.85930973e-013}, blow = {-79394.7623, -49.0619681}, ahigh = {1802857.941, -4656.35475, 16.93566597, 0.0002117402535, -1.272249096e-007, 2.066555839e-011, -1.152390667e-015}, bhigh = {-51908.1369, -58.4971916}, R = 27.73102948721054);
          constant IdealGases.Common.DataRecord Cs2SO4(name = "Cs2SO4", MM = 0.3618735, Hf = -3088515.279510658, H0 = 66288.81639578473, Tlimit = 1000, alow = {61536.3787, -638.137433, 6.89477209, 0.0396018291, -5.529862010000001e-005, 3.73633252e-008, -9.973241829999999e-012}, blow = {-133976.031, -1.284432588}, ahigh = {-522339.752, -944.415525, 19.70476453, -0.000282122599, 6.240792279999999e-008, -7.16346407e-012, 3.32266719e-016}, bhigh = {-136417.9983, -68.20406795}, R = 22.97618366639171);
          constant IdealGases.Common.DataRecord Cu(name = "Cu", MM = 0.06354600000000001, Hf = 5309539.54615554, H0 = 97526.64211752116, Tlimit = 1000, alow = {77.1313315, -1.169236206, 2.506987803, -2.116434879e-005, 3.44171471e-008, -2.862608999e-011, 9.559250991000001e-015}, blow = {39839.8121, 5.73081322}, ahigh = {2308090.411, -8503.261, 14.67859102, -0.00846713652, 2.887821016e-006, -4.27065918e-010, 2.304265084e-014}, bhigh = {92075.3562, -78.5470156}, R = 130.8417839045731);
          constant IdealGases.Common.DataRecord Cuplus(name = "Cuplus", MM = 0.06354545139999999, Hf = 17138594.56508637, H0 = 97527.48408362082, Tlimit = 1000, alow = {-0.002452340093, 2.606893531e-005, 2.49999989, 2.351922485e-010, -2.669362382e-013, 1.510315123e-016, -3.278224814e-020}, blow = {130240.0621, 5.07594077}, ahigh = {-2181443.016, 7217.85819, -6.94115475, 0.00620824892, -2.139340497e-006, 3.56643144e-010, -2.081198501e-014}, bhigh = {85164.56659999999, 71.16800670000001}, R = 130.842913486645);
          constant IdealGases.Common.DataRecord Cuminus(name = "Cuminus", MM = 0.0635465486, Hf = 3347445.812344244, H0 = 97525.80016595896, Tlimit = 1000, alow = {0, 0, 2.5, 0, 0, 0, 0}, blow = {24838.64954, 5.07596603}, ahigh = {0, 0, 2.5, 0, 0, 0, 0}, bhigh = {24838.64954, 5.07596603}, R = 130.8406543420047);
          constant IdealGases.Common.DataRecord CuCL(name = "CuCL", MM = 0.09899899999999999, Hf = 920110.3041444863, H0 = 95669.40070101718, Tlimit = 1000, alow = {14698.47736, -339.316725, 5.72345959, -0.002417296132, 2.858019864e-006, -1.757900548e-009, 4.455047290000001e-013}, blow = {11317.14015, -4.527352594}, ahigh = {-25771.2241, -7.10634606, 4.50562318, 5.39454827e-005, 5.38641796e-010, -6.348482770000001e-014, 3.006622062e-018}, bhigh = {9566.192870000001, 2.680067564}, R = 83.98541399408076);
          constant IdealGases.Common.DataRecord CuF(name = "CuF", MM = 0.08254440320000001, Hf = -152039.3813932136, H0 = 110038.847552053, Tlimit = 1000, alow = {37613.8541, -546.8104900000001, 5.83235172, -0.001695312068, 1.21815088e-006, -3.71328528e-010, 2.002276589e-014}, blow = {58.6514072, -7.15693009}, ahigh = {509415.483, -1415.00987, 5.63234938, -0.000162912841, -1.156611499e-007, 5.06603408e-011, -4.15320511e-015}, bhigh = {6305.506600000001, -7.40777361}, R = 100.7272652981032);
          constant IdealGases.Common.DataRecord CuF2(name = "CuF2", MM = 0.1015428064, Hf = -2628842.056506328, H0 = 118727.3665897026, Tlimit = 1000, alow = {65733.1128, -896.243084, 7.91756094, 0.001345667904, -4.16240001e-006, 3.71618109e-009, -1.155537597e-012}, blow = {-29168.87325, -15.87076153}, ahigh = {-1650355.082, 3775.36212, 3.88911655, 0.000682191264, 1.875979026e-007, -5.67736811e-011, 3.82324873e-015}, bhigh = {-59533.717, 15.22659664}, R = 81.88144778318831);
          constant IdealGases.Common.DataRecord CuO(name = "CuO", MM = 0.0795454, Hf = 3850254.068745647, H0 = 122583.317199989, Tlimit = 1000, alow = {4689.76224, -118.4808464, 4.56615524, 0.000430905802, -8.309886399999999e-007, 6.94663802e-010, -2.108693366e-013}, blow = {36151.9068, 1.733847344}, ahigh = {358228.017, -913.9366409999999, 5.13689867, 6.240577240000001e-005, -1.558613495e-007, 5.23680312e-011, -4.026745619999999e-015}, bhigh = {41507.9286, -2.63407096}, R = 104.5248625313343);
          constant IdealGases.Common.DataRecord Cu2(name = "Cu2", MM = 0.127092, Hf = 3818808.422245302, H0 = 78133.58826676737, Tlimit = 1000, alow = {-852.918348, -97.2004493, 4.8822337, -0.00073713694, 1.000575401e-006, -6.391989439999999e-010, 1.668655797e-013}, blow = {57493.0702, 1.105391325}, ahigh = {-86993.995, 320.910387, 3.97380288, 0.000508080967, -1.707470385e-007, 3.21910819e-011, -1.958830868e-015}, bhigh = {55060.9019, 6.91450217}, R = 65.42089195228652);
          constant IdealGases.Common.DataRecord Cu3CL3(name = "Cu3CL3", MM = 0.296997, Hf = -870614.854695502, H0 = 96713.73784920386, Tlimit = 1000, alow = {4487.88532, -873.6644439999999, 19.36189091, -0.00713744786, 8.56045487e-006, -5.419978740000001e-009, 1.40517535e-012}, blow = {-31626.8996, -59.76797945000001}, ahigh = {-91885.59679999999, -15.7193643, 16.0129417, -5.57299237e-006, 1.302748507e-009, -1.560175322e-013, 7.48082757e-018}, bhigh = {-36087.6074, -40.13211395}, R = 27.99513799802692);
          constant IdealGases.Common.DataRecord D(name = "D", MM = 0.002014102, Hf = 110083912.3341321, H0 = 3077017.946459514, Tlimit = 1000, alow = {0, 0, 2.5, 0, 0, 0, 0}, blow = {25921.287, 0.591714338}, ahigh = {60.50019210000001, -0.1810766064, 2.500210817, -1.220711706e-007, 3.71517217e-011, -5.66068021e-015, 3.393920393e-019}, bhigh = {25922.43752, 0.590212537}, R = 4128.128565484767);
          constant IdealGases.Common.DataRecord Dplus(name = "Dplus", MM = 0.0020135534, Hf = 764978136.6612874, H0 = 3077856.291270944, Tlimit = 1000, alow = {0, 0, 2.5, 0, 0, 0, 0}, blow = {184512.0037, -0.1018414521}, ahigh = {0, 0, 2.5, 0, 0, 0, 0}, bhigh = {184512.0037, -0.1018414521}, R = 4129.253289234842);
          constant IdealGases.Common.DataRecord Dminus(name = "Dminus", MM = 0.0020146506, Hf = 70857312.92562591, H0 = 3076180.058219525, Tlimit = 1000, alow = {0, 0, 2.5, 0, 0, 0, 0}, blow = {16423.73393, -0.1010243437}, ahigh = {0, 0, 2.5, 0, 0, 0, 0}, bhigh = {16423.73393, -0.1010243437}, R = 4127.004454271128);
          constant IdealGases.Common.DataRecord DBr(name = "DBr", MM = 0.08191810200000001, Hf = -452116.1391166021, H0 = 105814.2680112388, Tlimit = 1000, alow = {-19182.82458, 202.0175399, 3.044629038, -0.00148815745, 6.8708978e-006, -6.59116028e-009, 2.093769216e-012}, blow = {-6560.0744, 8.008690039999999}, ahigh = {665400.044, -2594.092228, 6.8858794, -0.001103284901, 2.894201105e-007, -3.152037514e-011, 1.011011776e-015}, bhigh = {10378.1314, -19.73703653}, R = 101.4973711182908);
          constant IdealGases.Common.DataRecord DCL(name = "DCL", MM = 0.037467102, Hf = -2496777.119297884, H0 = 231165.5702648153, Tlimit = 1000, alow = {10464.21033, -231.3813446, 5.42069326, -0.007508012650000001, 1.398672495e-005, -1.066249569e-008, 3.011167635e-012}, blow = {-11284.03374, -6.711842349}, ahigh = {411728.4899999999, -1764.535217, 5.6688771, -0.000347596417, 5.88268803e-008, 3.76025313e-013, -5.16460056e-016}, bhigh = {-1541.258092, -12.75659404}, R = 221.9139339893436);
          constant IdealGases.Common.DataRecord DF(name = "DF", MM = 0.0210125052, Hf = -13145866.87169505, H0 = 411093.4854164842, Tlimit = 1000, alow = {57213.7075, -731.17338, 7.22087087, -0.00942396935, 1.208025139e-005, -6.94181269e-009, 1.538525476e-012}, blow = {-30692.30024, -19.32760992}, ahigh = {800117.2800000001, -2438.386832, 5.62066445, -0.0002020416838, 1.714418979e-008, 2.697462563e-012, -2.88829741e-016}, bhigh = {-18574.47029, -14.73004444}, R = 395.6916093945809);
          constant IdealGases.Common.DataRecord DOCL(name = "DOCL", MM = 0.053466502, Hf = -1487635.865910959, H0 = 193108.0884999733, Tlimit = 1000, alow = {68528.3429, -767.344455, 5.93397014, 0.002868820667, -5.207166370000001e-006, 4.927572739999999e-009, -1.709526276e-012}, blow = {-6824.03505, -7.757833983}, ahigh = {604306.633, -2646.500381, 8.61247421, -0.0005532268239999999, 1.086845232e-007, -1.13738129e-011, 4.90429552e-016}, bhigh = {4845.79518, -25.88861801}, R = 155.5080599811822);
          constant IdealGases.Common.DataRecord DO2(name = "DO2", MM = 0.034012902, Hf = 190731.8581637051, H0 = 295927.2337303062, Tlimit = 1000, alow = {-21114.79735, 602.337175, -1.294877674, 0.0181294797, -2.161807666e-005, 1.391729127e-008, -3.69774028e-012}, blow = {-2976.943856, 32.7283448}, ahigh = {-1267224.927, 2799.947016, 2.325174609, 0.00272632507, -6.31450732e-007, 6.7292892e-011, -2.765192818e-015}, bhigh = {-19594.11733, 17.89922833}, R = 244.4505323303493);
          constant IdealGases.Common.DataRecord DO2minus(name = "DO2minus", MM = 0.0340134506, Hf = -3081002.019830355, H0 = 296350.11509241, Tlimit = 1000, alow = {104870.5051, -890.6236769999999, 5.66655356, 0.001904603784, -1.21526321e-006, 6.57751333e-010, -2.147376147e-013}, blow = {-8942.37725, -7.7972192}, ahigh = {552766.192, -2796.895783, 8.772206669999999, -0.0006291116630000001, 1.272331622e-007, -1.364440687e-011, 6.00519944e-016}, bhigh = {2512.066767, -28.90020381}, R = 244.4465896088767);
          constant IdealGases.Common.DataRecord D2(name = "D2", MM = 0.004028204, Hf = 0, H0 = 2127276.324634999, Tlimit = 1000, alow = {21257.90482, -299.6945907, 5.13031498, -0.004172970890000001, 5.014345719999999e-006, -2.126389969e-009, 2.386536969e-013}, blow = {394.49859, -11.64191209}, ahigh = {821516.856, -2365.623159, 5.34297451, 6.92814599e-005, -8.52367102e-008, 2.456447415e-011, -1.960597698e-015}, bhigh = {14342.14587, -17.12600356}, R = 2064.064282742384);
          constant IdealGases.Common.DataRecord D2plus(name = "D2plus", MM = 0.004027655400000001, Hf = 372069647.517511, H0 = 2147899.743359374, Tlimit = 1000, alow = {-96409.59090000001, 1243.052385, -2.557714366, 0.01343064234, -1.285600289e-005, 6.46342167e-009, -1.337616868e-012}, blow = {173096.6171, 33.5631492}, ahigh = {925595.135, -4505.21994, 11.03203365, -0.00470608903, 1.83806846e-006, -3.135924623e-010, 1.857684975e-014}, bhigh = {205829.889, -52.8391224}, R = 2064.345425380731);
          constant IdealGases.Common.DataRecord D2minus(name = "D2minus", MM = 0.004028752599999999, Hf = 58370578.77418436, H0 = 2162915.389741232, Tlimit = 1000, alow = {-4365.33206, 311.2987057, 0.548195003, 0.009956988109999999, -1.167000612e-005, 6.97565641e-009, -1.682718179e-012}, blow = {25978.97625, 14.42223161}, ahigh = {-57988.05190000001, -312.2959355, 4.73038828, 5.6959008e-005, 2.01897543e-008, -2.311492448e-012, 1.070266527e-016}, bhigh = {28512.00896, -9.15752792}, R = 2063.783216671086);
          constant IdealGases.Common.DataRecord D2O(name = "D2O", MM = 0.020027604, Hf = -12443325.72183872, H0 = 497314.4565870186, Tlimit = 1000, alow = {6958.27847, -12.80889437, 3.59587887, 0.001502093683, 3.59467505e-007, 5.3404172e-010, -5.18194127e-013}, blow = {-31019.44566, 2.895556576}, ahigh = {1544193.253, -5474.238899999999, 10.17542424, -0.0009619415540000001, 2.036545675e-007, -2.050566442e-011, 8.510770689999999e-016}, bhigh = {2983.24898, -44.6501157}, R = 415.1506091292798);
          constant IdealGases.Common.DataRecord D2O2(name = "D2O2", MM = 0.036027004, Hf = -4005328.891628069, H0 = 321026.9441222478, Tlimit = 1000, alow = {29577.11324, -68.9303757, 2.043905473, 0.01570281822, -1.935478714e-005, 1.384941336e-008, -4.1041849e-012}, blow = {-18025.02679, 13.47156482}, ahigh = {1147867.936, -5225.76093, 13.11088701, -0.001179811896, 2.729336904e-007, -2.961433535e-011, 1.310306129e-015}, bhigh = {12195.80532, -56.905382}, R = 230.7844415816536);
          constant IdealGases.Common.DataRecord D2S(name = "D2S", MM = 0.036093204, Hf = -665126.5983479882, H0 = 279513.9217898196, Tlimit = 1000, alow = {3988.38648, -66.2079256, 4.39335445, -0.002325113084, 1.189503465e-005, -1.146524521e-008, 3.62619045e-012}, blow = {-3787.38555, 0.9238754374999999}, ahigh = {423581.463, -2823.776231, 8.96282184, -0.000646923123, 1.634615644e-007, -1.797991376e-011, 8.16093957e-016}, bhigh = {12046.03509, -31.91047376}, R = 230.3611505368158);
          constant IdealGases.Common.DataRecord eminus(name = "eminus", MM = 5.48579903e-007, Hf = 0, H0 = 11297220270.20738, Tlimit = 1000, alow = {0, 0, 2.5, 0, 0, 0, 0}, blow = {-745.375, -11.72081224}, ahigh = {0, 0, 2.5, 0, 0, 0, 0}, bhigh = {-745.375, -11.72081224}, R = 15156355.44527048);
          constant IdealGases.Common.DataRecord F(name = "F", MM = 0.0189984032, Hf = 4178245.885422623, H0 = 343105.677428722, Tlimit = 1000, alow = {1137.409088, -145.3392797, 4.07740361, -0.004303360139999999, 5.72889774e-006, -3.8193129e-009, 1.018322509e-012}, blow = {9311.110120000001, -3.55898265}, ahigh = {14735.06226, 81.4992736, 2.444371819, 2.120210026e-005, -4.54691862e-009, 5.10952873e-013, -2.333894647e-017}, bhigh = {8388.37465, 5.47871064}, R = 437.6405697085111);
          constant IdealGases.Common.DataRecord Fplus(name = "Fplus", MM = 0.0189978546, Hf = 93000834.52581009, H0 = 353236.3070091083, Tlimit = 1000, alow = {-38716.8019, 321.881566, 2.200920452, -0.0002455492688, 7.85835506e-007, -6.43598792e-010, 1.839793564e-013}, blow = {209883.0937, 7.81699924}, ahigh = {16496.35664, 133.7351478, 2.332522942, 0.0001215277877, -4.8010377e-008, 9.027225149999999e-012, -5.47066494e-016}, bhigh = {211074.5327, 6.62581709}, R = 437.653207431117);
          constant IdealGases.Common.DataRecord Fminus(name = "Fminus", MM = 0.0189989518, Hf = -13426639.25280341, H0 = 326198.4169042421, Tlimit = 1000, alow = {0, 0, 2.5, 0, 0, 0, 0}, blow = {-31425.72443, 3.26488271}, ahigh = {0, 0, 2.5, 0, 0, 0, 0}, bhigh = {-31425.72443, 3.26488271}, R = 437.6279327157407);
          constant IdealGases.Common.DataRecord FCN(name = "FCN", MM = 0.0450158032, Hf = 762573.3977795602, H0 = 225008.9808460865, Tlimit = 1000, alow = {39844.5412, -698.865536, 7.05883966, -0.001404863382, 3.96465251e-006, -3.045609294e-009, 7.9230459e-013}, blow = {6172.627570000001, -15.05643174}, ahigh = {398187.355, -2302.971079, 9.027081389999999, -0.000522871026, 1.191059272e-007, -1.280287231e-011, 5.73380719e-016}, bhigh = {15884.49451, -29.84971982}, R = 184.7011806733685);
          constant IdealGases.Common.DataRecord FCO(name = "FCO", MM = 0.0470085032, Hf = -3816716.929629871, H0 = 220980.3395739688, Tlimit = 1000, alow = {11326.62744, -53.979185, 2.966927601, 0.00755995935, -6.21177358e-006, 2.40378155e-009, -3.36950775e-013}, blow = {-22403.69579, 10.92652142}, ahigh = {-60858.5158, -1022.397533, 7.52732256, -0.0001057942328, -1.365311093e-009, 2.484612871e-012, -9.9979168e-017}, bhigh = {-18050.98416, -16.30331278}, R = 176.8716601042511);
          constant IdealGases.Common.DataRecord FO(name = "FO", MM = 0.0349978032, Hf = 3114826.932908749, H0 = 268245.4080432112, Tlimit = 1000, alow = {-39121.8244, 796.703694, -1.634777767, 0.01601810071, -2.095210771e-005, 1.382740666e-008, -3.66452495e-012}, blow = {8375.525229999999, 33.83325720000001}, ahigh = {-1597940.503, 4377.376380000001, -0.489750764, 0.002682336321, -6.90080485e-007, 7.24647968e-011, -2.726912632e-015}, bhigh = {-16442.48211, 35.5992361}, R = 237.5712541866057);
          constant IdealGases.Common.DataRecord FO2_FOO(name = "FO2_FOO", MM = 0.0509972032, Hf = 498066.529264099, H0 = 220714.2998775274, Tlimit = 1000, alow = {5821.564, -234.7363967, 5.43733876, 0.002165855252, 3.67147219e-007, -2.071530827e-009, 9.43106685e-013}, blow = {2694.856027, -1.168202057}, ahigh = {-1213166.895, 2493.397189, 4.46506574, 0.0009416104040000001, -6.426472259999999e-008, -1.085643277e-011, 1.216995394e-015}, bhigh = {-15968.00286, 8.65519318}, R = 163.0378036103753);
          constant IdealGases.Common.DataRecord FO2_OFO(name = "FO2_OFO", MM = 0.0509972032, Hf = 7423936.534621569, H0 = 206646.2734960336, Tlimit = 1000, alow = {-77341.064, 1663.800209, -9.61281516, 0.04989156870000001, -6.94621713e-005, 4.7077878e-008, -1.258890729e-011}, blow = {36970.9273, 77.9594304}, ahigh = {-621186.7340000001, 936.947459, 6.40155875, 0.0002136110846, -4.33797765e-008, 4.66731224e-012, -2.059767432e-016}, bhigh = {36383.5727, -6.15026433}, R = 163.0378036103753);
          constant IdealGases.Common.DataRecord F2(name = "F2", MM = 0.0379968064, Hf = 0, H0 = 232259.1511269747, Tlimit = 1000, alow = {10181.76308, 22.74241183, 1.97135304, 0.008151604010000001, -1.14896009e-005, 7.95865253e-009, -2.167079526e-012}, blow = {-958.6943, 11.30600296}, ahigh = {-2941167.79, 9456.5977, -7.73861615, 0.00764471299, -2.241007605e-006, 2.915845236e-010, -1.425033974e-014}, bhigh = {-60710.0561, 84.23835080000001}, R = 218.8202848542556);
          constant IdealGases.Common.DataRecord F2O(name = "F2O", MM = 0.0539962064, Hf = 453735.5794684124, H0 = 202079.4001557858, Tlimit = 1000, alow = {30829.19995, -229.9506259, 2.603805825, 0.01586111264, -2.345633734e-005, 1.679619314e-008, -4.70356033e-012}, blow = {3055.185332, 10.50933022}, ahigh = {-188537.4518, -210.0729689, 7.15123916, 0.0001327687906, 1.804705706e-008, -1.416973671e-012, 6.489389390000001e-017}, bhigh = {1449.129965, -12.57858336}, R = 153.9825212609751);
          constant IdealGases.Common.DataRecord F2O2(name = "F2O2", MM = 0.06999560639999999, Hf = 274302.9311051158, H0 = 196847.3981246915, Tlimit = 1000, alow = {56122.70460000001, -998.595223, 8.98064939, 0.00642190087, -9.88851602e-006, 6.85806101e-009, -1.842137493e-012}, blow = {5298.71259, -22.39292195}, ahigh = {-219056.3532, -391.092081, 10.29107144, -0.0001163345202, 2.570922521e-008, -2.949199983e-012, 1.367386916e-016}, bhigh = {827.8020939999999, -27.56752555}, R = 118.7856270933028);
          constant IdealGases.Common.DataRecord FS2F(name = "FS2F", MM = 0.1021268064, Hf = -3294286.895472725, H0 = 142913.5063994325, Tlimit = 1000, alow = {114446.228, -1853.247564, 13.94098518, -0.00404368937, 1.370970316e-006, 6.7119031e-010, -4.624954960000001e-013}, blow = {-33510.6833, -48.49215357}, ahigh = {-207709.7921, -109.4539831, 10.08136946, -3.23953791e-005, 7.12522527e-009, -8.13478741e-013, 3.75502241e-017}, bhigh = {-43494.4606, -23.46916058}, R = 81.41321845936034);
          constant IdealGases.Common.DataRecord Fe(name = "Fe", MM = 0.055845, Hf = 7439717.07404423, H0 = 122666.7920136091, Tlimit = 1000, alow = {67908.2266, -1197.218407, 9.84339331, -0.01652324828, 1.917939959e-005, -1.149825371e-008, 2.832773807e-012}, blow = {54669.9594, -33.8394626}, ahigh = {-1954923.682, 6737.161099999999, -5.48641097, 0.004378803450000001, -1.116286672e-006, 1.544348856e-010, -8.023578182e-015}, bhigh = {7137.37006, 65.0497986}, R = 148.8848061599069);
          constant IdealGases.Common.DataRecord Feplus(name = "Feplus", MM = 0.0558444514, Hf = 21205649.25094779, H0 = 124197.6387290645, Tlimit = 1000, alow = {-56912.3162, 184.713439, 4.19697212, -0.00597827597, 1.054267912e-005, -8.059804319999999e-009, 2.256925874e-012}, blow = {140120.6571, -0.360254258}, ahigh = {-817645.009, 1925.359408, 1.717387154, 0.000338533898, -9.813533120000001e-008, 2.228179208e-011, -1.483964439e-015}, bhigh = {128635.2466, 15.00256262}, R = 148.8862687618774);
          constant IdealGases.Common.DataRecord Fe_CO_5(name = "Fe_CO_5", MM = 0.1958955, Hf = -3715501.377009681, H0 = 169199.3894704064, Tlimit = 1000, alow = {379780.571, -7285.92816, 54.0023618, -0.06935400750000001, 0.0001026705717, -7.20737313e-008, 1.958981996e-011}, blow = {-58545.8048, -260.4377836}, ahigh = {1116600.852, -8067.07473, 36.5294241, -0.0020471903, 4.44196775e-007, -4.93250713e-011, 2.235704865e-015}, bhigh = {-48606.6125, -175.4566113}, R = 42.44340477448436);
          constant IdealGases.Common.DataRecord FeCL(name = "FeCL", MM = 0.091298, Hf = 2749676.882297531, H0 = 113662.435102631, Tlimit = 1000, alow = {11173.40353, -54.0214429, 3.48605792, 0.00687983714, -1.273679557e-005, 1.025321859e-008, -3.051544011e-012}, blow = {29286.8199, 9.428962979}, ahigh = {528870.022, -1282.897413, 5.79844169, -0.0002896589776, 3.34390381e-008, -1.469606582e-013, -1.213444602e-016}, bhigh = {37261.9673, -4.075134191}, R = 91.06959626716905);
          constant IdealGases.Common.DataRecord FeCL2(name = "FeCL2", MM = 0.126751, Hf = -1112423.570622717, H0 = 112640.8312360455, Tlimit = 1000, alow = {23011.21607, -585.804406, 9.322903480000001, -0.003006298824, 2.590788666e-006, -1.080178662e-009, 2.341239608e-013}, blow = {-16210.41276, -18.16752393}, ahigh = {164412.3697, 692.800269, 4.63827014, 0.002754339782, -8.624218250000001e-007, 1.170827576e-010, -5.93806195e-015}, bhigh = {-22197.21855, 11.16897701}, R = 65.59689469905563);
          constant IdealGases.Common.DataRecord FeCL3(name = "FeCL3", MM = 0.162204, Hf = -6529458.509038001, H0 = 112291.1457177382, Tlimit = 1000, alow = {4284.772099999999, -574.170537, 12.20667256, -0.00468043412, 5.609500470000001e-006, -3.5495995e-009, 9.198543490000001e-013}, blow = {-127568.8456, -28.87584081}, ahigh = {-59157.40590000001, -10.33382228, 10.00849967, -3.65765187e-006, 8.545873639999999e-010, -1.023070527e-013, 4.90406364e-018}, bhigh = {-130501.205, -15.98494171}, R = 51.25935242040887);
          constant IdealGases.Common.DataRecord FeO(name = "FeO", MM = 0.07184439999999999, Hf = 3494218.060141083, H0 = 123002.363440992, Tlimit = 1000, alow = {15692.82213, -64.6018888, 2.45892547, 0.00701604736, -1.021405947e-005, 7.179297870000001e-009, -1.978966365e-012}, blow = {29645.72665, 13.26115545}, ahigh = {-119597.148, -362.486478, 5.51888075, -0.0009978856889999999, 4.37691383e-007, -6.79062946e-011, 3.63929268e-015}, bhigh = {30379.85806, -3.63365542}, R = 115.7288807478384);
          constant IdealGases.Common.DataRecord Fe_OH_2(name = "Fe_OH_2", MM = 0.08985968, Hf = -3678357.189787456, H0 = 158129.4413690323, Tlimit = 1000, alow = {444302.72, -6795.14089, 38.9472621, -0.0597300568, 7.046165430000001e-005, -4.087859510000001e-008, 9.368766340000001e-012}, blow = {-9051.42086, -193.1304058}, ahigh = {1612519.19, -6533.24199, 18.42922816, -0.002073249635, 4.26587436e-007, -4.56406313e-011, 1.990105746e-015}, bhigh = {-2992.568633, -84.45940589999999}, R = 92.52728253650581);
          constant IdealGases.Common.DataRecord Fe2CL4(name = "Fe2CL4", MM = 0.253502, Hf = -1701644.957436233, H0 = 117747.3984426158, Tlimit = 1000, alow = {1501.308814, -661.709651, 18.26924882, -0.00418149466, 4.196535049999999e-006, -2.188670747e-009, 5.395194700000001e-013}, blow = {-53400.5758, -49.3564068}, ahigh = {140245.0973, 693.606441, 13.1380041, 0.00275433307, -8.623996010000001e-007, 1.170782491e-010, -5.93777808e-015}, bhigh = {-59741.72730000001, -17.52491511}, R = 32.79844734952781);
          constant IdealGases.Common.DataRecord Fe2CL6(name = "Fe2CL6", MM = 0.324408, Hf = -2017143.843555029, H0 = 124682.0485314789, Tlimit = 1000, alow = {-10345.44447, -823.48766, 25.20090594, -0.00684579679, 8.25582075e-006, -5.24896566e-009, 1.365242237e-012}, blow = {-81318.4719, -80.10768938}, ahigh = {-99991.17999999999, -14.57716085, 22.0120846, -5.22948113e-006, 1.226831608e-009, -1.473195806e-013, 7.07827233e-018}, bhigh = {-85514.9619, -61.43757198}, R = 25.62967621020443);
          constant IdealGases.Common.DataRecord Ga(name = "Ga", MM = 0.06972299999999999, Hf = 3901151.700299758, H0 = 93959.16698937224, Tlimit = 1000, alow = {238794.789, -3121.634631, 16.30171272, -0.02347922342, 1.932327565e-005, -7.31631187e-009, 8.857387735e-013}, blow = {47327.1738, -75.47172759999999}, ahigh = {-55441.8652, 880.8624419999999, 1.760717716, 0.000299325946, -5.61082731e-008, 2.682832239e-012, 3.132134914e-016}, bhigh = {26843.51822, 12.52212023}, R = 119.2500609554953);
          constant IdealGases.Common.DataRecord Gaplus(name = "Gaplus", MM = 0.0697224514, Hf = 12287124.31645799, H0 = 88887.12137278639, Tlimit = 1000, alow = {0.000409834494, -4.34358162e-006, 2.500000019, -4.389036810000001e-011, 5.5639692e-014, -3.63822826e-017, 9.607881994999999e-021}, blow = {102289.9726, 5.21509046}, ahigh = {35666.0714, -110.8845546, 2.635517951, -8.33897352e-005, 2.734243614e-008, -4.55567374e-012, 3.035050101e-016}, bhigh = {102989.743, 4.25707541}, R = 119.2509992555999);
          constant IdealGases.Common.DataRecord GaBr(name = "GaBr", MM = 0.149627, Hf = -120084.4767321406, H0 = 66379.18958476746, Tlimit = 1000, alow = {-2185.275415, -76.72232969999999, 4.77080567, -0.000449788106, 5.5798536e-007, -3.065156457e-010, 7.042946040000001e-014}, blow = {-3138.004818, 2.948144294}, ahigh = {335980.503, -840.683668, 5.14508473, -3.265526e-005, -7.67102275e-008, 3.63512382e-011, -3.35817783e-015}, bhigh = {2035.179445, -0.427789408}, R = 55.56799240778737);
          constant IdealGases.Common.DataRecord GaBr2(name = "GaBr2", MM = 0.229531, Hf = -649938.1434316061, H0 = 61102.63101716108, Tlimit = 1000, alow = {7081.68162, -370.57661, 8.418425360000001, -0.003001930915, 3.59547767e-006, -2.276883735e-009, 5.91322177e-013}, blow = {-18211.16655, -9.512572049999999}, ahigh = {-330229.436, 476.672265, 7.1047254, -0.0005522771210000001, 3.159876015e-007, -5.51363715e-011, 3.17329323e-015}, bhigh = {-23656.8362, -1.162714892}, R = 36.22374319808653);
          constant IdealGases.Common.DataRecord GaBr3(name = "GaBr3", MM = 0.309435, Hf = -946766.4065151, H0 = 61522.89495370594, Tlimit = 1000, alow = {6363.80734, -514.204045, 11.96065138, -0.00413463765, 4.933978330000001e-006, -3.111868526e-009, 8.04363148e-013}, blow = {-35704.155, -25.56896053}, ahigh = {-51030.6375, -9.39520969, 10.00768765, -3.2959543e-006, 7.679838140000001e-010, -9.175147259999999e-014, 4.39118694e-018}, bhigh = {-38334.0215, -14.10315914}, R = 26.86984988769855);
          constant IdealGases.Common.DataRecord GaCL(name = "GaCL", MM = 0.105176, Hf = -661951.7760705864, H0 = 91362.24994295275, Tlimit = 1000, alow = {6417.143, -225.7949147, 5.31167369, -0.001562280975, 1.836579731e-006, -1.100421873e-009, 2.725812445e-013}, blow = {-8593.892329999999, -1.695911735}, ahigh = {-486484.224, 1452.870183, 2.719542959, 0.001144285628, -3.43145286e-007, 5.47044048e-011, -3.019622103e-015}, bhigh = {-18950.84035, 15.70926145}, R = 79.05293983418271);
          constant IdealGases.Common.DataRecord GaCL2(name = "GaCL2", MM = 0.140629, Hf = -1571360.878623897, H0 = 96092.49159135029, Tlimit = 1000, alow = {22254.81851, -604.621844, 9.220388910000001, -0.00455440079, 5.32485518e-006, -3.30860557e-009, 8.462235559999999e-013}, blow = {-25645.50386, -16.85409262}, ahigh = {-344540.53, 471.443716, 7.10877548, -0.000553943356, 3.16363824e-007, -5.51802393e-011, 3.17535303e-015}, bhigh = {-32309.3616, -3.74484074}, R = 59.12345248846255);
          constant IdealGases.Common.DataRecord GaCL3(name = "GaCL3", MM = 0.176082, Hf = -2456950.47193921, H0 = 98677.01979759432, Tlimit = 1000, alow = {47849.484, -1162.678125, 14.08954213, -0.008106134290000001, 9.21943016e-006, -5.59971055e-009, 1.4048526e-012}, blow = {-49159.0973, -42.82179383}, ahigh = {-94669.1731, -25.61468254, 10.02010048, -8.35407307e-006, 1.901439055e-009, -2.231168493e-013, 1.052949609e-017}, bhigh = {-55182.0143, -18.64958383}, R = 47.21931827216866);
          constant IdealGases.Common.DataRecord GaF(name = "GaF", MM = 0.08872140319999999, Hf = -2621785.528748265, H0 = 102355.3356063241, Tlimit = 1000, alow = {36703.9801, -533.6572219999999, 5.75929192, -0.001506128492, 9.509038450000001e-007, -1.83094113e-010, -3.135959193e-014}, blow = {-26470.7569, -6.58932188}, ahigh = {-298339.2279, 722.5157870000001, 3.63955743, 0.000554673676, -1.46490145e-007, 2.177385396e-011, -1.031058874e-015}, bhigh = {-34085.2871, 7.55396}, R = 93.71438796179906);
          constant IdealGases.Common.DataRecord GaF2(name = "GaF2", MM = 0.1077198064, Hf = -4796819.250503221, H0 = 112473.8282114105, Tlimit = 1000, alow = {68425.3953, -889.761774, 7.70745549, 0.00207215089, -5.28998193e-006, 4.56412742e-009, -1.406942753e-012}, blow = {-59198.5615, -13.41142116}, ahigh = {410025.1359999999, -1723.259472, 8.95132922, -0.001059205682, 2.745755154e-007, -2.907681311e-011, 1.084834142e-015}, bhigh = {-53765.716, -21.13835528}, R = 77.18610233224483);
          constant IdealGases.Common.DataRecord GaF3(name = "GaF3", MM = 0.1267182096, Hf = -7271859.237190485, H0 = 120132.8289600455, Tlimit = 1000, alow = {95932.66699999999, -1355.507494, 10.97760197, 0.00346157172, -8.469551949999999e-006, 7.22246893e-009, -2.213328653e-012}, blow = {-106147.9087, -32.1265185}, ahigh = {-209696.176, -143.1211274, 10.10568378, -4.18493682e-005, 9.16475945e-009, -1.042661957e-012, 4.79916345e-017}, bhigh = {-113667.8134, -23.9253933}, R = 65.61386896362842);
          constant IdealGases.Common.DataRecord GaH(name = "GaH", MM = 0.07073094000000001, Hf = 3030122.376431021, H0 = 122621.0764341602, Tlimit = 1000, alow = {-43918.762, 625.445712, 0.327686289, 0.0064966415, -3.80297001e-006, 2.852103704e-010, 3.62680913e-013}, blow = {21712.60108, 22.24032244}, ahigh = {3257993.99, -11090.99502, 18.01126796, -0.008167719110000001, 2.601293566e-006, -3.81345935e-010, 2.028049649e-014}, bhigh = {93697.78240000001, -98.2090022}, R = 117.5507069466347);
          constant IdealGases.Common.DataRecord GaI(name = "GaI", MM = 0.19662747, Hf = 228203.7296212986, H0 = 51412.56203927151, Tlimit = 1000, alow = {-3661.85503, -32.0692418, 4.59996192, -8.899475459999999e-005, 1.162792633e-007, -1.812351457e-011, -7.584619639999999e-015}, blow = {4198.64937, 4.92896465}, ahigh = {1498356.654, -4551.96724, 9.803005349999999, -0.002943735609, 8.658040509999999e-007, -1.106638204e-010, 5.00783686e-015}, bhigh = {32918.1191, -32.2735977}, R = 42.28540396720764);
          constant IdealGases.Common.DataRecord GaI2(name = "GaI2", MM = 0.32353194, Hf = -89496.46517125944, H0 = 44727.99501650439, Tlimit = 1000, alow = {-983.13959, -215.2621259, 7.84431876, -0.001819797849, 2.210706958e-006, -1.416110855e-009, 3.71376293e-013}, blow = {-4534.090230000001, -4.25051681}, ahigh = {-320287.975, 479.650233, 7.10233396, -0.000551265783, 3.157543452e-007, -5.51087194e-011, 3.17197773e-015}, bhigh = {-9180.919610000001, 0.727520488}, R = 25.69907626430948);
          constant IdealGases.Common.DataRecord GaI3(name = "GaI3", MM = 0.45043641, Hf = -257255.4914022159, H0 = 44673.91967714155, Tlimit = 1000, alow = {-5663.47265, -242.5937116, 10.95451119, -0.002059175489, 2.499117622e-006, -1.596502506e-009, 4.16764406e-013}, blow = {-15767.22832, -16.46129942}, ahigh = {-31640.4013, -4.20866283, 10.00352001, -1.53265018e-006, 3.61153154e-010, -4.35097887e-014, 2.095704875e-018}, bhigh = {-17000.58453, -10.90291552}, R = 18.45870319408682);
          constant IdealGases.Common.DataRecord GaO(name = "GaO", MM = 0.08572239999999999, Hf = 1712779.938499156, H0 = 104116.3919815591, Tlimit = 1000, alow = {73255.4391, -980.4269509999999, 7.84412276, -0.00717107285, 7.87278051e-006, -2.401464112e-009, -2.594919458e-013}, blow = {21405.81724, -17.99742951}, ahigh = {2937313.804, -11183.14112, 18.59366773, -0.00685475345, 1.665399251e-006, -1.887560878e-010, 7.97268692e-015}, bhigh = {85302.48080000001, -99.04040480000001}, R = 96.9929913301541);
          constant IdealGases.Common.DataRecord GaOH(name = "GaOH", MM = 0.08673034, Hf = -1656058.249051024, H0 = 122028.9001518961, Tlimit = 1000, alow = {69631.0187, -1247.828695, 10.22001611, -0.00592253439, 4.6433874e-006, -1.215936978e-009, -2.945243614e-014}, blow = {-12753.97794, -32.17063}, ahigh = {842905.101, -2372.324595, 8.009071090000001, 8.77723071e-005, -5.94306891e-008, 9.841943439999999e-012, -5.53155279e-016}, bhigh = {-3751.78825, -21.71053719}, R = 95.86578353088434);
          constant IdealGases.Common.DataRecord Ga2Br2(name = "Ga2Br2", MM = 0.299254, Hf = -457683.7469173344, H0 = 69483.80974022068, Tlimit = 1000, alow = {-11171.83241, -74.7147162, 10.30246084, -0.0006657929850000001, 8.20051585e-007, -5.29702152e-010, 1.394553192e-013}, blow = {-19132.97856, -13.17110316}, ahigh = {-18853.73131, -1.263621802, 10.00108034, -4.774975660000001e-007, 1.137227648e-010, -1.380820486e-014, 6.69024243e-019}, bhigh = {-19510.53843, -11.41667594}, R = 27.78399620389368);
          constant IdealGases.Common.DataRecord Ga2Br4(name = "Ga2Br4", MM = 0.459062, Hf = -905803.647437601, H0 = 66639.72622434443, Tlimit = 1000, alow = {-13155.45939, -413.655302, 17.62849229, -0.00351465216, 4.26693773e-006, -2.726515882e-009, 7.11892449e-013}, blow = {-52831.0181, -44.2752069}, ahigh = {-57421.6721, -7.16013737, 16.00598947, -2.60818227e-006, 6.14645595e-010, -7.40543865e-014, 3.56712663e-018}, bhigh = {-54933.8976, -34.79271660000001}, R = 18.11187159904327);
          constant IdealGases.Common.DataRecord Ga2Br6(name = "Ga2Br6", MM = 0.61887, Hf = -1088578.75159565, H0 = 66494.20072066832, Tlimit = 1000, alow = {-6725.11689, -806.752655, 25.11671424, -0.00663651457, 7.9775557e-006, -5.05968521e-009, 1.313546388e-012}, blow = {-83706.352, -75.8527872}, ahigh = {-95261.73, -14.4468188, 22.01192546, -5.14500674e-006, 1.204348449e-009, -1.443806473e-013, 6.928270740000001e-018}, bhigh = {-87822.23300000001, -57.6589512}, R = 13.43492494384927);
          constant IdealGases.Common.DataRecord Ga2CL2(name = "Ga2CL2", MM = 0.210352, Hf = -1050490.064273218, H0 = 92646.73024264089, Tlimit = 1000, alow = {-9246.1337, -259.2159934, 11.02847129, -0.002231983927, 2.720723456e-006, -1.743798624e-009, 4.56366604e-013}, blow = {-28340.59821, -20.67773272}, ahigh = {-36689.542, -4.43458068, 10.00373051, -1.630793333e-006, 3.85375332e-010, -4.65252773e-014, 2.244498791e-018}, bhigh = {-29656.33085, -14.69535817}, R = 39.52646991709135);
          constant IdealGases.Common.DataRecord Ga2CL4(name = "Ga2CL4", MM = 0.281258, Hf = -2141545.584481152, H0 = 97118.63129226546, Tlimit = 1000, alow = {16373.43702, -1140.637354, 20.30139827, -0.0089977462, 1.067254867e-005, -6.700290619999999e-009, 1.725749144e-012}, blow = {-71623.85010000001, -66.663619}, ahigh = {-112711.0409, -21.38517542, 16.01738077, -7.41556464e-006, 1.721694751e-009, -2.051361953e-013, 9.797311499999999e-018}, bhigh = {-77468.53380000001, -41.4728087}, R = 29.56172624423128);
          constant IdealGases.Common.DataRecord Ga2CL6(name = "Ga2CL6", MM = 0.352164, Hf = -2732998.069081451, H0 = 103246.7799093604, Tlimit = 1000, alow = {60894.69839999999, -2104.456302, 29.59653878, -0.01537809796, 1.779096808e-005, -1.095619429e-008, 2.779706493e-012}, blow = {-111840.0535, -112.1945814}, ahigh = {-189844.5426, -44.1365524, 22.03505047, -1.470071128e-005, 3.36951281e-009, -3.9754628e-013, 1.884231868e-017}, bhigh = {-122696.9998, -67.45332883}, R = 23.60965913608433);
          constant IdealGases.Common.DataRecord Ga2F2(name = "Ga2F2", MM = 0.1774428064, Hf = -3416488.705850406, H0 = 96638.77813871186, Tlimit = 1000, alow = {29781.3065, -1013.97552, 13.74933053, -0.00772805096, 9.06400733e-006, -5.64119322e-009, 1.443106716e-012}, blow = {-70860.8337, -41.09833949999999}, ahigh = {-87732.34929999999, -19.8377587, 10.0159403, -6.744402830000001e-006, 1.556129063e-009, -1.845316975e-013, 8.780884020000001e-018}, bhigh = {-76073.6863, -19.08325521}, R = 46.85719398089953);
          constant IdealGases.Common.DataRecord Ga2F4(name = "Ga2F4", MM = 0.2154396128, Hf = -6150229.016750257, H0 = 105148.9171632971, Tlimit = 1000, alow = {142207.9998, -2798.829941, 23.95904235, -0.01293904774, 1.223101326e-005, -6.25120982e-009, 1.334919195e-012}, blow = {-149601.371, -97.34153640000001}, ahigh = {-270436.609, -104.9957196, 16.07903965, -3.17889592e-005, 7.04958272e-009, -8.10237952e-013, 3.76054053e-017}, bhigh = {-164416.9589, -48.9659173}, R = 38.59305116612241);
          constant IdealGases.Common.DataRecord Ga2F6(name = "Ga2F6", MM = 0.2534364192, Hf = -7961065.143552976, H0 = 118891.6576990526, Tlimit = 1000, alow = {240904.5572, -4142.3798, 31.9577372, -0.0129791429, 8.95928219e-006, -2.733621068e-009, 1.554030994e-013}, blow = {-227278.524, -139.8403732}, ahigh = {-436866.48, -212.8208368, 22.15886373, -6.34628791e-005, 1.399738556e-008, -1.601732039e-012, 7.407584420000001e-017}, bhigh = {-249429.8707, -77.87635830000001}, R = 32.80693448181421);
          constant IdealGases.Common.DataRecord Ga2I2(name = "Ga2I2", MM = 0.39325494, Hf = 34382.14151867996, H0 = 54666.69280746988, Tlimit = 1000, alow = {-8960.25267, -31.7205721, 10.12930491, -0.0002860191165, 3.53533301e-007, -2.289632389e-010, 6.04008536e-014}, blow = {-1233.174068, -9.80196838}, ahigh = {-12190.56501, -0.528755447, 10.00045392, -2.011599344e-007, 4.79946781e-011, -5.83478108e-015, 2.829565069e-019}, bhigh = {-1393.258606, -9.052614910000001}, R = 21.14270198360382);
          constant IdealGases.Common.DataRecord Ga2I4(name = "Ga2I4", MM = 0.64706388, Hf = -246139.5109861487, H0 = 50356.40839664856, Tlimit = 1000, alow = {-17694.95991, -135.4871509, 16.54625333, -0.001199014713, 1.473754848e-006, -9.504796569999999e-010, 2.499380837e-013}, blow = {-23334.05795, -33.2571289}, ahigh = {-31712.9976, -2.287068836, 16.00194812, -8.58896438e-007, 2.041966602e-010, -2.476145769e-014, 1.198551944e-018}, bhigh = {-24019.40505, -30.08672521}, R = 12.84953813215474);
          constant IdealGases.Common.DataRecord Ga2I6(name = "Ga2I6", MM = 0.90087282, Hf = -352208.0097832234, H0 = 48367.95719955232, Tlimit = 1000, alow = {-21108.91458, -369.540416, 23.46811836, -0.00318917821, 3.89033904e-006, -2.494830378e-009, 6.53200728e-013}, blow = {-43011.8809, -60.148115}, ahigh = {-60156.9815, -6.32920923, 22.0053304, -2.332008629e-006, 5.51386848e-010, -6.659445350000001e-014, 3.21367043e-018}, bhigh = {-44886.9938, -51.6100875}, R = 9.22935159704341);
          constant IdealGases.Common.DataRecord Ga2O(name = "Ga2O", MM = 0.1554454, Hf = -639822.4521278854, H0 = 78243.51830288963, Tlimit = 1000, alow = {56971.9718, -807.4085990000001, 7.73306154, 0.001478450874, -4.097743430000001e-006, 3.58115892e-009, -1.107073202e-012}, blow = {-9512.216820000002, -12.56149299}, ahigh = {-119387.4986, -87.19341610000001, 7.0646782, -2.571524542e-005, 5.65110849e-009, -6.44819054e-013, 2.975374645e-017}, bhigh = {-13936.94796, -6.9428013}, R = 53.4880543264709);
          constant IdealGases.Common.DataRecord Ge(name = "Ge", MM = 0.07264, Hf = 5063325.991189428, H0 = 101852.7533039648, Tlimit = 1000, alow = {-20592.15242, -143.2022103, 4.50600233, 0.00154718784, -8.518296550000001e-006, 8.24382446e-009, -2.566167305e-012}, blow = {43630.7086, -6.224648889}, ahigh = {-856541.384, 3917.95866, -1.809888212, 0.002276482224, -5.36562755e-007, 5.984958090000001e-011, -2.541700646e-015}, bhigh = {19565.18798, 38.41408752}, R = 114.4613436123348);
          constant IdealGases.Common.DataRecord Geplus(name = "Geplus", MM = 0.0726394514, Hf = 15624903.03994779, H0 = 85432.70743919742, Tlimit = 1000, alow = {-345366.643, 4008.44971, -15.22395737, 0.0362603002, -3.35136726e-005, 1.431059219e-008, -2.236976459e-012}, blow = {115705.8042, 109.0158182}, ahigh = {-2244860.57, 5165.53114, -0.386751651, 0.000852873094, -1.386040003e-007, 1.155945775e-011, -3.784413134e-016}, bhigh = {100682.3985, 29.65845411}, R = 114.4622080667311);
          constant IdealGases.Common.DataRecord Geminus(name = "Geminus", MM = 0.07264054860000001, Hf = 3378313.417638478, H0 = 96102.52310236543, Tlimit = 1000, alow = {2989.142308, 86.5741723, 2.191868507, 0.0005900795269999999, -6.37580058e-007, 3.66113215e-010, -8.689778450000001e-014}, blow = {28356.9519, 9.417623637}, ahigh = {13520.40246, -0.504851878, 2.500140247, -3.65294416e-009, -6.7167222e-012, 1.378119925e-015, -8.475212940000001e-020}, bhigh = {28817.39757, 7.577953867}, R = 114.4604791709957);
          constant IdealGases.Common.DataRecord GeBr(name = "GeBr", MM = 0.152544, Hf = 900973.6141703377, H0 = 64664.09036081393, Tlimit = 1000, alow = {-56807.7463, 917.336126, -2.224809366, 0.02171239242, -3.035490868e-005, 2.011776305e-008, -5.20291504e-012}, blow = {11041.92901, 41.07865983}, ahigh = {225675.0476, -731.071546, 6.26328381, -0.001198085447, 4.00037777e-007, -5.42993788e-011, 2.510481953e-015}, bhigh = {19428.63606, -6.106507844}, R = 54.50540172015943);
          constant IdealGases.Common.DataRecord GeBr2(name = "GeBr2", MM = 0.232448, Hf = -262264.6656456497, H0 = 61060.80929928413, Tlimit = 1000, alow = {-1649.285325, -229.6262116, 7.90014468, -0.001936685504, 2.345776175e-006, -1.496279368e-009, 3.9014535e-013}, blow = {-8316.62421, -6.918156136}, ahigh = {-26366.55245, -3.99050902, 7.00332744, -1.445749787e-006, 3.40159136e-010, -4.09345703e-014, 1.969987842e-018}, bhigh = {-9484.999249999999, -1.673617098}, R = 35.76916987885463);
          constant IdealGases.Common.DataRecord GeBr3(name = "GeBr3", MM = 0.312352, Hf = -381080.665403135, H0 = 59386.49984632723, Tlimit = 1000, alow = {2725.568677, -518.621326, 11.99957754, -0.00425102453, 5.10362972e-006, -3.23370625e-009, 8.38835886e-013}, blow = {-14779.88251, -25.34555678}, ahigh = {-54339.267, -9.28032795, 10.00765019, -3.29732215e-006, 7.71292926e-010, -9.24157185e-014, 4.43287623e-018}, bhigh = {-17426.97092, -13.66946296}, R = 26.61891711914763);
          constant IdealGases.Common.DataRecord GeBr4(name = "GeBr4", MM = 0.392256, Hf = -741862.4571708272, H0 = 61090.76215532714, Tlimit = 1000, alow = {5431.940390000001, -677.7748800000001, 15.60015305, -0.00550778888, 6.59476429e-006, -4.17014614e-009, 1.080116643e-012}, blow = {-35576.064, -42.09122927}, ahigh = {-69777.5791, -11.79873115, 13.00953955, -4.02493875e-006, 9.203567209999999e-010, -1.076853369e-013, 5.0409548e-018}, bhigh = {-39041.4816, -26.8943607}, R = 21.19654511339533);
          constant IdealGases.Common.DataRecord GeCL(name = "GeCL", MM = 0.108093, Hf = 638615.960330456, H0 = 88804.2241403236, Tlimit = 1000, alow = {10697.24739, -22.41238266, 1.98737107, 0.01368270905, -2.358294414e-005, 1.78675602e-008, -5.10250325e-012}, blow = {7440.68339, 15.05837189}, ahigh = {-378419.325, 1429.309204, 3.19485464, 0.0008119939119999999, -2.652459954e-007, 5.15195788e-011, -3.53421457e-015}, bhigh = {-2087.046335, 13.8503382}, R = 76.91961551626839);
          constant IdealGases.Common.DataRecord GeCL2(name = "GeCL2", MM = 0.143546, Hf = -1191255.764702604, H0 = 92491.54278071142, Tlimit = 1000, alow = {18385.36673, -582.720534, 9.161404660000001, -0.00446478603, 5.24487355e-006, -3.26806721e-009, 8.36757542e-013}, blow = {-19757.98598, -17.3432163}, ahigh = {-48901.28920000001, -11.26210418, 7.00906353, -3.83918073e-006, 8.86563357e-010, -1.05200054e-013, 5.00842222e-018}, bhigh = {-22752.54424, -4.656641298}, R = 57.9220040962479);
          constant IdealGases.Common.DataRecord GeCL3(name = "GeCL3", MM = 0.178999, Hf = -1491352.404203376, H0 = 98097.96144112539, Tlimit = 1000, alow = {19358.59579, -834.202, 13.1270253, -0.006511390020000001, 7.69627748e-006, -4.81854173e-009, 1.238399959e-012}, blow = {-30972.18753, -35.52547183999999}, ahigh = {-75744.9982, -15.7921342, 10.01278835, -5.44173831e-006, 1.260936724e-009, -1.500137106e-013, 7.15640081e-018}, bhigh = {-35251.3172, -17.19719944}, R = 46.44982374203208);
          constant IdealGases.Common.DataRecord GeCL4(name = "GeCL4", MM = 0.214452, Hf = -2331524.070654505, H0 = 98561.09991979557, Tlimit = 1000, alow = {65028.7041, -1652.48276, 18.81327, -0.0115242681, 1.31080998e-005, -7.96195611e-009, 1.99750504e-012}, blow = {-55700.5239, -67.54438135}, ahigh = {-137626.121, -36.037695, 13.0281667, -1.16495696e-005, 2.63619338e-009, -3.07292672e-013, 1.43980875e-017}, bhigh = {-64262.8178, -33.18158305}, R = 38.77078320556581);
          constant IdealGases.Common.DataRecord GeF(name = "GeF", MM = 0.0916384032, Hf = -770342.8642894556, H0 = 99784.69376035572, Tlimit = 1000, alow = {50170.9782, -439.555252, 2.990864513, 0.01252766738, -2.337593226e-005, 1.841631179e-008, -5.398492260000001e-012}, blow = {-7093.5164, 7.064248931}, ahigh = {-515048.058, 1659.711777, 2.999610299, 0.0008198511679999999, -2.271982948e-007, 3.62418386e-011, -2.133840473e-015}, bhigh = {-20515.29487, 13.7315423}, R = 90.73130597718665);
          constant IdealGases.Common.DataRecord GeF2(name = "GeF2", MM = 0.1106368064, Hf = -5188146.862489335, H0 = 106542.1660616552, Tlimit = 1000, alow = {75430.9708, -1108.456469, 9.07889827, -0.001445630636, -6.57532232e-007, 1.475344103e-009, -5.81122027e-013}, blow = {-65106.8555, -22.00457134}, ahigh = {-127532.7716, -71.6259135, 7.05300237, -2.101910441e-005, 4.6078937e-009, -5.24639467e-013, 2.416218951e-017}, bhigh = {-71126.3763, -8.483841439000001}, R = 75.15104846699552);
          constant IdealGases.Common.DataRecord GeF3(name = "GeF3", MM = 0.1296352096, Hf = -6220014.111042869, H0 = 113226.8543807716, Tlimit = 1000, alow = {111412.1569, -1776.90803, 13.52166477, -0.002983622949, -3.61274975e-008, 1.621528436e-009, -7.19901178e-013}, blow = {-90382.62960000002, -45.68760023}, ahigh = {-206958.0129, -111.0233803, 10.08234122, -3.27167626e-005, 7.1838967e-009, -8.19038812e-013, 3.77632244e-017}, bhigh = {-99997.4909, -23.0346594}, R = 64.13745174366579);
          constant IdealGases.Common.DataRecord GeF4(name = "GeF4", MM = 0.1486336128, Hf = -8007273.574123873, H0 = 116346.6639492194, Tlimit = 1000, alow = {111981.8968, -1636.85963, 12.31371005, 0.01025446325, -1.893783072e-005, 1.476849261e-008, -4.32517411e-012}, blow = {-137426.5128, -41.04202391000001}, ahigh = {-325303.268, -253.8279448, 13.18792173, -7.46005855e-005, 1.637384721e-008, -1.866482733e-012, 8.605530500000001e-017}, bhigh = {-146617.6526, -41.22237561}, R = 55.93937900969867);
          constant IdealGases.Common.DataRecord GeH4(name = "GeH4", MM = 0.07667176000000001, Hf = 1184177.85114102, H0 = 0, Tlimit = 1000, alow = {-256839.8191, -41.3281145, 7.75511628, 0.002129640783, 7.255053610000001e-007, -5.09631401e-010, 1.431608261e-013}, blow = {7881.53804, -20.30107999}, ahigh = {-3809805.88, 10931.67093, -5.14967541, 0.009485931369999999, -1.649459859e-006, 1.419674126e-010, -5.00463975e-015}, bhigh = {-61585.1852, 71.68961272999999}, R = 108.4424304333173);
          constant IdealGases.Common.DataRecord GeI(name = "GeI", MM = 0.19954447, Hf = 1057253.653784542, H0 = 50139.80091755988, Tlimit = 1000, alow = {-74383.08219999999, 1033.759595, -1.520491112, 0.01555125152, -1.683415215e-005, 8.383557809999999e-009, -1.550875391e-012}, blow = {19129.20413, 39.59395473}, ahigh = {-213314.8437, -295.3079008, 6.87660736, -0.002102579132, 8.59629938e-007, -1.44769276e-010, 8.401042789999999e-015}, bhigh = {24523.26136, -8.549783749}, R = 41.66726344257999);
          constant IdealGases.Common.DataRecord GeO(name = "GeO", MM = 0.08863939999999999, Hf = -425254.5369215045, H0 = 99076.77624171645, Tlimit = 1000, alow = {-6781.22563, 279.1946972, 0.619895286, 0.01111378013, -1.501027092e-005, 1.006783003e-008, -2.687101932e-012}, blow = {-6711.84743, 21.56441027}, ahigh = {-1044508.485, 2734.866048, 1.298071298, 0.001832638634, -5.06098635e-007, 6.37495802e-011, -2.172371872e-015}, bhigh = {-23621.04658, 23.50906051}, R = 93.80108619868818);
          constant IdealGases.Common.DataRecord GeO2(name = "GeO2", MM = 0.1046388, Hf = -1014652.289590477, H0 = 107589.1734232426, Tlimit = 1000, alow = {-3824.17383, 156.3451633, 1.694198285, 0.01756205059, -2.418270032e-005, 1.632388154e-008, -4.36895786e-012}, blow = {-14775.3488, 15.56926515}, ahigh = {-172734.7526, -296.1154443, 7.72072625, -8.82493339e-005, 1.949874702e-008, -2.235834829e-012, 1.036128507e-016}, bhigh = {-13879.83288, -16.69845695}, R = 79.45878584234529);
          constant IdealGases.Common.DataRecord GeS(name = "GeS", MM = 0.104705, Hf = 883674.2275918056, H0 = 87303.47165846905, Tlimit = 1000, alow = {36610.1843, -565.9653500000001, 6.15293641, -0.002746908039, 2.733435196e-006, -1.460209187e-009, 3.27870947e-013}, blow = {12741.75754, -7.70565946}, ahigh = {-2351367.441, 7211.58186, -4.36252631, 0.00545173775, -1.721002774e-006, 2.632908084e-010, -1.39437404e-014}, bhigh = {-35833.1725, 65.05566890999999}, R = 79.40854782484124);
          constant IdealGases.Common.DataRecord GeS2(name = "GeS2", MM = 0.13677, Hf = 868741.9097755356, H0 = 95546.03348687578, Tlimit = 1000, alow = {55436.836, -1024.826697, 10.57594552, -0.00528788391, 5.29580308e-006, -2.87252953e-009, 6.51902755e-013}, blow = {17355.79668, -29.91719536}, ahigh = {-89709.8036, -33.64377, 7.52543372, -1.026368137e-005, 2.282230718e-009, -2.62874877e-013, 1.222218614e-017}, bhigh = {11952.32103, -11.35170616}, R = 60.79163559260072);
          constant IdealGases.Common.DataRecord Ge2(name = "Ge2", MM = 0.14528, Hf = 3245449.676486785, H0 = 73624.23595814979, Tlimit = 1000, alow = {216197.8929, -3079.621733, 18.88047728, -0.02604825861, 2.27915135e-005, -9.02835127e-009, 1.146806387e-012}, blow = {70324.02549999999, -79.01095968}, ahigh = {1969461.258, -5248.6871, 10.58277303, -0.00333655151, 1.052745838e-006, -1.484534793e-010, 7.475431760000001e-015}, bhigh = {89256.65990000001, -37.50233948}, R = 57.23067180616741);
          constant IdealGases.Common.DataRecord H(name = "H", MM = 0.00100794, Hf = 216281552.4733615, H0 = 6148608.052066591, Tlimit = 1000, alow = {0, 0, 2.5, 0, 0, 0, 0}, blow = {25473.70801, -0.446682853}, ahigh = {60.78774250000001, -0.1819354417, 2.500211817, -1.226512864e-007, 3.73287633e-011, -5.68774456e-015, 3.410210197e-019}, bhigh = {25474.86398, -0.448191777}, R = 8248.975137408972);
          constant IdealGases.Common.DataRecord Hplus(name = "Hplus", MM = 0.0010073914, Hf = 1524974233.450872, H0 = 6151956.429248851, Tlimit = 1000, alow = {0, 0, 2.5, 0, 0, 0, 0}, blow = {184021.4877, -1.140646644}, ahigh = {0, 0, 2.5, 0, 0, 0, 0}, bhigh = {184021.4877, -1.140646644}, R = 8253.46732163884);
          constant IdealGases.Common.DataRecord Hminus(name = "Hminus", MM = 0.0010084886, Hf = 137861080.4326395, H0 = 6145263.317800519, Tlimit = 1000, alow = {0, 0, 2.5, 0, 0, 0, 0}, blow = {15976.15494, -1.139013868}, ahigh = {0, 0, 2.5, 0, 0, 0, 0}, bhigh = {15976.15494, -1.139013868}, R = 8244.487840516989);
          constant IdealGases.Common.DataRecord HALO(name = "HALO", MM = 0.043988878, Hf = 41399.41918955059, H0 = 225857.476974066, Tlimit = 1000, alow = {26686.71568, -466.321459, 5.38380021, 0.002902425002, -2.24367761e-007, -1.516294323e-009, 6.95173766e-013}, blow = {1235.914473, -6.50883752}, ahigh = {78474.67049999999, -1515.62885, 8.572776230000001, -0.0004142611439999999, 8.9350817e-008, -1.007036446e-011, 4.60801725e-016}, bhigh = {6663.24189, -26.72948642}, R = 189.0130500714294);
          constant IdealGases.Common.DataRecord HALO2(name = "HALO2", MM = 0.059988278, Hf = -5925716.870886008, H0 = 199715.7511339132, Tlimit = 1000, alow = {3544.59891, 115.9819104, 0.863797069, 0.02460875785, -3.42316012e-005, 2.373182987e-008, -6.46760883e-012}, blow = {-44495.06140000001, 20.12317065}, ahigh = {698570.544, -2864.935042, 10.86721494, -5.3683127e-005, -2.836877559e-008, 6.29164119e-012, -3.88903311e-016}, bhigh = {-27643.92495, -37.7588634}, R = 138.6016114681605);
          constant IdealGases.Common.DataRecord HBO(name = "HBO", MM = 0.02781834, Hf = -7571307.454003366, H0 = 329527.139290123, Tlimit = 1000, alow = {63609.7503, -800.1557590000001, 6.21881613, -0.000780167998, 3.141286759e-006, -2.031853478e-009, 3.73855289e-013}, blow = {-22402.8291, -13.26952393}, ahigh = {886186.0229999999, -3913.06805, 9.8886448, -0.0008222269139999999, 1.621530554e-007, -1.703550237e-011, 7.372740020000001e-016}, bhigh = {-3161.852029, -40.3681272}, R = 298.8845488264217);
          constant IdealGases.Common.DataRecord HBOplus(name = "HBOplus", MM = 0.0278177914, Hf = 42247053.76861803, H0 = 326935.6962681085, Tlimit = 1000, alow = {65244.7147, -704.733736, 5.29427187, 0.001308494598, 2.088343622e-006, -2.564100339e-009, 8.198981249999999e-013}, blow = {143929.3436, -6.81674923}, ahigh = {18360.83606, -1212.975446, 6.82290196, 0.0009142272500000001, -2.665355457e-007, 3.29704981e-011, -1.530684082e-015}, bhigh = {146361.7504, -16.99646159}, R = 298.8904431859389);
          constant IdealGases.Common.DataRecord HBO2(name = "HBO2", MM = 0.04381774, Hf = -12785005.63926848, H0 = 249170.8152907932, Tlimit = 1000, alow = {6225.08747, 75.6615369, 1.253406833, 0.01748006535, -1.982688351e-005, 1.22965646e-008, -3.153609847e-012}, blow = {-68785.8878, 17.67793507}, ahigh = {1049369.185, -4479.145479999999, 11.97755861, -0.00047357434, 6.08020714e-008, -3.64156544e-012, 6.15597317e-017}, bhigh = {-42211.4947, -49.11366819999999}, R = 189.7512742555869);
          constant IdealGases.Common.DataRecord HBS(name = "HBS", MM = 0.04388394, Hf = 1144108.755959469, H0 = 211609.5090823659, Tlimit = 1000, alow = {56499.66209999999, -546.640995, 3.63645839, 0.009963657670000001, -1.395090386e-005, 1.032834167e-008, -3.047186722e-012}, blow = {7919.86053, 1.182455314}, ahigh = {-202183.6183, -505.202662, 6.41521828, 0.001049432932, -3.68314669e-007, 5.34296831e-011, -2.154140451e-015}, bhigh = {6485.47562, -13.31379864}, R = 189.4650298036138);
          constant IdealGases.Common.DataRecord HBSplus(name = "HBSplus", MM = 0.0438833914, Hf = 25737731.99762314, H0 = 230975.5394155794, Tlimit = 1000, alow = {148950.9163, -1847.661736, 11.4745802, -0.009476828369999999, 1.110746739e-005, -6.175558610000001e-009, 1.336853512e-012}, blow = {143782.2395, -41.13512498}, ahigh = {661065.836, -2791.498527, 9.131267640000001, -0.000506379529, 7.751213140000001e-008, -4.19113293e-012, 2.687149095e-017}, bhigh = {151073.4114, -30.81426166}, R = 189.4673983652048);
          constant IdealGases.Common.DataRecord HCN(name = "HCN", MM = 0.02702534, Hf = 4924358.398451231, H0 = 341733.4620026983, Tlimit = 1000, alow = {90982.86930000001, -1238.657512, 8.72130787, -0.00652824294, 8.87270083e-006, -4.808886669999999e-009, 9.317898499999999e-013}, blow = {20989.1545, -27.46678076}, ahigh = {1236889.278, -4446.732410000001, 9.73887485, -0.000585518264, 1.07279144e-007, -1.013313244e-011, 3.34824798e-016}, bhigh = {42215.1377, -40.05774072000001}, R = 307.6546678043644);
          constant IdealGases.Common.DataRecord HCO(name = "HCO", MM = 0.02901804, Hf = 1461085.931372346, H0 = 344249.6460822303, Tlimit = 1000, alow = {-11898.51887, 215.1536111, 2.730224028, 0.001806516108, 4.98430057e-006, -5.81456792e-009, 1.869689894e-012}, blow = {2905.75564, 11.3677254}, ahigh = {694960.6120000001, -3656.22338, 9.604731170000001, -0.001117129278, 2.875328019e-007, -3.62624774e-011, 1.808329595e-015}, bhigh = {25437.0444, -35.8247372}, R = 286.5276910501192);
          constant IdealGases.Common.DataRecord HCOplus(name = "HCOplus", MM = 0.0290174914, Hf = 28707995.06810571, H0 = 311742.7304553281, Tlimit = 1000, alow = {157344.2506, -1867.692159, 10.99235423, -0.01211637888, 1.659091514e-005, -1.016592642e-008, 2.391234771e-012}, blow = {108493.028, -40.7826162}, ahigh = {1219060.653, -4714.29489, 10.21192493, -0.000885451707, 1.667408026e-007, -1.683285548e-011, 7.04005178e-016}, bhigh = {127798.9027, -43.5115846}, R = 286.5331080963119);
          constant IdealGases.Common.DataRecord HCCN(name = "HCCN", MM = 0.03903604, Hf = 15637616.62299762, H0 = 303968.6146443133, Tlimit = 1000, alow = {2994.114377, -440.466068, 6.94003641, 0.00384630496, -1.264728529e-006, -3.63923513e-010, 2.748929104e-013}, blow = {73708.7864, -13.15302591}, ahigh = {939562.031, -4091.06775, 12.72233302, -0.000687440531, 1.231169968e-007, -1.186956238e-011, 4.76061035e-016}, bhigh = {95851.6482, -52.48695794}, R = 212.9947607390504);
          constant IdealGases.Common.DataRecord HCCO(name = "HCCO", MM = 0.04102874, Hf = 4303522.360179718, H0 = 284312.7524754599, Tlimit = 1000, alow = {69596.12700000001, -1164.594402, 9.456616260000001, -0.002331240632, 5.1618736e-006, -3.52616997e-009, 8.59914323e-013}, blow = {25350.03992, -27.26355351}, ahigh = {1093922.002, -4498.228209999999, 12.46446433, -0.00063433174, 1.108549019e-007, -1.125488678e-011, 5.68915194e-016}, bhigh = {46522.803, -50.9907043}, R = 202.6499473296036);
          constant IdealGases.Common.DataRecord HCL(name = "HCL", MM = 0.03646094, Hf = -2531750.415650282, H0 = 236968.7671244899, Tlimit = 1000, alow = {20625.88287, -309.3368855, 5.27541885, -0.00482887422, 6.1957946e-006, -3.040023782e-009, 4.916790029999999e-013}, blow = {-10677.82299, -7.309305408}, ahigh = {915774.951, -2770.550211, 5.97353979, -0.000362981006, 4.73552919e-008, 2.810262054e-012, -6.65610422e-016}, bhigh = {5674.95805, -16.42825822}, R = 228.0377850927595);
          constant IdealGases.Common.DataRecord HD(name = "HD", MM = 0.003022042, Hf = 106981.3060175868, H0 = 2815679.596775955, Tlimit = 1000, alow = {25191.20338, -276.1004999, 4.64444129, -0.002082376844, 1.418070803e-006, 2.839893835e-010, -3.20233103e-013}, blow = {391.361643, -9.395396119999999}, ahigh = {845583.0000000001, -1956.578537, 4.40437387, 0.000575168109, -2.131983152e-007, 4.03612668e-011, -2.727170705e-015}, bhigh = {12272.54163, -10.84742878}, R = 2751.276123892388);
          constant IdealGases.Common.DataRecord HDplus(name = "HDplus", MM = 0.0030214934, Hf = 495381726.7977484, H0 = 2850908.097300494, Tlimit = 1000, alow = {-80700.73049999999, 879.643258, 0.0596893216, 0.005418181009999999, -2.021515155e-006, -4.94526165e-010, 4.09293565e-013}, blow = {174499.2497, 19.34281193}, ahigh = {1340083.03, -5730.069570000001, 12.13836576, -0.00524333812, 1.976302344e-006, -3.30657353e-010, 1.937902763e-014}, bhigh = {213534.8051, -61.2875839}, R = 2751.775661664527);
          constant IdealGases.Common.DataRecord HDO(name = "HDO", MM = 0.019021442, Hf = -12894946.50300435, H0 = 521817.5888032044, Tlimit = 1000, alow = {-27377.95356, 431.0503899999999, 1.558479899, 0.005764969149999999, -4.85512936e-006, 3.38299017e-009, -1.040863879e-012}, blow = {-32732.2645, 14.87751891}, ahigh = {1711376.798, -5322.8723, 9.124351519999999, -0.000340066415, 4.152343519999999e-008, -4.78076857e-014, -1.46803517e-016}, bhigh = {3245.22468, -37.7460068}, R = 437.1104987729112);
          constant IdealGases.Common.DataRecord HDO2(name = "HDO2", MM = 0.035020842, Hf = -4004519.223152887, H0 = 323663.4059226788, Tlimit = 1000, alow = {-32164.7665, 749.2334460000001, -1.957676212, 0.02412802791, -2.915946684e-005, 1.930016324e-008, -5.22698631e-012}, blow = {-21510.59175, 36.7240674}, ahigh = {1313180.619, -5175.63712, 12.16490975, -0.000613669948, 1.227560176e-007, -1.079624939e-011, 3.87401372e-016}, bhigh = {13032.0967, -50.85276870000001}, R = 237.4149656367485);
          constant IdealGases.Common.DataRecord HF(name = "HF", MM = 0.0200063432, Hf = -13660667.38273289, H0 = 429818.8286603021, Tlimit = 1000, alow = {-3192.09897, 59.8680772, 3.055113902, 0.001684673783, -3.28739483e-006, 3.095923617e-009, -9.76469161e-013}, blow = {-34184.4316, 3.29490412}, ahigh = {725708.904, -1484.797741, 3.85552747, 0.000713898985, -2.106757333e-007, 3.050092453e-011, -1.639495583e-015}, bhigh = {-23554.5666, -3.20385683}, R = 415.5917909075958);
          constant IdealGases.Common.DataRecord HI(name = "HI", MM = 0.12791241, Hf = 206070.7010367485, H0 = 67675.21618895305, Tlimit = 1000, alow = {18728.8173, -343.178884, 5.95671243, -0.008543439599999999, 1.454780274e-005, -1.049104164e-008, 2.839734003e-012}, blow = {3682.95072, -8.14975609}, ahigh = {472492.145, -1923.465741, 5.75804897, -0.000406626638, 9.474332049999999e-008, -1.033534431e-011, 4.61161479e-016}, bhigh = {13948.57037, -11.82487652}, R = 65.00129268145288);
          constant IdealGases.Common.DataRecord HNC(name = "HNC", MM = 0.02702534, Hf = 7192439.429069164, H0 = 370049.8865139163, Tlimit = 1000, alow = {48706.2064, -989.1456249999999, 9.72215389, -0.01113593916, 1.668862707e-005, -1.113940941e-008, 2.893600868e-012}, blow = {26646.79758, -31.04826344}, ahigh = {1198791.66, -3918.94186, 9.11802009, -0.00034177611, 3.31480968e-008, -5.70157474e-013, -7.789455389999999e-017}, bhigh = {46588.103, -34.68575882}, R = 307.6546678043644);
          constant IdealGases.Common.DataRecord HNCO(name = "HNCO", MM = 0.04302474, Hf = -2743921.962108313, H0 = 254879.5413987394, Tlimit = 1000, alow = {75424.6008, -955.0937770000001, 6.72570587, 0.0047056875, -4.95947551e-006, 3.69425512e-009, -1.164859121e-012}, blow = {-10681.49742, -13.65584762}, ahigh = {1253216.926, -5021.091539999999, 12.47789314, -0.000689165525, 1.097738448e-007, -9.3064038e-012, 3.24260695e-016}, bhigh = {14531.55559, -53.06419819}, R = 193.2486285797427);
          constant IdealGases.Common.DataRecord HNO(name = "HNO", MM = 0.03101404, Hf = 3289888.224816889, H0 = 320560.6235111581, Tlimit = 1000, alow = {-68547.6486, 955.16272, -0.600072021, 0.007995176749999999, -6.54707916e-007, -3.6705134e-009, 1.783392519e-012}, blow = {6435.35126, 30.48166179}, ahigh = {-5795614.98, 19454.57427, -21.52568374, 0.01797428992, -4.97604067e-006, 6.397924169999999e-010, -3.142619368e-014}, bhigh = {-110419.2372, 181.8650338}, R = 268.0873565649622);
          constant IdealGases.Common.DataRecord HNO2(name = "HNO2", MM = 0.04701344, Hf = -1668712.648978675, H0 = 246680.2046393542, Tlimit = 1000, alow = {8591.985060000001, 120.3644046, 0.9412979119999999, 0.01942891839, -2.253174194e-005, 1.384587594e-008, -3.47355046e-012}, blow = {-11063.37202, 20.73967331}, ahigh = {878790.4129999999, -3990.45503, 11.87349269, -0.000488190061, 7.13363679e-008, -5.37630334e-012, 1.581778986e-016}, bhigh = {12463.43241, -46.08874688}, R = 176.8530871172158);
          constant IdealGases.Common.DataRecord HNO3(name = "HNO3", MM = 0.06301284, Hf = -2125167.965766977, H0 = 188475.7138386399, Tlimit = 1000, alow = {9202.86901, 109.3774496, -0.452104245, 0.02984914503, -3.1906355e-005, 1.720931528e-008, -3.78264983e-012}, blow = {-17640.48507, 27.46644879}, ahigh = {-94978.0964, -2733.024468, 14.49426995, -0.000782186805, 1.702693665e-007, -1.930543961e-011, 8.870455120000001e-016}, bhigh = {-4882.51778, -59.28392985000001}, R = 131.9488535987269);
          constant IdealGases.Common.DataRecord HOCL(name = "HOCL", MM = 0.05246034, Hf = -1443757.932182674, H0 = 194902.0155035213, Tlimit = 1000, alow = {-9739.307430000001, 354.756952, 0.1539514254, 0.01617051795, -2.179693631e-005, 1.509103049e-008, -4.12538351e-012}, blow = {-11763.23791, 24.73257759}, ahigh = {853045.781, -2847.760552, 7.94832904, -0.0001048782013, -1.482405043e-008, 4.59167827e-012, -3.060073987e-016}, bhigh = {7250.964950000001, -22.4983169}, R = 158.4906235834537);
          constant IdealGases.Common.DataRecord HOF(name = "HOF", MM = 0.0360057432, Hf = -2691187.915821163, H0 = 280189.6059737493, Tlimit = 1000, alow = {-36968.883, 780.900666, -2.077685317, 0.02038690173, -2.586919999e-005, 1.713797538e-008, -4.55128187e-012}, blow = {-16317.20064, 36.450525}, ahigh = {881201.823, -3120.013169, 8.223710069999999, -0.0002298036315, 1.459115709e-008, 1.095883303e-012, -1.404445608e-016}, bhigh = {6300.54671, -25.90150427}, R = 230.9207160039958);
          constant IdealGases.Common.DataRecord HO2(name = "HO2", MM = 0.03300674, Hf = 364168.0456779433, H0 = 0, Tlimit = 1000, alow = {-75988.8254, 1329.383918, -4.67738824, 0.02508308202, -3.006551588e-005, 1.895600056e-008, -4.82856739e-012}, blow = {0, -5873.35096}, ahigh = {10002.162, -1810669.724, 4963.19203, -1.039498992, 0.004560148530000001, -1.061859447e-006, 1.144567878e-010}, bhigh = {-4.76306416e-015, 0}, R = 251.9022478439252);
          constant IdealGases.Common.DataRecord HO2minus(name = "HO2minus", MM = 0.0330072886, Hf = -2966710.813077812, H0 = 0, Tlimit = 1000, alow = {110383.9835, -1047.963653, 6.36001399, 0.002942520461, -6.284141339999999e-006, 5.43825424e-009, -1.64730582e-012}, blow = {0, -7417.741590000001}, ahigh = {10245.32, 793330.6, -2503.312417, 7.54896233, 8.308390149999999e-005, -5.96973091e-008, 9.955377000000001e-012}, bhigh = {-5.60647728e-016, 0}, R = 251.8980610846054);
          constant IdealGases.Common.DataRecord HPO(name = "HPO", MM = 0.04798110100000001, Hf = -1185232.473093937, H0 = 209777.241251717, Tlimit = 1000, alow = {-38163.1412, 792.764187, -1.889940652, 0.01800907425, -1.857631793e-005, 1.018768867e-008, -2.355583619e-012}, blow = {-11576.4124, 36.9292591}, ahigh = {384245.945, -2434.951707, 8.582173920000001, -0.000405844857, -1.66948874e-008, 2.253640566e-011, -1.943063011e-015}, bhigh = {5761.63212, -26.49097544}, R = 173.2863945743971);
          constant IdealGases.Common.DataRecord HSO3F(name = "HSO3F", MM = 0.1000695432, Hf = -7525966.202272021, H0 = 150037.3991914055, Tlimit = 1000, alow = {6896.24213, -93.05810749999999, 1.570331788, 0.0378082075, -4.87272078e-005, 3.160796011e-008, -8.189782929999999e-012}, blow = {-91802.40820000001, 17.16315977}, ahigh = {554404.708, -3974.42432, 17.78869264, -0.000440932203, 5.94441139e-008, -3.92701581e-012, 8.891284000000001e-017}, bhigh = {-71534.3827, -76.13127849}, R = 83.08693868405707);
          constant IdealGases.Common.DataRecord H2(name = "H2", MM = 0.00201588, Hf = 0, H0 = 4200697.462150524, Tlimit = 1000, alow = {40783.2321, -800.918604, 8.21470201, -0.01269714457, 1.753605076e-005, -1.20286027e-008, 3.36809349e-012}, blow = {2682.484665, -30.43788844}, ahigh = {560812.801, -837.150474, 2.975364532, 0.001252249124, -3.74071619e-007, 5.936625200000001e-011, -3.6069941e-015}, bhigh = {5339.82441, -2.202774769}, R = 4124.487568704486);
          constant IdealGases.Common.DataRecord H2plus(name = "H2plus", MM = 0.0020153314, Hf = 741650941.3786734, H0 = 4258904.01945804, Tlimit = 1000, alow = {-31208.8606, 230.4622909, 3.33556442, -0.002419056763, 7.006022340000001e-006, -5.61001066e-009, 1.564169746e-012}, blow = {177410.4638, -0.8278523760000001}, ahigh = {1672225.964, -6595.18499, 12.79321925, -0.00550934526, 2.030669412e-006, -3.35102748e-010, 1.946089104e-014}, bhigh = {218999.9548, -67.9271078}, R = 4125.610309053885);
          constant IdealGases.Common.DataRecord H2minus(name = "H2minus", MM = 0.0020164286, Hf = 116626178.5812798, H0 = 4275178.402052024, Tlimit = 1000, alow = {-97535.65670000001, 1221.166236, -2.264588838, 0.01237202227, -1.12710002e-005, 5.36723995e-009, -1.04942016e-012}, blow = {21213.99948, 30.50556136}, ahigh = {95992.7562, -914.4682879999999, 5.14941881, -0.0001016559478, 5.446919560000001e-008, -6.155174449999999e-012, 2.822451181e-016}, bhigh = {32341.0518, -14.4078098}, R = 4123.365439272186);
          constant IdealGases.Common.DataRecord HBOH(name = "HBOH", MM = 0.02882628, Hf = -1690275.817760738, H0 = 379533.3979965504, Tlimit = 1000, alow = {-61596.4419, 1223.483035, -5.46037051, 0.0355538868, -4.67502014e-005, 3.23600885e-008, -8.98605332e-012}, blow = {-12636.59051, 54.2235229}, ahigh = {1534611.049, -5753.66643, 12.70893665, -0.000705962825, 1.027679375e-007, -7.67092674e-012, 2.211854768e-016}, bhigh = {27793.70419, -56.0886008}, R = 288.4337486488024);
          constant IdealGases.Common.DataRecord HCOOH(name = "HCOOH", MM = 0.04602538, Hf = -8225244.419492028, H0 = 237426.589416535, Tlimit = 1000, alow = {-29062.79097, 765.837888, -3.32841413, 0.02817542991, -2.370050804e-005, 1.166063663e-008, -2.79137317e-012}, blow = {-50064.4347, 43.8709423}, ahigh = {487233.645, -7632.238079999999, 21.32788153, -0.004402546540000001, 1.102001695e-006, -1.364343517e-010, 6.64842975e-015}, bhigh = {-5781.43191, -111.1790688}, R = 180.6497197850404);
          constant IdealGases.Common.DataRecord H2F2(name = "H2F2", MM = 0.0400126864, Hf = -14243576.95713228, H0 = 346625.8141567821, Tlimit = 1000, alow = {52592.1471, -991.3544890000001, 10.43577115, -0.002407796033, -6.376956159999999e-007, 2.7357849e-009, -1.10434859e-012}, blow = {-65724.60829999999, -30.38432132}, ahigh = {1464995.601, -3335.07492, 9.187487040000001, 0.001051127249, -3.27860557e-007, 4.45604623e-011, -2.281370136e-015}, bhigh = {-48254.42090000001, -26.39128168}, R = 207.7958954537979);
          constant IdealGases.Common.DataRecord H2O(name = "H2O", MM = 0.01801528, Hf = -13423382.81725291, H0 = 549760.6476280135, Tlimit = 1000, alow = {-39479.6083, 575.573102, 0.931782653, 0.00722271286, -7.34255737e-006, 4.95504349e-009, -1.336933246e-012}, blow = {-33039.7431, 17.24205775}, ahigh = {1034972.096, -2412.698562, 4.64611078, 0.002291998307, -6.836830479999999e-007, 9.426468930000001e-011, -4.82238053e-015}, bhigh = {-13842.86509, -7.97814851}, R = 461.5233290850878);
          constant IdealGases.Common.DataRecord H2Oplus(name = "H2Oplus", MM = 0.0180147314, Hf = 54488837.17466918, H0 = 551463.4539596855, Tlimit = 1000, alow = {-1753.89272, 224.9850054, 1.989400675, 0.00611789516, -7.09543664e-006, 5.54765947e-009, -1.704344789e-012}, blow = {115958.5952, 11.35409642}, ahigh = {622871.426, -2864.257487, 7.71756556, -0.000902780167, 6.17743686e-007, -1.201457479e-010, 7.407709940000001e-015}, bhigh = {134208.6651, -26.3661792}, R = 461.5373837880259);
          constant IdealGases.Common.DataRecord H2O2(name = "H2O2", MM = 0.03401468, Hf = -3994745.79799075, H0 = 328059.3849479107, Tlimit = 1000, alow = {-92795.3358, 1564.748385, -5.97646014, 0.0327074452, -3.93219326e-005, 2.509255235e-008, -6.46504529e-012}, blow = {-24940.04728, 58.7717418}, ahigh = {1489428.027, -5170.82178, 11.2820497, -8.04239779e-005, -1.818383769e-008, 6.94726559e-012, -4.8278319e-016}, bhigh = {14182.51038, -46.50855660000001}, R = 244.4377545224592);
          constant IdealGases.Common.DataRecord H2S(name = "H2S", MM = 0.03408088, Hf = -604444.4861752396, H0 = 292199.6439059085, Tlimit = 1000, alow = {9543.80881, -68.7517508, 4.05492196, -0.0003014557336, 3.76849775e-006, -2.239358925e-009, 3.086859108e-013}, blow = {-3278.45728, 1.415194691}, ahigh = {1430040.22, -5284.02865, 10.16182124, -0.000970384996, 2.154003405e-007, -2.1696957e-011, 9.318163070000001e-016}, bhigh = {29086.96214, -43.49160391}, R = 243.9629493135154);
          constant IdealGases.Common.DataRecord H2SO4(name = "H2SO4", MM = 0.09807848, Hf = -7470871.500047717, H0 = 168320.1962346888, Tlimit = 1000, alow = {-41291.5005, 668.158989, -2.632753507, 0.0541538248, -7.067502229999999e-005, 4.68461142e-008, -1.236791238e-011}, blow = {-93156.60120000001, 39.61096201}, ahigh = {1437877.914, -6614.90253, 21.57662058, -0.000480625597, 3.010775121e-008, 2.334842469e-012, -2.946330375e-016}, bhigh = {-52590.92950000001, -102.3603724}, R = 84.77366288710837);
          constant IdealGases.Common.DataRecord H2BOH(name = "H2BOH", MM = 0.02983422, Hf = -9708110.284096584, H0 = 356044.3678433691, Tlimit = 1000, alow = {-86867.6678, 1820.335089, -10.32373881, 0.0492280106, -5.97861991e-005, 3.94606273e-008, -1.068442257e-011}, blow = {-44152.3815, 79.86177289999999}, ahigh = {2294795.193, -9382.923559999999, 17.98228329, -0.001506116214, 2.635641095e-007, -2.483163637e-011, 9.736759120000001e-016}, bhigh = {20397.38953, -94.48201210000001}, R = 278.689102647899);
          constant IdealGases.Common.DataRecord HB_OH_2(name = "HB_OH_2", MM = 0.04583362000000001, Hf = -14060393.30954003, H0 = 265253.5845957617, Tlimit = 1000, alow = {-20670.44022, 953.4459160000001, -8.08895652, 0.0597242529, -8.071336320000001e-005, 5.61211835e-008, -1.553607566e-011}, blow = {-82642.70809999999, 65.3957164}, ahigh = {2122674.609, -8615.948939999998, 19.6617952, -0.00081484115, 8.955082040000001e-008, -3.34483051e-012, -6.919578139999999e-017}, bhigh = {-27887.90377, -99.8355961}, R = 181.4055272090662);
          constant IdealGases.Common.DataRecord H3BO3(name = "H3BO3", MM = 0.06183302, Hf = -16243108.32626322, H0 = 219299.6234050998, Tlimit = 1000, alow = {25689.01843, 113.8029495, -4.04509658, 0.0592452168, -8.148028410000001e-005, 5.65859329e-008, -1.54927705e-011}, blow = {-122170.205, 41.3222014}, ahigh = {2297369.132, -8933.57179, 21.93496552, -0.000309478349, -5.06405299e-008, 1.482296684e-011, -9.7644209e-016}, bhigh = {-69702.9847, -112.2292829}, R = 134.4665358412059);
          constant IdealGases.Common.DataRecord H3B3O3(name = "H3B3O3", MM = 0.08345502000000001, Hf = -14424069.24113133, H0 = 186966.8355480593, Tlimit = 1000, alow = {-198528.4188, 4104.13209, -28.27215617, 0.1269639253, -0.0001558505781, 9.96630407e-008, -2.60367765e-011}, blow = {-164849.4375, 176.3917407}, ahigh = {1286220.713, -10932.58784, 32.1351867, -0.002595402269, 5.35376647e-007, -5.833567379999999e-011, 2.600757913e-015}, bhigh = {-87528.5689, -177.0847815}, R = 99.62818294214057);
          constant IdealGases.Common.DataRecord H3B3O6(name = "H3B3O6", MM = 0.13145322, Hf = -17220480.49488632, H0 = 180268.3342408805, Tlimit = 1000, alow = {-20670.31503, 708.339154, -7.12382095, 0.1025049979, -0.0001295004602, 8.322737589999999e-008, -2.145746724e-011}, blow = {-277804.9216, 60.6359763}, ahigh = {1952433.352, -11549.32896, 38.9267611, -0.00111385228, 1.280317726e-007, -5.72846482e-012, -2.214529572e-017}, bhigh = {-212092.7136, -207.5566801}, R = 63.25042475186231);
          constant IdealGases.Common.DataRecord H3F3(name = "H3F3", MM = 0.0600190296, Hf = -14723276.86550934, H0 = 254303.0452461697, Tlimit = 1000, alow = {98885.62650000001, -1380.21288, 8.98045847, 0.01871609057, -3.127862508e-005, 2.589436165e-008, -8.04037227e-012}, blow = {-101366.1491, -25.85302557}, ahigh = {2515790.373, -8789.42332, 20.23933445, -0.001131704646, 1.692779921e-007, -1.308662105e-011, 3.978322e-016}, bhigh = {-54717.1365, -99.07787640000001}, R = 138.5305969691986);
          constant IdealGases.Common.DataRecord H3Oplus(name = "H3Oplus", MM = 0.0190226714, Hf = 31436173.57549476, H0 = 528129.1354273196, Tlimit = 1000, alow = {-64476.4015, 1181.817922, -3.80189306, 0.02220628313, -2.445343237e-005, 1.573297747e-008, -4.15883641e-012}, blow = {65306.1332, 42.8272313}, ahigh = {2955126.2, -9185.669409999999, 13.41398696, -0.000559033921, 1.138387119e-008, 7.25992721e-012, -6.13373436e-016}, bhigh = {129053.4257, -70.2182818}, R = 437.0822491314233);
          constant IdealGases.Common.DataRecord H4F4(name = "H4F4", MM = 0.08002537279999999, Hf = -14831950.09870669, H0 = 270590.1796186322, Tlimit = 1000, alow = {131510.9024, -1840.36156, 12.64093152, 0.0249540736, -4.17039461e-005, 3.45252384e-008, -1.07203436e-011}, blow = {-136400.3454, -41.2291353}, ahigh = {3354037.99, -11719.2204, 27.65243565, -0.001508934621, 2.257027572e-007, -1.744867255e-011, 5.304351740000001e-016}, bhigh = {-74202.13160000001, -138.8603332}, R = 103.897947726899);
          constant IdealGases.Common.DataRecord H5F5(name = "H5F5", MM = 0.100031716, Hf = -14897154.03862511, H0 = 280362.4602421096, Tlimit = 1000, alow = {164136.1897, -2300.510396, 16.30140545, 0.03119205422, -5.21292635e-005, 4.31561124e-008, -1.340031413e-011}, blow = {-171434.5408, -56.8215738}, ahigh = {4192290.73, -14649.03043, 35.0655492, -0.001886170471, 2.821289954e-007, -2.181090984e-011, 6.63047416e-016}, bhigh = {-93687.042, -178.8592044}, R = 83.11835818151914);
          constant IdealGases.Common.DataRecord H6F6(name = "H6F6", MM = 0.1200380592, Hf = -15041438.88224411, H0 = 286030.0577068977, Tlimit = 1000, alow = {195688.6618, -2753.90988, 19.75625055, 0.0380771701, -6.341218879999999e-005, 5.21923329e-008, -1.612185649e-011}, blow = {-207926.941, -71.13607980000001}, ahigh = {5024520.319999999, -17428.17868, 42.148705, -0.002084564791, 2.930051519e-007, -2.052079159e-011, 5.2069373e-016}, bhigh = {-115449.7653, -216.3947001}, R = 69.26529848459929);
          constant IdealGases.Common.DataRecord H7F7(name = "H7F7", MM = 0.1400444024, Hf = -14993094.60440098, H0 = 291530.7809546553, Tlimit = 1000, alow = {229386.7477, -3220.80786, 23.62235212, 0.0436680188, -7.29799034e-005, 6.041786420000001e-008, -1.876025627e-011}, blow = {-241863.7479, -88.38812259999999}, ahigh = {5868789.83, -20508.63473, 49.8917611, -0.002640634895, 3.94979633e-007, -3.053515088e-011, 9.28260178e-016}, bhigh = {-133017.7809, -259.2385142}, R = 59.37025584394225);
          constant IdealGases.Common.DataRecord He(name = "He", MM = 0.004002602, Hf = 0, H0 = 1548349.798456104, Tlimit = 1000, alow = {0, 0, 2.5, 0, 0, 0, 0}, blow = {-745.375, 0.9287239740000001}, ahigh = {0, 0, 2.5, 0, 0, 0, 0}, bhigh = {-745.375, 0.9287239740000001}, R = 2077.26673798694);
          constant IdealGases.Common.DataRecord Heplus(name = "Heplus", MM = 0.0040020534, Hf = 594325271.3719412, H0 = 1548562.045673853, Tlimit = 1000, alow = {0, 0, 2.5, 0, 0, 0, 0}, blow = {285323.3739, 1.621665557}, ahigh = {0, 0, 2.5, 0, 0, 0, 0}, bhigh = {285323.3739, 1.621665557}, R = 2077.551488943151);
          constant IdealGases.Common.DataRecord Hg(name = "Hg", MM = 0.20059, Hf = 305997.3079415724, H0 = 30895.99680941224, Tlimit = 1000, alow = {0, 0, 2.5, 0, 0, 0, 0}, blow = {6636.90008, 6.80020154}, ahigh = {51465.7351, -168.1269855, 2.718343098, -0.0001445026192, 5.15897766e-008, -9.47248501e-012, 7.034797406e-016}, bhigh = {7688.68493, 5.27123609}, R = 41.45008225734085);
          constant IdealGases.Common.DataRecord Hgplus(name = "Hgplus", MM = 0.2005894514, Hf = 5357425.928928983, H0 = 30896.08130809215, Tlimit = 1000, alow = {0.000409834494, -4.34358162e-006, 2.500000019, -4.389036810000001e-011, 5.5639692e-014, -3.63822826e-017, 9.607881994999999e-021}, blow = {128503.7483, 7.4933445}, ahigh = {-12299.84728, 27.32269908, 2.48418216, -4.42679761e-006, 7.489685859999999e-009, -2.549887287e-012, 2.819873366e-016}, bhigh = {128318.8257, 7.62524457}, R = 41.45019562080521);
          constant IdealGases.Common.DataRecord HgBr2(name = "HgBr2", MM = 0.3603980000000001, Hf = -253363.2706063851, H0 = 43447.47751097397, Tlimit = 1000, alow = {-1991.826537, -190.2186083, 8.246401260000001, -0.001607089392, 1.947654549e-006, -1.242873546e-009, 3.24181831e-013}, blow = {-12307.23096, -8.7166593}, ahigh = {-22436.4929, -3.31265227, 7.50276517, -1.202360213e-006, 2.830526999e-010, -3.4076867e-014, 1.640497921e-018}, bhigh = {-13274.83366, -4.3685556}, R = 23.07025011237576);
          constant IdealGases.Common.DataRecord I(name = "I", MM = 0.12690447, Hf = 841262.7230545938, H0 = 48835.37987274995, Tlimit = 1000, alow = {169.8199675, -2.716437233, 2.517385557, -5.73069207e-005, 1.031716184e-007, -9.670641930000001e-011, 3.706471651e-014}, blow = {12107.5009, 7.40582313}, ahigh = {-778586.0569999999, 2303.279568, 0.002886686091, 0.001180878463, -2.264074866e-007, 1.963511339e-011, -6.243525940999999e-016}, bhigh = {-2616.792742, 25.58922997}, R = 65.51756608730962);
          constant IdealGases.Common.DataRecord Iplus(name = "Iplus", MM = 0.1269039214, Hf = 8836220.470016144, H0 = 48835.59098592204, Tlimit = 1000, alow = {-801.496901, 8.13905261, 2.47017476, 4.16139382e-005, 7.133397859999999e-009, -7.10595014e-011, 4.873766102e-014}, blow = {134079.4213, 7.90342009}, ahigh = {-778838.5329999999, 2404.962651, -0.1791751142, 0.001227311979, -1.80149403e-007, 9.923983959999999e-012, -9.775286439000001e-017}, bhigh = {118853.1631, 27.10544347}, R = 65.5178493168297);
          constant IdealGases.Common.DataRecord Iminus(name = "Iminus", MM = 0.1269050186, Hf = -1533395.401905721, H0 = 48835.1687614031, Tlimit = 1000, alow = {0, 0, 2.5, 0, 0, 0, 0}, blow = {-24149.70936, 6.11346538}, ahigh = {0, 0, 2.5, 0, 0, 0, 0}, bhigh = {-24149.70936, 6.11346538}, R = 65.51728286023828);
          constant IdealGases.Common.DataRecord IF5(name = "IF5", MM = 0.221896486, Hf = -3790055.512641151, H0 = 90331.04742361714, Tlimit = 1000, alow = {202618.453, -3598.18317, 25.30349738, -0.01348602458, 1.105687229e-005, -4.70793951e-009, 7.86441147e-013}, blow = {-87001.4725, -111.2719941}, ahigh = {-362050.626, -163.9236093, 16.12272743, -4.91443155e-005, 1.086015052e-008, -1.244672691e-012, 5.76359362e-017}, bhigh = {-106164.2293, -53.9877336}, R = 37.47004808359156);
          constant IdealGases.Common.DataRecord IF7(name = "IF7", MM = 0.2598932924, Hf = -3699595.288208369, H0 = 92820.14467257561, Tlimit = 1000, alow = {299985.3564, -5501.55413, 36.2713398, -0.02095765818, 1.76197268e-005, -7.82914233e-009, 1.405939372e-012}, blow = {-93313.0766, -175.421615}, ahigh = {-561442.264, -264.6288326, 22.19879891, -7.984221469999999e-005, 1.768835964e-008, -2.031523755e-012, 9.423767390000001e-017}, bhigh = {-122524.5322, -87.74010730000001}, R = 31.99186836728072);
          constant IdealGases.Common.DataRecord I2(name = "I2", MM = 0.25380894, Hf = 245933.0234782116, H0 = 39857.23276729338, Tlimit = 1000, alow = {-5087.96877, -12.4958521, 4.50421909, 0.0001370962533, -1.390523014e-007, 1.174813853e-010, -2.337541043e-014}, blow = {6213.469810000001, 5.58383694}, ahigh = {-5632594.16, 17939.6156, -17.23055169, 0.0124421408, -3.33276858e-006, 4.12547794e-010, -1.960461713e-014}, bhigh = {-106850.5292, 160.0531883}, R = 32.75878304365481);
          constant IdealGases.Common.DataRecord In(name = "In", MM = 0.114818, Hf = 2096361.197721611, H0 = 53986.73552927241, Tlimit = 1000, alow = {51495.1805, -917.799902, 8.93868795, -0.02224112602, 3.82388194e-005, -2.890116948e-008, 8.047481221e-012}, blow = {32390.3162, -27.64567028}, ahigh = {-1683608.899, 2210.473186, 3.47221937, -0.001082267422, 3.47969998e-007, -5.15809241e-011, 3.183043089e-015}, bhigh = {10959.65206, 2.557189088}, R = 72.41436011775157);
          constant IdealGases.Common.DataRecord Inplus(name = "Inplus", MM = 0.1148174514, Hf = 60935.2055344437, H0 = 53976.35920701163, Tlimit = 1000, alow = {0.2150488908, -0.002523854672, 2.500011877, -2.869063919e-008, 3.7538704e-011, -2.525942419e-014, 6.84293266e-018}, blow = {96.1093017, 5.9632544}, ahigh = {45369.9514, -144.5142103, 2.681812441, -0.000115748006, 3.94771481e-008, -6.87924362e-012, 4.81920492e-016}, bhigh = {1004.26144, 4.68375918}, R = 72.4147061149626);
          constant IdealGases.Common.DataRecord InBr(name = "InBr", MM = 0.194722, Hf = -277913.5331395528, H0 = 51787.27108390423, Tlimit = 1000, alow = {-2540.851435, -52.94704710000001, 4.71743809, -0.000434466949, 6.51351364e-007, -4.40399515e-010, 1.234046209e-013}, blow = {-7607.59428, 4.258881}, ahigh = {794837.442, -2318.183711, 7.03288958, -0.001245920568, 3.27669335e-007, -3.003771106e-011, 5.85319277e-016}, bhigh = {6957.627289999999, -12.7350266}, R = 42.69919166812174);
          constant IdealGases.Common.DataRecord InBr2(name = "InBr2", MM = 0.274626, Hf = -545210.7921318448, H0 = 52523.58116128844, Tlimit = 1000, alow = {-66.19878900000001, -236.589731, 7.92654516, -0.001994547959, 2.420414153e-006, -1.548924922e-009, 4.05814839e-013}, blow = {-18953.55588, -5.33886754}, ahigh = {-321716.203, 479.2682100000001, 7.10264948, -0.000551401978, 3.157862421e-007, -5.51125451e-011, 3.17216139e-015}, bhigh = {-23709.23659, 0.1193263347}, R = 30.27561847749303);
          constant IdealGases.Common.DataRecord InBr3(name = "InBr3", MM = 0.35453, Hf = -723739.2237610358, H0 = 56166.56700420276, Tlimit = 1000, alow = {-4990.1153, -280.5930452, 11.1035606, -0.002379995808, 2.887825281e-006, -1.844500707e-009, 4.81439587e-013}, blow = {-32505.0491, -18.71748987}, ahigh = {-35057.831, -4.85872132, 10.00406131, -1.767620958e-006, 4.16401606e-010, -5.01552887e-014, 2.415414123e-018}, bhigh = {-33931.7924, -12.29069868}, R = 23.45209714269597);
          constant IdealGases.Common.DataRecord InCL(name = "InCL", MM = 0.150271, Hf = -480118.4726261222, H0 = 64896.86632816712, Tlimit = 1000, alow = {6430.71303, -217.8267726, 5.4188309, -0.0020480051, 2.702295936e-006, -1.795319798e-009, 4.84746909e-013}, blow = {-8959.84107, -1.202999253}, ahigh = {-274575.1636, 894.0986909999999, 3.26811839, 0.0009180277780000001, -3.083262431e-007, 5.74545881e-011, -3.64527575e-015}, bhigh = {-15616.0343, 12.66880374}, R = 55.32985073633636);
          constant IdealGases.Common.DataRecord InCL2(name = "InCL2", MM = 0.185724, Hf = -1084853.955331567, H0 = 75145.82929508302, Tlimit = 1000, alow = {9935.164869999999, -417.210186, 8.58849491, -0.00334838561, 3.99786664e-006, -2.525301853e-009, 6.54449033e-013}, blow = {-24264.80269, -11.64521725}, ahigh = {-332871.984, 475.769161, 7.10544369, -0.000552578696, 3.160567738e-007, -5.51445368e-011, 3.17368038e-015}, bhigh = {-29950.95917, -2.294555511}, R = 44.76789214102647);
          constant IdealGases.Common.DataRecord InCL3(name = "InCL3", MM = 0.221177, Hf = -1671481.333954254, H0 = 82769.78618934158, Tlimit = 1000, alow = {26147.43044, -841.460456, 13.1178479, -0.00643528247, 7.55487475e-006, -4.70507856e-009, 1.204213663e-012}, blow = {-43264.6958, -35.0008094}, ahigh = {-71140.4363, -16.28425068, 10.01309622, -5.54452888e-006, 1.279877732e-009, -1.518261505e-013, 7.22656277e-018}, bhigh = {-47589.7417, -16.69761011}, R = 37.59193767887258);
          constant IdealGases.Common.DataRecord InF(name = "InF", MM = 0.1338164032, Hf = -1445413.150964142, H0 = 68878.7830160421, Tlimit = 1000, alow = {31901.2332, -529.525426, 6.17288626, -0.002986035832, 3.23377803e-006, -1.856975575e-009, 4.46502497e-013}, blow = {-21871.76282, -7.59121662}, ahigh = {-468900.12, 1321.019051, 2.881329353, 0.001041591078, -3.090754867e-007, 4.92664656e-011, -2.699156425e-015}, bhigh = {-33083.6981, 13.96688303}, R = 62.13342909518585);
          constant IdealGases.Common.DataRecord InF2(name = "InF2", MM = 0.1528148064, Hf = -2991770.462367971, H0 = 82379.29489010562, Tlimit = 1000, alow = {75224.8021, -1088.616881, 9.481705209999999, -0.002893229367, 1.516224649e-006, -7.98951781e-011, -1.466781026e-013}, blow = {-51243.36150000001, -21.4666609}, ahigh = {431896.034, -1688.47242, 8.925863010000001, -0.001049188835, 2.72393419e-007, -2.882959902e-011, 1.073493779e-015}, bhigh = {-46736.91940000001, -19.32464363}, R = 54.40881152731022);
          constant IdealGases.Common.DataRecord InF3(name = "InF3", MM = 0.1718132096, Hf = -5023359.775475611, H0 = 94989.40179277113, Tlimit = 1000, alow = {107160.8336, -1660.071901, 13.95972813, -0.00501869326, 3.22636753e-006, -7.823385970000001e-010, -3.7269998e-014}, blow = {-97952.1499, -46.06353470000001}, ahigh = {-165685.5506, -79.6588929, 10.05913347, -2.350816639e-005, 5.16351293e-009, -5.88804881e-013, 2.715090697e-017}, bhigh = {-106868.2627, -21.32968218}, R = 48.39250730113827);
          constant IdealGases.Common.DataRecord InH(name = "InH", MM = 0.11582594, Hf = 1856379.529490544, H0 = 74975.46749890395, Tlimit = 1000, alow = {-51528.2872, 781.3792590000001, -0.8433385339999999, 0.01044471274, -9.761074819999999e-006, 4.540635199999999e-009, -8.169906900000001e-013}, blow = {21100.48908, 29.39346222}, ahigh = {779740.666, -3646.88208, 9.54296735, -0.00348128172, 1.304805603e-006, -2.182950701e-010, 1.274009748e-014}, bhigh = {46432.9546, -36.5932923}, R = 71.78419618265131);
          constant IdealGases.Common.DataRecord InI(name = "InI", MM = 0.24172247, Hf = 109287.0017421219, H0 = 42491.39105685955, Tlimit = 1000, alow = {-468.634874, -56.8553099, 4.78184969, -0.000661936831, 1.027666742e-006, -7.27059207e-010, 2.077991102e-013}, blow = {2095.570254, 4.88551513}, ahigh = {1529221.772, -4690.373680000001, 10.04593462, -0.00314043411, 9.460834390000001e-007, -1.253862848e-010, 5.94980564e-015}, bhigh = {31524.78143, -32.9769749}, R = 34.39676915431156);
          constant IdealGases.Common.DataRecord InI2(name = "InI2", MM = 0.36862694, Hf = -107047.637375608, H0 = 40454.87559862011, Tlimit = 1000, alow = {-4513.8906, -118.372687, 7.47311008, -0.00103447409, 1.271465534e-006, -8.2278203e-010, 2.178307019e-013}, blow = {-6278.53604, -0.66672975}, ahigh = {-313067.1665, 481.38107, 7.10091418, -0.000550655992, 3.156120735e-007, -5.50917054e-011, 3.17116283e-015}, bhigh = {-10430.1275, 2.141662307}, R = 22.55524786115741);
          constant IdealGases.Common.DataRecord InI3(name = "InI3", MM = 0.49553141, Hf = -212773.4223749813, H0 = 42080.25077562692, Tlimit = 1000, alow = {-8728.397169999998, -126.5914859, 10.5065912, -0.001106109314, 1.354335766e-006, -8.709453460000001e-010, 2.28518635e-013}, blow = {-15082.69063, -12.11411745}, ahigh = {-21967.42567, -2.151167521, 10.001822, -8.002214699999999e-007, 1.897343164e-010, -2.29625246e-014, 1.109838196e-018}, bhigh = {-15724.06335, -9.17088869}, R = 16.7789000499484);
          constant IdealGases.Common.DataRecord InO(name = "InO", MM = 0.1308174, Hf = 1116006.807962855, H0 = 69120.07882743428, Tlimit = 1000, alow = {-115472.7169, 1753.668703, -6.96503381, 0.02961698958, -3.123332334e-005, 1.43724472e-008, -2.235519898e-012}, blow = {8188.67959, 66.0862126}, ahigh = {-805213.0950000001, 1693.064203, 4.76086358, -0.00073407184, 3.22959776e-007, -4.58015038e-011, 1.954611725e-015}, bhigh = {4005.51981, 3.50074246}, R = 63.55784475153917);
          constant IdealGases.Common.DataRecord InOH(name = "InOH", MM = 0.13182534, Hf = -944032.0351155552, H0 = 81447.45919107813, Tlimit = 1000, alow = {61234.6302, -1195.863129, 10.41427981, -0.0068700025, 6.21412878e-006, -2.408203181e-009, 3.17764178e-013}, blow = {-10798.5217, -31.8989084}, ahigh = {852512.8929999999, -2350.317172, 7.98451514, 9.91581562e-005, -6.21238545e-008, 1.016169575e-011, -5.682557639999999e-016}, bhigh = {-1536.21361, -20.43107643}, R = 63.07187980702344);
          constant IdealGases.Common.DataRecord In2Br2(name = "In2Br2", MM = 0.389444, Hf = -504065.321329896, H0 = 55039.85938928319, Tlimit = 1000, alow = {-9221.1883, -36.1667703, 10.14728771, -0.00032557752, 4.02232748e-007, -2.604080237e-010, 6.86768955e-014}, blow = {-26448.83479, -10.00360073}, ahigh = {-12907.81804, -0.607540124, 10.00052163, -2.312058331e-007, 5.51731572e-011, -6.708620190000001e-015, 3.25384165e-019}, bhigh = {-26631.36878, -9.149949469999999}, R = 21.34959583406087);
          constant IdealGases.Common.DataRecord In2Br4(name = "In2Br4", MM = 0.549252, Hf = -794732.9877724615, H0 = 57893.49333275072, Tlimit = 1000, alow = {-14255.55379, -286.6009341, 17.13866292, -0.002473506655, 3.017273071e-006, -1.93490043e-009, 5.06587425e-013}, blow = {-55937.5306, -38.3710598}, ahigh = {-44534.6371, -4.91152398, 16.00413764, -1.810579975e-006, 4.28171574e-010, -5.17199802e-014, 2.496142837e-018}, bhigh = {-57391.7541, -31.7490639}, R = 15.13780923874652);
          constant IdealGases.Common.DataRecord In2Br6(name = "In2Br6", MM = 0.7090599999999999, Hf = -886643.4857416863, H0 = 60780.08631145462, Tlimit = 1000, alow = {-17589.40752, -485.903448, 23.91654505, -0.00414206149, 5.03384932e-006, -3.21910365e-009, 8.410220069999999e-013}, blow = {-79888.4446, -64.3787188}, ahigh = {-69450.3948, -8.402368170000001, 22.00703884, -3.068251799e-006, 7.23596573e-010, -8.722870580000001e-014, 4.20346991e-018}, bhigh = {-82357.59850000001, -53.221928}, R = 11.72604857134799);
          constant IdealGases.Common.DataRecord In2CL2(name = "In2CL2", MM = 0.300542, Hf = -772528.8811547139, H0 = 67291.41018559803, Tlimit = 1000, alow = {-11428.48747, -141.6015827, 10.56853314, -0.00124425344, 1.526076784e-006, -9.82640281e-010, 2.58076699e-013}, blow = {-30263.27087, -15.6510496}, ahigh = {-26166.06332, -2.401955489, 10.00203984, -8.97519003e-007, 2.130755553e-010, -2.5811409e-014, 1.248404822e-018}, bhigh = {-30980.16735, -12.34946505}, R = 27.66492536816818);
          constant IdealGases.Common.DataRecord In2CL4(name = "In2CL4", MM = 0.371448, Hf = -1559103.443819862, H0 = 79301.88074777628, Tlimit = 1000, alow = {9814.08308, -848.895901, 19.2341036, -0.00681567424, 8.129220730000001e-006, -5.12507685e-009, 1.324324755e-012}, blow = {-70276.8814, -55.82665695}, ahigh = {-85037.7179, -15.52464614, 16.01269668, -5.44154987e-006, 1.267594997e-009, -1.514110365e-013, 7.24540454e-018}, bhigh = {-74619.21740000001, -36.91150615}, R = 22.38394607051324);
          constant IdealGases.Common.DataRecord In2CL6(name = "In2CL6", MM = 0.442354, Hf = -1994646.886882452, H0 = 86861.3418212563, Tlimit = 1000, alow = {30046.63159, -1585.621359, 27.9648947, -0.01245506052, 1.475341213e-005, -9.2526892e-009, 2.381233885e-012}, blow = {-104882.83, -97.88404639999999}, ahigh = {-149940.0484, -29.88680816, 22.02425362, -1.033652301e-005, 2.397911922e-009, -2.855307516e-013, 1.363051661e-017}, bhigh = {-113011.0233, -62.93985679999999}, R = 18.79596883943629);
          constant IdealGases.Common.DataRecord In2F2(name = "In2F2", MM = 0.2676328064, Hf = -1988673.612772758, H0 = 67405.41356890992, Tlimit = 1000, alow = {7255.7171, -644.826565, 12.47539512, -0.00524608962, 6.28361621e-006, -3.97434234e-009, 1.029559356e-012}, blow = {-63848.92449999999, -31.08004586}, ahigh = {-64097.1544, -11.63740672, 10.00956511, -4.11408691e-006, 9.60882832e-010, -1.150013486e-013, 5.511437479999999e-018}, bhigh = {-67142.7785, -16.61718474}, R = 31.06671454759293);
          constant IdealGases.Common.DataRecord In2F4(name = "In2F4", MM = 0.3056296128, Hf = -4203742.282135299, H0 = 81023.15012323308, Tlimit = 1000, alow = {122521.3611, -2444.998374, 23.42425457, -0.0129142712, 1.308700476e-005, -7.18167393e-009, 1.648386404e-012}, blow = {-146694.2111, -89.43560050000001}, ahigh = {-220597.7486, -78.2208377, 16.05923872, -2.39402324e-005, 5.32958279e-009, -6.14460957e-013, 2.859099598e-017}, bhigh = {-159571.6347, -44.6957458}, R = 27.20440576365511);
          constant IdealGases.Common.DataRecord In2F6(name = "In2F6", MM = 0.3436264192, Hf = -5703868.767026397, H0 = 97684.86974356599, Tlimit = 1000, alow = {182549.0583, -3552.11171, 32.75425230000001, -0.01868064635, 1.892443162e-005, -1.038916311e-008, 2.386637271e-012}, blow = {-223964.876, -136.3626758}, ahigh = {-317096.638, -116.7440973, 22.08847456, -3.57773393e-005, 7.96895904e-009, -9.191681699999999e-013, 4.278494150000001e-017}, bhigh = {-242666.1118, -71.5532856}, R = 24.19625365056914);
          constant IdealGases.Common.DataRecord In2I2(name = "In2I2", MM = 0.48344494, Hf = -57531.93734947356, H0 = 45627.69030119541, Tlimit = 1000, alow = {-6681.92501, -14.49103222, 10.05929122, -0.0001314921486, 1.628373589e-007, -1.056087811e-010, 2.788963879e-014}, blow = {-6279.602599999999, -7.29113098}, ahigh = {-8148.157299999999, -0.2444544443, 10.00021086, -9.37678925e-008, 2.24298818e-011, -2.732299303e-015, 1.327135699e-019}, bhigh = {-6352.6515, -6.94772542}, R = 17.19838457715578);
          constant IdealGases.Common.DataRecord In2I4(name = "In2I4", MM = 0.73725388, Hf = -270114.7954623175, H0 = 45396.81635856565, Tlimit = 1000, alow = {-15010.28064, -97.447979, 16.3938395, -0.000865938021, 1.065671801e-006, -6.87925268e-010, 1.810240886e-013}, blow = {-28303.88443, -29.86294828}, ahigh = {-25056.99251, -1.642155234, 16.00140149, -6.18706628e-007, 1.472308565e-010, -1.786593333e-014, 8.652338779999999e-019}, bhigh = {-28796.55333, -27.57790375}, R = 11.2776239305787);
          constant IdealGases.Common.DataRecord In2I6(name = "In2I6", MM = 0.99106282, Hf = -322603.2028928297, H0 = 45494.71445210709, Tlimit = 1000, alow = {-22326.40607, -200.0786874, 22.80417927, -0.001761331313, 2.161510393e-006, -1.392400447e-009, 3.65817188e-013}, blow = {-44125.4768, -52.7380203}, ahigh = {-43119.0585, -3.39035059, 22.00288128, -1.268348706e-006, 3.0121219e-010, -3.64967671e-014, 1.765530622e-018}, bhigh = {-45138.2027, -48.0686837}, R = 8.389450024974199);
          constant IdealGases.Common.DataRecord In2O(name = "In2O", MM = 0.2456354, Hf = -141525.6392197542, H0 = 52095.52043394397, Tlimit = 1000, alow = {50656.535, -807.036367, 8.64206727, -0.001533035446, 2.866419382e-007, 4.98113497e-010, -2.554320291e-013}, blow = {-1924.908459, -14.90718759}, ahigh = {-92327.8636, -52.0112728, 7.03872362, -1.543821179e-005, 3.39970087e-009, -3.8854571e-013, 1.79512527e-017}, bhigh = {-6270.9268, -4.42182041}, R = 33.84883449209683);
          constant IdealGases.Common.DataRecord K(name = "K", MM = 0.0390983, Hf = 2276313.803924979, H0 = 158508.8865756312, Tlimit = 1000, alow = {9.665143929999999, -0.1458059455, 2.500865861, -2.601219276e-006, 4.187306579999999e-009, -3.43972211e-012, 1.131569009e-015}, blow = {9959.493490000001, 5.03582226}, ahigh = {-3566422.36, 10852.89825, -10.54134898, 0.00800980135, -2.696681041e-006, 4.71529415e-010, -2.97689735e-014}, bhigh = {-58753.3701, 97.3855124}, R = 212.6555886061542);
          constant IdealGases.Common.DataRecord Kplus(name = "Kplus", MM = 0.0390977514, Hf = 13146728.63769859, H0 = 158511.1106926727, Tlimit = 1000, alow = {0, 0, 2.5, 0, 0, 0, 0}, blow = {61075.1686, 4.34740444}, ahigh = {0, 0, 2.5, 0, 0, 0, 0}, bhigh = {61075.1686, 4.34740444}, R = 212.6585724825086);
          constant IdealGases.Common.DataRecord Kminus(name = "Kminus", MM = 0.0390988486, Hf = 880284.9503859814, H0 = 158506.6625210033, Tlimit = 1000, alow = {0, 0, 2.5, 0, 0, 0, 0}, blow = {3394.15071, 4.34744653}, ahigh = {0, 0, 2.5, 0, 0, 0, 0}, bhigh = {3394.15071, 4.34744653}, R = 212.6526048135341);
          constant IdealGases.Common.DataRecord KALF4(name = "KALF4", MM = 0.1420734508, Hf = -13428665.32245868, H0 = 150776.8754779904, Tlimit = 1000, alow = {119932.7246, -2156.457352, 18.15348004, 0.003115762809, -9.51199167e-006, 8.43175623e-009, -2.615583061e-012}, blow = {-222254.5809, -68.23299969999999}, ahigh = {-343513.537, -242.0680834, 16.1802925, -7.19381768e-005, 1.585712708e-008, -1.814017733e-012, 8.388391179999999e-017}, bhigh = {-233965.8121, -52.338987}, R = 58.5223484977814);
          constant IdealGases.Common.DataRecord KBO2(name = "KBO2", MM = 0.08190810000000001, Hf = -8155765.998722958, H0 = 172030.641658151, Tlimit = 1000, alow = {42963.08650000001, -720.961643, 8.34963935, 0.002407820401, -1.680003427e-007, -1.231930486e-009, 5.479951640000001e-013}, blow = {-78685.2347, -14.76077068}, ahigh = {88907.266, -1671.432436, 11.1748393, -0.000451376026, 9.69900346e-008, -1.09002769e-011, 4.9768458e-016}, bhigh = {-73753.36080000001, -32.7474937}, R = 101.5097652124759);
          constant IdealGases.Common.DataRecord KBr(name = "KBr", MM = 0.1190023, Hf = -1506280.786169679, H0 = 84948.96317130004, Tlimit = 1000, alow = {9203.30919, -213.7880361, 5.58850915, -0.002753854319, 4.013672610000001e-006, -2.856323127e-009, 8.13520974e-013}, blow = {-21883.87013, -1.707501588}, ahigh = {1562614.367, -4384.89478, 9.045464600000001, -0.0020456809, 4.40192385e-007, -1.448178289e-011, -2.27333876e-015}, bhigh = {5315.34201, -28.64647026}, R = 69.86816221199086);
          constant IdealGases.Common.DataRecord KCN(name = "KCN", MM = 0.0651157, Hf = 1220842.285347466, H0 = 191295.4018769667, Tlimit = 1000, alow = {17986.27669, -630.055897, 9.988695, -0.01076486132, 1.75101498e-005, -1.265592431e-008, 3.46976281e-012}, blow = {10580.25371, -25.94060289}, ahigh = {361566.387, -1749.018011, 8.680702670000001, -0.000440235004, 9.249499710000001e-008, -1.021916855e-011, 4.604767189999999e-016}, bhigh = {18137.34856, -22.81619241}, R = 127.6876697939207);
          constant IdealGases.Common.DataRecord KCL(name = "KCL", MM = 0.0745513, Hf = -2878217.831211528, H0 = 132594.8575008082, Tlimit = 1000, alow = {9058.35151, -245.6801212, 5.68069619, -0.002900127425, 4.13098306e-006, -2.907340629e-009, 8.22385087e-013}, blow = {-25973.04884, -3.677976854}, ahigh = {-212294.5722, 934.61589, 2.866264958, 0.001468386693, -5.83426078e-007, 1.255777709e-010, -9.150147999999999e-015}, bhigh = {-32737.8764, 14.01864636}, R = 111.5268546624942);
          constant IdealGases.Common.DataRecord KF(name = "KF", MM = 0.0580967032, Hf = -5653416.939500278, H0 = 162747.8579541842, Tlimit = 1000, alow = {14357.04906, -339.184823, 5.72790672, -0.002518371562, 3.26591564e-006, -2.21011086e-009, 6.21204205e-013}, blow = {-39142.5442, -5.81099962}, ahigh = {-1483743.237, 4550.04804, -1.081464319, 0.00350396588, -1.093902537e-006, 1.771028824e-010, -1.029032783e-014}, bhigh = {-69633.2981, 40.8863592}, R = 143.1143514525623);
          constant IdealGases.Common.DataRecord KH(name = "KH", MM = 0.04010624, Hf = 3126673.205964957, H0 = 219295.2019436377, Tlimit = 1000, alow = {223.778215, 179.3554623, 1.130735415, 0.009966112559999999, -1.341218171e-005, 9.034529940000001e-009, -2.40988777e-012}, blow = {13382.50358, 15.52739897}, ahigh = {-3752276.52, 11727.78444, -10.14137678, 0.008854548899999998, -2.517188074e-006, 3.34407236e-010, -1.701157835e-014}, bhigh = {-60251.033, 101.030459}, R = 207.3111815019309);
          constant IdealGases.Common.DataRecord KI(name = "KI", MM = 0.16600277, Hf = -773817.6718376447, H0 = 61710.55458893849, Tlimit = 1000, alow = {-457.281174, -63.31769730000001, 4.82329242, -0.0007876668320000001, 1.336737911e-006, -1.000530992e-009, 3.004503956e-013}, blow = {-16503.40769, 3.55259482}, ahigh = {3293747.78, -10028.24194, 16.29302044, -0.00669639675, 2.001412089e-006, -2.677673023e-010, 1.273359646e-014}, bhigh = {46763.4867, -78.5912337}, R = 50.08634494472592);
          constant IdealGases.Common.DataRecord KLi(name = "KLi", MM = 0.0460393, Hf = 3707752.355053183, H0 = 221487.3597122459, Tlimit = 1000, alow = {-3426.04369, -19.86063081, 4.48760705, 0.0005249535550000001, -1.207990165e-006, 1.705161047e-009, -7.879994210000001e-013}, blow = {19278.68247, 1.912285942}, ahigh = {12112968.43, -40810.0487, 56.8379212, -0.03128695515, 8.975062200000001e-006, -1.189303691e-009, 5.88012918e-014}, bhigh = {273879.2595, -366.96936}, R = 180.5951002730276);
          constant IdealGases.Common.DataRecord KNO2(name = "KNO2", MM = 0.08510380000000001, Hf = -2261913.028560417, H0 = 180281.0920311431, Tlimit = 1000, alow = {-72462.36930000001, 1226.722811, -2.258926457, 0.03019643903, -3.64025995e-005, 2.219135205e-008, -5.471654230000001e-012}, blow = {-30772.6896, 45.37172549}, ahigh = {-168941.4093, -851.5973150000001, 10.63209586, -0.000252325481, 5.57364441e-008, -6.39318592e-012, 2.964484049e-016}, bhigh = {-21877.01436, -27.55200432}, R = 97.69801113463794);
          constant IdealGases.Common.DataRecord KNO3(name = "KNO3", MM = 0.1011032, Hf = -3123866.386029324, H0 = 157434.304749998, Tlimit = 1000, alow = {-25961.12303, 820.060661, -2.886015624, 0.042072889, -5.27032309e-005, 3.30856173e-008, -8.36586037e-012}, blow = {-43350.5447, 46.03187331}, ahigh = {-318368.287, -1375.234449, 14.01587531, -0.000404066008, 8.90084778e-008, -1.018771993e-011, 4.71598202e-016}, bhigh = {-35138.5322, -48.04256439}, R = 82.23747616297011);
          constant IdealGases.Common.DataRecord KNa(name = "KNa", MM = 0.06208807, Hf = 2132524.444712164, H0 = 170308.5149852459, Tlimit = 1000, alow = {25424.05292, -411.920741, 6.84108795, -0.00625048138, 8.86101314e-006, -4.922415349999999e-009, 6.493212469999999e-013}, blow = {16525.9989, -9.13616691}, ahigh = {6260326.62, -25635.93614, 43.7333545, -0.0267265511, 8.23270579e-006, -1.12850359e-009, 5.66326489e-014}, bhigh = {169727.4909, -266.4117219}, R = 133.9141641864532);
          constant IdealGases.Common.DataRecord KO(name = "KO", MM = 0.05509770000000001, Hf = 1174882.327211481, H0 = 172078.2174210539, Tlimit = 1000, alow = {14625.62908, -338.476565, 5.71660764, -0.002363265083, 2.848716276e-006, -1.739858233e-009, 4.431006520000001e-013}, blow = {8141.83538, -4.02210152}, ahigh = {696010.338, -3304.83529, 10.05743444, -0.004331112, 1.747281632e-006, -3.012370548e-010, 1.79082787e-014}, bhigh = {26049.72496, -34.4878152}, R = 150.9041575238168);
          constant IdealGases.Common.DataRecord KOH(name = "KOH", MM = 0.05610564, Hf = -4135056.653840862, H0 = 208083.3584644966, Tlimit = 1000, alow = {17706.84196, -615.320522, 8.684075719999999, -0.00396284951, 3.40865059e-006, -9.60197222e-010, 8.494054970000001e-015}, blow = {-26779.03261, -21.74495666}, ahigh = {891727.195, -2334.179072, 7.97257871, 0.0001038863156, -6.315893469999999e-008, 1.027938106e-011, -5.73668582e-016}, bhigh = {-14436.96469, -20.76401416}, R = 148.1931584774721);
          constant IdealGases.Common.DataRecord K2(name = "K2", MM = 0.07819660000000001, Hf = 1618309.862577145, H0 = 137360.5629912298, Tlimit = 1000, alow = {15241.69293, -330.178936, 7.07079595, -0.00976707246, 2.021535863e-005, -1.886092452e-008, 6.11297464e-012}, blow = {15334.02849, -9.1010358}, ahigh = {-27344707.45, 65621.80009999999, -44.7635044, 0.008938859150000001, 2.984557092e-006, -1.064158914e-009, 8.334936929999999e-014}, bhigh = {-422624.383, 386.714251}, R = 106.3277943030771);
          constant IdealGases.Common.DataRecord K2plus(name = "K2plus", MM = 0.07819605140000001, Hf = 6709555.298594016, H0 = 138896.1565903314, Tlimit = 1000, alow = {51960.3657, -611.338253, 7.26499054, -0.00581063482, 6.5674965e-006, -2.378020865e-009, -1.318637581e-013}, blow = {64798.2027, -10.42370517}, ahigh = {11079507.39, -41774.8382, 64.48659840000001, -0.0402499803, 1.360587923e-005, -2.361920107e-009, 1.67343061e-013}, bhigh = {317761.205, -410.50631}, R = 106.3285402669322);
          constant IdealGases.Common.DataRecord K2Br2(name = "K2Br2", MM = 0.2380046, Hf = -2263587.31301832, H0 = 88031.0968779595, Tlimit = 1000, alow = {-10930.40504, -48.5665148, 10.19754923, -0.00043631571, 5.38717366e-007, -3.4861165e-010, 9.19070295e-014}, blow = {-67580.73239999999, -12.94944706}, ahigh = {-15892.29976, -0.810137946, 10.00069442, -3.074491521e-007, 7.33102416e-011, -8.908944780000001e-015, 4.319245429999999e-019}, bhigh = {-67825.9544, -11.80425726}, R = 34.93408110599543);
          constant IdealGases.Common.DataRecord K2CO3(name = "K2CO3", MM = 0.1382055, Hf = -5872770.352844134, H0 = 141208.8303287496, Tlimit = 1000, alow = {-44074.067, 706.981544, 0.693148247, 0.0396783132, -4.8509583e-005, 2.976635787e-008, -7.374761529999999e-012}, blow = {-103391.3232, 29.80651689}, ahigh = {-326426.316, -1521.168973, 17.12283562, -0.000446458071, 9.83337963e-008, -1.125478371e-011, 5.21006912e-016}, bhigh = {-94882.75099999999, -62.1522063}, R = 60.16021070073188);
          constant IdealGases.Common.DataRecord K2C2N2(name = "K2C2N2", MM = 0.1302314, Hf = -64254.85712355085, H0 = 190324.6221725329, Tlimit = 1000, alow = {4627.78948, -972.9810620000001, 19.9666591, -0.01953015081, 3.27533046e-005, -2.39398512e-008, 6.59654824e-012}, blow = {-777.4411259999999, -67.55613632000001}, ahigh = {726959.95, -3492.10868, 18.35676822, -0.000878547514, 1.845532919e-007, -2.038717568e-011, 9.18542099e-016}, bhigh = {15826.00773, -67.28484052}, R = 63.84383489696034);
          constant IdealGases.Common.DataRecord K2CL2(name = "K2CL2", MM = 0.1491026, Hf = -4127316.780525624, H0 = 133890.6766213332, Tlimit = 1000, alow = {-13285.26456, -124.1832429, 10.5003573, -0.001097776004, 1.348871402e-006, -8.69721512e-010, 2.28658408e-013}, blow = {-76443.6315, -17.90150052}, ahigh = {-26146.11331, -2.096229095, 10.00178456, -7.864803710000001e-007, 1.869281152e-010, -2.266272348e-014, 1.096790704e-018}, bhigh = {-77071.8933, -14.99720802}, R = 55.76342733124707);
          constant IdealGases.Common.DataRecord K2F2(name = "K2F2", MM = 0.1161934064, Hf = -7400381.034013649, H0 = 155995.3061157522, Tlimit = 1000, alow = {-1611.833856, -513.564489, 11.99922511, -0.004280109549999999, 5.16538754e-006, -3.2858149e-009, 8.549648749999999e-013}, blow = {-103924.8602, -30.39925079}, ahigh = {-57408.9253, -9.04618211, 10.00750685, -3.25076484e-006, 7.63006459e-010, -9.165649529999999e-014, 4.40504844e-018}, bhigh = {-106541.3546, -18.7403762}, R = 71.55717572628115);
          constant IdealGases.Common.DataRecord K2I2(name = "K2I2", MM = 0.33200554, Hf = -1261772.475242431, H0 = 64627.07821080335, Tlimit = 1000, alow = {-8977.75244, -27.76460169, 10.11331014, -0.0002508421994, 3.102352114e-007, -2.010096209e-010, 5.30443715e-014}, blow = {-53262.0839, -10.32092509}, ahigh = {-11796.34729, -0.472476155, 10.00040695, -1.807689054e-007, 4.32056097e-011, -5.259699430000001e-015, 2.553419144e-019}, bhigh = {-53402.10230000001, -9.6644405}, R = 25.04317247236296);
          constant IdealGases.Common.DataRecord K2O(name = "K2O", MM = 0.094196, Hf = -786518.5570512548, H0 = 147121.93723725, Tlimit = 1000, alow = {23920.44068, -544.535839, 8.82640323, -0.00348142943, 3.83454207e-006, -2.268494189e-009, 5.56921225e-013}, blow = {-8234.29168, -16.63099064}, ahigh = {-46114.6458, -13.63119524, 7.01053044, -4.32365826e-006, 9.747788419999999e-010, -1.135283694e-013, 5.325860600000001e-018}, bhigh = {-11072.22244, -5.76871872}, R = 88.26778207142554);
          constant IdealGases.Common.DataRecord K2Oplus(name = "K2Oplus", MM = 0.09419545139999999, Hf = 3910914.12084894, H0 = 150093.4576974701, Tlimit = 1000, alow = {7201.10273, -333.65264, 8.05073035, -0.001869353797, 1.921291874e-006, -1.064020951e-009, 2.457514799e-013}, blow = {43899.8636, -10.7320415}, ahigh = {-37800.335, -8.84953739, 7.00684789, -2.815510856e-006, 6.3550015e-010, -7.408551990000001e-014, 3.478334960000001e-018}, bhigh = {42145.19680000001, -4.41827661}, R = 88.26829614832124);
          constant IdealGases.Common.DataRecord K2O2(name = "K2O2", MM = 0.1101954, Hf = -1738423.754530588, H0 = 147826.5245191723, Tlimit = 1000, alow = {48108.7061, -1003.601504, 11.54229002, -0.000454633252, -1.516606209e-006, 1.79942609e-009, -6.12833729e-013}, blow = {-20571.5192, -31.8119979}, ahigh = {-147607.3073, -102.193204, 10.07664713, -3.076517991e-005, 6.81525281e-009, -7.82865445e-013, 3.6325061e-017}, bhigh = {-25920.56324, -21.62287184}, R = 75.45207876190838);
          constant IdealGases.Common.DataRecord K2O2H2(name = "K2O2H2", MM = 0.11221128, Hf = -5712438.179120673, H0 = 199489.1511798101, Tlimit = 1000, alow = {8174.78837, -1130.63068, 18.15303256, -0.00773743108, 6.83761401e-006, -2.054691111e-009, 7.67287751e-014}, blow = {-75749.6517, -63.9175947}, ahigh = {1773523.196, -4665.292469999999, 16.94308128, 0.0002085297143, -1.264714762e-007, 2.057506565e-011, -1.148042134e-015}, bhigh = {-50512.63310000001, -63.3477392}, R = 74.09657923873607);
          constant IdealGases.Common.DataRecord K2SO4(name = "K2SO4", MM = 0.1742592, Hf = -6288627.06244491, H0 = 127877.7763240047, Tlimit = 1000, alow = {62714.9231, -815.204992, 7.45541372, 0.0384990978, -5.40063098e-005, 3.6546568e-008, -9.760223570000001e-012}, blow = {-130469.2568, -10.3463477}, ahigh = {-544612.564, -959.0667229999999, 19.71591285, -0.0002866586009, 6.342480639999999e-008, -7.281462539999999e-012, 3.37788693e-016}, bhigh = {-133787.3384, -73.99103550999999}, R = 47.71324555604525);
          constant IdealGases.Common.DataRecord Kr(name = "Kr", MM = 0.0838, Hf = 0, H0 = 73954.98806682577, Tlimit = 1000, alow = {0, 0, 2.5, 0, 0, 0, 0}, blow = {-745.375, 5.49095651}, ahigh = {264.3639057, -0.7910050820000001, 2.500920585, -5.32816411e-007, 1.620730161e-010, -2.467898017e-014, 1.47858504e-018}, bhigh = {-740.348894, 5.48439815}, R = 99.21804295942721);
          constant IdealGases.Common.DataRecord Krplus(name = "Krplus", MM = 0.08379945139999999, Hf = 16192873.52518396, H0 = 73955.472219237, Tlimit = 1000, alow = {-5650.40286, 69.3074081, 2.157028132, 0.0008711228930000001, -1.18160973e-006, 7.86219863e-010, -1.832589387e-013}, blow = {162116.4118, 8.81824226}, ahigh = {-221656.7015, 1166.16784, 0.486965532, 0.001429223599, -3.94962861e-007, 4.98285351e-011, -2.406719258e-015}, bhigh = {155600.2861, 20.59230986}, R = 99.21869249850485);
          constant IdealGases.Common.DataRecord Li(name = "Li", MM = 0.006941, Hf = 22950583.48941075, H0 = 892872.4967583921, Tlimit = 1000, alow = {0, 0, 2.5, 0, 0, 0, 0}, blow = {18413.90197, 2.447622965}, ahigh = {1125610.652, -3463.53673, 6.56661192, -0.002260983356, 5.92228916e-007, -6.2816351e-011, 2.884948238e-015}, bhigh = {40346.374, -26.55918195}, R = 1197.878115545311);
          constant IdealGases.Common.DataRecord Liplus(name = "Liplus", MM = 0.0069404514, Hf = 98800407.70835166, H0 = 892943.0728381731, Tlimit = 1000, alow = {0, 0, 2.5, 0, 0, 0, 0}, blow = {81727.24550000001, 1.754357228}, ahigh = {0, 0, 2.5, 0, 0, 0, 0}, bhigh = {81727.24550000001, 1.754357228}, R = 1197.972800443499);
          constant IdealGases.Common.DataRecord Liminus(name = "Liminus", MM = 0.0069415486, Hf = 13465976.16560662, H0 = 892801.9318340579, Tlimit = 1000, alow = {0, 0, 2.5, 0, 0, 0, 0}, blow = {10496.98659, 1.754594332}, ahigh = {0, 0, 2.5, 0, 0, 0, 0}, bhigh = {10496.98659, 1.754594332}, R = 1197.783445613274);
          constant IdealGases.Common.DataRecord LiALF4(name = "LiALF4", MM = 0.1099161508, Hf = -16897315.00313783, H0 = 177720.8249908984, Tlimit = 1000, alow = {176152.936, -2999.300134, 20.7973957, -0.001742914656, -4.266642090000001e-006, 5.362829770000001e-009, -1.866817087e-012}, blow = {-211794.7828, -87.6272589}, ahigh = {-402070.0930000001, -278.3355146, 16.20791185, -8.316341570000001e-005, 1.83690569e-008, -2.104932808e-012, 9.74725302e-017}, bhigh = {-227869.6416, -55.9730484}, R = 75.64376972342085);
          constant IdealGases.Common.DataRecord LiBO2(name = "LiBO2", MM = 0.0497508, Hf = -13112397.71018758, H0 = 269899.6599049664, Tlimit = 1000, alow = {65189.8757, -1060.320845, 9.54804996, -5.46957607e-005, 2.658687599e-006, -2.931603694e-009, 9.66302973e-013}, blow = {-75062.0411, -24.90883998}, ahigh = {85544.04520000001, -1731.920898, 11.2146209, -0.000465916217, 9.999875689999999e-008, -1.122880592e-011, 5.12352946e-016}, bhigh = {-71547.4382, -36.1239729}, R = 167.1223779316112);
          constant IdealGases.Common.DataRecord LiBr(name = "LiBr", MM = 0.08684500000000001, Hf = -1740605.561632794, H0 = 105626.2306407968, Tlimit = 1000, alow = {38056.2047, -612.961499, 6.57736955, -0.00421989176, 5.391423370000001e-006, -3.69314481e-009, 1.049158761e-012}, blow = {-16374.87617, -11.28654012}, ahigh = {63801.9142, -259.7050764, 4.65676887, 0.0001022867706, -5.178252340000001e-008, 2.058203991e-011, -1.942190308e-015}, bhigh = {-17933.63752, -0.2290806613}, R = 95.73921354136679);
          constant IdealGases.Common.DataRecord LiCL(name = "LiCL", MM = 0.042394, Hf = -4570927.277444921, H0 = 213712.0583101382, Tlimit = 1000, alow = {49643.995, -718.734662, 6.78514703, -0.00452214546, 5.748483700000001e-006, -3.96625567e-009, 1.137911398e-012}, blow = {-20910.14703, -14.06383901}, ahigh = {-235276.9705, 612.004692, 3.63429373, 0.0006810973320000001, -2.174238799e-007, 4.22040361e-011, -2.848628426e-015}, bhigh = {-28623.58494, 5.618511135}, R = 196.1237911025145);
          constant IdealGases.Common.DataRecord LiF(name = "LiF", MM = 0.0259394032, Hf = -13143898.93133702, H0 = 340335.7406464926, Tlimit = 1000, alow = {29125.3732, -253.1413159, 3.53972798, 0.00369591704, -4.82946615e-006, 2.944090711e-009, -6.83879474e-013}, blow = {-40648.4966, 2.325294408}, ahigh = {-378424.649, 766.806246, 3.5854734, 0.0006031084350000001, -1.588764206e-007, 2.569397177e-011, -1.406798386e-015}, bhigh = {-47564.6945, 4.38516329}, R = 320.5344369680795);
          constant IdealGases.Common.DataRecord LiH(name = "LiH", MM = 0.00794894, Hf = 17519833.33626873, H0 = 1092737.396432732, Tlimit = 1000, alow = {-49137.31570000001, 775.6092190000001, -1.011102377, 0.01145479597, -1.151038734e-005, 5.87506896e-009, -1.196789735e-012}, blow = {12048.5891, 25.68801877}, ahigh = {-2633686.357, 6996.429169999999, -3.23353306, 0.00403393598, -9.09957964e-007, 8.775909869999999e-011, -2.889490251e-015}, bhigh = {-29900.43016, 49.71984499999999}, R = 1045.984999257763);
          constant IdealGases.Common.DataRecord LiI(name = "LiI", MM = 0.13384547, Hf = -637077.1308136167, H0 = 69409.2373839772, Tlimit = 1000, alow = {40719.7637, -666.7289470000001, 7.06425753, -0.00548004102, 6.90579016e-006, -4.53716852e-009, 1.226373296e-012}, blow = {-8235.487020000001, -12.96236234}, ahigh = {1616342.632, -4877.71708, 9.987186550000001, -0.002889137815, 8.03731377e-007, -9.028172610000001e-011, 3.078321954e-015}, bhigh = {19377.18764, -37.2927631}, R = 62.11993577369485);
          constant IdealGases.Common.DataRecord LiN(name = "LiN", MM = 0.0209477, Hf = 15978842.5459597, H0 = 429600.7676260401, Tlimit = 1000, alow = {37649.592, -488.345763, 5.18737345, 0.0002275252171, -1.416337564e-006, 1.450236258e-009, -4.79001433e-013}, blow = {41619.1531, -5.952157244}, ahigh = {-59934.8998, -40.3113455, 4.52962421, 8.744156930000001e-005, 2.557428364e-009, -2.9057518e-013, 1.336125922e-017}, bhigh = {38948.3113, -1.214952896}, R = 396.9157473135476);
          constant IdealGases.Common.DataRecord LiNO2(name = "LiNO2", MM = 0.0529465, Hf = -3815761.343998187, H0 = 252496.2367673028, Tlimit = 1000, alow = {-31337.73859, 538.9940809999999, -0.02059877081, 0.02610777454, -3.20923504e-005, 1.974344044e-008, -4.89219916e-012}, blow = {-28382.14963, 27.10019529}, ahigh = {-219667.116, -861.004271, 10.63956465, -0.0002554392587, 5.64447854e-008, -6.47614005e-012, 3.003536591e-016}, bhigh = {-23135.47535, -32.38277103}, R = 157.0353470012182);
          constant IdealGases.Common.DataRecord LiNO3(name = "LiNO3", MM = 0.06894589999999999, Hf = -4519263.291943394, H0 = 201829.6229362443, Tlimit = 1000, alow = {19672.05518, 90.52795259999999, -0.51097704, 0.0375660595, -4.78273865e-005, 3.029277805e-008, -7.70801315e-012}, blow = {-39075.6399, 27.46873033}, ahigh = {-355940.245, -1447.644737, 14.06684061, -0.000423637128, 9.320824509999999e-008, -1.065894872e-011, 4.93078062e-016}, bhigh = {-34348.2746, -52.75382697000001}, R = 120.5941470051156);
          constant IdealGases.Common.DataRecord LiO(name = "LiO", MM = 0.0229404, Hf = 3178423.785112727, H0 = 408105.9179438893, Tlimit = 1000, alow = {36270.2976, -349.936323, 4.39493318, 0.001079712984, -8.72403881e-007, 5.60796297e-010, -2.054288457e-013}, blow = {9533.330530000001, -0.9058149609999999}, ahigh = {1612392.133, -5551.31234, 11.20573851, -0.00343722688, 9.133194659999999e-007, -1.027902258e-010, 3.822991e-015}, bhigh = {42015.7547, -48.5735458}, R = 362.4379696953846);
          constant IdealGases.Common.DataRecord LiOF(name = "LiOF", MM = 0.0419388032, Hf = -2194817.042370918, H0 = 258170.5288147088, Tlimit = 1000, alow = {55101.10690000001, -577.068817, 4.42495493, 0.01102425422, -1.710493381e-005, 1.234361201e-008, -3.453693e-012}, blow = {-9278.986980000002, 0.1238830687}, ahigh = {-183356.3281, -211.0049301, 7.15681817, -6.24946201e-005, 1.376567002e-008, -1.574099067e-012, 7.277167140000001e-017}, bhigh = {-12545.30277, -12.72130479}, R = 198.2524861367527);
          constant IdealGases.Common.DataRecord LiOH(name = "LiOH", MM = 0.02394834, Hf = -9562249.408518503, H0 = 473392.3520377612, Tlimit = 1000, alow = {4574.19012, -103.0949027, 4.27240737, 0.008465219230000001, -1.386148524e-005, 1.101099795e-008, -3.29124821e-012}, blow = {-28487.28855, -0.8775745779999999}, ahigh = {850075.137, -2430.540791, 8.055314620000001, 6.895680879999999e-005, -5.527207459999999e-008, 9.368054029999999e-012, -5.31378568e-016}, bhigh = {-13658.94396, -24.57598093}, R = 347.1836461316316);
          constant IdealGases.Common.DataRecord LiON(name = "LiON", MM = 0.0369471, Hf = 4869448.48174823, H0 = 305869.1751179389, Tlimit = 1000, alow = {-9412.67267, 100.93323, 3.006187607, 0.0098904622, -1.128294343e-005, 6.41029174e-009, -1.466806373e-012}, blow = {19783.48318, 10.16334813}, ahigh = {-97406.29139999999, -511.1560940000001, 7.37849387, -0.0001508777884, 3.32991498e-008, -3.81751429e-012, 1.769558455e-016}, bhigh = {22110.06635, -14.5410504}, R = 225.0372018372214);
          constant IdealGases.Common.DataRecord Li2(name = "Li2", MM = 0.013882, Hf = 15552514.04696729, H0 = 696954.0412044374, Tlimit = 1000, alow = {6778.481580000001, -224.6205832, 5.29603744, -0.001272412017, 1.205843729e-006, -9.81854459e-011, -2.416702607e-013}, blow = {25736.38186, -6.86925758}, ahigh = {37676454, -118574.7185, 148.2167789, -0.0837853211, 2.424919798e-005, -3.27582024e-009, 1.652000081e-013}, bhigh = {772307.201, -1021.697298}, R = 598.9390577726553);
          constant IdealGases.Common.DataRecord Li2plus(name = "Li2plus", MM = 0.0138814514, Hf = 51983859.55520473, H0 = 719601.9862879757, Tlimit = 1000, alow = {-10400.56453, 26.75405642, 4.24727175, 0.001057450777, -1.281715096e-006, 1.067477239e-009, -2.759413508e-013}, blow = {85298.11309999999, 0.529818996}, ahigh = {12799310.73, -34928.6139, 38.4478536, -0.01380635397, 2.53139425e-006, -2.197815991e-010, 7.08766925e-015}, bhigh = {311796.445, -250.9543641}, R = 598.9627280617069);
          constant IdealGases.Common.DataRecord Li2Br2(name = "Li2Br2", MM = 0.17369, Hf = -2854703.920778398, H0 = 97590.39668374689, Tlimit = 1000, alow = {27863.12863, -1005.641437, 13.70963066, -0.0076336633, 8.94290592e-006, -5.56113509e-009, 1.421736784e-012}, blow = {-57628.4584, -41.2184376}, ahigh = {-89009.8336, -19.88916335, 10.01596418, -6.74913405e-006, 1.556299963e-009, -1.844693069e-013, 8.774879919999999e-018}, bhigh = {-62799.8996, -19.43063525}, R = 47.8696067706834);
          constant IdealGases.Common.DataRecord Li2F2(name = "Li2F2", MM = 0.05187880640000001, Hf = -18029001.4536649, H0 = 265349.7826040963, Tlimit = 1000, alow = {144316.6185, -2466.874678, 17.0286329, -0.0114554121, 1.086098792e-005, -5.569754279999999e-009, 1.193819664e-012}, blow = {-102607.0133, -70.0038661}, ahigh = {-218893.1683, -92.5981453, 10.06970757, -2.803632163e-005, 6.217549520000001e-009, -7.14626702e-013, 3.3168573e-017}, bhigh = {-115661.1527, -27.29853181}, R = 160.2672184840397);
          constant IdealGases.Common.DataRecord Li2I2(name = "Li2I2", MM = 0.26769094, Hf = -1355299.320178711, H0 = 65966.56577170672, Tlimit = 1000, alow = {10148.72266, -712.042101, 12.7039296, -0.00568553681, 6.770340380000001e-006, -4.26330489e-009, 1.100658186e-012}, blow = {-43130.6386, -33.03950980000001}, ahigh = {-69735.8706, -13.16825508, 10.01074855, -4.60020378e-006, 1.070510527e-009, -1.277717768e-013, 6.11061224e-018}, bhigh = {-46774.7005, -17.21894034}, R = 31.05996788684742);
          constant IdealGases.Common.DataRecord Li2O(name = "Li2O", MM = 0.0298814, Hf = -5600103.074153152, H0 = 428095.0691734658, Tlimit = 1000, alow = {26366.01597, -179.8629279, 4.05111522, 0.01189981375, -1.730095793e-005, 1.20558455e-008, -3.29273388e-012}, blow = {-20619.07963, 1.60599561}, ahigh = {726148.7039999999, -9543.783720000001, 28.87491643, -0.01959494099, 8.086339840000001e-006, -1.370211764e-009, 8.11172719e-014}, bhigh = {29907.26399, -156.4517822}, R = 278.2490780217794);
          constant IdealGases.Common.DataRecord Li2Oplus(name = "Li2Oplus", MM = 0.0298808514, Hf = 14694865.38793871, H0 = 436021.3444252797, Tlimit = 1000, alow = {108104.4623, -1261.961355, 9.69201129, -0.001793477428, 3.3803269e-007, 4.14339515e-010, -1.990530222e-013}, blow = {57549.7604, -29.15719946}, ahigh = {-130216.6086, -146.6759528, 7.61072945, -4.46942669e-005, 9.947160640000001e-009, -1.147038806e-012, 5.33924147e-017}, bhigh = {50987.6231, -15.28011047}, R = 278.2541865590885);
          constant IdealGases.Common.DataRecord Li2O2(name = "Li2O2", MM = 0.0458808, Hf = -6089649.439416925, H0 = 295287.1789506722, Tlimit = 1000, alow = {139261.973, -1764.036747, 10.29649198, 0.00760594995, -1.538519312e-005, 1.238199941e-008, -3.68543157e-012}, blow = {-26380.56746, -34.3771056}, ahigh = {-293923.2153, -235.9107937, 10.17463402, -6.931624359999999e-005, 1.521202834e-008, -1.733841909e-012, 7.99315462e-017}, bhigh = {-36184.8191, -29.04221044}, R = 181.2189848476923);
          constant IdealGases.Common.DataRecord Li2O2H2(name = "Li2O2H2", MM = 0.04789668, Hf = -15387287.80366405, H0 = 325263.4629373059, Tlimit = 1000, alow = {189441.3579, -2779.117524, 15.30234911, 0.01070738317, -2.510075145e-005, 2.224910575e-008, -6.94935958e-012}, blow = {-77027.7692, -65.1411694}, ahigh = {1215377.492, -3982.05744, 15.85877373, 0.000750914568, -2.585218184e-007, 3.6468417e-011, -1.904139406e-015}, bhigh = {-67387.4838, -66.472084}, R = 173.5918230658158);
          constant IdealGases.Common.DataRecord Li2SO4(name = "Li2SO4", MM = 0.1099446, Hf = -9475826.916465202, H0 = 178749.4610922228, Tlimit = 1000, alow = {100631.8702, -1543.457217, 9.670054909999999, 0.0339075738, -4.82348381e-005, 3.26736892e-008, -8.702518510000001e-012}, blow = {-120193.9538, -29.11716635}, ahigh = {-603875.0870000001, -1064.550976, 19.79557349, -0.000318897825, 7.062411120000001e-008, -8.11443907e-012, 3.76683021e-016}, bhigh = {-126890.8595, -80.14316674}, R = 75.62419618607918);
          constant IdealGases.Common.DataRecord Li3plus(name = "Li3plus", MM = 0.0208224514, Hf = 36335339.26749855, H0 = 638224.9978501572, Tlimit = 1000, alow = {-6873.222290000001, -282.0780467, 7.92269748, -0.001687949474, 1.771718993e-006, -9.972535399999999e-010, 2.332934849e-013}, blow = {90279.63650000001, -16.19319856}, ahigh = {-43550.9806, -6.05643275, 7.00472289, -1.953320933e-006, 4.429152229999999e-010, -5.18198319e-014, 2.439894871e-018}, bhigh = {88799.02870000001, -10.66880792}, R = 399.303225171653);
          constant IdealGases.Common.DataRecord Li3Br3(name = "Li3Br3", MM = 0.260535, Hf = -3165176.245034256, H0 = 99935.92799431937, Tlimit = 1000, alow = {64319.66250000001, -1823.743601, 22.43492856, -0.0127941826, 1.459200355e-005, -8.884688150000001e-009, 2.23370805e-012}, blow = {-94806.7767, -82.06864420000001}, ahigh = {-158467.0201, -40.4136094, 16.03177633, -1.322713911e-005, 3.014221791e-009, -3.54029216e-013, 1.672030783e-017}, bhigh = {-104247.0448, -44.05478160000001}, R = 31.9130711804556);
          constant IdealGases.Common.DataRecord Li3CL3(name = "Li3CL3", MM = 0.127182, Hf = -7674884.189586578, H0 = 193113.3808243305, Tlimit = 1000, alow = {98674.28999999999, -2324.074851, 23.78484564, -0.014835224, 1.634495612e-005, -9.675366520000001e-009, 2.377033859e-012}, blow = {-110383.9177, -94.74114379}, ahigh = {-200593.0356, -59.3940019, 16.04590929, -1.885890854e-005, 4.25351737e-009, -4.95556094e-013, 2.325408214e-017}, bhigh = {-122492.7071, -48.44413629}, R = 65.37459703417152);
          constant IdealGases.Common.DataRecord Li3F3(name = "Li3F3", MM = 0.07781820960000001, Hf = -19591775.93312298, H0 = 263906.5342875737, Tlimit = 1000, alow = {182387.4443, -3163.78624, 22.87811928, -0.007544903889999999, 3.44679023e-006, 2.690705228e-010, -5.25084096e-013}, blow = {-171244.6093, -99.7365499}, ahigh = {-361866.499, -193.7051395, 16.14457133, -5.7756285e-005, 1.274057038e-008, -1.458168875e-012, 6.744850030000001e-017}, bhigh = {-188208.1974, -56.3361288}, R = 106.8448123226932);
          constant IdealGases.Common.DataRecord Li3I3(name = "Li3I3", MM = 0.40153641, Hf = -1525284.611176356, H0 = 67821.2319525395, Tlimit = 1000, alow = {36117.6469, -1392.033487, 21.10791892, -0.01046866587, 1.222589173e-005, -7.58413543e-009, 1.93519918e-012}, blow = {-71530.8458, -70.9388203}, ahigh = {-126669.6511, -27.83891914, 16.02228068, -9.399364930000001e-006, 2.163919735e-009, -2.561731922e-013, 1.217394107e-017}, bhigh = {-78695.5068, -40.9172406}, R = 20.70664525789828);
          constant IdealGases.Common.DataRecord Mg(name = "Mg", MM = 0.024305, Hf = 6052252.622917095, H0 = 254985.7231022423, Tlimit = 1000, alow = {0, 0, 2.5, 0, 0, 0, 0}, blow = {16946.58761, 3.63433014}, ahigh = {-536483.155, 1973.709576, -0.36337769, 0.002071795561, -7.738051719999999e-007, 1.359277788e-010, -7.766898397000001e-015}, bhigh = {4829.188109999999, 23.39104998}, R = 342.0889528903518);
          constant IdealGases.Common.DataRecord Mgplus(name = "Mgplus", MM = 0.0243044514, Hf = 36661884.90886899, H0 = 254991.4786391763, Tlimit = 1000, alow = {0, 0, 2.5, 0, 0, 0, 0}, blow = {106422.3354, 4.32744346}, ahigh = {-19147.58821, 48.7734792, 2.457662661, 1.218104674e-005, 1.897261686e-009, -1.580433756e-012, 2.135732238e-016}, bhigh = {106102.2394, 4.64644286}, R = 342.0966745211126);
          constant IdealGases.Common.DataRecord MgBr(name = "MgBr", MM = 0.104209, Hf = 59143.78796457119, H0 = 92008.51174082852, Tlimit = 1000, alow = {7361.41914, -239.5789881, 5.36056042, -0.001667141829, 1.981137765e-006, -1.201637202e-009, 3.032148099e-013}, blow = {591.563444, -1.421771179}, ahigh = {24776.04216, -641.7687520000001, 6.01993209, -0.001391004302, 6.44333096e-007, -1.197734078e-010, 7.421644240000001e-015}, bhigh = {2824.060334, -6.26443992}, R = 79.78650596397624);
          constant IdealGases.Common.DataRecord MgBr2(name = "MgBr2", MM = 0.184113, Hf = -1666059.534090477, H0 = 80153.38949449523, Tlimit = 1000, alow = {21484.77999, -515.45289, 9.27325875, -0.00347004904, 3.92184079e-006, -2.377174864e-009, 5.967025559999999e-013}, blow = {-36524.4385, -17.91077763}, ahigh = {-43268.9901, -12.83036001, 7.5100243, -4.1523324e-006, 9.42649918e-010, -1.103873928e-013, 5.20110431e-018}, bhigh = {-39198.8602, -7.40916803}, R = 45.15961393274782);
          constant IdealGases.Common.DataRecord MgCL(name = "MgCL", MM = 0.05975800000000001, Hf = -915440.3929181029, H0 = 156683.8414940259, Tlimit = 1000, alow = {20439.9528, -407.215516, 5.8803723, -0.002594175042, 2.945953528e-006, -1.750648628e-009, 4.337959330000001e-013}, blow = {-5851.44495, -6.02354575}, ahigh = {1041328.453, -3380.15833, 8.637775469999999, -0.002447789643, 7.84196944e-007, -1.12640938e-010, 5.81062073e-015}, bhigh = {13271.88977, -27.03802395}, R = 139.1357140466548);
          constant IdealGases.Common.DataRecord MgCLplus(name = "MgCLplus", MM = 0.0597574514, Hf = 10816045.81282394, H0 = 159239.2543032717, Tlimit = 1000, alow = {8182.385119999999, -262.225376, 5.41986343, -0.001774129606, 2.185811127e-006, -1.418143432e-009, 3.91889858e-013}, blow = {77704.0371, -3.78093884}, ahigh = {-12683919.21, 34788.2454, -30.0422295, 0.01481739497, -2.470965605e-006, 1.424433718e-010, 2.789613105e-016}, bhigh = {-148701.374, 255.2015117}, R = 139.1369913744347);
          constant IdealGases.Common.DataRecord MgCL2(name = "MgCL2", MM = 0.095211, Hf = -4192476.667611936, H0 = 145997.8783964038, Tlimit = 1000, alow = {36378.2468, -730.784496, 9.66103051, -0.00366021294, 3.61081935e-006, -1.928769286e-009, 4.309912350000001e-013}, blow = {-46469.1457, -23.60112274}, ahigh = {-68352.1701, -24.90899393, 7.5187832, -7.56451826e-006, 1.67929377e-009, -1.931706275e-013, 8.971657820000001e-018}, bhigh = {-50326.8691, -10.53268382}, R = 87.32680047473507);
          constant IdealGases.Common.DataRecord MgF(name = "MgF", MM = 0.0433034032, Hf = -5363707.580377886, H0 = 207122.473921403, Tlimit = 1000, alow = {38230.0162, -480.331039, 5.06846894, 0.00052120293, -1.874026347e-006, 1.759241129e-009, -5.60940319e-013}, blow = {-26591.14296, -3.76896921}, ahigh = {-169588.3782, 358.875763, 3.93600165, 0.000481304414, -1.658385409e-007, 3.33605229e-011, -2.270104205e-015}, bhigh = {-31693.2399, 4.4105659}, R = 192.0050477695481);
          constant IdealGases.Common.DataRecord MgFplus(name = "MgFplus", MM = 0.0433028546, Hf = 11936113.21411591, H0 = 207131.2638128019, Tlimit = 1000, alow = {641329.384, -8518.53261, 48.1421691, -0.1163071535, 0.0001622560562, -1.06348459e-007, 2.634809398e-011}, blow = {102430.848, -245.0074295}, ahigh = {-10568523.62, 20771.22379, -6.11952356, 0.002245834831, -9.51573861e-008, -2.263502341e-011, 2.284524246e-015}, bhigh = {-83315.4988, 87.2710238}, R = 192.0074802643611);
          constant IdealGases.Common.DataRecord MgF2(name = "MgF2", MM = 0.0623018064, Hf = -11805405.05162624, H0 = 202601.5091594519, Tlimit = 1000, alow = {43384.2955, -661.651177, 7.45344852, 0.00352081405, -6.95576458e-006, 5.61992348e-009, -1.688028906e-012}, blow = {-86871.8367, -15.4547679}, ahigh = {-124441.9584, -86.87343749999999, 7.56358123, -2.499267978e-005, 5.44012914e-009, -6.15824576e-013, 2.822792666e-017}, bhigh = {-90600.19439999999, -14.2079699}, R = 133.4547500375527);
          constant IdealGases.Common.DataRecord MgF2plus(name = "MgF2plus", MM = 0.0623012578, Hf = 9352815.409771709, H0 = 199274.6926531554, Tlimit = 1000, alow = {78322.2026, -1176.632752, 10.25829767, -0.00375527289, 3.016621585e-006, -1.327098871e-009, 2.465289749e-013}, blow = {74132.2855, -29.90883551}, ahigh = {-150231.687, 77.2528956, 7.27156936, 0.000223752977, -1.003070865e-007, 1.996274309e-011, -1.233278721e-015}, bhigh = {66991.175, -10.94528006}, R = 133.4559251867945);
          constant IdealGases.Common.DataRecord MgH(name = "MgH", MM = 0.02531294, Hf = 9077811.743716849, H0 = 342990.7391239421, Tlimit = 1000, alow = {-49586.7915, 750.027865, -0.64420475, 0.00982630101, -8.789822439999999e-006, 3.82335352e-009, -6.00372576e-013}, blow = {23022.79383, 26.57165344}, ahigh = {-100574.8598, 1952.890106, -1.317191549, 0.0056036658, -2.13733498e-006, 3.3248805e-010, -1.824672746e-014}, bhigh = {15985.82755, 34.3123316}, R = 328.4672582481529);
          constant IdealGases.Common.DataRecord MgI(name = "MgI", MM = 0.15120947, Hf = 404778.3316745968, H0 = 64421.34212890238, Tlimit = 1000, alow = {2943.889099, -169.0248574, 5.14725183, -0.001321186997, 1.623056505e-006, -1.005114222e-009, 2.567337785e-013}, blow = {6845.89027, 0.859313225}, ahigh = {-2370562.811, 6916.45248, -3.18389449, 0.00405155114, -9.774290750000001e-007, 1.0233294e-010, -3.79030487e-015}, bhigh = {-38185.5111, 59.9310438}, R = 54.98645025341336);
          constant IdealGases.Common.DataRecord MgI2(name = "MgI2", MM = 0.27811394, Hf = -617394.5434018878, H0 = 54993.54329380253, Tlimit = 1000, alow = {15947.39709, -416.041272, 9.01063832, -0.003090359024, 3.62126426e-006, -2.260138392e-009, 5.80905555e-013}, blow = {-20804.414, -14.12175736}, ahigh = {-33402.9918, -9.34351178, 7.50746854, -3.14770789e-006, 7.241336470000001e-010, -8.56766495e-014, 4.0696917e-018}, bhigh = {-22945.55505, -5.24127259}, R = 29.89591963639075);
          constant IdealGases.Common.DataRecord MgN(name = "MgN", MM = 0.0383117, Hf = 7535452.616302591, H0 = 234624.9579110298, Tlimit = 1000, alow = {37595.3864, -485.316428, 5.16305424, 0.0002591539493, -1.518993975e-006, 1.522840476e-009, -4.992531450000001e-013}, blow = {36072.9461, -3.813908776}, ahigh = {-60185.891, -40.1086478, 4.52949895, 4.76049091e-005, 2.547271298e-009, -2.894058118e-013, 1.330639131e-017}, bhigh = {33412.857, 0.7925250209}, R = 217.0217453153997);
          constant IdealGases.Common.DataRecord MgO(name = "MgO", MM = 0.0403044, Hf = 800441.3165808199, H0 = 221045.5186034279, Tlimit = 1000, alow = {351365.974, -5287.19716, 33.8206006, -0.08400489629999999, 0.000121001616, -7.630795020000001e-008, 1.701022862e-011}, blow = {27906.79519, -162.4886199}, ahigh = {-15867383.67, 34204.681, -17.74087677, 0.00700496305, -1.104138249e-006, 8.957488529999999e-011, -3.052513649e-015}, bhigh = {-230050.4434, 173.8984472}, R = 206.2919185002134);
          constant IdealGases.Common.DataRecord MgOH(name = "MgOH", MM = 0.04131234, Hf = -3205555.047232862, H0 = 269272.6676823439, Tlimit = 1000, alow = {38398.5162, -736.7383640000001, 7.92066446, -0.000595094059, -2.112941162e-006, 3.22828211e-009, -1.214159329e-012}, blow = {-13923.26188, -19.16078109}, ahigh = {664866.475, -1770.750355, 7.26999927, 0.000533684276, -1.980894443e-007, 3.025677088e-011, -1.554849476e-015}, bhigh = {-6149.11456, -16.71027009}, R = 201.2588006392279);
          constant IdealGases.Common.DataRecord MgOHplus(name = "MgOHplus", MM = 0.0413117914, Hf = 14905414.38975217, H0 = 246621.3556645719, Tlimit = 1000, alow = {117022.4573, -1735.933343, 11.64059613, -0.008449876219999999, 7.35171374e-006, -2.790223071e-009, 3.51498213e-013}, blow = {81188.07670000001, -42.7117437}, ahigh = {829633.954, -2459.700177, 8.11873202, 3.5005791e-005, -4.67057475e-008, 8.312358260000001e-012, -4.80283622e-016}, bhigh = {88016.61709999999, -24.38155217}, R = 201.261473255793);
          constant IdealGases.Common.DataRecord Mg_OH_2(name = "Mg_OH_2", MM = 0.05831968, Hf = -9465000.631004833, H0 = 293752.6063243146, Tlimit = 1000, alow = {52458.9467, -1289.056383, 13.89327642, -0.000780669367, -4.15125723e-006, 6.10947304e-009, -2.274138833e-012}, blow = {-62950.8915, -50.1535334}, ahigh = {1713709.254, -4730.00535, 14.48925967, 0.0001907819857, -1.226834131e-007, 2.015343753e-011, -1.128993279e-015}, bhigh = {-38877.2467, -58.4049812}, R = 142.5671745798331);
          constant IdealGases.Common.DataRecord MgS(name = "MgS", MM = 0.05637, Hf = 2140310.643959553, H0 = 163812.5066524748, Tlimit = 1000, alow = {-9565.78809, 144.3637798, 1.813794717, 0.01147168775, -2.220170412e-005, 1.995344981e-008, -6.09068874e-012}, blow = {12765.01517, 14.61333093}, ahigh = {26507943.28, -77113.5586, 84.63771680000001, -0.0364425068, 8.403084420000002e-006, -9.53988217e-010, 4.264658029999999e-014}, bhigh = {507893.117, -583.4656096}, R = 147.4981727869434);
          constant IdealGases.Common.DataRecord Mg2(name = "Mg2", MM = 0.04861, Hf = 5894122.917095248, H0 = 196299.423986834, Tlimit = 1000, alow = {4545.195589999999, 411.585004, 0.484119617, 0.00489196965, -6.39553684e-006, 4.29976455e-009, -1.164624418e-012}, blow = {31816.4179, 26.40432143}, ahigh = {30382.24994, 59.4524046, 2.352706666, 0.0001378537924, -5.89569204e-008, 1.104045317e-011, -6.558868290000001e-016}, bhigh = {33510.3656, 15.88177377}, R = 171.0444764451759);
          constant IdealGases.Common.DataRecord Mg2F4(name = "Mg2F4", MM = 0.1246036128, Hf = -13790683.60367798, H0 = 169684.5984228156, Tlimit = 1000, alow = {151195.6137, -3122.595912, 25.17715088, -0.01558723757, 1.552195619e-005, -8.40872905e-009, 1.911618525e-012}, blow = {-195307.8883, -108.5103537}, ahigh = {-298061.0166, -120.1299868, 16.09110864, -3.68745629e-005, 8.22031705e-009, -9.48886978e-013, 4.41979533e-017}, bhigh = {-211734.969, -53.08655469999999}, R = 66.72737501877634);
          constant IdealGases.Common.DataRecord Mn(name = "Mn", MM = 0.054938049, Hf = 5140335.434918703, H0 = 112807.5734906421, Tlimit = 1000, alow = {0.1034061359, -0.001551537349, 2.500009148, -2.723162066e-008, 4.33389743e-011, -3.51109389e-014, 1.136032201e-017}, blow = {33219.3519, 6.649325463}, ahigh = {5855.15582, 883.8588440000001, -0.0364866258, 0.002703720687, -1.324971998e-006, 2.87260329e-010, -1.92363357e-014}, bhigh = {28678.03487, 22.92541198}, R = 151.3426878337088);
          constant IdealGases.Common.DataRecord Mnplus(name = "Mnplus", MM = 0.0549375004, Hf = 18309375.57544937, H0 = 112808.6999749992, Tlimit = 1000, alow = {345.80177, -4.25115133, 2.521281028, -5.56508728e-005, 8.03716221e-008, -6.09355097e-011, 1.900014268e-014}, blow = {120253.3602, 6.683468162}, ahigh = {647131.41, -2403.796253, 5.93771575, -0.002341014594, 7.46416564e-007, -9.075969730000001e-011, 4.467879847e-015}, bhigh = {134990.2108, -17.02666341}, R = 151.344199125594);
          constant IdealGases.Common.DataRecord Mo(name = "Mo", MM = 0.09594, Hf = 6863664.790494058, H0 = 64596.91473837815, Tlimit = 1000, alow = {76.46367910000001, -1.159269043, 2.506929462, -2.099249725e-005, 3.41477943e-008, -2.841269591e-011, 9.492443320999999e-015}, blow = {78458.99799999999, 7.60183566}, ahigh = {5573271, -16623.65811, 21.35147077, -0.01003069377, 2.409784357e-006, -1.811267352e-010, 1.034189087e-015}, bhigh = {184264.6473, -127.5326434}, R = 86.66324786324788);
          constant IdealGases.Common.DataRecord Moplus(name = "Moplus", MM = 0.0959394514, Hf = 14061086.53233346, H0 = 64597.28411580224, Tlimit = 1000, alow = {129.8236623, -1.560279908, 2.507600281, -1.923789063e-005, 2.673316651e-008, -1.937174292e-011, 5.729735412e-015}, blow = {161510.3759, 7.44254346}, ahigh = {12988911.2, -39482.7623, 48.6659978, -0.02605352326, 7.21543192e-006, -8.719164960000001e-010, 3.78842304e-014}, bhigh = {411894.857, -321.679103}, R = 86.66374342015469);
          constant IdealGases.Common.DataRecord Mominus(name = "Mominus", MM = 0.0959405486, Hf = 6048794.138331621, H0 = 64596.54536517837, Tlimit = 1000, alow = {0, 0, 2.5, 0, 0, 0, 0}, blow = {69051.2369, 7.48565954}, ahigh = {0, 0, 2.5, 0, 0, 0, 0}, bhigh = {69051.2369, 7.48565954}, R = 86.66275231200837);
          constant IdealGases.Common.DataRecord MoO(name = "MoO", MM = 0.1119394, Hf = 3198206.556404626, H0 = 91595.30067161338, Tlimit = 1000, alow = {-28011.52706, 513.988348, 1.075385931, 0.008681048470000001, -1.111118984e-005, 7.23434933e-009, -1.893138381e-012}, blow = {39413.7408, 22.72230239}, ahigh = {1573131.992, -5241.48358, 11.02656868, -0.00390299662, 1.147334134e-006, -1.358975691e-010, 5.77526858e-015}, bhigh = {74489.72, -42.5361293}, R = 74.27654605974304);
          constant IdealGases.Common.DataRecord MoO2(name = "MoO2", MM = 0.1279388, Hf = -121605.6348816778, H0 = 83688.64644658228, Tlimit = 1000, alow = {32471.8322, -190.4783783, 2.120771647, 0.01650280086, -2.381696822e-005, 1.652371586e-008, -4.49445243e-012}, blow = {-1862.932837, 16.40582056}, ahigh = {309614.3654, -1932.750274, 9.428673180000001, -0.001630508855, 5.752760170000001e-007, -7.59045747e-011, 3.46133778e-015}, bhigh = {7327.72518, -25.33315948}, R = 64.98788483243551);
          constant IdealGases.Common.DataRecord MoO3(name = "MoO3", MM = 0.1439382, Hf = -2531727.99854382, H0 = 91655.89815629208, Tlimit = 1000, alow = {59773.8536, -768.455783, 5.88184444, 0.01686119817, -2.582485043e-005, 1.850382718e-008, -5.15224985e-012}, blow = {-41558.7239, -6.52916216}, ahigh = {-409759.727, 237.9066513, 9.3111008, 0.000657933891, -2.895307725e-007, 5.69263726e-011, -3.48965731e-015}, bhigh = {-49237.38720000001, -21.14864892}, R = 57.76417934919292);
          constant IdealGases.Common.DataRecord MoO3minus(name = "MoO3minus", MM = 0.1439387486, Hf = -4552231.469101435, H0 = 94207.31479111943, Tlimit = 1000, alow = {182161.7352, -2224.65697, 13.20405967, -0.001249800352, -1.864166983e-006, 2.27657947e-009, -7.366564570000001e-013}, blow = {-69389.86, -46.9937721}, ahigh = {-488109.872, 18.48536991, 10.42644941, -0.000626555791, 3.020554347e-007, -4.73319589e-011, 2.527517727e-015}, bhigh = {-83378.155, -27.01522825}, R = 57.763959190069);
          constant IdealGases.Common.DataRecord Mo2O6(name = "Mo2O6", MM = 0.2878764, Hf = -3992848.364784332, H0 = 89495.00202170099, Tlimit = 1000, alow = {156837.5811, -2159.930184, 15.22500503, 0.031941989, -5.06387715e-005, 3.68937182e-008, -1.037872908e-011}, blow = {-130993.1161, -54.089185}, ahigh = {-631223.816, -664.282755, 22.4938357, -0.0001968470329, 4.33674061e-008, -4.959777990000001e-012, 2.293212733e-016}, bhigh = {-143057.325, -86.69628539999999}, R = 28.88208967459646);
          constant IdealGases.Common.DataRecord Mo3O9(name = "Mo3O9", MM = 0.4318146, Hf = -4404740.712796649, H0 = 94448.21689678858, Tlimit = 1000, alow = {148338.6187, -1863.895133, 17.38405871, 0.0623301232, -9.32844135e-005, 6.61363879e-008, -1.830607518e-011}, blow = {-224894.5631, -58.2666947}, ahigh = {-923029.054, -1076.171759, 34.8000726, -0.00031896338, 7.02839704e-008, -8.03964264e-012, 3.71787953e-016}, bhigh = {-235742.2166, -144.8769485}, R = 19.25472644973097);
          constant IdealGases.Common.DataRecord Mo4O12(name = "Mo4O12", MM = 0.5757528, Hf = -4560163.273196414, H0 = 95342.7408429451, Tlimit = 1000, alow = {223487.6437, -2996.203728, 26.82458827, 0.0759734572, -0.0001155247012, 8.25647257e-008, -2.296764551e-011}, blow = {-308461.9476, -106.5236386}, ahigh = {-1225283.388, -1368.141197, 47.0168781, -0.00040529102, 8.928472389999999e-008, -1.021091158e-011, 4.721091689999999e-016}, bhigh = {-325647.777, -204.7486535}, R = 14.44104483729823);
          constant IdealGases.Common.DataRecord Mo5O15(name = "Mo5O15", MM = 0.7196910000000001, Hf = -4625746.934448255, H0 = 95759.62739564618, Tlimit = 1000, alow = {276890.3251, -3757.9331, 34.0819776, 0.09485414959999999, -0.00014426721, 1.0311639e-007, -2.868600013e-011}, blow = {-391350.703, -142.6449143}, ahigh = {-1535389.516, -1710.367122, 59.2712578, -0.0005066836290000001, 1.116223706e-007, -1.276562233e-011, 5.90231891e-016}, bhigh = {-412897.2380000001, -265.1287167}, R = 11.55283586983858);
          constant IdealGases.Common.DataRecord N(name = "N", MM = 0.0140067, Hf = 33746706.93311058, H0 = 442461.6790535958, Tlimit = 1000, alow = {0, 0, 2.5, 0, 0, 0, 0}, blow = {56104.6378, 4.193905036}, ahigh = {88765.0138, -107.12315, 2.362188287, 0.0002916720081, -1.7295151e-007, 4.01265788e-011, -2.677227571e-015}, bhigh = {56973.5133, 4.865231506}, R = 593.6067739010616);
          constant IdealGases.Common.DataRecord Nplus(name = "Nplus", MM = 0.0140061514, Hf = 134378643.3723685, H0 = 508099.891023597, Tlimit = 1000, alow = {5237.07921, 2.299958315, 2.487488821, 2.737490756e-005, -3.134447576e-008, 1.850111332e-011, -4.447350984e-015}, blow = {225628.4738, 5.076830786}, ahigh = {290497.0374, -855.7908610000001, 3.47738929, -0.000528826719, 1.352350307e-007, -1.389834122e-011, 5.046166279e-016}, bhigh = {231080.9984, -1.994146545}, R = 593.6300245904811);
          constant IdealGases.Common.DataRecord Nminus(name = "Nminus", MM = 0.0140072486, Hf = 33806606.74502486, H0 = 463927.2983275246, Tlimit = 1000, alow = {1445.682471, 7.33520511, 2.476680939, 4.22786918e-005, -4.42629332e-008, 2.490985431e-011, -5.83160809e-015}, blow = {56176.25, 5.145753977}, ahigh = {2404.189576, 0.2954965336, 2.499789368, 8.30756497e-008, -1.82994277e-011, 2.100136461e-015, -9.754986710000001e-020}, bhigh = {56214.13890000001, 5.006484157}, R = 593.583525032889);
          constant IdealGases.Common.DataRecord NCO(name = "NCO", MM = 0.04201680000000001, Hf = 3137964.837874374, H0 = 242718.2698349232, Tlimit = 1000, alow = {11365.03036, -244.4613367, 4.6713761, 0.002309387548, 2.798649599e-006, -4.54635738e-009, 1.692880931e-012}, blow = {15776.49188, -0.2171476903}, ahigh = {108944.5289, -1735.459316, 8.65561033, -0.000405322926, 7.59971641e-008, -7.25380415e-012, 3.24487241e-016}, bhigh = {23657.92776, -26.1953297}, R = 197.8844652615144);
          constant IdealGases.Common.DataRecord ND(name = "ND", MM = 0.016020802, Hf = 22204805.78937309, H0 = 539798.1948718922, Tlimit = 1000, alow = {22901.55757, -395.738851, 6.17901033, -0.00884780697, 1.44158297e-005, -1.006647227e-008, 2.654403586e-012}, blow = {43559.13430000001, -11.80414072}, ahigh = {543965.796, -2084.583507, 5.83408972, -0.000418939336, 9.768929529999999e-008, -1.056317654e-011, 4.68307921e-016}, bhigh = {54666.3341, -14.80810939}, R = 518.9797614376608);
          constant IdealGases.Common.DataRecord ND2(name = "ND2", MM = 0.018034904, Hf = 10248824.72343629, H0 = 552391.3517920583, Tlimit = 1000, alow = {19352.22164, -213.0631713, 4.8440176, -0.002516949288, 7.61638154e-006, -5.46450177e-009, 1.292736999e-012}, blow = {22119.98374, -3.171712417}, ahigh = {1631308.357, -6564.63749, 12.80547961, -0.003094456779, 9.181660230000001e-007, -1.241119284e-010, 6.22629986e-015}, bhigh = {61089.2958, -61.24502693}, R = 461.0211398962811);
          constant IdealGases.Common.DataRecord ND3(name = "ND3", MM = 0.020049006, Hf = -2730913.742057836, H0 = 510449.2462120068, Tlimit = 1000, alow = {10451.2037, 161.0166943, 0.857496323, 0.01319688794, -1.153090144e-005, 7.14249556e-009, -2.109194351e-012}, blow = {-8220.948899999999, 16.75921299}, ahigh = {2599516.958, -10134.20124, 17.98028169, -0.0035826098, 1.009922e-006, -1.537638609e-010, 9.106175650000001e-015}, bhigh = {53972.0566, -98.10988569}, R = 414.7074423540001);
          constant IdealGases.Common.DataRecord NF(name = "NF", MM = 0.0330051032, Hf = 7059226.525914937, H0 = 264750.1190058391, Tlimit = 1000, alow = {-35049.2775, 667.450299, -1.201665982, 0.01452074253, -1.822873148e-005, 1.160136864e-008, -2.973416333e-012}, blow = {23954.14002, 30.89260431}, ahigh = {800298.733, -3237.69658, 8.703408870000001, -0.002701025798, 9.15004211e-007, -1.36525663e-010, 7.23462441e-015}, bhigh = {46428.19450000001, -30.19933248}, R = 251.9147402635602);
          constant IdealGases.Common.DataRecord NF2(name = "NF2", MM = 0.0520035064, Hf = 661906.270997142, H0 = 203478.6831220289, Tlimit = 1000, alow = {15118.31104, 91.9638994, 0.494730179, 0.02001847323, -2.767712684e-005, 1.872924867e-008, -5.020318040000001e-012}, blow = {2839.279606, 22.7051567}, ahigh = {-194501.0078, -353.603407, 7.26349436, -8.4023804e-005, 2.326721111e-008, -2.667577562e-012, 1.236070614e-016}, bhigh = {3435.44059, -13.36102511}, R = 159.8829112799979);
          constant IdealGases.Common.DataRecord NF3(name = "NF3", MM = 0.07100190960000001, Hf = -1854879.689038673, H0 = 166960.4531312493, Tlimit = 1000, alow = {87571.49280000001, -903.1832890000001, 4.02741727, 0.02314439555, -3.41510647e-005, 2.409483651e-008, -6.63346419e-012}, blow = {-12372.32074, 0.3026430713}, ahigh = {-349626.876, -497.372867, 10.36866128, 8.90068765e-005, 5.88265436e-008, -3.157737664e-012, 1.714329953e-016}, bhigh = {-17131.83352, -30.98920858}, R = 117.1020898851994);
          constant IdealGases.Common.DataRecord NH(name = "NH", MM = 0.01501464, Hf = 23778925.16903503, H0 = 572847.7672458348, Tlimit = 1000, alow = {13596.5132, -190.0296604, 4.51849679, -0.002432776899, 2.377587464e-006, -2.592797084e-010, -2.659680792e-013}, blow = {42809.7219, -3.886561616}, ahigh = {1958141.991, -5782.861300000001, 9.33574202, -0.002292910311, 6.07609248e-007, -6.647942750000001e-011, 2.384234783e-015}, bhigh = {78989.1234, -41.169704}, R = 553.7576658514623);
          constant IdealGases.Common.DataRecord NHplus(name = "NHplus", MM = 0.0150140914, Hf = 110948299.8085385, H0 = 632413.493899471, Tlimit = 1000, alow = {4253.656849999999, -245.8222206, 6.70891949, -0.0103848943, 1.509008623e-005, -9.58051219e-009, 2.333206758e-012}, blow = {200107.7797, -13.95057632}, ahigh = {1405709.438, -4136.21571, 7.63201448, -0.001228325778, 2.721187746e-007, -2.010098289e-011, 3.71719018e-017}, bhigh = {225897.596, -27.86785234}, R = 553.7778996070318);
          constant IdealGases.Common.DataRecord NHF(name = "NHF", MM = 0.0340130432, Hf = 3292854.43062031, H0 = 294878.8775242552, Tlimit = 1000, alow = {-51106.59820000001, 961.225643, -2.706446594, 0.0203656268, -2.425558952e-005, 1.551553017e-008, -4.05845826e-012}, blow = {7909.62834, 40.99317124}, ahigh = {901390.2720000001, -3463.39705, 8.705804860000001, -0.0004018963409999999, 2.322774501e-008, 6.28048733e-012, -6.28309569e-016}, bhigh = {33370.6534, -29.00483634}, R = 244.4495175309689);
          constant IdealGases.Common.DataRecord NHF2(name = "NHF2", MM = 0.0530114464, Hf = -1942976.602124933, H0 = 203869.9136494416, Tlimit = 1000, alow = {-56261.1342, 1205.756556, -6.01752942, 0.0376002769, -4.6191986e-005, 2.94247557e-008, -7.59873236e-012}, blow = {-18970.14374, 59.01714907}, ahigh = {739427.899, -4004.47177, 12.2132232, -0.000697043773, 1.271073981e-007, -1.247136898e-011, 5.08651759e-016}, bhigh = {9134.195979999999, -48.67843963}, R = 156.8429568448825);
          constant IdealGases.Common.DataRecord NH2(name = "NH2", MM = 0.01602258, Hf = 11804260.79944678, H0 = 620241.371863957, Tlimit = 1000, alow = {-31182.40659, 475.424339, 1.372395176, 0.006306429719999999, -5.98789356e-006, 4.49275234e-009, -1.414073548e-012}, blow = {19289.39662, 15.40126885}, ahigh = {2111053.74, -6880.62723, 11.32305924, -0.001829236741, 5.64389009e-007, -7.88645248e-011, 4.078593449999999e-015}, bhigh = {65037.7856, -53.59155744}, R = 518.9221710860547);
          constant IdealGases.Common.DataRecord NH2F(name = "NH2F", MM = 0.0350209832, Hf = -2141573.226876166, H0 = 288540.6141310162, Tlimit = 1000, alow = {-109237.476, 1844.91978, -7.6738716, 0.0322953344, -3.38810867e-005, 1.97187155e-008, -4.81020515e-012}, blow = {-18783.1896, 68.61483738999999}, ahigh = {1927205.34, -7500.447160000001, 13.9558958, -0.00118480442, 2.05067867e-007, -1.90876131e-011, 7.38923621e-016}, bhigh = {35529.2734, -67.31185490999999}, R = 237.4140084108204);
          constant IdealGases.Common.DataRecord NH3(name = "NH3", MM = 0.01703052, Hf = -2697510.117130892, H0 = 589713.1150428759, Tlimit = 1000, alow = {-76812.26149999999, 1270.951578, -3.89322913, 0.02145988418, -2.183766703e-005, 1.317385706e-008, -3.33232206e-012}, blow = {-12648.86413, 43.66014588}, ahigh = {2452389.535, -8040.89424, 12.71346201, -0.000398018658, 3.55250275e-008, 2.53092357e-012, -3.32270053e-016}, bhigh = {43861.91959999999, -64.62330602}, R = 488.2101075011215);
          constant IdealGases.Common.DataRecord NH2OH(name = "NH2OH", MM = 0.03302992, Hf = -1513779.022171413, H0 = 340169.7006835016, Tlimit = 1000, alow = {-56175.8667, 1209.290057, -6.17959906, 0.0405311644, -5.19010554e-005, 3.59454458e-008, -9.933681639999999e-012}, blow = {-12658.88352, 57.27932928000001}, ahigh = {4878285.05, -15336.04636, 22.2723999, -0.002514583678, 3.33958973e-007, -1.881744532e-011, 1.918174365e-016}, bhigh = {89230.2071, -126.9053624}, R = 251.7254658806319);
          constant IdealGases.Common.DataRecord NH4plus(name = "NH4plus", MM = 0.0180379114, Hf = 35752750.45424605, H0 = 553211.0552444559, Tlimit = 1000, alow = {-266831.5752, 3763.02069, -15.71327725, 0.0454882021, -4.37996212e-005, 2.464478293e-008, -5.96153233e-012}, blow = {58232.8472, 111.2087156}, ahigh = {4141889, -14420.72042, 20.11893564, -0.001971492619, 3.112721421e-007, -2.602979969e-011, 8.894342129999999e-016}, bhigh = {166419.6236, -120.1535761}, R = 460.944275399867);
          constant IdealGases.Common.DataRecord NO(name = "NO", MM = 0.0300061, Hf = 3041758.509103149, H0 = 305908.1320131574, Tlimit = 1000, alow = {-11439.16503, 153.6467592, 3.43146873, -0.002668592368, 8.48139912e-006, -7.685111050000001e-009, 2.386797655e-012}, blow = {9098.214410000001, 6.72872549}, ahigh = {223901.8716, -1289.651623, 5.43393603, -0.00036560349, 9.880966450000001e-008, -1.416076856e-011, 9.380184619999999e-016}, bhigh = {17503.17656, -8.50166909}, R = 277.0927244793559);
          constant IdealGases.Common.DataRecord NOCL(name = "NOCL", MM = 0.06545910000000001, Hf = 805064.9642295723, H0 = 173612.0417176527, Tlimit = 1000, alow = {23088.35209, -549.598384, 7.73046336, -0.0050739109, 1.062996184e-005, -8.7932497e-009, 2.648180166e-012}, blow = {7389.89839, -13.18393021}, ahigh = {-613341.333, -391.929883, 9.13891722, -0.002605664613, 1.295687247e-006, -2.215378352e-010, 1.280394898e-014}, bhigh = {4517.32842, -23.07323335}, R = 127.0178172324398);
          constant IdealGases.Common.DataRecord NOF(name = "NOF", MM = 0.0490045032, Hf = -1326408.712577255, H0 = 218760.9362398352, Tlimit = 1000, alow = {47550.2426, -725.3904170000001, 7.21399636, -0.002532427181, 6.3777439e-006, -5.51830588e-009, 1.681935713e-012}, blow = {-5609.72252, -12.89663616}, ahigh = {1889069.274, -6731.02266, 14.19018767, -0.00369312462, 9.93857514e-007, -1.080748188e-010, 4.21035443e-015}, bhigh = {32099.0078, -63.70266962}, R = 169.667509250456);
          constant IdealGases.Common.DataRecord NOF3(name = "NOF3", MM = 0.08700130959999999, Hf = -2149392.932816267, H0 = 157441.9174030456, Tlimit = 1000, alow = {148836.0135, -2241.049812, 13.02355027, 0.00546397668, -8.641865250000001e-006, 5.91365903e-009, -1.577009169e-012}, blow = {-13283.42568, -48.77320739}, ahigh = {-278562.5217, -1252.321663, 13.90824337, -0.00035666875, 7.78501106e-008, -8.85041197e-012, 4.07598003e-016}, bhigh = {-20256.51446, -51.06881858999999}, R = 95.56720511710552);
          constant IdealGases.Common.DataRecord NO2(name = "NO2", MM = 0.0460055, Hf = 743237.6346306421, H0 = 221890.3174620426, Tlimit = 1000, alow = {-56420.3878, 963.308572, -2.434510974, 0.01927760886, -1.874559328e-005, 9.145497730000001e-009, -1.777647635e-012}, blow = {-1547.925037, 40.6785121}, ahigh = {721300.157, -3832.6152, 11.13963285, -0.002238062246, 6.54772343e-007, -7.6113359e-011, 3.32836105e-015}, bhigh = {25024.97403, -43.0513004}, R = 180.7277825477389);
          constant IdealGases.Common.DataRecord NO2minus(name = "NO2minus", MM = 0.0460060486, Hf = -4348028.07646471, H0 = 221210.2388641132, Tlimit = 1000, alow = {-12820.67858, 699.013818, -2.812596273, 0.02412894252, -2.831606689e-005, 1.670509365e-008, -3.98333013e-012}, blow = {-28099.15579, 40.6327151}, ahigh = {132571.0335, -1557.032129, 8.12672192, -0.000272862678, -4.7075418e-008, 2.826729008e-011, -2.353985481e-015}, bhigh = {-17157.95217, -22.28576043}, R = 180.7256274558646);
          constant IdealGases.Common.DataRecord NO2CL(name = "NO2CL", MM = 0.0814585, Hf = 153452.3714529484, H0 = 149828.9190201145, Tlimit = 1000, alow = {8508.370340000001, -180.5383762, 3.78538856, 0.01414934934, -1.423946765e-005, 7.02822618e-009, -1.374688214e-012}, blow = {915.6246469999999, 6.958904458}, ahigh = {-108677.3327, -1452.231167, 11.05656962, -0.000400009928, 9.101543039999999e-008, -1.036656913e-011, 4.78166481e-016}, bhigh = {6294.26732, -35.21239681}, R = 102.0700356623311);
          constant IdealGases.Common.DataRecord NO2F(name = "NO2F", MM = 0.06500390319999999, Hf = -1676822.385028719, H0 = 174552.3028838675, Tlimit = 1000, alow = {56678.5695, -653.825195, 4.47277152, 0.01368870672, -1.460533236e-005, 7.779227940000001e-009, -1.689355106e-012}, blow = {-11021.79443, 0.329207431}, ahigh = {-100857.7842, -1704.722752, 11.22954945, -0.000468521597, 1.047692566e-007, -1.189150595e-011, 5.470307120000001e-016}, bhigh = {-6891.71918, -38.49788492}, R = 127.9072731127937);
          constant IdealGases.Common.DataRecord NO3(name = "NO3", MM = 0.0620049, Hf = 1147135.145770738, H0 = 176742.7090439627, Tlimit = 1000, alow = {34053.9841, 226.6670652, -3.79308163, 0.041707327, -5.709913270000001e-005, 3.83415811e-008, -1.021969284e-011}, blow = {7088.112200000001, 42.73091713}, ahigh = {-394387.271, -824.426353, 10.61325843, -0.0002448749816, 5.40606032e-008, -6.19546675e-012, 2.870000149e-016}, bhigh = {8982.01173, -34.44666597}, R = 134.0937893618085);
          constant IdealGases.Common.DataRecord NO3minus(name = "NO3minus", MM = 0.0620054486, Hf = -5012132.611197655, H0 = 173744.4892867044, Tlimit = 1000, alow = {92048.1361, -391.117115, -0.2354356764, 0.02836042108, -3.46132408e-005, 2.08178746e-008, -5.02160127e-012}, blow = {-35764.115, 22.99942308}, ahigh = {-311000.5758, -1369.087552, 11.01342913, -0.000403687882, 8.90208647e-008, -1.01973348e-011, 4.72333079e-016}, bhigh = {-33643.2109, -38.78432657}, R = 134.0926029523154);
          constant IdealGases.Common.DataRecord NO3F(name = "NO3F", MM = 0.0810033032, Hf = 185177.6335955643, H0 = 178315.6418243448, Tlimit = 1000, alow = {64728.3203, -821.3134309999999, 6.19491744, 0.01805438628, -1.99669324e-005, 1.124482018e-008, -2.680013077e-012}, blow = {4206.66179, -7.016104301}, ahigh = {-341179.33, -2353.908798, 16.28114887, -0.001910415273, 4.69087356e-007, -5.68604014e-011, 2.720906921e-015}, bhigh = {9760.583979999999, -65.58153684}, R = 102.6436166371052);
          constant IdealGases.Common.DataRecord N2(name = "N2", MM = 0.0280134, Hf = 0, H0 = 309498.4543111511, Tlimit = 1000, alow = {22103.71497, -381.846182, 6.08273836, -0.00853091441, 1.384646189e-005, -9.62579362e-009, 2.519705809e-012}, blow = {710.846086, -10.76003744}, ahigh = {587712.406, -2239.249073, 6.06694922, -0.00061396855, 1.491806679e-007, -1.923105485e-011, 1.061954386e-015}, bhigh = {12832.10415, -15.86640027}, R = 296.8033869505308);
          constant IdealGases.Common.DataRecord N2plus(name = "N2plus", MM = 0.0280128514, Hf = 53886282.4938985, H0 = 309540.07059774, Tlimit = 1000, alow = {-34740.4747, 269.6222703, 3.16491637, -0.002132239781, 6.730476399999999e-006, -5.63730497e-009, 1.621756e-012}, blow = {179000.4424, 6.832974166}, ahigh = {-2845599.002, 7058.89303, -2.884886385, 0.003068677059, -4.36165231e-007, 2.102514545e-011, 5.41199647e-016}, bhigh = {134038.8483, 50.90897022}, R = 296.809199509051);
          constant IdealGases.Common.DataRecord N2minus(name = "N2minus", MM = 0.0280139486, Hf = 5289624.969184102, H0 = 309641.5333609915, Tlimit = 1000, alow = {-81462.2711, 906.360079, -0.1520054079, 0.00602319084, -2.897138445e-006, -4.12910668e-011, 3.20698977e-013}, blow = {12188.08548, 26.38068855}, ahigh = {216963.7706, -1275.098516, 5.3910957, -0.000319890751, 7.311051349999999e-008, -8.202017370000001e-012, 3.7400447e-016}, bhigh = {24249.64308, -9.014934294}, R = 296.7975746196665);
          constant IdealGases.Common.DataRecord NCN(name = "NCN", MM = 0.0400241, Hf = 12503880.73685604, H0 = 254351.178415005, Tlimit = 1000, alow = {-56346.80699999999, 732.380458, -0.782140184, 0.01838552441, -1.950836491e-005, 1.035712021e-008, -2.208158483e-012}, blow = {55397.897, 29.05308985}, ahigh = {-164188.0975, -776.784075, 7.99998187, -0.0001659081508, 2.983403318e-008, -3.120157047e-012, 1.99269872e-016}, bhigh = {61844.24479999999, -21.4910882}, R = 207.7366386751982);
          constant IdealGases.Common.DataRecord N2D2_cis(name = "N2D2_cis", MM = 0.032041604, Hf = 6331060.392607061, H0 = 321707.8957720095, Tlimit = 1000, alow = {-27437.33656, 714.980883, -2.22324762, 0.02088722282, -1.821711897e-005, 8.84407994e-009, -1.918010649e-012}, blow = {20111.15649, 36.37100195}, ahigh = {879807.471, -5299.36204, 13.55007485, -0.001316635227, 2.755816197e-007, -3.036294387e-011, 1.365324117e-015}, bhigh = {53563.11889999999, -62.71215875}, R = 259.4898807188304);
          constant IdealGases.Common.DataRecord N2F2(name = "N2F2", MM = 0.06601020640000001, Hf = 944907.6044700869, H0 = 194951.0038193124, Tlimit = 1000, alow = {15438.9315, -218.363513, 3.89028425, 0.0167401774, -2.05639309e-005, 1.25869211e-008, -3.11049829e-012}, blow = {7052.0387, 5.265866442}, ahigh = {-182488.386, -953.402996, 10.6979548, -0.00027599687, 6.0554397e-008, -6.91121508e-012, 3.19258723e-016}, bhigh = {9283.46696, -32.46968772}, R = 125.9573701317801);
          constant IdealGases.Common.DataRecord N2F4(name = "N2F4", MM = 0.1040070128, Hf = -211524.1982990593, H0 = 171253.5580100807, Tlimit = 1000, alow = {116291.4512, -1538.660418, 9.054033049999999, 0.02862113563, -4.33228699e-005, 3.067642499e-008, -8.45547411e-012}, blow = {2865.277526, -24.76493006}, ahigh = {-518859.471, -670.225651, 16.50109262, -0.0002006404436, 4.43675251e-008, -5.08980888e-012, 2.359374159e-016}, bhigh = {-5281.4889, -60.40513435}, R = 79.94145563999894);
          constant IdealGases.Common.DataRecord N2H2(name = "N2H2", MM = 0.03002928, Hf = 7055072.782297812, H0 = 332907.1159881289, Tlimit = 1000, alow = {-150400.5163, 2346.687716, -9.40543029, 0.032842998, -3.121920401e-005, 1.72128319e-008, -4.014537220000001e-012}, blow = {13193.84041, 78.32382629999999}, ahigh = {6217567.87, -17539.52096, 20.22730509, -0.000975729766, -4.20841674e-007, 1.117921171e-010, -7.627102210000001e-015}, bhigh = {137415.2574, -119.9559168}, R = 276.8788329257312);
          constant IdealGases.Common.DataRecord NH2NO2(name = "NH2NO2", MM = 0.06202808, Hf = -419164.9975301508, H0 = 196102.3620270045, Tlimit = 1000, alow = {-45730.3506, 1201.365987, -8.10598411, 0.054027152, -6.43807445e-005, 4.02509792e-008, -1.02515419e-011}, blow = {-9615.78516, 68.67353357}, ahigh = {1654040.575, -8125.220880000001, 20.21742772, -0.001244291821, 2.122804183e-007, -1.948359653e-011, 7.43935136e-016}, bhigh = {42308.2258, -101.6190179}, R = 134.0436782824811);
          constant IdealGases.Common.DataRecord N2H4(name = "N2H4", MM = 0.03204516, Hf = 2970183.328777263, H0 = 357286.5293854048, Tlimit = 1000, alow = {-166075.6354, 3035.416736, -17.36889823, 0.0715983402, -8.8667993e-005, 5.79897028e-008, -1.530037218e-011}, blow = {-3731.92723, 119.0002218}, ahigh = {3293486.7, -11998.50628, 21.04406814, -0.001399381724, 1.933173351e-007, -1.318016127e-011, 3.16640017e-016}, bhigh = {83484.337, -115.5751024}, R = 259.4610855430274);
          constant IdealGases.Common.DataRecord N2O(name = "N2O", MM = 0.0440128, Hf = 1854006.107314236, H0 = 217685.1961247637, Tlimit = 1000, alow = {42882.2597, -644.011844, 6.03435143, 0.0002265394436, 3.47278285e-006, -3.62774864e-009, 1.137969552e-012}, blow = {11794.05506, -10.0312857}, ahigh = {343844.804, -2404.557558, 9.125636220000001, -0.000540166793, 1.315124031e-007, -1.4142151e-011, 6.38106687e-016}, bhigh = {21986.32638, -31.47805016}, R = 188.9103169986913);
          constant IdealGases.Common.DataRecord N2Oplus(name = "N2Oplus", MM = 0.04401225139999999, Hf = 30286033.19756553, H0 = 241373.8598248578, Tlimit = 1000, alow = {-56241.4708, 669.621161, 0.0878145619, 0.01524476027, -1.527290811e-005, 7.827237389999999e-009, -1.646739623e-012}, blow = {155729.5192, 25.62354785}, ahigh = {-29835.53254, -1179.455967, 8.30018669, -0.0002887267217, 5.70510501e-008, -5.95888512e-012, 2.835725557e-016}, bhigh = {164602.1769, -22.87356617}, R = 188.9126717112227);
          constant IdealGases.Common.DataRecord N2O3(name = "N2O3", MM = 0.0760116, Hf = 1139702.295439117, H0 = 225240.029153445, Tlimit = 1000, alow = {-92044.44170000001, 929.552015, 3.20366481, 0.01356473078, -6.26296607e-006, -1.402915559e-009, 1.43162093e-012}, blow = {3313.62208, 18.44430953}, ahigh = {778388.186, -4483.02466, 16.66668024, -0.002062143878, 5.309541710000001e-007, -6.19045122e-011, 2.692956658e-015}, bhigh = {33609.1245, -67.39212388}, R = 109.384251877345);
          constant IdealGases.Common.DataRecord N2O4(name = "N2O4", MM = 0.092011, Hf = 120756.4204279923, H0 = 181948.1475041028, Tlimit = 1000, alow = {-38047.5144, 561.2828890000001, -0.2083648324, 0.0388708782, -4.42241226e-005, 2.49881231e-008, -5.67910238e-012}, blow = {-3310.79473, 29.6392484}, ahigh = {-458284.3760000001, -1604.749805, 16.74102133, -0.0005091385080000001, 1.14363467e-007, -1.316288176e-011, 5.976316620000001e-016}, bhigh = {4306.90052, -65.69450380000001}, R = 90.36389127386944);
          constant IdealGases.Common.DataRecord N2O5(name = "N2O5", MM = 0.1080104, Hf = 123136.2905794257, H0 = 192550.2081281062, Tlimit = 1000, alow = {40078.2817, -876.9675120000001, 10.55932981, 0.01394613859, -8.884346920000001e-006, 8.500431150000001e-010, 7.79155091e-013}, blow = {3038.962037, -23.8683186}, ahigh = {-53255.7896, -3109.277389, 20.36088958, -0.000995990114, 2.401398635e-007, -3.057161911e-011, 1.495915511e-015}, bhigh = {13369.57281, -82.98623341000001}, R = 76.97843911327057);
          constant IdealGases.Common.DataRecord N3(name = "N3", MM = 0.0420201, Hf = 10375986.73016009, H0 = 227769.7102101138, Tlimit = 1000, alow = {33374.0679, -296.5683604, 3.31427915, 0.00672168536, -4.18112639e-006, 8.61844236e-010, 6.88335253e-014}, blow = {52988.4062, 5.312776486}, ahigh = {252926.4658, -2362.876591, 9.135267130000001, -0.000621287085, 1.324094351e-007, -1.47898964e-011, 6.721230470000001e-016}, bhigh = {64126.95389999999, -31.35825973}, R = 197.8689246336872);
          constant IdealGases.Common.DataRecord N3H(name = "N3H", MM = 0.04302804, Hf = 6832753.711300817, H0 = 254419.745821562, Tlimit = 1000, alow = {3242.57606, 66.9266489, 1.766142217, 0.01487411419, -1.53908644e-005, 9.172303550000001e-009, -2.337205474e-012}, blow = {33920.697, 15.13752057}, ahigh = {1170469.241, -5102.45199, 12.7828891, -0.000840948716, 1.592142834e-007, -1.512289051e-011, 6.102906629999999e-016}, bhigh = {64283.4447, -55.13119107999999}, R = 193.2338075357372);
          constant IdealGases.Common.DataRecord Na(name = "Na", MM = 0.02298977, Hf = 4675992.843773557, H0 = 269573.2928167615, Tlimit = 1000, alow = {0, 0, 2.5, 0, 0, 0, 0}, blow = {12183.82949, 4.24402818}, ahigh = {952572.3380000001, -2623.807254, 5.16259662, -0.001210218586, 2.306301844e-007, -1.249597843e-011, 7.226771190000001e-016}, bhigh = {29129.63564, -15.19717061}, R = 361.6596425279592);
          constant IdealGases.Common.DataRecord Naplus(name = "Naplus", MM = 0.0229892214, Hf = 26514291.9542286, H0 = 269579.7257405159, Tlimit = 1000, alow = {0, 0, 2.5, 0, 0, 0, 0}, blow = {72565.3707, 3.55084508}, ahigh = {0, 0, 2.5, 0, 0, 0, 0}, bhigh = {72565.3707, 3.55084508}, R = 361.6682729411619);
          constant IdealGases.Common.DataRecord Naminus(name = "Naminus", MM = 0.0229903186, Hf = 2107557.917879398, H0 = 269566.8602000148, Tlimit = 1000, alow = {0, 0, 2.500000001, 0, 0, 0, 0}, blow = {5082.19967, 3.55091679}, ahigh = {0, 0, 2.500000001, 0, 0, 0, 0}, bhigh = {5082.19967, 3.55091679}, R = 361.6510125266381);
          constant IdealGases.Common.DataRecord NaALF4(name = "NaALF4", MM = 0.1259649208, Hf = -14748880.35653812, H0 = 165194.1418916051, Tlimit = 1000, alow = {131535.9536, -2394.685695, 19.09539612, 0.001044339733, -6.949754129999999e-006, 6.769098139999999e-009, -2.176199098e-012}, blow = {-215051.2366, -75.0510213}, ahigh = {-357117.793, -249.6543879, 16.18634511, -7.449049300000001e-005, 1.644510872e-008, -1.883696066e-012, 8.719877450000001e-017}, bhigh = {-227953.3538, -53.7066514}, R = 66.00624957484196);
          constant IdealGases.Common.DataRecord NaBO2(name = "NaBO2", MM = 0.06579957, Hf = -9626952.425372992, H0 = 207944.6719788594, Tlimit = 1000, alow = {50468.6736, -869.99706, 8.942208239999999, 0.001092655824, 1.440608828e-006, -2.254633535e-009, 8.12331285e-013}, blow = {-73783.1836, -19.51815135}, ahigh = {85958.14049999999, -1691.902222, 11.18847517, -0.000456420231, 9.80450516e-008, -1.101655008e-011, 5.02917276e-016}, bhigh = {-69492.9206, -34.1635313}, R = 126.360582599552);
          constant IdealGases.Common.DataRecord NaBr(name = "NaBr", MM = 0.10289377, Hf = -1418247.985276465, H0 = 95449.10250640052, Tlimit = 1000, alow = {-14668.27136, 76.22282870000001, 3.92116347, 0.001793780908, -2.457391648e-006, 1.739334522e-009, -4.79670138e-013}, blow = {-19264.91755, 6.40438286}, ahigh = {897851.3019999999, -2697.899721, 7.54537716, -0.001566692184, 4.47039274e-007, -4.83281021e-011, 1.4239613e-015}, bhigh = {-1750.862247, -18.56478949}, R = 80.80636952072025);
          constant IdealGases.Common.DataRecord NaCN(name = "NaCN", MM = 0.04900717, Hf = 1923514.457170247, H0 = 247432.6103710947, Tlimit = 1000, alow = {24461.6541, -753.601997, 10.44966241, -0.01168769048, 1.849279385e-005, -1.319387974e-008, 3.58976656e-012}, blow = {12978.20493, -29.88519681}, ahigh = {366557.029, -1782.094358, 8.69962263, -0.0004463297, 9.36222931e-008, -1.03307132e-011, 4.650466810000001e-016}, bhigh = {20109.26347, -24.19276094}, R = 169.6582765338215);
          constant IdealGases.Common.DataRecord NaCL(name = "NaCL", MM = 0.05844277000000001, Hf = -3106370.283270283, H0 = 164521.9075002776, Tlimit = 1000, alow = {43623.78350000001, -758.303446, 8.259173000000001, -0.009640915140000001, 1.358854616e-005, -9.667032249999999e-009, 2.74626129e-012}, blow = {-19504.09477, -19.36687551}, ahigh = {331449.8760000001, -896.831565, 5.27728738, -0.0001475674008, -1.491128988e-008, 2.465673596e-011, -2.730355213e-015}, bhigh = {-17362.77667, -3.99828856}, R = 142.2669048712099);
          constant IdealGases.Common.DataRecord NaF(name = "NaF", MM = 0.0419881732, Hf = -7029524.423320232, H0 = 219707.3675022375, Tlimit = 1000, alow = {39598.8744, -653.4626069999999, 6.9411732, -0.00530781493, 6.97972066e-006, -4.82042573e-009, 1.364849175e-012}, blow = {-33529.4089, -14.03212229}, ahigh = {-1092926.912, 3293.30364, 0.413591984, 0.00263499447, -8.384295630000001e-007, 1.417053025e-010, -8.600270160000001e-015}, bhigh = {-57728.8463, 29.09489906}, R = 198.0193794189646);
          constant IdealGases.Common.DataRecord NaH(name = "NaH", MM = 0.02399771, Hf = 5868689.345775076, H0 = 363830.757184748, Tlimit = 1000, alow = {-32222.0641, 623.762201, -0.9216277509999999, 0.01360765851, -1.659249878e-005, 1.033284544e-008, -2.589726392e-012}, blow = {13073.81654, 26.41418597}, ahigh = {-4756184.75, 14520.47626, -13.27563485, 0.01055828277, -2.990041189e-006, 3.90532288e-010, -1.923931194e-014}, bhigh = {-76329.2227, 122.207006}, R = 346.4693922878474);
          constant IdealGases.Common.DataRecord NaI(name = "NaI", MM = 0.14989424, Hf = -604678.8722501947, H0 = 66394.27905968903, Tlimit = 1000, alow = {12288.68506, -285.71928, 5.961057, -0.00379253258, 5.56688131e-006, -4.045013680000001e-009, 1.172998933e-012}, blow = {-10882.50556, -3.99073992}, ahigh = {2281549.408, -7093.15663, 13.04994968, -0.00501349233, 1.581735155e-006, -2.285837754e-010, 1.19588995e-014}, bhigh = {32538.9362, -56.4002332}, R = 55.46892262170982);
          constant IdealGases.Common.DataRecord NaLi(name = "NaLi", MM = 0.02993077, Hf = 5967047.289461648, H0 = 333874.4709875489, Tlimit = 1000, alow = {-6569.99276, -5.25394043, 4.32938426, 0.001097919189, -1.965726597e-006, 2.086472026e-009, -8.19598811e-013}, blow = {20162.24293, 1.413253322}, ahigh = {10916648.6, -34800.1064, 46.14311919999999, -0.02260951051, 5.68532437e-006, -6.458143299999999e-010, 2.696508992e-014}, bhigh = {239443.1755, -296.1780126}, R = 277.7901136522716);
          constant IdealGases.Common.DataRecord NaNO2(name = "NaNO2", MM = 0.06899527000000001, Hf = -2410205.496695643, H0 = 214612.1176132798, Tlimit = 1000, alow = {-69344.60610000001, 1115.43267, -1.816284973, 0.02916366016, -3.50578852e-005, 2.128602855e-008, -5.226211990000001e-012}, blow = {-27072.75053, 41.21853253}, ahigh = {-176555.7495, -863.521295, 10.6410594, -0.0002559465044, 5.65446563e-008, -6.48671032e-012, 3.008171243e-016}, bhigh = {-18684.35672, -29.21304474}, R = 120.5078551036904);
          constant IdealGases.Common.DataRecord NaNO3(name = "NaNO3", MM = 0.08499466999999999, Hf = -3359371.958265148, H0 = 181202.997787979, Tlimit = 1000, alow = {-20438.52507, 691.709341, -2.397485012, 0.04100386430000001, -5.141702579999999e-005, 3.22862247e-008, -8.1646916e-012}, blow = {-39064.0132, 41.74749459}, ahigh = {-322559.219, -1396.782195, 14.03092409, -0.000409816177, 9.023845720000001e-008, -1.032543966e-011, 4.77867155e-016}, bhigh = {-31388.89875, -49.59121431}, R = 97.82345175291582);
          constant IdealGases.Common.DataRecord NaO(name = "NaO", MM = 0.03898917, Hf = 2731664.126217614, H0 = 250149.387637644, Tlimit = 1000, alow = {18577.48013, -337.149732, 5.64456002, -0.003136926368, 6.33077539e-006, -5.42946247e-009, 1.68718377e-012}, blow = {13203.32678, -4.99613115}, ahigh = {256974.4011, -2269.334161, 9.22439762, -0.0036512691, 1.446811119e-006, -2.443068386e-010, 1.428508328e-014}, bhigh = {24132.39357, -29.89159486}, R = 213.2508078525396);
          constant IdealGases.Common.DataRecord NaOH(name = "NaOH", MM = 0.03999711, Hf = -4775345.0186776, H0 = 284962.3635307651, Tlimit = 1000, alow = {34420.3674, -792.321818, 8.9979323, -0.00407984452, 3.065783937e-006, -5.11918934e-010, -1.541016409e-013}, blow = {-20869.51091, -25.1059009}, ahigh = {875378.776, -2342.514649, 7.97846989, 0.0001016451512, -6.26853195e-008, 1.022715136e-011, -5.71328641e-016}, bhigh = {-9509.90171, -22.02310401}, R = 207.87681910018);
          constant IdealGases.Common.DataRecord NaOHplus(name = "NaOHplus", MM = 0.0399965614, Hf = 17098029.10707219, H0 = 292396.9109004455, Tlimit = 1000, alow = {22780.39363, -667.219422, 8.921593120000001, -0.004481263270000001, 3.95188392e-006, -1.173341234e-009, 2.041806631e-014}, blow = {83633.8219, -22.58912262}, ahigh = {881541.365, -2363.159755, 8.053203930000001, 7.27127974e-005, -5.89741789e-008, 1.007487881e-011, -5.47440621e-016}, bhigh = {95844.41650000001, -20.79484012}, R = 207.8796703758639);
          constant IdealGases.Common.DataRecord Na2(name = "Na2", MM = 0.04597954, Hf = 3095705.720413906, H0 = 226255.5258273571, Tlimit = 1000, alow = {6848.62868, -153.0836599, 5.32523039, -0.001944906088, 2.657477888e-006, -9.096841120000001e-010, -2.44875673e-013}, blow = {16491.70574, -2.653564394}, ahigh = {19299407.58, -62692.8012, 82.67682110000001, -0.0456513781, 1.259515667e-005, -1.560445735e-009, 7.02467717e-014}, bhigh = {409082.08, -550.997089}, R = 180.8298212639796);
          constant IdealGases.Common.DataRecord Na2Br2(name = "Na2Br2", MM = 0.20578754, Hf = -2336625.691720694, H0 = 95202.78535814169, Tlimit = 1000, alow = {-13849.46731, -157.3241177, 10.63192589, -0.001383418339, 1.697156537e-006, -1.092992547e-009, 2.87098802e-013}, blow = {-60103.9182, -18.61473856}, ahigh = {-30217.30975, -2.65923278, 10.00225819, -9.935490630000001e-007, 2.358676253e-010, -2.857194816e-014, 1.381909879e-018}, bhigh = {-60900.3992, -14.94517863}, R = 40.40318476036013);
          constant IdealGases.Common.DataRecord Na2CL2(name = "Na2CL2", MM = 0.11688554, Hf = -4828670.706402178, H0 = 159979.8144406913, Tlimit = 1000, alow = {-10829.5525, -313.9528641, 11.24411332, -0.002697665795, 3.28635948e-006, -2.105377639e-009, 5.508056609999999e-013}, blow = {-69386.7543, -25.11011582}, ahigh = {-44121.0523, -5.39349128, 10.00453407, -1.981155641e-006, 4.6801882e-010, -5.64894068e-014, 2.724721177e-018}, bhigh = {-70980.6341, -17.87230397}, R = 71.13345243560495);
          constant IdealGases.Common.DataRecord Na2F2(name = "Na2F2", MM = 0.0839763464, Hf = -9932115.551052362, H0 = 198655.1060525777, Tlimit = 1000, alow = {23515.80802, -1000.737062, 13.75845076, -0.007838346979999999, 9.2762887e-006, -5.81361297e-009, 1.49535482e-012}, blow = {-98358.31880000001, -43.8114809}, ahigh = {-90307.50689999999, -18.93938035, 10.01535518, -6.53967545e-006, 1.516333603e-009, -1.804877459e-013, 8.61347492e-018}, bhigh = {-103489.6664, -21.78860202}, R = 99.00968970948229);
          constant IdealGases.Common.DataRecord Na2I2(name = "Na2I2", MM = 0.29978848, Hf = -1190405.755417953, H0 = 67534.50966494776, Tlimit = 1000, alow = {-13050.63983, -89.9250668, 10.36367976, -0.0008000070190000001, 9.84877798e-007, -6.35937312e-010, 1.673774775e-013}, blow = {-45514.6549, -14.87509548}, ahigh = {-22312.36397, -1.515570558, 10.00129413, -5.71503538e-007, 1.360287207e-010, -1.650916036e-014, 7.996149680000001e-019}, bhigh = {-45969.2175, -12.76523965}, R = 27.73446131085491);
          constant IdealGases.Common.DataRecord Na2O(name = "Na2O", MM = 0.06197894, Hf = -267184.7888976482, H0 = 232501.0721383747, Tlimit = 1000, alow = {39011.49290000001, -726.620789, 9.62371078, -0.00355641864, 3.47070435e-006, -1.835177736e-009, 4.06213471e-013}, blow = {-459.307325, -23.49565832}, ahigh = {-66005.2516, -25.69021634, 7.51938542, -7.81163524e-006, 1.73501589e-009, -1.996634558e-013, 9.27643355e-018}, bhigh = {-4297.33965, -10.63530214}, R = 134.1499548072297);
          constant IdealGases.Common.DataRecord Na2Oplus(name = "Na2Oplus", MM = 0.0619783914, Hf = 8403476.699461419, H0 = 235671.9442060253, Tlimit = 1000, alow = {26479.69755, -596.906216, 9.294129529999999, -0.003080702005, 3.080460653e-006, -1.669452593e-009, 3.78978414e-013}, blow = {63473.10249999999, -20.25274459}, ahigh = {-57409.0933, -19.89178143, 7.51529526, -6.25833323e-006, 1.407321045e-009, -1.635833952e-013, 7.66244513e-018}, bhigh = {60329.77469999999, -9.42652674}, R = 134.1511422318069);
          constant IdealGases.Common.DataRecord Na2O2(name = "Na2O2", MM = 0.07797834000000001, Hf = -1589291.911061456, H0 = 199609.9814384353, Tlimit = 1000, alow = {73824.5892, -1355.12534, 12.55579861, -0.002112046045, 4.92035352e-008, 1.003950603e-009, -4.44773207e-013}, blow = {-10588.58918, -40.2181178}, ahigh = {-173229.5239, -113.7561118, 10.08529143, -3.42209894e-005, 7.577721890000001e-009, -8.70125673e-013, 4.03606139e-017}, bhigh = {-17803.00574, -23.8678919}, R = 106.6254039262698);
          constant IdealGases.Common.DataRecord Na2O2H2(name = "Na2O2H2", MM = 0.07999422000000001, Hf = -7800563.59071943, H0 = 241844.2357460327, Tlimit = 1000, alow = {108941.4289, -2634.822704, 23.06608858, -0.01689310193, 1.67348319e-005, -7.821166009999999e-009, 1.47503447e-012}, blow = {-65931.37330000001, -98.0298758}, ahigh = {1675713.839, -4704.921170000001, 16.97343934, 0.0001961487616, -1.236951748e-007, 2.025311778e-011, -1.132991623e-015}, bhigh = {-48562.4557, -68.1348098}, R = 103.93840955009);
          constant IdealGases.Common.DataRecord Na2SO4(name = "Na2SO4", MM = 0.14204214, Hf = -7322702.276944011, H0 = 147644.1991087997, Tlimit = 1000, alow = {83442.8321, -1210.880769, 8.742353, 0.0360079663, -5.11764596e-005, 3.48156904e-008, -9.321931219999999e-012}, blow = {-121738.7045, -20.61505785}, ahigh = {-575448.311, -981.7060339999999, 19.73310397, -0.000293639552, 6.498708169999999e-008, -7.462479400000001e-012, 3.46249166e-016}, bhigh = {-127059.7762, -76.67871596000001}, R = 58.53524876490879);
          constant IdealGases.Common.DataRecord Na3CL3(name = "Na3CL3", MM = 0.17532831, Hf = -5205517.061106674, H0 = 168125.4841274635, Tlimit = 1000, alow = {-12542.1667, -550.216046, 18.16110649, -0.00465651526, 5.64643822e-006, -3.60476032e-009, 9.405556239999999e-013}, blow = {-111927.0172, -52.9010496}, ahigh = {-71602.9797, -9.57349381, 16.00799614, -3.47835697e-006, 8.190943689999999e-010, -9.86321003e-014, 4.74900535e-018}, bhigh = {-114725.2948, -40.31342160000001}, R = 47.42230162373664);
          constant IdealGases.Common.DataRecord Na3F3(name = "Na3F3", MM = 0.1259645196, Hf = -10701548.26359533, H0 = 203249.8840252791, Tlimit = 1000, alow = {52470.3035, -1775.597273, 22.50927609, -0.01333033615, 1.555790239e-005, -9.64601146e-009, 2.460254271e-012}, blow = {-158073.6389, -85.80122369999999}, ahigh = {-155399.5698, -35.5197019, 16.02841159, -1.198056094e-005, 2.75724873e-009, -3.26330097e-013, 1.550483212e-017}, bhigh = {-167214.1773, -47.5378068}, R = 66.00645980632153);
          constant IdealGases.Common.DataRecord Nb(name = "Nb", MM = 0.09290638, Hf = 7783244.799765098, H0 = 89919.54050948923, Tlimit = 1000, alow = {78896.60669999999, -1212.813914, 10.34579819, -0.01676630056, 1.979119979e-005, -1.218224409e-008, 3.058098336e-012}, blow = {91653.1514, -35.9474285}, ahigh = {-1096553.196, 2546.650713, 2.236054882, -0.001280029198, 8.464237990000001e-007, -1.486269508e-010, 8.714309406000001e-015}, bhigh = {68791.24550000001, 13.9816903}, R = 89.49301436564421);
          constant IdealGases.Common.DataRecord Nbplus(name = "Nbplus", MM = 0.0929058314, Hf = 15000185.17675092, H0 = 92449.25609696444, Tlimit = 1000, alow = {131444.7859, -2000.135035, 15.05024212, -0.02996583942, 3.72986863e-005, -2.269869569e-008, 5.449089902e-012}, blow = {176005.4029, -62.2459552}, ahigh = {-1077639.646, 2159.046421, 2.310604767, -0.0005363991760000001, 5.05791509e-007, -1.032401533e-010, 6.629241279999999e-015}, bhigh = {151794.5546, 12.10678502}, R = 89.49354281328783);
          constant IdealGases.Common.DataRecord Nbminus(name = "Nbminus", MM = 0.09290692859999999, Hf = 6792324.507001301, H0 = 93144.91535134014, Tlimit = 1000, alow = {-72209.2485, 525.950125, 3.47046839, -0.00495050553, 7.40185903e-006, -5.01630207e-009, 1.314100719e-012}, blow = {71788.28870000001, 5.15551007}, ahigh = {111745.8019, 134.0072834, 2.391474129, 4.52481343e-005, -1.025345e-008, 1.195577988e-012, -5.606330449999999e-017}, bhigh = {74739.9675, 9.67531561}, R = 89.49248592424141);
          constant IdealGases.Common.DataRecord NbCL5(name = "NbCL5", MM = 0.27017138, Hf = -2603273.522162118, H0 = 97505.37973341218, Tlimit = 1000, alow = {73482.7797, -1919.172996, 22.90799567, -0.01404352774, 1.638106985e-005, -1.018941843e-008, 2.612266639e-012}, blow = {-79741.1881, -84.39637087}, ahigh = {-156381.6638, -44.5052128, 16.0354463, -1.489949737e-005, 3.42077792e-009, -4.04112442e-013, 1.917266128e-017}, bhigh = {-89628.15120000001, -43.74582607}, R = 30.77480671712896);
          constant IdealGases.Common.DataRecord NbO(name = "NbO", MM = 0.10890578, Hf = 1937350.855023489, H0 = 80621.12038497864, Tlimit = 1000, alow = {-6797.834360000001, 283.9767438, 0.56457084, 0.01134928619, -1.549821141e-005, 1.047624988e-008, -2.762937835e-012}, blow = {23179.93893, 23.65708974}, ahigh = {553225.878, -1287.669306, 4.98006604, 0.0001116014163, 4.03183868e-008, 1.04877371e-011, -1.893595022e-015}, bhigh = {32684.5779, -1.868958549}, R = 76.34555300921586);
          constant IdealGases.Common.DataRecord NbOCL3(name = "NbOCL3", MM = 0.21526478, Hf = -3494765.841397743, H0 = 95840.10909727082, Tlimit = 1000, alow = {-110848.2359, 117.3066983, 11.65558433, 0.001287422064, -1.659778281e-006, 1.102107826e-009, -2.949903264e-013}, blow = {-95040.2392, -23.86450092}, ahigh = {-151555.1427, 53.0554523, 12.28915282, -0.0001527366571, 6.778576440000001e-008, -9.36517065e-012, 4.67003693e-016}, bhigh = {-94907.68120000001, -27.45263834}, R = 38.62439550027646);
          constant IdealGases.Common.DataRecord NbO2(name = "NbO2", MM = 0.12490518, Hf = -1611356.326454996, H0 = 85299.52080450147, Tlimit = 1000, alow = {17390.46582, 38.8102944, 0.902429912, 0.01909523319, -2.667013583e-005, 1.813901051e-008, -4.872365880000001e-012}, blow = {-25285.18558, 22.89323379}, ahigh = {-685185.99, 911.224932, 6.33919984, -8.2300803e-005, 1.984439605e-007, -3.26117149e-011, 1.602430854e-015}, bhigh = {-33250.2732, -3.55955655}, R = 66.56627051015819);
          constant IdealGases.Common.DataRecord Ne(name = "Ne", MM = 0.0201797, Hf = 0, H0 = 307111.9986917546, Tlimit = 1000, alow = {0, 0, 2.5, 0, 0, 0, 0}, blow = {-745.375, 3.35532272}, ahigh = {0, 0, 2.5, 0, 0, 0, 0}, bhigh = {-745.375, 3.35532272}, R = 412.0215860493466);
          constant IdealGases.Common.DataRecord Neplus(name = "Neplus", MM = 0.0201791514, Hf = 103421888.4942803, H0 = 312412.2454425909, Tlimit = 1000, alow = {72815.5148, -869.5697989999999, 6.10864697, -0.00584135693, 5.041044170000001e-006, -2.293759207e-009, 4.33906568e-013}, blow = {254599.689, -16.73449355}, ahigh = {-111274.2658, 476.569797, 2.196650531, 0.0001102593151, -2.287564425e-008, 2.510218183e-012, -1.126646096e-016}, bhigh = {247253.6944, 7.46614054}, R = 412.0327874639962);
          constant IdealGases.Common.DataRecord Ni(name = "Ni", MM = 0.0586934, Hf = 7328193.715136625, H0 = 116282.4610603577, Tlimit = 1000, alow = {-32358.1055, 601.526462, -1.079270657, 0.01089505519, -1.369578748e-005, 8.317725790000001e-009, -2.019206968e-012}, blow = {48138.1081, 27.188292}, ahigh = {-493826.221, 1092.909991, 2.410485014, -1.599071827e-005, -1.047414069e-008, 4.62479521e-012, -4.448865218e-017}, bhigh = {43360.7217, 9.677195599999999}, R = 141.6594029311643);
          constant IdealGases.Common.DataRecord Niplus(name = "Niplus", MM = 0.0586928514, Hf = 19978490.48104008, H0 = 105733.0296956743, Tlimit = 1000, alow = {-89693.86030000002, 1173.6015, -3.41062041, 0.01390739137, -1.501714923e-005, 7.8963379e-009, -1.648686761e-012}, blow = {134558.95, 40.3149516}, ahigh = {-3961999.32, 10170.84853, -6.02933129, 0.002770858029, -8.9020777e-008, -5.54100058e-011, 5.235342833e-015}, bhigh = {73403.9512, 71.37503100000001}, R = 141.6607270165784);
          constant IdealGases.Common.DataRecord Niminus(name = "Niminus", MM = 0.0586939486, Hf = 5311695.062887626, H0 = 105754.6331105078, Tlimit = 1000, alow = {-84376.2475, 1135.476552, -3.38061583, 0.01423003786, -1.582586302e-005, 8.608840410000001e-009, -1.875316029e-012}, blow = {31243.0759, 39.980613}, ahigh = {-543342.48, 1182.64533, 2.12644124, 3.73045594e-005, 6.95360843e-009, -1.945719381e-012, 1.271571579e-016}, bhigh = {28547.59122, 10.43462235}, R = 141.6580788705022);
          constant IdealGases.Common.DataRecord NiCL(name = "NiCL", MM = 0.09414640000000001, Hf = 1933201.906817467, H0 = 100534.0724658617, Tlimit = 1000, alow = {-23579.97085, 220.549163, 2.714260287, 0.00445359847, -2.849162229e-006, -1.691898007e-010, 5.15669926e-013}, blow = {19572.2949, 14.23643068}, ahigh = {-3905769.19, 9961.53399, -3.83943234, 0.002767979391, -7.00595938e-008, -5.74140366e-011, 5.3076358e-015}, bhigh = {-45053.9705, 67.69402548000001}, R = 88.31428498593679);
          constant IdealGases.Common.DataRecord NiCL2(name = "NiCL2", MM = 0.1295994, Hf = -570457.8879223206, H0 = 109618.9179888179, Tlimit = 1000, alow = {71099.7653, -1218.958288, 12.08750596, -0.00867440314, 1.043608104e-005, -6.298493009999999e-009, 1.485857653e-012}, blow = {-5006.998710000001, -34.99472036}, ahigh = {158588.9817, -1161.488738, 9.608015379999999, -0.0009461224509999999, 2.608172043e-007, -3.26463725e-011, 1.525950893e-015}, bhigh = {-4578.98554, -22.12397347}, R = 64.15517355790227);
          constant IdealGases.Common.DataRecord NiO(name = "NiO", MM = 0.0746928, Hf = 3977143.713985819, H0 = 118707.1444637234, Tlimit = 1000, alow = {23206.30462, -190.7136094, 3.19017862, 0.00529138349, -8.080210429999999e-006, 5.83981276e-009, -1.639499596e-012}, blow = {35767.1112, 7.84126567}, ahigh = {-72340.9194, -80.3555427, 4.55922102, 2.30515451e-005, 5.139832819999999e-009, -5.85347491e-013, 2.697077329e-017}, bhigh = {34611.4385, 1.209414814}, R = 111.3155752629437);
          constant IdealGases.Common.DataRecord NiS(name = "NiS", MM = 0.09075839999999999, Hf = 3938136.85565193, H0 = 101572.1630174177, Tlimit = 1000, alow = {-12724.88974, 143.2801381, 2.471951556, 0.00550527112, -3.41835528e-006, -3.099052241e-010, 6.26815467e-013}, blow = {41177.1608, 15.17981835}, ahigh = {-735812.898, 1178.231095, 4.83786558, -0.000393120139, 1.341974852e-007, -1.569591708e-011, 6.67704506e-016}, bhigh = {32755.6594, 3.690652283}, R = 91.61104647062973);
          constant IdealGases.Common.DataRecord O(name = "O", MM = 0.0159994, Hf = 15574021.71331425, H0 = 420353.4507544033, Tlimit = 1000, alow = {-7953.611300000001, 160.7177787, 1.966226438, 0.00101367031, -1.110415423e-006, 6.5175075e-010, -1.584779251e-013}, blow = {28403.62437, 8.404241819999999}, ahigh = {261902.0262, -729.872203, 3.31717727, -0.000428133436, 1.036104594e-007, -9.438304329999999e-012, 2.725038297e-016}, bhigh = {33924.2806, -0.667958535}, R = 519.6739877745415);
          constant IdealGases.Common.DataRecord Oplus(name = "Oplus", MM = 0.0159988514, Hf = 98056240.96239808, H0 = 387367.0581126842, Tlimit = 1000, alow = {0, 0, 2.5, 0, 0, 0, 0}, blow = {187935.2842, 4.39337676}, ahigh = {-216651.3208, 666.545615, 1.702064364, 0.000471499281, -1.427131823e-007, 2.016595903e-011, -9.107157761999999e-016}, bhigh = {183719.1966, 10.05690382}, R = 519.691807375622);
          constant IdealGases.Common.DataRecord Ominus(name = "Ominus", MM = 0.0159999486, Hf = 6365407.448871429, H0 = 410675.8192960695, Tlimit = 1000, alow = {-5695.857110000001, 109.9287334, 2.184719661, 0.0005326359799999999, -5.298878440000001e-007, 2.870216236e-010, -6.52469274e-014}, blow = {10932.87498, 6.72986386}, ahigh = {9769.363179999999, 7.15960478, 2.494961726, 1.968240938e-006, -4.30417485e-010, 4.912083080000001e-014, -2.271600083e-018}, bhigh = {11495.54438, 4.83703644}, R = 519.6561693954442);
          constant IdealGases.Common.DataRecord OD(name = "OD", MM = 0.018013502, Hf = 1952535.825626799, H0 = 0, Tlimit = 1000, alow = {21186.91536, -278.598236, 5.45621012, -0.00614811983, 9.117670560000001e-006, -5.527812710000001e-009, 1.239794711e-012}, blow = {0, 4464.87825}, ahigh = {8999.108, 783247.316, -2532.992554, 5.95212465, -0.000374359528, 4.95952762e-008, 3.45445473e-012}, bhigh = {-7.3806268e-016, 0}, R = 461.5688831633072);
          constant IdealGases.Common.DataRecord ODminus(name = "ODminus", MM = 0.0180140506, Hf = -8191266.876978795, H0 = 0, Tlimit = 1000, alow = {56061.2832, -751.415615, 7.54418847, -0.01092083064, 1.503688938e-005, -9.35166584e-009, 2.246466046e-012}, blow = {0, -15157.0737}, ahigh = {8642.103999999999, 302946.7029, -1079.684654, 4.21141738, 0.000626083007, -2.411665054e-007, 4.248425540000001e-011}, bhigh = {-2.387099102e-015, -11889.35343}, R = 461.5548265418994);
          constant IdealGases.Common.DataRecord OH(name = "OH", MM = 0.01700734, Hf = 2191889.266634288, H0 = 518194.2620068747, Tlimit = 1000, alow = {-1998.85899, 93.0013616, 3.050854229, 0.001529529288, -3.157890998e-006, 3.31544618e-009, -1.138762683e-012}, blow = {2991.214235, 4.67411079}, ahigh = {1017393.379, -2509.957276, 5.11654786, 0.000130529993, -8.284322259999999e-008, 2.006475941e-011, -1.556993656e-015}, bhigh = {20196.40206, -11.01282337}, R = 488.8755090449183);
          constant IdealGases.Common.DataRecord OHplus(name = "OHplus", MM = 0.0170067914, Hf = 76393787.19021624, H0 = 505862.7931427441, Tlimit = 1000, alow = {60316.3086, -757.35203, 7.30775293, -0.00950688167, 1.202555795e-005, -6.8290261e-009, 1.501588659e-012}, blow = {158926.2158, -19.50106996}, ahigh = {504072.91, -1380.052958, 4.1254622, 0.000833194884, -3.44285629e-007, 6.792853949999999e-011, -4.36387213e-015}, bhigh = {164383.9235, -3.99705849}, R = 488.8912790451466);
          constant IdealGases.Common.DataRecord OHminus(name = "OHminus", MM = 0.0170078886, Hf = -8540519.015393833, H0 = 506006.6068400753, Tlimit = 1000, alow = {29108.80827, -321.690494, 4.85102905, -0.002579035357, 2.004980024e-006, -7.956852959999999e-011, -2.320495634e-013}, blow = {-16888.86234, -7.1215913}, ahigh = {471133.117, -857.233669, 3.16618121, 0.001233581296, -3.99924458e-007, 6.23908167e-011, -3.35434322e-015}, bhigh = {-12248.49139, 1.48773626}, R = 488.8597400620322);
          constant IdealGases.Common.DataRecord O2(name = "O2", MM = 0.0319988, Hf = 0, H0 = 271263.4223783392, Tlimit = 1000, alow = {-34255.6342, 484.700097, 1.119010961, 0.00429388924, -6.83630052e-007, -2.0233727e-009, 1.039040018e-012}, blow = {-3391.45487, 18.4969947}, ahigh = {-1037939.022, 2344.830282, 1.819732036, 0.001267847582, -2.188067988e-007, 2.053719572e-011, -8.193467050000001e-016}, bhigh = {-16890.10929, 17.38716506}, R = 259.8369938872708);
          constant IdealGases.Common.DataRecord O2plus(name = "O2plus", MM = 0.0319982514, Hf = 36621639.76872811, H0 = 290988.1506837588, Tlimit = 1000, alow = {-86072.0545, 1051.875934, -0.543238047, 0.00657116654, -3.27426375e-006, 5.940645339999999e-011, 3.23878479e-013}, blow = {134554.4668, 29.0270975}, ahigh = {73846.5488, -845.955954, 4.98516416, -0.000161101089, 6.42708399e-008, -1.504939874e-011, 1.578465409e-015}, bhigh = {144632.1044, -5.81123065}, R = 259.841448711163);
          constant IdealGases.Common.DataRecord O2minus(name = "O2minus", MM = 0.0319993486, Hf = -1500903.084008404, H0 = 292181.2914654145, Tlimit = 1000, alow = {18838.74344, 114.9551768, 1.518876821, 0.00801611138, -9.850571029999999e-006, 6.04419621e-009, -1.486439845e-012}, blow = {-7101.53876, 15.0121038}, ahigh = {-56552.0805, -236.7815862, 4.67583367, -2.1972453e-005, 1.71150928e-008, -1.757645062e-012, 8.24817279e-017}, bhigh = {-5960.17775, -2.436885556}, R = 259.8325392161265);
          constant IdealGases.Common.DataRecord O3(name = "O3", MM = 0.0479982, Hf = 2954277.45207112, H0 = 215972.786479493, Tlimit = 1000, alow = {-12823.14507, 589.8216640000001, -2.547496763, 0.02690121526, -3.52825834e-005, 2.312290922e-008, -6.04489327e-012}, blow = {13483.68701, 38.5221858}, ahigh = {-38696624.8, 102334.4994, -89.615516, 0.0370614497, -4.13763874e-006, -2.725018591e-010, 5.24818811e-014}, bhigh = {-651791.818, 702.9109520000001}, R = 173.2246625915139);
          constant IdealGases.Common.DataRecord P(name = "P", MM = 0.030973761, Hf = 10218326.40860114, H0 = 200086.389250566, Tlimit = 1000, alow = {50.4086657, -0.7639418649999999, 2.504563992, -1.381689958e-005, 2.245585515e-008, -1.866399889e-011, 6.227063395e-015}, blow = {37324.2191, 5.359303481}, ahigh = {1261794.642, -4559.83819, 8.91807931, -0.00438140146, 1.454286224e-006, -2.030782763e-010, 1.021022887e-014}, bhigh = {65417.23959999999, -39.15974795}, R = 268.4359836056074);
          constant IdealGases.Common.DataRecord Pplus(name = "Pplus", MM = 0.0309732124, Hf = 43148687.41222335, H0 = 262867.7934614235, Tlimit = 1000, alow = {-73169.0811, 962.7979140000001, -0.369393805, 0.004766778340000001, -4.57476858e-006, 2.371262331e-009, -5.131314899999999e-013}, blow = {154940.6849, 23.76640762}, ahigh = {-559424.936, 1722.576545, 0.8430381089999999, 0.0006287368940000001, -6.317195459999999e-008, -1.810842484e-012, 4.31811257e-016}, bhigh = {149049.3431, 18.45275207}, R = 268.4407381650862);
          constant IdealGases.Common.DataRecord Pminus(name = "Pminus", MM = 0.0309743096, Hf = 7710479.041637784, H0 = 217853.6047176335, Tlimit = 1000, alow = {-10089.49093, 182.6468403, 1.962456304, 0.0009197377539999999, -9.21499863e-007, 5.012872360000001e-010, -1.142677121e-013}, blow = {27030.82432, 9.478137822000001}, ahigh = {15434.88016, 9.495009339999999, 2.493170861, 2.700167711e-006, -5.94921039e-010, 6.82376629e-014, -3.16695991e-018}, bhigh = {27975.72693, 6.246793872}, R = 268.4312292145488);
          constant IdealGases.Common.DataRecord PCL(name = "PCL", MM = 0.066426761, Hf = 2026519.28189002, H0 = 139870.0141348153, Tlimit = 1000, alow = {34888.617, -560.3119340000001, 6.26456848, -0.00318336437, 3.49532684e-006, -2.091034291e-009, 5.4187586e-013}, blow = {17746.53764, -8.074572148}, ahigh = {-347168.151, 993.555793, 3.39943027, 0.00040197117, 8.295588390000001e-008, -3.090212484e-011, 2.106384858e-015}, bhigh = {8433.59115, 10.64902238}, R = 125.1675059092525);
          constant IdealGases.Common.DataRecord PCL2(name = "PCL2", MM = 0.101879761, Hf = -532906.1676931104, H0 = 120227.5788613207, Tlimit = 1000, alow = {51900.845, -1060.9042, 10.5854994, -0.00687991753, 7.62077318e-006, -4.53011547e-009, 1.11667166e-012}, blow = {-3220.33118, -27.53518359}, ahigh = {-83498.33970000001, -26.2524947, 7.02036181, -8.388768680000001e-006, 1.89678474e-009, -2.21487992e-013, 1.04165355e-017}, bhigh = {-8742.99368, -6.23402319}, R = 81.61063510936191);
          constant IdealGases.Common.DataRecord PCL2minus(name = "PCL2minus", MM = 0.1018803096, Hf = -3497096.292687356, H0 = 122234.0906588686, Tlimit = 1000, alow = {49348.3277, -960.159054, 9.8847697, -0.00495020295, 4.94621787e-006, -2.678675201e-009, 6.07672361e-013}, blow = {-39980.724, -24.13131695}, ahigh = {-85649.8676, -31.8555594, 7.02449496, -1.002251172e-005, 2.253747944e-009, -2.619657192e-013, 1.227059658e-017}, bhigh = {-45038.1143, -6.721522361}, R = 81.6101956564922);
          constant IdealGases.Common.DataRecord PCL3(name = "PCL3", MM = 0.137332761, Hf = -2108018.493853772, H0 = 116013.585425549, Tlimit = 1000, alow = {77177.4547, -1617.65086, 15.3608047, -0.0101107705, 1.10340834e-005, -6.47612102e-009, 1.57915114e-012}, blow = {-29558.9364, -52.44338688}, ahigh = {-133307.693, -41.4588321, 10.0318606, -1.30194616e-005, 2.92291689e-009, -3.39186924e-013, 1.58642553e-017}, bhigh = {-38003.3668, -20.50737678}, R = 60.54252415416013);
          constant IdealGases.Common.DataRecord PCL5(name = "PCL5", MM = 0.208238761, Hf = -1805619.6559871, H0 = 111917.0412274975, Tlimit = 1000, alow = {102907.02, -2515.70746, 24.3011497, -0.0156461069, 1.70987303e-005, -1.00601143e-008, 2.46015879e-012}, blow = {-37225.8237, -98.16335170000001}, ahigh = {-225590.689, -69.38542820000001, 16.0535963, -2.20215696e-005, 4.97139229e-009, -5.79986005e-013, 2.72597077e-017}, bhigh = {-50342.3422, -48.7245487}, R = 39.92759061796377);
          constant IdealGases.Common.DataRecord PF(name = "PF", MM = 0.0499721642, Hf = -959429.9900263273, H0 = 177921.1915740883, Tlimit = 1000, alow = {22278.66728, -172.7508706, 3.085366934, 0.00548971823, -8.247972539999999e-006, 5.86888291e-009, -1.617252113e-012}, blow = {-5809.27965, 7.70971885}, ahigh = {-1132572.282, 3200.03855, 0.778050237, 0.001969518902, -4.36581897e-007, 5.243648570000001e-011, -2.618755712e-015}, bhigh = {-27667.73009, 27.59312302}, R = 166.3820675591232);
          constant IdealGases.Common.DataRecord PFplus(name = "PFplus", MM = 0.0499716156, Hf = 18040601.43294627, H0 = 188704.0850446308, Tlimit = 1000, alow = {-17791.38157, 390.999127, 1.257434156, 0.007861363749999999, -9.34231268e-006, 5.630915650000001e-009, -1.370669924e-012}, blow = {105487.3882, 19.01676576}, ahigh = {-40115.2518, -159.7504413, 4.62139748, -2.378072183e-005, 1.431831329e-008, -2.300109157e-012, 1.694069882e-016}, bhigh = {107863.3756, -0.04072343949999999}, R = 166.3838941400966);
          constant IdealGases.Common.DataRecord PFminus(name = "PFminus", MM = 0.0499727128, Hf = -3282711.520115834, H0 = 190416.7788145374, Tlimit = 1000, alow = {7705.21095, -115.2432706, 4.25495231, 0.001163521526, -1.627947259e-006, 1.084382815e-009, -2.809490132e-013}, blow = {-20355.58966, 2.217923793}, ahigh = {-130379.3388, 271.5526386, 4.13761981, 0.0002728610614, -7.64441625e-008, 1.182073001e-011, -6.083906640000001e-016}, bhigh = {-22907.99827, 3.7485261}, R = 166.3802410182543);
          constant IdealGases.Common.DataRecord PFCL(name = "PFCL", MM = 0.0854251642, Hf = -3314996.484373161, H0 = 136147.0136922488, Tlimit = 1000, alow = {54901.8853, -865.867948, 7.94446765, 0.0009503779099999999, -3.33909021e-006, 3.023977817e-009, -9.45037754e-013}, blow = {-31328.43217, -14.40426886}, ahigh = {-128214.3211, -97.3904355, 7.07266166, -2.903614134e-005, 6.408504490000001e-009, -7.33899377e-013, 3.39672317e-017}, bhigh = {-36006.4926, -7.626493035}, R = 97.33047724127172);
          constant IdealGases.Common.DataRecord PFCLminus(name = "PFCLminus", MM = 0.0854257128, Hf = -6195665.094877616, H0 = 137462.042927197, Tlimit = 1000, alow = {89343.3685, -1266.972513, 9.912194530000001, -0.0038779283, 3.036692411e-006, -1.29655938e-009, 2.32422658e-013}, blow = {-58945.0571, -26.1705046}, ahigh = {-123156.515, -97.6586111, 7.07402957, -2.997070692e-005, 6.685280620000001e-009, -7.7222622e-013, 3.5993823e-017}, bhigh = {-65588.1722, -8.208878059}, R = 97.32985218942183);
          constant IdealGases.Common.DataRecord PFCL2(name = "PFCL2", MM = 0.1208781642, Hf = -4235052.239484623, H0 = 122997.8970842114, Tlimit = 1000, alow = {82665.60560000001, -1512.138928, 13.03404207, -0.002903247162, 7.83665297e-007, 6.45834066e-010, -3.73764182e-013}, blow = {-56442.3787, -41.00905668}, ahigh = {-186458.9813, -114.8004379, 10.08599754, -3.44745652e-005, 7.62797201e-009, -8.75309743e-013, 4.05778835e-017}, bhigh = {-64504.7219, -21.74054563}, R = 68.7839036523025);
          constant IdealGases.Common.DataRecord PFCL4(name = "PFCL4", MM = 0.1917841642, Hf = -3311099.44686455, H0 = 113324.5807372035, Tlimit = 1000, alow = {141207.1573, -2882.47108, 23.87345233, -0.01239510207, 1.141290085e-005, -5.705483820000001e-009, 1.195229703e-012}, blow = {-66134.7865, -98.5969355}, ahigh = {-295437.588, -132.5154294, 16.1000538, -4.03535917e-005, 8.971130739999999e-009, -1.03328881e-012, 4.80446835e-017}, bhigh = {-81357.19399999999, -50.62403920000001}, R = 43.35327702723852);
          constant IdealGases.Common.DataRecord PF2(name = "PF2", MM = 0.06897056739999999, Hf = -7439458.284056397, H0 = 160669.2161271099, Tlimit = 1000, alow = {55247.6572, -628.05178, 5.13743792, 0.009120673350000001, -1.46877376e-005, 1.08126845e-008, -3.06494449e-012}, blow = {-59775.4147, -1.689589472}, ahigh = {-170846.712, -169.205154, 7.1254916, -4.990852849999999e-005, 1.09733523e-008, -1.25287026e-012, 5.78479114e-017}, bhigh = {-63382.9584, -10.41715673}, R = 120.5510163745587);
          constant IdealGases.Common.DataRecord PF2minus(name = "PF2minus", MM = 0.068971116, Hf = -10284565.30991901, H0 = 160133.0185812856, Tlimit = 1000, alow = {140479.3057, -1699.945753, 10.36823971, -0.00359958769, 1.9619258e-006, -3.82932987e-010, -3.37554952e-014}, blow = {-78104.34510000001, -32.17375182}, ahigh = {-165397.6056, -164.0604822, 7.12409152, -5.0154314e-005, 1.117302152e-008, -1.289308601e-012, 6.00470667e-017}, bhigh = {-87002.23319999999, -11.09074866}, R = 120.5500575052316);
          constant IdealGases.Common.DataRecord PF2CL(name = "PF2CL", MM = 0.1044235674, Hf = -7039375.308681516, H0 = 132884.9257452202, Tlimit = 1000, alow = {83975.9016, -1318.075883, 10.17203515, 0.005571231219999999, -1.100920478e-005, 8.72638318e-009, -2.567714099e-012}, blow = {-83816.56169999999, -27.66012951}, ahigh = {-240928.1185, -203.348467, 10.15150769, -6.04836731e-005, 1.333942001e-008, -1.526772528e-012, 7.063339179999999e-017}, bhigh = {-91011.1059, -24.17825549}, R = 79.62256229143155);
          constant IdealGases.Common.DataRecord PF2CL3(name = "PF2CL3", MM = 0.1753295674, Hf = -5011961.045881188, H0 = 115013.3334555869, Tlimit = 1000, alow = {184209.9543, -3334.40239, 23.88510873, -0.01030943175, 7.356504219999999e-006, -2.498621785e-009, 2.513997646e-013}, blow = {-92795.49890000001, -102.7991502}, ahigh = {-365527.254, -202.969416, 16.15238015, -6.11879657e-005, 1.355610375e-008, -1.55712964e-012, 7.22435997e-017}, bhigh = {-110494.1533, -53.87308191}, R = 47.42196152820714);
          constant IdealGases.Common.DataRecord PF3(name = "PF3", MM = 0.0879689706, Hf = -10883383.00959952, H0 = 147057.4557342837, Tlimit = 1000, alow = {84809.84480000001, -1102.340691, 7.16071511, 0.01438278338, -2.318137222e-005, 1.702244315e-008, -4.81099195e-012}, blow = {-111183.7495, -14.57652362}, ahigh = {-295704.0675, -299.7689931, 10.22297023, -8.890263450000001e-005, 1.958901359e-008, -2.240491075e-012, 1.035952523e-016}, bhigh = {-117374.3464, -27.77419177}, R = 94.51596333673592);
          constant IdealGases.Common.DataRecord PF3CL2(name = "PF3CL2", MM = 0.1588749706, Hf = -7062301.328916815, H0 = 119887.8837117469, Tlimit = 1000, alow = {182249.7206, -3089.610245, 20.67710388, -0.001168732391, -5.02054074e-006, 5.80457335e-009, -1.965367906e-012}, blow = {-122812.0667, -86.27977355}, ahigh = {-422383.8050000001, -320.796522, 16.24013, -9.62365531e-005, 2.129279712e-008, -2.443515401e-012, 1.13290435e-016}, bhigh = {-139264.2151, -55.31285745}, R = 52.33342903919946);
          constant IdealGases.Common.DataRecord PF4CL(name = "PF4CL", MM = 0.1424203738, Hf = -9583666.708505718, H0 = 128637.5292465354, Tlimit = 1000, alow = {180424.0323, -2741.096618, 16.80192838, 0.00991931525, -2.025636171e-005, 1.617979147e-008, -4.77261152e-012}, blow = {-153238.2587, -66.79119965}, ahigh = {-478114.004, -426.481283, 16.31812121, -0.0001271374903, 2.806715254e-008, -3.2151663e-012, 1.488512118e-016}, bhigh = {-168048.8362, -57.35844805}, R = 58.37979341127105);
          constant IdealGases.Common.DataRecord PF5(name = "PF5", MM = 0.125965777, Hf = -12648673.61553289, H0 = 131286.5239580112, Tlimit = 1000, alow = {185577.8473, -2436.979828, 12.01012406, 0.02339343049, -3.81394441e-005, 2.796136412e-008, -7.87070488e-012}, blow = {-181456.6568, -44.8648874}, ahigh = {-566872.784, -667.791791, 16.49789569, -0.000199009944, 4.3950071e-008, -5.03684954e-012, 2.332945909e-016}, bhigh = {-194432.1623, -62.61721040000001}, R = 66.00580092480199);
          constant IdealGases.Common.DataRecord PH(name = "PH", MM = 0.031981701, Hf = 7215129.176525038, H0 = 270407.881056733, Tlimit = 1000, alow = {22736.33198, -397.267406, 6.23369766, -0.0091817846, 1.523328123e-005, -1.085888585e-008, 2.929760547e-012}, blow = {28527.68404, -10.95191197}, ahigh = {781473.0649999999, -3038.451204, 7.46748102, -0.001837522255, 7.1659477e-007, -1.142128853e-010, 6.175410560000001e-015}, bhigh = {45362.6018, -24.6729814}, R = 259.9759156024878);
          constant IdealGases.Common.DataRecord PH2(name = "PH2", MM = 0.032989641, Hf = 3623970.021377317, H0 = 302566.8269624395, Tlimit = 1000, alow = {15552.68372, -184.1602025, 4.89589604, -0.0034954366, 1.053418945e-005, -8.377562920000002e-009, 2.27076615e-012}, blow = {14098.39468, -2.210564792}, ahigh = {1127884.913, -4715.238249999999, 10.214983, -0.00116757382, 2.150542671e-007, -1.624213739e-011, 3.76622524e-016}, bhigh = {41830.7463, -42.3162325}, R = 252.0328123607044);
          constant IdealGases.Common.DataRecord PH2minus(name = "PH2minus", MM = 0.0329901896, Hf = -280853.735984591, H0 = 301910.4200601502, Tlimit = 1000, alow = {69506.84490000001, -771.916291, 7.38353762, -0.00852294721, 1.474476507e-005, -9.84716081e-009, 2.413107871e-012}, blow = {1582.29585, -17.61306419}, ahigh = {1382525.815, -5213.40067, 10.21694832, -0.001116316349, 2.215004171e-007, -2.338005187e-011, 1.015580932e-015}, bhigh = {29906.83733, -43.78711584999999}, R = 252.0286212601821);
          constant IdealGases.Common.DataRecord PH3(name = "PH3", MM = 0.033997581, Hf = 159981.9704819587, H0 = 298157.1541810578, Tlimit = 1000, alow = {-6384.32534, 405.756741, -0.1565680086, 0.01338380613, -8.27539143e-006, 3.024360831e-009, -6.421764630000001e-013}, blow = {-2159.842124, 23.85561888}, ahigh = {1334801.106, -6725.46352, 14.45857073, -0.001639736883, 3.40921857e-007, -3.73627208e-011, 1.672947506e-015}, bhigh = {39103.2571, -71.9878119}, R = 244.5606938917213);
          constant IdealGases.Common.DataRecord PN(name = "PN", MM = 0.044980461, Hf = 3812484.380718108, H0 = 193464.1132290752, Tlimit = 1000, alow = {-51032.0384, 820.2926679999999, -1.392772765, 0.01287989789, -1.401425371e-005, 7.775633459999999e-009, -1.75153933e-012}, blow = {15732.2652, 32.51070633}, ahigh = {-249562.5593, 176.043883, 4.14412196, 0.0002478018097, -5.674896300000001e-008, 4.263645119999999e-012, 3.063920924e-016}, bhigh = {17703.17267, 1.325517397}, R = 184.8463047099495);
          constant IdealGases.Common.DataRecord PO(name = "PO", MM = 0.046973161, Hf = -593055.4045532512, H0 = 199903.7918695742, Tlimit = 1000, alow = {-68457.54059999999, 1141.295708, -2.77955606, 0.01678458047, -1.974879516e-005, 1.19260232e-008, -2.927460912e-012}, blow = {-9847.74504, 41.84328297}, ahigh = {-336666.744, 622.9355840000001, 3.56560546, 0.0006516620719999999, -2.061770841e-007, 3.18441323e-011, -1.573691908e-015}, bhigh = {-8939.79039, 6.954859188}, R = 177.0047368113038);
          constant IdealGases.Common.DataRecord POminus(name = "POminus", MM = 0.0469737096, Hf = -2981818.876829775, H0 = 186872.743812424, Tlimit = 1000, alow = {54343.46320000001, -412.362355, 3.74648388, 0.00385863647, -5.96334304e-006, 4.278286360000001e-009, -1.139722989e-012}, blow = {-15558.10937, 3.404436468}, ahigh = {-848.0294779999999, 347.054487, 2.877890167, 0.001622182445, -4.87019099e-007, 6.64575227e-011, -3.39570376e-015}, bhigh = {-19872.0954, 10.75444355}, R = 177.0026695954198);
          constant IdealGases.Common.DataRecord POCL3(name = "POCL3", MM = 0.153332161, Hf = -3706984.864056015, H0 = 115539.7268548246, Tlimit = 1000, alow = {47937.8816, -1270.38828, 13.6771646, 0.0010826721, -1.73610406e-006, 8.460890640000001e-010, -1.17466133e-013}, blow = {-65075.7188, -43.14705941}, ahigh = {-218593.797, -508.026968, 13.3778465, -0.000151073105, 3.34098678e-008, -3.83532976e-012, 1.77932988e-016}, bhigh = {-70093.15090000001, -39.84746301}, R = 54.2252319785671);
          constant IdealGases.Common.DataRecord POFCL2(name = "POFCL2", MM = 0.1368775642, Hf = -5799995.906122357, H0 = 120210.9936436172, Tlimit = 1000, alow = {46940.6156, -1069.2426, 10.71284279, 0.009491183299999999, -1.306412424e-005, 8.426906020000001e-009, -2.140294969e-012}, blow = {-92748.94989999999, -28.15077719}, ahigh = {-270764.7503, -655.650227, 13.48668421, -0.0001942765035, 4.29118449e-008, -4.9218448e-012, 2.282079808e-016}, bhigh = {-96543.92140000001, -41.64963494}, R = 60.74386294492521);
          constant IdealGases.Common.DataRecord POF2CL(name = "POF2CL", MM = 0.1204229674, Hf = -8491794.06618741, H0 = 123700.323298959, Tlimit = 1000, alow = {52264.3816, -969.1037700000001, 7.96363598, 0.01755599796, -2.398693997e-005, 1.572376747e-008, -4.079746e-012}, blow = {-120265.6893, -15.93967075}, ahigh = {-334708.659, -817.6104810000001, 13.6066432, -0.0002420426112, 5.34371272e-008, -6.126461349999999e-012, 2.839570116e-016}, bhigh = {-123340.1194, -44.96103437000001}, R = 69.04390565615641);
          constant IdealGases.Common.DataRecord POF3(name = "POF3", MM = 0.1039683706, Hf = -12042123.89570718, H0 = 136256.3337123223, Tlimit = 1000, alow = {29660.3071, -427.670575, 3.6691461, 0.0283949378, -3.768318e-005, 2.44910454e-008, -6.34197287e-012}, blow = {-150112.453, 5.160417996}, ahigh = {-369060.242, -1005.55254, 13.7452994, -0.000297103711, 6.55452664e-008, -7.50998733e-012, 3.47896991e-016}, bhigh = {-149976.173, -48.82214181000001}, R = 79.97116769280214);
          constant IdealGases.Common.DataRecord PO2(name = "PO2", MM = 0.062972561, Hf = -4470633.439856448, H0 = 166943.5994511959, Tlimit = 1000, alow = {-63726.9822, 1036.741044, -2.877797967, 0.02278134083, -2.567920328e-005, 1.465060412e-008, -3.3879967e-012}, blow = {-39935.4472, 44.2530938}, ahigh = {492621.0990000001, -2605.465745, 9.51760561, -0.001180371565, 2.532912819e-007, -1.789964539e-011, 1.800381054e-016}, bhigh = {-20288.84763, -29.69743125}, R = 132.0332517522989);
          constant IdealGases.Common.DataRecord PO2minus(name = "PO2minus", MM = 0.06297310960000001, Hf = -9490141.979585521, H0 = 168544.1463414727, Tlimit = 1000, alow = {27024.23135, 29.39956284, 1.162686114, 0.01599897972, -1.979229263e-005, 1.208847527e-008, -2.95664048e-012}, blow = {-72859.4627, 19.63118182}, ahigh = {1628679.152, -5989.19998, 13.69318836, -0.00362384379, 9.488198340000001e-007, -1.040308759e-010, 4.01903366e-015}, bhigh = {-36859.595, -59.5679622}, R = 132.0321015241718);
          constant IdealGases.Common.DataRecord PS(name = "PS", MM = 0.063038761, Hf = 2386330.467377048, H0 = 152542.9092745018, Tlimit = 1000, alow = {-768.9786800000001, -46.3782407, 4.10464917, 0.001555262273, -2.315757288e-006, 1.661425178e-009, -4.6312384e-013}, blow = {17078.75808, 4.230652171}, ahigh = {-270272.9081, 888.354822, 3.16919012, 0.001022480817, -3.80374048e-007, 7.019861879999999e-011, -4.26912231e-015}, bhigh = {11215.10462, 11.47334049}, R = 131.8945973573307);
          constant IdealGases.Common.DataRecord P2(name = "P2", MM = 0.061947522, Hf = 2324548.187738647, H0 = 143736.2902102848, Tlimit = 1000, alow = {30539.2251, -324.617759, 4.02246381, 0.00323209479, -5.511052450000001e-006, 4.19557293e-009, -1.21503218e-012}, blow = {17969.1087, 1.645350331}, ahigh = {-780693.649, 2307.91087, 1.41174313, 0.00210823742, -7.36085662e-007, 1.25936012e-010, -7.07975249e-015}, bhigh = {1329.82474, 21.69741365}, R = 134.2179918028037);
          constant IdealGases.Common.DataRecord P2O3(name = "P2O3", MM = 0.109945722, Hf = -6227120.633215725, H0 = 146389.3338205556, Tlimit = 1000, alow = {-66457.5389, 758.090055, 0.988843677, 0.0285987873, -3.23114211e-005, 1.83452517e-008, -4.2124893e-012}, blow = {-88200.3674, 26.89914458}, ahigh = {-217534.411, -1369.902, 14.0114296, -0.000402385696, 8.86834578e-008, -1.01565728e-011, 4.70435606e-016}, bhigh = {-79227.56999999999, -47.39487232000001}, R = 75.62342443846974);
          constant IdealGases.Common.DataRecord P2O4(name = "P2O4", MM = 0.125945122, Hf = -7413980.090471467, H0 = 137564.5179810934, Tlimit = 1000, alow = {-43760.9126, 490.510283, 0.964364937, 0.0388607358, -4.67550922e-005, 2.81262563e-008, -6.8244061e-012}, blow = {-116899.779, 23.79582767}, ahigh = {-367011.486, -1638.42373, 17.2118861, -0.000482658015, 1.0644619e-007, -1.21961007e-011, 5.65065458e-016}, bhigh = {-109043.593, -67.14276593}, R = 66.01662587614946);
          constant IdealGases.Common.DataRecord P2O5(name = "P2O5", MM = 0.141944522, Hf = -7921195.817616688, H0 = 155341.7116019454, Tlimit = 1000, alow = {-29991.5922, -45.9450659, 6.98748683, 0.0288330297, -3.10450953e-005, 1.64386868e-008, -3.4679423e-012}, blow = {-138190.14, -3.400572711}, ahigh = {-324708.016, -2016.97508, 20.4869934, -0.000591064115, 1.30199302e-007, -1.49068294e-011, 6.90357341e-016}, bhigh = {-130629.016, -80.31642952}, R = 58.57550459044837);
          constant IdealGases.Common.DataRecord P3(name = "P3", MM = 0.09292128300000001, Hf = 2259977.404745907, H0 = 129201.8535732013, Tlimit = 1000, alow = {46933.5895, -864.358932, 8.734247529999999, 1.09453456e-005, -1.99857332e-006, 2.09675051e-009, -6.92288971e-013}, blow = {27748.4734, -20.63587261}, ahigh = {-125351.306, -83.0673462, 7.56195677, -2.47440395e-005, 5.45727693e-009, -6.24466057e-013, 2.88772422e-017}, bhigh = {23087.2228, -12.28310621}, R = 89.47866120186912);
          constant IdealGases.Common.DataRecord P3O6(name = "P3O6", MM = 0.188917683, Hf = -8340569.310285264, H0 = 124071.403098883, Tlimit = 1000, alow = {201449.3813, -3053.488073, 16.02114098, 0.043168899, -6.37654426e-005, 4.42286792e-008, -1.200020334e-011}, blow = {-177650.3811, -65.5626789}, ahigh = {-878796.8100000001, -1651.988071, 27.72258397, -0.00048593267, 1.067867708e-007, -1.218015776e-011, 5.61491196e-016}, bhigh = {-190916.5548, -122.0665277}, R = 44.01108391743297);
          constant IdealGases.Common.DataRecord P4(name = "P4", MM = 0.123895044, Hf = 475402.3897840498, H0 = 113875.8060411198, Tlimit = 1000, alow = {120514.185, -2345.71179, 17.7394655, -0.0145631497, 1.58759409e-005, -9.314710310000001e-009, 2.27149167e-012}, blow = {16088.4648, -70.88590647000001}, ahigh = {-185870.586, -62.8454689, 10.0484488, -1.98766885e-005, 4.482408440000001e-009, -5.22610367e-013, 2.45579129e-017}, bhigh = {3848.21092, -24.77297797}, R = 67.10899590140184);
          constant IdealGases.Common.DataRecord P4O6(name = "P4O6", MM = 0.219891444, Hf = -7303603.863731961, H0 = 112771.659273837, Tlimit = 1000, alow = {376089.475, -5685.83262, 29.1422545, 0.0225153212, -4.51026963e-005, 3.58831269e-008, -1.05757206e-011}, blow = {-168856.248, -145.1359851}, ahigh = {-1008997.24, -887.275399, 28.6606483, -0.000263601894, 5.81094072e-008, -6.64808696e-012, 3.07439193e-016}, bhigh = {-199713.89, -128.1853821}, R = 37.81171221923487);
          constant IdealGases.Common.DataRecord P4O7(name = "P4O7", MM = 0.235890844, Hf = -8412569.183906095, H0 = 111798.8157268198, Tlimit = 1000, alow = {321858.696, -4871.44473, 24.54417068, 0.0401550955, -6.52419715e-005, 4.75655418e-008, -1.332975491e-011}, blow = {-218451.7427, -118.1504792}, ahigh = {-1102947.375, -1511.077128, 32.109795, -0.000437180376, 9.516639610000001e-008, -1.075204176e-011, 4.91102568e-016}, bhigh = {-242909.654, -147.3140859}, R = 35.24711624669926);
          constant IdealGases.Common.DataRecord P4O8(name = "P4O8", MM = 0.251890244, Hf = -9139750.874988236, H0 = 110946.4803249784, Tlimit = 1000, alow = {271932.6942, -4118.600390000001, 20.28714235, 0.0568541699, -8.40073963e-005, 5.823389060000001e-008, -1.57864716e-011}, blow = {-260453.9703, -94.0716207}, ahigh = {-1173177.311, -2200.632995, 35.6301013, -0.000648984974, 1.429623989e-007, -1.635821725e-011, 7.571070759999999e-016}, bhigh = {-278384.3144, -167.963696}, R = 33.00831293807473);
          constant IdealGases.Common.DataRecord P4O9(name = "P4O9", MM = 0.267889644, Hf = -9757671.375307065, H0 = 110199.6873010888, Tlimit = 1000, alow = {219991.6395, -3336.46494, 15.86413106, 0.07402452330000001, -0.0001034835052, 6.94443832e-008, -1.84073345e-011}, blow = {-301874.2692, -69.9796767}, ahigh = {-1233696.783, -2917.032808, 39.1775711, -0.000873640298, 1.937914895e-007, -2.230510118e-011, 1.037248818e-015}, bhigh = {-312963.7516, -189.717642}, R = 31.03692952012733);
          constant IdealGases.Common.DataRecord P4O10(name = "P4O10", MM = 0.283889044, Hf = -10237180.71698463, H0 = 109537.7002291078, Tlimit = 1000, alow = {167014.2268, -2540.2433, 11.36853403, 0.09137812159999999, -0.0001231990626, 8.08103223e-008, -2.106787011e-011}, blow = {-341015.191, -46.4343134}, ahigh = {-1337201.132, -3516.38233, 42.6040331, -0.001037406404, 2.287613089e-007, -2.620219946e-011, 1.213529528e-015}, bhigh = {-345951.505, -211.5498906}, R = 29.28775229522418);
          constant IdealGases.Common.DataRecord Pb(name = "Pb", MM = 0.2072, Hf = 942084.942084942, H0 = 29910.36679536679, Tlimit = 1000, alow = {1213.382285, -19.06116019, 2.619299546, -0.000382951961, 6.68818045e-007, -6.06123108e-010, 2.240022429e-013}, blow = {22820.96238, 6.2013692}, ahigh = {-9084313.07, 26726.7318, -26.26244039, 0.01358282305, -2.685523566e-006, 2.3524328e-010, -7.324114532e-015}, bhigh = {-148165.0666, 215.4011624}, R = 40.12776061776062);
          constant IdealGases.Common.DataRecord Pbplus(name = "Pbplus", MM = 0.2071994514, Hf = 4425670.636693588, H0 = 29910.44598875806, Tlimit = 1000, alow = {9.48668904, -0.1134955793, 2.500549799, -1.382464681e-006, 1.906022991e-009, -1.368396828e-012, 4.003528117e-016}, blow = {109543.8901, 7.5388559}, ahigh = {1320690.183, -4096.04801, 7.38910151, -0.002807751909, 7.83099165e-007, -9.31060091e-011, 4.016371727e-015}, bhigh = {135434.7306, -27.22020908}, R = 40.12786686364751);
          constant IdealGases.Common.DataRecord Pbminus(name = "Pbminus", MM = 0.2072005486, Hf = 742671.4313245792, H0 = 29910.28760239489, Tlimit = 1000, alow = {0, 0, 2.5, 0, 0, 0, 0}, blow = {17762.2614, 8.2351321}, ahigh = {0, 0, 2.5, 0, 0, 0, 0}, bhigh = {17762.2614, 8.2351321}, R = 40.12765437243635);
          constant IdealGases.Common.DataRecord PbBr(name = "PbBr", MM = 0.287104, Hf = 225776.4503455194, H0 = 35339.53549933125, Tlimit = 1000, alow = {-2393.487988, -45.7659121, 4.69260828, -0.000384529063, 5.8882135e-007, -4.08479289e-010, 1.202515201e-013}, blow = {6662.43477, 5.99170551}, ahigh = {-2581831.67, 7143.49076, -2.65819364, 0.00310772226, -4.63138154e-007, 1.095529211e-011, 1.508555088e-015}, bhigh = {-39704.4098, 59.5923865}, R = 28.95979157378512);
          constant IdealGases.Common.DataRecord PbBr2(name = "PbBr2", MM = 0.367008, Hf = -283122.0654590636, H0 = 40930.81622198971, Tlimit = 1000, alow = {-6172.76636, -67.8320763, 7.27315614, -0.000599059612, 7.35863461e-007, -4.74360481e-010, 1.246922291e-013}, blow = {-14278.90796, -0.698799215}, ahigh = {-13204.21479, -1.143074947, 7.00097254, -4.28447161e-007, 1.018044861e-010, -1.234016791e-014, 5.97133765e-019}, bhigh = {-14622.1375, 0.8868582650000001}, R = 22.65474322085622);
          constant IdealGases.Common.DataRecord PbBr3(name = "PbBr3", MM = 0.446912, Hf = -232732.5222862667, H0 = 44681.37575182587, Tlimit = 1000, alow = {-11644.2425, -155.5151665, 10.62317321, -0.001361947117, 1.668738791e-006, -1.073686019e-009, 2.81825034e-013}, blow = {-14782.08304, -14.43758579}, ahigh = {-27879.30766, -2.633265111, 10.00223199, -9.80776141e-007, 2.326232632e-010, -2.81599446e-014, 1.361283735e-018}, bhigh = {-15569.81182, -10.81765474}, R = 18.60427108692539);
          constant IdealGases.Common.DataRecord PbBr4(name = "PbBr4", MM = 0.5268160000000001, Hf = -346298.3489491587, H0 = 49109.15575836725, Tlimit = 1000, alow = {-11764.58199, -256.7166983, 14.02223292, -0.002224117068, 2.716201994e-006, -1.743340164e-009, 4.567359749999999e-013}, blow = {-24621.25551, -28.8202075}, ahigh = {-38802.1603, -4.38293377, 13.00369823, -1.620061335e-006, 3.83413088e-010, -4.63396718e-014, 2.237427066e-018}, bhigh = {-25923.27065, -22.87710636}, R = 15.78249711474215);
          constant IdealGases.Common.DataRecord PbCL(name = "PbCL", MM = 0.242653, Hf = 36344.56610880558, H0 = 40333.80176630827, Tlimit = 1000, alow = {-2902.700749, -82.93459710000001, 4.71482448, -0.0001931473228, 9.10557902e-008, 5.419385229999999e-011, -3.2849697e-014}, blow = {125.4460217, 4.323194455}, ahigh = {-385428.553, 692.21975, 4.50997124, -0.0006853421299999999, 5.0873955e-007, -9.772753809999999e-011, 5.807471579999999e-015}, bhigh = {-5269.68472, 6.646491015}, R = 34.26486381787986);
          constant IdealGases.Common.DataRecord PbCL2(name = "PbCL2", MM = 0.278106, Hf = -631225.0400926265, H0 = 50349.61129928876, Tlimit = 1000, alow = {2830.580143, -313.4256659, 8.21282729, -0.002585096173, 3.109456077e-006, -1.972978208e-009, 5.12354248e-013}, blow = {-21675.69042, -9.219412244000001}, ahigh = {-31491.92744, -5.56547758, 7.00459954, -1.986025455e-006, 4.65171959e-010, -5.57913701e-014, 2.67814357e-018}, bhigh = {-23274.39309, -2.140739285}, R = 29.896773172819);
          constant IdealGases.Common.DataRecord PbCL3(name = "PbCL3", MM = 0.313559, Hf = -566571.2258299076, H0 = 58220.91855121364, Tlimit = 1000, alow = {862.3588970000001, -525.315275, 12.03336337, -0.00433518498, 5.21569835e-006, -3.31002238e-009, 8.596970609999999e-013}, blow = {-21805.76379, -26.94206962}, ahigh = {-56644.1279, -9.329514789999999, 10.00771099, -3.32972757e-006, 7.79933081e-010, -9.3546061e-014, 4.4905897e-018}, bhigh = {-24485.08643, -15.0749087}, R = 26.51645144932851);
          constant IdealGases.Common.DataRecord PbCL4(name = "PbCL4", MM = 0.349012, Hf = -938163.6218811962, H0 = 67188.05657112076, Tlimit = 1000, alow = {17394.05796, -890.213932, 16.37565058, -0.00708899226, 8.43256734e-006, -5.30530939e-009, 1.368671294e-012}, blow = {-38882.1562, -48.50133784}, ahigh = {-82666.7956, -16.39486405, 13.01336711, -5.7161414e-006, 1.32936839e-009, -1.585922492e-013, 7.58176361e-018}, bhigh = {-43439.84220000001, -28.74559263}, R = 23.82288288081785);
          constant IdealGases.Common.DataRecord PbF(name = "PbF", MM = 0.2261984032, Hf = -437084.3807972558, H0 = 40973.37058478404, Tlimit = 1000, alow = {26848.64255, -475.172564, 6.01964437, -0.002718531008, 2.942882954e-006, -1.687543009e-009, 4.08612208e-013}, blow = {-10790.38653, -4.98304884}, ahigh = {-473420.275, 1027.785283, 3.88865873, -0.0001649813448, 2.93116326e-007, -5.76645965e-011, 3.24748619e-015}, bhigh = {-20261.65076, 9.319483890000001}, R = 36.75743012495325);
          constant IdealGases.Common.DataRecord PbF2(name = "PbF2", MM = 0.2451968064, Hf = -1808454.997886954, H0 = 51275.5862712574, Tlimit = 1000, alow = {59205.5228, -1052.529292, 10.28969284, -0.00587634627, 6.10233313e-006, -3.42479264e-009, 8.024025419999999e-013}, blow = {-49990.50000000001, -25.25115478}, ahigh = {-85058.99769999999, -30.74244334, 7.02334604, -9.455183590000001e-006, 2.108489027e-009, -2.434212002e-013, 1.133872429e-017}, bhigh = {-55522.3677, -5.49607322}, R = 33.90938129282258);
          constant IdealGases.Common.DataRecord PbF3(name = "PbF3", MM = 0.2641952096, Hf = -1853071.498689279, H0 = 58801.39546633173, Tlimit = 1000, alow = {85530.9087, -1731.036694, 15.5659615, -0.01022225919, 1.089984998e-005, -6.269998819999999e-009, 1.502616508e-012}, blow = {-53003.3567, -53.3573348}, ahigh = {-145970.4258, -48.2039643, 10.03685845, -1.501095764e-005, 3.36247499e-009, -3.89602071e-013, 1.820152103e-017}, bhigh = {-62068.6959, -20.06978661}, R = 31.47094155336267);
          constant IdealGases.Common.DataRecord PbF4(name = "PbF4", MM = 0.2831936128, Hf = -2824658.243845816, H0 = 69303.21205323456, Tlimit = 1000, alow = {130996.3563, -2307.299499, 19.4520144, -0.01024522729, 9.39587496e-006, -4.62689415e-009, 9.450881430000001e-013}, blow = {-88041.47459999999, -75.24645820000001}, ahigh = {-213261.6322, -85.7862195, 13.06428551, -2.575563598e-005, 5.69332459e-009, -6.526219550000001e-013, 3.02234196e-017}, bhigh = {-100290.8979, -35.9024028}, R = 29.35967346788974);
          constant IdealGases.Common.DataRecord PbI(name = "PbI", MM = 0.33410447, Hf = 325958.8954317193, H0 = 30945.78171911319, Tlimit = 1000, alow = {-2717.33406, -15.42541996, 4.5653279, -0.0001061215818, 2.414841342e-007, -1.876070301e-010, 6.460614269999999e-014}, blow = {11818.64978, 7.66980267}, ahigh = {-3901070.25, 11479.76718, -8.17610902, 0.0065234547, -1.522611682e-006, 1.626692202e-010, -6.55332888e-015}, bhigh = {-61544.3369, 99.38404970000001}, R = 24.88584483769403);
          constant IdealGases.Common.DataRecord PbI2(name = "PbI2", MM = 0.46100894, Hf = -22239.54702483644, H0 = 33073.91392453257, Tlimit = 1000, alow = {-5904.98672, -48.3330703, 7.19543803, -0.000429860009, 5.29141207e-007, -3.41640282e-010, 8.99134192e-014}, blow = {-3107.780155, 1.324773678}, ahigh = {-10883.39477, -0.816441837, 7.0006973, -3.079876763e-007, 7.331718140000001e-011, -8.89919388e-015, 4.3107125e-019}, bhigh = {-3352.09638, 2.458605463}, R = 18.03538126614204);
          constant IdealGases.Common.DataRecord PbI3(name = "PbI3", MM = 0.58791341, Hf = 37004.44934569531, H0 = 35830.12675284954, Tlimit = 1000, alow = {-10149.96331, -59.380782, 10.24073798, -0.000530469453, 6.538623030000001e-007, -4.22589572e-010, 1.113029818e-013}, blow = {-113.8482403, -8.975540499999999}, ahigh = {-16242.29573, -1.002945584, 10.00085839, -3.79667974e-007, 9.04675172e-011, -1.098840054e-014, 5.325388290000001e-019}, bhigh = {-413.825893, -7.57941711}, R = 14.14234113149418);
          constant IdealGases.Common.DataRecord PbI4(name = "PbI4", MM = 0.7148178799999999, Hf = -57674.07356962029, H0 = 38500.07361315585, Tlimit = 1000, alow = {-12170.96972, -72.2631744, 13.29294204, -0.00064546609, 7.95576185e-007, -5.14163283e-010, 1.354188031e-013}, blow = {-8528.148679999998, -20.10457498}, ahigh = {-19584.77744, -1.223385179, 13.00104732, -4.63329569e-007, 1.104213978e-010, -1.341397771e-014, 6.50170611e-019}, bhigh = {-8893.193200000002, -18.40570345}, R = 11.63159488959622);
          constant IdealGases.Common.DataRecord PbO(name = "PbO", MM = 0.2231994, Hf = 305274.6019926577, H0 = 40152.92155803287, Tlimit = 1000, alow = {34240.25, -419.805011, 4.72030641, 0.001451061751, -3.20603546e-006, 2.713956273e-009, -8.333275609999999e-013}, blow = {9253.186469999999, 0.448289121}, ahigh = {242699.685, -110.7731354, 3.18068449, 0.001991815065, -1.079782025e-006, 2.540990941e-010, -1.834652493e-014}, bhigh = {8339.391539999999, 10.50920926}, R = 37.25131877594653);
          constant IdealGases.Common.DataRecord PbO2(name = "PbO2", MM = 0.2391988, Hf = 569203.2485112802, H0 = 51216.53620335888, Tlimit = 1000, alow = {70710.58289999999, -1076.757462, 9.401711730000001, -0.001003861584, -1.233815617e-006, 1.858386979e-009, -6.83462121e-013}, blow = {19996.45981, -25.03976748}, ahigh = {-130779.667, -73.1436145, 7.55408341, -2.143421561e-005, 4.696463999999999e-009, -5.34497218e-013, 2.460760535e-017}, bhigh = {14133.86101, -12.52684909}, R = 34.75967270738816);
          constant IdealGases.Common.DataRecord PbS(name = "PbS", MM = 0.239265, Hf = 534743.1216433662, H0 = 39412.83932041879, Tlimit = 1000, alow = {15298.40503, -343.974635, 5.68063064, -0.002232931917, 2.504608209e-006, -1.470427328e-009, 3.57752719e-013}, blow = {15785.51202, -2.628900926}, ahigh = {2105441.649, -5520.41358, 9.36924926, -0.001410262019, -1.642683362e-007, 1.451447577e-010, -1.393564925e-014}, bhigh = {50093.3453, -32.17417657}, R = 34.75005537792824);
          constant IdealGases.Common.DataRecord PbS2(name = "PbS2", MM = 0.27133, Hf = 899454.5571812922, H0 = 51675.83754100174, Tlimit = 1000, alow = {24463.7588, -665.485546, 9.920201179999999, -0.00492419438, 5.71690629e-006, -3.52948883e-009, 8.971138350000001e-013}, blow = {30443.1013, -22.95765552}, ahigh = {-54167.003, -13.43600342, 7.51070036, -4.497318629999999e-006, 1.032444494e-009, -1.219580043e-013, 5.78580465e-018}, bhigh = {27012.29533, -8.714618097000001}, R = 30.64339365348469);
          constant IdealGases.Common.DataRecord Rb(name = "Rb", MM = 0.0854678, Hf = 946555.3108890132, H0 = 72511.8465667772, Tlimit = 1000, alow = {13.52856616, -0.2042232679, 2.501213823, -3.6506199e-006, 5.88472267e-009, -4.84227472e-012, 1.596211946e-015}, blow = {8985.56921, 6.20700548}, ahigh = {-1138274.064, 3804.04194, -2.750899258, 0.0038914607, -1.632296823e-006, 3.51189314e-010, -2.521064422e-014}, bhigh = {-14664.54849, 42.53442370000001}, R = 97.28192371864024);
          constant IdealGases.Common.DataRecord Rbplus(name = "Rbplus", MM = 0.0854672514, Hf = 5734700.952369694, H0 = 72512.3120081992, Tlimit = 1000, alow = {0, 0, 2.5, 0, 0, 0, 0}, blow = {58203.2736, 5.52050692}, ahigh = {0, 0, 2.5, 0, 0, 0, 0}, bhigh = {58203.2736, 5.52050692}, R = 97.28254815504691);
          constant IdealGases.Common.DataRecord Rbminus(name = "Rbminus", MM = 0.08546834859999999, Hf = 325483.3918716899, H0 = 72511.3811313303, Tlimit = 1000, alow = {0, 0, 2.5, 0, 0, 0, 0}, blow = {2600.405796, 5.52052617}, ahigh = {0, 0, 2.5, 0, 0, 0, 0}, bhigh = {2600.405796, 5.52052617}, R = 97.28129929024978);
          constant IdealGases.Common.DataRecord RbBO2(name = "RbBO2", MM = 0.1282776, Hf = -5293030.91108658, H0 = 111731.8846002731, Tlimit = 1000, alow = {41986.5775, -679.944833, 8.18515101, 0.002772536733, -6.17577047e-007, -9.42613625e-010, 4.72241424e-013}, blow = {-80203.3959, -12.39198887}, ahigh = {91544.63399999999, -1668.328341, 11.17268757, -0.000450553353, 9.68133059e-008, -1.088037652e-011, 4.96773366e-016}, bhigh = {-75079.0325, -31.31951149}, R = 64.81624227456703);
          constant IdealGases.Common.DataRecord RbBr(name = "RbBr", MM = 0.1653718, Hf = -1158062.474980619, H0 = 62399.53849447124, Tlimit = 1000, alow = {-5703.8597, 20.92281597, 4.37060737, 0.000449460478, -4.835438840000001e-007, 3.5260686e-010, -9.88653832e-014}, blow = {-24491.12515, 6.4274317}, ahigh = {1609092.474, -4466.77459, 9.06409206, -0.001995333358, 4.01663352e-007, -4.1630914e-012, -3.092443341e-015}, bhigh = {4427.46769, -27.57714089}, R = 50.27744754547027);
          constant IdealGases.Common.DataRecord RbCL(name = "RbCL", MM = 0.1209208, Hf = -1846852.476993206, H0 = 83171.14177213515, Tlimit = 1000, alow = {-5740.809579999999, -15.28835285, 4.5152009, 0.0001067270745, -6.38594559e-009, -1.301381692e-011, 1.641459742e-014}, blow = {-28142.43986, 4.190115141}, ahigh = {-1143043.004, 4003.86188, -1.096156782, 0.00403358128, -1.454151464e-006, 2.719263762e-010, -1.81046398e-014}, bhigh = {-52976.86040000001, 43.11317822}, R = 68.75965094508142);
          constant IdealGases.Common.DataRecord RbF(name = "RbF", MM = 0.1044662032, Hf = -3192533.803123803, H0 = 91781.98026057867, Tlimit = 1000, alow = {32873.6019, -608.090746, 7.40662236, -0.00727786472, 1.013930727e-005, -7.141372500000001e-009, 2.014332982e-012}, blow = {-38498.3413, -13.75750778}, ahigh = {-854481.1149999999, 2633.756505, 1.216216506, 0.002136494067, -6.72842608e-007, 1.134905135e-010, -6.825624440000001e-015}, bhigh = {-58101.7911, 25.88118896}, R = 79.59006592861414);
          constant IdealGases.Common.DataRecord RbH(name = "RbH", MM = 0.08647574, Hf = 1379856.431410705, H0 = 101995.1491597528, Tlimit = 1000, alow = {8908.270610000001, 46.498704, 1.833967623, 0.00844427914, -1.167538647e-005, 8.01038942e-009, -2.162578321e-012}, blow = {13282.48009, 12.79544976}, ahigh = {-2937703.53, 9379.81726, -7.54256346, 0.007493226800000001, -2.159228887e-006, 2.871300173e-010, -1.459981256e-014}, bhigh = {-45955.18829999999, 83.65664320000001}, R = 96.14802949358978);
          constant IdealGases.Common.DataRecord RbI(name = "RbI", MM = 0.21237227, Hf = -652066.6469308824, H0 = 49249.01918692116, Tlimit = 1000, alow = {-769.30528, -29.68608844, 4.64874584, -0.0002962415236, 5.97813534e-007, -4.351255250000001e-010, 1.313617336e-013}, blow = {-17866.10717, 5.81240983}, ahigh = {4297424.85, -12969.11161, 19.58730402, -0.008458425589999999, 2.463480035e-006, -3.19157573e-010, 1.458455589e-014}, bhigh = {64329.4804, -100.9499336}, R = 39.15045970926431);
          constant IdealGases.Common.DataRecord RbK(name = "RbK", MM = 0.1245661, Hf = 963450.9710105718, H0 = 86766.2229129755, Tlimit = 1000, alow = {24946.95429, -455.306066, 7.68618995, -0.01108112244, 2.093398586e-005, -1.824445366e-008, 5.61764536e-012}, blow = {15161.3616, -10.63563675}, ahigh = {-4175391.07, 5918.588510000001, 9.20264942, -0.010609336, 4.96482606e-006, -8.716698790000001e-010, 5.22188796e-014}, bhigh = {-32727.6904, -13.72401997}, R = 66.74746981722957);
          constant IdealGases.Common.DataRecord RbLi(name = "RbLi", MM = 0.0924088, Hf = 1776684.947753893, H0 = 111202.8724537057, Tlimit = 1000, alow = {-2263.413044, -28.8849691, 4.55582654, 0.000362566753, -9.598002220000001e-007, 1.631010244e-009, -8.159825980000001e-013}, blow = {18534.5525, 2.831743182}, ahigh = {10855151.94, -38051.0697, 55.1655504, -0.03128845612, 9.26165025e-006, -1.264671408e-009, 6.43166205e-014}, bhigh = {254396.4131, -352.075871}, R = 89.9748941659236);
          constant IdealGases.Common.DataRecord RbNO2(name = "RbNO2", MM = 0.1314733, Hf = -1427132.178168495, H0 = 119950.0050580612, Tlimit = 1000, alow = {-71994.2003, 1265.802595, -2.415715742, 0.0305464142, -3.68400405e-005, 2.247716481e-008, -5.54752743e-012}, blow = {-30373.81667, 48.14256904}, ahigh = {-164499.4453, -849.944344, 10.63080575, -0.0002517893988, 5.56142012e-008, -6.378805599999999e-012, 2.957679896e-016}, bhigh = {-21285.90982, -25.68367325}, R = 63.24076447461196);
          constant IdealGases.Common.DataRecord RbNO3(name = "RbNO3", MM = 0.1474727, Hf = -2135801.968771169, H0 = 110986.7589052075, Tlimit = 1000, alow = {-26646.69172, 880.763479, -3.159459457, 0.0427122902, -5.35102921e-005, 3.36096975e-008, -8.503290509999999e-012}, blow = {-43535.95460000001, 49.43052899}, ahigh = {-314741.3867, -1371.457846, 14.01319759, -0.000403027502, 8.878332289999999e-008, -1.016221214e-011, 4.704254090000001e-016}, bhigh = {-35043.3994, -46.19379241}, R = 56.37973672415301);
          constant IdealGases.Common.DataRecord RbNa(name = "RbNa", MM = 0.10845757, Hf = 1212182.127997151, H0 = 98334.56530512347, Tlimit = 1000, alow = {29744.55567, -508.279059, 7.73403352, -0.0101300684, 1.745455325e-005, -1.39872547e-008, 3.96810677e-012}, blow = {16823.79436, -12.4008529}, ahigh = {4297114.27, -18310.13588, 34.543942, -0.02230134993, 7.33987453e-006, -1.064784996e-009, 5.61759846e-014}, bhigh = {124261.0951, -199.366198}, R = 76.6610574070579);
          constant IdealGases.Common.DataRecord RbO(name = "RbO", MM = 0.1014672, Hf = 517303.3650283047, H0 = 101876.5078764369, Tlimit = 1000, alow = {190008.1316, -3428.22899, 24.80251273, -0.0505165458, 6.476338760000001e-005, -4.198012420000001e-008, 1.093889043e-011}, blow = {20838.9845, -109.755915}, ahigh = {617075.688, -2175.08962, 8.29041221, -0.002996487663, 1.233144275e-006, -2.115810631e-010, 1.24260743e-014}, bhigh = {18281.64827, -21.17386042}, R = 81.94246022359934);
          constant IdealGases.Common.DataRecord RbOH(name = "RbOH", MM = 0.10247514, Hf = -2322514.514251945, H0 = 114771.2703783572, Tlimit = 1000, alow = {12138.32721, -544.141159, 8.459814639999999, -0.00356152656, 2.99287239e-006, -7.2802674e-010, -4.55384718e-014}, blow = {-27872.62486, -19.13421991}, ahigh = {895858.313, -2332.339, 7.97119248, 0.0001044439355, -6.32825775e-008, 1.029358824e-011, -5.74327779e-016}, bhigh = {-15155.68947, -19.50051629}, R = 81.13647856445964);
          constant IdealGases.Common.DataRecord Rb2Br2(name = "Rb2Br2", MM = 0.3307436, Hf = -1668366.169443641, H0 = 65599.35248936033, Tlimit = 1000, alow = {-8005.5254, -21.11968088, 10.08629404, -0.000191193609, 2.36605949e-007, -1.53372238e-010, 4.04872506e-014}, blow = {-69273.136, -9.622434630000001}, ahigh = {-10145.35336, -0.360155032, 10.00031058, -1.380745805e-007, 3.30206795e-011, -4.02158182e-015, 1.953006729e-019}, bhigh = {-69379.6088, -9.12256236}, R = 25.13872377273513);
          constant IdealGases.Common.DataRecord Rb2CL2(name = "Rb2CL2", MM = 0.2418416, Hf = -2556936.176406375, H0 = 85947.05790897844, Tlimit = 1000, alow = {-11250.39522, -67.56953990000001, 10.27381915, -0.000603184899, 7.43332478e-007, -4.803372160000001e-010, 1.264973638e-013}, blow = {-77067.575, -13.63652575}, ahigh = {-18188.75133, -1.13790358, 10.00097318, -4.3021564e-007, 1.024725787e-010, -1.244291058e-014, 6.02891239e-019}, bhigh = {-77408.9767, -12.04843385}, R = 34.37982547254071);
          constant IdealGases.Common.DataRecord Rb2F2(name = "Rb2F2", MM = 0.2089324064, Hf = -4091817.840662175, H0 = 90410.85739392509, Tlimit = 1000, alow = {-8178.3471, -333.473354, 11.31540799, -0.002842914926, 3.45495958e-006, -2.209373419e-009, 5.77207327e-013}, blow = {-104223.0504, -23.68991587}, ahigh = {-43765.7439, -5.76145204, 10.00482691, -2.104159548e-006, 4.96241627e-010, -5.98217497e-014, 2.882751742e-018}, bhigh = {-105917.6154, -16.03254014}, R = 39.79503296430707);
          constant IdealGases.Common.DataRecord Rb2I2(name = "Rb2I2", MM = 0.42474454, Hf = -1019333.837699244, H0 = 52307.95432944236, Tlimit = 1000, alow = {-5839.66355, -9.5275379, 10.03904495, -8.668754e-005, 1.074384368e-007, -6.97212223e-011, 1.842065721e-014}, blow = {-55027.8096, -7.17411332}, ahigh = {-6799.83464, -0.1642527699, 10.000142, -6.321420729999999e-008, 1.51281117e-011, -1.843027189e-015, 8.951244179999999e-020}, bhigh = {-55075.7962, -6.94804539}, R = 19.57522985463216);
          constant IdealGases.Common.DataRecord Rb2O(name = "Rb2O", MM = 0.186935, Hf = -582712.8199641587, H0 = 76046.64188086768, Tlimit = 1000, alow = {19712.76138, -462.750886, 8.58829313, -0.003085559728, 3.45198014e-006, -2.068570368e-009, 5.132480499999999e-013}, blow = {-12848.54711, -12.64891828}, ahigh = {-38463.6009, -10.82831019, 7.00842189, -3.47622812e-006, 7.86978506e-010, -9.195683850000001e-014, 4.325201520000001e-018}, bhigh = {-15253.73768, -3.23038878}, R = 44.47787733704229);
          constant IdealGases.Common.DataRecord Rb2O2(name = "Rb2O2", MM = 0.2029344, Hf = -1063636.662882192, H0 = 83017.6254001293, Tlimit = 1000, alow = {32605.5323, -757.816472, 10.70815124, 0.001148004329, -3.29124367e-006, 2.853106637e-009, -8.72159068e-013}, blow = {-24753.17258, -24.3427646}, ahigh = {-131856.7396, -96.47737790000001, 10.07222726, -2.894921706e-005, 6.4056391e-009, -7.35141939e-013, 3.40856171e-017}, bhigh = {-28821.79733, -19.10948823}, R = 40.97123011179967);
          constant IdealGases.Common.DataRecord Rb2O2H2(name = "Rb2O2H2", MM = 0.20495028, Hf = -3117829.358418052, H0 = 114210.036697681, Tlimit = 1000, alow = {-4577.07653, -841.7631, 17.04083828, -0.00535788254, 3.95177877e-006, -2.051775094e-010, -4.08541691e-013}, blow = {-76949.8217, -54.3541278}, ahigh = {1792711.652, -4659.406599999999, 16.93822422, 0.0002106246492, -1.26961747e-007, 2.063382927e-011, -1.150861507e-015}, bhigh = {-50241.1763, -60.27054920000001}, R = 40.56823928222982);
          constant IdealGases.Common.DataRecord Rb2SO4(name = "Rb2SO4", MM = 0.2669982, Hf = -4107114.025487812, H0 = 87348.70871788649, Tlimit = 1000, alow = {60948.54930000001, -688.97059, 7.03424218, 0.0393394547, -5.49892051e-005, 3.71634108e-008, -9.919863380000001e-012}, blow = {-131187.7657, -4.450168488}, ahigh = {-530289.831, -951.543545, 19.71013517, -0.0002842916301, 6.289142939999999e-008, -7.2193243e-012, 3.34871819e-016}, bhigh = {-133871.3151, -70.56246881999999}, R = 31.1405545056109);
          constant IdealGases.Common.DataRecord Rn(name = "Rn", MM = 0.2220176, Hf = 0, H0 = 27914.12933028733, Tlimit = 1000, alow = {3.38943209e-006, -1.311675533e-007, 2.500000001, -2.978593139e-012, 4.33705073e-015, -3.18204022e-018, 9.24778703e-022}, blow = {-745.374999, 6.95244198}, ahigh = {27301.90029, -82.84672620000001, 2.598178483, -5.81372985e-005, 1.819136527e-008, -2.866656182e-012, 1.789322176e-016}, bhigh = {-220.280934, 6.25500571}, R = 37.44960759867686);
          constant IdealGases.Common.DataRecord Rnplus(name = "Rnplus", MM = 0.2220170514, Hf = 4699054.678103881, H0 = 27914.19830558113, Tlimit = 1000, alow = {-0.09697148970000001, 0.001106742047, 2.499994937, 1.189388239e-008, -1.515895754e-011, 9.95893481e-015, -2.640751817e-018}, blow = {124730.476, 8.338761699999999}, ahigh = {-19982.85319, 59.3067566, 2.432476003, 3.71602592e-005, -1.012057848e-008, 1.192256661e-012, -3.18452198e-017}, bhigh = {124352.8478, 8.821997789999999}, R = 37.44970013595992);
          constant IdealGases.Common.DataRecord S(name = "S", MM = 0.032065, Hf = 8644004.366131296, H0 = 207622.7974426945, Tlimit = 1000, alow = {-317.484182, -192.4704923, 4.68682593, -0.0058413656, 7.53853352e-006, -4.86358604e-009, 1.256976992e-012}, blow = {33235.9218, -5.718523969}, ahigh = {-485424.479, 1438.830408, 1.258504116, 0.000379799043, 1.630685864e-009, -9.547095849999999e-012, 8.041466646e-016}, bhigh = {23349.9527, 15.59554855}, R = 259.300545766412);
          constant IdealGases.Common.DataRecord Splus(name = "Splus", MM = 0.03206445140000001, Hf = 39997454.25240614, H0 = 193280.337863507, Tlimit = 1000, alow = {0, 0, 2.5, 0, 0, 0, 0}, blow = {153502.6117, 5.43622334}, ahigh = {1346218.684, -4056.87151, 7.15343655, -0.002523562352, 6.42953961e-007, -6.43167216e-011, 2.141387919e-015}, bhigh = {179282.3835, -27.86935079}, R = 259.3049822146652);
          constant IdealGases.Common.DataRecord Sminus(name = "Sminus", MM = 0.0320655486, Hf = 2194520.539093474, H0 = 201615.2937423937, Tlimit = 1000, alow = {-2596.051473, -142.2398653, 4.00782567, -0.00360885591, 4.23623e-006, -2.520987604e-009, 6.07947976e-013}, blow = {8197.79307, -2.582377345}, ahigh = {2730.311692, 141.4072078, 2.403340775, 3.69357753e-005, -7.94408044e-009, 8.95220838e-013, -4.09966282e-017}, bhigh = {6931.1957, 6.574986902}, R = 259.2961094699624);
          constant IdealGases.Common.DataRecord SCL(name = "SCL", MM = 0.067518, Hf = 2317382.031458278, H0 = 145430.122337747, Tlimit = 1000, alow = {16130.51454, -560.624955, 8.06493113, -0.0091863264, 1.224205395e-005, -8.209562019999999e-009, 2.205087197e-012}, blow = {19977.3928, -16.9335441}, ahigh = {-94051.23449999999, 362.976085, 4.06995936, 0.0003034029742, -8.24830139e-008, 1.252700734e-011, -6.41746365e-016}, bhigh = {15231.11069, 6.014528409}, R = 123.1445244231168);
          constant IdealGases.Common.DataRecord SCL2(name = "SCL2", MM = 0.102971, Hf = -170659.7003039691, H0 = 120862.4272853522, Tlimit = 1000, alow = {57923.1183, -1062.447681, 10.37694738, -0.00613022902, 6.463485999999999e-006, -3.67903805e-009, 8.73193165e-013}, blow = {1262.475821, -26.91769492}, ahigh = {-234103.9963, 428.7569580000001, 6.47141539, 0.000317947674, -9.74748308e-008, 1.39331857e-011, -6.39479517e-016}, bhigh = {-7210.4737, -2.774273151}, R = 80.7457633702693);
          constant IdealGases.Common.DataRecord SCL2plus(name = "SCL2plus", MM = 0.1029704514, Hf = 8753805.919510614, H0 = 120940.0059015377, Tlimit = 1000, alow = {49348.7687, -960.1641020000001, 9.88479278, -0.004950257170000001, 4.94628697e-006, -2.678720604e-009, 6.07684403e-013}, blow = {111281.2037, -23.42664942}, ahigh = {-233940.3605, 551.628326, 6.12521216, 0.000677105632, -2.674382769e-007, 4.93593888e-011, -2.955831247e-015}, bhigh = {102678.7381, 0.1365611954}, R = 80.74619356286517);
          constant IdealGases.Common.DataRecord SD(name = "SD", MM = 0.034079102, Hf = 4063806.962988637, H0 = 272731.4528416858, Tlimit = 1000, alow = {-32955.7499, 164.7104687, 4.97898455, -0.008031585710000001, 1.653766397e-005, -1.345197981e-008, 4.00043153e-012}, blow = {14358.62329, -1.997334017}, ahigh = {245673.249, -1180.034105, 5.28469005, -0.0002384319882, 6.009960290000001e-008, -7.13116364e-012, 4.0358587e-016}, bhigh = {22655.45124, -8.350483655}, R = 243.9756775281227);
          constant IdealGases.Common.DataRecord SF(name = "SF", MM = 0.0510634032, Hf = 302478.0964853514, H0 = 185457.9484823683, Tlimit = 1000, alow = {12064.56774, -291.027077, 5.48010157, -0.001975880235, 2.47368599e-006, -1.610904623e-009, 4.31003854e-013}, blow = {1991.337574, -4.544138975}, ahigh = {763458.075, -2423.63843, 7.35219871, -0.001598026079, 4.86851502e-007, -6.47800053e-011, 2.98029731e-015}, bhigh = {15774.61591, -19.02584835}, R = 162.8264369187207);
          constant IdealGases.Common.DataRecord SFplus(name = "SFplus", MM = 0.0510628546, Hf = 19477372.06998999, H0 = 173596.2094058094, Tlimit = 1000, alow = {65783.0367, -698.669997, 5.62129224, -0.000690927411, -1.28239278e-007, 4.02132042e-010, -1.424033453e-013}, blow = {122175.1247, -6.682377715}, ahigh = {-937494.39, 2377.21424, 2.052929113, 0.001081609837, -1.535468024e-007, 8.2226919e-012, 3.25974688e-017}, bhigh = {102684.9122, 18.87705768}, R = 162.8281862643848);
          constant IdealGases.Common.DataRecord SFminus(name = "SFminus", MM = 0.0510639518, Hf = -4530532.495920145, H0 = 173803.7634603909, Tlimit = 1000, alow = {50198.7928, -532.154931, 4.90484901, 0.0009915307419999999, -2.270841545e-006, 1.844749756e-009, -5.3570572e-013}, blow = {-26113.88646, -3.637468065}, ahigh = {206892.3125, -1324.372811, 6.64621124, -0.001641781588, 6.78388312e-007, -1.198331099e-010, 7.12173954e-015}, bhigh = {-21509.94816, -14.70239183}, R = 162.8246876106444);
          constant IdealGases.Common.DataRecord SF2(name = "SF2", MM = 0.0700618064, Hf = -4184715.597056031, H0 = 157694.6637219448, Tlimit = 1000, alow = {68708.71130000001, -843.241825, 6.2694964, 0.00648414245, -1.145126533e-005, 8.770291560000001e-009, -2.542611601e-012}, blow = {-32299.807, -8.80002578}, ahigh = {-170261.2287, -149.7038204, 7.11093414, -4.4077416e-005, 9.68220831e-009, -1.104468959e-012, 5.09530878e-017}, bhigh = {-37042.8949, -10.9525396}, R = 118.6733889293497);
          constant IdealGases.Common.DataRecord SF2plus(name = "SF2plus", MM = 0.07006125780000001, Hf = 10077122.83749493, H0 = 160347.9633789846, Tlimit = 1000, alow = {131164.8923, -1552.127914, 9.803424720000001, -0.002488118587, 7.501565130000001e-007, 3.135384495e-010, -1.986281446e-013}, blow = {91377.5839, -27.92300566}, ahigh = {-384753.102, 532.789302, 6.2819089, 0.000452279012, -1.426421096e-007, 2.079708115e-011, -9.69170989e-016}, bhigh = {78842.1381, -4.21611053}, R = 118.6743181764573);
          constant IdealGases.Common.DataRecord SF2minus(name = "SF2minus", MM = 0.07006235499999999, Hf = -5634908.504003327, H0 = 173058.3421011184, Tlimit = 1000, alow = {52845.7248, -1032.342905, 10.06404631, -0.00520950325, 5.16838137e-006, -2.783420867e-009, 6.28641008e-013}, blow = {-44233.03780000001, -26.9926697}, ahigh = {-93766.18949999999, -36.3899527, 7.02794282, -1.142143365e-005, 2.566295898e-009, -2.981125977e-013, 1.395700722e-017}, bhigh = {-49670.5033, -8.483087898999999}, R = 118.6724596967944);
          constant IdealGases.Common.DataRecord SF3(name = "SF3", MM = 0.0890602096, Hf = -5660231.401476514, H0 = 152124.7823337708, Tlimit = 1000, alow = {128400.8253, -2084.871223, 14.23421102, -0.00401503219, 9.152258039999999e-007, 1.109809721e-009, -5.98860272e-013}, blow = {-52395.1734, -51.87194724}, ahigh = {-241128.346, -141.8950375, 10.10590548, -4.231526910000001e-005, 9.336120599999999e-009, -1.068727665e-012, 4.9443426e-017}, bhigh = {-63584.4586, -24.89561796}, R = 93.35787595092299);
          constant IdealGases.Common.DataRecord SF3plus(name = "SF3plus", MM = 0.08905966100000001, Hf = 4419323.760956152, H0 = 139335.2260795154, Tlimit = 1000, alow = {215763.3857, -2564.488498, 13.23216044, -0.0001749580869, -3.71295871e-006, 3.58068842e-009, -1.084376153e-012}, blow = {58760.8842, -50.24130184}, ahigh = {-377826.075, -162.1770252, 9.96026597, 0.0001237729568, -6.442207889999999e-008, 1.300856025e-011, -7.915106980000001e-016}, bhigh = {44154.26809999999, -26.7390077}, R = 93.35845102756454);
          constant IdealGases.Common.DataRecord SF3minus(name = "SF3minus", MM = 0.08906075819999999, Hf = -8871743.548664289, H0 = 154084.6190550397, Tlimit = 1000, alow = {154650.0675, -2469.768526, 16.57548496, -0.01020137434, 9.36800846e-006, -4.7256726e-009, 1.009661431e-012}, blow = {-85001.579, -65.32166474}, ahigh = {-225386.0964, -130.8890242, 10.0997238, -4.05234553e-005, 9.064565220000001e-009, -1.049320641e-012, 4.89920126e-017}, bhigh = {-98002.272, -25.24368023}, R = 93.35730088136619);
          constant IdealGases.Common.DataRecord SF4(name = "SF4", MM = 0.1080586128, Hf = -7033220.03037966, H0 = 142356.8339570615, Tlimit = 1000, alow = {163618.3265, -2531.316261, 15.59106568, 0.00332670431, -1.054935525e-005, 9.39136305e-009, -2.914816694e-012}, blow = {-81155.5892, -61.31532958}, ahigh = {-377712.516, -293.7595659, 13.21902007, -8.74766945e-005, 1.929905265e-008, -2.209423506e-012, 1.022335559e-016}, bhigh = {-94831.7534, -42.43061278}, R = 76.944093437409);
          constant IdealGases.Common.DataRecord SF4plus(name = "SF4plus", MM = 0.1080580642, Hf = 3850818.215934689, H0 = 152747.0172836948, Tlimit = 1000, alow = {230958.7324, -3383.77767, 21.01756367, -0.01106529555, 9.038097280000001e-006, -4.05591264e-009, 7.71257518e-013}, blow = {64253.7515, -89.38224908000001}, ahigh = {-418917.275, 40.0288964, 12.83873264, 0.0001340235036, -4.81813518e-008, 7.574595959999999e-012, -3.69667078e-016}, bhigh = {44699.4423, -37.60378598}, R = 76.94448407488684);
          constant IdealGases.Common.DataRecord SF4minus(name = "SF4minus", MM = 0.1080591614, Hf = -8212763.346505111, H0 = 171020.3999417675, Tlimit = 1000, alow = {100552.3355, -2267.351994, 19.7928456, -0.01163761842, 1.161792139e-005, -6.28889547e-009, 1.426371272e-012}, blow = {-98955.9743, -79.16366078}, ahigh = {320882.208, -1710.416599, 14.94425485, -0.001057207329, 2.74284692e-007, -2.905761211e-011, 1.084482543e-015}, bhigh = {-100523.1225, -51.64256578}, R = 76.94370280389758);
          constant IdealGases.Common.DataRecord SF5(name = "SF5", MM = 0.127057016, Hf = -7104395.455029419, H0 = 148055.4131697851, Tlimit = 1000, alow = {222417.685, -4043.99175, 27.07196786, -0.01744176604, 1.604198715e-005, -7.99724302e-009, 1.667968061e-012}, blow = {-92200.7827, -123.2426279}, ahigh = {-389242.637, -182.5826536, 16.13765057, -5.54483423e-005, 1.231433706e-008, -1.417173409e-012, 6.5848894e-017}, bhigh = {-113567.1054, -55.77524201}, R = 65.43890500308933);
          constant IdealGases.Common.DataRecord SF5plus(name = "SF5plus", MM = 0.1270564674, Hf = 1358797.466456241, H0 = 128662.0377106439, Tlimit = 1000, alow = {377713.562, -5646.07512, 30.4940427, -0.02176488368, 1.941993792e-005, -9.55620114e-009, 2.000077302e-012}, blow = {45922.0481, -148.9889627}, ahigh = {-981434.232, 720.881739, 15.64386818, -0.0002452519972, 2.241690996e-007, -3.99003589e-011, 2.279087545e-015}, bhigh = {9090.95319, -55.31460631}, R = 65.43918755291948);
          constant IdealGases.Common.DataRecord SF5minus(name = "SF5minus", MM = 0.1270575646, Hf = -9480917.494305568, H0 = 149931.356389307, Tlimit = 1000, alow = {209503.1134, -3930.08114, 27.02127215, -0.01792973203, 1.718122152e-005, -9.00056019e-009, 1.988018147e-012}, blow = {-129181.6581, -123.1672857}, ahigh = {-373321.92, -179.3729521, 16.13699748, -5.57753997e-005, 1.249477046e-008, -1.448104799e-012, 6.7674623e-017}, bhigh = {-149855.4401, -56.31439391}, R = 65.4386224556991);
          constant IdealGases.Common.DataRecord SF6(name = "SF6", MM = 0.1460554192, Hf = -8348885.694752777, H0 = 115983.830608868, Tlimit = 1000, alow = {330952.674, -4737.68505, 22.47738068, 0.01046954309, -2.560641961e-005, 2.153716967e-008, -6.51609896e-012}, blow = {-125536.0583, -109.1760145}, ahigh = {-730672.65, -636.705655, 19.47442853, -0.0001894325671, 4.17872283e-008, -4.78374495e-012, 2.213516129e-016}, bhigh = {-151060.9837, -81.47574587}, R = 56.92682986733026);
          constant IdealGases.Common.DataRecord SF6minus(name = "SF6minus", MM = 0.1460559678, Hf = -9187409.821127487, H0 = 119753.504519245, Tlimit = 1000, alow = {498580.921, -6934.44574, 34.4131833, -0.0198403236, 1.497871222e-005, -6.133234650000001e-009, 1.045159581e-012}, blow = {-129706.9066, -174.7984495}, ahigh = {-681807.3509999999, -594.742996, 19.45009842, -0.0001820212649, 4.05707957e-008, -4.683820930000001e-012, 2.182263999e-016}, bhigh = {-165890.2452, -79.65845887}, R = 56.92661604478445);
          constant IdealGases.Common.DataRecord SH(name = "SH", MM = 0.03307294, Hf = 4297631.507812732, H0 = 275092.2355254779, Tlimit = 1000, alow = {6389.43468, -374.796092, 7.54814577, -0.01288875477, 1.907786343e-005, -1.265033728e-008, 3.23515869e-012}, blow = {17429.02395, -17.60761843}, ahigh = {1682631.601, -5177.15221, 9.198168519999999, -0.002323550224, 6.543914779999999e-007, -8.468470419999999e-011, 3.86474155e-015}, bhigh = {48992.14490000001, -37.70400275}, R = 251.3980311396568);
          constant IdealGases.Common.DataRecord SHminus(name = "SHminus", MM = 0.0330734886, Hf = -2617628.186946085, H0 = 261420.9859917832, Tlimit = 1000, alow = {38780.7076, -574.25906, 6.89854446, -0.01001352412, 1.486460572e-005, -9.7503689e-009, 2.446909813e-012}, blow = {-8735.38898, -16.15966489}, ahigh = {1198715.402, -3894.84682, 7.66042233, -0.00135523759, 3.34237024e-007, -3.35072231e-011, 9.05508478e-016}, bhigh = {13174.77387, -28.18370616}, R = 251.3938611241634);
          constant IdealGases.Common.DataRecord SN(name = "SN", MM = 0.0460717, Hf = 5803743.143838843, H0 = 203880.3213252387, Tlimit = 1000, alow = {-68354.1235, 1147.567483, -2.877802574, 0.0172486432, -2.058999904e-005, 1.26136964e-008, -3.139030141e-012}, blow = {25641.43612, 42.24006964}, ahigh = {-483728.446, 1058.07559, 3.086198804, 0.000911136078, -2.764061722e-007, 4.157370109999999e-011, -2.128351755e-015}, bhigh = {23793.45477, 10.33222139}, R = 180.4680964670286);
          constant IdealGases.Common.DataRecord SO(name = "SO", MM = 0.0480644, Hf = 99040.16278160134, H0 = 183048.2852173334, Tlimit = 1000, alow = {-33427.57, 640.38625, -1.006641228, 0.01381512705, -1.704486364e-005, 1.06129493e-008, -2.645796205e-012}, blow = {-3371.29219, 30.93861963}, ahigh = {-1443410.557, 4113.87436, -0.538369578, 0.002794153269, -6.63335226e-007, 7.838221189999999e-011, -3.56050907e-015}, bhigh = {-27088.38059, 36.15358329}, R = 172.9860770133404);
          constant IdealGases.Common.DataRecord SOminus(name = "SOminus", MM = 0.0480649486, Hf = -2204685.307829498, H0 = 196965.0291064703, Tlimit = 1000, alow = {8420.196969999999, -69.1658668, 3.83766661, 0.002166482062, -2.811222562e-006, 1.793426453e-009, -4.52179595e-013}, blow = {-13541.62531, 4.316141603}, ahigh = {176715.6147, -663.398736, 5.17727981, -0.0002853461125, 7.21442153e-008, -3.67646949e-012, -2.910092894e-016}, bhigh = {-9984.43801, -3.951456757}, R = 172.9841025982081);
          constant IdealGases.Common.DataRecord SOF2(name = "SOF2", MM = 0.08606120640000001, Hf = -6796933.687882859, H0 = 146696.1192865639, Tlimit = 1000, alow = {61145.7965, -908.1119669999999, 6.88362326, 0.01192459695, -1.67100641e-005, 1.110723646e-008, -2.910788226e-012}, blow = {-67429.3584, -11.25458745}, ahigh = {-251055.5779, -607.4439609999999, 10.45046709, -0.0001796384979, 3.96416973e-008, -4.54305524e-012, 2.104969923e-016}, bhigh = {-70720.71710000001, -29.04334504}, R = 96.61114859761017);
          constant IdealGases.Common.DataRecord SO2(name = "SO2", MM = 0.0640638, Hf = -4633037.690552231, H0 = 164650.3485587805, Tlimit = 1000, alow = {-53108.4214, 909.031167, -2.356891244, 0.02204449885, -2.510781471e-005, 1.446300484e-008, -3.36907094e-012}, blow = {-41137.52080000001, 40.45512519}, ahigh = {-112764.0116, -825.226138, 7.61617863, -0.000199932761, 5.65563143e-008, -5.45431661e-012, 2.918294102e-016}, bhigh = {-33513.0869, -16.55776085}, R = 129.7842463294403);
          constant IdealGases.Common.DataRecord SO2minus(name = "SO2minus", MM = 0.0640643486, Hf = -6378057.030614996, H0 = 167791.1542832732, Tlimit = 1000, alow = {94609.01659999999, -856.269105, 5.36007422, 0.00760957674, -1.092005659e-005, 7.20254773e-009, -1.85013822e-012}, blow = {-45800.9481, -3.929956604}, ahigh = {-179340.0005, -366.316876, 7.27444105, -0.0001101944663, 2.443328132e-008, -2.809735019e-012, 1.305160431e-016}, bhigh = {-49730.7261, -12.61421156}, R = 129.7831349525343);
          constant IdealGases.Common.DataRecord SO2CL2(name = "SO2CL2", MM = 0.1349698, Hf = -2628759.915181026, H0 = 118757.7072797026, Tlimit = 1000, alow = {6821.59239, -584.9569200000001, 8.59675691, 0.01197228675, -1.3337607e-005, 7.126128999999999e-009, -1.496150016e-012}, blow = {-42307.8381, -16.52458363}, ahigh = {-237663.9109, -964.7740410000001, 13.71469904, -0.0002850327057, 6.29350962e-008, -7.21784426e-012, 3.34684022e-016}, bhigh = {-41900.7299, -44.81942574999999}, R = 61.60246218042851);
          constant IdealGases.Common.DataRecord SO2FCL(name = "SO2FCL", MM = 0.1185152032, Hf = -4695363.843412791, H0 = 124042.9970422563, Tlimit = 1000, alow = {36845.613, -803.8358949999999, 7.52618688, 0.01620141782, -1.97059527e-005, 1.16625033e-008, -2.759241465e-012}, blow = {-65035.9819, -12.99745646}, ahigh = {-291540.2438, -1107.134423, 13.81845158, -0.000325851446, 7.184741e-008, -8.230625200000001e-012, 3.81292562e-016}, bhigh = {-65524.7269, -47.09166446}, R = 70.15531995476509);
          constant IdealGases.Common.DataRecord SO2F2(name = "SO2F2", MM = 0.1020606064, Hf = -7446555.794714541, H0 = 132180.7842991613, Tlimit = 1000, alow = {55331.5353, -807.463968, 5.28717216, 0.02294547775, -2.891025299e-005, 1.785631907e-008, -4.4189195e-012}, blow = {-88994.28820000001, -4.124256607}, ahigh = {-340390.445, -1309.288764, 13.96601541, -0.000384008815, 8.456688780000001e-008, -9.678246870000001e-012, 4.48001428e-016}, bhigh = {-89019.9414, -51.0927005}, R = 81.46602585735764);
          constant IdealGases.Common.DataRecord SO3(name = "SO3", MM = 0.0800632, Hf = -4944843.573576874, H0 = 145990.9046852986, Tlimit = 1000, alow = {-39528.5529, 620.857257, -1.437731716, 0.02764126467, -3.144958662e-005, 1.792798e-008, -4.12638666e-012}, blow = {-51841.0617, 33.91331216}, ahigh = {-216692.3781, -1301.022399, 10.96287985, -0.000383710002, 8.466889039999999e-008, -9.70539929e-012, 4.49839754e-016}, bhigh = {-43982.83990000001, -36.55217314}, R = 103.8488594010732);
          constant IdealGases.Common.DataRecord S2(name = "S2", MM = 0.06412999999999999, Hf = 2005301.730859192, H0 = 142399.9688133479, Tlimit = 1000, alow = {35280.9178, -422.215658, 4.67743349, 0.001724046361, -3.86220821e-006, 3.33615634e-009, -9.93066154e-013}, blow = {16547.67715, -0.7957279032}, ahigh = {-15881.28788, 631.548088, 2.449628069, 0.001986240565, -6.50792724e-007, 1.002813651e-010, -5.59699005e-015}, bhigh = {10855.08427, 14.58544515}, R = 129.650272883206);
          constant IdealGases.Common.DataRecord S2minus(name = "S2minus", MM = 0.0641305486, Hf = -579012.4333974589, H0 = 149649.6632183807, Tlimit = 1000, alow = {10255.58251, -616.6128249999999, 8.182052840000001, -0.008311203580000001, 9.72027416e-006, -5.77547678e-009, 1.393690695e-012}, blow = {-3063.559874, -19.06068041}, ahigh = {483020.403, -1319.171302, 6.03177848, -0.0007965560690000001, 2.28224169e-007, -2.504364698e-011, 7.28055078e-016}, bhigh = {2660.25849, -9.010480318999999}, R = 129.6491637996061);
          constant IdealGases.Common.DataRecord S2CL2(name = "S2CL2", MM = 0.135036, Hf = -123937.3204182589, H0 = 122343.382505406, Tlimit = 1000, alow = {79749.74740000001, -1636.770024, 15.83744686, -0.01186906072, 1.413008027e-005, -8.48490393e-009, 1.981743887e-012}, blow = {3276.87242, -52.93745205}, ahigh = {632881.6, -3442.7058, 15.1963735, -0.002960877176, 6.88828437e-007, -7.990692529999999e-011, 3.69246606e-015}, bhigh = {15266.72656, -54.53315445000001}, R = 61.5722622115584);
          constant IdealGases.Common.DataRecord S2F2(name = "S2F2", MM = 0.1021268064, Hf = -3930535.127357121, H0 = 134322.1479605574, Tlimit = 1000, alow = {125290.4041, -1941.243874, 13.25382183, -0.001341525124, -2.772465632e-006, 3.65816929e-009, -1.299531209e-012}, blow = {-40672.0825, -45.60828617}, ahigh = {-259175.7719, -105.9653069, 10.06588551, -2.274618148e-005, 5.41234314e-009, -9.91138714e-013, 9.621861440000001e-017}, bhigh = {-51476.75180000001, -23.7566614}, R = 81.41321845936034);
          constant IdealGases.Common.DataRecord S2O(name = "S2O", MM = 0.0801294, Hf = -699312.5494512626, H0 = 138882.320346839, Tlimit = 1000, alow = {10927.0331, -95.23099869999999, 3.14452543, 0.011768542, -1.58026684e-005, 1.03764504e-008, -2.70862226e-012}, blow = {-7500.4719, 11.04169896}, ahigh = {-144213.979, -327.643013, 7.24428611, -9.7765383e-005, 2.1627127e-008, -2.48278222e-012, 1.15177875e-016}, bhigh = {-7438.55393, -10.85180744}, R = 103.7630632452009);
          constant IdealGases.Common.DataRecord S3(name = "S3", MM = 0.09619499999999999, Hf = 1504634.627579396, H0 = 124479.733873902, Tlimit = 1000, alow = {72453.9574, -1162.146759, 9.95541368, -0.00415802622, 3.24177839e-006, -1.264648239e-009, 1.777450535e-013}, blow = {21462.75475, -25.87525865}, ahigh = {-111780.5401, -51.97908390000001, 7.03876084, -1.546804022e-005, 3.40834041e-009, -3.89686236e-013, 1.800860389e-017}, bhigh = {15254.06485, -7.610045099}, R = 86.43351525547067);
          constant IdealGases.Common.DataRecord S4(name = "S4", MM = 0.12826, Hf = 1057479.596132855, H0 = 111338.944331826, Tlimit = 1000, alow = {119866.4135, -2040.786521, 15.10235054, -0.0070411043, 5.350405e-006, -2.002348038e-009, 2.569940995e-013}, blow = {24109.09409, -55.03153238}, ahigh = {-206833.3537, -96.96151430000001, 10.0724321, -2.895047053e-005, 6.38780619e-009, -7.31176743e-013, 3.38226529e-017}, bhigh = {13211.0953, -23.44872237}, R = 64.82513644160301);
          constant IdealGases.Common.DataRecord S5(name = "S5", MM = 0.160325, Hf = 829523.4929050366, H0 = 118842.0645563699, Tlimit = 1000, alow = {136137.0439, -2955.476536, 26.33358816, -0.0412580653, 7.05681403e-005, -5.611095330000001e-008, 1.6594572e-011}, blow = {26753.07046, -106.9711067}, ahigh = {-4038495.16, 8601.80388, 7.4437809, 0.001533393812, -2.532718676e-007, 2.145210795e-011, -7.213102680000001e-016}, bhigh = {-46869.2542, 11.04229196}, R = 51.8601091532824);
          constant IdealGases.Common.DataRecord S6(name = "S6", MM = 0.19239, Hf = 526613.5454025677, H0 = 118442.6373512137, Tlimit = 1000, alow = {97803.07210000001, -2568.47013, 24.67025557, -0.01619983175, 1.712334526e-005, -9.74629674e-009, 2.486964867e-012}, blow = {20378.88389, -101.4410309}, ahigh = {3686845.06, -7695.0179, 18.60721995, 0.002290382548, -1.219834021e-006, 2.060780798e-010, -1.190354318e-014}, bhigh = {60324.8648, -74.90245718}, R = 43.21675762773533);
          constant IdealGases.Common.DataRecord S7(name = "S7", MM = 0.224455, Hf = 498498.0018266468, H0 = 117058.5106145998, Tlimit = 1000, alow = {123365.5613, -3200.54364, 30.15678887, -0.02197472483, 2.487892075e-005, -1.506049557e-008, 3.76892599e-012}, blow = {23900.05896, -127.5978171}, ahigh = {-272639.6041, -73.7321522, 19.05771987, -2.394575885e-005, 5.4426713e-009, -6.379663479999999e-013, 3.008212993e-017}, bhigh = {7308.458830000001, -61.59112428}, R = 37.04293510948742);
          constant IdealGases.Common.DataRecord S8(name = "S8", MM = 0.25652, Hf = 394811.7963511617, H0 = 123082.4964915017, Tlimit = 1000, alow = {314562.5719, -6116.51016, 48.7532754, -0.0624179465, 7.421831750000001e-005, -3.72644931e-008, 5.79942988e-012}, blow = {35738.9084, -228.8701977}, ahigh = {-8727921.130000001, 12216.27968, 22.09959617, -0.00349406483, 1.397604162e-006, -2.169815281e-010, 1.212304364e-014}, bhigh = {-86581.10819999999, -64.35742508}, R = 32.4125682208015);
          constant IdealGases.Common.DataRecord Sc(name = "Sc", MM = 0.04495591, Hf = 8401570.761219159, H0 = 155758.3641394424, Tlimit = 1000, alow = {-3700.80594, 169.2506026, 1.842242597, 0.001364835821, -1.580085847e-006, 9.61311115e-010, -2.392381918e-013}, blow = {43852.1524, 10.72781921}, ahigh = {8810382.649999999, -27112.32975, 34.7658866, -0.01861104581, 5.290283900000001e-006, -6.58540806e-010, 2.997850429e-014}, bhigh = {216246.0097, -222.5618519}, R = 184.9472516516738);
          constant IdealGases.Common.DataRecord Scplus(name = "Scplus", MM = 0.0449553614, Hf = 22625671.11739424, H0 = 159311.8546256421, Tlimit = 1000, alow = {-5884.93016, 147.9044408, 2.009576456, 0.000738956697, -6.61796482e-007, 6.84107376e-010, -2.164125532e-013}, blow = {120843.9139, 10.2657062}, ahigh = {1973658.531, -4954.3841, 6.36062735, -0.000716878592, 2.464991123e-008, 9.632373790000001e-012, -8.544709642000001e-016}, bhigh = {154342.3764, -22.61925782}, R = 184.9495086029939);
          constant IdealGases.Common.DataRecord Scminus(name = "Scminus", MM = 0.0449564586, Hf = 7842228.65810876, H0 = 137854.0079222344, Tlimit = 1000, alow = {0, 0, 2.5, 0, 0, 0, 0}, blow = {41657.4639, 8.270420720000001}, ahigh = {0, 0, 2.5, 0, 0, 0, 0}, bhigh = {41657.4639, 8.270420720000001}, R = 184.9449947554365);
          constant IdealGases.Common.DataRecord ScO(name = "ScO", MM = 0.06095531, Hf = -903371.5684490819, H0 = 144156.5304154798, Tlimit = 1000, alow = {-3136.784613, 227.1981685, 0.885075732, 0.01052430774, -1.431954596e-005, 9.652351069999999e-009, -2.586210633e-012}, blow = {-8550.798769999999, 20.12706847}, ahigh = {1224192.443, -3918.91572, 8.679848809999999, -0.001995893306, 4.06698985e-007, -1.504459913e-011, -8.4729989e-016}, bhigh = {16773.22594, -29.51656272}, R = 136.4027514584045);
          constant IdealGases.Common.DataRecord ScOplus(name = "ScOplus", MM = 0.0609547614, Hf = 9206991.432830053, H0 = 143993.1155238678, Tlimit = 1000, alow = {45885.6027, -336.711437, 3.5272064, 0.0039888503, -5.568985429999999e-006, 3.65028119e-009, -9.356491500000001e-013}, blow = {68383.5079, 4.3398127}, ahigh = {-1814357.141, 5853.42532, -3.65710121, 0.00573436451, -2.092878161e-006, 3.69564502e-010, -2.214038347e-014}, bhigh = {29553.64078, 56.6088941}, R = 136.4039790991619);
          constant IdealGases.Common.DataRecord ScO2(name = "ScO2", MM = 0.07695471000000001, Hf = -5375248.207679556, H0 = 139720.2718326143, Tlimit = 1000, alow = {47947.00960000001, -429.835859, 3.4172065, 0.01361338544, -2.047465023e-005, 1.455906041e-008, -4.03808769e-012}, blow = {-48610.4953, 7.67001333}, ahigh = {-190390.2542, -231.558882, 7.17210491, -6.85920964e-005, 1.510998557e-008, -1.727951588e-012, 7.9889805e-017}, bhigh = {-51129.74750000001, -10.90764597}, R = 108.0437051871159);
          constant IdealGases.Common.DataRecord Sc2O(name = "Sc2O", MM = 0.10591122, Hf = -217580.4225463553, H0 = 110562.827998771, Tlimit = 1000, alow = {25500.02615, -307.7608607, 4.67121504, 0.008696142249999999, -1.283873116e-005, 8.988966810000001e-009, -2.461994147e-012}, blow = {-2614.951944, 2.92001718}, ahigh = {-135088.0396, -185.4540692, 7.13829408, -5.52943214e-005, 1.221589315e-008, -1.400490463e-012, 6.48885811e-017}, bhigh = {-4240.235140000001, -9.37271095}, R = 78.50416603642182);
          constant IdealGases.Common.DataRecord Sc2O2(name = "Sc2O2", MM = 0.12191062, Hf = -4024022.02531658, H0 = 107732.8619934834, Tlimit = 1000, alow = {3468.44549, 261.7988314, -0.3052099624, 0.0307610407, -4.18347836e-005, 2.794218425e-008, -7.41267205e-012}, blow = {-61440.1321, 29.15248555}, ahigh = {-297734.8677, -580.367792, 10.43318377, -0.0001734165462, 3.83608979e-008, -4.403078029999999e-012, 2.042208922e-016}, bhigh = {-59648.6628, -28.55828369}, R = 68.20137572920227);
          constant IdealGases.Common.DataRecord Si(name = "Si", MM = 0.0280855, Hf = 16022502.71492407, H0 = 268831.1762297271, Tlimit = 1000, alow = {98.36140810000001, 154.6544523, 1.87643667, 0.001320637995, -1.529720059e-006, 8.95056277e-010, -1.95287349e-013}, blow = {52635.1031, 9.69828888}, ahigh = {-616929.885, 2240.683927, -0.444861932, 0.001710056321, -4.10771416e-007, 4.55888478e-011, -1.889515353e-015}, bhigh = {39535.5876, 26.79668061}, R = 296.0414448736893);
          constant IdealGases.Common.DataRecord Siplus(name = "Siplus", MM = 0.0280849514, Hf = 44241060.89070889, H0 = 261454.7874916387, Tlimit = 1000, alow = {-43297.9188, 679.5894490000001, 0.2257046144, 0.004118600490000001, -4.234881599999999e-006, 2.327995626e-009, -5.318388059e-013}, blow = {145203.9813, 19.3465051}, ahigh = {59193.9023, -48.5673095, 2.556312024, -3.50339716e-005, 1.190298787e-008, -2.082923821e-012, 1.471452049e-016}, bhigh = {149143.1392, 5.24426714}, R = 296.0472276266784);
          constant IdealGases.Common.DataRecord Siminus(name = "Siminus", MM = 0.0280860486, Hf = 10995406.73727952, H0 = 220658.5941747605, Tlimit = 1000, alow = {-794.014667, 5.56741842, 2.499837183, -9.48139446e-005, 3.17124693e-007, -4.19132318e-010, 2.03592438e-013}, blow = {36364.4338, 5.27011984}, ahigh = {-6162070.100000001, 18833.10402, -18.9930245, 0.01111021657, -2.535790208e-006, 2.699962923e-010, -1.105062911e-014}, bhigh = {-83140.8931, 159.5298253}, R = 296.0356623466072);
          constant IdealGases.Common.DataRecord SiBr(name = "SiBr", MM = 0.1079895, Hf = 1621985.665273013, H0 = 92926.82158913597, Tlimit = 1000, alow = {7370.283350000001, -505.106296, 8.280591039999999, -0.0100538392, 1.35705092e-005, -9.1154499e-009, 2.447477685e-012}, blow = {21844.05475, -16.62032657}, ahigh = {1317799.94, -3669.79502, 8.51168612, -0.001987602041, 5.03541489e-007, -4.49012019e-011, 8.23567007e-016}, bhigh = {43328.5155, -24.67075472}, R = 76.99333731520194);
          constant IdealGases.Common.DataRecord SiBr2(name = "SiBr2", MM = 0.1878935, Hf = -271430.3581550187, H0 = 70876.11333015778, Tlimit = 1000, alow = {23866.16151, -641.569347, 9.34587911, -0.00479290414, 5.58253847e-006, -3.45531238e-009, 8.80033894e-013}, blow = {-5014.75702, -17.40767596}, ahigh = {-51473.4394, -12.77719997, 7.01020224, -4.29633186e-006, 9.877651970000001e-010, -1.168133105e-013, 5.54668138e-018}, bhigh = {-8319.54225, -3.61182661}, R = 44.25098260450734);
          constant IdealGases.Common.DataRecord SiBr3(name = "SiBr3", MM = 0.2677975, Hf = -586263.8747561124, H0 = 65203.89473389407, Tlimit = 1000, alow = {39728.0686, -1066.475023, 13.80836232, -0.00764111472, 8.77586583e-006, -5.37243085e-009, 1.356453455e-012}, blow = {-16517.95623, -37.8405537}, ahigh = {-88880.24740000001, -22.67879074, 10.01791167, -7.48115075e-006, 1.70924269e-009, -2.011596627e-013, 9.51553415e-018}, bhigh = {-22030.26938, -15.37574366}, R = 31.04760873421148);
          constant IdealGases.Common.DataRecord SiBr4(name = "SiBr4", MM = 0.3477015, Hf = -1195853.33971812, H0 = 64174.93453436353, Tlimit = 1000, alow = {68210.8674, -1520.370776, 18.1045336, -0.00973468842, 1.072367466e-005, -6.34385829e-009, 1.5572282e-012}, blow = {-46165.8521, -59.7612063}, ahigh = {-127140.9281, -37.6808795, 13.02910642, -1.194958251e-005, 2.693843051e-009, -3.137193207e-013, 1.471642663e-017}, bhigh = {-54089.9883, -29.4038433}, R = 23.9126722202809);
          constant IdealGases.Common.DataRecord SiC(name = "SiC", MM = 0.04009620000000001, Hf = 18329570.88202872, H0 = 229874.9258034427, Tlimit = 1000, alow = {-6223.330889999999, 314.1790457, 0.389313083, 0.0118750754, -1.639277197e-005, 1.131808223e-008, -3.045324231e-012}, blow = {86062.2773, 23.10166717}, ahigh = {-62688.06030000001, 720.983692, 2.162879732, 0.002201299585, -6.569466590000001e-007, 9.177110259999999e-011, -4.96916674e-015}, bhigh = {83212.2585, 16.01675317}, R = 207.3630917643068);
          constant IdealGases.Common.DataRecord SiC2(name = "SiC2", MM = 0.05210690000000001, Hf = 12116647.46895325, H0 = 224258.3803680511, Tlimit = 1000, alow = {-41196.2899, 686.9747180000001, 0.02361343691, 0.01532939217, -1.435588838e-005, 6.37522874e-009, -1.059204957e-012}, blow = {71308.9071, 28.28850316}, ahigh = {7026893.09, -24661.48437, 39.15453030000001, -0.02002884068, 6.30735369e-006, -8.84838351e-010, 4.53054985e-014}, bhigh = {226732.5155, -236.6622024}, R = 159.5656621291998);
          constant IdealGases.Common.DataRecord SiCL(name = "SiCL", MM = 0.0635385, Hf = 2240581.993594435, H0 = 155561.100749939, Tlimit = 1000, alow = {12485.08319, -204.4170347, 4.97707085, -0.0005160756260000001, 2.931980374e-007, -3.66661523e-012, -3.39303713e-014}, blow = {16865.28019, -0.226395724}, ahigh = {376729.456, -1147.956509, 5.73540843, -0.0005784890810000001, 1.517304648e-007, -1.291391394e-011, 1.55462842e-016}, bhigh = {23062.78401, -6.098515228}, R = 130.857228294656);
          constant IdealGases.Common.DataRecord SiCL2(name = "SiCL2", MM = 0.0989915, Hf = -1647305.950510903, H0 = 126569.392321563, Tlimit = 1000, alow = {53055.5097, -1015.842786, 10.32848185, -0.00621226752, 6.7158254e-006, -3.90880164e-009, 9.46173215e-013}, blow = {-16502.18093, -26.49925756}, ahigh = {-80508.7157, -26.61025523, 7.02041472, -8.335824719999999e-006, 1.871092411e-009, -2.171552767e-013, 1.015849895e-017}, bhigh = {-21812.94885, -6.638764238}, R = 83.99177707176879);
          constant IdealGases.Common.DataRecord SiCL3(name = "SiCL3", MM = 0.1344445, Hf = -2501193.20611851, H0 = 116906.7310302764, Tlimit = 1000, alow = {86917.08459999999, -1677.556857, 15.1151906, -0.008937031, 9.097353529999999e-006, -5.01457996e-009, 1.155962016e-012}, blow = {-34774.773, -50.87215506}, ahigh = {-147718.0114, -53.3520098, 10.04043521, -1.63512106e-005, 3.64194011e-009, -4.20060059e-013, 1.955201587e-017}, bhigh = {-43605.7165, -20.06632444}, R = 61.84315461026669);
          constant IdealGases.Common.DataRecord SiCL4(name = "SiCL4", MM = 0.1698975, Hf = -3897644.16780706, H0 = 114512.797421975, Tlimit = 1000, alow = {117164.6285, -2185.821461, 19.08250496, -0.00963633572, 8.836847690000002e-006, -4.3600424e-009, 8.94023243e-013}, blow = {-72128.0211, -73.01532221000001}, ahigh = {-210033.0761, -84.76107710000001, 13.06364027, -2.55402042e-005, 5.65381847e-009, -6.48875267e-013, 3.008025543e-017}, bhigh = {-83722.6813, -35.92584781}, R = 48.93816565870598);
          constant IdealGases.Common.DataRecord SiF(name = "SiF", MM = 0.0470839032, Hf = -535908.9685665653, H0 = 200835.3886854478, Tlimit = 1000, alow = {14782.23768, 13.57998649, 2.324182643, 0.00713285118, -1.022951215e-005, 7.14839647e-009, -1.964689785e-012}, blow = {-3995.38065, 12.31128121}, ahigh = {-365363.788, 872.2860040000001, 3.37721827, 0.000750815243, -2.308898619e-007, 3.7667818e-011, -2.143080403e-015}, bhigh = {-10116.97664, 8.968132819999999}, R = 176.5884184385121);
          constant IdealGases.Common.DataRecord SiFCL(name = "SiFCL", MM = 0.0825369032, Hf = -4577667.701978914, H0 = 143364.7682579882, Tlimit = 1000, alow = {41769.5464, -592.6575380000001, 6.40977071, 0.00464168261, -7.978461910000001e-006, 6.00872837e-009, -1.720953157e-012}, blow = {-43982.9607, -6.259737448}, ahigh = {-128469.8945, -124.7550173, 7.09289385, -3.70731177e-005, 8.175173009999999e-009, -9.35643521e-013, 4.32854434e-017}, bhigh = {-47233.0972, -8.337392707999999}, R = 100.7364182280103);
          constant IdealGases.Common.DataRecord SiF2(name = "SiF2", MM = 0.0660823064, Hf = -8971207.881448884, H0 = 169553.2527599551, Tlimit = 1000, alow = {40532.3665, -399.763756, 4.05530388, 0.01153423049, -1.757586754e-005, 1.260369111e-008, -3.51710522e-012}, blow = {-70477.7046, 3.87985442}, ahigh = {-167520.0202, -184.621209, 7.13698635, -5.45076986e-005, 1.199012961e-008, -1.369470187e-012, 6.32488999e-017}, bhigh = {-72874.0102, -11.17933348}, R = 125.8199426283947);
          constant IdealGases.Common.DataRecord SiF3(name = "SiF3", MM = 0.08508070960000001, Hf = -11711662.74569953, H0 = 153014.1680905774, Tlimit = 1000, alow = {57553.6998, -717.959577, 5.37939694, 0.01813065142, -2.738373356e-005, 1.94651947e-008, -5.39079056e-012}, blow = {-117763.1169, -3.44290204}, ahigh = {-290328.3702, -342.787694, 10.25536335, -0.000101979669, 2.250326132e-008, -2.577117181e-012, 1.192923177e-016}, bhigh = {-121808.8967, -27.25261142}, R = 97.7245257954454);
          constant IdealGases.Common.DataRecord SiF4(name = "SiF4", MM = 0.1040791128, Hf = -15524536.638825, H0 = 147634.9344899489, Tlimit = 1000, alow = {10696.70784, -95.69638640000001, 2.513988593, 0.0330978103, -4.57304408e-005, 3.091834452e-008, -8.26930405e-012}, blow = {-195625.2529, 11.33722022}, ahigh = {-384589.232, -656.997991, 13.4892572, -5.10047136e-005, 5.34632958e-008, -4.95959811e-012, 2.299314375e-016}, bhigh = {-195728.4479, -46.7389465}, R = 79.88607681521283);
          constant IdealGases.Common.DataRecord SiH(name = "SiH", MM = 0.02909344, Hf = 12670767.36198951, H0 = 314335.809034614, Tlimit = 1000, alow = {-6426.676299999999, 74.17251210000001, 3.9734916, -0.00414940888, 1.022918384e-005, -8.592386360000002e-009, 2.567093743e-012}, blow = {42817.45800000001, 2.24693715}, ahigh = {404208.649, -2364.796524, 7.62749914, -0.002496591233, 1.10843641e-006, -1.943991955e-010, 1.136251507e-014}, bhigh = {57047.3768, -24.48054429}, R = 285.7851116952825);
          constant IdealGases.Common.DataRecord SiHplus(name = "SiHplus", MM = 0.0290928914, Hf = 39448508.27030551, H0 = 297469.951714734, Tlimit = 1000, alow = {-43071.5717, 388.495824, 2.546166863, -0.0006686325980000001, 5.186277020000001e-006, -4.81856974e-009, 1.44570264e-012}, blow = {134907.9707, 9.027950089999999}, ahigh = {171169.2033, -1045.807287, 4.83864824, 0.0001277940891, -6.883872450000001e-008, 1.41884348e-011, -7.85531696e-016}, bhigh = {143172.145, -7.55329914}, R = 285.7905006994252);
          constant IdealGases.Common.DataRecord SiHBr3(name = "SiHBr3", MM = 0.26880544, Hf = -1126919.157588478, H0 = 66306.05764526196, Tlimit = 1000, alow = {132281.1122, -2205.365507, 17.75084194, -0.01047794569, 1.286821723e-005, -7.743124260000001e-009, 1.819060891e-012}, blow = {-28349.92682, -63.3119954}, ahigh = {219997.3526, -2024.458552, 14.35781007, -0.000503701584, 1.054049755e-007, -1.160828765e-011, 5.21725768e-016}, bhigh = {-28420.86214, -45.3643717}, R = 30.9311894878318);
          constant IdealGases.Common.DataRecord SiHCL(name = "SiHCL", MM = 0.06454644000000001, Hf = 851261.7117225983, H0 = 165236.2701955367, Tlimit = 1000, alow = {59096.73050000001, -878.006403, 7.99968793, -0.00466314127, 8.594986e-006, -6.608073550000001e-009, 1.868076396e-012}, blow = {9567.57034, -16.90014311}, ahigh = {101215.289, -1163.239598, 7.37636655, 0.0001874151313, -1.661351093e-007, 3.96441404e-011, -2.64058008e-015}, bhigh = {11410.64087, -15.07678798}, R = 128.8137967020334);
          constant IdealGases.Common.DataRecord SiHCL3(name = "SiHCL3", MM = 0.13545244, Hf = -3663440.835764937, H0 = 119241.912511875, Tlimit = 1000, alow = {165956.5586, -2552.498122, 17.50117375, -0.007885728660000001, 8.092567490000001e-006, -3.9804206e-009, 7.05917332e-013}, blow = {-49513.16600000001, -67.58519052999999}, ahigh = {172864.4072, -2116.727327, 14.41802307, -0.000525524962, 1.098826178e-007, -1.209327059e-011, 5.432187049999999e-016}, bhigh = {-51303.5194, -50.36360893000001}, R = 61.3829621673851);
          constant IdealGases.Common.DataRecord SiHF(name = "SiHF", MM = 0.0480918432, Hf = -3382219.482076328, H0 = 212722.7887160707, Tlimit = 1000, alow = {21925.98671, -61.7522908, 2.127372739, 0.01123538312, -1.322503985e-005, 8.30730739e-009, -2.010545971e-012}, blow = {-20169.94845, 13.67184876}, ahigh = {4049752.24, -9295.526819999999, 10.71754366, 0.00186812422, -1.129820372e-006, 1.9602501e-010, -1.144682011e-014}, bhigh = {41332.252, -47.363176}, R = 172.8873639844189);
          constant IdealGases.Common.DataRecord SiHF3(name = "SiHF3", MM = 0.0860886496, Hf = -13948505.47173643, H0 = 157333.5632854439, Tlimit = 1000, alow = {83258.7939, -817.8594459999999, 4.04171724, 0.02726104628, -3.78783148e-005, 2.630589305e-008, -7.2895825e-012}, blow = {-141614.6961, 1.382806532}, ahigh = {78201.008, -2592.742261, 14.75104214, -0.00065310472, 1.372428937e-007, -1.516371051e-011, 6.83249715e-016}, bhigh = {-133661.1887, -58.5145095}, R = 96.58035105245745);
          constant IdealGases.Common.DataRecord SiHI3(name = "SiHI3", MM = 0.40980685, Hf = -181731.9549441402, H0 = 46693.76317160145, Tlimit = 1000, alow = {111384.0054, -1974.457175, 17.91396249, -0.01219208519, 1.613169233e-005, -1.038096942e-008, 2.613287761e-012}, blow = {-2256.377425, -59.9545513}, ahigh = {227942.4252, -1898.603142, 14.28134804, -0.000477602695, 1.003097496e-007, -1.107859145e-011, 4.990338200000001e-016}, bhigh = {-1614.547674, -41.2294307}, R = 20.28875798440168);
          constant IdealGases.Common.DataRecord SiH2(name = "SiH2", MM = 0.03010138, Hf = 9080398.639530813, H0 = 332812.05047742, Tlimit = 1000, alow = {-20638.63564, 330.58622, 2.099271145, 0.00354253937, 3.37887667e-006, -5.38384562e-009, 2.081191273e-012}, blow = {30117.84298, 12.8233357}, ahigh = {4624039.37, -11434.3611, 12.6488087, 0.00091148995, -8.766611539999999e-007, 1.646297357e-010, -9.965090370000001e-015}, bhigh = {107247.5101, -66.0607807}, R = 276.2156419406685);
          constant IdealGases.Common.DataRecord SiH2Br2(name = "SiH2Br2", MM = 0.18990938, Hf = -1002436.003950937, H0 = 75152.80182579713, Tlimit = 1000, alow = {165152.8003, -2468.145554, 16.08639427, -0.0094138409, 1.407423872e-005, -9.240751449999999e-009, 2.255641666e-012}, blow = {-12764.82614, -59.4537432}, ahigh = {566585.144, -3973.91962, 15.67387912, -0.0009943802290000002, 2.084856022e-007, -2.299521351e-011, 1.034736283e-015}, bhigh = {-2990.69018, -61.9047652}, R = 43.7812603042567);
          constant IdealGases.Common.DataRecord SiH2CL2(name = "SiH2CL2", MM = 0.10100738, Hf = -3172976.073629472, H0 = 132514.5548770793, Tlimit = 1000, alow = {189428.1754, -2650.960252, 15.49874174, -0.006533130749999999, 9.306098260000001e-006, -5.63969867e-009, 1.213664264e-012}, blow = {-27209.12007, -60.06331649}, ahigh = {540020.458, -4062.33729, 15.73118788, -0.001015038389, 2.127060243e-007, -2.345079226e-011, 1.054871984e-015}, bhigh = {-18230.98225, -65.44540499}, R = 82.31549021467541);
          constant IdealGases.Common.DataRecord SiH2F2(name = "SiH2F2", MM = 0.0680981864, Hf = -11612291.63072102, H0 = 176042.9555286953, Tlimit = 1000, alow = {127218.8427, -1241.72431, 4.89943589, 0.02123882008, -2.7257087e-005, 1.860857446e-008, -5.22660433e-012}, blow = {-89804.66919999999, -5.1118155}, ahigh = {472653.4080000001, -4388.51769, 15.96032597, -0.001102993658, 2.315838331e-007, -2.556974559e-011, 1.151499711e-015}, bhigh = {-73161.8446, -70.9898927}, R = 122.0953514262753);
          constant IdealGases.Common.DataRecord SiH2I2(name = "SiH2I2", MM = 0.28391032, Hf = -134105.7274705618, H0 = 52542.17599416605, Tlimit = 1000, alow = {153996.3319, -2414.436989, 16.84409354, -0.0121203758, 1.808453226e-005, -1.207740133e-008, 3.040799276e-012}, blow = {5073.07325, -60.9872082}, ahigh = {594239.083, -3924.81448, 15.63850289, -0.0009805706340000001, 2.054871384e-007, -2.265587199e-011, 1.019168663e-015}, bhigh = {15136.27352, -59.4081252}, R = 29.28555749576134);
          constant IdealGases.Common.DataRecord SiH3(name = "SiH3", MM = 0.03110932, Hf = 6569007.80859241, H0 = 330350.8080536636, Tlimit = 1000, alow = {4341.14282, 227.7185085, 0.650825035, 0.01221438558, -4.34760427e-006, -1.774916828e-009, 1.184191367e-012}, blow = {22599.93826, 19.68347482}, ahigh = {605632.122, -4721.254059999999, 13.29129523, -0.001256824868, 2.68828594e-007, -3.010741582e-011, 1.370945857e-015}, bhigh = {49744.2064, -61.405031}, R = 267.2662726154092);
          constant IdealGases.Common.DataRecord SiH3Br(name = "SiH3Br", MM = 0.11101332, Hf = -704789.2991579749, H0 = 106017.7103071956, Tlimit = 1000, alow = {152106.5525, -1987.744479, 11.31460743, -0.001806099407, 7.76926247e-006, -6.233478169999999e-009, 1.585331846e-012}, blow = {-924.886888, -38.4646369}, ahigh = {927928.578, -5917.93579, 16.99209298, -0.001487507364, 3.123451161e-007, -3.44907147e-011, 1.553431675e-015}, bhigh = {22415.60128, -79.4861159}, R = 74.89616561327956);
          constant IdealGases.Common.DataRecord SiH3CL(name = "SiH3CL", MM = 0.06656231999999999, Hf = -2130905.292964548, H0 = 171879.615974924, Tlimit = 1000, alow = {163338.8301, -2030.302118, 10.68016138, 0.0004942370140000001, 4.30517476e-006, -3.74832727e-009, 8.89350421e-013}, blow = {-8180.71498, -36.88956824}, ahigh = {908778.094, -5958.94956, 17.02076016, -0.001498450667, 3.146813781e-007, -3.47516846e-011, 1.565281681e-015}, bhigh = {14937.93206, -81.27277864}, R = 124.9125931908624);
          constant IdealGases.Common.DataRecord SiH3F(name = "SiH3F", MM = 0.0501077232, Hf = -7515009.183255008, H0 = 218162.4169265787, Tlimit = 1000, alow = {126783.629, -1177.593516, 4.49918896, 0.01658543011, -1.678598921e-005, 1.019153931e-008, -2.803920232e-012}, blow = {-40103.9297, -4.48108092}, ahigh = {862431.317, -6103.37057, 17.12733597, -0.001540724131, 3.23962872e-007, -3.58105689e-011, 1.61415138e-015}, bhigh = {-12614.87616, -83.99227209999999}, R = 165.9319455967618);
          constant IdealGases.Common.DataRecord SiH3I(name = "SiH3I", MM = 0.15801379, Hf = -13239.35081868488, H0 = 76437.81596530277, Tlimit = 1000, alow = {145938.4799, -1982.982746, 11.96516737, -0.00393750595, 1.087991733e-005, -8.43313592e-009, 2.196078188e-012}, blow = {8063.20831, -40.6455332}, ahigh = {936114.918, -5859.72029, 16.95376698, -0.00147349713, 3.094475885e-007, -3.41746247e-011, 1.539334807e-015}, bhigh = {31274.99231, -78.01700080000001}, R = 52.61864803065607);
          constant IdealGases.Common.DataRecord SiH4(name = "SiH4", MM = 0.03211726, Hf = 1080415.950800286, H0 = 328016.8046713823, Tlimit = 1000, alow = {78729.9329, -552.608705, 2.498944303, 0.01442118274, -8.467107309999999e-006, 2.726164641e-009, -5.43675437e-013}, blow = {6269.66906, 4.96546183}, ahigh = {1290378.74, -7813.39978, 18.28851664, -0.001975620946, 4.15650215e-007, -4.59674561e-011, 2.072777131e-015}, bhigh = {47668.8795, -98.0169746}, R = 258.8786216507884);
          constant IdealGases.Common.DataRecord SiI(name = "SiI", MM = 0.15498997, Hf = 1696583.462787947, H0 = 64140.40856966423, Tlimit = 1000, alow = {94920.8134, -1573.960062, 12.37908109, -0.0159386482, 1.692049005e-005, -9.215558319999999e-009, 2.046186848e-012}, blow = {37797.3567, -40.6640785}, ahigh = {852940.2790000001, -2442.039471, 7.72297752, -0.001985601243, 6.542866629999999e-007, -9.49415829e-011, 4.89004137e-015}, bhigh = {45781.6094, -17.33975081}, R = 53.64522620399243);
          constant IdealGases.Common.DataRecord SiI2(name = "SiI2", MM = 0.28189444, Hf = 328016.4021681308, H0 = 48927.9426724415, Tlimit = 1000, alow = {11633.52403, -451.448217, 8.707978130000001, -0.00358066493, 4.25371443e-006, -2.673478266e-009, 6.89155898e-013}, blow = {11262.47142, -11.55818335}, ahigh = {10021.68907, 14.33017148, 6.70133055, 0.00039607699, -2.073866165e-007, 4.5534571e-011, -3.01679828e-015}, bhigh = {9007.368100000002, 0.2887336913}, R = 29.49498400890774);
          constant IdealGases.Common.DataRecord SiN(name = "SiN", MM = 0.0420922, Hf = 9590100.707494499, H0 = 207547.6454069876, Tlimit = 1000, alow = {-14646.72152, 137.4993497, 3.67850858, -0.0061584992, 2.309417067e-005, -2.294461481e-008, 7.39566568e-012}, blow = {46732.1299, 6.494564115}, ahigh = {-2932685.132, 5853.68859, 1.321451677, 0.001258329284, -3.77388636e-007, 6.88776104e-011, -4.18984259e-015}, bhigh = {6527.14881, 25.53145732}, R = 197.5299936805394);
          constant IdealGases.Common.DataRecord SiO(name = "SiO", MM = 0.0440849, Hf = -2242092.371764482, H0 = 197689.1180426858, Tlimit = 1000, alow = {-47227.7105, 806.3137640000001, -1.636976133, 0.01454275546, -1.723202046e-005, 1.04239734e-008, -2.559365273e-012}, blow = {-16665.85903, 33.557957}, ahigh = {-176513.4162, -31.9917709, 4.47744193, 4.59176471e-006, 3.55814315e-008, -1.327012559e-011, 1.613253297e-015}, bhigh = {-13508.4236, -0.838695733}, R = 188.601357834542);
          constant IdealGases.Common.DataRecord SiO2(name = "SiO2", MM = 0.0600843, Hf = -5360359.977564856, H0 = 175462.1922865041, Tlimit = 1000, alow = {-33629.4878, 473.407892, 0.2309770671, 0.01850230806, -2.242786671e-005, 1.364981554e-008, -3.35193503e-012}, blow = {-42264.8749, 22.95803206}, ahigh = {-146403.1193, -626.144106, 7.96456371, -0.0001854119096, 4.09521467e-008, -4.69720676e-012, 2.17805428e-016}, bhigh = {-37918.3477, -20.45285414}, R = 138.3801092797952);
          constant IdealGases.Common.DataRecord SiS(name = "SiS", MM = 0.0601505, Hf = 1798728.306497868, H0 = 148495.9726020565, Tlimit = 1000, alow = {35994.4929, -423.9723299999999, 4.65401442, 0.001588470782, -3.31025436e-006, 2.706096479e-009, -8.113517820000001e-013}, blow = {14115.15571, -1.183201858}, ahigh = {-2102323.897, 6228.83618, -3.004120882, 0.004495499930000001, -1.368821364e-006, 1.998097253e-010, -9.882035800000001e-015}, bhigh = {-27955.38166, 54.05828786}, R = 138.2278119051379);
          constant IdealGases.Common.DataRecord SiS2(name = "SiS2", MM = 0.09221550000000001, Hf = 76155.49446676533, H0 = 132333.6857686614, Tlimit = 1000, alow = {43977.4566, -743.001312, 7.94158836, 0.00203808373, -4.6463376e-006, 3.84801436e-009, -1.157260149e-012}, blow = {2801.077853, -17.36888325}, ahigh = {-126648.9071, -97.7423522, 7.57280368, -2.905413928e-005, 6.40558784e-009, -7.329273340000001e-013, 3.38981978e-017}, bhigh = {-1244.672263, -13.52710463}, R = 90.1634974597546);
          constant IdealGases.Common.DataRecord Si2(name = "Si2", MM = 0.056171, Hf = 10329093.73164088, H0 = 183299.6386035499, Tlimit = 1000, alow = {12375.96221, -102.4904376, 4.35484852, 0.001281063335, -2.531991623e-006, 2.265694244e-009, -7.001290140000001e-013}, blow = {69069.4285, 3.2511252}, ahigh = {1370060.657, -4207.06004, 9.337432890000001, -0.002749217168, 9.586345959999999e-007, -1.372449748e-010, 6.765028100000001e-015}, bhigh = {95108.84539999999, -31.6838519}, R = 148.0207224368446);
          constant IdealGases.Common.DataRecord Si2C(name = "Si2C", MM = 0.06818170000000001, Hf = 8126724.678322773, H0 = 169874.7024494842, Tlimit = 1000, alow = {-4553.3662, 131.4796415, 2.469106923, 0.0127652068, -1.656910776e-005, 1.065289663e-008, -2.739192976e-012}, blow = {64700.4992, 14.6883898}, ahigh = {-125382.9442, -341.427779, 7.25436533, -0.0001017635503, 2.250902158e-008, -2.584074852e-012, 1.198884876e-016}, bhigh = {66080.0938, -11.46216579}, R = 121.9458007060545);
          constant IdealGases.Common.DataRecord Si2F6(name = "Si2F6", MM = 0.1701614192, Hf = -14006054.31715863, H0 = 156876.2891465118, Tlimit = 1000, alow = {15807.6745, -297.5292881, 7.5125915, 0.0455279623, -6.42187684e-005, 4.357408730000001e-008, -1.166762282e-011}, blow = {-288670.7444, -7.72178477}, ahigh = {-918630.314, 157.3948683, 21.41256129, -0.0002681898667, 7.21228011e-008, -9.18425266e-012, 4.53723836e-016}, bhigh = {-296617.6753, -78.4902247}, R = 48.8622628977227);
          constant IdealGases.Common.DataRecord Si2N(name = "Si2N", MM = 0.0701777, Hf = 5663907.480581439, H0 = 168460.7503523199, Tlimit = 1000, alow = {23182.71938, -367.484379, 5.4096586, 0.008176389399999999, -1.220312934e-005, 8.5806091e-009, -2.354927059e-012}, blow = {48092.7803, -3.042205654}, ahigh = {-280502.6986, 250.2366876, 7.0924561, 0.0002853120795, -9.79210181e-008, 1.552154244e-011, -8.03626045e-016}, bhigh = {43420.1479, -10.0957091}, R = 118.4774080655251);
          constant IdealGases.Common.DataRecord Si3(name = "Si3", MM = 0.0842565, Hf = 7451856.901247975, H0 = 148591.4202465092, Tlimit = 1000, alow = {-11142.08177, 157.5785843, 2.486135003, 0.01631637255, -2.208240021e-005, 1.372008287e-008, -3.2623307e-012}, blow = {73282.53850000001, 15.88081347}, ahigh = {-1699395.561, 4697.81538, 2.618198124, 0.001959082075, -2.581160603e-007, 6.10344486e-012, 6.08630924e-016}, bhigh = {42779.1681, 25.86540384}, R = 98.6804816245631);
          constant IdealGases.Common.DataRecord Sn(name = "Sn", MM = 0.11871, Hf = 2537275.713924691, H0 = 52352.05964114227, Tlimit = 1000, alow = {-124869.2263, 1618.84119, -4.60239735, 0.01045433308, 2.99826555e-006, -1.068699386e-008, 4.32342131e-012}, blow = {27483.64008, 48.0506723}, ahigh = {-5145695.64, 11405.75108, -4.17963206, 0.002236390679, -3.60321977e-007, 2.440237836e-011, -2.937628285e-016}, bhigh = {-42150.1357, 59.81450930000001}, R = 70.0401988038076);
          constant IdealGases.Common.DataRecord Snplus(name = "Snplus", MM = 0.1187094514, Hf = 8558290.144705359, H0 = 52206.6939650603, Tlimit = 1000, alow = {-5571.29778, 122.2323189, 1.566361415, 0.00339706141, -6.31292229e-006, 5.58545208e-009, -1.68902134e-012}, blow = {120902.412, 11.62634765}, ahigh = {4622916.850000001, -11859.58712, 12.37026473, -0.002773624217, 3.09851349e-007, -5.362951439999999e-012, -8.663474691e-016}, bhigh = {199432.2977, -68.3710828}, R = 70.04052248530566);
          constant IdealGases.Common.DataRecord Snminus(name = "Snminus", MM = 0.1187105486, Hf = 1512047.169496444, H0 = 54674.57674608034, Tlimit = 1000, alow = {272279.6, -3369.9358, 17.43906405, -0.02810527618, 2.790684767e-005, -1.442400675e-008, 3.070159757e-012}, blow = {37532.2775, -80.0785697}, ahigh = {-64477.1322, 743.05551, 1.921380256, 0.0002361623997, -5.28066836e-008, 6.10233938e-012, -2.843422486e-016}, bhigh = {16457.39414, 12.65436865}, R = 70.03987512530121);
          constant IdealGases.Common.DataRecord SnBr(name = "SnBr", MM = 0.198614, Hf = 380859.4560302899, H0 = 50279.03370356571, Tlimit = 1000, alow = {25641.93542, -525.912233, 7.68741069, -0.009654205420000001, 1.482863248e-005, -9.94223698e-009, 2.475051091e-012}, blow = {10204.83889, -11.26550396}, ahigh = {1815578.541, -6728.26458, 13.32464516, -0.0046081023, 1.236439537e-006, -1.528863587e-010, 6.971513500000001e-015}, bhigh = {49255.8309, -56.4096278}, R = 41.86246689558642);
          constant IdealGases.Common.DataRecord SnBr2(name = "SnBr2", MM = 0.278518, Hf = -427170.2080296426, H0 = 52403.82309222384, Tlimit = 1000, alow = {-4995.75306, -136.6498896, 7.54356385, -0.001181772507, 1.442445852e-006, -9.254217260000001e-010, 2.423731634e-013}, blow = {-15755.06969, -3.59558402}, ahigh = {-19408.94091, -2.335607932, 7.00196926, -8.622241869999999e-007, 2.039869794e-010, -2.464780359e-014, 1.189850894e-018}, bhigh = {-16448.28122, -0.434935475}, R = 29.85254812974386);
          constant IdealGases.Common.DataRecord SnBr3(name = "SnBr3", MM = 0.358422, Hf = -442820.0445285167, H0 = 53792.60759663191, Tlimit = 1000, alow = {-8050.41647, -289.7962523, 11.14443355, -0.002475383807, 3.01005135e-006, -1.92569532e-009, 5.032605089999999e-013}, blow = {-20700.66649, -18.95965383}, ahigh = {-38930.0453, -4.99084417, 10.00418437, -1.82499691e-006, 4.30564666e-010, -5.19188793e-014, 2.502455595e-018}, bhigh = {-22172.99473, -12.29854098}, R = 23.19743765728666);
          constant IdealGases.Common.DataRecord SnBr4(name = "SnBr4", MM = 0.4383260000000001, Hf = -739670.8887905348, H0 = 57133.31629882781, Tlimit = 1000, alow = {-5832.84092, -427.019847, 14.67466525, -0.00360426323, 4.366686289999999e-006, -2.78587513e-009, 7.26508787e-013}, blow = {-40829.1645, -34.4710436}, ahigh = {-51768.1292, -7.42612695, 13.00619502, -2.692587978e-006, 6.3367598e-010, -7.62708719e-014, 3.671112e-018}, bhigh = {-43001.6634, -24.71455118}, R = 18.96869453329257);
          constant IdealGases.Common.DataRecord SnCL(name = "SnCL", MM = 0.154163, Hf = 224819.2886749739, H0 = 62557.91597205554, Tlimit = 1000, alow = {33515.4973, -683.220862, 8.39391773, -0.01164434373, 1.812159499e-005, -1.26199917e-008, 3.29521347e-012}, blow = {6051.79089, -16.67834578}, ahigh = {774160.568, -3602.04848, 9.749059819999999, -0.002596198909, 6.36771216e-007, -6.75429557e-011, 2.484167602e-015}, bhigh = {24424.47662, -32.25454098}, R = 53.93299300091461);
          constant IdealGases.Common.DataRecord SnCL2(name = "SnCL2", MM = 0.189616, Hf = -1068729.869842207, H0 = 72154.86562315417, Tlimit = 1000, alow = {9524.6659, -432.984564, 8.64472028, -0.00345840961, 4.11785203e-006, -2.592639269e-009, 6.6923636e-013}, blow = {-24329.21127, -12.99143851}, ahigh = {-39035.88490000001, -7.94796268, 7.00648762, -2.776594714e-006, 6.461341420000001e-010, -7.1190674e-015, 3.68813118e-018}, bhigh = {-26545.29653, -3.368081174}, R = 43.84900008438107);
          constant IdealGases.Common.DataRecord SnCL3(name = "SnCL3", MM = 0.225069, Hf = -1299034.76267278, H0 = 77994.05959950059, Tlimit = 1000, alow = {17600.76813, -814.280548, 13.06326984, -0.00639559619, 7.57462457e-006, -4.74971146e-009, 1.222182656e-012}, blow = {-34134.3096, -34.5767967}, ahigh = {-74829.66399999999, -15.28523914, 10.01240305, -5.28558124e-006, 1.226095111e-009, -1.459895108e-013, 6.96887978e-018}, bhigh = {-38308.7827, -16.63063137}, R = 36.94188004567489);
          constant IdealGases.Common.DataRecord SnCL4(name = "SnCL4", MM = 0.260522, Hf = -1836566.466555608, H0 = 86265.38641650225, Tlimit = 1000, alow = {42459.6787, -1280.574318, 17.68445328, -0.00957477952, 1.115620323e-005, -6.90722927e-009, 1.759652856e-012}, blow = {-55040.09039999999, -58.27352999}, ahigh = {-107833.6862, -25.54305872, 13.02040391, -8.5951259e-006, 1.976570888e-009, -2.337929994e-013, 1.110287624e-017}, bhigh = {-61635.6203, -30.72702622}, R = 31.91466363685217);
          constant IdealGases.Common.DataRecord SnF(name = "SnF", MM = 0.1377084032, Hf = -689987.5954701362, H0 = 66343.88162014502, Tlimit = 1000, alow = {60985.03520000001, -986.0306290000001, 8.994731209999999, -0.01223012156, 1.840583578e-005, -1.267926607e-008, 3.29133102e-012}, blow = {-7882.61448, -22.21921734}, ahigh = {25897.15476, -1484.478214, 7.37851893, -0.00132424444, 2.847634043e-007, -2.360371345e-011, 6.11678371e-016}, bhigh = {-4776.68897, -16.81202516}, R = 60.3773757213968);
          constant IdealGases.Common.DataRecord SnF2(name = "SnF2", MM = 0.1567068064, Hf = -3260591.947077035, H0 = 78127.10424810239, Tlimit = 1000, alow = {70423.9185, -1137.505955, 10.08659352, -0.00472285593, 4.13570292e-006, -1.921143972e-009, 3.64076028e-013}, blow = {-57566.7082, -25.71566624}, ahigh = {-102775.8772, -44.5175162, 7.03326876, -1.329882797e-005, 2.934271938e-009, -3.35841592e-013, 1.553358669e-017}, bhigh = {-63621.2588, -6.81008452}, R = 53.05750395280852);
          constant IdealGases.Common.DataRecord SnF3(name = "SnF3", MM = 0.1757052096, Hf = -3680200.948350254, H0 = 84230.4051979572, Tlimit = 1000, alow = {109555.5529, -1950.557979, 15.41407832, -0.008546518770000001, 7.80055182e-006, -3.82594222e-009, 7.78774776e-013}, blow = {-70567.9492, -54.9463948}, ahigh = {-182931.9689, -75.5735868, 10.05671455, -2.275132668e-005, 5.03469582e-009, -5.77653096e-013, 2.67721417e-017}, bhigh = {-80918.97590000001, -21.91673972}, R = 47.32057756812237);
          constant IdealGases.Common.DataRecord SnF4(name = "SnF4", MM = 0.1947036128, Hf = -5263213.672632991, H0 = 96567.20144845716, Tlimit = 1000, alow = {139044.1119, -2168.784371, 16.91802917, -0.002318608529, -2.045847976e-006, 3.4300879e-009, -1.28922886e-012}, blow = {-115356.3396, -63.7060475}, ahigh = {-263569.2764, -144.5469969, 13.10690681, -4.23775579e-005, 9.28687195e-009, -1.057062694e-012, 4.86710868e-017}, bhigh = {-127153.9967, -38.039219}, R = 42.70322404618472);
          constant IdealGases.Common.DataRecord SnI(name = "SnI", MM = 0.24561447, Hf = 703236.3443407874, H0 = 41435.35191554471, Tlimit = 1000, alow = {26702.12235, -500.602635, 7.49067315, -0.008746346550000001, 1.289688397e-005, -8.211344929999999e-009, 1.936120475e-012}, blow = {21772.49172, -9.354806849999999}, ahigh = {-55549.91850000001, -1530.457453, 7.93260139, -0.002108885464, 7.8098036e-007, -1.348104887e-010, 8.207184700000001e-015}, bhigh = {27448.64953, -16.26822892}, R = 33.85171891542058);
          constant IdealGases.Common.DataRecord SnI2(name = "SnI2", MM = 0.37251894, Hf = -21654.5687583026, H0 = 39953.03701873522, Tlimit = 1000, alow = {-5739.25489, -94.3502445, 7.37795666, -0.0008258348939999999, 1.01168704e-006, -6.5084516e-010, 1.7081866e-013}, blow = {-2622.6445, -0.960581496}, ahigh = {-15594.5339, -1.594789414, 7.00135116, -5.93532424e-007, 1.407432564e-010, -1.703457321e-014, 8.23361171e-019}, bhigh = {-3100.608319, 1.235038509}, R = 22.31959534728624);
          constant IdealGases.Common.DataRecord SnI3(name = "SnI3", MM = 0.49942341, Hf = -16053.59268200904, H0 = 40665.61477364467, Tlimit = 1000, alow = {-10699.78099, -138.769354, 10.55637633, -0.001216435658, 1.490873499e-006, -9.594499629999999e-010, 2.518810438e-013}, blow = {-3314.22988, -12.49558209}, ahigh = {-25175.3487, -2.34828289, 10.0019913, -8.752764839999999e-007, 2.07646498e-010, -2.514060336e-014, 1.215482266e-018}, bhigh = {-4017.05561, -9.26390846}, R = 16.64814230474298);
          constant IdealGases.Common.DataRecord SnI4(name = "SnI4", MM = 0.6263278800000001, Hf = -189763.2961828236, H0 = 42702.80447997939, Tlimit = 1000, alow = {-12831.46793, -145.9885093, 13.58634885, -0.001283554646, 1.574557147e-006, -1.013990168e-009, 2.66337043e-013}, blow = {-17511.78045, -23.67973168}, ahigh = {-28018.40706, -2.474699366, 13.00210215, -9.250962339999999e-007, 2.196515817e-010, -2.661071614e-014, 1.287167546e-018}, bhigh = {-18250.83832, -20.27484281}, R = 13.27495113262402);
          constant IdealGases.Common.DataRecord SnO(name = "SnO", MM = 0.1347094, Hf = 162654.2319986579, H0 = 65875.92996479831, Tlimit = 1000, alow = {25477.88022, -229.3583066, 3.42159353, 0.00471330004, -7.36597132e-006, 5.37973887e-009, -1.519277855e-012}, blow = {2853.047679, 6.67396527}, ahigh = {-2555955.548, 7870.0535, -5.43976873, 0.00630781918, -2.06887137e-006, 3.30707488e-010, -1.848169716e-014}, bhigh = {-48416.6391, 71.75952460000001}, R = 61.72154281735352);
          constant IdealGases.Common.DataRecord SnO2(name = "SnO2", MM = 0.1507088, Hf = 77501.64555752551, H0 = 78268.02416315438, Tlimit = 1000, alow = {47076.7408, -611.456867, 6.11743391, 0.00751401466, -1.238371787e-005, 9.22091599e-009, -2.632417597e-012}, blow = {2981.068066, -8.14492641}, ahigh = {-157355.8926, -142.7316788, 7.60570076, -4.19800853e-005, 9.218742430000002e-009, -1.051372983e-012, 4.84957927e-017}, bhigh = {-523.453187, -14.28360169}, R = 55.16912084762137);
          constant IdealGases.Common.DataRecord SnS(name = "SnS", MM = 0.150775, Hf = 736853.0061349694, H0 = 61688.68844304427, Tlimit = 1000, alow = {27248.77813, -492.412638, 6.22165473, -0.00340940696, 3.98491368e-006, -2.470702874e-009, 6.33934146e-013}, blow = {14524.95307, -6.791285229}, ahigh = {-1797985.129, 6141.73632, -4.01950488, 0.005993829179999999, -2.188735204e-006, 3.899542e-010, -2.40656363e-014}, bhigh = {-26113.49815, 62.56820705}, R = 55.14489802686122);
          constant IdealGases.Common.DataRecord SnS2(name = "SnS2", MM = 0.18284, Hf = 818451.0774447605, H0 = 74573.91708597681, Tlimit = 1000, alow = {38771.4541, -848.695394, 10.4208399, -0.005686902670000001, 6.37382977e-006, -3.8251581e-009, 9.50248096e-013}, blow = {20060.27952, -27.23724769}, ahigh = {-67640.84050000001, -19.70189679, 7.51533387, -6.33255918e-006, 1.434207625e-009, -1.676383081e-013, 7.886911900000001e-018}, bhigh = {15650.53629, -9.922744838}, R = 45.47403194049442);
          constant IdealGases.Common.DataRecord Sn2(name = "Sn2", MM = 0.23742, Hf = 1774676.678460113, H0 = 47903.02838850982, Tlimit = 1000, alow = {-132966.6654, 1639.179787, -1.902868972, 0.01290431677, -1.120542242e-005, 4.50299386e-009, -6.05623777e-013}, blow = {40974.5271, 44.35408029999999}, ahigh = {-4275783.04, 12364.21656, -8.43833617, 0.00708805629, -1.712012737e-006, 1.805262952e-010, -7.058656809999999e-015}, bhigh = {-30014.1872, 100.5138538}, R = 35.0200994019038);
          constant IdealGases.Common.DataRecord Sr(name = "Sr", MM = 0.08762, Hf = 1831773.567678612, H0 = 70730.74640493038, Tlimit = 1000, alow = {4.19064984, -0.0630443758, 2.500373027, -1.115455943e-006, 1.785248643e-009, -1.456209589e-012, 4.750132981e-016}, blow = {18558.52648, 5.55577284}, ahigh = {14894144.1, -43753.3505, 51.3726628, -0.02592566025, 6.58299e-006, -6.949611799999999e-010, 2.417779662e-014}, bhigh = {297754.5522, -345.489077}, R = 94.89239899566309);
          constant IdealGases.Common.DataRecord Srplus(name = "Srplus", MM = 0.08761945140000001, Hf = 8173599.772162005, H0 = 70731.18926193139, Tlimit = 1000, alow = {11.27287678, -0.134695187, 2.500651495, -1.635163061e-006, 2.249493149e-009, -1.610827539e-012, 4.698612333e-016}, blow = {85389.81180000001, 6.24725924}, ahigh = {3145095.058, -9514.756889999999, 13.50086948, -0.00605971712, 1.594068746e-006, -1.718800946e-010, 6.322256169e-015}, bhigh = {145799.1907, -72.3641693}, R = 94.8929931328011);
          constant IdealGases.Common.DataRecord SrBr(name = "SrBr", MM = 0.167524, Hf = -381543.4087056183, H0 = 60332.38222583033, Tlimit = 1000, alow = {-754.9812420000001, -73.5953201, 4.82803581, -0.000716058465, 1.028527433e-006, -6.88837072e-010, 1.880257878e-013}, blow = {-8686.168450000001, 4.15115309}, ahigh = {3009976.114, -9193.19224, 15.26033745, -0.00600925395, 1.692611858e-006, -2.039438429e-010, 8.807775840000001e-015}, bhigh = {49228.06129999999, -70.6825383}, R = 49.63152742293642);
          constant IdealGases.Common.DataRecord SrBr2(name = "SrBr2", MM = 0.247428, Hf = -1643814.972436426, H0 = 66017.78699257966, Tlimit = 1000, alow = {-4632.45866, -72.4493864, 7.80021716, -0.000677212531, 8.53778024e-007, -5.63235174e-010, 1.510568795e-013}, blow = {-50822.3833, -5.28507476}, ahigh = {-11861.73059, -1.404383013, 7.50121557, -5.415983190000001e-007, 1.296969275e-010, -1.580911652e-014, 7.68158227e-019}, bhigh = {-51185.75210000001, -3.55203711}, R = 33.60360185589343);
          constant IdealGases.Common.DataRecord SrCL(name = "SrCL", MM = 0.123073, Hf = -1038959.658089102, H0 = 79628.49690833896, Tlimit = 1000, alow = {3555.34632, -172.9385697, 5.21345102, -0.001542910668, 2.025835568e-006, -1.327076633e-009, 3.54717596e-013}, blow = {-15882.88066, 0.4818144784}, ahigh = {2070217.324, -6199.89173, 11.50315072, -0.00367568546, 9.46173235e-007, -9.203765750000001e-011, 2.707242192e-015}, bhigh = {22714.71632, -45.61058514}, R = 67.55723838697359);
          constant IdealGases.Common.DataRecord SrCLplus(name = "SrCLplus", MM = 0.1230724514, Hf = 3316031.795560708, H0 = 77862.77018936505, Tlimit = 1000, alow = {1689.043529, -174.9284676, 5.06532864, -0.0009857327839999999, 1.071655138e-006, -5.988127129999999e-010, 1.395650932e-013}, blow = {48611.9026, 0.2492001711}, ahigh = {-21324.75783, -4.05794827, 4.50314939, 3.82620238e-005, 2.67695151e-009, 5.667610500000001e-014, 1.613288103e-018}, bhigh = {47693.2603, 3.637722496}, R = 67.55753952586014);
          constant IdealGases.Common.DataRecord SrCL2(name = "SrCL2", MM = 0.158526, Hf = -3058261.118050036, H0 = 90938.38234737518, Tlimit = 1000, alow = {1913.170011, -268.1156951, 8.0813343, -0.002393329295, 2.976397494e-006, -1.943902862e-009, 5.17430771e-013}, blow = {-59101.1185, -8.457827543000001}, ahigh = {-25927.16099, -5.26378585, 7.00447524, -1.970397814e-006, 4.6796712e-010, -5.67021588e-014, 2.742892387e-018}, bhigh = {-60453.9953, -2.192202775}, R = 52.44863303180551);
          constant IdealGases.Common.DataRecord SrF(name = "SrF", MM = 0.1066184032, Hf = -2847095.612851947, H0 = 87068.57091628248, Tlimit = 1000, alow = {27250.08615, -485.001972, 6.1126972, -0.003008469209, 3.3684658e-006, -1.987813072e-009, 4.88416109e-013}, blow = {-35368.9039, -6.68373918}, ahigh = {870234.0489999999, -2589.81877, 7.20026422, -0.001173446296, 1.996165066e-007, 1.103842667e-011, -2.343183859e-015}, bhigh = {-21336.71699, -16.67390309}, R = 77.98346017622593);
          constant IdealGases.Common.DataRecord SrFplus(name = "SrFplus", MM = 0.1066178546, Hf = 1964664.481252842, H0 = 85415.44973087461, Tlimit = 1000, alow = {40096.242, -594.494608, 6.18233033, -0.00271615975, 2.641369127e-006, -1.383123292e-009, 3.052935853e-013}, blow = {26971.49351, -8.313659449999999}, ahigh = {-47520.8206, -25.26802345, 4.51929528, 2.55642675e-005, 3.38254183e-009, -1.511394636e-013, 9.53749118e-018}, bhigh = {23838.61417, 1.892197325}, R = 77.98386143853246);
          constant IdealGases.Common.DataRecord SrF2(name = "SrF2", MM = 0.1256168064, Hf = -6247522.266256246, H0 = 105635.5545112792, Tlimit = 1000, alow = {41707.5965, -850.0132410000001, 10.02806064, -0.00609848519, 7.05626133e-006, -4.35937431e-009, 1.11130831e-012}, blow = {-92178.59849999999, -23.25100053}, ahigh = {-61262.6543, -19.64060822, 7.01555775, -6.51252678e-006, 1.490473613e-009, -1.75643704e-013, 8.317093849999999e-018}, bhigh = {-96567.0125, -5.40230876}, R = 66.1891687767028);
          constant IdealGases.Common.DataRecord SrH(name = "SrH", MM = 0.08862793999999999, Hf = 2473566.518639608, H0 = 98423.87174969881, Tlimit = 1000, alow = {-43177.9689, 767.64297, -1.594398638, 0.01500711077, -1.826296898e-005, 1.136757659e-008, -2.855588359e-012}, blow = {21796.85693, 33.2467423}, ahigh = {-226982.5428, 1345.209458, 1.152194421, 0.003109555289, -1.188080968e-006, 2.073091147e-010, -1.300839287e-014}, bhigh = {17355.47062, 21.12916944}, R = 93.81321511026886);
          constant IdealGases.Common.DataRecord SrI(name = "SrI", MM = 0.21452447, Hf = -36600.37477309698, H0 = 47934.50369554578, Tlimit = 1000, alow = {-2467.498501, -25.35495809, 4.60329848, -0.0001613676212, 2.694294049e-007, -1.564795762e-010, 3.82648228e-014}, blow = {-2175.538291, 6.41148188}, ahigh = {2201136.857, -7082.2307, 13.32597928, -0.00530586442, 1.63607115e-006, -2.205540591e-010, 1.078496824e-014}, bhigh = {42190.77170000001, -55.308332}, R = 38.75768577822381);
          constant IdealGases.Common.DataRecord SrI2(name = "SrI2", MM = 0.34142894, Hf = -814866.0216090645, H0 = 48916.93715242767, Tlimit = 1000, alow = {-4391.84969, -37.5726735, 7.6569935, -0.00035617213, 4.508689010000001e-007, -2.983230473e-010, 8.01865866e-014}, blow = {-35533.0346, -2.598281578}, ahigh = {-8090.381920000001, -0.734826323, 7.50064006, -2.863515042e-007, 6.87655697e-011, -8.398807359999999e-015, 4.08696816e-019}, bhigh = {-35721.0642, -1.693119087}, R = 24.35198375392549);
          constant IdealGases.Common.DataRecord SrO(name = "SrO", MM = 0.1036194, Hf = -137114.1986925228, H0 = 87243.40229725321, Tlimit = 1000, alow = {42248.9167, -591.3517869999999, 5.964363, -0.002105712736, 2.028373972e-006, -1.197885492e-009, 3.46628992e-013}, blow = {101.8053721, -7.51142747}, ahigh = {-51729330.5, 151082.1, -161.4513515, 0.08516051790000001, -2.091669402e-005, 2.452307103e-009, -1.108841292e-013}, bhigh = {-968782.9519999999, 1197.846971}, R = 80.24049550566787);
          constant IdealGases.Common.DataRecord SrOH(name = "SrOH", MM = 0.10462734, Hf = -1855019.213907187, H0 = 105581.0651403352, Tlimit = 1000, alow = {46400.06559999999, -993.868554, 9.78494896, -0.00577437762, 5.126164110000001e-006, -1.833418356e-009, 1.92601341e-013}, blow = {-20227.21196, -27.40670981}, ahigh = {2545648.379, -7416.19598, 13.81956204, -0.003096481934, 7.72389421e-007, -7.921280320000001e-011, 2.740829111e-015}, bhigh = {22313.02058, -61.4878113}, R = 79.46748909032763);
          constant IdealGases.Common.DataRecord SrOHplus(name = "SrOHplus", MM = 0.1046267914, Hf = 2964533.279188374, H0 = 106135.0429599431, Tlimit = 1000, alow = {38573.6694, -903.024233, 9.451103249999999, -0.00504251628, 4.190101890000001e-006, -1.196920603e-009, 1.606318743e-014}, blow = {39950.67720000001, -26.0148611}, ahigh = {867969.566, -2340.609538, 7.97682975, 0.0001023752569, -6.28603233e-008, 1.024851502e-011, -5.72368392e-016}, bhigh = {50731.5772, -20.27046917}, R = 79.46790577007029);
          constant IdealGases.Common.DataRecord Sr_OH_2(name = "Sr_OH_2", MM = 0.12163468, Hf = -4905628.666100819, H0 = 139860.7781925352, Tlimit = 1000, alow = {72050.90300000001, -1789.465226, 17.29429139, -0.01169280781, 1.106702894e-005, -4.49014609e-009, 6.603997460000001e-013}, blow = {-66053.94040000001, -63.88681219999999}, ahigh = {1750013.518, -4681.6015, 13.95548402, 0.000203501956, -1.253496115e-007, 2.044549432e-011, -1.142004855e-015}, bhigh = {-44271.7024, -50.5145685}, R = 68.35609712624722);
          constant IdealGases.Common.DataRecord SrS(name = "SrS", MM = 0.119685, Hf = 871882.9845009816, H0 = 79676.77653841332, Tlimit = 1000, alow = {12564.40807, -315.0436559, 5.72172251, -0.002630479718, 3.38469975e-006, -2.296256846e-009, 6.512086680000001e-013}, blow = {12772.9516, -3.693874943}, ahigh = {-13794329.47, 39213.3158, -36.0286638, 0.01818513386, -3.27585816e-006, 2.324312746e-010, -3.6145557e-015}, bhigh = {-241024.6179, 299.1679642}, R = 69.46962443079751);
          constant IdealGases.Common.DataRecord Sr2(name = "Sr2", MM = 0.17524, Hf = 1755137.736818078, H0 = 64854.6964163433, Tlimit = 1000, alow = {-110885.9753, 592.760401, 8.43968855, -0.02652801112, 4.43582597e-005, -3.39991549e-008, 9.965546679999999e-012}, blow = {31576.15033, -7.03396665}, ahigh = {209844.5682, 103.865013, 2.30933233, 0.0001330517507, -4.42643339e-008, 6.71141915e-012, -3.374128e-016}, bhigh = {36366.4269, 21.68088517}, R = 47.44619949783154);
          constant IdealGases.Common.DataRecord Ta(name = "Ta", MM = 0.1809479, Hf = 4324552.194305654, H0 = 34262.00580388057, Tlimit = 1000, alow = {-11509.07339, 47.8073043, 3.18558839, -0.00536652816, 1.288379705e-005, -1.045798666e-008, 3.050617695e-012}, blow = {92997.97630000001, 5.33605661}, ahigh = {1689726.898, -5986.85466, 9.565039670000001, -0.002511649459, 6.44303117e-007, -7.189237249999999e-011, 3.11335207e-015}, bhigh = {130671.0983, -43.3509627}, R = 45.94953575034582);
          constant IdealGases.Common.DataRecord Taplus(name = "Taplus", MM = 0.1809473514, Hf = 8564255.420209482, H0 = 35025.85117142533, Tlimit = 1000, alow = {286971.2865, -3084.920994, 14.30679704, -0.01984772164, 1.951445133e-005, -8.97094603e-009, 1.501974665e-012}, blow = {201382.8712, -63.0642783}, ahigh = {3656142.13, -12540.73524, 18.65022579, -0.007943274660000001, 2.151786937e-006, -2.816764844e-010, 1.413722944e-014}, bhigh = {263687.6455, -106.5864286}, R = 45.94967506111836);
          constant IdealGases.Common.DataRecord Taminus(name = "Taminus", MM = 0.1809484486, Hf = 4119789.004922146, H0 = 35458.32555980256, Tlimit = 1000, alow = {187398.2301, -1681.268679, 6.48004015, -0.0002254355782, 2.028434876e-006, -4.67980143e-009, 1.997027996e-012}, blow = {97934.96019999999, -20.49618155}, ahigh = {-4235467.48, 11010.56361, -4.73691107, 0.002503562129, -4.82185169e-007, 4.88640803e-011, -2.030547835e-015}, bhigh = {15746.99408, 64.918027}, R = 45.94939644041801);
          constant IdealGases.Common.DataRecord TaCL5(name = "TaCL5", MM = 0.3582129, Hf = -2135140.86176126, H0 = 75008.74479953123, Tlimit = 1000, alow = {63064.8329, -1771.221442, 22.55083356, -0.01360238041, 1.613009806e-005, -1.016307761e-008, 2.631917453e-012}, blow = {-87927.37109999999, -80.97864282000001}, ahigh = {-142644.4545, -38.9434032, 16.0314237, -1.333676355e-005, 3.084307611e-009, -3.66393006e-013, 1.745841795e-017}, bhigh = {-97012.9717, -42.56986372}, R = 23.21097872243015);
          constant IdealGases.Common.DataRecord TaO(name = "TaO", MM = 0.1969473, Hf = 1231470.068388853, H0 = 44509.90188745923, Tlimit = 1000, alow = {-13957.38049, 394.523699, -0.0630169641, 0.01308595441, -1.86233973e-005, 1.366155583e-008, -3.92455123e-012}, blow = {26451.95419, 27.41671469}, ahigh = {6106591.12, -19841.35246, 28.15797433, -0.01330045882, 3.87680749e-006, -5.068872769999999e-010, 2.443666035e-014}, bhigh = {152581.4433, -165.8307106}, R = 42.21673513675993);
          constant IdealGases.Common.DataRecord TaO2(name = "TaO2", MM = 0.2129467, Hf = -815518.6532592428, H0 = 50243.08430231603, Tlimit = 1000, alow = {15163.56303, 70.58414759999999, 0.691851699, 0.02020605733, -2.928317413e-005, 2.081417374e-008, -5.751106690000001e-012}, blow = {-22121.99468, 24.846625}, ahigh = {1297565.964, -4149.217430000001, 10.41680373, -0.001045478838, 2.754904271e-007, -3.29704313e-011, 1.356249074e-015}, bhigh = {3418.98567, -33.9264279}, R = 39.04485019021192);
          constant IdealGases.Common.DataRecord Ti(name = "Ti", MM = 0.047867, Hf = 9881546.785885891, H0 = 157501.8488729187, Tlimit = 1000, alow = {-45701.79399999999, 660.809202, 0.429525749, 0.00361502991, -3.54979281e-006, 1.759952494e-009, -3.052720871e-013}, blow = {52709.4793, 20.26149738}, ahigh = {-170478.6714, 1073.852803, 1.181955014, 0.0002245246352, 3.091697848e-007, -5.74002728e-011, 2.927371014e-015}, bhigh = {49780.69910000001, 17.40431368}, R = 173.699458917417);
          constant IdealGases.Common.DataRecord Tiplus(name = "Tiplus", MM = 0.0478664514, Hf = 23766625.59531204, H0 = 165038.9525219745, Tlimit = 1000, alow = {170745.7044, -1727.524602, 9.615885329999999, -0.0108965506, 8.20180965e-006, -2.871464413e-009, 3.420382976e-013}, blow = {144789.7558, -34.6314366}, ahigh = {-768546.308, 2545.8681, 0.342386278, 0.000709990136, 2.706231875e-008, -2.3716601e-011, 1.895443077e-015}, bhigh = {119882.1489, 24.8479915}, R = 173.7014496963525);
          constant IdealGases.Common.DataRecord Timinus(name = "Timinus", MM = 0.0478675486, Hf = 9593217.689865155, H0 = 157990.4177503671, Tlimit = 1000, alow = {-3006.48499, 204.0911689, 1.822638976, 0.001245254812, -1.309239865e-006, 7.37214322e-010, -1.724319779e-013}, blow = {53467.7205, 12.05926588}, ahigh = {23411.1764, 2.580413872, 2.497754577, 9.76907204e-007, -2.280024955e-010, 2.717202291e-014, -1.295670294e-018}, bhigh = {54546.62179999999, 7.99982395}, R = 173.6974681841134);
          constant IdealGases.Common.DataRecord TiCL(name = "TiCL", MM = 0.08331999999999999, Hf = 1810500.792126741, H0 = 116186.1017762842, Tlimit = 1000, alow = {-17141.77839, 310.3259047, 0.892093598, 0.01383436793, -1.885370354e-005, 1.212085912e-008, -3.032162427e-012}, blow = {15580.66453, 22.44778384}, ahigh = {-963322.542, 2868.829781, 1.973820676, 0.001752668011, -4.25543436e-007, 5.0491034e-011, -2.304720715e-015}, bhigh = {-1811.011561, 23.19450287}, R = 99.78963034085456);
          constant IdealGases.Common.DataRecord TiCL2(name = "TiCL2", MM = 0.118773, Hf = -1997339.462672493, H0 = 114043.343184057, Tlimit = 1000, alow = {13243.54656, -576.282932, 9.64004729, -0.00447512095, 5.398794869999999e-006, -3.52580049e-009, 9.74600455e-013}, blow = {-27920.66683, -22.18347625}, ahigh = {-3190012.95, 10157.63826, -4.84079477, 0.00682200634, -1.70075327e-006, 2.118106588e-010, -1.041245457e-014}, bhigh = {-94809.82980000001, 78.23195948}, R = 70.00304783073595);
          constant IdealGases.Common.DataRecord TiCL3(name = "TiCL3", MM = 0.154226, Hf = -3496946.04022668, H0 = 99691.01837563056, Tlimit = 1000, alow = {124328.2268, -2320.728253, 17.79895274, -0.01160889348, 1.006643154e-005, -4.81194069e-009, 1.006186828e-012}, blow = {-56096.0251, -67.32754065}, ahigh = {-94087.92310000002, -125.5351109, 10.66923972, -0.0002637456897, 7.95916935e-008, -1.176740054e-011, 6.41622348e-016}, bhigh = {-67691.1468, -23.69164216}, R = 53.91096183522882);
          constant IdealGases.Common.DataRecord TiCL4(name = "TiCL4", MM = 0.189679, Hf = -4023429.056458543, H0 = 113947.9910796662, Tlimit = 1000, alow = {81871.96800000001, -1758.32385, 18.92512121, -0.01133495876, 1.251952037e-005, -7.422681329999999e-009, 1.825440187e-012}, blow = {-86729.23109999999, -67.69594291}, ahigh = {-143256.2278, -43.1677485, 13.03337752, -1.371365738e-005, 3.093403483e-009, -3.60424516e-013, 1.691384523e-017}, bhigh = {-95889.46999999999, -32.47541011}, R = 43.83443607357694);
          constant IdealGases.Common.DataRecord TiO(name = "TiO", MM = 0.0638664, Hf = 775112.0307391679, H0 = 150205.9768516779, Tlimit = 1000, alow = {-11681.5246, 454.256565, -0.1139144613, 0.01275432333, -1.727656935e-005, 1.187369403e-008, -3.23657937e-012}, blow = {2924.306353, 27.02903947}, ahigh = {2330644.03, -7415.79386, 12.81799311, -0.004344555950000001, 1.186303111e-006, -1.367644275e-010, 5.70321225e-015}, bhigh = {51448.4136, -57.9399424}, R = 130.1853869953528);
          constant IdealGases.Common.DataRecord TiOplus(name = "TiOplus", MM = 0.06386585139999999, Hf = 10730632.97485454, H0 = 144257.2329036547, Tlimit = 1000, alow = {36912.5625, -149.2825538, 2.624977257, 0.00581862713, -7.52966211e-006, 4.74415435e-009, -1.186916989e-012}, blow = {82415.51169999999, 10.95428726}, ahigh = {342132.953, -2161.85106, 8.02517566, -0.002708700692, 1.004583805e-006, -1.50576685e-010, 8.04565811e-015}, bhigh = {93626.4699, -22.61587887}, R = 130.1865052722056);
          constant IdealGases.Common.DataRecord TiOCL(name = "TiOCL", MM = 0.0993194, Hf = -2459358.393224285, H0 = 122259.1054718413, Tlimit = 1000, alow = {35458.5651, -629.528714, 7.3701763, 0.00330201914, -6.12145875e-006, 4.73588025e-009, -1.374690357e-012}, blow = {-27970.87903, -12.94470159}, ahigh = {-127305.9873, -111.111885, 7.58288808, -3.3128718e-005, 7.31401409e-009, -8.37882226e-013, 3.87927506e-017}, bhigh = {-31393.56857, -12.47104274}, R = 83.71448075602551);
          constant IdealGases.Common.DataRecord TiOCL2(name = "TiOCL2", MM = 0.1347724, Hf = -4047950.470571125, H0 = 123986.8845549979, Tlimit = 1000, alow = {32890.5246, -738.918859, 10.55976857, 0.001577742341, -3.9071821e-006, 3.29057653e-009, -9.949471750000001e-013}, blow = {-64484.107, -24.17469934}, ahigh = {-132295.1633, -96.91212050000002, 10.07240769, -2.897248531e-005, 6.40177504e-009, -7.3384526e-013, 3.39928267e-017}, bhigh = {-68474.09640000001, -19.75043469}, R = 61.69269078832164);
          constant IdealGases.Common.DataRecord TiO2(name = "TiO2", MM = 0.07986579999999999, Hf = -3824290.246889157, H0 = 142134.5557172156, Tlimit = 1000, alow = {-1710.545601, 272.1435528, 0.596137896, 0.01925463599, -2.665500165e-005, 1.811109197e-008, -4.87671047e-012}, blow = {-39122.4177, 24.08605889}, ahigh = {154629.9764, -1046.25688, 7.78898583, -0.0001546805714, -7.05993595e-008, 3.100244802e-011, -2.49472543e-015}, bhigh = {-32663.3675, -15.9153466}, R = 104.1055370383819);
          constant IdealGases.Common.DataRecord U(name = "U", MM = 0.23802891, Hf = 2247626.139194605, H0 = 27304.64127235637, Tlimit = 1000, alow = {69657.3775, -1070.351517, 8.075842310000001, -0.01060034069, 9.25654801e-006, -3.21989976e-009, 4.058048809e-013}, blow = {68665.137, -22.40521678}, ahigh = {-4092498.96, 12748.88349, -12.18707506, 0.00725810568, -7.78777507e-007, -3.84435385e-011, 7.066508567e-015}, bhigh = {-16993.72664, 115.5026301}, R = 34.93051327252644);
          constant IdealGases.Common.DataRecord UF(name = "UF", MM = 0.2570273132, Hf = -191616.7444884609, H0 = 36521.65555143032, Tlimit = 1000, alow = {172553.3463, -1698.985561, 5.90292643, 0.01841977309, -4.633174310000001e-005, 4.376657200000001e-008, -1.451876566e-011}, blow = {2086.455917, -11.86592944}, ahigh = {-5325439.37, 15339.67086, -11.75044398, 0.00964721093, -2.49877819e-006, 3.155572896e-010, -1.544694839e-014}, bhigh = {-105719.6525, 122.0828149}, R = 32.34859321557893);
          constant IdealGases.Common.DataRecord UFplus(name = "UFplus", MM = 0.2570267646, Hf = 2167318.908857323, H0 = 36958.66854482439, Tlimit = 1000, alow = {1622640.597, -19173.26611, 87.5946122, -0.1699915707, 0.0001861251683, -1.036176702e-007, 2.324287412e-011}, blow = {161670.9643, -480.6891249}, ahigh = {539509.184, -2923.962095, 10.08511948, -0.002425945123, 5.92872548e-007, -6.79584554e-011, 3.152666445e-015}, bhigh = {82672.91220000001, -33.07089704}, R = 32.34866226067727);
          constant IdealGases.Common.DataRecord UFminus(name = "UFminus", MM = 0.2570278618, Hf = -605689.9509250013, H0 = 35622.62447300179, Tlimit = 1000, alow = {3692.32786, -149.1979606, 4.03398852, 0.002857429445, -5.17170256e-006, 3.98537911e-009, -9.47035544e-013}, blow = {-19152.81695, 6.059243838}, ahigh = {-4311143.56, 19748.17938, -27.60304763, 0.02276806261, -6.88874172e-006, 9.483511059999999e-010, -4.9106881e-014}, bhigh = {-138080.129, 224.0758537}, R = 32.34852417077533);
          constant IdealGases.Common.DataRecord UF2(name = "UF2", MM = 0.2760257164, Hf = -1938358.21523476, H0 = 54894.85616638001, Tlimit = 1000, alow = {-38824.9202, 445.493086, 4.71800919, 0.00164891148, 8.696243010000001e-006, -1.207439921e-008, 4.53679391e-012}, blow = {-68553.29330000001, 11.59725423}, ahigh = {-471677.682, 322.423686, 8.214090629999999, -0.000140741378, 6.54117805e-009, 1.818915497e-012, -2.287963719e-016}, bhigh = {-69952.3505, -9.648963156000001}, R = 30.12209191389676);
          constant IdealGases.Common.DataRecord UF2plus(name = "UF2plus", MM = 0.2760251678, Hf = 255214.1442806506, H0 = 52303.66170979283, Tlimit = 1000, alow = {5439.01016, -150.7096256, 6.87348519, 0.001530973969, -3.31669431e-006, 2.949453335e-009, -8.28170654e-013}, blow = {7256.01491, -2.205491833}, ahigh = {-3779149.61, 12987.37504, -10.27930641, 0.01048838051, -2.651047812e-006, 3.116973593e-010, -1.417216291e-014}, bhigh = {-74334.402, 118.9131079}, R = 30.12215178155214);
          constant IdealGases.Common.DataRecord UF2minus(name = "UF2minus", MM = 0.2760262650000001, Hf = -2457132.820313313, H0 = 47652.49060628342, Tlimit = 1000, alow = {806049.1730000001, -9404.254659999999, 44.4839277, -0.0686855877, 6.777809920000001e-005, -3.33882768e-008, 6.60485162e-012}, blow = {-36033.0613, -226.1594249}, ahigh = {10382318.55, -28401.13518, 34.9853337, -0.01053216396, 1.92462047e-006, -1.69874957e-010, 5.70985889e-015}, bhigh = {100157.2794, -209.4039075}, R = 30.12203204647934);
          constant IdealGases.Common.DataRecord UF3(name = "UF3", MM = 0.2950241196, Hf = -3596175.792130048, H0 = 63191.657093246, Tlimit = 1000, alow = {30710.27207, -634.078947, 11.14030799, -0.0002937554125, -1.865390371e-006, 2.37181947e-009, -7.44675407e-013}, blow = {-127183.8141, -23.48962529}, ahigh = {-3828876.42, 12968.91783, -7.26555429, 0.01048289752, -2.649840687e-006, 3.115594509e-010, -1.4165794e-014}, bhigh = {-211361.5913, 105.4464687}, R = 28.18234662058458);
          constant IdealGases.Common.DataRecord UF3plus(name = "UF3plus", MM = 0.295023571, Hf = -965161.6277127904, H0 = 63245.35336873134, Tlimit = 1000, alow = {-398415.589, 3480.89244, -2.519603507, 0.02283749282, -2.16884478e-005, 1.11126579e-008, -2.350882012e-012}, blow = {-55508.892, 58.53815975}, ahigh = {-1196316.388, 3972.78507, 4.53590711, 0.00413341615, -1.097960689e-006, 1.319641252e-010, -6.10112918e-015}, bhigh = {-61995.4174, 21.37623045}, R = 28.18239902600868);
          constant IdealGases.Common.DataRecord UF3minus(name = "UF3minus", MM = 0.2950246682, Hf = -4021496.840378447, H0 = 65615.96736336911, Tlimit = 1000, alow = {-231518.73, 2534.197822, -3.18071459, 0.02932741422, -2.85691226e-005, 1.345804152e-008, -2.465442912e-012}, blow = {-158038.7746, 59.73605435}, ahigh = {-191005.3149, -506.918131, 11.81482258, -0.0001711879404, -9.38483918e-008, 2.242009537e-011, -1.42819528e-015}, bhigh = {-143862.8681, -27.74680411}, R = 28.18229421535538);
          constant IdealGases.Common.DataRecord UF4(name = "UF4", MM = 0.3140225228, Hf = -5114784.136114202, H0 = 69716.59804778818, Tlimit = 1000, alow = {-50078.5238, -969.349355, 20.69596355, -0.02564852637, 4.19586012e-005, -3.20101339e-008, 9.422076860000001e-012}, blow = {-193162.6334, -71.72362935}, ahigh = {-1230291.173, 3876.46379, 7.62162591, 0.004094525220000001, -1.088530094e-006, 1.308042117e-010, -6.04439087e-015}, bhigh = {-221414.292, 6.270300658}, R = 26.47731100897964);
          constant IdealGases.Common.DataRecord UF4plus(name = "UF4plus", MM = 0.3140219742, Hf = -2042976.405184323, H0 = 64687.5622374837, Tlimit = 1000, alow = {38489.5633, -1343.282151, 16.46334809, -0.00719607986, 1.147165041e-005, -8.70587798e-009, 2.501076195e-012}, blow = {-74050.482, -52.95285534999999}, ahigh = {-1617630.159, 3765.93477, 9.16613853, 0.002426641067, -5.695320299999999e-007, 6.021096599999999e-011, -2.438874329e-015}, bhigh = {-106191.0645, -4.162641042}, R = 26.47735726514645);
          constant IdealGases.Common.DataRecord UF4minus(name = "UF4minus", MM = 0.3140230714, Hf = -5503846.766081914, H0 = 69666.13918674002, Tlimit = 1000, alow = {72031.7969, -1374.343862, 16.40877729, -0.00436306627, 2.460407959e-006, -1.498144731e-010, -1.239417075e-013}, blow = {-204517.4023, -51.72250795}, ahigh = {-4639200.11, 16268.24426, -9.59529105, 0.01441556535, -4.008173850000001e-006, 5.140230790000001e-010, -2.511687821e-014}, bhigh = {-312514.0933, 127.4038784}, R = 26.47726475297446);
          constant IdealGases.Common.DataRecord UF5(name = "UF5", MM = 0.333020926, Hf = -5854959.048429288, H0 = 70875.07167642673, Tlimit = 1000, alow = {161576.9193, -2976.611537, 25.3754242, -0.0193010567, 2.561852107e-005, -1.747402896e-008, 4.73205938e-012}, blow = {-223908.9616, -102.4264836}, ahigh = {-1695373.88, 3735.42535, 12.1874804, 0.002418608203, -5.67852616e-007, 6.00276561e-011, -2.430740809e-015}, bhigh = {-264510.7342, -18.83137475}, R = 24.96681544870847);
          constant IdealGases.Common.DataRecord UF5plus(name = "UF5plus", MM = 0.3330203774, Hf = -2563258.286668436, H0 = 70903.04258360378, Tlimit = 1000, alow = {162651.9713, -2915.370389, 24.36168077, -0.01382199656, 1.339849041e-005, -7.07857269e-009, 1.57340404e-012}, blow = {-92264.0485, -98.30243935}, ahigh = {-262847.8263, -117.4047323, 16.08988291, -3.66551671e-005, 8.221494970000001e-009, -9.53716052e-013, 4.46014508e-017}, bhigh = {-107629.3931, -47.63374165000001}, R = 24.96685657770803);
          constant IdealGases.Common.DataRecord UF5minus(name = "UF5minus", MM = 0.3330214746, Hf = -6874725.540597317, H0 = 75649.66502613644, Tlimit = 1000, alow = {-314213.0934, 2020.611008, 7.50849182, 0.01640034396, -1.562795472e-005, 7.99024601e-009, -1.671674534e-012}, blow = {-290764.7496, 4.690020575}, ahigh = {-1331518.187, 3905.51686, 10.58727667, 0.00411250727, -1.09327793e-006, 1.314215436e-010, -6.07577827e-015}, bhigh = {-304950.6955, -7.977511665}, R = 24.96677431984442);
          constant IdealGases.Common.DataRecord UF6(name = "UF6", MM = 0.3520193292, Hf = -6103760.418165129, H0 = 75630.36967459797, Tlimit = 1000, alow = {191567.406, -3426.39161, 28.0039589, -0.01326392086, 1.107123764e-005, -4.821754109999999e-009, 8.32513245e-013}, blow = {-246104.5194, -121.1612947}, ahigh = {-340902.792, -145.8950046, 19.10902533, -4.358451129999999e-005, 9.617604659999999e-009, -1.100915551e-012, 5.09264508e-017}, bhigh = {-264364.5561, -65.79130395999999}, R = 23.61936209268818);
          constant IdealGases.Common.DataRecord UF6minus(name = "UF6minus", MM = 0.3520198778, Hf = -7645324.368668365, H0 = 78775.65657174375, Tlimit = 1000, alow = {156923.0858, -3027.27994, 27.01442427, -0.01136860992, 8.29857616e-006, -2.582889593e-009, 2.790817248e-013}, blow = {-313530.7376, -112.8934794}, ahigh = {2699676.848, -6574.45967, 22.41731997, 0.000605779971, -4.51749171e-007, 7.821820469999999e-011, -4.56033472e-015}, bhigh = {-285566.7984, -91.82896336}, R = 23.61932528345421);
          constant IdealGases.Common.DataRecord UO(name = "UO", MM = 0.25402831, Hf = 120020.4378795419, H0 = 37548.13390680748, Tlimit = 1000, alow = {1007249.615, -12871.90666, 59.93123920000001, -0.1003445164, 8.529345550000001e-005, -2.634105066e-008, -5.350649790000001e-013}, blow = {66274.4252, -322.6786779}, ahigh = {-2458660.003, 3942.65216, 5.01603072, -0.0005450639469999999, 2.096321351e-007, -2.654652526e-011, 1.361108472e-015}, bhigh = {-26651.49645, 5.580598689}, R = 32.73049369969828);
          constant IdealGases.Common.DataRecord UOplus(name = "UOplus", MM = 0.2540277614, Hf = 2287041.084793814, H0 = 34737.25450859325, Tlimit = 1000, alow = {15628.34007, -122.4921454, 3.32414616, 0.002467496097, -6.851856739999999e-007, 5.55207319e-011, -5.89301203e-014}, blow = {69529.99740000001, 9.583869879}, ahigh = {-106371.4748, -1793.716133, 8.024449049999999, -0.001565116442, 5.06603391e-007, -7.37397747e-011, 4.07476643e-015}, bhigh = {77890.8584, -21.1528425}, R = 32.73056438468461);
          constant IdealGases.Common.DataRecord UOF(name = "UOF", MM = 0.2730267132, Hf = -1985823.063411511, H0 = 51297.01718871954, Tlimit = 1000, alow = {386.124847, 51.62286899999999, 4.88896252, 0.00726563356, -1.134176875e-005, 8.50904175e-009, -2.354655485e-012}, blow = {-67198.1338, 8.269521875000001}, ahigh = {-3805936.01, 12952.07909, -10.25373631, 0.01047841671, -2.648894949e-006, 3.114551623e-010, -1.416111895e-014}, bhigh = {-147897.6701, 118.4570601}, R = 30.45296155292104);
          constant IdealGases.Common.DataRecord UOF2(name = "UOF2", MM = 0.2920251164, Hf = -3819912.060139495, H0 = 61009.94400630945, Tlimit = 1000, alow = {-91066.1831, -101.4150169, 13.71057013, -0.01659931694, 3.082352584e-005, -2.490530206e-008, 7.580617639999999e-012}, blow = {-137468.6016, -34.70029665}, ahigh = {-1198944.06, 3852.17341, 4.63910975, 0.00408773844, -1.087067018e-006, 1.306398384e-010, -6.03690152e-015}, bhigh = {-161263.7583, 20.18237189}, R = 28.47177017682031);
          constant IdealGases.Common.DataRecord UOF3(name = "UOF3", MM = 0.3110235196, Hf = -4856989.72522334, H0 = 63748.10826364272, Tlimit = 1000, alow = {124925.6374, -2111.719498, 18.44931536, -0.01041493583, 1.469002962e-005, -1.049770051e-008, 2.922353602e-012}, blow = {-174384.4024, -66.49285435}, ahigh = {-1658296.049, 3714.43468, 9.20242554, 0.002412862293, -5.66624172e-007, 5.98906211e-011, -2.42453481e-015}, bhigh = {-210549.1714, -5.670532802}, R = 26.73261498260018);
          constant IdealGases.Common.DataRecord UOF4(name = "UOF4", MM = 0.3300219228, Hf = -5410584.593442651, H0 = 68912.79769248105, Tlimit = 1000, alow = {143716.4405, -2496.730363, 21.04827038, -0.00469256133, 8.741718649999999e-007, 1.517555623e-009, -7.76879974e-013}, blow = {-206128.4178, -82.46219615}, ahigh = {-299619.0288, -167.7276501, 16.12494871, -4.98441446e-005, 1.098253497e-008, -1.255803231e-012, 5.80448098e-017}, bhigh = {-219545.926, -50.24484645}, R = 25.1936960110336);
          constant IdealGases.Common.DataRecord UO2(name = "UO2", MM = 0.27002771, Hf = -1769521.73908374, H0 = 49847.09532218007, Tlimit = 1000, alow = {-112965.0727, 427.073027, 8.41369401, -0.00976428, 2.199903691e-005, -1.907665954e-008, 6.02988991e-012}, blow = {-62514.433, -13.00704863}, ahigh = {-1190542.635, 3832.18635, 2.153236312, 0.00408235191, -1.085924847e-006, 1.305134065e-010, -6.03121575e-015}, bhigh = {-83676.1801, 25.90742388}, R = 30.79118065327443);
          constant IdealGases.Common.DataRecord UO2plus(name = "UO2plus", MM = 0.2700271614, Hf = 190699.0346194115, H0 = 42871.68350020659, Tlimit = 1000, alow = {44880.2338, -648.923642, 6.87424356, 0.002281949316, 1.859218752e-007, -1.849509036e-009, 8.14996533e-013}, blow = {7891.74023, -10.28257233}, ahigh = {-1589957.42, 3657.74607, 3.74678542, 0.002394401448, -5.62410144e-007, 5.93945164e-011, -2.401047056e-015}, bhigh = {-20485.58817, 14.64015614}, R = 30.79124321009879);
          constant IdealGases.Common.DataRecord UO2minus(name = "UO2minus", MM = 0.2700282586, Hf = -2124592.529590901, H0 = 51021.98959261074, Tlimit = 1000, alow = {63419.7131, -719.61185, 8.30824162, 0.000962645545, -3.58373398e-006, 3.47355458e-009, -1.02279312e-012}, blow = {-67181.69869999999, -16.0603012}, ahigh = {-273825.164, 3228.53442, -0.2805114576, 0.006607310710000001, -2.070291015e-006, 2.851708013e-010, -1.466578192e-014}, bhigh = {-88934.3618, 41.45760316}, R = 30.79111809670427);
          constant IdealGases.Common.DataRecord UO2F(name = "UO2F", MM = 0.2890261132, Hf = -3452749.75659189, H0 = 54679.34307051395, Tlimit = 1000, alow = {83217.92390000001, -1327.082127, 12.4012539, -0.00408415101, 7.33159348e-006, -5.98740488e-009, 1.787869209e-012}, blow = {-115752.6888, -34.15298625}, ahigh = {-1614277.202, 3712.77186, 6.20330882, 0.002412618055, -5.665888019999999e-007, 5.98882352e-011, -2.424485602e-015}, bhigh = {-147835.7384, 8.736139132}, R = 28.76719998738163);
          constant IdealGases.Common.DataRecord UO2F2(name = "UO2F2", MM = 0.3080245164, Hf = -4396507.2742502, H0 = 61904.07576271744, Tlimit = 1000, alow = {99598.74299999999, -1530.010701, 13.58992177, 0.0053706558, -1.141444341e-005, 9.301197039999999e-009, -2.781220538e-012}, blow = {-158031.0734, -41.96260275}, ahigh = {-263655.2036, -203.2835065, 13.15092994, -6.00648683e-005, 1.321137913e-008, -1.508668985e-012, 6.96618275e-017}, bhigh = {-166444.4818, -35.66258175}, R = 26.99289036202185);
          constant IdealGases.Common.DataRecord UO3(name = "UO3", MM = 0.28602711, Hf = -2794278.524857312, H0 = 52948.63483394984, Tlimit = 1000, alow = {66376.77039999999, -758.2646579999999, 7.11284471, 0.01322149697, -2.106191042e-005, 1.545856318e-008, -4.37856629e-012}, blow = {-94133.69349999999, -8.588030428}, ahigh = {-1097362.721, 2808.784061, 5.96612147, 0.002861871152, -1.05284381e-006, 1.849985929e-010, -1.102849619e-014}, bhigh = {-117336.0017, 6.672828312}, R = 29.06882497956226);
          constant IdealGases.Common.DataRecord UO3minus(name = "UO3minus", MM = 0.2860276586, Hf = -4563036.051786915, H0 = 53935.6301258063, Tlimit = 1000, alow = {100014.3902, -1448.443754, 12.17532415, -0.002944956748, 5.675227999999999e-006, -4.91671042e-009, 1.525185713e-012}, blow = {-151925.3376, -34.71731375}, ahigh = {-1632938.634, 3667.49541, 6.24003065, 0.002396952522, -5.62949023e-007, 5.94541313e-011, -2.403730982e-015}, bhigh = {-184592.7385, 7.032059012}, R = 29.06876922566257);
          constant IdealGases.Common.DataRecord V(name = "V", MM = 0.0509415, Hf = 10154138.84553851, H0 = 155218.5153558494, Tlimit = 1000, alow = {-55353.7602, 559.333851, 2.675543482, -0.00624304963, 1.565902337e-005, -1.372845314e-008, 4.16838881e-012}, blow = {58206.6436, 9.524567490000001}, ahigh = {1200390.3, -5027.0053, 10.58830594, -0.005044326100000001, 1.488547375e-006, -1.785922508e-010, 8.113013866e-015}, bhigh = {91707.40909999999, -47.6833632}, R = 163.2160811911703);
          constant IdealGases.Common.DataRecord Vplus(name = "Vplus", MM = 0.0509409514, Hf = 23041293.47297585, H0 = 155038.5452753833, Tlimit = 1000, alow = {75688.3446, -841.527382, 7.55923271, -0.01441722656, 2.038356397e-005, -1.289073883e-008, 3.065656561e-012}, blow = {144447.8191, -19.91067645}, ahigh = {2347072.054, -9021.197190000001, 14.77349798, -0.00689189688, 1.968884877e-006, -2.539798544e-010, 1.226783122e-014}, bhigh = {195835.1444, -78.5559293}, R = 163.2178389192786);
          constant IdealGases.Common.DataRecord Vminus(name = "Vminus", MM = 0.0509420486, Hf = 9037446.974600077, H0 = 154651.4758733122, Tlimit = 1000, alow = {-3799.27356, 231.3840448, 1.72560819, 0.001429275357, -1.506038188e-006, 8.491815170000001e-010, -1.987980413e-013}, blow = {53474.0292, 12.61900982}, ahigh = {26001.0043, 2.096334097, 2.498006548, 8.991278839999999e-007, -2.139749508e-010, 2.581021334e-014, -1.240812796e-018}, bhigh = {54700.0708, 7.97790024}, R = 163.2143235009202);
          constant IdealGases.Common.DataRecord VCL4(name = "VCL4", MM = 0.1927535, Hf = -2734365.290383832, H0 = 113010.129517752, Tlimit = 1000, alow = {77198.3471, -1702.85404, 18.82697965, -0.01130896832, 1.266950765e-005, -7.625019339999999e-009, 1.907582465e-012}, blow = {-58637.4854, -65.59171105999999}, ahigh = {-1717776.251, 4550.441049999999, 8.164464799999999, 0.002224875998, -4.094111780000001e-007, 3.2717875e-011, -8.8586908e-016}, bhigh = {-96906.20060000001, 4.348293098}, R = 43.13525824433798);
          constant IdealGases.Common.DataRecord VN(name = "VN", MM = 0.0649482, Hf = 8052571.125912651, H0 = 135488.6509556847, Tlimit = 1000, alow = {-15817.37285, 486.816542, -1.045200388, 0.01685261043, -2.334616543e-005, 1.58871005e-008, -4.27908088e-012}, blow = {59814.814, 31.44995263}, ahigh = {1018619.667, -3932.30557, 9.856823609999999, -0.00328922171, 1.005181767e-006, -1.243211436e-010, 5.48677656e-015}, bhigh = {85573.18339999999, -35.52830762}, R = 128.0169735265951);
          constant IdealGases.Common.DataRecord VO(name = "VO", MM = 0.0669409, Hf = 2219610.223346266, H0 = 131057.4850353073, Tlimit = 1000, alow = {-13116.19784, 374.781697, 0.0930083486, 0.01244977714, -1.688540028e-005, 1.142443381e-008, -3.04058932e-012}, blow = {15237.8992, 25.36811755}, ahigh = {2986190.283, -10113.44974, 17.18161749, -0.00787670503, 2.562279547e-006, -3.54740035e-010, 1.770268056e-014}, bhigh = {79612.5488, -87.89993010000001}, R = 124.2061579691937);
          constant IdealGases.Common.DataRecord VO2(name = "VO2", MM = 0.0829403, Hf = -2805604.211704081, H0 = 128073.4335419573, Tlimit = 1000, alow = {-6678.58586, 391.159758, -1.028549847, 0.02401523419, -3.33737881e-005, 2.283623543e-008, -6.1991431e-012}, blow = {-30746.09276, 32.7791238}, ahigh = {121063.2401, -1627.832993, 9.252713099999999, -0.001572703139, 5.23143016e-007, -6.476071140000001e-011, 2.847226026e-015}, bhigh = {-20973.06345, -25.47380687}, R = 100.2464664342907);
          constant IdealGases.Common.DataRecord V4O10(name = "V4O10", MM = 0.36376, Hf = -7766561.705520123, H0 = 101154.6981526281, Tlimit = 1000, alow = {338573.916, -5353.92926, 29.93220131, 0.0581883652, -9.69634016e-005, 7.27475699e-008, -2.091628357e-011}, blow = {-318934.937, -146.6398396}, ahigh = {-1360273.27, -1341.69286, 40.9913859, -0.000393226553, 8.62849412e-008, -9.83594409e-012, 4.53563729e-016}, bhigh = {-348461.612, -190.7911348}, R = 22.85702661095228);
          constant IdealGases.Common.DataRecord W(name = "W", MM = 0.18384, Hf = 4630349.902088773, H0 = 33814.87162750217, Tlimit = 1000, alow = {159522.3922, -2673.843928, 20.60469727, -0.0625231523, 0.0001105654838, -8.45351161e-008, 2.336187771e-011}, blow = {113964.8616, -90.118369}, ahigh = {-8048745.96, 14657.00424, -0.2508531501, -0.002596486992, 1.409225475e-006, -2.233011706e-010, 1.262640862e-014}, bhigh = {-3091.130919, 39.5582219}, R = 45.22667536988686);
          constant IdealGases.Common.DataRecord Wplus(name = "Wplus", MM = 0.1838394514, Hf = 8854687.895353457, H0 = 33840.75046255279, Tlimit = 1000, alow = {-196928.4929, 2670.137332, -11.31686913, 0.0330818373, -3.6290355e-005, 2.066142971e-008, -4.808285562e-012}, blow = {182095.0862, 85.5210448}, ahigh = {6387743.399999999, -20618.11463, 27.59291576, -0.01244535845, 3.27120049e-006, -4.065463720000001e-010, 1.912595872e-014}, bhigh = {324517.443, -171.6919194}, R = 45.22681033196339);
          constant IdealGases.Common.DataRecord Wminus(name = "Wminus", MM = 0.1838405486, Hf = 4168783.948026143, H0 = 33710.8872182728, Tlimit = 1000, alow = {0, 0, 2.5, 0, 0, 0, 0}, blow = {91429.8137, 8.46116967}, ahigh = {0, 0, 2.5, 0, 0, 0, 0}, bhigh = {91429.8137, 8.46116967}, R = 45.22654040861582);
          constant IdealGases.Common.DataRecord WCL6(name = "WCL6", MM = 0.396558, Hf = -1244993.166195109, H0 = 77645.5272620903, Tlimit = 1000, alow = {33393.9167, -1697.36608, 25.60612352, -0.01425247881, 1.739682668e-005, -1.120537864e-008, 2.95152289e-012}, blow = {-56730.59239999999, -97.41466299000001}, ahigh = {-151673.5473, -34.5137925, 19.02867176, -1.242486969e-005, 2.917675103e-009, -3.50598144e-013, 1.685355887e-017}, bhigh = {-65357.79790000001, -58.94914959000001}, R = 20.96659757210799);
          constant IdealGases.Common.DataRecord WO(name = "WO", MM = 0.1998394, Hf = 2010292.820134568, H0 = 46332.76020644578, Tlimit = 1000, alow = {-19337.58411, 493.669084, -0.4116148220000001, 0.01307976507, -1.689145619e-005, 1.092748066e-008, -2.820593541e-012}, blow = {45110.179, 30.02592661}, ahigh = {1262156.956, -4177.263120000001, 9.35828647, -0.00288761222, 8.89393396e-007, -8.955318699999999e-011, 2.504359614e-015}, bhigh = {73133.8064, -31.4488229}, R = 41.60576943285459);
          constant IdealGases.Common.DataRecord WOCL4(name = "WOCL4", MM = 0.3416514000000001, Hf = -1678591.101924359, H0 = 65187.43666790184, Tlimit = 1000, alow = {26588.12693, -933.6913239999999, 13.21377385, 0.01270930811, -1.964716509e-005, 1.405106974e-008, -3.89938214e-012}, blow = {-67922.90790000001, -35.94869683}, ahigh = {-304902.8285, -353.942586, 16.26432703, -0.0001058245461, 2.340548846e-008, -2.685852021e-012, 1.245412684e-016}, bhigh = {-72725.46739999999, -49.91761253}, R = 24.33612740940034);
          constant IdealGases.Common.DataRecord WO2(name = "WO2", MM = 0.2158388, Hf = 134645.5966211821, H0 = 49642.82603498537, Tlimit = 1000, alow = {3120.918919, 241.3883468, -0.3024119184, 0.02324333417, -3.37881268e-005, 2.380934019e-008, -6.421592880000001e-012}, blow = {1442.005265, 29.57655179}, ahigh = {-753740.6680000001, 3204.64305, 0.6701965600000001, 0.00448823887, -8.858328459999999e-007, 6.24517175e-011, -9.046613900000001e-016}, bhigh = {-17694.2023, 34.0572401}, R = 38.52167450893908);
          constant IdealGases.Common.DataRecord WO2CL2(name = "WO2CL2", MM = 0.2867448, Hf = -2341915.180327594, H0 = 68018.03206195892, Tlimit = 1000, alow = {430.015448, -243.8547856, 7.97630699, 0.01715005722, -2.487975176e-005, 1.737644163e-008, -4.7719284e-012}, blow = {-82328.124, -7.844771421}, ahigh = {-240479.173, -306.8379644, 13.22814502, -9.10022071e-005, 2.00658102e-008, -2.296837607e-012, 1.062829386e-016}, bhigh = {-83674.93059999999, -34.91556626}, R = 28.99606897840868);
          constant IdealGases.Common.DataRecord WO3(name = "WO3", MM = 0.2318382, Hf = -1379087.712896321, H0 = 57229.32200129228, Tlimit = 1000, alow = {7262.46146, 34.2939109, 1.573061955, 0.02754971099, -3.99482731e-005, 2.809371537e-008, -7.77754573e-012}, blow = {-40017.3327, 18.57409963}, ahigh = {1732203.64, -6284.719779999999, 16.81358864, -0.00347208936, 8.079935799999999e-007, -6.589378720000001e-011, 1.142928957e-015}, bhigh = {-2473.075565, -74.0747409}, R = 35.86325290655294);
          constant IdealGases.Common.DataRecord WO3minus(name = "WO3minus", MM = 0.2318387486, Hf = -2805725.811271913, H0 = 59155.9395606572, Tlimit = 1000, alow = {163099.9684, -1967.609682, 12.01970914, 0.001681260779, -5.859360060000001e-006, 5.02861633e-009, -1.434012198e-012}, blow = {-70092.03989999999, -39.3690104}, ahigh = {463783.251, -1347.089334, 9.773637559999999, 0.001071395459, -3.84240366e-007, 5.7375319e-011, -3.157605982e-015}, bhigh = {-72395.1557, -23.91116343}, R = 35.86316804334235);
          constant IdealGases.Common.DataRecord Xe(name = "Xe", MM = 0.131293, Hf = 0, H0 = 47203.03443443291, Tlimit = 1000, alow = {0, 0, 2.5, 0, 0, 0, 0}, blow = {-745.375, 6.164454205}, ahigh = {4025.22668, -12.09507521, 2.514153347, -8.248102080000001e-006, 2.530232618e-009, -3.89233323e-013, 2.360439138e-017}, bhigh = {-668.5800730000001, 6.063710715}, R = 63.3276107637117);
          constant IdealGases.Common.DataRecord Xeplus(name = "Xeplus", MM = 0.1312924514, Hf = 8961309.042935578, H0 = 47203.23167033196, Tlimit = 1000, alow = {100.292362, -1.218753648, 2.506016493, -1.547411334e-005, 2.191372741e-008, -1.623684074e-011, 4.929132670000001e-015}, blow = {140766.5368, 7.516712465}, ahigh = {-12416.83887, -150.0654643, 2.964678293, -0.000469339666, 1.959138719e-007, -3.037761925e-011, 1.637361082e-015}, bhigh = {141496.6808, 4.565685735}, R = 63.32787537547646);
          constant IdealGases.Common.DataRecord Zn(name = "Zn", MM = 0.06539, Hf = 1994188.713870623, H0 = 94776.38782688484, Tlimit = 1000, alow = {0, 0, 2.5, 0, 0, 0, 0}, blow = {14938.05072, 5.11886101}, ahigh = {-175559.1489, 498.413924, 1.969386292, 0.0002608808787, -5.62719508e-008, 2.723336049e-012, 4.266685808e-016}, bhigh = {11737.73458, 8.961085649999999}, R = 127.1520415965744);
          constant IdealGases.Common.DataRecord Znplus(name = "Znplus", MM = 0.06538945139999999, Hf = 15950586.91683717, H0 = 94777.18297541796, Tlimit = 1000, alow = {0.000409834494, -4.34358162e-006, 2.500000019, -4.389036810000001e-011, 5.5639692e-014, -3.63822826e-017, 9.607881994999999e-021}, blow = {124697.9918, 5.8119955}, ahigh = {-343617.946, 956.7355239999999, 1.511478952, 0.000461346796, -8.786800980000001e-008, 7.558567779999999e-013, 1.168827311e-015}, bhigh = {118532.1933, 13.0074267}, R = 127.1531083681794);
          constant IdealGases.Common.DataRecord Zr(name = "Zr", MM = 0.091224, Hf = 6569747.116986759, H0 = 74712.91546084364, Tlimit = 1000, alow = {67158.9996, -943.5981740000001, 6.35975618, -0.0009790119730000001, -7.60822415e-006, 9.30871743e-009, -3.124675586e-012}, blow = {75880.19469999999, -16.65770522}, ahigh = {6006771.84, -15669.60605, 17.9698235, -0.00676340965, 1.733678968e-006, -2.064699786e-010, 9.334092610000001e-015}, bhigh = {173463.6249, -105.1117377}, R = 91.1434710163992);
          constant IdealGases.Common.DataRecord Zrplus(name = "Zrplus", MM = 0.0912234514, Hf = 13661468.32721131, H0 = 81907.57842779889, Tlimit = 1000, alow = {173984.2193, -2224.598466, 14.00787829, -0.02378785396, 2.641058912e-005, -1.442565487e-008, 3.135982142e-012}, blow = {159821.0714, -58.1672881}, ahigh = {729813.716, -2017.117556, 5.0374983, -0.000550337195, 1.023753499e-007, -1.261537793e-011, 7.092401041999999e-016}, bhigh = {162088.4945, -9.820640859999999}, R = 91.14401913541281);
          constant IdealGases.Common.DataRecord Zrminus(name = "Zrminus", MM = 0.09122454860000001, Hf = 6061442.961198715, H0 = 84949.69960311757, Tlimit = 1000, alow = {30466.62367, -807.427721, 9.21300611, -0.01614342054, 1.908653551e-005, -1.138888914e-008, 2.745019116e-012}, blow = {69030.3403, -28.62644371}, ahigh = {84718.61159999999, 317.543934, 2.251491246, 0.0001018645389, -2.285208242e-008, 2.647293502e-012, -1.235813604e-016}, bhigh = {64223.376, 10.81261057}, R = 91.14292290397806);
          constant IdealGases.Common.DataRecord ZrN(name = "ZrN", MM = 0.1052307, Hf = 6779124.342991162, H0 = 84223.2352345846, Tlimit = 1000, alow = {22591.09156, -180.2590198, 3.130058377, 0.00542770928, -8.264614840000001e-006, 5.95843822e-009, -1.670274535e-012}, blow = {85788.81490000001, 8.470724779999999}, ahigh = {-72557.9089, -81.267966, 4.5599244, 1.29116165e-005, 5.203830769999999e-009, -5.92743667e-013, 2.731548854e-017}, bhigh = {84686.48390000001, 1.493633264}, R = 79.01184730311593);
          constant IdealGases.Common.DataRecord ZrO(name = "ZrO", MM = 0.1072234, Hf = 782690.2336616821, H0 = 83658.11940304076, Tlimit = 1000, alow = {-509176.14, 8652.77009, -52.9474015, 0.1728961761, -0.0002457230895, 1.672135156e-007, -4.423012380000001e-011}, blow = {-30951.29818, 313.2576719}, ahigh = {464809.831, 344.231447, 4.81577918, -0.000466063314, 2.140489079e-007, -2.054364483e-011, 4.08466776e-016}, bhigh = {7317.3407, 1.502933548}, R = 77.54344667302101);
          constant IdealGases.Common.DataRecord ZrOplus(name = "ZrOplus", MM = 0.1072228514, Hf = 6720713.957808437, H0 = 88265.83024446596, Tlimit = 1000, alow = {10329.11549, 73.0466122, 2.606345299, 0.005099300290000001, -6.25033876e-006, 3.79746002e-009, -9.198416899999999e-013}, blow = {85332.33779999999, 12.67311019}, ahigh = {-493716.656, 669.990076, 4.57535347, -0.00069232379, 4.28060094e-007, -7.54766022e-011, 4.41427986e-015}, bhigh = {80188.80660000001, 2.928943704}, R = 77.5438434199298);
          constant IdealGases.Common.DataRecord ZrO2(name = "ZrO2", MM = 0.1232228, Hf = -2572922.681516732, H0 = 97451.63232778349, Tlimit = 1000, alow = {36376.49, -262.0658297, 3.69286687, 0.01214524156, -1.822445342e-005, 1.299215204e-008, -3.61590011e-012}, blow = {-38019.9088, 8.290857600000001}, ahigh = {2854363.887, -8738.589889999999, 16.21315114, -0.004417922830000001, 8.959096920000001e-007, -4.054667109999999e-011, -2.147732083e-015}, bhigh = {15275.10023, -74.8199549}, R = 67.47511012572349);
          annotation(
            Documentation(info = "<HTML>
<p>This package contains ideal gas models for the 1241 ideal gases from</p>
<blockquote>
  <p>McBride B.J., Zehe M.J., and Gordon S. (2002): <b>NASA Glenn Coefficients
  for Calculating Thermodynamic Properties of Individual Species</b>. NASA
  report TP-2002-211556</p>
</blockquote>

<pre>
 Ag        BaOH+           C2H4O_ethylen_o DF      In2I4    Nb      ScO2
 Ag+       Ba_OH_2         CH3CHO_ethanal  DOCl    In2I6    Nb+     Sc2O
 Ag-       BaS             CH3COOH         DO2     In2O     Nb-     Sc2O2
 Air       Ba2             OHCH2COOH       DO2-    K        NbCl5   Si
 Al        Be              C2H5            D2      K+       NbO     Si+
 Al+       Be+             C2H5Br          D2+     K-       NbOCl3  Si-
 Al-       Be++            C2H6            D2-     KAlF4    NbO2    SiBr
 AlBr      BeBr            CH3N2CH3        D2O     KBO2     Ne      SiBr2
 AlBr2     BeBr2           C2H5OH          D2O2    KBr      Ne+     SiBr3
 AlBr3     BeCl            CH3OCH3         D2S     KCN      Ni      SiBr4
 AlC       BeCl2           CH3O2CH3        e-      KCl      Ni+     SiC
 AlC2      BeF             CCN             F       KF       Ni-     SiC2
 AlCl      BeF2            CNC             F+      KH       NiCl    SiCl
 AlCl+     BeH             OCCN            F-      KI       NiCl2   SiCl2
 AlCl2     BeH+            C2N2            FCN     Kli      NiO     SiCl3
 AlCl3     BeH2            C2O             FCO     KNO2     NiS     SiCl4
 AlF       BeI             C3              FO      KNO3     O       SiF
 AlF+      BeI2            C3H3_1_propynl  FO2_FOO KNa      O+      SiFCl
 AlFCl     BeN             C3H3_2_propynl  FO2_OFO KO       O-      SiF2
 AlFCl2    BeO             C3H4_allene     F2      KOH      OD      SiF3
 AlF2      BeOH            C3H4_propyne    F2O     K2       OD-     SiF4
 AlF2-     BeOH+           C3H4_cyclo      F2O2    K2+      OH      SiH
 AlF2Cl    Be_OH_2         C3H5_allyl      FS2F    K2Br2    OH+     SiH+
 AlF3      BeS             C3H6_propylene  Fe      K2CO3    OH-     SiHBr3
 AlF4-     Be2             C3H6_cyclo      Fe+     K2C2N2   O2      SiHCl
 AlH       Be2Cl4          C3H6O_propylox  Fe_CO_5 K2Cl2    O2+     SiHCl3
 AlHCl     Be2F4           C3H6O_acetone   FeCl    K2F2     O2-     SiHF
 AlHCl2    Be2O            C3H6O_propanal  FeCl2   K2I2     O3      SiHF3
 AlHF      Be2OF2          C3H7_n_propyl   FeCl3   K2O      P       SiHI3
 AlHFCl    Be2O2           C3H7_i_propyl   FeO     K2O+     P+      SiH2
 AlHF2     Be3O3           C3H8            Fe_OH_2 K2O2     P-      SiH2Br2
 AlH2      Be4O4           C3H8O_1propanol Fe2Cl4  K2O2H2   PCl     SiH2Cl2
 AlH2Cl    Br              C3H8O_2propanol Fe2Cl6  K2SO4    PCl2    SiH2F2
 AlH2F     Br+             CNCOCN          Ga      Kr       PCl2-   SiH2I2
 AlH3      Br-             C3O2            Ga+     Kr+      PCl3    SiH3
 AlI       BrCl            C4              GaBr    li       PCl5    SiH3Br
 AlI2      BrF             C4H2_butadiyne  GaBr2   li+      PF      SiH3Cl
 AlI3      BrF3            C4H4_1_3-cyclo  GaBr3   li-      PF+     SiH3F
 AlN       BrF5            C4H6_butadiene  GaCl    liAlF4   PF-     SiH3I
 AlO       BrO             C4H6_1butyne    GaCl2   liBO2    PFCl    SiH4
 AlO+      OBrO            C4H6_2butyne    GaCl3   liBr     PFCl-   SiI
 AlO-      BrOO            C4H6_cyclo      GaF     liCl     PFCl2   SiI2
 AlOCl     BrO3            C4H8_1_butene   GaF2    liF      PFCl4   SiN
 AlOCl2    Br2             C4H8_cis2_buten GaF3    liH      PF2     SiO
 AlOF      BrBrO           C4H8_isobutene  GaH     liI      PF2-    SiO2
 AlOF2     BrOBr           C4H8_cyclo      GaI     liN      PF2Cl   SiS
 AlOF2-    C               C4H9_n_butyl    GaI2    liNO2    PF2Cl3  SiS2
 AlOH      C+              C4H9_i_butyl    GaI3    liNO3    PF3     Si2
 AlOHCl    C-              C4H9_s_butyl    GaO     liO      PF3Cl2  Si2C
 AlOHCl2   CBr             C4H9_t_butyl    GaOH    liOF     PF4Cl   Si2F6
 AlOHF     CBr2            C4H10_n_butane  Ga2Br2  liOH     PF5     Si2N
 AlOHF2    CBr3            C4H10_isobutane Ga2Br4  liON     PH      Si3
 AlO2      CBr4            C4N2            Ga2Br6  li2      PH2     Sn
 AlO2-     CCl             C5              Ga2Cl2  li2+     PH2-    Sn+
 Al_OH_2   CCl2            C5H6_1_3cyclo   Ga2Cl4  li2Br2   PH3     Sn-
 Al_OH_2Cl CCl2Br2         C5H8_cyclo      Ga2Cl6  li2F2    PN      SnBr
 Al_OH_2F  CCl3            C5H10_1_pentene Ga2F2   li2I2    PO      SnBr2
 Al_OH_3   CCl3Br          C5H10_cyclo     Ga2F4   li2O     PO-     SnBr3
 AlS       CCl4            C5H11_pentyl    Ga2F6   li2O+    POCl3   SnBr4
 AlS2      CF              C5H11_t_pentyl  Ga2I2   li2O2    POFCl2  SnCl
 Al2       CF+             C5H12_n_pentane Ga2I4   li2O2H2  POF2Cl  SnCl2
 Al2Br6    CFBr3           C5H12_i_pentane Ga2I6   li2SO4   POF3    SnCl3
 Al2C2     CFCl            CH3C_CH3_2CH3   Ga2O    li3+     PO2     SnCl4
 Al2Cl6    CFClBr2         C6D5_phenyl     Ge      li3Br3   PO2-    SnF
 Al2F6     CFCl2           C6D6            Ge+     li3Cl3   PS      SnF2
 Al2I6     CFCl2Br         C6H2            Ge-     li3F3    P2      SnF3
 Al2O      CFCl3           C6H5_phenyl     GeBr    li3I3    P2O3    SnF4
 Al2O+     CF2             C6H5O_phenoxy   GeBr2   Mg       P2O4    SnI
 Al2O2     CF2+            C6H6            GeBr3   Mg+      P2O5    SnI2
 Al2O2+    CF2Br2          C6H5OH_phenol   GeBr4   MgBr     P3      SnI3
 Al2O3     CF2Cl           C6H10_cyclo     GeCl    MgBr2    P3O6    SnI4
 Al2S      CF2ClBr         C6H12_1_hexene  GeCl2   MgCl     P4      SnO
 Al2S2     CF2Cl2          C6H12_cyclo     GeCl3   MgCl+    P4O6    SnO2
 Ar        CF3             C6H13_n_hexyl   GeCl4   MgCl2    P4O7    SnS
 Ar+       CF3+            C6H14_n_hexane  GeF     MgF      P4O8    SnS2
 B         CF3Br           C7H7_benzyl     GeF2    MgF+     P4O9    Sn2
 B+        CF3Cl           C7H8            GeF3    MgF2     P4O10   Sr
 B-        CF4             C7H8O_cresol_mx GeF4    MgF2+    Pb      Sr+
 BBr       CH+             C7H14_1_heptene GeH4    MgH      Pb+     SrBr
 BBr2      CHBr3           C7H15_n_heptyl  GeI     MgI      Pb-     SrBr2
 BBr3      CHCl            C7H16_n_heptane GeO     MgI2     PbBr    SrCl
 BC        CHClBr2         C7H16_2_methylh GeO2    MgN      PbBr2   SrCl+
 BC2       CHCl2           C8H8_styrene    GeS     MgO      PbBr3   SrCl2
 BCl       CHCl2Br         C8H10_ethylbenz GeS2    MgOH     PbBr4   SrF
 BCl+      CHCl3           C8H16_1_octene  Ge2     MgOH+    PbCl    SrF+
 BClOH     CHF             C8H17_n_octyl   H       Mg_OH_2  PbCl2   SrF2
 BCl_OH_2  CHFBr2          C8H18_n_octane  H+      MgS      PbCl3   SrH
 BCl2      CHFCl           C8H18_isooctane H-      Mg2      PbCl4   SrI
 BCl2+     CHFClBr         C9H19_n_nonyl   HAlO    Mg2F4    PbF     SrI2
 BCl2OH    CHFCl2          C10H8_naphthale HAlO2   Mn       PbF2    SrO
 BF        CHF2            C10H21_n_decyl  HBO     Mn+      PbF3    SrOH
 BFCl      CHF2Br          C12H9_o_bipheny HBO+    Mo       PbF4    SrOH+
 BFCl2     CHF2Cl          C12H10_biphenyl HBO2    Mo+      PbI     Sr_OH_2
 BFOH      CHF3            Ca              HBS     Mo-      PbI2    SrS
 BF_OH_2   CHI3            Ca+             HBS+    MoO      PbI3    Sr2
 BF2       CH2             CaBr            HCN     MoO2     PbI4    Ta
 BF2+      CH2Br2          CaBr2           HCO     MoO3     PbO     Ta+
 BF2-      CH2Cl           CaCl            HCO+    MoO3-    PbO2    Ta-
 BF2Cl     CH2ClBr         CaCl+           HCCN    Mo2O6    PbS     TaCl5
 BF2OH     CH2Cl2          CaCl2           HCCO    Mo3O9    PbS2    TaO
 BF3       CH2F            CaF             HCl     Mo4O12   Rb      TaO2
 BF4-      CH2FBr          CaF+            HD      Mo5O15   Rb+     Ti
 BH        CH2FCl          CaF2            HD+     N        Rb-     Ti+
 BHCl      CH2F2           CaH             HDO     N+       RbBO2   Ti-
 BHCl2     CH2I2           CaI             HDO2    N-       RbBr    TiCl
 BHF       CH3             CaI2            HF      NCO      RbCl    TiCl2
 BHFCl     CH3Br           CaO             HI      ND       RbF     TiCl3
 BHF2      CH3Cl           CaO+            HNC     ND2      RbH     TiCl4
 BH2       CH3F            CaOH            HNCO    ND3      RbI     TiO
 BH2Cl     CH3I            CaOH+           HNO     NF       RbK     TiO+
 BH2F      CH2OH           Ca_OH_2         HNO2    NF2      Rbli    TiOCl
 BH3       CH2OH+          CaS             HNO3    NF3      RbNO2   TiOCl2
 BH3NH3    CH3O            Ca2             HOCl    NH       RbNO3   TiO2
 BH4       CH4             Cd              HOF     NH+      RbNa    U
 BI        CH3OH           Cd+             HO2     NHF      RbO     UF
 BI2       CH3OOH          Cl              HO2-    NHF2     RbOH    UF+
 BI3       CI              Cl+             HPO     NH2      Rb2Br2  UF-
 BN        CI2             Cl-             HSO3F   NH2F     Rb2Cl2  UF2
 BO        CI3             ClCN            H2      NH3      Rb2F2   UF2+
 BO-       CI4             ClF             H2+     NH2OH    Rb2I2   UF2-
 BOCl      CN              ClF3            H2-     NH4+     Rb2O    UF3
 BOCl2     CN+             ClF5            HBOH    NO       Rb2O2   UF3+
 BOF       CN-             ClO             HCOOH   NOCl     Rb2O2H2 UF3-
 BOF2      CNN             ClO2            H2F2    NOF      Rb2SO4  UF4
 BOH       CO              Cl2             H2O     NOF3     Rn      UF4+
 BO2       CO+             Cl2O            H2O+    NO2      Rn+     UF4-
 BO2-      COCl            Co              H2O2    NO2-     S       UF5
 B_OH_2    COCl2           Co+             H2S     NO2Cl    S+      UF5+
 BS        COFCl           Co-             H2SO4   NO2F     S-      UF5-
 BS2       COF2            Cr              H2BOH   NO3      SCl     UF6
 B2        COHCl           Cr+             HB_OH_2 NO3-     SCl2    UF6-
 B2C       COHF            Cr-             H3BO3   NO3F     SCl2+   UO
 B2Cl4     COS             CrN             H3B3O3  N2       SD      UO+
 B2F4      CO2             CrO             H3B3O6  N2+      SF      UOF
 B2H       CO2+            CrO2            H3F3    N2-      SF+     UOF2
 B2H2      COOH            CrO3            H3O+    NCN      SF-     UOF3
 B2H3      CP              CrO3-           H4F4    N2D2_cis SF2     UOF4
 B2H3_db   CS              Cs              H5F5    N2F2     SF2+    UO2
 B2H4      CS2             Cs+             H6F6    N2F4     SF2-    UO2+
 B2H4_db   C2              Cs-             H7F7    N2H2     SF3     UO2-
 B2H5      C2+             CsBO2           He      NH2NO2   SF3+    UO2F
 B2H5_db   C2-             CsBr            He+     N2H4     SF3-    UO2F2
 B2H6      C2Cl            CsCl            Hg      N2O      SF4     UO3
 B2O       C2Cl2           CsF             Hg+     N2O+     SF4+    UO3-
 B2O2      C2Cl3           CsH             HgBr2   N2O3     SF4-    V
 B2O3      C2Cl4           CsI             I       N2O4     SF5     V+
 B2_OH_4   C2Cl6           Csli            I+      N2O5     SF5+    V-
 B2S       C2F             CsNO2           I-      N3       SF5-    VCl4
 B2S2      C2FCl           CsNO3           IF5     N3H      SF6     VN
 B2S3      C2FCl3          CsNa            IF7     Na       SF6-    VO
 B3H7_C2v  C2F2            CsO             I2      Na+      SH      VO2
 B3H7_Cs   C2F2Cl2         CsOH            In      Na-      SH-     V4O10
 B3H9      C2F3            CsRb            In+     NaAlF4   SN      W
 B3N3H6    C2F3Cl          Cs2             InBr    NaBO2    SO      W+
 B3O3Cl3   C2F4            Cs2Br2          InBr2   NaBr     SO-     W-
 B3O3FCl2  C2F6            Cs2CO3          InBr3   NaCN     SOF2    WCl6
 B3O3F2Cl  C2H             Cs2Cl2          InCl    NaCl     SO2     WO
 B3O3F3    C2HCl           Cs2F2           InCl2   NaF      SO2-    WOCl4
 B4H4      C2HCl3          Cs2I2           InCl3   NaH      SO2Cl2  WO2
 B4H10     C2HF            Cs2O            InF     NaI      SO2FCl  WO2Cl2
 B4H12     C2HFCl2         Cs2O+           InF2    Nali     SO2F2   WO3
 B5H9      C2HF2Cl         Cs2O2           InF3    NaNO2    SO3     WO3-
 Ba        C2HF3           Cs2O2H2         InH     NaNO3    S2      Xe
 Ba+       C2H2_vinylidene Cs2SO4          InI     NaO      S2-     Xe+
 BaBr      C2H2Cl2         Cu              InI2    NaOH     S2Cl2   Zn
 BaBr2     C2H2FCl         Cu+             InI3    NaOH+    S2F2    Zn+
 BaCl      C2H2F2          Cu-             InO     Na2      S2O     Zr
 BaCl+     CH2CO_ketene    CuCl            InOH    Na2Br2   S3      Zr+
 BaCl2     O_CH_2O         CuF             In2Br2  Na2Cl2   S4      Zr-
 BaF       HO_CO_2OH       CuF2            In2Br4  Na2F2    S5      ZrN
 BaF+      C2H3_vinyl      CuO             In2Br6  Na2I2    S6      ZrO
 BaF2      CH2Br-COOH      Cu2             In2Cl2  Na2O     S7      ZrO+
 BaH       C2H3Cl          Cu3Cl3          In2Cl4  Na2O+    S8      ZrO2
 BaI       CH2Cl-COOH      D               In2Cl6  Na2O2    Sc
 BaI2      C2H3F           D+              In2F2   Na2O2H2  Sc+
 BaO       CH3CN           D-              In2F4   Na2SO4   Sc-
 BaO+      CH3CO_acetyl    DBr             In2F6   Na3Cl3   ScO
 BaOH      C2H4            DCl             In2I2   Na3F3    ScO+
</pre>
</HTML>"));
        end SingleGasesData;
        annotation(
          Documentation(info = "<html>

</html>"));
      end Common;

      package SingleGases "Media models of ideal gases from NASA tables"
        extends Modelica.Icons.VariantsPackage;

        package Ar "Ideal gas \"Ar\" from NASA Glenn coefficients"
          extends Common.SingleGasNasa(mediumName = "Argon", data = Common.SingleGasesData.Ar, fluidConstants = {Common.FluidData.Ar});
          annotation(
            Documentation(info = "<HTML>
      <IMG src=\"modelica://Modelica/Resources/Images/Media/IdealGases/SingleGases/Ar.png\"></HTML>"));
        end Ar;

        package CH4 "Ideal gas \"CH4\" from NASA Glenn coefficients"
          extends Common.SingleGasNasa(mediumName = "Methane", data = Common.SingleGasesData.CH4, fluidConstants = {Common.FluidData.CH4});
          annotation(
            Documentation(info = "<HTML>
      <IMG src=\"modelica://Modelica/Resources/Images/Media/IdealGases/SingleGases/CH4.png\"></HTML>"));
        end CH4;

        package CH3OH "Ideal gas \"CH3OH\" from NASA Glenn coefficients"
          extends Common.SingleGasNasa(mediumName = "Methanol", data = Common.SingleGasesData.CH3OH, fluidConstants = {Common.FluidData.CH3OH});
          annotation(
            Documentation(info = "<HTML>
      <IMG src=\"modelica://Modelica/Resources/Images/Media/IdealGases/SingleGases/CH3OH.png\"></HTML>"));
        end CH3OH;

        package CO "Ideal gas \"CO\" from NASA Glenn coefficients"
          extends Common.SingleGasNasa(mediumName = "Carbon Monoxide", data = Common.SingleGasesData.CO, fluidConstants = {Common.FluidData.CO});
          annotation(
            Documentation(info = "<HTML>
      <IMG src=\"modelica://Modelica/Resources/Images/Media/IdealGases/SingleGases/CO.png\"></HTML>"));
        end CO;

        package CO2 "Ideal gas \"CO2\" from NASA Glenn coefficients"
          extends Common.SingleGasNasa(mediumName = "Carbon Dioxide", data = Common.SingleGasesData.CO2, fluidConstants = {Common.FluidData.CO2});
          annotation(
            Documentation(info = "<HTML>
      <IMG src=\"modelica://Modelica/Resources/Images/Media/IdealGases/SingleGases/CO2.png\"></HTML>"));
        end CO2;

        package C2H2_vinylidene "Ideal gas \"C2H2_vinylidene\" from NASA Glenn coefficients"
          extends Common.SingleGasNasa(mediumName = "Acetylene", data = Common.SingleGasesData.C2H2_vinylidene, fluidConstants = {Common.FluidData.C2H2_vinylidene});
          annotation(
            Documentation(info = "<HTML>
      <IMG src=\"modelica://Modelica/Resources/Images/Media/IdealGases/SingleGases/C2H2_vinylidene.png\"></HTML>"));
        end C2H2_vinylidene;

        package C2H4 "Ideal gas \"C2H4\" from NASA Glenn coefficients"
          extends Common.SingleGasNasa(mediumName = "Ethylene", data = Common.SingleGasesData.C2H4, fluidConstants = {Common.FluidData.C2H4});
          annotation(
            Documentation(info = "<HTML>
      <IMG src=\"modelica://Modelica/Resources/Images/Media/IdealGases/SingleGases/C2H4.png\"></HTML>"));
        end C2H4;

        package C2H5OH "Ideal gas \"C2H5OH\" from NASA Glenn coefficients"
          extends Common.SingleGasNasa(mediumName = "Ethanol", data = Common.SingleGasesData.C2H5OH, fluidConstants = {Common.FluidData.C2H5OH});
          annotation(
            Documentation(info = "<HTML>
      <IMG src=\"modelica://Modelica/Resources/Images/Media/IdealGases/SingleGases/C2H5OH.png\"></HTML>"));
        end C2H5OH;

        package C2H6 "Ideal gas \"C2H6\" from NASA Glenn coefficients"
          extends Common.SingleGasNasa(mediumName = "Ethane", data = Common.SingleGasesData.C2H6, fluidConstants = {Common.FluidData.C2H6});
          annotation(
            Documentation(info = "<HTML>
      <IMG src=\"modelica://Modelica/Resources/Images/Media/IdealGases/SingleGases/C2H6.png\"></HTML>"));
        end C2H6;

        package C3H6_propylene "Ideal gas \"C3H6_propylene\" from NASA Glenn coefficients"
          extends Common.SingleGasNasa(mediumName = "Propylene", data = Common.SingleGasesData.C3H6_propylene, fluidConstants = {Common.FluidData.C3H6_propylene});
          annotation(
            Documentation(info = "<HTML>
      <IMG src=\"modelica://Modelica/Resources/Images/Media/IdealGases/SingleGases/C3H6_propylene.png\"></HTML>"));
        end C3H6_propylene;

        package C3H8 "Ideal gas \"C3H8\" from NASA Glenn coefficients"
          extends Common.SingleGasNasa(mediumName = "Propane", data = Common.SingleGasesData.C3H8, fluidConstants = {Common.FluidData.C3H8});
          annotation(
            Documentation(info = "<HTML>
      <IMG src=\"modelica://Modelica/Resources/Images/Media/IdealGases/SingleGases/C3H8.png\"></HTML>"));
        end C3H8;

        package C3H8O_1propanol "Ideal gas \"C3H8O_1propanol\" from NASA Glenn coefficients"
          extends Common.SingleGasNasa(mediumName = "1-Propanol", data = Common.SingleGasesData.C3H8O_1propanol, fluidConstants = {Common.FluidData.C3H7OH});
          annotation(
            Documentation(info = "<HTML>
      <IMG src=\"modelica://Modelica/Resources/Images/Media/IdealGases/SingleGases/C3H8O_1propanol.png\"></HTML>"));
        end C3H8O_1propanol;

        package C4H8_1_butene "Ideal gas \"C4H8_1_butene\" from NASA Glenn coefficients"
          extends Common.SingleGasNasa(mediumName = "1-Butene", data = Common.SingleGasesData.C4H8_1_butene, fluidConstants = {Common.FluidData.C4H8_1_butene});
          annotation(
            Documentation(info = "<HTML>
      <IMG src=\"modelica://Modelica/Resources/Images/Media/IdealGases/SingleGases/C4H8_1_butene.png\"></HTML>"));
        end C4H8_1_butene;

        package C4H10_n_butane "Ideal gas \"C4H10_n_butane\" from NASA Glenn coefficients"
          extends Common.SingleGasNasa(mediumName = "N-Butane", data = Common.SingleGasesData.C4H10_n_butane, fluidConstants = {Common.FluidData.C4H10_n_butane});
          annotation(
            Documentation(info = "<HTML>
      <IMG src=\"modelica://Modelica/Resources/Images/Media/IdealGases/SingleGases/C4H10_n_butane.png\"></HTML>"));
        end C4H10_n_butane;

        package C5H10_1_pentene "Ideal gas \"C5H10_1_pentene\" from NASA Glenn coefficients"
          extends Common.SingleGasNasa(mediumName = "1-Pentene", data = Common.SingleGasesData.C5H10_1_pentene, fluidConstants = {Common.FluidData.C5H10_1_pentene});
          annotation(
            Documentation(info = "<HTML>
      <IMG src=\"modelica://Modelica/Resources/Images/Media/IdealGases/SingleGases/C5H10_1_pentene.png\"></HTML>"));
        end C5H10_1_pentene;

        package C5H12_n_pentane "Ideal gas \"C5H12_n_pentane\" from NASA Glenn coefficients"
          extends Common.SingleGasNasa(mediumName = "N-Pentane", data = Common.SingleGasesData.C5H12_n_pentane, fluidConstants = {Common.FluidData.C5H12_n_pentane});
          annotation(
            Documentation(info = "<HTML>
      <IMG src=\"modelica://Modelica/Resources/Images/Media/IdealGases/SingleGases/C5H12_n_pentane.png\"></HTML>"));
        end C5H12_n_pentane;

        package C6H6 "Ideal gas \"C6H6\" from NASA Glenn coefficients"
          extends Common.SingleGasNasa(mediumName = "Benzene", data = Common.SingleGasesData.C6H6, fluidConstants = {Common.FluidData.C6H6});
          annotation(
            Documentation(info = "<HTML>
      <IMG src=\"modelica://Modelica/Resources/Images/Media/IdealGases/SingleGases/C6H6.png\"></HTML>"));
        end C6H6;

        package C6H12_1_hexene "Ideal gas \"C6H12_1_hexene\" from NASA Glenn coefficients"
          extends Common.SingleGasNasa(mediumName = "1-Hexene", data = Common.SingleGasesData.C6H12_1_hexene, fluidConstants = {Common.FluidData.C6H12_1_hexene});
          annotation(
            Documentation(info = "<HTML>
      <IMG src=\"modelica://Modelica/Resources/Images/Media/IdealGases/SingleGases/C6H12_1_hexene.png\"></HTML>"));
        end C6H12_1_hexene;

        package C6H14_n_hexane "Ideal gas \"C6H14_n_hexane\" from NASA Glenn coefficients"
          extends Common.SingleGasNasa(mediumName = "N-Hexane", data = Common.SingleGasesData.C6H14_n_hexane, fluidConstants = {Common.FluidData.C6H14_n_hexane});
          annotation(
            Documentation(info = "<HTML>
      <IMG src=\"modelica://Modelica/Resources/Images/Media/IdealGases/SingleGases/C6H14_n_hexane.png\"></HTML>"));
        end C6H14_n_hexane;

        package C7H14_1_heptene "Ideal gas \"C7H14_1_heptene\" from NASA Glenn coefficients"
          extends Common.SingleGasNasa(mediumName = "1-Heptene", data = Common.SingleGasesData.C7H14_1_heptene, fluidConstants = {Common.FluidData.C7H14_1_heptene});
          annotation(
            Documentation(info = "<HTML>
      <IMG src=\"modelica://Modelica/Resources/Images/Media/IdealGases/SingleGases/C7H14_1_heptene.png\"></HTML>"));
        end C7H14_1_heptene;

        package C7H16_n_heptane "Ideal gas \"C7H16_n_heptane\" from NASA Glenn coefficients"
          extends Common.SingleGasNasa(mediumName = "N-Heptane", data = Common.SingleGasesData.C7H16_n_heptane, fluidConstants = {Common.FluidData.C7H16_n_heptane});
          annotation(
            Documentation(info = "<HTML>
      <IMG src=\"modelica://Modelica/Resources/Images/Media/IdealGases/SingleGases/C7H16_n_heptane.png\"></HTML>"));
        end C7H16_n_heptane;

        package C8H10_ethylbenz "Ideal gas \"C8H10_ethylbenz\" from NASA Glenn coefficients"
          extends Common.SingleGasNasa(mediumName = "Ethylbenzene", data = Common.SingleGasesData.C8H10_ethylbenz, fluidConstants = {Common.FluidData.C8H10_ethylbenz});
          annotation(
            Documentation(info = "<HTML>
      <IMG src=\"modelica://Modelica/Resources/Images/Media/IdealGases/SingleGases/C8H10_ethylbenz.png\"></HTML>"));
        end C8H10_ethylbenz;

        package C8H18_n_octane "Ideal gas \"C8H18_n_octane\" from NASA Glenn coefficients"
          extends Common.SingleGasNasa(mediumName = "N-Octane", data = Common.SingleGasesData.C8H18_n_octane, fluidConstants = {Common.FluidData.C8H18_n_octane});
          annotation(
            Documentation(info = "<HTML>
      <IMG src=\"modelica://Modelica/Resources/Images/Media/IdealGases/SingleGases/C8H18_n_octane.png\"></HTML>"));
        end C8H18_n_octane;

        package CL2 "Ideal gas \"Cl2\" from NASA Glenn coefficients"
          extends Common.SingleGasNasa(mediumName = "Chlorine", data = Common.SingleGasesData.CL2, fluidConstants = {Common.FluidData.CL2});
          annotation(
            Documentation(info = "<HTML>
      <IMG src=\"modelica://Modelica/Resources/Images/Media/IdealGases/SingleGases/CL2.png\"></HTML>"));
        end CL2;

        package F2 "Ideal gas \"F2\" from NASA Glenn coefficients"
          extends Common.SingleGasNasa(mediumName = "Fluorine", data = Common.SingleGasesData.F2, fluidConstants = {Common.FluidData.F2});
          annotation(
            Documentation(info = "<HTML>
      <IMG src=\"modelica://Modelica/Resources/Images/Media/IdealGases/SingleGases/F2.png\"></HTML>"));
        end F2;

        package H2 "Ideal gas \"H2\" from NASA Glenn coefficients"
          extends Common.SingleGasNasa(mediumName = "Hydrogen", data = Common.SingleGasesData.H2, fluidConstants = {Common.FluidData.H2});
          annotation(
            Documentation(info = "<HTML>
      <IMG src=\"modelica://Modelica/Resources/Images/Media/IdealGases/SingleGases/H2.png\"></HTML>"));
        end H2;

        package H2O "Ideal gas \"H2O\" from NASA Glenn coefficients"
          extends Common.SingleGasNasa(mediumName = "IdealGasSteam", data = Common.SingleGasesData.H2O, fluidConstants = {Common.FluidData.H2O});
          annotation(
            Documentation(info = "<HTML>
      <IMG src=\"modelica://Modelica/Resources/Images/Media/IdealGases/SingleGases/H2O.png\"></HTML>"));
        end H2O;

        package He "Ideal gas \"He\" from NASA Glenn coefficients"
          extends Common.SingleGasNasa(mediumName = "Helium", data = Common.SingleGasesData.He, fluidConstants = {Common.FluidData.He});
          annotation(
            Documentation(info = "<HTML>
      <IMG src=\"modelica://Modelica/Resources/Images/Media/IdealGases/SingleGases/He.png\"></HTML>"));
        end He;

        package NH3 "Ideal gas \"NH3\" from NASA Glenn coefficients"
          extends Common.SingleGasNasa(mediumName = "IdealGasAmmonia", data = Common.SingleGasesData.NH3, fluidConstants = {Common.FluidData.NH3});
          annotation(
            Documentation(info = "<HTML>
      <IMG src=\"modelica://Modelica/Resources/Images/Media/IdealGases/SingleGases/NH3.png\"></HTML>"));
        end NH3;

        package NO "Ideal gas \"NO\" from NASA Glenn coefficients"
          extends Common.SingleGasNasa(mediumName = "Nitric Oxide", data = Common.SingleGasesData.NO, fluidConstants = {Common.FluidData.NO});
          annotation(
            Documentation(info = "<HTML>
      <IMG src=\"modelica://Modelica/Resources/Images/Media/IdealGases/SingleGases/NO.png\"></HTML>"));
        end NO;

        package NO2 "Ideal gas \"NO2\" from NASA Glenn coefficients"
          extends Common.SingleGasNasa(mediumName = "Nitrogen Dioxide", data = Common.SingleGasesData.NO2, fluidConstants = {Common.FluidData.NO2});
          annotation(
            Documentation(info = "<HTML>
      <IMG src=\"modelica://Modelica/Resources/Images/Media/IdealGases/SingleGases/NO2.png\"></HTML>"));
        end NO2;

        package N2 "Ideal gas \"N2\" from NASA Glenn coefficients"
          extends Common.SingleGasNasa(mediumName = "Nitrogen", data = Common.SingleGasesData.N2, fluidConstants = {Common.FluidData.N2});
          annotation(
            Documentation(info = "<HTML>
      <IMG src=\"modelica://Modelica/Resources/Images/Media/IdealGases/SingleGases/N2.png\"></HTML>"));
        end N2;

        package N2O "Ideal gas \"N2O\" from NASA Glenn coefficients"
          extends Common.SingleGasNasa(mediumName = "Nitrous Oxide", data = Common.SingleGasesData.N2O, fluidConstants = {Common.FluidData.N2O});
          annotation(
            Documentation(info = "<HTML>
      <IMG src=\"modelica://Modelica/Resources/Images/Media/IdealGases/SingleGases/N2O.png\"></HTML>"));
        end N2O;

        package Ne "Ideal gas \"Ne\" from NASA Glenn coefficients"
          extends Common.SingleGasNasa(mediumName = "Neon", data = Common.SingleGasesData.Ne, fluidConstants = {Common.FluidData.Ne});
          annotation(
            Documentation(info = "<HTML>
      <IMG src=\"modelica://Modelica/Resources/Images/Media/IdealGases/SingleGases/Ne.png\"></HTML>"));
        end Ne;

        package O2 "Ideal gas \"O2\" from NASA Glenn coefficients"
          extends Common.SingleGasNasa(mediumName = "Oxygen", data = Common.SingleGasesData.O2, fluidConstants = {Common.FluidData.O2});
          annotation(
            Documentation(info = "<HTML>
      <IMG src=\"modelica://Modelica/Resources/Images/Media/IdealGases/SingleGases/O2.png\"></HTML>"));
        end O2;

        package SO2 "Ideal gas \"SO2\" from NASA Glenn coefficients"
          extends Common.SingleGasNasa(mediumName = "Sulfur Dioxide", data = Common.SingleGasesData.SO2, fluidConstants = {Common.FluidData.SO2});
          annotation(
            Documentation(info = "<HTML>
      <IMG src=\"modelica://Modelica/Resources/Images/Media/IdealGases/SingleGases/SO2.png\"></HTML>"));
        end SO2;

        package SO3 "Ideal gas \"SO3\" from NASA Glenn coefficients"
          extends Common.SingleGasNasa(mediumName = "Sulfur Trioxide", data = Common.SingleGasesData.SO3, fluidConstants = {Common.FluidData.SO3});
          annotation(
            Documentation(info = "<HTML>
      <IMG src=\"modelica://Modelica/Resources/Images/Media/IdealGases/SingleGases/SO3.png\"></HTML>"));
        end SO3;
        annotation(
          Documentation(info = "<HTML>
<p>
This package contains medium
models for the following 37 gases
(see also the description in
<a href=\"modelica://Modelica.Media.IdealGases\">Modelica.Media.IdealGases</a>):
</p>
<pre>
Argon             Methane          Methanol       Carbon Monoxide  Carbon Dioxide
Acetylene         Ethylene         Ethanol        Ethane           Propylene
Propane           1-Propanol       1-Butene       N-Butane         1-Pentene
N-Pentane         Benzene          1-Hexene       N-Hexane         1-Heptane
N-Heptane         Ethylbenzene     N-Octane       Chlorine         Fluorine
Hydrogen          Steam            Helium         Ammonia          Nitric Oxide
Nitrogen Dioxide  Nitrogen         Nitrous        Oxide            Neon Oxygen
Sulfur Dioxide    Sulfur Trioxide
</pre>
</HTML>"));
      end SingleGases;

      package MixtureGases "Medium models consisting of mixtures of ideal gases"
        extends Modelica.Icons.VariantsPackage;

        package CombustionAir "Air as mixture of N2 and O2"
          extends Common.MixtureGasNasa(mediumName = "CombustionAirN2O2", data = {Common.SingleGasesData.N2, Common.SingleGasesData.O2}, fluidConstants = {Common.FluidData.N2, Common.FluidData.O2}, substanceNames = {"Nitrogen", "Oxygen"}, reference_X = {0.768, 0.232});
          annotation(
            Documentation(info = "<html>

</html>"));
        end CombustionAir;

        package AirSteam "Air and steam mixture (no condensation!, pseudo-mixture)"
          extends Common.MixtureGasNasa(mediumName = "MoistAir", data = {Common.SingleGasesData.H2O, Common.SingleGasesData.Air}, fluidConstants = {Common.FluidData.H2O, Common.FluidData.N2}, substanceNames = {"Water", "Air"}, reference_X = {0.0, 1.0});
          annotation(
            Documentation(info = "<html>

</html>"));
        end AirSteam;

        package simpleMoistAir = AirSteam(reference_X = {0.03, 0.97}) "Moist air without condensation";

        package FlueGasLambdaOnePlus "Simple flue gas for over0stochiometric O2-fuel ratios"
          extends Common.MixtureGasNasa(mediumName = "FlueGasLambda1plus", data = {Common.SingleGasesData.N2, Common.SingleGasesData.O2, Common.SingleGasesData.H2O, Common.SingleGasesData.CO2}, fluidConstants = {Common.FluidData.N2, Common.FluidData.O2, Common.FluidData.H2O, Common.FluidData.CO2}, substanceNames = {"Nitrogen", "Oxygen", "Water", "Carbondioxide"}, reference_X = {0.768, 0.232, 0.0, 0.0});
          annotation(
            Documentation(info = "<html>

</html>"));
        end FlueGasLambdaOnePlus;

        package FlueGasSixComponents "Simplest flue gas for over-and understochiometric combustion of hydrocarbons"
          extends Common.MixtureGasNasa(mediumName = "FlueGasSixComponents", data = {Common.SingleGasesData.N2, Common.SingleGasesData.H2, Common.SingleGasesData.CO, Common.SingleGasesData.O2, Common.SingleGasesData.H2O, Common.SingleGasesData.CO2}, fluidConstants = {Common.FluidData.N2, Common.FluidData.H2, Common.FluidData.CO, Common.FluidData.O2, Common.FluidData.H2O, Common.FluidData.CO2}, substanceNames = {"Nitrogen", "Hydrogen,", "Carbonmonoxide", "Oxygen", "Water", "Carbondioxide"}, reference_X = {0.768, 0.0, 0.0, 0.232, 0.0, 0.0});
          annotation(
            Documentation(info = "<html>

</html>"));
        end FlueGasSixComponents;

        package SimpleNaturalGas "Simple natural gas mix with 6 components"
          extends Common.MixtureGasNasa(mediumName = "SimpleNaturalGas", data = {Common.SingleGasesData.CH4, Common.SingleGasesData.C2H6, Common.SingleGasesData.C3H8, Common.SingleGasesData.C4H10_n_butane, Common.SingleGasesData.N2, Common.SingleGasesData.CO2}, fluidConstants = {Common.FluidData.CH4, Common.FluidData.C2H6, Common.FluidData.C3H8, Common.FluidData.C4H10_n_butane, Common.FluidData.N2, Common.FluidData.CO2}, substanceNames = {"Methane", "Ethane", "Propane", "N-Butane,", "Nitrogen", "Carbondioxide"}, reference_X = {0.92, 0.048, 0.005, 0.002, 0.015, 0.01});
          annotation(
            Documentation(info = "<html>

</html>"));
        end SimpleNaturalGas;

        package SimpleNaturalGasFixedComposition "Same as SimpleNaturalGas but with fixed composition"
          extends SimpleNaturalGas(fixedX = true);
          annotation(
            Documentation(info = "<html>

</html>"));
        end SimpleNaturalGasFixedComposition;
        annotation(
          Documentation(info = "<html>

</html>"));
      end MixtureGases;
      annotation(
        Documentation(info = "<HTML>
<p>This package contains data for the 1241 ideal gases from</p>
<blockquote>
  <p>McBride B.J., Zehe M.J., and Gordon S. (2002): <b>NASA Glenn Coefficients
  for Calculating Thermodynamic Properties of Individual Species</b>. NASA
  report TP-2002-211556</p>
</blockquote>
<p>Medium models for some of these gases are available in package
<a href=\"modelica://Modelica.Media.IdealGases.SingleGases\">IdealGases.SingleGases</a>
and some examples for mixtures are available in package <a href=\"modelica://Modelica.Media.IdealGases.MixtureGases\">IdealGases.MixtureGases</a>
</p>
<h4>Using and Adapting Medium Models</h4>
<p>
The data records allow computing the ideal gas specific enthalpy, specific entropy and heat capacity of the substances listed below. From them, even the Gibbs energy and equilibrium constants for reactions can be computed. Critical data that is needed for computing the viscosity and thermal conductivity is not included. In order to add mixtures or single substance medium packages that are
subtypes of
<a href=\"modelica://Modelica.Media.Interfaces.PartialMedium\">Interfaces.PartialMedium</a>
(i.e., can be utilized at all places where PartialMedium is defined),
a few additional steps have to be performed:
</p>
<ol>
<li>
All single gas media need to define a constant instance of record
<a href=\"modelica://Modelica.Media.Interfaces.PartialMedium.FluidConstants\">IdealGases.Common.SingleGasNasa.FluidConstants</a>.
For 37 ideal gases such records are provided in package
<a href=\"modelica://Modelica.Media.IdealGases.Common.FluidData\">IdealGases.Common.FluidData</a>.
For the other gases, such a record instance has to be provided by the user, e.g., by getting
the data from a commercial or public data base. A public source of the needed data is for example the <a href=\"http://webbook.nist.gov/chemistry/\"> NIST Chemistry WebBook</a></li>

<li>When the data is available, and a user has an instance of a
<a href=\"modelica://Modelica.Media.Interfaces.PartialMedium.FluidConstants\">FluidConstants</a> record filled with data, a medium package has to be written. Note that only the dipole moment, the accentric factor and critical data are necessary for the viscosity and thermal conductivity functions.</li>
<li><ul>
<li>For single components, a new package following the pattern in
<a href=\"modelica://Modelica.Media.IdealGases.SingleGases\">IdealGases.SingleGases</a> has to be created, pointing both to a data record for cp and to a user-defined fluidContants record.</li>
<li>For mixtures of several components, a new package following the pattern in
<a href=\"modelica://Modelica.Media.IdealGases.MixtureGases\">IdealGases.MixtureGases</a> has to be created, building an array of data records for cp and an array of (partly) user-defined fluidContants records.</li>
</ul></li>
</ol>
<p>Note that many properties can computed for the full set of 1241 gases listed below, but due to the missing viscosity and thermal conductivity functions, no fully Modelica.Media-compliant media can be defined.</p>
<p>
Data records for heat capacity, specific enthalpy and specific entropy exist for the following substances and ions:
</p>
<pre>
 Ag        BaOH+           C2H4O_ethylen_o DF      In2I4    Nb      ScO2
 Ag+       Ba_OH_2         CH3CHO_ethanal  DOCl    In2I6    Nb+     Sc2O
 Ag-       BaS             CH3COOH         DO2     In2O     Nb-     Sc2O2
 Air       Ba2             OHCH2COOH       DO2-    K        NbCl5   Si
 Al        Be              C2H5            D2      K+       NbO     Si+
 Al+       Be+             C2H5Br          D2+     K-       NbOCl3  Si-
 Al-       Be++            C2H6            D2-     KAlF4    NbO2    SiBr
 AlBr      BeBr            CH3N2CH3        D2O     KBO2     Ne      SiBr2
 AlBr2     BeBr2           C2H5OH          D2O2    KBr      Ne+     SiBr3
 AlBr3     BeCl            CH3OCH3         D2S     KCN      Ni      SiBr4
 AlC       BeCl2           CH3O2CH3        e-      KCl      Ni+     SiC
 AlC2      BeF             CCN             F       KF       Ni-     SiC2
 AlCl      BeF2            CNC             F+      KH       NiCl    SiCl
 AlCl+     BeH             OCCN            F-      KI       NiCl2   SiCl2
 AlCl2     BeH+            C2N2            FCN     Kli      NiO     SiCl3
 AlCl3     BeH2            C2O             FCO     KNO2     NiS     SiCl4
 AlF       BeI             C3              FO      KNO3     O       SiF
 AlF+      BeI2            C3H3_1_propynl  FO2_FOO KNa      O+      SiFCl
 AlFCl     BeN             C3H3_2_propynl  FO2_OFO KO       O-      SiF2
 AlFCl2    BeO             C3H4_allene     F2      KOH      OD      SiF3
 AlF2      BeOH            C3H4_propyne    F2O     K2       OD-     SiF4
 AlF2-     BeOH+           C3H4_cyclo      F2O2    K2+      OH      SiH
 AlF2Cl    Be_OH_2         C3H5_allyl      FS2F    K2Br2    OH+     SiH+
 AlF3      BeS             C3H6_propylene  Fe      K2CO3    OH-     SiHBr3
 AlF4-     Be2             C3H6_cyclo      Fe+     K2C2N2   O2      SiHCl
 AlH       Be2Cl4          C3H6O_propylox  Fe_CO_5 K2Cl2    O2+     SiHCl3
 AlHCl     Be2F4           C3H6O_acetone   FeCl    K2F2     O2-     SiHF
 AlHCl2    Be2O            C3H6O_propanal  FeCl2   K2I2     O3      SiHF3
 AlHF      Be2OF2          C3H7_n_propyl   FeCl3   K2O      P       SiHI3
 AlHFCl    Be2O2           C3H7_i_propyl   FeO     K2O+     P+      SiH2
 AlHF2     Be3O3           C3H8            Fe_OH_2 K2O2     P-      SiH2Br2
 AlH2      Be4O4           C3H8O_1propanol Fe2Cl4  K2O2H2   PCl     SiH2Cl2
 AlH2Cl    Br              C3H8O_2propanol Fe2Cl6  K2SO4    PCl2    SiH2F2
 AlH2F     Br+             CNCOCN          Ga      Kr       PCl2-   SiH2I2
 AlH3      Br-             C3O2            Ga+     Kr+      PCl3    SiH3
 AlI       BrCl            C4              GaBr    li       PCl5    SiH3Br
 AlI2      BrF             C4H2_butadiyne  GaBr2   li+      PF      SiH3Cl
 AlI3      BrF3            C4H4_1_3-cyclo  GaBr3   li-      PF+     SiH3F
 AlN       BrF5            C4H6_butadiene  GaCl    liAlF4   PF-     SiH3I
 AlO       BrO             C4H6_1butyne    GaCl2   liBO2    PFCl    SiH4
 AlO+      OBrO            C4H6_2butyne    GaCl3   liBr     PFCl-   SiI
 AlO-      BrOO            C4H6_cyclo      GaF     liCl     PFCl2   SiI2
 AlOCl     BrO3            C4H8_1_butene   GaF2    liF      PFCl4   SiN
 AlOCl2    Br2             C4H8_cis2_buten GaF3    liH      PF2     SiO
 AlOF      BrBrO           C4H8_isobutene  GaH     liI      PF2-    SiO2
 AlOF2     BrOBr           C4H8_cyclo      GaI     liN      PF2Cl   SiS
 AlOF2-    C               C4H9_n_butyl    GaI2    liNO2    PF2Cl3  SiS2
 AlOH      C+              C4H9_i_butyl    GaI3    liNO3    PF3     Si2
 AlOHCl    C-              C4H9_s_butyl    GaO     liO      PF3Cl2  Si2C
 AlOHCl2   CBr             C4H9_t_butyl    GaOH    liOF     PF4Cl   Si2F6
 AlOHF     CBr2            C4H10_n_butane  Ga2Br2  liOH     PF5     Si2N
 AlOHF2    CBr3            C4H10_isobutane Ga2Br4  liON     PH      Si3
 AlO2      CBr4            C4N2            Ga2Br6  li2      PH2     Sn
 AlO2-     CCl             C5              Ga2Cl2  li2+     PH2-    Sn+
 Al_OH_2   CCl2            C5H6_1_3cyclo   Ga2Cl4  li2Br2   PH3     Sn-
 Al_OH_2Cl CCl2Br2         C5H8_cyclo      Ga2Cl6  li2F2    PN      SnBr
 Al_OH_2F  CCl3            C5H10_1_pentene Ga2F2   li2I2    PO      SnBr2
 Al_OH_3   CCl3Br          C5H10_cyclo     Ga2F4   li2O     PO-     SnBr3
 AlS       CCl4            C5H11_pentyl    Ga2F6   li2O+    POCl3   SnBr4
 AlS2      CF              C5H11_t_pentyl  Ga2I2   li2O2    POFCl2  SnCl
 Al2       CF+             C5H12_n_pentane Ga2I4   li2O2H2  POF2Cl  SnCl2
 Al2Br6    CFBr3           C5H12_i_pentane Ga2I6   li2SO4   POF3    SnCl3
 Al2C2     CFCl            CH3C_CH3_2CH3   Ga2O    li3+     PO2     SnCl4
 Al2Cl6    CFClBr2         C6D5_phenyl     Ge      li3Br3   PO2-    SnF
 Al2F6     CFCl2           C6D6            Ge+     li3Cl3   PS      SnF2
 Al2I6     CFCl2Br         C6H2            Ge-     li3F3    P2      SnF3
 Al2O      CFCl3           C6H5_phenyl     GeBr    li3I3    P2O3    SnF4
 Al2O+     CF2             C6H5O_phenoxy   GeBr2   Mg       P2O4    SnI
 Al2O2     CF2+            C6H6            GeBr3   Mg+      P2O5    SnI2
 Al2O2+    CF2Br2          C6H5OH_phenol   GeBr4   MgBr     P3      SnI3
 Al2O3     CF2Cl           C6H10_cyclo     GeCl    MgBr2    P3O6    SnI4
 Al2S      CF2ClBr         C6H12_1_hexene  GeCl2   MgCl     P4      SnO
 Al2S2     CF2Cl2          C6H12_cyclo     GeCl3   MgCl+    P4O6    SnO2
 Ar        CF3             C6H13_n_hexyl   GeCl4   MgCl2    P4O7    SnS
 Ar+       CF3+            C6H14_n_hexane  GeF     MgF      P4O8    SnS2
 B         CF3Br           C7H7_benzyl     GeF2    MgF+     P4O9    Sn2
 B+        CF3Cl           C7H8            GeF3    MgF2     P4O10   Sr
 B-        CF4             C7H8O_cresol_mx GeF4    MgF2+    Pb      Sr+
 BBr       CH+             C7H14_1_heptene GeH4    MgH      Pb+     SrBr
 BBr2      CHBr3           C7H15_n_heptyl  GeI     MgI      Pb-     SrBr2
 BBr3      CHCl            C7H16_n_heptane GeO     MgI2     PbBr    SrCl
 BC        CHClBr2         C7H16_2_methylh GeO2    MgN      PbBr2   SrCl+
 BC2       CHCl2           C8H8_styrene    GeS     MgO      PbBr3   SrCl2
 BCl       CHCl2Br         C8H10_ethylbenz GeS2    MgOH     PbBr4   SrF
 BCl+      CHCl3           C8H16_1_octene  Ge2     MgOH+    PbCl    SrF+
 BClOH     CHF             C8H17_n_octyl   H       Mg_OH_2  PbCl2   SrF2
 BCl_OH_2  CHFBr2          C8H18_n_octane  H+      MgS      PbCl3   SrH
 BCl2      CHFCl           C8H18_isooctane H-      Mg2      PbCl4   SrI
 BCl2+     CHFClBr         C9H19_n_nonyl   HAlO    Mg2F4    PbF     SrI2
 BCl2OH    CHFCl2          C10H8_naphthale HAlO2   Mn       PbF2    SrO
 BF        CHF2            C10H21_n_decyl  HBO     Mn+      PbF3    SrOH
 BFCl      CHF2Br          C12H9_o_bipheny HBO+    Mo       PbF4    SrOH+
 BFCl2     CHF2Cl          C12H10_biphenyl HBO2    Mo+      PbI     Sr_OH_2
 BFOH      CHF3            Ca              HBS     Mo-      PbI2    SrS
 BF_OH_2   CHI3            Ca+             HBS+    MoO      PbI3    Sr2
 BF2       CH2             CaBr            HCN     MoO2     PbI4    Ta
 BF2+      CH2Br2          CaBr2           HCO     MoO3     PbO     Ta+
 BF2-      CH2Cl           CaCl            HCO+    MoO3-    PbO2    Ta-
 BF2Cl     CH2ClBr         CaCl+           HCCN    Mo2O6    PbS     TaCl5
 BF2OH     CH2Cl2          CaCl2           HCCO    Mo3O9    PbS2    TaO
 BF3       CH2F            CaF             HCl     Mo4O12   Rb      TaO2
 BF4-      CH2FBr          CaF+            HD      Mo5O15   Rb+     Ti
 BH        CH2FCl          CaF2            HD+     N        Rb-     Ti+
 BHCl      CH2F2           CaH             HDO     N+       RbBO2   Ti-
 BHCl2     CH2I2           CaI             HDO2    N-       RbBr    TiCl
 BHF       CH3             CaI2            HF      NCO      RbCl    TiCl2
 BHFCl     CH3Br           CaO             HI      ND       RbF     TiCl3
 BHF2      CH3Cl           CaO+            HNC     ND2      RbH     TiCl4
 BH2       CH3F            CaOH            HNCO    ND3      RbI     TiO
 BH2Cl     CH3I            CaOH+           HNO     NF       RbK     TiO+
 BH2F      CH2OH           Ca_OH_2         HNO2    NF2      Rbli    TiOCl
 BH3       CH2OH+          CaS             HNO3    NF3      RbNO2   TiOCl2
 BH3NH3    CH3O            Ca2             HOCl    NH       RbNO3   TiO2
 BH4       CH4             Cd              HOF     NH+      RbNa    U
 BI        CH3OH           Cd+             HO2     NHF      RbO     UF
 BI2       CH3OOH          Cl              HO2-    NHF2     RbOH    UF+
 BI3       CI              Cl+             HPO     NH2      Rb2Br2  UF-
 BN        CI2             Cl-             HSO3F   NH2F     Rb2Cl2  UF2
 BO        CI3             ClCN            H2      NH3      Rb2F2   UF2+
 BO-       CI4             ClF             H2+     NH2OH    Rb2I2   UF2-
 BOCl      CN              ClF3            H2-     NH4+     Rb2O    UF3
 BOCl2     CN+             ClF5            HBOH    NO       Rb2O2   UF3+
 BOF       CN-             ClO             HCOOH   NOCl     Rb2O2H2 UF3-
 BOF2      CNN             ClO2            H2F2    NOF      Rb2SO4  UF4
 BOH       CO              Cl2             H2O     NOF3     Rn      UF4+
 BO2       CO+             Cl2O            H2O+    NO2      Rn+     UF4-
 BO2-      COCl            Co              H2O2    NO2-     S       UF5
 B_OH_2    COCl2           Co+             H2S     NO2Cl    S+      UF5+
 BS        COFCl           Co-             H2SO4   NO2F     S-      UF5-
 BS2       COF2            Cr              H2BOH   NO3      SCl     UF6
 B2        COHCl           Cr+             HB_OH_2 NO3-     SCl2    UF6-
 B2C       COHF            Cr-             H3BO3   NO3F     SCl2+   UO
 B2Cl4     COS             CrN             H3B3O3  N2       SD      UO+
 B2F4      CO2             CrO             H3B3O6  N2+      SF      UOF
 B2H       CO2+            CrO2            H3F3    N2-      SF+     UOF2
 B2H2      COOH            CrO3            H3O+    NCN      SF-     UOF3
 B2H3      CP              CrO3-           H4F4    N2D2_cis SF2     UOF4
 B2H3_db   CS              Cs              H5F5    N2F2     SF2+    UO2
 B2H4      CS2             Cs+             H6F6    N2F4     SF2-    UO2+
 B2H4_db   C2              Cs-             H7F7    N2H2     SF3     UO2-
 B2H5      C2+             CsBO2           He      NH2NO2   SF3+    UO2F
 B2H5_db   C2-             CsBr            He+     N2H4     SF3-    UO2F2
 B2H6      C2Cl            CsCl            Hg      N2O      SF4     UO3
 B2O       C2Cl2           CsF             Hg+     N2O+     SF4+    UO3-
 B2O2      C2Cl3           CsH             HgBr2   N2O3     SF4-    V
 B2O3      C2Cl4           CsI             I       N2O4     SF5     V+
 B2_OH_4   C2Cl6           Csli            I+      N2O5     SF5+    V-
 B2S       C2F             CsNO2           I-      N3       SF5-    VCl4
 B2S2      C2FCl           CsNO3           IF5     N3H      SF6     VN
 B2S3      C2FCl3          CsNa            IF7     Na       SF6-    VO
 B3H7_C2v  C2F2            CsO             I2      Na+      SH      VO2
 B3H7_Cs   C2F2Cl2         CsOH            In      Na-      SH-     V4O10
 B3H9      C2F3            CsRb            In+     NaAlF4   SN      W
 B3N3H6    C2F3Cl          Cs2             InBr    NaBO2    SO      W+
 B3O3Cl3   C2F4            Cs2Br2          InBr2   NaBr     SO-     W-
 B3O3FCl2  C2F6            Cs2CO3          InBr3   NaCN     SOF2    WCl6
 B3O3F2Cl  C2H             Cs2Cl2          InCl    NaCl     SO2     WO
 B3O3F3    C2HCl           Cs2F2           InCl2   NaF      SO2-    WOCl4
 B4H4      C2HCl3          Cs2I2           InCl3   NaH      SO2Cl2  WO2
 B4H10     C2HF            Cs2O            InF     NaI      SO2FCl  WO2Cl2
 B4H12     C2HFCl2         Cs2O+           InF2    Nali     SO2F2   WO3
 B5H9      C2HF2Cl         Cs2O2           InF3    NaNO2    SO3     WO3-
 Ba        C2HF3           Cs2O2H2         InH     NaNO3    S2      Xe
 Ba+       C2H2_vinylidene Cs2SO4          InI     NaO      S2-     Xe+
 BaBr      C2H2Cl2         Cu              InI2    NaOH     S2Cl2   Zn
 BaBr2     C2H2FCl         Cu+             InI3    NaOH+    S2F2    Zn+
 BaCl      C2H2F2          Cu-             InO     Na2      S2O     Zr
 BaCl+     CH2CO_ketene    CuCl            InOH    Na2Br2   S3      Zr+
 BaCl2     O_CH_2O         CuF             In2Br2  Na2Cl2   S4      Zr-
 BaF       HO_CO_2OH       CuF2            In2Br4  Na2F2    S5      ZrN
 BaF+      C2H3_vinyl      CuO             In2Br6  Na2I2    S6      ZrO
 BaF2      CH2Br-COOH      Cu2             In2Cl2  Na2O     S7      ZrO+
 BaH       C2H3Cl          Cu3Cl3          In2Cl4  Na2O+    S8      ZrO2
 BaI       CH2Cl-COOH      D               In2Cl6  Na2O2    Sc
 BaI2      C2H3F           D+              In2F2   Na2O2H2  Sc+
 BaO       CH3CN           D-              In2F4   Na2SO4   Sc-
 BaO+      CH3CO_acetyl    DBr             In2F6   Na3Cl3   ScO
 BaOH      C2H4            DCl             In2I2   Na3F3    ScO+
</pre></HTML>"));
    end IdealGases;

    package Incompressible "Medium model for T-dependent properties, defined by tables or polynomials"
      extends Modelica.Icons.VariantsPackage;
      import Modelica.Constants;
      import Modelica.Math;

      package Examples "Examples for incompressible media"
        extends Modelica.Icons.VariantsPackage;

        package Glycol47 "1,2-Propylene glycol, 47% mixture with water"
          extends TableBased(mediumName = "Glycol-Water 47%", T_min = Cv.from_degC(-30), T_max = Cv.from_degC(100), TinK = false, T0 = 273.15, tableDensity = [-30, 1066; -20, 1062; -10, 1058; 0, 1054; 20, 1044; 40, 1030; 60, 1015; 80, 999; 100, 984], tableHeatCapacity = [-30, 3450; -20, 3490; -10, 3520; 0, 3560; 20, 3620; 40, 3690; 60, 3760; 80, 3820; 100, 3890], tableConductivity = [-30, 0.397; -20, 0.396; -10, 0.395; 0, 0.395; 20, 0.394; 40, 0.393; 60, 0.392; 80, 0.391; 100, 0.390], tableViscosity = [-30, 0.160; -20, 0.0743; -10, 0.0317; 0, 0.0190; 20, 0.00626; 40, 0.00299; 60, 0.00162; 80, 0.00110; 100, 0.00081], tableVaporPressure = [0, 500; 20, 1.9e3; 40, 5.3e3; 60, 16e3; 80, 37e3; 100, 80e3]);
          annotation(
            Documentation(info = "<html>

</html>"));
        end Glycol47;

        package Essotherm650 "Essotherm thermal oil"
          extends TableBased(mediumName = "Essotherm 650", T_min = Cv.from_degC(0), T_max = Cv.from_degC(320), TinK = false, T0 = 273.15, tableDensity = [0, 909; 20, 897; 40, 884; 60, 871; 80, 859; 100, 846; 150, 813; 200, 781; 250, 748; 300, 715; 320, 702], tableHeatCapacity = [0, 1770; 20, 1850; 40, 1920; 60, 1990; 80, 2060; 100, 2130; 150, 2310; 200, 2490; 250, 2670; 300, 2850; 320, 2920], tableConductivity = [0, 0.1302; 20, 0.1288; 40, 0.1274; 60, 0.1260; 80, 0.1246; 100, 0.1232; 150, 0.1197; 200, 0.1163; 250, 0.1128; 300, 0.1093; 320, 0.1079], tableViscosity = [0, 14370; 20, 1917; 40, 424; 60, 134; 80, 54.5; 100, 26.64; 150, 7.47; 200, 3.22; 250, 1.76; 300, 1.10; 320, 0.94], tableVaporPressure = [160, 3; 180, 10; 200, 40; 220, 100; 240, 300; 260, 600; 280, 1600; 300, 3e3; 320, 5.5e3]);
          annotation(
            Documentation(info = "<html>

</html>"));
        end Essotherm650;

        model TestGlycol "Test Glycol47 Medium model"
          extends Modelica.Icons.Example;
          package Medium = Glycol47 "Medium model (Glycol47)";
          Medium.BaseProperties medium;
          Medium.DynamicViscosity eta = Medium.dynamicViscosity(medium.state);
          Medium.ThermalConductivity lambda = Medium.thermalConductivity(medium.state);
          Medium.SpecificEntropy s = Medium.specificEntropy(medium.state);
          Medium.SpecificHeatCapacity cv = Medium.specificHeatCapacityCv(medium.state);
          Medium.SpecificInternalEnergy u = Medium.specificInternalEnergy(medium.state);
          Medium.SpecificInternalEnergy h = Medium.specificEnthalpy(medium.state);
          Medium.SpecificInternalEnergy d = Medium.density(medium.state);
        protected
          constant Modelica.SIunits.Time timeUnit = 1;
          constant Modelica.SIunits.Temperature Ta = 1;
        equation
          medium.p = 1.013e5;
          medium.T = Medium.T_min + time / timeUnit * Ta;
          annotation(
            experiment(StopTime = 1.01));
        end TestGlycol;
        annotation(
          Documentation(info = "<HTML>

<p>
This package provides a few examples of how to construct medium models for
incompressible fluids. The package contains:
</p>
<ul>
<li><b>Glycol47</b>, a model of 47% glycol water mixture, based on tables of
density and heat capacity as functions of temperature.</li>
<li><b>Essotherm650</b>, a medium model for thermal oil, also based on tables.</li>
</ul>

</HTML>"));
      end Examples;

      package Common "Common data structures"
        extends Modelica.Icons.Package;
        // Extended record for input to functions based on polynomials

        record BaseProps_Tpoly "Fluid state record"
          extends Modelica.Icons.Record;
          SI.Temperature T "Temperature";
          SI.Pressure p "Pressure";
          //    SI.Density d "Density";
        end BaseProps_Tpoly;

        //     record BaseProps_Tpoly_old "Fluid state record"
        //       extends Modelica.Media.Interfaces.PartialMedium.ThermodynamicState;
        //       //      SI.SpecificHeatCapacity cp "Specific heat capacity";
        //       SI.Temperature T "Temperature";
        //       SI.Pressure p "Pressure";
        //       //    SI.Density d "Density";
        //       parameter Real[:] poly_rho "Polynomial coefficients";
        //       parameter Real[:] poly_Cp "Polynomial coefficients";
        //       parameter Real[:] poly_eta "Polynomial coefficients";
        //       parameter Real[:] poly_pVap "Polynomial coefficients";
        //       parameter Real[:] poly_lam "Polynomial coefficients";
        //       parameter Real[:] invTK "Inverse T [1/K]";
        //     end BaseProps_Tpoly_old;
      end Common;

      package TableBased "Incompressible medium properties based on tables"
        import Poly = Modelica.Media.Incompressible.TableBased.Polynomials_Temp;
        extends Modelica.Media.Interfaces.PartialMedium(ThermoStates = if enthalpyOfT then Modelica.Media.Interfaces.Choices.IndependentVariables.T else Modelica.Media.Interfaces.Choices.IndependentVariables.pT, final reducedX = true, final fixedX = true, mediumName = "tableMedium", redeclare record ThermodynamicState = Common.BaseProps_Tpoly, singleState = true, reference_p = 1.013e5, Temperature(min = T_min, max = T_max));
        // Constants to be set in actual Medium
        constant Boolean enthalpyOfT = true "True if enthalpy is approximated as a function of T only, (p-dependence neglected)";
        constant Boolean densityOfT = size(tableDensity, 1) > 1 "True if density is a function of temperature";
        constant Modelica.SIunits.Temperature T_min "Minimum temperature valid for medium model";
        constant Modelica.SIunits.Temperature T_max "Maximum temperature valid for medium model";
        constant Temperature T0 = 273.15 "Reference Temperature";
        constant SpecificEnthalpy h0 = 0 "Reference enthalpy at T0, reference_p";
        constant SpecificEntropy s0 = 0 "Reference entropy at T0, reference_p";
        constant MolarMass MM_const = 0.1 "Molar mass";
        constant Integer npol = 2 "Degree of polynomial used for fitting";
        constant Integer npolDensity = npol "Degree of polynomial used for fitting rho(T)";
        constant Integer npolHeatCapacity = npol "Degree of polynomial used for fitting Cp(T)";
        constant Integer npolViscosity = npol "Degree of polynomial used for fitting eta(T)";
        constant Integer npolVaporPressure = npol "Degree of polynomial used for fitting pVap(T)";
        constant Integer npolConductivity = npol "Degree of polynomial used for fitting lambda(T)";
        constant Integer neta = size(tableViscosity, 1) "Number of data points for viscosity";
        constant Real[:, 2] tableDensity "Table for rho(T)";
        constant Real[:, 2] tableHeatCapacity "Table for Cp(T)";
        constant Real[:, 2] tableViscosity "Table for eta(T)";
        constant Real[:, 2] tableVaporPressure "Table for pVap(T)";
        constant Real[:, 2] tableConductivity "Table for lambda(T)";
        //    constant Real[:] TK=tableViscosity[:,1]+T0*ones(neta) "Temperature for Viscosity";
        constant Boolean TinK "True if T[K],Kelvin used for table temperatures";
        constant Boolean hasDensity = not size(tableDensity, 1) == 0 "True if table tableDensity is present";
        constant Boolean hasHeatCapacity = not size(tableHeatCapacity, 1) == 0 "True if table tableHeatCapacity is present";
        constant Boolean hasViscosity = not size(tableViscosity, 1) == 0 "True if table tableViscosity is present";
        constant Boolean hasVaporPressure = not size(tableVaporPressure, 1) == 0 "True if table tableVaporPressure is present";
        final constant Real invTK[neta] = if size(tableViscosity, 1) > 0 then if TinK then 1 ./ tableViscosity[:, 1] else 1 ./ Cv.from_degC(tableViscosity[:, 1]) else fill(0, neta);
        final constant Real poly_rho[:] = if hasDensity then Poly.fitting(tableDensity[:, 1], tableDensity[:, 2], npolDensity) else zeros(npolDensity + 1);
        final constant Real poly_Cp[:] = if hasHeatCapacity then Poly.fitting(tableHeatCapacity[:, 1], tableHeatCapacity[:, 2], npolHeatCapacity) else zeros(npolHeatCapacity + 1);
        final constant Real poly_eta[:] = if hasViscosity then Poly.fitting(invTK, Math.log(tableViscosity[:, 2]), npolViscosity) else zeros(npolViscosity + 1);
        final constant Real poly_pVap[:] = if hasVaporPressure then Poly.fitting(tableVaporPressure[:, 1], tableVaporPressure[:, 2], npolVaporPressure) else zeros(npolVaporPressure + 1);
        final constant Real poly_lam[:] = if size(tableConductivity, 1) > 0 then Poly.fitting(tableConductivity[:, 1], tableConductivity[:, 2], npolConductivity) else zeros(npolConductivity + 1);

        function invertTemp "Function to invert temperatures"
          extends Modelica.Icons.Function;
          input Real[:] table "Table temperature data";
          input Boolean Tink "Flag for Celsius or Kelvin";
          output Real invTable[size(table, 1)] "Inverted temperatures";
        algorithm
          for i in 1:size(table, 1) loop
            invTable[i] := if TinK then 1 / table[i] else 1 / Cv.from_degC(table[i]);
          end for;
          annotation(
            smoothOrder = 3);
        end invertTemp;

        redeclare model extends BaseProperties(final standardOrderComponents = true, p_bar = Cv.to_bar(p), T_degC(start = T_start - 273.15) = Cv.to_degC(T), T(start = T_start, stateSelect = if preferredMediumStates then StateSelect.prefer else StateSelect.default)) "Base properties of T dependent medium"
            //  redeclare parameter SpecificHeatCapacity R=Modelica.Constants.R,
            SI.SpecificHeatCapacity cp "Specific heat capacity";
            parameter SI.Temperature T_start = 298.15 "Initial temperature";

          equation
            assert(hasDensity, "Medium " + mediumName + " can not be used without assigning tableDensity.");
            assert(T >= T_min and T <= T_max, "Temperature T (= " + String(T) + " K) is not in the allowed range (" + String(T_min) + " K <= T <= " + String(T_max) + " K) required from medium model \"" + mediumName + "\".");
            R = Modelica.Constants.R;
            cp = Poly.evaluate(poly_Cp, if TinK then T else T_degC);
            h = if enthalpyOfT then h_T(T) else h_pT(p, T, densityOfT);
            u = h - (if singleState then reference_p / d else state.p / d);
            d = Poly.evaluate(poly_rho, if TinK then T else T_degC);
            state.T = T;
            state.p = p;
            MM = MM_const;
          annotation(
            Documentation(info = "<html>
<p>
Note that the inner energy neglects the pressure dependence, which is only
true for an incompressible medium with d = constant. The neglected term is
p-reference_p)/rho*(T/rho)*(partial rho /partial T). This is very small for
liquids due to proportionality to 1/d^2, but can be problematic for gases that are
modeled incompressible.
</p>
<p>It should be noted that incompressible media only have 1 state per control volume (usually T),
but have both T and p as inputs for fully correct properties. The error of using only T-dependent
properties is small, therefore a Boolean flag enthalpyOfT exists. If it is true, the
enumeration Choices.independentVariables  is set to  Choices.independentVariables.T otherwise
it is set to Choices.independentVariables.pT.</p>
<p>
Enthalpy is never a function of T only (h = h(T) + (p-reference_p)/d), but the
error is also small and non-linear systems can be avoided. In particular,
non-linear systems are small and local as opposed to large and over all volumes.
</p>

<p>
Entropy is calculated as
</p>
<pre>
  s = s0 + integral(Cp(T)/T,dt)
</pre>
<p>
which is only exactly true for a fluid with constant density d=d0.
</p>
</html>"));
        end BaseProperties;

        redeclare function extends setState_pTX "Returns state record, given pressure and temperature"
          algorithm
            state := ThermodynamicState(p = p, T = T);
          annotation(
            smoothOrder = 3);
        end setState_pTX;

        redeclare function extends setState_dTX "Returns state record, given pressure and temperature"
          algorithm
            assert(false, "For incompressible media with d(T) only, state can not be set from density and temperature");
        end setState_dTX;

        function setState_pT "Returns state record as function of p and T"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input Temperature T "Temperature";
          output ThermodynamicState state "Thermodynamic state";
        algorithm
          state.T := T;
          state.p := p;
          annotation(
            smoothOrder = 3);
        end setState_pT;

        redeclare function extends setState_phX "Returns state record, given pressure and specific enthalpy"
          algorithm
            state := ThermodynamicState(p = p, T = T_ph(p, h));
          annotation(
            Inline = true,
            smoothOrder = 3);
        end setState_phX;

        function setState_ph "Returns state record as function of p and h"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SpecificEnthalpy h "Specific enthalpy";
          output ThermodynamicState state "Thermodynamic state";
        algorithm
          state := ThermodynamicState(p = p, T = T_ph(p, h));
          annotation(
            Inline = true,
            smoothOrder = 3);
        end setState_ph;

        redeclare function extends setState_psX "Returns state record, given pressure and specific entropy"
          algorithm
            state := ThermodynamicState(p = p, T = T_ps(p, s));
          annotation(
            Inline = true,
            smoothOrder = 3);
        end setState_psX;

        function setState_ps "Returns state record as function of p and s"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SpecificEntropy s "Specific entropy";
          output ThermodynamicState state "Thermodynamic state";
        algorithm
          state := ThermodynamicState(p = p, T = T_ps(p, s));
          annotation(
            Inline = true,
            smoothOrder = 3);
        end setState_ps;

        redeclare function extends setSmoothState "Return thermodynamic state so that it smoothly approximates: if x > 0 then state_a else state_b"
          algorithm
            state := ThermodynamicState(p = Media.Common.smoothStep(x, state_a.p, state_b.p, x_small), T = Media.Common.smoothStep(x, state_a.T, state_b.T, x_small));
          annotation(
            Inline = true,
            smoothOrder = 3);
        end setSmoothState;

        redeclare function extends specificHeatCapacityCv "Specific heat capacity at constant volume (or pressure) of medium"
          algorithm
            assert(hasHeatCapacity, "Specific Heat Capacity, Cv, is not defined for medium " + mediumName + ".");
            cv := Poly.evaluate(poly_Cp, if TinK then state.T else state.T - 273.15);
          annotation(
            smoothOrder = 2);
        end specificHeatCapacityCv;

        redeclare function extends specificHeatCapacityCp "Specific heat capacity at constant volume (or pressure) of medium"
          algorithm
            assert(hasHeatCapacity, "Specific Heat Capacity, Cv, is not defined for medium " + mediumName + ".");
            cp := Poly.evaluate(poly_Cp, if TinK then state.T else state.T - 273.15);
          annotation(
            smoothOrder = 2);
        end specificHeatCapacityCp;

        redeclare function extends dynamicViscosity "Return dynamic viscosity as a function of the thermodynamic state record"
          algorithm
            assert(size(tableViscosity, 1) > 0, "DynamicViscosity, eta, is not defined for medium " + mediumName + ".");
            eta := Math.exp(Poly.evaluate(poly_eta, 1 / state.T));
          annotation(
            smoothOrder = 2);
        end dynamicViscosity;

        redeclare function extends thermalConductivity "Return thermal conductivity as a function of the thermodynamic state record"
          algorithm
            assert(size(tableConductivity, 1) > 0, "ThermalConductivity, lambda, is not defined for medium " + mediumName + ".");
            lambda := Poly.evaluate(poly_lam, if TinK then state.T else Cv.to_degC(state.T));
          annotation(
            smoothOrder = 2);
        end thermalConductivity;

        function s_T "Compute specific entropy"
          extends Modelica.Icons.Function;
          input Temperature T "Temperature";
          output SpecificEntropy s "Specific entropy";
        algorithm
          s := s0 + (if TinK then Poly.integralValue(poly_Cp[1:npol], T, T0) else Poly.integralValue(poly_Cp[1:npol], Cv.to_degC(T), Cv.to_degC(T0))) + Modelica.Math.log(T / T0) * Poly.evaluate(poly_Cp, if TinK then 0 else Modelica.Constants.T_zero);
          annotation(
            Inline = true,
            smoothOrder = 2);
        end s_T;

        redeclare function extends specificEntropy "Return specific entropy
         as a function of the thermodynamic state record"
          protected
            Integer npol = size(poly_Cp, 1) - 1;

          algorithm
            assert(hasHeatCapacity, "Specific Entropy, s(T), is not defined for medium " + mediumName + ".");
            s := s_T(state.T);
          annotation(
            smoothOrder = 2);
        end specificEntropy;

        function h_T "Compute specific enthalpy from temperature"
          import Modelica.SIunits.Conversions.to_degC;
          extends Modelica.Icons.Function;
          input SI.Temperature T "Temperature";
          output SI.SpecificEnthalpy h "Specific enthalpy at p, T";
        algorithm
          h := h0 + Poly.integralValue(poly_Cp, if TinK then T else Cv.to_degC(T), if TinK then T0 else Cv.to_degC(T0));
          annotation(
            derivative = h_T_der);
        end h_T;

        function h_T_der "Compute specific enthalpy from temperature"
          import Modelica.SIunits.Conversions.to_degC;
          extends Modelica.Icons.Function;
          input SI.Temperature T "Temperature";
          input Real dT "Temperature derivative";
          output Real dh "Derivative of Specific enthalpy at T";
        algorithm
          dh := Poly.evaluate(poly_Cp, if TinK then T else Cv.to_degC(T)) * dT;
          annotation(
            smoothOrder = 1);
        end h_T_der;

        function h_pT "Compute specific enthalpy from pressure and temperature"
          import Modelica.SIunits.Conversions.to_degC;
          extends Modelica.Icons.Function;
          input SI.Pressure p "Pressure";
          input SI.Temperature T "Temperature";
          input Boolean densityOfT = false "Include or neglect density derivative dependence of enthalpy" annotation(
            Evaluate);
          output SI.SpecificEnthalpy h "Specific enthalpy at p, T";
        algorithm
          h := h0 + Poly.integralValue(poly_Cp, if TinK then T else Cv.to_degC(T), if TinK then T0 else Cv.to_degC(T0)) + (p - reference_p) / Poly.evaluate(poly_rho, if TinK then T else Cv.to_degC(T)) * (if densityOfT then 1 + T / Poly.evaluate(poly_rho, if TinK then T else Cv.to_degC(T)) * Poly.derivativeValue(poly_rho, if TinK then T else Cv.to_degC(T)) else 1.0);
          annotation(
            smoothOrder = 2);
        end h_pT;

        function density_T "Return density as function of temperature"
          extends Modelica.Icons.Function;
          input Temperature T "Temperature";
          output Density d "Density";
        algorithm
          d := Poly.evaluate(poly_rho, if TinK then T else Cv.to_degC(T));
          annotation(
            Inline = true,
            smoothOrder = 2);
        end density_T;

        redeclare function extends temperature "Return temperature as a function of the thermodynamic state record"
          algorithm
            T := state.T;
          annotation(
            Inline = true,
            smoothOrder = 2);
        end temperature;

        redeclare function extends pressure "Return pressure as a function of the thermodynamic state record"
          algorithm
            p := state.p;
          annotation(
            Inline = true,
            smoothOrder = 2);
        end pressure;

        redeclare function extends density "Return density as a function of the thermodynamic state record"
          algorithm
            d := Poly.evaluate(poly_rho, if TinK then state.T else Cv.to_degC(state.T));
          annotation(
            Inline = true,
            smoothOrder = 2);
        end density;

        redeclare function extends specificEnthalpy "Return specific enthalpy as a function of the thermodynamic state record"
          algorithm
            h := if enthalpyOfT then h_T(state.T) else h_pT(state.p, state.T);
          annotation(
            Inline = true,
            smoothOrder = 2);
        end specificEnthalpy;

        redeclare function extends specificInternalEnergy "Return specific internal energy as a function of the thermodynamic state record"
          algorithm
            u := (if enthalpyOfT then h_T(state.T) else h_pT(state.p, state.T)) - (if singleState then reference_p else state.p) / density(state);
          annotation(
            Inline = true,
            smoothOrder = 2);
        end specificInternalEnergy;

        function T_ph "Compute temperature from pressure and specific enthalpy"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SpecificEnthalpy h "Specific enthalpy";
          output Temperature T "Temperature";
        protected
          package Internal "Solve h(T) for T with given h (use only indirectly via temperature_phX)"
            extends Modelica.Media.Common.OneNonLinearEquation;

            redeclare record extends f_nonlinear_Data "Superfluous record, fix later when better structure of inverse functions exists"
                constant Real[5] dummy = {1, 2, 3, 4, 5};
            end f_nonlinear_Data;

            redeclare function extends f_nonlinear "P is smuggled in via vector"
              algorithm
                y := if singleState then h_T(x) else h_pT(p, x);
            end f_nonlinear;
          end Internal;
        algorithm
          T := Internal.solve(h, T_min, T_max, p, {1}, Internal.f_nonlinear_Data());
          annotation(
            Inline = false,
            LateInline = true,
            inverse(h = h_pT(p, T)));
        end T_ph;

        function T_ps "Compute temperature from pressure and specific enthalpy"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SpecificEntropy s "Specific entropy";
          output Temperature T "Temperature";
        protected
          package Internal "Solve h(T) for T with given h (use only indirectly via temperature_phX)"
            extends Modelica.Media.Common.OneNonLinearEquation;

            redeclare record extends f_nonlinear_Data "Superfluous record, fix later when better structure of inverse functions exists"
                constant Real[5] dummy = {1, 2, 3, 4, 5};
            end f_nonlinear_Data;

            redeclare function extends f_nonlinear "P is smuggled in via vector"
              algorithm
                y := s_T(x);
            end f_nonlinear;
          end Internal;
        algorithm
          T := Internal.solve(s, T_min, T_max, p, {1}, Internal.f_nonlinear_Data());
        end T_ps;

        package Polynomials_Temp "Temporary Functions operating on polynomials (including polynomial fitting); only to be used in Modelica.Media.Incompressible.TableBased"
          extends Modelica.Icons.Package;

          function evaluate "Evaluate polynomial at a given abscissa value"
            extends Modelica.Icons.Function;
            input Real p[:] "Polynomial coefficients (p[1] is coefficient of highest power)";
            input Real u "Abscissa value";
            output Real y "Value of polynomial at u";
          algorithm
            y := p[1];
            for j in 2:size(p, 1) loop
              y := p[j] + u * y;
            end for;
            annotation(
              derivative(zeroDerivative = p) = evaluate_der);
          end evaluate;

          function evaluateWithRange "Evaluate polynomial at a given abscissa value with linear extrapolation outside of the defined range"
            extends Modelica.Icons.Function;
            input Real p[:] "Polynomial coefficients (p[1] is coefficient of highest power)";
            input Real uMin "Polynomial valid in the range uMin .. uMax";
            input Real uMax "Polynomial valid in the range uMin .. uMax";
            input Real u "Abscissa value";
            output Real y "Value of polynomial at u. Outside of uMin,uMax, linear extrapolation is used";
          algorithm
            if u < uMin then
              y := evaluate(p, uMin) - evaluate_der(p, uMin, uMin - u);
            elseif u > uMax then
              y := evaluate(p, uMax) + evaluate_der(p, uMax, u - uMax);
            else
              y := evaluate(p, u);
            end if;
            annotation(
              derivative(zeroDerivative = p, zeroDerivative = uMin, zeroDerivative = uMax) = evaluateWithRange_der);
          end evaluateWithRange;

          function derivative "Derivative of polynomial"
            extends Modelica.Icons.Function;
            input Real p1[:] "Polynomial coefficients (p1[1] is coefficient of highest power)";
            output Real p2[size(p1, 1) - 1] "Derivative of polynomial p1";
          protected
            Integer n = size(p1, 1);
          algorithm
            for j in 1:n - 1 loop
              p2[j] := p1[j] * (n - j);
            end for;
          end derivative;

          function derivativeValue "Value of derivative of polynomial at abscissa value u"
            extends Modelica.Icons.Function;
            input Real p[:] "Polynomial coefficients (p[1] is coefficient of highest power)";
            input Real u "Abscissa value";
            output Real y "Value of derivative of polynomial at u";
          protected
            Integer n = size(p, 1);
          algorithm
            y := p[1] * (n - 1);
            for j in 2:size(p, 1) - 1 loop
              y := p[j] * (n - j) + u * y;
            end for;
            annotation(
              derivative(zeroDerivative = p) = derivativeValue_der);
          end derivativeValue;

          function secondDerivativeValue "Value of 2nd derivative of polynomial at abscissa value u"
            extends Modelica.Icons.Function;
            input Real p[:] "Polynomial coefficients (p[1] is coefficient of highest power)";
            input Real u "Abscissa value";
            output Real y "Value of 2nd derivative of polynomial at u";
          protected
            Integer n = size(p, 1);
          algorithm
            y := p[1] * (n - 1) * (n - 2);
            for j in 2:size(p, 1) - 2 loop
              y := p[j] * (n - j) * (n - j - 1) + u * y;
            end for;
          end secondDerivativeValue;

          function integral "Indefinite integral of polynomial p(u)"
            extends Modelica.Icons.Function;
            input Real p1[:] "Polynomial coefficients (p1[1] is coefficient of highest power)";
            output Real p2[size(p1, 1) + 1] "Polynomial coefficients of indefinite integral of polynomial p1 (polynomial p2 + C is the indefinite integral of p1, where C is an arbitrary constant)";
          protected
            Integer n = size(p1, 1) + 1 "Degree of output polynomial";
          algorithm
            for j in 1:n - 1 loop
              p2[j] := p1[j] / (n - j);
            end for;
            p2[n] := 0.0;
          end integral;

          function integralValue "Integral of polynomial p(u) from u_low to u_high"
            extends Modelica.Icons.Function;
            input Real p[:] "Polynomial coefficients";
            input Real u_high "High integrand value";
            input Real u_low = 0 "Low integrand value, default 0";
            output Real integral = 0.0 "Integral of polynomial p from u_low to u_high";
          protected
            Integer n = size(p, 1) "Degree of integrated polynomial";
            Real y_low = 0 "Value at lower integrand";
          algorithm
            for j in 1:n loop
              integral := u_high * (p[j] / (n - j + 1) + integral);
              y_low := u_low * (p[j] / (n - j + 1) + y_low);
            end for;
            integral := integral - y_low;
            annotation(
              derivative(zeroDerivative = p) = integralValue_der);
          end integralValue;

          function fitting "Computes the coefficients of a polynomial that fits a set of data points in a least-squares sense"
            extends Modelica.Icons.Function;
            input Real u[:] "Abscissa data values";
            input Real y[size(u, 1)] "Ordinate data values";
            input Integer n(min = 1) "Order of desired polynomial that fits the data points (u,y)";
            output Real p[n + 1] "Polynomial coefficients of polynomial that fits the date points";
          protected
            Real V[size(u, 1), n + 1] "Vandermonde matrix";
          algorithm
// Construct Vandermonde matrix
            V[:, n + 1] := ones(size(u, 1));
            for j in n:(-1):1 loop
              V[:, j] := {u[i] * V[i, j + 1] for i in 1:size(u, 1)};
            end for;
// Solve least squares problem
            p := Modelica.Math.Matrices.leastSquares(V, y);
            annotation(
              Documentation(info = "<HTML>
<p>
Polynomials.fitting(u,y,n) computes the coefficients of a polynomial
p(u) of degree \"n\" that fits the data \"p(u[i]) - y[i]\"
in a least squares sense. The polynomial is
returned as a vector p[n+1] that has the following definition:
</p>
<pre>
  p(u) = p[1]*u^n + p[2]*u^(n-1) + ... + p[n]*u + p[n+1];
</pre>
</HTML>"));
          end fitting;

          function evaluate_der "Evaluate derivative of polynomial at a given abscissa value"
            extends Modelica.Icons.Function;
            input Real p[:] "Polynomial coefficients (p[1] is coefficient of highest power)";
            input Real u "Abscissa value";
            input Real du "Delta of abscissa value";
            output Real dy "Value of derivative of polynomial at u";
          protected
            Integer n = size(p, 1);
          algorithm
            dy := p[1] * (n - 1);
            for j in 2:size(p, 1) - 1 loop
              dy := p[j] * (n - j) + u * dy;
            end for;
            dy := dy * du;
          end evaluate_der;

          function evaluateWithRange_der "Evaluate derivative of polynomial at a given abscissa value with extrapolation outside of the defined range"
            extends Modelica.Icons.Function;
            input Real p[:] "Polynomial coefficients (p[1] is coefficient of highest power)";
            input Real uMin "Polynomial valid in the range uMin .. uMax";
            input Real uMax "Polynomial valid in the range uMin .. uMax";
            input Real u "Abscissa value";
            input Real du "Delta of abscissa value";
            output Real dy "Value of derivative of polynomial at u";
          algorithm
            if u < uMin then
              dy := evaluate_der(p, uMin, du);
            elseif u > uMax then
              dy := evaluate_der(p, uMax, du);
            else
              dy := evaluate_der(p, u, du);
            end if;
          end evaluateWithRange_der;

          function integralValue_der "Time derivative of integral of polynomial p(u) from u_low to u_high, assuming only u_high as time-dependent (Leibnitz rule)"
            extends Modelica.Icons.Function;
            input Real p[:] "Polynomial coefficients";
            input Real u_high "High integrand value";
            input Real u_low = 0 "Low integrand value, default 0";
            input Real du_high "High integrand value";
            input Real du_low = 0 "Low integrand value, default 0";
            output Real dintegral = 0.0 "Integral of polynomial p from u_low to u_high";
          algorithm
            dintegral := evaluate(p, u_high) * du_high;
          end integralValue_der;

          function derivativeValue_der "Time derivative of derivative of polynomial"
            extends Modelica.Icons.Function;
            input Real p[:] "Polynomial coefficients (p[1] is coefficient of highest power)";
            input Real u "Abscissa value";
            input Real du "Delta of abscissa value";
            output Real dy "Time-derivative of derivative of polynomial w.r.t. input variable at u";
          protected
            Integer n = size(p, 1);
          algorithm
            dy := secondDerivativeValue(p, u) * du;
          end derivativeValue_der;
          annotation(
            Documentation(info = "<HTML>
<p>
This package contains functions to operate on polynomials,
in particular to determine the derivative and the integral
of a polynomial and to use a polynomial to fit a given set
of data points.
</p>

<p><b>Copyright &copy; 2004-2015, Modelica Association and DLR.</b></p>

<p><i>
This package is <b>free</b> software. It can be redistributed and/or modified
under the terms of the <b>Modelica license</b>, see the license conditions
and the accompanying <b>disclaimer</b> in the documentation of package
Modelica in file \"Modelica/package.mo\".
</i>
</p>

</html>", revisions = "<html>
<ul>
<li><i>Oct. 22, 2004</i> by Martin Otter (DLR):<br>
       Renamed functions to not have abbreviations.<br>
       Based fitting on LAPACK<br>
       New function to return the polynomial of an indefinite integral</li>
<li><i>Sept. 3, 2004</i> by Jonas Eborn (Scynamics):<br>
       polyderval, polyintval added</li>
<li><i>March 1, 2004</i> by Martin Otter (DLR):<br>
       first version implemented</li>
</ul>
</html>"));
        end Polynomials_Temp;
        annotation(
          Documentation(info = "<HTML>
<p>
This is the base package for medium models of incompressible fluids based on
tables. The minimal data to provide for a useful medium description is tables
of density and heat capacity as functions of temperature.
</p>

<p>It should be noted that incompressible media only have 1 state per control volume (usually T),
but have both T and p as inputs for fully correct properties. The error of using only T-dependent
properties is small, therefore a Boolean flag enthalpyOfT exists. If it is true, the
enumeration Choices.independentVariables  is set to  Choices.independentVariables.T otherwise
it is set to Choices.independentVariables.pT.</p>

<h4>Using the package TableBased</h4>
<p>
To implement a new medium model, create a package that <b>extends</b> TableBased
and provides one or more of the constant tables:
</p>

<pre>
tableDensity        = [T, d];
tableHeatCapacity   = [T, Cp];
tableConductivity   = [T, lam];
tableViscosity      = [T, eta];
tableVaporPressure  = [T, pVap];
</pre>

<p>
The table data is used to fit constant polynomials of order <b>npol</b>, the
temperature data points do not need to be same for different properties. Properties
like enthalpy, inner energy and entropy are calculated consistently from integrals
and derivatives of d(T) and Cp(T). The minimal
data for a useful medium model is thus density and heat capacity. Transport
properties and vapor pressure are optional, if the data tables are empty the corresponding
function calls can not be used.
</p>
</HTML>"));
      end TableBased;
      annotation(
        Documentation(info = "<HTML>
<h4>Incompressible media package</h4>
<p>
This package provides a structure and examples of how to create simple
medium models of incompressible fluids, meaning fluids with very little
pressure influence on density. The medium properties is typically described
in terms of tables, functions or polynomial coefficients.
</p>
<h4>Definitions</h4>
<p>
The common meaning of <em>incompressible</em> is that properties like density
and enthalpy are independent of pressure. Thus properties are conveniently
described as functions of temperature, e.g., as polynomials density(T) and cp(T).
However, enthalpy can not be independent of pressure since h = u - p/d. For liquids
it is anyway
common to neglect this dependence since for constant density the neglected term
is (p - p0)/d, which in comparison with cp is very small for most liquids. For
water, the equivalent change of temperature to increasing pressure 1 bar is
0.025 Kelvin.
</p>
<p>
Two Boolean flags are used to choose how enthalpy and inner energy is calculated:
</p>
<ul>
<li><b>enthalpyOfT</b>=true, means assuming that enthalpy is only a function
of temperature, neglecting the pressure dependent term.</li>
<li><b>singleState</b>=true, means also neglect the pressure influence on inner
energy, which makes all medium properties pure functions of temperature.</li>
</ul>
<p>
The default setting for both these flags is true, which enables the simulation tool
to choose temperature as the only medium state and avoids non-linear equation
systems, see the section about
<a href=\"modelica://Modelica.Media.UsersGuide.MediumDefinition.StaticStateSelection\">Static
state selection</a> in the Modelica.Media User's Guide.
</p>

<h4>Contents</h4>
<p>
Currently, the package contains the following parts:
</p>
<ol>
<li> <a href=\"modelica://Modelica.Media.Incompressible.TableBased\">
      Table based medium models</a></li>
<li> <a href=\"modelica://Modelica.Media.Incompressible.Examples\">
      Example medium models</a></li>
</ol>

<p>
A few examples are given in the Examples package. The model
<a href=\"modelica://Modelica.Media.Incompressible.Examples.Glycol47\">
Examples.Glycol47</a> shows how the medium models can be used. For more
realistic examples of how to implement volume models with medium properties
look in the <a href=\"modelica://Modelica.Media.UsersGuide.MediumUsage\">Medium
usage section</a> of the User's Guide.
</p>

</HTML>"));
    end Incompressible;

    package R134a "R134a: Medium model for R134a"
      extends Modelica.Icons.VariantsPackage;

      package Common
        extends Modelica.Icons.Package;

        record PhaseBoundaryProperties "Thermodynamic base properties on the phase boundary"
          extends Modelica.Icons.Record;
          Modelica.SIunits.Density d "Density";
          Modelica.SIunits.SpecificEnthalpy h "Enthalpy";
          Modelica.SIunits.SpecificEnergy u "Inner energy";
          Modelica.SIunits.SpecificEntropy s "Entropy";
          Modelica.SIunits.SpecificHeatCapacity cp "Heat capacity at constant pressure";
          Modelica.SIunits.SpecificHeatCapacity cv "Heat capacity at constant volume";
          Modelica.SIunits.IsothermalCompressibility kappa "Isentropic exponent";
          Modelica.SIunits.Velocity a "Velocity of sound";
          Modelica.Media.Interfaces.Types.IsobaricExpansionCoefficient beta "Isobaric expansion coefficient";
          Modelica.SIunits.IsentropicExponent gamma "Isentropic exponent";
          Modelica.SIunits.DerPressureByTemperature pt "Derivative of pressure wrt temperature";
          Modelica.SIunits.DerPressureByDensity pd "Derivative of pressure wrt density";
        end PhaseBoundaryProperties;

        record InverseDerivatives_rhoT "Derivatives required for inversion of density and temperature functions w.r.t. pressure and enthalpy states"
          extends Modelica.Icons.Record;
          Integer phase "Number of phases";
          Modelica.SIunits.Pressure p "Pressure";
          Modelica.SIunits.Temperature T "Kelvin-temperature";
          Modelica.SIunits.Density rho "Density";
          Modelica.SIunits.SpecificEnthalpy h "Specific enthalpy";
          Modelica.SIunits.SpecificHeatCapacity cv "Specific heat capacity at constant volume";
          Real pt "Derivative of pressure wrt temperature";
          Real pd "Derivative of pressure wrt density";
          Real dpT "dp/dT derivative of saturation curve";
        end InverseDerivatives_rhoT;

        record EOSIdealCoeff "Record for coefficients of ideal term of helmholtz equation of state"
          extends Modelica.Icons.Record;
          parameter Integer nc = 5 "No. of coefficients in a";
          parameter Real[nc] a "Coefficients of ideal term of helmholtz equation of state";
        end EOSIdealCoeff;

        record EOSResidualCoeff "Record for coefficients of residual term of helmholtz equation of state"
          extends Modelica.Icons.Record;
          parameter Integer nc = 20 "No. of coefficients in c, d, t, n";
          parameter Integer ns1 "No. of zero coefficients in c";
          parameter Real[nc] c "Coefficients of residual term of helmholtz equation of state";
          parameter Real[nc] d "Coefficients of residual term of helmholtz equation of state";
          parameter Real[nc] t "Coefficients of residual term of helmholtz equation of state";
          parameter Real[nc] n "Coefficients of residual term of helmholtz equation of state";
        end EOSResidualCoeff;

        function CubicSplineDerEval "Derivative of cubic spline"
          extends Modelica.Icons.Function;
          input Real x "Input";
          input Real[4] coefs "Spline coefficients";
          output Real yder "Spline derivative";
        algorithm
          yder := coefs[3] + x * (2.0 * coefs[2] + x * 3.0 * coefs[1]);
        end CubicSplineDerEval;

        function CubicSplineEval "Cubic spline"
          extends Modelica.Icons.Function;
          input Real x "Input";
          input Real[4] coefs "Spline coefficients";
          output Real y "Output";
        algorithm
          y := coefs[4] + x * (coefs[3] + x * (coefs[2] + x * coefs[1]));
        end CubicSplineEval;

        function cv2Phase "Compute isochoric specific heat capacity inside the two-phase region"
          extends Modelica.Icons.Function;
          import SI = Modelica.SIunits;
          input PhaseBoundaryProperties liq "Properties on the boiling curve";
          input PhaseBoundaryProperties vap "Properties on the condensation curve";
          input SI.MassFraction x "Vapour mass fraction";
          input SI.Temperature T "Temperature";
          input SI.Pressure p "Pressure";
          output SI.SpecificHeatCapacity cv "Isochoric specific heat capacity";
          output Real dpT "Derivative of pressure w.r.t. temperature";
        protected
          Real dxv "Derivative of vapour mass fraction w.r.t. specific volume";
          Real dvTl "Derivative of liquid specific volume w.r.t. temperature";
          Real dvTv "Derivative of vapour specific volume w.r.t. temperature";
          Real duTl "Derivative of liquid specific inner energy w.r.t. temperature";
          Real duTv "Derivative of vapour specific inner energy w.r.t. temperature";
          Real dxt "Derivative of vapour mass fraction w.r.t. temperature";
        algorithm
          dxv := if liq.d <> vap.d then liq.d * vap.d / (liq.d - vap.d) else 0.0;
          dpT := (vap.s - liq.s) * dxv;
// wrong at critical point
          dvTl := (liq.pt - dpT) / liq.pd / liq.d / liq.d;
          dvTv := (vap.pt - dpT) / vap.pd / vap.d / vap.d;
          dxt := -dxv * (dvTl + x * (dvTv - dvTl));
          duTl := liq.cv + (T * liq.pt - p) * dvTl;
          duTv := vap.cv + (T * vap.pt - p) * dvTv;
          cv := duTl + x * (duTv - duTl) + dxt * (vap.u - liq.u);
        end cv2Phase;

        function FindInterval "Half-interval search algorithm"
          extends Modelica.Icons.Function;
          input Real x "Input";
          input Real[:] breaks "Grid points defining the intervals";
          output Integer i "Found interval number";
          output Integer error = 0 "1=did not find interval";
        protected
          Integer n = scalar(size(breaks)) - 1 "Max value";
          Integer ix = 1 "Min value";
          Integer m = n "New interval";
        algorithm
          i := 1;
          if x < breaks[1] or x >= breaks[n] then
            error := 1;
          end if;
          if x < breaks[1] or x >= breaks[2] then
            while m <> ix loop
              if x < breaks[m] then
                n := m;
              else
                ix := m;
              end if;
              m := integer(div(ix + n, 2));
            end while;
            i := ix;
          end if;
        end FindInterval;

        function helmholtzToBoundaryProps "Calulate phase boundary property record from dimensionless Helmholtz function"
          extends Modelica.Icons.Function;
          import SI = Modelica.SIunits;
          input Modelica.Media.Common.HelmholtzDerivs f "Dimensionless derivatives of Helmholtz function";
          output PhaseBoundaryProperties sat "Phase boundary property record";
        protected
          SI.Pressure p "Pressure";
        algorithm
          p := f.R * f.d * f.T * f.delta * f.fdelta;
          sat.d := f.d;
          sat.h := f.R * f.T * (f.tau * f.ftau + f.delta * f.fdelta);
          sat.s := f.R * (f.tau * f.ftau - f.f);
          sat.u := f.R * f.T * f.tau * f.ftau;
          sat.cp := f.R * ((-f.tau * f.tau * f.ftautau) + (f.delta * f.fdelta - f.delta * f.tau * f.fdeltatau) ^ 2 / (2 * f.delta * f.fdelta + f.delta * f.delta * f.fdeltadelta));
          sat.cv := f.R * (-f.tau * f.tau * f.ftautau);
          sat.pt := f.R * f.d * (f.delta * (f.fdelta - f.tau * f.fdeltatau));
          sat.pd := f.R * f.T * (f.delta * (2.0 * f.fdelta + f.delta * f.fdeltadelta));
          sat.a := abs(f.R * f.T * (2 * f.delta * f.fdelta + f.delta * f.delta * f.fdeltadelta - (f.delta * f.fdelta - f.delta * f.tau * f.fdeltatau) * (f.delta * f.fdelta - f.delta * f.tau * f.fdeltatau) / (f.tau * f.tau * f.ftautau))) ^ 0.5;
          sat.kappa := 1 / (f.d * f.R * f.T * f.delta * (2.0 * f.fdelta + f.delta * f.fdeltadelta));
          sat.beta := f.R * f.d * f.delta * (f.fdelta - f.tau * f.fdeltatau) * sat.kappa;
          sat.gamma := sat.a ^ 2 / f.R / f.T;
        end helmholtzToBoundaryProps;
      end Common;

      package R134a_ph "Medium model for R134a and p,h as states"
        extends Modelica.Media.Interfaces.PartialTwoPhaseMedium(ThermoStates = Modelica.Media.Interfaces.Choices.IndependentVariables.ph, mediumName = "R134a_ph", substanceNames = {"tetrafluoroethan"}, singleState = false, SpecificEnthalpy(start = h_default, nominal = 5.0e5), Density(start = 4, nominal = 500), AbsolutePressure(start = p_default, nominal = 10e5), Temperature(start = T_default, nominal = 350), smoothModel = false, onePhase = false, fluidConstants = r134aConstants, h_default = 420e3);
        constant Boolean ph_explicit = true;
        constant Boolean dT_explicit = false;
        constant Modelica.Media.Interfaces.Types.FluidLimits[1] r134aLimits(each TMIN = 169.85, each TMAX = 455, each DMIN = 0, each DMAX = 1591.7, each PMIN = 389.563789, each PMAX = 7e7, each HMIN = 0, each HMAX = 0, each SMIN = 0, each SMAX = 0);
        constant Modelica.Media.Interfaces.Types.TwoPhase.FluidConstants[1] r134aConstants(each chemicalFormula = "CF3CH2F", each structureFormula = "1,1,1,2-tetrafluoroethane", each iupacName = "tetrafluoroethan", each casRegistryNumber = "811-97-2", each molarMass = R134aData.data.MM, each criticalMolarVolume = R134aData.data.MM / R134aData.data.DCRIT, each criticalTemperature = R134aData.data.TCRIT, each criticalPressure = R134aData.data.PCRIT, each triplePointTemperature = R134aData.data.TTRIPLE, each triplePointPressure = R134aData.data.PTRIPLE, each meltingPoint = 172.15, each normalBoilingPoint = 247.076, each acentricFactor = 0.32684, each dipoleMoment = 1.99, each hasCriticalData = true);

        redeclare record extends SaturationProperties
        end SaturationProperties;

        redeclare record extends ThermodynamicState "Thermodynamic state"
            SpecificEnthalpy h "Specific enthalpy";
            Density d "Density";
            Temperature T "Temperature";
            AbsolutePressure p "Pressure";
        end ThermodynamicState;

        redeclare replaceable model extends BaseProperties(h(stateSelect = StateSelect.prefer), d(stateSelect = StateSelect.default), T(stateSelect = StateSelect.default), p(stateSelect = StateSelect.prefer), sat(Tsat(start = 273.0), psat(start = 3.0e5))) "Base properties of R134a"
            Integer phase(min = 0, max = 2, start = 1, fixed = false) "2 for two-phase, 1 for one-phase, 0 if not known";
            MassFraction quality "Quality of vapour";

          equation
            MM = R134aData.data.MM;
            phase = 0;
            T = temperature_ph(p, h);
            d = density_ph(p, h);
            sat.Tsat = saturationTemperature(p = p);
            sat.psat = p;
            quality = vapourQuality(state);
            u = h - p / d;
            R = R134aData.data.R;
            h = state.h;
            p = state.p;
            T = state.T;
            d = state.d;
            phase = state.phase;
        end BaseProperties;

        redeclare function extends setState_phX "Set state for pressure and specific enthalpy (X not used since single substance)"
          protected
            SaturationProperties sat(psat = p, Tsat = 0) "Saturation temperature and pressure";
            Modelica.SIunits.SpecificEnthalpy hl = bubbleEnthalpy(sat) "Liquid enthalpy";
            Modelica.SIunits.SpecificEnthalpy hv = dewEnthalpy(sat) "Vapor enthalpy";

          algorithm
            state.p := p;
            state.h := h;
            state.phase := if state.h < hl or state.h > hv or state.p > R134aData.data.FPCRIT then 1 else 2;
            state.d := density_ph(p, h);
            state.T := temperature_ph(p, h);
          annotation(
            Documentation(info = "<html>
<p>This function should be used by default in order to calculate the thermodynamic state record used as input by many functions.</p>
<p>
Example:
</p>
<pre>
     parameter Medium.AbsolutePressure p = 3e5;
     parameter Medium.SpecificEnthalpy h = 4.2e5;

     Medium.Density rho;

     <b>equation</b>

     rho = Medium.density(setState_phX(p, h, fill(0, Medium.nX)));
</pre>


</html>"));
        end setState_phX;

        redeclare function extends setState_dTX "Set state for density and temperature (X not used since single substance)"
          protected
            Modelica.Media.Common.HelmholtzDerivs f "Helmholtz derivatives";
            Modelica.SIunits.SpecificHeatCapacity R "Specific gas constant";
            SaturationProperties sat "Saturation temperature and pressure";
            Modelica.SIunits.Density dl "Liquid density";
            Modelica.SIunits.Density dv "Vapor density";

          algorithm
            R := R134aData.data.R;
            f := f_R134a(d = d, T = T);
            state.p := d * R * T * f.delta * f.fdelta;
            state.h := R * T * (f.tau * f.ftau + f.delta * f.fdelta);
            sat.psat := state.p;
            state.T := T;
            state.d := d;
            sat.Tsat := state.T;
            dl := bubbleDensity(sat);
            dv := dewDensity(sat);
            state.phase := if state.d < dl or state.d > dv or state.p > R134aData.data.FPCRIT then 1 else 2;
          annotation(
            Documentation(revisions = "<html>
<p>2012-08-01  Stefan Wischhusen: Corrected passing-error of inputs.</p>
</html>", info = "<html>
<p>Although the medium package is explicit for pressure and specific enthalpy, this function may be used in order to calculate the thermodynamic state record used as input by many functions. It will calculate the missing states:</p>
<ul>
<li>pressure</li>
<li>specific enthalpy</li>
</ul>
<p>
Example:
</p>
<pre>
     parameter Medium.Density d = 4;
     parameter Medium.Temperature T = 298;

     Medium.SpecficEntropy s;

     <b>equation</b>

     s = Medium.specificEntropy(setState_dTX(d, T, fill(0, Medium.nX)));
</pre>

</html>"));
        end setState_dTX;

        redeclare function extends setState_psX "Set state for pressure and specific entropy (X not used since single substance)"
          protected
            Modelica.SIunits.Pressure delp = 1e-2 "Iteration accuracy for pressure";
            Modelica.SIunits.SpecificEntropy dels = 1e-1 "Iteration accuracy for entropy";
            Integer error "If newton iteration fails (too many calls)";
            Modelica.Media.Common.HelmholtzDerivs f "Helmholtz derivatives";
            SaturationProperties sat "Saturation temperature and pressure";

          algorithm
            state.p := p;
            if getPhase_ps(p, s) == 1 then
              (state.d, state.T, error) := dtofpsOnePhase(p = p, s = s, delp = delp, dels = dels);
              f := f_R134a(d = state.d, T = state.T);
              state.h := R134aData.R * state.T * (f.tau * f.ftau + f.delta * f.fdelta);
            else
              state.h := hofpsTwoPhase(p, s);
            end if;
          annotation(
            Documentation(info = "<html>
<p>This function may be used in order to calculate the thermodynamic state record used as input by many functions. It will calculate the missing states:</p>
<ul>
<li>density</li>
<li>pressure</li>
<li>specific enthalpy</li>
</ul>
<p>
Example:
</p>
<pre>
     parameter Medium.AbsolutePressure p = 3e5;
     parameter Medium.SpecficEntropy s = 1.7e3;

     Medium.SpecficEnthalpy h;

     <b>equation</b>

     h = Medium.specificEnthalpy(setState_psX(p, s, fill(0, Medium.nX)));
</pre>
</html>"));
        end setState_psX;

        redeclare function extends setState_pTX "Dummy function"
          algorithm
            assert(false, "A calculation of two-phase properties with input of pressure and temperature is not possible.\n" + "Please use setState_dTX or setState_phX instead.");
          annotation(
            Documentation(info = "<html>
<p>This function is intentionally provided although it will result in an error. Please note, that pressure and temperature cannot be used to determine states in two-phase region. Therefore, in order to avoid wrong usage, the function will give an error message.</p>
</html>"));
        end setState_pTX;

        redeclare function extends setBubbleState "Return the thermodynamic state on the bubble line"
          algorithm
            if sat.psat < Modelica.Media.R134a.R134aData.data.FPCRIT then
              state.p := sat.psat;
              state.T := saturationTemperature(sat.psat);
              state.d := bubbleDensity(sat);
              state.h := bubbleEnthalpy(sat);
            else
              assert(sat.psat < Modelica.Media.R134a.R134aData.data.FPCRIT, "Function setBubbleState is only valid in two-phase regime");
            end if;
          annotation(
            Documentation(info = "<html>
<p>This function shall be used in order to calculate the thermodynamic state record for the liquid phase boundary. It requires the saturation record as input which can be determined by both functions setSat_p and setSat_T:
</p>
<p>
Example:
<pre>
    Medium.AbsolutePressure p=3e5;
    // Viscosity on the liquid phase boundary
    Modelica.SIunits.DynamicViscosity eta_liq;

    equation

    eta_liq = Medium.DynamicViscosity(Medium.setBubbleState(Medium.setSat_p(p)));
</pre>

<h3> Restrictions</h3>
<p>It is only valid in the two-phase region (i.e., p<sub>triple</sub> &le; p &le; p<sub>crit</sub> ).
</p>
</html>"));
        end setBubbleState;

        redeclare function extends setDewState "Return the thermodynamic state on the dew line"
          algorithm
            if sat.psat < Modelica.Media.R134a.R134aData.data.FPCRIT then
              state.p := sat.psat;
              state.T := saturationTemperature(sat.psat);
              state.d := dewDensity(sat);
              state.h := dewEnthalpy(sat);
            else
              assert(sat.psat < Modelica.Media.R134a.R134aData.data.FPCRIT, "Function setDewState is only valid in two-phase regime");
            end if;
          annotation(
            Documentation(info = "<html>
<p>This function shall be used in order to calculate the thermodynamic state record for the vapor phase boundary. It requires the saturation record as input which can be determined by both functions setSat_p and setSat_T:
</p>
<p>
Example:
<pre>
    Medium.AbsolutePressure p=3e5;
    // Viscosity on the vapor phase boundary
    Modelica.SIunits.DynamicViscosity eta_vap;

    equation

    eta_vap = Medium.DynamicViscosity(Medium.setBubbleState(Medium.setSat_p(p)));
</pre>

<h3> Restrictions</h3>
<p>It is only valid in the two-phase region (i.e., p<sub>triple</sub> &le; p &le; p<sub>crit</sub> ).
</p>
</html>"));
        end setDewState;

        redeclare function density_ph "Density as function of pressure and specific enthalpy"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SpecificEnthalpy h "Specific enthalpy";
          input Integer phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          output Density d "Density";
        algorithm
          d := rho_props_ph(p, h, derivsOf_ph(p, h, getPhase_ph(p, h)));
          annotation(
            Inline = true,
            Documentation(info = "<html>
<p> This function calculates the density of R134a from the state variables p (absolute pressure) and h (specific enthalpy). The density is modelled by the fundamental equation of state of Tillner-Roth and Baehr (1994). </p>

<p><img src=\"modelica://Modelica/Resources/Images/Media/R134a/log(p)d-Diagram-R134a.png\"/> </p>

</html>"));
        end density_ph;

        redeclare function extends density "Density as function of pressure and specific enthalpy | use setState_phX function for input"
          algorithm
            d := state.d;
          annotation(
            Inline = true,
            Documentation(info = "<html>
<p>
This function calculates the density of R134a from the state record
(e.g., use setState_phX function for input). The density is modelled
by the fundamental equation of state of Tillner-Roth and Baehr (1994).
</p>

<p><img src=\"modelica://Modelica/Resources/Images/Media/R134a/log(p)d-Diagram-R134a.png\"/> </p>
</html>"));
        end density;

        redeclare function temperature_ph "Temperature as function of pressure and specific enthalpy"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SpecificEnthalpy h "Specific enthalpy";
          input Integer phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          output Temperature T "Temperature";
        algorithm
          T := T_props_ph(p, h, derivsOf_ph(p, h, getPhase_ph(p, h)));
          annotation(
            Inline = true,
            Documentation(info = "<html>
<p> This function calculates the Kelvin temperature of R134a from the state variables p (absolute pressure) and h (specific enthalpy). The temperature is modelled by the fundamental equation of state of Tillner-Roth and Baehr (1994). </p>

<p><img src=\"modelica://Modelica/Resources/Images/Media/R134a/log(p)h-Diagram-R134a.png\"/> </p>


</html>"));
        end temperature_ph;

        redeclare function extends temperature "Temperature as function of pressure and specific enthalpy | use setState_phX function for input"
          algorithm
            T := state.T;
          annotation(
            Inline = true,
            Documentation(info = "<html>
<p> This function calculates the Kelvin temperature of R134a from the state record (e.g., use setState_phX function for input). The temperature is modelled by the fundamental equation of state of Tillner-Roth and Baehr (1994). </p>
<p><img src=\"modelica://Modelica/Resources/Images/Media/R134a/log(p)h-Diagram-R134a.png\"/> </p>
</html>"));
        end temperature;

        redeclare function extends pressure "Pressure w.r.t. thermodynamic state"
          algorithm
            p := state.p;
          annotation(
            Inline = true,
            Documentation(info = "<html>
<p>This function is included for the sake of completness.</p>
<p><img src=\"modelica://Modelica/Resources/Images/Media/R134a/log(p)h-Diagram-R134a.png\"/> </p>
</html>"));
        end pressure;

        redeclare function extends specificInternalEnergy "Specific internal energy w.r.t. thermodynamic state"
          algorithm
            u := specificEnthalpy(state) - pressure(state) / density(state);
          annotation(
            Inline = true,
            Documentation(info = "<html>
<p> This function calculates the specific internal energy of R134a from the state record (e.g., use setState_phX function for input). The specific internal energy is modelled by the fundamental equation of state of Tillner-Roth and Baehr (1994). </p>
<p><img src=\"modelica://Modelica/Resources/Images/Media/R134a/log(p)u-Diagram-R134a.png\"/> </p>

</html>"));
        end specificInternalEnergy;

        redeclare function extends specificEnthalpy "Specific enthalpy w.r.t. thermodynamic state | use setState_phX function for input"
          algorithm
            h := state.h;
          annotation(
            Inline = true,
            Documentation(info = "<html>
<p>This function is included for the sake of completness.</p>
<p><img src=\"modelica://Modelica/Resources/Images/Media/R134a/log(p)h-Diagram-R134a.png\"/> </p>
</html>"));
        end specificEnthalpy;

        redeclare function extends specificEntropy "Specific entropy w.r.t. thermodynamic state | use setState_phX function for input if necessary"
          protected
            Modelica.Media.Common.HelmholtzDerivs f "Helmholtz derivatives";
            Common.PhaseBoundaryProperties liq "Properties on liquid phase boundary";
            SaturationProperties sat "Saturation temperature and pressure";
            Common.PhaseBoundaryProperties vap "Properties on vapor phase boundary";
            Modelica.SIunits.MassFraction x "Vapor quality";

          algorithm
            if getPhase_ph(state.p, state.h) == 2 then
              sat.psat := state.p;
              sat.Tsat := saturationTemperature(state.p);
              liq := R134a_liqofdT(T = state.T);
              vap := R134a_vapofdT(T = state.T);
              x := if liq.h <> vap.h then (state.h - liq.h) / (vap.h - liq.h) else if state.h >= vap.h then 1 else 0;
              s := liq.s + x * (vap.s - liq.s);
            else
              f := f_R134a(state.d, state.T);
              s := R134aData.R * (f.tau * f.ftau - f.f);
            end if;
          annotation(
            Documentation(info = "<html>
<p> This function calculates the specific entropy of R134a from the state record (e.g., use setState_phX function for input). The specific entropy is modelled by the fundamental equation of state of Tillner-Roth and Baehr (1994). </p>
<p><img src=\"modelica://Modelica/Resources/Images/Media/R134a/log(p)s-Diagram-R134a.png\"/> </p>
</html>"));
        end specificEntropy;

        redeclare function extends saturationTemperature "Saturation temperature in two-phase region"
          protected
            constant Real T_coef[:, :] = R134aData.Tcoef "Coefficients of cubic spline for Tsat(p)";
            constant Real p_breaks[:] = R134aData.pbreaks "Grid points of reduced pressure";
            Integer int "Interval number";
            Integer error "Interval for spline interpolation not found";
            Real pred "Reduced pressure";
            Real localx "Oordinate of local spline";

          algorithm
            pred := p / R134aData.data.FPCRIT;
            (int, error) := Common.FindInterval(pred, p_breaks);
            localx := pred - p_breaks[int];
            T := Common.CubicSplineEval(localx, T_coef[int, :]);
// annotation(smoothOrder=5);
          annotation(
            derivative = saturationTemperature_der_p,
            Documentation(info = "<html>
<p>This function calculates the saturation temperature of R134a from the state variable p (absolute pressure). It is modelled by cubic splines which are fitted with non-equidistant grid points derived from
the fundamental equation of state of Tillner-Roth and Baehr (1994) and the Maxwell criteria.
</p>
<h3> Restrictions</h3>
<p>It is only valid in the two-phase region (i.e., p<sub>triple</sub> &le; p &le; p<sub>crit</sub> ).
</p>
<p><img src=\"modelica://Modelica/Resources/Images/Media/R134a/log(p)Tsat-Diagram-R134a.png\"/> </p>
</html>"));
        end saturationTemperature;

        redeclare function extends saturationTemperature_derp "Derivative of saturation temperature in two-phase region"
          protected
            constant Real T_coef[:, :] = R134aData.Tcoef "Coefficients of cubic spline for Tsat(p)";
            constant Real p_breaks[:] = R134aData.pbreaks "Grid points of reduced pressure";
            Integer int "Interval number";
            Integer error "Interval for spline interpolation not found";
            Real pred "Reduced pressure";
            Real localx "Oordinate of local spline";

          algorithm
            pred := p / R134aData.data.FPCRIT;
            (int, error) := Common.FindInterval(pred, p_breaks);
            localx := pred - p_breaks[int];
            dTp := Common.CubicSplineDerEval(localx, T_coef[int, :]) / R134aData.data.FPCRIT;
          annotation(
            Documentation(info = "<html>
<p>This function calculates the derivative of saturation temperature of R134a with regard to the state variable p (absolute pressure). The non-derivative function is <a href=\"modelica://Modelica.Media.R134a.R134a_ph.saturationTemperature\"> saturatuionTemperature</a>.
</p>
<h3> Restrictions</h3>
<p>It is only valid in the two-phase region (i.e., p<sub>triple</sub> &le; p &le; p<sub>crit</sub> ).
</p>
</html>"));
        end saturationTemperature_derp;

        function saturationTemperature_der_p "Time derivative of saturation temperature in two-phase region"
          extends Modelica.Icons.Function;
          input Modelica.SIunits.AbsolutePressure p "Pressure";
          input Real der_p "Time derivative of pressure";
          output Real der_Tsat "Time derivative of saturation temperature";
        protected
          constant Real T_coef[:, :] = R134aData.Tcoef "Coefficients of cubic spline for Tsat(p)";
          constant Real p_breaks[:] = R134aData.pbreaks "Grid points of reduced pressure";
          Integer int "Interval number";
          Integer error "Interval for spline interpolation not found";
          Real pred "Reduced pressure";
          Real localx "Oordinate of local spline";
        algorithm
          pred := p / R134aData.data.FPCRIT;
          (int, error) := Common.FindInterval(pred, p_breaks);
          localx := pred - p_breaks[int];
          der_Tsat := Common.CubicSplineDerEval(localx, T_coef[int, :]) / R134aData.data.FPCRIT * der_p;
          annotation(
            Documentation(info = "<html>
<p>This function calculates the time derivative of saturation temperature of R134a with regard to the time derivative of p. The non-derivative function is <a href=\"modelica://Modelica.Media.R134a.R134a_ph.saturationTemperature\"> saturatuionTemperature</a>.
</p>
<h3> Restrictions</h3>
<p>It is only valid in the two-phase region (i.e., p<sub>triple</sub> &le; p &le; p<sub>crit</sub> ).
</p>
</html>"));
        end saturationTemperature_der_p;

        redeclare function extends bubbleDensity "Density of liquid phase w.r.t saturation pressure | use setSat_p function for input"
          protected
            constant Real dl_coef[:, :] = R134aData.dlcoef "Coefficients of cubic spline for d_liq(p)";
            constant Real p_breaks[:] = R134aData.pbreaks "Grid points of reduced pressure";
            Integer int "Interval number";
            Integer error "Interval for spline interpolation not found";
            Real pred "Reduced pressure";
            Real localx "Oordinate of local spline";

          algorithm
            pred := min(sat.psat / R134aData.data.FPCRIT, 1.0);
            (int, error) := Common.FindInterval(pred, p_breaks);
            localx := pred - p_breaks[int];
            dl := R134aData.data.FDCRIT * Common.CubicSplineEval(localx, dl_coef[int, 1:4]);
// annotation(smoothOrder=5);
          annotation(
            derivative = dBubbleDensity_dPressure_der_sat,
            Documentation(info = "<html>
<p>This function calculates the liquid phase density of R134a from the state variable p (absolute pressure). It is modelled by cubic splines which are fitted with non-equidistant grid points derived from
the fundamental equation of state of Tillner-Roth and Baehr (1994) and the Maxwell criteria.
</p>
<h3> Restrictions</h3>
<p>It is only valid in the two-phase region (i.e., p<sub>triple</sub> &le; p &le; p<sub>crit</sub> ).
</p>
</html>"));
        end bubbleDensity;

        redeclare function extends dBubbleDensity_dPressure "Derivative of liquid density in two-phase region w.r.t pressure"
          protected
            constant Real dl_coef[:, :] = R134aData.dlcoef "Coefficients of cubic spline for d_liq(p)";
            constant Real p_breaks[:] = R134aData.pbreaks "Grid points of reduced pressure";
            Integer int "Interval number";
            Integer error "Interval for spline interpolation not found";
            Real pred "Reduced pressure";
            Real localx "Oordinate of local spline";

          algorithm
            pred := min(sat.psat / R134aData.data.FPCRIT, 1.0);
            (int, error) := Common.FindInterval(pred, p_breaks);
            localx := pred - p_breaks[int];
            ddldp := R134aData.data.FDCRIT * Common.CubicSplineDerEval(localx, dl_coef[int, 1:4]) / R134aData.data.FPCRIT;
          annotation(
            Documentation(info = "<html>
<p>This function calculates the derivative of liquid density of R134a in the two-phase region with regard to the state variable p (absolute pressure). The non-derivative function is <a href=\"modelica://Modelica.Media.R134a.R134a_ph.bubbleDensity\"> bubbleDensity</a>.
</p>
<h3> Restrictions</h3>
<p>It is only valid in the two-phase region (i.e., p<sub>triple</sub> &le; p &le; p<sub>crit</sub> ).
</p>
</html>"));
        end dBubbleDensity_dPressure;

        function dBubbleDensity_dPressure_der_sat "Time derivative of liquid density in two-phase region w.r.t pressure"
          extends Modelica.Icons.Function;
          input SaturationProperties sat "Saturation properties | pressure is used for interpolation";
          input SaturationProperties der_sat "Derivative of saturation properties";
          output Real der_ddldp "Time derivative of liquid density in two-phase region w.r.t pressure";
        protected
          constant Real dl_coef[:, :] = R134aData.dlcoef "Coefficients of cubic spline for d_liq(p)";
          constant Real p_breaks[:] = R134aData.pbreaks "Grid points of reduced pressure";
          Integer int "Interval number";
          Integer error "Interval for spline interpolation not found";
          Real pred "Reduced pressure";
          Real localx "Oordinate of local spline";
        algorithm
          pred := min(sat.psat / R134aData.data.FPCRIT, 1.0);
          (int, error) := Common.FindInterval(pred, p_breaks);
          localx := pred - p_breaks[int];
          der_ddldp := R134aData.data.FDCRIT * Common.CubicSplineDerEval(localx, dl_coef[int, 1:4]) / R134aData.data.FPCRIT * der_sat.psat;
          annotation(
            Documentation(info = "<html>
<p>This function calculates the time derivative of liquid density of R134a with regard to the time derivative of p. The non-derivative function is <a href=\"modelica://Modelica.Media.R134a.R134a_ph.bubbleDensity\"> bubbleDensity</a>.
</p>
<h3> Restrictions</h3>
<p>It is only valid in the two-phase region (i.e., p<sub>triple</sub> &le; p &le; p<sub>crit</sub> ).
</p>
</html>"));
        end dBubbleDensity_dPressure_der_sat;

        redeclare function extends dewDensity "Density of vapor phase w.r.t saturation pressure | use setSat_p function for input"
          protected
            constant Real dv_coef[:, :] = R134aData.dvcoef "Coefficients of cubic spline for d_vap(p)";
            constant Real p_breaks[:] = R134aData.pbreaks "Grid points of reduced pressure";
            Integer int "Interval number";
            Integer error "Interval for spline interpolation not found";
            Real pred "Reduced pressure";
            Real localx "Oordinate of local spline";

          algorithm
            pred := min(sat.psat / R134aData.data.FPCRIT, 1.0);
            (int, error) := Common.FindInterval(pred, p_breaks);
            localx := pred - p_breaks[int];
            dv := R134aData.data.FDCRIT * Common.CubicSplineEval(localx, dv_coef[int, 1:4]);
// annotation(smoothOrder=5);
          annotation(
            derivative = dDewDensity_dPressure_der_sat,
            Documentation(info = "<html>
<p>This function calculates the vapor phase density of R134a from the state variable p (absolute pressure). It is modelled by cubic splines which are fitted with non-equidistant grid points derived from
the fundamental equation of state of Tillner-Roth and Baehr (1994) and the Maxwell criteria.
</p>
<h3> Restrictions</h3>
<p>It is only valid in the two-phase region (i.e., p<sub>triple</sub> &le; p &le; p<sub>crit</sub> ).
</p>
</html>"));
        end dewDensity;

        redeclare function extends dDewDensity_dPressure "Derivative of vapor density in two-phase region w.r.t pressure"
          protected
            constant Real dv_coef[:, :] = R134aData.dvcoef "Coefficients of cubic spline for d_vap(p)";
            constant Real p_breaks[:] = R134aData.pbreaks "Grid points of reduced pressure";
            Integer int "Interval number";
            Integer error "Interval for spline interpolation not found";
            Real pred "Reduced pressure";
            Real localx "Oordinate of local spline";

          algorithm
            pred := min(sat.psat / R134aData.data.FPCRIT, 1.0);
            (int, error) := Common.FindInterval(pred, p_breaks);
            localx := pred - p_breaks[int];
            ddvdp := R134aData.data.FDCRIT * Common.CubicSplineDerEval(localx, dv_coef[int, 1:4]) / R134aData.data.FPCRIT;
          annotation(
            Documentation(info = "<html>
<p>This function calculates the derivative of vapor density of R134a in two-phase region with regard to the state variable p (absolute pressure). The non-derivative function is <a href=\"modelica://Modelica.Media.R134a.R134a_ph.dewDensity\"> dewDensity</a>.
</p>
<h3> Restrictions</h3>
<p>It is only valid in the two-phase region (i.e., p<sub>triple</sub> &le; p &le; p<sub>crit</sub> ).
</p>
</html>"));
        end dDewDensity_dPressure;

        function dDewDensity_dPressure_der_sat "Time derivative of vapor density in two-phase region w.r.t pressure"
          extends Modelica.Icons.Function;
          input SaturationProperties sat "Saturation properties | pressure is used for interpolation";
          input SaturationProperties der_sat "Derivative of saturation properties";
          output Real der_ddvdp "Time derivative of vapor density in two-phase region w.r.t pressure";
        protected
          constant Real dv_coef[:, :] = R134aData.dvcoef "Coefficients of cubic spline for d_vap(p)";
          constant Real p_breaks[:] = R134aData.pbreaks "Grid points of reduced pressure";
          Integer int "Interval number";
          Integer error "Interval for spline interpolation not found";
          Real pred "Reduced pressure";
          Real localx "Oordinate of local spline";
        algorithm
          pred := min(sat.psat / R134aData.data.FPCRIT, 1.0);
          (int, error) := Common.FindInterval(pred, p_breaks);
          localx := pred - p_breaks[int];
          der_ddvdp := R134aData.data.FDCRIT * Common.CubicSplineDerEval(localx, dv_coef[int, 1:4]) / R134aData.data.FPCRIT * der_sat.psat;
          annotation(
            Documentation(info = "<html>
<p>This function calculates the time derivative of vapor density of R134a with regard to the time derivative of p. The non-derivative function is <a href=\"modelica://Modelica.Media.R134a.R134a_ph.dewDensity\"> dewDensity</a>.
</p>
<h3> Restrictions</h3>
<p>It is only valid in the two-phase region (i.e., p<sub>triple</sub> &le; p &le; p<sub>crit</sub> ).
</p>
</html>"));
        end dDewDensity_dPressure_der_sat;

        redeclare function extends bubbleEnthalpy "Specific enthalpy of liquid phase w.r.t saturation pressure | use setSat_p function for input"
          protected
            constant Real hl_coef[:, :] = R134aData.hlcoef "Coefficients of cubic spline for h_liq(p)";
            constant Real p_breaks[:] = R134aData.pbreaks "Grid points of reduced pressure";
            Integer int "Interval number";
            Integer error "Interval for spline interpolation not found";
            Real pred "Reduced pressure";
            Real localx "Oordinate of local spline";

          algorithm
            pred := min(sat.psat / R134aData.data.FPCRIT, 1.0);
            (int, error) := Common.FindInterval(pred, p_breaks);
            localx := pred - p_breaks[int];
            hl := R134aData.data.HCRIT * Common.CubicSplineEval(localx, hl_coef[int, 1:4]);
// annotation(smoothOrder=5);
          annotation(
            derivative = dBubbleEnthalpy_dPressure_der_sat,
            Documentation(info = "<html>
<p>This function calculates the liquid phase enthalpy of R134a from the state variable p (absolute pressure). It is modelled by cubic splines which are fitted with non-equidistant grid points derived from
the fundamental equation of state of Tillner-Roth and Baehr (1994) and the Maxwell criteria.
</p>
<h3> Restrictions</h3>
<p>It is only valid in the two-phase region (i.e., p<sub>triple</sub> &le; p &le; p<sub>crit</sub> ).
</p>
</html>"));
        end bubbleEnthalpy;

        redeclare function extends dBubbleEnthalpy_dPressure "Derivative of liquid specific enthalpy in two-phase region w.r.t pressure"
          protected
            constant Real hl_coef[:, :] = R134aData.hlcoef "Coefficients of cubic spline for h_liq(p)";
            constant Real p_breaks[:] = R134aData.pbreaks "Grid points of reduced pressure";
            Integer int "Interval number";
            Integer error "Interval for spline interpolation not found";
            Real pred "Reduced pressure";
            Real localx "Oordinate of local spline";

          algorithm
            pred := min(sat.psat / R134aData.data.FPCRIT, 1.0);
            (int, error) := Common.FindInterval(pred, p_breaks);
            localx := pred - p_breaks[int];
            dhldp := R134aData.data.HCRIT * Common.CubicSplineDerEval(localx, hl_coef[int, 1:4]) / R134aData.data.FPCRIT;
          annotation(
            Documentation(info = "<html>
<p>This function calculates the derivative of liquid enthalpy of R134a with regard to the state variable p (absolute pressure). The non-derivative function is <a href=\"modelica://Modelica.Media.R134a.R134a_ph.bubbleEnthalpy\"> bubbleEnthalpy</a>.
</p>
<h3> Restrictions</h3>
<p>It is only valid in the two-phase region (i.e., p<sub>triple</sub> &le; p &le; p<sub>crit</sub> ).
</p>
</html>"));
        end dBubbleEnthalpy_dPressure;

        function dBubbleEnthalpy_dPressure_der_sat "Time derivative of liquid specific enthalpy in two-phase region w.r.t pressure"
          extends Modelica.Icons.Function;
          input SaturationProperties sat "Saturation properties | pressure is used for interpolation";
          input SaturationProperties der_sat "Derivative of saturation properties";
          output Real der_dhldp "Time derivative of liquid specific enthalpy in two-phase region w.r.t pressure";
        protected
          constant Real hl_coef[:, :] = R134aData.hlcoef "Coefficients of cubic spline for h_liq(p)";
          constant Real p_breaks[:] = R134aData.pbreaks "Grid points of reduced pressure";
          Integer int "Interval number";
          Integer error "Interval for spline interpolation not found";
          Real pred "Reduced pressure";
          Real localx "Oordinate of local spline";
        algorithm
          pred := min(sat.psat / R134aData.data.FPCRIT, 1.0);
          (int, error) := Common.FindInterval(pred, p_breaks);
          localx := pred - p_breaks[int];
          der_dhldp := R134aData.data.HCRIT * Common.CubicSplineDerEval(localx, hl_coef[int, 1:4]) / R134aData.data.FPCRIT * der_sat.psat;
          annotation(
            Documentation(info = "<html>
<p>This function calculates the time derivative of liquid specific enthalpy of R134a with regard to the time derivative of p. The non-derivative function is <a href=\"modelica://Modelica.Media.R134a.R134a_ph.bubbleEnthalpy\"> bubbleEnthalpy</a>.
</p>
<h3> Restrictions</h3>
<p>It is only valid in the two-phase region (i.e., p<sub>triple</sub> &le; p &le; p<sub>crit</sub> ).
</p>
</html>"));
        end dBubbleEnthalpy_dPressure_der_sat;

        redeclare function extends dewEnthalpy "Specific enthalpy of vapor phase w.r.t saturation pressure | use setSat_p function for input"
          protected
            constant Real hv_coef[:, :] = R134aData.hvcoef "Coefficients of cubic spline for h_vap(p)";
            constant Real p_breaks[:] = R134aData.pbreaks "Grid points of reduced pressure";
            Integer int "Interval number";
            Integer error "Interval for spline interpolation not found";
            Real pred "Reduced pressure";
            Real localx "Oordinate of local spline";

          algorithm
            pred := min(sat.psat / R134aData.data.FPCRIT, 1.0);
            (int, error) := Common.FindInterval(pred, p_breaks);
            localx := pred - p_breaks[int];
            hv := R134aData.data.HCRIT * Common.CubicSplineEval(localx, hv_coef[int, 1:4]);
// annotation(smoothOrder=5);
          annotation(
            derivative = dDewEnthalpy_dPressure_der_sat,
            Documentation(info = "<html>
<p>This function calculates the vapor phase enthalpy of R134a from the state variable p (absolute pressure). It is modelled by cubic splines which are fitted with non-equidistant grid points derived from
the fundamental equation of state of Tillner-Roth and Baehr (1994) and the Maxwell criteria.
</p>
<h3> Restrictions</h3>
<p>It is only valid in the two-phase region (i.e., p<sub>triple</sub> &le; p &le; p<sub>crit</sub> ).
</p>
</html>"));
        end dewEnthalpy;

        redeclare function extends dDewEnthalpy_dPressure "Derivative of vapor specific enthalpy in two-phase region w.r.t pressure"
          protected
            constant Real hv_coef[:, :] = R134aData.hvcoef "Coefficients of cubic spline for h_vap(p)";
            constant Real p_breaks[:] = R134aData.pbreaks "Grid points of reduced pressure";
            Integer int "Interval number";
            Integer error "Interval for spline interpolation not found";
            Real pred "Reduced pressure";
            Real localx "Oordinate of local spline";

          algorithm
            pred := min(sat.psat / R134aData.data.FPCRIT, 1.0);
            (int, error) := Common.FindInterval(pred, p_breaks);
            localx := pred - p_breaks[int];
            dhvdp := R134aData.data.HCRIT * Common.CubicSplineDerEval(localx, hv_coef[int, 1:4]) / R134aData.data.FPCRIT;
          annotation(
            Documentation(info = "<html>
<p>This function calculates the derivative of vapor enthalpy of R134a in the two-phase region with regard to the state variable p (absolute pressure). The non-derivative function is <a href=\"modelica://Modelica.Media.R134a.R134a_ph.dewEnthalpy\"> dewEnthalpy</a>.
</p>
<h3> Restrictions</h3>
<p>It is only valid in the two-phase region (i.e., p<sub>triple</sub> &le; p &le; p<sub>crit</sub> ).
</p>
</html>"));
        end dDewEnthalpy_dPressure;

        function dDewEnthalpy_dPressure_der_sat "Time derivative of vapor specific enthalpy in two-phase region w.r.t pressure"
          extends Modelica.Icons.Function;
          input SaturationProperties sat "Saturation properties | pressure is used for interpolation";
          input SaturationProperties der_sat "Derivative of saturation properties";
          output Real der_dhvdp "Derivative of vapor specific enthalpy in two-phase region w.r.t pressure";
        protected
          constant Real hv_coef[:, :] = R134aData.hvcoef "Coefficients of cubic spline for h_vap(p)";
          constant Real p_breaks[:] = R134aData.pbreaks "Grid points of reduced pressure";
          Integer int "Interval number";
          Integer error "Interval for spline interpolation not found";
          Real pred "Reduced pressure";
          Real localx "Oordinate of local spline";
        algorithm
          pred := min(sat.psat / R134aData.data.FPCRIT, 1.0);
          (int, error) := Common.FindInterval(pred, p_breaks);
          localx := pred - p_breaks[int];
          der_dhvdp := R134aData.data.HCRIT * Common.CubicSplineDerEval(localx, hv_coef[int, 1:4]) / R134aData.data.FPCRIT * der_sat.psat;
          annotation(
            Documentation(info = "<html>
<p>This function calculates the time derivative of vapor enthalpy of R134a with regard to the time derivative of p. The non-derivative function is <a href=\"modelica://Modelica.Media.R134a.R134a_ph.dewEnthalpy\"> dewEnthalpy</a>.
</p>
<h3> Restrictions</h3>
<p>It is only valid in the two-phase region (i.e., p<sub>triple</sub> &le; p &le; p<sub>crit</sub> ).
</p>
</html>"));
        end dDewEnthalpy_dPressure_der_sat;

        redeclare function extends dewEntropy "Specific entropy of vapor phase w.r.t saturation pressure | use setSat_p function for input"
          protected
            constant Real sv_coef[:, :] = R134aData.svcoef "Coefficients of cubic spline for s_vap(p)";
            constant Real p_breaks[:] = R134aData.pbreaks "Grid points of reduced pressure";
            Integer int "Interval number";
            Integer error "Interval for spline interpolation not found";
            Real pred "Reduced pressure";
            Real localx "Oordinate of local spline";

          algorithm
            pred := min(sat.psat / R134aData.data.FPCRIT, 1.0);
            (int, error) := Common.FindInterval(pred, p_breaks);
            localx := pred - p_breaks[int];
            sv := R134aData.data.SCRIT * Common.CubicSplineEval(localx, sv_coef[int, 1:4]);
// annotation(smoothOrder=5);
          annotation(
            derivative = dDewEntropy_dPressure_der_sat,
            Documentation(info = "<html>
<p>This function calculates the vapor phase entropy of R134a from the state variable p (absolute pressure). It is modelled by cubic splines which are fitted with non-equidistant grid points derived from
the fundamental equation of state of Tillner-Roth and Baehr (1994) and the Maxwell criteria.
</p>
<h3> Restrictions</h3>
<p>It is only valid in the two-phase region (i.e., p<sub>triple</sub> &le; p &le; p<sub>crit</sub> ).
</p>
</html>"));
        end dewEntropy;

        function dDewEntropy_dPressure "Derivative of vapor specific entropy in two-phase region w.r.t pressure | use setState_phX function for input"
          extends Modelica.Icons.Function;
          input SaturationProperties sat "Saturation properties | pressure is used for interpolation";
          output Real dsvdp "Derivative of vapor specific entropy in two-phase region w.r.t pressure";
        protected
          constant Real sv_coef[:, :] = R134aData.svcoef "Coefficients of cubic spline for s_vap(p)";
          constant Real p_breaks[:] = R134aData.pbreaks "Grid points of reduced pressure";
          Integer int "Interval number";
          Integer error "Interval for spline interpolation not found";
          Real pred "Reduced pressure";
          Real localx "Oordinate of local spline";
        algorithm
          pred := min(sat.psat / R134aData.data.FPCRIT, 1.0);
          (int, error) := Common.FindInterval(pred, p_breaks);
          localx := pred - p_breaks[int];
          dsvdp := R134aData.data.SCRIT * Common.CubicSplineDerEval(localx, sv_coef[int, 1:4]) / R134aData.data.FPCRIT;
          annotation(
            Documentation(info = "<html>
<p>This function calculates the derivative of vapor entropy of R134a with regard to the state variable p (absolute pressure). The non-derivative function is <a href=\"modelica://Modelica.Media.R134a.R134a_ph.dewEntropy\"> dewEntropy</a>.
</p>
<h3> Restrictions</h3>
<p>It is only valid in the two-phase region (i.e., p<sub>triple</sub> &le; p &le; p<sub>crit</sub> ).
</p>
</html>"));
        end dDewEntropy_dPressure;

        function dDewEntropy_dPressure_der_sat "Time derivative of vapor specific entropy in two-phase region w.r.t pressure | use setState_phX function for input"
          extends Modelica.Icons.Function;
          input SaturationProperties sat "Saturation properties | pressure is used for interpolation";
          input SaturationProperties der_sat "Derivative of saturation properties";
          output Real der_dsvdp "Derivative of vapor specific entropy in two-phase region w.r.t pressure";
        protected
          constant Real sv_coef[:, :] = R134aData.svcoef "Coefficients of cubic spline for s_liq(p)";
          constant Real p_breaks[:] = R134aData.pbreaks "Grid points of reduced pressure";
          Integer int "Interval number";
          Integer error "Interval for spline interpolation not found";
          Real pred "Reduced pressure";
          Real localx "Oordinate of local spline";
        algorithm
          pred := min(sat.psat / R134aData.data.FPCRIT, 1.0);
          (int, error) := Common.FindInterval(pred, p_breaks);
          localx := pred - p_breaks[int];
          der_dsvdp := R134aData.data.SCRIT * Common.CubicSplineDerEval(localx, sv_coef[int, 1:4]) / R134aData.data.FPCRIT * der_sat.psat;
          annotation(
            Documentation(info = "<html>
<p>This function calculates the time derivative of vapor specific entropy of R134a with regard to the time derivative of p. The non-derivative function is <a href=\"modelica://Modelica.Media.R134a.R134a_ph.dewEntropy\"> dewEntropy</a>.
</p>
<h3> Restrictions</h3>
<p>It is only valid in the two-phase region (i.e., p<sub>triple</sub> &le; p &le; p<sub>crit</sub> ).
</p>
</html>"));
        end dDewEntropy_dPressure_der_sat;

        redeclare function extends bubbleEntropy "Specific entropy of liquid phase w.r.t saturation pressure | use setSat_p function for input"
          protected
            constant Real sl_coef[:, :] = R134aData.slcoef "Coefficients of cubic spline for s_liq(p)";
            constant Real p_breaks[:] = R134aData.pbreaks "Grid points of reduced pressure";
            Integer int "Interval number";
            Integer error "Interval for spline interpolation not found";
            Real pred "Reduced pressure";
            Real localx "Oordinate of local spline";

          algorithm
            pred := min(sat.psat / R134aData.data.FPCRIT, 1.0);
            (int, error) := Common.FindInterval(pred, p_breaks);
            localx := pred - p_breaks[int];
            sl := R134aData.data.SCRIT * Common.CubicSplineEval(localx, sl_coef[int, 1:4]);
// annotation(smoothOrder=5);
          annotation(
            derivative = dBubbleEntropy_dPressure_der_sat,
            Documentation(info = "<html>
<p>This function calculates the liquid phase entropy of R134a from the state variable p (absolute pressure). It is modelled by cubic splines which are fitted with non-equidistant grid points derived from
the fundamental equation of state of Tillner-Roth and Baehr (1994) and the Maxwell criteria.
</p>
<h3> Restrictions</h3>
<p>It is only valid in the two-phase region (i.e., p<sub>triple</sub> &le; p &le; p<sub>crit</sub> ).
</p>
</html>"));
        end bubbleEntropy;

        function dBubbleEntropy_dPressure "Derivative of liquid specific entropy in two-phase region w.r.t pressure | use setState_phX function for input"
          extends Modelica.Icons.Function;
          input SaturationProperties sat "Saturation properties | pressure is used for interpolation";
          output Real dsldp "Derivative of liquid specific entropy in two-phase region w.r.t pressure";
        protected
          constant Real sl_coef[:, :] = R134aData.slcoef "Coefficients of cubic spline for s_liq(p)";
          constant Real p_breaks[:] = R134aData.pbreaks "Grid points of reduced pressure";
          Integer int "Interval number";
          Integer error "Interval for spline interpolation not found";
          Real pred "Reduced pressure";
          Real localx "Oordinate of local spline";
        algorithm
          pred := min(sat.psat / R134aData.data.FPCRIT, 1.0);
          (int, error) := Common.FindInterval(pred, p_breaks);
          localx := pred - p_breaks[int];
          dsldp := R134aData.data.SCRIT * Common.CubicSplineDerEval(localx, sl_coef[int, 1:4]) / R134aData.data.FPCRIT;
          annotation(
            Documentation(info = "<html>
<p>This function calculates the derivative of liquid entropy of R134a with regard to the state variable p (absolute pressure). The non-derivative function is <a href=\"modelica://Modelica.Media.R134a.R134a_ph.bubbleEntropy\"> bubbleEntropy</a>.
</p>
<h3> Restrictions</h3>
<p>It is only valid in the two-phase region (i.e., p<sub>triple</sub> &le; p &le; p<sub>crit</sub> ).
</p>
</html>"));
        end dBubbleEntropy_dPressure;

        function dBubbleEntropy_dPressure_der_sat "Time derivative of liquid specific entropy in two-phase region w.r.t pressure | use setState_phX function for input"
          extends Modelica.Icons.Function;
          input SaturationProperties sat "Saturation properties | pressure is used for interpolation";
          input SaturationProperties der_sat "Derivative of saturation properties";
          output Real der_dsldp "Derivative of liquid specific entropy in two-phase region w.r.t pressure";
        protected
          constant Real sl_coef[:, :] = R134aData.slcoef "Coefficients of cubic spline for s_liq(p)";
          constant Real p_breaks[:] = R134aData.pbreaks "Grid points of reduced pressure";
          Integer int "Interval number";
          Integer error "Interval for spline interpolation not found";
          Real pred "Reduced pressure";
          Real localx "Oordinate of local spline";
        algorithm
          pred := min(sat.psat / R134aData.data.FPCRIT, 1.0);
          (int, error) := Common.FindInterval(pred, p_breaks);
          localx := pred - p_breaks[int];
          der_dsldp := R134aData.data.SCRIT * Common.CubicSplineDerEval(localx, sl_coef[int, 1:4]) / R134aData.data.FPCRIT * der_sat.psat;
          annotation(
            Documentation(info = "<html>
<p>This function calculates the time derivative of liquid specific entropy of R134a with regard to the time derivative of p. The non-derivative function is <a href=\"modelica://Modelica.Media.R134a.R134a_ph.bubbleEntropy\"> bubbleEntropy</a>.
</p>
<h3> Restrictions</h3>
<p>It is only valid in the two-phase region (i.e., p<sub>triple</sub> &le; p &le; p<sub>crit</sub> ).
</p>
</html>"));
        end dBubbleEntropy_dPressure_der_sat;

        redeclare function extends saturationPressure "Saturation pressure w.r.t. temperature"
          protected
            constant Real pt_coef[:, :] = R134aData.ptcoef "Coefficients of cubic spline for psat(T)";
            constant Real T_breaks[:] = R134aData.Tbreaks "Grid points of reduced pressure";
            Real Tred "Reduced temperature";
            Integer int "Interval number";
            Integer error "Interval for spline interpolation not found";
            Real pred "Reduced pressure";
            Real localx "Oordinate of local spline";

          algorithm
            Tred := T / R134aData.data.FTCRIT;
            (int, error) := Common.FindInterval(Tred, T_breaks);
            localx := Tred - T_breaks[int];
            p := R134aData.data.FPCRIT * Common.CubicSplineEval(localx, pt_coef[int, :]);
          annotation(
            smoothOrder = 5,
            Documentation(info = "<html>
<p>This function calculates the saturation pressure of R134a from the state variable T (temperature). It is modelled by cubic splines which are fitted with non-equidistant grid points derived from the fundamental equation of state of Tillner-Roth and Baehr (1994) and the Maxwell criteria.
</p>
<h3> Restrictions</h3>
<p>It is only valid in the two-phase region (i.e., p<sub>triple</sub> &le; p &le; p<sub>crit</sub> ).
</p>
<p><img src=\"modelica://Modelica/Resources/Images/Media/R134a/log(p)Tsat-Diagram-R134a.png\"/> </p>
</html>"));
        end saturationPressure;

        redeclare function extends specificHeatCapacityCp "Specific heat capacity at constant pressure | turns inifity in two-phase region! | use setState_phX function for input"
          protected
            Modelica.Media.Common.HelmholtzDerivs f "Helmholtz derivatives";

          algorithm
            f := f_R134a(state.d, state.T);
            cp := if getPhase_ph(state.p, state.h) == 2 then 0 else R134aData.R * ((-f.tau * f.tau * f.ftautau) + (f.delta * f.fdelta - f.delta * f.tau * f.fdeltatau) ^ 2 / (2 * f.delta * f.fdelta + f.delta * f.delta * f.fdeltadelta));
          annotation(
            Documentation(info = "<html>
<p> This function calculates the specific heat capacity of R134a at <b>constant pressure</b> from the state record (e.g., use setState_phX function for input). The specific heat capacity is modelled by the fundamental equation of state of Tillner-Roth and Baehr (1994). </p>
<h3> Restrictions</h3>
<p>This property is only defined in one-phase region.
</p>
<p><img src=\"modelica://Modelica/Resources/Images/Media/R134a/log(p)cp-Diagram-R134a.png\"/> </p>
</html>"));
        end specificHeatCapacityCp;

        redeclare function extends specificHeatCapacityCv "Specific heat capacity at constant volume | use setState_phX function for input"
          protected
            Modelica.Media.Common.HelmholtzDerivs f "Helmholtz derivatives";
            Common.PhaseBoundaryProperties liq "Properties on liquid phase boundary";
            SaturationProperties sat "Saturation temperature and pressure";
            Common.PhaseBoundaryProperties vap "Properties on vapor phase boundary";
            Modelica.SIunits.MassFraction x "Vapor quality";

          algorithm
            if getPhase_ph(state.p, state.h) == 2 then
              sat.psat := state.p;
              sat.Tsat := saturationTemperature(state.p);
              liq := R134a_liqofdT(T = state.T);
              vap := R134a_vapofdT(T = state.T);
              x := if liq.h <> vap.h then (state.h - liq.h) / (vap.h - liq.h) else if state.h >= vap.h then 1 else 0;
              cv := Common.cv2Phase(liq = liq, vap = vap, x = x, T = state.T, p = state.p);
            else
              f := f_R134a(state.d, state.T);
              cv := R134aData.R * (-f.tau * f.tau * f.ftautau);
            end if;
          annotation(
            Documentation(info = "<html>
<p> This function calculates the specific heat capacity of R134a at <b>constant volume</b> from the state record (e.g., use setState_phX function for input). The specific heat capacity is modelled by the fundamental equation of state of Tillner-Roth and Baehr (1994). </p>
<p>
Please note, that the function can also be called in the two-phase region, but the output is not continuous for a phase transition (see Tillner-Roth and Baehr, 1994). Values in two-phase region are considerably higher than in one-phase domain. The following figure just shows one-phase properties.
</p>
<p><img src=\"modelica://Modelica/Resources/Images/Media/R134a/log(p)cv-Diagram-R134a.png\"/> </p>
</html>"));
        end specificHeatCapacityCv;

        redeclare function extends dynamicViscosity "Dynamic viscosity w.r.t. temperature and density | use setState_phX function for input"
          protected
            Real omega_log "Log of collision integral";
            Real omega "Collision integral";
            constant Real K = 0.021357 "Constant for low density term eta_star";
            constant Modelica.SIunits.Length sigma = 0.50647e-09 "Hard-sphere diameter";
            constant Modelica.SIunits.Temperature epsilon_k = 288.82 "Empirical factor";
            constant Real a[5] = {2.218816e-01, -5.079322e-01, 1.285776e-01, -8.328165e-02, -2.713173e-02} "Coefficients for term of collision integral";
            constant Real b[13] = {-1.7999496, 4.6692621e+01, -5.3460794e+02, 3.3604074e+03, -1.3019164e+04, 3.3414230e+04, -5.8711743e+04, 7.1426686e+04, -5.9834012e+04, 3.3652741e+04, -1.2027350e+04, 2.4348205e+03, -2.0807957e+02} "Coefficients for term of 2nd viscosity virial coefficient";
            constant Real c[6] = {-0.331249e-01, -0.468509e-03, 0.156983, 3.073830, -0.306398, 0.215221} "Coefficients for term of residiual viscosity";
            constant Real DCRIT_mol = 4.9788 "Critical density in mol/l";
            Real d_red "Reduced density";
            Real B_eta "2nd visosity virial coeff.";
            Real E "Temperature contribution to residual viscosity";
            Real eta_res "Residual part";
            Real eta_star "Dilute part";

          algorithm
            omega_log := 0;
            for i in 1:5 loop
              omega_log := omega_log + a[i] * Modelica.Math.log(state.T / epsilon_k) ^ (i - 1);
            end for;
            omega := Modelica.Math.exp(omega_log);
            eta_star := K * sqrt(R134aData.data.MM * 1000) / ((sigma * 1e+09) ^ 2 * omega) * sqrt(state.T);
            d_red := state.d / R134aData.data.DCRIT;
            E := 1 + c[3] * state.T / R134aData.data.TCRIT;
            B_eta := 0;
            for i in 1:13 loop
              B_eta := B_eta + b[i] * sqrt(state.T / epsilon_k) ^ (i - 1);
            end for;
            B_eta := B_eta * 602.2137 * sigma ^ 3 * DCRIT_mol;
            eta_res := (c[1] + B_eta * eta_star) * d_red + c[2] * d_red ^ 2 + (c[5] + c[6] * d_red) / (c[4] * E - d_red) - c[5] / (c[4] * E);
            eta := (eta_star + eta_res * 1e3) * 1e-06;
          annotation(
            Documentation(info = "<html>
<p> This function calculates the dynamic viscosity of R134a from the state record (e.g., use setState_phX function for input). The dynamic viscosity is modelled by the corresponding states method of Klein, McLinden and Laesecke (1997). </p>
<h3> Restrictions</h3>
<p>This property is only defined in one-phase region.
</p>
<h3>References</h3>
<dl><dt>Klein, McLinden and Laesecke: </dt>
<dd><b>An improved extended corresponding states method for estimation of visocsity of pure refrigerants and mixtures</b>.
Int. J. Refrig., Vol. 20, No.3, pp. 208-217, 1997.</dd>
</dl>
<p><img src=\"modelica://Modelica/Resources/Images/Media/R134a/log(p)eta-Diagram-R134a.png\"/> </p>
</html>"));
        end dynamicViscosity;

        redeclare function extends thermalConductivity "Thermal conductivity w.r.t. thermodynamic state | use setState_phX function for input"
          protected
            Modelica.Media.Common.HelmholtzDerivs f "Helmholtz derivatives";
            Modelica.Media.Common.HelmholtzDerivs f_ref "Helmholtz derivatives for reference state";
            Modelica.SIunits.ThermalConductivity lambda_dg "Dilute gas contribution to lambda";
            R134aData.CoeffsThermalConductivity coeff "Coefficients of thermal conductivity model";
            Modelica.SIunits.ThermalConductivity lambda_reduced "Reduced lambda";
            Modelica.SIunits.ThermalConductivity lambda_crit "Enhancement of lambda in the critical region";
            Modelica.SIunits.ThermalConductivity chi_star "Correlation length";
            Modelica.SIunits.ThermalConductivity chi_star_ref "Correlation length";
            Modelica.SIunits.ThermalConductivity delta_chi "Chi_star - chi_star_ref";
            Real rho_molar "Molar density [mol/l]";
            Real dddp "Derivative of density w.r.t. pressure";
            Real dddp_ref "Derivative of density w.r.t. pressure for reference state";
            Modelica.SIunits.Length xi "Correlation length";
            Modelica.SIunits.SpecificHeatCapacity cp "Specific heat capacity at constant pressure";
            Modelica.SIunits.SpecificHeatCapacity cv "Specific heat capacity at constant volume";
            Modelica.SIunits.DynamicViscosity eta "Dynamic viscosity";
            Modelica.SIunits.ThermalConductivity omega "Crossover function";
            Modelica.SIunits.ThermalConductivity omega_0 "Crossover function";

          algorithm
            f := f_R134a(state.d, state.T);
            f_ref := f_R134a(state.d, coeff.T_ref);
// dilute gase contribution
            lambda_dg := coeff.a_0 + coeff.a_1 * state.T;
// reduced lambda
            rho_molar := state.d / (R134aData.data.MM * 1000);
// [mol/L]
            lambda_reduced := 0;
            for i in 1:4 loop
              lambda_reduced := lambda_reduced + coeff.b[i] * (rho_molar / coeff.rho_crit) ^ i;
            end for;
            lambda_reduced := lambda_reduced * coeff.lambda_reducing;
// critical lambda
            dddp := 1 / (f.R * f.T * f.delta * (2.0 * f.fdelta + f.delta * f.fdeltadelta));
            chi_star := state.d * coeff.p_crit * 1e3 / (coeff.rho_crit * R134aData.data.MM * 1000) ^ 2 * dddp;
            dddp_ref := 1 / (f_ref.R * f_ref.T * f_ref.delta * (2.0 * f_ref.fdelta + f_ref.delta * f_ref.fdeltadelta));
            chi_star_ref := state.d * coeff.p_crit * 1e3 / (coeff.rho_crit * R134aData.data.MM * 1000) ^ 2 * dddp_ref;
            delta_chi := max(1e-10, chi_star - chi_star_ref);
            xi := coeff.xi_0 * (1 / coeff.GAMMA * delta_chi) ^ (coeff.nu / coeff.gamma);
            cp := R134aData.data.R * ((-f.tau * f.tau * f.ftautau) + (f.delta * f.fdelta - f.delta * f.tau * f.fdeltatau) ^ 2 / (2 * f.delta * f.fdelta + f.delta * f.delta * f.fdeltadelta));
            eta := dynamicViscosity(state = state);
            cv := specificHeatCapacityCv(state = state);
            omega := 2 / Modelica.Constants.pi * ((cp - cv) / cp * Modelica.Math.atan(coeff.q_d * xi) + cv / cp * coeff.q_d * xi);
            omega_0 := 2 / Modelica.Constants.pi * (1 - Modelica.Math.exp(-1 / ((coeff.q_d * xi) ^ (-1) + 1 / 3 * (coeff.q_d * xi * coeff.rho_crit / (state.d / (R134aData.data.MM * 1000))) ^ 2)));
            lambda_crit := state.d * cp * (1.03 * Modelica.Constants.k * state.T) / (6 * Modelica.Constants.pi * eta * xi) * (omega - omega_0);
// conclusion
            lambda := max(lambda_dg + lambda_reduced + lambda_crit, 1e-8);
          annotation(
            Documentation(info = "<html>
<p> This function calculates the thermal conductivity of R134a from the state record (e.g., use setState_phX function for input). The thermal conductivity is modelled by the corresponding states model of McLinden, Klein. and Perkins (2000). </p>
<h3> Restrictions</h3>
<p>This property is only defined in one-phase region.
</p>
<h3>References</h3>
<dl><dt>McLinden, Klein. and Perkins: </dt>
<dd><b>An extended corresponding states model for the thermal conductivity
of refrigerants and refrigerant mixtures</b>.
Int. J. Refrig., 23 (2000) 43-63.</dd>
</dl>
<p><img src=\"modelica://Modelica/Resources/Images/Media/R134a/log(p)lambda-Diagram-R134a.png\"/> </p>
</html>"));
        end thermalConductivity;

        redeclare function extends surfaceTension "Surface tension as a function of temperature (below critical point)"
          protected
            Real tau "Reduced temperatur";
            R134aData.CoeffsSurfaceTension coeff "Polynomial coefficients";
            Modelica.SIunits.Temperature Tc = 374.21 "Critical temperature";

          algorithm
            if sat.Tsat > Tc then
              tau := 0.0;
              sigma := 0.0;
            else
              tau := 1.0 - sat.Tsat / Tc;
              sigma := 0.0;
              for i in 1:coeff.nterm loop
                sigma := sigma + coeff.sigmak[i] * tau ^ coeff.sigexp[i];
              end for;
            end if;
          annotation(
            Documentation(info = "<html>
<p> This function calculates the surface tension of R134a from the saturation record (e.g., use setSat_T function for input). The property is modelled by an approach of Okada and Higashi (1994). </p>
<h3> Restrictions</h3>
<p>This property is only defined in two-phase region.
</p>
<h3>References</h3>
<dl><dt>Okada and Higashi: </dt>
<dd><b>Surface tension correlation of HFC-134a and HCFC-123</b>.
Proceedings of the Joint Meeting of IIR Commissions B1, B2, E1, and E2, Padua, Italy, pp. 541-548, 1994.</dd>
</dl>
<p><img src=\"modelica://Modelica/Resources/Images/Media/R134a/Tsigma-Diagram-R134a.png\"/> </p>
</html>"));
        end surfaceTension;

        redeclare function extends velocityOfSound "Velocity of sound w.r.t. thermodynamic state (only valid for one-phase)"
          protected
            Modelica.Media.Common.HelmholtzDerivs f "Helmholtz derivatives";

          algorithm
            if getPhase_ph(state.p, state.h) == 2 then
              a := 0;
            else
              f := f_R134a(state.d, state.T);
              a := abs(R134aData.data.R * state.T * (2 * f.delta * f.fdelta + f.delta * f.delta * f.fdeltadelta - (f.delta * f.fdelta - f.delta * f.tau * f.fdeltatau) * (f.delta * f.fdelta - f.delta * f.tau * f.fdeltatau) / (f.tau * f.tau * f.ftautau))) ^ 0.5;
            end if;
// assert(getPhase_ph(state.p, state.h)==1, "Function for velocity of sound is only valid for one-phase regime!");
          annotation(
            Documentation(info = "<html>
<p> This function calculates the velocity of sound of R134a from the state record (e.g., use setState_phX function for input). The velocity of sound is modelled by the fundamental equation of state of Tillner-Roth and Baehr (1994). </p>
<h3> Restrictions</h3>
<p>This property is only defined in one-phase region.
</p>
<p><img src=\"modelica://Modelica/Resources/Images/Media/R134a/log(p)a-Diagram-R134a.png\"/> </p>
</html>"));
        end velocityOfSound;

        redeclare function extends isothermalCompressibility "Isothermal compressibility w.r.t. thermodynamic state (only valid for one-phase)"
          protected
            Modelica.Media.Common.HelmholtzDerivs f "Helmholtz derivatives";

          algorithm
            if getPhase_ph(state.p, state.h) == 2 then
              kappa := 0;
            else
              f := f_R134a(state.d, state.T);
              kappa := 1 / (state.d * R134aData.data.R * state.T * f.delta * (2.0 * f.fdelta + f.delta * f.fdeltadelta));
            end if;
// assert(getPhase_ph(state.p, state.h)==1, "Function for isothermal compressibility is only valid for one-phase regime!");
          annotation(
            Documentation(info = "<html>
<p> This function calculates the isothermal compressibility of R134a from the state record (e.g., use setState_phX function for input). The isothermal compressibility is modelled by the fundamental equation of state of Tillner-Roth and Baehr (1994). </p>
<h3> Restrictions</h3>
<p>This property is only defined in one-phase region.
</p>
</html>"));
        end isothermalCompressibility;

        redeclare function extends isobaricExpansionCoefficient "Isobaric expansion coefficient w.r.t. thermodynamic state (only valid for one-phase)"
          protected
            Modelica.Media.Common.HelmholtzDerivs f "Helmholtz derivatives";

          algorithm
            if getPhase_ph(state.p, state.h) == 2 then
              beta := 0;
            else
              f := f_R134a(state.d, state.T);
              beta := R134aData.data.R * state.d * f.delta * (f.fdelta - f.tau * f.fdeltatau) * isothermalCompressibility(state);
            end if;
// assert(getPhase_ph(state.p, state.h)==1, "Function for isobaric expansion coefficient is only valid for one-phase regime!");
          annotation(
            Documentation(info = "<html>
<p> This function calculates the isobaric expansion coefficient of R134a from the state record (e.g., use setState_phX function for input). The isobaric expansion coefficient is modelled by the fundamental equation of state of Tillner-Roth and Baehr (1994). </p>
<h3> Restrictions</h3>
<p>This property is only defined in one-phase region.
</p>
</html>"));
        end isobaricExpansionCoefficient;

        redeclare function extends isentropicExponent "Isentropic exponent gamma w.r.t. thermodynamic state | not defined in two-phase region | use setState_phX function for input"
          algorithm
            gamma := density(state) / (if pressure(state) > 1e-6 then pressure(state) else 1e-6) * velocityOfSound(state) ^ 2;
          annotation(
            Inline = true,
            Documentation(info = "<html>
<p> This function calculates the isentropic exponent of R134a from the state record (e.g., use setState_phX function for input). The isentropic exponent is modelled by the fundamental equation of state of Tillner-Roth and Baehr (1994). </p>
<h3> Restrictions</h3>
<p>This property is only defined in one-phase region.
</p>
</html>"));
        end isentropicExponent;

        redeclare function extends specificGibbsEnergy "Specific gibbs energy w.r.t. thermodynamic state"
          algorithm
            g := state.h - state.T * specificEntropy(state);
          annotation(
            Documentation(info = "<html>
<p> This function calculates the specific Gibbs energy of R134a from the state record (e.g., use setState_phX function for input). The isentropic exponent is modelled by the fundamental equation of state of Tillner-Roth and Baehr (1994).
</p>
</html>"));
        end specificGibbsEnergy;

        redeclare function extends specificHelmholtzEnergy "Helmholtz energy w.r.t. thermodynamic state"
          algorithm
            f := state.h - state.p / state.d - state.T * specificEntropy(state);
          annotation(
            Documentation(info = "<html>
<p> This function calculates the specific Helmholtz energy of R134a from the state record (e.g., use setState_phX function for input). The helmholtz energy is modelled by the fundamental equation of state of Tillner-Roth and Baehr (1994).
</p>
</html>"));
        end specificHelmholtzEnergy;

        redeclare function extends density_derh_p "Density derivative by specific enthalpy | use setState_phX function for input"
          protected
            Common.InverseDerivatives_rhoT derivs "Inverse derivatives for density and temperature";

          algorithm
            derivs := derivsOf_ph(state.p, state.h, getPhase_ph(p = state.p, h = state.h));
            ddhp := if getPhase_ph(p = state.p, h = state.h) == 2 then -state.d * state.d / (derivs.dpT * state.T) else -state.d * state.d * derivs.pt / (state.d * state.d * derivs.pd * derivs.cv + state.T * derivs.pt ^ 2);
          annotation(
            Documentation(info = "<html>
<p> This function calculates the density derivative w.r.t. specific enthalpy at constant pressure of R134a (e.g., use setState_phX function for input). The derivative is modelled by the fundamental equation of state of Tillner-Roth and Baehr (1994). It can be used for manual state transformations (e.g. from density to specific enthalpy).
</p>
</html>"));
        end density_derh_p;

        redeclare function extends density_derp_h "Density derivative by pressure | use setState_phX function for input"
          protected
            Common.InverseDerivatives_rhoT derivs "Inverse derivatives for density and temperature";

          algorithm
            derivs := derivsOf_ph(state.p, state.h, getPhase_ph(p = state.p, h = state.h));
            ddph := if getPhase_ph(p = state.p, h = state.h) == 2 then state.d * (state.d * derivs.cv / derivs.dpT + 1) / (derivs.dpT * state.T) else state.d * (derivs.cv * state.d + derivs.pt) / (state.d * state.d * derivs.pd * derivs.cv + state.T * derivs.pt ^ 2);
          annotation(
            Documentation(info = "<html>
<p> This function calculates the density derivative w.r.t. absolute pressure at constant specific enthalpy of R134a (e.g., use setState_phX function for input). The derivative is modelled by the fundamental equation of state of Tillner-Roth and Baehr (1994). It can be used for manual state transformations (e.g. from density to pressure).
</p>
</html>"));
        end density_derp_h;

        redeclare function extends isentropicEnthalpy "Isentropic enthalpy of downstream pressure and upstream thermodynamic state (specific entropy)"
          algorithm
            h_is := specificEnthalpy_psX(p_downstream, specificEntropy(refState), reference_X);
          annotation(
            Documentation(info = "<html>
<p>
This function calculates the specific enthalpy of R134a for an isentropic pressure change
from refState.p to p_downstream (e.g., use setState_phX function for input of refState).
</p>

<p>
The function can be used for instance to calculate an isentropic efficiency
of a compressor or calculate the power consumption (obtained from the isentropic enthalpy)
for a given efficiency.
</p>

<p>
Example:
</p>
<pre>
     Medium.AbsolutePressure p_downstream=10e5;
     Medium.SpecificEnthalpy h_downstream=4.1e5;
     Medium.AbsolutePressure p_upstream=3e5;
     Medium.SpecificEnthalpy h_upstream=4.0e5;

     // Isentropic efficiency of a compressor:
     Real eta_is;

    equation

     h_is = isentropicEnthalpy(p_downstream, Medium.setState_phX(p_upstream, h_upstream));

     eta_is = (h_is-h_upstream)/(h_downstream - h_upstream);
</pre>

<h4>Restrictions</h4>
<p>
The isentropic efficiency function should not be applied in liquid region.
</p>
</html>"));
        end isentropicEnthalpy;

        function derivsOf_ph "Derivatives required for inversion of temperature and density functions"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SpecificEnthalpy h "Specific enthalpy";
          input Integer phase "Number of phases";
          output Common.InverseDerivatives_rhoT derivs "Inverse derivatives for density and temperature";
        protected
          Modelica.Media.Common.HelmholtzDerivs f "Helmholtz derivatives";
          SaturationProperties sat "Saturation temperature and pressure";
          Common.PhaseBoundaryProperties liq "Properties on liquid phase boundary";
          Common.PhaseBoundaryProperties vap "Properties on vapor phase boundary";
          Modelica.Media.Common.NewtonDerivatives_ph newder "Newton derivatives";
          Real x "Vapor quality";
          Real dxv "1/(v_vap - v_liq)";
        algorithm
          derivs.p := p;
          derivs.h := h;
          derivs.phase := getPhase_ph(p, h);
          sat.psat := p;
          sat.Tsat := saturationTemperature(p);
          (derivs.rho, derivs.T) := dt_ph(p = p, h = h);
          if derivs.phase == 2 then
            liq := R134a_liqofdT(T = derivs.T);
            vap := R134a_vapofdT(T = derivs.T);
            x := if liq.h <> vap.h then (h - liq.h) / (vap.h - liq.h) else if h >= vap.h then 1 else 0;
            derivs.cv := Common.cv2Phase(liq = liq, vap = vap, x = x, p = p, T = derivs.T);
            dxv := if liq.d <> vap.d then liq.d * vap.d / (liq.d - vap.d) else 0.0;
            derivs.dpT := (vap.s - liq.s) * dxv;
          else
            f := f_R134a(d = derivs.rho, T = derivs.T);
            f.d := derivs.rho;
            f.T := derivs.T;
            f.R := R134aData.R;
            newder := Modelica.Media.Common.Helmholtz_ph(f = f);
            derivs.cv := -f.R * (f.tau * f.tau * f.ftautau);
            derivs.pd := newder.pd;
            derivs.pt := newder.pt;
          end if;
//derivs.dpT := 0;
          annotation(
            Documentation(info = "<html>
<p> This function calculates the derivatives required for an inversion of temperature and density function.
</p>
</html>"));
        end derivsOf_ph;

        function dt_ph "Density and temperature w.r.t. pressure and specific enthalpy"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SpecificEnthalpy h "Specific enthalpy";
          output Density d "Density";
          output Temperature T "Temperature";
        protected
          SaturationProperties sat(psat = p, Tsat = 0) "Saturation temperature and pressure";
          Modelica.SIunits.Pressure delp = 1.0e-2 "Relative error in p in iteration";
          Modelica.SIunits.SpecificEnthalpy delh = 1.0e-2 "Relative error in h in iteration";
          Modelica.SIunits.SpecificEnthalpy hvapor = dewEnthalpy(sat = sat) "Vapor enthalpy";
          Modelica.SIunits.SpecificEnthalpy hliquid = bubbleEnthalpy(sat = sat) "Liquid enthalpy";
          Integer error "Iteration error";
          Real x "Vapor quality";
          Real vvap "Specific volume vapor";
          Real vliq "Specific volume liquid";
        algorithm
          if h < hliquid or h > hvapor or p > R134aData.data.FPCRIT then
            (d, T, error) := Modelica.Media.R134a.R134a_ph.dtofphOnePhase(p, h, delp, delh);
          else
            vvap := 1 / dewDensity(sat = sat);
            vliq := 1 / bubbleDensity(sat = sat);
            x := (h - hliquid) / (hvapor - hliquid);
            error := 0;
            T := saturationTemperature(p);
            d := 1 / (vliq + x * (vvap - vliq));
          end if;
          annotation(
            Documentation(info = "<html>
<p> This function calculates the density and temperature of R134a from absolute pressure and specific enthalpy. In one-phase region the function calls the fundamental helmholtz equation of Tillner-Roth (1994). In two-phase the density and temperature is computed from cubic splines for saturated pressure, liquid and vapor density.
</p>
<h4>Restrictions</h4>
The function cannot be inverted in a numerical way. Please use functions <a href=\"modelica://Modelica.Media.R134a.R134a_ph.rho_props_ph\">rho_props_ph</a> and <a href=\"modelica://Modelica.Media.R134a.R134a_ph.T_props_ph\">T_props_ph</a> for this purpose.
</html>"));
        end dt_ph;

        function dtofphOnePhase "Density and temperature w.r.t. pressure and specific enthalpy in one-phase region"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SpecificEnthalpy h "Enthalpy";
          input AbsolutePressure delp "Absolute error in p in iteration";
          input SpecificEnthalpy delh "Absolute error in h in iteration";
          output Density d "Density";
          output Temperature T "Temperature";
          output Integer error "1 if did not converged";
        protected
          constant Real p_breaks[:] = R134aData.pbreaks "Grid points of reduced pressure";
          constant Real hl_coef[:, 4] = R134aData.hlcoef "Coefficients of cubic spline for h_liq(p)";
          constant Real dl_coef[:, 4] = R134aData.dlcoef "Coefficients of cubic spline for rho_liq(p)";
          constant Real T_coef[:, 4] = R134aData.Tcoef "Coefficients of cubic spline for Tsat(p)";
          constant Real dv_coef[:, 4] = R134aData.dvcoef "Coefficients of cubic spline for rho_vap(p)";
          Modelica.SIunits.SpecificEnthalpy hl "Liquid enthalpy";
          Boolean liquid "Is liquid";
          Boolean supercritical "Is supercritcal";
          Integer int "Interval number";
          Real pred "Reduced pressure";
          Real localx "Oordinate of local spline";
          Integer i "Newton iteration number";
          Real dh "Difference in h";
          Real dp "Difference in p";
          Real damping "Damping constant";
          Real det "Determinante";
          Real deld "Density change";
          Real delt "Temperature change";
          Modelica.Media.Common.HelmholtzDerivs f "Helmholtz derivatives";
          Modelica.Media.Common.NewtonDerivatives_ph nDerivs "Newton derivatives";
          Boolean found "Iteration converged";
        algorithm
          i := 0;
          error := 0;
          found := false;
          damping := 1.0;
//by default no damping
          pred := p / R134aData.data.FPCRIT;
          (int, error) := Common.FindInterval(pred, p_breaks);
          localx := pred - p_breaks[int];
// set decent initial guesses for d and T
          supercritical := p > R134aData.data.FPCRIT;
          if supercritical then
            d := R134aData.data.FDCRIT * 3.0;
            T := 270.0 + 120.0 * (h / R134aData.data.HCRIT);
          else
            liquid := h < 1.01 * R134aData.data.HCRIT * Common.CubicSplineEval(localx, hl_coef[int, 1:4]);
            if liquid then
              damping := 0.3;
              d := R134aData.data.FDCRIT * Common.CubicSplineEval(localx, dl_coef[int, 1:4]) * 1.02;
              T := Common.CubicSplineEval(localx, T_coef[int, 1:4]);
            else
              d := R134aData.data.FDCRIT * Common.CubicSplineEval(localx, dv_coef[int, 1:4]) * 0.95;
              T := Common.CubicSplineEval(localx, T_coef[int, 1:4]);
            end if;
          end if;
// iteration seems to work better if coming from high densities
//      dTR.T := data.TCRIT + 10.0;
// linear approximation that can be made generic
//damping on the liquid side
// the Newton iteration
          while i < 50 and not found loop
            f := f_R134a(d, T);
            nDerivs := Modelica.Media.Common.Helmholtz_ph(f);
            dh := nDerivs.h - h;
            dp := nDerivs.p - p;
            if abs(dh) <= delh and abs(dp) <= delp then
              found := true;
              error := if not found then 1 else 0;
            end if;
            det := nDerivs.ht * nDerivs.pd - nDerivs.pt * nDerivs.hd;
            if abs(det) < 1e-10 then
              det := sign(det) * 1e-10;
            end if;
            delt := (nDerivs.pd * dh - nDerivs.hd * dp) / det;
            deld := (nDerivs.ht * dp - nDerivs.pt * dh) / det;
            T := T - damping * delt;
            d := d - damping * deld;
            if T < R134aData.triple.TTRIPLE then
              T := R134aData.triple.TTRIPLE + 0.1;
            end if;
            if d <= 0.0 then
              d := R134aData.triple.DVTRIPLE;
            elseif d > 1.5 * R134aData.triple.DLTRIPLE then
              d := R134aData.triple.DLTRIPLE;
            end if;
            i := i + 1;
          end while;
// nDerivs are the symbolic derivatives needed in the iteration
// for given d and T
// Check for limit of state variables
          annotation(
            Documentation(info = "<html>
<p> This function calculates the density and temperature of R134a from absolute pressure and specific enthalpy in one-phase region. The function calls the fundamental helmholtz equation of Tillner-Roth (1994) which is requiring density and temperature for input. Thus, a newton iteration is performed to determine density and temperature. The newton iteration stops if the inputs for pressure difference delp and specific enthalpy difference delh are larger than the actual differences derived from the newton iteration.
</p>
<h4>Restrictions</h4>
The function shall only be used for one-phase inputs since the fundamental equation is not valid for two-phase states.
</html>"));
        end dtofphOnePhase;

        function dtofpsOnePhase "Inverse iteration in one phase region (d,T) = f(p,s)"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SpecificEntropy s "Specific entropy";
          input AbsolutePressure delp "Absolute iteration accuracy";
          input SpecificEntropy dels "Absolute iteration accuracy";
          output Density d "Density";
          output Temperature T "Temperature";
          output Integer error "Error flag: trouble if different from 0";
        protected
          constant Real p_breaks[:] = R134aData.pbreaks "Grid points of reduced pressure";
          constant Real dl_coef[:, 4] = R134aData.dlcoef "Coefficients of cubic spline for rho_liq(p)";
          constant Real dv_coef[:, 4] = R134aData.dvcoef "Coefficients of cubic spline for rho_vap(p)";
          constant Real sl_coef[:, 4] = R134aData.slcoef "Coefficients of cubic spline for s_liq(p)";
          constant Real sv_coef[:, 4] = R134aData.svcoef "Coefficients of cubic spline for s_vap(p)";
          constant Real T_coef[:, 4] = R134aData.Tcoef "Coefficients of cubic spline for Tsat(p)";
          Integer i "Newton iteration number";
          Real ds "Difference in s";
          Real dp "Difference in p";
          Real det "Determinante";
          Real deld "Density change";
          Real delt "Temperature change";
          Modelica.Media.Common.HelmholtzDerivs f "Dimensionless Helmholtz function and dervatives w.r.t. dimensionless d and T";
          Modelica.Media.Common.NewtonDerivatives_ps nDerivs "Newton derivatives";
          Boolean liquid "Is liquid";
          Boolean supercritical "Is supercritcal";
          Boolean found "Iteration converged";
          Integer int "Interval number";
          Real pred "Reduced pressure";
          Real localx "Oordinate of local spline";
          Real sl "Liquid entropy";
          Real sv "Vapor entropy";
        algorithm
          i := 0;
          error := 0;
          found := false;
          pred := p / R134aData.data.FPCRIT;
          (int, error) := Common.FindInterval(pred, p_breaks);
          localx := pred - p_breaks[int];
// set decent initial guesses for d and T
          supercritical := p > R134aData.data.FPCRIT;
          if supercritical then
            d := R134aData.data.FDCRIT + 500.0;
            T := R134aData.data.FTCRIT + 10.0;
          else
            sl := R134aData.data.SCRIT * Common.CubicSplineEval(localx, sl_coef[int, 1:4]);
            sv := R134aData.data.SCRIT * Common.CubicSplineEval(localx, sv_coef[int, 1:4]);
            liquid := s < sl;
            if liquid then
              d := R134aData.data.FDCRIT * Common.CubicSplineEval(localx, dl_coef[int, 1:4]) * 1.1;
              T := Common.CubicSplineEval(localx, T_coef[int, 1:4]) * 0.9;
            else
              d := R134aData.data.FDCRIT * Common.CubicSplineEval(localx, dv_coef[int, 1:4]) * 0.8;
              T := Common.CubicSplineEval(localx, T_coef[int, 1:4]) * 1.1 + 0.5 * (s - sv);
            end if;
          end if;
//1.05
//0.95
//0.9
          while i < 100 and not found loop
            f := f_R134a(d = d, T = T);
            nDerivs := Modelica.Media.Common.Helmholtz_ps(f);
            ds := nDerivs.s - s;
            dp := nDerivs.p - p;
            if abs(ds / s) <= dels and abs(dp / p) <= delp then
              found := true;
            end if;
            det := nDerivs.st * nDerivs.pd - nDerivs.pt * nDerivs.sd;
            delt := (nDerivs.pd * ds - nDerivs.sd * dp) / det;
            deld := (nDerivs.st * dp - nDerivs.pt * ds) / det;
            T := T - delt;
            d := d - deld;
            i := i + 1;
          end while;
          if not found then
            error := 1;
          end if;
          annotation(
            Documentation(info = "<html>
<p> This function calculates the density and temperature of R134a from absolute pressure and specific entropy in one-phase region. The function calls the fundamental helmholtz equation of Tillner-Roth (1994) which is requiring density and temperature for input. Thus, a newton iteration is performed to determine density and temperature. The newton iteration stops if the inputs for pressure difference delp and specific entropy difference dels are larger than the actual differences derived from the newton iteration.
</p>
<h4>Restrictions</h4>
The function shall only be used for one-phase inputs since the fundamental equation is not valid for two-phase states. The iteration could fail for liquid states with high pressures.
</html>"));
        end dtofpsOnePhase;

        function f_R134a "Calculation of helmholtz derivatives by density and temperature"
          extends Modelica.Icons.Function;
          input Density d "Density";
          input Temperature T "Temperature";
          output Modelica.Media.Common.HelmholtzDerivs f "Helmholtz derivatives";
        protected
          Real delta "Reduced density";
          Real tau "Reduced temperature";
          Modelica.Media.Common.HelmholtzDerivs fid "Helmholtz derivstives";
        algorithm
          delta := d / R134aData.data.DCRIT;
          tau := R134aData.data.TCRIT / T;
          fid := fid_R134a(delta = delta, tau = tau);
          f := fres_R134a(delta = delta, tau = tau);
          f.f := f.f + fid.f;
          f.fdelta := f.fdelta + fid.fdelta;
          f.fdeltadelta := f.fdeltadelta + fid.fdeltadelta;
          f.ftau := f.ftau + fid.ftau;
          f.ftautau := f.ftautau + fid.ftautau;
          f.d := d;
          f.T := T;
          f.R := R134aData.R;
          annotation(
            Documentation(info = "<html>
This function adds the ideal gas contribution of the fundamental equation to the residual contribution and computes the helmholtz derivatives w.r.t. temperature and density.
</html>"));
        end f_R134a;

        function fid_R134a "Helmholtz coefficients of ideal part"
          extends Modelica.Icons.Function;
          input Real delta "Reduced density (delta=d/dcrit)";
          input Real tau "Reduced temperature (tau=Tcrit/T)";
          output Modelica.Media.Common.HelmholtzDerivs fid "Helmholtz derivatives of ideal part";
        protected
          Modelica.Media.R134a.R134aData.Ideal id "Ideal coeffcients";
          Real atau = abs(tau) "|tau|";
          Real adelta = abs(delta) "|delta|";
        algorithm
          fid.delta := delta;
          fid.tau := tau;
          fid.f := Modelica.Math.log(adelta) + id.a[1] + id.a[2] * atau + id.a[3] * Modelica.Math.log(atau) + id.a[4] * atau ^ (-0.5) + id.a[5] * atau ^ (-0.75);
          fid.fdelta := 1.0 / adelta;
          fid.fdeltadelta := -1.0 / (adelta * adelta);
          fid.ftau := id.a[2] + id.a[3] / atau - 0.5 * id.a[4] * atau ^ (-1.5) - 0.75 * id.a[5] * atau ^ (-1.75);
          fid.ftautau := (-id.a[3] / (atau * atau)) + 0.75 * id.a[4] * atau ^ (-2.5) + 1.3125 * id.a[5] * atau ^ (-2.75);
          fid.fdeltatau := 0.0;
          fid.d := fid.delta * R134aData.data.DCRIT;
          fid.T := R134aData.data.TCRIT / fid.tau;
          fid.R := R134aData.R;
          annotation(
            Documentation(info = "<html>
This function computes the ideal gas helmholtz derivatives of the fundamental equation of Tillner-Roth and Baehr for R134a (1994) w.r.t. to reduced temperature (tau=T_crit/T) and reduced density (delta=rho/rho_crit).
</html>"));
        end fid_R134a;

        function fres_R134a "Calculation of helmholtz derivatives"
          extends Modelica.Icons.Function;
          input Real delta "Reduced density (delta=d/dcrit)";
          input Real tau "Reduced temperature (tau=Tcrit/T)";
          output Modelica.Media.Common.HelmholtzDerivs f "Helmholtz derivatives";
        protected
          Modelica.Media.R134a.R134aData.Residual res "Residual coefficient";
          Real k "Helping var";
          Real dc "Helping var";
          Real cdc "Helping var";
        algorithm
          f.tau := abs(tau);
          f.delta := abs(delta);
          f.f := 0.0;
          f.ftau := 0.0;
          f.fdelta := 0.0;
          f.ftautau := 0.0;
          f.fdeltadelta := 0.0;
          f.fdeltatau := 0.0;
          for i in 1:res.ns1 loop
            k := res.n[i] * f.delta ^ res.d[i] * f.tau ^ res.t[i];
            f.f := f.f + k;
            f.fdelta := f.fdelta + k * res.d[i];
            f.fdeltadelta := f.fdeltadelta + k * res.d[i] * (res.d[i] - 1.0);
            f.ftau := f.ftau + k * res.t[i];
            f.ftautau := f.ftautau + k * res.t[i] * (res.t[i] - 1.0);
            f.fdeltatau := f.fdeltatau + k * res.t[i] * res.d[i];
          end for;
          for i in res.ns1 + 1:21 loop
            dc := f.delta ^ res.c[i];
            cdc := res.c[i] * dc;
            k := Modelica.Math.exp(-dc) * res.n[i] * delta ^ res.d[i] * f.tau ^ res.t[i];
            f.f := f.f + k;
            f.fdelta := f.fdelta + k * (res.d[i] - cdc);
            f.fdeltadelta := f.fdeltadelta + k * ((res.d[i] - cdc) * (res.d[i] - 1.0 - cdc) - res.c[i] * cdc);
            f.ftau := f.ftau + k * res.t[i];
            f.ftautau := f.ftautau + k * res.t[i] * (res.t[i] - 1.0);
            f.fdeltatau := f.fdeltatau + k * res.t[i] * (res.d[i] - cdc);
          end for;
          f.fdelta := f.fdelta / delta;
          f.fdeltadelta := f.fdeltadelta / (delta * delta);
          f.ftau := f.ftau / tau;
          f.ftautau := f.ftautau / (tau * tau);
          f.fdeltatau := f.fdeltatau / (delta * tau);
          f.d := f.delta * R134aData.data.DCRIT;
          f.T := R134aData.data.TCRIT / f.tau;
          f.R := R134aData.R;
          annotation(
            Documentation(info = "<html>
This function computes the residual helmholtz derivatives of the fundamental equation of Tillner-Roth and Baehr for R134a (1994) w.r.t. to reduced temperature (tau=T_crit/T) and reduced density (delta=rho/rho_crit). The function can be used for special properties depending just on the residual derivative parts.
</html>"));
        end fres_R134a;

        function getPhase_ph "Number of phases by pressure and specific enthalpy"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SpecificEnthalpy h "Specific enthalpy";
          output Integer phase "Number of phases";
        protected
          SaturationProperties sat(psat = p, Tsat = 0) "Saturation temperature and pressure";
          Modelica.SIunits.SpecificEnthalpy hl = bubbleEnthalpy(sat) "Liquid enthalpy";
          Modelica.SIunits.SpecificEnthalpy hv = dewEnthalpy(sat) "Vapor enthalpy";
        algorithm
          phase := if h < hl or h > hv or p > R134aData.data.FPCRIT then 1 else 2;
          annotation(
            Documentation(info = "<html>
This function computes the number of phases for R134a depending on the inputs for absolute pressure and specific enthalpy. It makes use of cubic spline functions for liquid and vapor specific enthalpy.
</html>"));
        end getPhase_ph;

        function getPhase_ps "Number of phases by pressure and entropy"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SpecificEntropy s "Specific entropy";
          output Integer phase "Number of phases";
        protected
          SaturationProperties sat(psat = p, Tsat = 0) "Saturation temperature and pressure";
          Modelica.SIunits.SpecificEntropy sl = bubbleEntropy(sat) "Liquid entropy";
          Modelica.SIunits.SpecificEntropy sv = dewEntropy(sat) "Vapor entropy";
        algorithm
          phase := if s < sl or s > sv or p > R134aData.data.FPCRIT then 1 else 2;
          annotation(
            Documentation(info = "<html>
This function computes the number of phases for R134a depending on the inputs for absolute pressure and specific entropy. It makes use of cubic spline functions for liquid and vapor specific entropy.
</html>"));
        end getPhase_ps;

        function hofpsTwoPhase "Isentropic specific enthalpy in two phase region h(p,s)"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SpecificEntropy s "Specific entropy";
          output SpecificEnthalpy h "Specific enthalpy";
        protected
          SaturationProperties sat "Saturation temperature and pressure";
          Modelica.SIunits.MassFraction x "Vapor quality";
          Modelica.SIunits.SpecificEntropy sl "Liquid entropy";
          Modelica.SIunits.SpecificEntropy sv "Vapor entropy";
          Modelica.SIunits.SpecificEnthalpy hl "Liquid enthalpy";
          Modelica.SIunits.SpecificEnthalpy hv "Vapor enthalpy";
        algorithm
          sat.psat := p;
// dummy
          sat.Tsat := 0;
          hl := bubbleEnthalpy(sat = sat);
          hv := dewEnthalpy(sat = sat);
          sl := bubbleEntropy(sat = sat);
          sv := dewEntropy(sat = sat);
          x := (s - sl) / (sv - sl);
          h := hl + x * (hv - hl);
          annotation(
            Documentation(info = "<html>
This function computes the specific enthalpy in two-phase for R134a depending on the inputs for absolute pressure and specific entropy. It makes use of cubic spline functions for liquid and vapor specific enthalpy as well as specific entropy.
</html>"));
        end hofpsTwoPhase;

        function R134a_liqofdT "Properties on liquid boundary phase"
          extends Modelica.Icons.Function;
          input Temperature T "Temperature";
          output Common.PhaseBoundaryProperties liq "Properties on liquid boundary phase";
        protected
          Modelica.SIunits.Temperature T_liq "Liquid temperature";
          Modelica.SIunits.Density d_liq "Liquid density";
          Modelica.Media.Common.HelmholtzDerivs f "Helmholtz derivatives";
        algorithm
          if T < R134aData.data.TCRIT then
            d_liq := bubbleDensity(setSat_T(T));
            T_liq := T;
          else
            d_liq := R134aData.data.DCRIT;
            T_liq := R134aData.data.TCRIT;
          end if;
          f := f_R134a(d_liq, T_liq);
          liq := Common.helmholtzToBoundaryProps(f);
        end R134a_liqofdT;

        function R134a_vapofdT "Properties on vapor boundary phase"
          extends Modelica.Icons.Function;
          input Temperature T "Temperature";
          output Common.PhaseBoundaryProperties vap "Properties on vapor boundary phase";
        protected
          Modelica.SIunits.Temperature T_vap "Vapor temperature";
          Modelica.SIunits.Density d_vap "Vapor density";
          Modelica.Media.Common.HelmholtzDerivs f "Helmholtz derivatives";
        algorithm
          if T < R134aData.data.TCRIT then
            d_vap := dewDensity(setSat_T(T));
            T_vap := T;
          else
            d_vap := R134aData.data.DCRIT;
            T_vap := R134aData.data.TCRIT;
          end if;
          f := f_R134a(d_vap, T_vap);
          vap := Common.helmholtzToBoundaryProps(f);
        end R134a_vapofdT;

        function rho_ph_der "Time derivative function of density_ph"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SpecificEnthalpy h "Specific enthalpy";
          input Common.InverseDerivatives_rhoT derivs "Record for derivatives";
          input Real p_der "Derivative of pressure";
          input Real h_der "Derivative of specific enthalpy";
          output Real d_der "Derivative of density";
        algorithm
          if derivs.phase == 2 then
            d_der := derivs.rho * (derivs.rho * derivs.cv / derivs.dpT + 1.0) / (derivs.dpT * derivs.T) * p_der + (-derivs.rho * derivs.rho / (derivs.dpT * derivs.T)) * h_der;
          else
            d_der := derivs.rho * (derivs.cv * derivs.rho + derivs.pt) / (derivs.rho * derivs.rho * derivs.pd * derivs.cv + derivs.T * derivs.pt * derivs.pt) * p_der + (-derivs.rho * derivs.rho * derivs.pt / (derivs.rho * derivs.rho * derivs.pd * derivs.cv + derivs.T * derivs.pt * derivs.pt)) * h_der;
          end if;
          annotation(
            Documentation(info = "<html>
This function calculates the derivative of density w.r.t. time. It is used as derivative function for <a href=\"modelica://Modelica.Media.R134a.R134a_ph.rho_props_ph\"> rho_props_ph</a>.
</html>"));
        end rho_ph_der;

        function rho_props_ph "Density as function of pressure and specific enthalpy"
          extends Modelica.Icons.Function;
          input Modelica.SIunits.Pressure p "Pressure";
          input Modelica.SIunits.SpecificEnthalpy h "Specific enthalpy";
          input Common.InverseDerivatives_rhoT derivs "Record for the calculation of rho_ph_der";
          output Modelica.SIunits.Density d "Density";
        algorithm
          d := derivs.rho;
          annotation(
            derivative(noDerivative = derivs) = rho_ph_der,
            Inline = false,
            LateInline = true,
            Documentation(info = "<html>
This function integrates the derivative of density w.r.t. time in order to allow a numerical inversion for the complex fundamental equation of state.
</html>"));
        end rho_props_ph;

        function T_ph_der "Time derivative function of T_ph"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SpecificEnthalpy h "Specific enthalpy";
          input Common.InverseDerivatives_rhoT derivs "Auxiliary record";
          input Real p_der "Derivative of pressure";
          input Real h_der "Derivative of specific enthalpy";
          output Real T_der "Derivative of temperature";
        algorithm
          if derivs.phase == 2 then
            T_der := 1 / derivs.dpT * p_der;
          else
            T_der := ((-derivs.rho * derivs.pd) + derivs.T * derivs.pt) / (derivs.rho * derivs.rho * derivs.pd * derivs.cv + derivs.T * derivs.pt * derivs.pt) * p_der + derivs.rho * derivs.rho * derivs.pd / (derivs.rho * derivs.rho * derivs.pd * derivs.cv + derivs.T * derivs.pt * derivs.pt) * h_der;
          end if;
          annotation(
            Documentation(info = "<html>
This function calculates the derivative of temperature w.r.t. time. It is used as derivative function for <a href=\"modelica://Modelica.Media.R134a.R134a_ph.T_props_ph\"> T_props_ph</a>.
</html>"));
        end T_ph_der;

        function T_props_ph "Temperature as function of pressure and specific enthalpy"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SpecificEnthalpy h "Specific enthalpy";
          input Common.InverseDerivatives_rhoT derivs "Record for the calculation of T_ph_der";
          output Temperature T "Temperature";
        algorithm
          T := derivs.T;
          annotation(
            derivative(noDerivative = derivs) = T_ph_der,
            Inline = false,
            LateInline = true,
            Documentation(info = "<html>
This function integrates the derivative of temperature w.r.t. time in order to allow a numerical inversion for the complex fundamental equation of state.
</html>"));
        end T_props_ph;

        redeclare function extends setSmoothState "Smooth transition function between state_a and state_b"
            import Modelica.Media.Common.smoothStep;

          algorithm
            state := ThermodynamicState(p = smoothStep(x, state_a.p, state_b.p, x_small), h = smoothStep(x, state_a.h, state_b.h, x_small), T = temperature_ph(smoothStep(x, state_a.p, state_b.p, x_small), smoothStep(x, state_a.h, state_b.h, x_small)), d = density_ph(smoothStep(x, state_a.p, state_b.p, x_small), smoothStep(x, state_a.h, state_b.h, x_small)), phase = 0);
          annotation(
            Inline = true);
        end setSmoothState;
        annotation(
          Documentation(info = "<html>
<p>
Calculation of fluid properties for Tetrafluoroethane (R134a) in the fluid region of 0.0039 bar (Triple pressure) to 700 bar and 169.85 Kelvin (Triple temperature) to 455 Kelvin.
</p>
<h4>Restriction</h4>
<p>
The functions provided by this package shall be used inside of the restricted limits according to the referenced literature.
</p>
<ul>
 <li>
      <b> 0.0039 bar &le; p &le; 700 bar </b>
 </li>
 <li>
      <b> 169.85 Kelvin &le; T &le; 455 Kelvin  </b>
 </li>
 <li>
      <b> explicit for pressure and specific enthalpy </b>
 </li>
</ul>

<p><b>References</b> </p>
<dl><dt>Baehr, H.D. and Tillner-Roth, R.: </dt>
<dd><b>Thermodynamic Properties of Environmentally Acceptable Refrigerants -
Equations of State and Tables for Ammonia, R22, R134a, R152a, and R123</b>. Springer-Verlag, Berlin (Germany), 1994.</dd>
</dl>
<dl><dt>Klein, McLinden and Laesecke: </dt>
<dd><b>An improved extended corresponding states methode for estimation of visocsity of pure refrigerants and mixtures</b>.
Int. J. Refrig., Vol. 20, No.3, pp. 208-217, 1997.</dd>
</dl>
<dl><dt>McLinden, Klein. and Perkins: </dt>
<dd><b>An extended corresponding states model for the thermal conductivity
of refrigerants and refrigerant mixtures</b>.
Int. J. Refrig., 23 (2000) 43-63.</dd>
</dl>
<dl><dt>Okada and Higashi: </dt>
<dd><b>Surface tension correlation of HFC-134a and HCFC-123</b>.
Proceedings of the Joint Meeting of IIR Commissions B1, B2, E1, and E2, Padua, Italy, pp. 541-548, 1994.</dd>
</dl>
</html>"));
      end R134a_ph;

      package R134aData "R134a data required by package R134a_ph"
        import Modelica.SIunits;
        extends Modelica.Icons.Package;
        constant SIunits.SpecificHeatCapacity R = data.R;
        constant Integer Npoints = 478;
        constant Integer Ninterval = Npoints - 1;
        constant Real[Npoints] pbreaks = {9.597762848258994e-005, 9.869357747732370e-005, 1.010264375639950e-004, 1.065169584583328e-004, 1.123059413359126e-004, 1.184094248412308e-004, 1.248446933924201e-004, 1.316297064467259e-004, 1.387835295300911e-004, 1.463261079761978e-004, 1.542786111807819e-004, 1.626631846135783e-004, 1.715035062398977e-004, 1.808243694992149e-004, 1.906517107583061e-004, 2.010132485287069e-004, 2.119377912202213e-004, 2.234561174702443e-004, 2.356004528862385e-004, 2.484047512716141e-004, 2.616284653191377e-004, 2.747881183685521e-004, 2.885158790807569e-004, 3.028318826616158e-004, 3.177568016040576e-004, 3.333118542894548e-004, 3.495188133266056e-004, 3.664000141427236e-004, 3.839783631625294e-004, 4.022773464529003e-004, 4.213210377579724e-004, 4.411341070924892e-004, 4.617418287585031e-004, 4.831700895846934e-004, 5.054453973240350e-004, 5.285948883287491e-004, 5.526463359992557e-004, 5.776281585116855e-004, 6.035694268866237e-004, 6.304998727332601e-004, 6.584498962084673e-004, 6.874505733153568e-004, 7.175336642444060e-004, 7.487316202293097e-004, 7.810775914127821e-004, 8.146054342502127e-004, 8.493497185583605e-004, 8.853457349505343e-004, 9.226295020377572e-004, 9.612377731634476e-004, 1.001208043569667e-003, 1.042578557225471e-003, 1.085388313487843e-003, 1.129677073668393e-003, 1.175485367705820e-003, 1.222854500460934e-003, 1.271826558129384e-003, 1.322444414376414e-003, 1.374751736326610e-003, 1.428792990668195e-003, 1.484613449495073e-003, 1.542259195759381e-003, 1.601777129328165e-003, 1.663214972080042e-003, 1.726621273310258e-003, 1.792045415319891e-003, 1.859537618008464e-003, 1.929148944128722e-003, 2.000931304128618e-003, 2.074937460870742e-003, 2.151221034049469e-003, 2.229836504887513e-003, 2.310839220352569e-003, 2.394285397224690e-003, 2.480232126358037e-003, 2.568737376543922e-003, 2.659859998183566e-003, 2.753659727089886e-003, 2.850197187960054e-003, 2.949533897578886e-003, 3.051732268501956e-003, 3.156855611648015e-003, 3.264968139624009e-003, 3.376134969543659e-003, 3.490422125512034e-003, 3.607896541474759e-003, 3.728626063414313e-003, 3.852679451797347e-003, 3.980126383732962e-003, 4.111037454976533e-003, 4.245484181734639e-003, 4.383539002791262e-003, 4.525275280559336e-003, 4.670767303162268e-003, 4.820090285488170e-003, 4.973320370546478e-003, 5.130534630480654e-003, 5.291811067763014e-003, 5.457228615833621e-003, 5.626867140010384e-003, 5.800807438004561e-003, 5.979131240454509e-003, 6.161921211248477e-003, 6.349260947941199e-003, 6.541234981608406e-003, 6.737928777105548e-003, 6.939428732811780e-003, 7.145822180530093e-003, 7.357197385013741e-003, 7.573643543593235e-003, 7.795250785685322e-003, 8.022110172037484e-003, 8.254313693933356e-003, 8.491954272338212e-003, 8.735125756982369e-003, 8.983922925176295e-003, 9.238441480673905e-003, 9.498778052406051e-003, 9.765030192907764e-003, 1.003729637709093e-002, 1.031567600057533e-002, 1.060026937790312e-002, 1.089117774088576e-002, 1.118850323681089e-002, 1.149234892629966e-002, 1.180281878149178e-002, 1.212001768378737e-002, 1.244405142182634e-002, 1.277502668901607e-002, 1.311305108132637e-002, 1.345823309485172e-002, 1.381068212326552e-002, 1.417050845529529e-002, 1.453782327208102e-002, 1.491273864445383e-002, 1.529536753016332e-002, 1.568582377114743e-002, 1.608422209040166e-002, 1.649067808931178e-002, 1.690530824446280e-002, 1.732822990453687e-002, 1.775956128744187e-002, 1.819942147677864e-002, 1.864793041887473e-002, 1.910520891956503e-002, 1.957137864057372e-002, 2.004656209644468e-002, 2.053088265103482e-002, 2.102446451411387e-002, 2.152743273774831e-002, 2.203991321298626e-002, 2.256203266622321e-002, 2.309391865555178e-002, 2.363569956736602e-002, 2.418750461246241e-002, 2.474946382258750e-002, 2.532170804665175e-002, 2.590436894709933e-002, 2.649757899613318e-002, 2.710147147198215e-002, 2.771618045516845e-002, 2.834184082475825e-002, 2.897858825451354e-002, 2.962655920917527e-002, 3.028589094062469e-002, 3.095672148415131e-002, 3.163918965453092e-002, 3.233343504239619e-002, 3.303959801019879e-002, 3.375781968861326e-002, 3.448824197265431e-002, 3.523100751788461e-002, 3.598625973665024e-002, 3.675414279428406e-002, 3.753480160540696e-002, 3.832838183003045e-002, 3.913502986997648e-002, 3.995489286516166e-002, 4.078811868980567e-002, 4.163485594877166e-002, 4.249525397402870e-002, 4.336946282080187e-002, 4.425763326428060e-002, 4.515991679581355e-002, 4.607646561937220e-002, 4.700743264819789e-002, 4.795297150115203e-002, 4.891323649938621e-002, 4.988838266300229e-002, 5.087856570747507e-002, 5.188394204050219e-002, 5.290466875872184e-002, 5.394090364435762e-002, 5.499280516221097e-002, 5.606053245635714e-002, 5.714424534717350e-002, 5.824410432822266e-002, 5.936027056331945e-002, 6.049290588356860e-002, 6.164217278454836e-002, 6.280823442343543e-002, 6.399125461636106e-002, 6.519139783552948e-002, 6.640882920680599e-002, 6.764371450704471e-002, 6.889622016163087e-002, 7.016651324207315e-002, 7.145476146359577e-002, 7.276113318295722e-002, 7.408579739616779e-002, 7.542892373645665e-002, 7.679068247215846e-002, 7.817124450477091e-002, 7.957078136711258e-002, 8.098946522143942e-002, 8.242746885787376e-002, 8.388496569268521e-002, 8.536212976679605e-002, 8.685913574431177e-002, 8.837615891128831e-002, 8.991337517436447e-002, 9.147096105967481e-002, 9.304909371182504e-002, 9.464795089290083e-002, 9.626771098168203e-002, 9.790855297290042e-002, 9.957065647660095e-002, 1.012542017176483e-001, 1.029593695353304e-001, 1.046863413830815e-001, 1.064352993283296e-001, 1.082064260525101e-001, 1.099999048510082e-001, 1.118159196335355e-001, 1.136546549243387e-001, 1.155162958627491e-001, 1.174010282037917e-001, 1.193090383188843e-001, 1.212405131966888e-001, 1.231956404442420e-001, 1.251746082880320e-001, 1.271776055753336e-001, 1.292048217756560e-001, 1.312564469824424e-001, 1.333326719147161e-001, 1.354336879191573e-001, 1.375596869720684e-001, 1.397108616817379e-001, 1.418874052908662e-001, 1.440895116791160e-001, 1.463173753659727e-001, 1.485711915136508e-001, 1.508511559303043e-001, 1.531574650734013e-001, 1.554903160532023e-001, 1.578499066364259e-001, 1.602364352504390e-001, 1.626501009871427e-001, 1.650911036075191e-001, 1.675596435460891e-001, 1.700559219157807e-001, 1.725801405130011e-001, 1.751325018227866e-001, 1.777132090243812e-001, 1.803224659969848e-001, 1.829604773257499e-001, 1.856274483079854e-001, 1.883235849597385e-001, 1.910490940225762e-001, 1.938041829705809e-001, 1.965890600178195e-001, 1.994039341258457e-001, 2.022490150118090e-001, 2.051245131564867e-001, 2.080306398131010e-001, 2.109676070159534e-001, 2.139356275898521e-001, 2.169349151595292e-001, 2.199656841596673e-001, 2.230281498451127e-001, 2.261225283015071e-001, 2.292490364564299e-001, 2.324078920906696e-001, 2.355993138502352e-001, 2.388235212584440e-001, 2.420807347286962e-001, 2.453711755776573e-001, 2.486950660387031e-001, 2.520526292761376e-001, 2.554440893996220e-001, 2.588696714792741e-001, 2.623296015612792e-001, 2.658241066839430e-001, 2.693534148944132e-001, 2.729177552658184e-001, 2.765173579152221e-001, 2.801524540219758e-001, 2.838232758468616e-001, 2.875300567517182e-001, 2.912730312199488e-001, 2.950524348775986e-001, 2.988685045151655e-001, 3.027214781102109e-001, 3.066115948507304e-001, 3.105390951593055e-001, 3.145042207181398e-001, 3.185072144949388e-001, 3.225483207696795e-001, 3.266277851623652e-001, 3.307458546616600e-001, 3.349027776546629e-001, 3.390988039576249e-001, 3.433341848477569e-001, 3.476091730962973e-001, 3.519240230025824e-001, 3.562789904294610e-001, 3.606743328399952e-001, 3.651103093353738e-001, 3.695871806943437e-001, 3.741052094140343e-001, 3.786646597522729e-001, 3.832657977714583e-001, 3.879088913841363e-001, 3.925942104002511e-001, 3.973220265761317e-001, 4.020926136654290e-001, 4.069062474719831e-001, 4.117632059046955e-001, 4.166637690345744e-001, 4.216082191539507e-001, 4.265968408381747e-001, 4.316299210096456e-001, 4.367077490044181e-001, 4.418306166415828e-001, 4.469988182954611e-001, 4.522126509707460e-001, 4.574724143807979e-001, 4.627784110292650e-001, 4.681309462951913e-001, 4.735303285216456e-001, 4.789768691084685e-001, 4.844708826088187e-001, 4.900126868302581e-001, 4.956026029402373e-001, 5.012409555764841e-001, 5.069280729624649e-001, 5.126642870282773e-001, 5.184499335373062e-001, 5.242853522190180e-001, 5.302680522792256e-001, 5.362148870570488e-001, 5.421125034763555e-001, 5.479604123151721e-001, 5.537581634208009e-001, 5.595053445140967e-001, 5.652015800117042e-001, 5.708465298673534e-001, 5.764398884324107e-001, 5.819813833363990e-001, 5.874707743877503e-001, 5.929078524952066e-001, 5.982924386100538e-001, 6.036243826896056e-001, 6.089035626818212e-001, 6.141298835313674e-001, 6.193032762071714e-001, 6.244236967514051e-001, 6.294911253499476e-001, 6.345055654242815e-001, 6.394670427446655e-001, 6.443756045645810e-001, 6.492313187762216e-001, 6.540342730869820e-001, 6.587845742164332e-001, 6.634823471141452e-001, 6.681277341974906e-001, 6.727208946097391e-001, 6.772620034977512e-001, 6.817512513092869e-001, 6.861888431095735e-001, 6.905749979167365e-001, 6.949099480559251e-001, 6.991939385318118e-001, 7.034272264190118e-001, 7.076100802703395e-001, 7.117427795423493e-001, 7.158256140379710e-001, 7.198588833658458e-001, 7.238428964161435e-001, 7.277779708523336e-001, 7.316644326187892e-001, 7.355026154637049e-001, 7.392928604808265e-001, 7.430355156427725e-001, 7.467309354101930e-001, 7.503794802847638e-001, 7.539815165033564e-001, 7.575374158563575e-001, 7.610475525042479e-001, 7.645123095847116e-001, 7.679320711405823e-001, 7.713072258757592e-001, 7.746381659738916e-001, 7.779252867810481e-001, 7.811689864996086e-001, 7.843696658934009e-001, 7.875277280034720e-001, 7.906435778747776e-001, 7.937176222932615e-001, 7.967502695335140e-001, 8.000199630662463e-001, 8.033009258512434e-001, 8.065932139849678e-001, 8.098968847000179e-001, 8.132119945147386e-001, 8.165386053633329e-001, 8.198767769333629e-001, 8.232265712101035e-001, 8.265880515436206e-001, 8.299612827168834e-001, 8.333463310190913e-001, 8.367432643245871e-001, 8.401521521779828e-001, 8.435730658861569e-001, 8.470060786178580e-001, 8.504512655117438e-001, 8.539087037939039e-001, 8.573784729060409e-001, 8.608606546457506e-001, 8.643553333207711e-001, 8.678625959196853e-001, 8.713825323024099e-001, 8.749152354154032e-001, 8.784608015388341e-001, 8.820193305769569e-001, 8.855909230615221e-001, 8.891756908959754e-001, 8.927737444522241e-001, 8.963852006600686e-001, 9.000101817311734e-001, 9.036488156539342e-001, 9.073012367571003e-001, 9.109675863549037e-001, 9.146480134897557e-001, 9.183426757925104e-001, 9.220517404865118e-001, 9.257753855697142e-001, 9.295138012239329e-001, 9.332671915249889e-001, 9.370357765779688e-001, 9.408197916132383e-001, 9.446194993887446e-001, 9.484351827283455e-001, 9.522671554710838e-001, 9.561157672135285e-001, 9.599814113076552e-001, 9.638645356406839e-001, 9.677656576771091e-001, 9.716853843570205e-001, 9.756244470156604e-001, 9.795837477900006e-001, 9.825671883209557e-001, 9.855632858038894e-001, 9.886965153922347e-001, 9.917147058660856e-001, 9.939889544462242e-001, 9.955107773467575e-001, 9.966554757188424e-001, 9.976501440012482e-001, 9.985707411825207e-001, 9.989551169812724e-001, 9.991090113187927e-001, 9.993400213037331e-001, 9.995459937617709e-001, 9.997521475624808e-001, 9.998552980391523e-001, 9.999068928446231e-001, 1.0};
        constant Real[Npoints] Tbreaks = {0.453892958213191, 0.454558816062682, 0.455118075727735, 0.456391027473952, 0.457672503868159, 0.458962558356426, 0.460261324553926, 0.461568882629760, 0.462885339476068, 0.464210775261953, 0.465545296879551, 0.466888984497968, 0.468241971732374, 0.469604365474908, 0.470976245894674, 0.472357746606843, 0.473748947780518, 0.475149983030873, 0.476560959250045, 0.477981983330171, 0.479384515070455, 0.480720666793156, 0.482056818515856, 0.483392970238556, 0.484729121961257, 0.486065273683957, 0.487401425406658, 0.488737577129358, 0.490073728852059, 0.491409880574759, 0.492746032297459, 0.494082184020160, 0.495418335742860, 0.496754487465561, 0.498090639188261, 0.499426790910961, 0.500762942633662, 0.502099094356362, 0.503435246079063, 0.504771397801763, 0.506107549524464, 0.507443701247164, 0.508779852969864, 0.510116004692565, 0.511452156415265, 0.512788308137966, 0.514124459860666, 0.515460611583366, 0.516796763306067, 0.518132915028767, 0.519469066751468, 0.520805218474168, 0.522141370196869, 0.523477521919569, 0.524813673642269, 0.526149825364970, 0.527485977087670, 0.528822128810371, 0.530158280533071, 0.531494432255771, 0.532830583978472, 0.534166735701172, 0.535502887423873, 0.536839039146573, 0.538175190869274, 0.539511342591974, 0.540847494314674, 0.542183646037375, 0.543519797760075, 0.544855949482776, 0.546192101205476, 0.547528252928176, 0.548864404650877, 0.550200556373577, 0.551536708096278, 0.552872859818978, 0.554209011541679, 0.555545163264379, 0.556881314987079, 0.558217466709780, 0.559553618432480, 0.560889770155181, 0.562225921877881, 0.563562073600581, 0.564898225323282, 0.566234377045982, 0.567570528768683, 0.568906680491383, 0.570242832214084, 0.571578983936784, 0.572915135659484, 0.574251287382185, 0.575587439104885, 0.576923590827586, 0.578259742550286, 0.579595894272986, 0.580932045995687, 0.582268197718387, 0.583604349441088, 0.584940501163788, 0.586276652886489, 0.587612804609189, 0.588948956331889, 0.590285108054590, 0.591621259777290, 0.592957411499991, 0.594293563222691, 0.595629714945391, 0.596965866668092, 0.598302018390792, 0.599638170113493, 0.600974321836193, 0.602310473558894, 0.603646625281594, 0.604982777004294, 0.606318928726995, 0.607655080449695, 0.608991232172396, 0.610327383895096, 0.611663535617796, 0.612999687340497, 0.614335839063197, 0.615671990785898, 0.617008142508598, 0.618344294231299, 0.619680445953999, 0.621016597676699, 0.622352749399400, 0.623688901122100, 0.625025052844801, 0.626361204567501, 0.627697356290201, 0.629033508012902, 0.630369659735602, 0.631705811458303, 0.633041963181003, 0.634378114903704, 0.635714266626404, 0.637050418349104, 0.638386570071805, 0.639722721794505, 0.641058873517206, 0.642395025239906, 0.643731176962606, 0.645067328685307, 0.646403480408007, 0.647739632130708, 0.649075783853408, 0.650411935576109, 0.651748087298809, 0.653084239021509, 0.654420390744210, 0.655756542466910, 0.657092694189611, 0.658428845912311, 0.659764997635011, 0.661101149357712, 0.662437301080412, 0.663773452803113, 0.665109604525813, 0.666445756248514, 0.667781907971214, 0.669118059693914, 0.670454211416615, 0.671790363139315, 0.673126514862016, 0.674462666584716, 0.675798818307416, 0.677134970030117, 0.678471121752817, 0.679807273475518, 0.681143425198218, 0.682479576920918, 0.683815728643619, 0.685151880366319, 0.686488032089020, 0.687824183811720, 0.689160335534421, 0.690496487257121, 0.691832638979821, 0.693168790702522, 0.694504942425222, 0.695841094147923, 0.697177245870623, 0.698513397593323, 0.699849549316024, 0.701185701038724, 0.702521852761425, 0.703858004484125, 0.705194156206826, 0.706530307929526, 0.707866459652226, 0.709202611374927, 0.710538763097627, 0.711874914820328, 0.713211066543028, 0.714547218265728, 0.715883369988429, 0.717219521711129, 0.718555673433830, 0.719891825156530, 0.721227976879231, 0.722564128601931, 0.723900280324631, 0.725236432047332, 0.726572583770032, 0.727908735492733, 0.729244887215433, 0.730581038938133, 0.731917190660834, 0.733253342383534, 0.734589494106235, 0.735925645828935, 0.737261797551636, 0.738597949274336, 0.739934100997036, 0.741270252719737, 0.742606404442437, 0.743942556165138, 0.745278707887838, 0.746614859610538, 0.747951011333239, 0.749287163055939, 0.750623314778640, 0.751959466501340, 0.753295618224041, 0.754631769946741, 0.755967921669441, 0.757304073392142, 0.758640225114842, 0.759976376837543, 0.761312528560243, 0.762648680282943, 0.763984832005644, 0.765320983728344, 0.766657135451045, 0.767993287173745, 0.769329438896446, 0.770665590619146, 0.772001742341846, 0.773337894064547, 0.774674045787247, 0.776010197509948, 0.777346349232648, 0.778682500955348, 0.780018652678049, 0.781354804400749, 0.782690956123450, 0.784027107846150, 0.785363259568851, 0.786699411291551, 0.788035563014251, 0.789371714736952, 0.790707866459652, 0.792044018182353, 0.793380169905053, 0.794716321627753, 0.796052473350454, 0.797388625073154, 0.798724776795855, 0.800060928518555, 0.801397080241256, 0.802733231963956, 0.804069383686656, 0.805405535409357, 0.806741687132057, 0.808077838854758, 0.809413990577458, 0.810750142300158, 0.812086294022859, 0.813422445745559, 0.814758597468260, 0.816094749190960, 0.817430900913661, 0.818767052636361, 0.820103204359061, 0.821439356081762, 0.822775507804462, 0.824111659527163, 0.825447811249863, 0.826783962972563, 0.828120114695264, 0.829456266417964, 0.830792418140665, 0.832128569863365, 0.833464721586066, 0.834800873308766, 0.836137025031466, 0.837473176754167, 0.838809328476867, 0.840145480199568, 0.841481631922268, 0.842817783644968, 0.844153935367669, 0.845490087090369, 0.846826238813070, 0.848162390535770, 0.849498542258470, 0.850834693981171, 0.852170845703871, 0.853506997426572, 0.854843149149272, 0.856179300871973, 0.857515452594673, 0.858851604317373, 0.860187756040074, 0.861523907762774, 0.862860059485475, 0.864196211208175, 0.865532362930875, 0.866868514653576, 0.868204666376276, 0.869540818098977, 0.870876969821677, 0.872213121544378, 0.873549273267078, 0.874885424989778, 0.876221576712479, 0.877557728435179, 0.878893880157880, 0.880230031880580, 0.881566183603280, 0.882902335325981, 0.884238487048681, 0.885574638771382, 0.886910790494082, 0.888246942216783, 0.889583093939483, 0.890919245662183, 0.892255397384884, 0.893591549107584, 0.894927700830285, 0.896263852552985, 0.897600004275685, 0.898936155998386, 0.900272307721086, 0.901608459443787, 0.902944611166487, 0.904280762889188, 0.905616914611888, 0.906953066334588, 0.908289218057289, 0.909625369779989, 0.910961521502690, 0.912297673225390, 0.913633824948090, 0.914969976670791, 0.916306128393491, 0.917642280116192, 0.919000395123209, 0.920338788669353, 0.921654913780206, 0.922949140962045, 0.924221834556595, 0.925473352843603, 0.926704048141686, 0.927914266907520, 0.929104349833374, 0.930274631943016, 0.931425442686031, 0.932557106030558, 0.933669940554492, 0.934764259535182, 0.935840371037602, 0.936898578001085, 0.937939178324610, 0.938962464950653, 0.939968725947663, 0.940958244591156, 0.941931299443453, 0.942888164432106, 0.943829108927011, 0.944754397816244, 0.945664291580614, 0.946559046367014, 0.947438914060509, 0.948304142355261, 0.949154974824251, 0.949991650987844, 0.950814406381226, 0.951623472620704, 0.952419077468909, 0.953201444898923, 0.953970795157311, 0.954727344826145, 0.955471306883966, 0.956202890765733, 0.956922302421791, 0.957629744375846, 0.958325415781975, 0.959009512480699, 0.959682227054106, 0.960343748880064, 0.960994264185543, 0.961633956099033, 0.962263004702097, 0.962881587080076, 0.963489877371918, 0.964088046819229, 0.964676263814465, 0.965254693948328, 0.965823500056402, 0.966382842264975, 0.966932878036132, 0.967473762212069, 0.968005647058694, 0.968528682308485, 0.969043015202643, 0.969548790532545, 0.970046150680502, 0.970580611369582, 0.971115072058662, 0.971649532747743, 0.972183993436823, 0.972718454125903, 0.973252914814983, 0.973787375504063, 0.974321836193144, 0.974856296882223, 0.975390757571304, 0.975925218260384, 0.976459678949464, 0.976994139638544, 0.977528600327624, 0.978063061016705, 0.978597521705785, 0.979131982394865, 0.979666443083945, 0.980200903773025, 0.980735364462105, 0.981269825151186, 0.981804285840266, 0.982338746529346, 0.982873207218426, 0.983407667907506, 0.983942128596586, 0.984476589285667, 0.985011049974747, 0.985545510663827, 0.986079971352907, 0.986614432041987, 0.987148892731067, 0.987683353420148, 0.988217814109228, 0.988752274798308, 0.989286735487388, 0.989821196176468, 0.990355656865548, 0.990890117554629, 0.991424578243709, 0.991959038932789, 0.992493499621869, 0.993027960310949, 0.993562421000029, 0.994096881689110, 0.994631342378190, 0.995165803067270, 0.995700263756350, 0.996234724445430, 0.996769185134510, 0.997303645823591, 0.997704491340401, 0.998105336857211, 0.998522611695603, 0.998922580696883, 0.999222557447843, 0.999422541948483, 0.999572530323963, 0.999702520249380, 0.999822510949763, 0.999872507074923, 0.999892505524987, 0.999922503200083, 0.999949226234537, 0.999975949268991, 0.999989310786218, 0.999995991544832, 1.0};
        constant Real[Ninterval, 4] Tcoef = {{2.428119691060830e+012, -4.216479757645069e+008, 9.287051822649143e+004, 1.698508300000001e+002}, {2.166914762300397e+012, -3.992111678478829e+008, 9.062913441482277e+004, 1.700999999999999e+002}, {1.899159185577863e+012, -3.795555733430910e+008, 8.878532320450363e+004, 1.703092800000000e+002}, {1.629916579808073e+012, -3.435128543022376e+008, 8.477062663992558e+004, 1.707856300000000e+002}, {1.398839241865448e+012, -3.108983786183325e+008, 8.093962908382615e+004, 1.712651700000000e+002}, {1.200574221723735e+012, -2.813879938533571e+008, 7.728396604745965e+004, 1.717479200000000e+002}, {1.030417923000963e+012, -2.546836477814989e+008, 7.379541586188905e+004, 1.722339300000000e+002}, {8.844001037952972e+011, -2.305189284572275e+008, 7.046631066361128e+004, 1.727232300000000e+002}, {7.590860483164475e+011, -2.086514602087597e+008, 6.728926490972207e+004, 1.732158600000001e+002}, {6.515463964671414e+011, -1.888629690497167e+008, 6.425730760561715e+004, 1.737118499999999e+002}, {5.592552216502428e+011, -1.709551666980799e+008, 6.136372775951276e+004, 1.742112400000000e+002}, {4.800416585372977e+011, -1.547489317359679e+008, 5.860218639168952e+004, 1.747140600000000e+002}, {4.120619529393444e+011, -1.400824449235276e+008, 5.596654048742052e+004, 1.752203600000000e+002}, {3.537140308461040e+011, -1.268087414876019e+008, 5.345098341920615e+004, 1.757301800000000e+002}, {3.036357271971793e+011, -1.147956769847417e+008, 5.105003430189027e+004, 1.762435499999999e+002}, {2.606525213033651e+011, -1.039230394931327e+008, 4.875837260556154e+004, 1.767605200000000e+002}, {2.237609674834909e+011, -9.408262939444540e+007, 4.657101751284228e+004, 1.772811199999999e+002}, {1.922419405902699e+011, -8.518143962347658e+007, 4.448317923172022e+004, 1.778053999999999e+002}, {1.656321149901774e+011, -7.714416678199874e+007, 4.249042535564808e+004, 1.783334000000000e+002}, {1.431407593855788e+011, -6.989094076846488e+007, 4.058849075808411e+004, 1.788651600000000e+002}, {1.241037279204862e+011, -6.345000608556164e+007, 3.880841089253586e+004, 1.793900000000000e+002}, {1.078897726938036e+011, -5.787323844457100e+007, 3.719668427889198e+004, 1.798900000000000e+002}, {9.388245239329929e+010, -5.281986768088788e+007, 3.566287663310264e+004, 1.803899999999999e+002}, {8.177026870962473e+010, -4.823785073207039e+007, 3.420275088737812e+004, 1.808900000000001e+002}, {7.128653091837167e+010, -4.408054713844873e+007, 3.281232303846304e+004, 1.813900000000000e+002}, {6.220419978100047e+010, -4.030623531248382e+007, 3.148784749056206e+004, 1.818900000000000e+002}, {5.432814901566791e+010, -3.687745020027063e+007, 3.022579953184881e+004, 1.823900000000000e+002}, {4.749233588110072e+010, -3.376066397124648e+007, 2.902286362710196e+004, 1.828899999999999e+002}, {4.155352730607562e+010, -3.092570824858189e+007, 2.787591713537610e+004, 1.833900000000000e+002}, {3.638967234175739e+010, -2.834555680690369e+007, 2.678202091246125e+004, 1.838900000000000e+002}, {3.189545805614265e+010, -2.599587187121439e+007, 2.573840508245055e+004, 1.843899999999999e+002}, {2.798050203785508e+010, -2.385478375872732e+007, 2.474246009078133e+004, 1.848900000000000e+002}, {2.456737261968692e+010, -2.190263720157386e+007, 2.379172636307514e+004, 1.853900000000000e+002}, {2.158887725624487e+010, -2.012168454337252e+007, 2.288388355112532e+004, 1.858900000000000e+002}, {1.898760836682233e+010, -1.849598750549042e+007, 2.201674428367421e+004, 1.863900000000000e+002}, {1.671373899396782e+010, -1.701114199658637e+007, 2.118824358594005e+004, 1.868900000000001e+002}, {1.472442515376055e+010, -1.565417444528254e+007, 2.039643299085866e+004, 1.873900000000000e+002}, {1.298254187769645e+010, -1.441336250162147e+007, 1.963947244326254e+004, 1.878900000000000e+002}, {1.145612023308615e+010, -1.327813374098965e+007, 1.891562443180674e+004, 1.883899999999999e+002}, {1.011726329520874e+010, -1.223890854106894e+007, 1.822324676787706e+004, 1.888899999999999e+002}, {8.942115914174372e+009, -1.128706309621500e+007, 1.756078856084207e+004, 1.893899999999999e+002}, {7.909709317744029e+009, -1.041475590863545e+007, 1.692678236252691e+004, 1.898900000000000e+002}, {7.002011600872572e+009, -9.614913403719645e+006, 1.631984148717728e+004, 1.903899999999999e+002}, {6.203335937528857e+009, -8.881122302459281e+006, 1.573865385204634e+004, 1.908899999999999e+002}, {5.500000426851387e+009, -8.207563992121900e+006, 1.518197779409089e+004, 1.913900000000000e+002}, {4.880147872846049e+009, -7.588969652936688e+006, 1.464863839566741e+004, 1.918900000000000e+002}, {4.333458916781487e+009, -7.020558044503353e+006, 1.413752314725710e+004, 1.923900000000000e+002}, {3.850903162697156e+009, -6.497982633829402e+006, 1.364757825073601e+004, 1.928900000000000e+002}, {3.424640794532519e+009, -6.017301775739881e+006, 1.317780570385806e+004, 1.933900000000000e+002}, {3.047820397687019e+009, -5.574930915074369e+006, 1.272725963793938e+004, 1.938900000000000e+002}, {2.714455522862305e+009, -5.167610077477907e+006, 1.229504347489282e+004, 1.943900000000000e+002}, {2.419310099883856e+009, -4.792372987537484e+006, 1.188030713941842e+004, 1.948900000000000e+002}, {2.157814657986750e+009, -4.446521482506972e+006, 1.148224448765968e+004, 1.953900000000000e+002}, {1.925958574178695e+009, -4.127595989843101e+006, 1.110009063273798e+004, 1.958900000000000e+002}, {1.720235556842085e+009, -3.833356838509688e+006, 1.073311982718851e+004, 1.963900000000000e+002}, {1.537568498602969e+009, -3.561761260770541e+006, 1.038064313240985e+004, 1.968900000000000e+002}, {1.375254807055739e+009, -3.310945444478069e+006, 1.004200642469327e+004, 1.973900000000000e+002}, {1.230926157318009e+009, -3.079209502474114e+006, 9.716588529824085e+003, 1.978900000000000e+002}, {1.102501569080066e+009, -2.865000978909834e+006, 9.403799307559229e+003, 1.983899999999999e+002}, {9.881431890151527e+008, -2.666899927117780e+006, 9.103077924469371e+003, 1.988900000000000e+002}, {8.862460643504052e+008, -2.483611530634848e+006, 8.813891493500507e+003, 1.993900000000000e+002}, {7.953869975993569e+008, -2.313948910499454e+006, 8.535733187669037e+003, 1.998900000000000e+002}, {7.143138769691485e+008, -2.156827077065875e+006, 8.268121128719055e+003, 2.003900000000000e+002}, {6.419280859884250e+008, -2.011254157318892e+006, 8.010596993801432e+003, 2.008900000000001e+002}, {5.772519450401038e+008, -1.876319403262300e+006, 7.762724544343183e+003, 2.013900000000000e+002}, {5.194272793309037e+008, -1.751190083800481e+006, 7.524088760577367e+003, 2.018899999999999e+002}, {4.676942564542144e+008, -1.635102283023300e+006, 7.294294502273217e+003, 2.023900000000000e+002}, {4.213813991521418e+008, -1.527355501832372e+006, 7.072965529370125e+003, 2.028900000000000e+002}, {3.798929639616593e+008, -1.427306562607491e+006, 6.859743479795654e+003, 2.033900000000000e+002}, {3.427034243188576e+008, -1.334365732814233e+006, 6.654287012551113e+003, 2.038900000000000e+002}, {3.093461000479141e+008, -1.247990970222474e+006, 6.456270819862391e+003, 2.043900000000000e+002}, {2.794066491203365e+008, -1.167684115936397e+006, 6.265384847491296e+003, 2.048900000000000e+002}, {2.525185307660005e+008, -1.092987656856107e+006, 6.081333540245373e+003, 2.053899999999999e+002}, {2.283558222313167e+008, -1.023480600819874e+006, 5.903835032062275e+003, 2.058900000000000e+002}, {2.066285778103371e+008, -9.587754989355512e+005, 5.732620476110842e+003, 2.063899999999999e+002}, {1.870796358504928e+008, -8.985159698708151e+005, 5.567433409816551e+003, 2.068900000000000e+002}, {1.694800630152804e+008, -8.423737208584454e+005, 5.408029086050894e+003, 2.073900000000000e+002}, {1.536254323472121e+008, -7.900460905552147e+005, 5.254173889079566e+003, 2.078900000000000e+002}, {1.393350041292428e+008, -7.412546763455822e+005, 5.105644838061499e+003, 2.083900000000000e+002}, {1.264461946876812e+008, -6.957420884142442e+005, 4.962228931931791e+003, 2.088900000000000e+002}, {1.148149957717430e+008, -6.532714037371146e+005, 4.823722794654673e+003, 2.093900000000000e+002}, {1.043126918759436e+008, -6.136238260582621e+005, 4.689932103330038e+003, 2.098900000000000e+002}, {9.482399240733823e+007, -5.765972058455468e+005, 4.560671161847407e+003, 2.103900000000000e+002}, {8.624641502669747e+007, -5.420050981894244e+005, 4.435762520264600e+003, 2.108900000000000e+002}, {7.848790676326571e+007, -5.096749922949291e+005, 4.315036509192491e+003, 2.113899999999999e+002}, {7.146637230514374e+007, -4.794475711409992e+005, 4.198330916505517e+003, 2.118899999999999e+002}, {6.510826527489491e+007, -4.511754575867408e+005, 4.085490594353537e+003, 2.123900000000000e+002}, {5.934772889807401e+007, -4.247223114761246e+005, 3.976367129792119e+003, 2.128900000000000e+002}, {5.412562005348311e+007, -3.999618944172335e+005, 3.870818517414752e+003, 2.133899999999999e+002}, {4.938925600232123e+007, -3.767775105272770e+005, 3.768708878722507e+003, 2.138899999999999e+002}, {4.509087667312729e+007, -3.550607788424417e+005, 3.669908110384081e+003, 2.143900000000000e+002}, {4.118807316164903e+007, -3.347115756562438e+005, 3.574291699195102e+003, 2.148900000000000e+002}, {3.764247330924181e+007, -3.156368905942572e+005, 3.481740374155148e+003, 2.153899999999999e+002}, {3.441971899342819e+007, -2.977505663723726e+005, 3.392139916478927e+003, 2.158900000000000e+002}, {3.148883664649886e+007, -2.809726258964635e+005, 3.305380896883451e+003, 2.163899999999999e+002}, {2.882206369704212e+007, -2.652289174340525e+005, 3.221358471964407e+003, 2.168900000000000e+002}, {2.639430226411493e+007, -2.504505234176753e+005, 3.139972148385238e+003, 2.173900000000001e+002}, {2.418303052952018e+007, -2.365735031462724e+005, 3.061125608726445e+003, 2.178900000000000e+002}, {2.216790736643471e+007, -2.235384141346653e+005, 2.984726499486276e+003, 2.183900000000000e+002}, {2.033061705830633e+007, -2.112900306234645e+005, 2.910686262830227e+003, 2.188900000000000e+002}, {1.865461368328635e+007, -1.997769906568943e+005, 2.838919956827780e+003, 2.193900000000000e+002}, {1.712501178845796e+007, -1.889515587375750e+005, 2.769346100218770e+003, 2.198900000000000e+002}, {1.572830246673164e+007, -1.787692672877184e+005, 2.701886501082210e+003, 2.203900000000002e+002}, {1.445234208083232e+007, -1.691887831467449e+005, 2.636466131342296e+003, 2.208899999999999e+002}, {1.328611259790656e+007, -1.601715845366386e+005, 2.573012964462273e+003, 2.213899999999999e+002}, {1.221967792218837e+007, -1.516818185846242e+005, 2.511457857150583e+003, 2.218899999999999e+002}, {1.124402410901016e+007, -1.436860587692310e+005, 2.451734410827016e+003, 2.223900000000001e+002}, {1.035100231090916e+007, -1.361531635944861e+005, 2.393778859436568e+003, 2.228900000000000e+002}, {9.533236131047620e+006, -1.290541003018532e+005, 2.337529949475997e+003, 2.233899999999999e+002}, {8.784032342642559e+006, -1.223617814028497e+005, 2.282928828509020e+003, 2.238899999999999e+002}, {8.097324140450608e+006, -1.160509373912711e+005, 2.229918944431192e+003, 2.243900000000000e+002}, {7.467609003399250e+006, -1.100979854201498e+005, 2.178445944730142e+003, 2.248900000000000e+002}, {6.889898183747469e+006, -1.044809122220948e+005, 2.128457581571682e+003, 2.253900000000001e+002}, {6.359654158917572e+006, -9.917915038790123e+004, 2.079903618776534e+003, 2.258900000000000e+002}, {5.872759686767209e+006, -9.417349114733808e+004, 2.032735749928480e+003, 2.263900000000000e+002}, {5.425475631504861e+006, -8.944598462814202e+004, 1.986907513929262e+003, 2.268900000000000e+002}, {5.014390733055720e+006, -8.497983886637562e+004, 1.942374214543448e+003, 2.273900000000000e+002}, {4.636415792979381e+006, -8.075936791787535e+004, 1.899092852995913e+003, 2.278900000000000e+002}, {4.288733044060946e+006, -7.676988872480289e+004, 1.857022047834165e+003, 2.283900000000000e+002}, {3.968773876673203e+006, -7.299765929671984e+004, 1.816121971306828e+003, 2.288899999999999e+002}, {3.674202520100520e+006, -6.942982257803570e+004, 1.776354286367471e+003, 2.293900000000000e+002}, {3.402892107203013e+006, -6.605434102471954e+004, 1.737682082113611e+003, 2.298900000000000e+002}, {3.152896273279971e+006, -6.285993128052545e+004, 1.700069813095759e+003, 2.303899999999999e+002}, {2.922447601033651e+006, -5.983603289980221e+004, 1.663483248995434e+003, 2.308899999999999e+002}, {2.709926760057117e+006, -5.697273957445595e+004, 1.627889413642931e+003, 2.313900000000000e+002}, {2.513861217833642e+006, -5.426077246627386e+004, 1.593256539802403e+003, 2.318900000000001e+002}, {2.332900319601032e+006, -5.169142195505573e+004, 1.559554014347537e+003, 2.323900000000000e+002}, {2.165813772476697e+006, -4.925652436360114e+004, 1.526752336938422e+003, 2.328900000000000e+002}, {2.011475838578284e+006, -4.694841733751677e+004, 1.494823071235126e+003, 2.333899999999999e+002}, {1.868856269356124e+006, -4.475990809519066e+004, 1.463738803224226e+003, 2.338900000000000e+002}, {1.737013293432888e+006, -4.268424495431413e+004, 1.433473101024697e+003, 2.343900000000000e+002}, {1.615084644443825e+006, -4.071508653570022e+004, 1.404000474764923e+003, 2.348899999999998e+002}, {1.502280625890901e+006, -3.884647544872509e+004, 1.375296339470493e+003, 2.353900000000001e+002}, {1.397877494187559e+006, -3.707281331859772e+004, 1.347336979285312e+003, 2.358900000000000e+002}, {1.301213605644812e+006, -3.538884022447428e+004, 1.320099513857462e+003, 2.363899999999998e+002}, {1.211679248380561e+006, -3.378960565339209e+004, 1.293561863094709e+003, 2.368900000000000e+002}, {1.128718624184195e+006, -3.227045988925767e+004, 1.267702720329528e+003, 2.373900000000000e+002}, {1.051819631323617e+006, -3.082702432545408e+004, 1.242501517244634e+003, 2.378900000000000e+002}, {9.805119943329289e+005, -2.945517899158243e+004, 1.217938398261965e+003, 2.383900000000000e+002}, {9.143661234246076e+005, -2.815104963832588e+004, 1.193994193542140e+003, 2.388899999999999e+002}, {8.529843036096668e+005, -2.691098269234095e+004, 1.170650388729732e+003, 2.393900000000000e+002}, {7.960030849987015e+005, -2.573154186886301e+004, 1.147889104993057e+003, 2.398900000000000e+002}, {7.430884210129550e+005, -2.460948952764748e+004, 1.125693071288439e+003, 2.403900000000000e+002}, {6.939312882245765e+005, -2.354177155605786e+004, 1.104045600935325e+003, 2.408900000000000e+002}, {6.482492262091814e+005, -2.252551343062689e+004, 1.082930572906883e+003, 2.413899999999999e+002}, {6.057812463458836e+005, -2.155800267024627e+004, 1.062332406718743e+003, 2.418899999999999e+002}, {5.662875366021142e+005, -2.063668238211267e+004, 1.042236044331130e+003, 2.423899999999999e+002}, {5.295464858440637e+005, -1.975913928732001e+004, 1.022626929351548e+003, 2.428900000000000e+002}, {4.953547285884677e+005, -1.892309832108412e+004, 1.003490989941396e+003, 2.433899999999999e+002}, {4.635243665572284e+005, -1.812641112749775e+004, 9.848146189060615e+002, 2.438900000000000e+002}, {4.338818671474538e+005, -1.736704928321503e+004, 9.665846574512200e+002, 2.443900000000000e+002}, {4.062679337763968e+005, -1.664309916254490e+004, 9.487883794001973e+002, 2.448899999999999e+002}, {3.805346095862030e+005, -1.595275107074704e+004, 9.314134731830150e+002, 2.453899999999999e+002}, {3.565460578222151e+005, -1.529429731212101e+004, 9.144480292797495e+002, 2.458899999999999e+002}, {3.341763897850724e+005, -1.466612297632049e+004, 8.978805234049410e+002, 2.463900000000000e+002}, {3.133095571597457e+005, -1.406670233964544e+004, 8.816998038639424e+002, 2.468900000000000e+002}, {2.938381657336060e+005, -1.349459252458615e+004, 8.658950773076953e+002, 2.473899999999999e+002}, {2.756629705093029e+005, -1.294842921334388e+004, 8.504558962743154e+002, 2.478900000000001e+002}, {2.586922013389850e+005, -1.242692195684236e+004, 8.353721466343773e+002, 2.483899999999999e+002}, {2.428410028923686e+005, -1.192884991972681e+004, 8.206340356628965e+002, 2.488900000000001e+002}, {2.280307218527483e+005, -1.145305746217065e+004, 8.062320804248278e+002, 2.493899999999998e+002}, {2.141886791630994e+005, -1.099845109336764e+004, 7.921570972279910e+002, 2.498900000000000e+002}, {2.012474588055167e+005, -1.056399519338368e+004, 7.784001905236512e+002, 2.503900000000000e+002}, {1.891447383294132e+005, -1.014870937969869e+004, 7.649527432295186e+002, 2.508900000000000e+002}, {1.778225648933724e+005, -9.751664453811391e+003, 7.518064063753014e+002, 2.513900000000000e+002}, {1.672274901217288e+005, -9.371980845757489e+003, 7.389530906287266e+002, 2.518900000000000e+002}, {1.573096370463601e+005, -9.008824060035608e+003, 7.263849560163876e+002, 2.523900000000000e+002}, {1.480229998054410e+005, -8.661403967241104e+003, 7.140944046859372e+002, 2.528899999999999e+002}, {1.393247573204245e+005, -8.328970961973500e+003, 7.020740713322290e+002, 2.533900000000000e+002}, {1.311751868668219e+005, -8.010814302914930e+003, 6.903168158285998e+002, 2.538900000000000e+002}, {1.235374217684196e+005, -7.706259887656581e+003, 6.788157153740614e+002, 2.543900000000000e+002}, {1.163771876379430e+005, -7.414668078450832e+003, 6.675640569646922e+002, 2.548900000000000e+002}, {1.096627095692096e+005, -7.135432106638779e+003, 6.565553304580889e+002, 2.553900000000000e+002}, {1.033643398452726e+005, -6.867975679189847e+003, 6.457832212746632e+002, 2.558899999999999e+002}, {9.745461870325112e+004, -6.611752090606020e+003, 6.352416044310508e+002, 2.563899999999998e+002}, {9.190798963753549e+004, -6.366242084979492e+003, 6.249245375741506e+002, 2.568900000000000e+002}, {8.670064055849906e+004, -6.130952362236059e+003, 6.148262549854942e+002, 2.573900000000000e+002}, {8.181042020600053e+004, -5.905414374178652e+003, 6.049411618718673e+002, 2.578900000000000e+002}, {7.721677640854748e+004, -5.689183128996180e+003, 5.952638286607794e+002, 2.583899999999999e+002}, {7.290044135202718e+004, -5.481835253654937e+003, 5.857889850231684e+002, 2.588900000000000e+002}, {6.884362660689066e+004, -5.282968916694660e+003, 5.765115155427385e+002, 2.593900000000000e+002}, {6.502964570767200e+004, -5.092201572104827e+003, 5.674264535024774e+002, 2.598899999999999e+002}, {6.144296599615412e+004, -4.909169573093382e+003, 5.585289768075917e+002, 2.603900000000000e+002}, {5.806916350594535e+004, -4.733527235738733e+003, 5.498144031252836e+002, 2.608900000000001e+002}, {5.489471665728002e+004, -4.564945391819472e+003, 5.412781848879703e+002, 2.613899999999999e+002}, {5.190705215061930e+004, -4.403111022700504e+003, 5.329159054741148e+002, 2.618899999999999e+002}, {4.909445721484846e+004, -4.247726255069810e+003, 5.247232746761317e+002, 2.623900000000000e+002}, {4.644593911428791e+004, -4.098507286040056e+003, 5.166961244486026e+002, 2.628900000000000e+002}, {4.395128803961570e+004, -3.955184162531220e+003, 5.088304055117806e+002, 2.633899999999999e+002}, {4.160096100295921e+004, -3.817499734155457e+003, 5.011221830784011e+002, 2.638900000000001e+002}, {3.938601346943615e+004, -3.685208951092035e+003, 4.935676332701902e+002, 2.643900000000000e+002}, {3.729813534244107e+004, -3.558078575814604e+003, 4.861630399003062e+002, 2.648900000000000e+002}, {3.532951540796938e+004, -3.435886151050459e+003, 4.789047905747109e+002, 2.653900000000000e+002}, {3.347288342828854e+004, -3.318419829035272e+003, 4.717893738824646e+002, 2.658900000000001e+002}, {3.172142027917930e+004, -3.205477566352382e+003, 4.648133758676585e+002, 2.663899999999999e+002}, {3.006875992796082e+004, -3.096866799830403e+003, 4.579734772020614e+002, 2.668899999999999e+002}, {2.850893674371467e+004, -2.992403858160356e+003, 4.512664500780765e+002, 2.673900000000000e+002}, {2.703637806135581e+004, -2.891913613497721e+003, 4.446891554758121e+002, 2.678900000000000e+002}, {2.564585331227819e+004, -2.795228937660512e+003, 4.382385402712435e+002, 2.683899999999999e+002}, {2.433248558704846e+004, -2.702190470535179e+003, 4.319116347685376e+002, 2.688900000000000e+002}, {2.309167689222681e+004, -2.612645984757337e+003, 4.257055498446611e+002, 2.693900000000000e+002}, {2.191914997435190e+004, -2.526450353074852e+003, 4.196174748723824e+002, 2.698899999999999e+002}, {2.081087709618847e+004, -2.443464913240735e+003, 4.136446749057724e+002, 2.703900000000000e+002}, {1.976308961002175e+004, -2.363557314740986e+003, 4.077844886436558e+002, 2.708900000000000e+002}, {1.877225549790460e+004, -2.286601167769257e+003, 4.020343260965227e+002, 2.713900000000000e+002}, {1.783505491279766e+004, -2.212475713419265e+003, 3.963916663955540e+002, 2.718900000000000e+002}, {1.694838403963000e+004, -2.141065646731519e+003, 3.908540558216300e+002, 2.723900000000000e+002}, {1.610932000557480e+004, -2.072260732910480e+003, 3.854191056171448e+002, 2.728899999999999e+002}, {1.531512874676714e+004, -2.005955680031755e+003, 3.800844902084921e+002, 2.733900000000001e+002}, {1.456323697582539e+004, -1.942049804693102e+003, 3.748479451687998e+002, 2.738900000000000e+002}, {1.385123026469367e+004, -1.880446871012044e+003, 3.697072655023023e+002, 2.743899999999999e+002}, {1.317684399562190e+004, -1.821054876924206e+003, 3.646603038516667e+002, 2.748899999999999e+002}, {1.253794303834637e+004, -1.763785795953628e+003, 3.597049687290192e+002, 2.753899999999999e+002}, {1.193253057409521e+004, -1.708555492422938e+003, 3.548392230103814e+002, 2.758900000000000e+002}, {1.135872219923051e+004, -1.655283426022064e+003, 3.500610821583481e+002, 2.763900000000000e+002}, {1.081475019376878e+004, -1.603892563634609e+003, 3.453686128301867e+002, 2.768900000000001e+002}, {1.029894855746364e+004, -1.554309165597434e+003, 3.407599312946576e+002, 2.773900000000000e+002}, {9.809756431292144e+003, -1.506462692323420e+003, 3.362332020717720e+002, 2.778900000000001e+002}, {9.345698521645902e+003, -1.460285569720035e+003, 3.317866364015975e+002, 2.783899999999999e+002}, {8.905390176188423e+003, -1.415713129876954e+003, 3.274184910384032e+002, 2.788900000000000e+002}, {8.487528222283663e+003, -1.372683444667674e+003, 3.231270668582708e+002, 2.793900000000001e+002}, {8.090883603193332e+003, -1.331137188443026e+003, 3.189107075891501e+002, 2.798899999999998e+002}, {7.714300903571747e+003, -1.291017548047196e+003, 3.147677986317169e+002, 2.803899999999999e+002}, {7.356691193390297e+003, -1.252270086933305e+003, 3.106967658260311e+002, 2.808900000000000e+002}, {7.017027940720315e+003, -1.214842639685586e+003, 3.066960743158503e+002, 2.813900000000000e+002}, {6.694344504639365e+003, -1.178685218691503e+003, 3.027642274486991e+002, 2.818900000000000e+002}, {6.387729498401815e+003, -1.143749907833270e+003, 2.988997656793167e+002, 2.823900000000000e+002}, {6.096323639887130e+003, -1.109990771522367e+003, 2.951012655307434e+002, 2.828900000000000e+002}, {5.819316375605656e+003, -1.077363765273413e+003, 2.913673385813619e+002, 2.833899999999999e+002}, {5.555945230723103e+003, -1.045826664792471e+003, 2.876966304991468e+002, 2.838900000000000e+002}, {5.305486577084192e+003, -1.015338940849282e+003, 2.840878200262260e+002, 2.843899999999999e+002}, {5.067262644382695e+003, -9.858617563664993e+002, 2.805396181886476e+002, 2.848899999999999e+002}, {4.840629717527347e+003, -9.573578165466240e+002, 2.770507672483719e+002, 2.853900000000001e+002}, {4.624982316899136e+003, -9.297913562094941e+002, 2.736200399658823e+002, 2.858899999999998e+002}, {4.419748465054315e+003, -9.031280472312861e+002, 2.702462386898002e+002, 2.863899999999999e+002}, {4.224387583152579e+003, -8.773349366812882e+002, 2.669281945592340e+002, 2.868900000000001e+002}, {4.038388297731713e+003, -8.523803870286566e+002, 2.636647667234346e+002, 2.873900000000000e+002}, {3.861269499591353e+003, -8.282340390911418e+002, 2.604548416035581e+002, 2.878899999999999e+002}, {3.692574672364702e+003, -8.048667226026902e+002, 2.572973320823326e+002, 2.883900000000000e+002}, {3.531873216036285e+003, -7.822504319031273e+002, 2.541911768555779e+002, 2.888899999999999e+002}, {3.378757294095987e+003, -7.603582607212622e+002, 2.511353396959848e+002, 2.893899999999999e+002}, {3.232842821404854e+003, -7.391643734589071e+002, 2.481288088106870e+002, 2.898899999999998e+002}, {3.093764229251524e+003, -7.186439267744648e+002, 2.451705961230077e+002, 2.903899999999998e+002}, {2.961178144974160e+003, -6.987730702280496e+002, 2.422597367302420e+002, 2.908899999999999e+002}, {2.834757569070961e+003, -6.795288612382922e+002, 2.393952881854896e+002, 2.913899999999999e+002}, {2.714194398673150e+003, -6.608892613772416e+002, 2.365763299875632e+002, 2.918900000000000e+002}, {2.599196768073861e+003, -6.428330798822078e+002, 2.338019629427569e+002, 2.923899999999999e+002}, {2.489487678826580e+003, -6.253399364968090e+002, 2.310713086112715e+002, 2.928900000000000e+002}, {2.384805817113105e+003, -6.083902428896103e+002, 2.283835087915235e+002, 2.933899999999999e+002}, {2.284902791850232e+003, -5.919651508239373e+002, 2.257377249419197e+002, 2.938899999999999e+002}, {2.189543827088161e+003, -5.760465367399366e+002, 2.231331377069458e+002, 2.943900000000001e+002}, {2.098506646267296e+003, -5.606169661265348e+002, 2.205689463939640e+002, 2.948899999999999e+002}, {2.011580322023283e+003, -5.456596605580177e+002, 2.180443684809316e+002, 2.953900000000000e+002}, {1.928564097136700e+003, -5.311584690552852e+002, 2.155586391540247e+002, 2.958899999999999e+002}, {1.849269849310739e+003, -5.170978692137003e+002, 2.131110108940667e+002, 2.963899999999999e+002}, {1.773516707295410e+003, -5.034628931867380e+002, 2.107007529155836e+002, 2.968900000000000e+002}, {1.701135034894489e+003, -4.902391539141415e+002, 2.083271508720442e+002, 2.973899999999999e+002}, {1.631962729032615e+003, -4.774127857226459e+002, 2.059895063267681e+002, 2.978900000000001e+002}, {1.565846506555941e+003, -4.649704442461410e+002, 2.036871364135052e+002, 2.983900000000002e+002}, {1.502641202683996e+003, -4.528992802485624e+002, 2.014193734124408e+002, 2.988900000000002e+002}, {1.442208277348613e+003, -4.411869095737466e+002, 1.991855643479903e+002, 2.993899999999999e+002}, {1.384416824560510e+003, -4.298214105524195e+002, 1.969850706592951e+002, 2.998899999999999e+002}, {1.329142377546727e+003, -4.187912951655748e+002, 1.948172677994267e+002, 3.003899999999999e+002}, {1.276266832977576e+003, -4.080854951975836e+002, 1.926815448953401e+002, 3.008900000000000e+002}, {1.225677744534944e+003, -3.976933420081253e+002, 1.905773043972621e+002, 3.013899999999999e+002}, {1.177268608034691e+003, -3.876045575132537e+002, 1.885039617618329e+002, 3.018900000000000e+002}, {1.130938214911173e+003, -3.778092338214274e+002, 1.864609451086512e+002, 3.023900000000000e+002}, {1.086590010013511e+003, -3.682978157856858e+002, 1.844476949037983e+002, 3.028899999999999e+002}, {1.044132879850411e+003, -3.590610987951029e+002, 1.824636636814579e+002, 3.033900000000000e+002}, {1.003479421950269e+003, -3.500901978022080e+002, 1.805083156956392e+002, 3.038900000000000e+002}, {9.645474592133718e+002, -3.413765556572088e+002, 1.785811266916289e+002, 3.043899999999999e+002}, {9.272577564186632e+002, -3.329119057516371e+002, 1.766815835499424e+002, 3.048899999999999e+002}, {8.915361551869480e+002, -3.246882895151356e+002, 1.748091840997496e+002, 3.053899999999999e+002}, {8.573109718488198e+002, -3.166980141610849e+002, 1.729634367537797e+002, 3.058900000000000e+002}, {8.245149278413157e+002, -3.089336702127797e+002, 1.711438603429205e+002, 3.063900000000000e+002}, {7.930833414616905e+002, -3.013880987490236e+002, 1.693499837846950e+002, 3.068899999999999e+002}, {7.629551004957449e+002, -2.940543974650270e+002, 1.675813458946156e+002, 3.073899999999999e+002}, {7.340718127746320e+002, -2.869259009969820e+002, 1.658374951082294e+002, 3.078900000000000e+002}, {7.063778149375254e+002, -2.799961751758680e+002, 1.641179892660677e+002, 3.083899999999998e+002}, {6.798204571715311e+002, -2.732590129072642e+002, 1.624223953944419e+002, 3.088899999999998e+002}, {6.543490746840538e+002, -2.667084141948034e+002, 1.607502894508461e+002, 3.093900000000000e+002}, {6.299159798870081e+002, -2.603385935161680e+002, 1.591012561598878e+002, 3.098899999999998e+002}, {6.064752538059523e+002, -2.541439563436756e+002, 1.574748887477705e+002, 3.103899999999999e+002}, {5.839833827181259e+002, -2.481191036566246e+002, 1.558707887864211e+002, 3.108899999999999e+002}, {5.623990077331871e+002, -2.422588210581205e+002, 1.542885659710487e+002, 3.113900000000002e+002}, {5.416823557921136e+002, -2.365580664806819e+002, 1.527278379169916e+002, 3.118899999999998e+002}, {5.217959156466890e+002, -2.310119745447131e+002, 1.511882300065326e+002, 3.123900000000000e+002}, {5.027036118421444e+002, -2.256158391224129e+002, 1.496693751642845e+002, 3.128900000000001e+002}, {4.843711839704919e+002, -2.203651149878204e+002, 1.481709137107541e+002, 3.133899999999999e+002}, {4.667659843054528e+002, -2.152554094514039e+002, 1.466924931733644e+002, 3.138900000000000e+002}, {4.498567607109622e+002, -2.102824750478224e+002, 1.452337681136751e+002, 3.143899999999999e+002}, {4.336138489909362e+002, -2.054422081085836e+002, 1.437943999757566e+002, 3.148900000000000e+002}, {4.180087357727423e+002, -2.007306382064600e+002, 1.423740569063953e+002, 3.153900000000000e+002}, {4.030144488980337e+002, -1.961439302658990e+002, 1.409724136234017e+002, 3.158899999999998e+002}, {3.886050416261347e+002, -1.916783723946983e+002, 1.395891512326833e+002, 3.163899999999999e+002}, {3.747559197643732e+002, -1.873303777536728e+002, 1.382239571063902e+002, 3.168899999999999e+002}, {3.614434175252985e+002, -1.830964742279822e+002, 1.368765247137150e+002, 3.173900000000002e+002}, {3.486450944768750e+002, -1.789733061102790e+002, 1.355465535030190e+002, 3.178900000000001e+002}, {3.363394583248588e+002, -1.749576258474919e+002, 1.342337487433366e+002, 3.183899999999999e+002}, {3.245059552231471e+002, -1.710462911782989e+002, 1.329378213973440e+002, 3.188900000000000e+002}, {3.131249934400873e+002, -1.672362625597232e+002, 1.316584879935901e+002, 3.193900000000001e+002}, {3.021778376318846e+002, -1.635245982990388e+002, 1.303954704917012e+002, 3.198900000000000e+002}, {2.916465737356219e+002, -1.599084513847945e+002, 1.291484961596401e+002, 3.203900000000000e+002}, {2.815140960863762e+002, -1.563850665721780e+002, 1.279172974528968e+002, 3.208899999999999e+002}, {2.717640535010451e+002, -1.529517768326230e+002, 1.267016118931412e+002, 3.213899999999999e+002}, {2.623807958853618e+002, -1.496060000695831e+002, 1.255011819518896e+002, 3.218899999999999e+002}, {2.533493995716780e+002, -1.463452370568750e+002, 1.243157549397372e+002, 3.223900000000000e+002}, {2.446555324337382e+002, -1.431670669227680e+002, 1.231450828893681e+002, 3.228899999999998e+002}, {2.362855481501693e+002, -1.400691465868011e+002, 1.219889224559401e+002, 3.233900000000001e+002}, {2.282263709932994e+002, -1.370492063764239e+002, 1.208470348027104e+002, 3.238900000000001e+002}, {2.204654530507506e+002, -1.341050475503324e+002, 1.197191855005348e+002, 3.243900000000000e+002}, {2.129908699326103e+002, -1.312345418623600e+002, 1.186051444334383e+002, 3.248899999999999e+002}, {2.057911337612362e+002, -1.284356261218127e+002, 1.175046856861692e+002, 3.253900000000000e+002}, {1.988552794177001e+002, -1.257063023229668e+002, 1.164175874610275e+002, 3.258900000000000e+002}, {1.921728401770502e+002, -1.230446350569414e+002, 1.153436319785049e+002, 3.263899999999999e+002}, {1.857337349822465e+002, -1.204487478255335e+002, 1.142826053796536e+002, 3.268899999999998e+002}, {1.795283572515752e+002, -1.179168232471038e+002, 1.132342976468749e+002, 3.273899999999998e+002}, {1.735474970466655e+002, -1.154470997486489e+002, 1.121985025073361e+002, 3.278900000000000e+002}, {1.677823331351591e+002, -1.130378699689073e+002, 1.111750173492029e+002, 3.283900000000000e+002}, {1.622244016835726e+002, -1.106874787369236e+002, 1.101636431365233e+002, 3.288900000000000e+002}, {1.568656145421817e+002, -1.083943220437680e+002, 1.091641843297179e+002, 3.293899999999999e+002}, {1.516982207140161e+002, -1.061568447400119e+002, 1.081764488004225e+002, 3.298899999999999e+002}, {1.467147735728482e+002, -1.039735387115258e+002, 1.072002477527532e+002, 3.303900000000000e+002}, {1.419081528245269e+002, -1.018429421114725e+002, 1.062353956494466e+002, 3.308900000000001e+002}, {1.372715346463949e+002, -9.976363736558096e+001, 1.052817101318265e+002, 3.313899999999999e+002}, {1.327983662960232e+002, -9.773424955603815e+001, 1.043390119451407e+002, 3.318900000000000e+002}, {1.284823811146052e+002, -9.575344556746910e+001, 1.034071248680371e+002, 3.323899999999998e+002}, {1.243175382743625e+002, -9.381993180831613e+001, 1.024858756363024e+002, 3.328900000000000e+002}, {1.202980857716605e+002, -9.193245448523439e+001, 1.015750938798825e+002, 3.333900000000000e+002}, {1.164184832888737e+002, -9.008979675148045e+001, 1.006746120438895e+002, 3.338900000000000e+002}, {1.126734095942744e+002, -8.829077808760059e+001, 9.978426532565882e+001, 3.343900000000001e+002}, {1.090577734365635e+002, -8.653425359100095e+001, 9.890389160942104e+001, 3.348899999999999e+002}, {1.055666864095504e+002, -8.481911235197464e+001, 9.803333139647950e+001, 3.353900000000000e+002}, {1.021954515577647e+002, -8.314427636901104e+001, 9.717242774102454e+001, 3.358900000000000e+002}, {9.893956151385201e+001, -8.150869971730866e+001, 9.632102618748574e+001, 3.363900000000000e+002}, {9.579468908688170e+001, -7.991136750831623e+001, 9.547897470672068e+001, 3.368899999999999e+002}, {9.275669381231643e+001, -7.835129514294606e+001, 9.464612363393020e+001, 3.373900000000000e+002}, {8.982156217702787e+001, -7.682752633542408e+001, 9.382232560288357e+001, 3.378899999999999e+002}, {8.698548803471240e+001, -7.533913420559925e+001, 9.300743549507918e+001, 3.383900000000002e+002}, {8.424476965419449e+001, -7.388521810035263e+001, 9.220131036581432e+001, 3.388900000000000e+002}, {8.159588334052123e+001, -7.246490478296335e+001, 9.140380939689695e+001, 3.393900000000000e+002}, {7.903542293508620e+001, -7.107734622418560e+001, 9.061479382770111e+001, 3.398900000000000e+002}, {7.656012346841582e+001, -6.972171965580679e+001, 8.983412690204482e+001, 3.403900000000001e+002}, {7.416684232872031e+001, -6.839722645285069e+001, 8.906167380618470e+001, 3.408900000000000e+002}, {7.185255755015002e+001, -6.710309157552798e+001, 8.829730161148777e+001, 3.413899999999999e+002}, {6.961436087200231e+001, -6.583856282410744e+001, 8.754087921483382e+001, 3.418899999999998e+002}, {6.744071144058677e+001, -6.460275810641772e+001, 8.679227669290511e+001, 3.423900000000000e+002}, {6.533736714406254e+001, -6.339506228364169e+001, 8.605136666621638e+001, 3.428900000000000e+002}, {6.331010443040145e+001, -6.221493679466649e+001, 8.531802314304329e+001, 3.433900000000000e+002}, {6.136356979419681e+001, -6.104400934164141e+001, 8.458025754148792e+001, 3.438982188586610e+002}, {5.950910769914493e+001, -5.991653377419485e+001, 8.386060214553304e+001, 3.443990577691702e+002}, {5.774213795912628e+001, -5.883273750086214e+001, 8.315996185918330e+001, 3.448915636307768e+002}, {5.605740872805171e+001, -5.779050749965268e+001, 8.247767612460307e+001, 3.453758750902661e+002}, {5.445002414326789e+001, -5.678785488691579e+001, 8.181311338100387e+001, 3.458521284875891e+002}, {5.291543947575813e+001, -5.582290706691298e+001, 8.116566953938508e+001, 3.463204578942522e+002}, {5.144941514216728e+001, -5.489389939019187e+001, 8.053476654316889e+001, 3.467809951510519e+002}, {5.004800476275456e+001, -5.399916819680369e+001, 7.991985102044619e+001, 3.472338699051960e+002}, {4.870752053753856e+001, -5.313714386862832e+001, 7.932039301451977e+001, 3.476792096467971e+002}, {4.742453515559163e+001, -5.230634512701585e+001, 7.873588479339902e+001, 3.481171397447639e+002}, {4.619582778847320e+001, -5.150537243699205e+001, 7.816583972221676e+001, 3.485477834820968e+002}, {4.501839582534991e+001, -5.073290350231906e+001, 7.760979121186911e+001, 3.489712620905890e+002}, {4.388943451850130e+001, -4.998768810694993e+001, 7.706729172077169e+001, 3.493876947849560e+002}, {4.280631164785162e+001, -4.926854332619763e+001, 7.653791181582945e+001, 3.497971987964010e+002}, {4.176655526408100e+001, -4.857434938684333e+001, 7.602123928892645e+001, 3.501998894056100e+002}, {4.076786327604572e+001, -4.790404616352511e+001, 7.551687832371779e+001, 3.505958799752079e+002}, {3.980805733673829e+001, -4.725662870019470e+001, 7.502444870357830e+001, 3.509852819816741e+002}, {3.888510061148567e+001, -4.663114456970701e+001, 7.454358507290264e+001, 3.513682050467190e+002}, {3.799708329453679e+001, -4.602669045951641e+001, 7.407393623185125e+001, 3.517447569681489e+002}, {3.714219720456370e+001, -4.544240891400779e+001, 7.361516447128999e+001, 3.521150437502121e+002}, {3.631874498673679e+001, -4.487748602903891e+001, 7.316694494766297e+001, 3.524791696334351e+002}, {3.552513982192850e+001, -4.433114893405369e+001, 7.272896508813832e+001, 3.528372371239739e+002}, {3.475986373173623e+001, -4.380266269275900e+001, 7.230092402560834e+001, 3.531893470224679e+002}, {3.402150381919826e+001, -4.329132906864620e+001, 7.188253207215109e+001, 3.535355984524188e+002}, {3.330871802177212e+001, -4.279648372667590e+001, 7.147351021030326e+001, 3.538760888880901e+002}, {3.262024083566979e+001, -4.231749458406720e+001, 7.107358961785731e+001, 3.542109141819540e+002}, {3.195487579527898e+001, -4.185375988374079e+001, 7.068251121409816e+001, 3.545401685916690e+002}, {3.131148674881513e+001, -4.140470653297403e+001, 7.030002523183052e+001, 3.548639448066199e+002}, {3.068901235671067e+001, -4.096978874137064e+001, 6.992589080997549e+001, 3.551823339740079e+002}, {3.008642901657157e+001, -4.054848592928184e+001, 6.955987560500161e+001, 3.554954257245101e+002}, {2.950278142274513e+001, -4.014030203900851e+001, 6.920175542777736e+001, 3.558033081975119e+002}, {2.893714926772632e+001, -3.974476359913557e+001, 6.885131389159386e+001, 3.561060680659211e+002}, {2.838867997995928e+001, -3.936141920744957e+001, 6.850834208489954e+001, 3.564037905605628e+002}, {2.785653948598558e+001, -3.898983744199484e+001, 6.817263825199957e+001, 3.566965594941810e+002}, {2.733994891367559e+001, -3.862960663002127e+001, 6.784400749871969e+001, 3.569844572850220e+002}, {2.683817826345350e+001, -3.828033360882353e+001, 6.752226150569864e+001, 3.572675649800471e+002}, {2.635049716009461e+001, -3.794164206628756e+001, 6.720721825603974e+001, 3.575459622777420e+002}, {2.587625347730658e+001, -3.761317301384136e+001, 6.689870178268673e+001, 3.578197275505541e+002}, {2.541479976649183e+001, -3.729458259106318e+001, 6.659654191726963e+001, 3.580889378669559e+002}, {2.496551525675362e+001, -3.698554184967730e+001, 6.630057406013974e+001, 3.583536690131409e+002}, {2.452710525070210e+001, -3.668572794713256e+001, 6.601063893678912e+001, 3.586139955143569e+002}, {2.410360417001564e+001, -3.639490086428960e+001, 6.572658259420633e+001, 3.588699906558899e+002}, {2.368210844621412e+001, -3.611258607414033e+001, 6.544825524888769e+001, 3.591217265036901e+002}, {2.327622138968802e+001, -3.583876497287966e+001, 6.517551349931662e+001, 3.593692739246600e+002}, {2.285763234098723e+001, -3.557279939313234e+001, 6.490821598447343e+001, 3.596127026066080e+002}, {2.239716065038261e+001, -3.531437500835054e+001, 6.464622739113473e+001, 3.598520810778630e+002}, {2.261286818614434e+001, -3.507154486626255e+001, 6.438943523372159e+001, 3.600874767265670e+002}, {2.115907150454445e+001, -3.481174562600655e+001, 6.413762093896931e+001, 3.603189558196478e+002}, {2.159545498645849e+001, -3.459060253341110e+001, 6.389086012794930e+001, 3.605465835214680e+002}, {2.102568471888965e+001, -3.435729682605923e+001, 6.364880929050617e+001, 3.607704239121769e+002}, {2.067684918221991e+001, -3.413518689782911e+001, 6.341147902075354e+001, 3.609905400057469e+002}, {2.033460501975768e+001, -3.391960694365179e+001, 6.317872830742419e+001, 3.612069937677100e+002}, {1.999858565664685e+001, -3.371036657708588e+001, 6.295044799142224e+001, 3.614198461326062e+002}, {1.966841892720444e+001, -3.350728276179628e+001, 6.272653214279693e+001, 3.616291570211340e+002}, {1.934374242040421e+001, -3.331017987964569e+001, 6.250687794476657e+001, 3.618349853570229e+002}, {1.902410836898150e+001, -3.311888848756182e+001, 6.229138557796490e+001, 3.620373890836158e+002}, {1.870908669701693e+001, -3.293324699210280e+001, 6.207995811721918e+001, 3.622364251801869e+002}, {1.839809418279004e+001, -3.275309960418061e+001, 6.187250141735893e+001, 3.624321496779760e+002}, {1.808335136498075e+001, -3.257823292493270e+001, 6.166892388130226e+001, 3.626246176759661e+002}, {1.776248672564552e+001, -3.240847057790838e+001, 6.146913652788158e+001, 3.628138833563929e+002}, {1.743245820179932e+001, -3.224362752288467e+001, 6.127305262354953e+001, 3.629999999999999e+002}, {1.708940169785534e+001, -3.206925059654591e+001, 6.106275077012659e+001, 3.631999999999998e+002}, {1.727072380355401e+001, -3.190283448428098e+001, 6.085287055989324e+001, 3.634000000000002e+002}, {1.576467522519720e+001, -3.171731020772320e+001, 6.064333258200428e+001, 3.636000000000000e+002}, {1.639061885722927e+001, -3.156729873543955e+001, 6.043429544463123e+001, 3.638000000000000e+002}, {1.577581090427984e+001, -3.139814584618751e+001, 6.022552408630595e+001, 3.640000000000001e+002}, {1.543114797734251e+001, -3.123725025299474e+001, 6.001714130776343e+001, 3.642000000000000e+002}, {1.508206908869408e+001, -3.107920267388456e+001, 5.980909872430626e+001, 3.644000000000001e+002}, {1.472796538577313e+001, -3.092406207912170e+001, 5.960138055055287e+001, 3.646000000000000e+002}, {1.436821268934422e+001, -3.077189392866886e+001, 5.939397035423640e+001, 3.647999999999999e+002}, {1.400211036945577e+001, -3.062276983306427e+001, 5.918685099995032e+001, 3.649999999999999e+002}, {1.362888293958223e+001, -3.047676843426775e+001, 5.898000459303437e+001, 3.652000000000000e+002}, {1.324767438217522e+001, -3.033397615333581e+001, 5.877341241576365e+001, 3.654000000000000e+002}, {1.285753973119515e+001, -3.019448807281502e+001, 5.856705485715880e+001, 3.656000000000000e+002}, {1.245741665183489e+001, -3.005840871434270e+001, 5.836091133461882e+001, 3.657999999999999e+002}, {1.204608676545747e+001, -2.992585293758458e+001, 5.815496020672768e+001, 3.660000000000001e+002}, {1.162216928497416e+001, -2.979694764877785e+001, 5.794917867743944e+001, 3.661999999999999e+002}, {1.118402249211580e+001, -2.967183260501715e+001, 5.774354268368643e+001, 3.663999999999999e+002}, {1.072966463770549e+001, -2.955066252866774e+001, 5.753802677196912e+001, 3.666000000000000e+002}, {1.025660267328408e+001, -2.943360905079618e+001, 5.733260395197137e+001, 3.668000000000001e+002}, {9.761584790130781e+000, -2.932086371792684e+001, 5.712724552477193e+001, 3.670000000000000e+002}, {9.240175660907763e+000, -2.921264175923935e+001, 5.692192087203171e+001, 3.672000000000001e+002}, {8.686057647179084e+000, -2.910918755650368e+001, 5.671659719110131e+001, 3.673999999999998e+002}, {8.778234040826405e+000, -2.901811342102016e+001, 5.651125648369048e+001, 3.675999999999999e+002}, {6.691629539833685e+000, -2.890243901808726e+001, 5.630576331567251e+001, 3.677999999999998e+002}, {7.290370456106036e+000, -2.883742522272521e+001, 5.610033249324230e+001, 3.679999999999999e+002}, {6.272233268873873e+000, -2.874834766050094e+001, 5.589459420906075e+001, 3.682000000000001e+002}, {5.558422478759069e+000, -2.867317963045713e+001, 5.568870519404406e+001, 3.683999999999999e+002}, {4.792961753723030e+000, -2.860487749510562e+001, 5.548256579547770e+001, 3.685999999999999e+002}, {3.968238215541748e+000, -2.854396871234190e+001, 5.527612109369429e+001, 3.687999999999998e+002}, {3.075217193011998e+000, -2.849105390721893e+001, 5.506931114354074e+001, 3.690000000000001e+002}, {2.103060257219561e+000, -2.844681941762896e+001, 5.486207027055930e+001, 3.692000000000001e+002}, {1.038687074059420e+000, -2.841205356272049e+001, 5.465432623928827e+001, 3.694000000000000e+002}, {-1.339999048012931e-001, -2.838766434327866e+001, 5.444599925272737e+001, 3.696000000000000e+002}, {-1.435063121990103e+000, -2.837470417780358e+001, 5.423700075757526e+001, 3.698000000000000e+002}, {-2.890489367153537e+000, -2.837439436195233e+001, 5.402723196589630e+001, 3.700000000000001e+002}, {-4.535509502325382e+000, -2.838815668289008e+001, 5.381658203375368e+001, 3.702000000000000e+002}, {-6.421717110684526e+000, -2.841764003690377e+001, 5.360492570392729e+001, 3.704000000000001e+002}, {-8.029122269107946e+000, -2.847153711327977e+001, 5.339213717659963e+001, 3.705999999999999e+002}, {-1.150718130865884e+001, -2.852257064861725e+001, 5.317796925804201e+001, 3.708000000000000e+002}, {-1.357244772388494e+001, -2.862919062243810e+001, 5.296244024140553e+001, 3.710000000000001e+002}, {-1.717479675911297e+001, -2.874214450332900e+001, 5.274508640521118e+001, 3.711999999999999e+002}, {-2.123801901866396e+001, -2.889134404513250e+001, 5.252580043857113e+001, 3.714000000000001e+002}, {-2.619511142469762e+001, -2.907738790923127e+001, 5.230424646131181e+001, 3.716000000000000e+002}, {-3.236073058364635e+001, -2.930723253179286e+001, 5.208006194513168e+001, 3.717999999999999e+002}, {-4.022068942597398e+001, -2.958957539713408e+001, 5.185280422638054e+001, 3.720000000000000e+002}, {-5.027803753687388e+001, -2.993866920656852e+001, 5.162193160829787e+001, 3.721999999999999e+002}, {-6.436974496162212e+001, -3.035919422556437e+001, 5.138671614373845e+001, 3.724000000000002e+002}, {-8.360686397848249e+001, -3.088595220083816e+001, 5.114631483263473e+001, 3.726000000000000e+002}, {-1.100834389241486e+002, -3.155568586681570e+001, 5.089950843562296e+001, 3.728000000000001e+002}, {-1.472841062914758e+002, -3.245099931262556e+001, 5.064476196619719e+001, 3.730000000000000e+002}, {-2.021579877018242e+002, -3.370859211762419e+001, 5.037989000027908e+001, 3.732000000000001e+002}, {-2.897791479469411e+002, -3.471839082840622e+001, 5.017174758533965e+001, 3.733500000000000e+002}, {-4.370418417975305e+002, -3.595572803188362e+001, 4.995306621058514e+001, 3.735000000000001e+002}, {-6.808564431390644e+002, -3.803753250618856e+001, 4.971098502785520e+001, 3.736561480000000e+002}, {-1.079182893585270e+003, -4.178442673431590e+001, 4.945933042367851e+001, 3.738058199999999e+002}, {-1.681835788759286e+003, -4.662329946397544e+001, 4.925008325739737e+001, 3.739180740000000e+002}, {-2.582563222934587e+003, -5.134362044917342e+001, 4.909428751215069e+001, 3.739929099999999e+002}, {-4.106828068576426e+003, -5.577687841590389e+001, 4.896368572381315e+001, 3.740490370000001e+002}, {-5.107591845246606e+003, -6.580440293493800e+001, 4.883933501194808e+001, 3.740976804000001e+002}, {-9.771765920564003e+003, -7.560712452291881e+001, 4.870422526751046e+001, 3.741425820000000e+002}, {-8.755497642709564e+003, -8.742278426612906e+001, 4.864183118847620e+001, 3.741612909999998e+002}, {-9.815118604201449e+003, -9.075723406919788e+001, 4.861419440063168e+001, 3.741687745999999e+002}, {-1.598688798560589e+004, -9.374464327459545e+001, 4.857016747633733e+001, 3.741800000000001e+002}, {-1.920875724714269e+004, -1.019624881152255e+002, 4.852930969549549e+001, 3.741899999999998e+002}, {-1.636215913588905e+005, -7.659891465615223e+001, 4.848251558276280e+001, 3.741999999999999e+002}, {-1.286897688417260e+006, 1.186575607887113e+001, 4.845730381784014e+001, 3.742050000000000e+002}, {8.838952576730371e+007, -4.445028478910222e+003, 4.849792832024318e+001, 3.742074999999999e+002}};
        constant Real[Ninterval, 4] ptcoef = {{0.786553610980441, 0.073986317581331, 0.004029258982827, 0.000095977628483}, {0.804804842020852, 0.075513868977931, 0.004128852620146, 0.000098693577477}, {0.826483769874362, 0.076781180144682, 0.004214141994619, 0.000101026437564}, {0.852226781522051, 0.079838214007716, 0.004413722019840, 0.000106516958458}, {0.878734233127809, 0.083011720420838, 0.004622630654312, 0.000112305941336}, {0.905918938100631, 0.086306406812799, 0.004841289235230, 0.000118409424841}, {0.933873884997079, 0.089726230581858, 0.005070153132594, 0.000124844693392}, {0.962577238137988, 0.093275895699320, 0.005309687431187, 0.000131629706447}, {0.992070856558737, 0.096959926219076, 0.005560383405310, 0.000138783529530}, {1.022340779604642, 0.100783239811202, 0.005822748456693, 0.000146326107976}, {1.053408429650162, 0.104750722856737, 0.006097318061549, 0.000154278611181}, {1.085334401770672, 0.108867191838833, 0.006384645627683, 0.000162663184614}, {1.118066059070003, 0.113138431880924, 0.006685319756183, 0.000171503506240}, {1.151675830716446, 0.117569521390065, 0.006999950677252, 0.000180824369499}, {1.186152658253827, 0.122166182229820, 0.007329167943923, 0.000190651710758}, {1.221529325860328, 0.126934198374854, 0.007673642176701, 0.000201013248529}, {1.257786462283640, 0.131879624460586, 0.008034059624186, 0.000211937791220}, {1.294812956053328, 0.137009131430195, 0.008411150309429, 0.000223456117470}, {1.332050814897662, 0.142331923230738, 0.008805666323160, 0.000235600452886}, {1.369397755526468, 0.147855890364265, 0.009218393425041, 0.000248404751272}, {1.406816638143677, 0.153467769902578, 0.009641353399052, 0.000261628465319}, {1.444356079648527, 0.158956456158090, 0.010059134663008, 0.000274788118369}, {1.482496486580440, 0.164593208638941, 0.010491786565620, 0.000288515879081}, {1.521219525653765, 0.170380509776158, 0.010939707938024, 0.000302831882662}, {1.560542804418774, 0.176320624445618, 0.011403304275305, 0.000317756801604}, {1.600437543776211, 0.182416074041046, 0.011882987053658, 0.000333311854289}, {1.640936272107287, 0.188669018847875, 0.012379174551977, 0.000349518813327}, {1.681996400839473, 0.195082050643184, 0.012892290728097, 0.000366400014143}, {1.723656934024151, 0.201657263429387, 0.013422766529104, 0.000383978363163}, {1.765877582075693, 0.208397225399801, 0.013971038423670, 0.000402277346453}, {1.808673898875252, 0.215304119402563, 0.014537549727065, 0.000421321037758}, {1.852056948436205, 0.222380208630733, 0.015122749655067, 0.000441134107092}, {1.895976234842191, 0.229628047386924, 0.015727093310843, 0.000461741828759}, {1.940485837249081, 0.237049568378319, 0.016351042726376, 0.000483170089585}, {1.985535349395222, 0.244647339911478, 0.016995065014999, 0.000505445397324}, {2.031148612115400, 0.252423430625727, 0.017659634148470, 0.000528594888329}, {2.077302947391230, 0.260380191193645, 0.018345229588173, 0.000552646335999}, {2.124013371424510, 0.268519730288320, 0.019052337090330, 0.000577628158512}, {2.171233734600246, 0.276844461955414, 0.019781447908203, 0.000603569426887}, {2.219036879803186, 0.285356138275785, 0.020533059923821, 0.000630499872733}, {2.267319033113332, 0.294057511278019, 0.021307675460814, 0.000658449896208}, {2.316150758140982, 0.302950218193697, 0.022105804263753, 0.000687450573315}, {2.365499765564834, 0.312036591386713, 0.022927960463078, 0.000717533664244}, {2.415341254676885, 0.321318803772746, 0.023774664275808, 0.000748731620229}, {2.465702228666255, 0.330798721201480, 0.024646441777799, 0.000781077591413}, {2.516566337049237, 0.340478491545039, 0.025543824020845, 0.000814605434250}, {2.567900988365430, 0.350360262236221, 0.026467347650949, 0.000849349718558}, {2.619736333280843, 0.360445798567964, 0.027417555102175, 0.000885345734951}, {2.672048765758521, 0.370737202172238, 0.028394993495523, 0.000922629502038}, {2.724826587346702, 0.381236432325441, 0.029400215484282, 0.000961237773163}, {2.778063214522083, 0.391945380908662, 0.030433778913126, 0.001001208043570}, {2.831766036616288, 0.402865857403683, 0.031496246697041, 0.001042578557225}, {2.885908682390723, 0.413999836240006, 0.032588186505531, 0.001085388313488}, {2.940499375779766, 0.425349047466033, 0.033710171266653, 0.001129677073668}, {2.995518695471547, 0.436915364909673, 0.034862778477020, 0.001175485367706}, {3.050956509836956, 0.448700547312638, 0.036046590569830, 0.001222854500461}, {3.106820822095549, 0.460706240111525, 0.037262194688661, 0.001271826558129}, {3.163093309167843, 0.472934226350843, 0.038510182323064, 0.001322444414376}, {3.219746757773316, 0.485386251264479, 0.039791149597540, 0.001374751736327}, {3.276823378325092, 0.498063672720634, 0.041105697402525, 0.001428792990668}, {3.334256093194325, 0.510968455863627, 0.042454430092932, 0.001484613449495}, {3.392066452222572, 0.524101941700382, 0.043837957232333, 0.001542259195759}, {3.450267320729766, 0.537465592429579, 0.045256892052289, 0.001601777129328}, {3.508789067634338, 0.551061252504325, 0.046711851466142, 0.001663214972080}, {3.567677958706406, 0.564890022697125, 0.048203457357055, 0.001726621273310}, {3.626905151411771, 0.578953490302442, 0.049732334529406, 0.001792045415320}, {3.686462083749643, 0.593253045708126, 0.051299111947236, 0.001859537618008}, {3.746327635912001, 0.607790095318200, 0.052904422199431, 0.001929148944129}, {3.806521328539690, 0.622565797973926, 0.054548901665213, 0.002000931304129}, {3.867009632214078, 0.637581603413359, 0.056233189735702, 0.002074937460871}, {3.927782116924284, 0.652838734378914, 0.057957929579066, 0.002151221034049}, {3.988852365666806, 0.668338275550249, 0.059723767660887, 0.002229836504888}, {4.050192516216124, 0.684081532186310, 0.061531353270102, 0.002310839220353}, {4.111789278896813, 0.700069639448781, 0.063381339086498, 0.002394285397225}, {4.173651666210960, 0.716303589828222, 0.065274380831822, 0.002480232126358}, {4.235756676172436, 0.732784540271985, 0.067211136814012, 0.002568737376544}, {4.298085502042800, 0.749513538707623, 0.069192268324502, 0.002659859998184}, {4.360671661599279, 0.766491347937005, 0.071218439541212, 0.002753659727090}, {4.423438949366865, 0.783719304937090, 0.073290316468236, 0.002850197187960}, {4.486438551377852, 0.801197930575199, 0.075408568729196, 0.002949533897579}, {4.549635083346689, 0.818928297908401, 0.077573867304666, 0.003051732268502}, {4.613005208765145, 0.836911289865486, 0.079786885896258, 0.003156855611648}, {4.676574403683728, 0.855147500168120, 0.082048300614339, 0.003264968139624}, {4.740297050704474, 0.873637909632577, 0.084358789038483, 0.003376134969544}, {4.804190690507514, 0.892383063052667, 0.086719031350494, 0.003490422125512}, {4.868235121873531, 0.911383726822147, 0.089129709147211, 0.003607896541475}, {4.932419559693664, 0.930640548615199, 0.091591505998932, 0.003728626063414}, {4.996730104947655, 0.950154145354210, 0.094105107137860, 0.003852679451797}, {5.061189204396221, 0.969924932558094, 0.096671199500138, 0.003980126383733}, {5.125721774355183, 0.989953806668835, 0.099290470904280, 0.004111037454977}, {5.190399776670092, 1.010240773734387, 0.101963611687264, 0.004245484181735}, {5.255151436743157, 1.030786704419071, 0.104691312074425, 0.004383539002791}, {5.319997881680896, 1.051591809289908, 0.107474264565075, 0.004525275280559}, {5.384915083275455, 1.072656564602096, 0.110313162173150, 0.004670767303162}, {5.449915764807296, 1.093981202980726, 0.113208699187816, 0.004820090285488}, {5.514961084515890, 1.115566214746911, 0.116161570403907, 0.004973320370546}, {5.580069995165794, 1.137411702858405, 0.119172471924876, 0.005130534630481}, {5.645214917013816, 1.159518032770311, 0.122242100064668, 0.005291811067763}, {5.710397135075893, 1.181885343682942, 0.125371152069178, 0.005457228615834}, {5.775601905695197, 1.204513844192818, 0.128560325505952, 0.005626867140010}, {5.840835782784636, 1.227403598430226, 0.131810318497041, 0.005800807438005}, {5.906059494800497, 1.250554880485784, 0.135121829207348, 0.005979131240455}, {5.971301283679346, 1.273967536407251, 0.138495556564960, 0.006161921211248}, {6.036525184627135, 1.297641782816887, 0.141932199008561, 0.006349260947941}, {6.101742108554982, 1.321577504276583, 0.145432455530469, 0.006541234981608}, {6.166925944102037, 1.345774777484733, 0.148997024740760, 0.006737928777106}, {6.232083718610021, 1.370233441641100, 0.152626605432942, 0.006939428732812}, {6.297207184980851, 1.394953425336797, 0.156321895953925, 0.007145822180530}, {6.362277871137049, 1.419934665093411, 0.160083594379107, 0.007357197385014}, {6.427293032446297, 1.445176960347283, 0.163912398571472, 0.007573643543593}, {6.492248494825507, 1.470680105264633, 0.167809005838224, 0.007795250785685}, {6.557139915959902, 1.496443877947695, 0.171774112913575, 0.008022110172037}, {6.621948550316446, 1.522468096828492, 0.175808415864079, 0.008254313693933}, {6.686677764125916, 1.548752416568682, 0.179912610249625, 0.008491954272338}, {6.751318489523362, 1.575296555162762, 0.184087390680888, 0.008735125756982}, {6.815849328693517, 1.602100243679581, 0.188333450922353, 0.008983922925176}, {6.880294415977666, 1.629162944906189, 0.192651484067798, 0.009238441480674}, {6.944622895829550, 1.656484438791168, 0.197042181708333, 0.009498778052406}, {7.008826622849972, 1.684064290571623, 0.201506234762844, 0.009765030192908}, {7.072914856384893, 1.711901963064116, 0.206044333006924, 0.010037296377091}, {7.136877800207411, 1.739997032582018, 0.210657164762212, 0.010315676000575}, {7.200691051456989, 1.768349094724015, 0.215345417113497, 0.010600269377903}, {7.264377479650122, 1.796957457787893, 0.220109776102959, 0.010891177740886}, {7.327906141296870, 1.825821737448506, 0.224950925854684, 0.011188503236811}, {7.391293625276115, 1.854941234807067, 0.229869549469729, 0.011492348926300}, {7.454508371077754, 1.884315510467502, 0.234866328098409, 0.011802818781492}, {7.517569054229431, 1.913943797155383, 0.239941941727175, 0.012120017683787}, {7.580459415407503, 1.943825542494100, 0.245097068267364, 0.012444051421826}, {7.643171908708069, 1.973960094001435, 0.250332384098234, 0.012775026689016}, {7.705707996362623, 2.004346732841488, 0.255648563847159, 0.013113051081326}, {7.768059519623182, 2.034984784600641, 0.261046280194050, 0.013458233094852}, {7.830220904482947, 2.065873531807347, 0.266526203982097, 0.013810682123266}, {7.892188564751971, 2.097012226673120, 0.272089004114447, 0.014170508455295}, {7.953965487964128, 2.128400080683881, 0.277735347500195, 0.014537823272081}, {8.015523649434170, 2.160036441620303, 0.283465898846533, 0.014912738644454}, {8.076895462829125, 2.191920302660508, 0.289281321183433, 0.015295367530163}, {8.138044320167687, 2.224051063573816, 0.295182274777506, 0.015685823771147}, {8.198976439581513, 2.256427805724049, 0.301169418248213, 0.016084222090402}, {8.259707938402711, 2.289049595720479, 0.307243407828786, 0.016490678089312}, {8.320192607430975, 2.321915814614592, 0.313404897121920, 0.016905308244463}, {8.380470028810771, 2.355025314333463, 0.319654538135147, 0.017328229904537}, {8.440522518139051, 2.388377335023929, 0.325992979815505, 0.017759561287442}, {8.500322217978708, 2.421971086721057, 0.332420868948437, 0.018199421476779}, {8.559907453252212, 2.455805402507063, 0.338938850295470, 0.018647930418875}, {8.619247679352942, 2.489879545144593, 0.345547565459140, 0.019105208919565}, {8.678350338810887, 2.524192502690502, 0.352247654043904, 0.019571378640574}, {8.737200358793032, 2.558743383285288, 0.359039752909466, 0.020046562096445}, {8.795818738593697, 2.593531090067868, 0.365924496580261, 0.020530882651035}, {8.854183207405342, 2.628554783799626, 0.372902516607536, 0.021024464514114}, {8.912292637601727, 2.663813451198047, 0.379974442256104, 0.021527432737748}, {8.970164816146156, 2.699305998657141, 0.387140900144449, 0.022039913212986}, {9.027764581740044, 2.735031616100854, 0.394402513874834, 0.022562032666223}, {9.085123127706376, 2.770989086540987, 0.401759904933333, 0.023093918655552}, {9.142217145365851, 2.807177536491606, 0.409213691526334, 0.023635699567366}, {9.199057526581893, 2.843595861945240, 0.416764489523273, 0.024187504612462}, {9.255634156711325, 2.880243086753219, 0.424412911826686, 0.024749463822588}, {9.311949978406053, 2.917118141884187, 0.432159568722993, 0.025321708046652}, {9.368005011068972, 2.954219981819242, 0.440005067647167, 0.025904368947099}, {9.423798091407672, 2.991547565855996, 0.447950013236107, 0.026497578996133}, {9.479322284803363, 3.029099871775003, 0.455995007318796, 0.027101471471982}, {9.534586801784080, 3.066875784845970, 0.464140649013009, 0.027716180455168}, {9.589583111450359, 3.104874298352855, 0.472387534443667, 0.028341840824758}, {9.644318672936832, 3.143094307303569, 0.480736257056818, 0.028978588254514}, {9.698780209475908, 3.181534819712769, 0.489187407311572, 0.029626559209175}, {9.752988166020387, 3.220194655215614, 0.497741573065422, 0.030285890940625}, {9.806914003161001, 3.259072911744028, 0.506399338956173, 0.030956721484151}, {9.860588583346717, 3.298168334723664, 0.515161287270285, 0.031639189654531}, {9.913991884695866, 3.337479997260490, 0.524027996925208, 0.032333435042396}, {9.967130228266164, 3.377006786609229, 0.533000044348281, 0.033039598010199}, {10.020005553537565, 3.416747632921908, 0.542078003011462, 0.033757819688613}, {10.072615604413384, 3.456701490940380, 0.551262443523183, 0.034488241972654}, {10.124967253171006, 3.496867269778813, 0.560553933719096, 0.035231007517885}, {10.177046563723209, 3.537243989500037, 0.569953038482225, 0.035986259736650}, {10.228875748201631, 3.577830469437759, 0.579460320131242, 0.036754142794284}, {10.280445312958783, 3.618625745018311, 0.589076337835556, 0.037534801605407}, {10.331749969404274, 3.659628796758036, 0.598801648151134, 0.038328381830030}, {10.382798930610805, 3.700838525857476, 0.608636804932991, 0.039135029869976}, {10.433600732654398, 3.742253873156421, 0.618582359146748, 0.039954892865162}, {10.484133544637205, 3.783873935436875, 0.628638858864915, 0.040788118689806}, {10.534436373982112, 3.825697478101712, 0.638806849846641, 0.041634855948772}, {10.584473346798614, 3.867723723077579, 0.649086874493393, 0.042495253974029}, {10.634263232829829, 3.909951529450528, 0.659479473130387, 0.043369462820802}, {10.683818854528312, 3.952379855381508, 0.669985183113258, 0.044257633264281}, {10.733119485900088, 3.995007844903166, 0.680604538962431, 0.045159916795814}, {10.782187429189030, 4.037834386520546, 0.691338072953496, 0.046076465619372}, {10.831022040856825, 4.080858550106148, 0.702186314429305, 0.047007432648198}, {10.879606700772708, 4.124079466970156, 0.713149790186699, 0.047952971501152}, {10.927971273840956, 4.167496015473688, 0.724229024778087, 0.048913236499386}, {10.976106518013856, 4.211107350457580, 0.735424539778802, 0.049888382663002}, {11.024007342655198, 4.254912573065150, 0.746736854471000, 0.050878565707475}, {11.071698030565202, 4.298910646304973, 0.758166485812413, 0.051883942040502}, {11.119157876862268, 4.343100810847359, 0.769713947959052, 0.052904668758722}, {11.166410089929238, 4.387482048337676, 0.781379753083690, 0.053940903644358}, {11.213444532743557, 4.432053567134218, 0.793164410642923, 0.054992805162211}, {11.260274048538790, 4.476814442825981, 0.805068428005676, 0.056060532456357}, {11.306896593576145, 4.521763862164671, 0.817092310086160, 0.057144245347173}, {11.353322382593026, 4.566900954540457, 0.829236559658240, 0.058244104328223}, {11.399547252019955, 4.612224948003909, 0.841501677172442, 0.059360270563319}, {11.445587422729814, 4.657734972153432, 0.853888161066984, 0.060492905883569}, {11.491426255466461, 4.703430353321102, 0.866396507423689, 0.061642172784548}, {11.537096574565506, 4.749310152874991, 0.879027210611636, 0.062808234423435}, {11.582580184110803, 4.795373768279950, 0.891780762485170, 0.063991254616361}, {11.627892085691373, 4.841620390979311, 0.904657653309611, 0.065191397835530}, {11.673035757508945, 4.888049318737771, 0.917658371228776, 0.066408829206806}, {11.718011197256940, 4.934659877239165, 0.930783402516026, 0.067643714507045}, {11.762834239165681, 4.981451328650070, 0.944033231699365, 0.068896220161631}, {11.807497796646979, 5.028423090503301, 0.957408341337984, 0.070166513242073}, {11.852019325559198, 5.075574453530177, 0.970909212484283, 0.071454761463596}, {11.896393217304059, 5.122904870887758, 0.984536324306399, 0.072761133182957}, {11.940631343927475, 5.170413703210555, 0.998290154543469, 0.074085797396168}, {11.984740791999011, 5.218100377914232, 1.012171179272453, 0.075428923736457}, {12.028718632188408, 5.265964390903449, 1.026179873042613, 0.076790682472158}, {12.072586720369342, 5.314005127040347, 1.040316709128782, 0.078171244504771}, {12.116334688142151, 5.362222187928859, 1.054582159164051, 0.079570781367113}, {12.159978238056379, 5.410615029139965, 1.068976693754340, 0.080989465221439}, {12.203519113274206, 5.459183225116353, 1.083500782085046, 0.082427468857874}, {12.246973860534712, 5.507926297975900, 1.098154892266446, 0.083884965692685}, {12.290333050611526, 5.556843940294234, 1.112939491127611, 0.085362129766796}, {12.333616287920439, 5.605935690440304, 1.127855044729348, 0.086859135744312}, {12.376827944463761, 5.655201226431555, 1.142902017949415, 0.088376158911288}, {12.419970336161901, 5.704640252028573, 1.158080874821313, 0.089913375174364}, {12.463057868403830, 5.754252431854722, 1.173392078642345, 0.091470961059675}, {12.506093470453104, 5.804037534286757, 1.188836091849739, 0.093049093711825}, {12.549084484576550, 5.853995321776277, 1.204413376293114, 0.094647950892901}, {12.592041013933368, 5.904125575040546, 1.220124393236732, 0.096267710981682}, {12.634969844799203, 5.954428128657826, 1.235969603401023, 0.097908552972900}, {12.677879333523451, 6.004902838096064, 1.251949467106324, 0.099570656476601}, {12.720778138710589, 6.055549591140720, 1.268064444332499, 0.101254201717648}, {12.763677927417257, 6.106368298174700, 1.284314994816732, 0.102959369535330}, {12.806573690779381, 6.157358983209033, 1.300701578059685, 0.104686341383082}, {12.849495734467585, 6.208521508652972, 1.317224653725359, 0.106435299328330}, {12.892432395176041, 6.259856026588764, 1.333884681122604, 0.108206426052510}, {12.935407425706483, 6.311362500376266, 1.350682120030723, 0.109999904851008}, {12.978425056907367, 6.363041066848615, 1.367617430188535, 0.111815919633535}, {13.021491468318452, 6.414891872038170, 1.384691071730111, 0.113654654924339}, {13.064617370780807, 6.466915068601184, 1.401903505228866, 0.115516295862749}, {13.107822326512034, 6.519110816528661, 1.419255191755479, 0.117401028203792}, {13.151102428135824, 6.571479448359150, 1.436746592829065, 0.119309038318884}, {13.194475889857047, 6.624021192307916, 1.454378170915361, 0.121240513196689}, {13.237950456876277, 6.676736391559659, 1.472150389150132, 0.123195640444242}, {13.281543801507878, 6.729625380540582, 1.490063711663251, 0.125174608288032}, {13.325252146812181, 6.782688650527698, 1.508118603496211, 0.127177605575334}, {13.369105579817697, 6.835926541870936, 1.526315531104156, 0.129204821775656}, {13.413096345193450, 6.889339667250363, 1.544654961877694, 0.131256446982442}, {13.457252262710217, 6.942928465635696, 1.563137364930240, 0.133332671914716}, {13.501577668383591, 6.996693581650487, 1.581763210613397, 0.135433687919157}, {13.546081324560269, 7.050635659475550, 1.600532971040650, 0.137559686972068}, {13.590784720119347, 7.104755327585545, 1.619447120138809, 0.139710861681738}, {13.635691247339029, 7.159053373072407, 1.638506133564744, 0.141887405290866}, {13.680822070366473, 7.213530525265771, 1.657710489160656, 0.144089511679116}, {13.726188668268707, 7.268187637271742, 1.677060666796245, 0.146317375365973}, {13.771798297574913, 7.323025625062165, 1.696557148667505, 0.148571191513651}, {13.817664037870237, 7.378045410406364, 1.716200419477746, 0.150851155930304}, {13.863822536457358, 7.433247873019335, 1.735990966547698, 0.153157465073401}, {13.910252564596428, 7.488634271483426, 1.755929279538731, 0.155490316053202}, {13.957000220459447, 7.544205509430398, 1.776015851602120, 0.157849906636426}, {14.004059404788652, 7.599962884516280, 1.796251178364456, 0.160236435250439}, {14.051458652306392, 7.655907531048133, 1.816635759011231, 0.162650100987143}, {14.099212398107770, 7.712040754313932, 1.837170095861397, 0.165091103607519}, {14.147325830763135, 7.768363954548588, 1.857854694766285, 0.167559643546089}, {14.195829055713423, 7.824878452918044, 1.878690065435772, 0.170055921915781}, {14.244733952698137, 7.881585764229075, 1.899676721206714, 0.172580140513001}, {14.294057776086026, 7.938487429382151, 1.920815179546794, 0.175132501822787}, {14.343815086396987, 7.995585069263414, 1.942105962124181, 0.177713209024381}, {14.394032268232515, 8.052880315699051, 1.963549595058805, 0.180322465996985}, {14.444723227764078, 8.110374956342044, 1.985146608928002, 0.182960477325750}, {14.495902989738047, 8.168070830084497, 2.006897539164692, 0.185627448307985}, {14.547604591999210, 8.225969763841938, 2.028802926260754, 0.188323584959739}, {14.599830390476210, 8.284073839908597, 2.050863315657581, 0.191049094022576}, {14.652622712367227, 8.342384989911224, 2.073079258516169, 0.193804182970581}, {14.705976237638581, 8.400905505967810, 2.095451311216376, 0.196589060017819}, {14.759945876695406, 8.459637417523181, 2.117980036449677, 0.199403934125846}, {14.814518646197049, 8.518583246343280, 2.140666002382416, 0.202249015011809}, {14.869750175199750, 8.577745187148663, 2.163509784095016, 0.205124513156487}, {14.925639645519096, 8.637125883725918, 2.186511962629850, 0.208030639813101}, {14.982229688310740, 8.696727802523457, 2.209673126244849, 0.210967607015953}, {15.039533284178862, 8.756553699721664, 2.232993869911483, 0.213935627589852}, {15.097579737480817, 8.816606318080309, 2.256474796094395, 0.216934915159529}, {15.156404639376522, 8.876888492665323, 2.280116514767052, 0.219965684159667}, {15.216017087353354, 8.937403307410889, 2.303919643575055, 0.223028149845113}, {15.276468737781256, 8.998153712061299, 2.327884808606054, 0.226122528301507}, {15.337764957062324, 9.059143049053603, 2.352012643941528, 0.229249036456430}, {15.399953708033358, 9.120374511511402, 2.376303792775960, 0.232407892090670}, {15.463063557119732, 9.181851562515480, 2.400758907024612, 0.235599313850235}, {15.527112665343029, 9.243577821571789, 2.425378647972495, 0.238823521258444}, {15.592155238801022, 9.305556836457487, 2.450163686802990, 0.242080734728696}, {15.658205916070527, 9.367792530614247, 2.475114704329850, 0.245371175577657}, {15.725311601613303, 9.430288756971484, 2.500232392044810, 0.248695066038703}, {15.793507965239648, 9.493049601511352, 2.525517451946825, 0.252052629276138}, {15.862825932729402, 9.556079312219197, 2.550970597127956, 0.255444089399622}, {15.933312197602097, 9.619382197849491, 2.576592552254044, 0.258869671479274}, {16.004995953725416, 9.682962824454661, 2.602384053692833, 0.262329601561279}, {16.077934097300989, 9.746825764001034, 2.628345850269465, 0.265824106683943}, {16.152154592300423, 9.810975932600028, 2.654478703227895, 0.269353414894413}, {16.227712509060744, 9.875418249703314, 2.680783387196289, 0.272917755265819}, {16.304641683475939, 9.940157940705490, 2.707260690167921, 0.276517357915222}, {16.383003793998693, 10.005200254894007, 2.733911414380859, 0.280152454021976}, {16.462838398153838, 10.070550777666666, 2.760736376351767, 0.283823275846862}, {16.544196962610815, 10.136215205238340, 2.787736407777602, 0.287530056751718}, {16.627135322317265, 10.202199422476509, 2.814912355868767, 0.291273031219949}, {16.711706524449259, 10.268509549183181, 2.842265083851243, 0.295052434877599}, {16.797965223152705, 10.335151911490948, 2.869795471594543, 0.298868504515166}, {16.885973077432112, 10.402133026596109, 2.897504416190871, 0.302721478110211}, {16.975789197985613, 10.469459668908286, 2.925392832470037, 0.306611594850730}, {17.067475629391790, 10.537138837988799, 2.953461653692272, 0.310539095159306}, {17.161101261098594, 10.605177754876001, 2.981711832178946, 0.314504220718140}, {17.256727563447523, 10.673583946230488, 3.010144339891328, 0.318507214494939}, {17.354434081476601, 10.742365112825921, 3.038760169297755, 0.322548320769679}, {17.454286888967282, 10.811529328281166, 3.067560333823008, 0.326627785162365}, {17.556361872915055, 10.881084891722661, 3.096545868856221, 0.330745854661660}, {17.660747345879500, 10.951040356809601, 3.125717832414596, 0.334902777654663}, {17.767513251246932, 11.021404704495929, 3.155077305781520, 0.339098803957625}, {17.876753527147354, 11.092187100141274, 3.184625394700079, 0.343334184847757}, {17.988558020215013, 11.163397102005085, 3.214363229894611, 0.347609173096297}, {18.103011475980828, 11.235044648907479, 3.244291968095110, 0.351924023002582}, {18.220219931796926, 11.307139934127857, 3.274412793120600, 0.356278990429461}, {18.340280778986866, 11.379693610445296, 3.304726916567339, 0.360674332839995}, {18.463298080493942, 11.452716694435287, 3.335235579044130, 0.365110309335374}, {18.589380769544157, 11.526220600337037, 3.365940051173917, 0.369587180694344}, {18.718647779688055, 11.600217138800916, 3.396841634701593, 0.374105209414034}, {18.851216237562294, 11.674718604461052, 3.427941663564171, 0.378664659752273}, {18.987207173821421, 11.749737745984419, 3.459241505194474, 0.383265797771458}, {19.126757016277303, 11.825287735512903, 3.490742561796580, 0.387908891384136}, {19.270000698205109, 11.901382297719167, 3.522446271500800, 0.392594210400251}, {19.417078152378568, 11.978035678304561, 3.554354109854081, 0.397322026576132}, {19.568141318232165, 12.055262635750358, 3.586467591264484, 0.402092613665429}, {19.723341784072204, 12.133078538964597, 3.618788270391853, 0.406906247471983}, {19.882852937169588, 12.211499277098069, 3.651317743856020, 0.411763205904696}, {20.046837713802415, 12.290541476213873, 3.684057651628890, 0.416663769034575}, {20.215476229183036, 12.370222346595941, 3.717009679046383, 0.421608219153951}, {20.388962195235955, 12.450559766172979, 3.750175558448313, 0.426596840838175}, {20.567492357396723, 12.531572377068581, 3.783557070998268, 0.431629921009646}, {20.751272461251776, 12.613279573840439, 3.817156048763674, 0.436707749004418}, {20.940523114529331, 12.695701516360126, 3.850974376794579, 0.441830616641583}, {21.135476236295002, 12.778859203541924, 3.885013995236225, 0.446998818295461}, {21.336375814837982, 12.862774515603268, 3.919276901634197, 0.452212650970746}, {21.543467768371034, 12.947470302848114, 3.953765153318787, 0.457472414380798}, {21.757039843293096, 13.032970233643164, 3.988480870150037, 0.462778411029265}, {21.977357441327399, 13.119299219389090, 4.023426236698062, 0.468130946295191}, {22.204737719518644, 13.206483027886401, 4.058603505710217, 0.473530328521646}, {22.439488108542466, 13.294548737309782, 4.094015000456882, 0.478976869108469}, {22.681951162306543, 13.383524517175067, 4.129663118341967, 0.484470882608819}, {22.932481353049372, 13.473439862652034, 4.165550333923462, 0.490012686830258}, {23.191458098134174, 13.564325589516613, 4.201679202567487, 0.495602602940237}, {23.459284149429621, 13.656213940965090, 4.238052364104783, 0.501240955576484}, {23.736388399611037, 13.749138669767067, 4.274672546779892, 0.506928072962465}, {24.024430476092892, 13.843130314827746, 4.311542575726286, 0.512664287028277}, {24.322901123698131, 13.938228205581177, 4.348665367591055, 0.518449933537306}, {24.631006458958126, 14.034476212891667, 4.386043942312380, 0.524285352219018}, {24.947842383313581, 14.133566333461264, 4.424302227413089, 0.530268052279225}, {25.270996534558169, 14.232467569803379, 4.462269952561297, 0.536214887057049}, {25.600402310675729, 14.330975064669545, 4.499865784968846, 0.542112503476356}, {25.936148140993652, 14.429098478934334, 4.537090582375646, 0.547960412315172}, {26.278342961331273, 14.526847123997111, 4.573945317983731, 0.553758163420801}, {26.627081839098057, 14.624230171354640, 4.610431074409271, 0.559505344514097}, {26.982479035852489, 14.721256475920658, 4.646549038424966, 0.565201580011704}, {27.344634819583604, 14.817934767851131, 4.682300495420807, 0.570846529867353}, {27.713671593677677, 14.914273470914237, 4.717686824578717, 0.576439888432411}, {28.089691272397957, 15.010280930582828, 4.752709493736557, 0.581981383336399}, {28.472824854307039, 15.105965165341871, 4.787370055045711, 0.587470774387750}, {28.863193517870002, 15.201334102546408, 4.821670140178958, 0.592907852495206}, {29.260918302887685, 15.296395505394935, 4.855611456225897, 0.598292438610054}, {29.666132958462629, 15.391156926950673, 4.889195781620369, 0.603624382689606}, {30.078982460204806, 15.485625755858862, 4.922424962124026, 0.608903562681821}, {30.499592721193313, 15.579809342711769, 4.955300906990578, 0.614129883531367}, {30.928117472544855, 15.673714785982705, 4.987825585602939, 0.619303276207172}, {31.364710865969087, 15.767349102050195, 5.020001023779209, 0.624423696751405}, {31.809515123102404, 15.860719254916436, 5.051829300512150, 0.629491125349948}, {32.262705236081672, 15.953831979977263, 5.083312544958544, 0.634505565424281}, {32.724447907289544, 16.046694000766831, 5.114452933148136, 0.639467042744666}, {33.194897132039792, 16.139312001629271, 5.145252685204367, 0.644375604564581}, {33.674255019897664, 16.231692414917958, 5.175714062698377, 0.649231318776222}, {34.162693117856691, 16.323841771582561, 5.205839365624563, 0.654034273086982}, {34.660408283882070, 16.415766446679168, 5.235630930162420, 0.658784574216433}, {35.167590811037854, 16.507472809522142, 5.265091126045004, 0.663482347114145}, {35.684449944091867, 16.598967127675568, 5.294222354303641, 0.668127734197491}, {36.211206269977197, 16.690255627599420, 5.323027044937864, 0.672720894609739}, {36.748056707258620, 16.781344600488524, 5.351507654715894, 0.677262003497751}, {37.295248265383698, 16.872240144915182, 5.379666665308135, 0.681751251309287}, {37.852995971145809, 16.962948488394094, 5.407506580997999, 0.686188843109573}, {38.421564488506611, 17.053475690433864, 5.435029927062224, 0.690574997916736}, {39.001171266768537, 17.143827988284571, 5.462239247649191, 0.694909948055925}, {39.592097221535276, 17.234011403696112, 5.489137104365301, 0.699193938531812}, {40.194606324023972, 17.324032074846357, 5.515726074258302, 0.703427226419012}, {40.808938401803530, 17.413896207928254, 5.542008748346981, 0.707610080270340}, {41.435436776864577, 17.503609730835098, 5.567987730243535, 0.711742779542350}, {42.074330693130328, 17.593178988541467, 5.593665634167064, 0.715825614037971}, {42.725940101764373, 17.682610023334441, 5.619045084134955, 0.719858883365846}, {43.390590488356239, 17.771908949085155, 5.644128712167098, 0.723842896416144}, {44.070645121622796, 17.861077799265878, 5.668919158926054, 0.727777970852333}, {44.753184105407975, 17.950153724907196, 5.693419053900836, 0.731664432618789}, {45.474373691042267, 18.039048771108984, 5.717631098338662, 0.735502615463705}, {46.193899163412112, 18.127899243718634, 5.741557851475661, 0.739292860480826}, {46.996029946900421, 18.216517946983732, 5.765202097955019, 0.743035515642772}, {47.975217272766379, 18.304869267108689, 5.788566478022887, 0.746730935410193}, {46.875088973944962, 18.397435734790200, 5.811651899684304, 0.750379480284763}, {51.065991163454697, 18.476818607729076, 5.834469443821195, 0.753981516503356}, {49.333461744266728, 18.573099243289850, 5.857003438946910, 0.757537415856357}, {50.827892820844589, 18.659006028627026, 5.879277141033153, 0.761047552504248}, {51.658864029489543, 18.747265541446783, 5.901281540617321, 0.764512309584712}, {52.507342163173028, 18.835468843734557, 5.923021904004630, 0.767932071140582}, {53.373793178351164, 18.923622470147773, 5.944500865847515, 0.771307225875759}, {54.258789501497823, 19.011732877688232, 5.965721054625218, 0.774638165973891}, {55.162916478482941, 19.099806756700104, 5.986685091803454, 0.777925286781048}, {56.087116564192172, 19.187850477830338, 6.007395591520163, 0.781168986499608}, {57.032398932408853, 19.275871118680147, 6.027855159614455, 0.784369665893400}, {58.000369060093050, 19.363875690575604, 6.048066393969457, 0.787527728003472}, {59.016634487394832, 19.451836730736815, 6.068031895995421, 0.790643577874778}, {60.091857764934808, 19.539739992232406, 6.087754256968488, 0.793717622293261}, {61.240245331030316, 19.627560573034796, 6.107236083235058, 0.796750269533514}, {62.480561298708686, 19.723763384189539, 6.128269590423300, 0.800019963066246}, {61.700811904981713, 19.825193831111250, 6.149405839679586, 0.803300925851243}, {67.613280255293020, 19.914643859675749, 6.170653665148103, 0.806593213984968}, {65.160632861084636, 20.026986310421218, 6.191997393361786, 0.809896884700018}, {67.669033122940235, 20.127441775170933, 6.213461939269268, 0.813211994514738}, {69.158974688263967, 20.233552143778557, 6.235035431887980, 0.816538605363333}, {70.723256826348845, 20.341932263338009, 6.256723667414029, 0.819876776933363}, {72.367614994324157, 20.452692130214967, 6.278529139019087, 0.823226571210103}, {74.098101167452896, 20.565950434907393, 6.300454462542573, 0.826588051543621}, {75.921523426230067, 20.681834359896129, 6.322502386312879, 0.829961282716883}, {77.845391462064271, 20.800480873586661, 6.344675800844324, 0.833346331019091}, {79.877973252120924, 20.922037773350514, 6.366977749927341, 0.836743264324587}, {82.028526688840202, 21.046664524133615, 6.389411442986359, 0.840152152177982}, {84.307359657115470, 21.174533750826306, 6.411980268436992, 0.843573065886157}, {86.726134004965445, 21.305832439883137, 6.434687808816977, 0.847006078617858}, {89.298052645255225, 21.440763800191853, 6.457537857418896, 0.850451265511744}, {92.038364283034596, 21.579548930296038, 6.480534437230531, 0.853908703793904}, {94.965026588148092, 21.722429035678608, 6.503681822191650, 0.857378472906041}, {98.099651271544445, 21.869668254286683, 6.526984561549138, 0.860860654645750}, {101.469212866843510, 22.021556781161863, 6.550447508473278, 0.864355333320771}, {105.108611392323670, 22.178415351638186, 6.574075853974501, 0.867862595919685}, {109.065005486134880, 22.340601002668283, 6.597875168817166, 0.871382532302410}, {109.750042242080070, 22.514375499939113, 6.621849368866829, 0.874915235415403}, {122.164468808740280, 22.670441678991452, 6.646016608190354, 0.878460801538834}, {120.492493879440720, 22.868998811452705, 6.670353260952902, 0.882019330576957}, {127.651716860571530, 23.050715346380404, 6.694905768119855, 0.885590923061522}, {133.417901827374580, 23.246144421955755, 6.719657855196887, 0.889175690895975}, {139.724332490425070, 23.449952674292600, 6.744624090533837, 0.892773744452224}, {146.646067568266890, 23.662885976357035, 6.769813936652642, 0.896385200660068}, {154.272084490525660, 23.885788232412818, 6.795237725626818, 0.900010181731173}, {162.708903919517890, 24.119617882190067, 6.820906777869384, 0.903648815653934}, {172.084864641046120, 24.365469173499047, 6.846833542039902, 0.907301236757100}, {182.556942338038600, 24.624596217120128, 6.873031762800324, 0.910967586354904}, {194.319534415060760, 24.898444814009999, 6.899516681255775, 0.914648013489756}, {207.620595915041970, 25.188686585937077, 6.926305281144004, 0.918342675792510}, {222.788620730249990, 25.497261589113076, 6.953416591526287, 0.922051740486512}, {240.287692571162920, 25.826419168838576, 6.980872074207111, 0.925775385569714}, {256.799890482864780, 26.185216782196846, 7.008693832401432, 0.929513801223933}, {286.851617485832720, 26.548780722767628, 7.036921002156521, 0.933267191524989}, {308.757481229623580, 26.973589993440147, 7.065557892145567, 0.937035776577968}, {342.616615998311150, 27.414357072890436, 7.094674470820509, 0.940819791613239}, {381.652936748389780, 27.901112273738025, 7.124294167964932, 0.944619499388744}, {429.420511597585570, 28.436458075451569, 7.154472608250879, 0.948435182728345}, {488.956374804092890, 29.029524488453990, 7.185270948459126, 0.952267155471084}, {564.913569104604680, 29.691719970341111, 7.216763630423190, 0.956115767213528}, {662.593654854627520, 30.440873759588669, 7.249041648619902, 0.959981411307655}, {798.395259632387020, 31.285522685311911, 7.282225938437964, 0.963864535640684}, {984.858707574925800, 32.266683177856073, 7.316458408944520, 0.967765657677109}, {1240.731677732101800, 33.435525835453468, 7.351939105152393, 0.971685384357021}, {1599.976231716470900, 34.896887003483748, 7.388921872674265, 0.975624447015660}, {2129.911398579198200, 36.824993793515745, 7.427765289272872, 0.979583747790001}, {2973.337742375393600, 38.358182172438710, 7.458590833881339, 0.982567188320956}, {4392.068482735210600, 40.182279884917335, 7.491259295582522, 0.985563285803890}, {6744.182542887068800, 43.093253924408778, 7.527746112509292, 0.988696515392235}, {10586.401080300620000, 48.112109166481709, 7.566030993328872, 0.991714705866086}, {16414.657806854295000, 54.433838984731196, 7.598161821734584, 0.993988954446224}, {25171.400376638227000, 60.516762902335024, 7.622270849789598, 0.995510777346757}, {39924.646532205254000, 66.237216173833630, 7.642602684084054, 0.996655475718842}, {50014.862506044708000, 78.880685852650700, 7.662052700016630, 0.997650144001249}, {94512.909959942233000, 91.545289401322989, 7.683298623696445, 0.998570741182521}, {85789.631197521958000, 106.331709600833660, 7.693152458218265, 0.998955116981272}, {96484.149392753985000, 110.551277570634270, 7.697526525177595, 0.999109011318793}, {155745.736499270020000, 114.483229466648910, 7.704504194463604, 0.999340021303733}, {187336.117221347780000, 124.858748457008220, 7.710990375742048, 0.999545993761771}, {1561441.955568452100000, 93.976965631416093, 7.718432905786972, 0.999752147562480}, {12254471.880229050000000, -14.883471920491438, 7.722448708901911, 0.999855298039152}, {-840729689.033797860000000, 5473.418677569786300, 7.715971952179429, 0.999906892844623}};
        constant Real[Ninterval, 4] dltcoef = {{-7.119202491756649e-001, -2.739402377062618e-001, -1.927672596844717e+000, 3.108234546713844e+000}, {-7.187746754592991e-001, -2.753459585506838e-001, -1.928038361308511e+000, 3.106950869117843e+000}, {-7.267169555982382e-001, -2.765215049130652e-001, -1.928347041349235e+000, 3.105872508783626e+000}, {-7.359380114100472e-001, -2.792612028934110e-001, -1.929054601531175e+000, 3.103417366473884e+000}, {-7.451596984095644e-001, -2.820546952208173e-001, -1.929773991238450e+000, 3.100944868392513e+000}, {-7.543868380432980e-001, -2.849025526846495e-001, -1.930505474697466e+000, 3.098454883787075e+000}, {-7.636195252102033e-001, -2.878055700919983e-001, -1.931249367470398e+000, 3.095947126307967e+000}, {-7.728608352753940e-001, -2.907644204639990e-001, -1.932005961304827e+000, 3.093421411830454e+000}, {-7.821138534967362e-001, -2.937798575560094e-001, -1.932775569789591e+000, 3.090877503680519e+000}, {-7.913807227613616e-001, -2.968525969010559e-001, -1.933558497589903e+000, 3.088315215846500e+000}, {-8.006644142147472e-001, -2.999834307075162e-001, -1.934355071871375e+000, 3.085734309672668e+000}, {-8.099668105946853e-001, -3.031731138060267e-001, -1.935165610880807e+000, 3.083134597151535e+000}, {-8.192918853252418e-001, -3.064225359625613e-001, -1.935990472438392e+000, 3.080515785796513e+000}, {-8.286413378736912e-001, -3.097325618705826e-001, -1.936830006147945e+000, 3.077877633663473e+000}, {-8.380195338699306e-001, -3.131039924173268e-001, -1.937684552795189e+000, 3.075219949428117e+000}, {-8.474270870580881e-001, -3.165378097414413e-001, -1.938554494310329e+000, 3.072542437056092e+000}, {-8.568696991541474e-001, -3.200348522413296e-001, -1.939440187491448e+000, 3.069844902846438e+000}, {-8.663006219851039e-001, -3.235963509846717e-001, -1.940342031243627e+000, 3.067127048225000e+000}, {-8.755969029666063e-001, -3.272238814054915e-001, -1.941260416087275e+000, 3.064388625095321e+000}, {-8.847451895324255e-001, -3.309187279025791e-001, -1.942195741563215e+000, 3.061629384019479e+000}, {-8.937320642447604e-001, -3.346053531802966e-001, -1.943129242827890e+000, 3.058904739456490e+000}, {-9.025917571346555e-001, -3.381523238866969e-001, -1.944028228246023e+000, 3.056307824467905e+000}, {-9.114372727199264e-001, -3.417348650176046e-001, -1.944936739647034e+000, 3.053709701944858e+000}, {-9.202705051593279e-001, -3.453529129795982e-001, -1.945854872015144e+000, 3.051110359095721e+000}, {-9.290943179147365e-001, -3.490064060917521e-001, -1.946782720180552e+000, 3.048509783002059e+000}, {-9.379113035378066e-001, -3.526952951802167e-001, -1.947720378827961e+000, 3.045907960618840e+000}, {-9.467220586520030e-001, -3.564195514784557e-001, -1.948667942508438e+000, 3.043304878774594e+000}, {-9.555308122884657e-001, -3.601791344616129e-001, -1.949625505711145e+000, 3.040700524171566e+000}, {-9.643382215985582e-001, -3.639740319079109e-001, -1.950593162835531e+000, 3.038094883385813e+000}, {-9.731477743382022e-001, -3.678042249680064e-001, -1.951571008256408e+000, 3.035487942867281e+000}, {-9.819604804460426e-001, -3.716697186165744e-001, -1.952559136308239e+000, 3.032879688939842e+000}, {-9.907792924725812e-001, -3.755705130003580e-001, -1.953557641352945e+000, 3.030270107801308e+000}, {-9.996061903488108e-001, -3.795066236033756e-001, -1.954566617769396e+000, 3.027659185523410e+000}, {-1.008442332217312e+000, -3.834780801466576e-001, -1.955586159975327e+000, 3.025046908051736e+000}, {-1.017291187118612e+000, -3.874849056387030e-001, -1.956616362486219e+000, 3.022433261205659e+000}, {-1.026154224708753e+000, -3.915271430094308e-001, -1.957657319899389e+000, 3.019818230678225e+000}, {-1.035032728246099e+000, -3.956048454200966e-001, -1.958709126923303e+000, 3.017201802035997e+000}, {-1.043929801274928e+000, -3.997180632397729e-001, -1.959771878421853e+000, 3.014583960718895e+000}, {-1.052846917422804e+000, -4.038668640643446e-001, -1.960845669410642e+000, 3.011964692039990e+000}, {-1.061785874617104e+000, -4.080513211658256e-001, -1.961930595091244e+000, 3.009343981185274e+000}, {-1.070749281155416e+000, -4.122715113299520e-001, -1.963026750875285e+000, 3.006721813213400e+000}, {-1.079738284824320e+000, -4.165275287976867e-001, -1.964134232385564e+000, 3.004098173055395e+000}, {-1.088755056537119e+000, -4.208194669364273e-001, -1.965253135509263e+000, 3.001473045514344e+000}, {-1.097802354172137e+000, -4.251474262252926e-001, -1.966383556395710e+000, 2.998846415265045e+000}, {-1.106880826500578e+000, -4.295115260316672e-001, -1.967525591470941e+000, 2.996218266853638e+000}, {-1.115992785290996e+000, -4.339118828275575e-001, -1.968679337486387e+000, 2.993588584697207e+000}, {-1.125140601947682e+000, -4.383486216067864e-001, -1.969844891518973e+000, 2.990957353083350e+000}, {-1.134325959887290e+000, -4.428218781489602e-001, -1.971022350993147e+000, 2.988324556169727e+000}, {-1.143550497239480e+000, -4.473317957093480e-001, -1.972211813704686e+000, 2.985690177983577e+000}, {-1.152816353482765e+000, -4.518785235407696e-001, -1.973413377839173e+000, 2.983054202421210e+000}, {-1.162124575492226e+000, -4.564622237902882e-001, -1.974627141987564e+000, 2.980416613247473e+000}, {-1.171478417504371e+000, -4.610830531229002e-001, -1.975853205189499e+000, 2.977777394095186e+000}, {-1.180878493083438e+000, -4.657411912857257e-001, -1.977091666913681e+000, 2.975136528464554e+000}, {-1.190327393133407e+000, -4.704368136453667e-001, -1.978342627116694e+000, 2.972493999722552e+000}, {-1.199826475242167e+000, -4.751701105286942e-001, -1.979606186233481e+000, 2.969849791102279e+000}, {-1.209378307645038e+000, -4.799412725128706e-001, -1.980882445220469e+000, 2.967203885702293e+000}, {-1.218983781081318e+000, -4.847505072639017e-001, -1.982171505551098e+000, 2.964556266485916e+000}, {-1.228645580953087e+000, -4.895980192041626e-001, -1.983473469263982e+000, 2.961906916280511e+000}, {-1.238364766758997e+000, -4.944840304122741e-001, -1.984788438951109e+000, 2.959255817776730e+000}, {-1.248143707380005e+000, -4.994087621624195e-001, -1.986116517808350e+000, 2.956602953527749e+000}, {-1.257984483464209e+000, -5.043724445474235e-001, -1.987457809637882e+000, 2.953948305948456e+000}, {-1.267888763237574e+000, -5.093753175730373e-001, -1.988812418869192e+000, 2.951291857314631e+000}, {-1.277858155074884e+000, -5.144176299820401e-001, -1.990180450575876e+000, 2.948633589762087e+000}, {-1.287894587132058e+000, -5.194996353591673e-001, -1.991562010506334e+000, 2.945973485285792e+000}, {-1.297999978447886e+000, -5.246215949107307e-001, -1.992957205097942e+000, 2.943311525738964e+000}, {-1.308176257926076e+000, -5.297837777066512e-001, -1.994366141495533e+000, 2.940647692832132e+000}, {-1.318425322007352e+000, -5.349864596992644e-001, -1.995788927575479e+000, 2.937981968132178e+000}, {-1.328749277117991e+000, -5.402299243599632e-001, -1.997225671960814e+000, 2.935314333061348e+000}, {-1.339149199599445e+000, -5.455144676665747e-001, -1.998676484040282e+000, 2.932644768896240e+000}, {-1.349628368224229e+000, -5.508403810419320e-001, -2.000141474006426e+000, 2.929973256766762e+000}, {-1.360187219274824e+000, -5.562079806971391e-001, -2.001620752838174e+000, 2.927299777655058e+000}, {-1.370828781127042e+000, -5.616175744949384e-001, -2.003114432369282e+000, 2.924624312394423e+000}, {-1.381554684130381e+000, -5.670694862446681e-001, -2.004622625272832e+000, 2.921946841668168e+000}, {-1.392367149953658e+000, -5.725640441787859e-001, -2.006145445098469e+000, 2.919267346008476e+000}, {-1.403267693522663e+000, -5.781015902025207e-001, -2.007683006276334e+000, 2.916585805795218e+000}, {-1.414258411895148e+000, -5.836824691160817e-001, -2.009235424161747e+000, 2.913902201254752e+000}, {-1.425341560697221e+000, -5.893070331970658e-001, -2.010802815044349e+000, 2.911216512458685e+000}, {-1.436518650340928e+000, -5.949756469199823e-001, -2.012385296164895e+000, 2.908528719322610e+000}, {-1.447792531093776e+000, -6.006886752803999e-001, -2.013982985751103e+000, 2.905838801604813e+000}, {-1.459164304671427e+000, -6.064465023213990e-001, -2.015596003013732e+000, 2.903146738904952e+000}, {-1.470637134656586e+000, -6.122495073552353e-001, -2.017224468204838e+000, 2.900452510662714e+000}, {-1.482211863834624e+000, -6.180980925227824e-001, -2.018868502597047e+000, 2.897756096156425e+000}, {-1.493891852120214e+000, -6.239926530648990e-001, -2.020528228550773e+000, 2.895057474501652e+000}, {-1.505678096610527e+000, -6.299336067257686e-001, -2.022203769494007e+000, 2.892356624649756e+000}, {-1.517574031346761e+000, -6.359213660358718e-001, -2.023895249984035e+000, 2.889653525386434e+000}, {-1.529580796468173e+000, -6.419563650573666e-001, -2.025602795693617e+000, 2.886948155330209e+000}, {-1.541701356804014e+000, -6.480390354850254e-001, -2.027326533467826e+000, 2.884240492930909e+000}, {-1.553937356952073e+000, -6.541698276740846e-001, -2.029066591313422e+000, 2.881530516468101e+000}, {-1.566290954997507e+000, -6.603491961874959e-001, -2.030823098450255e+000, 2.878818204049505e+000}, {-1.578765461793385e+000, -6.665775985380139e-001, -2.032596185332172e+000, 2.876103533609368e+000}, {-1.591361479469684e+000, -6.728555174296306e-001, -2.034385983640837e+000, 2.873386482906806e+000}, {-1.604083053178186e+000, -6.791834242424827e-001, -2.036192626363599e+000, 2.870667029524126e+000}, {-1.616931466226678e+000, -6.855618163240326e-001, -2.038016247762119e+000, 2.867945150865093e+000}, {-1.629909893369509e+000, -6.919911893411197e-001, -2.039856983439587e+000, 2.865220824153184e+000}, {-1.643020022349849e+000, -6.984720579442955e-001, -2.041714970334005e+000, 2.862494026429801e+000}, {-1.656264551293298e+000, -7.050049397229221e-001, -2.043590346770784e+000, 2.859764734552450e+000}, {-1.669646268624311e+000, -7.115903625664921e-001, -2.045483252474063e+000, 2.857032925192886e+000}, {-1.683167576151129e+000, -7.182288656970074e-001, -2.047393828594889e+000, 2.854298574835218e+000}, {-1.696831423652399e+000, -7.249209956676710e-001, -2.049322217741730e+000, 2.851561659773992e+000}, {-1.710639128268257e+000, -7.316673198056414e-001, -2.051268563987778e+000, 2.848822156112223e+000}, {-1.724594742236566e+000, -7.384683987028911e-001, -2.053233012939716e+000, 2.846080039759406e+000}, {-1.738700069789140e+000, -7.453248180187038e-001, -2.055215711717725e+000, 2.843335286429471e+000}, {-1.752958138408101e+000, -7.522371655871375e-001, -2.057216809020459e+000, 2.840587871638726e+000}, {-1.767371569680276e+000, -7.592060430135096e-001, -2.059236455133438e+000, 2.837837770703736e+000}, {-1.781943384385722e+000, -7.662320613766317e-001, -2.061274801963785e+000, 2.835084958739185e+000}, {-1.796676013498676e+000, -7.733158456018825e-001, -2.063332003066546e+000, 2.832329410655683e+000}, {-1.811572772917376e+000, -7.804580273271474e-001, -2.065408213682221e+000, 2.829571101157551e+000}, {-1.826636048726881e+000, -7.876592550863618e-001, -2.067503590753522e+000, 2.826810004740544e+000}, {-1.841869688783150e+000, -7.949201807533446e-001, -2.069618292975332e+000, 2.824046095689560e+000}, {-1.857275569898410e+000, -8.022414792244761e-001, -2.071752480798810e+000, 2.821279348076279e+000}, {-1.872858144207920e+000, -8.096238224870805e-001, -2.073906316498295e+000, 2.818509735756786e+000}, {-1.888619059956391e+000, -8.170679126024469e-001, -2.076079964156223e+000, 2.815737232369135e+000}, {-1.904561956423886e+000, -8.245744503054575e-001, -2.078273589745324e+000, 2.812961811330880e+000}, {-1.920690668385609e+000, -8.321441499275529e-001, -2.080487361132164e+000, 2.810183445836552e+000}, {-1.937007416473138e+000, -8.397777468295377e-001, -2.082721448109063e+000, 2.807402108855102e+000}, {-1.953516633870803e+000, -8.474759771845204e-001, -2.084976022451227e+000, 2.804617773127296e+000}, {-1.970220662325402e+000, -8.552396029642130e-001, -2.087251257918434e+000, 2.801830411163055e+000}, {-1.987123236442702e+000, -8.630693899925094e-001, -2.089547330324482e+000, 2.799039995238766e+000}, {-2.004228350601878e+000, -8.709661185707598e-001, -2.091864417548962e+000, 2.796246497394527e+000}, {-2.021538725971305e+000, -8.789305897778639e-001, -2.094202699573659e+000, 2.793449889431357e+000}, {-2.039058367884046e+000, -8.869636101217421e-001, -2.096562358541322e+000, 2.790650142908356e+000}, {-2.056791020141507e+000, -8.950660040405472e-001, -2.098943578769776e+000, 2.787847229139806e+000}, {-2.074740298046955e+000, -9.032386105028429e-001, -2.101346546799721e+000, 2.785041119192230e+000}, {-2.092910200608639e+000, -9.114822800669383e-001, -2.103771451439775e+000, 2.782231783881398e+000}, {-2.111304179015425e+000, -9.197978847153946e-001, -2.106218483783340e+000, 2.779419193769277e+000}, {-2.129926301106460e+000, -9.281863059214009e-001, -2.108687837277904e+000, 2.776603319160936e+000}, {-2.148780529001271e+000, -9.366484426859110e-001, -2.111179707746509e+000, 2.773784130101387e+000}, {-2.167870835137651e+000, -9.451852095934209e-001, -2.113694293436472e+000, 2.770961596372379e+000}, {-2.187201600827261e+000, -9.537975352704542e-001, -2.116231795062518e+000, 2.768135687489128e+000}, {-2.206776744200663e+000, -9.624863678682748e-001, -2.118792415842629e+000, 2.765306372697000e+000}, {-2.226600913877821e+000, -9.712526682985949e-001, -2.121376361551813e+000, 2.762473620968124e+000}, {-2.246678099895653e+000, -9.800974184540910e-001, -2.123983840556277e+000, 2.759637400997960e+000}, {-2.267012600006316e+000, -9.890216163020363e-001, -2.126615063865472e+000, 2.756797681201793e+000}, {-2.287609020074218e+000, -9.980262753763182e-001, -2.129270245181486e+000, 2.753954429711178e+000}, {-2.308472981636114e+000, -1.007112421979191e+000, -2.131949600945738e+000, 2.751107614370312e+000}, {-2.329607982655612e+000, -1.016281113082512e+000, -2.134653350367198e+000, 2.748257202732345e+000}, {-2.351019199384163e+000, -1.025533415332722e+000, -2.137381715498646e+000, 2.745403162055640e+000}, {-2.372711351753493e+000, -1.034870417426487e+000, -2.140134921268429e+000, 2.742545459299940e+000}, {-2.394689420941603e+000, -1.044293225304602e+000, -2.142913195543392e+000, 2.739684061122500e+000}, {-2.416958489696154e+000, -1.053802964837059e+000, -2.145716769171991e+000, 2.736818933874121e+000}, {-2.439523927931083e+000, -1.063400781249023e+000, -2.148545876038628e+000, 2.733950043595136e+000}, {-2.462390547826898e+000, -1.073087842531493e+000, -2.151400753117987e+000, 2.731077356011314e+000}, {-2.485563955690444e+000, -1.082865334488783e+000, -2.154281640529923e+000, 2.728200836529693e+000}, {-2.509049754370408e+000, -1.092734464087250e+000, -2.157188781596602e+000, 2.725320450234344e+000}, {-2.532853214807481e+000, -1.102696462282614e+000, -2.160122422893467e+000, 2.722436161882056e+000}, {-2.556980090750864e+000, -1.112752579055204e+000, -2.163082814316411e+000, 2.719547935897943e+000}, {-2.581436437132852e+000, -1.122904087457842e+000, -2.166070209130614e+000, 2.716655736370978e+000}, {-2.606227654816659e+000, -1.133152285306878e+000, -2.169084864037961e+000, 2.713759527049444e+000}, {-2.631360439576148e+000, -1.143498488007834e+000, -2.172127039239977e+000, 2.710859271336302e+000}, {-2.656840304856598e+000, -1.153944042779445e+000, -2.175196998484420e+000, 2.707954932284481e+000}, {-2.682673341601513e+000, -1.164490316871617e+000, -2.178295009149678e+000, 2.705046472592083e+000}, {-2.708866825204242e+000, -1.175138696975488e+000, -2.181421342304900e+000, 2.702133854597497e+000}, {-2.735426422726402e+000, -1.185890604592462e+000, -2.184576272760956e+000, 2.699217040274428e+000}, {-2.762359314107140e+000, -1.196747478763960e+000, -2.187760079160614e+000, 2.696295991226843e+000}, {-2.789671673506554e+000, -1.207710791213431e+000, -2.190973044027281e+000, 2.693370668683814e+000}, {-2.817371158194170e+000, -1.218782032275562e+000, -2.194215453856300e+000, 2.690441033494273e+000}, {-2.845464441688514e+000, -1.229962726999865e+000, -2.197487599162562e+000, 2.687507046121677e+000}, {-2.873958455587959e+000, -1.241254426116775e+000, -2.200789774573439e+000, 2.684568666638566e+000}, {-2.902860980914899e+000, -1.252658706018288e+000, -2.204122278897234e+000, 2.681625854721035e+000}, {-2.932179792788460e+000, -1.264177171262446e+000, -2.207485415203479e+000, 2.678678569643088e+000}, {-2.961921972458911e+000, -1.275811461710634e+000, -2.210879490886982e+000, 2.675726770270900e+000}, {-2.992095784809014e+000, -1.287563242185971e+000, -2.214304817763162e+000, 2.672770415056979e+000}, {-3.022708807675713e+000, -1.299434212241244e+000, -2.217761712139564e+000, 2.669809462034204e+000}, {-3.053769494827838e+000, -1.311426099021429e+000, -2.221250494908468e+000, 2.666843868809768e+000}, {-3.085286256712582e+000, -1.323540662532576e+000, -2.224771491625306e+000, 2.663873592558999e+000}, {-3.117267156964171e+000, -1.335779699224753e+000, -2.228325032589873e+000, 2.660898590019072e+000}, {-3.149721717942327e+000, -1.348145031420924e+000, -2.231911452951636e+000, 2.657918817482606e+000}, {-3.182657768442030e+000, -1.360638526376643e+000, -2.235531092775834e+000, 2.654934230791132e+000}, {-3.216085761856677e+000, -1.373262071886569e+000, -2.239184297168944e+000, 2.651944785328451e+000}, {-3.250013826855947e+000, -1.386017607184507e+000, -2.242871416329658e+000, 2.648950436013852e+000}, {-3.284451500134391e+000, -1.398907099420707e+000, -2.246592805684378e+000, 2.645951137295225e+000}, {-3.319409250572173e+000, -1.411932549809613e+000, -2.250348825973040e+000, 2.642946843142023e+000}, {-3.354896753611417e+000, -1.425096003296100e+000, -2.254139843338936e+000, 2.639937507038096e+000}, {-3.390924039417435e+000, -1.438399542162570e+000, -2.257966229444829e+000, 2.636923081974400e+000}, {-3.427501254566009e+000, -1.451845290352722e+000, -2.261828361566645e+000, 2.633903520441551e+000}, {-3.464639359689941e+000, -1.465435408601526e+000, -2.265726622713691e+000, 2.630878774422248e+000}, {-3.502349053636324e+000, -1.479172101786477e+000, -2.269661401730241e+000, 2.627848795383550e+000}, {-3.540641220590683e+000, -1.493057617817210e+000, -2.273633093408675e+000, 2.624813534268998e+000}, {-3.579527769324496e+000, -1.507094243675857e+000, -2.277642098611021e+000, 2.621772941490594e+000}, {-3.619019405104672e+000, -1.521284319088120e+000, -2.281688824370349e+000, 2.618726966920613e+000}, {-3.659128845786773e+000, -1.535630218848213e+000, -2.285773684035279e+000, 2.615675559883270e+000}, {-3.699868104366120e+000, -1.550134370620169e+000, -2.289897097369691e+000, 2.612618669146215e+000}, {-3.741248903588338e+000, -1.564799252976718e+000, -2.294059490683182e+000, 2.609556242911866e+000}, {-3.783284874324993e+000, -1.579627382536116e+000, -2.298261296982422e+000, 2.606488228808574e+000}, {-3.825988545927405e+000, -1.594621335053040e+000, -2.302502956067142e+000, 2.603414573881602e+000}, {-3.869373496014526e+000, -1.609783731499074e+000, -2.306784914694967e+000, 2.600335224583944e+000}, {-3.913453221956383e+000, -1.625117249152555e+000, -2.311107626695294e+000, 2.597250126766952e+000}, {-3.958241447222634e+000, -1.640624618733666e+000, -2.315471553123667e+000, 2.594159225670774e+000}, {-4.003752472151792e+000, -1.656308622508239e+000, -2.319877162411532e+000, 2.591062465914611e+000}, {-4.050001634221138e+000, -1.672172096388015e+000, -2.324324930505149e+000, 2.587959791486772e+000}, {-4.097002876636228e+000, -1.688217943259190e+000, -2.328815341006177e+000, 2.584851145734526e+000}, {-4.144772509543070e+000, -1.704449112223655e+000, -2.333348885357084e+000, 2.581736471353761e+000}, {-4.193325486276812e+000, -1.720868623261894e+000, -2.337926062964376e+000, 2.578615710378421e+000}, {-4.242678130201291e+000, -1.737479551563644e+000, -2.342547381387367e+000, 2.575488804169740e+000}, {-4.292847565917512e+000, -1.754285033725772e+000, -2.347213356495573e+000, 2.572355693405246e+000}, {-4.343849344050583e+000, -1.771288281606365e+000, -2.351924512625498e+000, 2.569216318067544e+000}, {-4.395701890165288e+000, -1.788492558606178e+000, -2.356681382789128e+000, 2.566070617432879e+000}, {-4.448422980721418e+000, -1.805901202887346e+000, -2.361484508818656e+000, 2.562918530059439e+000}, {-4.502030354716064e+000, -1.823517623449364e+000, -2.366334441565460e+000, 2.559759993775440e+000}, {-4.556542840854151e+000, -1.841345298947618e+000, -2.371231741081224e+000, 2.556594945666951e+000}, {-4.611979475168832e+000, -1.859387780568530e+000, -2.376176976819913e+000, 2.553423322065466e+000}, {-4.668359753577363e+000, -1.877648695147924e+000, -2.381170727826738e+000, 2.550245058535221e+000}, {-4.725704074969167e+000, -1.896131743036145e+000, -2.386213582948206e+000, 2.547060089860238e+000}, {-4.784032727943202e+000, -1.914840707513675e+000, -2.391306141027599e+000, 2.543868350031094e+000}, {-4.843367240144604e+000, -1.933779447928860e+000, -2.396449011131275e+000, 2.540669772231420e+000}, {-4.903729030960412e+000, -1.952951909731893e+000, -2.401642812755355e+000, 2.537464288824098e+000}, {-4.965140432874901e+000, -1.972362121662210e+000, -2.406888176055385e+000, 2.534251831337176e+000}, {-5.027623797901383e+000, -1.992014202447187e+000, -2.412185742069771e+000, 2.531032330449472e+000}, {-5.091203355130014e+000, -2.011912351777201e+000, -2.417536162974205e+000, 2.527805715975873e+000}, {-5.155903044122922e+000, -2.032060866670441e+000, -2.422940102297541e+000, 2.524571916852306e+000}, {-5.221747451923521e+000, -2.052464137830248e+000, -2.428398235181793e+000, 2.521330861120394e+000}, {-5.288761616135504e+000, -2.073126654030853e+000, -2.433911248632754e+000, 2.518082475911771e+000}, {-5.356971788598320e+000, -2.094052999648703e+000, -2.439479841788043e+000, 2.514826687432050e+000}, {-5.426404613215012e+000, -2.115247861646304e+000, -2.445104726181699e+000, 2.511563420944447e+000}, {-5.497087522474739e+000, -2.136716031186819e+000, -2.450786626016956e+000, 2.508292600753025e+000}, {-5.569049288882915e+000, -2.158462403870986e+000, -2.456526278452476e+000, 2.505014150185588e+000}, {-5.642317740633977e+000, -2.180491994108471e+000, -2.462324433881294e+000, 2.501727991576171e+000}, {-5.716923840072115e+000, -2.202809915912630e+000, -2.468181856254582e+000, 2.498434046247153e+000}, {-5.792897745374091e+000, -2.225421410091371e+000, -2.474099323354432e+000, 2.495132234490956e+000}, {-5.870270221041334e+000, -2.248331836268295e+000, -2.480077627130501e+000, 2.491822475551335e+000}, {-5.949074871981971e+000, -2.271546665967843e+000, -2.486117574029875e+000, 2.488504687604241e+000}, {-6.029343786707989e+000, -2.295071511647872e+000, -2.492219985295749e+000, 2.485178787738241e+000}, {-6.111112084568413e+000, -2.318912101810149e+000, -2.498385697352468e+000, 2.481844691934497e+000}, {-6.194414527198641e+000, -2.343074305694364e+000, -2.504615562124872e+000, 2.478502315046275e+000}, {-6.279286904596403e+000, -2.367564130914826e+000, -2.510910447403784e+000, 2.475151570777977e+000}, {-6.365767209259786e+000, -2.392387718285614e+000, -2.517271237235579e+000, 2.471792371663692e+000}, {-6.453892994198572e+000, -2.417551362201253e+000, -2.523698832280891e+000, 2.468424629045237e+000}, {-6.543704169328149e+000, -2.443061498363987e+000, -2.530194150230223e+000, 2.465048253049680e+000}, {-6.635241010728419e+000, -2.468924721455782e+000, -2.536758126185958e+000, 2.461663152566336e+000}, {-6.728545843224950e+000, -2.495147778765465e+000, -2.543391713097591e+000, 2.458269235223206e+000}, {-6.823660845563034e+000, -2.521737588158918e+000, -2.550095882171187e+000, 2.454866407362863e+000}, {-6.920630794798460e+000, -2.548701226486825e+000, -2.556871623333734e+000, 2.451454574017748e+000}, {-7.019501381475465e+000, -2.576045946160612e+000, -2.563719945667194e+000, 2.448033638884867e+000}, {-7.120319480255796e+000, -2.603779178381579e+000, -2.570641877882507e+000, 2.444603504299888e+000}, {-7.223133969609193e+000, -2.631908532839725e+000, -2.577638468809950e+000, 2.441164071210579e+000}, {-7.327994580174460e+000, -2.660441811671938e+000, -2.584710787882002e+000, 2.437715239149616e+000}, {-7.434952494647915e+000, -2.689387012530814e+000, -2.591859925646200e+000, 2.434256906206702e+000}, {-7.544061135790023e+000, -2.718752328646964e+000, -2.599086994305352e+000, 2.430788968999999e+000}, {-7.655375515661243e+000, -2.748546159812257e+000, -2.606393128252138e+000, 2.427311322646844e+000}, {-7.768951708993694e+000, -2.778777123466263e+000, -2.613779484621389e+000, 2.423823860733729e+000}, {-7.884848282627553e+000, -2.809454050978603e+000, -2.621247243887760e+000, 2.420326475285515e+000}, {-8.003125847080343e+000, -2.840586000686433e+000, -2.628797610445466e+000, 2.416819056733866e+000}, {-8.123845717615904e+000, -2.872182270696010e+000, -2.636431813223878e+000, 2.413301493884877e+000}, {-8.247072986653183e+000, -2.904252390209657e+000, -2.644151106340080e+000, 2.409773673885860e+000}, {-8.372873324207069e+000, -2.936806146623549e+000, -2.651956769727907e+000, 2.406235482191265e+000}, {-8.501315321522739e+000, -2.969853579297517e+000, -2.659850109837497e+000, 2.402686802527732e+000}, {-8.632470179999418e+000, -3.003404992656809e+000, -2.667832460322147e+000, 2.399127516858195e+000}, {-8.766410786955008e+000, -3.037470967875636e+000, -2.675905182763388e+000, 2.395557505345058e+000}, {-8.903213411118998e+000, -3.072062364995864e+000, -2.684069667421548e+000, 2.391976646312385e+000}, {-9.042955255058262e+000, -3.107190347094537e+000, -2.692327333987842e+000, 2.388384816207067e+000}, {-9.185717838632552e+000, -3.142866369671718e+000, -2.700679632411345e+000, 2.384781889558960e+000}, {-9.331585097693958e+000, -3.179102204446007e+000, -2.709128043701907e+000, 2.381167738939921e+000}, {-9.480643607373500e+000, -3.215909949936703e+000, -2.717674080783783e+000, 2.377542234921738e+000}, {-9.632982569905030e+000, -3.253302040635393e+000, -2.726319289385702e+000, 2.373905246032896e+000}, {-9.788696395803125e+000, -3.291291246777491e+000, -2.735065248964523e+000, 2.370256638714140e+000}, {-9.947880046656847e+000, -3.329890716053707e+000, -2.743913573608185e+000, 2.366596277272798e+000}, {-1.011063370599249e+001, -3.369113955782007e+000, -2.752865913065797e+000, 2.362924023835830e+000}, {-1.027706158128749e+001, -3.408974858849373e+000, -2.761923953732860e+000, 2.359239738301529e+000}, {-1.044726980264801e+001, -3.449487728026834e+000, -2.771089419696060e+000, 2.355543278289857e+000}, {-1.062137120153357e+001, -3.490667263098932e+000, -2.780364073859961e+000, 2.351834499091345e+000}, {-1.079948025851573e+001, -3.532528610310817e+000, -2.789749719020175e+000, 2.348113253614504e+000}, {-1.098171707046586e+001, -3.575087353790894e+000, -2.799248199077059e+000, 2.344379392331725e+000}, {-1.116820588443619e+001, -3.618359541657062e+000, -2.808861400235142e+000, 2.340632763223555e+000}, {-1.135907707782098e+001, -3.662361694756724e+000, -2.818591252274326e+000, 2.336873211721342e+000}, {-1.155446370525415e+001, -3.707110846632381e+000, -2.828439729829784e+000, 2.333100580648148e+000}, {-1.175450638810228e+001, -3.752624533595280e+000, -2.838408853794547e+000, 2.329314710157896e+000}, {-1.195935053540618e+001, -3.798920836053745e+000, -2.848500692695762e+000, 2.325515437672646e+000}, {-1.216914587843163e+001, -3.846018400646425e+000, -2.858717364169532e+000, 2.321702597817966e+000}, {-1.238405202330118e+001, -3.893936434232603e+000, -2.869061036519223e+000, 2.317876022356294e+000}, {-1.260423031369346e+001, -3.942694775750631e+000, -2.879533930237916e+000, 2.314035540118215e+000}, {-1.282985291483982e+001, -3.992313868676216e+000, -2.890138319734129e+000, 2.310180976931594e+000}, {-1.306109620929062e+001, -4.042814830660411e+000, -2.900876534982684e+000, 2.306312155548434e+000}, {-1.329814643430201e+001, -4.094219444695344e+000, -2.911750963375111e+000, 2.302428895569415e+000}, {-1.354119627426263e+001, -4.146550213873349e+000, -2.922764051537747e+000, 2.298531013365968e+000}, {-1.379044740846033e+001, -4.199830379609585e+000, -2.933918307290487e+000, 2.294618321999826e+000}, {-1.404610889345345e+001, -4.254083961469080e+000, -2.945216301668451e+000, 2.290690631139917e+000}, {-1.430840142757132e+001, -4.309335770863983e+000, -2.956660671054686e+000, 2.286747746976495e+000}, {-1.457755349316863e+001, -4.365611471997627e+000, -2.968254119347887e+000, 2.282789472132388e+000}, {-1.485380456670902e+001, -4.422937601107329e+000, -2.979999420291443e+000, 2.278815605571237e+000}, {-1.513740534264527e+001, -4.481341609033519e+000, -2.991899419865233e+000, 2.274825942502599e+000}, {-1.542861733910674e+001, -4.540851908926687e+000, -3.003957038785516e+000, 2.270820274283760e+000}, {-1.572771438306834e+001, -4.601497912391182e+000, -3.016175275142619e+000, 2.266798388318140e+000}, {-1.603498363566405e+001, -4.663310074910363e+000, -3.028557207138266e+000, 2.262760067950089e+000}, {-1.635072484269092e+001, -4.726319953306938e+000, -3.041105995942990e+000, 2.258705092355959e+000}, {-1.667525327132274e+001, -4.790560243365801e+000, -3.053824888716676e+000, 2.254633236431245e+000}, {-1.700889813746121e+001, -4.856064846889706e+000, -3.066717221736760e+000, 2.250544270673617e+000}, {-1.735200585595545e+001, -4.922868920071202e+000, -3.079786423691872e+000, 2.246437961061666e+000}, {-1.770493843523546e+001, -4.991008942657080e+000, -3.093036019134606e+000, 2.242314068929141e+000}, {-1.806807798479443e+001, -5.060522765468733e+000, -3.106469632112602e+000, 2.238172350834463e+000}, {-1.844182350134685e+001, -5.131449703543621e+000, -3.120090989928735e+000, 2.234012558425291e+000}, {-1.882659593895549e+001, -5.203830582698908e+000, -3.133903927164026e+000, 2.229834438297898e+000}, {-1.922283720003022e+001, -5.277707832656685e+000, -3.147912389827138e+000, 2.225637731851087e+000}, {-1.963101137194863e+001, -5.353125565238721e+000, -3.162120439758187e+000, 2.221422175134389e+000}, {-2.005160793417239e+001, -5.430129651245946e+000, -3.176532259252670e+000, 2.217187498690242e+000}, {-2.048514159347870e+001, -5.508767821782203e+000, -3.191152155899943e+000, 2.212933427389840e+000}, {-2.093215458382820e+001, -5.589089761522009e+000, -3.205984567694959e+000, 2.208659680262342e+000}, {-2.139321785013910e+001, -5.671147213614056e+000, -3.221034068411279e+000, 2.204365970317081e+000}, {-2.186893386429054e+001, -5.754994082273456e+000, -3.236305373268270e+000, 2.200052004358388e+000}, {-2.235993893520058e+001, -5.840686550619682e+000, -3.251803344885138e+000, 2.195717482792680e+000}, {-2.286690183868918e+001, -5.928283224318049e+000, -3.267532999536663e+000, 2.191362099427353e+000}, {-2.339053129045003e+001, -6.017845229047717e+000, -3.283499513835679e+000, 2.186985541261074e+000}, {-2.393157553110568e+001, -6.109436372639191e+000, -3.299708231679054e+000, 2.182587488264950e+000}, {-2.449082365362512e+001, -6.203123298466807e+000, -3.316164671630298e+000, 2.178167613154133e+000}, {-2.506911261829099e+001, -6.298975620761307e+000, -3.332874534743562e+000, 2.173725581149273e+000}, {-2.566732526965500e+001, -6.397066125615121e+000, -3.349843712746693e+000, 2.169261049727270e+000}, {-2.628639811323098e+001, -6.497470921823482e+000, -3.367078296797307e+000, 2.164773668360732e+000}, {-2.692732173142186e+001, -6.600269655973709e+000, -3.384584586646084e+000, 2.160263078245440e+000}, {-2.759114524502430e+001, -6.705545710843315e+000, -3.402369100397087e+000, 2.155728912015202e+000}, {-2.827898248761451e+001, -6.813386415478488e+000, -3.420438584828478e+000, 2.151170793443279e+000}, {-2.899201259869876e+001, -6.923883300413103e+000, -3.438800026289747e+000, 2.146588337129650e+000}, {-2.973148847503503e+001, -7.037132326714247e+000, -3.457460662309122e+000, 2.141981148173250e+000}, {-3.049873840109150e+001, -7.153234177647235e+000, -3.476427993824973e+000, 2.137348821828268e+000}, {-3.129517493940490e+001, -7.272294522510483e+000, -3.495709798247626e+000, 2.132690943143583e+000}, {-3.212229765000851e+001, -7.394424343175200e+000, -3.515314143246334e+000, 2.128007086584252e+000}, {-3.298170177292683e+001, -7.519740249452365e+000, -3.535249401439987e+000, 2.123296815633989e+000}, {-3.387508454061701e+001, -7.648364836976240e+000, -3.555524265971750e+000, 2.118559682377430e+000}, {-3.480425051048727e+001, -7.780427073933807e+000, -3.576147767071773e+000, 2.113795227060935e+000}, {-3.577112387256164e+001, -7.916062682686504e+000, -3.597129289704889e+000, 2.109002977630553e+000}, {-3.677775248212046e+001, -8.055414602783463e+000, -3.618478592274180e+000, 2.104182449245703e+000}, {-3.782632044661916e+001, -8.198633436873966e+000, -3.640205826613887e+000, 2.099333143767018e+000}, {-3.891915659162862e+001, -8.345877962293837e+000, -3.662321559229458e+000, 2.094454549216663e+000}, {-4.005874745570689e+001, -8.497315658007004e+000, -3.684836793965542e+000, 2.089546139209324e+000}, {-4.124774609923581e+001, -8.653123302578791e+000, -3.707762996136288e+000, 2.084607372351954e+000}, {-4.248898846355697e+001, -8.813487578187926e+000, -3.731112118317829e+000, 2.079637691610172e+000}, {-4.378550526414175e+001, -8.978605753438103e+000, -3.754896627843079e+000, 2.074636523639093e+000}, {-4.514053901757636e+001, -9.148686395755780e+000, -3.779129536196960e+000, 2.069603278076170e+000}, {-4.655756027868422e+001, -9.323950155855000e+000, -3.803824430401465e+000, 2.064537346793447e+000}, {-4.804028430092423e+001, -9.504630615340995e+000, -3.828995506585277e+000, 2.059438103106440e+000}, {-4.959269431334278e+001, -9.690975174726935e+000, -3.854657605931726e+000, 2.054304900936606e+000}, {-5.121906225977325e+001, -9.883246042911827e+000, -3.880826253119925e+000, 2.049137073924143e+000}, {-5.292396969033102e+001, -1.008172131188975e+001, -3.907517697516740e+000, 2.043933934487611e+000}, {-5.471233926943821e+001, -1.028669607165458e+001, -3.934748957389588e+000, 2.038694772826543e+000}, {-5.658946080723199e+001, -1.049848367280677e+001, -3.962537867252546e+000, 2.033418855862916e+000}, {-5.856102198737246e+001, -1.071741707506913e+001, -3.990903128739948e+000, 2.028105426117049e+000}, {-6.063314629294836e+001, -1.094385028946904e+001, -4.019864365275073e+000, 2.022753700513037e+000}, {-6.281242946118387e+001, -1.117815997880301e+001, -4.049442180792742e+000, 2.017362869108480e+000}, {-6.510598207265966e+001, -1.142074718300361e+001, -4.079658222944135e+000, 2.011932093742810e+000}, {-6.752147690312931e+001, -1.167203919014480e+001, -4.110535251172350e+000, 2.006460506597967e+000}, {-7.006720223331806e+001, -1.193249157665400e+001, -4.142097210058299e+000, 2.000947208664683e+000}, {-7.275211711338568e+001, -1.220259045397827e+001, -4.174369308405988e+000, 1.995391268106987e+000}, {-7.558591731733223e+001, -1.248285489197702e+001, -4.207378104671925e+000, 1.989791718516892e+000}, {-7.857910654055216e+001, -1.277383958744936e+001, -4.241151599238290e+000, 1.984147557050459e+000}, {-8.174307486950895e+001, -1.307613778749139e+001, -4.275719334227814e+000, 1.978457742435633e+000}, {-8.509018865469528e+001, -1.339038448212000e+001, -4.311112501590751e+000, 1.972721192841308e+000}, {-8.863388899758650e+001, -1.371725992427822e+001, -4.347364060230715e+000, 1.966936783596048e+000}, {-9.238880450061555e+001, -1.405749348404934e+001, -4.384508863125920e+000, 1.961103344743806e+000}, {-9.638803576032707e+001, -1.441179912568236e+001, -4.422583856657013e+000, 1.955219658422624e+000}, {-1.006355886419617e+002, -1.478104683078891e+001, -4.461628093181375e+000, 1.949284456050977e+000}, {-1.051313454597849e+002, -1.516621147525332e+001, -4.501682994351671e+000, 1.943296415304725e+000}, {-1.098726320850258e+002, -1.557562130758561e+001, -4.543476494807336e+000, 1.937154374999355e+000}, {-1.148328382566474e+002, -1.599730363449987e+001, -4.585776601560153e+000, 1.931045251398180e+000}, {-1.200188108246691e+002, -1.643068297329990e+001, -4.628499471591676e+000, 1.924981823573037e+000}, {-1.254402600577610e+002, -1.687609306545174e+001, -4.671650064463239e+000, 1.918963711773316e+000}, {-1.311072854774274e+002, -1.733387495899903e+001, -4.715233405571223e+000, 1.912990539037578e+000}, {-1.370304007377400e+002, -1.780437708974399e+001, -4.759254580973940e+000, 1.907061931185782e+000}, {-1.432205478371830e+002, -1.828795544632071e+001, -4.803718732234705e+000, 1.901177516818668e+000}, {-1.496891184999602e+002, -1.878497370717625e+001, -4.848631051484921e+000, 1.895336927323218e+000}, {-1.564479780964446e+002, -1.929580338699566e+001, -4.893996776657341e+000, 1.889539796883510e+000}, {-1.635094896114932e+002, -1.982082401614414e+001, -4.939821186869632e+000, 1.883785762496075e+000}, {-1.708865276896965e+002, -2.036042337835779e+001, -4.986109598005116e+000, 1.878074463988980e+000}, {-1.785925165319611e+002, -2.091499764565038e+001, -5.032867358670587e+000, 1.872405544044007e+000}, {-1.866414510420746e+002, -2.148495164496683e+001, -5.080099846220948e+000, 1.866778648221036e+000}, {-1.950479162106498e+002, -2.207069912489180e+001, -5.127812463129048e+000, 1.861193424984080e+000}, {-2.038271353278870e+002, -2.267266292380108e+001, -5.176010633724887e+000, 1.855649525728495e+000}, {-2.129949758145855e+002, -2.329127536217849e+001, -5.224699800985325e+000, 1.850146604808510e+000}, {-2.225680072626292e+002, -2.392697840643329e+001, -5.273885423907177e+000, 1.844684319564753e+000}, {-2.325635046039472e+002, -2.458022411269052e+001, -5.323572974870670e+000, 1.839262330351306e+000}, {-2.429995182741653e+002, -2.525147478994882e+001, -5.373767937627715e+000, 1.833880300561732e+000}, {-2.538948748098448e+002, -2.594120350769362e+001, -5.424475805224988e+000, 1.828537896653741e+000}, {-2.652692415314800e+002, -2.664989431255637e+001, -5.475702078602022e+000, 1.823234788172320e+000}, {-2.771431560702175e+002, -2.737804267118220e+001, -5.527452265174031e+000, 1.817970647770634e+000}, {-2.895380660742452e+002, -2.812615585599748e+001, -5.579731877863270e+000, 1.812745151228864e+000}, {-3.024763750171533e+002, -2.889475333428489e+001, -5.632546434409225e+000, 1.807557977470461e+000}, {-3.159814902028902e+002, -2.968436718887993e+001, -5.685901456916128e+000, 1.802408808575871e+000}, {-3.300778700110466e+002, -3.049554255611214e+001, -5.739802471693491e+000, 1.797297329793238e+000}, {-3.447910678389576e+002, -3.132883810210219e+001, -5.794255009340000e+000, 1.792223229546456e+000}, {-3.601477887192862e+002, -3.218482647037611e+001, -5.849264605126227e+000, 1.787186199439965e+000}, {-3.761759502852180e+002, -3.306409475164691e+001, -5.904836799592490e+000, 1.782185934260666e+000}, {-3.929047215975117e+002, -3.396724504852546e+001, -5.960977139338727e+000, 1.777222131976677e+000}, {-4.103646058482501e+002, -3.489489490975335e+001, -6.017691178152736e+000, 1.772294493733011e+000}, {-4.285874676385886e+002, -3.584767798721607e+001, -6.074984478170861e+000, 1.767402723844187e+000}, {-4.476066601835161e+002, -3.682624437371988e+001, -6.132862611514633e+000, 1.762546529783868e+000}, {-4.674570002533873e+002, -3.783126149387554e+001, -6.191331161679662e+000, 1.757725622171409e+000}, {-4.881749356184829e+002, -3.886341433245930e+001, -6.250395725677553e+000, 1.752939714755772e+000}, {-5.097985523417300e+002, -3.992340630042679e+001, -6.310061915750649e+000, 1.748188524396310e+000}, {-5.323676746229096e+002, -4.101195973874807e+001, -6.370335361660072e+000, 1.743471771041181e+000}, {-5.559239469182820e+002, -4.212981655933941e+001, -6.431221712923479e+000, 1.738789177703054e+000}, {-5.805109168157189e+002, -4.327773889908156e+001, -6.492726641214050e+000, 1.734140470432367e+000}, {-6.061741146091180e+002, -4.445650981673200e+001, -6.554855842839679e+000, 1.729525378288342e+000}, {-6.329602015454766e+002, -4.566693587314825e+001, -6.617615040532088e+000, 1.724943633307832e+000}, {-6.609250424371172e+002, -4.690982976130863e+001, -6.681009994698973e+000, 1.720394970472067e+000}, {-6.901042593869157e+002, -4.818608458142636e+001, -6.745046467541486e+000, 1.715879127671711e+000}, {-7.205706164247110e+002, -4.949651492478494e+001, -6.809730323163137e+000, 1.711395845671781e+000}, {-7.523401581588573e+002, -5.084211154381994e+001, -6.875067360429169e+000, 1.706944868064385e+000}, {-7.854200256589894e+002, -5.222382897866345e+001, -6.941063483168687e+000, 1.702525941248357e+000}, {-8.210353457106370e+002, -5.364030461686616e+001, -7.007725542909286e+000, 1.698138814375824e+000}, {-8.554766922517680e+002, -5.510143617018551e+001, -7.075055206156207e+000, 1.693783239359508e+000}, {-8.945408949715370e+002, -5.659285841589260e+001, -7.143067888804235e+000, 1.689458970942078e+000}, {-9.336319987298878e+002, -5.812951993780914e+001, -7.211759170575077e+000, 1.685165765209987e+000}, {-9.748070167487797e+002, -5.970600021779582e+001, -7.281141145128838e+000, 1.680903383115732e+000}, {-1.017797161099866e+003, -6.132462116161106e+001, -7.351218688149022e+000, 1.676671586493826e+000}, {-1.062683684456890e+003, -6.298650584026640e+001, -7.421998074872098e+000, 1.672470140103473e+000}, {-1.109551487921742e+003, -6.469280936461375e+001, -7.493485645708336e+000, 1.668298811149195e+000}, {-1.158489280848639e+003, -6.644471976211437e+001, -7.565687809091735e+000, 1.664157369235286e+000}, {-1.209589622929883e+003, -6.824345904605310e+001, -7.638611044176486e+000, 1.660045586320429e+000}, {-1.262949116295308e+003, -7.009028388353097e+001, -7.712261903592744e+000, 1.655963236672332e+000}, {-1.318668341934761e+003, -7.198648673326566e+001, -7.786647015696424e+000, 1.651910096822905e+000}, {-1.378234074012758e+003, -7.393131086849101e+001, -7.861773788109874e+000, 1.647885945523818e+000}, {-1.442265174772427e+003, -7.592462610887877e+001, -7.937649607034357e+000, 1.643890563702867e+000}, {-1.511481571183503e+003, -7.796562144522292e+001, -8.014283223597630e+000, 1.639923734421354e+000}, {-1.586722725865685e+003, -8.026846357280738e+001, -8.098960584225544e+000, 1.635617913591073e+000}, {-1.668825233579326e+003, -8.268094464949859e+001, -8.186167899408106e+000, 1.631266166750611e+000}, {-1.751894172577563e+003, -8.522351785771092e+001, -8.276024876610972e+000, 1.626867109373032e+000}, {-1.847472908821830e+003, -8.787922432899295e+001, -8.368677996958821e+000, 1.622419288010019e+000}, {-1.946491084587533e+003, -9.068266529460510e+001, -8.464253729359603e+000, 1.617921174006821e+000}, {-2.053978412575095e+003, -9.363129093939379e+001, -8.562915811238364e+000, 1.613371162616998e+000}, {-2.169922983260752e+003, -9.673869964576350e+001, -8.664826684394154e+000, 1.608767561543369e+000}, {-2.295197216249843e+003, -1.000170527865890e+002, -8.770163821231140e+000, 1.604108587787459e+000}, {-2.430785442417563e+003, -1.034797311140764e+002, -8.879118505262669e+000, 1.599392359891713e+000}, {-2.577801026632838e+003, -1.071414867500088e+002, -8.991897240203381e+000, 1.594616890192578e+000}, {-2.737506932622896e+003, -1.110186171022473e+002, -9.108723338184889e+000, 1.589780076274507e+000}, {-2.911339732177706e+003, -1.151291662320393e+002, -9.229838713457117e+000, 1.584879691522251e+000}, {-3.100938315519513e+003, -1.194931571822090e+002, -9.355505914598652e+000, 1.579913374653328e+000}, {-3.308178313364557e+003, -1.241328611774950e+002, -9.486010432931598e+000, 1.574878618093619e+000}, {-3.535213086559769e+003, -1.290731114786262e+002, -9.621663331956111e+000, 1.569772755037022e+000}, {-3.784523337361933e+003, -1.343416691601648e+002, -9.762804252286623e+000, 1.564592945003904e+000}, {-4.058976804769223e+003, -1.399696522448835e+002, -9.909804855879647e+000, 1.559336157681449e+000}, {-4.361900810987743e+003, -1.459920399445989e+002, -1.006307278746745e+001, 1.553999154791511e+000}, {-4.697170408589283e+003, -1.524482682241805e+002, -1.022305624605249e+001, 1.548578469686142e+000}, {-5.069316087288656e+003, -1.593829353173996e+002, -1.039024927974076e+001, 1.543070384316234e+000}, {-5.483655244797968e+003, -1.668466418498926e+002, -1.056519794058328e+001, 1.537470903152320e+000}, {-5.946453353590173e+003, -1.748969941438437e+002, -1.074850746706512e+001, 1.531775723555805e+000}, {-6.470411734157407e+003, -1.835913253448870e+002, -1.094085372023932e+001, 1.525980202000237e+000}, {-7.043187071142154e+003, -1.930474897871591e+002, -1.114297042220280e+001, 1.520079315421806e+000}, {-7.711933761545253e+003, -2.032681531837435e+002, -1.135574068537159e+001, 1.514067616830629e+000}, {-8.457541572806047e+003, -2.144378352863221e+002, -1.158005302355428e+001, 1.507939179287543e+000}, {-9.310061797401568e+003, -2.266315901380780e+002, -1.181700489705569e+001, 1.501687551187454e+000}, {-1.028612055393173e+004, -2.399941811640263e+002, -1.206779207559716e+001, 1.495305668348962e+000}, {-1.140941135333577e+004, -2.546856980795141e+002, -1.233378335486043e+001, 1.488785783609071e+000}, {-1.270936879154894e+004, -2.708950151984197e+002, -1.261654223773738e+001, 1.482119368890722e+000}, {-1.422287401592777e+004, -2.888462620691078e+002, -1.291786359692049e+001, 1.475297002040697e+000}, {-1.599657041959849e+004, -3.088070502047960e+002, -1.323981906420460e+001, 1.468308231964111e+000}, {-1.809004382910636e+004, -3.310990266311976e+002, -1.358481369182558e+001, 1.461141416796625e+000}, {-2.058025235609896e+004, -3.561115228088582e+002, -1.395565733388563e+001, 1.453783528293291e+000}, {-2.356778164825942e+004, -3.843193794845403e+002, -1.435565545638336e+001, 1.446219913491735e+000}, {-2.718581393635035e+004, -4.163064498517156e+002, -1.478872591139304e+001, 1.438434001794348e+000}, {-3.161318342772590e+004, -4.527969315541988e+002, -1.525955085475275e+001, 1.430406941556335e+000}, {-3.710725880796295e+004, -4.946758410121437e+002, -1.577378467014829e+001, 1.422117144536678e+000}, {-4.396402756633394e+004, -5.431789342797099e+002, -1.633827036775515e+001, 1.413539708353296e+000}, {-5.273878604465988e+004, -5.996007772929380e+002, -1.696157367093251e+001, 1.404645675124891e+000}, {-6.407490165878369e+004, -6.659850765960473e+002, -1.765417026519191e+001, 1.395401054361457e+000}, {-7.904776202793667e+004, -7.447144090654609e+002, -1.842951853955794e+001, 1.385765574728270e+000}, {-9.926766194745344e+004, -8.390379482241829e+002, -1.930485080869606e+001, 1.375690927154873e+000}, {-1.273095825071022e+005, -9.532400246900349e+002, -2.030280355373593e+001, 1.365118418615304e+000}, {-1.674562138620607e+005, -1.092995537492447e+003, -2.145377509289186e+001, 1.353975640846269e+000}, {-2.272198539857275e+005, -1.265667878999013e+003, -2.279974485733070e+001, 1.342171664024869e+000}, {-3.205446595310491e+005, -1.480352798561740e+003, -2.440067528762999e+001, 1.329589871746868e+000}, {-4.757004171335059e+005, -1.745534398035771e+003, -2.634638501786641e+001, 1.316076873033058e+000}, {-7.277874286905073e+005, -2.104072122532931e+003, -2.876388941251714e+001, 1.301424533021810e+000}, {-1.158269327743195e+006, -2.638285390348861e+003, -3.185184183947293e+001, 1.285339230723633e+000}, {-1.959002083113540e+006, -3.532523254029535e+003, -3.592185311796366e+001, 1.267385221467623e+000}, {-3.596005342715708e+006, -3.892843755514912e+003, -4.023498447113133e+001, 1.252292339125201e+000}, {-7.337307110377396e+006, -3.598464703219049e+003, -4.636507095266333e+001, 1.235307228164419e+000}, {-1.605717610482991e+007, -3.192373063618026e+003, -5.564203153249309e+001, 1.214800598771303e+000}, {-3.619723757401069e+007, -6.348690590601902e+003, -6.892252303788369e+001, 1.191007390096094e+000}, {-8.251938353183653e+007, -1.344847387729171e+004, -8.574522926416114e+001, 1.168783843757871e+000}, {-2.040599684077241e+008, -1.069786567964125e+004, -1.061289066146439e+002, 1.150438269719548e+000}, {-5.121867096552509e+008, 1.456135255074863e+004, -1.331224976907382e+002, 1.133590960463892e+000}, {-1.208283139664454e+009, 1.667633657013471e+004, -1.694988472006016e+002, 1.115407413029604e+000}, {-2.713573785229895e+009, -2.376510051574436e+005, -2.229542789125316e+002, 1.093221801784601e+000}, {-5.496859099556583e+009, -4.498406556454219e+005, -2.698490407608527e+002, 1.081141798451352e+000}, {-1.281003550443853e+010, -1.454397978181947e+005, -3.068797733762085e+002, 1.075521362860637e+000}, {-3.803997568288454e+010, 7.244080350807136e+005, -3.862216588136204e+002, 1.065839017041407e+000}, {-1.282594878189864e+011, 3.702125120753230e+006, -5.256416986606254e+002, 1.055309382343471e+000}, {-4.144936674091608e+011, 2.980984288420191e+006, -6.792080235153045e+002, 1.041458768913244e+000}, {-8.964984509224136e+011, -5.905430282043110e+006, -8.516642522807352e+002, 1.031926967645291e+000}, {-7.355836465490852e+015, 4.518177895310270e+010, -6.933068977626010e+004, 1.025706312840203e+000}};
        constant Real[Ninterval, 4] dvtcoef = {{3.665200884480820e-001, 3.768263375039013e-002, 2.190792329657455e-003, 5.503960828374045e-005}, {3.740100562223658e-001, 3.839687254278058e-002, 2.241470067816089e-003, 5.651517998086539e-005}, {3.828499115332361e-001, 3.899054527468819e-002, 2.284797395124333e-003, 5.778081864475580e-005}, {3.933222065720126e-001, 4.041224383597725e-002, 2.385959187504062e-003, 6.075322570198209e-005}, {4.040330124879073e-001, 4.188279732592867e-002, 2.491507351266832e-003, 6.387796804156671e-005}, {4.149858832885122e-001, 4.340369917802586e-002, 2.601623832550185e-003, 6.716271877677985e-005}, {4.261843979498587e-001, 4.497658031312836e-002, 2.716504756553497e-003, 7.061574208320139e-005}, {4.376320460159308e-001, 4.660304907537514e-002, 2.836349505162887e-003, 7.424557939372635e-005}, {4.493322760556361e-001, 4.828479016768840e-002, 2.961367887534022e-003, 7.806127539077438e-005}, {4.612885541291512e-001, 5.002349607708437e-002, 3.091775532306176e-003, 8.207225038864295e-005}, {4.735043599311913e-001, 5.182093843609277e-002, 3.227799262656286e-003, 8.628847726670128e-005}, {4.859830663248401e-001, 5.367889630080813e-002, 3.369672145850532e-003, 9.072034242183301e-005}, {4.987281155931236e-001, 5.559927072115295e-002, 3.517642230639391e-003, 9.537893269130447e-005}, {5.117427716648794e-001, 5.758397405257353e-002, 3.671964680832330e-003, 1.002758063548368e-004}, {5.250302993025363e-001, 5.963492463599265e-002, 3.832901617045873e-003, 1.054230003569787e-004}, {5.385939624651210e-001, 6.175417109476790e-002, 4.000731818279412e-003, 1.108333568850559e-004}, {5.524368943414315e-001, 6.394372783418179e-002, 4.175738556303638e-003, 1.165201514339943e-004}, {5.664900425757722e-001, 6.620605134392597e-002, 4.358222679301490e-003, 1.224975427286338e-004}, {5.805583121436652e-001, 6.854424938506180e-002, 4.548492850733371e-003, 1.287802891593489e-004}, {5.946086214220634e-001, 7.096102605514981e-002, 4.746869857737302e-003, 1.353838849828342e-004}, {6.086065747191686e-001, 7.340678828072091e-002, 4.949478977042500e-003, 1.421827481986982e-004}, {6.225914771181178e-001, 7.579030259624107e-002, 5.148953763370412e-003, 1.489285080947021e-004}, {6.367315613084956e-001, 7.822925272680593e-002, 5.354873479106979e-003, 1.559450852216557e-004}, {6.510248604875336e-001, 8.072426859544872e-002, 5.567387089688100e-003, 1.632411903229065e-004}, {6.654693630800921e-001, 8.327597242098897e-002, 5.786645233200854e-003, 1.708257343021365e-004}, {6.800630053891468e-001, 8.588497856777670e-002, 6.012800199529966e-003, 1.787078304444174e-004}, {6.948036854574047e-001, 8.855189328122257e-002, 6.246005909138952e-003, 1.868967966091155e-004}, {7.096892568246661e-001, 9.127731455960417e-002, 6.486417891145978e-003, 1.954021573940052e-004}, {7.247175349874265e-001, 9.406183194968448e-002, 6.734193261069997e-003, 2.042336462698481e-004}, {7.398863032918654e-001, 9.690602637057491e-002, 6.989490698063166e-003, 2.134012076848844e-004}, {7.551933077292230e-001, 9.981047000578269e-002, 7.252470421664215e-003, 2.229149991386050e-004}, {7.706362681974270e-001, 1.027757261083910e-001, 7.523294168294181e-003, 2.327853932242564e-004}, {7.862128763870087e-001, 1.058023489048649e-001, 7.802125167244362e-003, 2.430229796396210e-004}, {8.019208000233606e-001, 1.088908834647968e-001, 8.089128116419406e-003, 2.536385671655344e-004}, {8.177576857661010e-001, 1.120418655935747e-001, 8.384469157748138e-003, 2.646431856117704e-004}, {8.337211625135900e-001, 1.152558217345552e-001, 8.688315852326128e-003, 2.760480877298891e-004}, {8.498088435549523e-001, 1.185332688891191e-001, 9.000837155310458e-003, 2.878647510927008e-004}, {8.660183298445744e-001, 1.218747145409911e-001, 9.322203390615966e-003, 3.001048799400485e-004}, {8.823472121350809e-001, 1.252806565984050e-001, 9.652586225422304e-003, 3.127804069906578e-004}, {8.987930767579581e-001, 1.287515833300165e-001, 9.992158644551460e-003, 3.259034952198205e-004}, {9.153535027634360e-001, 1.322879733585061e-001, 1.034109492467412e-002, 3.394865396027454e-004}, {9.320260723490239e-001, 1.358902955896432e-001, 1.069957060853094e-002, 3.535421688234263e-004}, {9.488083651027407e-001, 1.395590092481384e-001, 1.106776247896205e-002, 3.680832469489624e-004}, {9.656979703504189e-001, 1.432945638174619e-001, 1.144584853306668e-002, 3.831228750692304e-004}, {9.826924805368605e-001, 1.470973991054695e-001, 1.183400795620025e-002, 3.986743929019505e-004}, {9.997894997984816e-001, 1.509679452225744e-001, 1.223242109616551e-002, 4.147513803631108e-004}, {1.016986646503764e+000, 1.549066226184312e-001, 1.264126943736927e-002, 4.313676591028885e-004}, {1.034281553150528e+000, 1.589138421395006e-001, 1.306073557508202e-002, 4.485372940071116e-004}, {1.051671869872525e+000, 1.629900050717103e-001, 1.349100318986285e-002, 4.662745946644320e-004}, {1.069155270937662e+000, 1.671355031844444e-001, 1.393225702213025e-002, 4.845941167994041e-004}, {1.086729447284750e+000, 1.713507188579044e-001, 1.438468284683222e-002, 5.035106636716927e-004}, {1.104392125557418e+000, 1.756360250733073e-001, 1.484846744851311e-002, 5.230392874416344e-004}, {1.122141050214495e+000, 1.799917856279029e-001, 1.532379859632389e-002, 5.431952905025172e-004}, {1.139974004762330e+000, 1.844183551212992e-001, 1.581086501965071e-002, 5.639942267798111e-004}, {1.157888800935349e+000, 1.889160791550389e-001, 1.630985638370746e-002, 5.854519029978606e-004}, {1.175883287892217e+000, 1.934852944091692e-001, 1.682096326567359e-002, 6.075843799142949e-004}, {1.193955352468680e+000, 1.981263287911221e-001, 1.734437713105463e-002, 6.304079735227150e-004}, {1.212102920438635e+000, 2.028395015817585e-001, 1.788029031044544e-002, 6.539392562240593e-004}, {1.230323959893689e+000, 2.076251235778936e-001, 1.842889597669781e-002, 6.781950579671869e-004}, {1.248616484149628e+000, 2.124834972498307e-001, 1.899038812248679e-002, 7.031924673592142e-004}, {1.266978553097518e+000, 2.174149169184600e-001, 1.956496153830370e-002, 7.289488327461666e-004}, {1.285408269937146e+000, 2.224196689540113e-001, 2.015281179091836e-002, 7.554817632645458e-004}, {1.303903795153633e+000, 2.274980318933519e-001, 2.075413520241562e-002, 7.828091298644555e-004}, {1.322463336930379e+000, 2.326502767088909e-001, 2.136912882953291e-002, 8.109490663049821e-004}, {1.341085156891751e+000, 2.378766669763421e-001, 2.199799044372205e-002, 8.399199701224019e-004}, {1.359767575223034e+000, 2.431774590683698e-001, 2.264091851168786e-002, 8.697405035720730e-004}, {1.378508965237237e+000, 2.485529024112762e-001, 2.329811217643337e-002, 9.004295945446784e-004}, {1.397307764139623e+000, 2.540032396549401e-001, 2.396977123901983e-002, 9.320064374575882e-004}, {1.416162464351274e+000, 2.595287069634087e-001, 2.465609614076989e-002, 9.644904941221798e-004}, {1.435071623389463e+000, 2.651295341966611e-001, 2.535728794626573e-002, 9.979014945878779e-004}, {1.454033862914496e+000, 2.708059451751824e-001, 2.607354832685021e-002, 1.032259437963822e-003}, {1.473047868841412e+000, 2.765581579370460e-001, 2.680507954482812e-002, 1.067584593218963e-003}, {1.492112386818882e+000, 2.823863850138245e-001, 2.755208443834308e-002, 1.103897499961510e-003}, {1.511226244667304e+000, 2.882908335802246e-001, 2.831476640706065e-002, 1.141218969198648e-003}, {1.530388322738662e+000, 2.942717059001999e-001, 2.909332939819350e-002, 1.179570084077442e-003}, {1.549597582529973e+000, 3.003291994252179e-001, 2.988797789375669e-002, 1.218972200607816e-003}, {1.568853054060884e+000, 3.064635071695984e-001, 3.069891689809663e-002, 1.259446948368720e-003}, {1.588153834283365e+000, 3.126748179903948e-001, 3.152635192639908e-002, 1.301016231198290e-003}, {1.607499102743188e+000, 3.189633167940680e-001, 3.237048899399019e-002, 1.343702227869144e-003}, {1.626888102366270e+000, 3.253291849447598e-001, 3.323153460615338e-002, 1.387527392749791e-003}, {1.646320164489830e+000, 3.317726004172473e-001, 3.410969574909326e-002, 1.432514456453111e-003}, {1.665794682688201e+000, 3.382937382564183e-001, 3.500517988125847e-002, 1.478686426473104e-003}, {1.685311138825627e+000, 3.448927707221959e-001, 3.591819492593865e-002, 1.526066587810795e-003}, {1.704869084047406e+000, 3.515698677202811e-001, 3.684894926422587e-002, 1.574678503590567e-003}, {1.724468158770434e+000, 3.583251969986714e-001, 3.779765172916720e-002, 1.624546015667824e-003}, {1.744108067700997e+000, 3.651589246073122e-001, 3.876451160038527e-002, 1.675693245229228e-003}, {1.763788610501295e+000, 3.720712150319988e-001, 3.974973859999501e-002, 1.728144593386518e-003}, {1.783509661384779e+000, 3.790622316554051e-001, 4.075354288886299e-002, 1.781924741765163e-003}, {1.803271169098520e+000, 3.861321370576751e-001, 4.177613506405691e-002, 1.837058653088839e-003}, {1.823073183620778e+000, 3.932810932079064e-001, 4.281772615719370e-002, 1.893571571760980e-003}, {1.842915816106496e+000, 4.005092620304885e-001, 4.387852763320605e-002, 1.951489024444566e-003}, {1.862799271892769e+000, 4.078168055176666e-001, 4.495875139067014e-002, 2.010836820641158e-003}, {1.882723840320363e+000, 4.152038861479577e-001, 4.605860976245487e-002, 2.071641053270659e-003}, {1.902689896843988e+000, 4.226706672139174e-001, 4.717831551747944e-002, 2.133928099252665e-003}, {1.922697890394811e+000, 4.302173132196612e-001, 4.831808186330070e-002, 2.197724620090751e-003}, {1.942748370153087e+000, 4.378439900664124e-001, 4.947812244984428e-002, 2.263057562460898e-003}, {1.962841961294967e+000, 4.455508655515563e-001, 5.065865137358835e-002, 2.329954158805294e-003}, {1.982979377414200e+000, 4.533381096250566e-001, 5.185988318307852e-002, 2.398441927932571e-003}, {2.003161423573253e+000, 4.612058947322305e-001, 5.308203288516229e-002, 2.468548675625884e-003}, {2.023388978688615e+000, 4.691543962461522e-001, 5.432531595209598e-002, 2.540302495259932e-003}, {2.043663032156556e+000, 4.771837926209295e-001, 5.558994832988842e-002, 2.613731768428245e-003}, {2.063984632955979e+000, 4.852942660390235e-001, 5.687614644693947e-002, 2.688865165581934e-003}, {2.084354942844196e+000, 4.934860024349729e-001, 5.818412722451828e-002, 2.765731646681043e-003}, {2.104775202157050e+000, 5.017591920850383e-001, 5.951410808727019e-002, 2.844360461860065e-003}, {2.125246731719251e+000, 5.101140299143760e-001, 6.086630697524879e-002, 2.924781152108464e-003}, {2.145770960466568e+000, 5.185507157095545e-001, 6.224094235686584e-002, 3.007023549967835e-003}, {2.166349393201155e+000, 5.270694546814225e-001, 6.363823324234659e-002, 3.091117780246803e-003}, {2.186983629134377e+000, 5.356704576934424e-001, 6.505839919869993e-002, 3.177094260754839e-003}, {2.207675360445951e+000, 5.443539416465710e-001, 6.650166036532459e-002, 3.264983703056519e-003}, {2.228426369973906e+000, 5.531201298632031e-001, 6.796823747062697e-002, 3.354817113247326e-003}, {2.249238522655806e+000, 5.619692524824012e-001, 6.945835184963980e-002, 3.446625792752342e-003}, {2.270113792409218e+000, 5.709015466811348e-001, 7.097222546277789e-002, 3.540441339149131e-003}, {2.291054235680260e+000, 5.799172572176781e-001, 7.251008091513399e-002, 3.636295647016205e-003}, {2.312061997510860e+000, 5.890166367384575e-001, 7.407214147719843e-002, 3.734220908808123e-003}, {2.333139328790219e+000, 5.981999460648496e-001, 7.565863110647320e-002, 3.834249615758862e-003}, {2.354288566264979e+000, 6.074674546965987e-001, 7.726977446981624e-002, 3.936414558814613e-003}, {2.375512137218167e+000, 6.168194411357968e-001, 7.890579696711332e-002, 4.040748829597290e-003}, {2.396812574191011e+000, 6.262561931938135e-001, 8.056692475586602e-002, 4.147285821400253e-003}, {2.418192505865444e+000, 6.357780084481650e-001, 8.225338477660042e-002, 4.256059230217468e-003}, {2.439654638189181e+000, 6.453851947021946e-001, 8.396540477941994e-002, 4.367103055807411e-003}, {2.461201802244242e+000, 6.550780701048645e-001, 8.570321335190975e-002, 4.480451602793228e-003}, {2.482836910050617e+000, 6.648569638275532e-001, 8.746703994730097e-002, 4.596139481800508e-003}, {2.504562958561007e+000, 6.747222163788872e-001, 8.925711491435891e-002, 4.714201610633784e-003}, {2.526383089076794e+000, 6.846741796996655e-001, 9.107366952835992e-002, 4.834673215493565e-003}, {2.548300479768189e+000, 6.947132181729661e-001, 9.291693602203711e-002, 4.957589832234993e-003}, {2.570318477197365e+000, 7.048397082466733e-001, 9.478714761948791e-002, 5.082987307669528e-003}, {2.592440465382741e+000, 7.150540396336219e-001, 9.668453856884922e-002, 5.210901800911450e-003}, {2.614669982458294e+000, 7.253566149948452e-001, 9.860934417835919e-002, 5.341369784769924e-003}, {2.637010642561412e+000, 7.357478508657601e-001, 1.005618008515189e-001, 5.474428047188929e-003}, {2.659466158279287e+000, 7.462281778772136e-001, 1.025421461246414e-001, 5.610113692735653e-003}, {2.682040381265263e+000, 7.567980409850534e-001, 1.045506187051913e-001, 5.748464144139502e-003}, {2.704737228505560e+000, 7.674579003195780e-001, 1.065874585105618e-001, 5.889517143882839e-003}, {2.727560756610687e+000, 7.782082311537123e-001, 1.086529067092295e-001, 6.033310755844878e-003}, {2.750515116663080e+000, 7.890495246405542e-001, 1.107472057618116e-001, 6.179883367000695e-003}, {2.773604564123278e+000, 7.999822881587377e-001, 1.128705994640319e-001, 6.329273689176290e-003}, {2.796833481101817e+000, 8.110070456381986e-001, 1.150233329907212e-001, 6.481520760861736e-003}, {2.820206359500705e+000, 8.221243381278066e-001, 1.172056529406883e-001, 6.636663949083662e-003}, {2.843727786952501e+000, 8.333347242992452e-001, 1.194178073829910e-001, 6.794742951338803e-003}, {2.867402504168878e+000, 8.446387806027250e-001, 1.216600459047351e-001, 6.955797797590129e-003}, {2.891235314964451e+000, 8.560371022382086e-001, 1.239326196590929e-001, 7.119868852327294e-003}, {2.915231207685412e+000, 8.675303029206455e-001, 1.262357814161839e-001, 7.286996816692764e-003}, {2.939395228384068e+000, 8.791190161294715e-001, 1.285697856130808e-001, 7.457222730675805e-003}, {2.963732594119631e+000, 8.908038948858174e-001, 1.309348884073418e-001, 7.630587975375205e-003}, {2.988248610630383e+000, 9.025856127699617e-001, 1.333313477298497e-001, 7.807134275333398e-003}, {3.012948756836480e+000, 9.144648639316437e-001, 1.357594233405432e-001, 7.986903700942813e-003}, {3.037838552477580e+000, 9.264423643191631e-001, 1.382193768838317e-001, 8.169938670926896e-003}, {3.062923779323181e+000, 9.385188509558488e-001, 1.407114719478523e-001, 8.356281954897082e-003}, {3.088210210949279e+000, 9.506950840800934e-001, 1.432359741212169e-001, 8.545976675988148e-003}, {3.113703846840580e+000, 9.629718461669161e-001, 1.457931510558516e-001, 8.739066313572597e-003}, {3.139410816633882e+000, 9.753499432105148e-001, 1.483832725273125e-001, 8.935594706057733e-003}, {3.165337344775792e+000, 9.878302053998029e-001, 1.510066104981779e-001, 9.135606053765535e-003}, {3.191489836188858e+000, 1.000413487150973e+000, 1.536634391835181e-001, 9.339144921898232e-003}, {3.217874837362654e+000, 1.013100667985528e+000, 1.563540351164809e-001, 9.546256243591650e-003}, {3.244499017871583e+000, 1.025892653177059e+000, 1.590786772160487e-001, 9.756985323057694e-003}, {3.271369233348681e+000, 1.038790373981308e+000, 1.618376468567759e-001, 9.971377838818435e-003}, {3.298492456712199e+000, 1.051794788668075e+000, 1.646312279389412e-001, 1.018947984703397e-002}, {3.325875855766680e+000, 1.064906882678220e+000, 1.674597069616482e-001, 1.041133778492568e-002}, {3.353526720988624e+000, 1.078127669689009e+000, 1.703233730962368e-001, 1.063699847429761e-002}, {3.381452539482126e+000, 1.091458191811743e+000, 1.732225182626495e-001, 1.086650912515772e-002}, {3.409660934355382e+000, 1.104899520630803e+000, 1.761574372062440e-001, 1.109991733944163e-002}, {3.438159712928544e+000, 1.118452757551656e+000, 1.791284275774465e-001, 1.133727111484079e-002}, {3.466956886644510e+000, 1.132119034446490e+000, 1.821357900125424e-001, 1.157861884873769e-002}, {3.496060578876284e+000, 1.145899514855565e+000, 1.851798282158324e-001, 1.182400934225040e-002}, {3.525479158448411e+000, 1.159795393897511e+000, 1.882608490453275e-001, 1.207349180438865e-002}, {3.555221150625067e+000, 1.173807899612132e+000, 1.913791625981357e-001, 1.232711585632465e-002}, {3.585295253530584e+000, 1.187938293517513e+000, 1.945350822993449e-001, 1.258493153578014e-002}, {3.615710419947444e+000, 1.202187870979279e+000, 1.977289249926884e-001, 1.284698930153326e-002}, {3.646475741855814e+000, 1.216557962688612e+000, 2.009610110319655e-001, 1.311334003804746e-002}, {3.677600534221765e+000, 1.231049934891233e+000, 2.042316643764733e-001, 1.338403506022480e-002}, {3.709094341468303e+000, 1.245665190249085e+000, 2.075412126872400e-001, 1.365912611828753e-002}, {3.740966909400527e+000, 1.260405168872820e+000, 2.108899874254873e-001, 1.393866540278966e-002}, {3.773228176367919e+000, 1.275271349170827e+000, 2.142783239537165e-001, 1.422270554976199e-002}, {3.805888377290767e+000, 1.290265248201987e+000, 2.177065616394706e-001, 1.451129964599359e-002}, {3.838957884662454e+000, 1.305388423503501e+000, 2.211750439596317e-001, 1.480450123445293e-002}, {3.872447399709886e+000, 1.320642472860455e+000, 2.246841186100514e-001, 1.510236431985103e-002}, {3.906367799894601e+000, 1.336029036255819e+000, 2.282341376142729e-001, 1.540494337435146e-002}, {3.940730251550222e+000, 1.351549796109007e+000, 2.318254574376806e-001, 1.571229334342844e-002}, {3.975546173467691e+000, 1.367206478594881e+000, 2.354584391022789e-001, 1.602446965187828e-002}, {4.010827215553952e+000, 1.383000854730738e+000, 2.391334483049296e-001, 1.634152820998645e-002}, {4.046585350875875e+000, 1.398934740935574e+000, 2.428508555387394e-001, 1.666352541985450e-002}, {4.082832758036515e+000, 1.415010000798975e+000, 2.466110362157002e-001, 1.699051818189041e-002}, {4.119582040102078e+000, 1.431228545024562e+000, 2.504143707946231e-001, 1.732256390146564e-002}, {4.156845889599819e+000, 1.447592334476850e+000, 2.542612449081177e-001, 1.765972049574360e-002}, {4.194637505310675e+000, 1.464103378865951e+000, 2.581520494986178e-001, 1.800204640068241e-002}, {4.232970236952464e+000, 1.480763740213331e+000, 2.620871809503179e-001, 1.834960057821798e-002}, {4.271857846363231e+000, 1.497575532404973e+000, 2.660670412307083e-001, 1.870244252362888e-002}, {4.311314452870784e+000, 1.514540923036616e+000, 2.700920380312623e-001, 1.906063227308990e-002}, {4.351354365658449e+000, 1.531662135547257e+000, 2.741625849122226e-001, 1.942423041141704e-002}, {4.391992415882795e+000, 1.548941448644654e+000, 2.782791014540191e-001, 1.979329808000918e-002}, {4.433243686970854e+000, 1.566381199477114e+000, 2.824420134068151e-001, 2.016789698499184e-002}, {4.475123641314885e+000, 1.583983784131202e+000, 2.866517528484842e-001, 2.054808940556589e-002}, {4.517648207096858e+000, 1.601751658807016e+000, 2.909087583444591e-001, 2.093393820256836e-002}, {4.560833550448293e+000, 1.619687342607780e+000, 2.952134751100056e-001, 2.132550682724936e-002}, {4.604696396499985e+000, 1.637793417179381e+000, 2.995663551808326e-001, 2.172285933027002e-002}, {4.649253796372554e+000, 1.656072529916263e+000, 3.039678575822974e-001, 2.212606037092869e-002}, {4.694523265092098e+000, 1.674527394692921e+000, 3.084184485074046e-001, 2.253517522661828e-002}, {4.740522712955659e+000, 1.693160794008574e+000, 3.129186014966691e-001, 2.295026980252321e-002}, {4.787270576136158e+000, 1.711975580058748e+000, 3.174687976240633e-001, 2.337141064156014e-002}, {4.834785712163620e+000, 1.730974677253755e+000, 3.220695256857244e-001, 2.379866493456965e-002}, {4.883087485591644e+000, 1.750161083541156e+000, 3.267212823955581e-001, 2.423210053076392e-002}, {4.932195710571193e+000, 1.769537872699258e+000, 3.314245725842074e-001, 2.467178594843879e-002}, {4.982130846586524e+000, 1.789108195340259e+000, 3.361799094048092e-001, 2.511779038595532e-002}, {5.032913690335282e+000, 1.808875282735387e+000, 3.409878145406674e-001, 2.557018373299858e-002}, {5.084565761601779e+000, 1.828842446624347e+000, 3.458488184239917e-001, 2.602903658212011e-002}, {5.137109093797749e+000, 1.849013082945023e+000, 3.507634604540620e-001, 2.649442024057316e-002}, {5.190566200241511e+000, 1.869390674036088e+000, 3.557322892248459e-001, 2.696640674244556e-002}, {5.244960447391115e+000, 1.889978789056404e+000, 3.607558627598103e-001, 2.744506886110065e-002}, {5.300315503010739e+000, 1.910781089611307e+000, 3.658347487465493e-001, 2.793048012193356e-002}, {5.356655984977632e+000, 1.931801328375829e+000, 3.709695247877092e-001, 2.842271481544970e-002}, {5.414006939214302e+000, 1.953043355008492e+000, 3.761607786475629e-001, 2.892184801067763e-002}, {5.472394280173591e+000, 1.974511116073609e+000, 3.814091085150813e-001, 2.942795556892076e-002}, {5.531844552982697e+000, 1.996208659464594e+000, 3.867151232667687e-001, 2.994111415786121e-002}, {5.592384986946350e+000, 2.018140136745433e+000, 3.920794427409197e-001, 3.046140126602264e-002}, {5.654043699138815e+000, 2.040309805072126e+000, 3.975026980190738e-001, 3.098889521760342e-002}, {5.716849502278652e+000, 2.062722031519653e+000, 4.029855317122437e-001, 3.152367518768998e-002}, {5.780832066643861e+000, 2.085381295218880e+000, 4.085285982589873e-001, 3.206582121785987e-002}, {5.846021876769548e+000, 2.108292190975333e+000, 4.141325642292472e-001, 3.261541423218720e-002}, {5.912450295234973e+000, 2.131459432255390e+000, 4.197981086381419e-001, 3.317253605366047e-002}, {5.980149653199785e+000, 2.154887854369196e+000, 4.255259232680015e-001, 3.373726942102519e-002}, {6.049153080922429e+000, 2.178582419018171e+000, 4.313167129985944e-001, 3.430969800606264e-002}, {6.119494843356140e+000, 2.202548216154015e+000, 4.371711961503690e-001, 3.488990643131756e-002}, {6.191210090835239e+000, 2.226790469534412e+000, 4.430901048323289e-001, 3.547798028828864e-002}, {6.264335024681378e+000, 2.251314539600350e+000, 4.490741853049658e-001, 3.607400615609312e-002}, {6.338906985205288e+000, 2.276125927352816e+000, 4.551241983514239e-001, 3.667807162062186e-002}, {6.414964364223934e+000, 2.301230279236725e+000, 4.612409196589499e-001, 3.729026529419736e-002}, {6.492546755911735e+000, 2.326633390771490e+000, 4.674251402137614e-001, 3.791067683575024e-002}, {6.571694919608382e+000, 2.352341211514662e+000, 4.736776667056718e-001, 3.853939697152988e-002}, {6.652450889812159e+000, 2.378359849304511e+000, 4.799993219463535e-001, 3.917651751636464e-002}, {6.734857960020156e+000, 2.404695575420992e+000, 4.863909452991119e-001, 3.982213139548871e-002}, {6.818960762407748e+000, 2.431354829314535e+000, 4.928533931226813e-001, 4.047633266695239e-002}, {6.904805348414572e+000, 2.458344223655918e+000, 4.993875392280697e-001, 4.113921654463405e-002}, {6.992439181126046e+000, 2.485670550059953e+000, 5.059942753490215e-001, 4.181087942187155e-002}, {7.081911236169231e+000, 2.513340784316515e+000, 5.126745116281625e-001, 4.249141889573267e-002}, {7.173272004697938e+000, 2.541362092459206e+000, 5.194291771172804e-001, 4.318093379194453e-002}, {7.266573617083629e+000, 2.569741836357898e+000, 5.262592202942233e-001, 4.387952419050236e-002}, {7.361869906936396e+000, 2.598487580031925e+000, 5.331656095952206e-001, 4.458729145197930e-002}, {7.459216347534869e+000, 2.627607096681667e+000, 5.401493339641285e-001, 4.530433824455851e-002}, {7.558670349451412e+000, 2.657108374111654e+000, 5.472114034211771e-001, 4.603076857181219e-002}, {7.660291079236791e+000, 2.686999623211421e+000, 5.543528496461990e-001, 4.676668780125031e-002}, {7.764139728783404e+000, 2.717289283930944e+000, 5.615747265851068e-001, 4.751220269366371e-002}, {7.870279478141036e+000, 2.747986033541306e+000, 5.688781110727400e-001, 4.826742143328870e-002}, {7.978775648099929e+000, 2.779098793974625e+000, 5.762641034782924e-001, 4.903245365881855e-002}, {8.089695695459456e+000, 2.810636740433291e+000, 5.837338283705550e-001, 4.980741049529092e-002}, {8.203109430764046e+000, 2.842609309100674e+000, 5.912884352069108e-001, 5.059240458687938e-002}, {8.319088992819991e+000, 2.875026206633575e+000, 5.989290990433451e-001, 5.138755013061993e-002}, {8.437709001965207e+000, 2.907897418922941e+000, 6.066570212701816e-001, 5.219296291110302e-002}, {8.559046630712071e+000, 2.941233220726810e+000, 6.144734303718776e-001, 5.300876033616472e-002}, {8.683181769475304e+000, 2.975044185275829e+000, 6.223795827131393e-001, 5.383506147361033e-002}, {8.810197102877758e+000, 3.009341194858236e+000, 6.303767633513405e-001, 5.467198708900607e-002}, {8.940178073287749e+000, 3.044135452087358e+000, 6.384662868774791e-001, 5.551965968457501e-002}, {9.073213421245114e+000, 3.079438488879136e+000, 6.466494982886871e-001, 5.637820353923723e-002}, {9.209394871242861e+000, 3.115262180887050e+000, 6.549277738848125e-001, 5.724774474983270e-002}, {9.348817398676971e+000, 3.151618758416641e+000, 6.633025222039423e-001, 5.812841127356715e-002}, {9.491579474545942e+000, 3.188520818472828e+000, 6.717751849883596e-001, 5.902033297172826e-002}, {9.637783262374123e+000, 3.225981337955358e+000, 6.803472381840313e-001, 5.992364165471453e-002}, {9.787534478331198e+000, 3.264013689063900e+000, 6.890201929749196e-001, 6.083847112842355e-002}, {9.940942732178327e+000, 3.302631652132191e+000, 6.977955968597331e-001, 6.176495724204956e-002}, {1.009812202265841e+001, 3.341849429269576e+000, 7.066750347651324e-001, 6.270323793734337e-002}, {1.025919010297298e+001, 3.381681664493669e+000, 7.156601301940823e-001, 6.365345329938531e-002}, {1.042426957493940e+001, 3.422143454409642e+000, 7.247525464307929e-001, 6.461574560892680e-002}, {1.059348749063613e+001, 3.463250369255583e+000, 7.339539877745911e-001, 6.559025939636394e-002}, {1.076697585846694e+001, 3.505018468612565e+000, 7.432662008315984e-001, 6.657714149739567e-002}, {1.094487160471041e+001, 3.547464321529919e+000, 7.526909758485252e-001, 6.757654111043801e-002}, {1.112731703673556e+001, 3.590605024361887e+000, 7.622301481021555e-001, 6.858860985585549e-002}, {1.131446020990981e+001, 3.634458220845387e+000, 7.718855993386348e-001, 6.961350183708254e-002}, {1.150645448169887e+001, 3.679042126940943e+000, 7.816592592654319e-001, 7.065137370370449e-002}, {1.170345995510019e+001, 3.724375546279324e+000, 7.915531071140711e-001, 7.170238471657543e-002}, {1.190564234904387e+001, 3.770477901775988e+000, 8.015691732425808e-001, 7.276669681505513e-002}, {1.211317480895882e+001, 3.817369250831911e+000, 8.117095408270398e-001, 7.384447468644079e-002}, {1.232623678221967e+001, 3.865070319658420e+000, 8.219763475927884e-001, 7.493588583769051e-002}, {1.254501551614812e+001, 3.913602522481210e+000, 8.323717876409130e-001, 7.604110066951828e-002}, {1.276970570264624e+001, 3.962987994216706e+000, 8.428981133273581e-001, 7.716029255296762e-002}, {1.300051006140023e+001, 4.013249618094449e+000, 8.535576372305411e-001, 7.829363790855323e-002}, {1.323763982202276e+001, 4.064411055841034e+000, 8.643527341958465e-001, 7.944131628808258e-002}, {1.348131510325956e+001, 4.116496780178591e+000, 8.752858434632413e-001, 8.060351045926384e-002}, {1.373176523992096e+001, 4.169532109259019e+000, 8.863594708830151e-001, 8.178040649321636e-002}, {1.398922935368295e+001, 4.223543241273606e+000, 8.975761912265102e-001, 8.297219385500314e-002}, {1.425395688597875e+001, 4.278557291550713e+000, 9.089386505921364e-001, 8.417906549731447e-002}, {1.452620788323131e+001, 4.334602332815509e+000, 9.204495689124428e-001, 8.540121795743366e-002}, {1.480625390155680e+001, 4.391707434003711e+000, 9.321117425728144e-001, 8.663885145762519e-002}, {1.509437843512102e+001, 4.449902704716440e+000, 9.439280471367345e-001, 8.789217000909319e-002}, {1.539087726927183e+001, 4.509219341703720e+000, 9.559014401917926e-001, 8.916138151966196e-002}, {1.569605974666963e+001, 4.569689672974459e+000, 9.680349643254816e-001, 9.044669790534268e-002}, {1.601024873436720e+001, 4.631347212300324e+000, 9.803317502206551e-001, 9.174833520595821e-002}, {1.633378209073554e+001, 4.694226707614670e+000, 9.927950199017923e-001, 9.306651370500110e-002}, {1.666701262562578e+001, 4.758364201213424e+000, 1.005428090113279e+000, 9.440145805391902e-002}, {1.701031014311637e+001, 4.823797081430276e+000, 1.018234375865878e+000, 9.575339740102105e-002}, {1.736406085678517e+001, 4.890564153189856e+000, 1.031217394122572e+000, 9.712256552522060e-002}, {1.772866946580232e+001, 4.958705695446524e+000, 1.044380767679338e+000, 9.850920097482693e-002}, {1.810455972114648e+001, 5.028263532980517e+000, 1.057728229205364e+000, 9.991354721162990e-002}, {1.849217554137902e+001, 5.099281108341590e+000, 1.071263625479878e+000, 1.013358527605123e-001}, {1.889198201807828e+001, 5.171803558649142e+000, 1.084990921826914e+000, 1.027763713648547e-001}, {1.930446710896825e+001, 5.245877793634559e+000, 1.098914206763288e+000, 1.042353621480049e-001}, {1.973014205980786e+001, 5.321552585652118e+000, 1.113037696859451e+000, 1.057130897811017e-001}, {2.016954403973725e+001, 5.398878652454267e+000, 1.127365741851098e+000, 1.072098246575545e-001}, {2.062323620612181e+001, 5.477908760805788e+000, 1.141902829977046e+000, 1.087258430745073e-001}, {2.109181035758059e+001, 5.558697820064092e+000, 1.156653593603386e+000, 1.102614274216203e-001}, {2.157588804150967e+001, 5.641302992575035e+000, 1.171622815106442e+000, 1.118168663775389e-001}, {2.207612293451024e+001, 5.725783803384387e+000, 1.186815433061205e+000, 1.133924551144240e-001}, {2.259320168853458e+001, 5.812202265487298e+000, 1.202236548730634e+000, 1.149884955109506e-001}, {2.312784738048727e+001, 5.900622998303927e+000, 1.217891432902972e+000, 1.166052963742056e-001}, {2.368082059232428e+001, 5.991113369373331e+000, 1.233785533055696e+000, 1.182431736709384e-001}, {2.425292225768141e+001, 6.083743633219402e+000, 1.249924480910505e+000, 1.199024507686403e-001}, {2.484499632728939e+001, 6.178587082451594e+000, 1.266314100373835e+000, 1.215834586869736e-001}, {2.545793213201633e+001, 6.275720210738291e+000, 1.282960415894291e+000, 1.232865363600859e-001}, {2.609266741838787e+001, 6.375222882468171e+000, 1.299869661270916e+000, 1.250120309103875e-001}, {2.675019182339740e+001, 6.477178512979792e+000, 1.317048288932363e+000, 1.267602979344092e-001}, {2.743154975809655e+001, 6.581674264927436e+000, 1.334502979714424e+000, 1.285317018013878e-001}, {2.813784419975337e+001, 6.688801252695598e+000, 1.352240653180803e+000, 1.303266159652730e-001}, {2.887024103476939e+001, 6.798654759803430e+000, 1.370268478512396e+000, 1.321454232908995e-001}, {2.962997256101324e+001, 6.911334476958677e+000, 1.388593885998045e+000, 1.339885163951065e-001}, {3.041834222256269e+001, 7.026944749204763e+000, 1.407224579184756e+000, 1.358562980036433e-001}, {3.123673065041534e+001, 7.145594837012060e+000, 1.426168547717874e+000, 1.377491813247640e-001}, {3.208659867260553e+001, 7.267399213620009e+000, 1.445434080900596e+000, 1.396675904404572e-001}, {3.296949558164943e+001, 7.392477853249294e+000, 1.465029782083446e+000, 1.416119607163322e-001}, {3.388706321309404e+001, 7.520956569516788e+000, 1.484964583852694e+000, 1.435827392312659e-001}, {3.484104425992344e+001, 7.652967352967133e+000, 1.505247764160289e+000, 1.455803852279572e-001}, {3.583328871585933e+001, 7.788648749509054e+000, 1.525888963393957e+000, 1.476053705856567e-001}, {3.686576173093582e+001, 7.928146259087701e+000, 1.546898202497619e+000, 1.496581803163948e-001}, {3.794055292138633e+001, 8.071612759939052e+000, 1.568285902204809e+000, 1.517393130861525e-001}, {3.905988423567278e+001, 8.219208975969522e+000, 1.590062903450447e+000, 1.538492817625102e-001}, {4.022612278917213e+001, 8.371103955820873e+000, 1.612240489091282e+000, 1.559886139904228e-001}, {4.144178802231272e+001, 8.527475626063863e+000, 1.634830406955019e+000, 1.581578527979038e-001}, {4.270956760743552e+001, 8.688511337873317e+000, 1.657844894438090e+000, 1.603575572335084e-001}, {4.403232828703730e+001, 8.854408498051546e+000, 1.681296704630431e+000, 1.625883030377006e-001}, {4.541313048616726e+001, 9.025375228265101e+000, 1.705199134198361e+000, 1.648506833502761e-001}, {4.685524481402835e+001, 9.201631075280597e+000, 1.729566053114142e+000, 1.671453094562547e-001}, {4.836216874299397e+001, 9.383407786556877e+000, 1.754411936372791e+000, 1.694728115728024e-001}, {4.993764414063455e+001, 9.570950149359556e+000, 1.779751897873626e+000, 1.718338396799571e-001}, {5.158568206789742e+001, 9.764516870993175e+000, 1.805601726664191e+000, 1.742290643981639e-001}, {5.331057910775004e+001, 9.964381592476455e+000, 1.831977925638632e+000, 1.766591779158699e-001}, {5.511694763067611e+001, 1.017083391162707e+001, 1.858897753082626e+000, 1.791248949706736e-001}, {5.700974172252563e+001, 1.038418054494647e+001, 1.886379267098844e+000, 1.816269538878832e-001}, {5.899428511832413e+001, 1.060474658549166e+001, 1.914441373282820e+000, 1.841661176805712e-001}, {6.107630828637804e+001, 1.083287683601744e+001, 1.943103875942831e+000, 1.867431752156427e-001}, {6.326198278685594e+001, 1.106893730143000e+001, 1.972387533071627e+000, 1.893589424508101e-001}, {6.555796198384681e+001, 1.131331679368660e+001, 2.002314115513010e+000, 1.920142637477639e-001}, {6.797142895789277e+001, 1.156642867112690e+001, 2.032906470660536e+000, 1.947100132673632e-001}, {7.051014259118738e+001, 1.182871277708803e+001, 2.064188591054464e+000, 1.974470964531552e-001}, {7.318249755499194e+001, 1.210063750757317e+001, 2.096185688455698e+000, 2.002264516101225e-001}, {7.599758305520270e+001, 1.238270212416496e+001, 2.128924273762562e+000, 2.030490515862381e-001}, {7.896525487015090e+001, 1.267543924938139e+001, 2.162432243469748e+000, 2.059159055650721e-001}, {8.209620970424231e+001, 1.297941764134553e+001, 2.196738973190282e+000, 2.088280609785455e-001}, {8.540207522795785e+001, 1.329524520372681e+001, 2.231875419045990e+000, 2.117866055497768e-001}, {8.889550374943779e+001, 1.362357234194895e+001, 2.267874227597210e+000, 2.147926694769878e-001}, {9.259028368620527e+001, 1.396509562361649e+001, 2.304769855305945e+000, 2.178474277705093e-001}, {9.651829128424471e+001, 1.432049438243640e+001, 2.342598758507877e+000, 2.209521027562080e-001}, {1.006827813095549e+002, 1.469059898846239e+001, 2.381399399667429e+000, 2.241079667600002e-001}, {1.050830383090181e+002, 1.507634001775476e+001, 2.421212499782385e+000, 2.273163449897145e-001}, {1.097159532822494e+002, 1.548598574903566e+001, 2.462761236361068e+000, 2.306327012079619e-001}, {1.145551746011481e+002, 1.590751402734798e+001, 2.504820150180328e+000, 2.339568479963077e-001}, {1.196071879274800e+002, 1.634031428204418e+001, 2.547304783567245e+000, 2.372813206159512e-001}, {1.248813064046870e+002, 1.678468557395838e+001, 2.590218919465339e+000, 2.406057414176631e-001}, {1.303872537968743e+002, 1.724093475889964e+001, 2.633566376628655e+000, 2.439297408582211e-001}, {1.361351893482151e+002, 1.770937668244954e+001, 2.677351009715642e+000, 2.472529573723386e-001}, {1.421357200157020e+002, 1.819033445609385e+001, 2.721576709359743e+000, 2.505750372458958e-001}, {1.483999250325510e+002, 1.868413966908177e+001, 2.766247402420666e+000, 2.538956344906715e-001}, {1.549393793818828e+002, 1.919113263274907e+001, 2.811367052239077e+000, 2.572144107206163e-001}, {1.617661691499202e+002, 1.971166267036393e+001, 2.856939658935786e+000, 2.605310350297162e-001}, {1.688929248323650e+002, 2.024608833018471e+001, 2.902969259872029e+000, 2.638451838715473e-001}, {1.763328352717475e+002, 2.079477771318220e+001, 2.949459930036376e+000, 2.671565409405210e-001}, {1.840996840769975e+002, 2.135810869815029e+001, 2.996415782636564e+000, 2.704647970548957e-001}, {1.922078676314314e+002, 2.193646927847552e+001, 3.043840969616666e+000, 2.737696500416193e-001}, {2.006724283454303e+002, 2.253025783125822e+001, 3.091739682337686e+000, 2.770708046228763e-001}, {2.095090814552011e+002, 2.313988343722962e+001, 3.140116152252167e+000, 2.803679723045419e-001}, {2.187342515473353e+002, 2.376576617580840e+001, 3.188974651669187e+000, 2.836608712664611e-001}, {2.283650918868312e+002, 2.440833750459543e+001, 3.238319494505725e+000, 2.869492262544885e-001}, {2.384195342056883e+002, 2.506804051982763e+001, 3.288155037218739e+000, 2.902327684744187e-001}, {2.489163138911711e+002, 2.574533035608010e+001, 3.338485679629001e+000, 2.935112354877263e-001}, {2.598750056133280e+002, 2.644067453977188e+001, 3.389315865882125e+000, 2.967843711090281e-001}, {2.713160677769612e+002, 2.715455332610006e+001, 3.440650085468187e+000, 3.000519253054244e-001}, {2.832608847937721e+002, 2.788746007968089e+001, 3.492492874226473e+000, 3.033136540975304e-001}, {2.957317919551732e+002, 2.863990172314495e+001, 3.544848815373224e+000, 3.065693194622797e-001}, {3.087521441999503e+002, 2.941239903121475e+001, 3.597722540707405e+000, 3.098186892373309e-001}, {3.223463482749902e+002, 3.020548711562214e+001, 3.651118731652995e+000, 3.130615370273069e-001}, {3.365398966901275e+002, 3.101971587811786e+001, 3.705042120439269e+000, 3.162976421114645e-001}, {3.513594557285981e+002, 3.185565030489221e+001, 3.759497491412099e+000, 3.195267893531296e-001}, {3.668328823476213e+002, 3.271387106452257e+001, 3.814489682103804e+000, 3.227487691105892e-001}, {3.829892914752884e+002, 3.359497489941416e+001, 3.870023584585280e+000, 3.259633771495039e-001}, {3.998591128097355e+002, 3.449957509744066e+001, 3.926104146732770e+000, 3.291704145568308e-001}, {4.174741627149792e+002, 3.542830194413114e+001, 3.982736373535119e+000, 3.323696876561623e-001}, {4.358676805476378e+002, 3.638180331176448e+001, 4.039925328339042e+000, 3.355610079244262e-001}, {4.550744161800611e+002, 3.736074507936995e+001, 4.097676134274746e+000, 3.387441919099929e-001}, {4.751306901524957e+002, 3.836581169669577e+001, 4.155993975546389e+000, 3.419190611519825e-001}, {4.960744587857732e+002, 3.939770673337638e+001, 4.214884098795508e+000, 3.450854421009716e-001}, {5.179454167679796e+002, 4.045715335623541e+001, 4.274351814507482e+000, 3.482431660408029e-001}, {5.407850064016650e+002, 4.154489509408396e+001, 4.334402498261381e+000, 3.513920690115978e-001}, {5.646366068557300e+002, 4.266169608345259e+001, 4.395041592346873e+000, 3.545319917339359e-001}, {5.895455040095308e+002, 4.380834204753403e+001, 4.456274606836136e+000, 3.576627795341116e-001}, {6.155612607453704e+002, 4.498563613130256e+001, 4.518107123265989e+000, 3.607842822704025e-001}, {6.427223576125481e+002, 4.619443542018386e+001, 4.580544778714099e+000, 3.638963542604584e-001}, {6.711025648658506e+002, 4.743553149708603e+001, 4.643593333070903e+000, 3.669988542095214e-001}, {7.007318473518636e+002, 4.870987626352949e+001, 4.707258525584453e+000, 3.700916451436019e-001}, {7.316684143282685e+002, 5.001834629107649e+001, 4.771546260300777e+000, 3.731745943241331e-001}, {7.639419550281848e+002, 5.136190630000230e+001, 4.836462461289155e+000, 3.762475732019301e-001}, {7.981063801427506e+002, 5.274052724808852e+001, 4.902013540645013e+000, 3.793104573307563e-001}, {8.326445111213358e+002, 5.415893280165836e+001, 4.968203781164648e+000, 3.823631262876042e-001}, {8.700148252485711e+002, 5.561171210577200e+001, 5.035043285535354e+000, 3.854054635554983e-001}, {9.083445124296225e+002, 5.710569757459630e+001, 5.102533934520626e+000, 3.884373570657309e-001}, {9.485314487146718e+002, 5.863925936997953e+001, 5.170684545601461e+000, 3.914586976410323e-001}, {9.905087195242919e+002, 6.021400337316415e+001, 5.239500994607786e+000, 3.944693805727305e-001}, {1.034356723979786e+003, 6.183105555221405e+001, 5.308989797804546e+000, 3.974693046888213e-001}, {1.080159595815648e+003, 6.349157319187947e+001, 5.379157557964565e+000, 4.004583724710528e-001}, {1.128005029381322e+003, 6.519674650368150e+001, 5.450010965169889e+000, 4.034364899924996e-001}, {1.177984892396054e+003, 6.694779842280083e+001, 5.521556798559543e+000, 4.064035668556144e-001}, {1.230194856555584e+003, 6.874598675922586e+001, 5.593801926683446e+000, 4.093595161310650e-001}, {1.284735013104745e+003, 7.059260392256267e+001, 5.666753309253390e+000, 4.123042542968513e-001}, {1.343063556661926e+003, 7.248693578584985e+001, 5.740418684403948e+000, 4.152377011781763e-001}, {1.405790811881685e+003, 7.442887970010709e+001, 5.814805784873986e+000, 4.181597798876019e-001}, {1.473628412214821e+003, 7.641766308616965e+001, 5.889923696754936e+000, 4.210704167656976e-001}, {1.547407873141551e+003, 7.866215579136075e+001, 5.972913138476412e+000, 4.242404029888886e-001}, {1.626876447829412e+003, 8.101582333818058e+001, 6.058368241520675e+000, 4.274553962053971e-001}, {1.710703024156860e+003, 8.348992183014360e+001, 6.146409820007483e+000, 4.307167462684176e-001}, {1.802548130038754e+003, 8.608556958507408e+001, 6.237172430960432e+000, 4.340258706136082e-001}, {1.900281151393451e+003, 8.881903955788455e+001, 6.330791655209080e+000, 4.373842594703591e-001}, {2.005807564700644e+003, 9.169671712011228e+001, 6.427420948078083e+000, 4.407934798537481e-001}, {2.119670462816548e+003, 9.473022727133676e+001, 6.527221444915404e+000, 4.442551829095661e-001}, {2.242730739132749e+003, 9.793155623939180e+001, 6.630367354240099e+000, 4.477711093076878e-001}, {2.375959842441208e+003, 1.013138933328462e+002, 6.737046497801452e+000, 4.513430963808632e-001}, {2.520456586920368e+003, 1.048917812479968e+002, 6.847461700925177e+000, 4.549730857610047e-001}, {2.677467781456748e+003, 1.086812872066808e+002, 6.961832360108455e+000, 4.586631318040276e-001}, {2.848411757693430e+003, 1.127002025269144e+002, 7.080396212881112e+000, 4.624154109045756e-001}, {3.034907040997281e+003, 1.169682716660580e+002, 7.203411344095176e+000, 4.662322318172513e-001}, {3.238806348401873e+003, 1.215074586972124e+002, 7.331158464655384e+000, 4.701160471199219e-001}, {3.462237522477920e+003, 1.263422571775514e+002, 7.463943508185539e+000, 4.740694659762597e-001}, {3.707652735358812e+003, 1.315000523822533e+002, 7.602100598730761e+000, 4.780952683808143e-001}, {3.977888247502366e+003, 1.370115455154723e+002, 7.745995453845659e+000, 4.821964211012325e-001}, {4.276236889578777e+003, 1.429112533343024e+002, 7.896029299613393e+000, 4.863760955695669e-001}, {4.606537129126736e+003, 1.492380973967760e+002, 8.052643391311207e+000, 4.906376880197965e-001}, {4.973281708728467e+003, 1.560361047391914e+002, 8.216324251440401e+000, 4.949848422233832e-001}, {5.381752513871990e+003, 1.633552406824528e+002, 8.387609764809310e+000, 4.994214752410864e-001}, {5.838187327949046e+003, 1.712524067572989e+002, 8.567096297611432e+000, 5.039518066908266e-001}, {6.350807137284905e+003, 1.797913262134226e+002, 8.755447517735441e+000, 5.085803921309421e-001}, {6.925260369284571e+003, 1.890530284782858e+002, 8.953400783448563e+000, 5.133121612816783e-001}, {7.577609824964002e+003, 1.991109013463354e+002, 9.161790857189885e+000, 5.181524619605371e-001}, {8.315968280243349e+003, 2.100768343459340e+002, 9.381540170293199e+000, 5.231071115480921e-001}, {9.158060724816223e+003, 2.220603128331959e+002, 9.613703198825842e+000, 5.281824536278080e-001}, {1.012263317307919e+004, 2.351976050029865e+002, 9.859467222735550e+000, 5.333854293394884e-001}, {1.123327851126173e+004, 2.496472646859635e+002, 1.012018401525426e+001, 5.387236561640729e-001}, {1.251933369982379e+004, 2.655964641799241e+002, 1.039739833055877e+001, 5.442055229446516e-001}, {1.401759699687882e+004, 2.832674508259391e+002, 1.069288439806393e+001, 5.498403020789378e-001}, {1.577465893041188e+004, 3.029257728808081e+002, 1.100869114048111e+001, 5.556382833229867e-001}, {1.785011289937452e+004, 3.248908324502644e+002, 1.134719868472852e+001, 5.616109344677590e-001}, {2.032103368180682e+004, 3.495495532976501e+002, 1.171118962875876e+001, 5.677710957231201e-001}, {2.328832453018373e+004, 3.773742333550699e+002, 1.210393982355082e+001, 5.741332167790789e-001}, {2.688585484152241e+004, 4.089460996964476e+002, 1.252933527138577e+001, 5.807136484549550e-001}, {3.129385590668408e+004, 4.449866873513151e+002, 1.299202446564990e+001, 5.875310049495469e-001}, {3.675989831311496e+004, 4.863985403526961e+002, 1.349762008289207e+001, 5.946066185193436e-001}, {4.362525762024865e+004, 5.343309081037533e+002, 1.405296523588706e+001, 6.019651167819491e-001}, {5.238361908669422e+004, 5.902358641369289e+002, 1.466651102006858e+001, 6.096351651125723e-001}, {6.374468368510970e+004, 6.560106923559654e+002, 1.534880715327808e+001, 6.176504358167559e-001}, {7.877675453751416e+004, 7.341156222195222e+002, 1.611324444329973e+001, 6.260508899137485e-001}, {9.912569226684302e+004, 8.277977359185787e+002, 1.697708892301073e+001, 6.348845111387733e-001}, {1.274244982488466e+005, 9.413602952665652e+002, 1.796305186238983e+001, 6.442096900359834e-001}, {1.680683685966994e+005, 1.080502797828005e+003, 1.910170745210420e+001, 6.540985866240503e-001}, {2.287813361407409e+005, 1.252634522697668e+003, 2.043539039017945e+001, 6.646420006807282e-001}, {3.240258081936964e+005, 1.466745135891141e+003, 2.202482398980625e+001, 6.759566538738161e-001}, {4.831368343637143e+005, 1.731166540009913e+003, 2.396123098126818e+001, 6.881964979202373e-001}, {7.431179345815331e+005, 2.088970442340921e+003, 2.637425972841378e+001, 7.015710980094142e-001}, {1.189850152483235e+006, 2.623880187705014e+003, 2.946733032297955e+001, 7.163772638856115e-001}, {2.026594777734833e+006, 3.525448589659171e+003, 3.356057915998095e+001, 7.330575520262048e-001}, {3.751171121950996e+006, 3.860303175516215e+003, 3.792933932492554e+001, 7.472071449747023e-001}, {7.724209728178185e+006, 3.466433188520202e+003, 4.418715889618908e+001, 7.632728136051334e-001}, {1.704546786687731e+007, 2.883198938498763e+003, 5.372439594779554e+001, 7.828757764751581e-001}, {3.866227285949533e+007, 6.044071408154451e+003, 6.745335104083873e+001, 8.059157661363587e-001}, {8.834871476634434e+007, 1.351199514790031e+004, 8.499422051220881e+001, 8.277377240352702e-001}, {2.176025209942091e+008, 1.094228648933841e+004, 1.064266208208614e+002, 8.459822722289890e-001}, {5.417031709599584e+008, -1.429287025490565e+004, 1.349265846505103e+002, 8.629254292602316e-001}, {1.260427879479906e+009, -1.145901868204451e+004, 1.733305321359170e+002, 8.814128590616269e-001}, {2.813184466308078e+009, 2.559435030649040e+005, 2.304564317758250e+002, 9.042234397623542e-001}, {5.626351338748589e+009, 4.809820508206572e+005, 2.799571429006732e+002, 9.167366942666617e-001}, {1.296968662926528e+010, 1.817347953863243e+005, 3.184401164168030e+002, 9.225727665578075e-001}, {3.830247925197880e+010, -6.819911380346881e+005, 4.005374222642562e+002, 9.326388658049110e-001}, {1.287778778901221e+011, -3.655758707469311e+006, 5.430609568313491e+002, 9.435863614001090e-001}, {4.268494326672124e+011, -3.288472239150366e+006, 7.033859556657586e+002, 9.579454741567478e-001}, {1.034173853201191e+012, 4.083865055613008e+006, 8.820734693646336e+002, 9.677749072712484e-001}, {7.360535318331335e+015, -4.520887554536115e+010, 6.939753852333379e+004, 9.741584695812871e-001}};
        constant Real[Ninterval, 4] dlcoef = {{-1.238492681682227e+010, 2.155273044551285e+006, -4.784064764172629e+002, 3.108234546713846e+000}, {-1.105429598046165e+010, 2.040848348264662e+006, -4.669490356904236e+002, 3.106950869117842e+000}, {-9.690084649689146e+009, 1.940606112928544e+006, -4.575230025626220e+002, 3.105872508783625e+000}, {-8.318064398350635e+009, 1.756735969737722e+006, -4.369950152503960e+002, 3.103417366473883e+000}, {-7.140296347581058e+009, 1.590320343200648e+006, -4.174017246617847e+002, 3.100944868392514e+000}, {-6.129570508708958e+009, 1.439711927840652e+006, -3.987007300239189e+002, 3.098454883787075e+000}, {-5.261965140039352e+009, 1.303395508058584e+006, -3.808503361523602e+002, 3.095947126307966e+000}, {-4.517289395063499e+009, 1.180016669264855e+006, -3.638116590837542e+002, 3.093421411830455e+000}, {-3.878070615519906e+009, 1.068342896840991e+006, -3.475472372378163e+002, 3.090877503680521e+000}, {-3.329405690851281e+009, 9.672641008273838e+005, -3.320217101263023e+002, 3.088315215846500e+000}, {-2.858440705800047e+009, 8.757716096217774e+005, -3.172010279329125e+002, 3.085734309672665e+000}, {-2.454126160678405e+009, 7.929540898201873e+005, -3.030530265360140e+002, 3.083134597151535e+000}, {-2.107077441640843e+009, 7.179881425904200e+005, -2.895465270173005e+002, 3.080515785796512e+000}, {-1.809136833923553e+009, 6.501257495370673e+005, -2.766520552391303e+002, 3.077877633663473e+000}, {-1.553367992062939e+009, 5.886942597143862e+005, -2.643417895971250e+002, 3.075219949428117e+000}, {-1.333788491394850e+009, 5.330816167622649e+005, -2.525887134919396e+002, 3.072542437056091e+000}, {-1.145286689441753e+009, 4.827368491822371e+005, -2.413675362576479e+002, 3.069844902846438e+000}, {-9.842004535366566e+008, 4.371863251742306e+005, -2.306539304447653e+002, 3.067127048225000e+000}, {-8.481738668343931e+008, 3.960467172138331e+005, -2.204253861258150e+002, 3.064388625095322e+000}, {-7.331751535883956e+008, 3.589111400199431e+005, -2.106602384512079e+002, 3.061629384019479e+000}, {-6.358170209106354e+008, 3.259263366450451e+005, -2.015181373167670e+002, 3.058904739456489e+000}, {-5.528785395381644e+008, 2.973604756002514e+005, -1.932383784377713e+002, 3.056307824467903e+000}, {-4.812120579273270e+008, 2.714695510765162e+005, -1.853567632253349e+002, 3.053709701944857e+000}, {-4.192283489842466e+008, 2.479880869216736e+005, -1.778517183038584e+002, 3.051110359095722e+000}, {-3.655664548207343e+008, 2.266781162415372e+005, -1.707029553047223e+002, 3.048509783002060e+000}, {-3.190676294329081e+008, 2.073267347249426e+005, -1.638913968450548e+002, 3.045907960618840e+000}, {-2.787357163663924e+008, 1.897427160373918e+005, -1.573990874235835e+002, 3.043304878774594e+000}, {-2.437229798017070e+008, 1.737549055396368e+005, -1.512091344388983e+002, 3.040700524171564e+000}, {-2.132978996593800e+008, 1.592092608304189e+005, -1.453056252302879e+002, 3.038094883385813e+000}, {-1.868371040328432e+008, 1.459677646993695e+005, -1.396735798552919e+002, 3.035487942867281e+000}, {-1.638025501528403e+008, 1.339061001971329e+005, -1.342988785546285e+002, 3.032879688939840e+000}, {-1.437323978319605e+008, 1.229125376644445e+005, -1.291682166376668e+002, 3.030270107801310e+000}, {-1.262309428194057e+008, 1.128866441350699e+005, -1.242690519881137e+002, 3.027659185523409e+000}, {-1.109546795986899e+008, 1.037377107884577e+005, -1.195895502947782e+002, 3.025046908051736e+000}, {-9.761012432391328e+007, 9.538426669141714e+004, -1.151185537187768e+002, 3.022433261205660e+000}, {-8.594246238515522e+007, 8.775266482730758e+004, -1.108455268357315e+002, 3.019818230678227e+000}, {-7.573256602113859e+007, 8.077656161927809e+004, -1.067605270066560e+002, 3.017201802035995e+000}, {-6.679050808234609e+007, 7.439599879519979e+004, -1.028541631348516e+002, 3.014583960718893e+000}, {-5.895272357387779e+007, 6.855689098463632e+004, -9.911756599444276e+001, 3.011964692039987e+000}, {-5.207642434522393e+007, 6.321021881030477e+004, -9.554235139999366e+001, 3.009343981185272e+000}, {-4.603952482299992e+007, 5.831185001665622e+004, -9.212060006254087e+001, 3.006721813213399e+000}, {-4.073465329002590e+007, 5.382164282445671e+004, -8.884481728610524e+001, 3.004098173055394e+000}, {-3.606947830186065e+007, 4.970338252471794e+004, -8.570791975358135e+001, 3.001473045514342e+000}, {-3.196365751713583e+007, 4.592422769921850e+004, -8.270320400275625e+001, 2.998846415265044e+000}, {-2.834709613390578e+007, 4.245437522415161e+004, -7.982432520669138e+001, 2.996218266853638e+000}, {-2.515904141651271e+007, 3.926683364274947e+004, -7.706527859170076e+001, 2.993588584697207e+000}, {-2.234661006197257e+007, 3.633710429345421e+004, -7.442037731654195e+001, 2.990957353083351e+000}, {-1.986350933749488e+007, 3.364291054328384e+004, -7.188423367010003e+001, 2.988324556169726e+000}, {-1.766954634469551e+007, 3.116404764577806e+004, -6.945174434254167e+001, 2.985690177983576e+000}, {-1.572958710525628e+007, 2.888213723781383e+004, -6.711807172295707e+001, 2.983054202421209e+000}, {-1.401292644023167e+007, 2.678046159635360e+004, -6.487862947013215e+001, 2.980416613247471e+000}, {-1.249270190404953e+007, 2.484380595914790e+004, -6.272906834224337e+001, 2.977777394095186e+000}, {-1.114546757050897e+007, 2.305832820575798e+004, -6.066526313144936e+001, 2.975136528464554e+000}, {-9.950639193150809e+006, 2.141140730200463e+004, -5.868329902325385e+001, 2.972493999722552e+000}, {-8.890218298855454e+006, 1.989154845489648e+004, -5.677946086303228e+001, 2.969849791102278e+000}, {-7.948405774843407e+006, 1.848826513773720e+004, -5.495022127059669e+001, 2.967203885702293e+000}, {-7.111322017747801e+006, 1.719198739978693e+004, -5.319223050409758e+001, 2.964556266485915e+000}, {-6.366802347563332e+006, 1.599398533656803e+004, -5.150230698139536e+001, 2.961906916280511e+000}, {-5.704155755187705e+006, 1.488628445368870e+004, -4.987742750713409e+001, 2.959255817776730e+000}, {-5.113936916982337e+006, 1.386158907991517e+004, -4.831471847609879e+001, 2.956602953527748e+000}, {-4.587897845661234e+006, 1.291324557621612e+004, -4.681144900015597e+001, 2.953948305948456e+000}, {-4.118721316950063e+006, 1.203515316230080e+004, -4.536502121962786e+001, 2.951291857314630e+000}, {-3.699968686669407e+006, 1.122173374498905e+004, -4.397296471188466e+001, 2.948633589762085e+000}, {-3.325990419032289e+006, 1.046788709710375e+004, -4.263292938338680e+001, 2.945973485285792e+000}, {-2.991756337877007e+006, 9.768928695956120e+003, -4.134267792650781e+001, 2.943311525738965e+000}, {-2.692851052844252e+006, 9.120574676046892e+003, -4.010008148277442e+001, 2.940647692832129e+000}, {-2.425364090781395e+006, 8.518894238993196e+003, -3.890311277023815e+001, 2.937981968132176e+000}, {-2.185839011929014e+006, 7.960282125333197e+003, -3.774984110719230e+001, 2.935314333061347e+000}, {-1.971208238001473e+006, 7.441427276955104e+003, -3.663842719615118e+001, 2.932644768896239e+000}, {-1.778765554998854e+006, 6.959293235989100e+003, -3.556711877550735e+001, 2.929973256766761e+000}, {-1.606107055594805e+006, 6.511088393081703e+003, -3.453424555743582e+001, 2.927299777655057e+000}, {-1.451097834519724e+006, 6.094246543433254e+003, -3.353821526499588e+001, 2.924624312394422e+000}, {-1.311849093123074e+006, 5.706410424466297e+003, -3.257750979381026e+001, 2.921946841668166e+000}, {-1.186680867573490e+006, 5.345410405839227e+003, -3.165068106360730e+001, 2.919267346008474e+000}, {-1.074098219451101e+006, 5.009249251722629e+003, -3.075634760904577e+001, 2.916585805795216e+000}, {-9.727750675061081e+005, 4.696089510241781e+003, -2.989319134752533e+001, 2.913902201254751e+000}, {-8.815305960940811e+005, 4.404238143974260e+003, -2.905995415563471e+001, 2.911216512458684e+000}, {-7.993100073168422e+005, 4.132133894522016e+003, -2.825543489024561e+001, 2.908528719322610e+000}, {-7.251808108065209e+005, 3.878340434733169e+003, -2.747848687199868e+001, 2.905838801604811e+000}, {-6.583036582047840e+005, 3.641529386009533e+003, -2.672801450587870e+001, 2.903146738904952e+000}, {-5.979351488970650e+005, 3.420477830640781e+003, -2.600297151102254e+001, 2.900452510662714e+000}, {-5.434106276898081e+005, 3.214056119358886e+003, -2.530235797279994e+001, 2.897756096156425e+000}, {-4.941345433408149e+005, 3.021220273636314e+003, -2.462521817165470e+001, 2.895057474501652e+000}, {-4.495775001488223e+005, 2.841007253447881e+003, -2.397063865119245e+001, 2.892356624649757e+000}, {-4.092637696142438e+005, 2.672525730358106e+003, -2.333774582158626e+001, 2.889653525386433e+000}, {-3.727690426212300e+005, 2.514952386918597e+003, -2.272570432440947e+001, 2.886948155330209e+000}, {-3.397130883384123e+005, 2.367525415691009e+003, -2.213371501440320e+001, 2.884240492930909e+000}, {-3.097553540585333e+005, 2.229539894807050e+003, -2.156101328018909e+001, 2.881530516468100e+000}, {-2.825898974603600e+005, 2.100342961405521e+003, -2.100686736953992e+001, 2.878818204049503e+000}, {-2.579442208981614e+005, 1.979331007278646e+003, -2.047057696315193e+001, 2.876103533609366e+000}, {-2.355710977468511e+005, 1.865943220530542e+003, -1.995147135640385e+001, 2.873386482906805e+000}, {-2.152510718952486e+005, 1.759661471895166e+003, -1.944890854277082e+001, 2.870667029524125e+000}, {-1.967854522379427e+005, 1.660004272608494e+003, -1.896227340604679e+001, 2.867945150865090e+000}, {-1.799963227248120e+005, 1.566525526716963e+003, -1.849097676756957e+001, 2.865220824153183e+000}, {-1.647232316787083e+005, 1.478811021808096e+003, -1.803445403318663e+001, 2.862494026429800e+000}, {-1.508223569587723e+005, 1.396476642378965e+003, -1.759216415836827e+001, 2.859764734552452e+000}, {-1.381636164814771e+005, 1.319165275775219e+003, -1.716358843294191e+001, 2.857032925192887e+000}, {-1.266302756204947e+005, 1.246545535418823e+003, -1.674822959952733e+001, 2.854298574835218e+000}, {-1.161168631137604e+005, 1.178309260666872e+003, -1.634561076107631e+001, 2.851561659773991e+000}, {-1.065283929681333e+005, 1.114170085655529e+003, -1.595527452413494e+001, 2.848822156112222e+000}, {-9.777902898360457e+004, 1.053861608696577e+003, -1.557678207604774e+001, 2.846080039759405e+000}, {-8.979154479537536e+004, 9.971361867131301e+002, -1.520971239323694e+001, 2.843335286429471e+000}, {-8.249581780149181e+004, 9.437630558793404e+002, -1.485366135803301e+001, 2.840587871638727e+000}, {-7.582881406187733e+004, 8.935276800414810e+002, -1.450824112401567e+001, 2.837837770703734e+000}, {-6.973331684484918e+004, 8.462300547482139e+002, -1.417307927722939e+001, 2.835084958739184e+000}, {-6.415772301025310e+004, 8.016839926065475e+002, -1.384781823535527e+001, 2.832329410655682e+000}, {-5.905519971849246e+004, 7.597158559545809e+002, -1.353211453405775e+001, 2.829571101157552e+000}, {-5.438340177539967e+004, 7.201638399218540e+002, -1.322563825532957e+001, 2.826810004740545e+000}, {-5.010398989213503e+004, 6.828770586818224e+002, -1.292807241099001e+001, 2.824046095689559e+000}, {-4.618216375504041e+004, 6.477146970338505e+002, -1.263911237047971e+001, 2.821279348076278e+000}, {-4.258637278865216e+004, 6.145453580284924e+002, -1.235846534594279e+001, 2.818509735756785e+000}, {-3.928799334523628e+004, 5.832463833869973e+002, -1.208584987510642e+001, 2.815737232369135e+000}, {-3.626107008446607e+004, 5.537032511254856e+002, -1.182099533490190e+001, 2.812961811330881e+000}, {-3.348198605783714e+004, 5.258089299140838e+002, -1.156364146280048e+001, 2.810183445836552e+000}, {-3.092930799948825e+004, 4.994634350699143e+002, -1.131353793886240e+001, 2.807402108855102e+000}, {-2.858356718774496e+004, 4.745733110816813e+002, -1.107044395139554e+001, 2.804617773127295e+000}, {-2.642699292972316e+004, 4.510511035622531e+002, -1.083412778305086e+001, 2.801830411163054e+000}, {-2.444349338561342e+004, 4.288151035012024e+002, -1.060436646834351e+001, 2.799039995238765e+000}, {-2.261838452021139e+004, 4.077888022404431e+002, -1.038094537815669e+001, 2.796246497394526e+000}, {-2.093827593537573e+004, 3.879005747406519e+002, -1.016365789510950e+001, 2.793449889431356e+000}, {-1.939098963577922e+004, 3.690833932437830e+002, -9.952305091079492e+000, 2.790650142908357e+000}, {-1.796543440841462e+004, 3.512744857770907e+002, -9.746695394200888e+000, 2.787847229139806e+000}, {-1.665145478671165e+004, 3.344149940989051e+002, -9.546644276491429e+000, 2.785041119192228e+000}, {-1.543982996581212e+004, 3.184498209919203e+002, -9.351973998366489e+000, 2.782231783881397e+000}, {-1.432210669598613e+004, 3.033272642915332e+002, -9.162513291636291e+000, 2.779419193769276e+000}, {-1.329059840415068e+004, 2.889988870529622e+002, -8.978097130361970e+000, 2.776603319160937e+000}, {-1.233825052722389e+004, 2.754192080986946e+002, -8.798566446195549e+000, 2.773784130101386e+000}, {-1.145863709207734e+004, 2.625455884031810e+002, -8.623767918906500e+000, 2.770961596372379e+000}, {-1.064587614821500e+004, 2.503379954642477e+002, -8.453553723390042e+000, 2.768135687489127e+000}, {-9.894583021419403e+003, 2.387588397329175e+002, -8.287781316214854e+000, 2.765306372696999e+000}, {-9.199834391098546e+003, 2.277728276390388e+002, -8.126313229068519e+000, 2.762473620968123e+000}, {-8.557121363365739e+003, 2.173468013975267e+002, -7.969016862161412e+000, 2.759637400997958e+000}, {-7.962313251536238e+003, 2.074496022604224e+002, -7.815764293296074e+000, 2.756797681201794e+000}, {-7.411623241071581e+003, 1.980519411509770e+002, -7.666432093866367e+000, 2.753954429711179e+000}, {-6.901589290252236e+003, 1.891262933062386e+002, -7.520901156344275e+000, 2.751107614370310e+000}, {-6.429018668504029e+003, 1.806467434145949e+002, -7.379056511362362e+000, 2.748257202732345e+000}, {-5.991001909044038e+003, 1.725889472108416e+002, -7.240787192269098e+000, 2.745403162055639e+000}, {-5.584856439473709e+003, 1.649299712560422e+002, -7.105986051583309e+000, 2.742545459299940e+000}, {-5.208118092244479e+003, 1.576482310857538e+002, -6.974549631033893e+000, 2.739684061122499e+000}, {-4.858536339451712e+003, 1.507234256770707e+002, -6.846378022696069e+000, 2.736818933874121e+000}, {-4.534025576845524e+003, 1.441364021997527e+002, -6.721374710980710e+000, 2.733950043595136e+000}, {-4.232680321240590e+003, 1.378691440330893e+002, -6.599446472802768e+000, 2.731077356011313e+000}, {-3.952748909345426e+003, 1.319046712312123e+002, -6.480503232709889e+000, 2.728200836529692e+000}, {-3.692609478366457e+003, 1.262269600752981e+002, -6.364457941874171e+000, 2.725320450234344e+000}, {-3.450780099779326e+003, 1.208209269116236e+002, -6.251226483812403e+000, 2.722436161882055e+000}, {-3.225890664374303e+003, 1.156723332618167e+002, -6.140727542679162e+000, 2.719547935897942e+000}, {-3.016682125991474e+003, 1.107677541103619e+002, -6.032882511285902e+000, 2.716655736370977e+000}, {-2.821990248596265e+003, 1.060945140428862e+002, -5.927615383023561e+000, 2.713759527049444e+000}, {-2.640746879338374e+003, 1.016406614326807e+002, -5.824852664925329e+000, 2.710859271336301e+000}, {-2.471964523626565e+003, 9.739490626074014e+001, -5.724523273535363e+000, 2.707954932284481e+000}, {-2.314730837568900e+003, 9.334658567014867e+001, -5.626558451896237e+000, 2.705046472592083e+000}, {-2.168208526525156e+003, 8.948563835359066e+001, -5.530891688550211e+000, 2.702133854597495e+000}, {-2.031619097394213e+003, 8.580254532064717e+001, -5.437458623108411e+000, 2.699217040274426e+000}, {-1.904248100892504e+003, 8.228832275596824e+001, -5.346196983339631e+000, 2.696295991226843e+000}, {-1.785432953219038e+003, 7.893447181267172e+001, -5.257046496811301e+000, 2.693370668683814e+000}, {-1.674562836953798e+003, 7.573296108764812e+001, -5.169948826535308e+000, 2.690441033494273e+000}, {-1.571072232234745e+003, 7.267619282823553e+001, -5.084847496875256e+000, 2.687507046121676e+000}, {-1.474438413222446e+003, 6.975698100090553e+001, -5.001687829570725e+000, 2.684568666638568e+000}, {-1.384177903371559e+003, 6.696852671757033e+001, -4.920416878738944e+000, 2.681625854721034e+000}, {-1.299843535198407e+003, 6.430439600004813e+001, -4.840983369252954e+000, 2.678678569643088e+000}, {-1.221020634106669e+003, 6.175849648989043e+001, -4.763337636576003e+000, 2.675726770270898e+000}, {-1.147326026570235e+003, 5.932506215554464e+001, -4.687431572745481e+000, 2.672770415056979e+000}, {-1.078404076039887e+003, 5.699863018706918e+001, -4.613218568350649e+000, 2.669809462034204e+000}, {-1.013926021509312e+003, 5.477402792669884e+001, -4.540653463065229e+000, 2.666843868809767e+000}, {-9.535859458173311e+002, 5.264635101631857e+001, -4.469692491629795e+000, 2.663873592558998e+000}, {-8.971017903285012e+002, 5.061095614294467e+001, -4.400293240651299e+000, 2.660898590019072e+000}, {-8.442100723223150e+002, 4.866343605671213e+001, -4.332414594549864e+000, 2.657918817482606e+000}, {-7.946678515348818e+002, 4.679961714885733e+001, -4.266016699189970e+000, 2.654934230791131e+000}, {-7.482488676648289e+002, 4.501553841960071e+001, -4.201060911438582e+000, 2.651944785328449e+000}, {-7.047431728617579e+002, 4.330744315658079e+001, -4.137509761479857e+000, 2.648950436013851e+000}, {-6.639558744821053e+002, 4.167176730429927e+001, -4.075326912123187e+000, 2.645951137295226e+000}, {-6.257057100641964e+002, 4.010512796775672e+001, -4.014477119746307e+000, 2.642946843142023e+000}, {-5.898246346182218e+002, 3.860431527293839e+001, -3.954926198543035e+000, 2.639937507038096e+000}, {-5.561557244035386e+002, 3.716627939672841e+001, -3.896640982412575e+000, 2.636923081974399e+000}, {-5.245536700749274e+002, 3.578812653072018e+001, -3.839589294597946e+000, 2.633903520441549e+000}, {-4.948831814045453e+002, 3.446710725683307e+001, -3.783739911111038e+000, 2.630878774422248e+000}, {-4.670181331795835e+002, 3.320060869244811e+001, -3.729062529754870e+000, 2.627848795383549e+000}, {-4.408411680577592e+002, 3.198614837687174e+001, -3.675527740772351e+000, 2.624813534268998e+000}, {-4.162434320780575e+002, 3.082136815496841e+001, -3.623106997289613e+000, 2.621772941490593e+000}, {-3.931227261553422e+002, 2.970402334199991e+001, -3.571772583810418e+000, 2.618726966920612e+000}, {-3.713848005366851e+002, 2.863198333302012e+001, -3.521497594697411e+000, 2.615675559883269e+000}, {-3.509411299166285e+002, 2.760321878319193e+001, -3.472255900926836e+000, 2.612618669146214e+000}, {-3.317092974307500e+002, 2.661580012155591e+001, -3.424022129678430e+000, 2.609556242911867e+000}, {-3.136127904025766e+002, 2.566789269818100e+001, -3.376771639026537e+000, 2.606488228808575e+000}, {-2.965798073836402e+002, 2.475774875702146e+001, -3.330480491611001e+000, 2.603414573881601e+000}, {-2.805435913300593e+002, 2.388370594280104e+001, -3.285125435370047e+000, 2.600335224583943e+000}, {-2.654419571434289e+002, 2.304418190389038e+001, -3.240683879734343e+000, 2.597250126766951e+000}, {-2.512164673588557e+002, 2.223766834090091e+001, -3.197133873379682e+000, 2.594159225670773e+000}, {-2.378128746065777e+002, 2.146273034130477e+001, -3.154454087136033e+000, 2.591062465914610e+000}, {-2.251804620611885e+002, 2.071800061360175e+001, -3.112623791404769e+000, 2.587959791486772e+000}, {-2.132716404393391e+002, 2.000217565128110e+001, -3.071622837510624e+000, 2.584851145734526e+000}, {-2.020422322936485e+002, 1.931401462635231e+001, -3.031431641412206e+000, 2.581736471353761e+000}, {-1.914506566618751e+002, 1.865233349451230e+001, -2.992031162913385e+000, 2.578615710378421e+000}, {-1.814582337456608e+002, 1.801600450360726e+001, -2.953402891597071e+000, 2.575488804169740e+000}, {-1.720286483594544e+002, 1.740395163665135e+001, -2.915528828069439e+000, 2.572355693405244e+000}, {-1.631280051948557e+002, 1.681514914240465e+001, -2.878391469604711e+000, 2.569216318067544e+000}, {-1.547245214939870e+002, 1.624861828468630e+001, -2.841973793789774e+000, 2.566070617432879e+000}, {-1.467885075935258e+002, 1.570342560672776e+001, -2.806259244455594e+000, 2.562918530059438e+000}, {-1.392920759633892e+002, 1.517867995891996e+001, -2.771231716523010e+000, 2.559759993775438e+000}, {-1.322092377618008e+002, 1.467353144997319e+001, -2.736875543385945e+000, 2.556594945666950e+000}, {-1.255154520422071e+002, 1.418716782019382e+001, -2.703175481732913e+000, 2.553423322065465e+000}, {-1.191879144732180e+002, 1.371881463196889e+001, -2.670116701239096e+000, 2.550245058535220e+000}, {-1.132051254329855e+002, 1.326773160149195e+001, -2.637684769449511e+000, 2.547060089860237e+000}, {-1.075469689833722e+002, 1.283321196746067e+001, -2.605865641505036e+000, 2.543868350031094e+000}, {-1.021945855468274e+002, 1.241458057792133e+001, -2.574645647887416e+000, 2.540669772231418e+000}, {-9.713024121213921e+001, 1.201119215362063e+001, -2.544011483057522e+000, 2.537464288824097e+000}, {-9.233735633732306e+001, 1.162243039375014e+001, -2.513950195251489e+000, 2.534251831337175e+000}, {-8.780030483853737e+001, 1.124770586654619e+001, -2.484449174984496e+000, 2.531032330449471e+000}, {-8.350447678244254e+001, 1.088645546210887e+001, -2.455496146003123e+000, 2.527805715975874e+000}, {-7.943610746442556e+001, 1.053814048023886e+001, -2.427079154442550e+000, 2.524571916852306e+000}, {-7.558228169765363e+001, 1.020224588737254e+001, -2.399186560121682e+000, 2.521330861120392e+000}, {-7.193088597970828e+001, 9.878279171064129e+000, -2.371807027150768e+000, 2.518082475911769e+000}, {-6.847048516239306e+001, 9.565768889163360e+000, -2.344929514629048e+000, 2.514826687432048e+000}, {-6.519039610308946e+001, 9.264264386688476e+000, -2.318543269061701e+000, 2.511563420944446e+000}, {-6.208052306641781e+001, 8.973334052099887e+000, -2.292637814810399e+000, 2.508292600753025e+000}, {-5.913139743958612e+001, 8.692564973814662e+000, -2.267202947101239e+000, 2.505014150185589e+000}, {-5.633408892514787e+001, 8.421561745547976e+000, -2.242228723666145e+000, 2.501727991576170e+000}, {-5.368023596268498e+001, 8.159946044547962e+000, -2.217705457774197e+000, 2.498434046247153e+000}, {-5.116192490033033e+001, 7.907355281104470e+000, -2.193623710100769e+000, 2.495132234490955e+000}, {-4.877172908422913e+001, 7.663442373803837e+000, -2.169974282641104e+000, 2.491822475551334e+000}, {-4.650265985770452e+001, 7.427874856171875e+000, -2.146748211408460e+000, 2.488504687604243e+000}, {-4.434811779628213e+001, 7.200334084917268e+000, -2.123936759740916e+000, 2.485178787738240e+000}, {-4.230190317998800e+001, 6.980514860596383e+000, -2.101531412363795e+000, 2.481844691934497e+000}, {-4.035816750319500e+001, 6.768124621635819e+000, -2.079523868792742e+000, 2.478502315046275e+000}, {-3.851139744312152e+001, 6.562882936055023e+000, -2.057906037559458e+000, 2.475151570777977e+000}, {-3.675639931468597e+001, 6.364520986088379e+000, -2.036670030451280e+000, 2.471792371663693e+000}, {-3.508827447219454e+001, 6.172781005619186e+000, -2.015808156820983e+000, 2.468424629045237e+000}, {-3.350240391913771e+001, 5.987415808641527e+000, -1.995312918218178e+000, 2.465048253049681e+000}, {-3.199442787158013e+001, 5.808188295000401e+000, -1.975177003091340e+000, 2.461663152566335e+000}, {-3.056024706213614e+001, 5.634871106705811e+000, -1.955393281837274e+000, 2.458269235223206e+000}, {-2.919596443353868e+001, 5.467245898489124e+000, -1.935954801423811e+000, 2.454866407362862e+000}, {-2.789793559611147e+001, 5.305103425707614e+000, -1.916854781506048e+000, 2.451454574017747e+000}, {-2.666269215914849e+001, 5.148242631442525e+000, -1.898086608731207e+000, 2.448033638884868e+000}, {-2.548697421306633e+001, 4.996470669372890e+000, -1.879643833139071e+000, 2.444603504299887e+000}, {-2.436770123450288e+001, 4.849602373431799e+000, -1.861520163342853e+000, 2.441164071210578e+000}, {-2.330195977537568e+001, 4.707459922705984e+000, -1.843709462409692e+000, 2.437715239149616e+000}, {-2.228699227843552e+001, 4.569872530071029e+000, -1.826205743845993e+000, 2.434256906206701e+000}, {-2.132020684497282e+001, 4.436676277522927e+000, -1.809003167843803e+000, 2.430788968999998e+000}, {-2.039914018532746e+001, 4.307713579632853e+000, -1.792096036957894e+000, 2.427311322646843e+000}, {-1.952146936347992e+001, 4.182833098600734e+000, -1.775478792886058e+000, 2.423823860733728e+000}, {-1.868499316966931e+001, 4.061889380846185e+000, -1.759146012624790e+000, 2.420326475285513e+000}, {-1.788763992768876e+001, 3.944742723944577e+000, -1.743092405205007e+000, 2.416819056733864e+000}, {-1.712743344254924e+001, 3.831258705580187e+000, -1.727312807882854e+000, 2.413301493884874e+000}, {-1.640252210380573e+001, 3.721308274346385e+000, -1.711802183566103e+000, 2.409773673885858e+000}, {-1.571113655820867e+001, 3.614767188724737e+000, -1.696555616858599e+000, 2.406235482191265e+000}, {-1.505161347050935e+001, 3.511516100089723e+000, -1.681568311719631e+000, 2.402686802527732e+000}, {-1.442237501501945e+001, 3.411440183535207e+000, -1.666835588003566e+000, 2.399127516858194e+000}, {-1.382192281792874e+001, 3.314428957926218e+000, -1.652352878705905e+000, 2.395557505345059e+000}, {-1.324884440234419e+001, 3.220376204778547e+000, -1.638115727371166e+000, 2.391976646312384e+000}, {-1.270179534791704e+001, 3.129179660595857e+000, -1.624119785066449e+000, 2.388384816207066e+000}, {-1.217950533160762e+001, 3.040740960771689e+000, -1.610360808057413e+000, 2.384781889558960e+000}, {-1.168077185108037e+001, 2.954965440505362e+000, -1.596834655106244e+000, 2.381167738939920e+000}, {-1.120445287471011e+001, 2.871761949263363e+000, -1.583537284982829e+000, 2.377542234921738e+000}, {-1.074945938906329e+001, 2.791042690386217e+000, -1.570464754127387e+000, 2.373905246032895e+000}, {-1.031477532863617e+001, 2.712723290948242e+000, -1.557613214714723e+000, 2.370256638714139e+000}, {-9.899418059077929e+000, 2.636722293125727e+000, -1.544978911524978e+000, 2.366596277272797e+000}, {-9.502469882015735e+000, 2.562961407683305e+000, -1.532558180831734e+000, 2.362924023835828e+000}, {-9.123051186031617e+000, 2.491365115660259e+000, -1.520347447494355e+000, 2.359239738301530e+000}, {-8.760330847138221e+000, 2.421860710047537e+000, -1.508343223428605e+000, 2.355543278289859e+000}, {-8.413522164671106e+000, 2.354378147536363e+000, -1.496542105459269e+000, 2.351834499091346e+000}, {-8.081872986226925e+000, 2.288849868571785e+000, -1.484940773296089e+000, 2.348113253614503e+000}, {-7.764673182380162e+000, 2.225210809410545e+000, -1.473535988024564e+000, 2.344379392331724e+000}, {-7.461247253240972e+000, 2.163398232030816e+000, -1.462324590067311e+000, 2.340632763223554e+000}, {-7.170954402974354e+000, 2.103351655486919e+000, -1.451303497568699e+000, 2.336873211721342e+000}, {-6.893183429082948e+000, 2.045012734099102e+000, -1.440469704675801e+000, 2.333100580648147e+000}, {-6.627356535854197e+000, 1.988325235265094e+000, -1.429820280102758e+000, 2.329314710157896e+000}, {-6.372924033280025e+000, 1.933234904863000e+000, -1.419352365401565e+000, 2.325515437672646e+000}, {-6.129360718270444e+000, 1.879689376009046e+000, -1.409063173503607e+000, 2.321702597817965e+000}, {-5.896172583475862e+000, 1.827638183228592e+000, -1.398949987494580e+000, 2.317876022356293e+000}, {-5.672883417268298e+000, 1.777032546809523e+000, -1.389010158827584e+000, 2.314035540118214e+000}, {-5.459048060392687e+000, 1.727825480131581e+000, -1.379241106510718e+000, 2.310180976931593e+000}, {-5.254234210794342e+000, 1.679971515124929e+000, -1.369640315213699e+000, 2.306312155548433e+000}, {-5.058040409876274e+000, 1.633426877234746e+000, -1.360205334751793e+000, 2.302428895569415e+000}, {-4.870075449851535e+000, 1.588149172977625e+000, -1.350933778115679e+000, 2.298531013365968e+000}, {-4.689974661286671e+000, 1.544097561802145e+000, -1.341823321100738e+000, 2.294618321999826e+000}, {-4.517385486985614e+000, 1.501232515510214e+000, -1.332871700581220e+000, 2.290690631139916e+000}, {-4.351976351638122e+000, 1.459515900408742e+000, -1.324076713954947e+000, 2.286747746976495e+000}, {-4.193429409227905e+000, 1.418910831861334e+000, -1.315436217803243e+000, 2.282789472132387e+000}, {-4.041441822526155e+000, 1.379381663626085e+000, -1.306948127133261e+000, 2.278815605571236e+000}, {-3.895727955783459e+000, 1.340893969545224e+000, -1.298610414512591e+000, 2.274825942502597e+000}, {-3.756011643772165e+000, 1.303414407478659e+000, -1.290421108956039e+000, 2.270820274283760e+000}, {-3.622034405547364e+000, 1.266910796845365e+000, -1.282378295516774e+000, 2.266798388318138e+000}, {-3.493546180189254e+000, 1.231351952460242e+000, -1.274480114091720e+000, 2.262760067950088e+000}, {-3.370310607726903e+000, 1.196707735821732e+000, -1.266724759068044e+000, 2.258705092355959e+000}, {-3.252103437476587e+000, 1.162948988902917e+000, -1.259110478484196e+000, 2.254633236431245e+000}, {-3.138708072538556e+000, 1.130047448209561e+000, -1.251635573300534e+000, 2.250544270673615e+000}, {-3.029921490748561e+000, 1.097975801268001e+000, -1.244298397138752e+000, 2.246437961061666e+000}, {-2.925547647484275e+000, 1.066707556584218e+000, -1.237097355384933e+000, 2.242314068929141e+000}, {-2.825400740172198e+000, 1.036217072961828e+000, -1.230030904983982e+000, 2.238172350834462e+000}, {-2.729304069234395e+000, 1.006479509244643e+000, -1.223097553890867e+000, 2.234012558425291e+000}, {-2.637088175181507e+000, 9.774707739265413e-001, -1.216295860641661e+000, 2.229834438297897e+000}, {-2.548592797119484e+000, 9.491675292708579e-001, -1.209624434141700e+000, 2.225637731851086e+000}, {-2.463663423753487e+000, 9.215471164859220e-001, -1.203081933219051e+000, 2.221422175134388e+000}, {-2.382154682227609e+000, 8.945875849149885e-001, -1.196667066602152e+000, 2.217187498690240e+000}, {-2.303926482472558e+000, 8.682676066340345e-001, -1.190378592501879e+000, 2.212933427389838e+000}, {-2.228846495422617e+000, 8.425664957592878e-001, -1.184215318682858e+000, 2.208659680262342e+000}, {-2.156787119809440e+000, 8.174641394548649e-001, -1.178176102225793e+000, 2.204365970317081e+000}, {-2.087628043470757e+000, 7.929410200343601e-001, -1.172259849701592e+000, 2.200052004358389e+000}, {-2.021254023899635e+000, 7.689781539219797e-001, -1.166465517041273e+000, 2.195717482792678e+000}, {-1.957555062242275e+000, 7.455570794959029e-001, -1.160792109727399e+000, 2.191362099427352e+000}, {-1.896426533402995e+000, 7.226598392117595e-001, -1.155238682973756e+000, 2.186985541261074e+000}, {-1.837768826064559e+000, 7.002689535529851e-001, -1.149804341937003e+000, 2.182587488264949e+000}, {-1.781486665007183e+000, 6.783673926406706e-001, -1.144488242010365e+000, 2.178167613154132e+000}, {-1.727489645225360e+000, 6.569385677311423e-001, -1.139289589251027e+000, 2.173725581149271e+000}, {-1.675691487477748e+000, 6.359662992671422e-001, -1.134207640777898e+000, 2.169261049727270e+000}, {-1.626009927752474e+000, 6.154347993672957e-001, -1.129241705325752e+000, 2.164773668360731e+000}, {-1.578367042581602e+000, 5.953286591428831e-001, -1.124391143878772e+000, 2.160263078245440e+000}, {-1.532688000054525e+000, 5.756328107329074e-001, -1.119655370298758e+000, 2.155728912015201e+000}, {-1.488902369669526e+000, 5.563325343394894e-001, -1.115033852224612e+000, 2.151170793443280e+000}, {-1.446942756513707e+000, 5.374134146917063e-001, -1.110526111833913e+000, 2.146588337129652e+000}, {-1.406744919308124e+000, 5.188613303492935e-001, -1.106131726888593e+000, 2.141981148173249e+000}, {-1.368248520348087e+000, 5.006624488336312e-001, -1.101850331860153e+000, 2.137348821828268e+000}, {-1.331395630230789e+000, 4.828031818949505e-001, -1.097681619010324e+000, 2.132690943143583e+000}, {-1.296131540592206e+000, 4.652701873924490e-001, -1.093625339819990e+000, 2.128007086584252e+000}, {-1.262404709408866e+000, 4.480503487048833e-001, -1.089681306389480e+000, 2.123296815633988e+000}, {-1.230165851283160e+000, 4.311307419976398e-001, -1.085849392958450e+000, 2.118559682377429e+000}, {-1.199368710297917e+000, 4.144986359366915e-001, -1.082129537707425e+000, 2.113795227060934e+000}, {-1.169969608591567e+000, 3.981414637443046e-001, -1.078521744549609e+000, 2.109002977630553e+000}, {-1.141927349834720e+000, 3.820468046501531e-001, -1.075026085156083e+000, 2.104182449245702e+000}, {-1.115203082457214e+000, 3.662023649698341e-001, -1.071642701147781e+000, 2.099333143767018e+000}, {-1.089760490754199e+000, 3.505959636314658e-001, -1.068371806469075e+000, 2.094454549216662e+000}, {-1.065565676036384e+000, 3.352155103263428e-001, -1.065213689919650e+000, 2.089546139209323e+000}, {-1.042586729117127e+000, 3.200489791680136e-001, -1.062168717889230e+000, 2.084607372351953e+000}, {-1.020794201818640e+000, 3.050843996188034e-001, -1.059237337384420e+000, 2.079637691610172e+000}, {-1.000160851423267e+000, 2.903098297929833e-001, -1.056420079188220e+000, 2.074636523639092e+000}, {-9.806614973337507e-001, 2.757133333948358e-001, -1.053717561313666e+000, 2.069603278076170e+000}, {-9.622732693844609e-001, 2.612829629773606e-001, -1.051130492745526e+000, 2.064537346793446e+000}, {-9.449750965802405e-001, 2.470067276370440e-001, -1.048659677405065e+000, 2.059438103106439e+000}, {-9.287484495069965e-001, 2.328725849863756e-001, -1.046306018534049e+000, 2.054304900936606e+000}, {-9.135767353123816e-001, 2.188684014719849e-001, -1.044070523246075e+000, 2.049137073924143e+000}, {-8.994453596884484e-001, 2.049819299554718e-001, -1.041954307568271e+000, 2.043933934487611e+000}, {-8.863420305972949e-001, 1.912007886144718e-001, -1.039958601844730e+000, 2.038694772826542e+000}, {-8.742566684377657e-001, 1.775124284430896e-001, -1.038084756497729e+000, 2.033418855862917e+000}, {-8.631812697459577e-001, 1.639040998219162e-001, -1.036334248261912e+000, 2.028105426117050e+000}, {-8.531101234321912e-001, 1.503628248396395e-001, -1.034708686935291e+000, 2.022753700513037e+000}, {-8.440398362033877e-001, 1.368753624716768e-001, -1.033209822617653e+000, 2.017362869108479e+000}, {-8.359695776931154e-001, 1.234281750538496e-001, -1.031839553532223e+000, 2.011932093742809e+000}, {-8.289005178126103e-001, 1.100073763809098e-001, -1.030599934401732e+000, 2.006460506597966e+000}, {-8.228370327534438e-001, 9.659871547150999e-002, -1.029493185682019e+000, 2.000947208664684e+000}, {-8.177855931109604e-001, 8.318750209275284e-002, -1.028521703215806e+000, 1.995391268106987e+000}, {-8.137559409386451e-001, 6.975858534653368e-002, -1.027688069030039e+000, 1.989791718516891e+000}, {-8.107605455362672e-001, 5.629628320384371e-002, -1.026995062703656e+000, 1.984147557050458e+000}, {-8.088151684966699e-001, 4.278433685925803e-002, -1.026445673888504e+000, 1.978457742435633e+000}, {-8.079389176695084e-001, 2.920584547725917e-002, -1.026043115762180e+000, 1.972721192841307e+000}, {-8.081545055075236e-001, 1.554320066626404e-002, -1.025790839663862e+000, 1.966936783596047e+000}, {-8.094885690941598e-001, 1.778015394473135e-003, -1.025692550981584e+000, 1.961103344743805e+000}, {-8.119862698864362e-001, -1.210870698156008e-002, -1.025752227366026e+000, 1.955219658422625e+000}, {-8.156795691276541e-001, -2.613710375287741e-002, -1.025974136034128e+000, 1.949284456050977e+000}, {-8.205859499113112e-001, -4.032871488155882e-002, -1.026362854927959e+000, 1.943296415304725e+000}, {-8.267080955867543e-001, -5.494775286888999e-002, -1.026933948323617e+000, 1.937154374999355e+000}, {-8.340034681281544e-001, -6.956792894448306e-002, -1.027675693302118e+000, 1.931045251398180e+000}, {-8.424606360805104e-001, -8.417588056073176e-002, -1.028583863802415e+000, 1.924981823573036e+000}, {-8.520744170015872e-001, -9.878905079285071e-002, -1.029655444551397e+000, 1.918963711773316e+000}, {-8.628424957952819e-001, -1.134242872327940e-001, -1.030887586075559e+000, 1.912990539037579e+000}, {-8.747657918779217e-001, -1.280978812135765e-001, -1.032277595480934e+000, 1.907061931185783e+000}, {-8.878479382283583e-001, -1.428256278541051e-001, -1.033822927640486e+000, 1.901177516818667e+000}, {-9.020955906247847e-001, -1.576228613739366e-001, -1.035521177062222e+000, 1.895336927323218e+000}, {-9.175179556117891e-001, -1.725045083448280e-001, -1.037370070125957e+000, 1.889539796883510e+000}, {-9.341272433353944e-001, -1.874851158489946e-001, -1.039367457933408e+000, 1.883785762496073e+000}, {-9.519380335091618e-001, -2.025789039960535e-001, -1.041511309452197e+000, 1.878074463988979e+000}, {-9.709677271759698e-001, -2.177997890193544e-001, -1.043799705248612e+000, 1.872405544044008e+000}, {-9.912364175083539e-001, -2.331614219124641e-001, -1.046230831486630e+000, 1.866778648221036e+000}, {-1.012766648729506e+000, -2.486772255078792e-001, -1.048802974335914e+000, 1.861193424984079e+000}, {-1.035583653239251e+000, -2.643604184910795e-001, -1.051514514791885e+000, 1.855649525728494e+000}, {-1.059715403422719e+000, -2.802240450910093e-001, -1.054363923766542e+000, 1.850146604808510e+000}, {-1.085192428404830e+000, -2.962810076523094e-001, -1.057349757490633e+000, 1.844684319564752e+000}, {-1.112047840284971e+000, -3.125440918568987e-001, -1.060470653253161e+000, 1.839262330351305e+000}, {-1.140317779786924e+000, -3.290259844501270e-001, -1.063725325428120e+000, 1.833880300561731e+000}, {-1.170040772767701e+000, -3.457393129150245e-001, -1.067112561661264e+000, 1.828537896653743e+000}, {-1.201258334460249e+000, -3.626966549827271e-001, -1.070631219467131e+000, 1.823234788172321e+000}, {-1.234015008677897e+000, -3.799105645052093e-001, -1.074280222931547e+000, 1.817970647770632e+000}, {-1.268357889035586e+000, -3.973936035490504e-001, -1.078058559640830e+000, 1.812745151228863e+000}, {-1.304337381224103e+000, -4.151583481991378e-001, -1.081965277933464e+000, 1.807557977470460e+000}, {-1.342006818795918e+000, -4.332174201542663e-001, -1.085999484193767e+000, 1.802408808575871e+000}, {-1.381422811192989e+000, -4.515835009140273e-001, -1.090160340433320e+000, 1.797297329793238e+000}, {-1.422645114412084e+000, -4.702693556220330e-001, -1.094447061989566e+000, 1.792223229546457e+000}, {-1.465736793308509e+000, -4.892878500123289e-001, -1.098858915416110e+000, 1.787186199439965e+000}, {-1.510764665910317e+000, -5.086519632583179e-001, -1.103395216529062e+000, 1.782185934260665e+000}, {-1.557798713989053e+000, -5.283748197187160e-001, -1.108055328517688e+000, 1.777222131976678e+000}, {-1.606913024249304e+000, -5.484696901100445e-001, -1.112838660328233e+000, 1.772294493733011e+000}, {-1.658185052376375e+000, -5.689500256907437e-001, -1.117744665009237e+000, 1.767402723844188e+000}, {-1.711696835419274e+000, -5.898294555775258e-001, -1.122772838338994e+000, 1.762546529783867e+000}, {-1.767533848479072e+000, -6.111218280850936e-001, -1.127922717367707e+000, 1.757725622171410e+000}, {-1.825786241452084e+000, -6.328412048781662e-001, -1.133193879281465e+000, 1.752939714755770e+000}, {-1.886548675522292e+000, -6.550018871713900e-001, -1.138585940177355e+000, 1.748188524396311e+000}, {-1.949919791236933e+000, -6.776184419015450e-001, -1.144098553976703e+000, 1.743471771041181e+000}, {-2.016003811123379e+000, -7.007056923572437e-001, -1.149731411542488e+000, 1.738789177703054e+000}, {-2.084909407036162e+000, -7.242787611639050e-001, -1.155484239633304e+000, 1.734140470432365e+000}, {-2.156750372091127e+000, -7.483530737647965e-001, -1.161356800148957e+000, 1.729525378288342e+000}, {-2.231627926685809e+000, -7.729445823064577e-001, -1.167348888802051e+000, 1.724943633307832e+000}, {-2.309782982776231e+000, -7.980678185970754e-001, -1.173460339702013e+000, 1.720394970472067e+000}, {-2.391033663426555e+000, -8.237439135656604e-001, -1.179691000681487e+000, 1.715879127671712e+000}, {-2.475869227142783e+000, -8.499832002901067e-001, -1.186040793336751e+000, 1.711395845671781e+000}, {-2.563702595314027e+000, -8.768125482108226e-001, -1.192509607048069e+000, 1.706944868064386e+000}, {-2.653854883122921e+000, -9.042518387765622e-001, -1.199097403535986e+000, 1.702525941248357e+000}, {-2.766079959594875e+000, -9.320924259943205e-001, -1.205804687265605e+000, 1.698138814375825e+000}, {-2.836837888537308e+000, -9.612314004857536e-001, -1.212628993373840e+000, 1.693783239359507e+000}, {-2.962833314784326e+000, -9.901728646594288e-001, -1.219575772839467e+000, 1.689458970942077e+000}, {-3.065797818595779e+000, -1.020307105997146e+000, -1.226639028725659e+000, 1.685165765209986e+000}, {-3.179146555811669e+000, -1.051016019187351e+000, -1.233822305642099e+000, 1.680903383115731e+000}, {-3.297284348618891e+000, -1.082440812070858e+000, -1.241124966094870e+000, 1.676671586493826e+000}, {-3.420414306708996e+000, -1.114602326672745e+000, -1.248547175601712e+000, 1.672470140103473e+000}, {-3.548747934055633e+000, -1.147521940179529e+000, -1.256089127934521e+000, 1.668298811149195e+000}, {-3.682505857052307e+000, -1.181221572474609e+000, -1.263751044854284e+000, 1.664157369235286e+000}, {-3.821915807343334e+000, -1.215723723768695e+000, -1.271533175757266e+000, 1.660045586320429e+000}, {-3.967214733237526e+000, -1.251051449579780e+000, -1.279435797479266e+000, 1.655963236672333e+000}, {-4.118645118574411e+000, -1.287228408806640e+000, -1.287459213783149e+000, 1.651910096822906e+000}, {-4.280211341706425e+000, -1.324244351504146e+000, -1.295603825538186e+000, 1.647885945523818e+000}, {-4.453514772407470e+000, -1.362099245447999e+000, -1.303870050532300e+000, 1.643890563702867e+000}, {-4.640395688977453e+000, -1.400781827377156e+000, -1.312258462550433e+000, 1.639923734421353e+000}, {-4.842972282220500e+000, -1.444303609727521e+000, -1.321571915960135e+000, 1.635617913591072e+000}, {-5.077818607817462e+000, -1.489650226742146e+000, -1.331210828298044e+000, 1.631266166750613e+000}, {-5.269519269425579e+000, -1.537901012173168e+000, -1.341188853488693e+000, 1.626867109373032e+000}, {-5.551431069675780e+000, -1.587320333609772e+000, -1.351529036102523e+000, 1.622419288010019e+000}, {-5.808923042237106e+000, -1.639958440934728e+000, -1.362242057327564e+000, 1.617921174006821e+000}, {-6.095719050661056e+000, -1.695055063945981e+000, -1.373352317843512e+000, 1.613371162616998e+000}, {-6.404369551367121e+000, -1.752995378033357e+000, -1.384879806897964e+000, 1.608767561543370e+000}, {-6.737074974434417e+000, -1.813996281601904e+000, -1.396847279282773e+000, 1.604108587787460e+000}, {-7.096314427293298e+000, -1.878296278283565e+000, -1.409279272349471e+000, 1.599392359891712e+000}, {-7.484885584665084e+000, -1.946158190784704e+000, -1.422202288280279e+000, 1.594616890192578e+000}, {-7.905955622871398e+000, -2.017872214308587e+000, -1.435644999776433e+000, 1.589780076274508e+000}, {-8.363119801992593e+000, -2.093759464249109e+000, -1.449638482309929e+000, 1.584879691522251e+000}, {-8.860470601969707e+000, -2.174176076790771e+000, -1.464216477264085e+000, 1.579913374653328e+000}, {-9.402681541299199e+000, -2.259517940676243e+000, -1.479415690927240e+000, 1.574878618093620e+000}, {-9.995105596234213e+000, -2.350226224176583e+000, -1.495276135046780e+000, 1.569772755037022e+000}, {-1.064389482985112e+001, -2.446793798326975e+000, -1.511841516075586e+000, 1.564592945003904e+000}, {-1.135614240262710e+001, -2.549772781901460e+000, -1.529159681274163e+000, 1.559336157681449e+000}, {-1.214005414414541e+001, -2.659783392395340e+000, -1.547283131791800e+000, 1.553999154791512e+000}, {-1.300515543255768e+001, -2.777524377540540e+000, -1.566269614684388e+000, 1.548578469686142e+000}, {-1.396253928239969e+001, -2.903785372223432e+000, -1.586182808281310e+000, 1.543070384316234e+000}, {-1.502516407943721e+001, -3.039461566365826e+000, -1.607093118455835e+000, 1.537470903152320e+000}, {-1.620820834955008e+001, -3.185571181481919e+000, -1.629078606872898e+000, 1.531775723555805e+000}, {-1.756861317903349e+001, -3.342859695130933e+000, -1.652227062158571e+000, 1.525980202000237e+000}, {-1.896992968076159e+001, -3.514753551909730e+000, -1.676629844756208e+000, 1.520079315421806e+000}, {-2.070940498674979e+001, -3.698607679814522e+000, -1.702409666201476e+000, 1.514067616830629e+000}, {-2.258006445102759e+001, -3.900361852777759e+000, -1.729670196587357e+000, 1.507939179287543e+000}, {-2.472640864504724e+001, -4.119997968335903e+000, -1.758560156062154e+000, 1.501687551187454e+000}, {-2.717555760930571e+001, -4.360330688942441e+000, -1.789232432463443e+000, 1.495305668348961e+000}, {-2.998453612050399e+001, -4.624174865325230e+000, -1.821864001930955e+000, 1.488785783609070e+000}, {-3.322400657923019e+001, -4.914848588452856e+000, -1.856657070400842e+000, 1.482119368890723e+000}, {-3.698227739969207e+001, -5.236285870597177e+000, -1.893843826546765e+000, 1.475297002040697e+000}, {-4.137073781469906e+001, -5.593179752915376e+000, -1.933692326201029e+000, 1.468308231964111e+000}, {-4.653129624345377e+001, -5.991165877519198e+000, -1.976513837575755e+000, 1.461141416796625e+000}, {-5.264671942271787e+001, -6.437059666295744e+000, -2.022672094596102e+000, 1.453783528293291e+000}, {-5.995516440181747e+001, -6.939166169402911e+000, -2.072595066915272e+000, 1.446219913491736e+000}, {-6.877091752349767e+001, -7.507688498893050e+000, -2.126790093032183e+000, 1.438434001794347e+000}, {-7.951434352644553e+001, -8.155272910741649e+000, -2.185863560319914e+000, 1.430406941556337e+000}, {-9.282260741855799e+001, -8.896990368881466e+000, -2.250548709297550e+000, 1.422117144536678e+000}, {-1.092705800731509e+002, -9.755978248450887e+000, -2.321727653966861e+000, 1.413539708353296e+000}, {-1.303440861742615e+002, -1.075180658002727e+001, -2.400520525131451e+000, 1.404645675124892e+000}, {-1.573817172536041e+002, -1.192284129304266e+001, -2.488271989475859e+000, 1.395401054361456e+000}, {-1.929469781047582e+002, -1.330915824062231e+001, -2.586733173534589e+000, 1.385765574728271e+000}, {-2.407515409275657e+002, -1.496747918474029e+001, -2.698134813608466e+000, 1.375690927154873e+000}, {-3.067296511299643e+002, -1.697224306116501e+001, -2.825409191622562e+000, 1.365118418615304e+000}, {-4.007152627101729e+002, -1.942210540851691e+001, -2.972493248817588e+000, 1.353975640846269e+000}, {-5.399185840310040e+002, -2.244503642183699e+001, -3.144822779164391e+000, 1.342171664024869e+000}, {-7.560716744000814e+002, -2.620100919290446e+001, -3.350155100053136e+000, 1.329589871746869e+000}, {-1.113449606842879e+003, -3.084021110135518e+001, -3.600109952519693e+000, 1.316076873033058e+000}, {-1.690504862425243e+003, -3.710261655467448e+001, -3.911155756406296e+000, 1.301424533021810e+000}, {-2.670061268217115e+003, -4.640046513281821e+001, -4.309071967361942e+000, 1.285339230723633e+000}, {-4.481748438342077e+003, -6.187707883736660e+001, -4.834386672670042e+000, 1.267385221467624e+000}, {-8.164522417664276e+003, -6.838329256029518e+001, -5.390905548063032e+000, 1.252292339125201e+000}, {-1.652878983123607e+004, -6.410911523659486e+001, -6.181751840615706e+000, 1.235307228164419e+000}, {-3.589814816980765e+004, -5.850303396906464e+001, -7.379682885213981e+000, 1.214800598771303e+000}, {-8.037263246538142e+004, -1.135710262603827e+002, -9.097826769016773e+000, 1.191007390096093e+000}, {-1.821922090737477e+005, -2.354776316590442e+002, -1.127469694751144e+001, 1.168783843757871e+000}, {-4.484262360427870e+005, -1.929406030066214e+002, -1.390923719646678e+001, 1.150438269719548e+000}, {-1.121046496283366e+006, 2.244014433727998e+002, -1.739509749280907e+001, 1.133590960463892e+000}, {-2.637332052450561e+006, 2.537931543190984e+002, -2.209765636576909e+001, 1.115407413029603e+000}, {-5.907648512403362e+006, -4.012624018357776e+003, -2.901240599782692e+001, 1.093221801784601e+000}, {-1.195665812647725e+007, -7.565697091202631e+003, -3.507390000142389e+001, 1.081141798451352e+000}, {-2.784190228559279e+007, -2.474660221277177e+003, -3.985563387169906e+001, 1.075521362860637e+000}, {-8.260002532389586e+007, 1.207599307914400e+004, -5.010461096457156e+001, 1.065839017041407e+000}, {-2.783054150157432e+008, 6.191392637195736e+004, -6.812180721814856e+001, 1.055309382343470e+000}, {-8.984303648430897e+008, 4.972036367617193e+004, -8.797612069577760e+001, 1.041458768913245e+000}, {-1.938721107426129e+009, -9.947883920780817e+004, -1.102739497257638e+002, 1.031926967645291e+000}, {-1.596711708673011e+013, 7.576033677883548e+008, -8.980126598064539e+003, 1.025706312840203e+000}};
        constant Real[Ninterval, 4] dvcoef = {{4.918869918230695e+005, -1.567555888877598e+002, 5.437208630256651e-001, 5.503960828374048e-005}, {4.550378123449924e+005, -1.523735197978206e+002, 5.428795951631563e-001, 5.651517998086538e-005}, {4.154435854155241e+005, -1.485249049854512e+002, 5.421736401433307e-001, 5.778081864475579e-005}, {3.741482472743288e+005, -1.409517385019123e+002, 5.405774106280010e-001, 6.075322570198208e-005}, {3.367178855870678e+005, -1.337561394903371e+002, 5.389802274088923e-001, 6.387796804156671e-005}, {3.032331897449884e+005, -1.269325231900327e+002, 5.373822526407004e-001, 6.716271877677984e-005}, {2.729783236073526e+005, -1.204513648774790e+002, 5.357833746302725e-001, 7.061574208320135e-005}, {2.457737771951380e+005, -1.143004549600878e+002, 5.341836871213043e-001, 7.424557939372635e-005}, {2.212418903451594e+005, -1.084606313277702e+002, 5.325831835730194e-001, 7.806127539077440e-005}, {1.991932319373264e+005, -1.029188757389579e+002, 5.309819330350911e-001, 8.207225038864295e-005}, {1.793568834640014e+005, -9.765862295132199e+001, 5.293799351427024e-001, 8.628847726670124e-005}, {1.614383537622045e+005, -9.266330277412550e+001, 5.277772334623126e-001, 9.072034242183298e-005}, {1.453621714686960e+005, -8.792412794039890e+001, 5.261738672668539e-001, 9.537893269130447e-005}, {1.308585174350236e+005, -8.342408029607272e+001, 5.245698186895845e-001, 1.002758063548368e-004}, {1.178099872852326e+005, -7.915315147317665e+001, 5.229651805648193e-001, 1.054230003569786e-004}, {1.060542653282609e+005, -7.509881556093026e+001, 5.213599463406871e-001, 1.108333568850558e-004}, {9.549308124088842e+004, -7.125147962403636e+001, 5.197541987519517e-001, 1.165201514339942e-004}, {8.600354913972432e+004, -6.759972330879310e+001, 5.181479357221414e-001, 1.224975427286338e-004}, {7.761649543341427e+004, -6.414066678887514e+001, 5.165412858073596e-001, 1.287802891593490e-004}, {7.020537948517055e+004, -6.086565981704204e+001, 5.149343233861899e-001, 1.353838849828343e-004}, {6.362526510536840e+004, -5.781701794798269e+001, 5.133590847200070e-001, 1.421827481986982e-004}, {5.777602598727927e+004, -5.506082706782908e+001, 5.118681842941606e-001, 1.489285080947021e-004}, {5.249423222977891e+004, -5.245136198830269e+001, 5.103869134700294e-001, 1.559450852216556e-004}, {4.772771403525848e+004, -4.998041590145706e+001, 5.089152331130584e-001, 1.632411903229066e-004}, {4.341758916708818e+004, -4.763947665576765e+001, 5.074530908455733e-001, 1.708257343021365e-004}, {3.952482258441854e+004, -4.542149360328381e+001, 5.060004500938848e-001, 1.787078304444174e-004}, {3.599853857184100e+004, -4.331871999300164e+001, 5.045572556408491e-001, 1.868967966091156e-004}, {3.281073717790914e+004, -4.132521822148750e+001, 5.031234772300364e-001, 1.954021573940050e-004}, {2.991928864379426e+004, -3.943405746218934e+001, 5.016990585808311e-001, 2.042336462698480e-004}, {2.730136743929938e+004, -3.764000095251405e+001, 5.002839707362081e-001, 2.134012076848843e-004}, {2.492659063635537e+004, -3.593720253748594e+001, 4.988781632750814e-001, 2.229149991386049e-004}, {2.276955125736159e+004, -3.432046079500933e+001, 4.974815975109458e-001, 2.327853932242566e-004}, {2.081432755389784e+004, -3.278542777406340e+001, 4.960942426492074e-001, 2.430229796396210e-004}, {1.903499917129547e+004, -3.132692253363413e+001, 4.947160448050079e-001, 2.536385671655345e-004}, {1.741916761544805e+004, -2.994121669633407e+001, 4.933469804327021e-001, 2.646431856117704e-004}, {1.594856799673893e+004, -2.862400374594620e+001, 4.919870030103821e-001, 2.760480877298892e-004}, {1.461071366209510e+004, -2.737169558264001e+001, 4.906360810069271e-001, 2.878647510927006e-004}, {1.339168327690314e+004, -2.618061339571608e+001, 4.892941746173027e-001, 3.001048799400484e-004}, {1.228253049325166e+004, -2.504768084200302e+001, 4.879612544780885e-001, 3.127804069906575e-004}, {1.126892157829276e+004, -2.396930264453291e+001, 4.866372742930647e-001, 3.259034952198203e-004}, {1.034662065878251e+004, -2.294316241182801e+001, 4.853222187890683e-001, 3.394865396027453e-004}, {9.503700936201338e+003, -2.196597276324068e+001, 4.840160361000067e-001, 3.535421688234263e-004}, {8.734018189448401e+003, -2.103534947204093e+001, 4.827187038678896e-001, 3.680832469489622e-004}, {8.031366011875803e+003, -2.014888458104759e+001, 4.814301896622395e-001, 3.831228750692300e-004}, {7.388578355618841e+003, -1.930410181604127e+001, 4.801504565658148e-001, 3.986743929019504e-004}, {6.800574870580942e+003, -1.849890723067505e+001, 4.788794784376936e-001, 4.147513803631108e-004}, {6.262968091501257e+003, -1.773131649952538e+001, 4.776172260185485e-001, 4.313676591028885e-004}, {5.770389109765950e+003, -1.699924277022264e+001, 4.763636637585473e-001, 4.485372940071114e-004}, {5.319190052222050e+003, -1.630094266002941e+001, 4.751187671980786e-001, 4.662745946644320e-004}, {4.905748845902255e+003, -1.563469347603085e+001, 4.738825069166632e-001, 4.845941167994040e-004}, {4.526685334231427e+003, -1.499884943133974e+001, 4.726548535498104e-001, 5.035106636716925e-004}, {4.178772802276819e+003, -1.439183882285007e+001, 4.714357786189442e-001, 5.230392874416343e-004}, {3.859530682062243e+003, -1.381226047506382e+001, 4.702252575271995e-001, 5.431952905025171e-004}, {3.566271424600439e+003, -1.325870013942056e+001, 4.690232613675992e-001, 5.639942267798111e-004}, {3.296870650297050e+003, -1.272989003696022e+001, 4.678297654512661e-001, 5.854519029978606e-004}, {3.049276098691007e+003, -1.222459719155169e+001, 4.666447430986029e-001, 6.075843799142949e-004}, {2.821484408536768e+003, -1.174163338947488e+001, 4.654681678733132e-001, 6.304079735227145e-004}, {2.611896527283416e+003, -1.127992855916830e+001, 4.643000166997424e-001, 6.539392562240592e-004}, {2.419079453141731e+003, -1.083846133076433e+001, 4.631402651077743e-001, 6.781950579671866e-004}, {2.241307272262556e+003, -1.041617660727023e+001, 4.619888855086836e-001, 7.031924673592141e-004}, {2.077658736699405e+003, -1.001223689974768e+001, 4.608458599404298e-001, 7.289488327461664e-004}, {1.926777017074004e+003, -9.625702150971902e+000, 4.597111603812837e-001, 7.554817632645456e-004}, {1.787519449312663e+003, -9.255727615109393e+000, 4.585847646064639e-001, 7.828091298644552e-004}, {1.659179395402279e+003, -8.901591924409631e+000, 4.574666539228491e-001, 8.109490663049823e-004}, {1.540633616703145e+003, -8.562472211245941e+000, 4.563568005087656e-001, 8.399199701224020e-004}, {1.431161858142325e+003, -8.237690859137757e+000, 4.552551859319501e-001, 8.697405035720722e-004}, {1.330023377555299e+003, -7.926573334195409e+000, 4.541617883382710e-001, 9.004295945446778e-004}, {1.236573482343776e+003, -7.628487285308340e+000, 4.530765866476337e-001, 9.320064374575882e-004}, {1.150117985277077e+003, -7.342803184259996e+000, 4.519995585201517e-001, 9.644904941221798e-004}, {1.070158684587523e+003, -7.068970548333955e+000, 4.509306856964692e-001, 9.979014945878772e-004}, {9.961748371914198e+002, -6.806438520833332e+000, 4.498699469983624e-001, 1.032259437963822e-003}, {9.276477356080152e+002, -6.554674171966223e+000, 4.488173216777321e-001, 1.067584593218963e-003}, {8.641833883415839e+002, -6.313201947850644e+000, 4.477727918978972e-001, 1.103897499961509e-003}, {8.053877216782303e+002, -6.081553232617854e+000, 4.467363378201690e-001, 1.141218969198647e-003}, {7.508679837063625e+002, -5.859273363957273e+000, 4.457079396593727e-001, 1.179570084077442e-003}, {7.003121955106866e+002, -5.645950416545300e+000, 4.446875799042271e-001, 1.218972200607816e-003}, {6.534229224786275e+002, -5.441184593317924e+000, 4.436752399301383e-001, 1.259446948368720e-003}, {6.098701132482534e+002, -5.244576566367364e+000, 4.426709003094503e-001, 1.301016231198290e-003}, {5.694693580848956e+002, -5.055795182260655e+000, 4.416745463024567e-001, 1.343702227869144e-003}, {5.319187630216652e+002, -4.874464882088055e+000, 4.406861567650555e-001, 1.387527392749791e-003}, {4.970284952496925e+002, -4.700273550639881e+000, 4.397057168100576e-001, 1.432514456453111e-003}, {4.646073398256017e+002, -4.532911236187518e+000, 4.387332091303748e-001, 1.478686426473104e-003}, {4.344436320778534e+002, -4.372067882144371e+000, 4.377686155875123e-001, 1.526066587810795e-003}, {4.063942942181745e+002, -4.217474280320218e+000, 4.368119214276980e-001, 1.574678503590567e-003}, {3.802841770186233e+002, -4.068850532622352e+000, 4.358631089330863e-001, 1.624546015667824e-003}, {3.559769108797973e+002, -3.925944939917800e+000, 4.349221629490394e-001, 1.675693245229227e-003}, {3.333402909463835e+002, -3.788512906615899e+000, 4.339890675369079e-001, 1.728144593386517e-003}, {3.122536180170024e+002, -3.656321586479176e+000, 4.330638070558099e-001, 1.781924741765163e-003}, {2.925902763184365e+002, -3.529142319538135e+000, 4.321463655932412e-001, 1.837058653088838e-003}, {2.742740781970119e+002, -3.406778654781579e+000, 4.312367299211879e-001, 1.893571571760979e-003}, {2.571759037143193e+002, -3.289008496189263e+000, 4.303348822713388e-001, 1.951489024444565e-003}, {2.412316234199518e+002, -3.175655844670625e+000, 4.294408108599679e-001, 2.010836820641156e-003}, {2.263479765419118e+002, -3.066528653226592e+000, 4.285544996055142e-001, 2.071641053270657e-003}, {2.124540881441975e+002, -2.961454863028191e+000, 4.276759349468488e-001, 2.133928099252665e-003}, {1.994738967239204e+002, -2.860263446005821e+000, 4.268051024338398e-001, 2.197724620090749e-003}, {1.873507685079785e+002, -2.762800207342723e+000, 4.259419892085835e-001, 2.263057562460898e-003}, {1.760176881347635e+002, -2.668907362024899e+000, 4.250865807774918e-001, 2.329954158805295e-003}, {1.654238211245136e+002, -2.578442894138812e+000, 4.242388645871290e-001, 2.398441927932571e-003}, {1.555155973132484e+002, -2.491265813149157e+000, 4.233988271501935e-001, 2.468548675625884e-003}, {1.462467369395943e+002, -2.407244188793173e+000, 4.225664558320871e-001, 2.540302495259932e-003}, {1.375706942965182e+002, -2.326249402371831e+000, 4.217417377784478e-001, 2.613731768428245e-003}, {1.294517522416703e+002, -2.248163891647594e+000, 4.209246611933187e-001, 2.688865165581934e-003}, {1.218462678465571e+002, -2.172866771201884e+000, 4.201152129124728e-001, 2.765731646681044e-003}, {1.147234040530867e+002, -2.100250995085103e+000, 4.193133818831384e-001, 2.844360461860063e-003}, {1.080479787286075e+002, -2.030208123787794e+000, 4.185191557975788e-001, 2.924781152108462e-003}, {1.017920655477493e+002, -1.962638710731280e+000, 4.177325235641014e-001, 3.007023549967833e-003}, {9.592579946784062e+001, -1.897443919649623e+000, 4.169534734756494e-001, 3.091117780246804e-003}, {9.042335241081501e+001, -1.834531404942512e+000, 4.161819947150213e-001, 3.177094260754840e-003}, {8.526160053959171e+001, -1.773813276822706e+000, 4.154180766062474e-001, 3.264983703056518e-003}, {8.041750410261692e+001, -1.715203736662323e+000, 4.146617082903334e-001, 3.354817113247324e-003}, {7.586988596025167e+001, -1.658621120070368e+000, 4.139128794048740e-001, 3.446625792752341e-003}, {7.159914621790271e+001, -1.603987470082015e+000, 4.131715799027952e-001, 3.540441339149129e-003}, {6.758805543380534e+001, -1.551228969555801e+000, 4.124378000649125e-001, 3.636295647016206e-003}, {6.381901401260075e+001, -1.500273202161896e+000, 4.117115299747857e-001, 3.734220908808124e-003}, {6.027654566266498e+001, -1.451051972887889e+000, 4.109927603593492e-001, 3.834249615758861e-003}, {5.694686162949886e+001, -1.403500300021269e+000, 4.102814821535462e-001, 3.936414558814610e-003}, {5.381464616126993e+001, -1.357553381609505e+000, 4.095776860201926e-001, 4.040748829597288e-003}, {5.086862121268142e+001, -1.313152746254192e+000, 4.088813639116348e-001, 4.147285821400251e-003}, {4.809696746170313e+001, -1.270240281248384e+000, 4.081925071806428e-001, 4.256059230217466e-003}, {4.548795219446311e+001, -1.228759645993116e+000, 4.075111074687371e-001, 4.367103055807410e-003}, {4.303153877084944e+001, -1.188658139770851e+000, 4.068371571482173e-001, 4.480451602793230e-003}, {4.071884002550738e+001, -1.149885475769847e+000, 4.061706487359031e-001, 4.596139481800510e-003}, {3.853982359631466e+001, -1.112391283466330e+000, 4.055115744704507e-001, 4.714201610633783e-003}, {3.648706442220689e+001, -1.076129903100120e+000, 4.048599277728404e-001, 4.834673215493562e-003}, {3.455203104072965e+001, -1.041055261831986e+000, 4.042157015223190e-001, 4.957589832234991e-003}, {3.272824992696282e+001, -1.007125200219632e+000, 4.035788895850664e-001, 5.082987307669529e-003}, {3.100819917421405e+001, -9.742971021566538e-001, 4.029494853695961e-001, 5.210901800911449e-003}, {2.938585708234636e+001, -9.425316926626286e-001, 4.023274832668355e-001, 5.341369784769924e-003}, {2.785532702736859e+001, -9.117906925683219e-001, 4.017128776362615e-001, 5.474428047188925e-003}, {2.641089462970517e+001, -8.820370796524276e-001, 4.011056631033509e-001, 5.610113692735653e-003}, {2.504744509854464e+001, -8.532356843531500e-001, 4.005058347752553e-001, 5.748464144139499e-003}, {2.376012690680433e+001, -8.253525682988094e-001, 3.999133879843608e-001, 5.889517143882834e-003}, {2.254436515950055e+001, -7.983550679575092e-001, 3.993283183805698e-001, 6.033310755844880e-003}, {2.139577493846826e+001, -7.722116978256376e-001, 3.987506219417533e-001, 6.179883367000695e-003}, {2.031075898178054e+001, -7.468929148919709e-001, 3.981802951854477e-001, 6.329273689176286e-003}, {1.928498435038579e+001, -7.223687585378505e-001, 3.976173344075094e-001, 6.481520760861736e-003}, {1.831546512863807e+001, -6.986123544713665e-001, 3.970617370876304e-001, 6.636663949083660e-003}, {1.739874302112455e+001, -6.755964878582553e-001, 3.965135003282964e-001, 6.794742951338802e-003}, {1.653149368600032e+001, -6.532950634486524e-001, 3.959726218194969e-001, 6.955797797590126e-003}, {1.571136413855040e+001, -6.316842473860278e-001, 3.954391000465521e-001, 7.119868852327289e-003}, {1.493511547753208e+001, -6.107391370754174e-001, 3.949129329852181e-001, 7.286996816692762e-003}, {1.420039845144105e+001, -5.904373123215700e-001, 3.943941198649326e-001, 7.457222730675804e-003}, {1.350506994592831e+001, -5.707570615830940e-001, 3.938826600000119e-001, 7.630587975375203e-003}, {1.284647108422737e+001, -5.516762826357500e-001, 3.933785526503101e-001, 7.807134275333396e-003}, {1.222277796478104e+001, -5.331751751866478e-001, 3.928817982791715e-001, 7.986903700942806e-003}, {1.163191695951530e+001, -5.152338073353088e-001, 3.923923971864010e-001, 8.169938670926894e-003}, {1.107213284474227e+001, -4.978333946436397e-001, 3.919103503368385e-001, 8.356281954897082e-003}, {1.054149664309521e+001, -4.809552921213819e-001, 3.914356588874424e-001, 8.545976675988149e-003}, {1.003852662470317e+001, -4.645822992591858e-001, 3.909683248378932e-001, 8.739066313572592e-003}, {9.561664872804187e+000, -4.486973760480717e-001, 3.905083502824061e-001, 8.935594706057733e-003}, {9.109335246091527e+000, -4.332839483801887e-001, 3.900557377878584e-001, 9.135606053765538e-003}, {8.680382947929710e+000, -4.183267593479249e-001, 3.896104907230568e-001, 9.339144921898225e-003}, {8.273324247996497e+000, -4.038101468153956e-001, 3.891726123742362e-001, 9.546256243591643e-003}, {7.887079182136844e+000, -3.897198414232183e-001, 3.887421070574992e-001, 9.756985323057689e-003}, {7.520443323253075e+000, -3.760415771209294e-001, 3.883189792052398e-001, 9.971377838818433e-003}, {7.172391400254022e+000, -3.627618922961474e-001, 3.879032339530123e-001, 1.018947984703397e-002}, {6.841893907962348e+000, -3.498676232857845e-001, 3.874948768109323e-001, 1.041133778492567e-002}, {6.527997017874895e+000, -3.373461375524688e-001, 3.870939138736296e-001, 1.063699847429761e-002}, {6.229808364298923e+000, -3.251852702502093e-001, 3.867003517682037e-001, 1.086650912515772e-002}, {5.946507856507541e+000, -3.133733224079474e-001, 3.863141976591653e-001, 1.109991733944163e-002}, {5.677262450077949e+000, -3.018988533524630e-001, 3.859354591756860e-001, 1.133727111484078e-002}, {5.421352794634765e+000, -2.907510173390247e-001, 3.855641446431972e-001, 1.157861884873768e-002}, {5.178044952198035e+000, -2.799191865535771e-001, 3.852002628309458e-001, 1.182400934225040e-002}, {4.946713848640733e+000, -2.693932865271758e-001, 3.848438232049179e-001, 1.207349180438865e-002}, {4.726667248951112e+000, -2.591633075488187e-001, 3.844948356365955e-001, 1.232711585632465e-002}, {4.517394406394665e+000, -2.492199984536905e-001, 3.841533108804939e-001, 1.258493153578013e-002}, {4.318248173877903e+000, -2.395539223666578e-001, 3.838192599352582e-001, 1.284698930153326e-002}, {4.128755543462441e+000, -2.301564276641476e-001, 3.834926948075274e-001, 1.311334003804745e-002}, {3.948400040450002e+000, -2.210188987207532e-001, 3.831736278767949e-001, 1.338403506022480e-002}, {3.776706557031288e+000, -2.121330737099716e-001, 3.828620722535667e-001, 1.365912611828752e-002}, {3.613235615589715e+000, -2.034909988612184e-001, 3.825580417231296e-001, 1.393866540278966e-002}, {3.457552934484986e+000, -1.950849326143231e-001, 3.822615507012779e-001, 1.422270554976199e-002}, {3.309289747397522e+000, -1.869075184953642e-001, 3.819726143711710e-001, 1.451129964599359e-002}, {3.168027635565692e+000, -1.789514180717389e-001, 3.816912484468373e-001, 1.480450123445292e-002}, {3.033432541611965e+000, -1.712097771328688e-001, 3.814174695766434e-001, 1.510236431985102e-002}, {2.905177939665958e+000, -1.636758970404103e-001, 3.811512950075380e-001, 1.540494337435146e-002}, {2.782931144489786e+000, -1.563432183214555e-001, 3.808927426753809e-001, 1.571229334342843e-002}, {2.666380912718784e+000, -1.492054387832750e-001, 3.806418313441766e-001, 1.602446965187827e-002}, {2.555276695079573e+000, -1.422566004739850e-001, 3.803985806210800e-001, 1.634152820998644e-002}, {2.449295076606051e+000, -1.354906337033831e-001, 3.801630106447514e-001, 1.666352541985449e-002}, {2.348233464609053e+000, -1.289020699204486e-001, 3.799351427205061e-001, 1.699051818189040e-002}, {2.251825859416809e+000, -1.224852907459614e-001, 3.797149986002554e-001, 1.732256390146563e-002}, {2.159830546635063e+000, -1.162349453169496e-001, 3.795026010329486e-001, 1.765972049574360e-002}, {2.072058189589333e+000, -1.101459982440797e-001, 3.792979737057716e-001, 1.800204640068242e-002}, {1.988279437621665e+000, -1.042133641825461e-001, 3.791011409474683e-001, 1.834960057821797e-002}, {1.908304088873131e+000, -9.843225925896297e-002, 3.789121281419708e-001, 1.870244252362887e-002}, {1.831966481347027e+000, -9.279808329654883e-002, 3.787309615388108e-001, 1.906063227308990e-002}, {1.759063885369890e+000, -8.730619512825491e-002, 3.785576681372098e-001, 1.942423041141704e-002}, {1.689442954866666e+000, -8.195229176988321e-002, 3.783922761004630e-001, 1.979329808000917e-002}, {1.622952654628118e+000, -7.673215449994221e-002, 3.782348144266670e-001, 2.016789698499184e-002}, {1.559425467803564e+000, -7.164160304756834e-002, 3.780853130129168e-001, 2.054808940556589e-002}, {1.498739703281514e+000, -6.667675646191087e-002, 3.779438029211383e-001, 2.093393820256837e-002}, {1.440744669602029e+000, -6.183368436111730e-002, 3.778103160133016e-001, 2.132550682724936e-002}, {1.385323321761285e+000, -5.710870924118614e-002, 3.776848853473129e-001, 2.172285933027002e-002}, {1.332347619367673e+000, -5.249817996692605e-002, 3.775675449108211e-001, 2.212606037092868e-002}, {1.281706587252000e+000, -4.799861330288496e-002, 3.774583298354774e-001, 2.253517522661828e-002}, {1.233286775513597e+000, -4.360659269821209e-002, 3.773572762902370e-001, 2.295026980252321e-002}, {1.186989610982328e+000, -3.931885012278981e-002, 3.772644216056396e-001, 2.337141064156014e-002}, {1.142709310723802e+000, -3.513215772395206e-002, 3.771798041763876e-001, 2.379866493456963e-002}, {1.100364054184835e+000, -3.104347409546139e-002, 3.771034636540305e-001, 2.423210053076392e-002}, {1.059849057888973e+000, -2.704971323541650e-002, 3.770354407144567e-001, 2.467178594843880e-002}, {1.021092249072365e+000, -2.314803353433318e-002, 3.769757774433468e-001, 2.511779038595530e-002}, {9.840075377705944e-001, -1.933557117879185e-002, 3.769245169864941e-001, 2.557018373299856e-002}, {9.485193352610145e-001, -1.560958934131050e-002, 3.768817038258078e-001, 2.602903658212011e-002}, {9.145571430630910e-001, -1.196743397082525e-002, 3.768473837166201e-001, 2.649442024057316e-002}, {8.820469590864203e-001, -8.406493879188376e-003, 3.768216036884748e-001, 2.696640674244555e-002}, {8.509285119314395e-001, -4.924290603556751e-003, 3.768044121832929e-001, 2.744506886110065e-002}, {8.211340609415847e-001, -1.518356287428216e-003, 3.767958589199679e-001, 2.793048012193355e-002}, {7.926086375394551e-001, 1.813649114228547e-003, 3.767959950973827e-001, 2.842271481544971e-002}, {7.652929944655803e-001, 5.074044032923294e-003, 3.768048732667741e-001, 2.892184801067763e-002}, {7.391329197825297e-001, 8.265062590104030e-003, 3.768225474785089e-001, 2.942795556892074e-002}, {7.140800335116605e-001, 1.138886128687014e-002, 3.768490732805331e-001, 2.994111415786120e-002}, {6.900802188495476e-001, 1.444758203811515e-002, 3.768845076851303e-001, 3.046140126602262e-002}, {6.670923614724178e-001, 1.744324178592566e-002, 3.769289093561197e-001, 3.098889521760341e-002}, {6.450691071878270e-001, 2.037785914287436e-002, 3.769823384460632e-001, 3.152367518768998e-002}, {6.239696575093529e-001, 2.325336642648719e-002, 3.770448567964210e-001, 3.206582121785987e-002}, {6.037510611099373e-001, 2.607166756790367e-002, 3.771165278758674e-001, 3.261541423218719e-002}, {5.843794053727025e-001, 2.883457191774331e-002, 3.771974169112035e-001, 3.317253605366046e-002}, {5.658147755261427e-001, 3.154389410629092e-002, 3.772875907776753e-001, 3.373726942102517e-002}, {5.480231161544660e-001, 3.420136776566449e-002, 3.773871182015341e-001, 3.430969800606262e-002}, {5.309724570385872e-001, 3.680868184153350e-002, 3.774960697125371e-001, 3.488990643131756e-002}, {5.146295233740406e-001, 3.936750010466255e-002, 3.776145176754647e-001, 3.547798028828863e-002}, {4.989652245345753e-001, 4.187942343633730e-002, 3.777425363985507e-001, 3.607400615609310e-002}, {4.839508951570389e-001, 4.434602120425499e-002, 3.778802021164517e-001, 3.667807162062187e-002}, {4.695586251393293e-001, 4.676882656463798e-002, 3.780275930601468e-001, 3.729026529419736e-002}, {4.557625505429792e-001, 4.914932842369436e-002, 3.781847895177411e-001, 3.791067683575024e-002}, {4.425376937773825e-001, 5.148898190841291e-002, 3.783518738637359e-001, 3.853939697152988e-002}, {4.298602277084476e-001, 5.378920663485787e-002, 3.785289306177614e-001, 3.917651751636464e-002}, {4.177069453640965e-001, 5.605139045206042e-002, 3.787160464931026e-001, 3.982213139548867e-002}, {4.060584771999545e-001, 5.827687113517555e-002, 3.789133104735820e-001, 4.047633266695239e-002}, {3.948906268608474e-001, 6.046699738585702e-002, 3.791208137811170e-001, 4.113921654463403e-002}, {3.841870232133331e-001, 6.262303268347103e-002, 3.793386501060483e-001, 4.181087942187154e-002}, {3.739265762012089e-001, 6.474625902500740e-002, 3.795669154584302e-001, 4.249141889573269e-002}, {3.640921561536769e-001, 6.683790154597202e-002, 3.798057084001543e-001, 4.318093379194452e-002}, {3.546670821834087e-001, 6.889916478367825e-002, 3.800551300119356e-001, 4.387952419050235e-002}, {3.456348352625734e-001, 7.093123341959720e-002, 3.803152839747856e-001, 4.458729145197930e-002}, {3.369785561122604e-001, 7.293527279624670e-002, 3.805862766488284e-001, 4.530433824455851e-002}, {3.286851767658728e-001, 7.491239979095070e-002, 3.808682171819050e-001, 4.603076857181217e-002}, {3.207394347235971e-001, 7.686373551656341e-002, 3.811612174764944e-001, 4.676668780125029e-002}, {3.131280975545644e-001, 7.879036541765304e-002, 3.814653923662434e-001, 4.751220269366369e-002}, {3.058374247107702e-001, 8.069336289010372e-002, 3.817808596349681e-001, 4.826742143328867e-002}, {2.988566601058779e-001, 8.257376243678892e-002, 3.821077401446526e-001, 4.903245365881852e-002}, {2.921719464565475e-001, 8.443261163535294e-002, 3.824461578200114e-001, 4.980741049529087e-002}, {2.857739928784489e-001, 8.627090300499446e-002, 3.827962398805592e-001, 5.059240458687936e-002}, {2.796503942775182e-001, 8.808964658709946e-002, 3.831581167626421e-001, 5.138755013061990e-002}, {2.737916253179937e-001, 8.988981115731574e-002, 3.835319223574790e-001, 5.219296291110304e-002}, {2.681881525413893e-001, 9.167235767109909e-002, 3.839177940069860e-001, 5.300876033616471e-002}, {2.628297823694911e-001, 9.343824154080706e-002, 3.843158726191478e-001, 5.383506147361034e-002}, {2.577083882939691e-001, 9.518838846980050e-002, 3.847263028172936e-001, 5.467198708900604e-002}, {2.528146968098489e-001, 9.692372676861677e-002, 3.851492329671904e-001, 5.551965968457499e-002}, {2.481407625412646e-001, 9.864516159899474e-002, 3.855848153536583e-001, 5.637820353923723e-002}, {2.436792821405074e-001, 1.003535861532541e-001, 3.860332062506214e-001, 5.724774474983266e-002}, {2.394228253416791e-001, 1.020498895213405e-001, 3.864945660289071e-001, 5.812841127356713e-002}, {2.353627081090076e-001, 1.037349621793819e-001, 3.869690592847504e-001, 5.902033297172822e-002}, {2.314947402031453e-001, 1.054096452054131e-001, 3.874568550524591e-001, 5.992364165471450e-002}, {2.278102791323715e-001, 1.070748228351733e-001, 3.879581267137887e-001, 6.083847112842353e-002}, {2.243046124050026e-001, 1.087313304366365e-001, 3.884730523862753e-001, 6.176495724204952e-002}, {2.209710682148270e-001, 1.103800228757795e-001, 3.890018148594768e-001, 6.270323793734339e-002}, {2.178041486530274e-001, 1.120217365977945e-001, 3.895446018605192e-001, 6.365345329938535e-002}, {2.147992288835744e-001, 1.136572986272354e-001, 3.901016061552393e-001, 6.461574560892683e-002}, {2.119504441278575e-001, 1.152875476716101e-001, 3.906730256654490e-001, 6.559025939636390e-002}, {2.092533159443932e-001, 1.169133040170711e-001, 3.912590637138232e-001, 6.657714149739562e-002}, {2.067032739670428e-001, 1.185353897562482e-001, 3.918599291191595e-001, 6.757654111043797e-002}, {2.042961465696757e-001, 1.201546233006569e-001, 3.924758363922071e-001, 6.858860985585547e-002}, {2.020274264697294e-001, 1.217718271650500e-001, 3.931070059001712e-001, 6.961350183708251e-002}, {1.998934200278083e-001, 1.233878153258157e-001, 3.937536640855738e-001, 7.065137370370449e-002}, {1.978906676118772e-001, 1.250034030877950e-001, 3.944160436227127e-001, 7.170238471657542e-002}, {1.960149967375316e-001, 1.266194158856914e-001, 3.950943836109010e-001, 7.276669681505510e-002}, {1.942639589257347e-001, 1.282366627914691e-001, 3.957889298488906e-001, 7.384447468644079e-002}, {1.926334395158679e-001, 1.298559775451477e-001, 3.964999349368981e-001, 7.493588583769048e-002}, {1.911216527090522e-001, 1.314781710093624e-001, 3.972276586428074e-001, 7.604110066951825e-002}, {1.897244443573727e-001, 1.331040891931062e-001, 3.979723679677965e-001, 7.716029255296758e-002}, {1.884406453288555e-001, 1.347345473673733e-001, 3.987343375897471e-001, 7.829363790855320e-002}, {1.872665265601232e-001, 1.363704021692120e-001, 3.995138498928030e-001, 7.944131628808254e-002}, {1.862007615810149e-001, 1.380124863528179e-001, 4.003111954501998e-001, 8.060351045926381e-002}, {1.852405381185921e-001, 1.396616628531729e-001, 4.011266731139843e-001, 8.178040649321632e-002}, {1.843843241480039e-001, 1.413187870896886e-001, 4.019605904389793e-001, 8.297219385500310e-002}, {1.836301668974131e-001, 1.429847328561038e-001, 4.028132638873196e-001, 8.417906549731445e-002}, {1.829760794685091e-001, 1.446603849414294e-001, 4.036850191842593e-001, 8.540121795743359e-002}, {1.824211264871903e-001, 1.463466283642914e-001, 4.045761916625467e-001, 8.663885145762512e-002}, {1.819631618475816e-001, 1.480443786791565e-001, 4.054871265152293e-001, 8.789217000909319e-002}, {1.816016538481242e-001, 1.497545442283934e-001, 4.064181792768163e-001, 8.916138151966188e-002}, {1.813349324261531e-001, 1.514780668535952e-001, 4.073697160521965e-001, 9.044669790534266e-002}, {1.811620760240895e-001, 1.532158932804673e-001, 4.083421140201370e-001, 9.174833520595821e-002}, {1.810826213005452e-001, 1.549689846495924e-001, 4.093357617802799e-001, 9.306651370500112e-002}, {1.810952242372437e-001, 1.567383340769575e-001, 4.103510597385373e-001, 9.440145805391896e-002}, {1.811998433905139e-001, 1.585249366978412e-001, 4.113884206505568e-001, 9.575339740102104e-002}, {1.813956689375554e-001, 1.603298228620418e-001, 4.124482699601924e-001, 9.712256552522057e-002}, {1.816823933789322e-001, 1.621540381227494e-001, 4.135310463731760e-001, 9.850920097482685e-002}, {1.820599684503609e-001, 1.639986506841799e-001, 4.146372023209510e-001, 9.991354721162989e-002}, {1.825281279631850e-001, 1.658647591954069e-001, 4.157672044730891e-001, 1.013358527605122e-001}, {1.830871732976372e-001, 1.677534826607340e-001, 4.169215343307577e-001, 1.027763713648546e-001}, {1.837369499625757e-001, 1.696659773067486e-001, 4.181006887506906e-001, 1.042353621480049e-001}, {1.844781042590138e-001, 1.716034189674459e-001, 4.193051806207994e-001, 1.057130897811016e-001}, {1.853108783667900e-001, 1.735670247032512e-001, 4.205355394147057e-001, 1.072098246575544e-001}, {1.862361146072027e-001, 1.755580382624963e-001, 4.217923119273905e-001, 1.087258430745072e-001}, {1.872542658540640e-001, 1.775777478187758e-001, 4.230760629130649e-001, 1.102614274216203e-001}, {1.883664161066458e-001, 1.796274712137456e-001, 4.243873758852488e-001, 1.118168663775390e-001}, {1.895736042833884e-001, 1.817085709127874e-001, 4.257268538261469e-001, 1.133924551144239e-001}, {1.908769430286885e-001, 1.838224530798516e-001, 4.270951200211710e-001, 1.149884955109506e-001}, {1.922778327149997e-001, 1.859705667570249e-001, 4.284928189189925e-001, 1.166052963742057e-001}, {1.937778134238934e-001, 1.881544096840238e-001, 4.299206170087649e-001, 1.182431736709383e-001}, {1.953785161419524e-001, 1.903755314738871e-001, 4.313792037637430e-001, 1.199024507686402e-001}, {1.970818360106390e-001, 1.926355337251292e-001, 4.328692926418148e-001, 1.215834586869736e-001}, {1.988898297699773e-001, 1.949360754056711e-001, 4.343916221158705e-001, 1.232865363600858e-001}, {2.008046698894440e-001, 1.972788765871647e-001, 4.359469567739310e-001, 1.250120309103874e-001}, {2.028288942436377e-001, 1.996657181573041e-001, 4.375360884891066e-001, 1.267602979344092e-001}, {2.049649981449501e-001, 2.020984525641909e-001, 4.391598376128025e-001, 1.285317018013877e-001}, {2.072159412632179e-001, 2.045789987236268e-001, 4.408190543010555e-001, 1.303266159652730e-001}, {2.095848139500683e-001, 2.071093533752463e-001, 4.425146198364234e-001, 1.321454232908996e-001}, {2.120747994517689e-001, 2.096915950472666e-001, 4.442474480766267e-001, 1.339885163951064e-001}, {2.146896366486244e-001, 2.123278812982627e-001, 4.460184869960368e-001, 1.358562980036432e-001}, {2.174330624393193e-001, 2.150204647564746e-001, 4.478287202391585e-001, 1.377491813247640e-001}, {2.203091662968168e-001, 2.177716908634281e-001, 4.496791688610775e-001, 1.396675904404572e-001}, {2.233224795982417e-001, 2.205840035106147e-001, 4.515708931119363e-001, 1.416119607163322e-001}, {2.264775934106465e-001, 2.234599582551517e-001, 4.535049943124936e-001, 1.435827392312657e-001}, {2.297796161462891e-001, 2.264022198467781e-001, 4.554826169139364e-001, 1.455803852279571e-001}, {2.332339727580958e-001, 2.294135744742551e-001, 4.575049506012141e-001, 1.476053705856567e-001}, {2.368464516137537e-001, 2.324969364640791e-001, 4.595732325689487e-001, 1.496581803163948e-001}, {2.406231945523244e-001, 2.356553566145465e-001, 4.616887499294117e-001, 1.517393130861526e-001}, {2.445708289741049e-001, 2.388920286712658e-001, 4.638528422778431e-001, 1.538492817625101e-001}, {2.486964917723498e-001, 2.422102993641000e-001, 4.660669044016151e-001, 1.559886139904228e-001}, {2.530076500232799e-001, 2.456136819772180e-001, 4.683323891611289e-001, 1.581578527979037e-001}, {2.575124198255152e-001, 2.491058607958477e-001, 4.706508105994867e-001, 1.603575572335084e-001}, {2.622194715578140e-001, 2.526907061233002e-001, 4.730237471960758e-001, 1.625883030377006e-001}, {2.671380191367600e-001, 2.563722872425211e-001, 4.754528453647684e-001, 1.648506833502761e-001}, {2.722780089338211e-001, 2.601548827603011e-001, 4.779398231980150e-001, 1.671453094562546e-001}, {2.776499422163085e-001, 2.640429994212740e-001, 4.804864744277954e-001, 1.694728115728024e-001}, {2.832652297087600e-001, 2.680413810642935e-001, 4.830946727123900e-001, 1.718338396799571e-001}, {2.891360595449805e-001, 2.721550304002138e-001, 4.857663761527585e-001, 1.742290643981639e-001}, {2.952753902018427e-001, 2.763892276998674e-001, 4.885036321697845e-001, 1.766591779158700e-001}, {3.016972268760389e-001, 2.807495456513702e-001, 4.913085827436155e-001, 1.791248949706735e-001}, {3.084166142336471e-001, 2.852418733111838e-001, 4.941834699821898e-001, 1.816269538878832e-001}, {3.154496431133436e-001, 2.898724405299774e-001, 4.971306421192923e-001, 1.841661176805713e-001}, {3.228136488040175e-001, 2.946478403193760e-001, 5.001525599745869e-001, 1.867431752156427e-001}, {3.305272701993334e-001, 2.995750572746494e-001, 5.032518038700848e-001, 1.893589424508100e-001}, {3.386106566497629e-001, 3.046614955486481e-001, 5.064310810895980e-001, 1.920142637477638e-001}, {3.470853252620201e-001, 3.099150166755338e-001, 5.096932338829231e-001, 1.947100132673631e-001}, {3.559747578322673e-001, 3.153439644648628e-001, 5.130412481742362e-001, 1.974470964531553e-001}, {3.653040508329576e-001, 3.209572158003464e-001, 5.164782628307193e-001, 2.002264516101224e-001}, {3.751005268804078e-001, 3.267642117238987e-001, 5.200075798247715e-001, 2.030490515862380e-001}, {3.853936367929098e-001, 3.327750113808939e-001, 5.236326750964294e-001, 2.059159055650719e-001}, {3.962153242954108e-001, 3.390003384908145e-001, 5.273572104049557e-001, 2.088280609785455e-001}, {4.076002326752048e-001, 3.454516381993498e-001, 5.311850461311969e-001, 2.117866055497767e-001}, {4.195859580454628e-001, 3.521411385938809e-001, 5.351202551856762e-001, 2.147926694769877e-001}, {4.322133830106544e-001, 3.590819173408294e-001, 5.391671381277255e-001, 2.178474277705093e-001}, {4.455847504181922e-001, 3.662869712655928e-001, 5.433302434909894e-001, 2.209521027562080e-001}, {4.597024094143034e-001, 3.737717322143078e-001, 5.476143770879206e-001, 2.241079667600003e-001}, {4.745555429725229e-001, 3.815533474816013e-001, 5.520246277787561e-001, 2.273163449897146e-001}, {4.901257557583882e-001, 3.897937038015808e-001, 5.566421198401972e-001, 2.306327012079619e-001}, {5.063188762578222e-001, 3.982520969512932e-001, 5.613313187177943e-001, 2.339568479963076e-001}, {5.231506865568163e-001, 4.069158720726419e-001, 5.660827733888582e-001, 2.372813206159511e-001}, {5.406462578991202e-001, 4.157904644940312e-001, 5.708968304793926e-001, 2.406057414176632e-001}, {5.588316450226307e-001, 4.248814441409453e-001, 5.757738454519713e-001, 2.439297408582211e-001}, {5.777342003001672e-001, 4.341945133486757e-001, 5.807141825807703e-001, 2.472529573723387e-001}, {5.973823532328585e-001, 4.437355186814879e-001, 5.857182148492396e-001, 2.505750372458956e-001}, {6.178058903813788e-001, 4.535104497074449e-001, 5.907863239449198e-001, 2.538956344906717e-001}, {6.390358272679809e-001, 4.635254487171909e-001, 5.959189001915087e-001, 2.572144107206164e-001}, {6.611046329585031e-001, 4.737868116278242e-001, 6.011163425492457e-001, 2.605310350297160e-001}, {6.840461012924548e-001, 4.843009977730023e-001, 6.063790585717825e-001, 2.638451838715473e-001}, {7.078955339544499e-001, 4.950746315277593e-001, 6.117074644259439e-001, 2.671565409405211e-001}, {7.326898428585754e-001, 5.061145075622476e-001, 6.171019848856783e-001, 2.704647970548958e-001}, {7.584674735300413e-001, 5.174275997007870e-001, 6.225630533267513e-001, 2.737696500416191e-001}, {7.852685225837038e-001, 5.290210645205079e-001, 6.280911117643314e-001, 2.770708046228762e-001}, {8.131349523353069e-001, 5.409022444662868e-001, 6.336866108855748e-001, 2.803679723045419e-001}, {8.421104635778781e-001, 5.530786787453849e-001, 6.393500100554367e-001, 2.836608712664611e-001}, {8.722406215697443e-001, 5.655581071809576e-001, 6.450817773793988e-001, 2.869492262544884e-001}, {9.035731218920524e-001, 5.783484730140734e-001, 6.508823897607542e-001, 2.902327684744185e-001}, {9.361575777347176e-001, 5.914579359394194e-001, 6.567523329200065e-001, 2.935112354877266e-001}, {9.700458060496284e-001, 6.048948732292336e-001, 6.626921014911419e-001, 2.967843711090281e-001}, {1.005291989370011e+000, 6.186678860505348e-001, 6.687021990776449e-001, 3.000519253054242e-001}, {1.041952490869116e+000, 6.327858120001110e-001, 6.747831383002202e-001, 3.033136540975303e-001}, {1.080086214331775e+000, 6.472577257095189e-001, 6.809354409115697e-001, 3.065693194622796e-001}, {1.119754627068662e+000, 6.620929484858137e-001, 6.871596378556201e-001, 3.098186892373309e-001}, {1.161021912738447e+000, 6.773010550003773e-001, 6.934562693572517e-001, 3.130615370273069e-001}, {1.203954887498334e+000, 6.928918842065448e-001, 6.998258850011435e-001, 3.162976421114647e-001}, {1.248623406898427e+000, 7.088755407142894e-001, 7.062690438568339e-001, 3.195267893531296e-001}, {1.295100419859135e+000, 7.252624056040015e-001, 7.127863145497119e-001, 3.227487691105891e-001}, {1.343461839212578e+000, 7.420631486316124e-001, 7.193782753529641e-001, 3.259633771495041e-001}, {1.393787066100554e+000, 7.592887285293669e-001, 7.260455143290677e-001, 3.291704145568307e-001}, {1.446158832199879e+000, 7.769504079175785e-001, 7.327886293986805e-001, 3.323696876561625e-001}, {1.500663624657993e+000, 7.950597567053790e-001, 7.396082284751718e-001, 3.355610079244260e-001}, {1.557391483641027e+000, 8.136286676665213e-001, 7.465049295460156e-001, 3.387441919099931e-001}, {1.616436486202836e+000, 8.326693587008284e-001, 7.534793608199935e-001, 3.419190611519823e-001}, {1.677896904646256e+000, 8.521943838823333e-001, 7.605321608220843e-001, 3.450854421009717e-001}, {1.741874983769677e+000, 8.722166491849841e-001, 7.676639784917609e-001, 3.482431660408027e-001}, {1.808477704796359e+000, 8.927494114574271e-001, 7.748754733450998e-001, 3.513920690115978e-001}, {1.877816685279789e+000, 9.138062955612120e-001, 7.821673155490562e-001, 3.545319917339357e-001}, {1.950008110826083e+000, 9.354013076198702e-001, 7.895401860415601e-001, 3.576627795341115e-001}, {2.025176264842242e+000, 9.575488033850547e-001, 7.969947767714907e-001, 3.607842822704026e-001}, {2.103446939662919e+000, 9.802636143987048e-001, 8.045317904513486e-001, 3.638963542604584e-001}, {2.184913845243859e+000, 1.003561394684095e+000, 8.121519401501026e-001, 3.669988542095216e-001}, {2.269763220050740e+000, 1.027456793165435e+000, 8.198559544878306e-001, 3.700916451436019e-001}, {2.357786913578639e+000, 1.051969888877999e+000, 8.276445572115562e-001, 3.731745943241333e-001}, {2.448578982729761e+000, 1.077119371791434e+000, 8.355184943898608e-001, 3.762475732019303e-001}, {2.554495397712962e+000, 1.102780965056649e+000, 8.434788475409446e-001, 3.793104573307565e-001}, {2.636142944230747e+000, 1.129517864911584e+000, 8.515248441466035e-001, 3.823631262876039e-001}, {2.753576242195605e+000, 1.156408150188240e+000, 8.596606204334905e-001, 3.854054635554983e-001}, {2.858337969405045e+000, 1.184320228835573e+000, 8.678832015184743e-001, 3.884373570657307e-001}, {2.971207245092491e+000, 1.212877645710290e+000, 8.761955271692427e-001, 3.914586976410320e-001}, {3.088790819517250e+000, 1.242174717307695e+000, 8.845979360059516e-001, 3.944693805727305e-001}, {3.211290782525183e+000, 1.272231405920666e+000, 8.930912596503021e-001, 3.974693046888215e-001}, {3.338918141302322e+000, 1.303068219692468e+000, 9.016763420849130e-001, 4.004583724710529e-001}, {3.471892018272730e+000, 1.334706240150083e+000, 9.103540397044045e-001, 4.034364899924997e-001}, {3.610440645775533e+000, 1.367167120982722e+000, 9.191252214485987e-001, 4.064035668556145e-001}, {3.754800508814665e+000, 1.400473111418473e+000, 9.279907688104051e-001, 4.093595161310653e-001}, {3.905216477735271e+000, 1.434647058532653e+000, 9.369515758788880e-001, 4.123042542968515e-001}, {4.065670804884121e+000, 1.469678174488928e+000, 9.460086192933500e-001, 4.152377011781764e-001}, {4.237761502904625e+000, 1.505565867821763e+000, 9.551628791098028e-001, 4.181597798876018e-001}, {4.423332904730818e+000, 1.542298398896243e+000, 9.644154775777063e-001, 4.210704167656975e-001}, {4.624516092786303e+000, 1.583704725720241e+000, 9.746473707878920e-001, 4.242404029888883e-001}, {4.850196574686571e+000, 1.626991736963335e+000, 9.851937689726753e-001, 4.274553962053975e-001}, {5.056790030629979e+000, 1.672846557345158e+000, 9.960690537668240e-001, 4.307167462684177e-001}, {5.322780585611914e+000, 1.720316041896115e+000, 1.007293551783249e+000, 4.340258706136082e-001}, {5.581856116174721e+000, 1.770664322860376e+000, 1.018880860299036e+000, 4.373842594703593e-001}, {5.866088407465559e+000, 1.823520560177389e+000, 1.030853140388873e+000, 4.407934798537482e-001}, {6.171923002120571e+000, 1.879189565923113e+000, 1.043230569280668e+000, 4.442551829095663e-001}, {6.501535536191801e+000, 1.937885751613769e+000, 1.056035595849955e+000, 4.477711093076880e-001}, {6.857377194386117e+000, 1.999844889612192e+000, 1.069292431020710e+000, 4.513430963808631e-001}, {7.242214043652889e+000, 2.065326797427353e+000, 1.083027227973919e+000, 4.549730857610047e-001}, {7.659178646510535e+000, 2.134618343011173e+000, 1.097268285542566e+000, 4.586631318040277e-001}, {8.111827066134506e+000, 2.208036973520845e+000, 1.112046277772487e+000, 4.624154109045757e-001}, {8.604208529071581e+000, 2.285934758959754e+000, 1.127394514184514e+000, 4.662322318172514e-001}, {9.140948252644797e+000, 2.368703086413673e+000, 1.143349235435159e+000, 4.701160471199221e-001}, {9.727346257355944e+000, 2.456778125271054e+000, 1.159949950207535e+000, 4.740694659762598e-001}, {1.036949636473117e+001, 2.550647201445953e+000, 1.177239820295516e+000, 4.780952683808142e-001}, {1.107442943815378e+001, 2.650856267473131e+000, 1.195266102140809e+000, 4.821964211012327e-001}, {1.185028605617332e+001, 2.758018689992081e+000, 1.214080654794935e+000, 4.863760955695670e-001}, {1.270652798974013e+001, 2.872825592971998e+000, 1.233740526395416e+000, 4.906376880197966e-001}, {1.365419174621447e+001, 2.996058162729545e+000, 1.254308633466170e+000, 4.949848422233832e-001}, {1.470620059822895e+001, 3.128602227412618e+000, 1.275854551067133e+000, 4.994214752410865e-001}, {1.587774199700023e+001, 3.271465705618580e+000, 1.298455434935754e+000, 5.039518066908264e-001}, {1.720674254013149e+001, 3.425586345730544e+000, 1.322197606263238e+000, 5.085803921309422e-001}, {1.863402248089567e+001, 3.593353579496717e+000, 1.347174025974930e+000, 5.133121612816782e-001}, {2.032390211918452e+001, 3.774153760759719e+000, 1.373499219827730e+000, 5.181524619605369e-001}, {2.218601347832784e+001, 3.971869398731903e+000, 1.401284417001323e+000, 5.231071115480921e-001}, {2.431252464422925e+001, 4.187482117105547e+000, 1.430671346578243e+000, 5.281824536278078e-001}, {2.673985851010544e+001, 4.423583955149483e+000, 1.461812601611662e+000, 5.333854293394883e-001}, {2.952491937183151e+001, 4.682968036424375e+000, 1.494883366150635e+000, 5.387236561640727e-001}, {3.273828578159125e+001, 4.968928762622564e+000, 1.530083871593750e+000, 5.442055229446516e-001}, {3.646825750913934e+001, 5.285374439016692e+000, 1.567644134852697e+000, 5.498403020789376e-001}, {4.082633712570511e+001, 5.636970657586560e+000, 1.607829829067538e+000, 5.556382833229866e-001}, {4.595475926455129e+001, 6.029324175734520e+000, 1.650949619985925e+000, 5.616109344677590e-001}, {5.203696918320384e+001, 6.469220871161562e+000, 1.697364417633431e+000, 5.677710957231201e-001}, {5.931241581636025e+001, 6.964936570782171e+000, 1.747499159582017e+000, 5.741332167790794e-001}, {6.809776191782187e+001, 7.526647036007701e+000, 1.801857981792066e+000, 5.807136484549547e-001}, {7.881778054554678e+001, 8.166974540194117e+000, 1.861043982241702e+000, 5.875310049495474e-001}, {9.207856359262058e+001, 8.901415236315513e+000, 1.925786076833259e+000, 5.946066185193436e-001}, {1.085962925592044e+002, 9.751235331139156e+000, 1.996968777326911e+000, 6.019651167819492e-001}, {1.296612727594772e+002, 1.073953725331416e+001, 2.075696926484503e+000, 6.096351651125727e-001}, {1.568149508938095e+002, 1.190149594899319e+001, 2.163329563019325e+000, 6.176504358167556e-001}, {1.925895489261716e+002, 1.327895255374134e+001, 2.261610095071371e+000, 6.260508899137490e-001}, {2.407894220277205e+002, 1.492863095592906e+001, 2.372777247256969e+000, 6.348845111387734e-001}, {3.074933991972216e+002, 1.692543720677949e+001, 2.499777113694852e+000, 6.442096900359834e-001}, {4.028115941258301e+002, 1.936864049070909e+001, 2.646574749423806e+000, 6.540985866240505e-001}, {5.444627680553028e+002, 2.238732251366779e+001, 2.818658896530704e+000, 6.646420006807281e-001}, {7.654263901392872e+002, 2.613984326739216e+001, 3.023895327133340e+000, 6.759566538738164e-001}, {1.132473145221229e+003, 3.077464245663644e+001, 3.274097890822487e+000, 6.881964979202373e-001}, {1.728456306009293e+003, 3.703669144378617e+001, 3.586095407785845e+000, 7.015710980094141e-001}, {2.746345210190641e+003, 4.636511685430635e+001, 3.986312539776484e+000, 7.163772638856115e-001}, {4.641775374977525e+003, 6.199723710600262e+001, 4.516428772903993e+000, 7.330575520262050e-001}, {8.525657962231340e+003, 6.810054194649604e+001, 5.081630948855855e+000, 7.472071449747022e-001}, {1.741572862879567e+004, 6.219045752188775e+001, 5.890642485434674e+000, 7.632728136051337e-001}, {3.813458054210775e+004, 5.370618375512508e+001, 7.124228169883730e+000, 7.828757764751582e-001}, {8.589158440190039e+004, 1.090981683303350e+002, 8.902873285623791e+000, 8.059157661363586e-001}, {1.951303750681718e+005, 2.375849575768755e+002, 1.117514113283684e+001, 8.277377240352704e-001}, {4.782548713012288e+005, 1.986473214368117e+002, 1.394735114459777e+001, 8.459822722289891e-001}, {1.185655347397813e+006, -2.174684780917248e+002, 1.762979102357640e+001, 8.629254292602320e-001}, {2.750846962420604e+006, -1.629109294341851e+002, 2.259666061556802e+001, 8.814128590616266e-001}, {6.123913630880820e+006, 4.322208699496733e+003, 2.998861227777463e+001, 9.042234397623542e-001}, {1.223685185147719e+007, 8.090218357744150e+003, 3.638774289535504e+001, 9.167366942666613e-001}, {2.818621443600825e+007, 3.085711548192580e+003, 4.135729893033309e+001, 9.225727665578074e-001}, {8.316585285067827e+007, -1.136324148807467e+004, 5.196252092604191e+001, 9.326388658049112e-001}, {2.794244200228550e+008, -6.113667992222013e+004, 7.038061963269288e+001, 9.435863614001085e-001}, {9.252325428804928e+008, -5.487463721818197e+004, 9.110802490104044e+001, 9.579454741567480e-001}, {2.237265034221097e+009, 6.896819415082679e+004, 1.142108494843807e+002, 9.677749072712486e-001}, {1.597732512872630e+013, -7.580581567199930e+008, 8.988788796440838e+003, 9.741584695812871e-001}};
        constant Real[Ninterval, 4] hlcoef = {{7.395064117384627e+009, -1.280865388176200e+006, 2.821575750527614e+002, 1.833836171003976e-001}, {6.597504135407492e+009, -1.212511325876777e+006, 2.753491351615672e+002, 1.841406426859871e-001}, {5.780271674084454e+009, -1.152633155136978e+006, 2.697490006419553e+002, 1.847764786820451e-001}, {4.958860729534658e+009, -1.042899311995140e+006, 2.575580906113897e+002, 1.862237508639398e-001}, {4.254225882522524e+009, -9.436430636231016e+005, 2.459280611632260e+002, 1.876807622923263e-001}, {3.649929287231926e+009, -8.538684890799312e+005, 2.348330720290646e+002, 1.891475943673219e-001}, {3.131540328303725e+009, -7.726606490030770e+005, 2.242477593033248e+002, 1.906244199793511e-001}, {2.686884429215740e+009, -6.992021917100513e+005, 2.141484795719950e+002, 1.921113515976508e-001}, {2.305436681190764e+009, -6.327500405900241e+005, 2.045124826635191e+002, 1.936085324364108e-001}, {1.978225892136117e+009, -5.726354169435729e+005, 1.953183098525792e+002, 1.951160756856359e-001}, {1.697521363709874e+009, -5.182511799267425e+005, 1.865453222831365e+002, 1.966341253040380e-001}, {1.456683048691095e+009, -4.690492050108880e+005, 1.781740380829120e+002, 1.981627952385604e-001}, {1.250074360886440e+009, -4.245346083004282e+005, 1.701855963205966e+002, 1.997022606387281e-001}, {1.072800606962726e+009, -3.842582037066196e+005, 1.625621803354005e+002, 2.012526666726048e-001}, {9.207021541607516e+008, -3.478163169361438e+005, 1.552869834519622e+002, 2.028141285158312e-001}, {7.901943344753319e+008, -3.148419383589053e+005, 1.483437057813515e+002, 2.043868226073761e-001}, {6.782161334430934e+008, -2.850048993978956e+005, 1.417170959818027e+002, 2.059708649963459e-001}, {5.825727219202920e+008, -2.580214571990632e+005, 1.353924992211892e+002, 2.075664330398644e-001}, {5.018477901925672e+008, -2.336618459414496e+005, 1.293563597550240e+002, 2.091736741482984e-001}, {4.336339749859386e+008, -2.116826692784306e+005, 1.235956428311352e+002, 2.107927362153820e-001}, {3.759101976651321e+008, -1.921682462515702e+005, 1.182042406216403e+002, 2.123911161514204e-001}, {3.267571477181848e+008, -1.752745050595466e+005, 1.133228792803702e+002, 2.139142206063779e-001}, {2.843020873596044e+008, -1.599683613280694e+005, 1.086775876446765e+002, 2.154377045739531e-001}, {2.475979262638457e+008, -1.460916038079511e+005, 1.042554766441906e+002, 2.169615822712266e-001}, {2.158340827399827e+008, -1.335024950386115e+005, 1.000444324122937e+002, 2.184858675939239e-001}, {1.883207309006085e+008, -1.220742895035692e+005, 9.603307088337355e+001, 2.200105741258566e-001}, {1.644651841228906e+008, -1.116932087097448e+005, 9.221068373842154e+001, 2.215357151481048e-001}, {1.437632632379007e+008, -1.022574640228438e+005, 8.856720217299345e+001, 2.230613036479431e-001}, {1.257801607275091e+008, -9.367549193296053e+004, 8.509314677627712e+001, 2.245873523275273e-001}, {1.101454874055535e+008, -8.586529010338520e+004, 8.177959848067110e+001, 2.261138736123414e-001}, {9.653972368356156e+007, -7.875303244948787e+004, 7.861815481978856e+001, 2.276408796594130e-001}, {8.468871988652200e+007, -7.227239695689626e+004, 7.560090235615557e+001, 2.291683823653079e-001}, {7.435767250336401e+007, -6.636379223835093e+004, 7.272038489459962e+001, 2.306963933739068e-001}, {6.534286001294021e+007, -6.097342167031716e+004, 6.996957046343296e+001, 2.322249240839693e-001}, {5.747027807155222e+007, -5.605298520200417e+004, 6.734183209615867e+001, 2.337539856564991e-001}, {5.058892155208572e+007, -5.155884014645571e+004, 6.483091539577785e+001, 2.352835890219054e-001}, {4.456897749091873e+007, -4.745168641553626e+004, 6.243092046710518e+001, 2.368137448869720e-001}, {3.929797808634566e+007, -4.369602037553607e+004, 6.013627711697743e+001, 2.383444637416446e-001}, {3.467909626236541e+007, -4.025982723829874e+004, 5.794172688510435e+001, 2.398757558656324e-001}, {3.062784360211651e+007, -3.711410238256247e+004, 5.584230094500428e+001, 2.414076313348311e-001}, {2.707199455967587e+007, -3.423274054960324e+004, 5.383330780575171e+001, 2.429401000275848e-001}, {2.394806063151847e+007, -3.159200650084850e+004, 5.191030928225440e+001, 2.444731716307669e-001}, {2.120145854494023e+007, -2.917049319028297e+004, 5.006911233397096e+001, 2.460068556457158e-001}, {1.878470848130406e+007, -2.694879400670043e+004, 4.830575024370947e+001, 2.475411613940012e-001}, {1.665640139112113e+007, -2.490930349109187e+004, 4.661646985518280e+001, 2.490760980230453e-001}, {1.478065625824185e+007, -2.303608166989841e+004, 4.499772037807755e+001, 2.506116745115950e-001}, {1.312624886938225e+007, -2.131466500927661e+004, 4.344614015879794e+001, 2.521478996750478e-001}, {1.166585545469223e+007, -1.973190580002538e+004, 4.195854541513062e+001, 2.536847821706443e-001}, {1.037575804683703e+007, -1.827588228385652e+004, 4.053192137337701e+001, 2.552223305025196e-001}, {9.235229821301350e+006, -1.693575336267236e+004, 3.916341110770780e+001, 2.567605530266264e-001}, {8.226161529454760e+006, -1.570166014825697e+004, 3.785030689607822e+001, 2.582994579555294e-001}, {7.332714409838634e+006, -1.456463239248035e+004, 3.659004174391924e+001, 2.598390533630776e-001}, {6.541067280596239e+006, -1.351651109839700e+004, 3.538018157148768e+001, 2.613793471889549e-001}, {5.839089097217685e+006, -1.254985887421323e+004, 3.421841707983741e+001, 2.629203472431124e-001}, {5.216174993443639e+006, -1.165790355620624e+004, 3.310255732922824e+001, 2.644620612100892e-001}, {4.663018167146263e+006, -1.083446843025749e+004, 3.203052265688924e+001, 2.660044966532249e-001}, {4.171444464302824e+006, -1.007391792850541e+004, 3.100033862903940e+001, 2.675476610187589e-001}, {3.734291063599596e+006, -9.371112234283090e+003, 3.001013038565671e+001, 2.690915616398324e-001}, {3.345264196959495e+006, -8.721357281608769e+003, 2.905811682622430e+001, 2.706362057403845e-001}, {2.998804824052956e+006, -8.120359510245587e+003, 2.814260537246550e+001, 2.721816004389522e-001}, {2.690059117043254e+006, -7.564203866773062e+003, 2.726198786072066e+001, 2.737277527523759e-001}, {2.414722165518809e+006, -7.049301389752438e+003, 2.641473480407623e+001, 2.752746695994082e-001}, {2.169006576676412e+006, -6.572371181362734e+003, 2.559939204631222e+001, 2.768223578042399e-001}, {1.949589547380671e+006, -6.130413910998713e+003, 2.481457654061498e+001, 2.783708240999319e-001}, {1.753513335070667e+006, -5.720675273235230e+003, 2.405897188170692e+001, 2.799200751317631e-001}, {1.578181984153911e+006, -5.340636910887383e+003, 2.333132571024671e+001, 2.814701174605012e-001}, {1.421296797534924e+006, -4.987988415116040e+003, 2.263044564557126e+001, 2.830209575655875e-001}, {1.280826297625475e+006, -4.660611060652745e+003, 2.195519632940761e+001, 2.845726018482448e-001}, {1.154967887885402e+006, -4.356559333855659e+003, 2.130449633293048e+001, 2.861250566345134e-001}, {1.042131487251255e+006, -4.074049315833184e+003, 2.067731557349638e+001, 2.876783281782094e-001}, {9.409048828988477e+005, -3.811441190960783e+003, 2.007267232055989e+001, 2.892324226638147e-001}, {8.500340926125752e+005, -3.567227768237408e+003, 1.948963084447095e+001, 2.907873462092958e-001}, {7.684098011669957e+005, -3.340024748034876e+003, 1.892729914062116e+001, 2.923431048688557e-001}, {6.950454961539843e+005, -3.128558197222760e+003, 1.838482647741899e+001, 2.938997046356221e-001}, {6.290634426463042e+005, -2.931655562235562e+003, 1.786140137631819e+001, 2.954571514442698e-001}, {5.696851277746989e+005, -2.748238224841836e+003, 1.735624969777203e+001, 2.970154511735833e-001}, {5.162174096472578e+005, -2.577312465118152e+003, 1.686863261843808e+001, 2.985746096489578e-001}, {4.680412177608077e+005, -2.417962027029424e+003, 1.639784486931878e+001, 3.001346326448444e-001}, {4.246092785921820e+005, -2.269344049512283e+003, 1.594321324426641e+001, 3.016955258871379e-001}, {3.854290946306968e+005, -2.130679129811940e+003, 1.550409461031256e+001, 3.032572950555081e-001}, {3.500644784375472e+005, -2.001249801636266e+003, 1.507987485327908e+001, 3.048199457856796e-001}, {3.181254941087923e+005, -1.880393392444164e+003, 1.466996714224077e+001, 3.063834836716620e-001}, {2.892627964124076e+005, -1.767497553444107e+003, 1.427381064630017e+001, 3.079479142679245e-001}, {2.631658597077979e+005, -1.661997457406561e+003, 1.389086939175560e+001, 3.095132430915293e-001}, {2.395556802770178e+005, -1.563370394708744e+003, 1.352063085122437e+001, 3.110794756242089e-001}, {2.181834226552594e+005, -1.471133577957063e+003, 1.316260497547039e+001, 3.126466173144076e-001}, {1.988261272748282e+005, -1.384840332961980e+003, 1.281632300534341e+001, 3.142146735792670e-001}, {1.812841164270276e+005, -1.304077374256214e+003, 1.248133648021494e+001, 3.157836498065826e-001}, {1.653780267940605e+005, -1.228461972770334e+003, 1.215721625109387e+001, 3.173535513567038e-001}, {1.509481016353733e+005, -1.157640295687853e+003, 1.184355163838609e+001, 3.189243835644045e-001}, {1.378494330329677e+005, -1.091283635929976e+003, 1.153994936397996e+001, 3.204961517407096e-001}, {1.259533798538558e+005, -1.029088314073179e+003, 1.124603300581368e+001, 3.220688611746812e-001}, {1.151434947602917e+005, -9.707721560066344e+002, 1.096144193815056e+001, 3.236425171351746e-001}, {1.053155086752598e+005, -9.160737446488935e+002, 1.068583076706654e+001, 3.252171248725524e-001}, {9.637539799014138e+004, -8.647503663978431e+002, 1.041886853531754e+001, 3.267926896203659e-001}, {8.823888454783670e+004, -8.165769553308005e+002, 1.016023811189488e+001, 3.283692165970028e-001}, {8.082975256302909e+004, -7.713442852693606e+002, 9.909635478268301e+000, 3.299467110073037e-001}, {7.407960702001616e+004, -7.288582119329744e+002, 9.666769207898108e+000, 3.315251780441446e-001}, {6.792665838390439e+004, -6.889382110921721e+002, 9.431359824692734e+000, 3.331046228899912e-001}, {6.231526198828789e+004, -6.514165353801458e+002, 9.203139298263135e+000, 3.346850507184217e-001}, {5.719513812339059e+004, -6.161371424524319e+002, 8.981850501633565e+000, 3.362664666956208e-001}, {5.252104982234649e+004, -5.829549835215390e+002, 8.767246744918761e+000, 3.378488759818509e-001}, {4.825192751323921e+004, -5.517349069002831e+002, 8.559091257223685e+000, 3.394322837328866e-001}, {4.435085262412058e+004, -5.223512693230589e+002, 8.357156812102812e+000, 3.410166951014356e-001}, {4.078431692402485e+004, -4.946869454272916e+002, 8.161225235617962e+000, 3.426021152385257e-001}, {3.752210023182508e+004, -4.686329059593669e+002, 7.971087052548851e+000, 3.441885492948688e-001}, {3.453677849407373e+004, -4.440874766662903e+002, 7.786541067524770e+000, 3.457760024222062e-001}, {3.180355626502269e+004, -4.209559160395517e+002, 7.607394028698087e+000, 3.473644797746268e-001}, {2.929998425341201e+004, -3.991498796401928e+002, 7.433460265705899e+000, 3.489539865098665e-001}, {2.700568725535247e+004, -3.785869240593804e+002, 7.264561353833670e+000, 3.505445277905829e-001}, {2.490219323552633e+004, -3.591901229932722e+002, 7.100525811207305e+000, 3.521361087856166e-001}, {2.297274465666998e+004, -3.408876693791219e+002, 6.941188795306921e+000, 3.537287346712248e-001}, {2.120214641232764e+004, -3.236125217409932e+002, 6.786391817255598e+000, 3.553224106323019e-001}, {1.957657344633676e+004, -3.073020267951967e+002, 6.635982461076544e+000, 3.569171418635831e-001}, {1.808347931117164e+004, -2.918976581600808e+002, 6.489814137991496e+000, 3.585129335708239e-001}, {1.671146762926517e+004, -2.773447131490234e+002, 6.347745831608338e+000, 3.601097909719717e-001}, {1.545013740159694e+004, -2.635920046989097e+002, 6.209641855271004e+000, 3.617077192983143e-001}, {1.429006951012745e+004, -2.505917089097457e+002, 6.075371650375700e+000, 3.633067237956181e-001}, {1.322266968511326e+004, -2.382990480062072e+002, 5.944809543387017e+000, 3.649068097252486e-001}, {1.224010132039165e+004, -2.266721040737960e+002, 5.817834554960569e+000, 3.665079823652802e-001}, {1.133523722405783e+004, -2.156716504900735e+002, 5.694330210582269e+000, 3.681102470115885e-001}, {1.050158642348279e+004, -2.052609525375157e+002, 5.574184345532527e+000, 3.697136089789342e-001}, {9.733206313299734e+003, -1.954055676141257e+002, 5.457288921742155e+000, 3.713180736020321e-001}, {9.024700730222718e+003, -1.860732540723482e+002, 5.343539877398015e+000, 3.729236462366104e-001}, {8.371123402082400e+003, -1.772337586979163e+002, 5.232836941729546e+000, 3.745303322604583e-001}, {7.767976360117369e+003, -1.688587391346637e+002, 5.125083500057396e+000, 3.761381370744614e-001}, {7.211132037590188e+003, -1.609215840176348e+002, 5.020186427402752e+000, 3.777470661036311e-001}, {6.696830442802067e+003, -1.533973451085674e+002, 4.918055965107774e+000, 3.793571247981251e-001}, {6.221630107308341e+003, -1.462626001578099e+002, 4.818605572961186e+000, 3.809683186342531e-001}, {5.782380642858814e+003, -1.394953568738606e+002, 4.721751803873504e+000, 3.825806531154816e-001}, {5.376201651577415e+003, -1.330749668179309e+002, 4.627414182825382e+000, 3.841941337734274e-001}, {5.000455207451565e+003, -1.269820315075479e+002, 4.535515085732524e+000, 3.858087661688454e-001}, {4.652724834587218e+003, -1.211983226404149e+002, 4.445979627643788e+000, 3.874245558926092e-001}, {4.330795240187265e+003, -1.157067060614417e+002, 4.358735554805467e+000, 3.890415085666860e-001}, {4.032641019365463e+003, -1.104910799208306e+002, 4.273713143506992e+000, 3.906596298451056e-001}, {3.756394747613435e+003, -1.055362848885977e+002, 4.190845093212067e+000, 3.922789254149277e-001}, {3.500354297338667e+003, -1.008280802229431e+002, 4.110066446614686e+000, 3.938994009971991e-001}, {3.262950609227280e+003, -9.635305138220821e+001, 4.031314482734651e+000, 3.955210623479115e-001}, {3.042742369663052e+003, -9.209857304695458e+001, 3.954528640335044e+000, 3.971439152589546e-001}, {2.838413065597216e+003, -8.805277060758006e+001, 3.879650436609394e+000, 3.987679655590649e-001}, {2.648742989264185e+003, -8.420444188141711e+001, 3.806623374949973e+000, 4.003932191147761e-001}, {2.472617600983757e+003, -8.054304904610477e+001, 3.735392885948790e+000, 4.020196818313603e-001}, {2.309012318091831e+003, -7.705866086409024e+001, 3.665906242883802e+000, 4.036473596537750e-001}, {2.156978720419112e+003, -7.374190599668950e+001, 3.598112490902449e+000, 4.052762585676071e-001}, {2.015650036432498e+003, -7.058396265218144e+001, 3.531962391385521e+000, 4.069063846000124e-001}, {1.884225012048499e+003, -6.757650377249485e+001, 3.467408345228365e+000, 4.085377438206608e-001}, {1.761967295914424e+003, -6.471167807273380e+001, 3.404404338708559e+000, 4.101703423426775e-001}, {1.648196125371299e+003, -6.198207308290163e+001, 3.342905880402665e+000, 4.118041863235865e-001}, {1.542286784888126e+003, -5.938069940472492e+001, 3.282869949941535e+000, 4.134392819662572e-001}, {1.443661841752103e+003, -5.690095495331793e+001, 3.224254937399627e+000, 4.150756355198507e-001}, {1.351787821483400e+003, -5.453660450267201e+001, 3.167020594458915e+000, 4.167132532807668e-001}, {1.266175045567968e+003, -5.228176450788781e+001, 3.111127986917860e+000, 4.183521415935981e-001}, {1.186368387856992e+003, -5.013086904303047e+001, 3.056539439710153e+000, 4.199923068520857e-001}, {1.111950039713011e+003, -4.807866488602586e+001, 3.003218499628963e+000, 4.216337555000751e-001}, {1.042532610364972e+003, -4.612018271016708e+001, 2.951129883955490e+000, 4.232764940324793e-001}, {9.777589417052835e+002, -4.425072650328128e+001, 2.900239442556008e+000, 4.249205289962486e-001}, {9.172983979562611e+002, -4.246585399078810e+001, 2.850514114630586e+000, 4.265658669913391e-001}, {8.608453554430503e+002, -4.076136366761418e+001, 2.801921891149074e+000, 4.282125146716945e-001}, {8.081171471965247e+002, -3.913328048140521e+001, 2.754431776852438e+000, 4.298604787462255e-001}, {7.588523354947266e+002, -3.757784280141392e+001, 2.708013754148967e+000, 4.315097659798046e-001}, {7.128085053227204e+002, -3.609148886941360e+001, 2.662638747952767e+000, 4.331603831942572e-001}, {6.697616301899591e+002, -3.467084767166227e+001, 2.618278593896983e+000, 4.348123372693731e-001}, {6.295038171706216e+002, -3.331272564117698e+001, 2.574906004565767e+000, 4.364656351439127e-001}, {5.918428655279847e+002, -3.201409883532291e+001, 2.532494540378606e+000, 4.381202838166297e-001}, {5.565999606334636e+002, -3.077210029665694e+001, 2.491018578065730e+000, 4.397762903473017e-001}, {5.236101967728564e+002, -2.958401561514848e+001, 2.450453285249495e+000, 4.414336618577676e-001}, {4.927195823157568e+002, -2.844726859478019e+001, 2.410774588999436e+000, 4.430924055329756e-001}, {4.637860960803320e+002, -2.735941953218890e+001, 2.371959154294425e+000, 4.447525286220433e-001}, {4.366774822365882e+002, -2.631815311065120e+001, 2.333984354679967e+000, 4.464140384393267e-001}, {4.112710296977796e+002, -2.532127349245170e+001, 2.296828250141353e+000, 4.480769423654994e-001}, {3.874528219662709e+002, -2.436669747558107e+001, 2.260469563259040e+000, 4.497412478486488e-001}, {3.651169185882785e+002, -2.345244781267422e+001, 2.224887656378541e+000, 4.514069624053770e-001}, {3.441650927913553e+002, -2.257664839246537e+001, 2.190062510638771e+000, 4.530740936219221e-001}, {3.245056450427372e+002, -2.173751677003992e+001, 2.155974703738442e+000, 4.547426491552871e-001}, {3.060536399057885e+002, -2.093336163040314e+001, 2.122605392018528e+000, 4.564126367343871e-001}, {2.887300097307417e+002, -2.016257613322068e+001, 2.089936289195907e+000, 4.580840641612062e-001}, {2.724610482918949e+002, -1.942363328612018e+001, 2.057949648179311e+000, 4.597569393119745e-001}, {2.571781773457540e+002, -1.871508235114862e+001, 2.026628243849622e+000, 4.614312701383573e-001}, {2.428177694319144e+002, -1.803554520313706e+001, 1.995955355734756e+000, 4.631070646686624e-001}, {2.293201220245030e+002, -1.738371018461584e+001, 1.965914749708475e+000, 4.647843310090629e-001}, {2.166301416228736e+002, -1.675833219262003e+001, 1.936490665160709e+000, 4.664630773448359e-001}, {2.046961071710434e+002, -1.615822543778556e+001, 1.907667795779915e+000, 4.681433119416241e-001}, {1.934698629445243e+002, -1.558226242939191e+001, 1.879431277425193e+000, 4.698250431467114e-001}, {1.829066943899449e+002, -1.502937113484294e+001, 1.851766673341252e+000, 4.715082793903179e-001}, {1.729646537632953e+002, -1.449853037846985e+001, 1.824659958829607e+000, 4.731930291869178e-001}, {1.636047362947269e+002, -1.398876887679239e+001, 1.798097509845817e+000, 4.748793011365756e-001}, {1.547905996606153e+002, -1.349916208755315e+001, 1.772066089112747e+000, 4.765671039263029e-001}, {1.464881127185945e+002, -1.302882884420155e+001, 1.746552833172282e+000, 4.782564463314370e-001}, {1.386655781437395e+002, -1.257693080761831e+001, 1.721545242210061e+000, 4.799473372170469e-001}, {1.312933609201193e+002, -1.214266917026167e+001, 1.697031166940830e+000, 4.816397855393539e-001}, {1.243436673524583e+002, -1.172528246603632e+001, 1.672998797715616e+000, 4.833338003471823e-001}, {1.177906812224748e+002, -1.132404579365274e+001, 1.649436654853634e+000, 4.850293907834343e-001}, {1.116101176422379e+002, -1.093826750962181e+001, 1.626333576621976e+000, 4.867265660865868e-001}, {1.057793766234323e+002, -1.056728882118992e+001, 1.603678710849861e+000, 4.884253355922194e-001}, {1.002772484594161e+002, -1.021048121888435e+001, 1.581461504068653e+000, 4.901257087345642e-001}, {9.508393109691426e+001, -9.867245539491131e+000, 1.559671692999208e+000, 4.918276950480881e-001}, {9.018086100582940e+001, -9.537010120052251e+000, 1.538299295044362e+000, 4.935313041690984e-001}, {8.555069486770042e+001, -9.219229747811003e+000, 1.517334600000931e+000, 4.952365458373848e-001}, {8.117714599214482e+001, -8.913383954440521e+000, 1.496768161220113e+000, 4.969434298978837e-001}, {7.704503147414225e+001, -8.618976357910185e+000, 1.476590788161609e+000, 4.986519663023814e-001}, {7.314002294338168e+001, -8.335532613364549e+000, 1.456793537585971e+000, 5.003621651112420e-001}, {6.944879810386360e+001, -8.062600431450619e+000, 1.437367707389535e+000, 5.020740364951754e-001}, {6.595880156239269e+001, -7.799747511242599e+000, 1.418304827863057e+000, 5.037875907370351e-001}, {6.265828559603662e+001, -7.546561136231898e+000, 1.399596655606285e+000, 5.055028382336494e-001}, {5.953623627670859e+001, -7.302647062139070e+000, 1.381235166362647e+000, 5.072197894976949e-001}, {5.658229760929855e+001, -7.067628502368594e+000, 1.363212548354849e+000, 5.089384551595994e-001}, {5.378678727776725e+001, -6.841145600895276e+000, 1.345521196299155e+000, 5.106588459694897e-001}, {5.114058172786601e+001, -6.622854212012143e+000, 1.328153704681671e+000, 5.123809727991734e-001}, {4.863514814255605e+001, -6.412425552714074e+000, 1.311102862409988e+000, 5.141048466441640e-001}, {4.626245004092587e+001, -6.209545123139677e+000, 1.294361646523955e+000, 5.158304786257448e-001}, {4.401494669041566e+001, -6.013912245135423e+000, 1.277923217027002e+000, 5.175578799930805e-001}, {4.188556400289667e+001, -5.825239391259091e+000, 1.261780911389945e+000, 5.192870621253667e-001}, {3.986762775448643e+001, -5.643251367096637e+000, 1.245928239132817e+000, 5.210180365340277e-001}, {3.795489834408001e+001, -5.467685094511304e+000, 1.230358877298484e+000, 5.227508148649598e-001}, {3.614148268541150e+001, -5.298288645847125e+000, 1.215066664937051e+000, 5.244854089008243e-001}, {3.442185278609600e+001, -5.134821009449518e+000, 1.200045598933553e+000, 5.262218305633868e-001}, {3.279079584820779e+001, -4.977051405468687e+000, 1.185289829131247e+000, 5.279600919159078e-001}, {3.124342934154262e+001, -4.824759024073568e+000, 1.170793654218353e+000, 5.297002051655871e-001}, {2.977513427897120e+001, -4.677732260255681e+000, 1.156551516988663e+000, 5.314421826660609e-001}, {2.838157596586128e+001, -4.535768567068232e+000, 1.142558000732308e+000, 5.331860369199514e-001}, {2.705867366805578e+001, -4.398673923357351e+000, 1.128807824937147e+000, 5.349317805814788e-001}, {2.580257661383335e+001, -4.266262403536937e+000, 1.115295841412068e+000, 5.366794264591228e-001}, {2.460966531506359e+001, -4.138355917689477e+000, 1.102017030714079e+000, 5.384289875183563e-001}, {2.347652634935337e+001, -4.014783768732895e+000, 1.088966498339614e+000, 5.401804768844284e-001}, {2.239994189186168e+001, -3.895382342766132e+000, 1.076139471288119e+000, 5.419339078452174e-001}, {2.137688056201431e+001, -3.779994809266803e+000, 1.063531294680509e+000, 5.436892938541545e-001}, {2.040448375466456e+001, -3.668470797273908e+000, 1.051137428418400e+000, 5.454466485332009e-001}, {1.948005543173328e+001, -3.560666110129647e+000, 1.038953444001990e+000, 5.472059856759085e-001}, {1.860105179024829e+001, -3.456442449726411e+000, 1.026975021437601e+000, 5.489673192505425e-001}, {1.776507969598140e+001, -3.355667198759821e+000, 1.015197946286714e+000, 5.507306634032858e-001}, {1.696986640316189e+001, -3.258213021837012e+000, 1.003618106544807e+000, 5.524960324615093e-001}, {1.621328376128235e+001, -3.163957875125649e+000, 9.922314902636222e-001, 5.542634409371333e-001}, {1.549330911573373e+001, -3.072784521087606e+000, 9.810341822990482e-001, 5.560329035300551e-001}, {1.480803983183346e+001, -2.984580503003917e+000, 9.700223620937467e-001, 5.578044351316714e-001}, {1.415567829505791e+001, -2.899237854035364e+000, 9.591923008786090e-001, 5.595780508284809e-001}, {1.353452476308944e+001, -2.816652899819644e+000, 9.485403592247703e-001, 5.613537659057762e-001}, {1.294297073501526e+001, -2.736726076361296e+000, 9.380629846710600e-001, 5.631315958514251e-001}, {1.237950327534221e+001, -2.659361822105834e+000, 9.277567094758138e-001, 5.649115563597501e-001}, {1.184268567697761e+001, -2.584468285143181e+000, 9.176181481050070e-001, 5.666936633355001e-001}, {1.133116273830062e+001, -2.511957258765677e+000, 9.076439952813108e-001, 5.684779328979236e-001}, {1.084365039059536e+001, -2.441743973937185e+000, 8.978310237189942e-001, 5.702643813849461e-001}, {1.037893959350994e+001, -2.373747016076154e+000, 8.881760821711557e-001, 5.720530253574520e-001}, {9.935878258780905e+000, -2.307888066176576e+000, 8.786760932029253e-001, 5.738438816036776e-001}, {9.513385220488097e+000, -2.244091925599429e+000, 8.693280515875038e-001, 5.756369671437167e-001}, {9.110429465384806e+000, -2.182286226388107e+000, 8.601290220482226e-001, 5.774322992341427e-001}, {8.726039979294475e+000, -2.122401439791754e+000, 8.510761377549098e-001, 5.792298953727533e-001}, {8.359296620034248e+000, -2.064370690628830e+000, 8.421665983411981e-001, 5.810297733034353e-001}, {8.009325561958333e+000, -2.008129640143733e+000, 8.333976682171725e-001, 5.828319510211679e-001}, {7.675302688422919e+000, -1.953616435842042e+000, 8.247666750114966e-001, 5.846364467771462e-001}, {7.356443958617260e+000, -1.900771540092228e+000, 8.162710077803924e-001, 5.864432790840537e-001}, {7.052008531129983e+000, -1.849537692088942e+000, 8.079081155885138e-001, 5.882524667214695e-001}, {6.761294720818636e+000, -1.799859789804134e+000, 7.996755058885832e-001, 5.900640287414285e-001}, {6.483636498372520e+000, -1.751684788990636e+000, 7.915707430302340e-001, 5.918779844741284e-001}, {6.218399337511639e+000, -1.704961609984714e+000, 7.835914468426860e-001, 5.936943535337977e-001}, {5.964989501232742e+000, -1.659641156314279e+000, 7.757352914082837e-001, 5.955131558247273e-001}, {5.722834924601525e+000, -1.615676059262124e+000, 7.680000032840613e-001, 5.973344115474654e-001}, {5.491399638620130e+000, -1.573020782408741e+000, 7.603833606797970e-001, 5.991581412051937e-001}, {5.270171046295689e+000, -1.531631422179552e+000, 7.528831917957080e-001, 6.009843656102799e-001}, {5.058664298813443e+000, -1.491465712692528e+000, 7.454973738143860e-001, 6.028131058910203e-001}, {4.856420435950218e+000, -1.452482946378775e+000, 7.382238316110306e-001, 6.046443834985716e-001}, {4.663000885144658e+000, -1.414643871536506e+000, 7.310605365115268e-001, 6.064782202140927e-001}, {4.477991405897813e+000, -1.377910693546225e+000, 7.240055053241036e-001, 6.083146381560854e-001}, {4.300998038136363e+000, -1.342246980204123e+000, 7.170567991024387e-001, 6.101536597879579e-001}, {4.131646836878849e+000, -1.307617618348851e+000, 7.102125221204546e-001, 6.119953079258070e-001}, {3.969581617344602e+000, -1.273988750409006e+000, 7.034708208117358e-001, 6.138396057464396e-001}, {3.814464999241590e+000, -1.241327747944722e+000, 6.968298828181920e-001, 6.156865767956317e-001}, {3.665976419552412e+000, -1.209603147585341e+000, 6.902879359474882e-001, 6.175362449966364e-001}, {3.523809639869346e+000, -1.178784591650163e+000, 6.838432472111192e-001, 6.193886346589591e-001}, {3.387676101076204e+000, -1.148842831128321e+000, 6.774941220149650e-001, 6.212437704873943e-001}, {3.257298408715396e+000, -1.119749615703440e+000, 6.712389030704746e-001, 6.231016775913520e-001}, {3.132416234122967e+000, -1.091477733912388e+000, 6.650759697593417e-001, 6.249623814944699e-001}, {3.012777923461127e+000, -1.064000880818915e+000, 6.590037370124431e-001, 6.268259081445352e-001}, {2.898148500487827e+000, -1.037293729191567e+000, 6.530206548146291e-001, 6.286922839237189e-001}, {2.788300228906221e+000, -1.011331780554530e+000, 6.471252070509176e-001, 6.305615356591386e-001}, {2.683019744175228e+000, -9.860914343814429e-001, 6.413159110771507e-001, 6.324336906337702e-001}, {2.582101558543176e+000, -9.615498742641666e-001, 6.355913166870878e-001, 6.343087765977056e-001}, {2.485351726258790e+000, -9.376850950754241e-001, 6.299500055923686e-001, 6.361868217797954e-001}, {2.392584780774969e+000, -9.144758355198807e-001, 6.243905905769184e-001, 6.380678548996677e-001}, {2.303623809125252e+000, -8.919015614137316e-001, 6.189117148779868e-001, 6.399519051801563e-001}, {2.218301716484065e+000, -8.699424569246335e-001, 6.135120515563689e-001, 6.418390023601502e-001}, {2.136457313721100e+000, -8.485793527805717e-001, 6.081903027200146e-001, 6.437291767078821e-001}, {2.057939227794259e+000, -8.277937602603399e-001, 6.029451990970640e-001, 6.456224590346728e-001}, {1.982601361773348e+000, -8.075677868124656e-001, 5.977754992194658e-001, 6.475188807091614e-001}, {1.910305422321021e+000, -7.878841579305617e-001, 5.926799890174284e-001, 6.494184736720295e-001}, {1.840919973864347e+000, -7.687261800830212e-001, 5.876574811656280e-001, 6.513212704512502e-001}, {1.774318431447832e+000, -7.500776989994216e-001, 5.827068144958507e-001, 6.532273041778850e-001}, {1.710381605636034e+000, -7.319231195156424e-001, 5.778268536008011e-001, 6.551366086024576e-001}, {1.648994688751223e+000, -7.142473440233981e-001, 5.730164881656592e-001, 6.570492181119113e-001}, {1.590048725123274e+000, -6.970357821918966e-001, 5.682746325952985e-001, 6.589651677472103e-001}, {1.533439874027392e+000, -6.802743224911976e-001, 5.636002254787615e-001, 6.608844932215886e-001}, {1.479068697158344e+000, -6.639493087403419e-001, 5.589922291190146e-001, 6.628072309394806e-001}, {1.426840869536403e+000, -6.480475369725340e-001, 5.544496291369960e-001, 6.647334180161881e-001}, {1.376665618585289e+000, -6.325562192444207e-001, 5.499714339749598e-001, 6.666630922982865e-001}, {1.328457187000894e+000, -6.174629937261502e-001, 5.455566745780982e-001, 6.685962923848331e-001}, {1.282133030214725e+000, -6.027558831242498e-001, 5.412044038917494e-001, 6.705330576494062e-001}, {1.237614927340746e+000, -5.884233016021722e-001, 5.369136965737760e-001, 6.724734282630155e-001}, {1.194827579250900e+000, -5.744540212110448e-001, 5.326836485547319e-001, 6.744174452179302e-001}, {1.153699645170108e+000, -5.608371784476705e-001, 5.285133767701128e-001, 6.763651503524762e-001}, {1.114162811162155e+000, -5.475622466470426e-001, 5.244020187578001e-001, 6.783165863768431e-001}, {1.076151709993373e+000, -5.346190261721666e-001, 5.203487323673905e-001, 6.802717968999631e-001}, {1.039604087616986e+000, -5.219976374666073e-001, 5.163526954775388e-001, 6.822308264575041e-001}, {1.004460456200703e+000, -5.096885048943938e-001, 5.124131056874093e-001, 6.841937205410585e-001}, {9.706638785677234e-001, -4.976823447415940e-001, 5.085291800511675e-001, 6.861605256285703e-001}, {9.381600986563852e-001, -4.859701585652561e-001, 5.047001548417265e-001, 6.881312892160738e-001}, {9.068972242668986e-001, -4.745432188080312e-001, 5.009252852952014e-001, 6.901060598508284e-001}, {8.768256265898535e-001, -4.633930593762060e-001, 4.972038453990545e-001, 6.920848871659001e-001}, {8.478980422480926e-001, -4.525114688501092e-001, 4.935351276972386e-001, 6.940678219163016e-001}, {8.200690560722610e-001, -4.418904741373326e-001, 4.899184430790607e-001, 6.960549160167504e-001}, {7.932955365213841e-001, -4.315223407060223e-001, 4.863531206494093e-001, 6.980462225811568e-001}, {7.675361442985631e-001, -4.213995555141402e-001, 4.828385075364002e-001, 7.000417959639271e-001}, {7.427512593575009e-001, -4.115148200888941e-001, 4.793739687774626e-001, 7.020416918031972e-001}, {7.189033117426465e-001, -4.018610486495977e-001, 4.759588872229961e-001, 7.040459670661056e-001}, {6.959561135742333e-001, -3.924313488754276e-001, 4.725926633925903e-001, 7.060546800962171e-001}, {6.738752033854085e-001, -3.832190229838893e-001, 4.692747154481086e-001, 7.080678906632436e-001}, {6.526277468276821e-001, -3.742175581148609e-001, 4.660044791174469e-001, 7.100856600151843e-001}, {6.321821747106798e-001, -3.654206137379463e-001, 4.627814076433050e-001, 7.121080509330365e-001}, {6.125084635885677e-001, -3.568220214675603e-001, 4.596049718063618e-001, 7.141351277882394e-001}, {5.935779282079696e-001, -3.484157741858985e-001, 4.564746599098449e-001, 7.161669566030067e-001}, {5.753631542423251e-001, -3.401960190266252e-001, 4.533899778172436e-001, 7.182036051137486e-001}, {5.578379080634269e-001, -3.321570502209607e-001, 4.503504490128227e-001, 7.202451428377567e-001}, {5.409772378490252e-001, -3.242933053050536e-001, 4.473556146977857e-001, 7.222916411433742e-001}, {5.247573236514336e-001, -3.165993558732946e-001, 4.444050338858265e-001, 7.243431733238683e-001}, {5.091553583120583e-001, -3.090699001449694e-001, 4.414982835460874e-001, 7.263998146752452e-001}, {4.941496650179276e-001, -3.016997600084108e-001, 4.386349587914129e-001, 7.284616425782706e-001}, {4.797195893441171e-001, -2.944838727153321e-001, 4.358146730674687e-001, 7.305287365849668e-001}, {4.658454159189164e-001, -2.874172838318425e-001, 4.330370583894325e-001, 7.326011785098984e-001}, {4.525084286368681e-001, -2.804951429879555e-001, 4.303017656275229e-001, 7.346790525265499e-001}, {4.396907169839087e-001, -2.737126945641282e-001, 4.276084648056008e-001, 7.367624452691657e-001}, {4.273754112892716e-001, -2.670652769945731e-001, 4.249568454808719e-001, 7.388514459404123e-001}, {4.155464136560620e-001, -2.605483107212597e-001, 4.223466171076617e-001, 7.409461464252765e-001}, {4.041884380013781e-001, -2.541572941142669e-001, 4.197775095020359e-001, 7.430466414116397e-001}, {3.932870578905699e-001, -2.478877986781748e-001, 4.172492733426532e-001, 7.451530285179989e-001}, {3.828286467935785e-001, -2.417354613025955e-001, 4.147616807084972e-001, 7.472654084288583e-001}, {3.728003160347851e-001, -2.356959769333881e-001, 4.123145256850712e-001, 7.493838850383399e-001}, {3.631899496533657e-001, -2.297650933614912e-001, 4.099076250447146e-001, 7.515085656026301e-001}, {3.539861802648743e-001, -2.239386040074048e-001, 4.075408189796285e-001, 7.536395609019181e-001}, {3.451784395377778e-001, -2.182123418981641e-001, 4.052139719155342e-001, 7.557769854125394e-001}, {3.367567308090195e-001, -2.125821679594597e-001, 4.029269733841373e-001, 7.579209574901114e-001}, {3.287119984291083e-001, -2.070439719882496e-001, 4.006797390434232e-001, 7.600715995645146e-001}, {3.210357123305466e-001, -2.015936547953648e-001, 3.984722116891958e-001, 7.622290383476353e-001}, {3.137202183461909e-001, -1.962271288064989e-001, 3.963043624830746e-001, 7.643934050549112e-001}, {3.067585099743523e-001, -1.909403032422600e-001, 3.941761921936149e-001, 7.665648356417625e-001}, {3.001443621673701e-001, -1.857290784658068e-001, 3.920877326274295e-001, 7.687434710561505e-001}, {2.938722968103650e-001, -1.805893348436715e-001, 3.900390481551737e-001, 7.709294575085852e-001}, {2.879376064535850e-001, -1.755169227244073e-001, 3.880302373994634e-001, 7.731229467610636e-001}, {2.823363799596992e-001, -1.705076514427414e-001, 3.860614350793771e-001, 7.753240964365381e-001}, {2.770447885477484e-001, -1.655569164888013e-001, 3.841328126374664e-001, 7.775330703507134e-001}, {2.720819561369693e-001, -1.606606552857484e-001, 3.822445839373179e-001, 7.797500388681229e-001}, {2.674653113933975e-001, -1.558148244688060e-001, 3.803970049600344e-001, 7.819751792846662e-001}, {2.632075572084795e-001, -1.509385793371179e-001, 3.785610382540837e-001, 7.842454607100902e-001}, {2.593330708233911e-001, -1.461744656606448e-001, 3.767934811627194e-001, 7.864914180935034e-001}, {2.558290465748489e-001, -1.415248268445518e-001, 3.750961409463570e-001, 7.887085704890959e-001}, {2.526817192496980e-001, -1.369820489145218e-001, 3.734669282636506e-001, 7.908973098182515e-001}, {2.498786057610260e-001, -1.325389045601303e-001, 3.719038491266883e-001, 7.930580228550742e-001}, {2.474085267751381e-001, -1.281885279708624e-001, 3.704049999203535e-001, 7.951910912877180e-001}, {2.452614346122534e-001, -1.239243863767887e-001, 3.689685626896172e-001, 7.972968917785244e-001}, {2.434284168742104e-001, -1.197402577787785e-001, 3.675928007377211e-001, 7.993757960231415e-001}, {2.419015528622072e-001, -1.156302067314484e-001, 3.662760544672607e-001, 8.014281708086085e-001}, {2.406739735415517e-001, -1.115885662953330e-001, 3.650167374937393e-001, 8.034543780705155e-001}, {2.397396425750860e-001, -1.076099145903805e-001, 3.638133329505207e-001, 8.054547749492954e-001}, {2.390934524095718e-001, -1.036890607294186e-001, 3.626643900605662e-001, 8.074297138456664e-001}, {2.387311440820192e-001, -9.982102656507678e-002, 3.615685208685295e-001, 8.093795424753392e-001}, {2.386492238235064e-001, -9.600103014473647e-002, 3.605243971745443e-001, 8.113046039230223e-001}, {2.388449516089252e-001, -9.222447197204525e-002, 3.595307476540946e-001, 8.132052366957213e-001}, {2.393163909364370e-001, -8.848692277977732e-002, 3.585863551396568e-001, 8.150817747754554e-001}, {2.400622369503420e-001, -8.478410739725760e-002, 3.576900540368518e-001, 8.169345476714047e-001}, {2.410819093473880e-001, -8.111189587840884e-002, 3.568407279242882e-001, 8.187638804714793e-001}, {2.423755227229712e-001, -7.746629151231736e-002, 3.560373072597538e-001, 8.205700938934316e-001}, {2.439437903097885e-001, -7.384341869908345e-002, 3.552787672180464e-001, 8.223535043354756e-001}, {2.457880841300521e-001, -7.023951497682976e-002, 3.545641256673672e-001, 8.241144239264422e-001}, {2.479104628840352e-001, -6.665092199321671e-002, 3.538924412424807e-001, 8.258531605755627e-001}, {2.503135014091841e-001, -6.307407345043224e-002, 3.532628115142414e-001, 8.275700180218225e-001}, {2.530004736674350e-001, -5.950549117251455e-002, 3.526743712965212e-001, 8.292652958829614e-001}, {2.559752230366451e-001, -5.594177412647441e-002, 3.521262910014856e-001, 8.309392897040731e-001}, {2.592422310622398e-001, -5.237959267551613e-002, 3.516177751155357e-001, 8.325922910059250e-001}, {2.628065531267236e-001, -4.881568009199424e-002, 3.511480607379117e-001, 8.342245873328724e-001}, {2.666738567720074e-001, -4.524682709852055e-002, 3.507164162151467e-001, 8.358364623005091e-001}, {2.708504884539047e-001, -4.166987608244501e-002, 3.503221398361055e-001, 8.374281956429612e-001}, {2.753433071879429e-001, -3.808171199029962e-002, 3.499645585884929e-001, 8.390000632598814e-001}, {2.801599037196582e-001, -3.447926077765695e-002, 3.496430270117127e-001, 8.405523372631366e-001}, {2.853083921105367e-001, -3.085947960555782e-002, 3.493569260683962e-001, 8.420852860231959e-001}, {2.907976806313661e-001, -2.721935651026936e-002, 3.491056621189848e-001, 8.435991742151832e-001}, {2.966371720783940e-001, -2.355589928273860e-002, 3.488886658983132e-001, 8.450942628646833e-001}, {3.028370462889118e-001, -1.986613619310260e-002, 3.487053916015641e-001, 8.465708093931487e-001}, {3.094081917821564e-001, -1.614710890028537e-002, 3.485553159714244e-001, 8.480290676631032e-001}, {3.163620364755382e-001, -1.239586542273052e-002, 3.484379374449686e-001, 8.494692880229420e-001}, {3.237109529050010e-001, -8.609462422221065e-003, 3.483527753712113e-001, 8.508917173514538e-001}, {3.314679191860245e-001, -4.784954014192033e-003, 3.482993692124245e-001, 8.522965991020172e-001}, {3.396466627040896e-001, -9.193907609769285e-004, 3.482772778363755e-001, 8.536841733464595e-001}, {3.482573639782674e-001, 2.990234168183047e-003, 3.482860786936271e-001, 8.550546768185773e-001}, {3.573435870189764e-001, 6.946512750374164e-003, 3.483253684425491e-001, 8.564083429573437e-001}, {3.668456015562940e-001, 1.095357014539229e-002, 3.483947564910607e-001, 8.577454019497183e-001}, {3.768666032330697e-001, 1.501283216327796e-002, 3.484938791513281e-001, 8.590660807729463e-001}, {3.872502858387614e-001, 1.912964792928498e-002, 3.486223735021649e-001, 8.603706032376969e-001}, {3.978260416786913e-001, 2.330753852682181e-002, 3.487799024495905e-001, 8.616591900276418e-001}, {4.132101935168742e-001, 2.749646402204903e-002, 3.489662637387618e-001, 8.629320587422359e-001}, {4.184545943199193e-001, 3.190595255298934e-002, 3.491805658203261e-001, 8.641894239321634e-001}, {4.362824413649418e-001, 3.618297429472679e-002, 3.494237843557785e-001, 8.654314971271906e-001}, {4.482605603718436e-001, 4.065324239822417e-002, 3.496942108258070e-001, 8.666584870394793e-001}, {4.622330316257493e-001, 4.516985999042349e-002, 3.499923862104077e-001, 8.678705992003968e-001}, {4.768622286036364e-001, 4.976455970008947e-002, 3.503178747560284e-001, 8.690680364449238e-001}, {4.921745241725010e-001, 5.444066904880778e-002, 3.506704352553635e-001, 8.702509987148481e-001}, {5.081973839874943e-001, 5.920156908963686e-002, 3.510498387111092e-001, 8.714196831463595e-001}, {5.249595720422131e-001, 6.405069424380409e-002, 3.514558679881682e-001, 8.725742841082431e-001}, {5.424905347559086e-001, 6.899154202103656e-002, 3.518883174461503e-001, 8.737149932396073e-001}, {5.608210429624480e-001, 7.402766394847896e-002, 3.523469926362302e-001, 8.748419994872339e-001}, {5.799822127345404e-001, 7.916267804090524e-002, 3.528317099190555e-001, 8.759554891424319e-001}, {6.004830519290501e-001, 8.439587774540482e-002, 3.533423051135982e-001, 8.770556458774835e-001}, {6.225303530662254e-001, 8.972780116427569e-002, 3.538786230951095e-001, 8.781426507816023e-001}, {6.463610750175416e-001, 9.515751995040138e-002, 3.544405351945316e-001, 8.792166823964106e-001}, {6.722471844825647e-001, 1.012426405366949e-001, 3.550840958618995e-001, 8.803766342353709e-001}, {7.038651077223995e-001, 1.075468486721946e-001, 3.557708387936919e-001, 8.815427655306450e-001}, {7.251393379424058e-001, 1.142877363408631e-001, 3.565023423314873e-001, 8.827152564770329e-001}, {7.642524395207234e-001, 1.210851585997144e-001, 3.572820846551236e-001, 8.838942963354349e-001}, {7.967358094127701e-001, 1.283613467605374e-001, 3.581108233391086e-001, 8.850800842450453e-001}, {8.338699793357391e-001, 1.359403244468062e-001, 3.589921197256534e-001, 8.862728294157951e-001}, {8.739157757386329e-001, 1.438882241405927e-001, 3.599284806768164e-001, 8.874727525584538e-001}, {9.171674240348069e-001, 1.522338614296611e-001, 3.609228706786188e-001, 8.886800863594736e-001}, {9.639566900146391e-001, 1.610089362976866e-001, 3.619784903446237e-001, 8.898950765022780e-001}, {1.014658245902457e+000, 1.702483983739543e-001, 3.630988007319633e-001, 8.911179826999973e-001}, {1.069696500740114e+000, 1.799908580537642e-001, 3.642875508037436e-001, 8.923490798353885e-001}, {1.129553586989754e+000, 1.902790629163924e-001, 3.655488084631469e-001, 8.935886592217480e-001}, {1.194778730658964e+000, 2.011604495101677e-001, 3.668869957374470e-001, 8.948370300007601e-001}, {1.265999604715109e+000, 2.126877807800234e-001, 3.683069287846877e-001, 8.960945206957666e-001}, {1.343935603653501e+000, 2.249198920053187e-001, 3.698138634914989e-001, 8.973614809419384e-001}, {1.429414190654427e+000, 2.379225551311867e-001, 3.714135476414637e-001, 8.986382834183703e-001}, {1.523389963185659e+000, 2.517694999845817e-001, 3.731122807290428e-001, 8.999253260114501e-001}, {1.626967987465184e+000, 2.665436073713666e-001, 3.749169828303232e-001, 9.012230342439003e-001}, {1.741431587530111e+000, 2.823383211013446e-001, 3.768352741049624e-001, 9.025318640101248e-001}, {1.868275831068914e+000, 2.992593167802703e-001, 3.788755669150111e-001, 9.038523046659105e-001}, {2.009247253435497e+000, 3.174264875499637e-001, 3.810471728982508e-001, 9.051848825296176e-001}, {2.166391087168643e+000, 3.369763061640536e-001, 3.833604278633734e-001, 9.065301648630008e-001}, {2.349101814944340e+000, 3.579901484020978e-001, 3.858270141057033e-001, 9.078887644133517e-001}, {2.531929749610997e+000, 3.810226519207810e-001, 3.884588003903566e-001, 9.092613446151097e-001}, {2.767188003052499e+000, 4.055286490868840e-001, 3.912727602985009e-001, 9.106486255694756e-001}, {3.016217450507473e+000, 4.324970058988013e-001, 3.942818356613025e-001, 9.120513915273417e-001}, {3.303389261208595e+000, 4.618306962042632e-001, 3.975063679209401e-001, 9.134704971443424e-001}, {3.631549916126941e+000, 4.939281587987267e-001, 4.009666241813926e-001, 9.149068790734376e-001}, {4.008482779288279e+000, 5.291694395449363e-001, 4.046862435629703e-001, 9.163615656490546e-001}, {4.443846750534742e+000, 5.680030693371376e-001, 4.086922722032931e-001, 9.178356901063999e-001}, {4.949731196871531e+000, 6.109615086616571e-001, 4.130158106084739e-001, 9.193305059167702e-001}, {5.541409643335976e+000, 6.586807872014066e-001, 4.176928163006187e-001, 9.208474050863027e-001}, {6.238372675901206e+000, 7.119257358914435e-001, 4.227651072505875e-001, 9.223879401412338e-001}, {7.065766386999059e+000, 7.716226265930281e-001, 4.282816280479594e-001, 9.239538507389419e-001}, {8.056416694649682e+000, 8.389018864217007e-001, 4.343000631122791e-001, 9.255470961380512e-001}, {9.253723462876144e+000, 9.151544772853499e-001, 4.408889145566523e-001, 9.271698951653059e-001}, {1.071584561649215e+001, 1.002107267105648e+000, 4.481302092757962e-001, 9.288247758812972e-001}, {1.253214033900518e+001, 1.101808734948876e+000, 4.561233615389971e-001, 9.305146379456196e-001}, {1.477897062418504e+001, 1.217485995326902e+000, 4.649880047213420e-001, 9.322428318295400e-001}, {1.766908561621815e+001, 1.351719349121241e+000, 4.748770243380275e-001, 9.340132606989513e-001}, {2.138486266524388e+001, 1.509883643514754e+000, 4.859733964211321e-001, 9.358305148113010e-001}, {2.628754637519723e+001, 1.697451202895145e+000, 4.985169448163012e-001, 9.377000442304936e-001}, {3.289856626810693e+001, 1.922256316265470e+000, 5.128135873286412e-001, 9.396284013530487e-001}, {4.205492727375339e+001, 2.194581754116503e+000, 5.292667488868058e-001, 9.416235666436306e-001}, {5.514876558372439e+001, 2.528063775283584e+000, 5.484198767596652e-001, 9.436954119389488e-001}, {7.462633427723064e+001, 2.940368260878233e+000, 5.710261164676607e-001, 9.458563710445010e-001}, {1.050184348991562e+002, 3.453462949853480e+000, 5.981648072853771e-001, 9.481224427923183e-001}, {1.555495965673860e+002, 4.087350044552481e+000, 6.314608944041468e-001, 9.505147490704314e-001}, {2.376339218745235e+002, 4.944818621598538e+000, 6.732270162071558e-001, 9.530620700536182e-001}, {3.778931105604901e+002, 6.223871768745157e+000, 7.270999546100488e-001, 9.558052023498382e-001}, {6.392510022959895e+002, 8.369904048358965e+000, 7.988184626507228e-001, 9.588050299738381e-001}, {1.175214125741375e+003, 9.200515050596998e+000, 8.756729054310770e-001, 9.612797327158309e-001}, {2.404391521144330e+003, 8.351201828031256e+000, 9.861434004464017e-001, 9.640175305607321e-001}, {5.280178052685498e+003, 7.052137331238425e+000, 1.155225153751228e+000, 9.672632864915536e-001}, {1.195185547887987e+004, 1.436266930835130e+001, 1.399691464502437e+000, 9.709593911907505e-001}, {2.738163303986884e+004, 3.128192142643308e+001, 1.713086045447508e+000, 9.743575124934957e-001}, {6.813367902015221e+004, 2.245954110359909e+001, 2.098339086431259e+000, 9.771334786926074e-001}, {1.719549070722432e+005, -4.567581958920449e+001, 2.615376615379705e+000, 9.796670698052391e-001}, {4.072927452321919e+005, -5.632241381576800e+001, 3.317614691567385e+000, 9.823925310148151e-001}, {9.174334612404156e+005, 5.978559879130236e+002, 4.355002072346393e+000, 9.857167565271496e-001}, {1.863229120547296e+006, 1.146180212440694e+003, 5.277266586606779e+000, 9.875311446084982e-001}, {4.352587282561689e+006, 3.435290432960140e+002, 6.013725056438802e+000, 9.883772225822374e-001}, {1.294720023425507e+007, -1.952316825630777e+003, 7.598845776825704e+000, 9.898384444052550e-001}, {4.368922580067658e+007, -9.798242269889981e+003, 1.040268251220144e+001, 9.914339076346728e-001}, {1.412163489780696e+008, -7.929973860030538e+003, 1.348982180175535e+001, 9.935448201066974e-001}, {3.054619794272863e+008, 1.543075051399528e+004, 1.697364990930417e+001, 9.950069147908655e-001}, {2.513842293016319e+012, -1.192762798969687e+008, 1.413409677685541e+003, 9.959656981502529e-001}};
        constant Real[Ninterval, 4] hvcoef = {{3.418719024828741e+009, -6.021500887201227e+005, 1.372239895761628e+002, 8.595917694423992e-001}, {3.054310565670096e+009, -5.705938691014666e+005, 1.340221742560438e+002, 8.599600896018758e-001}, {2.680326377990845e+009, -5.429463074242631e+005, 1.313866487967003e+002, 8.602696780537780e-001}, {2.303779913727746e+009, -4.921392562770898e+005, 1.256410111646926e+002, 8.609751352703259e-001}, {1.980129780009944e+009, -4.460960668130493e+005, 1.201498857905039e+002, 8.616864231415984e-001}, {1.702029460983321e+009, -4.043727836958034e+005, 1.149020405579111e+002, 8.624035880113733e-001}, {1.463001841507740e+009, -3.665601613605881e+005, 1.098863963717011e+002, 8.631267209320035e-001}, {1.257578888846852e+009, -3.322921639231642e+005, 1.050926221990266e+002, 8.638558834346060e-001}, {1.081020151317802e+009, -3.012351730865443e+005, 1.005107466086200e+002, 8.645911520991180e-001}, {9.292788756444820e+008, -2.730884033777993e+005, 9.613135173247430e+001, 8.653325887404106e-001}, {7.988592072529383e+008, -2.475781452734592e+005, 9.194535249523334e+001, 8.660802702489665e-001}, {6.867517994789405e+008, -2.244567953084133e+005, 8.794416126688296e+001, 8.668342586830016e-001}, {5.903993090097249e+008, -2.035004093727184e+005, 8.411943557310387e+001, 8.675946462312558e-001}, {5.075739967569540e+008, -1.845052128758624e+005, 8.046328280923146e+001, 8.683615102740704e-001}, {4.363803981359935e+008, -1.672877964071615e+005, 7.696824858217629e+001, 8.691349132544352e-001}, {3.751811369685484e+008, -1.516810195389626e+005, 7.362707753938845e+001, 8.699149478489386e-001}, {3.225757764446142e+008, -1.375342639486588e+005, 7.043297686856677e+001, 8.707016766933142e-001}, {2.775632019164234e+008, -1.247180148068888e+005, 6.737939987231324e+001, 8.714951927224426e-001}, {2.395047937120040e+008, -1.131276569986563e+005, 6.446029045026621e+001, 8.722955738779207e-001}, {2.072894646248908e+008, -1.026516293164950e+005, 6.166981089182345e+001, 8.731028981691641e-001}, {1.799822430252400e+008, -9.333464537306117e+004, 5.905402123864285e+001, 8.739009311042776e-001}, {1.566914275764715e+008, -8.525624087305536e+004, 5.668205775153147e+001, 8.746623083398238e-001}, {1.365420998913238e+008, -7.792553082576592e+004, 5.442145459347433e+001, 8.754247647693702e-001}, {1.190945653358361e+008, -7.126911585540987e+004, 5.226630523340383e+001, 8.761882924491433e-001}, {1.039717710388903e+008, -6.522111942123056e+004, 5.021104820325436e+001, 8.769528833328070e-001}, {9.085238634246235e+007, -5.972253866425365e+004, 4.825044784911309e+001, 8.777185292709222e-001}, {7.945976285392952e+007, -5.472031010955200e+004, 4.637957044109646e+001, 8.784852220104472e-001}, {6.955828718931271e+007, -5.016689134145316e+004, 4.459376897054184e+001, 8.792529531942909e-001}, {6.094432939926156e+007, -4.601943690028419e+004, 4.288866070740672e+001, 8.800217143609221e-001}, {5.344423567813414e+007, -4.223952028854783e+004, 4.126011513927500e+001, 8.807914969440196e-001}, {4.690790693157751e+007, -3.879248539780872e+004, 3.970423429382681e+001, 8.815622922721852e-001}, {4.120636628965157e+007, -3.564715093489797e+004, 3.821734102078233e+001, 8.823340915687035e-001}, {3.622900976104898e+007, -3.277545861657008e+004, 3.679596497500520e+001, 8.831068859513512e-001}, {3.187967036644944e+007, -3.015203386420734e+004, 3.543682780067873e+001, 8.838806664322682e-001}, {2.807613578829654e+007, -2.775406516810798e+004, 3.413683515396600e+001, 8.846554239178743e-001}, {2.474692036340034e+007, -2.556090563904067e+004, 3.289306188961926e+001, 8.854311492088450e-001}, {2.183049697968674e+007, -2.355393699418945e+004, 3.170274439968934e+001, 8.862078330001338e-001}, {1.927347019859899e+007, -2.171631328406215e+004, 3.056326941876671e+001, 8.869854658810615e-001}, {1.702981751333125e+007, -2.003282321642497e+004, 2.947216618883089e+001, 8.877640383354423e-001}, {1.505930006594837e+007, -1.848966312427858e+004, 2.842709641996708e+001, 8.885435407417774e-001}, {1.332750187738691e+007, -1.707439527076683e+004, 2.742584914811091e+001, 8.893239633734973e-001}, {1.180409908173095e+007, -1.577569115143173e+004, 2.646632952888870e+001, 8.901052963992563e-001}, {1.046300091887967e+007, -1.458332162795913e+004, 2.554655563707482e+001, 8.908875298832820e-001}, {9.281478450960947e+006, -1.348800001880907e+004, 2.466464976005302e+001, 8.916706537857801e-001}, {8.239677365327914e+006, -1.248128939600677e+004, 2.381883271013455e+001, 8.924546579633877e-001}, {7.320375014028021e+006, -1.155554134563862e+004, 2.300741887638321e+001, 8.932395321696804e-001}, {6.508564560677815e+006, -1.070380653674440e+004, 2.222881017899560e+001, 8.940252660557324e-001}, {5.791092789698311e+006, -9.919758901292089e+003, 2.148149097099744e+001, 8.948118491707285e-001}, {5.156530698970656e+006, -9.197655328178946e+003, 2.076402412172987e+001, 8.955992709626238e-001}, {4.594876945645325e+006, -8.532266312855607e+003, 2.007504587669628e+001, 8.963875207788541e-001}, {4.097381930359949e+006, -7.918830001821591e+003, 1.941326196319786e+001, 8.971765878670993e-001}, {3.656383109691083e+006, -7.353008221313214e+003, 1.877744374229622e+001, 8.979664613760915e-001}, {3.265186685897589e+006, -6.830850362101170e+003, 1.816642466513669e+001, 8.987571303564701e-001}, {2.917909279006009e+006, -6.348750551159110e+003, 1.757909652930962e+001, 8.995485837616878e-001}, {2.609402060809861e+006, -5.903421540440110e+003, 1.701440658291143e+001, 9.003408104489590e-001}, {2.335140768578981e+006, -5.491861404071661e+003, 1.647135426048376e+001, 9.011337991802549e-001}, {2.091146761265873e+006, -5.111327878744832e+003, 1.594898842221833e+001, 9.019275386233419e-001}, {1.873929941363969e+006, -4.759317050348291e+003, 1.544640477086862e+001, 9.027220173528617e-001}, {1.680420118072653e+006, -4.433539445570736e+003, 1.496274316343266e+001, 9.035172238514534e-001}, {1.507901694720669e+006, -4.131898344129412e+003, 1.449718519666387e+001, 9.043131465109192e-001}, {1.354001888305344e+006, -3.852479913913011e+003, 1.404895235600572e+001, 9.051097736334237e-001}, {1.216613076782086e+006, -3.593527497504629e+003, 1.361730330460103e+001, 9.059070934327353e-001}, {1.093879091249427e+006, -3.353433477948261e+003, 1.320153238778303e+001, 9.067050940355028e-001}, {9.841701033319680e+005, -3.130726664519008e+003, 1.280096767078483e+001, 9.075037634825668e-001}, {8.860331097677258e+005, -2.924054367261154e+003, 1.241496883316571e+001, 9.083030897303053e-001}, {7.981918624483937e+005, -2.732178555007192e+003, 1.204292601630526e+001, 9.091030606520103e-001}, {7.195149237111326e+005, -2.553962145681973e+003, 1.168425789742630e+001, 9.099036640393030e-001}, {6.490013601859985e+005, -2.388361283829961e+003, 1.133841032591765e+001, 9.107048876035626e-001}, {5.857619638546082e+005, -2.234416399557152e+003, 1.100485487724143e+001, 9.115067189773976e-001}, {5.290116852906891e+005, -2.091246746576212e+003, 1.068308765988832e+001, 9.123091457161393e-001}, {4.780525034495146e+005, -1.958041849488108e+003, 1.037262790050762e+001, 9.131121552993566e-001}, {4.322639370581367e+005, -1.834056004290414e+003, 1.007301685198876e+001, 9.139157351323971e-001}, {3.910966072603547e+005, -1.718603636258115e+003, 9.783816732166672e+000, 9.147198725479478e-001}, {3.540614298472962e+005, -1.611053173940656e+003, 9.504609563667259e+000, 9.155245548076196e-001}, {3.207227948052809e+005, -1.510822716204718e+003, 9.234996229867011e+000, 9.163297691035424e-001}, {2.906939935011917e+005, -1.417376462988183e+003, 8.974595578775290e+000, 9.171355025599859e-001}, {2.636303793313987e+005, -1.330220294731111e+003, 8.723043464518558e+000, 9.179417422349862e-001}, {2.392237992914084e+005, -1.248898152561119e+003, 8.479991917683572e+000, 9.187484751219944e-001}, {2.172016592892052e+005, -1.172990137423081e+003, 8.245108450295883e+000, 9.195556881515287e-001}, {1.973183307798826e+005, -1.102107553390677e+003, 8.018075097948199e+000, 9.203633681928409e-001}, {1.793561202392878e+005, -1.035892282235062e+003, 7.798587940806567e+000, 9.211715020555885e-001}, {1.631202275964593e+005, -9.740132377691543e+002, 7.586356268239432e+000, 9.219800764915146e-001}, {1.484359488552449e+005, -9.161641833684364e+002, 7.381101972870521e+000, 9.227890781961310e-001}, {1.351478825430181e+005, -8.620624032575208e+002, 7.182559013917762e+000, 9.235984938104057e-001}, {1.231162519279247e+005, -8.114460113623057e+002, 6.990472737830404e+000, 9.244083099224488e-001}, {1.122162968415116e+005, -7.640729129202056e+002, 6.804599425001765e+000, 9.252185130692011e-001}, {1.023361278541669e+005, -7.197189192332255e+002, 6.624705719443992e+000, 9.260290897381156e-001}, {9.337545308914331e+004, -6.781764164807573e+002, 6.450568157340423e+000, 9.268400263688406e-001}, {8.524409931480192e+004, -6.392529690945879e+002, 6.281972695119674e+000, 9.276513093548937e-001}, {7.786169679505364e+004, -6.027705240567450e+002, 6.118714310110324e+000, 9.284629250453276e-001}, {7.115526319693035e+004, -5.685635173312871e+002, 5.960596482051478e+000, 9.292748597463896e-001}, {6.505997236969629e+004, -5.364788721222958e+002, 5.807430942404182e+000, 9.300870997231673e-001}, {5.951708365389362e+004, -5.063742243350591e+002, 5.659037156140247e+000, 9.308996312012252e-001}, {5.447396049317387e+004, -4.781175751106457e+002, 5.515242057235346e+000, 9.317124403682275e-001}, {4.988309687503692e+004, -4.515862663149072e+002, 5.375879663496088e+000, 9.325255133755377e-001}, {4.570187946719245e+004, -4.266664669336558e+002, 5.240790784971295e+000, 9.333388363398144e-001}, {4.189173749451424e+004, -4.032522688537788e+002, 5.109822677685524e+000, 9.341523953445763e-001}, {3.841803267326960e+004, -3.812453207114073e+002, 4.982828795533794e+000, 9.349661764417548e-001}, {3.524944619171859e+004, -3.605540968469249e+002, 4.859668478530708e+000, 9.357801656532230e-001}, {3.235775262493093e+004, -3.410934828706502e+002, 4.740206710402045e+000, 9.365943489722998e-001}, {2.971742749441942e+004, -3.227842408832863e+002, 4.624313855555895e+000, 9.374087123652322e-001}, {2.730548973676348e+004, -3.055526590426766e+002, 4.511865434271594e+000, 9.382232417726517e-001}, {2.510105818186500e+004, -2.893300011512208e+002, 4.402741870199007e+000, 9.390379231110015e-001}, {2.308534946321419e+004, -2.740523193986549e+002, 4.296828310919137e+000, 9.398527422739368e-001}, {2.124130171607308e+004, -2.596599553908384e+002, 4.194014387044316e+000, 9.406676851337009e-001}, {1.955351648008638e+004, -2.460973335068810e+002, 4.094194041865801e+000, 9.414827375424566e-001}, {1.800800960501098e+004, -2.333125885677086e+002, 3.997265326620004e+000, 9.422978853336017e-001}, {1.659212825450676e+004, -2.212573565008916e+002, 3.903130237563751e+000, 9.431131143230387e-001}, {1.529440870224837e+004, -2.098865061302326e+002, 3.811694539213896e+000, 9.439284103104196e-001}, {1.410443891357347e+004, -1.991578904408298e+002, 3.722867600418599e+000, 9.447437590803492e-001}, {1.301277276409646e+004, -1.890321548645134e+002, 3.636562246885636e+000, 9.455591464035549e-001}, {1.201083501871745e+004, -1.794725379233024e+002, 3.552694612840763e+000, 9.463745580380161e-001}, {1.109084452413765e+004, -1.704446937374686e+002, 3.471184001267795e+000, 9.471899797300593e-001}, {1.024571699447709e+004, -1.619165024398652e+002, 3.391952746393233e+000, 9.480053972154118e-001}, {9.469018895726013e+003, -1.538579391475356e+002, 3.314926093557165e+000, 9.488207962202151e-001}, {8.754902558872502e+003, -1.462409215712277e+002, 3.240032074179658e+000, 9.496361624619982e-001}, {8.098027596251695e+003, -1.390391545991529e+002, 3.167201386524235e+000, 9.504514816506080e-001}, {7.493554482491891e+003, -1.322280542592093e+002, 3.096367297109474e+000, 9.512667394890960e-001}, {6.937064709883011e+003, -1.257845873818821e+002, 3.027465520734622e+000, 9.520819216745656e-001}, {6.424526737209824e+003, -1.196871778019022e+002, 2.960434126813456e+000, 9.528970138989733e-001}, {5.952271592786267e+003, -1.139156214553171e+002, 2.895213446237914e+000, 9.537120018498824e-001}, {5.516955702461954e+003, -1.084509856459832e+002, 2.831745975134339e+000, 9.545268712111753e-001}, {5.115516193281917e+003, -1.032755079240082e+002, 2.769976284441141e+000, 9.553416076637196e-001}, {4.745169939215222e+003, -9.837255018506627e+001, 2.709850945954599e+000, 9.561561968859864e-001}, {4.403364440818750e+003, -9.372649095365175e+001, 2.651318440509086e+000, 9.569706245546216e-001}, {4.087776994953164e+003, -8.932268622183530e+001, 2.594329091528175e+000, 9.577848763449695e-001}, {3.796275090953450e+003, -8.514737826931575e+001, 2.538834982469134e+000, 9.585989379315498e-001}, {3.526914794156184e+003, -8.118766109964749e+001, 2.484789895870575e+000, 9.594127949884909e-001}, {3.277915891545599e+003, -7.743141105727378e+001, 2.432149240012017e+000, 9.602264331899009e-001}, {3.047647797709957e+003, -7.386723790355531e+001, 2.380869986715188e+000, 9.610398382102077e-001}, {2.834618789109123e+003, -7.048444111730269e+001, 2.330910611325176e+000, 9.618529957244332e-001}, {2.637461957139128e+003, -6.727296213941031e+001, 2.282231032479775e+000, 9.626658914084311e-001}, {2.454924432505965e+003, -6.422334375285149e+001, 2.234792556533058e+000, 9.634785109390691e-001}, {2.285857047983956e+003, -6.132669141282386e+001, 2.188557823874969e+000, 9.642908399943569e-001}, {2.129208461400737e+003, -5.857464160828975e+001, 2.143490758529177e+000, 9.651028642535350e-001}, {1.984008976569308e+003, -5.595931630261283e+001, 2.099556514899462e+000, 9.659145693971051e-001}, {1.849374081739265e+003, -5.347331044763390e+001, 2.056721437955760e+000, 9.667259411068088e-001}, {1.724488078442418e+003, -5.110964503808447e+001, 2.014953009798691e+000, 9.675369650655664e-001}, {1.608601317477169e+003, -4.886174815058493e+001, 1.974219811466410e+000, 9.683476269573521e-001}, {1.501028549020432e+003, -4.672343509185431e+001, 1.934491482280281e+000, 9.691579124670265e-001}, {1.401134787382134e+003, -4.468886871440314e+001, 1.895738673701978e+000, 9.699678072801187e-001}, {1.308339375774190e+003, -4.275255490962074e+001, 1.857933019816026e+000, 9.707772970825509e-001}, {1.222108228029781e+003, -4.090931316053910e+001, 1.821047094970709e+000, 9.715863675603226e-001}, {1.141946857646252e+003, -3.915425280944660e+001, 1.785054378319671e+000, 9.723950043991366e-001}, {1.067402981642193e+003, -3.748276735761835e+001, 1.749929225855599e+000, 9.732031932839743e-001}, {9.980583833327522e+002, -3.589050668361879e+001, 1.715646831952266e+000, 9.740109198986281e-001}, {9.335284820523360e+002, -3.437336708372298e+001, 1.682183202119270e+000, 9.748181699251741e-001}, {8.734576475077868e+002, -3.292747251868510e+001, 1.649515121362005e+000, 9.756249290433984e-001}, {8.175192743315979e+002, -3.154916618621505e+001, 1.617620128283343e+000, 9.764311829301763e-001}, {7.654113989059594e+002, -3.023499246048811e+001, 1.586476484703391e+000, 9.772369172587978e-001}, {7.168549261326022e+002, -2.898168620253393e+001, 1.556063150963355e+000, 9.780421176982415e-001}, {6.715934871869085e+002, -2.778616486120676e+001, 1.526359761996525e+000, 9.788467699124019e-001}, {6.293887964147959e+002, -2.664551120825474e+001, 1.497346599661789e+000, 9.796508595592697e-001}, {5.900219253252941e+002, -2.555697044888610e+001, 1.469004573803481e+000, 9.804543722900537e-001}, {5.532898465620945e+002, -2.451793564352644e+001, 1.441315196411080e+000, 9.812572937482607e-001}, {5.190052723541189e+002, -2.352594208801040e+001, 1.414260562393762e+000, 9.820596095687246e-001}, {4.869947579083630e+002, -2.257865727178197e+001, 1.387823327724877e+000, 9.828613053765841e-001}, {4.570979184019152e+002, -2.167387417193229e+001, 1.361986690424303e+000, 9.836623667862149e-001}, {4.291663436174557e+002, -2.080950381084881e+001, 1.336734371291882e+000, 9.844627794001081e-001}, {4.030627155807368e+002, -1.998356855725961e+001, 1.312050595629924e+000, 9.852625288077083e-001}, {3.786596714281382e+002, -1.919419513223841e+001, 1.287920075411151e+000, 9.860616005841896e-001}, {3.558394376680569e+002, -1.843960978347813e+001, 1.264327993086148e+000, 9.868599802892032e-001}, {3.344927057914483e+002, -1.771813153942693e+001, 1.241259984529670e+000, 9.876576534655511e-001}, {3.145183552120479e+002, -1.702816801518539e+001, 1.218702124146350e+000, 9.884546056378373e-001}, {2.958222994740287e+002, -1.636820898671438e+001, 1.196640908911924e+000, 9.892508223110540e-001}, {2.783177026940484e+002, -1.573682393871353e+001, 1.175063245364513e+000, 9.900462889691248e-001}, {2.619234894737551e+002, -1.513265481908928e+001, 1.153956433685712e+000, 9.908409910734041e-001}, {2.465648216723021e+002, -1.455441492324349e+001, 1.133308156602596e+000, 9.916349140611203e-001}, {2.321720020070755e+002, -1.400088277850550e+001, 1.113106464537580e+000, 9.924280433437820e-001}, {2.186803377779052e+002, -1.347089951583520e+001, 1.093339764259556e+000, 9.932203643055251e-001}, {2.060297477025833e+002, -1.296336530904619e+001, 1.073996806711151e+000, 9.940118623014244e-001}, {1.941643451397283e+002, -1.247723593031160e+001, 1.055066675366082e+000, 9.948025226557459e-001}, {1.830322811700511e+002, -1.201152018223532e+001, 1.036538775470238e+000, 9.955923306601620e-001}, {1.725851592956317e+002, -1.156527611442914e+001, 1.018402822732162e+000, 9.963812715719111e-001}, {1.627781181669557e+002, -1.113760956039830e+001, 1.000648834052838e+000, 9.971693306119173e-001}, {1.535693860011035e+002, -1.072767076272634e+001, 9.832671167175504e-001, 9.979564929628543e-001}, {1.449200191421523e+002, -1.033465196230555e+001, 9.662482590600698e-001, 9.987427437671671e-001}, {1.367937717065337e+002, -9.957785484393540e+000, 9.495831215921720e-001, 9.995280681250476e-001}, {1.291569872589011e+002, -9.596341801355912e+000, 9.332628281122665e-001, 1.000312451092355e+000}, {1.219781077764371e+002, -9.249626474109293e+000, 9.172787563965046e-001, 1.001095877678499e+000}, {1.152279668782842e+002, -8.916979963771544e+000, 9.016225314369730e-001, 1.001878332844262e+000}, {1.088792020192482e+002, -8.597774082924014e+000, 8.862860157568774e-001, 1.002659801499583e+000}, {1.029063235987208e+002, -8.291411320542892e+000, 8.712613030194463e-001, 1.003440268501293e+000}, {9.728564785438825e+001, -7.997323365549272e+000, 8.565407104574884e-001, 1.004219718650792e+000}, {9.199496304063413e+001, -7.714968773139151e+000, 8.421167710251819e-001, 1.004998136691686e+000}, {8.701360359900153e+001, -7.443832377910562e+000, 8.279822274408061e-001, 1.005775507307376e+000}, {8.232230443717428e+001, -7.183423666195218e+000, 8.141300250683886e-001, 1.006551815118586e+000}, {7.790298199800353e+001, -6.933275074571325e+000, 8.005533052628054e-001, 1.007327044680851e+000}, {7.373882736821794e+001, -6.692941599684643e+000, 7.872454000252229e-001, 1.008101180481954e+000}, {6.981412051815975e+001, -6.461999122789390e+000, 7.741998252916442e-001, 1.008874206939307e+000}, {6.611412526808580e+001, -6.240043296990975e+000, 7.614102753125023e-001, 1.009646108397281e+000}, {6.262513663235084e+001, -6.026689038670938e+000, 7.488706175635427e-001, 1.010416869124489e+000}, {5.933427241821190e+001, -5.821568909235316e+000, 7.365748866372953e-001, 1.011186473311012e+000}, {5.622953202093046e+001, -5.624332795860875e+000, 7.245172797819842e-001, 1.011954905065573e+000}, {5.329965683889058e+001, -5.434646641163655e+000, 7.126921513657769e-001, 1.012722148412655e+000}, {5.053413008036412e+001, -5.252191899271087e+000, 7.010940083901317e-001, 1.013488187289570e+000}, {4.792309187394095e+001, -5.076664589645531e+000, 6.897175055906588e-001, 1.014253005543466e+000}, {4.545732719574220e+001, -4.907774732842085e+000, 6.785574411182894e-001, 1.015016586928279e+000}, {4.312818391659056e+001, -4.745245470887149e+000, 6.676087519562042e-001, 1.015778915101634e+000}, {4.092758873926038e+001, -4.588812680293274e+000, 6.568665100095136e-001, 1.016539973621682e+000}, {3.884793008615984e+001, -4.438223961945115e+000, 6.463259175911253e-001, 1.017299745943874e+000}, {3.688212100151130e+001, -4.293238562403930e+000, 6.359823041079774e-001, 1.018058215417688e+000}, {3.502348648740307e+001, -4.153626360185827e+000, 6.258311216044381e-001, 1.018815365283285e+000}, {3.326577803136620e+001, -4.019167610116154e+000, 6.158679415223102e-001, 1.019571178668104e+000}, {3.160313640436182e+001, -3.889652370808637e+000, 6.060884509889981e-001, 1.020325638583392e+000}, {3.003005307713898e+001, -3.764879973889153e+000, 5.964884493327785e-001, 1.021078727920680e+000}, {2.854137434411961e+001, -3.644658724667432e+000, 5.870638449339153e-001, 1.021830429448176e+000}, {2.713224638151363e+001, -3.528805289640701e+000, 5.778106517458664e-001, 1.022580725807109e+000}, {2.579812528568391e+001, -3.417144473611440e+000, 5.687249864435943e-001, 1.023329599507988e+000}, {2.453473414913067e+001, -3.309508692174914e+000, 5.598030651918186e-001, 1.024077032926797e+000}, {2.333805810227199e+001, -3.205737695631187e+000, 5.510412008791300e-001, 1.024823008301128e+000}, {2.220432885559139e+001, -3.105678219024390e+000, 5.424358002555373e-001, 1.025567507726222e+000}, {2.112999429523282e+001, -3.009183576146176e+000, 5.339833611149614e-001, 1.026310513150946e+000}, {2.011172854430874e+001, -2.916113494140037e+000, 5.256804698481571e-001, 1.027052006373695e+000}, {1.914639375477926e+001, -2.826333658116357e+000, 5.175237986301524e-001, 1.027791969038206e+000}, {1.823104421288958e+001, -2.739715546993399e+000, 5.095101031537107e-001, 1.028530382629299e+000}, {1.736290349293186e+001, -2.656136094630625e+000, 5.016362201009934e-001, 1.029267228468528e+000}, {1.653936831205726e+001, -2.575477526738492e+000, 4.938990649439516e-001, 1.030002487709758e+000}, {1.575797811613206e+001, -2.497626987231605e+000, 4.862956294907959e-001, 1.030736141334639e+000}, {1.501642262596952e+001, -2.422476435238042e+000, 4.788229799338944e-001, 1.031468170148003e+000}, {1.431252592991174e+001, -2.349922366480642e+000, 4.714782545940999e-001, 1.032198554773166e+000}, {1.364423636027999e+001, -2.279865602649950e+000, 4.642586618963331e-001, 1.032927275647125e+000}, {1.300962339585762e+001, -2.212211125883548e+000, 4.571614784290629e-001, 1.033654313015678e+000}, {1.240686782027495e+001, -2.146867870831208e+000, 4.501840469791165e-001, 1.034379646928422e+000}, {1.183425342332221e+001, -2.083748539062776e+000, 4.433237746672978e-001, 1.035103257233662e+000}, {1.129016412610612e+001, -2.022769454725691e+000, 4.365781311814800e-001, 1.035825123573211e+000}, {1.077307541038712e+001, -1.963850383159112e+000, 4.299446469854610e-001, 1.036545225377079e+000}, {1.028154947275094e+001, -1.906914383701821e+000, 4.234209116354142e-001, 1.037263541858046e+000}, {9.814230022131259e+000, -1.851887664597415e+000, 4.170045721339577e-001, 1.037980052006132e+000}, {9.369839696925903e+000, -1.798699456143624e+000, 4.106933313435917e-001, 1.038694734582934e+000}, {8.947167622091373e+000, -1.747281824615215e+000, 4.044849463620441e-001, 1.039407568115843e+000}, {8.545077467190927e+000, -1.697569639716146e+000, 3.983772271819996e-001, 1.040118530892150e+000}, {8.162491034786111e+000, -1.649500353387555e+000, 3.923680350312642e-001, 1.040827600952992e+000}, {7.798393309046508e+000, -1.603013962393644e+000, 3.864552811298065e-001, 1.041534756087189e+000}, {7.451825003941011e+000, -1.558052859669292e+000, 3.806369252096470e-001, 1.042239973824938e+000}, {7.121879549937058e+000, -1.514561735162216e+000, 3.749109742091094e-001, 1.042943231431347e+000}, {6.807699199895982e+000, -1.472487475162612e+000, 3.692754809886303e-001, 1.043644505899842e+000}, {6.508476315599162e+000, -1.431779096689505e+000, 3.637285431064071e-001, 1.044343773945410e+000}, {6.223444600223672e+000, -1.392387605977197e+000, 3.582683015002018e-001, 1.045041011997687e+000}, {5.951881226565523e+000, -1.354265956495707e+000, 3.528929394082206e-001, 1.045736196193881e+000}, {5.693101078416663e+000, -1.317368936654040e+000, 3.476006811465485e-001, 1.046429302371536e+000}, {5.446458886227909e+000, -1.281653127167136e+000, 3.423897910577511e-001, 1.047120306061104e+000}, {5.211340506752048e+000, -1.247076770164139e+000, 3.372585723155903e-001, 1.047809182478361e+000}, {4.987168450830138e+000, -1.213599765364470e+000, 3.322053660151640e-001, 1.048495906516614e+000}, {4.773393379500752e+000, -1.181183539793155e+000, 3.272285500026099e-001, 1.049180452738738e+000}, {4.569496847863383e+000, -1.149791027460064e+000, 3.223265379911744e-001, 1.049862795368995e+000}, {4.374988004990743e+000, -1.119386587540072e+000, 3.174977785277567e-001, 1.050542908284660e+000}, {4.189400888067502e+000, -1.089935937326779e+000, 3.127407540491669e-001, 1.051220765007437e+000}, {4.012295649424565e+000, -1.061406120570848e+000, 3.080539800181962e-001, 1.051896338694638e+000}, {3.843254239286654e+000, -1.033765424196656e+000, 3.034360039600572e-001, 1.052569602130159e+000}, {3.681881374402221e+000, -1.006983350527110e+000, 2.988854046579620e-001, 1.053240527715198e+000}, {3.527802653665643e+000, -9.810305583932155e-001, 2.944007912737826e-001, 1.053909087458743e+000}, {3.380662761329736e+000, -9.558788092568029e-001, 2.899808025158087e-001, 1.054575252967808e+000}, {3.240123783209261e+000, -9.315009216150985e-001, 2.856241058570798e-001, 1.055238995437398e+000}, {3.105868201920377e+000, -9.078707625912555e-001, 2.813293968109665e-001, 1.055900285640210e+000}, {2.977591609870003e+000, -8.849631421045409e-001, 2.770953980214663e-001, 1.056559093916054e+000}, {2.855007736411922e+000, -8.627538388958096e-001, 2.729208587048513e-001, 1.057215390160977e+000}, {2.737843340741328e+000, -8.412195136795313e-001, 2.688045537828591e-001, 1.057869143816096e+000}, {2.625839966333147e+000, -8.203377040825038e-001, 2.647452832788387e-001, 1.058520323856107e+000}, {2.518752537811653e+000, -8.000867793714089e-001, 2.607418715874039e-001, 1.059168898777473e+000}, {2.416347515483972e+000, -7.804458970053758e-001, 2.567931667932268e-001, 1.059814836586282e+000}, {2.318403819543128e+000, -7.613949896269223e-001, 2.528980400707821e-001, 1.060458104785751e+000}, {2.224711437052068e+000, -7.429147243016000e-001, 2.490553850107021e-001, 1.061098670363362e+000}, {2.135070864890457e+000, -7.249864743581408e-001, 2.452641170068208e-001, 1.061736499777632e+000}, {2.049292370784272e+000, -7.075922907052119e-001, 2.415231726533505e-001, 1.062371558944488e+000}, {1.967195951634708e+000, -6.907148805172072e-001, 2.378315091641534e-001, 1.063003813223237e+000}, {1.888610625422974e+000, -6.743375783849550e-001, 2.341881037825931e-001, 1.063633227402109e+000}, {1.813373338023230e+000, -6.584443161814407e-001, 2.305919532083683e-001, 1.064259765683374e+000}, {1.741329942532738e+000, -6.430196157987128e-001, 2.270420730859128e-001, 1.064883391667998e+000}, {1.672332880740151e+000, -6.280485445058859e-001, 2.235374974033225e-001, 1.065504068339818e+000}, {1.606242962644326e+000, -6.135167194115672e-001, 2.200772780372606e-001, 1.066121758049242e+000}, {1.542926476725079e+000, -5.994102568101398e-001, 2.166604841494812e-001, 1.066736422496424e+000}, {1.482257663729330e+000, -5.857157876199568e-001, 2.132862017855761e-001, 1.067348022713911e+000}, {1.424115512595617e+000, -5.724204016631274e-001, 2.099535332621227e-001, 1.067956519048742e+000}, {1.368385956028459e+000, -5.595116632457267e-001, 2.066615967928144e-001, 1.068561871143958e+000}, {1.314959655325564e+000, -5.469775676628228e-001, 2.034095259190647e-001, 1.069164037919527e+000}, {1.263733008673720e+000, -5.348065427110748e-001, 2.001964691070868e-001, 1.069762977552636e+000}, {1.214607187882344e+000, -5.229874224736636e-001, 1.970215892474923e-001, 1.070358647457329e+000}, {1.167487900381954e+000, -5.115094337620333e-001, 1.938840632171667e-001, 1.070951004263476e+000}, {1.122285845216134e+000, -5.003621898130956e-001, 1.907830814535250e-001, 1.071540003795025e+000}, {1.078915247650273e+000, -4.895356599254621e-001, 1.877178474670392e-001, 1.072125601047531e+000}, {1.037295122562504e+000, -4.790201767587073e-001, 1.846875774806051e-001, 1.072707750164894e+000}, {9.973476732340229e-001, -4.688064029957253e-001, 1.816914999294402e-001, 1.073286404415327e+000}, {9.589990505709063e-001, -4.588853343284500e-001, 1.787288551054327e-001, 1.073861516166452e+000}, {9.221789716891475e-001, -4.492482827697643e-001, 1.757988947129753e-001, 1.074433036859547e+000}, {8.868200204717667e-001, -4.398868591505740e-001, 1.729008814488281e-001, 1.075000916982876e+000}, {8.528583447359696e-001, -4.307929744054440e-001, 1.700340886366837e-001, 1.075565106044062e+000}, {8.202326838157281e-001, -4.219588172117957e-001, 1.671977997847816e-001, 1.076125552541462e+000}, {7.888848153812587e-001, -4.133768531089824e-001, 1.643913082231699e-001, 1.076682203934512e+000}, {7.587591016827582e-001, -4.050398098899249e-001, 1.616139166886254e-001, 1.077235006612980e+000}, {7.298024187385580e-001, -3.969406696719444e-001, 1.588649369407936e-001, 1.077783905865066e+000}, {7.019641615586063e-001, -3.890726619994999e-001, 1.561436893787574e-001, 1.078328845844344e+000}, {6.751958155391025e-001, -3.814292509316639e-001, 1.534495026395216e-001, 1.078869769535422e+000}, {6.494512832440340e-001, -3.740041339282753e-001, 1.507817132366013e-001, 1.079406618718322e+000}, {6.246863366052458e-001, -3.667912267567835e-001, 1.481396651445827e-001, 1.079939333931487e+000}, {6.008588763477613e-001, -3.597846624325521e-001, 1.455227094421928e-001, 1.080467854433345e+000}, {5.779285217568081e-001, -3.529787788792315e-001, 1.429302039043457e-001, 1.080992118162387e+000}, {5.558567917843585e-001, -3.463681168961781e-001, 1.403615126355752e-001, 1.081512061695666e+000}, {5.346068604525098e-001, -3.399474109122525e-001, 1.378160056691672e-001, 1.082027620205633e+000}, {5.141434781643980e-001, -3.337115829553734e-001, 1.352930585791013e-001, 1.082538727415259e+000}, {4.944329639780074e-001, -3.276557382434219e-001, 1.327920520940830e-001, 1.083045315551323e+000}, {4.754430791164752e-001, -3.217751584695526e-001, 1.303123716960193e-001, 1.083547315295785e+000}, {4.571429700194347e-001, -3.160652967851284e-001, 1.278534072210391e-001, 1.084044655735171e+000}, {4.395030793576999e-001, -3.105217725167348e-001, 1.254145524556692e-001, 1.084537264307814e+000}, {4.224951347995459e-001, -3.051403671189442e-001, 1.229952047280423e-001, 1.085025066748905e+000}, {4.060920172580015e-001, -2.999170184671716e-001, 1.205947644878575e-001, 1.085507987033182e+000}, {3.902677687631728e-001, -2.948478175978236e-001, 1.182126348866422e-001, 1.085985947315169e+000}, {3.749974640139346e-001, -2.899290035159611e-001, 1.158482213427176e-001, 1.086458867866821e+000}, {3.602572392362872e-001, -2.851569605786520e-001, 1.135009311044318e-001, 1.086926667012439e+000}, {3.460241836412211e-001, -2.805282138082644e-001, 1.111701727955525e-001, 1.087389261060693e+000}, {3.322762639573303e-001, -2.760394252131759e-001, 1.088553559545610e-001, 1.087846564233622e+000}, {3.189924082087265e-001, -2.716873926785463e-001, 1.065558905672928e-001, 1.088298488592417e+000}, {3.061522734419424e-001, -2.674690435082296e-001, 1.042711865658687e-001, 1.088744943959822e+000}, {2.937363571423171e-001, -2.633814348735849e-001, 1.020006533438568e-001, 1.089185837838947e+000}, {2.817258998316861e-001, -2.594217496446171e-001, 9.974369922720212e-002, 1.089621075328317e+000}, {2.701028054319830e-001, -2.555872936090732e-001, 9.749973094036724e-002, 1.090050559032902e+000}, {2.588496724626022e-001, -2.518754945945531e-001, 9.526815305771809e-002, 1.090474188970921e+000}, {2.479497198536963e-001, -2.482838997491669e-001, 9.304836742418512e-002, 1.090891862476159e+000}, {2.373867619937749e-001, -2.448101740087793e-001, 9.083977255919505e-002, 1.091303474095537e+000}, {2.271451341955907e-001, -2.414520978672889e-001, 8.864176303197963e-002, 1.091708915481636e+000}, {2.172097526882940e-001, -2.382075679770417e-001, 8.645372882164712e-002, 1.092108075279883e+000}, {2.075659648124981e-001, -2.350745935908722e-001, 8.427505462681409e-002, 1.092500839010061e+000}, {1.981996108168476e-001, -2.320512978432888e-001, 8.210511916873743e-002, 1.092887088941785e+000}, {1.890969390096326e-001, -2.291359160192762e-001, 7.994329444160937e-002, 1.093266703963582e+000}, {1.802446007941124e-001, -2.263267957100681e-001, 7.778894493527934e-002, 1.093639559445137e+000}, {1.716296024717335e-001, -2.236223964383276e-001, 7.564142681292767e-002, 1.094005527092300e+000}, {1.632392977069316e-001, -2.210212903349189e-001, 7.350008705079136e-002, 1.094364474794345e+000}, {1.550613159026258e-001, -2.185221618314115e-001, 7.136426252241583e-002, 1.094716266463003e+000}, {1.470836025508062e-001, -2.161238099240935e-001, 6.923327904352281e-002, 1.095060761862684e+000}, {1.392943026502691e-001, -2.138251474258300e-001, 6.710645034366781e-002, 1.095397816431321e+000}, {1.316817983518666e-001, -2.116252040084782e-001, 6.498307699545154e-002, 1.095727281091176e+000}, {1.242346587117034e-001, -2.095231275596812e-001, 6.286244526583852e-002, 1.096049002048916e+000}, {1.169415784462329e-001, -2.075181858211773e-001, 6.074382589610382e-002, 1.096362820584195e+000}, {1.097913955238220e-001, -2.056097701035541e-001, 5.862647281078301e-002, 1.096668572825930e+000}, {1.027730154333125e-001, -2.037973975454059e-001, 5.650962173221349e-002, 1.096966089515387e+000}, {9.587540256029346e-002, -2.020807152969548e-001, 5.439248870826525e-002, 1.097255195755098e+000}, {8.908754193525671e-002, -2.004595045004713e-001, 5.227426853327376e-002, 1.097535710742557e+000}, {8.239836637349100e-002, -1.989336843919031e-001, 5.015413305815167e-002, 1.097807447487560e+000}, {7.579678506229054e-002, -1.975033192009564e-001, 4.803122938748919e-002, 1.098070212511922e+000}, {6.927156268654561e-002, -1.961686223190508e-001, 4.590467792438880e-002, 1.098323805530231e+000}, {6.281133833354630e-002, -1.949299647595855e-001, 4.377357029036171e-002, 1.098568019110113e+000}, {5.640453232697693e-002, -1.937878816798458e-001, 4.163696706650428e-002, 1.098802638310406e+000}, {5.003932121783335e-002, -1.927430817480175e-001, 3.949389537115199e-002, 1.099027440295432e+000}, {4.370356344501013e-002, -1.917964562372977e-001, 3.734334623333660e-002, 1.099242193923410e+000}, {3.738474704089313e-002, -1.909490901638774e-001, 3.518427175977259e-002, 1.099446659306848e+000}, {3.106990814761928e-002, -1.902022738733920e-001, 3.301558205824082e-002, 1.099640587342539e+000}, {2.471924771693251e-002, -1.895570588253832e-001, 3.083614013868847e-002, 1.099823719208536e+000}, {1.834223327637784e-002, -1.890154314600311e-001, 2.864476241935234e-002, 1.099995785825217e+000}, {1.195094886503533e-002, -1.885798454355235e-001, 2.644021168354557e-002, 1.100156507277238e+000}, {5.557050709009414e-003, -1.882515929288725e-001, 2.418461128139228e-002, 1.100307943910574e+000}, {-8.014493909795158e-004, -1.880402684286048e-001, 2.194575815737601e-002, 1.100445109478535e+000}, {-7.135417654424650e-003, -1.879436446578345e-001, 1.972726442004061e-002, 1.100567996583830e+000}, {-1.345870138684300e-002, -1.879591637675092e-001, 1.752795451986745e-002, 1.100676931096945e+000}, {-1.978471856691031e-002, -1.880845063496110e-001, 1.534670037602332e-002, 1.100772233147846e+000}, {-2.612655707280306e-002, -1.883175768416115e-001, 1.318241876916896e-002, 1.100854217210090e+000}, {-3.249697575622967e-002, -1.886564925021107e-001, 1.103406890701701e-002, 1.100923192183922e+000}, {-3.890850468817631e-002, -1.890995704946074e-001, 8.900650115975411e-003, 1.100979461478381e+000}, {-4.537347157514723e-002, -1.896453182395342e-001, 6.781199688611053e-003, 1.101023323092393e+000}, {-5.190403204992995e-002, -1.902924240651144e-001, 4.674790850459827e-003, 1.101055069694875e+000}, {-5.851225277558175e-002, -1.910397473018965e-001, 2.580530837908182e-003, 1.101074988703860e+000}, {-6.521011476274589e-002, -1.918863113150857e-001, 4.975591027557807e-004, 1.101083362364609e+000}, {-7.200957075288722e-002, -1.928312955496475e-001, -1.574954392253073e-003, 1.101080467826757e+000}, {-7.892258128041750e-002, -1.938740289010325e-001, -3.637810776014087e-003, 1.101066577220491e+000}, {-8.596114095058548e-002, -1.950139836709728e-001, -5.691783782005860e-003, 1.101041957731749e+000}, {-9.313731870582566e-002, -1.962507697828389e-001, -7.737621177713153e-003, 1.101006871676471e+000}, {-1.004633132706661e-001, -1.975841292444321e-001, -9.776046115403617e-003, 1.100961576573900e+000}, {-1.079514189138674e-001, -1.990139329502044e-001, -1.180775839988235e-002, 1.100906325218945e+000}, {-1.156141501458894e-001, -2.005401741857532e-001, -1.383343570756720e-002, 1.100841365753626e+000}, {-1.234641865756744e-001, -2.021629669142985e-001, -1.585373471362041e-002, 1.100766941737587e+000}, {-1.315144632458761e-001, -2.038825407987732e-001, -1.786929218092619e-002, 1.100683292217720e+000}, {-1.397781622796490e-001, -2.056992389395121e-001, -1.988072597730302e-002, 1.100590651796894e+000}, {-1.482687849379858e-001, -2.076135142055186e-001, -2.188863604735403e-002, 1.100489250701797e+000}, {-1.570001127509399e-001, -2.096259281418626e-001, -2.389360532338351e-002, 1.100379314849913e+000}, {-1.659863131041873e-001, -2.117371469439204e-001, -2.589620060511024e-002, 1.100261065915656e+000}, {-1.752419465890530e-001, -2.139479403721047e-001, -2.789697337688882e-002, 1.100134721395630e+000}, {-1.847819278769801e-001, -2.162591809460440e-001, -2.989646059049550e-002, 1.100000494673090e+000}, {-1.946216794315628e-001, -2.186718402076742e-001, -3.189518541573153e-002, 1.099858595081566e+000}, {-2.047770670769773e-001, -2.211869893365289e-001, -3.389365793731571e-002, 1.099709227967682e+000}, {-2.152644567239538e-001, -2.238057975178256e-001, -3.589237582835234e-002, 1.099552594753185e+000}, {-2.261007643374610e-001, -2.265295306054332e-001, -3.789182498937845e-002, 1.099388892996184e+000}, {-2.373034676588316e-001, -2.293595508675275e-001, -3.989248015126330e-002, 1.099218316451631e+000}, {-2.488906597672881e-001, -2.322973162338434e-001, -4.189480545039329e-002, 1.099041055131017e+000}, {-2.608810484226545e-001, -2.353443803241083e-001, -4.389925497691962e-002, 1.098857295361356e+000}, {-2.732940302195583e-001, -2.385023915702908e-001, -4.590627329721378e-002, 1.098667219843396e+000}, {-2.861497020009324e-001, -2.417730936839753e-001, -4.791629594706479e-002, 1.098471007709130e+000}, {-2.994689199938831e-001, -2.451583253792444e-001, -4.992974990525140e-002, 1.098268834578575e+000}, {-3.132732751354347e-001, -2.486600214001266e-001, -5.194705404153135e-002, 1.098060872615857e+000}, {-3.275852638322766e-001, -2.522802108711590e-001, -5.396861955037104e-002, 1.097847290584594e+000}, {-3.424281663994858e-001, -2.560210205428296e-001, -5.599485034861269e-002, 1.097628253902595e+000}, {-3.578293520435974e-001, -2.598846367187872e-001, -5.802614356800304e-002, 1.097403924695883e+000}, {-3.737991995924480e-001, -2.638735869985462e-001, -6.006288915407318e-002, 1.097174461852052e+000}, {-3.903902600560969e-001, -2.679898530112919e-001, -6.210547303843708e-002, 1.096940021072966e+000}, {-4.076012547942460e-001, -2.722364457397745e-001, -6.415427156013318e-002, 1.096700754925520e+000}, {-4.254343891046640e-001, -2.766161117295213e-001, -6.620965819623846e-002, 1.096456812898149e+000}, {-4.438339027573641e-001, -2.811320490392464e-001, -6.827199959147826e-002, 1.096208341444035e+000}, {-4.644750403946706e-001, -2.857679946652489e-001, -7.034170236902584e-002, 1.095955484037268e+000}, {-4.820890817525319e-001, -2.906000672424869e-001, -7.241891749894044e-002, 1.095698381226341e+000}, {-5.047134822541359e-001, -2.955056304565424e-001, -7.450444780138031e-002, 1.095437170699271e+000}, {-5.258914498568529e-001, -3.006012937060756e-001, -7.659813906378293e-002, 1.095171987136068e+000}, {-5.484047930470472e-001, -3.058375854914676e-001, -7.870062238535235e-002, 1.094902962737344e+000}, {-5.718129216158286e-001, -3.112278431359212e-001, -8.081217584455191e-002, 1.094630226742015e+000}, {-5.961547440676940e-001, -3.167755376343922e-001, -8.293313841275786e-002, 1.094353905753897e+000}, {-6.214708520460160e-001, -3.224842528277391e-001, -8.506384586954169e-002, 1.094074123729523e+000}, {-6.478032148383646e-001, -3.283576921832907e-001, -8.720463098789563e-002, 1.093791002023522e+000}, {-6.751957491746247e-001, -3.343996727291274e-001, -8.935582375150214e-002, 1.093504659433427e+000}, {-7.036936171960371e-001, -3.406141369349701e-001, -9.151775149673061e-002, 1.093215212243908e+000}, {-7.333438920356818e-001, -3.470051435467684e-001, -9.369073911646579e-002, 1.092922774270491e+000}, {-7.649284588825603e-001, -3.535701423882779e-001, -9.587512294577692e-002, 1.092627456902683e+000}, {-7.987573127935487e-001, -3.603086226499994e-001, -9.807123530043556e-002, 1.092329369146623e+000}, {-8.351881293636163e-001, -3.672179068043334e-001, -1.002794318903687e-001, 1.092028617667189e+000}, {-8.746337064830027e-001, -3.750216293989062e-001, -1.027084493542587e-001, 1.091696779574096e+000}, {-9.182631668618914e-001, -3.831991324672869e-001, -1.051985063743867e-001, 1.091355729081559e+000}, {-9.598866647526484e-001, -3.918556861719648e-001, -1.077524800454063e-001, 1.091005198961375e+000}, {-1.010790497453898e+000, -4.008622937465624e-001, -1.103741544582843e-001, 1.090644908830807e+000}, {-1.061585597914184e+000, -4.104074244317262e-001, -1.130664113126074e-001, 1.090274564104240e+000}, {-1.116923652713534e+000, -4.204470439736384e-001, -1.158334219482580e-001, 1.089893855365184e+000}, {-1.176390470616413e+000, -4.310341925159665e-001, -1.186791465288392e-001, 1.089502456776096e+000}, {-1.240398358090701e+000, -4.422099547341248e-001, -1.216079485825098e-001, 1.089100025144463e+000}, {-1.309411351967095e+000, -4.540194728668653e-001, -1.246245298507227e-001, 1.088686198529077e+000}, {-1.383952478324663e+000, -4.665124525546056e-001, -1.277339645466649e-001, 1.088260594774788e+000}, {-1.464613539394650e+000, -4.797437255410850e-001, -1.309417379885211e-001, 1.087822809897360e+000}, {-1.552065595474018e+000, -4.937739146636258e-001, -1.342537901400978e-001, 1.087372416299330e+000}, {-1.647071854814870e+000, -5.086701932065426e-001, -1.376765649336416e-001, 1.086908960794945e+000}, {-1.750503179964703e+000, -5.245071629179311e-001, -1.412170662450014e-001, 1.086431962418652e+000}, {-1.863356315547510e+000, -5.413678789587019e-001, -1.448829215901497e-001, 1.085940909987632e+000}, {-1.986775922871999e+000, -5.593450423418901e-001, -1.486824548572708e-001, 1.085435259383995e+000}, {-2.122081131299789e+000, -5.785423947412177e-001, -1.526247696085506e-001, 1.084914430516412e+000}, {-2.270797351520379e+000, -5.990763602365653e-001, -1.567198447858746e-001, 1.084377803914027e+000}, {-2.434695501360245e+000, -6.210779709042096e-001, -1.609786450924524e-001, 1.083824716897111e+000}, {-2.615838531681118e+000, -6.446951619454530e-001, -1.654132486612294e-001, 1.083254459258780e+000}, {-2.816639044616160e+000, -6.700954812249659e-001, -1.700369953943099e-001, 1.082666268379815e+000}, {-3.039928700415489e+000, -6.974693316356091e-001, -1.748646598439750e-001, 1.082059323683556e+000}, {-3.292219758098840e+000, -7.270000140981027e-001, -1.799127335700266e-001, 1.081432740319499e+000}, {-3.564782476198684e+000, -7.591050350223508e-001, -1.851990623476152e-001, 1.080785561941540e+000}, {-3.884190359823099e+000, -7.937345882311215e-001, -1.907452448062669e-001, 1.080116752418797e+000}, {-4.237141139726640e+000, -8.315523945506937e-001, -1.965727692029269e-001, 1.079425186076460e+000}, {-4.639142492737927e+000, -8.727752413556356e-001, -2.027083906666977e-001, 1.078709637232685e+000}, {-5.097189675608282e+000, -9.178820437390601e-001, -2.091811202970346e-001, 1.077968766554805e+000}, {-5.621762464449057e+000, -9.673950144480338e-001, -2.160241562151455e-001, 1.077201106424277e+000}, {-6.225833712033967e+000, -1.021929051176963e+000, -2.232753772451849e-001, 1.076405043117602e+000}, {-6.925601260591495e+000, -1.082212356404202e+000, -2.309782180242463e-001, 1.075578795840945e+000}, {-7.741476361496349e+000, -1.149112657464135e+000, -2.391827511126749e-001, 1.074720391772839e+000}, {-8.699439370277643e+000, -1.223670757770951e+000, -2.479470365689759e-001, 1.073827636152860e+000}, {-9.832921391371407e+000, -1.307143891200575e+000, -2.573388201788544e-001, 1.072898076168983e+000}, {-1.118545570250340e+001, -1.401062230883758e+000, -2.674376916575652e-001, 1.071928957010234e+000}, {-1.281446878655493e+001, -1.507303306245227e+000, -2.783378567732671e-001, 1.070917167920417e+000}, {-1.479678827882319e+001, -1.628191020093895e+000, -2.901517399548284e-001, 1.069859175350126e+000}, {-1.724144136142261e+001, -1.766576297530559e+000, -3.030148579310907e-001, 1.068750939259828e+000}, {-2.027826987203854e+001, -1.926337376059677e+000, -3.170912838064992e-001, 1.067587807127434e+000}, {-2.413687940816858e+001, -2.111734868657410e+000, -3.325851366637468e-001, 1.066364378021886e+000}, {-2.909401840847157e+001, -2.329151580543732e+000, -3.497470819764231e-001, 1.065074325623783e+000}, {-3.559985600470116e+001, -2.586219259879967e+000, -3.688972788252130e-001, 1.063710165062041e+000}, {-4.432903871576617e+001, -2.893237376748189e+000, -3.904455236187271e-001, 1.062262938236866e+000}, {-5.635486800356907e+001, -3.263787309390394e+000, -4.149289589297027e-001, 1.060721782952100e+000}, {-7.345422356224562e+001, -3.715852360320928e+000, -4.430674062157222e-001, 1.059073327378870e+000}, {-9.872541030429800e+001, -4.272847713002282e+000, -4.758525332055549e-001, 1.057300816644650e+000}, {-1.379029789580371e+002, -4.963698528158174e+000, -5.146977811000596e-001, 1.055382812700357e+000}, {-2.025227956751322e+002, -5.816502032157165e+000, -5.617124713275824e-001, 1.053291185212563e+000}, {-3.066576906266529e+002, -6.965314926095712e+000, -6.198853650303311e-001, 1.050987862948901e+000}, {-4.830543755124683e+002, -8.666062419141326e+000, -6.938823513423141e-001, 1.048419277617361e+000}, {-8.084833331246175e+002, -1.148692234324379e+001, -7.910281397083958e-001, 1.045506157577100e+000}, {-1.468144450984898e+003, -1.270342041479051e+001, -8.932720264828298e-001, 1.043022458452853e+000}, {-2.960185006157666e+003, -1.204645827816411e+001, -1.037687543523871e+000, 1.040192609451484e+000}, {-6.394379648177180e+003, -1.133095662667407e+001, -1.255214139021270e+000, 1.036731981255395e+000}, {-1.421291626844065e+004, -2.176830415540717e+001, -1.565859480023533e+000, 1.032664478961364e+000}, {-3.188506981409266e+004, -4.472235675466877e+001, -1.957122092195666e+000, 1.028823550217078e+000}, {-7.717088076510136e+004, -4.157109709425348e+001, -2.425675063473610e+000, 1.025629204891293e+000}, {-1.893604841837793e+005, 1.988107110128464e+001, -3.037933999829498e+000, 1.022682315051265e+000}, {-4.362759910617617e+005, 4.356252894797460e+000, -3.857056802145624e+000, 1.019493900338764e+000}, {-9.667858531739716e+005, -7.110743222205122e+002, -5.068028151693111e+000, 1.015606411778339e+000}, {-1.926807175753026e+006, -1.308643373681819e+003, -6.100046763833379e+000, 1.013498423615123e+000}, {-4.430874958584904e+006, -5.255191972894967e+002, -6.892511697752738e+000, 1.012521645013935e+000}, {-1.306500915854772e+007, 1.740498999891739e+003, -8.577603939805345e+000, 1.010846737556340e+000}, {-4.389793234302502e+007, 9.560482174458750e+003, -1.148948797863325e+001, 1.009039661353492e+000}, {-1.453953316964447e+008, 8.587054210867187e+003, -1.476464101007846e+001, 1.006692766581286e+000}, {-3.516933123906404e+008, -1.085943267171107e+004, -1.840303816166151e+001, 1.005101578506159e+000}, {-2.510574163087101e+012, 1.191163343802633e+008, -1.412900529450008e+003, 1.004074865378772e+000}};
        constant Real[Ninterval, 4] slcoef = {{1.177557139507648e+010, -1.993514199606193e+006, 4.143546065409241e+002, 2.641402961120132e-001}, {1.048671143722957e+010, -1.884478673305442e+006, 4.037631012049168e+002, 2.652511930832029e-001}, {9.168562051007404e+009, -1.788981183928967e+006, 3.950602310696419e+002, 2.661829932674829e-001}, {7.846446101028213e+009, -1.614583713308304e+006, 3.761535139366466e+002, 2.682996669625260e-001}, {6.715017635832893e+009, -1.457222530950107e+006, 3.581621886022840e+002, 2.704246269918255e-001}, {5.747015349679062e+009, -1.315240915328985e+006, 3.410420212198932e+002, 2.725579055981067e-001}, {4.918630356754790e+009, -1.187122828234776e+006, 3.247498175022562e+002, 2.746996665285123e-001}, {4.209783796072495e+009, -1.071515458129660e+006, 3.092453129575851e+002, 2.768499837872521e-001}, {3.603174902826428e+009, -9.671924631452678e+005, 2.944898874475471e+002, 2.790089741555691e-001}, {3.084079936755044e+009, -8.730521764425569e+005, 2.804471553915772e+002, 2.811767093063310e-001}, {2.639848105537634e+009, -7.880971686153368e+005, 2.670822304519017e+002, 2.833533034650044e-001}, {2.259641698967966e+009, -7.114290219738688e+005, 2.543622233288261e+002, 2.855388260072460e-001}, {1.934270134269886e+009, -6.422381187091104e+005, 2.422554107091234e+002, 2.877334319246923e-001}, {1.655784123271669e+009, -5.797918189723861e+005, 2.307318702957838e+002, 2.899372311407367e-001}, {1.417436324893234e+009, -5.234328119503804e+005, 2.197634071875102e+002, 2.921502891166354e-001}, {1.213429199043588e+009, -4.725651621534369e+005, 2.093227855305521e+002, 2.943727562457704e-001}, {1.038821174264480e+009, -4.266538020807887e+005, 1.993845312208418e+002, 2.966046954288835e-001}, {8.900554569418168e+008, -3.852391963057016e+005, 1.899242490751680e+002, 2.988462540463003e-001}, {7.648023186829292e+008, -3.479479314856909e+005, 1.809193670502903e+002, 3.010975350565891e-001}, {6.592177452001010e+008, -3.143877634345993e+005, 1.723482399369443e+002, 3.033586400558302e-001}, {5.700843185859680e+008, -2.846663102712536e+005, 1.643477796985073e+002, 3.055842723398675e-001}, {4.943633825796188e+008, -2.589969957748900e+005, 1.571225846083740e+002, 3.076990337830958e-001}, {4.291125156131715e+008, -2.357953619216334e+005, 1.502638626066520e+002, 3.098084456029034e-001}, {3.728306499579160e+008, -2.148103610406289e+005, 1.437508020718366e+002, 3.119125568514417e-001}, {3.242364285177285e+008, -1.958176234450135e+005, 1.375638868362061e+002, 3.140114157267036e-001}, {2.822412748713871e+008, -1.786169231686780e+005, 1.316848160932519e+002, 3.161050695947951e-001}, {2.459125658184237e+008, -1.630289616337319e+005, 1.260964138486158e+002, 3.181935650115526e-001}, {2.144580625278116e+008, -1.488936415091122e+005, 1.207825640937267e+002, 3.202769477435048e-001}, {1.871966290478286e+008, -1.360673141338019e+005, 1.157281282277565e+002, 3.223552627882310e-001}, {1.635488056210168e+008, -1.244215705088064e+005, 1.109188923271442e+002, 3.244285543941092e-001}, {1.430161329214414e+008, -1.138410894667223e+005, 1.063414954697406e+002, 3.264968660794893e-001}, {1.251717914268840e+008, -1.042224881444728e+005, 1.019833809887122e+002, 3.285602406513014e-001}, {1.096508061352110e+008, -9.547306166403441e+004, 9.783274285471715e+001, 3.306187202231238e-001}, {9.613753724667005e+007, -8.750933370030554e+004, 9.387847130169364e+001, 3.326723462327222e-001}, {8.436265639394236e+007, -8.025648609043579e+004, 9.011011773741680e+001, 3.347211594590819e-001}, {7.409313187822527e+007, -7.364708312272583e+004, 8.651784263733413e+001, 3.367652000389455e-001}, {6.512895658412796e+007, -6.762051692091678e+004, 8.309238335216898e+001, 3.388045074828699e-001}, {5.729728405538097e+007, -6.212215934894936e+004, 7.982501352086291e+001, 3.408391206908288e-001}, {5.044953216127181e+007, -5.710284583154797e+004, 7.670751215248156e+001, 3.428690779673607e-001}, {4.445647437035704e+007, -5.251814570452608e+004, 7.373212777286675e+001, 3.448944170362825e-001}, {3.920767247485867e+007, -4.832812939512341e+004, 7.089155600321682e+001, 3.469151750549936e-001}, {3.460643566473103e+007, -4.449659421205359e+004, 6.817890214139165e+001, 3.489313886283582e-001}, {3.056970641837368e+007, -4.099094122891565e+004, 6.558766506547076e+001, 3.509430938222040e-001}, {2.702540190216035e+007, -3.778168333466380e+004, 6.311170749756870e+001, 3.529503261764371e-001}, {2.391081714254122e+007, -3.484213008611726e+004, 6.074523461994335e+001, 3.549531207177877e-001}, {2.117170192432593e+007, -3.214816129623113e+004, 5.848277505538282e+001, 3.569515119721937e-001}, {1.876094064229434e+007, -2.967793653594457e+004, 5.631915951588184e+001, 3.589455339768409e-001}, {1.663741340545124e+007, -2.741164725010574e+004, 5.424950232335298e+001, 3.609352202918695e-001}, {1.476547731295308e+007, -2.533136468686681e+004, 5.226918630626200e+001, 3.629206040117464e-001}, {1.311405368894354e+007, -2.342082009135800e+004, 5.037384498385280e+001, 3.649017177763315e-001}, {1.165604536024709e+007, -2.166524918697802e+004, 4.855934824972481e+001, 3.668785937816321e-001}, {1.036780283368990e+007, -2.005124607418004e+004, 4.682178849186924e+001, 3.688512637902600e-001}, {9.228723479638483e+006, -1.856664063058170e+004, 4.515746777050035e+001, 3.708197591416083e-001}, {8.220767144960379e+006, -1.720036292993756e+004, 4.356288478792973e+001, 3.727841107617398e-001}, {7.328189892577781e+006, -1.594235240875964e+004, 4.203472422676561e+001, 3.747443491730145e-001}, {6.537204250437003e+006, -1.478345202257369e+004, 4.056984541826727e+001, 3.767005045034494e-001}, {5.835726559790004e+006, -1.371532397872642e+004, 3.916527248752735e+001, 3.786526064958268e-001}, {5.213185995368226e+006, -1.273037834190048e+004, 3.781818514258747e+001, 3.806006845165569e-001}, {4.660312354353476e+006, -1.182169731669702e+004, 3.652590940436652e+001, 3.825447675643013e-001}, {4.168939088305709e+006, -1.098296699999315e+004, 3.528590920102089e+001, 3.844848842783646e-001}, {3.731941349696544e+006, -1.020843939792505e+004, 3.409577949657835e+001, 3.864210629468634e-001}, {3.343018842151671e+006, -9.492857477753520e+003, 3.295323749642467e+001, 3.883533315146766e-001}, {2.996638160477085e+006, -8.831424276104683e+003, 3.185611695501934e+001, 3.902817175911858e-001}, {2.687950210708543e+006, -8.219762157299303e+003, 3.080236148343732e+001, 3.922062484578109e-001}, {2.412652443194333e+006, -7.653860521818649e+003, 2.979001765514563e+001, 3.941269510753455e-001}, {2.166972405327071e+006, -7.130058286584131e+003, 2.881723052100683e+001, 3.960438520911025e-001}, {1.947576947792416e+006, -6.645003522979915e+003, 2.788223738207444e+001, 3.979569778458746e-001}, {1.751525675665824e+006, -6.195628233643848e+003, 2.698336301772617e+001, 3.998663543807083e-001}, {1.576215494282452e+006, -5.779120941281210e+003, 2.611901481818039e+001, 4.017720074435056e-001}, {1.419353182727988e+006, -5.392908148824695e+003, 2.528767861012635e+001, 4.036739624954574e-001}, {1.278906533893276e+006, -5.034629032900475e+003, 2.448791404432028e+001, 4.055722447173105e-001}, {1.153074815308473e+006, -4.702117917310478e+003, 2.371835083258324e+001, 4.074668790154691e-001}, {1.040267269572240e+006, -4.393389187391892e+003, 2.297768513031833e+001, 4.093578900279491e-001}, {9.390721944414651e+005, -4.106619140813017e+003, 2.226467576001038e+001, 4.112453021301762e-001}, {8.482361512114307e+005, -3.840132438338376e+003, 2.157814101065838e+001, 4.131291394406401e-001}, {7.666490444341356e+005, -3.592390669372514e+003, 2.091695561139572e+001, 4.150094258264032e-001}, {6.933244377674338e+005, -3.361979229162954e+003, 2.028004761843898e+001, 4.168861849084756e-001}, {6.273833850441934e+005, -3.147596365453398e+003, 1.966639566026135e+001, 4.187594400670539e-001}, {5.680493701729885e+005, -2.948046437224073e+003, 1.907502654460529e+001, 4.206292144466324e-001}, {5.146259094288983e+005, -2.762226337105739e+003, 1.850501230753378e+001, 4.224955309609856e-001}, {4.664966079472982e+005, -2.589122141751236e+003, 1.795546841607669e+001, 4.243584122980267e-001}, {4.231115651009208e+005, -2.427799181763609e+003, 1.742555118911682e+001, 4.262178809245570e-001}, {3.839792678848923e+005, -2.277395453181725e+003, 1.691445579070118e+001, 4.280739590908861e-001}, {3.486632470541687e+005, -2.137117115184443e+003, 1.642141441733220e+001, 4.299266688353537e-001}, {3.167723452866394e+005, -2.006231019071146e+003, 1.594569419580369e+001, 4.317760319887326e-001}, {2.879581579606791e+005, -1.884061136618374e+003, 1.548659563100448e+001, 4.336220701785344e-001}, {2.619091112658384e+005, -1.769983168287696e+003, 1.504345081784956e+001, 4.354648048332044e-001}, {2.383466576671081e+005, -1.663420519322795e+003, 1.461562190867013e+001, 4.373042571862285e-001}, {2.170211247057410e+005, -1.563840223729814e+003, 1.420249960746107e+001, 4.391404482801311e-001}, {1.977102806012762e+005, -1.470750305289779e+003, 1.380350185368774e+001, 4.409733989703888e-001}, {1.802132568331950e+005, -1.383694745078228e+003, 1.341807226290494e+001, 4.428031299292535e-001}, {1.643517256222710e+005, -1.302252783281351e+003, 1.304567919411233e+001, 4.446296616494750e-001}, {1.499647429474115e+005, -1.226034288211031e+003, 1.268581423597655e+001, 4.464530144479596e-001}, {1.369083619764228e+005, -1.154678401042467e+003, 1.233799128533460e+001, 4.482732084693222e-001}, {1.250530635446248e+005, -1.087850707548286e+003, 1.200174537304730e+001, 4.500902636893757e-001}, {1.142828813183554e+005, -1.025241603778916e+003, 1.167663171816983e+001, 4.519041999185328e-001}, {1.044932010702937e+005, -9.665638335328233e+002, 1.136222467710369e+001, 4.537150368051356e-001}, {9.559024735983493e+004, -9.115512664941300e+002, 1.105811692791990e+001, 4.555227938387087e-001}, {8.748948014204207e+004, -8.599568992252292e+002, 1.076391852809859e+001, 4.573274903531447e-001}, {8.011488050377187e+004, -8.115515935776252e+002, 1.047925614072886e+001, 4.591291455298160e-001}, {7.339789345132106e+004, -7.661225797978357e+002, 1.020377222883108e+001, 4.609277784006200e-001}, {6.727691002196107e+004, -7.234723932600134e+002, 9.937124346632666e+000, 4.627234078509644e-001}, {6.169614021722738e+004, -6.834173972226075e+002, 9.678984382191800e+000, 4.645160526226781e-001}, {5.660547400068403e+004, -6.457871273196512e+002, 9.429037972024533e+000, 4.663057313168709e-001}, {5.195953611750653e+004, -6.104229732107484e+002, 9.186983791759216e+000, 4.680924623967295e-001}, {4.771745208001675e+004, -5.771775202529138e+002, 8.952533011439000e+000, 4.698762641902499e-001}, {4.384220812640152e+004, -5.459135464020653e+002, 8.725408685310237e+000, 4.716571548929249e-001}, {4.030038590987054e+004, -5.165033923275631e+002, 8.505345242117434e+000, 4.734351525703650e-001}, {3.706177619104521e+004, -4.888282167622580e+002, 8.292087953582172e+000, 4.752102751608759e-001}, {3.409901222107622e+004, -4.627773087875500e+002, 8.085392439164183e+000, 4.769825404779763e-001}, {3.138732351663624e+004, -4.382475368783908e+002, 7.885024214209901e+000, 4.787519662128720e-001}, {2.890427534083820e+004, -4.151427919551941e+002, 7.690758243308068e+000, 4.805185699368778e-001}, {2.662955418230496e+004, -3.933734889327271e+002, 7.502378519257654e+000, 4.822823691037926e-001}, {2.454471428578327e+004, -3.728560514675434e+002, 7.319677653248337e+000, 4.840433810522326e-001}, {2.263303970197622e+004, -3.535125325278318e+002, 7.142456508875918e+000, 4.858016230079136e-001}, {2.087936933258245e+004, -3.352701961397641e+002, 6.970523831131070e+000, 4.875571120858966e-001}, {1.926989719907531e+004, -3.180611010588355e+002, 6.803695893605354e+000, 4.893098652927868e-001}, {1.779213064394456e+004, -3.018218612541485e+002, 6.641796195081590e+000, 4.910598995288961e-001}, {1.643469380853188e+004, -2.864932315962904e+002, 6.484655115123119e+000, 4.928072315903642e-001}, {1.518723269686234e+004, -2.720198428014330e+002, 6.332109630922232e+000, 4.945518781712424e-001}, {1.404034267449176e+004, -2.583499599306953e+002, 6.184003039444280e+000, 4.962938558655389e-001}, {1.298547086145065e+004, -2.454352119567302e+002, 6.040184677120072e+000, 4.980331811692316e-001}, {1.201480477719220e+004, -2.332303254901078e+002, 5.900509656039089e+000, 4.997698704822443e-001}, {1.112125559868982e+004, -2.216929799079140e+002, 5.764838638887057e+000, 5.015039401103889e-001}, {1.029834063001183e+004, -2.107835348750359e+002, 5.633037579396170e+000, 5.032354062672774e-001}, {9.540169613654271e+003, -2.004649068816667e+002, 5.504977520065665e+000, 5.049642850761986e-001}, {8.841350078353322e+003, -1.907023386990655e+002, 5.380534358885717e+000, 5.066905925719676e-001}, {8.196974604965422e+003, -1.814632922354595e+002, 5.259588665038832e+000, 5.084143447027471e-001}, {7.602559262208660e+003, -1.727172694039041e+002, 5.142025470829316e+000, 5.101355573318360e-001}, {7.054006287042218e+003, -1.644356797777449e+002, 5.027734090331723e+000, 5.118542462394324e-001}, {6.547574282638359e+003, -1.565917209939956e+002, 4.916607944897258e+000, 5.135704271243708e-001}, {6.079842281410332e+003, -1.491602529370824e+002, 4.808544390805874e+000, 5.152841156058329e-001}, {5.647681402804354e+003, -1.421176889479255e+002, 4.703444558999890e+000, 5.169953272250335e-001}, {5.248227929512284e+003, -1.354418928357044e+002, 4.601213200953995e+000, 5.187040774468807e-001}, {4.878866346015187e+003, -1.291120924929621e+002, 4.501758543631354e+000, 5.204103816616156e-001}, {4.537191205039604e+003, -1.231087657275091e+002, 4.404992140311703e+000, 5.221142551864300e-001}, {4.221010702237321e+003, -1.174135968361089e+002, 4.310828751515844e+000, 5.238157132670570e-001}, {3.928309050855646e+003, -1.120093625136355e+002, 4.219186199256487e+000, 5.255147710793474e-001}, {3.657237561904996e+003, -1.068798773450843e+002, 4.129985254777306e+000, 5.272114437308226e-001}, {3.406108390579024e+003, -1.020099385885924e+002, 4.043149522766261e+000, 5.289057462622082e-001}, {3.173362528937843e+003, -9.738523071542197e+001, 3.958605315724437e+000, 5.305976936489506e-001}, {2.957575972851209e+003, -9.299230413876768e+001, 3.876281564344415e+000, 5.322873008027116e-001}, {2.757441473645700e+003, -8.881850292721700e+001, 3.796109702595772e+000, 5.339745825728525e-001}, {2.571752175640579e+003, -8.485190527654152e+001, 3.718023568465873e+000, 5.356595537478953e-001}, {2.399405315832375e+003, -8.108130218799091e+001, 3.641959321507089e+000, 5.373422290569717e-001}, {2.239383750765094e+003, -7.749613093565345e+001, 3.567855339770726e+000, 5.390226231712563e-001}, {2.090753731341121e+003, -7.408644637791140e+001, 3.495652141216298e+000, 5.407007507053830e-001}, {1.952653808168716e+003, -7.084287425064484e+001, 3.425292296888851e+000, 5.423766262188504e-001}, {1.824293969495761e+003, -6.775658698526449e+001, 3.356720357330965e+000, 5.440502642174112e-001}, {1.704945388258175e+003, -6.481925944163990e+001, 3.289882770290317e+000, 5.457216791544545e-001}, {1.593935912714465e+003, -6.202304097497358e+001, 3.224727811476121e+000, 5.473908854323666e-001}, {1.490648848056936e+003, -5.936053328755472e+001, 3.161205517518115e+000, 5.490578974038881e-001}, {1.394512592254002e+003, -5.682474952754494e+001, 3.099267612151868e+000, 5.507227293734606e-001}, {1.305002583866976e+003, -5.440910349376155e+001, 3.038867451236471e+000, 5.523853955985542e-001}, {1.221633497165447e+003, -5.210737493440172e+001, 2.979959954204727e+000, 5.540459102909946e-001}, {1.143958328479689e+003, -4.991369373166751e+001, 2.922501549923314e+000, 5.557042876182787e-001}, {1.071563973096530e+003, -4.782251531429407e+001, 2.866450117837696e+000, 5.573605417048770e-001}, {1.004069097487647e+003, -4.582860310204230e+001, 2.811764935547721e+000, 5.590146866335348e-001}, {9.411214959043084e+002, -4.392700984466779e+001, 2.758406626564467e+000, 5.606667364465596e-001}, {8.823958497130813e+002, -4.211306059866846e+001, 2.706337110626942e+000, 5.623167051471074e-001}, {8.275910331590102e+002, -4.038233540368031e+001, 2.655519555526163e+000, 5.639646067004543e-001}, {7.764290444119635e+002, -3.873065665643446e+001, 2.605918332887885e+000, 5.656104550352749e-001}, {7.286523598314500e+002, -3.715407256916351e+001, 2.557498972533741e+000, 5.672542640448997e-001}, {6.840231188009741e+002, -3.564884626586955e+001, 2.510228121970509e+000, 5.688960475885800e-001}, {6.423204961199782e+002, -3.421144032546041e+001, 2.464073504009842e+000, 5.705358194927445e-001}, {6.033409092767411e+002, -3.283850964957904e+001, 2.419003880919390e+000, 5.721735935522480e-001}, {5.668947663597646e+002, -3.152688470052121e+001, 2.374989013056672e+000, 5.738093835316227e-001}, {5.328072442146844e+002, -3.027356738128294e+001, 2.331999627743732e+000, 5.754432031663234e-001}, {5.009158770478630e+002, -2.907571683947941e+001, 2.290007380866240e+000, 5.770750661639714e-001}, {4.710701060225499e+002, -2.793064247095762e+001, 2.248984825962424e+000, 5.787049862055952e-001}, {4.431303458782849e+002, -2.683579521467626e+001, 2.208905382049609e+000, 5.803329769468701e-001}, {4.169669853608175e+002, -2.578875908229225e+001, 2.169743302771505e+000, 5.819590520193594e-001}, {3.924599612327291e+002, -2.478724467824711e+001, 2.131473647762967e+000, 5.835832250317513e-001}, {3.694974326826228e+002, -2.382908020411502e+001, 2.094072253087125e+000, 5.852055095710999e-001}, {3.479758690185427e+002, -2.291220742271244e+001, 2.057515706247509e+000, 5.868259192040625e-001}, {3.277990349520803e+002, -2.203467367031391e+001, 2.021781318214516e+000, 5.884444674781422e-001}, {3.088773751747854e+002, -2.119462597004226e+001, 1.986847098785308e+000, 5.900611679229281e-001}, {2.911276710946399e+002, -2.039030622074293e+001, 1.952691733147482e+000, 5.916760340513410e-001}, {2.744727583811454e+002, -1.962004641525278e+001, 1.919294558620836e+000, 5.932890793608796e-001}, {2.588404421844505e+002, -1.888226154495557e+001, 1.886635540737936e+000, 5.949003173348717e-001}, {2.441640574222745e+002, -1.817544854701309e+001, 1.854695254828505e+000, 5.965097614437249e-001}, {2.303811979773995e+002, -1.749817830600921e+001, 1.823454861736828e+000, 5.981174251461863e-001}, {2.174338229345144e+002, -1.684909368569987e+001, 1.792896090563822e+000, 5.997233218906050e-001}, {2.052680630074077e+002, -1.622690586536061e+001, 1.763001219089102e+000, 6.013274651161965e-001}, {1.938335084437520e+002, -1.563038905829366e+001, 1.733753053902124e+000, 6.029298682543182e-001}, {1.830833151646721e+002, -1.505837875682940e+001, 1.705134914485336e+000, 6.045305447297478e-001}, {1.729738794162456e+002, -1.450976794132127e+001, 1.677130615134082e+000, 6.061295079619681e-001}, {1.634643534138694e+002, -1.398350312812421e+001, 1.649724447957303e+000, 6.077267713664593e-001}, {1.545168100337110e+002, -1.347858315714356e+001, 1.622901168714866e+000, 6.093223483560010e-001}, {1.460958410201101e+002, -1.299405538735645e+001, 1.596645980019152e+000, 6.109162523419806e-001}, {1.381683087466291e+002, -1.252901301082886e+001, 1.570944516844422e+000, 6.125084967357085e-001}, {1.307034321865140e+002, -1.208259371162506e+001, 1.545782833329511e+000, 6.140990949497484e-001}, {1.236723317608070e+002, -1.165397599514970e+001, 1.521147387543556e+000, 6.156880603992511e-001}, {1.170481387221509e+002, -1.124237826774210e+001, 1.497025029811643e+000, 6.172754065033039e-001}, {1.108056868876664e+002, -1.084705592255838e+001, 1.473402988869848e+000, 6.188611466862878e-001}, {1.049215003603412e+002, -1.046729998779377e+001, 1.450268860366462e+000, 6.204452943792466e-001}, {9.937360667485126e+001, -1.010243494098661e+001, 1.427610594553690e+000, 6.220278630212706e-001}, {9.414149562320388e+001, -9.751817308176173e+000, 1.405416485272351e+000, 6.236088660608912e-001}, {8.920594267310152e+001, -9.414833664957618e+000, 1.383675158529162e+000, 6.251883169574887e-001}, {8.454903027300053e+001, -9.090899637706979e+000, 1.362375562499242e+000, 6.267662291827165e-001}, {8.015389759910782e+001, -8.779457645122895e+000, 1.341506956391433e+000, 6.283426162219348e-001}, {7.600486056876090e+001, -8.479976561864010e+000, 1.321058901855560e+000, 6.299174915756673e-001}, {7.208717369456534e+001, -8.191949484083496e+000, 1.301021252132937e+000, 6.314908687610693e-001}, {6.838705121588522e+001, -7.914893045725918e+000, 1.281384143757086e+000, 6.330627613134094e-001}, {6.489158353825010e+001, -7.648346098298011e+000, 1.262137987358314e+000, 6.346331827875775e-001}, {6.158865359007402e+001, -7.391868500456181e+000, 1.243273459030718e+000, 6.362021467596020e-001}, {5.846693732764923e+001, -7.145040366143715e+000, 1.224781492411206e+000, 6.377696668281924e-001}, {5.551578714608965e+001, -6.907460700958014e+000, 1.206653270182385e+000, 6.393357566162956e-001}, {5.272524836667247e+001, -6.678746838311933e+000, 1.188880216903054e+000, 6.409004297726797e-001}, {5.008596334489375e+001, -6.458533233599964e+000, 1.171453991082903e+000, 6.424636999735307e-001}, {4.758916027537743e+001, -6.246470825439416e+000, 1.154366478347685e+000, 6.440255809240784e-001}, {4.522661777454572e+001, -6.042226222837200e+000, 1.137609784387375e+000, 6.455860863602415e-001}, {4.299059529145632e+001, -5.845480767898403e+000, 1.121176228050880e+000, 6.471452300502958e-001}, {4.087385651641105e+001, -5.655930154963713e+000, 1.105058335319868e+000, 6.487030257965678e-001}, {3.886958211331500e+001, -5.473283382598018e+000, 1.089248832461124e+000, 6.502594874371552e-001}, {3.697137968566206e+001, -5.297262376818222e+000, 1.073740640458812e+000, 6.518146288476706e-001}, {3.517323133258112e+001, -5.127601210969972e+000, 1.058526868867246e+000, 6.533684639430138e-001}, {3.346950230781325e+001, -4.964045728469465e+000, 1.043600810436168e+000, 6.549210066791725e-001}, {3.185487406683293e+001, -4.806352703624465e+000, 1.028955935203351e+000, 6.564722710550539e-001}, {3.032435829488426e+001, -4.654289580618860e+000, 1.014585885688922e+000, 6.580222711143396e-001}, {2.887326431964138e+001, -4.507633864545065e+000, 1.000484471508354e+000, 6.595710209473816e-001}, {2.749717316692134e+001, -4.366172615618259e+000, 9.866456644267092e-001, 6.611185346931201e-001}, {2.619193414367822e+001, -4.229702099346732e+000, 9.730635937337092e-001, 6.626648265410450e-001}, {2.495363835111331e+001, -4.098027281300438e+000, 9.597325414590835e-001, 6.642099107331829e-001}, {2.377860550726019e+001, -3.970961446183043e+000, 9.466469379706444e-001, 6.657538015661222e-001}, {2.266337233016693e+001, -3.848325831387194e+000, 9.338013576655492e-001, 6.672965133930829e-001}, {2.160467672777616e+001, -3.729949242177387e+000, 9.211905147420358e-001, 6.688380606260099e-001}, {2.059944597797794e+001, -3.615667710660913e+000, 9.088092591623421e-001, 6.703784577377188e-001}, {1.964478435281932e+001, -3.505324163030878e+000, 8.966525727283551e-001, 6.719177192640737e-001}, {1.873796922977412e+001, -3.398768146844634e+000, 8.847155653316714e-001, 6.734558598062153e-001}, {1.787642149968706e+001, -3.295855398505439e+000, 8.729934710857658e-001, 6.749928940328205e-001}, {1.705772475648577e+001, -3.196447779163808e+000, 8.614816451578189e-001, 6.765288366824221e-001}, {1.627958838837261e+001, -3.100412774885759e+000, 8.501755598619032e-001, 6.780637025657604e-001}, {1.553985861020304e+001, -3.007623409690961e+000, 8.390708017061733e-001, 6.795975065681920e-001}, {1.483650268463432e+001, -2.917957920404044e+000, 8.281630679467027e-001, 6.811302636521479e-001}, {1.416760134746515e+001, -2.831299527564026e+000, 8.174481635066645e-001, 6.826619888596361e-001}, {1.353134120238906e+001, -2.747536216213803e+000, 8.069219979743360e-001, 6.841926973148044e-001}, {1.292601643446513e+001, -2.666560582542707e+000, 7.965805827418926e-001, 6.857224042265560e-001}, {1.235001093430196e+001, -2.588269530043726e+000, 7.864200279596690e-001, 6.872511248912188e-001}, {1.180180110648852e+001, -2.512564159266709e+000, 7.764365399883577e-001, 6.887788746952784e-001}, {1.127994572214277e+001, -2.439349541127092e+000, 7.666264186212466e-001, 6.903056691181669e-001}, {1.078308769186625e+001, -2.368534595338057e+000, 7.569860545909910e-001, 6.918315237351190e-001}, {1.030993774691330e+001, -2.300031826875883e+000, 7.475119268835549e-001, 6.933564542200881e-001}, {9.859284768878347e+000, -2.233757299471081e+000, 7.382006005926595e-001, 6.948804763487342e-001}, {9.429977302790977e+000, -2.169630350046961e+000, 7.290487242579775e-001, 6.964036060014773e-001}, {9.020930833273008e+000, -2.107573555178198e+000, 7.200530278593177e-001, 6.979258591666266e-001}, {8.631118936097311e+000, -2.047512534366139e+000, 7.112103204288415e-001, 6.994472519435792e-001}, {8.259568678315045e+000, -1.989375816183700e+000, 7.025174879430843e-001, 7.009678005461024e-001}, {7.905362523492994e+000, -1.933094759596863e+000, 6.939714913370646e-001, 7.024875213056877e-001}, {7.567629408573509e+000, -1.878603376629717e+000, 6.855693643504452e-001, 7.040064306749948e-001}, {7.245546619771394e+000, -1.825838268287014e+000, 6.773082117040159e-001, 7.055245452313755e-001}, {6.938335727792247e+000, -1.774738495351617e+000, 6.691852071288535e-001, 7.070418816804905e-001}, {6.645258809195010e+000, -1.725245462939452e+000, 6.611975915196259e-001, 7.085584568600147e-001}, {6.365614540873520e+000, -1.677302816738685e+000, 6.533426711767790e-001, 7.100742877434374e-001}, {6.098745266050226e+000, -1.630856430792350e+000, 6.456178162181659e-001, 7.115893914439659e-001}, {5.844020409205720e+000, -1.585854168116521e+000, 6.380204585489425e-001, 7.131037852185249e-001}, {5.600848065869145e+000, -1.542245944546190e+000, 6.305480906509974e-001, 7.146174864718647e-001}, {5.368663822551122e+000, -1.499983538515395e+000, 6.231982636835136e-001, 7.161305127607802e-001}, {5.146934075674603e+000, -1.459020574408473e+000, 6.159685861501743e-001, 7.176428817984416e-001}, {4.935153779849062e+000, -1.419312432471077e+000, 6.088567223360871e-001, 7.191546114588450e-001}, {4.732841752485663e+000, -1.380816146638106e+000, 6.018603908156497e-001, 7.206657197813855e-001}, {4.539543342955549e+000, -1.343490383791056e+000, 5.949773631865258e-001, 7.221762249755580e-001}, {4.354826796805688e+000, -1.307295347831567e+000, 5.882054626053450e-001, 7.236861454257876e-001}, {4.178282674581729e+000, -1.272192725811762e+000, 5.815425625070392e-001, 7.251954996964002e-001}, {4.009521634668678e+000, -1.238145618364219e+000, 5.749865853101563e-001, 7.267043065367352e-001}, {3.848175025968628e+000, -1.205118501925514e+000, 5.685355012244482e-001, 7.282125848864084e-001}, {3.693892899386763e+000, -1.173077159790875e+000, 5.621873269947402e-001, 7.297203538807227e-001}, {3.546341740891187e+000, -1.141988619090690e+000, 5.559401247257912e-001, 7.312276328562487e-001}, {3.405206929436500e+000, -1.111821138378985e+000, 5.497920008403084e-001, 7.327344413565636e-001}, {3.270187120186245e+000, -1.082544103575234e+000, 5.437411048209569e-001, 7.342407991381688e-001}, {3.140998924757143e+000, -1.054128048587643e+000, 5.377856283364672e-001, 7.357467261765852e-001}, {3.017369744506951e+000, -1.026544533480591e+000, 5.319238039757309e-001, 7.372522426726416e-001}, {2.899044153352361e+000, -9.997661913653924e-001, 5.261539045099664e-001, 7.387573690589510e-001}, {2.785775964091354e+000, -9.737665947672607e-001, 5.204742416212094e-001, 7.402621260065991e-001}, {2.677333888058895e+000, -9.485203014378125e-001, 5.148831652356617e-001, 7.417665344320410e-001}, {2.573496060829918e+000, -9.240027508565234e-001, 5.093790623710802e-001, 7.432706155042201e-001}, {2.474052809499689e+000, -9.001902757330506e-001, 5.039603564117439e-001, 7.447743906519216e-001}, {2.378804120443931e+000, -8.770600388487887e-001, 4.986255061308111e-001, 7.462778815713633e-001}, {2.287559402475405e+000, -8.545900081840703e-001, 4.933730048957201e-001, 7.477811102340413e-001}, {2.200138517682294e+000, -8.327589429822758e-001, 4.882013798861766e-001, 7.492840988948364e-001}, {2.116368371443866e+000, -8.115463254255343e-001, 4.831091911879579e-001, 7.507868701004001e-001}, {2.036086012815135e+000, -7.909323809234898e-001, 4.780950311870275e-001, 7.522894466978191e-001}, {1.959134838570954e+000, -7.708980017762587e-001, 4.731575236501228e-001, 7.537918518435930e-001}, {1.885366456932190e+000, -7.514247566493858e-001, 4.682953231458762e-001, 7.552941090129114e-001}, {1.814639954656823e+000, -7.324948551533699e-001, 4.635071142767641e-001, 7.567962420092698e-001}, {1.746820019907857e+000, -7.140911050589521e-001, 4.587916109601810e-001, 7.582982749744263e-001}, {1.681779029320273e+000, -6.961969236875881e-001, 4.541475558846255e-001, 7.598002323987182e-001}, {1.619394453544771e+000, -6.787962807595844e-001, 4.495737197402506e-001, 7.613021391317515e-001}, {1.559549975604258e+000, -6.618737004185191e-001, 4.450689006991003e-001, 7.628040203934934e-001}, {1.502134836632424e+000, -6.454142328694089e-001, 4.406319237709913e-001, 7.643059017857750e-001}, {1.447043171573348e+000, -6.294034296210121e-001, 4.362616402141224e-001, 7.658078093042219e-001}, {1.394174498370593e+000, -6.138273354867607e-001, 4.319569270116660e-001, 7.673097693506550e-001}, {1.343432420370009e+000, -5.986724547846682e-001, 4.277166862748373e-001, 7.688118087459539e-001}, {1.294725732732976e+000, -5.839257544439922e-001, 4.235398447916798e-001, 7.703139547434321e-001}, {1.247966881438976e+000, -5.695746253865943e-001, 4.194253534307958e-001, 7.718162350427407e-001}, {1.203072849158344e+000, -5.556068845041990e-001, 4.153721867289367e-001, 7.733186778043174e-001}, {1.159963947754655e+000, -5.420107427847227e-001, 4.113793423526519e-001, 7.748213116644277e-001}, {1.118564587734494e+000, -5.287748063807098e-001, 4.074458407063869e-001, 7.763241657508161e-001}, {1.078802507769043e+000, -5.158880509921915e-001, 4.035707244384177e-001, 7.778272696990009e-001}, {1.040608654311761e+000, -5.033398098623492e-001, 3.997530580345837e-001, 7.793306536692566e-001}, {1.003917267676832e+000, -4.911197646737863e-001, 3.959919274268831e-001, 7.808343483642983e-001}, {9.686655604482745e-001, -4.792179290087323e-001, 3.922864395821423e-001, 7.823383850477332e-001}, {9.347935275729121e-001, -4.676246357897185e-001, 3.886357221304772e-001, 7.838427955633055e-001}, {9.022439775911922e-001, -4.563305281577600e-001, 3.850389230167903e-001, 7.853476123549728e-001}, {8.709622750416257e-001, -4.453265454505745e-001, 3.814952101441460e-001, 7.868528684878805e-001}, {8.408962141922037e-001, -4.346039124857111e-001, 3.780037710512866e-001, 7.883585976702583e-001}, {8.119960630426542e-001, -4.241541312368614e-001, 3.745638126070998e-001, 7.898648342763168e-001}, {7.842141445487288e-001, -4.139689655197202e-001, 3.711745606968078e-001, 7.913716133701791e-001}, {7.575051094416114e-001, -4.040404377634261e-001, 3.678352599695338e-001, 7.928789707309243e-001}, {7.318255766054801e-001, -3.943608140101956e-001, 3.645451735466376e-001, 7.943869428787894e-001}, {7.071339973871145e-001, -3.849225950336849e-001, 3.613035827844493e-001, 7.958955671026191e-001}, {6.833909241888801e-001, -3.757185132023703e-001, 3.581097870611176e-001, 7.974048814886176e-001}, {6.605584479088350e-001, -3.667415147704448e-001, 3.549631035218736e-001, 7.989149249504874e-001}, {6.386004414563145e-001, -3.579847585634845e-001, 3.518628669202130e-001, 8.004257372610424e-001}, {6.174824768960409e-001, -3.494416066345683e-001, 3.488084294219762e-001, 8.019373590853794e-001}, {5.971715000221117e-001, -3.411056120245748e-001, 3.457991604274862e-001, 8.034498320157049e-001}, {5.776360443240841e-001, -3.329705170046958e-001, 3.428344464558457e-001, 8.049631986079188e-001}, {5.588460502815952e-001, -3.250302427624316e-001, 3.399136909984816e-001, 8.064775024200631e-001}, {5.407727810343643e-001, -3.172788818833830e-001, 3.370363144128132e-001, 8.079927880527609e-001}, {5.233887544226954e-001, -3.097106915065160e-001, 3.342017538376066e-001, 8.095091011917578e-001}, {5.066677875548836e-001, -3.023200884505798e-001, 3.314094631326712e-001, 8.110264886527152e-001}, {4.905848756317561e-001, -2.951016406887826e-001, 3.286589128198704e-001, 8.125449984283928e-001}, {4.751160769385945e-001, -2.880500600179367e-001, 3.259495900597049e-001, 8.140646797383735e-001}, {4.602386051186608e-001, -2.811601986672496e-001, 3.232809986635589e-001, 8.155855830815082e-001}, {4.459307014087881e-001, -2.744270408946369e-001, 3.206526591012421e-001, 8.171077602912539e-001}, {4.321715938551395e-001, -2.678456971393555e-001, 3.180641085520652e-001, 8.186312645941041e-001}, {4.189415011741373e-001, -2.614113988398635e-001, 3.155149009832065e-001, 8.201561506713140e-001}, {4.062214913409450e-001, -2.551194905970136e-001, 3.130046072430969e-001, 8.216824747241585e-001}, {3.939936339725952e-001, -2.489654281954533e-001, 3.105328152103881e-001, 8.232102945429569e-001}, {3.822408013278497e-001, -2.429447687817379e-001, 3.080991299323931e-001, 8.247396695801337e-001}, {3.709466702963435e-001, -2.370531663389923e-001, 3.057031738328623e-001, 8.262706610276031e-001}, {3.600957583030854e-001, -2.312863674323255e-001, 3.033445869493500e-001, 8.278033318987772e-001}, {3.496733671825422e-001, -2.256402043990980e-001, 3.010230271918857e-001, 8.293377471155482e-001}, {3.396655214813151e-001, -2.201105889016499e-001, 2.987381706501068e-001, 8.308739736005932e-001}, {3.300589897857049e-001, -2.146935073828444e-001, 2.964897119537793e-001, 8.324120803754020e-001}, {3.208412578073947e-001, -2.093850149119442e-001, 2.942773646665175e-001, 8.339521386644649e-001}, {3.120005459911270e-001, -2.041812297929947e-001, 2.921008617347819e-001, 8.354942220060717e-001}, {3.035256503638822e-001, -1.990783246823497e-001, 2.899599559785713e-001, 8.370384063702406e-001}, {2.954061908395492e-001, -1.940725263380565e-001, 2.878544206816350e-001, 8.385847702843330e-001}, {2.876323002050054e-001, -1.891601020496199e-001, 2.857840501721114e-001, 8.401333949669464e-001}, {2.801948834019232e-001, -1.843373598135978e-001, 2.837486605559522e-001, 8.416843644707666e-001}, {2.730854228772116e-001, -1.796006366227388e-001, 2.817480904497046e-001, 8.432377658350839e-001}, {2.662960891242228e-001, -1.749462945024303e-001, 2.797822018500715e-001, 8.447936892487767e-001}, {2.598196879188098e-001, -1.703707115781422e-001, 2.778508810576263e-001, 8.463522282246335e-001}, {2.536496867746277e-001, -1.658702750316326e-001, 2.759540397160137e-001, 8.479134797859673e-001}, {2.477802128823877e-001, -1.614413725921469e-001, 2.740916159487959e-001, 8.494775446665763e-001}, {2.421839309050305e-001, -1.570799986898184e-001, 2.722635741293099e-001, 8.510445275252113e-001}, {2.368786468237419e-001, -1.527827697979022e-001, 2.704699099458725e-001, 8.526145371758300e-001}, {2.318809810687322e-001, -1.485463921802132e-001, 2.687106490539978e-001, 8.541876868350633e-001}, {2.272033004004703e-001, -1.443013471384416e-001, 2.669578014924510e-001, 8.557900348291792e-001}, {2.228720273934477e-001, -1.401715084130047e-001, 2.652653339510075e-001, 8.573725333555661e-001}, {2.188737474564099e-001, -1.361583230500321e-001, 2.636349618021100e-001, 8.589321368338225e-001}, {2.151938108428024e-001, -1.322546419048931e-001, 2.620646877232015e-001, 8.604692374815954e-001}, {2.118188268906598e-001, -1.284537034390536e-001, 2.605526072877775e-001, 8.619842196559904e-001}, {2.087366333201549e-001, -1.247491073246053e-001, 2.590969039651040e-001, 8.634774600465008e-001}, {2.059361625352860e-001, -1.211347872081701e-001, 2.576958444119279e-001, 8.649493278615940e-001}, {2.034074149378808e-001, -1.176049878458108e-001, 2.563477740702578e-001, 8.664001850093022e-001}, {2.011413298783185e-001, -1.141542415902366e-001, 2.550511130233508e-001, 8.678303862720165e-001}, {1.991298125068586e-001, -1.107773498429863e-001, 2.538043521210758e-001, 8.692402794757654e-001}, {1.973655647896050e-001, -1.074693611653381e-001, 2.526060493134096e-001, 8.706302056542128e-001}, {1.958421213759138e-001, -1.042255560507937e-001, 2.514548262377977e-001, 8.720004992075637e-001}, {1.945537958776381e-001, -1.010414298773090e-001, 2.503493649859753e-001, 8.733514880566440e-001}, {1.934955970749939e-001, -9.791267666147004e-002, 2.492884050660330e-001, 8.746834937923431e-001}, {1.926632043856072e-001, -9.483517540616522e-002, 2.482707405530666e-001, 8.759968318205693e-001}, {1.920530013650005e-001, -9.180497812140505e-002, 2.472952174051379e-001, 8.772918115029676e-001}, {1.916619394589189e-001, -8.881829486636758e-002, 2.463607309170215e-001, 8.785687362935541e-001}, {1.914875823983919e-001, -8.587148427990259e-002, 2.454662233501901e-001, 8.798279038713941e-001}, {1.915280993815957e-001, -8.296104268947961e-002, 2.446106816792321e-001, 8.810696062695512e-001}, {1.917821599981230e-001, -8.008359226919655e-002, 2.437931354640896e-001, 8.822941300004101e-001}, {1.922489958278585e-001, -7.723587356759790e-002, 2.430126548622808e-001, 8.835017561775065e-001}, {1.929283914742223e-001, -7.441473634943605e-002, 2.422683487355767e-001, 8.846927606340636e-001}, {1.938205682251700e-001, -7.161712923792232e-002, 2.415593628605291e-001, 8.858674140382891e-001}, {1.949263025484482e-001, -6.884009487087414e-002, 2.408848782619473e-001, 8.870259820056076e-001}, {1.962468368843349e-001, -6.608076049330205e-002, 2.402441096090453e-001, 8.881687252078944e-001}, {1.977839043493731e-001, -6.333633184166496e-002, 2.396363037192911e-001, 8.892958994798933e-001}, {1.995397017281764e-001, -6.060408601603758e-002, 2.390607381333165e-001, 8.904077559228296e-001}, {2.015168884164766e-001, -5.788136574944486e-002, 2.385167197765595e-001, 8.915045410054051e-001}, {2.037186493235506e-001, -5.516557435579349e-002, 2.380035836869622e-001, 8.925864966622013e-001}, {2.061485578928091e-001, -5.245416769018642e-002, 2.375206918021687e-001, 8.936538603896150e-001}, {2.088107303179244e-001, -4.974465205292488e-002, 2.370674318361935e-001, 8.947068653393980e-001}, {2.117096846999846e-001, -4.703457622106781e-002, 2.366432161836148e-001, 8.957457404098935e-001}, {2.148505149907584e-001, -4.432153007089228e-002, 2.362474809142938e-001, 8.967707103350046e-001}, {2.182386749730879e-001, -4.160313569771003e-002, 2.358796847823731e-001, 8.977819957710477e-001}, {2.218802015852066e-001, -3.887704757707568e-002, 2.355393083306903e-001, 8.987798133814337e-001}, {2.257816115052553e-001, -3.614094572355721e-002, 2.352258530014959e-001, 8.997643759193978e-001}, {2.299498407093479e-001, -3.339253121132149e-002, 2.349388403129885e-001, 9.007358923086878e-001}, {2.343924725771772e-001, -3.062952596424444e-002, 2.346778110852589e-001, 9.016945677223748e-001}, {2.391175338601718e-001, -2.784966495255222e-002, 2.344423246756947e-001, 9.026406036597976e-001}, {2.441335689704040e-001, -2.505069440041885e-002, 2.342319582824293e-001, 9.035741980217020e-001}, {2.494466737102023e-001, -2.223033411770663e-002, 2.340463061820594e-001, 9.044955451836203e-001}, {2.550861106065699e-001, -1.938660447233538e-002, 2.338849799791759e-001, 9.054048360675704e-001}, {2.610094533269595e-001, -1.651647391820168e-002, 2.337476039501077e-001, 9.063022582120580e-001}, {2.672873914645409e-001, -1.361879651128243e-002, 2.336338247023430e-001, 9.071879958403298e-001}, {2.738106591896983e-001, -1.068971575200308e-002, 2.335432928777375e-001, 9.080622299280794e-001}, {2.804606750369759e-001, -7.726659110718780e-003, 2.334756804033219e-001, 9.089251382668985e-001}, {2.904290562079965e-001, -4.764098488806071e-003, 2.334307556704802e-001, 9.097768955293063e-001}, {2.933750710558286e-001, -1.656982949129257e-003, 2.334078134409505e-001, 9.106176733264468e-001}, {3.050168191487940e-001, 1.350589639466686e-003, 2.334074430077944e-001, 9.114476402584719e-001}, {3.126013688318755e-001, 4.484047200702626e-003, 2.334283798227455e-001, 9.122669621103098e-001}, {3.215543582266510e-001, 7.641858728022285e-003, 2.334709180402272e-001, 9.130758015729857e-001}, {3.309493854379925e-001, 1.084606285102256e-002, 2.335346788693272e-001, 9.138743186727137e-001}, {3.408036404253549e-001, 1.409903898921036e-002, 2.336194205790645e-001, 9.146626706292474e-001}, {3.511350954219710e-001, 1.740319488595970e-002, 2.337249122941095e-001, 9.154410119512660e-001}, {3.619625548115306e-001, 2.076096799975756e-002, 2.338509336517329e-001, 9.162094944895526e-001}, {3.733053046371622e-001, 2.417483133717817e-002, 2.339972744575906e-001, 9.169682674889458e-001}, {3.851835101117528e-001, 2.764728816995566e-002, 2.341637343834541e-001, 9.177174776391662e-001}, {3.976175439013757e-001, 3.118088055196995e-002, 2.343501226196413e-001, 9.184572691244551e-001}, {4.109386287917567e-001, 3.477533291071943e-002, 2.345562634056419e-001, 9.191877836721225e-001}, {4.252825118499027e-001, 3.843116371032156e-002, 2.347819891940276e-001, 9.199091605999706e-001}, {4.408049129756115e-001, 4.214792375866371e-002, 2.350271518933623e-001, 9.206215368626223e-001}, {4.576846245311496e-001, 4.630547834114645e-002, 2.353172753172934e-001, 9.213904696290003e-001}, {4.784513590715939e-001, 5.060507038628476e-002, 2.356363599220260e-001, 9.221630514821407e-001}, {4.920827258189274e-001, 5.519541297814200e-002, 2.359854289691031e-001, 9.229393998641423e-001}, {5.178937679266293e-001, 5.981545467360165e-002, 2.363668043436880e-001, 9.237196381740548e-001}, {5.391049374883102e-001, 6.475414966613934e-002, 2.367809391058319e-001, 9.245038963244220e-001}, {5.634449341225508e-001, 6.989029053033601e-002, 2.372302039790004e-001, 9.252923108029000e-001}, {5.897153325658239e-001, 7.526860985601419e-002, 2.377162421944522e-001, 9.260850256994325e-001}, {6.181112209433559e-001, 8.090819466323959e-002, 2.382410055571628e-001, 9.268821929717398e-001}, {6.488525677875635e-001, 8.683004454631531e-002, 2.388066021212003e-001, 9.276839731306028e-001}, {6.821878414375597e-001, 9.305731296729698e-002, 2.394153123100448e-001, 9.284905359221661e-001}, {7.183985201026177e-001, 9.961558066647835e-002, 2.400696071150384e-001, 9.293020610812173e-001}, {7.578044037403948e-001, 1.065331716202092e-001, 2.407721686690386e-001, 9.301187391645777e-001}, {8.007698251219902e-001, 1.138415194678852e-001, 2.415259135709149e-001, 9.309407724751908e-001}, {8.477112195704598e-001, 1.215755901968623e-001, 2.423340194130125e-001, 9.317683760891560e-001}, {8.991059063712774e-001, 1.297743778800449e-001, 2.431999550154734e-001, 9.326017789999588e-001}, {9.555029755126038e-001, 1.384814779165823e-001, 2.441275150268687e-001, 9.334412253964867e-001}, {1.017535937344632e+000, 1.477457654552941e-001, 2.451208595937071e-001, 9.342869760943128e-001}, {1.085938205408345e+000, 1.576221875223210e-001, 2.461845600471582e-001, 9.351393101430466e-001}, {1.161561582275224e+000, 1.681727002303767e-001, 2.473236516456673e-001, 9.359985266367291e-001}, {1.245398446129048e+000, 1.794673774990541e-001, 2.485436946915644e-001, 9.368649467591372e-001}, {1.338608148309784e+000, 1.915857290298620e-001, 2.498508455805677e-001, 9.377389161019090e-001}, {1.442548258110234e+000, 2.046182699257000e-001, 2.512519396760876e-001, 9.386208073007041e-001}, {1.563550207326141e+000, 2.186179243163746e-001, 2.527547076453380e-001, 9.395110230435863e-001}, {1.684329410181329e+000, 2.339579695432694e-001, 2.543669871107666e-001, 9.404099995167338e-001}, {1.840290137060555e+000, 2.502660355294551e-001, 2.561000525033712e-001, 9.413182103660764e-001}, {2.005174237802519e+000, 2.682089305034622e-001, 2.579624203753087e-001, 9.422361716947373e-001}, {2.195430692141073e+000, 2.877169741014608e-001, 2.599675919987584e-001, 9.431644460877734e-001}, {2.412904324653398e+000, 3.090556749171306e-001, 2.621289837025392e-001, 9.441036504515273e-001}, {2.662767304254523e+000, 3.324773127469931e-001, 2.644622565363322e-001, 9.450544623460853e-001}, {2.951438780835094e+000, 3.582796665096927e-001, 2.669853329547631e-001, 9.460176287724212e-001}, {3.286954018925421e+000, 3.868162802973410e-001, 2.697188268971239e-001, 9.469939763450140e-001}, {3.679465741376691e+000, 4.185095197683239e-001, 2.726865770992523e-001, 9.479844234324657e-001}, {4.141931236862925e+000, 4.538673480408977e-001, 2.759163138940035e-001, 9.489899947466461e-001}, {4.691069298400168e+000, 4.935050283817702e-001, 2.794405007063843e-001, 9.500118390046373e-001}, {5.348707073258958e+000, 5.381735273260513e-001, 2.832974063174888e-001, 9.510512504832676e-001}, {6.143705486256990e+000, 5.887970106525733e-001, 2.875324861260081e-001, 9.521096955549476e-001}, {7.114743725479455e+000, 6.465229722160121e-001, 2.922001819000289e-001, 9.531888456688770e-001}, {8.321324877449055e+000, 7.127119364606617e-001, 2.973664920699020e-001, 9.542906187727378e-001}, {9.814008057546337e+000, 7.895125341007474e-001, 3.031108408227720e-001, 9.554172319331690e-001}, {1.173472061857719e+001, 8.786331494420789e-001, 3.095347921261885e-001, 9.565712690270144e-001}, {1.420449343150175e+001, 9.836535262794647e-001, 3.167598605019126e-001, 9.577557700959503e-001}, {1.746385659351800e+001, 1.108209112465122e+000, 3.249454080118249e-001, 9.589743460224295e-001}, {2.185981459948901e+001, 1.257510034299458e+000, 3.342947309034364e-001, 9.602313398961775e-001}, {2.794948473262666e+001, 1.438394626077943e+000, 3.450759860826211e-001, 9.615320435137206e-001}, {3.665958868667153e+001, 1.659932031051734e+000, 3.576504808355101e-001, 9.628830054950714e-001}, {4.961877309657803e+001, 1.933871190036903e+000, 3.725190487198159e-001, 9.642924773126242e-001}, {6.984375504183272e+001, 2.274822138787811e+000, 3.903995922147242e-001, 9.657710805637312e-001}, {1.034774217208668e+002, 2.696074492740470e+000, 4.123731761320471e-001, 9.673328436111374e-001}, {1.581230920983324e+002, 3.266020040491545e+000, 4.399786317612097e-001, 9.689968887258295e-001}, {2.515136935955693e+002, 4.116442075807741e+000, 4.756363207390761e-001, 9.707903327720178e-001}, {4.255666011618887e+002, 5.543864685060169e+000, 5.231660521339230e-001, 9.727536601397832e-001}, {7.825530244524709e+002, 6.095169952013788e+000, 5.741651534308233e-001, 9.743751414991224e-001}, {1.601387467228535e+003, 5.526394469524922e+000, 6.475446574462375e-001, 9.761711567563309e-001}, {3.517379521473430e+003, 4.655838112548601e+000, 7.599439282112659e-001, 9.783035737380645e-001}, {7.962846678376361e+003, 9.518793744286704e+000, 9.225514220968945e-001, 9.807363487766135e-001}, {1.824488327643979e+004, 2.078256392728106e+001, 1.131128321260709e+000, 9.829773592443036e-001}, {4.540306810221719e+004, 1.488976251891372e+001, 1.387654000750968e+000, 9.848111708264992e-001}, {1.145958554994318e+005, -3.053821008062807e+001, 1.732038158743400e+000, 9.864872284392217e-001}, {2.714454865411694e+005, -3.766348045575876e+001, 2.199850765561659e+000, 9.882925910565035e-001}, {6.114567935601252e+005, 3.983023186009633e+002, 2.891002115453413e+000, 9.904976309221367e-001}, {1.241850100485509e+006, 7.637348298339904e+002, 3.505557353710815e+000, 9.917024335783234e-001}, {2.901091663911621e+006, 2.287151249154257e+002, 3.996356201581334e+000, 9.922645331286384e-001}, {8.629777329944847e+006, -1.301634709362246e+003, 5.052771746981934e+000, 9.932357014649955e-001}, {2.912091392721209e+007, -6.531467250818030e+003, 6.921517889213511e+000, 9.942966215510268e-001}, {9.412813865309173e+007, -5.286393541194419e+003, 8.979056034617720e+000, 9.957010756859428e-001}, {2.036070618782550e+008, 1.028469165095266e+004, 1.130107943827922e+001, 9.966743300764792e-001}, {1.674526211291653e+012, -7.945254783461587e+007, 9.414997405411956e+002, 9.973127499065179e-001}};
        constant Real[Ninterval, 4] svcoef = {{-1.167161242280822e+010, 1.878696788040639e+006, -3.414627808635271e+002, 1.257192752866231e+000}, {-1.035354888344887e+010, 1.770211704883063e+006, -3.314921184815092e+002, 1.256278981521677e+000}, {-9.010871046855169e+009, 1.675234821593873e+006, -3.233186928839511e+002, 1.255515159251215e+000}, {-7.670087323896805e+009, 1.503104694936961e+006, -3.056454829469166e+002, 1.253788981093666e+000}, {-6.528525227742667e+009, 1.348617412883818e+006, -2.889263517656612e+002, 1.252068489141656e+000}, {-5.556819130397623e+009, 1.209977756821469e+006, -2.731107664395464e+002, 1.250353786912934e+000}, {-4.729570822211771e+009, 1.085555117462545e+006, -2.581498340050509e+002, 1.248644873308041e+000}, {-4.025383319196040e+009, 9.738978630494837e+005, -2.439979849174128e+002, 1.246941821048949e+000}, {-3.425938223747980e+009, 8.736974103745150e+005, -2.306117633794419e+002, 1.245244670239726e+000}, {-2.915696321437560e+009, 7.837826255347068e+005, -2.179503198145804e+002, 1.243553497923841e+000}, {-2.481383208327286e+009, 7.030988445381357e+005, -2.059747132443750e+002, 1.241868349123961e+000}, {-2.111686390084153e+009, 6.307004543248878e+005, -1.946483220668824e+002, 1.240189304980033e+000}, {-1.797034775339657e+009, 5.657386555897944e+005, -1.839360670319048e+002, 1.238516382333624e+000}, {-1.529214800743476e+009, 5.074499659892993e+005, -1.738049564593241e+002, 1.236849634718179e+000}, {-1.301273763706718e+009, 4.551518279503796e+005, -1.642239738649408e+002, 1.235189150544434e+000}, {-1.107273660839951e+009, 4.082289549017351e+005, -1.551633835517684e+002, 1.233534955827648e+000}, {-9.421731741085697e+008, 3.661311815312429e+005, -1.465954002475946e+002, 1.231887143503433e+000}, {-8.023146343545597e+008, 3.283866390101237e+005, -1.384935880133614e+002, 1.230245745364984e+000}, {-6.852307594168044e+008, 2.946092675025202e+005, -1.308334802778386e+002, 1.228610827813656e+000}, {-5.870901801301618e+008, 2.644005082934081e+005, -1.235917413237229e+002, 1.226982459618363e+000}, {-5.047068181055782e+008, 2.378107583024218e+005, -1.168778656810235e+002, 1.225392995014713e+000}, {-4.351080450261749e+008, 2.149782115590958e+005, -1.108542675702452e+002, 1.223894955882774e+000}, {-3.754631816409877e+008, 1.944610872742684e+005, -1.051729507319718e+002, 1.222412562340188e+000}, {-3.243003876591392e+008, 1.760126494519494e+005, -9.981271620297964e+001, 1.220945658781368e+000}, {-2.803704024187839e+008, 1.594135405147079e+005, -9.475377996554323e+001, 1.219494091303989e+000}, {-2.426161580886689e+008, 1.444689114102572e+005, -8.997767687518686e+001, 1.218057707682420e+000}, {-2.101376500670390e+008, 1.310052300243061e+005, -8.546716076385087e+001, 1.216636357341712e+000}, {-1.821729409586426e+008, 1.188682298364962e+005, -8.120612512230242e+001, 1.215229891332141e+000}, {-1.580713395011833e+008, 1.079202781593172e+005, -7.717951465716868e+001, 1.213838162304302e+000}, {-1.372810899764895e+008, 9.803887891483332e+004, -7.337325952167046e+001, 1.212461024484717e+000}, {-1.193305568497935e+008, 8.911463290966404e+004, -6.977419934441511e+001, 1.211098333651978e+000}, {-1.038177842631264e+008, 8.104994158345387e+004, -6.637002475122108e+001, 1.209749947113391e+000}, {-9.040040036718562e+007, 7.375769881825958e+004, -6.314921765813308e+001, 1.208415723682116e+000}, {-7.878443856624676e+007, 6.715992689454205e+004, -6.010099316854539e+001, 1.207095523654803e+000}, {-6.871974414947894e+007, 6.118702847513706e+004, -5.721525561719147e+001, 1.205789208789701e+000}, {-5.999133752771001e+007, 5.577663404305128e+004, -5.448254540510249e+001, 1.204496642285233e+000}, {-5.241543869142986e+007, 5.087295055300280e+004, -5.189400007463385e+001, 1.203217688759031e+000}, {-4.583415581126005e+007, 4.642596046130997e+004, -4.944131149429202e+001, 1.201952214227427e+000}, {-4.011228675603470e+007, 4.239086070618492e+004, -4.711669043018321e+001, 1.200700086085366e+000}, {-3.513311357163611e+007, 3.872739999770453e+004, -4.491282933447061e+001, 1.199461173086765e+000}, {-3.079702502223688e+007, 3.539954399663577e+004, -4.282287446858340e+001, 1.198235345325283e+000}, {-2.701759418906320e+007, 3.237483582224888e+004, -4.084039003202591e+001, 1.197022474215511e+000}, {-2.372076973145581e+007, 2.962417023636633e+004, -3.895933616247480e+001, 1.195822432474547e+000}, {-2.084264185672811e+007, 2.712136254406240e+004, -3.717403962809318e+001, 1.194635094103993e+000}, {-1.832795661554030e+007, 2.484284118361123e+004, -3.547917043105402e+001, 1.193460334372309e+000}, {-1.612910315484070e+007, 2.276740549221483e+004, -3.386972055424017e+001, 1.192298029797559e+000}, {-1.420493817465342e+007, 2.087595838808486e+004, -3.234098209159364e+001, 1.191148058130523e+000}, {-1.251978525903208e+007, 1.915127628793518e+004, -3.088852777905769e+001, 1.190010298338169e+000}, {-1.104283839085355e+007, 1.757784331955985e+004, -2.950819407825404e+001, 1.188884630587475e+000}, {-9.747385274891593e+006, 1.614165584070271e+004, -2.819606332104048e+001, 1.187770936229595e+000}, {-8.610258724756623e+006, 1.483007183595590e+004, -2.694844841479200e+001, 1.186669097784365e+000}, {-7.611347860356514e+006, 1.363167257285838e+004, -2.576187830531837e+001, 1.185578998925132e+000}, {-6.733210449194347e+006, 1.253614352179986e+004, -2.463308451072768e+001, 1.184500524463906e+000}, {-5.960664814163764e+006, 1.153415408137276e+004, -2.355898800825970e+001, 1.183433560336828e+000}, {-5.280521597699627e+006, 1.061726682066825e+004, -2.253668783797127e+001, 1.182377993589938e+000}, {-4.681293638655025e+006, 9.777843148067454e+003, -2.156344973986173e+001, 1.181333712365246e+000}, {-4.152968837995382e+006, 9.008964808911329e+003, -2.063669596087836e+001, 1.180300605887091e+000}, {-3.686828565114908e+006, 8.304365766483923e+003, -1.975399574397403e+001, 1.179278564448786e+000}, {-3.275266327776558e+006, 7.658365108421354e+003, -1.891305611231615e+001, 1.178267479399534e+000}, {-2.911625574521497e+006, 7.065807076565250e+003, -1.811171341970251e+001, 1.177267243131628e+000}, {-2.590114661892789e+006, 6.522019111917333e+003, -1.734792596167091e+001, 1.176277749067896e+000}, {-2.305650275370526e+006, 6.022753490926198e+003, -1.661976587626541e+001, 1.175298891649416e+000}, {-2.053788951097264e+006, 5.564153643271859e+003, -1.592541287535999e+001, 1.174330566323474e+000}, {-1.830650417503742e+006, 5.142717130166380e+003, -1.526314768679637e+001, 1.173372669531767e+000}, {-1.632819252290972e+006, 4.755254885285894e+003, -1.463134563215786e+001, 1.172425098698841e+000}, {-1.457310558157280e+006, 4.398869159670537e+003, -1.402847159348952e+001, 1.171487752220765e+000}, {-1.301502456237975e+006, 4.070921570400577e+003, -1.345307431038991e+001, 1.170560529454023e+000}, {-1.163093581080859e+006, 3.769009894238661e+003, -1.290378157209061e+001, 1.169643330704624e+000}, {-1.040059054359292e+006, 3.490944949502028e+003, -1.237929552902521e+001, 1.168736057217427e+000}, {-9.306220845951777e+005, 3.234732704043220e+003, -1.187838848926460e+001, 1.167838611165672e+000}, {-8.332175097131218e+005, 2.998554122300491e+003, -1.139989865622131e+001, 1.166950895640710e+000}, {-7.464663784985374e+005, 2.780749557180561e+003, -1.094272640598105e+001, 1.166072814641923e+000}, {-6.691556891366435e+005, 2.579804982448421e+003, -1.050583075721938e+001, 1.165204273066841e+000}, {-6.002151747908264e+005, 2.394337509561513e+003, -1.008822589061445e+001, 1.164345176701438e+000}, {-5.387000166680566e+005, 2.223083665065604e+003, -9.688978048092071e+000, 1.163495432210602e+000}, {-4.837771715710186e+005, 2.064889150637393e+003, -9.307202614248977e+000, 1.162654947128783e+000}, {-4.347104808441157e+005, 1.918698359847450e+003, -8.942061260376542e+000, 1.161823629850803e+000}, {-3.908482209910512e+005, 1.783545349089750e+003, -8.592759344255985e+000, 1.161001389622835e+000}, {-3.516161249361212e+005, 1.658547033549894e+003, -8.258543553706538e+000, 1.160188136533529e+000}, {-3.165035478016587e+005, 1.542893819027835e+003, -7.938699394039895e+000, 1.159383781505299e+000}, {-2.850598559817247e+005, 1.435844989118608e+003, -7.632549251081720e+000, 1.158588236285744e+000}, {-2.568853836992341e+005, 1.336721560377161e+003, -7.339450190895453e+000, 1.157801413439231e+000}, {-2.316252113167582e+005, 1.244900875868593e+003, -7.058792075603022e+000, 1.157023226338597e+000}, {-2.089652497780896e+005, 1.159812303640403e+003, -6.789995822552769e+000, 1.156253589156990e+000}, {-1.886260028704987e+005, 1.080931941447458e+003, -6.532511605482741e+000, 1.155492416859840e+000}, {-1.703595756576603e+005, 1.007779146610457e+003, -6.285817340840077e+000, 1.154739625196958e+000}, {-1.539455303444279e+005, 9.399125006097200e+002, -6.049417130211369e+000, 1.153995130694738e+000}, {-1.391878353597992e+005, 8.769265149992824e+002, -5.822839859171091e+000, 1.153258850648495e+000}, {-1.259118679027340e+005, 8.184484746241886e+002, -5.605637854535695e+000, 1.152530703114902e+000}, {-1.139626359191102e+005, 7.641359827753867e+002, -5.397385660782585e+000, 1.151810606904539e+000}, {-1.032013550357087e+005, 7.136737153582978e+002, -5.197678765965424e+000, 1.151098481574539e+000}, {-9.350496389204710e+004, 6.667719731724156e+002, -5.006132610481427e+000, 1.150394247421343e+000}, {-8.476330492175093e+004, 6.231638226625424e+002, -4.822381409307250e+000, 1.149697825473546e+000}, {-7.687824464670279e+004, 5.826035987524953e+002, -4.646077237885241e+000, 1.149009137484835e+000}, {-6.976204933005408e+004, 5.448649346498113e+002, -4.476889054980945e+000, 1.148328105927014e+000}, {-6.333647369305392e+004, 5.097393221858990e+002, -4.314501843141601e+000, 1.147654653983123e+000}, {-5.753143008675712e+004, 4.770344420945451e+002, -4.158615740492024e+000, 1.146988705540629e+000}, {-5.228434076819440e+004, 4.465730200783362e+002, -4.008945287331126e+000, 1.146330185184698e+000}, {-4.753914508465970e+004, 4.181914609969991e+002, -3.865218653140307e+000, 1.145679018191555e+000}, {-4.324567909424259e+004, 3.917388090779465e+002, -3.727176949977069e+000, 1.145035130521896e+000}, {-3.935897394044224e+004, 3.670756753534608e+002, -3.594573558074095e+000, 1.144398448814387e+000}, {-3.583878264355163e+004, 3.440733666158342e+002, -3.467173508311998e+000, 1.143768900379221e+000}, {-3.264892946834709e+004, 3.226129150306674e+002, -3.344752869211387e+000, 1.143146413191748e+000}, {-2.975703079636661e+004, 3.025844320516610e+002, -3.227098213411754e+000, 1.142530915886152e+000}, {-2.713396553903700e+004, 2.838862715090525e+002, -3.114006060655961e+000, 1.141922337749209e+000}, {-2.475361222892791e+004, 2.664244520011594e+002, -3.005282394893281e+000, 1.141320608714076e+000}, {-2.259246929329500e+004, 2.501119931930805e+002, -2.900742176144558e+000, 1.140725659354155e+000}, {-2.062942089795645e+004, 2.348684065079063e+002, -2.800208902194028e+000, 1.140137420877001e+000}, {-1.884547713413705e+004, 2.206191719071575e+002, -2.703514177914426e+000, 1.139555825118279e+000}, {-1.722353739702725e+004, 2.072952590638357e+002, -2.610497311259647e+000, 1.138980804535772e+000}, {-1.574820565669981e+004, 1.948327142632579e+002, -2.521004936730899e+000, 1.138412292203431e+000}, {-1.440561272630618e+004, 1.831722652704424e+002, -2.434890653346931e+000, 1.137850221805471e+000}, {-1.318326353872169e+004, 1.722589632456998e+002, -2.352014682990452e+000, 1.137294527630510e+000}, {-1.206988185328727e+004, 1.620418365675164e+002, -2.272243543846235e+000, 1.136745144565746e+000}, {-1.105529962362607e+004, 1.524736029253813e+002, -2.195449748531088e+000, 1.136202008091175e+000}, {-1.013034181716861e+004, 1.435103830129309e+002, -2.121511510792387e+000, 1.135665054273839e+000}, {-9.286710899115280e+003, 1.351114286113316e+002, -2.050312467770768e+000, 1.135134219762123e+000}, {-8.516925185585011e+003, 1.272389163739304e+002, -1.981741426500199e+000, 1.134609441780069e+000}, {-7.814215167392292e+003, 1.198576991982375e+002, -1.915692106619513e+000, 1.134090658121731e+000}, {-7.172456309463685e+003, 1.129351140608649e+002, -1.852062910163217e+000, 1.133577807145568e+000}, {-6.586111850428207e+003, 1.064408049999422e+002, -1.790756700400648e+000, 1.133070827768850e+000}, {-6.050171835345558e+003, 1.003465460629108e+002, -1.731680587420741e+000, 1.132569659462107e+000}, {-5.560092621539761e+003, 9.462607475690936e+001, -1.674745725978995e+000, 1.132074242243597e+000}, {-5.111765690517253e+003, 8.925496645655235e+001, -1.619867130786789e+000, 1.131584516673807e+000}, {-4.701461004393561e+003, 8.421047987386929e+001, -1.566963488558861e+000, 1.131100423849972e+000}, {-4.325801937265625e+003, 7.947145063101340e+001, -1.515956992240450e+000, 1.130621905400621e+000}, {-3.981719970706350e+003, 7.501816080096197e+001, -1.466773172300048e+000, 1.130148903480148e+000}, {-3.666433998378832e+003, 7.083224791831155e+001, -1.419340747082342e+000, 1.129681360763410e+000}, {-3.377417860032481e+003, 6.689659881427446e+001, -1.373591471646139e+000, 1.129219220440331e+000}, {-3.112376370352019e+003, 6.319526122840436e+001, -1.329459999019620e+000, 1.128762426210546e+000}, {-2.869224586596547e+003, 5.971336256002279e+001, -1.286883747719661e+000, 1.128310922278051e+000}, {-2.646066878698053e+003, 5.643703113312434e+001, -1.245802774455418e+000, 1.127864653345879e+000}, {-2.441179052490817e+003, 5.335332525583549e+001, -1.206159654087104e+000, 1.127423564610798e+000}, {-2.252992087969359e+003, 5.045016740473830e+001, -1.167899365297260e+000, 1.126987601758015e+000}, {-2.080079180685374e+003, 4.771628543389871e+001, -1.130969182338600e+000, 1.126556710955913e+000}, {-1.921138426792311e+003, 4.514114975040469e+001, -1.095318569370784e+000, 1.126130838850798e+000}, {-1.774986715083261e+003, 4.271493085367734e+001, -1.060899085833134e+000, 1.125709932561658e+000}, {-1.640543981075735e+003, 4.042844155788508e+001, -1.027664288390328e+000, 1.125293939674950e+000}, {-1.516825450176648e+003, 3.827309661545938e+001, -9.955696444190738e-001, 1.124882808239393e+000}, {-1.402934644797308e+003, 3.624087344333366e+001, -9.645724472776511e-001, 1.124476486760781e+000}, {-1.298051190829407e+003, 3.432426598985062e+001, -9.346317324287821e-001, 1.124074924196815e+000}, {-1.201427843555840e+003, 3.251625738258812e+001, -9.057082044918685e-001, 1.123678069951939e+000}, {-1.112382332639507e+003, 3.081028253305661e+001, -8.777641610741774e-001, 1.123285873872205e+000}, {-1.030289990482704e+003, 2.920019517275006e+001, -8.507634226388910e-001, 1.122898286240140e+000}, {-9.545814037954651e+002, 2.768024521664869e+001, -8.246712685981502e-001, 1.122515257769640e+000}, {-8.847352179903480e+002, 2.624504692644900e+001, -7.994543708220493e-001, 1.122136739600862e+000}, {-8.202750157624278e+002, 2.488955712219283e+001, -7.750807349200021e-001, 1.121762683295148e+000}, {-7.607643646864409e+002, 2.360905024813576e+001, -7.515196415313357e-001, 1.121393040829949e+000}, {-7.058044555340915e+002, 2.239909971536202e+001, -7.287415925975793e-001, 1.121027764593767e+000}, {-6.550297990234562e+002, 2.125555560066715e+001, -7.067182574172425e-001, 1.120666807381118e+000}, {-6.081054766904550e+002, 2.017452705997341e+001, -6.854224233914087e-001, 1.120310122387493e+000}, {-5.647251310415043e+002, 1.915236652516444e+001, -6.648279487465394e-001, 1.119957663204345e+000}, {-5.246073073245650e+002, 1.818565101191613e+001, -6.449097155478253e-001, 1.119609383814090e+000}, {-4.874944380246073e+002, 1.727117028068883e+001, -6.256435880489638e-001, 1.119265238585110e+000}, {-4.531499690131564e+002, 1.640591080457108e+001, -6.070063697166392e-001, 1.118925182266775e+000}, {-4.213569929980251e+002, 1.558704419185258e+001, -5.889757645456558e-001, 1.118589169984485e+000}, {-3.919163181558395e+002, 1.481191450974968e+001, -5.715303387652269e-001, 1.118257157234713e+000}, {-3.646450553664385e+002, 1.407802747939932e+001, -5.546494850349343e-001, 1.117929099880068e+000}, {-3.393752203043368e+002, 1.338304002382417e+001, -5.383133878866853e-001, 1.117604954144370e+000}, {-3.159524776279070e+002, 1.272475057653111e+001, -5.225029907903754e-001, 1.117284676607735e+000}, {-2.942348741380071e+002, 1.210108978046953e+001, -5.071999645835167e-001, 1.116968224201675e+000}, {-2.740919863183586e+002, 1.151011260918757e+001, -4.923866777802942e-001, 1.116655554204213e+000}, {-2.554037558983814e+002, 1.094998990086499e+001, -4.780461674068305e-001, 1.116346624234999e+000}, {-2.380598009202213e+002, 1.041900155948028e+001, -4.641621119578042e-001, 1.116041392250460e+000}, {-2.219584008845426e+002, 9.915529102386795e+000, -4.507188047600041e-001, 1.115739816538935e+000}, {-2.070060510539739e+002, 9.438050073207766e+000, -4.377011294382324e-001, 1.115441855715845e+000}, {-1.931164504418314e+002, 8.985131047005984e+000, -4.250945352192337e-001, 1.115147468718868e+000}, {-1.802102310100627e+002, 8.555423116398927e+000, -4.128850148149304e-001, 1.114856614803115e+000}, {-1.682141656369273e+002, 8.147655878814247e+000, -4.010590817995265e-001, 1.114569253536338e+000}, {-1.570607576689517e+002, 7.760632934266394e+000, -3.896037500077707e-001, 1.114285344794130e+000}, {-1.466877631682851e+002, 7.393227269003488e+000, -3.785065133766781e-001, 1.114004848755154e+000}, {-1.370377407366794e+002, 7.044376929114794e+000, -3.677553266988977e-001, 1.113727725896368e+000}, {-1.280577195832078e+002, 6.713081210827975e+000, -3.573385873661134e-001, 1.113453936988272e+000}, {-1.196987359562168e+002, 6.398396598589270e+000, -3.472451175418697e-001, 1.113183443090161e+000}, {-1.119156299903978e+002, 6.099433714827095e+000, -3.374641476858871e-001, 1.112916205545393e+000}, {-1.046666771658784e+002, 5.815353767791457e+000, -3.279853001222415e-001, 1.112652185976661e+000}, {-9.791330185485612e+001, 5.545365486858695e+000, -3.187985736676302e-001, 1.112391346281281e+000}, {-9.161985391952307e+001, 5.288722384756864e+000, -3.098943289750698e-001, 1.112133648626488e+000}, {-8.575340530044392e+001, 5.044720137245840e+000, -3.012632743602935e-001, 1.111879055444745e+000}, {-8.028343773174927e+001, 4.812693734417916e+000, -2.928964520012951e-001, 1.111627529429055e+000}, {-7.518179322684912e+001, 4.592015664927063e+000, -2.847852254065942e-001, 1.111379033528286e+000}, {-7.042236090351322e+001, 4.382093167137382e+000, -2.769212664359781e-001, 1.111133530942511e+000}, {-6.598097748846658e+001, 4.182366469691380e+000, -2.692965437566774e-001, 1.110890985118343e+000}, {-6.183529536801900e+001, 3.992306887951382e+000, -2.619033113605164e-001, 1.110651359744292e+000}, {-5.796458338458645e+001, 3.811414767059530e+000, -2.547340973966659e-001, 1.110414618746117e+000}, {-5.434965024181786e+001, 3.639218005629175e+000, -2.477816939273175e-001, 1.110180726282199e+000}, {-5.097272117872193e+001, 3.475270394432624e+000, -2.410391467313756e-001, 1.109949646738908e+000}, {-4.781728753969381e+001, 3.319149959947196e+000, -2.344997455526618e-001, 1.109721344725985e+000}, {-4.486806590326451e+001, 3.170457809715834e+000, -2.281570150977289e-001, 1.109495785071926e+000}, {-4.211087764798335e+001, 3.028816655189627e+000, -2.220047059445052e-001, 1.109272932819375e+000}, {-3.953255089399676e+001, 2.893869530061677e+000, -2.160367860342251e-001, 1.109052753220513e+000}, {-3.712088444697060e+001, 2.765278795726399e+000, -2.102474326561080e-001, 1.108835211732461e+000}, {-3.486452988379367e+001, 2.642724832352832e+000, -2.046310243774980e-001, 1.108620274012686e+000}, {-3.275296521562360e+001, 2.525905196629955e+000, -1.991821337422336e-001, 1.108407905914401e+000}, {-3.077640612607013e+001, 2.414533510698032e+000, -1.938955198666938e-001, 1.108198073481974e+000}, {-2.892576875467384e+001, 2.308338647291225e+000, -1.887661216370296e-001, 1.107990742946342e+000}, {-2.719260655221808e+001, 2.207063812446913e+000, -1.837890509715300e-001, 1.107785880720415e+000}, {-2.556907195833716e+001, 2.110465784504171e+000, -1.789595864863502e-001, 1.107583453394488e+000}, {-2.404786193436341e+001, 2.018314105057650e+000, -1.742731673035715e-001, 1.107383427731652e+000}, {-2.262219267228373e+001, 1.930390438141688e+000, -1.697253872422143e-001, 1.107185770663204e+000}, {-2.128574048713125e+001, 1.846487791138621e+000, -1.653119890350547e-001, 1.106990449284039e+000}, {-2.003263310347830e+001, 1.766410019480312e+000, -1.610288590676970e-001, 1.106797430848068e+000}, {-1.885739666271851e+001, 1.689971111419810e+000, -1.568720219860535e-001, 1.106606682763607e+000}, {-1.775493736745766e+001, 1.616994694704592e+000, -1.528376357874969e-001, 1.106418172588765e+000}, {-1.672051102762514e+001, 1.547313487069378e+000, -1.489219869625882e-001, 1.106231868026832e+000}, {-1.574969663925623e+001, 1.480768795888941e+000, -1.451214858670679e-001, 1.106047736921662e+000}, {-1.483837797396761e+001, 1.417210073731107e+000, -1.414326622958197e-001, 1.105865747253029e+000}, {-1.398271440313103e+001, 1.356494434837758e+000, -1.378521611505417e-001, 1.105685867131996e+000}, {-1.317913026999323e+001, 1.298486293574206e+000, -1.343767384064626e-001, 1.105508064796260e+000}, {-1.242428692388935e+001, 1.243056912289198e+000, -1.310032570641007e-001, 1.105332308605480e+000}, {-1.171507278035640e+001, 1.190084079716421e+000, -1.277286834334243e-001, 1.105158567036609e+000}, {-1.104858517393140e+001, 1.139451742725932e+000, -1.245500834352500e-001, 1.104986808679193e+000}, {-1.042211251594205e+001, 1.091049661445034e+000, -1.214646190752247e-001, 1.104817002230662e+000}, {-9.833127930109274e+000, 1.044773141735465e+000, -1.184695451190952e-001, 1.104649116491605e+000}, {-9.279269222529393e+000, 1.000522693449853e+000, -1.155622057650780e-001, 1.104483120361017e+000}, {-8.758331594475001e+000, 9.582037846231296e-001, -1.127400315718840e-001, 1.104318982831536e+000}, {-8.268254945039644e+000, 9.177265681872020e-001, -1.100005364272110e-001, 1.104156672984648e+000}, {-7.807115847858402e+000, 8.790056460390960e-001, -1.073413146627254e-001, 1.103996159985876e+000}, {-7.373115629180458e+000, 8.419598198523868e-001, -1.047600382440301e-001, 1.103837413079935e+000}, {-6.964572964185544e+000, 8.065118808283763e-001, -1.022544541103427e-001, 1.103680401585863e+000}, {-6.579917319551989e+000, 7.725884103231008e-001, -9.982238159976631e-002, 1.103525094892129e+000}, {-6.217678026064315e+000, 7.401195591188906e-001, -9.746170992509612e-002, 1.103371462451692e+000}, {-5.876481003699939e+000, 7.090388893243341e-001, -9.517039583419153e-002, 1.103219473777050e+000}, {-5.555039937632146e+000, 6.792831791393206e-001, -9.294646126452588e-002, 1.103069098435232e+000}, {-5.252151337727452e+000, 6.507922663341224e-001, -9.078799114828991e-002, 1.102920306042765e+000}, {-4.966688624137329e+000, 6.235088884904741e-001, -8.869313127201903e-002, 1.102773066260599e+000}, {-4.697597099082865e+000, 5.973785350753152e-001, -8.666008621865683e-002, 1.102627348788986e+000}, {-4.443889321006219e+000, 5.723493085615286e-001, -8.468711739120274e-002, 1.102483123362320e+000}, {-4.204639802405735e+000, 5.483717858473124e-001, -8.277254109567965e-002, 1.102340359743926e+000}, {-3.978982672410267e+000, 5.253989052231248e-001, -8.091472672412192e-002, 1.102199027720805e+000}, {-3.766104946723917e+000, 5.033858263846861e-001, -7.911209494828850e-002, 1.102059097098311e+000}, {-3.565246496739293e+000, 4.822898463553391e-001, -7.736311607075418e-002, 1.101920537694806e+000}, {-3.375693166702355e+000, 4.620702628019282e-001, -7.566630831640270e-002, 1.101783319336218e+000}, {-3.196776175731432e+000, 4.426882955432985e-001, -7.402023630082621e-002, 1.101647411850568e+000}, {-3.027868289265530e+000, 4.241069821558813e-001, -7.242350948267441e-002, 1.101512785062427e+000}, {-2.868380956603266e+000, 4.062910862438896e-001, -7.087478069160028e-002, 1.101379408787292e+000}, {-2.717761648825286e+000, 3.892070113306511e-001, -6.937274471398268e-002, 1.101247252825911e+000}, {-2.575492776805616e+000, 3.728227297727842e-001, -6.791613694488873e-002, 1.101116286958535e+000}, {-2.441087575492705e+000, 3.571076890125111e-001, -6.650373203673411e-002, 1.100986480939071e+000}, {-2.314089601003278e+000, 3.420327522372873e-001, -6.513434266096387e-002, 1.100857804489185e+000}, {-2.194069980955622e+000, 3.275701217828866e-001, -6.380681826769433e-002, 1.100730227292302e+000}, {-2.080626276127093e+000, 3.136932793741042e-001, -6.252004391971047e-002, 1.100603718987523e+000}, {-1.973379870081554e+000, 3.003769146973490e-001, -6.127293913386867e-002, 1.100478249163457e+000}, {-1.871975877273442e+000, 2.875968805007567e-001, -6.006445681803470e-002, 1.100353787351948e+000}, {-1.776079761978281e+000, 2.753301177489863e-001, -5.889358216906679e-002, 1.100230303021708e+000}, {-1.685377974351372e+000, 2.635546224548870e-001, -5.775933170287378e-002, 1.100107765571849e+000}, {-1.599575248513736e+000, 2.522493798833729e-001, -5.666075222975589e-002, 1.099986144325295e+000}, {-1.518393998763328e+000, 2.413943236898121e-001, -5.559691992948053e-002, 1.099865408522100e+000}, {-1.441573373151687e+000, 2.309702912325227e-001, -5.456693943451667e-002, 1.099745527312619e+000}, {-1.368867734589837e+000, 2.209589757712106e-001, -5.356994293865919e-002, 1.099626469750589e+000}, {-1.300046167587472e+000, 2.113428906117814e-001, -5.260508936047872e-002, 1.099508204786056e+000}, {-1.234891346467230e+000, 2.021053268185756e-001, -5.167156351033650e-002, 1.099390701258173e+000}, {-1.173198705444031e+000, 1.932303174866275e-001, -5.076857530908452e-002, 1.099273927887873e+000}, {-1.114775448009057e+000, 1.847026001930233e-001, -4.989535901624339e-002, 1.099157853270368e+000}, {-1.059440641210220e+000, 1.765075920716501e-001, -4.905117251505244e-002, 1.099042445867529e+000}, {-1.007023132646987e+000, 1.686313435356952e-001, -4.823529656692645e-002, 1.098927674000076e+000}, {-9.573623681232882e-001, 1.610605238473588e-001, -4.744703416850136e-002, 1.098813505839624e+000}, {-9.103067285419199e-001, 1.537823810200982e-001, -4.668570986231731e-002, 1.098699909400549e+000}, {-8.657134181084795e-001, 1.467847197346915e-001, -4.595066911465587e-002, 1.098586852531666e+000}, {-8.234480247181129e-001, 1.400558758840660e-001, -4.524127770154400e-002, 1.098474302907740e+000}, {-7.833836306847527e-001, 1.335846880004829e-001, -4.455692110940590e-002, 1.098362228020779e+000}, {-7.454007724787563e-001, 1.273604788213846e-001, -4.389700397971351e-002, 1.098250595171141e+000}, {-7.093867102613078e-001, 1.213730291305892e-001, -4.326094954621643e-002, 1.098139371458416e+000}, {-6.752351859000392e-001, 1.156125586897596e-001, -4.264819911326340e-002, 1.098028523772093e+000}, {-6.428458916404382e-001, 1.100697043849075e-001, -4.205821154126008e-002, 1.097918018781999e+000}, {-6.121243204484043e-001, 1.047355035145177e-001, -4.149046276247566e-002, 1.097807822928491e+000}, {-5.829812077024473e-001, 9.960137266115672e-002, -4.094444529841480e-002, 1.097697902412399e+000}, {-5.553323463516771e-001, 9.465909211282406e-002, -4.041966781121276e-002, 1.097588223184712e+000}, {-5.290983018769104e-001, 8.990078930542095e-002, -3.991565466360545e-002, 1.097478750935989e+000}, {-5.042039900286437e-001, 8.531892107954550e-002, -3.943194549015325e-002, 1.097369451085481e+000}, {-4.805786848276957e-001, 8.090626225535701e-002, -3.896809480286970e-002, 1.097260288769965e+000}, {-4.581553822544535e-001, 7.665588629356712e-002, -3.852367158513845e-002, 1.097151228832265e+000}, {-4.368710839101541e-001, 7.256115807618124e-002, -3.809825893429213e-002, 1.097042235809448e+000}, {-4.166659728273346e-001, 6.861571312219809e-002, -3.769145367706548e-002, 1.096933273920688e+000}, {-3.974838071080961e-001, 6.481345301685161e-002, -3.730286604710058e-002, 1.096824307054777e+000}, {-3.792712543399244e-001, 6.114852713700537e-002, -3.693211933043777e-002, 1.096715298757269e+000}, {-3.619780828618504e-001, 5.761532647168631e-002, -3.657884956265647e-002, 1.096606212217247e+000}, {-3.455566815276582e-001, 5.420846842432888e-002, -3.624270520729187e-002, 1.096497010253680e+000}, {-3.299621094861046e-001, 5.092278977953486e-002, -3.592334687380059e-002, 1.096387655301374e+000}, {-3.151519038052869e-001, 4.775333582428087e-002, -3.562044703062108e-002, 1.096278109396484e+000}, {-3.010858568925735e-001, 4.469535018894293e-002, -3.533368973476431e-002, 1.096168334161571e+000}, {-2.877260222305396e-001, 4.174426753748317e-002, -3.506277037838072e-002, 1.096058290790178e+000}, {-2.750364404645829e-001, 3.889570313316693e-002, -3.480739543898168e-002, 1.095947940030928e+000}, {-2.629831468064490e-001, 3.614544648287300e-002, -3.456728225020585e-002, 1.095837242171080e+000}, {-2.515340199362101e-001, 3.348945271938262e-002, -3.434215877564163e-002, 1.095726157019559e+000}, {-2.406586002995336e-001, 3.092383411927447e-002, -3.413176339382135e-002, 1.095614643889409e+000}, {-2.303281947793209e-001, 2.844485621163648e-002, -3.393584471062294e-002, 1.095502661579664e+000}, {-2.205155458031833e-001, 2.604892674078030e-002, -3.375416134971540e-002, 1.095390168356583e+000}, {-2.111948886947761e-001, 2.373259209892301e-002, -3.358648178896612e-002, 1.095277121934247e+000}, {-2.023418904624849e-001, 2.149253081769280e-002, -3.343258418644718e-002, 1.095163479454475e+000}, {-1.939334528829362e-001, 1.932554604167274e-002, -3.329225621812505e-002, 1.095049197466017e+000}, {-1.859477537233797e-001, 1.722856136015816e-002, -3.316529493473407e-002, 1.094934231903014e+000}, {-1.783641264153182e-001, 1.519861470134868e-002, -3.305150662158253e-002, 1.094818538062666e+000}, {-1.711630023223341e-001, 1.323285303316979e-002, -3.295070667069486e-002, 1.094702070582085e+000}, {-1.643259022708403e-001, 1.132852818633101e-002, -3.286271946617035e-002, 1.094584783414292e+000}, {-1.578352716519453e-001, 9.482990227879400e-003, -3.278737827038020e-002, 1.094466629803301e+000}, {-1.516745199919222e-001, 7.693684441660621e-003, -3.272452513343139e-002, 1.094347562258269e+000}, {-1.458279673702965e-001, 5.958146760596810e-003, -3.267401080554415e-002, 1.094227532526641e+000}, {-1.402807290110952e-001, 4.273998186553273e-003, -3.263569465492298e-002, 1.094106491566266e+000}, {-1.350187580684957e-001, 2.638942378886687e-003, -3.260944461005679e-002, 1.093984389516410e+000}, {-1.300287214613593e-001, 1.050760098283643e-003, -3.259513709707835e-002, 1.093861175667613e+000}, {-1.252980510881369e-001, -4.926930328170846e-004, -3.259265700107719e-002, 1.093736798430343e+000}, {-1.208148059513224e-001, -1.993491944687124e-003, -3.260189762661594e-002, 1.093611205302381e+000}, {-1.165677227796157e-001, -3.453644121755644e-003, -3.262276068118421e-002, 1.093484342834852e+000}, {-1.125461563667504e-001, -4.875093794032855e-003, -3.265515625962785e-002, 1.093356156596868e+000}, {-1.087399966215703e-001, -6.259725904413122e-003, -3.269900284369064e-002, 1.093226591138662e+000}, {-1.051397545722765e-001, -7.609367460085121e-003, -3.275422732128897e-002, 1.093095589953180e+000}, {-1.017363957762588e-001, -8.925793255991769e-003, -3.282076499971125e-002, 1.092963095436013e+000}, {-9.852144216178711e-002, -1.021072638057550e-002, -3.289855965570027e-002, 1.092829048843598e+000}, {-9.548686792267386e-002, -1.146584294409076e-002, -3.298756357594890e-002, 1.092693390249574e+000}, {-9.262510156753394e-002, -1.269277437900251e-002, -3.308773762526614e-002, 1.092556058499214e+000}, {-8.992900988612509e-002, -1.389311018260152e-002, -3.319905132538444e-002, 1.092416991161816e+000}, {-8.739188464906093e-002, -1.506840068008259e-002, -3.332148294678389e-002, 1.092276124480920e+000}, {-8.500738921456238e-002, -1.622016040192641e-002, -3.345501961458242e-002, 1.092133393322250e+000}, {-8.276959948001254e-002, -1.734986965842096e-002, -3.359965743818405e-002, 1.091988731119247e+000}, {-8.067292166344241e-002, -1.845897846985461e-002, -3.375540164665517e-002, 1.091842069816039e+000}, {-7.871214577495787e-002, -1.954890787713057e-002, -3.392226675377865e-002, 1.091693339807705e+000}, {-7.688236749343953e-002, -2.062105393477184e-002, -3.410027672821032e-002, 1.091542469877672e+000}, {-7.517903122777866e-002, -2.167678909232968e-002, -3.428946519791137e-002, 1.091389387132079e+000}, {-7.359788435365057e-002, -2.271746557145173e-002, -3.448987566368195e-002, 1.091234016930905e+000}, {-7.213496529807099e-002, -2.374441795105308e-002, -3.470156173993946e-002, 1.091076282815684e+000}, {-7.078664709747612e-002, -2.475896478998424e-002, -3.492458742242282e-002, 1.090916106433582e+000}, {-6.954953734834717e-002, -2.576241320349298e-002, -3.515902736727997e-002, 1.090753407457601e+000}, {-6.842055334357715e-002, -2.675605967578415e-002, -3.540496721654386e-002, 1.090588103502687e+000}, {-6.739689298150288e-002, -2.774119353806004e-002, -3.566250393471657e-002, 1.090420110037451e+000}, {-6.647599481840270e-002, -2.871910028240180e-002, -3.593174618253033e-002, 1.090249340291247e+000}, {-6.565559167808195e-002, -2.969106335868627e-002, -3.621281472572473e-002, 1.090075705156262e+000}, {-6.493365627398073e-002, -3.065836816359168e-002, -3.650584287130890e-002, 1.089899113084334e+000}, {-6.430842408472148e-002, -3.162230451330045e-002, -3.681097694769039e-002, 1.089719469978098e+000}, {-6.377841573206161e-002, -3.258416951694268e-002, -3.712837682203768e-002, 1.089536679076100e+000}, {-6.334237459730568e-002, -3.354527207676258e-002, -3.745821645677120e-002, 1.089350640831460e+000}, {-6.299931960292200e-002, -3.450693519565747e-002, -3.780068452222145e-002, 1.089161252783616e+000}, {-6.274855690363090e-002, -3.547049958158794e-002, -3.815598505196914e-002, 1.088968409422674e+000}, {-6.258962932227907e-002, -3.643732858924495e-002, -3.852433814992687e-002, 1.088772002045827e+000}, {-6.252237552594815e-002, -3.740881109502971e-002, -3.890598076615168e-002, 1.088571918605265e+000}, {-6.254690896242934e-002, -3.838636646260465e-002, -3.930116752711852e-002, 1.088368043546958e+000}, {-6.266364244831302e-002, -3.937144869619517e-002, -3.971017164197188e-002, 1.088160257639610e+000}, {-6.287328319117271e-002, -4.036555176939004e-002, -4.013328587796861e-002, 1.087948437793056e+000}, {-6.317686471529196e-002, -4.137021437944927e-002, -4.057082362458787e-002, 1.087732456865278e+000}, {-6.357574767711911e-002, -4.238702597030984e-002, -4.102312004137679e-002, 1.087512183457172e+000}, {-6.407164470783205e-002, -4.341763253645162e-002, -4.149053330906206e-002, 1.087287481694067e+000}, {-6.466664636574100e-002, -4.446374318746806e-002, -4.197344598438829e-002, 1.087058210992956e+000}, {-6.536322059434012e-002, -4.552713774389385e-002, -4.247226647187895e-002, 1.086824225814272e+000}, {-6.616428759721223e-002, -4.660967333451058e-002, -4.298743063263810e-002, 1.086585375396910e+000}, {-6.707319059880698e-002, -4.771329437422422e-002, -4.351940351974055e-002, 1.086341503475118e+000}, {-6.809379051719827e-002, -4.884004012133992e-002, -4.406868129064020e-002, 1.086092447975686e+000}, {-6.923558511367642e-002, -4.999196690374660e-002, -4.463579361515062e-002, 1.085838040693730e+000}, {-7.049988138972356e-002, -5.117136544992598e-002, -4.522130521822819e-002, 1.085578106945194e+000}, {-7.188620542131699e-002, -5.238072225909109e-002, -4.582581885465249e-002, 1.085312465193973e+000}, {-7.339274099366680e-002, -5.364413985831130e-002, -4.646040027521749e-002, 1.085036412823779e+000}, {-7.500954773269826e-002, -5.492497782325354e-002, -4.710632451399192e-002, 1.084758207947855e+000}, {-7.673673432833014e-002, -5.622189579867198e-002, -4.776212196116022e-002, 1.084478467134268e+000}, {-7.857550333131255e-002, -5.753625488563768e-002, -4.842767881855225e-002, 1.084197220574224e+000}, {-8.052734288370235e-002, -5.886938522147935e-002, -4.910289264837146e-002, 1.083914499613667e+000}, {-8.259407010993335e-002, -6.022258916344622e-002, -4.978767167516485e-002, 1.083630336649792e+000}, {-8.477775911831006e-002, -6.159714721340068e-002, -5.048193412171477e-002, 1.083344765033191e+000}, {-8.708080113955191e-002, -6.299432003883736e-002, -5.118560761903612e-002, 1.083057818975364e+000}, {-8.950584765384662e-002, -6.441535367085308e-002, -5.189862863655786e-002, 1.082769533461264e+000}, {-9.205583518148568e-002, -6.586148203903283e-002, -5.262094197327757e-002, 1.082479944166658e+000}, {-9.473397681422530e-002, -6.733393026815319e-002, -5.335250028030908e-002, 1.082189087380031e+000}, {-9.754375455528717e-002, -6.883391781405844e-002, -5.409326361800470e-002, 1.081896999928780e+000}, {-1.004889430537072e-001, -7.036266070476557e-002, -5.484319904975906e-002, 1.081603719109501e+000}, {-1.035735781946822e-001, -7.192137498268748e-002, -5.560228025918928e-002, 1.081309282622144e+000}, {-1.068019749773503e-001, -7.351127863515777e-002, -5.637048720388151e-002, 1.081013728507841e+000}, {-1.101787499902488e-001, -7.513359354342586e-002, -5.714780579175685e-002, 1.080717095090215e+000}, {-1.137088060108335e-001, -7.678954829834185e-002, -5.793422757552009e-002, 1.080419420919993e+000}, {-1.173973070410327e-001, -7.848038080842838e-002, -5.872974947289480e-002, 1.080120744722770e+000}, {-1.212497678441141e-001, -8.020733845529093e-002, -5.953437351668309e-002, 1.079821105349743e+000}, {-1.252719700472134e-001, -8.197168217757889e-002, -6.034810660246008e-002, 1.079520541731282e+000}, {-1.294700211546631e-001, -8.377468696961547e-002, -6.117096027413883e-002, 1.079219092833192e+000}, {-1.338503700750963e-001, -8.561764374303535e-002, -6.200295051492537e-002, 1.078916797615539e+000}, {-1.384197709577187e-001, -8.750186189508519e-002, -6.284409755180367e-002, 1.078613694993897e+000}, {-1.431853353658080e-001, -8.942867014032002e-002, -6.369442568095514e-002, 1.078309823802917e+000}, {-1.481545174104211e-001, -9.139941856567219e-002, -6.455396310036206e-002, 1.078005222762096e+000}, {-1.533351702300495e-001, -9.341547956575609e-002, -6.542274175644600e-002, 1.077699930443627e+000}, {-1.587354634820050e-001, -9.547825103285042e-002, -6.630079719598207e-002, 1.077393985242266e+000}, {-1.643640090876028e-001, -9.758915575387375e-002, -6.718816844083814e-002, 1.077087425347061e+000}, {-1.702298127665787e-001, -9.974964419697471e-002, -6.808489785802697e-002, 1.076780288714918e+000}, {-1.763422817767454e-001, -1.019611961271160e-001, -6.899103104367133e-002, 1.076472613045863e+000}, {-1.827112750381146e-001, -1.042253212629891e-001, -6.990661672048851e-002, 1.076164435759950e+000}, {-1.893470894923299e-001, -1.065435614567023e-001, -7.083170663269980e-002, 1.075855793975729e+000}, {-1.962605127165320e-001, -1.089174914651409e-001, -7.176635545593288e-002, 1.075546724490185e+000}, {-2.034627974654094e-001, -1.113487211960143e-001, -7.271062070759818e-002, 1.075237263760114e+000}, {-2.109657066826481e-001, -1.138388965833026e-001, -7.366456266798066e-002, 1.074927447884822e+000}, {-2.187815400532935e-001, -1.163897009783962e-001, -7.462824430457526e-002, 1.074617312590122e+000}, {-2.269231322486189e-001, -1.190028571417117e-001, -7.560173119833084e-002, 1.074306893213548e+000}, {-2.354038806028248e-001, -1.216801282391285e-001, -7.658509148189741e-002, 1.073996224690738e+000}, {-2.442378202565233e-001, -1.244233187393023e-001, -7.757839577778078e-002, 1.073685341542923e+000}, {-2.534395283451278e-001, -1.272342778387637e-001, -7.858171713389295e-002, 1.073374277865470e+000}, {-2.630263592208533e-001, -1.301148743711012e-001, -7.959513104281542e-002, 1.073063067317448e+000}, {-2.730051134964604e-001, -1.330671775579355e-001, -8.061871489472258e-002, 1.072751743112126e+000}, {-2.834065958432523e-001, -1.360929306822043e-001, -8.165254963474328e-002, 1.072440338008417e+000}, {-2.942333101794055e-001, -1.391944214222225e-001, -8.269671635638208e-002, 1.072128884302168e+000}, {-3.054697678952821e-001, -1.423740192943417e-001, -8.375129913345693e-002, 1.071817413823248e+000}, {-3.170567570317931e-001, -1.456342483062608e-001, -8.481638378906832e-002, 1.071505957923588e+000}, {-3.306168617540374e-001, -1.489587293359699e-001, -8.589210145617054e-002, 1.071194547476514e+000}, {-3.409850517367228e-001, -1.524212808452965e-001, -8.697834037358347e-002, 1.070883212875128e+000}, {-3.560258309587398e-001, -1.559011033901600e-001, -8.807563275717535e-002, 1.070571984040678e+000}, {-3.693952037952778e-001, -1.595118380297836e-001, -8.918357985816929e-002, 1.070260890269993e+000}, {-3.838111583549971e-001, -1.632041776149361e-001, -9.030255764252720e-002, 1.069949960599530e+000}, {-3.988287520315456e-001, -1.669904093902224e-001, -9.143260204604514e-002, 1.069639223399451e+000}, {-4.144733321913969e-001, -1.708730877160433e-001, -9.257381383873017e-002, 1.069328706585730e+000}, {-4.307713785387899e-001, -1.748548319916569e-001, -9.372629570383198e-002, 1.069018437574774e+000}, {-4.477502767932388e-001, -1.789383318342014e-001, -9.489015220206132e-002, 1.068708443282907e+000}, {-4.654387110151121e-001, -1.831263430257065e-001, -9.606548976685919e-002, 1.068398750126322e+000}, {-4.838661501934946e-001, -1.874216961724761e-001, -9.725241665146278e-002, 1.068089384021467e+000}, {-5.030632737861358e-001, -1.918272907282402e-001, -9.845104292529755e-002, 1.067780370385895e+000}, {-5.235374589086956e-001, -1.963417315274094e-001, -9.966148935023214e-002, 1.067471734139486e+000}, {-5.454914032389386e-001, -2.009649239097508e-001, -1.008838772519371e-001, 1.067163499706100e+000}, {-5.691589559314858e-001, -2.056953492856035e-001, -1.021183461925116e-001, 1.066855691015596e+000}, {-5.948100549339186e-001, -2.110255040137948e-001, -1.034822753625355e-001, 1.066519576356437e+000}, {-6.235735359755632e-001, -2.165957325688549e-001, -1.048868425206513e-001, 1.066177762224400e+000}, {-6.499081473123812e-001, -2.224933799811154e-001, -1.063338865224673e-001, 1.065830074548712e+000}, {-6.837787055262851e-001, -2.285973841168222e-001, -1.078260017738615e-001, 1.065476330621825e+000}, {-7.167680580575481e-001, -2.350681790211194e-001, -1.093649281364889e-001, 1.065116338398494e+000}, {-7.529421660388130e-001, -2.418587175888764e-001, -1.109534924500986e-001, 1.064749896111096e+000}, {-7.918478434536762e-001, -2.490076352352779e-001, -1.125942698775614e-001, 1.064376791182773e+000}, {-8.337579503056568e-001, -2.565420848384871e-001, -1.142901236298031e-001, 1.063996799632633e+000}, {-8.789796651735242e-001, -2.644919033636966e-001, -1.160441401751793e-001, 1.063609685150805e+000}, {-9.278592389976171e-001, -2.728899480510872e-001, -1.178596518948347e-001, 1.063215198130870e+000}, {-9.807885348650018e-001, -2.817724684979601e-001, -1.197402626436758e-001, 1.062813074603089e+000}, {-1.038211963449425e+000, -2.911795476170365e-001, -1.216898765582236e-001, 1.062403035055771e+000}, {-1.100635026079546e+000, -3.011556050874361e-001, -1.237127306959247e-001, 1.061984783130254e+000}, {-1.168634591893341e+000, -3.117499794663595e-001, -1.258134320829727e-001, 1.061558004172562e+000}, {-1.242871047312425e+000, -3.230176068616857e-001, -1.279969998819422e-001, 1.061122363622202e+000}, {-1.324102816333368e+000, -3.350198128649087e-001, -1.302689135412735e-001, 1.060677505215274e+000}, {-1.413204046652451e+000, -3.478252360881645e-001, -1.326351679601656e-001, 1.060223048975250e+000}, {-1.511185630627330e+000, -3.615109178443726e-001, -1.351023368667481e-001, 1.059758588960120e+000}, {-1.619221197038109e+000, -3.761635786122374e-001, -1.376776459309125e-001, 1.059283690729083e+000}, {-1.738677889702752e+000, -3.918811400429350e-001, -1.403690573333936e-001, 1.058797888485215e+000}, {-1.871154359990751e+000, -4.087745223058986e-001, -1.431853680336985e-001, 1.058300681842389e+000}, {-2.018526589606020e+000, -4.269697926301481e-001, -1.461363243040554e-001, 1.057791532154715e+000}, {-2.185501792184186e+000, -4.465841288025065e-001, -1.492328186957302e-001, 1.057269858334624e+000}, {-2.364687153396257e+000, -4.679154024903533e-001, -1.524865743220330e-001, 1.056735032070606e+000}, {-2.576475123351074e+000, -4.908877748574237e-001, -1.559120016473587e-001, 1.056186372337097e+000}, {-2.809603261630487e+000, -5.159839044005667e-001, -1.595231052389059e-001, 1.055623138920609e+000}, {-3.075454975098822e+000, -5.433260214541975e-001, -1.633376814220988e-001, 1.055044525513794e+000}, {-3.378476048700296e+000, -5.732358362636583e-001, -1.673748772676717e-001, 1.054449650639562e+000}, {-3.725624075390214e+000, -6.060595365408650e-001, -1.716566425727572e-001, 1.053837547969356e+000}, {-4.125513088548122e+000, -6.422048242798819e-001, -1.762080296688844e-001, 1.053207154504926e+000}, {-4.588900079051337e+000, -6.821546123391310e-001, -1.810577742248455e-001, 1.052557296675667e+000}, {-5.129342151488261e+000, -7.264844420275155e-001, -1.862390135536310e-001, 1.051886673785578e+000}, {-5.764097830251668e+000, -7.758847747289787e-001, -1.917901826032635e-001, 1.051193838170567e+000}, {-6.515378231730130e+000, -8.311897985680916e-001, -1.977561416081993e-001, 1.050477171237241e+000}, {-7.412109909279641e+000, -8.934149834663464e-001, -2.041896093864123e-001, 1.049734854297595e+000}, {-8.492455381030945e+000, -9.638065423607585e-001, -2.111530046471992e-001, 1.048964832761054e+000}, {-9.807474389689094e+000, -1.043907256650206e+000, -2.187208392607184e-001, 1.048164771754150e+000}, {-1.142992555561826e+001, -1.135606773901190e+000, -2.269829647144973e-001, 1.047332000543470e+000}, {-1.344511926099734e+001, -1.241495308018550e+000, -2.360481455742939e-001, 1.046463442140266e+000}, {-1.600738269476274e+001, -1.364375395477736e+000, -2.460518881297009e-001, 1.045555523007033e+000}, {-1.929939535516280e+001, -1.508513275807175e+000, -2.571603987700460e-001, 1.044604055466647e+000}, {-2.362120150582920e+001, -1.678967132778218e+000, -2.695862540141585e-001, 1.043604082766197e+000}, {-2.942152881801653e+001, -1.882583607807427e+000, -2.836015162163547e-001, 1.042549669910840e+000}, {-3.741453654687421e+001, -2.128389007332268e+000, -2.995629186888021e-001, 1.041433617232253e+000}, {-4.878270682458398e+001, -2.428336462205647e+000, -3.179486577792697e-001, 1.040247057754742e+000}, {-6.558832836460495e+001, -2.797988793419757e+000, -3.394177234686278e-001, 1.038978876337398e+000}, {-9.164855359795394e+001, -3.256574496899152e+000, -3.649096511422550e-001, 1.037614844862927e+000}, {-1.346442592691005e+002, -3.822745465072996e+000, -3.958270232324012e-001, 1.036136285584020e+000}, {-2.039488892785857e+002, -4.585663115601926e+000, -4.341576010582943e-001, 1.034517909421716e+000}, {-3.213729265046176e+002, -5.715603682655638e+000, -4.830053729894114e-001, 1.032724118072463e+000}, {-5.380530697149483e+002, -7.590740438263201e+000, -5.472440337555892e-001, 1.030702209952531e+000}, {-9.773668143802523e+002, -8.397564013673986e+000, -6.149721615017588e-001, 1.028987687176493e+000}, {-1.971206182132680e+003, -7.954972432593549e+000, -7.107673680064054e-001, 1.027043503002403e+000}, {-4.259063749453282e+003, -7.470160381920227e+000, -8.552169229520203e-001, 1.024677777723123e+000}, {-9.468420868233850e+003, -1.441169226818975e+001, -1.061676619743433e+000, 1.021911421684383e+000}, {-2.124426475615268e+004, -2.969103328338514e+001, -1.321932106107769e+000, 1.019310988957590e+000}, {-5.142304350768700e+004, -2.757198727294517e+001, -1.633808439690381e+000, 1.017155605087379e+000}, {-1.261916077251016e+005, 1.340795831729274e+001, -2.041513938530926e+000, 1.015172127346691e+000}, {-2.907555625955585e+005, 3.100216772602235e+000, -2.587089312710111e+000, 1.013030579652139e+000}, {-6.443400705406987e+005, -4.736727283746432e+002, -3.393782437455662e+000, 1.010424690909639e+000}, {-1.284212584824039e+006, -8.719164629810688e+002, -4.081415290704422e+000, 1.009013628657760e+000}, {-2.953254945900618e+006, -3.499079388487411e+002, -4.609510455745138e+000, 1.008360191314845e+000}, {-8.708278203444760e+006, 1.160570409967432e+003, -5.732506388098228e+000, 1.007240267626251e+000}, {-2.925999835917269e+007, 6.373124182370374e+003, -7.673217433665648e+000, 1.006032670356375e+000}, {-9.691364693490808e+007, 5.724533606825865e+003, -9.856016988972671e+000, 1.004465302443802e+000}, {-2.344230326747042e+008, -7.237460849943119e+003, -1.228103670994250e+001, 1.003403193772207e+000}, {-1.674594981779917e+012, 7.945262725742146e+007, -9.424323791757101e+002, 1.002718092569686e+000}};

        record crit
          extends Modelica.Icons.Record;
          constant SIunits.Temperature TCRIT = 374.18;
          constant SIunits.Pressure PCRIT = 4056290.0;
          constant SIunits.Density DCRIT = 508.0;
        end crit;

        record data
          constant SIunits.SpecificHeatCapacity R = 81.4888564372;
          // 8.314471/0.102032
          constant SIunits.MolarMass MM = 0.102032;
          extends crit;
          constant SIunits.SpecificEnthalpy HCRIT = 389653;
          constant SIunits.SpecificEntropy SCRIT = 1562.135;
          extends fcrit;
          extends triple;
        end data;

        record fcrit
          extends Modelica.Icons.Record;
          constant SIunits.Temperature FTCRIT = 374.209;
          constant SIunits.Pressure FPCRIT = 4059280.0;
          constant SIunits.Density FDCRIT = 511.9;
        end fcrit;

        record Ideal
          extends Common.EOSIdealCoeff(nc = 5, a = {-1.019535e+0, 9.047135e+0, -1.629789e+0, -9.723916e+0, -3.927170e+0});
        end Ideal;

        record ReferenceStates
          extends Modelica.Icons.Record;
          constant SIunits.SpecificEnthalpy h0 = 1.0;
          constant SIunits.SpecificEntropy s0 = 1.0;
        end ReferenceStates;

        record Residual
          extends Common.EOSResidualCoeff(nc = 21, ns1 = 8, c = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 1.0, 1.0, 2.0, 2.0, 2.0, 2.0, 2.0, 2.0, 3.0, 3.0, 3.0, 4.0}, d = {2.0, 1.0, 3.0, 6.0, 6.0, 1.0, 1.0, 2.0, 5.0, 2.0, 2.0, 4.0, 1.0, 4.0, 1.0, 2.0, 4.0, 1.0, 5.0, 3.0, 10.0}, t = {-0.5, 0.0, 0.0, 0.0, 1.5, 1.5, 2.0, 2.0, 1.0, 3.0, 5.0, 1.0, 5.0, 5.0, 6.0, 10.0, 10.0, 10.0, 18.0, 22.0, 50.0}, n = {0.5586817000e-01, 0.4982230000e+00, 0.2458698000e-01, 0.8570145000e-03, 0.4788584000e-03, -0.1800808000e+01, 0.2671641000e+00, -0.4781652000e-01, 0.1423987000e-01, 0.3324062000e+00, -0.7485907000e-02, 0.1017263000e-03, -0.5184567000e+00, -0.8692288000e-01, 0.2057144000e+00, -0.5000457000e-02, 0.4603262000e-03, -0.3497836000e-02, 0.6995038000e-02, -0.1452184000e-01, -0.1285458000e-03});
        end Residual;

        record triple
          extends Modelica.Icons.Record;
          constant SIunits.Temperature TTRIPLE = 169.85;
          constant SIunits.Pressure PTRIPLE = 389.563789;
          constant SIunits.Density DLTRIPLE = 1591.107453;
          constant SIunits.Density DVTRIPLE = 0.028172;
        end triple;

        record CoeffsThermalConductivity
          extends Modelica.Icons.Record;
          constant Real a_0 = -1.05248e-2 "Empirical parameter, [W/(m K)]";
          constant Real a_1 = 8.00982e-5 "Empirical parameter, [W/(m K^2)]";
          constant Real b[4] = {1.836526, 5.126143, -1.436883, 0.626144} "Empirical parameter, [-]";
          constant Real lambda_reducing = 2.055e-3 "Reducing factor, [W/(m K)]";
          constant Real q_d = 1.89202e9 "Modified effective cutoff wave number, [1/m]";
          constant Real xi_0 = 1.94e-10 "Critical Amplitude, [m]";
          constant Real GAMMA = 0.0496 "Amplitude, [-]";
          constant Real T_ref = 561.411 "Arbitrary reference temperature, [K]";
          constant Real p_crit = 4059.28 "Critical pressure, [kPa]";
          constant Real rho_crit = 5.049886 "Critical density, [mol/L]";
          constant Real nu = 0.63 "Universal exponent, [-]";
          constant Real gamma = 1.239 "Universal exponent, [-]";
          constant Real R_0 = 1.03 "Universal amplitude, [-]";
        end CoeffsThermalConductivity;

        record CoeffsSurfaceTension
          extends Modelica.Icons.Record;
          constant Integer nterm = 1;
          constant Real sigmak[nterm] = {0.06016} "Empirical parameter";
          constant Real sigexp[nterm] = {1.26} "Empirical parameter";
        end CoeffsSurfaceTension;
      end R134aData;
      annotation(
        Documentation(info = "<html>
<p>
Calculation of fluid properties for Tetrafluoroethane (R134a) in the fluid region of 0.0039 bar (Triple pressure) to 700 bar and 169.85 Kelvin (Triple temperature) to 455 Kelvin.
</p>
<h4>Restriction</h4>
<p>
The functions provided by this package shall be used inside of the restricted limits according to the referenced literature.
</p>
<ul>
 <li>
      <b> 0.0039 bar &le; p &le; 700 bar </b>
 </li>
 <li>
      <b> 169.85 Kelvin &le; T &le; 455 Kelvin  </b>
 </li>
 <li>
      <b> explicit for pressure and specific enthalpy </b>
 </li>
</ul>

<p><b>References</b> </p>
<dl><dt>Baehr, H.D. and Tillner-Roth, R.: </dt>
<dd><b>Thermodynamic Properties of Environmentally Acceptable Refrigerants -
Equations of State and Tables for Ammonia, R22, R134a, R152a, and R123</b>. Springer-Verlag, Berlin (Germany), 1994.</dd>
</dl>
<dl><dt>Klein, McLinden and Laesecke: </dt>
<dd><b>An improved extended corresponding states methode for estimation of visocsity of pure refrigerants and mixtures</b>.
Int. J. Refrig., Vol. 20, No.3, pp. 208-217, 1997.</dd>
</dl>
<dl><dt>McLinden, Klein. and Perkins: </dt>
<dd><b>An extended corresponding states model for the thermal conductivity
of refrigerants and refrigerant mixtures</b>.
Int. J. Refrig., 23 (2000) 43-63.</dd>
</dl>
<dl><dt>Okada and Higashi: </dt>
<dd><b>Surface tension correlation of HFC-134a and HCFC-123</b>.
Proceedings of the Joint Meeting of IIR Commissions B1, B2, E1, and E2, Padua, Italy, pp. 541-548, 1994.</dd>
</dl>

<h4>Verification</h4>
<p>
The verification report for the development of this library is provided
<a href=\"modelica://Modelica/Resources/Documentation/Media/MoMoLib_VerificationResults_XRG.pdf\">here</a>.
</p>

<h4>Acknowledgment</h4>
<p>
This library was developed by XRG Simulation GmbH as part of the <a href=\"http://www.cleansky.eu/\">Clean Sky</a> JTI project (Project title: MoMoLib-Modelica Model Library Development for Media, Magnetic Systems and Wavelets; Project number: 296369; Theme: JTI-CS-2011-1-SGO-02-026: Modelica Model Library Development Part I). The partial financial support for the development of this library by the European Union is highly appreciated.
</p>

<p>
Some parts of this library refer to the ThermoFluid library developed at Lund University (<a href=\"http://thermofluid.sourceforge.net/\">http://thermofluid.sourceforge.net</a>).
</p>

<h4>Disclaimer</h4>
<p>
In no event will XRG Simulation GmbH be liable for any direct, indirect, incidental, special, exemplary, or consequential damages, arising in any way out of the use of this software, even if advised of the possibility of such damage.
</p>
<h4> Copyright (C) 2013, XRG Simulation GmbH </h4>
</html>"));
    end R134a;

    package Water "Medium models for water"
      extends Modelica.Icons.VariantsPackage;
      import Modelica.Media.Water.ConstantPropertyLiquidWater.simpleWaterConstants;
      constant Modelica.Media.Interfaces.Types.TwoPhase.FluidConstants[1] waterConstants(each chemicalFormula = "H2O", each structureFormula = "H2O", each casRegistryNumber = "7732-18-5", each iupacName = "oxidane", each molarMass = 0.018015268, each criticalTemperature = 647.096, each criticalPressure = 22064.0e3, each criticalMolarVolume = 1 / 322.0 * 0.018015268, each normalBoilingPoint = 373.124, each meltingPoint = 273.15, each triplePointTemperature = 273.16, each triplePointPressure = 611.657, each acentricFactor = 0.344, each dipoleMoment = 1.8, each hasCriticalData = true);

      package IdealSteam "Water: Steam as ideal gas from NASA source"
        extends IdealGases.SingleGases.H2O;
        annotation(
          Documentation(info = "<html>

</html>"));
      end IdealSteam;

      package ConstantPropertyLiquidWater "Water: Simple liquid water medium (incompressible, constant data)"
        //   redeclare record extends FluidConstants
        //   end FluidConstants;
        constant Modelica.Media.Interfaces.Types.Basic.FluidConstants[1] simpleWaterConstants(each chemicalFormula = "H2O", each structureFormula = "H2O", each casRegistryNumber = "7732-18-5", each iupacName = "oxidane", each molarMass = 0.018015268);
        extends Interfaces.PartialSimpleMedium(mediumName = "SimpleLiquidWater", cp_const = 4184, cv_const = 4184, d_const = 995.586, eta_const = 1.e-3, lambda_const = 0.598, a_const = 1484, T_min = Cv.from_degC(-1), T_max = Cv.from_degC(130), T0 = 273.15, MM_const = 0.018015268, fluidConstants = simpleWaterConstants);
        annotation(
          Documentation(info = "<html>

</html>"));
      end ConstantPropertyLiquidWater;

      package StandardWater = WaterIF97_ph "Water using the IF97 standard, explicit in p and h. Recommended for most applications";
      package StandardWaterOnePhase = WaterIF97_pT "Water using the IF97 standard, explicit in p and T. Recommended for one-phase applications";

      package WaterIF97OnePhase_ph "Water using the IF97 standard, explicit in p and h, and only valid outside the two-phase dome"
        extends WaterIF97_base(ThermoStates = Modelica.Media.Interfaces.Choices.IndependentVariables.ph, final ph_explicit = true, final dT_explicit = false, final pT_explicit = false, final smoothModel = true, final onePhase = true);
        annotation(
          Documentation(info = "<html>

</html>"));
      end WaterIF97OnePhase_ph;

      package WaterIF97_pT "Water using the IF97 standard, explicit in p and T"
        extends WaterIF97_base(ThermoStates = Modelica.Media.Interfaces.Choices.IndependentVariables.pT, final ph_explicit = false, final dT_explicit = false, final pT_explicit = true, final smoothModel = true, final onePhase = true);
      end WaterIF97_pT;

      package WaterIF97_ph "Water using the IF97 standard, explicit in p and h"
        extends WaterIF97_base(ThermoStates = Modelica.Media.Interfaces.Choices.IndependentVariables.ph, final ph_explicit = true, final dT_explicit = false, final pT_explicit = false, smoothModel = false, onePhase = false);
        annotation(
          Documentation(info = "<html>

</html>"));
      end WaterIF97_ph;

      partial package WaterIF97_base "Water: Steam properties as defined by IAPWS/IF97 standard"
        extends Interfaces.PartialTwoPhaseMedium(mediumName = "WaterIF97", substanceNames = {"water"}, singleState = false, SpecificEnthalpy(start = 1.0e5, nominal = 5.0e5), Density(start = 150, nominal = 500), AbsolutePressure(start = 50e5, nominal = 10e5, min = 611.657, max = 100e6), Temperature(start = 500, nominal = 500, min = 273.15, max = 2273.15), smoothModel = false, onePhase = false, fluidConstants = waterConstants);

        redeclare record extends ThermodynamicState "Thermodynamic state"
            SpecificEnthalpy h "Specific enthalpy";
            Density d "Density";
            Temperature T "Temperature";
            AbsolutePressure p "Pressure";
        end ThermodynamicState;

        constant Boolean ph_explicit "True if explicit in pressure and specific enthalpy";
        constant Boolean dT_explicit "True if explicit in density and temperature";
        constant Boolean pT_explicit "True if explicit in pressure and temperature";

        redeclare replaceable model extends BaseProperties(h(stateSelect = if ph_explicit and preferredMediumStates then StateSelect.prefer else StateSelect.default), d(stateSelect = if dT_explicit and preferredMediumStates then StateSelect.prefer else StateSelect.default), T(stateSelect = if (pT_explicit or dT_explicit) and preferredMediumStates then StateSelect.prefer else StateSelect.default), p(stateSelect = if (pT_explicit or ph_explicit) and preferredMediumStates then StateSelect.prefer else StateSelect.default)) "Base properties of water"
            Integer phase(min = 0, max = 2, start = 1, fixed = false) "2 for two-phase, 1 for one-phase, 0 if not known";

          equation
            MM = fluidConstants[1].molarMass;
            if smoothModel then
              if onePhase then
                phase = 1;
                if ph_explicit then
                  assert(h < bubbleEnthalpy(sat) or h > dewEnthalpy(sat) or p > fluidConstants[1].criticalPressure, "With onePhase=true this model may only be called with one-phase states h < hl or h > hv!" + "(p = " + String(p) + ", h = " + String(h) + ")");
                else
                  if dT_explicit then
                    assert(not (d < bubbleDensity(sat) and d > dewDensity(sat) and T < fluidConstants[1].criticalTemperature), "With onePhase=true this model may only be called with one-phase states d > dl or d < dv!" + "(d = " + String(d) + ", T = " + String(T) + ")");
                  end if;
                end if;
              else
                phase = 0;
              end if;
            else
              if ph_explicit then
                phase = if h < bubbleEnthalpy(sat) or h > dewEnthalpy(sat) or p > fluidConstants[1].criticalPressure then 1 else 2;
              elseif dT_explicit then
                phase = if not (d < bubbleDensity(sat) and d > dewDensity(sat) and T < fluidConstants[1].criticalTemperature) then 1 else 2;
              else
                phase = 1;
//this is for the one-phase only case pT
              end if;
            end if;
            if dT_explicit then
              p = pressure_dT(d, T, phase);
              h = specificEnthalpy_dT(d, T, phase);
              sat.Tsat = T;
              sat.psat = saturationPressure(T);
            elseif ph_explicit then
              d = density_ph(p, h, phase);
              T = temperature_ph(p, h, phase);
              sat.Tsat = saturationTemperature(p);
              sat.psat = p;
            else
              h = specificEnthalpy_pT(p, T);
              d = density_pT(p, T);
              sat.psat = p;
              sat.Tsat = saturationTemperature(p);
            end if;
            u = h - p / d;
            R = Modelica.Constants.R / fluidConstants[1].molarMass;
            h = state.h;
            p = state.p;
            T = state.T;
            d = state.d;
            phase = state.phase;
        end BaseProperties;

        redeclare function density_ph "Computes density as a function of pressure and specific enthalpy"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SpecificEnthalpy h "Specific enthalpy";
          input FixedPhase phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          output Density d "Density";
        algorithm
          d := IF97_Utilities.rho_ph(p, h, phase);
          annotation(
            Inline = true);
        end density_ph;

        redeclare function temperature_ph "Computes temperature as a function of pressure and specific enthalpy"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SpecificEnthalpy h "Specific enthalpy";
          input FixedPhase phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          output Temperature T "Temperature";
        algorithm
          T := IF97_Utilities.T_ph(p, h, phase);
          annotation(
            Inline = true);
        end temperature_ph;

        redeclare function temperature_ps "Compute temperature from pressure and specific enthalpy"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SpecificEntropy s "Specific entropy";
          input FixedPhase phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          output Temperature T "Temperature";
        algorithm
          T := IF97_Utilities.T_ps(p, s, phase);
          annotation(
            Inline = true);
        end temperature_ps;

        redeclare function density_ps "Computes density as a function of pressure and specific enthalpy"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SpecificEntropy s "Specific entropy";
          input FixedPhase phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          output Density d "Density";
        algorithm
          d := IF97_Utilities.rho_ps(p, s, phase);
          annotation(
            Inline = true);
        end density_ps;

        redeclare function pressure_dT "Computes pressure as a function of density and temperature"
          extends Modelica.Icons.Function;
          input Density d "Density";
          input Temperature T "Temperature";
          input FixedPhase phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          output AbsolutePressure p "Pressure";
        algorithm
          p := IF97_Utilities.p_dT(d, T, phase);
          annotation(
            Inline = true);
        end pressure_dT;

        redeclare function specificEnthalpy_dT "Computes specific enthalpy as a function of density and temperature"
          extends Modelica.Icons.Function;
          input Density d "Density";
          input Temperature T "Temperature";
          input FixedPhase phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          output SpecificEnthalpy h "Specific enthalpy";
        algorithm
          h := IF97_Utilities.h_dT(d, T, phase);
          annotation(
            Inline = true);
        end specificEnthalpy_dT;

        redeclare function specificEnthalpy_pT "Computes specific enthalpy as a function of pressure and temperature"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input Temperature T "Temperature";
          input FixedPhase phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          output SpecificEnthalpy h "Specific enthalpy";
        algorithm
          h := IF97_Utilities.h_pT(p, T);
          annotation(
            Inline = true);
        end specificEnthalpy_pT;

        redeclare function specificEnthalpy_ps "Computes specific enthalpy as a function of pressure and temperature"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SpecificEntropy s "Specific entropy";
          input FixedPhase phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          output SpecificEnthalpy h "Specific enthalpy";
        algorithm
          h := IF97_Utilities.h_ps(p, s, phase);
          annotation(
            Inline = true);
        end specificEnthalpy_ps;

        redeclare function density_pT "Computes density as a function of pressure and temperature"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input Temperature T "Temperature";
          input FixedPhase phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          output Density d "Density";
        algorithm
          d := IF97_Utilities.rho_pT(p, T);
          annotation(
            Inline = true);
        end density_pT;

        redeclare function extends setDewState "Set the thermodynamic state on the dew line"
          algorithm
            state := ThermodynamicState(phase = phase, p = sat.psat, T = sat.Tsat, h = dewEnthalpy(sat), d = dewDensity(sat));
          annotation(
            Inline = true);
        end setDewState;

        redeclare function extends setBubbleState "Set the thermodynamic state on the bubble line"
          algorithm
            state := ThermodynamicState(phase = phase, p = sat.psat, T = sat.Tsat, h = bubbleEnthalpy(sat), d = bubbleDensity(sat));
          annotation(
            Inline = true);
        end setBubbleState;

        redeclare function extends dynamicViscosity "Dynamic viscosity of water"
          algorithm
            eta := IF97_Utilities.dynamicViscosity(state.d, state.T, state.p, state.phase);
          annotation(
            Inline = true);
        end dynamicViscosity;

        redeclare function extends thermalConductivity "Thermal conductivity of water"
          algorithm
            lambda := IF97_Utilities.thermalConductivity(state.d, state.T, state.p, state.phase);
          annotation(
            Inline = true);
        end thermalConductivity;

        redeclare function extends surfaceTension "Surface tension in two phase region of water"
          algorithm
            sigma := IF97_Utilities.surfaceTension(sat.Tsat);
          annotation(
            Inline = true);
        end surfaceTension;

        redeclare function extends pressure "Return pressure of ideal gas"
          algorithm
            p := state.p;
          annotation(
            Inline = true);
        end pressure;

        redeclare function extends temperature "Return temperature of ideal gas"
          algorithm
            T := state.T;
          annotation(
            Inline = true);
        end temperature;

        redeclare function extends density "Return density of ideal gas"
          algorithm
            d := state.d;
          annotation(
            Inline = true);
        end density;

        redeclare function extends specificEnthalpy "Return specific enthalpy"
            extends Modelica.Icons.Function;

          algorithm
            h := state.h;
          annotation(
            Inline = true);
        end specificEnthalpy;

        redeclare function extends specificInternalEnergy "Return specific internal energy"
            extends Modelica.Icons.Function;

          algorithm
            u := state.h - state.p / state.d;
          annotation(
            Inline = true);
        end specificInternalEnergy;

        redeclare function extends specificGibbsEnergy "Return specific Gibbs energy"
            extends Modelica.Icons.Function;

          algorithm
            g := state.h - state.T * specificEntropy(state);
          annotation(
            Inline = true);
        end specificGibbsEnergy;

        redeclare function extends specificHelmholtzEnergy "Return specific Helmholtz energy"
            extends Modelica.Icons.Function;

          algorithm
            f := state.h - state.p / state.d - state.T * specificEntropy(state);
          annotation(
            Inline = true);
        end specificHelmholtzEnergy;

        redeclare function extends specificEntropy "Specific entropy of water"
          algorithm
            s := if dT_explicit then IF97_Utilities.s_dT(state.d, state.T, state.phase) else if pT_explicit then IF97_Utilities.s_pT(state.p, state.T) else IF97_Utilities.s_ph(state.p, state.h, state.phase);
          annotation(
            Inline = true);
        end specificEntropy;

        redeclare function extends specificHeatCapacityCp "Specific heat capacity at constant pressure of water"
          algorithm
            cp := if dT_explicit then IF97_Utilities.cp_dT(state.d, state.T, state.phase) else if pT_explicit then IF97_Utilities.cp_pT(state.p, state.T) else IF97_Utilities.cp_ph(state.p, state.h, state.phase);
          annotation(
            Inline = true,
            Documentation(info = "<html>
                                <p>In the two phase region this function returns the interpolated heat capacity between the
                                liquid and vapour state heat capacities.</p>
                                </html>"));
        end specificHeatCapacityCp;

        redeclare function extends specificHeatCapacityCv "Specific heat capacity at constant volume of water"
          algorithm
            cv := if dT_explicit then IF97_Utilities.cv_dT(state.d, state.T, state.phase) else if pT_explicit then IF97_Utilities.cv_pT(state.p, state.T) else IF97_Utilities.cv_ph(state.p, state.h, state.phase);
          annotation(
            Inline = true);
        end specificHeatCapacityCv;

        redeclare function extends isentropicExponent "Return isentropic exponent"
          algorithm
            gamma := if dT_explicit then IF97_Utilities.isentropicExponent_dT(state.d, state.T, state.phase) else if pT_explicit then IF97_Utilities.isentropicExponent_pT(state.p, state.T) else IF97_Utilities.isentropicExponent_ph(state.p, state.h, state.phase);
        end isentropicExponent;

        redeclare function extends isothermalCompressibility "Isothermal compressibility of water"
          algorithm
            kappa := if dT_explicit then IF97_Utilities.kappa_dT(state.d, state.T, state.phase) else if pT_explicit then IF97_Utilities.kappa_pT(state.p, state.T) else IF97_Utilities.kappa_ph(state.p, state.h, state.phase);
          annotation(
            Inline = true);
        end isothermalCompressibility;

        redeclare function extends isobaricExpansionCoefficient "Isobaric expansion coefficient of water"
          algorithm
            beta := if dT_explicit then IF97_Utilities.beta_dT(state.d, state.T, state.phase) else if pT_explicit then IF97_Utilities.beta_pT(state.p, state.T) else IF97_Utilities.beta_ph(state.p, state.h, state.phase);
          annotation(
            Inline = true);
        end isobaricExpansionCoefficient;

        redeclare function extends velocityOfSound "Return velocity of sound as a function of the thermodynamic state record"
          algorithm
            a := if dT_explicit then IF97_Utilities.velocityOfSound_dT(state.d, state.T, state.phase) else if pT_explicit then IF97_Utilities.velocityOfSound_pT(state.p, state.T) else IF97_Utilities.velocityOfSound_ph(state.p, state.h, state.phase);
          annotation(
            Inline = true);
        end velocityOfSound;

        redeclare function extends isentropicEnthalpy "Compute h(p,s)"
          algorithm
            h_is := IF97_Utilities.isentropicEnthalpy(p_downstream, specificEntropy(refState), 0);
          annotation(
            Inline = true);
        end isentropicEnthalpy;

        redeclare function extends density_derh_p "Density derivative by specific enthalpy"
          algorithm
            ddhp := IF97_Utilities.ddhp(state.p, state.h, state.phase);
          annotation(
            Inline = true);
        end density_derh_p;

        redeclare function extends density_derp_h "Density derivative by pressure"
          algorithm
            ddph := IF97_Utilities.ddph(state.p, state.h, state.phase);
          annotation(
            Inline = true);
        end density_derp_h;

        //   redeclare function extends density_derT_p
        //     "Density derivative by temperature"
        //   algorithm
        //     ddTp := IF97_Utilities.ddTp(state.p, state.h, state.phase);
        //   end density_derT_p;
        //
        //   redeclare function extends density_derp_T
        //     "Density derivative by pressure"
        //   algorithm
        //     ddpT := IF97_Utilities.ddpT(state.p, state.h, state.phase);
        //   end density_derp_T;

        redeclare function extends bubbleEnthalpy "Boiling curve specific enthalpy of water"
          algorithm
            hl := IF97_Utilities.BaseIF97.Regions.hl_p(sat.psat);
          annotation(
            Inline = true);
        end bubbleEnthalpy;

        redeclare function extends dewEnthalpy "Dew curve specific enthalpy of water"
          algorithm
            hv := IF97_Utilities.BaseIF97.Regions.hv_p(sat.psat);
          annotation(
            Inline = true);
        end dewEnthalpy;

        redeclare function extends bubbleEntropy "Boiling curve specific entropy of water"
          algorithm
            sl := IF97_Utilities.BaseIF97.Regions.sl_p(sat.psat);
          annotation(
            Inline = true);
        end bubbleEntropy;

        redeclare function extends dewEntropy "Dew curve specific entropy of water"
          algorithm
            sv := IF97_Utilities.BaseIF97.Regions.sv_p(sat.psat);
          annotation(
            Inline = true);
        end dewEntropy;

        redeclare function extends bubbleDensity "Boiling curve specific density of water"
          algorithm
            dl := if ph_explicit then IF97_Utilities.BaseIF97.Regions.rhol_p(sat.psat) else IF97_Utilities.BaseIF97.Regions.rhol_T(sat.Tsat);
          annotation(
            Inline = true);
        end bubbleDensity;

        redeclare function extends dewDensity "Dew curve specific density of water"
          algorithm
            dv := if ph_explicit or pT_explicit then IF97_Utilities.BaseIF97.Regions.rhov_p(sat.psat) else IF97_Utilities.BaseIF97.Regions.rhov_T(sat.Tsat);
          annotation(
            Inline = true);
        end dewDensity;

        redeclare function extends saturationTemperature "Saturation temperature of water"
          algorithm
            T := IF97_Utilities.BaseIF97.Basic.tsat(p);
          annotation(
            Inline = true);
        end saturationTemperature;

        redeclare function extends saturationTemperature_derp "Derivative of saturation temperature w.r.t. pressure"
          algorithm
            dTp := IF97_Utilities.BaseIF97.Basic.dtsatofp(p);
          annotation(
            Inline = true);
        end saturationTemperature_derp;

        redeclare function extends saturationPressure "Saturation pressure of water"
          algorithm
            p := IF97_Utilities.BaseIF97.Basic.psat(T);
          annotation(
            Inline = true);
        end saturationPressure;

        redeclare function extends dBubbleDensity_dPressure "Bubble point density derivative"
          algorithm
            ddldp := IF97_Utilities.BaseIF97.Regions.drhol_dp(sat.psat);
          annotation(
            Inline = true);
        end dBubbleDensity_dPressure;

        redeclare function extends dDewDensity_dPressure "Dew point density derivative"
          algorithm
            ddvdp := IF97_Utilities.BaseIF97.Regions.drhov_dp(sat.psat);
          annotation(
            Inline = true);
        end dDewDensity_dPressure;

        redeclare function extends dBubbleEnthalpy_dPressure "Bubble point specific enthalpy derivative"
          algorithm
            dhldp := IF97_Utilities.BaseIF97.Regions.dhl_dp(sat.psat);
          annotation(
            Inline = true);
        end dBubbleEnthalpy_dPressure;

        redeclare function extends dDewEnthalpy_dPressure "Dew point specific enthalpy derivative"
          algorithm
            dhvdp := IF97_Utilities.BaseIF97.Regions.dhv_dp(sat.psat);
          annotation(
            Inline = true);
        end dDewEnthalpy_dPressure;

        redeclare function extends setState_dTX "Return thermodynamic state of water as function of d and T"
          algorithm
            state := ThermodynamicState(d = d, T = T, phase = 0, h = specificEnthalpy_dT(d, T), p = pressure_dT(d, T));
          annotation(
            Inline = true);
        end setState_dTX;

        redeclare function extends setState_phX "Return thermodynamic state of water as function of p and h"
          algorithm
            state := ThermodynamicState(d = density_ph(p, h), T = temperature_ph(p, h), phase = 0, h = h, p = p);
          annotation(
            Inline = true);
        end setState_phX;

        redeclare function extends setState_psX "Return thermodynamic state of water as function of p and s"
          algorithm
            state := ThermodynamicState(d = density_ps(p, s), T = temperature_ps(p, s), phase = 0, h = specificEnthalpy_ps(p, s), p = p);
          annotation(
            Inline = true);
        end setState_psX;

        redeclare function extends setState_pTX "Return thermodynamic state of water as function of p and T"
          algorithm
            state := ThermodynamicState(d = density_pT(p, T), T = T, phase = 1, h = specificEnthalpy_pT(p, T), p = p);
          annotation(
            Inline = true);
        end setState_pTX;

        redeclare function extends setSmoothState "Return thermodynamic state so that it smoothly approximates: if x > 0 then state_a else state_b"
            import Modelica.Media.Common.smoothStep;

          algorithm
            state := ThermodynamicState(p = smoothStep(x, state_a.p, state_b.p, x_small), h = smoothStep(x, state_a.h, state_b.h, x_small), d = density_ph(smoothStep(x, state_a.p, state_b.p, x_small), smoothStep(x, state_a.h, state_b.h, x_small)), T = temperature_ph(smoothStep(x, state_a.p, state_b.p, x_small), smoothStep(x, state_a.h, state_b.h, x_small)), phase = 0);
          annotation(
            Inline = true);
        end setSmoothState;
        annotation(
          Documentation(info = "<HTML>
<p>
This model calculates medium properties
for water in the <b>liquid</b>, <b>gas</b> and <b>two phase</b> regions
according to the IAPWS/IF97 standard, i.e., the accepted industrial standard
and best compromise between accuracy and computation time.
For more details see <a href=\"modelica://Modelica.Media.Water.IF97_Utilities\">
Modelica.Media.Water.IF97_Utilities</a>. Three variable pairs can be the
independent variables of the model:
</p>
<ol>
<li>Pressure <b>p</b> and specific enthalpy <b>h</b> are the most natural choice for general applications. This is the recommended choice for most general purpose applications, in particular for power plants.</li>
<li>Pressure <b>p</b> and temperature <b>T</b> are the most natural choice for applications where water is always in the same phase, both for liquid water and steam.</li>
<li>Density <b>d</b> and temperature <b>T</b> are explicit variables of the Helmholtz function in the near-critical region and can be the best choice for applications with super-critical or near-critical states.</li>
</ol>
<p>
The following quantities are always computed:
</p>
<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"><b>Variable</b></td>
      <td valign=\"top\"><b>Unit</b></td>
      <td valign=\"top\"><b>Description</b></td></tr>
  <tr><td valign=\"top\">T</td>
      <td valign=\"top\">K</td>
      <td valign=\"top\">temperature</td></tr>
  <tr><td valign=\"top\">u</td>
      <td valign=\"top\">J/kg</td>
      <td valign=\"top\">specific internal energy</td></tr>
  <tr><td valign=\"top\">d</td>
      <td valign=\"top\">kg/m^3</td>
      <td valign=\"top\">density</td></tr>
  <tr><td valign=\"top\">p</td>
      <td valign=\"top\">Pa</td>
      <td valign=\"top\">pressure</td></tr>
  <tr><td valign=\"top\">h</td>
      <td valign=\"top\">J/kg</td>
      <td valign=\"top\">specific enthalpy</td></tr>
</table>
<p>
In some cases additional medium properties are needed.
A component that needs these optional properties has to call
one of the functions listed in
<a href=\"modelica://Modelica.Media.UsersGuide.MediumUsage.OptionalProperties\">
Modelica.Media.UsersGuide.MediumUsage.OptionalProperties</a> and in
<a href=\"modelica://Modelica.Media.UsersGuide.MediumUsage.TwoPhase\">
Modelica.Media.UsersGuide.MediumUsage.TwoPhase</a>.
</p>
<p>Many further properties can be computed. Using the well-known Bridgman's Tables, all first partial derivatives of the standard thermodynamic variables can be computed easily.</p>
</html>"));
      end WaterIF97_base;

      partial package WaterIF97_fixedregion "Water: Steam properties as defined by IAPWS/IF97 standard"
        extends Interfaces.PartialTwoPhaseMedium(mediumName = "WaterIF97", substanceNames = {"water"}, singleState = false, SpecificEnthalpy(start = 1.0e5, nominal = 5.0e5), Density(start = 150, nominal = 500), AbsolutePressure(start = 50e5, nominal = 10e5), Temperature(start = 500, nominal = 500), smoothModel = false, onePhase = false, fluidConstants = waterConstants);

        redeclare record extends ThermodynamicState "Thermodynamic state"
            SpecificEnthalpy h "Specific enthalpy";
            Density d "Density";
            Temperature T "Temperature";
            AbsolutePressure p "Pressure";
        end ThermodynamicState;

        constant Integer Region "Region of IF97, if known";
        constant Boolean ph_explicit "True if explicit in pressure and specific enthalpy";
        constant Boolean dT_explicit "True if explicit in density and temperature";
        constant Boolean pT_explicit "True if explicit in pressure and temperature";

        redeclare replaceable model extends BaseProperties(h(stateSelect = if ph_explicit then StateSelect.prefer else StateSelect.default), d(stateSelect = if dT_explicit then StateSelect.prefer else StateSelect.default), T(stateSelect = if pT_explicit or dT_explicit then StateSelect.prefer else StateSelect.default), p(stateSelect = if pT_explicit or ph_explicit then StateSelect.prefer else StateSelect.default)) "Base properties of water"
            Integer phase(min = 0, max = 2) "2 for two-phase, 1 for one-phase, 0 if not known";
            SaturationProperties sat(Tsat(start = 300.0), psat(start = 1.0e5)) "Saturation temperature and pressure";

          equation
            MM = fluidConstants[1].molarMass;
            if smoothModel then
              if onePhase then
                phase = 1;
                if ph_explicit then
                  assert(h < bubbleEnthalpy(sat) or h > dewEnthalpy(sat) or p > fluidConstants[1].criticalPressure, "With onePhase=true this model may only be called with one-phase states h < hl or h > hv!");
                else
                  assert(not (d < bubbleDensity(sat) and d > dewDensity(sat) and T < fluidConstants[1].criticalTemperature), "With onePhase=true this model may only be called with one-phase states d > dl or d < dv!");
                end if;
              else
                phase = 0;
              end if;
            else
              if ph_explicit then
                phase = if h < bubbleEnthalpy(sat) or h > dewEnthalpy(sat) or p > fluidConstants[1].criticalPressure then 1 else 2;
              elseif dT_explicit then
                phase = if not (d < bubbleDensity(sat) and d > dewDensity(sat) and T < fluidConstants[1].criticalTemperature) then 1 else 2;
              else
                phase = 1;
//this is for the one-phase only case pT
              end if;
            end if;
            if dT_explicit then
              p = pressure_dT(d, T, phase, Region);
              h = specificEnthalpy_dT(d, T, phase, Region);
              sat.Tsat = T;
              sat.psat = saturationPressure(T);
            elseif ph_explicit then
              d = density_ph(p, h, phase, Region);
              T = temperature_ph(p, h, phase, Region);
              sat.Tsat = saturationTemperature(p);
              sat.psat = p;
            else
              h = specificEnthalpy_pT(p, T, Region);
              d = density_pT(p, T, Region);
              sat.psat = p;
              sat.Tsat = saturationTemperature(p);
            end if;
            u = h - p / d;
            R = Modelica.Constants.R / fluidConstants[1].molarMass;
            h = state.h;
            p = state.p;
            T = state.T;
            d = state.d;
            phase = state.phase;
        end BaseProperties;

        redeclare function density_ph "Computes density as a function of pressure and specific enthalpy"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SpecificEnthalpy h "Specific enthalpy";
          input FixedPhase phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          input Integer region = 0 "If 0, region is unknown, otherwise known and this input";
          output Density d "Density";
        algorithm
          d := IF97_Utilities.rho_ph(p, h, phase, region);
          annotation(
            Inline = true);
        end density_ph;

        redeclare function temperature_ph "Computes temperature as a function of pressure and specific enthalpy"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SpecificEnthalpy h "Specific enthalpy";
          input FixedPhase phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          input Integer region = 0 "If 0, region is unknown, otherwise known and this input";
          output Temperature T "Temperature";
        algorithm
          T := IF97_Utilities.T_ph(p, h, phase, region);
          annotation(
            Inline = true);
        end temperature_ph;

        redeclare function temperature_ps "Compute temperature from pressure and specific enthalpy"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SpecificEntropy s "Specific entropy";
          input FixedPhase phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          input Integer region = 0 "If 0, region is unknown, otherwise known and this input";
          output Temperature T "Temperature";
        algorithm
          T := IF97_Utilities.T_ps(p, s, phase, region);
          annotation(
            Inline = true);
        end temperature_ps;

        redeclare function density_ps "Computes density as a function of pressure and specific enthalpy"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SpecificEntropy s "Specific entropy";
          input FixedPhase phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          input Integer region = 0 "If 0, region is unknown, otherwise known and this input";
          output Density d "Density";
        algorithm
          d := IF97_Utilities.rho_ps(p, s, phase, region);
          annotation(
            Inline = true);
        end density_ps;

        redeclare function pressure_dT "Computes pressure as a function of density and temperature"
          extends Modelica.Icons.Function;
          input Density d "Density";
          input Temperature T "Temperature";
          input FixedPhase phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          input Integer region = 0 "If 0, region is unknown, otherwise known and this input";
          output AbsolutePressure p "Pressure";
        algorithm
          p := IF97_Utilities.p_dT(d, T, phase, region);
          annotation(
            Inline = true);
        end pressure_dT;

        redeclare function specificEnthalpy_dT "Computes specific enthalpy as a function of density and temperature"
          extends Modelica.Icons.Function;
          input Density d "Density";
          input Temperature T "Temperature";
          input FixedPhase phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          input Integer region = 0 "If 0, region is unknown, otherwise known and this input";
          output SpecificEnthalpy h "Specific enthalpy";
        algorithm
          h := IF97_Utilities.h_dT(d, T, phase, region);
          annotation(
            Inline = true);
        end specificEnthalpy_dT;

        redeclare function specificEnthalpy_pT "Computes specific enthalpy as a function of pressure and temperature"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input Temperature T "Temperature";
          input FixedPhase phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          input Integer region = 0 "If 0, region is unknown, otherwise known and this input";
          output SpecificEnthalpy h "Specific enthalpy";
        algorithm
          h := IF97_Utilities.h_pT(p, T, region);
          annotation(
            Inline = true);
        end specificEnthalpy_pT;

        redeclare function specificEnthalpy_ps "Computes specific enthalpy as a function of pressure and temperature"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SpecificEntropy s "Specific entropy";
          input FixedPhase phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          input Integer region = 0 "If 0, region is unknown, otherwise known and this input";
          output SpecificEnthalpy h "Specific enthalpy";
        algorithm
          h := IF97_Utilities.h_ps(p, s, phase, region);
          annotation(
            Inline = true);
        end specificEnthalpy_ps;

        redeclare function density_pT "Computes density as a function of pressure and temperature"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input Temperature T "Temperature";
          input FixedPhase phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          input Integer region = 0 "If 0, region is unknown, otherwise known and this input";
          output Density d "Density";
        algorithm
          d := IF97_Utilities.rho_pT(p, T, region);
          annotation(
            Inline = true);
        end density_pT;

        redeclare function extends setDewState "Set the thermodynamic state on the dew line"
          algorithm
            state := ThermodynamicState(phase = phase, p = sat.psat, T = sat.Tsat, h = dewEnthalpy(sat), d = dewDensity(sat));
          annotation(
            Inline = true);
        end setDewState;

        redeclare function extends setBubbleState "Set the thermodynamic state on the bubble line"
          algorithm
            state := ThermodynamicState(phase = phase, p = sat.psat, T = sat.Tsat, h = bubbleEnthalpy(sat), d = bubbleDensity(sat));
          annotation(
            Inline = true);
        end setBubbleState;

        redeclare function extends dynamicViscosity "Dynamic viscosity of water"
          algorithm
            eta := IF97_Utilities.dynamicViscosity(state.d, state.T, state.p, state.phase);
          annotation(
            Inline = true);
        end dynamicViscosity;

        redeclare function extends thermalConductivity "Thermal conductivity of water"
          algorithm
            lambda := IF97_Utilities.thermalConductivity(state.d, state.T, state.p, state.phase);
          annotation(
            Inline = true);
        end thermalConductivity;

        redeclare function extends surfaceTension "Surface tension in two phase region of water"
          algorithm
            sigma := IF97_Utilities.surfaceTension(sat.Tsat);
          annotation(
            Inline = true);
        end surfaceTension;

        redeclare function extends pressure "Return pressure of ideal gas"
          algorithm
            p := state.p;
          annotation(
            Inline = true);
        end pressure;

        redeclare function extends temperature "Return temperature of ideal gas"
          algorithm
            T := state.T;
          annotation(
            Inline = true);
        end temperature;

        redeclare function extends density "Return density of ideal gas"
          algorithm
            d := state.d;
          annotation(
            Inline = true);
        end density;

        redeclare function extends specificEnthalpy "Return specific enthalpy"
            extends Modelica.Icons.Function;

          algorithm
            h := state.h;
          annotation(
            Inline = true);
        end specificEnthalpy;

        redeclare function extends specificInternalEnergy "Return specific internal energy"
            extends Modelica.Icons.Function;

          algorithm
            u := state.h - state.p / state.d;
          annotation(
            Inline = true);
        end specificInternalEnergy;

        redeclare function extends specificGibbsEnergy "Return specific Gibbs energy"
            extends Modelica.Icons.Function;

          algorithm
            g := state.h - state.T * specificEntropy(state);
          annotation(
            Inline = true);
        end specificGibbsEnergy;

        redeclare function extends specificHelmholtzEnergy "Return specific Helmholtz energy"
            extends Modelica.Icons.Function;

          algorithm
            f := state.h - state.p / state.d - state.T * specificEntropy(state);
          annotation(
            Inline = true);
        end specificHelmholtzEnergy;

        redeclare function extends specificEntropy "Specific entropy of water"
          algorithm
            s := if dT_explicit then IF97_Utilities.s_dT(state.d, state.T, state.phase, Region) else if pT_explicit then IF97_Utilities.s_pT(state.p, state.T, Region) else IF97_Utilities.s_ph(state.p, state.h, state.phase, Region);
          annotation(
            Inline = true);
        end specificEntropy;

        redeclare function extends specificHeatCapacityCp "Specific heat capacity at constant pressure of water"
          algorithm
            cp := if dT_explicit then IF97_Utilities.cp_dT(state.d, state.T, Region) else if pT_explicit then IF97_Utilities.cp_pT(state.p, state.T, Region) else IF97_Utilities.cp_ph(state.p, state.h, Region);
          annotation(
            Inline = true);
        end specificHeatCapacityCp;

        redeclare function extends specificHeatCapacityCv "Specific heat capacity at constant volume of water"
          algorithm
            cv := if dT_explicit then IF97_Utilities.cv_dT(state.d, state.T, state.phase, Region) else if pT_explicit then IF97_Utilities.cv_pT(state.p, state.T, Region) else IF97_Utilities.cv_ph(state.p, state.h, state.phase, Region);
          annotation(
            Inline = true);
        end specificHeatCapacityCv;

        redeclare function extends isentropicExponent "Return isentropic exponent"
          algorithm
            gamma := if dT_explicit then IF97_Utilities.isentropicExponent_dT(state.d, state.T, state.phase, Region) else if pT_explicit then IF97_Utilities.isentropicExponent_pT(state.p, state.T, Region) else IF97_Utilities.isentropicExponent_ph(state.p, state.h, state.phase, Region);
          annotation(
            Inline = true);
        end isentropicExponent;

        redeclare function extends isothermalCompressibility "Isothermal compressibility of water"
          algorithm
//    assert(state.phase <> 2, "Isothermal compressibility can not be computed with 2-phase inputs!");
            kappa := if dT_explicit then IF97_Utilities.kappa_dT(state.d, state.T, state.phase, Region) else if pT_explicit then IF97_Utilities.kappa_pT(state.p, state.T, Region) else IF97_Utilities.kappa_ph(state.p, state.h, state.phase, Region);
          annotation(
            Inline = true);
        end isothermalCompressibility;

        redeclare function extends isobaricExpansionCoefficient "Isobaric expansion coefficient of water"
          algorithm
//    assert(state.phase <> 2, "The isobaric expansion coefficient can not be computed with 2-phase inputs!");
            beta := if dT_explicit then IF97_Utilities.beta_dT(state.d, state.T, state.phase, Region) else if pT_explicit then IF97_Utilities.beta_pT(state.p, state.T, Region) else IF97_Utilities.beta_ph(state.p, state.h, state.phase, Region);
          annotation(
            Inline = true);
        end isobaricExpansionCoefficient;

        redeclare function extends velocityOfSound "Return velocity of sound as a function of the thermodynamic state record"
          algorithm
            a := if dT_explicit then IF97_Utilities.velocityOfSound_dT(state.d, state.T, state.phase, Region) else if pT_explicit then IF97_Utilities.velocityOfSound_pT(state.p, state.T, Region) else IF97_Utilities.velocityOfSound_ph(state.p, state.h, state.phase, Region);
          annotation(
            Inline = true);
        end velocityOfSound;

        redeclare function extends isentropicEnthalpy "Compute h(s,p)"
          algorithm
            h_is := IF97_Utilities.isentropicEnthalpy(p_downstream, specificEntropy(refState), 0);
          annotation(
            Inline = true);
        end isentropicEnthalpy;

        redeclare function extends density_derh_p "Density derivative by specific enthalpy"
          algorithm
            ddhp := IF97_Utilities.ddhp(state.p, state.h, state.phase, Region);
          annotation(
            Inline = true);
        end density_derh_p;

        redeclare function extends density_derp_h "Density derivative by pressure"
          algorithm
            ddph := IF97_Utilities.ddph(state.p, state.h, state.phase, Region);
          annotation(
            Inline = true);
        end density_derp_h;

        redeclare function extends bubbleEnthalpy "Boiling curve specific enthalpy of water"
          algorithm
            hl := IF97_Utilities.BaseIF97.Regions.hl_p(sat.psat);
          annotation(
            Inline = true);
        end bubbleEnthalpy;

        redeclare function extends dewEnthalpy "Dew curve specific enthalpy of water"
          algorithm
            hv := IF97_Utilities.BaseIF97.Regions.hv_p(sat.psat);
          annotation(
            Inline = true);
        end dewEnthalpy;

        redeclare function extends bubbleEntropy "Boiling curve specific entropy of water"
          algorithm
            sl := IF97_Utilities.BaseIF97.Regions.sl_p(sat.psat);
          annotation(
            Inline = true);
        end bubbleEntropy;

        redeclare function extends dewEntropy "Dew curve specific entropy of water"
          algorithm
            sv := IF97_Utilities.BaseIF97.Regions.sv_p(sat.psat);
          annotation(
            Inline = true);
        end dewEntropy;

        redeclare function extends bubbleDensity "Boiling curve specific density of water"
          algorithm
            dl := if ph_explicit or pT_explicit then IF97_Utilities.BaseIF97.Regions.rhol_p(sat.psat) else IF97_Utilities.BaseIF97.Regions.rhol_T(sat.Tsat);
          annotation(
            Inline = true);
        end bubbleDensity;

        redeclare function extends dewDensity "Dew curve specific density of water"
          algorithm
            dv := if ph_explicit or pT_explicit then IF97_Utilities.BaseIF97.Regions.rhov_p(sat.psat) else IF97_Utilities.BaseIF97.Regions.rhov_T(sat.Tsat);
          annotation(
            Inline = true);
        end dewDensity;

        redeclare function extends saturationTemperature "Saturation temperature of water"
          algorithm
            T := IF97_Utilities.BaseIF97.Basic.tsat(p);
          annotation(
            Inline = true);
        end saturationTemperature;

        redeclare function extends saturationTemperature_derp "Derivative of saturation temperature w.r.t. pressure"
          algorithm
            dTp := IF97_Utilities.BaseIF97.Basic.dtsatofp(p);
          annotation(
            Inline = true);
        end saturationTemperature_derp;

        redeclare function extends saturationPressure "Saturation pressure of water"
          algorithm
            p := IF97_Utilities.BaseIF97.Basic.psat(T);
          annotation(
            Inline = true);
        end saturationPressure;

        redeclare function extends dBubbleDensity_dPressure "Bubble point density derivative"
          algorithm
            ddldp := IF97_Utilities.BaseIF97.Regions.drhol_dp(sat.psat);
          annotation(
            Inline = true);
        end dBubbleDensity_dPressure;

        redeclare function extends dDewDensity_dPressure "Dew point density derivative"
          algorithm
            ddvdp := IF97_Utilities.BaseIF97.Regions.drhov_dp(sat.psat);
          annotation(
            Inline = true);
        end dDewDensity_dPressure;

        redeclare function extends dBubbleEnthalpy_dPressure "Bubble point specific enthalpy derivative"
          algorithm
            dhldp := IF97_Utilities.BaseIF97.Regions.dhl_dp(sat.psat);
          annotation(
            Inline = true);
        end dBubbleEnthalpy_dPressure;

        redeclare function extends dDewEnthalpy_dPressure "Dew point specific enthalpy derivative"
          algorithm
            dhvdp := IF97_Utilities.BaseIF97.Regions.dhv_dp(sat.psat);
          annotation(
            Inline = true);
        end dDewEnthalpy_dPressure;

        redeclare function extends setState_dTX "Return thermodynamic state of water as function of d, T, and optional region"
            input Integer region = 0 "If 0, region is unknown, otherwise known and this input";

          algorithm
            state := ThermodynamicState(d = d, T = T, phase = if region == 0 then IF97_Utilities.phase_dT(d, T) else if region == 4 then 2 else 1, h = specificEnthalpy_dT(d, T, region = region), p = pressure_dT(d, T, region = region));
          annotation(
            Inline = true);
        end setState_dTX;

        redeclare function extends setState_phX "Return thermodynamic state of water as function of p, h, and optional region"
            input Integer region = 0 "If 0, region is unknown, otherwise known and this input";

          algorithm
            state := ThermodynamicState(d = density_ph(p, h, region = region), T = temperature_ph(p, h, region = region), phase = if region == 0 then IF97_Utilities.phase_ph(p, h) else if region == 4 then 2 else 1, h = h, p = p);
          annotation(
            Inline = true);
        end setState_phX;

        redeclare function extends setState_psX "Return thermodynamic state of water as function of p, s, and optional region"
            input Integer region = 0 "If 0, region is unknown, otherwise known and this input";

          algorithm
            state := ThermodynamicState(d = density_ps(p, s, region = region), T = temperature_ps(p, s, region = region), phase = IF97_Utilities.phase_ps(p, s), h = specificEnthalpy_ps(p, s, region = region), p = p);
          annotation(
            Inline = true);
        end setState_psX;

        redeclare function extends setState_pTX "Return thermodynamic state of water as function of p, T, and optional region"
            input Integer region = 0 "If 0, region is unknown, otherwise known and this input";

          algorithm
            state := ThermodynamicState(d = density_pT(p, T, region = region), T = T, phase = 1, h = specificEnthalpy_pT(p, T, region = region), p = p);
          annotation(
            Inline = true);
        end setState_pTX;

        redeclare function extends setSmoothState "Return thermodynamic state so that it smoothly approximates: if x > 0 then state_a else state_b"
            import Modelica.Media.Common.smoothStep;

          algorithm
            state := ThermodynamicState(p = smoothStep(x, state_a.p, state_b.p, x_small), h = smoothStep(x, state_a.h, state_b.h, x_small), d = density_ph(smoothStep(x, state_a.p, state_b.p, x_small), smoothStep(x, state_a.h, state_b.h, x_small)), T = temperature_ph(smoothStep(x, state_a.p, state_b.p, x_small), smoothStep(x, state_a.h, state_b.h, x_small)), phase = 0);
          annotation(
            Inline = true);
        end setSmoothState;
        annotation(
          Documentation(info = "<HTML>
<p>
This model calculates medium properties
for water in the <b>liquid</b>, <b>gas</b> and <b>two phase</b> regions
according to the IAPWS/IF97 standard, i.e., the accepted industrial standard
and best compromise between accuracy and computation time.
For more details see <a href=\"modelica://Modelica.Media.Water.IF97_Utilities\">
Modelica.Media.Water.IF97_Utilities</a>. Three variable pairs can be the
independent variables of the model:
</p>
<ol>
<li>Pressure <b>p</b> and specific enthalpy <b>h</b> are the most natural choice for general applications. This is the recommended choice for most general purpose applications, in particular for power plants.</li>
<li>Pressure <b>p</b> and temperature <b>T</b> are the most natural choice for applications where water is always in the same phase, both for liquid water and steam.</li>
<li>Density <b>d</b> and temperature <b>T</b> are explicit variables of the Helmholtz function in the near-critical region and can be the best choice for applications with super-critical or near-critical states.</li>
</ol>
<p>
The following quantities are always computed:
</p>
<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"><b>Variable</b></td>
      <td valign=\"top\"><b>Unit</b></td>
      <td valign=\"top\"><b>Description</b></td></tr>
  <tr><td valign=\"top\">T</td>
      <td valign=\"top\">K</td>
      <td valign=\"top\">temperature</td></tr>
  <tr><td valign=\"top\">u</td>
      <td valign=\"top\">J/kg</td>
      <td valign=\"top\">specific internal energy</td></tr>
  <tr><td valign=\"top\">d</td>
      <td valign=\"top\">kg/m^3</td>
      <td valign=\"top\">density</td></tr>
  <tr><td valign=\"top\">p</td>
      <td valign=\"top\">Pa</td>
      <td valign=\"top\">pressure</td></tr>
  <tr><td valign=\"top\">h</td>
      <td valign=\"top\">J/kg</td>
      <td valign=\"top\">specific enthalpy</td></tr>
</table>
<p>
In some cases additional medium properties are needed.
A component that needs these optional properties has to call
one of the functions listed in
<a href=\"modelica://Modelica.Media.UsersGuide.MediumUsage.OptionalProperties\">
Modelica.Media.UsersGuide.MediumUsage.OptionalProperties</a> and in
<a href=\"modelica://Modelica.Media.UsersGuide.MediumUsage.TwoPhase\">
Modelica.Media.UsersGuide.MediumUsage.TwoPhase</a>.
</p>
<p>Many further properties can be computed. Using the well-known Bridgman's Tables, all first partial derivatives of the standard thermodynamic variables can be computed easily.
</p>
</html>"));
      end WaterIF97_fixedregion;

      package WaterIF97_R4ph "Region 4 water according to IF97 standard"
        extends WaterIF97_fixedregion(final Region = 4, ThermoStates = Modelica.Media.Interfaces.Choices.IndependentVariables.ph, final ph_explicit = true, final dT_explicit = false, final pT_explicit = false, smoothModel = true, onePhase = false);
        annotation(
          Documentation(info = "<html>

</html>"));
      end WaterIF97_R4ph;

      package WaterIF97_R5ph "Region 5 water according to IF97 standard"
        extends WaterIF97_fixedregion(ThermoStates = Modelica.Media.Interfaces.Choices.IndependentVariables.ph, final Region = 5, final ph_explicit = true, final dT_explicit = false, final pT_explicit = false, smoothModel = true, onePhase = true);
        annotation(
          Documentation(info = "<html>

</html>"));
      end WaterIF97_R5ph;

      package WaterIF97_R1pT "Region 1 (liquid) water according to IF97 standard"
        extends WaterIF97_fixedregion(ThermoStates = Modelica.Media.Interfaces.Choices.IndependentVariables.pT, final Region = 1, final ph_explicit = false, final dT_explicit = false, final pT_explicit = true, smoothModel = true, onePhase = true);
        annotation(
          Documentation(info = "<html>

</html>"));
      end WaterIF97_R1pT;

      package WaterIF97_R2pT "Region 2 (steam) water according to IF97 standard"
        extends WaterIF97_fixedregion(ThermoStates = Modelica.Media.Interfaces.Choices.IndependentVariables.pT, final Region = 2, final ph_explicit = false, final dT_explicit = false, final pT_explicit = true, smoothModel = true, onePhase = true);
        annotation(
          Documentation(info = "<html>

</html>"));
      end WaterIF97_R2pT;

      package WaterIF97_R1ph "Region 1 (liquid) water according to IF97 standard"
        extends WaterIF97_fixedregion(ThermoStates = Modelica.Media.Interfaces.Choices.IndependentVariables.ph, final Region = 1, final ph_explicit = true, final dT_explicit = false, final pT_explicit = false, smoothModel = true, onePhase = true);
        annotation(
          Documentation(info = "<html>

</html>"));
      end WaterIF97_R1ph;

      package WaterIF97_R2ph "Region 2 (steam) water according to IF97 standard"
        extends WaterIF97_fixedregion(ThermoStates = Modelica.Media.Interfaces.Choices.IndependentVariables.ph, final Region = 2, final ph_explicit = true, final dT_explicit = false, final pT_explicit = false, smoothModel = true, onePhase = true);
        annotation(
          Documentation(info = "<html>

</html>"));
      end WaterIF97_R2ph;

      package WaterIF97_R3ph "Region 3 water according to IF97 standard"
        extends WaterIF97_fixedregion(final Region = 3, ThermoStates = Modelica.Media.Interfaces.Choices.IndependentVariables.ph, final ph_explicit = true, final dT_explicit = false, final pT_explicit = false, smoothModel = true, onePhase = true);
        annotation(
          Documentation(info = "<html>

</html>"));
      end WaterIF97_R3ph;

      package IF97_Utilities "Low level and utility computation for high accuracy water properties according to the IAPWS/IF97 standard"
        extends Modelica.Icons.UtilitiesPackage;

        package BaseIF97 "Modelica Physical Property Model: the new industrial formulation IAPWS-IF97"
          extends Modelica.Icons.Package;

          record IterationData "Constants for iterations internal to some functions"
            extends Modelica.Icons.Record;
            constant Integer IMAX = 50 "Maximum number of iterations for inverse functions";
            constant Real DELP = 1.0e-6 "Maximum iteration error in pressure, Pa";
            constant Real DELS = 1.0e-8 "Maximum iteration error in specific entropy, J/{kg.K}";
            constant Real DELH = 1.0e-8 "Maximum iteration error in specific enthalpy, J/kg";
            constant Real DELD = 1.0e-8 "Maximum iteration error in density, kg/m^3";
          end IterationData;

          //===================================================================
          //                      Constant declarations
          //===================================================================

          record data "Constant IF97 data and region limits"
            extends Modelica.Icons.Record;
            constant SI.SpecificHeatCapacity RH2O = 461.526 "Specific gas constant of water vapour";
            constant SI.MolarMass MH2O = 0.01801528 "Molar weight of water";
            constant SI.Temperature TSTAR1 = 1386.0 "Normalization temperature for region 1 IF97";
            constant SI.Pressure PSTAR1 = 16.53e6 "Normalization pressure for region 1 IF97";
            constant SI.Temperature TSTAR2 = 540.0 "Normalization temperature for region 2 IF97";
            constant SI.Pressure PSTAR2 = 1.0e6 "Normalization pressure for region 2 IF97";
            constant SI.Temperature TSTAR5 = 1000.0 "Normalization temperature for region 5 IF97";
            constant SI.Pressure PSTAR5 = 1.0e6 "Normalization pressure for region 5 IF97";
            constant SI.SpecificEnthalpy HSTAR1 = 2.5e6 "Normalization specific enthalpy for region 1 IF97";
            constant Real IPSTAR = 1.0e-6 "Normalization pressure for inverse function in region 2 IF97";
            constant Real IHSTAR = 5.0e-7 "Normalization specific enthalpy for inverse function in region 2 IF97";
            constant SI.Temperature TLIMIT1 = 623.15 "Temperature limit between regions 1 and 3";
            constant SI.Temperature TLIMIT2 = 1073.15 "Temperature limit between regions 2 and 5";
            constant SI.Temperature TLIMIT5 = 2273.15 "Upper temperature limit of 5";
            constant SI.Pressure PLIMIT1 = 100.0e6 "Upper pressure limit for regions 1, 2 and 3";
            constant SI.Pressure PLIMIT4A = 16.5292e6 "Pressure limit between regions 1 and 2, important for for two-phase (region 4)";
            constant SI.Pressure PLIMIT5 = 10.0e6 "Upper limit of valid pressure in region 5";
            constant SI.Pressure PCRIT = 22064000.0 "The critical pressure";
            constant SI.Temperature TCRIT = 647.096 "The critical temperature";
            constant SI.Density DCRIT = 322.0 "The critical density";
            constant SI.SpecificEntropy SCRIT = 4412.02148223476 "The calculated specific entropy at the critical point";
            constant SI.SpecificEnthalpy HCRIT = 2087546.84511715 "The calculated specific enthalpy at the critical point";
            constant Real[5] n = array(0.34805185628969e3, -0.11671859879975e1, 0.10192970039326e-2, 0.57254459862746e3, 0.13918839778870e2) "Polynomial coefficients for boundary between regions 2 and 3";
            annotation(
              Documentation(info = "<HTML>
 <h4>Record description</h4>
                           <p>Constants needed in the international steam properties IF97.
                           SCRIT and HCRIT are calculated from Helmholtz function for region 3.</p>
<h4>Version Info and Revision history
</h4>
<ul>
<li>First implemented: <i>July, 2000</i>
       by Hubertus Tummescheit
       </li>
</ul>
 <address>Author: Hubertus Tummescheit, <br>
      Modelon AB<br>
      Ideon Science Park<br>
      SE-22370 Lund, Sweden<br>
      email: hubertus@modelon.se
 </address>
<ul>
 <li>Initial version: July 2000</li>
 <li>Documentation added: December 2002</li>
</ul>
</html>"));
          end data;

          function getTstar "Get normalization temperature for region 1, 2 or 5"
            extends Modelica.Icons.Function;
            input Integer region "IF 97 region";
            output SI.Temperature Tstar "Normalization temperature";
          algorithm
            if region == 1 then
              Tstar := data.TSTAR1;
            elseif region == 2 then
              Tstar := data.TSTAR2;
            else
              Tstar := data.TSTAR5;
            end if;
          end getTstar;

          function getpstar "Get normalization pressure for region 1, 2 or 5"
            extends Modelica.Icons.Function;
            input Integer region "IF 97 region";
            output SI.Pressure pstar "Normalization pressure";
          algorithm
            if region == 1 then
              pstar := data.PSTAR1;
            elseif region == 2 then
              pstar := data.PSTAR2;
            else
              pstar := data.PSTAR5;
            end if;
          end getpstar;

          record critical "Critical point data"
            extends Modelica.Icons.Record;
            constant SI.Pressure PCRIT = 22064000.0 "The critical pressure";
            constant SI.Temperature TCRIT = 647.096 "The critical temperature";
            constant SI.Density DCRIT = 322.0 "The critical density";
            constant SI.SpecificEnthalpy HCRIT = 2087546.84511715 "The calculated specific enthalpy at the critical point";
            constant SI.SpecificEntropy SCRIT = 4412.02148223476 "The calculated specific entropy at the critical point";
            annotation(
              Documentation(info = "<HTML>
 <h4>Record description</h4>
 <p>Critical point data for IF97 steam properties. SCRIT and HCRIT are calculated from Helmholtz function for region 3</p>
<h4>Version Info and Revision history
</h4>
<ul>
<li>First implemented: <i>July, 2000</i>
       by <a href=\"http://www.control.lth.se/~hubertus/\">Hubertus Tummescheit</a>
       </li>
</ul>
 <address>Author: Hubertus Tummescheit, <br>
      Modelon AB<br>
      Ideon Science Park<br>
      SE-22370 Lund, Sweden<br>
      email: hubertus@modelon.se
 </address>
<ul>
 <li>Initial version: July 2000</li>
 <li>Documentation added: December 2002</li>
</ul>
</html>"));
          end critical;

          record triple "Triple point data"
            extends Modelica.Icons.Record;
            constant SI.Temperature Ttriple = 273.16 "The triple point temperature";
            constant SI.Pressure ptriple = 611.657 "The triple point temperature";
            constant SI.Density dltriple = 999.792520031617642 "The triple point liquid density";
            constant SI.Density dvtriple = 0.485457572477861372e-2 "The triple point vapour density";
            annotation(
              Documentation(info = "<HTML>
 <h4>Record description</h4>
 <p>Vapour/liquid/ice triple point data for IF97 steam properties.</p>
<h4>Version Info and Revision history
</h4>
<ul>
<li>First implemented: <i>July, 2000</i>
       by <a href=\"http://www.control.lth.se/~hubertus/\">Hubertus Tummescheit</a>
       </li>
</ul>
 <address>Author: Hubertus Tummescheit, <br>
      Modelon AB<br>
      Ideon Science Park<br>
      SE-22370 Lund, Sweden<br>
      email: hubertus@modelon.se
 </address>
<ul>
 <li>Initial version: July 2000</li>
 <li>Documentation added: December 2002</li>
</ul>
</html>"));
          end triple;

          package Regions "Functions to find the current region for given pairs of input variables"
            extends Modelica.Icons.Package;
            //===================================================================
            //                      "Public" functions
            //===================================================================

            function boundary23ofT "Boundary function for region boundary between regions 2 and 3 (input temperature)"
              extends Modelica.Icons.Function;
              input SI.Temperature t "Temperature (K)";
              output SI.Pressure p "Pressure";
            protected
              constant Real[5] n = data.n;
            algorithm
              p := 1.0e6 * (n[1] + t * (n[2] + t * n[3]));
            end boundary23ofT;

            function boundary23ofp "Boundary function for region boundary between regions 2 and 3 (input pressure)"
              extends Modelica.Icons.Function;
              input SI.Pressure p "Pressure";
              output SI.Temperature t "Temperature (K)";
            protected
              constant Real[5] n = data.n;
              Real pi "Dimensionless pressure";
            algorithm
              pi := p / 1.0e6;
              assert(p > triple.ptriple, "IF97 medium function boundary23ofp called with too low pressure\n" + "p = " + String(p) + " Pa <= " + String(triple.ptriple) + " Pa (triple point pressure)");
              t := n[4] + ((pi - n[5]) / n[3]) ^ 0.5;
            end boundary23ofp;

            function hlowerofp5 "Explicit lower specific enthalpy limit of region 5 as function of pressure"
              extends Modelica.Icons.Function;
              input SI.Pressure p "Pressure";
              output SI.SpecificEnthalpy h "Specific enthalpy";
            protected
              Real pi "Dimensionless pressure";
            algorithm
              pi := p / data.PSTAR5;
              assert(p > triple.ptriple, "IF97 medium function hlowerofp5 called with too low pressure\n" + "p = " + String(p) + " Pa <= " + String(triple.ptriple) + " Pa (triple point pressure)");
              h := 461526. * (9.01505286876203 + pi * ((-0.00979043490246092) + ((-0.0000203245575263501) + 3.36540214679088e-7 * pi) * pi));
            end hlowerofp5;

            function hupperofp5 "Explicit upper specific enthalpy limit of region 5 as function of pressure"
              extends Modelica.Icons.Function;
              input SI.Pressure p "Pressure";
              output SI.SpecificEnthalpy h "Specific enthalpy";
            protected
              Real pi "Dimensionless pressure";
            algorithm
              pi := p / data.PSTAR5;
              assert(p > triple.ptriple, "IF97 medium function hupperofp5 called with too low pressure\n" + "p = " + String(p) + " Pa <= " + String(triple.ptriple) + " Pa (triple point pressure)");
              h := 461526. * (15.9838891400332 + pi * ((-0.000489898813722568) + ((-5.01510211858761e-8) + 7.5006972718273e-8 * pi) * pi));
            end hupperofp5;

            function slowerofp5 "Explicit lower specific entropy limit of region 5 as function of pressure"
              extends Modelica.Icons.Function;
              input SI.Pressure p "Pressure";
              output SI.SpecificEntropy s "Specific entropy";
            protected
              Real pi "Dimensionless pressure";
            algorithm
              pi := p / data.PSTAR5;
              assert(p > triple.ptriple, "IF97 medium function slowerofp5 called with too low pressure\n" + "p = " + String(p) + " Pa <= " + String(triple.ptriple) + " Pa (triple point pressure)");
              s := 461.526 * (18.4296209980112 + pi * ((-0.00730911805860036) + ((-0.0000168348072093888) + 2.09066899426354e-7 * pi) * pi) - Modelica.Math.log(pi));
            end slowerofp5;

            function supperofp5 "Explicit upper specific entropy limit of region 5 as function of pressure"
              extends Modelica.Icons.Function;
              input SI.Pressure p "Pressure";
              output SI.SpecificEntropy s "Specific entropy";
            protected
              Real pi "Dimensionless pressure";
            algorithm
              pi := p / data.PSTAR5;
              assert(p > triple.ptriple, "IF97 medium function supperofp5 called with too low pressure\n" + "p = " + String(p) + " Pa <= " + String(triple.ptriple) + " Pa (triple point pressure)");
              s := 461.526 * (22.7281531474243 + pi * ((-0.000656650220627603) + ((-1.96109739782049e-8) + 2.19979537113031e-8 * pi) * pi) - Modelica.Math.log(pi));
            end supperofp5;

            function hlowerofp1 "Explicit lower specific enthalpy limit of region 1 as function of pressure"
              extends Modelica.Icons.Function;
              input SI.Pressure p "Pressure";
              output SI.SpecificEnthalpy h "Specific enthalpy";
            protected
              Real pi1 "Dimensionless pressure";
              Real[3] o "Vector of auxiliary variables";
            algorithm
              pi1 := 7.1 - p / data.PSTAR1;
              assert(p > triple.ptriple, "IF97 medium function hlowerofp1 called with too low pressure\n" + "p = " + String(p) + " Pa <= " + String(triple.ptriple) + " Pa (triple point pressure)");
              o[1] := pi1 * pi1;
              o[2] := o[1] * o[1];
              o[3] := o[2] * o[2];
              h := 639675.036 * (0.173379420894777 + pi1 * ((-0.022914084306349) + pi1 * ((-0.00017146768241932) + pi1 * ((-4.18695814670391e-6) + pi1 * ((-2.41630417490008e-7) + pi1 * (1.73545618580828e-11 + o[1] * pi1 * (8.43755552264362e-14 + o[2] * o[3] * pi1 * (5.35429206228374e-35 + o[1] * ((-8.12140581014818e-38) + o[1] * o[2] * ((-1.43870236842915e-44) + pi1 * (1.73894459122923e-45 + ((-7.06381628462585e-47) + 9.64504638626269e-49 * pi1) * pi1)))))))))));
            end hlowerofp1;

            function hupperofp1 "Explicit upper specific enthalpy limit of region 1 as function of pressure (meets region 4 saturation pressure curve at 623.15 K)"
              extends Modelica.Icons.Function;
              input SI.Pressure p "Pressure";
              output SI.SpecificEnthalpy h "Specific enthalpy";
            protected
              Real pi1 "Dimensionless pressure";
              Real[3] o "Vector of auxiliary variables";
            algorithm
              pi1 := 7.1 - p / data.PSTAR1;
              assert(p > triple.ptriple, "IF97 medium function hupperofp1 called with too low pressure\n" + "p = " + String(p) + " Pa <= " + String(triple.ptriple) + " Pa (triple point pressure)");
              o[1] := pi1 * pi1;
              o[2] := o[1] * o[1];
              o[3] := o[2] * o[2];
              h := 639675.036 * (2.42896927729349 + pi1 * ((-0.00141131225285294) + pi1 * (0.00143759406818289 + pi1 * (0.000125338925082983 + pi1 * (0.0000123617764767172 + pi1 * (3.17834967400818e-6 + o[1] * pi1 * (1.46754947271665e-8 + o[2] * o[3] * pi1 * (1.86779322717506e-17 + o[1] * ((-4.18568363667416e-19) + o[1] * o[2] * ((-9.19148577641497e-22) + pi1 * (4.27026404402408e-22 + ((-6.66749357417962e-23) + 3.49930466305574e-24 * pi1) * pi1)))))))))));
            end hupperofp1;

            function slowerofp1 "Explicit lower specific entropy limit of region 1 as function of pressure"
              extends Modelica.Icons.Function;
              input SI.Pressure p "Pressure";
              output SI.SpecificEntropy s "Specific entropy";
            protected
              Real pi1 "Dimensionless pressure";
              Real[3] o "Vector of auxiliary variables";
            algorithm
              pi1 := 7.1 - p / data.PSTAR1;
              assert(p > triple.ptriple, "IF97 medium function slowerofp1 called with too low pressure\n" + "p = " + String(p) + " Pa <= " + String(triple.ptriple) + " Pa (triple point pressure)");
              o[1] := pi1 * pi1;
              o[2] := o[1] * o[1];
              o[3] := o[2] * o[2];
              s := 461.526 * ((-0.0268080988194267) + pi1 * (0.00834795890110168 + pi1 * ((-0.000486470924668433) + pi1 * ((-0.0000154902045012264) + pi1 * ((-1.07631751351358e-6) + pi1 * (9.64159058957115e-11 + o[1] * pi1 * (4.81921078863103e-13 + o[2] * o[3] * pi1 * (2.7879623870968e-34 + o[1] * ((-4.22182957646226e-37) + o[1] * o[2] * ((-7.44601427465175e-44) + pi1 * (8.99540001407168e-45 + ((-3.65230274480299e-46) + 4.98464639687285e-48 * pi1) * pi1)))))))))));
            end slowerofp1;

            function supperofp1 "Explicit upper specific entropy limit of region 1 as function of pressure (meets region 4 saturation pressure curve at 623.15 K)"
              extends Modelica.Icons.Function;
              input SI.Pressure p "Pressure";
              output SI.SpecificEntropy s "Specific entropy";
            protected
              Real pi1 "Dimensionless pressure";
              Real[3] o "Vector of auxiliary variables";
            algorithm
              pi1 := 7.1 - p / data.PSTAR1;
              assert(p > triple.ptriple, "IF97 medium function supperofp1 called with too low pressure\n" + "p = " + String(p) + " Pa <= " + String(triple.ptriple) + " Pa (triple point pressure)");
              o[1] := pi1 * pi1;
              o[2] := o[1] * o[1];
              o[3] := o[2] * o[2];
              s := 461.526 * (7.28316418503422 + pi1 * (0.070602197808399 + pi1 * (0.0039229343647356 + pi1 * (0.000313009170788845 + pi1 * (0.0000303619398631619 + pi1 * (7.46739440045781e-6 + o[1] * pi1 * (3.40562176858676e-8 + o[2] * o[3] * pi1 * (4.21886233340801e-17 + o[1] * ((-9.44504571473549e-19) + o[1] * o[2] * ((-2.06859611434475e-21) + pi1 * (9.60758422254987e-22 + ((-1.49967810652241e-22) + 7.86863124555783e-24 * pi1) * pi1)))))))))));
            end supperofp1;

            function hlowerofp2 "Explicit lower specific enthalpy limit of region 2 as function of pressure (meets region 4 saturation pressure curve at 623.15 K)"
              extends Modelica.Icons.Function;
              input SI.Pressure p "Pressure";
              output SI.SpecificEnthalpy h "Specific enthalpy";
            protected
              Real pi "Dimensionless pressure";
              Real q1 "Auxiliary variable";
              Real q2 "Auxiliary variable";
              Real[18] o "Vector of auxiliary variables";
            algorithm
              pi := p / data.PSTAR2;
              assert(p > triple.ptriple, "IF97 medium function hlowerofp2 called with too low pressure\n" + "p = " + String(p) + " Pa <= " + String(triple.ptriple) + " Pa (triple point pressure)");
              q1 := 572.54459862746 + 31.3220101646784 * ((-13.91883977887) + pi) ^ 0.5;
              q2 := (-0.5) + 540. / q1;
              o[1] := q1 * q1;
              o[2] := o[1] * o[1];
              o[3] := o[2] * o[2];
              o[4] := pi * pi;
              o[5] := o[4] * o[4];
              o[6] := q2 * q2;
              o[7] := o[6] * o[6];
              o[8] := o[6] * o[7];
              o[9] := o[5] * o[5];
              o[10] := o[7] * o[7];
              o[11] := o[9] * o[9];
              o[12] := o[10] * o[10];
              o[13] := o[12] * o[12];
              o[14] := o[7] * q2;
              o[15] := o[6] * q2;
              o[16] := o[10] * o[6];
              o[17] := o[13] * o[6];
              o[18] := o[13] * o[6] * q2;
              h := (4.63697573303507e9 + 3.74686560065793 * o[2] + 3.57966647812489e-6 * o[1] * o[2] + 2.81881548488163e-13 * o[3] - 7.64652332452145e7 * q1 - 0.00450789338787835 * o[2] * q1 - 1.55131504410292e-9 * o[1] * o[2] * q1 + o[1] * (2.51383707870341e6 - 4.78198198764471e6 * o[10] * o[11] * o[12] * o[13] * o[4] + 49.9651389369988 * o[11] * o[12] * o[13] * o[4] * o[5] * o[7] + o[15] * o[4] * (1.03746636552761e-13 - 0.00349547959376899 * o[16] - 2.55074501962569e-7 * o[8]) * o[9] + ((-242662.235426958 * o[10] * o[12]) - 3.46022402653609 * o[16]) * o[4] * o[5] * pi + o[4] * (0.109336249381227 - 2248.08924686956 * o[14] - 354742.725841972 * o[17] - 24.1331193696374 * o[6]) * pi - 3.09081828396912e-19 * o[11] * o[12] * o[5] * o[7] * pi - 1.24107527851371e-8 * o[11] * o[13] * o[4] * o[5] * o[6] * o[7] * pi + 3.99891272904219 * o[5] * o[8] * pi + 0.0641817365250892 * o[10] * o[7] * o[9] * pi + pi * ((-4444.87643334512) - 75253.6156722047 * o[14] - 43051.9020511789 * o[6] - 22926.6247146068 * q2) + o[4] * ((-8.23252840892034) - 3927.0508365636 * o[15] - 239.325789467604 * o[18] - 76407.3727417716 * o[8] - 94.4508644545118 * q2) + 0.360567666582363 * o[5] * ((-0.0161221195808321) + q2) * (0.0338039844460968 + q2) + o[11] * ((-0.000584580992538624 * o[10] * o[12] * o[7]) + 1.33248030241755e6 * o[12] * o[13] * q2) + o[9] * ((-7.38502736990986e7 * o[18]) + 0.0000224425477627799 * o[6] * o[7] * q2) + o[4] * o[5] * ((-2.08438767026518e8 * o[17]) - 0.0000124971648677697 * o[6] - 8442.30378348203 * o[10] * o[6] * o[7] * q2) + o[11] * o[9] * (4.73594929247646e-22 * o[10] * o[12] * q2 - 13.6411358215175 * o[10] * o[12] * o[13] * q2 + 5.52427169406836e-10 * o[13] * o[6] * o[7] * q2) + o[11] * o[5] * (2.67174673301715e-6 * o[17] + 4.44545133805865e-18 * o[12] * o[6] * q2 - 50.2465185106411 * o[10] * o[13] * o[6] * o[7] * q2))) / o[1];
            end hlowerofp2;

            function hupperofp2 "Explicit upper specific enthalpy limit of region 2 as function of pressure"
              extends Modelica.Icons.Function;
              input SI.Pressure p "Pressure";
              output SI.SpecificEnthalpy h "Specific enthalpy";
            protected
              Real pi "Dimensionless pressure";
              Real[2] o "Vector of auxiliary variables";
            algorithm
              pi := p / data.PSTAR2;
              assert(p > triple.ptriple, "IF97 medium function hupperofp2 called with too low pressure\n" + "p = " + String(p) + " Pa <= " + String(triple.ptriple) + " Pa (triple point pressure)");
              o[1] := pi * pi;
              o[2] := o[1] * o[1] * o[1];
              h := 4.16066337647071e6 + pi * ((-4518.48617188327) + pi * ((-8.53409968320258) + pi * (0.109090430596056 + pi * ((-0.000172486052272327) + pi * (4.2261295097284e-15 + pi * ((-1.27295130636232e-10) + pi * ((-3.79407294691742e-25) + pi * (7.56960433802525e-23 + pi * (7.16825117265975e-32 + pi * (3.37267475986401e-21 + ((-7.5656940729795e-74) + o[1] * ((-8.00969737237617e-134) + (1.6746290980312e-65 + pi * ((-3.71600586812966e-69) + pi * (8.06630589170884e-129 + ((-1.76117969553159e-103) + 1.88543121025106e-84 * pi) * pi))) * o[1])) * o[2]))))))))));
            end hupperofp2;

            function slowerofp2 "Explicit lower specific entropy limit of region 2 as function of pressure (meets region 4 saturation pressure curve at 623.15 K)"
              extends Modelica.Icons.Function;
              input SI.Pressure p "Pressure";
              output SI.SpecificEntropy s "Specific entropy";
            protected
              Real pi "Dimensionless pressure";
              Real q1 "Auxiliary variable";
              Real q2 "Auxiliary variable";
              Real[40] o "Vector of auxiliary variables";
            algorithm
              pi := p / data.PSTAR2;
              assert(p > triple.ptriple, "IF97 medium function slowerofp2 called with too low pressure\n" + "p = " + String(p) + " Pa <= " + String(triple.ptriple) + " Pa (triple point pressure)");
              q1 := 572.54459862746 + 31.3220101646784 * ((-13.91883977887) + pi) ^ 0.5;
              q2 := (-0.5) + 540.0 / q1;
              o[1] := pi * pi;
              o[2] := o[1] * pi;
              o[3] := o[1] * o[1];
              o[4] := o[1] * o[3] * pi;
              o[5] := q1 * q1;
              o[6] := o[5] * q1;
              o[7] := 1 / o[5];
              o[8] := 1 / q1;
              o[9] := o[5] * o[5];
              o[10] := o[9] * q1;
              o[11] := q2 * q2;
              o[12] := o[11] * q2;
              o[13] := o[1] * o[3];
              o[14] := o[11] * o[11];
              o[15] := o[3] * o[3];
              o[16] := o[1] * o[15];
              o[17] := o[11] * o[14];
              o[18] := o[11] * o[14] * q2;
              o[19] := o[3] * pi;
              o[20] := o[14] * o[14];
              o[21] := o[11] * o[20];
              o[22] := o[15] * pi;
              o[23] := o[14] * o[20] * q2;
              o[24] := o[20] * o[20];
              o[25] := o[15] * o[15];
              o[26] := o[25] * o[3];
              o[27] := o[14] * o[24];
              o[28] := o[25] * o[3] * pi;
              o[29] := o[20] * o[24] * q2;
              o[30] := o[15] * o[25];
              o[31] := o[24] * o[24];
              o[32] := o[11] * o[31] * q2;
              o[33] := o[14] * o[31];
              o[34] := o[1] * o[25] * o[3] * pi;
              o[35] := o[11] * o[14] * o[31] * q2;
              o[36] := o[1] * o[25] * o[3];
              o[37] := o[1] * o[25];
              o[38] := o[20] * o[24] * o[31] * q2;
              o[39] := o[14] * q2;
              o[40] := o[11] * o[31];
              s := 461.526 * (9.692768600217 + 1.22151969114703e-16 * o[10] + 0.00018948987516315 * o[1] * o[11] + 1.6714766451061e-11 * o[12] * o[13] + 0.0039392777243355 * o[1] * o[14] - 1.0406965210174e-19 * o[14] * o[16] + 0.043797295650573 * o[1] * o[18] - 2.2922076337661e-6 * o[18] * o[19] - 2.0481737692309e-8 * o[2] + 0.00003227767723857 * o[12] * o[2] + 0.0015033924542148 * o[17] * o[2] - 1.1256211360459e-11 * o[15] * o[20] + 1.0018179379511e-9 * o[11] * o[14] * o[16] * o[20] + 1.0234747095929e-13 * o[16] * o[21] - 1.9809712802088e-8 * o[22] * o[23] + 0.0021171472321355 * o[13] * o[24] - 8.9185845355421e-25 * o[26] * o[27] - 1.2790717852285e-8 * o[11] * o[3] - 4.8225372718507e-7 * o[12] * o[3] - 7.3087610595061e-29 * o[11] * o[20] * o[24] * o[30] - 0.10693031879409 * o[11] * o[24] * o[25] * o[31] + 4.2002467698208e-6 * o[24] * o[26] * o[31] - 5.5414715350778e-17 * o[20] * o[30] * o[31] + 9.436970724121e-7 * o[11] * o[20] * o[24] * o[30] * o[31] + 23.895741934104 * o[13] * o[32] + 0.040668253562649 * o[2] * o[32] - 3.0629316876232e-13 * o[26] * o[32] + 0.000026674547914087 * o[1] * o[33] + 8.2311340897998 * o[15] * o[33] + 1.2768608934681e-15 * o[34] * o[35] + 0.33662250574171 * o[37] * o[38] + 5.905956432427e-18 * o[4] + 0.038946842435739 * o[29] * o[4] - 4.88368302964335e-6 * o[5] - 3.34901734177133e6 / o[6] + 2.58538448402683e-9 * o[6] + 82839.5726841115 * o[7] - 5446.7940672972 * o[8] - 8.40318337484194e-13 * o[9] + 0.0017731742473213 * pi + 0.045996013696365 * o[11] * pi + 0.057581259083432 * o[12] * pi + 0.05032527872793 * o[17] * pi + o[8] * pi * (9.63082563787332 - 0.008917431146179 * q1) + 0.00811842799898148 * q1 + 0.000033032641670203 * o[1] * q2 - 4.3870667284435e-7 * o[2] * q2 + 8.0882908646985e-11 * o[14] * o[20] * o[24] * o[25] * q2 + 5.9056029685639e-26 * o[14] * o[24] * o[28] * q2 + 7.8847309559367e-10 * o[3] * q2 - 3.7826947613457e-6 * o[14] * o[24] * o[31] * o[36] * q2 + 1.2621808899101e-6 * o[11] * o[20] * o[4] * q2 + 540. * o[8] * (10.08665568018 - 0.000033032641670203 * o[1] - 6.2245802776607e-15 * o[10] - 0.015757110897342 * o[1] * o[12] - 5.0144299353183e-11 * o[11] * o[13] + 4.1627860840696e-19 * o[12] * o[16] - 0.306581069554011 * o[1] * o[17] + 9.0049690883672e-11 * o[15] * o[18] + 0.0000160454534363627 * o[17] * o[19] + 4.3870667284435e-7 * o[2] - 0.00009683303171571 * o[11] * o[2] + 2.57526266427144e-7 * o[14] * o[20] * o[22] - 1.40254511313154e-8 * o[16] * o[23] - 2.34560435076256e-9 * o[14] * o[20] * o[24] * o[25] - 1.24017662339842e-24 * o[27] * o[28] - 7.8847309559367e-10 * o[3] + 1.44676118155521e-6 * o[11] * o[3] + 1.90027787547159e-27 * o[29] * o[30] - 0.000960283724907132 * o[1] * o[32] - 296.320827232793 * o[15] * o[32] - 4.97975748452559e-14 * o[11] * o[14] * o[31] * o[34] + 2.21658861403112e-15 * o[30] * o[35] + 0.000200482822351322 * o[14] * o[24] * o[31] * o[36] - 19.1874828272775 * o[20] * o[24] * o[31] * o[37] - 0.0000547344301999018 * o[30] * o[38] - 0.0090203547252888 * o[2] * o[39] - 0.0000138839897890111 * o[21] * o[4] - 0.973671060893475 * o[20] * o[24] * o[4] - 836.35096769364 * o[13] * o[40] - 1.42338887469272 * o[2] * o[40] + 1.07202609066812e-11 * o[26] * o[40] + 0.0000150341259240398 * o[5] - 1.8087714924605e-8 * o[6] + 18605.6518987296 * o[7] - 306.813232163376 * o[8] + 1.43632471334824e-11 * o[9] + 1.13103675106207e-18 * o[5] * o[9] - 0.017834862292358 * pi - 0.172743777250296 * o[11] * pi - 0.30195167236758 * o[39] * pi + o[8] * pi * ((-49.6756947920742) + 0.045996013696365 * q1) - 0.0003789797503263 * o[1] * q2 - 0.033874355714168 * o[11] * o[13] * o[14] * o[20] * q2 - 1.0234747095929e-12 * o[16] * o[20] * q2 + 1.78371690710842e-23 * o[11] * o[24] * o[26] * q2 + 2.558143570457e-8 * o[3] * q2 + 5.3465159397045 * o[24] * o[25] * o[31] * q2 - 0.000201611844951398 * o[11] * o[14] * o[20] * o[26] * o[31] * q2) - Modelica.Math.log(pi));
            end slowerofp2;

            function supperofp2 "Explicit upper specific entropy limit of region 2 as function of pressure"
              extends Modelica.Icons.Function;
              input SI.Pressure p "Pressure";
              output SI.SpecificEntropy s "Specific entropy";
            protected
              Real pi "Dimensionless pressure";
              Real[2] o "Vector of auxiliary variables";
            algorithm
              pi := p / data.PSTAR2;
              assert(p > triple.ptriple, "IF97 medium function supperofp2 called with too low pressure\n" + "p = " + String(p) + " Pa <= " + String(triple.ptriple) + " Pa (triple point pressure)");
              o[1] := pi * pi;
              o[2] := o[1] * o[1] * o[1];
              s := 8505.73409708683 - 461.526 * Modelica.Math.log(pi) + pi * ((-3.36563543302584) + pi * ((-0.00790283552165338) + pi * (0.0000915558349202221 + pi * ((-1.59634706513e-7) + pi * (3.93449217595397e-18 + pi * ((-1.18367426347994e-13) + pi * (2.72575244843195e-15 + pi * (7.04803892603536e-26 + pi * (6.67637687381772e-35 + pi * (3.1377970315132e-24 + ((-7.04844558482265e-77) + o[1] * ((-7.46289531275314e-137) + (1.55998511254305e-68 + pi * ((-3.46166288915497e-72) + pi * (7.51557618628583e-132 + ((-1.64086406733212e-106) + 1.75648443097063e-87 * pi) * pi))) * o[1])) * o[2] * o[2]))))))))));
            end supperofp2;

            function d1n "Density in region 1 as function of p and T"
              extends Modelica.Icons.Function;
              input SI.Pressure p "Pressure";
              input SI.Temperature T "Temperature (K)";
              output SI.Density d "Density";
            protected
              Real pi "Dimensionless pressure";
              Real pi1 "Dimensionless pressure";
              Real tau "Dimensionless temperature";
              Real tau1 "Dimensionless temperature";
              Real gpi "Dimensionless Gibbs-derivative w.r.t. pi";
              Real[11] o "Auxiliary variables";
            algorithm
              pi := p / data.PSTAR1;
              tau := data.TSTAR1 / T;
              pi1 := 7.1 - pi;
              tau1 := tau - 1.222;
              o[1] := tau1 * tau1;
              o[2] := o[1] * o[1];
              o[3] := o[2] * o[2];
              o[4] := o[1] * o[2];
              o[5] := o[1] * tau1;
              o[6] := o[2] * tau1;
              o[7] := pi1 * pi1;
              o[8] := o[7] * o[7];
              o[9] := o[8] * o[8];
              o[10] := o[3] * o[3];
              o[11] := o[10] * o[10];
              gpi := pi1 * (pi1 * ((0.000095038934535162 + o[2] * (8.4812393955936e-6 + 2.55615384360309e-9 * o[4])) / o[2] + pi1 * ((8.9701127632e-6 + (2.60684891582404e-6 + 5.7366919751696e-13 * o[2] * o[3]) * o[5]) / o[6] + pi1 * (2.02584984300585e-6 / o[3] + o[7] * pi1 * (o[8] * o[9] * pi1 * (o[7] * (o[7] * o[8] * ((-7.63737668221055e-22 / (o[1] * o[11] * o[2])) + pi1 * (pi1 * ((-5.65070932023524e-23 / (o[11] * o[3])) + 2.99318679335866e-24 * pi1 / (o[11] * o[3] * tau1)) + 3.5842867920213e-22 / (o[1] * o[11] * o[2] * tau1))) - 3.33001080055983e-19 / (o[1] * o[10] * o[2] * o[3] * tau1)) + 1.44400475720615e-17 / (o[10] * o[2] * o[3] * tau1)) + (1.01874413933128e-8 + 1.39398969845072e-9 * o[6]) / (o[1] * o[3] * tau1))))) + (0.00094368642146534 + o[5] * (0.00060003561586052 + ((-0.000095322787813974) + o[1] * (8.8283690661692e-6 + 1.45389992595188e-15 * o[1] * o[2] * o[3])) * tau1)) / o[5]) + ((-0.00028319080123804) + o[1] * (0.00060706301565874 + o[4] * (0.018990068218419 + tau1 * (0.032529748770505 + (0.021841717175414 + 0.00005283835796993 * o[1]) * tau1)))) / (o[3] * tau1);
              d := p / (data.RH2O * T * pi * gpi);
            end d1n;

            function d2n "Density in region 2 as function of p and T"
              extends Modelica.Icons.Function;
              input SI.Pressure p "Pressure";
              input SI.Temperature T "Temperature (K)";
              output SI.Density d "Density";
            protected
              Real pi "Dimensionless pressure";
              Real tau "Dimensionless temperature";
              Real tau2 "Dimensionless temperature";
              Real gpi "Dimensionless Gibbs-derivative w.r.t. pi";
              Real[12] o "Auxiliary variables";
            algorithm
              pi := p / data.PSTAR2;
              tau := data.TSTAR2 / T;
              tau2 := tau - 0.5;
              o[1] := tau2 * tau2;
              o[2] := o[1] * tau2;
              o[3] := o[1] * o[1];
              o[4] := o[3] * o[3];
              o[5] := o[4] * o[4];
              o[6] := o[3] * o[4] * o[5] * tau2;
              o[7] := o[3] * o[4] * tau2;
              o[8] := o[1] * o[3] * o[4];
              o[9] := pi * pi;
              o[10] := o[9] * o[9];
              o[11] := o[3] * o[5] * tau2;
              o[12] := o[5] * o[5];
              gpi := (1. + pi * ((-0.0017731742473213) + tau2 * ((-0.017834862292358) + tau2 * ((-0.045996013696365) + ((-0.057581259083432) - 0.05032527872793 * o[2]) * tau2)) + pi * (tau2 * ((-0.000066065283340406) + ((-0.0003789797503263) + o[1] * ((-0.007878555448671) + o[2] * ((-0.087594591301146) - 0.000053349095828174 * o[6]))) * tau2) + pi * (6.1445213076927e-8 + (1.31612001853305e-6 + o[1] * ((-0.00009683303171571) + o[2] * ((-0.0045101773626444) - 0.122004760687947 * o[6]))) * tau2 + pi * (tau2 * ((-3.15389238237468e-9) + (5.116287140914e-8 + 1.92901490874028e-6 * tau2) * tau2) + pi * (0.0000114610381688305 * o[1] * o[3] * tau2 + pi * (o[2] * ((-1.00288598706366e-10) + o[7] * ((-0.012702883392813) - 143.374451604624 * o[1] * o[5] * tau2)) + pi * ((-4.1341695026989e-17) + o[1] * o[4] * ((-8.8352662293707e-6) - 0.272627897050173 * o[8]) * tau2 + pi * (o[4] * (9.0049690883672e-11 - 65.8490727183984 * o[3] * o[4] * o[5]) + pi * (1.78287415218792e-7 * o[7] + pi * (o[3] * (1.0406965210174e-18 + o[1] * ((-1.0234747095929e-12) - 1.0018179379511e-8 * o[3]) * o[3]) + o[10] * o[9] * (((-1.29412653835176e-9) + 1.71088510070544 * o[11]) * o[6] + o[9] * ((-6.05920510335078 * o[12] * o[4] * o[5] * tau2) + o[9] * (o[3] * o[5] * (1.78371690710842e-23 + o[1] * o[3] * o[4] * (6.1258633752464e-12 - 0.000084004935396416 * o[7]) * tau2) + pi * ((-1.24017662339842e-24 * o[11]) + pi * (0.0000832192847496054 * o[12] * o[3] * o[5] * tau2 + pi * (o[1] * o[4] * o[5] * (1.75410265428146e-27 + (1.32995316841867e-15 - 0.0000226487297378904 * o[1] * o[5]) * o[8]) * pi - 2.93678005497663e-14 * o[1] * o[12] * o[3] * tau2))))))))))))))))) / pi;
              d := p / (data.RH2O * T * pi * gpi);
            end d2n;

            function dhot1ofp "Density at upper temperature limit of region 1"
              extends Modelica.Icons.Function;
              input SI.Pressure p "Pressure";
              output SI.Density d "Density";
            protected
              Real pi "Dimensionless pressure";
              Real pi1 "Dimensionless pressure";
              Real[4] o "Auxiliary variables";
            algorithm
              pi := p / data.PSTAR1;
              pi1 := 7.1 - pi;
              o[1] := pi1 * pi1;
              o[2] := o[1] * o[1];
              o[3] := o[2] * o[2];
              o[4] := o[3] * o[3];
              d := 57.4756752485113 / (0.0737412153522555 + 0.000102697173772229 * o[1] + 1.99080616601101e-6 * o[2] + 1.35549330686006e-17 * o[2] * o[4] - 3.11228834832975e-19 * o[1] * o[2] * o[4] - 7.02987180039442e-22 * o[2] * o[3] * o[4] - 5.17859076694812e-23 * o[1] * o[2] * o[3] * o[4] + 0.00145092247736023 * pi1 + 0.0000114683182476084 * o[1] * pi1 + 1.13217858826367e-8 * o[1] * o[2] * pi1 + 3.29199117056433e-22 * o[2] * o[3] * o[4] * pi1 + 2.73712834080283e-24 * o[1] * o[2] * o[3] * o[4] * pi1);
            end dhot1ofp;

            function dupper1ofT "Density at upper pressure limit of region 1"
              extends Modelica.Icons.Function;
              input SI.Temperature T "Temperature (K)";
              output SI.Density d "Density";
            protected
              Real tau "Dimensionless temperature";
              Real[4] o "Auxiliary variables";
            algorithm
              tau := 1386.0 / T;
              o[1] := tau * tau;
              o[2] := o[1] * o[1];
              o[3] := o[2] * o[2];
              o[4] := o[3] * o[3];
              d := 57.4756752485113 / (2.24144616859917 + 40.9288231166229 * o[1] + 106.47246463213 * o[2] + 88.4481480270751 * o[1] * o[2] + 31.3207272417546 * o[3] + 5.47811738891798 * o[1] * o[3] + 0.515626225030717 * o[2] * o[3] + 0.0274905057899089 * o[1] * o[2] * o[3] + 0.000853742979250503 * o[4] + 0.0000155932210492199 * o[1] * o[4] + 1.6621051480279e-7 * o[2] * o[4] + 1.00606771839976e-9 * o[1] * o[2] * o[4] + 3.27598951831994e-12 * o[3] * o[4] + 5.20162317530099e-15 * o[1] * o[3] * o[4] + 3.33501889800275e-18 * o[2] * o[3] * o[4] + 5.50656040141221e-22 * o[1] * o[2] * o[3] * o[4] - 13.5354267762204 * tau - 78.3629702507642 * o[1] * tau - 109.374479648652 * o[2] * tau - 57.9035658513312 * o[1] * o[2] * tau - 14.215347150565 * o[3] * tau - 1.80906759985501 * o[1] * o[3] * tau - 0.127542214693871 * o[2] * o[3] * tau - 0.0051779458313163 * o[1] * o[2] * o[3] * tau - 0.000123304142684848 * o[4] * tau - 1.72405791469972e-6 * o[1] * o[4] * tau - 1.39155695911655e-8 * o[2] * o[4] * tau - 6.23333356847138e-11 * o[1] * o[2] * o[4] * tau - 1.44056015732082e-13 * o[3] * o[4] * tau - 1.50201626932938e-16 * o[1] * o[3] * o[4] * tau - 5.34588682252967e-20 * o[2] * o[3] * o[4] * tau - 2.73712834080283e-24 * o[1] * o[2] * o[3] * o[4] * tau);
            end dupper1ofT;

            function hl_p_R4b "Explicit approximation of liquid specific enthalpy on the boundary between regions 4 and 3"
              extends Modelica.Icons.Function;
              input SI.Pressure p "Pressure";
              output SI.SpecificEnthalpy h "Specific enthalpy";
            protected
              Real x "Auxiliary variable";
            algorithm
// documentation of accuracy in notebook ~hubertus/props/IAPWS/R3Approx.nb
// boundary between region IVa and III
              x := Modelica.Math.acos(p / data.PCRIT);
              h := (1 + x * ((-0.4945586958175176) + x * (1.346800016564904 + x * ((-3.889388153209752) + x * (6.679385472887931 + x * ((-6.75820241066552) + x * (3.558919744656498 + ((-0.7179818554978939) - 0.0001152032945617821 * x) * x))))))) * data.HCRIT;
            end hl_p_R4b;

            function hv_p_R4b "Explicit approximation of vapour specific enthalpy on the boundary between regions 4 and 3"
              extends Modelica.Icons.Function;
              input SI.Pressure p "Pressure";
              output SI.SpecificEnthalpy h "Specific enthalpy";
            protected
              Real x "Auxiliary variable";
            algorithm
// boundary between region IVa and III
              x := Modelica.Math.acos(p / data.PCRIT);
              h := (1 + x * (0.4880153718655694 + x * (0.2079670746250689 + x * ((-6.084122698421623) + x * (25.08887602293532 + x * ((-48.38215180269516) + x * (45.66489164833212 + ((-16.98555442961553) + 0.0006616936460057691 * x) * x))))))) * data.HCRIT;
            end hv_p_R4b;

            function sl_p_R4b "Explicit approximation of liquid specific entropy on the boundary between regions 4 and 3"
              extends Modelica.Icons.Function;
              input SI.Pressure p "Pressure";
              output SI.SpecificEntropy s "Specific entropy";
            protected
              Real x "Auxiliary variable";
            algorithm
// boundary between region IVa and III
              x := Modelica.Math.acos(p / data.PCRIT);
              s := (1 + x * ((-0.36160692245648063) + x * (0.9962778630486647 + x * ((-2.8595548144171103) + x * (4.906301159555333 + x * ((-4.974092309614206) + x * (2.6249651699204457 + ((-0.5319954375299023) - 0.00008064497431880644 * x) * x))))))) * data.SCRIT;
            end sl_p_R4b;

            function sv_p_R4b "Explicit approximation of vapour specific entropy on the boundary between regions 4 and 3"
              extends Modelica.Icons.Function;
              input SI.Pressure p "Pressure";
              output SI.SpecificEntropy s;
            protected
              Real x "Auxiliary variable";
            algorithm
// documentation of accuracy in notebook ~hubertus/props/IAPWS/R3Approx.nb
// boundary between region IVa and III
              x := Modelica.Math.acos(p / data.PCRIT);
              s := (1 + x * (0.35682641826674344 + x * (0.1642457027815487 + x * ((-4.425350377422446) + x * (18.324477859983133 + x * ((-35.338631625948665) + x * (33.36181025816282 + ((-12.408711490585757) + 0.0004810049834109226 * x) * x))))))) * data.SCRIT;
            end sv_p_R4b;

            function rhol_p_R4b "Explicit approximation of liquid density on the boundary between regions 4 and 3"
              extends Modelica.Icons.Function;
              input SI.Pressure p "Pressure";
              output SI.Density dl "Liquid density";
            protected
              Real x "Auxiliary variable";
            algorithm
              if p < data.PCRIT then
                x := Modelica.Math.acos(p / data.PCRIT);
                dl := (1 + x * (1.903224079094824 + x * ((-2.5314861802401123) + x * ((-8.191449323843552) + x * (94.34196116778385 + x * ((-369.3676833623383) + x * (796.6627910598293 + x * ((-994.5385383600702) + x * (673.2581177021598 + ((-191.43077336405156) + 0.00052536560808895 * x) * x))))))))) * data.DCRIT;
              else
                dl := data.DCRIT;
              end if;
            end rhol_p_R4b;

            function rhov_p_R4b "Explicit approximation of vapour density on the boundary between regions 4 and 2"
              extends Modelica.Icons.Function;
              input SI.Pressure p "Pressure";
              output SI.Density dv "Vapour density";
            protected
              Real x "Auxiliary variable";
            algorithm
              if p < data.PCRIT then
                x := Modelica.Math.acos(p / data.PCRIT);
                dv := (1 + x * ((-1.8463850803362596) + x * ((-1.1447872718878493) + x * (59.18702203076563 + x * ((-403.5391431811611) + x * (1437.2007245332388 + x * ((-3015.853540307519) + x * (3740.5790348670057 + x * ((-2537.375817253895) + (725.8761975803782 - 0.0011151111658332337 * x) * x))))))))) * data.DCRIT;
              else
                dv := data.DCRIT;
              end if;
            end rhov_p_R4b;

            function boilingcurve_p "Properties on the boiling curve"
              extends Modelica.Icons.Function;
              input SI.Pressure p "Pressure";
              output Common.IF97PhaseBoundaryProperties bpro "Property record";
            protected
              Common.GibbsDerivs g "Dimensionless Gibbs function and derivatives";
              Common.HelmholtzDerivs f "Dimensionless Helmholtz function and derivatives";
              SI.Pressure plim = min(p, data.PCRIT - 1e-7) "Pressure limited to critical pressure - epsilon";
            algorithm
              bpro.R := data.RH2O;
              bpro.T := Basic.tsat(plim);
              bpro.dpT := Basic.dptofT(bpro.T);
              bpro.region3boundary := bpro.T > data.TLIMIT1;
              if not bpro.region3boundary then
                g := Basic.g1(p, bpro.T);
                bpro.d := p / (bpro.R * bpro.T * g.pi * g.gpi);
                bpro.h := if p > plim then data.HCRIT else bpro.R * bpro.T * g.tau * g.gtau;
                bpro.s := g.R * (g.tau * g.gtau - g.g);
                bpro.cp := -bpro.R * g.tau * g.tau * g.gtautau;
                bpro.vt := bpro.R / p * (g.pi * g.gpi - g.tau * g.pi * g.gtaupi);
                bpro.vp := bpro.R * bpro.T / (p * p) * g.pi * g.pi * g.gpipi;
                bpro.pt := -p / bpro.T * (g.gpi - g.tau * g.gtaupi) / (g.gpipi * g.pi);
                bpro.pd := -bpro.R * bpro.T * g.gpi * g.gpi / g.gpipi;
              else
                bpro.d := rhol_p_R4b(plim);
                f := Basic.f3(bpro.d, bpro.T);
                bpro.h := hl_p_R4b(plim);
                bpro.s := f.R * (f.tau * f.ftau - f.f);
                bpro.cv := bpro.R * (-f.tau * f.tau * f.ftautau);
                bpro.pt := bpro.R * bpro.d * f.delta * (f.fdelta - f.tau * f.fdeltatau);
                bpro.pd := bpro.R * bpro.T * f.delta * (2.0 * f.fdelta + f.delta * f.fdeltadelta);
              end if;
// bpro.R*bpro.T*(f.tau*f.ftau + f.delta*f.fdelta);
            end boilingcurve_p;

            function dewcurve_p "Properties on the dew curve"
              extends Modelica.Icons.Function;
              input SI.Pressure p "Pressure";
              output Common.IF97PhaseBoundaryProperties bpro "Property record";
            protected
              Common.GibbsDerivs g "Dimensionless Gibbs function and derivatives";
              Common.HelmholtzDerivs f "Dimensionless Helmholtz function and derivatives";
              SI.Pressure plim = min(p, data.PCRIT - 1e-7) "Pressure limited to critical pressure - epsilon";
            algorithm
              bpro.R := data.RH2O;
              bpro.T := Basic.tsat(plim);
              bpro.dpT := Basic.dptofT(bpro.T);
              bpro.region3boundary := bpro.T > data.TLIMIT1;
              if not bpro.region3boundary then
                g := Basic.g2(p, bpro.T);
                bpro.d := p / (bpro.R * bpro.T * g.pi * g.gpi);
                bpro.h := if p > plim then data.HCRIT else bpro.R * bpro.T * g.tau * g.gtau;
                bpro.s := g.R * (g.tau * g.gtau - g.g);
                bpro.cp := -bpro.R * g.tau * g.tau * g.gtautau;
                bpro.vt := bpro.R / p * (g.pi * g.gpi - g.tau * g.pi * g.gtaupi);
                bpro.vp := bpro.R * bpro.T / (p * p) * g.pi * g.pi * g.gpipi;
                bpro.pt := -p / bpro.T * (g.gpi - g.tau * g.gtaupi) / (g.gpipi * g.pi);
                bpro.pd := -bpro.R * bpro.T * g.gpi * g.gpi / g.gpipi;
              else
                bpro.d := rhov_p_R4b(plim);
                f := Basic.f3(bpro.d, bpro.T);
                bpro.h := hv_p_R4b(plim);
                bpro.s := f.R * (f.tau * f.ftau - f.f);
                bpro.cv := bpro.R * (-f.tau * f.tau * f.ftautau);
                bpro.pt := bpro.R * bpro.d * f.delta * (f.fdelta - f.tau * f.fdeltatau);
                bpro.pd := bpro.R * bpro.T * f.delta * (2.0 * f.fdelta + f.delta * f.fdeltadelta);
              end if;
// bpro.R*bpro.T*(f.tau*f.ftau + f.delta*f.fdelta);
            end dewcurve_p;

            function hvl_p
              extends Modelica.Icons.Function;
              input SI.Pressure p "Pressure";
              input Common.IF97PhaseBoundaryProperties bpro "Property record";
              output SI.SpecificEnthalpy h "Specific enthalpy";
            algorithm
              h := bpro.h;
              annotation(
                derivative(noDerivative = bpro) = hvl_p_der,
                Inline = false,
                LateInline = true);
            end hvl_p;

            function hl_p "Liquid specific enthalpy on the boundary between regions 4 and 3 or 1"
              extends Modelica.Icons.Function;
              input SI.Pressure p "Pressure";
              output SI.SpecificEnthalpy h "Specific enthalpy";
            algorithm
              h := hvl_p(p, boilingcurve_p(p));
              annotation(
                Inline = true);
            end hl_p;

            function hv_p "Vapour specific enthalpy on the boundary between regions 4 and 3 or 2"
              extends Modelica.Icons.Function;
              input SI.Pressure p "Pressure";
              output SI.SpecificEnthalpy h "Specific enthalpy";
            algorithm
              h := hvl_p(p, dewcurve_p(p));
              annotation(
                Inline = true);
            end hv_p;

            function hvl_p_der "Derivative function for the specific enthalpy along the phase boundary"
              extends Modelica.Icons.Function;
              input SI.Pressure p "Pressure";
              input Common.IF97PhaseBoundaryProperties bpro "Property record";
              input Real p_der "Derivative of pressure";
              output Real h_der "Time derivative of specific enthalpy along the phase boundary";
            algorithm
              if bpro.region3boundary then
                h_der := ((bpro.d * bpro.pd - bpro.T * bpro.pt) * p_der + (bpro.T * bpro.pt * bpro.pt + bpro.d * bpro.d * bpro.pd * bpro.cv) / bpro.dpT * p_der) / (bpro.pd * bpro.d * bpro.d);
              else
                h_der := (1 / bpro.d - bpro.T * bpro.vt) * p_der + bpro.cp / bpro.dpT * p_der;
              end if;
              annotation(
                Inline = true);
            end hvl_p_der;

            function rhovl_p
              extends Modelica.Icons.Function;
              input SI.Pressure p "Pressure";
              input Common.IF97PhaseBoundaryProperties bpro "Property record";
              output SI.Density rho "Density";
            algorithm
              rho := bpro.d;
              annotation(
                derivative(noDerivative = bpro) = rhovl_p_der,
                Inline = false,
                LateInline = true);
            end rhovl_p;

            function rhol_p "Density of saturated water"
              extends Modelica.Icons.Function;
              input SI.Pressure p "Saturation pressure";
              output SI.Density rho "Density of steam at the condensation point";
            algorithm
              rho := rhovl_p(p, boilingcurve_p(p));
              annotation(
                Inline = true);
            end rhol_p;

            function rhov_p "Density of saturated vapour"
              extends Modelica.Icons.Function;
              input SI.Pressure p "Saturation pressure";
              output SI.Density rho "Density of steam at the condensation point";
            algorithm
              rho := rhovl_p(p, dewcurve_p(p));
              annotation(
                Inline = true);
            end rhov_p;

            function rhovl_p_der
              extends Modelica.Icons.Function;
              input SI.Pressure p "Saturation pressure";
              input Common.IF97PhaseBoundaryProperties bpro "Property record";
              input Real p_der "Derivative of pressure";
              output Real d_der "Time derivative of density along the phase boundary";
            algorithm
              d_der := if bpro.region3boundary then (p_der - bpro.pt * p_der / bpro.dpT) / bpro.pd else -bpro.d * bpro.d * (bpro.vp + bpro.vt / bpro.dpT) * p_der;
              annotation(
                Inline = true);
            end rhovl_p_der;

            function sl_p "Liquid specific entropy on the boundary between regions 4 and 3 or 1"
              extends Modelica.Icons.Function;
              input SI.Pressure p "Pressure";
              output SI.SpecificEntropy s "Specific entropy";
            protected
              SI.Temperature Tsat "Saturation temperature";
              SI.SpecificEnthalpy h "Specific enthalpy";
            algorithm
              if p < data.PLIMIT4A then
                Tsat := Basic.tsat(p);
                (h, s) := Isentropic.handsofpT1(p, Tsat);
              elseif p < data.PCRIT then
                s := sl_p_R4b(p);
              else
                s := data.SCRIT;
              end if;
            end sl_p;

            function sv_p "Vapour specific entropy on the boundary between regions 4 and 3 or 2"
              extends Modelica.Icons.Function;
              input SI.Pressure p "Pressure";
              output SI.SpecificEntropy s "Specific entropy";
            protected
              SI.Temperature Tsat "Saturation temperature";
              SI.SpecificEnthalpy h "Specific enthalpy";
            algorithm
              if p < data.PLIMIT4A then
                Tsat := Basic.tsat(p);
                (h, s) := Isentropic.handsofpT2(p, Tsat);
              elseif p < data.PCRIT then
                s := sv_p_R4b(p);
              else
                s := data.SCRIT;
              end if;
            end sv_p;

            function rhol_T "Density of saturated water"
              extends Modelica.Icons.Function;
              input SI.Temperature T "Temperature";
              output SI.Density d "Density of water at the boiling point";
            protected
              SI.Pressure p "Saturation pressure";
            algorithm
              p := Basic.psat(T);
              if T < data.TLIMIT1 then
                d := d1n(p, T);
              elseif T < data.TCRIT then
                d := rhol_p_R4b(p);
              else
                d := data.DCRIT;
              end if;
            end rhol_T;

            function rhov_T "Density of saturated vapour"
              extends Modelica.Icons.Function;
              input SI.Temperature T "Temperature";
              output SI.Density d "Density of steam at the condensation point";
            protected
              SI.Pressure p "Saturation pressure";
            algorithm
// assert(T <= data.TCRIT,"input temperature has to be below the critical temperature");
              p := Basic.psat(T);
              if T < data.TLIMIT1 then
                d := d2n(p, T);
              elseif T < data.TCRIT then
                d := rhov_p_R4b(p);
              else
                d := data.DCRIT;
              end if;
            end rhov_T;

            function region_ph "Return the current region (valid values: 1,2,3,4,5) in IF97 for given pressure and specific enthalpy"
              extends Modelica.Icons.Function;
              input SI.Pressure p "Pressure";
              input SI.SpecificEnthalpy h "Specific enthalpy";
              input Integer phase = 0 "Phase: 2 for two-phase, 1 for one phase, 0 if not known";
              input Integer mode = 0 "Mode: 0 means check, otherwise assume region=mode";
              output Integer region "Region (valid values: 1,2,3,4,5) in IF97";
              // If mode is different from 0, no checking for the region is done and
              // the mode is assumed to be the correct region. This can be used to
              // implement e.g., water-only steam tables when mode == 1
            protected
              Boolean hsubcrit;
              SI.Temperature Ttest;
              constant Real[5] n = data.n;
              SI.SpecificEnthalpy hl "Bubble enthalpy";
              SI.SpecificEnthalpy hv "Dew enthalpy";
            algorithm
              if mode <> 0 then
                region := mode;
              else
                hl := hl_p(p);
                hv := hv_p(p);
                if phase == 2 then
                  region := 4;
                else
                  if p < triple.ptriple or p > data.PLIMIT1 or h < hlowerofp1(p) or p < 10.0e6 and h > hupperofp5(p) or p >= 10.0e6 and h > hupperofp2(p) then
                    region := -1;
                  else
                    hsubcrit := h < data.HCRIT;
                    if p < data.PLIMIT4A then
                      if hsubcrit then
                        if phase == 1 then
                          region := 1;
                        else
                          if h < Isentropic.hofpT1(p, Basic.tsat(p)) then
                            region := 1;
                          else
                            region := 4;
                          end if;
                        end if;
                      else
                        if h > hlowerofp5(p) then
                          if p < data.PLIMIT5 and h < hupperofp5(p) then
                            region := 5;
                          else
                            region := -2;
                          end if;
                        else
                          if phase == 1 then
                            region := 2;
                          else
                            if h > Isentropic.hofpT2(p, Basic.tsat(p)) then
                              region := 2;
                            else
                              region := 4;
                            end if;
                          end if;
                        end if;
                      end if;
                    else
                      if hsubcrit then
                        if h < hupperofp1(p) then
                          region := 1;
                        else
                          if h < hl or p > data.PCRIT then
                            region := 3;
                          else
                            region := 4;
                          end if;
                        end if;
                      else
                        if h > hlowerofp2(p) then
                          region := 2;
                        else
                          if h > hv or p > data.PCRIT then
                            region := 3;
                          else
                            region := 4;
                          end if;
                        end if;
                      end if;
                    end if;
                  end if;
                end if;
              end if;
// check for regions 1, 2, 3 and 4
// phase == 1 or 0, now check if we are in the legal area
// outside of valid range
//region 5 and -1 check complete
// simple precheck: very simple if pressure < PLIMIT4A
// we can never be in region 3, so test for others
// external or internal phase check
// check for region 5
// pressure and specific enthalpy too high, but this should
// never happen
// external or internal phase check
// tests for region 2 or 5
// tests for sub or supercritical
// the pressure is over data.PLIMIT4A
// region 1 or 3 or 4
// end of test for region 1, 3 or 4
// region 2, 3 or 4
// test for 2 and 3
// tests above PLIMIT4A
// above or below PLIMIT4A
// check for grand limits of p and h
// all tests with phase == 1
// mode was == 0
// assert(region > 0,"IF97 function called outside the valid range!");
            end region_ph;

            function region_ps "Return the current region (valid values: 1,2,3,4,5) in IF97 for given pressure and specific entropy"
              extends Modelica.Icons.Function;
              input SI.Pressure p "Pressure";
              input SI.SpecificEntropy s "Specific entropy";
              input Integer phase = 0 "Phase: 2 for two-phase, 1 for one phase, 0 if unknown";
              input Integer mode = 0 "Mode: 0 means check, otherwise assume region=mode";
              output Integer region "Region (valid values: 1,2,3,4,5) in IF97";
              //  If mode is different from 0, no checking for the region is done and
              //    the mode is assumed to be the correct region. This can be used to
              //    implement e.g., water-only steam tables when mode == 1
            protected
              Boolean ssubcrit;
              SI.Temperature Ttest;
              constant Real[5] n = data.n;
              SI.SpecificEntropy sl "Bubble entropy";
              SI.SpecificEntropy sv "Dew entropy";
            algorithm
              if mode <> 0 then
                region := mode;
              else
                sl := sl_p(p);
                sv := sv_p(p);
                if phase == 2 or phase == 0 and s > sl and s < sv and p < data.PCRIT then
                  region := 4;
                else
                  region := 0;
                  if p < triple.ptriple then
                    region := -2;
                  end if;
                  if p > data.PLIMIT1 then
                    region := -3;
                  end if;
                  if p < 10.0e6 and s > supperofp5(p) then
                    region := -5;
                  end if;
                  if p >= 10.0e6 and s > supperofp2(p) then
                    region := -6;
                  end if;
                  if region < 0 then
                    assert(false, "Region computation from p and s failed: function called outside the legal region");
                  else
                    ssubcrit := s < data.SCRIT;
                    if p < data.PLIMIT4A then
                      if ssubcrit then
                        region := 1;
                      else
                        if s > slowerofp5(p) then
                          if p < data.PLIMIT5 and s < supperofp5(p) then
                            region := 5;
                          else
                            region := -1;
                          end if;
                        else
                          region := 2;
                        end if;
                      end if;
                    else
                      if ssubcrit then
                        if s < supperofp1(p) then
                          region := 1;
                        else
                          if s < sl or p > data.PCRIT then
                            region := 3;
                          else
                            region := 4;
                          end if;
                        end if;
                      else
                        if s > slowerofp2(p) then
                          region := 2;
                        else
                          if s > sv or p > data.PCRIT then
                            region := 3;
                          else
                            region := 4;
                          end if;
                        end if;
                      end if;
                    end if;
                  end if;
                end if;
              end if;
// check for regions 1, 2, 3, and 4
// check all cases two-phase
// phase == 1
// simple precheck: very simple if pressure < PLIMIT4A
// we can never be in region 3, so test for 1 and 2
// check for region 5
// pressure and specific entropy too high, should never happen!
// tests for region 2 or 5
// tests for sub or supercritical
// the pressure is over data.PLIMIT4A
// region 1 or 3
// test for region 1, 3 or 4
// region 2, 3 or 4
// test for 2,3 and 4
// tests above PLIMIT4A
// above or below PLIMIT4A
// grand test for limits of p and s
// all tests with phase == 1
// mode was == 0
            end region_ps;

            function region_pT "Return the current region (valid values: 1,2,3,5) in IF97, given pressure and temperature"
              extends Modelica.Icons.Function;
              input SI.Pressure p "Pressure";
              input SI.Temperature T "Temperature (K)";
              input Integer mode = 0 "Mode: 0 means check, otherwise assume region=mode";
              output Integer region "Region (valid values: 1,2,3,5) in IF97, region 4 is impossible!";
            algorithm
              if mode <> 0 then
                region := mode;
              else
                if p < data.PLIMIT4A then
                  if T > data.TLIMIT2 then
                    region := 5;
                  elseif T > Basic.tsat(p) then
                    region := 2;
                  else
                    region := 1;
                  end if;
                else
                  if T < data.TLIMIT1 then
                    region := 1;
                  elseif T < boundary23ofp(p) then
                    region := 3;
                  else
                    region := 2;
                  end if;
                end if;
              end if;
//test for regions 1,2,5
//test for regions 1,2,3
// mode was == 0
            end region_pT;

            function region_dT "Return the current region (valid values: 1,2,3,4,5) in IF97, given density and temperature"
              extends Modelica.Icons.Function;
              input SI.Density d "Density";
              input SI.Temperature T "Temperature (K)";
              input Integer phase = 0 "Phase: 2 for two-phase, 1 for one phase, 0 if not known";
              input Integer mode = 0 "Mode: 0 means check, otherwise assume region=mode";
              output Integer region "(valid values: 1,2,3,4,5) in IF97";
            protected
              Boolean Tovercrit "Flag if overcritical temperature";
              SI.Pressure p23 "Pressure needed to know if region 2 or 3";
            algorithm
              Tovercrit := T > data.TCRIT;
              if mode <> 0 then
                region := mode;
              else
                p23 := boundary23ofT(T);
                if T > data.TLIMIT2 then
                  if d < 20.5655874106483 then
                    region := 5;
                  else
                    assert(false, "Out of valid region for IF97, pressure above region 5!");
                  end if;
                elseif Tovercrit then
                  if d > d2n(p23, T) and T > data.TLIMIT1 then
                    region := 3;
                  elseif T < data.TLIMIT1 then
                    region := 1;
                  else
                    region := 2;
                  end if;
                elseif d > rhol_T(T) then
                  if T < data.TLIMIT1 then
                    region := 1;
                  else
                    region := 3;
                  end if;
                elseif d < rhov_T(T) then
                  if d > d2n(p23, T) and T > data.TLIMIT1 then
                    region := 3;
                  else
                    region := 2;
                  end if;
                else
                  region := 4;
                end if;
              end if;
// check for the density in the upper corner of validity!
//check for regions 1, 2 or 3
// d  < d2n(p23, T) and T > data.TLIMIT1
// below critical, check for regions 1, 2, 3 or 4
// either 1 or 3
// not liquid, not 2-phase, and not region 5, so either 2 or 3 or illegal
            end region_dT;

            function hvl_dp "Derivative function for the specific enthalpy along the phase boundary"
              extends Modelica.Icons.Function;
              input SI.Pressure p "Pressure";
              input Common.IF97PhaseBoundaryProperties bpro "Property record";
              output Real dh_dp "Derivative of specific enthalpy along the phase boundary";
            algorithm
              if bpro.region3boundary then
                dh_dp := (bpro.d * bpro.pd - bpro.T * bpro.pt + (bpro.T * bpro.pt * bpro.pt + bpro.d * bpro.d * bpro.pd * bpro.cv) / bpro.dpT) / (bpro.pd * bpro.d * bpro.d);
              else
                dh_dp := 1 / bpro.d - bpro.T * bpro.vt + bpro.cp / bpro.dpT;
              end if;
            end hvl_dp;

            function dhl_dp "Derivative of liquid specific enthalpy on the boundary between regions 4 and 3 or 1 w.r.t. pressure"
              extends Modelica.Icons.Function;
              input SI.Pressure p "Pressure";
              output SI.DerEnthalpyByPressure dh_dp "Specific enthalpy derivative w.r.t. pressure";
            algorithm
              dh_dp := hvl_dp(p, boilingcurve_p(p));
              annotation(
                Inline = true);
            end dhl_dp;

            function dhv_dp "Derivative of vapour specific enthalpy on the boundary between regions 4 and 3 or 1 w.r.t. pressure"
              extends Modelica.Icons.Function;
              input SI.Pressure p "Pressure";
              output SI.DerEnthalpyByPressure dh_dp "Specific enthalpy derivative w.r.t. pressure";
            algorithm
              dh_dp := hvl_dp(p, dewcurve_p(p));
              annotation(
                Inline = true);
            end dhv_dp;

            function drhovl_dp
              extends Modelica.Icons.Function;
              input SI.Pressure p "Saturation pressure";
              input Common.IF97PhaseBoundaryProperties bpro "Property record";
              output Real dd_dp(unit = "kg/(m3.Pa)") "Derivative of density along the phase boundary";
            algorithm
              dd_dp := if bpro.region3boundary then (1.0 - bpro.pt / bpro.dpT) / bpro.pd else -bpro.d * bpro.d * (bpro.vp + bpro.vt / bpro.dpT);
              annotation(
                Inline = true);
            end drhovl_dp;

            function drhol_dp "Derivative of density of saturated water w.r.t. pressure"
              extends Modelica.Icons.Function;
              input SI.Pressure p "Saturation pressure";
              output SI.DerDensityByPressure dd_dp "Derivative of density of water at the boiling point";
            algorithm
              dd_dp := drhovl_dp(p, boilingcurve_p(p));
              annotation(
                Inline = true);
            end drhol_dp;

            function drhov_dp "Derivative of density of saturated steam w.r.t. pressure"
              extends Modelica.Icons.Function;
              input SI.Pressure p "Saturation pressure";
              output SI.DerDensityByPressure dd_dp "Derivative of density of water at the boiling point";
            algorithm
              dd_dp := drhovl_dp(p, dewcurve_p(p));
              annotation(
                Inline = true);
            end drhov_dp;
            annotation(
              Documentation(info = "<HTML><h4>Package description</h4>
 <p>Package <b>Regions</b> contains a large number of auxiliary functions which are needed to compute the current region
 of the IAPWS/IF97 for a given pair of input variables as quickly as possible. The focus of this implementation was on
 computational efficiency, not on compact code. Many of the function values calculated in these functions could be obtained
 using the fundamental functions of IAPWS/IF97, but with considerable overhead. If the region of IAPWS/IF97 is known in advance,
 the input variable mode can be set to the region, then the somewhat costly region checks are omitted.
 The checking for the phase has to be done outside the region functions because many properties are not
 differentiable at the region boundary. If the input phase is 2, the output region will be set to 4 immediately.</p>
 <h4>Package contents</h4>
 <p> The main 4 functions in this package are the functions returning the appropriate region for two input variables.</p>
 <ul>
 <li>Function <b>region_ph</b> compute the region of IAPWS/IF97 for input pair pressure and specific enthalpy.</li>
 <li>Function <b>region_ps</b> compute the region of IAPWS/IF97 for input pair pressure and specific entropy</li>
 <li>Function <b>region_dT</b> compute the region of IAPWS/IF97 for input pair density and temperature.</li>
 <li>Function <b>region_pT</b> compute the region of IAPWS/IF97 for input pair pressure and temperature (only in phase region).</li>
 </ul>
 <p>In addition, functions of the boiling and condensation curves compute the specific enthalpy, specific entropy, or density on these
 curves. The functions for the saturation pressure and temperature are included in the package <b>Basic</b> because they are part of
 the original <a href=\"modelica://Modelica/Resources/Documentation/Media/Water/IF97documentation/IF97.pdf\">IAPWS/IF97 standards document</a>. These functions are also aliased to
 be used directly from package <b>Water</b>.
</p>
 <ul>
 <li>Function <b>hl_p</b> computes the liquid specific enthalpy as a function of pressure. For overcritical pressures,
 the critical specific enthalpy is returned. An approximation is used for temperatures > 623.15 K.</li>
 <li>Function <b>hv_p</b> computes the vapour specific enthalpy as a function of pressure. For overcritical pressures,
 the critical specific enthalpy is returned. An approximation is used for temperatures > 623.15 K.</li>
 <li>Function <b>sl_p</b> computes the liquid specific entropy as a function of pressure. For overcritical pressures,
 the critical  specific entropy is returned. An approximation is used for temperatures > 623.15 K.</li>
 <li>Function <b>sv_p</b> computes the vapour  specific entropy as a function of pressure. For overcritical pressures,
 the critical  specific entropy is returned. An approximation is used for temperatures > 623.15 K.</li>
 <li>Function <b>rhol_T</b> computes the liquid density as a function of temperature. For overcritical temperatures,
 the critical density is returned. An approximation is used for temperatures > 623.15 K.</li>
 <li>Function <b>rhol_T</b> computes the vapour density as a function of temperature. For overcritical temperatures,
 the critical density is returned. An approximation is used for temperatures > 623.15 K.</li>
 </ul>
 <p>All other functions are auxiliary functions called from the region functions to check a specific boundary.</p>
 <ul>
 <li>Function <b>boundary23ofT</b> computes the boundary pressure between regions 2 and 3 (input temperature)</li>
 <li>Function <b>boundary23ofp</b> computes the boundary temperature between regions 2 and 3 (input pressure)</li>
 <li>Function <b>hlowerofp5</b> computes the lower specific enthalpy limit of region 5 (input p, T=1073.15 K)</li>
 <li>Function <b>hupperofp5</b> computes the upper specific enthalpy limit of region 5 (input p, T=2273.15 K)</li>
 <li>Function <b>slowerofp5</b> computes the lower specific entropy limit of region 5 (input p, T=1073.15 K)</li>
 <li>Function <b>supperofp5</b> computes the upper specific entropy limit of region 5 (input p, T=2273.15 K)</li>
 <li>Function <b>hlowerofp1</b> computes the lower specific enthalpy limit of region 1 (input p, T=273.15 K)</li>
 <li>Function <b>hupperofp1</b> computes the upper specific enthalpy limit of region 1 (input p, T=623.15 K)</li>
 <li>Function <b>slowerofp1</b> computes the lower specific entropy limit of region 1 (input p, T=273.15 K)</li>
 <li>Function <b>supperofp1</b> computes the upper specific entropy limit of region 1 (input p, T=623.15 K)</li>
 <li>Function <b>hlowerofp2</b> computes the lower specific enthalpy limit of region 2 (input p, T=623.15 K)</li>
 <li>Function <b>hupperofp2</b> computes the upper specific enthalpy limit of region 2 (input p, T=1073.15 K)</li>
 <li>Function <b>slowerofp2</b> computes the lower specific entropy limit of region 2 (input p, T=623.15 K)</li>
 <li>Function <b>supperofp2</b> computes the upper specific entropy limit of region 2 (input p, T=1073.15 K)</li>
 <li>Function <b>d1n</b> computes the density in region 1 as function of pressure and temperature</li>
 <li>Function <b>d2n</b> computes the density in region 2 as function of pressure and temperature</li>
 <li>Function <b>dhot1ofp</b> computes the hot density limit of region 1 (input p, T=623.15 K)</li>
 <li>Function <b>dupper1ofT</b>computes the high pressure density limit of region 1 (input T, p=100MPa)</li>
 <li>Function <b>hl_p_R4b</b> computes a high accuracy approximation to the liquid enthalpy for temperatures > 623.15 K (input p)</li>
 <li>Function <b>hv_p_R4b</b> computes a high accuracy approximation to the vapour enthalpy for temperatures > 623.15 K (input p)</li>
 <li>Function <b>sl_p_R4b</b> computes a high accuracy approximation to the liquid entropy for temperatures > 623.15 K (input p)</li>
 <li>Function <b>sv_p_R4b</b> computes a high accuracy approximation to the vapour entropy for temperatures > 623.15 K (input p)</li>
 <li>Function <b>rhol_p_R4b</b> computes a high accuracy approximation to the liquid density for temperatures > 623.15 K (input p)</li>
 <li>Function <b>rhov_p_R4b</b> computes a high accuracy approximation to the vapour density for temperatures > 623.15 K (input p)</li>
 </ul>

<h4>Version Info and Revision history</h4>
<ul>
<li>First implemented: <i>July, 2000</i>
       by <a href=\"http://www.control.lth.se/~hubertus/\">Hubertus Tummescheit</a>
       </li>
</ul>
<address>Authors: Hubertus Tummescheit, Jonas Eborn and Falko Jens Wagner<br>
      Modelon AB<br>
      Ideon Science Park<br>
      SE-22370 Lund, Sweden<br>
      email: hubertus@modelon.se
 </address>
 <ul>
 <li>Initial version: July 2000</li>
 <li>Revised and extended for inclusion in Modelica.Thermal: December 2002</li>
</ul>
</html>"));
          end Regions;

          //===================================================================
          //                      Base functions
          //===================================================================

          package Basic "Base functions as described in IAWPS/IF97"
            extends Modelica.Icons.Package;

            function g1 "Gibbs function for region 1: g(p,T)"
              extends Modelica.Icons.Function;
              input SI.Pressure p "Pressure";
              input SI.Temperature T "Temperature (K)";
              output Modelica.Media.Common.GibbsDerivs g "Dimensionless Gibbs function and derivatives w.r.t. pi and tau";
            protected
              Real pi1 "Dimensionless pressure";
              Real tau1 "Dimensionless temperature";
              Real[45] o "Vector of auxiliary variables";
              Real pl "Auxiliary variable";
            algorithm
              pl := min(p, data.PCRIT - 1);
              assert(p > triple.ptriple, "IF97 medium function g1 called with too low pressure\n" + "p = " + String(p) + " Pa <= " + String(triple.ptriple) + " Pa (triple point pressure)");
              assert(p <= 100.0e6, "IF97 medium function g1: the input pressure (= " + String(p) + " Pa) is higher than 100 Mpa");
              assert(T >= 273.15, "IF97 medium function g1: the temperature (= " + String(T) + " K) is lower than 273.15 K!");
              g.p := p;
              g.T := T;
              g.R := data.RH2O;
              g.pi := p / data.PSTAR1;
              g.tau := data.TSTAR1 / T;
              pi1 := 7.1000000000000 - g.pi;
              tau1 := (-1.22200000000000) + g.tau;
              o[1] := tau1 * tau1;
              o[2] := o[1] * o[1];
              o[3] := o[2] * o[2];
              o[4] := o[3] * tau1;
              o[5] := 1 / o[4];
              o[6] := o[1] * o[2];
              o[7] := o[1] * tau1;
              o[8] := 1 / o[7];
              o[9] := o[1] * o[2] * o[3];
              o[10] := 1 / o[2];
              o[11] := o[2] * tau1;
              o[12] := 1 / o[11];
              o[13] := o[2] * o[3];
              o[14] := 1 / o[3];
              o[15] := pi1 * pi1;
              o[16] := o[15] * pi1;
              o[17] := o[15] * o[15];
              o[18] := o[17] * o[17];
              o[19] := o[17] * o[18] * pi1;
              o[20] := o[15] * o[17];
              o[21] := o[3] * o[3];
              o[22] := o[21] * o[21];
              o[23] := o[22] * o[3] * tau1;
              o[24] := 1 / o[23];
              o[25] := o[22] * o[3];
              o[26] := 1 / o[25];
              o[27] := o[1] * o[2] * o[22] * tau1;
              o[28] := 1 / o[27];
              o[29] := o[1] * o[2] * o[22];
              o[30] := 1 / o[29];
              o[31] := o[1] * o[2] * o[21] * o[3] * tau1;
              o[32] := 1 / o[31];
              o[33] := o[2] * o[21] * o[3] * tau1;
              o[34] := 1 / o[33];
              o[35] := o[1] * o[3] * tau1;
              o[36] := 1 / o[35];
              o[37] := o[1] * o[3];
              o[38] := 1 / o[37];
              o[39] := 1 / o[6];
              o[40] := o[1] * o[22] * o[3];
              o[41] := 1 / o[40];
              o[42] := 1 / o[22];
              o[43] := o[1] * o[2] * o[21] * o[3];
              o[44] := 1 / o[43];
              o[45] := 1 / o[13];
              g.g := pi1 * (pi1 * (pi1 * (o[10] * ((-0.000031679644845054) + o[2] * ((-2.82707979853120e-6) - 8.5205128120103e-10 * o[6])) + pi1 * (o[12] * ((-2.24252819080000e-6) + ((-6.5171222895601e-7) - 1.43417299379240e-13 * o[13]) * o[7]) + pi1 * ((-4.0516996860117e-7 * o[14]) + o[16] * (((-1.27343017416410e-9) - 1.74248712306340e-10 * o[11]) * o[36] + o[19] * ((-6.8762131295531e-19 * o[34]) + o[15] * (1.44783078285210e-20 * o[32] + o[20] * (2.63357816627950e-23 * o[30] + pi1 * ((-1.19476226400710e-23 * o[28]) + pi1 * (1.82280945814040e-24 * o[26] - 9.3537087292458e-26 * o[24] * pi1))))))))) + o[8] * ((-0.00047184321073267) + o[7] * ((-0.000300017807930260) + (0.000047661393906987 + o[1] * ((-4.4141845330846e-6) - 7.2694996297594e-16 * o[9])) * tau1))) + o[5] * (0.000283190801238040 + o[1] * ((-0.00060706301565874) + o[6] * ((-0.0189900682184190) + tau1 * ((-0.032529748770505) + ((-0.0218417171754140) - 0.000052838357969930 * o[1]) * tau1))))) + (0.146329712131670 + tau1 * ((-0.84548187169114) + tau1 * ((-3.7563603672040) + tau1 * (3.3855169168385 + tau1 * ((-0.95791963387872) + tau1 * (0.157720385132280 + ((-0.0166164171995010) + 0.00081214629983568 * tau1) * tau1)))))) / o[1];
              g.gpi := pi1 * (pi1 * (o[10] * (0.000095038934535162 + o[2] * (8.4812393955936e-6 + 2.55615384360309e-9 * o[6])) + pi1 * (o[12] * (8.9701127632000e-6 + (2.60684891582404e-6 + 5.7366919751696e-13 * o[13]) * o[7]) + pi1 * (2.02584984300585e-6 * o[14] + o[16] * ((1.01874413933128e-8 + 1.39398969845072e-9 * o[11]) * o[36] + o[19] * (1.44400475720615e-17 * o[34] + o[15] * ((-3.3300108005598e-19 * o[32]) + o[20] * ((-7.6373766822106e-22 * o[30]) + pi1 * (3.5842867920213e-22 * o[28] + pi1 * ((-5.6507093202352e-23 * o[26]) + 2.99318679335866e-24 * o[24] * pi1))))))))) + o[8] * (0.00094368642146534 + o[7] * (0.00060003561586052 + ((-0.000095322787813974) + o[1] * (8.8283690661692e-6 + 1.45389992595188e-15 * o[9])) * tau1))) + o[5] * ((-0.000283190801238040) + o[1] * (0.00060706301565874 + o[6] * (0.0189900682184190 + tau1 * (0.032529748770505 + (0.0218417171754140 + 0.000052838357969930 * o[1]) * tau1))));
              g.gpipi := pi1 * (o[10] * ((-0.000190077869070324) + o[2] * ((-0.0000169624787911872) - 5.1123076872062e-9 * o[6])) + pi1 * (o[12] * ((-0.0000269103382896000) + ((-7.8205467474721e-6) - 1.72100759255088e-12 * o[13]) * o[7]) + pi1 * ((-8.1033993720234e-6 * o[14]) + o[16] * (((-7.1312089753190e-8) - 9.7579278891550e-9 * o[11]) * o[36] + o[19] * ((-2.88800951441230e-16 * o[34]) + o[15] * (7.3260237612316e-18 * o[32] + o[20] * (2.13846547101895e-20 * o[30] + pi1 * ((-1.03944316968618e-20 * o[28]) + pi1 * (1.69521279607057e-21 * o[26] - 9.2788790594118e-23 * o[24] * pi1))))))))) + o[8] * ((-0.00094368642146534) + o[7] * ((-0.00060003561586052) + (0.000095322787813974 + o[1] * ((-8.8283690661692e-6) - 1.45389992595188e-15 * o[9])) * tau1));
              g.gtau := pi1 * (o[38] * ((-0.00254871721114236) + o[1] * (0.0042494411096112 + (0.0189900682184190 + ((-0.0218417171754140) - 0.000158515073909790 * o[1]) * o[1]) * o[6])) + pi1 * (o[10] * (0.00141552963219801 + o[2] * (0.000047661393906987 + o[1] * ((-0.0000132425535992538) - 1.23581493705910e-14 * o[9]))) + pi1 * (o[12] * (0.000126718579380216 - 5.1123076872062e-9 * o[37]) + pi1 * (o[39] * (0.0000112126409540000 + (1.30342445791202e-6 - 1.43417299379240e-12 * o[13]) * o[7]) + pi1 * (3.2413597488094e-6 * o[5] + o[16] * ((1.40077319158051e-8 + 1.04549227383804e-9 * o[11]) * o[45] + o[19] * (1.99410180757040e-17 * o[44] + o[15] * ((-4.4882754268415e-19 * o[42]) + o[20] * ((-1.00075970318621e-21 * o[28]) + pi1 * (4.6595728296277e-22 * o[26] + pi1 * ((-7.2912378325616e-23 * o[24]) + 3.8350205789908e-24 * o[41] * pi1))))))))))) + o[8] * ((-0.292659424263340) + tau1 * (0.84548187169114 + o[1] * (3.3855169168385 + tau1 * ((-1.91583926775744) + tau1 * (0.47316115539684 + ((-0.066465668798004) + 0.0040607314991784 * tau1) * tau1)))));
              g.gtautau := pi1 * (o[36] * (0.0254871721114236 + o[1] * ((-0.033995528876889) + ((-0.037980136436838) - 0.00031703014781958 * o[2]) * o[6])) + pi1 * (o[12] * ((-0.0056621185287920) + o[6] * ((-0.0000264851071985076) - 1.97730389929456e-13 * o[9])) + pi1 * (((-0.00063359289690108) - 2.55615384360309e-8 * o[37]) * o[39] + pi1 * (pi1 * ((-0.0000291722377392842 * o[38]) + o[16] * (o[19] * ((-5.9823054227112e-16 * o[32]) + o[15] * (o[20] * (3.9029628424262e-20 * o[26] + pi1 * ((-1.86382913185108e-20 * o[24]) + pi1 * (2.98940751135026e-21 * o[41] - 1.61070864317613e-22 * pi1 / (o[1] * o[22] * o[3] * tau1)))) + 1.43624813658928e-17 / (o[22] * tau1))) + ((-1.68092782989661e-7) - 7.3184459168663e-9 * o[11]) / (o[2] * o[3] * tau1))) + ((-0.000067275845724000) + ((-3.9102733737361e-6) - 1.29075569441316e-11 * o[13]) * o[7]) / (o[1] * o[2] * tau1))))) + o[10] * (0.87797827279002 + tau1 * ((-1.69096374338228) + o[7] * ((-1.91583926775744) + tau1 * (0.94632231079368 + ((-0.199397006394012) + 0.0162429259967136 * tau1) * tau1))));
              g.gtaupi := o[38] * (0.00254871721114236 + o[1] * ((-0.0042494411096112) + ((-0.0189900682184190) + (0.0218417171754140 + 0.000158515073909790 * o[1]) * o[1]) * o[6])) + pi1 * (o[10] * ((-0.00283105926439602) + o[2] * ((-0.000095322787813974) + o[1] * (0.0000264851071985076 + 2.47162987411820e-14 * o[9]))) + pi1 * (o[12] * ((-0.00038015573814065) + 1.53369230616185e-8 * o[37]) + pi1 * (o[39] * ((-0.000044850563816000) + ((-5.2136978316481e-6) + 5.7366919751696e-12 * o[13]) * o[7]) + pi1 * ((-0.0000162067987440468 * o[5]) + o[16] * (((-1.12061855326441e-7) - 8.3639381907043e-9 * o[11]) * o[45] + o[19] * ((-4.1876137958978e-16 * o[44]) + o[15] * (1.03230334817355e-17 * o[42] + o[20] * (2.90220313924001e-20 * o[28] + pi1 * ((-1.39787184888831e-20 * o[26]) + pi1 * (2.26028372809410e-21 * o[24] - 1.22720658527705e-22 * o[41] * pi1))))))))));
            end g1;

            function g2 "Gibbs function for region 2: g(p,T)"
              extends Modelica.Icons.Function;
              input SI.Pressure p "Pressure";
              input SI.Temperature T "Temperature (K)";
              output Modelica.Media.Common.GibbsDerivs g "Dimensionless Gibbs function and derivatives w.r.t. pi and tau";
            protected
              Real tau2 "Dimensionless temperature";
              Real[55] o "Vector of auxiliary variables";
            algorithm
              g.p := p;
              g.T := T;
              g.R := data.RH2O;
              assert(p > 0.0, "IF97 medium function g2 called with too low pressure\n" + "p = " + String(p) + " Pa <=  0.0 Pa");
              assert(p <= 100.0e6, "IF97 medium function g2: the input pressure (= " + String(p) + " Pa) is higher than 100 Mpa");
              assert(T >= 273.15, "IF97 medium function g2: the temperature (= " + String(T) + " K) is lower than 273.15 K!");
              assert(T <= 1073.15, "IF97 medium function g2: the input temperature (= " + String(T) + " K) is higher than the limit of 1073.15 K");
              g.pi := p / data.PSTAR2;
              g.tau := data.TSTAR2 / T;
              tau2 := (-0.5) + g.tau;
              o[1] := tau2 * tau2;
              o[2] := o[1] * tau2;
              o[3] := -0.050325278727930 * o[2];
              o[4] := (-0.057581259083432) + o[3];
              o[5] := o[4] * tau2;
              o[6] := (-0.045996013696365) + o[5];
              o[7] := o[6] * tau2;
              o[8] := (-0.0178348622923580) + o[7];
              o[9] := o[8] * tau2;
              o[10] := o[1] * o[1];
              o[11] := o[10] * o[10];
              o[12] := o[11] * o[11];
              o[13] := o[10] * o[11] * o[12] * tau2;
              o[14] := o[1] * o[10] * tau2;
              o[15] := o[10] * o[11] * tau2;
              o[16] := o[1] * o[12] * tau2;
              o[17] := o[1] * o[11] * tau2;
              o[18] := o[1] * o[10] * o[11];
              o[19] := o[10] * o[11] * o[12];
              o[20] := o[1] * o[10];
              o[21] := g.pi * g.pi;
              o[22] := o[21] * o[21];
              o[23] := o[21] * o[22];
              o[24] := o[10] * o[12] * tau2;
              o[25] := o[12] * o[12];
              o[26] := o[11] * o[12] * o[25] * tau2;
              o[27] := o[10] * o[12];
              o[28] := o[1] * o[10] * o[11] * tau2;
              o[29] := o[10] * o[12] * o[25] * tau2;
              o[30] := o[1] * o[10] * o[25] * tau2;
              o[31] := o[1] * o[11] * o[12];
              o[32] := o[1] * o[12];
              o[33] := g.tau * g.tau;
              o[34] := o[33] * o[33];
              o[35] := -0.000053349095828174 * o[13];
              o[36] := (-0.087594591301146) + o[35];
              o[37] := o[2] * o[36];
              o[38] := (-0.0078785554486710) + o[37];
              o[39] := o[1] * o[38];
              o[40] := (-0.00037897975032630) + o[39];
              o[41] := o[40] * tau2;
              o[42] := (-0.000066065283340406) + o[41];
              o[43] := o[42] * tau2;
              o[44] := 5.7870447262208e-6 * tau2;
              o[45] := -0.301951672367580 * o[2];
              o[46] := (-0.172743777250296) + o[45];
              o[47] := o[46] * tau2;
              o[48] := (-0.091992027392730) + o[47];
              o[49] := o[48] * tau2;
              o[50] := o[1] * o[11];
              o[51] := o[10] * o[11];
              o[52] := o[11] * o[12] * o[25];
              o[53] := o[10] * o[12] * o[25];
              o[54] := o[1] * o[10] * o[25];
              o[55] := o[11] * o[12] * tau2;
              g.g := g.pi * ((-0.00177317424732130) + o[9] + g.pi * (tau2 * ((-0.000033032641670203) + ((-0.000189489875163150) + o[1] * ((-0.0039392777243355) + ((-0.043797295650573) - 0.0000266745479140870 * o[13]) * o[2])) * tau2) + g.pi * (2.04817376923090e-8 + (4.3870667284435e-7 + o[1] * ((-0.000032277677238570) + ((-0.00150339245421480) - 0.040668253562649 * o[13]) * o[2])) * tau2 + g.pi * (g.pi * (2.29220763376610e-6 * o[14] + g.pi * (((-1.67147664510610e-11) + o[15] * ((-0.00211714723213550) - 23.8957419341040 * o[16])) * o[2] + g.pi * ((-5.9059564324270e-18) + o[17] * ((-1.26218088991010e-6) - 0.038946842435739 * o[18]) + g.pi * (o[11] * (1.12562113604590e-11 - 8.2311340897998 * o[19]) + g.pi * (1.98097128020880e-8 * o[15] + g.pi * (o[10] * (1.04069652101740e-19 + ((-1.02347470959290e-13) - 1.00181793795110e-9 * o[10]) * o[20]) + o[23] * (o[13] * ((-8.0882908646985e-11) + 0.106930318794090 * o[24]) + o[21] * ((-0.33662250574171 * o[26]) + o[21] * (o[27] * (8.9185845355421e-25 + (3.06293168762320e-13 - 4.2002467698208e-6 * o[15]) * o[28]) + g.pi * ((-5.9056029685639e-26 * o[24]) + g.pi * (3.7826947613457e-6 * o[29] + g.pi * ((-1.27686089346810e-15 * o[30]) + o[31] * (7.3087610595061e-29 + o[18] * (5.5414715350778e-17 - 9.4369707241210e-7 * o[32])) * g.pi)))))))))))) + tau2 * ((-7.8847309559367e-10) + (1.27907178522850e-8 + 4.8225372718507e-7 * tau2) * tau2))))) + ((-0.0056087911830200) + g.tau * (0.071452738814550 + g.tau * ((-0.40710498239280) + g.tau * (1.42408197144400 + g.tau * ((-4.3839511194500) + g.tau * ((-9.6927686002170) + g.tau * (10.0866556801800 + ((-0.284086326077200) + 0.0212684635330700 * g.tau) * g.tau) + Modelica.Math.log(g.pi))))))) / (o[34] * g.tau);
              g.gpi := (1.00000000000000 + g.pi * ((-0.00177317424732130) + o[9] + g.pi * (o[43] + g.pi * (6.1445213076927e-8 + (1.31612001853305e-6 + o[1] * ((-0.000096833031715710) + ((-0.0045101773626444) - 0.122004760687947 * o[13]) * o[2])) * tau2 + g.pi * (g.pi * (0.0000114610381688305 * o[14] + g.pi * (((-1.00288598706366e-10) + o[15] * ((-0.0127028833928130) - 143.374451604624 * o[16])) * o[2] + g.pi * ((-4.1341695026989e-17) + o[17] * ((-8.8352662293707e-6) - 0.272627897050173 * o[18]) + g.pi * (o[11] * (9.0049690883672e-11 - 65.849072718398 * o[19]) + g.pi * (1.78287415218792e-7 * o[15] + g.pi * (o[10] * (1.04069652101740e-18 + ((-1.02347470959290e-12) - 1.00181793795110e-8 * o[10]) * o[20]) + o[23] * (o[13] * ((-1.29412653835176e-9) + 1.71088510070544 * o[24]) + o[21] * ((-6.0592051033508 * o[26]) + o[21] * (o[27] * (1.78371690710842e-23 + (6.1258633752464e-12 - 0.000084004935396416 * o[15]) * o[28]) + g.pi * ((-1.24017662339842e-24 * o[24]) + g.pi * (0.000083219284749605 * o[29] + g.pi * ((-2.93678005497663e-14 * o[30]) + o[31] * (1.75410265428146e-27 + o[18] * (1.32995316841867e-15 - 0.0000226487297378904 * o[32])) * g.pi)))))))))))) + tau2 * ((-3.15389238237468e-9) + (5.1162871409140e-8 + 1.92901490874028e-6 * tau2) * tau2)))))) / g.pi;
              g.gpipi := ((-1.00000000000000) + o[21] * (o[43] + g.pi * (1.22890426153854e-7 + (2.63224003706610e-6 + o[1] * ((-0.000193666063431420) + ((-0.0090203547252888) - 0.244009521375894 * o[13]) * o[2])) * tau2 + g.pi * (g.pi * (0.000045844152675322 * o[14] + g.pi * (((-5.0144299353183e-10) + o[15] * ((-0.063514416964065) - 716.87225802312 * o[16])) * o[2] + g.pi * ((-2.48050170161934e-16) + o[17] * ((-0.000053011597376224) - 1.63576738230104 * o[18]) + g.pi * (o[11] * (6.3034783618570e-10 - 460.94350902879 * o[19]) + g.pi * (1.42629932175034e-6 * o[15] + g.pi * (o[10] * (9.3662686891566e-18 + ((-9.2112723863361e-12) - 9.0163614415599e-8 * o[10]) * o[20]) + o[23] * (o[13] * ((-1.94118980752764e-8) + 25.6632765105816 * o[24]) + o[21] * ((-103.006486756963 * o[26]) + o[21] * (o[27] * (3.3890621235060e-22 + (1.16391404129682e-10 - 0.00159609377253190 * o[15]) * o[28]) + g.pi * ((-2.48035324679684e-23 * o[24]) + g.pi * (0.00174760497974171 * o[29] + g.pi * ((-6.4609161209486e-13 * o[30]) + o[31] * (4.0344361048474e-26 + o[18] * (3.05889228736295e-14 - 0.00052092078397148 * o[32])) * g.pi)))))))))))) + tau2 * ((-9.4616771471240e-9) + (1.53488614227420e-7 + o[44]) * tau2))))) / o[21];
              g.gtau := (0.0280439559151000 + g.tau * ((-0.285810955258200) + g.tau * (1.22131494717840 + g.tau * ((-2.84816394288800) + g.tau * (4.3839511194500 + o[33] * (10.0866556801800 + ((-0.56817265215440) + 0.063805390599210 * g.tau) * g.tau)))))) / (o[33] * o[34]) + g.pi * ((-0.0178348622923580) + o[49] + g.pi * ((-0.000033032641670203) + ((-0.00037897975032630) + o[1] * ((-0.0157571108973420) + ((-0.306581069554011) - 0.00096028372490713 * o[13]) * o[2])) * tau2 + g.pi * (4.3870667284435e-7 + o[1] * ((-0.000096833031715710) + ((-0.0090203547252888) - 1.42338887469272 * o[13]) * o[2]) + g.pi * ((-7.8847309559367e-10) + g.pi * (0.0000160454534363627 * o[20] + g.pi * (o[1] * ((-5.0144299353183e-11) + o[15] * ((-0.033874355714168) - 836.35096769364 * o[16])) + g.pi * (((-0.0000138839897890111) - 0.97367106089347 * o[18]) * o[50] + g.pi * (o[14] * (9.0049690883672e-11 - 296.320827232793 * o[19]) + g.pi * (2.57526266427144e-7 * o[51] + g.pi * (o[2] * (4.1627860840696e-19 + ((-1.02347470959290e-12) - 1.40254511313154e-8 * o[10]) * o[20]) + o[23] * (o[19] * ((-2.34560435076256e-9) + 5.3465159397045 * o[24]) + o[21] * ((-19.1874828272775 * o[52]) + o[21] * (o[16] * (1.78371690710842e-23 + (1.07202609066812e-11 - 0.000201611844951398 * o[15]) * o[28]) + g.pi * ((-1.24017662339842e-24 * o[27]) + g.pi * (0.000200482822351322 * o[53] + g.pi * ((-4.9797574845256e-14 * o[54]) + (1.90027787547159e-27 + o[18] * (2.21658861403112e-15 - 0.000054734430199902 * o[32])) * o[55] * g.pi)))))))))))) + (2.55814357045700e-8 + 1.44676118155521e-6 * tau2) * tau2))));
              g.gtautau := ((-0.168263735490600) + g.tau * (1.42905477629100 + g.tau * ((-4.8852597887136) + g.tau * (8.5444918286640 + g.tau * ((-8.7679022389000) + o[33] * ((-0.56817265215440) + 0.127610781198420 * g.tau) * g.tau))))) / (o[33] * o[34] * g.tau) + g.pi * ((-0.091992027392730) + ((-0.34548755450059) - 1.50975836183790 * o[2]) * tau2 + g.pi * ((-0.00037897975032630) + o[1] * ((-0.047271332692026) + ((-1.83948641732407) - 0.033609930371750 * o[13]) * o[2]) + g.pi * (((-0.000193666063431420) + ((-0.045101773626444) - 48.395221739552 * o[13]) * o[2]) * tau2 + g.pi * (2.55814357045700e-8 + 2.89352236311042e-6 * tau2 + g.pi * (0.000096272720618176 * o[10] * tau2 + g.pi * (((-1.00288598706366e-10) + o[15] * ((-0.50811533571252) - 28435.9329015838 * o[16])) * tau2 + g.pi * (o[11] * ((-0.000138839897890111) - 23.3681054614434 * o[18]) * tau2 + g.pi * ((6.3034783618570e-10 - 10371.2289531477 * o[19]) * o[20] + g.pi * (3.09031519712573e-6 * o[17] + g.pi * (o[1] * (1.24883582522088e-18 + ((-9.2112723863361e-12) - 1.82330864707100e-7 * o[10]) * o[20]) + o[23] * (o[1] * o[11] * o[12] * ((-6.5676921821352e-8) + 261.979281045521 * o[24]) * tau2 + o[21] * ((-1074.49903832754 * o[1] * o[10] * o[12] * o[25] * tau2) + o[21] * ((3.3890621235060e-22 + (3.6448887082716e-10 - 0.0094757567127157 * o[15]) * o[28]) * o[32] + g.pi * ((-2.48035324679684e-23 * o[16]) + g.pi * (0.0104251067622687 * o[1] * o[12] * o[25] * tau2 + g.pi * (o[11] * o[12] * (4.7506946886790e-26 + o[18] * (8.6446955947214e-14 - 0.00311986252139440 * o[32])) * g.pi - 1.89230784411972e-12 * o[10] * o[25] * tau2))))))))))))))));
              g.gtaupi := (-0.0178348622923580) + o[49] + g.pi * ((-0.000066065283340406) + ((-0.00075795950065260) + o[1] * ((-0.0315142217946840) + ((-0.61316213910802) - 0.00192056744981426 * o[13]) * o[2])) * tau2 + g.pi * (1.31612001853305e-6 + o[1] * ((-0.000290499095147130) + ((-0.0270610641758664) - 4.2701666240781 * o[13]) * o[2]) + g.pi * ((-3.15389238237468e-9) + g.pi * (0.000080227267181813 * o[20] + g.pi * (o[1] * ((-3.00865796119098e-10) + o[15] * ((-0.203246134285008) - 5018.1058061618 * o[16])) + g.pi * (((-0.000097187928523078) - 6.8156974262543 * o[18]) * o[50] + g.pi * (o[14] * (7.2039752706938e-10 - 2370.56661786234 * o[19]) + g.pi * (2.31773639784430e-6 * o[51] + g.pi * (o[2] * (4.1627860840696e-18 + ((-1.02347470959290e-11) - 1.40254511313154e-7 * o[10]) * o[20]) + o[23] * (o[19] * ((-3.7529669612201e-8) + 85.544255035272 * o[24]) + o[21] * ((-345.37469089099 * o[52]) + o[21] * (o[16] * (3.5674338142168e-22 + (2.14405218133624e-10 - 0.0040322368990280 * o[15]) * o[28]) + g.pi * ((-2.60437090913668e-23 * o[27]) + g.pi * (0.0044106220917291 * o[53] + g.pi * ((-1.14534422144089e-12 * o[54]) + (4.5606669011318e-26 + o[18] * (5.3198126736747e-14 - 0.00131362632479764 * o[32])) * o[55] * g.pi)))))))))))) + (1.02325742818280e-7 + o[44]) * tau2)));
            end g2;

            function g2metastable "Gibbs function for metastable part of region 2: g(p,T)"
              extends Modelica.Icons.Function;
              input SI.Pressure p "Pressure";
              input SI.Temperature T "Temperature (K)";
              output Modelica.Media.Common.GibbsDerivs g "Dimensionless Gibbs function and derivatives w.r.t. pi and tau";
            protected
              Real pi "Dimensionless pressure";
              Real tau "Dimensionless temperature";
              Real tau2 "Dimensionless temperature";
              Real[27] o "Vector of auxiliary variables";
            algorithm
              assert(p > 0.0, "IF97 medium function g2metastable called with too low pressure\n" + "p = " + String(p) + " Pa <=  0.0 Pa");
              assert(p <= 100.0e6, "IF97 medium function g2metastable: the input pressure (= " + String(p) + " Pa) is higher than 100 Mpa");
              assert(T >= 273.15, "IF97 medium function g2metastable: the temperature (= " + String(T) + " K) is lower than 273.15 K!");
              assert(T <= 1073.15, "IF97 medium function g2metastable: the input temperature (= " + String(T) + " K) is higher than the limit of 1073.15 K");
              g.p := p;
              g.T := T;
              g.R := data.RH2O;
              g.pi := p / data.PSTAR2;
              g.tau := data.TSTAR2 / T;
              tau2 := (-0.5) + g.tau;
              o[1] := tau2 * tau2;
              o[2] := o[1] * tau2;
              o[3] := o[1] * o[1];
              o[4] := o[1] * o[3];
              o[5] := -0.0040813178534455 * o[4];
              o[6] := (-0.072334555213245) + o[5];
              o[7] := o[2] * o[6];
              o[8] := (-0.088223831943146) + o[7];
              o[9] := o[1] * o[8];
              o[10] := o[3] * o[3];
              o[11] := o[10] * tau2;
              o[12] := o[10] * o[3];
              o[13] := o[1] * o[3] * tau2;
              o[14] := g.tau * g.tau;
              o[15] := o[14] * o[14];
              o[16] := -0.015238081817394 * o[11];
              o[17] := (-0.106091843797284) + o[16];
              o[18] := o[17] * o[4];
              o[19] := 0.0040195606760414 + o[18];
              o[20] := o[19] * tau2;
              o[21] := g.pi * g.pi;
              o[22] := -0.0448944963879005 * o[4];
              o[23] := (-0.361672776066225) + o[22];
              o[24] := o[2] * o[23];
              o[25] := (-0.176447663886292) + o[24];
              o[26] := o[25] * tau2;
              o[27] := o[3] * tau2;
              g.g := g.pi * ((-0.0073362260186506) + o[9] + g.pi * (g.pi * (((-0.0063498037657313) - 0.086043093028588 * o[12]) * o[3] + g.pi * (o[13] * (0.007532158152277 - 0.0079238375446139 * o[2]) + o[11] * g.pi * ((-0.00022888160778447) - 0.002645650148281 * tau2))) + (0.0020097803380207 + ((-0.053045921898642) - 0.007619040908697 * o[11]) * o[4]) * tau2)) + ((-0.00560879118302) + g.tau * (0.07145273881455 + g.tau * ((-0.4071049823928) + g.tau * (1.424081971444 + g.tau * ((-4.38395111945) + g.tau * ((-9.6937268393049) + g.tau * (10.087275970006 + ((-0.2840863260772) + 0.02126846353307 * g.tau) * g.tau) + Modelica.Math.log(g.pi))))))) / (o[15] * g.tau);
              g.gpi := (1.0 + g.pi * ((-0.0073362260186506) + o[9] + g.pi * (o[20] + g.pi * (((-0.0190494112971939) - 0.258129279085764 * o[12]) * o[3] + g.pi * (o[13] * (0.030128632609108 - 0.0316953501784556 * o[2]) + o[11] * g.pi * ((-0.00114440803892235) - 0.013228250741405 * tau2)))))) / g.pi;
              g.gpipi := ((-1.) + o[21] * (o[20] + g.pi * (((-0.0380988225943878) - 0.516258558171528 * o[12]) * o[3] + g.pi * (o[13] * (0.090385897827324 - 0.0950860505353668 * o[2]) + o[11] * g.pi * ((-0.0045776321556894) - 0.05291300296562 * tau2))))) / o[21];
              g.gtau := (0.0280439559151 + g.tau * ((-0.2858109552582) + g.tau * (1.2213149471784 + g.tau * ((-2.848163942888) + g.tau * (4.38395111945 + o[14] * (10.087275970006 + ((-0.5681726521544) + 0.06380539059921 * g.tau) * g.tau)))))) / (o[14] * o[15]) + g.pi * (o[26] + g.pi * (0.0020097803380207 + ((-0.371321453290494) - 0.121904654539152 * o[11]) * o[4] + g.pi * (((-0.0253992150629252) - 1.37668948845741 * o[12]) * o[2] + g.pi * ((0.052725107065939 - 0.079238375446139 * o[2]) * o[4] + o[10] * g.pi * ((-0.00205993447006023) - 0.02645650148281 * tau2)))));
              g.gtautau := ((-0.1682637354906) + g.tau * (1.429054776291 + g.tau * ((-4.8852597887136) + g.tau * (8.544491828664 + g.tau * ((-8.7679022389) + o[14] * ((-0.5681726521544) + 0.12761078119842 * g.tau) * g.tau))))) / (o[14] * o[15] * g.tau) + g.pi * ((-0.176447663886292) + o[2] * ((-1.4466911042649) - 0.448944963879005 * o[4]) + g.pi * (((-2.22792871974296) - 1.82856981808728 * o[11]) * o[27] + g.pi * (o[1] * ((-0.0761976451887756) - 20.6503423268611 * o[12]) + g.pi * ((0.316350642395634 - 0.713145379015251 * o[2]) * o[27] + o[13] * g.pi * ((-0.0164794757604818) - 0.23810851334529 * tau2)))));
              g.gtaupi := o[26] + g.pi * (0.0040195606760414 + ((-0.742642906580988) - 0.243809309078304 * o[11]) * o[4] + g.pi * (((-0.0761976451887756) - 4.13006846537222 * o[12]) * o[2] + g.pi * ((0.210900428263756 - 0.316953501784556 * o[2]) * o[4] + o[10] * g.pi * ((-0.0102996723503012) - 0.13228250741405 * tau2))));
            end g2metastable;

            function f3 "Helmholtz function for region 3: f(d,T)"
              extends Modelica.Icons.Function;
              input SI.Density d "Density";
              input SI.Temperature T "Temperature (K)";
              output Modelica.Media.Common.HelmholtzDerivs f "Dimensionless Helmholtz function and derivatives w.r.t. delta and tau";
            protected
              Real[40] o "Vector of auxiliary variables";
            algorithm
              f.T := T;
              f.d := d;
              f.R := data.RH2O;
              f.tau := data.TCRIT / T;
              f.delta := if d == data.DCRIT and T == data.TCRIT then 1 - Modelica.Constants.eps else abs(d / data.DCRIT);
              o[1] := f.tau * f.tau;
              o[2] := o[1] * o[1];
              o[3] := o[2] * f.tau;
              o[4] := o[1] * f.tau;
              o[5] := o[2] * o[2];
              o[6] := o[1] * o[5] * f.tau;
              o[7] := o[5] * f.tau;
              o[8] := -0.64207765181607 * o[1];
              o[9] := 0.88521043984318 + o[8];
              o[10] := o[7] * o[9];
              o[11] := (-1.15244078066810) + o[10];
              o[12] := o[11] * o[2];
              o[13] := (-1.26543154777140) + o[12];
              o[14] := o[1] * o[13];
              o[15] := o[1] * o[2] * o[5] * f.tau;
              o[16] := o[2] * o[5];
              o[17] := o[1] * o[5];
              o[18] := o[5] * o[5];
              o[19] := o[1] * o[18] * o[2];
              o[20] := o[1] * o[18] * o[2] * f.tau;
              o[21] := o[18] * o[5];
              o[22] := o[1] * o[18] * o[5];
              o[23] := 0.251168168486160 * o[2];
              o[24] := 0.078841073758308 + o[23];
              o[25] := o[15] * o[24];
              o[26] := (-6.1005234513930) + o[25];
              o[27] := o[26] * f.tau;
              o[28] := 9.7944563083754 + o[27];
              o[29] := o[2] * o[28];
              o[30] := (-1.70429417648412) + o[29];
              o[31] := o[1] * o[30];
              o[32] := f.delta * f.delta;
              o[33] := -10.9153200808732 * o[1];
              o[34] := 13.2781565976477 + o[33];
              o[35] := o[34] * o[7];
              o[36] := (-6.9146446840086) + o[35];
              o[37] := o[2] * o[36];
              o[38] := (-2.53086309554280) + o[37];
              o[39] := o[38] * f.tau;
              o[40] := o[18] * o[5] * f.tau;
              f.f := (-15.7328452902390) + f.tau * (20.9443969743070 + ((-7.6867707878716) + o[3] * (2.61859477879540 + o[4] * ((-2.80807811486200) + o[1] * (1.20533696965170 - 0.0084566812812502 * o[6])))) * f.tau) + f.delta * (o[14] + f.delta * (0.38493460186671 + o[1] * ((-0.85214708824206) + o[2] * (4.8972281541877 + ((-3.05026172569650) + o[15] * (0.039420536879154 + 0.125584084243080 * o[2])) * f.tau)) + f.delta * ((-0.279993296987100) + o[1] * (1.38997995694600 + o[1] * ((-2.01899150235700) + o[16] * ((-0.0082147637173963) - 0.47596035734923 * o[17]))) + f.delta * (0.043984074473500 + o[1] * ((-0.44476435428739) + o[1] * (0.90572070719733 + 0.70522450087967 * o[19])) + f.delta * (f.delta * ((-0.0221754008730960) + o[1] * (0.094260751665092 + 0.164362784479610 * o[21]) + f.delta * ((-0.0135033722413480 * o[1]) + f.delta * ((-0.0148343453524720 * o[22]) + f.delta * (o[1] * (0.00057922953628084 + 0.0032308904703711 * o[21]) + f.delta * (0.000080964802996215 - 0.000044923899061815 * f.delta * o[22] - 0.000165576797950370 * f.tau))))) + (0.107705126263320 + o[1] * ((-0.32913623258954) - 0.50871062041158 * o[20])) * f.tau))))) + 1.06580700285130 * Modelica.Math.log(f.delta);
              f.fdelta := (1.06580700285130 + f.delta * (o[14] + f.delta * (0.76986920373342 + o[31] + f.delta * ((-0.83997989096130) + o[1] * (4.1699398708380 + o[1] * ((-6.0569745070710) + o[16] * ((-0.0246442911521889) - 1.42788107204769 * o[17]))) + f.delta * (0.175936297894000 + o[1] * ((-1.77905741714956) + o[1] * (3.6228828287893 + 2.82089800351868 * o[19])) + f.delta * (f.delta * ((-0.133052405238576) + o[1] * (0.56556450999055 + 0.98617670687766 * o[21]) + f.delta * ((-0.094523605689436 * o[1]) + f.delta * ((-0.118674762819776 * o[22]) + f.delta * (o[1] * (0.0052130658265276 + 0.0290780142333399 * o[21]) + f.delta * (0.00080964802996215 - 0.00049416288967996 * f.delta * o[22] - 0.00165576797950370 * f.tau))))) + (0.53852563131660 + o[1] * ((-1.64568116294770) - 2.54355310205790 * o[20])) * f.tau)))))) / f.delta;
              f.fdeltadelta := ((-1.06580700285130) + o[32] * (0.76986920373342 + o[31] + f.delta * ((-1.67995978192260) + o[1] * (8.3398797416760 + o[1] * ((-12.1139490141420) + o[16] * ((-0.049288582304378) - 2.85576214409538 * o[17]))) + f.delta * (0.52780889368200 + o[1] * ((-5.3371722514487) + o[1] * (10.8686484863680 + 8.4626940105560 * o[19])) + f.delta * (f.delta * ((-0.66526202619288) + o[1] * (2.82782254995276 + 4.9308835343883 * o[21]) + f.delta * ((-0.56714163413662 * o[1]) + f.delta * ((-0.83072333973843 * o[22]) + f.delta * (o[1] * (0.041704526612220 + 0.232624113866719 * o[21]) + f.delta * (0.0072868322696594 - 0.0049416288967996 * f.delta * o[22] - 0.0149019118155333 * f.tau))))) + (2.15410252526640 + o[1] * ((-6.5827246517908) - 10.1742124082316 * o[20])) * f.tau))))) / o[32];
              f.ftau := 20.9443969743070 + ((-15.3735415757432) + o[3] * (18.3301634515678 + o[4] * ((-28.0807811486200) + o[1] * (14.4640436358204 - 0.194503669468755 * o[6])))) * f.tau + f.delta * (o[39] + f.delta * (f.tau * ((-1.70429417648412) + o[2] * (29.3833689251262 + ((-21.3518320798755) + o[15] * (0.86725181134139 + 3.2651861903201 * o[2])) * f.tau)) + f.delta * ((2.77995991389200 + o[1] * ((-8.0759660094280) + o[16] * ((-0.131436219478341) - 12.3749692910800 * o[17]))) * f.tau + f.delta * (((-0.88952870857478) + o[1] * (3.6228828287893 + 18.3358370228714 * o[19])) * f.tau + f.delta * (0.107705126263320 + o[1] * ((-0.98740869776862) - 13.2264761307011 * o[20]) + f.delta * ((0.188521503330184 + 4.2734323964699 * o[21]) * f.tau + f.delta * ((-0.0270067444826960 * f.tau) + f.delta * ((-0.38569297916427 * o[40]) + f.delta * (f.delta * ((-0.000165576797950370) - 0.00116802137560719 * f.delta * o[40]) + (0.00115845907256168 + 0.084003152229649 * o[21]) * f.tau)))))))));
              f.ftautau := (-15.3735415757432) + o[3] * (109.980980709407 + o[4] * ((-252.727030337580) + o[1] * (159.104479994024 - 4.2790807283126 * o[6]))) + f.delta * ((-2.53086309554280) + o[2] * ((-34.573223420043) + (185.894192367068 - 174.645121293971 * o[1]) * o[7]) + f.delta * ((-1.70429417648412) + o[2] * (146.916844625631 + ((-128.110992479253) + o[15] * (18.2122880381691 + 81.629654758002 * o[2])) * f.tau) + f.delta * (2.77995991389200 + o[1] * ((-24.2278980282840) + o[16] * ((-1.97154329217511) - 309.374232277000 * o[17])) + f.delta * ((-0.88952870857478) + o[1] * (10.8686484863680 + 458.39592557179 * o[19]) + f.delta * (f.delta * (0.188521503330184 + 106.835809911747 * o[21] + f.delta * ((-0.0270067444826960) + f.delta * ((-9.6423244791068 * o[21]) + f.delta * (0.00115845907256168 + 2.10007880574121 * o[21] - 0.0292005343901797 * o[21] * o[32])))) + ((-1.97481739553724) - 330.66190326753 * o[20]) * f.tau)))));
              f.fdeltatau := o[39] + f.delta * (f.tau * ((-3.4085883529682) + o[2] * (58.766737850252 + ((-42.703664159751) + o[15] * (1.73450362268278 + 6.5303723806402 * o[2])) * f.tau)) + f.delta * ((8.3398797416760 + o[1] * ((-24.2278980282840) + o[16] * ((-0.39430865843502) - 37.124907873240 * o[17]))) * f.tau + f.delta * (((-3.5581148342991) + o[1] * (14.4915313151573 + 73.343348091486 * o[19])) * f.tau + f.delta * (0.53852563131660 + o[1] * ((-4.9370434888431) - 66.132380653505 * o[20]) + f.delta * ((1.13112901998110 + 25.6405943788192 * o[21]) * f.tau + f.delta * ((-0.189047211378872 * f.tau) + f.delta * ((-3.08554383331418 * o[40]) + f.delta * (f.delta * ((-0.00165576797950370) - 0.0128482351316791 * f.delta * o[40]) + (0.0104261316530551 + 0.75602837006684 * o[21]) * f.tau))))))));
            end f3;

            function g5 "Base function for region 5: g(p,T)"
              extends Modelica.Icons.Function;
              input SI.Pressure p "Pressure";
              input SI.Temperature T "Temperature (K)";
              output Modelica.Media.Common.GibbsDerivs g "Dimensionless Gibbs function and derivatives w.r.t. pi and tau";
            protected
              Real[11] o "Vector of auxiliary variables";
            algorithm
              assert(p > 0.0, "IF97 medium function g5 called with too low pressure\n" + "p = " + String(p) + " Pa <=  0.0 Pa");
              assert(p <= data.PLIMIT5, "IF97 medium function g5: input pressure (= " + String(p) + " Pa) is higher than 10 Mpa in region 5");
              assert(T <= 2273.15, "IF97 medium function g5: input temperature (= " + String(T) + " K) is higher than limit of 2273.15K in region 5");
              g.p := p;
              g.T := T;
              g.R := data.RH2O;
              g.pi := p / data.PSTAR5;
              g.tau := data.TSTAR5 / T;
              o[1] := g.tau * g.tau;
              o[2] := -0.0045942820899910 * o[1];
              o[3] := 0.00217746787145710 + o[2];
              o[4] := o[3] * g.tau;
              o[5] := o[1] * g.tau;
              o[6] := o[1] * o[1];
              o[7] := o[6] * o[6];
              o[8] := o[7] * g.tau;
              o[9] := -7.9449656719138e-6 * o[8];
              o[10] := g.pi * g.pi;
              o[11] := -0.0137828462699730 * o[1];
              g.g := g.pi * ((-0.000125631835895920) + o[4] + g.pi * ((-3.9724828359569e-6 * o[8]) + 1.29192282897840e-7 * o[5] * g.pi)) + ((-0.0248051489334660) + g.tau * (0.36901534980333 + g.tau * ((-3.11613182139250) + g.tau * ((-13.1799836742010) + (6.8540841634434 - 0.32961626538917 * g.tau) * g.tau + Modelica.Math.log(g.pi))))) / o[5];
              g.gpi := (1.0 + g.pi * ((-0.000125631835895920) + o[4] + g.pi * (o[9] + 3.8757684869352e-7 * o[5] * g.pi))) / g.pi;
              g.gpipi := ((-1.00000000000000) + o[10] * (o[9] + 7.7515369738704e-7 * o[5] * g.pi)) / o[10];
              g.gtau := g.pi * (0.00217746787145710 + o[11] + g.pi * ((-0.000035752345523612 * o[7]) + 3.8757684869352e-7 * o[1] * g.pi)) + (0.074415446800398 + g.tau * ((-0.73803069960666) + (3.11613182139250 + o[1] * (6.8540841634434 - 0.65923253077834 * g.tau)) * g.tau)) / o[6];
              g.gtautau := ((-0.297661787201592) + g.tau * (2.21409209881998 + ((-6.2322636427850) - 0.65923253077834 * o[5]) * g.tau)) / (o[6] * g.tau) + g.pi * ((-0.0275656925399460 * g.tau) + g.pi * ((-0.000286018764188897 * o[1] * o[6] * g.tau) + 7.7515369738704e-7 * g.pi * g.tau));
              g.gtaupi := 0.00217746787145710 + o[11] + g.pi * ((-0.000071504691047224 * o[7]) + 1.16273054608056e-6 * o[1] * g.pi);
            end g5;

            function gibbs "Gibbs function for region 1, 2 or 5: g(p,T,region)"
              extends Modelica.Icons.Function;
              input SI.Pressure p "Pressure";
              input SI.Temperature T "Temperature (K)";
              input Integer region "IF97 region, 1, 2 or 5";
              output Real g "Dimensionless Gibbs function";
            protected
              Modelica.Media.Common.GibbsDerivs gibbs "Dimensionless Gibbs function and derivatives w.r.t. pi and tau";
            algorithm
              assert(region == 1 or region == 2 or region == 5, "IF97 medium function gibbs called with wrong region (= " + String(region) + ").\n" + "Only regions 1, 2 or 5 are possible");
              if region == 1 then
                gibbs := g1(p, T);
              elseif region == 2 then
                gibbs := g2(p, T);
              else
                gibbs := g5(p, T);
              end if;
              g := gibbs.g;
            end gibbs;

            function g1pitau "Derivative of g w.r.t. pi and tau"
              extends Modelica.Icons.Function;
              input SI.Pressure p "Pressure";
              input SI.Temperature T "Temperature (K)";
              output Real pi "Dimensionless pressure";
              output Real tau "Dimensionless temperature";
              output Real gpi "Dimensionless derivative of Gibbs function w.r.t. pi";
              output Real gtau "Dimensionless derivative of Gibbs function w.r.t. tau";
            protected
              Real pi1 "Dimensionless pressure";
              Real tau1 "Dimensionless temperature";
              Real[28] o "Vector of auxiliary variables";
            algorithm
              assert(p > triple.ptriple, "IF97 medium function g1pitau called with too low pressure\n" + "p = " + String(p) + " Pa <= " + String(triple.ptriple) + " Pa (triple point pressure)");
              assert(p <= 100.0e6, "IF97 medium function g1pitau: the input pressure (= " + String(p) + " Pa) is higher than 100 Mpa");
              assert(T >= 273.15, "IF97 medium function g1pitau: the temperature (= " + String(T) + " K) is lower than 273.15 K!");
              pi := p / data.PSTAR1;
              tau := data.TSTAR1 / T;
              pi1 := 7.1 - pi;
              tau1 := (-1.222) + tau;
              o[1] := tau1 * tau1;
              o[2] := o[1] * tau1;
              o[3] := 1 / o[2];
              o[4] := o[1] * o[1];
              o[5] := o[4] * o[4];
              o[6] := o[1] * o[5];
              o[7] := o[1] * o[4];
              o[8] := 1 / o[4];
              o[9] := o[1] * o[4] * o[5];
              o[10] := o[4] * tau1;
              o[11] := 1 / o[10];
              o[12] := o[4] * o[5];
              o[13] := o[5] * tau1;
              o[14] := 1 / o[13];
              o[15] := pi1 * pi1;
              o[16] := o[15] * pi1;
              o[17] := o[15] * o[15];
              o[18] := o[17] * o[17];
              o[19] := o[17] * o[18] * pi1;
              o[20] := o[15] * o[17];
              o[21] := o[5] * o[5];
              o[22] := o[21] * o[21];
              o[23] := o[22] * o[5] * tau1;
              o[24] := 1 / o[23];
              o[25] := o[22] * o[5];
              o[26] := 1 / o[25];
              o[27] := o[1] * o[22] * o[4] * tau1;
              o[28] := 1 / o[27];
              gtau := pi1 * (((-0.00254871721114236) + o[1] * (0.00424944110961118 + (0.018990068218419 + ((-0.021841717175414) - 0.00015851507390979 * o[1]) * o[1]) * o[7])) / o[6] + pi1 * (o[8] * (0.00141552963219801 + o[4] * (0.000047661393906987 + o[1] * ((-0.0000132425535992538) - 1.2358149370591e-14 * o[9]))) + pi1 * (o[11] * (0.000126718579380216 - 5.11230768720618e-9 * o[6]) + pi1 * ((0.000011212640954 + (1.30342445791202e-6 - 1.4341729937924e-12 * o[12]) * o[2]) / o[7] + pi1 * (3.24135974880936e-6 * o[14] + o[16] * ((1.40077319158051e-8 + 1.04549227383804e-9 * o[10]) / o[12] + o[19] * (1.9941018075704e-17 / (o[1] * o[21] * o[4] * o[5]) + o[15] * ((-4.48827542684151e-19 / o[22]) + o[20] * ((-1.00075970318621e-21 * o[28]) + pi1 * (4.65957282962769e-22 * o[26] + pi1 * ((-7.2912378325616e-23 * o[24]) + 3.83502057899078e-24 * pi1 / (o[1] * o[22] * o[5])))))))))))) + o[3] * ((-0.29265942426334) + tau1 * (0.84548187169114 + o[1] * (3.3855169168385 + tau1 * ((-1.91583926775744) + tau1 * (0.47316115539684 + ((-0.066465668798004) + 0.0040607314991784 * tau1) * tau1)))));
              gpi := pi1 * (pi1 * ((0.000095038934535162 + o[4] * (8.4812393955936e-6 + 2.55615384360309e-9 * o[7])) * o[8] + pi1 * (o[11] * (8.9701127632e-6 + (2.60684891582404e-6 + 5.7366919751696e-13 * o[12]) * o[2]) + pi1 * (2.02584984300585e-6 / o[5] + o[16] * (o[19] * (o[15] * (o[20] * ((-7.63737668221055e-22 / (o[1] * o[22] * o[4])) + pi1 * (3.5842867920213e-22 * o[28] + pi1 * ((-5.65070932023524e-23 * o[26]) + 2.99318679335866e-24 * o[24] * pi1))) - 3.33001080055983e-19 / (o[1] * o[21] * o[4] * o[5] * tau1)) + 1.44400475720615e-17 / (o[21] * o[4] * o[5] * tau1)) + (1.01874413933128e-8 + 1.39398969845072e-9 * o[10]) / (o[1] * o[5] * tau1))))) + o[3] * (0.00094368642146534 + o[2] * (0.00060003561586052 + ((-0.000095322787813974) + o[1] * (8.8283690661692e-6 + 1.45389992595188e-15 * o[9])) * tau1))) + o[14] * ((-0.00028319080123804) + o[1] * (0.00060706301565874 + o[7] * (0.018990068218419 + tau1 * (0.032529748770505 + (0.021841717175414 + 0.00005283835796993 * o[1]) * tau1))));
            end g1pitau;

            function g2pitau "Derivative of g w.r.t. pi and tau"
              extends Modelica.Icons.Function;
              input SI.Pressure p "Pressure";
              input SI.Temperature T "Temperature (K)";
              output Real pi "Dimensionless pressure";
              output Real tau "Dimensionless temperature";
              output Real gpi "Dimensionless derivative of Gibbs function w.r.t. pi";
              output Real gtau "Dimensionless derivative of Gibbs function w.r.t. tau";
            protected
              Real tau2 "Dimensionless temperature";
              Real[22] o "Vector of auxiliary variables";
            algorithm
              assert(p > 0.0, "IF97 medium function g2pitau called with too low pressure\n" + "p = " + String(p) + " Pa <= " + String(triple.ptriple) + " Pa (triple point pressure)");
              assert(p <= 100.0e6, "IF97 medium function g2pitau: the input pressure (= " + String(p) + " Pa) is higher than 100 Mpa");
              assert(T >= 273.15, "IF97 medium function g2pitau: the temperature (= " + String(T) + " K) is lower than 273.15 K!");
              assert(T <= 1073.15, "IF97 medium function g2pitau: the input temperature (= " + String(T) + " K) is higher than the limit of 1073.15 K");
              pi := p / data.PSTAR2;
              tau := data.TSTAR2 / T;
              tau2 := (-0.5) + tau;
              o[1] := tau * tau;
              o[2] := o[1] * o[1];
              o[3] := tau2 * tau2;
              o[4] := o[3] * tau2;
              o[5] := o[3] * o[3];
              o[6] := o[5] * o[5];
              o[7] := o[6] * o[6];
              o[8] := o[5] * o[6] * o[7] * tau2;
              o[9] := o[3] * o[5];
              o[10] := o[5] * o[6] * tau2;
              o[11] := o[3] * o[7] * tau2;
              o[12] := o[3] * o[5] * o[6];
              o[13] := o[3] * o[5] * tau2;
              o[14] := o[5] * o[6] * o[7];
              o[15] := pi * pi;
              o[16] := o[15] * o[15];
              o[17] := o[15] * o[16];
              o[18] := o[5] * o[7] * tau2;
              o[19] := o[7] * o[7];
              o[20] := o[3] * o[5] * o[6] * tau2;
              o[21] := o[5] * o[7];
              o[22] := o[3] * o[7];
              gtau := (0.0280439559151 + tau * ((-0.2858109552582) + tau * (1.2213149471784 + tau * ((-2.848163942888) + tau * (4.38395111945 + o[1] * (10.08665568018 + ((-0.5681726521544) + 0.06380539059921 * tau) * tau)))))) / (o[1] * o[2]) + pi * ((-0.017834862292358) + tau2 * ((-0.09199202739273) + ((-0.172743777250296) - 0.30195167236758 * o[4]) * tau2) + pi * ((-0.000033032641670203) + ((-0.0003789797503263) + o[3] * ((-0.015757110897342) + o[4] * ((-0.306581069554011) - 0.000960283724907132 * o[8]))) * tau2 + pi * (4.3870667284435e-7 + o[3] * ((-0.00009683303171571) + o[4] * ((-0.0090203547252888) - 1.42338887469272 * o[8])) + pi * ((-7.8847309559367e-10) + (2.558143570457e-8 + 1.44676118155521e-6 * tau2) * tau2 + pi * (0.0000160454534363627 * o[9] + pi * (((-5.0144299353183e-11) + o[10] * ((-0.033874355714168) - 836.35096769364 * o[11])) * o[3] + pi * (((-0.0000138839897890111) - 0.973671060893475 * o[12]) * o[3] * o[6] + pi * (o[13] * (9.0049690883672e-11 - 296.320827232793 * o[14]) + pi * (2.57526266427144e-7 * o[5] * o[6] + pi * (o[4] * (4.1627860840696e-19 + ((-1.0234747095929e-12) - 1.40254511313154e-8 * o[5]) * o[9]) + o[17] * (o[14] * ((-2.34560435076256e-9) + 5.3465159397045 * o[18]) + o[15] * ((-19.1874828272775 * o[19] * o[6] * o[7]) + o[15] * (o[11] * (1.78371690710842e-23 + (1.07202609066812e-11 - 0.000201611844951398 * o[10]) * o[20]) + pi * ((-1.24017662339842e-24 * o[21]) + pi * (0.000200482822351322 * o[19] * o[5] * o[7] + pi * ((-4.97975748452559e-14 * o[19] * o[3] * o[5]) + (1.90027787547159e-27 + o[12] * (2.21658861403112e-15 - 0.0000547344301999018 * o[22])) * o[6] * o[7] * pi * tau2))))))))))))))));
              gpi := (1. + pi * ((-0.0017731742473213) + tau2 * ((-0.017834862292358) + tau2 * ((-0.045996013696365) + ((-0.057581259083432) - 0.05032527872793 * o[4]) * tau2)) + pi * (tau2 * ((-0.000066065283340406) + ((-0.0003789797503263) + o[3] * ((-0.007878555448671) + o[4] * ((-0.087594591301146) - 0.000053349095828174 * o[8]))) * tau2) + pi * (6.1445213076927e-8 + (1.31612001853305e-6 + o[3] * ((-0.00009683303171571) + o[4] * ((-0.0045101773626444) - 0.122004760687947 * o[8]))) * tau2 + pi * (tau2 * ((-3.15389238237468e-9) + (5.116287140914e-8 + 1.92901490874028e-6 * tau2) * tau2) + pi * (0.0000114610381688305 * o[13] + pi * (((-1.00288598706366e-10) + o[10] * ((-0.012702883392813) - 143.374451604624 * o[11])) * o[4] + pi * ((-4.1341695026989e-17) + ((-8.8352662293707e-6) - 0.272627897050173 * o[12]) * o[3] * o[6] * tau2 + pi * ((9.0049690883672e-11 - 65.8490727183984 * o[14]) * o[6] + pi * (1.78287415218792e-7 * o[10] + pi * (o[5] * (1.0406965210174e-18 + ((-1.0234747095929e-12) - 1.0018179379511e-8 * o[5]) * o[9]) + o[17] * (((-1.29412653835176e-9) + 1.71088510070544 * o[18]) * o[8] + o[15] * ((-6.05920510335078 * o[19] * o[6] * o[7] * tau2) + o[15] * ((1.78371690710842e-23 + (6.1258633752464e-12 - 0.000084004935396416 * o[10]) * o[20]) * o[21] + pi * ((-1.24017662339842e-24 * o[18]) + pi * (0.0000832192847496054 * o[19] * o[5] * o[7] * tau2 + pi * ((1.75410265428146e-27 + o[12] * (1.32995316841867e-15 - 0.0000226487297378904 * o[22])) * o[3] * o[6] * o[7] * pi - 2.93678005497663e-14 * o[19] * o[3] * o[5] * tau2))))))))))))))))) / pi;
            end g2pitau;

            function g5pitau "Derivative of g w.r.t. pi and tau"
              extends Modelica.Icons.Function;
              input SI.Pressure p "Pressure";
              input SI.Temperature T "Temperature (K)";
              output Real pi "Dimensionless pressure";
              output Real tau "Dimensionless temperature";
              output Real gpi "Dimensionless derivative of Gibbs function w.r.t. pi";
              output Real gtau "Dimensionless derivative of Gibbs function w.r.t. tau";
            protected
              Real[3] o "Vector of auxiliary variables";
            algorithm
              assert(p > 0.0, "IF97 medium function g5pitau called with too low pressure\n" + "p = " + String(p) + " Pa <= " + String(triple.ptriple) + " Pa (triple point pressure)");
              assert(p <= data.PLIMIT5, "IF97 medium function g5pitau: input pressure (= " + String(p) + " Pa) is higher than 10 Mpa in region 5");
              assert(T <= 2273.15, "IF97 medium function g5pitau: input temperature (= " + String(T) + " K) is higher than limit of 2273.15 K in region 5");
              pi := p / data.PSTAR5;
              tau := data.TSTAR5 / T;
              o[1] := tau * tau;
              o[2] := o[1] * o[1];
              o[3] := o[2] * o[2];
              gtau := pi * (0.0021774678714571 - 0.013782846269973 * o[1] + pi * ((-0.0000357523455236121 * o[3]) + 3.8757684869352e-7 * o[1] * pi)) + (0.074415446800398 + tau * ((-0.73803069960666) + (3.1161318213925 + o[1] * (6.8540841634434 - 0.65923253077834 * tau)) * tau)) / o[2];
              gpi := (1.0 + pi * ((-0.00012563183589592) + (0.0021774678714571 - 0.004594282089991 * o[1]) * tau + pi * ((-7.9449656719138e-6 * o[3] * tau) + 3.8757684869352e-7 * o[1] * pi * tau))) / pi;
            end g5pitau;

            function f3deltatau "1st derivatives of f w.r.t. delta and tau"
              extends Modelica.Icons.Function;
              input SI.Density d "Density";
              input SI.Temperature T "Temperature (K)";
              output Real delta "Dimensionless density";
              output Real tau "Dimensionless temperature";
              output Real fdelta "Dimensionless derivative of Helmholtz function w.r.t. delta";
              output Real ftau "Dimensionless derivative of Helmholtz function w.r.t. tau";
            protected
              Real[13] o "Vector of auxiliary variables";
            algorithm
              tau := data.TCRIT / T;
              delta := if d == data.DCRIT and T == data.TCRIT then 1 + Modelica.Constants.eps else d / data.DCRIT;
              o[1] := tau * tau;
              o[2] := o[1] * o[1];
              o[3] := o[2] * o[2];
              o[4] := o[3] * tau;
              o[5] := o[1] * o[2] * o[3] * tau;
              o[6] := o[2] * o[3];
              o[7] := o[1] * o[3];
              o[8] := o[3] * o[3];
              o[9] := o[1] * o[2] * o[8];
              o[10] := o[1] * o[2] * o[8] * tau;
              o[11] := o[3] * o[8];
              o[12] := o[1] * o[3] * o[8];
              o[13] := o[3] * o[8] * tau;
              fdelta := (1.0658070028513 + delta * (o[1] * ((-1.2654315477714) + o[2] * ((-1.1524407806681) + (0.88521043984318 - 0.64207765181607 * o[1]) * o[4])) + delta * (0.76986920373342 + o[1] * ((-1.70429417648412) + o[2] * (9.7944563083754 + ((-6.100523451393) + (0.078841073758308 + 0.25116816848616 * o[2]) * o[5]) * tau)) + delta * ((-0.8399798909613) + o[1] * (4.169939870838 + o[1] * ((-6.056974507071) + o[6] * ((-0.0246442911521889) - 1.42788107204769 * o[7]))) + delta * (0.175936297894 + o[1] * ((-1.77905741714956) + o[1] * (3.62288282878932 + 2.82089800351868 * o[9])) + delta * (delta * ((-0.133052405238576) + o[1] * (0.565564509990552 + 0.98617670687766 * o[11]) + delta * ((-0.094523605689436 * o[1]) + delta * ((-0.118674762819776 * o[12]) + delta * (o[1] * (0.00521306582652756 + 0.0290780142333399 * o[11]) + delta * (0.00080964802996215 - 0.000494162889679965 * delta * o[12] - 0.0016557679795037 * tau))))) + (0.5385256313166 + o[1] * ((-1.6456811629477) - 2.5435531020579 * o[10])) * tau)))))) / delta;
              ftau := 20.944396974307 + tau * ((-15.3735415757432) + o[2] * tau * (18.3301634515678 + o[1] * tau * ((-28.08078114862) + o[1] * (14.4640436358204 - 0.194503669468755 * o[1] * o[3] * tau)))) + delta * (((-2.5308630955428) + o[2] * ((-6.9146446840086) + (13.2781565976477 - 10.9153200808732 * o[1]) * o[4])) * tau + delta * (tau * ((-1.70429417648412) + o[2] * (29.3833689251262 + ((-21.3518320798755) + (0.867251811341388 + 3.26518619032008 * o[2]) * o[5]) * tau)) + delta * ((2.779959913892 + o[1] * ((-8.075966009428) + o[6] * ((-0.131436219478341) - 12.37496929108 * o[7]))) * tau + delta * (((-0.88952870857478) + o[1] * (3.62288282878932 + 18.3358370228714 * o[9])) * tau + delta * (0.10770512626332 + o[1] * ((-0.98740869776862) - 13.2264761307011 * o[10]) + delta * ((0.188521503330184 + 4.27343239646986 * o[11]) * tau + delta * ((-0.027006744482696 * tau) + delta * ((-0.385692979164272 * o[13]) + delta * (delta * ((-0.00016557679795037) - 0.00116802137560719 * delta * o[13]) + (0.00115845907256168 + 0.0840031522296486 * o[11]) * tau)))))))));
            end f3deltatau;

            function tph1 "Inverse function for region 1: T(p,h)"
              extends Modelica.Icons.Function;
              input SI.Pressure p "Pressure";
              input SI.SpecificEnthalpy h "Specific enthalpy";
              output SI.Temperature T "Temperature (K)";
            protected
              Real pi "Dimensionless pressure";
              Real eta1 "Dimensionless specific enthalpy";
              Real[3] o "Vector of auxiliary variables";
            algorithm
              assert(p > triple.ptriple, "IF97 medium function tph1 called with too low pressure\n" + "p = " + String(p) + " Pa <= " + String(triple.ptriple) + " Pa (triple point pressure)");
              pi := p / data.PSTAR2;
              eta1 := h / data.HSTAR1 + 1.0;
              o[1] := eta1 * eta1;
              o[2] := o[1] * o[1];
              o[3] := o[2] * o[2];
              T := (-238.724899245210) - 13.3917448726020 * pi + eta1 * (404.21188637945 + 43.211039183559 * pi + eta1 * (113.497468817180 - 54.010067170506 * pi + eta1 * (30.5358922039160 * pi + eta1 * ((-6.5964749423638 * pi) + o[1] * ((-5.8457616048039) + o[2] * (pi * (0.0093965400878363 + ((-0.0000258586412820730) + 6.6456186191635e-8 * pi) * pi) + o[2] * o[3] * ((-0.000152854824131400) + o[1] * o[3] * ((-1.08667076953770e-6) + pi * (1.15736475053400e-7 + pi * ((-4.0644363084799e-9) + pi * (8.0670734103027e-11 + pi * ((-9.3477771213947e-13) + (5.8265442020601e-15 - 1.50201859535030e-17 * pi) * pi))))))))))));
            end tph1;

            function tps1 "Inverse function for region 1: T(p,s)"
              extends Modelica.Icons.Function;
              input SI.Pressure p "Pressure";
              input SI.SpecificEntropy s "Specific entropy";
              output SI.Temperature T "Temperature (K)";
            protected
              constant SI.Pressure pstar = 1.0e6;
              constant SI.SpecificEntropy sstar = 1.0e3;
              Real pi "Dimensionless pressure";
              Real sigma1 "Dimensionless specific entropy";
              Real[6] o "Vector of auxiliary variables";
            algorithm
              pi := p / pstar;
              assert(p > triple.ptriple, "IF97 medium function tps1 called with too low pressure\n" + "p = " + String(p) + " Pa <= " + String(triple.ptriple) + " Pa (triple point pressure)");
              sigma1 := s / sstar + 2.0;
              o[1] := sigma1 * sigma1;
              o[2] := o[1] * o[1];
              o[3] := o[2] * o[2];
              o[4] := o[3] * o[3];
              o[5] := o[4] * o[4];
              o[6] := o[1] * o[2] * o[4];
              T := 174.782680583070 + sigma1 * (34.806930892873 + sigma1 * (6.5292584978455 + (0.33039981775489 + o[3] * ((-1.92813829231960e-7) - 2.49091972445730e-23 * o[2] * o[4])) * sigma1)) + pi * ((-0.261076364893320) + pi * (0.00056608900654837 + pi * (o[1] * o[3] * (2.64004413606890e-13 + 7.8124600459723e-29 * o[6]) - 3.07321999036680e-31 * o[5] * pi) + sigma1 * ((-0.00032635483139717) + sigma1 * (0.000044778286690632 + o[1] * o[2] * ((-5.1322156908507e-10) - 4.2522657042207e-26 * o[6]) * sigma1))) + sigma1 * (0.225929659815860 + sigma1 * ((-0.064256463395226) + sigma1 * (0.0078876289270526 + o[3] * sigma1 * (3.5672110607366e-10 + 1.73324969948950e-24 * o[1] * o[4] * sigma1)))));
            end tps1;

            function tph2 "Reverse function for region 2: T(p,h)"
              extends Modelica.Icons.Function;
              input SI.Pressure p "Pressure";
              input SI.SpecificEnthalpy h "Specific enthalpy";
              output SI.Temperature T "Temperature (K)";
            protected
              Real pi "Dimensionless pressure";
              Real pi2b "Dimensionless pressure";
              Real pi2c "Dimensionless pressure";
              Real eta "Dimensionless specific enthalpy";
              Real etabc "Dimensionless specific enthalpy";
              Real eta2a "Dimensionless specific enthalpy";
              Real eta2b "Dimensionless specific enthalpy";
              Real eta2c "Dimensionless specific enthalpy";
              Real[8] o "Vector of auxiliary variables";
            algorithm
              pi := p * data.IPSTAR;
              eta := h * data.IHSTAR;
              etabc := h * 1.0e-3;
              if pi < 4.0 then
                eta2a := eta - 2.1;
                o[1] := eta2a * eta2a;
                o[2] := o[1] * o[1];
                o[3] := pi * pi;
                o[4] := o[3] * o[3];
                o[5] := o[3] * pi;
                T := 1089.89523182880 + (1.84457493557900 - 0.0061707422868339 * pi) * pi + eta2a * (849.51654495535 - 4.1792700549624 * pi + eta2a * ((-107.817480918260) + (6.2478196935812 - 0.310780466295830 * pi) * pi + eta2a * (33.153654801263 - 17.3445631081140 * pi + o[2] * ((-7.4232016790248) + pi * ((-200.581768620960) + 11.6708730771070 * pi) + o[1] * (271.960654737960 * pi + o[1] * ((-455.11318285818 * pi) + eta2a * (1.38657242832260 * o[4] + o[1] * o[2] * (3091.96886047550 * pi + o[1] * (11.7650487243560 + o[2] * ((-13551.3342407750 * o[5]) + o[2] * ((-62.459855192507 * o[3] * o[4] * pi) + o[2] * (o[4] * (235988.325565140 + 7399.9835474766 * pi) + o[1] * (19127.7292396600 * o[3] * o[4] + o[1] * (o[3] * (1.28127984040460e8 - 551966.97030060 * o[5]) + o[1] * ((-9.8554909623276e8 * o[3]) + o[1] * (2.82245469730020e9 * o[3] + o[1] * (o[3] * ((-3.5948971410703e9) + 3.7154085996233e6 * o[5]) + o[1] * pi * (252266.403578720 + pi * (1.72273499131970e9 + pi * (1.28487346646500e7 + ((-1.31052365450540e7) - 415351.64835634 * o[3]) * pi))))))))))))))))))));
              elseif pi < (0.12809002730136e-03 * etabc - 0.67955786399241) * etabc + 0.90584278514723e3 then
                eta2b := eta - 2.6;
                pi2b := pi - 2.0;
                o[1] := pi2b * pi2b;
                o[2] := o[1] * pi2b;
                o[3] := o[1] * o[1];
                o[4] := eta2b * eta2b;
                o[5] := o[4] * o[4];
                o[6] := o[4] * o[5];
                o[7] := o[5] * o[5];
                T := 1489.50410795160 + 0.93747147377932 * pi2b + eta2b * (743.07798314034 + o[2] * (0.000110328317899990 - 1.75652339694070e-18 * o[1] * o[3]) + eta2b * ((-97.708318797837) + pi2b * (3.3593118604916 + pi2b * ((-0.0218107553247610) + pi2b * (0.000189552483879020 + (2.86402374774560e-7 - 8.1456365207833e-14 * o[2]) * pi2b))) + o[5] * (3.3809355601454 * pi2b + o[4] * ((-0.108297844036770 * o[1]) + o[5] * (2.47424647056740 + (0.168445396719040 + o[1] * (0.00308915411605370 - 0.0000107798573575120 * pi2b)) * pi2b + o[6] * ((-0.63281320016026) + pi2b * (0.73875745236695 + ((-0.046333324635812) + o[1] * ((-0.000076462712454814) + 2.82172816350400e-7 * pi2b)) * pi2b) + o[6] * (1.13859521296580 + pi2b * ((-0.47128737436186) + o[1] * (0.00135555045549490 + (0.0000140523928183160 + 1.27049022719450e-6 * pi2b) * pi2b)) + o[5] * ((-0.47811863648625) + (0.150202731397070 + o[2] * ((-0.0000310838143314340) + o[1] * ((-1.10301392389090e-8) - 2.51805456829620e-11 * pi2b))) * pi2b + o[5] * o[7] * (0.0085208123431544 + pi2b * ((-0.00217641142197500) + pi2b * (0.000071280351959551 + o[1] * ((-1.03027382121030e-6) + (7.3803353468292e-8 + 8.6934156344163e-15 * o[3]) * pi2b))))))))))));
              else
                eta2c := eta - 1.8;
                pi2c := pi + 25.0;
                o[1] := pi2c * pi2c;
                o[2] := o[1] * o[1];
                o[3] := o[1] * o[2] * pi2c;
                o[4] := 1 / o[3];
                o[5] := o[1] * o[2];
                o[6] := eta2c * eta2c;
                o[7] := o[2] * o[2];
                o[8] := o[6] * o[6];
                T := eta2c * ((859777.22535580 + o[1] * (482.19755109255 + 1.12615974072300e-12 * o[5])) / o[1] + eta2c * (((-5.8340131851590e11) + (2.08255445631710e10 + 31081.0884227140 * o[2]) * pi2c) / o[5] + o[6] * (o[8] * (o[6] * (1.23245796908320e-7 * o[5] + o[6] * ((-1.16069211309840e-6 * o[5]) + o[8] * (0.0000278463670885540 * o[5] + ((-0.00059270038474176 * o[5]) + 0.00129185829918780 * o[5] * o[6]) * o[8]))) - 10.8429848800770 * pi2c) + o[4] * (7.3263350902181e12 + o[7] * (3.7966001272486 + ((-0.045364172676660) - 1.78049822406860e-11 * o[2]) * pi2c))))) + o[4] * ((-3.2368398555242e12) + pi2c * (3.5825089945447e11 + pi2c * ((-1.07830682174700e10) + o[1] * pi2c * (610747.83564516 + pi2c * ((-25745.7236041700) + (1208.23158659360 + 1.45591156586980e-13 * o[5]) * pi2c)))));
              end if;
            end tph2;

            function tps2a "Reverse function for region 2a: T(p,s)"
              extends Modelica.Icons.Function;
              input SI.Pressure p "Pressure";
              input SI.SpecificEntropy s "Specific entropy";
              output SI.Temperature T "Temperature (K)";
            protected
              Real[12] o "Vector of auxiliary variables";
              constant Real IPSTAR = 1.0e-6 "Scaling variable";
              constant Real ISSTAR2A = 1 / 2000.0 "Scaling variable";
              Real pi "Dimensionless pressure";
              Real sigma2a "Dimensionless specific entropy";
            algorithm
              pi := p * IPSTAR;
              sigma2a := s * ISSTAR2A - 2.0;
              o[1] := pi ^ 0.5;
              o[2] := sigma2a * sigma2a;
              o[3] := o[2] * o[2];
              o[4] := o[3] * o[3];
              o[5] := o[4] * o[4];
              o[6] := pi ^ 0.25;
              o[7] := o[2] * o[4] * o[5];
              o[8] := 1 / o[7];
              o[9] := o[3] * sigma2a;
              o[10] := o[2] * o[3] * sigma2a;
              o[11] := o[3] * o[4] * sigma2a;
              o[12] := o[2] * sigma2a;
              T := (((-392359.83861984) + (515265.73827270 + o[3] * (40482.443161048 + o[2] * o[3] * ((-321.93790923902) + o[2] * (96.961424218694 - 22.8678463717730 * sigma2a)))) * sigma2a) / (o[4] * o[5]) + o[6] * (((-449429.14124357) + o[3] * ((-5011.8336020166) + 0.35684463560015 * o[4] * sigma2a)) / (o[2] * o[5] * sigma2a) + o[6] * (o[8] * (44235.335848190 + o[9] * ((-13673.3888117080) + o[3] * (421632.60207864 + (22516.9258374750 + o[10] * (474.42144865646 - 149.311307976470 * sigma2a)) * sigma2a))) + o[6] * (((-197811.263204520) - 23554.3994707600 * sigma2a) / (o[2] * o[3] * o[4] * sigma2a) + o[6] * (((-19070.6163020760) + o[11] * (55375.669883164 + (3829.3691437363 - 603.91860580567 * o[2]) * o[3])) * o[8] + o[6] * ((1936.31026203310 + o[2] * (4266.0643698610 + o[2] * o[3] * o[4] * ((-5978.0638872718) - 704.01463926862 * o[9]))) / (o[2] * o[4] * o[5] * sigma2a) + o[1] * ((338.36784107553 + o[12] * (20.8627866351870 + (0.033834172656196 - 0.000043124428414893 * o[12]) * o[3])) * sigma2a + o[6] * (166.537913564120 + sigma2a * ((-139.862920558980) + o[3] * ((-0.78849547999872) + (0.072132411753872 + o[3] * ((-0.0059754839398283) + ((-0.0000121413589539040) + 2.32270967338710e-7 * o[2]) * o[3])) * sigma2a)) + o[6] * ((-10.5384635661940) + o[3] * (2.07189254965020 + ((-0.072193155260427) + 2.07498870811200e-7 * o[4]) * o[9]) + o[6] * (o[6] * (o[12] * (0.210375278936190 + 0.000256812397299990 * o[3] * o[4]) + ((-0.0127990029337810) - 8.2198102652018e-6 * o[11]) * o[6] * o[9]) + o[10] * ((-0.0183406579113790) + 2.90362723486960e-7 * o[2] * o[4] * sigma2a))))))))))) / (o[1] * pi);
            end tps2a;

            function tps2b "Reverse function for region 2b: T(p,s)"
              extends Modelica.Icons.Function;
              input SI.Pressure p "Pressure";
              input SI.SpecificEntropy s "Specific entropy";
              output SI.Temperature T "Temperature (K)";
            protected
              Real[8] o "Vector of auxiliary variables";
              constant Real IPSTAR = 1.0e-6 "Scaling variable";
              constant Real ISSTAR2B = 1 / 785.3 "Scaling variable";
              Real pi "Dimensionless pressure";
              Real sigma2b "Dimensionless specific entropy";
            algorithm
              pi := p * IPSTAR;
              sigma2b := 10.0 - s * ISSTAR2B;
              o[1] := pi * pi;
              o[2] := o[1] * o[1];
              o[3] := sigma2b * sigma2b;
              o[4] := o[3] * o[3];
              o[5] := o[4] * o[4];
              o[6] := o[3] * o[5] * sigma2b;
              o[7] := o[3] * o[5];
              o[8] := o[3] * sigma2b;
              T := (316876.65083497 + 20.8641758818580 * o[6] + pi * ((-398593.99803599) - 21.8160585188770 * o[6] + pi * (223697.851942420 + ((-2784.17034458170) + 9.9207436071480 * o[7]) * sigma2b + pi * ((-75197.512299157) + (2970.86059511580 + o[7] * ((-3.4406878548526) + 0.38815564249115 * sigma2b)) * sigma2b + pi * (17511.2950857500 + sigma2b * ((-1423.71128544490) + (1.09438033641670 + 0.89971619308495 * o[4]) * o[4] * sigma2b) + pi * ((-3375.9740098958) + (471.62885818355 + o[4] * ((-1.91882419936790) + o[8] * (0.41078580492196 - 0.33465378172097 * sigma2b))) * sigma2b + pi * (1387.00347775050 + sigma2b * ((-406.63326195838) + sigma2b * (41.727347159610 + o[3] * (2.19325494345320 + sigma2b * ((-1.03200500090770) + (0.35882943516703 + 0.0052511453726066 * o[8]) * sigma2b)))) + pi * (12.8389164507050 + sigma2b * ((-2.86424372193810) + sigma2b * (0.56912683664855 + ((-0.099962954584931) + o[4] * ((-0.0032632037778459) + 0.000233209225767230 * sigma2b)) * sigma2b)) + pi * ((-0.153348098574500) + (0.0290722882399020 + 0.00037534702741167 * o[4]) * sigma2b + pi * (0.00172966917024110 + ((-0.00038556050844504) - 0.000035017712292608 * o[3]) * sigma2b + pi * ((-0.0000145663936314920) + 5.6420857267269e-6 * sigma2b + pi * (4.1286150074605e-8 + ((-2.06846711188240e-8) + 1.64093936747250e-9 * sigma2b) * sigma2b)))))))))))) / (o[1] * o[2]);
            end tps2b;

            function tps2c "Reverse function for region 2c: T(p,s)"
              extends Modelica.Icons.Function;
              input SI.Pressure p "Pressure";
              input SI.SpecificEntropy s "Specific entropy";
              output SI.Temperature T "Temperature (K)";
            protected
              constant Real IPSTAR = 1.0e-6 "Scaling variable";
              constant Real ISSTAR2C = 1 / 2925.1 "Scaling variable";
              Real pi "Dimensionless pressure";
              Real sigma2c "Dimensionless specific entropy";
              Real[3] o "Vector of auxiliary variables";
            algorithm
              pi := p * IPSTAR;
              sigma2c := 2.0 - s * ISSTAR2C;
              o[1] := pi * pi;
              o[2] := sigma2c * sigma2c;
              o[3] := o[2] * o[2];
              T := (909.68501005365 + 2404.56670884200 * sigma2c + pi * ((-591.62326387130) + pi * (541.45404128074 + sigma2c * ((-270.983084111920) + (979.76525097926 - 469.66772959435 * sigma2c) * sigma2c) + pi * (14.3992746047230 + ((-19.1042042304290) + o[2] * (5.3299167111971 - 21.2529753759340 * sigma2c)) * sigma2c + pi * ((-0.311473344137600) + (0.60334840894623 - 0.042764839702509 * sigma2c) * sigma2c + pi * (0.0058185597255259 + ((-0.0145970082847530) + 0.0056631175631027 * o[3]) * sigma2c + pi * ((-0.000076155864584577) + sigma2c * (0.000224403429193320 - 0.0000125610950134130 * o[2] * sigma2c) + pi * (6.3323132660934e-7 + ((-2.05419896753750e-6) + 3.6405370390082e-8 * sigma2c) * sigma2c + pi * ((-2.97598977892150e-9) + 1.01366185297630e-8 * sigma2c + pi * (5.9925719692351e-12 + sigma2c * ((-2.06778701051640e-11) + o[2] * ((-2.08742781818860e-11) + (1.01621668250890e-10 - 1.64298282813470e-10 * sigma2c) * sigma2c)))))))))))) / o[1];
            end tps2c;

            function tps2 "Reverse function for region 2: T(p,s)"
              extends Modelica.Icons.Function;
              input SI.Pressure p "Pressure";
              input SI.SpecificEntropy s "Specific entropy";
              output SI.Temperature T "Temperature (K)";
            protected
              Real pi "Dimensionless pressure";
              constant SI.SpecificEntropy SLIMIT = 5.85e3 "Subregion boundary specific entropy between regions 2a and 2b";
            algorithm
              if p < 4.0e6 then
                T := tps2a(p, s);
              elseif s > SLIMIT then
                T := tps2b(p, s);
              else
                T := tps2c(p, s);
              end if;
            end tps2;

            function tsat "Region 4 saturation temperature as a function of pressure"
              extends Modelica.Icons.Function;
              input SI.Pressure p "Pressure";
              output SI.Temperature t_sat "Temperature";
            protected
              Real pi "Dimensionless pressure";
              Real[20] o "Vector of auxiliary variables";
            algorithm
              assert(p > triple.ptriple, "IF97 medium function tsat called with too low pressure\n" + "p = " + String(p) + " Pa <= " + String(triple.ptriple) + " Pa (triple point pressure)");
//  assert(p <= data.PCRIT,
//    "tsat: input pressure is higher than the critical point pressure");
              pi := min(p, data.PCRIT) * data.IPSTAR;
              o[1] := pi ^ 0.25;
              o[2] := -3.2325550322333e6 * o[1];
              o[3] := pi ^ 0.5;
              o[4] := -724213.16703206 * o[3];
              o[5] := 405113.40542057 + o[2] + o[4];
              o[6] := -17.0738469400920 * o[1];
              o[7] := 14.9151086135300 + o[3] + o[6];
              o[8] := -4.0 * o[5] * o[7];
              o[9] := 12020.8247024700 * o[1];
              o[10] := 1167.05214527670 * o[3];
              o[11] := (-4823.2657361591) + o[10] + o[9];
              o[12] := o[11] * o[11];
              o[13] := o[12] + o[8];
              o[14] := o[13] ^ 0.5;
              o[15] := -o[14];
              o[16] := -12020.8247024700 * o[1];
              o[17] := -1167.05214527670 * o[3];
              o[18] := 4823.2657361591 + o[15] + o[16] + o[17];
              o[19] := 1 / o[18];
              o[20] := 2.0 * o[19] * o[5];
              t_sat := 0.5 * (650.17534844798 + o[20] - ((-4.0 * ((-0.238555575678490) + 1300.35069689596 * o[19] * o[5])) + (650.17534844798 + o[20]) ^ 2.0) ^ 0.5);
              annotation(
                derivative = tsat_der);
            end tsat;

            function dtsatofp "Derivative of saturation temperature w.r.t. pressure"
              extends Modelica.Icons.Function;
              input SI.Pressure p "Pressure";
              output Real dtsat(unit = "K/Pa") "Derivative of T w.r.t. p";
            protected
              Real pi "Dimensionless pressure";
              Real[49] o "Vector of auxiliary variables";
            algorithm
              pi := max(Modelica.Constants.small, p * data.IPSTAR);
              o[1] := pi ^ 0.75;
              o[2] := 1 / o[1];
              o[3] := -4.268461735023 * o[2];
              o[4] := sqrt(pi);
              o[5] := 1 / o[4];
              o[6] := 0.5 * o[5];
              o[7] := o[3] + o[6];
              o[8] := pi ^ 0.25;
              o[9] := -3.2325550322333e6 * o[8];
              o[10] := -724213.16703206 * o[4];
              o[11] := 405113.40542057 + o[10] + o[9];
              o[12] := -4 * o[11] * o[7];
              o[13] := -808138.758058325 * o[2];
              o[14] := -362106.58351603 * o[5];
              o[15] := o[13] + o[14];
              o[16] := -17.073846940092 * o[8];
              o[17] := 14.91510861353 + o[16] + o[4];
              o[18] := -4 * o[15] * o[17];
              o[19] := 3005.2061756175 * o[2];
              o[20] := 583.52607263835 * o[5];
              o[21] := o[19] + o[20];
              o[22] := 12020.82470247 * o[8];
              o[23] := 1167.0521452767 * o[4];
              o[24] := (-4823.2657361591) + o[22] + o[23];
              o[25] := 2.0 * o[21] * o[24];
              o[26] := o[12] + o[18] + o[25];
              o[27] := -4.0 * o[11] * o[17];
              o[28] := o[24] * o[24];
              o[29] := o[27] + o[28];
              o[30] := sqrt(o[29]);
              o[31] := 1 / o[30];
              o[32] := -o[30];
              o[33] := -12020.82470247 * o[8];
              o[34] := -1167.0521452767 * o[4];
              o[35] := 4823.2657361591 + o[32] + o[33] + o[34];
              o[36] := o[30];
              o[37] := (-4823.2657361591) + o[22] + o[23] + o[36];
              o[38] := o[37] * o[37];
              o[39] := 1 / o[38];
              o[40] := -1.72207339365771 * o[30];
              o[41] := 21592.2055343628 * o[8];
              o[42] := o[30] * o[8];
              o[43] := -8192.87114842946 * o[4];
              o[44] := -0.510632954559659 * o[30] * o[4];
              o[45] := -3100.02526152368 * o[1];
              o[46] := pi;
              o[47] := 1295.95640782102 * o[46];
              o[48] := 2862.09212505088 + o[40] + o[41] + o[42] + o[43] + o[44] + o[45] + o[47];
              o[49] := 1 / (o[35] * o[35]);
              dtsat := data.IPSTAR * 0.5 * (2.0 * o[15] / o[35] - 2. * o[11] * ((-3005.2061756175 * o[2]) - 0.5 * o[26] * o[31] - 583.52607263835 * o[5]) * o[49] - 20953.46356643991 * (o[39] * (1295.95640782102 + 5398.05138359071 * o[2] + 0.25 * o[2] * o[30] - 0.861036696828853 * o[26] * o[31] - 0.255316477279829 * o[26] * o[31] * o[4] - 4096.43557421473 * o[5] - 0.255316477279829 * o[30] * o[5] - 2325.01894614276 / o[8] + 0.5 * o[26] * o[31] * o[8]) - 2.0 * (o[19] + o[20] + 0.5 * o[26] * o[31]) * o[48] * o[37] ^ (-3)) / sqrt(o[39] * o[48]));
            end dtsatofp;

            function tsat_der "Derivative function for tsat"
              extends Modelica.Icons.Function;
              input SI.Pressure p "Pressure";
              input Real der_p(unit = "Pa/s") "Pressure derivative";
              output Real der_tsat(unit = "K/s") "Temperature derivative";
            protected
              Real dtp;
            algorithm
              dtp := dtsatofp(p);
              der_tsat := dtp * der_p;
            end tsat_der;

            function psat "Region 4 saturation pressure as a function of temperature"
              extends Modelica.Icons.Function;
              input SI.Temperature T "Temperature (K)";
              output SI.Pressure p_sat "Pressure";
            protected
              Real[8] o "Vector of auxiliary variables";
              Real Tlim = min(T, data.TCRIT);
            algorithm
              assert(T >= 273.16, "IF97 medium function psat: input temperature (= " + String(triple.ptriple) + " K).\n" + "lower than the triple point temperature 273.16 K");
              o[1] := (-650.17534844798) + Tlim;
              o[2] := 1 / o[1];
              o[3] := -0.238555575678490 * o[2];
              o[4] := o[3] + Tlim;
              o[5] := -4823.2657361591 * o[4];
              o[6] := o[4] * o[4];
              o[7] := 14.9151086135300 * o[6];
              o[8] := 405113.40542057 + o[5] + o[7];
              p_sat := 16.0e6 * o[8] * o[8] * o[8] * o[8] * 1 / (3.2325550322333e6 - 12020.8247024700 * o[4] + 17.0738469400920 * o[6] + ((-4.0 * ((-724213.16703206) + 1167.05214527670 * o[4] + o[6]) * o[8]) + ((-3.2325550322333e6) + 12020.8247024700 * o[4] - 17.0738469400920 * o[6]) ^ 2.0) ^ 0.5) ^ 4.0;
              annotation(
                derivative = psat_der);
            end psat;

            function dptofT "Derivative of pressure w.r.t. temperature along the saturation pressure curve"
              extends Modelica.Icons.Function;
              input SI.Temperature T "Temperature (K)";
              output Real dpt(unit = "Pa/K") "Temperature derivative of pressure";
            protected
              Real[31] o "Vector of auxiliary variables";
              Real Tlim "Temperature limited to TCRIT";
            algorithm
              Tlim := min(T, data.TCRIT);
              o[1] := (-650.17534844798) + Tlim;
              o[2] := 1 / o[1];
              o[3] := -0.238555575678490 * o[2];
              o[4] := o[3] + Tlim;
              o[5] := -4823.2657361591 * o[4];
              o[6] := o[4] * o[4];
              o[7] := 14.9151086135300 * o[6];
              o[8] := 405113.40542057 + o[5] + o[7];
              o[9] := o[8] * o[8];
              o[10] := o[9] * o[9];
              o[11] := o[1] * o[1];
              o[12] := 1 / o[11];
              o[13] := 0.238555575678490 * o[12];
              o[14] := 1.00000000000000 + o[13];
              o[15] := 12020.8247024700 * o[4];
              o[16] := -17.0738469400920 * o[6];
              o[17] := (-3.2325550322333e6) + o[15] + o[16];
              o[18] := -4823.2657361591 * o[14];
              o[19] := 29.8302172270600 * o[14] * o[4];
              o[20] := o[18] + o[19];
              o[21] := 1167.05214527670 * o[4];
              o[22] := (-724213.16703206) + o[21] + o[6];
              o[23] := o[17] * o[17];
              o[24] := -4.0000000000000 * o[22] * o[8];
              o[25] := o[23] + o[24];
              o[26] := sqrt(o[25]);
              o[27] := -12020.8247024700 * o[4];
              o[28] := 17.0738469400920 * o[6];
              o[29] := 3.2325550322333e6 + o[26] + o[27] + o[28];
              o[30] := o[29] * o[29];
              o[31] := o[30] * o[30];
              dpt := 1e6 * ((-64.0 * o[10] * ((-12020.8247024700 * o[14]) + 34.147693880184 * o[14] * o[4] + 0.5 * ((-4.0 * o[20] * o[22]) + 2.00000000000000 * o[17] * (12020.8247024700 * o[14] - 34.147693880184 * o[14] * o[4]) - 4.0 * (1167.05214527670 * o[14] + 2.0 * o[14] * o[4]) * o[8]) / o[26])) / (o[29] * o[31]) + 64. * o[20] * o[8] * o[9] / o[31]);
            end dptofT;

            function psat_der "Derivative function for psat"
              extends Modelica.Icons.Function;
              input SI.Temperature T "Temperature (K)";
              input Real der_T(unit = "K/s") "Temperature derivative";
              output Real der_psat(unit = "Pa/s") "Pressure";
            protected
              Real dpt;
            algorithm
              dpt := dptofT(T);
              der_psat := dpt * der_T;
            end psat_der;

            // Inverses p_hs from the 2001 addition to IAPWS97

            function p1_hs "Pressure as a function of enthalpy and entropy in region 1"
              extends Modelica.Icons.Function;
              input SI.SpecificEnthalpy h "Specific enthalpy";
              input SI.SpecificEntropy s "Specific entropy";
              output SI.Pressure p "Pressure";
              constant Real[:] n = {-0.691997014660582, -0.183612548787560e2, -0.928332409297335e1, 0.659639569909906e2, -0.162060388912024e2, 0.450620017338667e3, 0.854680678224170e3, 0.607523214001162e4, 0.326487682621856e2, -0.269408844582931e2, -0.319947848334300e3, -0.928354307043320e3, 0.303634537455249e2, -0.650540422444146e2, -0.430991316516130e4, -0.747512324096068e3, 0.730000345529245e3, 0.114284032569021e4, -0.436407041874559e3};
              constant Real[:] I = {0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 2, 2, 2, 3, 4, 4, 5};
              constant Real[:] J = {0, 1, 2, 4, 5, 6, 8, 14, 0, 1, 4, 6, 0, 1, 10, 4, 1, 4, 0};
              constant SI.SpecificEnthalpy hstar = 3400e3 "Normalization enthalpy";
              constant SI.Pressure pstar = 100e6 "Normalization pressure";
              constant SI.SpecificEntropy sstar = 7.6e3 "Normalization entropy";
            protected
              Real eta = h / hstar "Normalized specific enthalpy";
              Real sigma = s / sstar "Normalized specific entropy";
            algorithm
              p := sum(n[i] * (eta + 0.05) ^ I[i] * (sigma + 0.05) ^ J[i] for i in 1:19) * pstar;
              annotation(
                Documentation(info = "<html>
<p>
  Equation number 1 from:<br>
  The International Association for the Properties of Water and Steam<br>
  Gaithersburg, Maryland, USA<br>
  September 2001<br>
  Supplementary Release on&nbsp; Backward Equations for Pressure as a
  Function of Enthalpy and Entropy p(h,s) to the IAPWS Industrial
  Formulation 1997 for the Thermodynamic Properties of Water and Steam<br>
</p>
  </html>"));
            end p1_hs;

            function h2ab_s "Boundary between regions 2a and 2b"
              extends Modelica.Icons.Function;
              output SI.SpecificEnthalpy h "Enthalpy";
              input SI.SpecificEntropy s "Entropy";
            protected
              constant Real[:] n = {-0.349898083432139e4, 0.257560716905876e4, -0.421073558227969e3, 0.276349063799944e2};
              constant SI.SpecificEnthalpy hstar = 1e3 "Normalization enthalpy";
              constant SI.SpecificEntropy sstar = 1e3 "Normalization entropy";
              Real sigma = s / sstar "Normalized specific entropy";
            algorithm
              h := (n[1] + n[2] * sigma + n[3] * sigma ^ 2 + n[4] * sigma ^ 3) * hstar;
              annotation(
                Documentation(info = "<html>
  <p>
  Equation number 2 from:<br>
  The International Association for the Properties of Water and Steam<br>
  Gaithersburg, Maryland, USA<br>
  September 2001<br>
  Supplementary Release on&nbsp; Backward Equations for Pressure as a
  Function of Enthalpy and Entropy p(h,s) to the IAPWS Industrial
  Formulation 1997 for the Thermodynamic Properties of Water and Steam<br>
</p>
  </html>"));
            end h2ab_s;

            function p2a_hs "Pressure as a function of enthalpy and entropy in subregion 2a"
              extends Modelica.Icons.Function;
              input SI.SpecificEnthalpy h "Specific enthalpy";
              input SI.SpecificEntropy s "Specific entropy";
              output SI.Pressure p "Pressure";
              constant Real[:] n = {-0.182575361923032e-1, -0.125229548799536, 0.592290437320145, 0.604769706185122e1, 0.238624965444474e3, -0.298639090222922e3, 0.512250813040750e-1, -0.437266515606486, 0.413336902999504, -0.516468254574773e1, -0.557014838445711e1, 0.128555037824478e2, 0.114144108953290e2, -0.119504225652714e3, -0.284777985961560e4, 0.431757846408006e4, 0.112894040802650e1, 0.197409186206319e4, 0.151612444706087e4, 0.141324451421235e-1, 0.585501282219601, -0.297258075863012e1, 0.594567314847319e1, -0.623656565798905e4, 0.965986235133332e4, 0.681500934948134e1, -0.633207286824489e4, -0.558919224465760e1, 0.400645798472063e-1};
              constant Real[:] I = {0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 3, 3, 3, 3, 3, 4, 5, 5, 6, 7};
              constant Real[:] J = {1, 3, 6, 16, 20, 22, 0, 1, 2, 3, 5, 6, 10, 16, 20, 22, 3, 16, 20, 0, 2, 3, 6, 16, 16, 3, 16, 3, 1};
              constant SI.SpecificEnthalpy hstar = 4200e3 "Normalization enthalpy";
              constant SI.Pressure pstar = 4e6 "Normalization pressure";
              constant SI.SpecificEntropy sstar = 12e3 "Normalization entropy";
            protected
              Real eta = h / hstar "Normalized specific enthalpy";
              Real sigma = s / sstar "Normalized specific entropy";
            algorithm
              p := sum(n[i] * (eta - 0.5) ^ I[i] * (sigma - 1.2) ^ J[i] for i in 1:29) ^ 4 * pstar;
              annotation(
                Documentation(info = "<html>
  <p>
  Equation number 3 from:<br>
  The International Association for the Properties of Water and Steam<br>
  Gaithersburg, Maryland, USA<br>
  September 2001<br>
  Supplementary Release on&nbsp; Backward Equations for Pressure as a
  Function of Enthalpy and Entropy p(h,s) to the IAPWS Industrial
  Formulation 1997 for the Thermodynamic Properties of Water and Steam<br>
</p>
  </html>"));
            end p2a_hs;

            function p2b_hs "Pressure as a function of enthalpy and entropy in subregion 2a"
              extends Modelica.Icons.Function;
              input SI.SpecificEnthalpy h "Specific enthalpy";
              input SI.SpecificEntropy s "Specific entropy";
              output SI.Pressure p "Pressure";
              constant Real[:] n = {0.801496989929495e-1, -0.543862807146111, 0.337455597421283, 0.890555451157450e1, 0.313840736431485e3, 0.797367065977789, -0.121616973556240e1, 0.872803386937477e1, -0.169769781757602e2, -0.186552827328416e3, 0.951159274344237e5, -0.189168510120494e2, -0.433407037194840e4, 0.543212633012715e9, 0.144793408386013, 0.128024559637516e3, -0.672309534071268e5, 0.336972380095287e8, -0.586634196762720e3, -0.221403224769889e11, 0.171606668708389e4, -0.570817595806302e9, -0.312109693178482e4, -0.207841384633010e7, 0.305605946157786e13, 0.322157004314333e4, 0.326810259797295e12, -0.144104158934487e4, 0.410694867802691e3, 0.109077066873024e12, -0.247964654258893e14, 0.188801906865134e10, -0.123651009018773e15};
              constant Real[:] I = {0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 2, 2, 2, 3, 3, 3, 3, 4, 4, 5, 5, 6, 6, 6, 7, 7, 8, 8, 8, 8, 12, 14};
              constant Real[:] J = {0, 1, 2, 4, 8, 0, 1, 2, 3, 5, 12, 1, 6, 18, 0, 1, 7, 12, 1, 16, 1, 12, 1, 8, 18, 1, 16, 1, 3, 14, 18, 10, 16};
              constant SI.SpecificEnthalpy hstar = 4100e3 "Normalization enthalpy";
              constant SI.Pressure pstar = 100e6 "Normalization pressure";
              constant SI.SpecificEntropy sstar = 7.9e3 "Normalization entropy";
            protected
              Real eta = h / hstar "Normalized specific enthalpy";
              Real sigma = s / sstar "Normalized specific entropy";
            algorithm
              p := sum(n[i] * (eta - 0.6) ^ I[i] * (sigma - 1.01) ^ J[i] for i in 1:33) ^ 4 * pstar;
              annotation(
                Documentation(info = "<html>
<p>
Equation number 4 from:<br>
The International Association for the Properties of Water and Steam<br>
Gaithersburg, Maryland, USA<br>
September 2001<br>
Supplementary Release on&nbsp; Backward Equations for Pressure as a
Function of Enthalpy and Entropy p(h,s) to the IAPWS Industrial
Formulation 1997 for the Thermodynamic Properties of Water and Steam<br>
</p>
      </html>"));
            end p2b_hs;

            function p2c_hs "Pressure as a function of enthalpy and entropy in subregion 2c"
              extends Modelica.Icons.Function;
              input SI.SpecificEnthalpy h "Specific enthalpy";
              input SI.SpecificEntropy s "Specific entropy";
              output SI.Pressure p "Pressure";
              constant Real[:] n = {0.112225607199012, -0.339005953606712e1, -0.320503911730094e2, -0.197597305104900e3, -0.407693861553446e3, 0.132943775222331e5, 0.170846839774007e1, 0.373694198142245e2, 0.358144365815434e4, 0.423014446424664e6, -0.751071025760063e9, 0.523446127607898e2, -0.228351290812417e3, -0.960652417056937e6, -0.807059292526074e8, 0.162698017225669e13, 0.772465073604171, 0.463929973837746e5, -0.137317885134128e8, 0.170470392630512e13, -0.251104628187308e14, 0.317748830835520e14, 0.538685623675312e2, -0.553089094625169e5, -0.102861522421405e7, 0.204249418756234e13, 0.273918446626977e9, -0.263963146312685e16, -0.107890854108088e10, -0.296492620980124e11, -0.111754907323424e16};
              constant Real[:] I = {0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 3, 3, 3, 3, 3, 4, 5, 5, 5, 5, 6, 6, 10, 12, 16};
              constant Real[:] J = {0, 1, 2, 3, 4, 8, 0, 2, 5, 8, 14, 2, 3, 7, 10, 18, 0, 5, 8, 16, 18, 18, 1, 4, 6, 14, 8, 18, 7, 7, 10};
              constant SI.SpecificEnthalpy hstar = 3500e3 "Normalization enthalpy";
              constant SI.Pressure pstar = 100e6 "Normalization pressure";
              constant SI.SpecificEntropy sstar = 5.9e3 "Normalization entropy";
            protected
              Real eta = h / hstar "Normalized specific enthalpy";
              Real sigma = s / sstar "Normalized specific entropy";
            algorithm
              p := sum(n[i] * (eta - 0.7) ^ I[i] * (sigma - 1.1) ^ J[i] for i in 1:31) ^ 4 * pstar;
              annotation(
                Documentation(info = "<html>
      <p>
      Equation number 5 from:<br>
      The International Association for the Properties of Water and Steam<br>
      Gaithersburg, Maryland, USA<br>
      September 2001<br>
      Supplementary Release on&nbsp; Backward Equations for Pressure as a
      Function of Enthalpy and Entropy p(h,s) to the IAPWS Industrial
      Formulation 1997 for the Thermodynamic Properties of Water and Steam<br>
   </p>
      </html>"));
            end p2c_hs;

            // Inverses from the 2003 additions to IF97

            function h3ab_p "Region 3 a b boundary for pressure/enthalpy"
              extends Modelica.Icons.Function;
              output SI.SpecificEnthalpy h "Enthalpy";
              input SI.Pressure p "Pressure";
            protected
              constant Real[:] n = {0.201464004206875e4, 0.374696550136983e1, -0.219921901054187e-1, 0.875131686009950e-4};
              constant SI.SpecificEnthalpy hstar = 1000 "Normalization enthalpy";
              constant SI.Pressure pstar = 1e6 "Normalization pressure";
              Real pi = p / pstar "Normalized specific pressure";
            algorithm
              h := (n[1] + n[2] * pi + n[3] * pi ^ 2 + n[4] * pi ^ 3) * hstar;
              annotation(
                Documentation(info = "<html>
      <p>
      &nbsp;Equation number 1 from:
      </p>
      <div style=\"text-align: center;\">&nbsp;[1] The international Association
      for the Properties of Water and Steam<br>
      &nbsp;Vejle, Denmark<br>
      &nbsp;August 2003<br>
      &nbsp;Supplementary Release on Backward Equations for the Functions
      T(p,h), v(p,h) and T(p,s), <br>
      &nbsp;v(p,s) for Region 3 of the IAPWS Industrial Formulation 1997 for
      the Thermodynamic Properties of<br>
      &nbsp;Water and Steam</div>
      </html>"));
            end h3ab_p;

            function T3a_ph "Region 3 a: inverse function T(p,h)"
              extends Modelica.Icons.Function;
              input SI.Pressure p "Pressure";
              input SI.SpecificEnthalpy h "Specific enthalpy";
              output SI.Temp_K T "Temperature";
            protected
              constant Real[:] n = {-0.133645667811215e-6, 0.455912656802978e-5, -0.146294640700979e-4, 0.639341312970080e-2, 0.372783927268847e3, -0.718654377460447e4, 0.573494752103400e6, -0.267569329111439e7, -0.334066283302614e-4, -0.245479214069597e-1, 0.478087847764996e2, 0.764664131818904e-5, 0.128350627676972e-2, 0.171219081377331e-1, -0.851007304583213e1, -0.136513461629781e-1, -0.384460997596657e-5, 0.337423807911655e-2, -0.551624873066791, 0.729202277107470, -0.992522757376041e-2, -0.119308831407288, 0.793929190615421, 0.454270731799386, 0.209998591259910, -0.642109823904738e-2, -0.235155868604540e-1, 0.252233108341612e-2, -0.764885133368119e-2, 0.136176427574291e-1, -0.133027883575669e-1};
              constant Real[:] I = {-12, -12, -12, -12, -12, -12, -12, -12, -10, -10, -10, -8, -8, -8, -8, -5, -3, -2, -2, -2, -1, -1, 0, 0, 1, 3, 3, 4, 4, 10, 12};
              constant Real[:] J = {0, 1, 2, 6, 14, 16, 20, 22, 1, 5, 12, 0, 2, 4, 10, 2, 0, 1, 3, 4, 0, 2, 0, 1, 1, 0, 1, 0, 3, 4, 5};
              constant SI.SpecificEnthalpy hstar = 2300e3 "Normalization enthalpy";
              constant SI.Pressure pstar = 100e6 "Normalization pressure";
              constant SI.Temp_K Tstar = 760 "Normalization temperature";
              Real pi = p / pstar "Normalized specific pressure";
              Real eta = h / hstar "Normalized specific enthalpy";
            algorithm
              T := sum(n[i] * (pi + 0.240) ^ I[i] * (eta - 0.615) ^ J[i] for i in 1:31) * Tstar;
              annotation(
                Documentation(info = "<html>
<p>
 &nbsp;Equation number 2 from:
</p>
 <div style=\"text-align: center;\">&nbsp;[1] The international Association
 for the Properties of Water and Steam<br>
 &nbsp;Vejle, Denmark<br>
 &nbsp;August 2003<br>
 &nbsp;Supplementary Release on Backward Equations for the Functions
 T(p,h), v(p,h) and T(p,s), <br>
 &nbsp;v(p,s) for Region 3 of the IAPWS Industrial Formulation 1997 for
 the Thermodynamic Properties of<br>
 &nbsp;Water and Steam</div>
 </html>"));
            end T3a_ph;

            function T3b_ph "Region 3 b: inverse function T(p,h)"
              extends Modelica.Icons.Function;
              input SI.Pressure p "Pressure";
              input SI.SpecificEnthalpy h "Specific enthalpy";
              output SI.Temp_K T "Temperature";
            protected
              constant Real[:] n = {0.323254573644920e-4, -0.127575556587181e-3, -0.475851877356068e-3, 0.156183014181602e-2, 0.105724860113781, -0.858514221132534e2, 0.724140095480911e3, 0.296475810273257e-2, -0.592721983365988e-2, -0.126305422818666e-1, -0.115716196364853, 0.849000969739595e2, -0.108602260086615e-1, 0.154304475328851e-1, 0.750455441524466e-1, 0.252520973612982e-1, -0.602507901232996e-1, -0.307622221350501e1, -0.574011959864879e-1, 0.503471360939849e1, -0.925081888584834, 0.391733882917546e1, -0.773146007130190e2, 0.949308762098587e4, -0.141043719679409e7, 0.849166230819026e7, 0.861095729446704, 0.323346442811720, 0.873281936020439, -0.436653048526683, 0.286596714529479, -0.131778331276228, 0.676682064330275e-2};
              constant Real[:] I = {-12, -12, -10, -10, -10, -10, -10, -8, -8, -8, -8, -8, -6, -6, -6, -4, -4, -3, -2, -2, -1, -1, -1, -1, -1, -1, 0, 0, 1, 3, 5, 6, 8};
              constant Real[:] J = {0, 1, 0, 1, 5, 10, 12, 0, 1, 2, 4, 10, 0, 1, 2, 0, 1, 5, 0, 4, 2, 4, 6, 10, 14, 16, 0, 2, 1, 1, 1, 1, 1};
              constant SI.Temp_K Tstar = 860 "Normalization temperature";
              constant SI.Pressure pstar = 100e6 "Normalization pressure";
              constant SI.SpecificEnthalpy hstar = 2800e3 "Normalization enthalpy";
              Real pi = p / pstar "Normalized specific pressure";
              Real eta = h / hstar "Normalized specific enthalpy";
            algorithm
              T := sum(n[i] * (pi + 0.298) ^ I[i] * (eta - 0.720) ^ J[i] for i in 1:33) * Tstar;
              annotation(
                Documentation(info = "<html>
 <p>
 &nbsp;Equation number 3 from:
</p>
 <div style=\"text-align: center;\">&nbsp;[1] The international Association
 for the Properties of Water and Steam<br>
 &nbsp;Vejle, Denmark<br>
 &nbsp;August 2003<br>
 &nbsp;Supplementary Release on Backward Equations for the Functions
 T(p,h), v(p,h) and T(p,s), <br>
 &nbsp;v(p,s) for Region 3 of the IAPWS Industrial Formulation 1997 for
 the Thermodynamic Properties of<br>
 &nbsp;Water and Steam</div>
 </html>"));
            end T3b_ph;

            function v3a_ph "Region 3 a: inverse function v(p,h)"
              extends Modelica.Icons.Function;
              input SI.Pressure p "Pressure";
              input SI.SpecificEnthalpy h "Specific enthalpy";
              output SI.SpecificVolume v "Specific volume";
            protected
              constant Real[:] n = {0.529944062966028e-2, -0.170099690234461, 0.111323814312927e2, -0.217898123145125e4, -0.506061827980875e-3, 0.556495239685324, -0.943672726094016e1, -0.297856807561527, 0.939353943717186e2, 0.192944939465981e-1, 0.421740664704763, -0.368914126282330e7, -0.737566847600639e-2, -0.354753242424366, -0.199768169338727e1, 0.115456297059049e1, 0.568366875815960e4, 0.808169540124668e-2, 0.172416341519307, 0.104270175292927e1, -0.297691372792847, 0.560394465163593, 0.275234661176914, -0.148347894866012, -0.651142513478515e-1, -0.292468715386302e1, 0.664876096952665e-1, 0.352335014263844e1, -0.146340792313332e-1, -0.224503486668184e1, 0.110533464706142e1, -0.408757344495612e-1};
              constant Real[:] I = {-12, -12, -12, -12, -10, -10, -10, -8, -8, -6, -6, -6, -4, -4, -3, -2, -2, -1, -1, -1, -1, 0, 0, 1, 1, 1, 2, 2, 3, 4, 5, 8};
              constant Real[:] J = {6, 8, 12, 18, 4, 7, 10, 5, 12, 3, 4, 22, 2, 3, 7, 3, 16, 0, 1, 2, 3, 0, 1, 0, 1, 2, 0, 2, 0, 2, 2, 2};
              constant SI.Volume vstar = 0.0028 "Normalization temperature";
              constant SI.Pressure pstar = 100e6 "Normalization pressure";
              constant SI.SpecificEnthalpy hstar = 2100e3 "Normalization enthalpy";
              Real pi = p / pstar "Normalized specific pressure";
              Real eta = h / hstar "Normalized specific enthalpy";
            algorithm
              v := sum(n[i] * (pi + 0.128) ^ I[i] * (eta - 0.727) ^ J[i] for i in 1:32) * vstar;
              annotation(
                Documentation(info = "<html>
<p>
&nbsp;Equation number 4 from:
</p>
 <div style=\"text-align: center;\">&nbsp;[1] The international Association
 for the Properties of Water and Steam<br>
 &nbsp;Vejle, Denmark<br>
 &nbsp;August 2003<br>
 &nbsp;Supplementary Release on Backward Equations for the Functions
 T(p,h), v(p,h) and T(p,s), <br>
 &nbsp;v(p,s) for Region 3 of the IAPWS Industrial Formulation 1997 for
 the Thermodynamic Properties of<br>
 &nbsp;Water and Steam</div>
 </html>"));
            end v3a_ph;

            function v3b_ph "Region 3 b: inverse function v(p,h)"
              extends Modelica.Icons.Function;
              input SI.Pressure p "Pressure";
              input SI.SpecificEnthalpy h "Specific enthalpy";
              output SI.SpecificVolume v "Specific volume";
            protected
              constant Real[:] n = {-0.225196934336318e-8, 0.140674363313486e-7, 0.233784085280560e-5, -0.331833715229001e-4, 0.107956778514318e-2, -0.271382067378863, 0.107202262490333e1, -0.853821329075382, -0.215214194340526e-4, 0.769656088222730e-3, -0.431136580433864e-2, 0.453342167309331, -0.507749535873652, -0.100475154528389e3, -0.219201924648793, -0.321087965668917e1, 0.607567815637771e3, 0.557686450685932e-3, 0.187499040029550, 0.905368030448107e-2, 0.285417173048685, 0.329924030996098e-1, 0.239897419685483, 0.482754995951394e1, -0.118035753702231e2, 0.169490044091791, -0.179967222507787e-1, 0.371810116332674e-1, -0.536288335065096e-1, 0.160697101092520e1};
              constant Real[:] I = {-12, -12, -8, -8, -8, -8, -8, -8, -6, -6, -6, -6, -6, -6, -4, -4, -4, -3, -3, -2, -2, -1, -1, -1, -1, 0, 1, 1, 2, 2};
              constant Real[:] J = {0, 1, 0, 1, 3, 6, 7, 8, 0, 1, 2, 5, 6, 10, 3, 6, 10, 0, 2, 1, 2, 0, 1, 4, 5, 0, 0, 1, 2, 6};
              constant SI.Volume vstar = 0.0088 "Normalization temperature";
              constant SI.Pressure pstar = 100e6 "Normalization pressure";
              constant SI.SpecificEnthalpy hstar = 2800e3 "Normalization enthalpy";
              Real pi = p / pstar "Normalized specific pressure";
              Real eta = h / hstar "Normalized specific enthalpy";
            algorithm
              v := sum(n[i] * (pi + 0.0661) ^ I[i] * (eta - 0.720) ^ J[i] for i in 1:30) * vstar;
              annotation(
                Documentation(info = "<html>
<p>
 &nbsp;Equation number 5 from:
</p>
 <div style=\"text-align: center;\">&nbsp;[1] The international Association
 for the Properties of Water and Steam<br>
 &nbsp;Vejle, Denmark<br>
 &nbsp;August 2003<br>
 &nbsp;Supplementary Release on Backward Equations for the Functions
 T(p,h), v(p,h) and T(p,s), <br>
 &nbsp;v(p,s) for Region 3 of the IAPWS Industrial Formulation 1997 for
 the Thermodynamic Properties of<br>
 &nbsp;Water and Steam</div>
 </html>"));
            end v3b_ph;

            function T3a_ps "Region 3 a: inverse function T(p,s)"
              extends Modelica.Icons.Function;
              input SI.Pressure p "Pressure";
              input SI.SpecificEntropy s "Specific entropy";
              output SI.Temp_K T "Temperature";
            protected
              constant Real[:] n = {0.150042008263875e10, -0.159397258480424e12, 0.502181140217975e-3, -0.672057767855466e2, 0.145058545404456e4, -0.823889534888890e4, -0.154852214233853, 0.112305046746695e2, -0.297000213482822e2, 0.438565132635495e11, 0.137837838635464e-2, -0.297478527157462e1, 0.971777947349413e13, -0.571527767052398e-4, 0.288307949778420e5, -0.744428289262703e14, 0.128017324848921e2, -0.368275545889071e3, 0.664768904779177e16, 0.449359251958880e-1, -0.422897836099655e1, -0.240614376434179, -0.474341365254924e1, 0.724093999126110, 0.923874349695897, 0.399043655281015e1, 0.384066651868009e-1, -0.359344365571848e-2, -0.735196448821653, 0.188367048396131, 0.141064266818704e-3, -0.257418501496337e-2, 0.123220024851555e-2};
              constant Real[:] I = {-12, -12, -10, -10, -10, -10, -8, -8, -8, -8, -6, -6, -6, -5, -5, -5, -4, -4, -4, -2, -2, -1, -1, 0, 0, 0, 1, 2, 2, 3, 8, 8, 10};
              constant Real[:] J = {28, 32, 4, 10, 12, 14, 5, 7, 8, 28, 2, 6, 32, 0, 14, 32, 6, 10, 36, 1, 4, 1, 6, 0, 1, 4, 0, 0, 3, 2, 0, 1, 2};
              constant SI.Temp_K Tstar = 760 "Normalization temperature";
              constant SI.Pressure pstar = 100e6 "Normalization pressure";
              constant SI.SpecificEntropy sstar = 4.4e3 "Normalization entropy";
              Real pi = p / pstar "Normalized specific pressure";
              Real sigma = s / sstar "Normalized specific entropy";
            algorithm
              T := sum(n[i] * (pi + 0.240) ^ I[i] * (sigma - 0.703) ^ J[i] for i in 1:33) * Tstar;
              annotation(
                Documentation(info = "<html>
<p>
 &nbsp;Equation number 6 from:
</p>
 <div style=\"text-align: center;\">&nbsp;[1] The international Association
 for the Properties of Water and Steam<br>
 &nbsp;Vejle, Denmark<br>
 &nbsp;August 2003<br>
 &nbsp;Supplementary Release on Backward Equations for the Functions
 T(p,h), v(p,h) and T(p,s), <br>
 &nbsp;v(p,s) for Region 3 of the IAPWS Industrial Formulation 1997 for
 the Thermodynamic Properties of<br>
 &nbsp;Water and Steam</div>
 </html>"));
            end T3a_ps;

            function T3b_ps "Region 3 b: inverse function T(p,s)"
              extends Modelica.Icons.Function;
              input SI.Pressure p "Pressure";
              input SI.SpecificEntropy s "Specific entropy";
              output SI.Temp_K T "Temperature";
            protected
              constant Real[:] n = {0.527111701601660, -0.401317830052742e2, 0.153020073134484e3, -0.224799398218827e4, -0.193993484669048, -0.140467557893768e1, 0.426799878114024e2, 0.752810643416743, 0.226657238616417e2, -0.622873556909932e3, -0.660823667935396, 0.841267087271658, -0.253717501764397e2, 0.485708963532948e3, 0.880531517490555e3, 0.265015592794626e7, -0.359287150025783, -0.656991567673753e3, 0.241768149185367e1, 0.856873461222588, 0.655143675313458, -0.213535213206406, 0.562974957606348e-2, -0.316955725450471e15, -0.699997000152457e-3, 0.119845803210767e-1, 0.193848122022095e-4, -0.215095749182309e-4};
              constant Real[:] I = {-12, -12, -12, -12, -8, -8, -8, -6, -6, -6, -5, -5, -5, -5, -5, -4, -3, -3, -2, 0, 2, 3, 4, 5, 6, 8, 12, 14};
              constant Real[:] J = {1, 3, 4, 7, 0, 1, 3, 0, 2, 4, 0, 1, 2, 4, 6, 12, 1, 6, 2, 0, 1, 1, 0, 24, 0, 3, 1, 2};
              constant SI.Temp_K Tstar = 860 "Normalization temperature";
              constant SI.Pressure pstar = 100e6 "Normalization pressure";
              constant SI.SpecificEntropy sstar = 5.3e3 "Normalization entropy";
              Real pi = p / pstar "Normalized specific pressure";
              Real sigma = s / sstar "Normalized specific entropy";
            algorithm
              T := sum(n[i] * (pi + 0.760) ^ I[i] * (sigma - 0.818) ^ J[i] for i in 1:28) * Tstar;
              annotation(
                Documentation(info = "<html>
<p>
 &nbsp;Equation number 7 from:
</p>
 <div style=\"text-align: center;\">&nbsp;[1] The international Association
 for the Properties of Water and Steam<br>
 &nbsp;Vejle, Denmark<br>
 &nbsp;August 2003<br>
 &nbsp;Supplementary Release on Backward Equations for the Functions
 T(p,h), v(p,h) and T(p,s), <br>
 &nbsp;v(p,s) for Region 3 of the IAPWS Industrial Formulation 1997 for
 the Thermodynamic Properties of<br>
 &nbsp;Water and Steam</div>
</html>"));
            end T3b_ps;

            function v3a_ps "Region 3 a: inverse function v(p,s)"
              extends Modelica.Icons.Function;
              input SI.Pressure p "Pressure";
              input SI.SpecificEntropy s "Specific entropy";
              output SI.SpecificVolume v "Specific volume";
            protected
              constant Real[:] n = {0.795544074093975e2, -0.238261242984590e4, 0.176813100617787e5, -0.110524727080379e-2, -0.153213833655326e2, 0.297544599376982e3, -0.350315206871242e8, 0.277513761062119, -0.523964271036888, -0.148011182995403e6, 0.160014899374266e7, 0.170802322663427e13, 0.246866996006494e-3, 0.165326084797980e1, -0.118008384666987, 0.253798642355900e1, 0.965127704669424, -0.282172420532826e2, 0.203224612353823, 0.110648186063513e1, 0.526127948451280, 0.277000018736321, 0.108153340501132e1, -0.744127885357893e-1, 0.164094443541384e-1, -0.680468275301065e-1, 0.257988576101640e-1, -0.145749861944416e-3};
              constant Real[:] I = {-12, -12, -12, -10, -10, -10, -10, -8, -8, -8, -8, -6, -5, -4, -3, -3, -2, -2, -1, -1, 0, 0, 0, 1, 2, 4, 5, 6};
              constant Real[:] J = {10, 12, 14, 4, 8, 10, 20, 5, 6, 14, 16, 28, 1, 5, 2, 4, 3, 8, 1, 2, 0, 1, 3, 0, 0, 2, 2, 0};
              constant SI.Volume vstar = 0.0028 "Normalization temperature";
              constant SI.Pressure pstar = 100e6 "Normalization pressure";
              constant SI.SpecificEntropy sstar = 4.4e3 "Normalization entropy";
              Real pi = p / pstar "Normalized specific pressure";
              Real sigma = s / sstar "Normalized specific entropy";
            algorithm
              v := sum(n[i] * (pi + 0.187) ^ I[i] * (sigma - 0.755) ^ J[i] for i in 1:28) * vstar;
              annotation(
                Documentation(info = "<html>
<p>
 &nbsp;Equation number 8 from:
</p>
 <div style=\"text-align: center;\">&nbsp;[1] The international Association
 for the Properties of Water and Steam<br>
 &nbsp;Vejle, Denmark<br>
 &nbsp;August 2003<br>
 &nbsp;Supplementary Release on Backward Equations for the Functions
 T(p,h), v(p,h) and T(p,s), <br>
 &nbsp;v(p,s) for Region 3 of the IAPWS Industrial Formulation 1997 for
 the Thermodynamic Properties of<br>
 &nbsp;Water and Steam</div>
</html>"));
            end v3a_ps;

            function v3b_ps "Region 3 b: inverse function v(p,s)"
              extends Modelica.Icons.Function;
              input SI.Pressure p "Pressure";
              input SI.SpecificEntropy s "Specific entropy";
              output SI.SpecificVolume v "Specific volume";
            protected
              constant Real[:] n = {0.591599780322238e-4, -0.185465997137856e-2, 0.104190510480013e-1, 0.598647302038590e-2, -0.771391189901699, 0.172549765557036e1, -0.467076079846526e-3, 0.134533823384439e-1, -0.808094336805495e-1, 0.508139374365767, 0.128584643361683e-2, -0.163899353915435e1, 0.586938199318063e1, -0.292466667918613e1, -0.614076301499537e-2, 0.576199014049172e1, -0.121613320606788e2, 0.167637540957944e1, -0.744135838773463e1, 0.378168091437659e-1, 0.401432203027688e1, 0.160279837479185e2, 0.317848779347728e1, -0.358362310304853e1, -0.115995260446827e7, 0.199256573577909, -0.122270624794624, -0.191449143716586e2, -0.150448002905284e-1, 0.146407900162154e2, -0.327477787188230e1};
              constant Real[:] I = {-12, -12, -12, -12, -12, -12, -10, -10, -10, -10, -8, -5, -5, -5, -4, -4, -4, -4, -3, -2, -2, -2, -2, -2, -2, 0, 0, 0, 1, 1, 2};
              constant Real[:] J = {0, 1, 2, 3, 5, 6, 0, 1, 2, 4, 0, 1, 2, 3, 0, 1, 2, 3, 1, 0, 1, 2, 3, 4, 12, 0, 1, 2, 0, 2, 2};
              constant SI.Volume vstar = 0.0088 "Normalization temperature";
              constant SI.Pressure pstar = 100e6 "Normalization pressure";
              constant SI.SpecificEntropy sstar = 5.3e3 "Normalization entropy";
              Real pi = p / pstar "Normalized specific pressure";
              Real sigma = s / sstar "Normalized specific entropy";
            algorithm
              v := sum(n[i] * (pi + 0.298) ^ I[i] * (sigma - 0.816) ^ J[i] for i in 1:31) * vstar;
              annotation(
                Documentation(info = "<html>
<p>
 &nbsp;Equation number 9 from:
</p>
<div style=\"text-align: center;\">&nbsp;[1] The international Association
 for the Properties of Water and Steam<br>
 &nbsp;Vejle, Denmark<br>
 &nbsp;August 2003<br>
 &nbsp;Supplementary Release on Backward Equations for the Functions
 T(p,h), v(p,h) and T(p,s), <br>
 &nbsp;v(p,s) for Region 3 of the IAPWS Industrial Formulation 1997 for
 the Thermodynamic Properties of<br>
 &nbsp;Water and Steam</div>
</html>"));
            end v3b_ps;
            annotation(
              Documentation(info = "<HTML><h4>Package description</h4>
          <p>Package BaseIF97/Basic computes the the fundamental functions for the 5 regions of the steam tables
          as described in the standards document <a href=\"modelica://Modelica/Resources/Documentation/Media/Water/IF97documentation/IF97.pdf\">IF97.pdf</a>. The code of these
          functions has been generated using <b><i>Mathematica</i></b> and the add-on packages \"Format\" and \"Optimize\"
          to generate highly efficient, expression-optimized C-code from a symbolic representation of the thermodynamic
          functions. The C-code has than been transformed into Modelica code. An important feature of this optimization was to
          simultaneously optimize the functions and the directional derivatives because they share many common subexpressions.</p>
          <h4>Package contents</h4>
          <ul>
          <li>Function <b>g1</b> computes the dimensionless Gibbs function for region 1 and all derivatives up
          to order 2 w.r.t. pi and tau. Inputs: p and T.</li>
          <li>Function <b>g2</b> computes the dimensionless Gibbs function  for region 2 and all derivatives up
          to order 2 w.r.t. pi and tau. Inputs: p and T.</li>
          <li>Function <b>g2metastable</b> computes the dimensionless Gibbs function for metastable vapour
          (adjacent to region 2 but 2-phase at equilibrium) and all derivatives up
          to order 2 w.r.t. pi and tau. Inputs: p and T.</li>
          <li>Function <b>f3</b> computes the dimensionless Helmholtz function  for region 3 and all derivatives up
          to order 2 w.r.t. delta and tau. Inputs: d and T.</li>
          <li>Function <b>g5</b>computes the dimensionless Gibbs function for region 5 and all derivatives up
          to order 2 w.r.t. pi and tau. Inputs: p and T.</li>
          <li>Function <b>tph1</b> computes the inverse function T(p,h) in region 1.</li>
          <li>Function <b>tph2</b> computes the inverse function T(p,h) in region 2.</li>
          <li>Function <b>tps2a</b> computes the inverse function T(p,s) in region 2a.</li>
          <li>Function <b>tps2b</b> computes the inverse function T(p,s) in region 2b.</li>
          <li>Function <b>tps2c</b> computes the inverse function T(p,s) in region 2c.</li>
          <li>Function <b>tps2</b> computes the inverse function T(p,s) in region 2.</li>
          <li>Function <b>tsat</b> computes the saturation temperature as a function of pressure.</li>
          <li>Function <b>dtsatofp</b> computes the derivative of the saturation temperature w.r.t. pressure as
          a function of pressure.</li>
          <li>Function <b>tsat_der</b> computes the Modelica derivative function of tsat.</li>
          <li>Function <b>psat</b> computes the saturation pressure as a function of temperature.</li>
          <li>Function <b>dptofT</b>  computes the derivative of the saturation pressure w.r.t. temperature as
          a function of temperature.</li>
          <li>Function <b>psat_der</b> computes the Modelica derivative function of psat.</li>
          </ul>
          <h4>Version Info and Revision history
          </h4>
          <ul>
          <li>First implemented: <i>July, 2000</i>
          by <a href=\"http://www.control.lth.se/~hubertus/\">Hubertus Tummescheit</a>
          </li>
          </ul>
          <address>Author: Hubertus Tummescheit, <br>
      Modelon AB<br>
      Ideon Science Park<br>
      SE-22370 Lund, Sweden<br>
      email: hubertus@modelon.se
          </address>
          <ul>
          <li>Initial version: July 2000</li>
          <li>Documentation added: December 2002</li>
          </ul>
       <p>
       Equation from:
       </p>
       <div style=\"text-align: center;\">&nbsp;[1] The international Association
       for the Properties of Water and Steam<br>
       &nbsp;Vejle, Denmark<br>
       &nbsp;August 2003<br>
       &nbsp;Supplementary Release on Backward Equations for the Functions
       T(p,h), v(p,h) and T(p,s), <br>
       &nbsp;v(p,s) for Region 3 of the IAPWS Industrial Formulation 1997 for
       the Thermodynamic Properties of<br>
       &nbsp;Water and Steam</div>
       </html>"));
          end Basic;

          package IceBoundaries "The melting line and sublimation line curves from IAPWS"
            extends Modelica.Icons.Package;

            function pmIceI_T "Melting pressure of ice I (temperature range from 273.16 to 251.165 K)"
              extends Modelica.Icons.Function;
              input SI.Temp_K T "Temperature";
              output SI.Pressure pm "Melting pressure of iceI(for T from 273.16 to 251.165 K)";
            protected
              constant SI.Temp_K Tn = 273.16 "Normalization temperature";
              constant SI.Pressure pn = 611.657 "Normalization pressure";
              Real sigma = T / Tn "Normalized temperature";
            algorithm
              pm := (1 - 0.626000e6 * (1 - sigma ^ (-3)) + 0.197135e6 * (1 - sigma ^ 21.2)) * pn;
              annotation(
                Documentation(info = "<html>
  <p>
  Equation 1 from:<br>
  The International Association for the Properties of Water and Steam<br>
  Milan, Italy<br>
  September 1993<br>
  Release on the Pressure along the Melting and the Sublimation Curves of
  Ordinary Water Substance<br>
</p>
  </html>"));
            end pmIceI_T;

            function pmIceIII_T "Melting pressure of ice III (temperature range from 251.165 to 256.164 K)"
              extends Modelica.Icons.Function;
              input SI.Temp_K T "Temperature";
              output SI.Pressure pm "Melting pressure of iceIII(for T from 251.165 to 256.164 K)";
            protected
              constant SI.Temp_K Tn = 251.165 "Normalization temperature";
              constant SI.Pressure pn = 209.9e6 "Normalization pressure";
              Real sigma = T / Tn "Normalized temperature";
            algorithm
              pm := (1 - 0.295252 * (1 - sigma ^ 60)) * pn;
              annotation(
                Documentation(info = "<html>
  <p>
  Equation 2 from:<br>
  The International Association for the Properties of Water and Steam<br>
  Milan, Italy<br>
  September 1993<br>
  Release on the Pressure along the Melting and the Sublimation Curves of
  Ordinary Water Substance<br>
</p>
  </html>"));
            end pmIceIII_T;

            function pmIceV_T "Melting pressure of ice V (temperature range from 256.164 to 273.31 K)"
              extends Modelica.Icons.Function;
              input SI.Temp_K T "Temperature";
              output SI.Pressure pm "Melting pressure of iceV(for T from 256.164 to 273.31 K)";
            protected
              constant SI.Temp_K Tn = 256.164 "Normalization temperature";
              constant SI.Pressure pn = 350.1e6 "Normalization pressure";
              Real sigma = T / Tn "Normalized temperature";
            algorithm
              pm := (1 - 1.18721 * (1 - sigma ^ 8)) * pn;
              annotation(
                Documentation(info = "<html>
  <p>
  Equation 3 from:<br>
  The International Association for the Properties of Water and Steam<br>
  Milan, Italy<br>
  September 1993<br>
  Release on the Pressure along the Melting and the Sublimation Curves of
  Ordinary Water Substance<br>
</p>
  </html>"));
            end pmIceV_T;

            function sublimationPressure_T "Sublimation pressure, valid from 190 to 273.16 K"
              extends Modelica.Icons.Function;
              input SI.Temp_K T "Temperature";
              output SI.Pressure psubl "Sublimation pressure (for T from 190 to 273.16)";
            protected
              constant SI.Temp_K Tn = 273.16 "Normalization temperature";
              constant SI.Pressure pn = 611.657 "Normalization pressure";
              constant Real[2] a = {-13.9281690, 34.7078238} "Constant values";
              Real sigma = T / Tn "Normalized temperature";
            algorithm
              psubl := Modelica.Math.exp(a[1] * (1 - sigma ^ (-1.5)) + a[2] * (1 - sigma ^ (-1.25))) * pn;
              annotation(
                Documentation(info = "<html>
  <p>
  Equation 6 from:<br>
  The International Association for the Properties of Water and Steam<br>
  Milan, Italy<br>
  September 1993<br>
  Release on the Pressure along the Melting and the Sublimation Curves of
  Ordinary Water Substance<br>
</p>
  </html>"));
            end sublimationPressure_T;
            annotation(
              Documentation(info = "<html>
<p>
The International Association for the Properties of Water and Steam<br>
Milan, Italy<br>
September 1993<br>
Release on the Pressure along the Melting and the Sublimation Curves of
Ordinary Water Substance<br>
</p>
</html>"));
          end IceBoundaries;

          package Transport "Transport properties for water according to IAPWS/IF97"
            extends Modelica.Icons.Package;

            function visc_dTp "Dynamic viscosity eta(d,T,p), industrial formulation"
              extends Modelica.Icons.Function;
              input SI.Density d "Density";
              input SI.Temperature T "Temperature (K)";
              input SI.Pressure p "Pressure (only needed for region of validity)";
              input Integer phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
              output SI.DynamicViscosity eta "Dynamic viscosity";
            protected
              constant Real n0 = 1.0 "Viscosity coefficient";
              constant Real n1 = 0.978197 "Viscosity coefficient";
              constant Real n2 = 0.579829 "Viscosity coefficient";
              constant Real n3 = -0.202354 "Viscosity coefficient";
              constant Real[42] nn = array(0.5132047, 0.3205656, 0.0, 0.0, -0.7782567, 0.1885447, 0.2151778, 0.7317883, 1.241044, 1.476783, 0.0, 0.0, -0.2818107, -1.070786, -1.263184, 0.0, 0.0, 0.0, 0.1778064, 0.460504, 0.2340379, -0.4924179, 0.0, 0.0, -0.0417661, 0.0, 0.0, 0.1600435, 0.0, 0.0, 0.0, -0.01578386, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -0.003629481, 0.0, 0.0) "Viscosity coefficients";
              constant SI.Density rhostar = 317.763 "Scaling density";
              constant SI.DynamicViscosity etastar = 55.071e-6 "Scaling viscosity";
              constant SI.Temperature tstar = 647.226 "Scaling temperature";
              Integer i "Auxiliary variable";
              Integer j "Auxiliary variable";
              Real delta "Dimensionless density";
              Real deltam1 "Dimensionless density";
              Real tau "Dimensionless temperature";
              Real taum1 "Dimensionless temperature";
              Real Psi0 "Auxiliary variable";
              Real Psi1 "Auxiliary variable";
              Real tfun "Auxiliary variable";
              Real rhofun "Auxiliary variable";
              Real Tc = T - 273.15 "Celsius temperature for region check";
              //      Integer region "Region of IF97";
            algorithm
//      if phase == 0 then
//        region := BaseIF97.Regions.region_dT(d,T,0);
//      end if;
//      if phase == 2 then
//        region := 4;
//      end if;
// assert(phase <> 2, "Viscosity can not be computed for two-phase states");
              delta := d / rhostar;
              assert(d > triple.dvtriple, "IF97 medium function visc_dTp for viscosity called with too low density\n" + "d = " + String(d) + " <= " + String(triple.dvtriple) + " (triple point density)");
              assert(p <= 500e6 and Tc >= 0.0 and Tc <= 150 or p <= 350e6 and Tc > 150.0 and Tc <= 600 or p <= 300e6 and Tc > 600.0 and Tc <= 900, "IF97 medium function visc_dTp: viscosity computed outside the range\n" + "of validity of the IF97 formulation: p = " + String(p) + " Pa, Tc = " + String(Tc) + " K");
              deltam1 := delta - 1.0;
              tau := tstar / T;
              taum1 := tau - 1.0;
              Psi0 := 1 / (n0 + (n1 + (n2 + n3 * tau) * tau) * tau) / tau ^ 0.5;
              Psi1 := 0.0;
              tfun := 1.0;
              for i in 1:6 loop
                if i <> 1 then
                  tfun := tfun * taum1;
                end if;
                rhofun := 1.;
                for j in 0:6 loop
                  if j <> 0 then
                    rhofun := rhofun * deltam1;
                  end if;
                  Psi1 := Psi1 + nn[i + j * 6] * tfun * rhofun;
                end for;
              end for;
              eta := etastar * Psi0 * Modelica.Math.exp(delta * Psi1);
            end visc_dTp;

            function cond_dTp "Thermal conductivity lam(d,T,p) (industrial use version) only in one-phase region"
              extends Modelica.Icons.Function;
              input SI.Density d "Density";
              input SI.Temperature T "Temperature (K)";
              input SI.Pressure p "Pressure";
              input Integer phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
              input Boolean industrialMethod = true "If true, the industrial method is used, otherwise the scientific one";
              output SI.ThermalConductivity lambda "Thermal conductivity";
            protected
              Integer region(min = 1, max = 5) "IF97 region, valid values:1,2,3, and 5";
              constant Real n0 = 1.0 "Conductivity coefficient";
              constant Real n1 = 6.978267 "Conductivity coefficient";
              constant Real n2 = 2.599096 "Conductivity coefficient";
              constant Real n3 = -0.998254 "Conductivity coefficient";
              constant Real[30] nn = array(1.3293046, 1.7018363, 5.2246158, 8.7127675, -1.8525999, -0.40452437, -2.2156845, -10.124111, -9.5000611, 0.9340469, 0.2440949, 1.6511057, 4.9874687, 4.3786606, 0.0, 0.018660751, -0.76736002, -0.27297694, -0.91783782, 0.0, -0.12961068, 0.37283344, -0.43083393, 0.0, 0.0, 0.044809953, -0.1120316, 0.13333849, 0.0, 0.0) "Conductivity coefficient";
              constant SI.ThermalConductivity lamstar = 0.4945 "Scaling conductivity";
              constant SI.Density rhostar = 317.763 "Scaling density";
              constant SI.Temperature tstar = 647.226 "Scaling temperature";
              constant SI.Pressure pstar = 22.115e6 "Scaling pressure";
              constant SI.DynamicViscosity etastar = 55.071e-6 "Scaling viscosity";
              Integer i "Auxiliary variable";
              Integer j "Auxiliary variable";
              Real delta "Dimensionless density";
              Real tau "Dimensionless temperature";
              Real deltam1 "Dimensionless density";
              Real taum1 "Dimensionless temperature";
              Real Lam0 "Part of thermal conductivity";
              Real Lam1 "Part of thermal conductivity";
              Real Lam2 "Part of thermal conductivity";
              Real tfun "Auxiliary variable";
              Real rhofun "Auxiliary variable";
              Real dpitau "Auxiliary variable";
              Real ddelpi "Auxiliary variable";
              Real d2 "Auxiliary variable";
              Modelica.Media.Common.GibbsDerivs g "Dimensionless Gibbs function and derivatives w.r.t. pi and tau";
              Modelica.Media.Common.HelmholtzDerivs f "Dimensionless Helmholtz function and derivatives w.r.t. delta and tau";
              Real Tc = T - 273.15 "Celsius temperature for region check";
              Real Chi "Symmetrized compressibility";
              // slightly different variables for industrial use
              constant SI.Density rhostar2 = 317.7 "Reference density";
              constant SI.Temperature Tstar2 = 647.25 "Reference temperature";
              constant SI.ThermalConductivity lambdastar = 1 "Reference thermal conductivity";
              Real TREL = T / Tstar2 "Relative temperature";
              Real rhoREL = d / rhostar2 "Relative density";
              Real lambdaREL "Relative thermal conductivity";
              Real deltaTREL "Relative temperature increment";
              constant Real[:] C = {0.642857, -4.11717, -6.17937, 0.00308976, 0.0822994, 10.0932};
              constant Real[:] dpar = {0.0701309, 0.0118520, 0.00169937, -1.0200};
              constant Real[:] b = {-0.397070, 0.400302, 1.060000};
              constant Real[:] B = {-0.171587, 2.392190};
              constant Real[:] a = {0.0102811, 0.0299621, 0.0156146, -0.00422464};
              Real Q;
              Real S;
              Real lambdaREL2 "Function, part of the interpolating equation of the thermal conductivity";
              Real lambdaREL1 "Function, part of the interpolating equation of the thermal conductivity";
              Real lambdaREL0 "Function, part of the interpolating equation of the thermal conductivity";
            algorithm
// region := BaseIF97.Regions.region_dT(d,T,phase);
// simplified region check, assuming that calling arguments are legal
//  assert(phase <> 2,
//   "ThermalConductivity can not be called with 2-phase inputs!");
              assert(d > triple.dvtriple, "IF97 medium function cond_dTp called with too low density\n" + "d = " + String(d) + " <= " + String(triple.dvtriple) + " (triple point density)");
              assert(p <= 100e6 and Tc >= 0.0 and Tc <= 500 or p <= 70e6 and Tc > 500.0 and Tc <= 650 or p <= 40e6 and Tc > 650.0 and Tc <= 800, "IF97 medium function cond_dTp: thermal conductivity computed outside the range\n" + "of validity of the IF97 formulation: p = " + String(p) + " Pa, Tc = " + String(Tc) + " K");
              if industrialMethod == true then
                deltaTREL := abs(TREL - 1) + C[4];
                Q := 2 + C[5] / deltaTREL ^ (3 / 5);
                if TREL >= 1 then
                  S := 1 / deltaTREL;
                else
                  S := C[6] / deltaTREL ^ (3 / 5);
                end if;
                lambdaREL2 := (dpar[1] / TREL ^ 10 + dpar[2]) * rhoREL ^ (9 / 5) * Modelica.Math.exp(C[1] * (1 - rhoREL ^ (14 / 5))) + dpar[3] * S * rhoREL ^ Q * Modelica.Math.exp(Q / (1 + Q) * (1 - rhoREL ^ (1 + Q))) + dpar[4] * Modelica.Math.exp(C[2] * TREL ^ (3 / 2) + C[3] / rhoREL ^ 5);
                lambdaREL1 := b[1] + b[2] * rhoREL + b[3] * Modelica.Math.exp(B[1] * (rhoREL + B[2]) ^ 2);
                lambdaREL0 := TREL ^ (1 / 2) * sum(a[i] * TREL ^ (i - 1) for i in 1:4);
                lambdaREL := lambdaREL0 + lambdaREL1 + lambdaREL2;
                lambda := lambdaREL * lambdastar;
              else
                if p < data.PLIMIT4A then
                  if d > data.DCRIT then
                    region := 1;
                  else
                    region := 2;
                  end if;
                else
                  assert(false, "The scientific method works only for temperature up to 623.15 K");
                end if;
                tau := tstar / T;
                delta := d / rhostar;
                deltam1 := delta - 1.0;
                taum1 := tau - 1.0;
                Lam0 := 1 / (n0 + (n1 + (n2 + n3 * tau) * tau) * tau) / tau ^ 0.5;
                Lam1 := 0.0;
                tfun := 1.0;
                for i in 1:5 loop
                  if i <> 1 then
                    tfun := tfun * taum1;
                  end if;
                  rhofun := 1.0;
                  for j in 0:5 loop
                    if j <> 0 then
                      rhofun := rhofun * deltam1;
                    end if;
                    Lam1 := Lam1 + nn[i + j * 5] * tfun * rhofun;
                  end for;
                end for;
                if region == 1 then
                  g := Basic.g1(p, T);
                  dpitau := -tstar / pstar * (data.PSTAR1 * (g.gpi - data.TSTAR1 / T * g.gtaupi) / g.gpipi / T);
                  ddelpi := -pstar / rhostar * data.RH2O / data.PSTAR1 / data.PSTAR1 * T * d * d * g.gpipi;
                  Chi := delta * ddelpi;
                elseif region == 2 then
                  g := Basic.g2(p, T);
                  dpitau := -tstar / pstar * (data.PSTAR2 * (g.gpi - data.TSTAR2 / T * g.gtaupi) / g.gpipi / T);
                  ddelpi := -pstar / rhostar * data.RH2O / data.PSTAR2 / data.PSTAR2 * T * d * d * g.gpipi;
                  Chi := delta * ddelpi;
                else
                  assert(false, "Thermal conductivity can only be called in the one-phase regions below 623.15 K\n" + "(p = " + String(p) + " Pa, T = " + String(T) + " K, region = " + String(region) + ")");
                end if;
                taum1 := 1 / tau - 1;
                d2 := deltam1 * deltam1;
                Lam2 := 0.0013848 * etastar / visc_dTp(d, T, p) / (tau * tau * delta * delta) * dpitau * dpitau * max(Chi, Modelica.Constants.small) ^ 0.4678 * delta ^ 0.5 * Modelica.Math.exp((-18.66 * taum1 * taum1) - d2 * d2);
                lambda := lamstar * (Lam0 * Modelica.Math.exp(delta * Lam1) + Lam2);
              end if;
//regions are 1 or 2,
//region is 3, or illegal
// dp/dT @ cont d = -g.p/g.T*(g.gpi - g.tau*g.gtaupi)/(g.gpipi*g.pi);
//         elseif (region == 3) then
//           f := Basic.f3(T, d);
//            dpitau := tstar/pstar*(f.R*f.d*f.delta*(f.fdelta - f.tau*f.fdeltatau));
//           ddelpi := pstar*d*d/(rhostar*p*p)/(f.R*f.T*f.delta*(2.0*f.fdelta + f.delta*f.fdeltadelta));
//    Chi := delta*ddelpi;
            end cond_dTp;

            function surfaceTension "Surface tension in region 4 between steam and water"
              extends Modelica.Icons.Function;
              input SI.Temperature T "Temperature (K)";
              output SI.SurfaceTension sigma "Surface tension in SI units";
            protected
              Real Theta "Dimensionless temperature";
            algorithm
              Theta := min(1.0, T / data.TCRIT);
              sigma := 235.8e-3 * (1 - Theta) ^ 1.256 * (1 - 0.625 * (1 - Theta));
            end surfaceTension;
            annotation(
              Documentation(info = "<HTML><h4>Package description</h4>
          <h4>Package contents</h4>
          <ul>
          <li>Function <b>visc_dTp</b> implements a function to compute the industrial formulation of the
          dynamic viscosity of water as a function of density and temperature.
          The details are described in the document <a href=\"modelica://Modelica/Resources/Documentation/Media/Water/IF97documentation/visc.pdf\">visc.pdf</a>.</li>
          <li>Function <b>cond_dTp</b> implements a function to compute  the industrial formulation of the thermal conductivity of water as
          a function of density, temperature and pressure. <b>Important note</b>: Obviously only two of the three
          inputs are really needed, but using three inputs speeds up the computation and the three variables are known in most models anyways.
          The inputs d,T and p have to be consistent.
          The details are described in the document <a href=\"modelica://Modelica/Resources/Documentation/Media/Water/IF97documentation/surf.pdf\">surf.pdf</a>.</li>
          <li>Function <b>surfaceTension</b> implements a function to compute the surface tension between vapour
          and liquid water as a function of temperature.
          The details are described in the document <a href=\"modelica://Modelica/Resources/Documentation/Media/Water/IF97documentation/thcond.pdf\">thcond.pdf</a>.</li>
          </ul>
          <h4>Version Info and Revision history
          </h4>
          <ul>
          <li>First implemented: <i>October, 2002</i>
          by <a href=\"http://www.control.lth.se/~hubertus/\">Hubertus Tummescheit</a>
          </li>
          </ul>
          <address>Authors: Hubertus Tummescheit and Jonas Eborn<br>
      Modelon AB<br>
      Ideon Science Park<br>
      SE-22370 Lund, Sweden<br>
      email: hubertus@modelon.se
          </address>
          <ul>
          <li>Initial version: October 2002</li>
          </ul>
          </html>"));
          end Transport;

          package Isentropic "Functions for calculating the isentropic enthalpy from pressure p and specific entropy s"
            extends Modelica.Icons.Package;

            function hofpT1 "Intermediate function for isentropic specific enthalpy in region 1"
              extends Modelica.Icons.Function;
              input SI.Pressure p "Pressure";
              input SI.Temperature T "Temperature (K)";
              output SI.SpecificEnthalpy h "Specific enthalpy";
            protected
              Real[13] o "Vector of auxiliary variables";
              Real pi1 "Dimensionless pressure";
              Real tau "Dimensionless temperature";
              Real tau1 "Dimensionless temperature";
            algorithm
              tau := data.TSTAR1 / T;
              pi1 := 7.1 - p / data.PSTAR1;
              assert(p > triple.ptriple, "IF97 medium function hofpT1 called with too low pressure\n" + "p = " + String(p) + " Pa <= " + String(triple.ptriple) + " Pa (triple point pressure)");
              tau1 := (-1.222) + tau;
              o[1] := tau1 * tau1;
              o[2] := o[1] * tau1;
              o[3] := o[1] * o[1];
              o[4] := o[3] * o[3];
              o[5] := o[1] * o[4];
              o[6] := o[1] * o[3];
              o[7] := o[3] * tau1;
              o[8] := o[3] * o[4];
              o[9] := pi1 * pi1;
              o[10] := o[9] * o[9];
              o[11] := o[10] * o[10];
              o[12] := o[4] * o[4];
              o[13] := o[12] * o[12];
              h := data.RH2O * T * tau * (pi1 * (((-0.00254871721114236) + o[1] * (0.00424944110961118 + (0.018990068218419 + ((-0.021841717175414) - 0.00015851507390979 * o[1]) * o[1]) * o[6])) / o[5] + pi1 * ((0.00141552963219801 + o[3] * (0.000047661393906987 + o[1] * ((-0.0000132425535992538) - 1.2358149370591e-14 * o[1] * o[3] * o[4]))) / o[3] + pi1 * ((0.000126718579380216 - 5.11230768720618e-9 * o[5]) / o[7] + pi1 * ((0.000011212640954 + o[2] * (1.30342445791202e-6 - 1.4341729937924e-12 * o[8])) / o[6] + pi1 * (o[9] * pi1 * ((1.40077319158051e-8 + 1.04549227383804e-9 * o[7]) / o[8] + o[10] * o[11] * pi1 * (1.9941018075704e-17 / (o[1] * o[12] * o[3] * o[4]) + o[9] * ((-4.48827542684151e-19 / o[13]) + o[10] * o[9] * (pi1 * (4.65957282962769e-22 / (o[13] * o[4]) + pi1 * (3.83502057899078e-24 * pi1 / (o[1] * o[13] * o[4]) - 7.2912378325616e-23 / (o[13] * o[4] * tau1))) - 1.00075970318621e-21 / (o[1] * o[13] * o[3] * tau1))))) + 3.24135974880936e-6 / (o[4] * tau1)))))) + ((-0.29265942426334) + tau1 * (0.84548187169114 + o[1] * (3.3855169168385 + tau1 * ((-1.91583926775744) + tau1 * (0.47316115539684 + ((-0.066465668798004) + 0.0040607314991784 * tau1) * tau1))))) / o[2]);
            end hofpT1;

            function handsofpT1 "Special function for specific enthalpy and specific entropy in region 1"
              extends Modelica.Icons.Function;
              input SI.Pressure p "Pressure";
              input SI.Temperature T "Temperature (K)";
              output SI.SpecificEnthalpy h "Specific enthalpy";
              output SI.SpecificEntropy s "Specific entropy";
            protected
              Real[28] o "Vector of auxiliary variables";
              Real pi1 "Dimensionless pressure";
              Real tau "Dimensionless temperature";
              Real tau1 "Dimensionless temperature";
              Real g "Dimensionless Gibbs energy";
              Real gtau "Derivative of dimensionless Gibbs energy w.r.t. tau";
            algorithm
              assert(p > triple.ptriple, "IF97 medium function handsofpT1 called with too low pressure\n" + "p = " + String(p) + " Pa <= " + String(triple.ptriple) + " Pa (triple point pressure)");
              tau := data.TSTAR1 / T;
              pi1 := 7.1 - p / data.PSTAR1;
              tau1 := (-1.222) + tau;
              o[1] := tau1 * tau1;
              o[2] := o[1] * o[1];
              o[3] := o[2] * o[2];
              o[4] := o[3] * tau1;
              o[5] := 1 / o[4];
              o[6] := o[1] * o[2];
              o[7] := o[1] * tau1;
              o[8] := 1 / o[7];
              o[9] := o[1] * o[2] * o[3];
              o[10] := 1 / o[2];
              o[11] := o[2] * tau1;
              o[12] := 1 / o[11];
              o[13] := o[2] * o[3];
              o[14] := pi1 * pi1;
              o[15] := o[14] * pi1;
              o[16] := o[14] * o[14];
              o[17] := o[16] * o[16];
              o[18] := o[16] * o[17] * pi1;
              o[19] := o[14] * o[16];
              o[20] := o[3] * o[3];
              o[21] := o[20] * o[20];
              o[22] := o[21] * o[3] * tau1;
              o[23] := 1 / o[22];
              o[24] := o[21] * o[3];
              o[25] := 1 / o[24];
              o[26] := o[1] * o[2] * o[21] * tau1;
              o[27] := 1 / o[26];
              o[28] := o[1] * o[3];
              g := pi1 * (pi1 * (pi1 * (o[10] * ((-0.000031679644845054) + o[2] * ((-2.8270797985312e-6) - 8.5205128120103e-10 * o[6])) + pi1 * (o[12] * ((-2.2425281908e-6) + ((-6.5171222895601e-7) - 1.4341729937924e-13 * o[13]) * o[7]) + pi1 * ((-4.0516996860117e-7 / o[3]) + o[15] * (o[18] * (o[14] * (o[19] * (2.6335781662795e-23 / (o[1] * o[2] * o[21]) + pi1 * ((-1.1947622640071e-23 * o[27]) + pi1 * (1.8228094581404e-24 * o[25] - 9.3537087292458e-26 * o[23] * pi1))) + 1.4478307828521e-20 / (o[1] * o[2] * o[20] * o[3] * tau1)) - 6.8762131295531e-19 / (o[2] * o[20] * o[3] * tau1)) + ((-1.2734301741641e-9) - 1.7424871230634e-10 * o[11]) / (o[1] * o[3] * tau1))))) + o[8] * ((-0.00047184321073267) + o[7] * ((-0.00030001780793026) + (0.000047661393906987 + o[1] * ((-4.4141845330846e-6) - 7.2694996297594e-16 * o[9])) * tau1))) + o[5] * (0.00028319080123804 + o[1] * ((-0.00060706301565874) + o[6] * ((-0.018990068218419) + tau1 * ((-0.032529748770505) + ((-0.021841717175414) - 0.00005283835796993 * o[1]) * tau1))))) + (0.14632971213167 + tau1 * ((-0.84548187169114) + tau1 * ((-3.756360367204) + tau1 * (3.3855169168385 + tau1 * ((-0.95791963387872) + tau1 * (0.15772038513228 + ((-0.016616417199501) + 0.00081214629983568 * tau1) * tau1)))))) / o[1];
              gtau := pi1 * (((-0.00254871721114236) + o[1] * (0.00424944110961118 + (0.018990068218419 + ((-0.021841717175414) - 0.00015851507390979 * o[1]) * o[1]) * o[6])) / o[28] + pi1 * (o[10] * (0.00141552963219801 + o[2] * (0.000047661393906987 + o[1] * ((-0.0000132425535992538) - 1.2358149370591e-14 * o[9]))) + pi1 * (o[12] * (0.000126718579380216 - 5.11230768720618e-9 * o[28]) + pi1 * ((0.000011212640954 + (1.30342445791202e-6 - 1.4341729937924e-12 * o[13]) * o[7]) / o[6] + pi1 * (3.24135974880936e-6 * o[5] + o[15] * ((1.40077319158051e-8 + 1.04549227383804e-9 * o[11]) / o[13] + o[18] * (1.9941018075704e-17 / (o[1] * o[2] * o[20] * o[3]) + o[14] * ((-4.48827542684151e-19 / o[21]) + o[19] * ((-1.00075970318621e-21 * o[27]) + pi1 * (4.65957282962769e-22 * o[25] + pi1 * ((-7.2912378325616e-23 * o[23]) + 3.83502057899078e-24 * pi1 / (o[1] * o[21] * o[3])))))))))))) + o[8] * ((-0.29265942426334) + tau1 * (0.84548187169114 + o[1] * (3.3855169168385 + tau1 * ((-1.91583926775744) + tau1 * (0.47316115539684 + ((-0.066465668798004) + 0.0040607314991784 * tau1) * tau1)))));
              h := data.RH2O * T * tau * gtau;
              s := data.RH2O * (tau * gtau - g);
            end handsofpT1;

            function hofps1 "Function for isentropic specific enthalpy in region 1"
              extends Modelica.Icons.Function;
              input SI.Pressure p "Pressure";
              input SI.SpecificEntropy s "Specific entropy";
              output SI.SpecificEnthalpy h "Specific enthalpy";
            protected
              SI.Temperature T "Temperature (K)";
            algorithm
              T := Basic.tps1(p, s);
              h := hofpT1(p, T);
            end hofps1;

            // for isentropic specific enthalpy get T(p,s), then use this

            function hofpT2 "Intermediate function for isentropic specific enthalpy in region 2"
              extends Modelica.Icons.Function;
              input SI.Pressure p "Pressure";
              input SI.Temperature T "Temperature (K)";
              output SI.SpecificEnthalpy h "Specific enthalpy";
            protected
              Real[16] o "Vector of auxiliary variables";
              Real pi "Dimensionless pressure";
              Real tau "Dimensionless temperature";
              Real tau2 "Dimensionless temperature";
            algorithm
              assert(p > triple.ptriple, "IF97 medium function hofpT2 called with too low pressure\n" + "p = " + String(p) + " Pa <= " + String(triple.ptriple) + " Pa (triple point pressure)");
              pi := p / data.PSTAR2;
              tau := data.TSTAR2 / T;
              tau2 := (-0.5) + tau;
              o[1] := tau * tau;
              o[2] := o[1] * o[1];
              o[3] := tau2 * tau2;
              o[4] := o[3] * tau2;
              o[5] := o[3] * o[3];
              o[6] := o[5] * o[5];
              o[7] := o[6] * o[6];
              o[8] := o[5] * o[6] * o[7] * tau2;
              o[9] := o[3] * o[5];
              o[10] := o[5] * o[6] * tau2;
              o[11] := o[3] * o[7] * tau2;
              o[12] := o[3] * o[5] * o[6];
              o[13] := o[5] * o[6] * o[7];
              o[14] := pi * pi;
              o[15] := o[14] * o[14];
              o[16] := o[7] * o[7];
              h := data.RH2O * T * tau * ((0.0280439559151 + tau * ((-0.2858109552582) + tau * (1.2213149471784 + tau * ((-2.848163942888) + tau * (4.38395111945 + o[1] * (10.08665568018 + ((-0.5681726521544) + 0.06380539059921 * tau) * tau)))))) / (o[1] * o[2]) + pi * ((-0.017834862292358) + tau2 * ((-0.09199202739273) + ((-0.172743777250296) - 0.30195167236758 * o[4]) * tau2) + pi * ((-0.000033032641670203) + ((-0.0003789797503263) + o[3] * ((-0.015757110897342) + o[4] * ((-0.306581069554011) - 0.000960283724907132 * o[8]))) * tau2 + pi * (4.3870667284435e-7 + o[3] * ((-0.00009683303171571) + o[4] * ((-0.0090203547252888) - 1.42338887469272 * o[8])) + pi * ((-7.8847309559367e-10) + (2.558143570457e-8 + 1.44676118155521e-6 * tau2) * tau2 + pi * (0.0000160454534363627 * o[9] + pi * (((-5.0144299353183e-11) + o[10] * ((-0.033874355714168) - 836.35096769364 * o[11])) * o[3] + pi * (((-0.0000138839897890111) - 0.973671060893475 * o[12]) * o[3] * o[6] + pi * ((9.0049690883672e-11 - 296.320827232793 * o[13]) * o[3] * o[5] * tau2 + pi * (2.57526266427144e-7 * o[5] * o[6] + pi * (o[4] * (4.1627860840696e-19 + ((-1.0234747095929e-12) - 1.40254511313154e-8 * o[5]) * o[9]) + o[14] * o[15] * (o[13] * ((-2.34560435076256e-9) + 5.3465159397045 * o[5] * o[7] * tau2) + o[14] * ((-19.1874828272775 * o[16] * o[6] * o[7]) + o[14] * (o[11] * (1.78371690710842e-23 + (1.07202609066812e-11 - 0.000201611844951398 * o[10]) * o[3] * o[5] * o[6] * tau2) + pi * ((-1.24017662339842e-24 * o[5] * o[7]) + pi * (0.000200482822351322 * o[16] * o[5] * o[7] + pi * ((-4.97975748452559e-14 * o[16] * o[3] * o[5]) + o[6] * o[7] * (1.90027787547159e-27 + o[12] * (2.21658861403112e-15 - 0.0000547344301999018 * o[3] * o[7])) * pi * tau2)))))))))))))))));
            end hofpT2;

            function handsofpT2 "Function for isentropic specific enthalpy and specific entropy in region 2"
              extends Modelica.Icons.Function;
              input SI.Pressure p "Pressure";
              input SI.Temperature T "Temperature (K)";
              output SI.SpecificEnthalpy h "Specific enthalpy";
              output SI.SpecificEntropy s "Specific entropy";
            protected
              Real[22] o "Vector of auxiliary variables";
              Real pi "Dimensionless pressure";
              Real tau "Dimensionless temperature";
              Real tau2 "Dimensionless temperature";
              Real g "Dimensionless Gibbs energy";
              Real gtau "Derivative of dimensionless Gibbs energy w.r.t. tau";
            algorithm
              assert(p > triple.ptriple, "IF97 medium function handsofpT2 called with too low pressure\n" + "p = " + String(p) + " Pa <= " + String(triple.ptriple) + " Pa (triple point pressure)");
              tau := data.TSTAR2 / T;
              pi := p / data.PSTAR2;
              tau2 := tau - 0.5;
              o[1] := tau2 * tau2;
              o[2] := o[1] * tau2;
              o[3] := o[1] * o[1];
              o[4] := o[3] * o[3];
              o[5] := o[4] * o[4];
              o[6] := o[3] * o[4] * o[5] * tau2;
              o[7] := o[1] * o[3] * tau2;
              o[8] := o[3] * o[4] * tau2;
              o[9] := o[1] * o[5] * tau2;
              o[10] := o[1] * o[3] * o[4];
              o[11] := o[3] * o[4] * o[5];
              o[12] := o[1] * o[3];
              o[13] := pi * pi;
              o[14] := o[13] * o[13];
              o[15] := o[13] * o[14];
              o[16] := o[3] * o[5] * tau2;
              o[17] := o[5] * o[5];
              o[18] := o[3] * o[5];
              o[19] := o[1] * o[3] * o[4] * tau2;
              o[20] := o[1] * o[5];
              o[21] := tau * tau;
              o[22] := o[21] * o[21];
              g := pi * ((-0.0017731742473213) + tau2 * ((-0.017834862292358) + tau2 * ((-0.045996013696365) + ((-0.057581259083432) - 0.05032527872793 * o[2]) * tau2)) + pi * (tau2 * ((-0.000033032641670203) + ((-0.00018948987516315) + o[1] * ((-0.0039392777243355) + o[2] * ((-0.043797295650573) - 0.000026674547914087 * o[6]))) * tau2) + pi * (2.0481737692309e-8 + (4.3870667284435e-7 + o[1] * ((-0.00003227767723857) + o[2] * ((-0.0015033924542148) - 0.040668253562649 * o[6]))) * tau2 + pi * (tau2 * ((-7.8847309559367e-10) + (1.2790717852285e-8 + 4.8225372718507e-7 * tau2) * tau2) + pi * (2.2922076337661e-6 * o[7] + pi * (o[2] * ((-1.6714766451061e-11) + o[8] * ((-0.0021171472321355) - 23.895741934104 * o[9])) + pi * ((-5.905956432427e-18) + o[1] * ((-1.2621808899101e-6) - 0.038946842435739 * o[10]) * o[4] * tau2 + pi * ((1.1256211360459e-11 - 8.2311340897998 * o[11]) * o[4] + pi * (1.9809712802088e-8 * o[8] + pi * ((1.0406965210174e-19 + o[12] * ((-1.0234747095929e-13) - 1.0018179379511e-9 * o[3])) * o[3] + o[15] * (((-8.0882908646985e-11) + 0.10693031879409 * o[16]) * o[6] + o[13] * ((-0.33662250574171 * o[17] * o[4] * o[5] * tau2) + o[13] * (o[18] * (8.9185845355421e-25 + o[19] * (3.0629316876232e-13 - 4.2002467698208e-6 * o[8])) + pi * ((-5.9056029685639e-26 * o[16]) + pi * (3.7826947613457e-6 * o[17] * o[3] * o[5] * tau2 + pi * (o[1] * (7.3087610595061e-29 + o[10] * (5.5414715350778e-17 - 9.436970724121e-7 * o[20])) * o[4] * o[5] * pi - 1.2768608934681e-15 * o[1] * o[17] * o[3] * tau2)))))))))))))))) + ((-0.00560879118302) + tau * (0.07145273881455 + tau * ((-0.4071049823928) + tau * (1.424081971444 + tau * ((-4.38395111945) + tau * ((-9.692768600217) + tau * (10.08665568018 + ((-0.2840863260772) + 0.02126846353307 * tau) * tau) + Modelica.Math.log(pi))))))) / (o[22] * tau);
              gtau := (0.0280439559151 + tau * ((-0.2858109552582) + tau * (1.2213149471784 + tau * ((-2.848163942888) + tau * (4.38395111945 + o[21] * (10.08665568018 + ((-0.5681726521544) + 0.06380539059921 * tau) * tau)))))) / (o[21] * o[22]) + pi * ((-0.017834862292358) + tau2 * ((-0.09199202739273) + ((-0.172743777250296) - 0.30195167236758 * o[2]) * tau2) + pi * ((-0.000033032641670203) + ((-0.0003789797503263) + o[1] * ((-0.015757110897342) + o[2] * ((-0.306581069554011) - 0.000960283724907132 * o[6]))) * tau2 + pi * (4.3870667284435e-7 + o[1] * ((-0.00009683303171571) + o[2] * ((-0.0090203547252888) - 1.42338887469272 * o[6])) + pi * ((-7.8847309559367e-10) + (2.558143570457e-8 + 1.44676118155521e-6 * tau2) * tau2 + pi * (0.0000160454534363627 * o[12] + pi * (o[1] * ((-5.0144299353183e-11) + o[8] * ((-0.033874355714168) - 836.35096769364 * o[9])) + pi * (o[1] * ((-0.0000138839897890111) - 0.973671060893475 * o[10]) * o[4] + pi * ((9.0049690883672e-11 - 296.320827232793 * o[11]) * o[7] + pi * (2.57526266427144e-7 * o[3] * o[4] + pi * (o[2] * (4.1627860840696e-19 + o[12] * ((-1.0234747095929e-12) - 1.40254511313154e-8 * o[3])) + o[15] * (o[11] * ((-2.34560435076256e-9) + 5.3465159397045 * o[16]) + o[13] * ((-19.1874828272775 * o[17] * o[4] * o[5]) + o[13] * ((1.78371690710842e-23 + o[19] * (1.07202609066812e-11 - 0.000201611844951398 * o[8])) * o[9] + pi * ((-1.24017662339842e-24 * o[18]) + pi * (0.000200482822351322 * o[17] * o[3] * o[5] + pi * ((-4.97975748452559e-14 * o[1] * o[17] * o[3]) + (1.90027787547159e-27 + o[10] * (2.21658861403112e-15 - 0.0000547344301999018 * o[20])) * o[4] * o[5] * pi * tau2))))))))))))))));
              h := data.RH2O * T * tau * gtau;
              s := data.RH2O * (tau * gtau - g);
            end handsofpT2;

            function hofps2 "Function for isentropic specific enthalpy in region 2"
              extends Modelica.Icons.Function;
              input SI.Pressure p "Pressure";
              input SI.SpecificEntropy s "Specific entropy";
              output SI.SpecificEnthalpy h "Specific enthalpy";
            protected
              SI.Temperature T "Temperature (K)";
            algorithm
              T := Basic.tps2(p, s);
              h := hofpT2(p, T);
            end hofps2;

            // region 3 extra functions
            // for isentropic specific enthalpy get (d,T) = f(p,s), then use this
            // which needs a bloody iteration, ...
            // this is one thing that needs to be done somehow, ...

            function hofdT3 "Function for isentropic specific enthalpy in region 3"
              extends Modelica.Icons.Function;
              input SI.Density d "Density";
              input SI.Temperature T "Temperature (K)";
              output SI.SpecificEnthalpy h "Specific enthalpy";
            protected
              Real delta;
              Real tau "Dimensionless temperature";
              Real[13] o "Vector of auxiliary variables";
              Real ftau "Derivative of dimensionless Helmholtz energy w.r.t. tau";
              Real fdelta "Derivative of dimensionless Helmholtz energy w.r.t. delta";
            algorithm
              tau := data.TCRIT / T;
              delta := d / data.DCRIT;
              o[1] := tau * tau;
              o[2] := o[1] * o[1];
              o[3] := o[2] * o[2];
              o[4] := o[3] * tau;
              o[5] := o[1] * o[2] * o[3] * tau;
              o[6] := o[2] * o[3];
              o[7] := o[1] * o[3];
              o[8] := o[3] * o[3];
              o[9] := o[1] * o[2] * o[8];
              o[10] := o[1] * o[2] * o[8] * tau;
              o[11] := o[3] * o[8];
              o[12] := o[3] * o[8] * tau;
              o[13] := o[1] * o[3] * o[8];
              ftau := 20.944396974307 + tau * ((-15.3735415757432) + o[2] * tau * (18.3301634515678 + o[1] * tau * ((-28.08078114862) + o[1] * (14.4640436358204 - 0.194503669468755 * o[1] * o[3] * tau)))) + delta * (((-2.5308630955428) + o[2] * ((-6.9146446840086) + (13.2781565976477 - 10.9153200808732 * o[1]) * o[4])) * tau + delta * (tau * ((-1.70429417648412) + o[2] * (29.3833689251262 + ((-21.3518320798755) + (0.867251811341388 + 3.26518619032008 * o[2]) * o[5]) * tau)) + delta * ((2.779959913892 + o[1] * ((-8.075966009428) + o[6] * ((-0.131436219478341) - 12.37496929108 * o[7]))) * tau + delta * (((-0.88952870857478) + o[1] * (3.62288282878932 + 18.3358370228714 * o[9])) * tau + delta * (0.10770512626332 + o[1] * ((-0.98740869776862) - 13.2264761307011 * o[10]) + delta * ((0.188521503330184 + 4.27343239646986 * o[11]) * tau + delta * ((-0.027006744482696 * tau) + delta * ((-0.385692979164272 * o[12]) + delta * (delta * ((-0.00016557679795037) - 0.00116802137560719 * delta * o[12]) + (0.00115845907256168 + 0.0840031522296486 * o[11]) * tau)))))))));
              fdelta := (1.0658070028513 + delta * (o[1] * ((-1.2654315477714) + o[2] * ((-1.1524407806681) + (0.88521043984318 - 0.64207765181607 * o[1]) * o[4])) + delta * (0.76986920373342 + o[1] * ((-1.70429417648412) + o[2] * (9.7944563083754 + ((-6.100523451393) + (0.078841073758308 + 0.25116816848616 * o[2]) * o[5]) * tau)) + delta * ((-0.8399798909613) + o[1] * (4.169939870838 + o[1] * ((-6.056974507071) + o[6] * ((-0.0246442911521889) - 1.42788107204769 * o[7]))) + delta * (0.175936297894 + o[1] * ((-1.77905741714956) + o[1] * (3.62288282878932 + 2.82089800351868 * o[9])) + delta * (delta * ((-0.133052405238576) + o[1] * (0.565564509990552 + 0.98617670687766 * o[11]) + delta * ((-0.094523605689436 * o[1]) + delta * ((-0.118674762819776 * o[13]) + delta * (o[1] * (0.00521306582652756 + 0.0290780142333399 * o[11]) + delta * (0.00080964802996215 - 0.000494162889679965 * delta * o[13] - 0.0016557679795037 * tau))))) + (0.5385256313166 + o[1] * ((-1.6456811629477) - 2.5435531020579 * o[10])) * tau)))))) / delta;
              h := data.RH2O * T * (tau * ftau + delta * fdelta);
            end hofdT3;

            function hofps3 "Isentropic specific enthalpy in region 3 h(p,s)"
              extends Modelica.Icons.Function;
              input SI.Pressure p "Pressure";
              input SI.SpecificEntropy s "Specific entropy";
              output SI.SpecificEnthalpy h "Specific enthalpy";
            protected
              SI.Density d "Density";
              SI.Temperature T "Temperature (K)";
              SI.Pressure delp = IterationData.DELP "Iteration accuracy";
              SI.SpecificEntropy dels = IterationData.DELS "Iteration accuracy";
              Integer error "Error if not 0";
            algorithm
              (d, T, error) := Inverses.dtofps3(p = p, s = s, delp = delp, dels = dels);
              h := hofdT3(d, T);
            end hofps3;

            function hofpsdt3 "Isentropic specific enthalpy in region 3 h(p,s) with given good guess in d and T"
              extends Modelica.Icons.Function;
              input SI.Pressure p "Pressure";
              input SI.SpecificEntropy s "Specific entropy";
              input SI.Density dguess "Good guess density, e.g., from adjacent volume";
              input SI.Temperature Tguess "Good guess temperature, e.g., from adjacent volume";
              input SI.Pressure delp = IterationData.DELP "Relative error in p";
              input SI.SpecificEntropy dels = IterationData.DELS "Relative error in s";
              output SI.SpecificEnthalpy h "Specific enthalpy";
            protected
              SI.Density d "Density";
              SI.Temperature T "Temperature (K)";
              Integer error "Error flag";
            algorithm
              (d, T, error) := Inverses.dtofpsdt3(p = p, s = s, dguess = dguess, Tguess = Tguess, delp = delp, dels = dels);
              h := hofdT3(d, T);
            end hofpsdt3;

            function hofps4 "Isentropic specific enthalpy in region 4 h(p,s)"
              extends Modelica.Icons.Function;
              input SI.Pressure p "Pressure";
              input SI.SpecificEntropy s "Specific entropy";
              output SI.SpecificEnthalpy h "Specific enthalpy";
            protected
              SI.Temp_K Tsat "Saturation temperature";
              SI.MassFraction x "Dryness fraction";
              SI.SpecificEntropy sl "Saturated liquid specific entropy";
              SI.SpecificEntropy sv "Saturated vapour specific entropy";
              SI.SpecificEnthalpy hl "Saturated liquid specific enthalpy";
              SI.SpecificEnthalpy hv "Saturated vapour specific enthalpy";
            algorithm
              if p <= data.PLIMIT4A then
                Tsat := Basic.tsat(p);
                (hl, sl) := handsofpT1(p, Tsat);
                (hv, sv) := handsofpT2(p, Tsat);
              elseif p < data.PCRIT then
                sl := Regions.sl_p_R4b(p);
                sv := Regions.sv_p_R4b(p);
                hl := Regions.hl_p_R4b(p);
                hv := Regions.hv_p_R4b(p);
              end if;
              x := max(min(if sl <> sv then (s - sl) / (sv - sl) else 1.0, 1.0), 0.0);
              h := hl + x * (hv - hl);
            end hofps4;

            function hofpT5 "Specific enthalpy in region 5 h(p,T)"
              extends Modelica.Icons.Function;
              input SI.Pressure p "Pressure";
              input SI.Temperature T "Temperature (K)";
              output SI.SpecificEnthalpy h "Specific enthalpy";
            protected
              Real[4] o "Vector of auxiliary variables";
              Real tau "Dimensionless temperature";
              Real pi "Dimensionless pressure";
            algorithm
              tau := data.TSTAR5 / T;
              pi := p / data.PSTAR5;
              assert(p > triple.ptriple, "IF97 medium function hofpT5 called with too low pressure\n" + "p = " + String(p) + " Pa <= " + String(triple.ptriple) + " Pa (triple point pressure)");
              o[1] := tau * tau;
              o[2] := o[1] * o[1];
              o[3] := pi * pi;
              o[4] := o[2] * o[2];
              h := data.RH2O * T * tau * (6.8540841634434 + 3.1161318213925 / o[1] + 0.074415446800398 / o[2] - 0.0000357523455236121 * o[3] * o[4] + 0.0021774678714571 * pi - 0.013782846269973 * o[1] * pi + 3.8757684869352e-7 * o[1] * o[3] * pi - 0.73803069960666 / (o[1] * tau) - 0.65923253077834 * tau);
            end hofpT5;

            function water_hisentropic "Isentropic specific enthalpy from p,s (preferably use water_hisentropic_dyn in dynamic simulation!)"
              extends Modelica.Icons.Function;
              input SI.Pressure p "Pressure";
              input SI.SpecificEntropy s "Specific entropy";
              input Integer phase = 0 "Phase: 2 for two-phase, 1 for one phase, 0 if unknown";
              output SI.SpecificEnthalpy h "Specific enthalpy";
            protected
              Modelica.Media.Common.GibbsDerivs g "Derivatives of dimensionless Gibbs-function w.r.t. dimensionless pi and tau";
              Modelica.Media.Common.HelmholtzDerivs f "Derivatives of dimensionless Helmholtz-function w.r.t. dimensionless delta and tau";
              Integer region(min = 1, max = 5) "IF97 region";
              Integer error "Error if not 0";
              SI.Temperature T "Temperature";
              SI.Density d "Density";
            algorithm
              region := Regions.region_ps(p = p, s = s, phase = phase);
              if region == 1 then
                h := hofps1(p, s);
              elseif region == 2 then
                h := hofps2(p, s);
              elseif region == 3 then
                (d, T, error) := Inverses.dtofps3(p = p, s = s, delp = IterationData.DELP, dels = IterationData.DELS);
                h := hofdT3(d, T);
              elseif region == 4 then
                h := hofps4(p, s);
              elseif region == 5 then
                (T, error) := Inverses.tofps5(p = p, s = s, relds = IterationData.DELS);
                h := hofpT5(p, T);
              end if;
            end water_hisentropic;

            function water_hisentropic_dyn "Isentropic specific enthalpy from p,s and good guesses of d and T"
              extends Modelica.Icons.Function;
              input SI.Pressure p "Pressure";
              input SI.SpecificEntropy s "Specific entropy";
              input SI.Density dguess "Good guess density, e.g., from adjacent volume";
              input SI.Temperature Tguess "Good guess temperature, e.g., from adjacent volume";
              input Integer phase "1 for one phase, 2 for two phase";
              output SI.SpecificEnthalpy h "Specific enthalpy";
            protected
              Modelica.Media.Common.GibbsDerivs g "Derivatives of dimensionless Gibbs-function w.r.t. dimensionless pi and tau";
              Modelica.Media.Common.HelmholtzDerivs f "Derivatives of dimensionless Helmholtz-function w.r.t. dimensionless delta and tau";
              Integer region(min = 1, max = 5) "IF97 region";
              Integer error "Error if not 0";
              SI.Temperature T "Temperature";
              SI.Density d "Density";
            algorithm
              region := Regions.region_ps(p = p, s = s, phase = phase);
              if region == 1 then
                h := hofps1(p, s);
              elseif region == 2 then
                h := hofps2(p, s);
              elseif region == 3 then
                h := hofpsdt3(p = p, s = s, dguess = dguess, Tguess = Tguess, delp = IterationData.DELP, dels = IterationData.DELS);
              elseif region == 4 then
                h := hofps4(p, s);
              elseif region == 5 then
                (T, error) := Inverses.tofpst5(p = p, s = s, Tguess = Tguess, relds = IterationData.DELS);
                h := hofpT5(p, T);
              end if;
            end water_hisentropic_dyn;
            annotation(
              Documentation(info = "<HTML><h4>Package description</h4>
          <h4>Package contents</h4>
          <ul>
          <li>Function <b>hofpT1</b> computes h(p,T) in region 1.</li>
          <li>Function <b>handsofpT1</b> computes (s,h)=f(p,T) in region 1, needed for two-phase properties.</li>
          <li>Function <b>hofps1</b> computes h(p,s) in region 1.</li>
          <li>Function <b>hofpT2</b> computes h(p,T) in region 2.</li>
          <li>Function <b>handsofpT2</b> computes (s,h)=f(p,T) in region 2, needed for two-phase properties.</li>
          <li>Function <b>hofps2</b> computes h(p,s) in region 2.</li>
          <li>Function <b>hofdT3</b> computes h(d,T) in region 3.</li>
          <li>Function <b>hofpsdt3</b> computes h(p,s,dguess,Tguess) in region 3, where dguess and Tguess are initial guess
          values for the density and temperature consistent with p and s.</li>
          <li>Function <b>hofps4</b> computes h(p,s) in region 4.</li>
          <li>Function <b>hofpT5</b> computes h(p,T) in region 5.</li>
          <li>Function <b>water_hisentropic</b> computes h(p,s,phase) in all regions.
          The phase input is needed due to discontinuous derivatives at the phase boundary.</li>
          <li>Function <b>water_hisentropic_dyn</b> computes h(p,s,dguess,Tguess,phase) in all regions.
          The phase input is needed due to discontinuous derivatives at the phase boundary. Tguess and dguess are initial guess
          values for the density and temperature consistent with p and s. This function should be preferred in
          dynamic simulations where good guesses are often available.</li>
          </ul>
          <h4>Version Info and Revision history
          </h4>
          <ul>
          <li>First implemented: <i>July, 2000</i>
          by <a href=\"http://www.control.lth.se/~hubertus/\">Hubertus Tummescheit</a>
          </li>
          </ul>
          <address>Author: Hubertus Tummescheit, <br>
      Modelon AB<br>
      Ideon Science Park<br>
      SE-22370 Lund, Sweden<br>
      email: hubertus@modelon.se
          </address>
          <ul>
          <li>Initial version: July 2000</li>
          <li>Documentation added: December 2002</li>
          </ul>
          </html>"));
          end Isentropic;

          //work needed: (Pr,lam,eta) = f(d,T,p, region?)

          package Inverses "Efficient inverses for selected pairs of variables"
            extends Modelica.Icons.Package;
            //===================================================================
            //            Iterative version for some pairs/regions
            //===================================================================

            function fixdT "Region limits for inverse iteration in region 3"
              extends Modelica.Icons.Function;
              input SI.Density din "Density";
              input SI.Temperature Tin "Temperature";
              output SI.Density dout "Density";
              output SI.Temperature Tout "Temperature";
            protected
              SI.Temperature Tmin "Approximation of minimum temperature";
              SI.Temperature Tmax "Approximation of maximum temperature";
            algorithm
              if din > 765.0 then
                dout := 765.0;
              elseif din < 110.0 then
                dout := 110.0;
              else
                dout := din;
              end if;
              if dout < 390.0 then
                Tmax := 554.3557377 + dout * 0.809344262;
              else
                Tmax := 1116.85 - dout * 0.632948717;
              end if;
              if dout < data.DCRIT then
                Tmin := data.TCRIT * (1.0 - (dout - data.DCRIT) * (dout - data.DCRIT) / 1.0e6);
              else
                Tmin := data.TCRIT * (1.0 - (dout - data.DCRIT) * (dout - data.DCRIT) / 1.44e6);
              end if;
              if Tin < Tmin then
                Tout := Tmin;
              elseif Tin > Tmax then
                Tout := Tmax;
              else
                Tout := Tin;
              end if;
            end fixdT;

            function dofp13 "Density at the boundary between regions 1 and 3"
              extends Modelica.Icons.Function;
              input SI.Pressure p "Pressure";
              output SI.Density d "Density";
            protected
              Real p2 "Auxiliary variable";
              Real[3] o "Vector of auxiliary variables";
            algorithm
              p2 := 7.1 - 6.04960677555959e-8 * p;
              o[1] := p2 * p2;
              o[2] := o[1] * o[1];
              o[3] := o[2] * o[2];
              d := 57.4756752485113 / (0.0737412153522555 + p2 * (0.00145092247736023 + p2 * (0.000102697173772229 + p2 * (0.0000114683182476084 + p2 * (1.99080616601101e-6 + o[1] * p2 * (1.13217858826367e-8 + o[2] * o[3] * p2 * (1.35549330686006e-17 + o[1] * ((-3.11228834832975e-19) + o[1] * o[2] * ((-7.02987180039442e-22) + p2 * (3.29199117056433e-22 + ((-5.17859076694812e-23) + 2.73712834080283e-24 * p2) * p2))))))))));
            end dofp13;

            function dofp23 "Density at the boundary between regions 2 and 3"
              extends Modelica.Icons.Function;
              input SI.Pressure p "Pressure";
              output SI.Density d "Density";
            protected
              SI.Temperature T;
              Real[13] o "Vector of auxiliary variables";
              Real taug "Auxiliary variable";
              Real pi "Dimensionless pressure";
              Real gpi23 "Derivative of g w.r.t. pi on the boundary between regions 2 and 3";
            algorithm
              pi := p / data.PSTAR2;
              T := 572.54459862746 + 31.3220101646784 * ((-13.91883977887) + pi) ^ 0.5;
              o[1] := ((-13.91883977887) + pi) ^ 0.5;
              taug := (-0.5) + 540.0 / (572.54459862746 + 31.3220101646784 * o[1]);
              o[2] := taug * taug;
              o[3] := o[2] * taug;
              o[4] := o[2] * o[2];
              o[5] := o[4] * o[4];
              o[6] := o[5] * o[5];
              o[7] := o[4] * o[5] * o[6] * taug;
              o[8] := o[4] * o[5] * taug;
              o[9] := o[2] * o[4] * o[5];
              o[10] := pi * pi;
              o[11] := o[10] * o[10];
              o[12] := o[4] * o[6] * taug;
              o[13] := o[6] * o[6];
              gpi23 := (1.0 + pi * ((-0.0017731742473213) + taug * ((-0.017834862292358) + taug * ((-0.045996013696365) + ((-0.057581259083432) - 0.05032527872793 * o[3]) * taug)) + pi * (taug * ((-0.000066065283340406) + ((-0.0003789797503263) + o[2] * ((-0.007878555448671) + o[3] * ((-0.087594591301146) - 0.000053349095828174 * o[7]))) * taug) + pi * (6.1445213076927e-8 + (1.31612001853305e-6 + o[2] * ((-0.00009683303171571) + o[3] * ((-0.0045101773626444) - 0.122004760687947 * o[7]))) * taug + pi * (taug * ((-3.15389238237468e-9) + (5.116287140914e-8 + 1.92901490874028e-6 * taug) * taug) + pi * (0.0000114610381688305 * o[2] * o[4] * taug + pi * (o[3] * ((-1.00288598706366e-10) + o[8] * ((-0.012702883392813) - 143.374451604624 * o[2] * o[6] * taug)) + pi * ((-4.1341695026989e-17) + o[2] * o[5] * ((-8.8352662293707e-6) - 0.272627897050173 * o[9]) * taug + pi * (o[5] * (9.0049690883672e-11 - 65.8490727183984 * o[4] * o[5] * o[6]) + pi * (1.78287415218792e-7 * o[8] + pi * (o[4] * (1.0406965210174e-18 + o[2] * ((-1.0234747095929e-12) - 1.0018179379511e-8 * o[4]) * o[4]) + o[10] * o[11] * (((-1.29412653835176e-9) + 1.71088510070544 * o[12]) * o[7] + o[10] * ((-6.05920510335078 * o[13] * o[5] * o[6] * taug) + o[10] * (o[4] * o[6] * (1.78371690710842e-23 + o[2] * o[4] * o[5] * (6.1258633752464e-12 - 0.000084004935396416 * o[8]) * taug) + pi * ((-1.24017662339842e-24 * o[12]) + pi * (0.0000832192847496054 * o[13] * o[4] * o[6] * taug + pi * (o[2] * o[5] * o[6] * (1.75410265428146e-27 + (1.32995316841867e-15 - 0.0000226487297378904 * o[2] * o[6]) * o[9]) * pi - 2.93678005497663e-14 * o[13] * o[2] * o[4] * taug))))))))))))))))) / pi;
              d := p / (data.RH2O * T * pi * gpi23);
            end dofp23;

            function dofpt3 "Inverse iteration in region 3: (d) = f(p,T)"
              extends Modelica.Icons.Function;
              input SI.Pressure p "Pressure";
              input SI.Temperature T "Temperature (K)";
              input SI.Pressure delp "Iteration converged if (p-pre(p) < delp)";
              output SI.Density d "Density";
              output Integer error = 0 "Error flag: iteration failed if different from 0";
            protected
              SI.Density dguess "Guess density";
              Integer i = 0 "Loop counter";
              Real dp "Pressure difference";
              SI.Density deld "Density step";
              Modelica.Media.Common.HelmholtzDerivs f "Dimensionless Helmholtz function and derivatives w.r.t. delta and tau";
              Modelica.Media.Common.NewtonDerivatives_pT nDerivs "Derivatives needed in Newton iteration";
              Boolean found = false "Flag for iteration success";
              Boolean supercritical "Flag, true for supercritical states";
              Boolean liquid "Flag, true for liquid states";
              SI.Density dmin "Lower density limit";
              SI.Density dmax "Upper density limit";
              SI.Temperature Tmax "Maximum temperature";
              Real damping "Damping factor";
            algorithm
              found := false;
              assert(p >= data.PLIMIT4A, "BaseIF97.dofpt3: function called outside of region 3! p too low\n" + "p = " + String(p) + " Pa < " + String(data.PLIMIT4A) + " Pa");
              assert(T >= data.TLIMIT1, "BaseIF97.dofpt3: function called outside of region 3! T too low\n" + "T = " + String(T) + " K < " + String(data.TLIMIT1) + " K");
              assert(p >= Regions.boundary23ofT(T), "BaseIF97.dofpt3: function called outside of region 3! T too high\n" + "p = " + String(p) + " Pa, T = " + String(T) + " K");
              supercritical := p > data.PCRIT;
              damping := if supercritical then 1.0 else 1.0;
              Tmax := Regions.boundary23ofp(p);
              if supercritical then
                dmax := dofp13(p);
                dmin := dofp23(p);
                dguess := dmax - (T - data.TLIMIT1) / (data.TLIMIT1 - Tmax) * (dmax - dmin);
              else
                liquid := T < Basic.tsat(p);
                if liquid then
                  dmax := dofp13(p);
                  dmin := Regions.rhol_p_R4b(p);
                  dguess := 1.1 * Regions.rhol_T(T) "Guess: 10 percent more than on the phase boundary for same T";
                else
                  dmax := Regions.rhov_p_R4b(p);
                  dmin := dofp23(p);
                  dguess := 0.9 * Regions.rhov_T(T) "Guess: 10% less than on the phase boundary for same T";
                end if;
              end if;
//this may need even further improvement!!
//      dguess := 0.5*(dmax + dmin);
//      dguess := 0.5*(dmax + dmin);
              while i < IterationData.IMAX and not found loop
                d := dguess;
                f := Basic.f3(d, T);
                nDerivs := Modelica.Media.Common.Helmholtz_pT(f);
                dp := nDerivs.p - p;
                if abs(dp / p) <= delp then
                  found := true;
                end if;
                deld := dp / nDerivs.pd * damping;
                d := d - deld;
                if d > dmin and d < dmax then
                  dguess := d;
                else
                  if d > dmax then
                    dguess := dmax - sqrt(Modelica.Constants.eps);
                  else
                    dguess := dmin + sqrt(Modelica.Constants.eps);
                  end if;
                end if;
                i := i + 1;
              end while;
// Can be used for debugging in Dymola: not a standard function
//          LogVariable(deld);
// put it on the correct spot just inside the boundary here instead
              if not found then
                error := 1;
              end if;
              assert(error <> 1, "Error in inverse function dofpt3: iteration failed");
            end dofpt3;

            // for all iteration functions: project to boundaries possible
            // if p is input. Step 1 get dofpt_efficient(p,T) at boundary
            // for T, use boundary itself if off limits, for d a bit inside

            function dtofph3 "Inverse iteration in region 3: (d,T) = f(p,h)"
              extends Modelica.Icons.Function;
              input SI.Pressure p "Pressure";
              input SI.SpecificEnthalpy h "Specific enthalpy";
              input SI.Pressure delp "Iteration accuracy";
              input SI.SpecificEnthalpy delh "Iteration accuracy";
              output SI.Density d "Density";
              output SI.Temperature T "Temperature (K)";
              output Integer error "Error flag: iteration failed if different from 0";
            protected
              SI.Temperature Tguess "Initial temperature";
              SI.Density dguess "Initial density";
              Integer i "Iteration counter";
              Real dh "Newton-error in h-direction";
              Real dp "Newton-error in p-direction";
              Real det "Determinant of directional derivatives";
              Real deld "Newton-step in d-direction";
              Real delt "Newton-step in T-direction";
              Modelica.Media.Common.HelmholtzDerivs f "Dimensionless Helmholtz function and derivatives w.r.t. delta and tau";
              Modelica.Media.Common.NewtonDerivatives_ph nDerivs "Derivatives needed in Newton iteration";
              Boolean found = false "Flag for iteration success";
              Integer subregion "1 for subregion 3a, 2 for subregion 3b";
            algorithm
              if p < data.PCRIT then
                subregion := if h < Regions.hl_p(p) + 10.0 then 1 else if h > Regions.hv_p(p) - 10.0 then 2 else 0;
                assert(subregion <> 0, "Inverse iteration of dt from ph called in 2 phase region: this can not work");
              else
                subregion := if h < Basic.h3ab_p(p) then 1 else 2;
              end if;
// allow a 10 J margin inside the (well approximated) phase boundary
//supercritical
              T := if subregion == 1 then Basic.T3a_ph(p, h) else Basic.T3b_ph(p, h);
              d := if subregion == 1 then 1 / Basic.v3a_ph(p, h) else 1 / Basic.v3b_ph(p, h);
              i := 0;
              error := 0;
              while i < IterationData.IMAX and not found loop
                f := Basic.f3(d, T);
                nDerivs := Modelica.Media.Common.Helmholtz_ph(f);
                dh := nDerivs.h - h;
                dp := nDerivs.p - p;
                if abs(dh / h) <= delh and abs(dp / p) <= delp then
                  found := true;
                end if;
                det := nDerivs.ht * nDerivs.pd - nDerivs.pt * nDerivs.hd;
                delt := (nDerivs.pd * dh - nDerivs.hd * dp) / det;
                deld := (nDerivs.ht * dp - nDerivs.pt * dh) / det;
                T := T - delt;
                d := d - deld;
                dguess := d;
                Tguess := T;
                i := i + 1;
                (d, T) := fixdT(dguess, Tguess);
              end while;
              if not found then
                error := 1;
              end if;
              assert(error <> 1, "Error in inverse function dtofph3: iteration failed");
            end dtofph3;

            function dtofps3 "Inverse iteration in region 3: (d,T) = f(p,s)"
              extends Modelica.Icons.Function;
              input SI.Pressure p "Pressure";
              input SI.SpecificEntropy s "Specific entropy";
              input SI.Pressure delp "Iteration accuracy";
              input SI.SpecificEntropy dels "Iteration accuracy";
              output SI.Density d "Density";
              output SI.Temperature T "Temperature (K)";
              output Integer error "Error flag: iteration failed if different from 0";
            protected
              SI.Temperature Tguess "Initial temperature";
              SI.Density dguess "Initial density";
              Integer i "Iteration counter";
              Real ds "Newton-error in s-direction";
              Real dp "Newton-error in p-direction";
              Real det "Determinant of directional derivatives";
              Real deld "Newton-step in d-direction";
              Real delt "Newton-step in T-direction";
              Modelica.Media.Common.HelmholtzDerivs f "Dimensionless Helmholtz function and derivatives w.r.t. delta and tau";
              Modelica.Media.Common.NewtonDerivatives_ps nDerivs "Derivatives needed in Newton iteration";
              Boolean found "Flag for iteration success";
              Integer subregion "1 for subregion 3a, 2 for subregion 3b";
            algorithm
              i := 0;
              error := 0;
              found := false;
              if p < data.PCRIT then
                subregion := if s < Regions.sl_p(p) + 10.0 then 1 else if s > Regions.sv_p(p) - 10.0 then 2 else 0;
                assert(subregion <> 0, "Inverse iteration of dt from ps called in 2 phase region: this is illegal!");
              else
                subregion := if s < data.SCRIT then 1 else 2;
              end if;
// allow a 1 J/K margin inside the (well approximated) phase boundary
              T := if subregion == 1 then Basic.T3a_ps(p, s) else Basic.T3b_ps(p, s);
              d := if subregion == 1 then 1 / Basic.v3a_ps(p, s) else 1 / Basic.v3b_ps(p, s);
              while i < IterationData.IMAX and not found loop
                f := Basic.f3(d, T);
                nDerivs := Modelica.Media.Common.Helmholtz_ps(f);
                ds := nDerivs.s - s;
                dp := nDerivs.p - p;
                if abs(ds / s) <= dels and abs(dp / p) <= delp then
                  found := true;
                end if;
                det := nDerivs.st * nDerivs.pd - nDerivs.pt * nDerivs.sd;
                delt := (nDerivs.pd * ds - nDerivs.sd * dp) / det;
                deld := (nDerivs.st * dp - nDerivs.pt * ds) / det;
                T := T - delt;
                d := d - deld;
                dguess := d;
                Tguess := T;
                i := i + 1;
                (d, T) := fixdT(dguess, Tguess);
              end while;
              if not found then
                error := 1;
              end if;
              assert(error <> 1, "Error in inverse function dtofps3: iteration failed");
            end dtofps3;

            function dtofpsdt3 "Inverse iteration in region 3: (d,T) = f(p,s)"
              extends Modelica.Icons.Function;
              input SI.Pressure p "Pressure";
              input SI.SpecificEntropy s "Specific entropy";
              input SI.Density dguess "Guess density, e.g., from adjacent volume";
              input SI.Temperature Tguess "Guess temperature, e.g., from adjacent volume";
              input SI.Pressure delp "Iteration accuracy";
              input SI.SpecificEntropy dels "Iteration accuracy";
              output SI.Density d "Density";
              output SI.Temperature T "Temperature (K)";
              output Integer error "Error flag: iteration failed if different from 0";
            protected
              Integer i "Iteration counter";
              Real ds "Newton-error in s-direction";
              Real dp "Newton-error in p-direction";
              Real det "Determinant of directional derivatives";
              Real deld "Newton-step in d-direction";
              Real delt "Newton-step in T-direction";
              Modelica.Media.Common.HelmholtzDerivs f "Dimensionless Helmholtz function and derivatives w.r.t. delta and tau";
              Modelica.Media.Common.NewtonDerivatives_ps nDerivs "Derivatives needed in Newton iteration";
              Boolean found "Flag for iteration success";
              SI.Density diter "Density";
              SI.Temperature Titer "Temperature (K)";
            algorithm
              i := 0;
              error := 0;
              found := false;
              (diter, Titer) := fixdT(dguess, Tguess);
              while i < IterationData.IMAX and not found loop
                (d, T) := fixdT(diter, Titer);
                f := Basic.f3(d, T);
                nDerivs := Modelica.Media.Common.Helmholtz_ps(f);
                ds := nDerivs.s - s;
                dp := nDerivs.p - p;
                if abs(ds / s) <= dels and abs(dp / p) <= delp then
                  found := true;
                end if;
                det := nDerivs.st * nDerivs.pd - nDerivs.pt * nDerivs.sd;
                delt := (nDerivs.pd * ds - nDerivs.sd * dp) / det;
                deld := (nDerivs.st * dp - nDerivs.pt * ds) / det;
                T := T - delt;
                d := d - deld;
                diter := d;
                Titer := T;
                i := i + 1;
              end while;
              if not found then
                error := 1;
              end if;
              assert(error <> 1, "Error in inverse function dtofpsdt3: iteration failed");
            end dtofpsdt3;

            function pofdt125 "Inverse iteration in region 1,2 and 5: p = g(d,T)"
              extends Modelica.Icons.Function;
              input SI.Density d "Density";
              input SI.Temperature T "Temperature (K)";
              input SI.Pressure reldd "Relative iteration accuracy of density";
              input Integer region "Region in IAPWS/IF97 in which inverse should be calculated";
              output SI.Pressure p "Pressure";
              output Integer error "Error flag: iteration failed if different from 0";
            protected
              Integer i "Counter for while-loop";
              Modelica.Media.Common.GibbsDerivs g "Dimensionless Gibbs function and derivatives w.r.t. pi and tau";
              Boolean found "Flag if iteration has been successful";
              Real dd "Difference between density for guessed p and the current density";
              Real delp "Step in p in Newton-iteration";
              Real relerr "Relative error in d";
              SI.Pressure pguess1 = 1.0e6 "Initial pressure guess in region 1";
              SI.Pressure pguess2 "Initial pressure guess in region 2";
              constant SI.Pressure pguess5 = 0.5e6 "Initial pressure guess in region 5";
            algorithm
              i := 0;
              error := 0;
              pguess2 := 42800 * d;
              found := false;
              if region == 1 then
                p := pguess1;
              elseif region == 2 then
                p := pguess2;
              else
                p := pguess5;
              end if;
              while i < IterationData.IMAX and not found loop
                if region == 1 then
                  g := Basic.g1(p, T);
                elseif region == 2 then
                  g := Basic.g2(p, T);
                else
                  g := Basic.g5(p, T);
                end if;
                dd := p / (data.RH2O * T * g.pi * g.gpi) - d;
                relerr := dd / d;
                if abs(relerr) < reldd then
                  found := true;
                end if;
                delp := dd * (-p * p / (d * d * data.RH2O * T * g.pi * g.pi * g.gpipi));
                p := p - delp;
                i := i + 1;
                if not found then
                  if p < triple.ptriple then
                    p := 2.0 * triple.ptriple;
                  end if;
                  if p > data.PLIMIT1 then
                    p := 0.95 * data.PLIMIT1;
                  end if;
                end if;
              end while;
// print("i = " + i2s(i) + ", p = " + r2s(p/1.0e5) + ", delp = " + r2s(delp*1.0e-5) + "\n");
              if not found then
                error := 1;
              end if;
              assert(error <> 1, "Error in inverse function pofdt125: iteration failed");
            end pofdt125;

            function tofph5 "Inverse iteration in region 5: (p,T) = f(p,h)"
              extends Modelica.Icons.Function;
              input SI.Pressure p "Pressure";
              input SI.SpecificEnthalpy h "Specific enthalpy";
              input SI.SpecificEnthalpy reldh "Iteration accuracy";
              output SI.Temperature T "Temperature (K)";
              output Integer error "Error flag: iteration failed if different from 0";
            protected
              Modelica.Media.Common.GibbsDerivs g "Dimensionless Gibbs function and derivatives w.r.t. pi and tau";
              SI.SpecificEnthalpy proh "H for current guess in T";
              constant SI.Temperature Tguess = 1500 "Initial temperature";
              Integer i "Iteration counter";
              Real relerr "Relative error in h";
              Real dh "Newton-error in h-direction";
              Real dT "Newton-step in T-direction";
              Boolean found "Flag for iteration success";
            algorithm
              i := 0;
              error := 0;
              T := Tguess;
              found := false;
              while i < IterationData.IMAX and not found loop
                g := Basic.g5(p, T);
                proh := data.RH2O * T * g.tau * g.gtau;
                dh := proh - h;
                relerr := dh / h;
                if abs(relerr) < reldh then
                  found := true;
                end if;
                dT := dh / (-data.RH2O * g.tau * g.tau * g.gtautau);
                T := T - dT;
                i := i + 1;
              end while;
              if not found then
                error := 1;
              end if;
              assert(error <> 1, "Error in inverse function tofph5: iteration failed");
            end tofph5;

            function tofps5 "Inverse iteration in region 5: (p,T) = f(p,s)"
              extends Modelica.Icons.Function;
              input SI.Pressure p "Pressure";
              input SI.SpecificEntropy s "Specific entropy";
              input SI.SpecificEnthalpy relds "Iteration accuracy";
              output SI.Temperature T "Temperature (K)";
              output Integer error "Error flag: iteration failed if different from 0";
            protected
              Modelica.Media.Common.GibbsDerivs g "Dimensionless Gibbs function and derivatives w.r.t. pi and tau";
              SI.SpecificEntropy pros "S for current guess in T";
              parameter SI.Temperature Tguess = 1500 "Initial temperature";
              Integer i "Iteration counter";
              Real relerr "Relative error in s";
              Real ds "Newton-error in s-direction";
              Real dT "Newton-step in T-direction";
              Boolean found "Flag for iteration success";
            algorithm
              i := 0;
              error := 0;
              T := Tguess;
              found := false;
              while i < IterationData.IMAX and not found loop
                g := Basic.g5(p, T);
                pros := data.RH2O * (g.tau * g.gtau - g.g);
                ds := pros - s;
                relerr := ds / s;
                if abs(relerr) < relds then
                  found := true;
                end if;
                dT := ds * T / (-data.RH2O * g.tau * g.tau * g.gtautau);
                T := T - dT;
                i := i + 1;
              end while;
              if not found then
                error := 1;
              end if;
              assert(error <> 1, "Error in inverse function tofps5: iteration failed");
            end tofps5;

            function tofpst5 "Inverse iteration in region 5: (p,T) = f(p,s)"
              extends Modelica.Icons.Function;
              input SI.Pressure p "Pressure";
              input SI.SpecificEntropy s "Specific entropy";
              input SI.Temperature Tguess "Guess temperature, e.g., from adjacent volume";
              input SI.SpecificEntropy relds "Iteration accuracy";
              output SI.Temperature T "Temperature (K)";
              output Integer error "Error flag: iteration failed if different from 0";
            protected
              Modelica.Media.Common.GibbsDerivs g "Dimensionless Gibbs function and derivatives w.r.t. pi and tau";
              SI.SpecificEntropy pros "S for current guess in T";
              Integer i "Iteration counter";
              Real relerr "Relative error in s";
              Real ds "Newton-error in s-direction";
              Real dT "Newton-step in T-direction";
              Boolean found "Flag for iteration success";
            algorithm
              i := 0;
              error := 0;
              T := Tguess;
              found := false;
              while i < IterationData.IMAX and not found loop
                g := Basic.g5(p, T);
                pros := data.RH2O * (g.tau * g.gtau - g.g);
                ds := pros - s;
                relerr := ds / s;
                if abs(relerr) < relds then
                  found := true;
                end if;
                dT := ds * T / (-data.RH2O * g.tau * g.tau * g.gtautau);
                T := T - dT;
                i := i + 1;
              end while;
              if not found then
                error := 1;
              end if;
              assert(error <> 1, "Error in inverse function tofpst5: iteration failed");
            end tofpst5;
            annotation(
              Documentation(info = "<HTML><h4>Package description</h4>
          <h4>Package contents</h4>
          <ul>
          <li>Function <b>fixdT</b> constrains density and temperature to allowed region</li>
          <li>Function <b>dofp13</b> computes d as a function of p at boundary between regions 1 and 3</li>
          <li>Function <b>dofp23</b> computes d as a function of p at boundary between regions 2 and 3</li>
          <li>Function <b>dofpt3</b> iteration to compute d as a function of p and T in region 3</li>
          <li>Function <b>dtofph3</b> iteration to compute d and T as a function of p and h in region 3</li>
          <li>Function <b>dtofps3</b> iteration to compute d and T as a function of p and s in region 3</li>
          <li>Function <b>dtofpsdt3</b> iteration to compute d and T as a function of p and s in region 3,
          with initial guesses</li>
          <li>Function <b>pofdt125</b> iteration to compute p as a function of p and T in regions 1, 2 and 5</li>
          <li>Function <b>tofph5</b> iteration to compute T as a function of p and h in region 5</li>
          <li>Function <b>tofps5</b> iteration to compute T as a function of p and s in region 5</li>
          <li>Function <b>tofpst5</b> iteration to compute T as a function of p and s in region 5, with initial guess in T</li>
          </ul>
          <h4>Version Info and Revision history
          </h4>
          <ul>
          <li>First implemented: <i>July, 2000</i>
          by <a href=\"http://www.control.lth.se/~hubertus/\">Hubertus Tummescheit</a>
          </li>
          </ul>
          <address>Author: Hubertus Tummescheit, <br>
      Modelon AB<br>
      Ideon Science Park<br>
      SE-22370 Lund, Sweden<br>
      email: hubertus@modelon.se
          </address>
          <ul>
          <li>Initial version: July 2000</li>
          <li>Documentation added: December 2002</li>
          </ul>
          </html>"));
          end Inverses;

          package ByRegion "Simple explicit functions for one region only"
            extends Modelica.Icons.Package;

            function waterR1_pT "Standard properties for region 1, (p,T) as inputs"
              extends Modelica.Icons.Function;
              input SI.Pressure p "Pressure";
              input SI.Temperature T "Temperature (K)";
              output Modelica.Media.Common.ThermoFluidSpecial.ThermoProperties_pT pro "Thermodynamic property collection";
            protected
              Modelica.Media.Common.GibbsDerivs g "Dimensionless Gibbs function and derivatives w.r.t. pi and tau";
            algorithm
              g := Basic.g1(p, T);
              pro := Modelica.Media.Common.ThermoFluidSpecial.gibbsToProps_pT(g);
            end waterR1_pT;

            function waterR2_pT "Standard properties for region 2, (p,T) as inputs"
              extends Modelica.Icons.Function;
              input SI.Pressure p "Pressure";
              input SI.Temperature T "Temperature (K)";
              output Modelica.Media.Common.ThermoFluidSpecial.ThermoProperties_pT pro "Thermodynamic property collection";
            protected
              Modelica.Media.Common.GibbsDerivs g "Dimensionless Gibbs function and derivatives w.r.t. pi and tau";
            algorithm
              g := Basic.g2(p, T);
              pro := Modelica.Media.Common.ThermoFluidSpecial.gibbsToProps_pT(g);
            end waterR2_pT;

            function waterR3_dT "Standard properties for region 3, (d,T) as inputs"
              extends Modelica.Icons.Function;
              input SI.Density d "Density";
              input SI.Temperature T "Temperature (K)";
              output Common.ThermoFluidSpecial.ThermoProperties_dT pro "Thermodynamic property collection";
            protected
              Common.HelmholtzDerivs f "Dimensionless Helmholtz function and derivatives w.r.t. delta and tau";
            algorithm
              f := Basic.f3(d, T);
              pro := Modelica.Media.Common.ThermoFluidSpecial.helmholtzToProps_dT(f);
              assert(pro.p <= 100.0e6, "IF97 medium function waterR3_dT: the input pressure (= " + String(pro.p) + " Pa) is higher than 100 Mpa");
            end waterR3_dT;

            function waterR5_pT "Standard properties for region 5, (p,T) as inputs"
              extends Modelica.Icons.Function;
              input SI.Pressure p "Pressure";
              input SI.Temperature T "Temperature (K)";
              output Modelica.Media.Common.ThermoFluidSpecial.ThermoProperties_pT pro "Thermodynamic property collection";
            protected
              Modelica.Media.Common.GibbsDerivs g "Dimensionless Gibbs function and derivatives w.r.t. pi and tau";
            algorithm
              g := Basic.g5(p, T);
              pro := Modelica.Media.Common.ThermoFluidSpecial.gibbsToProps_pT(g);
            end waterR5_pT;
            annotation(
              Documentation(info = "<HTML><h4>Package description</h4>
          <p>Package ByRegion provides fast forward calls for dynamic property calculation records for all
          one phase regions of IAPWS/IF97
          </p>
          <h4>Package contents</h4>
          <ul>
          <li>Function <b>waterR1_pT</b> computes dynamic properties for region 1 using  (p,T) as inputs</li>
          <li>Function <b>waterR2_pT</b> computes dynamic properties for region 2 using  (p,T) as inputs</li>
          <li>Function <b>waterR3_dT</b> computes dynamic properties for region 3 using  (d,T) as inputs</li>
          <li>Function <b>waterR5_pT</b> computes dynamic properties for region 5 using  (p,T) as inputs</li>
          </ul>
          <h4>Version Info and Revision history
          </h4>
          <ul>
          <li>First implemented: <i>July, 2000</i>
          by <a href=\"http://www.control.lth.se/~hubertus/\">Hubertus Tummescheit</a>
          </li>
          </ul>
          <address>Author: Hubertus Tummescheit, <br>
      Modelon AB<br>
      Ideon Science Park<br>
      SE-22370 Lund, Sweden<br>
      email: hubertus@modelon.se
          </address>
          <ul>
          <li>Initial version: July 2000</li>
          <li>Documented and re-organized: January 2003</li>
          </ul>
          </html>"));
          end ByRegion;

          package TwoPhase "Steam properties in the two-phase region and on the phase boundaries"
            extends Modelica.Icons.Package;

            function waterLiq_p "Properties on the liquid phase boundary of region 4"
              extends Modelica.Icons.Function;
              input SI.Pressure p "Pressure";
              output Modelica.Media.Common.PhaseBoundaryProperties liq "Liquid thermodynamic property collection";
            protected
              SI.Temperature Tsat "Saturation temperature";
              Real dpT "Derivative of saturation pressure w.r.t. temperature";
              SI.Density dl "Liquid density";
              Modelica.Media.Common.GibbsDerivs g "Dimensionless Gibbs function and derivatives w.r.t. pi and tau";
              Modelica.Media.Common.HelmholtzDerivs f "Dimensionless Helmholtz function and derivatives w.r.t. delta and tau";
            algorithm
              Tsat := Basic.tsat(p);
              dpT := Basic.dptofT(Tsat);
              if p < data.PLIMIT4A then
                g := Basic.g1(p, Tsat);
                liq := Modelica.Media.Common.gibbsToBoundaryProps(g);
              else
                dl := Regions.rhol_p_R4b(p);
                f := Basic.f3(dl, Tsat);
                liq := Modelica.Media.Common.helmholtzToBoundaryProps(f);
              end if;
            end waterLiq_p;

            function waterVap_p "Properties on the vapour phase boundary of region 4"
              extends Modelica.Icons.Function;
              input SI.Pressure p "Pressure";
              output Modelica.Media.Common.PhaseBoundaryProperties vap "Vapour thermodynamic property collection";
            protected
              SI.Temperature Tsat "Saturation temperature";
              Real dpT "Derivative of saturation pressure w.r.t. temperature";
              SI.Density dv "Vapour density";
              Modelica.Media.Common.GibbsDerivs g "Dimensionless Gibbs function and derivatives w.r.t. pi and tau";
              Modelica.Media.Common.HelmholtzDerivs f "Dimensionless Helmholtz function and derivatives w.r.t. delta and tau";
            algorithm
              Tsat := Basic.tsat(p);
              dpT := Basic.dptofT(Tsat);
              if p < data.PLIMIT4A then
                g := Basic.g2(p, Tsat);
                vap := Modelica.Media.Common.gibbsToBoundaryProps(g);
              else
                dv := Regions.rhov_p_R4b(p);
                f := Basic.f3(dv, Tsat);
                vap := Modelica.Media.Common.helmholtzToBoundaryProps(f);
              end if;
            end waterVap_p;

            function waterSat_ph "Water saturation properties in the 2-phase region (4) as f(p,h)"
              extends Modelica.Icons.Function;
              input SI.Pressure p "Pressure";
              input SI.SpecificEnthalpy h "Specific enthalpy";
              output Modelica.Media.Common.SaturationProperties pro "Thermodynamic property collection";
            protected
              SI.Density dl "Liquid density";
              SI.Density dv "Vapour density";
              Modelica.Media.Common.GibbsDerivs gl "Dimensionless Gibbs function and derivatives w.r.t. pi and tau";
              Modelica.Media.Common.GibbsDerivs gv "Dimensionless Gibbs function and derivatives w.r.t. pi and tau";
              Modelica.Media.Common.HelmholtzDerivs fl "Dimensionless Helmholtz function and derivatives w.r.t. delta and tau";
              Modelica.Media.Common.HelmholtzDerivs fv "Dimensionless Helmholtz function and derivatives w.r.t. delta and tau";
            algorithm
              pro.h := h;
              pro.p := p;
              pro.T := Basic.tsat(p);
              pro.dpT := Basic.dptofT(pro.T);
              if p < data.PLIMIT4A then
                gl := Basic.g1(p, pro.T);
                gv := Basic.g2(p, pro.T);
                pro.liq := Modelica.Media.Common.gibbsToBoundaryProps(gl);
                pro.vap := Modelica.Media.Common.gibbsToBoundaryProps(gv);
              else
                dl := Regions.rhol_p_R4b(p);
                dv := Regions.rhov_p_R4b(p);
                fl := Basic.f3(dl, pro.T);
                fv := Basic.f3(dv, pro.T);
                pro.liq := Modelica.Media.Common.helmholtzToBoundaryProps(fl);
                pro.vap := Modelica.Media.Common.helmholtzToBoundaryProps(fv);
              end if;
              pro.x := if h < pro.liq.h then 0.0 else if pro.vap.h <> pro.liq.h then (h - pro.liq.h) / (pro.vap.h - pro.liq.h) else 1.0;
              pro.d := pro.liq.d * pro.vap.d / (pro.vap.d + pro.x * (pro.liq.d - pro.vap.d));
              pro.u := pro.x * pro.vap.u + (1 - pro.x) * pro.liq.u;
              pro.s := pro.x * pro.vap.s + (1 - pro.x) * pro.liq.s;
              pro.cp := Modelica.Constants.inf;
              pro.cv := Modelica.Media.Common.cv2Phase(pro.liq, pro.vap, pro.x, pro.T, p);
              pro.kappa := 1 / (pro.d * p) * pro.dpT * pro.dpT * pro.T / pro.cv;
              pro.R := data.RH2O;
            end waterSat_ph;

            function waterR4_ph "Water/Steam properties in region 4 of IAPWS/IF97 (two-phase)"
              extends Modelica.Icons.Function;
              input SI.Pressure p "Pressure";
              input SI.SpecificEnthalpy h "Specific enthalpy";
              output Modelica.Media.Common.ThermoFluidSpecial.ThermoProperties_ph pro "Thermodynamic property collection";
            protected
              SI.Density dl "Liquid density";
              SI.Density dv "Vapour density";
              Modelica.Media.Common.PhaseBoundaryProperties liq "Phase boundary property record";
              Modelica.Media.Common.PhaseBoundaryProperties vap "Phase boundary property record";
              Modelica.Media.Common.GibbsDerivs gl "Dimensionless Gibbs function and derivatives w.r.t. pi and tau";
              Modelica.Media.Common.GibbsDerivs gv "Dimensionless Gibbs function and derivatives w.r.t. pi and tau";
              Modelica.Media.Common.HelmholtzDerivs fl "Dimensionless Helmholtz function and derivatives w.r.t. delta and tau";
              Modelica.Media.Common.HelmholtzDerivs fv "Dimensionless Helmholtz function and derivatives w.r.t. delta and tau";
              Real x "Dryness fraction";
              Real dpT "Derivative of saturation curve";
            algorithm
              pro.T := Basic.tsat(p);
              dpT := Basic.dptofT(pro.T);
              dl := Regions.rhol_p_R4b(p);
              dv := Regions.rhov_p_R4b(p);
              if p < data.PLIMIT4A then
                gl := Basic.g1(p, pro.T);
                gv := Basic.g2(p, pro.T);
                liq := Modelica.Media.Common.gibbsToBoundaryProps(gl);
                vap := Modelica.Media.Common.gibbsToBoundaryProps(gv);
              else
                fl := Basic.f3(dl, pro.T);
                fv := Basic.f3(dv, pro.T);
                liq := Modelica.Media.Common.helmholtzToBoundaryProps(fl);
                vap := Modelica.Media.Common.helmholtzToBoundaryProps(fv);
              end if;
              x := if vap.h <> liq.h then (h - liq.h) / (vap.h - liq.h) else 1.0;
              pro.d := liq.d * vap.d / (vap.d + x * (liq.d - vap.d));
              pro.u := x * vap.u + (1 - x) * liq.u;
              pro.s := x * vap.s + (1 - x) * liq.s;
              pro.cp := Modelica.Constants.inf;
              pro.cv := Modelica.Media.Common.cv2Phase(liq, vap, x, pro.T, p);
              pro.kappa := 1 / (pro.d * p) * dpT * dpT * pro.T / pro.cv;
              pro.a := Modelica.Constants.inf;
              pro.R := data.RH2O;
              pro.ddph := pro.d * (pro.d * pro.cv / dpT + 1.0) / (dpT * pro.T);
              pro.ddhp := -pro.d * pro.d / (dpT * pro.T);
            end waterR4_ph;

            function waterR4_dT "Water properties in region 4 as function of d and T"
              extends Modelica.Icons.Function;
              input SI.Density d "Density";
              input SI.Temperature T "Temperature";
              output Modelica.Media.Common.ThermoFluidSpecial.ThermoProperties_dT pro "Thermodynamic property collection";
            protected
              SI.Density dl "Liquid density";
              SI.Density dv "Vapour density";
              Modelica.Media.Common.PhaseBoundaryProperties liq "Phase boundary property record";
              Modelica.Media.Common.PhaseBoundaryProperties vap "Phase boundary property record";
              Modelica.Media.Common.GibbsDerivs gl "Dimensionless Gibbs function and derivatives w.r.t. pi and tau";
              Modelica.Media.Common.GibbsDerivs gv "Dimensionless Gibbs function and derivatives w.r.t. pi and tau";
              Modelica.Media.Common.HelmholtzDerivs fl "Dimensionless Helmholtz function and derivatives w.r.t. delta and tau";
              Modelica.Media.Common.HelmholtzDerivs fv "Dimensionless Helmholtz function and derivatives w.r.t. delta and tau";
              Real x "Dryness fraction";
              Real dpT "Derivative of saturation curve";
            algorithm
              pro.p := Basic.psat(T);
              dpT := Basic.dptofT(T);
              dl := Regions.rhol_p_R4b(pro.p);
              dv := Regions.rhov_p_R4b(pro.p);
              if pro.p < data.PLIMIT4A then
                gl := Basic.g1(pro.p, T);
                gv := Basic.g2(pro.p, T);
                liq := Modelica.Media.Common.gibbsToBoundaryProps(gl);
                vap := Modelica.Media.Common.gibbsToBoundaryProps(gv);
              else
                fl := Basic.f3(dl, T);
                fv := Basic.f3(dv, T);
                liq := Modelica.Media.Common.helmholtzToBoundaryProps(fl);
                vap := Modelica.Media.Common.helmholtzToBoundaryProps(fv);
              end if;
              x := if vap.d <> liq.d then (1 / d - 1 / liq.d) / (1 / vap.d - 1 / liq.d) else 1.0;
              pro.u := x * vap.u + (1 - x) * liq.u;
              pro.h := x * vap.h + (1 - x) * liq.h;
              pro.cp := Modelica.Constants.inf;
              pro.cv := Modelica.Media.Common.cv2Phase(liq, vap, x, T, pro.p);
              pro.kappa := 1 / (d * pro.p) * dpT * dpT * T / pro.cv;
              pro.a := Modelica.Constants.inf;
              pro.R := data.RH2O;
              pro.dudT := (pro.p - T * dpT) / (d * d);
            end waterR4_dT;
            annotation(
              Documentation(info = "<HTML><h4>Package description</h4>
          <p>Package TwoPhase provides functions to compute the steam properties
          in the two-phase region and on the phase boundaries</p>
          <h4>Package contents</h4>
          <ul>
          <li>Function <b>WaterLiq_p</b> computes properties on the boiling boundary as a function of p</li>
          <li>Function <b>WaterVap_p</b> computes properties on the dew line boundary as a function of p</li>
          <li>Function <b>WaterSat_ph</b> computes properties on both phase boundaries and in the two
          phase region as a function of p</li>
          <li>Function <b>WaterR4_ph</b> computes dynamic simulation properties in region 4 with (p,h) as inputs</li>
          <li>Function <b>WaterR4_dT</b> computes dynamic simulation properties in region 4 with (d,T) as inputs</li>
          </ul>
          <h4>Version Info and Revision history
          </h4>
          <ul>
          <li>First implemented: <i>July, 2000</i>
          by <a href=\"http://www.control.lth.se/~hubertus/\">Hubertus Tummescheit</a>
          </li>
          </ul>
          <address>Author: Hubertus Tummescheit, <br>
      Modelon AB<br>
      Ideon Science Park<br>
      SE-22370 Lund, Sweden<br>
      email: hubertus@modelon.se
          </address>
          <ul>
          <li>Initial version: July 2000</li>
          <li>Documented and re-organized: January 2003</li>
          </ul>
          </html>"));
          end TwoPhase;

          function extraDerivs_ph "Function to calculate some extra thermophysical properties in regions 1, 2, 3 and 5 as f(p,h)"
            extends Modelica.Icons.Function;
            input SI.Pressure p "Pressure";
            input SI.SpecificEnthalpy h "Specific enthalpy";
            input Integer phase = 0 "Phase: 2 for two-phase, 1 for one phase, 0 if unknown";
            output Modelica.Media.Common.ExtraDerivatives dpro "Thermodynamic property collection";
          protected
            SI.Density d "Density";
            SI.Temperature T "Temperature";
            Integer region "IF97 region";
            Integer error "Error flag";
            Modelica.Media.Common.HelmholtzDerivs f "Dimensionless Helmholtz function and derivatives w.r.t. delta and tau";
            Modelica.Media.Common.GibbsDerivs g "Dimensionless Gibbs function and derivatives w.r.t. pi and tau";
          algorithm
            assert(phase == 1, "ExtraDerivs_ph: properties are not implemented in 2 phase region");
            region := Regions.region_ph(p = p, h = h, phase = phase);
            if region == 1 then
              T := Basic.tph1(p, h);
              g := Basic.g1(p, T);
              dpro := Modelica.Media.Common.gibbsToExtraDerivs(g);
            elseif region == 2 then
              T := Basic.tph2(p, h);
              g := Basic.g2(p, T);
              dpro := Modelica.Media.Common.gibbsToExtraDerivs(g);
            elseif region == 3 then
              (d, T, error) := Inverses.dtofph3(p = p, h = h, delp = 1.0e-7, delh = 1.0e-6);
              f := Basic.f3(d, T);
              dpro := Modelica.Media.Common.helmholtzToExtraDerivs(f);
            elseif region == 5 then
              (T, error) := Inverses.tofph5(p = p, h = h, reldh = 1.0e-7);
              g := Basic.g5(p, T);
              dpro := Modelica.Media.Common.gibbsToExtraDerivs(g);
            end if;
// region assumed to be 5
          end extraDerivs_ph;

          function extraDerivs_pT "Function to calculate some extra thermophysical properties in regions 1, 2, 3 and 5 as f(p,T)"
            extends Modelica.Icons.Function;
            input SI.Pressure p "Pressure";
            input SI.Temperature T "Temperature";
            output Modelica.Media.Common.ExtraDerivatives dpro "Thermodynamic property collection";
          protected
            SI.Density d "Density";
            Integer region "IF97 region";
            Integer error "Error flag";
            Modelica.Media.Common.HelmholtzDerivs f "Dimensionless Helmholtz function and derivatives w.r.t. delta and tau";
            Modelica.Media.Common.GibbsDerivs g "Dimensionless Gibbs function and derivatives w.r.t. pi and tau";
          algorithm
            region := Regions.region_pT(p = p, T = T);
            if region == 1 then
              g := Basic.g1(p, T);
              dpro := Modelica.Media.Common.gibbsToExtraDerivs(g);
            elseif region == 2 then
              g := Basic.g2(p, T);
              dpro := Modelica.Media.Common.gibbsToExtraDerivs(g);
            elseif region == 3 then
              (d, error) := Inverses.dofpt3(p = p, T = T, delp = 1.0e-7);
              f := Basic.f3(d, T);
              dpro := Modelica.Media.Common.helmholtzToExtraDerivs(f);
            elseif region == 5 then
              g := Basic.g5(p, T);
              dpro := Modelica.Media.Common.gibbsToExtraDerivs(g);
            end if;
// region assumed to be 5
          end extraDerivs_pT;
          annotation(
            Documentation(info = "<HTML>
<style type=\"text/css\">
.nobr
{
white-space:nowrap;
}
</style>
    <h4>Version Info and Revision history</h4>
        <ul>
        <li>First implemented: <i>July, 2000</i>
        by Hubertus Tummescheit
        for the ThermoFluid Library with help from Jonas Eborn and Falko Jens Wagner
        </li>
      <li>Code reorganization, enhanced documentation, additional functions:   <i>December, 2002</i>
      by <a href=\"mailto:Hubertus.Tummescheit@modelon.se\">Hubertus Tummescheit</a> and moved to Modelica
      properties library.</li>
        </ul>
      <address>Author: Hubertus Tummescheit, <br>
      Modelon AB<br>
      Ideon Science Park<br>
      SE-22370 Lund, Sweden<br>
      email: hubertus@modelon.se
      </address>
        <p>In September 1997, the International Association for the Properties
        of Water and Steam (<A HREF=\"http://www.iapws.org\">IAPWS</A>) adopted a
        new formulation for the thermodynamic properties of water and steam for
        industrial use. This new industrial standard is called \"IAPWS Industrial
        Formulation for the Thermodynamic Properties of Water and Steam\" (IAPWS-IF97).
        The formulation IAPWS-IF97 replaces the previous industrial standard IFC-67.
        <p>Based on this new formulation, a new steam table, titled \"<a href=\"http://www.springer.de/cgi-bin/search_book.pl?isbn=3-540-64339-7\">Properties of Water and Steam</a>\" by W. Wagner and A. Kruse, was published by
        the Springer-Verlag, Berlin - New-York - Tokyo in April 1998. This
        steam table, ref. <a href=\"#steamprop\">[1]</a> is bilingual (English /
        German) and contains a complete description of the equations of
        IAPWS-IF97. This reference is the authoritative source of information
        for this implementation. A mostly identical version has been published by the International
        Association for the Properties
        of Water and Steam (<A HREF=\"http://www.iapws.org\">IAPWS</A>) with permission granted to re-publish the
        information if credit is given to IAPWS. This document is distributed with this library as
        <a href=\"modelica://Modelica/Resources/Documentation/Media/Water/IF97documentation/IF97.pdf\">IF97.pdf</a>.
        In addition, the equations published by <A HREF=\"http://www.iapws.org\">IAPWS</A> for
        the transport properties dynamic viscosity (standards document: <a href=\"modelica://Modelica/Resources/Documentation/Media/Water/IF97documentation/visc.pdf\">visc.pdf</a>)
        and thermal conductivity (standards document: <a href=\"modelica://Modelica/Resources/Documentation/Media/Water/IF97documentation/thcond.pdf\">thcond.pdf</a>)
        and equations for the surface tension (standards document: <a href=\"modelica://Modelica/Resources/Documentation/Media/Water/IF97documentation/surf.pdf\">surf.pdf</a>)
        are also implemented in this library and included for reference.</p>
        <p>
        The functions in BaseIF97.mo are low level functions which should
        only be used in those exceptions when the standard user level
        functions in Water.mo do not contain the wanted properties.
     </p>
<p>Based on IAPWS-IF97, Modelica functions are available for calculating
the most common thermophysical properties (thermodynamic and transport
properties). The implementation requires part of the common medium
property infrastructure of the Modelica.Thermal.Properties library in the file
Common.mo. There are a few extensions from the version of IF97 as
documented in <a href=\"modelica://Modelica/Resources/Documentation/Media/Water/IF97documentation/IF97.pdf\">IF97.pdf</a> in order to improve performance for
dynamic simulations. Input variables for calculating the properties are
only implemented for a limited number of variable pairs which make sense as dynamic states: (p,h), (p,T), (p,s) and (d,T).
</p>
<hr size=3 width=\"70%\">
<h4><a name=\"regions\">1. Structure and Regions of IAPWS-IF97</a></h4>
<p>The IAPWS Industrial Formulation 1997 consists of
a set of equations for different regions which cover the following range
of validity:</p>
<table border=0 cellpadding=4>
<tr>
<td valign=\"top\">273,15 K &lt; <em>T</em> &lt; 1073,15 K</td>
<td valign=\"top\"><em>p</em> &lt; 100 MPa</td>
</tr>
<tr>
<td valign=\"top\">1073,15 K &lt; <em>T</em> &lt; 2273,15 K</td>
<td valign=\"top\"><em>p</em> &lt; 10 MPa</td>
</tr>
</table>
<p>
Figure 1 shows the 5 regions into which the entire range of validity of
IAPWS-IF97 is divided. The boundaries of the regions can be directly taken
from Fig. 1 except for the boundary between regions 2 and 3; this boundary,
which corresponds approximately to the isentropic line <span class=\"nobr\"><em>s</em> = 5.047 kJ kg
<sup>-1</sup>K<sup>-1</sup></span>, is defined
by a corresponding auxiliary equation. Both regions 1 and 2 are individually
covered by a fundamental equation for the specific Gibbs free energy <span class=\"nobr\"><em>g</em>( <em>p</em>,<em>T</em> )</span>, region 3 by a fundamental equation for the specific Helmholtz
free energy <span class=\"nobr\"><em>f </em>(<em> <font face=\"symbol\">r</font></em>,<em>T
</em>)</span>, and the saturation curve, corresponding to region 4, by a saturation-pressure
equation <span><em>p</em><sub>s</sub>( <em>T</em> )</span>. The high-temperature
region 5 is also covered by a <span class=\"nobr\"><em>g</em>( <em>p</em>,<em>T</em> )</span> equation. These
5 equations, shown in rectangular boxes in Fig. 1, form the so-called <em>basic
equations</em>.
</p>
<table border=\"0\" cellspacing=\"0\" cellpadding=\"2\">
  <caption align=\"bottom\">Figure 1: Regions and equations of IAPWS-IF97</caption>
  <tr>
    <td>
    <img src=\"modelica://Modelica/Resources/Images/Media/Water/if97.png\" alt=\"Regions and equations of IAPWS-IF97\">
    </td>
  </tr>
</table>
<p>
In addition to these basic equations, so-called <em>backward
equations</em> are provided for regions 1, 2, and 4 in form of
<span class=\"nobr\"><em>T</em>( <em>p</em>,<em>h</em> )</span> and <span class=\"nobr\"><em>T</em>( <em>
p</em>,<em>s</em> )</span> for regions 1 and 2, and <span class=\"nobr\"><em>T</em><sub>s</sub>( <em>p</em> )</span> for region 4. These
backward equations, marked in grey in Fig. 1, were developed in such a
way that they are numerically very consistent with the corresponding
basic equation. Thus, properties as functions of&nbsp; <em>p</em>,<em>h
</em>and of&nbsp;<em> p</em>,<em>s </em>for regions 1 and 2, and of
<em>p</em> for region 4 can be calculated without any iteration. As a
result of this special concept for the development of the new
industrial standard IAPWS-IF97, the most important properties can be
calculated extremely quickly. All Modelica functions are optimized
with regard to short computing times.
</p>
<p>
The complete description of the individual equations of the new industrial
formulation IAPWS-IF97 is given in <a href=\"modelica://Modelica/Resources/Documentation/Media/Water/IF97documentation/IF97.pdf\">IF97.pdf</a>. Comprehensive information on
IAPWS-IF97 (requirements, concept, accuracy, consistency along region boundaries,
and the increase of computing speed in comparison with IFC-67, etc.) can
be taken from <a href=\"modelica://Modelica/Resources/Documentation/Media/Water/IF97documentation/IF97.pdf\">IF97.pdf</a> or [2].
</p>
<p>
<a name=\"steamprop\">[1]<em>Wagner, W., Kruse, A.</em> Properties of Water
and Steam / Zustandsgr&ouml;&szlig;en von Wasser und Wasserdampf / IAPWS-IF97.
Springer-Verlag, Berlin, 1998.</a>
</p>
<p>
[2] <em>Wagner, W., Cooper, J. R., Dittmann, A., Kijima,
J., Kretzschmar, H.-J., Kruse, A., Mare&#353; R., Oguchi, K., Sato, H., St&ouml;cker,
I., &#352;ifner, O., Takaishi, Y., Tanishita, I., Tr&uuml;benbach, J., and Willkommen,
Th.</em> The IAPWS Industrial Formulation 1997 for the Thermodynamic Properties
of Water and Steam. ASME Journal of Engineering for Gas Turbines and Power 122 (2000), 150 - 182.
</p>
<HR size=3 width=\"90%\">
<h4>2. Calculable Properties      </h4>
<table border=\"1\" cellpadding=\"2\" cellspacing=\"0\">
       <tbody>
       <tr>
       <td valign=\"top\" bgcolor=\"#cccccc\"><br>
      </td>
      <td valign=\"top\" bgcolor=\"#cccccc\"><b>Common name</b><br>
       </td>
       <td valign=\"top\" bgcolor=\"#cccccc\"><b>Abbreviation </b><br>
       </td>
       <td valign=\"top\" bgcolor=\"#cccccc\"><b>Unit</b><br>
       </td>
       </tr>
       <tr>
       <td valign=\"top\">&nbsp;1<br>
      </td>
      <td valign=\"top\">Pressure</td>
       <td valign=\"top\">p<br>
        </td>
       <td valign=\"top\">Pa<br>
       </td>
       </tr>
       <tr>
       <td valign=\"top\">&nbsp;2<br>
      </td>
      <td valign=\"top\">Temperature</td>
       <td valign=\"top\">T<br>
       </td>
       <td valign=\"top\">K<br>
       </td>
       </tr>
       <tr>
       <td valign=\"top\">&nbsp;3<br>
      </td>
      <td valign=\"top\">Density</td>
        <td valign=\"top\">d<br>
        </td>
       <td valign=\"top\">kg/m<sup>3</sup><br>
       </td>
       </tr>
       <tr>
       <td valign=\"top\">&nbsp;4<br>
      </td>
      <td valign=\"top\">Specific volume</td>
        <td valign=\"top\">v<br>
        </td>
       <td valign=\"top\">m<sup>3</sup>/kg<br>
       </td>
       </tr>
       <tr>
       <td valign=\"top\">&nbsp;5<br>
      </td>
      <td valign=\"top\">Specific enthalpy</td>
       <td valign=\"top\">h<br>
       </td>
       <td valign=\"top\">J/kg<br>
       </td>
       </tr>
       <tr>
       <td valign=\"top\">&nbsp;6<br>
      </td>
      <td valign=\"top\">Specific entropy</td>
       <td valign=\"top\">s<br>
       </td>
       <td valign=\"top\">J/(kg K)<br>
       </td>
       </tr>
       <tr>
       <td valign=\"top\">&nbsp;7<br>
      </td>
      <td valign=\"top\">Specific internal energy<br>
       </td>
       <td valign=\"top\">u<br>
       </td>
       <td valign=\"top\">J/kg<br>
       </td>
       </tr>
       <tr>
       <td valign=\"top\">&nbsp;8<br>
      </td>
      <td valign=\"top\">Specific isobaric heat capacity</td>
       <td valign=\"top\">c<sub>p</sub><br>
       </td>
       <td valign=\"top\">J/(kg K)<br>
       </td>
       </tr>
       <tr>
       <td valign=\"top\">&nbsp;9<br>
      </td>
      <td valign=\"top\">Specific isochoric heat capacity</td>
       <td valign=\"top\">c<sub>v</sub><br>
       </td>
       <td valign=\"top\">J/(kg K)<br>
       </td>
       </tr>
       <tr>
       <td valign=\"top\">10<br>
      </td>
      <td valign=\"top\">Isentropic exponent, kappa<span class=\"nobr\">=<font face=\"Symbol\">-</font>(v/p)
(dp/dv)<sub>s</sub></span></td>
     <td valign=\"top\">kappa (<font face=\"Symbol\">k</font>)<br>
     </td>
     <td valign=\"top\">1<br>
     </td>
     </tr>
     <tr>
     <td valign=\"top\">11<br>
      </td>
      <td valign=\"top\">Speed of sound<br>
     </td>
     <td valign=\"top\">a<br>
     </td>
     <td valign=\"top\">m/s<br>
     </td>
     </tr>
     <tr>
     <td valign=\"top\">12<br>
      </td>
      <td valign=\"top\">Dryness fraction<br>
     </td>
     <td valign=\"top\">x<br>
     </td>
     <td valign=\"top\">kg/kg<br>
     </td>
     </tr>
     <tr>
     <td valign=\"top\">13<br>
      </td>
      <td valign=\"top\">Specific Helmholtz free energy,     f = u - Ts</td>
     <td valign=\"top\">f<br>
     </td>
     <td valign=\"top\">J/kg<br>
     </td>
     </tr>
     <tr>
     <td valign=\"top\">14<br>
      </td>
      <td valign=\"top\">Specific Gibbs free energy,     g = h - Ts</td>
     <td valign=\"top\">g<br>
     </td>
     <td valign=\"top\">J/kg<br>
     </td>
     </tr>
     <tr>
     <td valign=\"top\">15<br>
      </td>
      <td valign=\"top\">Isenthalpic exponent, <span class=\"nobr\"> theta = -(v/p)(dp/dv)<sub>h</sub></span></td>
     <td valign=\"top\">theta (<font face=\"Symbol\">q</font>)<br>
     </td>
     <td valign=\"top\">1<br>
     </td>
     </tr>
     <tr>
     <td valign=\"top\">16<br>
      </td>
      <td valign=\"top\">Isobaric volume expansion coefficient, alpha = v<sup>-1</sup>       (dv/dT)<sub>p</sub></td>
     <td valign=\"top\">alpha  (<font face=\"Symbol\">a</font>)<br>
     </td>
       <td valign=\"top\">1/K<br>
     </td>
     </tr>
     <tr>
     <td valign=\"top\">17<br>
      </td>
      <td valign=\"top\">Isochoric pressure coefficient,     <span class=\"nobr\">beta = p<sup><font face=\"Symbol\">-</font>1</sup>(dp/dT)<sub>v</sub></span></td>
     <td valign=\"top\">beta (<font face=\"Symbol\">b</font>)<br>
     </td>
     <td valign=\"top\">1/K<br>
     </td>
     </tr>
     <tr>
     <td valign=\"top\">18<br>
     </td>
     <td valign=\"top\">Isothermal compressibility, <span class=\"nobr\">gamma = <font
 face=\"Symbol\">-</font>v<sup><font face=\"Symbol\">-</font>1</sup>(dv/dp)<sub>T</sub></span></td>
     <td valign=\"top\">gamma (<font face=\"Symbol\">g</font>)<br>
     </td>
     <td valign=\"top\">1/Pa<br>
     </td>
     </tr>
     <!-- <tr><td valign=\"top\">f</td><td valign=\"top\">Fugacity</td></tr> --> <tr>
     <td valign=\"top\">19<br>
      </td>
      <td valign=\"top\">Dynamic viscosity</td>
     <td valign=\"top\">eta (<font face=\"Symbol\">h</font>)<br>
     </td>
     <td valign=\"top\">Pa s<br>
     </td>
     </tr>
     <tr>
     <td valign=\"top\">20<br>
      </td>
      <td valign=\"top\">Kinematic viscosity</td>
     <td valign=\"top\">nu (<font face=\"Symbol\">n</font>)<br>
     </td>
     <td valign=\"top\">m<sup>2</sup>/s<br>
     </td>
     </tr>
     <!-- <tr><td valign=\"top\">Pr</td><td valign=\"top\">Prandtl number</td></tr> --> <tr>
     <td valign=\"top\">21<br>
      </td>
      <td valign=\"top\">Thermal conductivity</td>
     <td valign=\"top\">lambda (<font face=\"Symbol\">l</font>)<br>
     </td>
     <td valign=\"top\">W/(m K)<br>
     </td>
     </tr>
     <tr>
     <td valign=\"top\">22 <br>
      </td>
      <td valign=\"top\">Surface tension</td>
     <td valign=\"top\">sigma (<font face=\"Symbol\">s</font>)<br>
     </td>
     <td valign=\"top\">N/m<br>
     </td>
     </tr>
  </tbody>
</table>
        <p>The properties 1-11 are calculated by default with the functions for dynamic
        simulation, 2 of these variables are the dynamic states and are the inputs
        to calculate all other properties. In addition to these properties
        of general interest, the entries to the thermodynamic Jacobian matrix which render
        the mass- and energy balances explicit in the input variables to the property calculation are also calculated.
        For an explanatory example using pressure and specific enthalpy as states, see the Examples sub-package.</p>
        <p>The high-level calls to steam properties are grouped into records comprising both the properties of general interest
        and the entries to the thermodynamic Jacobian. If additional properties are
        needed the low level functions in BaseIF97 provide more choice.</p>
        <HR size=3 width=\"90%\">
        <h4>Additional functions</h4>
        <ul>
        <li>Function <b>boundaryvals_p</b> computes the temperature and the specific enthalpy and
        entropy on both phase boundaries as a function of p</li>
        <li>Function <b>boundaryderivs_p</b> is the Modelica derivative function of <b>boundaryvals_p</b></li>
        <li>Function <b>extraDerivs_ph</b> computes all entries to Bridgmans tables for all
        one-phase regions of IF97 using inputs (p,h). All 336 directional derivatives of the
        thermodynamic surface can be computed as a ratio of two entries in the return data, see package Common
        for details.</li>
        <li>Function <b>extraDerivs_pT</b> computes all entries to Bridgmans tables for all
        one-phase regions of IF97 using inputs (p,T).</li>
        </ul>
        </HTML>"));
        end BaseIF97;

        replaceable record iter = BaseIF97.IterationData;

        function waterBaseProp_ph "Intermediate property record for water"
          extends Modelica.Icons.Function;
          input SI.Pressure p "Pressure";
          input SI.SpecificEnthalpy h "Specific enthalpy";
          input Integer phase = 0 "Phase: 2 for two-phase, 1 for one phase, 0 if unknown";
          input Integer region = 0 "If 0, do region computation, otherwise assume the region is this input";
          output Common.IF97BaseTwoPhase aux "Auxiliary record";
        protected
          Common.GibbsDerivs g "Dimensionless Gibbs function and derivatives w.r.t. pi and tau";
          Common.HelmholtzDerivs f "Dimensionless Helmholtz function and derivatives w.r.t. delta and tau";
          Integer error "Error flag for inverse iterations";
          SI.SpecificEnthalpy h_liq "Liquid specific enthalpy";
          SI.Density d_liq "Liquid density";
          SI.SpecificEnthalpy h_vap "Vapour specific enthalpy";
          SI.Density d_vap "Vapour density";
          Common.PhaseBoundaryProperties liq "Phase boundary property record";
          Common.PhaseBoundaryProperties vap "Phase boundary property record";
          Common.GibbsDerivs gl "Dimensionless Gibbs function and derivatives w.r.t. pi and tau";
          Common.GibbsDerivs gv "Dimensionless Gibbs function and derivatives w.r.t. pi and tau";
          Modelica.Media.Common.HelmholtzDerivs fl "Dimensionless Helmholtz function and derivatives w.r.t. delta and tau";
          Modelica.Media.Common.HelmholtzDerivs fv "Dimensionless Helmholtz function and derivatives w.r.t. delta and tau";
          SI.Temperature t1 "Temperature at phase boundary, using inverse from region 1";
          SI.Temperature t2 "Temperature at phase boundary, using inverse from region 2";
        algorithm
          aux.region := if region == 0 then if phase == 2 then 4 else BaseIF97.Regions.region_ph(p = p, h = h, phase = phase) else region;
          aux.phase := if phase <> 0 then phase else if aux.region == 4 then 2 else 1;
          aux.p := max(p, 611.657);
          aux.h := max(h, 1e3);
          aux.R := BaseIF97.data.RH2O;
          if aux.region == 1 then
            aux.T := BaseIF97.Basic.tph1(aux.p, aux.h);
            g := BaseIF97.Basic.g1(p, aux.T);
            aux.s := aux.R * (g.tau * g.gtau - g.g);
            aux.rho := p / (aux.R * aux.T * g.pi * g.gpi);
            aux.vt := aux.R / p * (g.pi * g.gpi - g.tau * g.pi * g.gtaupi);
            aux.vp := aux.R * aux.T / (p * p) * g.pi * g.pi * g.gpipi;
            aux.cp := -aux.R * g.tau * g.tau * g.gtautau;
            aux.cv := aux.R * ((-g.tau * g.tau * g.gtautau) + (g.gpi - g.tau * g.gtaupi) * (g.gpi - g.tau * g.gtaupi) / g.gpipi);
            aux.x := 0.0;
            aux.dpT := -aux.vt / aux.vp;
          elseif aux.region == 2 then
            aux.T := BaseIF97.Basic.tph2(aux.p, aux.h);
            g := BaseIF97.Basic.g2(p, aux.T);
            aux.s := aux.R * (g.tau * g.gtau - g.g);
            aux.rho := p / (aux.R * aux.T * g.pi * g.gpi);
            aux.vt := aux.R / p * (g.pi * g.gpi - g.tau * g.pi * g.gtaupi);
            aux.vp := aux.R * aux.T / (p * p) * g.pi * g.pi * g.gpipi;
            aux.cp := -aux.R * g.tau * g.tau * g.gtautau;
            aux.cv := aux.R * ((-g.tau * g.tau * g.gtautau) + (g.gpi - g.tau * g.gtaupi) * (g.gpi - g.tau * g.gtaupi) / g.gpipi);
            aux.x := 1.0;
            aux.dpT := -aux.vt / aux.vp;
          elseif aux.region == 3 then
            (aux.rho, aux.T, error) := BaseIF97.Inverses.dtofph3(p = aux.p, h = aux.h, delp = 1.0e-7, delh = 1.0e-6);
            f := BaseIF97.Basic.f3(aux.rho, aux.T);
            aux.h := aux.R * aux.T * (f.tau * f.ftau + f.delta * f.fdelta);
            aux.s := aux.R * (f.tau * f.ftau - f.f);
            aux.pd := aux.R * aux.T * f.delta * (2.0 * f.fdelta + f.delta * f.fdeltadelta);
            aux.pt := aux.R * aux.rho * f.delta * (f.fdelta - f.tau * f.fdeltatau);
            aux.cv := abs(aux.R * (-f.tau * f.tau * f.ftautau)) "Can be close to neg. infinity near critical point";
            aux.cp := (aux.rho * aux.rho * aux.pd * aux.cv + aux.T * aux.pt * aux.pt) / (aux.rho * aux.rho * aux.pd);
            aux.x := 0.0;
            aux.dpT := aux.pt;
          elseif aux.region == 4 then
            h_liq := hl_p(p);
            h_vap := hv_p(p);
            aux.x := if h_vap <> h_liq then (h - h_liq) / (h_vap - h_liq) else 1.0;
            if p < BaseIF97.data.PLIMIT4A then
              t1 := BaseIF97.Basic.tph1(aux.p, h_liq);
              t2 := BaseIF97.Basic.tph2(aux.p, h_vap);
              gl := BaseIF97.Basic.g1(aux.p, t1);
              gv := BaseIF97.Basic.g2(aux.p, t2);
              liq := Common.gibbsToBoundaryProps(gl);
              vap := Common.gibbsToBoundaryProps(gv);
              aux.T := t1 + aux.x * (t2 - t1);
            else
              aux.T := BaseIF97.Basic.tsat(aux.p);
              d_liq := rhol_T(aux.T);
              d_vap := rhov_T(aux.T);
              fl := BaseIF97.Basic.f3(d_liq, aux.T);
              fv := BaseIF97.Basic.f3(d_vap, aux.T);
              liq := Common.helmholtzToBoundaryProps(fl);
              vap := Common.helmholtzToBoundaryProps(fv);
            end if;
            aux.dpT := if liq.d <> vap.d then (vap.s - liq.s) * liq.d * vap.d / (liq.d - vap.d) else BaseIF97.Basic.dptofT(aux.T);
            aux.s := liq.s + aux.x * (vap.s - liq.s);
            aux.rho := liq.d * vap.d / (vap.d + aux.x * (liq.d - vap.d));
            aux.cv := Common.cv2Phase(liq, vap, aux.x, aux.T, p);
            aux.cp := liq.cp + aux.x * (vap.cp - liq.cp);
            aux.pt := liq.pt + aux.x * (vap.pt - liq.pt);
            aux.pd := liq.pd + aux.x * (vap.pd - liq.pd);
          elseif aux.region == 5 then
            (aux.T, error) := BaseIF97.Inverses.tofph5(p = aux.p, h = aux.h, reldh = 1.0e-7);
            assert(error == 0, "Error in inverse iteration of steam tables");
            g := BaseIF97.Basic.g5(aux.p, aux.T);
            aux.s := aux.R * (g.tau * g.gtau - g.g);
            aux.rho := p / (aux.R * aux.T * g.pi * g.gpi);
            aux.vt := aux.R / p * (g.pi * g.gpi - g.tau * g.pi * g.gtaupi);
            aux.vp := aux.R * aux.T / (p * p) * g.pi * g.pi * g.gpipi;
            aux.cp := -aux.R * g.tau * g.tau * g.gtautau;
            aux.cv := aux.R * ((-g.tau * g.tau * g.gtautau) + (g.gpi - g.tau * g.gtaupi) * (g.gpi - g.tau * g.gtaupi) / g.gpipi);
            aux.dpT := -aux.vt / aux.vp;
          else
            assert(false, "Error in region computation of IF97 steam tables" + "(p = " + String(p) + ", h = " + String(h) + ")");
          end if;
/*safety against div-by-0 in initialization*/
// how to avoid ?
//  aux.dpT := BaseIF97.Basic.dptofT(aux.T);
        end waterBaseProp_ph;

        function waterBaseProp_ps "Intermediate property record for water"
          extends Modelica.Icons.Function;
          input SI.Pressure p "Pressure";
          input SI.SpecificEntropy s "Specific entropy";
          input Integer phase = 0 "Phase: 2 for two-phase, 1 for one phase, 0 if unknown";
          input Integer region = 0 "If 0, do region computation, otherwise assume the region is this input";
          output Common.IF97BaseTwoPhase aux "Auxiliary record";
        protected
          Common.GibbsDerivs g "Dimensionless Gibbs function and derivatives w.r.t. pi and tau";
          Common.HelmholtzDerivs f "Dimensionless Helmholtz function and derivatives w.r.t. delta and tau";
          Integer error "Error flag for inverse iterations";
          SI.SpecificEntropy s_liq "Liquid specific entropy";
          SI.Density d_liq "Liquid density";
          SI.SpecificEntropy s_vap "Vapour specific entropy";
          SI.Density d_vap "Vapour density";
          Common.PhaseBoundaryProperties liq "Phase boundary property record";
          Common.PhaseBoundaryProperties vap "Phase boundary property record";
          Common.GibbsDerivs gl "Dimensionless Gibbs function and derivatives w.r.t. pi and tau";
          Common.GibbsDerivs gv "Dimensionless Gibbs function and derivatives w.r.t. pi and tau";
          Modelica.Media.Common.HelmholtzDerivs fl "Dimensionless Helmholtz function and derivatives w.r.t. delta and tau";
          Modelica.Media.Common.HelmholtzDerivs fv "Dimensionless Helmholtz function and derivatives w.r.t. delta and tau";
          SI.Temperature t1 "Temperature at phase boundary, using inverse from region 1";
          SI.Temperature t2 "Temperature at phase boundary, using inverse from region 2";
        algorithm
          aux.region := if region == 0 then if phase == 2 then 4 else BaseIF97.Regions.region_ps(p = p, s = s, phase = phase) else region;
          aux.phase := if phase <> 0 then phase else if aux.region == 4 then 2 else 1;
          aux.p := p;
          aux.s := s;
          aux.R := BaseIF97.data.RH2O;
          if aux.region == 1 then
            aux.T := BaseIF97.Basic.tps1(p, s);
            g := BaseIF97.Basic.g1(p, aux.T);
            aux.h := aux.R * aux.T * g.tau * g.gtau;
            aux.rho := p / (aux.R * aux.T * g.pi * g.gpi);
            aux.vt := aux.R / p * (g.pi * g.gpi - g.tau * g.pi * g.gtaupi);
            aux.vp := aux.R * aux.T / (p * p) * g.pi * g.pi * g.gpipi;
            aux.cp := -aux.R * g.tau * g.tau * g.gtautau;
            aux.cv := aux.R * ((-g.tau * g.tau * g.gtautau) + (g.gpi - g.tau * g.gtaupi) * (g.gpi - g.tau * g.gtaupi) / g.gpipi);
            aux.x := 0.0;
            aux.dpT := -aux.vt / aux.vp;
          elseif aux.region == 2 then
            aux.T := BaseIF97.Basic.tps2(p, s);
            g := BaseIF97.Basic.g2(p, aux.T);
            aux.h := aux.R * aux.T * g.tau * g.gtau;
            aux.rho := p / (aux.R * aux.T * g.pi * g.gpi);
            aux.vt := aux.R / p * (g.pi * g.gpi - g.tau * g.pi * g.gtaupi);
            aux.vp := aux.R * aux.T / (p * p) * g.pi * g.pi * g.gpipi;
            aux.cp := -aux.R * g.tau * g.tau * g.gtautau;
            aux.cv := aux.R * ((-g.tau * g.tau * g.gtautau) + (g.gpi - g.tau * g.gtaupi) * (g.gpi - g.tau * g.gtaupi) / g.gpipi);
            aux.x := 1.0;
            aux.dpT := -aux.vt / aux.vp;
          elseif aux.region == 3 then
            (aux.rho, aux.T, error) := BaseIF97.Inverses.dtofps3(p = p, s = s, delp = 1.0e-7, dels = 1.0e-6);
            f := BaseIF97.Basic.f3(aux.rho, aux.T);
            aux.h := aux.R * aux.T * (f.tau * f.ftau + f.delta * f.fdelta);
            aux.s := aux.R * (f.tau * f.ftau - f.f);
            aux.pd := aux.R * aux.T * f.delta * (2.0 * f.fdelta + f.delta * f.fdeltadelta);
            aux.pt := aux.R * aux.rho * f.delta * (f.fdelta - f.tau * f.fdeltatau);
            aux.cv := aux.R * (-f.tau * f.tau * f.ftautau);
            aux.cp := (aux.rho * aux.rho * aux.pd * aux.cv + aux.T * aux.pt * aux.pt) / (aux.rho * aux.rho * aux.pd);
            aux.x := 0.0;
            aux.dpT := aux.pt;
          elseif aux.region == 4 then
            s_liq := BaseIF97.Regions.sl_p(p);
            s_vap := BaseIF97.Regions.sv_p(p);
            aux.x := if s_vap <> s_liq then (s - s_liq) / (s_vap - s_liq) else 1.0;
            if p < BaseIF97.data.PLIMIT4A then
              t1 := BaseIF97.Basic.tps1(p, s_liq);
              t2 := BaseIF97.Basic.tps2(p, s_vap);
              gl := BaseIF97.Basic.g1(p, t1);
              gv := BaseIF97.Basic.g2(p, t2);
              liq := Common.gibbsToBoundaryProps(gl);
              vap := Common.gibbsToBoundaryProps(gv);
              aux.T := t1 + aux.x * (t2 - t1);
            else
              aux.T := BaseIF97.Basic.tsat(p);
              d_liq := rhol_T(aux.T);
              d_vap := rhov_T(aux.T);
              fl := BaseIF97.Basic.f3(d_liq, aux.T);
              fv := BaseIF97.Basic.f3(d_vap, aux.T);
              liq := Common.helmholtzToBoundaryProps(fl);
              vap := Common.helmholtzToBoundaryProps(fv);
            end if;
            aux.dpT := if liq.d <> vap.d then (vap.s - liq.s) * liq.d * vap.d / (liq.d - vap.d) else BaseIF97.Basic.dptofT(aux.T);
            aux.h := liq.h + aux.x * (vap.h - liq.h);
            aux.rho := liq.d * vap.d / (vap.d + aux.x * (liq.d - vap.d));
            aux.cv := Common.cv2Phase(liq, vap, aux.x, aux.T, p);
            aux.cp := liq.cp + aux.x * (vap.cp - liq.cp);
            aux.pt := liq.pt + aux.x * (vap.pt - liq.pt);
            aux.pd := liq.pd + aux.x * (vap.pd - liq.pd);
          elseif aux.region == 5 then
            (aux.T, error) := BaseIF97.Inverses.tofps5(p = p, s = s, relds = 1.0e-7);
            assert(error == 0, "Error in inverse iteration of steam tables");
            g := BaseIF97.Basic.g5(p, aux.T);
            aux.h := aux.R * aux.T * g.tau * g.gtau;
            aux.rho := p / (aux.R * aux.T * g.pi * g.gpi);
            aux.vt := aux.R / p * (g.pi * g.gpi - g.tau * g.pi * g.gtaupi);
            aux.vp := aux.R * aux.T / (p * p) * g.pi * g.pi * g.gpipi;
            aux.cp := -aux.R * g.tau * g.tau * g.gtautau;
            aux.cv := aux.R * ((-g.tau * g.tau * g.gtautau) + (g.gpi - g.tau * g.gtaupi) * (g.gpi - g.tau * g.gtaupi) / g.gpipi);
            aux.dpT := -aux.vt / aux.vp;
            aux.x := 1.0;
          else
            assert(false, "Error in region computation of IF97 steam tables" + "(p = " + String(p) + ", s = " + String(s) + ")");
          end if;
/*safety against div-by-0 in initialization*/
        end waterBaseProp_ps;

        function rho_props_ps "Density as function of pressure and specific entropy"
          extends Modelica.Icons.Function;
          input SI.Pressure p "Pressure";
          input SI.SpecificEntropy s "Specific entropy";
          input Common.IF97BaseTwoPhase properties "Auxiliary record";
          output SI.Density rho "Density";
        algorithm
          rho := properties.rho;
          annotation(
            Inline = false,
            LateInline = true);
        end rho_props_ps;

        function rho_ps "Density as function of pressure and specific entropy"
          extends Modelica.Icons.Function;
          input SI.Pressure p "Pressure";
          input SI.SpecificEntropy s "Specific entropy";
          input Integer phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          input Integer region = 0 "If 0, region is unknown, otherwise known and this input";
          output SI.Density rho "Density";
        algorithm
          rho := rho_props_ps(p, s, waterBaseProp_ps(p, s, phase, region));
          annotation(
            Inline = true);
        end rho_ps;

        function T_props_ps "Temperature as function of pressure and specific entropy"
          extends Modelica.Icons.Function;
          input SI.Pressure p "Pressure";
          input SI.SpecificEntropy s "Specific entropy";
          input Common.IF97BaseTwoPhase properties "Auxiliary record";
          output SI.Temperature T "Temperature";
        algorithm
          T := properties.T;
          annotation(
            Inline = false,
            LateInline = true);
        end T_props_ps;

        function T_ps "Temperature as function of pressure and specific entropy"
          extends Modelica.Icons.Function;
          input SI.Pressure p "Pressure";
          input SI.SpecificEntropy s "Specific entropy";
          input Integer phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          input Integer region = 0 "If 0, region is unknown, otherwise known and this input";
          output SI.Temperature T "Temperature";
        algorithm
          T := T_props_ps(p, s, waterBaseProp_ps(p, s, phase, region));
          annotation(
            Inline = true);
        end T_ps;

        function h_props_ps "Specific enthalpy as function or pressure and temperature"
          extends Modelica.Icons.Function;
          input SI.Pressure p "Pressure";
          input SI.SpecificEntropy s "Specific entropy";
          input Common.IF97BaseTwoPhase aux "Auxiliary record";
          output SI.SpecificEnthalpy h "Specific enthalpy";
        algorithm
          h := aux.h;
          annotation(
            Inline = false,
            LateInline = true);
        end h_props_ps;

        function h_ps "Specific enthalpy as function or pressure and temperature"
          extends Modelica.Icons.Function;
          input SI.Pressure p "Pressure";
          input SI.SpecificEntropy s "Specific entropy";
          input Integer phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          input Integer region = 0 "If 0, region is unknown, otherwise known and this input";
          output SI.SpecificEnthalpy h "Specific enthalpy";
        algorithm
          h := h_props_ps(p, s, waterBaseProp_ps(p, s, phase, region));
          annotation(
            Inline = true);
        end h_ps;

        function phase_ps "Phase as a function of pressure and specific entropy"
          extends Modelica.Icons.Function;
          input SI.Pressure p "Pressure";
          input SI.SpecificEntropy s "Specific entropy";
          output Integer phase "True if in liquid or gas or supercritical region";
        algorithm
          phase := if s < sl_p(p) or s > sv_p(p) or p > BaseIF97.data.PCRIT then 1 else 2;
          annotation(
            Inline = true);
        end phase_ps;

        function phase_ph "Phase as a function of pressure and specific enthalpy"
          extends Modelica.Icons.Function;
          input SI.Pressure p "Pressure";
          input SI.SpecificEnthalpy h "Specific enthalpy";
          output Integer phase "True if in liquid or gas or supercritical region";
        algorithm
          phase := if h < hl_p(p) or h > hv_p(p) or p > BaseIF97.data.PCRIT then 1 else 2;
          annotation(
            Inline = true);
        end phase_ph;

        function phase_dT "Phase as a function of pressure and temperature"
          extends Modelica.Icons.Function;
          input SI.Density rho "Density";
          input SI.Temperature T "Temperature";
          output Integer phase "True if in liquid or gas or supercritical region";
        algorithm
          phase := if not (rho < rhol_T(T) and rho > rhov_T(T) and T < BaseIF97.data.TCRIT) then 1 else 2;
          annotation(
            Inline = true);
        end phase_dT;

        function rho_props_ph "Density as function of pressure and specific enthalpy"
          extends Modelica.Icons.Function;
          input SI.Pressure p "Pressure";
          input SI.SpecificEnthalpy h "Specific enthalpy";
          input Common.IF97BaseTwoPhase properties "Auxiliary record";
          output SI.Density rho "Density";
        algorithm
          rho := properties.rho;
          annotation(
            derivative(noDerivative = properties) = rho_ph_der,
            Inline = false,
            LateInline = true);
        end rho_props_ph;

        function rho_ph "Density as function of pressure and specific enthalpy"
          extends Modelica.Icons.Function;
          input SI.Pressure p "Pressure";
          input SI.SpecificEnthalpy h "Specific enthalpy";
          input Integer phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          input Integer region = 0 "If 0, region is unknown, otherwise known and this input";
          output SI.Density rho "Density";
        algorithm
          rho := rho_props_ph(p, h, waterBaseProp_ph(p, h, phase, region));
          annotation(
            Inline = true);
        end rho_ph;

        function rho_ph_der "Derivative function of rho_ph"
          extends Modelica.Icons.Function;
          input SI.Pressure p "Pressure";
          input SI.SpecificEnthalpy h "Specific enthalpy";
          input Common.IF97BaseTwoPhase properties "Auxiliary record";
          input Real p_der "Derivative of pressure";
          input Real h_der "Derivative of specific enthalpy";
          output Real rho_der "Derivative of density";
        algorithm
          if properties.region == 4 then
            rho_der := properties.rho * (properties.rho * properties.cv / properties.dpT + 1.0) / (properties.dpT * properties.T) * p_der + (-properties.rho * properties.rho / (properties.dpT * properties.T)) * h_der;
          elseif properties.region == 3 then
            rho_der := properties.rho * (properties.cv * properties.rho + properties.pt) / (properties.rho * properties.rho * properties.pd * properties.cv + properties.T * properties.pt * properties.pt) * p_der + (-properties.rho * properties.rho * properties.pt / (properties.rho * properties.rho * properties.pd * properties.cv + properties.T * properties.pt * properties.pt)) * h_der;
          else
            rho_der := (-properties.rho * properties.rho * (properties.vp * properties.cp - properties.vt / properties.rho + properties.T * properties.vt * properties.vt) / properties.cp) * p_der + (-properties.rho * properties.rho * properties.vt / properties.cp) * h_der;
          end if;
//regions 1,2,5
        end rho_ph_der;

        function T_props_ph "Temperature as function of pressure and specific enthalpy"
          extends Modelica.Icons.Function;
          input SI.Pressure p "Pressure";
          input SI.SpecificEnthalpy h "Specific enthalpy";
          input Common.IF97BaseTwoPhase properties "Auxiliary record";
          output SI.Temperature T "Temperature";
        algorithm
          T := properties.T;
          annotation(
            derivative(noDerivative = properties) = T_ph_der,
            Inline = false,
            LateInline = true);
        end T_props_ph;

        function T_ph "Temperature as function of pressure and specific enthalpy"
          extends Modelica.Icons.Function;
          input SI.Pressure p "Pressure";
          input SI.SpecificEnthalpy h "Specific enthalpy";
          input Integer phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          input Integer region = 0 "If 0, region is unknown, otherwise known and this input";
          output SI.Temperature T "Temperature";
        algorithm
          T := T_props_ph(p, h, waterBaseProp_ph(p, h, phase, region));
          annotation(
            Inline = true);
        end T_ph;

        function T_ph_der "Derivative function of T_ph"
          extends Modelica.Icons.Function;
          input SI.Pressure p "Pressure";
          input SI.SpecificEnthalpy h "Specific enthalpy";
          input Common.IF97BaseTwoPhase properties "Auxiliary record";
          input Real p_der "Derivative of pressure";
          input Real h_der "Derivative of specific enthalpy";
          output Real T_der "Derivative of temperature";
        algorithm
          if properties.region == 4 then
            T_der := 1 / properties.dpT * p_der;
          elseif properties.region == 3 then
            T_der := ((-properties.rho * properties.pd) + properties.T * properties.pt) / (properties.rho * properties.rho * properties.pd * properties.cv + properties.T * properties.pt * properties.pt) * p_der + properties.rho * properties.rho * properties.pd / (properties.rho * properties.rho * properties.pd * properties.cv + properties.T * properties.pt * properties.pt) * h_der;
          else
            T_der := ((-1 / properties.rho) + properties.T * properties.vt) / properties.cp * p_der + 1 / properties.cp * h_der;
          end if;
//regions 1,2 or 5
        end T_ph_der;

        function s_props_ph "Specific entropy as function of pressure and specific enthalpy"
          extends Modelica.Icons.Function;
          input SI.Pressure p "Pressure";
          input SI.SpecificEnthalpy h "Specific enthalpy";
          input Common.IF97BaseTwoPhase properties "Auxiliary record";
          output SI.SpecificEntropy s "Specific entropy";
        algorithm
          s := properties.s;
          annotation(
            derivative(noDerivative = properties) = s_ph_der,
            Inline = false,
            LateInline = true);
        end s_props_ph;

        function s_ph "Specific entropy as function of pressure and specific enthalpy"
          extends Modelica.Icons.Function;
          input SI.Pressure p "Pressure";
          input SI.SpecificEnthalpy h "Specific enthalpy";
          input Integer phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          input Integer region = 0 "If 0, region is unknown, otherwise known and this input";
          output SI.SpecificEntropy s "Specific entropy";
        algorithm
          s := s_props_ph(p, h, waterBaseProp_ph(p, h, phase, region));
          annotation(
            Inline = true);
        end s_ph;

        function s_ph_der "Specific entropy as function of pressure and specific enthalpy"
          extends Modelica.Icons.Function;
          input SI.Pressure p "Pressure";
          input SI.SpecificEnthalpy h "Specific enthalpy";
          input Common.IF97BaseTwoPhase properties "Auxiliary record";
          input Real p_der "Derivative of pressure";
          input Real h_der "Derivative of specific enthalpy";
          output Real s_der "Derivative of entropy";
        algorithm
          s_der := (-1 / (properties.rho * properties.T) * p_der) + 1 / properties.T * h_der;
          annotation(
            Inline = true);
        end s_ph_der;

        function cv_props_ph "Specific heat capacity at constant volume as function of pressure and specific enthalpy"
          extends Modelica.Icons.Function;
          input SI.Pressure p "Pressure";
          input SI.SpecificEnthalpy h "Specific enthalpy";
          input Common.IF97BaseTwoPhase aux "Auxiliary record";
          output SI.SpecificHeatCapacity cv "Specific heat capacity";
        algorithm
          cv := aux.cv;
          annotation(
            Inline = false,
            LateInline = true);
        end cv_props_ph;

        function cv_ph "Specific heat capacity at constant volume as function of pressure and specific enthalpy"
          extends Modelica.Icons.Function;
          input SI.Pressure p "Pressure";
          input SI.SpecificEnthalpy h "Specific enthalpy";
          input Integer phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          input Integer region = 0 "If 0, region is unknown, otherwise known and this input";
          output SI.SpecificHeatCapacity cv "Specific heat capacity";
        algorithm
          cv := cv_props_ph(p, h, waterBaseProp_ph(p, h, phase, region));
          annotation(
            Inline = true);
        end cv_ph;

        function regionAssertReal "Assert function for inlining"
          extends Modelica.Icons.Function;
          input Boolean check "Condition to check";
          output Real dummy "Dummy output";
        algorithm
          assert(check, "This function can not be called with two-phase inputs!");
        end regionAssertReal;

        function cp_props_ph "Specific heat capacity at constant pressure as function of pressure and specific enthalpy"
          extends Modelica.Icons.Function;
          input SI.Pressure p "Pressure";
          input SI.SpecificEnthalpy h "Specific enthalpy";
          input Common.IF97BaseTwoPhase aux "Auxiliary record";
          output SI.SpecificHeatCapacity cp "Specific heat capacity";
        algorithm
          cp := aux.cp;
          annotation(
            Inline = false,
            LateInline = true);
        end cp_props_ph;

        function cp_ph "Specific heat capacity at constant pressure as function of pressure and specific enthalpy"
          extends Modelica.Icons.Function;
          input SI.Pressure p "Pressure";
          input SI.SpecificEnthalpy h "Specific enthalpy";
          input Integer phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          input Integer region = 0 "If 0, region is unknown, otherwise known and this input";
          output SI.SpecificHeatCapacity cp "Specific heat capacity";
        algorithm
          cp := cp_props_ph(p, h, waterBaseProp_ph(p, h, phase, region));
          annotation(
            Inline = true);
        end cp_ph;

        function beta_props_ph "Isobaric expansion coefficient as function of pressure and specific enthalpy"
          extends Modelica.Icons.Function;
          input SI.Pressure p "Pressure";
          input SI.SpecificEnthalpy h "Specific enthalpy";
          input Common.IF97BaseTwoPhase aux "Auxiliary record";
          output SI.RelativePressureCoefficient beta "Isobaric expansion coefficient";
        algorithm
          beta := if aux.region == 3 or aux.region == 4 then aux.pt / (aux.rho * aux.pd) else aux.vt * aux.rho;
          annotation(
            Inline = false,
            LateInline = true);
        end beta_props_ph;

        function beta_ph "Isobaric expansion coefficient as function of pressure and specific enthalpy"
          extends Modelica.Icons.Function;
          input SI.Pressure p "Pressure";
          input SI.SpecificEnthalpy h "Specific enthalpy";
          input Integer phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          input Integer region = 0 "If 0, region is unknown, otherwise known and this input";
          output SI.RelativePressureCoefficient beta "Isobaric expansion coefficient";
        algorithm
          beta := beta_props_ph(p, h, waterBaseProp_ph(p, h, phase, region));
          annotation(
            Inline = true);
        end beta_ph;

        function kappa_props_ph "Isothermal compressibility factor as function of pressure and specific enthalpy"
          extends Modelica.Icons.Function;
          input SI.Pressure p "Pressure";
          input SI.SpecificEnthalpy h "Specific enthalpy";
          input Common.IF97BaseTwoPhase aux "Auxiliary record";
          output SI.IsothermalCompressibility kappa "Isothermal compressibility factor";
        algorithm
          kappa := if aux.region == 3 or aux.region == 4 then 1 / (aux.rho * aux.pd) else -aux.vp * aux.rho;
          annotation(
            Inline = false,
            LateInline = true);
        end kappa_props_ph;

        function kappa_ph "Isothermal compressibility factor as function of pressure and specific enthalpy"
          extends Modelica.Icons.Function;
          input SI.Pressure p "Pressure";
          input SI.SpecificEnthalpy h "Specific enthalpy";
          input Integer phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          input Integer region = 0 "If 0, region is unknown, otherwise known and this input";
          output SI.IsothermalCompressibility kappa "Isothermal compressibility factor";
        algorithm
          kappa := kappa_props_ph(p, h, waterBaseProp_ph(p, h, phase, region));
          annotation(
            Inline = true);
        end kappa_ph;

        function velocityOfSound_props_ph "Speed of sound as function of pressure and specific enthalpy"
          extends Modelica.Icons.Function;
          input SI.Pressure p "Pressure";
          input SI.SpecificEnthalpy h "Specific enthalpy";
          input Common.IF97BaseTwoPhase aux "Auxiliary record";
          output SI.Velocity v_sound "Speed of sound";
        algorithm
// dp/drho at constant s
          v_sound := if aux.region == 3 then sqrt(max(0, (aux.pd * aux.rho * aux.rho * aux.cv + aux.pt * aux.pt * aux.T) / (aux.rho * aux.rho * aux.cv))) else if aux.region == 4 then sqrt(max(0, 1 / (aux.rho * (aux.rho * aux.cv / aux.dpT + 1.0) / (aux.dpT * aux.T) - 1 / aux.rho * aux.rho * aux.rho / (aux.dpT * aux.T)))) else sqrt(max(0, -aux.cp / (aux.rho * aux.rho * (aux.vp * aux.cp + aux.vt * aux.vt * aux.T))));
          annotation(
            Inline = false,
            LateInline = true);
        end velocityOfSound_props_ph;

        function velocityOfSound_ph
          extends Modelica.Icons.Function;
          input SI.Pressure p "Pressure";
          input SI.SpecificEnthalpy h "Specific enthalpy";
          input Integer phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          input Integer region = 0 "If 0, region is unknown, otherwise known and this input";
          output SI.Velocity v_sound "Speed of sound";
        algorithm
          v_sound := velocityOfSound_props_ph(p, h, waterBaseProp_ph(p, h, phase, region));
          annotation(
            Inline = true);
        end velocityOfSound_ph;

        function isentropicExponent_props_ph "Isentropic exponent as function of pressure and specific enthalpy"
          extends Modelica.Icons.Function;
          input SI.Pressure p "Pressure";
          input SI.SpecificEnthalpy h "Specific enthalpy";
          input Common.IF97BaseTwoPhase aux "Auxiliary record";
          output Real gamma "Isentropic exponent";
        algorithm
          gamma := if aux.region == 3 then 1 / (aux.rho * p) * ((aux.pd * aux.cv * aux.rho * aux.rho + aux.pt * aux.pt * aux.T) / aux.cv) else if aux.region == 4 then 1 / (aux.rho * p) * aux.dpT * aux.dpT * aux.T / aux.cv else -1 / (aux.rho * aux.p) * aux.cp / (aux.vp * aux.cp + aux.vt * aux.vt * aux.T);
          annotation(
            Inline = false,
            LateInline = true);
        end isentropicExponent_props_ph;

        function isentropicExponent_ph "Isentropic exponent as function of pressure and specific enthalpy"
          extends Modelica.Icons.Function;
          input SI.Pressure p "Pressure";
          input SI.SpecificEnthalpy h "Specific enthalpy";
          input Integer phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          input Integer region = 0 "If 0, region is unknown, otherwise known and this input";
          output Real gamma "Isentropic exponent";
        algorithm
          gamma := isentropicExponent_props_ph(p, h, waterBaseProp_ph(p, h, phase, region));
          annotation(
            Inline = false,
            LateInline = true);
        end isentropicExponent_ph;

        function ddph_props "Density derivative by pressure"
          extends Modelica.Icons.Function;
          input SI.Pressure p "Pressure";
          input SI.SpecificEnthalpy h "Specific enthalpy";
          input Common.IF97BaseTwoPhase aux "Auxiliary record";
          output SI.DerDensityByPressure ddph "Density derivative by pressure";
        algorithm
          ddph := if aux.region == 3 then aux.rho * (aux.cv * aux.rho + aux.pt) / (aux.rho * aux.rho * aux.pd * aux.cv + aux.T * aux.pt * aux.pt) else if aux.region == 4 then aux.rho * (aux.rho * aux.cv / aux.dpT + 1.0) / (aux.dpT * aux.T) else -aux.rho * aux.rho * (aux.vp * aux.cp - aux.vt / aux.rho + aux.T * aux.vt * aux.vt) / aux.cp;
          annotation(
            Inline = false,
            LateInline = true);
        end ddph_props;

        function ddph "Density derivative by pressure"
          extends Modelica.Icons.Function;
          input SI.Pressure p "Pressure";
          input SI.SpecificEnthalpy h "Specific enthalpy";
          input Integer phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          input Integer region = 0 "If 0, region is unknown, otherwise known and this input";
          output SI.DerDensityByPressure ddph "Density derivative by pressure";
        algorithm
          ddph := ddph_props(p, h, waterBaseProp_ph(p, h, phase, region));
          annotation(
            Inline = true);
        end ddph;

        function ddhp_props "Density derivative by specific enthalpy"
          extends Modelica.Icons.Function;
          input SI.Pressure p "Pressure";
          input SI.SpecificEnthalpy h "Specific enthalpy";
          input Common.IF97BaseTwoPhase aux "Auxiliary record";
          output SI.DerDensityByEnthalpy ddhp "Density derivative by specific enthalpy";
        algorithm
          ddhp := if aux.region == 3 then -aux.rho * aux.rho * aux.pt / (aux.rho * aux.rho * aux.pd * aux.cv + aux.T * aux.pt * aux.pt) else if aux.region == 4 then -aux.rho * aux.rho / (aux.dpT * aux.T) else -aux.rho * aux.rho * aux.vt / aux.cp;
          annotation(
            Inline = false,
            LateInline = true);
        end ddhp_props;

        function ddhp "Density derivative by specific enthalpy"
          extends Modelica.Icons.Function;
          input SI.Pressure p "Pressure";
          input SI.SpecificEnthalpy h "Specific enthalpy";
          input Integer phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          input Integer region = 0 "If 0, region is unknown, otherwise known and this input";
          output SI.DerDensityByEnthalpy ddhp "Density derivative by specific enthalpy";
        algorithm
          ddhp := ddhp_props(p, h, waterBaseProp_ph(p, h, phase, region));
          annotation(
            Inline = true);
        end ddhp;

        function waterBaseProp_pT "Intermediate property record for water (p and T preferred states)"
          extends Modelica.Icons.Function;
          input SI.Pressure p "Pressure";
          input SI.Temperature T "Temperature";
          input Integer region = 0 "If 0, do region computation, otherwise assume the region is this input";
          output Common.IF97BaseTwoPhase aux "Auxiliary record";
        protected
          Common.GibbsDerivs g "Dimensionless Gibbs function and derivatives w.r.t. pi and tau";
          Common.HelmholtzDerivs f "Dimensionless Helmholtz function and derivatives w.r.t. delta and tau";
          Integer error "Error flag for inverse iterations";
        algorithm
          aux.phase := 1;
          aux.region := if region == 0 then BaseIF97.Regions.region_pT(p = p, T = T) else region;
          aux.R := BaseIF97.data.RH2O;
          aux.p := p;
          aux.T := T;
/* OpenModelica patch */
          aux.dpT := 0.0;
          aux.pt := 0.0;
          aux.pd := 0.0;
          aux.x := 0.0;
          aux.rho := 0.0;
          aux.vt := 0.0;
          aux.vp := 0.0;
          aux.cp := 0.0;
/* end OpenModelica patch */
          if aux.region == 1 then
            g := BaseIF97.Basic.g1(p, T);
            aux.h := aux.R * aux.T * g.tau * g.gtau;
            aux.s := aux.R * (g.tau * g.gtau - g.g);
            aux.rho := p / (aux.R * T * g.pi * g.gpi);
            aux.vt := aux.R / p * (g.pi * g.gpi - g.tau * g.pi * g.gtaupi);
            aux.vp := aux.R * T / (p * p) * g.pi * g.pi * g.gpipi;
            aux.cp := -aux.R * g.tau * g.tau * g.gtautau;
            aux.cv := aux.R * ((-g.tau * g.tau * g.gtautau) + (g.gpi - g.tau * g.gtaupi) * (g.gpi - g.tau * g.gtaupi) / g.gpipi);
            aux.x := 0.0;
            aux.dpT := -aux.vt / aux.vp;
          elseif aux.region == 2 then
            g := BaseIF97.Basic.g2(p, T);
            aux.h := aux.R * aux.T * g.tau * g.gtau;
            aux.s := aux.R * (g.tau * g.gtau - g.g);
            aux.rho := p / (aux.R * T * g.pi * g.gpi);
            aux.vt := aux.R / p * (g.pi * g.gpi - g.tau * g.pi * g.gtaupi);
            aux.vp := aux.R * T / (p * p) * g.pi * g.pi * g.gpipi;
            aux.cp := -aux.R * g.tau * g.tau * g.gtautau;
            aux.cv := aux.R * ((-g.tau * g.tau * g.gtautau) + (g.gpi - g.tau * g.gtaupi) * (g.gpi - g.tau * g.gtaupi) / g.gpipi);
            aux.x := 1.0;
            aux.dpT := -aux.vt / aux.vp;
          elseif aux.region == 3 then
            (aux.rho, error) := BaseIF97.Inverses.dofpt3(p = p, T = T, delp = 1.0e-7);
            f := BaseIF97.Basic.f3(aux.rho, T);
            aux.h := aux.R * T * (f.tau * f.ftau + f.delta * f.fdelta);
            aux.s := aux.R * (f.tau * f.ftau - f.f);
            aux.pd := aux.R * T * f.delta * (2.0 * f.fdelta + f.delta * f.fdeltadelta);
            aux.pt := aux.R * aux.rho * f.delta * (f.fdelta - f.tau * f.fdeltatau);
            aux.cv := aux.R * (-f.tau * f.tau * f.ftautau);
            aux.x := 0.0;
            aux.dpT := aux.pt;
          elseif aux.region == 5 then
            g := BaseIF97.Basic.g5(p, T);
            aux.h := aux.R * aux.T * g.tau * g.gtau;
            aux.s := aux.R * (g.tau * g.gtau - g.g);
            aux.rho := p / (aux.R * T * g.pi * g.gpi);
            aux.vt := aux.R / p * (g.pi * g.gpi - g.tau * g.pi * g.gtaupi);
            aux.vp := aux.R * T / (p * p) * g.pi * g.pi * g.gpipi;
            aux.cp := -aux.R * g.tau * g.tau * g.gtautau;
            aux.cv := aux.R * ((-g.tau * g.tau * g.gtautau) + (g.gpi - g.tau * g.gtaupi) * (g.gpi - g.tau * g.gtaupi) / g.gpipi);
            aux.x := 1.0;
            aux.dpT := -aux.vt / aux.vp;
          else
            assert(false, "Error in region computation of IF97 steam tables" + "(p = " + String(p) + ", T = " + String(T) + ")");
          end if;
/*safety against div-by-0 in initialization*/
        end waterBaseProp_pT;

        function rho_props_pT "Density as function or pressure and temperature"
          extends Modelica.Icons.Function;
          input SI.Pressure p "Pressure";
          input SI.Temperature T "Temperature";
          input Common.IF97BaseTwoPhase aux "Auxiliary record";
          output SI.Density rho "Density";
        algorithm
          rho := aux.rho;
          annotation(
            derivative(noDerivative = aux) = rho_pT_der,
            Inline = false,
            LateInline = true);
        end rho_props_pT;

        function rho_pT "Density as function or pressure and temperature"
          extends Modelica.Icons.Function;
          input SI.Pressure p "Pressure";
          input SI.Temperature T "Temperature";
          input Integer region = 0 "If 0, region is unknown, otherwise known and this input";
          output SI.Density rho "Density";
        algorithm
          rho := rho_props_pT(p, T, waterBaseProp_pT(p, T, region));
          annotation(
            Inline = true);
        end rho_pT;

        function h_props_pT "Specific enthalpy as function or pressure and temperature"
          extends Modelica.Icons.Function;
          input SI.Pressure p "Pressure";
          input SI.Temperature T "Temperature";
          input Common.IF97BaseTwoPhase aux "Auxiliary record";
          output SI.SpecificEnthalpy h "Specific enthalpy";
        algorithm
          h := aux.h;
          annotation(
            derivative(noDerivative = aux) = h_pT_der,
            Inline = false,
            LateInline = true);
        end h_props_pT;

        function h_pT "Specific enthalpy as function or pressure and temperature"
          extends Modelica.Icons.Function;
          input SI.Pressure p "Pressure";
          input SI.Temperature T "Temperature";
          input Integer region = 0 "If 0, region is unknown, otherwise known and this input";
          output SI.SpecificEnthalpy h "Specific enthalpy";
        algorithm
          h := h_props_pT(p, T, waterBaseProp_pT(p, T, region));
          annotation(
            Inline = true);
        end h_pT;

        function h_pT_der "Derivative function of h_pT"
          extends Modelica.Icons.Function;
          input SI.Pressure p "Pressure";
          input SI.Temperature T "Temperature";
          input Common.IF97BaseTwoPhase aux "Auxiliary record";
          input Real p_der "Derivative of pressure";
          input Real T_der "Derivative of temperature";
          output Real h_der "Derivative of specific enthalpy";
        algorithm
          if aux.region == 3 then
            h_der := ((-aux.rho * aux.pd) + T * aux.pt) / (aux.rho * aux.rho * aux.pd) * p_der + (aux.rho * aux.rho * aux.pd * aux.cv + aux.T * aux.pt * aux.pt) / (aux.rho * aux.rho * aux.pd) * T_der;
          else
            h_der := (1 / aux.rho - aux.T * aux.vt) * p_der + aux.cp * T_der;
          end if;
//regions 1,2 or 5
        end h_pT_der;

        function rho_pT_der "Derivative function of rho_pT"
          extends Modelica.Icons.Function;
          input SI.Pressure p "Pressure";
          input SI.Temperature T "Temperature";
          input Common.IF97BaseTwoPhase aux "Auxiliary record";
          input Real p_der "Derivative of pressure";
          input Real T_der "Derivative of temperature";
          output Real rho_der "Derivative of density";
        algorithm
          if aux.region == 3 then
            rho_der := 1 / aux.pd * p_der - aux.pt / aux.pd * T_der;
          else
            rho_der := (-aux.rho * aux.rho * aux.vp) * p_der + (-aux.rho * aux.rho * aux.vt) * T_der;
          end if;
//regions 1,2 or 5
        end rho_pT_der;

        function s_props_pT "Specific entropy as function of pressure and temperature"
          extends Modelica.Icons.Function;
          input SI.Pressure p "Pressure";
          input SI.Temperature T "Temperature";
          input Common.IF97BaseTwoPhase aux "Auxiliary record";
          output SI.SpecificEntropy s "Specific entropy";
        algorithm
          s := aux.s;
          annotation(
            Inline = false,
            LateInline = true);
        end s_props_pT;

        function s_pT "Temperature as function of pressure and temperature"
          extends Modelica.Icons.Function;
          input SI.Pressure p "Pressure";
          input SI.Temperature T "Temperature";
          input Integer region = 0 "If 0, region is unknown, otherwise known and this input";
          output SI.SpecificEntropy s "Specific entropy";
        algorithm
          s := s_props_pT(p, T, waterBaseProp_pT(p, T, region));
          annotation(
            Inline = true);
        end s_pT;

        function cv_props_pT "Specific heat capacity at constant volume as function of pressure and temperature"
          extends Modelica.Icons.Function;
          input SI.Pressure p "Pressure";
          input SI.Temperature T "Temperature";
          input Common.IF97BaseTwoPhase aux "Auxiliary record";
          output SI.SpecificHeatCapacity cv "Specific heat capacity";
        algorithm
          cv := aux.cv;
          annotation(
            Inline = false,
            LateInline = true);
        end cv_props_pT;

        function cv_pT "Specific heat capacity at constant volume as function of pressure and temperature"
          extends Modelica.Icons.Function;
          input SI.Pressure p "Pressure";
          input SI.Temperature T "Temperature";
          input Integer region = 0 "If 0, region is unknown, otherwise known and this input";
          output SI.SpecificHeatCapacity cv "Specific heat capacity";
        algorithm
          cv := cv_props_pT(p, T, waterBaseProp_pT(p, T, region));
          annotation(
            Inline = true);
        end cv_pT;

        function cp_props_pT "Specific heat capacity at constant pressure as function of pressure and temperature"
          extends Modelica.Icons.Function;
          input SI.Pressure p "Pressure";
          input SI.Temperature T "Temperature";
          input Common.IF97BaseTwoPhase aux "Auxiliary record";
          output SI.SpecificHeatCapacity cp "Specific heat capacity";
        algorithm
          cp := if aux.region == 3 then (aux.rho * aux.rho * aux.pd * aux.cv + aux.T * aux.pt * aux.pt) / (aux.rho * aux.rho * aux.pd) else aux.cp;
          annotation(
            Inline = false,
            LateInline = true);
        end cp_props_pT;

        function cp_pT "Specific heat capacity at constant pressure as function of pressure and temperature"
          extends Modelica.Icons.Function;
          input SI.Pressure p "Pressure";
          input SI.Temperature T "Temperature";
          input Integer region = 0 "If 0, region is unknown, otherwise known and this input";
          output SI.SpecificHeatCapacity cp "Specific heat capacity";
        algorithm
          cp := cp_props_pT(p, T, waterBaseProp_pT(p, T, region));
          annotation(
            Inline = true);
        end cp_pT;

        function beta_props_pT "Isobaric expansion coefficient as function of pressure and temperature"
          extends Modelica.Icons.Function;
          input SI.Pressure p "Pressure";
          input SI.Temperature T "Temperature";
          input Common.IF97BaseTwoPhase aux "Auxiliary record";
          output SI.RelativePressureCoefficient beta "Isobaric expansion coefficient";
        algorithm
          beta := if aux.region == 3 then aux.pt / (aux.rho * aux.pd) else aux.vt * aux.rho;
          annotation(
            Inline = false,
            LateInline = true);
        end beta_props_pT;

        function beta_pT "Isobaric expansion coefficient as function of pressure and temperature"
          extends Modelica.Icons.Function;
          input SI.Pressure p "Pressure";
          input SI.Temperature T "Temperature";
          input Integer region = 0 "If 0, region is unknown, otherwise known and this input";
          output SI.RelativePressureCoefficient beta "Isobaric expansion coefficient";
        algorithm
          beta := beta_props_pT(p, T, waterBaseProp_pT(p, T, region));
          annotation(
            Inline = true);
        end beta_pT;

        function kappa_props_pT "Isothermal compressibility factor as function of pressure and temperature"
          extends Modelica.Icons.Function;
          input SI.Pressure p "Pressure";
          input SI.Temperature T "Temperature";
          input Common.IF97BaseTwoPhase aux "Auxiliary record";
          output SI.IsothermalCompressibility kappa "Isothermal compressibility factor";
        algorithm
          kappa := if aux.region == 3 then 1 / (aux.rho * aux.pd) else -aux.vp * aux.rho;
          annotation(
            Inline = false,
            LateInline = true);
        end kappa_props_pT;

        function kappa_pT "Isothermal compressibility factor as function of pressure and temperature"
          extends Modelica.Icons.Function;
          input SI.Pressure p "Pressure";
          input SI.Temperature T "Temperature";
          input Integer region = 0 "If 0, region is unknown, otherwise known and this input";
          output SI.IsothermalCompressibility kappa "Isothermal compressibility factor";
        algorithm
          kappa := kappa_props_pT(p, T, waterBaseProp_pT(p, T, region));
          annotation(
            Inline = true);
        end kappa_pT;

        function velocityOfSound_props_pT "Speed of sound as function of pressure and temperature"
          extends Modelica.Icons.Function;
          input SI.Pressure p "Pressure";
          input SI.Temperature T "Temperature";
          input Common.IF97BaseTwoPhase aux "Auxiliary record";
          output SI.Velocity v_sound "Speed of sound";
        algorithm
// dp/drho at constant s
          v_sound := if aux.region == 3 then sqrt(max(0, (aux.pd * aux.rho * aux.rho * aux.cv + aux.pt * aux.pt * aux.T) / (aux.rho * aux.rho * aux.cv))) else sqrt(max(0, -aux.cp / (aux.rho * aux.rho * (aux.vp * aux.cp + aux.vt * aux.vt * aux.T))));
          annotation(
            Inline = false,
            LateInline = true);
        end velocityOfSound_props_pT;

        function velocityOfSound_pT "Speed of sound as function of pressure and temperature"
          extends Modelica.Icons.Function;
          input SI.Pressure p "Pressure";
          input SI.Temperature T "Temperature";
          input Integer region = 0 "If 0, region is unknown, otherwise known and this input";
          output SI.Velocity v_sound "Speed of sound";
        algorithm
          v_sound := velocityOfSound_props_pT(p, T, waterBaseProp_pT(p, T, region));
          annotation(
            Inline = true);
        end velocityOfSound_pT;

        function isentropicExponent_props_pT "Isentropic exponent as function of pressure and temperature"
          extends Modelica.Icons.Function;
          input SI.Pressure p "Pressure";
          input SI.Temperature T "Temperature";
          input Common.IF97BaseTwoPhase aux "Auxiliary record";
          output Real gamma "Isentropic exponent";
        algorithm
          gamma := if aux.region == 3 then 1 / (aux.rho * p) * ((aux.pd * aux.cv * aux.rho * aux.rho + aux.pt * aux.pt * aux.T) / aux.cv) else -1 / (aux.rho * aux.p) * aux.cp / (aux.vp * aux.cp + aux.vt * aux.vt * aux.T);
          annotation(
            Inline = false,
            LateInline = true);
        end isentropicExponent_props_pT;

        function isentropicExponent_pT "Isentropic exponent as function of pressure and temperature"
          extends Modelica.Icons.Function;
          input SI.Pressure p "Pressure";
          input SI.Temperature T "Temperature";
          input Integer region = 0 "If 0, region is unknown, otherwise known and this input";
          output Real gamma "Isentropic exponent";
        algorithm
          gamma := isentropicExponent_props_pT(p, T, waterBaseProp_pT(p, T, region));
          annotation(
            Inline = false,
            LateInline = true);
        end isentropicExponent_pT;

        function waterBaseProp_dT "Intermediate property record for water (d and T preferred states)"
          extends Modelica.Icons.Function;
          input SI.Density rho "Density";
          input SI.Temperature T "Temperature";
          input Integer phase = 0 "Phase: 2 for two-phase, 1 for one phase, 0 if unknown";
          input Integer region = 0 "If 0, do region computation, otherwise assume the region is this input";
          output Common.IF97BaseTwoPhase aux "Auxiliary record";
        protected
          SI.SpecificEnthalpy h_liq "Liquid specific enthalpy";
          SI.Density d_liq "Liquid density";
          SI.SpecificEnthalpy h_vap "Vapour specific enthalpy";
          SI.Density d_vap "Vapour density";
          Common.GibbsDerivs g "Dimensionless Gibbs function and derivatives w.r.t. pi and tau";
          Common.HelmholtzDerivs f "Dimensionless Helmholtz function and derivatives w.r.t. delta and tau";
          Modelica.Media.Common.PhaseBoundaryProperties liq "Phase boundary property record";
          Modelica.Media.Common.PhaseBoundaryProperties vap "Phase boundary property record";
          Modelica.Media.Common.GibbsDerivs gl "Dimensionless Gibbs function and derivatives w.r.t. pi and tau";
          Modelica.Media.Common.GibbsDerivs gv "Dimensionless Gibbs function and derivatives w.r.t. pi and tau";
          Modelica.Media.Common.HelmholtzDerivs fl "Dimensionless Helmholtz function and derivatives w.r.t. delta and tau";
          Modelica.Media.Common.HelmholtzDerivs fv "Dimensionless Helmholtz function and derivatives w.r.t. delta and tau";
          Integer error "Error flag for inverse iterations";
        algorithm
          aux.region := if region == 0 then if phase == 2 then 4 else BaseIF97.Regions.region_dT(d = rho, T = T, phase = phase) else region;
          aux.phase := if aux.region == 4 then 2 else 1;
          aux.R := BaseIF97.data.RH2O;
          aux.rho := rho;
          aux.T := T;
          if aux.region == 1 then
            (aux.p, error) := BaseIF97.Inverses.pofdt125(d = rho, T = T, reldd = 1.0e-8, region = 1);
            g := BaseIF97.Basic.g1(aux.p, T);
            aux.h := aux.R * aux.T * g.tau * g.gtau;
            aux.s := aux.R * (g.tau * g.gtau - g.g);
            aux.rho := aux.p / (aux.R * T * g.pi * g.gpi);
            aux.vt := aux.R / aux.p * (g.pi * g.gpi - g.tau * g.pi * g.gtaupi);
            aux.vp := aux.R * T / (aux.p * aux.p) * g.pi * g.pi * g.gpipi;
            aux.cp := -aux.R * g.tau * g.tau * g.gtautau;
            aux.cv := aux.R * ((-g.tau * g.tau * g.gtautau) + (g.gpi - g.tau * g.gtaupi) * (g.gpi - g.tau * g.gtaupi) / g.gpipi);
            aux.x := 0.0;
          elseif aux.region == 2 then
            (aux.p, error) := BaseIF97.Inverses.pofdt125(d = rho, T = T, reldd = 1.0e-8, region = 2);
            g := BaseIF97.Basic.g2(aux.p, T);
            aux.h := aux.R * aux.T * g.tau * g.gtau;
            aux.s := aux.R * (g.tau * g.gtau - g.g);
            aux.rho := aux.p / (aux.R * T * g.pi * g.gpi);
            aux.vt := aux.R / aux.p * (g.pi * g.gpi - g.tau * g.pi * g.gtaupi);
            aux.vp := aux.R * T / (aux.p * aux.p) * g.pi * g.pi * g.gpipi;
            aux.cp := -aux.R * g.tau * g.tau * g.gtautau;
            aux.cv := aux.R * ((-g.tau * g.tau * g.gtautau) + (g.gpi - g.tau * g.gtaupi) * (g.gpi - g.tau * g.gtaupi) / g.gpipi);
            aux.x := 1.0;
          elseif aux.region == 3 then
            f := BaseIF97.Basic.f3(rho, T);
            aux.p := aux.R * rho * T * f.delta * f.fdelta;
            aux.h := aux.R * T * (f.tau * f.ftau + f.delta * f.fdelta);
            aux.s := aux.R * (f.tau * f.ftau - f.f);
            aux.pd := aux.R * T * f.delta * (2.0 * f.fdelta + f.delta * f.fdeltadelta);
            aux.pt := aux.R * rho * f.delta * (f.fdelta - f.tau * f.fdeltatau);
            aux.cv := aux.R * (-f.tau * f.tau * f.ftautau);
            aux.cp := (aux.rho * aux.rho * aux.pd * aux.cv + aux.T * aux.pt * aux.pt) / (aux.rho * aux.rho * aux.pd);
            aux.x := 0.0;
          elseif aux.region == 4 then
            aux.p := BaseIF97.Basic.psat(T);
            d_liq := rhol_T(T);
            d_vap := rhov_T(T);
            h_liq := hl_p(aux.p);
            h_vap := hv_p(aux.p);
            aux.x := if d_vap <> d_liq then (1 / rho - 1 / d_liq) / (1 / d_vap - 1 / d_liq) else 1.0;
            aux.h := h_liq + aux.x * (h_vap - h_liq);
            if T < BaseIF97.data.TLIMIT1 then
              gl := BaseIF97.Basic.g1(aux.p, T);
              gv := BaseIF97.Basic.g2(aux.p, T);
              liq := Common.gibbsToBoundaryProps(gl);
              vap := Common.gibbsToBoundaryProps(gv);
            else
              fl := BaseIF97.Basic.f3(d_liq, T);
              fv := BaseIF97.Basic.f3(d_vap, T);
              liq := Common.helmholtzToBoundaryProps(fl);
              vap := Common.helmholtzToBoundaryProps(fv);
            end if;
            aux.dpT := if liq.d <> vap.d then (vap.s - liq.s) * liq.d * vap.d / (liq.d - vap.d) else BaseIF97.Basic.dptofT(aux.T);
            aux.s := liq.s + aux.x * (vap.s - liq.s);
            aux.cv := Common.cv2Phase(liq, vap, aux.x, aux.T, aux.p);
            aux.cp := liq.cp + aux.x * (vap.cp - liq.cp);
            aux.pt := liq.pt + aux.x * (vap.pt - liq.pt);
            aux.pd := liq.pd + aux.x * (vap.pd - liq.pd);
          elseif aux.region == 5 then
            (aux.p, error) := BaseIF97.Inverses.pofdt125(d = rho, T = T, reldd = 1.0e-8, region = 5);
            g := BaseIF97.Basic.g2(aux.p, T);
            aux.h := aux.R * aux.T * g.tau * g.gtau;
            aux.s := aux.R * (g.tau * g.gtau - g.g);
            aux.rho := aux.p / (aux.R * T * g.pi * g.gpi);
            aux.vt := aux.R / aux.p * (g.pi * g.gpi - g.tau * g.pi * g.gtaupi);
            aux.vp := aux.R * T / (aux.p * aux.p) * g.pi * g.pi * g.gpipi;
            aux.cp := -aux.R * g.tau * g.tau * g.gtautau;
            aux.cv := aux.R * ((-g.tau * g.tau * g.gtautau) + (g.gpi - g.tau * g.gtaupi) * (g.gpi - g.tau * g.gtaupi) / g.gpipi);
          else
            assert(false, "Error in region computation of IF97 steam tables" + "(rho = " + String(rho) + ", T = " + String(T) + ")");
          end if;
        end waterBaseProp_dT;

        function h_props_dT "Specific enthalpy as function of density and temperature"
          extends Modelica.Icons.Function;
          input SI.Density d "Density";
          input SI.Temperature T "Temperature";
          input Common.IF97BaseTwoPhase aux "Auxiliary record";
          output SI.SpecificEnthalpy h "Specific enthalpy";
        algorithm
          h := aux.h;
          annotation(
            derivative(noDerivative = aux) = h_dT_der,
            Inline = false,
            LateInline = true);
        end h_props_dT;

        function h_dT "Specific enthalpy as function of density and temperature"
          extends Modelica.Icons.Function;
          input SI.Density d "Density";
          input SI.Temperature T "Temperature";
          input Integer phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          input Integer region = 0 "If 0, region is unknown, otherwise known and this input";
          output SI.SpecificEnthalpy h "Specific enthalpy";
        algorithm
          h := h_props_dT(d, T, waterBaseProp_dT(d, T, phase, region));
          annotation(
            Inline = true);
        end h_dT;

        function h_dT_der "Derivative function of h_dT"
          extends Modelica.Icons.Function;
          input SI.Density d "Density";
          input SI.Temperature T "Temperature";
          input Common.IF97BaseTwoPhase aux "Auxiliary record";
          input Real d_der "Derivative of density";
          input Real T_der "Derivative of temperature";
          output Real h_der "Derivative of specific enthalpy";
        algorithm
          if aux.region == 3 then
            h_der := ((-d * aux.pd) + T * aux.pt) / (d * d) * d_der + (aux.cv * d + aux.pt) / d * T_der;
          elseif aux.region == 4 then
            h_der := T * aux.dpT / (d * d) * d_der + (aux.cv * d + aux.dpT) / d * T_der;
          else
            h_der := (-((-1 / d) + T * aux.vt) / (d * d * aux.vp)) * d_der + (aux.vp * aux.cp - aux.vt / d + T * aux.vt * aux.vt) / aux.vp * T_der;
          end if;
//regions 1,2 or 5
        end h_dT_der;

        function p_props_dT "Pressure as function of density and temperature"
          extends Modelica.Icons.Function;
          input SI.Density d "Density";
          input SI.Temperature T "Temperature";
          input Common.IF97BaseTwoPhase aux "Auxiliary record";
          output SI.Pressure p "Pressure";
        algorithm
          p := aux.p;
          annotation(
            derivative(noDerivative = aux) = p_dT_der,
            Inline = false,
            LateInline = true);
        end p_props_dT;

        function p_dT "Pressure as function of density and temperature"
          extends Modelica.Icons.Function;
          input SI.Density d "Density";
          input SI.Temperature T "Temperature";
          input Integer phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          input Integer region = 0 "If 0, region is unknown, otherwise known and this input";
          output SI.Pressure p "Pressure";
        algorithm
          p := p_props_dT(d, T, waterBaseProp_dT(d, T, phase, region));
          annotation(
            Inline = true);
        end p_dT;

        function p_dT_der "Derivative function of p_dT"
          extends Modelica.Icons.Function;
          input SI.Density d "Density";
          input SI.Temperature T "Temperature";
          input Common.IF97BaseTwoPhase aux "Auxiliary record";
          input Real d_der "Derivative of density";
          input Real T_der "Derivative of temperature";
          output Real p_der "Derivative of pressure";
        algorithm
          if aux.region == 3 then
            p_der := aux.pd * d_der + aux.pt * T_der;
          elseif aux.region == 4 then
            p_der := aux.dpT * T_der;
          else
            p_der := (-1 / (d * d * aux.vp)) * d_der + (-aux.vt / aux.vp) * T_der;
          end if;
/*density derivative is 0.0*/
//regions 1,2 or 5
        end p_dT_der;

        function s_props_dT "Specific entropy as function of density and temperature"
          extends Modelica.Icons.Function;
          input SI.Density d "Density";
          input SI.Temperature T "Temperature";
          input Common.IF97BaseTwoPhase aux "Auxiliary record";
          output SI.SpecificEntropy s "Specific entropy";
        algorithm
          s := aux.s;
          annotation(
            Inline = false,
            LateInline = true);
        end s_props_dT;

        function s_dT "Temperature as function of density and temperature"
          extends Modelica.Icons.Function;
          input SI.Density d "Density";
          input SI.Temperature T "Temperature";
          input Integer phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          input Integer region = 0 "If 0, region is unknown, otherwise known and this input";
          output SI.SpecificEntropy s "Specific entropy";
        algorithm
          s := s_props_dT(d, T, waterBaseProp_dT(d, T, phase, region));
          annotation(
            Inline = true);
        end s_dT;

        function cv_props_dT "Specific heat capacity at constant volume as function of density and temperature"
          extends Modelica.Icons.Function;
          input SI.Density d "Density";
          input SI.Temperature T "Temperature";
          input Common.IF97BaseTwoPhase aux "Auxiliary record";
          output SI.SpecificHeatCapacity cv "Specific heat capacity";
        algorithm
          cv := aux.cv;
          annotation(
            Inline = false,
            LateInline = true);
        end cv_props_dT;

        function cv_dT "Specific heat capacity at constant volume as function of density and temperature"
          extends Modelica.Icons.Function;
          input SI.Density d "Density";
          input SI.Temperature T "Temperature";
          input Integer phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          input Integer region = 0 "If 0, region is unknown, otherwise known and this input";
          output SI.SpecificHeatCapacity cv "Specific heat capacity";
        algorithm
          cv := cv_props_dT(d, T, waterBaseProp_dT(d, T, phase, region));
          annotation(
            Inline = true);
        end cv_dT;

        function cp_props_dT "Specific heat capacity at constant pressure as function of density and temperature"
          extends Modelica.Icons.Function;
          input SI.Density d "Density";
          input SI.Temperature T "Temperature";
          input Common.IF97BaseTwoPhase aux "Auxiliary record";
          output SI.SpecificHeatCapacity cp "Specific heat capacity";
        algorithm
          cp := aux.cp;
          annotation(
            Inline = false,
            LateInline = true);
        end cp_props_dT;

        function cp_dT "Specific heat capacity at constant pressure as function of density and temperature"
          extends Modelica.Icons.Function;
          input SI.Density d "Density";
          input SI.Temperature T "Temperature";
          input Integer phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          input Integer region = 0 "If 0, region is unknown, otherwise known and this input";
          output SI.SpecificHeatCapacity cp "Specific heat capacity";
        algorithm
          cp := cp_props_dT(d, T, waterBaseProp_dT(d, T, phase, region));
          annotation(
            Inline = true);
        end cp_dT;

        function beta_props_dT "Isobaric expansion coefficient as function of density and temperature"
          extends Modelica.Icons.Function;
          input SI.Density d "Density";
          input SI.Temperature T "Temperature";
          input Common.IF97BaseTwoPhase aux "Auxiliary record";
          output SI.RelativePressureCoefficient beta "Isobaric expansion coefficient";
        algorithm
          beta := if aux.region == 3 or aux.region == 4 then aux.pt / (aux.rho * aux.pd) else aux.vt * aux.rho;
          annotation(
            Inline = false,
            LateInline = true);
        end beta_props_dT;

        function beta_dT "Isobaric expansion coefficient as function of density and temperature"
          extends Modelica.Icons.Function;
          input SI.Density d "Density";
          input SI.Temperature T "Temperature";
          input Integer phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          input Integer region = 0 "If 0, region is unknown, otherwise known and this input";
          output SI.RelativePressureCoefficient beta "Isobaric expansion coefficient";
        algorithm
          beta := beta_props_dT(d, T, waterBaseProp_dT(d, T, phase, region));
          annotation(
            Inline = true);
        end beta_dT;

        function kappa_props_dT "Isothermal compressibility factor as function of density and temperature"
          extends Modelica.Icons.Function;
          input SI.Density d "Density";
          input SI.Temperature T "Temperature";
          input Common.IF97BaseTwoPhase aux "Auxiliary record";
          output SI.IsothermalCompressibility kappa "Isothermal compressibility factor";
        algorithm
          kappa := if aux.region == 3 or aux.region == 4 then 1 / (aux.rho * aux.pd) else -aux.vp * aux.rho;
          annotation(
            Inline = false,
            LateInline = true);
        end kappa_props_dT;

        function kappa_dT "Isothermal compressibility factor as function of density and temperature"
          extends Modelica.Icons.Function;
          input SI.Density d "Density";
          input SI.Temperature T "Temperature";
          input Integer phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          input Integer region = 0 "If 0, region is unknown, otherwise known and this input";
          output SI.IsothermalCompressibility kappa "Isothermal compressibility factor";
        algorithm
          kappa := kappa_props_dT(d, T, waterBaseProp_dT(d, T, phase, region));
          annotation(
            Inline = true);
        end kappa_dT;

        function velocityOfSound_props_dT "Speed of sound as function of density and temperature"
          extends Modelica.Icons.Function;
          input SI.Density d "Density";
          input SI.Temperature T "Temperature";
          input Common.IF97BaseTwoPhase aux "Auxiliary record";
          output SI.Velocity v_sound "Speed of sound";
        algorithm
// dp/drho at constant s
          v_sound := if aux.region == 3 then sqrt(max(0, (aux.pd * aux.rho * aux.rho * aux.cv + aux.pt * aux.pt * aux.T) / (aux.rho * aux.rho * aux.cv))) else if aux.region == 4 then sqrt(max(0, 1 / (aux.rho * (aux.rho * aux.cv / aux.dpT + 1.0) / (aux.dpT * aux.T) - 1 / aux.rho * aux.rho * aux.rho / (aux.dpT * aux.T)))) else sqrt(max(0, -aux.cp / (aux.rho * aux.rho * (aux.vp * aux.cp + aux.vt * aux.vt * aux.T))));
          annotation(
            Inline = false,
            LateInline = true);
        end velocityOfSound_props_dT;

        function velocityOfSound_dT "Speed of sound as function of density and temperature"
          extends Modelica.Icons.Function;
          input SI.Density d "Density";
          input SI.Temperature T "Temperature";
          input Integer phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          input Integer region = 0 "If 0, region is unknown, otherwise known and this input";
          output SI.Velocity v_sound "Speed of sound";
        algorithm
          v_sound := velocityOfSound_props_dT(d, T, waterBaseProp_dT(d, T, phase, region));
          annotation(
            Inline = true);
        end velocityOfSound_dT;

        function isentropicExponent_props_dT "Isentropic exponent as function of density and temperature"
          extends Modelica.Icons.Function;
          input SI.Density d "Density";
          input SI.Temperature T "Temperature";
          input Common.IF97BaseTwoPhase aux "Auxiliary record";
          output Real gamma "Isentropic exponent";
        algorithm
          gamma := if aux.region == 3 then 1 / (aux.rho * aux.p) * ((aux.pd * aux.cv * aux.rho * aux.rho + aux.pt * aux.pt * aux.T) / aux.cv) else if aux.region == 4 then 1 / (aux.rho * aux.p) * aux.dpT * aux.dpT * aux.T / aux.cv else -1 / (aux.rho * aux.p) * aux.cp / (aux.vp * aux.cp + aux.vt * aux.vt * aux.T);
          annotation(
            Inline = false,
            LateInline = true);
        end isentropicExponent_props_dT;

        function isentropicExponent_dT "Isentropic exponent as function of density and temperature"
          extends Modelica.Icons.Function;
          input SI.Density d "Density";
          input SI.Temperature T "Temperature";
          input Integer phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          input Integer region = 0 "If 0, region is unknown, otherwise known and this input";
          output Real gamma "Isentropic exponent";
        algorithm
          gamma := isentropicExponent_props_dT(d, T, waterBaseProp_dT(d, T, phase, region));
          annotation(
            Inline = false,
            LateInline = true);
        end isentropicExponent_dT;
      protected
        package ThermoFluidSpecial
          function water_ph "Calculate the property record for dynamic simulation properties using p,h as states"
            extends Modelica.Icons.Function;
            input SI.Pressure p "Pressure";
            input SI.SpecificEnthalpy h "Specific enthalpy";
            input Integer phase = 0 "Phase: 2 for two-phase, 1 for one phase, 0 if unknown";
            output Modelica.Media.Common.ThermoFluidSpecial.ThermoProperties_ph pro "Property record for dynamic simulation";
          protected
            Modelica.Media.Common.GibbsDerivs g "Dimensionless Gibbs function and derivatives w.r.t. pi and tau";
            Modelica.Media.Common.HelmholtzDerivs f "Dimensionless Helmholtz function and derivatives w.r.t. delta and tau";
            Integer region(min = 1, max = 5) "IF97 region";
            Integer error "Error flag";
            SI.Temperature T "Temperature";
            SI.Density d "Density";
          algorithm
            region := BaseIF97.Regions.region_ph(p, h, phase);
            if region == 1 then
              T := BaseIF97.Basic.tph1(p, h);
              g := BaseIF97.Basic.g1(p, T);
              pro := Modelica.Media.Common.ThermoFluidSpecial.gibbsToProps_ph(g);
            elseif region == 2 then
              T := BaseIF97.Basic.tph2(p, h);
              g := BaseIF97.Basic.g2(p, T);
              pro := Modelica.Media.Common.ThermoFluidSpecial.gibbsToProps_ph(g);
            elseif region == 3 then
              (d, T, error) := BaseIF97.Inverses.dtofph3(p = p, h = h, delp = 1.0e-7, delh = 1.0e-6);
              f := BaseIF97.Basic.f3(d, T);
              pro := Modelica.Media.Common.ThermoFluidSpecial.helmholtzToProps_ph(f);
            elseif region == 4 then
              pro := BaseIF97.TwoPhase.waterR4_ph(p = p, h = h);
            elseif region == 5 then
              (T, error) := BaseIF97.Inverses.tofph5(p = p, h = h, reldh = 1.0e-7);
              g := BaseIF97.Basic.g5(p, T);
              pro := Modelica.Media.Common.ThermoFluidSpecial.gibbsToProps_ph(g);
            end if;
          end water_ph;

          function water_dT "Calculate property record for dynamic simulation properties using d and T as dynamic states"
            extends Modelica.Icons.Function;
            input SI.Density d "Density";
            input SI.Temperature T "Temperature";
            input Integer phase = 0 "Phase: 2 for two-phase, 1 for one phase, 0 if unknown";
            output Modelica.Media.Common.ThermoFluidSpecial.ThermoProperties_dT pro "Property record for dynamic simulation";
          protected
            SI.Pressure p "Pressure";
            Integer region(min = 1, max = 5) "IF97 region";
            Modelica.Media.Common.GibbsDerivs g "Dimensionless Gibbs function and derivatives w.r.t. pi and tau";
            Modelica.Media.Common.HelmholtzDerivs f "Dimensionless Helmholtz function and derivatives w.r.t. delta and tau";
            Integer error "Error flag";
          algorithm
            region := BaseIF97.Regions.region_dT(d, T, phase);
            if region == 1 then
              (p, error) := BaseIF97.Inverses.pofdt125(d = d, T = T, reldd = iter.DELD, region = 1);
              g := BaseIF97.Basic.g1(p, T);
              pro := Modelica.Media.Common.ThermoFluidSpecial.gibbsToProps_dT(g);
            elseif region == 2 then
              (p, error) := BaseIF97.Inverses.pofdt125(d = d, T = T, reldd = iter.DELD, region = 2);
              g := BaseIF97.Basic.g2(p, T);
              pro := Modelica.Media.Common.ThermoFluidSpecial.gibbsToProps_dT(g);
            elseif region == 3 then
              f := BaseIF97.Basic.f3(d, T);
              pro := Modelica.Media.Common.ThermoFluidSpecial.helmholtzToProps_dT(f);
            elseif region == 4 then
              pro := BaseIF97.TwoPhase.waterR4_dT(d = d, T = T);
            elseif region == 5 then
              (p, error) := BaseIF97.Inverses.pofdt125(d = d, T = T, reldd = iter.DELD, region = 5);
              g := BaseIF97.Basic.g5(p, T);
              pro := Modelica.Media.Common.ThermoFluidSpecial.gibbsToProps_dT(g);
            end if;
          end water_dT;

          function water_pT "Calculate property record for dynamic simulation properties using p and T as dynamic states"
            extends Modelica.Icons.Function;
            input SI.Pressure p "Pressure";
            input SI.Temperature T "Temperature";
            output Modelica.Media.Common.ThermoFluidSpecial.ThermoProperties_pT pro "Property record for dynamic simulation";
          protected
            SI.Density d "Density";
            Integer region(min = 1, max = 5) "IF97 region";
            Modelica.Media.Common.GibbsDerivs g "Dimensionless Gibbs function and derivatives w.r.t. pi and tau";
            Modelica.Media.Common.HelmholtzDerivs f "Dimensionless Helmholtz function and derivatives w.r.t. delta and tau";
            Integer error "Error flag";
          algorithm
            region := BaseIF97.Regions.region_pT(p, T);
            if region == 1 then
              g := BaseIF97.Basic.g1(p, T);
              pro := Modelica.Media.Common.ThermoFluidSpecial.gibbsToProps_pT(g);
            elseif region == 2 then
              g := BaseIF97.Basic.g2(p, T);
              pro := Modelica.Media.Common.ThermoFluidSpecial.gibbsToProps_pT(g);
            elseif region == 3 then
              (d, error) := BaseIF97.Inverses.dofpt3(p = p, T = T, delp = iter.DELP);
              f := BaseIF97.Basic.f3(d, T);
              pro := Modelica.Media.Common.ThermoFluidSpecial.helmholtzToProps_pT(f);
            elseif region == 5 then
              g := BaseIF97.Basic.g5(p, T);
              pro := Modelica.Media.Common.ThermoFluidSpecial.gibbsToProps_pT(g);
            end if;
          end water_pT;
        end ThermoFluidSpecial;
      public
        function hl_p = BaseIF97.Regions.hl_p "Compute the saturated liquid specific h(p)";
        function hv_p = BaseIF97.Regions.hv_p "Compute the saturated vapour specific h(p)";
        function sl_p = BaseIF97.Regions.sl_p "Compute the saturated liquid specific s(p)";
        function sv_p = BaseIF97.Regions.sv_p "Compute the saturated vapour specific s(p)";
        function rhol_T = BaseIF97.Regions.rhol_T "Compute the saturated liquid d(T)";
        function rhov_T = BaseIF97.Regions.rhov_T "Compute the saturated vapour d(T)";
        function rhol_p = BaseIF97.Regions.rhol_p "Compute the saturated liquid d(p)";
        function rhov_p = BaseIF97.Regions.rhov_p "Compute the saturated vapour d(p)";
        function dynamicViscosity = BaseIF97.Transport.visc_dTp "Compute eta(d,T) in the one-phase region";
        function thermalConductivity = BaseIF97.Transport.cond_dTp "Compute lambda(d,T,p) in the one-phase region";
        function surfaceTension = BaseIF97.Transport.surfaceTension "Compute sigma(T) at saturation T";
        //   function isentropicEnthalpy
        //     "Isentropic specific enthalpy from p,s (preferably use dynamicIsentropicEnthalpy in dynamic simulation!)"
        //     extends Modelica.Icons.Function;
        //     input SI.Pressure p "Pressure";
        //     input SI.SpecificEntropy s "Specific entropy";
        //     input Integer phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
        //     output SI.SpecificEnthalpy h "Specific enthalpy";
        //   algorithm
        //    h := BaseIF97.Isentropic.water_hisentropic(p,s,phase);
        //   end isentropicEnthalpy;

        function isentropicEnthalpy "Isentropic specific enthalpy from p,s (preferably use dynamicIsentropicEnthalpy in dynamic simulation!)"
          extends Modelica.Icons.Function;
          input SI.Pressure p "Pressure";
          input SI.SpecificEntropy s "Specific entropy";
          input Integer phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          input Integer region = 0 "If 0, region is unknown, otherwise known and this input";
          output SI.SpecificEnthalpy h "Specific enthalpy";
        algorithm
          h := isentropicEnthalpy_props(p, s, waterBaseProp_ps(p, s, phase, region));
          annotation(
            Inline = true);
        end isentropicEnthalpy;

        function isentropicEnthalpy_props
          extends Modelica.Icons.Function;
          input SI.Pressure p "Pressure";
          input SI.SpecificEntropy s "Specific entropy";
          input Common.IF97BaseTwoPhase aux "Auxiliary record";
          output SI.SpecificEnthalpy h "Isentropic enthalpy";
        algorithm
          h := aux.h;
          annotation(
            derivative(noDerivative = aux) = isentropicEnthalpy_der,
            Inline = false,
            LateInline = true);
        end isentropicEnthalpy_props;

        function isentropicEnthalpy_der "Derivative of isentropic specific enthalpy from p,s"
          extends Modelica.Icons.Function;
          input SI.Pressure p "Pressure";
          input SI.SpecificEntropy s "Specific entropy";
          input Common.IF97BaseTwoPhase aux "Auxiliary record";
          input Real p_der "Pressure derivative";
          input Real s_der "Entropy derivative";
          output Real h_der "Specific enthalpy derivative";
        algorithm
          h_der := 1 / aux.rho * p_der + aux.T * s_der;
          annotation(
            Inline = true);
        end isentropicEnthalpy_der;

        function dynamicIsentropicEnthalpy "Isentropic specific enthalpy from p,s and good guesses of d and T"
          extends Modelica.Icons.Function;
          input SI.Pressure p "Pressure";
          input SI.SpecificEntropy s "Specific entropy";
          input SI.Density dguess "Good guess density, e.g., from adjacent volume";
          input SI.Temperature Tguess "Good guess temperature, e.g., from adjacent volume";
          input Integer phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          output SI.SpecificEnthalpy h "Specific enthalpy";
        algorithm
          h := BaseIF97.Isentropic.water_hisentropic_dyn(p, s, dguess, Tguess, 0);
          annotation(
            Inline = true);
        end dynamicIsentropicEnthalpy;
        annotation(
          Documentation(info = "<HTML>
      <h4>Package description:</h4>
      <p>This package provides high accuracy physical properties for water according
      to the IAPWS/IF97 standard. It has been part of the ThermoFluid Modelica library and been extended,
      reorganized and documented to become part of the Modelica Standard library.</p>
      <p>An important feature that distinguishes this implementation of the IF97 steam property standard
      is that this implementation has been explicitly designed to work well in dynamic simulations. Computational
      performance has been of high importance. This means that there often exist several ways to get the same result
      from different functions if one of the functions is called often but can be optimized for that purpose.
      </p>
      <p>
      The original documentation of the IAPWS/IF97 steam properties can freely be distributed with computer
      implementations, so for curious minds the complete standard documentation is provided with the Modelica
      properties library. The following documents are included
      (in directory Modelica/Resources/Documentation/Media/Water/IF97documentation):
      </p>
      <ul>
      <li><a href=\"modelica://Modelica/Resources/Documentation/Media/Water/IF97documentation/IF97.pdf\">IF97.pdf</a> The standards document for the main part of the IF97.</li>
      <li><a href=\"modelica://Modelica/Resources/Documentation/Media/Water/IF97documentation/Back3.pdf\">Back3.pdf</a> The backwards equations for region 3.</li>
      <li><a href=\"modelica://Modelica/Resources/Documentation/Media/Water/IF97documentation/crits.pdf\">crits.pdf</a> The critical point data.</li>
      <li><a href=\"modelica://Modelica/Resources/Documentation/Media/Water/IF97documentation/meltsub.pdf\">meltsub.pdf</a> The melting- and sublimation line formulation (in IF97_Utilities.BaseIF97.IceBoundaries)</li>
      <li><a href=\"modelica://Modelica/Resources/Documentation/Media/Water/IF97documentation/surf.pdf\">surf.pdf</a> The surface tension standard definition</li>
      <li><a href=\"modelica://Modelica/Resources/Documentation/Media/Water/IF97documentation/thcond.pdf\">thcond.pdf</a> The thermal conductivity standard definition</li>
      <li><a href=\"modelica://Modelica/Resources/Documentation/Media/Water/IF97documentation/visc.pdf\">visc.pdf</a> The viscosity standard definition</li>
      </ul>
      <h4>Package contents
      </h4>
      <ul>
      <li>Package <b>BaseIF97</b> contains the implementation of the IAPWS-IF97 as described in
      <a href=\"modelica://Modelica/Resources/Documentation/Media/Water/IF97documentation/IF97.pdf\">IF97.pdf</a>. The explicit backwards equations for region 3 from
      <a href=\"modelica://Modelica/Resources/Documentation/Media/Water/IF97documentation/Back3.pdf\">Back3.pdf</a> are implemented as initial values for an inverse iteration of the exact
      function in IF97 for the input pairs (p,h) and (p,s).
      The low-level functions in BaseIF97 are not needed for standard simulation usage,
      but can be useful for experts and some special purposes.</li>
      <li>Function <b>water_ph</b> returns all properties needed for a dynamic control volume model and properties of general
      interest using pressure p and specific entropy enthalpy h as dynamic states in the record ThermoProperties_ph. </li>
      <li>Function <b>water_ps</b> returns all properties needed for a dynamic control volume model and properties of general
      interest using pressure p and specific entropy s as dynamic states in the record ThermoProperties_ps. </li>
      <li>Function <b>water_dT</b> returns all properties needed for a dynamic control volume model and properties of general
      interest using density d and temperature T as dynamic states in the record ThermoProperties_dT. </li>
      <li>Function <b>water_pT</b> returns all properties needed for a dynamic control volume model and properties of general
      interest using pressure p and temperature T as dynamic states in the record ThermoProperties_pT. Due to the coupling of
      pressure and temperature in the two-phase region, this model can obviously
      only be used for one-phase models or models treating both phases independently.</li>
      <li>Function <b>hl_p</b> computes the liquid specific enthalpy as a function of pressure. For overcritical pressures,
      the critical specific enthalpy is returned</li>
      <li>Function <b>hv_p</b> computes the vapour specific enthalpy as a function of pressure. For overcritical pressures,
      the critical specific enthalpy is returned</li>
      <li>Function <b>sl_p</b> computes the liquid specific entropy as a function of pressure. For overcritical pressures,
      the critical  specific entropy is returned</li>
      <li>Function <b>sv_p</b> computes the vapour  specific entropy as a function of pressure. For overcritical pressures,
      the critical  specific entropy is returned</li>
      <li>Function <b>rhol_T</b> computes the liquid density as a function of temperature. For overcritical temperatures,
      the critical density is returned</li>
      <li>Function <b>rhol_T</b> computes the vapour density as a function of temperature. For overcritical temperatures,
      the critical density is returned</li>
      <li>Function <b>dynamicViscosity</b> computes the dynamic viscosity as a function of density and temperature.</li>
      <li>Function <b>thermalConductivity</b> computes the thermal conductivity as a function of density, temperature and pressure.
      <b>Important note</b>: Obviously only two of the three
      inputs are really needed, but using three inputs speeds up the computation and the three variables
      are known in most models anyways. The inputs d,T and p have to be consistent.</li>
      <li>Function <b>surfaceTension</b> computes the surface tension between vapour
          and liquid water as a function of temperature.</li>
      <li>Function <b>isentropicEnthalpy</b> computes the specific enthalpy h(p,s,phase) in all regions.
          The phase input is needed due to discontinuous derivatives at the phase boundary.</li>
      <li>Function <b>dynamicIsentropicEnthalpy</b> computes the specific enthalpy h(p,s,,dguess,Tguess,phase) in all regions.
          The phase input is needed due to discontinuous derivatives at the phase boundary. Tguess and dguess are initial guess
          values for the density and temperature consistent with p and s. This function should be preferred in
          dynamic simulations where good guesses are often available.</li>
      </ul>
      <h4>Version Info and Revision history
      </h4>
      <ul>
      <li>First implemented: <i>July, 2000</i>
      by Hubertus Tummescheit for the ThermoFluid Library with help from Jonas Eborn and Falko Jens Wagner
      </li>
      <li>Code reorganization, enhanced documentation, additional functions:   <i>December, 2002</i>
      by <a href=\"mailto:Hubertus.Tummescheit@modelon.se\">Hubertus Tummescheit</a> and moved to Modelica
      properties library.</li>
      </ul>
      <address>Author: Hubertus Tummescheit, <br>
      Modelon AB<br>
      Ideon Science Park<br>
      SE-22370 Lund, Sweden<br>
      email: hubertus@modelon.se
      </address>
      </HTML>", revisions = "<h4>Intermediate release notes during development</h4>
<p>Currently the Events/noEvents switch is only implemented for p-h states. Only after testing that implementation, it will be extended to dT.</p>"));
      end IF97_Utilities;
      annotation(
        Documentation(info = "<html>
<p>This package contains different medium models for water:</p>
<ul>
<li><b>ConstantPropertyLiquidWater</b><br>
    Simple liquid water medium (incompressible, constant data).</li>
<li><b>IdealSteam</b><br>
    Steam water medium as ideal gas from Media.IdealGases.SingleGases.H2O</li>
<li><b>WaterIF97 derived models</b><br>
    High precision water model according to the IAPWS/IF97 standard
    (liquid, steam, two phase region). Models with different independent
    variables are provided as well as models valid only
    for particular regions. The <b>WaterIF97_ph</b> model is valid
    in all regions and is the recommended one to use.</li>
</ul>
<h4>Overview of WaterIF97 derived water models</h4>
<p>
The WaterIF97 models calculate medium properties
for water in the <b>liquid</b>, <b>gas</b> and <b>two phase</b> regions
according to the IAPWS/IF97 standard, i.e., the accepted industrial standard
and best compromise between accuracy and computation time.
It has been part of the ThermoFluid Modelica library and been extended,
reorganized and documented to become part of the Modelica Standard library.</p>
<p>An important feature that distinguishes this implementation of the IF97 steam property standard
is that this implementation has been explicitly designed to work well in dynamic simulations. Computational
performance has been of high importance. This means that there often exist several ways to get the same result
from different functions if one of the functions is called often but can be optimized for that purpose.
</p>
<p>Three variable pairs can be the independent variables of the model:
</p>
<ol>
<li>Pressure <b>p</b> and specific enthalpy <b>h</b> are
    the most natural choice for general applications.
    This is the recommended choice for most general purpose
    applications, in particular for power plants.</li>
<li>Pressure <b>p</b> and temperature <b>T</b> are the most natural
    choice for applications where water is always in the same phase,
    both for liquid water and steam.</li>
<li>Density <b>d</b> and temperature <b>T</b> are explicit
    variables of the Helmholtz function in the near-critical
    region and can be the best choice for applications with
    super-critical or near-critical states.</li>
</ol>
<p>
The following quantities are always computed in Medium.BaseProperties:
</p>
<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"><b>Variable</b></td>
      <td valign=\"top\"><b>Unit</b></td>
      <td valign=\"top\"><b>Description</b></td></tr>
  <tr><td valign=\"top\">T</td>
      <td valign=\"top\">K</td>
      <td valign=\"top\">temperature</td></tr>
  <tr><td valign=\"top\">u</td>
      <td valign=\"top\">J/kg</td>
      <td valign=\"top\">specific internal energy</td></tr>
  <tr><td valign=\"top\">d</td>
      <td valign=\"top\">kg/m^3</td>
      <td valign=\"top\">density</td></tr>
  <tr><td valign=\"top\">p</td>
      <td valign=\"top\">Pa</td>
      <td valign=\"top\">pressure</td></tr>
  <tr><td valign=\"top\">h</td>
      <td valign=\"top\">J/kg</td>
      <td valign=\"top\">specific enthalpy</td></tr>
</table>
<p>
In some cases additional medium properties are needed.
A component that needs these optional properties has to call
one of the following functions:
</p>
<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"><b>Function call</b></td>
      <td valign=\"top\"><b>Unit</b></td>
      <td valign=\"top\"><b>Description</b></td></tr>
  <tr><td valign=\"top\">Medium.dynamicViscosity(medium.state)</td>
      <td valign=\"top\">Pa.s</td>
      <td valign=\"top\">dynamic viscosity</td></tr>
  <tr><td valign=\"top\">Medium.thermalConductivity(medium.state)</td>
      <td valign=\"top\">W/(m.K)</td>
      <td valign=\"top\">thermal conductivity</td></tr>
  <tr><td valign=\"top\">Medium.prandtlNumber(medium.state)</td>
      <td valign=\"top\">1</td>
      <td valign=\"top\">Prandtl number</td></tr>
  <tr><td valign=\"top\">Medium.specificEntropy(medium.state)</td>
      <td valign=\"top\">J/(kg.K)</td>
      <td valign=\"top\">specific entropy</td></tr>
  <tr><td valign=\"top\">Medium.heatCapacity_cp(medium.state)</td>
      <td valign=\"top\">J/(kg.K)</td>
      <td valign=\"top\">specific heat capacity at constant pressure</td></tr>
  <tr><td valign=\"top\">Medium.heatCapacity_cv(medium.state)</td>
      <td valign=\"top\">J/(kg.K)</td>
      <td valign=\"top\">specific heat capacity at constant density</td></tr>
  <tr><td valign=\"top\">Medium.isentropicExponent(medium.state)</td>
      <td valign=\"top\">1</td>
      <td valign=\"top\">isentropic exponent</td></tr>
  <tr><td valign=\"top\">Medium.isentropicEnthalpy(pressure, medium.state)</td>
      <td valign=\"top\">J/kg</td>
      <td valign=\"top\">isentropic enthalpy</td></tr>
  <tr><td valign=\"top\">Medium.velocityOfSound(medium.state)</td>
      <td valign=\"top\">m/s</td>
      <td valign=\"top\">velocity of sound</td></tr>
  <tr><td valign=\"top\">Medium.isobaricExpansionCoefficient(medium.state)</td>
      <td valign=\"top\">1/K</td>
      <td valign=\"top\">isobaric expansion coefficient</td></tr>
  <tr><td valign=\"top\">Medium.isothermalCompressibility(medium.state)</td>
      <td valign=\"top\">1/Pa</td>
      <td valign=\"top\">isothermal compressibility</td></tr>
  <tr><td valign=\"top\">Medium.density_derp_h(medium.state)</td>
      <td valign=\"top\">kg/(m3.Pa)</td>
      <td valign=\"top\">derivative of density by pressure at constant enthalpy</td></tr>
  <tr><td valign=\"top\">Medium.density_derh_p(medium.state)</td>
      <td valign=\"top\">kg2/(m3.J)</td>
      <td valign=\"top\">derivative of density by enthalpy at constant pressure</td></tr>
  <tr><td valign=\"top\">Medium.density_derp_T(medium.state)</td>
      <td valign=\"top\">kg/(m3.Pa)</td>
      <td valign=\"top\">derivative of density by pressure at constant temperature</td></tr>
  <tr><td valign=\"top\">Medium.density_derT_p(medium.state)</td>
      <td valign=\"top\">kg/(m3.K)</td>
      <td valign=\"top\">derivative of density by temperature at constant pressure</td></tr>
  <tr><td valign=\"top\">Medium.density_derX(medium.state)</td>
      <td valign=\"top\">kg/m3</td>
      <td valign=\"top\">derivative of density by mass fraction</td></tr>
  <tr><td valign=\"top\">Medium.molarMass(medium.state)</td>
      <td valign=\"top\">kg/mol</td>
      <td valign=\"top\">molar mass</td></tr>
</table>
<p>More details are given in
<a href=\"modelica://Modelica.Media.UsersGuide.MediumUsage.OptionalProperties\">
Modelica.Media.UsersGuide.MediumUsage.OptionalProperties</a>.

Many additional optional functions are defined to compute properties of
saturated media, either liquid (bubble point) or vapour (dew point).
The argument to such functions is a SaturationProperties record, which can be
set starting from either the saturation pressure or the saturation temperature.
With reference to a model defining a pressure p, a temperature T, and a
SaturationProperties record sat, the following functions are provided:
</p>
<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"><b>Function call</b></td>
      <td valign=\"top\"><b>Unit</b></td>
      <td valign=\"top\"><b>Description</b></td></tr>
  <tr><td valign=\"top\">Medium.saturationPressure(T)</td>
      <td valign=\"top\">Pa</td>
      <td valign=\"top\">Saturation pressure at temperature T</td></tr>
  <tr><td valign=\"top\">Medium.saturationTemperature(p)</td>
      <td valign=\"top\">K</td>
      <td valign=\"top\">Saturation temperature at pressure p</td></tr>
  <tr><td valign=\"top\">Medium.saturationTemperature_derp(p)</td>
      <td valign=\"top\">K/Pa</td>
      <td valign=\"top\">Derivative of saturation temperature with respect to pressure</td></tr>
  <tr><td valign=\"top\">Medium.bubbleEnthalpy(sat)</td>
      <td valign=\"top\">J/kg</td>
      <td valign=\"top\">Specific enthalpy at bubble point</td></tr>
  <tr><td valign=\"top\">Medium.dewEnthalpy(sat)</td>
      <td valign=\"top\">J/kg</td>
      <td valign=\"top\">Specific enthalpy at dew point</td></tr>
  <tr><td valign=\"top\">Medium.bubbleEntropy(sat)</td>
      <td valign=\"top\">J/(kg.K)</td>
      <td valign=\"top\">Specific entropy at bubble point</td></tr>
  <tr><td valign=\"top\">Medium.dewEntropy(sat)</td>
      <td valign=\"top\">J/(kg.K)</td>
      <td valign=\"top\">Specific entropy at dew point</td></tr>
  <tr><td valign=\"top\">Medium.bubbleDensity(sat)</td>
      <td valign=\"top\">kg/m3</td>
      <td valign=\"top\">Density at bubble point</td></tr>
  <tr><td valign=\"top\">Medium.dewDensity(sat)</td>
      <td valign=\"top\">kg/m3</td>
      <td valign=\"top\">Density at dew point</td></tr>
  <tr><td valign=\"top\">Medium.dBubbleDensity_dPressure(sat)</td>
      <td valign=\"top\">kg/(m3.Pa)</td>
      <td valign=\"top\">Derivative of density at bubble point with respect to pressure</td></tr>
  <tr><td valign=\"top\">Medium.dDewDensity_dPressure(sat)</td>
      <td valign=\"top\">kg/(m3.Pa)</td>
      <td valign=\"top\">Derivative of density at dew point with respect to pressure</td></tr>
  <tr><td valign=\"top\">Medium.dBubbleEnthalpy_dPressure(sat)</td>
      <td valign=\"top\">J/(kg.Pa)</td>
      <td valign=\"top\">Derivative of specific enthalpy at bubble point with respect to pressure</td></tr>
  <tr><td valign=\"top\">Medium.dDewEnthalpy_dPressure(sat)</td>
      <td valign=\"top\">J/(kg.Pa)</td>
      <td valign=\"top\">Derivative of specific enthalpy at dew point with respect to pressure</td></tr>
  <tr><td valign=\"top\">Medium.surfaceTension(sat)</td>
      <td valign=\"top\">N/m</td>
      <td valign=\"top\">Surface tension between liquid and vapour phase</td></tr>
</table>
<p>Details on usage and some examples are given in:
<a href=\"modelica://Modelica.Media.UsersGuide.MediumUsage.TwoPhase\">
Modelica.Media.UsersGuide.MediumUsage.TwoPhase</a>.
</p>
<p>Many further properties can be computed. Using the well-known Bridgman's Tables,
all first partial derivatives of the standard thermodynamic variables can be computed easily.
</p>
<p>
The documentation of the IAPWS/IF97 steam properties can be freely
distributed with computer implementations and are included here
(in directory Modelica/Resources/Documentation/Media/Water/IF97documentation):
</p>
<ul>
<li><a href=\"modelica://Modelica/Resources/Documentation/Media/Water/IF97documentation/IF97.pdf\">IF97.pdf</a> The standards document for the main part of the IF97.</li>
<li><a href=\"modelica://Modelica/Resources/Documentation/Media/Water/IF97documentation/Back3.pdf\">Back3.pdf</a> The backwards equations for region 3.</li>
<li><a href=\"modelica://Modelica/Resources/Documentation/Media/Water/IF97documentation/crits.pdf\">crits.pdf</a> The critical point data.</li>
<li><a href=\"modelica://Modelica/Resources/Documentation/Media/Water/IF97documentation/meltsub.pdf\">meltsub.pdf</a> The melting- and sublimation line formulation (not implemented)</li>
<li><a href=\"modelica://Modelica/Resources/Documentation/Media/Water/IF97documentation/surf.pdf\">surf.pdf</a> The surface tension standard definition</li>
<li><a href=\"modelica://Modelica/Resources/Documentation/Media/Water/IF97documentation/thcond.pdf\">thcond.pdf</a> The thermal conductivity standard definition</li>
<li><a href=\"modelica://Modelica/Resources/Documentation/Media/Water/IF97documentation/visc.pdf\">visc.pdf</a> The viscosity standard definition</li>
</ul>
</html>"));
    end Water;
    annotation(
      preferredView = "info",
      Documentation(info = "<HTML>
<p>
This library contains <a href=\"modelica://Modelica.Media.Interfaces\">interface</a>
definitions for media and the following <b>property</b> models for
single and multiple substance fluids with one and multiple phases:
</p>
<ul>
<li> <a href=\"modelica://Modelica.Media.IdealGases\">Ideal gases:</a><br>
     1241 high precision gas models based on the
     NASA Glenn coefficients, plus ideal gas mixture models based
     on the same data.</li>
<li> <a href=\"modelica://Modelica.Media.Water\">Water models:</a><br>
     ConstantPropertyLiquidWater, WaterIF97 (high precision
     water model according to the IAPWS/IF97 standard)</li>
<li> <a href=\"modelica://Modelica.Media.Air\">Air models:</a><br>
     SimpleAir, DryAirNasa, ReferenceAir, MoistAir, ReferenceMoistAir.</li>
<li> <a href=\"modelica://Modelica.Media.Incompressible\">
     Incompressible media:</a><br>
     TableBased incompressible fluid models (properties are defined by tables rho(T),
     HeatCapacity_cp(T), etc.)</li>
<li> <a href=\"modelica://Modelica.Media.CompressibleLiquids\">
     Compressible liquids:</a><br>
     Simple liquid models with linear compressibility</li>
<li> <a href=\"modelica://Modelica.Media.R134a\">Refrigerant Tetrafluoroethane (R134a)</a>.</li>
</ul>
<p>
The following parts are useful, when newly starting with this library:
<ul>
<li> <a href=\"modelica://Modelica.Media.UsersGuide\">Modelica.Media.UsersGuide</a>.</li>
<li> <a href=\"modelica://Modelica.Media.UsersGuide.MediumUsage\">Modelica.Media.UsersGuide.MediumUsage</a>
     describes how to use a medium model in a component model.</li>
<li> <a href=\"modelica://Modelica.Media.UsersGuide.MediumDefinition\">
     Modelica.Media.UsersGuide.MediumDefinition</a>
     describes how a new fluid medium model has to be implemented.</li>
<li> <a href=\"modelica://Modelica.Media.UsersGuide.ReleaseNotes\">Modelica.Media.UsersGuide.ReleaseNotes</a>
     summarizes the changes of the library releases.</li>
<li> <a href=\"modelica://Modelica.Media.Examples\">Modelica.Media.Examples</a>
     contains examples that demonstrate the usage of this library.</li>
</ul>
<p>
Copyright &copy; 1998-2015, Modelica Association.
</p>
<p>
<i>This Modelica package is <u>free</u> software and the use is completely at <u>your own risk</u>; it can be redistributed and/or modified under the terms of the Modelica License 2. For license conditions (including the disclaimer of warranty) see <a href=\"modelica://Modelica.UsersGuide.ModelicaLicense2\">Modelica.UsersGuide.ModelicaLicense2</a> or visit <a href=\"https://www.modelica.org/licenses/ModelicaLicense2\"> https://www.modelica.org/licenses/ModelicaLicense2</a>.</i>
</p>
</HTML>", revisions = "<html>
<ul>
<li><i>May 16, 2013</i> by Stefan Wischhusen (XRG Simulation):<br/>
    Added new media models Air.ReferenceMoistAir, Air.ReferenceAir, R134a.</li>
<li><i>May 25, 2011</i> by Francesco Casella:<br/>Added min/max attributes to Water, TableBased, MixtureGasNasa, SimpleAir and MoistAir local types.</li>
<li><i>May 25, 2011</i> by Stefan Wischhusen:<br/>Added individual settings for polynomial fittings of properties.</li>
</ul>
</html>"),
      Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-76, -80}, {-62, -30}, {-32, 40}, {4, 66}, {48, 66}, {73, 45}, {62, -8}, {48, -50}, {38, -80}}, color = {64, 64, 64}, smooth = Smooth.Bezier), Line(points = {{-40, 20}, {68, 20}}, color = {175, 175, 175}, smooth = Smooth.None), Line(points = {{-40, 20}, {-44, 88}, {-44, 88}}, color = {175, 175, 175}, smooth = Smooth.None), Line(points = {{68, 20}, {86, -58}}, color = {175, 175, 175}, smooth = Smooth.None), Line(points = {{-60, -28}, {56, -28}}, color = {175, 175, 175}, smooth = Smooth.None), Line(points = {{-60, -28}, {-74, 84}, {-74, 84}}, color = {175, 175, 175}, smooth = Smooth.None), Line(points = {{56, -28}, {70, -80}}, color = {175, 175, 175}, smooth = Smooth.None), Line(points = {{-76, -80}, {38, -80}}, color = {175, 175, 175}, smooth = Smooth.None), Line(points = {{-76, -80}, {-94, -16}, {-94, -16}}, color = {175, 175, 175}, smooth = Smooth.None)}));
  end Media;

  package Thermal "Library of thermal system components to model heat transfer and simple thermo-fluid pipe flow"
    extends Modelica.Icons.Package;

    package FluidHeatFlow "Simple components for 1-dimensional incompressible thermo-fluid flow models"
      extends Modelica.Icons.Package;

      package Examples "Examples that demonstrate the usage of the FluidHeatFlow components"
        extends Modelica.Icons.ExamplesPackage;

        model SimpleCooling "Example: simple cooling circuit"
          extends Modelica.Icons.Example;
          parameter FluidHeatFlow.Media.Medium medium = FluidHeatFlow.Media.Medium() "Cooling medium" annotation(
            choicesAllMatching = true);
          parameter Modelica.SIunits.Temperature TAmb(displayUnit = "degC") = 293.15 "Ambient temperature";
          output Modelica.SIunits.TemperatureDifference dTSource = prescribedHeatFlow.port.T - TAmb "Source over Ambient";
          output Modelica.SIunits.TemperatureDifference dTtoPipe = prescribedHeatFlow.port.T - pipe.heatPort.T "Source over Coolant";
          output Modelica.SIunits.TemperatureDifference dTCoolant = pipe.dT "Coolant's temperature increase";
          FluidHeatFlow.Sources.Ambient ambient1(constantAmbientTemperature = TAmb, medium = medium, constantAmbientPressure = 0) annotation(
            Placement(transformation(extent = {{-60, -10}, {-80, 10}}, rotation = 0)));
          Sources.VolumeFlow pump(medium = medium, m = 0, T0 = TAmb, useVolumeFlowInput = true, constantVolumeFlow = 1) annotation(
            Placement(transformation(extent = {{-40, -10}, {-20, 10}}, rotation = 0)));
          FluidHeatFlow.Components.HeatedPipe pipe(medium = medium, m = 0.1, T0 = TAmb, V_flowLaminar = 0.1, dpLaminar(displayUnit = "Pa") = 0.1, V_flowNominal = 1, dpNominal(displayUnit = "Pa") = 1, h_g = 0, T0fixed = true) annotation(
            Placement(transformation(extent = {{0, -10}, {20, 10}}, rotation = 0)));
          FluidHeatFlow.Sources.Ambient ambient2(constantAmbientTemperature = TAmb, medium = medium, constantAmbientPressure = 0) annotation(
            Placement(transformation(extent = {{40, -10}, {60, 10}}, rotation = 0)));
          Modelica.Thermal.HeatTransfer.Components.HeatCapacitor heatCapacitor(C = 0.1, T(start = TAmb, fixed = true)) annotation(
            Placement(transformation(origin = {40, -50}, extent = {{-10, 10}, {10, -10}}, rotation = 90)));
          Modelica.Thermal.HeatTransfer.Sources.PrescribedHeatFlow prescribedHeatFlow annotation(
            Placement(transformation(extent = {{-30, -40}, {-10, -60}}, rotation = 0)));
          Modelica.Blocks.Sources.Constant volumeFlow(k = 1) annotation(
            Placement(transformation(extent = {{-60, 10}, {-40, 30}}, rotation = 0)));
          Modelica.Blocks.Sources.Constant heatFlow(k = 10) annotation(
            Placement(transformation(extent = {{-60, -60}, {-40, -40}}, rotation = 0)));
          Modelica.Thermal.HeatTransfer.Components.Convection convection annotation(
            Placement(transformation(origin = {10, -30}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
          Modelica.Blocks.Sources.Constant thermalConductance(k = 1) annotation(
            Placement(transformation(extent = {{-30, -40}, {-10, -20}}, rotation = 0)));
        equation
          connect(ambient1.flowPort, pump.flowPort_a) annotation(
            Line(points = {{-60, 0}, {-40, 0}}, color = {255, 0, 0}));
          connect(pump.flowPort_b, pipe.flowPort_a) annotation(
            Line(points = {{-20, 0}, {0, 0}}, color = {255, 0, 0}));
          connect(pipe.flowPort_b, ambient2.flowPort) annotation(
            Line(points = {{20, 0}, {40, 0}}, color = {255, 0, 0}));
          connect(heatFlow.y, prescribedHeatFlow.Q_flow) annotation(
            Line(points = {{-39, -50}, {-30, -50}}, color = {0, 0, 255}));
          connect(convection.solid, prescribedHeatFlow.port) annotation(
            Line(points = {{10, -40}, {10, -50}, {-10, -50}}, color = {191, 0, 0}));
          connect(convection.solid, heatCapacitor.port) annotation(
            Line(points = {{10, -40}, {10, -50}, {30, -50}}, color = {191, 0, 0}));
          connect(pipe.heatPort, convection.fluid) annotation(
            Line(points = {{10, -10}, {10, -20}}, color = {191, 0, 0}));
          connect(thermalConductance.y, convection.Gc) annotation(
            Line(points = {{-9, -30}, {0, -30}}, color = {0, 0, 127}));
          connect(volumeFlow.y, pump.volumeFlow) annotation(
            Line(points = {{-39, 20}, {-30, 20}, {-30, 10}}, color = {0, 0, 127}, smooth = Smooth.None));
          annotation(
            Documentation(info = "<HTML>
<p>
1st test example: SimpleCooling
</p>
A prescribed heat source dissipates its heat through a thermal conductor to a coolant flow. The coolant flow is taken from an ambient and driven by a pump with prescribed mass flow.<br>
<b>Results</b>:<br>
<table>
<tr>
<td valign=\"top\"><b>output</b></td>
<td valign=\"top\"><b>explanation</b></td>
<td valign=\"top\"><b>formula</b></td>
<td valign=\"top\"><b>actual steady-state value</b></td>
</tr>
<tr>
<td valign=\"top\">dTSource</td>
<td valign=\"top\">Source over Ambient</td>
<td valign=\"top\">dtCoolant + dtToPipe</td>
<td valign=\"top\">20 K</td>
</tr>
<tr>
<td valign=\"top\">dTtoPipe</td>
<td valign=\"top\">Source over Coolant</td>
<td valign=\"top\">Losses / ThermalConductor.G</td>
<td valign=\"top\">10 K</td>
</tr>
<tr>
<td valign=\"top\">dTCoolant</td>
<td valign=\"top\">Coolant's temperature increase</td>
<td valign=\"top\">Losses * cp * massFlow</td>
<td valign=\"top\">10 K</td>
</tr>
</table>
</HTML>"),
            experiment(StopTime = 1.0, Interval = 0.001));
        end SimpleCooling;

        model ParallelCooling "Example: cooling circuit with parallel branches"
          extends Modelica.Icons.Example;
          parameter FluidHeatFlow.Media.Medium medium = FluidHeatFlow.Media.Medium() "Cooling medium" annotation(
            choicesAllMatching = true);
          parameter Modelica.SIunits.Temperature TAmb(displayUnit = "degC") = 293.15 "Ambient temperature";
          output Modelica.SIunits.TemperatureDifference dTSource1 = prescribedHeatFlow1.port.T - TAmb "Source1 over Ambient";
          output Modelica.SIunits.TemperatureDifference dTtoPipe1 = prescribedHeatFlow1.port.T - pipe1.heatPort.T "Source1 over Coolant1";
          output Modelica.SIunits.TemperatureDifference dTCoolant1 = pipe1.dT "Coolant1's temperature increase";
          output Modelica.SIunits.TemperatureDifference dTSource2 = prescribedHeatFlow2.port.T - TAmb "Source2 over Ambient";
          output Modelica.SIunits.TemperatureDifference dTtoPipe2 = prescribedHeatFlow2.port.T - pipe2.heatPort.T "Source2 over Coolant2";
          output Modelica.SIunits.TemperatureDifference dTCoolant2 = pipe2.dT "Coolant2's temperature increase";
          output Modelica.SIunits.TemperatureDifference dTmixedCoolant = ambient2.T_port - ambient1.T_port "mixed Coolant's temperature increase";
          FluidHeatFlow.Sources.Ambient ambient1(constantAmbientTemperature = TAmb, medium = medium, constantAmbientPressure = 0) annotation(
            Placement(transformation(extent = {{-60, -10}, {-80, 10}}, rotation = 0)));
          Sources.VolumeFlow pump(medium = medium, m = 0, T0 = TAmb, useVolumeFlowInput = true, constantVolumeFlow = 1) annotation(
            Placement(transformation(extent = {{-40, -10}, {-20, 10}}, rotation = 0)));
          FluidHeatFlow.Components.HeatedPipe pipe1(medium = medium, m = 0.1, T0 = TAmb, V_flowLaminar = 0.1, dpLaminar(displayUnit = "Pa") = 0.1, V_flowNominal = 1, dpNominal(displayUnit = "Pa") = 1, h_g = 0, V_flow(start = 0), T0fixed = true) annotation(
            Placement(transformation(extent = {{0, -20}, {20, 0}}, rotation = 0)));
          FluidHeatFlow.Components.HeatedPipe pipe2(medium = medium, m = 0.1, T0 = TAmb, V_flowLaminar = 0.1, dpLaminar(displayUnit = "Pa") = 0.1, V_flowNominal = 1, dpNominal(displayUnit = "Pa") = 1, h_g = 0, V_flow(start = 0), T0fixed = true) annotation(
            Placement(transformation(extent = {{0, 20}, {20, 0}}, rotation = 0)));
          FluidHeatFlow.Components.IsolatedPipe pipe3(medium = medium, m = 0.1, T0 = TAmb, V_flowLaminar = 0.1, dpLaminar(displayUnit = "Pa") = 0.1, V_flowNominal = 1, dpNominal(displayUnit = "Pa") = 1, h_g = 0, T0fixed = true) annotation(
            Placement(transformation(extent = {{40, -10}, {60, 10}}, rotation = 0)));
          FluidHeatFlow.Sources.Ambient ambient2(constantAmbientTemperature = TAmb, medium = medium, constantAmbientPressure = 0) annotation(
            Placement(transformation(extent = {{80, -10}, {100, 10}}, rotation = 0)));
          Modelica.Thermal.HeatTransfer.Components.HeatCapacitor heatCapacitor1(C = 0.1, T(start = TAmb, fixed = true)) annotation(
            Placement(transformation(origin = {40, -60}, extent = {{-10, 10}, {10, -10}}, rotation = 90)));
          Modelica.Thermal.HeatTransfer.Sources.PrescribedHeatFlow prescribedHeatFlow1 annotation(
            Placement(transformation(origin = {-20, -60}, extent = {{10, -10}, {-10, 10}}, rotation = 180)));
          Modelica.Thermal.HeatTransfer.Components.Convection convection1 annotation(
            Placement(transformation(origin = {10, -40}, extent = {{10, 10}, {-10, -10}}, rotation = 270)));
          Modelica.Thermal.HeatTransfer.Components.HeatCapacitor heatCapacitor2(C = 0.1, T(start = TAmb, fixed = true)) annotation(
            Placement(transformation(origin = {38, 60}, extent = {{10, -10}, {-10, 10}}, rotation = 270)));
          Modelica.Thermal.HeatTransfer.Sources.PrescribedHeatFlow prescribedHeatFlow2 annotation(
            Placement(transformation(origin = {-20, 60}, extent = {{10, 10}, {-10, -10}}, rotation = 180)));
          Modelica.Thermal.HeatTransfer.Components.Convection convection2 annotation(
            Placement(transformation(origin = {10, 40}, extent = {{10, -10}, {-10, 10}}, rotation = 90)));
          Modelica.Blocks.Sources.Constant volumeFlow(k = 1) annotation(
            Placement(transformation(extent = {{-60, 10}, {-40, 30}}, rotation = 0)));
          Modelica.Blocks.Sources.Constant heatFlow1(k = 5) annotation(
            Placement(transformation(extent = {{-60, -70}, {-40, -50}}, rotation = 0)));
          Modelica.Blocks.Sources.Constant heatFlow2(k = 10) annotation(
            Placement(transformation(extent = {{-60, 50}, {-40, 70}}, rotation = 0)));
          Modelica.Blocks.Sources.Constant thermalConductance1(k = 1) annotation(
            Placement(transformation(extent = {{-30, -50}, {-10, -30}}, rotation = 0)));
          Modelica.Blocks.Sources.Constant thermalConductance2(k = 1) annotation(
            Placement(transformation(extent = {{-30, 30}, {-10, 50}}, rotation = 0)));
        equation
          connect(ambient1.flowPort, pump.flowPort_a) annotation(
            Line(points = {{-60, 0}, {-40, 0}}, color = {255, 0, 0}));
          connect(pump.flowPort_b, pipe1.flowPort_a) annotation(
            Line(points = {{-20, 0}, {-10, 0}, {-10, -10}, {0, -10}}, color = {255, 0, 0}));
          connect(pump.flowPort_b, pipe2.flowPort_a) annotation(
            Line(points = {{-20, 0}, {-10, 0}, {-10, 10}, {0, 10}}, color = {255, 0, 0}));
          connect(heatFlow2.y, prescribedHeatFlow2.Q_flow) annotation(
            Line(points = {{-39, 60}, {-30, 60}}, color = {0, 0, 255}));
          connect(heatFlow1.y, prescribedHeatFlow1.Q_flow) annotation(
            Line(points = {{-39, -60}, {-30, -60}}, color = {0, 0, 255}));
          connect(thermalConductance2.y, convection2.Gc) annotation(
            Line(points = {{-9, 40}, {0, 40}}, color = {0, 0, 127}));
          connect(thermalConductance1.y, convection1.Gc) annotation(
            Line(points = {{-9, -40}, {0, -40}}, color = {0, 0, 127}));
          connect(pipe1.heatPort, convection1.fluid) annotation(
            Line(points = {{10, -20}, {10, -30}}, color = {191, 0, 0}));
          connect(convection2.fluid, pipe2.heatPort) annotation(
            Line(points = {{10, 30}, {10, 20}}, color = {191, 0, 0}));
          connect(convection2.solid, prescribedHeatFlow2.port) annotation(
            Line(points = {{10, 50}, {10, 60}, {-10, 60}}, color = {191, 0, 0}));
          connect(convection2.solid, heatCapacitor2.port) annotation(
            Line(points = {{10, 50}, {10, 60}, {28, 60}}, color = {191, 0, 0}));
          connect(convection1.solid, prescribedHeatFlow1.port) annotation(
            Line(points = {{10, -50}, {10, -60}, {-10, -60}}, color = {191, 0, 0}));
          connect(convection1.solid, heatCapacitor1.port) annotation(
            Line(points = {{10, -50}, {10, -60}, {30, -60}}, color = {191, 0, 0}));
          connect(pipe2.flowPort_b, pipe3.flowPort_a) annotation(
            Line(points = {{20, 10}, {30, 10}, {30, 0}, {40, 0}}, color = {255, 0, 0}));
          connect(pipe1.flowPort_b, pipe3.flowPort_a) annotation(
            Line(points = {{20, -10}, {30, -10}, {30, 0}, {40, 0}}, color = {255, 0, 0}));
          connect(pipe3.flowPort_b, ambient2.flowPort) annotation(
            Line(points = {{60, 0}, {80, 0}}, color = {255, 0, 0}));
          connect(volumeFlow.y, pump.volumeFlow) annotation(
            Line(points = {{-39, 20}, {-30, 20}, {-30, 10}}, color = {0, 0, 127}, smooth = Smooth.None));
          annotation(
            Documentation(info = "<HTML>
<p>
2nd test example: ParallelCooling
</p>
Two prescribed heat sources dissipate their heat through thermal conductors to coolant flows. The coolant flow is taken from an ambient and driven by a pump with prescribed mass flow, then split into two coolant flows connected to the two heat sources, and afterwards merged. Splitting of coolant flows is determined by pressure drop characteristic of the two pipes.<br>
<b>Results</b>:<br>
<table>
<tr>
<td valign=\"top\"><b>output</b></td>
<td valign=\"top\"><b>explanation</b></td>
<td valign=\"top\"><b>formula</b></td>
<td valign=\"top\"><b>actual steady-state value</b></td>
</tr>
<tr>
<td valign=\"top\">dTSource1</td>
<td valign=\"top\">Source1 over Ambient</td>
<td valign=\"top\">dTCoolant1 + dTtoPipe1</td>
<td valign=\"top\">15 K</td>
</tr>
<tr>
<td valign=\"top\">dTtoPipe1</td>
<td valign=\"top\">Source1 over Coolant1</td>
<td valign=\"top\">Losses1 / ThermalConductor1.G</td>
<td valign=\"top\"> 5 K</td>
</tr>
<tr>
<td valign=\"top\">dTCoolant1</td>
<td valign=\"top\">Coolant's temperature increase</td>
<td valign=\"top\">Losses * cp * totalMassFlow/2</td>
<td valign=\"top\">10 K</td>
</tr>
<tr>
<td valign=\"top\">dTSource2</td>
<td valign=\"top\">Source2 over Ambient</td>
<td valign=\"top\">dTCoolant2 + dTtoPipe2</td>
<td valign=\"top\">30 K</td>
</tr>
<tr>
<td valign=\"top\">dTtoPipe2</td>
<td valign=\"top\">Source2 over Coolant2</td>
<td valign=\"top\">Losses2 / ThermalConductor2.G</td>
<td valign=\"top\">10 K</td>
</tr>
<tr>
<td valign=\"top\">dTCoolant2</td>
<td valign=\"top\">Coolant's temperature increase</td>
<td valign=\"top\">Losses * cp * totalMassFlow/2</td>
<td valign=\"top\">20 K</td>
</tr>
<tr>
<td valign=\"top\">dTmixedCoolant</td>
<td valign=\"top\">mixed Coolant's temperature increase</td>
<td valign=\"top\">(dTCoolant1+dTCoolant2)/2</td>
<td valign=\"top\">15 K</td>
</tr>
</table>
</HTML>"),
            experiment(StopTime = 1.0, Interval = 0.001));
        end ParallelCooling;

        model IndirectCooling "Example: indirect cooling circuit"
          extends Modelica.Icons.Example;
          parameter FluidHeatFlow.Media.Medium outerMedium = FluidHeatFlow.Media.Medium() "Outer medium" annotation(
            choicesAllMatching = true);
          parameter FluidHeatFlow.Media.Medium innerMedium = FluidHeatFlow.Media.Medium() "Inner medium" annotation(
            choicesAllMatching = true);
          parameter Modelica.SIunits.Temperature TAmb(displayUnit = "degC") = 293.15 "Ambient temperature";
          output Modelica.SIunits.TemperatureDifference dTSource = prescribedHeatFlow.port.T - TAmb "Source over Ambient";
          output Modelica.SIunits.TemperatureDifference dTtoPipe = prescribedHeatFlow.port.T - pipe1.heatPort.T "Source over inner Coolant";
          output Modelica.SIunits.TemperatureDifference dTinnerCoolant = pipe1.dT "inner Coolant's temperature increase";
          output Modelica.SIunits.TemperatureDifference dTCooler = innerPipe.heatPort.T - outerPipe.heatPort.T "Cooler's temperature increase between inner and outer pipes";
          output Modelica.SIunits.TemperatureDifference dTouterCoolant = outerPipe.dT "outer Coolant's temperature increase";
          FluidHeatFlow.Sources.Ambient ambient1(constantAmbientTemperature = TAmb, medium = outerMedium, constantAmbientPressure = 0) annotation(
            Placement(transformation(extent = {{-60, 60}, {-80, 80}}, rotation = 0)));
          Sources.VolumeFlow outerPump(medium = outerMedium, m = 0, T0 = TAmb, useVolumeFlowInput = true, constantVolumeFlow = 1) annotation(
            Placement(transformation(extent = {{-40, 60}, {-20, 80}}, rotation = 0)));
          FluidHeatFlow.Sources.Ambient ambient2(constantAmbientTemperature = TAmb, medium = outerMedium, constantAmbientPressure = 0) annotation(
            Placement(transformation(extent = {{40, 60}, {60, 80}}, rotation = 0)));
          Modelica.Thermal.HeatTransfer.Components.ThermalConductor thermalConductor(G = 1) annotation(
            Placement(transformation(origin = {10, -70}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
          Modelica.Thermal.HeatTransfer.Components.HeatCapacitor heatCapacitor(C = 0.05, T(start = TAmb, fixed = true)) annotation(
            Placement(transformation(origin = {40, -90}, extent = {{-10, 10}, {10, -10}}, rotation = 90)));
          Modelica.Thermal.HeatTransfer.Sources.PrescribedHeatFlow prescribedHeatFlow annotation(
            Placement(transformation(origin = {-20, -90}, extent = {{10, -10}, {-10, 10}}, rotation = 180)));
          FluidHeatFlow.Components.HeatedPipe pipe1(medium = innerMedium, m = 0.1, T0 = TAmb, V_flowLaminar = 1, V_flowNominal = 2, h_g = 0, T0fixed = true, dpLaminar = 1000, dpNominal = 2000) annotation(
            Placement(transformation(extent = {{20, -50}, {0, -30}}, rotation = 0)));
          FluidHeatFlow.Sources.AbsolutePressure absolutePressure(p = 10000, medium = innerMedium) annotation(
            Placement(transformation(extent = {{40, -40}, {60, -20}}, rotation = 0)));
          Sources.VolumeFlow innerPump(medium = innerMedium, m = 0, T0 = TAmb, useVolumeFlowInput = true, constantVolumeFlow = 1) annotation(
            Placement(transformation(origin = {-20, -30}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
          Modelica.Blocks.Sources.Constant heatFlow(k = 10) annotation(
            Placement(transformation(extent = {{-60, -100}, {-40, -80}}, rotation = 0)));
          Modelica.Blocks.Sources.Constant outerVolumeFlow(k = 1) annotation(
            Placement(transformation(extent = {{-60, 80}, {-40, 100}}, rotation = 0)));
          Modelica.Blocks.Sources.Constant innerVolumeFlow(k = 1) annotation(
            Placement(transformation(extent = {{-60, -40}, {-40, -20}}, rotation = 0)));
          Modelica.Blocks.Sources.Constant outerGc(k = 2) annotation(
            Placement(transformation(extent = {{-40, 30}, {-20, 50}}, rotation = 0)));
          Modelica.Blocks.Sources.Constant innerGc(k = 2) annotation(
            Placement(transformation(extent = {{-40, 20}, {-20, 0}}, rotation = 0)));
          FluidHeatFlow.Components.HeatedPipe outerPipe(medium = outerMedium, m = 0.1, T0 = TAmb, V_flowLaminar = 0.1, dpLaminar(displayUnit = "Pa") = 0.1, V_flowNominal = 1, dpNominal(displayUnit = "Pa") = 1, h_g = 0, T0fixed = true) annotation(
            Placement(transformation(extent = {{0, 60}, {20, 80}}, rotation = 0)));
          FluidHeatFlow.Components.HeatedPipe innerPipe(medium = innerMedium, m = 0.1, T0 = TAmb, V_flowLaminar = 0.1, dpLaminar(displayUnit = "Pa") = 0.1, V_flowNominal = 1, dpNominal(displayUnit = "Pa") = 1, h_g = 0, T0fixed = true) annotation(
            Placement(transformation(extent = {{0, -10}, {20, -30}}, rotation = 0)));
          Modelica.Thermal.HeatTransfer.Components.Convection innerConvection annotation(
            Placement(transformation(origin = {10, 10}, extent = {{-10, 10}, {10, -10}}, rotation = 270)));
          Modelica.Thermal.HeatTransfer.Components.Convection outerConvection annotation(
            Placement(transformation(origin = {10, 40}, extent = {{10, 10}, {-10, -10}}, rotation = 270)));
        equation
          connect(ambient1.flowPort, outerPump.flowPort_a) annotation(
            Line(points = {{-60, 70}, {-40, 70}}, color = {255, 0, 0}));
          connect(prescribedHeatFlow.port, thermalConductor.port_a) annotation(
            Line(points = {{-10, -90}, {10, -90}, {10, -80}}, color = {191, 0, 0}));
          connect(heatCapacitor.port, thermalConductor.port_a) annotation(
            Line(points = {{30, -90}, {10, -90}, {10, -80}}, color = {191, 0, 0}));
          connect(pipe1.heatPort, thermalConductor.port_b) annotation(
            Line(points = {{10, -50}, {10, -60}}, color = {191, 0, 0}));
          connect(pipe1.flowPort_b, innerPump.flowPort_a) annotation(
            Line(points = {{0, -40}, {-20, -40}}, color = {255, 0, 0}));
          connect(absolutePressure.flowPort, pipe1.flowPort_a) annotation(
            Line(points = {{40, -30}, {40, -40}, {20, -40}}, color = {255, 0, 0}));
          connect(heatFlow.y, prescribedHeatFlow.Q_flow) annotation(
            Line(points = {{-39, -90}, {-30, -90}}, color = {0, 0, 255}));
          connect(innerPump.flowPort_b, innerPipe.flowPort_a) annotation(
            Line(points = {{-20, -20}, {0, -20}}, color = {255, 0, 0}));
          connect(innerPipe.flowPort_b, absolutePressure.flowPort) annotation(
            Line(points = {{20, -20}, {40, -20}, {40, -30}}, color = {255, 0, 0}));
          connect(outerPump.flowPort_b, outerPipe.flowPort_a) annotation(
            Line(points = {{-20, 70}, {0, 70}}, color = {255, 0, 0}));
          connect(outerPipe.flowPort_b, ambient2.flowPort) annotation(
            Line(points = {{20, 70}, {40, 70}}, color = {255, 0, 0}));
          connect(outerPipe.heatPort, outerConvection.fluid) annotation(
            Line(points = {{10, 60}, {10, 55}, {10, 50}}, color = {191, 0, 0}));
          connect(outerConvection.solid, innerConvection.solid) annotation(
            Line(points = {{10, 30}, {10, 20}, {10, 20}}, color = {191, 0, 0}));
          connect(innerConvection.fluid, innerPipe.heatPort) annotation(
            Line(points = {{10, 0}, {10, -5}, {10, -10}}, color = {191, 0, 0}));
          connect(innerGc.y, innerConvection.Gc) annotation(
            Line(points = {{-19, 10}, {-9.5, 10}, {-9.5, 10}, {0, 10}}, color = {0, 0, 127}));
          connect(outerGc.y, outerConvection.Gc) annotation(
            Line(points = {{-19, 40}, {0, 40}}, color = {0, 0, 127}));
          connect(outerVolumeFlow.y, outerPump.volumeFlow) annotation(
            Line(points = {{-39, 90}, {-30, 90}, {-30, 80}}, color = {0, 0, 127}, smooth = Smooth.None));
          connect(innerVolumeFlow.y, innerPump.volumeFlow) annotation(
            Line(points = {{-39, -30}, {-30, -30}}, color = {0, 0, 127}, smooth = Smooth.None));
          annotation(
            Documentation(info = "<HTML>
<p>
3rd test example: IndirectCooling
</p>
A prescribed heat sources dissipates its heat through a thermal conductor to the inner coolant cycle. It is necessary to define the pressure level of the inner coolant cycle. The inner coolant cycle is coupled to the outer coolant flow through a thermal conductor.<br>
Inner coolant's temperature rise near the source is the same as temperature drop near the cooler.<br>
<b>Results</b>:<br>
<table>
<tr>
<td valign=\"top\"><b>output</b></td>
<td valign=\"top\"><b>explanation</b></td>
<td valign=\"top\"><b>formula</b></td>
<td valign=\"top\"><b>actual steady-state value</b></td>
</tr>
<tr>
<td valign=\"top\">dTSource</td>
<td valign=\"top\">Source over Ambient</td>
<td valign=\"top\">dtouterCoolant + dtCooler + dTinnerCoolant + dtToPipe</td>
<td valign=\"top\">40 K</td>
</tr>
<tr>
<td valign=\"top\">dTtoPipe</td>
<td valign=\"top\">Source over inner Coolant</td>
<td valign=\"top\">Losses / ThermalConductor.G</td>
<td valign=\"top\">10 K</td>
</tr>
<tr>
<td valign=\"top\">dTinnerColant</td>
<td valign=\"top\">inner Coolant's temperature increase</td>
<td valign=\"top\">Losses * cp * innerMassFlow</td>
<td valign=\"top\">10 K</td>
</tr>
<tr>
<td valign=\"top\">dTCooler</td>
<td valign=\"top\">Cooler's temperature rise between inner and outer pipes</td>
<td valign=\"top\">Losses * (innerGc + outerGc)</td>
<td valign=\"top\">10 K</td>
</tr>
<tr>
<td valign=\"top\">dTouterColant</td>
<td valign=\"top\">outer Coolant's temperature increase</td>
<td valign=\"top\">Losses * cp * outerMassFlow</td>
<td valign=\"top\">10 K</td>
</tr>
</table>
</HTML>"),
            experiment(StopTime = 1.5, Interval = 0.001));
        end IndirectCooling;

        model PumpAndValve "Example: cooling circuit with pump and valve"
          extends Modelica.Icons.Example;
          parameter FluidHeatFlow.Media.Medium medium = FluidHeatFlow.Media.Medium() "Cooling medium" annotation(
            choicesAllMatching = true);
          parameter Modelica.SIunits.Temperature TAmb(displayUnit = "degC") = 293.15 "Ambient temperature";
          output Modelica.SIunits.TemperatureDifference dTSource = prescribedHeatFlow.port.T - TAmb "Source over Ambient";
          output Modelica.SIunits.TemperatureDifference dTtoPipe = prescribedHeatFlow.port.T - pipe.heatPort.T "Source over Coolant";
          output Modelica.SIunits.TemperatureDifference dTCoolant = pipe.dT "Coolant's temperature increase";
          FluidHeatFlow.Sources.Ambient ambient1(constantAmbientTemperature = TAmb, medium = medium, constantAmbientPressure = 0) annotation(
            Placement(transformation(extent = {{-70, -10}, {-90, 10}}, rotation = 0)));
          FluidHeatFlow.Sources.IdealPump idealPump(medium = medium, m = 0, T0 = TAmb, V_flow0 = 2, V_flow(start = 0), wNominal(displayUnit = "rad/s") = 1, dp0(displayUnit = "Pa") = 2) annotation(
            Placement(transformation(extent = {{-60, 10}, {-40, -10}}, rotation = 0)));
          FluidHeatFlow.Components.Valve valve(medium = medium, m = 0, T0 = TAmb, LinearCharacteristic = false, y1 = 1, Kv1 = 1, kv0 = 0.01, dp0(displayUnit = "Pa") = 1, rho0 = 10, frictionLoss = 0) annotation(
            Placement(transformation(extent = {{-30, -10}, {-10, 10}}, rotation = 0)));
          FluidHeatFlow.Components.HeatedPipe pipe(medium = medium, T0 = TAmb, m = 0.1, V_flowLaminar = 0.1, dpLaminar(displayUnit = "Pa") = 0.1, V_flowNominal = 1, dpNominal(displayUnit = "Pa") = 1, h_g = 0, T0fixed = true) annotation(
            Placement(transformation(extent = {{0, -10}, {20, 10}}, rotation = 0)));
          FluidHeatFlow.Sources.Ambient ambient2(constantAmbientTemperature = TAmb, medium = medium, constantAmbientPressure = 0) annotation(
            Placement(transformation(extent = {{40, -10}, {60, 10}}, rotation = 0)));
          Modelica.Thermal.HeatTransfer.Components.HeatCapacitor heatCapacitor(C = 0.1, T(start = TAmb, fixed = true)) annotation(
            Placement(transformation(origin = {40, -50}, extent = {{-10, 10}, {10, -10}}, rotation = 90)));
          Modelica.Thermal.HeatTransfer.Sources.PrescribedHeatFlow prescribedHeatFlow annotation(
            Placement(transformation(extent = {{-30, -40}, {-10, -60}}, rotation = 0)));
          Modelica.Blocks.Sources.Constant heatFlow(k = 10) annotation(
            Placement(transformation(extent = {{-60, -60}, {-40, -40}}, rotation = 0)));
          Modelica.Thermal.HeatTransfer.Components.Convection convection annotation(
            Placement(transformation(origin = {10, -30}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
          Modelica.Blocks.Sources.Constant thermalConductance(k = 1) annotation(
            Placement(transformation(extent = {{-30, -40}, {-10, -20}}, rotation = 0)));
          Modelica.Mechanics.Rotational.Sources.Speed speed(exact = true, useSupport = false) annotation(
            Placement(transformation(origin = {-50, 30}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          Modelica.Blocks.Sources.Ramp speedRamp(height = 0.5, offset = 0.5, duration = 0.1, startTime = 0.4) annotation(
            Placement(transformation(extent = {{-80, 40}, {-60, 60}}, rotation = 0)));
          Modelica.Blocks.Sources.Ramp valveRamp(height = 0.5, offset = 0.5, duration = 0.1, startTime = 0.9) annotation(
            Placement(transformation(extent = {{12, 40}, {-8, 60}}, rotation = 0)));
        equation
          connect(pipe.flowPort_b, ambient2.flowPort) annotation(
            Line(points = {{20, 0}, {40, 0}}, color = {255, 0, 0}));
          connect(heatFlow.y, prescribedHeatFlow.Q_flow) annotation(
            Line(points = {{-39, -50}, {-30, -50}}, color = {0, 0, 255}));
          connect(convection.solid, prescribedHeatFlow.port) annotation(
            Line(points = {{10, -40}, {10, -50}, {-10, -50}}, color = {191, 0, 0}));
          connect(convection.solid, heatCapacitor.port) annotation(
            Line(points = {{10, -40}, {10, -50}, {30, -50}}, color = {191, 0, 0}));
          connect(pipe.heatPort, convection.fluid) annotation(
            Line(points = {{10, -10}, {10, -20}}, color = {191, 0, 0}));
          connect(thermalConductance.y, convection.Gc) annotation(
            Line(points = {{-9, -30}, {0, -30}}, color = {0, 0, 127}));
          connect(ambient1.flowPort, idealPump.flowPort_a) annotation(
            Line(points = {{-70, 0}, {-60, 0}}, color = {255, 0, 0}));
          connect(idealPump.flowPort_b, valve.flowPort_a) annotation(
            Line(points = {{-40, 0}, {-30, 0}}, color = {255, 0, 0}));
          connect(valve.flowPort_b, pipe.flowPort_a) annotation(
            Line(points = {{-10, 0}, {0, 0}}, color = {255, 0, 0}));
          connect(speedRamp.y, speed.w_ref) annotation(
            Line(points = {{-59, 50}, {-50, 50}, {-50, 42}}, color = {0, 0, 127}));
          connect(valveRamp.y, valve.y) annotation(
            Line(points = {{-9, 50}, {-20, 50}, {-20, 9}}, color = {0, 0, 127}));
          connect(speed.flange, idealPump.flange_a) annotation(
            Line(points = {{-50, 20}, {-50, 10}}, color = {0, 0, 0}, smooth = Smooth.None));
          annotation(
            Documentation(info = "<HTML>
<p>
4th test example: PumpAndValve
</p>
The pump is running with half speed for 0.4 s,
afterwards with full speed (using a ramp of 0.1 s).<br>
The valve is half open for 0.9 s, afterwards full open (using a ramp of 0.1 s).<br>
You may try to:
<ul>
<li>drive the pump with variable speed and let the valve full open
    to regulate the volume flow rate of coolant</li>
<li>drive the pump with constant speed and throttle the valve
    to regulate the volume flow rate of coolant</li>
</ul>
</HTML>"),
            experiment(StopTime = 2.0, Interval = 0.001));
        end PumpAndValve;

        model PumpDropOut "Example: cooling circuit with drop out of pump"
          extends Modelica.Icons.Example;
          parameter FluidHeatFlow.Media.Medium medium = FluidHeatFlow.Media.Medium() "Cooling medium" annotation(
            choicesAllMatching = true);
          parameter Modelica.SIunits.Temperature TAmb(displayUnit = "degC") = 293.15 "Ambient temperature";
          output Modelica.SIunits.TemperatureDifference dTSource = prescribedHeatFlow.port.T - TAmb "Source over Ambient";
          output Modelica.SIunits.TemperatureDifference dTtoPipe = prescribedHeatFlow.port.T - pipe.heatPort.T "Source over Coolant";
          output Modelica.SIunits.TemperatureDifference dTCoolant = pipe.dT "Coolant's temperature increase";
          FluidHeatFlow.Sources.Ambient ambient1(constantAmbientTemperature = TAmb, medium = medium, constantAmbientPressure = 0) annotation(
            Placement(transformation(extent = {{-60, -10}, {-80, 10}}, rotation = 0)));
          Sources.VolumeFlow pump(medium = medium, m = 0, T0 = TAmb, useVolumeFlowInput = true, constantVolumeFlow = 1) annotation(
            Placement(transformation(extent = {{-40, -10}, {-20, 10}}, rotation = 0)));
          FluidHeatFlow.Components.HeatedPipe pipe(medium = medium, T0 = TAmb, m = 0.1, V_flowLaminar = 0.1, dpLaminar(displayUnit = "Pa") = 0.1, V_flowNominal = 1, dpNominal(displayUnit = "Pa") = 1, h_g = 0, T0fixed = true) annotation(
            Placement(transformation(extent = {{0, -10}, {20, 10}}, rotation = 0)));
          FluidHeatFlow.Sources.Ambient ambient2(constantAmbientTemperature = TAmb, medium = medium, constantAmbientPressure = 0) annotation(
            Placement(transformation(extent = {{40, -10}, {60, 10}}, rotation = 0)));
          Modelica.Thermal.HeatTransfer.Components.HeatCapacitor heatCapacitor(C = 0.1, T(start = TAmb, fixed = true)) annotation(
            Placement(transformation(origin = {40, -50}, extent = {{-10, 10}, {10, -10}}, rotation = 90)));
          Modelica.Thermal.HeatTransfer.Sources.PrescribedHeatFlow prescribedHeatFlow annotation(
            Placement(transformation(extent = {{-30, -40}, {-10, -60}}, rotation = 0)));
          Utilities.DoubleRamp volumeFlow annotation(
            Placement(transformation(extent = {{-60, 10}, {-40, 30}}, rotation = 0)));
          Modelica.Blocks.Sources.Constant heatFlow(k = 10) annotation(
            Placement(transformation(extent = {{-60, -60}, {-40, -40}}, rotation = 0)));
          Modelica.Thermal.HeatTransfer.Components.Convection convection annotation(
            Placement(transformation(origin = {10, -30}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
          Modelica.Blocks.Sources.Constant thermalConductance(k = 1) annotation(
            Placement(transformation(extent = {{-30, -40}, {-10, -20}}, rotation = 0)));
        equation
          connect(ambient1.flowPort, pump.flowPort_a) annotation(
            Line(points = {{-60, 0}, {-40, 0}}, color = {255, 0, 0}));
          connect(pump.flowPort_b, pipe.flowPort_a) annotation(
            Line(points = {{-20, 0}, {0, 0}}, color = {255, 0, 0}));
          connect(pipe.flowPort_b, ambient2.flowPort) annotation(
            Line(points = {{20, 0}, {40, 0}}, color = {255, 0, 0}));
          connect(heatFlow.y, prescribedHeatFlow.Q_flow) annotation(
            Line(points = {{-39, -50}, {-30, -50}}, color = {0, 0, 255}));
          connect(convection.solid, prescribedHeatFlow.port) annotation(
            Line(points = {{10, -40}, {10, -50}, {-10, -50}}, color = {191, 0, 0}));
          connect(convection.solid, heatCapacitor.port) annotation(
            Line(points = {{10, -40}, {10, -50}, {30, -50}}, color = {191, 0, 0}));
          connect(pipe.heatPort, convection.fluid) annotation(
            Line(points = {{10, -10}, {10, -20}}, color = {191, 0, 0}));
          connect(thermalConductance.y, convection.Gc) annotation(
            Line(points = {{-9, -30}, {0, -30}}, color = {0, 0, 127}));
          connect(volumeFlow.y, pump.volumeFlow) annotation(
            Line(points = {{-39, 20}, {-30, 20}, {-30, 10}}, color = {0, 0, 127}, smooth = Smooth.None));
          annotation(
            Documentation(info = "<HTML>
<p>
5th test example: PumpDropOut
</p>
Same as 1st test example, but with a drop out of the pump:<br>
The pump is running for 0.2 s, then shut down (using a ramp of 0.2 s) for 0.2 s,
then started again (using a ramp of 0.2 s).
</HTML>"),
            experiment(StopTime = 2.0, Interval = 0.001));
        end PumpDropOut;

        model ParallelPumpDropOut "Example: cooling circuit with parallel branches and drop out of pump"
          extends Modelica.Icons.Example;
          parameter FluidHeatFlow.Media.Medium medium = FluidHeatFlow.Media.Medium() "Cooling medium" annotation(
            choicesAllMatching = true);
          parameter Modelica.SIunits.Temperature TAmb(displayUnit = "degC") = 293.15 "Ambient temperature";
          output Modelica.SIunits.TemperatureDifference dTSource1 = prescribedHeatFlow1.port.T - TAmb "Source1 over Ambient";
          output Modelica.SIunits.TemperatureDifference dTtoPipe1 = prescribedHeatFlow1.port.T - pipe1.heatPort.T "Source1 over Coolant1";
          output Modelica.SIunits.TemperatureDifference dTCoolant1 = pipe1.dT "Coolant1's temperature increase";
          output Modelica.SIunits.TemperatureDifference dTSource2 = prescribedHeatFlow2.port.T - TAmb "Source2 over Ambient";
          output Modelica.SIunits.TemperatureDifference dTtoPipe2 = prescribedHeatFlow2.port.T - pipe2.heatPort.T "Source2 over Coolant2";
          output Modelica.SIunits.TemperatureDifference dTCoolant2 = pipe2.dT "Coolant2's temperature increase";
          output Modelica.SIunits.TemperatureDifference dTmixedCoolant = ambient2.T_port - ambient1.T_port "mixed Coolant's temperature increase";
          FluidHeatFlow.Sources.Ambient ambient1(constantAmbientTemperature = TAmb, medium = medium, constantAmbientPressure = 0) annotation(
            Placement(transformation(extent = {{-60, -10}, {-80, 10}}, rotation = 0)));
          Sources.VolumeFlow pump(medium = medium, m = 0, T0 = TAmb, useVolumeFlowInput = true, constantVolumeFlow = 1) annotation(
            Placement(transformation(extent = {{-40, -10}, {-20, 10}}, rotation = 0)));
          FluidHeatFlow.Components.HeatedPipe pipe1(medium = medium, m = 0.1, T0 = TAmb, V_flowLaminar = 0.1, dpLaminar(displayUnit = "Pa") = 0.1, V_flowNominal = 1, dpNominal(displayUnit = "Pa") = 1, h_g = 0, V_flow(start = 0), T0fixed = true) annotation(
            Placement(transformation(extent = {{0, -20}, {20, 0}}, rotation = 0)));
          FluidHeatFlow.Components.HeatedPipe pipe2(medium = medium, m = 0.1, T0 = TAmb, V_flowLaminar = 0.1, dpLaminar(displayUnit = "Pa") = 0.1, V_flowNominal = 1, dpNominal(displayUnit = "Pa") = 1, h_g = 0, V_flow(start = 0), T0fixed = true) annotation(
            Placement(transformation(extent = {{0, 20}, {20, 0}}, rotation = 0)));
          FluidHeatFlow.Components.IsolatedPipe pipe3(medium = medium, T0 = TAmb, m = 0.1, V_flowLaminar = 0.1, dpLaminar(displayUnit = "Pa") = 0.1, V_flowNominal = 1, dpNominal(displayUnit = "Pa") = 1, h_g = 0, T0fixed = true) annotation(
            Placement(transformation(extent = {{40, -10}, {60, 10}}, rotation = 0)));
          FluidHeatFlow.Sources.Ambient ambient2(constantAmbientTemperature = TAmb, medium = medium, constantAmbientPressure = 0) annotation(
            Placement(transformation(extent = {{80, -10}, {100, 10}}, rotation = 0)));
          Modelica.Thermal.HeatTransfer.Components.HeatCapacitor heatCapacitor1(C = 0.1, T(start = TAmb, fixed = true)) annotation(
            Placement(transformation(origin = {40, -60}, extent = {{-10, 10}, {10, -10}}, rotation = 90)));
          Modelica.Thermal.HeatTransfer.Sources.PrescribedHeatFlow prescribedHeatFlow1 annotation(
            Placement(transformation(origin = {-20, -60}, extent = {{10, -10}, {-10, 10}}, rotation = 180)));
          Modelica.Thermal.HeatTransfer.Components.Convection Convection1 annotation(
            Placement(transformation(origin = {10, -40}, extent = {{10, 10}, {-10, -10}}, rotation = 270)));
          Modelica.Thermal.HeatTransfer.Components.HeatCapacitor heatCapacitor2(C = 0.1, T(start = TAmb, fixed = true)) annotation(
            Placement(transformation(origin = {38, 60}, extent = {{10, -10}, {-10, 10}}, rotation = 270)));
          Modelica.Thermal.HeatTransfer.Sources.PrescribedHeatFlow prescribedHeatFlow2 annotation(
            Placement(transformation(origin = {-20, 60}, extent = {{10, 10}, {-10, -10}}, rotation = 180)));
          Modelica.Thermal.HeatTransfer.Components.Convection convection2 annotation(
            Placement(transformation(origin = {10, 40}, extent = {{10, -10}, {-10, 10}}, rotation = 90)));
          Utilities.DoubleRamp volumeFlow annotation(
            Placement(transformation(extent = {{-60, 10}, {-40, 30}}, rotation = 0)));
          Modelica.Blocks.Sources.Constant heatFlow1(k = 5) annotation(
            Placement(transformation(extent = {{-60, -70}, {-40, -50}}, rotation = 0)));
          Modelica.Blocks.Sources.Constant heatFlow2(k = 10) annotation(
            Placement(transformation(extent = {{-60, 50}, {-40, 70}}, rotation = 0)));
          Modelica.Blocks.Sources.Constant thermalConductance1(k = 1) annotation(
            Placement(transformation(extent = {{-30, -50}, {-10, -30}}, rotation = 0)));
          Modelica.Blocks.Sources.Constant thermalConductance2(k = 1) annotation(
            Placement(transformation(extent = {{-30, 30}, {-10, 50}}, rotation = 0)));
        equation
          connect(ambient1.flowPort, pump.flowPort_a) annotation(
            Line(points = {{-60, 0}, {-60, 0}, {-40, 0}}, color = {255, 0, 0}));
          connect(pump.flowPort_b, pipe1.flowPort_a) annotation(
            Line(points = {{-20, 0}, {-10, 0}, {-10, -10}, {0, -10}}, color = {255, 0, 0}));
          connect(pump.flowPort_b, pipe2.flowPort_a) annotation(
            Line(points = {{-20, 0}, {-10, 0}, {-10, 10}, {0, 10}}, color = {255, 0, 0}));
          connect(heatFlow2.y, prescribedHeatFlow2.Q_flow) annotation(
            Line(points = {{-39, 60}, {-30, 60}}, color = {0, 0, 255}));
          connect(heatFlow1.y, prescribedHeatFlow1.Q_flow) annotation(
            Line(points = {{-39, -60}, {-30, -60}}, color = {0, 0, 255}));
          connect(thermalConductance2.y, convection2.Gc) annotation(
            Line(points = {{-9, 40}, {0, 40}}, color = {0, 0, 127}));
          connect(thermalConductance1.y, Convection1.Gc) annotation(
            Line(points = {{-9, -40}, {0, -40}}, color = {0, 0, 127}));
          connect(pipe1.heatPort, Convection1.fluid) annotation(
            Line(points = {{10, -20}, {10, -25}, {10, -30}}, color = {191, 0, 0}));
          connect(convection2.fluid, pipe2.heatPort) annotation(
            Line(points = {{10, 30}, {10, 20}}, color = {191, 0, 0}));
          connect(convection2.solid, prescribedHeatFlow2.port) annotation(
            Line(points = {{10, 50}, {10, 60}, {-10, 60}}, color = {191, 0, 0}));
          connect(convection2.solid, heatCapacitor2.port) annotation(
            Line(points = {{10, 50}, {10, 60}, {28, 60}}, color = {191, 0, 0}));
          connect(Convection1.solid, prescribedHeatFlow1.port) annotation(
            Line(points = {{10, -50}, {10, -60}, {-10, -60}}, color = {191, 0, 0}));
          connect(Convection1.solid, heatCapacitor1.port) annotation(
            Line(points = {{10, -50}, {10, -60}, {30, -60}}, color = {191, 0, 0}));
          connect(pipe2.flowPort_b, pipe3.flowPort_a) annotation(
            Line(points = {{20, 10}, {30, 10}, {30, 0}, {40, 0}}, color = {255, 0, 0}));
          connect(pipe1.flowPort_b, pipe3.flowPort_a) annotation(
            Line(points = {{20, -10}, {30, -10}, {30, 0}, {40, 0}}, color = {255, 0, 0}));
          connect(pipe3.flowPort_b, ambient2.flowPort) annotation(
            Line(points = {{60, 0}, {80, 0}}, color = {255, 0, 0}));
          connect(volumeFlow.y, pump.volumeFlow) annotation(
            Line(points = {{-39, 20}, {-30, 20}, {-30, 10}}, color = {0, 0, 127}, smooth = Smooth.None));
          annotation(
            Documentation(info = "<HTML>
<p>
6th test example: ParallelPumpDropOut
</p>
Same as 2nd test example, but with a drop out of the pump:<br>
The pump is running for 0.2 s, then shut down (using a ramp of 0.2 s) for 0.2 s,
then started again (using a ramp of 0.2 s).
</HTML>"),
            experiment(StopTime = 2.0, Interval = 0.001));
        end ParallelPumpDropOut;

        model OneMass "Example: cooling of one hot mass"
          extends Modelica.Icons.Example;
          parameter FluidHeatFlow.Media.Medium medium = FluidHeatFlow.Media.Medium() "Cooling medium" annotation(
            choicesAllMatching = true);
          parameter Modelica.SIunits.Temperature TAmb(displayUnit = "degC") = 293.15 "Ambient temperature";
          parameter Modelica.SIunits.Temperature TMass(displayUnit = "degC") = 313.15 "Inital temperature of mass";
          output Modelica.SIunits.TemperatureDifference dTMass = heatCapacitor.port.T - TAmb "Mass over Ambient";
          output Modelica.SIunits.TemperatureDifference dTtoPipe = heatCapacitor.port.T - pipe.heatPort.T "Mass over Coolant";
          output Modelica.SIunits.TemperatureDifference dTCoolant = pipe.dT "Coolant's temperature increase";
          FluidHeatFlow.Sources.Ambient ambient1(constantAmbientTemperature = TAmb, medium = medium, constantAmbientPressure = 0) annotation(
            Placement(transformation(extent = {{-60, -10}, {-80, 10}}, rotation = 0)));
          Sources.VolumeFlow pump(medium = medium, m = 0, T0 = TAmb, useVolumeFlowInput = true, constantVolumeFlow = 1) annotation(
            Placement(transformation(extent = {{-40, -10}, {-20, 10}}, rotation = 0)));
          FluidHeatFlow.Components.HeatedPipe pipe(medium = medium, m = 0.1, T0 = TAmb, V_flowLaminar = 0.1, dpLaminar(displayUnit = "Pa") = 0.1, V_flowNominal = 1, dpNominal(displayUnit = "Pa") = 1, h_g = 0, T0fixed = true) annotation(
            Placement(transformation(extent = {{0, -10}, {20, 10}}, rotation = 0)));
          FluidHeatFlow.Sources.Ambient ambient2(constantAmbientTemperature = TAmb, medium = medium, constantAmbientPressure = 0) annotation(
            Placement(transformation(extent = {{40, -10}, {60, 10}}, rotation = 0)));
          Modelica.Thermal.HeatTransfer.Components.HeatCapacitor heatCapacitor(C = 0.1, T(start = TMass, fixed = true)) annotation(
            Placement(transformation(origin = {10, -60}, extent = {{10, -10}, {-10, 10}}, rotation = 180)));
          Modelica.Thermal.HeatTransfer.Components.ThermalConductor thermalConductor(G = 1) annotation(
            Placement(transformation(origin = {10, -30}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
          Utilities.DoubleRamp volumeFlow(offset = 0, height_1 = 1, height_2 = -2) annotation(
            Placement(transformation(extent = {{-60, 10}, {-40, 30}}, rotation = 0)));
        equation
          connect(ambient1.flowPort, pump.flowPort_a) annotation(
            Line(points = {{-60, 0}, {-40, 0}}, color = {255, 0, 0}));
          connect(pump.flowPort_b, pipe.flowPort_a) annotation(
            Line(points = {{-20, 0}, {0, 0}}, color = {255, 0, 0}));
          connect(pipe.flowPort_b, ambient2.flowPort) annotation(
            Line(points = {{20, 0}, {40, 0}}, color = {255, 0, 0}));
          connect(thermalConductor.port_a, heatCapacitor.port) annotation(
            Line(points = {{10, -40}, {10, -45}, {10, -50}}, color = {191, 0, 0}));
          connect(pipe.heatPort, thermalConductor.port_b) annotation(
            Line(points = {{10, -10}, {10, -20}}, color = {191, 0, 0}));
          connect(volumeFlow.y, pump.volumeFlow) annotation(
            Line(points = {{-39, 20}, {-30, 20}, {-30, 10}}, color = {0, 0, 127}, smooth = Smooth.None));
          annotation(
            Documentation(info = "<HTML>
<p>
7th test example: OneMass
</p>
A thermal capacity is coupled with a coolant flow.
Different inital temperatures of thermal capacity and pipe's coolant get ambient's temperature,
the time behaviour depending on coolant flow.
</HTML>"),
            experiment(StopTime = 1.0, Interval = 0.001));
        end OneMass;

        model TwoMass "Example: cooling of two hot masses"
          extends Modelica.Icons.Example;
          parameter FluidHeatFlow.Media.Medium medium = FluidHeatFlow.Media.Medium() "Cooling medium" annotation(
            choicesAllMatching = true);
          parameter Modelica.SIunits.Temperature TAmb(displayUnit = "degC") = 293.15 "Ambient temperature";
          parameter Modelica.SIunits.Temperature TMass1(displayUnit = "degC") = 313.15 "Inital temperature of mass1";
          parameter Modelica.SIunits.Temperature TMass2(displayUnit = "degC") = 333.15 "Inital temperature of mass2";
          output Modelica.SIunits.TemperatureDifference dTMass1 = heatCapacitor1.port.T - TAmb "Mass1 over Ambient";
          output Modelica.SIunits.TemperatureDifference dTtoPipe1 = heatCapacitor1.port.T - pipe1.heatPort.T "Mass1 over Coolant1";
          output Modelica.SIunits.TemperatureDifference dTCoolant1 = pipe1.dT "Coolant1's temperature increase";
          output Modelica.SIunits.TemperatureDifference dTMass2 = heatCapacitor2.port.T - TAmb "Mass2 over Ambient";
          output Modelica.SIunits.TemperatureDifference dTtoPipe2 = heatCapacitor2.port.T - pipe2.heatPort.T "Mass2 over Coolant2";
          output Modelica.SIunits.TemperatureDifference dTCoolant2 = pipe2.dT "Coolant2's temperature increase";
          output Modelica.SIunits.TemperatureDifference dTmixedCoolant = ambient2.T_port - ambient1.T_port "mixed Coolant's temperature increase";
          FluidHeatFlow.Sources.Ambient ambient1(constantAmbientTemperature = TAmb, medium = medium, constantAmbientPressure = 0) annotation(
            Placement(transformation(extent = {{-60, -10}, {-80, 10}}, rotation = 0)));
          Sources.VolumeFlow pump(medium = medium, m = 0, T0 = TAmb, useVolumeFlowInput = true, constantVolumeFlow = 1) annotation(
            Placement(transformation(extent = {{-40, -10}, {-20, 10}}, rotation = 0)));
          FluidHeatFlow.Components.HeatedPipe pipe1(medium = medium, m = 0.1, T0 = TAmb, V_flow(start = 0), V_flowLaminar = 0.1, dpLaminar(displayUnit = "Pa") = 0.1, V_flowNominal = 1, dpNominal(displayUnit = "Pa") = 1, h_g = 0, T0fixed = true) annotation(
            Placement(transformation(extent = {{0, -20}, {20, 0}}, rotation = 0)));
          FluidHeatFlow.Components.HeatedPipe pipe2(medium = medium, m = 0.1, T0 = TAmb, V_flow(start = 0), V_flowLaminar = 0.1, dpLaminar(displayUnit = "Pa") = 0.1, V_flowNominal = 1, dpNominal(displayUnit = "Pa") = 1, h_g = 0, T0fixed = true) annotation(
            Placement(transformation(extent = {{0, 20}, {20, 0}}, rotation = 0)));
          FluidHeatFlow.Components.IsolatedPipe pipe3(medium = medium, m = 0.1, T0 = TAmb, V_flowLaminar = 0.1, dpLaminar(displayUnit = "Pa") = 0.1, V_flowNominal = 1, dpNominal(displayUnit = "Pa") = 1, h_g = 0, T0fixed = true) annotation(
            Placement(transformation(extent = {{40, -10}, {60, 10}}, rotation = 0)));
          FluidHeatFlow.Sources.Ambient ambient2(constantAmbientTemperature = TAmb, medium = medium, constantAmbientPressure = 0) annotation(
            Placement(transformation(extent = {{80, -10}, {100, 10}}, rotation = 0)));
          Modelica.Thermal.HeatTransfer.Components.HeatCapacitor heatCapacitor1(C = 0.1, T(start = TMass1, fixed = true)) annotation(
            Placement(transformation(origin = {10, -70}, extent = {{10, -10}, {-10, 10}}, rotation = 180)));
          Modelica.Thermal.HeatTransfer.Components.ThermalConductor thermalConductor1(G = 1) annotation(
            Placement(transformation(origin = {10, -40}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
          Modelica.Thermal.HeatTransfer.Components.HeatCapacitor heatCapacitor2(C = 0.1, T(start = TMass2, fixed = true)) annotation(
            Placement(transformation(origin = {10, 70}, extent = {{10, 10}, {-10, -10}}, rotation = 180)));
          Modelica.Thermal.HeatTransfer.Components.ThermalConductor thermalConductor2(G = 1) annotation(
            Placement(transformation(origin = {10, 40}, extent = {{10, -10}, {-10, 10}}, rotation = 90)));
          Utilities.DoubleRamp volumeFlow(offset = 0, height_1 = 1, height_2 = -2) annotation(
            Placement(transformation(extent = {{-60, 10}, {-40, 30}}, rotation = 0)));
        equation
          connect(ambient1.flowPort, pump.flowPort_a) annotation(
            Line(points = {{-60, 0}, {-40, 0}}, color = {255, 0, 0}));
          connect(pump.flowPort_b, pipe1.flowPort_a) annotation(
            Line(points = {{-20, 0}, {-10, 0}, {-10, -10}, {0, -10}}, color = {255, 0, 0}));
          connect(pump.flowPort_b, pipe2.flowPort_a) annotation(
            Line(points = {{-20, 0}, {-10, 0}, {-10, 10}, {0, 10}}, color = {255, 0, 0}));
          connect(pipe2.flowPort_b, pipe3.flowPort_a) annotation(
            Line(points = {{20, 10}, {30, 10}, {30, 0}, {40, 0}}, color = {255, 0, 0}));
          connect(pipe1.flowPort_b, pipe3.flowPort_a) annotation(
            Line(points = {{20, -10}, {30, -10}, {30, 0}, {40, 0}}, color = {255, 0, 0}));
          connect(pipe3.flowPort_b, ambient2.flowPort) annotation(
            Line(points = {{60, 0}, {80, 0}}, color = {255, 0, 0}));
          connect(heatCapacitor2.port, thermalConductor2.port_a) annotation(
            Line(points = {{10, 60}, {10, 55}, {10, 50}}, color = {191, 0, 0}));
          connect(thermalConductor2.port_b, pipe2.heatPort) annotation(
            Line(points = {{10, 30}, {10, 20}}, color = {191, 0, 0}));
          connect(pipe1.heatPort, thermalConductor1.port_b) annotation(
            Line(points = {{10, -20}, {10, -30}}, color = {191, 0, 0}));
          connect(thermalConductor1.port_a, heatCapacitor1.port) annotation(
            Line(points = {{10, -50}, {10, -55}, {10, -60}}, color = {191, 0, 0}));
          connect(volumeFlow.y, pump.volumeFlow) annotation(
            Line(points = {{-39, 20}, {-30, 20}, {-30, 10}}, color = {0, 0, 127}, smooth = Smooth.None));
          annotation(
            Documentation(info = "<HTML>
<p>
8th test example: TwoMass
</p>
Two thermal capacities are coupled with two parallel coolant flow.
Different inital temperatures of thermal capacities and pipe's coolants get ambient's temperature,
the time behaviour depending on coolant flow.
</HTML>"),
            experiment(StopTime = 1.0, Interval = 0.001));
        end TwoMass;

        package Utilities "Utility models for examples"
          extends Modelica.Icons.UtilitiesPackage;

          model DoubleRamp "Ramp going up and down"
            extends Modelica.Blocks.Interfaces.SO;
            parameter Real offset = 1 "Offset of ramps";
            parameter Modelica.SIunits.Time startTime = 0.2 "StartTime of 1st ramp";
            parameter Modelica.SIunits.Time interval = 0.2 "Interval between end of 1st and beginning of 2nd ramp";
            parameter Real height_1 = -1 "Height of ramp" annotation(
              Dialog(group = "Ramp 1"));
            parameter Modelica.SIunits.Time duration_1(min = Modelica.Constants.small) = 0.2 "Duration of ramp" annotation(
              Dialog(group = "Ramp 1"));
            parameter Real height_2 = 1 "Height of ramp" annotation(
              Dialog(group = "Ramp 2"));
            parameter Modelica.SIunits.Time duration_2(min = Modelica.Constants.small) = 0.2 "Duration of ramp" annotation(
              Dialog(group = "Ramp 2"));
            Modelica.Blocks.Math.Add add annotation(
              Placement(transformation(extent = {{10, -10}, {30, 10}}, rotation = 0)));
            Modelica.Blocks.Sources.Ramp ramp1(final height = height_1, final duration = duration_1, final startTime = startTime, final offset = offset) annotation(
              Placement(transformation(extent = {{-30, 10}, {-10, 30}}, rotation = 0)));
            Modelica.Blocks.Sources.Ramp ramp2(final height = height_2, final duration = duration_2, final startTime = startTime + duration_1 + interval, final offset = 0) annotation(
              Placement(transformation(extent = {{-30, -30}, {-10, -10}}, rotation = 0)));
          equation
            connect(ramp1.y, add.u1) annotation(
              Line(points = {{-9, 20}, {0, 20}, {0, 6}, {8, 6}}, color = {0, 0, 127}));
            connect(ramp2.y, add.u2) annotation(
              Line(points = {{-9, -20}, {0, -20}, {0, -6}, {8, -6}}, color = {0, 0, 127}));
            connect(add.y, y) annotation(
              Line(points = {{31, 0}, {110, 0}}, color = {0, 0, 127}));
            annotation(
              Documentation(info = "<HTML>
Block generating the sum of two ramps.
</HTML>"),
              Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-80, 68}, {-80, -80}}, color = {192, 192, 192}), Polygon(points = {{-80, 90}, {-88, 68}, {-72, 68}, {-80, 90}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-90, -70}, {82, -70}}, color = {192, 192, 192}), Polygon(points = {{90, -70}, {68, -62}, {68, -78}, {90, -70}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-80, -60}, {-50, -60}, {-30, 60}, {10, 60}, {30, -20}, {70, -20}}, color = {0, 0, 0})}));
          end DoubleRamp;
          annotation(
            Documentation(info = "<html>
This package contains utility components used for the test examples.<br>
<dl>
  <dt><b>Main Authors:</b></dt>
  <dd>
  <p>
  <a href=\"http://www.haumer.at/\">Anton Haumer</a><br>
  Technical Consulting &amp; Electrical Engineering<br>
  A-3423 St.Andrae-Woerdern, Austria<br>
  email: <a href=\"mailto:a.haumer@haumer.at\">a.haumer@haumer.at</a>
</p>
<p>
  Dr. Christian Kral<br>
  <a href=\"http://christiankral.net/\">Electric Machines, Drives and Systems</a><br>
  A-1060 Vienna, Austria<br>
  email: <a href=\"mailto:dr.christian.kral@gmail.com\">dr.christian.kral@gmail.com</a>
</p>
  </dd>
</dl>
<p>
Copyright &copy; 1998-2015, Modelica Association, Anton Haumer, Christian Kral and Austrian Institute of Technology, AIT.
</p>

<p>
<i>This Modelica package is <u>free</u> software and the use is completely at <u>your own risk</u>; it can be redistributed and/or modified under the terms of the Modelica License 2. For license conditions (including the disclaimer of warranty) see <a href=\"modelica://Modelica.UsersGuide.ModelicaLicense2\">Modelica.UsersGuide.ModelicaLicense2</a> or visit <a href=\"https://www.modelica.org/licenses/ModelicaLicense2\"> https://www.modelica.org/licenses/ModelicaLicense2</a>.</i>
</p>
</HTML>", revisions = "<HTML>
  <ul>
  <li> v1.41 Beta 2005/06/17 Anton Haumer<br>
       first used</li>
  </ul>
</HTML>"));
        end Utilities;
        annotation(
          Documentation(info = "<HTML>
This package contains test examples:
<ul>
<li>1.SimpleCooling: heat is dissipated through a media flow</li>
<li>2.ParallelCooling: two heat sources dissipate through merged media flows</li>
<li>3.IndirectCooling: heat is dissipated through two cooling cycles</li>
<li>4.PumpAndValve: demonstrates usage of an IdealPump and a Valve</li>
<li>5.PumpDropOut: demonstrates shutdown and restart of a pump</li>
<li>6.ParallelPumpDropOut: demonstrates shutdown and restart of a pump in a parallel circuit</li>
<li>7.OneMass: cooling of a mass (thermal capacity) by a coolant flow</li>
<li>8.TwoMass: cooling of two masses (thermal capacities) by two parallel coolant flows</li>
</ul>
<dl>
  <dt><b>Main Authors:</b></dt>
  <dd>
  <p>
  <a href=\"http://www.haumer.at/\">Anton Haumer</a><br>
  Technical Consulting &amp; Electrical Engineering<br>
  A-3423 St.Andrae-Woerdern, Austria<br>
  email: <a href=\"mailto:a.haumer@haumer.at\">a.haumer@haumer.at</a>
</p>
<p>
  Dr. Christian Kral<br>
  <a href=\"http://christiankral.net/\">Electric Machines, Drives and Systems</a><br>
  A-1060 Vienna, Austria<br>
  email: <a href=\"mailto:dr.christian.kral@gmail.com\">dr.christian.kral@gmail.com</a>
</p>
  </dd>
</dl>
<p>
Copyright &copy; 1998-2015, Modelica Association, Anton Haumer, Christian Kral and Austrian Institute of Technology, AIT.
</p>

<p>
<i>This Modelica package is <u>free</u> software and the use is completely at <u>your own risk</u>; it can be redistributed and/or modified under the terms of the Modelica License 2. For license conditions (including the disclaimer of warranty) see <a href=\"modelica://Modelica.UsersGuide.ModelicaLicense2\">Modelica.UsersGuide.ModelicaLicense2</a> or visit <a href=\"https://www.modelica.org/licenses/ModelicaLicense2\"> https://www.modelica.org/licenses/ModelicaLicense2</a>.</i>
</p>
</HTML>", revisions = "<HTML>
  <ul>
  <li> v1.00 2005/02/01 Anton Haumer<br>
       first stable official release</li>
  <li> v1.31 Beta 2005/06/04 Anton Haumer<br>
       <i>new example: PumpAndValve</i><br>
       <i>new example: PumpDropOut</i></li>
  <li> v1.42 Beta 2005/06/18 Anton Haumer<br>
       <i>new test example: ParallelPumpDropOut</i></li>
  <li> v1.43 Beta 2005/06/20 Anton Haumer<br>
       Test of mixing / semiLinear<br>
       <i>new test example: OneMass</i><br>
       <i>new test example: TwoMass</i></li>
  </ul>
</html>"));
      end Examples;

      package Components "Basic components (pipes, valves)"
        extends Modelica.Icons.Package;

        model IsolatedPipe "Pipe without heat exchange"
          extends Interfaces.Partials.TwoPort(final tapT = 1);
          extends Interfaces.Partials.SimpleFriction;
          parameter Modelica.SIunits.Length h_g(start = 0) "Geodetic height (height difference from flowPort_a to flowPort_b)";
        equation
// coupling with FrictionModel
          volumeFlow = V_flow;
          dp = pressureDrop + medium.rho * Modelica.Constants.g_n * h_g;
// no energy exchange with medium
          Q_flow = Q_friction;
          annotation(
            Documentation(info = "<HTML>
Pipe without heat exchange.<br>
Thermodynamic equations are defined by Partials.TwoPortMass(Q_flow = 0).<br>
<b>Note:</b> Setting parameter m (mass of medium within pipe) to zero
leads to neglection of temperature transient cv*m*der(T).
</HTML>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-90, 20}, {90, -20}}, lineColor = {255, 0, 0}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-150, 100}, {150, 40}}, textString = "%name")}));
        end IsolatedPipe;

        model HeatedPipe "Pipe with heat exchange"
          extends Interfaces.Partials.TwoPort;
          extends Interfaces.Partials.SimpleFriction;
          parameter Modelica.SIunits.Length h_g(start = 0) "Geodetic height (heigth difference from flowPort_a to flowPort_b)";
          Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a heatPort annotation(
            Placement(transformation(extent = {{-10, -110}, {10, -90}}, rotation = 0)));
        equation
// coupling with FrictionModel
          volumeFlow = V_flow;
          dp = pressureDrop + medium.rho * Modelica.Constants.g_n * h_g;
// energy exchange with medium
          Q_flow = heatPort.Q_flow + Q_friction;
// defines heatPort's temperature
          heatPort.T = T_q;
          annotation(
            Documentation(info = "<HTML>
Pipe with heat exchange.<br>
Thermodynamic equations are defined by Partials.TwoPort.<br>
Q_flow is defined by heatPort.Q_flow.<br>
<b>Note:</b> Setting parameter m (mass of medium within pipe) to zero
leads to neglection of temperature transient cv*m*der(T).<br>
<b>Note:</b> Injecting heat into a pipe with zero massflow causes
temperature rise defined by storing heat in medium's mass.
</HTML>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-90, 20}, {90, -20}}, lineColor = {255, 0, 0}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-150, 100}, {150, 40}}, textString = "%name"), Polygon(points = {{-10, -90}, {-10, -40}, {0, -20}, {10, -40}, {10, -90}, {-10, -90}}, lineColor = {255, 0, 0})}));
        end HeatedPipe;

        model Valve "Simple valve"
          extends Interfaces.Partials.TwoPort(m(start = 0), final tapT = 1);
          parameter Boolean LinearCharacteristic(start = true) "Type of characteristic" annotation(
            Dialog(group = "Standard characteristic"),
            choices(choice = true "Linear", choice = false "Exponential"));
          parameter Real y1(min = small, start = 1) "Max. valve opening" annotation(
            Dialog(group = "Standard characteristic"));
          parameter Modelica.SIunits.VolumeFlowRate Kv1(min = small, start = 1) "Max. flow @ y = y1" annotation(
            Dialog(group = "Standard characteristic"));
          parameter Real kv0(min = small, max = 1 - small, start = 0.01) "Leakage flow / max.flow @ y = 0" annotation(
            Dialog(group = "Standard characteristic"));
          parameter Modelica.SIunits.Pressure dp0(start = 1) "Standard pressure drop" annotation(
            Dialog(group = "Standard characteristic"));
          parameter Modelica.SIunits.Density rho0(start = 10) "Standard medium's density" annotation(
            Dialog(group = "Standard characteristic"));
          parameter Real frictionLoss(min = 0, max = 1, start = 0) "Part of friction losses fed to medium";
        protected
          constant Modelica.SIunits.VolumeFlowRate unitVolumeFlowRate = 1;
          constant Real small = Modelica.Constants.small;
          constant Modelica.SIunits.VolumeFlowRate smallVolumeFlowRate = eps * unitVolumeFlowRate;
          constant Real eps = Modelica.Constants.eps;
          Real yLim = max(min(y, y1), 0) "Limited valve opening";
          Modelica.SIunits.VolumeFlowRate Kv "Standard flow rate";
        public
          Modelica.Blocks.Interfaces.RealInput y annotation(
            Placement(transformation(origin = {0, 90}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
        initial algorithm
          assert(y1 > small, "Valve characteristic: y1 has to be > 0 !");
          assert(Kv1 > smallVolumeFlowRate, "Valve characteristic: Kv1 has to be > 0 !");
          assert(kv0 > small, "Valve characteristic: kv0 has to be > 0 !");
          assert(kv0 < 1 - eps, "Valve characteristic: kv0 has to be < 1 !");
        equation
// evaluate standard characteristic
          Kv / Kv1 = if LinearCharacteristic then kv0 + (1 - kv0) * yLim / y1 else kv0 * exp(Modelica.Math.log(1 / kv0) * yLim / y1);
// pressure drop under real conditions
          dp / dp0 = medium.rho / rho0 * (V_flow / Kv) * abs(V_flow / Kv);
// no energy exchange with medium
          Q_flow = frictionLoss * V_flow * dp;
          annotation(
            Documentation(info = "<HTML>
<p>Simple controlled valve.</p>
<p>
Standard characteristic Kv=<i>f </i>(y) is given at standard conditions (dp0, rho0),
</p>
<ul>
<li>either linear :<code> Kv/Kv1 = Kv0/Kv1 + (1-Kv0/Kv1) * y/Y1</code></li>
<li>or exponential:<code> Kv/Kv1 = Kv0/Kv1 * exp[log(Kv1/Kv0) * y/Y1]</code></li>
</ul>
<p>
where:
</p>
<ul>
<li><code>Kv0 ... min. flow @ y = 0</code></li>
<li><code>Y1 .... max. valve opening</code></li>
<li><code>Kv1 ... max. flow @ y = Y1</code></li>
</ul>
<p>
Flow resistance under real conditions is calculated by
</p>
<blockquote><pre>
V_flow**2 * rho / dp = Kv(y)**2 * rho0 / dp0
</pre></blockquote>
</HTML>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-150, -60}, {150, -120}}, textString = "%name"), Polygon(points = {{-90, 10}, {-60, 10}, {-60, 60}, {0, 0}, {60, 60}, {60, 10}, {90, 10}, {90, -10}, {60, -10}, {60, -60}, {0, 0}, {-60, -60}, {-60, -10}, {-90, -10}, {-90, 10}}, lineColor = {255, 0, 0}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid), Line(points = {{0, 80}, {0, 0}}, color = {0, 0, 127})}));
        end Valve;
        annotation(
          Documentation(info = "<HTML>
This package contains components:
<ul>
<li>pipe without heat exchange</li>
<li>pipe with heat exchange</li>
<li>valve (simple controlled valve)</li>
</ul>
Pressure drop is taken from partial model SimpleFriction.<br>
Thermodynamic equations are defined in partial models (package Partials).<br>
<dl>
  <dt><b>Main Authors:</b></dt>
  <dd>
  <p>
  <a href=\"http://www.haumer.at/\">Anton Haumer</a><br>
  Technical Consulting &amp; Electrical Engineering<br>
  A-3423 St.Andrae-Woerdern, Austria<br>
  email: <a href=\"mailto:a.haumer@haumer.at\">a.haumer@haumer.at</a>
</p>
<p>
  Dr. Christian Kral<br>
  <a href=\"http://christiankral.net/\">Electric Machines, Drives and Systems</a><br>
  A-1060 Vienna, Austria<br>
  email: <a href=\"mailto:dr.christian.kral@gmail.com\">dr.christian.kral@gmail.com</a>
</p>
  </dd>
</dl>
<p>
Copyright &copy; 1998-2015, Modelica Association, Anton Haumer, Christian Kral and Austrian Institute of Technology, AIT.
</p>
<p>
<i>This Modelica package is <u>free</u> software and the use is completely at <u>your own risk</u>; it can be redistributed and/or modified under the terms of the Modelica License 2. For license conditions (including the disclaimer of warranty) see <a href=\"modelica://Modelica.UsersGuide.ModelicaLicense2\">Modelica.UsersGuide.ModelicaLicense2</a> or visit <a href=\"https://www.modelica.org/licenses/ModelicaLicense2\"> https://www.modelica.org/licenses/ModelicaLicense2</a>.</i>
</p>
</HTML>", revisions = "<HTML>
  <ul>
  <li> v1.00 2005/02/01 Anton Haumer<br>
       first stable official release</li>
  <li> v1.20 Beta 2005/02/18 Anton Haumer<br>
       introduced geodetic height in Components.Pipes<br>
       <i>new models: Components.Valve</i></li>
  <li> v1.30 Beta 2005/06/02 Anton Haumer<br>
       friction losses are fed to medium</li>
  </ul>
</html>"),
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(origin = {6, 40}, lineColor = {0, 0, 255}, fillColor = {0, 128, 255}, fillPattern = FillPattern.Solid, points = {{-56, 10}, {-56, -90}, {-6, -40}, {44, 10}, {44, -90}, {-56, 10}}), Polygon(origin = {6, 40}, lineColor = {0, 0, 127}, fillColor = {0, 0, 127}, fillPattern = FillPattern.Solid, points = {{-16, 10}, {4, 10}, {-6, -10}, {-16, 10}}), Line(origin = {6, 40}, points = {{-6, -10}, {-6, -40}, {-6, -38}}, color = {0, 0, 127})}));
      end Components;

      package Media "Medium properties"
        extends Modelica.Icons.MaterialPropertiesPackage;

        record Medium "Record containing media properties"
          extends Modelica.Icons.Record;
          parameter Modelica.SIunits.Density rho = 1 "Density";
          parameter Modelica.SIunits.SpecificHeatCapacity cp = 1 "Specific heat capacity at constant pressure";
          parameter Modelica.SIunits.SpecificHeatCapacity cv = 1 "Specific heat capacity at constant volume";
          parameter Modelica.SIunits.ThermalConductivity lamda = 1 "Thermal conductivity";
          parameter Modelica.SIunits.KinematicViscosity nue = 1 "Kinematic viscosity";
          annotation(
            Documentation(info = "<html>
Record containing (constant) medium properties.
</html>"));
        end Medium;

        record Air_30degC "Medium: properties of air at 30 degC"
          extends Medium(rho = 1.149, cp = 1007, cv = 720, lamda = 0.0264, nue = 16.3E-6);
          annotation(
            Documentation(info = "<html>
Medium: properties of air at 30 degC
</html>"));
        end Air_30degC;

        record Air_70degC "Medium: properties of air at 70 degC"
          extends Medium(rho = 1.015, cp = 1010, cv = 723, lamda = 0.0293, nue = 20.3E-6);
          annotation(
            Documentation(info = "<html>
Medium: properties of air at 70 degC
</html>"));
        end Air_70degC;

        record Water "Medium: properties of water"
          extends Medium(rho = 995.6, cp = 4177, cv = 4177, lamda = 0.615, nue = 0.8E-6);
          annotation(
            Documentation(info = "<html>
Medium: properties of water
</html>"));
        end Water;
        annotation(
          Documentation(info = "<HTML>
This package contains definitions of medium properties.<br>
<dl>
  <dt><b>Main Authors:</b></dt>
  <dd>
  <p>
  <a href=\"http://www.haumer.at/\">Anton Haumer</a><br>
  Technical Consulting &amp; Electrical Engineering<br>
  A-3423 St.Andrae-Woerdern, Austria<br>
  email: <a href=\"mailto:a.haumer@haumer.at\">a.haumer@haumer.at</a>
</p>
<p>
  Dr. Christian Kral<br>
  <a href=\"http://christiankral.net/\">Electric Machines, Drives and Systems</a><br>
  A-1060 Vienna, Austria<br>
  email: <a href=\"mailto:dr.christian.kral@gmail.com\">dr.christian.kral@gmail.com</a>
</p>
  </dd>
</dl>
<p>
Copyright &copy; 1998-2015, Modelica Association, Anton Haumer, Christian Kral and Austrian Institute of Technology, AIT.
</p>
<p>
<i>This Modelica package is <u>free</u> software and the use is completely at <u>your own risk</u>; it can be redistributed and/or modified under the terms of the Modelica License 2. For license conditions (including the disclaimer of warranty) see <a href=\"modelica://Modelica.UsersGuide.ModelicaLicense2\">Modelica.UsersGuide.ModelicaLicense2</a> or visit <a href=\"https://www.modelica.org/licenses/ModelicaLicense2\"> https://www.modelica.org/licenses/ModelicaLicense2</a>.</i>
</p>
</HTML>", revisions = "<HTML>
  <ul>
  <li> v1.00 2005/02/01 Anton Haumer<br>
       first stable official release</li>
  <li> v1.11 2005/02/18 Anton Haumer<br>
       corrected usage of cv and cp</li>
  </ul>
</html>"));
      end Media;

      package Sensors "Ideal sensors to measure port properties"
        extends Modelica.Icons.SensorsPackage;

        model PressureSensor "Absolute pressure sensor"
          extends Interfaces.Partials.AbsoluteSensor(y(unit = "Pa", displayUnit = "bar") "Absolute pressure as output signal");
        equation
          y = flowPort.p;
          annotation(
            Documentation(info = "<HTML>
pSensor measures the absolute pressure.<br>
Thermodynamic equations are defined by Partials.AbsoluteSensor.
</HTML>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-22, -20}, {20, -60}}, lineColor = {0, 0, 255}, textString = "p")}));
        end PressureSensor;

        model TemperatureSensor "Absolute temperature sensor"
          extends Interfaces.Partials.AbsoluteSensor(y(unit = "K") "Absolute temperature as output signal");
        equation
          medium.cp * y = flowPort.h;
          annotation(
            Documentation(info = "<HTML>
TSensor measures the absolute temperature (Kelvin).<br>
Thermodynamic equations are defined by Partials.AbsoluteSensor.
</HTML>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-22, -20}, {20, -60}}, lineColor = {0, 0, 255}, textString = "T")}));
        end TemperatureSensor;

        model RelPressureSensor "Pressure difference sensor"
          extends Interfaces.Partials.RelativeSensor(y(unit = "Pa", displayUnit = "bar") "Pressure difference as output signal");
        equation
          y = flowPort_a.p - flowPort_b.p;
          annotation(
            Documentation(info = "<HTML>
dpSensor measures the pressure drop between flowPort_a and flowPort_b.<br>
Thermodynamic equations are defined by Partials.RelativeSensor.
</HTML>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-22, -20}, {20, -60}}, lineColor = {0, 0, 255}, textString = "dp")}));
        end RelPressureSensor;

        model RelTemperatureSensor "Temperature difference sensor"
          extends Interfaces.Partials.RelativeSensor(y(unit = "K") "Temperature difference as output signal");
        equation
          medium.cp * y = flowPort_a.h - flowPort_b.h;
          annotation(
            Documentation(info = "<HTML>
dTSensor measures the temperature difference between flowPort_a and flowPort_b.<br>
Thermodynamic equations are defined by Partials.RelativeSensor.<br>
<b>Note:</b> Connected flowPorts have the same temperature (mixing temperature)!<br>
Since mixing my occur, the outlet temperature of a component may be different from the connector's temperature.<br>
Outlet temperature is defined by variable T of the corresponding component.
</HTML>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-22, -20}, {20, -60}}, lineColor = {0, 0, 255}, textString = "dT")}));
        end RelTemperatureSensor;

        model MassFlowSensor "Mass flow sensor"
          extends Interfaces.Partials.FlowSensor(y(unit = "kg/s") "Mass flow as output signal");
        equation
          y = V_flow * medium.rho;
          annotation(
            Documentation(info = "<HTML>
m_flowSensor measures the mass flow rate.<br>
Thermodynamic equations are defined by Partials.FlowSensor.
</HTML>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-22, -20}, {20, -60}}, lineColor = {0, 0, 255}, textString = "m")}));
        end MassFlowSensor;

        model VolumeFlowSensor "Volume flow sensor"
          extends Interfaces.Partials.FlowSensor(y(unit = "m3/s") "Volume flow as output signal");
        equation
          y = V_flow;
          annotation(
            Documentation(info = "<HTML>
V_flowSensor measures the volume flow rate.<br>
Thermodynamic equations are defined by Partials.FlowSensor.
</HTML>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-22, -20}, {20, -60}}, lineColor = {0, 0, 255}, textString = "V")}));
        end VolumeFlowSensor;

        model EnthalpyFlowSensor "Enthalpy flow sensor"
          extends Interfaces.Partials.FlowSensor(y(unit = "W") "Enthalpy flow as output signal");
        equation
          y = flowPort_a.H_flow;
          annotation(
            Documentation(info = "<HTML>
H_flowSensor measures the enthalpy flow rate.<br>
Thermodynamic equations are defined by Partials.FlowSensor.
</HTML>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-22, -20}, {20, -60}}, lineColor = {0, 0, 255}, textString = "H")}));
        end EnthalpyFlowSensor;
        annotation(
          Documentation(info = "<HTML>
This package contains sensors:
<ul>
<li>pSensor: absolute pressure</li>
<li>TSensor: absolute temperature (Kelvin)</li>
<li>dpSensor: pressure drop between flowPort_a and flowPort_b</li>
<li>dTSensor: temperature difference between flowPort_a and flowPort_b</li>
<li>m_flowSensor: measures mass flow rate</li>
<li>V_flowSensor: measures volume flow rate</li>
<li>H_flowSensor: measures enthalpy flow rate</li>
</ul>
Some of the sensors do not need access to medium properties for measuring,
but it is necessary to define the medium in the connector (check of connections).<br>
Thermodynamic equations are defined in partial models (package Interfaces.Partials).<br>
All sensors are considered massless, they do not change mass flow or enthalpy flow.<br>
<dl>
  <dt><b>Main Authors:</b></dt>
  <dd>
  <p>
  <a href=\"http://www.haumer.at/\">Anton Haumer</a><br>
  Technical Consulting &amp; Electrical Engineering<br>
  A-3423 St.Andrae-Woerdern, Austria<br>
  email: <a href=\"mailto:a.haumer@haumer.at\">a.haumer@haumer.at</a>
</p>
<p>
  Dr. Christian Kral<br>
  <a href=\"http://christiankral.net/\">Electric Machines, Drives and Systems</a><br>
  A-1060 Vienna, Austria<br>
  email: <a href=\"mailto:dr.christian.kral@gmail.com\">dr.christian.kral@gmail.com</a>
</p>
  </dd>
</dl>
<p>
Copyright &copy; 1998-2015, Modelica Association, Anton Haumer, Christian Kral and Austrian Institute of Technology, AIT.
</p>
<p>
<i>This Modelica package is <u>free</u> software and the use is completely at <u>your own risk</u>; it can be redistributed and/or modified under the terms of the Modelica License 2. For license conditions (including the disclaimer of warranty) see <a href=\"modelica://Modelica.UsersGuide.ModelicaLicense2\">Modelica.UsersGuide.ModelicaLicense2</a> or visit <a href=\"https://www.modelica.org/licenses/ModelicaLicense2\"> https://www.modelica.org/licenses/ModelicaLicense2</a>.</i>
</p>
</HTML>", revisions = "<HTML>
  <ul>
  <li> v1.00 2005/02/01 Anton Haumer<br>
       first stable official release</li>
  </ul>
</html>"));
      end Sensors;

      package Sources "Ideal fluid sources, e.g., ambient, volume flow"
        extends Modelica.Icons.SourcesPackage;

        model Ambient "Ambient with constant properties"
          extends Interfaces.Partials.Ambient;
          parameter Boolean usePressureInput = false "Enable / disable pressure input" annotation(
            Evaluate = true);
          parameter Modelica.SIunits.Pressure constantAmbientPressure(start = 0) "Ambient pressure" annotation(
            Dialog(enable = not usePressureInput));
          parameter Boolean useTemperatureInput = false "Enable / disable temperature input" annotation(
            Evaluate = true);
          parameter Modelica.SIunits.Temperature constantAmbientTemperature(start = 293.15, displayUnit = "degC") "Ambient temperature" annotation(
            Dialog(enable = not useTemperatureInput));
          Modelica.Blocks.Interfaces.RealInput ambientPressure = pAmbient if usePressureInput annotation(
            Placement(transformation(extent = {{110, 60}, {90, 80}}, rotation = 0)));
          Modelica.Blocks.Interfaces.RealInput ambientTemperature = TAmbient if useTemperatureInput annotation(
            Placement(transformation(extent = {{110, -60}, {90, -80}}, rotation = 0)));
        protected
          Modelica.SIunits.Pressure pAmbient;
          Modelica.SIunits.Temperature TAmbient;
        equation
          if not usePressureInput then
            pAmbient = constantAmbientPressure;
          end if;
          if not useTemperatureInput then
            TAmbient = constantAmbientTemperature;
          end if;
          flowPort.p = pAmbient;
          T = TAmbient;
          annotation(
            Documentation(info = "<HTML>
(Infinite) ambient with constant pressure and temperature.<br>
Thermodynamic equations are defined by Partials.Ambient.
</HTML>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{20, 80}, {80, 20}}, lineColor = {0, 0, 0}, textString = "p"), Text(extent = {{20, -20}, {80, -80}}, lineColor = {0, 0, 0}, textString = "T")}));
        end Ambient;

        model AbsolutePressure "Defines absolute pressure level"
          parameter FluidHeatFlow.Media.Medium medium = FluidHeatFlow.Media.Medium() "medium" annotation(
            choicesAllMatching = true);
          parameter Modelica.SIunits.Pressure p(start = 0) "Pressure ground";
          Interfaces.FlowPort_a flowPort(final medium = medium) annotation(
            Placement(transformation(extent = {{-110, -10}, {-90, 10}}, rotation = 0)));
        equation
// defining pressure
          flowPort.p = p;
// no energy exchange; no mass flow by default
          flowPort.H_flow = 0;
          annotation(
            Documentation(info = "<HTML>
AbsolutePressure to define pressure level of a closed cooling cycle.
Coolant's mass flow, temperature and enthalpy flow are not affected.<br>
</HTML>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-150, 150}, {150, 90}}, lineColor = {0, 0, 255}, textString = "%name"), Ellipse(extent = {{-90, 90}, {90, -90}}, lineColor = {255, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid)}));
        end AbsolutePressure;

        model VolumeFlow "Enforces constant volume flow"
          extends Interfaces.Partials.TwoPort(final tapT = 1);
          parameter Boolean useVolumeFlowInput = false "Enable / disable volume flow input" annotation(
            Evaluate = true);
          parameter Modelica.SIunits.VolumeFlowRate constantVolumeFlow(start = 1) "Volume flow rate" annotation(
            Dialog(enable = not useVolumeFlowInput));
          Modelica.Blocks.Interfaces.RealInput volumeFlow = internalVolumeFlow if useVolumeFlowInput annotation(
            Placement(transformation(origin = {0, 100}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
        protected
          Modelica.SIunits.VolumeFlowRate internalVolumeFlow;
        equation
          if not useVolumeFlowInput then
            internalVolumeFlow = constantVolumeFlow;
          end if;
          Q_flow = 0;
          V_flow = internalVolumeFlow;
          annotation(
            Documentation(info = "<HTML>
Fan resp. pump with constant volume flow rate. Pressure increase is the response of the whole system.
Coolant's temperature and enthalpy flow are not affected.<br>
Setting parameter m (mass of medium within fan/pump) to zero
leads to neglection of temperature transient cv*m*der(T).<br>
Thermodynamic equations are defined by Partials.TwoPort.
</HTML>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-90, 90}, {90, -90}}, lineColor = {255, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-150, -90}, {150, -150}}, lineColor = {0, 0, 255}, textString = "%name"), Polygon(points = {{-60, 68}, {90, 10}, {90, -10}, {-60, -68}, {-60, 68}}, lineColor = {255, 0, 0}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-40, 20}, {0, -20}}, lineColor = {0, 0, 0}, textString = "V")}));
        end VolumeFlow;

        model PressureIncrease "Enforces constant pressure increase"
          extends Interfaces.Partials.TwoPort(final tapT = 1);
          parameter Boolean usePressureIncreaseInput = false "Enable / disable pressure increase input" annotation(
            Evaluate = true);
          parameter Modelica.SIunits.Pressure constantPressureIncrease(start = 1) "Pressure increase" annotation(
            Dialog(enable = not usePressureIncreaseInput));
          Modelica.Blocks.Interfaces.RealInput pressureIncrease = internalPressureIncrease if usePressureIncreaseInput annotation(
            Placement(transformation(origin = {0, 100}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
        protected
          Modelica.SIunits.Pressure internalPressureIncrease;
        equation
          if not usePressureIncreaseInput then
            internalPressureIncrease = constantPressureIncrease;
          end if;
          Q_flow = 0;
          dp = -internalPressureIncrease;
          annotation(
            Documentation(info = "<HTML>
Fan resp. pump with constant pressure increase. Mass resp. volume flow is the response of the whole system.
Coolant's temperature and enthalpy flow are not affected.<br>
Setting parameter m (mass of medium within fan/pump) to zero
leads to neglection of temperature transient cv*m*der(T).<br>
Thermodynamic equations are defined by Partials.TwoPort.
</HTML>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-90, 90}, {90, -90}}, lineColor = {255, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-150, -90}, {150, -150}}, lineColor = {0, 0, 255}, textString = "%name"), Polygon(points = {{-60, 68}, {90, 10}, {90, -10}, {-60, -68}, {-60, 68}}, lineColor = {0, 0, 255}, fillColor = {255, 0, 0}, fillPattern = FillPattern.Solid), Text(extent = {{-40, 20}, {0, -20}}, lineColor = {0, 0, 0}, textString = "dp")}));
        end PressureIncrease;

        model IdealPump "Model of an ideal pump"
          extends Interfaces.Partials.TwoPort(final tapT = 1);
          parameter Modelica.SIunits.AngularVelocity wNominal(start = 1, displayUnit = "1/min") "Nominal speed" annotation(
            Dialog(group = "Pump characteristic"));
          parameter Modelica.SIunits.Pressure dp0(start = 2) "Max. pressure increase @ V_flow=0" annotation(
            Dialog(group = "Pump characteristic"));
          parameter Modelica.SIunits.VolumeFlowRate V_flow0(start = 2) "Max. volume flow rate @ dp=0" annotation(
            Dialog(group = "Pump characteristic"));
          Modelica.SIunits.AngularVelocity w = der(flange_a.phi) "Speed";
        protected
          Modelica.SIunits.Pressure dp1;
          Modelica.SIunits.VolumeFlowRate V_flow1;
        public
          Modelica.Mechanics.Rotational.Interfaces.Flange_a flange_a annotation(
            Placement(transformation(extent = {{-10, -110}, {10, -90}}, rotation = 0)));
        equation
// pump characteristic
          dp1 = dp0 * sign(w / wNominal) * (w / wNominal) ^ 2;
          V_flow1 = V_flow0 * (w / wNominal);
          if noEvent(abs(w) < Modelica.Constants.small) then
            dp = 0;
            flange_a.tau = 0;
          else
            dp = -dp1 * (1 - noEvent(abs(V_flow / V_flow1)));
            flange_a.tau * w = -dp * V_flow;
          end if;
// no energy exchange with medium
          Q_flow = 0;
          annotation(
            Documentation(info = "<HTML>
Simple fan resp. pump where characteristic is dependent on shaft's speed, <br>
torque * speed = pressure increase * volume flow (without losses)<br>
Pressure increase versus volume flow is defined by a linear function,
from dp0(V_flow=0) to V_flow0(dp=0).<br>
The axis intersections vary with speed as follows:
<ul>
<li>dp prop. speed^2</li>
<li>V_flow prop. speed</li>
</ul>
Coolant's temperature and enthalpy flow are not affected.<br>
Setting parameter m (mass of medium within fan/pump) to zero
leads to neglection of temperature transient cv*m*der(T).<br>
Thermodynamic equations are defined by Partials.TwoPort.
</HTML>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-90, 90}, {90, -90}}, lineColor = {255, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-150, 150}, {150, 90}}, lineColor = {0, 0, 255}, textString = "%name"), Rectangle(extent = {{-10, -40}, {10, -100}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.VerticalCylinder, fillColor = {175, 175, 175}), Polygon(points = {{-60, 68}, {90, 10}, {90, -10}, {-60, -68}, {-60, 68}}, lineColor = {0, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {0, 0, 255})}));
        end IdealPump;
        annotation(
          Documentation(info = "<HTML>
This package contains different types of sources:
<ul>
<li>Ambient with constant or prescribed pressure and temperature</li>
<li>AbsolutePressure to define pressure level of a closed cooling cycle.</li>
<li>Constant and prescribed volume flow</li>
<li>Constant and prescribed pressure increase</li>
<li>Simple pump with mechanical flange</li>
</ul>
Thermodynamic equations are defined in partial models (package Interfaces.Partials).<br>
All fans / pumps are considered without losses, they do not change enthalpy flow.<br>
<dl>
  <dt><b>Main Authors:</b></dt>
  <dd>
  <p>
  <a href=\"http://www.haumer.at/\">Anton Haumer</a><br>
  Technical Consulting &amp; Electrical Engineering<br>
  A-3423 St.Andrae-Woerdern, Austria<br>
  email: <a href=\"mailto:a.haumer@haumer.at\">a.haumer@haumer.at</a>
</p>
<p>
  Dr. Christian Kral<br>
  <a href=\"http://christiankral.net/\">Electric Machines, Drives and Systems</a><br>
  A-1060 Vienna, Austria<br>
  email: <a href=\"mailto:dr.christian.kral@gmail.com\">dr.christian.kral@gmail.com</a>
</p>
  </dd>
</dl>
<p>
Copyright &copy; 1998-2015, Modelica Association, Anton Haumer, Christian Kral and Austrian Institute of Technology, AIT.
</p>
<p>
<i>This Modelica package is <u>free</u> software and the use is completely at <u>your own risk</u>; it can be redistributed and/or modified under the terms of the Modelica License 2. For license conditions (including the disclaimer of warranty) see <a href=\"modelica://Modelica.UsersGuide.ModelicaLicense2\">Modelica.UsersGuide.ModelicaLicense2</a> or visit <a href=\"https://www.modelica.org/licenses/ModelicaLicense2\"> https://www.modelica.org/licenses/ModelicaLicense2</a>.</i>
</p>
</HTML>", revisions = "<HTML>
  <ul>
  <li> v1.00 2005/02/01 Anton Haumer<br>
       first stable official release</li>
  <li> v1.20 Beta 2005/02/18 Anton Haumer<br>
       <i>new model: IdealPump</i></li>
  </ul>
</html>"));
      end Sources;

      package Interfaces "Connectors and partial models"
        extends Modelica.Icons.InterfacesPackage;

        connector FlowPort "Connector flow port"
          parameter FluidHeatFlow.Media.Medium medium "Medium in the connector";
          Modelica.SIunits.Pressure p;
          flow Modelica.SIunits.MassFlowRate m_flow;
          Modelica.SIunits.SpecificEnthalpy h;
          flow Modelica.SIunits.EnthalpyFlowRate H_flow;
          annotation(
            Documentation(info = "<HTML>
Basic definition of the connector.<br>
<b>Variables:</b>
<ul>
<li>Pressure p</li>
<li>flow MassFlowRate m_flow</li>
<li>Specific Enthalpy h</li>
<li>flow EnthaplyFlowRate H_flow</li>
</ul>
If ports with different media are connected, the simulation is asserted due to the check of parameter.
</HTML>"));
        end FlowPort;

        connector FlowPort_a "Filled flow port (used upstream)"
          extends FlowPort;
          annotation(
            Documentation(info = "<HTML>
Same as FlowPort, but icon allows to differentiate direction of flow.
</HTML>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, 100}, {100, -100}}, lineColor = {255, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-98, 98}, {98, -98}}, lineColor = {0, 0, 255}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid)}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-50, 50}, {50, -50}}, lineColor = {255, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-48, 48}, {48, -48}}, lineColor = {0, 0, 255}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-100, 110}, {100, 50}}, lineColor = {0, 0, 255}, textString = "%name")}));
        end FlowPort_a;

        connector FlowPort_b "Hollow flow port (used downstream)"
          extends FlowPort;
          annotation(
            Documentation(info = "<HTML>
Same as FlowPort, but icon allows to differentiate direction of flow.
</HTML>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, 100}, {100, -100}}, lineColor = {255, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-98, 98}, {98, -98}}, lineColor = {0, 0, 255})}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-50, 50}, {50, -50}}, lineColor = {255, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-48, 48}, {48, -48}}, lineColor = {0, 0, 255}), Text(extent = {{-100, 110}, {100, 50}}, lineColor = {0, 0, 255}, textString = "%name")}));
        end FlowPort_b;

        package Partials "Partial models"
          extends Modelica.Icons.Package;

          partial model SimpleFriction "Simple friction model"
            parameter Modelica.SIunits.VolumeFlowRate V_flowLaminar(min = Modelica.Constants.small, start = 0.1) "Laminar volume flow" annotation(
              Dialog(group = "Simple Friction"));
            parameter Modelica.SIunits.Pressure dpLaminar(start = 0.1) "Laminar pressure drop" annotation(
              Dialog(group = "Simple Friction"));
            parameter Modelica.SIunits.VolumeFlowRate V_flowNominal(start = 1) "Nominal volume flow" annotation(
              Dialog(group = "Simple Friction"));
            parameter Modelica.SIunits.Pressure dpNominal(start = 1) "Nominal pressure drop" annotation(
              Dialog(group = "Simple Friction"));
            parameter Real frictionLoss(min = 0, max = 1) = 0 "Part of friction losses fed to medium" annotation(
              Dialog(group = "Simple Friction"));
            Modelica.SIunits.Pressure pressureDrop;
            Modelica.SIunits.VolumeFlowRate volumeFlow;
            Modelica.SIunits.Power Q_friction;
          protected
            parameter Modelica.SIunits.Pressure dpNomMin = dpLaminar / V_flowLaminar * V_flowNominal;
            parameter Real k(final unit = "Pa.s2/m6", fixed = false);
          initial algorithm
            assert(V_flowNominal > V_flowLaminar, "SimpleFriction: V_flowNominal has to be > V_flowLaminar!");
            assert(dpNominal >= dpNomMin, "SimpleFriction: dpNominal has to be > dpLaminar/V_flowLaminar*V_flowNominal!");
            k := (dpNominal - dpNomMin) / (V_flowNominal - V_flowLaminar) ^ 2;
          equation
            if volumeFlow > (+V_flowLaminar) then
              pressureDrop = (+dpLaminar / V_flowLaminar * volumeFlow) + k * (volumeFlow - V_flowLaminar) ^ 2;
            elseif volumeFlow < (-V_flowLaminar) then
              pressureDrop = (+dpLaminar / V_flowLaminar * volumeFlow) - k * (volumeFlow + V_flowLaminar) ^ 2;
            else
              pressureDrop = dpLaminar / V_flowLaminar * volumeFlow;
            end if;
            Q_friction = frictionLoss * volumeFlow * pressureDrop;
            annotation(
              Documentation(info = "<HTML>
Definition of relationship between pressure drop and volume flow rate:<br>
-V_flowLaminar &lt; VolumeFlow &lt; +V_flowLaminar: laminar i.e., linear dependency of pressure drop on volume flow.<br>
-V_flowLaminar &gt; VolumeFlow or VolumeFlow &lt; +V_flowLaminar: turbulent i.e., quadratic dependency of pressure drop on volume flow.<br>
Linear and quadratic dependency are coupled smoothly at V_flowLaminar / dpLaminar.<br>
Quadratic dependency is defined by nominal volume flow and pressure drop (V_flowNominal / dpNominal).<br>
See also sketch at diagram layer.
</HTML>"),
              Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-80, 0}, {80, 0}}, color = {0, 0, 255}), Line(points = {{0, 80}, {0, -80}}, color = {0, 0, 255}), Line(points = {{-40, -20}, {40, 20}}, color = {0, 0, 255}), Line(points = {{40, 20}, {60, 40}, {70, 60}, {74, 80}}, color = {0, 0, 255}), Line(points = {{-40, -20}, {-60, -40}, {-70, -60}, {-74, -80}}, color = {0, 0, 255}), Line(points = {{40, 20}, {40, 0}}, color = {0, 0, 255}), Line(points = {{60, 40}, {60, 0}}, color = {0, 0, 255}), Line(points = {{40, 20}, {0, 20}}, color = {0, 0, 255}), Line(points = {{60, 40}, {0, 40}}, color = {0, 0, 255}), Text(extent = {{18, 0}, {48, -20}}, lineColor = {0, 0, 255}, textString = "V_flowLaminar"), Text(extent = {{50, 0}, {80, -20}}, lineColor = {0, 0, 255}, textString = "V_flowNominal"), Text(extent = {{-30, 30}, {-4, 10}}, lineColor = {0, 0, 255}, textString = "dpLaminar"), Text(extent = {{-30, 50}, {-4, 30}}, lineColor = {0, 0, 255}, textString = "dpNominal"), Text(extent = {{0, 20}, {30, 0}}, lineColor = {0, 0, 255}, textString = "dp ~ V_flow"), Text(extent = {{30, 60}, {60, 40}}, lineColor = {0, 0, 255}, textString = "dp ~ V_flow^2")}));
          end SimpleFriction;

          partial model TwoPort "Partial model of two port"
            parameter FluidHeatFlow.Media.Medium medium = FluidHeatFlow.Media.Medium() "Medium in the component" annotation(
              choicesAllMatching = true);
            parameter Modelica.SIunits.Mass m(start = 1) "Mass of medium";
            parameter Modelica.SIunits.Temperature T0(start = 293.15, displayUnit = "degC") "Initial temperature of medium" annotation(
              Dialog(enable = m > Modelica.Constants.small));
            parameter Boolean T0fixed = false "Initial temperature guess value or fixed" annotation(
              choices(checkBox = true),
              Dialog(enable = m > Modelica.Constants.small));
            parameter Real tapT(final min = 0, final max = 1) = 1 "Defines temperature of heatPort between inlet and outlet temperature";
            Modelica.SIunits.Pressure dp "Pressure drop a->b";
            Modelica.SIunits.VolumeFlowRate V_flow "Volume flow a->b";
            Modelica.SIunits.HeatFlowRate Q_flow "Heat exchange with ambient";
            output Modelica.SIunits.Temperature T(start = T0, fixed = T0fixed) "Outlet temperature of medium";
            output Modelica.SIunits.Temperature T_a "Temperature at flowPort_a";
            output Modelica.SIunits.Temperature T_b "Temperature at flowPort_b";
            output Modelica.SIunits.TemperatureDifference dT "Temperature increase of coolant in flow direction";
          protected
            Modelica.SIunits.SpecificEnthalpy h "Medium's specific enthalpy";
            Modelica.SIunits.Temperature T_q "Temperature relevant for heat exchange with ambient";
          public
            Interfaces.FlowPort_a flowPort_a(final medium = medium) annotation(
              Placement(transformation(extent = {{-110, -10}, {-90, 10}}, rotation = 0)));
            Interfaces.FlowPort_b flowPort_b(final medium = medium) annotation(
              Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
          equation
            dp = flowPort_a.p - flowPort_b.p;
            V_flow = flowPort_a.m_flow / medium.rho;
            T_a = flowPort_a.h / medium.cp;
            T_b = flowPort_b.h / medium.cp;
            dT = if noEvent(V_flow >= 0) then T - T_a else T_b - T;
            h = medium.cp * T;
            T_q = T - noEvent(sign(V_flow)) * (1 - tapT) * dT;
// mass balance
            flowPort_a.m_flow + flowPort_b.m_flow = 0;
// energy balance
            if m > Modelica.Constants.small then
              flowPort_a.H_flow + flowPort_b.H_flow + Q_flow = m * medium.cv * der(T);
            else
              flowPort_a.H_flow + flowPort_b.H_flow + Q_flow = 0;
            end if;
// massflow a->b mixing rule at a, energy flow at b defined by medium's temperature
// massflow b->a mixing rule at b, energy flow at a defined by medium's temperature
            flowPort_a.H_flow = semiLinear(flowPort_a.m_flow, flowPort_a.h, h);
            flowPort_b.H_flow = semiLinear(flowPort_b.m_flow, flowPort_b.h, h);
            annotation(
              Documentation(info = "<HTML>
Partial model with two flowPorts.<br>
Possible heat exchange with the ambient is defined by Q_flow; setting this = 0 means no energy exchange.<br>
Setting parameter m (mass of medium within pipe) to zero
leads to neglection of temperature transient cv*m*der(T).<br>
Mixing rule is applied.<br>
Parameter 0 &lt; tapT &lt; 1 defines temperature of heatPort between medium's inlet and outlet temperature.
</HTML>"));
          end TwoPort;

          partial model Ambient "Partial model of ambient"
            parameter FluidHeatFlow.Media.Medium medium = FluidHeatFlow.Media.Medium() "Ambient medium" annotation(
              choicesAllMatching = true);
            output Modelica.SIunits.Temperature T "Outlet temperature of medium";
            output Modelica.SIunits.Temperature T_port "Temperature at flowPort_a";
          protected
            Modelica.SIunits.SpecificEnthalpy h;
          public
            Interfaces.FlowPort_a flowPort(final medium = medium) annotation(
              Placement(transformation(extent = {{-110, -10}, {-90, 10}}, rotation = 0)));
          equation
            T_port = flowPort.h / medium.cp;
            h = medium.cp * T;
// massflow -> ambient: mixing rule
// massflow <- ambient: energy flow defined by ambient's temperature
            flowPort.H_flow = semiLinear(flowPort.m_flow, flowPort.h, h);
            annotation(
              Documentation(info = "<HTML>
<p>
Partial model of (Infinite) ambient, defines pressure and temperature.
</p>
</HTML>"),
              Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-90, 90}, {90, -90}}, lineColor = {255, 0, 0}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-150, 150}, {150, 90}}, lineColor = {0, 0, 255}, textString = "%name")}));
          end Ambient;

          partial model AbsoluteSensor "Partial model of absolute sensor"
            extends Modelica.Icons.RotationalSensor;
            parameter FluidHeatFlow.Media.Medium medium = FluidHeatFlow.Media.Medium() "Sensor's medium" annotation(
              choicesAllMatching = true);
            Interfaces.FlowPort_a flowPort(final medium = medium) annotation(
              Placement(transformation(extent = {{-110, -10}, {-90, 10}}, rotation = 0)));
            Modelica.Blocks.Interfaces.RealOutput y annotation(
              Placement(transformation(extent = {{100, -10}, {120, 10}}, rotation = 0)));
          equation
// no mass exchange
            flowPort.m_flow = 0;
// no energy exchange
            flowPort.H_flow = 0;
            annotation(
              Documentation(info = "<HTML>
Partial model for an absolute sensor (pressure/temperature).<br>
Pressure, mass flow, temperature and enthalpy flow of medium are not affected.
</HTML>"),
              Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-70, 0}, {-90, 0}}, color = {0, 0, 0}), Line(points = {{70, 0}, {100, 0}}), Text(extent = {{-150, 130}, {150, 70}}, lineColor = {0, 0, 255}, textString = "%name")}));
          end AbsoluteSensor;

          partial model RelativeSensor "Partial model of relative sensor"
            extends Modelica.Icons.RotationalSensor;
            parameter FluidHeatFlow.Media.Medium medium = FluidHeatFlow.Media.Medium() "Sensor's medium" annotation(
              choicesAllMatching = true);
            Interfaces.FlowPort_a flowPort_a(final medium = medium) annotation(
              Placement(transformation(extent = {{-110, -10}, {-90, 10}}, rotation = 0)));
            Interfaces.FlowPort_b flowPort_b(final medium = medium) annotation(
              Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
            Modelica.Blocks.Interfaces.RealOutput y annotation(
              Placement(transformation(origin = {0, -110}, extent = {{10, -10}, {-10, 10}}, rotation = 90)));
          equation
// no mass exchange
            flowPort_a.m_flow = 0;
            flowPort_b.m_flow = 0;
// no energy exchange
            flowPort_a.H_flow = 0;
            flowPort_b.H_flow = 0;
            annotation(
              Documentation(info = "<HTML>
Partial model for a relative sensor (pressure drop/temperature difference).<br>
Pressure, mass flow, temperature and enthalpy flow of medium are not affected.
</HTML>"),
              Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-70, 0}, {-90, 0}}, color = {0, 0, 0}), Line(points = {{70, 0}, {90, 0}}, color = {0, 0, 0}), Line(points = {{0, -100}, {0, -70}}), Text(extent = {{-150, 130}, {150, 70}}, lineColor = {0, 0, 255}, textString = "%name")}));
          end RelativeSensor;

          partial model FlowSensor "Partial model of flow sensor"
            extends Modelica.Icons.RotationalSensor;
            extends TwoPort(final m = 0, final T0 = 0, final tapT = 1);
            Modelica.Blocks.Interfaces.RealOutput y annotation(
              Placement(transformation(origin = {0, -110}, extent = {{10, -10}, {-10, 10}}, rotation = 90)));
          equation
// no pressure drop
            dp = 0;
// no energy exchange
            Q_flow = 0;
            annotation(
              Documentation(info = "<HTML>
Partial model for a flow sensor (mass flow/heat flow).<br>
Pressure, mass flow, temperature and enthalpy flow of medium are not affected, but mixing rule is applied.
</HTML>"),
              Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-70, 0}, {-90, 0}}, color = {0, 0, 0}), Line(points = {{70, 0}, {90, 0}}, color = {0, 0, 0}), Line(points = {{0, -100}, {0, -70}}), Text(extent = {{-150, 130}, {150, 70}}, lineColor = {0, 0, 255}, textString = "%name")}));
          end FlowSensor;
          annotation(
            Documentation(info = "<HTML>
This package contains partial models, defining in a very compact way the basic thermodynamic equations used by the different components.<br>
<dl>
  <dt><b>Main Authors:</b></dt>
  <dd>
  <p>
  <a href=\"http://www.haumer.at/\">Anton Haumer</a><br>
  Technical Consulting &amp; Electrical Engineering<br>
  A-3423 St.Andrae-Woerdern, Austria<br>
  email: <a href=\"mailto:a.haumer@haumer.at\">a.haumer@haumer.at</a>
</p>
<p>
  Dr. Christian Kral<br>
  <a href=\"http://christiankral.net/\">Electric Machines, Drives and Systems</a><br>
  A-1060 Vienna, Austria<br>
  email: <a href=\"mailto:dr.christian.kral@gmail.com\">dr.christian.kral@gmail.com</a>
</p>
  </dd>
</dl>
<p>
Copyright &copy; 1998-2015, Modelica Association, Anton Haumer, Christian Kral and Austrian Institute of Technology, AIT.
</p>
<p>
<i>This Modelica package is <u>free</u> software and the use is completely at <u>your own risk</u>; it can be redistributed and/or modified under the terms of the Modelica License 2. For license conditions (including the disclaimer of warranty) see <a href=\"modelica://Modelica.UsersGuide.ModelicaLicense2\">Modelica.UsersGuide.ModelicaLicense2</a> or visit <a href=\"https://www.modelica.org/licenses/ModelicaLicense2\"> https://www.modelica.org/licenses/ModelicaLicense2</a>.</i>
</p>
</HTML>", revisions = "<HTML>
  <ul>
  <li> v1.00 2005/02/01 Anton Haumer<br>
       first stable official release</li>
  <li> v1.10 2005/02/15 Anton Haumer<br>
       moved Partials into Interfaces</li>
  <li> v1.11 2005/02/18 Anton Haumer<br>
       corrected usage of cv and cp</li>
  <li> v1.30 Beta 2005/06/02 Anton Haumer<br>
       friction losses are fed to medium</li>
  <li> v1.31 Beta 2005/06/04 Anton Haumer<br>
       searching solution for problems @ m_flow=0</li>
  <li> v1.33 Beta 2005/06/07 Anton Haumer<br>
       corrected usage of simpleFlow</li>
  <li> v1.43 Beta 2005/06/20 Anton Haumer<br>
       Test of mixing / semiLinear</li>
  <li> v1.50 2005/09/07 Anton Haumer<br>
       semiLinear works fine<br>
       removed test-version of semiLinear</li>
  <li> v1.60 2007/01/23 Anton Haumer<br>
       new parameter tapT defining Temperature of heatPort</li>
  </ul>
</html>"));
        end Partials;
        annotation(
          Documentation(info = "<HTML>
This package contains connectors and partial models:
<ul>
<li>FlowPort: basic definition of the connector.</li>
<li>FlowPort_a &amp; FlowPort_b: same as FlowPort with different icons to differentiate direction of flow</li>
<li>package Partials (defining basic thermodynamic equations)</li>
</ul>
<dl>
  <dt><b>Main Authors:</b></dt>
  <dd>
  <p>
  <a href=\"http://www.haumer.at/\">Anton Haumer</a><br>
  Technical Consulting &amp; Electrical Engineering<br>
  A-3423 St.Andrae-Woerdern, Austria<br>
  email: <a href=\"mailto:a.haumer@haumer.at\">a.haumer@haumer.at</a>
</p>
<p>
  Dr. Christian Kral<br>
  <a href=\"http://christiankral.net/\">Electric Machines, Drives and Systems</a><br>
  A-1060 Vienna, Austria<br>
  email: <a href=\"mailto:dr.christian.kral@gmail.com\">dr.christian.kral@gmail.com</a>
</p>
  </dd>
</dl>
<p>
Copyright &copy; 1998-2015, Modelica Association, Anton Haumer, Christian Kral and Austrian Institute of Technology, AIT.
</p>
<p>
<i>This Modelica package is <u>free</u> software and the use is completely at <u>your own risk</u>; it can be redistributed and/or modified under the terms of the Modelica License 2. For license conditions (including the disclaimer of warranty) see <a href=\"modelica://Modelica.UsersGuide.ModelicaLicense2\">Modelica.UsersGuide.ModelicaLicense2</a> or visit <a href=\"https://www.modelica.org/licenses/ModelicaLicense2\"> https://www.modelica.org/licenses/ModelicaLicense2</a>.</i>
</p>
</HTML>", revisions = "<HTML>
  <ul>
  <li> v1.00 2005/02/01 Anton Haumer<br>
       first stable official release</li>
  <li> v1.10 2005/02/15 Anton Haumer<br>
       moved Partials into Interfaces</li>
  <li> v1.11 2005/02/18 Anton Haumer<br>
       corrected usage of cv and cp</li>
  <li> v1.30 Beta 2005/06/02 Anton Haumer<br>
       friction losses are fed to medium</li>
  <li> v1.33 Beta 2005/06/07 Anton Haumer<br>
       corrected usage of simpleFlow</li>
  <li> v1.43 Beta 2005/06/20 Anton Haumer<br>
       Test of mixing / semiLinear</li>
  <li> v1.50 2005/09/07 Anton Haumer<br>
       semiLinear works fine</li>
  </ul>
</html>"));
      end Interfaces;
      annotation(
        Documentation(info = "<HTML>
This package contains very simple-to-use components to model coolant flows as needed to simulate cooling e.g., of electric machines:
<ul>
<li>Components: components like different types of pipe models</li>
<li>Examples: some test examples</li>
<li>Interfaces: definition of connectors and partial models
(containing the core thermodynamic equations)</li>
<li>Media: definition of media properties</li>
<li>Sensors: various sensors for pressure, temperature, volume and enthalpy flow</li>
<li>Sources: various flow sources</li>
</ul>
<b>Variables used in connectors:</b>
<ul>
<li>Pressure p</li>
<li>flow MassFlowRate m_flow</li>
<li>SpecificEnthalpy h</li>
<li>flow EnthalpyFlowRate H_flow</li>
</ul>
EnthalpyFlowRate means the Enthalpy = cp<sub>constant</sub> * m * T that is carried by the medium's flow.<br><br>
<b>Limitations and assumptions:</b>
<ul>
<li>Splitting and mixing of coolant flows (media with the same cp) is possible.</li>
<li>Reversing the direction of flow is possible.</li>
<li>The medium is considered to be incompressible.</li>
<li>No mixtures of media is taken into consideration.</li>
<li>The medium may not change its phase.</li>
<li>Medium properties are kept constant.</li>
<li>Pressure changes are only due to pressure drop and geodetic height difference rho*g*h (if h > 0).</li>
<li>A user-defined part (0..1) of the friction losses (V_flow*dp) are fed to the medium.</li>
<li><b>Note:</b> Connected flowPorts have the same temperature (mixing temperature)!<br>
Since mixing may occur, the outlet temperature may be different from the connector's temperature.<br>
Outlet temperature is defined by variable T of the corresponding component.</li>
</ul>
<b>Further development:</b>
<ul>
<li>Additional components like tanks (if needed)</li>
</ul>
<dl>
  <dt><b>Main Authors:</b></dt>
  <dd>
  <p>
  <a href=\"http://www.haumer.at/\">Anton Haumer</a><br>
  Technical Consulting &amp; Electrical Engineering<br>
  A-3423 St.Andrae-Woerdern, Austria<br>
  email: <a href=\"mailto:a.haumer@haumer.at\">a.haumer@haumer.at</a>
</p>
<p>
  Dr. Christian Kral<br>
  <a href=\"http://christiankral.net/\">Electric Machines, Drives and Systems</a><br>
  A-1060 Vienna, Austria<br>
  email: <a href=\"mailto:dr.christian.kral@gmail.com\">dr.christian.kral@gmail.com</a>
</p>
  </dd>
</dl>
<p>
Copyright &copy; 1998-2015, Modelica Association, Anton Haumer, Christian Kral and AIT.
</p>
<p>
<i>This Modelica package is <u>free</u> software and the use is completely at <u>your own risk</u>; it can be redistributed and/or modified under the terms of the Modelica License 2. For license conditions (including the disclaimer of warranty) see <a href=\"modelica://Modelica.UsersGuide.ModelicaLicense2\">Modelica.UsersGuide.ModelicaLicense2</a> or visit <a href=\"https://www.modelica.org/licenses/ModelicaLicense2\"> https://www.modelica.org/licenses/ModelicaLicense2</a>.</i>
</p>
</HTML>", revisions = "<HTML>
  <ul>
  <li> v1.00 2005/02/01 Anton Haumer<br>
       first stable official release</li>
  <li> v1.10 2005/02/15 Anton Haumer<br>
       reorganisation of the package</li>
  <li> v1.11 2005/02/18 Anton Haumer<br>
       corrected usage of cv and cp</li>
  <li> v1.20 Beta 2005/02/18 Anton Haumer<br>
       introduced geodetic height in Components.Pipes<br>
       <i>new models: Components.Valve, Sources.IdealPump</i></li>
  <li> v1.30 Beta 2005/06/02 Anton Haumer<br>
       friction losses are fed to medium</li>
  <li> v1.31 Beta 2005/06/04 Anton Haumer<br>
       <i>new example: PumpAndValve</i><br>
       <i>new example: PumpDropOut</i></li>
  <li> v1.33 Beta 2005/06/07 Anton Haumer<br>
       corrected usage of simpleFlow</li>
  <li> v1.40 2005/06/13 Anton Haumer<br>
       stable release</li>
  <li> v1.42 Beta 2005/06/18 Anton Haumer<br>
       <i>new test example: ParallelPumpDropOut</i></li>
  <li> v1.43 Beta 2005/06/20 Anton Haumer<br>
       Test of mixing / semiLinear<br>
       <i>new test example: OneMass</i><br>
       <i>new test example: TwoMass</i></li>
  <li> v1.50 2005/09/07 Anton Haumer<br>
       semiLinear works fine</li>
  <li> v1.60 2007/01/23 Anton Haumer<br>
       new parameter tapT defining Temperature of heatPort </li>
  <li> v1.6.1 2007/08/12 Anton Haumer<br>
       improved documentation<br>
       removed type TemperatureDifference since this is defined in Modelica.SIunits</li>
  <li> v1.6.2 2007/08/20 Anton Haumer<br>
       improved documentation</li>
  <li> v1.6.3 2007/08/21 Anton Haumer<br>
       improved documentation</li>
  <li> v1.6.4 2007/08/24 Anton Haumer<br>
       removed redeclare type SignalType</li>
  <li> v1.6.5 2007/08/26 Anton Haumer<br>
       fixed unit bug in SimpleFriction</li>
  <li> v1.6.6 2007/11/13 Anton Haumer<br>
       replaced all nonSIunits<br>
       some renaming to be more concise</li>
  <li> v1.6.7 2010/06/25 Christian Kral<br>
       changed company name to AIT</li>
  </ul>
</html>"),
        Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(origin = {11.533, 37.938}, lineColor = {0, 128, 255}, fillColor = {0, 128, 255}, fillPattern = FillPattern.Solid, points = {{-80, 10}, {-60, -10}, {-80, -30}, {-20, -30}, {0, -10}, {-20, 10}, {-80, 10}}), Polygon(origin = {11.533, 37.938}, lineColor = {255, 0, 0}, fillColor = {255, 0, 0}, fillPattern = FillPattern.Solid, points = {{-40, -90}, {-20, -70}, {0, -90}, {0, -50}, {-20, -30}, {-40, -50}, {-40, -90}}), Polygon(origin = {11.533, 37.938}, lineColor = {255, 128, 0}, fillColor = {255, 128, 0}, fillPattern = FillPattern.Solid, points = {{-20, 10}, {0, -10}, {-20, -30}, {40, -30}, {60, -10}, {40, 10}, {-20, 10}})}));
    end FluidHeatFlow;

    package HeatTransfer "Library of 1-dimensional heat transfer with lumped elements"
      extends Modelica.Icons.Package;

      package Examples "Example models to demonstrate the usage of package Modelica.Thermal.HeatTransfer"
        extends Modelica.Icons.ExamplesPackage;

        model TwoMasses "Simple conduction demo"
          extends Modelica.Icons.Example;
          parameter Modelica.SIunits.Temperature T_final_K(fixed = false) "Projected final temperature";
          HeatTransfer.Components.HeatCapacitor mass1(C = 15, T(start = 373.15, fixed = true)) annotation(
            Placement(transformation(extent = {{-100, 20}, {-40, 80}}, rotation = 0)));
          HeatTransfer.Components.HeatCapacitor mass2(C = 15, T(start = 273.15, fixed = true)) annotation(
            Placement(transformation(extent = {{40, 20}, {100, 80}}, rotation = 0)));
          HeatTransfer.Components.ThermalConductor conduction(G = 10) annotation(
            Placement(transformation(extent = {{-30, -20}, {30, 40}}, rotation = 0)));
          HeatTransfer.Celsius.TemperatureSensor Tsensor1 annotation(
            Placement(transformation(extent = {{-60, -80}, {-20, -40}}, rotation = 0)));
          HeatTransfer.Celsius.TemperatureSensor Tsensor2 annotation(
            Placement(transformation(extent = {{60, -80}, {20, -40}}, rotation = 0)));
        equation
          connect(mass1.port, conduction.port_a) annotation(
            Line(points = {{-70, 20}, {-70, 10}, {-30, 10}}, color = {191, 0, 0}));
          connect(conduction.port_b, mass2.port) annotation(
            Line(points = {{30, 10}, {70, 10}, {70, 20}}, color = {191, 0, 0}));
          connect(mass1.port, Tsensor1.port) annotation(
            Line(points = {{-70, 20}, {-70, -60}, {-60, -60}}, color = {191, 0, 0}));
          connect(mass2.port, Tsensor2.port) annotation(
            Line(points = {{70, 20}, {70, -60}, {60, -60}}, color = {191, 0, 0}));
        initial equation
          T_final_K = (mass1.T * mass1.C + mass2.T * mass2.C) / (mass1.C + mass2.C);
          annotation(
            Documentation(info = "<HTML>
<p>
This example demonstrates the thermal response of two masses connected by
a conducting element. The two masses have the same heat capacity but different
initial temperatures (T1=100 [degC], T2= 0 [degC]). The mass with the higher
temperature will cool off while the mass with the lower temperature heats up.
They will each asymptotically approach the calculated temperature <b>T_final_K</b>
(<b>T_final_degC</b>) that results from dividing the total initial energy in the system by the sum
of the heat capacities of each element.
</p>
<p>
Simulate for 5 s and plot the variables<br>
mass1.T, mass2.T, T_final_K or <br>
Tsensor1.T, Tsensor2.T, T_final_degC
</p>
</HTML>"),
            experiment(StopTime = 1.0, Interval = 0.001));
        end TwoMasses;

        model ControlledTemperature "Control temperature of a resistor"
          extends Modelica.Icons.Example;
          parameter Modelica.SIunits.Temperature TAmb(displayUnit = "degC") = 293.15 "Ambient Temperature";
          parameter Modelica.SIunits.TemperatureDifference TDif = 2 "Error in Temperature";
          output Modelica.SIunits.Temperature TRes(displayUnit = "degC") = heatingResistor.T_heatPort "Resulting Temperature";
          Modelica.Electrical.Analog.Basic.Ground ground annotation(
            Placement(transformation(extent = {{-100, -100}, {-80, -80}}, rotation = 0)));
          Modelica.Electrical.Analog.Sources.ConstantVoltage constantVoltage(V = 10) annotation(
            Placement(transformation(origin = {-90, -50}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          HeatTransfer.Components.HeatCapacitor heatCapacitor(C = 1, T(start = TAmb, fixed = true)) annotation(
            Placement(transformation(extent = {{0, -60}, {20, -80}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.HeatingResistor heatingResistor(R_ref = 10, T_ref = 293.15, alpha = 1 / 255) annotation(
            Placement(transformation(origin = {-30, -50}, extent = {{-10, 10}, {10, -10}}, rotation = 270)));
          HeatTransfer.Sources.FixedTemperature fixedTemperature(T = TAmb) annotation(
            Placement(transformation(extent = {{100, -60}, {80, -40}}, rotation = 0)));
          HeatTransfer.Celsius.TemperatureSensor temperatureSensor annotation(
            Placement(transformation(origin = {10, -30}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
          HeatTransfer.Components.ThermalConductor thermalConductor(G = 0.1) annotation(
            Placement(transformation(extent = {{40, -60}, {60, -40}}, rotation = 0)));
          Modelica.Electrical.Analog.Ideal.IdealOpeningSwitch idealSwitch annotation(
            Placement(transformation(extent = {{-70, -50}, {-50, -30}}, rotation = 0)));
          Modelica.Blocks.Sources.Ramp ramp(height = 25, duration = 6, offset = 25, startTime = 2) annotation(
            Placement(transformation(extent = {{40, 0}, {20, 20}}, rotation = 0)));
          Modelica.Blocks.Logical.OnOffController onOffController(bandwidth = TDif) annotation(
            Placement(transformation(extent = {{0, -20}, {-20, 0}}, rotation = 0)));
          Modelica.Blocks.Logical.Not logicalNot annotation(
            Placement(transformation(extent = {{-30, -20}, {-50, 0}}, rotation = 0)));
        equation
          connect(constantVoltage.n, heatingResistor.n) annotation(
            Line(points = {{-90, -60}, {-30, -60}}, color = {0, 0, 255}));
          connect(constantVoltage.n, ground.p) annotation(
            Line(points = {{-90, -60}, {-90, -80}}, color = {0, 0, 255}));
          connect(heatingResistor.heatPort, thermalConductor.port_a) annotation(
            Line(points = {{-20, -50}, {40, -50}}, color = {191, 0, 0}));
          connect(thermalConductor.port_b, fixedTemperature.port) annotation(
            Line(points = {{60, -50}, {80, -50}}, color = {191, 0, 0}));
          connect(heatingResistor.heatPort, temperatureSensor.port) annotation(
            Line(points = {{-20, -50}, {10, -50}, {10, -40}}, color = {191, 0, 0}));
          connect(heatingResistor.heatPort, heatCapacitor.port) annotation(
            Line(points = {{-20, -50}, {10, -50}, {10, -60}}, color = {191, 0, 0}));
          connect(constantVoltage.p, idealSwitch.p) annotation(
            Line(points = {{-90, -40}, {-70, -40}}, color = {0, 0, 255}));
          connect(idealSwitch.n, heatingResistor.p) annotation(
            Line(points = {{-50, -40}, {-30, -40}}, color = {0, 0, 255}));
          connect(ramp.y, onOffController.reference) annotation(
            Line(points = {{19, 10}, {10, 10}, {10, -4}, {2, -4}}, color = {0, 0, 127}));
          connect(temperatureSensor.T, onOffController.u) annotation(
            Line(points = {{10, -20}, {10, -16}, {2, -16}}, color = {0, 0, 127}));
          connect(onOffController.y, logicalNot.u) annotation(
            Line(points = {{-21, -10}, {-28, -10}}, color = {255, 0, 255}));
          connect(logicalNot.y, idealSwitch.control) annotation(
            Line(points = {{-51, -10}, {-60, -10}, {-60, -33}}, color = {255, 0, 255}));
          annotation(
            Documentation(info = "<HTML>
<P>
A constant voltage of 10 V is applied to a
temperature dependent resistor of 10*(1+(T-20C)/(235+20C)) Ohms,
whose losses v**2/r are dissipated via a
thermal conductance of 0.1 W/K to ambient temperature 20 degree C.
The resistor is assumed to have a thermal capacity of 1 J/K,
having ambient temperature at the beginning of the experiment.
The temperature of this heating resistor is held by an OnOff-controller
at reference temperature within a given bandwidth +/- 1 K
by switching on and off the voltage source.
The reference temperature starts at 25 degree C
and rises between t = 2 and 8 seconds linear to 50 degree C.
An appropriate simulating time would be 10 seconds.
</P>
</html>"),
            experiment(StopTime = 10, Interval = 0.001));
        end ControlledTemperature;

        model Motor "Second order thermal model of a motor"
          extends Modelica.Icons.Example;
          parameter Modelica.SIunits.Temperature TAmb(displayUnit = "degC") = 293.15 "Ambient temperature";
          Modelica.Blocks.Sources.CombiTimeTable lossTable(extrapolation = Modelica.Blocks.Types.Extrapolation.Periodic, table = [0, 100, 500; 360, 100, 500; 360, 1000, 500; 600, 1000, 500]) annotation(
            Placement(transformation(origin = {-40, 70}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          HeatTransfer.Sources.PrescribedHeatFlow windingLosses(T_ref = 368.15, alpha = 3.03E-3) annotation(
            Placement(transformation(origin = {-80, 10}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          HeatTransfer.Components.HeatCapacitor winding(C = 2500, T(start = TAmb, fixed = true)) annotation(
            Placement(transformation(extent = {{-90, -20}, {-70, -40}}, rotation = 0)));
          HeatTransfer.Celsius.TemperatureSensor Twinding annotation(
            Placement(transformation(origin = {-60, -50}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          HeatTransfer.Components.ThermalConductor winding2core(G = 10) annotation(
            Placement(transformation(extent = {{-50, -20}, {-30, 0}}, rotation = 0)));
          HeatTransfer.Sources.PrescribedHeatFlow coreLosses annotation(
            Placement(transformation(origin = {0, 10}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          HeatTransfer.Components.HeatCapacitor core(C = 25000, T(start = TAmb, fixed = true)) annotation(
            Placement(transformation(extent = {{-10, -20}, {10, -40}}, rotation = 0)));
          HeatTransfer.Celsius.TemperatureSensor Tcore annotation(
            Placement(transformation(origin = {-20, -50}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          Modelica.Blocks.Sources.Constant convectionConstant(k = 25) annotation(
            Placement(transformation(origin = {40, 30}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          HeatTransfer.Components.Convection convection annotation(
            Placement(transformation(extent = {{30, -20}, {50, 0}}, rotation = 0)));
          HeatTransfer.Sources.FixedTemperature environment(T = TAmb) annotation(
            Placement(transformation(origin = {80, -10}, extent = {{-10, -10}, {10, 10}}, rotation = 180)));
        equation
          connect(windingLosses.port, winding.port) annotation(
            Line(points = {{-80, 0}, {-80, -20}}, color = {191, 0, 0}));
          connect(coreLosses.port, core.port) annotation(
            Line(points = {{0, 0}, {0, -10}, {0, -20}}, color = {191, 0, 0}));
          connect(winding.port, winding2core.port_a) annotation(
            Line(points = {{-80, -20}, {-80, -10}, {-50, -10}}, color = {191, 0, 0}));
          connect(winding2core.port_b, core.port) annotation(
            Line(points = {{-30, -10}, {0, -10}, {0, -20}}, color = {191, 0, 0}));
          connect(winding.port, Twinding.port) annotation(
            Line(points = {{-80, -20}, {-80, -10}, {-60, -10}, {-60, -40}}, color = {191, 0, 0}));
          connect(core.port, Tcore.port) annotation(
            Line(points = {{0, -20}, {0, -10}, {-20, -10}, {-20, -40}}, color = {191, 0, 0}));
          connect(winding2core.port_b, convection.solid) annotation(
            Line(points = {{-30, -10}, {30, -10}}, color = {191, 0, 0}));
          connect(convection.fluid, environment.port) annotation(
            Line(points = {{50, -10}, {60, -10}, {70, -10}}, color = {191, 0, 0}));
          connect(convectionConstant.y, convection.Gc) annotation(
            Line(points = {{40, 19}, {40, 0}}, color = {0, 0, 127}));
          connect(lossTable.y[1], windingLosses.Q_flow) annotation(
            Line(points = {{-40, 59}, {-40, 40}, {-80, 40}, {-80, 20}}, color = {0, 0, 127}));
          connect(lossTable.y[2], coreLosses.Q_flow) annotation(
            Line(points = {{-40, 59}, {-40, 40}, {0, 40}, {0, 20}}, color = {0, 0, 127}));
          annotation(
            Documentation(info = "<HTML>
<p>
This example contains a simple second order thermal model of a motor.
The periodic power losses are described by table \"lossTable\":
</p>
<table>
<tr><td valign=\"top\">time</td><td valign=\"top\">winding losses</td><td valign=\"top\">core losses</td></tr>
<tr><td valign=\"top\">   0</td><td valign=\"top\">           100</td><td valign=\"top\">        500</td></tr>
<tr><td valign=\"top\"> 360</td><td valign=\"top\">           100</td><td valign=\"top\">        500</td></tr>
<tr><td valign=\"top\"> 360</td><td valign=\"top\">          1000</td><td valign=\"top\">        500</td></tr>
<tr><td valign=\"top\"> 600</td><td valign=\"top\">          1000</td><td valign=\"top\">        500</td></tr>
</table>
<p>
Since constant speed is assumed, the core losses keep constant
whereas the winding losses are low for 6 minutes (no-load) and high for 4 minutes (over load).
</p>
<p>
The winding losses are corrected by (1 + alpha*(T - T_ref)) because the winding's resistance is temperature dependent whereas the core losses are kept constant (alpha = 0).
</p>
<p>
The power dissipation to the environment is approximated by heat flow through
a thermal conductance between winding and core,
partially storage of the heat in the winding's heat capacity
as well as the core's heat capacity and finally by forced convection to the environment.<br>
Since constant speed is assumed, the convective conductance keeps constant.<br>
Using Modelica.Thermal.FluidHeatFlow it would be possible to model the coolant air flow, too
(instead of simple dissipation to a constant ambient's temperature).
</p>
<p>
Simulate for 7200 s; plot Twinding.T and Tcore.T.
</p>
</HTML>"),
            experiment(StopTime = 7200, Interval = 0.01));
        end Motor;
        annotation(
          Documentation(info = "<html>

</html>"));
      end Examples;

      package Components "Lumped thermal components"
        extends Modelica.Icons.Package;

        model HeatCapacitor "Lumped thermal element storing heat"
          parameter Modelica.SIunits.HeatCapacity C "Heat capacity of element (= cp*m)";
          Modelica.SIunits.Temperature T(start = 293.15, displayUnit = "degC") "Temperature of element";
          Modelica.SIunits.TemperatureSlope der_T(start = 0) "Time derivative of temperature (= der(T))";
          Interfaces.HeatPort_a port annotation(
            Placement(transformation(origin = {0, -100}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
        equation
          T = port.T;
          der_T = der(T);
          C * der(T) = port.Q_flow;
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-150, 110}, {150, 70}}, textString = "%name", lineColor = {0, 0, 255}), Polygon(points = {{0, 67}, {-20, 63}, {-40, 57}, {-52, 43}, {-58, 35}, {-68, 25}, {-72, 13}, {-76, -1}, {-78, -15}, {-76, -31}, {-76, -43}, {-76, -53}, {-70, -65}, {-64, -73}, {-48, -77}, {-30, -83}, {-18, -83}, {-2, -85}, {8, -89}, {22, -89}, {32, -87}, {42, -81}, {54, -75}, {56, -73}, {66, -61}, {68, -53}, {70, -51}, {72, -35}, {76, -21}, {78, -13}, {78, 3}, {74, 15}, {66, 25}, {54, 33}, {44, 41}, {36, 57}, {26, 65}, {0, 67}}, lineColor = {160, 160, 164}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Polygon(points = {{-58, 35}, {-68, 25}, {-72, 13}, {-76, -1}, {-78, -15}, {-76, -31}, {-76, -43}, {-76, -53}, {-70, -65}, {-64, -73}, {-48, -77}, {-30, -83}, {-18, -83}, {-2, -85}, {8, -89}, {22, -89}, {32, -87}, {42, -81}, {54, -75}, {42, -77}, {40, -77}, {30, -79}, {20, -81}, {18, -81}, {10, -81}, {2, -77}, {-12, -73}, {-22, -73}, {-30, -71}, {-40, -65}, {-50, -55}, {-56, -43}, {-58, -35}, {-58, -25}, {-60, -13}, {-60, -5}, {-60, 7}, {-58, 17}, {-56, 19}, {-52, 27}, {-48, 35}, {-44, 45}, {-40, 57}, {-58, 35}}, lineColor = {0, 0, 0}, fillColor = {160, 160, 164}, fillPattern = FillPattern.Solid), Text(extent = {{-69, 7}, {71, -24}}, lineColor = {0, 0, 0}, textString = "%C")}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{0, 67}, {-20, 63}, {-40, 57}, {-52, 43}, {-58, 35}, {-68, 25}, {-72, 13}, {-76, -1}, {-78, -15}, {-76, -31}, {-76, -43}, {-76, -53}, {-70, -65}, {-64, -73}, {-48, -77}, {-30, -83}, {-18, -83}, {-2, -85}, {8, -89}, {22, -89}, {32, -87}, {42, -81}, {54, -75}, {56, -73}, {66, -61}, {68, -53}, {70, -51}, {72, -35}, {76, -21}, {78, -13}, {78, 3}, {74, 15}, {66, 25}, {54, 33}, {44, 41}, {36, 57}, {26, 65}, {0, 67}}, lineColor = {160, 160, 164}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Polygon(points = {{-58, 35}, {-68, 25}, {-72, 13}, {-76, -1}, {-78, -15}, {-76, -31}, {-76, -43}, {-76, -53}, {-70, -65}, {-64, -73}, {-48, -77}, {-30, -83}, {-18, -83}, {-2, -85}, {8, -89}, {22, -89}, {32, -87}, {42, -81}, {54, -75}, {42, -77}, {40, -77}, {30, -79}, {20, -81}, {18, -81}, {10, -81}, {2, -77}, {-12, -73}, {-22, -73}, {-30, -71}, {-40, -65}, {-50, -55}, {-56, -43}, {-58, -35}, {-58, -25}, {-60, -13}, {-60, -5}, {-60, 7}, {-58, 17}, {-56, 19}, {-52, 27}, {-48, 35}, {-44, 45}, {-40, 57}, {-58, 35}}, lineColor = {0, 0, 0}, fillColor = {160, 160, 164}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-6, -1}, {6, -12}}, lineColor = {255, 0, 0}, fillColor = {191, 0, 0}, fillPattern = FillPattern.Solid), Text(extent = {{11, 13}, {50, -25}}, lineColor = {0, 0, 0}, textString = "T"), Line(points = {{0, -12}, {0, -96}}, color = {255, 0, 0})}),
            Documentation(info = "<HTML>
<p>
This is a generic model for the heat capacity of a material.
No specific geometry is assumed beyond a total volume with
uniform temperature for the entire volume.
Furthermore, it is assumed that the heat capacity
is constant (independent of temperature).
</p>
<p>
The temperature T [Kelvin] of this component is a <b>state</b>.
A default of T = 25 degree Celsius (= SIunits.Conversions.from_degC(25))
is used as start value for initialization.
This usually means that at start of integration the temperature of this
component is 25 degrees Celsius. You may, of course, define a different
temperature as start value for initialization. Alternatively, it is possible
to set parameter <b>steadyStateStart</b> to <b>true</b>. In this case
the additional equation '<b>der</b>(T) = 0' is used during
initialization, i.e., the temperature T is computed in such a way that
the component starts in <b>steady state</b>. This is useful in cases,
where one would like to start simulation in a suitable operating
point without being forced to integrate for a long time to arrive
at this point.
</p>
<p>
Note, that parameter <b>steadyStateStart</b> is not available in
the parameter menu of the simulation window, because its value
is utilized during translation to generate quite different
equations depending on its setting. Therefore, the value of this
parameter can only be changed before translating the model.
</p>
<p>
This component may be used for complicated geometries where
the heat capacity C is determined my measurements. If the component
consists mainly of one type of material, the <b>mass m</b> of the
component may be measured or calculated and multiplied with the
<b>specific heat capacity cp</b> of the component material to
compute C:
</p>
<pre>
   C = cp*m.
   Typical values for cp at 20 degC in J/(kg.K):
      aluminium   896
      concrete    840
      copper      383
      iron        452
      silver      235
      steel       420 ... 500 (V2A)
      wood       2500
</pre>
</html>"));
        end HeatCapacitor;

        model ThermalConductor "Lumped thermal element transporting heat without storing it"
          extends Interfaces.Element1D;
          parameter Modelica.SIunits.ThermalConductance G "Constant thermal conductance of material";
        equation
          Q_flow = G * dT;
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-90, 70}, {90, -70}}, lineColor = {0, 0, 0}, pattern = LinePattern.None, fillColor = {192, 192, 192}, fillPattern = FillPattern.Backward), Line(points = {{-90, 70}, {-90, -70}}, color = {0, 0, 0}, thickness = 0.5), Line(points = {{90, 70}, {90, -70}}, color = {0, 0, 0}, thickness = 0.5), Text(extent = {{-150, 115}, {150, 75}}, textString = "%name", lineColor = {0, 0, 255}), Text(extent = {{-150, -75}, {150, -105}}, lineColor = {0, 0, 0}, textString = "G=%G")}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-80, 0}, {80, 0}}, color = {255, 0, 0}, thickness = 0.5, arrow = {Arrow.None, Arrow.Filled}), Text(extent = {{-100, -20}, {100, -40}}, lineColor = {255, 0, 0}, textString = "Q_flow"), Text(extent = {{-100, 40}, {100, 20}}, lineColor = {0, 0, 0}, textString = "dT = port_a.T - port_b.T")}),
            Documentation(info = "<HTML>
<p>
This is a model for transport of heat without storing it; see also:
<a href=\"modelica://Modelica.Thermal.HeatTransfer.Components.ThermalResistor\">ThermalResistor</a>.
It may be used for complicated geometries where
the thermal conductance G (= inverse of thermal resistance)
is determined by measurements and is assumed to be constant
over the range of operations. If the component consists mainly of
one type of material and a regular geometry, it may be calculated,
e.g., with one of the following equations:
</p>
<ul>
<li><p>
    Conductance for a <b>box</b> geometry under the assumption
    that heat flows along the box length:</p>
    <pre>
    G = k*A/L
    k: Thermal conductivity (material constant)
    A: Area of box
    L: Length of box
    </pre>
    </li>
<li><p>
    Conductance for a <b>cylindrical</b> geometry under the assumption
    that heat flows from the inside to the outside radius
    of the cylinder:</p>
    <pre>
    G = 2*pi*k*L/log(r_out/r_in)
    pi   : Modelica.Constants.pi
    k    : Thermal conductivity (material constant)
    L    : Length of cylinder
    log  : Modelica.Math.log;
    r_out: Outer radius of cylinder
    r_in : Inner radius of cylinder
    </pre>
    </li>
</ul>
<pre>
    Typical values for k at 20 degC in W/(m.K):
      aluminium   220
      concrete      1
      copper      384
      iron         74
      silver      407
      steel        45 .. 15 (V2A)
      wood         0.1 ... 0.2
</pre>
</html>"));
        end ThermalConductor;

        model ThermalResistor "Lumped thermal element transporting heat without storing it"
          extends Interfaces.Element1D;
          parameter Modelica.SIunits.ThermalResistance R "Constant thermal resistance of material";
        equation
          dT = R * Q_flow;
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-90, 70}, {90, -70}}, lineColor = {0, 0, 0}, pattern = LinePattern.None, fillColor = {192, 192, 192}, fillPattern = FillPattern.Forward), Line(points = {{-90, 70}, {-90, -70}}, color = {0, 0, 0}, thickness = 0.5), Line(points = {{90, 70}, {90, -70}}, color = {0, 0, 0}, thickness = 0.5), Text(extent = {{-150, 115}, {150, 75}}, textString = "%name", lineColor = {0, 0, 255}), Text(extent = {{-150, -75}, {150, -105}}, lineColor = {0, 0, 0}, textString = "R=%R")}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-80, 0}, {80, 0}}, color = {255, 0, 0}, thickness = 0.5, arrow = {Arrow.None, Arrow.Filled}), Text(extent = {{-100, -20}, {100, -40}}, lineColor = {255, 0, 0}, textString = "Q_flow"), Text(extent = {{-100, 40}, {100, 20}}, lineColor = {0, 0, 0}, textString = "dT = port_a.T - port_b.T")}),
            Documentation(info = "<HTML>
<p>
This is a model for transport of heat without storing it, same as the
<a href=\"modelica://Modelica.Thermal.HeatTransfer.Components.ThermalConductor\">ThermalConductor</a>
but using the thermal resistance instead of the thermal conductance as a parameter.
This is advantageous for series connections of ThermalResistors,
especially if it shall be allowed that a ThermalResistance is defined to be zero (i.e. no temperature difference).
</p>
</html>"));
        end ThermalResistor;

        model Convection "Lumped thermal element for heat convection (Q_flow = Gc*dT)"
          Modelica.SIunits.HeatFlowRate Q_flow "Heat flow rate from solid -> fluid";
          Modelica.SIunits.TemperatureDifference dT "= solid.T - fluid.T";
          Modelica.Blocks.Interfaces.RealInput Gc(unit = "W/K") "Signal representing the convective thermal conductance in [W/K]" annotation(
            Placement(transformation(origin = {0, 100}, extent = {{-20, -20}, {20, 20}}, rotation = 270)));
          Interfaces.HeatPort_a solid annotation(
            Placement(transformation(extent = {{-110, -10}, {-90, 10}}, rotation = 0)));
          Interfaces.HeatPort_b fluid annotation(
            Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
        equation
          dT = solid.T - fluid.T;
          solid.Q_flow = Q_flow;
          fluid.Q_flow = -Q_flow;
          Q_flow = Gc * dT;
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-62, 80}, {98, -80}}, lineColor = {255, 255, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-90, 80}, {-60, -80}}, lineColor = {0, 0, 0}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Backward), Text(extent = {{-150, -90}, {150, -130}}, textString = "%name", lineColor = {0, 0, 255}), Line(points = {{100, 0}, {100, 0}}, color = {0, 127, 255}), Line(points = {{-60, 20}, {76, 20}}, color = {191, 0, 0}), Line(points = {{-60, -20}, {76, -20}}, color = {191, 0, 0}), Line(points = {{-34, 80}, {-34, -80}}, color = {0, 127, 255}), Line(points = {{6, 80}, {6, -80}}, color = {0, 127, 255}), Line(points = {{40, 80}, {40, -80}}, color = {0, 127, 255}), Line(points = {{76, 80}, {76, -80}}, color = {0, 127, 255}), Line(points = {{-34, -80}, {-44, -60}}, color = {0, 127, 255}), Line(points = {{-34, -80}, {-24, -60}}, color = {0, 127, 255}), Line(points = {{6, -80}, {-4, -60}}, color = {0, 127, 255}), Line(points = {{6, -80}, {16, -60}}, color = {0, 127, 255}), Line(points = {{40, -80}, {30, -60}}, color = {0, 127, 255}), Line(points = {{40, -80}, {50, -60}}, color = {0, 127, 255}), Line(points = {{76, -80}, {66, -60}}, color = {0, 127, 255}), Line(points = {{76, -80}, {86, -60}}, color = {0, 127, 255}), Line(points = {{56, -30}, {76, -20}}, color = {191, 0, 0}), Line(points = {{56, -10}, {76, -20}}, color = {191, 0, 0}), Line(points = {{56, 10}, {76, 20}}, color = {191, 0, 0}), Line(points = {{56, 30}, {76, 20}}, color = {191, 0, 0}), Text(extent = {{22, 124}, {92, 98}}, lineColor = {0, 0, 0}, textString = "Gc")}),
            Documentation(info = "<HTML>
<p>
This is a model of linear heat convection, e.g., the heat transfer between a plate and the surrounding air; see also:
<a href=\"modelica://Modelica.Thermal.HeatTransfer.Components.ConvectiveResistor\">ConvectiveResistor</a>.
It may be used for complicated solid geometries and fluid flow over the solid by determining the
convective thermal conductance Gc by measurements. The basic constitutive equation for convection is
</p>
<pre>
   Q_flow = Gc*(solid.T - fluid.T);
   Q_flow: Heat flow rate from connector 'solid' (e.g., a plate)
      to connector 'fluid' (e.g., the surrounding air)
</pre>
<p>
Gc = G.signal[1] is an input signal to the component, since Gc is
nearly never constant in practice. For example, Gc may be a function
of the speed of a cooling fan. For simple situations,
Gc may be <i>calculated</i> according to
</p>
<pre>
   Gc = A*h
   A: Convection area (e.g., perimeter*length of a box)
   h: Heat transfer coefficient
</pre>
<p>
where the heat transfer coefficient h is calculated
from properties of the fluid flowing over the solid. Examples:
</p>
<p>
<b>Machines cooled by air</b> (empirical, very rough approximation according
to R. Fischer: Elektrische Maschinen, 10th edition, Hanser-Verlag 1999,
p. 378):
</p>
<pre>
    h = 7.8*v^0.78 [W/(m2.K)] (forced convection)
      = 12         [W/(m2.K)] (free convection)
    where
      v: Air velocity in [m/s]
</pre>
<p><b>Laminar</b> flow with constant velocity of a fluid along a
<b>flat plate</b> where the heat flow rate from the plate
to the fluid (= solid.Q_flow) is kept constant
(according to J.P.Holman: Heat Transfer, 8th edition,
McGraw-Hill, 1997, p.270):
</p>
<pre>
   h  = Nu*k/x;
   Nu = 0.453*Re^(1/2)*Pr^(1/3);
   where
      h  : Heat transfer coefficient
      Nu : = h*x/k       (Nusselt number)
      Re : = v*x*rho/mue (Reynolds number)
      Pr : = cp*mue/k    (Prandtl number)
      v  : Absolute velocity of fluid
      x  : distance from leading edge of flat plate
      rho: density of fluid (material constant
      mue: dynamic viscosity of fluid (material constant)
      cp : specific heat capacity of fluid (material constant)
      k  : thermal conductivity of fluid (material constant)
   and the equation for h holds, provided
      Re &lt; 5e5 and 0.6 &lt; Pr &lt; 50
</pre>
</html>"),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-90, 80}, {-60, -80}}, lineColor = {0, 0, 0}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Backward), Line(points = {{100, 0}, {100, 0}}, color = {0, 127, 255}), Line(points = {{100, 0}, {100, 0}}, color = {0, 127, 255}), Line(points = {{100, 0}, {100, 0}}, color = {0, 127, 255}), Text(extent = {{-40, 40}, {80, 20}}, lineColor = {255, 0, 0}, textString = "Q_flow"), Line(points = {{-60, 20}, {76, 20}}, color = {191, 0, 0}), Line(points = {{-60, -20}, {76, -20}}, color = {191, 0, 0}), Line(points = {{-34, 80}, {-34, -80}}, color = {0, 127, 255}), Line(points = {{6, 80}, {6, -80}}, color = {0, 127, 255}), Line(points = {{40, 80}, {40, -80}}, color = {0, 127, 255}), Line(points = {{76, 80}, {76, -80}}, color = {0, 127, 255}), Line(points = {{-34, -80}, {-44, -60}}, color = {0, 127, 255}), Line(points = {{-34, -80}, {-24, -60}}, color = {0, 127, 255}), Line(points = {{6, -80}, {-4, -60}}, color = {0, 127, 255}), Line(points = {{6, -80}, {16, -60}}, color = {0, 127, 255}), Line(points = {{40, -80}, {30, -60}}, color = {0, 127, 255}), Line(points = {{40, -80}, {50, -60}}, color = {0, 127, 255}), Line(points = {{76, -80}, {66, -60}}, color = {0, 127, 255}), Line(points = {{76, -80}, {86, -60}}, color = {0, 127, 255}), Line(points = {{56, -30}, {76, -20}}, color = {191, 0, 0}), Line(points = {{56, -10}, {76, -20}}, color = {191, 0, 0}), Line(points = {{56, 10}, {76, 20}}, color = {191, 0, 0}), Line(points = {{56, 30}, {76, 20}}, color = {191, 0, 0})}));
        end Convection;

        model ConvectiveResistor "Lumped thermal element for heat convection (dT = Rc*Q_flow)"
          Modelica.SIunits.HeatFlowRate Q_flow "Heat flow rate from solid -> fluid";
          Modelica.SIunits.TemperatureDifference dT "= solid.T - fluid.T";
          Modelica.Blocks.Interfaces.RealInput Rc(unit = "K/W") "Signal representing the convective thermal resistance in [K/W]" annotation(
            Placement(transformation(origin = {0, 100}, extent = {{-20, -20}, {20, 20}}, rotation = 270)));
          Interfaces.HeatPort_a solid annotation(
            Placement(transformation(extent = {{-110, -10}, {-90, 10}}, rotation = 0)));
          Interfaces.HeatPort_b fluid annotation(
            Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
        equation
          dT = solid.T - fluid.T;
          solid.Q_flow = Q_flow;
          fluid.Q_flow = -Q_flow;
          dT = Rc * Q_flow;
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-62, 80}, {98, -80}}, lineColor = {255, 255, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-90, 80}, {-60, -80}}, lineColor = {0, 0, 0}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Forward), Text(extent = {{-150, -90}, {150, -130}}, textString = "%name", lineColor = {0, 0, 255}), Line(points = {{100, 0}, {100, 0}}, color = {0, 127, 255}), Line(points = {{-60, 20}, {76, 20}}, color = {191, 0, 0}), Line(points = {{-60, -20}, {76, -20}}, color = {191, 0, 0}), Line(points = {{-34, 80}, {-34, -80}}, color = {0, 127, 255}), Line(points = {{6, 80}, {6, -80}}, color = {0, 127, 255}), Line(points = {{40, 80}, {40, -80}}, color = {0, 127, 255}), Line(points = {{76, 80}, {76, -80}}, color = {0, 127, 255}), Line(points = {{-34, -80}, {-44, -60}}, color = {0, 127, 255}), Line(points = {{-34, -80}, {-24, -60}}, color = {0, 127, 255}), Line(points = {{6, -80}, {-4, -60}}, color = {0, 127, 255}), Line(points = {{6, -80}, {16, -60}}, color = {0, 127, 255}), Line(points = {{40, -80}, {30, -60}}, color = {0, 127, 255}), Line(points = {{40, -80}, {50, -60}}, color = {0, 127, 255}), Line(points = {{76, -80}, {66, -60}}, color = {0, 127, 255}), Line(points = {{76, -80}, {86, -60}}, color = {0, 127, 255}), Line(points = {{56, -30}, {76, -20}}, color = {191, 0, 0}), Line(points = {{56, -10}, {76, -20}}, color = {191, 0, 0}), Line(points = {{56, 10}, {76, 20}}, color = {191, 0, 0}), Line(points = {{56, 30}, {76, 20}}, color = {191, 0, 0}), Text(extent = {{22, 124}, {92, 98}}, lineColor = {0, 0, 0}, textString = "Rc")}),
            Documentation(info = "<HTML>
<p>
This is a model of linear heat convection, e.g., the heat transfer between a plate and the surrounding air; same as the
<a href=\"modelica://Modelica.Thermal.HeatTransfer.Components.Convection\">Convection</a> component
but using the convective resistance instead of the convective conductance as an input.
This is advantageous for series connections of ConvectiveResistors,
especially if it shall be allowed that a convective resistance is defined to be zero (i.e. no temperature difference).
</p>
</html>"),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-90, 80}, {-60, -80}}, lineColor = {0, 0, 0}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Forward), Line(points = {{100, 0}, {100, 0}}, color = {0, 127, 255}), Line(points = {{100, 0}, {100, 0}}, color = {0, 127, 255}), Line(points = {{100, 0}, {100, 0}}, color = {0, 127, 255}), Text(extent = {{-40, 40}, {80, 20}}, lineColor = {255, 0, 0}, textString = "Q_flow"), Line(points = {{-60, 20}, {76, 20}}, color = {191, 0, 0}), Line(points = {{-60, -20}, {76, -20}}, color = {191, 0, 0}), Line(points = {{-34, 80}, {-34, -80}}, color = {0, 127, 255}), Line(points = {{6, 80}, {6, -80}}, color = {0, 127, 255}), Line(points = {{40, 80}, {40, -80}}, color = {0, 127, 255}), Line(points = {{76, 80}, {76, -80}}, color = {0, 127, 255}), Line(points = {{-34, -80}, {-44, -60}}, color = {0, 127, 255}), Line(points = {{-34, -80}, {-24, -60}}, color = {0, 127, 255}), Line(points = {{6, -80}, {-4, -60}}, color = {0, 127, 255}), Line(points = {{6, -80}, {16, -60}}, color = {0, 127, 255}), Line(points = {{40, -80}, {30, -60}}, color = {0, 127, 255}), Line(points = {{40, -80}, {50, -60}}, color = {0, 127, 255}), Line(points = {{76, -80}, {66, -60}}, color = {0, 127, 255}), Line(points = {{76, -80}, {86, -60}}, color = {0, 127, 255}), Line(points = {{56, -30}, {76, -20}}, color = {191, 0, 0}), Line(points = {{56, -10}, {76, -20}}, color = {191, 0, 0}), Line(points = {{56, 10}, {76, 20}}, color = {191, 0, 0}), Line(points = {{56, 30}, {76, 20}}, color = {191, 0, 0})}));
        end ConvectiveResistor;

        model BodyRadiation "Lumped thermal element for radiation heat transfer"
          extends Interfaces.Element1D;
          parameter Real Gr(unit = "m2") "Net radiation conductance between two surfaces (see docu)";
        equation
          Q_flow = Gr * Modelica.Constants.sigma * (port_a.T ^ 4 - port_b.T ^ 4);
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{50, 80}, {90, -80}}, lineColor = {0, 0, 0}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Backward), Rectangle(extent = {{-90, 80}, {-50, -80}}, lineColor = {0, 0, 0}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Backward), Line(points = {{-36, 10}, {36, 10}}, color = {191, 0, 0}), Line(points = {{-36, 10}, {-26, 16}}, color = {191, 0, 0}), Line(points = {{-36, 10}, {-26, 4}}, color = {191, 0, 0}), Line(points = {{-36, -10}, {36, -10}}, color = {191, 0, 0}), Line(points = {{26, -16}, {36, -10}}, color = {191, 0, 0}), Line(points = {{26, -4}, {36, -10}}, color = {191, 0, 0}), Line(points = {{-36, -30}, {36, -30}}, color = {191, 0, 0}), Line(points = {{-36, -30}, {-26, -24}}, color = {191, 0, 0}), Line(points = {{-36, -30}, {-26, -36}}, color = {191, 0, 0}), Line(points = {{-36, 30}, {36, 30}}, color = {191, 0, 0}), Line(points = {{26, 24}, {36, 30}}, color = {191, 0, 0}), Line(points = {{26, 36}, {36, 30}}, color = {191, 0, 0}), Text(extent = {{-150, 125}, {150, 85}}, textString = "%name", lineColor = {0, 0, 255}), Text(extent = {{-150, -90}, {150, -120}}, lineColor = {0, 0, 0}, textString = "Gr=%Gr"), Rectangle(extent = {{-50, 80}, {-44, -80}}, lineColor = {191, 0, 0}, fillColor = {191, 0, 0}, fillPattern = FillPattern.Solid), Rectangle(extent = {{45, 80}, {50, -80}}, lineColor = {191, 0, 0}, fillColor = {191, 0, 0}, fillPattern = FillPattern.Solid)}),
            Documentation(info = "<HTML>
<p>
This is a model describing the thermal radiation, i.e., electromagnetic
radiation emitted between two bodies as a result of their temperatures.
The following constitutive equation is used:
</p>
<pre>
    Q_flow = Gr*sigma*(port_a.T^4 - port_b.T^4);
</pre>
<p>
where Gr is the radiation conductance and sigma is the Stefan-Boltzmann
constant (= Modelica.Constants.sigma). Gr may be determined by
measurements and is assumed to be constant over the range of operations.
</p>
<p>
For simple cases, Gr may be analytically computed. The analytical
equations use epsilon, the emission value of a body which is in the
range 0..1. Epsilon=1, if the body absorbs all radiation (= black body).
Epsilon=0, if the body reflects all radiation and does not absorb any.
</p>
<pre>
   Typical values for epsilon:
   aluminium, polished    0.04
   copper, polished       0.04
   gold, polished         0.02
   paper                  0.09
   rubber                 0.95
   silver, polished       0.02
   wood                   0.85..0.9
</pre>
<p><b>Analytical Equations for Gr</b></p>
<p>
<b>Small convex object in large enclosure</b>
(e.g., a hot machine in a room):
</p>
<pre>
    Gr = e*A
    where
       e: Emission value of object (0..1)
       A: Surface area of object where radiation
          heat transfer takes place
</pre>
<p><b>Two parallel plates</b>:</p>
<pre>
    Gr = A/(1/e1 + 1/e2 - 1)
    where
       e1: Emission value of plate1 (0..1)
       e2: Emission value of plate2 (0..1)
       A : Area of plate1 (= area of plate2)
</pre>
<p><b>Two long cylinders in each other</b>, where radiation takes
place from the inner to the outer cylinder):
</p>
<pre>
    Gr = 2*pi*r1*L/(1/e1 + (1/e2 - 1)*(r1/r2))
    where
       pi: = Modelica.Constants.pi
       r1: Radius of inner cylinder
       r2: Radius of outer cylinder
       L : Length of the two cylinders
       e1: Emission value of inner cylinder (0..1)
       e2: Emission value of outer cylinder (0..1)
</pre>
</html>"),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-90, 80}, {-56, -80}}, lineColor = {0, 0, 0}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Backward), Line(points = {{-56, 80}, {-56, -80}}, color = {0, 0, 0}, thickness = 1), Line(points = {{50, 80}, {50, -80}}, color = {0, 0, 0}, thickness = 1), Rectangle(extent = {{50, 80}, {90, -80}}, lineColor = {0, 0, 0}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Backward), Line(points = {{-40, 10}, {40, 10}}, color = {191, 0, 0}), Line(points = {{-40, 10}, {-30, 16}}, color = {191, 0, 0}), Line(points = {{-40, 10}, {-30, 4}}, color = {191, 0, 0}), Line(points = {{-40, -10}, {40, -10}}, color = {191, 0, 0}), Line(points = {{30, -16}, {40, -10}}, color = {191, 0, 0}), Line(points = {{30, -4}, {40, -10}}, color = {191, 0, 0}), Line(points = {{-40, -30}, {40, -30}}, color = {191, 0, 0}), Line(points = {{-40, -30}, {-30, -24}}, color = {191, 0, 0}), Line(points = {{-40, -30}, {-30, -36}}, color = {191, 0, 0}), Line(points = {{-40, 30}, {40, 30}}, color = {191, 0, 0}), Line(points = {{30, 24}, {40, 30}}, color = {191, 0, 0}), Line(points = {{30, 36}, {40, 30}}, color = {191, 0, 0})}));
        end BodyRadiation;

        model ThermalCollector "Collects m heat flows"
          parameter Integer m(min = 1) = 3 "Number of collected heat flows";
          Interfaces.HeatPort_a port_a[m] annotation(
            Placement(transformation(extent = {{-10, 110}, {10, 90}})));
          Interfaces.HeatPort_b port_b annotation(
            Placement(transformation(extent = {{-10, -110}, {10, -90}})));
        equation
          port_b.Q_flow + sum(port_a.Q_flow) = 0;
          port_a.T = fill(port_b.T, m);
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-150, -30}, {150, -70}}, textString = "%name", lineColor = {0, 0, 255}), Text(extent = {{-150, 80}, {150, 50}}, lineColor = {0, 0, 0}, textString = "m=%m"), Line(points = {{0, 90}, {0, 40}}, color = {181, 0, 0}, smooth = Smooth.None), Rectangle(extent = {{-60, 40}, {60, 30}}, lineColor = {181, 0, 0}, fillColor = {181, 0, 0}, fillPattern = FillPattern.Solid), Line(points = {{-60, 30}, {0, -30}, {0, -90}}, color = {181, 0, 0}, smooth = Smooth.None), Line(points = {{0, -30}, {-20, 30}}, color = {181, 0, 0}, smooth = Smooth.None), Line(points = {{0, -30}, {20, 30}}, color = {181, 0, 0}, smooth = Smooth.None), Line(points = {{0, -30}, {60, 30}}, color = {181, 0, 0}, smooth = Smooth.None)}),
            Documentation(info = "<html>
<p>
This is a model to collect the heat flows from <i>m</i> heatports to one single heatport.
</p>
</html>"));
        end ThermalCollector;
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(origin = {12, 40}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Backward, extent = {{-100, -100}, {-70, 18}}), Line(origin = {12, 40}, points = {{-44, 16}, {-44, -100}}, color = {0, 127, 255}), Line(origin = {12, 40}, points = {{-4, 16}, {-4, -100}}, color = {0, 127, 255}), Line(origin = {12, 40}, points = {{30, 18}, {30, -100}}, color = {0, 127, 255}), Line(origin = {12, 40}, points = {{66, 18}, {66, -100}}, color = {0, 127, 255}), Line(origin = {12, 40}, points = {{66, -100}, {76, -80}}, color = {0, 127, 255}), Line(origin = {12, 40}, points = {{66, -100}, {56, -80}}, color = {0, 127, 255}), Line(origin = {12, 40}, points = {{30, -100}, {40, -80}}, color = {0, 127, 255}), Line(origin = {12, 40}, points = {{30, -100}, {20, -80}}, color = {0, 127, 255}), Line(origin = {12, 40}, points = {{-4, -100}, {6, -80}}, color = {0, 127, 255}), Line(origin = {12, 40}, points = {{-4, -100}, {-14, -80}}, color = {0, 127, 255}), Line(origin = {12, 40}, points = {{-44, -100}, {-34, -80}}, color = {0, 127, 255}), Line(origin = {12, 40}, points = {{-44, -100}, {-54, -80}}, color = {0, 127, 255}), Line(origin = {12, 40}, points = {{-70, -60}, {66, -60}}, color = {191, 0, 0}), Line(origin = {12, 40}, points = {{46, -70}, {66, -60}}, color = {191, 0, 0}), Line(origin = {12, 40}, points = {{46, -50}, {66, -60}}, color = {191, 0, 0}), Line(origin = {12, 40}, points = {{46, -30}, {66, -20}}, color = {191, 0, 0}), Line(origin = {12, 40}, points = {{46, -10}, {66, -20}}, color = {191, 0, 0}), Line(origin = {12, 40}, points = {{-70, -20}, {66, -20}}, color = {191, 0, 0})}),
          Documentation(info = "<html>

</html>"));
      end Components;

      package Sensors "Thermal sensors"
        extends Modelica.Icons.SensorsPackage;

        model TemperatureSensor "Absolute temperature sensor in Kelvin"
          Modelica.Blocks.Interfaces.RealOutput T(unit = "K") "Absolute temperature as output signal" annotation(
            Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
          Interfaces.HeatPort_a port annotation(
            Placement(transformation(extent = {{-110, -10}, {-90, 10}}, rotation = 0)));
        equation
          T = port.T;
          port.Q_flow = 0;
          annotation(
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-20, -98}, {20, -60}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {191, 0, 0}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-12, 40}, {12, -68}}, lineColor = {191, 0, 0}, fillColor = {191, 0, 0}, fillPattern = FillPattern.Solid), Line(points = {{12, 0}, {90, 0}}, color = {0, 0, 255}), Line(points = {{-94, 0}, {-14, 0}}, color = {191, 0, 0}), Polygon(points = {{-12, 40}, {-12, 80}, {-10, 86}, {-6, 88}, {0, 90}, {6, 88}, {10, 86}, {12, 80}, {12, 40}, {-12, 40}}, lineColor = {0, 0, 0}, lineThickness = 0.5), Line(points = {{-12, 40}, {-12, -64}}, color = {0, 0, 0}, thickness = 0.5), Line(points = {{12, 40}, {12, -64}}, color = {0, 0, 0}, thickness = 0.5), Line(points = {{-40, -20}, {-12, -20}}, color = {0, 0, 0}), Line(points = {{-40, 20}, {-12, 20}}, color = {0, 0, 0}), Line(points = {{-40, 60}, {-12, 60}}, color = {0, 0, 0}), Text(extent = {{102, -28}, {60, -78}}, lineColor = {0, 0, 0}, textString = "K")}),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-20, -98}, {20, -60}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {191, 0, 0}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-12, 40}, {12, -68}}, lineColor = {191, 0, 0}, fillColor = {191, 0, 0}, fillPattern = FillPattern.Solid), Line(points = {{12, 0}, {90, 0}}, color = {0, 0, 255}), Line(points = {{-90, 0}, {-12, 0}}, color = {191, 0, 0}), Polygon(points = {{-12, 40}, {-12, 80}, {-10, 86}, {-6, 88}, {0, 90}, {6, 88}, {10, 86}, {12, 80}, {12, 40}, {-12, 40}}, lineColor = {0, 0, 0}, lineThickness = 0.5), Line(points = {{-12, 40}, {-12, -64}}, color = {0, 0, 0}, thickness = 0.5), Line(points = {{12, 40}, {12, -64}}, color = {0, 0, 0}, thickness = 0.5), Line(points = {{-40, -20}, {-12, -20}}, color = {0, 0, 0}), Line(points = {{-40, 20}, {-12, 20}}, color = {0, 0, 0}), Line(points = {{-40, 60}, {-12, 60}}, color = {0, 0, 0}), Text(extent = {{126, -20}, {26, -120}}, lineColor = {0, 0, 0}, textString = "K"), Text(extent = {{-150, 130}, {150, 90}}, textString = "%name", lineColor = {0, 0, 255})}),
            Documentation(info = "<HTML>
<p>
This is an ideal absolute temperature sensor which returns
the temperature of the connected port in Kelvin as an output
signal.  The sensor itself has no thermal interaction with
whatever it is connected to.  Furthermore, no
thermocouple-like lags are associated with this
sensor model.
</p>
</html>"));
        end TemperatureSensor;

        model RelTemperatureSensor "Relative Temperature sensor"
          extends Modelica.Icons.TranslationalSensor;
          Interfaces.HeatPort_a port_a annotation(
            Placement(transformation(extent = {{-110, -10}, {-90, 10}}, rotation = 0)));
          Interfaces.HeatPort_b port_b annotation(
            Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
          Modelica.Blocks.Interfaces.RealOutput T_rel(unit = "K", displayUnit = "K") "Relative temperature as output signal" annotation(
            Placement(transformation(origin = {0, -90}, extent = {{10, -10}, {-10, 10}}, rotation = 90)));
        equation
          T_rel = port_a.T - port_b.T;
          0 = port_a.Q_flow;
          0 = port_b.Q_flow;
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-90, 0}, {-70, 0}, {-70, 0}}, color = {191, 0, 0}), Line(points = {{-90, 0}, {-70, 0}, {-70, 0}}, color = {191, 0, 0}), Line(points = {{70, 0}, {90, 0}, {90, 0}}, color = {191, 0, 0}), Line(points = {{0, -30}, {0, -80}}, color = {0, 0, 255}), Text(extent = {{-150, 80}, {150, 40}}, textString = "%name", lineColor = {0, 0, 255}), Text(extent = {{92, -62}, {34, -122}}, lineColor = {0, 0, 0}, textString = "K")}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-90, 0}, {-70, 0}, {-70, 0}}, color = {191, 0, 0}), Line(points = {{-98, 0}, {-70, 0}, {-70, 0}}, color = {191, 0, 0}), Line(points = {{70, 0}, {94, 0}, {94, 0}}, color = {191, 0, 0}), Line(points = {{0, -30}, {0, -80}}, color = {0, 0, 255}), Text(extent = {{64, -74}, {32, -102}}, lineColor = {0, 0, 0}, textString = "K")}),
            Documentation(info = "<HTML>
<p>
The relative temperature \"port_a.T - port_b.T\" is determined between
the two ports of this component and is provided as output signal in Kelvin.
</p>
</html>"));
        end RelTemperatureSensor;

        model HeatFlowSensor "Heat flow rate sensor"
          extends Modelica.Icons.RotationalSensor;
          Modelica.Blocks.Interfaces.RealOutput Q_flow(unit = "W") "Heat flow from port_a to port_b as output signal" annotation(
            Placement(transformation(origin = {0, -100}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
          Interfaces.HeatPort_a port_a annotation(
            Placement(transformation(extent = {{-110, -10}, {-90, 10}}, rotation = 0)));
          Interfaces.HeatPort_b port_b annotation(
            Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
        equation
          port_a.T = port_b.T;
          port_a.Q_flow + port_b.Q_flow = 0;
          Q_flow = port_a.Q_flow;
          annotation(
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-70, 0}, {-95, 0}}, color = {191, 0, 0}), Line(points = {{0, -70}, {0, -90}}, color = {0, 0, 127}), Line(points = {{94, 0}, {69, 0}}, color = {191, 0, 0})}),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{5, -86}, {116, -110}}, lineColor = {0, 0, 0}, textString = "Q_flow"), Line(points = {{-70, 0}, {-90, 0}}, color = {191, 0, 0}), Line(points = {{69, 0}, {90, 0}}, color = {191, 0, 0}), Line(points = {{0, -70}, {0, -90}}, color = {0, 0, 127}), Text(extent = {{-150, 125}, {150, 85}}, textString = "%name", lineColor = {0, 0, 255})}),
            Documentation(info = "<HTML>
<p>
This model is capable of monitoring the heat flow rate flowing through
this component. The sensed value of heat flow rate is the amount that
passes through this sensor while keeping the temperature drop across the
sensor zero.  This is an ideal model so it does not absorb any energy
and it has no direct effect on the thermal response of a system it is included in.
The output signal is positive, if the heat flows from port_a to port_b.
</p>
</html>"));
        end HeatFlowSensor;

        model ConditionalFixedHeatFlowSensor "HeatFlowSensor, conditional fixed Temperature"
          extends Modelica.Icons.RotationalSensor;
          parameter Boolean useFixedTemperature(start = false) "Fixed Temperature if true" annotation(
            Evaluate = true);
          Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a port_a annotation(
            Placement(transformation(extent = {{-110, -10}, {-90, 10}})));
          Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_b port_b annotation(
            Placement(transformation(extent = {{90, -10}, {110, 10}})));
          Modelica.Thermal.HeatTransfer.Sensors.HeatFlowSensor heatFlowSensor annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}})));
          Modelica.Thermal.HeatTransfer.Sources.FixedTemperature fixedTemperature(final T(displayUnit = "K") = 293.15) if useFixedTemperature annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 90, origin = {-50, -30})));
          Modelica.Blocks.Interfaces.RealOutput Q_flow(unit = "W") "Heat flow from port_a to port_b as output signal" annotation(
            Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 270, origin = {0, -100})));
        equation
          connect(heatFlowSensor.port_b, port_b) annotation(
            Line(points = {{10, 0}, {100, 0}}, color = {191, 0, 0}, smooth = Smooth.None));
          connect(port_a, heatFlowSensor.port_a) annotation(
            Line(points = {{-100, 0}, {-10, 0}}, color = {191, 0, 0}, smooth = Smooth.None));
          connect(fixedTemperature.port, heatFlowSensor.port_a) annotation(
            Line(points = {{-50, -20}, {-50, 0}, {-10, 0}}, color = {191, 0, 0}, smooth = Smooth.None));
          connect(heatFlowSensor.Q_flow, Q_flow) annotation(
            Line(points = {{0, -10}, {0, -70}}, color = {0, 0, 127}, smooth = Smooth.None));
          annotation(
            Icon(graphics = {Rectangle(lineColor = {255, 0, 0}, fillColor = {255, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, extent = {{-90, -10}, {-70, 10}}), Rectangle(lineColor = {255, 0, 0}, fillColor = {255, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, extent = {{70, -10}, {90, 10}}), Text(lineColor = {0, 0, 255}, extent = {{-150, 85}, {150, 125}}, textString = "%name"), Line(points = {{0, -70}, {0, -90}}, color = {0, 0, 127})}, coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = true, initialScale = 0.1)),
            Documentation(info = "<html>
<p>
If useFixedTemperature = false, this sensor acts just as a normal
<a href=\"modelica://Modelica.Thermal.HeatTransfer.Sensors.HeatFlowSensor\">HeatFlowSensor</a>.
</p>
<p>
If useFixedTemperature = true, it is assumed that the connections to both heatPorts of this sensor are conditionally removed;
in this case, the measured Q_flow is reported = 0 automatically.
To avoid a singular equation system, the temperature of the sensor is set to 293.15 K.
</p>
</html>"));
        end ConditionalFixedHeatFlowSensor;
        annotation(
          Documentation(info = "<html>

</html>"));
      end Sensors;

      package Sources "Thermal sources"
        extends Modelica.Icons.SourcesPackage;

        model FixedTemperature "Fixed temperature boundary condition in Kelvin"
          parameter Modelica.SIunits.Temperature T "Fixed temperature at port";
          Interfaces.HeatPort_b port annotation(
            Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
        equation
          port.T = T;
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-150, 150}, {150, 110}}, textString = "%name", lineColor = {0, 0, 255}), Text(extent = {{-150, -110}, {150, -140}}, lineColor = {0, 0, 0}, textString = "T=%T"), Rectangle(extent = {{-100, 100}, {100, -100}}, lineColor = {0, 0, 0}, pattern = LinePattern.None, fillColor = {159, 159, 223}, fillPattern = FillPattern.Backward), Text(extent = {{0, 0}, {-100, -100}}, lineColor = {0, 0, 0}, textString = "K"), Line(points = {{-52, 0}, {56, 0}}, color = {191, 0, 0}, thickness = 0.5), Polygon(points = {{50, -20}, {50, 20}, {90, 0}, {50, -20}}, lineColor = {191, 0, 0}, fillColor = {191, 0, 0}, fillPattern = FillPattern.Solid)}),
            Documentation(info = "<HTML>
<p>
This model defines a fixed temperature T at its port in Kelvin,
i.e., it defines a fixed temperature as a boundary condition.
</p>
</html>"),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, 100}, {100, -101}}, lineColor = {0, 0, 0}, pattern = LinePattern.None, fillColor = {159, 159, 223}, fillPattern = FillPattern.Backward), Line(points = {{-52, 0}, {56, 0}}, color = {191, 0, 0}, thickness = 0.5), Text(extent = {{0, 0}, {-100, -100}}, lineColor = {0, 0, 0}, textString = "K"), Polygon(points = {{52, -20}, {52, 20}, {90, 0}, {52, -20}}, lineColor = {191, 0, 0}, fillColor = {191, 0, 0}, fillPattern = FillPattern.Solid)}));
        end FixedTemperature;

        model PrescribedTemperature "Variable temperature boundary condition in Kelvin"
          Interfaces.HeatPort_b port annotation(
            Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
          Modelica.Blocks.Interfaces.RealInput T(unit = "K") annotation(
            Placement(transformation(extent = {{-140, -20}, {-100, 20}}, rotation = 0)));
        equation
          port.T = T;
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, 100}, {100, -100}}, lineColor = {0, 0, 0}, pattern = LinePattern.None, fillColor = {159, 159, 223}, fillPattern = FillPattern.Backward), Line(points = {{-102, 0}, {64, 0}}, color = {191, 0, 0}, thickness = 0.5), Text(extent = {{0, 0}, {-100, -100}}, lineColor = {0, 0, 0}, textString = "K"), Text(extent = {{-150, 150}, {150, 110}}, textString = "%name", lineColor = {0, 0, 255}), Polygon(points = {{50, -20}, {50, 20}, {90, 0}, {50, -20}}, lineColor = {191, 0, 0}, fillColor = {191, 0, 0}, fillPattern = FillPattern.Solid)}),
            Documentation(info = "<HTML>
<p>
This model represents a variable temperature boundary condition.
The temperature in [K] is given as input signal <b>T</b>
to the model. The effect is that an instance of this model acts as
an infinite reservoir able to absorb or generate as much energy
as required to keep the temperature at the specified value.
</p>
</html>"),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, 100}, {100, -100}}, lineColor = {0, 0, 0}, pattern = LinePattern.None, fillColor = {159, 159, 223}, fillPattern = FillPattern.Backward), Text(extent = {{0, 0}, {-100, -100}}, lineColor = {0, 0, 0}, textString = "K"), Line(points = {{-102, 0}, {64, 0}}, color = {191, 0, 0}, thickness = 0.5), Polygon(points = {{52, -20}, {52, 20}, {90, 0}, {52, -20}}, lineColor = {191, 0, 0}, fillColor = {191, 0, 0}, fillPattern = FillPattern.Solid)}));
        end PrescribedTemperature;

        model FixedHeatFlow "Fixed heat flow boundary condition"
          parameter Modelica.SIunits.HeatFlowRate Q_flow "Fixed heat flow rate at port";
          parameter Modelica.SIunits.Temperature T_ref = 293.15 "Reference temperature";
          parameter Modelica.SIunits.LinearTemperatureCoefficient alpha = 0 "Temperature coefficient of heat flow rate";
          Interfaces.HeatPort_b port annotation(
            Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
        equation
          port.Q_flow = -Q_flow * (1 + alpha * (port.T - T_ref));
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-150, 100}, {150, 60}}, textString = "%name", lineColor = {0, 0, 255}), Text(extent = {{-150, -55}, {150, -85}}, lineColor = {0, 0, 0}, textString = "Q_flow=%Q_flow"), Line(points = {{-100, -20}, {48, -20}}, color = {191, 0, 0}, thickness = 0.5), Line(points = {{-100, 20}, {46, 20}}, color = {191, 0, 0}, thickness = 0.5), Polygon(points = {{40, 0}, {40, 40}, {70, 20}, {40, 0}}, lineColor = {191, 0, 0}, fillColor = {191, 0, 0}, fillPattern = FillPattern.Solid), Polygon(points = {{40, -40}, {40, 0}, {70, -20}, {40, -40}}, lineColor = {191, 0, 0}, fillColor = {191, 0, 0}, fillPattern = FillPattern.Solid), Rectangle(extent = {{70, 40}, {90, -40}}, lineColor = {191, 0, 0}, fillColor = {191, 0, 0}, fillPattern = FillPattern.Solid)}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-100, 40}, {0, -36}}, lineColor = {0, 0, 0}, textString = "Q_flow=const."), Line(points = {{-48, -20}, {60, -20}}, color = {191, 0, 0}, thickness = 0.5), Line(points = {{-48, 20}, {60, 20}}, color = {191, 0, 0}, thickness = 0.5), Polygon(points = {{60, 0}, {60, 40}, {90, 20}, {60, 0}}, lineColor = {191, 0, 0}, fillColor = {191, 0, 0}, fillPattern = FillPattern.Solid), Polygon(points = {{60, -40}, {60, 0}, {90, -20}, {60, -40}}, lineColor = {191, 0, 0}, fillColor = {191, 0, 0}, fillPattern = FillPattern.Solid)}),
            Documentation(info = "<HTML>
<p>
This model allows a specified amount of heat flow rate to be \"injected\"
into a thermal system at a given port.  The constant amount of heat
flow rate Q_flow is given as a parameter. The heat flows into the
component to which the component FixedHeatFlow is connected,
if parameter Q_flow is positive.
</p>
<p>
If parameter alpha is &lt;&gt; 0, the heat flow is multiplied by (1 + alpha*(port.T - T_ref))
in order to simulate temperature dependent losses (which are given with respect to reference temperature T_ref).
</p>
</html>"));
        end FixedHeatFlow;

        model PrescribedHeatFlow "Prescribed heat flow boundary condition"
          parameter Modelica.SIunits.Temperature T_ref = 293.15 "Reference temperature";
          parameter Modelica.SIunits.LinearTemperatureCoefficient alpha = 0 "Temperature coefficient of heat flow rate";
          Modelica.Blocks.Interfaces.RealInput Q_flow(unit = "W") annotation(
            Placement(transformation(origin = {-100, 0}, extent = {{20, -20}, {-20, 20}}, rotation = 180)));
          Interfaces.HeatPort_b port annotation(
            Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
        equation
          port.Q_flow = -Q_flow * (1 + alpha * (port.T - T_ref));
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-60, -20}, {40, -20}}, color = {191, 0, 0}, thickness = 0.5), Line(points = {{-60, 20}, {40, 20}}, color = {191, 0, 0}, thickness = 0.5), Line(points = {{-80, 0}, {-60, -20}}, color = {191, 0, 0}, thickness = 0.5), Line(points = {{-80, 0}, {-60, 20}}, color = {191, 0, 0}, thickness = 0.5), Polygon(points = {{40, 0}, {40, 40}, {70, 20}, {40, 0}}, lineColor = {191, 0, 0}, fillColor = {191, 0, 0}, fillPattern = FillPattern.Solid), Polygon(points = {{40, -40}, {40, 0}, {70, -20}, {40, -40}}, lineColor = {191, 0, 0}, fillColor = {191, 0, 0}, fillPattern = FillPattern.Solid), Rectangle(extent = {{70, 40}, {90, -40}}, lineColor = {191, 0, 0}, fillColor = {191, 0, 0}, fillPattern = FillPattern.Solid), Text(extent = {{-150, 100}, {150, 60}}, textString = "%name", lineColor = {0, 0, 255})}),
            Documentation(info = "<HTML>
<p>
This model allows a specified amount of heat flow rate to be \"injected\"
into a thermal system at a given port.  The amount of heat
is given by the input signal Q_flow into the model. The heat flows into the
component to which the component PrescribedHeatFlow is connected,
if the input signal is positive.
</p>
<p>
If parameter alpha is &lt;&gt; 0, the heat flow is multiplied by (1 + alpha*(port.T - T_ref))
in order to simulate temperature dependent losses (which are given with respect to reference temperature T_ref).
</p>
</html>"),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-60, -20}, {68, -20}}, color = {191, 0, 0}, thickness = 0.5), Line(points = {{-60, 20}, {68, 20}}, color = {191, 0, 0}, thickness = 0.5), Line(points = {{-80, 0}, {-60, -20}}, color = {191, 0, 0}, thickness = 0.5), Line(points = {{-80, 0}, {-60, 20}}, color = {191, 0, 0}, thickness = 0.5), Polygon(points = {{60, 0}, {60, 40}, {90, 20}, {60, 0}}, lineColor = {191, 0, 0}, fillColor = {191, 0, 0}, fillPattern = FillPattern.Solid), Polygon(points = {{60, -40}, {60, 0}, {90, -20}, {60, -40}}, lineColor = {191, 0, 0}, fillColor = {191, 0, 0}, fillPattern = FillPattern.Solid)}));
        end PrescribedHeatFlow;
        annotation(
          Documentation(info = "<html>

</html>"));
      end Sources;

      package Celsius "Components with Celsius input and/or output"
        extends Modelica.Icons.VariantsPackage;

        model ToKelvin "Conversion block from degCelsius to Kelvin"
          Modelica.Blocks.Interfaces.RealInput Celsius(unit = "degC") annotation(
            Placement(transformation(extent = {{-140, -20}, {-100, 20}}, rotation = 0)));
          Modelica.Blocks.Interfaces.RealOutput Kelvin(unit = "K") annotation(
            Placement(transformation(extent = {{100, -10}, {120, 10}}, rotation = 0)));
        equation
          Kelvin = Modelica.SIunits.Conversions.from_degC(Celsius);
          annotation(
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-40, 40}, {40, -40}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-40, -50}, {-99, -99}}, lineColor = {0, 0, 0}, lineThickness = 1, textString = "degC"), Text(extent = {{100, -47}, {44, -100}}, lineColor = {0, 0, 0}, textString = "K"), Line(points = {{-100, 0}, {-40, 0}}, color = {0, 0, 255}), Line(points = {{41, 0}, {100, 0}}, color = {0, 0, 255})}),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-40, 40}, {40, -40}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Text(extent = {{124, -38}, {52, -94}}, lineColor = {0, 0, 0}, textString = "K"), Text(extent = {{-62, -38}, {-141, -97}}, lineColor = {0, 0, 0}, lineThickness = 1, textString = "degC"), Line(points = {{-41, 0}, {-100, 0}}, color = {0, 0, 255}), Line(points = {{100, 0}, {40, 0}}, color = {0, 0, 255}), Text(extent = {{-150, 90}, {150, 50}}, textString = "%name", lineColor = {0, 0, 255})}),
            Documentation(info = "<HTML>
<p>
This component converts an input signal from Celsius to Kelvin
and provide is as output signal.
</p>
</html>"));
        end ToKelvin;

        model FromKelvin "Conversion from Kelvin to degCelsius"
          Modelica.Blocks.Interfaces.RealInput Kelvin(unit = "K") annotation(
            Placement(transformation(extent = {{-140, -20}, {-100, 20}}, rotation = 0)));
          Modelica.Blocks.Interfaces.RealOutput Celsius(unit = "degC") annotation(
            Placement(transformation(extent = {{100, -10}, {120, 10}}, rotation = 0)));
        equation
          Celsius = Modelica.SIunits.Conversions.to_degC(Kelvin);
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-150, 90}, {150, 50}}, textString = "%name", lineColor = {0, 0, 255}), Ellipse(extent = {{-40, 40}, {40, -40}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-72, -46}, {-152, -108}}, lineColor = {0, 0, 0}, lineThickness = 1, textString = "K"), Text(extent = {{122, -48}, {38, -109}}, lineColor = {0, 0, 0}, textString = "degC"), Line(points = {{-40, 0}, {-100, 0}}, color = {0, 0, 255}), Line(points = {{40, 0}, {100, 0}}, color = {0, 0, 255})}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-40, 40}, {40, -40}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-42, -41}, {-101, -98}}, lineColor = {0, 0, 0}, lineThickness = 1, textString = "K"), Text(extent = {{100, -40}, {30, -100}}, lineColor = {0, 0, 0}, textString = "degC"), Line(points = {{-100, 0}, {-40, 0}}, color = {0, 0, 255}), Line(points = {{40, 0}, {100, 0}}, color = {0, 0, 255})}),
            Documentation(info = "<HTML>
<p>
This component converts an input signal from Kelvin to Celsius
and provides is as output signal.
</p>
</html>"));
        end FromKelvin;

        model FixedTemperature "Fixed temperature boundary condition in degree Celsius"
          parameter Modelica.SIunits.Conversions.NonSIunits.Temperature_degC T "Fixed Temperature at the port";
          Interfaces.HeatPort_b port annotation(
            Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
        equation
          port.T = Modelica.SIunits.Conversions.from_degC(T);
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-150, 150}, {150, 110}}, textString = "%name", lineColor = {0, 0, 255}), Rectangle(extent = {{-100, 100}, {100, -100}}, lineColor = {0, 0, 0}, pattern = LinePattern.None, fillColor = {159, 159, 223}, fillPattern = FillPattern.Backward), Text(extent = {{0, 0}, {-100, -100}}, lineColor = {0, 0, 0}, textString = "degC"), Polygon(points = {{52, -20}, {52, 20}, {90, 0}, {52, -20}}, lineColor = {191, 0, 0}, fillColor = {191, 0, 0}, fillPattern = FillPattern.Solid), Text(extent = {{-150, -110}, {150, -140}}, lineColor = {0, 0, 0}, textString = "T=%T"), Line(points = {{-42, 0}, {66, 0}}, color = {191, 0, 0}, thickness = 0.5)}),
            Documentation(info = "<HTML>
<p>
This model defines a fixed temperature T at its port in [degC],
i.e., it defines a fixed temperature as a boundary condition.
</p>
</html>"),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, 100}, {100, -100}}, lineColor = {0, 0, 0}, pattern = LinePattern.None, fillColor = {159, 159, 223}, fillPattern = FillPattern.Backward), Line(points = {{-42, 0}, {66, 0}}, color = {191, 0, 0}, thickness = 0.5), Text(extent = {{0, 0}, {-100, -100}}, lineColor = {0, 0, 0}, textString = "degC"), Polygon(points = {{52, -20}, {52, 20}, {90, 0}, {52, -20}}, lineColor = {191, 0, 0}, fillColor = {191, 0, 0}, fillPattern = FillPattern.Solid)}));
        end FixedTemperature;

        model PrescribedTemperature "Variable temperature boundary condition in degCelsius"
          Interfaces.HeatPort_b port annotation(
            Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
          Modelica.Blocks.Interfaces.RealInput T(unit = "degC") annotation(
            Placement(transformation(extent = {{-140, -20}, {-100, 20}}, rotation = 0)));
        equation
          port.T = Modelica.SIunits.Conversions.from_degC(T);
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, 100}, {100, -100}}, lineColor = {0, 0, 0}, pattern = LinePattern.None, fillColor = {159, 159, 223}, fillPattern = FillPattern.Backward), Line(points = {{-102, 0}, {64, 0}}, color = {191, 0, 0}, thickness = 0.5), Text(extent = {{0, 0}, {-100, -100}}, lineColor = {0, 0, 0}, textString = "degC"), Text(extent = {{-150, 150}, {150, 110}}, textString = "%name", lineColor = {0, 0, 255}), Line(points = {{-102, 0}, {64, 0}}, color = {191, 0, 0}, thickness = 0.5), Polygon(points = {{52, -20}, {52, 20}, {90, 0}, {52, -20}}, lineColor = {191, 0, 0}, fillColor = {191, 0, 0}, fillPattern = FillPattern.Solid)}),
            Documentation(info = "<HTML>
<p>
This model represents a variable temperature boundary condition
The temperature value in [degC] is given by the input signal
to the model. The effect is that an instance of this model acts as
an infinite reservoir able to absorb or generate as much energy
as required to keep the temperature at the specified value.
</p>
</html>"),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, 100}, {100, -100}}, lineColor = {0, 0, 0}, pattern = LinePattern.None, fillColor = {159, 159, 223}, fillPattern = FillPattern.Backward), Line(points = {{-102, 0}, {64, 0}}, color = {191, 0, 0}, thickness = 0.5), Text(extent = {{0, 0}, {-100, -100}}, lineColor = {0, 0, 0}, textString = "degC"), Polygon(points = {{52, -20}, {52, 20}, {90, 0}, {52, -20}}, lineColor = {191, 0, 0}, fillColor = {191, 0, 0}, fillPattern = FillPattern.Solid)}));
        end PrescribedTemperature;

        model TemperatureSensor "Absolute temperature sensor in degCelsius"
          Modelica.Blocks.Interfaces.RealOutput T(unit = "degC") "Absolute temperature in degree Celsius as output signal" annotation(
            Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
          Interfaces.HeatPort_a port annotation(
            Placement(transformation(extent = {{-110, -10}, {-90, 10}}, rotation = 0)));
        equation
          T = Modelica.SIunits.Conversions.to_degC(port.T);
          port.Q_flow = 0;
          annotation(
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-20, -98}, {20, -60}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {191, 0, 0}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-12, 40}, {12, -68}}, lineColor = {191, 0, 0}, fillColor = {191, 0, 0}, fillPattern = FillPattern.Solid), Line(points = {{12, 0}, {90, 0}}, color = {0, 0, 255}), Line(points = {{-94, 0}, {-12, 0}}, color = {191, 0, 0}), Polygon(points = {{-12, 40}, {-12, 80}, {-10, 86}, {-6, 88}, {0, 90}, {6, 88}, {10, 86}, {12, 80}, {12, 40}, {-12, 40}}, lineColor = {0, 0, 0}, lineThickness = 0.5), Line(points = {{-12, 40}, {-12, -64}}, color = {0, 0, 0}, thickness = 0.5), Line(points = {{12, 40}, {12, -64}}, color = {0, 0, 0}, thickness = 0.5), Line(points = {{-40, -20}, {-12, -20}}, color = {0, 0, 0}), Line(points = {{-40, 20}, {-12, 20}}, color = {0, 0, 0}), Line(points = {{-40, 60}, {-12, 60}}, color = {0, 0, 0}), Text(extent = {{102, -22}, {60, -74}}, lineColor = {0, 0, 0}, textString = "degC")}),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-20, -98}, {20, -60}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {191, 0, 0}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-12, 40}, {12, -68}}, lineColor = {191, 0, 0}, fillColor = {191, 0, 0}, fillPattern = FillPattern.Solid), Line(points = {{12, 0}, {90, 0}}, color = {0, 0, 255}), Line(points = {{-90, 0}, {-12, 0}}, color = {191, 0, 0}), Polygon(points = {{-12, 40}, {-12, 80}, {-10, 86}, {-6, 88}, {0, 90}, {6, 88}, {10, 86}, {12, 80}, {12, 40}, {-12, 40}}, lineColor = {0, 0, 0}, lineThickness = 0.5), Line(points = {{-12, 40}, {-12, -64}}, color = {0, 0, 0}, thickness = 0.5), Line(points = {{12, 40}, {12, -64}}, color = {0, 0, 0}, thickness = 0.5), Line(points = {{-40, -20}, {-12, -20}}, color = {0, 0, 0}), Line(points = {{-40, 20}, {-12, 20}}, color = {0, 0, 0}), Line(points = {{-40, 60}, {-12, 60}}, color = {0, 0, 0}), Text(extent = {{150, -22}, {30, -92}}, lineColor = {0, 0, 0}, textString = "degC"), Text(extent = {{-150, 135}, {150, 95}}, textString = "%name", lineColor = {0, 0, 255})}),
            Documentation(info = "<HTML>
<p>
This is an ideal absolute temperature sensor which returns
the temperature of the connected port in Celsius as an output
signal.  The sensor itself has no thermal interaction with
whatever it is connected to.  Furthermore, no
thermocouple-like lags are associated with this
sensor model.
</p>
</html>"));
        end TemperatureSensor;
        annotation(
          Documentation(info = "<HTML>
<p>
The components of this package are provided for the convenience of
people working mostly with Celsius units, since all models
in package HeatTransfer are based on Kelvin units.
</p>
<p>
Note, that in package SIunits.Conversions, functions are provided
to convert between the units Kelvin, degree Celsius, degree Fahrenheit,
and degree Rankine. These functions allow, e.g., a direct conversion
of units at all places where Kelvin is required as parameter.
Example:
</p>
<pre>
    <b>import</b> SIunits.Conversions.*;
    Modelica.Thermal.HeatTransfer.HeatCapacitor C(T0 = from_degC(20));
</pre>
</html>"));
      end Celsius;

      package Fahrenheit "Components with Fahrenheit input and/or output"
        extends Modelica.Icons.VariantsPackage;

        model ToKelvin "Conversion block from degFahrenheit to Kelvin"
          Modelica.Blocks.Interfaces.RealInput Fahrenheit(unit = "degF") annotation(
            Placement(transformation(extent = {{-140, -20}, {-100, 20}}, rotation = 0)));
          Modelica.Blocks.Interfaces.RealOutput Kelvin(unit = "K") annotation(
            Placement(transformation(extent = {{100, -10}, {120, 10}}, rotation = 0)));
        equation
          Kelvin = Modelica.SIunits.Conversions.from_degF(Fahrenheit);
          annotation(
            Diagram(graphics = {Ellipse(extent = {{-40, 40}, {40, -40}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-40, -50}, {-99, -99}}, lineColor = {0, 0, 0}, lineThickness = 1, textString = "degF"), Text(extent = {{100, -47}, {44, -100}}, lineColor = {0, 0, 0}, textString = "K"), Line(points = {{-100, 0}, {-40, 0}}, color = {0, 0, 255}), Line(points = {{41, 0}, {100, 0}}, color = {0, 0, 255})}),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-40, 40}, {40, -40}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Text(extent = {{130, -42}, {38, -94}}, lineColor = {0, 0, 0}, textString = "K"), Line(points = {{-41, 0}, {-100, 0}}, color = {0, 0, 255}), Line(points = {{100, 0}, {40, 0}}, color = {0, 0, 255}), Text(extent = {{-150, 90}, {150, 50}}, textString = "%name", lineColor = {0, 0, 255}), Text(extent = {{-42, -44}, {-153, -91}}, lineColor = {0, 0, 0}, lineThickness = 1, textString = "degF")}),
            Documentation(info = "<HTML>
<p>
This component converts a input signal from degree Fahrenheit to Kelvin
and provides is as output signal.
</p>
</html>"));
        end ToKelvin;

        model FromKelvin "Conversion from Kelvin to degFahrenheit"
          parameter Integer n = 1 "Only kept for backwards compatibility (parameter is not used in the model and will be removed in a future version)" annotation(
            Dialog(enable = false));
          Modelica.Blocks.Interfaces.RealInput Kelvin(unit = "K") annotation(
            Placement(transformation(extent = {{-140, -20}, {-100, 20}}, rotation = 0)));
          Modelica.Blocks.Interfaces.RealOutput Fahrenheit(unit = "degF") annotation(
            Placement(transformation(extent = {{100, -10}, {120, 10}}, rotation = 0)));
        equation
          Fahrenheit = Modelica.SIunits.Conversions.to_degF(Kelvin);
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-150, 90}, {150, 50}}, textString = "%name", lineColor = {0, 0, 255}), Ellipse(extent = {{-40, 40}, {40, -40}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-60, -52}, {-144, -96}}, lineColor = {0, 0, 0}, lineThickness = 1, textString = "K"), Line(points = {{-40, 0}, {-100, 0}}, color = {0, 0, 255}), Line(points = {{40, 0}, {100, 0}}, color = {0, 0, 255}), Text(extent = {{138, -52}, {27, -99}}, lineColor = {0, 0, 0}, lineThickness = 1, textString = "degF")}),
            Diagram(graphics = {Ellipse(extent = {{-40, 40}, {40, -40}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-42, -41}, {-101, -98}}, lineColor = {0, 0, 0}, lineThickness = 1, textString = "K"), Text(extent = {{100, -40}, {30, -100}}, lineColor = {0, 0, 0}, textString = "degF"), Line(points = {{-100, 0}, {-40, 0}}, color = {0, 0, 255}), Line(points = {{40, 0}, {100, 0}}, color = {0, 0, 255})}),
            Documentation(info = "<HTML>
<p>
This component converts all input signals from Kelvin to Fahrenheit
and provides them as output signals.
</p>
</html>"));
        end FromKelvin;

        model FixedTemperature "Fixed temperature boundary condition in degFahrenheit"
          parameter Modelica.SIunits.Conversions.NonSIunits.Temperature_degF T "Fixed Temperature at the port";
          Interfaces.HeatPort_b port annotation(
            Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
        equation
          port.T = Modelica.SIunits.Conversions.from_degF(T);
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-150, 150}, {150, 110}}, textString = "%name", lineColor = {0, 0, 255}), Rectangle(extent = {{-100, 100}, {100, -100}}, lineColor = {0, 0, 0}, pattern = LinePattern.None, fillColor = {159, 159, 223}, fillPattern = FillPattern.Backward), Text(extent = {{0, 0}, {-100, -100}}, lineColor = {0, 0, 0}, textString = "degF"), Polygon(points = {{52, -20}, {52, 20}, {90, 0}, {52, -20}}, lineColor = {191, 0, 0}, fillColor = {191, 0, 0}, fillPattern = FillPattern.Solid), Text(extent = {{-150, -110}, {150, -140}}, lineColor = {0, 0, 0}, textString = "T=%T"), Line(points = {{-42, 0}, {66, 0}}, color = {191, 0, 0}, thickness = 0.5)}),
            Documentation(info = "<HTML>
<p>
This model defines a fixed temperature T at its port in [degF],
i.e., it defines a fixed temperature as a boundary condition.
</p>
</html>"),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, 100}, {100, -100}}, lineColor = {0, 0, 0}, pattern = LinePattern.None, fillColor = {159, 159, 223}, fillPattern = FillPattern.Backward), Line(points = {{-42, 0}, {66, 0}}, color = {191, 0, 0}, thickness = 0.5), Text(extent = {{0, 0}, {-100, -100}}, lineColor = {0, 0, 0}, textString = "degF"), Polygon(points = {{52, -20}, {52, 20}, {90, 0}, {52, -20}}, lineColor = {191, 0, 0}, fillColor = {191, 0, 0}, fillPattern = FillPattern.Solid)}));
        end FixedTemperature;

        model PrescribedTemperature "Variable temperature boundary condition in degFahrenheit"
          Interfaces.HeatPort_b port annotation(
            Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
          Modelica.Blocks.Interfaces.RealInput T(unit = "degF") annotation(
            Placement(transformation(extent = {{-140, -20}, {-100, 20}}, rotation = 0)));
        equation
          port.T = Modelica.SIunits.Conversions.from_degF(T);
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, 100}, {100, -100}}, lineColor = {0, 0, 0}, pattern = LinePattern.None, fillColor = {159, 159, 223}, fillPattern = FillPattern.Backward), Line(points = {{-102, 0}, {64, 0}}, color = {191, 0, 0}, thickness = 0.5), Text(extent = {{0, 0}, {-100, -100}}, lineColor = {0, 0, 0}, textString = "degF"), Text(extent = {{-150, 150}, {150, 110}}, textString = "%name", lineColor = {0, 0, 255}), Line(points = {{-102, 0}, {64, 0}}, color = {191, 0, 0}, thickness = 0.5), Polygon(points = {{52, -20}, {52, 20}, {90, 0}, {52, -20}}, lineColor = {191, 0, 0}, fillColor = {191, 0, 0}, fillPattern = FillPattern.Solid)}),
            Documentation(info = "<HTML>
<p>
This model represents a variable temperature boundary condition
The temperature value in [degF] is given by the input signal
to the model. The effect is that an instance of this model acts as
an infinite reservoir able to absorb or generate as much energy
as required to keep the temperature at the specified value.
</p>
</html>"),
            Diagram(graphics = {Rectangle(extent = {{-100, 100}, {100, -100}}, lineColor = {0, 0, 0}, pattern = LinePattern.None, fillColor = {159, 159, 223}, fillPattern = FillPattern.Backward), Line(points = {{-102, 0}, {64, 0}}, color = {191, 0, 0}, thickness = 0.5), Text(extent = {{0, 0}, {-100, -100}}, lineColor = {0, 0, 0}, textString = "degF"), Polygon(points = {{52, -20}, {52, 20}, {90, 0}, {52, -20}}, lineColor = {191, 0, 0}, fillColor = {191, 0, 0}, fillPattern = FillPattern.Solid)}));
        end PrescribedTemperature;

        model TemperatureSensor "Absolute temperature sensor in degFahrenheit"
          Modelica.Blocks.Interfaces.RealOutput T annotation(
            Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
          Interfaces.HeatPort_a port annotation(
            Placement(transformation(extent = {{-110, -10}, {-90, 10}}, rotation = 0)));
        equation
          T = Modelica.SIunits.Conversions.to_degF(port.T);
          port.Q_flow = 0;
          annotation(
            Diagram(graphics = {Ellipse(extent = {{-20, -98}, {20, -60}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {191, 0, 0}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-12, 40}, {12, -68}}, lineColor = {191, 0, 0}, fillColor = {191, 0, 0}, fillPattern = FillPattern.Solid), Line(points = {{12, 0}, {90, 0}}, color = {0, 0, 255}), Line(points = {{-94, 0}, {-12, 0}}, color = {191, 0, 0}), Polygon(points = {{-12, 40}, {-12, 80}, {-10, 86}, {-6, 88}, {0, 90}, {6, 88}, {10, 86}, {12, 80}, {12, 40}, {-12, 40}}, lineColor = {0, 0, 0}, lineThickness = 0.5), Line(points = {{-12, 40}, {-12, -64}}, color = {0, 0, 0}, thickness = 0.5), Line(points = {{12, 40}, {12, -64}}, color = {0, 0, 0}, thickness = 0.5), Line(points = {{-40, -20}, {-12, -20}}, color = {0, 0, 0}), Line(points = {{-40, 20}, {-12, 20}}, color = {0, 0, 0}), Line(points = {{-40, 60}, {-12, 60}}, color = {0, 0, 0}), Text(extent = {{102, -22}, {60, -74}}, lineColor = {0, 0, 0}, textString = "degF")}),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-20, -98}, {20, -60}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {191, 0, 0}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-12, 40}, {12, -68}}, lineColor = {191, 0, 0}, fillColor = {191, 0, 0}, fillPattern = FillPattern.Solid), Line(points = {{12, 0}, {90, 0}}, color = {0, 0, 255}), Line(points = {{-90, 0}, {-12, 0}}, color = {191, 0, 0}), Polygon(points = {{-12, 40}, {-12, 80}, {-10, 86}, {-6, 88}, {0, 90}, {6, 88}, {10, 86}, {12, 80}, {12, 40}, {-12, 40}}, lineColor = {0, 0, 0}, lineThickness = 0.5), Line(points = {{-12, 40}, {-12, -64}}, color = {0, 0, 0}, thickness = 0.5), Line(points = {{12, 40}, {12, -64}}, color = {0, 0, 0}, thickness = 0.5), Line(points = {{-40, -20}, {-12, -20}}, color = {0, 0, 0}), Line(points = {{-40, 20}, {-12, 20}}, color = {0, 0, 0}), Line(points = {{-40, 60}, {-12, 60}}, color = {0, 0, 0}), Text(extent = {{150, -22}, {30, -92}}, lineColor = {0, 0, 0}, textString = "degF"), Text(extent = {{-150, 135}, {150, 95}}, textString = "%name", lineColor = {0, 0, 255})}),
            Documentation(info = "<HTML>
<p>
This is an ideal absolute temperature sensor which returns
the temperature of the connected port in Fahrenheit as an output
signal.  The sensor itself has no thermal interaction with
whatever it is connected to.  Furthermore, no
thermocouple-like lags are associated with this
sensor model.
</p>
</html>"));
        end TemperatureSensor;
        annotation(
          Documentation(info = "<HTML>
<p>
The components of this package are provided for the convenience of
people working mostly with Fahrenheit units, since all models
in package HeatTransfer are based on Kelvin units.
</p>
<p>
Note, that in package SIunits.Conversions, functions are provided
to convert between the units Kelvin, degree Celsius, degree Fahrenheit
and degree Rankine. These functions allow, e.g., a direct conversion
of units at all places where Kelvin is required as parameter.
Example:
</p>
<pre>
    <b>import</b> SIunits.Conversions.*;
    Modelica.Thermal.HeatTransfer.HeatCapacitor C(T0 = from_degF(70));
</pre>
</html>"));
      end Fahrenheit;

      package Rankine "Components with Rankine input and/or output"
        extends Modelica.Icons.VariantsPackage;

        model ToKelvin "Conversion block from degRankine to Kelvin"
          parameter Integer n = 1 "Only kept for backwards compatibility (parameter is not used in the model and will be removed in a future version)" annotation(
            Dialog(enable = false));
          Modelica.Blocks.Interfaces.RealInput Rankine(unit = "degRk") annotation(
            Placement(transformation(extent = {{-140, -20}, {-100, 20}}, rotation = 0)));
          Modelica.Blocks.Interfaces.RealOutput Kelvin(unit = "K") annotation(
            Placement(transformation(extent = {{100, -10}, {120, 10}}, rotation = 0)));
        equation
          Kelvin = Modelica.SIunits.Conversions.from_degRk(Rankine);
          annotation(
            Diagram(graphics = {Ellipse(extent = {{-40, 40}, {40, -40}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-40, -50}, {-99, -99}}, lineColor = {0, 0, 0}, lineThickness = 1, textString = "degRk"), Text(extent = {{100, -47}, {44, -100}}, lineColor = {0, 0, 0}, textString = "K"), Line(points = {{-100, 0}, {-40, 0}}, color = {0, 0, 255}), Line(points = {{41, 0}, {100, 0}}, color = {0, 0, 255})}),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-40, 40}, {40, -40}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Text(extent = {{140, -38}, {56, -94}}, lineColor = {0, 0, 0}, textString = "K"), Text(extent = {{-44, -42}, {-155, -89}}, lineColor = {0, 0, 0}, lineThickness = 1, textString = "degRk"), Line(points = {{-41, 0}, {-100, 0}}, color = {0, 0, 255}), Line(points = {{100, 0}, {40, 0}}, color = {0, 0, 255}), Text(extent = {{-150, 90}, {150, 50}}, textString = "%name", lineColor = {0, 0, 255})}),
            Documentation(info = "<HTML>
<p>
This component converts all input signals from degree Rankine to Kelvin
and provides them as output signals.
</p>
</html>"));
        end ToKelvin;

        model FromKelvin "Conversion from Kelvin to degRankine"
          parameter Integer n = 1 "Only kept for backwards compatibility (parameter is not used in the model and will be removed in a future version)" annotation(
            Dialog(enable = false));
          Modelica.Blocks.Interfaces.RealInput Kelvin(unit = "K") annotation(
            Placement(transformation(extent = {{-140, -20}, {-100, 20}}, rotation = 0)));
          Modelica.Blocks.Interfaces.RealOutput Rankine(unit = "degRk") annotation(
            Placement(transformation(extent = {{100, -10}, {120, 10}}, rotation = 0)));
        equation
          Rankine = Modelica.SIunits.Conversions.to_degRk(Kelvin);
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-150, 90}, {150, 50}}, textString = "%name", lineColor = {0, 0, 255}), Ellipse(extent = {{-40, 40}, {40, -40}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-54, -38}, {-162, -90}}, lineColor = {0, 0, 0}, lineThickness = 1, textString = "K"), Text(extent = {{142, -48}, {26, -91}}, lineColor = {0, 0, 0}, textString = "degRk"), Line(points = {{-40, 0}, {-100, 0}}, color = {0, 0, 255}), Line(points = {{40, 0}, {100, 0}}, color = {0, 0, 255})}),
            Diagram(graphics = {Ellipse(extent = {{-40, 40}, {40, -40}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-42, -41}, {-101, -98}}, lineColor = {0, 0, 0}, lineThickness = 1, textString = "K"), Text(extent = {{100, -40}, {30, -100}}, lineColor = {0, 0, 0}, textString = "degRk"), Line(points = {{-100, 0}, {-40, 0}}, color = {0, 0, 255}), Line(points = {{40, 0}, {100, 0}}, color = {0, 0, 255})}),
            Documentation(info = "<HTML>
<p>
This component converts all input signals from Kelvin to Rankine
and provides them as output signals.
</p>
</html>"));
        end FromKelvin;

        model FixedTemperature "Fixed temperature boundary condition in degRankine"
          parameter Modelica.SIunits.Conversions.NonSIunits.Temperature_degRk T "Fixed Temperature at the port";
          Interfaces.HeatPort_b port annotation(
            Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
        equation
          port.T = Modelica.SIunits.Conversions.from_degRk(T);
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-118, 165}, {122, 105}}, textString = "%name", lineColor = {0, 0, 255}), Rectangle(extent = {{-100, 100}, {100, -100}}, lineColor = {0, 0, 0}, pattern = LinePattern.None, fillColor = {159, 159, 223}, fillPattern = FillPattern.Backward), Text(extent = {{48, -22}, {-100, -86}}, lineColor = {0, 0, 0}, textString = "degRk"), Polygon(points = {{52, -20}, {52, 20}, {90, 0}, {52, -20}}, lineColor = {191, 0, 0}, fillColor = {191, 0, 0}, fillPattern = FillPattern.Solid), Text(extent = {{-150, -110}, {150, -150}}, lineColor = {0, 0, 0}, textString = "T=%T"), Line(points = {{-42, 0}, {66, 0}}, color = {191, 0, 0}, thickness = 0.5)}),
            Documentation(info = "<HTML>
<p>
This model defines a fixed temperature T at its port in degree Rankine,
[degRk], i.e., it defines a fixed temperature as a boundary condition.
</p>
</html>"),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, 100}, {100, -100}}, lineColor = {0, 0, 0}, pattern = LinePattern.None, fillColor = {159, 159, 223}, fillPattern = FillPattern.Backward), Line(points = {{-42, 0}, {66, 0}}, color = {191, 0, 0}, thickness = 0.5), Text(extent = {{0, 0}, {-100, -100}}, lineColor = {0, 0, 0}, textString = "degRk"), Polygon(points = {{52, -20}, {52, 20}, {90, 0}, {52, -20}}, lineColor = {191, 0, 0}, fillColor = {191, 0, 0}, fillPattern = FillPattern.Solid)}));
        end FixedTemperature;

        model PrescribedTemperature "Variable temperature boundary condition in degRankine"
          Interfaces.HeatPort_b port annotation(
            Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
          Modelica.Blocks.Interfaces.RealInput T(unit = "degRk") annotation(
            Placement(transformation(extent = {{-140, -20}, {-100, 20}}, rotation = 0)));
        equation
          port.T = Modelica.SIunits.Conversions.from_degRk(T);
          annotation(
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, 100}, {100, -100}}, lineColor = {0, 0, 0}, pattern = LinePattern.None, fillColor = {159, 159, 223}, fillPattern = FillPattern.Backward), Line(points = {{-102, 0}, {64, 0}}, color = {191, 0, 0}, thickness = 0.5), Text(extent = {{56, -34}, {-100, -100}}, lineColor = {0, 0, 0}, textString = "degRk"), Text(extent = {{-150, 150}, {150, 110}}, textString = "%name", lineColor = {0, 0, 255}), Line(points = {{-102, 0}, {64, 0}}, color = {191, 0, 0}, thickness = 0.5), Polygon(points = {{52, -20}, {52, 20}, {90, 0}, {52, -20}}, lineColor = {191, 0, 0}, fillColor = {191, 0, 0}, fillPattern = FillPattern.Solid)}),
            Documentation(info = "<HTML>
<p>
This model represents a variable temperature boundary condition
The temperature value in degree Rankine, [degRk] is given by the input signal
to the model. The effect is that an instance of this model acts as
an infinite reservoir able to absorb or generate as much energy
as required to keep the temperature at the specified value.
</p>
</html>"),
            Diagram(graphics = {Rectangle(extent = {{-100, 100}, {100, -100}}, lineColor = {0, 0, 0}, pattern = LinePattern.None, fillColor = {159, 159, 223}, fillPattern = FillPattern.Backward), Line(points = {{-102, 0}, {64, 0}}, color = {191, 0, 0}, thickness = 0.5), Text(extent = {{0, 0}, {-100, -100}}, lineColor = {0, 0, 0}, textString = "degRk"), Polygon(points = {{52, -20}, {52, 20}, {90, 0}, {52, -20}}, lineColor = {191, 0, 0}, fillColor = {191, 0, 0}, fillPattern = FillPattern.Solid)}));
        end PrescribedTemperature;

        model TemperatureSensor "Absolute temperature sensor in degRankine"
          Modelica.Blocks.Interfaces.RealOutput T annotation(
            Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
          Interfaces.HeatPort_a port annotation(
            Placement(transformation(extent = {{-110, -10}, {-90, 10}}, rotation = 0)));
        equation
          T = Modelica.SIunits.Conversions.to_degRk(port.T);
          port.Q_flow = 0;
          annotation(
            Diagram(graphics = {Ellipse(extent = {{-20, -98}, {20, -60}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {191, 0, 0}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-12, 40}, {12, -68}}, lineColor = {191, 0, 0}, fillColor = {191, 0, 0}, fillPattern = FillPattern.Solid), Line(points = {{12, 0}, {90, 0}}, color = {0, 0, 255}), Line(points = {{-94, 0}, {-12, 0}}, color = {191, 0, 0}), Polygon(points = {{-12, 40}, {-12, 80}, {-10, 86}, {-6, 88}, {0, 90}, {6, 88}, {10, 86}, {12, 80}, {12, 40}, {-12, 40}}, lineColor = {0, 0, 0}, lineThickness = 0.5), Line(points = {{-12, 40}, {-12, -64}}, color = {0, 0, 0}, thickness = 0.5), Line(points = {{12, 40}, {12, -64}}, color = {0, 0, 0}, thickness = 0.5), Line(points = {{-40, -20}, {-12, -20}}, color = {0, 0, 0}), Line(points = {{-40, 20}, {-12, 20}}, color = {0, 0, 0}), Line(points = {{-40, 60}, {-12, 60}}, color = {0, 0, 0}), Text(extent = {{102, -22}, {60, -74}}, lineColor = {0, 0, 0}, textString = "degRk")}),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-20, -98}, {20, -60}}, lineColor = {0, 0, 0}, lineThickness = 0.5, fillColor = {191, 0, 0}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-12, 40}, {12, -68}}, lineColor = {191, 0, 0}, fillColor = {191, 0, 0}, fillPattern = FillPattern.Solid), Line(points = {{12, 0}, {90, 0}}, color = {0, 0, 255}), Line(points = {{-90, 0}, {-12, 0}}, color = {191, 0, 0}), Polygon(points = {{-12, 40}, {-12, 80}, {-10, 86}, {-6, 88}, {0, 90}, {6, 88}, {10, 86}, {12, 80}, {12, 40}, {-12, 40}}, lineColor = {0, 0, 0}, lineThickness = 0.5), Line(points = {{-12, 40}, {-12, -64}}, color = {0, 0, 0}, thickness = 0.5), Line(points = {{12, 40}, {12, -64}}, color = {0, 0, 0}, thickness = 0.5), Line(points = {{-40, -20}, {-12, -20}}, color = {0, 0, 0}), Line(points = {{-40, 20}, {-12, 20}}, color = {0, 0, 0}), Line(points = {{-40, 60}, {-12, 60}}, color = {0, 0, 0}), Text(extent = {{144, -34}, {32, -82}}, lineColor = {0, 0, 0}, textString = "degRk"), Text(extent = {{-150, 135}, {150, 95}}, textString = "%name", lineColor = {0, 0, 255})}),
            Documentation(info = "<HTML>
<p>
This is an ideal absolute temperature sensor which returns
the temperature of the connected port in Rankine as an output
signal.  The sensor itself has no thermal interaction with
whatever it is connected to.  Furthermore, no
thermocouple-like lags are associated with this
sensor model.
</p>
</html>"));
        end TemperatureSensor;
        annotation(
          Documentation(info = "<HTML>
<p>
The components of this package are provided for the convenience of
people working mostly with Rankine units, since all models
in package HeatTransfer are based on Kelvin units.
</p>
<p>
Note, that in package SIunits.Conversions, functions are provided
to convert between the units Kelvin, degree Celsius, degree Fahrenheit
and degree Rankine. These functions allow, e.g., a direct conversion
of units at all places where Kelvin is required as parameter.
Example:
</p>
<pre>
    <b>import</b> SIunits.Conversions.*;
    Modelica.Thermal.HeatTransfer.HeatCapacitor C(T0 = from_degRk(500));
</pre>
</html>"));
      end Rankine;

      package Interfaces "Connectors and partial models"
        extends Modelica.Icons.InterfacesPackage;

        partial connector HeatPort "Thermal port for 1-dim. heat transfer"
          Modelica.SIunits.Temperature T "Port temperature";
          flow Modelica.SIunits.HeatFlowRate Q_flow "Heat flow rate (positive if flowing from outside into the component)";
          annotation(
            Documentation(info = "<html>

</html>"));
        end HeatPort;

        connector HeatPort_a "Thermal port for 1-dim. heat transfer (filled rectangular icon)"
          extends HeatPort;
          annotation(
            defaultComponentName = "port_a",
            Documentation(info = "<HTML>
<p>This connector is used for 1-dimensional heat flow between components.
The variables in the connector are:</p>
<pre>
   T       Temperature in [Kelvin].
   Q_flow  Heat flow rate in [Watt].
</pre>
<p>According to the Modelica sign convention, a <b>positive</b> heat flow
rate <b>Q_flow</b> is considered to flow <b>into</b> a component. This
convention has to be used whenever this connector is used in a model
class.</p>
<p>Note, that the two connector classes <b>HeatPort_a</b> and
<b>HeatPort_b</b> are identical with the only exception of the different
<b>icon layout</b>.</p></html>"),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, 100}, {100, -100}}, lineColor = {191, 0, 0}, fillColor = {191, 0, 0}, fillPattern = FillPattern.Solid)}),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-50, 50}, {50, -50}}, lineColor = {191, 0, 0}, fillColor = {191, 0, 0}, fillPattern = FillPattern.Solid), Text(extent = {{-120, 120}, {100, 60}}, lineColor = {191, 0, 0}, textString = "%name")}));
        end HeatPort_a;

        connector HeatPort_b "Thermal port for 1-dim. heat transfer (unfilled rectangular icon)"
          extends HeatPort;
          annotation(
            defaultComponentName = "port_b",
            Documentation(info = "<HTML>
<p>This connector is used for 1-dimensional heat flow between components.
The variables in the connector are:</p>
<pre>
   T       Temperature in [Kelvin].
   Q_flow  Heat flow rate in [Watt].
</pre>
<p>According to the Modelica sign convention, a <b>positive</b> heat flow
rate <b>Q_flow</b> is considered to flow <b>into</b> a component. This
convention has to be used whenever this connector is used in a model
class.</p>
<p>Note, that the two connector classes <b>HeatPort_a</b> and
<b>HeatPort_b</b> are identical with the only exception of the different
<b>icon layout</b>.</p></html>"),
            Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-50, 50}, {50, -50}}, lineColor = {191, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Text(extent = {{-100, 120}, {120, 60}}, lineColor = {191, 0, 0}, textString = "%name")}),
            Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, 100}, {100, -100}}, lineColor = {191, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid)}));
        end HeatPort_b;

        partial model Element1D "Partial heat transfer element with two HeatPort connectors that does not store energy"
          Modelica.SIunits.HeatFlowRate Q_flow "Heat flow rate from port_a -> port_b";
          Modelica.SIunits.TemperatureDifference dT "port_a.T - port_b.T";
        public
          HeatPort_a port_a annotation(
            Placement(transformation(extent = {{-110, -10}, {-90, 10}}, rotation = 0)));
          HeatPort_b port_b annotation(
            Placement(transformation(extent = {{90, -10}, {110, 10}}, rotation = 0)));
        equation
          dT = port_a.T - port_b.T;
          port_a.Q_flow = Q_flow;
          port_b.Q_flow = -Q_flow;
          annotation(
            Documentation(info = "<HTML>
<p>
This partial model contains the basic connectors and variables to
allow heat transfer models to be created that <b>do not store energy</b>,
This model defines and includes equations for the temperature
drop across the element, <b>dT</b>, and the heat flow rate
through the element from port_a to port_b, <b>Q_flow</b>.
</p>
<p>
By extending this model, it is possible to write simple
constitutive equations for many types of heat transfer components.
</p>
</html>"));
        end Element1D;

        partial model PartialElementaryConditionalHeatPort "Partial model to include a conditional HeatPort in order to dissipate losses, used for textual modeling, i.e., for elementary models"
          parameter Boolean useHeatPort = false "=true, if heatPort is enabled" annotation(
            Evaluate = true,
            HideResult = true,
            choices(checkBox = true));
          parameter Modelica.SIunits.Temperature T = 293.15 "Fixed device temperature if useHeatPort = false" annotation(
            Dialog(enable = not useHeatPort));
          Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a heatPort(final T = TheatPort, final Q_flow = -lossPower) if useHeatPort "Optional port to which dissipated losses are transported in form of heat" annotation(
            Placement(transformation(extent = {{-110, -110}, {-90, -90}}), iconTransformation(extent = {{-110, -110}, {-90, -90}})));
          Modelica.SIunits.Power lossPower "Loss power leaving component via heatPort (> 0, if heat is flowing out of component)";
          Modelica.SIunits.Temperature TheatPort "Temperature of heatPort";
        equation
          if not useHeatPort then
            TheatPort = T;
          end if;
          annotation(
            Documentation(info = "<html>
<p>
This partial model provides a conditional heat port for dissipating losses.
</p>
<ul>
<li>If <b>useHeatPort</b> is set to <b>false</b> (default), no heat port is available, and the thermal loss power is dissipated internally.
In this case, the parameter <b>T</b> specifies the fixed device temperature (the default for T = 20&deg;C) </li>
<li>If <b>useHeatPort</b> is set to <b>true</b>, the heat port is available. </li>
</ul>
<p>
If this model is used, the loss power has to be provided by an equation in the model which inherits from PartialElementaryConditionalHeatPort model
(<b>lossPower = ...</b>). The device temperature <b>TheatPort</b> can be used to describe the influence of the device temperature on the model behaviour.
</p>
</html>"));
        end PartialElementaryConditionalHeatPort;

        partial model PartialElementaryConditionalHeatPortWithoutT "Partial model to include a conditional HeatPort in order to dissipate losses, used for textual modeling, i.e., for elementary models"
          parameter Boolean useHeatPort = false "=true, if heatPort is enabled" annotation(
            Evaluate = true,
            HideResult = true,
            choices(checkBox = true));
          Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a heatPort(final Q_flow = -lossPower) if useHeatPort "Optional port to which dissipated losses are transported in form of heat" annotation(
            Placement(transformation(extent = {{-110, -110}, {-90, -90}}), iconTransformation(extent = {{-110, -110}, {-90, -90}})));
          Modelica.SIunits.Power lossPower "Loss power leaving component via heatPort (> 0, if heat is flowing out of component)";
          annotation(
            Documentation(info = "<html>
<p>
This partial model provides a conditional heat port for dissipating losses.
</p>
<ul>
<li>If <b>useHeatPort</b> is set to <b>false</b> (default), no heat port is available, and the thermal loss power is dissipated internally.
<li>If <b>useHeatPort</b> is set to <b>true</b>, the heat port is available and must be connected from the outside.</li>
</ul>
<p>
If this model is used, the loss power has to be provided by an equation in the model which inherits from the PartialElementaryConditionalHeatPortWithoutT model
(<b>lossPower = ...</b>).
</p>

<p>
Note, this partial model is used in cases, where heatPort.T (that is the device temperature) is not utilized in the model. If this is desired, inherit instead from partial model
<a href=\"modelica://Modelica.Thermal.HeatTransfer.Interfaces.PartialElementaryConditionalHeatPort\">PartialElementaryConditionalHeatPort</a>.
</p>
</html>"));
        end PartialElementaryConditionalHeatPortWithoutT;

        partial model PartialConditionalHeatPort "Partial model to include a conditional HeatPort in order to dissipate losses, used for graphical modeling, i.e., for building models by drag-and-drop"
          parameter Boolean useHeatPort = false "=true, if HeatPort is enabled" annotation(
            Evaluate = true,
            HideResult = true,
            choices(checkBox = true));
          parameter Modelica.SIunits.Temperature T = 293.15 "Fixed device temperature if useHeatPort = false" annotation(
            Dialog(enable = not useHeatPort));
          Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a heatPort if useHeatPort "Optional port to which dissipated losses are transported in form of heat" annotation(
            Placement(transformation(extent = {{-110, -110}, {-90, -90}}), iconTransformation(extent = {{-110, -110}, {-90, -90}})));
          Modelica.Thermal.HeatTransfer.Sources.FixedTemperature fixedTemperature(final T = T) if not useHeatPort annotation(
            Placement(transformation(extent = {{-60, -90}, {-80, -70}})));
        protected
          HeatPort_a internalHeatPort annotation(
            Placement(transformation(extent = {{-104, -84}, {-96, -76}})));
        equation
          connect(heatPort, internalHeatPort) annotation(
            Line(points = {{-100, -100}, {-100, -80}}, color = {191, 0, 0}, smooth = Smooth.None));
          connect(fixedTemperature.port, internalHeatPort) annotation(
            Line(points = {{-80, -80}, {-100, -80}}, color = {191, 0, 0}, smooth = Smooth.None));
          annotation(
            Documentation(info = "<html>
<p>
This partial model provides a conditional heat port for dissipating losses.
</p>
<ul>
<li>If <b>useHeatPort</b> is set to <b>false</b> (default), no heat port is available, and the thermal loss power is dissipated internally.
In this case, the parameter <b>T</b> specifies the fixed device temperature (the default for T = 20&deg;C) </li>
<li>If <b>useHeatPort</b> is set to <b>true</b>, the heat port is available. </li>
</ul>
<p>
If this model is used, the <b>internalHeatPort</b> has to be connected in the model which inherits from PartialElementaryConditionalHeatPort model.
The device temperature <b>internalHeatPort.T</b> can be used to describe the influence of the device temperature on the model behaviour.
</p>
</html>"));
        end PartialConditionalHeatPort;
        annotation(
          Documentation(info = "<html>

</html>"));
      end Interfaces;
      annotation(
        Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(origin = {13.758, 27.517}, lineColor = {128, 128, 128}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid, points = {{-54, -6}, {-61, -7}, {-75, -15}, {-79, -24}, {-80, -34}, {-78, -42}, {-73, -49}, {-64, -51}, {-57, -51}, {-47, -50}, {-41, -43}, {-38, -35}, {-40, -27}, {-40, -20}, {-42, -13}, {-47, -7}, {-54, -5}, {-54, -6}}), Polygon(origin = {13.758, 27.517}, fillColor = {160, 160, 164}, fillPattern = FillPattern.Solid, points = {{-75, -15}, {-79, -25}, {-80, -34}, {-78, -42}, {-72, -49}, {-64, -51}, {-57, -51}, {-47, -50}, {-57, -47}, {-65, -45}, {-71, -40}, {-74, -33}, {-76, -23}, {-75, -15}, {-75, -15}}), Polygon(origin = {13.758, 27.517}, lineColor = {160, 160, 164}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid, points = {{39, -6}, {32, -7}, {18, -15}, {14, -24}, {13, -34}, {15, -42}, {20, -49}, {29, -51}, {36, -51}, {46, -50}, {52, -43}, {55, -35}, {53, -27}, {53, -20}, {51, -13}, {46, -7}, {39, -5}, {39, -6}}), Polygon(origin = {13.758, 27.517}, fillColor = {160, 160, 164}, fillPattern = FillPattern.Solid, points = {{18, -15}, {14, -25}, {13, -34}, {15, -42}, {21, -49}, {29, -51}, {36, -51}, {46, -50}, {36, -47}, {28, -45}, {22, -40}, {19, -33}, {17, -23}, {18, -15}, {18, -15}}), Polygon(origin = {13.758, 27.517}, lineColor = {191, 0, 0}, fillColor = {191, 0, 0}, fillPattern = FillPattern.Solid, points = {{-9, -23}, {-9, -10}, {18, -17}, {-9, -23}}), Line(origin = {13.758, 27.517}, points = {{-41, -17}, {-9, -17}}, color = {191, 0, 0}, thickness = 0.5), Line(origin = {13.758, 27.517}, points = {{-17, -40}, {15, -40}}, color = {191, 0, 0}, thickness = 0.5), Polygon(origin = {13.758, 27.517}, lineColor = {191, 0, 0}, fillColor = {191, 0, 0}, fillPattern = FillPattern.Solid, points = {{-17, -46}, {-17, -34}, {-40, -40}, {-17, -46}})}),
        Documentation(info = "<HTML>
<p>
This package contains components to model <b>1-dimensional heat transfer</b>
with lumped elements. This allows especially to model heat transfer in
machines provided the parameters of the lumped elements, such as
the heat capacity of a part, can be determined by measurements
(due to the complex geometries and many materials used in machines,
calculating the lumped element parameters from some basic analytic
formulas is usually not possible).
</p>
<p>
Example models how to use this library are given in subpackage <b>Examples</b>.<br>
For a first simple example, see <b>Examples.TwoMasses</b> where two masses
with different initial temperatures are getting in contact to each
other and arriving after some time at a common temperature.<br>
<b>Examples.ControlledTemperature</b> shows how to hold a temperature
within desired limits by switching on and off an electric resistor.<br>
A more realistic example is provided in <b>Examples.Motor</b> where the
heating of an electrical motor is modelled, see the following screen shot
of this example:
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Thermal/HeatTransfer/driveWithHeatTransfer.png\" ALT=\"driveWithHeatTransfer\">
</p>

<p>
The <b>filled</b> and <b>non-filled red squares</b> at the left and
right side of a component represent <b>thermal ports</b> (connector HeatPort).
Drawing a line between such squares means that they are thermally connected.
The variables of a HeatPort connector are the temperature <b>T</b> at the port
and the heat flow rate <b>Q_flow</b> flowing into the component (if Q_flow is positive,
the heat flows into the element, otherwise it flows out of the element):
</p>
<pre>   Modelica.SIunits.Temperature  T  \"absolute temperature at port in Kelvin\";
   Modelica.SIunits.HeatFlowRate Q_flow  \"flow rate at the port in Watt\";
</pre>
<p>
Note, that all temperatures of this package, including initial conditions,
are given in Kelvin. For convenience, in subpackages <b>HeatTransfer.Celsius</b>,
 <b>HeatTransfer.Fahrenheit</b> and <b>HeatTransfer.Rankine</b> components are provided such that source and
sensor information is available in degree Celsius, degree Fahrenheit, or degree Rankine,
respectively. Additionally, in package <b>SIunits.Conversions</b> conversion
functions between the units Kelvin and Celsius, Fahrenheit, Rankine are
provided. These functions may be used in the following way:
</p>
<pre>  <b>import</b> SI=Modelica.SIunits;
  <b>import</b> Modelica.SIunits.Conversions.*;
     ...
  <b>parameter</b> SI.Temperature T = from_degC(25);  // convert 25 degree Celsius to Kelvin
</pre>

<p>
There are several other components available, such as AxialConduction (discretized PDE in
axial direction), which have been temporarily removed from this library. The reason is that
these components reference material properties, such as thermal conductivity, and currently
the Modelica design group is discussing a general scheme to describe material properties.
</p>
<p>
For technical details in the design of this library, see the following reference:<br>
<b>Michael Tiller (2001)</b>: <a href=\"http://www.amazon.de\">
Introduction to Physical Modeling with Modelica</a>.
Kluwer Academic Publishers Boston.
</p>
<p>
<b>Acknowledgements:</b><br>
Several helpful remarks from the following persons are acknowledged:
John Batteh, Ford Motors, Dearborn, U.S.A;
<a href=\"http://www.haumer.at/\">Anton Haumer</a>, Technical Consulting &amp; Electrical Engineering, Austria;
Ludwig Marvan, VA TECH ELIN EBG Elektronik GmbH, Wien, Austria;
Hans Olsson, Dassault Syst&egrave;mes AB, Sweden;
Hubertus Tummescheit, Lund Institute of Technology, Lund, Sweden.
</p>
<dl>
  <dt><b>Main Authors:</b></dt>
  <dd>
  <p>
  <a href=\"http://www.haumer.at/\">Anton Haumer</a><br>
  Technical Consulting &amp; Electrical Engineering<br>
  A-3423 St.Andrae-Woerdern, Austria<br>
  email: <a href=\"mailto:a.haumer@haumer.at\">a.haumer@haumer.at</a>
</p>
  </dd>
</dl>
<p><b>Copyright &copy; 2001-2015, Modelica Association, Michael Tiller and DLR.</b></p>

<p>
<i>This Modelica package is <u>free</u> software and the use is completely at <u>your own risk</u>; it can be redistributed and/or modified under the terms of the Modelica License 2. For license conditions (including the disclaimer of warranty) see <a href=\"modelica://Modelica.UsersGuide.ModelicaLicense2\">Modelica.UsersGuide.ModelicaLicense2</a> or visit <a href=\"https://www.modelica.org/licenses/ModelicaLicense2\"> https://www.modelica.org/licenses/ModelicaLicense2</a>.</i>
</p>
</html>", revisions = "<html>
<ul>
<li><i>July 15, 2002</i>
       by Michael Tiller, <a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a>
       and Nikolaus Sch&uuml;rmann:<br>
       Implemented.
</li>
<li><i>June 13, 2005</i>
       by <a href=\"http://www.haumer.at/\">Anton Haumer</a><br>
       Refined placing of connectors (cosmetic).<br>
       Refined all Examples; removed Examples.FrequencyInverter, introducing Examples.Motor<br>
       Introduced temperature dependent correction (1 + alpha*(T - T_ref)) in Fixed/PrescribedHeatFlow<br>
</li>
  <li> v1.1.1 2007/11/13 Anton Haumer<br>
       components moved to sub-packages</li>
  <li> v1.2.0 2009/08/26 Anton Haumer<br>
       added component ThermalCollector</li>

</ul>
</html>"));
    end HeatTransfer;
    annotation(
      Icon(coordinateSystem(extent = {{-100.0, -100.0}, {100.0, 100.0}}), graphics = {Line(origin = {-47.5, 11.6667}, points = {{-2.5, -91.6667}, {17.5, -71.6667}, {-22.5, -51.6667}, {17.5, -31.6667}, {-22.5, -11.667}, {17.5, 8.3333}, {-2.5, 28.3333}, {-2.5, 48.3333}}, smooth = Smooth.Bezier), Polygon(origin = {-50.0, 68.333}, pattern = LinePattern.None, fillPattern = FillPattern.Solid, points = {{0.0, 21.667}, {-10.0, -8.333}, {10.0, -8.333}}), Line(origin = {2.5, 11.6667}, points = {{-2.5, -91.6667}, {17.5, -71.6667}, {-22.5, -51.6667}, {17.5, -31.6667}, {-22.5, -11.667}, {17.5, 8.3333}, {-2.5, 28.3333}, {-2.5, 48.3333}}, smooth = Smooth.Bezier), Polygon(origin = {0.0, 68.333}, pattern = LinePattern.None, fillPattern = FillPattern.Solid, points = {{0.0, 21.667}, {-10.0, -8.333}, {10.0, -8.333}}), Line(origin = {52.5, 11.6667}, points = {{-2.5, -91.6667}, {17.5, -71.6667}, {-22.5, -51.6667}, {17.5, -31.6667}, {-22.5, -11.667}, {17.5, 8.3333}, {-2.5, 28.3333}, {-2.5, 48.3333}}, smooth = Smooth.Bezier), Polygon(origin = {50.0, 68.333}, pattern = LinePattern.None, fillPattern = FillPattern.Solid, points = {{0.0, 21.667}, {-10.0, -8.333}, {10.0, -8.333}})}),
      Documentation(info = "<html>
<p>
This package contains libraries to model heat transfer
and fluid heat flow.
</p>
</html>"));
  end Thermal;

  package Math "Library of mathematical functions (e.g., sin, cos) and of functions operating on vectors and matrices"
    import SI = Modelica.SIunits;
    extends Modelica.Icons.Package;

    package Icons "Icons for Math"
      extends Modelica.Icons.IconsPackage;

      partial function AxisLeft "Basic icon for mathematical function with y-axis on left side"
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, 100}, {100, -100}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{-80, -80}, {-80, 68}}, color = {192, 192, 192}), Polygon(points = {{-80, 90}, {-88, 68}, {-72, 68}, {-80, 90}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Text(extent = {{-150, 150}, {150, 110}}, textString = "%name", lineColor = {0, 0, 255})}),
          Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-80, 80}, {-88, 80}}, color = {95, 95, 95}), Line(points = {{-80, -80}, {-88, -80}}, color = {95, 95, 95}), Line(points = {{-80, -90}, {-80, 84}}, color = {95, 95, 95}), Text(extent = {{-75, 104}, {-55, 84}}, lineColor = {95, 95, 95}, textString = "y"), Polygon(points = {{-80, 98}, {-86, 82}, {-74, 82}, {-80, 98}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid)}),
          Documentation(info = "<html>
<p>
Icon for a mathematical function, consisting of an y-axis on the left side.
It is expected, that an x-axis is added and a plot of the function.
</p>
</html>"));
      end AxisLeft;

      partial function AxisCenter "Basic icon for mathematical function with y-axis in the center"
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, 100}, {100, -100}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{0, -80}, {0, 68}}, color = {192, 192, 192}), Polygon(points = {{0, 90}, {-8, 68}, {8, 68}, {0, 90}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Text(extent = {{-150, 150}, {150, 110}}, textString = "%name", lineColor = {0, 0, 255})}),
          Diagram(graphics = {Line(points = {{0, 80}, {-8, 80}}, color = {95, 95, 95}), Line(points = {{0, -80}, {-8, -80}}, color = {95, 95, 95}), Line(points = {{0, -90}, {0, 84}}, color = {95, 95, 95}), Text(extent = {{5, 104}, {25, 84}}, lineColor = {95, 95, 95}, textString = "y"), Polygon(points = {{0, 98}, {-6, 82}, {6, 82}, {0, 98}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid)}),
          Documentation(info = "<html>
<p>
Icon for a mathematical function, consisting of an y-axis in the middle.
It is expected, that an x-axis is added and a plot of the function.
</p>
</html>"));
      end AxisCenter;
    end Icons;

    package Vectors "Library of functions operating on vectors"
      extends Modelica.Icons.Package;

      function toString "Convert a real vector in to a string representation"
        extends Modelica.Icons.Function;
        import Modelica.Utilities.Strings;
        input Real v[:] "Real vector";
        input String name = "" "Independent variable name used for printing";
        input Integer significantDigits = 6 "Number of significant digits that are shown";
        output String s = "";
      protected
        String blanks = Strings.repeat(significantDigits);
        String space = Strings.repeat(8);
        Integer r = size(v, 1);
      algorithm
        if r == 0 then
          s := if name == "" then "[]" else name + " = []";
        else
          s := if name == "" then "\n" else "\n" + name + " = \n";
          for i in 1:r loop
            s := s + space;
            if v[i] >= 0 then
              s := s + " ";
            end if;
            s := s + String(v[i], significantDigits = significantDigits) + Strings.repeat(significantDigits + 8 - Strings.length(String(abs(v[i]))));
            s := s + "\n";
          end for;
        end if;
        annotation(
          Documentation(info = "<HTML>
<h4>Syntax</h4>
<blockquote><pre>
Vectors.<b>toString</b>(v);
Vectors.<b>toString</b>(v,name=\"\",significantDigits=6);
</pre></blockquote>
<h4>Description</h4>
<p>
The function call \"<code>Vectors.<b>toString</b>(v)</code>\" returns the string representation of vector <b>v</b>.
With the optional arguments \"name\" and \"significantDigits\" a name and the number of the digits are defined.
The default values of \"name\" and \"significantDigits\" are \"\" and 6 respectively. If name==\"\" (empty string) then the prefix \"&lt;name&gt; =\" is leaved out at the output-string.

<h4>Example</h4>
<blockquote><pre>
  v = {2.12, -4.34, -2.56, -1.67};
  <b>toString</b>(v);
                         // = \"
                         //           2.12
                         //          -4.34
                         //          -2.56
                         //          -1.67\"
  <b>toString</b>(v,\"vv\",1);
                         // = \"vv =
                         //           2
                         //          -4
                         //          -3
                         //          -2\"
</pre></blockquote>

<h4>See also</h4>
<p>
<a href=\"modelica://Modelica.Math.Matrices.toString\">Matrices.toString</a>,
</p>
</HTML>", revisions = "<html>

</html>"));
      end toString;

      function isEqual "Determine if two Real vectors are numerically identical"
        extends Modelica.Icons.Function;
        input Real v1[:] "First vector";
        input Real v2[:] "Second vector (may have different length as v1)";
        input Real eps(min = 0) = 0 "Two elements e1 and e2 of the two vectors are identical if abs(e1-e2) <= eps";
        output Boolean result "= true, if vectors have the same length and the same elements";
      protected
        Integer n = size(v1, 1) "Dimension of vector v1";
        Integer i = 1;
      algorithm
        result := false;
        if size(v2, 1) == n then
          result := true;
          while i <= n loop
            if abs(v1[i] - v2[i]) > eps then
              result := false;
              i := n;
            end if;
            i := i + 1;
          end while;
        end if;
        annotation(
          Documentation(info = "<HTML>
<h4>Syntax</h4>
<blockquote><pre>
Vectors.<b>isEqual</b>(v1, v2);
Vectors.<b>isEqual</b>(v1, v2, eps=0);
</pre></blockquote>
<h4>Description</h4>
<p>
The function call \"<code>Vectors.isEqual(v1, v2)</code>\" returns <b>true</b>,
if the two Real vectors v1 and v2 have the same dimensions and
the same elements. Otherwise the function
returns <b>false</b>. Two elements e1 and e2 of the two vectors
are checked on equality by the test \"abs(e1-e2) &le; eps\", where \"eps\"
can be provided as third argument of the function. Default is \"eps = 0\".
</p>
<h4>Example</h4>
<blockquote><pre>
  Real v1[3] = {1, 2, 3};
  Real v2[4] = {1, 2, 3, 4};
  Real v3[3] = {1, 2, 3.0001};
  Boolean result;
<b>algorithm</b>
  result := Vectors.isEqual(v1,v2);     // = <b>false</b>
  result := Vectors.isEqual(v1,v3);     // = <b>false</b>
  result := Vectors.isEqual(v1,v1);     // = <b>true</b>
  result := Vectors.isEqual(v1,v3,0.1); // = <b>true</b>
</pre></blockquote>
<h4>See also</h4>
<p>
<a href=\"modelica://Modelica.Math.Vectors.find\">Vectors.find</a>,
<a href=\"modelica://Modelica.Math.Matrices.isEqual\">Matrices.isEqual</a>,
<a href=\"modelica://Modelica.Utilities.Strings.isEqual\">Strings.isEqual</a>
</p>
</HTML>"));
      end isEqual;

      function norm "Return the p-norm of a vector"
        extends Modelica.Icons.Function;
        input Real v[:] "Vector";
        input Real p(min = 1) = 2 "Type of p-norm (often used: 1, 2, or Modelica.Constants.inf)";
        output Real result = 0.0 "p-norm of vector v";
      protected
        Real eps = 10 * Modelica.Constants.eps;
      algorithm
        if size(v, 1) > 0 then
          if p >= 2 - eps and p <= 2 + eps then
            result := sqrt(v * v);
          elseif p >= Modelica.Constants.inf then
            result := max(abs(v));
          elseif p >= 1 - eps and p <= 1 + eps then
            result := sum(abs(v));
          elseif p >= 1 then
            result := sum(abs(v[i]) ^ p for i in 1:size(v, 1)) ^ (1 / p);
          else
            assert(false, "Optional argument \"p\" (= " + String(p) + ") of function \"norm\" >= 1 required");
          end if;
        end if;
        annotation(
          Documentation(info = "<HTML>
<h4>Syntax</h4>
<blockquote><pre>
Vectors.<b>norm</b>(v);
Vectors.<b>norm</b>(v,p=2);   // 1 &le; p &le; &#8734;
</pre></blockquote>
<h4>Description</h4>
<p>
The function call \"<code>Vectors.<b>norm</b>(v)</code>\" returns the
<b>Euclidean norm</b> \"<code>sqrt(v*v)</code>\" of vector v.
With the optional
second argument \"p\", any other p-norm can be computed:
</p>
<center>
<IMG src=\"modelica://Modelica/Resources/Images/Math/Vectors/vectorNorm.png\" ALT=\"function Vectors.norm\">
</center>
<p>
Besides the Euclidean norm (p=2), also the 1-norm and the
infinity-norm are sometimes used:
</p>
<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"><b>1-norm</b></td>
      <td valign=\"top\">= sum(abs(v))</td>
      <td valign=\"top\"><b>norm</b>(v,1)</td>
  </tr>
  <tr><td valign=\"top\"><b>2-norm</b></td>
      <td valign=\"top\">= sqrt(v*v)</td>
      <td valign=\"top\"><b>norm</b>(v) or <b>norm</b>(v,2)</td>
  </tr>
  <tr><td valign=\"top\"><b>infinity-norm</b></td>
      <td valign=\"top\">= max(abs(v))</td>
      <td valign=\"top\"><b>norm</b>(v,Modelica.Constants.<b>inf</b>)</td>
  </tr>
</table>
<p>
Note, for any vector norm the following inequality holds:
</p>
<blockquote><pre>
<b>norm</b>(v1+v2,p) &le; <b>norm</b>(v1,p) + <b>norm</b>(v2,p)
</pre></blockquote>
<h4>Example</h4>
<blockquote><pre>
  v = {2, -4, -2, -1};
  <b>norm</b>(v,1);    // = 9
  <b>norm</b>(v,2);    // = 5
  <b>norm</b>(v);      // = 5
  <b>norm</b>(v,10.5); // = 4.00052597412635
  <b>norm</b>(v,Modelica.Constants.inf);  // = 4
</pre></blockquote>
<h4>See also</h4>
<p>
<a href=\"modelica://Modelica.Math.Matrices.norm\">Matrices.norm</a>
</p>
</HTML>"));
      end norm;

      function length "Return length of a vector (better as norm(), if further symbolic processing is performed)"
        extends Modelica.Icons.Function;
        input Real v[:] "Vector";
        output Real result "Length of vector v";
      algorithm
        result := sqrt(v * v);
        annotation(
          Inline = true,
          Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
Vectors.<b>length</b>(v);
</pre></blockquote>
<h4>Description</h4>
<p>
The function call \"<code>Vectors.<b>length</b>(v)</code>\" returns the
<b>Euclidean length</b> \"<code>sqrt(v*v)</code>\" of vector v.
The function call is equivalent to Vectors.norm(v). The advantage of
length(v) over norm(v)\"is that function length(..) is implemented
in one statement and therefore the function is usually automatically
inlined. Further symbolic processing is therefore possible, which is
not the case with function norm(..).
</p>
<h4>Example</h4>
<blockquote><pre>
  v = {2, -4, -2, -1};
  <b>length</b>(v);  // = 5
</pre></blockquote>
<h4>See also</h4>
<p>
<a href=\"modelica://Modelica.Math.Vectors.norm\">Vectors.norm</a>
</p>
</html>"));
      end length;

      function normalize "Return normalized vector such that length = 1 and prevent zero-division for zero vector"
        extends Modelica.Icons.Function;
        input Real v[:] "Vector";
        input Real eps(min = 0.0) = 100 * Modelica.Constants.eps "if |v| < eps then result = v/eps";
        output Real result[size(v, 1)] "Input vector v normalized to length=1";
      algorithm
        result := smooth(0, noEvent(if length(v) >= eps then v / length(v) else v / eps));
        annotation(
          Inline = true,
          Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
Vectors.<b>normalize</b>(v);
Vectors.<b>normalize</b>(v,eps=100*Modelica.Constants.eps);
</pre></blockquote>
<h4>Description</h4>
<p>
The function call \"<code>Vectors.<b>normalize</b>(v)</code>\" returns the
<b>unit vector</b> \"<code>v/length(v)</code>\" of vector v.
If length(v) is close to zero (more precisely, if length(v) &lt; eps),
v/eps is returned in order to avoid
a division by zero. For many applications this is useful, because
often the unit vector <b>e</b> = <b>v</b>/length(<b>v</b>) is used to compute
a vector x*<b>e</b>, where the scalar x is in the order of length(<b>v</b>),
i.e., x*<b>e</b> is small, when length(<b>v</b>) is small and then
it is fine to replace <b>e</b> by <b>v</b> to avoid a division by zero.
</p>
<p>
Since the function has the \"Inline\" annotation, it
is usually inlined and symbolic processing is applied.
</p>
<h4>Example</h4>
<blockquote><pre>
  <b>normalize</b>({1,2,3});  // = {0.267, 0.534, 0.802}
  <b>normalize</b>({0,0,0});  // = {0,0,0}
</pre></blockquote>
<h4>See also</h4>
<p>
<a href=\"modelica://Modelica.Math.Vectors.length\">Vectors.length</a>,
<a href=\"modelica://Modelica.Math.Vectors.normalize\">Vectors.normalizeWithAssert</a>
</p>
</html>"));
      end normalize;

      function normalizeWithAssert "Return normalized vector such that length = 1 (trigger an assert for zero vector)"
        import Modelica.Math.Vectors.length;
        extends Modelica.Icons.Function;
        input Real v[:] "Vector";
        output Real result[size(v, 1)] "Input vector v normalized to length=1";
      algorithm
        assert(length(v) > 0.0, "Vector v={0,0,0} shall be normalized (= v/sqrt(v*v)), but this results in a division by zero.\nProvide a non-zero vector!");
        result := v / length(v);
        annotation(
          Inline = true,
          Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
Vectors.<b>normalizeWithAssert</b>(v);
</pre></blockquote>
<h4>Description</h4>
<p>
The function call \"<code>Vectors.<b>normalizeWithAssert</b>(v)</code>\" returns the
<b>unit vector</b> \"<code>v/sqrt(v*v)</code>\" of vector v.
If vector v is a zero vector, an assert is triggered.
</p>
<p>
Since the function has the \"Inline\" annotation, it
is usually inlined and symbolic processing is applied.
</p>
<h4>Example</h4>
<blockquote><pre>
  <b>normalizeWithAssert</b>({1,2,3});  // = {0.267, 0.534, 0.802}
  <b>normalizeWithAssert</b>({0,0,0});  // error (an assert is triggered)
</pre></blockquote>
<h4>See also</h4>
<p>
<a href=\"modelica://Modelica.Math.Vectors.length\">Vectors.length</a>,
<a href=\"modelica://Modelica.Math.Vectors.normalize\">Vectors.normalize</a>
</p>
</html>"));
      end normalizeWithAssert;

      function reverse "Reverse vector elements (e.g., v[1] becomes last element)"
        extends Modelica.Icons.Function;
        input Real v[:] "Vector";
        output Real result[size(v, 1)] "Elements of vector v in reversed order";
      algorithm
        result := {v[end - i + 1] for i in 1:size(v, 1)};
        annotation(
          Inline = true,
          Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
Vectors.<b>reverse</b>(v);
</pre></blockquote>
<h4>Description</h4>
<p>
The function call \"<code>Vectors.<b>reverse</b>(v)</code>\" returns the
vector elements in reverse order.
</p>
<h4>Example</h4>
<blockquote><pre>
  <b>reverse</b>({1,2,3,4});  // = {4,3,2,1}
</pre></blockquote>
</html>"));
      end reverse;

      function sort "Sort elements of vector in ascending or descending order"
        extends Modelica.Icons.Function;
        input Real v[:] "Vector to be sorted";
        input Boolean ascending = true "= true if ascending order, otherwise descending order";
        output Real sorted_v[size(v, 1)] = v "Sorted vector";
        output Integer indices[size(v, 1)] = 1:size(v, 1) "sorted_v = v[indices]";
        /* shellsort algorithm; should be improved later */
      protected
        Integer gap;
        Integer i;
        Integer j;
        Real wv;
        Integer wi;
        Integer nv = size(v, 1);
        Boolean swap;
      algorithm
        gap := div(nv, 2);
        while gap > 0 loop
          i := gap;
          while i < nv loop
            j := i - gap;
            if j >= 0 then
              if ascending then
                swap := sorted_v[j + 1] > sorted_v[j + gap + 1];
              else
                swap := sorted_v[j + 1] < sorted_v[j + gap + 1];
              end if;
            else
              swap := false;
            end if;
            while swap loop
              wv := sorted_v[j + 1];
              wi := indices[j + 1];
              sorted_v[j + 1] := sorted_v[j + gap + 1];
              sorted_v[j + gap + 1] := wv;
              indices[j + 1] := indices[j + gap + 1];
              indices[j + gap + 1] := wi;
              j := j - gap;
              if j >= 0 then
                if ascending then
                  swap := sorted_v[j + 1] > sorted_v[j + gap + 1];
                else
                  swap := sorted_v[j + 1] < sorted_v[j + gap + 1];
                end if;
              else
                swap := false;
              end if;
            end while;
            i := i + 1;
          end while;
          gap := div(gap, 2);
        end while;
        annotation(
          Documentation(info = "<HTML>
<h4>Syntax</h4>
<blockquote><pre>
           sorted_v = Vectors.<b>sort</b>(v);
(sorted_v, indices) = Vectors.<b>sort</b>(v, ascending=true);
</pre></blockquote>
<h4>Description</h4>
<p>
Function <b>sort</b>(..) sorts a Real vector v
in ascending order and returns the result in sorted_v.
If the optional argument \"ascending\" is <b>false</b>, the vector
is sorted in descending order. In the optional second
output argument the indices of the sorted vector with respect
to the original vector are given, such that sorted_v = v[indices].
</p>
<h4>Example</h4>
<blockquote><pre>
  (v2, i2) := Vectors.sort({-1, 8, 3, 6, 2});
       -> v2 = {-1, 2, 3, 6, 8}
          i2 = {1, 5, 3, 4, 2}
</pre></blockquote>
</HTML>"));
      end sort;

      function find "Find element in a vector"
        extends Modelica.Icons.Function;
        input Real e "Search for e";
        input Real v[:] "Integer vector";
        input Real eps(min = 0) = 0 "Element e is equal to a element v[i] of vector v if abs(e-v[i]) <= eps";
        output Integer result "v[result] = e (first occurrence of e); result=0, if not found";
      protected
        Integer i;
      algorithm
        result := 0;
        i := 1;
        while i <= size(v, 1) loop
          if abs(v[i] - e) <= eps then
            result := i;
            i := size(v, 1) + 1;
          else
            i := i + 1;
          end if;
        end while;
        annotation(
          Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
Vectors.<b>find</b>(e, v);
Vectors.<b>find</b>(e, v, eps=0);
</pre></blockquote>
<h4>Description</h4>
<p>
The function call \"<code>Vectors.find(e, v)</code>\" returns the index of the first occurrence of input e in vector <b>v</b>.
The test of equality is performed by \"abs(e-v[i]) &le; eps\", where \"eps\"
can be provided as third argument of the function. Default is \"eps = 0\".
</p>
<h4>Example</h4>
<blockquote><pre>
  Real v[3] = {1, 2, 3};
  Real e1 = 2;
  Real e2 = 3.01;
  Boolean result;
<b>algorithm</b>
  result := Vectors.find(e1,v);          // = <b>2</b>
  result := Vectors.find(e2,v);          // = <b>0</b>
  result := Vectors.find(e2,v,eps=0.1);  // = <b>3</b>
</pre></blockquote>
<h4>See also</h4>
<p>
<a href=\"modelica://Modelica.Math.Vectors.isEqual\">Vectors.isEqual</a>
</p>
</html>"));
      end find;

      function interpolate "Interpolate in a vector"
        extends Modelica.Icons.Function;
        input Real x[:] "Abscissa table vector (strict monotonically increasing values required)";
        input Real y[size(x, 1)] "Ordinate table vector";
        input Real xi "Desired abscissa value";
        input Integer iLast = 1 "Index used in last search";
        output Real yi "Ordinate value corresponding to xi";
        output Integer iNew = 1 "xi is in the interval x[iNew] <= xi < x[iNew+1]";
      protected
        Integer i;
        Integer nx = size(x, 1);
        Real x1;
        Real x2;
        Real y1;
        Real y2;
      algorithm
        assert(nx > 0, "The table vectors must have at least 1 entry.");
        if nx == 1 then
          yi := y[1];
        else
          i := min(max(iLast, 1), nx - 1);
          if xi >= x[i] then
            while i < nx and xi >= x[i] loop
              i := i + 1;
            end while;
            i := i - 1;
          else
            while i > 1 and xi < x[i] loop
              i := i - 1;
            end while;
          end if;
          x1 := x[i];
          x2 := x[i + 1];
          y1 := y[i];
          y2 := y[i + 1];
          assert(x2 > x1, "Abscissa table vector values must be increasing");
          yi := y1 + (y2 - y1) * (xi - x1) / (x2 - x1);
          iNew := i;
        end if;
// Search interval
// search forward
// search backward
// Get interpolation data
// Interpolate
        annotation(
          Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
// Real    x[:], y[:], xi, yi;
// Integer iLast, iNew;
        yi = Vectors.<b>interpolate</b>(x,y,xi);
(yi, iNew) = Vectors.<b>interpolate</b>(x,y,xi,iLast=1);
</pre></blockquote>
<h4>Description</h4>
<p>
The function call \"<code>Vectors.interpolate(x,y,xi)</code>\" interpolates in vectors
(x,y) and returns the value yi that corresponds to xi. Vector x[:] must consist
of strictly monotonically increasing values. If xi &lt; x[1] or &gt; x[end], then
extrapolation takes places through the first or last two x[:] values, respectively.
The search for the interval x[iNew] &le; xi &lt; x[iNew+1] starts at the optional
input argument \"iLast\". The index \"iNew\" is returned as output argument.
The usage of \"iLast\" and \"iNew\" is useful to increase the efficiency of the call,
if many interpolations take place.
</p>

<h4>Example</h4>

<blockquote><pre>
  Real x[:] = { 0,  2,  4,  6,  8, 10};
  Real y[:] = {10, 20, 30, 40, 50, 60};
<b>algorithm</b>
  (yi, iNew) := Vectors.interpolate(x,y,5);  // yi = 35, iNew=3
</pre></blockquote>

</html>"));
      end interpolate;

      function relNodePositions "Return vector of relative node positions (0..1)"
        extends Modelica.Icons.Function;
        input Integer nNodes "Number of nodes (including node at left and right position)";
        output Real xsi[nNodes] "Relative node positions";
      protected
        Real delta;
      algorithm
        if nNodes >= 1 then
          xsi[1] := 0;
        end if;
        if nNodes >= 2 then
          xsi[nNodes] := 1;
        end if;
        if nNodes == 3 then
          xsi[2] := 0.5;
        elseif nNodes > 3 then
          delta := 1 / (nNodes - 2);
          for i in 2:nNodes - 1 loop
            xsi[i] := (i - 1.5) * delta;
          end for;
        end if;
        annotation(
          Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
Vectors.<b>relNodePositions</b>(nNodes);
</pre></blockquote>
<h4>Description</h4>
<p>
The function call \"<code>relNodePositions(nNodes)</code>\" returns a vector
with the relative positions of the nodes of a discretized pipe with nNodes nodes (including the node
at the left and at the right side of the pipe), see next figure:
</p>

<blockquote><p>
<img src=\"modelica://Modelica/Resources/Images/Math/Vectors/relNodePositions.png\">
</p></blockquote>

<h4>Example</h4>

<blockquote><pre>
  Real xsi[7];
<b>algorithm</b>
  xsi = relNodePositions(7);  // xsi = {0, 0.1, 0.3, 0.5, 0.7, 0.9, 1}
</pre></blockquote>

<h4>See also</h4>
<p>
<a href=\"modelica://Modelica.Mechanics.MultiBody.Visualizers.PipeWithScalarField\">MultiBody.Visualizers.PipeWithScalarField</a>
</p>
</html>"));
      end relNodePositions;

      package Utilities "Utility functions that should not be directly utilized by the user"
        extends Modelica.Icons.UtilitiesPackage;

        function householderVector "Calculate a normalized householder vector to reflect vector a onto vector b"
          extends Modelica.Icons.Function;
          import Modelica.Math.Vectors.norm;
          input Real a[:] "Real vector to be reflected";
          input Real b[size(a, 1)] "Real vector b vector a is mapped onto";
          output Real u[size(a, 1)] "Householder vector to map a onto b";
        protected
          Real norm_a = norm(a, 2);
          Real norm_b = norm(b, 2);
          Real alpha;
        algorithm
          assert(norm_b > 0, "Vector b in function householderVector is zero vector, but at least one element should be different from zero");
          assert(norm_a > 0, "Vector a in function householderVector is zero vector, but at least one element should be different from zero");
          alpha := if norm(a + norm_a / norm_b * b, 2) > norm(a - norm_a / norm_b * b, 2) then norm_a / norm_b else -norm_a / norm_b;
          u := (a + alpha * b) / length(a + alpha * b);
          annotation(
            Documentation(info = "<HTML>
<h4>Syntax</h4>
<blockquote><pre>
Vectors.Utilities.<b>householderVector</b>(a,b);
</pre></blockquote>
<h4>Description</h4>
<p>
The function call \"<code>householderVector(a, b)</code>\" returns the normalized Householder vector
<b>u</b> for Householder reflection of input vector <b>a</b> onto vector <b>b</b>, i.e., Householder vector <b>u</b> is the normal
vector of the reflection plane. Algebraically, the reflection is performed by transformation matrix <b>Q</b>
</p>
<blockquote>
<p>
<b>Q</b> = <b>I</b> - 2*<b>u</b>*<b>u</b>',
</p>
</blockquote>
i.e., vector <b>a</b> is mapped to
<blockquote>
<p>
<b>a</b> -> <b>Q</b>*<b>a</b>=c*<b>b</b>
</p>
</blockquote>
with scalar c, |c| = ||<b>a</b>|| / ||<b>b</b>||. <b>Q</b>*<b>a</b> is the reflection of <b>a</b> about the hyperplane orthogonal to <b>u</b>.
<b>Q</b> is an orthogonal matrix, i.e.
<blockquote>
<p>
    <b>Q</b> = inv(<b>Q</b>) = <b>Q</b>'
</p>
</blockquote>
<h4>Example</h4>
<blockquote><pre>
  a = {2, -4, -2, -1};
  b = {1, 0, 0, 0};

  u = <b>householderVector</b>(a,b);    // {0.837, -0.478, -0.239, -0.119}
                               // Computation (identity(4) - 2*matrix(u)*transpose(matrix(u)))*a results in
                               // {-5, 0, 0, 0} = -5*b
</pre></blockquote>
<h4>See also</h4>
<a href=\"modelica://Modelica.Math.Vectors.Utilities.householderReflection\">Vectors.Utilities.householderReflection</a><br>
<a href=\"modelica://Modelica.Math.Matrices.Utilities.householderReflection\">Matrices.Utilities.householderReflection</a><br>
<a href=\"modelica://Modelica.Math.Matrices.Utilities.householderSimilarityTransformation\">Matrices.Utilities.householderSimilarityTransformation</a>
</html>", revisions = "<html>
<ul>
<li><i>2010/04/30 </i>
       by Marcus Baur, DLR-RM</li>
</ul>

</html>"));
        end householderVector;

        function householderReflection "Reflect a vector a on a plane with orthogonal vector u"
          extends Modelica.Icons.Function;
          import Modelica.Math.Vectors;
          input Real a[:] "Real vector a to be reflected";
          input Real u[size(a, 1)] "householder vector";
          output Real ra[size(u, 1)] "reflexion of a";
        protected
          Real norm_a = Vectors.length(a);
          Real h = 2 * u * a;
        algorithm
          ra := a - h * u;
// Values close to zero are set to zero.
          for i in 1:size(ra, 1) loop
            ra[i] := if abs(ra[i]) >= norm_a * 1e-12 then ra[i] else 0;
          end for;
          annotation(
            Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
Vectors.Utilities.<b>householderReflection</b>(a,u);
</pre></blockquote>
<h4>Description</h4>
<p>
Function \"<code>householderReflection(a, u)</code>\" performs the reflection of vector
<b>a</b> about a plane orthogonal to vector <b>u</b> (Householder vector).
Algebraically the operation is defined by
</p>
<blockquote>
<p>
<b>b</b>=<b>Q</b>*<b>a</b>
</p>
</blockquote>
with
<blockquote>
<p>
   <b>Q</b> = <b>I</b> - 2*<b>u</b>*<b>u</b>',
</p>
</blockquote>
where <b>Q</b> is an orthogonal matrix, i.e.
<blockquote>
<p>
    <b>Q</b> = inv(<b>Q</b>) = <b>Q</b>'
</p>
</blockquote>
<h4>Example</h4>
<blockquote><pre>
  a = {2, -4, -2, -1};
  u = {0.837, -0.478, -0.239, -0.119};

  <b>householderReflection</b>(a,u);    //  = {-5.0, -0.001, -0.0005, -0.0044}
</pre></blockquote>
<h4>See also</h4>
<a href=\"modelica://Modelica.Math.Vectors.Utilities.householderVector\">Utilities.householderVector</a><br>
<a href=\"modelica://Modelica.Math.Matrices.Utilities.householderReflection\">Matrices.Utilities.householderReflection</a><br>
<a href=\"modelica://Modelica.Math.Matrices.Utilities.householderSimilarityTransformation\">Matrices.Utilities.householderSimilarityTransformation</a>

</html>", revisions = "<html>
<ul>
<li><i>2010/04/30 </i>
       by Marcus Baur, DLR-RM</li>
</ul>
</html>"));
        end householderReflection;

        encapsulated function roots "Compute zeros of a polynomial where the highest coefficient is assumed as not to be zero"
          import Modelica.Math.Matrices;
          import Modelica;
          extends Modelica.Icons.Function;
          input Real p[:] "Vector with polynomial coefficients p[1]*x^n + p[2]*x^(n-1) + p[n]*x +p[n-1]";
          output Real roots[max(0, size(p, 1) - 1), 2] = fill(0, max(0, size(p, 1) - 1), 2) "roots[:,1] and roots[:,2] are the real and imaginary parts of the roots of polynomial p";
        protected
          Integer np = size(p, 1);
          Integer n = size(p, 1) - 1;
          Real A[max(size(p, 1) - 1, 0), max(size(p, 1) - 1, 0)] "Companion matrix";
          Real ev[max(size(p, 1) - 1, 0), 2] "Eigenvalues";
        algorithm
          if n > 0 then
            assert(abs(p[1]) > 0, "Computing the roots of a polynomial with function \"Modelica.Math.Vectors.Utilities.roots\"\n" + "failed because the first element of the coefficient vector is zero, but should not be.");
            A[1, :] := -p[2:np] / p[1];
            A[2:n, :] := [identity(n - 1), zeros(n - 1)];
            roots := Matrices.Utilities.eigenvaluesHessenberg(A);
          end if;
// companion matrix
// roots are the eigenvalues of the companion matrix
          annotation(
            Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
  r = Vectors.Utilities.<b>roots</b>(p);
</pre></blockquote>
<h4>Description</h4>
<p>
This function computes the roots of a polynomial P of x
</p>
<blockquote><pre>
  P = p[1]*x^n + p[2]*x^(n-1) + ... + p[n-1]*x + p[n+1];
</pre></blockquote>
<p>
with the coefficient vector <b>p</b>. It is assumed that the first element of <b>p</b> is not zero, i.e., that the polynomial is of order size(p,1)-1.
<p>
To compute the roots, the eigenvalues of the corresponding companion matrix <b>C</b>
</p>
<blockquote><pre>
         |-p[2]/p[1]  -p[3]/p[1]  ...  -p[n-2]/p[1]  -p[n-1]/p[1]  -p[n]/p[1] |
         |    1            0                0               0           0     |
         |    0            1      ...       0               0           0     |
  <b>C</b> =    |    .            .      ...       .               .           .     |
         |    .            .      ...       .               .           .     |
         |    0            0      ...       0               1           0     |
</pre></blockquote>
<p>
are calculated. These are the roots of the polynomial.<br>
Since the companion matrix has already Hessenberg form, the transformation to Hessenberg form has not to be performed.
Function <a href=\"modelica://Modelica.Math.Matrices.Utilities.eigenvaluesHessenberg\">eigenvaluesHessenberg</a><br>
provides efficient eigenvalue computation for those matrices.
</p>
<h4>Example</h4>
<blockquote><pre>
  r = <b>roots</b>({1,2,3});
  // r = [-1.0,  1.41421356237309;
  //      -1.0, -1.41421356237309]
  // which corresponds to the roots: -1.0 +/- j*1.41421356237309
</pre></blockquote>
</html>"));
        end roots;
        annotation(
          Documentation(info = "<html>
<p>
This package contains utility functions that are utilized by higher level vector
and matrix functions. These functions are usually not useful for an end-user.
</p>

</html>"));
      end Utilities;
      annotation(
        preferredView = "info",
        Documentation(info = "<HTML>
<h4>Library content</h4>
<p>
This library provides functions operating on vectors:
</p>

<ul>
<li> <a href=\"modelica://Modelica.Math.Vectors.toString\">toString</a>(v)
     - returns the string representation of vector v.</li>

<li> <a href=\"modelica://Modelica.Math.Vectors.isEqual\">isEqual</a>(v1, v2)
     - returns true if vectors v1 and v2 have the same size and the same elements.</li>

<li> <a href=\"modelica://Modelica.Math.Vectors.norm\">norm</a>(v,p)
     - returns the p-norm of vector v.</li>

<li> <a href=\"modelica://Modelica.Math.Vectors.length\">length</a>(v)
     - returns the length of vector v (= norm(v,2), but inlined and therefore usable in
       symbolic manipulations)</li>

<li> <a href=\"modelica://Modelica.Math.Vectors.normalize\">normalize</a>(v)
     - returns vector in direction of v with lenght = 1 and prevents
       zero-division for zero vector.</li>

<li> <a href=\"modelica://Modelica.Math.Vectors.reverse\">reverse</a>(v)
     - reverses the vector elements of v. </li>

<li> <a href=\"modelica://Modelica.Math.Vectors.sort\">sort</a>(v)
     - sorts the elements of vector v in ascending or descending order.</li>

<li> <a href=\"modelica://Modelica.Math.Vectors.find\">find</a>(e, v)
     - returns the index of the first occurrence of scalar e in vector v.</li>

<li> <a href=\"modelica://Modelica.Math.Vectors.interpolate\">interpolate</a>(x, y, xi)
     - returns the interpolated value in (x,y) that corresponds to xi.</li>

<li> <a href=\"modelica://Modelica.Math.Vectors.relNodePositions\">relNodePositions</a>(nNodes)
     - returns a vector of relative node positions (0..1).</li>
</ul>

<h4>See also</h4>
<a href=\"modelica://Modelica.Math.Matrices\">Matrices</a>
</HTML>"));
    end Vectors;

    package Matrices "Library of functions operating on matrices"
      package Examples "Examples demonstrating the usage of the Math.Matrices functions"
        extends Modelica.Icons.ExamplesPackage;

        function solveLinearEquations "Demonstrate the solution of linear equation systems"
          extends Modelica.Icons.Function;
          import Modelica.Utilities.Streams.print;
          // solve and solve2
        protected
          Real A0[0, 0];
          Real A1[2, 2] = [1, 2; 3, 4];
          Real x1_ref[2] = {-2, 3};
          Real b1[2] = A1 * x1_ref;
          Real x1[2];
          Real B2[2, 3] = [b1, 2 * b1, -3 * b1];
          Real X2[2, 3];
          // leastSquares and leastSquares2
          Integer rank;
          Real a[3] = {2, 3, -1};
          Real A3[3, 3] = transpose([{2, 3, -4}, a, 3 * a]);
          Real x3_ref[3] = {-2, 3, 5};
          Real b3[3] = A3 * x3_ref;
          Real x3[3];
          Real B3[3, 2] = [b3, -3 * b3];
          Real X3[3, 2];
        algorithm
          print("\nDemonstrate how to solve linear equation systems:\n");
// Solve regular linear equation with a right hand side vector
          x1 := Math.Matrices.solve(A1, b1);
          print("diff1 = " + String(Vectors.norm(x1 - x1_ref)));
// Solve regular linear equation with a right hand side matrix
          X2 := Math.Matrices.solve2(A1, B2);
          print("diff2 = " + String(Matrices.norm(X2 - [x1_ref, 2 * x1_ref, -3 * x1_ref])));
// Solve singular linear equation with a right hand side vector
          (x3, rank) := Math.Matrices.leastSquares(A3, b3);
          print("diff3 = " + String(Vectors.norm(A3 * x3 - b3)) + ", n = " + String(size(A3, 1)) + ", rank = " + String(rank));
// Solve singular linear equation with a right hand side matrix
          (X3, rank) := Math.Matrices.leastSquares2(A3, B3);
          print("diff4 = " + String(Matrices.norm(A3 * X3 - B3)) + ", n = " + String(size(A3, 1)) + ", rank = " + String(rank));
          annotation(
            Documentation(info = "<html>
<p>
With simple examples this function demonstrates how to solve
regular linear equation systems with Matrices.solve and Matrices.solve2,
and how to solve singular linear equation systems with
Matrices.leastSquares and Matrices.leastSquares2.
</p>
</html>"));
        end solveLinearEquations;
      end Examples;

      function toString "Convert a matrix into its string representation"
        extends Modelica.Icons.Function;
        import Modelica.Utilities.Strings;
        input Real M[:, :] "Real matrix";
        input String name = "" "Independent variable name used for printing";
        input Integer significantDigits = 6 "Number of significant digits that are shown";
        output String s = "" "String expression of matrix M";
      protected
        String blanks = Strings.repeat(significantDigits);
        String space = Strings.repeat(8);
        String space2 = Strings.repeat(3);
        Integer r = size(M, 1);
        Integer c = size(M, 2);
      algorithm
        if r == 0 or c == 0 then
          s := name + " = []";
        else
          s := if name == "" then "\n" else "\n" + name + " = \n";
          for i in 1:r loop
            s := s + space;
            for j in 1:c loop
              if M[i, j] >= 0 then
                s := s + " ";
              end if;
              s := s + String(M[i, j], significantDigits = significantDigits) + Strings.repeat(significantDigits + 8 - Strings.length(String(abs(M[i, j]))));
            end for;
            s := s + "\n";
          end for;
        end if;
        annotation(
          Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
Matrices.<b>toString</b>(A);
Matrices.<b>toString</b>(A, name=\"\", significantDigits=6);
</pre></blockquote>
<h4>Description</h4>
<p>
The function call \"<code>Matrices.<b>toString</b>(A)</code>\" returns the
string representation of matrix <b>A</b>.
With the optional arguments \"name\" and \"significantDigits\", a name and the number of the digits are defined.
The default values of name and significantDigits are \"\" and 6 respectively. If name==\"\" then the
prefix \"&lt;name&gt; =\" is leaved out.

<h4>Example</h4>
<blockquote><pre>
  A = [2.12, -4.34; -2.56, -1.67];

  toString(A);
  // = \"
  //      2.12   -4.34
  //     -2.56   -1.67\";

  toString(A,\"A\",1);
  // = \"A =
  //         2     -4
  //        -3     -2\"
</pre></blockquote>

<h4>See also</h4>
<p>
<a href=\"modelica://Modelica.Math.Vectors.toString\">Vectors.toString</a>
</p>

</HTML>", revisions = "<html>
</html>"));
      end toString;

      extends Modelica.Icons.Package;

      function isEqual "Compare whether two Real matrices are identical"
        extends Modelica.Icons.Function;
        input Real M1[:, :] "First matrix";
        input Real M2[:, :] "Second matrix (may have different size as M1)";
        input Real eps(min = 0) = 0 "Two elements e1 and e2 of the two matrices are identical if abs(e1-e2) <= eps";
        output Boolean result "= true, if matrices have the same size and the same elements";
      protected
        Integer nrow = size(M1, 1) "Number of rows of matrix M1";
        Integer ncol = size(M1, 2) "Number of columns of matrix M1";
        Integer i = 1;
        Integer j;
      algorithm
        result := false;
        if size(M2, 1) == nrow and size(M2, 2) == ncol then
          result := true;
          while i <= nrow loop
            j := 1;
            while j <= ncol loop
              if abs(M1[i, j] - M2[i, j]) > eps then
                result := false;
                i := nrow;
                j := ncol;
              end if;
              j := j + 1;
            end while;
            i := i + 1;
          end while;
        end if;
        annotation(
          Documentation(info = "<HTML>
<h4>Syntax</h4>
<blockquote><pre>
Matrices.<b>isEqual</b>(M1, M2);
Matrices.<b>isEqual</b>(M1, M2, eps=0);
</pre></blockquote>
<h4>Description</h4>
<p>
The function call \"<code>Matrices.isEqual(M1, M2)</code>\" returns <b>true</b>,
if the two Real matrices M1 and M2 have the same dimensions and
the same elements. Otherwise the function
returns <b>false</b>. Two elements e1 and e2 of the two matrices
are checked on equality by the test \"abs(e1-e2) &le; eps\", where \"eps\"
can be provided as third argument of the function. Default is \"eps = 0\".
</p>
<h4>Example</h4>
<blockquote><pre>
  Real A1[2,2] = [1,2; 3,4];
  Real A2[3,2] = [1,2; 3,4; 5,6];
  Real A3[2,2] = [1,2, 3,4.0001];
  Boolean result;
<b>algorithm</b>
  result := Matrices.isEqual(M1,M2);     // = <b>false</b>
  result := Matrices.isEqual(M1,M3);     // = <b>false</b>
  result := Matrices.isEqual(M1,M1);     // = <b>true</b>
  result := Matrices.isEqual(M1,M3,0.1); // = <b>true</b>
</pre></blockquote>
<h4>See also</h4>
<a href=\"modelica://Modelica.Math.Vectors.isEqual\">Vectors.isEqual</a>,
<a href=\"modelica://Modelica.Utilities.Strings.isEqual\">Strings.isEqual</a>
</HTML>"));
      end isEqual;

      function solve "Solve real system of linear equations A*x=b with a b vector (Gaussian elimination with partial pivoting)"
        extends Modelica.Icons.Function;
        input Real A[:, size(A, 1)] "Matrix A of A*x = b";
        input Real b[size(A, 1)] "Vector b of A*x = b";
        output Real x[size(b, 1)] "Vector x such that A*x = b";
      protected
        Integer info;
      algorithm
        (x, info) := LAPACK.dgesv_vec(A, b);
        assert(info == 0, "Solving a linear system of equations with function
\"Matrices.solve\" is not possible, because the system has either
no or infinitely many solutions (A is singular).");
        annotation(
          Documentation(info = "<HTML>
<h4>Syntax</h4>
<blockquote><pre>
Matrices.<b>solve</b>(A,b);
</pre></blockquote>
<h4>Description</h4>
<p>
This function call returns the
solution <b>x</b> of the linear system of equations
</p>
<blockquote>
<p>
<b>A</b>*<b>x</b> = <b>b</b>
</p>
</blockquote>
<p>
If a unique solution <b>x</b> does not exist (since <b>A</b> is singular),
an assertion is triggered. If this is not desired, use instead
<a href=\"modelica://Modelica.Math.Matrices.leastSquares\">Matrices.leastSquares</a>
and inquire the singularity of the solution with the return argument rank
(a unique solution is computed if rank = size(A,1)).
</p>

<p>
Note, the solution is computed with the LAPACK function \"dgesv\",
i.e., by Gaussian elimination with partial pivoting.
</p>
<h4>Example</h4>
<blockquote><pre>
  Real A[3,3] = [1,2,3;
                 3,4,5;
                 2,1,4];
  Real b[3] = {10,22,12};
  Real x[3];
<b>algorithm</b>
  x := Matrices.solve(A,b);  // x = {3,2,1}
</pre></blockquote>
<h4>See also</h4>
<a href=\"modelica://Modelica.Math.Matrices.LU\">Matrices.LU</a>,
<a href=\"modelica://Modelica.Math.Matrices.LU_solve\">Matrices.LU_solve</a>,
<a href=\"modelica://Modelica.Math.Matrices.leastSquares\">Matrices.leastSquares</a>.
</HTML>"));
      end solve;

      function solve2 "Solve real system of linear equations A*X=B with a B matrix (Gaussian elimination with partial pivoting)"
        extends Modelica.Icons.Function;
        input Real A[:, size(A, 1)] "Matrix A of A*X = B";
        input Real B[size(A, 1), :] "Matrix B of A*X = B";
        output Real X[size(B, 1), size(B, 2)] "Matrix X such that A*X = B";
      protected
        Integer info;
      algorithm
        (X, info) := LAPACK.dgesv(A, B);
        assert(info == 0, "Solving a linear system of equations with function
\"Matrices.solve2\" is not possible, because the system has either
no or infinitely many solutions (A is singular).");
        annotation(
          Documentation(info = "<HTML>
<h4>Syntax</h4>
<blockquote><pre>
Matrices.<b>solve2</b>(A,b);
</pre></blockquote>
<h4>Description</h4>
<p>
This function call returns the
solution <b>X</b> of the linear system of equations
</p>
<blockquote>
<p>
<b>A</b>*<b>X</b> = <b>B</b>
</p>
</blockquote>
<p>
If a unique solution <b>X</b> does not exist (since <b>A</b> is singular),
an assertion is triggered. If this is not desired, use instead
<a href=\"modelica://Modelica.Math.Matrices.leastSquares2\">Matrices.leastSquares2</a>
and inquire the singularity of the solution with the return argument rank
(a unique solution is computed if rank = size(A,1)).

</p>
<p>
Note, the solution is computed with the LAPACK function \"dgesv\",
i.e., by Gaussian elimination with partial pivoting.
</p>
<h4>Example</h4>
<blockquote><pre>
  Real A[3,3] = [1,2,3;
                 3,4,5;
                 2,1,4];
  Real B[3,2] = [10, 20;
                 22, 44;
                 12, 24];
  Real X[3,2];
<b>algorithm</b>
  X := Matrices.solve2(A, B);  /* X = [3, 6;
                                       2, 4;
                                       1, 2] */
</pre></blockquote>

<h4>See also</h4>
<p>
<a href=\"modelica://Modelica.Math.Matrices.LU\">Matrices.LU</a>,
<a href=\"modelica://Modelica.Math.Matrices.LU_solve2\">Matrices.LU_solve2</a>,
<a href=\"modelica://Modelica.Math.Matrices.leastSquares2\">Matrices.leastSquares2</a>.
</p>
</HTML>"));
      end solve2;

      function leastSquares "Solve linear equation A*x = b (exactly if possible, or otherwise in a least square sense; A may be non-square and may be rank deficient)"
        extends Modelica.Icons.Function;
        input Real A[:, :] "Matrix A";
        input Real b[size(A, 1)] "Vector b";
        input Real rcond = 100 * Modelica.Constants.eps "Reciprocal condition number to estimate the rank of A";
        output Real x[size(A, 2)] "Vector x such that min|A*x-b|^2 if size(A,1) >= size(A,2) or min|x|^2 and A*x=b, if size(A,1) < size(A,2)";
        output Integer rank "Rank of A";
      protected
        Integer info;
        Real xx[max(size(A, 1), size(A, 2))];
      algorithm
        if min(size(A)) > 0 then
          (xx, info, rank) := LAPACK.dgelsx_vec(A, b, rcond);
          x := xx[1:size(A, 2)];
          assert(info == 0, "Solving an overdetermined or underdetermined linear system\n" + "of equations with function \"Matrices.leastSquares\" failed.");
        else
          x := fill(0.0, size(A, 2));
        end if;
        annotation(
          Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
x = Matrices.<b>leastSquares</b>(A,b);
</pre></blockquote>
<h4>Description</h4>
<p>
Returns a solution of equation A*x = b in a least
square sense (A may be rank deficient):
</p>
<pre>
  minimize | A*x - b |
</pre>

<p>
Several different cases can be distinguished (note, <b>rank</b> is an
output argument of this function):
</p>

<p>
<b>size(A,1) = size(A,2)</b>
</p>

<p> A solution is returned for a regular, as well as a singular matrix A:
</p>

<ul>
<li> <b>rank</b> = size(A,1):<br>
     A is <b>regular</b> and the returned solution x fulfills the equation
     A*x = b uniquely.</li>

<li> <b>rank</b> &lt; size(A,1):<br>
     A is <b>singular</b> and no unique solution for equation A*x = b exists.
     <ul>
     <li>  If an infinite number of solutions exists, the one is selected that fulfills
           the equation and at the same time has the minimum norm |x| for all solution
           vectors that fulfill the equation.</li>
     <li>  If no solution exists, x is selected such that |A*x - b| is as small as
           possible (but A*x - b is not zero).</li>
     </ul>
</ul>

<p>
<b>size(A,1) &gt; size(A,2):</b>
</p>

<p>
The equation A*x = b has no unique solution. The solution x is selected such that
|A*x - b| is as small as possible. If rank = size(A,2), this minimum norm solution is
unique. If rank &lt; size(A,2), there are an infinite number of solutions leading to the
same minimum value of |A*x - b|. From these infinite number of solutions, the one with the
minimum norm |x| is selected. This gives a unique solution that minimizes both
|A*x - b| and |x|.
</p>

<p>
<b>size(A,1) &lt; size(A,2):</b>
</p>

<ul>
<li> <b>rank</b> = size(A,1):<br>
     There are an infinite number of solutions that fulfill the equation A*x = b.
     From this infinite number, the unique solution is selected that minimizes |x|.
     </li>

<li> <b>rank</b> &lt; size(A,1):<br>
     There is either no solution of equation A*x = b, or there are again an infinite
     number of solutions. The unique solution x is returned that minimizes
      both |A*x - b| and |x|.</li>
</ul>

<p>
Note, the solution is computed with the LAPACK function \"dgelsx\",
i.e., QR or LQ factorization of A with column pivoting.
</p>

<h4>Algorithmic details</h4>

<p>
The function first computes a QR factorization with column pivoting:
</p>

<pre>
      A * P = Q * [ R11 R12 ]
                  [  0  R22 ]
</pre>

<p>
with R11 defined as the largest leading submatrix whose estimated
condition number is less than 1/rcond.  The order of R11, <b>rank</b>,
is the effective rank of A.
</p>

<p>
Then, R22 is considered to be negligible, and R12 is annihilated
by orthogonal transformations from the right, arriving at the
complete orthogonal factorization:
</p>

<pre>
     A * P = Q * [ T11 0 ] * Z
                 [  0  0 ]
</pre>

<p>
The minimum-norm solution is then
</p>

<pre>
     x = P * Z' [ inv(T11)*Q1'*b ]
                [        0       ]
</pre>

<p>
where Q1 consists of the first \"rank\" columns of Q.
</p>

<h4>See also</h4>

<p>
<a href=\"modelica://Modelica.Math.Matrices.leastSquares2\">Matrices.leastSquares2</a>
(same as leastSquares, but with a right hand side matrix), <br>
<a href=\"modelica://Modelica.Math.Matrices.solve\">Matrices.solve</a>
(for square, regular matrices A)
</p>

</html>"));
      end leastSquares;

      function leastSquares2 "Solve linear equation A*X = B (exactly if possible, or otherwise in a least square sense; A may be non-square and may be rank deficient)"
        extends Modelica.Icons.Function;
        input Real A[:, :] "Matrix A";
        input Real B[size(A, 1), :] "Matrix B";
        input Real rcond = 100 * Modelica.Constants.eps "Reciprocal condition number to estimate rank of A";
        output Real X[size(A, 2), size(B, 2)] "Matrix X such that min|A*X-B|^2 if size(A,1) >= size(A,2) or min|X|^2 and A*X=B, if size(A,1) < size(A,2)";
        output Integer rank "Rank of A";
      protected
        Integer info;
        Real XX[max(size(A, 1), size(A, 2)), size(B, 2)];
      algorithm
        (XX, info, rank) := LAPACK.dgelsx(A, B, rcond);
        X := XX[1:size(A, 2), :];
        assert(info == 0, "Solving an overdetermined or underdetermined linear system of
equations with function \"Matrices.leastSquares2\" failed.");
        annotation(
          Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
X = Matrices.<b>leastSquares2</b>(A,B);
</pre></blockquote>
<h4>Description</h4>
<p>
Returns a solution of equation A*X = B in a least
square sense (A may be rank deficient):
</p>
<pre>
  minimize | A*X - B |
</pre>

<p>
Several different cases can be distinguished (note, <b>rank</b> is an
output argument of this function):
</p>

<p>
<b>size(A,1) = size(A,2)</b>
</p>

<p> A solution is returned for a regular, as well as a singular matrix A:
</p>

<ul>
<li> <b>rank</b> = size(A,1):<br>
     A is <b>regular</b> and the returned solution X fulfills the equation
     A*X = B uniquely.</li>

<li> <b>rank</b> &lt; size(A,1):<br>
     A is <b>singular</b> and no unique solution for equation A*X = B exists.
     <ul>
     <li>  If an infinite number of solutions exists, the one is selected that fulfills
           the equation and at the same time has the minimum norm |x| for all solution
           vectors that fulfill the equation.</li>
     <li>  If no solution exists, X is selected such that |A*X - B| is as small as
           possible (but A*X - B is not zero).</li>
     </ul>
</ul>

<p>
<b>size(A,1) &gt; size(A,2):</b>
</p>

<p>
The equation A*X = B has no unique solution. The solution X is selected such that
|A*X - B| is as small as possible. If rank = size(A,2), this minimum norm solution is
unique. If rank &lt; size(A,2), there are an infinite number of solutions leading to the
same minimum value of |A*X - B|. From these infinite number of solutions, the one with the
minimum norm |X| is selected. This gives a unique solution that minimizes both
|A*X - B| and |X|.
</p>

<p>
<b>size(A,1) &lt; size(A,2):</b>
</p>

<ul>
<li> <b>rank</b> = size(A,1):<br>
     There are an infinite number of solutions that fulfill the equation A*X = B.
     From this infinite number, the unique solution is selected that minimizes |X|.
     </li>

<li> <b>rank</b> &lt; size(A,1):<br>
     There is either no solution of equation A*X = B, or there are again an infinite
     number of solutions. The unique solution X is returned that minimizes
      both |A*X - B| and |X|.</li>
</ul>

<p>
Note, the solution is computed with the LAPACK function \"dgelsx\",
i.e., QR or LQ factorization of A with column pivoting.
</p>

<h4>Algorithmic details</h4>

<p>
The function first computes a QR factorization with column pivoting:
</p>

<pre>
      A * P = Q * [ R11 R12 ]
                  [  0  R22 ]
</pre>

<p>
with R11 defined as the largest leading submatrix whose estimated
condition number is less than 1/rcond.  The order of R11, <b>rank</b>,
is the effective rank of A.
</p>

<p>
Then, R22 is considered to be negligible, and R12 is annihilated
by orthogonal transformations from the right, arriving at the
complete orthogonal factorization:
</p>

<pre>
     A * P = Q * [ T11 0 ] * Z
                 [  0  0 ]
</pre>

<p>
The minimum-norm solution is then
</p>

<pre>
     X = P * Z' [ inv(T11)*Q1'*B ]
                [        0       ]
</pre>

<p>
where Q1 consists of the first \"rank\" columns of Q.
</p>

<h4>See also</h4>

<p>
<a href=\"modelica://Modelica.Math.Matrices.leastSquares\">Matrices.leastSquares</a>
(same as leastSquares2, but with a right hand side vector), <br>
<a href=\"modelica://Modelica.Math.Matrices.solve2\">Matrices.solve2</a>
(for square, regular matrices A)
</p>

</html>"));
      end leastSquares2;

      function equalityLeastSquares "Solve a linear equality constrained least squares problem"
        extends Modelica.Icons.Function;
        input Real A[:, :] "Minimize |A*x - a|^2";
        input Real a[size(A, 1)];
        input Real B[:, size(A, 2)] "subject to B*x=b";
        input Real b[size(B, 1)];
        output Real x[size(A, 2)] "solution vector";
      protected
        Integer info;
      algorithm
        assert(size(A, 2) >= size(B, 1) and size(A, 2) <= size(A, 1) + size(B, 1), "It is required that size(B,1) <= size(A,2) <= size(A,1) + size(B,1)\n" + "This relationship is not fulfilled, since the matrices are declared as:\n" + "  A[" + String(size(A, 1)) + "," + String(size(A, 2)) + "], B[" + String(size(B, 1)) + "," + String(size(B, 2)) + "]\n");
        (x, info) := LAPACK.dgglse_vec(A, a, B, b);
        assert(info == 0, "Solving a linear equality-constrained least squares problem
with function \"Matrices.equalityLeastSquares\" failed.");
        annotation(
          Documentation(info = "<HTML>
<h4>Syntax</h4>
<blockquote><pre>
x = Matrices.<b>equalityLeastSquares</b>(A,a,B,b);
</pre></blockquote>
<h4>Description</h4>
<p>
This function returns the
solution <b>x</b> of the linear equality-constrained least squares problem:
</p>
<blockquote>
<p>
min|<b>A</b>*<b>x</b> - <b>a</b>|^2 over <b>x</b>, subject to <b>B</b>*<b>x</b> = <b>b</b>
</p>
</blockquote>

<p>
It is required that the dimensions of A and B fulfill the following
relationship:
</p>

<blockquote>
size(B,1) &le; size(A,2) &le; size(A,1) + size(B,1)
</blockquote>

<p>
Note, the solution is computed with the LAPACK function \"dgglse\"
using the generalized RQ factorization under the assumptions that
B has full row rank (= size(B,1)) and the matrix [A;B] has
full column rank (= size(A,2)). In this case, the problem
has a unique solution.
</p>
</HTML>"));
      end equalityLeastSquares;

      function LU "LU decomposition of square or rectangular matrix"
        extends Modelica.Icons.Function;
        input Real A[:, :] "Square or rectangular matrix";
        output Real LU[size(A, 1), size(A, 2)] = A "L,U factors (used with LU_solve(..))";
        output Integer pivots[min(size(A, 1), size(A, 2))] "pivot indices (used with LU_solve(..))";
        output Integer info "Information";
      protected
        Integer lda = max(1, size(A, 1));

        external "FORTRAN 77" dgetrf(size(A, 1), size(A, 2), LU, lda, pivots, info) annotation(
          Library = "lapack");
        annotation(
          Documentation(info = "<HTML>
<h4>Syntax</h4>
<blockquote><pre>
(LU, pivots)       = Matrices.<b>LU</b>(A);
(LU, pivots, info) = Matrices.<b>LU</b>(A);
</pre></blockquote>
<h4>Description</h4>
<p>
This function call returns the
LU decomposition of a \"Real[m,n]\" matrix A, i.e.,
</p>
<blockquote>
<p>
<b>P</b>*<b>L</b>*<b>U</b> = <b>A</b>
</p>
</blockquote>
<p>
where <b>P</b> is a permutation matrix (implicitly
defined by vector <code>pivots</code>),
<b>L</b> is a lower triangular matrix with unit
diagonal elements (lower trapezoidal if m &gt; n), and
<b>U</b> is an upper triangular matrix (upper trapezoidal if m &lt; n).
Matrices <b>L</b> and <b>U</b> are stored in the returned
matrix <code>LU</code> (the diagonal of <b>L</b> is not stored).
With the companion function
<a href=\"modelica://Modelica.Math.Matrices.LU_solve\">Matrices.LU_solve</a>,
this decomposition can be used to solve
linear systems (<b>P</b>*<b>L</b>*<b>U</b>)*<b>x</b> = <b>b</b> with different right
hand side vectors <b>b</b>. If a linear system of equations with
just one right hand side vector <b>b</b> shall be solved, it is
more convenient to just use the function
<a href=\"modelica://Modelica.Math.Matrices.solve\">Matrices.solve</a>.
</p>
<p>
The optional third (Integer) output argument has the following meaning:</p>
<table border=0 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\">info = 0:</td>
      <td valign=\"top\">successful exit</td></tr>
  <tr><td valign=\"top\">info &gt; 0:</td>
      <td valign=\"top\">if info = i, U[i,i] is exactly zero. The factorization
          has been completed, <br>
          but the factor U is exactly
          singular, and division by zero will occur<br> if it is used
          to solve a system of equations.</td></tr>
</table>
<p>
The LU factorization is computed
with the LAPACK function \"dgetrf\",
i.e., by Gaussian elimination using partial pivoting
with row interchanges. Vector \"pivots\" are the
pivot indices, i.e., for 1 &le; i &le; min(m,n), row i of
matrix A was interchanged with row pivots[i].
</p>
<h4>Example</h4>
<blockquote><pre>
  Real A[3,3] = [1,2,3;
                 3,4,5;
                 2,1,4];
  Real b1[3] = {10,22,12};
  Real b2[3] = { 7,13,10};
  Real    LU[3,3];
  Integer pivots[3];
  Real    x1[3];
  Real    x2[3];
<b>algorithm</b>
  (LU, pivots) := Matrices.LU(A);
  x1 := Matrices.LU_solve(LU, pivots, b1);  // x1 = {3,2,1}
  x2 := Matrices.LU_solve(LU, pivots, b2);  // x2 = {1,0,2}
</pre></blockquote>
<h4>See also</h4>
<p>
<a href=\"modelica://Modelica.Math.Matrices.LU_solve\">Matrices.LU_solve</a>,
<a href=\"modelica://Modelica.Math.Matrices.solve\">Matrices.solve</a>,
</p>
</HTML>"));
      end LU;

      function LU_solve "Solve real system of linear equations P*L*U*x=b with a b vector and an LU decomposition (from LU(..))"
        extends Modelica.Icons.Function;
        input Real LU[:, size(LU, 1)] "L,U factors of Matrices.LU(..) for a square matrix";
        input Integer pivots[size(LU, 1)] "Pivots indices of Matrices.LU(..)";
        input Real b[size(LU, 1)] "Right hand side vector of P*L*U*x=b";
        output Real x[size(b, 1)] "Solution vector such that P*L*U*x = b";
      algorithm
        for i in 1:size(LU, 1) loop
          assert(LU[i, i] <> 0, "Solving a linear system of equations with function
\"Matrices.LU_solve\" is not possible, since the LU decomposition
is singular, i.e., no unique solution exists.");
        end for;
        x := LAPACK.dgetrs_vec(LU, pivots, b);
        annotation(
          Documentation(info = "<HTML>
<h4>Syntax</h4>
<blockquote><pre>
Matrices.<b>LU_solve</b>(LU, pivots, b);
</pre></blockquote>
<h4>Description</h4>
<p>
This function call returns the
solution <b>x</b> of the linear systems of equations
</p>
<blockquote>
<p>
<b>P</b>*<b>L</b>*<b>U</b>*<b>x</b> = <b>b</b>;
</p>
</blockquote>
<p>
where <b>P</b> is a permutation matrix (implicitly
defined by vector <code>pivots</code>),
<b>L</b> is a lower triangular matrix with unit
diagonal elements (lower trapezoidal if m &gt; n), and
<b>U</b> is an upper triangular matrix (upper trapezoidal if m &lt; n).
The matrices of this decomposition are computed with function
<a href=\"modelica://Modelica.Math.Matrices.LU\">Matrices.LU</a> that
returns arguments <code>LU</code> and <code>pivots</code>
used as input arguments of <code>Matrices.LU_solve</code>.
With <code>Matrices.LU</code> and <code>Matrices.LU_solve</code>
it is possible to efficiently solve linear systems
with different right hand side vectors. If a linear system of equations with
just one right hand side vector shall be solved, it is
more convenient to just use the function
<a href=\"modelica://Modelica.Math.Matrices.solve\">Matrices.solve</a>.
</p>
<p>
If a unique solution <b>x</b> does not exist (since the
LU decomposition is singular), an exception is raised.
</p>
<p>
The LU factorization is computed
with the LAPACK function \"dgetrf\",
i.e., by Gaussian elimination using partial pivoting
with row interchanges. Vector \"pivots\" are the
pivot indices, i.e., for 1 &le; i &le; min(m,n), row i of
matrix A was interchanged with row pivots[i].
</p>
<h4>Example</h4>
<blockquote><pre>
  Real A[3,3] = [1,2,3;
                 3,4,5;
                 2,1,4];
  Real b1[3] = {10,22,12};
  Real b2[3] = { 7,13,10};
  Real    LU[3,3];
  Integer pivots[3];
  Real    x1[3];
  Real    x2[3];
<b>algorithm</b>
  (LU, pivots) := Matrices.LU(A);
  x1 := Matrices.LU_solve(LU, pivots, b1);  // x1 = {3,2,1}
  x2 := Matrices.LU_solve(LU, pivots, b2);  // x2 = {1,0,2}
</pre></blockquote>
<h4>See also</h4>
<a href=\"modelica://Modelica.Math.Matrices.LU\">Matrices.LU</a>,
<a href=\"modelica://Modelica.Math.Matrices.solve\">Matrices.solve</a>,
</HTML>"));
      end LU_solve;

      function LU_solve2 "Solve real system of linear equations P*L*U*X=B with a B matrix and an LU decomposition (from LU(..))"
        extends Modelica.Icons.Function;
        input Real LU[:, size(LU, 1)] "L,U factors of Matrices.LU(..) for a square matrix";
        input Integer pivots[size(LU, 1)] "Pivots indices of Matrices.LU(..)";
        input Real B[size(LU, 1), :] "Right hand side matrix of P*L*U*X=B";
        output Real X[size(B, 1), size(B, 2)] "Solution matrix such that P*L*U*X = B";
      algorithm
        for i in 1:size(LU, 1) loop
          assert(LU[i, i] <> 0, "Solving a linear system of equations with function
\"Matrices.LU_solve2\" is not possible, since the LU decomposition
is singular, i.e., no unique solution exists.");
        end for;
        X := Modelica.Math.Matrices.LAPACK.dgetrs(LU, pivots, B);
        annotation(
          Documentation(info = "<HTML>
<h4>Syntax</h4>
<blockquote><pre>
Matrices.<b>LU_solve2</b>(LU, pivots, B);
</pre></blockquote>
<h4>Description</h4>
<p>
This function call returns the
solution <b>X</b> of the linear systems of equations
</p>
<blockquote>
<p>
<b>P</b>*<b>L</b>*<b>U</b>*<b>X</b> = <b>B</b>;
</p>
</blockquote>
<p>
where <b>P</b> is a permutation matrix (implicitly
defined by vector <code>pivots</code>),
<b>L</b> is a lower triangular matrix with unit
diagonal elements (lower trapezoidal if m &gt; n), and
<b>U</b> is an upper triangular matrix (upper trapezoidal if m &lt; n).
The matrices of this decomposition are computed with function
<a href=\"modelica://Modelica.Math.Matrices.LU\">Matrices.LU</a> that
returns arguments <code>LU</code> and <code>pivots</code>
used as input arguments of <code>Matrices.LU_solve2</code>.
With <code>Matrices.LU</code> and <code>Matrices.LU_solve2</code>
it is possible to efficiently solve linear systems
with different right hand side <b>matrices</b>. If a linear system of equations with
just one right hand side matrix shall be solved, it is
more convenient to just use the function
<a href=\"modelica://Modelica.Math.Matrices.solve2\">Matrices.solve2</a>.
</p>
<p>
If a unique solution <b>X</b> does not exist (since the
LU decomposition is singular), an exception is raised.
</p>
<p>
The LU factorization is computed
with the LAPACK function \"dgetrf\",
i.e., by Gaussian elimination using partial pivoting
with row interchanges. Vector \"pivots\" are the
pivot indices, i.e., for 1 &le; i &le; min(m,n), row i of
matrix A was interchanged with row pivots[i].
</p>
<h4>Example</h4>
<blockquote><pre>
  Real A[3,3] = [1,2,3;
                 3,4,5;
                 2,1,4];
  Real B1[3] = [10, 20;
                22, 44;
                12, 24];
  Real B2[3] = [ 7, 14;
                13, 26;
                10, 20];
  Real    LU[3,3];
  Integer pivots[3];
  Real    X1[3,2];
  Real    X2[3,2];
<b>algorithm</b>
  (LU, pivots) := Matrices.LU(A);
  X1 := Matrices.LU_solve2(LU, pivots, B1);  /* X1 = [3, 6;
                                                      2, 4;
                                                      1, 2] */
  X2 := Matrices.LU_solve2(LU, pivots, B2);  /* X2 = [1, 2;
                                                      0, 0;
                                                      2, 4] */
</pre></blockquote>
<h4>See also</h4>
<a href=\"modelica://Modelica.Math.Matrices.LU\">Matrices.LU</a>,
<a href=\"modelica://Modelica.Math.Matrices.solve2\">Matrices.solve2</a>,
</HTML>"));
      end LU_solve2;

      function eigenValues "Return eigenvalues and eigenvectors for a real, nonsymmetric matrix in a Real representation"
        extends Modelica.Icons.Function;
        input Real A[:, size(A, 1)] "Matrix";
        output Real eigenvalues[size(A, 1), 2] "Eigenvalues of matrix A (Re: first column, Im: second column)";
        output Real eigenvectors[size(A, 1), size(A, 2)] "Real-valued eigenvector matrix";
      protected
        Integer info;
        // replace with "isPresent(..)" if supported by Dymola
        Boolean onlyEigenvalues = false;
      algorithm
        if size(A, 1) > 0 then
          if onlyEigenvalues then
            (eigenvalues[:, 1], eigenvalues[:, 2], info) := LAPACK.dgeev_eigenValues(A);
            eigenvectors := zeros(size(A, 1), size(A, 1));
          else
            (eigenvalues[:, 1], eigenvalues[:, 2], eigenvectors, info) := LAPACK.dgeev(A);
          end if;
          assert(info == 0, "Calculating the eigen values with function
\"Matrices.eigenvalues\" is not possible, since the
numerical algorithm does not converge.");
        end if;
        annotation(
          Documentation(info = "<HTML>
<h4>Syntax</h4>
<blockquote><pre>
                eigenvalues = Matrices.<b>eigenValues</b>(A);
(eigenvalues, eigenvectors) = Matrices.<b>eigenValues</b>(A);
</pre></blockquote>
<h4>Description</h4>
<p>
This function call returns the eigenvalues and
optionally the (right) eigenvectors of a square matrix
<b>A</b>. The first column of \"eigenvalues\" contains the real and the
second column contains the imaginary part of the eigenvalues.
If the i-th eigenvalue has no imaginary part, then eigenvectors[:,i] is
the corresponding real eigenvector. If the i-th eigenvalue
has an imaginary part, then eigenvalues[i+1,:] is the conjugate complex
eigenvalue and eigenvectors[:,i] is the real and eigenvectors[:,i+1] is the
imaginary part of the eigenvector of the i-th eigenvalue.
With function
<a href=\"modelica://Modelica.Math.Matrices.eigenValueMatrix\">Matrices.eigenValueMatrix</a>,
a real block diagonal matrix is constructed from the eigenvalues
such that
</p>
<blockquote>
<pre>
A = eigenvectors * eigenValueMatrix(eigenvalues) * inv(eigenvectors)
</pre>
</blockquote>
<p>
provided the eigenvector matrix \"eigenvectors\" can be inverted
(an inversion is possible, if all eigenvalues are different;
in some cases, an inversion is also possible if some eigenvalues are
the same).
</p>
<h4>Example</h4>
<blockquote><pre>
  Real A[3,3] = [1,2,3;
                 3,4,5;
                 2,1,4];
  Real eval;
<b>algorithm</b>
  eval := Matrices.eigenValues(A);  // eval = [-0.618, 0;
                                    //          8.0  , 0;
                                    //          1.618, 0];
</pre>
</blockquote>
<p>
i.e., matrix A has the 3 real eigenvalues -0.618, 8, 1.618.
</p>

<h4>See also</h4>
<a href=\"modelica://Modelica.Math.Matrices.eigenValueMatrix\">Matrices.eigenValueMatrix</a>,
<a href=\"modelica://Modelica.Math.Matrices.singularValues\">Matrices.singularValues</a>
</html>"));
      end eigenValues;

      function eigenValueMatrix "Return real valued block diagonal matrix J of eigenvalues of matrix A (A=V*J*Vinv)"
        extends Modelica.Icons.Function;
        input Real eigenValues[:, 2] "Eigen values from function eigenValues(..) (Re: first column, Im: second column)";
        output Real J[size(eigenValues, 1), size(eigenValues, 1)] "Real valued block diagonal matrix with eigen values (Re: 1x1 block, Im: 2x2 block)";
      protected
        Integer n = size(eigenValues, 1);
        Integer i;
      algorithm
        J := zeros(n, n);
        i := 1;
        while i <= n loop
          if eigenValues[i, 2] == 0 then
            J[i, i] := eigenValues[i, 1];
            i := i + 1;
          else
            J[i, i] := eigenValues[i, 1];
            J[i, i + 1] := eigenValues[i, 2];
            J[i + 1, i] := eigenValues[i + 1, 2];
            J[i + 1, i + 1] := eigenValues[i + 1, 1];
            i := i + 2;
          end if;
        end while;
        annotation(
          Documentation(info = "<HTML>
<h4>Syntax</h4>
<blockquote><pre>
Matrices.<b>eigenValueMatrix</b>(eigenvalues);
</pre></blockquote>
<h4>Description</h4>
<p>
The function call returns a block diagonal matrix <b>J</b>
from the the two-column matrix <code>eigenvalues</code>
(computed by function
<a href=\"modelica://Modelica.Math.Matrices.eigenValues\">Matrices.eigenValues</a>).
Matrix <code>eigenvalues</code> must have the real part of the
eigenvalues in the first column and the imaginary part in the
second column. If an eigenvalue i has a vanishing imaginary
part, then <b>J</b>[i,i] = eigenvalues[i,1], i.e., the diagonal
element of <b>J</b> is the real eigenvalue.
Otherwise, eigenvalue i and conjugate complex eigenvalue i+1
are used to construct a 2 by 2 diagonal block of <b>J</b>:
</p>
<blockquote>
<pre>
  J[i  , i]   := eigenvalues[i,1];
  J[i  , i+1] := eigenvalues[i,2];
  J[i+1, i]   := eigenvalues[i+1,2];
  J[i+1, i+1] := eigenvalues[i+1,1];
</pre>
</blockquote>
<h4>See also</h4>
<a href=\"modelica://Modelica.Math.Matrices.eigenValues\">Matrices.eigenValues</a>
</HTML>"));
      end eigenValueMatrix;

      function singularValues "Return singular values and left and right singular vectors"
        extends Modelica.Icons.Function;
        input Real A[:, :] "Matrix";
        output Real sigma[min(size(A, 1), size(A, 2))] "Singular values";
        output Real U[size(A, 1), size(A, 1)] = identity(size(A, 1)) "Left orthogonal matrix";
        output Real VT[size(A, 2), size(A, 2)] = identity(size(A, 2)) "Transposed right orthogonal matrix";
      protected
        Integer info;
        Integer n = min(size(A, 1), size(A, 2)) "Number of singular values";
      algorithm
        if n > 0 then
          (sigma, U, VT, info) := Modelica.Math.Matrices.LAPACK.dgesvd(A);
          assert(info == 0, "The numerical algorithm to compute the
singular value decomposition did not converge");
        end if;
        annotation(
          Documentation(info = "<HTML>
<h4>Syntax</h4>
<blockquote><pre>
         sigma = Matrices.<b>singularValues</b>(A);
(sigma, U, VT) = Matrices.<b>singularValues</b>(A);
</pre></blockquote>
<h4>Description</h4>
<p>
This function computes the singular values and optionally the
singular vectors of matrix A. Basically the singular
value decomposition of A is computed, i.e.,
</p>
<blockquote><pre>
<b>A</b> = <b>U</b> <b><font face=\"Symbol\">S</font></b> <b>V</b><sup>T</sup>
  = U*Sigma*VT
</pre></blockquote>
<p>
where <b>U</b> and <b>V</b> are orthogonal matrices (<b>UU</b><sup>T</sup>=<b>I,
</b><b>VV</b><sup>T</sup>=<b>I</b>). <b><font face=\"Symbol\">S
</font></b> = [diagonal(<font face=\"Symbol\">s</font><sub>i</sub>), zeros(n,m-n)], if n=size(A,1) &le;
m=size(A,2)) or [diagonal(<font face=\"Symbol\">s</font><sub>i</sub>); zeros(n-m,m)], if n &gt;
m=size(A,2)). <b><font face=\"Symbol\">S</font></b> has the same size as matrix A with
nonnegative diagonal elements in decreasing order and with all other elements zero
(<font face=\"Symbol\">s</font><sub>1</sub> is the largest element). The function
returns the singular values <font face=\"Symbol\">s</font><sub>i</sub>
in vector <code>sigma</code> and the orthogonal matrices in
matrices <code>U</code> and <code>VT</code>.
</p>
<h4>Example</h4>
<blockquote><pre>
  A = [1, 2,  3,  4;
       3, 4,  5, -2;
      -1, 2, -3,  5];
  (sigma, U, VT) = singularValues(A);
  results in:
     sigma = {8.33, 6.94, 2.31};
  i.e.
     Sigma = [8.33,    0,    0, 0;
                 0, 6.94,    0, 0;
                 0,    0, 2.31, 0]
</pre></blockquote>
<h4>See also</h4>
<a href=\"modelica://Modelica.Math.Matrices.eigenValues\">Matrices.eigenValues</a>
</HTML>"));
      end singularValues;

      function QR "Return the QR decomposition of a square matrix with optional column pivoting (A(:,p) = Q*R)"
        extends Modelica.Icons.Function;
        input Real A[:, :] "Rectangular matrix with size(A,1) >= size(A,2)";
        input Boolean pivoting = true "True if column pivoting is performed. True is default";
        output Real Q[size(A, 1), size(A, 2)] "Rectangular matrix with orthonormal columns such that Q*R=A[:,p]";
        output Real R[size(A, 2), size(A, 2)] "Square upper triangular matrix";
        output Integer p[size(A, 2)] "Column permutation vector";
      protected
        Integer nrow = size(A, 1);
        Integer ncol = size(A, 2);
        Real tau[size(A, 2)];
      algorithm
        assert(nrow >= ncol, "\nInput matrix A[" + String(nrow) + "," + String(ncol) + "] has more columns as rows.
This is not allowed when calling Modelica.Matrices.QR(A).");
        if pivoting then
          (Q, tau, p) := LAPACK.dgeqpf(A);
        else
          (Q, tau) := Modelica.Math.Matrices.LAPACK.dgeqrf(A);
          p := 1:ncol;
        end if;
// determine R
        R := zeros(ncol, ncol);
        for i in 1:ncol loop
          for j in i:ncol loop
            R[i, j] := Q[i, j];
          end for;
        end for;
// if isPresent(Q) then (not yet supported by Dymola)
        Q := LAPACK.dorgqr(Q, tau);
        annotation(
          Documentation(info = "<HTML>
<h4>Syntax</h4>
<blockquote><pre>
(Q,R,p) = Matrices.<b>QR</b>(A);
</pre></blockquote>
<h4>Description</h4>
<p>
This function returns the QR decomposition of
a rectangular matrix <b>A</b> (the number of columns of <b>A</b>
must be less than or equal to the number of rows):
</p>
<blockquote>
<p>
<b>Q</b>*<b>R</b> = <b>A</b>[:,<b>p</b>]
</p>
</blockquote>
<p>
where <b>Q</b> is a rectangular matrix that has orthonormal columns and
has the same size as A (<b>Q</b><sup>T</sup><b>Q</b>=<b>I</b>),
<b>R</b> is a square, upper triangular matrix and <b>p</b> is a permutation
vector. Matrix <b>R</b> has the following important properties:
</p>
<ul>
<li> The absolute value of a diagonal element of <b>R</b> is the largest
     value in this row, i.e.,
     abs(R[i,i]) &ge; abs(R[i,j]).</li>
<li> The diagonal elements of <b>R</b> are sorted according to size, such that
     the largest absolute value is abs(R[1,1]) and
     abs(R[i,i]) &ge; abs(R[j,j]) with i &lt; j. </li>
</ul>
<p>
This means that if abs(R[i,i]) &le; &epsilon; then abs(R[j,k]) &le; &epsilon;
for j &ge; i, i.e., the i-th row up to the last row of <b>R</b> have
small elements and can be treated as being zero.
This allows to, e.g., estimate the row-rank
of <b>R</b> (which is the same row-rank as <b>A</b>). Furthermore,
<b>R</b> can be partitioned in two parts
</p>
<blockquote>
<pre>
   <b>A</b>[:,<b>p</b>] = <b>Q</b> * [<b>R</b><sub>1</sub>, <b>R</b><sub>2</sub>;
                 <b>0</b>,  <b>0</b>]
</pre>
</blockquote>
<p>
where <b>R</b><sub>1</sub> is a regular, upper triangular matrix.
</p>
<p>
Note, the solution is computed with the LAPACK functions \"dgeqpf\"
and \"dorgqr\", i.e., by Householder transformations with
column pivoting. If <b>Q</b> is not needed, the function may be
called as: <code>(,R,p) = QR(A)</code>.
</p>
<h4>Example</h4>
<blockquote><pre>
  Real A[3,3] = [1,2,3;
                 3,4,5;
                 2,1,4];
  Real R[3,3];
<b>algorithm</b>
  (,R) := Matrices.QR(A);  // R = [-7.07.., -4.24.., -3.67..;
                                    0     , -1.73.., -0.23..;
                                    0     ,  0     ,  0.65..];
</pre></blockquote>
</HTML>"));
      end QR;

      function hessenberg "Return upper Hessenberg form of a matrix"
        extends Modelica.Icons.Function;
        import Modelica;
        import Modelica.Math.Matrices;
        input Real A[:, size(A, 1)] "Square matrix A";
        output Real H[size(A, 1), size(A, 2)] "Hessenberg form of A";
        output Real U[size(A, 1), size(A, 2)] "Transformation matrix";
      protected
        Real V[size(A, 1), size(A, 2)] "V=[v1,v2,..vn-1,0] with vi are vectors which define the elementary reflectors";
        Real tau[max(0, size(A, 1) - 1)] "Scalar factors of the elementary reflectors";
      algorithm
        (H, V, tau) := Matrices.Utilities.toUpperHessenberg(A, 1, size(A, 1));
        U := Matrices.LAPACK.dorghr(V, 1, size(A, 1), tau);
        annotation(
          Documentation(info = "<html>

<h4>Syntax</h4>
<blockquote><pre>
         H = Matrices.<b>hessenberg</b>(A);
    (H, U) = Matrices.<b>hessenberg</b>(A);
 </pre></blockquote>

<h4>Description</h4>
<p>
Function <b>hessenberg</b> computes the Hessenberg matrix <b>H</b> of matrix <b>A</b> as well as the orthogonal transformation matrix <b>U</b> that holds <b>H</b> = <b>U</b>'*<b>A</b>*<b>U</b>.
The Hessenberg form of a matrix is computed by repeated Householder similarity transformation. The elementary reflectors and the corresponding scalar factors are provided
by function \"Utilities.toUpperHessenberg()\". The transformation matrix <b>U</b> is then computed by
<a href=\"modelica://Modelica.Math.Matrices.LAPACK.dorghr\">LAPACK.dorghr</a>.
</p>

<h4>Example</h4>
<blockquote><pre>
 A  = [1, 2,  3;
       6, 5,  4;
       1, 0,  0];

 (H, U) = hessenberg(A);

  results in:

 H = [1.0,  -2.466,  2.630;
     -6.083, 5.514, -3.081;
      0.0,   0.919, -0.514]

 U = [1.0,    0.0,      0.0;
      0.0,   -0.9864,  -0.1644;
      0.0,   -0.1644,   0.9864]

  and therefore,

 U*H*transpose(U) = [1.0, 2.0, 3.0;
                     6.0, 5.0, 4.0;
                     1.0, 0.0, 0.0]

</pre></blockquote>

<h4>See also</h4>
<p>
<a href=\"modelica://Modelica.Math.Matrices.Utilities.toUpperHessenberg\">Matrices.Utilities.toUpperHessenberg</a>
</p>

</html>", revisions = "<html>
<ul>
<li><i>2010/05/31 </i>
       by Marcus Baur, DLR-RM</li>
</ul>
</html>"));
      end hessenberg;

      function realSchur "Return the real Schur form (rsf) S of a square matrix A, A=QZ*S*QZ'"
        extends Modelica.Icons.Function;
        import Modelica.Math.Matrices;
        input Real A[:, size(A, 1)] "Square matrix";
      public
        output Real S[size(A, 1), size(A, 2)] "Real Schur form of A";
        output Real QZ[size(A, 1), size(A, 2)] "Schur vector Matrix";
        output Real alphaReal[size(A, 1)] "Real part of eigenvalue=alphaReal+i*alphaImag";
        output Real alphaImag[size(A, 1)] "Imaginary part of eigenvalue=alphaReal+i*alphaImag";
      protected
        Integer info;
      algorithm
        if size(A, 1) > 1 then
          (S, QZ, alphaReal, alphaImag, info) := Matrices.LAPACK.dgees(A);
          assert(info == 0, "The output info of LAPACK.dgees should be zero, else if\n
     info < 0:  if info = -i, the i-th argument of dgees had an illegal value\n
     info > 0:  if INFO = i, and i is
               <= N: the QR algorithm failed to compute all the
                     eigenvalues; elements 1:ILO-1 and i+1:N of WR and WI
                     contain those eigenvalues which have converged; if
                     JOBVS = 'V', VS contains the matrix which reduces A
                     to its partially converged Schur form.\n");
        else
          S := A;
          QZ := fill(1, size(A, 1), size(A, 2));
          alphaReal := fill(1, size(A, 1));
          alphaImag := fill(0, size(A, 1));
        end if;
        annotation(
          Documentation(info = "<html>
<h4>Syntax</h4>

<blockquote><pre>
                            S = Matrices.<b>realSchur</b>(A);
(S, QZ, alphaReal, alphaImag) = Matrices.<b>realSchur</b>(A);
</pre></blockquote>

<h4>Description</h4>

<p>
Function <b>realSchur</b> calculates the real Schur form of a real square matrix <b>A</b>, i.e.
</p>

<blockquote><pre>
 <b>A</b> = <b>QZ</b>*<b>S</b>*transpose(<b>QZ</b>)
</pre></blockquote>

<p>
with the real nxn matrices <b>S</b> and <b>QZ</b>. <b>S</b> is a block upper triangular matrix with 1x1 and 2x2 blocks in the diagonal. <b>QZ</b> is an orthogonal matrix.
The 1x1 blocks contains the real eigenvalues of <b>A</b>. The 2x2 blocks [s11, s12; s21, s11] represents the conjugated complex pairs of eigenvalues, whereas the real parts of the eigenvalues
are the elements of the diagonal (s11). The imaginary parts are the positive and negative square roots of the product of the two elements s12 and s21 (imag = +-sqrt(s12*s21)).
</p>

<p>
The calculation in lapack.dgees is performed stepwise, i.e., using the internal methods of balancing and scaling of dgees.
</p>

<h4>Example</h4>
<blockquote><pre>
   Real A[3,3] = [1, 2, 3; 4, 5, 6; 7, 8, 9];
   Real T[3,3];
   Real Z[3,3];
   Real alphaReal[3];
   Real alphaImag[3];

<b>algorithm</b>
  (T, Z, alphaReal, alphaImag):=Modelica.Math.Matrices.realSchur(A);
//   T = [16.12, 4.9,   1.59E-015;
//        0,    -1.12, -1.12E-015;
//        0,     0,    -1.30E-015]
//   Z = [-0.23,  -0.88,   0.41;
//        -0.52,  -0.24,  -0.82;
//        -0.82,   0.4,    0.41]
//alphaReal = {16.12, -1.12, -1.32E-015}
//alphaImag = {0, 0, 0}
</pre></blockquote>

<h4>See also</h4>
<a href=\"modelica://Modelica.Math.Matrices.Utilities.reorderRSF\">Math.Matrices.Utilities.reorderRSF</a>

</html>", revisions = "<html>
<ul>
<li><i>2010/05/31 </i>
       by Marcus Baur, DLR-RM</li>
</ul>
</html>"));
      end realSchur;

      function cholesky "Return the Cholesky factorization of a symmetric positive definite matrix"
        extends Modelica.Icons.Function;
        import Modelica.Math.Matrices.LAPACK;
        input Real A[:, size(A, 1)] "Symmetric positive definite matrix";
        input Boolean upper = true "True if the right cholesky factor (upper triangle) should be returned";
        output Real H[size(A, 1), size(A, 2)] "Cholesky factor U (upper=true) or L (upper=false) for A = U'*U or A = L*L'";
      protected
        Integer n = size(A, 1);
        Integer info;
      algorithm
        if size(A, 1) > 0 then
          (H, info) := LAPACK.dpotrf(A, upper);
        else
          H := fill(0, 0, 0);
          info := 0;
        end if;
        if info < 0 then
          assert(info == 0, "Cholesky factorization failed in function \"Matrices.cholesky\" due to illegal value of input " + String(info) + " for LAPACK routine DPOTRF");
        else
          assert(info == 0, "Cholesky factorization failed in function \"Matrices.cholesky\" since matrix A is not positive definite");
        end if;
        if upper then
          for i in 2:n loop
            for j in 1:i - 1 loop
              H[i, j] := 0.0;
            end for;
          end for;
        else
          for i in 1:n - 1 loop
            for j in i + 1:n loop
              H[i, j] := 0.0;
            end for;
          end for;
        end if;
        annotation(
          Documentation(info = "<html>
<h4>Syntax</h4>

<blockquote><pre>
         H = Matrices.<b>cholesky</b>(A);
         H = Matrices.<b>cholesky</b>(A, upper=true);
 </pre></blockquote>

<h4>Description</h4>
<p>
Function <b>cholesky</b> computes the Cholesky factorization of a real symmetric positive definite matrix A.
The optional Boolean input \"upper\" specifies whether the upper or the lower triangular matrix is returned, i.e.
</p>

<blockquote><pre>
 A = H'*H   if upper is true (H is upper triangular)
 A = H*H'   if upper is false (H is lower triangular)
</pre></blockquote>

<p>
The computation is performed by <a href=\"modelica://Modelica.Math.Matrices.LAPACK.dpotrf\">LAPACK.dpotrf</a>.
</p>

<h4>Example</h4>

<blockquote><pre>
  A  = [1, 0,  0;
        6, 5,  0;
        1, -2,  2];
  S = A*transpose(A);

  H = Matrices.cholesky(S);

  results in:

  H = [1.0,  6.0,  1.0;
       0.0,  5.0, -2.0;
       0.0,  0.0,  2.0]

  with

  transpose(H)*H = [1.0,  6.0,   1;
                    6.0, 61.0,  -4.0;
                    1.0, -4.0,   9.0] //=S

</pre></blockquote>

</html>", revisions = "<html>
<ul>
<li><i>2010/05/31 </i>
       by Marcus Baur, DLR-RM</li>
</ul>

</html>"));
      end cholesky;

      function balance "Return a balanced form of matrix A to improve the condition of A"
        extends Modelica.Icons.Function;
        input Real A[:, size(A, 1)];
        output Real D[size(A, 1)] "diagonal(D)=T is transformation matrix, such that
                  B = inv(T)*A*T has smaller condition as A";
        output Real B[size(A, 1), size(A, 1)] "Balanced matrix (= inv(diagonal(D))*A*diagonal(D) )";
      protected
        Integer na = size(A, 1);
        Integer radix = 2 "Radix of exponent representation must be 'radix'
                  or a multiple of 'radix'";
        Integer radix2 = radix * radix;
        Boolean noconv = true;
        Integer i = 1;
        Integer j = 1;
        Real CO;
        Real RO;
        Real G;
        Real F;
        Real S;
        /*auxiliary variables*/
      algorithm
// B = inv(D)*A*D, so that cond(B)<=cond(A)
        D := ones(na);
        B := A;
        while noconv loop
          noconv := false;
          for i in 1:na loop
            CO := sum(abs(B[:, i])) - abs(B[i, i]);
            RO := sum(abs(B[i, :])) - abs(B[i, i]);
            G := RO / radix;
            F := 1;
            S := CO + RO;
            while not (CO >= G or CO == 0) loop
              F := F * radix;
              CO := CO * radix2;
            end while;
            G := RO * radix;
            while not (CO < G or RO == 0) loop
              F := F / radix;
              CO := CO / radix2;
            end while;
            if not (CO + RO) / F >= 0.95 * S then
              G := 1 / F;
              D[i] := D[i] * F;
              B[i, :] := B[i, :] * G;
              B[:, i] := B[:, i] * F;
              noconv := true;
            end if;
          end for;
        end while;
        annotation(
          Documentation(info = "<HTML>

<h4>Syntax</h4>
<blockquote><pre>
(D,B) = Matrices.<b>balance</b>(A);
</pre></blockquote>

<h4>Description</h4>

<p>
This function returns a vector D, such that B=inv(diagonal(D))*A*diagonal(D) has a
better condition as matrix A, i.e., conditionNumber(B) &le; conditionNumber(A). The elements of D
are multiples of 2 which means that this function does not introduce round-off errors.
Balancing attempts to make the norm of each row of B equal to the
norm of the respective column.
</p>

<p>
Balancing is used to minimize roundoff errors induced
through large matrix calculations like Taylor-series approximation
or computation of eigenvalues.
</p>

<h4>Example</h4>

<blockquote>
<pre>       - A = [1, 10,  1000; 0.01,  0,  10; 0.005,  0.01,  10]
       - Matrices.norm(A, 1);
         = 1020.0
       - (T,B)=Matrices.balance(A)
       - T
         = {256, 16, 0.5}
       - B
         =  [1,     0.625,   1.953125;
             0.16,  0,       0.3125;
             2.56,  0.32,   10.0]
       - Matrices.norm(B, 1);
         = 12.265625
</pre>
</blockquote>

<p>
The Algorithm is taken from
</p>
<dl>
<dt>H. D. Joos, G. Grbel:
<dd><b>RASP'91 Regulator Analysis and Synthesis Programs</b><br>
    DLR - Control Systems Group 1991
</dl>
<p>
which based on the <code>balance</code> function from EISPACK.
</p>

</html>", revisions = "<html>
<h4>Release Notes:</h4>
<ul>
<li><i>July 5, 2002</i>
       by H. D. Joos and Nico Walther<br>
       Implemented.
</li>
</ul>
</html>"));
      end balance;

      function trace "Return the trace of matrix A, i.e., the sum of the diagonal elements"
        extends Modelica.Icons.Function;
        input Real A[:, size(A, 1)] "Square matrix A";
        output Real result "Trace of A";
      algorithm
        result := sum(A[i, i] for i in 1:size(A, 1));
        annotation(
          Inline = true,
          Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
  r = Matrices.<b>trace</b>(A);
</pre></blockquote>

<h4>Description</h4>
<p>
This function computes the trace, i.e., the sum of the elements in the diagonal of matrix <b>A</b>.
</p>

<h4>Example</h4>
<blockquote><pre>
  A = [1, 3;
       2, 1];
  r = trace(A);

  results in:

  r = 2.0
</pre></blockquote>

</html>", revisions = "<html>
<ul>
<li><i>2010/05/31 </i>
       by Marcus Baur, DLR-RM</li>
</ul>
</html>"));
      end trace;

      function det "Return determinant of a matrix (computed by LU decomposition; try to avoid det(..))"
        extends Modelica.Icons.Function;
        input Real A[:, size(A, 1)];
        output Real result "Determinant of matrix A";
      protected
        Real LU[size(A, 1), size(A, 1)];
        Integer pivots[size(A, 1)];
      algorithm
        (LU, pivots) := Matrices.LU(A);
        result := product(LU[i, i] for i in 1:size(A, 1)) * product(if pivots[i] == i then 1 else -1 for i in 1:size(pivots, 1));
        annotation(
          Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
result = Matrices.<b>det</b>(A);
</pre></blockquote>

<h4>Description</h4>
<p>
This function returns the determinant \"result\" of matrix A
computed by a LU decomposition with row pivoting. For details about determinants, see
<a href=\"http://en.wikipedia.org/wiki/Determinant\">http://en.wikipedia.org/wiki/Determinant</a>.
Usually, this function should never be used, because
there are nearly always better numerical algorithms
as by computing the determinant. Examples:
</p>

<ul>
<li> Use <a href=\"modelica://Modelica.Math.Matrices.rank\">Matrices.rank</a>
     to compute whether det(A) = 0 (i.e., Matrices.rank(A) &lt; size(A,1)).</li>

<li> Use <a href=\"modelica://Modelica.Math.Matrices.solve\">Matrices.solve</a>
     to solve the linear equation A*x = b, instead of using determinants to
     compute the solution.</li>
</ul>

<h4>See also</h4>
<a href=\"modelica://Modelica.Math.Matrices.rank\">Matrices.rank</a>,
<a href=\"modelica://Modelica.Math.Matrices.solve\">Matrices.solve</a>
</html>"));
      end det;

      function inv "Return inverse of a matrix (try to avoid inv(..))"
        extends Modelica.Icons.Function;
        input Real A[:, size(A, 1)];
        output Real invA[size(A, 1), size(A, 2)] "Inverse of matrix A";
      protected
        Integer info;
        Integer pivots[size(A, 1)] "Pivot vector";
        Real LU[size(A, 1), size(A, 2)] "LU factors of A";
      algorithm
        (LU, pivots, info) := LAPACK.dgetrf(A);
        assert(info == 0, "Calculating an inverse matrix with function
\"Matrices.inv\" is not possible, since matrix A is singular.");
        invA := LAPACK.dgetri(LU, pivots);
        annotation(
          Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
invA = Matrices.<b>inv</b>(A);
</pre></blockquote>

<h4>Description</h4>
<p>
This function returns the inverse of matrix A, i.e., A*inv(A) = identity(size(A,1))
computed by a LU decomposition with row pivoting.
Usually, this function should not be used, because
there are nearly always better numerical algorithms
as by computing directly the inverse. Example:
</p>

<blockquote>
Use x = <a href=\"modelica://Modelica.Math.Matrices.solve\">Matrices.solve</a>(A,b)
to solve the linear equation A*x = b, instead of computing the solution by
x = inv(A)*b, because this is much more efficient and much more reliable.
</blockquote>

<h4>See also</h4>
<a href=\"modelica://Modelica.Math.Matrices.solve\">Matrices.solve</a>
<a href=\"modelica://Modelica.Math.Matrices.solve2\">Matrices.solve2</a>
</html>"));
      end inv;

      function rank "Return rank of a rectangular matrix (computed with singular values)"
        extends Modelica.Icons.Function;
        input Real A[:, :] "Matrix";
        input Real eps = 0 "If eps > 0, the singular values are checked against eps; otherwise eps=max(size(A))*norm(A)*Modelica.Constants.eps is used";
        output Integer result "Rank of matrix A";
      protected
        Integer n = min(size(A, 1), size(A, 2));
        Integer i = n;
        Real sigma[min(size(A, 1), size(A, 2))];
        Real eps2;
      algorithm
        result := 0;
        if n > 0 then
          sigma := Modelica.Math.Matrices.singularValues(A);
          eps2 := if eps > 0 then eps else max(size(A)) * sigma[1] * Modelica.Constants.eps;
          while i > 0 loop
            if sigma[i] > eps2 then
              result := i;
              i := 0;
            end if;
            i := i - 1;
          end while;
        end if;
        annotation(
          Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
result = Matrices.<b>rank</b>(A);
result = Matrices.<b>rank</b>(A,eps=0);
</pre></blockquote>

<h4>Description</h4>
<p>
This function returns the rank of a square or rectangular matrix A computed by singular value decomposition.
For details about the rank of a matrix, see
<a href=\"http://en.wikipedia.org/wiki/Matrix_rank\">http://en.wikipedia.org/wiki/Matrix_rank</a>.
To be more precise:
</p>

<ul>
<li> rank(A) returns the number of singular values of A that are larger than
     max(size(A))*norm(A)*Modelica.Constants.eps.</li>
<li> rank(A, eps) returns the number of singular values of A that are larger than \"eps\".</li>
</ul>

<h4>See also</h4>
<a href=\"modelica://Modelica.Math.Matrices.rcond\">Matrices.rcond</a>.
</html>"));
      end rank;

      function conditionNumber "Return the condition number norm(A)*norm(inv(A)) of a matrix A"
        extends Modelica.Icons.Function;
        input Real A[:, :] "Input matrix";
        input Real p(min = 1) = 2 "Type of p-norm (only allowed: 1, 2 or Modelica.Constants.inf)";
        output Real result = 0.0 "Condition number of matrix A";
      protected
        Real eps = 1e-25;
        Real eps2 = 10 * Modelica.Constants.eps;
        Real s[size(A, 1)] "singular values";
      algorithm
        if min(size(A)) > 0 then
          if p >= 2 - eps2 and p <= 2 + eps2 then
            s := Modelica.Math.Matrices.singularValues(A);
            if min(s) < eps then
              result := Modelica.Constants.inf;
            else
              result := max(s) / min(s);
            end if;
          else
            result := Modelica.Math.Matrices.norm(A, p) * Modelica.Math.Matrices.norm(Modelica.Math.Matrices.inv(A), p);
          end if;
        end if;
        annotation(
          Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
r = Matrices.<b>conditionNumber</b>(A);
</pre></blockquote>

<h4>Description</h4>
<p>
This function calculates the condition number (norm(A) * norm(inv(A))) of a general real matrix <b>A</b>, in either the 1-norm, 2-norm or the infinity-norm.
In the case of 2-norm the result is the ratio of the largest to the smallest singular value of <b>A</b>.
For more details, see <a href=\"http://en.wikipedia.org/wiki/Condition_number\">http://en.wikipedia.org/wiki/Condition_number</a>.
</p>

<h4>Example</h4>
<blockquote><pre>
  A = [1, 2;
       2, 1];
  r = conditionNumber(A);

  results in:

  r = 3.0
</pre></blockquote>

<h4>See also</h4>
<p>
<a href=\"modelica://Modelica.Math.Matrices.rcond\">Matrices.rcond</a>
</p>

</html>", revisions = "<html>
<ul>
<li><i>2010/05/31 </i>
       by Marcus Baur, DLR-RM</li>
</ul>
</html>"));
      end conditionNumber;

      function rcond "Return the reciprocal condition number of a matrix"
        extends Modelica.Icons.Function;
        input Real A[:, size(A, 1)] "Square real matrix";
        input Boolean inf = false "Is true if infinity norm is used and false for 1-norm";
        output Real rcond "Reciprocal condition number of A";
        output Integer info "Information";
      protected
        Real LU[size(A, 1), size(A, 1)] "LU factorization of matrix A, returned by dgetrf";
        Real anorm "Norm of matrix A";
        String normspec = if inf then "I" else "1" "Specifies the norm 1 or inf";
      algorithm
        if min(size(A)) > 0 then
          (LU, , info) := Modelica.Math.Matrices.LAPACK.dgetrf(A);
          anorm := Modelica.Math.Matrices.LAPACK.dlange(A, normspec);
          (rcond, info) := Modelica.Math.Matrices.LAPACK.dgecon(LU, inf, anorm);
        else
          rcond := Modelica.Constants.inf;
          info := 0;
        end if;
        annotation(
          Documentation(info = "<HTML>
<h4>Syntax</h4>
<blockquote><pre>
r = Matrices.<b>rcond</b>(A);
</pre></blockquote>

<h4>Description</h4>
<p>
This function estimates the reciprocal of the condition number (norm(A) * norm(inv(A))) of a general real matrix <b>A</b>, in either the 1-norm or
the infinity-norm, using the LAPACK function <a href=\"modelica://Modelica.Math.Matrices.LAPACK.dgecon\">DGECON</a>.
If rcond(A) is near 1.0, <b>A</b> is well conditioned and <b>A</b> is ill conditioned if rcond(A) is near zero.
</p>

<h4>Example</h4>
<blockquote><pre>
  A = [1, 2;
       2, 1];
  r = rcond(A);

  results in:

  r = 0.3333
</pre></blockquote>

<h4>See also</h4>
<p>
<a href=\"modelica://Modelica.Math.Matrices.conditionNumber\">Matrices.conditionNumber</a>
</p>

</HTML>", revisions = "<html>
<ul>
<li><i>2010/05/31 </i>
       by Marcus Baur, DLR-RM</li>
</ul>
</html>"));
      end rcond;

      function norm "Return the p-norm of a matrix"
        extends Modelica.Icons.Function;
        input Real A[:, :] "Input matrix";
        input Real p(min = 1) = 2 "Type of p-norm (only allowed: 1, 2 or Modelica.Constants.inf)";
        output Real result = 0.0 "p-norm of matrix A";
      protected
        Real eps = 10 * Modelica.Constants.eps;
      algorithm
        if min(size(A)) > 0 then
          if p >= 1 - eps and p <= 1 + eps then
            for i in 1:size(A, 2) loop
              result := max(result, sum(abs(A[:, i])));
            end for;
          elseif p >= 2 - eps and p <= 2 + eps then
            result := max(singularValues(A));
          elseif p >= Modelica.Constants.inf then
            for i in 1:size(A, 1) loop
              result := max(result, sum(abs(A[i, :])));
            end for;
          else
            assert(false, "Optional argument \"p\" (= " + String(p) + ") of function \"norm\" must be
1, 2 or Modelica.Constants.inf");
          end if;
        end if;
// column sum norm
// largest singular value
// row sum norm
        annotation(
          Documentation(info = "<HTML>
<h4>Syntax</h4>
<blockquote><pre>
Matrices.<b>norm</b>(A);
Matrices.<b>norm</b>(A, p=2);
</pre></blockquote>

<h4>Description</h4>
<p>
The function call \"<code>Matrices.norm(A)</code>\" returns the
2-norm of matrix A, i.e., the largest singular value of A.<br>
The function call \"<code>Matrices.norm(A, p)</code>\" returns the
p-norm of matrix A. The only allowed values for p are
</p>

<ul>
<li> \"p=1\": the largest column sum of A</li>
<li> \"p=2\": the largest singular value of A</li>
<li> \"p=Modelica.Constants.inf\": the largest row sum of A</li>
</ul>

<p>
Note, for any matrices A1, A2 the following inequality holds:
</p>

<blockquote><pre>
Matrices.<b>norm</b>(A1+A2,p) &le; Matrices.<b>norm</b>(A1,p) + Matrices.<b>norm</b>(A2,p)
</pre></blockquote>

<p>
Note, for any matrix A and vector v the following inequality holds:
</p>

<blockquote><pre>
Vectors.<b>norm</b>(A*v,p) &le; Matrices.<b>norm</b>(A,p)*Vectors.<b>norm</b>(A,p)
</pre></blockquote>

<h4>See also</h4>
<p>
<a href=\"modelica://Modelica.Math.Matrices.frobeniusNorm\">Matrices.frobeniusNorm</a>
</p>

</HTML>"));
      end norm;

      function frobeniusNorm "Return the Frobenius norm of a matrix"
        extends Modelica.Icons.Function;
        input Real A[:, :] "Input matrix";
        output Real result "Frobenius norm of matrix A";
      algorithm
        result := if min(size(A)) > 0 then sqrt(sum(A .* A)) else 0.0;
        annotation(
          Inline = true,
          Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
  r = Matrices.<b>frobeniusNorm</b>(A);
</pre></blockquote>

<h4>Description</h4>

<p>
This function computes the Frobenius norm of a general real matrix <b>A</b>, i.e., the square root of the sum of the squares of all elements.
</p>

<h4>Example</h4>
<blockquote><pre>
  A = [1, 2;
       2, 1];
  r = frobeniusNorm(A);

  results in:

  r = 3.162;

</pre></blockquote>

<h4>See also</h4>
<p>
<a href=\"modelica://Modelica.Math.Matrices.norm\">Matrices.norm</a>
</p>

</html>", revisions = "<html>
<ul>
<li><i>2010/05/31 </i>
       by Marcus Baur, DLR-RM</li>
</ul>
</html>"));
      end frobeniusNorm;

      function nullSpace "Return the orthonormal nullspace of a matrix"
        extends Modelica.Icons.Function;
        input Real A[:, :] "Input matrix";
        output Real Z[size(A, 2), :] "Orthonormal nullspace of matrix A";
        output Integer nullity "Nullity, i.e., the dimension of the nullspace";
      protected
        Real V[size(A, 2), size(A, 2)] "Right orthogonal matrix";
        Real sigma[min(size(A, 1), size(A, 2))] "singular values";
        Integer rank "rank of matrix A";
        Real eps "tolerance for rank determination";
        Integer n = min(size(A, 1), size(A, 2));
        Integer i = n;
      algorithm
        (sigma, , V) := Modelica.Math.Matrices.singularValues(A);
        V := transpose(V);
// rank computation
        eps := max(size(A, 1), size(A, 2)) * max(sigma) * Modelica.Constants.eps;
        rank := 0;
        if n > 0 then
          while i > 0 loop
            if sigma[i] > eps then
              rank := i;
              i := 0;
            end if;
            i := i - 1;
          end while;
        end if;
        Z := V[:, rank + 1:size(A, 2)];
// nullspace computation
        nullity := size(A, 2) - rank;
// nullity
        annotation(
          Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
           Z = Matrices.<b>nullspace</b>(A);
(Z, nullity) = Matrices.<b>nullspace</b>(A);
</pre></blockquote>

<h4>Description</h4>
<p>
This function calculates an orthonormal basis <b>Z</b>=[<b>z</b>_1, <b>z</b>_2, ...] of the nullspace of a matrix <b>A</b>, i.e., <b>A</b>*<b>z</b>_i=<b>0</b>.
</p>

<p>
The nullspace is obtained by SVD method. That is, matrix <b>A</b> is decomposed into the matrices <b>S</b>, <b>U</b>, <b>V</b>:
</p>

<blockquote><pre>
 <b>A</b> = <b>U</b>*<b>S</b>*transpose(<b>V</b>)
</pre></blockquote>

<p>
with the orthonormal matrices <b>U</b> and <b>V</b> and the matrix <b>S</b> with
</p>

<blockquote><pre>
 <b>S</b> = [<b>S</b>1, <b>0</b>]
 <b>S</b>1 = [diag(s); <b>0</b>]
</pre></blockquote>

<p>
and the singular values <b>s</b>={s1, s2, ..., sr} of <b>A</b> and r=rank(<b>A</b>). Note, that <b>S</b> has the same size as <b>A</b>. Since <b>U</b> and <b>V</b> are orthonormal we may write
</p>

<blockquote><pre>
 transpose(<b>U</b>)*<b>A</b>*<b>V</b> = [<b>S</b>1, <b>0</b>].
</pre></blockquote>

<p>
Matrix <b>S</b>1 obviously has full column rank and therefore, the left n-r rows (n is the number of columns of <b>A</b> or <b>S</b>) of matrix <b>V</b> span a nullspace of <b>A</b>.
</p>

<p>
The nullity of matrix <b>A</b> is the dimension of the nullspace of <b>A</b>. In view of the above, it becomes clear that nullity holds
</p>
<blockquote><pre>
 nullity = n - r
</pre></blockquote>
<p>
with
</p>
<blockquote><pre>
 n = number of columns of matrix <b>A</b>
 r = rank(<b>A</b>)
</pre></blockquote>

<h4>Example</h4>
<blockquote><pre>
  A = [1, 2,  3, 1;
       3, 4,  5, 2;
      -1, 2, -3, 3];
  (Z, nullity) = nullspace(A);

  results in:

  Z=[0.1715;
    -0.686;
     0.1715;
     0.686]

  nullity = 1
</pre></blockquote>

<h4>See also</h4>
<p>
<a href=\"modelica://Modelica.Math.Matrices.singularValues\">Matrices.singularValues</a>
</p>
</html>", revisions = "<html>
<ul>
<li><i>2010/05/31 </i>
       by Marcus Baur, DLR-RM</li>
</ul>
</html>"));
      end nullSpace;

      function exp "Return the exponential of a matrix by adaptive Taylor series expansion with scaling and balancing"
        extends Modelica.Icons.Function;
        input Real A[:, size(A, 1)];
        input Real T = 1;
        output Real phi[size(A, 1), size(A, 1)] "= exp(A*T)";
      protected
        parameter Integer nmax = 21;
        /*max number of iterations*/
        parameter Integer na = size(A, 1);
        Integer j = 1;
        Integer k = 0;
        Boolean done = false;
        Real Anorm;
        Real Tscaled = 1;
        Real Atransf[na, na];
        Real D[na, na];
        /*D: dummy variable for psi*/
        Real M[na, na];
        /*M: dummy matrix*/
        Real Diag[na];
        /*diagonal transformation matrix for balancing*/

        encapsulated function columnNorm "Returns the column norm of a matrix"
          import Modelica;
          extends Modelica.Icons.Function;
          input Real A[:, :] "Input matrix";
          output Real result = 0.0 "1-norm of matrix A";
        algorithm
          for i in 1:size(A, 2) loop
            result := max(result, sum(abs(A[:, i])));
          end for;
        end columnNorm;
      algorithm
// balancing of A
        (Diag, Atransf) := balance(A);
// scaling of T until norm(A)*/(2^k) < 1
        Tscaled := T;
/*Anorm: column-norm of matrix A*/
        Anorm := columnNorm(Atransf);
        Anorm := Anorm * T;
        while Anorm >= 0.5 loop
          Anorm := Anorm / 2;
          Tscaled := Tscaled / 2;
          k := k + 1;
        end while;
// Computation of psi by Taylor-series approximation
        M := identity(na);
        D := M;
        while j < nmax and not done loop
          M := Atransf * M * Tscaled / j;
          if columnNorm(D + M - D) == 0 then
            done := true;
          else
            D := M + D;
            j := j + 1;
          end if;
        end while;
//stop if the new element of the series is small
// re-scaling
        for i in 1:k loop
          D := D * D;
        end for;
// re-balancing: psi := diagonal(Diag)*D*inv(diagonal(Diag));
        for j in 1:na loop
          for k in 1:na loop
            phi[j, k] := D[j, k] * Diag[j] / Diag[k];
          end for;
        end for;
        annotation(
          Documentation(info = "<HTML>
<h4>Syntax</h4>
<blockquote><pre>
phi = Matrices.<b>exp</b>(A);
phi = Matrices.<b>exp</b>(A,T=1);
</pre></blockquote>

<h4>Description</h4>
<p>
This function computes the exponential e<sup><b>A</b>T</sup> of matrix <b>A</b>, i.e.
</p>
<blockquote>
<pre>                            (<b>A</b>T)^2   (<b>A</b>T)^3
     <font size=4> <b>&Phi;</b></font> = e^(<b>A</b>T) = <b>I</b> + <b>A</b>T + ------ + ------ + ....
                              2!       3!
</pre></blockquote>

<p>where e=2.71828..., <b>A</b> is an n x n matrix with real elements and T is a real number,
e.g., the sampling time.
<b>A</b> may be singular. With the exponential of a matrix it is, e.g., possible
to compute the solution of a linear system of differential equations</p>
<pre>    der(<b>x</b>) = <b>A</b>*<b>x</b>   ->   <b>x</b>(t0 + T) = e^(<b>A</b>T)*x(t0)
</pre>

<h4>Algorithmic details:</h4>

<p>The algorithm is taken from</p>
<dl>
<dt>H. D. Joos, G. Gruebel:
<dd><b>RASP'91 Regulator Analysis and Synthesis Programs</b><br>
    DLR - Control Systems Group 1991
</dl>
<p>The following steps are performed to calculate the exponential of A:</p>
<ol>
  <li>Matrix <b>A</b> is balanced <br>
  (= is transformed with a diagonal matrix <b>D</b>, such that inv(<b>D</b>)*<b>A</b>*<b>D</b>
  has a smaller condition as <b>A</b>).</li>
  <li>The scalar T is divided by a multiple of 2 such that norm(
       inv(<b>D</b>)*<b>A</b>*<b>D</b>*T/2^k ) &lt; 0.5. Note, that (1) and (2) are implemented such that no round-off errors
  are introduced.</li>
  <li>The matrix from (2) is approximated by explicitly performing the Taylor
  series expansion with a variable number of terms.
  Truncation occurs if a new term does no longer contribute to the value of <b>&Phi;</b>
  from the previous iteration.</li>
  <li>The resulting matrix is transformed back, by reverting the steps of (2)
  and (1).</li>
</ol>
<p>In several sources it is not recommended to use Taylor series expansion to
calculate the exponential of a matrix, such as in 'C.B. Moler and C.F. Van Loan:
Nineteen dubious ways to compute the exponential of a matrix. SIAM Review 20,
pp. 801-836, 1979' or in the documentation of m-file expm2 in Matlab version 6
(http://www.MathWorks.com) where it is stated that 'As a practical numerical
method, this is often slow and inaccurate'. These statements are valid for a
direct implementation of the Taylor series expansion, but <i>not</i> for the
implementation variant used in this function.
</p>

</html>", revisions = "<html>
<p><b>Release Notes:</b></p>
<ul>
<li><i>July 5, 2002</i>
       by H. D. Joos and Nico Walther<br>
       Implemented.
</li>
</ul>
</html>"));
      end exp;

      function integralExp "Return the exponential and the integral of the exponential of a matrix"
        extends Modelica.Icons.Function;
        input Real A[:, size(A, 1)];
        input Real B[size(A, 1), :];
        input Real T = 1;
        output Real phi[size(A, 1), size(A, 1)] "= exp(A*T)";
        output Real gamma[size(A, 1), size(B, 2)] "= integral(phi)*B";
      protected
        parameter Integer nmax = 21;
        /*max number of iterations*/
        parameter Integer na = size(A, 1);
        Integer j = 2;
        Integer k = 0;
        Boolean done = false;
        Real Anorm;
        Real Tscaled = 1;
        Real Atransf[na, na];
        Real Psi[na, na];
        /*Psi: dummy variable for psi*/
        Real M[na, na];
        /*M: dummy matrix*/
        Real Diag[na];
        /*diagonal transformation matrix for balancing*/

        encapsulated function columnNorm "Returns the column norm of a matrix"
          import Modelica;
          extends Modelica.Icons.Function;
          input Real A[:, :] "Input matrix";
          output Real result = 0.0 "1-norm of matrix A";
        algorithm
          for i in 1:size(A, 2) loop
            result := max(result, sum(abs(A[:, i])));
          end for;
        end columnNorm;
      algorithm
// balancing of A
        (Diag, Atransf) := balance(A);
// scaling of T until norm(A)*/(2^k) < 0.5
        Tscaled := T;
/*Anorm: column-norm of matrix A*/
// Anorm := norm(Atransf, 1);
        Anorm := columnNorm(Atransf);
        Anorm := Anorm * T;
        while Anorm >= 0.5 loop
          Anorm := Anorm / 2;
          Tscaled := Tscaled / 2;
          k := k + 1;
        end while;
// Computation of psi by Taylor-series approximation
        M := identity(na) * Tscaled;
        Psi := M;
        while j < nmax and not done loop
          M := Atransf * M * Tscaled / j;
          if columnNorm(Psi + M - Psi) == 0 then
            done := true;
          else
            Psi := M + Psi;
            j := j + 1;
          end if;
        end while;
//stop if the new element of the series is small
// if norm((Psi + M) - Psi, 1) == 0 then
// re-scaling
        for j in 1:k loop
          Psi := Atransf * Psi * Psi + 2 * Psi;
        end for;
// re-balancing: psi := diagonal(Diag)*D*inv(diagonal(Diag));
        for j in 1:na loop
          for k in 1:na loop
            Psi[j, k] := Psi[j, k] * Diag[j] / Diag[k];
          end for;
        end for;
        gamma := Psi * B;
        phi := A * Psi + identity(na);
        annotation(
          Documentation(info = "<HTML>
<h4>Syntax</h4>
<blockquote><pre>
(phi,gamma) = Matrices.<b>integralExp</b>(A,B);
(phi,gamma) = Matrices.<b>integralExp</b>(A,B,T=1);
</pre></blockquote>

<h4>Description</h4>
<p>
This function computes the exponential phi = e^(<b>A</b>T) of matrix <b>A</b>
and the integral gamma = integral(phi*dt)*B.
</p>

<p>
The function uses a Taylor series expansion with Balancing and
scaling/squaring to approximate the integral <b>&Psi;</b> of the matrix
exponential <b>&Phi;</b>=e^(AT):
</p>
<pre>                                 AT^2   A^2 * T^3          A^k * T^(k+1)
        <b>&Psi;</b> = int(e^(As))ds = IT + ---- + --------- + ... + --------------
                                  2!        3!                (k+1)!
</pre>
<p>
<b>&Phi;</b> is calculated through <b>&Phi;</b> = I + A*<b>&Psi;</b>, so A may be singular. <b>&Gamma;</b> is
simply <b>&Psi;</b>*B.
</p>
<p>The algorithm runs in the following steps:</p>
<ol>
  <li>Balancing</li>
  <li>Scaling</li>
  <li>Taylor series expansion</li>
  <li>Re-scaling</li>
  <li>Re-Balancing</li>
</ol>
<p>Balancing put the bad condition of a square matrix <i>A</i> into a diagonal
transformation matrix <i>D</i>. This reduce the effort of following calculations.
Afterwards the result have to be re-balanced by transformation D*A<small>transf</small>
*inv(D).<br>
Scaling halfen T&nbsp; k-times, until the norm of A*T is less than 0.5. This
guarantees minimum rounding errors in the following series
expansion. The re-scaling based on the equation&nbsp; exp(A*2T) = exp(AT)^2.
The needed re-scaling formula for psi thus becomes:
</p>
<pre>         <b>&Phi;</b> = <b>&Phi;</b>'*<b>&Phi;</b>'
   I + A*<b>&Psi;</b> = I + 2A*<b>&Psi;</b>' + A^2*<b>&Psi;</b>'^2
         <b>&Psi;</b> = A*<b>&Psi;</b>'^2 + 2*<b>&Psi;</b>'
</pre>
<p>
where psi' is the scaled result from the series expansion while psi is the
re-scaled matrix.
</p>
<p>
The function is normally used to discretize a state-space system as the
zero-order-hold equivalent:
</p>
<pre>      x(k+1) = <b>&Phi;</b>*x(k) + <b>&Gamma;</b>*u(k)
        y(k) = C*x(k) + D*u(k)
</pre>
<p>
The zero-order-hold sampling, also known as step-invariant method, gives
exact values of the state variables, under the assumption that the control
signal u is constant between the sampling instants. Zero-order-hold sampling
is described in
</p>
<dl>
<dt>K. J. Astroem, B. Wittenmark:
<dd><b>Computer Controlled Systems - Theory and Design</b><br>
    Third Edition, p. 32
</dl>
<pre><b>Syntax:</b>
      (phi,gamma) = Matrices.expIntegral(A,B,T)
                       A,phi: [n,n] square matrices
                     B,gamma: [n,m] input matrix
                           T: scalar, e.g., sampling time
</pre>
<p>
The Algorithm to calculate psi is taken from
</p>
<dl>
<dt>H. D. Joos, G. Gruebel:
<dd><b>RASP'91 Regulator Analysis and Synthesis Programs</b><br>
    DLR - Control Systems Group 1991
</dl>
</html>", revisions = "<html>
<p><b>Release Notes:</b></p>
<ul>
<li><i>July 5, 2002</i>
       by H. D. Joos and Nico Walther<br>
       Implemented.
</li>
</ul>
</html>"));
      end integralExp;

      function integralExpT "Return the exponential, the integral of the exponential, and time-weighted integral of the exponential of a matrix"
        extends Modelica.Icons.Function;
        input Real A[:, size(A, 1)];
        input Real B[size(A, 1), :];
        input Real T = 1;
        output Real phi[size(A, 1), size(A, 1)] "= exp(A*T)";
        output Real gamma[size(A, 1), size(B, 2)] "= integral(phi)*B";
        output Real gamma1[size(A, 1), size(B, 2)] "= integral((T-t)*exp(A*t))*B";
      protected
        Integer nmax = 200;
        /*max number of iterations*/
        parameter Integer na = size(A, 1);
        parameter Integer nb = size(B, 2);
        Integer j = 1;
        Boolean done = false;
        Real F[na + 2 * nb, na + 2 * nb];
      algorithm
        F := [A, B, zeros(na, nb); zeros(2 * nb, na), zeros(2 * nb, nb), [identity(nb); zeros(nb, nb)]];
        F := exp(F, T);
        phi := F[1:na, 1:na];
        gamma := F[1:na, na + 1:na + nb];
        gamma1 := F[1:na, na + nb + 1:na + 2 * nb];
        annotation(
          Documentation(info = "<HTML>
<blockquote><pre>
(phi,gamma,gamma1) = Matrices.<b>integralExp</b>(A,B);
(phi,gamma,gamma1) = Matrices.<b>integralExp</b>(A,B,T=1);
</pre></blockquote>

<h4>Description</h4>
<p>
This function computes the exponential phi = e^(<b>A</b>T) of matrix <b>A</b>
and the integral gamma = integral(phi*dt)*B and the integral
integral((T-t)*exp(A*t)*dt)*B, where A is a square (n,n) matrix and
B, gamma, and gamma1 are (n,m) matrices.
</p>

<p>
The function calculates the matrices phi,gamma,gamma1 through the equation:
</p>
<pre>                                 [ A B 0 ]
[phi gamma gamma1] = [I 0 0]*exp([ 0 0 I ]*T)
                                 [ 0 0 0 ]
</pre>

<p>
The matrices define the discretized first-order-hold equivalent of
a state-space system:
<pre>      x(k+1) = phi*x(k) + gamma*u(k) + gamma1/T*(u(k+1) - u(k))
</pre>
<p>
The first-order-hold sampling, also known as ramp-invariant method, gives
more smooth control signals as the ZOH equivalent. First-order-hold sampling
is, e.g., described in
</p>

<dl>
<dt>K. J. Astroem, B. Wittenmark:
<dd><b>Computer Controlled Systems - Theory and Design</b><br>
    Third Edition, p. 256
</dl>

</html>", revisions = "<html>
<p><b>Release Notes:</b></p>
<ul>
<li><i>July 31, 2002</i>
       by Nico Walther<br>
       Realized.
</li>
</ul>
</html>"));
      end integralExpT;

      //
      // protected

      function continuousLyapunov "Return solution X of the continuous-time Lyapunov equation X*A + A'*X = C"
        extends Modelica.Icons.Function;
        import Modelica.Math.Matrices;
        input Real A[:, size(A, 1)] "Square matrix A in X*A + A'*X = C";
        input Real C[size(A, 1), size(A, 2)] "Square matrix C in X*A + A'*X = C";
        input Boolean ATisSchur = false "True if transpose(A) has already real Schur form";
        input Real eps = Modelica.Math.Matrices.norm(A, 1) * 10 * 1e-15 "Tolerance eps";
      protected
        Integer n = size(A, 1);
        Real R[size(A, 1), size(A, 2)] "rsf of A', i.e., R=U'A'U";
        Real U[size(A, 1), size(A, 2)] "transformation matrix U for R=U'A'U";
        Real D[size(A, 1), size(A, 2)] "Matrix D=U'*C*U";
        Real R11[size(A, 1), size(A, 2)];
        Real R22[size(A, 1), size(A, 2)];
        Real R12[size(A, 1), size(A, 2)];
        Real R21[size(A, 1), size(A, 2)];
        Real R2[2 * size(A, 1), 2 * size(A, 2)];
        Real I[size(A, 1), size(A, 1)] = identity(size(A, 1));
        Real y[2 * size(A, 1)];
        Real c[2 * size(A, 1)];
        Real CC[size(A, 1), 2];
        Integer k;
      public
        output Real X[size(A, 1), size(A, 2)] "Solution X of the Lyapunov equation X*A + A'*X = C";
      algorithm
        if n > 1 then
          if ATisSchur then
            R := transpose(A);
            U := identity(n);
            D := C;
          else
            (R, U) := Modelica.Math.Matrices.realSchur(transpose(A));
            D := transpose(U) * C * U;
          end if;
          X := zeros(n, n);
          R22 := R;
          for i1 in 1:n loop
            R22[i1, i1] := R[i1, i1] + R[n, n];
          end for;
          if abs(R[n, n - 1]) < eps then
            X[:, n] := Matrices.solve(R22, D[:, n]);
            k := n - 1;
          else
            R11 := R;
            R12 := zeros(n, n);
            R21 := zeros(n, n);
            for i1 in 1:n loop
              R11[i1, i1] := R[i1, i1] + R[n - 1, n - 1];
              R12[i1, i1] := R[n - 1, n];
              R21[i1, i1] := R[n, n - 1];
            end for;
            R2 := [R11, R12; R21, R22];
            c := cat(1, D[:, n - 1], D[:, n]);
            y := Matrices.solve(R2, c);
            X[:, n - 1] := y[1:n];
            X[:, n] := y[n + 1:2 * n];
            k := n - 2;
          end if;
          while k > 1 loop
            R22 := R;
            for i1 in 1:n loop
              R22[i1, i1] := R[i1, i1] + R[k, k];
            end for;
            if abs(R[k, k - 1]) < eps then
              X[:, k] := Matrices.solve(R22, D[:, k] - vector(X[:, k + 1:n] * matrix(R[k, k + 1:n])));
              k := k - 1;
            else
              R11 := R;
              R12 := zeros(n, n);
              R21 := zeros(n, n);
              for i1 in 1:n loop
                R11[i1, i1] := R[i1, i1] + R[k - 1, k - 1];
                R12[i1, i1] := R[k - 1, k];
                R21[i1, i1] := R[k, k - 1];
              end for;
              R2 := [R11, R12; R21, R22];
              CC := D[:, k - 1:k] - X[:, k + 1:n] * transpose(R[k - 1:k, k + 1:n]);
              c := cat(1, CC[:, 1], CC[:, 2]);
              y := Matrices.solve(R2, c);
              X[:, k - 1] := y[1:n];
              X[:, k] := y[n + 1:2 * n];
              k := k - 2;
            end if;
          end while;
          if k == 1 then
            R22 := R;
            for i1 in 1:n loop
              R22[i1, i1] := R[i1, i1] + R[1, 1];
            end for;
            X[:, 1] := Matrices.solve(R22, D[:, 1] - vector(X[:, 2:n] * matrix(R[1, 2:n])));
          end if;
          if not ATisSchur then
            X := U * X * transpose(U);
          end if;
        elseif n == 1 then
          X[1, 1] := C[1, 1] / (2 * A[1, 1]);
        else
          X := fill(0, 0, 0);
        end if;
// Calculate the last 1 or 2 columns of X
// solve 2nx2n equation for 2x2 Schur bump with Kronecker product and vec operator approach
// Calculate the rest of X
//real eigenvalue
// conjugated complex eigenvalues
// k=1 or k=0
// if k=1 the first column (if there exist a real eigenvalue) has to be calculated separately
// transform X corresponding to the original form
//simple scalar equation
        annotation(
          Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
         X = Matrices.<b>continuousLyapunov</b>(A, C);
         X = Matrices.<b>continuousLyapunov</b>(A, C, ATisSchur, eps);
</pre></blockquote>

<h4>Description</h4>
<p>
This function computes the solution <b>X</b> of the continuous-time Lyapunov equation
</p>

<blockquote><pre>
 <b>X</b>*<b>A</b> + <b>A</b>'*<b>X</b> = <b>C</b>
</pre></blockquote>

<p>
using the Schur method for Lyapunov equations proposed by Bartels and Stewart [1].
</p>

<p>
In a nutshell, the problem is reduced to the corresponding problem
</p>

<blockquote><pre>
 <b>Y</b>*<b>R</b>' + <b>R</b>*<b>Y</b> = <b>D</b>
</pre></blockquote>

<p>
with <b>R</b>=<b>U</b>'*<b>A'</b>*<b>U</b> is the real Schur form of <b>A</b>' and <b>D</b>=<b>U</b>'*<b>C</b>*<b>U</b> and <b>Y</b>=<b>U</b>'*<b>X</b>*<b>U</b>
are the corresponding transformations of <b>C</b> and <b>X</b>. This problem is solved sequentially for the 1x1 or 2x2 Schur blocks by exploiting the block triangular form of <b>R</b>.
Finally the solution of the original problem is recovered as <b>X</b>=<b>U</b>*<b>Y</b>*<b>U</b>'.<br>
The Boolean input \"ATisSchur\" indicates to omit the transformation to Schur in the case that <b>A</b>' has already Schur form.
</p>

<h4>References</h4>
<pre>
  [1] Bartels, R.H. and Stewart G.W.
      Algorithm 432: Solution of the matrix equation AX + XB = C.
      Comm. ACM., Vol. 15, pp. 820-826, 1972.
</pre>

<h4>Example</h4>
<blockquote><pre>
  A = [1, 2,  3,  4;
       3, 4,  5, -2;
      -1, 2, -3, -5;
       0, 2,  0,  6];

  C =  [-2, 3, 1, 0;
        -6, 8, 0, 1;
         2, 3, 4, 5;
        0, -2, 0, 0];

  X = continuousLyapunov(A, C);

  results in:

  X = [1.633, -0.761,  0.575, -0.656;
      -1.158,  1.216,  0.047,  0.343;
      -1.066, -0.052, -0.916,  1.61;
      -2.473,  0.717, -0.986,  1.48]
</pre></blockquote>

<h4>See also</h4>
<p>
<a href=\"modelica://Modelica.Math.Matrices.continuousSylvester\">Matrices.continuousSylvester</a>,
<a href=\"modelica://Modelica.Math.Matrices.discreteLyapunov\">Matrices.discreteLyapunov</a>
</p>

</html>", revisions = "<html>
<ul>
<li><i>2010/05/31 </i>
       by Marcus Baur, DLR-RM</li>
</ul>
</html>"));
      end continuousLyapunov;

      function continuousSylvester "Return solution X of the continuous-time Sylvester equation A*X + X*B = C"
        extends Modelica.Icons.Function;
        import Modelica.Math.Matrices;
        input Real A[:, :] "Square matrix A";
        input Real B[:, :] "Square matrix B";
        input Real C[size(A, 1), size(B, 2)] "Matrix C";
        input Boolean AisSchur = false "True if A has already real Schur form";
        input Boolean BisSchur = false "True if B has already real Schur form";
        output Real X[size(A, 1), size(B, 2)] "Solution of the continuous Sylvester equation";
      protected
        Integer n = size(A, 1);
        Integer m = size(B, 1);
        Real S[size(A, 1), size(A, 2)];
        Real T[size(B, 1), size(B, 2)];
        Real U[size(A, 1), size(A, 1)];
        Real V[size(B, 1), size(B, 1)];
        Real Chat[size(C, 1), size(C, 2)];
        Real scale;
        Integer info;
      algorithm
        if n > 1 and m > 1 then
          if AisSchur then
            S := A;
            U := identity(n);
          else
            (S, U) := Modelica.Math.Matrices.realSchur(A);
          end if;
          if BisSchur then
            T := B;
            V := identity(m);
          else
            (T, V) := Modelica.Math.Matrices.realSchur(B);
          end if;
          Chat := if AisSchur and BisSchur then C else if AisSchur then C * V else if BisSchur then transpose(U) * C else transpose(U) * C * V;
          (X, scale, info) := Matrices.LAPACK.dtrsyl(S, T, Chat);
          assert(info == 0, "Solving of Sylvester equation with Matrices.continuousSylvester was not successful.\n
                    The value of info is " + String(info) + ", but should be zero. A value unequal to zero means:\n
            < 0: if INFO = -i, the i-th argument had an illegal value\n
            = 1: A and B have common or very close eigenvalues; perturbed
                 values were used to solve the equation (but the matrices
                 A and B are unchanged).");
          X := if AisSchur and BisSchur then scale * X else if AisSchur then scale * X * transpose(V) else if BisSchur then scale * U * X else scale * U * X * transpose(V);
        else
          X := fill(0, n, m);
        end if;
        annotation(
          Documentation(info = "<html>
 <h4>Syntax</h4>
<blockquote><pre>
         X = Matrices.<b>continuousSylvester</b>(A, B, C);
         X = Matrices.<b>continuousSylvester</b>(A, B, C, AisSchur, BisSchur);
</pre></blockquote>

<h4>Description</h4>
<p>
Function <b>continuousSylvester</b> computes the solution <b>X</b> of the continuous-time Sylvester equation
</p>

<blockquote><pre>
 <b>A</b>*<b>X</b> + <b>X</b>*<b>B</b> = <b>C</b>.
</pre></blockquote>

<p>
using the Schur method for Sylvester equations proposed by Bartels and Stewart [1].
</p>

<p>
In a nutshell, the problem is reduced to the corresponding problem
</p>
<blockquote><pre>
 <b>S</b>*<b>Y</b> + <b>Y</b>*<b>T</b> = <b>D</b>.
</pre></blockquote>
<p>
with <b>S</b>=<b>U</b>'*<b>A</b>*<b>U</b> is the real Schur of <b>A</b>,  <b>T</b>=<b>V</b>'*<b>T</b>*<b>V</b> is the real Schur form of <b>B</b> and
<b>D</b>=<b>U</b>'*<b>C</b>*<b>V</b> and <b>Y</b>=<b>U</b>*<b>X</b>*<b>V</b>'
are the corresponding transformations of <b>C</b> and <b>X</b>. This problem is solved sequentially by exploiting the block triangular form of <b>S</b> and <b>T</b>.
Finally the solution of the original problem is recovered as <b>X</b>=<b>U</b>'*<b>Y</b>*<b>V</b>.<br>
The Boolean inputs \"AisSchur\" and \"BisSchur\" indicate to omit one or both of the transformation to Schur in the case that <b>A</b> and/or <b>B</b> have already Schur form.
</p>

<p>
The function applies LAPACK-routine DTRSYL. See <a href=\"modelica://Modelica.Math.Matrices.LAPACK.dtrsyl\">LAPACK.dtrsyl</a>
for more information.
</p>

<h4>References</h4>
<pre>
  [1] Bartels, R.H. and Stewart G.W.
      Algorithm 432: Solution of the matrix equation AX + XB = C.
      Comm. ACM., Vol. 15, pp. 820-826, 1972.
</pre>

<h4>Example</h4>
<blockquote><pre>
  A = [17.0,   24.0,   1.0,   8.0,   15.0 ;
       23.0,    5.0,   7.0,  14.0,   16.0 ;
        0.0,    6.0,  13.0,  20.0,   22.0;
        0.0,    0.0,  19.0,  21.0,    3.0 ;
        0.0,    0.0,   0.0,   2.0,    9.0];

  B =  [8.0, 1.0, 6.0;
        0.0, 5.0, 7.0;
        0.0, 9.0, 2.0];

  C = [62.0,  -12.0, 26.0;
       59.0,  -10.0, 31.0;
       70.0,  -6.0,   9.0;
       35.0,  31.0,  -7.0;
       36.0, -15.0,   7.0];

  X = continuousSylvester(A, B, C);

  results in:

  X = [0.0,  0.0,  1.0;
       1.0,  0.0,  0.0;
       0.0,  1.0,  0.0;
       1.0,  1.0, -1.0;
       2.0, -2.0,  1.0];
</pre></blockquote>
<h4>See also</h4>
<p>
<a href=\"modelica://Modelica.Math.Matrices.discreteSylvester\">Matrices.discreteSylvester</a>,
<a href=\"modelica://Modelica.Math.Matrices.continuousLyapunov\">Matrices.continuousLyapunov</a>
</p>

</html>", revisions = "<html>
<ul>
<li><i>2010/05/31 </i>
       by Marcus Baur, DLR-RM</li>
</ul>
</html>"));
      end continuousSylvester;

      function continuousRiccati "Return solution X of the continuous-time algebraic Riccati equation A'*X + X*A - X*B*inv(R)*B'*X + Q = 0 (care)"
        extends Modelica.Icons.Function;
        import Modelica.Math.Matrices;
        input Real A[:, size(A, 1)] "Square matrix A in CARE";
        input Real B[size(A, 1), :] "Matrix B in CARE";
        input Real R[size(B, 2), size(B, 2)] = identity(size(B, 2)) "Matrix R in CARE";
        input Real Q[size(A, 1), size(A, 1)] = identity(size(A, 1)) "Matrix Q in CARE";
        input Boolean refine = false "True for subsequent refinement";
      protected
        Integer n = size(A, 1);
        Real G[size(A, 1), size(A, 1)] = B * Modelica.Math.Matrices.solve2(R, transpose(B));
        Real H[2 * size(A, 1), 2 * size(A, 1)] = [A, -G; -Q, -transpose(A)];
        Real H_RSF[2 * size(A, 1), 2 * size(A, 1)] = H;
        Real Z[size(H, 1), size(H, 2)];
        Real Z11[size(A, 1), size(A, 2)];
        Real Z21[size(A, 1), size(A, 2)];
        Integer info;
      public
        output Real X[size(A, 1), size(A, 2)] "stabilizing solution of CARE";
        output Real alphaReal[size(H, 1)] "Real parts of eigenvalue=alphaReal+i*alphaImag";
        output Real alphaImag[size(H, 1)] "Imaginary parts of eigenvalue=alphaReal+i*alphaImag";
      algorithm
        if n > 1 then
          (H_RSF, Z, alphaReal, alphaImag) := Modelica.Math.Matrices.realSchur(H);
          (H_RSF, Z, alphaReal, alphaImag) := Matrices.Utilities.reorderRSF(H_RSF, Z, alphaReal, alphaImag, true);
          Z11 := Z[1:n, 1:n];
          Z21 := Z[n + 1:2 * n, 1:n];
          if size(Z11, 1) > 0 then
            (X, info) := Matrices.LAPACK.dgesvx(Z11, transpose(Z21));
            assert(info == 0, "Solving a linear system of equations with function \"Matrices.LAPACK.dgesvx\" is not possible, because the system has either no or infinitely many solutions (input A is singular).");
            X := transpose(X);
            if refine then
              X := Modelica.Math.Matrices.Utilities.continuousRiccatiIterative(A, B, R, Q, X);
            end if;
          else
            X := fill(0, size(Z21, 1), size(Z11, 1));
          end if;
        elseif n == 1 then
          X := matrix((A[1, 1] - sqrt(A[1, 1] * A[1, 1] + G[1, 1] * Q[1, 1])) / G[1, 1]);
          if X[1, 1] * G[1, 1] < A[1, 1] then
            X := matrix((A[1, 1] + sqrt(A[1, 1] * A[1, 1] + G[1, 1] * Q[1, 1])) / G[1, 1]);
          end if;
        else
          X := fill(0, 0, 0);
        end if;
//this function does not need to transpose Z11 as solve2 does with //  X := transpose(Matrices.solve2(transpose(Z11), transpose(Z21)));
        annotation(
          Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
                                X = Matrices.<b>continuousRiccati</b>(A, B, R, Q);
        (X, alphaReal, alphaImag) = Matrices.<b>continuousRiccati</b>(A, B, R, Q, true);
</pre></blockquote>

<h4>Description</h4>
<p>
Function <b>continuousRiccati</b> computes the solution <b>X</b> of the continuous-time algebraic Riccati equation
</p>

<blockquote><pre>
 <b>A</b>'*<b>X</b> + <b>X</b>*<b>A</b> - <b>X</b>*<b>G</b>*<b>X</b> + <b>Q</b> = <b>0</b>
</pre></blockquote>

<p>
with <code><b>G</b> = <b>B</b>*inv(<b>R</b>)*<b>B</b>'</code>
using the Schur vector approach proposed by Laub [1].
</p>

<p>
It is assumed that <b>Q</b> is symmetric and positive semidefinite and <b>R</b> is symmetric, nonsingular and positive definite,
(<b>A</b>,<b>B</b>) is stabilizable and (<b>A</b>,<b>Q</b>) is detectable.
</p>

<p><b>
These assumptions are not checked in this function !!
</b><br>
</p>

<p>
The assumptions guarantee that the Hamiltonian matrix
</p>

<blockquote><pre>
<b>H</b> = [<b>A</b>, -<b>G</b>; -<b>Q</b>, -<b>A</b>']
</pre></blockquote>

<p>
has no pure imaginary eigenvalue and can be put
to an ordered real Schur form
</p>

<blockquote><pre>
<b>U</b>'*<b>H</b>*<b>U</b> = <b>S</b> = [<b>S</b>11, <b>S</b>12; <b>0</b>, <b>S</b>22]
</pre></blockquote>

<p>
with orthogonal similarity transformation <b>U</b>. <b>S</b> is ordered in such a way,
that <b>S</b>11 contains the n stable eigenvalues of the closed loop system with system matrix
<b>A</b> - <b>B</b>*inv(<b>R</b>)*<b>B</b>'*<b>X</b>.
If <b>U</b> is partitioned to
</p>

<blockquote><pre>
<b>U</b> = [<b>U</b>11, <b>U</b>12; <b>U</b>21, <b>U</b>22]
</pre></blockquote>

<p>
with dimensions according to <b>S</b>, the solution <b>X</b> is calculated by
</p>

<blockquote><pre>
<b>X</b>*<b>U</b>11 = <b>U</b>21.
</pre></blockquote>

<p>
With optional input <code>refinement=true</code> a subsequent iterative refinement based on Newton's method with exact line search is applied.
See <a href=\"modelica://Modelica.Math.Matrices.Utilities.continuousRiccatiIterative\">continuousRiccatiIterative</a>
for more information.
</p>

<h4>References</h4>
<PRE>
  [1] Laub, A.J.
      A Schur Method for Solving Algebraic Riccati equations.
      IEEE Trans. Auto. Contr., AC-24, pp. 913-921, 1979.
</PRE>

<h4>Example</h4>
<blockquote><pre>
  A = [0.0, 1.0;
       0.0, 0.0];

  B = [0.0;
       1.0];

  R = [1];

  Q = [1.0, 0.0;
       0.0, 2.0];

X = continuousRiccati(A, B, R, Q);

  results in:

X = [2.0, 1.0;
     1.0, 2.0];
</pre></blockquote>

<h4>See also</h4>
<p>
<a href=\"modelica://Modelica.Math.Matrices.Utilities.continuousRiccatiIterative\">Matrices.Utilities.continuousRiccatiIterative</a>,
<a href=\"modelica://Modelica.Math.Matrices.discreteRiccati\">Matrices.discreteRiccati</a>
</p>

</html>", revisions = "<html><ul>
<li><i>2010/05/31 </i>
       by Marcus Baur, DLR-RM</li>
</ul>
</html>"));
      end continuousRiccati;

      function discreteLyapunov "Return solution X of the discrete-time Lyapunov equation A'*X*A + sgn*X = C"
        extends Modelica.Icons.Function;
        import Modelica.Math.Matrices;
        input Real A[:, size(A, 1)] "Square matrix A in A'*X*A + sgn*X = C";
        input Real C[size(A, 1), size(A, 2)] "Square matrix C in A'*X*A + sgn*X = C";
        input Boolean ATisSchur = false "True if transpose(A) has already real Schur form";
        input Integer sgn = 1 "Specifies the sign in A'*X*A + sgn*X = C";
        input Real eps = Matrices.norm(A, 1) * 10 * Modelica.Constants.eps "Tolerance eps";
      protected
        Integer n = size(A, 1);
        Real R[size(A, 1), size(A, 2)] "RSF of A', i.e., R=U'A'U";
        Real U[size(A, 1), size(A, 2)] "Transformation matrix U for R=U'A'U";
        Real D[size(A, 1), size(A, 2)] "Matrix D=U'*C*U";
        Real R22[size(A, 1), size(A, 2)];
        Real R11[size(A, 1), size(A, 2)];
        Integer k;
        Real g[size(A, 1)];
        Real w[size(A, 1)];
        Real y[2 * size(A, 1)];
        Boolean crit;
      public
        output Real X[size(A, 1), size(A, 2)] "Solution X of the Lyapunov equation A'*X*A + sgn*X = C";
      algorithm
        assert(sgn == 1 or sgn == (-1), "Input sgn in function Math.Matrices.discreteLyapunov() must be 1 or -1, however it is " + String(sgn));
        X := zeros(n, n);
        k := n;
        if n > 1 then
          if ATisSchur then
            R := transpose(A);
            U := identity(n);
            D := C;
          else
            (R, U) := Modelica.Math.Matrices.realSchur(transpose(A));
            D := transpose(U) * C * U;
          end if;
          while k > 0 loop
            w := D[:, k] - R * X[:, k + 1:n] * R[k, k + 1:n];
            crit := if k > 1 then abs(R[k, k - 1]) < eps else false;
            if k == 1 or crit then
              R22 := R[k, k] * R;
              for i in 1:n loop
                R22[i, i] := R22[i, i] + sgn;
              end for;
              X[:, k] := Matrices.solve(R22, w);
              k := k - 1;
            else
              g := D[:, k - 1] - R * X[:, k + 1:n] * R[k - 1, k + 1:n];
              R11 := R[k - 1, k - 1] * R;
              R22 := R[k, k] * R;
              for i in 1:n loop
                R11[i, i] := R11[i, i] + sgn;
                R22[i, i] := R22[i, i] + sgn;
              end for;
              y := Matrices.solve([R11, R[k - 1, k] * R; R[k, k - 1] * R, R22], cat(1, g, w));
              X[:, k - 1] := y[1:n];
              X[:, k] := y[n + 1:2 * n];
              k := k - 2;
            end if;
          end while;
          if not ATisSchur then
            X := U * X * transpose(U);
          end if;
        elseif n == 1 then
          X[1, 1] := C[1, 1] / (A[1, 1] * A[1, 1] + sgn);
        else
          X := fill(0, 0, 0);
        end if;
// transform X corresponding to the original form
        annotation(
          Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
         X = Matrices.<b>discreteLyapunov</b>(A, C);
         X = Matrices.<b>discreteLyapunov</b>(A, C, ATisSchur, sgn, eps);
</pre></blockquote>

<h4>Description</h4>
<p>
This function computes the solution <b>X</b> of the discrete-time Lyapunov equation
</p>

<blockquote><pre>
 <b>A</b>'*<b>X</b>*<b>A</b> + sgn*<b>X</b> = <b>C</b>
</pre></blockquote>

<p>
where sgn=1 or sgn =-1. For sgn = -1, the discrete Lyapunov equation is a special case of the Stein equation:
</p>

<blockquote><pre>
 <b>A</b>*<b>X</b>*<b>B</b> - <b>X</b> + <b>Q</b> = <b>0</b>.
</pre></blockquote>

<p>
The algorithm uses the Schur method for Lyapunov equations proposed by Bartels and Stewart [1].
</p>

<p>
In a nutshell, the problem is reduced to the corresponding problem
</p>

<blockquote><pre>
 <b>R</b>*<b>Y</b>*<b>R</b>' + sgn*<b>Y</b> = <b>D</b>.
</pre></blockquote>

<p>
with <b>R</b>=<b>U</b>'*<b>A'</b>*<b>U</b> is the the real Schur form of <b>A</b>' and <b>D</b>=<b>U</b>'*<b>C</b>*<b>U</b> and <b>Y</b>=<b>U</b>'*<b>X</b>*<b>U</b>
are the corresponding transformations of <b>C</b> and <b>X</b>. This problem is solved sequentially by exploiting the block triangular form of <b>R</b>.
Finally the solution of the original problem is recovered as <b>X</b>=<b>U</b>*<b>Y</b>*<b>U</b>'.<br>
The Boolean input \"ATisSchur\" indicates to omit the transformation to Schur in the case that <b>A</b>' has already Schur form.
</p>

<h4>References</h4>
<PRE>
  [1] Bartels, R.H. and Stewart G.W.
      Algorithm 432: Solution of the matrix equation AX + XB = C.
      Comm. ACM., Vol. 15, pp. 820-826, 1972.
</PRE>

<h4>Example</h4>
<blockquote><pre>
  A = [1, 2,  3,  4;
       3, 4,  5, -2;
      -1, 2, -3, -5;
       0, 2,  0,  6];

  C =  [-2,  3, 1, 0;
        -6,  8, 0, 1;
         2,  3, 4, 5;
         0, -2, 0, 0];

  X = discreteLyapunov(A, C, sgn=-1);

  results in:

  X  = [7.5735,   -3.1426,  2.7205, -2.5958;
       -2.6105,    1.2384, -0.9232,  0.9632;
        6.6090,   -2.6775,  2.6415, -2.6928;
       -0.3572,    0.2298,  0.0533, -0.27410];

</pre></blockquote>

<h4>See also</h4>
<p>
<a href=\"modelica://Modelica.Math.Matrices.discreteSylvester\">Matrices.discreteSylvester</a>,
<a href=\"modelica://Modelica.Math.Matrices.continuousLyapunov\">Matrices.continuousLyapunov</a>
</p>

</html>", revisions = "<html>
<ul>
<li><i>2010/05/31 </i>
       by Marcus Baur, DLR-RM</li>
</ul>
</html>"));
      end discreteLyapunov;

      function discreteSylvester "Return solution of the discrete-time Sylvester equation A*X*B + sgn*X = C"
        extends Modelica.Icons.Function;
        import Modelica.Math.Matrices;
        input Real A[:, size(A, 1)] "Square matrix A in A*X*B + sgn*X = C";
        input Real B[:, size(B, 1)] "Square matrix B in A*X*B + sgn*X = C";
        input Real C[size(A, 2), size(B, 1)] "Rectangular matrix C in A*X*B + sgn*X = C";
        input Boolean AisHess = false "True if A has already Hessenberg form";
        input Boolean BTisSchur = false "True if B' has already real Schur form";
        input Integer sgn = 1 "Specifies the sign in A*X*B + sgn*X = C";
        input Real eps = Matrices.norm(A, 1) * 10 * Modelica.Constants.eps "Tolerance";
      protected
        Integer n = size(A, 1);
        Integer m = size(B, 1);
        Real H[size(A, 1), size(A, 1)] "Hessenberg form of A, i.e., H=U'AU";
        Real U[size(A, 1), size(A, 1)] "Transformation matrix U for H=U'AU";
        Real S[size(B, 1), size(B, 1)] "RSF form of B, i.e., S=Z'BZ";
        Real Z[size(B, 1), size(B, 1)] "Transformation matrix Z for S=Z'BZ";
        Real F[size(A, 1), size(B, 1)] "Appropriate transformation of the right side C, F=U'*C*Z";
        Real R22[size(A, 1), size(A, 1)];
        Real R11[size(A, 1), size(A, 1)];
        Integer k;
        Real w[size(A, 1)];
        Real g[size(A, 1)];
        Real y[2 * size(A, 1)];
        Boolean crit;
      public
        output Real X[size(A, 2), size(B, 1)] "solution of the discrete Sylvester equation A*X*B + sgn*X = C";
      algorithm
        assert(sgn == 1 or sgn == (-1), "Input sgn in function Math.Matrices.discreteLyapunov() must be 1 or -1, however it is " + String(sgn));
        X := zeros(n, m);
        k := m;
        if n > 1 and m > 1 then
          if AisHess then
            H := A;
            U := identity(n);
            if BTisSchur then
              S := B;
              Z := identity(m);
              F := C;
            else
              (S, Z) := Matrices.realSchur(transpose(B));
              S := transpose(S);
              F := C * Z;
            end if;
          else
            (H, U) := Matrices.hessenberg(A);
            if BTisSchur then
              S := B;
              Z := identity(m);
              F := transpose(U) * C;
            else
              (S, Z) := Matrices.realSchur(transpose(B));
              S := transpose(S);
              F := transpose(U) * C * Z;
            end if;
          end if;
          while k > 0 loop
            w := F[:, k] - H * X[:, k + 1:m] * S[k + 1:m, k];
            crit := if k > 1 then abs(S[k - 1, k]) < eps else false;
            if k == 1 or crit then
              R22 := S[k, k] * H;
              for i in 1:n loop
                R22[i, i] := R22[i, i] + sgn;
              end for;
              X[:, k] := Matrices.solve(R22, w);
              k := k - 1;
            else
              g := F[:, k - 1] - H * X[:, k + 1:m] * S[k + 1:m, k - 1];
              R22 := S[k, k] * H;
              R11 := S[k - 1, k - 1] * H;
              for i in 1:n loop
                R11[i, i] := R11[i, i] + sgn;
                R22[i, i] := R22[i, i] + sgn;
              end for;
              y := Matrices.solve([R11, S[k, k - 1] * H; S[k - 1, k] * H, R22], cat(1, g, w));
              X[:, k - 1] := y[1:n];
              X[:, k] := y[n + 1:2 * n];
              k := k - 2;
            end if;
          end while;
          if not (AisHess and BTisSchur) then
            X := if AisHess then X * transpose(Z) else if BTisSchur then U * X else U * X * transpose(Z);
          end if;
        elseif n == 1 and m == 1 then
          X[1, 1] := C[1, 1] / (A[1, 1] * B[1, 1] + sgn);
        else
          X := fill(0, 0, 0);
        end if;
//real eigenvalue in Schur form
// solve one column in X for one real eigenvalue
// pair of complex eigenvalues, i.e., 2x2 Schur bump
// solve two columns in X for one conjugated complex pole pair
// transform X corresponding to the original form
// simple scalar equation
        annotation(
          Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
         X = Matrices.<b>discreteSylvester</b>(A, B, C);
         X = Matrices.<b>discreteSylvester</b>(A, B, C, AisHess, BTisSchur, sgn, eps);
</pre></blockquote>

<h4>Description</h4>
<p>
Function <b>discreteSylvester</b> computes the solution <b>X</b> of the discrete-time Sylvester equation
</p>

<blockquote><pre>
 <b>A</b>*<b>X</b>*<b>B</b> + sgn*<b>X</b> = <b>C</b>.

</pre></blockquote>

<p>
where sgn = 1 or sgn = -1. The algorithm applies the Hessenberg-Schur method proposed by Golub et al [1].
For sgn = -1, the discrete Sylvester equation is also known as Stein equation:
</p>

<blockquote><pre>
 <b>A</b>*<b>X</b>*<b>B</b> - <b>X</b> + <b>Q</b> = <b>0</b>.
</pre></blockquote>

<p>
In a nutshell, the problem is reduced to the corresponding problem
</p>
<blockquote><pre>
 <b>H</b>*<b>Y</b>*<b>S</b>' + sgn*<b>Y</b> = <b>F</b>.
</pre></blockquote>

<p>
with <b>H</b>=<b>U</b>'*<b>A</b>*<b>U</b> is the Hessenberg form of <b>A</b> and <b>S</b>=<b>V</b>'*<b>B</b>'*<b>V</b> is the real Schur form of <b>B</b>',
<b>F</b>=<b>U</b>'*<b>C</b>*<b>V</b> and <b>Y</b>=<b>U</b>*<b>X</b>*<b>V</b>'
are appropriate transformations of <b>C</b> and <b>X</b>. This problem is solved sequentially by exploiting the specific forms of <b>S</b> and <b>H</b>.
Finally the solution of the original problem is recovered as <b>X</b>=<b>U</b>'*<b>Y</b>*<b>V</b>.<br>
The Boolean inputs \"AisHess\" and \"BTisSchur\" indicate to omit one or both of the transformation to Hessenberg form or Schur form respectively in the case that <b>A</b> and/or <b>B</b> have already Hessenberg form or Schur respectively.
</p>

<h4>References</h4>
<PRE>
  [1] Golub, G.H., Nash, S. and Van Loan, C.F.
      A Hessenberg-Schur method for the problem AX + XB = C.
      IEEE Transaction on Automatic Control, AC-24, no. 6, pp. 909-913, 1979.

</PRE>

<h4>Example</h4>
<blockquote><pre>
  A = [1.0,   2.0,   3.0;
       6.0,   7.0,   8.0;
       9.0,   2.0,   3.0];

  B = [7.0,   2.0,   3.0;
       2.0,   1.0,   2.0;
       3.0,   4.0,   1.0];

  C = [271.0,   135.0,   147.0;
       923.0,   494.0,   482.0;
       578.0,   383.0,   287.0];

  X = discreteSylvester(A, B, C);

  results in:
  X = [2.0,   3.0,   6.0;
       4.0,   7.0,   1.0;
       5.0,   3.0,   2.0];

</pre></blockquote>

<h4>See also</h4>
<p>
<a href=\"modelica://Modelica.Math.Matrices.continuousSylvester\">Matrices.continuousSylvester</a>,
<a href=\"modelica://Modelica.Math.Matrices.discreteLyapunov\">Matrices.discreteLyapunov</a>
</p>

</html>", revisions = "<html>
<ul>
<li><i>2010/05/31 </i>
       by Marcus Baur, DLR-RM</li>
</ul>
</html>"));
      end discreteSylvester;

      function discreteRiccati "Return solution of discrete-time algebraic Riccati equation A'*X*A - X - A'*X*B*inv(R + B'*X*B)*B'*X*A + Q = 0 (dare)"
        extends Modelica.Icons.Function;
        import Modelica.Math.Matrices;
        input Real A[:, size(A, 1)] "Square matrix A in DARE";
        input Real B[size(A, 1), :] "Matrix B in DARE";
        input Real R[size(B, 2), size(B, 2)] = identity(size(B, 2)) "Matrix R in DARE";
        input Real Q[size(A, 1), size(A, 1)] = identity(size(A, 1)) "Matrix Q in DARE";
        input Boolean refine = false "True for subsequent refinement";
      protected
        Integer n = size(A, 1);
        Real G[size(A, 1), size(A, 1)] = B * Matrices.solve2(R, transpose(B));
        Real AT[size(A, 1), size(A, 1)] = transpose(A);
        Real LU[size(A, 1), size(A, 1)];
        Integer p[size(A, 1)];
        Real H[2 * size(A, 1), 2 * size(A, 1)];
        Real H11[size(A, 1), size(A, 1)];
        Real H12[size(A, 1), size(A, 1)];
        Real H21[size(A, 1), size(A, 1)];
        Real H22[size(A, 1), size(A, 1)];
        Real H_RSF[2 * size(A, 1), 2 * size(A, 1)];
        Real Z[size(H, 1), size(H, 2)];
        Real Z11[size(A, 1), size(A, 2)];
        Real Z21[size(A, 1), size(A, 2)];
        Integer info;
      public
        output Real X[size(A, 1), size(A, 2)] "orthogonal matrix of the Schur vectors associated to ordered rsf";
        output Real alphaReal[size(H, 1)] "Real part of eigenvalue=alphaReal+i*alphaImag";
        output Real alphaImag[size(H, 1)] "Imaginary part of eigenvalue=alphaReal+i*alphaImag";
      algorithm
        (LU, p) := Modelica.Math.Matrices.LU(AT);
        H21 := Modelica.Math.Matrices.LU_solve2(LU, p, -Q);
        H22 := Modelica.Math.Matrices.LU_solve2(LU, p, identity(n));
        (LU, p) := Modelica.Math.Matrices.LU(A);
        H12 := Modelica.Math.Matrices.LU_solve2(LU, p, -G);
        H12 := transpose(H12);
        H11 := A - H12 * Q;
        H := [H11, H12; H21, H22];
        (H_RSF, Z, alphaReal, alphaImag) := Modelica.Math.Matrices.realSchur(H);
// put H to Schur form
        (H_RSF, Z, alphaReal, alphaImag) := Matrices.Utilities.reorderRSF(H_RSF, Z, alphaReal, alphaImag, false);
// ordered Schur form
        Z11 := Z[1:n, 1:n];
        Z21 := Z[n + 1:2 * n, 1:n];
        if size(Z11, 1) > 0 then
          (X, info) := Matrices.LAPACK.dgesvx(Z11, transpose(Z21));
          X := transpose(X);
          assert(info == 0, "Solving a linear system of equations with function
\"Matrices.LAPACK.dgesvx\" is not possible, because the system has either
no or infinitely many solutions (input A is singular).");
          if refine then
            X := Modelica.Math.Matrices.Utilities.discreteRiccatiIterative(A, B, R, Q, X);
          end if;
        else
          X := fill(0, size(Z21, 1), size(Z11, 1));
        end if;
//  X := transpose(Matrices.solve2(transpose(Z11), transpose(Z21)));
//function does not need to transpose Z11 as solve2 does
        annotation(
          Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
                                 X = Matrices.<b>discreteRiccati</b>(A, B, R, Q);
         (X, alphaReal, alphaImag) = Matrices.<b>discreteRiccati</b>(A, B, R, Q, true);
</pre></blockquote>

<h4>Description</h4>

<p>
Function <b>discreteRiccati</b> computes the solution <b>X</b> of the discrete-time algebraic Riccati equation
</p>

<blockquote><pre>
 <b>A</b>'*<b>X</b>*<b>A</b> - <b>X</b> - <b>A</b>'*<b>X</b>*<b>B</b>*inv(<b>R</b> + <b>B</b>'*<b>X</b>*<b>B</b>)*<b>B</b>'*<b>X</b>*<b>A</b> + <b>Q</b> = <b>0</b>
</pre></blockquote>

<p>
using the Schur vector approach proposed by Laub [1].
</p>

<p>
It is assumed that <b>Q</b> is symmetric and positive semidefinite and <b>R</b> is symmetric, nonsingular and positive definite,
(<b>A</b>,<b>B</b>) is stabilizable and (<b>A</b>,<b>Q</b>) is detectable. Using this method, <b>A</b> has also to be invertible.
</p>

<p>
<b>These assumptions are not checked in this function !!!</b>
</p>

<p>
The assumptions guarantee that the Hamiltonian matrix.
</p>
<blockquote><pre>
<b>H</b> = [<b>A</b> + <b>G</b>*<b>T</b>*<b>Q</b>, -<b>G</b>*<b>T</b>; -<b>T</b>*<b>Q</b>, <b>T</b>]
</pre></blockquote>
<p>
with
</p>
<blockquote><pre>
     -T
<b>T</b> = <b>A</b>
</pre></blockquote>

<p>
and
</p>

<blockquote><pre>
       -1
<b>G</b> = <b>B</b>*<b>R</b> *<b>B</b>'
</pre></blockquote>

<p>
has no eigenvalue on the unit circle and can be put
to an ordered real Schur form
</p>

<blockquote><pre>
<b>U</b>'*<b>H</b>*<b>U</b> = <b>S</b> = [<b>S11</b>, <b>S12</b>; <b>0</b>, <b>S22</b>]
</pre></blockquote>

<p>
with orthogonal similarity transformation <b>U</b>. <b>S</b> is ordered in such a way,
that <b>S11</b> contains the n stable eigenvalues of the closed loop system with system matrix
</p>

<blockquote><pre>
                  -1
<b>A</b> - <b>B</b>*(<b>R</b> + <b>B</b>'*<b>X</b>*<b>B</b>)  *<b>B</b>'*<b>X</b>*<b>A</b>
</pre></blockquote>

<p>
If <b>U</b> is partitioned to
</p>

<blockquote><pre>
<b>U</b> = [<b>U11</b>, <b>U12</b>; <b>U21</b>, <b>U22</b>]
</pre></blockquote>

<p>
according to <b>S</b>, the solution <b>X</b> can be calculated by
</p>

<blockquote><pre>
<b>X</b>*<b>U11</b> = <b>U21</b>.
</pre></blockquote>

<h4>References</h4>
<PRE>
  [1] Laub, A.J.
      A Schur Method for Solving Algebraic Riccati equations.
      IEEE Trans. Auto. Contr., AC-24, pp. 913-921, 1979.
</PRE>

<h4>Example</h4>
<blockquote><pre>
 A  = [4.0    3.0]
      -4.5,  -3.5];

 B  = [ 1.0;
       -1.0];

 R = [1.0];

 Q = [9.0, 6.0;
      6.0, 4.0]

X = discreteRiccati(A, B, R, Q);

  results in:

X = [14.5623, 9.7082;
      9.7082, 6.4721];
</pre></blockquote>

<h4>See also</h4>
<p>
<a href=\"modelica://Modelica.Math.Matrices.continuousRiccati\">Matrices.continuousRiccati</a>
</p>
</html>", revisions = "<html>
<ul>
<li><i>2010/05/31 </i>
       by Marcus Baur, DLR-RM</li>
</ul>
</html>"));
      end discreteRiccati;

      function sort "Sort the rows or columns of a matrix in ascending or descending order"
        extends Modelica.Icons.Function;
        input Real M[:, :] "Matrix to be sorted";
        input Boolean sortRows = true "= true if rows are sorted, otherwise columns";
        input Boolean ascending = true "= true if ascending order, otherwise descending order";
        output Real sorted_M[size(M, 1), size(M, 2)] = M "Sorted matrix";
        output Integer indices[if sortRows then size(M, 1) else size(M, 2)] "sorted_M = if sortRows then M[indices,:] else M[:,indices]";
        /* shellsort algorithm; should be improved later */
      protected
        Integer gap;
        Integer i;
        Integer j;
        Real wM2[size(M, 2)];
        Integer wi;
        Integer nM1 = size(M, 1);
        Boolean swap;
        Real sorted_MT[size(M, 2), size(M, 1)];

        encapsulated function greater "Compare whether vector v1 > v2"
          import Modelica;
          extends Modelica.Icons.Function;
          import Modelica.Utilities.Types.Compare;
          input Real v1[:];
          input Real v2[size(v1, 1)];
          output Boolean result;
        protected
          Integer n = size(v1, 1);
          Integer i = 1;
        algorithm
          result := false;
          while i <= n loop
            if v1[i] > v2[i] then
              result := true;
              i := n;
            elseif v1[i] < v2[i] then
              i := n;
            end if;
            i := i + 1;
          end while;
        end greater;

        encapsulated function less "Compare whether vector v1 < v2"
          import Modelica;
          extends Modelica.Icons.Function;
          import Modelica.Utilities.Types.Compare;
          input Real v1[:];
          input Real v2[size(v1, 1)];
          output Boolean result;
        protected
          Integer n = size(v1, 1);
          Integer i = 1;
        algorithm
          result := false;
          while i <= n loop
            if v1[i] < v2[i] then
              result := true;
              i := n;
            elseif v1[i] > v2[i] then
              i := n;
            end if;
            i := i + 1;
          end while;
        end less;
      algorithm
        if not sortRows then
          (sorted_MT, indices) := sort(transpose(M), ascending = ascending);
          sorted_M := transpose(sorted_MT);
        else
          indices := 1:size(M, 1);
          gap := div(nM1, 2);
          while gap > 0 loop
            i := gap;
            while i < nM1 loop
              j := i - gap;
              if j >= 0 then
                if ascending then
                  swap := greater(sorted_M[j + 1, :], sorted_M[j + gap + 1, :]);
                else
                  swap := less(sorted_M[j + 1, :], sorted_M[j + gap + 1, :]);
                end if;
              else
                swap := false;
              end if;
              while swap loop
                wM2 := sorted_M[j + 1, :];
                wi := indices[j + 1];
                sorted_M[j + 1, :] := sorted_M[j + gap + 1, :];
                sorted_M[j + gap + 1, :] := wM2;
                indices[j + 1] := indices[j + gap + 1];
                indices[j + gap + 1] := wi;
                j := j - gap;
                if j >= 0 then
                  if ascending then
                    swap := greater(sorted_M[j + 1, :], sorted_M[j + gap + 1, :]);
                  else
                    swap := less(sorted_M[j + 1, :], sorted_M[j + gap + 1, :]);
                  end if;
                else
                  swap := false;
                end if;
              end while;
              i := i + 1;
            end while;
            gap := div(gap, 2);
          end while;
        end if;
        annotation(
          Documentation(info = "<HTML>
<h4>Syntax</h4>
<blockquote><pre>
           sorted_M = Matrices.<b>sort</b>(M);
(sorted_M, indices) = Matrices.<b>sort</b>(M, sortRows=true, ascending=true);
</pre></blockquote>

<h4>Description</h4>
<p>
Function <b>sort</b>(..) sorts the rows of a Real matrix M
in ascending order and returns the result in sorted_M.
If the optional argument \"sortRows\" is <b>false</b>, the columns
of the matrix are sorted.
If the optional argument \"ascending\" is <b>false</b>, the rows or
columns are sorted in descending order. In the optional second
output argument, the indices of the sorted rows or columns with respect
to the original matrix are given, such that
</p>

<pre>
   sorted_M = <b>if</b> sortedRow <b>then</b> M[indices,:] <b>else</b> M[:,indices];
</pre>

<h4>Example</h4>
<blockquote><pre>
  (M2, i2) := Matrices.sort([2, 1,  0;
                             2, 0, -1]);
       -> M2 = [2, 0, -1;
                2, 1, 0 ];
          i2 = {2,1};
</pre></blockquote>
</HTML>"));
      end sort;

      function flipLeftRight "Flip the columns of a matrix in left/right direction"
        extends Modelica.Icons.Function;
        input Real A[:, :] "Matrix to be flipped";
        output Real Aflip[size(A, 1), size(A, 2)] "Flipped matrix";
      algorithm
        Aflip := A[:, {i for i in size(A, 2):(-1):1}];
        annotation(
          Inline = true,
          Documentation(info = "<html>
 <h4>Syntax</h4>
<blockquote><pre>
         A_flr = Matrices.<b>flipLeftRight</b>(A);
</pre></blockquote>

<h4>Description</h4>
<p>
Function <b>flipLeftRight</b> computes from matrix <b>A</b> a matrix <b>A_flr</b> with flipped columns, i.e., <b>A_flr</b>[:,i]=<b>A</b>[:,n-i+1], i=1,..., n.
</p>

<h4>Example</h4>
<blockquote><pre>
  A = [1, 2,  3;
       3, 4,  5;
      -1, 2, -3];

  A_flr = flipLeftRight(A);

  results in:

  A_flr = [3, 2,  1;
           5, 4,  3;
          -3, 2, -1]
</pre></blockquote>

<h4>See also</h4>
<p>
<a href=\"modelica://Modelica.Math.Matrices.flipUpDown\">Matrices.flipUpDown</a>
</p>

</html>", revisions = "<html>
<ul>
<li><i>2010/05/31 </i>
       by Marcus Baur, DLR-RM</li>
</ul>
</html>"));
      end flipLeftRight;

      function flipUpDown "Flip the rows of a matrix in up/down direction"
        extends Modelica.Icons.Function;
        input Real A[:, :] "Matrix to be flipped";
        output Real Aflip[size(A, 1), size(A, 2)] "Flipped matrix";
      algorithm
        Aflip := A[{i for i in size(A, 1):(-1):1}, :];
        annotation(
          Inline = true,
          Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
         A_fud = Matrices.<b>flipUpDown</b>(A);
</pre></blockquote>

<h4>Description</h4>
<p>
Function <b>flipUpDown</b> computes from matrix <b>A</b> a matrix <b>A_fud</b> with flipped rows, i.e., <b>A_fud</b>[i,:]=<b>A</b>[n-i+1,:], i=1,..., n.
</p>

<h4>Example</h4>
<blockquote><pre>
  A = [1, 2,  3;
       3, 4,  5;
      -1, 2, -3];

  A_fud = flipUpDown(A);

  results in:

  A_fud  = [-1, 2, -3;
             3, 4,  5;
             1, 2,  3]
</pre></blockquote>

<h4>See also</h4>
<p>
<a href=\"modelica://Modelica.Math.Matrices.flipLeftRight\">Matrices.flipLeftRight</a>
</p>

</html>", revisions = "<html>
<ul>
<li><i>2010/05/31 </i>
       by Marcus Baur, DLR-RM</li>
</ul>
</html>"));
      end flipUpDown;

      package LAPACK "Interface to LAPACK library (should usually not directly be used but only indirectly via Modelica.Math.Matrices)"
        extends Modelica.Icons.Package;

        function dgeev "Compute eigenvalues and (right) eigenvectors for real nonsymmetric matrix A"
          extends Modelica.Icons.Function;
          input Real A[:, size(A, 1)];
          output Real eigenReal[size(A, 1)] "Real part of eigen values";
          output Real eigenImag[size(A, 1)] "Imaginary part of eigen values";
          output Real eigenVectors[size(A, 1), size(A, 1)] "Right eigen vectors";
          output Integer info;
        protected
          Real dummy[1, 1];
          Integer n = size(A, 1);
          Integer lwork = 12 * n;
          Real Awork[size(A, 1), size(A, 1)] = A;
          Real work[12 * size(A, 1)];

          external "FORTRAN 77" dgeev("N", "V", n, Awork, n, eigenReal, eigenImag, dummy, 1, eigenVectors, n, work, size(work, 1), info) annotation(
            Library = "lapack");
          annotation(
            Documentation(info = "This function is not a full interface to the LAPACK function DGEEV,
but calls it in such a way that only eigenvalues and right eigenvectors
are computed.

Lapack documentation
    Purpose
    =======

    DGEEV computes for an N-by-N real nonsymmetric matrix A, the
    eigenvalues and, optionally, the left and/or right eigenvectors.

    The right eigenvector v(j) of A satisfies
                     A * v(j) = lambda(j) * v(j)
    where lambda(j) is its eigenvalue.
    The left eigenvector u(j) of A satisfies
                  u(j)**H * A = lambda(j) * u(j)**H
    where u(j)**H denotes the conjugate transpose of u(j).

    The computed eigenvectors are normalized to have Euclidean norm
    equal to 1 and largest component real.

    Arguments
    =========

    JOBVL   (input) CHARACTER*1
            = 'N': left eigenvectors of A are not computed;
            = 'V': left eigenvectors of A are computed.

    JOBVR   (input) CHARACTER*1
            = 'N': right eigenvectors of A are not computed;
            = 'V': right eigenvectors of A are computed.

    N       (input) INTEGER
            The order of the matrix A. N >= 0.

    A       (input/output) DOUBLE PRECISION array, dimension (LDA,N)
            On entry, the N-by-N matrix A.
            On exit, A has been overwritten.

    LDA     (input) INTEGER
            The leading dimension of the array A.  LDA >= max(1,N).

    WR      (output) DOUBLE PRECISION array, dimension (N)
    WI      (output) DOUBLE PRECISION array, dimension (N)
            WR and WI contain the real and imaginary parts,
            respectively, of the computed eigenvalues.  Complex
            conjugate pairs of eigenvalues appear consecutively
            with the eigenvalue having the positive imaginary part
            first.

    VL      (output) DOUBLE PRECISION array, dimension (LDVL,N)
            If JOBVL = 'V', the left eigenvectors u(j) are stored one
            after another in the columns of VL, in the same order
            as their eigenvalues.
            If JOBVL = 'N', VL is not referenced.
            If the j-th eigenvalue is real, then u(j) = VL(:,j),
            the j-th column of VL.
            If the j-th and (j+1)-st eigenvalues form a complex
            conjugate pair, then u(j) = VL(:,j) + i*VL(:,j+1) and
            u(j+1) = VL(:,j) - i*VL(:,j+1).

    LDVL    (input) INTEGER
            The leading dimension of the array VL.  LDVL >= 1; if
            JOBVL = 'V', LDVL >= N.

    VR      (output) DOUBLE PRECISION array, dimension (LDVR,N)
            If JOBVR = 'V', the right eigenvectors v(j) are stored one
            after another in the columns of VR, in the same order
            as their eigenvalues.
            If JOBVR = 'N', VR is not referenced.
            If the j-th eigenvalue is real, then v(j) = VR(:,j),
            the j-th column of VR.
            If the j-th and (j+1)-st eigenvalues form a complex
            conjugate pair, then v(j) = VR(:,j) + i*VR(:,j+1) and
            v(j+1) = VR(:,j) - i*VR(:,j+1).

    LDVR    (input) INTEGER
            The leading dimension of the array VR.  LDVR >= 1; if
            JOBVR = 'V', LDVR >= N.

    WORK    (workspace/output) DOUBLE PRECISION array, dimension (MAX(1,LWORK))
            On exit, if INFO = 0, WORK(1) returns the optimal LWORK.

    LWORK   (input) INTEGER
            The dimension of the array WORK.  LWORK >= max(1,3*N), and
            if JOBVL = 'V' or JOBVR = 'V', LWORK >= 4*N.  For good
            performance, LWORK must generally be larger.

            If LWORK = -1, then a workspace query is assumed; the routine
            only calculates the optimal size of the WORK array, returns
            this value as the first entry of the WORK array, and no error
            message related to LWORK is issued by XERBLA.

    INFO    (output) INTEGER
            = 0:  successful exit
            < 0:  if INFO = -i, the i-th argument had an illegal value.
            > 0:  if INFO = i, the QR algorithm failed to compute all the
                  eigenvalues, and no eigenvectors have been computed;
                  elements i+1:N of WR and WI contain eigenvalues which
                  have converged.
            "));
        end dgeev;

        function dgeev_eigenValues "Compute eigenvalues for real nonsymmetric matrix A"
          extends Modelica.Icons.Function;
          input Real A[:, size(A, 1)];
          output Real EigenReal[size(A, 1)];
          output Real EigenImag[size(A, 1)];
          /*
                output Real Eigenvectors[size(A, 1), size(A, 1)]=zeros(size(A, 1), size(
                    A, 1)); */
          output Integer info;
        protected
          Integer lwork = 8 * size(A, 1);
          Real Awork[size(A, 1), size(A, 1)] = A;
          Real work[8 * size(A, 1)];
          Real EigenvectorsL[size(A, 1), size(A, 1)] = zeros(size(A, 1), size(A, 1));
          /*
              external "FORTRAN 77" dgeev("N", "V", size(A, 1), Awork, size(A, 1),
                  EigenReal, EigenImag, EigenvectorsL, size(EigenvectorsL, 1),
                  Eigenvectors, size(Eigenvectors, 1), work, size(work, 1), info)
          */

          external "FORTRAN 77" dgeev("N", "N", size(A, 1), Awork, size(A, 1), EigenReal, EigenImag, EigenvectorsL, size(EigenvectorsL, 1), EigenvectorsL, size(EigenvectorsL, 1), work, size(work, 1), info) annotation(
            Library = "lapack");
          annotation(
            Documentation(info = "Lapack documentation
    Purpose
    =======

    DGEEV computes for an N-by-N real nonsymmetric matrix A, the
    eigenvalues and, optionally, the left and/or right eigenvectors.

    The right eigenvector v(j) of A satisfies
                     A * v(j) = lambda(j) * v(j)
    where lambda(j) is its eigenvalue.
    The left eigenvector u(j) of A satisfies
                  u(j)**H * A = lambda(j) * u(j)**H
    where u(j)**H denotes the conjugate transpose of u(j).

    The computed eigenvectors are normalized to have Euclidean norm
    equal to 1 and largest component real.

    Arguments
    =========

    JOBVL   (input) CHARACTER*1
            = 'N': left eigenvectors of A are not computed;
            = 'V': left eigenvectors of A are computed.

    JOBVR   (input) CHARACTER*1
            = 'N': right eigenvectors of A are not computed;
            = 'V': right eigenvectors of A are computed.

    N       (input) INTEGER
            The order of the matrix A. N >= 0.

    A       (input/output) DOUBLE PRECISION array, dimension (LDA,N)
            On entry, the N-by-N matrix A.
            On exit, A has been overwritten.

    LDA     (input) INTEGER
            The leading dimension of the array A.  LDA >= max(1,N).

    WR      (output) DOUBLE PRECISION array, dimension (N)
    WI      (output) DOUBLE PRECISION array, dimension (N)
            WR and WI contain the real and imaginary parts,
            respectively, of the computed eigenvalues.  Complex
            conjugate pairs of eigenvalues appear consecutively
            with the eigenvalue having the positive imaginary part
            first.

    VL      (output) DOUBLE PRECISION array, dimension (LDVL,N)
            If JOBVL = 'V', the left eigenvectors u(j) are stored one
            after another in the columns of VL, in the same order
            as their eigenvalues.
            If JOBVL = 'N', VL is not referenced.
            If the j-th eigenvalue is real, then u(j) = VL(:,j),
            the j-th column of VL.
            If the j-th and (j+1)-st eigenvalues form a complex
            conjugate pair, then u(j) = VL(:,j) + i*VL(:,j+1) and
            u(j+1) = VL(:,j) - i*VL(:,j+1).

    LDVL    (input) INTEGER
            The leading dimension of the array VL.  LDVL >= 1; if
            JOBVL = 'V', LDVL >= N.

    VR      (output) DOUBLE PRECISION array, dimension (LDVR,N)
            If JOBVR = 'V', the right eigenvectors v(j) are stored one
            after another in the columns of VR, in the same order
            as their eigenvalues.
            If JOBVR = 'N', VR is not referenced.
            If the j-th eigenvalue is real, then v(j) = VR(:,j),
            the j-th column of VR.
            If the j-th and (j+1)-st eigenvalues form a complex
            conjugate pair, then v(j) = VR(:,j) + i*VR(:,j+1) and
            v(j+1) = VR(:,j) - i*VR(:,j+1).

    LDVR    (input) INTEGER
            The leading dimension of the array VR.  LDVR >= 1; if
            JOBVR = 'V', LDVR >= N.

    WORK    (workspace/output) DOUBLE PRECISION array, dimension (MAX(1,LWORK))
            On exit, if INFO = 0, WORK(1) returns the optimal LWORK.

    LWORK   (input) INTEGER
            The dimension of the array WORK.  LWORK >= max(1,3*N), and
            if JOBVL = 'V' or JOBVR = 'V', LWORK >= 4*N.  For good
            performance, LWORK must generally be larger.

            If LWORK = -1, then a workspace query is assumed; the routine
            only calculates the optimal size of the WORK array, returns
            this value as the first entry of the WORK array, and no error
            message related to LWORK is issued by XERBLA.

    INFO    (output) INTEGER
            = 0:  successful exit
            < 0:  if INFO = -i, the i-th argument had an illegal value.
            > 0:  if INFO = i, the QR algorithm failed to compute all the
                  eigenvalues, and no eigenvectors have been computed;
                  elements i+1:N of WR and WI contain eigenvalues which
                  have converged.
            "));
        end dgeev_eigenValues;

        function dgegv "Compute generalized eigenvalues for a (A,B) system"
          extends Modelica.Icons.Function;
          input Real A[:, size(A, 1)];
          input Real B[size(A, 1), size(A, 1)];
          output Real alphaReal[size(A, 1)] "Real part of alpha (eigenvalue=(alphaReal+i*alphaImag)/beta)";
          output Real alphaImag[size(A, 1)] "Imaginary part of alpha";
          output Real beta[size(A, 1)] "Denominator of eigenvalue";
          output Integer info;
        protected
          Integer n = size(A, 1);
          Integer lwork = 12 * n;
          Real Awork[size(A, 1), size(A, 1)] = A;
          Real Bwork[size(A, 1), size(A, 1)] = B;
          Real work[12 * size(A, 1)];
          Real dummy1[1, 1];
          Real dummy2[1, 1];

          external "FORTRAN 77" dgegv("N", "N", n, Awork, n, Bwork, n, alphaReal, alphaImag, beta, dummy1, 1, dummy2, 1, work, size(work, 1), info) annotation(
            Library = "lapack");
          annotation(
            Documentation(info = "Lapack documentation
    Purpose
    =======

    This routine is deprecated and has been replaced by routine DGGEV.

    DGEGV computes the eigenvalues and, optionally, the left and/or right
    eigenvectors of a real matrix pair (A,B).
    Given two square matrices A and B,
    the generalized nonsymmetric eigenvalue problem (GNEP) is to find the
    eigenvalues lambda and corresponding (non-zero) eigenvectors x such
    that

       A*x = lambda*B*x.

    An alternate form is to find the eigenvalues mu and corresponding
    eigenvectors y such that

       mu*A*y = B*y.

    These two forms are equivalent with mu = 1/lambda and x = y if
    neither lambda nor mu is zero.  In order to deal with the case that
    lambda or mu is zero or small, two values alpha and beta are returned
    for each eigenvalue, such that lambda = alpha/beta and
    mu = beta/alpha.

    The vectors x and y in the above equations are right eigenvectors of
    the matrix pair (A,B).  Vectors u and v satisfying

       u**H*A = lambda*u**H*B  or  mu*v**H*A = v**H*B

    are left eigenvectors of (A,B).

    Note: this routine performs \"full balancing\" on A and B -- see
    \"Further Details\", below.

    Arguments
    =========

    JOBVL   (input) CHARACTER*1
            = 'N':  do not compute the left generalized eigenvectors;
            = 'V':  compute the left generalized eigenvectors (returned
                    in VL).

    JOBVR   (input) CHARACTER*1
            = 'N':  do not compute the right generalized eigenvectors;
            = 'V':  compute the right generalized eigenvectors (returned
                    in VR).

    N       (input) INTEGER
            The order of the matrices A, B, VL, and VR.  N >= 0.

    A       (input/output) DOUBLE PRECISION array, dimension (LDA, N)
            On entry, the matrix A.
            If JOBVL = 'V' or JOBVR = 'V', then on exit A
            contains the real Schur form of A from the generalized Schur
            factorization of the pair (A,B) after balancing.
            If no eigenvectors were computed, then only the diagonal
            blocks from the Schur form will be correct.  See DGGHRD and
            DHGEQZ for details.

    LDA     (input) INTEGER
            The leading dimension of A.  LDA >= max(1,N).

    B       (input/output) DOUBLE PRECISION array, dimension (LDB, N)
            On entry, the matrix B.
            If JOBVL = 'V' or JOBVR = 'V', then on exit B contains the
            upper triangular matrix obtained from B in the generalized
            Schur factorization of the pair (A,B) after balancing.
            If no eigenvectors were computed, then only those elements of
            B corresponding to the diagonal blocks from the Schur form of
            A will be correct.  See DGGHRD and DHGEQZ for details.

    LDB     (input) INTEGER
            The leading dimension of B.  LDB >= max(1,N).

    ALPHAR  (output) DOUBLE PRECISION array, dimension (N)
            The real parts of each scalar alpha defining an eigenvalue of
            GNEP.

    ALPHAI  (output) DOUBLE PRECISION array, dimension (N)
            The imaginary parts of each scalar alpha defining an
            eigenvalue of GNEP.  If ALPHAI(j) is zero, then the j-th
            eigenvalue is real; if positive, then the j-th and
            (j+1)-st eigenvalues are a complex conjugate pair, with
            ALPHAI(j+1) = -ALPHAI(j).

    BETA    (output) DOUBLE PRECISION array, dimension (N)
            The scalars beta that define the eigenvalues of GNEP.

            Together, the quantities alpha = (ALPHAR(j),ALPHAI(j)) and
            beta = BETA(j) represent the j-th eigenvalue of the matrix
            pair (A,B), in one of the forms lambda = alpha/beta or
            mu = beta/alpha.  Since either lambda or mu may overflow,
            they should not, in general, be computed.

    VL      (output) DOUBLE PRECISION array, dimension (LDVL,N)
            If JOBVL = 'V', the left eigenvectors u(j) are stored
            in the columns of VL, in the same order as their eigenvalues.
            If the j-th eigenvalue is real, then u(j) = VL(:,j).
            If the j-th and (j+1)-st eigenvalues form a complex conjugate
            pair, then
               u(j) = VL(:,j) + i*VL(:,j+1)
            and
              u(j+1) = VL(:,j) - i*VL(:,j+1).

            Each eigenvector is scaled so that its largest component has
            abs(real part) + abs(imag. part) = 1, except for eigenvectors
            corresponding to an eigenvalue with alpha = beta = 0, which
            are set to zero.
            Not referenced if JOBVL = 'N'.

    LDVL    (input) INTEGER
            The leading dimension of the matrix VL. LDVL >= 1, and
            if JOBVL = 'V', LDVL >= N.

    VR      (output) DOUBLE PRECISION array, dimension (LDVR,N)
            If JOBVR = 'V', the right eigenvectors x(j) are stored
            in the columns of VR, in the same order as their eigenvalues.
            If the j-th eigenvalue is real, then x(j) = VR(:,j).
            If the j-th and (j+1)-st eigenvalues form a complex conjugate
            pair, then
              x(j) = VR(:,j) + i*VR(:,j+1)
            and
              x(j+1) = VR(:,j) - i*VR(:,j+1).

            Each eigenvector is scaled so that its largest component has
            abs(real part) + abs(imag. part) = 1, except for eigenvalues
            corresponding to an eigenvalue with alpha = beta = 0, which
            are set to zero.
            Not referenced if JOBVR = 'N'.

    LDVR    (input) INTEGER
            The leading dimension of the matrix VR. LDVR >= 1, and
            if JOBVR = 'V', LDVR >= N.

    WORK    (workspace/output) DOUBLE PRECISION array, dimension (MAX(1,LWORK))
            On exit, if INFO = 0, WORK(1) returns the optimal LWORK.

    LWORK   (input) INTEGER
            The dimension of the array WORK.  LWORK >= max(1,8*N).
            For good performance, LWORK must generally be larger.
            To compute the optimal value of LWORK, call ILAENV to get
            blocksizes (for DGEQRF, DORMQR, and DORGQR.)  Then compute:
            NB  -- MAX of the blocksizes for DGEQRF, DORMQR, and DORGQR;
            The optimal LWORK is:
                2*N + MAX( 6*N, N*(NB+1) ).

            If LWORK = -1, then a workspace query is assumed; the routine
            only calculates the optimal size of the WORK array, returns
            this value as the first entry of the WORK array, and no error
            message related to LWORK is issued by XERBLA.

    INFO    (output) INTEGER
            = 0:  successful exit
            < 0:  if INFO = -i, the i-th argument had an illegal value.
            = 1,...,N:
                  The QZ iteration failed.  No eigenvectors have been
                  calculated, but ALPHAR(j), ALPHAI(j), and BETA(j)
                  should be correct for j=INFO+1,...,N.
            > N:  errors that usually indicate LAPACK problems:
                  =N+1: error return from DGGBAL
                  =N+2: error return from DGEQRF
                  =N+3: error return from DORMQR
                  =N+4: error return from DORGQR
                  =N+5: error return from DGGHRD
                  =N+6: error return from DHGEQZ (other than failed
                                                  iteration)
                  =N+7: error return from DTGEVC
                  =N+8: error return from DGGBAK (computing VL)
                  =N+9: error return from DGGBAK (computing VR)
                  =N+10: error return from DLASCL (various calls)

    Further Details
    ===============

    Balancing
    ---------

    This driver calls DGGBAL to both permute and scale rows and columns
    of A and B.  The permutations PL and PR are chosen so that PL*A*PR
    and PL*B*R will be upper triangular except for the diagonal blocks
    A(i:j,i:j) and B(i:j,i:j), with i and j as close together as
    possible.  The diagonal scaling matrices DL and DR are chosen so
    that the pair  DL*PL*A*PR*DR, DL*PL*B*PR*DR have elements close to
    one (except for the elements that start out zero.)

    After the eigenvalues and eigenvectors of the balanced matrices
    have been computed, DGGBAK transforms the eigenvectors back to what
    they would have been (in perfect arithmetic) if they had not been
    balanced.

    Contents of A and B on Exit
    -------- -- - --- - -- ----

    If any eigenvectors are computed (either JOBVL='V' or JOBVR='V' or
    both), then on exit the arrays A and B will contain the real Schur
    form[*] of the \"balanced\" versions of A and B.  If no eigenvectors
    are computed, then only the diagonal blocks will be correct.

    [*] See DHGEQZ, DGEGS, or read the book \"Matrix Computations\",
        by Golub & van Loan, pub. by Johns Hopkins U. Press.
            "));
        end dgegv;

        function dgelsx "Computes the minimum-norm solution to a real linear least squares problem with rank deficient A"
          extends Modelica.Icons.Function;
          input Real A[:, :];
          input Real B[size(A, 1), :];
          input Real rcond = 0.0 "Reciprocal condition number to estimate rank";
          output Real X[max(size(A, 1), size(A, 2)), size(B, 2)] = cat(1, B, zeros(max(nrow, ncol) - nrow, nrhs)) "Solution is in first size(A,2) rows";
          output Integer info;
          output Integer rank "Effective rank of A";
        protected
          Integer nrow = size(A, 1);
          Integer ncol = size(A, 2);
          Integer nx = max(nrow, ncol);
          Integer nrhs = size(B, 2);
          Real work[max(min(size(A, 1), size(A, 2)) + 3 * size(A, 2), 2 * min(size(A, 1), size(A, 2)) + size(B, 2))];
          Real Awork[size(A, 1), size(A, 2)] = A;
          Integer jpvt[size(A, 2)] = zeros(ncol);

          external "FORTRAN 77" dgelsx(nrow, ncol, nrhs, Awork, nrow, X, nx, jpvt, rcond, rank, work, info) annotation(
            Library = "lapack");
          annotation(
            Documentation(info = "Lapack documentation
    Purpose
    =======

    This routine is deprecated and has been replaced by routine DGELSY.

    DGELSX computes the minimum-norm solution to a real linear least
    squares problem:
        minimize || A * X - B ||
    using a complete orthogonal factorization of A.  A is an M-by-N
    matrix which may be rank-deficient.

    Several right hand side vectors b and solution vectors x can be
    handled in a single call; they are stored as the columns of the
    M-by-NRHS right hand side matrix B and the N-by-NRHS solution
    matrix X.

    The routine first computes a QR factorization with column pivoting:
        A * P = Q * [ R11 R12 ]
                    [  0  R22 ]
    with R11 defined as the largest leading submatrix whose estimated
    condition number is less than 1/RCOND.  The order of R11, RANK,
    is the effective rank of A.

    Then, R22 is considered to be negligible, and R12 is annihilated
    by orthogonal transformations from the right, arriving at the
    complete orthogonal factorization:
       A * P = Q * [ T11 0 ] * Z
                   [  0  0 ]
    The minimum-norm solution is then
       X = P * Z' [ inv(T11)*Q1'*B ]
                  [        0       ]
    where Q1 consists of the first RANK columns of Q.

    Arguments
    =========

    M       (input) INTEGER
            The number of rows of the matrix A.  M >= 0.

    N       (input) INTEGER
            The number of columns of the matrix A.  N >= 0.

    NRHS    (input) INTEGER
            The number of right hand sides, i.e., the number of
            columns of matrices B and X. NRHS >= 0.

    A       (input/output) DOUBLE PRECISION array, dimension (LDA,N)
            On entry, the M-by-N matrix A.
            On exit, A has been overwritten by details of its
            complete orthogonal factorization.

    LDA     (input) INTEGER
            The leading dimension of the array A.  LDA >= max(1,M).

    B       (input/output) DOUBLE PRECISION array, dimension (LDB,NRHS)
            On entry, the M-by-NRHS right hand side matrix B.
            On exit, the N-by-NRHS solution matrix X.
            If m >= n and RANK = n, the residual sum-of-squares for
            the solution in the i-th column is given by the sum of
            squares of elements N+1:M in that column.

    LDB     (input) INTEGER
            The leading dimension of the array B. LDB >= max(1,M,N).

    JPVT    (input/output) INTEGER array, dimension (N)
            On entry, if JPVT(i) .ne. 0, the i-th column of A is an
            initial column, otherwise it is a free column.  Before
            the QR factorization of A, all initial columns are
            permuted to the leading positions; only the remaining
            free columns are moved as a result of column pivoting
            during the factorization.
            On exit, if JPVT(i) = k, then the i-th column of A*P
            was the k-th column of A.

    RCOND   (input) DOUBLE PRECISION
            RCOND is used to determine the effective rank of A, which
            is defined as the order of the largest leading triangular
            submatrix R11 in the QR factorization with pivoting of A,
            whose estimated condition number < 1/RCOND.

    RANK    (output) INTEGER
            The effective rank of A, i.e., the order of the submatrix
            R11.  This is the same as the order of the submatrix T11
            in the complete orthogonal factorization of A.

    WORK    (workspace) DOUBLE PRECISION array, dimension
                        (max( min(M,N)+3*N, 2*min(M,N)+NRHS )),

    INFO    (output) INTEGER
            = 0:  successful exit
            < 0:  if INFO = -i, the i-th argument had an illegal value
            "));
        end dgelsx;

        function dgelsx_vec "Computes the minimum-norm solution to a real linear least squares problem with rank deficient A"
          extends Modelica.Icons.Function;
          input Real A[:, :];
          input Real b[size(A, 1)];
          input Real rcond = 0.0 "Reciprocal condition number to estimate rank";
          output Real x[max(size(A, 1), size(A, 2))] = cat(1, b, zeros(max(nrow, ncol) - nrow)) "solution is in first size(A,2) rows";
          output Integer info;
          output Integer rank "Effective rank of A";
        protected
          Integer nrow = size(A, 1);
          Integer ncol = size(A, 2);
          Integer nx = max(nrow, ncol);
          Real work[max(min(size(A, 1), size(A, 2)) + 3 * size(A, 2), 2 * min(size(A, 1), size(A, 2)) + 1)];
          Real Awork[size(A, 1), size(A, 2)] = A;
          Integer jpvt[size(A, 2)] = zeros(ncol);

          external "FORTRAN 77" dgelsx(nrow, ncol, 1, Awork, nrow, x, nx, jpvt, rcond, rank, work, info) annotation(
            Library = "lapack");
          annotation(
            Documentation(info = "Lapack documentation
    Purpose
    =======

    This routine is deprecated and has been replaced by routine DGELSY.

    DGELSX computes the minimum-norm solution to a real linear least
    squares problem:
        minimize || A * X - B ||
    using a complete orthogonal factorization of A.  A is an M-by-N
    matrix which may be rank-deficient.

    Several right hand side vectors b and solution vectors x can be
    handled in a single call; they are stored as the columns of the
    M-by-NRHS right hand side matrix B and the N-by-NRHS solution
    matrix X.

    The routine first computes a QR factorization with column pivoting:
        A * P = Q * [ R11 R12 ]
                    [  0  R22 ]
    with R11 defined as the largest leading submatrix whose estimated
    condition number is less than 1/RCOND.  The order of R11, RANK,
    is the effective rank of A.

    Then, R22 is considered to be negligible, and R12 is annihilated
    by orthogonal transformations from the right, arriving at the
    complete orthogonal factorization:
       A * P = Q * [ T11 0 ] * Z
                   [  0  0 ]
    The minimum-norm solution is then
       X = P * Z' [ inv(T11)*Q1'*B ]
                  [        0       ]
    where Q1 consists of the first RANK columns of Q.

    Arguments
    =========

    M       (input) INTEGER
            The number of rows of the matrix A.  M >= 0.

    N       (input) INTEGER
            The number of columns of the matrix A.  N >= 0.

    NRHS    (input) INTEGER
            The number of right hand sides, i.e., the number of
            columns of matrices B and X. NRHS >= 0.

    A       (input/output) DOUBLE PRECISION array, dimension (LDA,N)
            On entry, the M-by-N matrix A.
            On exit, A has been overwritten by details of its
            complete orthogonal factorization.

    LDA     (input) INTEGER
            The leading dimension of the array A.  LDA >= max(1,M).

    B       (input/output) DOUBLE PRECISION array, dimension (LDB,NRHS)
            On entry, the M-by-NRHS right hand side matrix B.
            On exit, the N-by-NRHS solution matrix X.
            If m >= n and RANK = n, the residual sum-of-squares for
            the solution in the i-th column is given by the sum of
            squares of elements N+1:M in that column.

    LDB     (input) INTEGER
            The leading dimension of the array B. LDB >= max(1,M,N).

    JPVT    (input/output) INTEGER array, dimension (N)
            On entry, if JPVT(i) .ne. 0, the i-th column of A is an
            initial column, otherwise it is a free column.  Before
            the QR factorization of A, all initial columns are
            permuted to the leading positions; only the remaining
            free columns are moved as a result of column pivoting
            during the factorization.
            On exit, if JPVT(i) = k, then the i-th column of A*P
            was the k-th column of A.

    RCOND   (input) DOUBLE PRECISION
            RCOND is used to determine the effective rank of A, which
            is defined as the order of the largest leading triangular
            submatrix R11 in the QR factorization with pivoting of A,
            whose estimated condition number < 1/RCOND.

    RANK    (output) INTEGER
            The effective rank of A, i.e., the order of the submatrix
            R11.  This is the same as the order of the submatrix T11
            in the complete orthogonal factorization of A.

    WORK    (workspace) DOUBLE PRECISION array, dimension
                        (max( min(M,N)+3*N, 2*min(M,N)+NRHS )),

    INFO    (output) INTEGER
            = 0:  successful exit
            < 0:  if INFO = -i, the i-th argument had an illegal value
            "));
        end dgelsx_vec;

        function dgels_vec "Solves overdetermined or underdetermined real linear equations A*x=b with a b vector"
          extends Modelica.Icons.Function;
          input Real A[:, :];
          input Real b[size(A, 1)];
          output Real x[max(size(A, 1), size(A, 2))] = cat(1, b, zeros(nx - nrow)) "solution is in first size(A,2) rows";
          output Integer info;
        protected
          Integer nrow = size(A, 1);
          Integer ncol = size(A, 2);
          Integer nx = max(nrow, ncol);
          Integer lwork = min(nrow, ncol) + nx;
          Real work[size(A, 1) + size(A, 2)];
          Real Awork[size(A, 1), size(A, 2)] = A;

          external "FORTRAN 77" dgels("N", nrow, ncol, 1, Awork, nrow, x, nx, work, lwork, info) annotation(
            Library = "lapack");
          annotation(
            Documentation(info = "Lapack documentation
    Purpose
    =======

    DGELS solves overdetermined or underdetermined real linear systems
    involving an M-by-N matrix A, or its transpose, using a QR or LQ
    factorization of A.  It is assumed that A has full rank.

    The following options are provided:

    1. If TRANS = 'N' and m >= n:  find the least squares solution of
       an overdetermined system, i.e., solve the least squares problem
                    minimize || B - A*X ||.

    2. If TRANS = 'N' and m < n:  find the minimum norm solution of
       an underdetermined system A * X = B.

    3. If TRANS = 'T' and m >= n:  find the minimum norm solution of
       an undetermined system A**T * X = B.

    4. If TRANS = 'T' and m < n:  find the least squares solution of
       an overdetermined system, i.e., solve the least squares problem
                    minimize || B - A**T * X ||.

    Several right hand side vectors b and solution vectors x can be
    handled in a single call; they are stored as the columns of the
    M-by-NRHS right hand side matrix B and the N-by-NRHS solution
    matrix X.

    Arguments
    =========

    TRANS   (input) CHARACTER*1
            = 'N': the linear system involves A;
            = 'T': the linear system involves A**T.

    M       (input) INTEGER
            The number of rows of the matrix A.  M >= 0.

    N       (input) INTEGER
            The number of columns of the matrix A.  N >= 0.

    NRHS    (input) INTEGER
            The number of right hand sides, i.e., the number of
            columns of the matrices B and X. NRHS >=0.

    A       (input/output) DOUBLE PRECISION array, dimension (LDA,N)
            On entry, the M-by-N matrix A.
            On exit,
              if M >= N, A is overwritten by details of its QR
                         factorization as returned by DGEQRF;
              if M <  N, A is overwritten by details of its LQ
                         factorization as returned by DGELQF.

    LDA     (input) INTEGER
            The leading dimension of the array A.  LDA >= max(1,M).

    B       (input/output) DOUBLE PRECISION array, dimension (LDB,NRHS)
            On entry, the matrix B of right hand side vectors, stored
            columnwise; B is M-by-NRHS if TRANS = 'N', or N-by-NRHS
            if TRANS = 'T'.
            On exit, if INFO = 0, B is overwritten by the solution
            vectors, stored columnwise:
            if TRANS = 'N' and m >= n, rows 1 to n of B contain the least
            squares solution vectors; the residual sum of squares for the
            solution in each column is given by the sum of squares of
            elements N+1 to M in that column;
            if TRANS = 'N' and m < n, rows 1 to N of B contain the
            minimum norm solution vectors;
            if TRANS = 'T' and m >= n, rows 1 to M of B contain the
            minimum norm solution vectors;
            if TRANS = 'T' and m < n, rows 1 to M of B contain the
            least squares solution vectors; the residual sum of squares
            for the solution in each column is given by the sum of
            squares of elements M+1 to N in that column.

    LDB     (input) INTEGER
            The leading dimension of the array B. LDB >= MAX(1,M,N).

    WORK    (workspace/output) DOUBLE PRECISION array, dimension (MAX(1,LWORK))
            On exit, if INFO = 0, WORK(1) returns the optimal LWORK.

    LWORK   (input) INTEGER
            The dimension of the array WORK.
            LWORK >= max( 1, MN + max( MN, NRHS ) ).
            For optimal performance,
            LWORK >= max( 1, MN + max( MN, NRHS )*NB ).
            where MN = min(M,N) and NB is the optimum block size.

            If LWORK = -1, then a workspace query is assumed; the routine
            only calculates the optimal size of the WORK array, returns
            this value as the first entry of the WORK array, and no error
            message related to LWORK is issued by XERBLA.

    INFO    (output) INTEGER
            = 0:  successful exit
            < 0:  if INFO = -i, the i-th argument had an illegal value
            > 0:  if INFO =  i, the i-th diagonal element of the
                  triangular factor of A is zero, so that A does not have
                  full rank; the least squares solution could not be
                  computed.
            "));
        end dgels_vec;

        function dgesv "Solve real system of linear equations A*X=B with a B matrix"
          extends Modelica.Icons.Function;
          input Real A[:, size(A, 1)];
          input Real B[size(A, 1), :];
          output Real X[size(A, 1), size(B, 2)] = B;
          output Integer info;
        protected
          Real Awork[size(A, 1), size(A, 1)] = A;
          Integer lda = max(1, size(A, 1));
          Integer ldb = max(1, size(B, 1));
          Integer ipiv[size(A, 1)];

          external "FORTRAN 77" dgesv(size(A, 1), size(B, 2), Awork, lda, ipiv, X, ldb, info) annotation(
            Library = "lapack");
          annotation(
            Documentation(info = "Lapack documentation
    Purpose
    =======

    DGESV computes the solution to a real system of linear equations
       A * X = B,
    where A is an N-by-N matrix and X and B are N-by-NRHS matrices.

    The LU decomposition with partial pivoting and row interchanges is
    used to factor A as
       A = P * L * U,
    where P is a permutation matrix, L is unit lower triangular, and U is
    upper triangular.  The factored form of A is then used to solve the
    system of equations A * X = B.

    Arguments
    =========

    N       (input) INTEGER
            The number of linear equations, i.e., the order of the
            matrix A.  N >= 0.

    NRHS    (input) INTEGER
            The number of right hand sides, i.e., the number of columns
            of the matrix B.  NRHS >= 0.

    A       (input/output) DOUBLE PRECISION array, dimension (LDA,N)
            On entry, the N-by-N coefficient matrix A.
            On exit, the factors L and U from the factorization
            A = P*L*U; the unit diagonal elements of L are not stored.

    LDA     (input) INTEGER
            The leading dimension of the array A.  LDA >= max(1,N).

    IPIV    (output) INTEGER array, dimension (N)
            The pivot indices that define the permutation matrix P;
            row i of the matrix was interchanged with row IPIV(i).

    B       (input/output) DOUBLE PRECISION array, dimension (LDB,NRHS)
            On entry, the N-by-NRHS matrix of right hand side matrix B.
            On exit, if INFO = 0, the N-by-NRHS solution matrix X.

    LDB     (input) INTEGER
            The leading dimension of the array B.  LDB >= max(1,N).

    INFO    (output) INTEGER
            = 0:  successful exit
            < 0:  if INFO = -i, the i-th argument had an illegal value
            > 0:  if INFO = i, U(i,i) is exactly zero.  The factorization
                  has been completed, but the factor U is exactly
                  singular, so the solution could not be computed.
            "));
        end dgesv;

        function dgesv_vec "Solve real system of linear equations A*x=b with a b vector"
          extends Modelica.Icons.Function;
          input Real A[:, size(A, 1)];
          input Real b[size(A, 1)];
          output Real x[size(A, 1)] = b;
          output Integer info;
        protected
          Real Awork[size(A, 1), size(A, 1)] = A;
          Integer lda = max(1, size(A, 1));
          Integer ldb = max(1, size(b, 1));
          Integer ipiv[size(A, 1)];

          external "FORTRAN 77" dgesv(size(A, 1), 1, Awork, lda, ipiv, x, ldb, info) annotation(
            Library = "lapack");
          annotation(
            Documentation(info = "
Same as function LAPACK.dgesv, but right hand side is a vector and not a matrix.
For details of the arguments, see documentation of dgesv.
            "));
        end dgesv_vec;

        function dgglse_vec "Solve a linear equality constrained least squares problem"
          extends Modelica.Icons.Function;
          input Real A[:, :] "Minimize |A*x - c|^2";
          input Real c[size(A, 1)];
          input Real B[:, size(A, 2)] "subject to B*x=d";
          input Real d[size(B, 1)];
          output Real x[size(A, 2)] "solution vector";
          output Integer info;
        protected
          Integer nrow_A = size(A, 1);
          Integer nrow_B = size(B, 1);
          Integer ncol_A = size(A, 2) "(min=nrow_B,max=nrow_A+nrow_B) required";
          Real Awork[size(A, 1), size(A, 2)] = A;
          Real Bwork[size(B, 1), size(A, 2)] = B;
          Real cwork[size(A, 1)] = c;
          Real dwork[size(B, 1)] = d;
          Integer lwork = ncol_A + nrow_B + max(nrow_A, max(ncol_A, nrow_B)) * 5;
          Real work[size(A, 2) + size(B, 1) + max(size(A, 1), max(size(A, 2), size(B, 1))) * 5];

          external "FORTRAN 77" dgglse(nrow_A, ncol_A, nrow_B, Awork, nrow_A, Bwork, nrow_B, cwork, dwork, x, work, lwork, info) annotation(
            Library = "lapack");
          annotation(
            Documentation(info = "Lapack documentation
    Purpose
    =======

    DGGLSE solves the linear equality-constrained least squares (LSE)
    problem:

            minimize || c - A*x ||_2   subject to   B*x = d

    where A is an M-by-N matrix, B is a P-by-N matrix, c is a given
    M-vector, and d is a given P-vector. It is assumed that
    P <= N <= M+P, and

             rank(B) = P and  rank( (A) ) = N.
                                  ( (B) )

    These conditions ensure that the LSE problem has a unique solution,
    which is obtained using a generalized RQ factorization of the
    matrices (B, A) given by

       B = (0 R)*Q,   A = Z*T*Q.

    Arguments
    =========

    M       (input) INTEGER
            The number of rows of the matrix A.  M >= 0.

    N       (input) INTEGER
            The number of columns of the matrices A and B. N >= 0.

    P       (input) INTEGER
            The number of rows of the matrix B. 0 <= P <= N <= M+P.

    A       (input/output) DOUBLE PRECISION array, dimension (LDA,N)
            On entry, the M-by-N matrix A.
            On exit, the elements on and above the diagonal of the array
            contain the min(M,N)-by-N upper trapezoidal matrix T.

    LDA     (input) INTEGER
            The leading dimension of the array A. LDA >= max(1,M).

    B       (input/output) DOUBLE PRECISION array, dimension (LDB,N)
            On entry, the P-by-N matrix B.
            On exit, the upper triangle of the subarray B(1:P,N-P+1:N)
            contains the P-by-P upper triangular matrix R.

    LDB     (input) INTEGER
            The leading dimension of the array B. LDB >= max(1,P).

    C       (input/output) DOUBLE PRECISION array, dimension (M)
            On entry, C contains the right hand side vector for the
            least squares part of the LSE problem.
            On exit, the residual sum of squares for the solution
            is given by the sum of squares of elements N-P+1 to M of
            vector C.

    D       (input/output) DOUBLE PRECISION array, dimension (P)
            On entry, D contains the right hand side vector for the
            constrained equation.
            On exit, D is destroyed.

    X       (output) DOUBLE PRECISION array, dimension (N)
            On exit, X is the solution of the LSE problem.

    WORK    (workspace/output) DOUBLE PRECISION array, dimension (MAX(1,LWORK))
            On exit, if INFO = 0, WORK(1) returns the optimal LWORK.

    LWORK   (input) INTEGER
            The dimension of the array WORK. LWORK >= max(1,M+N+P).
            For optimum performance LWORK >= P+min(M,N)+max(M,N)*NB,
            where NB is an upper bound for the optimal blocksizes for
            DGEQRF, SGERQF, DORMQR and SORMRQ.

            If LWORK = -1, then a workspace query is assumed; the routine
            only calculates the optimal size of the WORK array, returns
            this value as the first entry of the WORK array, and no error
            message related to LWORK is issued by XERBLA.

    INFO    (output) INTEGER
            = 0:  successful exit.
            < 0:  if INFO = -i, the i-th argument had an illegal value.
            = 1:  the upper triangular factor R associated with B in the
                  generalized RQ factorization of the pair (B, A) is
                  singular, so that rank(B) < P; the least squares
                  solution could not be computed.
            = 2:  the (N-P) by (N-P) part of the upper trapezoidal factor
                  T associated with A in the generalized RQ factorization
                  of the pair (B, A) is singular, so that
                  rank( (A) ) < N; the least squares solution could not
                      ( (B) )
                  be computed.
            "));
        end dgglse_vec;

        function dgtsv "Solve real system of linear equations A*X=B with B matrix and tridiagonal A"
          extends Modelica.Icons.Function;
          input Real superdiag[:];
          input Real diag[size(superdiag, 1) + 1];
          input Real subdiag[size(superdiag, 1)];
          input Real B[size(diag, 1), :];
          output Real X[size(B, 1), size(B, 2)] = B;
          output Integer info;
        protected
          Real superdiagwork[size(superdiag, 1)] = superdiag;
          Real diagwork[size(diag, 1)] = diag;
          Real subdiagwork[size(subdiag, 1)] = subdiag;

          external "FORTRAN 77" dgtsv(size(diag, 1), size(B, 2), subdiagwork, diagwork, superdiagwork, X, size(B, 1), info) annotation(
            Library = "lapack");
          annotation(
            Documentation(info = "Lapack documentation
    Purpose
    =======

    DGTSV  solves the equation

       A*X = B,

    where A is an n by n tridiagonal matrix, by Gaussian elimination with
    partial pivoting.

    Note that the equation  A'*X = B  may be solved by interchanging the
    order of the arguments DU and DL.

    Arguments
    =========

    N       (input) INTEGER
            The order of the matrix A.  N >= 0.

    NRHS    (input) INTEGER
            The number of right hand sides, i.e., the number of columns
            of the matrix B.  NRHS >= 0.

    DL      (input/output) DOUBLE PRECISION array, dimension (N-1)
            On entry, DL must contain the (n-1) sub-diagonal elements of
            A.

            On exit, DL is overwritten by the (n-2) elements of the
            second super-diagonal of the upper triangular matrix U from
            the LU factorization of A, in DL(1), ..., DL(n-2).

    D       (input/output) DOUBLE PRECISION array, dimension (N)
            On entry, D must contain the diagonal elements of A.

            On exit, D is overwritten by the n diagonal elements of U.

    DU      (input/output) DOUBLE PRECISION array, dimension (N-1)
            On entry, DU must contain the (n-1) super-diagonal elements
            of A.

            On exit, DU is overwritten by the (n-1) elements of the first
            super-diagonal of U.

    B       (input/output) DOUBLE PRECISION array, dimension (LDB,NRHS)
            On entry, the N by NRHS matrix of right hand side matrix B.
            On exit, if INFO = 0, the N by NRHS solution matrix X.

    LDB     (input) INTEGER
            The leading dimension of the array B.  LDB >= max(1,N).

    INFO    (output) INTEGER
            = 0: successful exit
            < 0: if INFO = -i, the i-th argument had an illegal value
            > 0: if INFO = i, U(i,i) is exactly zero, and the solution
                 has not been computed.  The factorization has not been
                 completed unless i = N.
            "));
        end dgtsv;

        function dgtsv_vec "Solve real system of linear equations A*x=b with b vector and tridiagonal A"
          extends Modelica.Icons.Function;
          input Real superdiag[:];
          input Real diag[size(superdiag, 1) + 1];
          input Real subdiag[size(superdiag, 1)];
          input Real b[size(diag, 1)];
          output Real x[size(b, 1)] = b;
          output Integer info;
        protected
          Real superdiagwork[size(superdiag, 1)] = superdiag;
          Real diagwork[size(diag, 1)] = diag;
          Real subdiagwork[size(subdiag, 1)] = subdiag;

          external "FORTRAN 77" dgtsv(size(diag, 1), 1, subdiagwork, diagwork, superdiagwork, x, size(b, 1), info) annotation(
            Library = "lapack");
          annotation(
            Documentation(info = "
Same as function LAPACK.dgtsv, but right hand side is a vector and not a matrix.
For details of the arguments, see documentation of dgtsv.
            "));
        end dgtsv_vec;

        function dgbsv "Solve real system of linear equations A*X=B with a B matrix"
          extends Modelica.Icons.Function;
          input Integer n "Number of equations";
          input Integer kLower "Number of lower bands";
          input Integer kUpper "Number of upper bands";
          input Real A[2 * kLower + kUpper + 1, n];
          input Real B[n, :];
          output Real X[n, size(B, 2)] = B;
          output Integer info;
        protected
          Real Awork[size(A, 1), size(A, 2)] = A;
          Integer ipiv[n];

          external "FORTRAN 77" dgbsv(n, kLower, kUpper, size(B, 2), Awork, size(Awork, 1), ipiv, X, n, info) annotation(
            Library = "lapack");
          annotation(
            Documentation(info = "Lapack documentation
    Purpose
    =======

    DGBSV computes the solution to a real system of linear equations
    A * X = B, where A is a band matrix of order N with KL subdiagonals
    and KU superdiagonals, and X and B are N-by-NRHS matrices.

    The LU decomposition with partial pivoting and row interchanges is
    used to factor A as A = L * U, where L is a product of permutation
    and unit lower triangular matrices with KL subdiagonals, and U is
    upper triangular with KL+KU superdiagonals.  The factored form of A
    is then used to solve the system of equations A * X = B.

    Arguments
    =========

    N       (input) INTEGER
            The number of linear equations, i.e., the order of the
            matrix A.  N >= 0.

    KL      (input) INTEGER
            The number of subdiagonals within the band of A.  KL >= 0.

    KU      (input) INTEGER
            The number of superdiagonals within the band of A.  KU >= 0.

    NRHS    (input) INTEGER
            The number of right hand sides, i.e., the number of columns
            of the matrix B.  NRHS >= 0.

    AB      (input/output) DOUBLE PRECISION array, dimension (LDAB,N)
            On entry, the matrix A in band storage, in rows KL+1 to
            2*KL+KU+1; rows 1 to KL of the array need not be set.
            The j-th column of A is stored in the j-th column of the
            array AB as follows:
            AB(KL+KU+1+i-j,j) = A(i,j) for max(1,j-KU)<=i<=min(N,j+KL)
            On exit, details of the factorization: U is stored as an
            upper triangular band matrix with KL+KU superdiagonals in
            rows 1 to KL+KU+1, and the multipliers used during the
            factorization are stored in rows KL+KU+2 to 2*KL+KU+1.
            See below for further details.

    LDAB    (input) INTEGER
            The leading dimension of the array AB.  LDAB >= 2*KL+KU+1.

    IPIV    (output) INTEGER array, dimension (N)
            The pivot indices that define the permutation matrix P;
            row i of the matrix was interchanged with row IPIV(i).

    B       (input/output) DOUBLE PRECISION array, dimension (LDB,NRHS)
            On entry, the N-by-NRHS right hand side matrix B.
            On exit, if INFO = 0, the N-by-NRHS solution matrix X.

    LDB     (input) INTEGER
            The leading dimension of the array B.  LDB >= max(1,N).

    INFO    (output) INTEGER
            = 0:  successful exit
            < 0:  if INFO = -i, the i-th argument had an illegal value
            > 0:  if INFO = i, U(i,i) is exactly zero.  The factorization
                  has been completed, but the factor U is exactly
                  singular, and the solution has not been computed.

    Further Details
    ===============

    The band storage scheme is illustrated by the following example, when
    M = N = 6, KL = 2, KU = 1:

    On entry:                       On exit:

        *    *    *    +    +    +       *    *    *   u14  u25  u36
        *    *    +    +    +    +       *    *   u13  u24  u35  u46
        *   a12  a23  a34  a45  a56      *   u12  u23  u34  u45  u56
       a11  a22  a33  a44  a55  a66     u11  u22  u33  u44  u55  u66
       a21  a32  a43  a54  a65   *      m21  m32  m43  m54  m65   *
       a31  a42  a53  a64   *    *      m31  m42  m53  m64   *    *

    Array elements marked * are not used by the routine; elements marked
    + need not be set on entry, but are required by the routine to store
    elements of U because of fill-in resulting from the row interchanges.
            "));
        end dgbsv;

        function dgbsv_vec "Solve real system of linear equations A*x=b with a b vector"
          extends Modelica.Icons.Function;
          input Integer n "Number of equations";
          input Integer kLower "Number of lower bands";
          input Integer kUpper "Number of upper bands";
          input Real A[2 * kLower + kUpper + 1, n];
          input Real b[n];
          output Real x[n] = b;
          output Integer info;
        protected
          Real Awork[size(A, 1), size(A, 2)] = A;
          Integer ipiv[n];

          external "FORTRAN 77" dgbsv(n, kLower, kUpper, 1, Awork, size(Awork, 1), ipiv, x, n, info) annotation(
            Library = "lapack");
          annotation(
            Documentation(info = "
Same as function LAPACK.dgbsv, but right hand side is a vector and not a matrix.
For details of the arguments, see documentation of dgbsv.
            "));
        end dgbsv_vec;

        function dgesvd "Determine singular value decomposition"
          extends Modelica.Icons.Function;
          input Real A[:, :];
          output Real sigma[min(size(A, 1), size(A, 2))];
          output Real U[size(A, 1), size(A, 1)] = zeros(size(A, 1), size(A, 1));
          output Real VT[size(A, 2), size(A, 2)] = zeros(size(A, 2), size(A, 2));
          output Integer info;
        protected
          Real Awork[size(A, 1), size(A, 2)] = A;
          Integer lwork = 5 * size(A, 1) + 5 * size(A, 2);
          Real work[5 * size(A, 1) + 5 * size(A, 2)];

          external "FORTRAN 77" dgesvd("A", "A", size(A, 1), size(A, 2), Awork, size(A, 1), sigma, U, size(A, 1), VT, size(A, 2), work, lwork, info) annotation(
            Library = "lapack");
          annotation(
            Documentation(info = "Lapack documentation
    Purpose
    =======

    DGESVD computes the singular value decomposition (SVD) of a real
    M-by-N matrix A, optionally computing the left and/or right singular
    vectors. The SVD is written

         A = U * SIGMA * transpose(V)

    where SIGMA is an M-by-N matrix which is zero except for its
    min(m,n) diagonal elements, U is an M-by-M orthogonal matrix, and
    V is an N-by-N orthogonal matrix.  The diagonal elements of SIGMA
    are the singular values of A; they are real and non-negative, and
    are returned in descending order.  The first min(m,n) columns of
    U and V are the left and right singular vectors of A.

    Note that the routine returns V**T, not V.

    Arguments
    =========

    JOBU    (input) CHARACTER*1
            Specifies options for computing all or part of the matrix U:
            = 'A':  all M columns of U are returned in array U:
            = 'S':  the first min(m,n) columns of U (the left singular
                    vectors) are returned in the array U;
            = 'O':  the first min(m,n) columns of U (the left singular
                    vectors) are overwritten on the array A;
            = 'N':  no columns of U (no left singular vectors) are
                    computed.

    JOBVT   (input) CHARACTER*1
            Specifies options for computing all or part of the matrix
            V**T:
            = 'A':  all N rows of V**T are returned in the array VT;
            = 'S':  the first min(m,n) rows of V**T (the right singular
                    vectors) are returned in the array VT;
            = 'O':  the first min(m,n) rows of V**T (the right singular
                    vectors) are overwritten on the array A;
            = 'N':  no rows of V**T (no right singular vectors) are
                    computed.

            JOBVT and JOBU cannot both be 'O'.

    M       (input) INTEGER
            The number of rows of the input matrix A.  M >= 0.

    N       (input) INTEGER
            The number of columns of the input matrix A.  N >= 0.

    A       (input/output) DOUBLE PRECISION array, dimension (LDA,N)
            On entry, the M-by-N matrix A.
            On exit,
            if JOBU = 'O',  A is overwritten with the first min(m,n)
                            columns of U (the left singular vectors,
                            stored columnwise);
            if JOBVT = 'O', A is overwritten with the first min(m,n)
                            rows of V**T (the right singular vectors,
                            stored rowwise);
            if JOBU .ne. 'O' and JOBVT .ne. 'O', the contents of A
                            are destroyed.

    LDA     (input) INTEGER
            The leading dimension of the array A.  LDA >= max(1,M).

    S       (output) DOUBLE PRECISION array, dimension (min(M,N))
            The singular values of A, sorted so that S(i) >= S(i+1).

    U       (output) DOUBLE PRECISION array, dimension (LDU,UCOL)
            (LDU,M) if JOBU = 'A' or (LDU,min(M,N)) if JOBU = 'S'.
            If JOBU = 'A', U contains the M-by-M orthogonal matrix U;
            if JOBU = 'S', U contains the first min(m,n) columns of U
            (the left singular vectors, stored columnwise);
            if JOBU = 'N' or 'O', U is not referenced.

    LDU     (input) INTEGER
            The leading dimension of the array U.  LDU >= 1; if
            JOBU = 'S' or 'A', LDU >= M.

    VT      (output) DOUBLE PRECISION array, dimension (LDVT,N)
            If JOBVT = 'A', VT contains the N-by-N orthogonal matrix
            V**T;
            if JOBVT = 'S', VT contains the first min(m,n) rows of
            V**T (the right singular vectors, stored rowwise);
            if JOBVT = 'N' or 'O', VT is not referenced.

    LDVT    (input) INTEGER
            The leading dimension of the array VT.  LDVT >= 1; if
            JOBVT = 'A', LDVT >= N; if JOBVT = 'S', LDVT >= min(M,N).

    WORK    (workspace/output) DOUBLE PRECISION array, dimension (MAX(1,LWORK))
            On exit, if INFO = 0, WORK(1) returns the optimal LWORK;
            if INFO > 0, WORK(2:MIN(M,N)) contains the unconverged
            superdiagonal elements of an upper bidiagonal matrix B
            whose diagonal is in S (not necessarily sorted). B
            satisfies A = U * B * VT, so it has the same singular values
            as A, and singular vectors related by U and VT.

    LWORK   (input) INTEGER
            The dimension of the array WORK.
            LWORK >= MAX(1,3*MIN(M,N)+MAX(M,N),5*MIN(M,N)).
            For good performance, LWORK should generally be larger.

            If LWORK = -1, then a workspace query is assumed; the routine
            only calculates the optimal size of the WORK array, returns
            this value as the first entry of the WORK array, and no error
            message related to LWORK is issued by XERBLA.

    INFO    (output) INTEGER
            = 0:  successful exit.
            < 0:  if INFO = -i, the i-th argument had an illegal value.
            > 0:  if DBDSQR did not converge, INFO specifies how many
                  superdiagonals of an intermediate bidiagonal form B
                  did not converge to zero. See the description of WORK
                  above for details.
            "));
        end dgesvd;

        function dgesvd_sigma "Determine singular values"
          extends Modelica.Icons.Function;
          input Real A[:, :];
          output Real sigma[min(size(A, 1), size(A, 2))];
          output Integer info;
        protected
          Real Awork[size(A, 1), size(A, 2)] = A;
          Real U[size(A, 1), size(A, 1)];
          Real VT[size(A, 2), size(A, 2)];
          Integer lwork = 5 * size(A, 1) + 5 * size(A, 2);
          Real work[5 * size(A, 1) + 5 * size(A, 2)];

          external "FORTRAN 77" dgesvd("N", "N", size(A, 1), size(A, 2), Awork, size(A, 1), sigma, U, size(A, 1), VT, size(A, 2), work, lwork, info) annotation(
            Library = "lapack");
          annotation(
            Documentation(info = "Lapack documentation
    Purpose
    =======

    DGESVD computes the singular value decomposition (SVD) of a real
    M-by-N matrix A, optionally computing the left and/or right singular
    vectors. The SVD is written

         A = U * SIGMA * transpose(V)

    where SIGMA is an M-by-N matrix which is zero except for its
    min(m,n) diagonal elements, U is an M-by-M orthogonal matrix, and
    V is an N-by-N orthogonal matrix.  The diagonal elements of SIGMA
    are the singular values of A; they are real and non-negative, and
    are returned in descending order.  The first min(m,n) columns of
    U and V are the left and right singular vectors of A.

    Note that the routine returns V**T, not V.

    Arguments
    =========

    JOBU    (input) CHARACTER*1
            Specifies options for computing all or part of the matrix U:
            = 'A':  all M columns of U are returned in array U:
            = 'S':  the first min(m,n) columns of U (the left singular
                    vectors) are returned in the array U;
            = 'O':  the first min(m,n) columns of U (the left singular
                    vectors) are overwritten on the array A;
            = 'N':  no columns of U (no left singular vectors) are
                    computed.

    JOBVT   (input) CHARACTER*1
            Specifies options for computing all or part of the matrix
            V**T:
            = 'A':  all N rows of V**T are returned in the array VT;
            = 'S':  the first min(m,n) rows of V**T (the right singular
                    vectors) are returned in the array VT;
            = 'O':  the first min(m,n) rows of V**T (the right singular
                    vectors) are overwritten on the array A;
            = 'N':  no rows of V**T (no right singular vectors) are
                    computed.

            JOBVT and JOBU cannot both be 'O'.

    M       (input) INTEGER
            The number of rows of the input matrix A.  M >= 0.

    N       (input) INTEGER
            The number of columns of the input matrix A.  N >= 0.

    A       (input/output) DOUBLE PRECISION array, dimension (LDA,N)
            On entry, the M-by-N matrix A.
            On exit,
            if JOBU = 'O',  A is overwritten with the first min(m,n)
                            columns of U (the left singular vectors,
                            stored columnwise);
            if JOBVT = 'O', A is overwritten with the first min(m,n)
                            rows of V**T (the right singular vectors,
                            stored rowwise);
            if JOBU .ne. 'O' and JOBVT .ne. 'O', the contents of A
                            are destroyed.

    LDA     (input) INTEGER
            The leading dimension of the array A.  LDA >= max(1,M).

    S       (output) DOUBLE PRECISION array, dimension (min(M,N))
            The singular values of A, sorted so that S(i) >= S(i+1).

    U       (output) DOUBLE PRECISION array, dimension (LDU,UCOL)
            (LDU,M) if JOBU = 'A' or (LDU,min(M,N)) if JOBU = 'S'.
            If JOBU = 'A', U contains the M-by-M orthogonal matrix U;
            if JOBU = 'S', U contains the first min(m,n) columns of U
            (the left singular vectors, stored columnwise);
            if JOBU = 'N' or 'O', U is not referenced.

    LDU     (input) INTEGER
            The leading dimension of the array U.  LDU >= 1; if
            JOBU = 'S' or 'A', LDU >= M.

    VT      (output) DOUBLE PRECISION array, dimension (LDVT,N)
            If JOBVT = 'A', VT contains the N-by-N orthogonal matrix
            V**T;
            if JOBVT = 'S', VT contains the first min(m,n) rows of
            V**T (the right singular vectors, stored rowwise);
            if JOBVT = 'N' or 'O', VT is not referenced.

    LDVT    (input) INTEGER
            The leading dimension of the array VT.  LDVT >= 1; if
            JOBVT = 'A', LDVT >= N; if JOBVT = 'S', LDVT >= min(M,N).

    WORK    (workspace/output) DOUBLE PRECISION array, dimension (MAX(1,LWORK))
            On exit, if INFO = 0, WORK(1) returns the optimal LWORK;
            if INFO > 0, WORK(2:MIN(M,N)) contains the unconverged
            superdiagonal elements of an upper bidiagonal matrix B
            whose diagonal is in S (not necessarily sorted). B
            satisfies A = U * B * VT, so it has the same singular values
            as A, and singular vectors related by U and VT.

    LWORK   (input) INTEGER
            The dimension of the array WORK.
            LWORK >= MAX(1,3*MIN(M,N)+MAX(M,N),5*MIN(M,N)).
            For good performance, LWORK should generally be larger.

            If LWORK = -1, then a workspace query is assumed; the routine
            only calculates the optimal size of the WORK array, returns
            this value as the first entry of the WORK array, and no error
            message related to LWORK is issued by XERBLA.

    INFO    (output) INTEGER
            = 0:  successful exit.
            < 0:  if INFO = -i, the i-th argument had an illegal value.
            > 0:  if DBDSQR did not converge, INFO specifies how many
                  superdiagonals of an intermediate bidiagonal form B
                  did not converge to zero. See the description of WORK
                  above for details.
            "));
        end dgesvd_sigma;

        function dgetrf "Compute LU factorization of square or rectangular matrix A (A = P*L*U)"
          extends Modelica.Icons.Function;
          input Real A[:, :] "Square or rectangular matrix";
          output Real LU[size(A, 1), size(A, 2)] = A;
          output Integer pivots[min(size(A, 1), size(A, 2))] "Pivot vector";
          output Integer info "Information";
        protected
          Integer lda = max(1, size(A, 1));

          external "FORTRAN 77" dgetrf(size(A, 1), size(A, 2), LU, lda, pivots, info) annotation(
            Library = "lapack");
          annotation(
            Documentation(info = "Lapack documentation
    Purpose
    =======

    DGETRF computes an LU factorization of a general M-by-N matrix A
    using partial pivoting with row interchanges.

    The factorization has the form
       A = P * L * U
    where P is a permutation matrix, L is lower triangular with unit
    diagonal elements (lower trapezoidal if m > n), and U is upper
    triangular (upper trapezoidal if m < n).

    This is the right-looking Level 3 BLAS version of the algorithm.

    Arguments
    =========

    M       (input) INTEGER
            The number of rows of the matrix A.  M >= 0.

    N       (input) INTEGER
            The number of columns of the matrix A.  N >= 0.

    A       (input/output) DOUBLE PRECISION array, dimension (LDA,N)
            On entry, the M-by-N matrix to be factored.
            On exit, the factors L and U from the factorization
            A = P*L*U; the unit diagonal elements of L are not stored.

    LDA     (input) INTEGER
            The leading dimension of the array A.  LDA >= max(1,M).

    IPIV    (output) INTEGER array, dimension (min(M,N))
            The pivot indices; for 1 <= i <= min(M,N), row i of the
            matrix was interchanged with row IPIV(i).

    INFO    (output) INTEGER
            = 0:  successful exit
            < 0:  if INFO = -i, the i-th argument had an illegal value
            > 0:  if INFO = i, U(i,i) is exactly zero. The factorization
                  has been completed, but the factor U is exactly
                  singular, and division by zero will occur if it is used
                  to solve a system of equations.
            "));
        end dgetrf;

        function dgetrs "Solves a system of linear equations with the LU decomposition from dgetrf(..)"
          extends Modelica.Icons.Function;
          input Real LU[:, size(LU, 1)] "LU factorization of dgetrf of a square matrix";
          input Integer pivots[size(LU, 1)] "Pivot vector of dgetrf";
          input Real B[size(LU, 1), :] "Right hand side matrix B";
          output Real X[size(B, 1), size(B, 2)] = B "Solution matrix X";
          output Integer info;
        protected
          Real work[size(LU, 1), size(LU, 1)] = LU;
          Integer lda = max(1, size(LU, 1));
          Integer ldb = max(1, size(B, 1));

          external "FORTRAN 77" dgetrs("N", size(LU, 1), size(B, 2), work, lda, pivots, X, ldb, info) annotation(
            Library = "lapack");
          annotation(
            Documentation(info = "Lapack documentation
    Purpose
    =======

    DGETRS solves a system of linear equations
       A * X = B  or  A' * X = B
    with a general N-by-N matrix A using the LU factorization computed
    by DGETRF.

    Arguments
    =========

    TRANS   (input) CHARACTER*1
            Specifies the form of the system of equations:
            = 'N':  A * X = B  (No transpose)
            = 'T':  A'* X = B  (Transpose)
            = 'C':  A'* X = B  (Conjugate transpose = Transpose)

    N       (input) INTEGER
            The order of the matrix A.  N >= 0.

    NRHS    (input) INTEGER
            The number of right hand sides, i.e., the number of columns
            of the matrix B.  NRHS >= 0.

    A       (input) DOUBLE PRECISION array, dimension (LDA,N)
            The factors L and U from the factorization A = P*L*U
            as computed by DGETRF.

    LDA     (input) INTEGER
            The leading dimension of the array A.  LDA >= max(1,N).

    IPIV    (input) INTEGER array, dimension (N)
            The pivot indices from DGETRF; for 1<=i<=N, row i of the
            matrix was interchanged with row IPIV(i).

    B       (input/output) DOUBLE PRECISION array, dimension (LDB,NRHS)
            On entry, the right hand side matrix B.
            On exit, the solution matrix X.

    LDB     (input) INTEGER
            The leading dimension of the array B.  LDB >= max(1,N).

    INFO    (output) INTEGER
            = 0:  successful exit
            < 0:  if INFO = -i, the i-th argument had an illegal value
            "));
        end dgetrs;

        function dgetrs_vec "Solves a system of linear equations with the LU decomposition from dgetrf(..)"
          extends Modelica.Icons.Function;
          input Real LU[:, size(LU, 1)] "LU factorization of dgetrf of a square matrix";
          input Integer pivots[size(LU, 1)] "Pivot vector of dgetrf";
          input Real b[size(LU, 1)] "Right hand side vector b";
          output Real x[size(b, 1)] = b;
          output Integer info;
        protected
          Real work[size(LU, 1), size(LU, 1)] = LU;
          Integer lda = max(1, size(LU, 1));
          Integer ldb = max(1, size(b, 1));

          external "FORTRAN 77" dgetrs("N", size(LU, 1), 1, work, lda, pivots, x, ldb, info) annotation(
            Library = "lapack");
          annotation(
            Documentation(info = "Lapack documentation
    Purpose
    =======

    DGETRS solves a system of linear equations
       A * X = B  or  A' * X = B
    with a general N-by-N matrix A using the LU factorization computed
    by DGETRF.

    Arguments
    =========

    TRANS   (input) CHARACTER*1
            Specifies the form of the system of equations:
            = 'N':  A * X = B  (No transpose)
            = 'T':  A'* X = B  (Transpose)
            = 'C':  A'* X = B  (Conjugate transpose = Transpose)

    N       (input) INTEGER
            The order of the matrix A.  N >= 0.

    NRHS    (input) INTEGER
            The number of right hand sides, i.e., the number of columns
            of the matrix B.  NRHS >= 0.

    A       (input) DOUBLE PRECISION array, dimension (LDA,N)
            The factors L and U from the factorization A = P*L*U
            as computed by DGETRF.

    LDA     (input) INTEGER
            The leading dimension of the array A.  LDA >= max(1,N).

    IPIV    (input) INTEGER array, dimension (N)
            The pivot indices from DGETRF; for 1<=i<=N, row i of the
            matrix was interchanged with row IPIV(i).

    B       (input/output) DOUBLE PRECISION array, dimension (LDB,NRHS)
            On entry, the right hand side matrix B.
            On exit, the solution matrix X.

    LDB     (input) INTEGER
            The leading dimension of the array B.  LDB >= max(1,N).

    INFO    (output) INTEGER
            = 0:  successful exit
            < 0:  if INFO = -i, the i-th argument had an illegal value
            "));
        end dgetrs_vec;

        function dgetri "Computes the inverse of a matrix using the LU factorization from dgetrf(..)"
          extends Modelica.Icons.Function;
          input Real LU[:, size(LU, 1)] "LU factorization of dgetrf of a square matrix";
          input Integer pivots[size(LU, 1)] "Pivot vector of dgetrf";
          output Real inv[size(LU, 1), size(LU, 2)] = LU "Inverse of matrix P*L*U";
          output Integer info;
        protected
          Integer lda = max(1, size(LU, 1));
          Integer lwork = max(1, min(10, size(LU, 1)) * size(LU, 1)) "Length of work array";
          Real work[max(1, min(10, size(LU, 1)) * size(LU, 1))];

          external "FORTRAN 77" dgetri(size(LU, 1), inv, lda, pivots, work, lwork, info) annotation(
            Library = "lapack");
          annotation(
            Documentation(info = "Lapack documentation
    Purpose
    =======

    DGETRI computes the inverse of a matrix using the LU factorization
    computed by DGETRF.

    This method inverts U and then computes inv(A) by solving the system
    inv(A)*L = inv(U) for inv(A).

    Arguments
    =========

    N       (input) INTEGER
            The order of the matrix A.  N >= 0.

    A       (input/output) DOUBLE PRECISION array, dimension (LDA,N)
            On entry, the factors L and U from the factorization
            A = P*L*U as computed by DGETRF.
            On exit, if INFO = 0, the inverse of the original matrix A.

    LDA     (input) INTEGER
            The leading dimension of the array A.  LDA >= max(1,N).

    IPIV    (input) INTEGER array, dimension (N)
            The pivot indices from DGETRF; for 1<=i<=N, row i of the
            matrix was interchanged with row IPIV(i).

    WORK    (workspace/output) DOUBLE PRECISION array, dimension (MAX(1,LWORK))
            On exit, if INFO=0, then WORK(1) returns the optimal LWORK.

    LWORK   (input) INTEGER
            The dimension of the array WORK.  LWORK >= max(1,N).
            For optimal performance LWORK >= N*NB, where NB is
            the optimal blocksize returned by ILAENV.

            If LWORK = -1, then a workspace query is assumed; the routine
            only calculates the optimal size of the WORK array, returns
            this value as the first entry of the WORK array, and no error
            message related to LWORK is issued by XERBLA.

    INFO    (output) INTEGER
            = 0:  successful exit
            < 0:  if INFO = -i, the i-th argument had an illegal value
            > 0:  if INFO = i, U(i,i) is exactly zero; the matrix is
                  singular and its inverse could not be computed.
            "));
        end dgetri;

        function dgeqpf "Compute QR factorization of square or rectangular matrix A with column pivoting (A(:,p) = Q*R)"
          extends Modelica.Icons.Function;
          input Real A[:, :] "Square or rectangular matrix";
          output Real QR[size(A, 1), size(A, 2)] = A "QR factorization in packed format";
          output Real tau[min(size(A, 1), size(A, 2))] "The scalar factors of the elementary reflectors of Q";
          output Integer p[size(A, 2)] = zeros(size(A, 2)) "Pivot vector";
          output Integer info;
        protected
          Integer lda = max(1, size(A, 1));
          Integer ncol = size(A, 2) "Column dimension of A";
          Real work[3 * size(A, 2)] "work array";

          external "FORTRAN 77" dgeqpf(size(A, 1), ncol, QR, lda, p, tau, work, info) annotation(
            Library = {"lapack"});
          annotation(
            Documentation(info = "Lapack documentation
    Purpose
    =======

    This routine is deprecated and has been replaced by routine DGEQP3.

    DGEQPF computes a QR factorization with column pivoting of a
    real M-by-N matrix A: A*P = Q*R.

    Arguments
    =========

    M       (input) INTEGER
            The number of rows of the matrix A. M >= 0.

    N       (input) INTEGER
            The number of columns of the matrix A. N >= 0

    A       (input/output) DOUBLE PRECISION array, dimension (LDA,N)
            On entry, the M-by-N matrix A.
            On exit, the upper triangle of the array contains the
            min(M,N)-by-N upper triangular matrix R; the elements
            below the diagonal, together with the array TAU,
            represent the orthogonal matrix Q as a product of
            min(m,n) elementary reflectors.

    LDA     (input) INTEGER
            The leading dimension of the array A. LDA >= max(1,M).

    JPVT    (input/output) INTEGER array, dimension (N)
            On entry, if JPVT(i) .ne. 0, the i-th column of A is permuted
            to the front of A*P (a leading column); if JPVT(i) = 0,
            the i-th column of A is a free column.
            On exit, if JPVT(i) = k, then the i-th column of A*P
            was the k-th column of A.

    TAU     (output) DOUBLE PRECISION array, dimension (min(M,N))
            The scalar factors of the elementary reflectors.

    WORK    (workspace) DOUBLE PRECISION array, dimension (3*N)

    INFO    (output) INTEGER
            = 0:  successful exit
            < 0:  if INFO = -i, the i-th argument had an illegal value

    Further Details
    ===============

    The matrix Q is represented as a product of elementary reflectors

       Q = H(1) H(2) . . . H(n)

    Each H(i) has the form

       H = I - tau * v * v'

    where tau is a real scalar, and v is a real vector with
    v(1:i-1) = 0 and v(i) = 1; v(i+1:m) is stored on exit in A(i+1:m,i).

    The matrix P is represented in jpvt as follows: If
       jpvt(j) = i
    then the jth column of P is the ith canonical unit vector.
            "));
        end dgeqpf;

        function dorgqr "Generates a Real orthogonal matrix Q which is defined as the product of elementary reflectors as returned from dgeqpf"
          extends Modelica.Icons.Function;
          input Real QR[:, :] "QR from dgeqpf";
          input Real tau[min(size(QR, 1), size(QR, 2))] "The scalar factors of the elementary reflectors of Q";
          output Real Q[size(QR, 1), size(QR, 2)] = QR "Orthogonal matrix Q";
          output Integer info;
        protected
          Integer lda = max(1, size(Q, 1));
          Integer lwork = max(1, min(10, size(QR, 2)) * size(QR, 2)) "Length of work array";
          Real work[max(1, min(10, size(QR, 2)) * size(QR, 2))];

          external "FORTRAN 77" dorgqr(size(QR, 1), size(QR, 2), size(tau, 1), Q, lda, tau, work, lwork, info) annotation(
            Library = {"lapack"});
          annotation(
            Documentation(info = "Lapack documentation
    Purpose
    =======

    DORGQR generates an M-by-N real matrix Q with orthonormal columns,
    which is defined as the first N columns of a product of K elementary
    reflectors of order M

          Q  =  H(1) H(2) . . . H(k)

    as returned by DGEQRF.

    Arguments
    =========

    M       (input) INTEGER
            The number of rows of the matrix Q. M >= 0.

    N       (input) INTEGER
            The number of columns of the matrix Q. M >= N >= 0.

    K       (input) INTEGER
            The number of elementary reflectors whose product defines the
            matrix Q. N >= K >= 0.

    A       (input/output) DOUBLE PRECISION array, dimension (LDA,N)
            On entry, the i-th column must contain the vector which
            defines the elementary reflector H(i), for i = 1,2,...,k, as
            returned by DGEQRF in the first k columns of its array
            argument A.
            On exit, the M-by-N matrix Q.

    LDA     (input) INTEGER
            The first dimension of the array A. LDA >= max(1,M).

    TAU     (input) DOUBLE PRECISION array, dimension (K)
            TAU(i) must contain the scalar factor of the elementary
            reflector H(i), as returned by DGEQRF.

    WORK    (workspace/output) DOUBLE PRECISION array, dimension (MAX(1,LWORK))
            On exit, if INFO = 0, WORK(1) returns the optimal LWORK.

    LWORK   (input) INTEGER
            The dimension of the array WORK. LWORK >= max(1,N).
            For optimum performance LWORK >= N*NB, where NB is the
            optimal blocksize.

            If LWORK = -1, then a workspace query is assumed; the routine
            only calculates the optimal size of the WORK array, returns
            this value as the first entry of the WORK array, and no error
            message related to LWORK is issued by XERBLA.

    INFO    (output) INTEGER
            = 0:  successful exit
            < 0:  if INFO = -i, the i-th argument has an illegal value
            "));
        end dorgqr;

        function dgees "Computes real Schur form T of real nonsymmetric matrix A, and, optionally, the matrix of Schur vectors Z as well as the eigenvalues"
          extends Modelica.Icons.Function;
          input Real A[:, size(A, 1)] "Square matrix";
          output Real T[size(A, 1), size(A, 2)] = A "Real Schur form with A = Z*T*Z'";
          output Real Z[size(A, 1), size(A, 1)] "orthogonal matrix Z of Schur vectors";
          output Real eval_real[size(A, 1)] "real part of the eigenvectors of A";
          output Real eval_imag[size(A, 1)] "imaginary part of the eigenvectors of A";
          output Integer info;
        protected
          constant Integer dummyFunctionPointerNotUsed[1] = {0};
          Integer n = size(A, 1) "Row dimension of A";
          Integer lda = max(1, n);
          Integer sdim = 0;
          Integer lwork = max(1, 10 * size(A, 1));
          Real work[lwork];
          Boolean bwork[size(A, 1)];

          external "FORTRAN 77" dgees("V", "N", dummyFunctionPointerNotUsed, n, T, lda, sdim, eval_real, eval_imag, Z, lda, work, lwork, bwork, info) annotation(
            Library = {"lapack"});
          annotation(
            Documentation(info = "Lapack documentation
    Purpose
    =======

    DGEES computes for an N-by-N real nonsymmetric matrix A, the
    eigenvalues, the real Schur form T, and, optionally, the matrix of
    Schur vectors Z.  This gives the Schur factorization A = Z*T*(Z**T).

    Optionally, it also orders the eigenvalues on the diagonal of the
    real Schur form so that selected eigenvalues are at the top left.
    The leading columns of Z then form an orthonormal basis for the
    invariant subspace corresponding to the selected eigenvalues.

    A matrix is in real Schur form if it is upper quasi-triangular with
    1-by-1 and 2-by-2 blocks. 2-by-2 blocks will be standardized in the
    form
            [  a  b  ]
            [  c  a  ]

    where b*c < 0. The eigenvalues of such a block are a +- sqrt(bc).

    Arguments
    =========

    JOBVS   (input) CHARACTER*1
            = 'N': Schur vectors are not computed;
            = 'V': Schur vectors are computed.

    SORT    (input) CHARACTER*1
            Specifies whether or not to order the eigenvalues on the
            diagonal of the Schur form.
            = 'N': Eigenvalues are not ordered;
            = 'S': Eigenvalues are ordered (see SELECT).

    SELECT  (external procedure) LOGICAL FUNCTION of two DOUBLE PRECISION arguments
            SELECT must be declared EXTERNAL in the calling subroutine.
            If SORT = 'S', SELECT is used to select eigenvalues to sort
            to the top left of the Schur form.
            If SORT = 'N', SELECT is not referenced.
            An eigenvalue WR(j)+sqrt(-1)*WI(j) is selected if
            SELECT(WR(j),WI(j)) is true; i.e., if either one of a complex
            conjugate pair of eigenvalues is selected, then both complex
            eigenvalues are selected.
            Note that a selected complex eigenvalue may no longer
            satisfy SELECT(WR(j),WI(j)) = .TRUE. after ordering, since
            ordering may change the value of complex eigenvalues
            (especially if the eigenvalue is ill-conditioned); in this
            case INFO is set to N+2 (see INFO below).

    N       (input) INTEGER
            The order of the matrix A. N >= 0.

    A       (input/output) DOUBLE PRECISION array, dimension (LDA,N)
            On entry, the N-by-N matrix A.
            On exit, A has been overwritten by its real Schur form T.

    LDA     (input) INTEGER
            The leading dimension of the array A.  LDA >= max(1,N).

    SDIM    (output) INTEGER
            If SORT = 'N', SDIM = 0.
            If SORT = 'S', SDIM = number of eigenvalues (after sorting)
                           for which SELECT is true. (Complex conjugate
                           pairs for which SELECT is true for either
                           eigenvalue count as 2.)

    WR      (output) DOUBLE PRECISION array, dimension (N)
    WI      (output) DOUBLE PRECISION array, dimension (N)
            WR and WI contain the real and imaginary parts,
            respectively, of the computed eigenvalues in the same order
            that they appear on the diagonal of the output Schur form T.
            Complex conjugate pairs of eigenvalues will appear
            consecutively with the eigenvalue having the positive
            imaginary part first.

    VS      (output) DOUBLE PRECISION array, dimension (LDVS,N)
            If JOBVS = 'V', VS contains the orthogonal matrix Z of Schur
            vectors.
            If JOBVS = 'N', VS is not referenced.

    LDVS    (input) INTEGER
            The leading dimension of the array VS.  LDVS >= 1; if
            JOBVS = 'V', LDVS >= N.

    WORK    (workspace/output) DOUBLE PRECISION array, dimension (MAX(1,LWORK))
            On exit, if INFO = 0, WORK(1) contains the optimal LWORK.

    LWORK   (input) INTEGER
            The dimension of the array WORK.  LWORK >= max(1,3*N).
            For good performance, LWORK must generally be larger.

            If LWORK = -1, then a workspace query is assumed; the routine
            only calculates the optimal size of the WORK array, returns
            this value as the first entry of the WORK array, and no error
            message related to LWORK is issued by XERBLA.

    BWORK   (workspace) LOGICAL array, dimension (N)
            Not referenced if SORT = 'N'.

    INFO    (output) INTEGER
            = 0: successful exit
            < 0: if INFO = -i, the i-th argument had an illegal value.
            > 0: if INFO = i, and i is
               <= N: the QR algorithm failed to compute all the
                     eigenvalues; elements 1:ILO-1 and i+1:N of WR and WI
                     contain those eigenvalues which have converged; if
                     JOBVS = 'V', VS contains the matrix which reduces A
                     to its partially converged Schur form.
               = N+1: the eigenvalues could not be reordered because some
                     eigenvalues were too close to separate (the problem
                     is very ill-conditioned);
               = N+2: after reordering, roundoff changed values of some
                     complex eigenvalues so that leading eigenvalues in
                     the Schur form no longer satisfy SELECT=.TRUE.  This
                     could also be caused by underflow due to scaling.
            "));
        end dgees;

        function dtrsen "Reorder the real Schur factorization of a real matrix"
          extends Modelica.Icons.Function;
          input String job = "N" "Specifies the usage of a condition number";
          input String compq = "V" "Is \"V\" if Schur vector matrix is to be updated";
          input Boolean select[:] "Specifies the eigenvalues to reorder";
          input Real T[:, :] "Real Schur form to be reordered";
          input Real Q[:, size(T, 2)] "Matrix of the Schur vectors";
        protected
          Integer n = size(T, 2);
          Integer ldt = max(1, n);
          Integer ldq = if compq == "V" then max(n, 1) else 1;
          Integer lwork = if job == "N" then max(1, n) else if job == "E" then n * n else 2 * n * n;
          Real work[if job == "N" then max(1, size(T, 2)) else if job == "E" then size(T, 2) * size(T, 2) else 2 * size(T, 2) * size(T, 2)];
          Integer liwork = if job == "N" or job == "E" then 1 else n * n;
          Integer iwork[if job == "N" or job == "E" then 1 else size(T, 2) * size(T, 2)];
        public
          output Real To[:, :] = T "Reordered Schur form";
          output Real Qo[:, :] = Q "Reordered Schur vectors";
          output Real wr[size(T, 2)] "Reordered eigenvalues, real part";
          output Real wi[size(T, 2)] "Reordered eigenvalues, imaginary part";
          output Integer m "Dimension of the invariant sub space spanned bei the selected eigenvalues";
          output Real s "Lower bound of the reciprocal condition number. Not referenced for job==V";
          output Real sep "Estimated reciprocal condition number of the specified invariant subspace";
          output Integer info;

          external "FORTRAN 77" dtrsen(job, compq, select, n, To, ldt, Qo, ldq, wr, wi, m, s, sep, work, lwork, iwork, liwork, info) annotation(
            Library = {"lapack"});
          annotation(
            Documentation(info = "Lapack documentation
    Purpose
    =======

    DTRSEN reorders the real Schur factorization of a real matrix
    A = Q*T*Q**T, so that a selected cluster of eigenvalues appears in
    the leading diagonal blocks of the upper quasi-triangular matrix T,
    and the leading columns of Q form an orthonormal basis of the
    corresponding right invariant subspace.

    Optionally the routine computes the reciprocal condition numbers of
    the cluster of eigenvalues and/or the invariant subspace.

    T must be in Schur canonical form (as returned by DHSEQR), that is,
    block upper triangular with 1-by-1 and 2-by-2 diagonal blocks; each
    2-by-2 diagonal block has its diagonal elemnts equal and its
    off-diagonal elements of opposite sign.

    Arguments
    =========

    JOB     (input) CHARACTER*1
            Specifies whether condition numbers are required for the
            cluster of eigenvalues (S) or the invariant subspace (SEP):
            = 'N': none;
            = 'E': for eigenvalues only (S);
            = 'V': for invariant subspace only (SEP);
            = 'B': for both eigenvalues and invariant subspace (S and
                   SEP).

    COMPQ   (input) CHARACTER*1
            = 'V': update the matrix Q of Schur vectors;
            = 'N': do not update Q.

    SELECT  (input) LOGICAL array, dimension (N)
            SELECT specifies the eigenvalues in the selected cluster. To
            select a real eigenvalue w(j), SELECT(j) must be set to
            .TRUE.. To select a complex conjugate pair of eigenvalues
            w(j) and w(j+1), corresponding to a 2-by-2 diagonal block,
            either SELECT(j) or SELECT(j+1) or both must be set to
            .TRUE.; a complex conjugate pair of eigenvalues must be
            either both included in the cluster or both excluded.

    N       (input) INTEGER
            The order of the matrix T. N >= 0.

    T       (input/output) DOUBLE PRECISION array, dimension (LDT,N)
            On entry, the upper quasi-triangular matrix T, in Schur
            canonical form.
            On exit, T is overwritten by the reordered matrix T, again in
            Schur canonical form, with the selected eigenvalues in the
            leading diagonal blocks.

    LDT     (input) INTEGER
            The leading dimension of the array T. LDT >= max(1,N).

    Q       (input/output) DOUBLE PRECISION array, dimension (LDQ,N)
            On entry, if COMPQ = 'V', the matrix Q of Schur vectors.
            On exit, if COMPQ = 'V', Q has been postmultiplied by the
            orthogonal transformation matrix which reorders T; the
            leading M columns of Q form an orthonormal basis for the
            specified invariant subspace.
            If COMPQ = 'N', Q is not referenced.

    LDQ     (input) INTEGER
            The leading dimension of the array Q.
            LDQ >= 1; and if COMPQ = 'V', LDQ >= N.

    WR      (output) DOUBLE PRECISION array, dimension (N)
    WI      (output) DOUBLE PRECISION array, dimension (N)
            The real and imaginary parts, respectively, of the reordered
            eigenvalues of T. The eigenvalues are stored in the same
            order as on the diagonal of T, with WR(i) = T(i,i) and, if
            T(i:i+1,i:i+1) is a 2-by-2 diagonal block, WI(i) > 0 and
            WI(i+1) = -WI(i). Note that if a complex eigenvalue is
            sufficiently ill-conditioned, then its value may differ
            significantly from its value before reordering.

    M       (output) INTEGER
            The dimension of the specified invariant subspace.
            0 < = M <= N.

    S       (output) DOUBLE PRECISION
            If JOB = 'E' or 'B', S is a lower bound on the reciprocal
            condition number for the selected cluster of eigenvalues.
            S cannot underestimate the true reciprocal condition number
            by more than a factor of sqrt(N). If M = 0 or N, S = 1.
            If JOB = 'N' or 'V', S is not referenced.

    SEP     (output) DOUBLE PRECISION
            If JOB = 'V' or 'B', SEP is the estimated reciprocal
            condition number of the specified invariant subspace. If
            M = 0 or N, SEP = norm(T).
            If JOB = 'N' or 'E', SEP is not referenced.

    WORK    (workspace/output) DOUBLE PRECISION array, dimension (MAX(1,LWORK))
            On exit, if INFO = 0, WORK(1) returns the optimal LWORK.

    LWORK   (input) INTEGER
            The dimension of the array WORK.
            If JOB = 'N', LWORK >= max(1,N);
            if JOB = 'E', LWORK >= max(1,M*(N-M));
            if JOB = 'V' or 'B', LWORK >= max(1,2*M*(N-M)).

            If LWORK = -1, then a workspace query is assumed; the routine
            only calculates the optimal size of the WORK array, returns
            this value as the first entry of the WORK array, and no error
            message related to LWORK is issued by XERBLA.

    IWORK   (workspace) INTEGER array, dimension (MAX(1,LIWORK))
            On exit, if INFO = 0, IWORK(1) returns the optimal LIWORK.

    LIWORK  (input) INTEGER
            The dimension of the array IWORK.
            If JOB = 'N' or 'E', LIWORK >= 1;
            if JOB = 'V' or 'B', LIWORK >= max(1,M*(N-M)).

            If LIWORK = -1, then a workspace query is assumed; the
            routine only calculates the optimal size of the IWORK array,
            returns this value as the first entry of the IWORK array, and
            no error message related to LIWORK is issued by XERBLA.

    INFO    (output) INTEGER
            = 0: successful exit
            < 0: if INFO = -i, the i-th argument had an illegal value
            = 1: reordering of T failed because some eigenvalues are too
                 close to separate (the problem is very ill-conditioned);
                 T may have been partially reordered, and WR and WI
                 contain the eigenvalues in the same order as in T; S and
                 SEP (if requested) are set to zero.

    Further Details
    ===============

    DTRSEN first collects the selected eigenvalues by computing an
    orthogonal transformation Z to move them to the top left corner of T.
    In other words, the selected eigenvalues are the eigenvalues of T11
    in:

                  Z'*T*Z = ( T11 T12 ) n1
                           (  0  T22 ) n2
                              n1  n2

    where N = n1+n2 and Z' means the transpose of Z. The first n1 columns
    of Z span the specified invariant subspace of T.

    If T has been obtained from the real Schur factorization of a matrix
    A = Q*T*Q', then the reordered real Schur factorization of A is given
    by A = (Q*Z)*(Z'*T*Z)*(Q*Z)', and the first n1 columns of Q*Z span
    the corresponding invariant subspace of A.

    The reciprocal condition number of the average of the eigenvalues of
    T11 may be returned in S. S lies between 0 (very badly conditioned)
    and 1 (very well conditioned). It is computed as follows. First we
    compute R so that

                           P = ( I  R ) n1
                               ( 0  0 ) n2
                                 n1 n2

    is the projector on the invariant subspace associated with T11.
    R is the solution of the Sylvester equation:

                          T11*R - R*T22 = T12.

    Let F-norm(M) denote the Frobenius-norm of M and 2-norm(M) denote
    the two-norm of M. Then S is computed as the lower bound

                        (1 + F-norm(R)**2)**(-1/2)

    on the reciprocal of 2-norm(P), the true reciprocal condition number.
    S cannot underestimate 1 / 2-norm(P) by more than a factor of
    sqrt(N).

    An approximate error bound for the computed average of the
    eigenvalues of T11 is

                           EPS * norm(T) / S

    where EPS is the machine precision.

    The reciprocal condition number of the right invariant subspace
    spanned by the first n1 columns of Z (or of Q*Z) is returned in SEP.
    SEP is defined as the separation of T11 and T22:

                       sep( T11, T22 ) = sigma-min( C )

    where sigma-min(C) is the smallest singular value of the
    n1*n2-by-n1*n2 matrix

       C  = kprod( I(n2), T11 ) - kprod( transpose(T22), I(n1) )

    I(m) is an m by m identity matrix, and kprod denotes the Kronecker
    product. We estimate sigma-min(C) by the reciprocal of an estimate of
    the 1-norm of inverse(C). The true reciprocal 1-norm of inverse(C)
    cannot differ from sigma-min(C) by more than a factor of sqrt(n1*n2).

    When SEP is small, small changes in T can cause large changes in
    the invariant subspace. An approximate bound on the maximum angular
    error in the computed right invariant subspace is

                        EPS * norm(T) / SEP
            "));
        end dtrsen;

        function dgesvx "Solve real system of linear equations op(A)*X=B, op(A) is A or A' according to the Boolean input transposed"
          extends Modelica.Icons.Function;
          input Real A[:, size(A, 1)] "Real square matrix A";
          input Real B[size(A, 1), :] "Real matrix B";
          input Boolean transposed = true "True if the equation to be solved is A'*X=B";
          output Real X[size(A, 1), size(B, 2)] "Solution matrix";
          output Integer info;
          output Real rcond "reciprocal condition number of the matrix A";
        protected
          String transA = if transposed then "T" else "N";
          Real Awork[size(A, 1), size(A, 2)] = A;
          Real Bwork[size(B, 1), size(B, 2)] = B;
          Real AF[size(A, 1), size(A, 2)];
          Real R[size(A, 1)];
          Real C[size(A, 1)];
          Real ferr[size(B, 2)];
          Real berr[size(B, 2)];
          Integer lda = max(1, size(A, 1));
          Real work[4 * size(A, 1)];
          Integer ipiv[size(A, 1)];
          Integer iwork[size(A, 1)];
          String equed = " ";

          external "FORTRAN 77" dgesvx("N", transA, size(A, 1), size(B, 2), Awork, lda, AF, lda, ipiv, equed, R, C, B, lda, X, lda, rcond, ferr, berr, work, iwork, info) annotation(
            Library = "lapack");
          annotation(
            Documentation(info = "Lapack documentation
    Purpose
    =======

    DGESVX uses the LU factorization to compute the solution to a real
    system of linear equations
       A * X = B,
    where A is an N-by-N matrix and X and B are N-by-NRHS matrices.

    Error bounds on the solution and a condition estimate are also
    provided.

    Description
    ===========

    The following steps are performed:

    1. If FACT = 'E', real scaling factors are computed to equilibrate
       the system:
          TRANS = 'N':  diag(R)*A*diag(C)     *inv(diag(C))*X = diag(R)*B
          TRANS = 'T': (diag(R)*A*diag(C))**T *inv(diag(R))*X = diag(C)*B
          TRANS = 'C': (diag(R)*A*diag(C))**H *inv(diag(R))*X = diag(C)*B
       Whether or not the system will be equilibrated depends on the
       scaling of the matrix A, but if equilibration is used, A is
       overwritten by diag(R)*A*diag(C) and B by diag(R)*B (if TRANS='N')
       or diag(C)*B (if TRANS = 'T' or 'C').

    2. If FACT = 'N' or 'E', the LU decomposition is used to factor the
       matrix A (after equilibration if FACT = 'E') as
          A = P * L * U,
       where P is a permutation matrix, L is a unit lower triangular
       matrix, and U is upper triangular.

    3. If some U(i,i)=0, so that U is exactly singular, then the routine
       returns with INFO = i. Otherwise, the factored form of A is used
       to estimate the condition number of the matrix A.  If the
       reciprocal of the condition number is less than machine precision,
       INFO = N+1 is returned as a warning, but the routine still goes on
       to solve for X and compute error bounds as described below.

    4. The system of equations is solved for X using the factored form
       of A.

    5. Iterative refinement is applied to improve the computed solution
       matrix and calculate error bounds and backward error estimates
       for it.

    6. If equilibration was used, the matrix X is premultiplied by
       diag(C) (if TRANS = 'N') or diag(R) (if TRANS = 'T' or 'C') so
       that it solves the original system before equilibration.

    Arguments
    =========

    FACT    (input) CHARACTER*1
            Specifies whether or not the factored form of the matrix A is
            supplied on entry, and if not, whether the matrix A should be
            equilibrated before it is factored.
            = 'F':  On entry, AF and IPIV contain the factored form of A.
                    If EQUED is not 'N', the matrix A has been
                    equilibrated with scaling factors given by R and C.
                    A, AF, and IPIV are not modified.
            = 'N':  The matrix A will be copied to AF and factored.
            = 'E':  The matrix A will be equilibrated if necessary, then
                    copied to AF and factored.

    TRANS   (input) CHARACTER*1
            Specifies the form of the system of equations:
            = 'N':  A * X = B     (No transpose)
            = 'T':  A**T * X = B  (Transpose)
            = 'C':  A**H * X = B  (Transpose)

    N       (input) INTEGER
            The number of linear equations, i.e., the order of the
            matrix A.  N >= 0.

    NRHS    (input) INTEGER
            The number of right hand sides, i.e., the number of columns
            of the matrices B and X.  NRHS >= 0.

    A       (input/output) DOUBLE PRECISION array, dimension (LDA,N)
            On entry, the N-by-N matrix A.  If FACT = 'F' and EQUED is
            not 'N', then A must have been equilibrated by the scaling
            factors in R and/or C.  A is not modified if FACT = 'F' or
            'N', or if FACT = 'E' and EQUED = 'N' on exit.

            On exit, if EQUED .ne. 'N', A is scaled as follows:
            EQUED = 'R':  A := diag(R) * A
            EQUED = 'C':  A := A * diag(C)
            EQUED = 'B':  A := diag(R) * A * diag(C).

    LDA     (input) INTEGER
            The leading dimension of the array A.  LDA >= max(1,N).

    AF      (input or output) DOUBLE PRECISION array, dimension (LDAF,N)
            If FACT = 'F', then AF is an input argument and on entry
            contains the factors L and U from the factorization
            A = P*L*U as computed by DGETRF.  If EQUED .ne. 'N', then
            AF is the factored form of the equilibrated matrix A.

            If FACT = 'N', then AF is an output argument and on exit
            returns the factors L and U from the factorization A = P*L*U
            of the original matrix A.

            If FACT = 'E', then AF is an output argument and on exit
            returns the factors L and U from the factorization A = P*L*U
            of the equilibrated matrix A (see the description of A for
            the form of the equilibrated matrix).

    LDAF    (input) INTEGER
            The leading dimension of the array AF.  LDAF >= max(1,N).

    IPIV    (input or output) INTEGER array, dimension (N)
            If FACT = 'F', then IPIV is an input argument and on entry
            contains the pivot indices from the factorization A = P*L*U
            as computed by DGETRF; row i of the matrix was interchanged
            with row IPIV(i).

            If FACT = 'N', then IPIV is an output argument and on exit
            contains the pivot indices from the factorization A = P*L*U
            of the original matrix A.

            If FACT = 'E', then IPIV is an output argument and on exit
            contains the pivot indices from the factorization A = P*L*U
            of the equilibrated matrix A.

    EQUED   (input or output) CHARACTER*1
            Specifies the form of equilibration that was done.
            = 'N':  No equilibration (always true if FACT = 'N').
            = 'R':  Row equilibration, i.e., A has been premultiplied by
                    diag(R).
            = 'C':  Column equilibration, i.e., A has been postmultiplied
                    by diag(C).
            = 'B':  Both row and column equilibration, i.e., A has been
                    replaced by diag(R) * A * diag(C).
            EQUED is an input argument if FACT = 'F'; otherwise, it is an
            output argument.

    R       (input or output) DOUBLE PRECISION array, dimension (N)
            The row scale factors for A.  If EQUED = 'R' or 'B', A is
            multiplied on the left by diag(R); if EQUED = 'N' or 'C', R
            is not accessed.  R is an input argument if FACT = 'F';
            otherwise, R is an output argument.  If FACT = 'F' and
            EQUED = 'R' or 'B', each element of R must be positive.

    C       (input or output) DOUBLE PRECISION array, dimension (N)
            The column scale factors for A.  If EQUED = 'C' or 'B', A is
            multiplied on the right by diag(C); if EQUED = 'N' or 'R', C
            is not accessed.  C is an input argument if FACT = 'F';
            otherwise, C is an output argument.  If FACT = 'F' and
            EQUED = 'C' or 'B', each element of C must be positive.

    B       (input/output) DOUBLE PRECISION array, dimension (LDB,NRHS)
            On entry, the N-by-NRHS right hand side matrix B.
            On exit,
            if EQUED = 'N', B is not modified;
            if TRANS = 'N' and EQUED = 'R' or 'B', B is overwritten by
            diag(R)*B;
            if TRANS = 'T' or 'C' and EQUED = 'C' or 'B', B is
            overwritten by diag(C)*B.

    LDB     (input) INTEGER
            The leading dimension of the array B.  LDB >= max(1,N).

    X       (output) DOUBLE PRECISION array, dimension (LDX,NRHS)
            If INFO = 0 or INFO = N+1, the N-by-NRHS solution matrix X
            to the original system of equations.  Note that A and B are
            modified on exit if EQUED .ne. 'N', and the solution to the
            equilibrated system is inv(diag(C))*X if TRANS = 'N' and
            EQUED = 'C' or 'B', or inv(diag(R))*X if TRANS = 'T' or 'C'
            and EQUED = 'R' or 'B'.

    LDX     (input) INTEGER
            The leading dimension of the array X.  LDX >= max(1,N).

    RCOND   (output) DOUBLE PRECISION
            The estimate of the reciprocal condition number of the matrix
            A after equilibration (if done).  If RCOND is less than the
            machine precision (in particular, if RCOND = 0), the matrix
            is singular to working precision.  This condition is
            indicated by a return code of INFO > 0.

    FERR    (output) DOUBLE PRECISION array, dimension (NRHS)
            The estimated forward error bound for each solution vector
            X(j) (the j-th column of the solution matrix X).
            If XTRUE is the true solution corresponding to X(j), FERR(j)
            is an estimated upper bound for the magnitude of the largest
            element in (X(j) - XTRUE) divided by the magnitude of the
            largest element in X(j).  The estimate is as reliable as
            the estimate for RCOND, and is almost always a slight
            overestimate of the true error.

    BERR    (output) DOUBLE PRECISION array, dimension (NRHS)
            The componentwise relative backward error of each solution
            vector X(j) (i.e., the smallest relative change in
            any element of A or B that makes X(j) an exact solution).

    WORK    (workspace/output) DOUBLE PRECISION array, dimension (4*N)
            On exit, WORK(1) contains the reciprocal pivot growth
            factor norm(A)/norm(U). The \"max absolute element\" norm is
            used. If WORK(1) is much less than 1, then the stability
            of the LU factorization of the (equilibrated) matrix A
            could be poor. This also means that the solution X, condition
            estimator RCOND, and forward error bound FERR could be
            unreliable. If factorization fails with 0<INFO<=N, then
            WORK(1) contains the reciprocal pivot growth factor for the
            leading INFO columns of A.

    IWORK   (workspace) INTEGER array, dimension (N)

    INFO    (output) INTEGER
            = 0:  successful exit
            < 0:  if INFO = -i, the i-th argument had an illegal value
            > 0:  if INFO = i, and i is
                  <= N:  U(i,i) is exactly zero.  The factorization has
                         been completed, but the factor U is exactly
                         singular, so the solution and error bounds
                         could not be computed. RCOND = 0 is returned.
                  = N+1: U is nonsingular, but RCOND is less than machine
                         precision, meaning that the matrix is singular
                         to working precision.  Nevertheless, the
                         solution and error bounds are computed because
                         there are a number of situations where the
                         computed solution can be more accurate than the
                         value of RCOND would suggest.
            "));
        end dgesvx;

        function dtrsyl "Solve the real Sylvester matrix equation op(A)*X + X*op(B) = scale*C or op(A)*X - X*op(B) = scale*C"
          extends Modelica.Icons.Function;
          input Real A[:, :] "Upper quais-triangular matrix";
          input Real B[:, :] "Upper quais-triangular matrix";
          input Real C[if tranA then size(A, 1) else size(A, 2), if tranB then size(B, 1) else size(B, 2)] "Right side of the Sylvester equation";
          input Boolean tranA = false "True if op(A)=A'";
          input Boolean tranB = false "True if op(B)=B'";
          input Integer isgn = 1 "Specifies the sign in the equation, +1 or -1";
          output Real X[size(C, 1), size(C, 2)] = C "Solution of the Sylvester equation";
          output Real scale "Scale factor";
          output Integer info;
        protected
          Integer m = if tranA then size(A, 1) else size(A, 2);
          Integer n = if tranB then size(B, 1) else size(B, 2);
          String trana = if tranA then "T" else "N";
          String tranb = if tranB then "T" else "N";
          Integer lda = max(1, m);
          Integer ldb = max(1, n);

          external "FORTRAN 77" dtrsyl(trana, tranb, isgn, m, n, A, lda, B, ldb, X, lda, scale, info) annotation(
            Library = {"lapack"});
          annotation(
            Documentation(info = "Lapack documentation
    Purpose
    =======

    DTRSYL solves the real Sylvester matrix equation:

       op(A)*X + X*op(B) = scale*C or
       op(A)*X - X*op(B) = scale*C,

    where op(A) = A or A**T, and  A and B are both upper quasi-
    triangular. A is M-by-M and B is N-by-N; the right hand side C and
    the solution X are M-by-N; and scale is an output scale factor, set
    <= 1 to avoid overflow in X.

    A and B must be in Schur canonical form (as returned by DHSEQR), that
    is, block upper triangular with 1-by-1 and 2-by-2 diagonal blocks;
    each 2-by-2 diagonal block has its diagonal elements equal and its
    off-diagonal elements of opposite sign.

    Arguments
    =========

    TRANA   (input) CHARACTER*1
            Specifies the option op(A):
            = 'N': op(A) = A    (No transpose)
            = 'T': op(A) = A**T (Transpose)
            = 'C': op(A) = A**H (Conjugate transpose = Transpose)

    TRANB   (input) CHARACTER*1
            Specifies the option op(B):
            = 'N': op(B) = B    (No transpose)
            = 'T': op(B) = B**T (Transpose)
            = 'C': op(B) = B**H (Conjugate transpose = Transpose)

    ISGN    (input) INTEGER
            Specifies the sign in the equation:
            = +1: solve op(A)*X + X*op(B) = scale*C
            = -1: solve op(A)*X - X*op(B) = scale*C

    M       (input) INTEGER
            The order of the matrix A, and the number of rows in the
            matrices X and C. M >= 0.

    N       (input) INTEGER
            The order of the matrix B, and the number of columns in the
            matrices X and C. N >= 0.

    A       (input) DOUBLE PRECISION array, dimension (LDA,M)
            The upper quasi-triangular matrix A, in Schur canonical form.

    LDA     (input) INTEGER
            The leading dimension of the array A. LDA >= max(1,M).

    B       (input) DOUBLE PRECISION array, dimension (LDB,N)
            The upper quasi-triangular matrix B, in Schur canonical form.

    LDB     (input) INTEGER
            The leading dimension of the array B. LDB >= max(1,N).

    C       (input/output) DOUBLE PRECISION array, dimension (LDC,N)
            On entry, the M-by-N right hand side matrix C.
            On exit, C is overwritten by the solution matrix X.

    LDC     (input) INTEGER
            The leading dimension of the array C. LDC >= max(1,M)

    SCALE   (output) DOUBLE PRECISION
            The scale factor, scale, set <= 1 to avoid overflow in X.

    INFO    (output) INTEGER
            = 0: successful exit
            < 0: if INFO = -i, the i-th argument had an illegal value
            = 1: A and B have common or very close eigenvalues; perturbed
                 values were used to solve the equation (but the matrices
                 A and B are unchanged).
            "));
        end dtrsyl;

        function dhseqr "Compute eigenvalues of a matrix H using lapack routine DHSEQR for Hessenberg form matrix"
          extends Modelica.Icons.Function;
          input Real H[:, size(H, 1)] "Matrix H with Hessenberg form";
          input Boolean eigenValuesOnly = true "True to compute the eigenvalues. False to compute the Schur form too";
          input String compz = "N" "Specifies the computation of the Schur vectors";
          input Real Z[:, :] = H "Matrix Z";
          output Real alphaReal[size(H, 1)] "Real part of alpha (eigenvalue=(alphaReal+i*alphaImag))";
          output Real alphaImag[size(H, 1)] "Imaginary part of alpha (eigenvalue=(alphaReal+i*alphaImag))";
          output Integer info;
          output Real Ho[:, :] = H "Schur decomposition (if eigenValuesOnly==false, unspecified else)";
          output Real Zo[:, :] = Z;
          output Real work[3 * max(1, size(H, 1))];
        protected
          Integer n = size(H, 1);
          String job = if eigenValuesOnly then "E" else "S";
          Integer ilo = 1;
          Integer ihi = n;
          Integer ldh = max(n, 1);
          Integer lwork = 3 * max(1, size(H, 1)) "Dimension of the dwork array used in dhseqr";

          external "FORTRAN 77" dhseqr(job, compz, n, ilo, ihi, Ho, ldh, alphaReal, alphaImag, Zo, ldh, work, lwork, info) annotation(
            Library = {"lapack"});
          annotation(
            Documentation(info = "Lapack documentation
    Purpose
    =======

    DHSEQR computes the eigenvalues of a Hessenberg matrix H
    and, optionally, the matrices T and Z from the Schur decomposition
    H = Z T Z**T, where T is an upper quasi-triangular matrix (the
    Schur form), and Z is the orthogonal matrix of Schur vectors.

    Optionally Z may be postmultiplied into an input orthogonal
    matrix Q so that this routine can give the Schur factorization
    of a matrix A which has been reduced to the Hessenberg form H
    by the orthogonal matrix Q:  A = Q*H*Q**T = (QZ)*T*(QZ)**T.

    Arguments
    =========

    JOB   (input) CHARACTER*1
          = 'E':  compute eigenvalues only;
          = 'S':  compute eigenvalues and the Schur form T.

    COMPZ (input) CHARACTER*1
          = 'N':  no Schur vectors are computed;
          = 'I':  Z is initialized to the unit matrix and the matrix Z
                  of Schur vectors of H is returned;
          = 'V':  Z must contain an orthogonal matrix Q on entry, and
                  the product Q*Z is returned.

    N     (input) INTEGER
          The order of the matrix H.  N >= 0.

    ILO   (input) INTEGER
    IHI   (input) INTEGER
          It is assumed that H is already upper triangular in rows
          and columns 1:ILO-1 and IHI+1:N. ILO and IHI are normally
          set by a previous call to DGEBAL, and then passed to DGEHRD
          when the matrix output by DGEBAL is reduced to Hessenberg
          form. Otherwise ILO and IHI should be set to 1 and N
          respectively.  If N>0, then 1<=ILO<=IHI<=N.
          If N = 0, then ILO = 1 and IHI = 0.

    H     (input/output) DOUBLE PRECISION array, dimension (LDH,N)
          On entry, the upper Hessenberg matrix H.
          On exit, if INFO = 0 and JOB = 'S', then H contains the
          upper quasi-triangular matrix T from the Schur decomposition
          (the Schur form); 2-by-2 diagonal blocks (corresponding to
          complex conjugate pairs of eigenvalues) are returned in
          standard form, with H(i,i) = H(i+1,i+1) and
          H(i+1,i)*H(i,i+1)<0. If INFO = 0 and JOB = 'E', the
          contents of H are unspecified on exit.  (The output value of
          H when INFO>0 is given under the description of INFO
          below.)

          Unlike earlier versions of DHSEQR, this subroutine may
          explicitly H(i,j) = 0 for i>j and j = 1, 2, ... ILO-1
          or j = IHI+1, IHI+2, ... N.

    LDH   (input) INTEGER
          The leading dimension of the array H. LDH >= max(1,N).

    WR    (output) DOUBLE PRECISION array, dimension (N)
    WI    (output) DOUBLE PRECISION array, dimension (N)
          The real and imaginary parts, respectively, of the computed
          eigenvalues. If two eigenvalues are computed as a complex
          conjugate pair, they are stored in consecutive elements of
          WR and WI, say the i-th and (i+1)th, with WI(i) > 0 and
          WI(i+1) < 0. If JOB = 'S', the eigenvalues are stored in
          the same order as on the diagonal of the Schur form returned
          in H, with WR(i) = H(i,i) and, if H(i:i+1,i:i+1) is a 2-by-2
          diagonal block, WI(i) = sqrt(-H(i+1,i)*H(i,i+1)) and
          WI(i+1) = -WI(i).

    Z     (input/output) DOUBLE PRECISION array, dimension (LDZ,N)
          If COMPZ = 'N', Z is not referenced.
          If COMPZ = 'I', on entry Z need not be set and on exit,
          if INFO = 0, Z contains the orthogonal matrix Z of the Schur
          vectors of H.  If COMPZ = 'V', on entry Z must contain an
          N-by-N matrix Q, which is assumed to be equal to the unit
          matrix except for the submatrix Z(ILO:IHI,ILO:IHI). On exit,
          if INFO = 0, Z contains Q*Z.
          Normally Q is the orthogonal matrix generated by DORGHR
          after the call to DGEHRD which formed the Hessenberg matrix
          H. (The output value of Z when INFO>0 is given under
          the description of INFO below.)

    LDZ   (input) INTEGER
          The leading dimension of the array Z.  if COMPZ = 'I' or
          COMPZ = 'V', then LDZ>=MAX(1,N).  Otherwize, LDZ>=1.

    WORK  (workspace/output) DOUBLE PRECISION array, dimension (LWORK)
          On exit, if INFO = 0, WORK(1) returns an estimate of
          the optimal value for LWORK.

    LWORK (input) INTEGER
          The dimension of the array WORK.  LWORK >= max(1,N)
          is sufficient and delivers very good and sometimes
          optimal performance.  However, LWORK as large as 11*N
          may be required for optimal performance.  A workspace
          query is recommended to determine the optimal workspace
          size.

          If LWORK = -1, then DHSEQR does a workspace query.
          In this case, DHSEQR checks the input parameters and
          estimates the optimal workspace size for the given
          values of N, ILO and IHI.  The estimate is returned
          in WORK(1).  No error message related to LWORK is
          issued by XERBLA.  Neither H nor Z are accessed.


    INFO  (output) INTEGER
          = 0: successful exit
          < 0: if INFO = -i, the i-th argument had an illegal
               value
          > 0: if INFO = i, DHSEQR failed to compute all of
               the eigenvalues.  Elements 1:ilo-1 and i+1:n of WR
               and WI contain those eigenvalues which have been
               successfully computed.  (Failures are rare.)

               If INFO > 0 and JOB = 'E', then on exit, the
               remaining unconverged eigenvalues are the eigen-
               values of the upper Hessenberg matrix rows and
               columns ILO through INFO of the final, output
               value of H.

               If INFO > 0 and JOB   = 'S', then on exit

          (*)  (initial value of H)*U  = U*(final value of H)

               where U is an orthogonal matrix.  The final
               value of H is upper Hessenberg and quasi-triangular
               in rows and columns INFO+1 through IHI.

               If INFO > 0 and COMPZ = 'V', then on exit

                 (final value of Z)  =  (initial value of Z)*U

               where U is the orthogonal matrix in (*) (regard-
               less of the value of JOB.)

               If INFO > 0 and COMPZ = 'I', then on exit
                     (final value of Z)  = U
               where U is the orthogonal matrix in (*) (regard-
               less of the value of JOB.)

               If INFO > 0 and COMPZ = 'N', then Z is not
               accessed.
            "));
        end dhseqr;

        function dlange "Norm of a matrix"
          extends Modelica.Icons.Function;
          input Real A[:, :] "Real matrix A";
          input String norm = "1" "specifies the norm, i.e., 1, I, F, M";
          output Real anorm "norm of A";
        protected
          Integer m = size(A, 1);
          Integer n = size(A, 2);
          Integer lda = max(1, size(A, 1));
          Real work[lda];

          external "FORTRAN 77" anorm = dlange(norm, m, n, A, lda, work) annotation(
            Library = {"lapack"});
          annotation(
            Documentation(info = "Lapack documentation
    Purpose
    =======

    DLANGE  returns the value of the one norm,  or the Frobenius norm, or
    the  infinity norm,  or the  element of  largest absolute value  of a
    real matrix A.

    Description
    ===========

    DLANGE returns the value

       DLANGE = ( max(abs(A(i,j))), NORM = 'M' or 'm'
                (
                ( norm1(A),         NORM = '1', 'O' or 'o'
                (
                ( normI(A),         NORM = 'I' or 'i'
                (
                ( normF(A),         NORM = 'F', 'f', 'E' or 'e'

    where  norm1  denotes the  one norm of a matrix (maximum column sum),
    normI  denotes the  infinity norm  of a matrix  (maximum row sum) and
    normF  denotes the  Frobenius norm of a matrix (square root of sum of
    squares).  Note that  max(abs(A(i,j)))  is not a consistent matrix norm.

    Arguments
    =========

    NORM    (input) CHARACTER*1
            Specifies the value to be returned in DLANGE as described
            above.

    M       (input) INTEGER
            The number of rows of the matrix A.  M >= 0.  When M = 0,
            DLANGE is set to zero.

    N       (input) INTEGER
            The number of columns of the matrix A.  N >= 0.  When N = 0,
            DLANGE is set to zero.

    A       (input) DOUBLE PRECISION array, dimension (LDA,N)
            The m by n matrix A.

    LDA     (input) INTEGER
            The leading dimension of the array A.  LDA >= max(M,1).

    WORK    (workspace) DOUBLE PRECISION array, dimension (MAX(1,LWORK)),
            where LWORK >= M when NORM = 'I'; otherwise, WORK is not
            referenced.
            "));
        end dlange;

        function dgecon "Estimates the reciprocal of the condition number of a general real matrix A"
          extends Modelica.Icons.Function;
          input Real LU_of_A[:, :] "LU factorization of a real matrix A";
          input Boolean inf = false "Is true if infinity norm is used and false for 1-norm";
          input Real anorm "norm of A";
          output Real rcond "Reciprocal condition number of A";
          output Integer info;
        protected
          Integer n = size(LU_of_A, 2);
          Integer lda = max(1, size(LU_of_A, 1));
          Real work[4 * size(LU_of_A, 2)];
          Integer iwork[size(LU_of_A, 2)];
          String norm = if inf then "I" else "1";

          external "FORTRAN 77" dgecon(norm, n, LU_of_A, lda, anorm, rcond, work, iwork, info) annotation(
            Library = {"lapack"});
          annotation(
            Documentation(info = "Lapack documentation
    Purpose
    =======

    DGECON estimates the reciprocal of the condition number of a general
    real matrix A, in either the 1-norm or the infinity-norm, using
    the LU factorization computed by DGETRF.

    An estimate is obtained for norm(inv(A)), and the reciprocal of the
    condition number is computed as
       RCOND = 1 / ( norm(A) * norm(inv(A)) ).

    Arguments
    =========

    NORM    (input) CHARACTER*1
            Specifies whether the 1-norm condition number or the
            infinity-norm condition number is required:
            = '1' or 'O':  1-norm;
            = 'I':         Infinity-norm.

    N       (input) INTEGER
            The order of the matrix A.  N >= 0.

    A       (input) DOUBLE PRECISION array, dimension (LDA,N)
            The factors L and U from the factorization A = P*L*U
            as computed by DGETRF.

    LDA     (input) INTEGER
            The leading dimension of the array A.  LDA >= max(1,N).

    ANORM   (input) DOUBLE PRECISION
            If NORM = '1' or 'O', the 1-norm of the original matrix A.
            If NORM = 'I', the infinity-norm of the original matrix A.

    RCOND   (output) DOUBLE PRECISION
            The reciprocal of the condition number of the matrix A,
            computed as RCOND = 1/(norm(A) * norm(inv(A))).

    WORK    (workspace) DOUBLE PRECISION array, dimension (4*N)

    IWORK   (workspace) INTEGER array, dimension (N)

    INFO    (output) INTEGER
            = 0:  successful exit
            < 0:  if INFO = -i, the i-th argument had an illegal value
            "));
        end dgecon;

        function dgehrd "reduces a real general matrix A to upper Hessenberg form H by an orthogonal similarity transformation:  Q' * A * Q = H"
          extends Modelica.Icons.Function;
          input Real A[:, size(A, 2)];
          input Integer ilo = 1 "lowest index where the original matrix had been Hessenbergform";
          input Integer ihi = size(A, 1) "highest index where the original matrix had been Hessenbergform";
          output Real Aout[size(A, 1), size(A, 2)] = A "contains the Hessenberg form in the upper triangle and the first subdiagonal and below the first subdiagonal it contains the elementary reflectors which represents (with array tau) as a product the orthogonal matrix Q";
          output Real tau[max(size(A, 1), 1) - 1] "scalar factors of the elementary reflectors";
          output Integer info;
        protected
          Integer n = size(A, 1);
          Integer lda = max(1, n);
          Integer lwork = max(1, 3 * n);
          Real work[max(1, 3 * size(A, 1))];

          external "FORTRAN 77" dgehrd(n, ilo, ihi, Aout, lda, tau, work, lwork, info) annotation(
            Library = {"lapack"});
          annotation(
            Documentation(info = "Lapack documentation
    Purpose
    =======

    DGEHRD reduces a real general matrix A to upper Hessenberg form H by
    an orthogonal similarity transformation:  Q' * A * Q = H .

    Arguments
    =========

    N       (input) INTEGER
            The order of the matrix A.  N >= 0.

    ILO     (input) INTEGER
    IHI     (input) INTEGER
            It is assumed that A is already upper triangular in rows
            and columns 1:ILO-1 and IHI+1:N. ILO and IHI are normally
            set by a previous call to DGEBAL; otherwise they should be
            set to 1 and N respectively. See Further Details.
            1 <= ILO <= IHI <= N, if N > 0; ILO=1 and IHI=0, if N=0.

    A       (input/output) DOUBLE PRECISION array, dimension (LDA,N)
            On entry, the N-by-N general matrix to be reduced.
            On exit, the upper triangle and the first subdiagonal of A
            are overwritten with the upper Hessenberg matrix H, and the
            elements below the first subdiagonal, with the array TAU,
            represent the orthogonal matrix Q as a product of elementary
            reflectors. See Further Details.

    LDA     (input) INTEGER
            The leading dimension of the array A.  LDA >= max(1,N).

    TAU     (output) DOUBLE PRECISION array, dimension (N-1)
            The scalar factors of the elementary reflectors (see Further
            Details). Elements 1:ILO-1 and IHI:N-1 of TAU are set to
            zero.

    WORK    (workspace/output) DOUBLE PRECISION array, dimension (LWORK)
            On exit, if INFO = 0, WORK(1) returns the optimal LWORK.

    LWORK   (input) INTEGER
            The length of the array WORK.  LWORK >= max(1,N).
            For optimum performance LWORK >= N*NB, where NB is the
            optimal blocksize.

            If LWORK = -1, then a workspace query is assumed; the routine
            only calculates the optimal size of the WORK array, returns
            this value as the first entry of the WORK array, and no error
            message related to LWORK is issued by XERBLA.

    INFO    (output) INTEGER
            = 0:  successful exit
            < 0:  if INFO = -i, the i-th argument had an illegal value.

    Further Details
    ===============

    The matrix Q is represented as a product of (ihi-ilo) elementary
    reflectors

       Q = H(ilo) H(ilo+1) . . . H(ihi-1).

    Each H(i) has the form

       H(i) = I - tau * v * v'

    where tau is a real scalar, and v is a real vector with
    v(1:i) = 0, v(i+1) = 1 and v(ihi+1:n) = 0; v(i+2:ihi) is stored on
    exit in A(i+2:ihi,i), and tau in TAU(i).

    The contents of A are illustrated by the following example, with
    n = 7, ilo = 2 and ihi = 6:

    on entry,                        on exit,

    ( a   a   a   a   a   a   a )    (  a   a   h   h   h   h   a )
    (     a   a   a   a   a   a )    (      a   h   h   h   h   a )
    (     a   a   a   a   a   a )    (      h   h   h   h   h   h )
    (     a   a   a   a   a   a )    (      v2  h   h   h   h   h )
    (     a   a   a   a   a   a )    (      v2  v3  h   h   h   h )
    (     a   a   a   a   a   a )    (      v2  v3  v4  h   h   h )
    (                         a )    (                          a )

    where a denotes an element of the original matrix A, h denotes a
    modified element of the upper Hessenberg matrix H, and vi denotes an
    element of the vector defining H(i).
            "));
        end dgehrd;

        function dgeqrf "computes a QR factorization without pivoting"
          extends Modelica.Icons.Function;
          input Real A[:, :] "Square or rectangular matrix";
          output Real Aout[size(A, 1), size(A, 2)] = A "the upper triangle of the array contains the upper trapezoidal matrix R; the elements below the diagonal, together with the array TAU, represent the orthogonal matrix Q as a product of elementary reflectors";
          output Real tau[min(size(A, 1), size(A, 2))] "scalar factors of the elementary reflectors";
          output Integer info;
          output Real work[3 * max(1, size(A, 2))];
        protected
          Integer m = size(A, 1);
          Integer n = size(A, 2);
          Integer lda = max(1, m);
          Integer lwork = 3 * max(1, n);

          external "FORTRAN 77" dgeqrf(m, n, Aout, lda, tau, work, lwork, info) annotation(
            Library = {"lapack"});
          annotation(
            Documentation(info = "Lapack documentation
    Purpose
    =======

    DGEQRF computes a QR factorization of a real M-by-N matrix A:
    A = Q * R.

    Arguments
    =========

    M       (input) INTEGER
            The number of rows of the matrix A.  M >= 0.

    N       (input) INTEGER
            The number of columns of the matrix A.  N >= 0.

    A       (input/output) DOUBLE PRECISION array, dimension (LDA,N)
            On entry, the M-by-N matrix A.
            On exit, the elements on and above the diagonal of the array
            contain the min(M,N)-by-N upper trapezoidal matrix R (R is
            upper triangular if m >= n); the elements below the diagonal,
            with the array TAU, represent the orthogonal matrix Q as a
            product of min(m,n) elementary reflectors (see Further
            Details).

    LDA     (input) INTEGER
            The leading dimension of the array A.  LDA >= max(1,M).

    TAU     (output) DOUBLE PRECISION array, dimension (min(M,N))
            The scalar factors of the elementary reflectors (see Further
            Details).

    WORK    (workspace/output) DOUBLE PRECISION array, dimension (MAX(1,LWORK))
            On exit, if INFO = 0, WORK(1) returns the optimal LWORK.

    LWORK   (input) INTEGER
            The dimension of the array WORK.  LWORK >= max(1,N).
            For optimum performance LWORK >= N*NB, where NB is
            the optimal blocksize.

            If LWORK = -1, then a workspace query is assumed; the routine
            only calculates the optimal size of the WORK array, returns
            this value as the first entry of the WORK array, and no error
            message related to LWORK is issued by XERBLA.

    INFO    (output) INTEGER
            = 0:  successful exit
            < 0:  if INFO = -i, the i-th argument had an illegal value

    Further Details
    ===============

    The matrix Q is represented as a product of elementary reflectors

       Q = H(1) H(2) . . . H(k), where k = min(m,n).

    Each H(i) has the form

       H(i) = I - tau * v * v'

    where tau is a real scalar, and v is a real vector with
    v(1:i-1) = 0 and v(i) = 1; v(i+1:m) is stored on exit in A(i+1:m,i),
    and tau in TAU(i).
            "));
        end dgeqrf;

        function dgeevx "Compute the eigenvalues and the (real) left and right eigenvectors of matrix A, using lapack routine dgeevx"
          extends Modelica.Icons.Function;
          input Real A[:, size(A, 1)];
          output Real alphaReal[size(A, 1)] "Real part of alpha (eigenvalue=(alphaReal+i*alphaImag))";
          output Real alphaImag[size(A, 1)] "Imaginary part of alpha (eigenvalue=(alphaReal+i*alphaImag))";
          output Real lEigenVectors[size(A, 1), size(A, 1)] "left eigenvectors of matrix A";
          output Real rEigenVectors[size(A, 1), size(A, 1)] "right eigenvectors of matrix A";
          output Real AS[size(A, 1), size(A, 2)] = A "AS iss the real Schur form of the balanced version of the input matrix A";
          output Integer info;
        protected
          Integer n = size(A, 1);
          Integer ilo;
          Integer ihi;
          Real scale[size(A, 1)];
          Real abnrm;
          Real rconde[size(A, 1)];
          Real rcondv[size(A, 1)];
          Integer lwork = n * (n + 6);
          Real work[size(A, 1) * (size(A, 1) + 6)];
          Integer iwork[1];

          external "FORTRAN 77" dgeevx("B", "V", "V", "E", n, AS, n, alphaReal, alphaImag, lEigenVectors, n, rEigenVectors, n, ilo, ihi, scale, abnrm, rconde, rcondv, work, lwork, iwork, info) annotation(
            Library = {"lapack"});
          annotation(
            Documentation(info = "Lapack documentation
    Purpose
    =======

    DGEEVX computes for an N-by-N real nonsymmetric matrix A, the
    eigenvalues and, optionally, the left and/or right eigenvectors.

    Optionally also, it computes a balancing transformation to improve
    the conditioning of the eigenvalues and eigenvectors (ILO, IHI,
    SCALE, and ABNRM), reciprocal condition numbers for the eigenvalues
    (RCONDE), and reciprocal condition numbers for the right
    eigenvectors (RCONDV).

    The right eigenvector v(j) of A satisfies
                     A * v(j) = lambda(j) * v(j)
    where lambda(j) is its eigenvalue.
    The left eigenvector u(j) of A satisfies
                  u(j)**H * A = lambda(j) * u(j)**H
    where u(j)**H denotes the conjugate transpose of u(j).

    The computed eigenvectors are normalized to have Euclidean norm
    equal to 1 and largest component real.

    Balancing a matrix means permuting the rows and columns to make it
    more nearly upper triangular, and applying a diagonal similarity
    transformation D * A * D**(-1), where D is a diagonal matrix, to
    make its rows and columns closer in norm and the condition numbers
    of its eigenvalues and eigenvectors smaller.  The computed
    reciprocal condition numbers correspond to the balanced matrix.
    Permuting rows and columns will not change the condition numbers
    (in exact arithmetic) but diagonal scaling will.  For further
    explanation of balancing, see section 4.10.2 of the LAPACK
    Users' Guide.

    Arguments
    =========

    BALANC  (input) CHARACTER*1
            Indicates how the input matrix should be diagonally scaled
            and/or permuted to improve the conditioning of its
            eigenvalues.
            = 'N': Do not diagonally scale or permute;
            = 'P': Perform permutations to make the matrix more nearly
                   upper triangular. Do not diagonally scale;
            = 'S': Diagonally scale the matrix, i.e. replace A by
                   D*A*D**(-1), where D is a diagonal matrix chosen
                   to make the rows and columns of A more equal in
                   norm. Do not permute;
            = 'B': Both diagonally scale and permute A.

            Computed reciprocal condition numbers will be for the matrix
            after balancing and/or permuting. Permuting does not change
            condition numbers (in exact arithmetic), but balancing does.

    JOBVL   (input) CHARACTER*1
            = 'N': left eigenvectors of A are not computed;
            = 'V': left eigenvectors of A are computed.
            If SENSE = 'E' or 'B', JOBVL must = 'V'.

    JOBVR   (input) CHARACTER*1
            = 'N': right eigenvectors of A are not computed;
            = 'V': right eigenvectors of A are computed.
            If SENSE = 'E' or 'B', JOBVR must = 'V'.

    SENSE   (input) CHARACTER*1
            Determines which reciprocal condition numbers are computed.
            = 'N': None are computed;
            = 'E': Computed for eigenvalues only;
            = 'V': Computed for right eigenvectors only;
            = 'B': Computed for eigenvalues and right eigenvectors.

            If SENSE = 'E' or 'B', both left and right eigenvectors
            must also be computed (JOBVL = 'V' and JOBVR = 'V').

    N       (input) INTEGER
            The order of the matrix A. N >= 0.

    A       (input/output) DOUBLE PRECISION array, dimension (LDA,N)
            On entry, the N-by-N matrix A.
            On exit, A has been overwritten.  If JOBVL = 'V' or
            JOBVR = 'V', A contains the real Schur form of the balanced
            version of the input matrix A.

    LDA     (input) INTEGER
            The leading dimension of the array A.  LDA >= max(1,N).

    WR      (output) DOUBLE PRECISION array, dimension (N)
    WI      (output) DOUBLE PRECISION array, dimension (N)
            WR and WI contain the real and imaginary parts,
            respectively, of the computed eigenvalues.  Complex
            conjugate pairs of eigenvalues will appear consecutively
            with the eigenvalue having the positive imaginary part
            first.

    VL      (output) DOUBLE PRECISION array, dimension (LDVL,N)
            If JOBVL = 'V', the left eigenvectors u(j) are stored one
            after another in the columns of VL, in the same order
            as their eigenvalues.
            If JOBVL = 'N', VL is not referenced.
            If the j-th eigenvalue is real, then u(j) = VL(:,j),
            the j-th column of VL.
            If the j-th and (j+1)-st eigenvalues form a complex
            conjugate pair, then u(j) = VL(:,j) + i*VL(:,j+1) and
            u(j+1) = VL(:,j) - i*VL(:,j+1).

    LDVL    (input) INTEGER
            The leading dimension of the array VL.  LDVL >= 1; if
            JOBVL = 'V', LDVL >= N.

    VR      (output) DOUBLE PRECISION array, dimension (LDVR,N)
            If JOBVR = 'V', the right eigenvectors v(j) are stored one
            after another in the columns of VR, in the same order
            as their eigenvalues.
            If JOBVR = 'N', VR is not referenced.
            If the j-th eigenvalue is real, then v(j) = VR(:,j),
            the j-th column of VR.
            If the j-th and (j+1)-st eigenvalues form a complex
            conjugate pair, then v(j) = VR(:,j) + i*VR(:,j+1) and
            v(j+1) = VR(:,j) - i*VR(:,j+1).

    LDVR    (input) INTEGER
            The leading dimension of the array VR.  LDVR >= 1, and if
            JOBVR = 'V', LDVR >= N.

    ILO     (output) INTEGER
    IHI     (output) INTEGER
            ILO and IHI are integer values determined when A was
            balanced.  The balanced A(i,j) = 0 if I > J and
            J = 1,...,ILO-1 or I = IHI+1,...,N.

    SCALE   (output) DOUBLE PRECISION array, dimension (N)
            Details of the permutations and scaling factors applied
            when balancing A.  If P(j) is the index of the row and column
            interchanged with row and column j, and D(j) is the scaling
            factor applied to row and column j, then
            SCALE(J) = P(J),    for J = 1,...,ILO-1
                     = D(J),    for J = ILO,...,IHI
                     = P(J)     for J = IHI+1,...,N.
            The order in which the interchanges are made is N to IHI+1,
            then 1 to ILO-1.

    ABNRM   (output) DOUBLE PRECISION
            The one-norm of the balanced matrix (the maximum
            of the sum of absolute values of elements of any column).

    RCONDE  (output) DOUBLE PRECISION array, dimension (N)
            RCONDE(j) is the reciprocal condition number of the j-th
            eigenvalue.

    RCONDV  (output) DOUBLE PRECISION array, dimension (N)
            RCONDV(j) is the reciprocal condition number of the j-th
            right eigenvector.

    WORK    (workspace/output) DOUBLE PRECISION array, dimension (MAX(1,LWORK))
            On exit, if INFO = 0, WORK(1) returns the optimal LWORK.

    LWORK   (input) INTEGER
            The dimension of the array WORK.   If SENSE = 'N' or 'E',
            LWORK >= max(1,2*N), and if JOBVL = 'V' or JOBVR = 'V',
            LWORK >= 3*N.  If SENSE = 'V' or 'B', LWORK >= N*(N+6).
            For good performance, LWORK must generally be larger.

            If LWORK = -1, then a workspace query is assumed; the routine
            only calculates the optimal size of the WORK array, returns
            this value as the first entry of the WORK array, and no error
            message related to LWORK is issued by XERBLA.

    IWORK   (workspace) INTEGER array, dimension (2*N-2)
            If SENSE = 'N' or 'E', not referenced.

    INFO    (output) INTEGER
            = 0:  successful exit
            < 0:  if INFO = -i, the i-th argument had an illegal value.
            > 0:  if INFO = i, the QR algorithm failed to compute all the
                  eigenvalues, and no eigenvectors or condition numbers
                  have been computed; elements 1:ILO-1 and i+1:N of WR
                  and WI contain eigenvalues which have converged.
            "));
        end dgeevx;

        function dgesdd "Determine singular value decomposition"
          extends Modelica.Icons.Function;
          input Real A[:, :];
          output Real sigma[min(size(A, 1), size(A, 2))];
          output Real U[size(A, 1), size(A, 1)] = zeros(size(A, 1), size(A, 1));
          output Real VT[size(A, 2), size(A, 2)] = zeros(size(A, 2), size(A, 2));
          output Integer info;
        protected
          Real Awork[size(A, 1), size(A, 2)] = A;
          Integer lda = max(1, size(A, 1));
          Integer ldu = max(1, size(A, 1));
          Integer ldvt = max(1, size(A, 2));
          Integer lwork = max(1, 3 * (3 * min(size(A, 1), size(A, 2)) * min(size(A, 1), size(A, 2)) + max(max(size(A, 1), size(A, 2)), 4 * min(size(A, 1), size(A, 2)) * min(size(A, 1), size(A, 2)) + 4 * min(size(A, 1), size(A, 2)))));
          Integer iwork = max(1, 8 * min(size(A, 1), size(A, 2)));
          Real work[max(1, 3 * (3 * min(size(A, 1), size(A, 2)) * min(size(A, 1), size(A, 2)) + max(max(size(A, 1), size(A, 2)), 4 * min(size(A, 1), size(A, 2)) * min(size(A, 1), size(A, 2)) + 4 * min(size(A, 1), size(A, 2)))))];

          external "FORTRAN 77" dgesdd("A", size(A, 1), size(A, 2), Awork, lda, sigma, U, ldu, VT, ldvt, work, lwork, iwork, info) annotation(
            Library = "lapack");
          annotation(
            Documentation(info = "Lapack documentation
    Purpose
    =======

    DGESDD computes the singular value decomposition (SVD) of a real
    M-by-N matrix A, optionally computing the left and right singular
    vectors.  If singular vectors are desired, it uses a
    divide-and-conquer algorithm.

    The SVD is written

         A = U * SIGMA * transpose(V)

    where SIGMA is an M-by-N matrix which is zero except for its
    min(m,n) diagonal elements, U is an M-by-M orthogonal matrix, and
    V is an N-by-N orthogonal matrix.  The diagonal elements of SIGMA
    are the singular values of A; they are real and non-negative, and
    are returned in descending order.  The first min(m,n) columns of
    U and V are the left and right singular vectors of A.

    Note that the routine returns VT = V**T, not V.

    The divide and conquer algorithm makes very mild assumptions about
    floating point arithmetic. It will work on machines with a guard
    digit in add/subtract, or on those binary machines without guard
    digits which subtract like the Cray X-MP, Cray Y-MP, Cray C-90, or
    Cray-2. It could conceivably fail on hexadecimal or decimal machines
    without guard digits, but we know of none.

    Arguments
    =========

    JOBZ    (input) CHARACTER*1
            Specifies options for computing all or part of the matrix U:
            = 'A':  all M columns of U and all N rows of V**T are
                    returned in the arrays U and VT;
            = 'S':  the first min(M,N) columns of U and the first
                    min(M,N) rows of V**T are returned in the arrays U
                    and VT;
            = 'O':  If M >= N, the first N columns of U are overwritten
                    on the array A and all rows of V**T are returned in
                    the array VT;
                    otherwise, all columns of U are returned in the
                    array U and the first M rows of V**T are overwritten
                    in the array A;
            = 'N':  no columns of U or rows of V**T are computed.

    M       (input) INTEGER
            The number of rows of the input matrix A.  M >= 0.

    N       (input) INTEGER
            The number of columns of the input matrix A.  N >= 0.

    A       (input/output) DOUBLE PRECISION array, dimension (LDA,N)
            On entry, the M-by-N matrix A.
            On exit,
            if JOBZ = 'O',  A is overwritten with the first N columns
                            of U (the left singular vectors, stored
                            columnwise) if M >= N;
                            A is overwritten with the first M rows
                            of V**T (the right singular vectors, stored
                            rowwise) otherwise.
            if JOBZ .ne. 'O', the contents of A are destroyed.

    LDA     (input) INTEGER
            The leading dimension of the array A.  LDA >= max(1,M).

    S       (output) DOUBLE PRECISION array, dimension (min(M,N))
            The singular values of A, sorted so that S(i) >= S(i+1).

    U       (output) DOUBLE PRECISION array, dimension (LDU,UCOL)
            UCOL = M if JOBZ = 'A' or JOBZ = 'O' and M < N;
            UCOL = min(M,N) if JOBZ = 'S'.
            If JOBZ = 'A' or JOBZ = 'O' and M < N, U contains the M-by-M
            orthogonal matrix U;
            if JOBZ = 'S', U contains the first min(M,N) columns of U
            (the left singular vectors, stored columnwise);
            if JOBZ = 'O' and M >= N, or JOBZ = 'N', U is not referenced.

    LDU     (input) INTEGER
            The leading dimension of the array U.  LDU >= 1; if
            JOBZ = 'S' or 'A' or JOBZ = 'O' and M < N, LDU >= M.

    VT      (output) DOUBLE PRECISION array, dimension (LDVT,N)
            If JOBZ = 'A' or JOBZ = 'O' and M >= N, VT contains the
            N-by-N orthogonal matrix V**T;
            if JOBZ = 'S', VT contains the first min(M,N) rows of
            V**T (the right singular vectors, stored rowwise);
            if JOBZ = 'O' and M < N, or JOBZ = 'N', VT is not referenced.

    LDVT    (input) INTEGER
            The leading dimension of the array VT.  LDVT >= 1; if
            JOBZ = 'A' or JOBZ = 'O' and M >= N, LDVT >= N;
            if JOBZ = 'S', LDVT >= min(M,N).

    WORK    (workspace/output) DOUBLE PRECISION array, dimension (MAX(1,LWORK))
            On exit, if INFO = 0, WORK(1) returns the optimal LWORK;

    LWORK   (input) INTEGER
            The dimension of the array WORK. LWORK >= 1.
            If JOBZ = 'N',
              LWORK >= 3*min(M,N) + max(max(M,N),7*min(M,N)).
            If JOBZ = 'O',
              LWORK >= 3*min(M,N) +
                       max(max(M,N),5*min(M,N)*min(M,N)+4*min(M,N)).
            If JOBZ = 'S' or 'A'
              LWORK >= 3*min(M,N) +
                       max(max(M,N),4*min(M,N)*min(M,N)+4*min(M,N)).
            For good performance, LWORK should generally be larger.
            If LWORK = -1 but other input arguments are legal, WORK(1)
            returns the optimal LWORK.

    IWORK   (workspace) INTEGER array, dimension (8*min(M,N))

    INFO    (output) INTEGER
            = 0:  successful exit.
            < 0:  if INFO = -i, the i-th argument had an illegal value.
            > 0:  DBDSDC did not converge, updating process failed.

    Further Details
    ===============

    Based on contributions by
       Ming Gu and Huan Ren, Computer Science Division, University of
       California at Berkeley, USA
            "));
        end dgesdd;

        function dggev "Compute generalized eigenvalues, as well as the left and right eigenvectors for a (A,B) system"
          extends Modelica.Icons.Function;
          input Real A[:, size(A, 1)];
          input Real B[size(A, 1), size(A, 1)];
          input Integer nA = size(A, 1) "The actual dimensions of matrices A and B (the computation is performed for A[1:nA,1:nA], B[1:nA,1:nA])";
          output Real alphaReal[size(A, 1)] "Real part of alpha (eigenvalue=(alphaReal+i*alphaImag)/beta)";
          output Real alphaImag[size(A, 1)] "Imaginary part of alpha";
          output Real beta[size(A, 1)] "Denominator of eigenvalue";
          output Real lEigenVectors[size(A, 1), size(A, 1)] "left eigenvectors of matrix A";
          output Real rEigenVectors[size(A, 1), size(A, 1)] "right eigenvectors of matrix A";
          output Integer info;
        protected
          Integer n = size(A, 1);
          Integer lwork = max(1, 8 * n);
          Real Awork[size(A, 1), size(A, 1)] = A;
          Real Bwork[size(A, 1), size(A, 1)] = B;
          Real work[max(1, 8 * size(A, 1))];
          Integer lda = max(1, n);

          external "FORTRAN 77" dggev("V", "V", nA, Awork, lda, Bwork, lda, alphaReal, alphaImag, beta, lEigenVectors, lda, rEigenVectors, lda, work, lwork, info) annotation(
            Library = {"lapack"});
          annotation(
            Documentation(info = "Lapack documentation
    Purpose
    =======

    DGGEV computes for a pair of N-by-N real nonsymmetric matrices (A,B)
    the generalized eigenvalues, and optionally, the left and/or right
    generalized eigenvectors.

    A generalized eigenvalue for a pair of matrices (A,B) is a scalar
    lambda or a ratio alpha/beta = lambda, such that A - lambda*B is
    singular. It is usually represented as the pair (alpha,beta), as
    there is a reasonable interpretation for beta=0, and even for both
    being zero.

    The right eigenvector v(j) corresponding to the eigenvalue lambda(j)
    of (A,B) satisfies

                     A * v(j) = lambda(j) * B * v(j).

    The left eigenvector u(j) corresponding to the eigenvalue lambda(j)
    of (A,B) satisfies

                     u(j)**H * A  = lambda(j) * u(j)**H * B .

    where u(j)**H is the conjugate-transpose of u(j).


    Arguments
    =========

    JOBVL   (input) CHARACTER*1
            = 'N':  do not compute the left generalized eigenvectors;
            = 'V':  compute the left generalized eigenvectors.

    JOBVR   (input) CHARACTER*1
            = 'N':  do not compute the right generalized eigenvectors;
            = 'V':  compute the right generalized eigenvectors.

    N       (input) INTEGER
            The order of the matrices A, B, VL, and VR.  N >= 0.

    A       (input/output) DOUBLE PRECISION array, dimension (LDA, N)
            On entry, the matrix A in the pair (A,B).
            On exit, A has been overwritten.

    LDA     (input) INTEGER
            The leading dimension of A.  LDA >= max(1,N).

    B       (input/output) DOUBLE PRECISION array, dimension (LDB, N)
            On entry, the matrix B in the pair (A,B).
            On exit, B has been overwritten.

    LDB     (input) INTEGER
            The leading dimension of B.  LDB >= max(1,N).

    ALPHAR  (output) DOUBLE PRECISION array, dimension (N)
    ALPHAI  (output) DOUBLE PRECISION array, dimension (N)
    BETA    (output) DOUBLE PRECISION array, dimension (N)
            On exit, (ALPHAR(j) + ALPHAI(j)*i)/BETA(j), j=1,...,N, will
            be the generalized eigenvalues.  If ALPHAI(j) is zero, then
            the j-th eigenvalue is real; if positive, then the j-th and
            (j+1)-st eigenvalues are a complex conjugate pair, with
            ALPHAI(j+1) negative.

            Note: the quotients ALPHAR(j)/BETA(j) and ALPHAI(j)/BETA(j)
            may easily over- or underflow, and BETA(j) may even be zero.
            Thus, the user should avoid naively computing the ratio
            alpha/beta.  However, ALPHAR and ALPHAI will be always less
            than and usually comparable with norm(A) in magnitude, and
            BETA always less than and usually comparable with norm(B).

    VL      (output) DOUBLE PRECISION array, dimension (LDVL,N)
            If JOBVL = 'V', the left eigenvectors u(j) are stored one
            after another in the columns of VL, in the same order as
            their eigenvalues. If the j-th eigenvalue is real, then
            u(j) = VL(:,j), the j-th column of VL. If the j-th and
            (j+1)-th eigenvalues form a complex conjugate pair, then
            u(j) = VL(:,j)+i*VL(:,j+1) and u(j+1) = VL(:,j)-i*VL(:,j+1).
            Each eigenvector is scaled so the largest component has
            abs(real part)+abs(imag. part)=1.
            Not referenced if JOBVL = 'N'.

    LDVL    (input) INTEGER
            The leading dimension of the matrix VL. LDVL >= 1, and
            if JOBVL = 'V', LDVL >= N.

    VR      (output) DOUBLE PRECISION array, dimension (LDVR,N)
            If JOBVR = 'V', the right eigenvectors v(j) are stored one
            after another in the columns of VR, in the same order as
            their eigenvalues. If the j-th eigenvalue is real, then
            v(j) = VR(:,j), the j-th column of VR. If the j-th and
            (j+1)-th eigenvalues form a complex conjugate pair, then
            v(j) = VR(:,j)+i*VR(:,j+1) and v(j+1) = VR(:,j)-i*VR(:,j+1).
            Each eigenvector is scaled so the largest component has
            abs(real part)+abs(imag. part)=1.
            Not referenced if JOBVR = 'N'.

    LDVR    (input) INTEGER
            The leading dimension of the matrix VR. LDVR >= 1, and
            if JOBVR = 'V', LDVR >= N.

    WORK    (workspace/output) DOUBLE PRECISION array, dimension (MAX(1,LWORK))
            On exit, if INFO = 0, WORK(1) returns the optimal LWORK.

    LWORK   (input) INTEGER
            The dimension of the array WORK.  LWORK >= max(1,8*N).
            For good performance, LWORK must generally be larger.

            If LWORK = -1, then a workspace query is assumed; the routine
            only calculates the optimal size of the WORK array, returns
            this value as the first entry of the WORK array, and no error
            message related to LWORK is issued by XERBLA.

    INFO    (output) INTEGER
            = 0:  successful exit
            < 0:  if INFO = -i, the i-th argument had an illegal value.
            = 1,...,N:
                  The QZ iteration failed.  No eigenvectors have been
                  calculated, but ALPHAR(j), ALPHAI(j), and BETA(j)
                  should be correct for j=INFO+1,...,N.
            > N:  =N+1: other than QZ iteration failed in DHGEQZ.
                  =N+2: error return from DTGEVC.
            "));
        end dggev;

        function dggevx "Compute generalized eigenvalues for a (A,B) system, using lapack routine dggevx"
          extends Modelica.Icons.Function;
          input Real A[:, size(A, 1)];
          input Real B[size(A, 1), size(A, 1)];
          output Real alphaReal[size(A, 1)] "Real part of alpha (eigenvalue=(alphaReal+i*alphaImag)/beta)";
          output Real alphaImag[size(A, 1)] "Imaginary part of alpha";
          output Real beta[size(A, 1)] "Denominator of eigenvalue";
          output Real lEigenVectors[size(A, 1), size(A, 1)] "left eigenvectors of matrix A";
          output Real rEigenVectors[size(A, 1), size(A, 1)] "right eigenvectors of matrix A";
          output Integer info;
        protected
          Integer n = size(A, 1);
          Integer lda = max(1, size(A, 1));
          Integer ilo;
          Integer ihi;
          Real lscale[size(A, 1)];
          Real rscale[size(A, 1)];
          Real abnrm;
          Real bbnrm;
          Real rconde[size(A, 1)];
          Real rcondv[size(A, 1)];
          Integer lwork = 2 * n * n + 12 * n + 16;
          Real work[2 * size(A, 1) * size(A, 1) + 12 * size(A, 1) + 16];
          Integer iwork[size(A, 1) + 6];
          Integer bwork[size(A, 1)];

          external "FORTRAN 77" dggevx("B", "V", "V", "B", n, A, lda, B, lda, alphaReal, alphaImag, beta, lEigenVectors, lda, rEigenVectors, lda, ilo, ihi, lscale, rscale, abnrm, bbnrm, rconde, rcondv, work, lwork, iwork, bwork, info) annotation(
            Library = {"lapack"});
          annotation(
            Documentation(info = "Lapack documentation
    Purpose
    =======

    DGGEVX computes for a pair of N-by-N real nonsymmetric matrices (A,B)
    the generalized eigenvalues, and optionally, the left and/or right
    generalized eigenvectors.

    Optionally also, it computes a balancing transformation to improve
    the conditioning of the eigenvalues and eigenvectors (ILO, IHI,
    LSCALE, RSCALE, ABNRM, and BBNRM), reciprocal condition numbers for
    the eigenvalues (RCONDE), and reciprocal condition numbers for the
    right eigenvectors (RCONDV).

    A generalized eigenvalue for a pair of matrices (A,B) is a scalar
    lambda or a ratio alpha/beta = lambda, such that A - lambda*B is
    singular. It is usually represented as the pair (alpha,beta), as
    there is a reasonable interpretation for beta=0, and even for both
    being zero.

    The right eigenvector v(j) corresponding to the eigenvalue lambda(j)
    of (A,B) satisfies

                     A * v(j) = lambda(j) * B * v(j) .

    The left eigenvector u(j) corresponding to the eigenvalue lambda(j)
    of (A,B) satisfies

                     u(j)**H * A  = lambda(j) * u(j)**H * B.

    where u(j)**H is the conjugate-transpose of u(j).


    Arguments
    =========

    BALANC  (input) CHARACTER*1
            Specifies the balance option to be performed.
            = 'N':  do not diagonally scale or permute;
            = 'P':  permute only;
            = 'S':  scale only;
            = 'B':  both permute and scale.
            Computed reciprocal condition numbers will be for the
            matrices after permuting and/or balancing. Permuting does
            not change condition numbers (in exact arithmetic), but
            balancing does.

    JOBVL   (input) CHARACTER*1
            = 'N':  do not compute the left generalized eigenvectors;
            = 'V':  compute the left generalized eigenvectors.

    JOBVR   (input) CHARACTER*1
            = 'N':  do not compute the right generalized eigenvectors;
            = 'V':  compute the right generalized eigenvectors.

    SENSE   (input) CHARACTER*1
            Determines which reciprocal condition numbers are computed.
            = 'N': none are computed;
            = 'E': computed for eigenvalues only;
            = 'V': computed for eigenvectors only;
            = 'B': computed for eigenvalues and eigenvectors.

    N       (input) INTEGER
            The order of the matrices A, B, VL, and VR.  N >= 0.

    A       (input/output) DOUBLE PRECISION array, dimension (LDA, N)
            On entry, the matrix A in the pair (A,B).
            On exit, A has been overwritten. If JOBVL='V' or JOBVR='V'
            or both, then A contains the first part of the real Schur
            form of the \"balanced\" versions of the input A and B.

    LDA     (input) INTEGER
            The leading dimension of A.  LDA >= max(1,N).

    B       (input/output) DOUBLE PRECISION array, dimension (LDB, N)
            On entry, the matrix B in the pair (A,B).
            On exit, B has been overwritten. If JOBVL='V' or JOBVR='V'
            or both, then B contains the second part of the real Schur
            form of the \"balanced\" versions of the input A and B.

    LDB     (input) INTEGER
            The leading dimension of B.  LDB >= max(1,N).

    ALPHAR  (output) DOUBLE PRECISION array, dimension (N)
    ALPHAI  (output) DOUBLE PRECISION array, dimension (N)
    BETA    (output) DOUBLE PRECISION array, dimension (N)
            On exit, (ALPHAR(j) + ALPHAI(j)*i)/BETA(j), j=1,...,N, will
            be the generalized eigenvalues.  If ALPHAI(j) is zero, then
            the j-th eigenvalue is real; if positive, then the j-th and
            (j+1)-st eigenvalues are a complex conjugate pair, with
            ALPHAI(j+1) negative.

            Note: the quotients ALPHAR(j)/BETA(j) and ALPHAI(j)/BETA(j)
            may easily over- or underflow, and BETA(j) may even be zero.
            Thus, the user should avoid naively computing the ratio
            ALPHA/BETA. However, ALPHAR and ALPHAI will be always less
            than and usually comparable with norm(A) in magnitude, and
            BETA always less than and usually comparable with norm(B).

    VL      (output) DOUBLE PRECISION array, dimension (LDVL,N)
            If JOBVL = 'V', the left eigenvectors u(j) are stored one
            after another in the columns of VL, in the same order as
            their eigenvalues. If the j-th eigenvalue is real, then
            u(j) = VL(:,j), the j-th column of VL. If the j-th and
            (j+1)-th eigenvalues form a complex conjugate pair, then
            u(j) = VL(:,j)+i*VL(:,j+1) and u(j+1) = VL(:,j)-i*VL(:,j+1).
            Each eigenvector will be scaled so the largest component have
            abs(real part) + abs(imag. part) = 1.
            Not referenced if JOBVL = 'N'.

    LDVL    (input) INTEGER
            The leading dimension of the matrix VL. LDVL >= 1, and
            if JOBVL = 'V', LDVL >= N.

    VR      (output) DOUBLE PRECISION array, dimension (LDVR,N)
            If JOBVR = 'V', the right eigenvectors v(j) are stored one
            after another in the columns of VR, in the same order as
            their eigenvalues. If the j-th eigenvalue is real, then
            v(j) = VR(:,j), the j-th column of VR. If the j-th and
            (j+1)-th eigenvalues form a complex conjugate pair, then
            v(j) = VR(:,j)+i*VR(:,j+1) and v(j+1) = VR(:,j)-i*VR(:,j+1).
            Each eigenvector will be scaled so the largest component have
            abs(real part) + abs(imag. part) = 1.
            Not referenced if JOBVR = 'N'.

    LDVR    (input) INTEGER
            The leading dimension of the matrix VR. LDVR >= 1, and
            if JOBVR = 'V', LDVR >= N.

    ILO     (output) INTEGER
    IHI     (output) INTEGER
            ILO and IHI are integer values such that on exit
            A(i,j) = 0 and B(i,j) = 0 if i > j and
            j = 1,...,ILO-1 or i = IHI+1,...,N.
            If BALANC = 'N' or 'S', ILO = 1 and IHI = N.

    LSCALE  (output) DOUBLE PRECISION array, dimension (N)
            Details of the permutations and scaling factors applied
            to the left side of A and B.  If PL(j) is the index of the
            row interchanged with row j, and DL(j) is the scaling
            factor applied to row j, then
              LSCALE(j) = PL(j)  for j = 1,...,ILO-1
                        = DL(j)  for j = ILO,...,IHI
                        = PL(j)  for j = IHI+1,...,N.
            The order in which the interchanges are made is N to IHI+1,
            then 1 to ILO-1.

    RSCALE  (output) DOUBLE PRECISION array, dimension (N)
            Details of the permutations and scaling factors applied
            to the right side of A and B.  If PR(j) is the index of the
            column interchanged with column j, and DR(j) is the scaling
            factor applied to column j, then
              RSCALE(j) = PR(j)  for j = 1,...,ILO-1
                        = DR(j)  for j = ILO,...,IHI
                        = PR(j)  for j = IHI+1,...,N
            The order in which the interchanges are made is N to IHI+1,
            then 1 to ILO-1.

    ABNRM   (output) DOUBLE PRECISION
            The one-norm of the balanced matrix A.

    BBNRM   (output) DOUBLE PRECISION
            The one-norm of the balanced matrix B.

    RCONDE  (output) DOUBLE PRECISION array, dimension (N)
            If SENSE = 'E' or 'B', the reciprocal condition numbers of
            the eigenvalues, stored in consecutive elements of the array.
            For a complex conjugate pair of eigenvalues two consecutive
            elements of RCONDE are set to the same value. Thus RCONDE(j),
            RCONDV(j), and the j-th columns of VL and VR all correspond
            to the j-th eigenpair.
            If SENSE = 'N or 'V', RCONDE is not referenced.

    RCONDV  (output) DOUBLE PRECISION array, dimension (N)
            If SENSE = 'V' or 'B', the estimated reciprocal condition
            numbers of the eigenvectors, stored in consecutive elements
            of the array. For a complex eigenvector two consecutive
            elements of RCONDV are set to the same value. If the
            eigenvalues cannot be reordered to compute RCONDV(j),
            RCONDV(j) is set to 0; this can only occur when the true
            value would be very small anyway.
            If SENSE = 'N' or 'E', RCONDV is not referenced.

    WORK    (workspace/output) DOUBLE PRECISION array, dimension (MAX(1,LWORK))
            On exit, if INFO = 0, WORK(1) returns the optimal LWORK.

    LWORK   (input) INTEGER
            The dimension of the array WORK. LWORK >= max(1,2*N).
            If BALANC = 'S' or 'B', or JOBVL = 'V', or JOBVR = 'V',
            LWORK >= max(1,6*N).
            If SENSE = 'E' or 'B', LWORK >= max(1,10*N).
            If SENSE = 'V' or 'B', LWORK >= 2*N*N+8*N+16.

            If LWORK = -1, then a workspace query is assumed; the routine
            only calculates the optimal size of the WORK array, returns
            this value as the first entry of the WORK array, and no error
            message related to LWORK is issued by XERBLA.

    IWORK   (workspace) INTEGER array, dimension (N+6)
            If SENSE = 'E', IWORK is not referenced.

    BWORK   (workspace) LOGICAL array, dimension (N)
            If SENSE = 'N', BWORK is not referenced.

    INFO    (output) INTEGER
            = 0:  successful exit
            < 0:  if INFO = -i, the i-th argument had an illegal value.
            = 1,...,N:
                  The QZ iteration failed.  No eigenvectors have been
                  calculated, but ALPHAR(j), ALPHAI(j), and BETA(j)
                  should be correct for j=INFO+1,...,N.
            > N:  =N+1: other than QZ iteration failed in DHGEQZ.
                  =N+2: error return from DTGEVC.

    Further Details
    ===============

    Balancing a matrix pair (A,B) includes, first, permuting rows and
    columns to isolate eigenvalues, second, applying diagonal similarity
    transformation to the rows and columns to make the rows and columns
    as close in norm as possible. The computed reciprocal condition
    numbers correspond to the balanced matrix. Permuting rows and columns
    will not change the condition numbers (in exact arithmetic) but
    diagonal scaling will.  For further explanation of balancing, see
    section 4.11.1.2 of LAPACK Users' Guide.

    An approximate error bound on the chordal distance between the i-th
    computed generalized eigenvalue w and the corresponding exact
    eigenvalue lambda is

         chord(w, lambda) <= EPS * norm(ABNRM, BBNRM) / RCONDE(I)

    An approximate error bound for the angle between the i-th computed
    eigenvector VL(i) or VR(i) is given by

         EPS * norm(ABNRM, BBNRM) / DIF(i).

    For further explanation of the reciprocal condition numbers RCONDE
    and RCONDV, see section 4.11 of LAPACK User's Guide.
            "));
        end dggevx;

        function dhgeqz "Compute generalized eigenvalues for a (A,B) system"
          extends Modelica.Icons.Function;
          input Real A[:, size(A, 1)];
          input Real B[size(A, 1), size(A, 1)];
          output Real alphaReal[size(A, 1)] "Real part of alpha (eigenvalue=(alphaReal+i*alphaImag)/beta)";
          output Real alphaImag[size(A, 1)] "Imaginary part of alpha";
          output Real beta[size(A, 1)] "Denominator of eigenvalue";
          output Integer info;
        protected
          Integer n = size(A, 1);
          Integer ilo = 1;
          Integer ihi = n;
          Integer lwork = max(1, 3 * n);
          Real work[max(1, 3 * size(A, 1))];
          Real Awork[size(A, 1), size(A, 1)] = A;
          Real Bwork[size(A, 1), size(A, 1)] = B;
          Integer lda = max(1, n);
          Real Q[1, 1] = fill(0, 1, n);

          external "FORTRAN 77" dhgeqz("E", "N", "N", n, ilo, ihi, Awork, lda, Bwork, lda, alphaReal, alphaImag, beta, Q, 1, Q, 1, work, lwork, info) annotation(
            Library = {"lapack"});
          annotation(
            Documentation(info = "Lapack documentation
    Purpose
    =======

    DHGEQZ computes the eigenvalues of a real matrix pair (H,T),
    where H is an upper Hessenberg matrix and T is upper triangular,
    using the double-shift QZ method.
    Matrix pairs of this type are produced by the reduction to
    generalized upper Hessenberg form of a real matrix pair (A,B):

       A = Q1*H*Z1**T,  B = Q1*T*Z1**T,

    as computed by DGGHRD.

    If JOB='S', then the Hessenberg-triangular pair (H,T) is
    also reduced to generalized Schur form,

       H = Q*S*Z**T,  T = Q*P*Z**T,

    where Q and Z are orthogonal matrices, P is an upper triangular
    matrix, and S is a quasi-triangular matrix with 1-by-1 and 2-by-2
    diagonal blocks.

    The 1-by-1 blocks correspond to real eigenvalues of the matrix pair
    (H,T) and the 2-by-2 blocks correspond to complex conjugate pairs of
    eigenvalues.

    Additionally, the 2-by-2 upper triangular diagonal blocks of P
    corresponding to 2-by-2 blocks of S are reduced to positive diagonal
    form, i.e., if S(j+1,j) is non-zero, then P(j+1,j) = P(j,j+1) = 0,
    P(j,j) > 0, and P(j+1,j+1) > 0.

    Optionally, the orthogonal matrix Q from the generalized Schur
    factorization may be postmultiplied into an input matrix Q1, and the
    orthogonal matrix Z may be postmultiplied into an input matrix Z1.
    If Q1 and Z1 are the orthogonal matrices from DGGHRD that reduced
    the matrix pair (A,B) to generalized upper Hessenberg form, then the
    output matrices Q1*Q and Z1*Z are the orthogonal factors from the
    generalized Schur factorization of (A,B):

       A = (Q1*Q)*S*(Z1*Z)**T,  B = (Q1*Q)*P*(Z1*Z)**T.

    To avoid overflow, eigenvalues of the matrix pair (H,T) (equivalently,
    of (A,B)) are computed as a pair of values (alpha,beta), where alpha is
    complex and beta real.
    If beta is nonzero, lambda = alpha / beta is an eigenvalue of the
    generalized nonsymmetric eigenvalue problem (GNEP)
       A*x = lambda*B*x
    and if alpha is nonzero, mu = beta / alpha is an eigenvalue of the
    alternate form of the GNEP
       mu*A*y = B*y.
    Real eigenvalues can be read directly from the generalized Schur
    form:
      alpha = S(i,i), beta = P(i,i).

    Ref: C.B. Moler & G.W. Stewart, \"An Algorithm for Generalized Matrix
         Eigenvalue Problems\", SIAM J. Numer. Anal., 10(1973),
         pp. 241--256.

    Arguments
    =========

    JOB     (input) CHARACTER*1
            = 'E': Compute eigenvalues only;
            = 'S': Compute eigenvalues and the Schur form.

    COMPQ   (input) CHARACTER*1
            = 'N': Left Schur vectors (Q) are not computed;
            = 'I': Q is initialized to the unit matrix and the matrix Q
                   of left Schur vectors of (H,T) is returned;
            = 'V': Q must contain an orthogonal matrix Q1 on entry and
                   the product Q1*Q is returned.

    COMPZ   (input) CHARACTER*1
            = 'N': Right Schur vectors (Z) are not computed;
            = 'I': Z is initialized to the unit matrix and the matrix Z
                   of right Schur vectors of (H,T) is returned;
            = 'V': Z must contain an orthogonal matrix Z1 on entry and
                   the product Z1*Z is returned.

    N       (input) INTEGER
            The order of the matrices H, T, Q, and Z.  N >= 0.

    ILO     (input) INTEGER
    IHI     (input) INTEGER
            ILO and IHI mark the rows and columns of H which are in
            Hessenberg form.  It is assumed that A is already upper
            triangular in rows and columns 1:ILO-1 and IHI+1:N.
            If N > 0, 1 <= ILO <= IHI <= N; if N = 0, ILO=1 and IHI=0.

    H       (input/output) DOUBLE PRECISION array, dimension (LDH, N)
            On entry, the N-by-N upper Hessenberg matrix H.
            On exit, if JOB = 'S', H contains the upper quasi-triangular
            matrix S from the generalized Schur factorization;
            2-by-2 diagonal blocks (corresponding to complex conjugate
            pairs of eigenvalues) are returned in standard form, with
            H(i,i) = H(i+1,i+1) and H(i+1,i)*H(i,i+1) < 0.
            If JOB = 'E', the diagonal blocks of H match those of S, but
            the rest of H is unspecified.

    LDH     (input) INTEGER
            The leading dimension of the array H.  LDH >= max( 1, N ).

    T       (input/output) DOUBLE PRECISION array, dimension (LDT, N)
            On entry, the N-by-N upper triangular matrix T.
            On exit, if JOB = 'S', T contains the upper triangular
            matrix P from the generalized Schur factorization;
            2-by-2 diagonal blocks of P corresponding to 2-by-2 blocks of S
            are reduced to positive diagonal form, i.e., if H(j+1,j) is
            non-zero, then T(j+1,j) = T(j,j+1) = 0, T(j,j) > 0, and
            T(j+1,j+1) > 0.
            If JOB = 'E', the diagonal blocks of T match those of P, but
            the rest of T is unspecified.

    LDT     (input) INTEGER
            The leading dimension of the array T.  LDT >= max( 1, N ).

    ALPHAR  (output) DOUBLE PRECISION array, dimension (N)
            The real parts of each scalar alpha defining an eigenvalue
            of GNEP.

    ALPHAI  (output) DOUBLE PRECISION array, dimension (N)
            The imaginary parts of each scalar alpha defining an
            eigenvalue of GNEP.
            If ALPHAI(j) is zero, then the j-th eigenvalue is real; if
            positive, then the j-th and (j+1)-st eigenvalues are a
            complex conjugate pair, with ALPHAI(j+1) = -ALPHAI(j).

    BETA    (output) DOUBLE PRECISION array, dimension (N)
            The scalars beta that define the eigenvalues of GNEP.
            Together, the quantities alpha = (ALPHAR(j),ALPHAI(j)) and
            beta = BETA(j) represent the j-th eigenvalue of the matrix
            pair (A,B), in one of the forms lambda = alpha/beta or
            mu = beta/alpha.  Since either lambda or mu may overflow,
            they should not, in general, be computed.

    Q       (input/output) DOUBLE PRECISION array, dimension (LDQ, N)
            On entry, if COMPZ = 'V', the orthogonal matrix Q1 used in
            the reduction of (A,B) to generalized Hessenberg form.
            On exit, if COMPZ = 'I', the orthogonal matrix of left Schur
            vectors of (H,T), and if COMPZ = 'V', the orthogonal matrix
            of left Schur vectors of (A,B).
            Not referenced if COMPZ = 'N'.

    LDQ     (input) INTEGER
            The leading dimension of the array Q.  LDQ >= 1.
            If COMPQ='V' or 'I', then LDQ >= N.

    Z       (input/output) DOUBLE PRECISION array, dimension (LDZ, N)
            On entry, if COMPZ = 'V', the orthogonal matrix Z1 used in
            the reduction of (A,B) to generalized Hessenberg form.
            On exit, if COMPZ = 'I', the orthogonal matrix of
            right Schur vectors of (H,T), and if COMPZ = 'V', the
            orthogonal matrix of right Schur vectors of (A,B).
            Not referenced if COMPZ = 'N'.

    LDZ     (input) INTEGER
            The leading dimension of the array Z.  LDZ >= 1.
            If COMPZ='V' or 'I', then LDZ >= N.

    WORK    (workspace/output) DOUBLE PRECISION array, dimension (MAX(1,LWORK))
            On exit, if INFO >= 0, WORK(1) returns the optimal LWORK.

    LWORK   (input) INTEGER
            The dimension of the array WORK.  LWORK >= max(1,N).

            If LWORK = -1, then a workspace query is assumed; the routine
            only calculates the optimal size of the WORK array, returns
            this value as the first entry of the WORK array, and no error
            message related to LWORK is issued by XERBLA.

    INFO    (output) INTEGER
            = 0: successful exit
            < 0: if INFO = -i, the i-th argument had an illegal value
            = 1,...,N: the QZ iteration did not converge.  (H,T) is not
                       in Schur form, but ALPHAR(i), ALPHAI(i), and
                       BETA(i), i=INFO+1,...,N should be correct.
            = N+1,...,2*N: the shift calculation failed.  (H,T) is not
                       in Schur form, but ALPHAR(i), ALPHAI(i), and
                       BETA(i), i=INFO-N+1,...,N should be correct.

    Further Details
    ===============

    Iteration counters:

    JITER  -- counts iterations.
    IITER  -- counts iterations run since ILAST was last
              changed.  This is therefore reset only when a 1-by-1 or
              2-by-2 block deflates off the bottom.
            "));
        end dhgeqz;

        function dormhr "overwrites the general real M-by-N matrix C with Q * C or C * Q or Q' * C or C * Q', where Q is an orthogonal matrix as returned by dgehrd"
          extends Modelica.Icons.Function;
          input Real C[:, :];
          input Real A[:, :];
          input Real tau[if side == "L" then size(C, 2) - 1 else size(C, 1) - 1];
          input String side = "L";
          input String trans = "N";
          input Integer ilo = 1 "lowest index where the original matrix had been Hessenbergform";
          input Integer ihi = if side == "L" then size(C, 1) else size(C, 2) "highest index where the original matrix had been Hessenbergform";
          output Real Cout[size(C, 1), size(C, 2)] = C "contains the Hessenberg form in the upper triangle and the first subdiagonal and below the first subdiagonal it contains the elementary reflectors which represents (with array tau) as a product the orthogonal matrix Q";
          output Integer info;
        protected
          Integer m = size(C, 1);
          Integer n = size(C, 2);
          Integer lda = max(1, size(A, 2));
          Integer ldc = max(1, m);
          Integer lwork = max(1, 2 * size(A, 2));
          Real work[max(1, 2 * size(A, 2))];

          external "FORTRAN 77" dormhr(side, trans, m, n, ilo, ihi, A, lda, tau, Cout, ldc, work, lwork, info) annotation(
            Library = {"lapack"});
          annotation(
            Documentation(info = "Lapack documentation
    Purpose
    =======

    DORMHR overwrites the general real M-by-N matrix C with

                    SIDE = 'L'     SIDE = 'R'
    TRANS = 'N':      Q * C          C * Q
    TRANS = 'T':      Q**T * C       C * Q**T

    where Q is a real orthogonal matrix of order nq, with nq = m if
    SIDE = 'L' and nq = n if SIDE = 'R'. Q is defined as the product of
    IHI-ILO elementary reflectors, as returned by DGEHRD:

    Q = H(ilo) H(ilo+1) . . . H(ihi-1).

    Arguments
    =========

    SIDE    (input) CHARACTER*1
            = 'L': apply Q or Q**T from the Left;
            = 'R': apply Q or Q**T from the Right.

    TRANS   (input) CHARACTER*1
            = 'N':  No transpose, apply Q;
            = 'T':  Transpose, apply Q**T.

    M       (input) INTEGER
            The number of rows of the matrix C. M >= 0.

    N       (input) INTEGER
            The number of columns of the matrix C. N >= 0.

    ILO     (input) INTEGER
    IHI     (input) INTEGER
            ILO and IHI must have the same values as in the previous call
            of DGEHRD. Q is equal to the unit matrix except in the
            submatrix Q(ilo+1:ihi,ilo+1:ihi).
            If SIDE = 'L', then 1 <= ILO <= IHI <= M, if M > 0, and
            ILO = 1 and IHI = 0, if M = 0;
            if SIDE = 'R', then 1 <= ILO <= IHI <= N, if N > 0, and
            ILO = 1 and IHI = 0, if N = 0.

    A       (input) DOUBLE PRECISION array, dimension
                                 (LDA,M) if SIDE = 'L'
                                 (LDA,N) if SIDE = 'R'
            The vectors which define the elementary reflectors, as
            returned by DGEHRD.

    LDA     (input) INTEGER
            The leading dimension of the array A.
            LDA >= max(1,M) if SIDE = 'L'; LDA >= max(1,N) if SIDE = 'R'.

    TAU     (input) DOUBLE PRECISION array, dimension
                                 (M-1) if SIDE = 'L'
                                 (N-1) if SIDE = 'R'
            TAU(i) must contain the scalar factor of the elementary
            reflector H(i), as returned by DGEHRD.

    C       (input/output) DOUBLE PRECISION array, dimension (LDC,N)
            On entry, the M-by-N matrix C.
            On exit, C is overwritten by Q*C or Q**T*C or C*Q**T or C*Q.

    LDC     (input) INTEGER
            The leading dimension of the array C. LDC >= max(1,M).

    WORK    (workspace/output) DOUBLE PRECISION array, dimension (MAX(1,LWORK))
            On exit, if INFO = 0, WORK(1) returns the optimal LWORK.

    LWORK   (input) INTEGER
            The dimension of the array WORK.
            If SIDE = 'L', LWORK >= max(1,N);
            if SIDE = 'R', LWORK >= max(1,M).
            For optimum performance LWORK >= N*NB if SIDE = 'L', and
            LWORK >= M*NB if SIDE = 'R', where NB is the optimal
            blocksize.

            If LWORK = -1, then a workspace query is assumed; the routine
            only calculates the optimal size of the WORK array, returns
            this value as the first entry of the WORK array, and no error
            message related to LWORK is issued by XERBLA.

    INFO    (output) INTEGER
            = 0:  successful exit
            < 0:  if INFO = -i, the i-th argument had an illegal value
            "));
        end dormhr;

        function dormqr "overwrites the general real M-by-N matrix C with Q * C or C * Q or Q' * C or C * Q', where Q is an orthogonal matrix of a QR factorization as returned by dgeqrf"
          extends Modelica.Icons.Function;
          input Real C[:, :];
          input Real A[:, :];
          input Real tau[:];
          input String side = "L";
          input String trans = "N";
          output Real Cout[size(C, 1), size(C, 2)] = C "contains Q*C or Q**T*C or C*Q**T or C*Q";
          output Integer info;
        protected
          Integer m = size(C, 1);
          Integer n = size(C, 2);
          Integer k = if side == "L" then m else n;
          Integer lda = if side == "L" then max(1, m) else max(1, n);
          Integer ldc = max(1, m);
          Integer lwork = if side == "L" then max(1, n) else max(1, m);
          Real work[if side == "L" then max(1, size(C, 2)) else max(1, size(C, 1))];

          external "FORTRAN 77" dormqr(side, trans, m, n, k, A, lda, tau, Cout, ldc, work, lwork, info) annotation(
            Library = {"lapack"});
          annotation(
            Documentation(info = "Lapack documentation
    Purpose
    =======

    DORMQR overwrites the general real M-by-N matrix C with

                    SIDE = 'L'     SIDE = 'R'
    TRANS = 'N':      Q * C          C * Q
    TRANS = 'T':      Q**T * C       C * Q**T

    where Q is a real orthogonal matrix defined as the product of k
    elementary reflectors

          Q = H(1) H(2) . . . H(k)

    as returned by DGEQRF. Q is of order M if SIDE = 'L' and of order N
    if SIDE = 'R'.

    Arguments
    =========

    SIDE    (input) CHARACTER*1
            = 'L': apply Q or Q**T from the Left;
            = 'R': apply Q or Q**T from the Right.

    TRANS   (input) CHARACTER*1
            = 'N':  No transpose, apply Q;
            = 'T':  Transpose, apply Q**T.

    M       (input) INTEGER
            The number of rows of the matrix C. M >= 0.

    N       (input) INTEGER
            The number of columns of the matrix C. N >= 0.

    K       (input) INTEGER
            The number of elementary reflectors whose product defines
            the matrix Q.
            If SIDE = 'L', M >= K >= 0;
            if SIDE = 'R', N >= K >= 0.

    A       (input) DOUBLE PRECISION array, dimension (LDA,K)
            The i-th column must contain the vector which defines the
            elementary reflector H(i), for i = 1,2,...,k, as returned by
            DGEQRF in the first k columns of its array argument A.
            A is modified by the routine but restored on exit.

    LDA     (input) INTEGER
            The leading dimension of the array A.
            If SIDE = 'L', LDA >= max(1,M);
            if SIDE = 'R', LDA >= max(1,N).

    TAU     (input) DOUBLE PRECISION array, dimension (K)
            TAU(i) must contain the scalar factor of the elementary
            reflector H(i), as returned by DGEQRF.

    C       (input/output) DOUBLE PRECISION array, dimension (LDC,N)
            On entry, the M-by-N matrix C.
            On exit, C is overwritten by Q*C or Q**T*C or C*Q**T or C*Q.

    LDC     (input) INTEGER
            The leading dimension of the array C. LDC >= max(1,M).

    WORK    (workspace/output) DOUBLE PRECISION array, dimension (MAX(1,LWORK))
            On exit, if INFO = 0, WORK(1) returns the optimal LWORK.

    LWORK   (input) INTEGER
            The dimension of the array WORK.
            If SIDE = 'L', LWORK >= max(1,N);
            if SIDE = 'R', LWORK >= max(1,M).
            For optimum performance LWORK >= N*NB if SIDE = 'L', and
            LWORK >= M*NB if SIDE = 'R', where NB is the optimal
            blocksize.

            If LWORK = -1, then a workspace query is assumed; the routine
            only calculates the optimal size of the WORK array, returns
            this value as the first entry of the WORK array, and no error
            message related to LWORK is issued by XERBLA.

    INFO    (output) INTEGER
            = 0:  successful exit
            < 0:  if INFO = -i, the i-th argument had an illegal value
            "));
        end dormqr;

        function dtrevc "Compute the right and/or left eigenvectors of a real upper quasi-triangular matrix T"
          extends Modelica.Icons.Function;
          input Real T[:, size(T, 1)] "Upper quasi triangular matrix";
          input String side = "R" "Specify which eigenvectors";
          input String howmny = "B" "Specify how many eigenvectors";
          input Real Q[size(T, 1), size(T, 1)] "Orthogonal matrix Q of Schur vectors returned by DHSEQR";
          output Real lEigenVectors[size(T, 1), size(T, 1)] = Q "left eigenvectors of matrix T";
          output Real rEigenVectors[size(T, 1), size(T, 1)] = Q "right eigenvectors of matrix T";
          output Integer info;
        protected
          Integer n = size(T, 1);
          Boolean select[size(T, 1)];
          Integer ldt = max(1, n);
          Integer ldvl = max(1, n);
          Integer ldvr = max(1, n);
          Real work[3 * size(T, 1)];

          external "FORTRAN 77" dtrevc(side, howmny, select, n, T, ldt, lEigenVectors, ldvl, rEigenVectors, ldvr, n, n, work, info) annotation(
            Library = {"lapack"});
          annotation(
            Documentation(info = "Lapack documentation
    Purpose
    =======

    DTREVC computes some or all of the right and/or left eigenvectors of
    a real upper quasi-triangular matrix T.
    Matrices of this type are produced by the Schur factorization of
    a real general matrix:  A = Q*T*Q**T, as computed by DHSEQR.

    The right eigenvector x and the left eigenvector y of T corresponding
    to an eigenvalue w are defined by:

       T*x = w*x,     (y**H)*T = w*(y**H)

    where y**H denotes the conjugate transpose of y.
    The eigenvalues are not input to this routine, but are read directly
    from the diagonal blocks of T.

    This routine returns the matrices X and/or Y of right and left
    eigenvectors of T, or the products Q*X and/or Q*Y, where Q is an
    input matrix.  If Q is the orthogonal factor that reduces a matrix
    A to Schur form T, then Q*X and Q*Y are the matrices of right and
    left eigenvectors of A.

    Arguments
    =========

    SIDE    (input) CHARACTER*1
            = 'R':  compute right eigenvectors only;
            = 'L':  compute left eigenvectors only;
            = 'B':  compute both right and left eigenvectors.

    HOWMNY  (input) CHARACTER*1
            = 'A':  compute all right and/or left eigenvectors;
            = 'B':  compute all right and/or left eigenvectors,
                    backtransformed by the matrices in VR and/or VL;
            = 'S':  compute selected right and/or left eigenvectors,
                    as indicated by the logical array SELECT.

    SELECT  (input/output) LOGICAL array, dimension (N)
            If HOWMNY = 'S', SELECT specifies the eigenvectors to be
            computed.
            If w(j) is a real eigenvalue, the corresponding real
            eigenvector is computed if SELECT(j) is .TRUE..
            If w(j) and w(j+1) are the real and imaginary parts of a
            complex eigenvalue, the corresponding complex eigenvector is
            computed if either SELECT(j) or SELECT(j+1) is .TRUE., and
            on exit SELECT(j) is set to .TRUE. and SELECT(j+1) is set to
            .FALSE..
            Not referenced if HOWMNY = 'A' or 'B'.

    N       (input) INTEGER
            The order of the matrix T. N >= 0.

    T       (input) DOUBLE PRECISION array, dimension (LDT,N)
            The upper quasi-triangular matrix T in Schur canonical form.

    LDT     (input) INTEGER
            The leading dimension of the array T. LDT >= max(1,N).

    VL      (input/output) DOUBLE PRECISION array, dimension (LDVL,MM)
            On entry, if SIDE = 'L' or 'B' and HOWMNY = 'B', VL must
            contain an N-by-N matrix Q (usually the orthogonal matrix Q
            of Schur vectors returned by DHSEQR).
            On exit, if SIDE = 'L' or 'B', VL contains:
            if HOWMNY = 'A', the matrix Y of left eigenvectors of T;
            if HOWMNY = 'B', the matrix Q*Y;
            if HOWMNY = 'S', the left eigenvectors of T specified by
                             SELECT, stored consecutively in the columns
                             of VL, in the same order as their
                             eigenvalues.
            A complex eigenvector corresponding to a complex eigenvalue
            is stored in two consecutive columns, the first holding the
            real part, and the second the imaginary part.
            Not referenced if SIDE = 'R'.

    LDVL    (input) INTEGER
            The leading dimension of the array VL.  LDVL >= 1, and if
            SIDE = 'L' or 'B', LDVL >= N.

    VR      (input/output) DOUBLE PRECISION array, dimension (LDVR,MM)
            On entry, if SIDE = 'R' or 'B' and HOWMNY = 'B', VR must
            contain an N-by-N matrix Q (usually the orthogonal matrix Q
            of Schur vectors returned by DHSEQR).
            On exit, if SIDE = 'R' or 'B', VR contains:
            if HOWMNY = 'A', the matrix X of right eigenvectors of T;
            if HOWMNY = 'B', the matrix Q*X;
            if HOWMNY = 'S', the right eigenvectors of T specified by
                             SELECT, stored consecutively in the columns
                             of VR, in the same order as their
                             eigenvalues.
            A complex eigenvector corresponding to a complex eigenvalue
            is stored in two consecutive columns, the first holding the
            real part and the second the imaginary part.
            Not referenced if SIDE = 'L'.

    LDVR    (input) INTEGER
            The leading dimension of the array VR.  LDVR >= 1, and if
            SIDE = 'R' or 'B', LDVR >= N.

    MM      (input) INTEGER
            The number of columns in the arrays VL and/or VR. MM >= M.

    M       (output) INTEGER
            The number of columns in the arrays VL and/or VR actually
            used to store the eigenvectors.
            If HOWMNY = 'A' or 'B', M is set to N.
            Each selected real eigenvector occupies one column and each
            selected complex eigenvector occupies two columns.

    WORK    (workspace) DOUBLE PRECISION array, dimension (3*N)

    INFO    (output) INTEGER
            = 0:  successful exit
            < 0:  if INFO = -i, the i-th argument had an illegal value

    Further Details
    ===============

    The algorithm used in this program is basically backward (forward)
    substitution, with scaling to make the the code robust against
    possible overflow.

    Each eigenvector is normalized so that the element of largest
    magnitude has magnitude 1; here the magnitude of a complex number
    (x,y) is taken to be |x| + |y|.
            "));
        end dtrevc;

        function dpotrf "Computes the Cholesky factorization of a real symmetric positive definite matrix A"
          extends Modelica.Icons.Function;
          input Real A[:, size(A, 1)] "Real symmetric positive definite matrix A";
          input Boolean upper = true "True if the upper triangle of A is provided";
          output Real Acholesky[size(A, 1), size(A, 1)] = A "Cholesky factor";
          output Integer info;
        protected
          String uplo = if upper then "U" else "L";
          Integer n = size(A, 1);
          Integer lda = max(1, n);

          external "FORTRAN 77" dpotrf(uplo, n, Acholesky, lda, info) annotation(
            Library = "lapack");
          annotation(
            Documentation(info = "Lapack documentation
    Purpose
    =======

    DPOTRF computes the Cholesky factorization of a real symmetric
    positive definite matrix A.

    The factorization has the form
       A = U**T * U,  if UPLO = 'U', or
       A = L  * L**T,  if UPLO = 'L',
    where U is an upper triangular matrix and L is lower triangular.

    This is the block version of the algorithm, calling Level 3 BLAS.

    Arguments
    =========

    UPLO    (input) CHARACTER*1
            = 'U':  Upper triangle of A is stored;
            = 'L':  Lower triangle of A is stored.

    N       (input) INTEGER
            The order of the matrix A.  N >= 0.

    A       (input/output) DOUBLE PRECISION array, dimension (LDA,N)
            On entry, the symmetric matrix A.  If UPLO = 'U', the leading
            N-by-N upper triangular part of A contains the upper
            triangular part of the matrix A, and the strictly lower
            triangular part of A is not referenced.  If UPLO = 'L', the
            leading N-by-N lower triangular part of A contains the lower
            triangular part of the matrix A, and the strictly upper
            triangular part of A is not referenced.

            On exit, if INFO = 0, the factor U or L from the Cholesky
            factorization A = U**T*U or A = L*L**T.

    LDA     (input) INTEGER
            The leading dimension of the array A.  LDA >= max(1,N).

    INFO    (output) INTEGER
            = 0:  successful exit
            < 0:  if INFO = -i, the i-th argument had an illegal value
            > 0:  if INFO = i, the leading minor of order i is not
                  positive definite, and the factorization could not be
                  completed.
            "));
        end dpotrf;

        function dtrsm "Solve one of the matrix equations op( A )*X = alpha*B, or X*op( A ) = alpha*B, where A is triangular matrix. BLAS routine"
          extends Modelica.Icons.Function;
          input Real A[:, :] "Input matrix A";
          input Real B[:, :] "Input matrix B";
          input Real alpha = 1 "Factor alpha";
          input Boolean right = true "True if A is right multiplication";
          input Boolean upper = true "True if A is upper triangular";
          input Boolean trans = false "True if op(A) means transposed(A)";
          input Boolean unitTriangular = false "True if A is unit triangular, i.e., all diagonal elements of A are equal to 1";
          output Real X[size(B, 1), size(B, 2)] = B "Matrix Bout=alpha*op( A )*B, or B := alpha*B*op( A )";
        protected
          String side = if right then "R" else "L";
          String uplo = if upper then "U" else "L";
          String transA = if trans then "T" else "N";
          String diag = if unitTriangular then "U" else "N";
          Integer m = size(B, 1) "Number of rows of B";
          Integer n = size(B, 2) "Number of columns of B";
          Integer lda = if right then max(1, n) else max(1, m) "First dimension of A";
          Integer ldb = max(1, m) "First dimension of B";

          external "FORTRAN 77" dtrsm(side, uplo, transA, diag, m, n, alpha, A, lda, X, ldb) annotation(
            Library = {"lapack"});
          annotation(
            Documentation(info = "Lapack documentation
    Purpose
    =======

    DTRSM solves one of the matrix equations

       op( A )*X = alpha*B,   or   X*op( A ) = alpha*B,

    where alpha is a scalar, X and B are m by n matrices, A is a unit, or
    non-unit,  upper or lower triangular matrix  and  op( A )  is one  of

       op( A ) = A   or   op( A ) = A'.

    The matrix X is overwritten on B.

    Arguments
    ==========

    SIDE   - CHARACTER*1.
             On entry, SIDE specifies whether op( A ) appears on the left
             or right of X as follows:

                SIDE = 'L' or 'l'   op( A )*X = alpha*B.

                SIDE = 'R' or 'r'   X*op( A ) = alpha*B.

             Unchanged on exit.

    UPLO   - CHARACTER*1.
             On entry, UPLO specifies whether the matrix A is an upper or
             lower triangular matrix as follows:

                UPLO = 'U' or 'u'   A is an upper triangular matrix.

                UPLO = 'L' or 'l'   A is a lower triangular matrix.

             Unchanged on exit.

    TRANSA - CHARACTER*1.
             On entry, TRANSA specifies the form of op( A ) to be used in
             the matrix multiplication as follows:

                TRANSA = 'N' or 'n'   op( A ) = A.

                TRANSA = 'T' or 't'   op( A ) = A'.

                TRANSA = 'C' or 'c'   op( A ) = A'.

             Unchanged on exit.

    DIAG   - CHARACTER*1.
             On entry, DIAG specifies whether or not A is unit triangular
             as follows:

                DIAG = 'U' or 'u'   A is assumed to be unit triangular.

                DIAG = 'N' or 'n'   A is not assumed to be unit
                                    triangular.

             Unchanged on exit.

    M      - INTEGER.
             On entry, M specifies the number of rows of B. M must be at
             least zero.
             Unchanged on exit.

    N      - INTEGER.
             On entry, N specifies the number of columns of B.  N must be
             at least zero.
             Unchanged on exit.

    ALPHA  - DOUBLE PRECISION.
             On entry,  ALPHA specifies the scalar  alpha. When  alpha is
             zero then  A is not referenced and  B need not be set before
             entry.
             Unchanged on exit.

    A      - DOUBLE PRECISION array of DIMENSION ( LDA, k ), where k is m
             when  SIDE = 'L' or 'l'  and is  n  when  SIDE = 'R' or 'r'.
             Before entry  with  UPLO = 'U' or 'u',  the  leading  k by k
             upper triangular part of the array  A must contain the upper
             triangular matrix  and the strictly lower triangular part of
             A is not referenced.
             Before entry  with  UPLO = 'L' or 'l',  the  leading  k by k
             lower triangular part of the array  A must contain the lower
             triangular matrix  and the strictly upper triangular part of
             A is not referenced.
             Note that when  DIAG = 'U' or 'u',  the diagonal elements of
             A  are not referenced either,  but are assumed to be  unity.
             Unchanged on exit.

    LDA    - INTEGER.
             On entry, LDA specifies the first dimension of A as declared
             in the calling (sub) program.  When  SIDE = 'L' or 'l'  then
             LDA  must be at least  max( 1, m ),  when  SIDE = 'R' or 'r'
             then LDA must be at least max( 1, n ).
             Unchanged on exit.

    B      - DOUBLE PRECISION array of DIMENSION ( LDB, n ).
             Before entry,  the leading  m by n part of the array  B must
             contain  the  right-hand  side  matrix  B,  and  on exit  is
             overwritten by the solution matrix  X.

    LDB    - INTEGER.
             On entry, LDB specifies the first dimension of B as declared
             in  the  calling  (sub)  program.   LDB  must  be  at  least
             max( 1, m ).
             Unchanged on exit.

    Level 3 Blas routine.
            "));
        end dtrsm;

        function dorghr "Generates a real orthogonal matrix Q which is defined as the product of IHI-ILO elementary reflectors of order N, as returned by DGEHRD"
          extends Modelica.Icons.Function;
          input Real A[:, size(A, 1)] "Square matrix with the elementary reflectors";
          input Integer ilo = 1 "lowest index where the original matrix had been Hessenbergform - ilo must have the same value as in the previous call of DGEHRD";
          input Integer ihi = size(A, 1) "highest index where the original matrix had been Hessenbergform - ihi must have the same value as in the previous call of DGEHRD";
          input Real tau[max(0, size(A, 1) - 1)] "scalar factors of the elementary reflectors";
          output Real Aout[size(A, 1), size(A, 2)] = A "Orthogonal matrix as a result of elementary reflectors";
          output Integer info;
        protected
          Integer n = size(A, 1);
          Integer lda = max(1, n);
          Integer lwork = max(1, 3 * n);
          Real work[max(1, 3 * size(A, 1))];

          external "FORTRAN 77" dorghr(n, ilo, ihi, Aout, lda, tau, work, lwork, info) annotation(
            Library = {"lapack"});
          annotation(
            Documentation(info = "Lapack documentation
    Purpose
    =======

    DORGHR generates a real orthogonal matrix Q which is defined as the
    product of IHI-ILO elementary reflectors of order N, as returned by
    DGEHRD:

    Q = H(ilo) H(ilo+1) . . . H(ihi-1).

    Arguments
    =========

    N       (input) INTEGER
            The order of the matrix Q. N >= 0.

    ILO     (input) INTEGER
    IHI     (input) INTEGER
            ILO and IHI must have the same values as in the previous call
            of DGEHRD. Q is equal to the unit matrix except in the
            submatrix Q(ilo+1:ihi,ilo+1:ihi).
            1 <= ILO <= IHI <= N, if N > 0; ILO=1 and IHI=0, if N=0.

    A       (input/output) DOUBLE PRECISION array, dimension (LDA,N)
            On entry, the vectors which define the elementary reflectors,
            as returned by DGEHRD.
            On exit, the N-by-N orthogonal matrix Q.

    LDA     (input) INTEGER
            The leading dimension of the array A. LDA >= max(1,N).

    TAU     (input) DOUBLE PRECISION array, dimension (N-1)
            TAU(i) must contain the scalar factor of the elementary
            reflector H(i), as returned by DGEHRD.

    WORK    (workspace/output) DOUBLE PRECISION array, dimension (MAX(1,LWORK))
            On exit, if INFO = 0, WORK(1) returns the optimal LWORK.

    LWORK   (input) INTEGER
            The dimension of the array WORK. LWORK >= IHI-ILO.
            For optimum performance LWORK >= (IHI-ILO)*NB, where NB is
            the optimal blocksize.

            If LWORK = -1, then a workspace query is assumed; the routine
            only calculates the optimal size of the WORK array, returns
            this value as the first entry of the WORK array, and no error
            message related to LWORK is issued by XERBLA.

    INFO    (output) INTEGER
            = 0:  successful exit
            < 0:  if INFO = -i, the i-th argument had an illegal value
            "));
        end dorghr;
        annotation(
          Documentation(info = "<html>
<p>
This package contains external Modelica functions as interface to the
LAPACK library
(<a href=\"http://www.netlib.org/lapack\">http://www.netlib.org/lapack</a>)
that provides FORTRAN subroutines to solve linear algebra
tasks. Usually, these functions are not directly called, but only via
the much more convenient interface of
<a href=\"modelica://Modelica.Math.Matrices\">Modelica.Math.Matrices</a>.
The documentation of the LAPACK functions is a copy of the original
FORTRAN code. The details of LAPACK are described in:
</p>

<dl>
<dt>Anderson E., Bai Z., Bischof C., Blackford S., Demmel J., Dongarra J.,
    Du Croz J., Greenbaum A., Hammarling S., McKenney A., and Sorensen D.:</dt>
<dd> <a href=\"http://www.netlib.org/lapack/lug/lapack_lug.html\">Lapack Users' Guide</a>.
     Third Edition, SIAM, 1999.</dd>
</dl>

<p>
See also <a href=\"http://en.wikipedia.org/wiki/Lapack\">http://en.wikipedia.org/wiki/Lapack</a>.
</p>

<p>
This package contains a direct interface to the LAPACK subroutines
</p>

</html>"));
      end LAPACK;

      package Utilities "Utility functions that should not be directly utilized by the user"
        extends Modelica.Icons.UtilitiesPackage;

        function continuousRiccatiIterative "Newton's method with exact line search for iterative solving continuous algebraic Riccati equation"
          extends Modelica.Icons.Function;
          import Modelica.Math.Matrices;
          input Real A[:, size(A, 1)] "Matrix A of Riccati equation X*A + A'*X -X*G*X +Q = 0";
          input Real B[size(A, 1), :] "Matrix B in G = B*inv(R)*B'";
          input Real R[size(B, 2), size(B, 2)] = identity(size(B, 2)) "Matrix R in G = B*inv(R)*B'";
          input Real Q[size(A, 1), size(A, 2)] = identity(size(A, 1)) "Matrix Q of Riccati equation X*A + A'*X -X*G*X +Q = 0";
          input Real X0[size(A, 1), size(A, 2)] = identity(size(A, 1)) "Initial approximate solution for X*A + A'*X -X*G*X +Q = 0";
          input Integer maxSteps = 10 "Maximal number of iteration steps";
          input Real eps = Matrices.frobeniusNorm(A) * 1e-9 "Tolerance for stop criterion";
          output Real X[size(X0, 1), size(X0, 2)] "Solution X of Riccati equation X*A + A'*X -X*G*X +Q = 0";
          output Real r "Norm of X*A + A'*X - X*G*X + Q, zero for exact solution";
        protected
          Integer n = size(A, 1);
          Real G[size(A, 1), size(A, 2)] = B * Matrices.solve2(R, transpose(B));
          Real Xk[size(A, 1), size(A, 2)];
          Real Ak[size(A, 1), size(A, 2)];
          Real Rk[size(A, 1), size(A, 2)];
          Real Nk[size(A, 1), size(A, 2)];
          Real Vk[size(A, 1), size(A, 2)];
          Real tk;
          Integer k;
          Boolean stop;
        algorithm
          if n > 1 then
            k := 0;
            stop := false;
            Xk := X0;
            while not stop loop
              k := k + 1;
              Ak := A - G * Xk;
              Rk := transpose(A) * Xk + Xk * A + Q - Xk * G * Xk;
              Nk := Matrices.continuousLyapunov(Ak, -Rk);
              Vk := Nk * G * Nk;
              tk := Matrices.Utilities.findLocal_tk(Rk, Vk);
              stop := eps > Matrices.frobeniusNorm(tk * Nk) / Matrices.frobeniusNorm(Xk) or k >= maxSteps;
              Xk := Xk + tk * Nk;
            end while;
            X := Xk;
            r := Matrices.frobeniusNorm(X * A + transpose(A) * X - X * G * X + Q);
          elseif n == 1 then
            X := matrix((A[1, 1] - sqrt(A[1, 1] * A[1, 1] + G[1, 1] * Q[1, 1])) / G[1, 1]);
            if X[1, 1] * G[1, 1] < A[1, 1] then
              X := matrix((A[1, 1] + sqrt(A[1, 1] * A[1, 1] + G[1, 1] * Q[1, 1])) / G[1, 1]);
            end if;
            r := 0;
          else
            X := fill(0, 0, 0);
            r := 0;
          end if;
// exact calculation
          annotation(
            Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
           X = Matrices.Utilities.<b>continuousRiccatiIterative</b>(A, B, R, Q, X0);
      (X, r) = Matrices.Utilities.<b>continuousRiccatiIterative</b>(A, B, R, Q, X0, maxSteps, eps);
</pre></blockquote>
<h4>Description</h4>
<p>
This function provides a Newton-like method for solving continuous algebraic Riccati equations (care). It utilizes Exact Line Search to improve the sometimes erratic
convergence of Newton's method. Exact line search in this case means means, that at each iteration <code>i</code> a Newton step <code><b>delta</b>_i</code>
</p>
<blockquote><pre>
  <b>X</b>_i+1 = <b>X</b>_i + <b>delta</b>_i
</pre></blockquote>
<p>
is taken in the direction to minimize the Frobenius norm of the residual
</p>
<blockquote><pre>
    r = || <b>X</b>_i+1*<b>A</b> +<b>A</b>'*<b>X</b>_i+1 - <b>X</b>_i+1*<b>G</b>*<b>X</b>_i+1 + <b>Q</b> ||.
</pre></blockquote>
<p>
with
</p>
<blockquote><pre>
        -1
  G = <b>B</b>*<b>R</b> *<b>B</b>'
</pre></blockquote>
<p>

The inputs \"maxSteps\" and \"eps\" specify the termination of the iteration. The iteration is terminated if either
maxSteps iteration steps have been performed or the relative change <b>delta</b>_i/<b>X</b>_i became smaller than eps.
<p>

With an appropriate initial value <b>X</b>0 a sufficiently accurate solution might be reach within a few iteration steps. Although a Lyapunov equation
of order <code>n</code> (n is the order of the Riccati equation) is to be solved at each iteration step, the algorithm might be faster
than a direct method like <a href=\"modelica://Modelica.Math.Matrices.continuousRiccati\">Matrices.continuousRiccati</a>, since direct methods have to solve the 2*n-order Hamiltonian
system equation.<br>

The algorithm is taken from [1] and [2].

<h4>References</h4>
<PRE>
  [1] Benner, P., Byers, R.
      An Exact Line Search Method for Solving Generalized Continuous-Time Algebraic Riccati Equations
      IEEE Transactions On Automatic Control, Vol. 43, No. 1, pp. 101-107, 1998.
  [2] Datta, B.N.
      Numerical Methods for Linear Control Systems
      Elsevier Academic Press, 2004.
</PRE>

<h4>Example</h4>
<blockquote><pre>
     A=[0.0,         1.0,         0.0,         0.0;
        0.0,        -1.890,       3.900e-01,  -5.530;
        0.0,        -3.400e-02,  -2.980,       2.430;
        3.400e-02,  -1.100e-03,  -9.900e-01,  -2.100e-01];

     B=[ 0.0,         0.0;
         3.600e-01,  -1.60;
        -9.500e-01,  -3.200e-02;
         3.000e-02,   0.0];

     R=[1, 0; 0, 1];

     Q=[2.313,       2.727,       6.880e-01,   2.300e-02;
        2.727,       4.271,       1.148,       3.230e-01;
        6.880e-01,   1.148,       3.130e-01,   1.020e-01;
        2.300e-02,   3.230e-01,   1.020e-01,   8.300e-02];

    X0=identity(4);

    (X,r) = Matrices.Utilities.continuousRiccatiIterative(A, B, R, Q, X0);

  //  X = [1.3239,  0.9015,  0.5466, -1.7672;
           0.9015,  0.9607,  0.4334, -1.1989;
           0.5466,  0.4334,  0.4605, -1.3633;
          -1.7672, -1.1989, -1.3633,  4.4612]
  // r =  2.48809423389491E-015

    (,r) = Matrices.Utilities.continuousRiccatiIterative(A, B, R, Q, X0,4);

   // r =  0.0004;

<br>
</pre></blockquote>
<h4>See also</h4>
<a href=\"modelica://Modelica.Math.Matrices.Utilities.discreteRiccatiIterative\">Matrices.Utilities.discreteRiccatiIterative</a><br>
<a href=\"modelica://Modelica.Math.Matrices.continuousRiccati\">Matrices.continuousRiccati</a>
</html>", revisions = "<html>
<ul>
<li><i>2010/04/30 </i>
       by Marcus Baur, DLR-RM</li>
</ul>
</html>"));
        end continuousRiccatiIterative;

        function discreteRiccatiIterative "Newton's method with exact line search for solving discrete algebraic Riccati equation"
          extends Modelica.Icons.Function;
          import Modelica.Math.Matrices;
          input Real A[:, size(A, 1)] "Matrix A of discrete Riccati equation";
          input Real B[size(A, 1), :] "Matrix B of discrete Riccati equation";
          input Real R[size(B, 2), size(B, 2)] = identity(size(B, 2)) "Matrix R of discrete Riccati equation";
          input Real Q[size(A, 1), size(A, 2)] = identity(size(A, 1)) "Matrix Q of discrete Riccati equation";
          input Real X0[size(A, 1), size(A, 2)] = identity(size(A, 1)) "Initial approximate solution discrete Riccati equation";
          input Integer maxSteps = 10 "Maximal number of iteration steps";
          input Real eps = Matrices.frobeniusNorm(A) * 1e-9 "Tolerance for stop criterion";
          output Real X[size(X0, 1), size(X0, 2)];
          output Real r;
        protected
          Integer n = size(A, 1);
          Real Xk[size(A, 1), size(A, 2)];
          Real Ak[size(A, 1), size(A, 2)];
          Real Rk[size(A, 1), size(A, 2)];
          Real Nk[size(A, 1), size(A, 2)];
          Real Hk[size(B, 2), size(B, 1)];
          Real Vk[size(A, 1), size(A, 2)];
          Real AT[size(A, 2), size(A, 2)] = transpose(A);
          Real BT[size(B, 2), size(B, 1)] = transpose(B);
          Real tk;
          Integer k;
          Boolean stop;
        algorithm
          if n > 0 then
            k := 0;
            stop := false;
            Xk := X0;
            while not stop loop
              k := k + 1;
              Hk := Matrices.solve2(R + BT * Xk * B, BT);
              Ak := A - B * Hk * Xk * A;
              Rk := AT * Xk * A - Xk + Q - AT * Xk * B * Hk * Xk * A;
              Nk := Modelica.Math.Matrices.discreteLyapunov(A = Ak, C = -Rk, sgn = -1);
              Vk := transpose(Ak) * Nk * B * Hk * Nk * Ak;
              tk := Modelica.Math.Matrices.Utilities.findLocal_tk(Rk, Vk);
              stop := eps > Matrices.frobeniusNorm(tk * Nk) / Matrices.frobeniusNorm(Xk) or k >= maxSteps;
              Xk := Xk + tk * Nk;
            end while;
            X := Xk;
            r := Matrices.frobeniusNorm(AT * X * A - X + Q - AT * X * B * Matrices.solve2(R + BT * X * B, BT) * X * A);
          else
            X := fill(0, 0, 0);
            r := 0;
          end if;
          annotation(
            Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
           X = Matrices.Utilities.<b>discreteRiccatiIterative</b>(A, B, R, Q, X0);
      (X, r) = Matrices.Utilities.<b>discreteRiccatiIterative</b>(A, B, R, Q, X0, maxSteps, eps);
</pre></blockquote>
<h4>Description</h4>
<p>
This function provides a Newton-like method for solving discrete-time algebraic Riccati equations. It uses Exact Line Search to improve the sometimes erratic
convergence of Newton's method. Exact line search in this case means means, that at each iteration <code>i</code> a Newton step <code><b>delta</b>_i</code>
</p>
<blockquote><pre>
  <b>X</b>_i+1 = <b>X</b>_i + <b>delta</b>_i
</pre></blockquote>
<p>
is taken in the direction to minimize the Frobenius norm of the residual
</p>
<blockquote><pre>
  r = || <b>A</b>'<b>X</b>_i+1*<b>A</b> - <b>X</b>_i+1 - <b>A</b>'<b>X</b>_i+1*<b>G</b>_i*<b>X</b>_i+1*<b>A</b> + <b>Q</b> ||
</pre></blockquote>
<p>
with
</p>
<blockquote><pre>
                       -1
  G_i = <b>B</b>*(<b>R</b> + <b>B</b>'*<b>X</b>_i*<b>B</b>) *<b>B</b>'
</pre></blockquote>
<p>

Output <code>r</code> is the norm of the residual of the last iteration.<br>
<p>
The inputs \"maxSteps\" and \"eps\" specify the termination of the iteration. The iteration is terminated if either
maxSteps iteration steps have been performed or the relative change <b>delta</b>_i/<b>X</b>_i became smaller than eps.
<p>

With an appropriate initial value <b>X</b>0 a sufficiently accurate solution might be reach with a few iteration steps. Although a Lyapunov equation of
order <code>n</code> (n is the order of the Riccati equation) is to be solved at each iteration step, the algorithm might be faster
than a direct method like <a href=\"modelica://Modelica.Math.Matrices.discreteRiccati\">Matrices.discreteRiccati</a>, since direct methods have to solve the 2*n-order Hamiltonian
system equation.

The algorithm is taken from [1] and [2].

<h4>References</h4>
<PRE>
  [1] Benner, P., Byers, R.
      An Exact Line Search Method for Solving Generalized Continuous-Time Algebraic Riccati Equations
      IEEE Transactions On Automatic Control, Vol. 43, No. 1, pp. 101-107, 1998.
  [2] Datta, B.N.
      Numerical Methods for Linear Control Systems
      Elsevier Academic Press, 2004.
</PRE>

<h4>Example</h4>
<blockquote><pre>
     A  = [0.9970,    0.0000,    0.0000,    0.0000;
           1.0000,    0.0000,    0.0000,    0.0000;
           0.0000,    1.0000,    0.0000,    0.0000;
           0.0000,    0.0000,    1.0000,    0.0000];

     B  = [0.0150;
           0.0000;
           0.0000;
           0.0000];

     R = [0.2500];

     Q = [0, 0, 0, 0;
          0, 0, 0, 0;
          0, 0, 0, 0;
          0, 0, 0, 1];

    X0=identity(4);

    (X,r) = Matrices.Utilities.discreteRiccatiIterative(A, B, R, Q, X0);

  //  X = [30.625, 0.0, 0.0, 0.0;
            0.0,   1.0, 0.0, 0.0;
            0.0,   0.0, 1.0, 0.0;
            0.0,   0.0, 0.0, 1.0];

  // r =   3.10862446895044E-015
</pre></blockquote>
<h4>See also</h4>
<a href=\"modelica://Modelica.Math.Matrices.Utilities.continuousRiccatiIterative\">Matrices.Utilities.continuousRiccatiIterative</a><br>
<a href=\"modelica://Modelica.Math.Matrices.discreteRiccati\">Matrices.discreteRiccati</a>
</html>", revisions = "<html>
<ul>
<li><i>2010/04/30 </i>
       by Marcus Baur, DLR-RM</li>
</ul>
</html>"));
        end discreteRiccatiIterative;

        function householderReflection "Reflect each of the vectors a_i of matrix  A=[a_1, a_2, ..., a_n] on a plane with orthogonal vector u"
          extends Modelica.Icons.Function;
          import Modelica.Math.Vectors;
          input Real A[:, :] "Rectangular matrix";
          input Real u[size(A, 1)] "Householder vector";
          output Real RA[size(A, 1), size(A, 2)] "Reflexion of A";
        protected
          Integer n = size(A, 2);
          Real h;
          Real lu = Vectors.length(u) ^ 2;
        algorithm
          for i in 1:n loop
            h := scalar(2 * transpose(matrix(u)) * A[:, i] / lu);
            RA[:, i] := A[:, i] - h * u;
          end for;
          annotation(
            Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
Matrices.<b>householderReflection</b>(A,u);
</pre></blockquote>
<h4>Description</h4>
<p>
This function computes the Householder reflection (transformation)
</p>
<blockquote>
 <b>Ar</b> = <b>Q</b>*<b>A</b>
</blockquote>
with
<blockquote>
 <b>Q</b> = <b>I</b> -2*<b>u</b>*<b>u</b>'/(<b>u</b>'*<b>u</b>)
</blockquote>
<p>
where <b>u</b> is Householder vector, i.e., the normal vector of the reflection plane.
<p>
Householder reflection is widely used in numerical linear algebra, e.g., to perform QR decompositions.
</p>
<h4>Example</h4>
<blockquote><pre>
// First step of QR decomposition
  import   Modelica.Math.Vectors.Utilities;

  Real A[3,3] = [1,2,3;
                 3,4,5;
                 2,1,4];
  Real Ar[3,3];
  Real u[:];

  u=Utilities.householderVector(A[:,1],{1,0,0});
  // u= {0.763, 0.646, 0}

  Ar=householderReflection(A,u);
 // Ar = [-6.0828,   -5.2608,   -4.4388;
 //        0.0,      -1.1508,   -2.3016;
 //        0.0,       2.0,       0.0]

</pre></blockquote>

<h4>See also</h4>
<p>
<a href=\"modelica://Modelica.Math.Matrices.Utilities.householderSimilarityTransformation\">Matrices.Utilities.housholderSimilarityTransformation</a>,<br>
<a href=\"modelica://Modelica.Math.Vectors.Utilities.householderReflection\">Vectors.Utilities.householderReflection</a>,<br>
<a href=\"modelica://Modelica.Math.Vectors.Utilities.householderVector\">Vectors.Utilities.householderVector</a>
</p>
</html>", revisions = "<html>
<ul>
<li><i>2010/04/30 </i>
       by Marcus Baur, DLR-RM</li>
</ul>
</html>"));
        end householderReflection;

        function householderSimilarityTransformation "Perform the similarity transformation S*A*S of matrix A with symmetric householder matrix S = I - 2u*u'"
          extends Modelica.Icons.Function;
          import Modelica;
          import Modelica.Math.Vectors;
          input Real A[:, size(A, 1)] "Square matrix A";
          input Real u[size(A, 1)] "Householder vector";
          output Real SAS[size(A, 1), size(A, 1)] "Transformation of matrix A";
        protected
          Integer na = size(A, 1);
          Real S[size(A, 1), size(A, 1)] "Symmetric matrix";
          Integer i;
        algorithm
          if na > 0 then
            S := -2 * matrix(u) * transpose(matrix(u)) / (Vectors.length(u) * Vectors.length(u));
            for i in 1:na loop
              S[i, i] := 1.0 + S[i, i];
            end for;
            SAS := S * A * S;
          else
            SAS := fill(0.0, 0, 0);
          end if;
          annotation(
            Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
  As = Matrices.<b>householderSimilarityTransformation</b>(A,u);
</pre></blockquote>
<h4>Description</h4>
<p>
This function computes the Householder similarity transformation
</p>
<blockquote>
 <b>As</b> = <b>S</b>*<b>A</b>*<b>S</b>
</blockquote>
with
<blockquote>
 <b>S</b> = <b>I</b> -2*<b>u</b>*<b>u</b>'/(<b>u</b>'*<b>u</b>).
</blockquote>
<p>
This transformation is widely used for transforming non-symmetric matrices to a Hessenberg form.
</p>
<h4>Example</h4>
<blockquote><pre>
// First step of Hessenberg decomposition
  import   Modelica.Math.Vectors.Utilities;

  Real A[4,4] = [1,2,3,4;
                 3,4,5,6;
                 9,8,7,6;
                 1,2,0,0];
  Real Ar[4,4];
  Real u[4]={0,0,0,0};

  u[2:4]=Utilities.householderVector(A[2:4,1],{1,0,0});
  // u= = {0, 0.8107, 0.5819, 0.0647}

  Ar=householderSimilarityTransformation(A,u);
 //  Ar = [1.0,     -3.8787,    -1.2193,    3.531;
          -9.5394, 11.3407,      6.4336,   -5.9243;
           0.0,     3.1307,      0.7525,   -3.3670;
           0.0,     0.8021,     -1.1656,   -1.0932]
</pre></blockquote>

<h4>See also</h4>
<p>
<a href=\"modelica://Modelica.Math.Matrices.Utilities.householderReflection\">Matrices.Utilities.householderReflection</a>,<br>
<a href=\"modelica://Modelica.Math.Vectors.Utilities.householderReflection\">Vectors.Utilities.householderReflection</a>,<br>
<a href=\"modelica://Modelica.Math.Vectors.Utilities.householderVector\">Vectors.Utilities.householderVector</a>
</p>
</html>", revisions = "<html>
<ul>
<li><i>2010/04/30 </i>
       by Marcus Baur, DLR-RM</li>
</ul>
</html>"));
        end householderSimilarityTransformation;

        function toUpperHessenberg "Transform a real square matrix A to upper Hessenberg form H by orthogonal similarity transformation:  Q' * A * Q = H"
          extends Modelica.Icons.Function;
          import Modelica.Math.Matrices;
          import Modelica.Math.Matrices.LAPACK;
          input Real A[:, size(A, 1)] "Square matrix A";
          input Integer ilo = 1 "Lowest index where the original matrix had been Hessenbergform";
          input Integer ihi = size(A, 1) "Highest index where the original matrix had been Hessenbergform";
          output Real H[size(A, 1), size(A, 2)] "Upper Hessenberg form";
          output Real V[size(A, 1), size(A, 2)] "V=[v1,v2,..vn-1,0] with vi are vectors which define the elementary reflectors";
          output Real tau[max(0, size(A, 1) - 1)] "Scalar factors of the elementary reflectors";
          output Integer info "Information of successful function call";
        protected
          Integer n = size(A, 1);
          Real Aout[size(A, 1), size(A, 2)];
          Integer i;
        algorithm
          if n > 0 then
            (Aout, tau, info) := LAPACK.dgehrd(A, ilo, ihi);
            H[1:2, 1:ihi] := Aout[1:2, 1:ihi];
            H[1:2, ihi + 1:n] := A[1:2, ihi + 1:n];
            for i in 3:n loop
              H[i, i - 1:ihi] := Aout[i, i - 1:ihi];
              H[i, ihi + 1:n] := A[i, ihi + 1:n];
            end for;
            for i in 1:min(n - 2, ihi) loop
              V[i + 1, i] := 1.0;
              V[i + 2:n, i] := Aout[i + 2:n, i];
            end for;
            V[n, n - 1] := 1;
          end if;
          annotation(
            Documentation(info = "<html>
   <h4>Syntax</h4>
<blockquote><pre>
         H = Matrices.Utilities.<b>toUpperHessenberg</b>(A);
         (H, V, tau, info) = Matrices.Utilities.<b>toUpperHessenberg</b>(A,ilo, ihi);
</pre></blockquote>
<h4>Description</h4>
<p>
Function <b>toUpperHessenberg</b> computes a upper Hessenberg form <b>H</b> of a matrix <b>A</b> by orthogonal similarity transformation:  <b>Q</b>' * <b>A</b> * <b>Q</b> = <b>H</b>.
With the optional inputs ilo and ihi, also partial transformation is possible. The function calls LAPACK function DGEHRD.
See <a href=\"modelica://Modelica.Math.Matrices.LAPACK.dgehrd\">Matrices.LAPACK.dgehrd</a> for more information about the additional outputs V, tau, info and inputs ilo, ihi.
</p>

<h4>Example</h4>
<blockquote><pre>
 A  = [1, 2, 3;
       6, 5, 4;
       1, 0, 0];

 H = toUpperHessenberg(A);

  results in:

 H = [1.0,  -2.466,  2.630;
     -6.083, 5.514, -3.081;
      0.0,   0.919, -0.514]

</pre></blockquote>

<h4>See also</h4>
<a href=\"modelica://Modelica.Math.Matrices.hessenberg\">Matrices.hessenberg</a>
</html>", revisions = "<html><ul>
<li><i>2010/04/30 </i>
       by Marcus Baur, DLR-RM</li>
</ul>
</html>"));
        end toUpperHessenberg;

        function eigenvaluesHessenberg "Compute eigenvalues of an upper Hessenberg form matrix"
          extends Modelica.Icons.Function;
          import Modelica.Math.Matrices.LAPACK;
          input Real H[:, size(H, 1)] "Hessenberg matrix H";
          output Real ev[size(H, 1), 2] "Eigenvalues";
          output Integer info = 0;
        protected
          Real alphaReal[size(H, 1)] "Real part of alpha (eigenvalue=(alphaReal+i*alphaImag))";
          Real alphaImag[size(H, 1)] "Imaginary part of alpha (eigenvalue=(alphaReal+i*alphaImag))";
        algorithm
          if size(H, 1) > 0 then
            (alphaReal, alphaImag, info) := LAPACK.dhseqr(H);
          else
            alphaReal := fill(0, size(H, 1));
            alphaImag := fill(0, size(H, 1));
          end if;
          ev := [alphaReal, alphaImag];
          annotation(
            Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
           ev = Matrices.Utilities.<b>eigenvaluesHessenberg</b>(H);
   (ev, info) = Matrices.Utilities.<b>eigenvaluesHessenberg</b>(H);
</pre></blockquote>
<h4>Description</h4>
<p>
This function computes the eigenvalues of a Hessenberg form matrix. Transformation to Hessenberg form is the first step in eigenvalue computation for arbitrary matrices with QR decomposition.
This step can be skipped if the matrix has already Hessenberg form.
</p>

<p>
The function uses the LAPACK-routine dhseqr. Output <code>info</code> is 0 for a successful call of this
function.<br>
See <a href=\"modelica://Modelica.Math.Matrices.LAPACK.dhseqr\">Matrices.LAPACK.dhseqr</a> for details
</p>

<h4>Example</h4>
<blockquote><pre>
     Real A[3,3] = [1,2,3;
                    9,8,7;
                    0,1,0];

     Real ev[3,2];

     ev := Matrices.Utilities.eigenvaluesHessenberg(A);

  // ev  = [10.7538,    0.0;
            -0.8769,    1.0444;
            -0.8769,   -1.0444]
  // = {10.7538,  -0.8769 +- i*1.0444}
</pre></blockquote>
<br>

<h4>See also</h4>
<p>
<a href=\"modelica://Modelica.Math.Matrices.eigenValues\">Matrices.eigenValues</a>,
<a href=\"modelica://Modelica.Math.Matrices.hessenberg\">Matrices.hessenberg</a>
</p>
</html>", revisions = "<html>
<ul>
<li><i>2010/04/30 </i>
       by Marcus Baur, DLR-RM</li>
</ul>
</html>"));
        end eigenvaluesHessenberg;

        function reorderRSF "Reorders a real Schur form to clusters of stable and unstable eigenvalues"
          extends Modelica.Icons.Function;
          import Modelica.Math.Matrices.LAPACK;
          input Real T[:, :] "Real Schur form";
          input Real Q[:, size(T, 2)] "Schur vector Matrix";
          input Real alphaReal[size(T, 1)] "Real part of eigenvalue=alphaReal+i*alphaImag";
          input Real alphaImag[size(T, 1)] "Imaginary part of eigenvalue=alphaReal+i*alphaImag";
          input Boolean iscontinuous = true "True if the according system is continuous. False for discrete systems";
          output Real To[size(T, 1), size(T, 2)] "Reordered Schur form";
          output Real Qo[size(T, 1), size(T, 2)] "Reordered Schur vector matrix";
          output Real wr[size(T, 2)] "Reordered eigenvalues, real part";
          output Real wi[size(T, 2)] "Reordered eigenvalues, imaginary part";
        protected
          Integer n = size(T, 2);
          Boolean select[size(T, 2)] = fill(false, size(T, 2));
          Integer i;
        algorithm
          if iscontinuous then
            for i in 1:n loop
              if alphaReal[i] < 0 then
                select[i] := true;
              end if;
            end for;
          else
            for i in 1:n loop
              if alphaReal[i] ^ 2 + alphaImag[i] ^ 2 < 1 then
                select[i] := true;
              end if;
            end for;
          end if;
          (To, Qo, wr, wi) := LAPACK.dtrsen("E", "V", select, T, Q);
          annotation(
            Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
              To = Matrices.Utilities.<b>reorderRSF</b>(T, Q, alphaReal, alphaImag);
(To, Qo, wr, wi) = Matrices.Utilities.<b>reorderRSF</b>(T, Q, alphaReal, alphaImag, iscontinuous);
</pre></blockquote>
<h4>Description</h4>
<p>
Function <b>reorderRSF</b>() reorders a real Schur form such that the stable eigenvalues of
the system are in the 1-by-1 and 2-by-2 diagonal blocks of the block <b>upper</b> triangular matrix.
If the Schur form is referenced to a continuous system the staple eigenvalues are in the left complex half plane.
The stable eigenvalues of a discrete system are inside the complex unit circle.<br>
This function is used for example to solve algebraic Riccati equations
(<a href=\"modelica://Modelica.Math.Matrices.continuousRiccati\">continuousRiccati</a>,
<a href=\"modelica://Modelica.Math.Matrices.discreteRiccati\">discreteRiccati</a>). In this context the Schur form
as well as the corresponding eigenvalues and the transformation matrix <b>Q</b> are known, why the eigenvalues and the transformation matrix are inputs to <b>reorderRSF()</b>.<br>

The Schur vector matrix <b>Qo</b> is also reordered according to <b>To</b>. The vectors <b>wr</b> and <b>wi</b> contains the real and imaginary parts of the
reordered eigenvalues respectively.
</p>

<h4>Example</h4>
<blockquote><pre>
  T := [-1,2, 3,4;
         0,2, 6,5;
         0,0,-3,5;
         0,0, 0,6];
  To := Matrices.Utilities.reorderRSF(T,identity(4),{-1, 2, -3, 6},{0, 0, 0, 0}, true);

  // To = [-1.0, -0.384, 3.585, 4.0;
  //        0.0, -3.0,   6.0,   0.64;
  //        0.0,  0.0,   2.0,   7.04;
  //        0.0,  0.0,   0.0,   6.0]
</pre></blockquote>
<p>
See also <a href=\"modelica://Modelica.Math.Matrices.realSchur\">Matrices.realSchur</a>
</html>", revisions = "<html>
<ul>
<li><i>2010/04/30 </i>
       by Marcus Baur, DLR-RM</li>
</ul>
</html>"));
        end reorderRSF;

        function findLocal_tk "Find a local minimizer tk to define the length of the step tk*Nk in continuousRiccatiIterative and discreteRiccatiIterative"
          extends Modelica.Icons.Function;
          import Modelica.Math.Matrices;
          import Modelica.Math.Vectors;
          input Real Rk[:, size(Rk, 2)];
          input Real Vk[size(Rk, 1), size(Rk, 2)];
          output Real tk;
        protected
          Real alpha_k;
          Real beta_k;
          Real gamma_k;
          Real p[3, 2];
          Boolean h;
        algorithm
          alpha_k := Matrices.trace(Rk * Rk);
          beta_k := Matrices.trace(Rk * Vk);
          gamma_k := Matrices.trace(Vk * Vk);
          if gamma_k > Modelica.Constants.eps then
            p := Vectors.Utilities.roots({4 * gamma_k, 6 * beta_k, 2 * (alpha_k - 2 * beta_k), -2 * alpha_k});
            h := false;
            for i1 in 1:3 loop
              if abs(p[i1, 2]) < Modelica.Constants.eps then
                if abs(p[i1, 1] - 1) <= 1 then
                  tk := p[i1, 1];
                  h := true;
                end if;
              end if;
            end for;
            if not h then
              tk := 1;
            end if;
          else
            tk := 1;
          end if;
          annotation(
            Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
           tk = Matrices.Utilities.<b>findLocal_tk</b>(Rk, Vk);
</pre></blockquote>
<h4>Description</h4>
<p>
Function <code>findLocal_tk()</code> is an auxiliary function called in iterative solver for algebraic Riccati equation based on Newton's method with
exact line search like <a href=\"modelica://Modelica.Math.Matrices.Utilities.continuousRiccatiIterative\">continuousRiccatiIterative</a><br>
and <a href=\"modelica://Modelica.Math.Matrices.Utilities.discreteRiccatiIterative\">discreteRiccatiIterative</a>.<br>
The function computes the local minimum of the function f_k(t_k)
</p>
<blockquote><pre>
  f_k(t_k) = alpha_k*(1-t_k)^2 + 2*beta_k*(1-t)*t^2 + gamma_k*t^4
</pre></blockquote>
<p>
by calculating the zeros of the derivation d f_k/d t_k. It is known that the function f_k(t_k) has a local minimum at some value t_k_min in [0, 2].<br>
With t_k_min the norm of the next residual of the algorithm will be minimized.<br>
See [1] for more information

<h4>References</h4>
<PRE>
  [1] Benner, P., Byers, R.
      An Exact Line Search Method for Solving Generalized Continuous-Time Algebraic Riccati Equations
      IEEE Transactions On Automatic Control, Vol. 43, No. 1, pp. 101-107, 1998.
</PRE>

<h4>See also</h4>
<a href=\"modelica://Modelica.Math.Matrices.Utilities.continuousRiccatiIterative\">Matrices.Utilities.continuousRiccatiIterative</a><br>
<a href=\"modelica://Modelica.Math.Matrices.Utilities.discreteRiccatiIterative\">Matrices.Utilities.discreteRiccatiIterative</a><br>
</html>", revisions = "<html>
<ul>
<li><i>2010/04/30 </i>
       by Marcus Baur, DLR-RM</li>
</ul>
</html>"));
        end findLocal_tk;
        annotation(
          Documentation(info = "<html>
<p>
This package contains utility functions that are utilized by higher level matrix functions.
These functions are usually not useful for an end-user.
</p>
</html>"));
      end Utilities;
      annotation(
        Documentation(info = "<HTML>
<h4>Library content</h4>
<p>
This library provides functions operating on matrices. Below, the
functions are ordered according to categories and a typical
call of the respective function is shown.
Most functions are solely an interface to the external
<a href=\"modelica://Modelica.Math.Matrices.LAPACK\">LAPACK</a> library.
</p>

<p>
Note: A' is a short hand notation of transpose(A):
</p>

<p><b>Basic Information</b></p>
<ul>
<li> <a href=\"modelica://Modelica.Math.Matrices.toString\">toString</a>(A)
     - returns the string representation of matrix A.</li>

<li> <a href=\"modelica://Modelica.Math.Matrices.isEqual\">isEqual</a>(M1, M2)
     - returns true if matrices M1 and M2 have the same size and the same elements.</li>
</ul>

<p><b>Linear Equations</b></p>
<ul>
<li> <a href=\"modelica://Modelica.Math.Matrices.solve\">solve</a>(A,b)
     - returns solution x of the linear equation A*x=b (where b is a vector,
       and A is a square matrix that must be regular).</li>

<li> <a href=\"modelica://Modelica.Math.Matrices.solve2\">solve2</a>(A,B)
     - returns solution X of the linear equation A*X=B (where B is a matrix,
       and A is a square matrix that must be regular)</li>

<li> <a href=\"modelica://Modelica.Math.Matrices.leastSquares\">leastSquares</a>(A,b)
     - returns solution x of the linear equation A*x=b in a least squares sense
       (where b is a vector and A may be non-square and may be rank deficient)</li>

<li> <a href=\"modelica://Modelica.Math.Matrices.leastSquares2\">leastSquares2</a>(A,B)
     - returns solution X of the linear equation A*X=B in a least squares sense
       (where B is a matrix and A may be non-square and may be rank deficient)</li>

<li> <a href=\"modelica://Modelica.Math.Matrices.equalityLeastSquares\">equalityLeastSquares</a>(A,a,B,b)
     - returns solution x of a linear equality constrained least squares problem:
       min|A*x-a|^2 subject to B*x=b</<li>

<li> (LU,p,info) = <a href=\"modelica://Modelica.Math.Matrices.LU\">LU</a>(A)
     - returns the LU decomposition with row pivoting of a rectangular matrix A.</li>

<li> <a href=\"modelica://Modelica.Math.Matrices.LU_solve\">LU_solve</a>(LU,p,b)
     - returns solution x of the linear equation L*U*x[p]=b with a b
       vector and an LU decomposition from \"LU(..)\".</li>

<li> <a href=\"modelica://Modelica.Math.Matrices.LU_solve2\">LU_solve2</a>(LU,p,B)
     - returns solution X of the linear equation L*U*X[p,:]=B with a B
       matrix and an LU decomposition from \"LU(..)\".</li>
</ul>

<p><b>Matrix Factorizations</b></p>
<ul>
<li> (eval,evec) = <a href=\"modelica://Modelica.Math.Matrices.eigenValues\">eigenValues</a>(A)
     - returns eigen values \"eval\" and eigen vectors \"evec\" for a real,
       nonsymmetric matrix A in a Real representation.</li>

<li> <a href=\"modelica://Modelica.Math.Matrices.eigenValueMatrix\">eigenValueMatrix</a>(eval)
     - returns real valued block diagonal matrix of the eigenvalues \"eval\" of matrix A.</li>

<li> (sigma,U,VT) = <a href=\"modelica://Modelica.Math.Matrices.singularValues\">singularValues</a>(A)
     - returns singular values \"sigma\" and left and right singular vectors U and VT
       of a rectangular matrix A.</li>

<li> (Q,R,p) = <a href=\"modelica://Modelica.Math.Matrices.QR\">QR</a>(A)
     - returns the QR decomposition with column pivoting of a rectangular matrix A
       such that Q*R = A[:,p].</li>

<li> (H,U) = <a href=\"modelica://Modelica.Math.Matrices.hessenberg\">hessenberg</a>(A)
     - returns the upper Hessenberg form H and the orthogonal transformation matrix U
       of a square matrix A such that H = U'*A*U.</li>

<li> <a href=\"modelica://Modelica.Math.Matrices.realSchur\">realSchur</a>(A)
     - returns the real Schur form of a square matrix A.</li>

<li> <a href=\"modelica://Modelica.Math.Matrices.cholesky\">cholesky</a>(A)
     - returns the cholesky factor H of a real symmetric positive definite matrix A so that A = H'*H.</li>

<li> (D,Aimproved) = <a href=\"modelica://Modelica.Math.Matrices.balance\">balance</a>(A)
     - returns an improved form Aimproved of a square matrix A that has a smaller condition as A,
       with Aimproved = inv(diagonal(D))*A*diagonal(D).</li>
</ul>

<p><b>Matrix Properties</b></p>
<ul>
<li> <a href=\"modelica://Modelica.Math.Matrices.trace\">trace</a>(A)
     - returns the trace of square matrix A, i.e., the sum of the diagonal elements.</li>

<li> <a href=\"modelica://Modelica.Math.Matrices.det\">det</a>(A)
     - returns the determinant of square matrix A (using LU decomposition; try to avoid det(..))</li>

<li> <a href=\"modelica://Modelica.Math.Matrices.inv\">inv</a>(A)
     - returns the inverse of square matrix A (try to avoid, use instead \"solve2(..) with B=identity(..))</li>

<li> <a href=\"modelica://Modelica.Math.Matrices.rank\">rank</a>(A)
     - returns the rank of square matrix A (computed with singular value decomposition)</li>

<li> <a href=\"modelica://Modelica.Math.Matrices.conditionNumber\">conditionNumber</a>(A)
     - returns the condition number norm(A)*norm(inv(A)) of a square matrix A in the range 1..&infin;.</li>

<li> <a href=\"modelica://Modelica.Math.Matrices.rcond\">rcond</a>(A)
     - returns the reciprocal condition number 1/conditionNumber(A) of a square matrix A in the range 0..1.</li>

<li> <a href=\"modelica://Modelica.Math.Matrices.norm\">norm</a>(A)
     - returns the 1-, 2-, or infinity-norm of matrix A.</li>

<li> <a href=\"modelica://Modelica.Math.Matrices.frobeniusNorm\">frobeniusNorm</a>(A)
     - returns the Frobenius norm of matrix A.</li>

<li> <a href=\"modelica://Modelica.Math.Matrices.nullSpace\">nullSpace</a>(A)
     - returns the null space of matrix A.</li>
</ul>

<p><b>Matrix Exponentials</b></p>
<ul>
<li> <a href=\"modelica://Modelica.Math.Matrices.exp\">exp</a>(A)
     - returns the exponential e^A of a matrix A by adaptive Taylor series
       expansion with scaling and balancing</li>

<li> (phi, gamma) = <a href=\"modelica://Modelica.Math.Matrices.integralExp\">integralExp</a>(A,B)
     - returns the exponential phi=e^A and the integral gamma=integral(exp(A*t)*dt)*B as needed
       for a discretized system with zero order hold.</li>

<li> (phi, gamma, gamma1) = <a href=\"modelica://Modelica.Math.Matrices.integralExpT\">integralExpT</a>(A,B)
     - returns the exponential phi=e^A, the integral gamma=integral(exp(A*t)*dt)*B,
       and the time-weighted integral gamma1 = integral((T-t)*exp(A*t)*dt)*B as needed
       for a discretized system with first order hold.</li>
</ul>

<p><b>Matrix Equations</b></p>
<ul>
<li> <a href=\"modelica://Modelica.Math.Matrices.continuousLyapunov\">continuousLyapunov</a>(A,C)
     - returns solution X of the continuous-time Lyapunov equation X*A + A'*X = C</li>

<li> <a href=\"modelica://Modelica.Math.Matrices.continuousSylvester\">continuousSylvester</a>(A,B,C)
     - returns solution X of the continuous-time Sylvester equation A*X + X*B = C</li>

<li> <a href=\"modelica://Modelica.Math.Matrices.continuousRiccati\">continuousRiccati</a>(A,B,R,Q)
     - returns solution X of the continuous-time algebraic Riccati equation
       A'*X + X*A - X*B*inv(R)*B'*X + Q = 0</li>

<li> <a href=\"modelica://Modelica.Math.Matrices.discreteLyapunov\">discreteLyapunov</a>(A,C)
     - returns solution X of the discrete-time Lyapunov equation A'*X*A + sgn*X = C</li>

<li> <a href=\"modelica://Modelica.Math.Matrices.discreteSylvester\">discreteSylvester</a>(A,B,C)
     - returns solution X of the discrete-time Sylvester equation A*X*B + sgn*X = C</li>

<li> <a href=\"modelica://Modelica.Math.Matrices.discreteRiccati\">discreteRiccati</a>(A,B,R,Q)
     - returns solution X of the discrete-time algebraic Riccati equation
       A'*X*A - X - A'*X*B*inv(R + B'*X*B)*B'*X*A + Q = 0</li>
</ul>

<p><b>Matrix Manipulation</b></p>
<ul>
<li> <a href=\"modelica://Modelica.Math.Matrices.sort\">sort</a>(M)
     - returns the sorted rows or columns of matrix M in ascending or descending order.</li>

<li> <a href=\"modelica://Modelica.Math.Matrices.flipLeftRight\">flipLeftRight</a>(M)
     - returns matrix M so that the columns of M are flipped in left/right direction.</li>

<li> <a href=\"modelica://Modelica.Math.Matrices.flipUpDown\">flipUpDown</a>(M)
     - returns matrix M so that the rows of M are flipped in up/down direction.</li>
</ul>

<h4>See also</h4>
<a href=\"modelica://Modelica.Math.Vectors\">Vectors</a>

</html>"));
    end Matrices;

    package Nonlinear "Library of functions operating on nonlinear equations"
      extends Modelica.Icons.Package;

      package Examples "Examples demonstrating the usage of the functions in package Nonlinear"
        extends Modelica.Icons.ExamplesPackage;

        function quadratureLobatto1 "Integrate integral with fixed inputs"
          extends Modelica.Icons.Function;
          import Modelica.Utilities.Streams.print;
          input Real tolerance = 1e-5 "Error tolerance of integral values";
        protected
          Real I_numerical[3] "Numerical integral values";
          Real I_analytical[size(I_numerical, 1)] "Analytical integral values";
          Real I_err[size(I_numerical, 1)] "Absolute errors between numerical and analytical integral values";
        algorithm
          I_numerical[1] := Modelica.Math.Nonlinear.quadratureLobatto(function Modelica.Math.Nonlinear.Examples.UtilityFunctions.fun4(), 0, 1, tolerance);
          I_analytical[1] := (-cos(1)) + cos(0);
          I_numerical[2] := Modelica.Math.Nonlinear.quadratureLobatto(function Modelica.Math.Nonlinear.Examples.UtilityFunctions.fun5(w = 5), 0, 13, tolerance);
          I_analytical[2] := (-cos(5 * 13) / 5) + cos(5 * 0) / 5;
          I_numerical[3] := Modelica.Math.Nonlinear.quadratureLobatto(function Modelica.Math.Nonlinear.Examples.UtilityFunctions.fun6(k = 1 / sqrt(2)), 0, Modelica.Constants.pi / 2, tolerance);
          I_analytical[3] := 1.8540746773013719184338503;
          I_err := abs(I_numerical - I_analytical);
          print("\n... Results of Modelica.Math.Nonlinear.Examples.quadratureLobatto1:");
          print("Function 1 ( integral(sin(x)*dx) from x=0 to x=1): ");
          print("Analytical integral value = " + String(I_analytical[1], format = "2.16f"));
          print("Numerical integral value  = " + String(I_numerical[1], format = "2.16f"));
          print("Absolute difference       = " + String(I_err[1], format = "2.0e"));
          print("");
          print("Function 2 (integral(sin(5*x)*dx) from x=0 to x=13): ");
          print("Analytical integral value = " + String(I_analytical[2], format = "2.16f"));
          print("Numerical integral value  = " + String(I_numerical[2], format = "2.16f"));
          print("Absolute difference       = " + String(I_err[2], format = "2.0e"));
          print("");
          print("Function 3 (Elliptic integral from x=0 to pi/2): ");
          print("Analytical integral value = " + String(I_analytical[3], format = "2.16f"));
          print("Numerical integral value  = " + String(I_numerical[3], format = "2.16f"));
          print("Absolute difference       = " + String(I_err[3], format = "2.0e"));
          annotation(
            Documentation(info = "<html>
<p>
This example integrates the following integrands with function
<a href=\"modelica://Modelica.Math.Nonlinear.quadratureLobatto\">quadratureLobatto</a>
and compares the result with an analytical solution.
The examples also demonstrate how additional input arguments to the integrand
function can be passed as additional arguments.
The following integrals are computed:
</p>

<ul>
<li> integral(sin(x)*dx) from x=0 to x=1 </li>
<li> integral(sin(5*x)*dx) from x=0 to x=13 </li>
<li> elliptic integral from x=0 to pi/2 </li>
</ul>

</html>"));
        end quadratureLobatto1;

        function quadratureLobatto2 "Integrate integral with user dependent inputs"
          extends Modelica.Icons.Function;
          import Modelica.Utilities.Streams.print;
          input Real Tolerance = 1e-5 "Error tolerance of integral value" annotation(
            Dialog(group = "General"));
          input Real a1 = 0 "Lower limit" annotation(
            Dialog(group = "Sine"));
          input Real b1 = 1 "Upper limit" annotation(
            Dialog(group = "Sine"));
          input Real a2 = 0 "Lower limit" annotation(
            Dialog(group = "Sine w"));
          input Real b2 = 13 "Upper limit" annotation(
            Dialog(group = "Sine w"));
          input Real w = 5 "Angular velocity" annotation(
            Dialog(group = "Sine w"));
          input Real a3 = 0 "Lower limit" annotation(
            Dialog(group = "Elliptic integral"));
          input Real b3 = Modelica.Constants.pi / 2 "Upper limit" annotation(
            Dialog(group = "Elliptic integral"));
          input Real k = 1 / sqrt(2) "Modul" annotation(
            Dialog(group = "Elliptic integral"));
        protected
          Real I[3] "Numerical integral values";
        algorithm
          I[1] := Modelica.Math.Nonlinear.quadratureLobatto(function Modelica.Math.Nonlinear.Examples.UtilityFunctions.fun4(), a1, b1, Tolerance);
          I[2] := Modelica.Math.Nonlinear.quadratureLobatto(function Modelica.Math.Nonlinear.Examples.UtilityFunctions.fun5(w = w), a2, b2, Tolerance);
          I[3] := Modelica.Math.Nonlinear.quadratureLobatto(function Modelica.Math.Nonlinear.Examples.UtilityFunctions.fun6(k = k), a3, b3, Tolerance);
          print("\n... Results of Modelica.Math.Nonlinear.Examples.quadratureLobatto2:");
          print("Function 1 (integral(sin(x)*dx)): ");
          print("Numerical integral value  = " + String(I[1], format = "2.16f"));
          print("");
          print("Function 2 (integral(sin(w*x)*dx)): ");
          print("Numerical integral value  = " + String(I[2], format = "2.16f"));
          print("");
          print("Function 3 (Elliptic integral): ");
          print("Numerical integral value  = " + String(I[3], format = "2.16f"));
          annotation(
            Documentation(info = "<html>
<p>
This example solves the following integrands with function
<a href=\"modelica://Modelica.Math.Nonlinear.quadratureLobatto\">quadratureLobatto</a>.
The user can set the parameters, like \"w\" or \"k\", and can experiment with
different integration intervals.
The following integrals are computed:
</p>

<ul>
<li> integral(sin(x)*dx) </li>
<li> integral(sin(w*x)*dx) </li>
<li> elliptic integral </li>
</ul>

</html>"));
        end quadratureLobatto2;

        function solveNonlinearEquations1 "Solve nonlinear equations with fixed inputs"
          extends Modelica.Icons.Function;
          import Modelica.Utilities.Streams.print;
          input Real tolerance = 100 * Modelica.Constants.eps "Relative tolerance of solution u";
        protected
          Real u_numerical[3];
          Real u_analytical[3];
          Real u_err[3];
        algorithm
          u_numerical[1] := Modelica.Math.Nonlinear.solveOneNonlinearEquation(function Modelica.Math.Nonlinear.Examples.UtilityFunctions.fun1(), -0.5, 10, tolerance);
          u_analytical[1] := 1.0;
          u_numerical[2] := Modelica.Math.Nonlinear.solveOneNonlinearEquation(function Modelica.Math.Nonlinear.Examples.UtilityFunctions.fun2(w = 3), 0, 5, tolerance);
          u_analytical[2] := 0.6448544035840080891877538;
          u_numerical[3] := Modelica.Math.Nonlinear.solveOneNonlinearEquation(function Modelica.Math.Nonlinear.Examples.UtilityFunctions.fun3(p = {5, 1}, m = 1), 1, 8, tolerance);
          u_analytical[3] := 6.9368474072202187221643182;
          u_err := abs(u_numerical - u_analytical);
          print("\n... Results of Modelica.Math.Nonlinear.Examples.solveNonlinearEquations1:");
          print("Solve 3 nonlinear equations with relative tolerance = " + String(tolerance) + "\n");
          print("Function 1 (u^2 - 1 = 0): ");
          print("Analytical zero     = " + String(u_analytical[1], format = "2.16f"));
          print("Numerical zero      = " + String(u_numerical[1], format = "2.16f"));
          print("Absolute difference = " + String(u_err[1], format = "2.0e"));
          print("");
          print("Function 2 (3*u - sin(3*u) - 1 = 0): ");
          print("Analytical zero     = " + String(u_analytical[2], format = "2.16f"));
          print("Numerical zero      = " + String(u_numerical[2], format = "2.16f"));
          print("Absolute difference = " + String(u_err[2], format = "2.0e"));
          print("");
          print("Function 3 (5 + log(u) - u = 0): ");
          print("Analytical zero     = " + String(u_analytical[3], format = "2.16f"));
          print("Numerical zero      = " + String(u_numerical[3], format = "2.16f"));
          print("Absolute difference = " + String(u_err[3], format = "2.0e"));
          annotation(
            Documentation(info = "<html>
<p>
This example solves the following nonlinear equations with function
<a href=\"modelica://Modelica.Math.Nonlinear.solveOneNonlinearEquation\">solveOneNonlinearEquation</a>
and compares the result with the available analytical solution.
The examples also demonstrate how additional input arguments to the nonlinear equation
function can be passes as additional arguments.
The following nonlinear equations are solved:
</p>

<ul>
<li> 0 = u^2 - 1 </li>
<li> 0 = 3*u - sin(3*u) - 1 </li>
<li> 0 = 5 + log(u) - u </li>
</ul>

</html>"));
        end solveNonlinearEquations1;

        function solveNonlinearEquations2 "Solve nonlinear equations with user dependent inputs"
          extends Modelica.Icons.Function;
          import Modelica.Utilities.Streams.print;
          input Real tolerance = 100 * Modelica.Constants.eps "Relative tolerance of solution u" annotation(
            Dialog(group = "General"));
          input Real u_min1 = -0.5 "Lower limit" annotation(
            Dialog(group = "u^2-1"));
          input Real u_max1 = 10 "Upper limit" annotation(
            Dialog(group = "u^2-1"));
          input Real u_min2 = 0 "Lower limit" annotation(
            Dialog(group = "3*u - sin(w*u) - 1"));
          input Real u_max2 = 5 "Upper limit" annotation(
            Dialog(group = "3*u - sin(w*u) - 1"));
          input Real w = 3 "Angular velocity" annotation(
            Dialog(group = "3*u - sin(w*u) - 1"));
          input Real u_min3 = 1 "Lower limit" annotation(
            Dialog(group = "p[1] + log(p[2]*u) - m*u"));
          input Real u_max3 = 8 "Upper limit" annotation(
            Dialog(group = "p[1] + log(p[2]*u) - m*u"));
          input Real p[2] = {5, 1} "Parameter vector" annotation(
            Dialog(group = "p[1] + log(p[2]*u) - m*u"));
          input Real m = 1 "Parameter" annotation(
            Dialog(group = "p[1] + log(p[2]*u) - m*u"));
        protected
          Real u[3];
        algorithm
          u[1] := Modelica.Math.Nonlinear.solveOneNonlinearEquation(function Modelica.Math.Nonlinear.Examples.UtilityFunctions.fun1(), u_min1, u_max1, tolerance);
          u[2] := Modelica.Math.Nonlinear.solveOneNonlinearEquation(function Modelica.Math.Nonlinear.Examples.UtilityFunctions.fun2(w = w), u_min2, u_max2, tolerance);
          u[3] := Modelica.Math.Nonlinear.solveOneNonlinearEquation(function Modelica.Math.Nonlinear.Examples.UtilityFunctions.fun3(p = p, m = m), u_min3, u_max3, tolerance);
          print("\n... Results of Modelica.Math.Nonlinear.Examples.solveNonlinearEquations2:");
          print("Solve 3 nonlinear equations with relative tolerance = " + String(tolerance) + "\n");
          print("Function 1 (u^2 - 1): ");
          print("Numerical zero = " + String(u[1], format = "2.16f"));
          print("");
          print("Function 2 (3*u - sin(w*u) - 1): ");
          print("Numerical zero = " + String(u[2], format = "2.16f"));
          print("");
          print("Function 3 (p[1] + log(p[2]*u) - m*u): ");
          print("Numerical zero = " + String(u[3], format = "2.16f"));
          annotation(
            Documentation(info = "<html>
<p>
This example solves the following nonlinear equations with function
<a href=\"modelica://Modelica.Math.Nonlinear.solveOneNonlinearEquation\">solveOneNonlinearEquation</a>.
The user can set the parameters, like \"w\" or \"m\", and can experiment with
different start intervals.
The following nonlinear equations are solved:
</p>

<ul>
<li> 0 = u^2 - 1 </li>
<li> 0 = 3*u - sin(w*u) - 1 </li>
<li> 0 = p[1] + log(p[2]*u) - m*u </li>
</ul>

</html>"));
        end solveNonlinearEquations2;

        model quadratureLobatto3 "Integrate function in a model"
          extends Modelica.Icons.Example;
          parameter Real A = 1 "Amplitude of integrand of s";
          parameter Real ws = 2 "Angular frequency of integrand of s";
          parameter Real wq = 3 "Angular frequency of q";
          Real q(start = 1, fixed = true);
          Real qd(start = 0, fixed = true);
          Real x;
          final parameter Real s = Modelica.Math.Nonlinear.quadratureLobatto(function UtilityFunctions.fun7(A = A, w = ws), 0, 1);
        equation
          qd = der(q);
          der(qd) + wq * q = 0;
          x = s * q;
          annotation(
            Documentation(info = "<html>
<p>
This example demonstrates how to utilize a function as input argument
to a function in a model.
</p>
</html>"),
            experiment(StopTime = 5));
        end quadratureLobatto3;

        package UtilityFunctions "Utility functions that are used as function arguments to the examples"
          extends Modelica.Icons.UtilitiesPackage;

          function fun1 "y = u^2 - 1"
            extends Modelica.Math.Nonlinear.Interfaces.partialScalarFunction;
          algorithm
            y := u ^ 2 - 1;
          end fun1;

          function fun2 "y = 3*u - sin(w*u) - 1"
            extends Modelica.Math.Nonlinear.Interfaces.partialScalarFunction;
            input Real w "Angular velocity";
          algorithm
            y := 3 * u - sin(w * u) - 1;
          end fun2;

          function fun3 "y = p[1] + log(p[2]*u) - m*u"
            extends Modelica.Math.Nonlinear.Interfaces.partialScalarFunction;
            input Real p[2];
            input Real m;
          algorithm
            y := p[1] + log(p[2] * u) - m * u;
          end fun3;

          function fun4 "y = sin(u)"
            extends Modelica.Math.Nonlinear.Interfaces.partialScalarFunction;
          algorithm
            y := sin(u);
          end fun4;

          function fun5 "y = sin(w*u)"
            extends Modelica.Math.Nonlinear.Interfaces.partialScalarFunction;
            input Real w "Angular velocity";
          algorithm
            y := sin(w * u);
          end fun5;

          function fun6 "y = sqrt(1/(1 - k^2*sin(u)^2))"
            extends Modelica.Math.Nonlinear.Interfaces.partialScalarFunction;
            input Real k "Modul";
          algorithm
            y := sqrt(1 / (1 - k ^ 2 * sin(u) ^ 2));
          end fun6;

          function fun7 "y = A*sin(w*u)*q(t)"
            extends Modelica.Math.Nonlinear.Interfaces.partialScalarFunction;
            input Real A "Amplitude";
            input Real w "Angular frequency";
          algorithm
            y := A * sin(w * u);
          end fun7;
          annotation(
            Documentation(info = "<html>
<p>
This package provides utility functions that are used as input
arguments to the example functions.
</p>

</html>"));
        end UtilityFunctions;
      end Examples;

      package Interfaces "Interfaces for functions"
        extends Modelica.Icons.InterfacesPackage;

        encapsulated partial function partialScalarFunction "Interface for a function with one input and one output Real signal"
          import Modelica;
          extends Modelica.Icons.Function;
          input Real u "Independent variable";
          output Real y "Dependent variable y=f(u)";
          annotation(
            Documentation(info = "<html>
<p>
This partial function defines the interface of a function with
one input and one output Real signal. The scalar functions
of <a href=\"modelica://Modelica.Math.Nonlinear\">Modelica.Math.Nonlinear</a>
are derived from this base type by inheritance.
This allows to use these functions directly as function arguments
to a function, see, .e.g.,
<a href=\"modelica://Modelica.Math.Nonlinear.Examples\">Math.Nonlinear.Examples</a>.
</p>

</html>"));
        end partialScalarFunction;
        annotation(
          Documentation(info = "<html>
<p>
Interface definitions of functions. The main purpose is to use functions
derived from these interface definitions as function arguments
to a function, see, .e.g.,
<a href=\"modelica://Modelica.Math.Nonlinear.Examples\">Math.Nonlinear.Examples</a>.
</p>
</html>"));
      end Interfaces;

      function quadratureLobatto "Return the integral of an integrand function using an adaptive Lobatto rule"
        extends Modelica.Icons.Function;
        input Modelica.Math.Nonlinear.Interfaces.partialScalarFunction f "Integrand function";
        input Real a "Lower limit of integration interval";
        input Real b "Upper limit of integration interval";
        input Real tolerance = 100 * Modelica.Constants.eps "Relative tolerance for integral value";
        output Real integral "integral value";
      protected
        constant Real x1 = 0.942882415695480;
        constant Real x2 = 0.641853342345781;
        constant Real x3 = 0.236383199662150;
        constant Real eps = 10 * Modelica.Constants.eps;
        Real m;
        Real h;
        Real alpha;
        Real beta;
        Real x[13];
        Real y[13];
        Real fa;
        Real fb;
        Real i1;
        Real i2;
        Real is;
        Real erri1;
        Real erri2;
        Real R;
        Real tol;
        Integer s;

        function quadStep "Recursive function used by quadrature"
          input Modelica.Math.Nonlinear.Interfaces.partialScalarFunction f;
          input Real a "Right interval end";
          input Real b "Left interval end";
          input Real fa "Function value at a";
          input Real fb "Function value at b";
          input Real is "First approximation of the integral";
          output Real I "integral value";
        protected
          Real m;
          Real h;
          Real alpha;
          Real beta;
          Real x[5];
          Real y[5];
          Real mll;
          Real ml;
          Real mr;
          Real mrr;
          Real fmll;
          Real fml;
          Real fm;
          Real fmr;
          Real fmrr;
          Real i1;
          Real i2;
        algorithm
          h := (b - a) / 2;
          m := (a + b) / 2;
          alpha := sqrt(2 / 3);
          beta := 1 / sqrt(5);
          mll := m - alpha * h;
          ml := m - beta * h;
          mr := m + beta * h;
          mrr := m + alpha * h;
          x := {mll, ml, m, mr, mrr};
          for i in 1:size(x, 1) loop
            y[i] := f(x[i]);
          end for;
          fmll := y[1];
          fml := y[2];
          fm := y[3];
          fmr := y[4];
          fmrr := y[5];
          i2 := h / 6 * (fa + fb + 5 * (fml + fmr));
          i1 := h / 1470 * (77 * (fa + fb) + 432 * (fmll + fmrr) + 625 * (fml + fmr) + 672 * fm);
          if is + i1 - i2 == is or mll <= a or b <= mrr then
            I := i1;
          else
            I := quadStep(f, a, mll, fa, fmll, is) + quadStep(f, mll, ml, fmll, fml, is) + quadStep(f, ml, m, fml, fm, is) + quadStep(f, m, mr, fm, fmr, is) + quadStep(f, mr, mrr, fmr, fmrr, is) + quadStep(f, mrr, b, fmrr, fb, is);
          end if;
        end quadStep;
      algorithm
/*
        Numerically evaluate integral using adaptive
        Lobatto rule.
        see Walter Gander: Adaptive Quadrature - Revisited, 1998
                        ftp.inf.ethz.ch in doc/tech-reports/1998/306.ps

        x[:] are the nodes
        y[:] = f(x[:]) are function values at the nodes
        */
        tol := tolerance;
        m := (a + b) / 2;
        h := (b - a) / 2;
        alpha := sqrt(2 / 3);
        beta := 1 / sqrt(5);
        x := {a, m - x1 * h, m - alpha * h, m - x2 * h, m - beta * h, m - x3 * h, m, m + x3 * h, m + beta * h, m + x2 * h, m + alpha * h, m + x1 * h, b};
        for i in 1:size(x, 1) loop
          y[i] := f(x[i]);
        end for;
        fa := y[1];
        fb := y[13];
        i2 := h / 6 * (y[1] + y[13] + 5 * (y[5] + y[9]));
        i1 := h / 1470 * (77 * (y[1] + y[13]) + 432 * (y[3] + y[11]) + 625 * (y[5] + y[9]) + 672 * y[7]);
        is := h * (0.0158271919734802 * (y[1] + y[13]) + 0.0942738402188500 * (y[2] + y[12]) + 0.155071987336585 * (y[3] + y[11]) + 0.188821573960182 * (y[4] + y[10]) + 0.199773405226859 * (y[5] + y[9]) + 0.224926465333340 * (y[6] + y[8]) + 0.242611071901408 * y[7]);
        s := sign(is);
        if s == 0 then
          s := 1;
        end if;
        erri1 := abs(i1 - is);
        erri2 := abs(i2 - is);
        R := 1;
        if erri2 <> 0 then
          R := erri1 / erri2;
        end if;
        if R > 0 and R < 1 then
          tol := tol / R;
        end if;
        is := s * abs(is) * tol / eps;
        if is == 0 then
          is := b - a;
        end if;
        integral := quadStep(f, a, b, fa, fb, is);
        annotation(
          Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
<b>quadratureLobatto</b>(function f(), a, b);
<b>quadratureLobatto</b>(function f(), a, b, tolerance=100*Modelica.Constants.eps);
</pre></blockquote>

<h4>Description</h4>

<p>
Compute definite integral over function f(u,...) from u=a up to u=b
using the adaptive Lobatto rule according to:
</p>

<blockquote>
<dl>
<dt>Walter Gander:</dt>
<dd> <b>Adaptive Quadrature - Revisited</b>. 1998.
     <a href=\"ftp://ftp.inf.ethz.ch/doc/tech-reports/1998/306.ps\">ftp://ftp.inf.ethz.ch/doc/tech-reports/1998/306.ps</a>
     </dd>
</dl>
</blockquote>

<h4>Example</h4>

<p>
See the examples in <a href=\"modelica://Modelica.Math.Nonlinear.Examples\">Modelica.Math.Nonlinear.Examples</a>.
</p>

</html>"));
      end quadratureLobatto;

      function solveOneNonlinearEquation "Solve f(u) = 0 in a very reliable and efficient way (f(u_min) and f(u_max) must have different signs)"
        extends Modelica.Icons.Function;
        import Modelica.Utilities.Streams.error;
        input Modelica.Math.Nonlinear.Interfaces.partialScalarFunction f "Function y = f(u); u is computed so that y=0";
        input Real u_min "Lower bound of search interval";
        input Real u_max "Upper bound of search interval";
        input Real tolerance = 100 * Modelica.Constants.eps "Relative tolerance of solution u";
        output Real u "Value of independent variable u so that f(u) = 0";
      protected
        constant Real eps = Modelica.Constants.eps "machine epsilon";
        Real a = u_min "Current best minimum interval value";
        Real b = u_max "Current best maximum interval value";
        Real c "Intermediate point a <= c <= b";
        Real d;
        Real e "b - a";
        Real m;
        Real s;
        Real p;
        Real q;
        Real r;
        Real tol;
        Real fa "= f(a)";
        Real fb "= f(b)";
        Real fc;
        Boolean found = false;
      algorithm
// Check that f(u_min) and f(u_max) have different sign
        fa := f(u_min);
        fb := f(u_max);
        fc := fb;
        if fa > 0.0 and fb > 0.0 or fa < 0.0 and fb < 0.0 then
          error("The arguments u_min and u_max provided in the function call\n" + "    solveOneNonlinearEquation(f,u_min,u_max)\n" + "do not bracket the root of the single non-linear equation 0=f(u):\n" + "  u_min  = " + String(u_min) + "\n" + "  u_max  = " + String(u_max) + "\n" + "  fa = f(u_min) = " + String(fa) + "\n" + "  fb = f(u_max) = " + String(fb) + "\n" + "fa and fb must have opposite sign which is not the case");
        end if;
// Initialize variables
        c := a;
        fc := fa;
        e := b - a;
        d := e;
// Search loop
        while not found loop
          if abs(fc) < abs(fb) then
            a := b;
            b := c;
            c := a;
            fa := fb;
            fb := fc;
            fc := fa;
          end if;
          tol := 2 * eps * abs(b) + tolerance;
          m := (c - b) / 2;
          if abs(m) <= tol or fb == 0.0 then
            found := true;
            u := b;
          else
            if abs(e) < tol or abs(fa) <= abs(fb) then
              e := m;
              d := e;
            else
              s := fb / fa;
              if a == c then
                p := 2 * m * s;
                q := 1 - s;
              else
                q := fa / fc;
                r := fb / fc;
                p := s * (2 * m * q * (q - r) - (b - a) * (r - 1));
                q := (q - 1) * (r - 1) * (s - 1);
              end if;
              if p > 0 then
                q := -q;
              else
                p := -p;
              end if;
              s := e;
              e := d;
              if 2 * p < 3 * m * q - abs(tol * q) and p < abs(0.5 * s * q) then
                d := p / q;
              else
                e := m;
                d := e;
              end if;
            end if;
            a := b;
            fa := fb;
            b := b + (if abs(d) > tol then d else if m > 0 then tol else -tol);
            fb := f(b);
            if fb > 0 and fc > 0 or fb < 0 and fc < 0 then
              c := a;
              fc := fa;
              e := b - a;
              d := e;
            end if;
          end if;
        end while;
// root found (interval is small enough)
// Determine if a bisection is needed
// linear interpolation
// inverse quadratic interpolation
// interpolation successful
// use bi-section
// Best guess value is defined as "a"
// initialize variables
        annotation(
          Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
<b>solveOneNonlinearEquation</b>(function f(), u_min, u_max);
<b>solveOneNonlinearEquation</b>(function f(), u_min, u_max, tolerance=100*Modelica.Constants.eps);
</pre></blockquote>

<h4>Description</h4>

<p>
This function determines the solution of <b>one non-linear algebraic equation</b> \"y=f(u)\"
in <b>one unknown</b> \"u\" in a reliable way. It is one of the best numerical
algorithms for this purpose. As input, the nonlinear function f(u)
has to be given, as well as an interval u_min, u_max that
contains the solution, i.e., \"f(u_min)\" and \"f(u_max)\" must
have a different sign. The function computes a smaller interval
in which a sign change is present using the relative tolerance
\"tolerance\" that can be given as 4th input argument.
</p>

<p>
The interval reduction is performed using
inverse quadratic interpolation (interpolating with a quadratic polynomial
through the last 3 points and computing the zero). If this fails,
bisection is used, which always reduces the interval by a factor of 2.
The inverse quadratic interpolation method has superlinear convergence.
This is roughly the same convergence rate as a globally convergent Newton
method, but without the need to compute derivatives of the non-linear
function. The solver function is a direct mapping of the Algol 60 procedure
\"zero\" to Modelica, from:
</p>

<blockquote>
<dl>
<dt> Brent R.P.:</dt>
<dd> <b>Algorithms for Minimization without derivatives</b>.
     Prentice Hall, 1973, pp. 58-59.<br>
     Download: <a href=\"http://wwwmaths.anu.edu.au/~brent/pd/rpb011i.pdf\">http://wwwmaths.anu.edu.au/~brent/pd/rpb011i.pdf</a><br>
     Errata and new print: <a href=\"http://wwwmaths.anu.edu.au/~brent/pub/pub011.html\">http://wwwmaths.anu.edu.au/~brent/pub/pub011.html</a>
</dd>
</dl>
</blockquote>

<h4>Example</h4>

<p>
See the examples in <a href=\"modelica://Modelica.Math.Nonlinear.Examples\">Modelica.Math.Nonlinear.Examples</a>.
</p>
</html>"));
      end solveOneNonlinearEquation;
      annotation(
        Documentation(info = "<html>
<p>
This package contains functions to perform tasks such as numerically integrating
a function, or solving a nonlinear algebraic equation system.
The common feature of the functions in this package is
that the nonlinear characteristics are passed as user definable
functions.
</p>

<p>
For details about how to define and to use functions as input arguments
to functions, see
<a href=\"modelica://ModelicaReference.Classes.'function'\">ModelicaReference.Classes.'function'</a>
or the Modelica Language  Specification, Chapter 12.4.2.
</p>

</html>", revisions = "<html>
<ul>
<li><i>July 2010 </i> by Martin Otter (DLR-RM): <br>
    Included in MSL3.2, adapted, and documentation improved</li>

<li><i>March 2010 </i> by Andreas Pfeiffer (DLR-RM): <br>
    Adapted the quadrature function from Gerhard Schillhuber and
    the solution of one non-linear equation in one unknown from
    Modelica.Media.Common.OneNonLinearEquation so that
    function objects are used.</li>

<li><i>June 2002 </i> by Gerhard Schillhuber (master thesis at DLR-RM):<br>
       Adaptive quadrature to compute the curve length of a Spline.</li>
</ul>
</html>"));
    end Nonlinear;

    package BooleanVectors "Library of functions operating on Boolean vectors"
      extends Modelica.Icons.Package;

      function allTrue "Returns true, if all elements of the Boolean input vector are true ('and')"
        extends Modelica.Icons.Function;
        input Boolean b[:] "Boolean vector";
        output Boolean result "= true, if all elements of b are true";
      algorithm
        result := size(b, 1) > 0;
        for i in 1:size(b, 1) loop
          result := result and b[i];
        end for;
        annotation(
          Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
<b>allTrue</b>(b);
</pre></blockquote>

<h4>Description</h4>
<p>
Returns <b>true</b> if all elements of the Boolean input vector b are <b>true</b>.
Otherwise the function returns <b>false</b>. If b is an empty vector,
i.e., size(b,1)=0, the function returns <b>false</b>.
</p>

<h4>Example</h4>
<blockquote><pre>
  Boolean b1[3] = {true, true, true};
  Boolean b2[3] = {false, true, false};
  Boolean r1, r2;
<b>algorithm</b>
  r1 = allTrue(b1);  // r1 = true
  r2 = allTrue(b2);  // r2 = false
</pre></blockquote>

<h4>See also</h4>
<p>
<a href=\"modelica://Modelica.Math.BooleanVectors.anyTrue\">anyTrue</a>,
<a href=\"modelica://Modelica.Math.BooleanVectors.countTrue\">countTrue</a>,
<a href=\"modelica://Modelica.Math.BooleanVectors.enumerate\">enumerate</a>,
<a href=\"modelica://Modelica.Math.BooleanVectors.firstTrueIndex\">firstTrueIndex</a>,
<a href=\"modelica://Modelica.Math.BooleanVectors.index\">index</a>, and
<a href=\"modelica://Modelica.Math.BooleanVectors.oneTrue\">oneTrue</a>.
</p>

</html>"));
      end allTrue;

      function anyTrue "Returns true, if at least on element of the Boolean input vector is true ('or')"
        extends Modelica.Icons.Function;
        input Boolean b[:];
        output Boolean result;
      algorithm
        result := false;
        for i in 1:size(b, 1) loop
          result := result or b[i];
        end for;
        annotation(
          Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
<b>anyTrue</b>(b);
</pre></blockquote>

<h4>Description</h4>
<p>
Returns <b>true</b> if at least one elements of the input Boolean vector b is <b>true</b>.
Otherwise the function returns <b>false</b>. If b is an empty vector,
i.e., size(b,1)=0, the function returns <b>false</b>.
</p>

<h4>Example</h4>
<blockquote><pre>
  Boolean b1[3] = {false, false, false};
  Boolean b2[3] = {false, true, false};
  Boolean r1, r2;
<b>algorithm</b>
  r1 = anyTrue(b1);  // r1 = false
  r2 = anyTrue(b2);  // r2 = true
</pre></blockquote>

<h4>See also</h4>
<p>
<a href=\"modelica://Modelica.Math.BooleanVectors.allTrue\">allTrue</a>,
<a href=\"modelica://Modelica.Math.BooleanVectors.countTrue\">countTrue</a>,
<a href=\"modelica://Modelica.Math.BooleanVectors.enumerate\">enumerate</a>,
<a href=\"modelica://Modelica.Math.BooleanVectors.firstTrueIndex\">firstTrueIndex</a>,
<a href=\"modelica://Modelica.Math.BooleanVectors.index\">index</a>, and
<a href=\"modelica://Modelica.Math.BooleanVectors.oneTrue\">oneTrue</a>.
</p>
</html>"));
      end anyTrue;

      function countTrue "Returns the number of true entries in a Boolean vector"
        extends Modelica.Icons.Function;
        input Boolean b[:] "Boolean vector";
        output Integer n "Number of true entries";
      algorithm
        n := sum(if b[i] then 1 else 0 for i in 1:size(b, 1));
        annotation(
          Inline = true,
          Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
<b>countTrue</b>(b);
</pre></blockquote>

<h4>Description</h4>
<p>
This function returns the number of <b>true</b> entries in a Boolean vector b.
</p>

<h4>Example</h4>
<p><code>countTrue({false, true, false, true})</code> returns 2.</p>

<h4>See also</h4>
<p>
<a href=\"modelica://Modelica.Math.BooleanVectors.allTrue\">allTrue</a>,
<a href=\"modelica://Modelica.Math.BooleanVectors.anyTrue\">anyTrue</a>,
<a href=\"modelica://Modelica.Math.BooleanVectors.enumerate\">enumerate</a>,
<a href=\"modelica://Modelica.Math.BooleanVectors.firstTrueIndex\">firstTrueIndex</a>,
<a href=\"modelica://Modelica.Math.BooleanVectors.index\">index</a>, and
<a href=\"modelica://Modelica.Math.BooleanVectors.oneTrue\">oneTrue</a>.
</p>
</html>"));
      end countTrue;

      function enumerate "Enumerates the true entries in a Boolean vector (0 for false entries)"
        extends Modelica.Icons.Function;
        input Boolean b[:] "Boolean vector";
        output Integer enumerated[size(b, 1)] "Indices of the true entries (increasing order; 0 for false entries)";
      protected
        Integer count;
      algorithm
        count := 1;
        for i in 1:size(b, 1) loop
          if b[i] then
            enumerated[i] := count;
            count := count + 1;
          else
            enumerated[i] := 0;
          end if;
        end for;
        annotation(
          Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
<b>enumerate</b>(b);
</pre></blockquote>

<h4>Description</h4>
<p>
This function returns an integer vector that consecutively numbers
the <b>true</b> entries in a Boolean vector b.  The <b>false</b> entries are
indicated by 0.
</p>

<h4>Example</h4>
<p><code>enumerate({false, true, false, true})</code> returns <code>{0,1,0,2}</code>.</p>

<h4>See also</h4>
<p>
<a href=\"modelica://Modelica.Math.BooleanVectors.allTrue\">allTrue</a>,
<a href=\"modelica://Modelica.Math.BooleanVectors.anyTrue\">anyTrue</a>,
<a href=\"modelica://Modelica.Math.BooleanVectors.countTrue\">countTrue</a>,
<a href=\"modelica://Modelica.Math.BooleanVectors.firstTrueIndex\">firstTrueIndex</a>,
<a href=\"modelica://Modelica.Math.BooleanVectors.index\">index</a>, and
<a href=\"modelica://Modelica.Math.BooleanVectors.oneTrue\">oneTrue</a>.
</p>
</html>"));
      end enumerate;

      function firstTrueIndex "Returns the index of the first true element of a Boolean vector"
        extends Modelica.Icons.Function;
        input Boolean b[:];
        output Integer index;
      algorithm
        index := 0;
        for i in 1:size(b, 1) loop
          if b[i] then
            index := i;
            return;
          end if;
        end for;
        annotation(
          Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
<b>firstTrueIndex</b>(b);
</pre></blockquote>

<h4>Description</h4>
<p>
Returns the index of the first <b>true</b> element of the Boolean vector b.
If no element is <b>true</b> or b is an empty vector (i.e., size(b,1)=0) the
function returns 0.
</p>

<h4>Example</h4>
<blockquote><pre>
  Boolean b1[3] = {false, false, false};
  Boolean b2[3] = {false, true, false};
  Boolean b3[4] = {false, true, false, true};
  Integer r1, r2, r3;
<b>algorithm</b>
  r1 = firstTrueIndex(b1);  // r1 = 0
  r2 = firstTrueIndex(b2);  // r2 = 2
  r3 = firstTrueIndex(b3);  // r3 = 2
</pre></blockquote>

<h4>See also</h4>
<p>
<a href=\"modelica://Modelica.Math.BooleanVectors.allTrue\">allTrue</a>,
<a href=\"modelica://Modelica.Math.BooleanVectors.anyTrue\">anyTrue</a>,
<a href=\"modelica://Modelica.Math.BooleanVectors.countTrue\">countTrue</a>,
<a href=\"modelica://Modelica.Math.BooleanVectors.enumerate\">enumerate</a>,
<a href=\"modelica://Modelica.Math.BooleanVectors.index\">index</a>, and
<a href=\"modelica://Modelica.Math.BooleanVectors.oneTrue\">oneTrue</a>.
</p>
</html>"));
      end firstTrueIndex;

      function index "Returns the indices of the true entries of a Boolean vector"
        extends Modelica.Icons.Function;
        input Boolean b[:] "Boolean vector";
        output Integer indices[countTrue(b)] "Indices of the true entries";
      protected
        Integer count;
      algorithm
        count := 1;
        for i in 1:size(b, 1) loop
          if b[i] then
            indices[count] := i;
            count := count + 1;
          end if;
        end for;
        annotation(
          Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
<b>index</b>(b);
</pre></blockquote>

<h4>Description</h4>
<p>
This function returns an integer vector that contains indices to the
<b>true</b> entries in a Boolean vector b.  The number of entries in
the integer vector is the number of <b>true</b> entries in b.
</p>

<h4>Example</h4>
<code>index({false, true, false, true})</code> returns <code>{2,4}</code>.

  <h4>See also</h4>
<p>
<a href=\"modelica://Modelica.Math.BooleanVectors.allTrue\">allTrue</a>,
<a href=\"modelica://Modelica.Math.BooleanVectors.anyTrue\">anyTrue</a>,
<a href=\"modelica://Modelica.Math.BooleanVectors.countTrue\">countTrue</a>,
<a href=\"modelica://Modelica.Math.BooleanVectors.enumerate\">enumerate</a>,
<a href=\"modelica://Modelica.Math.BooleanVectors.firstTrueIndex\">firstTrueIndex</a>, and
<a href=\"modelica://Modelica.Math.BooleanVectors.oneTrue\">oneTrue</a>.
</p>
</html>"));
      end index;

      function oneTrue "Returns true, if exactly one element of the Boolean input vector is true (\"xor\")"
        extends Modelica.Icons.Function;
        input Boolean b[:];
        output Boolean result;
      algorithm
        result := countTrue(b) == 1;
        annotation(
          Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
<b>oneTrue</b>(b);
</pre></blockquote>

<h4>Description</h4>
<p>
Returns <b>true</b> if exactly one element of the input Boolean vector b is <b>true</b>.
Otherwise the function returns <b>false</b>. If b is an empty vector,
i.e., size(b,1)=0, the function returns <b>false</b>.
</p>

<h4>Example</h4>
<blockquote><pre>
  Boolean b1[3] = {false, false, false};
  Boolean b2[3] = {false, true, false};
  Boolean b3[3] = {false, true, true};
  Boolean r1, r2, r3;
<b>algorithm</b>
  r1 = oneTrue(b1);  // r1 = false
  r2 = oneTrue(b2);  // r2 = true
  r3 = oneTrue(b3);  // r3 = false
</pre></blockquote>

<h4>See also</h4>
<p>
<a href=\"modelica://Modelica.Math.BooleanVectors.allTrue\">allTrue</a>,
<a href=\"modelica://Modelica.Math.BooleanVectors.anyTrue\">anyTrue</a>,
<a href=\"modelica://Modelica.Math.BooleanVectors.countTrue\">countTrue</a>,
<a href=\"modelica://Modelica.Math.BooleanVectors.enumerate\">enumerate</a>,
<a href=\"modelica://Modelica.Math.BooleanVectors.firstTrueIndex\">firstTrueIndex</a>, and
<a href=\"modelica://Modelica.Math.BooleanVectors.index\">index</a>.
</p>
</html>"));
      end oneTrue;
      annotation(
        Documentation(info = "<html>
<p>
This library provides functions operating on vectors that have
a Boolean vector as input argument.
</p>
</html>"));
    end BooleanVectors;

    function isEqual "Determine if two Real scalars are numerically identical"
      extends Modelica.Icons.Function;
      input Real s1 "First scalar";
      input Real s2 "Second scalar";
      input Real eps(min = 0) = 0 "The two scalars are identical if abs(s1-s2) <= eps";
      output Boolean result "= true, if scalars are identical";
    algorithm
      result := abs(s1 - s2) <= eps;
      annotation(
        Inline = true,
        Documentation(info = "<HTML>
<h4>Syntax</h4>
<blockquote><pre>
Math.<b>isEqual</b>(s1, s2);
Math.<b>isEqual</b>(s1, s2, eps=0);
</pre></blockquote>
<h4>Description</h4>
<p>
The function call \"<code>Math.isEqual(s1, s2)</code>\" returns <b>true</b>,
if the two Real scalars s1 and s2 are identical. Otherwise the function
returns <b>false</b>. The equality check is performed by
\"abs(s1-s2) &le; eps\", where \"eps\"
can be provided as third argument of the function. Default is \"eps = 0\".
</p>
<h4>Example</h4>
<blockquote><pre>
  Real s1 = 2.0;
  Real s2 = 2.0;
  Real s3 = 2.000001;
  Boolean result;
<b>algorithm</b>
  result := Math.isEqual(s1,s2);     // = <b>true</b>
  result := Math.isEqual(s1,s3);     // = <b>false</b>
  result := Math.isEqual(s1,s3,0.1); // = <b>true</b>
</pre></blockquote>
<h4>See also</h4>
<p>
<a href=\"modelica://Modelica.Math.Vectors.isEqual\">Vectors.isEqual</a>,
<a href=\"modelica://Modelica.Math.Matrices.isEqual\">Matrices.isEqual</a>,
<a href=\"modelica://Modelica.Utilities.Strings.isEqual\">Strings.isEqual</a>
</p>
</HTML>"));
    end isEqual;

    function isPowerOf2 "Determine if the integer input is a power of 2"
      extends Modelica.Icons.Function;
      input Integer i(min = 1) "Integer scalar";
      output Boolean result "= true, if integer scalar is a power of 2";
    algorithm
      assert(i >= 1, "Integer input to isPowerOf2 has to be >= 1");
      result := i == 2 ^ integer(log(i) / log(2) + 0.5);
      annotation(
        Inline = true,
        Documentation(info = "<HTML>
<h4>Syntax</h4>
<blockquote><pre>
Math.<b>isPowerOf2</b>(i);
</pre></blockquote>
<h4>Description</h4>
<p>
The function call \"<code>Math.isPowerOf2(i)</code>\" returns <b>true</b>,
if the Integer input i is a power of 2. Otherwise the function
returns <b>false</b>. The Integer input has to be &gt;=1.
</p>
<h4>Example</h4>
<blockquote><pre>
  Integer i1 = 1;
  Integer i2 = 4;
  Integer i3 = 9;
  Boolean result;
<b>algorithm</b>
  result := Math.isPowerOf2(i1);     // = <b>true</b> 2^0
  result := Math.isPowerOf2(i2);     // = <b>true</b> 2^2
  result := Math.isPowerOf2(i3);     // = <b>false</b>
</pre></blockquote>
</HTML>"));
    end isPowerOf2;

    function sin "Sine"
      extends Modelica.Math.Icons.AxisLeft;
      input Modelica.SIunits.Angle u;
      output Real y;

      external "builtin" y = sin(u);
      annotation(
        Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-90, 0}, {68, 0}}, color = {192, 192, 192}), Polygon(points = {{90, 0}, {68, 8}, {68, -8}, {90, 0}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-80, 0}, {-68.7, 34.2}, {-61.5, 53.1}, {-55.1, 66.4}, {-49.4, 74.6}, {-43.8, 79.1}, {-38.2, 79.8}, {-32.6, 76.6}, {-26.9, 69.7}, {-21.3, 59.4}, {-14.9, 44.1}, {-6.83, 21.2}, {10.1, -30.8}, {17.3, -50.2}, {23.7, -64.2}, {29.3, -73.1}, {35, -78.4}, {40.6, -80}, {46.2, -77.6}, {51.9, -71.5}, {57.5, -61.9}, {63.9, -47.2}, {72, -24.8}, {80, 0}}, color = {0, 0, 0}), Text(extent = {{12, 84}, {84, 36}}, lineColor = {192, 192, 192}, textString = "sin")}),
        Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-100, 0}, {84, 0}}, color = {95, 95, 95}), Polygon(points = {{100, 0}, {84, 6}, {84, -6}, {100, 0}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Line(points = {{-80, 0}, {-68.7, 34.2}, {-61.5, 53.1}, {-55.1, 66.4}, {-49.4, 74.6}, {-43.8, 79.1}, {-38.2, 79.8}, {-32.6, 76.6}, {-26.9, 69.7}, {-21.3, 59.4}, {-14.9, 44.1}, {-6.83, 21.2}, {10.1, -30.8}, {17.3, -50.2}, {23.7, -64.2}, {29.3, -73.1}, {35, -78.4}, {40.6, -80}, {46.2, -77.6}, {51.9, -71.5}, {57.5, -61.9}, {63.9, -47.2}, {72, -24.8}, {80, 0}}, color = {0, 0, 255}, thickness = 0.5), Text(extent = {{-105, 72}, {-85, 88}}, textString = "1", lineColor = {0, 0, 255}), Text(extent = {{70, 25}, {90, 5}}, textString = "2*pi", lineColor = {0, 0, 255}), Text(extent = {{-103, -72}, {-83, -88}}, textString = "-1", lineColor = {0, 0, 255}), Text(extent = {{82, -6}, {102, -26}}, lineColor = {95, 95, 95}, textString = "u"), Line(points = {{-80, 80}, {-28, 80}}, color = {175, 175, 175}, smooth = Smooth.None), Line(points = {{-80, -80}, {50, -80}}, color = {175, 175, 175}, smooth = Smooth.None)}),
        Documentation(info = "<html>
<p>
This function returns y = sin(u), with -&infin; &lt; u &lt; &infin;:
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Math/sin.png\">
</p>
</html>"));
    end sin;

    function cos "Cosine"
      extends Modelica.Math.Icons.AxisLeft;
      input SI.Angle u;
      output Real y;

      external "builtin" y = cos(u);
      annotation(
        Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-90, 0}, {68, 0}}, color = {192, 192, 192}), Polygon(points = {{90, 0}, {68, 8}, {68, -8}, {90, 0}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-80, 80}, {-74.4, 78.1}, {-68.7, 72.3}, {-63.1, 63}, {-56.7, 48.7}, {-48.6, 26.6}, {-29.3, -32.5}, {-22.1, -51.7}, {-15.7, -65.3}, {-10.1, -73.8}, {-4.42, -78.8}, {1.21, -79.9}, {6.83, -77.1}, {12.5, -70.6}, {18.1, -60.6}, {24.5, -45.7}, {32.6, -23}, {50.3, 31.3}, {57.5, 50.7}, {63.9, 64.6}, {69.5, 73.4}, {75.2, 78.6}, {80, 80}}, color = {0, 0, 0}), Text(extent = {{-36, 82}, {36, 34}}, lineColor = {192, 192, 192}, textString = "cos")}),
        Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-103, 72}, {-83, 88}}, textString = "1", lineColor = {0, 0, 255}), Text(extent = {{-103, -72}, {-83, -88}}, textString = "-1", lineColor = {0, 0, 255}), Text(extent = {{70, 25}, {90, 5}}, textString = "2*pi", lineColor = {0, 0, 255}), Line(points = {{-100, 0}, {84, 0}}, color = {95, 95, 95}), Polygon(points = {{98, 0}, {82, 6}, {82, -6}, {98, 0}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Line(points = {{-80, 80}, {-74.4, 78.1}, {-68.7, 72.3}, {-63.1, 63}, {-56.7, 48.7}, {-48.6, 26.6}, {-29.3, -32.5}, {-22.1, -51.7}, {-15.7, -65.3}, {-10.1, -73.8}, {-4.42, -78.8}, {1.21, -79.9}, {6.83, -77.1}, {12.5, -70.6}, {18.1, -60.6}, {24.5, -45.7}, {32.6, -23}, {50.3, 31.3}, {57.5, 50.7}, {63.9, 64.6}, {69.5, 73.4}, {75.2, 78.6}, {80, 80}}, color = {0, 0, 255}, thickness = 0.5), Text(extent = {{78, -6}, {98, -26}}, lineColor = {95, 95, 95}, textString = "u"), Line(points = {{-80, -80}, {18, -80}}, color = {175, 175, 175}, smooth = Smooth.None)}),
        Documentation(info = "<html>
<p>
This function returns y = cos(u), with -&infin; &lt; u &lt; &infin;:
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Math/cos.png\">
</p>
</html>"));
    end cos;

    function tan "Tangent (u shall not be -pi/2, pi/2, 3*pi/2, ...)"
      extends Modelica.Math.Icons.AxisCenter;
      input SI.Angle u;
      output Real y;

      external "builtin" y = tan(u);
      annotation(
        Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-90, 0}, {68, 0}}, color = {192, 192, 192}), Polygon(points = {{90, 0}, {68, 8}, {68, -8}, {90, 0}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-80, -80}, {-78.4, -68.4}, {-76.8, -59.7}, {-74.4, -50}, {-71.2, -40.9}, {-67.1, -33}, {-60.7, -24.8}, {-51.1, -17.2}, {-35.8, -9.98}, {-4.42, -1.07}, {33.4, 9.12}, {49.4, 16.2}, {59.1, 23.2}, {65.5, 30.6}, {70.4, 39.1}, {73.6, 47.4}, {76, 56.1}, {77.6, 63.8}, {80, 80}}, color = {0, 0, 0}), Text(extent = {{-90, 72}, {-18, 24}}, lineColor = {192, 192, 192}, textString = "tan")}),
        Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-37, -72}, {-17, -88}}, textString = "-5.8", lineColor = {0, 0, 255}), Text(extent = {{-33, 86}, {-13, 70}}, textString = " 5.8", lineColor = {0, 0, 255}), Text(extent = {{68, -13}, {88, -33}}, textString = "1.4", lineColor = {0, 0, 255}), Line(points = {{-100, 0}, {84, 0}}, color = {95, 95, 95}), Polygon(points = {{98, 0}, {82, 6}, {82, -6}, {98, 0}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Line(points = {{-80, -80}, {-78.4, -68.4}, {-76.8, -59.7}, {-74.4, -50}, {-71.2, -40.9}, {-67.1, -33}, {-60.7, -24.8}, {-51.1, -17.2}, {-35.8, -9.98}, {-4.42, -1.07}, {33.4, 9.12}, {49.4, 16.2}, {59.1, 23.2}, {65.5, 30.6}, {70.4, 39.1}, {73.6, 47.4}, {76, 56.1}, {77.6, 63.8}, {80, 80}}, color = {0, 0, 255}, thickness = 0.5), Text(extent = {{82, 22}, {102, 2}}, lineColor = {95, 95, 95}, textString = "u"), Line(points = {{0, 80}, {86, 80}}, color = {175, 175, 175}, smooth = Smooth.None), Line(points = {{80, 88}, {80, -16}}, color = {175, 175, 175}, smooth = Smooth.None)}),
        Documentation(info = "<html>
<p>
This function returns y = tan(u), with -&infin; &lt; u &lt; &infin;
(if u is a multiple of (2n-1)*pi/2, y = tan(u) is +/- infinity).
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Math/tan.png\">
</p>
</html>"));
    end tan;

    function asin "Inverse sine (-1 <= u <= 1)"
      extends Modelica.Math.Icons.AxisCenter;
      input Real u;
      output SI.Angle y;

      external "builtin" y = asin(u);
      annotation(
        Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-90, 0}, {68, 0}}, color = {192, 192, 192}), Polygon(points = {{90, 0}, {68, 8}, {68, -8}, {90, 0}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-80, -80}, {-79.2, -72.8}, {-77.6, -67.5}, {-73.6, -59.4}, {-66.3, -49.8}, {-53.5, -37.3}, {-30.2, -19.7}, {37.4, 24.8}, {57.5, 40.8}, {68.7, 52.7}, {75.2, 62.2}, {77.6, 67.5}, {80, 80}}, color = {0, 0, 0}), Text(extent = {{-88, 78}, {-16, 30}}, lineColor = {192, 192, 192}, textString = "asin")}),
        Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-40, -72}, {-15, -88}}, textString = "-pi/2", lineColor = {0, 0, 255}), Text(extent = {{-38, 88}, {-13, 72}}, textString = " pi/2", lineColor = {0, 0, 255}), Text(extent = {{68, -9}, {88, -29}}, textString = "+1", lineColor = {0, 0, 255}), Text(extent = {{-90, 21}, {-70, 1}}, textString = "-1", lineColor = {0, 0, 255}), Line(points = {{-100, 0}, {84, 0}}, color = {95, 95, 95}), Polygon(points = {{98, 0}, {82, 6}, {82, -6}, {98, 0}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Line(points = {{-80, -80}, {-79.2, -72.8}, {-77.6, -67.5}, {-73.6, -59.4}, {-66.3, -49.8}, {-53.5, -37.3}, {-30.2, -19.7}, {37.4, 24.8}, {57.5, 40.8}, {68.7, 52.7}, {75.2, 62.2}, {77.6, 67.5}, {80, 80}}, color = {0, 0, 255}, thickness = 0.5), Text(extent = {{82, 24}, {102, 4}}, lineColor = {95, 95, 95}, textString = "u"), Line(points = {{0, 80}, {86, 80}}, color = {175, 175, 175}, smooth = Smooth.None), Line(points = {{80, 86}, {80, -10}}, color = {175, 175, 175}, smooth = Smooth.None)}),
        Documentation(info = "<html>
<p>
This function returns y = asin(u), with -1 &le; u &le; +1:
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Math/asin.png\">
</p>
</html>"));
    end asin;

    function acos "Inverse cosine (-1 <= u <= 1)"
      extends Modelica.Math.Icons.AxisCenter;
      input Real u;
      output SI.Angle y;

      external "builtin" y = acos(u);
      annotation(
        Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-90, -80}, {68, -80}}, color = {192, 192, 192}), Polygon(points = {{90, -80}, {68, -72}, {68, -88}, {90, -80}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-80, 80}, {-79.2, 72.8}, {-77.6, 67.5}, {-73.6, 59.4}, {-66.3, 49.8}, {-53.5, 37.3}, {-30.2, 19.7}, {37.4, -24.8}, {57.5, -40.8}, {68.7, -52.7}, {75.2, -62.2}, {77.6, -67.5}, {80, -80}}, color = {0, 0, 0}), Text(extent = {{-86, -14}, {-14, -62}}, lineColor = {192, 192, 192}, textString = "acos")}),
        Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-100, -80}, {84, -80}}, color = {95, 95, 95}), Polygon(points = {{98, -80}, {82, -74}, {82, -86}, {98, -80}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Line(points = {{-80, 80}, {-79.2, 72.8}, {-77.6, 67.5}, {-73.6, 59.4}, {-66.3, 49.8}, {-53.5, 37.3}, {-30.2, 19.7}, {37.4, -24.8}, {57.5, -40.8}, {68.7, -52.7}, {75.2, -62.2}, {77.6, -67.5}, {80, -80}}, color = {0, 0, 255}, thickness = 0.5), Text(extent = {{-30, 88}, {-5, 72}}, textString = " pi", lineColor = {0, 0, 255}), Text(extent = {{-94, -57}, {-74, -77}}, textString = "-1", lineColor = {0, 0, 255}), Text(extent = {{60, -81}, {80, -101}}, textString = "+1", lineColor = {0, 0, 255}), Text(extent = {{82, -56}, {102, -76}}, lineColor = {95, 95, 95}, textString = "u"), Line(points = {{-2, 80}, {84, 80}}, color = {175, 175, 175}, smooth = Smooth.None), Line(points = {{80, 82}, {80, -86}}, color = {175, 175, 175}, smooth = Smooth.None)}),
        Documentation(info = "<html>
<p>
This function returns y = acos(u), with -1 &le; u &le; +1:
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Math/acos.png\">
</p>
</html>"));
    end acos;

    function atan "Inverse tangent"
      extends Modelica.Math.Icons.AxisCenter;
      input Real u;
      output SI.Angle y;

      external "builtin" y = atan(u);
      annotation(
        Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-90, 0}, {68, 0}}, color = {192, 192, 192}), Polygon(points = {{90, 0}, {68, 8}, {68, -8}, {90, 0}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-80, -80}, {-52.7, -75.2}, {-37.4, -69.7}, {-26.9, -63}, {-19.7, -55.2}, {-14.1, -45.8}, {-10.1, -36.4}, {-6.03, -23.9}, {-1.21, -5.06}, {5.23, 21}, {9.25, 34.1}, {13.3, 44.2}, {18.1, 52.9}, {24.5, 60.8}, {33.4, 67.6}, {47, 73.6}, {69.5, 78.6}, {80, 80}}, color = {0, 0, 0}), Text(extent = {{-86, 68}, {-14, 20}}, lineColor = {192, 192, 192}, textString = "atan")}),
        Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-100, 0}, {84, 0}}, color = {95, 95, 95}), Polygon(points = {{96, 0}, {80, 6}, {80, -6}, {96, 0}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Line(points = {{-80, -80}, {-52.7, -75.2}, {-37.4, -69.7}, {-26.9, -63}, {-19.7, -55.2}, {-14.1, -45.8}, {-10.1, -36.4}, {-6.03, -23.9}, {-1.21, -5.06}, {5.23, 21}, {9.25, 34.1}, {13.3, 44.2}, {18.1, 52.9}, {24.5, 60.8}, {33.4, 67.6}, {47, 73.6}, {65, 77}, {80, 78}}, color = {0, 0, 255}, thickness = 0.5), Text(extent = {{-34, 87}, {-14, 74}}, textString = "pi/2", lineColor = {0, 0, 255}), Text(extent = {{-32, -71}, {-12, -91}}, textString = "-pi/2", lineColor = {0, 0, 255}), Text(extent = {{84, -4}, {104, -24}}, lineColor = {95, 95, 95}, textString = "u"), Line(points = {{-2, 80}, {84, 80}}, color = {175, 175, 175}, smooth = Smooth.None)}),
        Documentation(info = "<html>
<p>
This function returns y = atan(u), with -&infin; &lt; u &lt; &infin;:
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Math/atan.png\">
</p>
</html>"));
    end atan;

    function atan2 "Four quadrant inverse tangent"
      extends Modelica.Math.Icons.AxisCenter;
      input Real u1;
      input Real u2;
      output SI.Angle y;

      external "builtin" y = atan2(u1, u2);
      annotation(
        Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-90, 0}, {68, 0}}, color = {192, 192, 192}), Polygon(points = {{90, 0}, {68, 8}, {68, -8}, {90, 0}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{0, -80}, {8.93, -67.2}, {17.1, -59.3}, {27.3, -53.6}, {42.1, -49.4}, {69.9, -45.8}, {80, -45.1}}, color = {0, 0, 0}), Line(points = {{-80, -34.9}, {-46.1, -31.4}, {-29.4, -27.1}, {-18.3, -21.5}, {-10.3, -14.5}, {-2.03, -3.17}, {7.97, 11.6}, {15.5, 19.4}, {24.3, 25}, {39, 30}, {62.1, 33.5}, {80, 34.9}}, color = {0, 0, 0}), Line(points = {{-80, 45.1}, {-45.9, 48.7}, {-29.1, 52.9}, {-18.1, 58.6}, {-10.2, 65.8}, {-1.82, 77.2}, {0, 80}}, color = {0, 0, 0}), Text(extent = {{-90, -46}, {-18, -94}}, lineColor = {192, 192, 192}, textString = "atan2")}),
        Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-100, 0}, {84, 0}}, color = {95, 95, 95}), Polygon(points = {{96, 0}, {80, 6}, {80, -6}, {96, 0}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Line(points = {{0, -80}, {8.93, -67.2}, {17.1, -59.3}, {27.3, -53.6}, {42.1, -49.4}, {69.9, -45.8}, {80, -45.1}}, color = {0, 0, 255}, thickness = 0.5), Line(points = {{-80, -34.9}, {-46.1, -31.4}, {-29.4, -27.1}, {-18.3, -21.5}, {-10.3, -14.5}, {-2.03, -3.17}, {7.97, 11.6}, {15.5, 19.4}, {24.3, 25}, {39, 30}, {62.1, 33.5}, {80, 34.9}}, color = {0, 0, 255}, thickness = 0.5), Line(points = {{-80, 45.1}, {-45.9, 48.7}, {-29.1, 52.9}, {-18.1, 58.6}, {-10.2, 65.8}, {-1.82, 77.2}, {0, 80}}, color = {0, 0, 255}, thickness = 0.5), Text(extent = {{-32, 89}, {-10, 74}}, textString = "pi", lineColor = {0, 0, 255}), Text(extent = {{-32, -72}, {-4, -88}}, textString = "-pi", lineColor = {0, 0, 255}), Text(extent = {{0, 55}, {20, 42}}, textString = "pi/2", lineColor = {0, 0, 255}), Line(points = {{0, 40}, {-8, 40}}, color = {192, 192, 192}), Line(points = {{0, -40}, {-8, -40}}, color = {192, 192, 192}), Text(extent = {{0, -23}, {20, -42}}, textString = "-pi/2", lineColor = {0, 0, 255}), Text(extent = {{62, -4}, {94, -26}}, lineColor = {95, 95, 95}, textString = "u1, u2"), Line(points = {{-88, 40}, {86, 40}}, color = {175, 175, 175}, smooth = Smooth.None), Line(points = {{-86, -40}, {86, -40}}, color = {175, 175, 175}, smooth = Smooth.None)}),
        Documentation(info = "<HTML>
<p>
This function returns y = atan2(u1,u2) such that tan(y) = u1/u2 and
y is in the range -pi &lt; y &le; pi. u2 may be zero, provided
u1 is not zero. Usually u1, u2 is provided in such a form that
u1 = sin(y) and u2 = cos(y):
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Math/atan2.png\">
</p>

</html>"));
    end atan2;

    function atan3 "Four quadrant inverse tangent (select solution that is closest to given angle y0)"
      import Modelica.Math;
      extends Modelica.Math.Icons.AxisCenter;
      input Real u1;
      input Real u2;
      input Modelica.SIunits.Angle y0 = 0 "y shall be in the range: -pi < y-y0 <= pi";
      output Modelica.SIunits.Angle y;
    protected
      constant Real pi = Modelica.Constants.pi;
      constant Real pi2 = 2 * pi;
      Real w;
    algorithm
      w := Math.atan2(u1, u2);
      if y0 == 0 then
        y := w;
      else
        y := w + pi2 * integer((pi + y0 - w) / pi2);
      end if;
// For the default (y0 = 0), exactly the same result as with atan2(..) is returned
/* -pi < y - y0 <= pi
       -pi < w + 2*pi*N - y0 <= pi
       (-pi+y0-w)/(2*pi) < N <= (pi+y0-w)/(2*pi)
       -> N := integer( (pi+y0-w)/(2*pi) )
    */
      annotation(
        Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-90, 0}, {68, 0}}, color = {192, 192, 192}), Polygon(points = {{90, 0}, {68, 8}, {68, -8}, {90, 0}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{0, -80}, {8.93, -67.2}, {17.1, -59.3}, {27.3, -53.6}, {42.1, -49.4}, {69.9, -45.8}, {80, -45.1}}, color = {0, 0, 0}), Line(points = {{-80, -34.9}, {-46.1, -31.4}, {-29.4, -27.1}, {-18.3, -21.5}, {-10.3, -14.5}, {-2.03, -3.17}, {7.97, 11.6}, {15.5, 19.4}, {24.3, 25}, {39, 30}, {62.1, 33.5}, {80, 34.9}}, color = {0, 0, 0}), Line(points = {{-80, 45.1}, {-45.9, 48.7}, {-29.1, 52.9}, {-18.1, 58.6}, {-10.2, 65.8}, {-1.82, 77.2}, {0, 80}}, color = {0, 0, 0}), Text(extent = {{-90, -46}, {-18, -94}}, lineColor = {192, 192, 192}, textString = "atan3")}),
        Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-100, -86}, {84, -86}}, color = {95, 95, 95}), Polygon(points = {{98, -86}, {82, -80}, {82, -92}, {98, -86}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Line(points = {{0, -80}, {8.93, -67.2}, {17.1, -59.3}, {27.3, -53.6}, {42.1, -49.4}, {69.9, -45.8}, {80, -45.1}}, color = {0, 0, 255}, thickness = 0.5), Line(points = {{-80, -34.9}, {-46.1, -31.4}, {-29.4, -27.1}, {-18.3, -21.5}, {-10.3, -14.5}, {-2.03, -3.17}, {7.97, 11.6}, {15.5, 19.4}, {24.3, 25}, {39, 30}, {62.1, 33.5}, {80, 34.9}}, color = {0, 0, 255}, thickness = 0.5), Line(points = {{-80, 45.1}, {-45.9, 48.7}, {-29.1, 52.9}, {-18.1, 58.6}, {-10.2, 65.8}, {-1.82, 77.2}, {0, 80}}, color = {0, 0, 255}, thickness = 0.5), Text(extent = {{-56, 82}, {-12, 72}}, textString = "(2*N-1)*pi", lineColor = {0, 0, 255}), Text(extent = {{-52, -72}, {-10, -88}}, textString = "(2*N-3)*pi", lineColor = {0, 0, 255}), Line(points = {{0, 40}, {-8, 40}}, color = {192, 192, 192}), Line(points = {{0, -40}, {-8, -40}}, color = {192, 192, 192}), Text(extent = {{38, -68}, {78, -84}}, lineColor = {95, 95, 95}, textString = "u1, u2, y0"), Line(points = {{-84, 40}, {88, 40}}, color = {175, 175, 175}, smooth = Smooth.None), Line(points = {{-84, -40}, {88, -40}}, color = {175, 175, 175}, smooth = Smooth.None)}),
        Documentation(info = "<HTML>
<p>
This function returns y = <b>atan3</b>(u1,u2,y0) such that
<b>tan</b>(y) = u1/u2 and
y is in the range: -pi &lt; y-y0 &le; pi.<br>
u2 may be zero, provided u1 is not zero. The difference to
Modelica.Math.atan2(..) is the optional third argument y0 that
allows to specify which of the infinite many solutions
shall be returned:
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Math/atan3.png\">
</p>

<p>
Note, for the default case (y0=0), exactly the same result as with atan2(..)
is returned.
</p>
</html>"));
    end atan3;

    function sinh "Hyperbolic sine"
      extends Modelica.Math.Icons.AxisCenter;
      input Real u;
      output Real y;

      external "builtin" y = sinh(u);
      annotation(
        Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-90, 0}, {68, 0}}, color = {192, 192, 192}), Polygon(points = {{90, 0}, {68, 8}, {68, -8}, {90, 0}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-80, -80}, {-76, -65.4}, {-71.2, -51.4}, {-65.5, -38.8}, {-59.1, -28.1}, {-51.1, -18.7}, {-41.4, -11.4}, {-27.7, -5.5}, {-4.42, -0.653}, {24.5, 4.57}, {39, 10.1}, {49.4, 17.2}, {57.5, 25.9}, {63.9, 35.8}, {69.5, 47.4}, {74.4, 60.4}, {78.4, 73.8}, {80, 80}}, color = {0, 0, 0}), Text(extent = {{-88, 80}, {-16, 32}}, lineColor = {192, 192, 192}, textString = "sinh")}),
        Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-100, 0}, {84, 0}}, color = {95, 95, 95}), Polygon(points = {{98, 0}, {82, 6}, {82, -6}, {98, 0}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Line(points = {{-80, -80}, {-76, -65.4}, {-71.2, -51.4}, {-65.5, -38.8}, {-59.1, -28.1}, {-51.1, -18.7}, {-41.4, -11.4}, {-27.7, -5.5}, {-4.42, -0.653}, {24.5, 4.57}, {39, 10.1}, {49.4, 17.2}, {57.5, 25.9}, {63.9, 35.8}, {69.5, 47.4}, {74.4, 60.4}, {78.4, 73.8}, {80, 80}}, color = {0, 0, 255}, thickness = 0.5), Text(extent = {{-31, 72}, {-11, 88}}, textString = "27", lineColor = {0, 0, 255}), Text(extent = {{-35, -88}, {-15, -72}}, textString = "-27", lineColor = {0, 0, 255}), Text(extent = {{68, -7}, {88, -27}}, textString = "4", lineColor = {0, 0, 255}), Text(extent = {{-98, 21}, {-78, 1}}, textString = "-4", lineColor = {0, 0, 255}), Text(extent = {{80, 26}, {100, 6}}, lineColor = {95, 95, 95}, textString = "u"), Line(points = {{0, 80}, {86, 80}}, color = {175, 175, 175}, smooth = Smooth.None), Line(points = {{80, 84}, {80, -6}}, color = {175, 175, 175}, smooth = Smooth.None)}),
        Documentation(info = "<html>
<p>
This function returns y = sinh(u), with -&infin; &lt; u &lt; &infin;:
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Math/sinh.png\">
</p>
</html>"));
    end sinh;

    function cosh "Hyperbolic cosine"
      extends Modelica.Math.Icons.AxisCenter;
      input Real u;
      output Real y;

      external "builtin" y = cosh(u);
      annotation(
        Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-90, -86.083}, {68, -86.083}}, color = {192, 192, 192}), Polygon(points = {{90, -86.083}, {68, -78.083}, {68, -94.083}, {90, -86.083}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-80, 80}, {-77.6, 61.1}, {-74.4, 39.3}, {-71.2, 20.7}, {-67.1, 1.29}, {-63.1, -14.6}, {-58.3, -29.8}, {-52.7, -43.5}, {-46.2, -55.1}, {-39, -64.3}, {-30.2, -71.7}, {-18.9, -77.1}, {-4.42, -79.9}, {10.9, -79.1}, {23.7, -75.2}, {34.2, -68.7}, {42.2, -60.6}, {48.6, -51.2}, {54.3, -40}, {59.1, -27.5}, {63.1, -14.6}, {67.1, 1.29}, {71.2, 20.7}, {74.4, 39.3}, {77.6, 61.1}, {80, 80}}, color = {0, 0, 0}), Text(extent = {{4, 66}, {66, 20}}, lineColor = {192, 192, 192}, textString = "cosh")}),
        Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-100, -84.083}, {84, -84.083}}, color = {95, 95, 95}), Polygon(points = {{98, -84.083}, {82, -78.083}, {82, -90.083}, {98, -84.083}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Line(points = {{-80, 80}, {-77.6, 61.1}, {-74.4, 39.3}, {-71.2, 20.7}, {-67.1, 1.29}, {-63.1, -14.6}, {-58.3, -29.8}, {-52.7, -43.5}, {-46.2, -55.1}, {-39, -64.3}, {-30.2, -71.7}, {-18.9, -77.1}, {-4.42, -79.9}, {10.9, -79.1}, {23.7, -75.2}, {34.2, -68.7}, {42.2, -60.6}, {48.6, -51.2}, {54.3, -40}, {59.1, -27.5}, {63.1, -14.6}, {67.1, 1.29}, {71.2, 20.7}, {74.4, 39.3}, {77.6, 61.1}, {80, 80}}, color = {0, 0, 255}, thickness = 0.5), Text(extent = {{-31, 72}, {-11, 88}}, textString = "27", lineColor = {0, 0, 255}), Text(extent = {{64, -83}, {84, -103}}, textString = "4", lineColor = {0, 0, 255}), Text(extent = {{-94, -63}, {-74, -83}}, textString = "-4", lineColor = {0, 0, 255}), Text(extent = {{80, -60}, {100, -80}}, lineColor = {95, 95, 95}, textString = "u"), Line(points = {{0, 80}, {88, 80}}, color = {175, 175, 175}, smooth = Smooth.None), Line(points = {{80, 84}, {80, -90}}, color = {175, 175, 175}, smooth = Smooth.None)}),
        Documentation(info = "<html>
<p>
This function returns y = cosh(u), with -&infin; &lt; u &lt; &infin;:
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Math/cosh.png\">
</p>
</html>"));
    end cosh;

    function tanh "Hyperbolic tangent"
      extends Modelica.Math.Icons.AxisCenter;
      input Real u;
      output Real y;

      external "builtin" y = tanh(u);
      annotation(
        Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-90, 0}, {68, 0}}, color = {192, 192, 192}), Polygon(points = {{90, 0}, {68, 8}, {68, -8}, {90, 0}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-80, -80}, {-47.8, -78.7}, {-35.8, -75.7}, {-27.7, -70.6}, {-22.1, -64.2}, {-17.3, -55.9}, {-12.5, -44.3}, {-7.64, -29.2}, {-1.21, -4.82}, {6.83, 26.3}, {11.7, 42}, {16.5, 54.2}, {21.3, 63.1}, {26.9, 69.9}, {34.2, 75}, {45.4, 78.4}, {72, 79.9}, {80, 80}}, color = {0, 0, 0}), Text(extent = {{-88, 72}, {-16, 24}}, lineColor = {192, 192, 192}, textString = "tanh")}),
        Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-100, 0}, {84, 0}}, color = {95, 95, 95}), Polygon(points = {{96, 0}, {80, 6}, {80, -6}, {96, 0}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Line(points = {{-80, -80.5}, {-47.8, -79.2}, {-35.8, -76.2}, {-27.7, -71.1}, {-22.1, -64.7}, {-17.3, -56.4}, {-12.5, -44.8}, {-7.64, -29.7}, {-1.21, -5.32}, {6.83, 25.8}, {11.7, 41.5}, {16.5, 53.7}, {21.3, 62.6}, {26.9, 69.4}, {34.2, 74.5}, {45.4, 77.9}, {72, 79.4}, {80, 79.5}}, color = {0, 0, 255}, thickness = 0.5), Text(extent = {{-29, 72}, {-9, 88}}, textString = "1", lineColor = {0, 0, 255}), Text(extent = {{3, -72}, {23, -88}}, textString = "-1", lineColor = {0, 0, 255}), Text(extent = {{82, -2}, {102, -22}}, lineColor = {95, 95, 95}, textString = "u"), Line(points = {{0, 80}, {88, 80}}, color = {175, 175, 175}, smooth = Smooth.None)}),
        Documentation(info = "<html>
<p>
This function returns y = tanh(u), with -&infin; &lt; u &lt; &infin;:
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Math/tanh.png\">
</p>
</html>"));
    end tanh;

    function asinh "Inverse of sinh (area hyperbolic sine)"
      extends Modelica.Math.Icons.AxisCenter;
      input Real u;
      output Real y;
    algorithm
      y := Modelica.Math.log(u + sqrt(u * u + 1));
      annotation(
        Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-90, 0}, {68, 0}}, color = {192, 192, 192}), Polygon(points = {{90, 0}, {68, 8}, {68, -8}, {90, 0}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-80, -80}, {-56.7, -68.4}, {-39.8, -56.8}, {-26.9, -44.7}, {-17.3, -32.4}, {-9.25, -19}, {9.25, 19}, {17.3, 32.4}, {26.9, 44.7}, {39.8, 56.8}, {56.7, 68.4}, {80, 80}}, color = {0, 0, 0}), Text(extent = {{-90, 80}, {-6, 26}}, lineColor = {192, 192, 192}, textString = "asinh")}),
        Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-100, 0}, {84, 0}}, color = {95, 95, 95}), Polygon(points = {{98, 0}, {82, 6}, {82, -6}, {98, 0}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Line(points = {{-80, -80}, {-56.7, -68.4}, {-39.8, -56.8}, {-26.9, -44.7}, {-17.3, -32.4}, {-9.25, -19}, {9.25, 19}, {17.3, 32.4}, {26.9, 44.7}, {39.8, 56.8}, {56.7, 68.4}, {80, 80}}, color = {0, 0, 255}, thickness = 0.5), Text(extent = {{-31, 72}, {-11, 88}}, textString = "2.31", lineColor = {0, 0, 255}), Text(extent = {{-35, -88}, {-15, -72}}, textString = "-2.31", lineColor = {0, 0, 255}), Text(extent = {{72, -13}, {92, -33}}, textString = "5", lineColor = {0, 0, 255}), Text(extent = {{-96, 21}, {-76, 1}}, textString = "-5", lineColor = {0, 0, 255}), Text(extent = {{80, 22}, {100, 2}}, lineColor = {95, 95, 95}, textString = "u"), Line(points = {{0, 80}, {88, 80}}, color = {175, 175, 175}, smooth = Smooth.None), Line(points = {{80, 86}, {80, -12}}, color = {175, 175, 175}, smooth = Smooth.None)}),
        Documentation(info = "<html>
<p>
The function returns the area hyperbolic sine of its
input argument u. This inverse of sinh(..) is unique
and there is no restriction on the input argument u of
asinh(u) (-&infin; &lt; u &lt; &infin;):
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Math/asinh.png\">
</p>
</html>"));
    end asinh;

    function acosh "Inverse of cosh (area hyperbolic cosine)"
      extends Modelica.Math.Icons.AxisLeft;
      input Real u;
      output Real y;
    algorithm
      assert(u >= 1.0, "Input argument u (= " + String(u) + ") of acosh(u) must be >= 1.0");
      y := Modelica.Math.log(u + sqrt(u * u - 1));
      annotation(
        Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-90, -80}, {68, -80}}, color = {192, 192, 192}), Polygon(points = {{90, -80}, {68, -72}, {68, -88}, {90, -80}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-66, -80}, {-65.2, -66}, {-64.4, -60.3}, {-62.8, -52.2}, {-60.4, -43.4}, {-56.4, -32.4}, {-49.9, -19.3}, {-41.1, -5.65}, {-29, 8.8}, {-12.9, 23.8}, {7.97, 39.2}, {35.3, 55}, {69.9, 70.8}, {94, 80}}, color = {0, 0, 0}), Text(extent = {{-14, 2}, {76, -54}}, lineColor = {192, 192, 192}, textString = "arcosh")}),
        Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-100, -70}, {84, -70}}, color = {95, 95, 95}), Polygon(points = {{100, -70}, {84, -64}, {84, -76}, {100, -70}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Line(points = {{-60, -70}, {-59.2, -56}, {-58.4, -50.3}, {-56.8, -42.2}, {-54.4, -33.4}, {-50.4, -22.4}, {-43.9, -9.3}, {-35.1, 4.35}, {-23, 18.8}, {-6.9, 33.8}, {13.97, 49.2}, {41.3, 65}, {75.9, 80.8}, {100, 90}}, color = {0, 0, 255}, thickness = 0.5), Text(extent = {{-80, 66}, {-58, 78}}, textString = "2.29", lineColor = {0, 0, 255}), Text(extent = {{-73, -86}, {-50, -70}}, textString = "1.0", lineColor = {0, 0, 255}), Text(extent = {{64, -77}, {84, -92}}, textString = "5", lineColor = {0, 0, 255}), Text(extent = {{-96, -55}, {-76, -70}}, textString = "0", lineColor = {0, 0, 255}), Text(extent = {{74, -44}, {94, -64}}, lineColor = {95, 95, 95}, textString = "u"), Line(points = {{-80, 80}, {100, 80}}, color = {175, 175, 175}, smooth = Smooth.None), Line(points = {{74, -78}, {74, 88}}, color = {175, 175, 175}, smooth = Smooth.None)}),
        Documentation(info = "<html>
<p>
This function returns the area hyperbolic cosine of its
input argument u. The valid range of u is
</p>
<pre>
  +1 &le; u &lt; +&infin;
</pre>
<p>
If the function is called with u &lt; 1, an error occurs.
The function cosh(u) has two inverse functions (the curve
looks similar to a sqrt(..) function). acosh(..) returns
the inverse that is positive. At u=1, the derivative dy/du is infinite.
Therefore, this function should not be used in a model, if u
can become close to 1:
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Math/acosh.png\">
</p>
</html>"));
    end acosh;

    function exp "Exponential, base e"
      extends Modelica.Math.Icons.AxisCenter;
      input Real u;
      output Real y;

      external "builtin" y = exp(u);
      annotation(
        Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-90, -80.3976}, {68, -80.3976}}, color = {192, 192, 192}), Polygon(points = {{90, -80.3976}, {68, -72.3976}, {68, -88.3976}, {90, -80.3976}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-80, -80}, {-31, -77.9}, {-6.03, -74}, {10.9, -68.4}, {23.7, -61}, {34.2, -51.6}, {43, -40.3}, {50.3, -27.8}, {56.7, -13.5}, {62.3, 2.23}, {67.1, 18.6}, {72, 38.2}, {76, 57.6}, {80, 80}}, color = {0, 0, 0}), Text(extent = {{-86, 50}, {-14, 2}}, lineColor = {192, 192, 192}, textString = "exp")}),
        Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-100, -80.3976}, {84, -80.3976}}, color = {95, 95, 95}), Polygon(points = {{98, -80.3976}, {82, -74.3976}, {82, -86.3976}, {98, -80.3976}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Line(points = {{-80, -80}, {-31, -77.9}, {-6.03, -74}, {10.9, -68.4}, {23.7, -61}, {34.2, -51.6}, {43, -40.3}, {50.3, -27.8}, {56.7, -13.5}, {62.3, 2.23}, {67.1, 18.6}, {72, 38.2}, {76, 57.6}, {80, 80}}, color = {0, 0, 255}, thickness = 0.5), Text(extent = {{-31, 72}, {-11, 88}}, textString = "20", lineColor = {0, 0, 255}), Text(extent = {{-92, -81}, {-72, -101}}, textString = "-3", lineColor = {0, 0, 255}), Text(extent = {{66, -81}, {86, -101}}, textString = "3", lineColor = {0, 0, 255}), Text(extent = {{2, -69}, {22, -89}}, textString = "1", lineColor = {0, 0, 255}), Text(extent = {{78, -54}, {98, -74}}, lineColor = {95, 95, 95}, textString = "u"), Line(points = {{0, 80}, {88, 80}}, color = {175, 175, 175}, smooth = Smooth.None), Line(points = {{80, 84}, {80, -84}}, color = {175, 175, 175}, smooth = Smooth.None)}),
        Documentation(info = "<html>
<p>
This function returns y = exp(u), with -&infin; &lt; u &lt; &infin;:
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Math/exp.png\">
</p>
</html>"));
    end exp;

    function log "Natural (base e) logarithm (u shall be > 0)"
      extends Modelica.Math.Icons.AxisLeft;
      input Real u;
      output Real y;

      external "builtin" y = log(u);
      annotation(
        Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-90, 0}, {68, 0}}, color = {192, 192, 192}), Polygon(points = {{90, 0}, {68, 8}, {68, -8}, {90, 0}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-80, -80}, {-79.2, -50.6}, {-78.4, -37}, {-77.6, -28}, {-76.8, -21.3}, {-75.2, -11.4}, {-72.8, -1.31}, {-69.5, 8.08}, {-64.7, 17.9}, {-57.5, 28}, {-47, 38.1}, {-31.8, 48.1}, {-10.1, 58}, {22.1, 68}, {68.7, 78.1}, {80, 80}}, color = {0, 0, 0}), Text(extent = {{-6, -24}, {66, -72}}, lineColor = {192, 192, 192}, textString = "log")}),
        Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-100, 0}, {84, 0}}, color = {95, 95, 95}), Polygon(points = {{100, 0}, {84, 6}, {84, -6}, {100, 0}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Line(points = {{-78, -80}, {-77.2, -50.6}, {-76.4, -37}, {-75.6, -28}, {-74.8, -21.3}, {-73.2, -11.4}, {-70.8, -1.31}, {-67.5, 8.08}, {-62.7, 17.9}, {-55.5, 28}, {-45, 38.1}, {-29.8, 48.1}, {-8.1, 58}, {24.1, 68}, {70.7, 78.1}, {82, 80}}, color = {0, 0, 255}, thickness = 0.5), Text(extent = {{-105, 72}, {-85, 88}}, textString = "3", lineColor = {0, 0, 255}), Text(extent = {{60, -3}, {80, -23}}, textString = "20", lineColor = {0, 0, 255}), Text(extent = {{-78, -7}, {-58, -27}}, textString = "1", lineColor = {0, 0, 255}), Text(extent = {{84, 26}, {104, 6}}, lineColor = {95, 95, 95}, textString = "u"), Text(extent = {{-100, 9}, {-80, -11}}, textString = "0", lineColor = {0, 0, 255}), Line(points = {{-80, 80}, {84, 80}}, color = {175, 175, 175}, smooth = Smooth.None), Line(points = {{82, 82}, {82, -6}}, color = {175, 175, 175}, smooth = Smooth.None)}),
        Documentation(info = "<html>
<p>
This function returns y = log(10) (the natural logarithm of u),
with u &gt; 0:
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Math/log.png\">
</p>
</html>"));
    end log;

    function log10 "Base 10 logarithm (u shall be > 0)"
      extends Modelica.Math.Icons.AxisLeft;
      input Real u;
      output Real y;

      external "builtin" y = log10(u);
      annotation(
        Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-90, 0}, {68, 0}}, color = {192, 192, 192}), Polygon(points = {{90, 0}, {68, 8}, {68, -8}, {90, 0}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-79.8, -80}, {-79.2, -50.6}, {-78.4, -37}, {-77.6, -28}, {-76.8, -21.3}, {-75.2, -11.4}, {-72.8, -1.31}, {-69.5, 8.08}, {-64.7, 17.9}, {-57.5, 28}, {-47, 38.1}, {-31.8, 48.1}, {-10.1, 58}, {22.1, 68}, {68.7, 78.1}, {80, 80}}, color = {0, 0, 0}), Text(extent = {{-30, -22}, {60, -70}}, lineColor = {192, 192, 192}, textString = "log10")}),
        Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-100, 0}, {84, 0}}, color = {95, 95, 95}), Polygon(points = {{98, 0}, {82, 6}, {82, -6}, {98, 0}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Line(points = {{-77.8, -80}, {-77.2, -50.6}, {-76.4, -37}, {-75.6, -28}, {-74.8, -21.3}, {-73.2, -11.4}, {-70.8, -1.31}, {-67.5, 8.08}, {-62.7, 17.9}, {-55.5, 28}, {-45, 38.1}, {-29.8, 48.1}, {-8.1, 58}, {24.1, 68}, {70.7, 78.1}, {82, 80}}, color = {0, 0, 255}, thickness = 0.5), Text(extent = {{66, -13}, {86, -33}}, textString = "20", lineColor = {0, 0, 255}), Text(extent = {{-78, -1}, {-58, -21}}, textString = "1", lineColor = {0, 0, 255}), Text(extent = {{-83, 62}, {-63, 78}}, textString = " 1.3", lineColor = {0, 0, 255}), Text(extent = {{80, 24}, {100, 4}}, lineColor = {95, 95, 95}, textString = "u"), Text(extent = {{-100, 9}, {-80, -11}}, textString = "0", lineColor = {0, 0, 255}), Line(points = {{-80, 80}, {86, 80}}, color = {175, 175, 175}, smooth = Smooth.None), Line(points = {{80, 92}, {80, -12}}, color = {175, 175, 175}, smooth = Smooth.None)}),
        Documentation(info = "<html>
<p>
This function returns y = log10(u),
with u &gt; 0:
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/Math/log10.png\">
</p>
</html>"));
    end log10;

    partial function baseIcon1 "This icon will be removed in future Modelica versions, use Modelica.Math.Icons.AxisLeft instead."
      import Modelica;
      // extends Modelica.Icons.ObsoleteModel;
      annotation(
        Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, 100}, {100, -100}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{-80, -80}, {-80, 68}}, color = {192, 192, 192}), Polygon(points = {{-80, 90}, {-88, 68}, {-72, 68}, {-80, 90}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Text(extent = {{-150, 150}, {150, 110}}, textString = "%name", lineColor = {0, 0, 255})}),
        Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-80, 80}, {-88, 80}}, color = {95, 95, 95}), Line(points = {{-80, -80}, {-88, -80}}, color = {95, 95, 95}), Line(points = {{-80, -90}, {-80, 84}}, color = {95, 95, 95}), Text(extent = {{-75, 104}, {-55, 84}}, lineColor = {95, 95, 95}, textString = "y"), Polygon(points = {{-80, 98}, {-86, 82}, {-74, 82}, {-80, 98}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid)}),
        Documentation(info = "<html>
<p>This icon will be removed in future versions of the Modelica Standard Library.
Instead the icon <a href=\"modelica://Modelica.Math.Icons.AxisLeft\">Modelica.Math.Icons.AxisLeft</a> shall be used.
</p>
</html>"));
    end baseIcon1;

    partial function baseIcon2 "This icon will be removed in future Modelica versions, use Modelica.Math.Icons.AxisCenter instead."
      import Modelica;
      // extends Modelica.Icons.ObsoleteModel;
      annotation(
        Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, 100}, {100, -100}}, lineColor = {0, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{0, -80}, {0, 68}}, color = {192, 192, 192}), Polygon(points = {{0, 90}, {-8, 68}, {8, 68}, {0, 90}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Text(extent = {{-150, 150}, {150, 110}}, textString = "%name", lineColor = {0, 0, 255})}),
        Diagram(graphics = {Line(points = {{0, 80}, {-8, 80}}, color = {95, 95, 95}), Line(points = {{0, -80}, {-8, -80}}, color = {95, 95, 95}), Line(points = {{0, -90}, {0, 84}}, color = {95, 95, 95}), Text(extent = {{5, 104}, {25, 84}}, lineColor = {95, 95, 95}, textString = "y"), Polygon(points = {{0, 98}, {-6, 82}, {6, 82}, {0, 98}}, lineColor = {95, 95, 95}, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid)}),
        Documentation(info = "<html>
<p>This icon will be removed in future versions of the Modelica Standard Library.
Instead the icon <a href=\"modelica://Modelica.Math.Icons.AxisCenter\">Modelica.Math.Icons.AxisCenter</a> shall be used.
</p>
</html>"));
    end baseIcon2;

    function tempInterpol1 "Temporary function for linear interpolation (will be removed)"
      extends Modelica.Icons.Function;
      extends Modelica.Icons.ObsoleteModel;
      input Real u "input value (first column of table)";
      input Real table[:, :] "table to be interpolated";
      input Integer icol "column of table to be interpolated";
      output Real y "interpolated input value (icol column of table)";
    protected
      Integer i;
      Integer n "number of rows of table";
      Real u1;
      Real u2;
      Real y1;
      Real y2;
    algorithm
      n := size(table, 1);
      if n <= 1 then
        y := table[1, icol];
      else
        if u <= table[1, 1] then
          i := 1;
        else
          i := 2;
          while i < n and u >= table[i, 1] loop
            i := i + 1;
          end while;
          i := i - 1;
        end if;
        u1 := table[i, 1];
        u2 := table[i + 1, 1];
        y1 := table[i, icol];
        y2 := table[i + 1, icol];
        assert(u2 > u1, "Table index must be increasing");
        y := y1 + (y2 - y1) * (u - u1) / (u2 - u1);
      end if;
// Search interval
// Supports duplicate table[i, 1] values
// in the interior to allow discontinuities.
// Interior means that
// if table[i, 1] = table[i+1, 1] we require i>1 and i+1<n
// Get interpolation data
// Interpolate
      annotation(
        Documentation(info = "<html>

</html>"),
        derivative(zeroDerivative = table, zeroDerivative = icol) = tempInterpol1_der);
    end tempInterpol1;

    function tempInterpol1_der "Temporary function for linear interpolation (will be removed)"
      input Real u "input value (first column of table)";
      input Real table[:, :] "table to be interpolated";
      input Integer icol "column of table to be interpolated";
      input Real du;
      output Real dy "interpolated input value (icol column of table)";
    protected
      Integer i;
      Integer n "number of rows of table";
      Real u1;
      Real u2;
      Real y1;
      Real y2;
    algorithm
      n := size(table, 1);
      if n <= 1 then
        dy := 0;
      else
        if u <= table[1, 1] then
          i := 1;
        else
          i := 2;
          while i < n and u >= table[i, 1] loop
            i := i + 1;
          end while;
          i := i - 1;
        end if;
        u1 := table[i, 1];
        u2 := table[i + 1, 1];
        y1 := table[i, icol];
        y2 := table[i + 1, icol];
        assert(u2 > u1, "Table index must be increasing");
        dy := (y2 - y1) / (u2 - u1);
      end if;
// Search interval
// Supports duplicate table[i, 1] values
// in the interior to allow discontinuities.
// Interior means that
// if table[i, 1] = table[i+1, 1] we require i>1 and i+1<n
// Get interpolation data
// Interpolate
      annotation(
        Documentation(info = "<html>

</html>"));
    end tempInterpol1_der;

    function tempInterpol2 "Temporary function for vectorized linear interpolation (will be removed)"
      extends Modelica.Icons.Function;
      extends Modelica.Icons.ObsoleteModel;
      input Real u "input value (first column of table)";
      input Real table[:, :] "table to be interpolated";
      input Integer icol[:] "column(s) of table to be interpolated";
      output Real y[1, size(icol, 1)] "interpolated input value(s) (column(s) icol of table)";
    protected
      Integer i;
      Integer n "number of rows of table";
      Real u1;
      Real u2;
      Real y1[1, size(icol, 1)];
      Real y2[1, size(icol, 1)];
    algorithm
      n := size(table, 1);
      if n <= 1 then
        y := transpose([table[1, icol]]);
      else
        if u <= table[1, 1] then
          i := 1;
        else
          i := 2;
          while i < n and u >= table[i, 1] loop
            i := i + 1;
          end while;
          i := i - 1;
        end if;
        u1 := table[i, 1];
        u2 := table[i + 1, 1];
        y1 := transpose([table[i, icol]]);
        y2 := transpose([table[i + 1, icol]]);
        assert(u2 > u1, "Table index must be increasing");
        y := y1 + (y2 - y1) * (u - u1) / (u2 - u1);
      end if;
// Search interval
// Supports duplicate table[i, 1] values
// in the interior to allow discontinuities.
// Interior means that
// if table[i, 1] = table[i+1, 1] we require i>1 and i+1<n
// Get interpolation data
// Interpolate
      annotation(
        Documentation(info = "<html>

</html>"),
        derivative(zeroDerivative = table, zeroDerivative = icol) = tempInterpol2_der);
    end tempInterpol2;

    function tempInterpol2_der "Temporary function for vectorized linear interpolation (will be removed)"
      extends Modelica.Icons.Function;
      extends Modelica.Icons.ObsoleteModel;
      input Real u "input value (first column of table)";
      input Real table[:, :] "table to be interpolated";
      input Integer icol[:] "column(s) of table to be interpolated";
      input Real du "input value (first column of table)";
      output Real dy[1, size(icol, 1)] "interpolated input value(s) (column(s) icol of table)";
    protected
      Integer i;
      Integer n "number of rows of table";
      Real u1;
      Real u2;
      Real y1[1, size(icol, 1)];
      Real y2[1, size(icol, 1)];
    algorithm
      n := size(table, 1);
      if n <= 1 then
        dy := zeros(1, size(icol, 1));
      else
        if u <= table[1, 1] then
          i := 1;
        else
          i := 2;
          while i < n and u >= table[i, 1] loop
            i := i + 1;
          end while;
          i := i - 1;
        end if;
        u1 := table[i, 1];
        u2 := table[i + 1, 1];
        y1 := transpose([table[i, icol]]);
        y2 := transpose([table[i + 1, icol]]);
        assert(u2 > u1, "Table index must be increasing");
        dy := (y2 - y1) / (u2 - u1);
      end if;
// Search interval
// Supports duplicate table[i, 1] values
// in the interior to allow discontinuities.
// Interior means that
// if table[i, 1] = table[i+1, 1] we require i>1 and i+1<n
// Get interpolation data
// Interpolate
      annotation(
        Documentation(info = "<html>

</html>"));
    end tempInterpol2_der;
    annotation(
      Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-80, 0}, {-68.7, 34.2}, {-61.5, 53.1}, {-55.1, 66.4}, {-49.4, 74.6}, {-43.8, 79.1}, {-38.2, 79.8}, {-32.6, 76.6}, {-26.9, 69.7}, {-21.3, 59.4}, {-14.9, 44.1}, {-6.83, 21.2}, {10.1, -30.8}, {17.3, -50.2}, {23.7, -64.2}, {29.3, -73.1}, {35, -78.4}, {40.6, -80}, {46.2, -77.6}, {51.9, -71.5}, {57.5, -61.9}, {63.9, -47.2}, {72, -24.8}, {80, 0}}, color = {0, 0, 0}, smooth = Smooth.Bezier)}),
      Documentation(info = "<HTML>
<p>
This package contains <b>basic mathematical functions</b> (such as sin(..)),
as well as functions operating on
<a href=\"modelica://Modelica.Math.Vectors\">vectors</a>,
<a href=\"modelica://Modelica.Math.Matrices\">matrices</a>,
<a href=\"modelica://Modelica.Math.Nonlinear\">nonlinear functions</a>, and
<a href=\"modelica://Modelica.Math.BooleanVectors\">Boolean vectors</a>.
</p>

<dl>
<dt><b>Main Authors:</b>
<dd><a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a> and
    Marcus Baur<br>
    Deutsches Zentrum f&uuml;r Luft und Raumfahrt e.V. (DLR)<br>
    Institut f&uuml;r Robotik und Mechatronik<br>
    Postfach 1116<br>
    D-82230 Wessling<br>
    Germany<br>
    email: <A HREF=\"mailto:Martin.Otter@dlr.de\">Martin.Otter@dlr.de</A><br>
</dl>

<p>
Copyright &copy; 1998-2015, Modelica Association and DLR.
</p>
<p>
<i>This Modelica package is <u>free</u> software and the use is completely at <u>your own risk</u>; it can be redistributed and/or modified under the terms of the Modelica License 2. For license conditions (including the disclaimer of warranty) see <a href=\"modelica://Modelica.UsersGuide.ModelicaLicense2\">Modelica.UsersGuide.ModelicaLicense2</a> or visit <a href=\"https://www.modelica.org/licenses/ModelicaLicense2\"> https://www.modelica.org/licenses/ModelicaLicense2</a>.</i>
</p>
</html>", revisions = "<html>
<ul>
<li><i>October 21, 2002</i>
       by <a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a>
       and <a href=\"http://www.robotic.dlr.de/Christian.Schweiger/\">Christian Schweiger</a>:<br>
       Function tempInterpol2 added.</li>
<li><i>Oct. 24, 1999</i>
       by <a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a>:<br>
       Icons for icon and diagram level introduced.</li>
<li><i>June 30, 1999</i>
       by <a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a>:<br>
       Realized.</li>
</ul>

</html>"));
  end Math;

  package ComplexMath "Library of complex mathematical functions (e.g., sin, cos) and of functions operating on complex vectors and matrices"
    extends Modelica.Icons.Package;
    final constant Complex j = Complex(0, 1) "Imaginary unit";

    package Vectors "Library of functions operating on complex vectors"
      extends Modelica.Icons.Package;

      function norm "Returns the p-norm of a complex vector"
        extends Modelica.Icons.Function;
        input Complex v[:] "Vector";
        input Real p(min = 1) = 2 "Type of p-norm (often used: 1, 2, or Modelica.Constants.inf)";
        output Real result "p-norm of vector v";
      algorithm
        if p == 2 then
          result := sqrt(sum(v[i].re ^ 2 + v[i].im ^ 2 for i in 1:size(v, 1)));
        elseif p == Modelica.Constants.inf then
          result := ComplexMath.'abs'(ComplexMath.'max'(v));
        elseif p == 1 then
          result := sum(ComplexMath.'abs'(v[i]) for i in 1:size(v, 1));
        else
          result := sum(ComplexMath.'abs'(v[i]) ^ p for i in 1:size(v, 1)) ^ (1 / p);
        end if;
        annotation(
          Documentation(info = "<HTML>
<h4>Syntax</h4>
<blockquote><pre>
Vectors.<b>norm</b>(v);
Vectors.<b>norm</b>(v,p=2);   // 1 &le; p &le; &#8734;
</pre></blockquote>

<h4>Description</h4>
<p>
The function call \"<code>Vectors.<b>norm</b>(v)</code>\" returns the
<b>Euclidean norm</b> \"<code>sqrt(v*v)</code>\" of vector v.
With the optional
second argument \"p\", any other p-norm can be computed:
</p>
<center>
<IMG src=\"modelica://Modelica/Resources/Images/Math/Vectors/vectorNorm.png\" ALT=\"function Vectors.norm\">
</center>
<p>
Besides the Euclidean norm (p=2), also the 1-norm and the
infinity-norm are sometimes used:
</p>
<table border=1 cellspacing=0 cellpadding=2>
  <tr><td><b>1-norm</b></td>
      <td>= sum(abs(v))</td>
      <td><b>norm</b>(v,1)</td>
  </tr>
  <tr><td><b>2-norm</b></td>
      <td>= sqrt(v*v)</td>
      <td><b>norm</b>(v) or <b>norm</b>(v,2)</td>
  </tr>
  <tr><td><b>infinity-norm</b></td>
      <td>= max(abs(v))</td>
      <td><b>norm</b>(v,Modelica.Constants.<b>inf</b>)</td>
  </tr>
</table>
<p>
Note, for any vector norm the following inequality holds:
</p>
<blockquote><pre>
<b>norm</b>(v1+v2,p) &le; <b>norm</b>(v1,p) + <b>norm</b>(v2,p)
</pre></blockquote>

<h4>Example</h4>
<blockquote><pre>
  v = {2, -4, -2, -1};
  <b>norm</b>(v,1);    // = 9
  <b>norm</b>(v,2);    // = 5
  <b>norm</b>(v);      // = 5
  <b>norm</b>(v,10.5); // = 4.00052597412635
  <b>norm</b>(v,Modelica.Constants.inf);  // = 4
</pre></blockquote>

<h4>See also</h4>
<p>
<a href=\"modelica://Modelica.Math.Matrices.norm\">Matrices.norm</a>
</p>
</HTML>"));
      end norm;

      function length "Return length of a complex vector"
        extends Modelica.Icons.Function;
        input Complex v[:] "Vector";
        output Real result "Length of vector v";
      algorithm
        result := sqrt(sum({v[i].re ^ 2 + v[i].im ^ 2 for i in 1:size(v, 1)}));
        annotation(
          Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
Vectors.<b>length</b>(v);
</pre></blockquote>

<h4>Description</h4>

<p>
The function call \"<code>Vectors.<b>length</b>(v)</code>\" returns the
<b>Euclidean length</b> \"<code>sqrt(v*v)</code>\" of vector v.
The function call is equivalent to Vectors.norm(v). The advantage of
length(v) over norm(v)\"is that function length(..) is implemented
in one statement and therefore the function is usually automatically
inlined. Further symbolic processing is therefore possible, which is
not the case with function norm(..).
</p>

<h4>Example</h4>
<blockquote><pre>
  v = {2, -4, -2, -1};
  <b>length</b>(v);  // = 5
</pre></blockquote>

<h4>See also</h4>
<p>
<a href=\"modelica://Modelica.Math.Vectors.norm\">Vectors.norm</a>
</p>
</html>"));
      end length;

      function normalize "Return normalized complex vector such that length = 1 and prevent zero-division for zero vector"
        extends Modelica.Icons.Function;
        input Complex v[:] "Vector";
        input Real eps = 100 * Modelica.Constants.eps "if |v| < eps then result = v";
        output Complex result[size(v, 1)] "Input vector v normalized to length=1";
      protected
        Real length_v = length(v);
      algorithm
        if length_v >= eps then
          for i in 1:size(v, 1) loop
            result[i] := v[i].re / length_v + v[i].im / length_v * j;
          end for;
        else
          result := v;
        end if;
        annotation(
          Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
Vectors.<b>normalize</b>(v);
Vectors.<b>normalize</b>(v,eps=100*Modelica.Constants.eps);
</pre></blockquote>

<h4>Description</h4>
<p>
The function call \"<code>Vectors.<b>normalize</b>(v)</code>\" returns the
<b>unit vector</b> \"<code>v/length(v)</code>\" of vector v.
If length(v) is close to zero (more precisely, if length(v) &lt; eps),
v is returned in order to avoid
a division by zero. For many applications this is useful, because
often the unit vector <b>e</b> = <b>v</b>/length(<b>v</b>) is used to compute
a vector x*<b>e</b>, where the scalar x is in the order of length(<b>v</b>),
i.e., x*<b>e</b> is small, when length(<b>v</b>) is small and then
it is fine to replace <b>e</b> by <b>v</b> to avoid a division by zero.
</p>
<p>
Since the function is implemented in one statement,
it is usually inlined and therefore symbolic processing is
possible.
</p>

<h4>Example</h4>
<blockquote><pre>
  <b>normalize</b>({1,2,3});  // = {0.267, 0.534, 0.802}
  <b>normalize</b>({0,0,0});  // = {0,0,0}
</pre></blockquote>

<h4>See also</h4>
<p>
<a href=\"modelica://Modelica.Math.Vectors.length\">Vectors.length</a>
</p>
</html>"));
      end normalize;

      function reverse "Reverse vector elements (e.g., v[1] becomes last element)"
        extends Modelica.Icons.Function;
        input Complex v[:] "Vector";
        output Complex result[size(v, 1)] "Elements of vector v in reversed order";
      algorithm
        result := {v[end - i + 1] for i in 1:size(v, 1)};
        annotation(
          Inline = true,
          Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>Vectors.<b>reverse</b>(v);</pre></blockquote>
<h4>Description</h4>
The function call &quot;<code>Vectors.<b>reverse</b>(v)</code>&quot; returns the complex vector elements in reverse order.
<h4>Example</h4>
<blockquote><pre>  <b>reverse</b>({1,2,3,4});  // = {4,3,2,1}</pre></blockquote>
</html>"));
      end reverse;

      function sort "Sort elements of complex vector"
        extends Modelica.Icons.Function;
        input Complex v[:] "Vector to be sorted";
        input Boolean ascending = true "= true if ascending order, otherwise descending order";
        input Boolean sortFrequency = true "= true, if sorting is first for imaginary then for real value; = false, if sorting is for absolute value";
        output Complex sorted_v[size(v, 1)] = v "Sorted vector";
        output Integer indices[size(v, 1)] = 1:size(v, 1) "sorted_v = v[indices]";
        /* shellsort algorithm; should be improved later */
      protected
        Integer gap;
        Integer i;
        Integer j;
        Complex wv;
        Integer wi;
        Integer nv = size(v, 1);
        Boolean swap;
        Integer k1;
        Integer k2;
      algorithm
        gap := div(nv, 2);
        while gap > 0 loop
          i := gap;
          while i < nv loop
            j := i - gap;
            if j >= 0 then
              k1 := j + 1;
              k2 := j + gap + 1;
              if sortFrequency then
                if ascending then
                  swap := abs(sorted_v[k1].im) > abs(sorted_v[k2].im) or abs(sorted_v[k1].im) == abs(sorted_v[k2].im) and (sorted_v[k1].re > sorted_v[k2].re or sorted_v[k1].re == sorted_v[k2].re and sorted_v[k1].im < sorted_v[k2].im);
                else
                  swap := abs(sorted_v[k1].im) < abs(sorted_v[k2].im) or abs(sorted_v[k1].im) == abs(sorted_v[k2].im) and (sorted_v[k1].re < sorted_v[k2].re or sorted_v[k1].re == sorted_v[k2].re and sorted_v[k1].im < sorted_v[k2].im);
                end if;
              else
                if ascending then
                  swap := ComplexMath.'abs'(sorted_v[k1]) > ComplexMath.'abs'(sorted_v[k2]);
                else
                  swap := ComplexMath.'abs'(sorted_v[k1]) < ComplexMath.'abs'(sorted_v[k2]);
                end if;
              end if;
            else
              swap := false;
            end if;
            while swap loop
              wv := sorted_v[j + 1];
              wi := indices[j + 1];
              sorted_v[j + 1] := sorted_v[j + gap + 1];
              sorted_v[j + gap + 1] := wv;
              indices[j + 1] := indices[j + gap + 1];
              indices[j + gap + 1] := wi;
              j := j - gap;
              if j >= 0 then
                k1 := j + 1;
                k2 := j + gap + 1;
                if sortFrequency then
                  if ascending then
                    swap := abs(sorted_v[k1].im) > abs(sorted_v[k2].im) or abs(sorted_v[k1].im) == abs(sorted_v[k2].im) and (sorted_v[k1].re > sorted_v[k2].re or sorted_v[k1].re == sorted_v[k2].re and sorted_v[k1].im < sorted_v[k2].im);
                  else
                    swap := abs(sorted_v[k1].im) < abs(sorted_v[k2].im) or abs(sorted_v[k1].im) == abs(sorted_v[k2].im) and (sorted_v[k1].re < sorted_v[k2].re or sorted_v[k1].re == sorted_v[k2].re and sorted_v[k1].im < sorted_v[k2].im);
                  end if;
                else
                  if ascending then
                    swap := ComplexMath.'abs'(sorted_v[k1]) > ComplexMath.'abs'(sorted_v[k2]);
                  else
                    swap := ComplexMath.'abs'(sorted_v[k1]) < ComplexMath.'abs'(sorted_v[k2]);
                  end if;
                end if;
              else
                swap := false;
              end if;
            end while;
            i := i + 1;
          end while;
          gap := div(gap, 2);
        end while;
        annotation(
          Documentation(info = "<HTML>
<h4>Syntax</h4>
<blockquote><pre>
           sorted_v = Vectors.<b>sort</b>(v);
(sorted_v, indices) = Vectors.<b>sort</b>(v, ascending=true);
</pre></blockquote>

<h4>Description</h4>
<p>
Function <b>sort</b>(..) sorts a Real vector v
in ascending order and returns the result in sorted_v.
If the optional argument \"ascending\" is <b>false</b>, the vector
is sorted in descending order. In the optional second
output argument the indices of the sorted vector with respect
to the original vector are given, such that sorted_v = v[indices].
</p>

<h4>Example</h4>
<blockquote><pre>
  (v2, i2) := Vectors.sort({-1, 8, 3, 6, 2});
       -> v2 = {-1, 2, 3, 6, 8}
          i2 = {1, 5, 3, 4, 2}
</pre></blockquote>

</HTML>"));
      end sort;
      annotation(
        Documentation(info = "<html>
<p>
This library provides functions operating on vectors
of Complex numbers.
</p>
</html>"));
    end Vectors;

    function sin "Sine of complex number"
      extends Modelica.Icons.Function;
      input Complex c1 "Complex number";
      output Complex c2 "sin(c1)";
    algorithm
      c2 := (exp(Complex(-c1.im, +c1.re)) - exp(Complex(+c1.im, -c1.re))) / Complex(0, 2);
      annotation(
        Inline = true,
        Documentation(info = "<html>
<p>This function returns the Complex sine of the Complex input.</p>
</html>"));
    end sin;

    function cos "Cosine of complex number"
      extends Modelica.Icons.Function;
      input Complex c1 "Complex number";
      output Complex c2 "= cos(c1)";
    algorithm
      c2 := (exp(Complex(-c1.im, +c1.re)) + exp(Complex(+c1.im, -c1.re))) / 2;
      annotation(
        Inline = true,
        Documentation(info = "<html>
<p>This function returns the Complex cosine of the Complex input.</p>
</html>"));
    end cos;

    function tan "Tangent of complex number"
      extends Modelica.Icons.Function;
      input Complex c1 "Complex number";
      output Complex c2 "= tan(c1)";
    algorithm
      c2 := sin(c1) / cos(c1);
      annotation(
        Inline = true,
        Documentation(info = "<html>
<p>This function returns the Complex tangent of the Complex input.</p>
</html>"));
    end tan;

    function asin "Arc-sine of complex number"
      extends Modelica.Icons.Function;
      input Complex c1 "Complex number";
      output Complex c2 "arc_sin(c1)";
    algorithm
      c2 := -j * log(j * c1 + 'sqrt'(1 - c1 * c1));
      annotation(
        Inline = true,
        Documentation(info = "<html>
<p>This function returns the inverse Complex sine of the Complex input.</p>
</html>"));
    end asin;

    function acos "Arc-cosine of complex number"
      extends Modelica.Icons.Function;
      input Complex c1 "Complex number";
      output Complex c2 "= arc_cos(c1)";
    algorithm
      c2 := -j * log(c1 + j * 'sqrt'(1 - c1 * c1));
      annotation(
        Inline = true,
        Documentation(info = "<html>
<p>This function returns the inverse Complex cosine of the Complex input.</p>
</html>"));
    end acos;

    function atan "Arc-tangent of complex number"
      extends Modelica.Icons.Function;
      input Complex c1 "Complex number";
      output Complex c2 "= arc_tan(c1)";
    algorithm
      c2 := 0.5 * j * log((j + c1) / (j - c1));
      annotation(
        Inline = true,
        Documentation(info = "<html>
<p>This function returns the inverse Complex tangent of the Complex input.</p>
</html>"));
    end atan;

    function sinh "Hyperbolic-sine of complex number"
      extends Modelica.Icons.Function;
      input Complex c1 "Complex number";
      output Complex c2 "sinh(c1)";
    algorithm
      c2 := Complex(Math.sinh(c1.re) * Math.cos(c1.im), Math.cosh(c1.re) * Math.sin(c1.im));
      annotation(
        Inline = true,
        Documentation(info = "<html>
<p>This function returns the Complex hyperbolic sine of the Complex input.</p>
</html>"));
    end sinh;

    function cosh "Hyperbolic-cosine of complex number"
      extends Modelica.Icons.Function;
      input Complex c1 "Complex number";
      output Complex c2 "= cosh(c1)";
    algorithm
      c2 := Complex(Math.cosh(c1.re) * Math.cos(c1.im), Math.sinh(c1.re) * Math.sin(c1.im));
      annotation(
        Inline = true,
        Documentation(info = "<html>
<p>This function returns the Complex hyperbolic cosine of the Complex input.</p>
</html>"));
    end cosh;

    function tanh "Hyperbolic-tangent of complex number"
      extends Modelica.Icons.Function;
      input Complex c1 "Complex number";
      output Complex c2 "= tanh(c1)";
    algorithm
      c2 := sinh(c1) / cosh(c1);
      annotation(
        Inline = true,
        Documentation(info = "<html>
<p>This function returns the Complex hyperbolic tangent of the Complex input.</p>
</html>"));
    end tanh;

    function asinh "Area-hyperbolic-sine of complex number"
      extends Modelica.Icons.Function;
      input Complex c1 "Complex number";
      output Complex c2 "ar_sinh(c1)";
    algorithm
      c2 := log(c1 + 'sqrt'(c1 * c1 + 1));
      annotation(
        Inline = true,
        Documentation(info = "<html>
<p>This function returns the inverse Complex hyperbolic sine of the Complex input.</p>
</html>"));
    end asinh;

    function acosh "Area-hyperbolic-cosine of complex number"
      extends Modelica.Icons.Function;
      input Complex c1 "Complex number";
      output Complex c2 "= ar_cosh(c1)";
    algorithm
      c2 := log(c1 + (c1 + 1) * 'sqrt'((c1 - 1) / (c1 + 1)));
      annotation(
        Inline = true,
        Documentation(info = "<html>
<p>This function returns the inverse Complex hyperbolic cosine of the Complex input.</p>
</html>"));
    end acosh;

    function atanh "Area-hyperbolic-tangent of complex number"
      extends Modelica.Icons.Function;
      input Complex c1 "Complex number";
      output Complex c2 "= ar_tanh(c1)";
    algorithm
      c2 := 0.5 * log((1 + c1) / (1 - c1));
      annotation(
        Inline = true,
        Documentation(info = "<html>
<p>This function returns the inverse Complex hyperbolic tangent of the Complex input.</p>
</html>"));
    end atanh;

    function exp "Exponential of complex number"
      extends Modelica.Icons.Function;
      input Complex c1 "Complex number";
      output Complex c2 "= exp(c1)";
    algorithm
      c2 := Complex(Math.exp(c1.re) * Math.cos(c1.im), Math.exp(c1.re) * Math.sin(c1.im));
      annotation(
        Inline = true,
        Documentation(info = "<html>
<p>This function returns the Complex natural exponential of the Complex input.</p>
</html>"));
    end exp;

    function log "Logarithm of complex number"
      extends Modelica.Icons.Function;
      input Complex c1 "Complex number";
      output Complex c2 "= log(c1)";
    algorithm
      c2 := Complex(Modelica.Math.log('abs'(c1)), arg(c1));
      annotation(
        Inline = true,
        Documentation(info = "<html>
<p>This function returns the Complex natural logarithm of the Complex input.</p>
</html>"));
    end log;

    function 'abs' "Absolute value of complex number"
      extends Modelica.Icons.Function;
      input Complex c "Complex number";
      output Real result "= abs(c)";
    algorithm
      result := (c.re ^ 2 + c.im ^ 2) ^ 0.5;
//changed from sqrt
      annotation(
        Inline = true,
        Documentation(info = "<html>
<p>This function returns the Real absolute of the Complex input, i.e., its length.</p>
</html>"));
    end 'abs';

    function arg "Phase angle of complex number"
      extends Modelica.Icons.Function;
      input Complex c "Complex number";
      input Modelica.SIunits.Angle phi0 = 0 "Phase angle phi shall be in the range: -pi < phi-phi0 < pi";
      output Modelica.SIunits.Angle phi "= phase angle of c";
    algorithm
      phi := Modelica.Math.atan3(c.im, c.re, phi0);
      annotation(
        Inline = true,
        Documentation(info = "<html>
<p>This function returns the Real argument of the Complex input, i.e., its angle.</p>
</html>"));
    end arg;

    function conj "Conjugate of complex number"
      extends Modelica.Icons.Function;
      input Complex c1 "Complex number";
      output Complex c2 "= c1.re - j*c1.im";
    algorithm
      c2 := Complex(c1.re, -c1.im);
      annotation(
        Inline = true,
        Documentation(info = "<html>
<p>This function returns the Complex conjugate of the Complex input.</p>
</html>"));
    end conj;

    function real "Real part of complex number"
      extends Modelica.Icons.Function;
      input Complex c "Complex number";
      output Real r "= c.re";
    algorithm
      r := c.re;
      annotation(
        Inline = true,
        Documentation(info = "<html>
<p>This function returns the real part of the Complex input.</p>
</html>"));
    end real;

    function imag "Imaginary part of complex number"
      extends Modelica.Icons.Function;
      input Complex c "Complex number";
      output Real r "= c.im";
    algorithm
      r := c.im;
      annotation(
        Inline = true,
        Documentation(info = "<html>
<p>This function returns the imaginary part of the Complex input.</p>
</html>"));
    end imag;

    function fromPolar "Complex from polar representation"
      extends Modelica.Icons.Function;
      input Real len "abs of complex";
      input Modelica.SIunits.Angle phi "arg of complex";
      output Complex c "= len*cos(phi) + j*len*sin(phi)";
    algorithm
      c := Complex(len * Modelica.Math.cos(phi), len * Modelica.Math.sin(phi));
      annotation(
        Inline = true,
        Documentation(info = "<html>
<p>This function constructs a Complex number from its length (absolute) and angle (argument).</p>
</html>"));
    end fromPolar;

    function 'sqrt' "Square root of complex number"
      extends Modelica.Icons.Function;
      input Complex c1 "Complex number";
      output Complex c2 "= sqrt(c1)";
    algorithm
      c2 := Complex(sqrt('abs'(c1)) * Math.cos(arg(c1) / 2), sqrt('abs'(c1)) * Math.sin(arg(c1) / 2));
      annotation(
        Inline = true,
        Documentation(info = "<html>
<p>This function returns the Complex square root of the Complex input.</p>
</html>"));
    end 'sqrt';

    function 'max' "Return maximum element of complex vector"
      extends Modelica.Icons.Function;
      input Complex v[:] "Vector";
      output Complex result "Element of v with largest absolute value";
      output Integer index "v[index] has the largest absolute value";
    protected
      Real absv_i;
      Real absres;
    algorithm
      if size(v, 1) > 0 then
        absres := 'abs'(v[1]);
        index := 1;
        for i in 2:size(v, 1) loop
          absv_i := 'abs'(v[i]);
          if absv_i > absres then
            absres := absv_i;
            index := i;
          end if;
        end for;
        result := v[index];
      else
        result := Complex(0);
        index := 0;
      end if;
      annotation(
        Documentation(info = "<html>
<p>This function returns the largest element of the Complex input vector, defined by the Complex absolute.</p>
</html>"));
    end 'max';

    function 'min' "Return minimum element of complex vector"
      extends Modelica.Icons.Function;
      input Complex v[:] "Vector";
      output Complex result "Element of v with smallest absolute value";
      output Integer index "v[index] has the smallest absolute value";
    protected
      Real absv_i;
      Real absres;
    algorithm
      if size(v, 1) > 0 then
        absres := 'abs'(v[1]);
        index := 1;
        for i in 2:size(v, 1) loop
          absv_i := 'abs'(v[i]);
          if absv_i < absres then
            absres := absv_i;
            index := i;
          end if;
        end for;
        result := v[index];
      else
        result := Complex(0);
        index := 0;
      end if;
      annotation(
        Documentation(info = "<html>
<p>This function returns the smallest element of the Complex input vector, defined by the Complex absolute.</p>
</html>"));
    end 'min';

    function 'sum' "Return sum of complex vector"
      extends Modelica.Icons.Function;
      input Complex v[:] "Vector";
      output Complex result "Complex sum of vector elements";
    algorithm
      result := Complex(0);
      for i in 1:size(v, 1) loop
        result := result + v[i];
      end for;
      annotation(
        Documentation(info = "<html>
<p>This function returns the Complex sum of the Complex input vector</p>
</html>"));
    end 'sum';

    function 'product' "Return product of complex vector"
      extends Modelica.Icons.Function;
      input Complex v[:] "Vector";
      output Complex result "Complex product of vector elements";
    algorithm
      result := Complex(1);
      for i in 1:size(v, 1) loop
        result := result * v[i];
      end for;
      annotation(
        Documentation(info = "<html>
<p>This function returns the Complex product of the Complex input vector</p>
</html>"));
    end 'product';
    annotation(
      Documentation(info = "<html>
<p>
This package contains <b>basic mathematical functions</b>
operating on complex numbers (such as sin(..)),
as well as functions operating on vectors of complex numbers.
</p>

</html>"),
      Icon(coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = false), graphics = {Line(points = {{32, -86}, {32, 88}}, color = {175, 175, 175}), Line(points = {{-84, 2}, {88, 2}}, color = {175, 175, 175}), Line(points = {{-50, 75}, {-5, 30}}, color = {0, 0, 0}, smooth = Smooth.None), Line(points = {{-50, 30}, {-5, 75}}, color = {0, 0, 0}, smooth = Smooth.None), Line(points = {{-50, -30}, {-5, -75}}, color = {0, 0, 0}, smooth = Smooth.None), Line(points = {{-50, -75}, {-5, -30}}, color = {0, 0, 0}, smooth = Smooth.None)}));
  end ComplexMath;

  package Utilities "Library of utility functions dedicated to scripting (operating on files, streams, strings, system)"
    extends Modelica.Icons.Package;

    package UsersGuide "User's Guide of Utilities Library"
      extends Modelica.Icons.Information;

      class ImplementationNotes "Implementation Notes"
        extends Modelica.Icons.Information;
        annotation(
          Documentation(info = "<html>
<p>
Below the major design decisions of this library are summarized.
</p>
<ul>
<li> <b>C-Function Interface</b><br>
     This library contains several interfaces to C-functions in order
     to operate with the environment. As will become clear, it is usually
     required that a Modelica tool vendor provides an implementation
     of these C-functions that are suited for his environment.
     In directory \"Modelica/Resources/C-Sources\" a reference implementation
     is given for Microsoft Windows Systems and for POSIX environments.
     The files \"ModelicaInternal.c\" and \"ModelicaStrings.c\" can be
     used as a basis for the integration in the vendors environment.<br>&nbsp;</li>
<li> <b>Character Encoding</b><br>
     The representation of characters is different in operating systems.
     The more modern ones (e.g., Windows-NT) use an early variant of
     Unicode (16 bit per character)
     other (e.g., Windows-ME) use 8-bit encoding. Also 32 bit per character
     and multi-byte representations are in use. This is important, since e.g.,
     Japanese Modelica users need Unicode representation. The design in this
     library is done in such a way that a basic set of calls to the operating
     system hides the actual character representation. This means, that all
     functions of this package can be used independent from the underlying
     character representation.<br>
     The C-interface of the Modelica language provides only an 8-bit
     character encoding passing mechanism of strings. As a result, the
     reference implementation in \"Modelica.Utilities\\C-Source\" needs to
     be adapted to the character representation supported in the
     Modelica vendor environment.<br>&nbsp;</li>
<li> <b>Internal String Representation</b><br>
     The design of this package was made in order that string handling
     is convenient. This is in contrast to, e.g., the C-language, where
     string handling is inconvenient, cumbersome and error prone, but on the
     other hand is in some sense \"efficient\".
     The standard reference implementation in \"Modelica.Utilities\\C-Source\"
     is based on the standard C definition of a string, i.e., a pointer to
     a sequence of characters, ended with a null terminating character.
     In order that the string handling in this package is convenient,
     some assumptions have been made, especially, that the access to
     a substring is efficient (O(1) access instead of O(n) as in standard C).
     This allows to hide string pointer arithmetic from the user.
     In such a case, a similar efficiency as in C can be expected for
     most high level operations, such as find, sort, replace.
     The \"efficient character access\" can be reached if, e.g.,
     the number of characters
     are stored in a string, and the length of a character is fixed,
     say 16 or 32 bit (if all Unicode characters shall be represented).
     A vendor should adapt the reference implementation in this
     respect.<br>&nbsp;</li>
<li> <b>String copy = pointer copy</b><br>
     The Modelica language has no mechanism to change a character
     of a string. When a string has to be modified, the only way
     to achieve this is to generate it newly. The advantage is that
     a Modelica tool can treat a string as a constant entity and
     can replace (expensive) string copy operations by pointer
     copy operations. For example, when sorting a set of strings
     the following type of operations occur:
     <pre>
     String s[:], s_temp;
      ...
     s_temp := s[i];
     s[i]   := s[j];
     s[j]   := s_temp;
     </pre>
     Formally, three strings are copied. Due to the feature
     sketched above, a Modelica tool can replace this
     copy operation by pointer assignments, a very \"cheap\"
     operation. The Modelica.Utilities functions will perform
     efficiently, if such types of optimizations are supported
     by the tool.</li>
</ul>
</html>"));
      end ImplementationNotes;

      class ReleaseNotes "Release notes"
        extends Modelica.Icons.ReleaseNotes;
        annotation(
          Documentation(info = "<HTML>
<h4>Version 1.0, 2004-09-29</h4>
<p>
First version implemented.
</p>
</html>"));
      end ReleaseNotes;

      class Contact "Contact"
        extends Modelica.Icons.Contact;
        annotation(
          Documentation(info = "<html>
<dl>
<dt><b>Responsible for Library:</b>
<dd>Dag Br&uuml;ck, Dassault Syst&egrave;mes AB, Sweden.<br>
    email: <A HREF=\"mailto:Dag.BRUCK@3ds.com\">Dag.BRUCK@3ds.com</A><br>
</dl>
<p><b>Acknowledgements:</b></p>
<ul>
<li> This library has been designed by:<br>
     <blockquote>
     Dag Br&uuml;ck, Dassault Syst&egrave;mes AB, Sweden <br>
     Hilding Elmqvist, Dassault Syst&egrave;mes AB, Sweden <br>
     Hans Olsson, Dassault Syst&egrave;mes AB, Sweden <br>
     Martin Otter, DLR Oberpfaffenhofen, Germany.
     </blockquote></li>
<li> The library including the C reference implementation has
     been implemented by Martin Otter and Dag Br&uuml;ck.</li>
<li> The Examples.calculator demonstration to implement a calculator
     with this library is from Hilding Elmqvist.</li>
<li> Helpful comments from Kaj Nystr&ouml;m, PELAB, Link&ouml;ping, Sweden,
     are appreciated, as well as discussions at the 34th, 36th, and 40th
     Modelica Design Meetings in Vienna, Link&ouml;ping, and Dresden. </li>
</ul>
</html>"));
      end Contact;
      annotation(
        DocumentationClass = true,
        Documentation(info = "<HTML>
<p>
Library <b>Modelica.Utilities</b> contains Modelica <b>functions</b> that are
especially suited for <b>scripting</b>. Currently, only a rudimentary
User's Guide is present. This will be improved in the next releases.
The User's Guide has currently the following chapters:
</p>
<ol>
<li>
<a href=\"modelica://Modelica.Utilities.UsersGuide.ReleaseNotes\">Release Notes</a>
  summarizes the differences between different versions of this
  library.
</li>
<li>
<a href=\"modelica://Modelica.Utilities.UsersGuide.ImplementationNotes\">ImplementationNotes</a>
  describes design decisions for this library especially for
  Modelica tool vendors.
</li>
<li>
<a href=\"modelica://Modelica.Utilities.UsersGuide.Contact\">Contact</a> provides
  information about the authors of the library as well as acknowledgments.
</li>
</ol>
<p>
<b>Error handling</b><br>
In case of error, all functions in this library use a Modelica \"assert(..)\"
to provide an error message and to cancel all actions. This means that
functions do not return, if an error is triggered inside the function.
In the near future, an exception handling mechanism will be introduced
in Modelica that will allow to catch errors at a defined place.
</p>
</HTML>"));
    end UsersGuide;

    package Examples "Examples to demonstrate the usage of package Modelica.Utilities"
      extends Modelica.Icons.ExamplesPackage;

      function calculator "Interpreter to evaluate simple expressions consisting of +,-,*,/,(),sin(), cos(), tan(), sqrt(), pi"
        import Modelica.Utilities.Strings;
        extends Modelica.Icons.Function;
        input String string "Expression that is evaluated";
        output Real result "Value of expression";
      protected
        Integer nextIndex;
      algorithm
        (result, nextIndex) := expression(string, 1);
        Strings.scanNoToken(string, nextIndex);
        annotation(
          Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
result = <b>calculator</b>(expression);
</pre></blockquote>
<h4>Description</h4>
<p>
This function demonstrates how a simple expression calculator
can be implemented in form of a recursive decent parser
using basically the Strings.scanToken(..) and Strings.scanDelimiter(..)
function.
</p>
<p>
The following operations are supported (pi=3.14.. is a predefined constant):
</p>
<pre>
   +, -
   *, /
   (expression)
   sin(expression)
   cos(expression)
   tan(expression)
   sqrt(expression)
   pi
</pre>
<h4>Example</h4>
<blockquote><pre>
  calculator(\"2+3*(4-1)\");  // returns 11
  calculator(\"sin(pi/6)\");  // returns 0.5
</pre></blockquote>
</html>"));
      end calculator;

      function expression "Expression interpreter that returns with the position after the expression (expression may consist of +,-,*,/,(),sin(), cos(), tan(), sqrt(), pi"
        import Modelica.Utilities.Types;
        import Modelica.Utilities.Strings;
        import Modelica.Math;
        import Modelica.Constants;
        extends Modelica.Icons.Function;
        input String string "Expression that is evaluated";
        input Integer startIndex = 1 "Start scanning of expression at character startIndex";
        input String message = "" "Message used in error message if scan is not successful";
        output Real result "Value of expression";
        output Integer nextIndex "Index after the scanned expression";
      protected
        function term "Evaluate term of an expression"
          extends Modelica.Icons.Function;
          input String string;
          input Integer startIndex;
          input String message = "";
          output Real result;
          output Integer nextIndex;
        protected
          Real result2;
          Boolean scanning = true;
          String opString;
        algorithm
// scan for "primary * primary" or "primary / primary"
          (result, nextIndex) := primary(string, startIndex, message);
          while scanning loop
            (opString, nextIndex) := Strings.scanDelimiter(string, nextIndex, {"*", "/", ""}, message);
            if opString == "" then
              scanning := false;
            else
              (result2, nextIndex) := primary(string, nextIndex, message);
              result := if opString == "*" then result * result2 else result / result2;
            end if;
          end while;
        end term;

        function primary "Evaluate primary of an expression"
          extends Modelica.Icons.Function;
          input String string;
          input Integer startIndex;
          input String message = "";
          output Real result;
          output Integer nextIndex;
        protected
          Types.TokenValue token;
          Real result2;
          String delimiter;
          String functionName;
          Real pi = Constants.pi;
        algorithm
          (token, nextIndex) := Strings.scanToken(string, startIndex, unsigned = true);
          if token.tokenType == Types.TokenType.DelimiterToken and token.string == "(" then
            (result, nextIndex) := expression(string, nextIndex, message);
            (delimiter, nextIndex) := Strings.scanDelimiter(string, nextIndex, {")"}, message);
          elseif token.tokenType == Types.TokenType.RealToken then
            result := token.real;
          elseif token.tokenType == Types.TokenType.IntegerToken then
            result := token.integer;
          elseif token.tokenType == Types.TokenType.IdentifierToken then
            if token.string == "pi" then
              result := pi;
            else
              functionName := token.string;
              (delimiter, nextIndex) := Strings.scanDelimiter(string, nextIndex, {"("}, message);
              (result, nextIndex) := expression(string, nextIndex, message);
              (delimiter, nextIndex) := Strings.scanDelimiter(string, nextIndex, {")"}, message);
              if functionName == "sin" then
                result := Math.sin(result);
              elseif functionName == "cos" then
                result := Math.cos(result);
              elseif functionName == "tan" then
                result := Math.tan(result);
              elseif functionName == "sqrt" then
                if result < 0.0 then
                  Strings.syntaxError(string, startIndex, "Argument of call \"sqrt(" + String(result) + ")\" is negative.\n" + "Imaginary numbers are not supported by the calculator.\n" + message);
                end if;
                result := sqrt(result);
              else
                Strings.syntaxError(string, startIndex, "Function \"" + functionName + "\" is unknown (not supported)\n" + message);
              end if;
            end if;
          else
            Strings.syntaxError(string, startIndex, "Invalid primary of expression.\n" + message);
          end if;
        end primary;

        Real result2;
        String signOfNumber;
        Boolean scanning = true;
        String opString;
      algorithm
// scan for optional leading "+" or "-" sign
        (signOfNumber, nextIndex) := Strings.scanDelimiter(string, startIndex, {"+", "-", ""}, message);
// scan for "term + term" or "term - term"
        (result, nextIndex) := term(string, nextIndex, message);
        if signOfNumber == "-" then
          result := -result;
        end if;
        while scanning loop
          (opString, nextIndex) := Strings.scanDelimiter(string, nextIndex, {"+", "-", ""}, message);
          if opString == "" then
            scanning := false;
          else
            (result2, nextIndex) := term(string, nextIndex, message);
            result := if opString == "+" then result + result2 else result - result2;
          end if;
        end while;
        annotation(
          Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
             result = <b>expression</b>(string);
(result, nextIndex) = <b>expression</b>(string, startIndex=1, message=\"\");
</pre></blockquote>
<h4>Description</h4>
<p>
This function is nearly the same as Examples.<b>calculator</b>.
The essential difference is that function \"expression\" might be
used in other parsing operations: After the expression is
parsed and evaluated, the function returns with the value
of the expression as well as the position of the character
directly after the expression.
</p>
<p>
This function demonstrates how a simple expression calculator
can be implemented in form of a recursive decent parser
using basically the Strings.scanToken(..) and scanDelimiters(..)
function. There are 2 local functions (term, primary) that
implement the corresponding part of the grammar.
</p>
<p>
The following operations are supported (pi=3.14.. is a predefined constant):
</p>
<pre>
   +, -
   *, /
   (expression)
   sin(expression)
   cos(expression)
   tan(expression)
   sqrt(expression)
   pi
</pre>
<p>
The optional argument \"startIndex\" defines at which position
scanning of the expression starts.
</p>
<p>
In case of error,
the optional argument \"message\" is appended to the error
message, in order to give additional information where
the error occurred.
</p>
<p>
This function parses the following grammar
</p>
<pre>
  expression: [ add_op ] term { add_op term }
  add_op    : \"+\" | \"-\"
  term      : primary { mul_op primary }
  mul_op    : \"*\" | \"/\"
  primary   : UNSIGNED_NUMBER
              | pi
              | ( expression )
              | functionName( expression )
  function  :   sin
              | cos
              | tan
              | sqrt
</pre>
<p>
Note, in Examples.readRealParameter it is shown, how the expression
function can be used as part of another scan operation.
</p>
<h4>Example</h4>
<blockquote><pre>
  expression(\"2+3*(4-1)\");  // returns 11
  expression(\"sin(pi/6)\");  // returns 0.5
</pre></blockquote>
</html>"));
      end expression;

      function readRealParameter "Read the value of a Real parameter from file"
        extends Modelica.Icons.Function;
        input String fileName "Name of file" annotation(
          Dialog(loadSelector(filter = "Text files (*.txt)", caption = "Open file in which Real parameters are present")));
        input String name "Name of parameter";
        output Real result "Actual value of parameter on file";
      protected
        String line;
        String identifier;
        String delimiter;
        Integer nextIndex;
        Integer iline = 1;
        Types.TokenValue token;
        String message = "in file \"" + fileName + "\" on line ";
        String message2;
        Boolean found = false;
        Boolean endOfFile = false;
      algorithm
        (line, endOfFile) := Streams.readLine(fileName, iline);
        while not found and not endOfFile loop
          (token, nextIndex) := Strings.scanToken(line);
          if token.tokenType == Types.TokenType.NoToken then
            iline := iline + 1;
          elseif token.tokenType == Types.TokenType.IdentifierToken then
            if token.string == name then
              message2 := message + String(iline);
              (delimiter, nextIndex) := Strings.scanDelimiter(line, nextIndex, {"="}, message2);
              (result, nextIndex) := Examples.expression(line, nextIndex, message2);
              (delimiter, nextIndex) := Strings.scanDelimiter(line, nextIndex, {";", ""}, message2);
              Strings.scanNoToken(line, nextIndex, message2);
              found := true;
            else
              iline := iline + 1;
            end if;
          else
            Strings.syntaxError(line, nextIndex, "Expected identifier " + message + String(iline));
          end if;
          (line, endOfFile) := Streams.readLine(fileName, iline);
        end while;
// skip line
// name found, get value of "name = value;"
// wrong name, skip line
// wrong token
// read next line
        if not found then
          Streams.error("Parameter \"" + name + "\" not found in file \"" + fileName + "\"");
        end if;
        annotation(
          Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
result = <b>readRealParameter</b>(fileName, name);
</pre></blockquote>
<h4>Description</h4>
<p>
This function demonstrates how a function can be implemented
that reads the value of a parameter from file. The function
performs the following actions:
</p>
<ol>
<li> It opens file \"fileName\" and reads the lines of the file.</li>
<li> In every line, Modelica line comments (\"// ... end-of-line\")
     are skipped </li>
<li> If a line consists of \"name = expression\" and the \"name\"
     in this line is identical to the second argument \"name\"
     of the function call, the expression calculator Examples.expression
     is used to evaluate the expression after the \"=\" character.
     The expression can optionally be terminated with a \";\".</li>
<li> The result of the expression evaluation is returned as
     the value of the parameter \"name\". </li>
</ol>
<h4>Example</h4>
<p>
On file \"test.txt\" the following lines might be present:
</p>
<blockquote><pre>
// Motor data
J        = 2.3     // inertia
w_rel0   = 1.5*2;  // relative angular velocity
phi_rel0 = pi/3
</pre></blockquote>
<p>
The function returns the value \"3.0\" when called as:
</p>
<blockquote><pre>
readRealParameter(\"test.txt\", \"w_rel0\")
</pre></blockquote>
</html>"));
      end readRealParameter;

      model readRealParameterModel "Demonstrate usage of Examples.readRealParameter/.expression"
        import SI = Modelica.SIunits;
        extends Modelica.Icons.Example;
        parameter String file = Modelica.Utilities.Files.loadResource("modelica://Modelica/Resources/Data/Utilities/Examples_readRealParameters.txt") "File on which data is present" annotation(
          Dialog(loadSelector(filter = "Text files (*.txt)", caption = "Open text file to read parameters of the form \"name = value\"")));
        parameter SI.Inertia J = readRealParameter(file, "J") "Inertia";
        parameter SI.Angle phi_rel0 = readRealParameter(file, "phi_rel0") "Relative angle";
        parameter SI.AngularVelocity w_rel0 = readRealParameter(file, "w_rel0") "Relative angular velocity";
        annotation(
          Documentation(info = "<html>
<p>
Model that shows the usage of Examples.readRealParameter and Examples.expression.
The model has 3 parameters and the values of these parameters are read
from a file.
</p>
</html>"),
          experiment(StopTime = 1.01));
      end readRealParameterModel;
      annotation(
        Documentation(info = "<html>
<p>
This package contains quite involved examples that demonstrate how to
use the functions of package Modelica.Utilities. In particular
the following examples are present.
</p>
<ul>
<li> Function <a href=\"modelica://Modelica.Utilities.Examples.calculator\">calculator</a>
     is an interpreter to evaluate
     expressions consisting of +,-,*,/,(),sin(), cos(), tan(), sqrt(), pi.
     For example: calculator(\"1.5*sin(pi/6)\"); <br>&nbsp;</li>
<li> Function <a href=\"modelica://Modelica.Utilities.Examples.expression\">expression</a>
     is the basic function used in \"calculator\" to evaluate an expression.
     It is useful if the expression interpreter is used in a larger
     scan operation (such as readRealParameter below).<br>&nbsp;</li>
<li> Function <a href=\"modelica://Modelica.Utilities.Examples.readRealParameter\">readRealParameter</a>
     reads the value of a parameter
     from file, given the file and the parameter name. The value
     on file is interpreted with the Examples.expression function
     and can therefore be an expression.<br>&nbsp;</li>
<li> Model <a href=\"modelica://Modelica.Utilities.Examples.readRealParameterModel\">readRealParameterModel</a>
     is a test model to demonstrate the usage of \"readRealParameter\". The model
     contains 3 parameters that are read from file \"Modelica.Utilities/data/Examples_readRealParameters.txt\".
     </li>
</ul>
</html>"));
    end Examples;

    package Files "Functions to work with files and directories"
      extends Modelica.Icons.Package;

      function list "List content of file or directory"
        extends Modelica.Icons.Function;
        input String name "If name is a directory, list directory content. If it is a file, list the file content";
        //..............................................................
      protected
        Types.FileType fileType;

        function listFile "List content of file"
          extends Modelica.Icons.Function;
          input String name;
        protected
          String file[Streams.countLines(name)] = Streams.readFile(name);
        algorithm
          for i in 1:min(size(file, 1), 100) loop
            Streams.print(file[i]);
          end for;
        end listFile;

        function sortDirectory "Sort directory in directories and files with alphabetic order"
          extends Modelica.Icons.Function;
          input String directory "Directory that was read (including a trailing '/')";
          input String names[:] "File and directory names of a directory in any order";
          output String orderedNames[size(names, 1)] "Names of directories followed by names of files";
          output Integer nDirectories "The first nDirectories entries in orderedNames are directories";
        protected
          Integer nEntries = size(names, 1);
          Integer nFiles;
          Integer lenDirectory = Strings.length(directory);
          String directory2;
        algorithm
// Construct directory with a trailing "/"
          directory2 := if Strings.substring(directory, lenDirectory, lenDirectory) == "/" then directory else directory + "/";
// Distinguish directories and files
          nDirectories := 0;
          nFiles := 0;
          for i in 1:nEntries loop
            if Modelica.Utilities.Internal.FileSystem.stat(directory2 + names[i]) == Types.FileType.Directory then
              nDirectories := nDirectories + 1;
              orderedNames[nDirectories] := names[i];
            else
              nFiles := nFiles + 1;
              orderedNames[nEntries - nFiles + 1] := names[i];
            end if;
          end for;
// Sort files and directories alphabetically
          if nDirectories > 0 then
            orderedNames[1:nDirectories] := Strings.sort(orderedNames[1:nDirectories], caseSensitive = false);
          end if;
          if nFiles > 0 then
            orderedNames[nDirectories + 1:nEntries] := Strings.sort(orderedNames[nDirectories + 1:nEntries], caseSensitive = false);
          end if;
        end sortDirectory;

        function listDirectory "List content of directory"
          extends Modelica.Icons.Function;
          input String directoryName;
          input Integer nEntries;
        protected
          String files[nEntries];
          Integer nDirectories;
        algorithm
          if nEntries > 0 then
            Streams.print("\nDirectory \"" + directoryName + "\":");
            files := Modelica.Utilities.Internal.FileSystem.readDirectory(directoryName, nEntries);
            (files, nDirectories) := sortDirectory(directoryName, files);
            if nDirectories > 0 then
              Streams.print("  Subdirectories:");
              for i in 1:nDirectories loop
                Streams.print("    " + files[i]);
              end for;
              Streams.print(" ");
            end if;
            if nDirectories < nEntries then
              Streams.print("  Files:");
              for i in nDirectories + 1:nEntries loop
                Streams.print("    " + files[i]);
              end for;
            end if;
          else
            Streams.print("... Directory\"" + directoryName + "\" is empty");
          end if;
// List directories
// List files
        end listDirectory;
      algorithm
        fileType := Modelica.Utilities.Internal.FileSystem.stat(name);
        if fileType == Types.FileType.RegularFile then
          listFile(name);
        elseif fileType == Types.FileType.Directory then
          listDirectory(name, Modelica.Utilities.Internal.FileSystem.getNumberOfFiles(name));
        elseif fileType == Types.FileType.SpecialFile then
          Streams.error("Cannot list file \"" + name + "\"\n" + "since it is not a regular file (pipe, device, ...)");
        else
          Streams.error("Cannot list file or directory \"" + name + "\"\n" + "since it does not exist");
        end if;
        annotation(
          Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
Files.<b>list</b>(name);
</pre></blockquote>
<h4>Description</h4>
<p>
If \"name\" is a regular file, the content of the
file is printed.
</p>
<p>
If \"name\" is a directory, the directory and file names
in the \"name\" directory are printed in sorted order.
</p>
</html>"));
      end list;

      function copy "Generate a copy of a file or of a directory"
        extends Modelica.Icons.Function;
        input String oldName "Name of file or directory to be copied";
        input String newName "Name of copy of the file or of the directory";
        input Boolean replace = false "= true, if an existing file may be replaced by the required copy";
        //..............................................................
      protected
        function copyDirectory "Copy a directory"
          extends Modelica.Icons.Function;
          input String oldName "Old directory name without trailing '/'; existance is guaranteed";
          input String newName "New directory name without trailing '/'; directory was already created";
          input Boolean replace "= true, if an existing newName may be replaced";
        algorithm
          copyDirectoryContents(Modelica.Utilities.Internal.FileSystem.readDirectory(oldName, Modelica.Utilities.Internal.FileSystem.getNumberOfFiles(oldName)), oldName, newName, replace);
        end copyDirectory;

        function copyDirectoryContents
          extends Modelica.Icons.Function;
          input String oldNames[:];
          input String oldName;
          input String newName;
          input Boolean replace;
        protected
          String oldName_i;
          String newName_i;
        algorithm
          for i in 1:size(oldNames, 1) loop
            oldName_i := oldName + "/" + oldNames[i];
            newName_i := newName + "/" + oldNames[i];
            Files.copy(oldName_i, newName_i, replace);
          end for;
        end copyDirectoryContents;

        //..............................................................
        Integer lenOldName = Strings.length(oldName);
        Integer lenNewName = Strings.length(newName);
        String oldName2 = if Strings.substring(oldName, lenOldName, lenOldName) == "/" then Strings.substring(oldName, 1, lenOldName - 1) else oldName;
        String newName2 = if Strings.substring(newName, lenNewName, lenNewName) == "/" then Strings.substring(newName, 1, lenNewName - 1) else newName;
        Types.FileType oldFileType = Modelica.Utilities.Internal.FileSystem.stat(oldName2);
        Types.FileType newFileType;
      algorithm
        if oldFileType == Types.FileType.NoFile then
          Streams.error("It is not possible to copy the file or directory\n" + "\"" + oldName2 + "\" because it does not exist.");
        elseif oldFileType == Types.FileType.Directory then
          newFileType := Modelica.Utilities.Internal.FileSystem.stat(newName2);
          if newFileType == Types.FileType.NoFile then
            createDirectory(newName2);
          elseif newFileType == Types.FileType.RegularFile or newFileType == Types.FileType.SpecialFile then
            if replace then
              Files.removeFile(newName2);
              Files.createDirectory(newName2);
            else
              Streams.error("Directory \"" + oldName2 + "\" should be copied to\n" + "\"" + newName2 + "\" which is an existing file.\n" + "Since argument replace=false, this is not allowed");
            end if;
          end if;
          copyDirectory(oldName2, newName2, replace);
        else
          if replace then
            Files.removeFile(newName2);
          else
            Files.assertNew(newName2, "File \"" + oldName2 + "\" should be copied or moved to\n" + "\"" + newName2 + "\" which is an existing file or directory.\n" + "Since argument replace=false, this is not allowed");
          end if;
          Modelica.Utilities.Internal.FileSystem.copyFile(oldName2, newName2);
        end if;
// regular or special file
        annotation(
          Documentation(info = "<HTML>
<h4>Syntax</h4>
<blockquote><pre>
Files.<b>copy</b>(oldName, newName);
Files.<b>copy</b>(oldName, newName, replace = true);
</pre></blockquote>
<h4>Description</h4>
<p>
Function <b>copy</b>(..) copies a file or a directory
to a new location. Via the optional argument <b>replace</b>
it can be defined whether an already existing file may
be replaced by the required copy.
</p>
<p>
If oldName/newName are directories, then the newName
directory may exist. In such a case the content of oldName
is copied into directory newName. If replace = <b>false</b>
it is required that the existing files
in newName are different from the existing files in
oldName.
</p>
<h4>Example</h4>
<blockquote><pre>
  copy(\"C:/test1/directory1\", \"C:/test2/directory2\");
     -> the content of directory1 is copied into directory2
        if \"C:/test2/directory2\" does not exist, it is newly
        created. If \"replace=true\", files in directory2
        may be overwritten by their copy
  copy(\"test1.txt\", \"test2.txt\")
     -> make a copy of file \"test1.txt\" with the name \"test2.txt\"
        in the current directory
</pre></blockquote>
</HTML>"));
      end copy;

      function move "Move a file or a directory to another place"
        extends Modelica.Icons.Function;
        input String oldName "Name of file or directory to be moved";
        input String newName "New name of the moved file or directory";
        input Boolean replace = false "= true, if an existing file or directory may be replaced";
      algorithm
// if both oldName and newName are in the current directory
// use Internal.renameFile
        if Strings.find(oldName, "/") == 0 and Strings.find(newName, "/") == 0 then
          Modelica.Utilities.Internal.FileSystem.rename(oldName, newName);
        else
          Files.copy(oldName, newName, replace);
          Files.remove(oldName);
        end if;
        annotation(
          Documentation(info = "<HTML>
<h4>Syntax</h4>
<blockquote><pre>
Files.<b>move</b>(oldName, newName);
Files.<b>move</b>(oldName, newName, replace = true);
</pre></blockquote>
<h4>Description</h4>
<p>
Function <b>move</b>(..) moves a file or a directory
to a new location. Via the optional argument <b>replace</b>
it can be defined whether an already existing file may
be replaced.
</p>
<p>
If oldName/newName are directories, then the newName
directory may exist. In such a case the content of oldName
is moved into directory newName. If replace = <b>false</b>
it is required that the existing files
in newName are different from the existing files in
oldName.
</p>
<h4>Example</h4>
<blockquote><pre>
  move(\"C:/test1/directory1\", \"C:/test2/directory2\");
     -> the content of directory1 is moved into directory2.
        Afterwards directory1 is deleted.
        if \"C:/test2/directory2\" does not exist, it is newly
        created. If \"replace=true\", files in directory2
        may be overwritten
   move(\"test1.txt\", \"test2.txt\")
     -> rename file \"test1.txt\" into \"test2.txt\"
        within the current directory
</pre></blockquote>
</HTML>"));
      end move;

      function remove "Remove file or directory (ignore call, if it does not exist)"
        extends Modelica.Icons.Function;
        input String name "Name of file or directory to be removed";
        //..............................................................
      protected
        function removeDirectory "Remove a directory, even if it is not empty"
          extends Modelica.Icons.Function;
          input String name;
        protected
          Integer lenName = Strings.length(name);
          // remove an optional trailing "/"
          String name2 = if Strings.substring(name, lenName, lenName) == "/" then Strings.substring(name, lenName - 1, lenName - 1) else name;
        algorithm
          removeDirectoryContents(Modelica.Utilities.Internal.FileSystem.readDirectory(name2, Modelica.Utilities.Internal.FileSystem.getNumberOfFiles(name2)), name2);
          Modelica.Utilities.Internal.FileSystem.rmdir(name2);
        end removeDirectory;

        function removeDirectoryContents
          extends Modelica.Icons.Function;
          input String fileNames[:];
          input String name2;
        algorithm
          for i in 1:size(fileNames, 1) loop
            Files.remove(name2 + "/" + fileNames[i]);
          end for;
        end removeDirectoryContents;

        //..............................................................
        String fullName = Files.fullPathName(name);
        Types.FileType fileType = Modelica.Utilities.Internal.FileSystem.stat(fullName);
      algorithm
        if fileType == Types.FileType.RegularFile or fileType == Types.FileType.SpecialFile then
          Modelica.Utilities.Internal.FileSystem.removeFile(fullName);
        elseif fileType == Types.FileType.Directory then
          removeDirectory(fullName);
        end if;
        annotation(
          Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
Files.<b>remove</b>(name);
</pre></blockquote>
<h4>Description</h4>
<p>
Removes the file or directory \"name\". If \"name\" does not exist,
the function call is ignored. If \"name\" is a directory, first
the content of the directory is removed and afterwards
the directory itself.
</p>
<p>
This function is silent, i.e., it does not print a message.
</p>
</html>"));
      end remove;

      function removeFile "Remove file (ignore call, if it does not exist)"
        extends Modelica.Icons.Function;
        input String fileName "Name of file that should be removed";
      protected
        Types.FileType fileType = Modelica.Utilities.Internal.FileSystem.stat(fileName);
      algorithm
        if fileType == Types.FileType.RegularFile then
          Modelica.Utilities.Internal.FileSystem.removeFile(fileName);
        elseif fileType == Types.FileType.Directory then
          Streams.error("File \"" + fileName + "\" should be removed.\n" + "This is not possible, because it is a directory");
        elseif fileType == Types.FileType.SpecialFile then
          Streams.error("File \"" + fileName + "\" should be removed.\n" + "This is not possible, because it is a special file (pipe, device, etc.)");
        end if;
        annotation(
          Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
Files.<b>removeFile</b>(fileName);
</pre></blockquote>
<h4>Description</h4>
<p>
Removes the file \"fileName\". If \"fileName\" does not exist,
the function call is ignored. If \"fileName\" exists but is
no regular file (e.g., directory, pipe, device, etc.) an
error is triggered.
</p>
<p>
This function is silent, i.e., it does not print a message.
</p>
</html>"));
      end removeFile;

      function createDirectory "Create directory (if directory already exists, ignore call)"
        extends Modelica.Icons.Function;
        input String directoryName "Name of directory to be created (if present, ignore call)";
        //..............................................................
      protected
        function existDirectory "Inquire whether directory exists; if present and not a directory, trigger an error"
          extends Modelica.Icons.Function;
          input String directoryName;
          output Boolean exists "true if directory exists";
        protected
          Types.FileType fileType = Modelica.Utilities.Internal.FileSystem.stat(directoryName);
        algorithm
          if fileType == Types.FileType.RegularFile or fileType == Types.FileType.SpecialFile then
            Streams.error("Directory \"" + directoryName + "\" cannot be created\n" + "because this is an existing file.");
          elseif fileType == Types.FileType.Directory then
            exists := true;
          else
            exists := false;
          end if;
        end existDirectory;

        function assertCorrectIndex "Print error, if index to last essential character in directory is wrong"
          extends Modelica.Icons.Function;
          input Integer index "Index must be > 0";
          input String directoryName "Directory name for error message";
        algorithm
          if index < 1 then
            Streams.error("It is not possible to create the directory\n" + "\"" + directoryName + "\"\n" + "because this directory name is not valid");
          end if;
        end assertCorrectIndex;

        //..............................................................
        String fullName;
        Integer index;
        Integer oldIndex;
        Integer lastIndex;
        Boolean found;
        Boolean finished;
        Integer nDirectories = 0 "Number of directories that need to be generated";
      algorithm
// Ignore call, if directory exists
        if not existDirectory(directoryName) then
          fullName := Files.fullPathName(directoryName);
          index := Strings.length(fullName);
          if Strings.substring(fullName, index, index) == "/" then
            index := index - 1;
            assertCorrectIndex(index, fullName);
          end if;
          lastIndex := index;
          fullName := Strings.substring(fullName, 1, index);
          found := false;
          while not found loop
            oldIndex := index;
            index := Strings.findLast(fullName, "/", startIndex = index);
            if index == 0 then
              index := oldIndex;
              found := true;
            else
              index := index - 1;
              assertCorrectIndex(index, fullName);
              found := existDirectory(Strings.substring(fullName, 1, index));
            end if;
          end while;
          index := oldIndex;
          finished := false;
          while not finished loop
            Modelica.Utilities.Internal.FileSystem.mkdir(Strings.substring(fullName, 1, index));
            if index >= lastIndex then
              finished := true;
            elseif index < lastIndex then
              index := Strings.find(fullName, "/", startIndex = index + 2);
              if index == 0 then
                index := lastIndex;
              end if;
            end if;
          end while;
        end if;
// Remove a trailing "/"
// Search upper directories until a directory is found that exists
// ??? check the following while loop later, if also cases such as
//  "c:/", "c:", "//name" are handled correctly ???
// Create directories
        annotation(
          Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
Files.<b>createDirectory</b>(directoryName);
</pre></blockquote>
<h4>Description</h4>
<p>
Creates directory \"directoryName\". If this directory already exists,
the function call is ignored. If several directories in \"directoryName\"
do not exist, all of them are created. For example, assume
that directory \"E:/test1\" exists and that directory
\"E:/test1/test2/test3\" shall be created. In this case
the directories \"test2\" in \"test1\" and \"test3\" in \"test2\"
are created.
</p>
<p>
This function is silent, i.e., it does not print a message.
In case of error (e.g., \"directoryName\" is an existing regular
file), an assert is triggered.
</p>
</html>"));
      end createDirectory;

      function exist "Inquire whether file or directory exists"
        extends Modelica.Icons.Function;
        input String name "Name of file or directory";
        output Boolean result "= true, if file or directory exists";
      algorithm
        result := Modelica.Utilities.Internal.FileSystem.stat(name) > Types.FileType.NoFile;
        annotation(
          Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
result = Files.<b>exist</b>(name);
</pre></blockquote>
<h4>Description</h4>
<p>
Returns true, if \"name\" is an existing file or directory.
If this is not the case, the function returns false.
</p>
</html>"));
      end exist;

      function assertNew "Trigger an assert, if a file or directory exists"
        extends Modelica.Icons.Function;
        input String name "Name of file or directory";
        input String message = "This is not allowed." "Message that should be printed after the default message in a new line";
      protected
        Types.FileType fileType = Modelica.Utilities.Internal.FileSystem.stat(name);
      algorithm
        if fileType == Types.FileType.RegularFile then
          Streams.error("File \"" + name + "\" already exists.\n" + message);
        elseif fileType == Types.FileType.Directory then
          Streams.error("Directory \"" + name + "\" already exists.\n" + message);
        elseif fileType == Types.FileType.SpecialFile then
          Streams.error("A special file (pipe, device, etc.) \"" + name + "\" already exists.\n" + message);
        end if;
        annotation(
          Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
Files.<b>assertNew</b>(name);
Files.<b>assertNew</b>(name, message=\"This is not allowed\");
</pre></blockquote>
<h4>Description</h4>
<p>
Triggers an assert, if \"name\" is an existing file or
directory. The error message has the following structure:
</p>
<pre>
  File \"&lt;name&gt;\" already exists.
  &lt;message&gt;
</pre>
</html>"));
      end assertNew;

      function fullPathName "Get full path name of file or directory name"
        extends Modelica.Icons.Function;
        input String name "Absolute or relative file or directory name";
        output String fullName "Full path of 'name'";

        external "C" fullName = ModelicaInternal_fullPathName(name) annotation(
          Library = "ModelicaExternalC");
        annotation(
          Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
fullName = Files.<b>fullPathName</b>(name);
</pre></blockquote>
<h4>Description</h4>
<p>
Returns the full path name of a file or directory \"name\".
</p>
</html>"));
      end fullPathName;

      function splitPathName "Split path name in directory, file name kernel, file name extension"
        extends Modelica.Icons.Function;
        input String pathName "Absolute or relative file or directory name";
        output String directory "Name of the directory including a trailing '/'";
        output String name "Name of the file without the extension";
        output String extension "Extension of the file name. Starts with '.'";
      protected
        Integer lenPath = Strings.length(pathName);
        Integer i = lenPath;
        Integer indexDot = 0;
        Integer indexSlash = 0;
        String c;
      algorithm
        while i >= 1 loop
          c := Strings.substring(pathName, i, i);
          if c == "." then
            indexDot := i;
            i := 0;
          elseif c == "/" then
            indexSlash := i;
            i := 0;
          else
            i := i - 1;
          end if;
        end while;
        if indexSlash == lenPath then
          directory := pathName;
          name := "";
          extension := "";
        elseif indexDot > 0 then
          indexSlash := Strings.findLast(pathName, "/", startIndex = indexDot - 1);
          if indexSlash == 0 then
            directory := "";
            name := Strings.substring(pathName, 1, indexDot - 1);
          else
            directory := Strings.substring(pathName, 1, indexSlash);
            name := Strings.substring(pathName, indexSlash + 1, indexDot - 1);
          end if;
          extension := Strings.substring(pathName, indexDot, lenPath);
        else
          extension := "";
          if indexSlash > 0 then
            directory := Strings.substring(pathName, 1, indexSlash);
            name := Strings.substring(pathName, indexSlash + 1, lenPath);
          else
            directory := "";
            name := pathName;
          end if;
        end if;
        annotation(
          Documentation(info = "<HTML>
<h4>Syntax</h4>
<blockquote><pre>
(directory, name, extension) = Files.<b>splitPathName</b>(pathName);
</pre></blockquote>
<h4>Description</h4>
<p>
Function <b>splitPathName</b>(..) splits a path name into its parts.
</p>
<h4>Example</h4>
<pre>
  (directory, name, extension) = Files.splitPathName(\"C:/user/test/input.txt\")

  -> directory = \"C:/user/test/\"
     name      = \"input\"
     extension = \".txt\"
</pre>
</HTML>"));
      end splitPathName;

      function temporaryFileName "Return arbitrary name of a file that does not exist and is in a directory where access rights allow to write to this file (useful for temporary output of files)"
        extends Modelica.Icons.Function;
        output String fileName "Full path name of temporary file";

        external "C" fileName = ModelicaInternal_temporaryFileName(0) annotation(
          Library = "ModelicaExternalC");
        annotation(
          Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
fileName = Files.<b>temporaryFileName</b>();
</pre></blockquote>
<h4>Description</h4>
<p>
Return arbitrary name of a file that does not exist
and is in a directory where access rights allow to
write to this file (useful for temporary output of files).
</p>
</html>"));
      end temporaryFileName;

      function loadResource "Return the absolute path name of a URI or local file name"
        extends Modelica.Utilities.Internal.PartialModelicaServices.ExternalReferences.PartialLoadResource;
        extends ModelicaServices.ExternalReferences.loadResource;
        annotation(
          Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
fileReference = FileSystem.<b>loadResource</b>(uri);
</pre></blockquote>
<h4>Description</h4>
<p>
The function call \"<code>FileSystem.<b>loadResource</b>(uri)</code>\" returns the
<b>absolute path name</b> of the file that is either defined by an URI or by a local
path name. With the returned file name it is possible to
access the file with function calls of the C standard library.
If the data or file is stored in a data-base,
this might require copying the resource to a temporary folder and referencing that.
</p>

<p>
The implementation of this function is tool specific. However, at least Modelica URIs
(see \"chapter 13.2.3 External Resources\" of the Modelica Specification),
as well as absolute local file path names are supported.
</p>

<h4>Example</h4>
<blockquote><pre>
  file1 = loadResource(\"modelica://Modelica/Resources/Data/Utilities/Examples_readRealParameters.txt\")
          // file1 is the absolute path name of the file
  file2 = loadResource(\"C:\\\\data\\\\readParameters.txt\")
          file2 = \"C:/data/readParameters.txt\"
</pre></blockquote>
</html>"));
      end loadResource;
      annotation(
        Documentation(info = "<HTML>
<p>
This package contains functions to work with files and directories.
As a general convention of this package, '/' is used as directory
separator both for input and output arguments of all functions.
For example:
</p>
<pre>
   exist(\"Modelica/Mechanics/Rotational.mo\");
</pre>
<p>
The functions provide the mapping to the directory separator of the
underlying operating system. Note, that on Windows system the usage
of '\\' as directory separator would be inconvenient, because this
character is also the escape character in Modelica and C Strings.
</p>
<p>
In the table below an example call to every function is given:
</p>
<table border=1 cellspacing=0 cellpadding=2>
  <tr><th><b><i>Function/type</i></b></th><th><b><i>Description</i></b></th></tr>
  <tr><td valign=\"top\"><a href=\"modelica://Modelica.Utilities.Files.list\">list</a>(name)</td>
      <td valign=\"top\"> List content of file or of directory.</td>
  </tr>
  <tr><td valign=\"top\"><a href=\"modelica://Modelica.Utilities.Files.copy\">copy</a>(oldName, newName)<br>
          <a href=\"modelica://Modelica.Utilities.Files.copy\">copy</a>(oldName, newName, replace=false)</td>
      <td valign=\"top\"> Generate a copy of a file or of a directory.</td>
  </tr>
  <tr><td valign=\"top\"><a href=\"modelica://Modelica.Utilities.Files.move\">move</a>(oldName, newName)<br>
          <a href=\"modelica://Modelica.Utilities.Files.move\">move</a>(oldName, newName, replace=false)</td>
      <td valign=\"top\"> Move a file or a directory to another place.</td>
  </tr>
  <tr><td valign=\"top\"><a href=\"modelica://Modelica.Utilities.Files.remove\">remove</a>(name)</td>
      <td valign=\"top\"> Remove file or directory (ignore call, if it does not exist).</td>
  </tr>
  <tr><td valign=\"top\"><a href=\"modelica://Modelica.Utilities.Files.removeFile\">removeFile</a>(name)</td>
      <td valign=\"top\"> Remove file (ignore call, if it does not exist)</td>
  </tr>
  <tr><td valign=\"top\"><a href=\"modelica://Modelica.Utilities.Files.createDirectory\">createDirectory</a>(name)</td>
      <td valign=\"top\"> Create directory (if directory already exists, ignore call).</td>
  </tr>
  <tr><td valign=\"top\">result = <a href=\"modelica://Modelica.Utilities.Files.exist\">exist</a>(name)</td>
      <td valign=\"top\"> Inquire whether file or directory exists.</td>
  </tr>
  <tr><td valign=\"top\"><a href=\"modelica://Modelica.Utilities.Files.assertNew\">assertNew</a>(name,message)</td>
      <td valign=\"top\"> Trigger an assert, if a file or directory exists.</td>
  </tr>
  <tr><td valign=\"top\">fullName = <a href=\"modelica://Modelica.Utilities.Files.fullPathName\">fullPathName</a>(name)</td>
      <td valign=\"top\"> Get full path name of file or directory name.</td>
  </tr>
  <tr><td valign=\"top\">(directory, name, extension) = <a href=\"modelica://Modelica.Utilities.Files.splitPathName\">splitPathName</a>(name)</td>
      <td valign=\"top\"> Split path name in directory, file name kernel, file name extension.</td>
  </tr>
  <tr><td valign=\"top\">fileName = <a href=\"modelica://Modelica.Utilities.Files.temporaryFileName\">temporaryFileName</a>()</td>
      <td valign=\"top\"> Return arbitrary name of a file that does not exist<br>
           and is in a directory where access rights allow to <br>
           write to this file (useful for temporary output of files).</td>
  </tr>
</table>
</HTML>"));
    end Files;

    package Streams "Read from files and write to files"
      extends Modelica.Icons.Package;

      function print "Print string to terminal or file"
        extends Modelica.Icons.Function;
        input String string = "" "String to be printed";
        input String fileName = "" "File where to print (empty string is the terminal)" annotation(
          Dialog(saveSelector(filter = "Text files (*.txt)", caption = "Text file to store the output of print(..)")));

        external "C" ModelicaInternal_print(string, fileName) annotation(
          Library = "ModelicaExternalC");
        annotation(
          Documentation(info = "<HTML>
<h4>Syntax</h4>
<blockquote><pre>
Streams.<b>print</b>(string);
Streams.<b>print</b>(string,fileName);
</pre></blockquote>
<h4>Description</h4>
<p>
Function <b>print</b>(..) opens automatically the given file, if
it is not yet open. If the file does not exist, it is created.
If the file does exist, the given string is appended to the file.
If this is not desired, call \"Files.remove(fileName)\" before calling print
(\"remove(..)\" is silent, if the file does not exist).
The Modelica environment may close the file whenever appropriate.
This can be enforced by calling <b>Streams.close</b>(fileName).
After every call of \"print(..)\" a \"new line\" is printed automatically.
</p>
<h4>Example</h4>
<blockquote><pre>
  Streams.print(\"x = \" + String(x));
  Streams.print(\"y = \" + String(y));
  Streams.print(\"x = \" + String(y), \"mytestfile.txt\");
</pre></blockquote>
<h4>See also</h4>
<p>
<a href=\"modelica://Modelica.Utilities.Streams\">Streams</a>,
<a href=\"modelica://Modelica.Utilities.Streams.error\">Streams.error</a>,
<a href=\"modelica://ModelicaReference.Operators.'String()'\">ModelicaReference.Operators.'String()'</a>
</p>
</HTML>"));
      end print;

      function readFile "Read content of a file and return it in a vector of strings"
        extends Modelica.Icons.Function;
        input String fileName "Name of the file that shall be read" annotation(
          Dialog(loadSelector(filter = "Text files (*.txt)", caption = "Open text file for reading")));
        output String stringVector[countLines(fileName)] "Content of file";
      algorithm
        for i in 1:size(stringVector, 1) loop
          stringVector[i] := readLine(fileName, i);
        end for;
        Streams.close(fileName);
        annotation(
          Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
stringVector = Streams.<b>readFile</b>(fileName)
</pre></blockquote>
<h4>Description</h4>
<p>
Function <b>readFile</b>(..) opens the given file, reads the complete
content, closes the file and returns the content as a vector of strings. Lines are separated by LF or CR-LF; the returned strings do not contain the line separators.
Note, a fileName can be defined as URI by using the helper function
<a href=\"modelica://Modelica.Utilities.Files.loadResource\">loadResource</a>.
</p>
</html>"));
      end readFile;

      function readLine "Reads a line of text from a file and returns it in a string"
        extends Modelica.Icons.Function;
        input String fileName "Name of the file that shall be read" annotation(
          Dialog(loadSelector(filter = "Text files (*.txt)", caption = "Open text file for reading")));
        input Integer lineNumber(min = 1) "Number of line to read";
        output String string "Line of text";
        output Boolean endOfFile "If true, end-of-file was reached when trying to read line";

        external "C" string = ModelicaInternal_readLine(fileName, lineNumber, endOfFile) annotation(
          Library = "ModelicaExternalC");
        annotation(
          Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
(string, endOfFile) = Streams.<b>readLine</b>(fileName, lineNumber)
</pre></blockquote>
<h4>Description</h4>
<p>
Function <b>readLine</b>(..) opens the given file, reads enough of the
content to get the requested line, and returns the line as a string.
Lines are separated by LF or CR-LF; the returned string does not
contain the line separator. The file might remain open after
the call.
</p>
<p>
If lineNumber > countLines(fileName), an empty string is returned
and endOfFile=true. Otherwise endOfFile=false.
</p>
</html>"));
      end readLine;

      function countLines "Returns the number of lines in a file"
        extends Modelica.Icons.Function;
        input String fileName "Name of the file that shall be read" annotation(
          Dialog(loadSelector(filter = "Text files (*.txt)", caption = "Open text file for counting lines")));
        output Integer numberOfLines "Number of lines in file";

        external "C" numberOfLines = ModelicaInternal_countLines(fileName) annotation(
          Library = "ModelicaExternalC");
        annotation(
          Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
numberOfLines = Streams.<b>countLines</b>(fileName)
</pre></blockquote>
<h4>Description</h4>
<p>
Function <b>countLines</b>(..) opens the given file, reads the complete
content, closes the file and returns the number of lines. Lines are
separated by LF or CR-LF.
</p>
</html>"));
      end countLines;

      function error "Print error message and cancel all actions"
        extends Modelica.Icons.Function;
        input String string "String to be printed to error message window";

        external "C" ModelicaError(string) annotation(
          Library = "ModelicaExternalC");
        annotation(
          Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
Streams.<b>error</b>(string);
</pre></blockquote>
<h4>Description</h4>
<p>
Print the string \"string\" as error message and
cancel all actions. Line breaks are characterized
by \"\\n\" in the string.
</p>
<h4>Example</h4>
<blockquote><pre>
  Streams.error(\"x (= \" + String(x) + \")\\nhas to be in the range 0 .. 1\");
</pre></blockquote>
<h4>See also</h4>
<p>
<a href=\"modelica://Modelica.Utilities.Streams\">Streams</a>,
<a href=\"modelica://Modelica.Utilities.Streams.print\">Streams.print</a>,
<a href=\"modelica://ModelicaReference.Operators.'String()'\">ModelicaReference.Operators.'String()'</a>
</p>
</html>"));
      end error;

      function close "Close file"
        extends Modelica.Icons.Function;
        input String fileName "Name of the file that shall be closed" annotation(
          Dialog(loadSelector(filter = "Text files (*.txt)", caption = "Close text file")));

        external "C" ModelicaStreams_closeFile(fileName) annotation(
          Library = "ModelicaExternalC");
        annotation(
          Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
Streams.<b>close</b>(fileName)
</pre></blockquote>
<h4>Description</h4>
<p>
Close file if it is open. Ignore call if
file is already closed or does not exist.
</p>
</html>"));
      end close;
      annotation(
        Documentation(info = "<HTML>
<h4>Library content</h4>
<p>
Package <b>Streams</b> contains functions to input and output strings
to a message window or on files. Note that a string is interpreted
and displayed as html text (e.g., with print(..) or error(..))
if it is enclosed with the Modelica html quotation, e.g.,
</p>
<center>
string = \"&lt;html&gt; first line &lt;br&gt; second line &lt;/html&gt;\".
</center>
<p>
It is a quality of implementation, whether (a) all tags of html are supported
or only a subset, (b) how html tags are interpreted if the output device
does not allow to display formatted text.
</p>
<p>
In the table below an example call to every function is given:
</p>
<table border=1 cellspacing=0 cellpadding=2>
  <tr><th><b><i>Function/type</i></b></th><th><b><i>Description</i></b></th></tr>
  <tr><td valign=\"top\"><a href=\"modelica://Modelica.Utilities.Streams.print\">print</a>(string)<br>
          <a href=\"modelica://Modelica.Utilities.Streams.print\">print</a>(string,fileName)</td>
      <td valign=\"top\"> Print string \"string\" or vector of strings to message window or on
           file \"fileName\".</td>
  </tr>
  <tr><td valign=\"top\">stringVector =
         <a href=\"modelica://Modelica.Utilities.Streams.readFile\">readFile</a>(fileName)</td>
      <td valign=\"top\"> Read complete text file and return it as a vector of strings.</td>
  </tr>
  <tr><td valign=\"top\">(string, endOfFile) =
         <a href=\"modelica://Modelica.Utilities.Streams.readLine\">readLine</a>(fileName, lineNumber)</td>
      <td valign=\"top\">Returns from the file the content of line lineNumber.</td>
  </tr>
  <tr><td valign=\"top\">lines =
         <a href=\"modelica://Modelica.Utilities.Streams.countLines\">countLines</a>(fileName)</td>
      <td valign=\"top\">Returns the number of lines in a file.</td>
  </tr>
  <tr><td valign=\"top\"><a href=\"modelica://Modelica.Utilities.Streams.error\">error</a>(string)</td>
      <td valign=\"top\"> Print error message \"string\" to message window
           and cancel all actions</td>
  </tr>
  <tr><td valign=\"top\"><a href=\"modelica://Modelica.Utilities.Streams.close\">close</a>(fileName)</td>
      <td valign=\"top\"> Close file if it is still open. Ignore call if
           file is already closed or does not exist. </td>
  </tr>
</table>
<p>
Use functions <b>scanXXX</b> from package
<a href=\"modelica://Modelica.Utilities.Strings\">Strings</a>
to parse a string.
</p>
<p>
If Real, Integer or Boolean values shall be printed
or used in an error message, they have to be first converted
to strings with the builtin operator
<a href=\"modelica://ModelicaReference.Operators.'String()'\">ModelicaReference.Operators.'String()'</a>(...).
Example:
</p>
<pre>
  <b>if</b> x &lt; 0 <b>or</b> x &gt; 1 <b>then</b>
     Streams.error(\"x (= \" + String(x) + \") has to be in the range 0 .. 1\");
  <b>end if</b>;
</pre>
</html>"));
    end Streams;

    package Strings "Operations on strings"
      extends Modelica.Icons.Package;

      function length "Returns length of string"
        extends Modelica.Icons.Function;
        input String string;
        output Integer result "Number of characters of string";

        external "C" result = ModelicaStrings_length(string) annotation(
          Library = "ModelicaExternalC");
        annotation(
          Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
Strings.<b>length</b>(string);
</pre></blockquote>
<h4>Description</h4>
<p>
Returns the number of characters of \"string\".
</p>
</html>"));
      end length;

      function substring "Returns a substring defined by start and end index"
        extends Modelica.Icons.Function;
        input String string "String from which a substring is inquired";
        input Integer startIndex(min = 1) "Character position of substring begin (index=1 is first character in string)";
        input Integer endIndex(min = 1) "Character position of substring end";
        output String result "String containing substring string[startIndex:endIndex]";

        external "C" result = ModelicaStrings_substring(string, startIndex, endIndex) annotation(
          Library = "ModelicaExternalC");
        annotation(
          Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
string2 = Strings.<b>substring</b>(string, startIndex, endIndex);
</pre></blockquote>
<h4>Description</h4>
<p>
This function returns
the substring from position startIndex
up to and including position endIndex of \"string\" .
</p>
<p>
If index, startIndex, or endIndex are not correct, e.g.,
if endIndex &gt; length(string), an assert is triggered.
</p>
<h4>Example</h4>
<blockquote><pre>
  string1 := \"This is line 111\";
  string2 := Strings.substring(string1,9,12); // string2 = \"line\"
</pre></blockquote>
</html>"));
      end substring;

      function repeat "Repeat a string n times"
        extends Modelica.Icons.Function;
        input Integer n(min = 0) = 1 "Number of occurrences";
        input String string = " " "String that is repeated";
        output String repeatedString "String containing n concatenated strings";
      algorithm
        repeatedString := "";
        for i in 1:n loop
          repeatedString := repeatedString + string;
        end for;
        annotation(
          Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
string2 = Strings.<b>repeat</b>(n);
string2 = Strings.<b>repeat</b>(n, string=\" \");
</pre></blockquote>
<h4>Description</h4>
<p>
The first form returns a string consisting of n blanks.
</p>
<p>
The second form returns a string consisting of n substrings
defined by the optional argument \"string\".
</p>
</html>"));
      end repeat;

      function compare "Compare two strings lexicographically"
        extends Modelica.Icons.Function;
        input String string1;
        input String string2;
        input Boolean caseSensitive = true "= false, if case of letters is ignored";
        output Modelica.Utilities.Types.Compare result "Result of comparison";

        external "C" result = ModelicaStrings_compare(string1, string2, caseSensitive) annotation(
          Library = "ModelicaExternalC");
        annotation(
          Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
result = Strings.<b>compare</b>(string1, string2);
result = Strings.<b>compare</b>(string1, string2, caseSensitive=true);
</pre></blockquote>
<h4>Description</h4>
<p>
Compares two strings. If the optional argument caseSensitive=false,
upper case letters are treated as if they would be lower case letters.
The result of the comparison is returned as:
</p>
<pre>
  result = Modelica.Utilities.Types.Compare.Less     // string1 &lt; string2
         = Modelica.Utilities.Types.Compare.Equal    // string1 = string2
         = Modelica.Utilities.Types.Compare.Greater  // string1 &gt; string2
</pre>
<p>
Comparison is with regards to lexicographical order,
e.g., \"a\" &lt; \"b\";
</p>
</html>"));
      end compare;

      function isEqual "Determine whether two strings are identical"
        extends Modelica.Icons.Function;
        input String string1;
        input String string2;
        input Boolean caseSensitive = true "= false, if lower and upper case are ignored for the comparison";
        output Boolean identical "True, if string1 is identical to string2";
      algorithm
        identical := compare(string1, string2, caseSensitive) == Types.Compare.Equal;
        annotation(
          Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
Strings.<b>isEqual</b>(string1, string2);
Strings.<b>isEqual</b>(string1, string2, caseSensitive=true);
</pre></blockquote>
<h4>Description</h4>
<p>
Compare whether two strings are identical,
optionally ignoring case.
</p>
</html>"));
      end isEqual;

      function isEmpty "Return true if a string is empty (has only white space characters)"
        extends Modelica.Icons.Function;
        input String string;
        output Boolean result "True, if string is empty";
      protected
        Integer nextIndex;
        Integer len;
      algorithm
        nextIndex := Strings.Advanced.skipWhiteSpace(string);
        len := Strings.length(string);
        if len < 1 or nextIndex > len then
          result := true;
        else
          result := false;
        end if;
        annotation(
          Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
Strings.<b>isEmpty</b>(string);
</pre></blockquote>
<h4>Description</h4>
<p>
Returns true if the string has no characters or if the string consists
only of white space characters. Otherwise, false is returned.
</p>

<h4>Example</h4>
<blockquote><pre>
  isEmpty(\"\");       // returns true
  isEmpty(\"   \");    // returns true
  isEmpty(\"  abc\");  // returns false
  isEmpty(\"a\");      // returns false
</pre></blockquote>
</html>"));
      end isEmpty;

      function count "Count the number of non-overlapping occurrences of a string"
        extends Modelica.Icons.Function;
        input String string "String that is analyzed";
        input String searchString "String that is searched for in string";
        input Integer startIndex(min = 1) = 1 "Start search at index startIndex";
        input Boolean caseSensitive = true "= false, if lower and upper case are ignored for count";
        output Integer result "Number of occurrences of 'searchString' in 'string'";
      protected
        Integer lenSearchString = length(searchString);
        Integer i = startIndex;
      algorithm
        result := 0;
        while i <> 0 loop
          i := find(string, searchString, i, caseSensitive);
          if i > 0 then
            result := result + 1;
            i := i + lenSearchString;
          end if;
        end while;
        annotation(
          Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
Strings.<b>count</b>(string, searchString)
Strings.<b>count</b>(string, searchString, startIndex=1,
                     caseSensitive=true)
</pre></blockquote>
<h4>Description</h4>
<p>
Returns the number of non-overlapping occurrences of string \"searchString\"
in \"string\". The search is started at index \"startIndex\" (default = 1).
If the optional argument \"caseSensitive\" is false,
for the counting it does not matter whether a letter is upper
or lower case.
/p>
</html>"));
      end count;

      function find "Find first occurrence of a string within another string"
        extends Modelica.Icons.Function;
        input String string "String that is analyzed";
        input String searchString "String that is searched for in string";
        input Integer startIndex(min = 1) = 1 "Start search at index startIndex";
        input Boolean caseSensitive = true "= false, if lower and upper case are ignored for the search";
        output Integer index "Index of the beginning of the first occurrence of 'searchString' within 'string', or zero if not present";
      protected
        Integer lengthSearchString = length(searchString);
        Integer len = lengthSearchString - 1;
        Integer i = startIndex;
        Integer i_max = length(string) - lengthSearchString + 1;
      algorithm
        index := 0;
        while i <= i_max loop
          if isEqual(substring(string, i, i + len), searchString, caseSensitive) then
            index := i;
            i := i_max + 1;
          else
            i := i + 1;
          end if;
        end while;
        annotation(
          Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
index = Strings.<b>find</b>(string, searchString);
index = Strings.<b>find</b>(string, searchString, startIndex=1,
                     caseSensitive=true);
</pre></blockquote>
<h4>Description</h4>
<p>
Finds first occurrence of \"searchString\" within \"string\"
and return the corresponding index.
Start search at index \"startIndex\" (default = 1).
If the optional argument \"caseSensitive\" is false, lower
and upper case are ignored for the search.
If \"searchString\" is not found, a value of \"0\" is returned.
</p>
</html>"));
      end find;

      function findLast "Find last occurrence of a string within another string"
        extends Modelica.Icons.Function;
        input String string "String that is analyzed";
        input String searchString "String that is searched for in string";
        input Integer startIndex(min = 0) = 0 "Start search at index startIndex. If startIndex = 0, start at length(string)";
        input Boolean caseSensitive = true "= false, if lower and upper case are ignored for the search";
        output Integer index "Index of the beginning of the last occurrence of 'searchString' within 'string', or zero if not present";
      protected
        Integer lenString = length(string);
        Integer lenSearchString = length(searchString);
        Integer iMax = lenString - lenSearchString + 1;
        Integer i;
      algorithm
        i := if startIndex == 0 or startIndex > iMax then iMax else startIndex;
        index := 0;
        while i >= 1 loop
          if isEqual(substring(string, i, i + lenSearchString - 1), searchString, caseSensitive) then
            index := i;
            i := 0;
          else
            i := i - 1;
          end if;
        end while;
        annotation(
          Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
index = Strings.<b>findLast</b>(string, searchString);
index = Strings.<b>findLast</b>(string, searchString,
                         startIndex=length(string), caseSensitive=true,
</pre></blockquote>
<h4>Description</h4>
<p>
Finds first occurrence of \"searchString\" within \"string\"
when searching from the last character of \"string\"
backwards, and return the corresponding index.
Start search at index \"startIndex\" (default = 0;
if startIndex = 0, search starts at length(string)).
If the optional argument \"caseSensitive\" is false, lower
and upper case are ignored for the search.
If \"searchString\" is not found, a value of \"0\" is returned.
</p>
</html>"));
      end findLast;

      function replace "Replace non-overlapping occurrences of a string from left to right"
        extends Modelica.Icons.Function;
        input String string "String to be modified";
        input String searchString "Replace non-overlapping occurrences of 'searchString' in 'string' with 'replaceString'";
        input String replaceString "String that replaces 'searchString' in 'string'";
        input Integer startIndex = 1 "Start search at index startIndex";
        input Boolean replaceAll = true "if false, replace only the first occurrence, otherwise all occurrences";
        input Boolean caseSensitive = true "= false, if lower and upper case are ignored when searching for searchString";
        output String result "Resultant string of replacement operation";
      protected
        Integer lenString = length(string);
        Integer lenSearchString = length(searchString);
        Integer i = startIndex;
        Integer i_found;
      algorithm
        result := if startIndex == 1 then "" else substring(string, 1, startIndex - 1);
        while i > 0 loop
          i_found := find(string, searchString, i, caseSensitive);
          if i_found > 0 then
            result := if i_found == 1 then replaceString else result + (if i_found - 1 < i then "" else substring(string, i, i_found - 1)) + replaceString;
            i := i_found + lenSearchString;
            if i > lenString then
              i := 0;
            elseif not replaceAll then
              result := result + substring(string, i, lenString);
              i := 0;
            end if;
          elseif lenString < i then
            i := 0;
          else
            result := result + substring(string, i, lenString);
            i := 0;
          end if;
        end while;
        annotation(
          Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
Strings.<b>replace</b>(string, searchString, replaceString);
Strings.<b>replace</b>(string, searchString, replaceString,
                startIndex=1, replaceAll=true, caseSensitive=true);
</pre></blockquote>
<h4>Description</h4>
<p>
Search in \"string\" for \"searchString\" and replace the found
substring by \"replaceString\".
</p>
<ul>
<li> The search starts at the first character of \"string\",
     or at character position \"startIndex\",
     if this optional argument is provided.</li>
<li> If the optional argument \"replaceAll\" is <b>true</b> (default),
     all occurrences of \"searchString\" are replaced.
     If the argument is <b>false</b>, only the first occurrence
     is replaced. </li>
<li> The search for \"searchString\" distinguishes upper and lower
     case letters. If the optional argument \"caseSensitive\" is
     <b>false</b>,
     the search ignores whether letters are upper
     or lower case. </li>
</ul>
<p>
The function returns the \"string\" with the
performed replacements.
</p>
</html>"));
      end replace;

      function sort "Sort vector of strings in alphabetic order"
        extends Modelica.Icons.Function;
        input String stringVector1[:] "vector of strings";
        input Boolean caseSensitive = true "= false, if lower and upper case are ignored when comparing elements of stringVector1";
        output String stringVector2[size(stringVector1, 1)] "string1 sorted in alphabetical order";
        /* shellsort algorithm; should be improved later */
      protected
        Integer gap;
        Integer i;
        Integer j;
        String tempString;
        Integer nStringVector1 = size(stringVector1, 1);
        Boolean swap;
      algorithm
        stringVector2 := stringVector1;
        gap := div(nStringVector1, 2);
        while gap > 0 loop
          i := gap;
          while i < nStringVector1 loop
            j := i - gap;
            if j >= 0 then
              swap := compare(stringVector2[j + 1], stringVector2[j + gap + 1], caseSensitive) == Modelica.Utilities.Types.Compare.Greater;
            else
              swap := false;
            end if;
            while swap loop
              tempString := stringVector2[j + 1];
              stringVector2[j + 1] := stringVector2[j + gap + 1];
              stringVector2[j + gap + 1] := tempString;
              j := j - gap;
              if j >= 0 then
                swap := compare(stringVector2[j + 1], stringVector2[j + gap + 1], caseSensitive) == Modelica.Utilities.Types.Compare.Greater;
              else
                swap := false;
              end if;
            end while;
            i := i + 1;
          end while;
          gap := div(gap, 2);
        end while;
        annotation(
          Documentation(info = "<HTML>
<h4>Syntax</h4>
<blockquote><pre>
stringVector2 = Streams.<b>sort</b>(stringVector1);
stringVector2 = Streams.<b>sort</b>(stringVector1, caseSensitive=true);
</pre></blockquote>
<h4>Description</h4>
<p>
Function <b>sort</b>(..) sorts a string vector stringVector1
in lexicographical order and returns the result in stringVector2.
If the optional argument \"caseSensitive\" is <b>false</b>, lower
and upper case letters are not distinguished.
</p>
<h4>Example</h4>
<blockquote><pre>
  s1 = {\"force\", \"angle\", \"pressure\"};
  s2 = Strings.sort(s1);
       -> s2 = {\"angle\", \"force\", \"pressure\"};
</pre></blockquote>
</HTML>"));
      end sort;

      function scanToken "Scan for the next token and return it"
        extends Modelica.Icons.Function;
        input String string "String to be scanned";
        input Integer startIndex(min = 1) = 1 "Start scanning of string at character startIndex";
        input Boolean unsigned = false "= true, if Real and Integer tokens shall not start with a sign";
        output Types.TokenValue token "Scanned token";
        output Integer nextIndex "Index of character after the found token; = 0, if NoToken";
      protected
        Integer startTokenIndex;
      algorithm
// Initialize token
        token.real := 0.0;
        token.integer := 0;
        token.boolean := false;
        token.string := "";
// skip white space and line comments
        startTokenIndex := Advanced.skipLineComments(string, startIndex);
        if startTokenIndex > length(string) then
          token.tokenType := Modelica.Utilities.Types.TokenType.NoToken;
          nextIndex := startTokenIndex;
        else
          (nextIndex, token.integer) := Advanced.scanInteger(string, startTokenIndex, unsigned);
          token.tokenType := Types.TokenType.IntegerToken;
          if nextIndex == startTokenIndex then
            (nextIndex, token.real) := Advanced.scanReal(string, startTokenIndex, unsigned);
            token.tokenType := Types.TokenType.RealToken;
          end if;
          if nextIndex == startTokenIndex then
            (nextIndex, token.string) := Advanced.scanString(string, startTokenIndex);
            token.tokenType := Types.TokenType.StringToken;
          end if;
          if nextIndex == startTokenIndex then
            (nextIndex, token.string) := Advanced.scanIdentifier(string, startTokenIndex);
            if nextIndex > startTokenIndex then
              if token.string == "false" then
                token.string := "";
                token.boolean := false;
                token.tokenType := Types.TokenType.BooleanToken;
              elseif token.string == "true" then
                token.string := "";
                token.boolean := true;
                token.tokenType := Types.TokenType.BooleanToken;
              else
                token.tokenType := Types.TokenType.IdentifierToken;
              end if;
            end if;
          end if;
          if nextIndex == startTokenIndex then
            token.string := substring(string, startTokenIndex, startTokenIndex);
            token.tokenType := Types.TokenType.DelimiterToken;
            nextIndex := startTokenIndex + 1;
          end if;
        end if;
// scan Integer number
// scan Real number
// scan String
// scan Identifier or Boolean
// scan Delimiter
        annotation(
          Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
(token, nextIndex) = Strings.<b>scanToken</b>(string, startIndex, unsigned=false);
</pre></blockquote>
<h4>Description</h4>
<p>
Function <b>scanToken</b> scans the string starting at index
\"startIndex\" and returns the next token, as well as the
index directly after the token. The returned token is a record
that holds the type of the token and the value of the token:
</p>
<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\">token.tokenType</td>
      <td valign=\"top\">Type of the token, see below</td></tr>
  <tr><td valign=\"top\">token.real</td>
      <td valign=\"top\">Real value if tokenType == TokenType.RealToken</td></tr>
  <tr><td valign=\"top\">token.integer</td>
      <td valign=\"top\">Integer value if tokenType == TokenType.IntegerToken</td></tr>
  <tr><td valign=\"top\">token.boolean</td>
      <td valign=\"top\">Boolean value if tokenType == TokenType.BooleanToken</td></tr>
  <tr><td valign=\"top\">token.string</td>
      <td valign=\"top\">String value if tokenType == TokenType.StringToken/IdentifierToken/DelimiterToken</td></tr>
</table>
<p>
Variable token.tokenType is an enumeration (emulated as a package
with constants) that can have the following values:
</p>
<pre>
   import T = Modelica.Utilities.Types.TokenType;
</pre>
<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\">T.RealToken</td>
      <td valign=\"top\">Modelica Real literal (e.g., 1.23e-4)</td></tr>
  <tr><td valign=\"top\">T.IntegerToken</td>
      <td valign=\"top\">Modelica Integer literal (e.g., 123)</td></tr>
  <tr><td valign=\"top\">T.BooleanToken</td>
      <td valign=\"top\">Modelica Boolean literal (e.g., false)</td></tr>
  <tr><td valign=\"top\">T.StringToken</td>
      <td valign=\"top\">Modelica String literal (e.g., \"string 123\")</td></tr>
  <tr><td valign=\"top\">T.IdentifierToken</td>
      <td valign=\"top\">Modelica identifier (e.g., \"force_a\")</td></tr>
  <tr><td valign=\"top\">T.DelimiterToken</td>
      <td valign=\"top\">any character without white space that does not appear<br>
          as first character in the tokens above (e.g., \"&amp;\")</td></tr>
  <tr><td valign=\"top\">T.NoToken</td>
      <td valign=\"top\">White space, line comments and no other token<br>
          until the end of the string</td></tr>
</table>
<p>
Modelica line comments (\"// ... end-of-line/end-of-string\")
as well as white space is ignored.
If \"unsigned=true\", a Real or Integer literal
is not allowed to start with a \"+\" or \"-\" sign.
</p>
<h4>Example</h4>
<blockquote><pre>
  import Modelica.Utilities.Strings;
  import T = Modelica.Utilities.Types.TokenType;
  (token, index) := Strings.scanToken(string);
  <b>if</b> token.tokenType == T.RealToken <b>then</b>
     realValue := token.real;
  <b>elseif</b> token.tokenType == T.IntegerToken <b>then</b>
     integerValue := token.integer;
  <b>elseif</b> token.tokenType == T.BooleanToken<b> then</b>
     booleanValue := token.boolean;
  <b>elseif</b> token.tokenType == T.Identifier <b>then</b>
     name := token.string;
  <b>else</b>
     Strings.syntaxError(string,index,\"Expected Real, Integer, Boolean or identifier token\");
  <b>end if</b>;
</pre></blockquote>
</html>"));
      end scanToken;

      function scanReal "Scan for the next Real number and trigger an assert if not present"
        extends Modelica.Icons.Function;
        input String string "String to be scanned";
        input Integer startIndex(min = 1) = 1 "Start scanning of string at character startIndex";
        input Boolean unsigned = false "= true, if Real token shall not start with a sign";
        input String message = "" "Message used in error message if scan is not successful";
        output Real number "Value of real number";
        output Integer nextIndex "index of character after the found number";
      algorithm
        (nextIndex, number) := Advanced.scanReal(string, startIndex, unsigned);
        if nextIndex == startIndex then
          nextIndex := Advanced.skipWhiteSpace(string, startIndex);
          if unsigned then
            syntaxError(string, nextIndex, "Expected a Real number without a sign " + message);
          else
            syntaxError(string, nextIndex, "Expected a Real number " + message);
          end if;
        end if;
        annotation(
          Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
             number = Strings.<b>scanReal</b>(string);
(number, nextIndex) = Strings.<b>scanReal</b>(string, startIndex=1,
                                            unsigned=false, message=\"\");
</pre></blockquote>
<h4>Description</h4>
<p>
The first form, \"scanReal(string)\", scans \"string\" for a
Real number with leading white space and returns the value.
</p>
<p>
The second form, \"scanReal(string,startIndex,unsigned)\",
scans the string starting at index
\"startIndex\", checks whether the next token is a Real literal
and returns its value as a Real number, as well as the
index directly after the Real number.
If the optional argument \"unsigned\" is <b>true</b>,
the real number shall not have a leading \"+\" or \"-\" sign.
</p>
<p>
If the required Real number with leading white space
is not present in \"string\", an assert is triggered.
</p>
</html>"));
      end scanReal;

      function scanInteger "Scan for the next Integer number and trigger an assert if not present"
        extends Modelica.Icons.Function;
        input String string "String to be scanned";
        input Integer startIndex(min = 1) = 1 "Start scanning of string at character startIndex";
        input Boolean unsigned = false "= true, if Integer token shall not start with a sign";
        input String message = "" "Message used in error message if scan is not successful";
        output Integer number "Value of Integer number";
        output Integer nextIndex "Index of character after the found number";
      algorithm
        (nextIndex, number) := Advanced.scanInteger(string, startIndex, unsigned);
        if nextIndex == startIndex then
          nextIndex := Advanced.skipWhiteSpace(string, startIndex);
          if unsigned then
            syntaxError(string, nextIndex, "Expected an Integer number without a sign " + message);
          else
            syntaxError(string, nextIndex, "Expected an Integer number " + message);
          end if;
        end if;
        annotation(
          Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
             number = Strings.<b>scanInteger</b>(string);
(number, nextIndex) = Strings.<b>scanInteger</b>(string, startIndex=1,
                                               unsigned=false, message=\"\");
</pre></blockquote>
<h4>Description</h4>
<p>
Function <b>scanInteger</b> scans the string starting at index
\"startIndex\", checks whether the next token is an Integer literal
and returns its value as an Integer number, as well as the
index directly after the Integer number. An assert is triggered,
if the scanned string does not contain an Integer literal with optional
leading white space.
</p>
</html>"));
      end scanInteger;

      function scanBoolean "Scan for the next Boolean number and trigger an assert if not present"
        extends Modelica.Icons.Function;
        input String string "String to be scanned";
        input Integer startIndex(min = 1) = 1 "Start scanning of string at character startIndex";
        input String message = "" "Message used in error message if scan is not successful";
        output Boolean number "Value of Boolean";
        output Integer nextIndex "Index of character after the found number";
      protected
        String identifier;
      algorithm
        (nextIndex, identifier) := Advanced.scanIdentifier(string, startIndex);
        if nextIndex > startIndex then
          if identifier == "false" then
            number := false;
          elseif identifier == "true" then
            number := true;
          else
            nextIndex := startIndex;
          end if;
        end if;
        if nextIndex == startIndex then
          nextIndex := Advanced.skipWhiteSpace(string, startIndex);
          syntaxError(string, nextIndex, "Expected a Boolean constant, i.e., \"false\" or \"true\" " + message);
        end if;
        annotation(
          Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
             number = Strings.<b>scanBoolean</b>(string);
(number, nextIndex) = Strings.<b>scanBoolean</b>(string, startIndex=1, message=\"\");
</pre></blockquote>
<h4>Description</h4>
<p>
Function <b>scanBoolean</b> scans the string starting at index
\"startIndex\", checks whether the next token is a Boolean literal
(i.e., is either the string \"false\" or \"true\", if converted to lower case letters)
and returns its value as a Boolean number, as well as the
index directly after the Boolean number. An assert is triggered,
if the scanned string does not contain a Boolean literal with optional
leading white space.
</p>
</html>"));
      end scanBoolean;

      function scanString "Scan for the next Modelica string and trigger an assert if not present"
        extends Modelica.Icons.Function;
        input String string "String to be scanned";
        input Integer startIndex(min = 1) = 1 "Start scanning of string at character startIndex";
        input String message = "" "Message used in error message if scan is not successful";
        output String result "Value of string";
        output Integer nextIndex "Index of character after the found string";
      algorithm
        (nextIndex, result) := Advanced.scanString(string, startIndex);
        if nextIndex == startIndex then
          nextIndex := Advanced.skipWhiteSpace(string, startIndex);
          syntaxError(string, nextIndex, "Expected a string enclosed in double quotes " + message);
        end if;
        annotation(
          Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
             string2 = Strings.<b>scanString</b>(string);
(string2, nextIndex) = Strings.<b>scanString</b>(string, startIndex=1, message=\"\");
</pre></blockquote>
<h4>Description</h4>
<p>
Function <b>scanString</b> scans the string starting at index
\"startIndex\", checks whether the next token is a String literal
and returns its value as a String, as well as the
index directly after the String. An assert is triggered,
if the scanned string does not contain a String literal with optional
leading white space.
</p>
</html>"));
      end scanString;

      function scanIdentifier "Scan for the next Identifier and trigger an assert if not present"
        extends Modelica.Icons.Function;
        input String string "String to be scanned";
        input Integer startIndex(min = 1) = 1 "Start scanning of identifier at character startIndex";
        input String message = "" "Message used in error message if scan is not successful";
        output String identifier "Value of Identifier";
        output Integer nextIndex "Index of character after the found identifier";
      algorithm
        (nextIndex, identifier) := Advanced.scanIdentifier(string, startIndex);
        if nextIndex == startIndex then
          nextIndex := Advanced.skipWhiteSpace(string, startIndex);
          syntaxError(string, nextIndex, "Expected an identifier " + message);
        end if;
        annotation(
          Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
             identifier = Strings.<b>scanIdentifier</b>(string);
(identifier, nextIndex) = Strings.<b>scanIdentifier</b>(string, startIndex=1, message=\"\");
</pre></blockquote>
<h4>Description</h4>
<p>
Function <b>scanIdentifier</b> scans the string starting at index
\"startIndex\", checks whether the next token is an Identifier
and returns its value as a string, as well as the
index directly after the Identifier. An assert is triggered,
if the scanned string does not contain an Identifier with optional
leading white space.
</p>
</html>"));
      end scanIdentifier;

      function scanDelimiter "Scan for the next delimiter and trigger an assert if not present"
        extends Modelica.Icons.Function;
        input String string "String to be scanned";
        input Integer startIndex(min = 1) = 1 "Start scanning of delimiters at character startIndex";
        input String requiredDelimiters[:] = {","} "Delimiters that are searched";
        input String message = "" "Message used in error message if scan is not successful";
        output String delimiter "Found delimiter";
        output Integer nextIndex "Index of character after the found delimiter";
      protected
        Integer lenString = length(string);
        Integer lenDelimiter;
        Integer nDelimiters = size(requiredDelimiters, 1);
        Integer endIndex;
        Boolean found;
        Integer i;

        encapsulated function concatenate "Concatenate strings together"
          import Modelica;
          extends Modelica.Icons.Function;
          input String strings[:];
          output String string;
        algorithm
          string := "{\"";
          for i in 1:size(strings, 1) loop
            if i == 1 then
              string := "{\"" + strings[1] + "\"";
            else
              string := string + ", \"" + strings[i] + "\"";
            end if;
          end for;
          string := string + "}";
        end concatenate;
      algorithm
        nextIndex := Advanced.skipLineComments(string, startIndex);
        found := false;
        i := 1;
        while not found and i <= nDelimiters loop
          lenDelimiter := length(requiredDelimiters[i]);
          if lenDelimiter == 0 then
            found := true;
            delimiter := "";
          else
            endIndex := nextIndex + lenDelimiter - 1;
            if endIndex <= lenString then
              if substring(string, nextIndex, endIndex) == requiredDelimiters[i] then
                found := true;
                delimiter := requiredDelimiters[i];
              end if;
            end if;
            i := i + 1;
          end if;
        end while;
        if found then
          nextIndex := nextIndex + lenDelimiter;
        else
          if size(requiredDelimiters, 1) == 1 then
            syntaxError(string, nextIndex, "Expected delimiter \"" + requiredDelimiters[1] + "\"\n" + message);
          else
            syntaxError(string, nextIndex, "Expected a delimiter out of " + concatenate(requiredDelimiters) + "\n" + message);
          end if;
        end if;
        annotation(
          Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
             delimiter = Strings.<b>scanDelimiter</b>(string);
(delimiter, nextIndex) = Strings.<b>scanDelimiter</b>(string, startIndex=1,
                                 requiredDelimiters={\",\"}, message=\"\");
</pre></blockquote>
<h4>Description</h4>
<p>
Function <b>scanDelimiter</b> scans the string starting at index
\"startIndex\", checks whether the next token is a delimiter string
and returns its value as a string, as well as the
index directly after the delimiter. An assert is triggered,
if the scanned string does not contain a delimiter out of the
list of requiredDelimiters. Input argument requiredDelimiters is a vector
of strings. The elements may have any length, including length 0.
If an element of the requiredDelimiters is zero, white space
is treated as delimiter. The function returns delimiter=\"\" and nextIndex
is the index of the first non white space character.
</p>
</html>"));
      end scanDelimiter;

      function scanNoToken "Scan string and check that it contains no more token"
        extends Modelica.Icons.Function;
        input String string "String to be scanned";
        input Integer startIndex(min = 1) = 1 "Start scanning of string at character startIndex";
        input String message = "" "Message used in error message if scan is not successful";
      protected
        Integer nextIndex;
      algorithm
        nextIndex := Advanced.skipLineComments(string, startIndex);
        if nextIndex <= length(string) then
          syntaxError(string, nextIndex, "Expected no more token " + message);
        end if;
        annotation(
          Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
Strings.<b>scanNoToken</b>(string, startIndex=1, message=\"\");
</pre></blockquote>
<h4>Description</h4>
<p>
Function <b>scanNoToken</b> scans the string starting at index
\"startIndex\" and checks whether there is no more token in the
string. An assert is triggered if this is not the case,
using the \"message\" argument as additional explanation in
the error text.
</p>
</html>"));
      end scanNoToken;

      function syntaxError "Print an error message, a string and the index at which scanning detected an error"
        extends Modelica.Icons.Function;
        input String string "String that has an error at position index";
        input Integer index "Index of string at which scanning detected an error";
        input String message = "" "String printed at end of error message";
      protected
        Integer maxIndex = 40;
        Integer maxLenString = 60;
        Integer lenString = length(string);
        String errString;
        Integer index2 = if index < 1 then 1 else if index > lenString then lenString else index;
      algorithm
// if "string" is too long, skip part of the string when printing it
        if index2 <= maxIndex then
          errString := string;
        else
          errString := "... " + substring(string, index2 - maxIndex, lenString);
          index2 := maxIndex + 5;
        end if;
// To mark right position
        if length(errString) > maxLenString then
          errString := substring(errString, 1, maxLenString) + " ...";
        end if;
// Print error message
        Streams.error("Syntax error at character " + String(index) + " of\n" + errString + "\n" + repeat(index2 - 1, " ") + "*" + "\n" + message);
        annotation(
          Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
Strings.<b>syntaxError</b>(string, index, message);
</pre></blockquote>
<h4>Description</h4>
<p>
Function <b>syntaxError</b> prints an error message in the
following form:
</p>
<pre>
   Syntax error at column &lt;index&gt; of
   &lt;string&gt;
       ^       // shows character that is wrong
   &lt;message&gt;
</pre>
<p>
where the strings withing &lt;..&gt; are the actual values of the
input arguments of the function.
</p>
<p>
If the given string is too long, only a relevant
part of the string is printed.
</p>
</html>"));
      end syntaxError;

      package Advanced "Advanced scanning functions"
        extends Modelica.Icons.Package;

        function scanReal "Scans a signed real number"
          extends Modelica.Icons.Function;
          input String string;
          input Integer startIndex(min = 1) = 1 "Index where scanning starts";
          input Boolean unsigned = false "= true, if number shall not start with '+' or '-'";
          output Integer nextIndex "Index after the found token (success=true) or index at which scanning failed (success=false)";
          output Real number "Value of Real number";

          external "C" ModelicaStrings_scanReal(string, startIndex, unsigned, nextIndex, number) annotation(
            Library = "ModelicaExternalC");
          annotation(
            Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
(nextIndex, realNumber) = <b>scanReal</b>(string, startIndex=1, unsigned=false);
</pre></blockquote>
<h4>Description</h4>
<p>
Starts scanning of \"string\" at position \"startIndex\".
First skips white space and scans afterwards a number
of type Real with an optional sign according to the Modelica grammar:
</p>
<pre>
    real     ::= [sign] unsigned [fraction] [exponent]
    sign     ::= '+' | '-'
    unsigned ::= digit [unsigned]
    fraction ::= '.' [unsigned]
    exponent ::= ('e' | 'E') [sign] unsigned
    digit    ::= '0'|'1'|'2'|'3'|'4'|'5'|'6'|'7'|'8'|'9'
</pre>
<p>
If successful, the function returns nextIndex = index of character
directly after the found real number, as well as the value
in the second output argument.
</p>
<p>
If not successful, on return nextIndex = startIndex and
the second output argument is zero.
</p>
<p>
If the optional argument \"unsigned\" is <b>true</b>, the number
shall not start with '+' or '-'. The default of \"unsigned\" is <b>false</b>.
</p>
<h4>See also</h4>
<a href=\"modelica://Modelica.Utilities.Strings.Advanced\">Strings.Advanced</a>.
</html>"));
        end scanReal;

        function scanInteger "Scans signed integer number"
          extends Modelica.Icons.Function;
          input String string;
          input Integer startIndex(min = 1) = 1;
          input Boolean unsigned = false "= true, if number shall not start with '+' or '-'";
          output Integer nextIndex "Index after the found token (success=true) or index at which scanning failed (success=false)";
          output Integer number "Value of Integer number";

          external "C" ModelicaStrings_scanInteger(string, startIndex, unsigned, nextIndex, number) annotation(
            Library = "ModelicaExternalC");
          annotation(
            Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
(nextIndex, integerNumber) = <b>scanInteger</b>(string, startIndex=1, unsigned=false);
</pre></blockquote>
<h4>Description</h4>
<p>
Starts scanning of \"string\" at position \"startIndex\".
First skips white space and scans afterwards a signed number
of type Integer. An Integer starts with an optional '+'
or '-', immediately
followed by a non-empty sequence of digits.
</p>
<p>
If successful, the function returns nextIndex = index of character
directly after the found Integer number, as well as the Integer value
in the second output argument.
</p>
<p>
If not successful, on return nextIndex = startIndex and
the second output argument is zero.
</p>
<p>
Note, a Real number, such as \"123.4\", is not treated
as an Integer number and scanInteger will return
nextIndex = startIndex in this case.
</p>
<p>
If the optional argument \"unsigned\" is <b>true</b>, the number
shall not start with '+' or '-'. The default of \"unsigned\" is <b>false</b>.
</p>
<h4>See also</h4>
<a href=\"modelica://Modelica.Utilities.Strings.Advanced\">Strings.Advanced</a>.
</html>"));
        end scanInteger;

        function scanString "Scan string"
          extends Modelica.Icons.Function;
          input String string;
          input Integer startIndex(min = 1) = 1 "Index where scanning starts";
          output Integer nextIndex "Index after the found token (success=true) or index at which scanning failed (success=false)";
          output String string2 "Value of String token";

          external "C" ModelicaStrings_scanString(string, startIndex, nextIndex, string2) annotation(
            Library = "ModelicaExternalC");
          annotation(
            Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
(nextIndex, string2) = <b>scanString</b>(string, startIndex=1);
</pre></blockquote>
<h4>Description</h4>
<p>
Starts scanning of \"string\" at position \"startIndex\".
First skips white space and scans afterwards a string
according to the Modelica grammar, i.e., a string
enclosed in double quotes.
</p>
<p>
If successful, the function returns nextIndex = index of character
directly after the found string, as well as the string value
in the second output argument.
</p>
<p>
If not successful, on return nextIndex = startIndex and
the second output argument is an empty string.
</p>
<h4>See also</h4>
<a href=\"modelica://Modelica.Utilities.Strings.Advanced\">Strings.Advanced</a>.
</html>"));
        end scanString;

        function scanIdentifier "Scans simple identifiers"
          extends Modelica.Icons.Function;
          input String string;
          input Integer startIndex(min = 1) = 1 "Index where scanning starts";
          output Integer nextIndex "Index after the found token (success=true) or index at which scanning failed (success=false)";
          output String identifier "Value of identifier token";

          external "C" ModelicaStrings_scanIdentifier(string, startIndex, nextIndex, identifier) annotation(
            Library = "ModelicaExternalC");
          annotation(
            Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
(nextIndex, identifier) = <b>scanIdentifier</b>(string, startIndex=1);
</pre></blockquote>
<h4>Description</h4>
<p>
Starts scanning of \"string\" at position \"startIndex\".
First skips white space and scans afterwards a Modelica
identifier, i.e., a sequence of characters starting with
a letter (\"a\"..\"z\" or \"A\"..\"Z\") followed by letters,
digits or underscores (\"_\").
</p>
<p>
If successful, the function returns nextIndex = index of character
directly after the found identifier, as well as the identifier
as string in the second output argument.
</p>
<p>
If not successful, on return nextIndex = startIndex and
the second output argument is an empty string.
</p>
<h4>See also</h4>
<a href=\"modelica://Modelica.Utilities.Strings.Advanced\">Strings.Advanced</a>.
</html>"));
        end scanIdentifier;

        function skipWhiteSpace "Scans white space"
          extends Modelica.Icons.Function;
          input String string;
          input Integer startIndex(min = 1) = 1;
          output Integer nextIndex;

          external "C" nextIndex = ModelicaStrings_skipWhiteSpace(string, startIndex) annotation(
            Library = "ModelicaExternalC");
          annotation(
            Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
nextIndex = <b>skipWhiteSpace</b>(string, startIndex);
</pre></blockquote>
<h4>Description</h4>
<p>
Starts scanning of \"string\" at position \"startIndex\" and
skips white space. The function returns nextIndex = index of character
of the first non white space character.
</p>
<h4>See also</h4>
<a href=\"modelica://Modelica.Utilities.Strings.Advanced\">Strings.Advanced</a>.
</html>"));
        end skipWhiteSpace;

        function skipLineComments "Scans comments and white space"
          extends Modelica.Icons.Function;
          input String string;
          input Integer startIndex(min = 1) = 1;
          output Integer nextIndex;
        protected
          Integer lenString = length(string);
          Boolean scanning;
          Boolean lineComment;
        algorithm
          nextIndex := startIndex;
          scanning := true;
          while scanning loop
            nextIndex := Advanced.skipWhiteSpace(string, nextIndex);
            if nextIndex + 1 <= lenString then
              if substring(string, nextIndex, nextIndex + 1) == "//" then
                nextIndex := nextIndex + 2;
                if nextIndex <= lenString then
                  lineComment := true;
                  while lineComment loop
                    if substring(string, nextIndex, nextIndex) == "\n" then
                      lineComment := false;
                    end if;
                    nextIndex := nextIndex + 1;
                    if nextIndex > lenString then
                      lineComment := false;
                      scanning := false;
                    end if;
                  end while;
                else
                  scanning := false;
                end if;
              else
                scanning := false;
              end if;
            else
              scanning := false;
            end if;
          end while;
// search end of line comment
          annotation(
            Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
nextIndex = <b>skipLineComments</b>(string, startIndex);
</pre></blockquote>
<h4>Description</h4>
<p>
Starts scanning of \"string\" at position \"startIndex\".
First skips white space and scans afterwards a Modelica (C/C++)
line comment, i.e., a sequence of characters that
starts with \"//\" and ends with an end-of-line \"\\n\" or
with the end of the string. If end-of-line is reached,
the function continues to skip white space and
scanning of line comments until end-of-string is
reached, or another token is detected.
</p>
<p>
If successful, the function returns nextIndex = index of character
directly after the found line comment.
</p>
<p>
If not successful, on return nextIndex = startIndex.
</p>
<h4>See also</h4>
<a href=\"modelica://Modelica.Utilities.Strings.Advanced\">Strings.Advanced</a>.
</html>"));
        end skipLineComments;
        annotation(
          Documentation(info = "<html>
<h4>Library content</h4>
<p>
Package <b>Strings.Advanced</b> contains basic scanning
functions. These functions should be <b>not called</b> directly, because
it is much simpler to utilize the higher level functions \"Strings.scanXXX\".
The functions of the \"Strings.Advanced\" library provide
the basic interface in order to implement the higher level
functions in package \"Strings\".
</p>
<p>
Library \"Advanced\" provides the following functions:
</p>
<pre>
  (nextIndex, realNumber)    = <a href=\"modelica://Modelica.Utilities.Strings.Advanced.scanReal\">scanReal</a>        (string, startIndex, unsigned=false);
  (nextIndex, integerNumber) = <a href=\"modelica://Modelica.Utilities.Strings.Advanced.scanInteger\">scanInteger</a>     (string, startIndex, unsigned=false);
  (nextIndex, string2)       = <a href=\"modelica://Modelica.Utilities.Strings.Advanced.scanString\">scanString</a>      (string, startIndex);
  (nextIndex, identifier)    = <a href=\"modelica://Modelica.Utilities.Strings.Advanced.scanIdentifier\">scanIdentifier</a>  (string, startIndex);
   nextIndex                 = <a href=\"modelica://Modelica.Utilities.Strings.Advanced.skipWhiteSpace\">skipWhiteSpace</a>  (string, startIndex);
   nextIndex                 = <a href=\"modelica://Modelica.Utilities.Strings.Advanced.skipLineComments\">skipLineComments</a>(string, startIndex);
</pre>
<p>
All functions perform the following actions:
</p>
<ol>
<li> Scanning starts at character position \"startIndex\" of
     \"string\" (startIndex has a default of 1).
<li> First, white space is skipped, such as blanks (\" \"), tabs (\"\\t\"), or newline (\"\\n\")</li>
<li> Afterwards, the required token is scanned.</li>
<li> If successful, on return nextIndex = index of character
     directly after the found token and the token value is returned
     as second output argument.<br>
     If not successful, on return nextIndex = startIndex.
     </li>
</ol>
<p>
The following additional rules apply for the scanning:
</p>
<ul>
<li> Function <a href=\"modelica://Modelica.Utilities.Strings.Advanced.scanReal\">scanReal</a>:<br>
     Scans a full number including one optional leading \"+\" or \"-\" (if unsigned=false)
     according to the Modelica grammar. For example, \"+1.23e-5\", \"0.123\" are
     Real numbers, but \".1\" is not.
     Note, an Integer number, such as \"123\" is also treated as a Real number.<br>&nbsp;</li>
<li> Function <a href=\"modelica://Modelica.Utilities.Strings.Advanced.scanInteger\">scanInteger</a>:<br>
     Scans an Integer number including one optional leading \"+\"
     or \"-\" (if unsigned=false) according to the Modelica (and C/C++) grammar.
     For example, \"+123\", \"20\" are Integer numbers.
     Note, a Real number, such as \"123.4\" is not an Integer and
     scanInteger returns nextIndex = startIndex.<br>&nbsp;</li>
<li> Function <a href=\"modelica://Modelica.Utilities.Strings.Advanced.scanString\">scanString</a>:<br>
     Scans a String according to the Modelica (and C/C++) grammar, e.g.,
     \"This is a \"string\"\" is a valid string token.<br>&nbsp;</li>
<li> Function <a href=\"modelica://Modelica.Utilities.Strings.Advanced.scanIdentifier\">scanIdentifier</a>:<br>
     Scans a Modelica identifier, i.e., the identifier starts either
     with a letter, followed by letters, digits or \"_\".
     For example, \"w_rel\", \"T12\".<br>&nbsp;</li>
<li> Function <a href=\"modelica://Modelica.Utilities.Strings.Advanced.skipLineComments\">skipLineComments</a><br>
     Skips white space and Modelica (C/C++) line comments iteratively.
     A line comment starts with \"//\" and ends either with an
     end-of-line (\"\\n\") or the end of the \"string\". </li>
</ul>
</html>"));
      end Advanced;
      annotation(
        Documentation(info = "<HTML>
<h4>Library content</h4>
<p>
Package <b>Strings</b> contains functions to manipulate strings.
</p>
<p>
In the table below an example
call to every function is given using the <b>default</b> options.
</p>
<table border=1 cellspacing=0 cellpadding=2>
  <tr><th><b><i>Function</i></b></th><th><b><i>Description</i></b></th></tr>
  <tr><td valign=\"top\">len = <a href=\"modelica://Modelica.Utilities.Strings.length\">length</a>(string)</td>
      <td valign=\"top\">Returns length of string</td></tr>
  <tr><td valign=\"top\">string2 = <a href=\"modelica://Modelica.Utilities.Strings.substring\">substring</a>(string1,startIndex,endIndex)
       </td>
      <td valign=\"top\">Returns a substring defined by start and end index</td></tr>
  <tr><td valign=\"top\">result = <a href=\"modelica://Modelica.Utilities.Strings.repeat\">repeat</a>(n)<br>
 result = <a href=\"modelica://Modelica.Utilities.Strings.repeat\">repeat</a>(n,string)</td>
      <td valign=\"top\">Repeat a blank or a string n times.</td></tr>
  <tr><td valign=\"top\">result = <a href=\"modelica://Modelica.Utilities.Strings.compare\">compare</a>(string1, string2)</td>
      <td valign=\"top\">Compares two substrings with regards to alphabetical order</td></tr>
  <tr><td valign=\"top\">identical =
<a href=\"modelica://Modelica.Utilities.Strings.isEqual\">isEqual</a>(string1,string2)</td>
      <td valign=\"top\">Determine whether two strings are identical</td></tr>
  <tr><td valign=\"top\">result = <a href=\"modelica://Modelica.Utilities.Strings.count\">count</a>(string,searchString)</td>
      <td valign=\"top\">Count the number of occurrences of a string</td></tr>
  <tr>
<td valign=\"top\">index = <a href=\"modelica://Modelica.Utilities.Strings.find\">find</a>(string,searchString)</td>
      <td valign=\"top\">Find first occurrence of a string in another string</td></tr>
<tr>
<td valign=\"top\">index = <a href=\"modelica://Modelica.Utilities.Strings.findLast\">findLast</a>(string,searchString)</td>
      <td valign=\"top\">Find last occurrence of a string in another string</td></tr>
  <tr><td valign=\"top\">string2 = <a href=\"modelica://Modelica.Utilities.Strings.replace\">replace</a>(string,searchString,replaceString)</td>
      <td valign=\"top\">Replace one or all occurrences of a string</td></tr>
  <tr><td valign=\"top\">stringVector2 = <a href=\"modelica://Modelica.Utilities.Strings.sort\">sort</a>(stringVector1)</td>
      <td valign=\"top\">Sort vector of strings in alphabetic order</td></tr>
  <tr><td valign=\"top\">(token, index) = <a href=\"modelica://Modelica.Utilities.Strings.scanToken\">scanToken</a>(string,startIndex)</td>
      <td valign=\"top\">Scan for a token (Real/Integer/Boolean/String/Identifier/Delimiter/NoToken)</td></tr>
  <tr><td valign=\"top\">(number, index) = <a href=\"modelica://Modelica.Utilities.Strings.scanReal\">scanReal</a>(string,startIndex)</td>
      <td valign=\"top\">Scan for a Real constant</td></tr>
  <tr><td valign=\"top\">(number, index) = <a href=\"modelica://Modelica.Utilities.Strings.scanInteger\">scanInteger</a>(string,startIndex)</td>
      <td valign=\"top\">Scan for an Integer constant</td></tr>
  <tr><td valign=\"top\">(boolean, index) = <a href=\"modelica://Modelica.Utilities.Strings.scanBoolean\">scanBoolean</a>(string,startIndex)</td>
      <td valign=\"top\">Scan for a Boolean constant</td></tr>
  <tr><td valign=\"top\">(string2, index) = <a href=\"modelica://Modelica.Utilities.Strings.scanString\">scanString</a>(string,startIndex)</td>
      <td valign=\"top\">Scan for a String constant</td></tr>
  <tr><td valign=\"top\">(identifier, index) = <a href=\"modelica://Modelica.Utilities.Strings.scanIdentifier\">scanIdentifier</a>(string,startIndex)</td>
      <td valign=\"top\">Scan for an identifier</td></tr>
  <tr><td valign=\"top\">(delimiter, index) = <a href=\"modelica://Modelica.Utilities.Strings.scanDelimiter\">scanDelimiter</a>(string,startIndex)</td>
      <td valign=\"top\">Scan for delimiters</td></tr>
  <tr><td valign=\"top\"><a href=\"modelica://Modelica.Utilities.Strings.scanNoToken\">scanNoToken</a>(string,startIndex)</td>
      <td valign=\"top\">Check that remaining part of string consists solely of <br>
          white space or line comments (\"// ...\\n\").</td></tr>
  <tr><td valign=\"top\"><a href=\"modelica://Modelica.Utilities.Strings.syntaxError\">syntaxError</a>(string,index,message)</td>
      <td valign=\"top\"> Print a \"syntax error message\" as well as a string and the <br>
           index at which scanning detected an error</td></tr>
</table>
<p>
The functions \"compare\", \"isEqual\", \"count\", \"find\", \"findLast\", \"replace\", \"sort\"
have the optional
input argument <b>caseSensitive</b> with default <b>true</b>.
If <b>false</b>, the operation is carried out without taking
into account whether a character is upper or lower case.
</p>
</HTML>"));
    end Strings;

    package System "Interaction with environment"
      extends Modelica.Icons.Package;

      function getWorkDirectory "Get full path name of work directory"
        extends Modelica.Icons.Function;
        output String directory "Full path name of work directory";
        // POSIX function "getcwd"

        external "C" directory = ModelicaInternal_getcwd(0) annotation(
          Library = "ModelicaExternalC");
        annotation(
          Documentation(info = "<html>

</html>"));
      end getWorkDirectory;

      function setWorkDirectory "Set work directory"
        extends Modelica.Icons.Function;
        input String directory "New work directory";
        // POSIX function "chdir"

        external "C" ModelicaInternal_chdir(directory) annotation(
          Library = "ModelicaExternalC");
        annotation(
          Documentation(info = "<html>

</html>"));
      end setWorkDirectory;

      function getEnvironmentVariable "Get content of environment variable"
        extends Modelica.Icons.Function;
        input String name "Name of environment variable";
        input Boolean convertToSlash = false "True, if native directory separators in 'result' shall be changed to '/'";
        output String content "Content of environment variable (empty, if not existent)";
        output Boolean exist "= true, if environment variable exists; = false, if it does not exist";

        external "C" ModelicaInternal_getenv(name, convertToSlash, content, exist) annotation(
          Library = "ModelicaExternalC");
        annotation(
          Documentation(info = "<html>

</html>"));
      end getEnvironmentVariable;

      function setEnvironmentVariable "Set content of local environment variable"
        extends Modelica.Icons.Function;
        input String name "Name of environment variable";
        input String content "Value of the environment variable";
        input Boolean convertFromSlash = false "True, if '/' in content shall be changed to the native directory separator";

        external "C" ModelicaInternal_setenv(name, content, convertFromSlash) annotation(
          Library = "ModelicaExternalC");
        annotation(
          Documentation(info = "<html>

</html>"));
      end setEnvironmentVariable;

      function command "Execute command in default shell"
        extends Modelica.Icons.Function;
        input String string "String to be passed to shell";
        output Integer result "Return value from command (depends on environment)";

        external "C" result = system(string) annotation(
          Library = "ModelicaExternalC");
        annotation(
          Documentation(info = "<html>

</html>"));
      end command;

      function exit "Terminate execution of Modelica environment"
        extends Modelica.Icons.Function;
        input Integer status = 0 "Result to be returned by environment (0 means success)";

        external "C" ModelicaInternal_exit(status) annotation(
          Library = "ModelicaExternalC");
        annotation(
          Documentation(info = "<html>

</html>"));
      end exit;
      annotation(
        Documentation(info = "<html>
<p>
This package contains functions to interact with the environment.
</p>
</html>"));
    end System;

    package Types "Type definitions used in package Modelica.Utilities"
      extends Modelica.Icons.TypesPackage;
      type Compare = enumeration(Less "String 1 is lexicographically less than string 2", Equal "String 1 is identical to string 2", Greater "String 1 is lexicographically greater than string 2") "Enumeration defining comparison of two strings";
      type FileType = enumeration(NoFile "No file exists", RegularFile "Regular file", Directory "Directory", SpecialFile "Special file (pipe, FIFO, device, etc.)") "Enumeration defining the type of a file";
      type TokenType = enumeration(RealToken, IntegerToken, BooleanToken, StringToken, IdentifierToken, DelimiterToken, NoToken) "Enumeration defining the token type";

      record TokenValue "Value of token"
        extends Modelica.Icons.Record;
        TokenType tokenType "Type of token";
        Real real "Value if tokenType == TokenType.RealToken";
        Integer integer "Value if tokenType == TokenType.IntegerToken";
        Boolean boolean "Value if tokenType == TokenType.BooleanToken";
        String string "Value if tokenType == TokenType.StringToken/IdentifierToken/DelimiterToken";
        annotation(
          Documentation(info = "<html>

</html>"));
      end TokenValue;
      annotation(
        Documentation(info = "<html>
<p>
This package contains type definitions used in Modelica.Utilities.
</p>

</html>"));
    end Types;

    package Internal "Internal components that a user should usually not directly utilize"
      extends Modelica.Icons.InternalPackage;

      partial package PartialModelicaServices "Interfaces of components requiring a tool specific implementation"
        extends Modelica.Icons.InternalPackage;

        package Animation "Models and functions for 3-dim. animation"
          extends Modelica.Icons.Package;

          partial model PartialShape "Interface for 3D animation of elementary shapes"
            import SI = Modelica.SIunits;
            import Modelica.Mechanics.MultiBody.Frames;
            import Modelica.Mechanics.MultiBody.Types;
            parameter Types.ShapeType shapeType = "box" "Type of shape (box, sphere, cylinder, pipecylinder, cone, pipe, beam, gearwheel, spring, <external shape>)";
            input Frames.Orientation R = Frames.nullRotation() "Orientation object to rotate the world frame into the object frame" annotation(
              Dialog);
            input SI.Position r[3] = {0, 0, 0} "Position vector from origin of world frame to origin of object frame, resolved in world frame" annotation(
              Dialog);
            input SI.Position r_shape[3] = {0, 0, 0} "Position vector from origin of object frame to shape origin, resolved in object frame" annotation(
              Dialog);
            input Real lengthDirection[3](each final unit = "1") = {1, 0, 0} "Vector in length direction, resolved in object frame" annotation(
              Dialog);
            input Real widthDirection[3](each final unit = "1") = {0, 1, 0} "Vector in width direction, resolved in object frame" annotation(
              Dialog);
            input SI.Length length = 0 "Length of visual object" annotation(
              Dialog);
            input SI.Length width = 0 "Width of visual object" annotation(
              Dialog);
            input SI.Length height = 0 "Height of visual object" annotation(
              Dialog);
            input Types.ShapeExtra extra = 0.0 "Additional size data for some of the shape types" annotation(
              Dialog);
            input Real color[3] = {255, 0, 0} "Color of shape" annotation(
              Dialog(colorSelector = true));
            input Types.SpecularCoefficient specularCoefficient = 0.7 "Reflection of ambient light (= 0: light is completely absorbed)" annotation(
              Dialog);
            annotation(
              Documentation(info = "<html>

<p>
This model is documented at
<a href=\"modelica://Modelica.Mechanics.MultiBody.Visualizers.Advanced.Shape\">Modelica.Mechanics.MultiBody.Visualizers.Advanced.Shape</a>.
</p>

</html>"));
          end PartialShape;

          model PartialSurface "Interface for 3D animation of surfaces"
            import Modelica.Mechanics.MultiBody.Frames;
            import Modelica.Mechanics.MultiBody.Types;
            input Frames.Orientation R = Frames.nullRotation() "Orientation object to rotate the world frame into the surface frame" annotation(
              Dialog(group = "Surface frame"));
            input Modelica.SIunits.Position r_0[3] = {0, 0, 0} "Position vector from origin of world frame to origin of surface frame, resolved in world frame" annotation(
              Dialog(group = "Surface frame"));
            parameter Integer nu = 2 "Number of points in u-Dimension" annotation(
              Dialog(group = "Surface properties"));
            parameter Integer nv = 2 "Number of points in v-Dimension" annotation(
              Dialog(group = "Surface properties"));
            replaceable function surfaceCharacteristic = Modelica.Mechanics.MultiBody.Interfaces.partialSurfaceCharacteristic "Function defining the surface characteristic" annotation(
              choicesAllMatching = true,
              Dialog(group = "Surface properties"));
            parameter Boolean wireframe = false "= true: 3D model will be displayed without faces" annotation(
              Dialog(group = "Material properties"),
              choices(checkBox = true));
            parameter Boolean multiColoredSurface = false "= true: Color is defined for each surface point" annotation(
              Dialog(group = "Material properties"),
              choices(checkBox = true));
            input Real color[3] = {255, 0, 0} "Color of surface" annotation(
              Dialog(colorSelector = true, group = "Material properties", enable = not multiColoredSurface));
            input Types.SpecularCoefficient specularCoefficient = 0.7 "Reflection of ambient light (= 0: light is completely absorbed)" annotation(
              Dialog(group = "Material properties"));
            input Real transparency = 0 "Transparency of shape: 0 (= opaque) ... 1 (= fully transparent)" annotation(
              Dialog(group = "Material properties"));
            annotation(
              Documentation(info = "<html>
<p>
This model is documented at
<a href=\"modelica://Modelica.Mechanics.MultiBody.Visualizers.Advanced.Surface\">Modelica.Mechanics.MultiBody.Visualizers.Advanced.Surface</a>.
</p>

</html>"));
          end PartialSurface;
        end Animation;

        package ExternalReferences "Functions to access external resources"
          extends Modelica.Icons.InternalPackage;

          partial function PartialLoadResource "Interface for tool specific function to return the absolute path name of a URI or local file name"
            extends Modelica.Icons.Function;
            input String uri "URI or local file name";
            output String fileReference "Absolute path name of file";
            annotation(
              Documentation(info = "<html>
<p>
This partial function defines the function interface of a tool-specific implementation
in package ModelicaServices. The interface is documented at
<a href=\"modelica://Modelica.Utilities.Files.loadResource\">Modelica.Utilities.Internal.FileSystem.loadResource</a>.
</p>

</html>"));
          end PartialLoadResource;
        end ExternalReferences;
        annotation(
          Documentation(info = "<html>

<p>
This package contains interfaces of a set of functions and models used in the
Modelica Standard Library that requires a <b>tool specific implementation</b>.
There is an associated package called <b>ModelicaServices</b>. A tool vendor
should provide a proper implementation of this library for the corresponding
tool. The default implementation is \"do nothing\".
In the Modelica Standard Library, the models and functions of ModelicaServices
are used.
</p>
</html>"));
      end PartialModelicaServices;

      package FileSystem "Internal package with external functions as interface to the file system"
        extends Modelica.Icons.InternalPackage;

        function mkdir "Make directory (POSIX: 'mkdir')"
          extends Modelica.Icons.Function;
          input String directoryName "Make a new directory";

          external "C" ModelicaInternal_mkdir(directoryName) annotation(
            Library = "ModelicaExternalC");
        end mkdir;

        function rmdir "Remove empty directory (POSIX function 'rmdir')"
          extends Modelica.Icons.Function;
          input String directoryName "Empty directory to be removed";

          external "C" ModelicaInternal_rmdir(directoryName) annotation(
            Library = "ModelicaExternalC");
        end rmdir;

        function stat "Inquire file information (POSIX function 'stat')"
          extends Modelica.Icons.Function;
          input String name "Name of file, directory, pipe etc.";
          output Types.FileType fileType "Type of file";

          external "C" fileType = ModelicaInternal_stat(name) annotation(
            Library = "ModelicaExternalC");
        end stat;

        function rename "Rename existing file or directory (C function 'rename')"
          extends Modelica.Icons.Function;
          input String oldName "Current name";
          input String newName "New name";

          external "C" ModelicaInternal_rename(oldName, newName) annotation(
            Library = "ModelicaExternalC");
        end rename;

        function removeFile "Remove existing file (C function 'remove')"
          extends Modelica.Icons.Function;
          input String fileName "File to be removed";

          external "C" ModelicaInternal_removeFile(fileName) annotation(
            Library = "ModelicaExternalC");
        end removeFile;

        function copyFile "Copy existing file (C functions 'fopen', 'getc', 'putc', 'fclose')"
          extends Modelica.Icons.Function;
          input String fromName "Name of file to be copied";
          input String toName "Name of copy of file";

          external "C" ModelicaInternal_copyFile(fromName, toName) annotation(
            Library = "ModelicaExternalC");
        end copyFile;

        function readDirectory "Read names of a directory (POSIX functions opendir, readdir, closedir)"
          extends Modelica.Icons.Function;
          input String directory "Name of the directory from which information is desired";
          input Integer nNames "Number of names that are returned (inquire with getNumberOfFiles)";
          output String names[nNames] "All file and directory names in any order from the desired directory";

          external "C" ModelicaInternal_readDirectory(directory, nNames, names) annotation(
            Library = "ModelicaExternalC");
        end readDirectory;

        function getNumberOfFiles "Get number of files and directories in a directory (POSIX functions opendir, readdir, closedir)"
          extends Modelica.Icons.Function;
          input String directory "Directory name";
          output Integer result "Number of files and directories present in 'directory'";

          external "C" result = ModelicaInternal_getNumberOfFiles(directory) annotation(
            Library = "ModelicaExternalC");
        end getNumberOfFiles;
        annotation(
          Documentation(info = "<html>
<p>
Package <b>Internal.FileSystem</b> is an internal package that contains
low level functions as interface to the file system.
These functions should not be called directly in a scripting
environment since more convenient functions are provided
in packages Files and Systems.
</p>
<p>
Note, the functions in this package are direct interfaces to
functions of POSIX and of the standard C library. Errors
occurring in these functions are treated by triggering
a Modelica assert. Therefore, the functions in this package
return only for a successful operation. Furthermore, the
representation of a string is hidden by this interface,
especially if the operating system supports Unicode characters.
</p>
</html>"));
      end FileSystem;
    end Internal;
    annotation(
      Icon(coordinateSystem(extent = {{-100.0, -100.0}, {100.0, 100.0}}), graphics = {Polygon(origin = {1.3835, -4.1418}, rotation = 45.0, fillColor = {64, 64, 64}, pattern = LinePattern.None, fillPattern = FillPattern.Solid, points = {{-15.0, 93.333}, {-15.0, 68.333}, {0.0, 58.333}, {15.0, 68.333}, {15.0, 93.333}, {20.0, 93.333}, {25.0, 83.333}, {25.0, 58.333}, {10.0, 43.333}, {10.0, -41.667}, {25.0, -56.667}, {25.0, -76.667}, {10.0, -91.667}, {0.0, -91.667}, {0.0, -81.667}, {5.0, -81.667}, {15.0, -71.667}, {15.0, -61.667}, {5.0, -51.667}, {-5.0, -51.667}, {-15.0, -61.667}, {-15.0, -71.667}, {-5.0, -81.667}, {0.0, -81.667}, {0.0, -91.667}, {-10.0, -91.667}, {-25.0, -76.667}, {-25.0, -56.667}, {-10.0, -41.667}, {-10.0, 43.333}, {-25.0, 58.333}, {-25.0, 83.333}, {-20.0, 93.333}}), Polygon(origin = {10.1018, 5.218}, rotation = -45.0, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, points = {{-15.0, 87.273}, {15.0, 87.273}, {20.0, 82.273}, {20.0, 27.273}, {10.0, 17.273}, {10.0, 7.273}, {20.0, 2.273}, {20.0, -2.727}, {5.0, -2.727}, {5.0, -77.727}, {10.0, -87.727}, {5.0, -112.727}, {-5.0, -112.727}, {-10.0, -87.727}, {-5.0, -77.727}, {-5.0, -2.727}, {-20.0, -2.727}, {-20.0, 2.273}, {-10.0, 7.273}, {-10.0, 17.273}, {-20.0, 27.273}, {-20.0, 82.273}})}),
      Documentation(info = "<html>
<p>
This package contains Modelica <b>functions</b> that are
especially suited for <b>scripting</b>. The functions might
be used to work with strings, read data from file, write data
to file or copy, move and remove files.
</p>
<p>
For an introduction, have especially a look at:
</p>
<ul>
<li> <a href=\"modelica://Modelica.Utilities.UsersGuide\">Modelica.Utilities.User's Guide</a>
     discusses the most important aspects of this library.</li>
<li> <a href=\"modelica://Modelica.Utilities.Examples\">Modelica.Utilities.Examples</a>
     contains examples that demonstrate the usage of this library.</li>
</ul>
<p>
The following main sublibraries are available:
</p>
<ul>
<li> <a href=\"modelica://Modelica.Utilities.Files\">Files</a>
     provides functions to operate on files and directories, e.g.,
     to copy, move, remove files.</li>
<li> <a href=\"modelica://Modelica.Utilities.Streams\">Streams</a>
     provides functions to read from files and write to files.</li>
<li> <a href=\"modelica://Modelica.Utilities.Strings\">Strings</a>
     provides functions to operate on strings. E.g.
     substring, find, replace, sort, scanToken.</li>
<li> <a href=\"modelica://Modelica.Utilities.System\">System</a>
     provides functions to interact with the environment.
     E.g., get or set the working directory or environment
     variables and to send a command to the default shell.</li>
</ul>

<p>
Copyright &copy; 1998-2015, Modelica Association, DLR, and Dassault Syst&egrave;mes AB.
</p>

<p>
<i>This Modelica package is <u>free</u> software and the use is completely at <u>your own risk</u>; it can be redistributed and/or modified under the terms of the Modelica License 2. For license conditions (including the disclaimer of warranty) see <a href=\"modelica://Modelica.UsersGuide.ModelicaLicense2\">Modelica.UsersGuide.ModelicaLicense2</a> or visit <a href=\"https://www.modelica.org/licenses/ModelicaLicense2\"> https://www.modelica.org/licenses/ModelicaLicense2</a>.</i>
</p>

</html>"));
  end Utilities;

  package Constants "Library of mathematical constants and constants of nature (e.g., pi, eps, R, sigma)"
    import SI = Modelica.SIunits;
    import NonSI = Modelica.SIunits.Conversions.NonSIunits;
    extends Modelica.Icons.Package;
    // Mathematical constants
    final constant Real e = Modelica.Math.exp(1.0);
    final constant Real pi = 2 * Modelica.Math.asin(1.0);
    // 3.14159265358979;
    final constant Real D2R = pi / 180 "Degree to Radian";
    final constant Real R2D = 180 / pi "Radian to Degree";
    final constant Real gamma = 0.57721566490153286060 "see http://en.wikipedia.org/wiki/Euler_constant";
    // Machine dependent constants
    final constant Real eps = ModelicaServices.Machine.eps "Biggest number such that 1.0 + eps = 1.0";
    final constant Real small = ModelicaServices.Machine.small "Smallest number such that small and -small are representable on the machine";
    final constant Real inf = ModelicaServices.Machine.inf "Biggest Real number such that inf and -inf are representable on the machine";
    final constant Integer Integer_inf = ModelicaServices.Machine.Integer_inf "Biggest Integer number such that Integer_inf and -Integer_inf are representable on the machine";
    // Constants of nature
    // (name, value, description from http://physics.nist.gov/cuu/Constants/)
    final constant SI.Velocity c = 299792458 "Speed of light in vacuum";
    final constant SI.Acceleration g_n = 9.80665 "Standard acceleration of gravity on earth";
    final constant Real G(final unit = "m3/(kg.s2)") = 6.6742e-11 "Newtonian constant of gravitation";
    final constant SI.FaradayConstant F = 9.64853399e4 "Faraday constant, C/mol";
    final constant Real h(final unit = "J.s") = 6.6260693e-34 "Planck constant";
    final constant Real k(final unit = "J/K") = 1.3806505e-23 "Boltzmann constant";
    final constant Real R(final unit = "J/(mol.K)") = 8.314472 "Molar gas constant";
    final constant Real sigma(final unit = "W/(m2.K4)") = 5.670400e-8 "Stefan-Boltzmann constant";
    final constant Real N_A(final unit = "1/mol") = 6.0221415e23 "Avogadro constant";
    final constant Real mue_0(final unit = "N/A2") = 4 * pi * 1.e-7 "Magnetic constant";
    final constant Real epsilon_0(final unit = "F/m") = 1 / (mue_0 * c * c) "Electric constant";
    final constant NonSI.Temperature_degC T_zero = -273.15 "Absolute zero temperature";
    annotation(
      Documentation(info = "<html>
<p>
This package provides often needed constants from mathematics, machine
dependent constants and constants from nature. The latter constants
(name, value, description) are from the following source:
</p>

<dl>
<dt>Peter J. Mohr and Barry N. Taylor (1999):</dt>
<dd><b>CODATA Recommended Values of the Fundamental Physical Constants: 1998</b>.
    Journal of Physical and Chemical Reference Data, Vol. 28, No. 6, 1999 and
    Reviews of Modern Physics, Vol. 72, No. 2, 2000. See also <a href=
\"http://physics.nist.gov/cuu/Constants/\">http://physics.nist.gov/cuu/Constants/</a></dd>
</dl>

<p>CODATA is the Committee on Data for Science and Technology.</p>

<dl>
<dt><b>Main Author:</b></dt>
<dd><a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a><br>
    Deutsches Zentrum f&uuml;r Luft und Raumfahrt e. V. (DLR)<br>
    Oberpfaffenhofen<br>
    Postfach 11 16<br>
    D-82230 We&szlig;ling<br>
    email: <a href=\"mailto:Martin.Otter@dlr.de\">Martin.Otter@dlr.de</a></dd>
</dl>

<p>
Copyright &copy; 1998-2015, Modelica Association and DLR.
</p>
<p>
<i>This Modelica package is <u>free</u> software and the use is completely at <u>your own risk</u>; it can be redistributed and/or modified under the terms of the Modelica License 2. For license conditions (including the disclaimer of warranty) see <a href=\"modelica://Modelica.UsersGuide.ModelicaLicense2\">Modelica.UsersGuide.ModelicaLicense2</a> or visit <a href=\"https://www.modelica.org/licenses/ModelicaLicense2\"> https://www.modelica.org/licenses/ModelicaLicense2</a>.</i>
</p>
</html>", revisions = "<html>
<ul>
<li><i>Nov 8, 2004</i>
       by <a href=\"http://www.robotic.dlr.de/Christian.Schweiger/\">Christian Schweiger</a>:<br>
       Constants updated according to 2002 CODATA values.</li>
<li><i>Dec 9, 1999</i>
       by <a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a>:<br>
       Constants updated according to 1998 CODATA values. Using names, values
       and description text from this source. Included magnetic and
       electric constant.</li>
<li><i>Sep 18, 1999</i>
       by <a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a>:<br>
       Constants eps, inf, small introduced.</li>
<li><i>Nov 15, 1997</i>
       by <a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a>:<br>
       Realized.</li>
</ul>
</html>"),
      Icon(coordinateSystem(extent = {{-100.0, -100.0}, {100.0, 100.0}}), graphics = {Polygon(origin = {-9.2597, 25.6673}, fillColor = {102, 102, 102}, pattern = LinePattern.None, fillPattern = FillPattern.Solid, points = {{48.017, 11.336}, {48.017, 11.336}, {10.766, 11.336}, {-25.684, 10.95}, {-34.944, -15.111}, {-34.944, -15.111}, {-32.298, -15.244}, {-32.298, -15.244}, {-22.112, 0.168}, {11.292, 0.234}, {48.267, -0.097}, {48.267, -0.097}}, smooth = Smooth.Bezier), Polygon(origin = {-19.9923, -8.3993}, fillColor = {102, 102, 102}, pattern = LinePattern.None, fillPattern = FillPattern.Solid, points = {{3.239, 37.343}, {3.305, 37.343}, {-0.399, 2.683}, {-16.936, -20.071}, {-7.808, -28.604}, {6.811, -22.519}, {9.986, 37.145}, {9.986, 37.145}}, smooth = Smooth.Bezier), Polygon(origin = {23.753, -11.5422}, fillColor = {102, 102, 102}, pattern = LinePattern.None, fillPattern = FillPattern.Solid, points = {{-10.873, 41.478}, {-10.873, 41.478}, {-14.048, -4.162}, {-9.352, -24.8}, {7.912, -24.469}, {16.247, 0.27}, {16.247, 0.27}, {13.336, 0.071}, {13.336, 0.071}, {7.515, -9.983}, {-3.134, -7.271}, {-2.671, 41.214}, {-2.671, 41.214}}, smooth = Smooth.Bezier)}));
  end Constants;

  package Icons "Library of icons"
    partial class Information "Icon for general information packages"
      annotation(
        Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(lineColor = {75, 138, 73}, fillColor = {75, 138, 73}, pattern = LinePattern.None, fillPattern = FillPattern.Solid, extent = {{-100.0, -100.0}, {100.0, 100.0}}), Polygon(origin = {-4.167, -15.0}, fillColor = {255, 255, 255}, pattern = LinePattern.None, fillPattern = FillPattern.Solid, points = {{-15.833, 20.0}, {-15.833, 30.0}, {14.167, 40.0}, {24.167, 20.0}, {4.167, -30.0}, {14.167, -30.0}, {24.167, -30.0}, {24.167, -40.0}, {-5.833, -50.0}, {-15.833, -30.0}, {4.167, 20.0}, {-5.833, 20.0}}, smooth = Smooth.Bezier), Ellipse(origin = {7.5, 56.5}, fillColor = {255, 255, 255}, pattern = LinePattern.None, fillPattern = FillPattern.Solid, extent = {{-12.5, -12.5}, {12.5, 12.5}})}),
        Documentation(info = "<html>
<p>This icon indicates classes containing only documentation, intended for general description of, e.g., concepts and features of a package.</p>
</html>"));
    end Information;

    extends Icons.Package;

    partial class Contact "Icon for contact information"
      annotation(
        Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, 70}, {100, -72}}, lineColor = {0, 0, 0}, fillColor = {235, 235, 235}, fillPattern = FillPattern.Solid), Polygon(points = {{-100, -72}, {100, -72}, {0, 20}, {-100, -72}}, lineColor = {0, 0, 0}, smooth = Smooth.None, fillColor = {215, 215, 215}, fillPattern = FillPattern.Solid), Polygon(points = {{22, 0}, {100, 70}, {100, -72}, {22, 0}}, lineColor = {0, 0, 0}, smooth = Smooth.None, fillColor = {235, 235, 235}, fillPattern = FillPattern.Solid), Polygon(points = {{-100, 70}, {100, 70}, {0, -20}, {-100, 70}}, lineColor = {0, 0, 0}, smooth = Smooth.None, fillColor = {241, 241, 241}, fillPattern = FillPattern.Solid)}),
        Documentation(info = "<html>
<p>This icon shall be used for the contact information of the library developers.</p>
</html>"));
    end Contact;

    partial class ReleaseNotes "Icon for release notes in documentation"
      annotation(
        Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{-80, -100}, {-80, 100}, {0, 100}, {0, 20}, {80, 20}, {80, -100}, {-80, -100}}, lineColor = {0, 0, 0}, fillColor = {245, 245, 245}, fillPattern = FillPattern.Solid), Polygon(points = {{0, 100}, {80, 20}, {0, 20}, {0, 100}}, lineColor = {0, 0, 0}, fillColor = {215, 215, 215}, fillPattern = FillPattern.Solid), Line(points = {{2, -12}, {50, -12}}, color = {0, 0, 0}), Ellipse(extent = {{-56, 2}, {-28, -26}}, lineColor = {0, 0, 0}, fillColor = {215, 215, 215}, fillPattern = FillPattern.Solid), Line(points = {{2, -60}, {50, -60}}, color = {0, 0, 0}), Ellipse(extent = {{-56, -46}, {-28, -74}}, lineColor = {0, 0, 0}, fillColor = {215, 215, 215}, fillPattern = FillPattern.Solid)}),
        Documentation(info = "<html>
<p>This icon indicates release notes and the revision history of a library.</p>
</html>"));
    end ReleaseNotes;

    partial class References "Icon for external references"
      annotation(
        Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{-100, -80}, {-100, 60}, {-80, 54}, {-80, 80}, {-40, 58}, {-40, 100}, {-10, 60}, {90, 60}, {100, 40}, {100, -100}, {-20, -100}, {-100, -80}}, lineColor = {0, 0, 255}, pattern = LinePattern.None, fillColor = {245, 245, 245}, fillPattern = FillPattern.Solid), Polygon(points = {{-20, -100}, {-10, -80}, {90, -80}, {100, -100}, {-20, -100}}, lineColor = {0, 0, 0}), Line(points = {{90, -80}, {90, 60}, {100, 40}, {100, -100}}, color = {0, 0, 0}), Line(points = {{90, 60}, {-10, 60}, {-10, -80}}, color = {0, 0, 0}), Line(points = {{-10, 60}, {-40, 100}, {-40, -40}, {-10, -80}, {-10, 60}}, color = {0, 0, 0}), Line(points = {{-20, -88}, {-80, -60}, {-80, 80}, {-40, 58}}, color = {0, 0, 0}), Line(points = {{-20, -100}, {-100, -80}, {-100, 60}, {-80, 54}}, color = {0, 0, 0}), Line(points = {{10, 30}, {72, 30}}, color = {0, 0, 0}), Line(points = {{10, -10}, {70, -10}}, color = {0, 0, 0}), Line(points = {{10, -50}, {70, -50}}, color = {0, 0, 0})}),
        Documentation(info = "<html>
<p>This icon indicates a documentation class containing references to external documentation and literature.</p>
</html>"));
    end References;

    partial package ExamplesPackage "Icon for packages containing runnable examples"
      extends Modelica.Icons.Package;
      annotation(
        Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(origin = {8.0, 14.0}, lineColor = {78, 138, 73}, fillColor = {78, 138, 73}, pattern = LinePattern.None, fillPattern = FillPattern.Solid, points = {{-58.0, 46.0}, {42.0, -14.0}, {-58.0, -74.0}, {-58.0, 46.0}})}),
        Documentation(info = "<html>
<p>This icon indicates a package that contains executable examples.</p>
</html>"));
    end ExamplesPackage;

    partial model Example "Icon for runnable examples"
      annotation(
        Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(lineColor = {75, 138, 73}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, extent = {{-100, -100}, {100, 100}}), Polygon(lineColor = {0, 0, 255}, fillColor = {75, 138, 73}, pattern = LinePattern.None, fillPattern = FillPattern.Solid, points = {{-36, 60}, {64, 0}, {-36, -60}, {-36, 60}})}),
        Documentation(info = "<html>
<p>This icon indicates an example. The play button suggests that the example can be executed.</p>
</html>"));
    end Example;

    partial package Package "Icon for standard packages"
      annotation(
        Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(lineColor = {200, 200, 200}, fillColor = {248, 248, 248}, fillPattern = FillPattern.HorizontalCylinder, extent = {{-100.0, -100.0}, {100.0, 100.0}}, radius = 25.0), Rectangle(lineColor = {128, 128, 128}, fillPattern = FillPattern.None, extent = {{-100.0, -100.0}, {100.0, 100.0}}, radius = 25.0)}),
        Documentation(info = "<html>
<p>Standard package icon.</p>
</html>"));
    end Package;

    partial package BasesPackage "Icon for packages containing base classes"
      extends Modelica.Icons.Package;
      annotation(
        Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(extent = {{-30.0, -30.0}, {30.0, 30.0}}, lineColor = {128, 128, 128}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid)}),
        Documentation(info = "<html>
<p>This icon shall be used for a package/library that contains base models and classes, respectively.</p>
</html>"));
    end BasesPackage;

    partial package VariantsPackage "Icon for package containing variants"
      extends Modelica.Icons.Package;
      annotation(
        Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(origin = {10.0, 10.0}, fillColor = {76, 76, 76}, pattern = LinePattern.None, fillPattern = FillPattern.Solid, extent = {{-80.0, -80.0}, {-20.0, -20.0}}), Ellipse(origin = {10.0, 10.0}, pattern = LinePattern.None, fillPattern = FillPattern.Solid, extent = {{0.0, -80.0}, {60.0, -20.0}}), Ellipse(origin = {10.0, 10.0}, fillColor = {128, 128, 128}, pattern = LinePattern.None, fillPattern = FillPattern.Solid, extent = {{0.0, 0.0}, {60.0, 60.0}}), Ellipse(origin = {10.0, 10.0}, lineColor = {128, 128, 128}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, extent = {{-80.0, 0.0}, {-20.0, 60.0}})}),
        Documentation(info = "<html>
<p>This icon shall be used for a package/library that contains several variants of one component.</p>
</html>"));
    end VariantsPackage;

    partial package InterfacesPackage "Icon for packages containing interfaces"
      extends Modelica.Icons.Package;
      annotation(
        Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(origin = {20.0, 0.0}, lineColor = {64, 64, 64}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, points = {{-10.0, 70.0}, {10.0, 70.0}, {40.0, 20.0}, {80.0, 20.0}, {80.0, -20.0}, {40.0, -20.0}, {10.0, -70.0}, {-10.0, -70.0}}), Polygon(fillColor = {102, 102, 102}, pattern = LinePattern.None, fillPattern = FillPattern.Solid, points = {{-100.0, 20.0}, {-60.0, 20.0}, {-30.0, 70.0}, {-10.0, 70.0}, {-10.0, -70.0}, {-30.0, -70.0}, {-60.0, -20.0}, {-100.0, -20.0}})}),
        Documentation(info = "<html>
<p>This icon indicates packages containing interfaces.</p>
</html>"));
    end InterfacesPackage;

    partial package SourcesPackage "Icon for packages containing sources"
      extends Modelica.Icons.Package;
      annotation(
        Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(origin = {23.3333, 0.0}, fillColor = {128, 128, 128}, pattern = LinePattern.None, fillPattern = FillPattern.Solid, points = {{-23.333, 30.0}, {46.667, 0.0}, {-23.333, -30.0}}), Rectangle(fillColor = {128, 128, 128}, pattern = LinePattern.None, fillPattern = FillPattern.Solid, extent = {{-70, -4.5}, {0, 4.5}})}),
        Documentation(info = "<html>
<p>This icon indicates a package which contains sources.</p>
</html>"));
    end SourcesPackage;

    partial package SensorsPackage "Icon for packages containing sensors"
      extends Modelica.Icons.Package;
      annotation(
        Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(origin = {0.0, -30.0}, fillColor = {255, 255, 255}, extent = {{-90.0, -90.0}, {90.0, 90.0}}, startAngle = 20.0, endAngle = 160.0), Ellipse(origin = {0.0, -30.0}, fillColor = {128, 128, 128}, pattern = LinePattern.None, fillPattern = FillPattern.Solid, extent = {{-20.0, -20.0}, {20.0, 20.0}}), Line(origin = {0.0, -30.0}, points = {{0.0, 60.0}, {0.0, 90.0}}), Ellipse(origin = {-0.0, -30.0}, fillColor = {64, 64, 64}, pattern = LinePattern.None, fillPattern = FillPattern.Solid, extent = {{-10.0, -10.0}, {10.0, 10.0}}), Polygon(origin = {-0.0, -30.0}, rotation = -35.0, fillColor = {64, 64, 64}, pattern = LinePattern.None, fillPattern = FillPattern.Solid, points = {{-7.0, 0.0}, {-3.0, 85.0}, {0.0, 90.0}, {3.0, 85.0}, {7.0, 0.0}})}),
        Documentation(info = "<html>
<p>This icon indicates a package containing sensors.</p>
</html>"));
    end SensorsPackage;

    partial package UtilitiesPackage "Icon for utility packages"
      extends Modelica.Icons.Package;
      annotation(
        Icon(coordinateSystem(extent = {{-100.0, -100.0}, {100.0, 100.0}}), graphics = {Polygon(origin = {1.3835, -4.1418}, rotation = 45.0, fillColor = {64, 64, 64}, pattern = LinePattern.None, fillPattern = FillPattern.Solid, points = {{-15.0, 93.333}, {-15.0, 68.333}, {0.0, 58.333}, {15.0, 68.333}, {15.0, 93.333}, {20.0, 93.333}, {25.0, 83.333}, {25.0, 58.333}, {10.0, 43.333}, {10.0, -41.667}, {25.0, -56.667}, {25.0, -76.667}, {10.0, -91.667}, {0.0, -91.667}, {0.0, -81.667}, {5.0, -81.667}, {15.0, -71.667}, {15.0, -61.667}, {5.0, -51.667}, {-5.0, -51.667}, {-15.0, -61.667}, {-15.0, -71.667}, {-5.0, -81.667}, {0.0, -81.667}, {0.0, -91.667}, {-10.0, -91.667}, {-25.0, -76.667}, {-25.0, -56.667}, {-10.0, -41.667}, {-10.0, 43.333}, {-25.0, 58.333}, {-25.0, 83.333}, {-20.0, 93.333}}), Polygon(origin = {10.1018, 5.218}, rotation = -45.0, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, points = {{-15.0, 87.273}, {15.0, 87.273}, {20.0, 82.273}, {20.0, 27.273}, {10.0, 17.273}, {10.0, 7.273}, {20.0, 2.273}, {20.0, -2.727}, {5.0, -2.727}, {5.0, -77.727}, {10.0, -87.727}, {5.0, -112.727}, {-5.0, -112.727}, {-10.0, -87.727}, {-5.0, -77.727}, {-5.0, -2.727}, {-20.0, -2.727}, {-20.0, 2.273}, {-10.0, 7.273}, {-10.0, 17.273}, {-20.0, 27.273}, {-20.0, 82.273}})}),
        Documentation(info = "<html>
<p>This icon indicates a package containing utility classes.</p>
</html>"));
    end UtilitiesPackage;

    partial package TypesPackage "Icon for packages containing type definitions"
      extends Modelica.Icons.Package;
      annotation(
        Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(origin = {-12.167, -23}, fillColor = {128, 128, 128}, pattern = LinePattern.None, fillPattern = FillPattern.Solid, points = {{12.167, 65}, {14.167, 93}, {36.167, 89}, {24.167, 20}, {4.167, -30}, {14.167, -30}, {24.167, -30}, {24.167, -40}, {-5.833, -50}, {-15.833, -30}, {4.167, 20}, {12.167, 65}}, smooth = Smooth.Bezier, lineColor = {0, 0, 0}), Polygon(origin = {2.7403, 1.6673}, fillColor = {128, 128, 128}, pattern = LinePattern.None, fillPattern = FillPattern.Solid, points = {{49.2597, 22.3327}, {31.2597, 24.3327}, {7.2597, 18.3327}, {-26.7403, 10.3327}, {-46.7403, 14.3327}, {-48.7403, 6.3327}, {-32.7403, 0.3327}, {-6.7403, 4.3327}, {33.2597, 14.3327}, {49.2597, 14.3327}, {49.2597, 22.3327}}, smooth = Smooth.Bezier)}));
    end TypesPackage;

    partial package IconsPackage "Icon for packages containing icons"
      extends Modelica.Icons.Package;
      annotation(
        Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(origin = {-8.167, -17}, fillColor = {128, 128, 128}, pattern = LinePattern.None, fillPattern = FillPattern.Solid, points = {{-15.833, 20.0}, {-15.833, 30.0}, {14.167, 40.0}, {24.167, 20.0}, {4.167, -30.0}, {14.167, -30.0}, {24.167, -30.0}, {24.167, -40.0}, {-5.833, -50.0}, {-15.833, -30.0}, {4.167, 20.0}, {-5.833, 20.0}}, smooth = Smooth.Bezier, lineColor = {0, 0, 0}), Ellipse(origin = {-0.5, 56.5}, fillColor = {128, 128, 128}, pattern = LinePattern.None, fillPattern = FillPattern.Solid, extent = {{-12.5, -12.5}, {12.5, 12.5}}, lineColor = {0, 0, 0})}));
    end IconsPackage;

    partial package InternalPackage "Icon for an internal package (indicating that the package should not be directly utilized by user)"
      annotation(
        Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(lineColor = {215, 215, 215}, fillColor = {255, 255, 255}, fillPattern = FillPattern.HorizontalCylinder, extent = {{-100, -100}, {100, 100}}, radius = 25), Rectangle(lineColor = {215, 215, 215}, fillPattern = FillPattern.None, extent = {{-100, -100}, {100, 100}}, radius = 25), Ellipse(extent = {{-80, 80}, {80, -80}}, lineColor = {215, 215, 215}, fillColor = {215, 215, 215}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-55, 55}, {55, -55}}, lineColor = {255, 255, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-60, 14}, {60, -14}}, lineColor = {215, 215, 215}, fillColor = {215, 215, 215}, fillPattern = FillPattern.Solid, origin = {0, 0}, rotation = 45)}),
        Documentation(info = "<html>

<p>
This icon shall be used for a package that contains internal classes not to be
directly utilized by a user.
</p>
</html>"));
    end InternalPackage;

    partial package MaterialPropertiesPackage "Icon for package containing property classes"
      extends Modelica.Icons.Package;
      annotation(
        Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(lineColor = {102, 102, 102}, fillColor = {204, 204, 204}, pattern = LinePattern.None, fillPattern = FillPattern.Sphere, extent = {{-60.0, -60.0}, {60.0, 60.0}})}),
        Documentation(info = "<html>
<p>This icon indicates a package that contains properties</p>
</html>"));
    end MaterialPropertiesPackage;

    partial class MaterialProperty "Icon for property classes"
      annotation(
        Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(lineColor = {102, 102, 102}, fillColor = {204, 204, 204}, pattern = LinePattern.None, fillPattern = FillPattern.Sphere, extent = {{-100.0, -100.0}, {100.0, 100.0}})}),
        Documentation(info = "<html>
<p>This icon indicates a property class.</p>
</html>"));
    end MaterialProperty;

    partial class RotationalSensor "Icon representing a round measurement device"
      annotation(
        Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(fillColor = {245, 245, 245}, fillPattern = FillPattern.Solid, extent = {{-70.0, -70.0}, {70.0, 70.0}}), Line(points = {{0.0, 70.0}, {0.0, 40.0}}), Line(points = {{22.9, 32.8}, {40.2, 57.3}}), Line(points = {{-22.9, 32.8}, {-40.2, 57.3}}), Line(points = {{37.6, 13.7}, {65.8, 23.9}}), Line(points = {{-37.6, 13.7}, {-65.8, 23.9}}), Ellipse(lineColor = {64, 64, 64}, fillColor = {255, 255, 255}, extent = {{-12.0, -12.0}, {12.0, 12.0}}), Polygon(origin = {0, 0}, rotation = -17.5, fillColor = {64, 64, 64}, pattern = LinePattern.None, fillPattern = FillPattern.Solid, points = {{-5.0, 0.0}, {-2.0, 60.0}, {0.0, 65.0}, {2.0, 60.0}, {5.0, 0.0}}), Ellipse(fillColor = {64, 64, 64}, pattern = LinePattern.None, fillPattern = FillPattern.Solid, extent = {{-7.0, -7.0}, {7.0, 7.0}})}),
        Documentation(info = "<html>
<p>
This icon is designed for a <b>rotational sensor</b> model.
</p>
</html>"));
    end RotationalSensor;

    partial class TranslationalSensor "Icon representing a linear measurement device"
      annotation(
        Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(fillColor = {245, 245, 245}, fillPattern = FillPattern.Solid, extent = {{-70.0, -60.0}, {70.0, 20.0}}), Polygon(pattern = LinePattern.None, fillPattern = FillPattern.Solid, points = {{0.0, -40.0}, {-10.0, -16.0}, {10.0, -16.0}, {0.0, -40.0}}), Line(points = {{0.0, 0.0}, {0.0, -16.0}}), Line(points = {{-70.0, 0.0}, {0.0, 0.0}}), Line(points = {{-50.0, -40.0}, {-50.0, -60.0}}), Line(points = {{-30.0, -40.0}, {-30.0, -60.0}}), Line(points = {{-10.0, -40.0}, {-10.0, -60.0}}), Line(points = {{10.0, -40.0}, {10.0, -60.0}}), Line(points = {{30.0, -40.0}, {30.0, -60.0}}), Line(points = {{50.0, -40.0}, {50.0, -60.0}})}),
        Documentation(info = "<html>
<p>
This icon is designed for a <b>translational sensor</b> model.
</p></html>"));
    end TranslationalSensor;

    partial function Function "Icon for functions"
      annotation(
        Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Text(lineColor = {0, 0, 255}, extent = {{-150, 105}, {150, 145}}, textString = "%name"), Ellipse(lineColor = {108, 88, 49}, fillColor = {255, 215, 136}, fillPattern = FillPattern.Solid, extent = {{-100, -100}, {100, 100}}), Text(lineColor = {108, 88, 49}, extent = {{-90.0, -90.0}, {90.0, 90.0}}, textString = "f")}),
        Documentation(info = "<html>
<p>This icon indicates Modelica functions.</p>
</html>"));
    end Function;

    partial record Record "Icon for records"
      annotation(
        Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(lineColor = {0, 0, 255}, extent = {{-150, 60}, {150, 100}}, textString = "%name"), Rectangle(origin = {0.0, -25.0}, lineColor = {64, 64, 64}, fillColor = {255, 215, 136}, fillPattern = FillPattern.Solid, extent = {{-100.0, -75.0}, {100.0, 75.0}}, radius = 25.0), Line(points = {{-100.0, 0.0}, {100.0, 0.0}}, color = {64, 64, 64}), Line(origin = {0.0, -50.0}, points = {{-100.0, 0.0}, {100.0, 0.0}}, color = {64, 64, 64}), Line(origin = {0.0, -25.0}, points = {{0.0, 75.0}, {0.0, -75.0}}, color = {64, 64, 64})}),
        Documentation(info = "<html>
<p>
This icon is indicates a record.
</p>
</html>"));
    end Record;

    operator record TypeComplex = Complex "Obsolete class kept only for backwards compatibility (use Complex instead)" annotation(
      Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(lineColor = {160, 160, 164}, fillColor = {160, 160, 164}, fillPattern = FillPattern.Solid, extent = {{-100.0, -100.0}, {100.0, 100.0}}, radius = 25.0), Text(lineColor = {255, 255, 255}, extent = {{-90.0, -50.0}, {90.0, 50.0}}, textString = "C")}),
      Documentation(info = "<html>
<p>
Obsolete class, only kept for backwards compatibility.
The type classes TypeReal, TypeInteger etc. have been introduced to associate an icon to the
built-in base classes Real, Integer etc. Instead for Complex, an icon is already introducded in its
definition (which is not possible for the built-in classes). Therefore, TypeComplex is just an alias
to Complex and is therefore superfluous.
</p>
</html>"));

    type TypeReal "Icon for Real types"
      extends Real;
      annotation(
        Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(lineColor = {160, 160, 164}, fillColor = {160, 160, 164}, fillPattern = FillPattern.Solid, extent = {{-100.0, -100.0}, {100.0, 100.0}}, radius = 25.0), Text(lineColor = {255, 255, 255}, extent = {{-90.0, -50.0}, {90.0, 50.0}}, textString = "R")}),
        Documentation(info = "<html>
<p>
This icon is designed for a <b>Real</b> type.
</p>
</html>"));
    end TypeReal;

    type TypeInteger "Icon for Integer types"
      extends Integer;
      annotation(
        Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(lineColor = {160, 160, 164}, fillColor = {160, 160, 164}, fillPattern = FillPattern.Solid, extent = {{-100.0, -100.0}, {100.0, 100.0}}, radius = 25.0), Text(lineColor = {255, 255, 255}, extent = {{-90.0, -50.0}, {90.0, 50.0}}, textString = "I")}),
        Documentation(info = "<html>
<p>
This icon is designed for an <b>Integer</b> type.
</p>
</html>"));
    end TypeInteger;

    type TypeBoolean "Icon for Boolean types"
      extends Boolean;
      annotation(
        Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(lineColor = {160, 160, 164}, fillColor = {160, 160, 164}, fillPattern = FillPattern.Solid, extent = {{-100.0, -100.0}, {100.0, 100.0}}, radius = 25.0), Text(lineColor = {255, 255, 255}, extent = {{-90.0, -50.0}, {90.0, 50.0}}, textString = "B")}),
        Documentation(info = "<html>
<p>
This icon is designed for a <b>Boolean</b> type.
</p></html>"));
    end TypeBoolean;

    type TypeString "Icon for String types"
      extends String;
      annotation(
        Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(lineColor = {160, 160, 164}, fillColor = {160, 160, 164}, fillPattern = FillPattern.Solid, extent = {{-100.0, -100.0}, {100.0, 100.0}}, radius = 25.0), Text(lineColor = {255, 255, 255}, extent = {{-90.0, -50.0}, {90.0, 50.0}}, textString = "S")}),
        Documentation(info = "<html>
<p>
This icon is designed for a <b>String</b> type.
</p>
</html>"));
    end TypeString;

    expandable connector SignalBus "Icon for signal bus"
      annotation(
        Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}, initialScale = 0.2), graphics = {Rectangle(lineColor = {255, 204, 51}, lineThickness = 0.5, extent = {{-20.0, -2.0}, {20.0, 2.0}}), Polygon(fillColor = {255, 215, 136}, fillPattern = FillPattern.Solid, points = {{-80.0, 50.0}, {80.0, 50.0}, {100.0, 30.0}, {80.0, -40.0}, {60.0, -50.0}, {-60.0, -50.0}, {-80.0, -40.0}, {-100.0, 30.0}}, smooth = Smooth.Bezier), Ellipse(fillPattern = FillPattern.Solid, extent = {{-65.0, 15.0}, {-55.0, 25.0}}), Ellipse(fillPattern = FillPattern.Solid, extent = {{-5.0, 15.0}, {5.0, 25.0}}), Ellipse(fillPattern = FillPattern.Solid, extent = {{55.0, 15.0}, {65.0, 25.0}}), Ellipse(fillPattern = FillPattern.Solid, extent = {{-35.0, -25.0}, {-25.0, -15.0}}), Ellipse(fillPattern = FillPattern.Solid, extent = {{25.0, -25.0}, {35.0, -15.0}})}),
        Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}, initialScale = 0.2), graphics = {Polygon(points = {{-40, 25}, {40, 25}, {50, 15}, {40, -20}, {30, -25}, {-30, -25}, {-40, -20}, {-50, 15}}, lineColor = {0, 0, 0}, fillColor = {255, 204, 51}, fillPattern = FillPattern.Solid, smooth = Smooth.Bezier), Ellipse(extent = {{-32.5, 7.5}, {-27.5, 12.5}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-2.5, 12.5}, {2.5, 7.5}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Ellipse(extent = {{27.5, 12.5}, {32.5, 7.5}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-17.5, -7.5}, {-12.5, -12.5}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Ellipse(extent = {{12.5, -7.5}, {17.5, -12.5}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Text(extent = {{-150, 70}, {150, 40}}, lineColor = {0, 0, 0}, textString = "%name")}),
        Documentation(info = "<html>
This icon is designed for a <b>signal bus</b> connector.
</html>"));
    end SignalBus;

    expandable connector SignalSubBus "Icon for signal sub-bus"
      annotation(
        Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}, initialScale = 0.1), graphics = {Line(points = {{-16.0, 2.0}, {16.0, 2.0}}, color = {255, 204, 51}, thickness = 0.5), Rectangle(lineColor = {255, 204, 51}, lineThickness = 0.5, extent = {{-10.0, 0.0}, {8.0, 8.0}}), Polygon(fillColor = {255, 215, 136}, fillPattern = FillPattern.Solid, points = {{-80.0, 50.0}, {80.0, 50.0}, {100.0, 30.0}, {80.0, -40.0}, {60.0, -50.0}, {-60.0, -50.0}, {-80.0, -40.0}, {-100.0, 30.0}}, smooth = Smooth.Bezier), Ellipse(fillPattern = FillPattern.Solid, extent = {{-55.0, 15.0}, {-45.0, 25.0}}), Ellipse(fillPattern = FillPattern.Solid, extent = {{45.0, 15.0}, {55.0, 25.0}}), Ellipse(fillPattern = FillPattern.Solid, extent = {{-5.0, -25.0}, {5.0, -15.0}}), Rectangle(lineColor = {255, 215, 136}, lineThickness = 0.5, extent = {{-20.0, 0.0}, {20.0, 4.0}})}),
        Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}, initialScale = 0.1), graphics = {Polygon(points = {{-40, 25}, {40, 25}, {50, 15}, {40, -20}, {30, -25}, {-30, -25}, {-40, -20}, {-50, 15}}, lineColor = {0, 0, 0}, fillColor = {255, 204, 51}, fillPattern = FillPattern.Solid, smooth = Smooth.Bezier), Ellipse(extent = {{-22.5, 7.5}, {-17.5, 12.5}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Ellipse(extent = {{17.5, 12.5}, {22.5, 7.5}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-2.5, -7.5}, {2.5, -12.5}}, lineColor = {0, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid), Text(extent = {{-150, 70}, {150, 40}}, lineColor = {0, 0, 0}, textString = "%name")}),
        Documentation(info = "<html>
<p>
This icon is designed for a <b>sub-bus</b> in a signal connector.
</p>
</html>"));
    end SignalSubBus;

    partial class UnderConstruction "Icon for classes that are still under construction"
      annotation(
        Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{-100, -100}, {0, 80}, {100, -100}, {-100, -100}}, lineColor = {255, 0, 0}, lineThickness = 0.5)}),
        Documentation(info = "<html>
<p>Library developers can use this icon to indicate that the respective model is under construction.</p>
</html>"));
    end UnderConstruction;

    partial class ObsoleteModel "Icon for classes that are obsolete and will be removed in later versions"
      annotation(
        Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-102, 102}, {102, -102}}, lineColor = {255, 0, 0}, pattern = LinePattern.Dash, lineThickness = 0.5)}),
        Documentation(info = "<html>
<p>
This partial class is intended to provide a <u>default icon
for an obsolete model</u> that will be removed from the
corresponding library in a future release.
</p>
</html>"));
    end ObsoleteModel;

    partial package Library "This icon will be removed in future Modelica versions, use Package instead"
      // extends Modelica.Icons.ObsoleteModel;
      annotation(
        Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(lineColor = {200, 200, 200}, fillColor = {248, 248, 248}, fillPattern = FillPattern.HorizontalCylinder, extent = {{-100.0, -100.0}, {100.0, 100.0}}, radius = 25.0), Rectangle(lineColor = {128, 128, 128}, fillPattern = FillPattern.None, extent = {{-100.0, -100.0}, {100.0, 100.0}}, radius = 25.0)}),
        Documentation(info = "<html>
<p>This icon of a package will be removed in future versions of the library.</p>
<h5>Note</h5>
<p>This icon will be removed in future versions of the Modelica Standard Library. Instead the icon <a href=\"modelica://Modelica.Icons.Package\">Package</a> shall be used.</p>
</html>"));
    end Library;

    partial package Library2 "This icon will be removed in future Modelica versions, use Package instead"
      // extends Modelica.Icons.ObsoleteModel;
      annotation(
        Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(lineColor = {200, 200, 200}, fillColor = {248, 248, 248}, fillPattern = FillPattern.HorizontalCylinder, extent = {{-100.0, -100.0}, {100.0, 100.0}}, radius = 25.0), Rectangle(lineColor = {128, 128, 128}, fillPattern = FillPattern.None, extent = {{-100.0, -100.0}, {100.0, 100.0}}, radius = 25.0)}),
        Documentation(info = "<html>
<p>This icon of a package will be removed in future versions of the library.</p>
<h5>Note</h5>
<p>This icon will be removed in future versions of the Modelica Standard Library. Instead the icon <a href=\"modelica://Modelica.Icons.Package\">Package</a> shall be used.</p></html>"));
    end Library2;

    partial class GearIcon "This icon will be removed in future Modelica versions"
      // extends Modelica.Icons.ObsoleteModel;
      annotation(
        Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(lineColor = {64, 64, 64}, fillColor = {255, 255, 255}, fillPattern = FillPattern.HorizontalCylinder, extent = {{-90.0, -10.0}, {-60.0, 10.0}}), Polygon(fillColor = {192, 192, 192}, fillPattern = FillPattern.HorizontalCylinder, points = {{-60.0, 10.0}, {-60.0, 20.0}, {-40.0, 40.0}, {-40.0, -40.0}, {-60.0, -20.0}, {-60.0, 10.0}}), Rectangle(lineColor = {64, 64, 64}, fillColor = {255, 255, 255}, fillPattern = FillPattern.HorizontalCylinder, extent = {{-40.0, -60.0}, {40.0, 60.0}}, radius = 10.0), Polygon(fillColor = {192, 192, 192}, fillPattern = FillPattern.HorizontalCylinder, points = {{60.0, 20.0}, {40.0, 40.0}, {40.0, -40.0}, {60.0, -20.0}, {60.0, 20.0}}), Rectangle(lineColor = {64, 64, 64}, fillColor = {255, 255, 255}, fillPattern = FillPattern.HorizontalCylinder, extent = {{60.0, -10.0}, {90.0, 10.0}}), Polygon(fillColor = {64, 64, 64}, fillPattern = FillPattern.Solid, points = {{-60.0, -90.0}, {-50.0, -90.0}, {-20.0, -30.0}, {20.0, -30.0}, {48.0, -90.0}, {60.0, -90.0}, {60.0, -100.0}, {-60.0, -100.0}, {-60.0, -90.0}})}),
        Documentation(info = "<html>
<p>
This icon of a <b>gearbox</b> will be removed in future versions of the library. Please use one of the icons of <a href=\"modelica://Modelica.Mechanics.Rotational.Icons\">Mechanics.Rotational.Icons</a> instead.
</p>
</html>"));
    end GearIcon;

    partial class MotorIcon "This icon will be removed in future Modelica versions."
      // extends Modelica.Icons.ObsoleteModel;
      annotation(
        Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(lineColor = {82, 0, 2}, fillColor = {252, 37, 57}, fillPattern = FillPattern.HorizontalCylinder, extent = {{-100.0, -50.0}, {30.0, 50.0}}, radius = 10.0), Polygon(fillColor = {64, 64, 64}, fillPattern = FillPattern.Solid, points = {{-100.0, -90.0}, {-90.0, -90.0}, {-60.0, -20.0}, {-10.0, -20.0}, {20.0, -90.0}, {30.0, -90.0}, {30.0, -100.0}, {-100.0, -100.0}, {-100.0, -90.0}}), Rectangle(lineColor = {64, 64, 64}, fillColor = {255, 255, 255}, fillPattern = FillPattern.HorizontalCylinder, extent = {{30.0, -10.0}, {90.0, 10.0}})}),
        Documentation(info = "<html>
<p>
This icon of an <b>electrical motor</b> model will be removed in future versions of the library. Please use a locally defined icon in your user defined libraries and applications.
</p>
</html>"));
    end MotorIcon;

    partial class Info "This icon will be removed in future Modelica versions."
      // extends Modelica.Icons.ObsoleteModel;
      annotation(
        Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Ellipse(lineColor = {75, 138, 73}, fillColor = {75, 138, 73}, pattern = LinePattern.None, fillPattern = FillPattern.Solid, extent = {{-100.0, -100.0}, {100.0, 100.0}}), Polygon(origin = {-4.167, -15.0}, fillColor = {255, 255, 255}, pattern = LinePattern.None, fillPattern = FillPattern.Solid, points = {{-15.833, 20.0}, {-15.833, 30.0}, {14.167, 40.0}, {24.167, 20.0}, {4.167, -30.0}, {14.167, -30.0}, {24.167, -30.0}, {24.167, -40.0}, {-5.833, -50.0}, {-15.833, -30.0}, {4.167, 20.0}, {-5.833, 20.0}}, smooth = Smooth.Bezier), Ellipse(origin = {7.5, 56.5}, fillColor = {255, 255, 255}, pattern = LinePattern.None, fillPattern = FillPattern.Solid, extent = {{-12.5, -12.5}, {12.5, 12.5}})}),
        Documentation(info = "<html>
<p>This icon indicate classes containing only documentation, intended for general description of e.g., concepts and features of a package.</p>
<h5>Note</h5>
<p>This icon will be removed in future versions of the Modelica Standard Library. Instead the icon <a href=\"modelica://Modelica.Icons.Information\">Information</a> shall be used.</p></html>"));
    end Info;
    annotation(
      Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(origin = {-8.167, -17}, fillColor = {128, 128, 128}, pattern = LinePattern.None, fillPattern = FillPattern.Solid, points = {{-15.833, 20.0}, {-15.833, 30.0}, {14.167, 40.0}, {24.167, 20.0}, {4.167, -30.0}, {14.167, -30.0}, {24.167, -30.0}, {24.167, -40.0}, {-5.833, -50.0}, {-15.833, -30.0}, {4.167, 20.0}, {-5.833, 20.0}}, smooth = Smooth.Bezier, lineColor = {0, 0, 0}), Ellipse(origin = {-0.5, 56.5}, fillColor = {128, 128, 128}, pattern = LinePattern.None, fillPattern = FillPattern.Solid, extent = {{-12.5, -12.5}, {12.5, 12.5}}, lineColor = {0, 0, 0})}),
      Documentation(info = "<html>
<p>This package contains definitions for the graphical layout of components which may be used in different libraries. The icons can be utilized by inheriting them in the desired class using &quot;extends&quot; or by directly copying the &quot;icon&quot; layer. </p>

<h4>Main Authors:</h4>

<dl>
<dt><a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a></dt>
    <dd>Deutsches Zentrum fuer Luft und Raumfahrt e.V. (DLR)</dd>
    <dd>Oberpfaffenhofen</dd>
    <dd>Postfach 1116</dd>
    <dd>D-82230 Wessling</dd>
    <dd>email: <a href=\"mailto:Martin.Otter@dlr.de\">Martin.Otter@dlr.de</a></dd>
<dt>Christian Kral</dt>
    <dd><a href=\"http://christiankral.net/\">Electric Machines, Drives and Systems</a></dd>
    <dd>1060 Vienna, Austria</dd>
    <dd>email: <a href=\"mailto:dr.christian.kral@gmail.com\">dr.christian.kral@gmail.com</a></dd>
<dt>Johan Andreasson</dt>
    <dd><a href=\"http://www.modelon.se/\">Modelon AB</a></dd>
    <dd>Ideon Science Park</dd>
    <dd>22370 Lund, Sweden</dd>
    <dd>email: <a href=\"mailto:johan.andreasson@modelon.se\">johan.andreasson@modelon.se</a></dd>
</dl>

<p>Copyright &copy; 1998-2015, Modelica Association, DLR, AIT, and Modelon AB. </p>
<p><i>This Modelica package is <b>free</b> software; it can be redistributed and/or modified under the terms of the <b>Modelica license</b>, see the license conditions and the accompanying <b>disclaimer</b> in <a href=\"modelica://Modelica.UsersGuide.ModelicaLicense2\">Modelica.UsersGuide.ModelicaLicense2</a>.</i> </p>
</html>"));
  end Icons;

  package SIunits "Library of type and unit definitions based on SI units according to ISO 31-1992"
    extends Modelica.Icons.Package;

    package UsersGuide "User's Guide of SIunits Library"
      extends Modelica.Icons.Information;

      class HowToUseSIunits "How to use SIunits"
        extends Modelica.Icons.Information;
        annotation(
          DocumentationClass = true,
          Documentation(info = "<html>
<p>
When implementing a Modelica model, every variable needs to
be declared. Physical variables should be declared with a unit.
The basic approach in Modelica is that the unit attribute of
a variable is the <b>unit</b> in which the <b>equations</b> are <b>written</b>,
for example:
</p>

<pre>   <b>model</b> MassOnGround
     <b>parameter</b> Real m(quantity=\"Mass\", unit=\"kg\") \"Mass\";
     <b>parameter</b> Real f(quantity=\"Force\", unit=\"N\") \"Driving force\";
     Real s(unit=\"m\") \"Position of mass\";
     Real v(unit=\"m/s\") \"Velocity of mass\";
   <b>equation</b>
     <b>der</b>(s) = v;
     m*<b>der</b>(v) = f;
   <b>end</b> MassOnGround;
</pre>

<p>
This means that the equations in the equation section are only correct
for the specified units. A different issue is the user interface, i.e.,
in which unit the variable is presented to the user in graphical
user interfaces, both for input (e.g., parameter menu), as well as
for output (e.g., in the plot window). Preferably, the Modelica tool
should provide a list of units from which the user can select, e.g.,
\"m\", \"cm\", \"km\", \"inch\" for quantity \"Length\". When storing the value in
the model as a Modelica modifier, it has to be converted to the unit defined
in the declaration. Additionally, the unit used in the graphical
user interface has to be stored. In order to have a standardized way
of doing this, Modelica provides the following three attributes
for a variable of type Real:
</p>

<ul>
<li><b>quantity</b> to define the physical quantity (e.g., \"Length\", or \"Energy\").</li>
<li><b>unit</b> to define the unit that has to be used
    in order that the equations are correct (e.g., \"N.m\").</li>
<li><b>displayUnit</b> to define the unit used in the graphical
    user interface as default display unit for input and/or output.</li>
</ul>

<p>
Note, a unit, such as \"N.m\", is not sufficient to define uniquely the
physical quantity, since, e.g., \"N.m\" might be either \"torque\" or
\"energy\". The \"quantity\" attribute might therefore be used by a tool
to select the corresponding menu from which the user can select
a unit for the corresponding variable.
</p>

<p>
For example, after providing a value for \"m\" and \"f\" in a parameter
menu of an instance of MassOnGround, a tool might generate the following code:
</p>

<pre>
   MassOnGround myObject(m(displayUnit=\"g\")=2, f=3);
</pre>

<p>
The meaning is that in the equations a value of \"2\" is used
and that in the graphical user interface a value of \"2000\" should be used,
together with the unit \"g\" from the unit set \"Mass\" (= the quantity name).
Note, according to the Modelica specification
a tool might ignore the \"displayUnit\" attribute.
</p>

<p>
In order to help the Modelica model developer, the Modelica.SIunits
library provides about 450 predefined type names,
together with values for the attributes <b>quantity</b>, <b>unit</b> and sometimes
<b>displayUnit</b> and <b>min</b>. The unit is always selected as SI-unit according to the
ISO standard. The type and the quantity names are the
quantity names used in the ISO standard. \"quantity\" and \"unit\" are defined
as \"<b>final</b>\" in order that they cannot be modified. Attributes \"displayUnit\"
and \"min\" can, however, be changed in a model via a modification. The example above,
might therefore be alternatively also defined as:
</p>

<pre>   <b>model</b> MassOnGround
     <b>parameter</b> Modelica.SIunits.Mass  m \"Mass\";
     <b>parameter</b> Modelica.SIunits.Force f \"Driving force\";
     ...
   <b>end</b> MassOnGround;
</pre>

<p>
or in a short hand notation as
</p>

<pre>
   <b>model</b> MassOnGround
     <b>import</b> SI = Modelica.SIunits;
     <b>parameter</b> SI.Mass  m \"Mass\";
     <b>parameter</b> SI.Force f \"Driving force\";
     ...
   <b>end</b> MassOnGround;
</pre>

<p>
For some often
used Non SI-units (like hour), some additional type definitions are
present as Modelica.SIunits.Conversions.NonSIunits. If this is not sufficient,
the user has to define its own types or use the attributes directly
in the declaration as in the example at the beginning.
</p>

<p>
<b>Complex units</b> are also included in Modelica.SIunits. A complex unit is declared as:
</p>
<pre>
  <b>model</b> QuasiStationaryMachine
    <b>parameter</b> Modelica.SIunits.ComplexPower SNominal = Complex(10000,4400)
       \"Nominal complex power\";
   ...
   <b>end</b> QuasiStationaryMachine;
</pre>
</html>"));
      end HowToUseSIunits;

      class Conventions "Conventions"
        extends Modelica.Icons.Information;
        annotation(
          DocumentationClass = true,
          Documentation(info = "<html>
<p>The following conventions are used in package SIunits:</p>
<ul>
<li>Modelica quantity names are defined according to the recommendations
    of ISO 31. Some of these name are rather long, such as
    \"ThermodynamicTemperature\". Shorter alias names are defined, e.g.,
    \"type Temp_K = ThermodynamicTemperature;\".</li>
<li>Modelica units are defined according to the SI base units without
    multiples (only exception \"kg\").</li>
<li>For some quantities, more convenient units for an engineer are
    defined as \"displayUnit\", i.e., the default unit for display
    purposes (e.g., displayUnit=\"deg\" for quantity=\"Angle\").</li>
<li>The type name is identical to the quantity name, following
    the convention of type names.</li>
<li>All quantity and unit attributes are defined as final in order
    that they cannot be redefined to another value.</li>
<li>Similar quantities, such as \"Length, Breadth, Height, Thickness,
    Radius\" are defined as the same quantity (here: \"Length\").</li>
<li>The ordering of the type declarations in this package follows ISO 31:
<pre>
  Chapter  1: <b>Space and Time</b>
  Chapter  2: <b>Periodic and Related Phenomena</b>
  Chapter  3: <b>Mechanics</b>
  Chapter  4: <b>Heat</b>
  Chapter  5: <b>Electricity and Magnetism</b>
  Chapter  6: <b>Light and Related Electro-Magnetic Radiations</b>
  Chapter  7: <b>Acoustics</b>
  Chapter  8: <b>Physical Chemistry</b>
  Chapter  9: <b>Atomic and Nuclear Physics</b>
  Chapter 10: <b>Nuclear Reactions and Ionizing Radiations</b>
  Chapter 11: (not defined in ISO 31-1992)
  Chapter 12: <b>Characteristic Numbers</b>
  Chapter 13: <b>Solid State Physics</b>
</pre>
</li>
<li>Conversion functions between SI and non-SI units are available in subpackage
    <b>Conversions</b>.</li>
</ul>
</html>"));
      end Conventions;

      class Literature "Literature"
        extends Modelica.Icons.References;
        annotation(
          Documentation(info = "<html>
<p> This package is based on the following references
</p>

<dl>
<dt>ISO 31-1992:</dt>
<dd> <b>General principles concerning
    quantities, units and symbols</b>.<br>&nbsp;</dd>

<dt>ISO 1000-1992:</dt>
<dd> <b>SI units and recommendations for the use
    of their multiples and of certain other units</b>.<br>&nbsp;</dd>

<dt>Cardarelli F.:</dt>
<dd> <b>Scientific Unit Conversion - A Practical
     Guide to Metrication</b>. Springer 1997.</dd>
</dl>

</html>"));
      end Literature;

      class Contact "Contact"
        extends Modelica.Icons.Contact;
        annotation(
          Documentation(info = "<html>
<dl>
<dt><b>Main Author:</b>
<dd><a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a><br>
    Deutsches Zentrum f&uuml;r Luft und Raumfahrt e.V. (DLR)<br>
    Institut f&uuml;r Robotik und Mechatronik<br>
    Abteilung f&uuml;r Entwurfsorientierte Regelungstechnik<br>
    Postfach 1116<br>
    D-82230 Wessling<br>
    Germany<br>
    email: <A HREF=\"mailto:Martin.Otter@dlr.de\">Martin.Otter@dlr.de</A><br>
</dl>

<p><b>Acknowledgements:</b></p>
<ul>
<li> Astrid Jaschinski, Hubertus Tummescheit and Christian Schweiger
     contributed to the implementation of this package.</li>
</ul>
</html>"));
      end Contact;
      annotation(
        DocumentationClass = true,
        Documentation(info = "<HTML>
<p>
Library <b>SIunits</b> is a <b>free</b> Modelica package providing
predefined types, such as <i>Mass</i>,
<i>Length</i>, <i>Time</i>, based on the international standard
on units.</p>

</HTML>"));
    end UsersGuide;

    package Icons "Icons for SIunits"
      extends Modelica.Icons.IconsPackage;

      partial function Conversion "Base icon for conversion functions"
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, 100}, {100, -100}}, lineColor = {191, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{-90, 0}, {30, 0}}, color = {191, 0, 0}), Polygon(points = {{90, 0}, {30, 20}, {30, -20}, {90, 0}}, lineColor = {191, 0, 0}, fillColor = {191, 0, 0}, fillPattern = FillPattern.Solid), Text(extent = {{-115, 155}, {115, 105}}, textString = "%name", lineColor = {0, 0, 255})}));
      end Conversion;
    end Icons;

    package Conversions "Conversion functions to/from non SI units and type definitions of non SI units"
      extends Modelica.Icons.Package;

      package NonSIunits "Type definitions of non SI units"
        extends Modelica.Icons.Package;
        type Temperature_degC = Real(final quantity = "ThermodynamicTemperature", final unit = "degC") "Absolute temperature in degree Celsius (for relative temperature use SIunits.TemperatureDifference)" annotation(
          absoluteValue = true);
        type Temperature_degF = Real(final quantity = "ThermodynamicTemperature", final unit = "degF") "Absolute temperature in degree Fahrenheit (for relative temperature use SIunits.TemperatureDifference)" annotation(
          absoluteValue = true);
        type Temperature_degRk = Real(final quantity = "ThermodynamicTemperature", final unit = "degRk") "Absolute temperature in degree Rankine (for relative temperature use SIunits.TemperatureDifference)" annotation(
          absoluteValue = true);
        type Angle_deg = Real(final quantity = "Angle", final unit = "deg") "Angle in degree";
        type AngularVelocity_rpm = Real(final quantity = "AngularVelocity", final unit = "1/min") "Angular velocity in revolutions per minute. Alias unit names that are outside of the SI system: rpm, r/min, rev/min";
        type Velocity_kmh = Real(final quantity = "Velocity", final unit = "km/h") "Velocity in kilometers per hour";
        type Time_day = Real(final quantity = "Time", final unit = "d") "Time in days";
        type Time_hour = Real(final quantity = "Time", final unit = "h") "Time in hours";
        type Time_minute = Real(final quantity = "Time", final unit = "min") "Time in minutes";
        type Volume_litre = Real(final quantity = "Volume", final unit = "l") "Volume in litres";
        type ElectricCharge_Ah = Real(final quantity = "ElectricCharge", final unit = "A.h") "Electric charge in Ampere hours";
        type Energy_Wh = Real(final quantity = "Energy", final unit = "W.h") "Energy in Watt hours";
        type Energy_kWh = Real(final quantity = "Energy", final unit = "kW.h") "Energy in kilo watt hours";
        type Pressure_bar = Real(final quantity = "Pressure", final unit = "bar") "Absolute pressure in bar";
        type MassFlowRate_gps = Real(final quantity = "MassFlowRate", final unit = "g/s") "Mass flow rate in gram per second";
        type FirstOrderTemperaturCoefficient = Real(final quantity = "FirstOrderTemperatureCoefficient", final unit = "Ohm/degC") "First Order Temperature Coefficient" annotation(
          absoluteValue = true);
        type SecondOrderTemperaturCoefficient = Real(final quantity = "SecondOrderTemperatureCoefficient", final unit = "Ohm/degC2") "Second Order Temperature Coefficient" annotation(
          absoluteValue = true);
        type Area_cm = Real(final quantity = "Area", final unit = "cm2") "Area in cm";
        type PerArea_cm = Real(final quantity = "PerArea", final unit = "1/cm2") "Per Area in cm";
        type Area_cmPerVoltageSecond = Real(final quantity = "AreaPerVoltageSecond", final unit = "cm2/(V.s)") "Area in cm per voltage second";
        annotation(
          Documentation(info = "<HTML>
<p>
This package provides predefined types, such as <b>Angle_deg</b> (angle in
degree), <b>AngularVelocity_rpm</b> (angular velocity in revolutions per
minute) or <b>Temperature_degF</b> (temperature in degree Fahrenheit),
which are in common use but are not part of the international standard on
units according to ISO 31-1992 \"General principles concerning quantities,
units and symbols\" and ISO 1000-1992 \"SI units and recommendations for
the use of their multiples and of certain other units\".</p>
<p>If possible, the types in this package should not be used. Use instead
types of package Modelica.SIunits. For more information on units, see also
the book of Francois Cardarelli <b>Scientific Unit Conversion - A
Practical Guide to Metrication</b> (Springer 1997).</p>
<p>Some units, such as <b>Temperature_degC/Temp_C</b> are both defined in
Modelica.SIunits and in Modelica.Conversions.NonSIunits. The reason is that these
definitions have been placed erroneously in Modelica.SIunits although they
are not SIunits. For backward compatibility, these type definitions are
still kept in Modelica.SIunits.</p>
</html>"),
          Icon(coordinateSystem(extent = {{-100, -100}, {100, 100}}), graphics = {Text(origin = {15.0, 51.8518}, extent = {{-105.0, -86.8518}, {75.0, -16.8518}}, lineColor = {0, 0, 0}, textString = "[km/h]")}));
      end NonSIunits;

      function to_unit1 "Change the unit of a Real number to unit=\"1\""
        extends Modelica.SIunits.Icons.Conversion;
        input Real r "Real number";
        output Real result(unit = "1") "Real number r with unit=\"1\"";
      algorithm
        result := r;
        annotation(
          Inline = true,
          Documentation(info = "<HTML>
<h4>Syntax</h4>
<blockquote><pre>
SIunits.Conversions.<b>to_unit1</b>(r);
</pre></blockquote>
<h4>Description</h4>
<p>
The function call \"<code>Conversions.<b>to_unit1</b>(r)</code>\" returns r with unit=\"1\".

<h4>Example</h4>
<blockquote><pre>
  Modelica.SIunits.Velocity v = {3,2,1};
  Real direction[3](unit=\"1\") = to_unit1(v);   // Automatically vectorized call of to_unit1
</pre></blockquote>
</HTML>"),
          Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-90, 86}, {32, 50}}, lineColor = {0, 0, 0}, textString = "any", horizontalAlignment = TextAlignment.Left), Text(extent = {{-36, -52}, {86, -88}}, lineColor = {0, 0, 0}, horizontalAlignment = TextAlignment.Right, textString = "1")}));
      end to_unit1;

      function to_degC "Convert from Kelvin to degCelsius"
        extends Modelica.SIunits.Icons.Conversion;
        input Temperature Kelvin "Kelvin value";
        output NonSIunits.Temperature_degC Celsius "Celsius value";
      algorithm
        Celsius := Kelvin + Modelica.Constants.T_zero;
        annotation(
          Inline = true,
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-20, 100}, {-100, 20}}, lineColor = {0, 0, 0}, textString = "K"), Text(extent = {{100, -20}, {20, -100}}, lineColor = {0, 0, 0}, textString = "degC")}));
      end to_degC;

      function from_degC "Convert from degCelsius to Kelvin"
        extends Modelica.SIunits.Icons.Conversion;
        input NonSIunits.Temperature_degC Celsius "Celsius value";
        output Temperature Kelvin "Kelvin value";
      algorithm
        Kelvin := Celsius - Modelica.Constants.T_zero;
        annotation(
          Inline = true,
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-20, 100}, {-100, 20}}, lineColor = {0, 0, 0}, textString = "degC"), Text(extent = {{100, -20}, {20, -100}}, lineColor = {0, 0, 0}, textString = "K")}));
      end from_degC;

      function to_degF "Convert from Kelvin to degFahrenheit"
        extends Modelica.SIunits.Icons.Conversion;
        input Temperature Kelvin "Kelvin value";
        output NonSIunits.Temperature_degF Fahrenheit "Fahrenheit value";
      algorithm
        Fahrenheit := (Kelvin + Modelica.Constants.T_zero) * (9 / 5) + 32;
        annotation(
          Inline = true,
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-20, 100}, {-100, 20}}, lineColor = {0, 0, 0}, textString = "K"), Text(extent = {{100, -20}, {20, -100}}, lineColor = {0, 0, 0}, textString = "degF")}));
      end to_degF;

      function from_degF "Convert from degFahrenheit to Kelvin"
        extends Modelica.SIunits.Icons.Conversion;
        input NonSIunits.Temperature_degF Fahrenheit "Fahrenheit value";
        output Temperature Kelvin "Kelvin value";
      algorithm
        Kelvin := (Fahrenheit - 32) * (5 / 9) - Modelica.Constants.T_zero;
        annotation(
          Inline = true,
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-20, 100}, {-100, 20}}, lineColor = {0, 0, 0}, textString = "degF"), Text(extent = {{100, -20}, {20, -100}}, lineColor = {0, 0, 0}, textString = "K"), Text(extent = {{-20, 100}, {-100, 20}}, lineColor = {0, 0, 0}, textString = "degF")}));
      end from_degF;

      function to_degRk "Convert from Kelvin to degRankine"
        extends Modelica.SIunits.Icons.Conversion;
        input Temperature Kelvin "Kelvin value";
        output NonSIunits.Temperature_degRk Rankine "Rankine value";
      algorithm
        Rankine := 9 / 5 * Kelvin;
        annotation(
          Inline = true,
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-20, 100}, {-100, 20}}, lineColor = {0, 0, 0}, textString = "K"), Text(extent = {{100, -32}, {-18, -100}}, lineColor = {0, 0, 0}, textString = "degRk")}));
      end to_degRk;

      function from_degRk "Convert from degRankine to Kelvin"
        extends Modelica.SIunits.Icons.Conversion;
        input NonSIunits.Temperature_degRk Rankine "Rankine value";
        output Temperature Kelvin "Kelvin value";
      algorithm
        Kelvin := 5 / 9 * Rankine;
        annotation(
          Inline = true,
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-8, 100}, {-100, 42}}, lineColor = {0, 0, 0}, textString = "degRk"), Text(extent = {{100, -20}, {20, -100}}, lineColor = {0, 0, 0}, textString = "K")}));
      end from_degRk;

      function to_deg "Convert from radian to degree"
        extends Modelica.SIunits.Icons.Conversion;
        input Angle radian "radian value";
        output NonSIunits.Angle_deg degree "degree value";
      algorithm
        degree := 180.0 / Modelica.Constants.pi * radian;
        annotation(
          Inline = true,
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{10, 100}, {-100, 46}}, lineColor = {0, 0, 0}, textString = "rad"), Text(extent = {{100, -44}, {-10, -100}}, lineColor = {0, 0, 0}, textString = "deg")}));
      end to_deg;

      function from_deg "Convert from degree to radian"
        extends Modelica.SIunits.Icons.Conversion;
        input NonSIunits.Angle_deg degree "degree value";
        output Angle radian "radian value";
      algorithm
        radian := Modelica.Constants.pi / 180.0 * degree;
        annotation(
          Inline = true,
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{4, 100}, {-102, 46}}, lineColor = {0, 0, 0}, textString = "deg"), Text(extent = {{100, -32}, {-18, -100}}, lineColor = {0, 0, 0}, textString = "rad")}));
      end from_deg;

      function to_rpm "Convert from radian per second to revolutions per minute"
        extends Modelica.SIunits.Icons.Conversion;
        input AngularVelocity rs "radian per second value";
        output NonSIunits.AngularVelocity_rpm rpm "revolutions per minute value";
      algorithm
        rpm := 30 / Modelica.Constants.pi * rs;
        annotation(
          Inline = true,
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{30, 100}, {-100, 50}}, lineColor = {0, 0, 0}, textString = "rad/s"), Text(extent = {{100, -52}, {-40, -98}}, lineColor = {0, 0, 0}, textString = "1/min")}));
      end to_rpm;

      function from_rpm "Convert from revolutions per minute to radian per second"
        extends Modelica.SIunits.Icons.Conversion;
        input NonSIunits.AngularVelocity_rpm rpm "revolutions per minute value";
        output AngularVelocity rs "radian per second value";
      algorithm
        rs := Modelica.Constants.pi / 30 * rpm;
        annotation(
          Inline = true,
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{14, 100}, {-102, 56}}, lineColor = {0, 0, 0}, textString = "1/min"), Text(extent = {{100, -56}, {-32, -102}}, lineColor = {0, 0, 0}, textString = "rad/s")}));
      end from_rpm;

      function to_kmh "Convert from metre per second to kilometre per hour"
        extends Modelica.SIunits.Icons.Conversion;
        input Velocity ms "metre per second value";
        output NonSIunits.Velocity_kmh kmh "kilometre per hour value";
      algorithm
        kmh := 3.6 * ms;
        annotation(
          Inline = true,
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{8, 100}, {-100, 58}}, lineColor = {0, 0, 0}, textString = "m/s"), Text(extent = {{100, -56}, {-16, -100}}, lineColor = {0, 0, 0}, textString = "km/h")}));
      end to_kmh;

      function from_kmh "Convert from kilometre per hour to metre per second"
        extends Modelica.SIunits.Icons.Conversion;
        input NonSIunits.Velocity_kmh kmh "kilometre per hour value";
        output Velocity ms "metre per second value";
      algorithm
        ms := kmh / 3.6;
        annotation(
          Inline = true,
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{10, 100}, {-100, 56}}, lineColor = {0, 0, 0}, textString = "km/h"), Text(extent = {{100, -50}, {-20, -100}}, lineColor = {0, 0, 0}, textString = "m/s")}));
      end from_kmh;

      function to_day "Convert from second to day"
        extends Modelica.SIunits.Icons.Conversion;
        input Time s "second value";
        output NonSIunits.Time_day day "day value";
      algorithm
        day := s / 86400;
        annotation(
          Inline = true,
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-6, 100}, {-100, 48}}, lineColor = {0, 0, 0}, textString = "s"), Text(extent = {{100, -48}, {-10, -98}}, lineColor = {0, 0, 0}, textString = "day")}));
      end to_day;

      function from_day "Convert from day to second"
        extends Modelica.SIunits.Icons.Conversion;
        input NonSIunits.Time_day day "day value";
        output Time s "second value";
      algorithm
        s := 86400 * day;
        annotation(
          Inline = true,
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{10, 100}, {-100, 52}}, lineColor = {0, 0, 0}, textString = "day"), Text(extent = {{100, -54}, {20, -100}}, lineColor = {0, 0, 0}, textString = "s")}));
      end from_day;

      function to_hour "Convert from second to hour"
        extends Modelica.SIunits.Icons.Conversion;
        input Time s "second value";
        output NonSIunits.Time_hour hour "hour value";
      algorithm
        hour := s / 3600;
        annotation(
          Inline = true,
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{12, 100}, {-100, 50}}, lineColor = {0, 0, 0}, textString = "s"), Text(extent = {{100, -56}, {-20, -100}}, lineColor = {0, 0, 0}, textString = "hour")}));
      end to_hour;

      function from_hour "Convert from hour to second"
        extends Modelica.SIunits.Icons.Conversion;
        input NonSIunits.Time_hour hour "hour value";
        output Time s "second value";
      algorithm
        s := 3600 * hour;
        annotation(
          Inline = true,
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{12, 100}, {-100, 58}}, lineColor = {0, 0, 0}, textString = "hour"), Text(extent = {{100, -50}, {16, -100}}, lineColor = {0, 0, 0}, textString = "s")}));
      end from_hour;

      function to_minute "Convert from second to minute"
        extends Modelica.SIunits.Icons.Conversion;
        input Time s "second value";
        output NonSIunits.Time_minute minute "minute value";
      algorithm
        minute := s / 60;
        annotation(
          Inline = true,
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-26, 100}, {-100, 52}}, lineColor = {0, 0, 0}, textString = "s"), Text(extent = {{100, -54}, {-20, -100}}, lineColor = {0, 0, 0}, textString = "min")}));
      end to_minute;

      function from_minute "Convert from minute to second"
        extends Modelica.SIunits.Icons.Conversion;
        input NonSIunits.Time_minute minute "minute value";
        output Time s "second value";
      algorithm
        s := 60 * minute;
        annotation(
          Inline = true,
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{26, 100}, {-100, 48}}, lineColor = {0, 0, 0}, textString = "min"), Text(extent = {{100, -46}, {0, -100}}, lineColor = {0, 0, 0}, textString = "s")}));
      end from_minute;

      function to_litre "Convert from cubic metre to litre"
        extends Modelica.SIunits.Icons.Conversion;
        input Volume m3 "cubic metre value";
        output NonSIunits.Volume_litre litre "litre value";
      algorithm
        litre := 1000 * m3;
        annotation(
          Inline = true,
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{100, -56}, {0, -100}}, lineColor = {0, 0, 0}, textString = "litre"), Text(extent = {{6, 100}, {-100, 56}}, lineColor = {0, 0, 0}, textString = "m3")}));
      end to_litre;

      function from_litre "Convert from litre to cubic metre"
        extends Modelica.SIunits.Icons.Conversion;
        input NonSIunits.Volume_litre litre "litre value";
        output Volume m3 "cubic metre value";
      algorithm
        m3 := litre / 1000;
        annotation(
          Inline = true,
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-4, 100}, {-100, 62}}, lineColor = {0, 0, 0}, textString = "litre"), Text(extent = {{100, -56}, {-6, -100}}, lineColor = {0, 0, 0}, textString = "m3")}));
      end from_litre;

      function from_Ah "Convert from Ampere hours to Coulomb"
        extends Modelica.SIunits.Icons.Conversion;
        input Modelica.SIunits.Conversions.NonSIunits.ElectricCharge_Ah AmpereHour "Ampere hours";
        output Modelica.SIunits.ElectricCharge Coulomb "Coulomb";
      algorithm
        Coulomb := AmpereHour * 3600;
        annotation(
          Icon(graphics = {Text(extent = {{-2, 100}, {-100, 48}}, lineColor = {0, 0, 0}, textString = "Ah"), Text(extent = {{100, -46}, {0, -100}}, lineColor = {0, 0, 0}, textString = "C")}));
      end from_Ah;

      function to_Ah "Convert from Coulomb to Ampere hours"
        extends Modelica.SIunits.Icons.Conversion;
        input Modelica.SIunits.ElectricCharge Coulomb "Coulomb";
        output Modelica.SIunits.Conversions.NonSIunits.ElectricCharge_Ah AmpereHour "Ampere hours";
      algorithm
        AmpereHour := Coulomb / 3600;
        annotation(
          Icon(graphics = {Text(extent = {{-18, 100}, {-100, 48}}, lineColor = {0, 0, 0}, textString = "C"), Text(extent = {{100, -48}, {2, -100}}, lineColor = {0, 0, 0}, textString = "Ah")}));
      end to_Ah;

      function from_Wh "Convert from Watt hour to Joule"
        extends Modelica.SIunits.Icons.Conversion;
        input NonSIunits.Energy_Wh WattHour "Watt hour";
        output Modelica.SIunits.Energy Joule "Joule";
      algorithm
        Joule := WattHour * 3600;
        annotation(
          Icon(graphics = {Text(extent = {{-20, 100}, {-100, 54}}, lineColor = {0, 0, 0}, textString = "Wh"), Text(extent = {{100, -38}, {4, -100}}, lineColor = {0, 0, 0}, textString = "J")}));
      end from_Wh;

      function to_Wh "Convert from Joule to Watt hour"
        extends Modelica.SIunits.Icons.Conversion;
        input Modelica.SIunits.Energy Joule "Joule";
        output NonSIunits.Energy_Wh WattHour "Watt hour";
      algorithm
        WattHour := Joule / 3600;
        annotation(
          Icon(graphics = {Text(extent = {{-30, 100}, {-100, 48}}, lineColor = {0, 0, 0}, textString = "J"), Text(extent = {{100, -46}, {-14, -100}}, lineColor = {0, 0, 0}, textString = "Wh")}));
      end to_Wh;

      function to_kWh "Convert from Joule to kilo Watt hour"
        extends Modelica.SIunits.Icons.Conversion;
        input Energy J "Joule value";
        output NonSIunits.Energy_kWh kWh "kWh value";
      algorithm
        kWh := J / 3.6e6;
        annotation(
          Inline = true,
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-20, 100}, {-100, 54}}, lineColor = {0, 0, 0}, textString = "J"), Text(extent = {{100, -50}, {-10, -100}}, lineColor = {0, 0, 0}, textString = "kWh")}));
      end to_kWh;

      function from_kWh "Convert from kilo Watt hour to Joule"
        extends Modelica.SIunits.Icons.Conversion;
        input NonSIunits.Energy_kWh kWh "kWh value";
        output Energy J "Joule value";
      algorithm
        J := 3.6e6 * kWh;
        annotation(
          Inline = true,
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{12, 100}, {-100, 52}}, lineColor = {0, 0, 0}, textString = "kWh"), Text(extent = {{100, -44}, {12, -100}}, lineColor = {0, 0, 0}, textString = "J")}));
      end from_kWh;

      function to_bar "Convert from Pascal to bar"
        extends Modelica.SIunits.Icons.Conversion;
        input Pressure Pa "Pascal value";
        output NonSIunits.Pressure_bar bar "bar value";
      algorithm
        bar := Pa / 1e5;
        annotation(
          Inline = true,
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-12, 100}, {-100, 56}}, lineColor = {0, 0, 0}, textString = "Pa"), Text(extent = {{98, -52}, {-4, -100}}, lineColor = {0, 0, 0}, textString = "bar")}));
      end to_bar;

      function from_bar "Convert from bar to Pascal"
        extends Modelica.SIunits.Icons.Conversion;
        input NonSIunits.Pressure_bar bar "bar value";
        output Pressure Pa "Pascal value";
      algorithm
        Pa := 1e5 * bar;
        annotation(
          Inline = true,
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{100, -56}, {12, -100}}, lineColor = {0, 0, 0}, textString = "Pa"), Text(extent = {{2, 100}, {-100, 52}}, lineColor = {0, 0, 0}, textString = "bar")}));
      end from_bar;

      function to_gps "Convert from kilogram per second to gram per second"
        extends Modelica.SIunits.Icons.Conversion;
        input MassFlowRate kgps "kg/s value";
        output NonSIunits.MassFlowRate_gps gps "g/s value";
      algorithm
        gps := 1000 * kgps;
        annotation(
          Inline = true,
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-12, 100}, {-100, 60}}, lineColor = {0, 0, 0}, textString = "kg/s"), Text(extent = {{100, -46}, {-6, -100}}, lineColor = {0, 0, 0}, textString = "g/s")}));
      end to_gps;

      function from_gps "Convert from gram per second to kilogram per second"
        extends Modelica.SIunits.Icons.Conversion;
        input NonSIunits.MassFlowRate_gps gps "g/s value";
        output MassFlowRate kgps "kg/s value";
      algorithm
        kgps := gps / 1000;
        annotation(
          Inline = true,
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-8, 100}, {-100, 54}}, lineColor = {0, 0, 0}, textString = "g/s"), Text(extent = {{100, -44}, {-10, -100}}, lineColor = {0, 0, 0}, textString = "kg/s")}));
      end from_gps;

      function from_Hz "Convert from Hz to rad/s"
        extends Modelica.SIunits.Icons.Conversion;
        input SIunits.Frequency f "frequency";
        output SIunits.AngularVelocity w "angular velocity";
      algorithm
        w := 2 * Modelica.Constants.pi * f;
        annotation(
          Inline = true,
          Icon(graphics = {Text(extent = {{2, 100}, {-100, 52}}, lineColor = {0, 0, 0}, textString = "Hz"), Text(extent = {{100, -56}, {12, -100}}, lineColor = {0, 0, 0}, textString = "1/s")}));
      end from_Hz;

      function to_Hz "Convert from rad/s to Hz"
        extends Modelica.SIunits.Icons.Conversion;
        input SIunits.AngularVelocity w "angular velocity";
        output SIunits.Frequency f "frequency";
      algorithm
        f := w / (2 * Modelica.Constants.pi);
        annotation(
          Inline = true,
          Icon(graphics = {Text(extent = {{100, -52}, {-2, -100}}, lineColor = {0, 0, 0}, textString = "Hz"), Text(extent = {{-12, 100}, {-100, 56}}, lineColor = {0, 0, 0}, textString = "1/s")}));
      end to_Hz;

      function to_cm2 "Convert from square metre to square centimetre"
        extends Modelica.SIunits.Icons.Conversion;
        input Area m2 "square metre value";
        output NonSIunits.Area_cm cm2 "square centimetre value";
      algorithm
        cm2 := 10000 * m2;
        annotation(
          Inline = true,
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{-20, 100}, {-100, 58}}, lineColor = {0, 0, 0}, textString = "m/s"), Text(extent = {{100, -50}, {-18, -100}}, lineColor = {0, 0, 0}, textString = "cm2")}));
      end to_cm2;

      function from_cm2 "Convert from square centimetre to square metre"
        extends Modelica.SIunits.Icons.Conversion;
        input NonSIunits.Area_cm cm2 "square centimetre value";
        output Area m2 "square metre value";
      algorithm
        m2 := 0.0001 * cm2;
        annotation(
          Inline = true,
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Text(extent = {{2, 100}, {-100, 58}}, lineColor = {0, 0, 0}, textString = "cm2"), Text(extent = {{100, -50}, {-16, -98}}, lineColor = {0, 0, 0}, textString = "m/s")}));
      end from_cm2;

      partial function ConversionIcon "This icon will be removed in future Modelica versions."
        extends Modelica.Icons.ObsoleteModel;
        annotation(
          Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, 100}, {100, -100}}, lineColor = {191, 0, 0}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{-90, 0}, {30, 0}}, color = {191, 0, 0}), Polygon(points = {{90, 0}, {30, 20}, {30, -20}, {90, 0}}, lineColor = {191, 0, 0}, fillColor = {191, 0, 0}, fillPattern = FillPattern.Solid), Text(extent = {{-115, 155}, {115, 105}}, textString = "%name", lineColor = {0, 0, 255})}),
          Documentation(info = "<html>
<p>
This icon of a <b>conversion symbol</b> will be removed in future versions of the library. Instead the icon <a href=\"modelica://Modelica.SIunits.Icons.Conversion\">Modelica.SIunits.Icons.Conversion</a> shall be used.
</p>
</html>"));
      end ConversionIcon;
      annotation(
        Documentation(info = "<HTML>
<p>This package provides conversion functions from the non SI Units
defined in package Modelica.SIunits.Conversions.NonSIunits to the
corresponding SI Units defined in package Modelica.SIunits and vice
versa. It is recommended to use these functions in the following
way (note, that all functions have one Real input and one Real output
argument):</p>
<pre>
  <b>import</b> SI = Modelica.SIunits;
  <b>import</b> Modelica.SIunits.Conversions.*;
     ...
  <b>parameter</b> SI.Temperature     T   = from_degC(25);   // convert 25 degree Celsius to Kelvin
  <b>parameter</b> SI.Angle           phi = from_deg(180);   // convert 180 degree to radian
  <b>parameter</b> SI.AngularVelocity w   = from_rpm(3600);  // convert 3600 revolutions per minutes
                                                      // to radian per seconds
</pre>

</html>"));
    end Conversions;

    // Space and Time (chapter 1 of ISO 31-1992)
    type Angle = Real(final quantity = "Angle", final unit = "rad", displayUnit = "deg");
    type SolidAngle = Real(final quantity = "SolidAngle", final unit = "sr");
    type Length = Real(final quantity = "Length", final unit = "m");
    type PathLength = Length;
    type Position = Length;
    type Distance = Length(min = 0);
    type Breadth = Length(min = 0);
    type Height = Length(min = 0);
    type Thickness = Length(min = 0);
    type Radius = Length(min = 0);
    type Diameter = Length(min = 0);
    type Area = Real(final quantity = "Area", final unit = "m2");
    type Volume = Real(final quantity = "Volume", final unit = "m3");
    type Time = Real(final quantity = "Time", final unit = "s");
    type Duration = Time;
    type AngularVelocity = Real(final quantity = "AngularVelocity", final unit = "rad/s");
    type AngularAcceleration = Real(final quantity = "AngularAcceleration", final unit = "rad/s2");
    type Velocity = Real(final quantity = "Velocity", final unit = "m/s");
    type Acceleration = Real(final quantity = "Acceleration", final unit = "m/s2");
    // Periodic and related phenomens (chapter 2 of ISO 31-1992)
    type Period = Real(final quantity = "Time", final unit = "s");
    type Frequency = Real(final quantity = "Frequency", final unit = "Hz");
    type AngularFrequency = Real(final quantity = "AngularFrequency", final unit = "rad/s");
    type Wavelength = Real(final quantity = "Wavelength", final unit = "m");
    type Wavelenght = Wavelength;
    // For compatibility reasons only
    type WaveNumber = Real(final quantity = "WaveNumber", final unit = "m-1");
    type CircularWaveNumber = Real(final quantity = "CircularWaveNumber", final unit = "rad/m");
    type AmplitudeLevelDifference = Real(final quantity = "AmplitudeLevelDifference", final unit = "dB");
    type PowerLevelDifference = Real(final quantity = "PowerLevelDifference", final unit = "dB");
    type DampingCoefficient = Real(final quantity = "DampingCoefficient", final unit = "s-1");
    type LogarithmicDecrement = Real(final quantity = "LogarithmicDecrement", final unit = "1/S");
    type AttenuationCoefficient = Real(final quantity = "AttenuationCoefficient", final unit = "m-1");
    type PhaseCoefficient = Real(final quantity = "PhaseCoefficient", final unit = "m-1");
    type PropagationCoefficient = Real(final quantity = "PropagationCoefficient", final unit = "m-1");
    // added to ISO-chapter
    type Damping = DampingCoefficient;
    // Mechanics (chapter 3 of ISO 31-1992)
    type Mass = Real(quantity = "Mass", final unit = "kg", min = 0);
    type Density = Real(final quantity = "Density", final unit = "kg/m3", displayUnit = "g/cm3", min = 0.0);
    type RelativeDensity = Real(final quantity = "RelativeDensity", final unit = "1", min = 0.0);
    type SpecificVolume = Real(final quantity = "SpecificVolume", final unit = "m3/kg", min = 0.0);
    type LinearDensity = Real(final quantity = "LinearDensity", final unit = "kg/m", min = 0);
    type SurfaceDensity = Real(final quantity = "SurfaceDensity", final unit = "kg/m2", min = 0);
    type Momentum = Real(final quantity = "Momentum", final unit = "kg.m/s");
    type Impulse = Real(final quantity = "Impulse", final unit = "N.s");
    type AngularMomentum = Real(final quantity = "AngularMomentum", final unit = "kg.m2/s");
    type AngularImpulse = Real(final quantity = "AngularImpulse", final unit = "N.m.s");
    type MomentOfInertia = Real(final quantity = "MomentOfInertia", final unit = "kg.m2");
    type Inertia = MomentOfInertia;
    type Force = Real(final quantity = "Force", final unit = "N");
    type TranslationalSpringConstant = Real(final quantity = "TranslationalSpringConstant", final unit = "N/m");
    type TranslationalDampingConstant = Real(final quantity = "TranslationalDampingConstant", final unit = "N.s/m");
    type Weight = Force;
    type Torque = Real(final quantity = "Torque", final unit = "N.m");
    type ElectricalTorqueConstant = Real(final quantity = "ElectricalTorqueConstant", final unit = "N.m/A");
    type MomentOfForce = Torque;
    type ImpulseFlowRate = Real(final quantity = "ImpulseFlowRate", final unit = "N");
    type AngularImpulseFlowRate = Real(final quantity = "AngularImpulseFlowRate", final unit = "N.m");
    type RotationalSpringConstant = Real(final quantity = "RotationalSpringConstant", final unit = "N.m/rad");
    type RotationalDampingConstant = Real(final quantity = "RotationalDampingConstant", final unit = "N.m.s/rad");
    type Pressure = Real(final quantity = "Pressure", final unit = "Pa", displayUnit = "bar");
    type AbsolutePressure = Pressure(min = 0.0, nominal = 1e5);
    type PressureDifference = Pressure;
    type BulkModulus = AbsolutePressure;
    type Stress = Real(final unit = "Pa");
    type NormalStress = Stress;
    type ShearStress = Stress;
    type Strain = Real(final quantity = "Strain", final unit = "1");
    type LinearStrain = Strain;
    type ShearStrain = Strain;
    type VolumeStrain = Real(final quantity = "VolumeStrain", final unit = "1");
    type PoissonNumber = Real(final quantity = "PoissonNumber", final unit = "1");
    type ModulusOfElasticity = Stress;
    type ShearModulus = Stress;
    type SecondMomentOfArea = Real(final quantity = "SecondMomentOfArea", final unit = "m4");
    type SecondPolarMomentOfArea = SecondMomentOfArea;
    type SectionModulus = Real(final quantity = "SectionModulus", final unit = "m3");
    type CoefficientOfFriction = Real(final quantity = "CoefficientOfFriction", final unit = "1");
    type DynamicViscosity = Real(final quantity = "DynamicViscosity", final unit = "Pa.s", min = 0);
    type KinematicViscosity = Real(final quantity = "KinematicViscosity", final unit = "m2/s", min = 0);
    type SurfaceTension = Real(final quantity = "SurfaceTension", final unit = "N/m");
    type Work = Real(final quantity = "Work", final unit = "J");
    type Energy = Real(final quantity = "Energy", final unit = "J");
    type EnergyDensity = Real(final quantity = "EnergyDensity", final unit = "J/m3");
    type PotentialEnergy = Energy;
    type KineticEnergy = Energy;
    type Power = Real(final quantity = "Power", final unit = "W");
    type EnergyFlowRate = Power;
    type EnthalpyFlowRate = Real(final quantity = "EnthalpyFlowRate", final unit = "W");
    type Efficiency = Real(final quantity = "Efficiency", final unit = "1", min = 0);
    type MassFlowRate = Real(quantity = "MassFlowRate", final unit = "kg/s");
    type VolumeFlowRate = Real(final quantity = "VolumeFlowRate", final unit = "m3/s");
    // added to ISO-chapter 3
    type MomentumFlux = Real(final quantity = "MomentumFlux", final unit = "N");
    type AngularMomentumFlux = Real(final quantity = "AngularMomentumFlux", final unit = "N.m");
    // Heat (chapter 4 of ISO 31-1992)
    type ThermodynamicTemperature = Real(final quantity = "ThermodynamicTemperature", final unit = "K", min = 0.0, start = 288.15, nominal = 300, displayUnit = "degC") "Absolute temperature (use type TemperatureDifference for relative temperatures)" annotation(
      absoluteValue = true);
    type Temp_K = ThermodynamicTemperature;
    type Temperature = ThermodynamicTemperature;
    type TemperatureDifference = Real(final quantity = "ThermodynamicTemperature", final unit = "K") annotation(
      absoluteValue = false);
    type Temp_C = SIunits.Conversions.NonSIunits.Temperature_degC;
    type TemperatureSlope = Real(final quantity = "TemperatureSlope", final unit = "K/s");
    type LinearTemperatureCoefficient = Real(final quantity = "LinearTemperatureCoefficient", final unit = "1/K");
    type QuadraticTemperatureCoefficient = Real(final quantity = "QuadraticTemperatureCoefficient", final unit = "1/K2");
    type LinearExpansionCoefficient = Real(final quantity = "LinearExpansionCoefficient", final unit = "1/K");
    type CubicExpansionCoefficient = Real(final quantity = "CubicExpansionCoefficient", final unit = "1/K");
    type RelativePressureCoefficient = Real(final quantity = "RelativePressureCoefficient", final unit = "1/K");
    type PressureCoefficient = Real(final quantity = "PressureCoefficient", final unit = "Pa/K");
    type Compressibility = Real(final quantity = "Compressibility", final unit = "1/Pa");
    type IsothermalCompressibility = Compressibility;
    type IsentropicCompressibility = Compressibility;
    type Heat = Real(final quantity = "Energy", final unit = "J");
    type HeatFlowRate = Real(final quantity = "Power", final unit = "W");
    type HeatFlux = Real(final quantity = "HeatFlux", final unit = "W/m2");
    type DensityOfHeatFlowRate = Real(final quantity = "DensityOfHeatFlowRate", final unit = "W/m2");
    type ThermalConductivity = Real(final quantity = "ThermalConductivity", final unit = "W/(m.K)");
    type CoefficientOfHeatTransfer = Real(final quantity = "CoefficientOfHeatTransfer", final unit = "W/(m2.K)");
    type SurfaceCoefficientOfHeatTransfer = CoefficientOfHeatTransfer;
    type ThermalInsulance = Real(final quantity = "ThermalInsulance", final unit = "m2.K/W");
    type ThermalResistance = Real(final quantity = "ThermalResistance", final unit = "K/W");
    type ThermalConductance = Real(final quantity = "ThermalConductance", final unit = "W/K");
    type ThermalDiffusivity = Real(final quantity = "ThermalDiffusivity", final unit = "m2/s");
    type HeatCapacity = Real(final quantity = "HeatCapacity", final unit = "J/K");
    type SpecificHeatCapacity = Real(final quantity = "SpecificHeatCapacity", final unit = "J/(kg.K)");
    type SpecificHeatCapacityAtConstantPressure = SpecificHeatCapacity;
    type SpecificHeatCapacityAtConstantVolume = SpecificHeatCapacity;
    type SpecificHeatCapacityAtSaturation = SpecificHeatCapacity;
    type RatioOfSpecificHeatCapacities = Real(final quantity = "RatioOfSpecificHeatCapacities", final unit = "1");
    type IsentropicExponent = Real(final quantity = "IsentropicExponent", final unit = "1");
    type Entropy = Real(final quantity = "Entropy", final unit = "J/K");
    type EntropyFlowRate = Real(final quantity = "EntropyFlowRate", final unit = "J/(K.s)");
    type SpecificEntropy = Real(final quantity = "SpecificEntropy", final unit = "J/(kg.K)");
    type InternalEnergy = Heat;
    type Enthalpy = Heat;
    type HelmholtzFreeEnergy = Heat;
    type GibbsFreeEnergy = Heat;
    type SpecificEnergy = Real(final quantity = "SpecificEnergy", final unit = "J/kg");
    type SpecificInternalEnergy = SpecificEnergy;
    type SpecificEnthalpy = SpecificEnergy;
    type SpecificHelmholtzFreeEnergy = SpecificEnergy;
    type SpecificGibbsFreeEnergy = SpecificEnergy;
    type MassieuFunction = Real(final quantity = "MassieuFunction", final unit = "J/K");
    type PlanckFunction = Real(final quantity = "PlanckFunction", final unit = "J/K");
    // added to ISO-chapter 4
    type DerDensityByEnthalpy = Real(final unit = "kg.s2/m5");
    type DerDensityByPressure = Real(final unit = "s2/m2");
    type DerDensityByTemperature = Real(final unit = "kg/(m3.K)");
    type DerEnthalpyByPressure = Real(final unit = "J.m.s2/kg2");
    type DerEnergyByDensity = Real(final unit = "J.m3/kg");
    type DerEnergyByPressure = Real(final unit = "J.m.s2/kg");
    type DerPressureByDensity = Real(final unit = "Pa.m3/kg");
    type DerPressureByTemperature = Real(final unit = "Pa/K");
    // Electricity and Magnetism (chapter 5 of ISO 31-1992)
    type ElectricCurrent = Real(final quantity = "ElectricCurrent", final unit = "A");
    type Current = ElectricCurrent;
    type CurrentSlope = Real(final quantity = "CurrentSlope", final unit = "A/s");
    type ElectricCharge = Real(final quantity = "ElectricCharge", final unit = "C");
    type Charge = ElectricCharge;
    type VolumeDensityOfCharge = Real(final quantity = "VolumeDensityOfCharge", final unit = "C/m3", min = 0);
    type SurfaceDensityOfCharge = Real(final quantity = "SurfaceDensityOfCharge", final unit = "C/m2", min = 0);
    type ElectricFieldStrength = Real(final quantity = "ElectricFieldStrength", final unit = "V/m");
    type ElectricPotential = Real(final quantity = "ElectricPotential", final unit = "V");
    type Voltage = ElectricPotential;
    type PotentialDifference = ElectricPotential;
    type ElectromotiveForce = ElectricPotential;
    type VoltageSecond = Real(final quantity = "VoltageSecond", final unit = "V.s") "Voltage second";
    type VoltageSlope = Real(final quantity = "VoltageSlope", final unit = "V/s");
    type ElectricFluxDensity = Real(final quantity = "ElectricFluxDensity", final unit = "C/m2");
    type ElectricFlux = Real(final quantity = "ElectricFlux", final unit = "C");
    type Capacitance = Real(final quantity = "Capacitance", final unit = "F", min = 0);
    type CapacitancePerArea = Real(final quantity = "CapacitancePerArea", final unit = "F/m2") "Capacitance per area";
    type Permittivity = Real(final quantity = "Permittivity", final unit = "F/m", min = 0);
    type PermittivityOfVacuum = Permittivity;
    type RelativePermittivity = Real(final quantity = "RelativePermittivity", final unit = "1");
    type ElectricSusceptibility = Real(final quantity = "ElectricSusceptibility", final unit = "1");
    type ElectricPolarization = Real(final quantity = "ElectricPolarization", final unit = "C/m2");
    type Electrization = Real(final quantity = "Electrization", final unit = "V/m");
    type ElectricDipoleMoment = Real(final quantity = "ElectricDipoleMoment", final unit = "C.m");
    type CurrentDensity = Real(final quantity = "CurrentDensity", final unit = "A/m2");
    type LinearCurrentDensity = Real(final quantity = "LinearCurrentDensity", final unit = "A/m");
    type MagneticFieldStrength = Real(final quantity = "MagneticFieldStrength", final unit = "A/m");
    type MagneticPotential = Real(final quantity = "MagneticPotential", final unit = "A");
    type MagneticPotentialDifference = Real(final quantity = "MagneticPotential", final unit = "A");
    type MagnetomotiveForce = Real(final quantity = "MagnetomotiveForce", final unit = "A");
    type CurrentLinkage = Real(final quantity = "CurrentLinkage", final unit = "A");
    type MagneticFluxDensity = Real(final quantity = "MagneticFluxDensity", final unit = "T");
    type MagneticFlux = Real(final quantity = "MagneticFlux", final unit = "Wb");
    type MagneticVectorPotential = Real(final quantity = "MagneticVectorPotential", final unit = "Wb/m");
    type Inductance = Real(final quantity = "Inductance", final unit = "H");
    type SelfInductance = Inductance(min = 0);
    type MutualInductance = Inductance;
    type CouplingCoefficient = Real(final quantity = "CouplingCoefficient", final unit = "1");
    type LeakageCoefficient = Real(final quantity = "LeakageCoefficient", final unit = "1");
    type Permeability = Real(final quantity = "Permeability", final unit = "H/m");
    type PermeabilityOfVacuum = Permeability;
    type RelativePermeability = Real(final quantity = "RelativePermeability", final unit = "1");
    type MagneticSusceptibility = Real(final quantity = "MagneticSusceptibility", final unit = "1");
    type ElectromagneticMoment = Real(final quantity = "ElectromagneticMoment", final unit = "A.m2");
    type MagneticDipoleMoment = Real(final quantity = "MagneticDipoleMoment", final unit = "Wb.m");
    type Magnetization = Real(final quantity = "Magnetization", final unit = "A/m");
    type MagneticPolarization = Real(final quantity = "MagneticPolarization", final unit = "T");
    type ElectromagneticEnergyDensity = Real(final quantity = "EnergyDensity", final unit = "J/m3");
    type PoyntingVector = Real(final quantity = "PoyntingVector", final unit = "W/m2");
    type Resistance = Real(final quantity = "Resistance", final unit = "Ohm");
    type Resistivity = Real(final quantity = "Resistivity", final unit = "Ohm.m");
    type Conductivity = Real(final quantity = "Conductivity", final unit = "S/m");
    type Reluctance = Real(final quantity = "Reluctance", final unit = "H-1");
    type Permeance = Real(final quantity = "Permeance", final unit = "H");
    type PhaseDifference = Real(final quantity = "Angle", final unit = "rad", displayUnit = "deg");
    type Impedance = Resistance;
    type ModulusOfImpedance = Resistance;
    type Reactance = Resistance;
    type QualityFactor = Real(final quantity = "QualityFactor", final unit = "1");
    type LossAngle = Real(final quantity = "Angle", final unit = "rad", displayUnit = "deg");
    type Conductance = Real(final quantity = "Conductance", final unit = "S");
    type Admittance = Conductance;
    type ModulusOfAdmittance = Conductance;
    type Susceptance = Conductance;
    type InstantaneousPower = Real(final quantity = "Power", final unit = "W");
    type ActivePower = Real(final quantity = "Power", final unit = "W");
    type ApparentPower = Real(final quantity = "Power", final unit = "VA");
    type ReactivePower = Real(final quantity = "Power", final unit = "var");
    type PowerFactor = Real(final quantity = "PowerFactor", final unit = "1");
    // added to ISO-chapter 5
    type Transconductance = Real(final quantity = "Transconductance", final unit = "A/V2");
    type InversePotential = Real(final quantity = "InversePotential", final unit = "1/V");
    type ElectricalForceConstant = Real(final quantity = "ElectricalForceConstant", final unit = "N/A");
    // Light and Related Electromagnetic Radiations (chapter 6 of ISO 31-1992)
    type RadiantEnergy = Real(final quantity = "Energy", final unit = "J");
    type RadiantEnergyDensity = Real(final quantity = "EnergyDensity", final unit = "J/m3");
    type SpectralRadiantEnergyDensity = Real(final quantity = "SpectralRadiantEnergyDensity", final unit = "J/m4");
    type RadiantPower = Real(final quantity = "Power", final unit = "W");
    type RadiantEnergyFluenceRate = Real(final quantity = "RadiantEnergyFluenceRate", final unit = "W/m2");
    type RadiantIntensity = Real(final quantity = "RadiantIntensity", final unit = "W/sr");
    type Radiance = Real(final quantity = "Radiance", final unit = "W/(sr.m2)");
    type RadiantExtiance = Real(final quantity = "RadiantExtiance", final unit = "W/m2");
    type Irradiance = Real(final quantity = "Irradiance", final unit = "W/m2");
    type Emissivity = Real(final quantity = "Emissivity", final unit = "1");
    type SpectralEmissivity = Real(final quantity = "SpectralEmissivity", final unit = "1");
    type DirectionalSpectralEmissivity = Real(final quantity = "DirectionalSpectralEmissivity", final unit = "1");
    type LuminousIntensity = Real(final quantity = "LuminousIntensity", final unit = "cd");
    type LuminousFlux = Real(final quantity = "LuminousFlux", final unit = "lm");
    type QuantityOfLight = Real(final quantity = "QuantityOfLight", final unit = "lm.s");
    type Luminance = Real(final quantity = "Luminance", final unit = "cd/m2");
    type LuminousExitance = Real(final quantity = "LuminousExitance", final unit = "lm/m2");
    type Illuminance = Real(final quantity = "Illuminance", final unit = "lx");
    type LightExposure = Real(final quantity = "LightExposure", final unit = "lx.s");
    type LuminousEfficacy = Real(final quantity = "LuminousEfficacy", final unit = "lm/W");
    type SpectralLuminousEfficacy = Real(final quantity = "SpectralLuminousEfficacy", final unit = "lm/W");
    type LuminousEfficiency = Real(final quantity = "LuminousEfficiency", final unit = "1");
    type SpectralLuminousEfficiency = Real(final quantity = "SpectralLuminousEfficiency", final unit = "1");
    type CIESpectralTristimulusValues = Real(final quantity = "CIESpectralTristimulusValues", final unit = "1");
    type ChromaticityCoordinates = Real(final quantity = "CromaticityCoordinates", final unit = "1");
    type SpectralAbsorptionFactor = Real(final quantity = "SpectralAbsorptionFactor", final unit = "1");
    type SpectralReflectionFactor = Real(final quantity = "SpectralReflectionFactor", final unit = "1");
    type SpectralTransmissionFactor = Real(final quantity = "SpectralTransmissionFactor", final unit = "1");
    type SpectralRadianceFactor = Real(final quantity = "SpectralRadianceFactor", final unit = "1");
    type LinearAttenuationCoefficient = Real(final quantity = "AttenuationCoefficient", final unit = "m-1");
    type LinearAbsorptionCoefficient = Real(final quantity = "LinearAbsorptionCoefficient", final unit = "m-1");
    type MolarAbsorptionCoefficient = Real(final quantity = "MolarAbsorptionCoefficient", final unit = "m2/mol");
    type RefractiveIndex = Real(final quantity = "RefractiveIndex", final unit = "1");
    // Acoustics (chapter 7 of ISO 31-1992)
    type StaticPressure = AbsolutePressure;
    type SoundPressure = StaticPressure;
    type SoundParticleDisplacement = Real(final quantity = "Length", final unit = "m");
    type SoundParticleVelocity = Real(final quantity = "Velocity", final unit = "m/s");
    type SoundParticleAcceleration = Real(final quantity = "Acceleration", final unit = "m/s2");
    type VelocityOfSound = Real(final quantity = "Velocity", final unit = "m/s");
    type SoundEnergyDensity = Real(final quantity = "EnergyDensity", final unit = "J/m3");
    type SoundPower = Real(final quantity = "Power", final unit = "W");
    type SoundIntensity = Real(final quantity = "SoundIntensity", final unit = "W/m2");
    type AcousticImpedance = Real(final quantity = "AcousticImpedance", final unit = "Pa.s/m3");
    type SpecificAcousticImpedance = Real(final quantity = "SpecificAcousticImpedance", final unit = "Pa.s/m");
    type MechanicalImpedance = Real(final quantity = "MechanicalImpedance", final unit = "N.s/m");
    type SoundPressureLevel = Real(final quantity = "SoundPressureLevel", final unit = "dB");
    type SoundPowerLevel = Real(final quantity = "SoundPowerLevel", final unit = "dB");
    type DissipationCoefficient = Real(final quantity = "DissipationCoefficient", final unit = "1");
    type ReflectionCoefficient = Real(final quantity = "ReflectionCoefficient", final unit = "1");
    type TransmissionCoefficient = Real(final quantity = "TransmissionCoefficient", final unit = "1");
    type AcousticAbsorptionCoefficient = Real(final quantity = "AcousticAbsorptionCoefficient", final unit = "1");
    type SoundReductionIndex = Real(final quantity = "SoundReductionIndex", final unit = "dB");
    type EquivalentAbsorptionArea = Real(final quantity = "Area", final unit = "m2");
    type ReverberationTime = Real(final quantity = "Time", final unit = "s");
    type LoudnessLevel = Real(final quantity = "LoudnessLevel", final unit = "phon");
    type Loudness = Real(final quantity = "Loudness", final unit = "sone");
    type LoundnessLevel = Real(final quantity = "LoundnessLevel", final unit = "phon") "Obsolete type, use LoudnessLevel instead!";
    type Loundness = Real(final quantity = "Loundness", final unit = "sone") "Obsolete type, use Loudness instead!";
    // Physical chemistry and molecular physics (chapter 8 of ISO 31-1992)
    type RelativeAtomicMass = Real(final quantity = "RelativeAtomicMass", final unit = "1");
    type RelativeMolecularMass = Real(final quantity = "RelativeMolecularMass", final unit = "1");
    type NumberOfMolecules = Real(final quantity = "NumberOfMolecules", final unit = "1");
    type AmountOfSubstance = Real(final quantity = "AmountOfSubstance", final unit = "mol", min = 0);
    type MolarMass = Real(final quantity = "MolarMass", final unit = "kg/mol", min = 0);
    type MolarVolume = Real(final quantity = "MolarVolume", final unit = "m3/mol", min = 0);
    type MolarDensity = Real(final quantity = "MolarDensity", unit = "mol/m3");
    type MolarEnergy = Real(final quantity = "MolarEnergy", final unit = "J/mol", nominal = 2e4);
    type MolarInternalEnergy = MolarEnergy;
    type MolarHeatCapacity = Real(final quantity = "MolarHeatCapacity", final unit = "J/(mol.K)");
    type MolarEntropy = Real(final quantity = "MolarEntropy", final unit = "J/(mol.K)");
    type MolarEnthalpy = MolarEnergy;
    type MolarFlowRate = Real(final quantity = "MolarFlowRate", final unit = "mol/s");
    type NumberDensityOfMolecules = Real(final quantity = "NumberDensityOfMolecules", final unit = "m-3");
    type MolecularConcentration = Real(final quantity = "MolecularConcentration", final unit = "m-3");
    type MassConcentration = Real(final quantity = "MassConcentration", final unit = "kg/m3");
    type MassFraction = Real(final quantity = "MassFraction", final unit = "1", min = 0, max = 1);
    type Concentration = Real(final quantity = "Concentration", final unit = "mol/m3");
    type VolumeFraction = Real(final quantity = "VolumeFraction", final unit = "1");
    type MoleFraction = Real(final quantity = "MoleFraction", final unit = "1", min = 0, max = 1);
    type ChemicalPotential = Real(final quantity = "ChemicalPotential", final unit = "J/mol");
    type AbsoluteActivity = Real(final quantity = "AbsoluteActivity", final unit = "1");
    type PartialPressure = AbsolutePressure;
    type Fugacity = Real(final quantity = "Fugacity", final unit = "Pa");
    type StandardAbsoluteActivity = Real(final quantity = "StandardAbsoluteActivity", final unit = "1");
    type ActivityCoefficient = Real(final quantity = "ActivityCoefficient", final unit = "1");
    type ActivityOfSolute = Real(final quantity = "ActivityOfSolute", final unit = "1");
    type ActivityCoefficientOfSolute = Real(final quantity = "ActivityCoefficientOfSolute", final unit = "1");
    type StandardAbsoluteActivityOfSolute = Real(final quantity = "StandardAbsoluteActivityOfSolute", final unit = "1");
    type ActivityOfSolvent = Real(final quantity = "ActivityOfSolvent", final unit = "1");
    type OsmoticCoefficientOfSolvent = Real(final quantity = "OsmoticCoefficientOfSolvent", final unit = "1");
    type StandardAbsoluteActivityOfSolvent = Real(final quantity = "StandardAbsoluteActivityOfSolvent", final unit = "1");
    type OsmoticPressure = Real(final quantity = "Pressure", final unit = "Pa", displayUnit = "bar", min = 0);
    type StoichiometricNumber = Real(final quantity = "StoichiometricNumber", final unit = "1");
    type Affinity = Real(final quantity = "Affinity", final unit = "J/mol");
    type MassOfMolecule = Real(final quantity = "Mass", final unit = "kg");
    type ElectricDipoleMomentOfMolecule = Real(final quantity = "ElectricDipoleMomentOfMolecule", final unit = "C.m");
    type ElectricPolarizabilityOfAMolecule = Real(final quantity = "ElectricPolarizabilityOfAMolecule", final unit = "C.m2/V");
    type MicrocanonicalPartitionFunction = Real(final quantity = "MicrocanonicalPartitionFunction", final unit = "1");
    type CanonicalPartitionFunction = Real(final quantity = "CanonicalPartitionFunction", final unit = "1");
    type GrandCanonicalPartitionFunction = Real(final quantity = "GrandCanonicalPartitionFunction", final unit = "1");
    type MolecularPartitionFunction = Real(final quantity = "MolecularPartitionFunction", final unit = "1");
    type StatisticalWeight = Real(final quantity = "StatisticalWeight", final unit = "1");
    type MeanFreePath = Length;
    type DiffusionCoefficient = Real(final quantity = "DiffusionCoefficient", final unit = "m2/s");
    type ThermalDiffusionRatio = Real(final quantity = "ThermalDiffusionRatio", final unit = "1");
    type ThermalDiffusionFactor = Real(final quantity = "ThermalDiffusionFactor", final unit = "1");
    type ThermalDiffusionCoefficient = Real(final quantity = "ThermalDiffusionCoefficient", final unit = "m2/s");
    type ElementaryCharge = Real(final quantity = "ElementaryCharge", final unit = "C");
    type ChargeNumberOfIon = Real(final quantity = "ChargeNumberOfIon", final unit = "1");
    type FaradayConstant = Real(final quantity = "FaradayConstant", final unit = "C/mol");
    type IonicStrength = Real(final quantity = "IonicStrength", final unit = "mol/kg");
    type DegreeOfDissociation = Real(final quantity = "DegreeOfDissociation", final unit = "1");
    type ElectrolyticConductivity = Real(final quantity = "ElectrolyticConductivity", final unit = "S/m");
    type MolarConductivity = Real(final quantity = "MolarConductivity", final unit = "S.m2/mol");
    type TransportNumberOfIonic = Real(final quantity = "TransportNumberOfIonic", final unit = "1");
    // Atomic and Nuclear Physics (chapter 9 of ISO 31-1992)
    type ProtonNumber = Real(final quantity = "ProtonNumber", final unit = "1");
    type NeutronNumber = Real(final quantity = "NeutronNumber", final unit = "1");
    type NucleonNumber = Real(final quantity = "NucleonNumber", final unit = "1");
    type AtomicMassConstant = Real(final quantity = "Mass", final unit = "kg");
    type MassOfElectron = Real(final quantity = "Mass", final unit = "kg");
    type MassOfProton = Real(final quantity = "Mass", final unit = "kg");
    type MassOfNeutron = Real(final quantity = "Mass", final unit = "kg");
    type HartreeEnergy = Real(final quantity = "Energy", final unit = "J");
    type MagneticMomentOfParticle = Real(final quantity = "MagneticMomentOfParticle", final unit = "A.m2");
    type BohrMagneton = MagneticMomentOfParticle;
    type NuclearMagneton = MagneticMomentOfParticle;
    type GyromagneticCoefficient = Real(final quantity = "GyromagneticCoefficient", final unit = "A.m2/(J.s)");
    type GFactorOfAtom = Real(final quantity = "GFactorOfAtom", final unit = "1");
    type GFactorOfNucleus = Real(final quantity = "GFactorOfNucleus", final unit = "1");
    type LarmorAngularFrequency = Real(final quantity = "AngularFrequency", final unit = "s-1");
    type NuclearPrecessionAngularFrequency = Real(final quantity = "AngularFrequency", final unit = "s-1");
    type CyclotronAngularFrequency = Real(final quantity = "AngularFrequency", final unit = "s-1");
    type NuclearQuadrupoleMoment = Real(final quantity = "NuclearQuadrupoleMoment", final unit = "m2");
    type NuclearRadius = Real(final quantity = "Length", final unit = "m");
    type ElectronRadius = Real(final quantity = "Length", final unit = "m");
    type ComptonWavelength = Real(final quantity = "Length", final unit = "m");
    type MassExcess = Real(final quantity = "Mass", final unit = "kg");
    type MassDefect = Real(final quantity = "Mass", final unit = "kg");
    type RelativeMassExcess = Real(final quantity = "RelativeMassExcess", final unit = "1");
    type RelativeMassDefect = Real(final quantity = "RelativeMassDefect", final unit = "1");
    type PackingFraction = Real(final quantity = "PackingFraction", final unit = "1");
    type BindingFraction = Real(final quantity = "BindingFraction", final unit = "1");
    type MeanLife = Real(final quantity = "Time", final unit = "s");
    type LevelWidth = Real(final quantity = "LevelWidth", final unit = "J");
    type Activity = Real(final quantity = "Activity", final unit = "Bq");
    type SpecificActivity = Real(final quantity = "SpecificActivity", final unit = "Bq/kg");
    type DecayConstant = Real(final quantity = "DecayConstant", final unit = "s-1");
    type HalfLife = Real(final quantity = "Time", final unit = "s");
    type AlphaDisintegrationEnergy = Real(final quantity = "Energy", final unit = "J");
    type MaximumBetaParticleEnergy = Real(final quantity = "Energy", final unit = "J");
    type BetaDisintegrationEnergy = Real(final quantity = "Energy", final unit = "J");
    // Nuclear Reactions and Ionizing Radiations (chapter 10 of ISO 31-1992)
    type ReactionEnergy = Real(final quantity = "Energy", final unit = "J");
    type ResonanceEnergy = Real(final quantity = "Energy", final unit = "J");
    type CrossSection = Real(final quantity = "Area", final unit = "m2");
    type TotalCrossSection = Real(final quantity = "Area", final unit = "m2");
    type AngularCrossSection = Real(final quantity = "AngularCrossSection", final unit = "m2/sr");
    type SpectralCrossSection = Real(final quantity = "SpectralCrossSection", final unit = "m2/J");
    type SpectralAngularCrossSection = Real(final quantity = "SpectralAngularCrossSection", final unit = "m2/(sr.J)");
    type MacroscopicCrossSection = Real(final quantity = "MacroscopicCrossSection", final unit = "m-1");
    type TotalMacroscopicCrossSection = Real(final quantity = "TotalMacroscopicCrossSection", final unit = "m-1");
    type ParticleFluence = Real(final quantity = "ParticleFluence", final unit = "m-2");
    type ParticleFluenceRate = Real(final quantity = "ParticleFluenceRate", final unit = "s-1.m2");
    type EnergyFluence = Real(final quantity = "EnergyFluence", final unit = "J/m2");
    type EnergyFluenceRate = Real(final quantity = "EnergyFluenceRate", final unit = "W/m2");
    type CurrentDensityOfParticles = Real(final quantity = "CurrentDensityOfParticles", final unit = "m-2.s-1");
    type MassAttenuationCoefficient = Real(final quantity = "MassAttenuationCoefficient", final unit = "m2/kg");
    type MolarAttenuationCoefficient = Real(final quantity = "MolarAttenuationCoefficient", final unit = "m2/mol");
    type AtomicAttenuationCoefficient = Real(final quantity = "AtomicAttenuationCoefficient", final unit = "m2");
    type HalfThickness = Real(final quantity = "Length", final unit = "m");
    type TotalLinearStoppingPower = Real(final quantity = "TotalLinearStoppingPower", final unit = "J/m");
    type TotalAtomicStoppingPower = Real(final quantity = "TotalAtomicStoppingPower", final unit = "J.m2");
    type TotalMassStoppingPower = Real(final quantity = "TotalMassStoppingPower", final unit = "J.m2/kg");
    type MeanLinearRange = Real(final quantity = "Length", final unit = "m");
    type MeanMassRange = Real(final quantity = "MeanMassRange", final unit = "kg/m2");
    type LinearIonization = Real(final quantity = "LinearIonization", final unit = "m-1");
    type TotalIonization = Real(final quantity = "TotalIonization", final unit = "1");
    type Mobility = Real(final quantity = "Mobility", final unit = "m2/(V.s)");
    type IonNumberDensity = Real(final quantity = "IonNumberDensity", final unit = "m-3");
    type RecombinationCoefficient = Real(final quantity = "RecombinationCoefficient", final unit = "m3/s");
    type NeutronNumberDensity = Real(final quantity = "NeutronNumberDensity", final unit = "m-3");
    type NeutronSpeed = Real(final quantity = "Velocity", final unit = "m/s");
    type NeutronFluenceRate = Real(final quantity = "NeutronFluenceRate", final unit = "s-1.m-2");
    type TotalNeutronSourceDensity = Real(final quantity = "TotalNeutronSourceDesity", final unit = "s-1.m-3");
    type SlowingDownDensity = Real(final quantity = "SlowingDownDensity", final unit = "s-1.m-3");
    type ResonanceEscapeProbability = Real(final quantity = "ResonanceEscapeProbability", final unit = "1");
    type Lethargy = Real(final quantity = "Lethargy", final unit = "1");
    type SlowingDownArea = Real(final quantity = "Area", final unit = "m2");
    type DiffusionArea = Real(final quantity = "Area", final unit = "m2");
    type MigrationArea = Real(final quantity = "Area", final unit = "m2");
    type SlowingDownLength = Real(final quantity = "SLength", final unit = "m");
    type DiffusionLength = Length;
    type MigrationLength = Length;
    type NeutronYieldPerFission = Real(final quantity = "NeutronYieldPerFission", final unit = "1");
    type NeutronYieldPerAbsorption = Real(final quantity = "NeutronYieldPerAbsorption", final unit = "1");
    type FastFissionFactor = Real(final quantity = "FastFissionFactor", final unit = "1");
    type ThermalUtilizationFactor = Real(final quantity = "ThermalUtilizationFactor", final unit = "1");
    type NonLeakageProbability = Real(final quantity = "NonLeakageProbability", final unit = "1");
    type Reactivity = Real(final quantity = "Reactivity", final unit = "1");
    type ReactorTimeConstant = Real(final quantity = "Time", final unit = "s");
    type EnergyImparted = Real(final quantity = "Energy", final unit = "J");
    type MeanEnergyImparted = Real(final quantity = "Energy", final unit = "J");
    type SpecificEnergyImparted = Real(final quantity = "SpecificEnergy", final unit = "Gy");
    type AbsorbedDose = Real(final quantity = "AbsorbedDose", final unit = "Gy");
    type DoseEquivalent = Real(final quantity = "DoseEquivalent", final unit = "Sv");
    type AbsorbedDoseRate = Real(final quantity = "AbsorbedDoseRate", final unit = "Gy/s");
    type LinearEnergyTransfer = Real(final quantity = "LinearEnergyTransfer", final unit = "J/m");
    type Kerma = Real(final quantity = "Kerma", final unit = "Gy");
    type KermaRate = Real(final quantity = "KermaRate", final unit = "Gy/s");
    type MassEnergyTransferCoefficient = Real(final quantity = "MassEnergyTransferCoefficient", final unit = "m2/kg");
    type Exposure = Real(final quantity = "Exposure", final unit = "C/kg");
    type ExposureRate = Real(final quantity = "ExposureRate", final unit = "C/(kg.s)");
    // chapter 11 is not defined in ISO 31-1992
    // Characteristic Numbers (chapter 12 of ISO 31-1992)
    type ReynoldsNumber = Real(final quantity = "ReynoldsNumber", final unit = "1");
    type EulerNumber = Real(final quantity = "EulerNumber", final unit = "1");
    type FroudeNumber = Real(final quantity = "FroudeNumber", final unit = "1");
    type GrashofNumber = Real(final quantity = "GrashofNumber", final unit = "1");
    type WeberNumber = Real(final quantity = "WeberNumber", final unit = "1");
    type MachNumber = Real(final quantity = "MachNumber", final unit = "1");
    type KnudsenNumber = Real(final quantity = "KnudsenNumber", final unit = "1");
    type StrouhalNumber = Real(final quantity = "StrouhalNumber", final unit = "1");
    type FourierNumber = Real(final quantity = "FourierNumber", final unit = "1");
    type PecletNumber = Real(final quantity = "PecletNumber", final unit = "1");
    type RayleighNumber = Real(final quantity = "RayleighNumber", final unit = "1");
    type NusseltNumber = Real(final quantity = "NusseltNumber", final unit = "1");
    type BiotNumber = NusseltNumber;
    // The Biot number (Bi) is used when
    // the Nusselt number is reserved
    // for convective transport of heat.
    type StantonNumber = Real(final quantity = "StantonNumber", final unit = "1");
    type FourierNumberOfMassTransfer = Real(final quantity = "FourierNumberOfMassTransfer", final unit = "1");
    type PecletNumberOfMassTransfer = Real(final quantity = "PecletNumberOfMassTransfer", final unit = "1");
    type GrashofNumberOfMassTransfer = Real(final quantity = "GrashofNumberOfMassTransfer", final unit = "1");
    type NusseltNumberOfMassTransfer = Real(final quantity = "NusseltNumberOfMassTransfer", final unit = "1");
    type StantonNumberOfMassTransfer = Real(final quantity = "StantonNumberOfMassTransfer", final unit = "1");
    type PrandtlNumber = Real(final quantity = "PrandtlNumber", final unit = "1");
    type SchmidtNumber = Real(final quantity = "SchmidtNumber", final unit = "1");
    type LewisNumber = Real(final quantity = "LewisNumber", final unit = "1");
    type MagneticReynoldsNumber = Real(final quantity = "MagneticReynoldsNumber", final unit = "1");
    type AlfvenNumber = Real(final quantity = "AlfvenNumber", final unit = "1");
    type HartmannNumber = Real(final quantity = "HartmannNumber", final unit = "1");
    type CowlingNumber = Real(final quantity = "CowlingNumber", final unit = "1");
    // Solid State Physics (chapter 13 of ISO 31-1992)
    type BraggAngle = Angle;
    type OrderOfReflexion = Real(final quantity = "OrderOfReflexion", final unit = "1");
    type ShortRangeOrderParameter = Real(final quantity = "RangeOrderParameter", final unit = "1");
    type LongRangeOrderParameter = Real(final quantity = "RangeOrderParameter", final unit = "1");
    type DebyeWallerFactor = Real(final quantity = "DebyeWallerFactor", final unit = "1");
    type CircularWavenumber = Real(final quantity = "CircularWavenumber", final unit = "m-1");
    type FermiCircularWavenumber = Real(final quantity = "FermiCircularWavenumber", final unit = "m-1");
    type DebyeCircularWavenumber = Real(final quantity = "DebyeCircularWavenumber", final unit = "m-1");
    type DebyeCircularFrequency = Real(final quantity = "AngularFrequency", final unit = "s-1");
    type DebyeTemperature = ThermodynamicTemperature;
    type SpectralConcentration = Real(final quantity = "SpectralConcentration", final unit = "s/m3");
    type GrueneisenParameter = Real(final quantity = "GrueneisenParameter", final unit = "1");
    type MadelungConstant = Real(final quantity = "MadelungConstant", final unit = "1");
    type DensityOfStates = Real(final quantity = "DensityOfStates", final unit = "J-1/m-3");
    type ResidualResistivity = Real(final quantity = "ResidualResistivity", final unit = "Ohm.m");
    type LorenzCoefficient = Real(final quantity = "LorenzCoefficient", final unit = "V2/K2");
    type HallCoefficient = Real(final quantity = "HallCoefficient", final unit = "m3/C");
    type ThermoelectromotiveForce = Real(final quantity = "ThermoelectromotiveForce", final unit = "V");
    type SeebeckCoefficient = Real(final quantity = "SeebeckCoefficient", final unit = "V/K");
    type PeltierCoefficient = Real(final quantity = "PeltierCoefficient", final unit = "V");
    type ThomsonCoefficient = Real(final quantity = "ThomsonCoefficient", final unit = "V/K");
    type RichardsonConstant = Real(final quantity = "RichardsonConstant", final unit = "A/(m2.K2)");
    type FermiEnergy = Real(final quantity = "Energy", final unit = "eV");
    type GapEnergy = Real(final quantity = "Energy", final unit = "eV");
    type DonorIonizationEnergy = Real(final quantity = "Energy", final unit = "eV");
    type AcceptorIonizationEnergy = Real(final quantity = "Energy", final unit = "eV");
    type ActivationEnergy = Real(final quantity = "Energy", final unit = "eV");
    type FermiTemperature = ThermodynamicTemperature;
    type ElectronNumberDensity = Real(final quantity = "ElectronNumberDensity", final unit = "m-3");
    type HoleNumberDensity = Real(final quantity = "HoleNumberDensity", final unit = "m-3");
    type IntrinsicNumberDensity = Real(final quantity = "IntrinsicNumberDensity", final unit = "m-3");
    type DonorNumberDensity = Real(final quantity = "DonorNumberDensity", final unit = "m-3");
    type AcceptorNumberDensity = Real(final quantity = "AcceptorNumberDensity", final unit = "m-3");
    type EffectiveMass = Mass;
    type MobilityRatio = Real(final quantity = "MobilityRatio", final unit = "1");
    type RelaxationTime = Time;
    type CarrierLifeTime = Time;
    type ExchangeIntegral = Real(final quantity = "Energy", final unit = "eV");
    type CurieTemperature = ThermodynamicTemperature;
    type NeelTemperature = ThermodynamicTemperature;
    type LondonPenetrationDepth = Length;
    type CoherenceLength = Length;
    type LandauGinzburgParameter = Real(final quantity = "LandauGinzburgParameter", final unit = "1");
    type FluxiodQuantum = Real(final quantity = "FluxiodQuantum", final unit = "Wb");
    type TimeAging = Real(final quantity = "1/Modelica.SIunits.Time", final unit = "1/s");
    type ChargeAging = Real(final quantity = "1/Modelica.SIunits.ElectricCharge", final unit = "1/(A.s)");
    // Other types not defined in ISO 31-1992
    type PerUnit = Real(unit = "1");
    // Complex types for electrical systems (not defined in ISO 31-1992)
    operator record ComplexCurrent = Complex(redeclare Modelica.SIunits.Current re, redeclare Modelica.SIunits.Current im) "Complex electrical current";
    operator record ComplexCurrentSlope = Complex(redeclare Modelica.SIunits.CurrentSlope re, redeclare Modelica.SIunits.CurrentSlope im) "Complex current slope";
    operator record ComplexCurrentDensity = Complex(redeclare Modelica.SIunits.CurrentDensity re, redeclare Modelica.SIunits.CurrentDensity im) "Complex electrical current density";
    operator record ComplexElectricPotential = Complex(redeclare Modelica.SIunits.ElectricPotential re, redeclare Modelica.SIunits.ElectricPotential im) "Complex electric potential";
    operator record ComplexPotentialDifference = Complex(redeclare Modelica.SIunits.PotentialDifference re, redeclare Modelica.SIunits.PotentialDifference im) "Complex electric potential difference";
    operator record ComplexVoltage = Complex(redeclare Modelica.SIunits.Voltage re, redeclare Modelica.SIunits.Voltage im) "Complex electrical voltage";
    operator record ComplexVoltageSlope = Complex(redeclare Modelica.SIunits.VoltageSlope re, redeclare Modelica.SIunits.VoltageSlope im) "Complex voltage slope";
    operator record ComplexElectricFieldStrength = Complex(redeclare Modelica.SIunits.ElectricFieldStrength re, redeclare Modelica.SIunits.ElectricFieldStrength im) "Complex electric field strength";
    operator record ComplexElectricFluxDensity = Complex(redeclare Modelica.SIunits.ElectricFluxDensity re, redeclare Modelica.SIunits.ElectricFluxDensity im) "Complex electric flux density";
    operator record ComplexElectricFlux = Complex(redeclare Modelica.SIunits.ElectricFlux re, redeclare Modelica.SIunits.ElectricFlux im) "Complex electric flux";
    operator record ComplexMagneticFieldStrength = Complex(redeclare Modelica.SIunits.MagneticFieldStrength re, redeclare Modelica.SIunits.MagneticFieldStrength im) "Complex magnetic field strength";
    operator record ComplexMagneticPotential = Complex(redeclare Modelica.SIunits.MagneticPotential re, redeclare Modelica.SIunits.MagneticPotential im) "Complex magnetic potential";
    operator record ComplexMagneticPotentialDifference = Complex(redeclare Modelica.SIunits.MagneticPotentialDifference re, redeclare Modelica.SIunits.MagneticPotentialDifference im) "Complex magnetic potential difference";
    operator record ComplexMagnetomotiveForce = Complex(redeclare Modelica.SIunits.MagnetomotiveForce re, redeclare Modelica.SIunits.MagnetomotiveForce im) "Complex magneto motive force";
    operator record ComplexMagneticFluxDensity = Complex(redeclare Modelica.SIunits.MagneticFluxDensity re, redeclare Modelica.SIunits.MagneticFluxDensity im) "Complex magnetic flux density";
    operator record ComplexMagneticFlux = Complex(redeclare Modelica.SIunits.MagneticFlux re, redeclare Modelica.SIunits.MagneticFlux im) "Complex magnetic flux";
    operator record ComplexReluctance = Complex(redeclare Modelica.SIunits.Reluctance re, redeclare Modelica.SIunits.Reluctance im) "Complex reluctance" annotation(
      Documentation(info = "<html>
<p>
Since magnetic material properties like reluctance and permeance often are anisotropic resp. salient,
a special operator instead of multiplication (compare: tensor vs. vector) is required.
<a href=\"modelica://Modelica.Magnetic.FundamentalWave\">Modelica.Magnetic.FundamentalWave</a> uses a
special record <a href=\"modelica://Modelica.Magnetic.FundamentalWave.Types.Salient\">Salient</a>
which is only valid in the rotor-fixed coordinate system.
</p>
<p>
<b>Note:</b> To avoid confusion, no magnetic material properties should be defined as Complex units.
</p>
</html>"));
    operator record ComplexImpedance = Complex(redeclare Resistance re, redeclare Reactance im) "Complex electrical impedance";
    operator record ComplexAdmittance = Complex(redeclare Conductance re, redeclare Susceptance im) "Complex electrical admittance";
    operator record ComplexPower = Complex(redeclare ActivePower re, redeclare ReactivePower im) "Complex electrical power";
    annotation(
      Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-66, 78}, {-66, -40}}, color = {64, 64, 64}, smooth = Smooth.None), Ellipse(extent = {{12, 36}, {68, -38}}, lineColor = {64, 64, 64}, fillColor = {175, 175, 175}, fillPattern = FillPattern.Solid), Rectangle(extent = {{-74, 78}, {-66, -40}}, lineColor = {64, 64, 64}, fillColor = {175, 175, 175}, fillPattern = FillPattern.Solid), Polygon(points = {{-66, -4}, {-66, 6}, {-16, 56}, {-16, 46}, {-66, -4}}, lineColor = {64, 64, 64}, smooth = Smooth.None, fillColor = {175, 175, 175}, fillPattern = FillPattern.Solid), Polygon(points = {{-46, 16}, {-40, 22}, {-2, -40}, {-10, -40}, {-46, 16}}, lineColor = {64, 64, 64}, smooth = Smooth.None, fillColor = {175, 175, 175}, fillPattern = FillPattern.Solid), Ellipse(extent = {{22, 26}, {58, -28}}, lineColor = {64, 64, 64}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Polygon(points = {{68, 2}, {68, -46}, {64, -60}, {58, -68}, {48, -72}, {18, -72}, {18, -64}, {46, -64}, {54, -60}, {58, -54}, {60, -46}, {60, -26}, {64, -20}, {68, -6}, {68, 2}}, lineColor = {64, 64, 64}, smooth = Smooth.Bezier, fillColor = {175, 175, 175}, fillPattern = FillPattern.Solid)}),
      Documentation(info = "<html>
<p>This package provides predefined types, such as <i>Mass</i>,
<i>Angle</i>, <i>Time</i>, based on the international standard
on units, e.g.,
</p>

<pre>   <b>type</b> Angle = Real(<b>final</b> quantity = \"Angle\",
                     <b>final</b> unit     = \"rad\",
                     displayUnit    = \"deg\");
</pre>

<p>
as well as conversion functions from non SI-units to SI-units
and vice versa in subpackage
<a href=\"modelica://Modelica.SIunits.Conversions\">Conversions</a>.
</p>

<p>
For an introduction how units are used in the Modelica standard library
with package SIunits, have a look at:
<a href=\"modelica://Modelica.SIunits.UsersGuide.HowToUseSIunits\">How to use SIunits</a>.
</p>

<p>
Copyright &copy; 1998-2015, Modelica Association and DLR.
</p>
<p>
<i>This Modelica package is <u>free</u> software and the use is completely at <u>your own risk</u>; it can be redistributed and/or modified under the terms of the Modelica License 2. For license conditions (including the disclaimer of warranty) see <a href=\"modelica://Modelica.UsersGuide.ModelicaLicense2\">Modelica.UsersGuide.ModelicaLicense2</a> or visit <a href=\"https://www.modelica.org/licenses/ModelicaLicense2\"> https://www.modelica.org/licenses/ModelicaLicense2</a>.</i>
</p>
</html>", revisions = "<html>
<ul>
<li><i>May 25, 2011</i> by Stefan Wischhusen:<br/>Added molar units for energy and enthalpy.</li>
<li><i>Jan. 27, 2010</i> by Christian Kral:<br/>Added complex units.</li>
<li><i>Dec. 14, 2005</i> by <a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a>:<br/>Add User&#39;;s Guide and removed &quot;min&quot; values for Resistance and Conductance.</li>
<li><i>October 21, 2002</i> by <a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a> and <a href=\"http://www.robotic.dlr.de/Christian.Schweiger/\">Christian Schweiger</a>:<br/>Added new package <b>Conversions</b>. Corrected typo <i>Wavelenght</i>.</li>
<li><i>June 6, 2000</i> by <a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a>:<br/>Introduced the following new types<br/>type Temperature = ThermodynamicTemperature;<br/>types DerDensityByEnthalpy, DerDensityByPressure, DerDensityByTemperature, DerEnthalpyByPressure, DerEnergyByDensity, DerEnergyByPressure<br/>Attribute &quot;final&quot; removed from min and max values in order that these values can still be changed to narrow the allowed range of values.<br/>Quantity=&quot;Stress&quot; removed from type &quot;Stress&quot;, in order that a type &quot;Stress&quot; can be connected to a type &quot;Pressure&quot;.</li>
<li><i>Oct. 27, 1999</i> by <a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a>:<br/>New types due to electrical library: Transconductance, InversePotential, Damping.</li>
<li><i>Sept. 18, 1999</i> by <a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a>:<br/>Renamed from SIunit to SIunits. Subpackages expanded, i.e., the SIunits package, does no longer contain subpackages.</li>
<li><i>Aug 12, 1999</i> by <a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a>:<br/>Type &quot;Pressure&quot; renamed to &quot;AbsolutePressure&quot; and introduced a new type &quot;Pressure&quot; which does not contain a minimum of zero in order to allow convenient handling of relative pressure. Redefined BulkModulus as an alias to AbsolutePressure instead of Stress, since needed in hydraulics.</li>
<li><i>June 29, 1999</i> by <a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a>:<br/>Bug-fix: Double definition of &quot;Compressibility&quot; removed and appropriate &quot;extends Heat&quot; clause introduced in package SolidStatePhysics to incorporate ThermodynamicTemperature.</li>
<li><i>April 8, 1998</i> by <a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a> and Astrid Jaschinski:<br/>Complete ISO 31 chapters realized.</li>
<li><i>Nov. 15, 1997</i> by <a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a> and <a href=\"http://www.control.lth.se/~hubertus/\">Hubertus Tummescheit</a>:<br/>Some chapters realized.</li>
</ul>
</html>"));
  end SIunits;
  annotation(
    preferredView = "info",
    version = "3.2.1",
    versionBuild = 4,
    versionDate = "2013-08-14",
    dateModified = "2015-09-30 09:15:00Z",
    revisionId = "$Id::                                       $",
    uses(Complex(version = "3.2.1"), ModelicaServices(version = "3.2.1")),
    conversion(noneFromVersion = "3.2", noneFromVersion = "3.1", noneFromVersion = "3.0.1", noneFromVersion = "3.0", from(version = "2.1", script = "modelica://Modelica/Resources/Scripts/Dymola/ConvertModelica_from_2.2.2_to_3.0.mos"), from(version = "2.2", script = "modelica://Modelica/Resources/Scripts/Dymola/ConvertModelica_from_2.2.2_to_3.0.mos"), from(version = "2.2.1", script = "modelica://Modelica/Resources/Scripts/Dymola/ConvertModelica_from_2.2.2_to_3.0.mos"), from(version = "2.2.2", script = "modelica://Modelica/Resources/Scripts/Dymola/ConvertModelica_from_2.2.2_to_3.0.mos")),
    Icon(coordinateSystem(extent = {{-100.0, -100.0}, {100.0, 100.0}}), graphics = {Polygon(origin = {-6.9888, 20.048}, fillColor = {0, 0, 0}, pattern = LinePattern.None, fillPattern = FillPattern.Solid, points = {{-93.0112, 10.3188}, {-93.0112, 10.3188}, {-73.011, 24.6}, {-63.011, 31.221}, {-51.219, 36.777}, {-39.842, 38.629}, {-31.376, 36.248}, {-25.819, 29.369}, {-24.232, 22.49}, {-23.703, 17.463}, {-15.501, 25.135}, {-6.24, 32.015}, {3.02, 36.777}, {15.191, 39.423}, {27.097, 37.306}, {32.653, 29.633}, {35.035, 20.108}, {43.501, 28.046}, {54.085, 35.19}, {65.991, 39.952}, {77.897, 39.688}, {87.422, 33.338}, {91.126, 21.696}, {90.068, 9.525}, {86.099, -1.058}, {79.749, -10.054}, {71.283, -21.431}, {62.816, -33.337}, {60.964, -32.808}, {70.489, -16.14}, {77.368, -2.381}, {81.072, 10.054}, {79.749, 19.05}, {72.605, 24.342}, {61.758, 23.019}, {49.587, 14.817}, {39.003, 4.763}, {29.214, -6.085}, {21.012, -16.669}, {13.339, -26.458}, {5.401, -36.777}, {-1.213, -46.037}, {-6.24, -53.446}, {-8.092, -52.387}, {-0.684, -40.746}, {5.401, -30.692}, {12.81, -17.198}, {19.424, -3.969}, {23.658, 7.938}, {22.335, 18.785}, {16.514, 23.283}, {8.047, 23.019}, {-1.478, 19.05}, {-11.267, 11.113}, {-19.734, 2.381}, {-29.259, -8.202}, {-38.519, -19.579}, {-48.044, -31.221}, {-56.511, -43.392}, {-64.449, -55.298}, {-72.386, -66.939}, {-77.678, -74.612}, {-79.53, -74.083}, {-71.857, -61.383}, {-62.861, -46.037}, {-52.278, -28.046}, {-44.869, -15.346}, {-38.784, -2.117}, {-35.344, 8.731}, {-36.403, 19.844}, {-42.488, 23.813}, {-52.013, 22.49}, {-60.744, 16.933}, {-68.947, 10.054}, {-76.884, 2.646}, {-93.0112, -12.1707}, {-93.0112, -12.1707}}, smooth = Smooth.Bezier), Ellipse(origin = {40.8208, -37.7602}, fillColor = {161, 0, 4}, pattern = LinePattern.None, fillPattern = FillPattern.Solid, extent = {{-17.8562, -17.8563}, {17.8563, 17.8562}})}),
    Documentation(info = "<HTML>
<p>
Package <b>Modelica&reg;</b> is a <b>standardized</b> and <b>free</b> package
that is developed together with the Modelica&reg; language from the
Modelica Association, see
<a href=\"https://www.Modelica.org\">https://www.Modelica.org</a>.
It is also called <b>Modelica Standard Library</b>.
It provides model components in many domains that are based on
standardized interface definitions. Some typical examples are shown
in the next figure:
</p>

<p>
<img src=\"modelica://Modelica/Resources/Images/UsersGuide/ModelicaLibraries.png\">
</p>

<p>
For an introduction, have especially a look at:
</p>
<ul>
<li> <a href=\"modelica://Modelica.UsersGuide.Overview\">Overview</a>
  provides an overview of the Modelica Standard Library
  inside the <a href=\"modelica://Modelica.UsersGuide\">User's Guide</a>.</li>
<li><a href=\"modelica://Modelica.UsersGuide.ReleaseNotes\">Release Notes</a>
 summarizes the changes of new versions of this package.</li>
<li> <a href=\"modelica://Modelica.UsersGuide.Contact\">Contact</a>
  lists the contributors of the Modelica Standard Library.</li>
<li> The <b>Examples</b> packages in the various libraries, demonstrate
  how to use the components of the corresponding sublibrary.</li>
</ul>

<p>
This version of the Modelica Standard Library consists of
</p>
<ul>
<li><b>1360</b> models and blocks, and</li>
<li><b>1280</b> functions</li>
</ul>
<p>
that are directly usable (= number of public, non-partial classes). It is fully compliant
to <a href=\"https://www.modelica.org/documents/ModelicaSpec32Revision2.pdf\">Modelica Specification Version 3.2 Revision 2</a>
and it has been tested with Modelica tools from different vendors.
</p>

<p>
<b>Licensed by the Modelica Association under the Modelica License 2</b><br>
Copyright &copy; 1998-2015, ABB, AIT, T.&nbsp;B&ouml;drich, DLR, Dassault Syst&egrave;mes AB, Fraunhofer, A.&nbsp;Haumer, ITI, C.&nbsp;Kral, Modelon,
TU Hamburg-Harburg, Politecnico di Milano, XRG Simulation.
</p>

<p>
<i>This Modelica package is <u>free</u> software and the use is completely at <u>your own risk</u>; it can be redistributed and/or modified under the terms of the Modelica License 2. For license conditions (including the disclaimer of warranty) see <a href=\"modelica://Modelica.UsersGuide.ModelicaLicense2\">Modelica.UsersGuide.ModelicaLicense2</a> or visit <a href=\"https://www.modelica.org/licenses/ModelicaLicense2\"> https://www.modelica.org/licenses/ModelicaLicense2</a>.</i>
</p>

<p>
<b>Modelica&reg;</b> is a registered trademark of the Modelica Association.
</p>
</html>"));
end Modelica;